/*
 * Khoros: $Id: kfile.h,v 1.27.2.1 1997/03/11 23:49:51 mirage Exp $
 */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Khoros Transport Declarations
   >>>>
   >>>>    Written By: Mark Young
   >>>>
   >>>>          Date: Jul 08, 1992
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kutils_kfile_h_
#define _kutils_kfile_h_


	/*-------------------------------------*
	|       #include 
	---------------------------------------*/


	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

/*
 *  Define the different open options for kopen()
#define KOPEN_RDONLY	0x00
#define KOPEN_WRONLY	0x01
#define KOPEN_RDWR	0x02
#define KOPEN_NONBLOCK	0x04
#define KOPEN_APPEND	0x08
#define KOPEN_CREAT	0x10
#define KOPEN_TRUNC	0x20
#define KOPEN_EXCL	0x40
#define KOPEN_NOCTTY	0x80
#define KOPEN_STREAM    0x100
 */
#define KOPEN_RDONLY	O_RDONLY
#define KOPEN_WRONLY	O_WRONLY
#define KOPEN_RDWR	O_RDWR
#define KOPEN_NONBLOCK	O_NONBLOCK
#define KOPEN_APPEND	O_APPEND
#define KOPEN_CREAT	O_CREAT
#define KOPEN_TRUNC	O_TRUNC
#define KOPEN_EXCL	O_EXCL
#define KOPEN_NOCTTY	O_NOCTTY
#define KOPEN_STREAM    0x100000
#define KOPEN_LOCK	0x200000
#define KOPEN_TEMP	0x400000

/*
 *  Define the access mode KOPEN_ACCMODE
 */
#define KOPEN_ACCMODE       (KOPEN_RDONLY|KOPEN_WRONLY|KOPEN_RDWR)

/*
 *  Define the different seek options for klseek()
 */
#define KSEEK_SET 0
#define KSEEK_CUR 1
#define KSEEK_END 2

/*
 *  Need to see if we are on a system V machine.  If so then we need to define
 *  LOCK_EX, LOCK_UN, lockf
 */
#define KLOCK_SH         1       /* shared lock */
#define KLOCK_EX         2       /* exclusive lock */
#define KLOCK_NB         4       /* don't block when locking */
#define KLOCK_UN         8       /* unlock */

/*
 *  kfile stderr, stdout, and stdin...
 */
#define kstderr ((kfile *) stderr)
#define kstdin  ((kfile *) stdin)
#define kstdout ((kfile *) stdout)

/*
 *  fileno for kstdin, kstdout, kstderr
 */
#define KSTDIN_FILENO    0
#define KSTDOUT_FILENO   1
#define KSTDERR_FILENO   2

/*
 *  kfile state flags...
 */
#define KFILE_READ	(1L << 0)
#define KFILE_RDWR	(1L << 1)
#define KFILE_WRITE	(1L << 2)
#define KFILE_EOF	(1L << 3)
#define KFILE_ERR	(1L << 4)
#define KFILE_PERM	(1L << 5)
#define KFILE_OPEN	(1L << 6)
#define KFILE_DUP	(1L << 7)
#define KFILE_TEMP	(1L << 8)
#define KFILE_STREAM	(1L << 9)
#define KFILE_LOCK	(1L << 10)
#define KFILE_LINEBUF	(1L << 11)
#define KFILE_FULLBUF	(1L << 12)
#define KFILE_MYBUF	(1L << 13)
#define KFILE_REACQUIRE	(1L << 14)

/*
 *  Standard Inter-Process Communication Channel Identifiers
 */
#define KIPC_PROCESS	"Process"
#define KIPC_TRANSPORT  "Transport"


	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/

/*
 *  typedef structure for kfile.  The internal structure definition of a
 *  Khoros File.  The kfile strucutre is used by the different transport
 *  mechanisms as internal information for the different operations such
 *  as reading, writing, opening, closing, etc.
 *
 *  The actual kfile defintion is located in:
 *
 *		 $BOOTSTRAP/include/kutils/kfileP.h
 */
typedef struct _kobject kfile;


	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/


	/*-------------------------------------*
	|       macros
	---------------------------------------*/

/************************************************************
*
*  Routine Name: kgetc - get character from khoros transport
*
*       Purpose: This function is a replacement for the system getc()
*		 routine.  The only difference is that kgetc() supports
*		 all available transport mechanisms.  The routine will get
*		 a single character from the specified transport.  If the
*		 character could not be read, EOF is returned; otherwise,
*		 the character that is read is returned.
*
*         Input: file - the kfile transport to be read which was opened earlier
*			with kfopen().
*        Output:
*       Returns: returns the character read or EOF upon error.
*
*  Restrictions: 
*    Written By: Mark Young
*          Date: Sep 12, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kgetc(
*		 !  kfile *file)
*
*************************************************************/
#define kgetc(file) kfgetc(file)


/************************************************************
*
*  Routine Name: kgets - reads from kstdin until a newline or EOF
*
*       Purpose: This function is a replacement for the system gets()
*		 routine.  The only difference is that kgets() uses
*		 the khoros transport mechanisms.  The routine will get
*		 a single line from kstdin.  It reads until a newline or
*		 end of file is encountered.
*
*         Input: buffer - the buffer to read the data into.
*        Output:
*       Returns: returns the string read from kstdin or NULL upon error.
*
*  Restrictions: 
*    Written By: Mark Young
*          Date: Feb 08, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: char *kgets(
*		 !  char *buffer)
*
*************************************************************/
#define kgets(buffer) kfgets(buffer, -1, kstdin)

/************************************************************
*
*  Routine Name: kputc - put a character onto the khoros transport
*
*       Purpose: This function is a replacement for the system "fputc()"
*		 routine.  The only difference is that kfputc() supports
*		 more than just files, it supports other data transports
*		 such as shared memory, pipes, files, etc.
*
*		 The routine will put a single character onto the
*		 specified transport.  If the character could not
*		 be written, EOF is returned; otherwise, the character
*		 that is written is returned.
*
*         Input: character - the character to be written
*		 file      - the kfile transport to be read which was
*			     opened earlier with kfopen().
*        Output:
*       Returns: returns the character written or EOF upon error.
*
*  Restrictions: 
*    Written By: Mark Young
*          Date: Sep 12, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kputc(
*		 !  int   character,
*		 !  kfile *file)
*
*************************************************************/
#define kputc(character, file) kfputc(character, file)


/************************************************************
*
*  Routine Name: kinput - opens a file for reading using kopen()
*
*       Purpose: This function is just a simplified interface to kopen()
*		 that opens a file for reading.  The macros calls
*		 kopen using the following syntax:
*
*		 !    kopen(filename, KOPEN_RDONLY, 0666)
*
*
*         Input: filename - the filename to be opened for reading using
*			     the kopen().
*        Output:
*       Returns: returns the kfile id on success, -1 otherwise
*
*  Restrictions: 
*    Written By: Mark Young
*          Date: Feb 01, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kinput(
*		 !  char *filename)
*
*************************************************************/
#define kinput(filename) kopen(filename, KOPEN_RDONLY, 0666)


/************************************************************
*
*  Routine Name: koutput - opens/creates a file for writing using kopen()
*
*       Purpose: This function is just a simplified interface to kopen()
*		 that creates and opens a file for writing.  The file is
*		 truncated upon a successful kopen call.  The macros calls
*		 kopen using the following syntax:
*
*		 ! kopen(filename, KOPEN_WRONLY|KOPEN_CREAT|KOPEN_TRUNC, 0666)
*
*
*         Input: filename - the filename to be opened for writing using
*			     the kopen().
*        Output:
*       Returns: returns the kfile id on success, -1 otherwise
*
*  Restrictions: 
*    Written By: Mark Young
*          Date: Feb 01, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int koutput(
*		 !  char *filename)
*
*************************************************************/
#define koutput(filename) kopen(filename, KOPEN_WRONLY|KOPEN_CREAT|KOPEN_TRUNC, 0666)


/************************************************************
*
*  Routine Name: kfinput - opens a file for reading using kfopen()
*
*       Purpose: This function is just a simplified interface to kfopen()
*		 that opens a file for reading.  The macros calls
*		 kfopen using the following syntax:
*
*		 !      kfopen(filename, "r")
*
*
*         Input: filename - the filename to be opened for reading using
*			     the kfopen().
*        Output:
*       Returns: returns the kfile structure on success, NULL upon failure
*
*  Restrictions: 
*    Written By: Mark Young
*          Date: Feb 01, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: kfile *kfinput(
*		 !  char *filename)
*
*************************************************************/
#define kfinput(filename) kfopen(filename, "r")


/************************************************************
*
*  Routine Name: kfoutput - opens/creates a file for writing using kfopen()
*
*       Purpose: This function is just a simplified interface to kfopen()
*		 that creates and opens a file for writing.  The file is
*		 truncated upon a successful kfopen call.  The macros calls
*		 kfopen using the following syntax:
*
*			! kfopen(filename, "w")
*
*
*         Input: filename - the filename to be opened for writing using
*			     the kfopen().
*        Output:
*       Returns: returns the kfile structure on success, NULL upon failure
*
*  Restrictions: 
*    Written By: Mark Young
*          Date: Feb 01, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration:
*   Declaration: kfile *kfoutput(
*		 !  char *filename)
*
*************************************************************/
#define kfoutput(filename) kfopen(filename, "w")


/************************************************************
*
*  Routine Name: kfile_isread - khoros transport is readable
*
*       Purpose: Whether a khoros transport is readable.
*
*         Input: file - the khoros transport to test whether readable
*        Output: 
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_isread(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_isread(file)  (kfile_state(file) & KFILE_READ || \
			     kfile_state(file) & KFILE_RDWR)


/************************************************************
*
*  Routine Name: kfile_iswrite - khoros transport is writeable
*
*       Purpose: Whether a khoros transport is writeable.
*
*         Input: file - the khoros transport to test whether writeable
*        Output: 
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_iswrite(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_iswrite(file)  (kfile_state(file) & KFILE_WRITE || \
			      kfile_state(file) & KFILE_RDWR)


/************************************************************
*
*  Routine Name: kfile_isrdwr - khoros transport is both readable and
*				writeable
*
*       Purpose: Whether a khoros transport is readable and writeable.
*
*         Input: file - the khoros transport to test whether it's
*			readable and writeable
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_isrdwr(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_isrdwr(file)  (kfile_state(file) & KFILE_RDWR)


/************************************************************
*
*  Routine Name: kfile_ismybuf - khoros transport buffer was set
*				 by the application
*
*       Purpose: Whether a khoros transport's read/write buffer was
*		 set by the programmer or was internally allocated by the
*		 open transport process.
*
*         Input: file - the khoros transport to test whether the
*			stream buffer was set by the programmer.
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_ismybuf(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_ismybuf(file)  (kfile_state(file) & KFILE_MYBUF)


/************************************************************
*
*  Routine Name: kfile_ispermanent - khoros transport has permanence 
*
*       Purpose: Whether a khoros transport has data permanence.  If
*		 a transport has permanence then the data can be accessed
*		 multiple times.  In the case of files, shared memory, memory
*		 mapped files, these transports have permanence.  Where as
*		 for pipes and sockets the data can only be accessed (read or
*		 written) to once.
*
*         Input: file - the khoros transport to test whether the
*			transport has data permanence.
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_ispermanent(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_ispermanent(file)  (kfile_state(file) & KFILE_PERM)


/************************************************************
*
*  Routine Name: kfile_isstream - khoros transport is a stream based transport
*
*       Purpose: Whether a khoros transport is stream based transport which
*		 kutils will endeavor to make it appear to have permanence.
*
*         Input: file - the khoros transport to test whether
*			is a stream based transport.
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_isstream(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_isstream(file)  (kfile_state(file) & KFILE_STREAM)


/************************************************************
*
*  Routine Name: kfile_istemp - khoros transport is temporary
*
*       Purpose: Whether a khoros transport has been created as
*		 a temporary.  A transport created via ktempnam() or
*		 kfile_tempnam() is flagged as a temporary transport.
*
*         Input: file - the khoros transport to test whether the
*			transport is a temporary
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: May 11, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_istemp(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_istemp(file)  (kfile_state(file) & KFILE_TEMP)


/************************************************************
*
*  Routine Name: kfile_isopen - khoros transport has been 
*				properly opened
*
*       Purpose: Whether a khoros transport has been opened.
*
*         Input: file - the khoros transport to test whether the
*			transport has been opened.
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_isopen(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_isopen(file)  (kfile_state(file) & KFILE_OPEN)

/************************************************************
*
*  Routine Name: kfile_islinebuf - khoros transport is line buffered
*
*       Purpose: Whether a khoros transport is line buffered
*
*         Input: file - the khoros transport to test whether the
*			transport is line buffered
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_islinebuf(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_islinebuf(file)  (kfile_state(file) & KFILE_LINEBUF)


/************************************************************
*
*  Routine Name: kfile_isfullbuf - khoros transport is full buffered
*
*       Purpose: Whether a khoros transport is full buffered
*
*         Input: file - the khoros transport to test whether the
*			transport is full buffered
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_isfullbuf(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_isfullbuf(file)  (kfile_state(file) & KFILE_FULLBUF)


/************************************************************
*
*  Routine Name: kfile_isbufferd - khoros transport is buffered or not
*
*       Purpose: Whether a khoros transport is buffered or not
*
*         Input: file - the khoros transport to test whether the
*			transport is buffered or not
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_isbuffered(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_isbuffered(file)  (kfile_state(file) & KFILE_FULLBUF || \
			         kfile_state(file) & KFILE_LINEBUF)


/************************************************************
*
*  Routine Name: kfile_isdup - khoros transport has been 
*			       has been/is a dupped transport
*
*       Purpose: Whether a khoros transport has been or is a
*		 dupped transport.  A transport can be dupped by
*		 the routines kdup(), kdup2(), kfdup(), kfdup2().
*
*         Input: file - the khoros transport to test whether the
*			transport has been dupped.
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_isdup(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_isdup(file)  (kfile_state(file) & KFILE_DUP)


/************************************************************
*
*  Routine Name: kfile_iseof - khoros transport is at end of
*			       file (eof)
*
*       Purpose: Whether a khoros transport is currently at end of
*		 file.  A transport end of file flag can be cleared using
*		 the kclearerr() routine.
*
*         Input: file - the khoros transport to test whether the
*			transport is at end of file (eof).
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_iseof(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_iseof(file)   (kfile_state(file) & KFILE_EOF)


/************************************************************
*
*  Routine Name: kfile_islock - khoros transport has been/is a
*			        locked transport
*
*       Purpose: Whether a khoros transport has been or is a
*		 locked transport.  A transport can be locked using
*		 the KFILE_LOCK flag on open.
*
*         Input: file - the khoros transport to test whether the
*			transport has been opened lock.
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Jan 13, 1995
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_islock(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_islock(file)  (kfile_state(file) & KFILE_LOCK)


/************************************************************
*
*  Routine Name: kfile_iseof - khoros transport is at end of
*			       file (eof)
*
*       Purpose: Whether a khoros transport is currently at end of
*		 file.  A transport end of file flag can be cleared using
*		 the kclearerr() routine.
*
*         Input: file - the khoros transport to test whether the
*			transport is at end of file (eof).
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_iseof(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_iseof(file)   (kfile_state(file) & KFILE_EOF)

/************************************************************
*
*  Routine Name: kfile_isreacquire - khoros transport will be reacquired
*
*       Purpose: Whether a khoros transport will be reacquired.  This
*		 is how applications like cantata can get fast IPC
*		 between processes.
*
*         Input: file - the khoros transport to test whether the
*			transport is allowed to be reacquired
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Nov 1, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: unsigned long int kfile_isreacquire(
*		 !  kfile *file)
*
*************************************************************/
#define kfile_isreacquire(file)  (kfile_state(file) & KFILE_REACQUIRE)

#define kfile_state(file) kfile_getstate(file)
	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/*
 *  Declaration for public routines.  These routines should used instead of
 *  their system equivalents.  For the most part they have the same parameters
 *  as their system counterparts, but a few are different so you should consult
 *  the appropriate cgparm documentation.  For online doc consult the man3
 *  vgparm man page or for offline doc the section on vgparm in chapter 8 of
 *  the "Khoros Programmer's Manual".
 */

/* Remote.c */
char *kremote_exec     PROTO((const char *, const char *, char *));
char *kremote_file     PROTO((const char *, const char *, char *));
char *kremote_location PROTO((const char *, char *));
int  kremote_check     PROTO((const char *));
int  kremote_start     PROTO((const char *));
int  kremote_stop      PROTO((const char *));
int  kremote_running   PROTO((const char *));
char **kremote_list    PROTO((size_t *));
pid_t kreminfo	       PROTO((kfile *));

/* ktransport.c */
kfile  *kpopen		 PROTO((const char *, const char *));
char   *ktempnam	 PROTO((const char *, const char *));
int     krename		 PROTO((const char *, const char *));
int     kopen		 PROTO((const char *, int, kvalist));
int     kclose		 PROTO((int));
ssize_t kread		 PROTO((int, kaddr, size_t));
ssize_t kwrite		 PROTO((int, kaddr, size_t));
int     kaccess		 PROTO((const char *, int));
int     kunlink		 PROTO((const char *));
int     kcreat		 PROTO((const char *, mode_t));
int     ktouch		 PROTO((const char *, mode_t));
int     kflock		 PROTO((int, int));
int     kfcntl		 PROTO((int, int, kvalist));
off_t   klseek		 PROTO((int, off_t, int));
long int ktell		 PROTO((int));
int     kdup		 PROTO((int));
int     kdup2		 PROTO((int, int));
int     kpclose		 PROTO((kfile *));
pid_t   kpinfo		 PROTO((kfile *));
pid_t   kspawn		 PROTO((const char *));
int     ksystem		 PROTO((const char *));
int     kexecvp		 PROTO((const char *, char *const*));
int     kfileno		 PROTO((kfile *));
kfile  *kfidfile	 PROTO((int));
int     ktmpfid		 PROTO((void));
int     kgetdescriptors	 PROTO((kfile *, int *, int *));
int     kgethostname	 PROTO((char *, int));
void    kexit		 PROTO((kexit_status));
int     kexit_handler	 PROTO((void (*) PROTO((kexit_status, kaddr)),kaddr));

/* kstruct.c */
int   kstruct_undefine  PROTO((int));
int   kstruct_check     PROTO((char *));
void  kstruct_free      PROTO((kaddr, int, int));
int   kstruct_compare   PROTO((kaddr, kaddr, int));
kaddr kstruct_duplicate PROTO((kaddr, int, kaddr));
kaddr kstruct_flatten   PROTO((kaddr, int, size_t *));
kaddr kstruct_unflatten PROTO((kaddr, int, size_t));
int   kstruct_define    PROTO((const char *, size_t,
			       int  (*)PROTO((int, kaddr, int)),
			       int  (*)PROTO((int, kaddr, int)),
			       int  (*)PROTO((kaddr, kaddr, int)),
			       void (*)PROTO((kaddr, int, int))));
int kstruct_setinfo PROTO((int, const char *, ssize_t,
			   int  (*)PROTO((int, kaddr, int)),
			   int  (*)PROTO((int, kaddr, int)),
			   int  (*)PROTO((kaddr, kaddr, int)),
			   void (*)PROTO((kaddr, int, int))));
int kstruct_getinfo PROTO((int, char **, size_t *,
			   int  (**)PROTO((int, kaddr, int)),
			   int  (**)PROTO((int, kaddr, int)),
			   int  (**)PROTO((kaddr, kaddr, int)),
			   void (**)PROTO((kaddr, int, int))));
int kstruct_tmptrans PROTO((char *));


/* kstdio.c */
kfile   *kfopen		PROTO((const char *, const char *));
kfile   *kfdopen	PROTO((int, const char *));
kfile   *kfreopen	PROTO((const char *, const char *, kfile *));
kfile   *kfdup		PROTO((kfile *));
kfile   *kfdup2		PROTO((kfile *, kfile *));
kfile   *ktmpfile	PROTO((void));
char    *kfgets		PROTO((char *, int, kfile *));
char    *kgetbuffer	PROTO((kfile *, size_t *));
void     krewind	PROTO((kfile *));
void     kclearerr	PROTO((kfile *));
void     ksetvbuf	PROTO((kfile *, char *, int, size_t));
int      kscanf		PROTO((const char *, kvalist));
int      kfscanf	PROTO((kfile *, const char *, kvalist));
int      ksscanf	PROTO((const char *, const char *, kvalist));
int      kprintf	PROTO((const char *, kvalist));
int      kfprintf	PROTO((kfile *, const char *, kvalist));
int      ksprintf	PROTO((char *, const char *, kvalist));
int      kvfprintf	PROTO((kfile *, const char *, kva_list));
int      kvsprintf	PROTO((char *, const char *, kva_list));
int      kvscanf	PROTO((const char *, kva_list));
int      kvfscanf	PROTO((kfile *, const char *, kva_list));
int      kvsscanf	PROTO((const char *, const char *, kva_list));
int      kvprintf	PROTO((const char *, kva_list));
int      kfclose	PROTO((kfile *));
size_t   kfread		PROTO((kaddr, size_t, size_t, kfile *));
size_t   kfwrite	PROTO((kaddr, size_t, size_t, kfile *));
int      kfputc		PROTO((int, kfile *));
int      kputs		PROTO((const char *));
int      kfputs		PROTO((const char *, kfile *));
int      kfgetc		PROTO((kfile *));
int      kungetc	PROTO((int, kfile *));
int      kfseek		PROTO((kfile *, long int, int));
long int kftell		PROTO((kfile *));
int      kfflush	PROTO((kfile *));
int      kfeof		PROTO((kfile *));

/*
 *  Other utilities that maybe handy but are usually needed only by more
 *  complex applications that wish to do more than the basic transport/
 *  remote execution.
 */
/* kfile.c */
kaddr             kfile_readdata      PROTO((int, kaddr, size_t *));
char             *kfile_filename      PROTO((int));
char             *kfile_type	      PROTO((kfile *));
kfile            *kfile_reopen	      PROTO((char *, char *, kfile *));
unsigned long int kfile_getstate      PROTO((kfile *));
mode_t            kfile_mode	      PROTO((int));
int               kfile_closeall      PROTO((void));
int               kfile_getpermanence PROTO((char *));
int               kfile_getmachtype   PROTO((int));
int               kfile_setmachtype   PROTO((int, int));
int               kfile_remotehost    PROTO((char *));
ssize_t           kfile_writedata     PROTO((int, kaddr, size_t));
int               kfile_seddata	      PROTO((int, int, kvalist));
ssize_t           kfile_copydata      PROTO((int, int, size_t));
int               kfile_comparedata   PROTO((int, int, size_t, size_t *));
void              kfile_listall	      PROTO((void));
int               kfile_checkbuf      PROTO((kfile *));
unsigned long int kfile_flags         PROTO((int));
unsigned long int kfile_setstate      PROTO((kfile *, unsigned long int));
unsigned long int kfile_clrstate      PROTO((kfile *, unsigned long int));

char  *kflags_to_type	   PROTO((int, char *));
int   ktype_to_flags	   PROTO((const char *));
char  **kcommand_to_args   PROTO((const char *));

char **ktransport_list	     PROTO((int, int, int, size_t *));
char  *ktransport_identifier PROTO((const char *, char *));
char  *ktransport_path       PROTO((const char *, char *));
char  *ktransport_command    PROTO((const char *, char *));
char  *ktransport_location   PROTO((const char *, char *));
char  *ktransport_username   PROTO((const char *, char *));
char  *ktransport_remotecmd  PROTO((const char *, char *));

/*
 *  Transport negotiator which can be used by processes that wishes to
 *  arbitrate or simply recieve information about what transport state
 *  a process maybe in.  (developed with cantata in mind)
 */
/* knegotiator.c */
int  knegotiate_transport      PROTO((int));
int  knegotiate_continuous_run PROTO((void));
int  knegotiate_check	       PROTO((const char *));
int  knegotiate_open	       PROTO((const char *, int));
int  knegotiate_close	       PROTO((const char *, int));
void knegotiate_initialize     PROTO((kaddr, 
				      void (*) PROTO((kaddr)),
				      void (*) PROTO((kexit_status, kaddr)),
				      int  (*) PROTO((kaddr, char *, int))));

/* kipc.c */
kfile *kipc_getipc	PROTO((void));
kfile *kipc_start	PROTO((void));
int    kipc_stop	PROTO((void));
void   kipc_enable	PROTO((int));

/* khost.c */
char **khost_list        PROTO((size_t *));
char *khost_location     PROTO((const char *, char *));
char *khost_username     PROTO((const char *, char *));
char *khost_remotecmd    PROTO((const char *, char *));
char *khost_entry        PROTO((const char *, char *));
int khost_add            PROTO((const char *));
int khost_delete         PROTO((const char *));
int khost_save           PROTO((const char *));

#endif /* _kutils_kfile_h_ */
/* don`t add after the endif */
