 /*
  * Khoros: $Id: kmachine.h,v 1.6 1997/01/20 03:49:58 mirage Exp $
  */

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: This file contains the Platform data
   >>>>		       ordering definitions.
   >>>>
   >>>>    Written By: Jeremy Worley & Mark Young & John Salas
   >>>>
   >>>>          Date: Jul 16, 1994
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

/*
 * The Khoros 2.0 machine type description uses a single byte, so that
 * it can be recognized without knowing the format a priori.  The
 * upper three bits are used to encode the word ordering.  The lower 5
 * bits are used to encode the floating point format.  Also, since
 * IEEE has so many variants (which typically differ only by word size
 * chosen for some of the datatypes), we encode variants of IEEE in
 * the floating point format portion of the byte.  The IEEE format
 * refers to the generic 32bit ieee format in which shorts are 2
 * bytes, ints, longs, and floats are 4 bytes and doubles are 8 bytes.
 * If a new format is added that is essentially IEEE, but uses
 * different word sizes or is on a 64bit machine, etc. it is given its
 * own format type.  Note that we are going to try to reserve the mach
 * bytes with odd values for IEEE based machines.  Right now, only the
 * generic 32bit IEEE and the DEC Alpha IEEE variant are specified.
 *
 * NOTE: The VAX is basically little endian, but its floating point
 * values are also reordered on half word boundaries.  We don't
 * consider this a unique word ordering, even though there is a pretty
 * good argument that it should be.  The conversion code in kconvert.c
 * in kutils copes with this peculiarity internally.
 *
 *
 * 	bit  7   6   5   4   3   2   1   0
 *	   +---+---+---+---+---+---+---+---+
 *	   | o | o | o | f | f | f | f | f |
 *	   +---+---+---+---+---+---+---+---+
 *
 *	o = order
 *		0x00               --> unknown
 *		0x01 << 5  (0x200) --> big endian
 *		0x02 << 5  (0x400) --> little endian
 *		...
 *
 *	f = floating point format
 *
 *		0x00	   --> unknown
 *		0x01	   --> ieee
 *		0x02	   --> vax
 *		0x03	   --> alpha ieee
 *		0x04	   --> cray/cyber
 *		... 
 */

#ifndef _machine_kmachine_h_
#define _machine_kmachine_h_

        /*-------------------------------------*
        |       #include 
        ---------------------------------------*/

        /*-------------------------------------*
        |       #defines 
        ---------------------------------------*/

#define KFORMAT_UNKNOWN      0x0
#define KFORMAT_IEEE         0x1
#define KFORMAT_VAX          0x2
#define KFORMAT_IEEE_ALPHA   0x3
#define KFORMAT_CRAY         0x4
#define KFORMAT_IEEE_T3E     0x5

#define KORDER_UNKNOWN       0x00
#define KORDER_BIG_ENDIAN    0x20
#define KORDER_LITTLE_ENDIAN 0x40



/*
 *  Defines for raw storage format of the different machine architecture
 *  definitions
 */
#define KMACH_UNKNOWN	0x0
#define KMACH_LITTLE_ENDIAN_IEEE  (KORDER_LITTLE_ENDIAN | KFORMAT_IEEE)
#define KMACH_LITTLE_ENDIAN_VAX   (KORDER_LITTLE_ENDIAN | KFORMAT_VAX)
#define KMACH_LITTLE_ENDIAN_ALPHA (KORDER_LITTLE_ENDIAN | KFORMAT_IEEE_ALPHA)
#define KMACH_BIG_ENDIAN_IEEE	  (KORDER_BIG_ENDIAN | KFORMAT_IEEE)
#define KMACH_BIG_ENDIAN_CRAY  	  (KORDER_BIG_ENDIAN | KFORMAT_CRAY)
#define KMACH_BIG_ENDIAN_IEEE_T3E (KORDER_BIG_ENDIAN | KFORMAT_IEEE_T3E)

              /* This one for the Dec Alpha */
#ifdef KARCH_ALPHA
#define KMACH_LOCAL KMACH_LITTLE_ENDIAN_ALPHA
#endif
              /* This one for the Vax */
#ifdef KARCH_VAX
#define KMACH_LOCAL KMACH_LITTLE_ENDIAN_VAX
#endif
              /* This one for the Sun Solaris Sparc */
#ifdef KARCH_SPARC
#define KMACH_LOCAL KMACH_BIG_ENDIAN_IEEE
#endif
	      /* This one for the Sun Solaris i386 */
              /* This works for the Intel 386/486*/
#ifdef KARCH_I80x86
#define KMACH_LOCAL KMACH_LITTLE_ENDIAN_IEEE
#endif
              /* This one for Sony */
              /* This one for NeXT */
              /* This one for MC680x0 */
              /* This works for the Mac */
#ifdef KARCH_MC680x0
#define KMACH_LOCAL KMACH_BIG_ENDIAN_IEEE
#endif
	       /* This one is for the MIPS chips running in big endian */
#ifdef KARCH_MIPSEB
#define KMACH_LOCAL KMACH_BIG_ENDIAN_IEEE	
#endif
               /* This one is for the ENCORE and SEQUENT */
#ifdef KARCH_NS32000
#define KMACH_LOCAL KMACH_LITTLE_ENDIAN_IEEE
#endif
	       /* This one is for the MIPS chips running in little endian */
#ifdef KARCH_MIPSEL
#define KMACH_LOCAL KMACH_LITTLE_ENDIAN_IEEE
#endif
               /* This works for APOLLO DN10000 */
#ifdef KARCH_APOLLO
#define KMACH_LOCAL KMACH_BIG_ENDIAN_IEEE
#endif
	       /* this works for hp9000s{300,400,700,800} */
#ifdef KARCH_PARISC
#define KMACH_LOCAL KMACH_BIG_ENDIAN_IEEE
#endif
               /* This works for the cyber-like crays */
#ifdef KARCH_CRAY
#define KMACH_LOCAL KMACH_BIG_ENDIAN_CRAY
#endif

		/* This works for the cray t3e and *maybe* the t3d */
#ifdef KARCH_CRAY_T3E
#define KMACH_LOCAL KMACH_BIG_ENDIAN_IEEE_T3E
#endif

               /* This works for ibm 6000 */
#ifdef KARCH_RS6000
#define KMACH_LOCAL KMACH_BIG_ENDIAN_IEEE
#endif
               /* This works for the Convex */
#ifdef KARCH_CONVEX
#define KMACH_LOCAL KMACH_BIG_ENDIAN_IEEE
#endif
               /* This works for the Data General */
               /* This works for the Luna 88000 */
               /* This works for the Motorola 88000 */
#ifdef KARCH_M88x00
#define KMACH_LOCAL KMACH_BIG_ENDIAN_IEEE
#endif
              /* This works for the Paragon */
#ifdef KARCH_I860
#define KMACH_LOCAL KMACH_LITTLE_ENDIAN_IEEE
#endif

               /* unknown type */
#if !defined(KMACH_LOCAL)
#define KMACH_LOCAL KMACH_UNKNOWN
#endif
        /*-------------------------------------*
        |       typedefs
        ---------------------------------------*/

        /*-------------------------------------*
        |       global variable declarations
        ---------------------------------------*/

        /*-------------------------------------*
        |       macros
        ---------------------------------------*/

/*
 * these macros return the bits that correspond to the format portion
 * of a machtype byte and the order portion of the machtype byte.
 */
#define KMACH_FORMAT(machtype) (machtype & 0x1f)
#define KMACH_ORDER(machtype)  (machtype & 0xe0)

        /*-------------------------------------*
        |       routine definitions
        ---------------------------------------*/


#endif /* _machine_kmachine_h_ */
/* Don't add after this point */
