/*
 * Khoros: $Id: utilities.c,v 1.6 1997/01/20 03:53:20 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.6 1997/01/20 03:53:20 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */
 

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>             _static_routines()
   >>>>  Public:
   >>>>             public_routines()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "kecho.h"

unsigned long reqtype = KCMS_KROUTINE;

/*-----------------------------------------------------------
| Routine Name: print_object_info - print software object information
|
| Purpose:	This function takes a software object, and prints out a
|		number of object attributes, in a format which is
|		conducive to being parsed.
|
| Input:	object - software object to print information for
| Returns:	TRUE (1) if the object information was successfully echoed,
|		FALSE (0) otherwise.
|
| Written By:	Neil Bowers
| Date:		18-aug-94
------------------------------------------------------------*/
int
print_object_info(
   kobject  object)
{
   kobject       toolbox;
   kobject       tbname;
   kstring       oname;
   kstring       category;
   kstring       subcategory;
   kstring       icon;
   kstring       description;
   kstring       bname;
   int           installed;
   unsigned long otype;


   if (!kcms_get_attributes(object,
			    KCMS_NAME,               &oname,
			    KCMS_PARENT,             &toolbox,
			    KCMS_CMOBJ_CATEGORY,     &category,
			    KCMS_CMOBJ_SUBCATEGORY,  &subcategory,
			    KCMS_CMOBJ_ICON_NAME,    &icon,
			    KCMS_CMOBJ_BNAME,        &bname,
			    KCMS_CMOBJ_SHORT_DESC,   &description,
			    KCMS_CMOBJ_CI_INSTALLED, &installed,
			    KCMS_CMOBJ_PROGTYPE,     &otype,
			    KCMS_END)
       || !kcms_get_attribute(toolbox, KCMS_NAME, &tbname))
      return FALSE;

   if (clui_info->type_flag && otype != reqtype)
      return TRUE;

   kprintf("begin %s::%s\n", tbname, oname);
   kprintf("\tbinary-name=%s\n", (bname != NULL ? bname : ""));
   kprintf("\tcategory=%s\n",    (category != NULL ? category : ""));
   kprintf("\tsubcategory=%s\n", (subcategory != NULL ? subcategory : ""));
   kprintf("\tshort-description=%s\n", (description != NULL?description : ""));
   kprintf("\ticon-name=%s\n", (icon != NULL ? icon : ""));
   kprintf("\tin-cantata=%s\n", (installed ? "YES" : "NO"));
   kprintf("end %s::%s\n", tbname, oname);

   return TRUE;
}

/*-----------------------------------------------------------
| Routine Name: kecho_print_keywords - print object keywords
|
| Purpose:	This function takes a toolbox or software object,
|		and prints out any keywords defined for the object.
|
| Input:	object - object to print keywords from
| Returns:	TRUE (1) if the keywords were successfully printed.
|		FALSE (0) otherwise.
|
| Written By:	Neil Bowers
| Date:		25-oct-1995
------------------------------------------------------------*/
int
kecho_print_keywords(
   kobject  object)
{
   kstring    routine       = "kecho_print_keywords()";
   int        otype;
   kcms_attr  attribute;
   klist     *keyword_list;
   kstring    keyword;


   if (!kcms_get_attribute(object, KCMS_TYPE, &otype))
      return FALSE;

   if (otype == KOBJ_CMSTB)
      attribute = KCMS_TB_KEYWORDS;
   else if (otype == KOBJ_CMSOBJ)
      attribute = KCMS_CMOBJ_KEYWORDS;
   else
   {
      errno = KCMS_EINTERNAL;
      kerror(NULL, routine, "Unexpected object type (%d), "
			    "expecting toolbox or software object!\n", otype);
      return FALSE;
   }

   if (!kcms_get_attribute(object, attribute, &keyword_list))
      return FALSE;

   for (; keyword_list != NULL; keyword_list = klist_next(keyword_list))
   {
      keyword = (kstring)klist_clientdata(keyword_list);
      if (keyword == NULL)
         continue;
      kprintf("%s\n", keyword);
   }

   return TRUE;
}
