/*
 * Khoros: $Id: do_pane.c,v 1.10 1997/02/07 22:56:52 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_pane.c,v 1.10 1997/02/07 22:56:52 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>  Private:
   >>>>             pane_command()
   >>>>  Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "kexec.h"
#include "kforms/kforms.h"
#include "kforms/kformsP.h"

extern uis_struct kstdargs_uis_spec; 

int kgen_clui_get_pane_info       PROTO((kform *));
void kgen_clui_print_usage        PROTO((kfile *));

/*-----------------------------------------------------------
|
|  Routine Name: pane_command  - get the pane command
|
|       Purpose: This routine creates a command according to the pane's
|		 form.
|
|         Input: oname     - the object name
|		 toolbox   - the toolbox name
|		 pane_path - the path to the pane to be executed
|		 arguments - the arguments to the pane object
|        Output: 
|       Returns: the resulting command to be executed or NULL on failure
|
|    Written By: Mark Young & Danielle Argiro
|          Date: Feb 25, 1994
| Modifications:
|
------------------------------------------------------------*/

int pane_command(
  char   *tbname,
  char   *oname,
  char   *pane_path,
  char   *arguments,
  char   *command)
{
	ssize_t i, ac;
	kform  *pane;
	char   *cmdline, **av;
	char    temp[KLENGTH];
	char    extra_arg[KLENGTH];

	kprog_set_toolbox(tbname);
	kprog_set_program(oname);
	av = kparse_string_delimit(arguments, " ", KDELIM_CLEAN, &ac);
	av = (char **) karray_insert(av, KSTRING, oname, (size_t) ac,
				     KLIST_HEAD, TRUE);
	kprog_set_argc((int)(ac+1)); kprog_set_argv(av);

	if ((pane = kgen_initialize(pane_path, KGEN_PANE, 
				     tbname, oname, NULL)) == NULL)
	{
	   return(FALSE);
	}
	cmdline = kvf_form_to_cmd(pane, pane->subform, NULL, TRUE);

	if (!(kclui_consolidate_args(pane)))
              return(FALSE);

	kvf_destroy_form(pane);
	ac = (ssize_t) kprog_get_argc();
	av = kprog_get_argv();

	ksprintf(temp, cmdline);
	kfree_and_NULL(cmdline);
	if (ac > 1)
	{
	   i = 1;
	   while (i < ac) 
	   {
	      ksprintf(extra_arg, " %s", av[i++]);
	      (void) kstring_cat(temp, extra_arg, temp);
	   }
	}

	kstrcpy(command, temp); 
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: print_pane_usage - print the usage for a pane object
|
|       Purpose: This routine replicates code from kcodegen, but having
|	  	 it here allows us to avoid linking cms into every 
|		 kroutine.  
|
|         Input: pane_path - path to the pane file
|		 program   - the pane software object
|		 short_usage - if TRUE, print the short usage, 
|		 	       otherwise, print the long usage	
|        Output: prints the usage to kstderr
|       Returns: 
|
|    Written By: Steve Jorgensen & Steve Kubica
|          Date: Feb 3, 1997
| Modifications:
|
------------------------------------------------------------*/

void print_pane_usage(
    char       *pane_path,
    kobject     program,
    int         short_usage)
{
        kform   *form = NULL;
	char    *prog_desc = NULL;
	char    *oname = NULL;

	if ((form = kvf_create_form(pane_path, KNONE, NULL, NULL)) == NULL)
	{
	    kerror("kexec", "print_pane_usage",
		   "Can't interpret syntax on NULL pane!");
	    kexit(KEXIT_FAILURE);
	}

	if (short_usage)
	   kfprintf(kstderr,"(Brief usage;  use [-usage] for standardized "
		    "Khoros arguments)\n\n");
	else
	   kvf_append_selections(form, &kstdargs_uis_spec);

	/*
	 *  get information off the pane: set global variables such as
         *  kgen_req_sel, kgen_opt_sels printing usage.
	 */
        if (!(kgen_clui_get_pane_info(form)))
        {
            kerror("kexec", "print_pane_usage", 
		  "Unable to obtain UIS information from *.pane form tree\n");
            kexit(KEXIT_FAILURE);
        }

	/*  
	 *  a pane object has no usage_additions routine (or any code
	 *  of its own) so we have to rely on kcms to get the short
	 *  program description * from the -M line of the *.pane UIS
	 *  file.  prone to failure, but give it a try anyway, and
	 *  print if we find anything.
	 */
	if (program != NULL)
	   kcms_get_attributes(program, 
			       KCMS_CMOBJ_SHORT_DESC, &prog_desc,
			       KCMS_NAME, &oname,
			       KCMS_END);

	kfprintf(kstderr, "Usage for %s: ", oname);

	if (prog_desc != NULL)
	   kfprintf(kstderr, "%s", prog_desc);

	kfprintf(kstderr, "\n%% %s\n", oname);

	/*
	 * kgen_clui_print_usage() does the real work of printing usage.
	 */
        kgen_clui_print_usage(kstderr);

	kexit(KEXIT_SUCCESS);
}
