 /*
  * Khoros: $Id: kexec.c,v 1.21.2.3 1997/05/31 20:52:15 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kexec.c,v 1.21.2.3 1997/05/31 20:52:15 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for kexec
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "kexec.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Execute a process
|
|       Purpose: main program for kexec
|
|         Input:
|		char *clui_info->cmd_string; {command to be executed}
|		int   clui_info->cmd_flag; {TRUE if -cmd specified}
|
|		char *clui_info->args_string; {command arguments}
|		int   clui_info->args_flag; {TRUE if -args specified}
|
|		char *clui_info->m_string; {machine}
|		int   clui_info->m_flag; {TRUE if -m specified}
|
|		M.I. group:
|		    char *clui_info->tb_string; {toolbox to execute object from}
|		    int   clui_info->tb_flag; {TRUE if -tb specified}
|
|		    char *clui_info->oname_string; {Program Object Name to be executed}
|		    int   clui_info->oname_flag; {TRUE if -oname specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	int	showgui = FALSE;
	char  *temp, *tbname, *oname, *arguments, command[KLENGTH * 4],
	      **args, leftover[KLENGTH * 4], *path = NULL, *wksp_path = NULL;
	kobject toolbox = NULL, program = NULL, paneobj = NULL, fileobj = NULL;
	kform *pane;
	unsigned long objtype;
/* -main_variable_list_end */

	khoros_init(argc, argv, "BOOTSTRAP", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$BOOTSTRAP/objects/kroutine/kexec");
	kexit_handler(kexec_free_args, NULL);

/* -main_get_args_call */
        /*
         *      IMPORTANT NOTE  (Danielle, Jan 20, 1997):
         *
         * kexec must call kgen_init, kexit_handler, kman_get_args and
         * kvf_destroy_form itself, rather than making one call to kclui_init.
         * The reason for this is that for when programs are called with the
	 * [-gui] option, kexec will need to call xvrun with extra 
	 * arguments that are meant for the program that xvrun is to execute,
	 * not for kexec or xvrun.  these will be rejected by kclui_init.
         */
        pane = kgen_init("BOOTSTRAP", "kexec", KGEN_NONE, 
                         &clui_uis_spec, kexec_usage_additions);
        kexec_get_args(pane);
        kvf_destroy_form(pane);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	if (!clui_info->tb_flag && !clui_info->oname_flag)
	{
	   ktoolbox_initialize();
	   command[0] = '\0';
	   if (clui_info->cmd_string != NULL)
	      kstrcat(command, clui_info->cmd_string);

	   if (clui_info->args_string != NULL)
	   {
	      kstrcat(command, " ");
	      kstrcat(command, clui_info->args_string);
	   }
	   (void) kclui_arguments(leftover);
	   kstrcat(command, " ");
	   kstrcat(command, leftover);
	}
	else
	{
	   tbname = clui_info->tb_string;
	   oname  = clui_info->oname_string;
	   if ((toolbox = kcms_open_toolbox(tbname)) == NULL ||
	       (program = kcms_open_cmobj(toolbox, oname)) == NULL)
	   {
	      (void) kerror(NULL, "main()", "Cannot open %s '%s'.",
			(!toolbox) ? "toolbox" : "program",
			(!toolbox) ? clui_info->tb_string :
			clui_info->oname_string);
	      kexit(KEXIT_FAILURE);
	   }

	   /*
	    *  Get the pane in order to pass it to pane_command()
	    */
	   kcms_get_attribute(program, KCMS_CMOBJ_UIS_PANE, &paneobj);
	   if (paneobj == NULL)
	   {
              kerror(NULL, "main()", "Object (%s) has no pane object",
		     clui_info->oname_string);
	      kexit(KEXIT_FAILURE);
	   }
	   kcms_get_attribute(paneobj, KCMS_PATH, &path);
	   arguments = clui_info->args_string;

	   /*  a pane object has no usage_additions routine (or any
	    *  code of its own) so we have to rely on kcms to get
	    *  the short program description * from the -M line of the
	    *  *.pane UIS file.  prone to failure, but give it a try
	    *  anyway, and print if we find anything.
	    */
	   kcms_get_attribute(program, KCMS_CMOBJ_TYPE, &objtype);
	   
	   if (objtype == KCMS_PANE)
	   {
	      if (kstrstr(arguments, "-usage") != NULL)
		 print_pane_usage(path, program, FALSE);

	      if (kstrstr(arguments, "-U") != NULL)
		 print_pane_usage(path, program, TRUE);
	   }
	   

	   if (kcms_get_attribute(program, KCMS_CMOBJ_WORKSPACE, &fileobj) &&
	       fileobj && kcms_get_attribute(fileobj, KCMS_PATH, &wksp_path) &&
	       wksp_path != NULL)
	   {
	      if (kstrstr(arguments, "-gui") != NULL)
	      {
		 showgui = TRUE;
		 kstring_replace(arguments, "-gui", NULL, arguments);
	      }
	   }

           if (showgui == FALSE && pane_command(tbname, oname, path,
			arguments, command) == FALSE)
	   {
              kexit(KEXIT_FAILURE);
	   }

	   /*
	    *  If this pane object is specialized to be an encapsulated
	    *  workspace object, call runwksp to run the workspace.
	    */
	   if (wksp_path != NULL)
	   {
	      if (arguments != NULL)
	      {
	         ksprintf(command, "runwksp -hidden -wksp %s -args '%s'",
			wksp_path, arguments);
	      }
	      else
	         ksprintf(command, "runwksp -hidden -wksp %s", wksp_path);

	      if (showgui)
	         kstrcat(command, " -showgui");
	   }
	   kcms_close(program); kcms_close(toolbox);
	}
/* -main_before_lib_call_end */

/* -main_library_call */
	if ((temp = kremote_exec(clui_info->m_string, command, NULL)) == NULL)
	{
	   kerror(NULL, "main()", "Unable to build command for exec'ing.  The \
command string is '%s'\nand the machine specified is '%s'\n", command,
		(!clui_info->m_string ? "<local>" : clui_info->m_string));
	   kexit(KEXIT_FAILURE);
	}
	kinfo(KVERBOSE,"kexec: '%s'\n", temp);
	temp = kfullpath(temp, NULL, NULL);
	args = kcommand_to_args(temp);
	kexecvp(args[0], args);
	kerror(NULL, "main()", "Unable to execute command:\n '%s'", command);
	kexit(KEXIT_FAILURE);
	/* never will get here, but oh well... */
/* -main_library_call_end */

/* -main_after_lib_call */
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: kexec_usage_additions
| 
|       Purpose: Prints usage additions in kexec_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kexec
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void kexec_usage_additions(void)
{
	kfprintf(kstderr, "\tExecute a process\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: kexec_free_args
| 
|       Purpose: Frees CLUI struct allocated in kexec_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kexec
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
kexec_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->cmd_string);
		kfree_and_NULL(clui_info->args_string);
		kfree_and_NULL(clui_info->m_string);
		kfree_and_NULL(clui_info->tb_string);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
