 /*
  * Khoros: $Id: rpc_cout.c,v 1.15.2.1 1997/03/12 00:10:04 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: rpc_cout.c,v 1.15.2.1 1997/03/12 00:10:04 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>      	      RPC .c Generator Code
   >>>>
   >>>>	  This was khoros-ized from the file rpc_cout.c 
   >>>>   in the program rpcgen by Sun Microsystems, Inc. 
   >>>>
   >>>>   Static:
   >>>>			findtype()
   >>>>			undefined()
   >>>>			emit_reader()
   >>>>			emit_writer()
   >>>>			emit_match()
   >>>>			emit_free()
   >>>>			emit_header()
   >>>>			emit_code()
   >>>>			emit_trailer()
   >>>>			emit_define()
   >>>>			emit_enum()
   >>>>			emit_union()
   >>>>			emit_struct()
   >>>>			emit_typedef()
   >>>>			print_ifstat()
   >>>>			print_stat()
   >>>>  Private:
   >>>>			emit_definition()
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

/* @(#)rpc_cout.c	2.1 88/08/01 4.0 RPCSRC */
/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user.
 * 
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

/*
 * rpc_cout.c, XDR routine outputter for the RPC protocol compiler 
 * Copyright (C) 1987, Sun Microsystems, Inc. 
 */
#include "kgenstruct.h"

static void emit_reader    PROTO((definition *));
static void emit_writer    PROTO((definition *));
static void emit_match     PROTO((definition *));
static void emit_free      PROTO((definition *));
static void emit_define    PROTO((definition *));
static void emit_code      PROTO((definition *, int));
static void emit_enum      PROTO((definition *, int));
static void emit_union     PROTO((definition *, int));
static void emit_struct    PROTO((definition *, int));
static void emit_typedef   PROTO((definition *, int));

static void print_ifstat   PROTO((int, char *, char *, relation, char *, 
				  char *, char *, int));
static void print_stat     PROTO((declaration *, int));

/*-- GLOBAL variables --*/
#define KEMIT_READ  0
#define KEMIT_WRITE 1
#define KEMIT_MATCH 2
#define KEMIT_FREE  3

/****    =============================================================
 ****    Routine Emitters
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: emit_definition - emit all C code for a given definition
|
|       Purpose: This routine will emit all the necessary C
|		 code to support a given defintion.
|
|		 The generated code will consist of :
|	
|	           o  an integer KTYPE type identifier
|		   o  a reader function
|		   o  a writer function
|		   o  a compare function
|		   o  a free function
|		   o  a define function
|
|         Input: def - the definition
|
|        Output: prints into the global file specified by fout
|
|       Returns: nothing
|
|    Written By: ported from rpcgen
|          Date: August 19, 1994
| Modifications:
|
------------------------------------------------------------*/
void
emit_definition(definition *def)
{
   /*-- sanity check --*/
   if (def == NULL)
      return;
   
   /*-- emit no C code (only header code) for programs or constants --*/
   if (def->def_kind == DEF_PROGRAM || def->def_kind == DEF_CONST) 
      return;

   /*-- print a header comment --*/
   kfprintf(fout, "\n\n\n/****\n ****    --- %s routines ---\n ****/\n", 
	    def->def_name);
   
   /*-- print the type identifier --*/
   kfprintf(fout, "\n/*-- type identifier for %s --*/\n", def->def_name);
   kfprintf(fout, "int %s = KUNDEFINED;\n\n\n", type_name(def->def_name));

   /*--------------------------------------------------------------------
   |    emit reader writer match and free functions
   | . . . . . . . . . . . . . 
   |    structures and enums always have all of these functions.
   |
   |    typedefs are special :
   |       if they are typedefs to most, do not generate a free
   |       if they are typedefs to pointers, generate everything
   \---------------------------------------------------------------------*/

   /*-- these cases will be set equal to their original case --*/
   if ((def->def_kind != DEF_TYPEDEF && def->def_kind != DEF_ENUM) ||
       is_typedef_pointer(def))
   {
      emit_reader(def);
      emit_writer(def);
      emit_match(def);
      emit_free(def);
   }
   
   /*-- emit the define function --*/
   emit_define(def);
   
   return;
}


/*-----------------------------------------------------------
|  Routine Name: emit_reader - emit reader function for definition
|         Input: def - the definition 
|        Output: prints into the global file specified by fout
|       Returns: nothing
|    Written By: ported from rpcgen
|          Date: August 19, 1994
------------------------------------------------------------*/
static void
emit_reader(definition *def)
{
   char s[8]; /* 'struct' */
   
   s[0] = '\0';

   kfprintf(fout, "static int\nkread_%s(\n", def->def_name);
   kfprintf(fout, "   int    file,\n   kaddr  data,\n   int    type)\n{\n");

   if (def->def_kind == DEF_STRUCT)
      ksprintf(s, "struct ");

   kfprintf(fout, "   %s%s *objp = (%s%s *) data;\n\n", 
	    s, def->def_name, s, def->def_name);

   emit_code(def, KEMIT_READ);

   kfprintf(fout, "\n   return TRUE;\n}\n\n\n");
   return;
}

/*-----------------------------------------------------------
|  Routine Name: emit_writer - emit writer function for definition
|         Input: def - the definition 
|        Output: prints into the global file specified by fout
|       Returns: nothing
|    Written By: ported from rpcgen
|          Date: August 19, 1994
------------------------------------------------------------*/
static void
emit_writer(definition *def)
{
   char s[8]; /* 'struct' */
   
   s[0] = '\0';

   kfprintf(fout, "static int\nkwrite_%s(\n", def->def_name);
   kfprintf(fout, "   int    file,\n   kaddr  data,\n   int    type)\n{\n");

   if (def->def_kind == DEF_STRUCT)
      ksprintf(s, "struct ");

   kfprintf(fout, "   %s%s *objp = (%s%s *) data;\n\n", 
	    s, def->def_name, s, def->def_name);
   
   emit_code(def, KEMIT_WRITE);

   kfprintf(fout, "\n   return TRUE;\n}\n\n\n");
   return;
}

/*-----------------------------------------------------------
|  Routine Name: emit_match - emit match function for definition
|         Input: def - the definition 
|        Output: prints into the global file specified by fout
|       Returns: nothing
|    Written By: ported from rpcgen
|          Date: August 19, 1994
------------------------------------------------------------*/
static void
emit_match(definition *def)
{
   char s[8]; /* 'struct' */
   
   s[0] = '\0';

   kfprintf(fout, "static int\nkmatch_%s(\n", def->def_name);
   kfprintf(fout, "   kaddr  data1,\n   kaddr  data2,\n   int    type)\n{\n");

   if (def->def_kind == DEF_STRUCT)
      ksprintf(s, "struct ");

   if (is_typedef_pointer(def) && is_c_type(def->def.ty.old_type))
   {
      kfprintf(fout, "   %s%s *d1  = (%s%s *) data1;\n",
	       s, def->def_name, s, def->def_name);
      kfprintf(fout, "   %s%s *d2  = (%s%s *) data2;\n",
	       s, def->def_name, s, def->def_name);
      kfprintf(fout, "   %s%s obj1, obj2;\n\n", s, def->def_name);
      kfprintf(fout, "   obj1 = (d1 != NULL) ? *d1 : NULL;\n");
      kfprintf(fout, "   obj2 = (d2 != NULL) ? *d2 : NULL;\n\n");
   }
   else
   {
      kfprintf(fout, "   %s%s *obj1 = (%s%s *) data1;\n", 
	       s, def->def_name, s, def->def_name);
      kfprintf(fout, "   %s%s *obj2 = (%s%s *) data2;\n\n", 
	       s, def->def_name, s, def->def_name);
   }

   emit_code(def, KEMIT_MATCH);

   kfprintf(fout, "\n   return TRUE;\n}\n\n\n");
   return;
}

/*-----------------------------------------------------------
|  Routine Name: emit_free - emit free function for definition
|         Input: def - the definition 
|        Output: prints into the global file specified by fout
|       Returns: nothing
|    Written By: ported from rpcgen
|          Date: August 19, 1994
------------------------------------------------------------*/
static void
emit_free(definition *def)
{
   char s[8]; /* 'struct' */
   
   s[0] = '\0';

   kfprintf(fout, "static void\nkfree_%s(\n", def->def_name);
   kfprintf(fout, "   kaddr  data,\n   int    type,\n   int    free_self)\n{\n");

   if (def->def_kind == DEF_STRUCT)
      ksprintf(s, "struct ");

   kfprintf(fout, "   %s%s *objp = (%s%s *) data;\n\n", 
	    s, def->def_name, s, def->def_name);

   if (def->def_kind != DEF_TYPEDEF)
   {
      emit_code(def, KEMIT_FREE);

      kfprintf(fout, "\n   if (free_self)");
      kfprintf(fout, "\n      kstruct_free((kaddr)objp, KNONE, TRUE);\n");
   }
   else   /*-- typedefed pointers *must* always free --*/
   {
      kfprintf(fout, "\n   if (objp != NULL && *objp != NULL)");

      if (is_c_type(def->def.ty.old_type))
	 kfprintf(fout, "\n      kstruct_free((kaddr)*objp, KNONE, TRUE);\n");
      else
	 kfprintf(fout, "\n      kstruct_free((kaddr)*objp, %s, TRUE);\n", 
		  type_name(def->def.ty.old_type));
   }
   
   
   kfprintf(fout, "\n   return;\n}\n\n\n");
   return;
}

/*-----------------------------------------------------------
|  Routine Name: emit_define - emit define function for definition
|         Input: def - the definition 
|        Output: prints into the global file specified by fout
|       Returns: nothing
|    Written By: ported from rpcgen
|          Date: August 19, 1994
------------------------------------------------------------*/
static void
emit_define(definition *def)
{
   char versioned_name[KLENGTH];

   if (def->def_version == NULL)
      ksprintf(versioned_name, "%s", type_name(def->def_name));
   else
      ksprintf(versioned_name, "%s version %s", 
	       type_name(def->def_name), def->def_version);

   kfprintf(fout, "\n\nint\n");

   kfprintf(fout, "kdefine_%s(void)\n", def->def_name);
   kfprintf(fout, "{\n");

   kfprintf(fout, "   /*-- don't define if it has already been defined --*/");
   kfprintf(fout, "\n   if (%s == KUNDEFINED)", type_name(def->def_name));
   kfprintf(fout, "\n   {");


   if ((def->def_kind == DEF_TYPEDEF || def->def_kind == DEF_ENUM) &&
       !is_typedef_pointer(def))
   {
      if (def->def_kind == DEF_TYPEDEF)
      {
	 /*-- define thing being typedefed FIRST --*/
	 if (is_structure(def->def.ty.old_type))
	    kfprintf(fout, "\n      kdefine_%s();\n", def->def.ty.old_type);

	 kfprintf(fout, "\n      %s = ", type_name(def->def_name));
	 kfprintf(fout, "%s;\n", type_name(def->def.ty.old_type));
      }
      else /*-- enum --*/
      {
	 kfprintf(fout, "\n      %s = KINT;\n", type_name(def->def_name));
      }
   }
   else
   {
      kfprintf(fout, "\n      if ((%s = kstruct_define(\"%s\",", 
	       type_name(def->def_name), versioned_name);

      switch (def->def_kind)
      {
	 case DEF_STRUCT :
	    kfprintf(fout, " sizeof(struct %s), ", def->def_name);
	    kfprintf(fout, "kread_%s, kwrite_%s, kmatch_%s, kfree_%s", 
		     def->def_name, def->def_name, def->def_name, 
		     def->def_name);
	    break;

	 case DEF_TYPEDEF :
	    kfprintf(fout, " sizeof(%s), kread_%s, kwrite_%s, kmatch_%s, ",
		     def->def_name, def->def_name, def->def_name, 
		     def->def_name);

	    /*-- non-pointer typedefs can't be freed cause they are static --*/
	    /* if (is_typedef_pointer(def)) */
	    kfprintf(fout, "kfree_%s", def->def_name);
	    /* else 
	       kfprintf(fout, "NULL"); */
	    
	    break;

	 case DEF_ENUM :
	    kfprintf(fout, " sizeof(%s), ", def->def_name);
	    kfprintf(fout, "kread_%s, kwrite_%s, kmatch_%s, NULL", 
		     def->def_name, def->def_name, def->def_name);
	    break;

	 case DEF_CONST :
	 case DEF_UNION :
	 case DEF_PROGRAM :
	 default :
	    /*-- these have no handler routines --*/
	    break;
      }
      
      kfprintf(fout, ")) == KNONE)\n");
      kfprintf(fout, "         return FALSE;\n\n");

      /*-- to avoid recursive defines : define needed definitions LAST --*/
      switch (def->def_kind)
      {
	 decl_list *dl;
	 
	 case DEF_STRUCT :
	    for (dl = def->def.st.decls; dl != NULL; dl = dl->next) 
	       if (is_structure(dl->decl.type) && 
		   kstrcmp(dl->decl.type,def->def_name) != 0)
		  kfprintf(fout, "\n      kdefine_%s();", dl->decl.type);
	    break;
	 case DEF_TYPEDEF :
	    if (is_structure(def->def.ty.old_type))
	       kfprintf(fout, "\n      kdefine_%s();", def->def.ty.old_type);
	    break;
	 case DEF_CONST : 
	 case DEF_UNION :
	 case DEF_ENUM  :
	 default :
	    /*-- these require no recursive definitions --*/
	    break;
      }
   }
   kfprintf(fout, "\n   }\n");

   kfprintf(fout, "   return TRUE;\n}\n");
   return;
}

/****    =============================================================
 ****    Routine Code Emitters
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: emit_code - emit body code of read/write/free function
|
|       Purpose: This routine will emit the proper body code for either
|		 a read, write, or free function.  This function, 
|		 preceded by a call to emit_header and followed by
|		 a call to emit_trailer will emit an entire function.
|
|         Input: def  - the definition
|		 emit - what to emit, KEMIT_READ, KEMIT_WRITE, KEMIT_FREE
|
|        Output: prints into the global file specified by fout
|
|       Returns: nothing
|
|    Written By: ported from rpcgen
|          Date: August 19, 1994
| Modifications:
|
-------------------------------------------------------------*/
static void
emit_code(definition *def, int emit)
{
   if (!def) return;
   switch (def->def_kind) 
   {
      case DEF_ENUM:
	 emit_enum(def, emit);
	 break;
      case DEF_UNION:
	 emit_union(def, emit);
	 break;
      case DEF_STRUCT:
	 emit_struct(def, emit);
	 break;
      case DEF_TYPEDEF:
	 emit_typedef(def, emit);
	 break;
      default:
	 break;
   }
   return;
}


/*-----------------------------------------------------------
|  Routine Name: emit_enum - emit function for an enumerated type
|         Input: def - the definition 
|		 emit - the type of function to emit
|        Output: prints into the global file specified by fout
|       Returns: nothing
|    Written By: ported from rpcgen
|          Date: August 19, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static void
emit_enum(
   definition *def,
   int         emit)
{
   switch (emit)
   {
      case KEMIT_READ :
         kfprintf(fout, "   ");
         kfprintf(fout, "if (kread_generic(file");
	 kfprintf(fout, ", (kaddr) objp, (size_t) 1, KINT");
	 kfprintf(fout, ") < 1)\n");
	 kfprintf(fout, "      return FALSE;\n");
         break;
      case KEMIT_WRITE :
         kfprintf(fout, "   ");
	 kfprintf(fout, "if (kwrite_generic(file");
	 kfprintf(fout, ", (kaddr) objp, (size_t) 1, KINT");
	 kfprintf(fout, ") < 1)\n");
	 kfprintf(fout, "      return FALSE;\n");
         break;
      case KEMIT_MATCH :
         kfprintf(fout, "   ");
         kfprintf(fout, "if (*obj1 != *obj2)\n");
	 kfprintf(fout, "      return FALSE;\n");
	 break;
      case KEMIT_FREE :
      default:
         break;
   }
   return;
}


/*-----------------------------------------------------------
|  Routine Name: emit_union - emit function for a union type
|         Input: def - the definition 
|		 emit - the type of function to emit
|        Output: prints into the global file specified by fout
|       Returns: nothing
|    Written By: ported from rpcgen
|          Date: August 19, 1994
|
| *** The functioning of this routine has not been verified ***
|
------------------------------------------------------------*/
static void
emit_union(
   definition *def,
   int         emit)
{
   declaration *dflt;
   case_list *cl;
   declaration *cs;
   char *object;
   char *format = "&objp->%s_u.%s";
   
   print_stat(&def->def.un.enum_decl, emit);
   kfprintf(fout, "   switch (objp->%s) {\n", def->def.un.enum_decl.name);
   for (cl = def->def.un.cases; cl != NULL; cl = cl->next) 
   {
      cs = &cl->case_decl;
      kfprintf(fout, "   case %s:\n", cl->case_name);
      if (!streq(cs->type, "void")) 
      {
	 object = (char *)kmalloc(kstrlen(def->def_name) + kstrlen(format) +
			  kstrlen(cs->name) + 1);
	 ksprintf(object, format, def->def_name, cs->name);
	 print_ifstat(2, cs->prefix, cs->type, cs->rel, cs->array_max,
		      object, cs->name, emit);
	 free(object);
      }
      kfprintf(fout, "      break;\n");
   }
   
   dflt = def->def.un.default_decl;
   if (dflt != NULL) 
   {
      if (!streq(dflt->type, "void")) 
      {
	 kfprintf(fout, "   default:\n");
	 object = (char *)kmalloc(kstrlen(def->def_name) + kstrlen(format) +
			kstrlen(dflt->name) + 1);
	 ksprintf(object, format, def->def_name, dflt->name);
	 print_ifstat(2, dflt->prefix, dflt->type, dflt->rel,
		      dflt->array_max, object, dflt->name, emit);
	 free(object);
	 kfprintf(fout, "      break;\n");
      }
   } 
   else 
   {
      kfprintf(fout, "   default:\n");
      kfprintf(fout, "      return FALSE;\n");
   }
   kfprintf(fout, "   }\n");
   return;
}

/*-----------------------------------------------------------
|  Routine Name: emit_struct - emit function for a structure type
|         Input: def - the definition 
|		 emit - the type of function to emit
|        Output: prints into the global file specified by fout
|       Returns: nothing
|    Written By: ported from rpcgen
|          Date: August 19, 1994
------------------------------------------------------------*/
static void
emit_struct(
   definition *def,
   int         emit)
{
   decl_list *dl;

   for (dl = def->def.st.decls; dl != NULL; dl = dl->next) 
   {
#if 0
      declaration *d;

      /*-- resolve all known typedef to their original types at this time --*/
      d = resolve_typedef(&dl->decl);
      print_stat(d, emit);
      kfree_and_NULL(d);
#else
      print_stat(&dl->decl, emit);
#endif
   }

   return;
}

/*-----------------------------------------------------------
|  Routine Name: emit_typedef - emit function for a typedef type
|         Input: def - the definition 
|		 emit - the type of function to emit
|        Output: prints into the global file specified by fout
|       Returns: nothing
|    Written By: ported from rpcgen
|          Date: August 19, 1994
------------------------------------------------------------*/
static void
emit_typedef(
   definition *def,
   int         emit)
{
   char *prefix = def->def.ty.old_prefix;
   char *type = def->def.ty.old_type;
   char *amax = def->def.ty.array_max;
   relation rel = def->def.ty.rel;

   /*------------------------------------------------------------ 
   |  double-bleah -  typedef's are always considered to be
   |  typedef'd pointers, we need to dereference the pointer 
   |  we've been given.  We had to do the following
   |  in the generated free routine which is calling us	:
   |	     1) we must dereference it here since we were passed 
   |		the address of the field.  This is done in case it
   |            really was just a regular typedef and *wasn't* a 
   |		typedef to a pointer.   
   \------------------------------------------------------------*/
   if (emit == KEMIT_FREE)
   {
      if ((def->def_kind != DEF_TYPEDEF && def->def_kind != DEF_ENUM) ||
	  is_typedef_pointer(def))
      {
	 print_ifstat(1, prefix, type, rel, amax, 
		      "*((kaddr *)objp)", NULL, emit);
      }
      else /*-- try to resolve it and emit any code for actual structure --*/
      {
	 definition *d;

/*	 if (!is_ctype(def->def.ty.old_type))
	 {
*/
	    d = resolve_type(def->def.ty.old_type, def->def.ty.old_type);
	    emit_code(d, KEMIT_FREE);
	    kfree_and_NULL(d);
/*	 }
*/
      }
   }
   else
      print_ifstat(1, prefix, type, rel, amax, "objp", NULL, emit);

   return;
}

/****    =============================================================
 ****    Component Printers
 ****    =============================================================
 ****/

static void
print_ifpointer(char *objname, char *name, char *type, int indent, int emit)
{
   char n[KLENGTH];
   char *o1, *o2, *o1p, *o2p, *o1v = NULL, *o2v = NULL; 

   if (objname == NULL)
      return;
   
   if (name == NULL)
      ksprintf(n, "(kaddr *) %s", objname);
   else
   {
      if (emit != KEMIT_FREE && emit != KEMIT_MATCH)
         ksprintf(n, "(kaddr *) &%s->%s", objname, name);
      else
         ksprintf(n, "(kaddr *) %s->%s", objname, name);
   }

   tabify(fout, indent);

   switch (emit)
   {
      case KEMIT_READ :
	 kfprintf(fout, "if (kread_pointer(file, %s, %s) < 1)\n", 
		  n, type_name(type));
	 tabify(fout, indent); 
	 kfprintf(fout, "   return FALSE;\n");
	 break;
      case KEMIT_WRITE :
	 kfprintf(fout, "if (kwrite_pointer(file, %s, %s) < 1)\n",
		  n, type_name(type));
	 tabify(fout, indent); 
	 kfprintf(fout, "   return FALSE;\n");
	 break;
      case KEMIT_FREE :
	 if (!is_c_type(type))
	    kfprintf(fout, "kstruct_free(%s, %s, TRUE);\n", n, type_name(type));
	 else
	 {
	    if (name != NULL)
	       kfprintf(fout, "kstruct_free(%s->%s, KNONE, TRUE);\n", objname, name);
	    else
	       kfprintf(fout, "kstruct_free(%s, KNONE, TRUE);\n", objname);
	 }
	 break;
      case KEMIT_MATCH :

	 if (objname[0] == '*')  /*-- please fire steve k...  --*/
	 {
	    o1 = kstring_replace(n, objname, "*obj1", NULL);
	    o2 = kstring_replace(n, objname, "*obj2", NULL);
	    o1p = kstring_replace(o1, "(kaddr *) *obj1", "**obj1", NULL);
	    o2p = kstring_replace(o2, "(kaddr *) *obj2", "**obj2", NULL);
	 }
	 else
	 {
	    o1 = kstring_replace(n, objname, "obj1", NULL);
	    o2 = kstring_replace(n, objname, "obj2", NULL);
	    o1v = kstring_replace(n, objname, "*obj1", NULL);
	    o2v = kstring_replace(n, objname, "*obj2", NULL);
	    o1p = kstring_replace(o1, "(kaddr *) obj1", "*obj1", NULL);
	    o2p = kstring_replace(o2, "(kaddr *) obj2", "*obj2", NULL);
	 }

	 kfprintf(fout, "if (%s != NULL && %s != NULL)\n", o1, o2);
   	 tabify(fout, indent);
	 kfprintf(fout, "{\n");

   	 tabify(fout, indent);

	 if (is_structure(type))
	 {
	    if (name == NULL)
	       kfprintf(fout, "   if (!kstruct_compare(%s, %s, %s))\n", 
			o1v, o2v, type_name(type));
	    else
	       kfprintf(fout, "   if (!kstruct_compare(%s, %s, %s))\n", 
			o1, o2, type_name(type));
	 }

	 else if (kstrcmp(type, "kstring") == 0)
	    kfprintf(fout, "   if (kstrcmp(%s, %s) != 0)\n", o1, o2);
	 else
	    kfprintf(fout, "   if (%s != %s)\n", o1p, o2p);
	 
   	 tabify(fout, indent);
	 kfprintf(fout, "      return FALSE;\n");

   	 tabify(fout, indent);
	 kfprintf(fout, "}\n");
   	 tabify(fout, indent);
	 kfprintf(fout, "else if (!(%s == NULL && %s == NULL))\n", o1, o2);
   	 tabify(fout, indent);
	 kfprintf(fout, "   return FALSE;\n\n");
	 kfree_and_NULL(o1);  kfree_and_NULL(o2); 
	 kfree_and_NULL(o1p); kfree_and_NULL(o2p);
	 kfree_and_NULL(o1v); kfree_and_NULL(o2v);
	 break;
   }
   return;
}

static void
print_ifarray(char *objname, char *name, char *type, char *amax, char *prefix,
	      int indent, int emit)
{
   if (objname == NULL || name == NULL)
      return;
   
   tabify(fout, indent);

   switch (emit)
   {
      case KEMIT_READ :
	 kfprintf(fout, "{\n      int len;\n");
	 kfprintf(fout, "      if ((len = kread_array(file");
      	 kfprintf(fout, ", (kaddr *) &%s->%s.%s_val, (size_t) %s", 
	          objname, name, name, amax);
	 kfprintf(fout, ", %s)", type_name(type));
	 kfprintf(fout, ") < 0)\n");
	 tabify(fout, indent); 
	 kfprintf(fout, "         return FALSE;\n");
	 kfprintf(fout, "      %s->%s.%s_len = (unsigned int) len;\n   }\n\n", 
		  objname, name, name);
	 break;
      case KEMIT_WRITE :
	 kfprintf(fout, "if (kwrite_array(file");
	 /*-- rpcgen has this stupid syntax which lets you "limit" variable
	      array size --*/
	 if (atoi(amax) == 0)
      	    kfprintf(fout, ", (kaddr *) %s->%s.%s_val, (size_t) %s->%s.%s_len",
	             objname, name, name, objname, name, name);
 	 else
      	    kfprintf(fout, 
	             ", (kaddr *) %s->%s.%s_val, (size_t) kmin(%s->%s.%s_len,%s)", 
	             objname, name, name, objname, name, name, amax);
	 kfprintf(fout, ", %s", type_name(type));
	 kfprintf(fout, ") < 0)\n");
	 tabify(fout, indent); 
	 kfprintf(fout, "   return FALSE;\n");
	 break;
      case KEMIT_MATCH :
	 kfprintf(fout, "if (obj1->%s.%s_len != obj2->%s.%s_len)\n",
		  name, name, name, name);
         tabify(fout, indent);
	 kfprintf(fout, "   return FALSE;\n");
         tabify(fout, indent);
	 kfprintf(fout, "{\n"); 
         tabify(fout, indent);
	 kfprintf(fout, "   int i;\n");
         tabify(fout, indent);
	 kfprintf(fout, "   for (i = 0; i < obj1->%s.%s_len; i++)\n", 
		  name, name);
         tabify(fout, indent);

	 if (is_structure(type))
	    kfprintf(fout, "      if (!kstruct_compare(&obj1->%s.%s_val[i], &obj2->%s.%s_val[i], %s))\n", name, name, name, name, type_name(type));
	 else if (kstrcmp(type, "kstring") == 0)
	    kfprintf(fout, "      if (kstrcmp(obj1->%s.%s_val[i], obj2->%s.%s_val[i]) != 0)\n", name, name, name, name);
	 else
	    kfprintf(fout, "      if (obj1->%s.%s_val[i] != obj2->%s.%s_val[i])\n", name, name, name, name);

         tabify(fout, indent);
	 kfprintf(fout, "         return FALSE;\n");
         tabify(fout, indent);
	 kfprintf(fout, "}\n\n");
 	 break;
      case KEMIT_FREE :
	 /*-- if array of strings : must free all strings --*/
         if (kstrcmp(type, "kstring") == 0)
	 {
	    kfprintf(fout, "{\n      int i;\n");
	    tabify(fout, indent); 
	    kfprintf(fout, "   for (i = 0; i < %s->%s.%s_len; i++)\n", 
		     objname, name, name);
	    tabify(fout, indent); 
	    kfprintf(fout, "      kfree(%s->%s.%s_val[i]);\n", 
		     objname, name, name);
	    tabify(fout, indent); 
	    kfprintf(fout, "}\n");
	    tabify(fout, indent); 
	 }

	 /*-- if array of typedefs, could be pointers, so free them all --*/
	 else if (is_structure(type) || (!is_c_type(type) && !prefix))	 
	 {
	    kfprintf(fout, "{\n      int i;\n");
	    tabify(fout, indent); 
	    kfprintf(fout, "   for (i = 0; i < %s->%s.%s_len; i++)\n", 
		     objname, name, name);
	    tabify(fout, indent); 
	    kfprintf(fout, 
		     "      kstruct_free((kaddr)&(%s->%s.%s_val[i]), %s, FALSE);\n", 
		     objname, name, name, type_name(type));
	    tabify(fout, indent); 
	    kfprintf(fout, "}\n");
	    tabify(fout, indent); 
	 }

	 /*-- this won't work on strings or arrays of structures --*/
	 kfprintf(fout, "kstruct_free(%s->%s.%s_val, KNONE, TRUE);\n", 
		  objname, name, name);
	 break;
   }
}

static void
print_ifvalue(char *objname, char *name, char *type, char *amax, char *prefix,
	      int indent, int emit)
{
   char n[KLENGTH];
   char *o1, *o2;

   if (objname == NULL)
      return;
   
#if 0
   type = fixtype(type);
#endif

   if (name == NULL)
      ksprintf(n, "(kaddr) %s", objname);
   else
   {
      if (atoi(amax) == 1)
	 ksprintf(n, "(kaddr *) &%s->%s", objname, name);
      else
	 ksprintf(n, "(kaddr) %s->%s", objname, name);

      /*-- bleah - special casts for matching --*/
      if (emit == KEMIT_MATCH)
      {
         if (kstrcmp(type, "kstring") == 0)
	    ksprintf(n, "(char *) %s->%s", objname, name);
	 else if (is_c_type(type))
	    ksprintf(n, "%s->%s", objname, name);
      }

      /*------------------------------------------------------------ 
      |  double-bleah -  typedef's are always considered to be
      |  typedef'd pointers we need to do the following :
      |	     1) pass the address of the field, in case it really
      |         is just a regular typedef and *isn't* a typedef to 
      |		a pointer.   we'll dereference it in the typedef'd
      |		pointer free routine (note that typedefs which 
      |	        are not typedef'd to a pointer have no free, and
      |	 	are safe).
      \------------------------------------------------------------*/
      if (emit == KEMIT_FREE && !is_c_type(type) /* && !prefix */)
	 ksprintf(n, "(kaddr) &(%s->%s)", objname, name);
   }
   

   switch (emit)
   {
      case KEMIT_READ :
	 tabify(fout, indent);
	 kfprintf(fout, "if (kread_generic(file, %s, (size_t) %s, %s) < 1)\n",
		  n, amax, type_name(type));
	 tabify(fout, indent); 
	 kfprintf(fout, "   return FALSE;\n");
	 break;
      case KEMIT_WRITE :
	 tabify(fout, indent);
	 kfprintf(fout, "if (kwrite_generic(file, %s, (size_t) %s, %s) < 1)\n",
		  n, amax, type_name(type));
	 tabify(fout, indent); 
	 kfprintf(fout, "   return FALSE;\n");
	 break;
      case KEMIT_MATCH :
	 o1 = kstring_replace(n, objname, "obj1", NULL);
	 o2 = kstring_replace(n, objname, "obj2", NULL);

	 tabify(fout, indent);
	 if (atoi(amax) == 1)
	 {
	    if (!is_c_type(type))
	       kfprintf(fout, "if (!kstruct_compare(%s, %s, %s))\n", 
			o1, o2, type_name(type));
	    else if (kstrcmp(type, "kstring") == 0)
	      kfprintf(fout, "if (kstrcmp(%s, %s) != 0)\n", o1, o2);
	    else
	    {
	       if (name == NULL)
	       {
		  char *o1p, *o2p;
		  
		  o1p = kstring_replace(o1, "obj1", "*obj1", NULL);
		  o2p = kstring_replace(o2, "obj2", "*obj2", NULL);

		  kfprintf(fout, "if (%s != %s)\n", o1p, o2p);
		  kfree_and_NULL(o1p); kfree_and_NULL(o2p);
	       }
	       else
		  kfprintf(fout, "if (%s != %s)\n", o1, o2);
	    }
	    tabify(fout, indent);
	    kfprintf(fout, "   return FALSE;\n\n");
	 }
	 else
  	 {
	    kfprintf(fout, "{\n"); 
	    tabify(fout, indent);
	    kfprintf(fout, "   int i;\n");
	    tabify(fout, indent);
	    kfprintf(fout, "   for (i = 0; i < %d; i++)\n", atoi(amax));
	    tabify(fout, indent);
	    {
	       char *o1a, *o2a;
	       char *o1p, *o2p;
	       char *o1m, *o2m;
	       o1a = kstring_replace(o1, "obj1", "&obj1", NULL);
	       o2a = kstring_replace(o2, "obj2", "&obj2", NULL);

	       o1p = kstring_replace(o1, "obj1", "(*obj1)", NULL);
	       o2p = kstring_replace(o2, "obj2", "(*obj2)", NULL);

	       o1m = kstring_replace(o1, "obj1", "&((*obj1)[i])", NULL);
	       o2m = kstring_replace(o2, "obj2", "&((*obj2)[i])", NULL);

	       if (is_structure(type))
	       {
		  if (name == NULL)
		     kfprintf(fout,
			      "      if (!kstruct_compare(%s, %s, %s))\n",
			      o1m, o2m, type_name(type));
		  else
		     kfprintf(fout,
			      "      if (!kstruct_compare(%s[i], %s[i], %s))\n",
			      o1a, o2a, type_name(type));
	       }
	       
	       else if (kstrcmp(type, "kstring") == 0)
		  kfprintf(fout, 
			   "	      if (kstrcmp(%s[i], %s[i]) != 0)\n", 
			   o1a, o2a);
	       else
	       {
		  if (name == NULL)
		     kfprintf(fout, 
			      "      if (%s[i] != %s[i])\n", o1p, o2p);
		  else
		     kfprintf(fout, 
			      "      if (%s[i] != %s[i])\n", o1, o2);
	       }

	       kfree_and_NULL(o1a); kfree_and_NULL(o2a);
	       kfree_and_NULL(o1p); kfree_and_NULL(o2p);
	       kfree_and_NULL(o1m); kfree_and_NULL(o2m);
	    }
	    tabify(fout, indent);
	    kfprintf(fout, "         return FALSE;\n");
	    tabify(fout, indent);
	    kfprintf(fout, "}\n\n");
  	 }
	 kfree_and_NULL(o1); kfree_and_NULL(o2); 
	 break;
      case KEMIT_FREE :

	 /*-- could be a typedef to a pointer, so if we are not a c type
	      or a static structure (indicated by prefix), then we'd 
	      better try to free it ...
	      --*/
	 if (atoi(amax) == 1)
	 {
	    if (kstrcmp(type, "kstring") == 0)
	    {
	       kfprintf(fout, "   kstruct_free((kaddr)%s->%s, KNONE, TRUE);\n", 
			objname, name);
	    }
	    else if (is_structure(type) || (!is_c_type(type)/* && !prefix*/))
	    {
	       kfprintf(fout, "   kstruct_free(%s, %s, FALSE);\n", 
			n, type_name(type));
	    }

	    /*-- if it was not a typedef to a pointer, this'll call will
	      have no effect 'cause typedefs don't have free routines
	      unless they are typedef'd to a pointer.  
	      --*/
	 }
	 else
	 {
	    if (kstrcmp(type, "kstring") == 0)
	    {
	       tabify(fout, indent);
	       kfprintf(fout, "{\n"); 
	       tabify(fout, indent);
	       kfprintf(fout, "   int i;\n");
	       tabify(fout, indent);
	       kfprintf(fout, "   for (i = 0; i < %d; i++)\n", atoi(amax));
	       tabify(fout, indent);

	       kfprintf(fout, "      kstruct_free((kaddr)%s->%s[i], KNONE, TRUE);\n", 
			objname, name);
	       tabify(fout, indent);
	       kfprintf(fout, "}\n\n");
	    }
	    else if (is_structure(type) || (!is_c_type(type)/* && !prefix*/))
	    {
	       tabify(fout, indent);
	       kfprintf(fout, "{\n"); 
	       tabify(fout, indent);
	       kfprintf(fout, "   int i;\n");
	       tabify(fout, indent);
	       kfprintf(fout, "   for (i = 0; i < %d; i++)\n", atoi(amax));

	       kfprintf(fout, "      kstruct_free((kaddr)&(%s->%s[i]), %s, FALSE);\n",
			objname, name, type_name(type));
	       tabify(fout, indent);
	       kfprintf(fout, "}\n\n");
	    }
	 }
	 

	 break;
   }
   return;
}

static void
print_ifstat(
   int       indent, 
   char     *prefix, 
   char     *type, 
   relation  rel, 
   char     *amax, 
   char     *objname, 
   char     *name,
   int       emit)
{
   switch (rel) 
   {
      case REL_POINTER:
	 print_ifpointer(objname, name, type, indent, emit);
	 break;

      case REL_VECTOR:
	 print_ifvalue(objname, name, type, amax, prefix, indent, emit);
	 break;

      case REL_ARRAY:
	 print_ifarray(objname, name, type, amax, prefix, indent, emit);
	 break;

      case REL_ALIAS:
	 if (amax == NULL)
	   print_ifvalue(objname, name, type, "1", prefix, indent, emit);
	 else
	   print_ifvalue(objname, name, type, amax, prefix, indent, emit);
	 break;
   }
   return;
}

static void
print_stat(
   declaration *dec,
   int          emit)
{
   char *prefix = dec->prefix;
   char *type = dec->type;
   char *amax = dec->array_max;
   relation rel = dec->rel;

   print_ifstat(1, prefix, type, rel, amax, "objp", dec->name, emit);

   return;
}
