 /*
  * Khoros: $Id: kgentb.c,v 1.14.2.3 1997/05/31 20:53:59 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kgentb.c,v 1.14.2.3 1997/05/31 20:53:59 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for kgentb
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "kgentb.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Creates a toolbox from scratch.
|
|       Purpose: main program for kgentb
|
|         Input:
|		char *clui_info->tb_string; {toolbox to create}
|		int   clui_info->tb_flag; {TRUE if -tb specified}
|
|		char *clui_info->path_string; {toolbox path}
|		int   clui_info->path_flag; {TRUE if -path specified}
|
|		char *clui_info->title_string; {toolbox title}
|		int   clui_info->title_flag; {TRUE if -title specified}
|
|		char *clui_info->info_string; {toolbox description file}
|		int   clui_info->info_flag; {TRUE if -info specified}
|
|		char *clui_info->author_string; {toolbox author}
|		int   clui_info->author_flag; {TRUE if -author specified}
|
|		char *clui_info->email_string; {toolbox author email}
|		int   clui_info->email_flag; {TRUE if -email specified}
|
|		int clui_info->add_flag; {TRUE if -add specified}
|
|        Output:
|       Returns:
|
|    Written By: Steven Jorgensen
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	kobject		toolbox;
/* -main_variable_list_end */

	khoros_init(argc, argv, "BOOTSTRAP", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$BOOTSTRAP/objects/kroutine/kgentb");
	kexit_handler(kgentb_free_args, NULL);

/* -main_get_args_call */
	kclui_init("BOOTSTRAP", "kgentb", KGEN_KROUTINE,  &clui_uis_spec,
		   kgentb_usage_additions, kgentb_get_args,
 		   kgentb_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	if (clui_info->tb_string == NULL)
	{
		errno = KNULL_PARAMETER;
		kerror(NULL, "main()", "Toolbox field is NULL");
		kexit(KEXIT_FAILURE);
	}
	if ((toolbox = kcms_open_toolbox(clui_info->tb_string)) != NULL)
	{
		kerror(NULL, "main()", "Toolbox '%s' already exists",
		       clui_info->tb_string);
		kcms_close(toolbox);
		kexit(KEXIT_FAILURE);
	}
/* -main_before_lib_call_end */

/* -main_library_call */
   if (clui_info->add_flag)
      toolbox = kcms_add_toolbox(clui_info->tb_string, clui_info->path_string);
   else
      toolbox = kcms_create_toolbox(clui_info->tb_string,
                                    clui_info->path_string);

	if (toolbox == NULL)
	{
		kerror(NULL,"main()","Error %s toolbox '%s'",
			(clui_info->add_flag == TRUE) ? "adding" : "creating",
			clui_info->tb_string);
		kexit(KEXIT_FAILURE);
	}

	if (!clui_info->add_flag)
	{
	   if (clui_info->author_string)
	      kcms_set_attribute(toolbox, KCMS_TB_AUTHOR,
			clui_info->author_string);

	   if (clui_info->email_string)
	      kcms_set_attribute(toolbox, KCMS_TB_AUTHOR_EMAIL,
			clui_info->email_string);

	   if (clui_info->title_string)
	      kcms_set_attribute(toolbox, KCMS_TB_TITLE,
			clui_info->title_string);

	   if (clui_info->info_string)
	      kcms_set_attribute(toolbox, KCMS_TB_INFO_FILE,
			clui_info->info_string);
	}
/* -main_library_call_end */

/* -main_after_lib_call */
	kcms_close(toolbox);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: kgentb_usage_additions
| 
|       Purpose: Prints usage additions in kgentb_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kgentb
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void kgentb_usage_additions(void)
{
	kfprintf(kstderr, "\tCreates a toolbox from scratch.\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: kgentb_free_args
| 
|       Purpose: Frees CLUI struct allocated in kgentb_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kgentb
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
kgentb_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->tb_string);
		kfree_and_NULL(clui_info->path_string);
		kfree_and_NULL(clui_info->title_string);
		kfree_and_NULL(clui_info->info_string);
		kfree_and_NULL(clui_info->author_string);
		kfree_and_NULL(clui_info->email_string);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
