/*
 * Khoros: $Id: utilities.c,v 1.6 1997/01/20 04:01:10 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.6 1997/01/20 04:01:10 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            Phantomd utilities
   >>>>
   >>>>  Static:
   >>>>  Private:
   >>>>             process_command()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "phantomd.h"


#define ALLOCSIZE 512000

/*-----------------------------------------------------------
|
|  Routine Name: process_command - phantomd process command routine
|
|       Purpose: This routine processes all the remote commands.
|         Input: command - the command to be processed
|
|    Written By: Mark Young
|          Date: June 10, 1996
|
------------------------------------------------------------*/
int process_command(
   char *command)
{
	char   temp[KLENGTH], mesg[KLENGTH];

	kstring_cleanup(command, command);
	if (kstring_subcmp(command, "read:") == 0 ||
	    kstring_subcmp(command, "write:") == 0 ||
	    kstring_subcmp(command, "exec:") == 0)
	{
	   size_t cnt;
	   int    status;
	   char   data[ALLOCSIZE];
	   kfile  *file, *in, *out;

	   if (ksscanf(command, "%*[^:]:%*[ ]%[^\n]", temp) != 1)
	      return(FALSE);

	   if (kstring_subcmp(command, "read:") == 0)
	      file = in = kfopen(temp, "r"), out = ofile;
	   else if (kstring_subcmp(command, "write:") == 0)
	      file = out = kfopen(temp, "w"), in = ifile;
	   else
	      file = in = kpopen(temp, "r"), out = ofile;

	   if (kstring_subcmp(command, "write:") == 0 && file != NULL)
	   {
	      while (kfgets(data, KLENGTH, ifile) != NULL &&
		     ksscanf(data, "%d", &cnt) == 1 && cnt > 0)
	      {
	         cnt = kfread(data, 1, ALLOCSIZE, in);
	         kfprintf(out, "%d\n", cnt);
	         kfwrite(data, 1, cnt, out);
	      }
	   }
	   else if (file != NULL)
	   {
	      while ((cnt = kfread(data, 1, ALLOCSIZE, in)) > 0)
	      {
	         kfprintf(out, "%d\n", cnt);
	         kfwrite(data, 1, cnt, out);
	      }
	   }

	   if (file == NULL)
	      status = -1;
	   else
	      status = 0;

	   if (kstring_subcmp(command, "exec:") == 0)
	   {
	      if (file == NULL)
	      {
	         ksprintf(mesg, "Unable to execute command:\n '%s'\n", temp);
		 kfprintf(out, "%d\n", kstrlen(mesg));
		 kfwrite(mesg, 1, kstrlen(mesg), out);
	      }
	      else
		 status = kpclose(file);
	   }
	   else if (file != NULL)
	      kfclose(file);

	   kfputs("-1\n", ofile);
	   kfprintf(ofile, "%d\n", status);
	}
	else if (kstring_subcmp(command, "access:") == 0)
	{
	   int mode;
	   char flag[KLENGTH];

	   if (ksscanf(command, "%*[^']'%[^']' %*[^']'%[^']'", flag, temp) != 2)
	      return(FALSE);

	   if (kstrcmp(flag, "r") == 0)
	      mode = R_OK;
	   else if (kstrcmp(flag, "w") == 0)
	      mode = W_OK;
	   else if (kstrcmp(flag, "x") == 0)
	      mode = X_OK;
	   else if (kstrcmp(flag, "f") == 0)
	      mode = F_OK;
	   else
	      return(FALSE);

	   if (kaccess(temp, mode) == 0)
	      kfputs("yes\n", ofile);
	   else
	      kfputs("no\n", ofile);
	}
	else if (kstring_subcmp(command, "unlink:") == 0)
	{
	   if (ksscanf(command, "%*[^']'%[^']'", temp) != 1)
	      return(FALSE);

	   if (kunlink(temp) == 0)
	      kfputs("yes\n", ofile);
	   else
	      kfputs("no\n", ofile);
	}
	else if (kstring_subcmp(command, "putenv:") == 0)
	{
	   if (ksscanf(command, "%*[^']'%[^']'", temp) != 1)
	      return(FALSE);

	   if (kputenv(temp) == 0)
	      kfputs("yes\n", ofile);
	   else
	      kfputs("no\n", ofile);
	}
	else
	   return(FALSE);

	return(TRUE);
}
