/*
 * Khoros: $Id: cmobj_destroy.c,v 1.13.2.4 1997/04/21 21:57:53 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: cmobj_destroy.c,v 1.13.2.4 1997/04/21 21:57:53 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Routines for destroying a software object
   >>>>		
   >>>>  Private:
   >>>>		kcms_cmobj_destroy()
   >>>>
   >>>>   Static:
   >>>>
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include "cmobjP.h"
#include "databaseP.h"

static void _kcms_cmobj_delete_binaries(kobject);
static void _kcms_cmobj_delete_libroutine(kobject);
static void _kcms_cmobj_update_whatis(kobject);


/*-----------------------------------------------------------
| Routine Name:	kcms_cmobj_destroy - remove cmobj from its containing toolbox
|
| Purpose:	This routine removes a software object from a toolbox.
|
| Input:	cmobj - the cmobj to remove
| Returns:	TRUE (1) on success, FALSE (0) otherwise
| Written By:	Steven Jorgensen and Neil Bowers
| Date:		7-feb-93
------------------------------------------------------------*/
int
kcms_cmobj_destroy(
   kobject cmobj)
{
   kstring        routine = "Destroy Object";
   kobject        toolbox;
   kobject        fobj;
   kdbm          *dbm;
   kstring       *dlist = NULL;
   kstring        tbpath;
   kstring        tbname;
   kstring        opath;
   char           temp[KLENGTH];
   char           path[KLENGTH];
   char           cwd[KLENGTH];
   char           dirpath[KLENGTH];
   unsigned long  otype;
   size_t         count;
   int            incantata;
   klist         *doc_list;
   kstring        oname = NULL;
   kstring        bname = NULL;
   kstring        typestring;
   kdatum         key;
   int            force;
   size_t         i, dcount;


   if (kgetcwd(cwd, (size_t)KLENGTH) == NULL)
   {
      kerror(KCMS, routine, "Could not determine the current directory.\n");
      return FALSE;
   }

   if (!kcms_legal_object(cmobj, routine))
      return FALSE;

   if (!kcms_get_attributes(cmobj,
			    KCMS_PATH,            &opath,
			    KCMS_PARENT,          &toolbox,
			    KCMS_NAME,            &oname,
			    KCMS_CMOBJ_TYPE,      &otype,
			    KCMS_CMOBJ_BNAME,     &bname,
			    KCMS_CMOBJ_INCANTATA, &incantata,
			    KCMS_CMOBJ_ALL_DOC,   &doc_list,
			    KCMS_END)
       || !kcms_get_attributes(toolbox,
			       KCMS_NAME, &tbname,
			       KCMS_PATH, &tbpath,
			       KCMS_END))
      return FALSE;
   typestring = kcms_attr_int2string(KCMS_CMOBJ_TYPE, otype);

   if (kfullpath(opath, NULL, path) == NULL)
   {
      kerror(KCMS, routine, "Unable to resolve the full path of the "
	     "software object being deleted:\n\n"
	     "\tPath: %s\n\n"
	     "Deletion will not proceed.", opath);
      return FALSE;
   }

   if (!kstrncmp(path, cwd, kstrlen(path)))
   {
      kerror(KCMS, routine,
	     "Your current directory is within the object's "
	     "directory structure,\n"
	     "so the object cannot be removed.\n");
      return FALSE;
   }

   if (kcms_query_bit(cmobj, KCMS_CMOBJ_FLAGS, KCMS_BIT_CMOBJ_FORCE, &force)
       && !force
       && !kprompt(KSTANDARD, NULL, NULL, 0,
		"\nOperation: delete software object from toolbox\n\n"
		"\tToolbox: %s\n"
		"\tObject:  %s\n\n"
		"Do you want to continue with removal of object?",
		tbname, cmobj->oname))
   {
      kinfo(KSYSLIB, "%s: %s.\n", routine,
	    "destroy aborted at request of user");
      return TRUE;
   }

   /*-------------------------------------------------------------------
   | Update the whatis file in $TOOLBOX/repos/config/whatis
   | This is used when you run "kman -k"
   -------------------------------------------------------------------*/
   _kcms_cmobj_update_whatis(cmobj);

   /*-- we'll open the manpage dbm, ready to update if needs be ------*/
   if (doc_list != NULL)
   {
      ksprintf(temp, "$%s/repos/db/manpage", tbname);
      dbm = kdbm_open(temp, (O_WRONLY | O_CREAT), 0666);
      for (; doc_list != NULL; doc_list = klist_next(doc_list))
      {
	 fobj = (kobject)klist_clientdata(doc_list);
         kcms_update_man_db(dbm, fobj, FALSE);
      }
      kdbm_close(dbm);
   }

   /*-- remove it from the objects cache? --*/
   ksprintf(path, "%s/repos/db/cache/objects", tbpath);
   if ((dbm = kdbm_open(path, (O_RDWR | O_CREAT), 0666)) == NULL)
      kwarn(KCMS, routine, "Could not open objects cache for toolbox "
	    "`%s'.\n"
	    "The objects cache should be at %s\n",
	    tbname, path);
   else
   {
      key.dptr = cmobj->oname;
      key.dsize = kstrlen(cmobj->oname) + 1;
      (void)kdbm_delete(dbm, key);
      kdbm_close(dbm);
   }


   /*-------------------------------------------------------------------
   | If the object is a KROUTINE with associated library, see if the
   | user wants to blow away the file from the associated library.
   -------------------------------------------------------------------*/
   if (otype == KCMS_KROUTINE && cmobj->library != NULL)
      _kcms_cmobj_delete_libroutine(cmobj);


   /*--------------------------------------------------------------------
   |	Delete any binaries associated with the software object
   --------------------------------------------------------------------*/
   _kcms_cmobj_delete_binaries(cmobj);


   /*UPDATE------------------------------------------------------
   |    at this point we should check:
   |    a) are there any references out of the object?
   |    b) any references from other objects into us?
   +----------------------------------------------------------- */

   /*-- if it is a library, delete any public includes --*/
   if (otype == KCMS_LIBRARY)
   {
      ksprintf(path, "$%s/include/%s", tbname, oname);

      if (kaccess(path, R_OK) == 0 && !kremove_dir(path))
	 kerror(KCMS, routine,
		"Failed to remove public include directory.");
   }

   /*-- check whether object-type directory can be removed as well? --*/
   (void)ksprintf(dirpath, "$%s/objects/%s", tbname, typestring);
   if (kfullpath(dirpath, NULL, path) == NULL)
      return FALSE;

   dlist = karray_dirlist(dirpath, NULL, NULL, KDIR, FALSE, &count);
   for (i = 0, dcount = 0; i < count; i++)
   {
      if (kstrcasecmp(dlist[i], oname) == 0)
         dcount++;
      else if (kstrcmp(dlist[i], "CVS") == 0)
	 dcount++;
      else if (kstrcmp(dlist[i], "..") == 0)
	 dcount++;
   }
   if (dcount != count)
      kstrcpy(dirpath, cmobj->opath);
   karray_free(dlist, KSTRING, count, NULL);

   if (!kremove_dir(dirpath))
      kerror(KCMS, routine, "\n"
	     "The object's directory structure could not "
	     "be completely deleted.\n"
	     "The most likely reason was that you were in "
	     "the directory you were trying to delete.\n");

   /*-- remove the program object from the toolbox's manpage dbm --*/
   ksprintf(temp, "$%s/repos/db/manpage", tbname);
   if ((dbm = kdbm_open(temp, O_WRONLY, 0666)) != NULL)
   {
      key.dptr = oname;
      key.dsize = kstrlen(key.dptr) + 1;
      (void)kdbm_delete(dbm, key);
      kdbm_close(dbm);
   }

   /*-- remake imake and make files --*/
   if (kdirname(dirpath, path) != NULL)
      kcms_dir_imakeandmake(path, tbname, 1);

   /*--------------------------------------------------------------------
   | If the object was a LIBRARY, then update the toolbox.def file,
   | toolbox.h file, and rebuild all Makefiles in the toolbox.
   --------------------------------------------------------------------*/
   if (otype == KCMS_LIBRARY)
      kcms_cmobj_destroy_checklib(tbname, tbpath, oname, bname);

   cmobj->parent->del_sub_obj(cmobj->parent, cmobj);
   kcms_cmobj_free(cmobj);

   return TRUE;
}

/*-----------------------------------------------------------
| Routine Name:	kcms_cmobj_destroy_checklib - check for lib in toolbox.def
|
| Purpose:	This routine is called after deleting a library object from
|		a toolbox to update the toolbox.def file, toolbox.h file,
|               and rebuild all Makefiles in the toolbox.
|
| Input:	tbname - the name of the toolbox
|		tbpath - full path to top of the toolbox
|		oname  - the name of the library object which was deleted.
|		bname  - the binary name of the library object which was deleted.
|
| Written By:	Neil Bowers
| Date:		10-may-96
| Modifications: kevins automated this process. It used to just
|               tell you to do it.
------------------------------------------------------------*/
void
kcms_cmobj_destroy_checklib(
   kstring        tbname,
   kstring        tbpath,
   kstring        oname,
   kstring        bname)
{
   kstring   routine        = "kcms_cmobj_destroy()";
   char      filename       [KLENGTH];
   char      search_key     [KLENGTH];
   char      command        [KLENGTH];
   char      tblower        [KLENGTH];
   kstring   result_string  = NULL;
   kstring   filedata       = NULL;
   int       search_status;
   size_t    nBytes;
   int       updated = FALSE;

   /* Remove the AddLibrary(bname,...) line from toolbox.def */

   ksprintf(filename, "%s/repos/config/imake_config/toolbox.def", tbpath);
   ksprintf(search_key, "^\\s*AddLibrary\\s*\\(\\s*%s,.*\\n", bname);

   filedata = (kstring)kreadfile(filename, &nBytes);
   if (filedata == NULL)
   {
      kerror(KCMS, routine,
             "Failed to read the toolbox.def file.\n"
             "This file may be out of date now.\n");
      return;
   }
   else
   {	
      (void) kparse_string_search(filedata, search_key, KLITERAL, 
                                  NULL, &search_status);
      if (search_status == KPARSE_OK)
      {
         result_string = kregex_replace(filedata, search_key, "", NULL);
         if (result_string != NULL)
         {
            updated = TRUE;
            kwritefile(filename, result_string, (size_t)0);
            kfree_and_NULL(result_string);
            kannounce(KCMS, routine, 
			"Updated toolbox Imake configuration file.\n");
         }
      }
      kfree_and_NULL(filedata);
   }

   /* Remove the #include <oname/oname.h> line from tbname.h */

   kstring_lower(tbname, tblower);
   ksprintf(filename, "%s/include/%s.h", tbpath, tblower);
   ksprintf(search_key, "\\n#\\s*include\\s*<%s/%s\\.h>", oname, oname);
   filedata = (kstring)kreadfile(filename, &nBytes);
   if (filedata == NULL)
   {
      kerror(KCMS, routine,
             "Failed to read the toolbox include file.\n"
             "This file may be out of date now.\n");
   }
   else
   {	
      (void) kparse_string_search(filedata, search_key, KLITERAL, 
                                  NULL, &search_status);
      if (search_status == KPARSE_OK)
      {
         result_string = kregex_replace(filedata, search_key, "", NULL);
         if (result_string != NULL)
         {
            updated = TRUE;
            kwritefile(filename, result_string, (size_t)0);
            kfree_and_NULL(result_string);
            kannounce(KCMS, routine, "Updated toolbox include file.");
         }
      }
      kfree_and_NULL(filedata);
   }

   /* Regenerate Makefiles if necessary */

   if (updated)
   {
      kannounce(KCMS, routine, "Regenerating Makefiles for toolbox");
      ksprintf(command, "cd %s/objects;kgenimake;make Makefiles", tbpath);
      ksystem(command);
   }
}


/*-----------------------------------------------------------
| Routine Name:	_kcms_cmobj_delete_binaries - delete binary or archive
|					      files of software object
|
| Purpose:	This routine is called to delete any binaries or library
|		archive files associated with the software object:
|			(a) for libraries these will be archive files.
|			(b) for pane objects, binary is object name (script)
|			(c) for other objects, the executable.
|
| Input:	object - kcms object reference for the software object
|			 being deleted.
|
| Written By:	Neil Bowers
| Date:		22-may-96
------------------------------------------------------------*/
static void
_kcms_cmobj_delete_binaries(
   kobject  object)
{
   kstring routine      = "_kcms_cmobj_delete_binaries()";
   kstring path;
   char    command[KLENGTH];

   if (!kcms_get_attribute(object, KCMS_CMOBJ_BUILDPATH, &path))
   {
      kerror(KCMS, routine,
             "Operation aborted -- failed to get object attributes.\n");
      return;
   }

   ksprintf(command, "cd %s; make uninstall", path);
   ksystem(command);
}

/*-----------------------------------------------------------
| Routine Name:	_kcms_cmobj_delete_libroutine - delete associated
|					        library routine
|
| Purpose:	This routine is called to delete the library function
|		associated with a kroutine, if the user so desires.
|
| Input:	object - kcms object reference for the software object
|			 being deleted.
|
| Written By:	Neil Bowers
| Date:		22-may-96
------------------------------------------------------------*/
static void
_kcms_cmobj_delete_libroutine(
   kobject  object)
{
   kstring   routine     = "_kcms_cmobj_delete_libroutine()";
   kobject   lib_object;	/*-- associated library object --*/
   kobject   file_object;	/*-- file with associated lib function --*/
   klist    *file_list;		/*-- list of file objects in library --*/
   klist    *target;
   kstring   lib_path;
   ktoken    file_token;
   int       force;		/*-- force flag bit on software object --*/
   char      lfile[KLENGTH];
   kstring   tbname;


   if (!kcms_query_bit(object, KCMS_CMOBJ_FLAGS, KCMS_BIT_CMOBJ_FORCE, &force)
       ||!kcms_get_indirect_attribute(object, KCMS_PARENT, KCMS_NAME, &tbname))
   {
      return;
   }

   /*-------------------------------------------------------------------
   | Check if the user wants us to blow away the associated library
   | routine. Return if they Just Say No.
   -------------------------------------------------------------------*/
   if (!force &&
       !kprompt(KSTANDARD, NULL, NULL, 0,
		"\nOperation: delete software object from toolbox\n"
		"\n"
		"\tToolbox: %s\n"
		"\tObject:  %s\n"
		"\n"
		"Do you want to remove l%s.c from library (%s)?",
		tbname, object->oname, object->oname, object->library))
      return;

   /*-------------------------------------------------------------------
   | open the associated library.
   | If the library doesn't exist in the toolbox, then quietly return,
   | because we're deleting the object anyway, so who cares?!
   | If we get NULL for some other reason, then generate an error
   | message but do nothing more.
   -------------------------------------------------------------------*/
   lib_object = kcms_open_cmobj(object->parent, object->library);
   if (lib_object == NULL && errno == KCMS_ENOTFOUND)
      return;

   if (lib_object == NULL)
   {
      errno = KCMS_EINTERNAL;
      kerror(KCMS, routine,
	     "Failed to open associated library object `%s'.\n"
	     "Nothing was deleted from the library.\n",
	     object->library);
   }

   if (!kcms_get_attribute(lib_object, KCMS_PATH, &lib_path))
   {
      kerror(KCMS, routine,
	     "Could not determine the path to the object directory "
	     "for the associated library `%s'.\n",
	     object->library);
      return;
   }

   /*-------------------------------------------------------------------
   | Get the list of all C files in the library object.
   | Look for l<object-name>.c in the list of files. If we find it,
   | then call kcms_destroy() on the file object to remove it from
   | the library.
   -------------------------------------------------------------------*/
   if (!kcms_get_attribute(lib_object,
          (object->lang_type == KCMS_LANG_CPLUSPLUS ?
              KCMS_CMOBJ_ALL_CPLUSPLUS : KCMS_CMOBJ_ALL_C),
          &file_list))
      return;

   ksprintf(lfile, "%s/src/l%s.%c", lib_path, object->oname,
            (object->lang_type == KCMS_LANG_CPLUSPLUS ? 'C' : 'c'));
   file_token = kstring_to_token(lfile);
   target = klist_locate(file_list, (kaddr)file_token);
   if (target != NULL)
   {
      file_object = (kobject)klist_clientdata(target);
      if (file_object != NULL)
	 kcms_destroy(file_object);
   }

   /*-------------------------------------------------------------------
   | Get the list of all library man pages (man(3)) in the library object.
   | Look for l<object-name>.3 in the list of files. If we find it,
   | then call kcms_destroy() on the file object to remove it from
   | the library.
   -------------------------------------------------------------------*/
   if (!kcms_get_attribute(lib_object, KCMS_CMOBJ_ALL_LIBMAN, &file_list))
      return;

   ksprintf(lfile, "%s/man/l%s.3", lib_path, object->oname);
   file_token = kstring_to_token(lfile);
   target = klist_locate(file_list, (kaddr)file_token);
   if (target != NULL)
   {
      file_object = (kobject)klist_clientdata(target);
      if (file_object != NULL)
	 kcms_destroy(file_object);
   }

   /*-------------------------------------------------------------------
   | Get the list of all library html pages in the library object.
   | Look for l<object-name>.htm in the list of files. If we find it,
   | then call kcms_destroy() on the file object to remove it from
   | the library.
   -------------------------------------------------------------------*/
   if (!kcms_get_attribute(lib_object, KCMS_CMOBJ_ALL_HTML, &file_list))
      return;

   ksprintf(lfile, "%s/html/l%s.htm", lib_path, object->oname);
   file_token = kstring_to_token(lfile);
   target = klist_locate(file_list, (kaddr)file_token);
   if (target != NULL)
   {
      file_object = (kobject)klist_clientdata(target);
      if (file_object != NULL)
	 kcms_destroy(file_object);
   }

   kcms_set_attribute(lib_object, KCMS_CMOBJ_UPDATE_DB,
		      (KCMS_UPDATE_SYNC | KCMS_UPDATE_REGEN));

   kcms_close(lib_object);
}


/*-----------------------------------------------------------
| Routine Name:	_kcms_cmobj_update_whatis - update whatis database after
|					    deleting an object
|
| Purpose:	This routine is called to update the whatis file when
|		deleting a software object.
|
| Input:	object - kcms object reference for the software object
|			 being deleted.
|
| Written By:	Neil Bowers
| Date:		22-may-96
------------------------------------------------------------*/
static void
_kcms_cmobj_update_whatis(
   kobject  object)
{
   kstring        routine        = "_kcms_cmobj_update_whatis()";
   kobject        toolbox;
   unsigned long  object_type;
   kstring        toolbox_name;
   kstring        toolbox_path;
   kstring        whatis_path;
   kstring        whatis_data;
   size_t         nBytes;
   kstring        result_string;
   char           pattern[KLENGTH];
   kstring        object_name;
   klist         *object_list;


   if (!kcms_get_attributes(object,
			    KCMS_PARENT,      &toolbox,
			    KCMS_CMOBJ_TYPE,  &object_type,
			    KCMS_NAME,        &object_name,
			    KCMS_END)
       || !kcms_get_attributes(toolbox,
			       KCMS_NAME,                &toolbox_name,
			       KCMS_PATH,                &toolbox_path,
			       KCMS_TB_SOFTWARE_OBJECTS, &object_list,
			       KCMS_END))
   {
      return;
   }

   /*-- whatis file should be at $TOOLBOX/repos/config/whatis --------*/
   whatis_path = kstring_cat(toolbox_path, "/repos/config/whatis", NULL);

   /*-- if it's not there, don't bother with it ----------------------*/
   if (kaccess(whatis_path, (R_OK | W_OK)) == -1)
   {
      kfree(whatis_path);
      return;
   }

   /*-------------------------------------------------------------------
   | If this is the only object in the toolbox, then we just unlink the
   | whatis file.
   -------------------------------------------------------------------*/
   if (klist_size(object_list) <= 1)
   {
      if (kunlink(whatis_path) == -1)
	 kerror(KCMS, routine,
		"I may have failed to delete the whatis file:\n"
		"\n"
		"\t%s\n"
		"\n"
		"You may have to delete this by hand. Sorry about that.\n",
		whatis_path);
      return;
   }

   /*-- suck the contents of the whatis file into a large string -----*/
   whatis_data = (kstring)kreadfile(whatis_path, &nBytes);
   if (whatis_data == NULL)
   {
      kfree(whatis_path);
      kerror(KCMS, routine,
	     "Failed to read the whatis file.\n"
	     "This file may be out of date now.\n");
      return;
   }

   switch (object_type)
   {
      /*----------------------------------------------------------------
      | For program objects (non-library), there will be a single
      | simple entry in the whatis file
      ----------------------------------------------------------------*/
      case KCMS_KROUTINE:
      case KCMS_XVROUTINE:
      case KCMS_SCRIPT:
      case KCMS_PANE:
         /* actually, l<oname> will only show up for kroutines, but wth */
         if (object->library != NULL)
	    ksprintf(pattern, "^((l%s)|([\\w\\d_]+\\s+\\(%s\\))).*$", 
                     object_name, object_name);
         else
	    ksprintf(pattern, "^[\\w\\d_]+\\s+\\(%s\\).*$", 
                     object_name);

	 errno = 0;
	 result_string = kregex_replace(whatis_data, pattern, "", NULL);
	 if (result_string != NULL)
	 {
	    kwritefile(whatis_path, result_string, (size_t)0);
	 }
	 else if (errno == 0)
	    kunlink(whatis_path);
	 else
	    kerror(KCMS, routine, "Unable to update whatis config file.");
	 break;

      /*----------------------------------------------------------------
      | For library objects, there will be an entry in the whatis file
      | for each public function. The lines all have the format:
      |		^function-name() (library-name)
      ----------------------------------------------------------------*/
      case KCMS_LIBRARY:
	 ksprintf(pattern, "^[:\\w\\d_]+\\(\\)\\s+\\(%s\\).*$", object_name);
	 
	 errno = 0;
	 result_string = kregex_replace(whatis_data, pattern, "", NULL);
	 if (result_string != NULL)
	 {
	    kwritefile(whatis_path, result_string, (size_t)0);
	 } else if (errno == 0)
	    kunlink(whatis_path);
	 else
	    kerror(KCMS, routine, "Unable to update whatis config file.");
	 break;

      default:
	 kerror(KCMS, routine, "Unexpected software object type (%ld)\n",
		object_type);
	 break;
   }
}
