/*
 * Khoros: $Id: cmobj_dup.c,v 1.9.2.2 1997/05/02 22:11:17 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: cmobj_dup.c,v 1.9.2.2 1997/05/02 22:11:17 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Software Object Duplication
   >>>>		
   >>>>  Private:
   >>>>		kcms_cmobj_duplicate()
   >>>>
   >>>>   Static:
   >>>>		_kcms_duplicate_library()
   >>>>
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include "cmobjP.h"
#include "databaseP.h"

static int  _kcms_duplicate_library		(kobject, kobject, int);
static void _kcms_cmobj_copy_ulong_attributes	(kobject, kobject);
static void _kcms_cmobj_copy_int_attributes	(kobject, kobject);
static void _kcms_cmobj_copy_string_attributes	(kobject, kobject);
static void _kcms_cmobj_copy_keywords		(kobject, kobject);

static char *library_options[2] =
{
   "Create library, copy just the l*.c file over",
   "Copy the entire library as well"
};

/*------------------------------------------------------------
| Routine Name:	kcms_cmobj_duplicate - duplicate an existing software object.
|
| Purpose:	This routine generates a copy of an existing software object
|		in another toolbox.  This is the back-end for kcms_duplicate(),
|		when called with a software object.
|
| Input:	cmobj   - the software object being copied.
|		toolbox - the open toolbox object to copy the object into.
|
| Returns:	a pointer to the new (duplicate) software object,
|		or NULL on error
|
| Written By:	Steven Jorgensen and Neil Bowers
| Date:		Jan 24, 1993 00:37
------------------------------------------------------------*/
kobject
kcms_cmobj_duplicate(
   kobject  cmobj,
   kobject  toolbox)
{
   kstring          routine = "Duplicate Object";
   long             fgen;
   int              force_flag = FALSE;
   kobject          new_cmobj = NULL;
   kobject          fileobj1;
   kobject          fileobj2;
   char             temp[KLENGTH];
   kstring          fpath;
   klist           *flist;
   klist           *destn_files         = NULL;
   klist           *df_attr;
   ktoken           token;
   ktoken           dest_token;
   ktoken           dbToken;
   ktoken           makeToken;
   kstring          otbname;
   kstring          newtbname;
   kdbm            *dbm;
   int              genexec;
   long             ftype;
   kbool            updating   = FALSE;	/* updating object in destination? */
   char             destpath[KLENGTH];
   kstring          relpath;
   kobject          fobj;
   kobject          libobj;
   int              libchoice;
   char             retstring[KLENGTH];


   if (!kcms_legal_object(cmobj, routine)
       || !kcms_legal_object(toolbox, routine))
      return NULL;

   if (!kcms_get_attribute(cmobj, KCMS_CMOBJ_ALL_FILES, &flist) ||
       !kcms_get_indirect_attribute(cmobj, KCMS_PARENT, KCMS_NAME, &otbname) ||
       !kcms_get_attribute(toolbox, KCMS_NAME, &newtbname))
      return NULL;


   if (!kstrcasecmp(newtbname, otbname))
   {
      kerror(KCMS, routine, "source and destination toolboxes are the same!");
      return NULL;
   }

   /*-- lkroutine check: how to deal with the library? ---------------*/
   if (cmobj->prog_type == KCMS_KROUTINE && cmobj->library != NULL)
   {
      if ((libobj = kcms_open_cmobj(toolbox, cmobj->library)) == NULL)
      {
         if (!(kcms_query_bit(cmobj, KCMS_CMOBJ_FLAGS, KCMS_BIT_CMOBJ_FORCE,
	     &force_flag) && force_flag))
	    kchoose(KSTANDARD, library_options, 2, -1, retstring, &libchoice,
		    "The object `%s' is marked as having an associated "
		    "library routine in library `%s'.\n"
		    "This library does not exist in toolbox `%s'.\n",
		    cmobj->oname, cmobj->library, newtbname);
	 if (force_flag)
	 {
	    kannounce(KCMS, routine, "Creating empty library `%s' to be the "
		      "associated library\nfor `%s' in `%s'", cmobj->library,
		      cmobj->oname, newtbname);
	    libchoice = 1;
	 }
	 if (libchoice == 0)
	    return NULL;
      }
      else
      {
	 kcms_close(libobj);
	 libchoice = 3;
      }
   }

   /*-- is there already a version of the object in destination? -----*/
   if ((new_cmobj = kcms_open_cmobj(toolbox, cmobj->oname)) != NULL)
   {
      if (new_cmobj->prog_type != cmobj->prog_type)
      {
         if ((kcms_query_bit(cmobj, KCMS_CMOBJ_FLAGS, KCMS_BIT_CMOBJ_FORCE,
	     &force_flag) && force_flag) || kprompt(KSTANDARD, NULL, NULL, 0,
		      "\nOperation: duplicate software object\n\n"
		      "\tToolbox:             %s\n"
		      "\tObject:              %s\n"
		      "\tDestination Toolbox: %s\n\n"
		      "The \"%s\" object in toolbox \"%s\" is not the "
		      "same type (%s) as the source object (%s).\n"
		      "Do you want to delete the object in the destination "
		      "toolbox?",
		      otbname, cmobj->oname,
		      cmobj->oname, newtbname,
		      kcms_attr_int2string(KCMS_CMOBJ_TYPE,
					   new_cmobj->prog_type),
		      kcms_attr_int2string(KCMS_CMOBJ_TYPE,
					   cmobj->prog_type)))
	 {
	    kcms_close(new_cmobj);
	    return NULL;
	 }
	 kcms_set_bit(new_cmobj, KCMS_CMOBJ_FLAGS, KCMS_BIT_CMOBJ_FORCE);
	 kcms_destroy(new_cmobj);
      }
      else
      {
	 updating = TRUE;

	 /*-- get list of files in destination, and make copy -----------*/
	 if (!kcms_get_attribute(new_cmobj, KCMS_CMOBJ_ALL_FILES, &df_attr))
	    return NULL;
	 destn_files = klist_copy(df_attr);
      }
   }

   if (!updating)
   {
      new_cmobj = kcms_create_cmobj(toolbox, cmobj->oname, cmobj->prog_type);
      if (new_cmobj == NULL)
         return NULL;
   }
   else
   {
      if ((kcms_query_bit(cmobj, KCMS_CMOBJ_FLAGS, KCMS_BIT_CMOBJ_FORCE,
	  &force_flag) && force_flag) || kprompt(KSTANDARD, NULL, NULL, 0,
			"\nOperation: duplicate software object\n\n"
			"\tToolbox:             %s\n"
			"\tObject:              %s\n"
			"\tDestination Toolbox: %s\n\n"
			"The object \"%s\" already exists in \"%s\",\n"
			"Do you want to update the destination object "
			"from source object?",
			otbname, cmobj->oname, newtbname, cmobj->oname,
			newtbname))
         kannounce(KCMS, routine, "updating %s in %s", cmobj->oname,newtbname);
      else
	 return NULL;
   }

   /*-- kroutine with associated library, duplicate the library? --*/
   if (cmobj->library != NULL)
   {
      if (!_kcms_duplicate_library(cmobj, toolbox, libchoice))
      {
	 /* kcms_destroy(new_cmobj); */
	 return NULL;
      }
      new_cmobj->library = kstrdup(cmobj->library);
   }

   kcms_get_attribute(cmobj->database, KCMS_FOBJ_FULLNAME_TKN, &dbToken);
   if (cmobj->makefile != NULL) 
      kcms_get_attribute(cmobj->makefile, KCMS_FOBJ_FULLNAME_TKN, &makeToken);

   /*-- we'll open the manpage dbm, ready to update if needs be ------*/
   ksprintf(temp, "$%s/repos/db/manpage", newtbname);
   dbm = kdbm_open(temp, (O_WRONLY | O_CREAT), 0666);


   for (; flist != NULL; flist = klist_next(flist))
   {
      token = (ktoken)klist_identifier(flist);

      fileobj1 = (kobject) klist_clientdata(flist);
      fpath = "(unknown)";
      if (!kcms_get_attributes(fileobj1,
                               KCMS_PATH,           &fpath,
			       KCMS_FOBJ_GENERATED, &fgen,
			       KCMS_FOBJ_TYPE,      &ftype,
			       KCMS_END)
          || (relpath = kstrchr(fpath, '/')) == NULL)
      {
         kwarn(KCMS, routine, "Could not get all attributes for file,\n"
	       "so file will not be copied with object:\n\n"
	       "\tFile: %s\n", fpath);
         continue;
      }


      /*-- don't copy the Makefile or object database! ---------------*/
      if (token == dbToken || token == makeToken)
	 continue;

      /*-- don't copy if file is generated ---------------------------*/
      if (fgen == KCMS_FOBJ_GEN_LFILE)
      {
	 continue;
      }

      /*-- is the file actually there? -------------------------------*/
      if (kaccess(fpath, R_OK) == -1)
      {
	 kwarn(KCMS, routine,
	       "The following file does not exist in the source object:\n"
	       "\t%s\n"
	       "It will not appear in the new object.\n", fpath);
	 continue;
      }

      if (updating)
      {
         ksprintf(destpath, "$%s%s", newtbname, relpath);
         dest_token = kstring_to_token(destpath);

         /*-- does the file exist in the destination list of files? ---*/
         if (klist_locate(destn_files, (kaddr)dest_token) != NULL)
            destn_files = klist_delete(destn_files, (kaddr)dest_token);
         else
	 {
            kannounce(KCMS, routine, "adding %s", destpath);
	 }
      }

      fileobj2 = kcms_duplicate(fileobj1, new_cmobj);
      if (fileobj2 == NULL)
      {
	 int itmp = errno;
	 errno = itmp;
	 kerror(NULL, routine, "%s%s%s%s%s%s%s",
		"Cannot copy file '", fpath, "' in cmobj '",
		cmobj->oname, "'.  The copy will continue, ",
		"but please note that this file will not ",
		"exist in the copied cmobj.");
      }

      /*-- update manpage database in destination toolbox ------------*/
      if (dbm != NULL && ftype == KCMS_FOBJ_TYPE_MAN)
         kcms_update_man_db(dbm, fileobj2, TRUE);
   }

   /*-- any other files left in destination list should be removed ---*/
   if (updating)
   {
      kcms_get_attribute(new_cmobj->database,
			 KCMS_FOBJ_FULLNAME_TKN, &dbToken);
      if (cmobj->makefile != NULL)
         kcms_get_attribute(new_cmobj->makefile,
			 KCMS_FOBJ_FULLNAME_TKN, &makeToken);

      for (flist = destn_files; flist != NULL; flist = klist_next(flist))
      {
         fobj = (kobject)klist_clientdata(flist);
         token = (ktoken)klist_identifier(flist);
         if (!kcms_get_attributes(fobj,
				 KCMS_PATH, &fpath,
				 KCMS_FOBJ_GENERATED, &fgen,
				 KCMS_END))
            continue;
   
         /*-- don't delete file if it is generated ----------------------*/
         if (token == dbToken || token == makeToken
	     || fgen == KCMS_FOBJ_GEN_LFILE)
	    continue;
   
         if ((kcms_query_bit(cmobj, KCMS_CMOBJ_FLAGS, KCMS_BIT_CMOBJ_FORCE,
			     &force_flag)
	      && force_flag)
	     || kprompt(KSTANDARD, NULL, NULL, 0,
			"\nOperation: duplicate software object\n\n"
			"\tToolbox:             %s\n"
			"\tObject:              %s\n"
			"\tDestination Toolbox: %s\n\n"
			"The destination object contains a file \"%s\",\n"
			"which does not appear in the source object.\n"
			"Do you want to remove file \"%s\" "
			"from destination object?",
			otbname, cmobj->oname, newtbname, fpath, fpath))
         {
            kannounce(KCMS, routine, "deleting %s", fpath);
            kcms_destroy(fobj);
         }
      }
      klist_free(destn_files, NULL);
   }

   if (dbm != NULL)
      kdbm_close(dbm);

   /*-------------------------------------------------------------------
   | Propagate attributes from the source object to the destination
   | object. Most of this is done with specific functions, which you'll
   | find defined below.
   -------------------------------------------------------------------*/

   kcms_set_attribute(new_cmobj, KCMS_CMOBJ_GEN_LIBNAME, cmobj->library);
   kcms_set_bit(cmobj, KCMS_CMOBJ_FLAGS, KCMS_BIT_CMOBJ_FORCE);
   if (cmobj->bname != NULL && kstrcmp(cmobj->oname, cmobj->bname))
      kcms_set_attribute(new_cmobj, KCMS_CMOBJ_BNAME, cmobj->bname);

   _kcms_cmobj_copy_string_attributes(cmobj, new_cmobj);
   _kcms_cmobj_copy_int_attributes(cmobj, new_cmobj);
   _kcms_cmobj_copy_ulong_attributes(cmobj, new_cmobj);
   _kcms_cmobj_copy_keywords(cmobj, new_cmobj);


   if (cmobj->prog_type == KCMS_PANE)
   {
      kstring  src_tbname, src_oname, src_args;
      kstring  dest_tbname, dest_oname, dest_args;

      if (kcms_get_attribute(cmobj, KCMS_CMOBJ_GENERATE_EXECUTABLE, &genexec))
	 kcms_set_attribute(new_cmobj, KCMS_CMOBJ_GENERATE_EXECUTABLE, genexec);

      if (kcms_get_attribute(cmobj, KCMS_CMOBJ_ASSOCIATED_OBJECT,
			     &src_tbname, &src_oname, &src_args)
	  && kcms_get_attribute(new_cmobj, KCMS_CMOBJ_ASSOCIATED_OBJECT,
			     &dest_tbname, &dest_oname, &dest_args)
	  && (kstrcmp(src_tbname, dest_tbname) != 0
	      || kstrcmp(src_oname, dest_oname) != 0
	      || kstrcmp(src_args, dest_args) != 0))
	 kcms_set_attribute(new_cmobj, KCMS_CMOBJ_ASSOCIATED_OBJECT,
			    src_tbname, src_oname, src_args);
   }
   else if ((cmobj->prog_type == KCMS_LIBRARY) && !updating)
   {
      kcms_add_tb_lib_dep(new_cmobj);
   }

   kcms_clear_bit(cmobj, KCMS_CMOBJ_FLAGS, KCMS_BIT_CMOBJ_FORCE);

   /*UPDATE: should set this via an attributes --*/
   kcms_set_bit(new_cmobj, KCMS_CMOBJ_FLAGS, KCMS_BIT_CMOBJ_FORCE);
   kcms_set_attribute(new_cmobj, KCMS_CMOBJ_UPDATE_DB,
		      (KCMS_UPDATE_SYNC | KCMS_UPDATE_REGEN |
		       KCMS_UPDATE_NEW | KCMS_UPDATE_CACHE));

   return new_cmobj;
}


/*-----------------------------------------------------------
| Routine Name:	_kcms_duplicate_library - duplicate lroutine associated with
|		a kroutine
|
| Purpose:	This function is used to duplicate the lroutine associated
|		with a kroutine.  It is calle when you are duplicating a
|		kroutine, for example when copying to a new toolbox.
|		In this situation, you have to ensure the library and lroutine
|		exist in the destination toolbox as well.
|
| Input:	cmobj		- the software object being duplicated
|		toolbox	- the destination toolbox
|
| Returns:	TRUE (1) on success, FALSE (0) otherwise
|
| Written By:	Neil Bowers
| Date:		17-nov-93
------------------------------------------------------------*/
static int
_kcms_duplicate_library(
   kobject  cmobj,
   kobject  toolbox,
   int      copy_choice)
{
   kstring       routine  = "Duplicate Associated Library";
   klist        *list     = NULL;
   kstring       tbname   = NULL;
   kstring       bname    = NULL;
   kstring       fobjpath = NULL;
   kstring       otbname  = NULL;
   kdbm		*dbm;
   kobject       oldlib;
   kobject       libobj;
   kobject       newfobj;
   kobject       fobj;
   char          curfpath[KLENGTH];
   char          curmpath[KLENGTH];
   char          temp[KLENGTH];
   kstring       cat;
   kstring       subcat;
   unsigned long lang_type;
   int           result = FALSE;


   /*UPDATE: need to clean up the logic of this routine! --*/

   if (!kcms_get_attribute(toolbox, KCMS_NAME, &tbname))
      return FALSE;

   if ((oldlib = kcms_open_cmobj(cmobj->parent, cmobj->library)) == NULL)
   {
      kerror(KCMS, "Duplicate Associated Library",
	     "Could not open library `%s' in source toolbox, %s.",
	     cmobj->library, tbname);
      return FALSE;
   }

   switch (copy_choice)
   {
      case 1:
	 /*-- Create library, copy just the l*.c file over ------------*/
	 if (!kcms_get_attributes(oldlib,
				  KCMS_CMOBJ_BNAME,       &bname,
				  KCMS_CMOBJ_CATEGORY,    &cat,
				  KCMS_CMOBJ_SUBCATEGORY, &subcat,
				  KCMS_CMOBJ_LANGTYPE,    &lang_type,
				  KCMS_END))
	    return FALSE;

	 libobj = kcms_generate_cmobj(toolbox, cmobj->library,
				      KCMS_LIBRARY, bname, cat, subcat,
				      "a most excellent library",
				      NULL, lang_type, FALSE, FALSE, FALSE,
				      NULL);
	 if (libobj == NULL)
	    return FALSE;
	 break;

      case 2:
	 /*-- Copy the entire library as well -------------------------*/
	 if ((libobj = kcms_duplicate(oldlib, toolbox)) == NULL)
	 {
	    kerror(KCMS, routine, "Library copy failed :-(");
	    result = FALSE;
	 }
	 else
	 {
	    kcms_close(libobj);
	    result = TRUE;
	 }
	 kcms_close(oldlib);
	 break;

      case 3:
	 /*-- The library already exists in the destination toolbox ---*/
	 libobj = kcms_open_cmobj(toolbox, cmobj->library);
	 if (libobj == NULL)
	 {
	    kerror(KCMS, routine, "Couldn't open destination object.");
	    return FALSE;
	 }
	 break;

      default:
	 errno = KCMS_EINTERNAL;
	 kerror(KCMS, routine, "Invalid copy-library-option specified.");
	 return FALSE;
   }

   /*-- entire library should be copied, so quit at this point --------*/
   if (copy_choice == 2)
      return result;

   kcms_get_attribute(cmobj->parent, KCMS_NAME, &otbname);

   ksprintf(curfpath, "$%s/objects/library/%s/src/l%s.c",
	    otbname, cmobj->library, cmobj->oname);
   ksprintf(curmpath, "$%s/objects/library/%s/man/l%s.3",
	    otbname, cmobj->library, cmobj->oname);
   if (!kcms_get_attribute(oldlib, KCMS_CMOBJ_ALL_FILES, &list))
   {
      kerror(KCMS, routine, "Unable to get list of files from old library.");

      /*UPDATE: should we do more on error? -- */
      return FALSE;
   }
   for (; list != NULL; list = klist_next(list))
   {
      fobj = (kobject) klist_clientdata(list);
      fobjpath = NULL;
      kcms_get_attribute(fobj, KCMS_PATH, &fobjpath);
      if (!kstrcasecmp(curfpath, fobjpath) || !kstrcasecmp(curmpath, fobjpath))
      {
	 newfobj = kcms_duplicate(fobj, libobj);
	 if (newfobj == NULL)
	    kerror(KCMS, routine, "newfobj failed");
	 if (!kstrcasecmp(curmpath, fobjpath))
	 {
            ksprintf(temp, "$%s/repos/db/manpage", tbname);
            dbm = kdbm_open(temp, (O_WRONLY | O_CREAT), 0666);
	    kcms_update_man_db(dbm, newfobj, TRUE);
	    kdbm_close(dbm);
	 }
      }
   }

   kcms_close(libobj);
   return TRUE;
}

/*-----------------------------------------------------------
| Routine Name:	_kcms_cmobj_copy_int_attributes - copy integer attrs
|
| Purpose:	This function propagates all software object attributes of
|		type int from a source to destination software object.
|
| Input:	source      - source software object
|		destination - destination software object
|
| Written By:	Neil Bowers
| Date:		8-may-96
------------------------------------------------------------*/
static void
_kcms_cmobj_copy_int_attributes(
   kobject  source,
   kobject  destination)
{
   static kcms_attr  attributes[] = { KCMS_CMOBJ_INCANTATA };
   int               i;
   int               source_value;
   int               dest_value;


   for (i = 0; i < (int)(sizeof(attributes)/sizeof(kcms_attr));i++)
   {
      if (kcms_get_attribute(source, attributes[i], &source_value)
	  && kcms_get_attribute(destination, attributes[i], &dest_value)
	  && source_value != dest_value)
      {
	 kcms_set_attribute(destination, attributes[i], source_value);
      }
   }
}

/*-----------------------------------------------------------
| Routine Name:	_kcms_cmobj_copy_ulong_attributes - copy unsigned long attrs
|
| Purpose:	This function propagates all software object attributes of
|		type unsigned long from a source to destination software
|		object.
|
| Input:	source      - source software object
|		destination - destination software object
|
| Written By:	Neil Bowers
| Date:		8-may-96
------------------------------------------------------------*/
static void
_kcms_cmobj_copy_ulong_attributes(
   kobject  source,
   kobject  destination)
{
   static kcms_attr  attributes[] = { KCMS_CMOBJ_LANGTYPE };
   int               i;
   unsigned long     source_value;
   unsigned long     dest_value;


   for (i = 0; i < (int)(sizeof(attributes)/sizeof(kcms_attr));i++)
   {
      if (kcms_get_attribute(source, attributes[i], &source_value)
	  && kcms_get_attribute(destination, attributes[i], &dest_value)
	  && source_value != dest_value)
      {
	 kcms_set_attribute(destination, attributes[i], source_value);
      }
   }
}

/*-----------------------------------------------------------
| Routine Name:	_kcms_cmobj_copy_string_attributes - copy string attributes
|
| Purpose:	This function propagates all software object attributes of
|		type string from a source to destination software
|		object.
|
| Input:	source      - source software object
|		destination - destination software object
|
| Written By:	Neil Bowers
| Date:		8-may-96
------------------------------------------------------------*/
static void
_kcms_cmobj_copy_string_attributes(
   kobject  source,
   kobject  destination)
{
   static kcms_attr  attributes[] = {	KCMS_CMOBJ_ICON_NAME,
					KCMS_CMOBJ_AUTHOR,
					KCMS_CMOBJ_AUTHOR_EMAIL,
					KCMS_CMOBJ_CATEGORY,
					KCMS_CMOBJ_SUBCATEGORY,
					KCMS_CMOBJ_SHORT_DESC,
					KCMS_CMOBJ_VERSION
				    };
   int               i;
   kstring           source_value;
   kstring           dest_value;


   for (i = 0; i < sizeof(attributes) / sizeof(kcms_attr); i++)
   {
      if (kcms_get_attribute(source, attributes[i], &source_value)
	  && kcms_get_attribute(destination, attributes[i], &dest_value)
	  && kstrcmp(source_value, dest_value) != 0)
      {
	 kcms_set_attribute(destination, attributes[i], source_value);
      }
   }
}

/*-----------------------------------------------------------
| Routine Name:	_kcms_cmobj_copy_keywords - copy object keywords
|
| Purpose:	This function propagates all software object keywords
|		from a source to destination software object.
|
| Input:	source      - source software object
|		destination - destination software object
|
| Written By:	Neil Bowers
| Date:		8-may-96
------------------------------------------------------------*/
static void
_kcms_cmobj_copy_keywords(
   kobject  source,
   kobject  destination)
{
   klist   *keywords;
   ktoken   token;
   kstring  word;


   for (keywords = klist_head(source->keywords);
	keywords != NULL;
	keywords = klist_next(keywords))
   {
      token = (ktoken)klist_identifier(keywords);
      word  = (kstring)klist_clientdata(keywords);

      destination->keywords = klist_add(destination->keywords, (kaddr)token,
					(kaddr)kstrdup(word));
   }
}
