/*
 * Khoros: $Id: environment.c,v 1.11 1997/01/20 04:06:19 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: environment.c,v 1.11 1997/01/20 04:06:19 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		
   >>>>   Public:
   >>>>                 kcms_query_toolboxes()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


/************************************************************
* Routine Name:	kcms_query_toolboxes - determine available toolboxes
*
* Purpose:	This routine determines the list of available toolboxes,
*		and returns an array of string pointers containing the names
*		of the toolboxes.
*
*		The use of this function is illustrated by the following code:
*		!\f(CW\s-1
*		!   kstring  *toolbox_names;
*		!   int       toolbox_count;
*		!   int       i;
*		!
*		!   toolbox_names = kcms_query_toolboxes(&toolbox_count);
*		!   for (i = 0; i < toolbox_count; i++)
*		!      kprintf("Toolbox %d is %s\n", i, toolbox_names[i]);
*		!\s+1\fP
*
* Output:	count - A pointer to an integer.  The number of toolbox
*			names returned will be stored in the integer
*			referenced. If this argument is NULL, then
*			kcms_query_toolboxes() will return NULL.
* Returns:	An array of string pointers, or NULL on failure.
*
* Written By:	Steven Jorgensen, Neil Bowers
* Date:		30-aug-94
*************************************************************/
kstring *
kcms_query_toolboxes(
   size_t *count)
{
   size_t    numtb;
   char    **list;


   if (count == NULL)
   {
      errno = KCMS_EINTERNAL;
      kerror(KCMS, "kcms_query_toolboxes()",
	     "Function called with a NULL pointer for the toolbox count.\n");
      return NULL;
   }

   list = ktoolbox_list(&numtb);
   *count = numtb;

   return (kstring *)karray_copy(list, KSTRING, numtb, TRUE);
}
