/*
 * Khoros: $Id: clui_codedoc.c,v 1.17.2.4 1997/05/24 17:48:57 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: clui_codedoc.c,v 1.17.2.4 1997/05/24 17:48:57 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Generate Manpage and Whatis for a Software Object
   >>>>		
   >>>>   Static:
   >>>>                 _kgen_insert_lp()
   >>>>   Public:
   >>>>                 kgen_generate_whatis()
   >>>>                 kgen_generate_manpage()
   >>>>                 kgen_generate_htmlpage()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

#define WHATISFILE "repos/config/whatis"

/*-----------------------------------------------------------
|  Routine Name: _kgen_insert_lp - insert .LP into a text block
|
|       Purpose: This routine looks for a blank lines in the text
|		 block, and replaces them with a .LP line.
|
|         Input: file   - an open kfile pointer
|		 string - the text block to fix
|        Output:
|       Returns:
|
|    Written By: Steven Jorgensen
|          Date: Dec 10, 1992 05:09
------------------------------------------------------------*/
static void 
_kgen_insert_lp(
   kfile    *file,
   kstring   string)
{
   kstring  newstr;
   kstring  tmp;


   if (string == NULL)
   {
      kfprintf(file, "none\n");
      return;
   }

   tmp = kstring_replace(string, "\n\n", "\n.LP\n", NULL);
   newstr = kregex_replace(tmp, "^(\\.LP\n)+", ".LP\n", NULL);
   kfree_and_NULL(tmp);
/*
   (void) kparse_string_search(newstr,"\\.LP\n\\.LP",KLITERAL,NULL,&status);
   while (status == KPARSE_OK)
   {
      tmp = kstring_replace(newstr, "^.LP\n.LP", ".LP", NULL);
      kfree_and_NULL(newstr);
      newstr = tmp;
      tmp = NULL;
      (void)kparse_string_search(newstr,"\\.LP\n\\.LP",KLITERAL,NULL,&status);
   }
*/
   kfprintf(file, "%s\n", newstr);
   kfree_and_NULL(newstr);
}

/************************************************************
*  Routine Name: kgen_generate_whatis - update man directory whatis file
*
*       Purpose: This routine takes a new routine name, its short
*		 description, the library name, and the toolbox name,
*		 and either inserts the item into the toolboxes whatis file,
*		 or updates the current entry in the whatis file.
*
*         Input: object     - The kobject pointer of the entry to be
*			      added to the
*		 func_name  - The function name in the object.
*		 short_desc - The short description of the routine
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Steven Jorgensen
*          Date: Feb 16, 1993 13:16
*************************************************************/
int 
kgen_generate_whatis(
   kobject	 object,
   char		*func_name,
   char		*short_desc)
{
   kstring         routine  = "kgen_generate_whatis";
   kobject	   toolbox;
   char		   key[KLENGTH], path[KLENGTH];
   char		   temp[KLENGTH], tbname[KLENGTH];
   char		   paren[KLENGTH];
   char		  *lib_name, *tb_name, *data, *newstr, **array, *sd,*tmp;
   size_t	   i, array_cnt;
   int             status;
   unsigned long   otype;
   kfile	  *file;


   if (!func_name)
      return FALSE;

   if (!kcms_get_attributes(object,
			    KCMS_NAME,       &lib_name,
			    KCMS_PARENT,     &toolbox,
			    KCMS_CMOBJ_TYPE, &otype,
			    KCMS_END)
       || !kcms_get_attribute(toolbox, KCMS_NAME, &tb_name))
      return FALSE;

   kstring_upper(tb_name, tbname);
   kstrcpy(paren, "()");
   if (otype != KCMS_LIBRARY)
      paren[0] = '\0';
   sd = (short_desc == NULL) ? "" : short_desc;
   ksprintf(key, "^%s\\(?\\)? \\(%s\\)", func_name, lib_name);
   ksprintf(path, "$%s/%s", tbname, WHATISFILE);
   ksprintf(temp, "%s%s (%s)\t\t-  %s", func_name, paren, lib_name, sd);
   newstr = kstrdup(temp);
   data = (char *) kreadfile(path, NULL);
   array = kparse_string_delimit(data, "\n", KDELIM_CLEAN,
				 (ssize_t *) &array_cnt);
   (void) kparse_string_search(data, key, KLITERAL, NULL, &status);
   kfree_and_NULL(data);
   if (status == KPARSE_OK)
   {
      if ((tmp = kre_comp(key)) != NULL)
      {
	 kerror(KCODEGEN, routine, "kre_comp returned the error '%s'", tmp);
	 karray_free(array, KSTRING, array_cnt, NULL);	
	 kfree_and_NULL(newstr);
	 return FALSE;
      }
			  
      for (i = 0; i < array_cnt; i++)
	 if (kre_exec(array[i]))
	    break;
      if (i < array_cnt)
      {
	 kfree_and_NULL(array[i]);
	 array[i] = newstr;
      }
      else
      {
	 kerror(KCODEGEN, routine,
		"kparse_string_search says that '%s' is in "
		"the whatis file, but the entry cannot be "
		"found in the array.\n", func_name);
	 karray_free(array, KSTRING, array_cnt, NULL);
	 kfree_and_NULL(newstr);
	 return FALSE;
      }
   }
   else
   {
      array = (char **)karray_add(array, KSTRING, newstr, array_cnt);
      array_cnt++;
      array = (char **)karray_sort(array, KSTRING, array_cnt, FALSE);
   }

   if ((file = kfopen(path, "w")) == NULL)
   {
      kerror(KCODEGEN, routine,
	     "Cannot write to '%s' to update the whatis file.", path);
      karray_free(array, KSTRING, array_cnt, NULL);
      return FALSE;
   }
   for (i = 0; i < array_cnt; i++)
      kfprintf(file, "%s\n", array[i]);
   kfclose(file);

   karray_free(array, KSTRING, array_cnt, NULL);
   return TRUE;
}

/************************************************************
*  Routine Name: kgen_generate_manpage - generate a man page
*
*       Purpose: This routine generates a man page given a
*		 header_info structure to work from.
*
*         Input: object   - the library object to generate a man page for
*                header   - the header_info to generate a man page from
*		 path     - the place to put the man page
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*    Written By: Steven Jorgensen
*          Date: Feb 15, 1993 14:46
*************************************************************/
int 
kgen_generate_manpage(
	kobject		object,
	header_info	*header,
	char		*path)
{
   kstring         routine  = "kgen_generate_manpage";
   kobject		toolbox;
   char		*lib_name, *tb_name, *opath;
   char		manpath[KLENGTH];
   kfile		*file;
   kstring         copyright;
   char		*add_begin	= "^.SH ADDITIONAL INFORMATION$";
   char		*add_end	= "^.SH (EXAMPLES|SIDE EFFECTS)$";
   char		*add_info	= NULL;
   char		*sa_begin	= "^.SH SEE ALSO$";
   char		*sa_end		= "^.SH COPYRIGHT$";
   char		*sa_info	= NULL;
   char		*upper;
   char		*pagename;
   int		i, status;


   if (!kcms_get_attributes(object,
			    KCMS_NAME,   &lib_name,
			    KCMS_PATH,   &opath,
			    KCMS_PARENT, &toolbox,
			    KCMS_END)
       || !kcms_get_attribute(toolbox, KCMS_NAME, &tb_name))
   {
      kwarn(KCODEGEN, routine,
	    "Could not get all attributes required,\n"
	    "so man page was not generated.");
      return (FALSE);
   }
 
   /* change any colons in name to underscores */
   pagename = kchar_replace(header->name, ':', '_', NULL);

   if (path == NULL)
      sprintf(manpath, "%s/man/%s.3", opath, pagename);
   else
      sprintf(manpath, "%s/%s.3", path, pagename);

   copyright = kcms_get_copyright(toolbox, KCMS_TB_COPYRIGHT_SHORT,
				  KCMS_LANG_NONE);

   if ((kaccess(manpath, R_OK) == 0) &&
       ((file = kfopen(manpath, "r")) != NULL))
   {
      add_info = kparse_file_scan(file, add_begin, add_end, KIGNORE_CASE,
				  NULL, NULL, &status);
      krewind(file);
      sa_info = kparse_file_scan(file, sa_begin, sa_end, KIGNORE_CASE,
				 NULL, NULL, &status);
      kfclose(file);
   }

   if ((file = kfopen(manpath, "w")) == NULL)
   {
      kerror(KCODEGEN, "kgen_generate_manpage",
	     "Unable to open '%s' for writing\n", manpath);
      kfree_and_NULL(add_info);
      kfree_and_NULL(sa_info);
      return (FALSE);
   }

   upper = kstring_upper(tb_name, NULL);
   kfprintf(file, ".TH \"%s\" \"%s\" \"LIBRARY\" \"(%s)\" \"%s\"\n",
	    header->name, upper, lib_name, PRODUCT_RELEASE_DATE);
   kfree_and_NULL(upper);
   kfprintf(file, ".nr LL 7.0i\n");
   kfprintf(file, ".SH LIBRARY ROUTINE\n");
   if (header->shortdesc == NULL)
      kfprintf(file, "%s\n", header->name);
   else
      kfprintf(file, "%s - %s\n", header->name, header->shortdesc);

   kfprintf(file, ".SH LIBRARY CALL\n");
   if (header->declaration != NULL)
   {
      kfprintf(file, ".nf\n");
      kfprintf(file, "%s\n", header->declaration);
      kfprintf(file, ".fi\n");
   }
   else
   {
      kwarn("kcodegen", routine,
	    "Declaration for routine %s is missing!\n"
	    "Please check header of macro, it must include the "
	    "'Declaration:' field", header->name);
   }

   kfprintf(file, ".SH INPUT\n");
   if (header->in_cnt <= 0)
   {
      kfprintf(file, ".LP\n");
      kfprintf(file, "none\n");
   }
   else
      for (i = 0; i < header->in_cnt; i++)
      {
	 kfprintf(file, ".IP \"%s -\" 18\n", header->input[i][0]);
	 if (header->input[i][1] != NULL)
	    kfprintf(file, "%s\n", header->input[i][1]);
	 else
	 {
	    kwarn(KCODEGEN, routine,
		  "routine '%s' does not have an "
		  "explanation for the input parameter "
		  "'%s'\n",
		  header->name, header->input[i][0]);
	    kfprintf(file, "No explaination available.\n");
	 }
      }
   kfprintf(file, ".SH OUTPUT\n");
   if (header->out_cnt <= 0)
   {
      kfprintf(file, ".LP\n");
      kfprintf(file, "none\n");
   }
   else
      for (i = 0; i < header->out_cnt; i++)
      {
	 kfprintf(file, ".IP \"%s -\" 18\n", header->output[i][0]);
	 if (header->output[i][1] != NULL)
	    kfprintf(file, "%s\n", header->output[i][1]);
	 else
	 {
	    kwarn(KCODEGEN, routine,
		  "routine '%s' does not have an "
		  "explanation for the output parameter "
		  "'%s'\n",
		  header->name, header->output[i][0]);
	    kfprintf(file, "No explaination available.\n");
	 }
      }

   kfprintf(file, ".SH RETURN VALUE\n");
   if (header->returns == NULL)
      kfprintf(file, "none\n");
   else
      kfprintf(file, "%s\n", header->returns);

   kfprintf(file, ".SH DESCRIPTION\n");
   _kgen_insert_lp(file, header->purpose);

   kfprintf(file, ".SH ADDITIONAL INFORMATION\n");
   if (add_info == NULL)
   {
      kfprintf(file, ".LP\n");
      kfprintf(file, "none\n");
   }
   else
      kfprintf(file, "%s", add_info);
   kfree_and_NULL(add_info);

   kfprintf(file, ".SH EXAMPLES\n");
   _kgen_insert_lp(file, header->examples);
   kfprintf(file, ".SH SIDE EFFECTS\n");
   _kgen_insert_lp(file, header->sideeff);

   kfprintf(file, ".SH RESTRICTIONS\n");
   _kgen_insert_lp(file, header->restrictions);

   kfprintf(file, ".SH MODIFICATION\n");
   _kgen_insert_lp(file, header->mods);

   kfprintf(file, ".SH FILES\n");
   _kgen_insert_lp(file, header->filename);

   kfprintf(file, ".SH SEE ALSO\n");
   if (sa_info == NULL)
      kfprintf(file, "%s(3)\n", lib_name);
   else
      kfprintf(file, "%s", sa_info);
   kfree_and_NULL(sa_info);

/*
    kfprintf(file, ".SH AUTHOR\n");
    _kgen_insert_lp(file, header->author);
*/

   kfprintf(file, ".SH COPYRIGHT\n");
   if (copyright != NULL)
   {
      kfprintf(file, "%s\n",copyright);
      kfree_and_NULL(copyright);
   }

   kfclose(file);

   if (!kgen_generate_whatis(object, header->name, header->shortdesc))
      kinfo(KSYSLIB, "Unable to update the toolbox whatis file.");


   /* Add the manpage to the toolbox manpage database */
   if (path != NULL)
   {
      kdbm   *dbm;
      kdatum  key, data;
      char    temp[KLENGTH], temp2[KLENGTH];


      ksprintf(temp, "$%s/repos/db/manpage", tb_name);
      if ((dbm = kdbm_open(temp, O_WRONLY | O_CREAT, 0666)) != NULL)
      {
	 ksprintf(temp, "$%s", tb_name);
	 if (kfullpath(temp, NULL, temp2) == NULL)
	    return(FALSE);
	 key.dptr = header->name;
	 key.dsize = kstrlen(key.dptr) + 1;
	 data.dptr = kstring_replace(manpath, temp2, temp, manpath);
	 data.dsize = kstrlen(data.dptr) + 1;
	 kdbm_store(dbm, key, data, KDBM_REPLACE);
	 kdbm_close(dbm);
      }

      ksprintf(temp, "%s/db/function", opath);
      if ((dbm = kdbm_open(temp, O_WRONLY | O_CREAT, 0666)) != NULL)
      {
	 key.dptr = header->name;
	 key.dsize = kstrlen(key.dptr) + 1;
	 data.dptr = header->filename;
	 data.dsize = kstrlen(data.dptr) + 1;
	 kdbm_store(dbm, key, data, KDBM_REPLACE);
	 kdbm_close(dbm);
      }
   }
   kfree_and_NULL(pagename);
   return (TRUE);
}
/************************************************************
*  Routine Name: kgen_generate_htmlpage - generate an html page
*
*       Purpose: This routine generates an html page given a
*		 header_info structure to work from.
*
*         Input: object   - the library object to generate an html page for
*                header   - the header_info to generate an html page for
*		 path     - the place to put the html page
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*    Written By: Kevin Sullivan, after Steven Jorgensen
*          Date: Nov 5, 1996 14:46
*************************************************************/
int 
kgen_generate_htmlpage(
	kobject		object,
	header_info	*header,
	char		*path)
{
   kstring         routine  = "kgen_generate_htmlpage";
   kobject		toolbox;
   char		*lib_name, *tb_name, *opath;
   char		manpath[KLENGTH];
   kfile		*file;
   kstring         copyright;
   char		*add_begin	= "^.SH ADDITIONAL INFORMATION$";
   char		*add_end	= "^.SH (EXAMPLES|SIDE EFFECTS)$";
   char		*add_info	= NULL;
   char		*sa_begin	= "^.SH SEE ALSO$";
   char		*sa_end		= "^.SH COPYRIGHT$";
   char		*sa_info	= NULL;
   char		*upper;
   char		*pagename;
   int		i, status;


   if (!kcms_get_attributes(object,
			    KCMS_NAME,   &lib_name,
			    KCMS_PATH,   &opath,
			    KCMS_PARENT, &toolbox,
			    KCMS_END)
       || !kcms_get_attribute(toolbox, KCMS_NAME, &tb_name))
   {
      kwarn(KCODEGEN, routine,
	    "Could not get all attributes required,\n"
	    "so man page was not generated.");
      return (FALSE);
   }

   /* change any colons in name to underscores */
   pagename = kchar_replace(header->name, ':', '_', NULL);

   /* yes, this is correct.  get info from man page for html page */
   sprintf(manpath, "%s/man/%s.3", opath, pagename);

   copyright = kcms_get_copyright(toolbox, KCMS_TB_COPYRIGHT_SHORT,
				  KCMS_LANG_NONE);

   if ((kaccess(manpath, R_OK) == 0) &&
       ((file = kfopen(manpath, "r")) != NULL))
   {
      add_info = kparse_file_scan(file, add_begin, add_end, KIGNORE_CASE,
				  NULL, NULL, &status);
      krewind(file);
      sa_info = kparse_file_scan(file, sa_begin, sa_end, KIGNORE_CASE,
				 NULL, NULL, &status);
      kfclose(file);
   }

   if (path == NULL)
      sprintf(manpath, "%s/html/%s.htm", opath, pagename);
   else
      sprintf(manpath, "%s/%s.htm", path, pagename);

   if ((file = kfopen(manpath, "w")) == NULL)
   {
      kerror(KCODEGEN, routine,
	     "Unable to open '%s' for writing\n", manpath);
      kfree_and_NULL(add_info);
      kfree_and_NULL(sa_info);
      return (FALSE);
   }

   upper = kstring_upper(tb_name, NULL);

   kfprintf(file, "<HTML>\n");
   kfprintf(file, "<HEAD>\n");
   kfprintf(file, "     <TITLE> %s Online Help Page </TITLE>\n", header->name);
   kfprintf(file, "</HEAD>\n");
   kfprintf(file, "<BODY>\n");

   kfprintf(file, "<H2>%s (<A HREF=\"%s.htm\">%s</A>) functions</H2><HR>\n", 
            upper, lib_name, lib_name);
   kfree_and_NULL(upper);

   kfprintf(file, "<H3>LIBRARY ROUTINE </H3>\n");
   if (header->shortdesc == NULL)
      kfprintf(file, "%s\n", header->name);
   else
      kfprintf(file, "%s - %s\n", header->name, header->shortdesc);

   kfprintf(file, "<H3>LIBRARY CALL </H3>\n");
   if (header->declaration != NULL)
   {
      kfprintf(file, "<CODE><PRE>\n");
      kgen_roff2html(file, header->declaration);
      kfprintf(file, "</PRE></CODE>\n");
   }
   else
   {
      kwarn("kcodegen", routine,
	    "Declaration for routine %s is missing!\n"
	    "Please check header of macro, it must include the "
	    "'Declaration:' field", header->name);
   }

   kfprintf(file, "<H3>INPUT </H3>\n");
   if (header->in_cnt <= 0)
   {
      kfprintf(file, "none\n");
   }
   else
   {
      kfprintf(file, "<UL>\n");
      for (i = 0; i < header->in_cnt; i++)
      {
	 kfprintf(file, "<LI><EM>%s</EM>", header->input[i][0]);
	 if (header->input[i][1] != NULL)
         {
	    kfprintf(file, " - ");
            kgen_roff2html(file, header->input[i][1]);
         }
	 else
	 {
	    kwarn(KCODEGEN, routine,
		  "routine '%s' does not have an "
		  "explanation for the input parameter "
		  "'%s'\n",
		  header->name, header->input[i][0]);
	    kfprintf(file, "No explanation available.\n");
	 }
      }
      kfprintf(file, "</UL>\n");
   }

   kfprintf(file, "<H3>OUTPUT </H3>\n");
   if (header->out_cnt <= 0)
   {
      kfprintf(file, "<P>\n");
      kfprintf(file, "none\n");
   }
   else
   {
      kfprintf(file, "<UL>\n");
      for (i = 0; i < header->out_cnt; i++)
      {
	 kfprintf(file, "<LI><EM>%s</EM>", header->output[i][0]);
	 if (header->output[i][1] != NULL)
         {
	    kfprintf(file, " - ");
            kgen_roff2html(file, header->output[i][1]);
         }
	 else
	 {
	    kwarn(KCODEGEN, routine,
		  "routine '%s' does not have an "
		  "explanation for the output parameter "
		  "'%s'\n",
		  header->name, header->output[i][0]);
	    kfprintf(file, "No explanation available.\n");
	 }
      }
      kfprintf(file, "</UL>\n");
   }

   kfprintf(file, "<H3>RETURN VALUE </H3>\n");
   if (header->returns == NULL)
      kfprintf(file, "none\n");
   else
      kgen_roff2html(file, header->returns);

   kfprintf(file, "<H3>DESCRIPTION </H3>\n");
   kgen_roff2html(file, header->purpose);

   kfprintf(file, "<H3>ADDITIONAL INFORMATION </H3>\n");
   if (add_info == NULL)
   {
      kfprintf(file, "none\n");
   }
   else
      kgen_roff2html(file, add_info);
   kfree_and_NULL(add_info);

   kfprintf(file, "<H3>EXAMPLES </H3>\n");
   if (kstrstr(header->examples, ".begin code") == NULL)
   {
      kfprintf(file, "<PRE>\n");
      kgen_roff2html(file, header->examples);
      kfprintf(file, "</PRE>\n");
   }
   else
   {
      kgen_roff2html(file, header->examples);
   }

   kfprintf(file, "<H3>SIDE EFFECTS </H3>\n");
   kgen_roff2html(file, header->sideeff);

   kfprintf(file, "<H3>RESTRICTIONS </H3>\n");
   kgen_roff2html(file, header->restrictions);

   kfprintf(file, "<H3>MODIFICATION </H3>\n");
   kgen_roff2html(file, header->mods);

   kfprintf(file, "<H3>FILES </H3>\n");
   kfprintf(file, "<CODE>\n");
   kgen_roff2html(file, header->filename);
   kfprintf(file, "</CODE>\n");

   kfprintf(file, "<H3>SEE ALSO </H3>\n");
   if (sa_info == NULL)
      kfprintf(file, "%s(3)\n", lib_name);
   else
      kfprintf(file, "%s", sa_info);
   kfree_and_NULL(sa_info);

/*
    kfprintf(file, "<H3>AUTHOR </H3>\n");
    kgen_roff2html(file, header->author);
*/

   kfprintf(file, "<H3>COPYRIGHT </H3>\n");
   if (copyright != NULL)
   {
      kfprintf(file, "%s\n",copyright);
      kfree_and_NULL(copyright);
   }

   kfprintf(file, "</BODY>\n");
   kfprintf(file, "</HTML>\n");

   kfclose(file);

   kfree_and_NULL(pagename);
   return (TRUE);
}
