/*
 * Khoros: $Id: clui_getargs.c,v 1.16.2.1 1997/03/11 23:53:59 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: clui_getargs.c,v 1.16.2.1 1997/03/11 23:53:59 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Routines to generate usage.c file
   >>>>
   >>>>  Private:
   >>>>                 kgen_clui_generate_getargs_file()
   >>>>   Static:
   >>>>                 print_get_args_routine()
   >>>>                 print_get_single_arg()
   >>>>                 print_kclui_get_call()
   >>>>                 number_toggle_found()
   >>>>                 print_uis_spec()
   >>>>                 print_init_uis_spec()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static void print_get_args_routine  PROTO((kobject, kform *, kfile *));
static void print_get_single_arg    PROTO((kobject, kfile *, kselection *));
static void print_kclui_get_call    PROTO((kobject, kfile *, kselection *, 
                                           char *, int, char *));
static int  number_toggle_found     PROTO((kselection *));
static int  print_uis_spec          PROTO((kobject, kfile *));
static int  print_init_uis_spec     PROTO((char *, kfile *, char *));



/*-----------------------------------------------------------
|
|  Routine Name: kgen_clui_generate_getargs_file
|
|       Purpose: This routine creates the file usage.c. 
|                Note that "usage.c" is a historical misnomer
|                since the generated file does not contain any code to deal
|                with usage, which is done interpretively by
|                kgen_clui_print_usage(). The usage.c file contains:
|                    1 - {oname}_get_args()
|                        which gets the CLUI args off the command line and
|                        into the clui_info structure
|		     2 - the clui_uis and gui_uis static arrays of strings
|			 that come from the *.pane and *.form files of the
|			 software object
|
|         Input: program   -  program object being created
|                opath     -  path to program object being created
|                oname     -  name of program object being generated
|                bname     -  name of the program binary being generated
|
|        Output: Returns TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: Spet 2, 1993
| Modifications: Converted from Khoros 1.0 (DA)
|
------------------------------------------------------------*/
int  kgen_clui_generate_getargs_file(
   kobject   program,
   kform    *form,
   char     *opath,
   char     *oname)
{
        kstring       routine     = "kgen_clui_generate_getargs_file()";
	int           force_flag  = FALSE;
	int           isnewfile   = FALSE;
	kobject       file_object = NULL;
	char         *filepath    = NULL;	
        kobject       toolbox;
        kfile        *file;
	unsigned long lang_type;
	long	      sub_type = KCMS_FOBJ_SUBTYPE_C;
	char         *copyright;
	char          path[KLENGTH]; 
	char          usage[KLENGTH];
	char          temp[KLENGTH];
	kstring	      rcsid_info = NULL;
	kstring	      rcslog_info = NULL;

        /* 
         * get file object representing usage.c file 
         */
        if (!kcms_get_attributes(program,
				 KCMS_PARENT,          &toolbox,
				 KCMS_CMOBJ_GEN_MAIN,  &file_object,
				 KCMS_CMOBJ_LANGTYPE,  &lang_type,
	                         KCMS_END)
	    || !kcms_query_bit(program, KCMS_CMOBJ_FLAGS, KCMS_BIT_CMOBJ_FORCE,
			       &force_flag))
	    return(FALSE);

	/*
         * 1st time usage.c file has been created - create file object,
         * so that kcms registers the usage.c file in the kcms database
         */
        if (file_object == NULL)
        {
	    if (lang_type == KCMS_LANG_CPLUSPLUS)
	    {
		ksprintf(temp, "%s/src/usage.C", opath);
		sub_type = KCMS_FOBJ_SUBTYPE_CPLUSPLUS;
	    }
	    else
		ksprintf(temp, "%s/src/usage.c", opath);
            file_object = kcms_create_fileobj(program, temp, NULL,
					      KCMS_FOBJ_TYPE_SRC,
					      sub_type,
					      KCMS_FOBJ_GEN_GWFILE,
					      KCMS_FOBJ_ACCESS_RDONLY);
            if (file_object == NULL)
                return(FALSE);
            isnewfile = TRUE;
	    filepath = temp;
	}

        /*
         * the usage.c file has already been created & added to kcms database;
         * simply get the path to the help page.
         */
	else if (!kcms_get_attribute(file_object, KCMS_PATH, &filepath))
            return(FALSE);
	
        /*
         *  see if usage.c file already exists.  If so, prompt to over-write,
         *  return TRUE if the answer is NO
         */
        if(kdirname(filepath, path) == NULL)
            ksprintf(path, ".");

	if (lang_type == KCMS_LANG_CPLUSPLUS)
           ksprintf(usage, "%s/usage.C", path);
	else
           ksprintf(usage, "%s/usage.c", path);
	if (kaccess(usage, R_OK) != 0)
	   isnewfile = TRUE;

        if (!force_flag && koverwrite(kget_notify(), usage) == FALSE)
           return(TRUE);

	/*
         * Save RCS information
         */
	if (isnewfile == FALSE &&
	    kgen_rcs_save(usage, &rcsid_info, &rcslog_info) == FALSE)
	{
	    kerror(KCODEGEN, routine, "Could not save RCS headers for "
		   "%s. Aborting..", usage);
	    return(FALSE);
        }
        /*
         *  create & open *.c file
         */
        if ((file = kfopen(usage, "w"))== NULL)
        {
            kerror(KCODEGEN, routine,
                   "Could not create file '%s'", usage);
	    kfree_and_NULL(rcsid_info);
	    kfree_and_NULL(rcslog_info);
            return(FALSE);
        }

        /*
         * begin l*.c file with RCS header
         */
        kgen_rcs_chdr(file, rcsid_info, rcslog_info);
	kfree_and_NULL(rcsid_info);
	kfree_and_NULL(rcslog_info);

        /*
         * follow RCS header with copyright
         */
	copyright = kcms_get_copyright(toolbox, KCMS_TB_COPYRIGHT_LONG,
				       KCMS_LANG_C);

        if (copyright != NULL)
            kfprintf(file, "%s\n", copyright);
        kfree_and_NULL(copyright);

        /*
         *  generate kgen_get_args() routine
         */
	kfprintf(file, "#include \"%s.h\"\n", oname);
        print_get_args_routine(program, form, file);
        kfprintf(file, "\n\n");

	/*
	 * generate UIS structure definition
	 */
        print_uis_spec(program, file);
        kfprintf(file, "\n\n");

	/*
	 *  close file and finish
	 */
        kfclose(file);

        return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: print_get_args_routine
|
|       Purpose: This routine prints the get_args() routine in file *.c
|
|         Input: object  - The software object being generated.
|		 form    - pointer to the form tree from *.pane
|                file    - open stream to .c file
|        Output: none
|    Written By: Danielle Argiro
|          Date: March 28, 1994
| Modifications: 
|
------------------------------------------------------------*/

static void print_get_args_routine(
   kobject  object,
   kform   *form,
   kfile   *file)
{
	char           routine_name[KLENGTH]; 
	kobject        form_file;
	unsigned long  object_type;
	kstring        oname;
	char           purpose[KLENGTH]; 
	char           temp[KLENGTH];
	kselection    *selection;
	kselection    *group;
	kselection    *subgroup;
	int            hybrid = FALSE;


	if (!kcms_get_attributes(object,
				 KCMS_CMOBJ_TYPE,     &object_type,
				 KCMS_CMOBJ_UIS_FORM, &form_file,
				 KCMS_NAME,           &oname,
                                 KCMS_END))
	   return;

	hybrid = (object_type == KCMS_XVROUTINE && form_file == NULL);

	/*
	 *   put in header for get_args routine
	 */
	ksprintf(routine_name, "%s_get_args", oname);
	ksprintf(purpose, "Gets command line args for %s", oname);
        ksprintf(temp, "ghostwriter -oname %s", oname);
        kgen_privsrc_hdr(file, routine_name, purpose, 
			 "None", "None", temp, NULL);
	/*
	 *   put in beginning of kgen_get_args routine
	 */
	kfprintf(file, "void %s_get_args(\n    kaddr opane)\n{\n\n", oname);

	if ((kgen_arg_total > 0) || (object_type == KCMS_XVROUTINE
				      && !hybrid))
	{
	   kfprintf(file, "\tstatic char *sync_error_mesg = \"Generated "
		    "code for %s is out of sync with arguments specified "
		    "in %s's *.pane file; please rerun ghostwriter on "
		    "%s.\";\n\n", oname, oname, oname);
	}

	kfprintf(file, "\tkform *pane = (kform *) opane;\n\n");
	
	if (number_toggle_found(form->subform->guide->pane->sel_list))
	    kfprintf(file, "\tchar  *value_string;\n\n");

        /*
         *  print code to allocate room for argument structure
         */
         if (kgen_arg_total > 0 || (object_type == KCMS_XVROUTINE && !hybrid))
         {
             kfprintf(file, "\n\t/*\n\t * allocate the %s structure\n\t */\n",
		      oname);
             kfprintf(file, "\tclui_info = (clui_info_struct *) \n"
		      "\t       kcalloc((size_t)1, "
		      "sizeof (clui_info_struct));\n\n");
         }
	selection = form->subform->guide->pane->sel_list;
	while (selection != NULL)
	{
	    print_get_single_arg(object, file, selection);

	    group = selection->group_next;
	    while (group != NULL)
	    {
		print_get_single_arg(object, file, group);
		subgroup = group->group_next;
		while (subgroup != NULL)
		{
		    print_get_single_arg(object, file, subgroup);
		    subgroup = subgroup->next;
		}
		group = group->next;
	    }
	    selection = selection->next;
	}

	/*
	 *  xvroutines have the special-purpose [-form] argument
 	 */
	if (object_type == KCMS_XVROUTINE && !hybrid)
	{
	     kfprintf(file, "\tif (!(kclui_get_infile(pane, \"form\",\n");
             kfprintf(file,  "\t                &(clui_info->form_file),\n");
             kfprintf(file,  "\t                &(clui_info->form_flag))))\n");
	     kfprintf(file,  "\t{\n\t    kerror(NULL, \"%s_get_args\", sync_error_mesg);\n", oname);
	     kfprintf(file, "\t    kexit(KEXIT_FAILURE);\n\t}\n");

	     kfprintf(file, "\tif (!(kclui_get_int(pane, \"x\",\n");
             kfprintf(file,  "\t                &(clui_info->x_int),\n");
             kfprintf(file,  "\t                &(clui_info->x_flag))))\n");
	     kfprintf(file,  "\t{\n\t    kerror(NULL, \"%s_get_args\", sync_error_mesg);\n", oname);
	     kfprintf(file, "\t    kexit(KEXIT_FAILURE);\n\t}\n");

	     kfprintf(file, "\tif (!(kclui_get_int(pane, \"y\",\n");
             kfprintf(file,  "\t                &(clui_info->y_int),\n");
             kfprintf(file,  "\t                &(clui_info->y_flag))))\n");
	     kfprintf(file,  "\t{\n\t    kerror(NULL, \"%s_get_args\", sync_error_mesg);\n", oname);
	     kfprintf(file, "\t    kexit(KEXIT_FAILURE);\n\t}\n");
	}

        kfprintf(file, "\n}\n\n");

}

/*-----------------------------------------------------------
|
|       Routine: print_get_single_arg
|
|       Purpose: This routine prints the code for getting a single
|                required or optional argument from the command line
|
|         Input: file      - open stream to .c file
|                selection - selection representing argument
|                name      - name of the program being generated
|        Output: none
|
|    Written By: Danielle Argiro
|          Date: March 28, 1994
| Modifications: 
|
------------------------------------------------------------*/

static void print_get_single_arg(
   kobject      object,
   kfile       *file,
   kselection  *selection)
{
        unsigned long object_type;
	kobject       form_file;
	kstring       oname;
	kstring       variable;
	int           toggle_type = -1;


	if (!kcms_get_attributes(object,
				 KCMS_NAME,           &oname,
				 KCMS_CMOBJ_UIS_FORM, &form_file,
				 KCMS_CMOBJ_TYPE,     &object_type,
				 KCMS_END))
	   return;

	/* 
	 * see if it's a selection we need to generate code for,
         * ignore it if it's not.
	 */
	switch(selection->type) {
	    case KUIS_INPUTFILE:
	    case KUIS_OUTPUTFILE:
	    case KUIS_INTEGER:
	    case KUIS_FLOAT:
	    case KUIS_STRING:
	    case KUIS_LOGICAL:
	    case KUIS_CYCLE:
	    case KUIS_LIST:
	    case KUIS_FLAG:
	    case KUIS_STRINGLIST:
	    case KUIS_DOUBLE:
	    case KUIS_DISPLAYLIST:
	    case KUIS_TOGGLE:
	         variable = ktoken_to_string(selection->var_token);

	         if (((kstrcmp(variable, "x") == 0) ||
		      (kstrcmp(variable, "y") == 0)) && 
		      (selection->type == KUIS_INTEGER) &&
		      (object_type == KCMS_XVROUTINE) &&
		    form_file != NULL)
		     break;

		 if (selection->type == KUIS_TOGGLE)
		     kvf_get_attribute(selection->back_kformstruct,
                                   KVF_TOGGLE_TYPE, &toggle_type);
                 kfprintf(file, "\tif (!(");
	         print_kclui_get_call(object, file, selection, variable, 
				      toggle_type, "\t\t\t  ");
                 kfprintf(file, "))\n");
	         kfprintf(file, "\t{\n");
	         kfprintf(file, "\t    kerror(NULL, \"%s_get_args\", sync_error_mesg);\n", oname);
  	         kfprintf(file, "\t    kexit(KEXIT_FAILURE);\n");
	         kfprintf(file, "\t}\n");

		 if (selection->type == KUIS_TOGGLE)
		 {
		     kvf_get_attribute(selection->back_kformstruct, 
				       KVF_TOGGLE_TYPE, &toggle_type);
		     if ((toggle_type == KUIS_FLAG)    ||
		         (toggle_type == KUIS_LOGICAL) ||
		         (toggle_type == KUIS_INTEGER))
			kfprintf(file, "\telse\n\t{\n\t    clui_info->%s_toggle = atoi(value_string);\n\t    kfree_and_NULL(value_string);\n\t}\n", variable);
		     else if (toggle_type == KUIS_FLOAT)   
			kfprintf(file, "\telse clui_info->%s_toggle = (float) atof(value_string);\n\n", variable);
		     else if (toggle_type == KUIS_DOUBLE)   
			kfprintf(file, "\telse clui_info->%s_toggle = atof(value_string);\n\n", variable);
		 }
		 else kfprintf(file, "\n");
		 break;

	    default:
		 break;
	}
}


/*-----------------------------------------------------------
|
|  Routine Name: print_kclui_get_call
|
|       Purpose: This routine takes a UIS line and prints
|                the appropriate kclui_get_xxx routine to
|                get the value of the argument
|
|         Input: 
|        Output: none
|       Returns: none
|
|    Written By: Danielle Argiro
|          Date: March 28, 1994
| Modifications:
|
------------------------------------------------------------*/

static void print_kclui_get_call(
   kobject     object,
   kfile      *file,
   kselection *selection,
   char       *variable,
   int        toggle_type,
   char       *indent)
{
   unsigned long object_type;
   kobject       form_file    = NULL;


   if (!kcms_get_attributes(object,
			    KCMS_CMOBJ_TYPE,     &object_type,
			    KCMS_CMOBJ_UIS_FORM, &form_file,
			    KCMS_END))
      return;
	
    switch(selection->type)
    {
        case KUIS_INPUTFILE:
             kfprintf(file, "kclui_get_infile(pane, \"%s\",\n", variable);
             kfprintf(file,  "%s&(clui_info->%s_file),\n",
                      indent, variable);
             kfprintf(file,  "%s&(clui_info->%s_flag))",
                      indent, variable);
             break;

        case KUIS_OUTPUTFILE:
             kfprintf(file, "kclui_get_outfile(pane, \"%s\",\n", variable);
             kfprintf(file,  "%s&(clui_info->%s_file),\n",
                      indent, variable);
             kfprintf(file,  "%s&(clui_info->%s_flag))",
                      indent, variable);
             break;

        case KUIS_INTEGER:
	     if (((kstrcmp(variable, "x") == 0) ||
		  (kstrcmp(variable, "y") == 0)) && 
		  (object_type == KCMS_XVROUTINE) &&
		form_file != NULL)
		 break;
             kfprintf(file, "kclui_get_int(pane, \"%s\",\n", variable);
             kfprintf(file,  "%s&(clui_info->%s_int),\n",
                      indent, variable);
             kfprintf(file,  "%s&(clui_info->%s_flag))",
                      indent, variable);
             break;

        case KUIS_FLOAT:
             kfprintf(file, "kclui_get_float(pane, \"%s\",\n", variable);
             kfprintf(file,  "%s&(clui_info->%s_float),\n",
                      indent, variable);
             kfprintf(file,  "%s&(clui_info->%s_flag))",
                      indent, variable);
             break;


        case KUIS_DOUBLE:
             kfprintf(file, "kclui_get_double(pane, \"%s\",\n", variable);
             kfprintf(file,  "%s&(clui_info->%s_double),\n",
                      indent, variable);
             kfprintf(file,  "%s&(clui_info->%s_flag))",
                      indent, variable);
             break;

        case KUIS_STRING:
        case KUIS_STRINGLIST:
             kfprintf(file, "kclui_get_string(pane, \"%s\",\n", variable);
             kfprintf(file,  "%s&(clui_info->%s_string),\n",
                      indent, variable);
             kfprintf(file,  "%s&(clui_info->%s_flag))",
                      indent, variable);
             break;

        case KUIS_LOGICAL:
             kfprintf(file, "kclui_get_logical(pane, \"%s\",\n", variable);
             kfprintf(file,  "%s&(clui_info->%s_logic),\n",
                      indent, variable);
             kfprintf(file,  "%s&(clui_info->%s_flag))",
                      indent, variable);
             break;

        case KUIS_FLAG:
             kfprintf(file, "kclui_get_flag(pane, \"%s\",\n", variable);
             kfprintf(file,  "%s&(clui_info->%s_flag))",
                      indent, variable);
             break;

	case KUIS_TOGGLE:
             kfprintf(file, "kclui_get_value(pane, \"%s\", KVF_TOGGLE_VAL,\n",
                      variable);
	     if ((toggle_type == KUIS_LOGICAL) ||
		 (toggle_type == KUIS_FLAG)    ||
		 (toggle_type == KUIS_INTEGER) ||
		 (toggle_type == KUIS_FLOAT)   ||
		 (toggle_type == KUIS_DOUBLE))
                 kfprintf(file,  "%s&value_string,\n", indent);
	     else kfprintf(file,  "%s&(clui_info->%s_toggle),\n",
                      indent, variable);

             kfprintf(file,  "%s&(clui_info->%s_flag))",
                      indent, variable);
	     break;

	case KUIS_LIST:
	case KUIS_DISPLAYLIST:
	case KUIS_CYCLE:
             kfprintf(file, "kclui_get_value(pane, \"%s\", KVF_LIST_LABEL,\n", 
		      variable);
             kfprintf(file,  "%s&(clui_info->%s_label),\n",
                      indent, variable);
	     kfprintf(file,  "%s&(clui_info->%s_flag)) &&\n",
                      indent, variable);

	     kfprintf(file, "\t      kclui_get_value(pane, \"%s\", KVF_LIST_VAL,\n", 
                      variable);
	     
	     if (selection->type == KUIS_CYCLE)
                 kfprintf(file,  "%s&(clui_info->%s_cycle),\n",
                          indent, variable);
	     else kfprintf(file,  "%s&(clui_info->%s_list),\n",
                          indent, variable);

	     kfprintf(file,  "%s&(clui_info->%s_flag))",
                      indent, variable);

             break;

    }  /* end switch */
} /* end print_kclui_get_call */


/*-----------------------------------------------------------
|
|  Routine Name: number_toggle_found
|
|       Purpose:  All this rigamarole is to determine whether there
|                 is a float, integer, or double toggle in a selection
|                 list, so that we only declare value_string in the 
|                 generated get_args() code if we really need it, for toggle 
|                 selections having integer values.  heaven forbid we should 
|                 generate code that declares something unnecessarily, and
|                 makes lint complain!  ;-)
|
|         Input: sel_list - selection list of *.pane file
|        Output: none
|       Returns: TRUE if there is a number toggle in the selection list, 
|                FALSE otherwise.
|
|    Written By: Danielle Argiro
|          Date: April 29, 1994
| Modifications:
|
------------------------------------------------------------*/

static int number_toggle_found(
    kselection *sel_list)
{
	int        toggle_type;
	kselection *selection;
	
        selection = sel_list;
        while (selection != NULL)
        {
            if (selection->type == KUIS_TOGGLE)
            {
                kvf_get_attribute(selection->back_kformstruct,
                                  KVF_TOGGLE_TYPE, &toggle_type);
                if ((toggle_type == KUIS_LOGICAL) ||
                    (toggle_type == KUIS_FLAG)    ||
                    (toggle_type == KUIS_INTEGER) ||
                    (toggle_type == KUIS_FLOAT)   ||
                    (toggle_type == KUIS_DOUBLE))
		    return(TRUE);
            }
	    else if ((selection->type == KUIS_MUTEXCL) ||
		     (selection->type == KUIS_MUTINCL) ||
		     (selection->type == KUIS_GROUP))
	    {
		if (number_toggle_found(selection->group_next))
		    return(TRUE);
	    }
            selection = selection->next;
        }
	return(FALSE);
}

/*-----------------------------------------------------------
|
|       Routine: print_uis_spec
|
|       Purpose: The front end to print_init_uis_spec(),
|                this routine prints definitions and static
|                initializations for the *.pane file (and
|                *.form file if the software object is an xvroutine)
|
|         Input: program  - The software object being generated.
|                file     - open stream to usage .c file
|       Returns: TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: Nov 12, 1996
|
------------------------------------------------------------*/

static int print_uis_spec(
   kobject  program,
   kfile   *file)
{
	ktoken   filetoken;

        /*
         *  if there is a *.pane file, generate UIS structure for it
         */
	if (kcms_get_indirect_attribute(program, KCMS_CMOBJ_UIS_PANE, 
	       KCMS_FOBJ_FULLNAME_TKN, &filetoken))
	{
	    if (!print_init_uis_spec(ktoken_to_string(filetoken), file, "clui"))
	        return(FALSE);
	}

        /*
         *  if there is a *.form file, generate UIS structure for it
         */
	if (kcms_get_indirect_attribute(program, KCMS_CMOBJ_UIS_FORM, 
		KCMS_FOBJ_FULLNAME_TKN, &filetoken))
	{
	    if (!print_init_uis_spec(ktoken_to_string(filetoken), file, "gui"))
	        return(FALSE);
	}
	return(TRUE);

}

/*-----------------------------------------------------------
|
|       Routine: print_init_uis_spec
|
|       Purpose: This routine prints the definition and static
|                initialization of the UIS structure for a UIS file,
|                which will be a *.pane or *.form file.
|                This way the program can reference the
|                uis struct internally rather than depending on the
|                *.pane or *.form file themselves
|		 (faster, enables standalone binaries).
|
|         Input: program  - The software object being generated.
|                file     - open stream to usage .c file
|       Returns: TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: Nov 12, 1996
|
------------------------------------------------------------*/

static int print_init_uis_spec(
   char    *filepath,
   kfile   *file,
   char    *prefix)
{
	int        i;
	uis_struct *uis_spec;
	char       *tmp, *tmp2;
	kform      *form;

        /* create UIS structure */
        if ((uis_spec = kvf_read_uis_file(filepath)) == NULL)
            return(FALSE);

        /* create form tree, so that we get error checking of the UIS */
	form = kvf_build_form(uis_spec, KNONE, NULL, NULL);
	if (form == NULL) 
	{
	    kerror(NULL, "print_init_uis_spec", "Error in UIS specified by %s; "
	           "please correct errors and then generate code again.",
                    filepath); 
	    return(FALSE);
	}
	kvf_destroy_form(form);

        /*
         * write declaration for uis array
         */
        kfprintf(file, "/*\n * internal representation of %s UIS file\n */\n",
                 prefix);
        kfprintf(file, "char *%s_uis[] = {\n\n", prefix);
        for (i = 0; i < uis_spec->linenum; i++)
	{
	     /*
	      * if there are any \'s in the uis, need to leave them there,
	      * and not have kfprintf do away with them, so need to replace
	      * \ with \\ before calling kfprintf.
	      */
	     if (kstrchr(uis_spec->uis[i], '\'') != NULL)
                tmp = kstring_replace(uis_spec->uis[i], "\\", "\\\\", NULL);
	     else tmp = kstrdup(uis_spec->uis[i]);
             /*
              * also need to escape quotes
              */
             if (kstrchr(tmp, '\'') != NULL)
                tmp2 = kstring_replace(tmp, "\"", "\\\"", NULL);
             else tmp2 = kstrdup(tmp);

             kfprintf(file, "  \"%s\",\n", tmp2);
	     kfree(tmp);
	     kfree(tmp2);
	}
        kfprintf(file, "};\n\n");

        /*
         * write declaration for linenum_lookup array
         */
        kfprintf(file, "size_t %s_line_lookup[] = {", prefix);
        for (i = 0; i < uis_spec->linenum; i++)
	{
	     if ((i % 20) == 0) kfprintf(file, "\n  ");
             kfprintf(file, "%d,", uis_spec->line_lookup[i]);
	}
        kfprintf(file, "};\n\n");

        /*
         * write declaration for filename_lookup array
         */
        kfprintf(file, "size_t %s_file_lookup[] = {", prefix);
        for (i = 0; i < uis_spec->linenum; i++)
	{
	     if ((i % 20) == 0) kfprintf(file, "\n  ");
             kfprintf(file, "%d,", uis_spec->file_lookup[i]);
	}
        kfprintf(file, "};\n");

        /*
         * write declaration for uis filenames array
         */
        kfprintf(file, "/*\n * array of uis filenames and line numbers ");
        kfprintf(file, "for error messaging\n */\n");
        kfprintf(file, "char *%s_filenames[] = {\n", prefix);
        for (i = 0; i < uis_spec->filecount; i++)
             kfprintf(file, "  \"%s\",\n", uis_spec->filenames[i]);
        kfprintf(file, "};\n\n");


	/*
	 * declare UIS struct itself
	 */
	kfprintf(file, "/*\n * UIS struct for %s\n */\n", prefix);
	kfprintf(file, "uis_struct %s_uis_spec = {\n", prefix);
	kfprintf(file, "\t%s_uis,\n",         prefix);
	kfprintf(file, "\t%d,\n",             uis_spec->linenum);
        kfprintf(file, "\t%s_file_lookup,\n", prefix);
        kfprintf(file, "\t%s_line_lookup,\n", prefix);
	kfprintf(file, "\t%s_filenames,\n",   prefix);
	kfprintf(file, "\t%d,\n",             uis_spec->filecount);
        kfprintf(file, "\t%d};\n",            uis_spec->alloc_size);

	kvf_free_uis_struct(uis_spec);

        return(TRUE);
}

