/*
 * Khoros: $Id: clui_html.c,v 1.6.2.1 1997/03/06 23:18:41 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: clui_html.c,v 1.6.2.1 1997/03/06 23:18:41 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Routines to generate online html file (*.htm)
   >>>>
   >>>>  Private:
   >>>>                 kgen_clui_generate_htmlfile()
   >>>>   Static:
   >>>>                 print_html_req_args()
   >>>>                 print_html_opt_args()
   >>>>                 print_html_req_group_args()
   >>>>                 print_html_opt_group_args()
   >>>>                 print_html_single_arg()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static void print_html_req_args       PROTO((kfile *));
static void print_html_opt_args       PROTO((kfile *));
static void print_html_req_group_args PROTO((kfile *, kselection *, int));
static void print_html_opt_group_args PROTO((kfile *, kselection *, int));
static void print_html_single_arg     PROTO((kfile *, kselection *));



/*---------------------------------------------------------------
|
|  Routine Name: kgen_clui_generate_htmlfile
|
|       Purpose: Creates the html file 
|
|         Input: program   -  program object
|                prog_spec -  pointer to the internal PS struct
|                opath     -  path to program object
|                tbname   -  name of toolbox
|		 oname     -  name of program object being generated
|		 bname     -  name of the program binary being generated
|
|        Output: Returns TRUE on success, FALSE on failure
|    Written By: Kevin Sullivan, after Danielle Argiro
|          Date: Nov 22, 1996
| Modifications: 
|
--------------------------------------------------------------*/

int  kgen_clui_generate_htmlfile(
   kobject   program,
   char    **prog_spec,
   char     *opath,
   char     *tbname,
   char     *oname,
   char     *bname)
{
   kstring        routine          = "kgen_clui_generate_htmlfile()";
   kobject        file_object      = NULL;
   kobject        toolbox;
   kfile         *file;
   char          *filepath, *copyright; 
   char           temp[KLENGTH], directory[KLENGTH];
   kstring        short_description = NULL;
   int            force_flag;
   int            isnewfile         = FALSE;


   /*-- get file object representing html page -----------------------*/
   if (!kcms_get_attributes(program,
			    KCMS_PARENT,           &toolbox,
			    KCMS_CMOBJ_GEN_HTML,   &file_object,
			    KCMS_CMOBJ_SHORT_DESC, &short_description,
			    KCMS_END)
       || !kcms_query_bit(program, KCMS_CMOBJ_FLAGS, KCMS_BIT_CMOBJ_FORCE,
			  &force_flag))
      return FALSE;

   /*-------------------------------------------------------------------
   | The first time html page has been created - create file object,
   | so that kcms registers the html page in the kcms database
   -------------------------------------------------------------------*/
   if (file_object == NULL)
   {
      ksprintf(temp, "%s/html/%s.htm", opath, oname);
      file_object = kcms_create_fileobj(program, temp, NULL,
					KCMS_FOBJ_TYPE_HELP,
					KCMS_FOBJ_SUBTYPE_HTML,
					KCMS_FOBJ_GEN_HTML,
					KCMS_FOBJ_ACCESS_RDWR);
      if (file_object == NULL)
	 return FALSE;
      isnewfile = TRUE;
      filepath = kstrdup(temp);
   }

   /*
   * the html file has already been created & added to kcms database;
   * simply get the path to the html page.
   */
   else if (!kcms_get_attribute(file_object, KCMS_PATH, &filepath))
      return FALSE;

   /*
   *  if somehow the database GHOST_HTML key got deleted from the kcms
   *  database, but the html file really is there, don't want to 
   *  clobber it!
   */
   if ((isnewfile) && (kaccess(filepath, R_OK) == 0))
   {
      kerror(KCODEGEN, routine,
	     "WARNING!  CMS database appears to have been corrupted; "
	     "cms dbm key GHOST_HTML is missing, but the %s html file "
	     "does exist.  Man1 file will NOT be updated. Adding "
	     "GHOST_HTML cms key to database; next run of ghostwriter "
	     "should be OK.", oname);
      return FALSE;
   }

   /*
   * get the directory to the html file.  if the directory does
   * not exist, it will get created by kmake_dir();  if kmake_dir()
   * cannot create it, it will return the ENOENT error.
   */
   (void)kdirname(filepath, directory);
   if (!kmake_dir(directory, (mode_t) 0777) && errno != ENOENT)
   {
      kerror(KCODEGEN, routine,
	     "Can't create directory %s in which to deposit html file",
	     directory);
      return FALSE;
   }

   /*	
   *  see if html file already exists.  If so, prompt to over-write,
   *  return TRUE if the answer is NO
   */
   if (!force_flag && koverwrite(kget_notify(), filepath) == FALSE)
      return TRUE;

   /*-- create & open html file --------------------------------------*/
   if (kaccess(filepath, R_OK) != 0)
      isnewfile = TRUE;
   if ((file = kfopen(filepath, "w"))== NULL)
   {
      kerror(KCODEGEN, routine,
	     "Could not create file '%s'", filepath);
      return FALSE;
   }
	
   /*-- print the top of the html file ------------------------*/
   if (kstring_capitalize(oname, temp) == NULL)
      kstrcpy(temp, oname);
   kfprintf(file, "<HTML>\n");
   kfprintf(file, "<HEAD>\n");
   kfprintf(file, "     <TITLE> %s Online Help Page </TITLE>\n", temp);
   kfprintf(file, "</HEAD>\n");
   kfprintf(file, "<BODY>\n");
       
   kfprintf(file, "<H2>%s commands</H2><HR>\n", tbname);

   /*-- print out name and short description -------------------------*/

   kfprintf(file, "<H3>PROGRAM NAME </H3>\n");
   kfprintf(file, "%s", bname);
   if (short_description != NULL)
      kfprintf(file, " - %s \n", short_description);


   /*-- print out description ----------------------------------------*/
   kfprintf(file, "<H3> DESCRIPTION </H3>\n");
   if (kstrlen(prog_spec[MAN1_LONGDESC]) > 0)
      kgen_roff2html(file, prog_spec[MAN1_LONGDESC]);

   /*-- section for required arguments --------------------------------*/
   kfprintf(file, "<H3> REQUIRED ARGUMENTS </H3>\n");
   if (kgen_req_num > 0)
      print_html_req_args(file);
   else
      kfprintf(file, "none\n<BR>\n");

   /*-- section for optional arguments --------------------------------*/
   kfprintf(file, "<H3> OPTIONAL ARGUMENTS </H3>\n");
   if (kgen_opt_num > 0)
      print_html_opt_args(file);
   else
      kfprintf(file, "none\n<BR>\n");
	
   /*-- EXAMPLES section ----------------------------------------------*/
   kfprintf(file, "<H3> EXAMPLES </H3>\n");
   if (kstrlen(prog_spec[MAN1_EXAMPLES]) > 0) 
   {
      if (kstrstr(prog_spec[MAN1_EXAMPLES], ".begin code") == NULL)
      {
         kfprintf(file, "<PRE>\n");
         kgen_roff2html(file, prog_spec[MAN1_EXAMPLES]);
         kfprintf(file, "</PRE>\n");
      }
      else
      {
         kgen_roff2html(file, prog_spec[MAN1_EXAMPLES]);
      }
   }
	
   /*-- SEE ALSO section of the man page ------------------------------*/
   kfprintf(file, "<H3> SEE ALSO </H3>\n");
   if (kstrlen(prog_spec[MAN1_SEEALSO]) > 0) 
      kgen_roff2html(file, prog_spec[MAN1_SEEALSO]);
	
   /*-- RESTRICTIONS section of the man page --------------------------*/
   kfprintf(file, "<H3> RESTRICTIONS </H3>\n");
   if (kstrlen(prog_spec[MAN1_RESTRICTIONS]) > 0) 
      kgen_roff2html(file, prog_spec[MAN1_RESTRICTIONS]);
	
   /*-- REFERENCES section of the man page ----------------------------*/
   kfprintf(file, "<H3> REFERENCES </H3>\n");
   if (kstrlen(prog_spec[MAN1_REFERENCES]) > 0) 
      kgen_roff2html(file, prog_spec[MAN1_REFERENCES]);

   /*-- COPYRIGHT section ---------------------------------------------*/
   kfprintf (file, "<H3> COPYRIGHT </H3>\n");
   copyright = kcms_get_copyright(toolbox, KCMS_TB_COPYRIGHT_SHORT,
				  KCMS_LANG_NONE);
   if (copyright != NULL)
      kfprintf(file, "%s\n", copyright);
   kfree_and_NULL(copyright);
	
   /*-- that's the end... --------------------------------------------*/
   kfprintf(file, "</BODY>\n");
   kfprintf(file, "</HTML>\n");

   kfclose(file);

   return TRUE;
}


/*-----------------------------------------------------------
|
|  Routine Name: print_html_req_args
|
|       Purpose: Prints the required arguments to the html file 
|
|         Input: file     - open stream to html page
|
| 	 Output: none
|    Written By: Kevin Sullivan, after Danielle Argiro
|          Date: Nov 22, 1996
| Modifications: Converted from Khoros 1.0 (DA)
|
--------------------------------------------------------------*/
static void
print_html_req_args(
   kfile *file)
{
   int         indx;
   kselection *selection;

   kfprintf(file, "<DL>\n");
   indx = 0;
   while (indx < kgen_req_num)
   {
      selection = kgen_req_sels[indx];

      if ((selection->type == KUIS_MUTEXCL) ||
	  (selection->type == KUIS_MUTINCL) ||
	  (selection->type == KUIS_GROUP))
      {
	 print_html_req_group_args(file, selection, FALSE);
      }
      else
	 print_html_single_arg(file, selection);

      indx++;
   }
   kfprintf(file, "</DL>\n");
}

/*-----------------------------------------------------------
|
|  Routine Name: print_html_opt_args
|
|       Purpose: Prints the optional arguments to the html file 
|
|         Input: file     - open stream to html page
|
|        Output: none
|    Written By: Kevin Sullivan, after Danielle Argiro
|          Date: Nov 22, 1996
| Modifications: Converted from Khoros 1.0 (DA)
|
--------------------------------------------------------------*/
static void
print_html_opt_args(
   kfile *file)
{
   int         indx;
   kselection *selection;

   kfprintf(file, "<DL>\n");
   indx = 0;
   while (indx < kgen_opt_num)
   {
      selection = kgen_opt_sels[indx];

      if ((selection->type == KUIS_MUTEXCL) ||
	  (selection->type == KUIS_MUTINCL) ||
	  (selection->type == KUIS_GROUP))
	 print_html_opt_group_args(file, selection, FALSE);
      else
	 print_html_single_arg(file, selection);

      indx++;
   }
   kfprintf(file, "</DL>\n");
}

/*-----------------------------------------------------------
|
|  Routine Name: print_html_req_group_args
|
|       Purpose: Prints the required ME/MI group arguments to the html file
|
|         Input: file        - open stream to html page
|                group_start - beginning of group list (-C or -B line)
|                nested      - TRUE if nested group
|        Output: none
|    Written By: Kevin Sullivan, after Danielle Argiro
|          Date: Nov 22, 1996
| Modifications: 
|
--------------------------------------------------------------*/

static void print_html_req_group_args(
    kfile      *file,
    kselection *group_start,
    int        nested)
{
   int          optional;
   kselection  *groupmember;

   kfprintf(file, "<DT><P>");
   if (!nested)
   {
      if (group_start->type == KUIS_MUTEXCL)
	 kfprintf(file, 
		  "Mutually Exclusive Group; you <EM>must</EM> specify <B>ONE</B> of:\n");
      else if (group_start->type == KUIS_MUTINCL)
	 kfprintf(file, 
		  "Mutually Inclusive Group; you <EM>must</EM> specify <B>ALL</B> of:\n");
      else
	 kfprintf(file, 
		  "Group; you <EM>must</EM> specify <B>AT LEAST ONE</B> of:\n");
   }
   else
   {
      if (group_start->type == KUIS_MUTEXCL)
	 kfprintf(file, " <B>ONE OF</B> the Mutually Exclusive Group:\n");
      else if (group_start->type == KUIS_MUTINCL)
	 kfprintf(file, " <B>ALL OF</B> the Mutually Inclusive Group:\n");
      else
	 kfprintf(file, " <B>AT LEAST ONE OF</B> the Group:\n");
   }

   kfprintf(file, "<DD><DL>\n");
   groupmember = group_start->group_next;
   while (groupmember != NULL)
   {
      if (groupmember->type == KUIS_BLANK)
      {
	 groupmember = groupmember->next;
	 continue;
      }
      if (groupmember->group_next == NULL)
      {
	 print_html_single_arg(file, groupmember);

	 if (groupmember->next != NULL) 
	 {
	       if (group_start->type == KUIS_MUTEXCL)
		  kfprintf(file, "<BLOCKQUOTE> OR </BLOCKQUOTE>\n");
	       else if (group_start->type == KUIS_MUTINCL)
		  kfprintf(file, "<BLOCKQUOTE> AND </BLOCKQUOTE>\n");
	       else
		  kfprintf(file, "<BLOCKQUOTE> AND/OR </BLOCKQUOTE>\n");
	 }
      }
      else
      {
	 kvf_get_attribute(group_start->back_kformstruct,
			   KVF_OPTIONAL, &optional);
	 if (!optional)
	    print_html_req_group_args(file, groupmember, TRUE);
	 else
	    print_html_opt_group_args(file, groupmember, TRUE);
      }
      groupmember = groupmember->next;
   }
   kfprintf(file, "</DL><P>\n");
}

/*-----------------------------------------------------------
|
|  Routine Name: print_html_opt_group_args
|
|       Purpose: Prints the optional ME/MI group arguments to the html file
|
|         Input: file        - open stream to html page
|                group_start - beginning of group list (-C or -B line)
|                nested      - TRUE if nested group
|        Output: none
|    Written By: Kevin Sullivan, after Danielle Argiro
|          Date: Nov 22, 1996
| Modifications: 
|
--------------------------------------------------------------*/
static void
print_html_opt_group_args(
    kfile      *file,
    kselection *group_start,
    int        nested)
{
   kselection *groupmember;
   char       *group_def_var = NULL, *group_def_val = NULL;
   int        optional, optsel;

   kfprintf(file, "<DT><P>");
   if (!nested)
   {
      if (group_start->type == KUIS_MUTEXCL)
	 kfprintf(file,
		  "Mutually Exclusive Group; <EM>if desired</EM>, specify <B>ONE</B> of:\n");
      else if (group_start->type == KUIS_MUTINCL)
	 kfprintf(file, 
		  "Mutually Inclusive Group; <EM>if desired</EM>, specify <B>ALL</B> of:\n");
      else
	 kfprintf(file,
		  "Group; specify <B>AT LEAST ONE</B> of: \n");
   }
   else
   {
      if (group_start->type == KUIS_MUTEXCL)
	 kfprintf(file, 
		  " <B>ONE OF</B> the Mutually Exclusive Group:\n");
      else if (group_start->type == KUIS_MUTINCL)
	 kfprintf(file, " <B>ALL OF</B> the Mutually Inclusive Group:\n");
      else
	 kfprintf(file, " <B>AT LEAST ONE OF</B> the Group:\n");
   }

   kfprintf(file, "<DD><DL>\n");
   groupmember = group_start->group_next;
   while (groupmember != NULL)
   {
      if (groupmember->type == KUIS_BLANK)
      {
	 groupmember = groupmember->next;
	 continue;
      }
      if (groupmember->group_next == NULL)
      {
	 print_html_single_arg(file, groupmember);
	 kvf_get_attribute(groupmember->back_kformstruct, KVF_OPTSEL, &optsel);
	 if (optsel)
	 {
	    if ((groupmember->type != KUIS_INPUTFILE)
		&& (groupmember->type != KUIS_OUTPUTFILE))
	       group_def_val = kgen_ascii_def_val(groupmember);
	    else
	       group_def_val = kstrdup(" ");
	 }

	 if (groupmember->next != NULL)
	 {
	       if (group_start->type == KUIS_MUTEXCL)
		  kfprintf(file, "<BLOCKQUOTE> OR </BLOCKQUOTE>\n");
	       else if (group_start->type == KUIS_MUTINCL)
		  kfprintf(file, "<BLOCKQUOTE> AND </BLOCKQUOTE>\n");
	       else
		  kfprintf(file, "<BLOCKQUOTE> AND/OR </BLOCKQUOTE>\n");
	 }
      }
      else
      {
	 kvf_get_attribute(group_start->back_kformstruct,
			   KVF_OPTIONAL, &optional);
	 if (!optional)
	    print_html_req_group_args(file, groupmember, TRUE);
	 else
	    print_html_opt_group_args(file, groupmember, TRUE);
      }

      if ((group_start->type == KUIS_MUTEXCL) && 
	  (group_def_var != NULL) &&
	  (group_def_val != NULL))
      {
	 kfprintf(file, "<BR>\n");
	 kfprintf(file, "Default argument of");
	 kfprintf(file, " mutually exclusive group:\n");
	 kfprintf(file, "<BR>\n");
	 kfprintf(file, "-%s %s\n", group_def_var, group_def_val);
      }
      groupmember = groupmember->next;
   }
   kfprintf(file, "</DL><P>\n");
}


/*-----------------------------------------------------------
|
|  Routine Name: print_html_single_arg
|
|       Purpose: Prints info on a single argument to the html file
|
|         Input: file      - open stream to html page
|                selection - selection corresponding to argument
|        Output: none
|    Written By: Kevin Sullivan, after Danielle Argiro
|          Date: Nov 22, 1996
| Modifications:
|
--------------------------------------------------------------*/

static void print_html_single_arg(
    kfile      *file,
    kselection *selection)
{
	char   *variable, *description = NULL;
	char   *ascii_default; 
	char   typestring[KLENGTH];
	int    optional, toggle_type;

	kvf_get_attribute(selection->back_kformstruct, 
		          KVF_OPTIONAL, &optional);

	if (selection->type == KUIS_STDIN)
	    description = kstrdup("standard input");
	else if (selection->type == KUIS_STDOUT)
	    description = kstrdup("standard output");
	else kvf_get_attribute(selection->back_kformstruct,
			       KVF_DESCRIPTION, &description);
       	
	if (description == NULL) 
	    description = kstrdup("no description");
	variable = ktoken_to_string(selection->var_token);
	kfprintf(file, "<DT> <B>-%s</B>\n<DD>\n", variable);
	
	if (selection->type == KUIS_TOGGLE)
        {
                kvf_get_attribute(selection->back_kformstruct,
                                  KVF_TOGGLE_TYPE, &toggle_type);
                if (toggle_type == KUIS_LOGICAL || toggle_type == KUIS_FLAG)
		    ksprintf(typestring, "integer toggle");
                else ksprintf(typestring, "%s toggle", 
			      kgen_ascii_datatype(toggle_type));
        }
        else ksprintf(typestring, "%s", kgen_ascii_datatype(selection->type));

	kfprintf(file, "type: %s\n<BR>\n", typestring);
	kfprintf(file, "desc: ");
	if (description != NULL)
	    kfprintf(file, "%s\n<BR>\n", description);
	else kfprintf(file, "\n<BR>\n");

	if ((selection->type != KUIS_FLAG &&
             selection->type != KUIS_STDIN &&
	     selection->type != KUIS_STDOUT) && (optional))
	{
	    ascii_default = kgen_ascii_def_val(selection);
	    kfprintf(file, "default: %s\n<BR>\n", ascii_default);
	    kfree_and_NULL(ascii_default);
	}
	
	if ((selection->type == KUIS_INTEGER) ||
	    (selection->type == KUIS_FLOAT)   ||
	    (selection->type == KUIS_DOUBLE))
            kgen_clui_print_bounds(file, selection, KGEN_CLUI_HTML_SYNTAX);

	else if (selection->type == KUIS_TOGGLE)
        {
	    kfprintf(file, "allowed values: \n<BR>\n");
            kgen_clui_print_toggle_vals(file, selection,
                                        KGEN_CLUI_HTML_SYNTAX, FALSE);
        }
	/*-- kvf_get_attributes kstrdups this value, so free it --*/
	kfree(description);
}
