/*
 * Khoros: $Id
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: clui_libpixmap.c,v 1.6.2.3 1997/05/04 16:15:02 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>	 Generate Pixmap Initialization File for Libraries
   >>>>		
   >>>>   Static:
   >>>>                 print_cache_pixmaps_routine()
   >>>>   Public:
   >>>>                 kgen_generate_libpixmap()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static int print_cache_pixmaps_routine PROTO((kobject, kfile *));

/************************************************************
*
*  Routine Name: kgen_generate_libpixmap - 
*			generate routine to initialize pixmaps for libraries
*
*       Purpose: This routine, used only for libraries, generates 
*                a file called "pixmaps.c" containing one routine:
*
*		 void xxx_cache_pixmaps(void)
*
*
*		 This routine is used to cache all the pixmaps that
*		 are used in the library.  Caching of pixmaps is required
*		 so that pixmap filenames are compiled into the library 
*		 archive. Being compiled into the library archive means that
*                pixmaps ill not hinder the creation of standalone applications
*                (you can't have a standalone application depend on a
*                library that needs to open a pixmap file at runtime,
*                as you won't have the pixmap file when you need it
*                if the library wasn't distributed with the application).
*
*         Input: object  - the library object for which to generate pixmap.c 
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*    Written By: Danielle Argiro & Steven Jorgensen
*
*          Date: Jan 9, 1997
*
*************************************************************/
int 
kgen_generate_libpixmap(
	kobject	 object)

{
        int           force_flag  = FALSE;
        int           isnewfile   = FALSE;
        kobject       file_object = NULL;
        char         *filepath    = NULL;
        kobject       toolbox;
        kfile        *file;
        unsigned long lang_type;
        long          sub_type = KCMS_FOBJ_SUBTYPE_C;
        char         *copyright;
	char         *opath;
        char          path[KLENGTH];
        char          pixmaps[KLENGTH];
        char          temp[KLENGTH];
        kstring       rcsid_info  = NULL;
        kstring       rcslog_info = NULL;
        kstring       routine     = "kgen_clui_generate_getargs_file()";

        /*
         * get file object representing "pixmaps.c" file
         */
        if (!kcms_get_attributes(object,
                                 KCMS_PARENT,             &toolbox,
                                 KCMS_CMOBJ_GEN_PIXMAP,   &file_object,
                                 KCMS_CMOBJ_LANGTYPE,     &lang_type,
                                 KCMS_PATH,               &opath,
                                 KCMS_END)
            || !kcms_query_bit(object, KCMS_CMOBJ_FLAGS, KCMS_BIT_CMOBJ_FORCE,
                               &force_flag))
            return(FALSE);

        /*
         * 1st time pixmaps.c file has been created - create file object,
         * so that kcms registers the pixmaps.c file in the kcms database
         */
        if (file_object == NULL)
        {
            if (lang_type == KCMS_LANG_CPLUSPLUS)
            {
                ksprintf(temp, "%s/src/pixmaps.C", opath);
                sub_type = KCMS_FOBJ_SUBTYPE_CPLUSPLUS;
            }
            else
                ksprintf(temp, "%s/src/pixmaps.c", opath);

            file_object = kcms_create_fileobj(object, temp, NULL,
                                              KCMS_FOBJ_TYPE_SRC,
                                              sub_type,
                                              KCMS_FOBJ_GEN_GWFILE,
                                              KCMS_FOBJ_ACCESS_RDONLY);
            if (file_object == NULL)
                return(FALSE);
            isnewfile = TRUE;
            filepath = temp;
        }

        /*
         * the pixmaps.c file has already been created & added to 
	 * kcms database; simply get the path to the help page.
         */
        else if (!kcms_get_attribute(file_object, KCMS_PATH, &filepath))
            return(FALSE);

        /*
         *  see if pixmaps.c file already exists.  If so, prompt to over-write,
         *  return TRUE if the answer is NO
         */
        if(kdirname(filepath, path) == NULL)
            ksprintf(path, ".");

        if (lang_type == KCMS_LANG_CPLUSPLUS)
           ksprintf(pixmaps, "%s/pixmaps.C", path);
        else
           ksprintf(pixmaps, "%s/pixmaps.c", path);
        if (kaccess(pixmaps, R_OK) != 0)
           isnewfile = TRUE;

        if (!force_flag && koverwrite(kget_notify(), pixmaps) == FALSE)
           return(TRUE);

        /*
         * Save RCS information
         */
        if (isnewfile == FALSE &&
            kgen_rcs_save(pixmaps, &rcsid_info, &rcslog_info) == FALSE)
        {
            kerror(KCODEGEN, routine, "Could not save RCS headers for "
                   "%s. Aborting..", pixmaps);
            return(FALSE);
        }
        /*
         *  create & open pixmaps.c file
         */
        if ((file = kfopen(pixmaps, "w"))== NULL)
        {
            kerror(KCODEGEN, routine,
                   "Could not create file '%s'", pixmaps);
            kfree_and_NULL(rcsid_info);
            kfree_and_NULL(rcslog_info);
            return(FALSE);
        }

        /*
         * begin pixmaps.c file with RCS header
         */
        kgen_rcs_chdr(file, rcsid_info, rcslog_info);
        kfree_and_NULL(rcsid_info);
        kfree_and_NULL(rcslog_info);

        /*
         * follow RCS header with copyright
         */
        copyright = kcms_get_copyright(toolbox, KCMS_TB_COPYRIGHT_LONG,
                                       KCMS_LANG_C);

        if (copyright != NULL)
            kfprintf(file, "%s\n", copyright);
        kfree_and_NULL(copyright);


	/*
         *  generate xxx_cache_pixmaps() routine
         */
        print_cache_pixmaps_routine(object, file);
        kfprintf(file, "\n\n");

        /*
         *  close file and finish
         */
        kfclose(file);

        kinfo(KHOSTILE, "for all the good it will do you.");
        return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: print_cache_pixmaps_routine
|
|       Purpose: This routine prints the xxx_cache_pixmaps() routine 
|                for the library in file pixmaps.c
|
|         Input: object - library software object being generated.
|                file   - open stream to pixmaps.c file
|    Written By: Danielle Argiro
|          Date: January 9, 1997
| Modifications:
|
------------------------------------------------------------*/

static int print_cache_pixmaps_routine(
   kobject  object,
   kfile   *file)
{
	kstring  oname;
	kstring  basename;
	kstring  fullname;
	klist    *pixmaplist, *listptr;
	kobject  fobject;
	char     temp[KLENGTH];
	char     purpose[KLENGTH];
	char     pixmap_name[KLENGTH];
	char     new_name[KLENGTH];
	char     *routine_name = "print_cache_pixmaps_routine";
	kfile    *pixmapfile;
	int      i, count = 0;
	char     *file_buffer, *substr;
	char     **pixmap_names = NULL;
	char     **pixmap_filenames = NULL;


        if (!kcms_get_attributes(object,
                                 KCMS_CMOBJ_ONAME,   &oname,
                                 KCMS_CMOBJ_PIXMAPS, &pixmaplist,
                                 KCMS_END))
           return(FALSE);

	kfprintf(file, "#include \"internals.h\"\n\n");

	/*
  	 *  This loop does two things, for each pixmap file:
         *
         *  1) it sucks in the pixmap file and spits it out 
         *     verbetim into the pixmaps.c file,
         *
	 *  2) it saves the pixmap filepath and the pixmap name itself 
         *     in matching string arrays for use later when generating 
	 *     calls to xvw_cache_pixmap  
 	 */
	listptr = klist_head(pixmaplist);
	while (listptr != NULL)
	{
	   /* get the basename & fullname from pixmap file object */
	   fobject = (kobject)klist_clientdata(listptr);
	   if (!kcms_get_attributes(fobject,
				    KCMS_FOBJ_BASENAME, &basename,
				    KCMS_FOBJ_FULLNAME, &fullname,
				    KCMS_END))
		return(FALSE);

	   /* open pixmap file just to be sure its readable before continuing */
	   if ((pixmapfile = kfopen(fullname, "r")) == NULL)
	   {
		kerror(KCODEGEN, routine_name, 
		      "Warning: can't open pixmap file %s. Omitting pixmap.",
		       fullname);
	   }
	   else
	   {
		/* close file, because kreadfile wants to open it */
	        kfclose(pixmapfile);

	        /* read pixmap def from pixmap file into file_buffer */
		file_buffer = (char *)kreadfile(fullname, NULL);

		/* extract pixmap name from the pixmap def in file_buffer */
                substr = kstrstr(file_buffer, "static char *");
                ksscanf(substr, "static char *%[^[]", pixmap_name);

	        /* now, construct new name that is guaranteed to be unique */
		ksprintf(new_name, "%s_%d", basename, count);
	        kstring_replace(new_name, ".", "_", new_name);

		/* write pixmap def with substituted name into pixmaps.c */
		pixmapfile = kfopen(fullname, "r");
	        kfile_seddata(kfileno(pixmapfile), kfileno(file),
			      pixmap_name, new_name, FALSE,
			      "\"\\s*,?\\s*};", "\",\nNULL};", TRUE,
			      NULL);
		kfclose(pixmapfile);
	        kfree(file_buffer);

		/* save new pixmap name & fullpath in arrays for later use */
		pixmap_names = (char **)karray_add(pixmap_names, KSTRING, 
					  kstrdup(new_name), count);
		pixmap_filenames = (char **)karray_add(pixmap_filenames,
					  KSTRING, kstrdup(fullname), count++);
	   }
	   kfprintf(file, "\n\n");
	
	   listptr = klist_next(listptr);
	}
	kfprintf(file, "\n");

        /*
         *   put in header for xxx_cache_pixmaps() routine
         */
        ksprintf(routine_name, "%s_cache_pixmaps", oname);
        ksprintf(purpose, "Caches all pixmaps for the %s library", oname);
        ksprintf(temp, "ghostwriter -oname %s", oname);
        kgen_privsrc_hdr(file, routine_name, purpose,
                         "None", "None", temp, NULL);

        /*
         *   put in beginning of kgen_get_args routine
         */
        kfprintf(file, "void %s_cache_pixmaps(void)\n{\n\n", oname);

	kfprintf(file, "    static int initialized = FALSE;\n\n");
	kfprintf(file, "    /* Only need to initialize pixmaps once... */\n");
	kfprintf(file, "    if (initialized == TRUE)\n\t    return;\n\n");
	kfprintf(file, "    initialized = TRUE;\n\n");

	kfprintf(file, "    /*\n");
	kfprintf(file, "     * and now, a call to xvw_cache_pixmap()\n");
	kfprintf(file, "     * for each pixmap in the pixmaps/ directory...\n");
	kfprintf(file, "     */\n");

	/*
  	 *  a loop to print out a call to xvw_cache_pixmap for every
 	 *  pixmap in the misc/pixmaps directory: something like:
	 *
	 * xvw_cache_pixmap("$TB/objects/library/xvobjects/pixmaps/info.xpm",
         *                  info_xpm_5);
 	 */
	listptr = klist_head(pixmaplist);
	for (i = 0; i < count; i++)
        {
            kfprintf(file, "    xvw_cache_pixmap(\"%s\",\n", 
		     pixmap_filenames[i]);
	    kfprintf(file, "                     %s);\n", 
		     pixmap_names[i]);
        }
        kfprintf(file, "\n");

	kfprintf(file, "}\n");
	karray_free(pixmap_names, KSTRING, count, NULL);
	karray_free(pixmap_filenames, KSTRING, count, NULL);
	return(TRUE);

	
}
