/*
 * Khoros: $Id: clui_library.c,v 1.12.2.1 1997/03/06 21:10:40 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: clui_library.c,v 1.12.2.1 1997/03/06 21:10:40 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Routines to Generate library headers
   >>>>
   >>>>  Private:
   >>>>                 kgen_clui_generate_library()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


/*---------------------------------------------------------------
|  Routine Name: kgen_clui_generate_library
|
|       Purpose: Creates a library object 
|
|         Input: library   -  pointer to the library object
|
|        Output: Returns TRUE on success, FALSE on failure
|    Written By: Danielle Argiro & Steve Jorgensen
|          Date: Jan 14, 1993 14:58
--------------------------------------------------------------*/
int
kgen_clui_generate_library(
   kobject library)
{
   kstring       routine = "kgen_clui_generate_library()";
   header_info  *header;
   kstring       path;
   kstring       oname;
   unsigned long lang_type;
   klist         *pixmaplist;
   char          install_manpath[KLENGTH];
   char          install_htmlpath[KLENGTH];
   kobject       file_object;
   char          postfix;
   char          mesg[KLENGTH];

   if (!kcms_get_attributes(library,
			    KCMS_PATH, &path,
			    KCMS_NAME, &oname,
			    KCMS_CMOBJ_LANGTYPE, &lang_type,
			    KCMS_END))
      return FALSE;

   ksprintf(install_manpath,"%s/man", path);
   if (kmkdir(install_manpath, (mode_t) 0777) != 0 && errno != EEXIST)
   {
      kerror(KCODEGEN, routine, "Cannot create man install directory '%s'\n",
	     library);
      return FALSE;
   }

   ksprintf(install_htmlpath,"%s/html", path);
   if (kmkdir(install_htmlpath, (mode_t) 0777) != 0 && errno != EEXIST)
   {
      kerror(KCODEGEN, routine, "Cannot create html install directory '%s'\n",
	     install_htmlpath);
      return FALSE;
   }


   /*
    *  see if the library has pixmaps that require a pixmaps.c file
    *  to be generated
    */
   if (!kcms_get_attribute(library, KCMS_CMOBJ_PIXMAPS, &pixmaplist))
      return(FALSE);
 
   /*
    *  there are pixmaps - generate "pixmaps.c" file
    */
   if (pixmaplist != NULL)
   {
	if (lang_type == KCMS_LANG_CPLUSPLUS)
	    postfix = 'C';
	else postfix = 'c';

	ksprintf(mesg, "Generating %s/src/pixmaps.%c", path, postfix);
	if (kgen_generate_libpixmap(library))
           kannounce(KCODEGEN, routine, "%s... ok", mesg);
	else kannounce(KCODEGEN, routine, "%s... Error!", mesg);
   }

   /*
    *  if there are no pixmaps, but a pixmaps.c file does exist, 
    *  remove it.
    */
   else if (pixmaplist == NULL)
   {
	kcms_get_attribute(library, KCMS_CMOBJ_GEN_PIXMAP, &file_object);
	if (file_object != NULL)
	   kcms_destroy(file_object);
   }

   kannounce(KCODEGEN, routine, 
             "Generating man pages and html pages for functions:");

   while (kcms_get_attribute(library, KCMS_CMOBJ_HEADERINFO, &header))
   {
      if (!kgen_generate_manpage(library, header, install_manpath))
	 kannounce(KCODEGEN, routine,
		   "\t\t '%s' man page... Error!", header->name);
      else kannounce(KCODEGEN, routine,
		   "\t\t '%s' man page... ok", header->name);

      if (!kgen_generate_htmlpage(library, header, install_htmlpath))
	 kannounce(KCODEGEN, routine,
		   "\t\t '%s' html page... Error!", header->name);
      else kannounce(KCODEGEN, routine,
		   "\t\t '%s' html page... ok", header->name);
   }

   return TRUE;
}
