/*
  * Khoros: $Id: eval.c,v 1.7.2.4 1997/05/28 18:19:00 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: eval.c,v 1.7.2.4 1997/05/28 18:19:00 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Private Khoros Expression Evaluation Routines
   >>>>		
   >>>>  Private:
   >>>>                 kexpr_eval_symbol()
   >>>>                 kexpr_eval_symlist()
   >>>>   Static:
   >>>>                 eval_function()
   >>>>                 eval_ufunction()
   >>>>                 eval_operator()
   >>>>                 eval_instruction()
   >>>>                 eval_symlist()
   >>>>                 get_numeric_value(Symbol *)
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include "y.tab.h"


static SymbolList *eval_symlist PROTO((SymbolList *, int, Symbol *result));
static double get_numeric_value(Symbol *sym);


/*-----------------------------------------------------------
|
|  Routine Name: eval_function - evaluate a function
|
|       Purpose: eval_function() is called when we want to evaluate
|                a function.
|
|        Input:  symbol   - the function symbol
|		 symlist  - the argument symbols for the function
|		 abort_when_undefined - whether to abort when an undefined
|					variable is found.
|
|        Output: symbol   - the result stored in the "result" symbol
|		 RETURNS: the symbol list on success, NULL otherwise
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
| Modifications:
|    PH (04/01/97)  Argument type checking
|                   Put all function pointers in a union
|                   Handle STRING return types
|
------------------------------------------------------------*/

static SymbolList *eval_function(
   Symbol     *symbol,
   SymbolList *symlist,
   int        abort_when_undefined,
   Symbol     *result)
{
	Symbol   arg;
	double   vals[20];
	int	 i, num_args;
	register double	value;
	struct _function *function_def;
	int      have_error = FALSE;
	union {
	   double   (*function0)(void);
	   double   (*function1)(double);
	   double   (*function2)(double, double);
	   double   (*function3)(double, double, double);
	   double   (*function4)(double, double, double, double);
	   double   (*function5)(double, double, double, double, double);
	   double   (*function6)(double, double, double, double, double,
				       double);
	   double   (*function7)(double, double, double, double, double,
				       double, double);
	   double   (*function8)(double, double, double, double, double,
				       double, double, double);
	   double   (*function9)(double, double, double, double, double,
				       double, double, double, double);
	   char*    (*sfunction1)(char *);
	   char*    (*sfunction2)(char *, char *);
	} functions;

	kexpr_init_symbol(&arg);

	/*
	 *  Evaluate the different argument expressions and then call the
	 *  function with the different evaluated arguments.
	 */
	num_args = symbol->num;
	function_def = symbol->Function;
	functions.function0 = *(function_def->routine);
	if (result == NULL)
	{
	   for (i = 0; i < num_args; i++)
	      symlist = eval_symlist(symlist, abort_when_undefined, NULL);

	   return(symlist);
	}

	for (i = 0; i < num_args; i++)
	{
	   symlist = eval_symlist(symlist, abort_when_undefined, &arg);
	   vals[i] = arg.Value;
	   if (arg.type != function_def->arg_types[i])
	   {
	      char ebuf[80];
	      if (function_def->arg_types[i] == NUMBER  &&  arg.type == STRING)
	      {
		 if (ksscanf(STRING_VALUE(&arg), "%lf", vals+i) == 1)
		 {
		    /* quietly cast argument from string to number */
		    RESET_SYMBOL(&arg);
		    continue;
		 }
	      }
	      ksprintf(ebuf, "The data type of argument %d is %s "
		       "but should be %s",
		       i+1,
		       expr_type_name(arg.type),
		       expr_type_name(function_def->arg_types[i]));
	      kexpr_error(ebuf);
	      have_error = TRUE;
	      RESET_SYMBOL(&arg);
	   }
	}

	if (have_error)
	{
	   result->type = function_def->return_type;
	   if (result->type == STRING)
	      ASSIGN_CSTRING(result, "ERROR");
	   else
	      result->Value = 0;
	   RESET_SYMBOL(&arg);
	   return symlist;
	}
	
	switch (num_args)
	{
	   case 0:
		value = functions.function0();
		break;

	   case 1:
	        if (symbol->exprType == STRING)
		{
		   char *tstr;
		   /* result->String = functions.sfunction1(STRING_VALUE(&arg)); */
		   tstr = functions.sfunction1(STRING_VALUE(&arg));
		   ASSIGN_CSTRING(result, tstr);
		   kfree(tstr);
		   result->type = STRING;
		   RESET_SYMBOL(&arg);
		   return symlist;
		}
		   
		value = functions.function1(vals[0]);
		break;

	   case 2:
		value = functions.function2(vals[0], vals[1]);
		break;

	   case 3:
		value = functions.function3(vals[0], vals[1], vals[2]);
		break;

	   case 4:
		value = functions.function4(vals[0], vals[1], vals[2], vals[3]);
		break;

	   case 5:
		value = functions.function5(vals[0], vals[1], vals[2], vals[3], vals[4]);
		break;

	   case 6:
		value = functions.function6(vals[0], vals[1], vals[2], vals[3], vals[4],
				vals[5]);
		break;

	   case 7:
		value = functions.function7(vals[0], vals[1], vals[2], vals[3], vals[4],
				vals[5], vals[6]);
		break;

	   case 8:
		value = functions.function8(vals[0], vals[1], vals[2], vals[3], vals[4],
				vals[5], vals[6], vals[7]);
		break;

	   case 9:
		value = functions.function9(vals[0], vals[1], vals[2], vals[3], vals[4],
				vals[5], vals[6], vals[7], vals[8]);
		break;

	   default:
		value = 0.0;
	        break;
	}
	result->type  = NUMBER;
	result->Value = value;
	return(symlist);
}

/*-----------------------------------------------------------
|
|  Routine Name: eval_ufunction - evaluate user defined function
|
|       Purpose: eval_ufunction() is called when we want to evaluate
|		 a user defined function.
|
|        Input:  symbol   - the function symbol
|		 symlist  - the argument symbols for the function
|		 abort_when_undefined - whether to abort when an undefined
|					variable is found.
|
|        Output: symbol   - the result stored in the "result" symbol
|		 RETURNS: the symbol list on success, NULL otherwise
|
|    Written By: Mark Young  
|          Date: 6/25/92
| Modifications:
|
------------------------------------------------------------*/


static SymbolList *eval_ufunction(
   Symbol     *symbol,
   SymbolList *symlist,
   int        abort_when_undefined,
   Symbol     *result)
{
	SavedValue values[100];
	SymbolList *arglist;
	Symbol   *arg, arg1;
	register int i, num_args;

	/*
	 *  Sanity check to make sure symlist is not NULL
	 */
	if (symlist == NULL)
	   return(NULL);

	kexpr_init_symbol(&arg1);
	
	/*
	 *  Evaluate the different argument expressions and then call the
	 *  eval_symlist() with the different evaluated arguments, but we
	 *  first need to save the argument values so that if we act just
	 *  like a stack would.
	 */
	num_args = symbol->num;
	if (result == NULL)
	{
	   for (i = 0; i < num_args; i++)
	      symlist = eval_symlist(symlist, abort_when_undefined, &arg1);

	   return(symlist);
	}

	arglist  = symbol->deflist;
	for (i = 0; i < num_args; i++)
	{
	   arg = arglist->symbol;
	   if (arg->exprType == STRING  ||  arg->type == STRING)
	   {
	      values[i].type = STRING;
	      values[i].u.string_value = kstrdup(STRING_VALUE(arg));
	   }
	   else
	   {
	      values[i].type = NUMBER;
	      values[i].u.double_value = arg->Value;
	   }
	   arglist = arglist->next;
	}
	arglist  = symbol->deflist;
	for (i = 0; i < num_args; i++)
	{
	   arg = arglist->symbol;
	   if (symlist->symbol->type == NUMBER)
           {
	      COPY_VALUE(arg, symlist->symbol);
/*	      arg->Value = symlist->symbol->Value;  */
	      symlist = symlist->next;
           }
	   else
	   {
	      symlist = eval_symlist(symlist, abort_when_undefined, &arg1);
	      COPY_VALUE(arg, &arg1);
	      RESET_SYMBOL(&arg1);
	   }
	   arglist = arglist->next;
	}
	(void) eval_symlist(arglist, abort_when_undefined, result);

	arglist = symbol->deflist;
	for (i = 0; i < num_args; i++)
	{
	   arg = arglist->symbol;
/*	   arg->Value = values[i]; */
	   if (values[i].type == STRING)
	   {
	      ASSIGN_CSTRING(arg, values[i].u.string_value);
	      kfree(values[i].u.string_value);
	   }
	   else
	   {
	      RESET_SYMBOL(arg);
	      arg->Value = values[i].u.double_value;
	   }
	   arglist = arglist->next;
	}
	return(symlist);
}

/*-----------------------------------------------------------
|
|  Routine Name: eval_operator - evaluate an operator
|
|       Purpose: eval_operator() is called when we want to evaluate
|		 an operation.
|
|        Input:  symbol   - the operator symbol
|		 symlist  - the argument symbols for the operator
|		 abort_when_undefined - whether to abort when an undefined
|					variable is found.
|
|        Output: symbol   - the result stored in the "result" symbol
|		 RETURNS: the symbol list on success, NULL otherwise
|
|    Written By: Mark Young  
|          Date: 6/25/92
| Modifications:
|
------------------------------------------------------------*/

#ifdef CRAY
#define vpow(x,y) ((val1) < 0.0 && (val2) == ((int) (val2)))            ? \
        (((((int) (val2)) % 2 == 0) ? 1 : -1) * pow(fabs(val1),val2))   : \
        pow(val1, val2)
#else
#define vpow pow
#endif

static SymbolList *eval_operator(
   Symbol     *symbol,
   SymbolList *symlist,
   int        abort_when_undefined,
   Symbol     *result)
{
	Symbol	 arg1, arg2;
	register double	value, val1, val2;
	char	 str1[2*KLENGTH], str2[2*KLENGTH], *tstr;


	if (result == NULL)
	{
	   symlist = eval_symlist(symlist, abort_when_undefined, NULL);
	   if (symbol->type == OPERATOR)
	      symlist = eval_symlist(symlist, abort_when_undefined, NULL);

	   return(symlist);
	}

	kexpr_init_symbol(&arg1);
	kexpr_init_symbol(&arg2);

	symlist = eval_symlist(symlist, abort_when_undefined, &arg1);
	val1 = get_numeric_value(&arg1);

	if (symbol->type == OPERATOR)
	{
	   symlist = eval_symlist(symlist, abort_when_undefined, &arg2);
	   val2 = get_numeric_value(&arg2);
	}
	else
	   val2 = 0.0;

	switch (symbol->Operator)
	{
	   case AND:
		value = ((val1 && val2) ? TRUE : FALSE);
		break;

	   case OR:
		value = ((val1 || val2) ? TRUE : FALSE);
		break;

	   case EQ:
	   case NE:
	        if (arg1.type == arg2.type)
		{
		   if (arg1.type == STRING)
		   {
		      value = (kstrcmp(STRING_VALUE(&arg1),
				       STRING_VALUE(&arg2)) == 0);
		   }
		   else
		      value = ((val1 == val2) ? TRUE : FALSE);
		}
		else
		   value = FALSE;
		if (symbol->Operator == NE)
		   value = !value;
		break;

	   case LE:
	        if (arg1.type == arg2.type)
		{
		   if (arg1.type == STRING)
		   {
		      value = (kstrcmp(STRING_VALUE(&arg1),
				       STRING_VALUE(&arg2)) <= 0);
		   }
		   else
		      value = ((val1 <= val2) ? TRUE : FALSE);
		}
		else
		   value = FALSE;
		break;

	   case GE:
	        if (arg1.type == arg2.type)
		{
		   if (arg1.type == STRING)
		   {
		      value = (kstrcmp(STRING_VALUE(&arg1),
				       STRING_VALUE(&arg2)) >= 0);
		   }
		   else
		      value = ((val1 >= val2) ? TRUE : FALSE);
		}
		else
		   value = FALSE;
		break;

	   case LT:
	        if (arg1.type == arg2.type)
		{
		   if (arg1.type == STRING)
		   {
		      value = (kstrcmp(STRING_VALUE(&arg1),
				       STRING_VALUE(&arg2)) < 0);
		   }
		   else
		      value = ((val1 < val2) ? TRUE : FALSE);
		}
		else
		   value = FALSE;
		break;

	   case GT:
	        if (arg1.type == arg2.type)
		{
		   if (arg1.type == STRING)
		   {
		      value = (kstrcmp(STRING_VALUE(&arg1),
				       STRING_VALUE(&arg2)) > 0);
		   }
		   else
		      value = ((val1 > val2) ? TRUE : FALSE);
		}
		else
		   value = FALSE;
		break;

	   case '+':
		value = val1+val2;
		break;

	   case '-':
		value = val1 - val2;
		break;

	   case '*':
		value = val1*val2;
		break;

	   case '/':
		if (abort_when_undefined == TRUE)
	        {
		   if (val2 == 0.0)
		   {
		      kexpr_error("Floating divide by zero");
		      return(NULL);
		   }
		   value = val1/val2;
	        }
		else
		   value = (val2 != 0.0) ? val1/val2 : val1;
		break;

	   case POW:
		value = vpow(val1, val2);
		break;

	   case '%':
		value = ((((int) val2) != 0) ? ((int) val1) % ((int) val2) : 0);
		break;

	   case '&':
		value = ((int) val1) & ((int) val2);
		break;

	   case '|':
		value = ((int) val1) | ((int) val2);
		break;

	   case '^':
		value = ((int) val1) ^ ((int) val2);
		break;

	   case SL:
		value = ((int) val1) << ((int) val2);
		break;

	   case SR:
		value = ((int) val1) >> ((int) val2);
		break;

	   case '~':
		value = ~((int) val1);
		break;

	   case '!':
		value = !val1;
		break;

	   case UMINUS:
		value = -val1;
		break;

	   case CINT:
		value = (int) val1;
		break;

	   case CFLOAT:
		value = (float) val1;
		break;

	   case CSTRING:
		if (arg1.type == NUMBER)
		   ksprintf(str1,"%g", arg1.Value);
		else if (arg1.type == STRING)
		   kstrcpy(str1, STRING_VALUE(&arg1));
		else
		   str1[0] = '\0';

		ASSIGN_CSTRING(result, str1);
		RESET_SYMBOL(&arg1);
		RESET_SYMBOL(&arg2);
		return(symlist);

	   case STRCAT:
		if (arg1.type == NUMBER)
		   (void) ksprintf(str1,"%g", arg1.Value);
		else if (arg1.type == STRING)
		   kstrcpy(str1, STRING_VALUE(&arg1));
		else
		   (void) kstrcpy(str1,"");

		if (arg2.type == NUMBER)
		   (void) ksprintf(str2,"%g", arg2.Value);
		else if (arg2.type == STRING)
		   kstrcpy(str2, STRING_VALUE(&arg2));
		else
		   (void) kstrcpy(str2,"");

		tstr = kstring_cat(str1, str2, NULL);
		ASSIGN_CSTRING(result, tstr);
		kfree(tstr);
		RESET_SYMBOL(&arg1);
		RESET_SYMBOL(&arg2);
		return(symlist);

	   default:
		value = 0.0;
		break;
	}
	RESET_SYMBOL(&arg1);
	RESET_SYMBOL(&arg2);
	RESET_SYMBOL(result);
	result->type  = NUMBER;
	result->Value = value;
	return(symlist);
}

/*-----------------------------------------------------------
|
|  Routine Name: eval_instruction - evaluate an instruction
|
|       Purpose: eval_instruction() is called when we want to evaluate
|		 an instruction.
|
|        Input:  symbol   - the instruction symbol
|		 symlist  - the argument symbols for the instruction
|		 abort_when_undefined - whether to abort when an undefined
|					variable is found.
|
|        Output: symbol   - the result stored in the "result" symbol
|		 RETURNS: the symbol list on success, NULL otherwise
|
|    Written By: Mark Young  
|          Date: 6/25/92
| Modifications:
|
------------------------------------------------------------*/


static SymbolList *eval_instruction(
   Symbol     *symbol,
   SymbolList *symlist,
   int        abort_when_undefined,
   Symbol     *result)
{
	Symbol	arg;


	if (result == NULL)
	{
	   if (symbol->Instruction == IF)
	   {
	      symlist = eval_symlist(symlist, abort_when_undefined, NULL);
	      symlist = eval_symlist(symlist, abort_when_undefined, NULL);
	      symlist = eval_symlist(symlist, abort_when_undefined, NULL);
	   }
	   return(symlist);
	}

	kexpr_init_symbol(&arg);

	switch (symbol->Instruction)
	{
	    case IF:
		 symlist = eval_symlist(symlist, abort_when_undefined, &arg);
		 if (arg.Value)
		 {
		    symlist = eval_symlist(symlist, abort_when_undefined, &arg);
		    symlist = eval_symlist(symlist, abort_when_undefined, NULL);
	         }
		 else
		 {
		    symlist = eval_symlist(symlist, abort_when_undefined, NULL);
		    symlist = eval_symlist(symlist, abort_when_undefined, &arg);
		 }
		 break;
	}

	result->type = arg.type;
	if (arg.type == NUMBER)
	   result->Value = arg.Value;
	else
	   ASSIGN_STRING(result, &arg);
	RESET_SYMBOL(&arg);
	return(symlist);
}

/*-----------------------------------------------------------
|
|  Routine Name: eval_symlist - evaluate a symbol list
|
|       Purpose: eval_symlist() is called when we want to evaluate
|		 a symbol list.
|
|        Input:  symlist  - the current symbol list to be evaluated
|		 abort_when_undefined - whether to abort when an undefined
|					variable is found.
|
|        Output: symbol   - the result stored in the "result" symbol
|		 RETURNS: the symbol list on success, NULL otherwise
|
|    Written By: Mark Young  
|          Date: 6/25/92
| Modifications: changed parameter list so that "result" was last in the
|		 list - (6/25/92 MY)
|    PH (04/01/97) added some debugging code
|
------------------------------------------------------------*/


static SymbolList *eval_symlist(
   SymbolList *symlist,
   int        abort_when_undefined,
   Symbol     *result)
{
	SymbolList *tmplist;
	Symbol	*symbol, *tmp;
	char	error[2*KLENGTH];
	SymbolList *rc;

	/*
	 *  If the current symlist is NULL then the endlist will become
	 *  the beginning of the symbol list, so we just return it.
	 */
	if (symlist == NULL)
	{
	   ksprintf(error,
		    "Error! end of symbol list encountered, but more expected.");
	   kexpr_error(error);
	   return(NULL);
	}

	if (result != NULL)
	   RESET_SYMBOL(result);
	
	symbol = symlist->symbol;
#ifdef DEBUG_KEXPR
	kexpr_printf("Evaluating %s\n", expr_type_name(symbol->type));
	kexpr_indent(2);
#endif
		     
	switch (symbol->type)
	{
	   case NUMBER:
		if (result != NULL)
		{
		   result->type  = NUMBER;
		   result->Value = symbol->Value;
		}
		symlist = symlist->next;
		break;
	   case CONSTANT:
		if (result != NULL)
		{
		   result->type  = NUMBER;
		   result->Value = symbol->Value;
		}
		symlist = symlist->next;
		break;
	   case EXPRESSION:
		if (result != NULL)
		{
		   if (symbol->exprType == STRING)
		   {
		      ASSIGN_STRING(result, symbol);
		   }
		   else
		   {
		      result->type  = NUMBER;
		      result->Value = symbol->Value;
		   }
		}
		symlist = symlist->next;
		break;

	   case STRING:
		if (result != NULL)
		{
		   ASSIGN_STRING(result, symbol);
		}
		symlist = symlist->next;
		break;

	   case VARIABLE:
		if ((unsigned int) symbol->eval == FALSE)
		{
		   symbol->eval = TRUE;
		   (void) eval_symlist(symbol->deflist,
				       abort_when_undefined, result);
		   symbol->eval = FALSE;
		}
		else if (result != NULL)
		{
		   result->type  = NUMBER;
		   result->Value = symbol->Value;
		}
		symlist = symlist->next;
		break;

	   case FUNCTION:
		symlist = symlist->next;
		symlist = eval_function(symbol,
					symlist, abort_when_undefined, result);
		break;

	   case UFUNCTION:
		symlist = symlist->next;
		symlist = eval_ufunction(symbol,
					 symlist, abort_when_undefined,
					 result);
		break;

	   case UOPERATOR:
	   case OPERATOR:
		symlist = symlist->next;
		symlist = eval_operator(symbol,
					symlist, abort_when_undefined, result);
		break;

	   case INSTRUCTION:
		symlist = symlist->next;
		symlist = eval_instruction(symbol, symlist,
					   abort_when_undefined, result);
	        break;

	   case UNDEFINED:
		tmp = kexpr_reget_symbol(symbol->idlist, symbol);
		if (tmp != NULL)
		{
		   tmplist = symlist;
		   while (tmplist != NULL)
		   {
		      if (tmplist->symbol == symbol)
			 tmplist->symbol = tmp;

		      tmplist = tmplist->next;
		   }
		   rc = eval_symlist(symlist, abort_when_undefined, result);
#ifdef DEBUG_KEXPR
		   kexpr_indent(-2);
#endif
		   return(rc);
	        }

		if (abort_when_undefined)
		{
		   ksprintf(error,"Error! Unknown variable or constant '%s'",
				symbol->name);
		   kexpr_error(error);
		   return(NULL);
		}
		symlist = symlist->next;

		if (result != NULL)
		{
		   result->type  = UNDEFINED;
		   result->Value = 0.0;
		}
		break;

	   default:
		(void) ksprintf(error,"Error!  Unknown symbol type '%d'",
				symbol->type);
		kexpr_error(error);
		return(NULL);
	}
#ifdef DEBUG_KEXPR
	kexpr_indent(-2);
#endif
	return(symlist);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_eval_symbol - evaluate a single symbol
|
|       Purpose: kexpr_eval_symbol() is called when we want to evaluate
|		 a symbol.
|
|        Input:  symbol  - the current symbol to be evaluated
|		 abort_when_undefined - whether to abort when an undefined
|					variable is found.
|
|        Output: symbol   - the result stored in the "result" symbol
|                RETURNS: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Mark Young  
|          Date: 6/25/92
| Modifications: changed parameter list so that "result" was last in the
|		 list - (6/25/92 MY)
|
------------------------------------------------------------*/


int kexpr_eval_symbol(
   Symbol *symbol,
   int    abort_when_undefined,
   Symbol *result)
{
	int	   i;
	Symbol	   temp_symbol;
	SymbolList list, *symlist;
	char       error[2*KLENGTH];

	/*
	 *  If the current symbol is NULL then print out an error
	 *  and abort.
	 */
	error_found = FALSE;
	if (symbol == NULL)
	{
	   ksprintf(error,"Error! NULL symbol list encountered.\n");
	   kexpr_error(error);
	   return(FALSE);
	}
	else if (result == NULL)
	   result = &temp_symbol;

	kexpr_init_symbol(&temp_symbol);

 	if (symbol->type == VARIABLE)
	{
	   symbol->eval = TRUE;
	   eval_symlist(symbol->deflist, abort_when_undefined, result);
	   COPY_VALUE(symbol, result);
	   symbol->eval = FALSE;
	}
	else if (symbol->type == EXPRESSION)
	{
	   eval_symlist(symbol->deflist, abort_when_undefined, result);
	   COPY_VALUE(symbol, result);
	}
	else if (symbol->type == UFUNCTION)
	{
	   symlist = symbol->deflist;
	   for (i = 0; i < (unsigned int)symbol->num; i++)
	      symlist = symlist->next;

           if (abort_when_undefined)
           {
              /* I don't think we will ever be here */
	      (void) eval_symlist(symlist, abort_when_undefined, result);
	      COPY_VALUE(symbol, result);
           }
           else
           {
              /* 
                 normal case - don't really evaluate since args are
                 undefined anyway and evaluation may cause SIGFPEs
              */
              (void) eval_symlist(symlist, abort_when_undefined, NULL);
              ASSIGN_DOUBLE(result, 0.0);
              COPY_VALUE(symbol, result);
           }
	}
	else
	{
	   list.symbol = symbol;
	   list.next   = NULL;
	   (void) eval_symlist(&list, abort_when_undefined, result);
	}
	return(error_found == FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_eval_symlist - evaluate a symbol list
|
|       Purpose: kexpr_eval_symlist() is called when we want to evaluate
|		 a list of symbols.
|
|        Input:  symlist  - the current symbol to be evaluated
|		 abort_when_undefined - whether to abort when an undefined
|					variable is found.
|
|        Output: symbol   - the result stored in the "result" symbol
|                RETURNS: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Mark Young  
|          Date: 6/25/92
| Modifications: changed parameter list so that "result" was last in the
|		 list - (6/25/92 MY)
|
------------------------------------------------------------*/


int kexpr_eval_symlist(
   SymbolList *symlist,
   int        abort_when_undefined,
   Symbol     *result)
{
	char	   error[2*KLENGTH];


	/*
	 *  If the current symlist is NULL then the endlist will become
	 *  the beginning of the symbol list, so we just return it.
	 */
	error_found = FALSE;
	if (symlist == NULL)
	{
	   ksprintf(error,"Error! NULL symbol list encountered.\n");
	   kexpr_error(error);
	   return(FALSE);
	}

	if (result == NULL)
	{
	   ksprintf(error,"Error! NULL result symbol encountered.\n");
	   kexpr_error(error);
	   return(FALSE);
	}
	(void) eval_symlist(symlist, abort_when_undefined, result);
	return(error_found == FALSE);
}


/*-----------------------------------------------------------
|
|  Routine Name: get_numeric_value - get a numeric value from a Symbol
|
|       Purpose: get_numeric_value gets a double value from a symbol.
|                If the symbol is a string, it it attempts to convert
|                it to a number.
|
|        Input:  symbol   - the variable or constant symbol
|
|        Output: none
|
|        Return: numeric value
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
| Modifications:
|
------------------------------------------------------------*/
static double get_numeric_value(Symbol *sym)
{
   double result = sym->Value;

   if (sym->type == STRING)
   {
      if (ksscanf(STRING_VALUE(sym), "%lf", &result) != 1)
      {
#if 0
	 char *buf;
	 buf = kstring_3cat("Unable to convert \"",
			    STRING_VALUE(sym),
			    "\" to a number.",
			    NULL);
	 if (buf != NULL)
	 {
	    kexpr_error(buf);
	    KSTRFREE(buf);
	 }
#endif
	 result = 0;
      }
   }
   return result;
}

