/*
  * Khoros: $Id: function.c,v 1.10.2.5 1997/05/27 22:11:28 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: function.c,v 1.10.2.5 1997/05/27 22:11:28 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Various Private Print Routines For the kexpr Library
   >>>>		
   >>>>  Private:
   >>>>                 kexpr_add_function()
   >>>>                 kexpr_check_function()
   >>>>                 kexpr_get_function()
   >>>>   Static:
   >>>>                 init_function()
   >>>>                 kexpr_pathpart()
   >>>>                 kexpr_filepart()
   >>>>                 kexpr_extpart()
   >>>>                 kexpr_dumpsym()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include "y.tab.h"

static double kexpr_ldexp(double, double);
static char *kexpr_upcase(char *str);
static char *kexpr_dncase(char *str);
static char *kexpr_pathpart(char *str);
static char *kexpr_filepart(char *str);
static char *kexpr_extpart(char *str);
static char *kexpr_dumpsym(char *sym);
static char *kexpr_dumpdef(char *sym);
static char *kexpr_getenv(char *var);
static char *kexpr_putenv(char *envstr);

kfunc_double f1  = (kfunc_double) ksinc;
kfunc_double f2  = (kfunc_double) kfact;
kfunc_double f3  = (kfunc_double) kimpulse;
kfunc_double f4  = (kfunc_double) kstep;
kfunc_double f5  = (kfunc_double) ksign;
kfunc_double f6  = (kfunc_double) kurng;
kfunc_double ke_ldexp = (kfunc_double) kexpr_ldexp;

kfunc_double upcase  = (kfunc_double) kexpr_upcase;
kfunc_double dncase  = (kfunc_double) kexpr_dncase;
kfunc_double pathpart  = (kfunc_double) kexpr_pathpart;
kfunc_double filepart  = (kfunc_double) kexpr_filepart;
kfunc_double extpart  = (kfunc_double) kexpr_extpart;
kfunc_double dumpsym  = (kfunc_double) kexpr_dumpsym;
kfunc_double dumpdef  = (kfunc_double) kexpr_dumpdef;
kfunc_double kxgetenv = (kfunc_double) kexpr_getenv;
kfunc_double kxputenv = (kfunc_double) kexpr_putenv;

static int one_num[] = { NUMBER };
static int two_nums[] = { NUMBER, NUMBER };
static int one_str[] = { STRING };

/*
 * Function declaration list
 */
static struct _function functions[] =
{
   /*                                               return   argument */
   /* name        pointer to function         args   type      types  */
   /* -----       -----------------------     ----  ------   -------- */
    { "sin",	  (kfunc_double *) &ksin,	 1, NUMBER,  one_num },
    { "cos",	  (kfunc_double *) &kcos,	 1, NUMBER,  one_num },
    { "tan",	  (kfunc_double *) &ktan,	 1, NUMBER,  one_num },

    { "sinh",	  (kfunc_double *) &ksinh,       1, NUMBER,  one_num },
    { "cosh",	  (kfunc_double *) &kcosh,       1, NUMBER,  one_num },
    { "tanh",	  (kfunc_double *) &ktanh,       1, NUMBER,  one_num },

    { "asin",	  (kfunc_double *) &kasin,       1, NUMBER,  one_num },
    { "acos",	  (kfunc_double *) &kacos,       1, NUMBER,  one_num },
    { "atan",	  (kfunc_double *) &katan,       1, NUMBER,  one_num },
    { "atan2",	  (kfunc_double *) &katan2,      2, NUMBER,  two_nums},
    { "sinc",	  (kfunc_double *) &f1,          1, NUMBER,  one_num },

    { "asinh",	  (kfunc_double *) &kasinh,      1, NUMBER,  one_num },
    { "acosh",	  (kfunc_double *) &kacosh,      1, NUMBER,  one_num },
    { "atanh",	  (kfunc_double *) &katanh,      1, NUMBER,  one_num },

    { "ln",	  (kfunc_double *) &klog,	 1, NUMBER,  one_num },
    { "log2",	  (kfunc_double *) &klog2,       1, NUMBER,  one_num },
    { "log10",	  (kfunc_double *) &klog10,      1, NUMBER,  one_num },
    { "log1p",	  (kfunc_double *) &klog1p,      1, NUMBER,  one_num },

    { "exp",	  (kfunc_double *) &kexp,	 1, NUMBER,  one_num },
    { "exp2",	  (kfunc_double *) &kexp2,       1, NUMBER,  one_num },
    { "exp10",	  (kfunc_double *) &kexp10,      1, NUMBER,  one_num },
    { "expm1",	  (kfunc_double *) &kexpm1,      1, NUMBER,  one_num },

    { "pow",	  (kfunc_double *) &kpow,        2, NUMBER,  two_nums },
    { "sqrt",	  (kfunc_double *) &ksqrt,       1, NUMBER,  one_num },
    { "cbrt",	  (kfunc_double *) &kcbrt,       1, NUMBER,  one_num },

    { "abs",	  (kfunc_double *) &kfabs,       1, NUMBER,  one_num },
    { "trunc",	  (kfunc_double *) &ktrunc,      1, NUMBER,  one_num },
    { "fmod",	  (kfunc_double *) &kfmod,       2, NUMBER,  two_nums },
    { "ldexp",	  (kfunc_double *) &ke_ldexp,    2, NUMBER,  two_nums },

    { "floor",	  (kfunc_double *) &kfloor,      1, NUMBER,  one_num },
    { "ceil",	  (kfunc_double *) &kceil,       1, NUMBER,  one_num },
    { "fact",	  (kfunc_double *) &f2,          1, NUMBER,  one_num },

    { "impulse",  (kfunc_double *) &f3,          1, NUMBER,  one_num },
    { "step",	  (kfunc_double *) &f4,          1, NUMBER,  one_num },
    { "sign",	  (kfunc_double *) &f5,          1, NUMBER,  one_num },
    { "erf",	  (kfunc_double *) &kerf,        1, NUMBER,  one_num },
    { "erfc",	  (kfunc_double *) &kerfc,       1, NUMBER,  one_num },
    { "gamma",	  (kfunc_double *) &kgamma,      1, NUMBER,  one_num },

    { "j0",	  (kfunc_double *) &kj0,	 1, NUMBER,  one_num },
    { "j1",	  (kfunc_double *) &kj1,	 1, NUMBER,  one_num },
    { "y0",	  (kfunc_double *) &ky0,         1, NUMBER,  one_num },
    { "y1",	  (kfunc_double *) &ky1,	 1, NUMBER,  one_num },

    { "hypot",	  (kfunc_double *) &khypot,      2, NUMBER,  two_nums },
    { "urng",	  (kfunc_double *) &f6,          0, NUMBER,  one_num },
    { "unoise",	  (kfunc_double *) &f6,          0, NUMBER,  one_num },

    { "make_upper",  (kfunc_double *) &upcase, 1, STRING, one_str },
    { "make_lower",  (kfunc_double *) &dncase, 1, STRING, one_str },
    { "dirname",  (kfunc_double *) &pathpart, 1, STRING, one_str },
    { "basename",  (kfunc_double *) &filepart, 1, STRING, one_str },
    { "extension",  (kfunc_double *) &extpart, 1, STRING, one_str },
    { "kexpr_dump",  (kfunc_double *) &dumpsym, 1, STRING, one_str },
    { "kexpr_def",   (kfunc_double *) &dumpdef, 1, STRING, one_str },
    { "getenv",   (kfunc_double *) &kxgetenv, 1, STRING, one_str },
    { "putenv",   (kfunc_double *) &kxputenv, 1, STRING, one_str },

    { NULL,	  0,	       0, 0 },
};

/*-----------------------------------------------------------
|
| Routine Name: init_function - initialize the kexpr parser
|
|       Purpose: This routine initializes the kexpr parser.  This call
|		 used to initialize the constants, functions, and
|		 global variable lists.  It MUST be called prior to ANY
|		 other calls to the kexpr library.
|
|         Input:
|        Output:
|	Returns:
|
|  Restrictions: 
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
|      Verified:
|  Side Effects:
| Modifications:
|
*-----------------------------------------------------------*/

static void init_function(void)
{
	int	i;
	Symbol  *symbol;
	IdList *idlist;
	static int initialized = FALSE;
	long    id = KEXPR_FUNCTION_ID;


	if (initialized == TRUE)
	   return;

	/*
	 * Install the idlist for Constants, Functions, and Global
	 * variables.
	 */
	initialized = TRUE;
	if ((idlist = kexpr_get_idlist(id)) != NULL)
	{
	   for (i = 0; functions[i].name != NULL; i++)
	   {
	       symbol = kexpr_add_symbol(idlist, functions[i].name, FUNCTION,
				0.0);
	       symbol->num      = functions[i].num_args;
	       symbol->Function = functions + i;
	       symbol->exprType = functions[i].return_type;
	   }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_add_function()
|
|       Purpose: kexpr_add_function() is called when we want to add a
|		 function to the symbol list.  The function symbol
|		 does not need to be copied, since we want to share
|		 functions within the system.  The current symbol list
|		 is passed in and the number is then tacked onto the
|		 end of the symbol list.  If the symbol list is currently
|		 NULL then we will create a new list and add the variable
|		 to it.
|
|        Input:  symlist    - the current symbol list
|		 function   - the symbol to be added to the list
|		 num_args   - the number of arguments specified with
|			      the function.
|
|       Output:  returns the new symbol list.
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
| Modifications:
|
|----------------------------------------------------------*/

SymbolList *kexpr_add_function(
   SymbolList *symlist,
   Symbol     *function,
   int	      num_args)
{
	char	   error[KLENGTH];
	SymbolList *list, *entry;


	/*
	 *  Allocate the entry for our new function
	 */
	if (function->num != num_args && function->type != UNDEFINED)
	{
	   (void) ksprintf(error, "Error! Incorrect number of arguments specified."
			   "\nFunction '%s' takes %d arguments, but you specified %d",
			function->name, (int) function->num, num_args);
	   kexpr_error(error);
	}

	/*
	 *  Allocate the entry for our new function
	 */
	if ((entry = (SymbolList *) kcalloc((size_t) 1, sizeof(SymbolList))) == NULL)
	{
	   (void) ksprintf(error, "Error! Ran out of memory trying to allocate \
reference list for function '%s'", function->name);
	   kexpr_error(error);
	}
	entry->symbol = function;
	entry->next   = NULL;

	/*
	 *  If the current symlist is NULL then return entry as our symlist.
	 */
	if (symlist == NULL)
	{
	   return(entry);
	}

	/*
	 *  The current symbol list is not empty so add the entry to the
	 *  end of the current list.
	 */
	list = symlist;
	while (list->next != NULL)
	   list = list->next;

	list->next = entry;
	return(symlist);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_check_function()
|
|       Purpose: kexpr_check_function() is called when we want to
|		 create a user defined function.  The main
|		 purpose of this routine is make the function's
|		 argument list be local to only the function and
|		 not global to the rest of the system.
|
|        Input:  symbol   - the current variable symbol
|
|       Output:  None.  Make the functions symlist have only local
|		 variables and then update the function expression
|		 to use these local variables.
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
| Modifications:
|
|----------------------------------------------------------*/

void kexpr_check_function(
    Symbol     *function,
    SymbolList *arglist,
    SymbolList *symlist)
{
	SymbolList *list;
	Symbol	   *symbol, *temp;
	int	   num_args = 0;
	char	   error[KLENGTH];


	/*
	 *  If the current function is NULL then just return
	 */
	if (function == NULL)
	   return;

	/*
	 *  Change all function arguments from global to local variables.
	 *  At the same time change the expression to use these local
	 *  variables.
	 */
	while (arglist != NULL)
	{
	   symbol = arglist->symbol;
	   if (symbol->type == VARIABLE || symbol->type == EXPRESSION ||
	       symbol->type == UNDEFINED)
	   {
	      /*
	       *  Allocate the symbol to store our local variable
	       */
	      if ((temp = (Symbol *)kcalloc((size_t)1,sizeof(Symbol))) == NULL)
	      {
	         (void) ksprintf(error, "Error! Ran out of memory trying to \
allocate local argument '%s' for function '%s' ", symbol->name, function->name);
		 kexpr_error(error);
	      }
	      temp->Value = 0.0;
	      temp->type  = EXPRESSION;
	      temp->name  = kstrdup(symbol->name);
	      arglist->symbol = temp;

	      list = symlist;
	      while (list != NULL)
	      {
		 if (list->symbol == symbol)
		    list->symbol = temp;

	         list = list->next;
	      }
	   }
	   arglist = arglist->next; num_args++;
	}
	function->num = num_args;
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_get_function - get function symbol
|
|       Purpose: kexpr_get_function() - pulls function name from
|		 the expression string.
|
|         Input: string - the string which contains the desired function
|
|        Output: None, except that the resulting function is stored
|		 in the global kexpr_lval.symbol.
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
| Modifications:
|
------------------------------------------------------------*/

Symbol *kexpr_get_function(
  char	*string)
{
	IdList *idlist;
	Symbol  *symbol = NULL;
	long    id = KEXPR_FUNCTION_ID;

	/*
	 * Check check to see if it is a defined function
	 */
	init_function();
	if ((idlist = kexpr_get_idlist(id)) != NULL)
	{
	   if ((symbol = kexpr_get_symbol(idlist, string, TRUE)) != NULL)
	   {
	      return(symbol);
	   }
	}

	/*
	 * Check to see if it is in the current idlist list.
	 */
	if ((idlist = kexpr_get_idlist(current_id)) != NULL)
	{
	   if ((symbol = kexpr_get_symbol(idlist, string, TRUE)) == NULL)
	   {
	      symbol = kexpr_add_symbol(idlist, string, UNDEFINED, 0.0);
	      return(symbol);
	   }
	}
	return(symbol);
}


static char *kexpr_upcase(char *str)
{
   return kstring_upper(str, NULL);
}

static char *kexpr_dncase(char *str)
{
   return kstring_lower(str, NULL);
}


/*-----------------------------------------------------------
|
|  Routine Name: kexpr_pathpart
|
|       Purpose: kexpr_pathpart extracts that path portion
|                of a path+filename.
|
|         Input: str - input pathname+filename
|
|        Output: none
|
|        Return: string containing path
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
static char *kexpr_pathpart(char *str)
{
   char *result, *buf, *ptr;

   buf = kstrdup(str);
   ptr = kstrrchr(buf, '/');
   if (ptr)
   {
      *(ptr+1)= 0;
      result = kstrdup(buf);
   }
   else
   {
      result = kstrdup("");
   }
   kfree(buf);
   return result;
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_filepart
|
|       Purpose: kexpr_filepart extracts that filename portion
|                of a path+filename.
|
|         Input: str - input pathname+filename
|
|        Output: none
|
|        Return: string containing filename
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
static char *kexpr_filepart(char *str)
{
   char *result, *buf, *ptr;

   buf = kstrdup(str);
   ptr = kstrrchr(buf, '/');
   if (ptr)
   {
      result = kstrdup(ptr+1);
   }
   else
   {
      result = kstrdup(str);
   }
   kfree(buf);
   return result;
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_extpart
|
|       Purpose: kexpr_filepart extracts that file extension
|                portion of a path+filename.
|
|         Input: str - input pathname+filename
|
|        Output: none
|
|        Return: string containing file extension
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
static char *kexpr_extpart(char *str)
{
   char *result, *buf, *ptr;

   buf = kexpr_filepart(str);
   ptr = kstrrchr(buf, '.');
   if (ptr)
   {
      result = kstrdup(ptr+1);
   }
   else
   {
      result = kstrdup("");
   }
   kfree(buf);
   return result;
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_dumpsym
|
|       Purpose: kexpr_dumpsym dumps the named symbol
|
|         Input: symname - symbol name
|
|        Output: none
|
|        Return: 0.0
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
static char *kexpr_dumpsym(char *symname)
{
   Symbol *sym;
   int tsave = kexpr_trace;

   sym = kexpr_find_variable(symname, 1);
   if (sym != NULL)
   {
      kexpr_trace = 1;
      kexpr_dump_symbol("Variable dump", sym);
      kexpr_trace = tsave;
   }
   else kprintf("Symbol \"%s\" was not found.\n", symname);
   return kstrdup(symname);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_dumpdef
|
|       Purpose: kexpr_dumpdef dumps the named symbol's definitino
|
|         Input: symname - symbol name
|
|        Output: none
|
|        Return: 0.0
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
static char *kexpr_dumpdef(char *symname)
{
   char *result = "UNKNOWN";
   Symbol *sym;

   sym = kexpr_find_variable(symname, 1);
   if (sym != NULL)
   {
      result = sym->definition;
   }
   return kstrdup(result);
}


/*-----------------------------------------------------------
|
|  Routine Name: kexpr_getenv
|
|       Purpose: kexpr_getenv retrieves the value of an
|                environment using kgetenv
|
|         Input: var - env var name
|
|        Output: none
|
|        Return: 0.0
|
|    Written By: Pete Humphrey
|          Date: April 28, 1997
|
------------------------------------------------------------*/
static char *kexpr_getenv(char *var)
{
   char *rc;

   rc = kgetenv(var);
   if (rc) rc = kstrdup(rc);
   else rc = kstrdup("");
   return rc;
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_putenv
|
|       Purpose: kexpr_putenv adds an variable/value pair to
|                the environment space using kputenv
|
|         Input: envstr - a string of the form "var=value"
|
|        Output: none
|
|        Return: the input string (string functions have to
|                return a string, for now)
|
|    Written By: Pete Humphrey
|          Date: April 28, 1997
|
------------------------------------------------------------*/
static char *kexpr_putenv(char *envstr)
{
   kputenv(envstr);
   return kstrdup(envstr);
}


/*-----------------------------------------------------------
|
|  Routine Name: kexpr_ldexp
|
|       Purpose: kexpr_ldexp is a wrapper around kldexp.
|
|         Input: x
|                n
|
|        Output: 
|
|        Return: x * (2^n)
|
|    Written By: Pete Humphrey
|          Date: May 27, 1997
|
------------------------------------------------------------*/

static double kexpr_ldexp(double x, double n)
{
   return kldexp(x, (int)n);
}
