/*
 * Khoros: $Id: internals.h,v 1.6.2.4 1997/05/24 17:49:11 mirage Exp $
 */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Global Function and Constant definition
   >>>>		       used by the khoros expression parser.
   >>>>
   >>>>    Written By: & Mark Young
   >>>>
   >>>>          Date: Apr 04, 1994
   >>>>
   >>>> Modifications:
   >>>>   PH (04/01/97) - added fields to Symbol and struct
   >>>>                   _function. Added new function
   >>>>                   prototypes
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#ifndef _kexpr_internals_h_
#define _kexpr_internals_h_

	/*-------------------------------------*
	|       #include 
	---------------------------------------*/

#include <bootstrap.h>


	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

#define Value	    u.value
#define Operator    u.oper
#define Instruction u.instruction
#define Function    u.function
#define String	    u.string

#ifdef DEBUG_KEXPR

#define HAS_STRING(x) ((x)->type == STRING  ||  ((x)->exprType == STRING  &&  ((x)->type == EXPRESSION || (x)->type == VARIABLE)))
#define HAS_DOUBLE(x) ((x)->type == NUMBER  ||  ((x)->exprType == NUMBER  &&  ((x)->type == EXPRESSION || (x)->type == VARIABLE)))
#define FREE_SYMBOL(x) kexpr_free_symbol((x), __LINE__, __FILE__)
#define FREE_SYMLIST(x) kexpr_free_symlist((x), __LINE__, __FILE__)
#define RESET_SYMBOL(x) kexpr_reset_symbol((x), __LINE__, __FILE__)
#define RESET_VALUE(x) kexpr_reset_value((x), __LINE__, __FILE__)
#define COPY_VALUE(x, y) kexpr_copy_value((x), (y), __LINE__, __FILE__)

#define KSTRDUP(x)     kexpr_strdup((x), __LINE__, __FILE__)
#define KSTRFREE(x)    kexpr_strfree((x), __LINE__, __FILE__)
#define STRING_VALUE(s) (s)->String
#define ASSIGN_DOUBLE(s, v) kexpr_assign_double((s), (v), __LINE__, __FILE__)
#define ASSIGN_STRING(s, v) kexpr_assign_string((s), (v), __LINE__, __FILE__)
#define ASSIGN_CSTRING(s, v) kexpr_assign_cstring((s), (v), __LINE__, __FILE__)

#else

#define HAS_STRING(x) ((x)->type == STRING  ||  ((x)->exprType == STRING  &&  ((x)->type == EXPRESSION || (x)->type == VARIABLE)))
#define HAS_DOUBLE(x) ((x)->type == NUMBER  ||  ((x)->exprType == NUMBER  &&  ((x)->type == EXPRESSION || (x)->type == VARIABLE)))
#define FREE_SYMBOL(x) kexpr_free_symbol((x))
#define FREE_SYMLIST(x) kexpr_free_symlist((x))
#define RESET_SYMBOL(x) kexpr_reset_symbol((x))
#define RESET_VALUE(x) kexpr_reset_value((x))
#define COPY_VALUE(x, y) kexpr_copy_value((x), (y))

#define KSTRDUP(x)     kstrdup((x))
#define KSTRFREE(x)    kfree((x))
#define STRING_VALUE(s) (s)->String
#define ASSIGN_DOUBLE(s, v) kexpr_assign_double((s), (v))
#define ASSIGN_STRING(s, v) kexpr_assign_string((s), (v))
#define ASSIGN_CSTRING(s, v) kexpr_assign_cstring((s), (v))

#endif


	/*-------------------------------------*
	|       typdefs
	---------------------------------------*/

/*
 * Function structure declaration
 */
struct _function
{
	char         *name;	  /* public name */
	kfunc_double *routine;	  /* pointer to function */
	int          num_args;	  /* how many args */
        int          return_type; /* return datatype */
        int          *arg_types;  /* argument datatypes */
};

/*
 * Constant structure declaration
 */
struct _constant
{
	char *name;
	double value;
};


/*
 *  The following definition is used to define a single token.
 *  Such as the definition of x, or the definition of a function, etc.
 */
typedef struct _symbol
{
	char *name;
	char *definition;
	int  type;
	unsigned char eval, num;
	struct _symbol_list *deflist;
	union
	{
	   double value;
	   char   *string;
	   int    oper;
	   int    instruction;
	   struct _function *function;
	} u;
	struct _idlist *idlist;
        int  exprType;		/* if type = EXPRESSION, the datatype */
} Symbol;

typedef struct _SavedValue
{
   int type;
   union {
      double double_value;
      char *string_value;
   } u;
} SavedValue;

typedef struct _symbol_list
{
	Symbol *symbol;
	struct _symbol_list *next;
} SymbolList;

typedef struct _idlist
{
	long	   id;
	SymbolList *varlist;
	struct _idlist *next, *parent;
} IdList;


	/*-------------------------------------*
	|      global variable declarations
	---------------------------------------*/


extern long	current_id;
extern Symbol   FinalValue;
extern Symbol   *CompiledSymbol;
extern jmp_buf	jump_dest;
extern int	jump_dest_enabled;
extern int	error_found;
extern char	*sptr, *string_array, *kexpr_error_string;
extern int      kexpr_trace;
extern int      kexprDefineMode;


	/*-------------------------------------*
	|	routine definitions
	---------------------------------------*/
 
    /* build.c */
 
void kexpr_error                  (char *);
SymbolList *kexpr_add_number      (SymbolList *, double);
SymbolList *kexpr_add_operator    (SymbolList *, int, char *, int );
SymbolList *kexpr_add_instruction (SymbolList *, int, char *, int );
SymbolList *kexpr_add_string      (SymbolList *, char *);
SymbolList *kexpr_add_symlist     (SymbolList *, SymbolList *);
SymbolList *kexpr_copy_symlist    (SymbolList *);
 
    /* constant.c */
 
Symbol *kexpr_get_constant (char *);
 
    /* eval.c */
 
int kexpr_eval_symbol  (Symbol *, int, Symbol *);
int kexpr_eval_symlist (SymbolList *, int, Symbol *);
 
    /* function.c */
 
SymbolList *kexpr_add_function (SymbolList *, Symbol *, int);
void kexpr_check_function      (Symbol *, SymbolList *, SymbolList *);
Symbol *kexpr_get_function     (char *);
 
    /* routines.c */

char *kexpr_substitute_exprs  (char *str, long id);
int kexpr_substitute_vars     (char *str, char *result);

    /* utilities.c */
 
IdList *kexpr_get_idlist    (long);
int kexpr_delete_idlist     (long);
Symbol *kexpr_get_symbol    (IdList *, char *, int);
Symbol *kexpr_reget_symbol  (IdList *, Symbol *);
Symbol *kexpr_add_symbol    (IdList *, char *, int, double);
int kexpr_delete_symbol     (long, Symbol *);

#ifdef DEBUG_KEXPR
void kexpr_free_symbol      (Symbol *, int line, char *file);
void kexpr_free_symlist     (SymbolList *, int line, char *file);
#else
void kexpr_free_symbol      (Symbol *);
void kexpr_free_symlist     (SymbolList *);
#endif

int kexpr_eval_expression   (long, char *, char *);
void kexpr_dump_symlist     (char *title, SymbolList *syms);
void kexpr_dump_symbol      (char *title, Symbol *symbol);
void kexpr_printf           (char *format, kvalist);
void kexpr_indent           (int delta);
char *expr_type_name        (int type);
void kexpr_set_define_mode  (int true_or_false);
void kexpr_init_symbol      (Symbol *);

#ifdef DEBUG_KEXPR
char *kexpr_strdup          (char *, int line, char *file);
void kexpr_strfree          (char *, int line, char *file);
void kexpr_assign_string    (Symbol *to, Symbol *from, int line, char *file);
void kexpr_assign_cstring   (Symbol *to, char *from, int line, char *file);
void kexpr_assign_double    (Symbol *, double dval, int line, char *file);
void kexpr_reset_value      (Symbol *sym, int line, char *file);
void kexpr_reset_symbol     (Symbol *sym, int line, char *file);
void kexpr_copy_value       (Symbol *to, Symbol *from, int line, char *file);
#else
void kexpr_assign_string    (Symbol *to, Symbol *from);
void kexpr_assign_cstring   (Symbol *to, char *from);
void kexpr_assign_double    (Symbol *sym, double dval);
void kexpr_reset_value      (Symbol *sym);
void kexpr_reset_symbol     (Symbol *sym);
void kexpr_copy_value       (Symbol *to, Symbol *from);
#endif



    /* variable.c */
 
SymbolList *kexpr_add_variable (SymbolList *, Symbol *);
void kexpr_check_variable      (Symbol *);
Symbol *kexpr_get_variable     (char *, int);
Symbol *kexpr_find_variable    (char *, int);

    /* kexpr_lex.l */
void kexpr_set_value        (void);
void kexpr_set_string       (void);

    /* generated from kexpr_yacc.y
This breaks some compilers since it is prototyped differently on different
machines.
*/

#ifdef __cplusplus
int kexpr_parse (void);
#endif

#endif /* _kexpr_internals_h_ */
/* Don't add after this point */
