 /*
  * Khoros: $Id: kexpr_lex.l,v 1.7.2.3 1997/05/24 17:49:12 mirage Exp $
  */

%{

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id: kexpr_lex.l,v 1.7.2.3 1997/05/24 17:49:12 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>      Private Khoros Expression Lexical Analyzer Routines
   >>>>
   >>>>  Private:
   >>>>		     kexpr_set_value()
   >>>>		     kexpr_set_string()
   >>>>
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include "y.tab.h"

#undef output
#define output(charval) (kprintf("unrecognized character: %c\n", charval))

/*
#undef input
#define input() (*sptr++)
 */
#if !defined(__bsdi__) && !defined(FLEX_SCANNER)
#undef  input
#define input()  (*sptr++)
#else
#undef YY_INPUT
#define YY_INPUT(buf, result, max_size) \
(result = (*sptr == '\0') ? 0 : (buf[0] = *sptr++, 1))
#endif
 
#undef  unput
#define unput(c) ((*--sptr)=(char)(c))

extern YYSTYPE yylval;

%}

%o 5000
%a 3000

S	[ \t\n]*
B	[a-zA-Z][a-zA-Z0-9_]*
D	[0-9]
E	[Ee][-+]?{D}+

%%


"#"[^\n]*[\n]  ;  
"/*"	{	     
	   register int c;
	   for (;;)
	   {
	      while ((c = input()) != '*' && c != EOF && c != '\0')
				;
	      if (c == '*')
	      {
	         if ((c = input()) == '/')
		    break;
	      }
	      else
		 break;
	   }
	}

\"      {
           char buf[256], *ptr, c;
           int done = 0, escaped = 0;

           ptr = buf;
	   *ptr = 0;
           while (!done  &&  ((ptr - buf) < sizeof(buf) - 2))
           {
              c = input();
              switch(c)
              {
                 case '\0':
                 case EOF:
                    done = 1;
                    break;
                 case '\"':
		    if (escaped)
                    {
		       *(ptr++) = c;
                       escaped = 0;
                    }
                    else
                       done = 1;
                    break;
                 case '$':
                    if (escaped)
                    {
                       *(ptr++) = '\\';
                       escaped = 0;
                    }
                    *(ptr++) = c;
                    break;
                 case '\\':
                    if (escaped)
                    {
                       *(ptr++) = c;
                       escaped = 0;
                    }
                    else
                    {
                       escaped = 1;
                    }
                    break;
                 default:
                    if (escaped)
                    {
                       switch(c)
                       {
                          case 'n':  *(ptr++) = '\n';  break;
                          case 'r':  *(ptr++) = '\r';  break;
                          case 't':  *(ptr++) = '\t';  break;
                          case '\"': *(ptr++) = c;  break;
                          default:   *(ptr++) = c;  break;
                       }
                       escaped = 0;
                    }
                    else *(ptr++) = c;
                    break;
              }
           }
           *ptr = 0;
           yylval.string = kstrdup(buf);
           return STRING;
        }

"++"	{ return(INCREMENT); }
"--"	{ return(DECREMENT); }
"<<"	{ return(SL); }
">>"	{ return(SR); }
"=="	{ return(EQ); }
"!="	{ return(NE); }
"<="	{ return(LE); }
">="	{ return(GE); }
".GT."	{ return(GT); }
".LT."	{ return(LT); }
".EQ."	{ return(EQ); }
".LE."	{ return(LE); }
".GE."	{ return(GE); }
".NE."	{ return(NE); }
"**"	{ return(POW); }
"<"	{ return(LT); }
">"	{ return(GT); }
"("	{ return('('); }
")"	{ return(')'); }
","	{ return(','); }
";"	{ return(';'); }
"="	{ return('='); }
"+"	{ return('+'); }
"-"	{ return('-'); }
"*"	{ return('*'); }
"/"	{ return('/'); }
"~"	{ return('~'); }
"!"	{ return('!'); }
"%"	{ return('%'); }
"&"	{ return('&'); }
"^"	{ return('^'); }
"|"	{ return('|'); }
"&&"	{ return(AND); }
".AND."	{ return(AND); }
"||"	{ return(OR); }
".OR."	{ return(OR); }
"+="	{ return(AADD); }
"-="	{ return(ASUB); }
"*="	{ return(AMUL); }
"/="	{ return(ADIV); }
"|="	{ return(AOR); }
"&="	{ return(AAND); }
"^="	{ return(AXOR); }
"<<="	{ return(ASL); }
">>="	{ return(ASR); }
".="    { return(ACAT); }
":"	{ return(':'); }
"?"	{ return('?'); }
"IF"	{ return(IF); }
"if"	{ return(IF); }
"THEN"	{ return(THEN); }
"then"	{ return(THEN); }
"ELSE"	{ return(ELSE); }
"else"	{ return(ELSE); }
""	{ return('\0'); }


{D}+		  |
{D}+"."{D}*({E})? |
{D}*"."{D}+({E})? |
{D}+{E} {
	   yylval.value = atof((kstring)yytext);
	   return(NUMBER);
	}

{B}/{S}"=" {
	   yylval.symbol = kexpr_get_variable((kstring)yytext, TRUE);
	   return(VARIABLE);
	}

"eval"/{S}"(" {
	   return(EVALUATE);
	}

{B}/"(" {
	   yylval.symbol = kexpr_get_function((kstring)yytext);
	   return(FUNCTION);
	}

{B} {
	   yylval.symbol = kexpr_get_constant((kstring)yytext);
	   if (yylval.symbol->type == UNDEFINED ||
	       yylval.symbol->type == EXPRESSION)
	   {
	      return(VARIABLE);
	   }
	   else
	   {
	      return(yylval.symbol->type);
	   }
	}

"("{S}"int"{S}")" {
	   return(CINT);
	}

"("{S}"float"{S}")" {
	   return(CFLOAT);
	}

"("{S}"string"{S}")" {
	   return(CSTRING);
	}

{S}		 ;	/* skip blanks and tabs */
"."	return(STRCAT);

%%

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_set_string - set string rules
|
|       Purpose: kexpr_set_string() - indicates that the lex string
|		 manipulation rules should be used instead of the
|		 numerical value lex rules.
|
|         Input: None
|
|        Output: None, except that all future calls to parsing will
|		 use the string manipulation lex rules.
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
| Modifications: PH 4/1/97 - these don't do anything anymore
|
------------------------------------------------------------*/


void kexpr_set_string(void)
{
   return;
}

void kexpr_set_value(void)
{
   return;
}


