 /*
  * Khoros: $Id: kexpr_yacc.y,v 1.11.2.5 1997/05/27 22:11:31 mirage Exp $
  */

%{
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id: kexpr_yacc.y,v 1.11.2.5 1997/05/27 22:11:31 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>    Private Khoros Expression Grammar (yacc) Routines
   >>>>
   >>>>  Private:
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
static int        num_args;

#ifdef __cplusplus
extern "C" {
#endif
int kexpr_lex(void);
static SymbolList *make_num_assignment_def(double val);
static SymbolList *make_str_assignment_def(char *val);
static void create_constant_definition(Symbol *sym, Symbol *val);
static SymbolList *do_binary_op(SymbolList *expr1, SymbolList *expr2,
				int op, char *opname);
#ifdef __cplusplus
}
#endif

%}

%union {
	double     value;
	char       *string;
	Symbol     *symbol;
	SymbolList *deflist;
}

%token  <string>   STRING PRINTF
%token  <value>    NUMBER OPERATOR UOPERATOR INSTRUCTION
%token  <symbol>   VARIABLE EXPRESSION FUNCTION UFUNCTION EVALUATE
		   CONSTANT UNDEFINED

%type   <deflist>  expr sexpr arglist var_or_const
%type	<symbol>   assign

%right 	';' '?' ':' '=' AADD ASUB AMUL ADIV AOR AAND AXOR ASL ASR STRCAT
	IF THEN ELSE ACAT
%left	OR
%left	AND
%left	NE LT LE GT GE EQ INCREMENT DECREMENT
%left 	'+' '-'
%left	'*' '/'
%left	'|' '&' '%' '^' SL SR
%left	UMINUS CINT CFLOAT CSTRING '~' '!'
%right 	POW
%%

top:    stmts  stmt
        | stmts lstmt
        ;

stmts:  stmts stmt
	| stmts error
	  {
	     yyerrok;
	  }
        |
	;

stmt:   list_types ';'
        | ';'
        ;

lstmt:  list_types ;

list_types:
	expr
	  {
	     kexpr_eval_symlist($1, TRUE, &FinalValue);
	     FREE_SYMLIST($1);
	  }
	| assign
	  {
	     if ($1->type != UFUNCTION) kexpr_check_variable($1);
	     kexpr_eval_symbol($1, FALSE, &FinalValue);
	     CompiledSymbol = $1;
	  }
	;

assign:	  VARIABLE '=' expr
	  {
	     kexpr_eval_symlist($3, TRUE, &FinalValue);
	     FREE_SYMLIST($3);
	     if (kstrcmp($1->name, "kexpr_debug") == 0)
	     {
		 yydebug = (FinalValue.Value > 0.01  ||
			    FinalValue.Value < -0.01);
		 kprintf("yydebug is %s\n", yydebug ? "on" : "off");
	     }
	     if (kstrcmp($1->name, "kexpr_trace") == 0)
	     {
		 kexpr_trace = (FinalValue.Value > 0.01  ||
				FinalValue.Value < -0.01);
		 kprintf("Tracing is %s\n", kexpr_trace ? "on" : "off");
	     }
	     RESET_SYMBOL($1);
	     create_constant_definition($1, &FinalValue);
	     if (FinalValue.type == STRING)
	     {
		SymbolList *symlist;
		$1->type = STRING;
		symlist = make_str_assignment_def(STRING_VALUE(&FinalValue));
		$1->deflist = symlist;
		ASSIGN_STRING($1, &FinalValue);
	     }
	     else
	     {
		$1->deflist = make_num_assignment_def(FinalValue.Value);
		COPY_VALUE($1, &FinalValue);
	     }
	     $$ = $1;
	  }
	| VARIABLE ':' '=' expr
	  {
	     RESET_SYMBOL($1);
	     $1->deflist = $4;
	     kfree_and_NULL($1->definition);
	     $1->definition = kstrdup(string_array);
	     $$ = $1;
	  }
	| VARIABLE ':' expr '=' expr
	  {
	     RESET_SYMBOL($1);
	     kexpr_eval_symlist($3, TRUE, &FinalValue);
	     FREE_SYMLIST($3);
	     COPY_VALUE($1, &FinalValue);
	     /* $$->Value = $1->Value = FinalValue.Value; */
	     FREE_SYMBOL($1);
	     $1->deflist = $5;
	     kfree_and_NULL($1->definition);
	     $1->definition = kstrdup(string_array);
	     $$ = $1;
	  }
	| VARIABLE '=' assign
	  {
	     RESET_SYMBOL($1);
	     $$->deflist = $1->deflist = kexpr_copy_symlist($3->deflist);
	     kexpr_check_variable($3);
	     kexpr_eval_symbol($3, FALSE, &FinalValue);
	     kfree_and_NULL($1->definition);
	     $1->definition = kstrdup(string_array);
	  }
	| FUNCTION '(' arglist ')' '=' expr
	  {
	     char error[KLENGTH];

	     /*
	      *  Don't allow the user to redefine a predefined function,
	      *  such as cos().  Only allow them to change or add user
	      *  defined functions.
	      */
	     if ($1->type == FUNCTION)
	     {
		ksprintf(error,"Error!  '%s' is pre-defined function not a "
			"user-defined one.\nYou are only allowed to define "
			"or change user defined functions.", $1->name);
		kexpr_error(error);
	     }

	     /*
	      *  Change the type to be a user defined function.  Also, since
	      *  this is a user defined function we need to change the function
	      *  argument variables.
	      *
	      *  (see kexpr_check_function() for more details).
	      */
	     FREE_SYMBOL($1);
	     $1->type = UFUNCTION;
	     kexpr_check_function($1, $3, $6);
	     $$->deflist = $1->deflist = kexpr_add_symlist($3, $6);
	     kfree_and_NULL($1->definition);
	     $1->definition = kstrdup(string_array);
	     $$ = $1;
	  }
	| VARIABLE AADD expr
	  {
	     double val = 0.0;
	     Symbol temp;

	     kexpr_init_symbol(&temp);
	     
	     if ($1->type == UNDEFINED)
	     {
		kexpr_delete_symbol(current_id, $1);
	        kexpr_error("Undefined variable");
	     }
             else if (!HAS_DOUBLE($1))
	     {
	        kexpr_error("Non-numeric variable");
	     }
	     else
	     {
		 kexpr_eval_symlist($3, TRUE, &temp);
		 FREE_SYMLIST($3);
		 if (temp.type == NUMBER)
		 {
		     val = $1->Value + temp.Value;
		 }
		 FREE_SYMBOL($1);
		 $$->Value = $1->Value = val;
		 $$->deflist = $1->deflist = make_num_assignment_def(val);
	     }
	  }
	| VARIABLE ASUB expr
	  {
	     double val = 0.0;
	     Symbol temp;

	     kexpr_init_symbol(&temp);
	     
	     if ($1->type == UNDEFINED)
	     {
		kexpr_delete_symbol(current_id, $1);
	        kexpr_error("Undefined variable");
	     }
             else if (!HAS_DOUBLE($1))
	     {
	        kexpr_error("Non-numeric variable");
	     }
	     else
	     {
		 kexpr_eval_symlist($3, TRUE, &temp);
		 FREE_SYMLIST($3);
		 if (temp.type == NUMBER)
		 {
		     val = $1->Value - temp.Value;
		 }
		 FREE_SYMBOL($1);
		 $$->Value = $1->Value = val;
		 $$->deflist = $1->deflist = make_num_assignment_def(val);
	     }
	  }
	| VARIABLE INCREMENT
	  {
	     if ($1->type == UNDEFINED)
	     {
		kexpr_delete_symbol(current_id, $1);
	        kexpr_error("Undefined variable");
	     }
             else if (!HAS_DOUBLE($1))
	     {
	        kexpr_error("Non-numeric variable");
	     }
	     else
	     {
		 double val = $1->Value + 1;
		 $$->Value = $1->Value = val;
		 $$->deflist = $1->deflist = make_num_assignment_def(val);
	     }
	  }
	| VARIABLE DECREMENT
	  {
	     if ($1->type == UNDEFINED)
	     {
		kexpr_delete_symbol(current_id, $1);
	        kexpr_error("Undefined variable");
	     }
             else if (!HAS_DOUBLE($1))
	     {
	        kexpr_error("Non-numeric variable");
	     }
	     else
	     {
		 double val = $1->Value - 1;
		 $$->Value = $1->Value = val;
		 $$->deflist = $1->deflist = make_num_assignment_def(val);
	     }
	  }
	| VARIABLE AMUL expr
	  {
	     double val = 1.0;
	     Symbol temp;

	     kexpr_init_symbol(&temp);
	     
	     if ($1->type == UNDEFINED)
	     {
		kexpr_delete_symbol(current_id, $1);
	        kexpr_error("Undefined variable");
	     }
             else if (!HAS_DOUBLE($1))
	     {
	        kexpr_error("Non-numeric variable");
	     }
	     else
	     {
		 kexpr_eval_symlist($3, TRUE, &temp);
		 FREE_SYMLIST($3);
		 if (temp.type == NUMBER)
		 {
		     val = $1->Value * temp.Value;
		 }
		 FREE_SYMBOL($1);
		 $$->Value = $1->Value = val;
		 $$->deflist = $1->deflist = make_num_assignment_def(val);
	     }
	  }
	| VARIABLE ADIV expr
	  {
	     double val = $1->Value;
	     Symbol temp;

	     kexpr_init_symbol(&temp);
	     
	     if ($1->type == UNDEFINED)
	     {
		kexpr_delete_symbol(current_id, $1);
	        kexpr_error("Undefined variable");
	     }
             else if (!HAS_DOUBLE($1))
	     {
	        kexpr_error("Non-numeric variable");
	     }
	     else
	     {
		 kexpr_eval_symlist($3, TRUE, &temp);
		 FREE_SYMLIST($3);
		 if (temp.type == NUMBER)
		 {
		     if (temp.Value == 0.0)
			 kexpr_error("Attempt to divide by zero");
		     else
			 val = $1->Value / temp.Value;
		 }
		 FREE_SYMBOL($1);
		 $$->Value = $1->Value = val;
		 $$->deflist = $1->deflist = make_num_assignment_def(val);
	     }
	  }
	| VARIABLE AOR expr
	  {
	     double val = $1->Value;
	     Symbol temp;

	     kexpr_init_symbol(&temp);
	     
	     if ($1->type == UNDEFINED)
	     {
		kexpr_delete_symbol(current_id, $1);
	        kexpr_error("Undefined variable");
	     }
             else if (!HAS_DOUBLE($1))
	     {
	        kexpr_error("Non-numeric variable");
	     }
	     else
	     {
		 kexpr_eval_symlist($3, TRUE, &temp);
		 FREE_SYMLIST($3);
		 if (temp.type == NUMBER)
		 {
		     val = (long)$1->Value | (long)temp.Value;
		 }
		 FREE_SYMBOL($1);
		 $$->Value = $1->Value = val;
		 $$->deflist = $1->deflist = make_num_assignment_def(val);
	     }
	  }
	| VARIABLE AAND expr
	  {
	     double val = $1->Value;
	     Symbol temp;

	     kexpr_init_symbol(&temp);
	     
	     if ($1->type == UNDEFINED)
	     {
		kexpr_delete_symbol(current_id, $1);
	        kexpr_error("Undefined variable");
	     }
             else if (!HAS_DOUBLE($1))
	     {
	        kexpr_error("Non-numeric variable");
	     }
	     else
	     {
		 kexpr_eval_symlist($3, TRUE, &temp);
		 FREE_SYMLIST($3);
		 if (temp.type == NUMBER)
		 {
		     val = (long)$1->Value & (long)temp.Value;
		 }
		 FREE_SYMBOL($1);
		 $$->Value = $1->Value = val;
		 $$->deflist = $1->deflist = make_num_assignment_def(val);
	     }
	  }
	| VARIABLE AXOR expr
	  {
	     double val = $1->Value;
	     Symbol temp;

	     kexpr_init_symbol(&temp);
	     
	     if ($1->type == UNDEFINED)
	     {
		kexpr_delete_symbol(current_id, $1);
	        kexpr_error("Undefined variable");
	     }
             else if (!HAS_DOUBLE($1))
	     {
	        kexpr_error("Non-numeric variable");
	     }
	     else
	     {
		 kexpr_eval_symlist($3, TRUE, &temp);
		 FREE_SYMLIST($3);
		 if (temp.type == NUMBER)
		 {
		     val = (long)$1->Value ^ (long)temp.Value;
		 }
		 FREE_SYMBOL($1);
		 $$->Value = $1->Value = val;
		 $$->deflist = $1->deflist = make_num_assignment_def(val);
	     }
	  }
	| VARIABLE ASL expr
	  {
	     double val = $1->Value;
	     Symbol temp;

	     kexpr_init_symbol(&temp);
	     
	     if ($1->type == UNDEFINED)
	     {
		kexpr_delete_symbol(current_id, $1);
	        kexpr_error("Undefined variable");
	     }
             else if (!HAS_DOUBLE($1))
	     {
	        kexpr_error("Non-numeric variable");
	     }
	     else
	     {
		 kexpr_eval_symlist($3, TRUE, &temp);
		 FREE_SYMLIST($3);
		 if (temp.type == NUMBER)
		 {
		     val = (long)$1->Value << (int)temp.Value;
		 }
		 FREE_SYMBOL($1);
		 $$->Value = $1->Value = val;
		 $$->deflist = $1->deflist = make_num_assignment_def(val);
	     }
	  }
	| VARIABLE ASR expr
	  {
	     double val = $1->Value;
	     Symbol temp;

	     kexpr_init_symbol(&temp);
	     
	     if ($1->type == UNDEFINED)
	     {
		kexpr_delete_symbol(current_id, $1);
	        kexpr_error("Undefined variable");
	     }
             else if (!HAS_DOUBLE($1))
	     {
	        kexpr_error("Non-numeric variable");
	     }
	     else
	     {
		 kexpr_eval_symlist($3, TRUE, &temp);
		 FREE_SYMLIST($3);
		 if (temp.type == NUMBER)
		 {
		     val = (long)$1->Value >> (int)temp.Value;
		 }
		 FREE_SYMBOL($1);
		 $$->Value = $1->Value = val;
		 $$->deflist = $1->deflist = make_num_assignment_def(val);
	     }
	  }
	| VARIABLE ACAT expr
	  {
	     char *val1, *val2, *newval;
	     char buf[64];
	     Symbol temp;
	     int free_val1, free_val2;

	     kexpr_init_symbol(&temp);

	     if ($1->type == UNDEFINED)
	     {
		kexpr_delete_symbol(current_id, $1);
	        kexpr_error("Undefined variable");
	     }
	     else
	     {
		 /* first get the variable's string value */
		 if ($1->exprType != STRING)
		 {
		    /* we're assuming it must be a NUMBER */
		    ksprintf(buf, "%g", $1->Value);
		    val1 = kstrdup(buf);
		    free_val1 = TRUE;
		 }
		 else
		 {
		    val1 = STRING_VALUE($1);
		    free_val1 = FALSE;
		 }
		 
		 /* evaluate expr and point exprval at the result */
		 kexpr_eval_symlist($3, TRUE, &temp);
		 FREE_SYMLIST($3);

		 if (temp.type != STRING)
		 {
		     ksprintf(buf, "%g", temp.Value);
		     val2 = kstrdup(buf);
		     free_val2 = TRUE;
		 }
		 else
		 {
		     val2 = STRING_VALUE(&temp);
		     free_val2 = FALSE;
		 }
		 /* build the concatenated string */
		 newval = kstring_cat(val1, val2, NULL);

		 /* free the original strings */
		 RESET_SYMBOL($1);
		 if (free_val1) kfree(val1);
		 if (free_val2) kfree(val2);
		 
		 RESET_SYMBOL(&temp);
		 
		 /* put the concatenated result back in VARIABLE */
		 /* $1->String = newval; */
		 ASSIGN_CSTRING($1, newval);
		 $1->deflist = make_str_assignment_def(newval);
		 $$ = $1;
		 kfree(newval);
	     }
	  }
	;

sexpr:  sexpr STRING
          {
             $$ = kexpr_add_operator(NULL, STRCAT, "", OPERATOR);
             $$ = kexpr_add_symlist($$, $1);
             $$ = kexpr_add_string($$, $2);
	     kfree($2);
          }
        | STRING
          {
	     $$ = kexpr_add_string(NULL, $1);
	     kfree($1);
	  }
        ;

var_or_const: VARIABLE
	  {
	     $$ = kexpr_add_variable(NULL, $1);
	  }
	| CONSTANT
	  {
	     if ($1->type == UNDEFINED)
	     {
		kexpr_delete_symbol(current_id, $1);
		kexpr_error("Undefined constant");
	     }
	     $$ = kexpr_add_variable(NULL, $1);
	  }
	;
	  
expr:     var_or_const
	| NUMBER
	  {
	     $$ = kexpr_add_number(NULL, $1);
	  }
	| NUMBER var_or_const %prec '*'
	  {
	     $$ = kexpr_add_operator(NULL, '*', "*", OPERATOR);
	     $$ = kexpr_add_number($$, $1);
	     $$ = kexpr_add_symlist($$, $2);
	  }
        | sexpr
          {
	     $$ = $1;
	  }
	| IF '(' expr ')' expr ELSE expr
	  {
	     $$ = kexpr_add_instruction(NULL, IF, "if", 3);
	     $$ = kexpr_add_symlist($$, $3);
	     $$ = kexpr_add_symlist($$, $5);
	     $$ = kexpr_add_symlist($$, $7);
	  }
	| expr '?' expr ':' expr
	  {
	     $$ = kexpr_add_instruction(NULL, IF, "if", 3);
	     $$ = kexpr_add_symlist($$, $1);
	     $$ = kexpr_add_symlist($$, $3);
	     $$ = kexpr_add_symlist($$, $5);
	  }
	| EVALUATE '(' expr ')'
	  {
	     Symbol temp;

	     kexpr_init_symbol(&temp);

	     kexpr_eval_symlist($3, TRUE, &temp);
	     FREE_SYMLIST($3);
	     if (temp.type == NUMBER)
	        $$ = kexpr_add_number(NULL, temp.Value);
	     else if (temp.type == STRING)
		$$ = kexpr_add_string(NULL, STRING_VALUE(&temp));
	     else
		kexpr_error("Can only evaluate numerical or string expressions");
	  }
	| FUNCTION '(' arglist ')'
	  {
	     $$ = kexpr_add_function(NULL, $1, num_args);
	     $$ = kexpr_add_symlist($$, $3);
	  }
	| expr AND expr
	  {
	     $$ = do_binary_op($1, $3, AND, "&&");
	  }
	| expr OR  expr
	  {
	     $$ = do_binary_op($1, $3, OR, "||");
	  }
	| expr EQ  expr
	  {
	     $$ = do_binary_op($1, $3, EQ, "==");
	  }
	| expr NE  expr
	  {
	     $$ = do_binary_op($1, $3, NE, "!=");
	  }
	| expr LE  expr
	  {
	     $$ = do_binary_op($1, $3, LE, "<=");
	  }
	| expr GE  expr
	  {
	     $$ = do_binary_op($1, $3, GE, ">=");
	  }
	| expr LT  expr
	  {
	     $$ = do_binary_op($1, $3, LT, "<");
	  }
	| expr GT  expr
	  {
	     $$ = do_binary_op($1, $3, GT, ">");
	  }
	| expr '+' expr
	  {
	     $$ = do_binary_op($1, $3, '+', "+");
	  }
	| expr '-' expr
	  {
	     $$ = do_binary_op($1, $3, '-', "-");
	  }
	| expr '*' expr
	  {
	     $$ = do_binary_op($1, $3, '*', "*");
	  }
	| expr '/' expr
	  {
	     $$ = do_binary_op($1, $3, '/', "/");
	  }
	| expr POW expr
	  {
	     $$ = do_binary_op($1, $3, POW, "**");
	  }
	| expr '%' expr
	  {
	     $$ = do_binary_op($1, $3, '%', "%");
	  }
	| expr '&' expr
	  {
	     $$ = do_binary_op($1, $3, '&', "&");
	  }
	| expr '|' expr
	  {
	     $$ = do_binary_op($1, $3, '|', "|");
	  }
	| expr '^' expr
	  {
	  }
	| expr SL expr
	  {
	     $$ = do_binary_op($1, $3, SL, "<<");
	  }
	| expr SR expr
	  {
	     $$ = do_binary_op($1, $3, SR, ">>");
	  }
	| '~' expr
	  {
	     $$ = kexpr_add_operator(NULL, '~', "~", UOPERATOR);
	     $$ = kexpr_add_symlist($$, $2);
	  }
	| CINT expr
	  {
	     $$ = kexpr_add_operator(NULL, CINT, "(int)", UOPERATOR);
	     $$ = kexpr_add_symlist($$, $2);
	  }
	| CFLOAT expr
	  {
	     $$ = kexpr_add_operator(NULL, CFLOAT, "(float)", UOPERATOR);
	     $$ = kexpr_add_symlist($$, $2);
	  }
	| CSTRING expr
	  {
	     $$ = kexpr_add_operator(NULL, CSTRING, "(string)", UOPERATOR);
	     $$ = kexpr_add_symlist($$, $2);
	  }
	| '!' expr
	  {
	     $$ = kexpr_add_operator(NULL, '!', "!", UOPERATOR);
	     $$ = kexpr_add_symlist($$, $2);
	  }
	| '(' expr ')'
	  {
	     $$ = $2;
	  }
	| '-' expr %prec UMINUS
	  {
	     $$ = kexpr_add_operator(NULL, UMINUS, "-", UOPERATOR);
	     $$ = kexpr_add_symlist($$, $2);
	  }
	| expr STRCAT  expr
	  {
	     $$ = do_binary_op($1, $3, STRCAT, ".");
	  }
	;

arglist:  {
	     num_args = 0;
	     $$ = NULL;
	  }

	| expr
	  {
	     num_args = 1;
	     $$ = kexpr_add_symlist(NULL, $1);
	  }

	| expr ',' arglist
	  {
	     num_args += 1;
	     $$ = kexpr_add_symlist(NULL, $1);
	     $$ = kexpr_add_symlist($$, $3);
	  }
	;

%%



/*-----------------------------------------------------------
|
|  Routine Name: do_binary_op
|
|       Purpose: do_binary_op builds up a SymbolList for a
|		 binary operator
|
|         Input: expr1 - expression on left side
|                expr2 - expression on right side
|                op - operator id
|                opname - operator name string
|
|        Output: none
|
|        Return: returns a symbol list on success, NULL otherwise
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
static SymbolList *do_binary_op(SymbolList *expr1, SymbolList *expr2,
				int op, char *opname)
{
   SymbolList *result;

   result = kexpr_add_operator(NULL, op, opname, OPERATOR);
   result = kexpr_add_symlist(result, expr1);
   result = kexpr_add_symlist(result, expr2);
   return result;
}


/*-----------------------------------------------------------
|
|  Routine Name: make_num_assignment_def
|
|       Purpose: make_num_assignment_def creates a deflist for
|		 a numeric assignment expression.
|
|         Input: val - value of expression
|
|        Output:
|
|        Return: returns the symbol list on success, NULL otherwise
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
static SymbolList *make_num_assignment_def(double val)
{
    SymbolList *result;

    result = kexpr_add_number(NULL, val);
    return result;
}

/*-----------------------------------------------------------
|
|  Routine Name: make_str_assignment_def
|
|       Purpose: make_str_assignment_def creates a deflist for
|		 a string assignment expression.
|
|         Input: str - value of expression
|
|        Output:
|
|        Return: returns the symbol list on success, NULL otherwise
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
static SymbolList *make_str_assignment_def(char *str)
{
    SymbolList *result;
    char buf[KLENGTH];

    kexpr_substitute_vars(str, buf);
    result = kexpr_add_string(NULL, buf);
    return result;
}

/*-----------------------------------------------------------
|
|  Routine Name: create_constant_definition
|
|       Purpose: create_constant_definition creates the
|		 definition string for an expression.  This
|                only happens if the current definition is
|                NULL or kexprDefineMode is TRUE.
|
|         Input: sym - the symbol representing the expression
|                val - a symbol containing the value of the expr
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
static void create_constant_definition(Symbol *sym, Symbol *val)
{
    char buf[KLENGTH];

    if (sym->definition == NULL  ||  kexprDefineMode != FALSE)
    {
       if (val->type == NUMBER)
	  ksprintf(buf, "%s = %g", sym->name, val->Value);
       else if (val->type == STRING)
	  ksprintf(buf, "%s = \"%s\"", sym->name, STRING_VALUE(val));
    
       kfree_and_NULL(sym->definition);
       sym->definition = kstrdup(buf);
    }
}

