/*
  * Khoros: $Id: routines.c,v 1.13.2.6 1997/05/24 17:49:15 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: routines.c,v 1.13.2.6 1997/05/24 17:49:15 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Public Khoros Expression Routines
   >>>>		
   >>>>   Public:
   >>>>                 kexpr_evaluate_generic()
   >>>>                 kexpr_eval()
   >>>>                 kexpr_substitute_vars()
   >>>>                 kexpr_substitute_exprs()
   >>>>                 kexpr_compile_function()
   >>>>                 kexpr_execute_function()
   >>>>                 kexpr_free_function()
   >>>>                 kexpr_parent_set()
   >>>>                 kexpr_parent_unset()
   >>>>                 kexpr_variables_list()
   >>>>                 kexpr_variables_def()
   >>>>                 kexpr_variables_reset()
   >>>>                 kexpr_variables_print()
   >>>>                 kexpr_variables_copy()
   >>>>                 kexpr_variables_clear()
   >>>>                 kexpr_variable_delete()
   >>>>                 kexpr_variable_add()
   >>>>   Static:
   >>>>                 eval_embedded_var()
   >>>>                 eval_embedded_expr()
   >>>>                 locate_expr()
   >>>>                 translate_filename()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include "y.tab.h"


/*  flag to control whether assignments change a variable's
    definition.
*/
int kexprDefineMode = FALSE;

/*-----------------------------------------------------------
|
|  Routine Name: translate_filename
|
|       Purpose: translate_filename translates filename-style
|                input from xvlang (e.g. "file.$1") into a
|                form that kexpr_parse will like.  This is
|                for compatibility with current cantata
|                practices.
|
|         Input: in - input string
|
|        Output: none
|
|        Return: a translate string - caller must free this value.
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
static char *translate_filename(char *in)
{
   char buf[1024], *ptr;
   int in_quote = 0;

   ptr = buf;
   while (*in)
   {
      switch(*in)
      {
	 case '\"':
	    in_quote = !in_quote;
	    *(ptr++) = *in;
	    break;
	 case '.':
	 case '-':
	 case ',':
	    if (!in_quote)
	    {
	       if (ptr > buf)
	       {
		  kstrcpy(ptr, " . ");
		  ptr += 3;
	       }
	       *(ptr++) = '\"';
	       in_quote = 1;
	    }
	    *(ptr++) = *in;
	    break;
	 case '$':
	    if (!in_quote)
	    {
	       if (ptr > buf)
	       {
		  kstrcpy(ptr, " . ");
		  ptr += 3;
	       }
	       *(ptr++) = '\"';
	       in_quote = 1;
	    }
	    *(ptr++) = *in;
	    break;
	 default:
	    if (!in_quote)
	    {
	       if (ptr > buf)
	       {
		  kstrcpy(ptr, " . ");
		  ptr += 3;
	       }
	       *(ptr++) = '\"';
	       in_quote = 1;
	    }
	    *(ptr++) = *in;
	    break;
      }
      in++;
      if ((ptr - buf) > 1000) break;
   }
   if (in_quote) *(ptr++) = '\"';
   *ptr = 0;
   return kstrdup(buf);
}



/*-----------------------------------------------------------
|
|  Routine Name: locate_expr
|
|       Purpose: locate_expr finds the next embedded expression
|                in a string (e.g. "file.$1").
|
|         Input: str - input string
|
|        Output: exprlen - length of next embedded expression
|
|        Return: pointer to start of next embedded expression
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
static char *locate_expr(char *str, int *exprlen)
{
   char *ptr;
   int len = 0;

   /* find the first dollar sign */
   ptr = kstrchr(str, '$');
   if (ptr)
   {
      if (ptr > str  &&  *(ptr-1) == '\\')
      {
	 ptr--;
	 len = 2;
      }
      else
      {
	 char *ptr2 = ptr+1;
	 if (*ptr2 == '{')
	 {
	    ptr2++;
	    while(*ptr2  &&  *ptr2 != '}') ptr2++;
	    len = ptr2 - ptr;
	    if (*ptr2 == '}') len++;
	 }
	 else
	 {
	    ptr2++;
	    len = 2;
	    while (*ptr2  &&  (*ptr2 == '_'  ||  isalnum(*ptr2)))
	    {
	       ptr2++;
	       len++;
	    }
	 }
      }
   }
   *exprlen = len;
   return ptr;
}

/*-----------------------------------------------------------
|
|  Routine Name: eval_embedded_expr
|
|       Purpose: eval_embedded_expr evaluates an variable ref
|                that is embedded in a string.  It is a macho
|                version of eval_embedded_var that will
|                handle whole expressions.
|
|         Input: expr - expression string to evaluate
|                exprlen - length of expr (it is not null-terminated)
|                result_len - length of result buffer
|                id - namespace id
|
|        Output: result - buffer to hold the result of evaluation
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
static void eval_embedded_expr(char *expr, int exprlen,
			       char *result, int result_len, long id)
{
   char *ptr, *envvar;
   char buf[KLENGTH], format[KLENGTH], errstr[KLENGTH], tresult[KLENGTH];
   char exprsave[KLENGTH];
   int  have_format = 0;

   /* save the expression */
   kstrncpy(exprsave, expr, (size_t)exprlen);
   exprsave[exprlen] = 0;
   
   /* check for escaped dollar sign */
   if (*expr == '\\')
   {
      *result = *(expr+1);
      *(result+1) = 0;
      return;
   }
   
   /* account for leading dollar sign and squiggly brackets */
   if (*expr == '$')
   {
      expr++;
      exprlen--;
   }
   if (*expr == '{')
   {
      expr++;
      exprlen -= 2;
   }
   if (exprlen <= 0)
   {
      kstrcpy(result, "ERROR");
      return;
   }

   /* make a local copy of the expression */
   kstrncpy(buf, expr, (size_t)exprlen);
   buf[exprlen] = 0;

   /* look for format spec inside expression */
   ptr = kstrchr(buf, ':');
   if (ptr != NULL)
   {
      /* we have a format string - save it */
      *(ptr++) = 0;
      if (*ptr == '%') ptr++;
      format[0] = '%';
      kstrcpy(format+1, ptr);
      have_format = 1;
   }
   
   /* see if it is an environment variable */
   envvar = kgetenv(buf);
   if (envvar != NULL)
   {
      /* it is an environment variable - put it in FinalValue */
      FREE_SYMBOL(&FinalValue);
      ASSIGN_CSTRING(&FinalValue, envvar);
   }
   /* if not, try to evaluate the expression */
   else if (kexpr_eval_expression(id, buf, errstr) == FALSE)
   {
      /* kprintf(errstr); */
      kstrcpy(result, exprsave);
      return;
   }

   /* format the result value in tresult */
   switch(FinalValue.type)
   {
      case NUMBER:
	 if (!have_format) kstrcpy(format, "%g");
	 if (format[kstrlen(format) - 1] == 'd')
	 {
	    ksprintf(tresult, format, (int)(FinalValue.Value + 0.5));
	 }
	 else
	 {
	    ksprintf(tresult, format, FinalValue.Value);
	 }
	 break;
      case STRING:
	 if (!have_format) kstrcpy(format, "%s");
	 ksprintf(tresult, format, STRING_VALUE(&FinalValue));
	 break;
      default:
	 kstrcpy(tresult, "ERROR");
	 break;
   }

   /* copy the result into caller's buffer */
   if (result_len < kstrlen(tresult))
      kstrcpy(result, "ERROR");
   else
      kstrcpy(result, tresult);
   FREE_SYMBOL(&FinalValue);
}
   
/*-----------------------------------------------------------
|
|  Routine Name: eval_embedded_var
|
|       Purpose: eval_embedded_var evaluates an variable ref
|                that is embedded in a string.  It is a wimpy
|                version of eval_embedded_expr that doesn't
|                handle whole expressions.  It is used when
|                we are in the expression parser and can't
|                call ourselves recursively.
|
|         Input: expr - expression string to evaluate
|                exprlen - length of expr (it is not null-terminated)
|                result_len - length of result buffer
|                id - namespace id
|
|        Output: result - buffer to hold the result of evaluation
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
static void eval_embedded_var(char *expr, int exprlen,
			       char *result, int result_len)
{
   char *ptr, *envvar;
   char buf[KLENGTH], format[KLENGTH], tresult[KLENGTH];
   char exprsave[KLENGTH];
   int  have_format = 0;
   Symbol *tsym;
   Symbol value;

   kexpr_init_symbol(&value);

   /* save the expression */
   kstrncpy(exprsave, expr, (size_t)exprlen);
   exprsave[exprlen] = 0;
   
   /* check for escaped dollar sign */
   if (*expr == '\\')
   {
      *result = *(expr+1);
      *(result+1) = 0;
      return;
   }
   
   /* account for leading dollar sign and squiggly brackets */
   if (*expr == '$')
   {
      expr++;
      exprlen--;
   }
   if (*expr == '{')
   {
      expr++;
      exprlen -= 2;
   }
   if (exprlen <= 0)
   {
      kstrcpy(result, "ERROR");
      return;
   }

   /* make a local copy of the expression */
   kstrncpy(buf, expr, (size_t)exprlen);
   buf[exprlen] = 0;

   /* look for format spec inside expression */
   ptr = kstrchr(buf, ':');
   if (ptr != NULL)
   {
      /* we have a format string - save it */
      *(ptr++) = 0;
      if (*ptr == '%') ptr++;
      format[0] = '%';
      kstrcpy(format+1, ptr);
      have_format = 1;
   }

   /* check for an environment variable */
   envvar = kgetenv(buf);
   if (envvar != NULL)
   {
      /* it is an environment variable - put it in value */
      ASSIGN_CSTRING(&value, envvar);
   }
   else
   {
      /* look the variable up */
      tsym = kexpr_find_variable(buf, TRUE);
      if (tsym == NULL)
      {
	 kstrcpy(result, exprsave);
	 return;
      }
      /* format the result value in tresult */
      kexpr_eval_symlist(tsym->deflist, TRUE, &value);
   }

   switch(value.type)
   {
      case NUMBER:
	 if (!have_format) kstrcpy(format, "%g");
	 if (format[kstrlen(format) - 1] == 'd')
	 {
	    ksprintf(tresult, format, (int)(value.Value + 0.5));
	 }
	 else
	 {
	    ksprintf(tresult, format, value.Value);
	 }
	 break;
      case STRING:
	 if (!have_format) kstrcpy(format, "%s");
	 ksprintf(tresult, format, STRING_VALUE(&value));
	 break;
      default:
	 kstrcpy(tresult, "ERROR");
	 break;
   }

   /* copy the result into caller's buffer */
   if (result_len < kstrlen(tresult))
      kstrcpy(result, "ERROR");
   else
      kstrcpy(result, tresult);
   FREE_SYMBOL(&value);
}
   
/*************************************************************
*
*  Routine Name: kexpr_substitute_exprs
*
*       Purpose: kexpr_substitute_exprs evaluates all embedded
*                expressions within a string.  These are
*                of the form, $name, ${name}, or ${name:fmt}.
*
*         Input: str - string to process
*                id -  variable id.  Use KEXPR_GLOBAL_ID to use
*                      the global variable context.
*
*        Output: none
*
*       Returns: result of substituting.  This string should be
*                freed by the caller.
*
*  Restrictions:
*    Written By: Pete Humphrey
*          Date: April 1, 1997
*      Examples: result = kexpr_substitute_exprs("file.${i:04d}", KEXPR_GLOBAL_ID)
*
**************************************************************/

char *kexpr_substitute_exprs(char *str, long id)
{
   char *result = NULL, *ptr, *tstr;
   char expr[256];
   int len;
   char *orig = kstrdup(str);	/* make we aren't re-using FinalValue's string */

   str = orig;
   while (str != NULL &&  *str != 0)
   {
      ptr = locate_expr(str, &len);
      if (ptr != NULL)
      {
	 int tl = ptr - str;
	 if (tl > 0)
	 {
	    kstrncpy(expr, str, (size_t)tl);
	    expr[tl] = 0;
	    tstr = result;
	    result = kstring_cat(result, expr, NULL);
	    kfree(tstr);
	 }
	 eval_embedded_expr(ptr, len, expr, (int)sizeof(expr), id);
	 tstr = result;
	 result = kstring_cat(result, expr, NULL);
	 kfree(tstr);
	 str = ptr + len;
      }
      else
      {
	 tstr = result;
	 result = kstring_cat(result, str, NULL);
	 kfree(tstr);
	 str = NULL;
      }
   }
   kfree(orig);
   return result;
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_substitute_vars
|
|       Purpose: kexpr_substitute_vars finds all embedded
|                variable references within a string and replaces
|                them with the variable's value.  These are
|                of the form, $name, ${name}, or ${name:fmt}.
|
|         Input: str - string to process
|
|        Output: result - result of substituting
|
|        Return: 1 for success, 0 for failure
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
int kexpr_substitute_vars(char *str, char *result)
{
   char *ptr, *optr;
   char expr[256];
   int len;

   optr = result;
   while (str != NULL &&  *str != 0)
   {
      ptr = locate_expr(str, &len);
      if (ptr != NULL)
      {
	 int tl = ptr - str;
	 if (tl > 0)
	 {
	    kstrncpy(optr, str, (size_t)tl);
	    optr += tl;
	    *optr = 0;
	 }
	 eval_embedded_var(ptr, len, expr, (int)sizeof(expr));
	 kstrcpy(optr, expr);
	 optr += kstrlen(optr);
	 str = ptr + len;
      }
      else
      {
	 kstrcpy(optr, str);
	 str = NULL;
      }
   }
   kexpr_printf("kexpr_substitute_vars: result (%p) is \"%s\"\n",
		result, result);
   return 1;
}


/****************************************************************************
*
*  Routine Name: kexpr_eval - evaluate an expression
*
*       Purpose: This routine evaluates an input expression and 
*		 stores the resulting value in a KexprResult structure.
*                Currently, the result can be one of two types, KSTRING
*                or KDOUBLE.  The result type is stored in result->type.
*                The result value is stored in result->string_value
*                 or result->double_value.
*
*         Input: id     - the variable identifier.  This specifies the
*                         variable context to use.  Use KEXPR_GLOBAL_ID
*                         to use the global variable context.
*                string - the string to be evaluated.
*
*        Output: result - holds the result of the evaluation.  If this
*                         argument is NULL, a new KexprResult structure will
*                         be allocated.  
*                         The result can be one of two types, KSTRING
*                         or KDOUBLE.  The result type is stored in
*                         result->type. The result value is stored in
*                         the string_value or double_value members of result.
*                         Use the function kexpr_delete_result
*                         to free a KexprResult structure.  If the result is
*                         a string, this function will allocate or
*                         re-allocate space in the
*                         KexprResult structure to hold it, if necessary.
*		 error  - If an error occurs the error message is stored in
*			  the error string and FALSE is returned.  The
*			  error string must be an array of at least 
*			  1024 characters.  It is allocated by the calling
*                         routine.  If
*			  the error string has not been allocated by the
*			  calling routine (error is passed in as NULL) then the
*			  error message is output with the kerror facility.
*
*	Returns: TRUE (1) on success, FALSE (0) otherwise
*
*      Examples: res = kexpr_eval(KEXPR_GLOBAL_ID, "x = 25", NULL, NULL);
*
*                res = kexpr_eval(KEXPR_GLOBAL_ID, "ln(x)", res, myerror);
*  Restrictions: 
*    Written By: Pete Humphrey
*          Date: April 1, 1997
*      Verified:
*  Side Effects: Variable values may be changed by assignment statements.
* Modifications:
*
*****************************************************************************/

KexprResult *kexpr_eval(
   long        id,
   char        *string,
   KexprResult *result,
   char        *error)
{
   char *tmp;
   int status;

   if ((tmp = kstring_cleanup(string, NULL)) != NULL)
   {
      status = kexpr_eval_expression(id, tmp, error);
      kfree(tmp);
   }
   else
      status = FALSE;
   
   
   if (result == NULL)
   {
      result = (KexprResult *) kmalloc(sizeof(KexprResult));
      if (result != NULL)
      {
	 result->string_value = NULL;
         result->strlength = 0;
      }
   }

   if (result == NULL)
      return result;

   result->status = status;

   if (status)
   {   
      if (FinalValue.type == STRING)
      {
	 result->type = KSTRING;
	 if (kstrchr(STRING_VALUE(&FinalValue), '$') == NULL)
	 {
	    /* no expression substitution is necessary */
	    if (result->strlength < kstrlen(STRING_VALUE(&FinalValue)))
	    {
	       /* we have to allocate more room for the string */
	       kfree(result->string_value);
	       result->string_value = kstrdup(STRING_VALUE(&FinalValue));
	       result->strlength = kstrlen(result->string_value);
	    }
	    else
	    {
	       kstrcpy(result->string_value, STRING_VALUE(&FinalValue));
	    }
	 }
	 else
	 {
	    char *newstr;
	    kfree(result->string_value);
	    newstr = kexpr_substitute_exprs(STRING_VALUE(&FinalValue), id);
	    result->string_value = newstr;
	    result->strlength = kstrlen(newstr);
	 }
      }
      else
      {
	 result->double_value = FinalValue.Value;
	 result->type = KDOUBLE;
      }
   }
   return(result);
}



/**********************************************************************
*
*  Routine Name: kexpr_evaluate_generic - evaluate an expression
*                and return result using the desired data type
*
*       Purpose: This routine evaluates the input string and returns
*		 a generic value of the expression according to the
*		 desired data type.  See the kexpr_evaluate_{data type}
*		 for specific information about evaluation of an expression
*		 for the desired data type.
*
*         Input: id     - the variable identifier.
*                string - the string to be evaluated.
*                type   - the data type for the value pointer.  Valid data
*			  types are: KBYTE, KUBYTE, KSHORT, KUSHORT, KINT,
*			  KUINT, KLONG, KULONG, KFLOAT, KDOUBLE, KCOMPLEX,
*			  KDCOMPLEX, KSTRING.
*
*        Output: value  - if no error occurred then the value of
*			  the expression is stored and True returned.
*		 error  - if an error occurred the error message is stored in
*			  the error string array and False returned.  The
*			  error string array must be at least a 1024 string
*			  array that is allocated by the calling routine.  If
*			  the error string array has not been allocated by the
*			  calling routine (error is passed in as NULL) then the
*			  error message is output with the kerror facility.
*
*	Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: 
*    Written By: Mark Young  
*          Date: Thu Jun 25 1992
*      Verified:
*  Side Effects: Variable values may be changed by assignment statements.
* Modifications: 
*
****************************************************************************/

int kexpr_evaluate_generic(
   long  id,
   char  *string,
   int   type,
   kaddr value,
   char  *error)
{
	char *tmp;
	int status;
	int quoted;


	if ((tmp = kstring_cleanup(string, NULL)) != NULL)
	{
	   char *expr;

	   quoted = (*tmp == '\"');
	   if (kstrchr(tmp, '$') == NULL)
	      expr = tmp;
	   else
	      expr = translate_filename(tmp);

	   status = kexpr_eval_expression(id, expr, error);
	   if (tmp != expr)
	      kfree(expr);
	}
	else
	   status = FALSE;

	kfree(tmp);
	if (value == NULL)
	{
	   return(status);
	}
	else if (FinalValue.type == STRING)
	{
	   if (type == KSTRING)
	   {
	      char *result = (char *)value;
	      
	      if (quoted)
		 *(result++) = '\"';
	      tmp = kexpr_substitute_exprs(STRING_VALUE(&FinalValue), id);
	      kstrcpy(result, tmp);
	      kfree(tmp);
	      if (quoted)
		 kstrcat(result, "\"");
	      return(status);
	   }
	   else
	      return 0;
	}

	switch (type)
	{
           case KBYTE:
		*((char *) value) = (char) FinalValue.Value;
		break;

           case KUBYTE:
		*((unsigned char *) value) = (unsigned char) FinalValue.Value;
		break;

           case KSHORT:
		*((short *) value) = (short) FinalValue.Value;
		break;

           case KUSHORT:
		*((unsigned short *) value) = (unsigned short) 
				FinalValue.Value;
		break;

           case KLONG:
		*((long *) value) = (long) FinalValue.Value;
		break;

           case KULONG:
		*((unsigned long *) value) = (unsigned char) FinalValue.Value;
		break;

           case KINT:
		*((int *) value) = (int) FinalValue.Value;
		break;

           case KUINT:
		*((unsigned int *) value) = (unsigned int) FinalValue.Value;
		break;

           case KFLOAT:
		*((float *) value) = (float) FinalValue.Value;
		break;

           case KDOUBLE:
		*((double *) value) = (double) FinalValue.Value;
		break;

           case KCOMPLEX:
		break;

           case KDCOMPLEX:
		break;

           case KSTRING:
		ksprintf((kstring) value, "%g", FinalValue.Value);
		break;

           default:
		return(FALSE);
	}
	return(status);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_compile_function - compile a function
|
|       Purpose: This routine compiles a function into it's corresponding
|		 symbol list.  The symbol list pointer is passed back and
|		 can be used in conjunction with the kexpr_execute_function()
|		 routine.
|         Input: id     - the variable identifier.
|                string - the string to be evaluated.
|        Output: error  - if an error occurred the error message is stored in
|			  the error string array and False returned.  The
|			  error string array must be at least a 1024 string
|			  array that is allocated by the calling routine.  If
|			  the error string array has not been allocated by the
|			  calling routine (error is passed in as NULL) then the
|			  error message is output with the kerror facility.
|
|	Returns: kaddr pointing to a SymbolList for the function, or NULL 
|  Restrictions: 
|    Written By: Mark Young  
|          Date: Dec 31, 1994
|      Verified:
|  Side Effects:
| Modifications: 03/20/97 (PH): documented return value correctly
|
--------------------------------------------------------------*/

kaddr kexpr_compile_function(
   long  id,
   char  *string,
   char  *error)
{
	int        i;
	Symbol	   *symbol;
	SymbolList *symlist = NULL, *arglist = NULL;


	if (string == NULL)
	   return(NULL);

	kexpr_set_value();
	if (!kexpr_eval_expression(id, string, error))
	   return(NULL);

	/*
	 *  Since we have the function (specified by symbol), go ahead and
	 *  add it to the function list.
	 */
	symbol = CompiledSymbol;
	symlist = kexpr_add_function(symlist, symbol, symbol->num);

	/*
	 *  Next we build our expression list.
	 */
	for (i = 0; i < (int) symbol->num; i++)
           arglist = kexpr_add_number(arglist, 0.0);

        symlist = kexpr_add_symlist(symlist, arglist);
	return(symlist);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_execute_function - execute a previous compiled function
|
|       Purpose: This routine executes a previous compiled function in order
|		 to avoid the overhead of repeated calls to the yyparse().
|         Input: function - the variable identifier.
|                type     - the data type for the value pointer.
|        Output: value    - pointer to the data pointer.
|	Returns: TRUE (1) on success, FALSE (0) otherwise
|
|  Restrictions: 
|    Written By: Mark Young  
|          Date: Dec 31, 1994
|      Verified:
|  Side Effects:
| Modifications:
|
-------------------------------------------------------------*/

int kexpr_execute_function(
   kaddr function,
   int   type,
   kaddr value,
   char  *error,
   kvalist)
{
	kva_list   list;
	double     number, result;
	Symbol     *symbol;
	int        i, status;
	SymbolList *symlist, *arglist;


	/*
	 *  Sanity check...
	 */
	if (!function)
	   return(FALSE);

	/*
	 *  Get the function (which is really a symbol list pointer).
	 */
	symlist = (SymbolList *) function;
	symbol =  symlist->symbol;
	arglist = symlist->next;

	kva_start(list, error);
	for (i = 0; i < (int) symbol->num; i++)
	{
	   number = kva_arg(list, double);
	   arglist->symbol->Value = number;
	   arglist = arglist->next;
	}
	string_array = NULL;
	kexpr_error_string = error;
	status = kexpr_eval_symlist(symlist, TRUE, &FinalValue);

	if (value == NULL)
	   return(status);

	if (FinalValue.type == STRING)
	{
	   result = 0.0;
	   ksscanf(STRING_VALUE(&FinalValue), "%lg", &result);
	}
	else
	{
	   result = FinalValue.Value;
	}

	if (type == KDOUBLE)
	{
	   *((double *) value) = (double) result;
	   return(status);
	}

	switch (type)
	{
           case KBYTE:
		*((char *) value) = (char) result;
		break;

           case KUBYTE:
		*((unsigned char *) value) = (unsigned char) result;
		break;

           case KSHORT:
		*((short *) value) = (short) result;
		break;

           case KUSHORT:
		*((unsigned short *) value) = (unsigned short) 
				result;
		break;

           case KLONG:
		*((long *) value) = (long) result;
		break;

           case KULONG:
		*((unsigned long *) value) = (unsigned char) result;
		break;

           case KINT:
		*((int *) value) = (int) result;
		break;

           case KUINT:
		*((unsigned int *) value) = (unsigned int) result;
		break;

           case KFLOAT:
		*((float *) value) = (float) result;
		break;

           case KDOUBLE:
		*((double *) value) = (double) result;
		break;

           case KCOMPLEX:
		break;

           case KDCOMPLEX:
		break;

           case KSTRING:
		ksprintf((kstring) value, "%g", result);
		break;

           default:
		return(FALSE);
	}
	return(status);
}

/************************************************************
*
*  Routine Name: kexpr_free_function - frees up a previous compiled function
*
*       Purpose: This routine frees the resources associated with a previous
*		 compiled function in order to avoid the overhead of repeated
*		 calls to the kexpr_compile_function().
*         Input: function - the variable identifier.
*                type     - the data type for the value pointer.
*        Output: value    - pointer to the data pointer.
*	Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: 
*    Written By: Mark Young  
*          Date: Dec 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

int kexpr_free_function(
   kaddr function)
{
	FREE_SYMLIST((SymbolList *) function);
	return(TRUE);
}

/************************************************************
*
*  Routine Name: kexpr_parent_set() - set the parent variable list associated
*                                     with an id
*
*       Purpose: This routine sets the child alternate variable list
*		 specified by the parent id.  This variable list
*		 will be searched when a variable is not found in
*		 the current variable list.  If the parent's variable
*		 list "parent" is set then the procedure is repeated
*		 until either the variable is found or no more
*		 variable lists are available to search.  In short, this
*		 routine provides a method of expanding the scope
*		 of variables used.
*
*        Input:  id   - the id variable (for the child) in which to set the
*			parent variable list.
*		 parent - the id variable for the parent.
*
*       Output:  
*	Returns: TRUE (1) if the child's parent variable list was set to that
*		 of the parent, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young  
*          Date: Thu Jun 25 1992
*      Verified:
*  Side Effects:
* Modifications:
*
************************************************************/

int kexpr_parent_set(
   long id,
   long parent)
{
	IdList *idlist, *parlist;


	/*
	 *  Get the child's variable list.
	 */
	if ((idlist = kexpr_get_idlist(id)) == NULL)
	{
	   kfprintf(kstderr,"Unable to get list for child id (%d)\n", id);
	   return(FALSE);
	}

	/*
	 *  Get the parents's variable list.
	 */
	if ((parlist = kexpr_get_idlist(parent)) == NULL)
	{
	   kfprintf(kstderr,"Unable to get list for parent id (%d)\n", parent);
	   return(FALSE);
	}

	/*
	 *  Set the child's parent list to be that of the parent.
	 */
	idlist->parent = parlist;
	return(TRUE);
}

/************************************************************
*
*  Routine Name: kexpr_parent_unset() -  unset the parent variable list
*                                        associated with an id
*
*       Purpose: This routine unsets the parent variable list previously
*		 specified by kexpr_parent_set().  The result of a call
*		 to this routine is that if a variable is not found in the
*		 variable list associated with the kexpr identifier, then
*		 no other variable list will be checked.  In short, this
*		 routine provides a method of forcing variables used to
*		 be "local".
*
*        Input:  id   - the id variable (for the child) in which
*			to unset the parent variable list.
*	Returns: TRUE (1) if the child's parent list was successfully set to
*		 NULL, FALSE (0) otherwise.
*
*  Restrictions:
*    Written By: Mark Young  
*          Date: Thu Jun 25 1992
*      Verified:
*  Side Effects:
* Modifications:
*
************************************************************/

int kexpr_parent_unset(
   long id)
{
	IdList *idlist;


	/*
	 *  Get the child's variable list.
	 */
	if ((idlist = kexpr_get_idlist(id)) == NULL)
	{
	   kfprintf(kstderr,"Unable to get list for child id (%d)\n", id);
	   return(FALSE);
	}

	/*
	 *  Set the child's parent list to be NULL
	 */
	idlist->parent = NULL;
	return(TRUE);
}

/************************************************************
*
*  Routine Name: kexpr_variables_list() - list variables associated with an id
*
*       Purpose: This routines extracts the current list of variables for a
*		 particular id variable and returns a pointer to the list of
*		 variables.
*
*        Input:  id   - the id variable whose list of current variables will
*			be extracted.
*		 mode - list mode of what to be included in the list of
*			variables.
*	Output:	 num  - number of variables in list or NULL if not needed
*      Returns:  the list of variables or NULL upon failure
*
*  Restrictions:
*    Written By: Mark Young  
*          Date: Dec 13, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
************************************************************/

char **kexpr_variables_list(
   long id,
   int  mode,
   int  *num)
{
	IdList    *idlist;
	Symbol     *symbol;
	SymbolList *symlist;

	Symbol value;
	size_t size = 0;
	char   temp[KLENGTH], **list = NULL;

	kexpr_init_symbol(&value);

	/*
	 * Check to see if it is a current variable list.
	 */
	if ((idlist = kexpr_get_idlist(id)) != NULL)
	{
	   symlist = idlist->varlist;
	   while (symlist != NULL)
	   {
	      symbol = symlist->symbol;
	      if ((symbol->type == UNDEFINED) || (symbol->idlist->id != id))
	      {
		 symlist = symlist->next;
		 continue;
	      }

	      if (mode == 1)
	      {
		 kexpr_eval_symbol(symbol, FALSE, &value);
		 if (value.type == STRING)
		 {
		    ksprintf(temp, "%s = \"%s\"",
			     symbol->name, STRING_VALUE(&value));
		 }
		 else
		 {
		    ksprintf(temp, "%s = %g", symbol->name, value.Value);
		 }
		 FREE_SYMBOL(&value);
	      }
	      else if (mode == 0)
	         kstrcpy(temp, symbol->name);

	      list = (char **)karray_insert(list, KSTRING, kstrdup(temp), 
				   size++, KLIST_HEAD, TRUE);
	      symlist = symlist->next;
	   }
	}

	if (num) *num = (int) size;
	return(list);
}

/************************************************************
*
*  Routine Name: kexpr_variables_def() - list variable definitions 
*				         associated with an id
*
*       Purpose: This routine extracts the current list of variable
*		 definitions for a particular id variable and returns
*		 a pointer to the list of variable definitions.
*
*        Input:  id   - the id whose list of current variables will
*                       be extracted.
*       Output:  num  - number of variables in list or NULL if not needed
*      Returns:  the list of variable definitions or NULL upon failure
*
*  Restrictions:
*    Written By: Mark Young, John Salas & Danielle Argiro
*          Date: July 13, 1995
*      Verified:
*  Side Effects:
* Modifications:
*
************************************************************/

char **kexpr_variables_def(
   long id,
   int  *num)
{
        IdList     *idlist;
        Symbol     *symbol;
        SymbolList *symlist;
	size_t      size = 0;
	char      **list = NULL;

        /*
         * Check to see if it is a current variable list.
         */

	*num = 0;
        if ((idlist = kexpr_get_idlist(id)) != NULL)
        {
           symlist = idlist->varlist;
           if (symlist == NULL)
              return(NULL);

           while (symlist != NULL)
           {
              symbol = symlist->symbol;
	      if ((symbol->definition != NULL) && (symbol->idlist->id == id))
	      {
	         list = (char **) karray_insert(list, KSTRING,
						kstrdup(symbol->definition),
						size++, KLIST_HEAD, TRUE);
	      }
              symlist = symlist->next;
           }
        }
        else return(NULL);

	*num = (int) size;
        return(list);
}

/************************************************************
*
*  Routine Name: kexpr_variables_reset() - reset variable values 
*				         to original definitions
*
*       Purpose: This routine resets the current list of variables
*		 to the values implied by their original definitions.
*
*        Input:  id   - the id whose list of current variables will
*                       be reset.
*       Output:
*      Returns:  TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Danielle Argiro, Mark Young & John Salas
*          Date: July 25, 1995
*      Verified:
*  Side Effects:
* Modifications:
*
************************************************************/

int kexpr_variables_reset(
   long id)
{
	char       *tmp;
        Symbol     *symbol;
        IdList     *idlist;
        SymbolList *symlist;

        /*
         * Check to see if it is a current variable list.
         */
        if ((idlist = kexpr_get_idlist(id)) == NULL)
	   return(FALSE);

        symlist = idlist->varlist;
        if (symlist == NULL)
           return(TRUE);

        while (symlist != NULL)
        {
           symbol = symlist->symbol;
	   if (symbol->definition != NULL)
	   {
	      tmp = kstrdup(symbol->definition);
	      kexpr_evaluate_generic(id, tmp, KDOUBLE, NULL, NULL);
	      kfree(tmp);
	   }
           symlist = symlist->next;
        }
        return(TRUE);
}


/************************************************************
*
*  Routine Name: kexpr_variables_print() - prints variables associated with
*                                         an id
*
*       Purpose: This routine prints the current list of variables
*		 for a particular id into a file that has been opened
*		 for writing.
*
*        Input:  id   - the list of id from which we will list 
*			current variables.
*		 file - the kfile pointer in which we will print
*			this information to
*
*       Output:  returns the TRUE or FALSE if we were able
*		 to list the workspace.
*
*	Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Mark Young  
*          Date: Thu Jun 25 1992
*      Verified:
*  Side Effects:
* Modifications:
*
************************************************************/

int kexpr_variables_print(
   long id,
   kfile *file)
{
	IdList    *idlist;
	Symbol     *symbol;
	SymbolList *symlist;


	/*
	 * Check to see if it is a current variable list.
	 */
	kfprintf(file,"\n\n");
	if ((idlist = kexpr_get_idlist(id)) != NULL)
	{
	   symlist = idlist->varlist;
	   if (symlist == NULL)
	   {
	      kfprintf(file,"\n# No variables currently declared. #\n\n");
	      return(TRUE);
	   }

	   kfprintf(file,"#\n# list of currently declared variables\n#\n\n");
	   while (symlist != NULL)
	   {
	      symbol = symlist->symbol;
	      if (symbol->definition != NULL)
	          kfprintf(file, "%s\n", symbol->definition);
	      symlist = symlist->next;
	   }
	   kfprintf(file,"\n");
	}
	else
	{
	   kfprintf(file,"\n\n# Error! Unable to retrieve expressions. #\n\n");
	   return(FALSE);
	}
	return(TRUE);
}

/************************************************************
*
*  Routine Name: kexpr_variables_copy() - copies variables from one id to
*                                         another
*
*       Purpose: This routine copies the current list of variables
*		 from one particular id to another.
*
*        Input:  from   - the id variable whose list of current viariables
*			  will be copied the id.
*		 to     - the id variable to recieve the copied list of
*			  variables.
*
*       Output:
*	Returns: returns TRUE (1) if the list of variables copied successfully,
*		 or FALSE (0) if the list of variables was not copied.
*
*  Restrictions:
*    Written By: Mark Young  
*          Date: Thu Jun 25 1992
*      Verified:
*  Side Effects:
* Modifications:
*
************************************************************/


int kexpr_variables_copy(
   long from,
   long to)
{
/*
	IdList	   *fromlist, *tolist;
	Symbol	   *symbol;
	SymbolList *symlist;


	if ((fromlist = kexpr_get_idlist(from)) == NULL)
	{
	}

	if ((tolist = kexpr_get_idlist(to)) == NULL)
	{
	}
 */
	to = from;
	return(TRUE);
}

/************************************************************
*
*  Routine Name: kexpr_variables_clear() - deletes variables associated with
*                                          an id
*
*       Purpose: This routine deletes the current list of variables
*		 for a particular id.  The id is also removed.
*
*	  Input: id - the id the variable to be to be cleared.
*	 Output:
*	Returns: returns TRUE (1) if the list of variables were cleared
*		 successfully, or FALSE (0) if the list of variables were not
*		 cleared.
*  Restrictions:
*    Written By: Mark Young  
*          Date: Thu Jun 25 1992
*      Verified:
*  Side Effects:
* Modifications:
*
************************************************************/

int kexpr_variables_clear(
   long id)
{
	return(kexpr_delete_idlist(id));
}

/************************************************************
*
*  Routine Name: kexpr_variable_delete() - delete a variable associated
*                                          with an id
*
*       Purpose: This routine deletes a variable from the current list of
*	         variables for a particular id.  kexpr_variable_delete()
*		 deletes the specified variable from the id's variable list.
*		 a particular id.
*
*         Input: id - the id variable whose list of variables will have one
*		      variable deleted.
*		 variable - the variable to delete from the list of variables.
*        Output:
*	Returns: returns TRUE (1) if the variable was deleted from the list
*		 of variables or FALSE (0) if the variable was not deleted from
*		 the list of variables or was not in the list of variables.
*
*  Restrictions:
*    Written By: Mark Young  
*          Date: Dec 13, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
************************************************************/

int kexpr_variable_delete(
   long id,
   char *variable)
{
	Symbol  *symbol;
	IdList *idlist;

	/*
	 * Check to see if it is in the id idlist list.
	 */
	if ((idlist = kexpr_get_idlist(id)) == NULL)
	   return(FALSE);

	/*
	 * Check to see if it is in the id idlist list.
	 */
	if ((symbol = kexpr_get_symbol(idlist, variable, FALSE)) == NULL)
	   return(FALSE);

	return(kexpr_delete_symbol(id, symbol));
}

/************************************************************
*
*  Routine Name: kexpr_variable_add() - add a variable associated
*                                          with an id
*
*       Purpose: This routine adds a variable to the current list of
*                variables for a particular id. 
*
*         Input: id       - the id to which to add the variable
*                variable - the variable to add to the list of variables.
*        Output:
*       Returns: returns TRUE (1) if the variable was added to the list
*                of variables or FALSE (0) if the variable was not added to
*                the list of variables
*  Restrictions:
*    Written By: Mark Young
*          Date: Dec 13, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
************************************************************/

int kexpr_variable_add(
   long id,
   char *variable)
{
        Symbol *symbol;
        IdList *idlist;

        /*
         * Check to see if it is in the id idlist list.
         */
        if ((idlist = kexpr_get_idlist(id)) == NULL)
           return(FALSE);

        /*
         * Check to see if it is in the id idlist list.
         */
        if ((symbol = kexpr_get_symbol(idlist, variable, FALSE)) != NULL)
           return(TRUE);

        symbol = kexpr_add_symbol(idlist, variable, UNDEFINED, 0.0);
	if (symbol == NULL)
	    return(FALSE);
	else return(TRUE);

}


/*********************************************************************
*
*  Routine Name: kexpr_delete_result - delete a KexprResult struct
*
*       Purpose: This routine frees a KexprResult structure allocated
*                by the kexpr_eval() function.
*
*         Input: result - pointer to the structure to free
*
*        Output: none
*
*	Returns: none
*
*  Restrictions: 
*    Written By: Pete Humphrey
*          Date: April 7, 1997
*      Verified:
*  Side Effects:
* Modifications:
*
********************************************************************/

void kexpr_delete_result(KexprResult *result)
{
   if (result->strlength > 0)
      kfree(result->string_value);
   kfree(result);
}


/*********************************************************************
*
*  Routine Name: kexpr_set_define_mode - turn define mode on or off.
*
*       Purpose: This routine sets a flag that determines whether an
*                assignment to a variable will affect its definition.
*                When set to FALSE, the variable's definition is not
*                changed, unless this is the first assignment to the
*                variable.  When set to TRUE, the variable's definition
*                is updated.
*
*         Input: flag - TRUE or FALSE
*
*        Output: none
*
*	Returns: none
*
*  Restrictions: 
*    Written By: Pete Humphrey
*          Date: April 7, 1997
*      Verified:
*  Side Effects:
* Modifications:
*
********************************************************************/

void kexpr_set_define_mode(int tf)
{
   kexprDefineMode = tf;
}
