/*
  * Khoros: $Id: utilities.c,v 1.9.2.8 1997/05/24 17:49:17 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.9.2.8 1997/05/24 17:49:17 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Various Private Utility Routines For the kexpr Library
   >>>>		
   >>>>  Private:
   >>>>                 kexpr_get_idlist()
   >>>>                 kexpr_delete_idlist()
   >>>>                 kexpr_get_symbol()
   >>>>                 kexpr_reget_symbol()
   >>>>                 kexpr_add_symbol()
   >>>>                 kexpr_delete_symbol()
   >>>>                 kexpr_free_symbol()
   >>>>                 kexpr_free_symlist()
   >>>>                 kexpr_eval_expression()
   >>>>                 expr_type_name()
   >>>>                 kexpr_printf()
   >>>>                 kexpr_dump_symlist()
   >>>>                 kexpr_dump_symbol()
   >>>>                 kexpr_indent()
   >>>>                 kexpr_assign_double()
   >>>>                 kexpr_assign_cstring()
   >>>>                 kexpr_assign_string()
   >>>>                 kexpr_reset_value()
   >>>>                 kexpr_reset_symbol()
   >>>>                 kexpr_copy_value()
   >>>>  Static:
   >>>>                 do_indent()
   >>>>                 
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include "y.tab.h"

static IdList  *var_head = NULL;
static int ilevel;		/* indent level for kexpr_printf */
int kexpr_trace = FALSE;	/* turns tracing on or off */


long	current_id;
jmp_buf	jump_dest;
int    	error_found = FALSE;
int    	jump_dest_enabled = FALSE;
Symbol  FinalValue, *CompiledSymbol;
char	*sptr, *string_array, *kexpr_error_string;

int kexpr_parse(void);
	
static char *kexpr_symname(Symbol *sym)
{
   if (sym == &FinalValue) return "FinalValue";
   else if (sym->name == NULL) return "anon";
   else return sym->name;
}
	   

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_get_idlist - get the idlist
|
|       Purpose: kexpr_get_idlist() gets the idlist associated
|		 for a particular id.  If no idlist exists for the
|		 id then a list is created and added to the "var_head".
|
|         Input: id - the variable identifier
|
|        Output: RETURNS: returns the idlist on success, NULL otherwise
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
|
------------------------------------------------------------*/
IdList *kexpr_get_idlist(
   long id)
{
	IdList *idlist;


	idlist = var_head;
	while (idlist != NULL)
	{
	   if (idlist->id == id)
	      break;

	   idlist = idlist->next;
	}

	if (idlist == NULL)
	{
	   if ((idlist = (IdList *) kcalloc((size_t) 1, sizeof(IdList))) == NULL)
	      return(NULL);

	   idlist->id	    = id;
	   idlist->varlist = NULL;
	   idlist->parent  = NULL;
	   idlist->next    = var_head;
	   var_head         = idlist;
	}
	return(idlist);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_delete_idlist - delete the idlist
|
|       Purpose: kexpr_delete_idlist() deletes the idlist associated
|		 with a particular id.
|
|         Input: id - the variable identifier
|
|        Output: RETURNS: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
|
------------------------------------------------------------*/
int kexpr_delete_idlist(
   long id)
{
	IdList *idlist, *list;


	list = idlist = var_head;
	while (list != NULL)
	{
	   if (list->id == id)
	      break;

	   idlist = list;
	   list = list->next;
	}

	/*
	 *  If we don't find the id in the current list then just return
	 */
	if (list == NULL)
	   return(FALSE);

	/*
	 *  If the variable list is at the head then advance 'var_head' to the
	 *  next variable list, otherwise we just link around the variable list.
	 */
	if (idlist == list)
	   var_head = list->next;
	else
	   idlist->next = list->next;

	/*
	 *  Make sure that none of the other variable list use the deleted
	 *  idlist as a parent.
	 */
	idlist = var_head;
	while (idlist != NULL)
	{
	   if (idlist->parent == list)
	      idlist->parent = NULL;

	   idlist = idlist->next;
	}
	FREE_SYMLIST(list->varlist);
	kfree_and_NULL(list);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_get_symbol - get a symbol
|
|       Purpose: kexpr_get_symbol() gets the symbol associated with
|		 a particular idlist and name identifier.  If this
|		 list does not contain the particular symbol and
|		 the idlist is a child then the kexpr_get_symbol()
|		 is recursed with the parent's idlist.
|
|         Input: idlist - the list of variable list from which
|			   we search the symbol
|		 name    - the name of the symbol we are looking
|			   for.
|		 parent  - whether we should be allowed to search
|			   the parent's idlist if one exists.
|
|        Output: RETURNS: returns the symbol on success, NULL otherwise
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
|
------------------------------------------------------------*/
Symbol *kexpr_get_symbol(
   IdList *idlist,
   char    *name,
   int     parent)
{
	Symbol	   *symbol;
	SymbolList *symlist;


	/*
	 *  Race through the symbol list looking to see if we can find the
	 *  symbol.
	 */
	symlist = idlist->varlist;
	while (symlist != NULL)
	{
	   symbol = symlist->symbol;
	   if (kstrcmp(name, symbol->name) == 0)
	      return(symbol);

	   symlist = symlist->next;
	}

	/*
	 *  If the list doesn't have the symbol then try list's parent
	 *  if one exists.
	 */
	if (idlist->parent != NULL && parent == TRUE)
	   symbol = kexpr_get_symbol(idlist->parent, name, parent);
	else
	   symbol = NULL;

	return(symbol);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_reget_symbol - reget a symbol
|
|       Purpose: When a symbol is undefined, we may need to search
|		 the parent or global list to see if the symbol is
|		 defined elsewhere.  This mechanism allows a lazy
|		 creation and evaluation of expressions.
|
|         Input: idlist - the list of variable list from which
|			   we search the symbol
|		 oldsymbol  - the old symbol we are looking to replace.
|        Output:
|	Returns: returns the new symbol on success, NULL otherwise
|
|    Written By: Mark Young
|          Date: Jul 27, 1995
|
------------------------------------------------------------*/
Symbol *kexpr_reget_symbol(
   IdList *idlist,
   Symbol  *oldsymbol)
{
	Symbol	   *symbol;
	SymbolList *symlist, *tmplist;

	/*
	 *  See if the symbol is defined elsewhere
	 */
	if (idlist->parent != NULL)
	   symbol = kexpr_get_symbol(idlist->parent, oldsymbol->name, TRUE);
	else
	   symbol = NULL;

	/* if no other symbol exists then return NULL */
	if (symbol == NULL)
	   return(NULL);

	/*
	 *  Race through all the idlist's (not just the one in which symbol
	 *  is defined) and replace the old symbol with the new one.
	 */
	idlist = var_head;
	while (idlist != NULL)
	{
	   symlist = idlist->varlist;
	   while (symlist != NULL)
	   {
	      if (symlist->symbol == oldsymbol)
	         symlist->symbol = symbol;

	      if (symlist->symbol != NULL)
	      {
	         tmplist = symlist->symbol->deflist;
	         while (tmplist != NULL)
	         {
	            if (tmplist->symbol == oldsymbol)
	               tmplist->symbol = symbol;

	            tmplist = tmplist->next;
	         }
	      }
	      symlist = symlist->next;
	   }
	   idlist = idlist->next;
	}
	kfree_and_NULL(oldsymbol);
	return(symbol);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_add_symbol - add a symbol to a idlist
|
|       Purpose: kexpr_add_symbol() adds the symbol associated
|		 with a particular idlist and name identifier.
|
|         Input: idlist - the list of variable list to which
|			   we add the symbol
|		 name    - the name of the symbol we are adding
|                type    - the type of the symbol we are adding
|                value   - the current value of that symbol
|
|        Output: RETURNS: returns the symbol on success, NULL otherwise
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
|
------------------------------------------------------------*/
Symbol *kexpr_add_symbol(
   IdList *idlist,
   char    *name,
   int     type,
   double  value)
{
	Symbol	    *symbol;
	SymbolList  *symlist;


	if ((symlist = (SymbolList *) kcalloc((size_t) 1, sizeof(SymbolList))) == NULL)
	   return(NULL);

	if ((symbol = (Symbol *) kcalloc((size_t)1, sizeof(Symbol))) == NULL)
	   return(NULL);

	kexpr_printf("Added symbol %s at %p\n", name, symbol);
	symbol->name =  kstrdup(name);
	symbol->type =  type;
	symbol->eval =  FALSE;
	symbol->Value   = value;
	symbol->idlist = idlist;

	symlist->symbol  = symbol;
	symlist->next    = idlist->varlist;
	idlist->varlist = symlist;
	return(symbol);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_delete_symbol - delete a symbol from a idlist
|
|       Purpose: kexpr_delete_symbol() deletes the symbol associated
|		 with a particular idlist and name identifier.  If
|		 this list does not contain the particular symbol then
|		 we return false, otherwise we delete it from the idlist
|		 and then the kexpr_get_symbol() is recursed with the
|		 parent's idlist.
|
|         Input: id     - the id from which the symbol came from
|		 symbol - the name of the symbol we are adding
|
|        Output: RETURNS: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
|
------------------------------------------------------------*/
int kexpr_delete_symbol(
   long   id,
   Symbol *symbol)
{
	IdList	   *idlist;
	SymbolList *symlist, *list;

	/*
	 * Check to see if it is in the id idlist list.
	 */
	if ((idlist = kexpr_get_idlist(id)) == NULL)
	   return(FALSE);

	list = symlist = idlist->varlist;
	while (list != NULL)
	{
	   if (list->symbol == symbol)
	      break;

	   symlist = list;
	   list = list->next;
	}

	if (list == NULL)
	   return(FALSE);

	if (symlist == list)
	   idlist->varlist = list->next;
	else
	   symlist->next = list->next;

	list->next = NULL;
	FREE_SYMLIST(list);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_free_symbol - free a symbol
|
|       Purpose: kexpr_free_symbol() is called when we want to free
|		 the contents of a symbol, but not the symbol itself.
|		 We do this before re-using the symbol.
|
|         Input: symbol - the current symbol to be freed
|
|        Output: None
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
|
------------------------------------------------------------*/

#ifdef DEBUG_KEXPR

void kexpr_free_symbol(
   Symbol *symbol,
   int line,
   char *file)
{
	Symbol     temp;
	SymbolList list;


	
	/*
	 *  If the current symbol is NULL then just return
	 */
	if (symbol == NULL)
	   return;

	kexpr_init_symbol(&temp);
	kexpr_printf("%d:%s  kexpr_free_symbol(%p) [%s]\n",
		line, file, symbol, kexpr_symname(symbol));

	/*
	 *  Free all the resources that are allocated with the symbol,
	 *  ie). the symlist and the string (if the symbol is of type
	 *  string).
	 */
	if (symbol->type == STRING && STRING_VALUE(symbol) != NULL)
	{
	   KSTRFREE(symbol->String);
	   symbol->String = NULL;
	}
	else if (symbol->type != UNDEFINED && symbol->deflist != NULL &&
	         (symbol->type == VARIABLE || symbol->type == EXPRESSION))
	{
	   
	   list.symbol = symbol;
	   list.next   = NULL;
	   kexpr_eval_symlist(&list, FALSE, &temp);
	   COPY_VALUE(symbol, &temp);
	   RESET_VALUE(&temp);
	}

	if (symbol->deflist != NULL)
	   FREE_SYMLIST(symbol->deflist);

	symbol->deflist = NULL;
}

#else

void kexpr_free_symbol(
   Symbol *symbol)
{
	Symbol     temp;
	SymbolList list;

	
	/*
	 *  If the current symbol is NULL then just return
	 */
	if (symbol == NULL)
	   return;

	kexpr_init_symbol(&temp);

	/*
	 *  Free all the resources that are allocated with the symbol,
	 *  ie). the symlist and the string (if the symbol is of type
	 *  string).
	 */
	if (symbol->type == STRING && STRING_VALUE(symbol) != NULL)
	{
	   KSTRFREE(symbol->String);
	   symbol->String = NULL;
	}
	else if (symbol->type != UNDEFINED && symbol->deflist != NULL &&
	         (symbol->type == VARIABLE || symbol->type == EXPRESSION))
	{
	   
	   list.symbol = symbol;
	   list.next   = NULL;
	   kexpr_eval_symlist(&list, FALSE, &temp);
	   COPY_VALUE(symbol, &temp);
	   RESET_VALUE(&temp);
	}

	if (symbol->deflist != NULL)
	   FREE_SYMLIST(symbol->deflist);

	symbol->deflist = NULL;
}

#endif

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_free_symlist - free a symbol list
|
|       Purpose: kexpr_free_symlist() is called when we want to free
|		 the contents of a symbol list.
|
|         Input: symbol - the current symbol to be freed
|
|        Output: None
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
|
------------------------------------------------------------*/
#ifdef DEBUG_KEXPR
void kexpr_free_symlist(
   SymbolList *symlist,
   int line,
   char *file)
{
	Symbol	   *symbol;
	SymbolList *temp;

	/*
	 *  Free all the resources that are allocated with the symlist,
	 *  ie). the symbol and the symlist itself
	 */
	while (symlist != NULL)
	{
	   symbol = symlist->symbol;
	   if (symbol->type == NUMBER || symbol->type == STRING ||
	       symbol->type == OPERATOR  ||  symbol->type == UOPERATOR)
	   {
	      FREE_SYMBOL(symbol);
	      kfree_and_NULL(symbol->definition);
	      kfree_and_NULL(symbol->name);
	      kfree_and_NULL(symbol);
	   }
	   temp = symlist->next;
	   kfree_and_NULL(symlist);
	   symlist = temp;
	}
}

#else

void kexpr_free_symlist(
   SymbolList *symlist)
{
	Symbol	   *symbol;
	SymbolList *temp;

	/*
	 *  Free all the resources that are allocated with the symlist,
	 *  ie). the symbol and the symlist itself
	 */
	while (symlist != NULL)
	{
	   symbol = symlist->symbol;
	   if (symbol->type == NUMBER || symbol->type == STRING ||
	       symbol->type == OPERATOR  ||  symbol->type == UOPERATOR)
	   {
	      FREE_SYMBOL(symbol);
	      kfree_and_NULL(symbol->definition);
	      kfree_and_NULL(symbol->name);
	      kfree_and_NULL(symbol);
	   }
	   temp = symlist->next;
	   kfree_and_NULL(symlist);
	   symlist = temp;
	}
}

#endif

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_eval_expression - evaluate an expression
|
|       Purpose: kexpr_eval_expression() evaluates the string
|		 and returns whether we succesfully evaluated
|		 that expression.
|
|         Input: id     - the variable identifier
|		 string - the string to be evaluated
|
|        Output: error  - if an error occurred the message is
|			  stored in the string array and False
|			  returned.  If this field is NULL then
|			  the error message is printed to kstderr.
|		 RETURNS: returns the idlist on success, NULL otherwise
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
|
------------------------------------------------------------*/
int kexpr_eval_expression(
   long id,
   char *mystr,
   char *error)
{
   kexpr_indent(-200);		/* set debug indent back to 0 */
   error_found = FALSE;
   kexpr_error_string = error;
   if (setjmp(jump_dest) == 0)
   {
      current_id = id;
      jump_dest_enabled = TRUE;
      mystr = kstring_cleanup(mystr, mystr);
      string_array = sptr = mystr;
      (void) kexpr_parse();
   }
   jump_dest_enabled = FALSE;
   return(error_found == FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: expr_type_name
|
|       Purpose: expr_type_name returns a string containing
|		 the name of a kexpr type constant.
|
|         Input: etype - type value
|
|        Output: none
|
|        Return: string containing the name - don't free it
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
char *expr_type_name(int etype)
{
   char *result = "???";

   switch(etype)
   {
      case NUMBER: result = "number";  break;
      case STRING: result = "string"; break;
      case CONSTANT: result = "constant"; break;
      case EXPRESSION: result = "expression"; break;
      case VARIABLE: result = "variable"; break;
      case FUNCTION: result = "function"; break;
      case UFUNCTION: result = "ufunction"; break;
      case OPERATOR: result = "operator"; break;
      case UOPERATOR: result = "uop"; break;
      case INSTRUCTION: result = "instr"; break;
   }
   return result;
}

/*-----------------------------------------------------------
|
|  Routine Name: do_indent
|
|       Purpose: do_indent indents the number of spaces
|		 specified in ilevel (module variable)
|
|         Input: none
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
static void do_indent(void)
{
   int i;
   for (i = 0;  i < ilevel;  i++) kputc(' ', kstdout);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_printf
|
|       Purpose: kexpr_printf prints a debugging message if
|		 kexpr_trace (a global variable) is nonzero.
|
|         Input: format - format string, ala printf
|                kvalist - args
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
void kexpr_printf(
    char *format,
    kvalist)
{
   if (kexpr_trace)
   {
      kva_list args;

      do_indent();
      kva_start(args, format);
      kvprintf(format, args);
      kva_end(args);
   }
}


/*-----------------------------------------------------------
|
|  Routine Name: kexpr_dump_symlist
|
|       Purpose: kexpr_dump_symlist dumps a SymbolList using
|                kexpr_printf.
|
|         Input: title - a title to print
|                syms - symbol list
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
void kexpr_dump_symlist(char *title, SymbolList *syms)
{
   if (!kexpr_trace) return;
   if (syms)
   {
      if (title != NULL)
	 kexpr_printf("SymbolList dump for %s:\n", title);
      else
	 kexpr_printf("SymbolList dump:\n");
      while(syms)
      {
	 ilevel += 2;
	 kexpr_dump_symbol(NULL, syms->symbol);
	 ilevel -= 2;
	 syms = syms->next;
      }
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_dump_symbol
|
|       Purpose: kexpr_dump_symbol dumps a Symbol using
|                kexpr_printf.
|
|         Input: title - a title to print
|                symbol - symbol to dump
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
void kexpr_dump_symbol( char *title, Symbol *symbol )
{
   static int dumping_function = FALSE;
   char *symtype, *exprtype;
   
   if (!kexpr_trace) return;

   symtype = expr_type_name(symbol->type);
   exprtype = expr_type_name(symbol->exprType);
   
   if (title != NULL)
   {
      kexpr_printf("%s of %s at %p: type = %s  exprType = %s\n",
		   title, kexpr_symname(symbol), symbol, symtype, exprtype);
   }
   else
   {
      kexpr_printf("%s %s at %p:  exprType = %s\n",
		   symtype, kexpr_symname(symbol), symbol, exprtype);
   }

   if (symbol->definition != NULL)
   {
      kexpr_printf("  Definition:\"%s\"\n", symbol->definition);
   }

   if (HAS_STRING(symbol))
      kexpr_printf("  Value: \"%s\"\n", STRING_VALUE(symbol));
   else if (HAS_DOUBLE(symbol))
      kexpr_printf("  Value: %lg\n", symbol->Value);

   if (symbol->type == EXPRESSION)
   {
      ilevel+=2;
      kexpr_dump_symlist("deflist", symbol->deflist);
      ilevel-=2;
   }
   else if (symbol->type == VARIABLE)
   {
      ilevel+=2;
      kexpr_dump_symlist("deflist", symbol->deflist);
      ilevel-=2;
   }
   else if (symbol->type == UFUNCTION)
   {
      if (!dumping_function)
      {
	 dumping_function = TRUE; /* recursion is bad */
	 ilevel+=2;
	 kexpr_dump_symlist(NULL, symbol->deflist);
	 ilevel-=2;
	 dumping_function = FALSE;
      }
   }
}


/*-----------------------------------------------------------
|
|  Routine Name: kexpr_indent
|
|       Purpose: kexpr_indent adjusts the indent level used
|                by kexpr_printf.
|
|         Input: delta - how much to adjuste the indentation by
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: April 1, 1997
|
------------------------------------------------------------*/
void kexpr_indent(int delta)
{
   ilevel += delta;
   if (ilevel < 0) ilevel = 0;
   if (ilevel > 40) ilevel = 40;
}



/*-----------------------------------------------------------
|
|  Routine Name: kexpr_init_symbol
|
|       Purpose: kexpr_init_symbol initializes a symbol to
|                reasonable values
|
|         Input: sym - the symbol to initialize
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
void kexpr_init_symbol(Symbol *sym)
{
   kmemset(sym, 0, sizeof(Symbol));
}


#ifndef DEBUG_KEXPR


/*-----------------------------------------------------------
|
|  Routine Name: kexpr_assign_double
|
|       Purpose: kexpr_assign_double assigns a double value
|                to a symbol
|
|         Input: sym - symbol to assign to
|                dval - value to assign
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
void kexpr_assign_double(Symbol *sym, double dval)
{
   RESET_VALUE(sym);
   sym->Value = dval;
   if (sym->type == EXPRESSION  ||  sym->type == VARIABLE)
      sym->exprType = NUMBER;
   else
      sym->type = NUMBER;
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_assign_cstring
|
|       Purpose: kexpr_assign_cstring assigns a C string value
|                to a symbol
|
|         Input: sym - symbol to assign to
|                cstr - string to assign
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
void kexpr_assign_cstring(Symbol *sym, char *cstr)
{
   RESET_VALUE(sym);
   sym->String = kstrdup(cstr);
   if (sym->type == EXPRESSION  ||  sym->type == VARIABLE)
      sym->exprType = STRING;
   else
      sym->type = STRING;
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_assign_string
|
|       Purpose: kexpr_assign_string assigns a string value
|                from one symbol to another
|
|         Input: to - symbol to assign to
|                from - symbol to assign from
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
void kexpr_assign_string(Symbol *to, Symbol *from)
{
   RESET_VALUE(to);
   to->String = kstrdup(from->String);
   if (to->type == EXPRESSION  ||  to->type == VARIABLE)
      to->exprType = STRING;
   else
      to->type = STRING;
}
   


/*-----------------------------------------------------------
|
|  Routine Name: kexpr_reset_value
|
|       Purpose: kexpr_reset_value resets a symbol's value
|
|         Input: sym - symbol to reset
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
void kexpr_reset_value(Symbol *sym)
{
   switch(sym->type)
   {
      case STRING:
         KSTRFREE(sym->String);
	 sym->String = NULL;
         sym->type = NUMBER;
         break;
      case EXPRESSION:
      case VARIABLE:
         if (sym->exprType == STRING)
         {
            KSTRFREE(sym->String);
         }
         sym->String = NULL;
         sym->exprType = NUMBER;
         break;
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_reset_symbol
|
|       Purpose: kexpr_reset_symbol resets a symbol's value
|                and clears its deflist
|
|         Input: sym - symbol to reset
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
void kexpr_reset_symbol(Symbol *sym)
{
   switch(sym->type)
   {
      case STRING:
         KSTRFREE(sym->String);
	 sym->String = NULL;
         sym->type = NUMBER;
         FREE_SYMLIST(sym->deflist);
         sym->deflist = NULL;
         break;
      case EXPRESSION:
      case VARIABLE:
         if (sym->exprType == STRING)
         {
            KSTRFREE(sym->String);
         }
         sym->String = NULL;
         sym->exprType = NUMBER;
         FREE_SYMLIST(sym->deflist);
         sym->deflist = NULL;
         break;
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_copy_value
|
|       Purpose: kexpr_copy_value copies one symbol's value
|                to another
|
|         Input: to - symbol to copy to
|                from - symbol to copy from
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
void kexpr_copy_value(Symbol *to, Symbol *from)
{
   if (to->type == UFUNCTION  ||  to->type == FUNCTION)
      return;
   if (to != from)
   {
      if (from->type == NUMBER)
      {
	 ASSIGN_DOUBLE(to, from->Value);
      }
      else if (from->type == STRING)
      {
	 ASSIGN_STRING(to, from);
      }
   }
}
      

#else

/*-----------------------------------------------------------
|
| *********** DEBUGGING ROUTINES *****************
|
------------------------------------------------------------*/



/*-----------------------------------------------------------
|
|  Routine Name: kexpr_strdup - duplicate a string
|
|       Purpose: kexpr_strdup is a debugging routine to 
|                duplicate a string
|
|         Input: str - string to duplicate
|                line - source line this was called from
|                file - source file this was called from
|
|        Output: none
|
|        Return: new string
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
char *kexpr_strdup(char *str, int line, char *file)
{
   char *result = kstrdup(str);
   kexpr_printf("%s:%d  kstrdup \"%s\" (%p) -> %p\n", file, line,
		str, str, result);
   return result;
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_strfree - free a string
|
|       Purpose: kexpr_strfree is a debugging routine to 
|                free a string
|
|         Input: str - string to duplicate
|                line - source line this was called from
|                file - source file this was called from
|
|        Output: none
|
|        Return: new string
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
void kexpr_strfree(char *str, int line, char *file)
{
   if (str == NULL)
      kexpr_printf("%s:%d  attempt to free a null string.\n", file, line);
   else
   {
      kexpr_printf("%s:%d  free string \"%s\" at %p\n", file, line, str, str);
      kmemset(str, 'X', kstrlen(str));
   }
   kfree(str);
}


/*-----------------------------------------------------------
|
|  Routine Name: kexpr_assign_double
|
|       Purpose: kexpr_assign_double assigns a double value
|                to a symbol
|
|         Input: sym - symbol to assign to
|                dval - value to assign
|                line - source line this was called from
|                file - source file this was called from
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
void kexpr_assign_double(Symbol *sym, double dval, int line, char *file)
{
   kexpr_printf("%s:%d - assigning double (%lg) to symbol %s (%p)\n  ",
	      file, line, dval, kexpr_symname(sym), sym);
   RESET_VALUE(sym);
   sym->Value = dval;
   if (sym->type == EXPRESSION  ||  sym->type == VARIABLE)
      sym->exprType = NUMBER;
   else
      sym->type = NUMBER;
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_assign_cstring
|
|       Purpose: kexpr_assign_cstring assigns a C string value
|                to a symbol
|
|         Input: sym - symbol to assign to
|                cstr - string to assign
|                line - source line this was called from
|                file - source file this was called from
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
void kexpr_assign_cstring(Symbol *sym, char *cstr, int line, char *file)
{
   kexpr_printf("%s:%d - assigning c-string at %p (\"%s\") to symbol %s (%p)\n  ",
	      file, line, cstr, cstr, kexpr_symname(sym), sym);
   RESET_VALUE(sym);
   sym->String = kexpr_strdup(cstr, line, file);
   if (sym->type == EXPRESSION  ||  sym->type == VARIABLE)
      sym->exprType = STRING;
   else
      sym->type = STRING;
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_assign_string
|
|       Purpose: kexpr_assign_string assigns a string value
|                from one symbol to another
|
|         Input: to - symbol to assign to
|                from - symbol to assign from
|                line - source line this was called from
|                file - source file this was called from
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
void kexpr_assign_string(Symbol *to, Symbol *from, int line, char *file)
{
   char *fname, *tname;

   kexpr_printf("%s:%d - assigning string \"%s\" from %s (%p) to %s (%p)\n  ",
	   file, line, from->String,
	   kexpr_symname(from), from, kexpr_symname(to), to);
   RESET_VALUE(to);
   to->String = kexpr_strdup(from->String, line, file);
   if (to->type == EXPRESSION  ||  to->type == VARIABLE)
      to->exprType = STRING;
   else
      to->type = STRING;
}
   


/*-----------------------------------------------------------
|
|  Routine Name: kexpr_reset_value
|
|       Purpose: kexpr_reset_value resets a symbol's value
|
|         Input: sym - symbol to reset
|                line - source line this was called from
|                file - source file this was called from
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
void kexpr_reset_value(Symbol *sym, int line, char *file)
{
   if (sym->type == UFUNCTION  ||  sym->type == FUNCTION)
      return;
   kexpr_printf("%d:%s  reset symbol value: symbol %s at %p\n",
	   line, file, kexpr_symname(sym), sym);
   if (sym->type == STRING  ||
       ((sym->type == EXPRESSION  ||  sym->type == VARIABLE)
	&&  sym->exprType == STRING))
      KSTRFREE(sym->String);
   sym->String = NULL;
   if (sym->type == EXPRESSION  ||  sym->type == VARIABLE)
      sym->exprType = NUMBER;
   else
      sym->type = NUMBER;
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_reset_symbol
|
|       Purpose: kexpr_reset_symbol resets a symbol's value
|                and clears its deflist
|
|         Input: sym - symbol to reset
|                line - source line this was called from
|                file - source file this was called from
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
void kexpr_reset_symbol(Symbol *sym, int line, char *file)
{
   if (sym->type == UFUNCTION  ||  sym->type == FUNCTION)
      return;
   kexpr_printf("%d:%s  reset symbol value: symbol %s at %p\n",
	   line, file, kexpr_symname(sym), sym);
   if (sym->type == STRING  ||
       ((sym->type == EXPRESSION  ||  sym->type == VARIABLE)
	&&  sym->exprType == STRING))
      KSTRFREE(sym->String);
   sym->String = NULL;
   if (sym->type == EXPRESSION  ||  sym->type == VARIABLE)
      sym->exprType = NUMBER;
   else
      sym->type = NUMBER;
   if (sym->deflist != NULL)
      FREE_SYMLIST(sym->deflist);
   sym->deflist = NULL;
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_copy_value
|
|       Purpose: kexpr_copy_value copies one symbol's value
|                to another
|
|         Input: to - symbol to copy to
|                from - symbol to copy from
|                line - source line this was called from
|                file - source file this was called from
|
|        Output: none
|
|        Return: none
|
|    Written By: Pete Humphrey
|          Date: May 23, 1997
|
------------------------------------------------------------*/
void kexpr_copy_value(Symbol *to, Symbol *from, int line, char *file)
{
   if (to->type == UFUNCTION  ||  to->type == FUNCTION)
      return;
   kexpr_printf("%d:%s  copy value from %s (%p) to %s (%p)\n",
	   line, file, kexpr_symname(from), from, kexpr_symname(to), to);
   if (to != from)
   {
      if (from->type == NUMBER)
      {
	 ASSIGN_DOUBLE(to, from->Value);
      }
      else if (from->type == STRING)
      {
	 ASSIGN_STRING(to, from);
      }
   }
}
      
#endif


