/*
 * Khoros: $Id: attrib_geom.c,v 1.6 1997/01/20 04:14:06 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: attrib_geom.c,v 1.6 1997/01/20 04:14:06 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		<<<<
   >>>>		Geometry Attribute Actions                <<<<
   >>>>		<<<<
   >>>>  Private:
   >>>>                 kvf_get_width()
   >>>>                 kvf_set_width()
   >>>>                 kvf_get_height()
   >>>>                 kvf_set_height()
   >>>>                 kvf_get_width_tacked_sel()
   >>>>                 kvf_set_width_tacked_sel()
   >>>>                 kvf_get_height_tacked_sel()
   >>>>                 kvf_set_height_tacked_sel()
   >>>>                 kvf_get_width_sel()
   >>>>                 kvf_get_height_sel()
   >>>>                 kvf_set_height_sel()
   >>>>                 kvf_set_width_sel()
   >>>>                 kvf_get_x()
   >>>>                 kvf_set_x()
   >>>>                 kvf_get_y()
   >>>>                 kvf_set_y()
   >>>>                 kvf_get_xpos()
   >>>>                 kvf_set_xpos()
   >>>>                 kvf_get_ypos()
   >>>>                 kvf_set_ypos()
   >>>>                 kvf_get_buttonwidth()
   >>>>                 kvf_set_buttonwidth()
   >>>>                 kvf_get_buttonheight()
   >>>>                 kvf_set_buttonheight()
   >>>>                 kvf_get_buttonx()
   >>>>                 kvf_set_buttonx()
   >>>>                 kvf_get_buttony()
   >>>>                 kvf_set_buttony()
   >>>>                 kvf_get_guidepane_title_xpos()
   >>>>                 kvf_set_guidepane_title_xpos()
   >>>>                 kvf_get_guidepane_title_ypos()
   >>>>                 kvf_set_guidepane_title_ypos()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


/*-----------------------------------------------------------
|
|       Routine: kvf_get_width
|       Purpose: Gets the width value of the geometry
|                from the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_WIDTH 
|        Output: calldata  - passes back float width value 
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_width(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	float        *width;
	Line_Info    line_info;

        /* the value to which KVF_WIDTH is to be set */
        width = (float *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	*width = line_info.width;
	kvf_free_line_info_strings(&line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_width
|       Purpose: Sets the width value of the geometry
|                on the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_WIDTH 
|                calldata  - provides float width value 
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_width(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float        *width;
	Line_Info    line_info;

	/* the value to which KVF_WIDTH is to be set */
        width = (float *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	line_info.width = *width;
	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_get_height
|       Purpose: Gets the height value of the geometry
|                from the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_HEIGHT 
|        Output: calldata  - passes back float height value 
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_height(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	float        *height;
	Line_Info    line_info;

        /* set the pointer to be returned */
        height = (float *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	*height = line_info.height;
	kvf_free_line_info_strings(&line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_height
|       Purpose: Sets the height value of the geometry
|                on the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_HEIGHT 
|                calldata  - provides float value for height
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_height(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float      *height;
	Line_Info  line_info;

	/* the value to which KVF_HEIGHT is to be set */
        height = (float *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	line_info.height = *height;
	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_width_tacked_sel
|       Purpose: Gets TRUE/FALSE value of whether or not width is
|		 tacked to the right (ie, if width == -1.0)
|                from the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute   - KVF_WIDTH_TACKED 
|        Output: calldata    - passes back boolean tacked value 
|
|          Date: August 14, 1995
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_width_tacked_sel(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	int       *tacked;
	Line_Info line_info;

        /* the value to which KVF_WIDTH_TACKED is to be set */
        tacked = (int *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	*tacked = ((line_info.width == -1.0)? TRUE : FALSE);
	kvf_free_line_info_strings(&line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_width_tacked_sel
|       Purpose: Sets TRUE/FALSE value indicating whether or not width is
|                tacked to the right (ie, sets width = -1.0)
|                on the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute   - KVF_WIDTH_TACKED 
|                calldata    - provides boolean tacked value 
|        Output: none
|
|          Date: August 14, 1995
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_width_tacked_sel(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info line_info;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	line_info.width = -1.0;
	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_get_height_tacked_sel
|       Purpose: Gets TRUE/FALSE value of whether or not height is
|                tacked to the bottom (ie, if height == -1.0)
|                from the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute   - KVF_HEIGHT_TACKED 
|        Output: calldata    - passes back boolean tacked value 
|
|          Date: August 14, 1995
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_height_tacked_sel(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	int       *tacked;
	Line_Info line_info;

        tacked = (int *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

        *tacked = ((line_info.height == -1.0)? TRUE : FALSE);
	kvf_free_line_info_strings(&line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_height_tacked_sel
|       Purpose: Sets TRUE/FALSE value indicating whether or not height is
|                tacked to the bottom (ie, sets height = -1.0)
|                on the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute   - KVF_HEIGHT_TACKED 
|                calldata    - provides boolean tacked value
|        Output: none
|
|          Date: August 14, 1995
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_height_tacked_sel(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info line_info;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	line_info.height = -1.0;
	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_width_sel
|       Purpose: Gets the boolean TRUE or FALSE depending on whether
|                or not explicit width is set on a selection (as opposed
|		 to -1, which implies tacking)
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_WIDTH_SEL
|        Output: calldata    - passes back TRUE if width explicitly set
|          Date: August 15, 1995
|    Written By: Danielle Argiro
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_width_sel(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        Line_Info    line_info;
        int          *width_set;

        /* set the pointer to be returned */
        width_set = (int *) calldata;

        if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
           return(FALSE);

        if (line_info.width != -1.0)
            *width_set = TRUE;
        else *width_set = FALSE;

        kvf_free_line_info_strings(&line_info);
        return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_height_sel
|       Purpose: Gets the boolean TRUE or FALSE depending on whether
|                or not explicit height is set on a selection (as opposed
|                to -1, which implies tacking)
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_HEIGHT_SEL
|        Output: calldata    - passes back TRUE if width explicitly set
|          Date: August 15, 1995
|    Written By: Danielle Argiro
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_height_sel(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        Line_Info    line_info;
        int          *height_set;

        /* set the pointer to be returned */
        height_set = (int *) calldata;

        if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
           return(FALSE);

        if (line_info.height != -1.0)
            *height_set = TRUE;
        else *height_set = FALSE;

        kvf_free_line_info_strings(&line_info);
        return(TRUE);
}

/* ARGSUSED */
int kvf_set_height_sel(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        Line_Info    line_info;

        if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
           return(FALSE);

        line_info.height = 30.0;
	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}

/* ARGSUSED */
int kvf_set_width_sel(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        Line_Info    line_info;

        if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
           return(FALSE);

        line_info.width = 30.0;
        kvf_gen_deparse_kformstruct(kformstruct, &line_info);


	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_x
|       Purpose: Gets the x value of the geometry
|                from the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_X 
|        Output: calldata  - passes back float X value 
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_x(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	float        *x;
	Line_Info    line_info;

        /* set the pointer to be returned */
        x = (float *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	*x = line_info.x;
	kvf_free_line_info_strings(&line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_x
|       Purpose: Sets the x value of the geometry
|                on the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_X 
|                calldata  - provides float value for x
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_x(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float        *x;
	Line_Info    line_info;

        /* set the pointer to be returned */
        x = (float *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	line_info.x = *x;
	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_get_y
|       Purpose: Gets the y value of the geometry
|                from the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_Y 
|        Output: calldata  - passes back float Y value 
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_y(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	float        *y;
	Line_Info    line_info;

        /* set the pointer to be returned */
        y = (float *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	*y = line_info.y;
	kvf_free_line_info_strings(&line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_y
|       Purpose: Sets the y value of the geometry
|                on the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_Y 
|                calldata  - provides float value for y
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_y(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float        *y;
	Line_Info    line_info;

	/* the value to which KVF_Y is to be set */
        y = (float *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	line_info.y = *y;
	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_get_xpos
|       Purpose: Gets the xpos value of the geometry
|                from the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_XPOS 
|        Output: calldata  - passes back float x position value
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_xpos(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float        *xpos;
        Line_Info    line_info;

        /* set the pointer to be returned */
        xpos = (float *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

        *xpos = line_info.xpos;
        kvf_free_line_info_strings(&line_info);

        return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_set_xpos
|       Purpose: Sets the x value of the title offset
|                on the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_XPOS 
|                calldata  - provides float value for xpos
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_xpos(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float        *xpos;
	Line_Info    line_info;
	ksubform     *subform;

	/* the value to which KVF_XPOS is to be set */
        xpos = (float *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	line_info.xpos = *xpos;
	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	/* for titles of subforms, need to move the text */
	if (kformstruct->type == KSUBFORM)
	{
            kvf_clear_line_info(&line_info);
            subform = kformstruct->Subformptr;
            kvf_parse_startsubform_line(subform->control_line, &line_info);
	    line_info.xpos = *xpos;
	    kvf_deparse_startsubform_line(&line_info, &subform->control_line);
        }

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_ypos
|       Purpose: Gets the ypos value of the geometry
|                from the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_YPOS 
|        Output: calldata  - passes back float y position value
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_ypos(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float        *ypos;
        Line_Info    line_info;
	ksubform     *subform;

        /* set the pointer to be returned */
        ypos = (float *) calldata;

	if (kformstruct->flag == KUIS_SUBFORMBUTTON) 
	{
	    kvf_clear_line_info(&line_info);
	    subform = kformstruct->Subformptr;
	    kvf_parse_startsubform_line(subform->control_line, &line_info);
	}
	else
	{
	    if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	        return(FALSE);

	}
        *ypos = line_info.ypos;

        kvf_free_line_info_strings(&line_info);
        return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_set_ypos
|       Purpose: Sets the y value of the title offset
|                on the UIS line of a GUI item
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_YPOS 
|                calldata  - provides float value for ypos
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_ypos(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float        *ypos;
	Line_Info    line_info;
	ksubform     *subform;

	/* the value to which KVF_YPOS is to be set */
        ypos = (float *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	line_info.ypos = *ypos;
	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	/* for titles of subforms, need to move the text */
	if (kformstruct->type == KSUBFORM)
	{
            kvf_clear_line_info(&line_info);
            subform = kformstruct->Subformptr;
            kvf_parse_startsubform_line(subform->control_line, &line_info);
	    line_info.ypos = *ypos;
	    kvf_deparse_startsubform_line(&line_info, &subform->control_line);
        }

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_buttonwidth
|       Purpose: Gets the value of the width field from the -d UIS line
|                This routine is specialized for use with subforms, where
|                you can't tell from the kformstruct if they want to get
|                the width of the subform button or of the subform itself.
|                However, it will also work for any of the other button-specific
|                UIS lines (-g, -m, -n, -a, -R, -H, -Q), as it just turns
|                around and calls kvf_get_width().
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_BUTTONWIDTH 
|        Output: calldata  - passes back float width value 
|
|          Date: November 28, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_buttonwidth(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	float        *width;
	Line_Info    line_info;
	char         *line = NULL;

        if (kformstruct->type != KSUBFORM)
            return(kvf_get_width(kformstruct, KVF_WIDTH, calldata));

        /* the value to which KVF_WIDTH is to be set */
        width = (float *) calldata;

        /*
         * Note: we are parsing the UIS line directly here,
         *       because kvf_gen_parse_kformstruct isn't smart
         *       enough to know whether you want the -M or the -d
         *       for a subform.
         */
        kvf_clear_line_info(&line_info);
        line = kformstruct->Subformptr->line;
        if (!(kvf_parse_subformbutton_line(line, &line_info)))
            return(FALSE);
	*width = line_info.width;
	kvf_free_line_info_strings(&line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_buttonwidth
|       Purpose: Sets the value of the width field from the -d UIS line
|                This routine is specialized for use with subforms, where
|                you can't tell from the kformstruct if they want to set
|                the width of the subform button or of the subform itself.
|                However, it will also work for any of the other button-specific
|                UIS lines (-g, -m, -n, -a, -R, -H, -Q), as it just turns
|                around and calls kvf_set_width().
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_BUTTONWIDTH 
|                calldata  - provides float width value 
|        Output: none
|
|          Date: November 28, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_buttonwidth(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float        *width;
	Line_Info    line_info;
	char         *line = NULL;

        if (kformstruct->type != KSUBFORM)
            return(kvf_set_width(kformstruct, KVF_WIDTH, calldata));

	/* the value to which KVF_WIDTH is to be set */
        width = (float *) calldata;

        /*
         * Note: we are parsing the UIS line directly here,
         *       because kvf_gen_parse_kformstruct isn't smart
         *       enough to know whether you want the -M or the -d
         *       for a subform.
         */
        kvf_clear_line_info(&line_info);
        line = kformstruct->Subformptr->line;
        if (!(kvf_parse_subformbutton_line(line, &line_info)))
            return(FALSE);
	line_info.width = *width;
        kvf_gen_deparse(&line_info, &kformstruct->Subformptr->line);
        kvf_free_line_info_strings(&line_info);
	return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_get_buttonheight
|       Purpose: Gets the value of the height field from the -d UIS line
|                This routine is specialized for use with subforms, where
|                you can't tell from the kformstruct if they want to get
|                the height of the subform button or of the subform itself.
|                However, it will also work for any of the other button-specific
|                UIS lines (-g, -m, -n, -a, -R, -H, -Q), as it just turns
|                around and calls kvf_get_height().
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_BUTTONHEIGHT 
|        Output: calldata  - passes back float height value 
|
|          Date: November 28, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_buttonheight(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	float        *height;
	Line_Info    line_info;
	char         *line = NULL;

        if (kformstruct->type != KSUBFORM)
            return(kvf_get_height(kformstruct, KVF_HEIGHT, calldata));

        /* set the pointer to be returned */
        height = (float *) calldata;

        /*
         * Note: we are parsing the UIS line directly here,
         *       because kvf_gen_parse_kformstruct isn't smart
         *       enough to know whether you want the -M or the -d
         *       for a subform.
         */
	kvf_clear_line_info(&line_info);
        line = kformstruct->Subformptr->line;
        if (!(kvf_parse_subformbutton_line(line, &line_info)))
            return(FALSE);
	*height = line_info.height;
	kvf_free_line_info_strings(&line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_buttonheight
|       Purpose: Sets the value of the height field from the -d UIS line
|                This routine is specialized for use with subforms, where
|                you can't tell from the kformstruct if they want to set
|                the height of the subform button or of the subform itself.
|                However, it will also work for any of the other button-specific
|                UIS lines (-g, -m, -n, -a, -R, -H, -Q), as it just turns
|                around and calls kvf_set_height().
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_BUTTONHEIGHT 
|                calldata  - provides float value for height
|        Output: none
|
|          Date: November 28, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_buttonheight(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float      *height;
	Line_Info  line_info;
	char       *line = NULL;

        if (kformstruct->type != KSUBFORM)
            return(kvf_set_height(kformstruct, KVF_HEIGHT, calldata));

	/* the value to which KVF_HEIGHT is to be set */
        height = (float *) calldata;

        /*
         * Note: we are parsing the UIS line directly here,
         *       because kvf_gen_parse_kformstruct isn't smart
         *       enough to know whether you want the -M or the -d
         *       for a subform.
         */
        kvf_clear_line_info(&line_info);
        line = kformstruct->Subformptr->line;
        if (!(kvf_parse_subformbutton_line(line, &line_info)))
            return(FALSE);
        line_info.height = *height;
        kvf_gen_deparse(&line_info, &kformstruct->Subformptr->line);
        kvf_free_line_info_strings(&line_info);

	return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_get_buttonx
|       Purpose: Gets the value of the x field from the -d UIS line
|                This routine is specialized for use with subforms, where
|                you can't tell from the kformstruct if they want to get
|                the x value of the subform button or of the subform itself.
|                However, it will also work for any of the other button-specific
|                UIS lines (-g, -m, -n, -a, -R, -H, -Q), as it just turns
|                around and calls kvf_get_x().
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_BUTTONX 
|        Output: calldata  - passes back float X value 
|
|          Date: November 28, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_buttonx(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	float        *x;
	Line_Info    line_info;
	char         *line = NULL;

        if (kformstruct->type != KSUBFORM)
            return(kvf_get_x(kformstruct, KVF_X, calldata));

        /* set the pointer to be returned */
        x = (float *) calldata;

        /*
         * Note: we are parsing the UIS line directly here,
         *       because kvf_gen_parse_kformstruct isn't smart
         *       enough to know whether you want the -M or the -d
         *       for a subform.
         */
        kvf_clear_line_info(&line_info);
        line = kformstruct->Subformptr->line;
        if (!(kvf_parse_subformbutton_line(line, &line_info)))
            return(FALSE);
	*x = line_info.x;
	kvf_free_line_info_strings(&line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_buttonx
|       Purpose: Sets the value of the x field from the -d UIS line
|                This routine is specialized for use with subforms, where
|                you can't tell from the kformstruct if they want to set
|                the x value of the subform button or of the subform itself.
|                However, it will also work for any of the other button-specific
|                UIS lines (-g, -m, -n, -a, -R, -H, -Q), as it just turns
|                around and calls kvf_set_x().
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_BUTTONX 
|                calldata  - provides float value for x
|        Output: none
|
|          Date: November 28, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_buttonx(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float        *x;
	Line_Info    line_info;
	char         *line = NULL;

        if (kformstruct->type != KSUBFORM)
            return(kvf_set_x(kformstruct, KVF_X, calldata));

        /* set the pointer to be returned */
        x = (float *) calldata;

        /*
         * Note: we are parsing the UIS line directly here,
         *       because kvf_gen_parse_kformstruct isn't smart
         *       enough to know whether you want the -M or the -d
         *       for a subform.
         */
        kvf_clear_line_info(&line_info);
        line = kformstruct->Subformptr->line;
        if (!(kvf_parse_subformbutton_line(line, &line_info)))
            return(FALSE);
	line_info.x = *x;
        kvf_gen_deparse(&line_info, &kformstruct->Subformptr->line);
        kvf_free_line_info_strings(&line_info);

	return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_get_buttony
|       Purpose: Gets the value of the y field from the -d UIS line
|                This routine is specialized for use with subforms, where
|                you can't tell from the kformstruct if they want to get
|                the y value of the subform button or of the subform itself.
|                However, it will also work for any of the other button-specific
|                UIS lines (-g, -m, -n, -a, -R, -H, -Q), as it just turns
|                around and calls kvf_get_y().
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_BUTTONY 
|        Output: calldata  - passes back float Y value 
|
|          Date: November 28, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_buttony(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	float        *y;
	Line_Info    line_info;
	char         *line = NULL;

        if (kformstruct->type != KSUBFORM)
            return(kvf_get_y(kformstruct, KVF_Y, calldata));

        /* set the pointer to be returned */
        y = (float *) calldata;

        /*
         * Note: we are parsing the UIS line directly here,
         *       because kvf_gen_parse_kformstruct isn't smart
         *       enough to know whether you want the -M or the -d
         *       for a subform.
         */
        kvf_clear_line_info(&line_info);
        line = kformstruct->Subformptr->line;
        if (!(kvf_parse_subformbutton_line(line, &line_info)))
            return(FALSE);
	*y = line_info.y;
	kvf_free_line_info_strings(&line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_buttony
|       Purpose: Sets the value of the y field from the -d UIS line
|                This routine is specialized for use with subforms, where
|                you can't tell from the kformstruct if they want to set
|                the y value of the subform button or of the subform itself.
|                However, it will also work for any of the other button-specific
|                UIS lines (-g, -m, -n, -a, -R, -H, -Q), as it just turns
|                around and calls kvf_set_y().
|
|         Input: kformstruct - generic kformstruct holding ptr to GUI item
|                attribute - KVF_BUTTONY 
|                calldata  - provides float value for y
|        Output: none
|
|          Date: November 28, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_buttony(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float        *y;
	Line_Info    line_info;
	char         *line = NULL;

        if (kformstruct->type != KSUBFORM)
            return(kvf_set_y(kformstruct, KVF_Y, calldata));

	/* the value to which KVF_Y is to be set */
        y = (float *) calldata;

        /*
         * Note: we are parsing the UIS line directly here,
         *       because kvf_gen_parse_kformstruct isn't smart
         *       enough to know whether you want the -M or the -d
         *       for a subform.
         */
        kvf_clear_line_info(&line_info);
        line = kformstruct->Subformptr->line;
        if (!(kvf_parse_subformbutton_line(line, &line_info)))
            return(FALSE);
	line_info.y = *y;
        kvf_gen_deparse(&line_info, &kformstruct->Subformptr->line);
        kvf_free_line_info_strings(&line_info);

	return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_get_guidepane_title_xpos
|       Purpose: Gets the xpos value from the -G StartGuide UIS line 
|
|         Input: kformstruct - subform's kformstruct 
|                attribute   - KVF_GUIDEPANE_TITLE_XPOS 
|        Output: calldata    - passes back float x position value
|
|          Date: August 9, 1995
|    Written By: Danielle Argiro
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_guidepane_title_xpos(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float        *xpos;
        Line_Info    line_info;
	kcontrol     *guidepane;

        /* set the pointer to be returned */
        xpos = (float *) calldata;

        /* kformstruct coming in is from the subform */
        guidepane = kformstruct->Subformptr->guidepane;
        if (guidepane == NULL)
            return(FALSE);

	if (!(kvf_gen_parse_kformstruct(guidepane->back_kformstruct, 
				        &line_info)))
	    return(FALSE);

        *xpos = line_info.xpos;
        kvf_free_line_info_strings(&line_info);

        return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_set_guidepane_title_xpos
|       Purpose: Sets the xpos value on the -G StartGuide UIS line
|
|         Input: kformstruct - subform's kformstruct
|                attribute   - KVF_GUIDEPANE_TITLE_XPOS 
|                calldata    - provides float value for xpos
|        Output: none
|
|          Date: August 9, 1995
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_guidepane_title_xpos(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float        *xpos;
	Line_Info    line_info;
	kcontrol     *guidepane;

	/* the value to which KVF_GUIDEPANE_TITLE_XPOS is to be set */
        xpos = (float *) calldata;

        /* kformstruct coming in is from the subform */
        guidepane = kformstruct->Subformptr->guidepane;
        if (guidepane == NULL)
            return(FALSE);

	if (!(kvf_gen_parse_kformstruct(guidepane->back_kformstruct, 
				        &line_info)))
	    return(FALSE);

	line_info.xpos = *xpos;
	kvf_gen_deparse_kformstruct(guidepane->back_kformstruct, &line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_guidepane_title_ypos
|       Purpose: Gets the ypos value from the -G StartGuide UIS line
|
|         Input: kformstruct - subform's kformstruct 
|                attribute   - KVF_GUIDEPANE_TITLE_YPOS
|        Output: calldata    - passes back float y position value
|
|          Date: August 9, 1995
|    Written By: Danielle Argiro
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_guidepane_title_ypos(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float        *ypos;
        Line_Info    line_info;
	kcontrol     *guidepane;

        /* set the pointer to be returned */
        ypos = (float *) calldata;

        /* kformstruct coming in is from the subform */
        guidepane = kformstruct->Subformptr->guidepane;
        if (guidepane == NULL)
            return(FALSE);

        if (!(kvf_gen_parse_kformstruct(guidepane->back_kformstruct, 
                                        &line_info)))
            return(FALSE);

        *ypos = line_info.ypos;

        kvf_free_line_info_strings(&line_info);
        return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_set_guidepane_title_ypos
|       Purpose: Sets the ypos value on the -G StartGuide UIS line
|
|         Input: kformstruct - subform's kformstruct
|                attribute   - KVF_GUIDEPANE_TITLE_YPOS
|                calldata    - provides float value for ypos
|        Output: none
|
|          Date: August 9, 1995
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_guidepane_title_ypos(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        float        *ypos;
	Line_Info    line_info;
	kcontrol     *guidepane;

	/* the value to which KVF_YPOS is to be set */
        ypos = (float *) calldata;

        /* kformstruct coming in is from the subform */
        guidepane = kformstruct->Subformptr->guidepane;
        if (guidepane == NULL)
            return(FALSE);

	if (!(kvf_gen_parse_kformstruct(guidepane->back_kformstruct, 
					&line_info)))
	    return(FALSE);

	line_info.ypos = *ypos;
	kvf_gen_deparse_kformstruct(guidepane->back_kformstruct, &line_info);

	return(TRUE);
}

