/*
 * Khoros: $Id: db_util.c,v 1.13 1997/02/05 22:14:37 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: db_util.c,v 1.13 1997/02/05 22:14:37 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Routines to Read the UIS file            
   >>>>
   >>>>  Private:
   >>>>                 kvf_read_uis_file()
   >>>>   Static:
   >>>>                 kvf_read_form()
   >>>>                 kvf_read_subform()
   >>>>                 kvf_read_pane()
   >>>>                 kvf_realloc_uis_struct()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


/* db_util.c */
static int         kvf_read_subform       PROTO((uis_struct *, char *));
static int         kvf_read_pane          PROTO((uis_struct *, char *));
static uis_struct *kvf_realloc_uis_struct PROTO((uis_struct *, size_t));


/*------------------------------------------------------------
|
|    Routine Name: kvf_read_uis_file
|
|         Purpose: Reads in a UIS file and returns a pointer to a 
|                  UIS structure with the array of strings representing
|                  the UIS, plus all related info including allocated
|		   UIS size, number of lines in the UIS, and arrays for
|		   error messaging.
|
|           Input: filename  - open stream to UIS file opened for reading
|         Returns: A pointer to the uis_spec struct that represents the UIS
|                  (see public kforms.h file for details)
|
|    Written By: Danielle Argiro
|          Date: Nov 13, 1996 
|
------------------------------------------------------------*/

uis_struct *kvf_read_uis_file(
   char *filename)
{
	kfile      *file;
	uis_struct *uis_spec;
        Line_Info  line_info;
        char       temp[KLENGTH*20];
        int        j, file_num, done = FALSE, count = -1;
        size_t     length;
        int        internal_linenum = 1, flag = -1;
	char       *routine = "kvf_read_uis_file";

	/* allocate uis_spec structure */
	uis_spec = (uis_struct *) kcalloc((size_t)1, sizeof(uis_struct));
	if (uis_spec == NULL)
        {
            kerror(KFORMS, routine, "Can't allocate room for UIS structure");
            return(NULL);
        }
	uis_spec->linenum    = 0; 
	uis_spec->filecount  = 1; 
	uis_spec->alloc_size = KUIS_ALLOC_SIZE;

	kvf_clear_line_info(&line_info);
	file_num = (int) uis_spec->filecount;

	
	/* allocate internal UIS structure */
	uis_spec->uis = (char **) kcalloc((size_t) 1, sizeof(char *) *
					              KUIS_ALLOC_SIZE);
	if (uis_spec->uis == NULL)
	{
	    kerror(KFORMS, routine, "Can't allocate room for UIS array");
	    return(NULL);
	}

	uis_spec->file_lookup = (size_t *) kmalloc(sizeof(size_t) * 
						    KUIS_ALLOC_SIZE);
	if (uis_spec->file_lookup == NULL)
        {
	    kerror(KFORMS, routine, 
                   "Can't allocate room for UIS filename lookup table");
            return(NULL);
        }

	uis_spec->line_lookup = (size_t *) kmalloc(sizeof(size_t) * 
				  	            KUIS_ALLOC_SIZE);
	if (uis_spec->line_lookup == NULL)
        {
	    kerror(KFORMS, routine, 
		   "Can't allocate room for UIS linenum lookup table");
            return(NULL);
        }

        uis_spec->filenames = (char **) kcalloc((size_t)1, sizeof(char*));
        if (uis_spec->filenames == NULL)
        {
            kerror(KFORMS, routine,
                   "Unable to allocate UIS filenames array");
            return(NULL);
        }
	uis_spec->filenames[0] = kstrdup(filename);

        /* open the UIS file */
        if ((file = kfopen(filename, "r")) == NULL)
        {
            kerror(KFORMS, routine,
                    "Unable to open '%s' to read UIS", filename);
	    kvf_free_uis_struct(uis_spec);
            return(NULL);
        }

	/* premature end of file */
	if (kfeof(file))
	{
	    kvf_free_uis_struct(uis_spec);
	    kfclose(file);
	    return(NULL);
	}

	while (!done)
 	{
           j = 0;
	   if (kfgets(temp, KLENGTH*20, file) == NULL) 
	   {
	      errno = KUIS_SYNTAX;
	      kerror(KFORMS, routine, 
                     "Premature end of file encountered in UIS");
	      kvf_free_uis_struct(uis_spec);
	      kfclose(file);
	      return(NULL);
	   }

	   while(temp[j] == ' ' || temp[j] == '\t') j++;

	   flag = kvf_get_line_type(&temp[j]);
	   if (flag == -1)
	   {
	      kvf_free_uis_struct(uis_spec);
	      kfclose(file);
	      return(NULL); 
	   }
	   switch (flag)
	   {
		 case KUIS_INCLUDEPANE:
		      kvf_parse_includepane_line(&temp[j], &line_info);
		      if (!kvf_read_pane(uis_spec, line_info.filename))
	              {
		         kvf_free_line_info_strings(&line_info);
                         kvf_free_uis_struct(uis_spec);
	                 kfclose(file);
                         return(NULL);
		      }
		      internal_linenum++;
		      if (uis_spec->uis == NULL)
		      {
			 errno = KUIS_SYNTAX;
	                 kerror(KFORMS, routine,
				"Error in line '%s' of included UIS file %s", 
				 &temp[j], line_info.filename);
	                 kvf_free_uis_struct(uis_spec);
		         kvf_free_line_info_strings(&line_info);
	                 kfclose(file);
		         return(NULL);
		      }
		      kvf_free_line_info_strings(&line_info);
		      break;

		 case KUIS_INCLUDESUBFORM:
		      kvf_parse_includesubform_line(&temp[j], &line_info);
		      if (!kvf_read_subform(uis_spec, line_info.filename))
		      {
		         kvf_free_line_info_strings(&line_info);
			 kvf_free_uis_struct(uis_spec);
	                 kfclose(file);
 			 return(NULL);
		      }
		      internal_linenum++;
		      if (uis_spec->uis == NULL)
		      {
			 errno = KUIS_SYNTAX;
	                 kerror(KFORMS, routine,
				"Error in line '%s' of included UIS file %s", 
                                &temp[j], line_info.filename);
	                 kvf_free_uis_struct(uis_spec);
		         kvf_free_line_info_strings(&line_info);
	                 kfclose(file);
		         return(NULL);
		      }
		      kvf_free_line_info_strings(&line_info);
		      break;

		 case KUIS_STARTFORM:
		      count = 1;
		      break;

		 case KUIS_STARTMASTER:
		 case KUIS_STARTSUBMENU:
		 case KUIS_STARTSUBFORM:
		 case KUIS_STARTGUIDE:
		 case KUIS_STARTPANE:
		 case KUIS_TOGGLE:
		 case KUIS_MUTEXCL:
		 case KUIS_MUTINCL:
		 case KUIS_GROUP:
		      count++;
		      break;

		 case KUIS_END:
		      count--;
		      break;

	   }

	   if (flag != KUIS_INCLUDEPANE && flag != KUIS_INCLUDESUBFORM)
	   {
		 if (uis_spec->linenum == (uis_spec->alloc_size-1))
		 {
		    uis_spec = kvf_realloc_uis_struct(uis_spec, 
		 				      KUIS_ALLOC_SIZE);
		    if (uis_spec == NULL) 
		    {
			kfclose(file);
			return(NULL);
		    }
		 }
	         length = kstrlen(&temp[j]);
	         uis_spec->uis[uis_spec->linenum] = (char *) kcalloc((size_t) 1,
					              sizeof(char)*length);
		 if (uis_spec->uis[uis_spec->linenum] == NULL)
		 {
		    kerror(KFORMS, routine,
			   "Unable to allocate line in internal UIS");
		    kfclose(file);
		    return(NULL);
		 }
	         (void) kstrncpy(uis_spec->uis[uis_spec->linenum], 
			         &temp[j], (size_t) length);
	         uis_spec->uis[uis_spec->linenum][length-1] = '\0';
	         uis_spec->file_lookup[uis_spec->linenum] = 
						uis_spec->filecount-1;
	         uis_spec->line_lookup[uis_spec->linenum] = internal_linenum;
	         uis_spec->linenum++;
		 internal_linenum++;
	    }

	    if (count == 0) done = TRUE;
	}

	/*
	 * error check for missing control lines
	 */
        flag = kvf_get_line_type(uis_spec->uis[uis_spec->linenum-1]);
        if ((flag != KUIS_END) && (flag != KUIS_COMMENT))
        {
            errno = KUIS_SYNTAX;
            kerror(KFORMS, routine,
                   "possible missing control [-F, -S, -M, -G, -P, -T, "
                   "or -C] line, or misplaced end [-E] line");
	    kvf_free_uis_struct(uis_spec);
	    kfclose(file);
            return(NULL);
        }
	flag = kvf_get_line_type(uis_spec->uis[uis_spec->linenum-2]);
	if ((flag != KUIS_END) && (flag != KUIS_COMMENT))
	{
	    errno = KUIS_SYNTAX;
	    kerror(KFORMS, routine, 
                   "possible missing control [-F, -S, -M, -G, -P, -T, "
		   "or -C] line, or misplaced end [-E] line");
	    kvf_free_uis_struct(uis_spec);
	    kfclose(file);
	    return(NULL);
	}
	flag = kvf_get_line_type(uis_spec->uis[uis_spec->linenum-3]);
	if ((flag != KUIS_END) && (flag != KUIS_COMMENT))
	{
	    errno = KUIS_SYNTAX;
	    kerror(KFORMS, routine, 
                   "possible missing control [-F, -S, -M, -G, -P, -T, or "
                   "-C] line, or misplaced end [-E] line");
	    kvf_free_uis_struct(uis_spec);
	    kfclose(file);
	    return(NULL);
	}

	if (uis_spec->alloc_size < KUIS_ALLOC_SIZE)
	{
	    uis_spec->file_lookup = (size_t *) krealloc(uis_spec->file_lookup,
						        uis_spec->alloc_size);
	    if (uis_spec->file_lookup == NULL)

	    {
		kerror(KFORMS, routine,
		       "Unable to allocate UIS file_lookup array");
	        kfclose(file);
		return(NULL);
	    }

	    uis_spec->line_lookup  = (size_t *) krealloc(uis_spec->file_lookup, 
					                 uis_spec->alloc_size);
	    if (uis_spec->line_lookup == NULL)
	    {
		kerror(KFORMS, routine,
		       "Unable to allocate UIS line_lookup array");
	        kfclose(file);
		return(NULL);
	    }

	    uis_spec->uis = (char **) krealloc(uis_spec->uis, 
					       uis_spec->alloc_size);
	    if (uis_spec->uis == NULL)
	    {
		kerror(KFORMS, routine,
		       "Unable to re-allocate UIS array");
	        kfclose(file);
		return(NULL);
	    }
	}

	kfclose(file);
	return(uis_spec);
}


/*--------------------------------------------------------------
|
|   Routine Name: kvf_read_subform
|
|        Purpose: Reads in a UIS subform definition from the specified 
|                 UIS file into the UIS structure.  This routine is called
|		  when kvf_read_uis_file() finds a [-k] UIS line (used
|		  to include *.subform files in a *.form file). 
|                 Note this routine discards the [-F]/[-E] lines from the
|                 *.subform files, as they will not be needed.
|
|          Input: filename - name of the UIS file in which to get the
|			     subform definition.
|		  uis_spec - the UIS structure
|        Returns: TRUE on success, FALSE on failure
|          Date: Nov 13, 1996
|    Written By: Danielle Argiro & Mark Young
|
------------------------------------------------------------*/


int kvf_read_subform(
   uis_struct *uis_spec,
   char       *filename)
{
	uis_struct *subform_uis_spec;
	char       temp[KLENGTH];
	Line_Info  line_info;
        int        i, j, offset = 0; 
	char       *routine = "kvf_read_subform";

	/*
	 * add one more name to the filenames array, increment filecount - 
         * need to allocate another string for the filenames array
	 */
        uis_spec->filecount++;
	uis_spec->filenames = (char **) krealloc(uis_spec->filenames,
					uis_spec->filecount*sizeof(char*));
	if (uis_spec->filenames == NULL)
	{
	    kerror(KFORMS, routine,
		   "Unable to allocate UIS filenames array");
	    return(FALSE);
	}
	uis_spec->filenames[uis_spec->filecount-1] = kstrdup(filename);

	/*
         * read the subform specification into a temporary uis struct
	 */
	subform_uis_spec = kvf_read_uis_file(filename);
	if (subform_uis_spec == NULL)
 	    return(FALSE);

        /*
         *  Check to see if the pane specification starts with
         *  a KUIS_STARTFORM specification.
         */
        kvf_clear_line_info(&line_info);
        (void) kvf_get_comments(subform_uis_spec->uis, &offset);
        if (kvf_get_line_type(subform_uis_spec->uis[offset]) == KUIS_STARTFORM)
        {
           if (!(kvf_parse_startform_line(subform_uis_spec->uis[offset], 
					  &line_info)))
                return(FALSE);
           ksprintf(temp, "@%s", subform_uis_spec->uis[offset]);
           kfree_and_NULL(subform_uis_spec->uis[offset]);
           subform_uis_spec->uis[offset] = kstrdup(temp);
           kvf_free_line_info_strings(&line_info);
           (subform_uis_spec->linenum)--; offset++;
        }

	/*
	 *  Copy the rest of the UIS, which should be the subform,
	 *  into old UIS.  Copy the associated entries from filename
         *  array into old filename lookup table.
	 */
	for (j = uis_spec->linenum, i = -1; 
             i < (int) subform_uis_spec->linenum; i++, j++)
	{
	    if (j == (uis_spec->alloc_size-1))
	    {
	       uis_spec = kvf_realloc_uis_struct(uis_spec, KUIS_ALLOC_SIZE);
	       if (uis_spec == NULL) return(FALSE);
	    }
	    if (i == -1)
	    {
		ksprintf(temp, "@-k %s", filename);
		uis_spec->uis[j] = kstrdup(temp);
	        uis_spec->file_lookup[j] = uis_spec->filecount-1;
	        uis_spec->line_lookup[j] = offset-1;
	    }
	    else
	    {
	        uis_spec->uis[j]         = kstrdup(subform_uis_spec->uis[i]);
	        uis_spec->file_lookup[j] = uis_spec->filecount-1;
	        uis_spec->line_lookup[j] = subform_uis_spec->line_lookup[i];
	        subform_uis_spec->uis[i] = NULL;
	    }
	}

	uis_spec->linenum = j;
	return(TRUE);
}



/*--------------------------------------------------------------
|
|   Routine Name: kvf_read_pane
|
|        Purpose: Reads in a UIS pane definition from the specified
|                 UIS file into the UIS structure.  This routine is called
|                 when kvf_read_uis_file() finds a [-p] UIS line (used
|                 to include *.pane files in a *.form or *.subform file).
|                 Note this routine discards the [-F]/[-E] and [-M]/[-E]
|                 lines from the *.subform files, as they will not be needed.
|
|          Input: filename - name of the UIS file in which to get the
|                            subform definition.
|                 uis_spec - the UIS structure
|        Returns: TRUE on success, FALSE on failure
|           Date: Nov 13, 1996
|     Written By: Danielle Argiro & Mark Young
|
------------------------------------------------------------*/

static int kvf_read_pane(
   uis_struct *uis_spec,
   char       *filename)
{
	uis_struct *pane_uis_spec; 
	char       temp[KLENGTH];
	Line_Info  line_info;
	int	   i, j, offset = 0; 
	char       *routine = "kvf_read_pane";

	/*
	 * add one more name to the filenames array, increment filecount - 
         * need to allocate another string for the filenames array
	 */
        uis_spec->filecount++;
	uis_spec->filenames = (char **) krealloc(uis_spec->filenames,  
		   	        (unsigned) (uis_spec->filecount)*sizeof(char*));
	if (uis_spec->filenames == NULL)
	{
	    kerror(KFORMS, routine,
		   "Unable to allocate UIS filenames array");
	    return(FALSE);
	}
	uis_spec->filenames[uis_spec->filecount-1] = kstrdup(filename);

	/*
         * read the pane specification into a temporary UIS;
	 */
	pane_uis_spec = kvf_read_uis_file(filename);
	if (pane_uis_spec == NULL) return(FALSE);

	/*
	 *  Check to see if the pane specification starts with
	 *  a KUIS_STARTFORM specification.
	 */
	kvf_clear_line_info(&line_info);
	(void) kvf_get_comments(pane_uis_spec->uis, &offset);
	if (kvf_get_line_type(pane_uis_spec->uis[offset]) == KUIS_STARTFORM)
	{
	   if (!(kvf_parse_startform_line(pane_uis_spec->uis[offset], 
				          &line_info))) return(FALSE);
	   ksprintf(temp, "@%s", pane_uis_spec->uis[offset]);
           kfree_and_NULL(pane_uis_spec->uis[offset]);
           pane_uis_spec->uis[offset] = kstrdup(temp);
	   kvf_free_line_info_strings(&line_info);
	   (pane_uis_spec->linenum)--; offset++;
	}

	/*
	 *  Check to see if the pane specification starts with
	 *  a KUIS_STARTSUBFORM specification.
	 */
	(void) kvf_get_comments(pane_uis_spec->uis, &offset);
	if (kvf_get_line_type(pane_uis_spec->uis[offset]) == KUIS_STARTSUBFORM)
	{
	   if (!(kvf_parse_startsubform_line(pane_uis_spec->uis[offset],
					     &line_info))) return(FALSE);
	   ksprintf(temp, "@%s", pane_uis_spec->uis[offset]);
           kfree_and_NULL(pane_uis_spec->uis[offset]);
           pane_uis_spec->uis[offset] = kstrdup(temp);
	   kvf_free_line_info_strings(&line_info);
	   (pane_uis_spec->linenum)--; offset++;
	}

	/*
	 *  Copy the rest of the UIS, which should be the pane, into old UIS.
	 */
	for (j = uis_spec->linenum, i = -1; 
             i < (int) pane_uis_spec->linenum; i++, j++)
	{
	    if (j == (uis_spec->alloc_size-1))
	    {
	       uis_spec = kvf_realloc_uis_struct(uis_spec, KUIS_ALLOC_SIZE);
	       if (uis_spec == NULL)
	          return(FALSE);
	    }
	    if (i == -1)
            {
                ksprintf(temp, "@-p %s", filename);
                uis_spec->uis[j] = kstrdup(temp);
	        uis_spec->file_lookup[j] = uis_spec->filecount-1;
	        uis_spec->line_lookup[j] = offset-1;
            }
	    else
	    {
	        uis_spec->uis[j]         = kstrdup(pane_uis_spec->uis[i]);
	        uis_spec->file_lookup[j] = uis_spec->filecount-1;
	        uis_spec->line_lookup[j] = pane_uis_spec->line_lookup[i];
	    }
	}

	/*
	 *  Free the temporary UIS.
	 */
	kvf_free_uis_struct(pane_uis_spec);
	uis_spec->linenum = j;

	return(TRUE);
}

/*------------------------------------------------------------
|
|  Routine Name: kvf_realloc_uis_struct
|
|       Purpose: Reallocates the UIS structure, including the new 
|                size for UIS array, as well as new size for the 
|                file_lookup and line_lookup arrays.
|
|         Input: uis_spec - current UIS structure
|                request  - new number of lines in the UIS
|
|       Returns: the uis_spec structure, reallocated to its new size
|
|          Date: Nov 13, 1996
|    Written By: Danielle Argiro & Mark Young
|
------------------------------------------------------------*/

static uis_struct *kvf_realloc_uis_struct(
   uis_struct *uis_spec,
   size_t     request)
{
	size_t  num_bytes;
	char    *routine = "kvf_realloc_uis_spec";

	if (uis_spec->uis == NULL)
	{
	   num_bytes = request * sizeof(char *);
	   uis_spec->uis = (char **) kcalloc((size_t) 1, num_bytes);
	   if (uis_spec->uis == NULL)
	   {
	        kerror(KFORMS, routine,
		       "Cannot allocate room for UIS array");
	        return(NULL);
	   }

	   num_bytes = request * sizeof(size_t);
	   uis_spec->file_lookup = (size_t *) kmalloc(num_bytes);
	   if (uis_spec->file_lookup == NULL)
	   {
	        kerror(KFORMS, routine,
		       "Unable to allocate UIS file_lookup array");
	        return(NULL);
	   }

	   uis_spec->file_lookup = (size_t *) kmalloc(num_bytes);
	   if (uis_spec->file_lookup == NULL)
	   {
	        kerror(KFORMS, routine,
		       "Unable to allocate UIS line_lookup array");
	        return(NULL);
	   }
	   uis_spec->alloc_size = request;
	}
	else
	{
	   num_bytes = (uis_spec->alloc_size + request) * sizeof(char *);
	   uis_spec->uis = (char **) krealloc(uis_spec->uis, num_bytes);
	   if (uis_spec->uis == NULL)
	   {
	        kerror(KFORMS, routine,
		       "Cannot allocate room for UIS array");
	        return(NULL);
	   }

	   /*
	    *  Need to put nulls in the new part of the allocated array.
	    */
	   num_bytes = request * sizeof(char *);
	   kmemset((char *) &uis_spec->uis[uis_spec->alloc_size], 0, num_bytes);

	   num_bytes = (uis_spec->alloc_size + request) * sizeof(int);
	   uis_spec->file_lookup = (size_t *) 
				krealloc(uis_spec->file_lookup, num_bytes);
	   if (uis_spec->file_lookup == NULL)
	   {
	        kerror(KFORMS, routine,
		       "Unable to allocate UIS file_lookup array");
	        return(NULL);
	   }

	   uis_spec->line_lookup  = (size_t *) 
				krealloc(uis_spec->line_lookup, num_bytes);
	   if (uis_spec->line_lookup == NULL)
	   {
	        kerror(KFORMS, routine,
		       "Unable to allocate UIS line_lookup array");
	        return(NULL);
	   }

	   uis_spec->alloc_size += request;
	}

	return(uis_spec);
}

/*-------------------------------------------------------------
|
|  Routine Name: kvf_free_uis_struct
|
|       Purpose: Frees UIS struct
|
|         Input: uis_spec  - the UIS struct to be freed
|          Date: Nov 13, 1996
|    Written By: Danielle Argiro 
| Modifications: 
|
------------------------------------------------------------*/

void kvf_free_uis_struct(
   uis_struct *uis_spec)
{
	int i;

	for (i = 0; i < uis_spec->filecount; i++)
	    kfree(uis_spec->filenames[i]);
	kfree(uis_spec->filenames);

	for (i =  0; i < uis_spec->alloc_size; i++)
	    kfree(uis_spec->uis[i]);
	kfree (uis_spec->uis);

	kfree(uis_spec->file_lookup);
	kfree(uis_spec->line_lookup);

	kfree(uis_spec);
}
