/*
 * Khoros: $Id: internals.h,v 1.8.2.1 1997/05/24 17:49:31 mirage Exp $
 */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Private include file for kforms  library
   >>>>
   >>>>    Written By: Danielle Argiro
   >>>>
   >>>>          Date: March 29, 1993
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kforms_internals_h_
#define _kforms_internals_h_

        /*-------------------------------------*
        |       #include
        ---------------------------------------*/

#include <bootstrap.h>
#include "attributes.h"

        /*-------------------------------------*
        |       #defines
        ---------------------------------------*/

#define KUIS_ALLOC_SIZE ((size_t) 512)
#define KFORMS "kforms"

        /*-------------------------------------*
        |       typedefs
        ---------------------------------------*/

        /*-------------------------------------*
        |       global variable declarations
        ---------------------------------------*/

extern int          kvf_context;

        /*-------------------------------------*
        |       routine definitions
        ---------------------------------------*/

/*
 *   Utilities for checking the UIS (check.c)
 */
int kvf_check_for_subform_buttons PROTO((char **, int));
int kvf_check_for_guide_buttons   PROTO((char **, int));
int kvf_check_me_member_default   PROTO((uis_struct *, int *, int, int));
int kvf_check_mi_member_default   PROTO((uis_struct *, int *, int, int));
int kvf_check_me_group_default    PROTO((uis_struct *, int *, int));
int kvf_check_int                 PROTO((int, int, int, char *, char *));
int kvf_check_float               PROTO((float, float, float, char *, char *));
int kvf_check_double              PROTO((double, double, double, char *, char *));
int kvf_check_bounds              PROTO((double, double, double));

/*
 *   Cleanup Routines (cleanup.c)
 */
void kvf_free_line_info         PROTO((Line_Info *));
void kvf_free_line_info_strings PROTO((Line_Info *));
void kvf_destroy_subform        PROTO((ksubform *));
void kvf_destroy_control        PROTO((kcontrol *));
void kvf_destroy_guide          PROTO((kguide *));
void kvf_destroy_sel_list       PROTO((kselection *));
void kvf_destroy_selection      PROTO((kselection *));

/*
 *   Copy Form Routines (copy.c)
 *   (see kformsP.h)
 */

/*
 *   Utilities for Form Tree Creation (create_util.c)
 */

char *kvf_find_matching_M_line PROTO((uis_struct *, int, int *));
char *kvf_find_matching_P_line PROTO((uis_struct *, int, int *));
int   kvf_set_subform_values   PROTO((uis_struct *, kcontrol *, ksubform *, 
                                      char *, int, kform *, kselection *, int));
int kvf_set_guide_values       PROTO((uis_struct *, kcontrol *, kguide *, 
                                      char *, kform *, ksubform *, 
                                      kselection *, int));
int  kvf_set_selection_values PROTO((kselection *, char *, int, kform *,
                                      ksubform *, kguide *, kcontrol *,
                                      kselection *, char *, char *));
int kvf_error_on_submenu       PROTO((uis_struct *, int, int));
int kvf_error_with_filetype    PROTO((int, int, int, int *, int *, char **));

ksubform   *kvf_find_subform   PROTO((ksubform *,   ktoken));
kguide     *kvf_find_guide     PROTO((kguide *,     ktoken));
kselection *kvf_find_selection PROTO((kselection *, ktoken));

/*
 *   ME/MI/Loose Group Utilities (group.c)
 */
void kvf_call_do_group       PROTO((kselection *, int));
void kvf_set_group_selection PROTO((kselection *, int));


/*
 * routines for form tree linkage
 */
void kvf_fastlink_subform   PROTO((kcontrol *, ksubform *, ksubform **));
void kvf_fastlink_guide     PROTO((kcontrol *, kguide *, kguide **));
void kvf_fastlink_selection PROTO((kcontrol *, kselection *, kselection **));
void kvf_fastlink_group     PROTO((kselection *, kselection *, kselection **));
void kvf_fastlink_toggle    PROTO((kselection *, kselection *, kselection **));
void kvf_fastlink_submenu   PROTO((kselection *, kform_struct *, kform_struct **));

void kvf_link_selection     PROTO((kcontrol *,   kselection *));
void kvf_link_toggle        PROTO((kselection *, kselection *));
void kvf_link_group         PROTO((kselection *, kselection *));

/*
 *  UIS Parsing Routines (parser.c)
 *  (see kformsP.h)
 */
int kvf_realloc_val_labelnum      PROTO((Line_Info *, size_t));

/*
 *   Utilities for printing form tree (print.c)
 */
void kvf_print_names PROTO((kform *));

int  kvf_print_panefile    PROTO((kcontrol *, int));
int  kvf_print_subformfile PROTO((ksubform *, int));

char *kvf_retrieve_subform_filename PROTO((ksubform *));
char *kvf_retrieve_pane_filename    PROTO((kcontrol *));


/*
 *   General Utilities (utilities.c)
 */
char *kvf_get_comments             PROTO((char **, int *));
void  kvf_initialize               PROTO((void));
void  kvf_parsing_error_mesg       PROTO((int, size_t *, size_t *, char **));
void  kvf_allocation_error_mesg    PROTO((void));
int   kvf_set_int_from_literal     PROTO((kselection *, char *));
int   kvf_set_float_from_literal   PROTO((kselection *, char *));
int   kvf_set_double_from_literal  PROTO((kselection *, char *));
int   kvf_set_file_from_literal    PROTO((kselection *, char *));
int   kvf_set_string_from_literal  PROTO((kselection *, char *));


/*
 *   Routine to deal with kform_struct's (kformstruct.c)
 */
kform_struct *kvf_create_struct PROTO((kaddr, int, int));


#endif /* _kforms_internals_h_ */
/* Do not add after this line */
