/*
 * Khoros: $Id: kdata_minmax.c,v 1.5 1997/01/20 04:23:52 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kdata_minmax.c,v 1.5 1997/01/20 04:23:52 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Data Manipulation Utilities
   >>>>		
   >>>>   Public:
   >>>>                 kdata_minmax()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/*************************************************************
*
*  Routine Name: kdata_minmax - find the min/max values for a region of data
*
*       Purpose: This function is used to find the min/max values
*		 associated with a region of memory.  A datatype
*                is used to determine how to search the data set.
*		 The data types are byte, unsigned byte, short,
*		 unsigned short, long, unsigned long, int, unsigned int,
*		 float, and double.
*
*         Input: data      - point to the data to find the min/max value
*		 num       - number of points to be searched
*		 datatype  - data type of the data 
*
*        Output: minval - returns the minimum value
*		 maxval - returns the maximum value
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*    Written By: Mark Young
*          Date: Jun 05, 1995
* Modifications:
*
*************************************************************/

int kdata_minmax(
   kaddr  data,
   int    num,
   int    datatype,
   double *minval,
   double *maxval)
{
	register int i;

	switch (datatype)
	{
	   case KUSHORT:
	        {
		   register unsigned short tmpmax, tmpmin;
	           register unsigned short *tmp = (unsigned short *) data;

		   tmpmin = tmpmax = *tmp++;
	           for (i = 1; i < num; i++, tmp++)
		   {
	               tmpmin = kmin(tmpmin, *tmp);
	               tmpmax = kmax(tmpmax, *tmp);
		   }
		   *minval = tmpmin;
		   *maxval = tmpmax;
 	        }
	        break;

	   case KUINT:
	        {
		   register unsigned int tmpmax, tmpmin;
	           register unsigned int *tmp = (unsigned int *) data;

		   tmpmin = tmpmax = *tmp++;
	           for (i = 1; i < num; i++, tmp++)
		   {
	               tmpmin = kmin(tmpmin, *tmp);
	               tmpmax = kmax(tmpmax, *tmp);
		   }
		   *minval = tmpmin;
		   *maxval = tmpmax;
 	        }
		break;

	   case KULONG:
	        {
		   register unsigned long tmpmax, tmpmin;
	           register unsigned long *tmp = (unsigned long *) data;

		   tmpmin = tmpmax = *tmp++;
	           for (i = 1; i < num; i++, tmp++)
		   {
	               tmpmin = kmin(tmpmin, *tmp);
	               tmpmax = kmax(tmpmax, *tmp);
		   }
		   *minval = tmpmin;
		   *maxval = tmpmax;
 	        }
	        break;

	   case KUBYTE:
	        {
		   register unsigned char tmpmax, tmpmin;
	           register unsigned char *tmp = (unsigned char *) data;

		   tmpmin = tmpmax = *tmp++;
	           for (i = 1; i < num; i++, tmp++)
		   {
	               tmpmin = kmin(tmpmin, *tmp);
	               tmpmax = kmax(tmpmax, *tmp);
		   }
		   *minval = tmpmin;
		   *maxval = tmpmax;
 	        }
		break;

	   case KSHORT:
	        {
		   register signed short tmpmax, tmpmin;
	           register signed short *tmp = (signed short *) data;

		   tmpmin = tmpmax = *tmp++;
	           for (i = 1; i < num; i++, tmp++)
		   {
	               tmpmin = kmin(tmpmin, *tmp);
	               tmpmax = kmax(tmpmax, *tmp);
		   }
		   *minval = tmpmin;
		   *maxval = tmpmax;
 	        }
		break;

	   case KINT:
	        {
		   register signed int tmpmax, tmpmin;
	           register signed int *tmp = (signed int *) data;

		   tmpmin = tmpmax = *tmp++;
	           for (i = 1; i < num; i++, tmp++)
		   {
	               tmpmin = kmin(tmpmin, *tmp);
	               tmpmax = kmax(tmpmax, *tmp);
		   }
		   *minval = tmpmin;
		   *maxval = tmpmax;
 	        }
		break;

	   case KLONG:
	        {
		   register signed long tmpmax, tmpmin;
	           register signed long *tmp = (signed long *) data;

		   tmpmin = tmpmax = *tmp++;
	           for (i = 1; i < num; i++, tmp++)
		   {
	               tmpmin = kmin(tmpmin, *tmp);
	               tmpmax = kmax(tmpmax, *tmp);
		   }
		   *minval = tmpmin;
		   *maxval = tmpmax;
 	        }
		break;

	   case KBYTE:
	        {
		   register signed char tmpmax, tmpmin;
	           register signed char *tmp = (signed char *) data;

		   tmpmin = tmpmax = *tmp++;
	           for (i = 1; i < num; i++, tmp++)
		   {
	               tmpmin = kmin(tmpmin, *tmp);
	               tmpmax = kmax(tmpmax, *tmp);
		   }
		   *minval = tmpmin;
		   *maxval = tmpmax;
 	        }
		break;

	   case KFLOAT:
	        {
		   register float tmpmax, tmpmin;
	           register float *tmp = (float *) data;

		   tmpmin = tmpmax = *tmp++;
	           for (i = 1; i < num; i++, tmp++)
		   {
	               tmpmin = kmin(tmpmin, *tmp);
	               tmpmax = kmax(tmpmax, *tmp);
		   }
		   *minval = tmpmin;
		   *maxval = tmpmax;
 	        }
		break;

	   case KDOUBLE:
	        {
		   register double tmpmax, tmpmin;
	           register double *tmp = (double *) data;

		   tmpmin = tmpmax = *tmp++;
	           for (i = 1; i < num; i++, tmp++)
		   {
	               tmpmin = kmin(tmpmin, *tmp);
	               tmpmax = kmax(tmpmax, *tmp);
		   }
		   *minval = tmpmin;
		   *maxval = tmpmax;
 	        }
		break;
	}
	return(TRUE);
}
