/*
 * Khoros: $Id: Mmap.c,v 1.17.2.1 1997/04/17 22:12:10 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Mmap.c,v 1.17.2.1 1997/04/17 22:12:10 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Sun Virtual Memory Transport Routines
   >>>>		
   >>>>   Static:
   >>>>                 mmap_reopen()
   >>>>                 mmap_tempnam()
   >>>>                 mmap_open()
   >>>>                 mmap_close()
   >>>>                 mmap_read()
   >>>>                 mmap_write()
   >>>>                 mmap_lseek()
   >>>>                 mmap_access()
   >>>>                 mmap_unlink()
   >>>>                 mmap_lock()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifdef KOPSYS_HPUX
#ifndef _HPUX_SOURCE
#define   _HPUX_SOURCE
#endif
#endif
#include "internals.h"	


#if !defined(KMMAP_DEF)
TransportInformation mmap_transport[] = { {NULL} };
#else

/* memory mapped transport */
#include <sys/mman.h>

static int mmap_tempnam	  PROTO((const char *, const char *, char *));
static int mmap_open	  PROTO((const char *, int, mode_t, kfile *));
static int mmap_close	  PROTO((kfile *));
static ssize_t mmap_read  PROTO((kfile *, kaddr, size_t));
static ssize_t mmap_write PROTO((kfile *, kaddr, size_t));
static off_t mmap_lseek	  PROTO((kfile *, off_t, int));
static int mmap_access	  PROTO((const char *, const char *, int));
static int mmap_unlink	  PROTO((const char *, const char *));
static int mmap_lock	  PROTO((kfile *, int));

TransportInformation mmap_transport[] =
{
    {
	"Memory Mapped Management",
	"mmap",
	TRUE,
	mmap_tempnam,
	mmap_open,
	mmap_close,
	mmap_read,
	mmap_write,
	mmap_lseek,
	mmap_access,
	mmap_unlink,
	mmap_lock,
	NULL,
    }
};

typedef struct
{
	int	 fid;
	char	 *addr;
	off_t	 offset;
	int	 prot;
	int	 map;
	size_t   size;

} ResourceStruct;


/*-------------------------------------------------------------------*
|  
|		  Private Sun Memory Mapped Routines
|  
--------------------------------------------------------------------*/

static void mmap_reopen(
   kfile *file,
   size_t num_bytes)
{
	char	*addr;
	ResourceStruct *resources = (ResourceStruct *) file->resources;
	int	map = resources->map;


	/*
	 *  Go ahead and seek to the position in which we are know that
	 *  data will be written.  This extends the file so that the size
	 *  of the memory map will be valid to this point.
	 */
	lseek(resources->fid, (off_t) num_bytes - 1, 0);
	if (write(resources->fid, "", (size_t) 1) != 1)
	{
	   kinfo(KSYSLIB, "mmap_reopen:  Write failure, before mmap call. "
			  "Unable to remap file '%s'", file->path);
	   return;
	}

	/*
	 *  Attach to file the appropriate memory map space...
	 */
	if ((addr = (char *)mmap(resources->addr, num_bytes, resources->prot, map,
			 resources->fid, (off_t) 0)) == (char *) -1)
	{
	   kinfo(KSYSLIB, "mmap_reopen:  Memory Map (mmap) failure.  Unable "
			  "to remap file '%s'", file->path);
	   return;
	}
	resources->addr = addr;
	resources->size = num_bytes;
}

/*-------------------------------------------------------------------*
|  
|		  Sun Memory Mapped Routines
|  
--------------------------------------------------------------------*/

/*-------------------------------------------------------------
|  
|  Routine Name: mmap_tempnam
|  
|       Purpose: This function initializes a tempnam for a "mmap"
|	         type transport.  Given a request with a filename of
|	         the following syntax:
|  
|		  	"mmap=XXXXXXXXXX"
|		  	     ...etc....
|  
|         Input:  identifier - the transport identifier
|		  ktemplate  - the filename template to be modeled
|        Output:  result     - the newly created transport name
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int mmap_tempnam(
   const char *identifier,
   const char *ktemplate,
   char *result)
{
	char *temp, path[KLENGTH];

	(void) kstrcpy(path, ktemplate);
	if ((temp = ktmpnam(path)) == NULL)
	   return(-1);

	if (identifier == NULL)
	   (void) kstrcpy(result, temp);
	else
	   (void) sprintf(result,"%s=%s", identifier, temp);

	kfree(temp);
	return(0);
}

/*-------------------------------------------------------------
|  
|  Routine Name: mmap_open
|  
|       Purpose: This function opens a "mmap" connection.  It is the
|	         internal driver to open a file, which is called by
|	         the user when they call kopen().  kopen() calls when
|	         it gets an open request with a filename of the following
|	         syntax:
|  
|		  	"mmap=XXXXXXXXX"
|         Input: 
|        Output:  returns 0 or -1 if an error occurs
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int mmap_open(
   const char  *path,
   int   flags,
   mode_t mode,
   kfile *file)
{
	char	 *addr;
	size_t   size;
	struct   stat buf;
	ResourceStruct *resources;
	int	 fid, map, prot, flg = flags;

	/*
	 *  Get memory map (mmap) protections from the open permission
	 *  flags...
	 */
	if ((flags & KOPEN_ACCMODE) == KOPEN_RDONLY)
	{
	   map = MAP_SHARED; 
	   prot = PROT_READ;
	}
	else
	{
	   map = MAP_SHARED; 
	   prot = PROT_WRITE | PROT_READ;
	   if (flags & KOPEN_WRONLY)
	   {
	      flg ^= KOPEN_WRONLY;
	      flg |= KOPEN_RDWR;
	   }
	}

	/*
	 *  Need to modify the flags so that we never open a file using
	 *  append.  This really screws us up when we try to close the
	 *  memory map.  Instead we will set the offset to initially be
	 *  the size of the file.  This should achieve the same result...
	 */
	if (flags & KOPEN_APPEND)
	   flg ^= KOPEN_APPEND;

	/*
	 *  Open the file for the memory map (mmap) call.
	 */
	if ((fid = open(path, flg, mode)) < 0)
	{
	   kinfo(KSYSLIB, "mmap_open:  Unable to open file '%s'", path);
	   return(-1);
	}

	/*
	 *  Get the size of the file
	 */
	if (stat(path, &buf) == -1 || buf.st_size < 0)
	{
	   kinfo(KSYSLIB, "mmap_open:  Unable to stat file '%s'", path);
	   return(-1);
	}

	if ((size = buf.st_size) == 0 && write(fid, "", (size_t) 1) != 1)
        {
           kinfo(KSYSLIB, "mmap_open:  Write failure, before mmap call. "
			  "Unable to map file '%s'", path);
	   return(kfile_rename(file, "file="));
        }

	/*
	 *  Attach to file the appropriate memory map space...
	 */
	size = kmax(size, 1);
	if ((addr = (char *) mmap(NULL, size, prot, map, fid, (off_t) 0)) == (char *) -1)
	{
	   /*
	    *  Since we failed to memory map the device revert to using
	    *  the UNIX file transport.
	    */
	   kinfo(KSYSLIB, "mmap_open:  Unable to memory map file '%s'", path);
	   return(kfile_rename(file, "file="));
	}

	if ((resources = (ResourceStruct *) kcalloc((size_t) 1,
			sizeof(ResourceStruct))) == NULL)
	{
           kinfo(KSYSLIB, "mmap_open: Not enough memory to kcalloc (%d) bytes"
	      " for the khoros mmap structure.\n", sizeof(ResourceStruct));
	   return(-1);
	}
	resources->fid    = fid;
	resources->addr   = addr;
	resources->size   = size;
	resources->prot   = prot;
	resources->map    = map;
	resources->offset = (flags & KOPEN_APPEND) ? size : 0;

	file->resources = (kaddr) resources;
	return(0);
}

/*-------------------------------------------------------------
|  
|  Routine Name: mmap_close
|  
|       Purpose: This function closes a "mmap".  It is the internal driver
|	         to close a data mmap, which is called by the user
|	         when they call kclose().  kclose() calls the "mmap" internal
|	         drivers by the nature of the transport, which is dictated
|	         by the initial kopen().
|  
|         Input:  file - the kfile structure.
|        Output:  returns whether we were able to close the file
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int mmap_close(
   kfile *file)
{
	char   *addr;
	size_t size;
	int    fid, offset;
	ResourceStruct *resources = (ResourceStruct *) file->resources;

	/*
	 *  Call munmap() and close() since "kfile" is really a memory mapped
	 *  file.
	 */
	fid    = resources->fid;
	size   = resources->size;
	addr   = resources->addr;
	offset = resources->offset;

	/*
	 *  If this data file is opened for writing and data has been written
	 *  then we better make sure that the extent of the file matches that
	 *  of the memory map.  We do this by seeking to the offset and writing
	 *  the last byte into the file.  This guarantees that at least this
	 *  amount of the memory map will be written into the file.
	 */
	if (kfile_iswrite(file) && addr != NULL && offset > 0)
	{
	   lseek(resources->fid, (off_t) offset-1, 0);
	   if (write(resources->fid, &addr[offset-1], (size_t) 1) != 1)
	   {
	      kinfo(KSYSLIB, "mmap_close: Write failure in close call for "
			     "file '%s'.  Ignoring and flushing as much of the "
			     "memory map as possible.", file->path);
	   }
	}
	return(munmap(addr, size) | close(fid));
}

/*-------------------------------------------------------------
|  
|  Routine Name: mmap_read
|  
|       Purpose: This function reads a "mmap".  It is the internal driver
|	         to read data from a mmap, which is called by the user
|	         when they call kread().  kread() calls the "mmap" internal
|	         drivers by the nature of the transport, which is dictated
|	         by the kopen().
|  
|         Input:  file   - the kfile structure.
|		  ptr    - the pointer to store the data into.
|		  nbytes - the number of bytes to read.
|        Output:  returns the number of bytes read from the mmap
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static ssize_t mmap_read(
   kfile *file,
   kaddr ptr,
   size_t nbytes)
{
	char   *addr;
	size_t size;
	off_t offset;
	ResourceStruct *resources = (ResourceStruct *) file->resources;

	/*
	 *  Simply call kmemcpy() since "kfile" is just a regular 
	 *  memory pointer.
	 */
	addr   = resources->addr;
	size   = resources->size;
	offset = resources->offset;

	if (offset >= size)
	{
	   struct stat buf;

	   if (fstat(resources->fid, &buf) == -1 || offset >= buf.st_size)
	      return(EOF);
	   resources->size = size = buf.st_size;
	}

	if (addr == ptr || addr+offset == ptr)
	{
	   if (offset+nbytes > size)
	      nbytes = size-offset;
	}
	else if (offset+nbytes > size)
	{
	   kmemcpy(ptr, addr+offset, (size_t) (size-offset));
	   nbytes = size-offset;
	}
	else
	   kmemcpy(ptr, addr+offset, nbytes);

	resources->offset = offset+nbytes;
	return((ssize_t) nbytes);
}

/*-------------------------------------------------------------
|  
|  Routine Name: mmap_write
|  
|       Purpose: This function writes to a "mmap".  It is the internal driver
|	         to write data from the supplied data array to a mmap, which
|	         is called by the user when they call kwrite().  kwrite() calls
|	         the "mmap" internal drivers by the nature of the transport,
|	         which is dictated by the kopen().
|  
|         Input:  file   - the kfile structure.
|		  ptr  - the pointer to write the data from.
|		  nbytes - the number of bytes to read.
|        Output:  returns the number of bytes written to the kfile
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static ssize_t mmap_write(
   kfile  *file,
   kaddr  ptr,
   size_t nbytes)
{
	char   *addr;
	size_t size;
	off_t  offset;
	ResourceStruct *resources = (ResourceStruct *) file->resources;

	/*
	 *  Simply call kmemcpy() since "kfile" is just a regular 
	 *  memory pointer.
	 */
	addr   = resources->addr;
	size   = resources->size;
	offset = resources->offset;

	/*
	 *  Make sure that we aren't trying to read more bytes than we
	 *  currently have.  If so then try re-opening the file to see
	 *  if it's gotten bigger.
	 */
	if (offset+nbytes > size)
	{
	   mmap_reopen(file, (size_t) (offset+nbytes));
	   addr   = resources->addr;
	   size   = resources->size;
	}

	if (offset >= size)
	   return(EOF);
	else if (addr == ptr || addr+offset == ptr)
	{
	   if (offset+nbytes > size)
	      nbytes = size-offset;
	}
	else if (offset+nbytes > size)
	{
	   kmemcpy(addr+offset, ptr, (size_t) (size-offset));
	   nbytes = size-offset;
	}
	else
	   kmemcpy(addr+offset, ptr, nbytes);

	resources->offset = offset+nbytes;
	return((ssize_t) nbytes);
}

/*-------------------------------------------------------------
|  
|  Routine Name: mmap_lseek
|  
|       Purpose: This function is used to do a "lseek".  It is the internal
|	         driver to rewind to a specific point so that the data can be
|	         skipped or re-read.  This is called when the user calls
|	         klseek().  klseek() calls the "mmap" internal drivers by the
|	         nature of the transport, which is dictated by the kopen().
|  
|         Input:  file   - the kfile structure.
|		  offset - the offset in which to seek
|		  whence - the control of how the offset will be applied
|        Output:  returns the number of bytes written to the kfile
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static off_t mmap_lseek(
   kfile *file,
   off_t  offset,
   int    whence)
{
	ResourceStruct *resources = (ResourceStruct *) file->resources;

	/*
	 *  Simply call lseek() since "kfile" is just a regular mmap
	 */
	if (whence == KSEEK_SET)
	   resources->offset = offset;
	else if (whence == KSEEK_CUR)
	   resources->offset += offset;
	else if (whence == KSEEK_END)
	   resources->offset = resources->size + offset;
	else
	{
	   errno = EINVAL;
	   kinfo(KSYSLIB, "mmap_lseek:  Invalid whence argument value for "
			  "file '%s'", file->path);
	   return(-1);
	}

	if (resources->offset > resources->size)
	/*EMPTY*/
	{
	   /* not sure as of yet */
	}
	return(resources->offset);
}

/*-------------------------------------------------------------
|  
|  Routine Name: mmap_access
|  
|       Purpose: This function is used to do see if a file is accessable.
|	         This is called when the user calls kaccess().  kaccess()
|	         calls the "mmap" internal drivers by the nature of the
|	         transport, which is dictated by the ktransport_identifier().
|  
|         Input:  name - the original path name passed to kaccess
|		  path - the filename that they wish to access
|		  mode - the access mode information
|        Output:  returns the result
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int mmap_access(
   const char *name,
   const char *path,
   int  mode)
{
	int	   status;

	/*
	 *  Simply call access() since "path" is just a regular filename
	 */
	status = access(path, mode);
	return(status);
}

/*-------------------------------------------------------------
|  
|  Routine Name: mmap_unlink
|  
|       Purpose: This function initializes a tempnam for a "mmap"
|	         type transport.  Given a request with a filename of
|	         the following syntax:
|  
|		  	"mmap=XXXXXXXXXX"
|
|         Input:  name - the original path name passed to kunlink
|		  path - the filename that they wish to unlink
|        Output:  returns 0 or -1 depending whether we
|		  sucessfully created the template
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int mmap_unlink(
   const char *name,
   const char *path)
{
	int	status;

	status = unlink(path);
	return(status);
}

/*-------------------------------------------------------------
|  
|  Routine Name: mmap_lock
|  
|       Purpose: This function locks a "mmap" transport.  Given
|	         a request with a filename of the following syntax:
|  
|		  	"mmap=filename"
|         Input: 
|        Output:  returns 0 or -1 depending whether we
|		  sucessfully created the template
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int mmap_lock(
   kfile *file,
   int   operation)
{
	int	status = 0, type = -1;
/*
	ResourceStruct *resources = (ResourceStruct *) file->resources;
*/

	/*
	 *  Translate the lock operation in fcntl() lock type.
	 */
	if (operation & KLOCK_SH)
	   type = 1;
	else if (operation & KLOCK_EX)
	   type = 2;
	else if (operation & KLOCK_UN)
	   type = 8;
	else
	   return(-1);
/*
	status = flock(resources->fid, operation);
 */
	return(status);
}

#endif  /* KMMAP_DEF */
/* don't add after the endif */
