/*
 * Khoros: $Id: Remote.c,v 1.27.2.1 1997/03/06 22:48:40 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Remote.c,v 1.27.2.1 1997/03/06 22:48:40 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros Remote Transport Routines
   >>>>		
   >>>>   Static:
   >>>>                 remote_tempnam()
   >>>>                 remote_open()
   >>>>                 remote_exec()
   >>>>                 remote_close()
   >>>>                 remote_read()
   >>>>                 remote_write()
   >>>>                 remote_access()
   >>>>                 remote_unlink()
   >>>>                 remote_descriptors()
   >>>>   Public:
   >>>>                 kremote_exec()
   >>>>                 kremote_file()
   >>>>                 kremote_location()
   >>>>                 kremote_check()
   >>>>                 kremote_running()
   >>>>                 kremote_start()
   >>>>                 kremote_stop()
   >>>>                 kremote_list()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	

static int remote_tempnam     PROTO((const char *, const char *, char *));
static int remote_open	      PROTO((const char *, int, mode_t, kfile *));
static int remote_close	      PROTO((kfile *));
static ssize_t remote_read    PROTO((kfile *, kaddr, size_t));
static ssize_t remote_write   PROTO((kfile *, kaddr, size_t));
static int remote_access      PROTO((const char *, const char *, int));
static int remote_unlink      PROTO((const char *, const char *));
static int remote_descriptors PROTO((kfile *, int *, int *));

/*
 *  Most people smell bad.  But that's no excuse to not let them change
 *  their default remote command.
 */
char *DefaultRemoteCmd = "rsh";

TransportInformation remote_transport[] =
{
    {
	"Khoros Remote Transport",
	"remote",
	FALSE,
	remote_tempnam,
	remote_open,
	remote_close,
	remote_read,
	remote_write,
	NULL,
	remote_access,
	remote_unlink,
	NULL,
	remote_descriptors,
    }
};

#define REMOTE_TYPE_SH       0
#define REMOTE_TYPE_KEXEC    1
#define REMOTE_TYPE_PHANTOMD 2

/*
 *  Internal resource structures for "standard remote" transport
 */
typedef struct
{
	pid_t pid;
	int   fid[2];
	int   type;
	char  *remotecmd;
	char  *location;
	char  *user;
	int   busy;

	char   *eos;
	size_t readnum;
	int    reader;
} ResourceStruct;

static int	        hnum  = 0;
static ResourceStruct **hinfo = NULL;

static char *EosSymbol = "CommandEos\n";
#define EosSize kstrlen(EosSymbol)

/*-------------------------------------------------------------------*
|  
|                 Private Remote Transport Routines
|  
--------------------------------------------------------------------*/

/*ARGSUSED*/
static int remote_readline(
   ResourceStruct *resources,
   char  *buf,
   size_t size)
{
	size_t cnt = 0;
	char *tmp = buf;

	do 
	{
	   if (read(resources->fid[0], tmp, (size_t) 1) != 1)
	      break;

	   if (*tmp == '\n')
	   {
	      *tmp = '\0';
	      return(TRUE);
	   }
	   tmp++;
	} while (++cnt < size-1);

	*tmp = '\0';
	return(FALSE);
}

/*ARGSUSED*/
static int remote_writeline(
   ResourceStruct *resources,
   char  *buf)
{
	ssize_t size;

	size = (ssize_t) kstrlen(buf);
	if (write(resources->fid[1], (kaddr) buf, (size_t) size) != size)
	   return(FALSE);

	return(TRUE);
}

/*ARGSUSED*/
static int remote_setenv(
   ResourceStruct *resources,
   const char *env,
   const char *value)
{
	char temp[KLENGTH];
	char phcmd[] = "putenv: '%s=%s'\n";
	char shcmd[] = "%s=\"%s\"; export %s; if [ $? = 0 ]; "
		       "then echo yes; else echo no; fi\n";

	if (resources->type == REMOTE_TYPE_PHANTOMD)
	   ksprintf(temp, phcmd, env, value);
	else
	   ksprintf(temp, shcmd, env, value, env);

	remote_writeline(resources, temp);
	remote_readline(resources, temp, (size_t) KLENGTH);
	if (kstrcmp(temp, "yes") == 0)
	   return(TRUE);

	return(FALSE);
}

/*ARGSUSED*/
static int remote_daemon(
   ResourceStruct *resources,
   const char *cmd,
   int type)
{
	pid_t pid;
	int   to_child[2], from_child[2];
	char  command[KLENGTH], **args;

	if (pipe(from_child) == -1 || pipe(to_child) == -1)
	   return(FALSE);

	if (kstrcmp(resources->location, "localtest") == 0)
	   ksprintf(command, "sh -c '%s'", cmd);
	else
	   ksprintf(command, "%s %s -l %s %s", resources->remotecmd,
			resources->location, resources->user, cmd);

	args = kcommand_to_args(command);
	if ((pid = kfork()) == 0)
	{
	   dup2(to_child[0], STDIN_FILENO);
	   close(to_child[0]); close(to_child[1]);
	   dup2(from_child[1], STDERR_FILENO);
	   dup2(from_child[1], STDOUT_FILENO);
	   close(from_child[1]); close(from_child[0]);

           (void) setsid();
	   kexecvp(args[0], args);
	   kfprintf(kstderr,"\nUnable to execute the following command:\n\n %s",
			command);
           _exit(KEXIT_FAILURE);
	}

        /*
         *  Check to see if we failed to create the process.
         */
        if (pid == -1)
        {
           kfprintf(kstderr, "\nUnable to create a process to execute the "
			"command (errno=%d)\n", errno);
           return(FALSE);
        }
	close(to_child[0]);
	close(from_child[1]);

	resources->pid = pid;
	resources->type = type;
	resources->fid[0] = from_child[0];
	resources->fid[1] = to_child[1];

	if (remote_setenv(resources, "KhorosDaemonType", cmd) == FALSE)
	{
	   kill(resources->pid, SIGTERM);
	   close(resources->fid[0]), close(resources->fid[1]);
	   return(FALSE);
	}

	if (kgetenv("DISPLAY") != NULL)
	   remote_setenv(resources, "DISPLAY", kgetenv("DISPLAY"));

	return(TRUE);
}

/*ARGSUSED*/
static ResourceStruct *start_daemon(
   const char *host)
{
	int i;
	ResourceStruct *resources;
	char location[KLENGTH], user[KLENGTH], remotecmd[KLENGTH];

	ktransport_username(host, user);
	ktransport_location(host, location);
	ktransport_remotecmd(host, remotecmd);
	for (i = 0; i < hnum; i++)
	{
	   if (hinfo[i]->busy == FALSE &&
	       kstrcmp(location, hinfo[i]->location) == 0 &&
	       kstrcmp(user, hinfo[i]->user) == 0 &&
	       kstrcmp(remotecmd, hinfo[i]->remotecmd) == 0)
	   {
	      return(hinfo[i]);
	   }
	}

        if ((resources = (ResourceStruct *) kcalloc((size_t) 1,
                                sizeof(ResourceStruct))) == NULL)
        {
           kinfo(KSYSLIB, "start_daemon: Not enough memory to kcalloc (%d) "
			  "bytes for the khoros file structure.",
			  sizeof(ResourceStruct));
           return(NULL);
        }
	resources->user      = kstrdup(user);
	resources->location  = kstrdup(location);
	resources->remotecmd = kstrdup(remotecmd);

	if (!remote_daemon(resources, "phantomd", REMOTE_TYPE_PHANTOMD) &&
	    !remote_daemon(resources, "kexec sh", REMOTE_TYPE_KEXEC)    &&
	    !remote_daemon(resources, "sh",       REMOTE_TYPE_SH))
	{
	   kfree(resources->user);
	   kfree(resources->location);
	   kfree(resources);
	   return(NULL);
	}
	hinfo = (ResourceStruct **) karray_insert(hinfo, KSTRUCT, resources,hnum++,KLIST_TAIL,TRUE);
	return(resources);
}

static int stop_daemon(
   const char *host)
{
	int i;
	ResourceStruct *resources;
	char location[KLENGTH], user[KLENGTH], remotecmd[KLENGTH];

	ktransport_username(host, user);
	ktransport_location(host, location);
	ktransport_remotecmd(host, remotecmd);
	for (i = 0; i < hnum; i++)
	{
	   if (kstrcmp(location, hinfo[i]->location) == 0 &&
	       kstrcmp(user, hinfo[i]->user) == 0 &&
	       kstrcmp(remotecmd, hinfo[i]->remotecmd) == 0)
	   {
	      resources = hinfo[i];
	      hinfo = (ResourceStruct **) karray_delete(hinfo, KSTRUCT,
				hinfo[i], hnum--);
	      kfree(resources->user);
	      kfree(resources->location);
	      kfree(resources->remotecmd);
	      kill(resources->pid, SIGTERM);
	      close(resources->fid[0]), close(resources->fid[1]);
	      kfree(resources);
	      return(TRUE);
	   }
	}
	return(FALSE);
}

/*-------------------------------------------------------------------*
|
|                 Remote Transport Routines
|
--------------------------------------------------------------------*/

/*-------------------------------------------------------------
|  
|  Routine Name: remote_tempnam
|  
|       Purpose: This function initializes a tempnam for a "remote"
|	         type transport.  Given a request with a filename of
|	         the following syntax:
|  
|  
|	         This routine creates a remote and returns the template
|	         in the following syntax:
|  
|  
|         Input:  identifier - the transport identifier
|		  ktemplate  - the filename template to be modeled
|        Output:  result     - the newly created transport name
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int remote_tempnam(
   const char *identifier,
   const char *ktemplate,
   char *result)
{
	char *temp, path[KLENGTH];

	(void) kstrcpy(path, ktemplate);
	if ((temp = ktmpnam(path)) == NULL)
	   return(-1);

	if (identifier == NULL)
	   (void) kstrcpy(result, temp);
	else
	   (void) sprintf(result,"%s=%s", identifier, temp);

	kfree(temp);
	return(0);
}

/*-------------------------------------------------------------
|  
|  Routine Name: remote_open
|  
|       Purpose: This function opens a "remote" connection.  It is the
|	         internal driver to open a remote, which is called by
|	         the user when they call kopen().  kopen() calls when
|	         it gets an open request with a filename of the following
|	         syntax:
|  
|         Input: 
|        Output:  returns 0 or -1 if an error occurs
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int remote_open(
   const char *path,
   int    flags,
   mode_t mode,
   kfile  *file)
{
	ResourceStruct *resources;
	char  command[KLENGTH];


	if ((resources = start_daemon(file->path)) == NULL)
	   return(-1);
	else
	   file->resources = (kaddr) resources;

	/*
	 *  Get all the pertinent info for the remote open command
	 */
	resources->busy = TRUE;
	if (resources->type == REMOTE_TYPE_PHANTOMD)
	{
	   if (kstrncmp(file->path, "remotexec", kstrlen("remotexec")) == 0)
	      ksprintf(command, "exec: %s\n", path), resources->reader = TRUE;
	   else if (flags & KOPEN_WRONLY || flags & KOPEN_RDWR)
	      ksprintf(command, "write: %s\n", path), resources->reader = FALSE;
	   else
	      ksprintf(command, "read: %s\n", path), resources->reader = TRUE;
	}
	else
	{
	   if (kstrncmp(file->path, "remotexec", kstrlen("remotexec")) == 0)
	      ksprintf(command, "%s; status=$?; echo %s", path, EosSymbol),
	      resources->reader = TRUE, resources->eos = EosSymbol;
	   else if (flags & KOPEN_WRONLY || flags & KOPEN_RDWR)
	      ksprintf(command,"cat > %s; status=$?; echo %s", path, EosSymbol),
	      resources->reader = FALSE;
	   else
	      ksprintf(command, "cat %s; status=$?; echo %s", path, EosSymbol),
	      resources->reader = TRUE, resources->eos = EosSymbol;
	}

	/*
	 *  Save the read and write transport for the remote pipe and pid (if
	 *  a remote command).
	if (resources->type != REMOTE_TYPE_PHANTOMD)
	{
	   kfile_clrstate(file, KFILE_STREAM);
	   ksetvbuf(file, NULL, _IOLBF, (size_t) 0);
	}
	 */
	remote_writeline(resources, command);
	return(0);
}

/*-------------------------------------------------------------
|  
|  Routine Name: remote_close
|  
|       Purpose: This function closes a "remote".  It is the internal driver
|	         to close a data remote, which is called by the user
|	         when they call kclose().  kclose() calls the "remote" internal
|	         drivers by the nature of the transport, which is dictated
|	         by the initial kopen().
|  
|         Input:  file - the kfile structure.
|        Output:  returns whether we were able to close the remote
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int remote_close(
   kfile *file)
{
	int status;
	char temp[KLENGTH], *tmp;
	ResourceStruct *resources = (ResourceStruct *) file->resources;

	while (resources->reader == TRUE && resources->busy != -1)
	   remote_read(file, temp, (size_t) KLENGTH);

	if (resources->type != REMOTE_TYPE_PHANTOMD)
	   remote_writeline(resources, "echo $status\n");

	remote_readline(resources, temp, (size_t) KLENGTH);
	status = strtol(temp, &tmp, 10);
	if (temp == tmp)
	   status = -1;

	resources->busy = FALSE;
	file->resources = NULL;
	return(status);
}

/*-------------------------------------------------------------
|  
|  Routine Name: remote_read
|  
|       Purpose: This function reads a "remote".  It is the internal driver
|	         to read data from a remote, which is called by the user
|	         when they call kread().  kread() calls the "remote" internal
|	         drivers by the nature of the transport, which is dictated
|	         by the kopen().
|  
|         Input:  file - the kfile structure.
|		  ptr  - the pointer to store the data into.
|		  nbytes - the number of bytes to read.
|        Output:  returns the number of bytes read from the file
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static ssize_t remote_read(
   kfile *file,
   kaddr ptr,
   size_t nbytes)
{
	size_t size;
	char *tmp, *buf = (char *)ptr;
	ssize_t count, numread = 0, cnt;
	ResourceStruct *resources = (ResourceStruct *) file->resources;

	/*
	 *  Simply call read() since "kfile" is just a regular file
	 *  descriptor.
	 */
	if (resources->busy == -1)
	   return(0);

	if (resources->type == REMOTE_TYPE_PHANTOMD)
	{
	   int  num;
	   char temp[KLENGTH];
           tmp = temp;

	   if (resources->readnum == 0)
	   {
	      do {
		if (remote_readline(resources, temp, (size_t) KLENGTH) == FALSE)
		   break;
	      } while (kstrlen(temp) == 0);

	      num = strtol(temp, &tmp, 10);
	      if (temp == tmp || num == -1)
	      {
	         kfile_setstate(file, KFILE_EOF);
		 resources->busy = -1;
	         return(0);
	      }
	      nbytes = kmin(nbytes, num);
	      resources->readnum = num - nbytes;
	   }
	   else
	   {
	      nbytes = kmin(nbytes, resources->readnum);
	      resources->readnum -= nbytes;
	   }
	}

	do
	{
	   size = nbytes - numread;
	   count = read(resources->fid[0], (kaddr) buf, size);
	   numread += count; buf += count;

	   if (resources->type != REMOTE_TYPE_PHANTOMD)
	   {
	      tmp = (char *) ptr;
	      cnt = kmin(numread, EosSize);
	      if (kstrncmp(&tmp[numread-cnt],resources->eos,(size_t) cnt) == 0)
	      {
	         resources->eos += cnt;
	         if ((resources->eos - EosSymbol) == EosSize)
	         {
	            if ((numread -= cnt) == 0)
		       numread = -1;
		    else
	               tmp[numread] = '\0';

	            kfile_setstate(file, KFILE_EOF);
	            resources->busy = -1;
	            resources->eos = EosSymbol;
		    break;
	         }
	      }
	      else
	         resources->eos = EosSymbol;
	   }
	}
	while (numread < nbytes && count > 0);
	kinfo(KDEBUG,"remote_read: read %d out of %d bytes '%s'\n", numread,
		     nbytes, file->path);
	return(numread);
}

/*-------------------------------------------------------------
|  
|  Routine Name: remote_write
|  
|       Purpose: This function writes to a "remote".  It is the internal driver
|	         to write data from the supplied data array to a remote, which
|	         is called by the user when they call kwrite().  kwrite() calls
|	         the "remote" internal drivers by the nature of the transport,
|	         which is dictated by the kopen().
|  
|         Input:  id   - the kfile structure.
|		  ptr  - the pointer to write the data from.
|		  nbytes - the number of bytes to read.
|        Output:  returns the number of bytes written to the kfile
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static ssize_t remote_write(
   kfile  *file,
   kaddr  ptr,
   size_t nbytes)
{
	size_t size;
	char *buf = (char *) ptr;
	ssize_t count, numwrite = 0;
	ResourceStruct *resources = (ResourceStruct *) file->resources;

	/*
	 *  Safety check..
	 */
	if (nbytes == 0)
	   return(0);
	else if (resources->busy == -1)
	   return(0);

	/*
	 *  Simply call write() since "kfile" is just a regular file
	 *  descriptor.
	 */
	do
	{
	   size = nbytes - numwrite;
	   count = write(resources->fid[1], (kaddr) buf, size);
	   numwrite += count; buf += count;
	} while (numwrite < nbytes && count > 0);
	kinfo(KDEBUG, "remote_write: wrote %d out of %d bytes '%s'\n", numwrite,
		      nbytes, file->path);
	return(numwrite);
}

/*-------------------------------------------------------------
|  
|  Routine Name: remote_access
|  
|       Purpose: This function is used to do see if a remote is accessable.
|	         This is called when the user calls kaccess().  kaccess()
|	         calls the "file" internal drivers by the nature of the
|	         transport, which is dictated by the ktransport_identifier().
|  
|         Input:  name - the original path name passed to kaccess
|		  path - the filename that they wish to access
|		  mode - the access mode information
|        Output:  returns the result
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int remote_access(
   const char *name,
   const char *path,
   int  mode)
{
	char  *flag = "-f";
	ResourceStruct *resources;
	char  command[KLENGTH], temp[KLENGTH];
	char  phcmd[] = "access: '%s' '%s'\n";
	char  shcmd[] = "if [ -%s %s ]; then echo yes; else echo no; fi\n";

	/*
	 *  Get all the pertinent info for the remote access command
	 */
	if ((resources = start_daemon(name)) == NULL)
	   return(-1);

	if (mode & W_OK) flag = "w";
	if (mode & X_OK) flag = "x";
	if (mode & R_OK) flag = "r";

	if (resources->type == REMOTE_TYPE_PHANTOMD)
	   ksprintf(command, phcmd, flag, path);
	else
	   ksprintf(command, shcmd, flag, path);

	remote_writeline(resources, command);
	remote_readline(resources, temp, (size_t) KLENGTH);
	if (kstrcmp(temp, "yes") == 0)
	{
	   errno = ENOENT;
	   return(-1);
	}
	return(0);
}

/*-------------------------------------------------------------
|  
|  Routine Name: remote_unlink
|  
|       Purpose: This function initializes a tempnam for a "file"
|	         type transport.  Given a request with a filename of
|	         the following syntax:
|  
|		  	"remote=[in,out]"
|  
|         Input:  name - the original path name passed to kunlink
|		  path - the filename that they wish to unlink
|        Output:  returns -1 or 0 depending whether we
|		  sucessfully created the template
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int remote_unlink(
   const char *name,
   const char *path)
{
	ResourceStruct *resources;
	char  command[KLENGTH], temp[KLENGTH];
	char  phcmd[] = "unlink: '%s'\n";
	char  shcmd[] = "/bin/rm -f %s; if [ $? = 0 ]; then echo yes;"
			"else echo no; fi\n";

	/*
	 *  Get all the pertinent info for the remote access command
	 */
        if ((resources = start_daemon(name)) == NULL)
           return(-1);

	if (resources->type == REMOTE_TYPE_PHANTOMD)
	   ksprintf(command, phcmd, path);
	else
	   ksprintf(command, shcmd, path);

	remote_writeline(resources, command);
	remote_readline(resources, temp, (size_t) KLENGTH);
	if (kstrcmp(temp, "yes") == 0)
	{
	   errno = ENOENT;
	   return(-1);
	}
	return(0);
}

/*-------------------------------------------------------------
|  
|  Routine Name: remote_descriptors
|  
|       Purpose: This function shouldn't exist, but there is occasion
|	         when the user wants to know the actual file descriptor
|	         associated with the file descriptor.
|  
|         Input: file structure
|        Output:  inum - the input file descriptor
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int remote_descriptors(
   kfile *file,
   int   *inum,
   int   *onum)
{
	ResourceStruct *resources = (ResourceStruct *) file->resources;

	if (inum != NULL) *inum = resources->fid[0];
	if (onum != NULL) *onum = resources->fid[1];
	return(0);
}

/*-------------------------------------------------------------
|  
|  Routine Name: kreminfo
|  
|       Purpose: This function shouldn't exist, but there is occasion
|	         when the user wants to know the actual process id
|	         of the associated local process controlling the
|		 remote daemon.
|  
|         Input: file structure
|        Output:  pid - the pid or -1 if it doesn't exist
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
pid_t kreminfo(
   kfile *file)
{
        ResourceStruct *resources;

	/*
	 *  Make sure the entry is within our table.  If not then it
	 *  has probably been closed already.
	 */
	if ((file = kfile_checkfile(file)) == NULL ||
	    file->routines != remote_transport)
	{
	   return(-1);
	}
	resources = (ResourceStruct *) file->resources;
	return(resources->pid);
}


/**************************************************************
*  
*  Routine Name: kremote_exec - creates a khoros command string which can
*				exec'ed on a remote host
*  
*       Purpose: This function make a proper command that the transport
*	         mechanisms will interpet as rsh://host/command.  If a
*	         command of "ls -al" is given and they wish to execute
*	         that command on the host "borris".  Then a new string
*	         will that looks like "rsh://borris/ls -al" will be returned.
*
*  
*         Input: host    - the host which we want to exec on
*	         command - the command to be executed
*        Output: remotexec - if NULL malloc the return string
*	Returns:
*    Written By: Mark Young
*          Date: Jun 17, 1995
*  
**************************************************************/

char *kremote_exec(
   const char *machine,
   const char *command,
   char *remotexec)
{
	char temp1[10*KLENGTH], temp2[KLENGTH], temp3[KLENGTH],
	     cmd[KLENGTH], user[KLENGTH], location[KLENGTH], *tmp1, *tmp2;

	if (remotexec == NULL)
	   remotexec = temp1;

	tmp1 = kstring_cleanup(command, temp2);
	tmp2 = kstring_cleanup(machine, temp3);
	tmp2 = kexpand_variable(tmp2, tmp2);
	if (tmp2 != NULL && kstrcmp(tmp2, "localhost") != 0)
	{
	   khost_entry(tmp2, tmp2);
	   ktransport_username(NULL, user); kstrcpy(cmd, DefaultRemoteCmd);
	   if (ksscanf(tmp2, "%[^:]:%[^:]:%[^:]", location, user, cmd) > 0)
	      ksprintf(remotexec,"%s://%s:%s/%s", cmd, location, user, tmp1);
	   else
	      kstrcpy(remotexec, tmp1);
	}
	else
	   kstrcpy(remotexec, tmp1);

	return(remotexec == temp1 ? kstrdup(remotexec) : remotexec);
}

/**************************************************************
*  
*  Routine Name: kremote_file - creates a khoros file string which can
*				be accessed on a remote machine
*  
*       Purpose: This function make a proper file that the transport
*	         mechanisms will interpet as rsh://host/file.  If the
*	         file ".login" is given and they wish to access
*	         that file on the host "borris".  Then a new string
*	         will that looks like "rsh://borris/.login" will be returned.
*  
*         Input: host     - the host which we want to access the file on
*	         filename - the file to be accessed
*        Output: remotefile - if NULL malloc the return string
*	Returns:
*  
*    Written By: Mark Young
*          Date: Jun 17, 1995
*  
**************************************************************/

char *kremote_file(
   const char *machine,
   const char *filename,
   char *remotefile)
{
	char temp1[10*KLENGTH], temp2[KLENGTH], temp3[KLENGTH],
	     cmd[KLENGTH], user[KLENGTH], location[KLENGTH], *file, *name;

	if (remotefile == NULL)
	   remotefile = temp1;

	file = kstring_cleanup(filename, temp2);
	name = kstring_cleanup(machine, temp3);
	name = kexpand_variable(name, name);
	if (name != NULL && kstrcmp(name, "localhost") != 0)
	{
	   khost_entry(name, name);
	   ktransport_username(NULL, user); kstrcpy(cmd, DefaultRemoteCmd);
	   if (ksscanf(name, "%[^:]:%[^:]:%[^:]", location, user, cmd) > 0)
	      ksprintf(remotefile,"%s://%s:%s/%s", cmd, location, user, file);
	   else
	      kstrcpy(remotefile, file);
	}
	else
	   kstrcpy(remotefile, file);

	return(remotefile == temp1 ? kstrdup(remotefile) : remotefile);
}

/**************************************************************
*  
*  Routine Name: kremote_location - return the remote location
*  
*       Purpose: This function is used to find the location or
*                machine in which the user wants to re-direct to
*                via some transport.  The location returned is
*	         the form of "machine" or "machine:username" or
*		 NULL if the location is local.
*  
*         Input: name     - the file or command to be parsed
*        Output: location - if NULL malloc the return string
*	Returns:
*  
*    Written By: Mark Young
*          Date: Aug 4, 1995
*  
**************************************************************/

char *kremote_location(
   const char *name,
   char *location)
{
	char temp1[10*KLENGTH], temp2[10*KLENGTH],
	     machine[10*KLENGTH], user[10*KLENGTH], cmd[10*KLENGTH], *tmp;

	if (location == NULL)
	   location = temp1;
	else
	   location[0] = '\0';

	ktransport_username(NULL, user);
	tmp = kexpand_variable(name, temp2);
	if (ksscanf(tmp, "%[^:]://%[^:/]:%[^/]", cmd, machine, user) < 2)
	   return(NULL);

	ksprintf(location, "%s:%s:%s", machine, user, cmd);
	return(location == temp1 ? kstrdup(location) : location);
}

/**************************************************************
*  
*  Routine Name: kremote_check - check to see if we can access host
*
*       Purpose: This function is used to see if we can successfully
*	         access the host in which to distributed processes.
*                Hostname must be specified in the form "machine:user:cmd",
*                for example, cibola:sandy:rsh
*
*         Input: host - the host to be checked
*	Returns: return TRUE (1) on success, FALSE (0) otherwise
*    Written By: Mark Young
*          Date: Jun 27, 1995
**************************************************************/
int kremote_check(
   const char *host)
{
	return(TRUE);
}

/**************************************************************
*  
*  Routine Name: kremote_start - starts daemon on specified host
*
*       Purpose: This function is used to 
*	         start the khoros process daemon on the specified host.
*                Hostname must be specified in the form "machine:user:cmd",
*                for example, sandia:george:rsh
*
*         Input: host - the host to be started
*	Returns: return TRUE (1) on success, FALSE (0) otherwise
*    Written By: Mark Young
*          Date: Jun 27, 1995
**************************************************************/
int kremote_start(
   const char *host)
{
	char location[KLENGTH], user[KLENGTH], cmd[KLENGTH], temp[KLENGTH];

	khost_username(host, user);
	khost_location(host, location);
	khost_remotecmd(host, cmd);
	ksprintf(temp, "%s://%s:%s", cmd, location, user);
	if (start_daemon(temp) == NULL)
	   return(FALSE);

	return(TRUE);
}

/**************************************************************
*  
*  Routine Name: kremote_stop - stops daemon on the specified host
*
*       Purpose: Stops the khoros process daemon on the specified host.
*                Hostname must be specified in the form "machine:user:cmd",
*                for example, acoma:debbie:rsh
*
*         Input: host - the host to be stopped
*	Returns: return TRUE (1) on success, FALSE (0) otherwise
*    Written By: Mark Young
*          Date: Jun 27, 1995
*
**************************************************************/
int kremote_stop(
   const char *host)
{
	char location[KLENGTH], user[KLENGTH], cmd[KLENGTH], temp[KLENGTH];

	khost_username(host, user);
	khost_location(host, location);
	khost_remotecmd(host, cmd);
	ksprintf(temp, "%s://%s:%s", cmd, location, user);
	if (stop_daemon(temp) == FALSE)
	   return(FALSE);

	return(TRUE);
}

/**************************************************************
*  
*  Routine Name: kremote_running - Check to see if host is currently 
*                                  running daemon
*
*       Purpose: Returns TRUE if the host is currently running a 
*                khoros process daemon, FALSE otherwise.
*                Hostname must be specified in the form "machine:user:cmd",
*                for example, nambe:arthur:rsh
*
*         Input: host - the host to check
*	Returns: return TRUE (1) if the host is running, FALSE (0) otherwise
*    Written By: Mark Young 
*          Date: June 1, 1996
*
**************************************************************/

int kremote_running(
   const char *host)
{
	int  i;
	char location[KLENGTH], user[KLENGTH], remotecmd[KLENGTH];

	khost_username(host, user);
	khost_location(host, location);
	khost_remotecmd(host, remotecmd);
	for (i = 0; i < hnum; i++)
	{
	   if (kstrcmp(location, hinfo[i]->location) == 0 &&
	       kstrcmp(user, hinfo[i]->user) == 0 &&
	       kstrcmp(remotecmd, hinfo[i]->remotecmd) == 0)
	   {
	      return(TRUE);
	   }
	}
	return(FALSE);
}

/**************************************************************
*
*  Routine Name: kremote_list - List the remote hosts with daemons started
*
*       Purpose: This function returns an allocated list of the remote hosts 
*                that have khoros process daemons started.
*                Hostnames will be specified in the form "machine:user:cmd",
*                for example, fantase:blair:rsh
*
*         Input: host - the host to check
*       Returns: An array of strings containing the remote hosts with
*                active daemons on success, NULL on failure
*    Written By: Danielle Argiro & Mark Young
*          Date: July 1, 1996
*
**************************************************************/
char **kremote_list(
   size_t *num)
{
	char   **all = NULL, **active = NULL;
	int    i; 
	size_t allnum = 0, activenum = 0;

	all = khost_list(&allnum);
	for (i = 0; i < allnum; i++)
        {
	   if (kstrncmp("localhost", all[i], kstrlen("localhost")) == 0 ||
	       kremote_running(all[i]))
           {
	      active = (char **) krealloc(active, (size_t)(activenum+1)*sizeof(char *));
	      active[activenum++] = kstrdup(all[i]);
           }
        }
	*num = activenum;
	return(active);
}
