/*
 * Khoros: $Id: array.c,v 1.15.2.1 1997/03/06 22:55:10 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: array.c,v 1.15.2.1 1997/03/06 22:55:10 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Array manipulation utilities
   >>>>		
   >>>>   Static:
   >>>>                 kvarg_entry()
   >>>>                 ktype_sizeof()
   >>>>                 kfind_index()
   >>>>                 compare_entry()
   >>>>   Public:
   >>>>                 karray_insert()
   >>>>                 karray_delete()
   >>>>                 karray_free()
   >>>>                 karray_locate()
   >>>>                 karray_sort()
   >>>>                 karray_merge()
   >>>>                 karray_to_list()
   >>>>                 karray_to_string()
   >>>>                 karray_filelist()
   >>>>                 karray_dirlist()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

#define KMAXENTRIES  KLENGTH

/*-----------------------------------------------------------
|
|  Routine Name: kvarg_entry - gets the entry off the arg list
|
|       Purpose: This routine gets the entry off the arg list.
|         Input: list - the variable arg list
|       Returns: returns a pointer to the argument
|
|    Written By: Mark Young & Steve Jorgensen & John Salas
|          Date: Jun 06, 1995
|
------------------------------------------------------------*/
static kaddr kvarg_entry(int type, kaddr entry, kva_list *list)
{
	switch (type)
	{
	   case KBYTE:
		{
		   signed char *data = (signed char *) entry;
		   *data = (signed char) kva_arg(*list, signed int);
		}
		break;

	   case KUBYTE:
		{
		   unsigned char *data = (unsigned char *) entry;
		   *data = (unsigned char) kva_arg(*list, unsigned int);
		}
		break;

	   case KSHORT:
		{
		   short *data = (signed short *) entry;
		   *data = (signed short) kva_arg(*list, signed int);
		}
		break;

	   case KUSHORT:
		{
		   unsigned short *data = (unsigned short *) entry;
		   *data = (unsigned short) kva_arg(*list, unsigned int);
		}
		break;

	   case KINT:
		{
		   signed int *data = (signed int *) entry;
		   *data = (signed int) kva_arg(*list, signed int);
		}
		break;

	   case KUINT:
		{
		   unsigned int *data = (unsigned int *) entry;
		   *data = (unsigned int) kva_arg(*list, unsigned int);
		}
		break;

	   case KLONG:
		{
		   signed long *data = (signed long *) entry;
		   *data = (signed long) kva_arg(*list, signed long);
		}
		break;

	   case KULONG:
		{
		   unsigned long *data = (unsigned long *) entry;
		   *data = (unsigned long) kva_arg(*list, unsigned long);
		}
		break;

	   case KFLOAT:
		{
		   float *data = (float *) entry;
		   *data = (float) kva_arg(*list, double);
		}
		break;

	   case KDOUBLE:
		{
		   double *data = (double *) entry;
		   *data = (double) kva_arg(*list, double);
		}
		break;

	   case KSTRING:
		{
		   char **data = (char **) entry;
		   *data = (char *) kva_arg(*list, char *);
		}
		break;

	   case KSTRUCT:
		{
		   kaddr *data = (kaddr *) entry;
		   *data = (kaddr) kva_arg(*list, kaddr);
		}
		break;

	   case KLOGICAL:
		{
		   kbool *data = (kbool *) entry;
		   *data = (kbool) kva_arg(*list, int);
		}
		break;

	   default:
		return(NULL);
	}
	return(entry);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktype_sizeof - find the entry in the array
|
|       Purpose: This routine finds the 
|         Input: 
|       Returns: 
|
|    Written By: Mark Young & Steve Jorgensen & John Salas
|          Date: Jun 06, 1995
|
------------------------------------------------------------*/
static size_t ktype_sizeof(int type)
{
	static size_t esize;
	static int otype = -1;

	if (type == otype)
	   return(esize);

	switch (type)
	{
	   case KSTRING:
	   case KSTRUCT:
	        esize = sizeof(kaddr);
		break;

	   case KBYTE:
	   case KUBYTE:
		esize = sizeof(char);
		break;

	   case KSHORT:
	   case KUSHORT:
		esize = sizeof(short);
		break;

	   case KINT:
	   case KUINT:
		esize = sizeof(int);
		break;

	   case KLONG:
	   case KULONG:
		esize = sizeof(long);
		break;

	   case KFLOAT:
		esize = sizeof(float);
		break;

	   case KDOUBLE:
		esize = sizeof(double);
		break;

	   case KLOGICAL:
		esize = sizeof(kbool);
		break;

	   default:
		return((size_t) -1);
	}
	otype = type;
	return(esize);
}

/*-----------------------------------------------------------
|
|  Routine Name: kfind_index - find the entry in the array
|
|       Purpose: This routine finds the entry within the array
|         Input: 
|       Returns: 
|
|    Written By: Mark Young & Steve Jorgensen & John Salas
|          Date: Jun 06, 1995
|
------------------------------------------------------------*/
static int kfind_index(void *array, int type, kaddr entry, size_t num)
{
	size_t i, esize;
	char *tmp = (char *) array;

	esize = ktype_sizeof(type);
        if (type == KSTRING)
	{
	   for (i = 0, tmp = (char *)array; i < num; i++, tmp += esize)
              if (kstrcmp(*((char **) tmp), *((char **) entry)) == 0)
                 return(i);
	}
	else if (type == KSTRUCT)
	{
	   for (i = 0, tmp = (char *)array; i < num; i++, tmp += esize)
              if (*((void **) tmp) == *((void **) entry))
                 return(i);
	}
	else
	{
	   for (i = 0, tmp = (char *)array; i < num; i++, tmp += esize)
              if (kmemcmp(tmp, entry, esize) == 0)
                 return(i);
	}
	return(-1);
}

/************************************************************
*
*  Routine Name: karray_insert - insert an entry into an array 
*
*       Purpose: Inserts an entry into the array at the specified
*		 position, and returns the modified array to the
*		 calling routine.
*
*		 The array utilities, including karray_insert(), support 
*		 arrays of a variety of data types.  The data type of the 
*		 array being used is specified with the 'type' parameter; 
*		 supported data types include:
*
*		 !	KBYTE    - array of characters
*		 !	KUBYTE   - array of unsigned characters
*		 !	KSHORT   - array of short integers
*		 !	KUSHORT  - array of unsigned short integers
*		 !	KINT     - array of integers
*		 !	KUINT    - array of unsigned integers
*		 !	KLONG    - array of long integers
*		 !	KULONG   - array of unsigned long integers
*		 !	KFLOAT   - array of floating point numbers
*		 !	KDOUBLE  - array of double precision numbers
*		 !	KSTRING  - array of strings
*		 !	KSTRUCT  - array of pointers to structures
*		 !	KLOGICAL - array of TRUE/FALSE values
*
*		 Note that the array and the entry passed in MUST
*		 both be of the type specified by the type parameter,
*		 or the results of this routine are unpredictable.
*
*		 The position parameter indicates where in the array
*		 an entry should be inserted.  Supported positions include:
*
*		 !	KLIST_HEAD - insert entry at the head of the array
*		 !	KLIST_PREV - insert entry just before current position
*		 !	KLIST_NEXT - insert entry just after current position
*		 !	KLIST_TAIL - insert entry at the end of the array
*
*
*	  Input: array      - The array to which to add the entry
*		 type	    - The data type of the array and the entry
*		 entry      - The entry to be added to the array
*		 num	    - The number of entries currently in the array
*			      (before the new entry is inserted)
*		 position   - Where in the array to add the entry; provide
*			      one of KLIST_HEAD, KLIST_PREV, KLIST_NEXT, or
*			      KLIST_TAIL.
*		 duplicates - TRUE if multiple occurances of an entry 
*			      should be allowed in the array, FALSE if only
*			      one occurance of each entry should be allowed.
*			      If this routine is called with 'duplicates' set 
*			      to FALSE and the entry provided already appears 
*			      as in the array, the array will remain unchanged.
*        Output: 
*       Returns: The head of the modified array
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Jun 06, 1995
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: <type> *karray_insert(
*		!  <type> *array,
*		!   int    type,
*		!  <type>  entry,
*		!   size_t num,
*		!   int    position,
*		!   int    duplicates)
*
*************************************************************/

void *karray_insert(
   void *array,
   int   type,
   kvalist)
{
	kva_list list;
	kaddr    entry;
	char    *tmp, space[KLENGTH];
	int      position, duplicates;
	size_t   num, size, pos, esize;


	/*
	 *  Get the entry off the list
	 */
	kva_start(list, type);
	entry = kvarg_entry(type, space, &list);

	/*
	 *  Get the rest of the arguments
	 */
	num        = kva_arg(list, size_t);
	position   = kva_arg(list, int);
	duplicates = kva_arg(list, int);

	/*
	 *  If duplicate entries aren't allowed then make sure that the
	 *  entry doesn't currently exist anywhere in the list
	 */
	if (!duplicates && kfind_index(array, type, entry, num) != -1)
	   return(array);

	if (array == NULL)
	   num = 0;

	esize = ktype_sizeof(type);
	if (num == 0 || (num % KMAXENTRIES) == 0)
	{
	   size = (num + KMAXENTRIES) * esize;
	   if ((array = (void *) krealloc(array, size)) == NULL)
	   {
              kerror("kutils", "karray_insert", "Unable to krealloc enough "
		     "memory for an array list of size %d", num);
              return(NULL);
	   }
	}

	/*
	 *  indicates that the programmer wants the entry inserted at
	 *  the head or tail of the list.
	 */
	tmp = (char *)array;
	if (position == KLIST_HEAD || position == KLIST_PREV)
	{
	   pos = 0;
	   if (num > 0) kmemmove(tmp+esize, tmp, num*esize);
	}
	else pos = num*esize;

	if (type == KSTRUCT || type == KSTRING)
	   *((kaddr *) (tmp+pos)) = *((kaddr *) entry);
	else
	   kmemcpy(tmp+pos, entry, esize);
	return(array);
}

/************************************************************
*
*  Routine Name: karray_delete - delete an entry from an array
*
*       Purpose: Deletes an entry from the array, and returns
*		 the modified array to the calling routine.
*
*                The array utilities, including karray_delete(), support
*                arrays of a variety of data types.  The data type of the
*                array being used is specified with the 'type' parameter;
*                supported data types include:
*
*                !      KBYTE    - array of characters
*                !      KUBYTE   - array of unsigned characters
*                !      KSHORT   - array of short integers
*                !      KUSHORT  - array of unsigned short integers
*                !      KINT     - array of integers
*                !      KUINT    - array of unsigned integers
*                !      KLONG    - array of long integers
*                !      KULONG   - array of unsigned long integers
*                !      KFLOAT   - array of floating point numbers
*                !      KDOUBLE  - array of double precision numbers
*                !      KSTRING  - array of strings
*                !      KSTRUCT  - array of pointers to structures
*                !      KLOGICAL - array of TRUE/FALSE values
*
*		 Note that the array and the entry passed in MUST
*		 both be of the type specified by the type parameter,
*		 or the results of this routine are unpredictable.
*
*	  Input: array - The array from which to delete the entry  
*		 entry - The entry to be deleted from the array 
*		 num   - The number of entries currently in the array
*                        (before the entry is deleted).
*
*       Returns: The modified array when there are still entries in the array, 
*		 NULL when the array becomes empty.
*    Written By: Mark Young
*          Date: Jun 06, 1995
*
*   Declaration: <type> *karray_delete(
*	         !  <type> *array,
*	         !   int    type,
*	         !  <type>  entry,
*	         !   size_t num)
*
*************************************************************/

void *karray_delete(
   void *array,
   int   type,
   kvalist)
{
	kva_list list;
	kaddr entry;
	size_t num, indx, esize;
	char  *tmp, space[KLENGTH];


	/*
	 *  Sanity check...
	 */
	if (array == NULL)
	   return(NULL);

	/*
	 *  Get the entry off the list
	 */
	kva_start(list, type);
	entry = kvarg_entry(type, space, &list);

	/*
	 *  Get the rest of the arguments
	 */
	num = kva_arg(list, size_t);

	/*
	 *  Make sure that the entry exists
	 */
	if ((indx = kfind_index(array, type, entry, num--)) == -1)
	   return(array);

	/*
	 *  If the number left over is 0 and the entry we just delete is 0
	 *  this means that the array is empty and we need to free it.
	 */
	esize = ktype_sizeof(type);
	tmp = (char *)array;
	if (num == 0)
	   kfree_and_NULL(array);
	else
	   kmemcpy(tmp+esize*indx, tmp+esize*(indx+1),
			(unsigned) (num-indx)*esize);

	return(array);
}

/************************************************************
*
*  Routine Name: karray_free - free memory used by an array 
*
*       Purpose: Frees the memory associated with the use of an array.  
*
*                The array utilities, including karray_free(), support
*                arrays of a variety of data types.  The data type of the
*                array being used is specified with the 'type' parameter;
*                supported data types include:
*
*                !      KBYTE    - array of characters
*                !      KUBYTE   - array of unsigned characters
*                !      KSHORT   - array of short integers
*                !      KUSHORT  - array of unsigned short integers
*                !      KINT     - array of integers
*                !      KUINT    - array of unsigned integers
*                !      KLONG    - array of long integers
*                !      KULONG   - array of unsigned long integers
*                !      KFLOAT   - array of floating point numbers
*                !      KDOUBLE  - array of double precision numbers
*                !      KSTRING  - array of strings
*                !      KSTRUCT  - array of pointers to structures
*                !      KLOGICAL - array of TRUE/FALSE values
*
*		 If desired, you may provide a routine to destroy each 
*		 element of the array.  Generally, this feature is used only
*		 for arrays of type KSTRUCT, where the structure in question
*		 contains pointers to strings or other structures that must 
*		 be freed individually.  If used, the routine specified
*		 must be declared as follows:
*                !
*		 ! int free_routine(void *)
*                !
*		 It must return TRUE on success and FALSE on failure.  
*		 It must take a single parameter with which to pass
*		 the pointer to the element to free, cast as a (void *).  
*		 If the routine parameter is NULL, then karray_free() simply 
*		 uses kfree_and_NULL() to free the array.
*
*         Input: array    - The array to be freed
*                type     - The data type of the array and the entry
*		 num      - The number of entries currently in the array
*		 routine  - The routine to destroy each element if a 
*			    specialized free routine is desired; NULL
*		            implies that kfree_and_NULL() should be used.
*        Output: 
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*    Written By: Mark Young
*          Date: Jun 06, 1995
*   Declaration: <type> *karray_free(
*                !  <type> *array,
*                !  int     type,
*                !  size_t  num,
*		 !  void  (*routine) (<type>)
*
*************************************************************/

void karray_free(
   void    *array,
   int      type,
   size_t   num,
   void   (*routine) PROTO((void *)) )
{
	kaddr *ptr;
	size_t  i, esize;

	if (!array)
	   return;

	esize = ktype_sizeof(type);
	if (type == KSTRING)
	{
           for (i = 0, ptr = (kaddr *) array; i < num; i++)
	      (!routine) ? kfree_and_NULL(ptr[i]) : (*routine)(ptr[i]);
	}
	else if (type == KSTRUCT && routine != NULL)
	{
           for (i = 0, ptr = (kaddr *) array; i < num; i++)
	      if (routine && ptr[i]) (*routine)(ptr[i]);
	}
	kfree_and_NULL(array);
}

/************************************************************
*
*  Routine Name: karray_locate - locate an entry in an array
*
*       Purpose: Locate an entry in the array.  If the entry exists
*		 in the array, then the index to that entry is returned.  
*		 If the entry doesn't exist in the array, then -1 is returned.
*
*                The array utilities, including karray_locate(), support
*                arrays of a variety of data types.  The data type of the
*                array being used is specified with the 'type' parameter;
*                supported data types include:
*
*                !      KBYTE    - array of characters
*                !      KUBYTE   - array of unsigned characters
*                !      KSHORT   - array of short integers
*                !      KUSHORT  - array of unsigned short integers
*                !      KINT     - array of integers
*                !      KUINT    - array of unsigned integers
*                !      KLONG    - array of long integers
*                !      KULONG   - array of unsigned long integers
*                !      KFLOAT   - array of floating point numbers
*                !      KDOUBLE  - array of double precision numbers
*                !      KSTRING  - array of strings
*                !      KSTRUCT  - array of pointers to structures
*                !      KLOGICAL - array of TRUE/FALSE values
*
*	  Input: array - The array in which to search for the entry 
*                type  - The data type of the array
*		 entry - The entry to be be located in the array
*		 num   - The number of entries currently in the array
*
*        Output: 
*       Returns: Returns the index of the entry if it is found, -1 otherwise
*    Written By: Mark Young
*          Date: Jun 06, 1995
*
*   Declaration: ssize_t karray_locate(
*		 !  <type>  *array,
*		 !   int    type,
*		 !  <type>  entry,
*		 !   size_t num)
*
*************************************************************/

ssize_t karray_locate(
   void  *array,
   int   type,
   kvalist)
{
	size_t num;
	kva_list list;
	kaddr entry;
	char  space[KLENGTH];


	/*
	 *  Sanity check...
	 */
	if (array == NULL)
	   return(-1);

	/*
	 *  Get the entry off the list
	 */
	kva_start(list, type);
	entry = kvarg_entry(type, space, &list);

	/*
	 *  Get the rest of the arguments
	 */
	num = kva_arg(list, size_t);
	return(kfind_index(array, type, entry, num));
}

/************************************************************
*
*  Routine Name: karray_sort - sort an array 
*
*       Purpose: Sorts a string array into ascending order. 
*
*                Other array utilities support arrays of a variety of 
*                data types.  The karray_sort() routine, however, can
*                only be used with string arrays.
*
*         Input: array      - The array to sort
*                type       - The data type of the array
*		 num        - The number of entries in the array
*		 duplicates - TRUE if any duplicate entries should be left
*		              in the sorted array, FALSE to remove any
*		              duplicate entries from the sorted array
*        Output: 
*       Returns: The pointer to the header of the sorted array; 
*		 NULL on failure
*    Written By: Mark Young
*          Date: Jun 06, 1995
*  Side Effects: The sorted array passed back is the same as the original
*		 array passed in.  So original input array should not be
*		 sorted.
*
*   Declaration: <type> *karray_sort(
*		 !  <type> *array,
*		 !  int     type,
*		 !  size_t  num,
*		 !  int     duplicates)
*
*************************************************************/

static int compare_entry(
   const void *str1,
   const void *str2)
{
	return(kstrcmp(*((char **) str1), *((char **) str2)));
}

void *karray_sort(
   void *array,
   int  type,
   size_t num,
   int  duplicates)
{
	size_t esize;

	if (array == NULL || type != KSTRING)
	   return(NULL);

	esize = ktype_sizeof(type);
	qsort(array, num, esize, compare_entry);
	return(array);
}

/************************************************************
*
*  Routine Name: karray_merge - merge two arrays into one
*
*       Purpose: Merges two arrays into a single array by
*		 concatenating the second array onto the first.  
*
*                The array utilities, including karray_merge(), support
*                arrays of a variety of data types.  The data type of the
*                array being used is specified with the 'type' parameter;
*                supported data types include:
*
*                !      KBYTE    - array of characters
*                !      KUBYTE   - array of unsigned characters
*                !      KSHORT   - array of short integers
*                !      KUSHORT  - array of unsigned short integers
*                !      KINT     - array of integers
*                !      KUINT    - array of unsigned integers
*                !      KLONG    - array of long integers
*                !      KULONG   - array of unsigned long integers
*                !      KFLOAT   - array of floating point numbers
*                !      KDOUBLE  - array of double precision numbers
*                !      KSTRING  - array of strings
*                !      KSTRUCT  - array of pointers to structures
*                !      KLOGICAL - array of TRUE/FALSE values
*
*		 For string arrays ('type' == KSTRING), strings in 
*		 the merged array are only copied if the 'copy_entries' 
*		 parameter is passed as TRUE.  Otherwise, each entry in
*		 the merged array will simply point to the old string.
*
*         Input: array1	      - First array of strings
*		 array2       - Second array of strings
*                type         - The data type of the array
*		 num1         - Number of entries in array1
*		 num2         - Number of entries in array2
*		 copy_entries - For string arrays, specifies whether or
*		                not to copy the strings into the merged
*			        array, or simply to duplicate the pointers;
*				ignored for arrays of all other data types.
*        Output: 
*       Returns: A pointer to the merged array on success, NULL on failure.
*    Written By: Mark Young
*          Date: Jun 06, 1995
*  Side Effects: Allocates memory for the merged array
* 
*   Declaration: <type> *karray_merge(
*                !  <type> *array1,
*                !  <type> *array2,
*                !  int     type,
*                !  size_t  num1,
*                !  size_t  num2,
*                !  int     copy_entries)
*
************************************************************/

void *karray_merge(
   void  *array1,
   void  *array2,
   int    type,
   size_t num1,
   size_t num2,
   int    copy_entries)
{
	char *tmp, *temp;
	size_t i, num, esize, indx = 0;


	if (array1 == NULL && array2 == NULL)
	   return(NULL);

	num = num1+num2;
	if ((num % KMAXENTRIES) != 0)
	   num += KMAXENTRIES - (num % KMAXENTRIES);

	esize = ktype_sizeof(type);
	temp = (char *) kcalloc(num, esize);

	tmp = (char *)array1;
	if (tmp != NULL)
	{
	   for (i = 0; i < num1; i++, indx++)
	   {
	      if (copy_entries == TRUE && type == KSTRING)
		 ((char **) temp)[indx] = kstrdup(((char **) tmp)[i]);
	      else
		 kmemcpy(temp+indx*esize, tmp+i*esize, esize);
	   }
	}

	tmp = (char *)array2;
	if (tmp != NULL)
	{
	   for (i = 0; i < num2; i++, indx++)
	   {
	      if (copy_entries == TRUE && type == KSTRING)
		 ((char **) temp)[indx] = kstrdup(((char **) tmp)[i]);
	      else
		 kmemcpy(temp+indx*esize, tmp+i*esize, esize);
	   }
	}
	return(temp);
}

/************************************************************
*
*  Routine Name: karray_to_list - convert an array into a linked list
*
*       Purpose: Creates a linked list from an array.  The resulting
*		 list may be used with the Khoros linked list utilities,
*		 such as klist_add(), klist_insert(), klist_delete(), 
*		 klist_copy(), and so on.
*
*                The array utilities, including karray_to_list(), support
*                arrays of a variety of data types.  The data type of the
*                array being used is specified with the 'type' parameter;
*                supported data types include:
*
*                !      KBYTE    - array of characters
*                !      KUBYTE   - array of unsigned characters
*                !      KSHORT   - array of short integers
*                !      KUSHORT  - array of unsigned short integers
*                !      KINT     - array of integers
*                !      KUINT    - array of unsigned integers
*                !      KLONG    - array of long integers
*                !      KULONG   - array of unsigned long integers
*                !      KFLOAT   - array of floating point numbers
*                !      KDOUBLE  - array of double precision numbers
*                !      KSTRING  - array of strings
*                !      KSTRUCT  - array of pointers to structures
*                !      KLOGICAL - array of TRUE/FALSE values
*
*	  Input: array - The array from which the linked list 
*			 will be created.
*                type  - The data type of the array
*		 num   - The number of entries in the array.
*
*       Returns: The newly created linked list on success,
*		 NULL upon failure
*    Written By: Mark Young
*          Date: Jun 06, 1995
*
*   Declaration: klist *karray_to_list(
*                !  <type> *array,
*                !  int     type,
*                !  size_t  num)
*
*************************************************************/

klist *karray_to_list(
   void *array,
   int  type,
   size_t num)
{
	size_t   i;
	klist   *list = NULL;
	char    **tmp = (char **) array;


	/*
	 *  Copy each list entry into the array
	 */
	for (i = 0; i < num; i++)
	   list = klist_insert(list, tmp[i], NULL, KLIST_TAIL, TRUE);

	return(list);
}

/************************************************************
*
*  Routine Name: karray_to_string - convert a string array 
*				    into a single big string
*
*       Purpose: Creates one big string from an array of strings.
*
*		 While most of the array routines support a variety
*		 of data types, karray_to_string() requires a
*		 string array.  Arrays of other data types are
*		 not supported as of yet.
*
*		 A specified separation string is inserted
*		 between each pair of strings in the array. 
*		 For example, if the input array had the contents:
*		 !  array[0] = "ls"
*		 !  array[1] = "-a"
*		 !  array[2] = "/tmp"
*
*		 and the 'separator' parameter was specified as " ", 
*		 then the string returned will be:
*		 !  "ls -a /tmp"
*
*		 If the 'separator' parameter was passed in as NULL,
*		 then the resulting string would be:
*		 !  "ls-a/tmp"
*
*	  Input: array     - The string array from which the return
*			     string will be created.
*                type      - The data type of the array
*		 num       - The number of entries in the array.
*		 separator - The separation string to be inserted between
*			     each pair of strings in the array.
*
*       Returns: The newly created string on success, NULL on failure
*    Written By: Mark Young
*          Date: Jun 06, 1995
*
*************************************************************/

char *karray_to_string(
   char   **array,
   int      type,
   size_t   num,
   char    *separator)
{
	size_t i;
	char  *temp, *string = NULL;
	char  **tmp = (char **) array;

	/*
	 *  sanity check
	 */
	if (num <= 0)
	   return(NULL);
	if (type != KSTRING)
	{
	    kerror("kutils", "karray_to_string",
		   "Only arrays of type KSTRING are supported");
	    return(NULL);
	}

	/*
	 *  Copy each list entry into the array
	 */
	for (i = 0; i < num - 1; i++)
	{
	   temp = kstring_3cat(string, tmp[i], separator, NULL);
	   kfree_and_NULL(string); string = temp;
	}
	temp = kstring_cat(string, tmp[i], NULL);
	kfree_and_NULL(string); string = temp;
	return(string);
}

/************************************************************
*
*  Routine Name: karray_filelist - create an array of strings 
*				   reflecting the contents of a file
*
*       Purpose: Translates the contents of a file into an
*                array of strings, where each string in the array
*                is a line of the specified file.  If desired,
*		 the resulting string array may also be sorted
*		 in alphabetical order.
*
*                The 'global_dir' parameter specifies the directory
*                path to the file which will be used to create of the 
*		 string array. This directory path may include toolbox 
*		 names (for example, $BOOTSTRAP) or tildas (for example, 
*		 ~fred).  Passing NULL implies that the 'filename' parameter 
*		 will be used to specify the directory as well as the 
*		 filename.
*
*		 The 'filename' parameter specifies the name of the
*		 file which is to have its contents translated into
*		 a string array.  If the 'global_dir' parameter is used
*		 to specify the directory path, the 'filename' parameter
*		 should only contain the name of the file;  if the 'global_dir'
*		 parameter is passed as NULL, then the 'filename' parameter
*		 must include the directory path to the file. 
*
*         Input: filename    - The name of the file to have its contents
*		               translated into a string array.
*		 global_dir  - The prefix to the filename
*		 sort_flag   - Pass TRUE if the resulting array is to be
*		               sorted alphabetially, FALSE otherwise
*
*        Output: num         - The number of entries in the resulting
*			       string array 
*
*       Returns: The array of strings representing the lines in the file
*		 on success, NULL on failure.
*  Restrictions: 
*    Written By: Mark Young
*          Date: Jun 06, 1995
*
*************************************************************/

char **karray_filelist(
   char   *filename,
   char   *global_dir,
   int     sort_flag,
   size_t *num)
{
	size_t	inum = 0;
	char    *data, *begin, *tmp, path[KLENGTH], **array = NULL;


	if (num) *num= 0;
	if (global_dir == NULL && filename == NULL)
	{
	   errno = KNULL_PARAMETER;
	   kerror("kutils", "kfilearray",
		  "NULL input filename and directory encountered");
	   return(NULL);
	}

	/*
	 *  Expand the path to get the full path to the directory.
	 */
	if (kstrlen(filename) == 0)
	   kfullpath(global_dir, NULL, path);
	else
	   kfullpath(filename, global_dir, path);

	/*
	 *  Read the data and create an array of NULL terminated strings
	 *  for each line within the file.
	 */
	begin = data = (char *)kreadfile(path, NULL);
	while (data != NULL)
	{
	   if ((tmp = kstrchr(data, '\n')) != NULL)
	      *tmp++ = '\0';

	   if ((data = kstring_cleanup(data, NULL)) != NULL)
	      array = (char **) karray_insert(array, KSTRING, data, inum++,
					      KLIST_TAIL, TRUE);

	   data = tmp;
	}
	kfree_and_NULL(begin);

	/*
	 *  Need to see if the user wants to sort the entries into
	 *  ascending order.
	 */
	if (inum > 0 && sort_flag == TRUE)
	   array = (char **) karray_sort(array, KSTRING, inum, TRUE);

	if (num) *num= inum;
	return(array);
}

/************************************************************
*
*  Routine Name: karray_dirlist - create an array of strings
*				  reflecting directory contents
*
*       Purpose: Translates the contents of a directory into an
*		 array of strings, where each string in the array
*		 is the name of an entry in the specified directory.
*
*		 The 'global_dir' parameter specifies the directory
*		 path to use for creating of the string array. This
*		 directory path may include toolbox names (for example,
*		 $BOOTSTRAP) or tildas (for example, ~fred).  Passing
*		 NULL implies that the 'basename' parameter will be
*		 used to specify the directory.
*
*	         The 'basename' parameter allows you to specify a
*		 set of letters which must be matched by the entries in
*		 the directory before they are included in the resulting
*		 array.  For example, providing "img" as the basename will
*		 cause only those entries in the directory beginning with the
*		 letters "img" to be included in the array. Passing NULL
*		 indicates that ALL entries in the directory specified by
*		 'global_dir' are acceptable for inclusion in the array 
*		 (unless otherwise specified using the 'filter' parameter).
* 
*		 For your convenience, the values of both the 'global_dir'
*		 and the 'basename' parameters may be combined into the
*		 value of the 'basename' parameter.  For example, if you
*		 wanted to create the string array from all entries in the 
*		 ~fred/data directory beginning with the letters "d3", then
*		 you could pass "~fred/data/d3" as the 'basename' and NULL
*		 as the 'global_dir'.
*
*		 More powerful and flexible than the 'basename' parameter, as 
*		 well as more difficult to use, the 'filter' parameter allows 
*		 you to specify a more complicated filter that entries in the 
*		 directory must match before they are included in the array. 
*		 The regular expression syntax is used to specify the filter;
*		 see \fIkre_comp()\fP for the syntax required to specify the
*		 desired pattern matching.
*
*		 The 'list_mode' parameter indicates which entries
*		 in the directory are to be included in the resulting
*		 array of strings.  The following list modes are supported:
*
*		 !	KPATH		- prepend the path to each file
*		 !	KFILE		- list plain text files
*		 !	KDIR		- list subdirectories
*		 !	KDOT		- list dot files
*		 !	KLINK		- list symbolic files
*		 !	KSOCK		- list socket files
*		 !	KREAD		- file is readable by caller
*		 !	KWRITE		- file is writable by caller
*		 !	KEXEC		- file is executable by caller
*		 !	KRECURSE	- recursively list all subdirectories
*
*		The list modes may be or'ed together in order to specify
*		the entries in the directory that should be included in the
*		string array produced.  For example, "KFILE | KDIR" will cause
*		the resulting string array to have entries for will list only 
*		files and directories.
*
*         Input: basename    - Basename of the entries in the directory to be 
*			       included as strings in the array.
*		 global_dir  - The prefix to the basename
*		 filter	     - Filter to be used in matching the directory
*			       entries. If filter is NULL then all entries are 
*			       accepted (unless otherwise indicated by 
*			       basename).
*		 list_mode   - A flag indicating which directory entries are
*			       to be included (eg. files, directories, dot 
*			       files, symbolic links, etc.)
*		 format      - Whether the files should be postfixed with
*			       the symbol indicating the type of entry (eg. 
*			       "@" for symbolic links, "/" for subdirectories,
*			       etc)
*
*        Output: num - The number of entries in the resulting string array 
*
*       Returns: The array of strings representing the directory contents
*		 on success, NULL on failure
*    Written By: Mark Young
*          Date: Jun 06, 1995
*
*************************************************************/

char **karray_dirlist(
   char   *basename,
   char   *global_dir,
   char   *filter,
   int     list_mode,
   int     format,
   size_t *num)
{
	DIR	*dir;
	struct  dirent *file;

	struct  stat buf;
	int	check_perm, status;
	char    *dirpath, *fullpath, entry[KLENGTH], **array = NULL,
		**dirlist = NULL;
        size_t  i, inum = 0, dirnum = 0, base_len;

	*num = 0;
	if (global_dir == NULL && basename == NULL)
	{
	   errno = KNULL_PARAMETER;
	   kerror("kutils", "kdirarray",
		  "NULL input basename and directory encountered");
	   return(NULL);
	}

	/*
	 *  Expand the path to get the full path to the directory.
	 */
	if (kstrlen(basename) == 0)
	{
	   kstring_cat(global_dir, "/", entry);
	   fullpath = kfullpath(entry, NULL,  NULL);
	}
	else
	   fullpath = kfullpath(basename, global_dir, NULL);

	/*
	 *  Get the directory path to the file
	 */
	if (stat(fullpath, &buf) == -1 || !S_ISDIR(buf.st_mode))
	{
	   if ((dirpath = kdirname(fullpath,NULL)) == NULL)
	   {
	      kfree_and_NULL(fullpath);
	      return(NULL);
	   }
	   basename = kbasename(fullpath,NULL);
	}
	else
	{
	   basename = NULL;
	   dirpath  = kstrdup(fullpath);
	}

	/*
	 *  Open the directory
	 */
	if ((dir = opendir(dirpath)) == NULL)
	{
	   kfree_and_NULL(fullpath); kfree_and_NULL(dirpath); kfree_and_NULL(basename);
	   return(NULL);
	}
	base_len = kstrlen(basename);

	/*
	 *  Read the entries and see if the file compares to that of the
	 *  basename we are looking for.  If the basename is NULL then
	 *  we add all regular files.
	 */
	check_perm = 0;
	if (list_mode & KREAD)  check_perm |= R_OK;
	if (list_mode & KWRITE) check_perm |= W_OK;
	if (list_mode & KEXEC)  check_perm |= X_OK;
	while ((file = readdir(dir)) != NULL)
	{
	   if (basename != NULL)
	   {
	      if (kstrncmp(file->d_name, basename, base_len) != 0)
	         continue;
	   }
	   else if ((list_mode & KDOT) == 0 && file->d_name[0] == '.')
	   {
	      if (((list_mode & KDIR) && kstrcmp(file->d_name,"..") != 0) ||
		  ((list_mode & KDIR) == 0))
	         continue;
	   }

	   /*
	    *  Check to see if the user wants to look at file permissions.
	    *  If so then we check the access of the "check_perm" mode(s).
	    */
	   if (dirpath[kstrlen(dirpath)-1] == '/')
	      (void) ksprintf(entry, "%s%s", dirpath, file->d_name);
	   else
	      (void) ksprintf(entry, "%s/%s", dirpath, file->d_name);
	   if (check_perm)
	   {
	      if (kaccess(entry, check_perm) == -1)
	         continue;
	   }

	   /*
	    *  Need to get infomation (status) about the file.  If the user
	    *  wants to know about symbolic links then we'll call lstat()
	    *  otherwise we will call stat().
	    */
#ifndef SYSV
	   if (list_mode & KLINK)
	   {
	      if (lstat(entry, &buf) == -1) /* get stats for file */
	         continue;
	   }
	   else
#endif
	   {
	      if (stat(entry, &buf) == -1)  /*  get stats for file */
	         continue;
	   }

	   /*
	    *  If we are suppose to recurse then we better save the directories
	    *  so that we don't have to rewind and stat all the files again.
	    */
	   if ((list_mode & KRECURSE) && S_ISDIR(buf.st_mode) &&
	       kstrcmp(file->d_name,"..") != 0 &&
	       kstrcmp(file->d_name,".")  != 0)
	   {
	      dirlist = (char **) karray_add(dirlist, KSTRING, kstrdup(entry),
					     dirnum++);
	   }

	   /*
	    *  Unless the want the entire path, get rid of it.
	    */
	   if ((list_mode & KPATH) == 0)
	      (void) ksprintf(entry, "%s", file->d_name);


	   /*
	    *  If the filter is not NULL then see if the entry matches
	    *  what we are looking for.
	    */
	   if (filter != NULL)
	   {
	      (void) kparse_string_search(entry,filter,KLITERAL,NULL,&status);
	      if (status != KPARSE_OK)
		 continue;
	   }

	   /*
	    *  Check the kind of file we have.  Make sure it part of the
	    *  array of files to be be arrayed.
	    */
	   if (S_ISDIR(buf.st_mode) && (list_mode & KDIR))
	   {
	      if (format) kstrcat(entry,"/");
	   }

#if defined(S_ISLNK)
	   else if (S_ISLNK(buf.st_mode) && (list_mode & KLINK))
	   {
	      if (format) kstrcat(entry,"@");
	   }
#endif

#if defined(S_ISSOCK)
	   else if (S_ISSOCK(buf.st_mode) && (list_mode & KSOCK))
	   {
	      if (format) kstrcat(entry,"=");
	   }
#endif
	   else if (S_ISREG(buf.st_mode) && (list_mode & KFILE))
	   {
	      if (format) kstrcat(entry," ");
	   }
	   else
	      continue;

	   array = (char **) karray_add(array, KSTRING, kstrdup(entry), inum);
	   inum++;
	}

	/*
	 *  We need to make sure that the array of filenames is greater than 0.
	 *  If not then kfree_and_NULL the array and set it to NULL, otherwise we need
	 *  to sort the entries into descending order.
	 */
	if (inum == 0)
	{
	   kfree_and_NULL(array);
	   array = NULL;
	}
	else
	{
	   array = (char **) karray_sort(array, KSTRING, inum, TRUE);
	}

	if (list_mode & KRECURSE)
	{
	   size_t lnum = 0;
	   char **list, **tmp;

	   for (i = 0; i < dirnum; i++)
	   {
	      list = karray_dirlist(dirlist[i], NULL, filter, list_mode,
					format, &lnum);
	      tmp = (char **) karray_merge(array, list, KSTRING, inum, lnum,
					   FALSE);
	      kfree_and_NULL(array);
	      kfree_and_NULL(list);
	      array = tmp;
	      inum += lnum;
	   }
	   karray_free(dirlist, KSTRING, dirnum, NULL);
	}
	*num= inum;

	/*
	 *  Cleanup time.  Free the fullpath and dirpath, and the basename if
	 *  currently kallocated.
	 */
	kfree_and_NULL(fullpath); kfree_and_NULL(dirpath);
	if (basename != NULL) kfree_and_NULL(basename);
	closedir(dir);
	return(array);
}
