/*
 * Khoros: $Id: hash.c,v 1.22.2.4 1997/04/14 22:04:40 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: hash.c,v 1.22.2.4 1997/04/14 22:04:40 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $CONVERT/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros Hash Utility
   >>>>		
   >>>>   Static:
   >>>>                 ktype_sizeof()
   >>>>                 knext_entry()
   >>>>                 kvarg_entry()
   >>>>			krehash_entries()
   >>>>                 klocate_entry()
   >>>>                 kadd_entry()
   >>>>                 khash_resort()
   >>>>   Public:
   >>>>                 khash_location()
   >>>>                 khash_value()
   >>>>                 khash_init()
   >>>>                 khash_add()
   >>>>                 khash_clientdata()
   >>>>                 khash_check()
   >>>>                 khash_delete()
   >>>>                 khash_free()
   >>>>                 khash_reset()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/*-----------------------------------------------------------
|
|  Routine Name: ktype_sizeof - find the size of the type
|
|       Purpose: This routine finds the 
|         Input: 
|       Returns: 
|
|    Written By: Mark Young & Steve Jorgensen & John Salas
|          Date: Jun 06, 1995
|
------------------------------------------------------------*/
static size_t ktype_sizeof(int type)
{
	switch (type)
	{
	   case KSTRING:
	   case KSTRUCT:
	        return(sizeof(kaddr));
	   default:
		return(kmach_sizeof(KMACH_LOCAL, (unsigned) type));
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: kvarg_entry - gets the entry off the arg list
|
|       Purpose: This routine gets the entry off the arg list.
|         Input: list - the variable arg list
|       Returns: returns a pointer to the argument
|
|    Written By: Mark Young
|          Date: Sep 13, 1995
|
------------------------------------------------------------*/
static kaddr kvarg_entry(int type, kaddr entry, kva_list *list)
{

#define doit(dtype, ltype, entry, list) \
{								\
	dtype *data = (dtype *) entry;				\
	*data  = (dtype) kva_arg(*list, ltype);			\
}
	switch (type)
	{
	   case KBYTE:   doit(signed char, signed int, entry, list); break;
	   case KUBYTE:  doit(unsigned char, unsigned int, entry, list); break;
	   case KSHORT:  doit(signed short, signed int, entry, list); break;
	   case KUSHORT: doit(unsigned short, unsigned int, entry, list); break;
	   case KINT:    doit(signed int, signed int, entry, list); break;
	   case KUINT:   doit(unsigned int, unsigned int, entry, list); break;
	   case KLONG:   doit(signed long, signed long, entry, list); break;
	   case KULONG:  doit(unsigned long, unsigned long, entry, list); break;
	   case KFLOAT:  doit(float, double, entry, list); break;
	   case KDOUBLE: doit(double, double, entry, list); break;
	   case KSTRUCT: doit(kaddr, kaddr, entry, list); break;
	   case KLOGICAL: doit(kbool, signed int, entry, list); break;
	   case KSTRING:  entry = kva_arg(*list, kstring); break;
	   default:
		return(NULL);
	}
#undef doit
	return(entry);
}

/*-----------------------------------------------------------
|
|  Routine Name: knext_entry - find the next available entry
|
|       Purpose: This routine finds the next available entry
|		 for the hash routines.
|         Input: 
|       Returns: 
|
|    Written By: Mark Young
|          Date: Jan 11, 1997
|
------------------------------------------------------------*/
static long knext_entry(
   unsigned char *occupied,
   unsigned long indx,
   size_t size)
{
	int  i;

	if (occupied[indx] == FALSE)
	   return(indx);

	for (i = indx+1; i < size; i++)
	   if (occupied[i] == FALSE) return(i);

	for (i = 0; i < indx; i++)
	   if (occupied[i] == FALSE) return(i);

	return(-1);
}

/*-----------------------------------------------------------
|
|  Routine Name: krehash_entries - rehashes the entries
|
|       Purpose: This routine rehashes the entries in order to allow
|		 the hash to dynamically resize.
|         Input: 
|       Returns: 
|
|    Written By: Mark Young
|          Date: Sep 13, 1995
|
------------------------------------------------------------*/
static void krehash_entries(khash *hash, size_t size)
{
	int	 i;
	long	 indx;
	kaddr    table, *data;
	unsigned char *occupied;
	unsigned long *hashvals;

	table = (kaddr) kmalloc(size * hash->esize);
	data  = (kaddr *) kmalloc(size * sizeof(kaddr));
	occupied = (unsigned char *) kcalloc(size, sizeof(unsigned char));
	hashvals = (unsigned long *) kmalloc(size * sizeof(unsigned long));

	if (!hashvals || !occupied || !data || !table)
	{
	   kerror(NULL, NULL, "Cannot resize table to be size '%d'", size);
	   return;
	}

#define doit(dtype, indx) \
{									      \
	dtype *ntable = (dtype *) table;				      \
	dtype *otable = (dtype *) hash->table;				      \
	for (i = 0; i < hash->size; i++)				      \
	{								      \
	   if (hash->occupied[i] == FALSE)				      \
	      continue;							      \
									      \
	   indx = knext_entry(occupied, hash->hashvals[i] % size, size);      \
	   occupied[indx] = TRUE;					      \
	   ntable[indx]   = otable[i];					      \
	   data[indx]     = hash->data[i];				      \
	   hashvals[indx] = hash->hashvals[i];				      \
	}								      \
}
	switch (hash->type)
	{
	   case KBYTE:    doit(signed char, indx); break;
	   case KUBYTE:   doit(unsigned char, indx); break;
	   case KSHORT:   doit(signed short, indx); break;
	   case KUSHORT:  doit(unsigned short, indx); break;
	   case KINT:     doit(signed int, indx); break;
	   case KUINT:    doit(unsigned int, indx); break;
	   case KLONG:    doit(signed long, indx); break;
	   case KULONG:   doit(unsigned long, indx); break;
	   case KFLOAT:   doit(float, indx); break;
	   case KDOUBLE:  doit(double, indx); break;
	   case KSTRUCT:  doit(kaddr, indx); break;
	   case KLOGICAL: doit(kbool, indx); break;
	   case KSTRING:  doit(kstring, indx); break;
	   default: break;
	}
#undef doit
	hash->size = size;
	kfree(hash->data);  hash->data  = data;
        kfree(hash->table); hash->table = table;
        kfree(hash->hashvals); hash->hashvals = hashvals;
        kfree(hash->occupied); hash->occupied = occupied;
}

/*-----------------------------------------------------------
|
|  Routine Name: kadd_entry - adds the entry into the array
|
|       Purpose: This routine adds the entry in to the array
|         Input: 
|       Returns: 
|
|    Written By: Mark Young
|          Date: Sep 13, 1995
|
------------------------------------------------------------*/
static int kadd_entry(khash *hash, unsigned long hash_val, kaddr entry, kaddr client_data)
{
	int      indx;
	size_t   size = hash->size;

	/*
	 *  Bump our count and see if we have plenty of room left in the
	 *  hash.  Since this is a closed hash, if the hash goes full we
	 *  will need to grow the hash and rehash the current entries.
	 */
	if (++hash->count > (size>>2))
	{
	   krehash_entries(hash, size<<2);
	   size = hash->size;
	}
	indx = knext_entry(hash->occupied, hash_val % size, size);

#define doit(dtype, indx, table, entry) \
{									      \
	dtype *data = (dtype *) entry;					      \
	dtype *table = (dtype *) hash->table;				      \
	table[indx] = *data;						      \
}
	switch (hash->type)
	{
	   case KBYTE:    doit(signed char, indx, table, entry); break;
	   case KUBYTE:   doit(unsigned char, indx, table, entry); break;
	   case KSHORT:   doit(signed short, indx, table, entry); break;
	   case KUSHORT:  doit(unsigned short, indx, table, entry); break;
	   case KINT:     doit(signed int, indx, table, entry); break;
	   case KUINT:    doit(unsigned int, indx, table, entry); break;
	   case KLONG:    doit(signed long, indx, table, entry); break;
	   case KULONG:   doit(unsigned long, indx, table, entry); break;
	   case KFLOAT:   doit(float, indx, table, entry); break;
	   case KDOUBLE:  doit(double, indx, table, entry); break;
	   case KSTRUCT:  doit(kaddr, indx, table, entry); break;
	   case KLOGICAL: doit(kbool, indx, table, entry); break;
	   case KSTRING:
		{
		   kstring data = (kstring) entry;
		   kstring *table = (kstring *) hash->table;
		   table[indx] = kstrdup(data);
		}
		break;
	   default: return(FALSE);
	}
#undef doit
	hash->occupied[indx] = TRUE;
	hash->hashvals[indx] = hash_val;
	hash->data[indx]  = client_data;
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: klocate_entry - gets the location of the entry 
|
|       Purpose: This routine gets the location of the entry in the array
|         Input: 
|       Returns: 
|
|    Written By: Mark Young
|          Date: Sep 13, 1995
|
------------------------------------------------------------*/
static ssize_t klocate_entry(khash *hash, ssize_t indx, kaddr entry)
{
	int i;
	size_t size = hash->size;
	unsigned char *occupied = hash->occupied;

#define doit(dtype, table, entry) \
{								\
	dtype *data = (dtype *) entry;				\
	dtype *table = (dtype *) hash->table;			\
	for (i = indx; i < size; i++)				\
	   if (!occupied[i] || table[i] == *data)		\
	      return(!occupied[i] ? -1 : i);			\
	for (i = 0; i < indx; i++)				\
	   if (!occupied[i] || table[i] == *data)		\
	      return(!occupied[i] ? -1 : i);			\
}
	switch (hash->type)
	{
	   case KBYTE:   doit(signed char, table, entry); break;
	   case KUBYTE:  doit(unsigned char, table, entry); break;
	   case KSHORT:  doit(signed short, table, entry); break;
	   case KUSHORT: doit(unsigned short, table, entry); break;
	   case KINT:    doit(signed int, table, entry); break;
	   case KUINT:   doit(unsigned int, table, entry); break;
	   case KLONG:   doit(signed long, table, entry); break;
	   case KULONG:  doit(unsigned long, table, entry); break;
	   case KFLOAT:  doit(float, table, entry); break;
	   case KDOUBLE: doit(double, table, entry); break;
	   case KSTRUCT: doit(kaddr, table, entry); break;
	   case KLOGICAL: doit(kbool, table, entry); break;
	   case KSTRING:
		{
		   kstring data  = (kstring) entry;
		   kstring *table = (kstring *) hash->table;
		   for (i = indx; i < size; i++)
		      if (!occupied[i] || !kstrcmp(table[i], data))
		         return(!occupied[i] ? -1 : i);
		   for (i = 0; i < indx; i++)
		      if (!occupied[i] || !kstrcmp(table[i], data))
		         return(!occupied[i] ? -1 : i);
		}
		break;
	   default: return(-1);
	}
#undef doit
	return(-1);
}

/*-----------------------------------------------------------
|
|  Routine Name: khash_resort - resorts the hash array after a deletion
|
|       Purpose: This routine fills moves up collisions after an element
|		 is removed from the list.
|
|    Written By: Mark Young
|          Date: Sep 13, 1995
|
------------------------------------------------------------*/
static void khash_resort(khash *hash, size_t indx)
{
   size_t i, start = indx;

#define doit(dtype) 						      \
{								      \
   dtype *table = (dtype *) hash->table;			      \
   for (i = start+1; i < hash->size; i++)			      \
      if (hash->occupied[i] && hash->hashvals[i] % hash->size <= indx) \
      {								      \
         table[indx]          = table[i];			      \
         hash->occupied[indx] = TRUE;				      \
         hash->data[indx]     = hash->data[i];			      \
         indx                 = i;				      \
	 hash->occupied[indx] = FALSE;				      \
      }								      \
      else if (!hash->occupied[i]) break;			      \
   if (i == hash->size)						      \
      for (i = 0; i < start; i++)				      \
         if (hash->occupied[i] &&				      \
	     (hash->hashvals[i] % hash->size <= indx || indx >= start)) \
         {							      \
            table[indx]          = table[i];			      \
            hash->occupied[indx] = TRUE;			      \
            hash->data[indx]     = hash->data[i];		      \
            indx                 = i;				      \
	    hash->occupied[indx] = FALSE;			      \
         }							      \
         else if (!hash->occupied[i]) break;			      \
}
   switch (hash->type)
   {
      case KBYTE:    doit(signed char); break;
      case KUBYTE:   doit(unsigned char); break;
      case KSHORT:   doit(signed short); break;
      case KUSHORT:  doit(unsigned short); break;
      case KINT:     doit(signed int); break;
      case KUINT:    doit(unsigned int); break;
      case KLONG:    doit(signed long); break;
      case KULONG:   doit(unsigned long); break;
      case KFLOAT:   doit(float); break;
      case KDOUBLE:  doit(double); break;
      case KSTRUCT:  doit(kaddr); break;
      case KLOGICAL: doit(kbool); break;
      case KSTRING:  doit(kstring); break;
      default: break;
   }
#undef doit
}

/************************************************************
*
*  Routine Name: khash_value - polynomial conversion 
*
*       Purpose: polynomial conversion ignoring overflows
*		 [this seems to work remarkably well, in fact better
*		 then the ndbm hash function. Replace at your own risk]
*
*		 ! 	use:  65599	nice.
*		 ! 	      65587   even better. 
*
*         Input: data - the data in which to create the has for
*		 length - the length of the hash to be used.  If -1,
*			  then the length is computed using kstrlen().
*        Output:
*       Returns: returns the associated hash for the given data.
*
*  Restrictions:
*    Written By: Ozan S. Yigit (oz@nexus.yorku.ca)
*          Date: 1991 
*      Verified:
*  Side Effects:
* Modifications: Integrated into Khoros 2.0 (MY) 4/93
*   Declaration: unsigned long int khash_value(
*		!   char *data,
*		!   ssize_t length)
*
*************************************************************/

unsigned long khash_value(
   register const char *data,
   register ssize_t  length)
{
	unsigned long n;

	if (length == 4)
	   return(((((data[0]*65599+data[1])*65599)+data[2])*65599)+data[3]);
	else if (length <= -1)
	   length = (ssize_t) kstrlen(data);

	n = 0;
#define HASHC	n = *data++ + 65599 * n
	switch (length & 7)
	{
	   case 7: HASHC;
	   case 6: HASHC;
	   case 5: HASHC;
	   case 4: HASHC;
	   case 3: HASHC;
	   case 2: HASHC;
	   case 1: HASHC;
	}

	/*
	 *  Divide the length since we know that the rest is divisible
	 *  by 8.
	 */
	length >>= 3;

	while (length-- > 0)
	{
	    HASHC; HASHC; HASHC; HASHC;
	    HASHC; HASHC; HASHC; HASHC;
	}
#undef HASHC

	return n;
}

/************************************************************
*
*  Routine Name: khash_init - initializes a hash array 
*
*       Purpose: Initializes a hash array, which can then be used
*		 to add, delete, check, and retrieve data associated
*		 with hash entries.
*
*		 The hash utilities, including khash_add(), support 
*		 hash arrays of a variety of data types.  The data type of the 
*		 array being used is specified with the 'type' parameter; 
*		 supported data types include:
*
*		 !	KBYTE    - hash array of characters
*		 !	KUBYTE   - hash array of unsigned characters
*		 !	KSHORT   - hash array of short integers
*		 !	KUSHORT  - hash array of unsigned short integers
*		 !	KINT     - hash array of integers
*		 !	KUINT    - hash array of unsigned integers
*		 !	KLONG    - hash array of long integers
*		 !	KULONG   - hash array of unsigned long integers
*		 !	KFLOAT   - hash array of floating point numbers
*		 !	KDOUBLE  - hash array of double precision numbers
*		 !	KSTRING  - hash array of strings
*		 !	KSTRUCT  - hash array of pointers to structures
*		 !	KLOGICAL - hash array of TRUE/FALSE values
*
*	  Input: type - The hash data type
*		 size - The initial hash size
*        Output: 
*       Returns: the newly initialized hash structure, or NULL on failure
*    Written By: Mark Young
*          Date: Sep 13, 1995
*************************************************************/
khash *khash_init(
   int    type,
   size_t size)
{
	khash *hash;

	if ((hash = (khash *) kcalloc((size_t) 1, sizeof(khash))) == NULL)
	   return(NULL);

	hash->size  = size;
	hash->type  = type;
	hash->esize = ktype_sizeof(type);
	hash->table = (kaddr)   kmalloc(size * hash->esize);
	hash->data  = (kaddr *) kmalloc(size * sizeof(kaddr));
	hash->occupied = (unsigned char *) kcalloc(size, sizeof(char));
	hash->hashvals = (unsigned long *) kmalloc(size*sizeof(unsigned long));
	return(hash);
}

/************************************************************
*
*  Routine Name: khash_add - adds an entry into the hash array
*
*       Purpose: Inserts an entry into the hash array, and an
*		 associated piece of data.
*
*		 The hash utilities, including khash_add(), support 
*		 hash arrays of a variety of data types.  The data type of the 
*		 array being used is specified with the 'type' parameter; 
*		 supported data types include:
*
*		 !	KBYTE    - hash array of characters
*		 !	KUBYTE   - hash array of unsigned characters
*		 !	KSHORT   - hash array of short integers
*		 !	KUSHORT  - hash array of unsigned short integers
*		 !	KINT     - hash array of integers
*		 !	KUINT    - hash array of unsigned integers
*		 !	KLONG    - hash array of long integers
*		 !	KULONG   - hash array of unsigned long integers
*		 !	KFLOAT   - hash array of floating point numbers
*		 !	KDOUBLE  - hash array of double precision numbers
*		 !	KSTRING  - hash array of strings
*		 !	KSTRUCT  - hash array of pointers to structures
*		 !	KLOGICAL - hash array of TRUE/FALSE values
*
*		 Note that the hash array and the entry passed in MUST
*		 both be of the type specified by the type parameter,
*		 originally used in creating the hash array or the results
*		 of this routine are unpredictable.
*
*	  Input: hash        - The hash array to which to add the entry
*		 entry       - The entry to be added to the hash array
*		 client_data - The piece of data to be associated with the
*			       hash.
*        Output: 
*       Returns: TRUE (1) on success, FALSE (0) otherwise.
*    Written By: Mark Young
*          Date: Sep 13, 1995
*
*   Declaration: int khash_add(
*		!  khash *hash,
*		!  <type>  entry,
*		!  kaddr client_data)
*************************************************************/
int khash_add(
   khash *hash,
   kvalist)
{
	kva_list list;
	unsigned long indx;
	char     space[KLENGTH];
	kaddr    entry, client_data;

	/*  Sanity check */
	if (hash == NULL)
	   return(FALSE);

	/*
	 *  Get the entry off the list
	 */
	kva_start(list, hash);
	entry = kvarg_entry(hash->type, space, &list);

	/*
	 *  Get the rest of the arguments
	 */
	client_data = kva_arg(list, kaddr);
	kva_end(list);

	if (hash->type == KSTRING)
	   indx = khash_value((const char *) entry, -1);
	else
	   indx = khash_value((const char *) entry, hash->esize);
	if (kadd_entry(hash, indx, entry, client_data) == FALSE)
	   return(FALSE);

	return(TRUE);
}

/************************************************************
*
*  Routine Name: khash_location - return the location of the entry within
*				  hash
*
*       Purpose: This function returns the location within the hash of the
*		 entry.  This is sometimes useful when the index is needed
*		 for temporary amount of time.
*
*	  Input: hash - the hash entry
*       Returns: The index of the entry or -1 if the entry is not found or
*		 is not valid.
*    Written By: Mark Young
*          Date: Sep 13, 1995
*
*   Declaration: long khash_location(
*		!  khash *hash,
*		!  <type>  entry)
*************************************************************/
long khash_location(
   khash *hash,
   kvalist)
{
        ssize_t indx;
        kva_list list;
        kaddr entry;
        char  space[KLENGTH];
 
        /*  Sanity check */
        if (hash == NULL)
           return(FALSE);
 
        /*
         *  Get the entry off the list
         */
        kva_start(list, hash);
        entry = kvarg_entry(hash->type, space, &list);
        kva_end(list);
 
	if (hash->type == KSTRING)
	   indx = khash_value((const char *) entry, -1) % hash->size;
	else
	   indx = khash_value((const char *) entry, hash->esize) % hash->size;

        indx = klocate_entry(hash, indx, entry);
        return((long) indx);
}

/************************************************************
*
*  Routine Name: khash_clientdata - return the client data associated with
*				 the hash entry
*
*       Purpose: This function returns the client data that is associated with
*		 the hash entry.  The idea is that a unique hash entry
*		 is given for each entry sent to the function
*		 khash_add().  This routine is used to retrieve a
*		 pointer to the hash's associated client data.
*
*	  Input: hash - the hash entry
*       Returns: The token's corresponding string or NULL, if the token is
*		 not valid.
*    Written By: Mark Young
*          Date: Sep 13, 1995
*
*   Declaration: int khash_clientdata(
*		!  khash *hash,
*		!  <type>  entry,
*		!  kaddr *client_data)
*************************************************************/
int khash_clientdata(
   khash *hash,
   kvalist)
{
	kva_list list;
	ssize_t  indx;
	char     space[KLENGTH];
	kaddr    entry, *client_data;

	/*  Sanity check */
	if (hash == NULL)
	   return(FALSE);

	/*
	 *  Get the entry off the list
	 */
	kva_start(list, hash);
	entry = kvarg_entry(hash->type, space, &list);

	/*
	 *  Get the rest of the arguments
	 */
	client_data = kva_arg(list, kaddr *);
	kva_end(list);

	if (hash->type == KSTRING)
	   indx = khash_value((const char *) entry, -1) % hash->size;
	else
	   indx = khash_value((const char *) entry, hash->esize) % hash->size;
	if ((indx = klocate_entry(hash, indx, entry)) == -1)
	   return(FALSE);

	*client_data = hash->data[indx];
	return(TRUE);
}

/************************************************************
*
*  Routine Name: khash_check - check to see if a hash entry exists
*
*       Purpose: This function checks to see if a hash entry has already
*		 added using khash_add().
*
*	  Input: hash - the hash array
*       Returns: TRUE (1) on success, FALSE (0) otherwise.
*    Written By: Mark Young
*          Date: Sep 13, 1995
*
*   Declaration: int khash_check(
*		!  khash *hash,
*		!  <type>  entry)
*************************************************************/
int khash_check(
   khash *hash,
   kvalist)
{
	kva_list list;
	kaddr    entry;
	unsigned long indx;
	char     space[KLENGTH];

	/*  Sanity check */
	if (hash == NULL)
	   return(FALSE);
	
	/*
	 *  Get the entry off the list
	 */
	kva_start(list, hash);
	entry = kvarg_entry(hash->type, space, &list);
	kva_end(list);

	if (hash->type == KSTRING)
	   indx = khash_value((const char *) entry, -1) % hash->size;
	else
	   indx = khash_value((const char *) entry, hash->esize) % hash->size;
        if (klocate_entry(hash, (ssize_t) indx, entry) == -1)
           return(FALSE);

	return(TRUE);
}

/************************************************************
*
*  Routine Name: khash_delete - delete an entry from the hash array
*
*       Purpose: This function deletes an entry from the hash array.
*
*	  Input: hash - the hash array to delete the entry from
*       Returns: TRUE (1) on success, FALSE (0) otherwise.
*    Written By: Mark Young
*          Date: Sep 13, 1995
*
*   Declaration: int khash_delete(
*		!  khash *hash,
*		!  <type>  entry)
*************************************************************/
int khash_delete(
   khash *hash,
   kvalist)
{
	kva_list list;
	kaddr    entry;
	unsigned long indx;
	char     space[KLENGTH];

	/*  Sanity check */
	if (hash == NULL)
	   return(FALSE);

	/*
	 *  Get the entry off the list
	 */
	kva_start(list, hash);
	entry = kvarg_entry(hash->type, space, &list);
	kva_end(list);
	if (hash->type == KSTRING)
	   indx = khash_value((const char *) entry, -1) % hash->size;
	else
	   indx = khash_value((const char *) entry, hash->esize) % hash->size;

	/*
	 *  Locate the entry in the hash.  If it's not there then
	 *  return FALSE.
	 */
	if ((indx = klocate_entry(hash, (ssize_t) indx, entry)) == -1)
	   return(FALSE);

	/*
	 *  Set the entry to be unoccupied...
	 */
	hash->count--;
	hash->occupied[indx] = FALSE;
	khash_resort(hash, indx);
	return(TRUE);
}

/************************************************************
*
*  Routine Name: khash_free - frees the hash array and all memory
*
*       Purpose: This function frees the hash array and all associated
*		 memory.
*
*	  Input: hash    - the hash array to be freed
*		 routine - A routine to free the clientdata pointer for each
*			   hash array member.  If routine is NULL, then it
*			   simply calls kfree() to free that data.
*       Returns: TRUE (1) on success, FALSE (0) otherwise.
*    Written By: Mark Young
*          Date: Sep 13, 1995
*
*   Declaration: int khash_free(
*		!  khash  *hash,
*		!  void  (*routine) PROTO((kaddr)) )
*************************************************************/
int khash_free(
   khash *hash,
   void (*routine) PROTO((kaddr)) )
{
	int i;
	size_t size;
	unsigned char *occupied;

	/*  Sanity check */
	if (hash == NULL)
	   return(FALSE);

	size = hash->size;
	occupied = hash->occupied;
	if (routine != NULL)
	{
	   kaddr *data = hash->data;
	   for (i = 0; i < size; i++)
	      if (occupied[i] && data[i] != NULL) (*routine)(data[i]);
	}

	if (hash->type == KSTRING)
	{
	   kstring *table = (char **) hash->table;
	   for (i = 0; i < size; i++)
	      if (occupied[i] && table[i] != NULL) kfree(table[i]);
	}
	kfree(hash->data);
	kfree(hash->table);
	kfree(hash->hashvals);
	kfree(hash->occupied);
	kfree(hash);
	return(TRUE);
}

/************************************************************
*
*  Routine Name: khash_reinit - reinitializes the hash to be empty
*
*       Purpose: This function reinitializes the hash to be empty.  This
*		 is useful if you wish to reuse a hash repeatedly.
*
*	  Input: hash    - the hash array to be reinitialized
*		 routine - A routine to free the clientdata pointer for each
*			   hash array member.  If routine is NULL, then no
*			   action is perform.
*       Returns: TRUE (1) on success, FALSE (0) otherwise.
*    Written By: Mark Young
*          Date: Jan 12, 1997
*
*   Declaration: int khash_reinit(
*		!  khash  *hash,
*		!  void  (*routine) PROTO((kaddr)) )
*************************************************************/
int khash_reinit(
   khash *hash,
   void (*routine) PROTO((kaddr)) )
{
	int i;
	size_t size;
	unsigned char *occupied;

	/*  Sanity check */
	if (hash == NULL)
	   return(FALSE);

	size = hash->size;
	occupied = hash->occupied;
	if (routine != NULL)
	{
	   kaddr *data = hash->data;
	   for (i = 0; i < size; i++)
	      if (occupied[i] && data[i] != NULL) (*routine)(data[i]);
	}

	if (hash->type == KSTRING)
	{
	   kstring *table = (char **) hash->table;
	   for (i = 0; i < size; i++)
	      if (occupied[i] && table[i] != NULL) kfree(table[i]);
	}
	kmemset(hash->occupied, 0, size);
	hash->count = 0;
	return(TRUE);
}
