/*
 * Khoros: $Id: ipc.c,v 1.22.2.1 1997/05/24 17:50:10 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: ipc.c,v 1.22.2.1 1997/05/24 17:50:10 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Inter Process Communication Support Utility
   >>>>		
   >>>>  Private:
   >>>>                 kipc_getipc()
   >>>>                 kipc_start()
   >>>>                 kipc_stop()
   >>>>                 kipc_enable()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


/*
 *  The following are private routines that the user should not call but
 *  should be private to the phantom library that enables a negotiation
 *  method between processes thru some user defined negotiator.
 *
 *	routines:	kipc_getipc
 *			kipc_start
 *			kipc_stop
 *			kipc_enable
 */
static char *transport = NULL;
kfile *server_ipc = NULL;
kfile *client_ipc = NULL;

#define KhorosIpcIdentifier "KHOROS_IPC"

/*-----------------------------------------------------------
|
|  Routine Name: kipc_getipc - gets the ipc transport
|
|       Purpose: This function gets the kfile structure associated
|		 with the ipc used in communicating with some
|		 other process.  This used by more complex routines
|		 such as cantata to communicate with sub-process
|		 so that special considerations can be made when
|		 a process needs to open, close, or a special
|		 notification.
|
|       Returns: transport on success, NULL otherwise
|    Written By: Mark Young
|          Date: Oct 14, 1992
|
------------------------------------------------------------*/
kfile *kipc_getipc(void)
{
	char   *path;
	static int busy = FALSE;

	/* if we are the server then we can't also be a client! */
	if (server_ipc != NULL)
	   return(NULL);

	/* if we've already initialized the client then return it */
	if (client_ipc != NULL)
	   return(client_ipc);

        if (busy == TRUE)
           return(NULL);
        else busy = TRUE;

	kinfo(KDEBUG, "Entering kipc_getipc");
	if ((path = kgetenv(KhorosIpcIdentifier)) != NULL)
	{
	   /*
	    *  Make sure that the transport is supported before
	    *  attempting to make a connection.
	    */
	   if ((client_ipc = kfopen(path, "w")) != NULL)
	   {
	      kfile_clrstate(client_ipc, KFILE_STREAM);
              ksetvbuf(client_ipc, NULL, _IOLBF, (size_t) 0);
	      kremenv(KhorosIpcIdentifier);
	   }
	}
	else
	   kinfo(KDEBUG, "kipc_getipc:  ipc not available...");

	busy = FALSE;
	return(client_ipc);
}

/*-----------------------------------------------------------
|
|  Routine Name: kipc_start - initialize the ipc for the supplied identifier
|
|       Purpose: This routine is used to initialize the ipc
|                so that we will be able to communicate sub-
|                processes.
|
|       Returns: return the result from kipc_setdescriptors() or -1 upon failure
|    Written By: Mark Young
|          Date: Oct 14, 1992
|
------------------------------------------------------------*/
kfile *kipc_start(void)
{
	int  fid[2];
	char temp[KLENGTH];

	/*
	 *  Check to see if the IPC has already been started.  If so then
	 *  return it.
	 */
	if (server_ipc != NULL)
	   return(server_ipc);

	kinfo(KDEBUG, "Entering kipc_start");

	/* we use a pipe to implement Khoros IPC's. */
	if (pipe(fid) == -1)
	   return(NULL);

	/* initialize the transport that will be used by the server (us). */
	ksprintf(temp, "pipe=[%d,%d]", fid[0], -1);
	if ((server_ipc = kfopen(temp, "R")) == NULL)
	   return(NULL);

	kfile_clrstate(server_ipc, KFILE_STREAM);
	ksetvbuf(server_ipc, NULL, _IOLBF, (size_t) 0);

	/* initialize the transport that will be used by the clients (them). */
	ksprintf(temp, "pipe=[%d,%d]", -1, fid[1]);
	transport = kstrdup(temp);
	return(server_ipc);
}

/*-----------------------------------------------------------
|
|  Routine Name: kipc_stop - stops the ipc for the supplied identifier
|
|       Purpose: This routine is used to stop or discontinue an ipc
|                so that processes will no longer communicate with
|                each other.  This is done by actually closing the
|		 ipc transports and removing it from the user's
|		 environment variables.
|
|       Returns: 0 on success, -1 upon failure
|    Written By: Mark Young
|          Date: Oct 14, 1992
|
------------------------------------------------------------*/
int kipc_stop(void)
{
	kinfo(KDEBUG, "Entering kipc_stop");

	if (server_ipc != NULL)
	   kfclose(server_ipc), server_ipc = NULL;
	if (client_ipc != NULL)
	   kfclose(client_ipc), client_ipc = NULL;

	kfree_and_NULL(transport);
	return(0);
}

/*-----------------------------------------------------------
|
|  Routine Name: kipc_enable - enables the client ipc
|
|       Purpose: This routine is used to enable or disable the client
|		 ipc so that processes exec'ed will communicate with
|                the server.  This is done by actually adding or removing
|		 the KhorosIpcIdentifier from the environment.
|
|       Returns: 0 on success, -1 upon failure
|    Written By: Mark Young
|          Date: Oct 14, 1992
|
------------------------------------------------------------*/
void kipc_enable(
    int enable)
{
	char env[KLENGTH];

	kinfo(KDEBUG, "Entering kipc_enable");

	if (enable)
	{
	   kipc_start();
	   ksprintf(env, "%s=%s", KhorosIpcIdentifier, transport), kputenv(env);
	}
	else
	   kremenv(KhorosIpcIdentifier);
}
