/*
 * Khoros: $Id: kfile.c,v 1.38.2.6 1997/06/09 23:20:28 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kfile.c,v 1.38.2.6 1997/06/09 23:20:28 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Transport Utilities
   >>>>		
   >>>>  Private:
   >>>>                 kfile_initialize()
   >>>>                 kfile_cleanup()
   >>>>                 kfile_checkid()
   >>>>                 kfile_checkfile()
   >>>>                 kfile_add()
   >>>>                 kfile_delete()
   >>>>                 kfile_free()
   >>>>                 kfile_read()
   >>>>                 kfile_write()
   >>>>                 kfile_close()
   >>>>                 kfile_open()
   >>>>                 kfile_access()
   >>>>                 kfile_rename()
   >>>>                 kfile_dup()
   >>>>                 kfile_seek()
   >>>>                 kfile_tell()
   >>>>                 kfile_closeall()
   >>>>                 kfile_listall()
   >>>>                 kfile_readstring()
   >>>>                 kfile_checkbuf()
   >>>>                 ktransport_routines()
   >>>>                 ktransport_identifier()
   >>>>                 ktransport_path()
   >>>>                 ktransport_command()
   >>>>                 ktransport_location()
   >>>>                 ktransport_username()
   >>>>                 ktransport_remotecmd()
   >>>>                 kcommand_to_args()
   >>>>   Static:
   >>>>   Public:
   >>>>                 kfile_reopen()
   >>>>                 kfile_getpermanence()
   >>>>                 kfile_getmachtype()
   >>>>                 kfile_setmachtype()
   >>>>                 kfile_remotehost()
   >>>>                 kfile_readdata()
   >>>>                 kfile_writedata()
   >>>>                 kfile_seddata()
   >>>>                 kfile_copydata()
   >>>>                 kfile_comparedata()
   >>>>                 kfile_filename()
   >>>>                 kfile_flags()
   >>>>                 kfile_setstate()
   >>>>                 kfile_clrstate()
   >>>>                 kfile_getstate()
   >>>>                 kfile_type()
   >>>>                 kfile_mode()
   >>>>                 ktransport_list()
   >>>>                 ktransport_add()
   >>>>                 ktransport_delete()
   >>>>                 kflags_to_type()
   >>>>                 ktype_to_flags()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "transport.h"	

static size_t    kfile_bufsiz;
static size_t    num_kfile_entries = 0;
static kfile     **kfile_entries = NULL;
static khash     *reacquire_list = NULL;

extern kfile *server_ipc;
extern kfile *client_ipc;
extern char *DefaultRemoteCmd;

/*
 *  If the user doesn't specify any convention what is the default transport.
 *  Most likely it should be FILE, but in some cases it may be different.
 */
static TransportInformation *DefaultTransport = file_transport;

/*
 *  Initialize the transports to be used...
 */
static size_t    num_transport_entries = 0;
static TransportInformation **transport_entries = NULL;

/*
 *  The following are semiprivate routines and should *usually* not be called
 *  by the application programmer.  These routines are available for the
 *  more sophisticated application.  These routines are used as utilities
 *  by the public routines found in "transport.c".  The reason why they are
 *  semi-public is that there maybe an occasion when the user needs a
 *  "transport" routine like kopen(), but slightly different.  In which case
 *  may find these routines helpful.
 *
 *	routines:
 *			kfile_initialize
 *			kfile_cleanup
 *			kfile_checkid
 *			kfile_checkfile
 *			kfile_add
 *			kfile_delete
 *			kfile_free
 *			kfile_read
 *			kfile_write
 *			kfile_seek
 *			kfile_tell
 *			kfile_close
 *			kfile_open
 *			kfile_access
 *			kfile_closeall
 *			kfile_listall
 *			kfile_readstring
 *			kfile_checkbuf
 */


/*-------------------------------------------------------------
|
|  Routine Name: kfile_initialize -
|
|       Purpose: This function is used to retrieve a kfile structure
|	         from the kfile structure list "kfile_entries".
|
|    Written By: Mark Young
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/
static int transport_initialized = FALSE;

void kfile_initialize(void)
{
	TransportInformation *tmp;

	int  i;
	char *bufsiz, *temp;

	/*
	 *  Initialize the kfile entry structure if it has not already been
	 *  done.
	 */
	if (!transport_initialized)
	{
	   transport_initialized = -1;
	   for (i = 0; i < NumTransports; i++)
	   {
	      if (transports[i] != NULL && transports[i]->identifier != NULL)
	      {
		 transport_entries = (TransportInformation **) karray_add(
			transport_entries, KSTRUCT, (kaddr) transports[i],
			num_transport_entries++);
	      }
	   }

	   /*
	    *  See if the default transport is good enough..
	    */
	   if (kgetenv("KHOROS_TESTHOST") == NULL)
              kputenv("KHOROS_TESTHOST=localtest");
	   if ((tmp = ktransport_routines(kgetenv("KHOROS_TRANSPORT"))) != NULL)
              DefaultTransport = tmp;
	   if ((temp = kgetenv("KHOROS_REMOTECMD")) != NULL)
              DefaultRemoteCmd = temp;

	   kfile_bufsiz = KLENGTH;
	   if ((bufsiz = kgetenv("KHOROS_TRANSPORT_BUFSIZE")) != NULL)
	   {
	      i = strtol(bufsiz, &temp, 0);
	      if (bufsiz != temp && i >= 0) kfile_bufsiz = i;
	   }
	   knegotiate_transport(FALSE);
	   (void) kfile_open("file=-", KOPEN_RDONLY|KOPEN_STREAM, (mode_t) 0);
	   (void) kfile_open("file=-", KOPEN_WRONLY|KOPEN_STREAM, (mode_t) 0);
	   (void) kfile_open("file=#", KOPEN_WRONLY|KOPEN_STREAM, (mode_t) 0);
	   knegotiate_transport(TRUE);
	   transport_initialized = TRUE;
	}
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_cleanup -
|
|       Purpose: This function is used to cleanup the transports on exit.
|
|    Written By: Mark Young & Steve Jorgensen
|          Date: Mar 9, 1997
|
-------------------------------------------------------------*/
void kfile_cleanup(void)
{
        khash_free(reacquire_list, NULL);
        karray_free(kfile_entries, KSTRUCT, num_kfile_entries, NULL);
        karray_free(transport_entries, KSTRUCT, num_transport_entries, NULL);
        transport_entries = NULL;
        kfile_entries = NULL;
        reacquire_list = NULL;
        num_transport_entries = 0;
        num_kfile_entries = 0;
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_checkid -
|
|       Purpose: This function is used to retrieve a kfile structure
|	         from the kfile structure list "kfile_entries".
|
|         Input: id - the identifier to be checked for
|    Written By: Mark Young
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/

kfile *kfile_checkid(
   int id)
{
	/*
	 *  Initialize the kfile entry structure if it has not already been
	 *  done.
	 */
	if (!transport_initialized)
	   kfile_initialize();

	if (id < 0 || id >= num_kfile_entries || !kfile_entries ||
	    !kfile_entries[id])
	{
	   errno = EBADF;
	   return(NULL);
	}
	return(kfile_entries[id]);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_checkfile
|
|       Purpose: This function is used to retrieve a kfile structure
|	         from the kfile structure list.
|
|         Input: file - the kfile structure to be checked.
|    Written By: Mark Young
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/

kfile *kfile_checkfile(
   kfile *file)
{
	if (!transport_initialized)
	   kfile_initialize();

	if (file)
	{
	   if (file == kstderr)
	      return(num_kfile_entries > 2 ? kfile_entries[2] : NULL);
	   else if (file == kstdout)
	      return(num_kfile_entries > 1 ? kfile_entries[1] : NULL);
	   else if (file == kstdin)
	      return(num_kfile_entries > 0 ? kfile_entries[0] : NULL);
	   else if (kfile_checkid(file->id) == file)
	      return(file);
	}
	errno = EBADF;
	return(NULL);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_add(entry)
|
|       Purpose: This function is used to retrieve a kfile structure
|	         from 
|	         with the kfile structure.
|
|         Input: file - the kfile structure to be freed.
|    Written By: Mark Young
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/

int kfile_add(
   kfile *file)
{
	int  id;

	/*
	 *  See if there are any free entries in the current table
	 *  before allocating any more space for new entries.
	 */
	if (!transport_initialized)
	   kfile_initialize();

	for (id = 0; id < num_kfile_entries; id++)
	{
	   if (kfile_entries[id] == NULL)
	   {
	      file->id = id;
	      kfile_entries[id] = file;
	      return(id);
	   }
	}
	file->id = num_kfile_entries;
	kfile_entries = (kfile **) karray_add(kfile_entries, KSTRUCT, file,
				num_kfile_entries++);
	return(file->id);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_delete
|
|       Purpose: This function is used to remove a kfile entry
|	         from the entry table.
|
|         Input: file - the kfile structure to be freed
|        Output: returns 0 or -1 if an error occurs
|    Written By: Mark Young
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/

int kfile_delete(
   kfile *file)
{
	if (file->id >= num_kfile_entries || kfile_entries == NULL)
	   return(-1);
	else
	   kfile_entries[file->id] = NULL;

	return(0);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_free
|
|       Purpose: This function is used to free the resources associated
|	         with the kfile structure.
|
|         Input: file - the kfile structure to be freed.
|       Returns: 0 on success, -1 otherwise
|    Written By: Mark Young
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/

int kfile_free(
   kfile *file)
{
	/*
	 *  If object is NULL or the object routines are NULL then
	 *  return.
	 */
	if (file == NULL)
	   return(-1);

	if (file->path != NULL)
	   kfree_and_NULL(file->path);

	if (file->resources != NULL)
	   kfree_and_NULL(file->resources);

	if (kfile_ismybuf(file))
	   kfree_and_NULL(file->buf);

	kfree_and_NULL(file->type);

	/*
	 *  Delete us from the list
	 */
	if (file->next != NULL)
	{
	   file->next->prev = file->prev;
	   if (!file->next->prev && !file->next->next)
	      kfile_clrstate(file->next, KFILE_DUP);
	}

	if (file->prev != NULL)
	{
	   file->prev->next = file->next;
	   if (!file->prev->next && !file->prev->prev)
	      kfile_clrstate(file->prev, KFILE_DUP);
	}
	kfile_delete(file);
	kfree_and_NULL(file);
	return(0);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_read()
|
|       Purpose: This function is used to read using the async
|		 read/write buffer.
|
|         Input:  file   - the kfile structure.
|                 data   - the pointer to store the data into.
|                 nbytes - the number of bytes to read.
|       Returns: returns the number of bytes read from the kfile
|    Written By: Mark Young
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/
ssize_t kfile_read(
   kfile *file,
   char  *ptr,
   size_t nbytes)
{
        ssize_t count;
	ssize_t numread = 0;
	TransportInformation *routines = file->routines;


	/*
	 *  Sanity check to make sure that a read buffer exists
	 */
	if (nbytes <= 0 || kfile_iseof(file))
	   return(0);

	/*
	 *  Read from our transport buffer before reading directly from the
	 *  transport.
	 */
	if ((numread = kmin(file->num_bytes - file->offset, nbytes)) > 0)
	{
	   kmemcpy(ptr, file->buf + file->offset, (size_t) numread);
	   file->offset += numread;
	   nbytes -= numread;
	}

	if (nbytes > 0)
	{
	   /*
	    *  Lock the file shared if we are performing locking read/write.
	    */
	   if (kfile_islock(file) && !kfile_iswrite(file))
	      kfile_lock(file, KLOCK_SH);

	   if (file->dirty == TRUE)
	      kfflush(file);

	   if (nbytes > file->bufsiz || kfile_isstream(file))
	   {
	      if ((count = routines->read(file, &ptr[numread], nbytes)) <= 0)
	      {
		 count = 0;
	         if ((file->flags & KOPEN_NONBLOCK) == FALSE)
		    kfile_setstate(file, KFILE_EOF);
	      }
	      else if (kfile_isstream(file) && file->buf)
	      {
                 /* Increase the transport buffer by what we want to read. */
                 if (file->maxbufsiz < file->bufsiz+count)
                 {
		    /* realloc to the size we need plus KLENGTH for growth */
                    file->maxbufsiz = file->bufsiz + count + KLENGTH;
                    file->buf = (char *) krealloc(file->buf, file->maxbufsiz);
                 }
	         kmemcpy(&file->buf[file->bufsiz], &ptr[numread],(size_t)count);
	         file->offset = file->num_bytes = file->bufsiz += count;
	      }
	      else
		 file->offset = file->num_bytes = 0;

	      numread += count;
	   }
	   else if ((count = routines->read(file, file->buf, file->bufsiz)) > 0)
	   {
	      file->num_bytes = count;
	      file->offset    = kmin(count, nbytes);
	      kmemcpy(&ptr[numread], file->buf, file->offset);
	      numread += file->offset;
	   }
	   else if ((file->flags & KOPEN_NONBLOCK) == FALSE)
	      kfile_setstate(file, KFILE_EOF);

	   /*
	    *  Unlock the file if we are performing locking read/write.
	    */
	   if (kfile_islock(file) && !kfile_iswrite(file))
	      kfile_lock(file, KLOCK_UN);
	}
	file->position += numread;
	return(numread);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_write()
|
|       Purpose: This function is used to write using the async
|		 read/write buffer.
|
|         Input:  file   - the kfile structure.
|                 ptr    - the pointer to store the data into.
|                 nbytes - the number of bytes to write.
|	Returns: returns the number of bytes written to the kfile
|    Written By: Mark Young
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/
ssize_t kfile_write(
   kfile *file,
   char  *ptr,
   size_t nbytes)
{
	ssize_t numwrite = 0;
	TransportInformation *routines = file->routines;


	/*
	 *  Sanity check to make sure that a write buffer exists
	 */
	if (nbytes <= 0)
	   return(0);

	if (kfile_isstream(file) && (file->bufsiz - file->offset) < nbytes)
	{
	   /* Increase the transport buffer by the amount we want to read. */
	   file->bufsiz = nbytes + file->offset;
           if (file->maxbufsiz < file->bufsiz)
           {
	      /* realloc to the size we need plus KLENGTH for growth */
              file->maxbufsiz = file->bufsiz + KLENGTH;
              file->buf = (char *) krealloc(file->buf, file->maxbufsiz);
           }
	}

	if (file->dirty == FALSE && routines->lseek != NULL)
	   routines->lseek(file, (off_t) (file->position - file->offset), 0);

	if ((file->bufsiz - file->offset) >= nbytes && file->buf)
	{
	   kmemcpy(file->buf + file->offset, ptr, nbytes);
	   if ((file->offset += nbytes) > file->num_bytes)
	      file->num_bytes = file->offset;

	   numwrite = nbytes;
	}
	else
	{
	   /*
	    *  Lock the file exclusive if we are performing locking read/write.
	    */
	   if (kfile_islock(file) && !kfile_isread(file))
	      kfile_lock(file, KLOCK_EX);

	   if (file->offset > 0)
	      (void) routines->write(file, file->buf, file->offset);

	   file->offset = file->num_bytes = 0;
	   if (nbytes <= file->bufsiz && file->buf)
	   {
	      kmemcpy(file->buf, ptr, nbytes);
	      file->offset = file->num_bytes = numwrite = nbytes;
	   }
	   else
	      numwrite = routines->write(file, ptr, nbytes);

	   /*
	    *  Unlock the file if we are performing locking read/write.
	    */
	   if (kfile_islock(file) && !kfile_isread(file))
	      kfile_lock(file, KLOCK_UN);
	}
	file->dirty = TRUE;
	file->position += numwrite;
	return(numwrite);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_close()
|
|       Purpose: This function is used to close a transport and
|		 delete it from the kfile entry structure.
|
|	Returns: returns the kfile structure or NULL upon failure
|    Written By: Mark Young
|          Date: Oct 13, 1992
|
-------------------------------------------------------------*/

int kfile_close(
   kfile *file)
{
	int	status;
	TransportInformation *routines;

	/*
	 *  Call the "close()" routine to close the transport
	 */
	if ((routines = file->routines) == NULL)
	   return(0);

	/*
	 *  Flush the buffer before closing...
	 */
	(void) kfflush(file);

	/*
	 *  After closing the transport unlink the file if it's a temporary
	 *  file.  Otherwise check to see if we should inform the negotiator
	 *  of the fact that the transport for this file has been closed.
	 */
	if (kfile_istemp(file))
	   kunlink(file->path);
	else if (transport_initialized == TRUE)
	   (void) knegotiate_close(file->path,  file->flags);

	/*
	 *  Need to close the file, but if reacquire is set, then go ahead
	 *  and add it to the reacquire list.  This list will be checked
	 *  and the reacquired.  This helps with visualization routines
	 *  that can't afford the overhead of re-opening the file ever
	 *  iteration.
	 */
	if (kfile_isreacquire(file))
	{
	   kinfo(KDEBUG, "kfile_close: Adding file to reacquire list '%s'",
			 file->path);
	   if (reacquire_list == NULL)
	      reacquire_list = khash_init(KSTRING, (size_t) 100);

	   kfile_seek(file, (off_t) 0, KSEEK_SET);
	   khash_add(reacquire_list, file->path, file);
	   status = 0;
	}
	else
	{
	   kinfo(KDEBUG, "kfile_close: Attempting to close file '%s'",
			 file->path);
	   status = routines->close(file);
	   kfile_free(file);
	   kinfo(KDEBUG,"kfile_close: after close status %d", status);
	}
	return(status);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_open - open the file
|
|       Purpose: This function is a replacement for the system "open"
|	         call.  The only difference is that kopen() supports
|	         more than just files, it supports other data transports
|	         as well.  The path should be in the form of an identifier,
|	         followed by an "=" equal indicator, and then the transport
|	         token. ie) a Shared memory path would look like:
|
|  			"shm=1032"
|
|	         If a file was desired then either:
|
|  			"/usr/tmp/vadd1235"
|  			"file=/usr/tmp/vadd1235"
|
|         Input: path - is the string containing the path name to the
|		        desired file to be open.  The path name identifies
|		        the file to be opened, just like the regular UNIX
|		        open() command.
|
|	        flags - how the file is to be opened.
|
|  			KOPEN_RDONLY - Open file for only reading
|  			KOPEN_WRONLY - Open file for only writing
|  			KOPEN_RDWR   - Open file for both reading & writing
|  			KOPEN_NONBLOCK - Whether to block when reading and
|  				         writing to the file.
|  			KOPEN_APPEND - Append to the file (writing).
|  			KOPEN_TRUNC  - Truncate the file for writing.
|  			KOPEN_CREAT  - Create the file if it does not alread
|				       exist
|  			KOPEN_EXCL   - Error if file exists
|
|	         mode - the permissions to be used when creating a new file.
|       Returns: the kfile id on success, -1 otherwise
|    Written By: Mark Young  
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/

kfile *kfile_open(
   const char *path,
   int         flags,
   mode_t      mode)
{
	kfile   *file;
	int	locking;
	TransportInformation *routines;
	char	*identifier, *filename, name[5*KLENGTH], temp[5*KLENGTH];


	/*
	 *  Initialize the kfile entry structure if it has not already been
	 *  done.
	 */
	if (!transport_initialized)
	   kfile_initialize();

	kinfo(KDEBUG, "kfile_open: Attempting to open file '%s'", path);

	/*
	 *  Expand the path to be it's fullpath name
	 */
	if (!kfullpath(path, NULL, name))
	   return(NULL);

	/*
	 *  Can we cheat and reacquire the file.  If so this will save
	 *  alot of time.
	 */
	if (reacquire_list != NULL &&
	    khash_clientdata(reacquire_list, name, &file) == TRUE &&
	    file && knegotiate_open(file->path, file->flags) == TRUE)
	{
	   kinfo(KDEBUG, "Reacquired '%s", file->path);
	   khash_delete(reacquire_list, file->path);
	   return(file);
	}

	/*
	 *  According to the transport routines
	 */
	identifier = ktransport_identifier(name, temp);
	if ((routines = ktransport_routines(identifier)) == NULL)
	   return(NULL);

	/*
	 *  Get the filename component...
	 */
	if ((filename = ktransport_path(name, temp)) == NULL)
	   return(NULL);

	/*
	 *  Malloc the file structure.
	 */
	if ((file = (kfile *) kcalloc((size_t)1,sizeof(struct _kobject))) ==
	     NULL)
	{
	   kinfo(KSYSLIB, "kfile_open:  Not enough memory.  Unable to kmalloc "
			  "(%d) bytes for the khoros file structure.",
			  sizeof(kfile));
	   return(NULL);
	}

	file->state  = 0;
	file->mode   = mode;
	file->flags  = flags;
	file->type   = kflags_to_type(flags, NULL);
	file->routines = routines;
	file->machtype = kmach_type(NULL);
	file->path     = kstrdup(name);

	/*
	 *  Set the read/write state of the transport
	 */
	if (flags & KOPEN_WRONLY)
	   kfile_setstate(file, KFILE_WRITE);
	else if (flags & KOPEN_RDWR)
	   kfile_setstate(file, KFILE_RDWR);
	else
	   kfile_setstate(file, KFILE_READ);

	if (flags & KOPEN_TEMP)
	   kfile_setstate(file, KFILE_TEMP);

	if ((flags & KOPEN_STREAM) == 0)
	   kfile_setstate(file, KFILE_STREAM);

	/*
	 *  Initialize the file buf to use a local malloc'ed copy
	 */
	file->bufsiz = file->maxbufsiz = kfile_bufsiz;
	file->buf = (char *) kmalloc(file->bufsiz);
	kfile_setstate(file, KFILE_MYBUF|KFILE_FULLBUF);

	/*
	 *  Before opening the transport check to see if we should
	 *  negotiate a new transport for this file, or if not to
	 *  actually negotiate to inform the "negotiator" of our
	 *  attentions to open this transport.
	 */
	if (transport_initialized == TRUE)
	{
	   if (flags & KOPEN_TEMP)
	   {
	      flags &= ~KOPEN_LOCK;
	   }
	   else if (knegotiate_open(file->path, flags) == TRUE)
	   {
	      flags &= ~KOPEN_LOCK;
	      kfile_setstate(file, KFILE_REACQUIRE);
	      kinfo(KDEBUG, "Flagging %s to be reacquired\n", path);
	   }
	}

	/*
	 *  add the file entry to the kfile table and then open the transport
	 */
	(void) kfile_add(file);
	if (routines->open(filename, flags, mode, file) == -1)
	{
	   kfile_free(file);
	   return(NULL);
	}
	kfile_setstate(file, KFILE_OPEN);

	/*
	 *  See if the transport has data permanence
	 */
	routines = file->routines;
	if (routines->data_permanence)
	{
	   kfile_setstate(file, KFILE_PERM);
	   kfile_clrstate(file, KFILE_STREAM);
	}

	if ((flags & KOPEN_LOCK) && routines->lock != NULL)
	{
	   if (kfile_iswrite(file))
	      locking = KLOCK_EX;
	   else
	      locking = KLOCK_SH;

	   if (flags & KOPEN_NONBLOCK)
	      locking |= KLOCK_NB;

	   kfile_setstate(file, KFILE_LOCK);
	   (void) routines->lock(file, locking);
	}
	return(file);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_access - determine accessibility of file
|
|  Purpose: This function is a replacement for the system "access"
|          call.  The only difference is that kaccess() checks
|          more than just files, it will also check other data
|          transports as well, such as shared memory, pipes, files,
|          etc.
|
|          The routine will check to see if the token specified by
|          the transport is either readable (R_OK), writeable (W_OK),
|          or executable (X_OK), or accessible (F_OK) any possible
|          combination by or'ing the flags together.
|
|  Input: path - is the string containing the path name to the
|                  desired file to be accessed.  The path name ident-
|                  ifies the file/token to be tested, just like the
|                  regular UNIX access() command.
|
|          mode - the mode in which kaccess() should test for the
|                  file.  The mode is comprised of an or'ing of the
|                  following flags:
|
|          !    R_OK - the file readable
|          !    W_OK - the file writeable
|          !    X_OK - the file executable
|          !    F_OK - the file and directory leading to the
|          !           file accessible.
|
|   Output: none
|
|  Returns: return -1 for failure otherwise 0 for success
|
|  Restrictions:
|    Written By: Mark Young
|          Date: Jul 08, 1992 16:24
|      Verified:
|  Side Effects:
| Modifications:
|
-------------------------------------------------------------*/
int kfile_access(
   const char *path,
   int         mode)
{
	TransportInformation *routines;

	int   check;
	char  *identifier, *filename;
	char  name[5*KLENGTH], temp[5*KLENGTH];


        /*
         *  Initialize the kfile entry structure if it has not already been
         *  done.
         */
        if (!transport_initialized)
           kfile_initialize();

	/*
	 *  kfullpath() is used to expand the path before opening it...
	 */
	if ((path = kfullpath(path, NULL, name)) == NULL)
	   return(-1);

	/*
	 *  According to the transport routines
	 */
	identifier = ktransport_identifier(path, temp);
	if ((routines = ktransport_routines(identifier)) == NULL)
	   return(-1);

	/*
	 *  Get the filename component
	 */
	if ((filename = ktransport_path(path, temp)) == NULL)
	   return(-1);

	if (routines->access != NULL)
	   check = routines->access(path, filename, mode);
	else if (routines->open == NULL)
	   check = access(filename, mode);
	else
	   check = access(filename, mode);

	if (check == -1) errno = ENOENT;
	return(check);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_rename()
|
|       Purpose: This function is used to re-name a transport using
|		 a different 
|
|         Input: file - the kfile structure to be re-named
|		 rename - the rename to be used
|	Returns: returns 0 or -1 if an error occurs
|    Written By: Mark Young
|          Date: Oct 13, 1992
|
-------------------------------------------------------------*/

int kfile_rename(
   kfile *file,
   const char  *name)
{
	TransportInformation *routines;
	char	*filename, *path, *identifier,
                temp1[5*KLENGTH], temp2[5*KLENGTH];


	/*
	 *  According to the identifier what are the transport routines
	 */
	identifier = ktransport_identifier(name, temp1);
	if ((routines = ktransport_routines(identifier)) == NULL)
	   return(-1);

	/*
	 *  Get the filename component of the new path or if that is empty
	 *  then get old path filename
	 */
	if ((filename = ktransport_path(name, temp2)) == NULL		&&
	    (filename = ktransport_path(file->path, temp2)) == NULL)
	   return(FALSE);

	/*
	 *  Build the new identifier...
	 */
	if (identifier != NULL)
	   path = kstring_3cat(identifier, "=", filename, NULL);
	else
	   path = kstrdup(filename);

	/*
	 *  close the file if it is currently opened
	 */
	if (kfile_isopen(file))
	{
	   (void) file->routines->close(file);
	   errno = 0;
	}

	/*
	 *  Free up the contents of the old file structure.  Namely the
	 *  resources and path
	 */
	kfree_and_NULL(file->path);
	kfree_and_NULL(file->resources);

	/*
	 *  open the file entry to the kfile table
	 */
	file->path     = path;
	file->routines = routines;
	if (routines->open(filename, file->flags, file->mode, file) == -1)
	{
#if 0
	   kfile_free(file);
#endif
	   return(-1);
	}
	kfile_setstate(file, KFILE_OPEN);
	return(0);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_dup()
|
|       Purpose: This function is used to dup a transport using
|		 the specified id2 is the new descriptor entry.
|
|         Input:  id1 - the old descriptor
|		  id2 - the new descriptor
|	Returns: returns new dup'ed descriptor on succes or -1 upon failure
|    Written By: Mark Young
|          Date: Oct 13, 1992
|
-------------------------------------------------------------*/
/* ARGSUSED */
int kfile_dup(
   int id1,
   int id2)
{
	int   flags;
	kfile *file1, *file2;


	if ((file1 = kfile_checkid(id1)) == NULL)
	   return(-1);

	flags = ~KOPEN_TRUNC & file1->flags;
	if ((file2 = kfile_open(file1->path, flags, file1->mode)) == NULL)
	   return(-1);

	/*
	 *  Go ahead and link file2 into the file1's list of dup'ed objects,
	 *  as well designate the fact that both transports have dup'ed
	 *  objects associated with it.
	 */
	kfile_setstate(file1, KFILE_DUP);
	kfile_setstate(file2, KFILE_DUP);

	/*
	 *  Copy the machine type, since it is suppose to be a dup of the
	 *  original.
	 */
	file2->machtype = file1->machtype;

	if (file1->next != NULL)
	   file1->next->prev = file2;

	file2->prev = file1;
	file2->next = file1->next;
	file1->next = file2;
	return(kfileno(file2));
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_seek
|
|       Purpose: This function is used to seek on the transport,
|		 which also affects the async read/write buffer.
|
|         Input:  file   - the kfile structure.
|  		  offset - the offset in which to seek
|  		  whence - the control of how the offset will be applied
|    Written By: Mark Young
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/

off_t kfile_seek(
   kfile *file,
   off_t  offset,
   int   whence)
{
	char buffer[KLENGTH*10];
	TransportInformation *routines;
        off_t begin, num, pos = -1, status = 0;


	/*
	 *  if whence is KSEEK_CUR then return the position
	 */
	if (whence == KSEEK_CUR && offset == 0L)
	   return(file->position);

	/*
	 *  Rewind to the initial dup'ed transport...
	 */
	while (file->prev != NULL)
	   file = file->prev;

	do
	{
	   /*
	    *  If we are seeking within our transport buffer, then all we
	    *  should have to do is update our "offset" and "position" fields
	    *  within the file structure.
	    */
	   if (whence == KSEEK_CUR || whence == KSEEK_SET)
	   {
	      /*
	       *  Calculate the absolute position of what we want our offset
	       *  to be.  We will use this to see if we are still within our
	       *  transport buffer.
	       */
	      if (whence == KSEEK_CUR)
	      {
		 offset += file->position;
		 whence = KSEEK_SET;
	      }
	      pos = offset;

	      /*
	       *  If the desired file position is less than our current then
	       *  make sure that EOF is not set.
	       */
	      if (kfile_iseof(file))
	         kfile_clrstate(file, KFILE_EOF);

	      /*
	       *  Calculate where the beginning of our transport buffer is,
	       *  relative our seek pointer.  We will then see if the desired
	       *  position "pos" is within current transport buffer.
	       */
	      begin = file->position - (int) file->offset;
	      if (pos >= begin && pos < begin+file->num_bytes)
	      {
		 /*
		  *  Reset our read/write offset pointer within the transport
		  *  buffer relative to our new position.  Also, update our
		  *  file position to the requested new position.
		  */
		 file->offset = pos - begin;
		 file->position = pos;

		 /*
		  *  Advance to our next dup'ed transport.
	          */
		 file = file->next;
		 continue;
	      }
	   }

	   /*
	    *  Call the "seek()" routine to seek to the desired position.
	    */
	   if (kfile_isstream(file))
	   {
	      if (whence == KSEEK_END)
	      {
	         if (!kfile_iswrite(file))
		 {
		    while (!kfile_iseof(file))
	               kfile_read(file, buffer, (size_t) KLENGTH*10);
		 }
		 else
		    file->position = file->offset = file->num_bytes;
	      }
	      else if (whence == KSEEK_SET)
	      {
	         if (kfile_iswrite(file))
		    file->position = file->offset = file->num_bytes;

		 num = offset - file->position;
		 while (num > 0 && !kfile_iseof(file))
		 {
	            if (kfile_iswrite(file))
	               num -= kfile_write(file, buffer,
					  (size_t) kmin(num, KLENGTH*10));
		    else
		       num -= kfile_read(file, buffer,
					 (size_t) kmin(num, KLENGTH*10));
		 }
	      }
	      pos = file->position;
	   }
	   else if ((routines = file->routines) != NULL && routines->lseek)
	   {
	      (void) kfflush(file);

	      /* call seek method to seek to the desired position */
	      if ((pos = routines->lseek(file, offset, whence)) != -1)
	      {
	         file->position = pos;
		 file->offset = file->num_bytes = 0;
	      }
	      else
		 status = -1;
	   }
	   else status = -1;

	   file = file->next;
	} while (file != NULL);

	return(status == -1 ? -1 : pos);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_lock
|
|       Purpose: This function is used to lock a transport
|
|         Input:  file   - the kfile structure.
|    Written By: Mark Young
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/
int kfile_lock(
   kfile *file,
   int   operation)
{
	int     status;
	TransportInformation *routines;

	/*
	 * Get the routines, but make sure that we have a valid "lock"
	 * routine.
	 */
	if ((routines = file->routines) == NULL || routines->lock == NULL)
	   return(0);

	/*
	 *  lock the transport
	 */
	kinfo(KDEBUG, "kfile_lock: Attempting to lock file '%s'", file->path);
	status = routines->lock(file, operation);
	return(status);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_tell
|
|       Purpose: This function is used to tell on the transport.
|
|         Input:  file   - the kfile structure.
|    Written By: Mark Young
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/
long int kfile_tell(
   kfile *file)
{
	return((long int) file->position);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_closeall()
|
|       Purpose: This function is used to close all entries and
|	         remove the entries from the table.  A status
|	         of 0 or -1 will be returned indicating
|	         whether the all entries were successfully closed.
|	         If any entry could not be closed then a status
|	         of -1 is returned, otherwise if *all* entries
|	         were successfully closed then 0 is returned.
|
|	Returns: returns the status of the closing of the open() files.
|    Written By: Mark Young
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/
int kfile_closeall(void)
{
	kfile	*file;
	int	i, status = TRUE;


        /*
         *  Hey!  We gotta start at 3 instead of 0, since if errors or
         *  IPCs are open we need to have stdin, stdout, and stderr
         *  closed last.
         */
	for (i = 3; i < num_kfile_entries; i++)
	{
	   /*
	    *  Retrieve the file entry from the table and then NULL the
	    *  entry out of the table.
	    */
	   file = kfile_entries[i];
	   if (file != NULL && file != server_ipc && file != client_ipc)
	   {
	      kfile_clrstate(file, KFILE_REACQUIRE);
	      status |= kfclose(file);
	   }
	}
	kipc_stop();	        /* will close server and client ipc. dammit! */
        kfclose(kstderr);       /* mortal combat rules!                      */
        kfclose(kstdout);       /* fry a goose!                              */
        kfclose(kstdin);        /* kill a chicken!      		     */

	/*
	 *  Clean it up...
	 */
	num_kfile_entries = 0;
	kfree_and_NULL(kfile_entries);
	return(status);
}

/*-------------------------------------------------------------
|
|  Routine Name: kfile_listall
|
|       Purpose: This function is used to print all entries to kstderr.
|	         It is mainly used as a debugging tool.
|
|    Written By: Mark Young
|          Date: Apr 01, 1994
|
-------------------------------------------------------------*/
void kfile_listall(void)
{
	int	i;
	char	*name;

	for (i = 0; i < num_kfile_entries; i++)
	{
	   /*
	    *  Retrieve the file entry from the table and then NULL the
	    *  entry out of the table.
	    */
	   if (kfile_entries[i] != NULL && (name = kfile_filename(i)) != NULL)
	      kfprintf(kstderr,"%d: %s\n", i, name);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: kfile_readstring - private read for reading a string
|
|       Purpose: The kfile_readstring routine is the grunt routine that
|		 reads a string from the transport.  The return string
|		 is malloc'ed.  The delimit character is passed in.
|		 This is a *fast* implementation, since it dives
|		 directly into the transport buffer.
|
|         Input: kfile  - the device in which we will read the data from
|		 delim  - the character to delimit the string on
|		 string - if not NULL then the return string is read
|			  directly into this buffer
|       Returns: the string read or NULL upon error
|
|    Written By: Mark Young  
|          Date: Aug 29, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void cleanup_handler(kexit_status status, kaddr client_data)
{
   kfree_and_NULL(*((kaddr *) client_data));
}

char *kfile_readstring(
   kfile *file,
   int   delim,
   char  *string,
   int   num)
{
	char *tmp, *end, *buf;
	ssize_t size, numread = 0;

	static char *temp = NULL;
	static ssize_t tempsize = 0;

	if (string != NULL)
	   buf = string;
	else
	   buf = temp;

	num--;
	while (num <= 0 || numread < num)
	{
	   /* may need to grow the temporary array */
	   if (string == NULL && tempsize == numread)
	   {
	      if (temp == NULL)
		 kexit_handler(cleanup_handler, (kaddr) &temp);

	      tempsize += KLENGTH;
	      temp = (char *)krealloc(temp, (size_t)tempsize);
	      buf = &temp[numread];
	   }

	   if ((size = file->num_bytes - file->offset) > 0)
	   {
	      if (num > 0 && (num - numread) < size)
		 size = (num - numread);
	      if (string == NULL)
		 size = kmin(size, tempsize - numread);

	      tmp = &file->buf[file->offset];
	      if ((end=(char *)kmemccpy(buf, tmp, delim, (size_t) size))!=NULL)
		 size = (end - buf);

	      buf += size; numread += size;
	      file->position += size;  file->offset += size;

	      if (end != NULL)
		 break;
	   }
	   else
	   {
	      if (kfile_read(file, buf, (size_t) 1) <= 0)
		 break;

	      numread++;
	      if ((int) *buf++ == delim)
		 break;
	   }
	}
	*buf = '\0';

	if (buf == string)
	   return(NULL);

	return(string == NULL ? kstrdup(temp) : string);
}

/*-----------------------------------------------------------
|
|  Routine Name: kfile_checkbuf - check to see if any data is buffered
|
|       Purpose: The kfile_checkbuf routine checks to see if there is
|		 any data buffered.
|
|         Input: kfile  - the device in which to check if data is buffered.
|       Returns: returns TRUE if data is buffered otherwise FALSE.
|
|    Written By: Mark Young  
|          Date: Jun 15, 1996
| Modifications:
|
------------------------------------------------------------*/
int kfile_checkbuf(
   kfile *file)
{
        /*
         *  Check the transport buffer and if num_bytes - offset is zero
         *  then the answer is no.
         */
        if ((file->num_bytes - file->offset) == 0)
	   return(FALSE);

	return(TRUE);
}

/*
 *  The following are private routines and should not be called by the
 *  application programmer.
 *
 *	routines:
 *			ktransport_routines
 *			ktransport_identifier
 *			ktransport_location
 *			ktransport_username
 *			ktransport_remotecmd
 *			ktransport_path
 *			ktransport_command
 */

/*-------------------------------------------------------------
|
|  Routine Name: ktransport_routines(identifier)
|
|       Purpose: This function is used to find the appropriate transport
|	         information
|
|         Input: identifier - the transport identifier string
|    Written By: Mark Young
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/

TransportInformation *ktransport_routines(
   const char *identifier)
{
	int	i;

        /*
         *  Initialize the kfile entry structure if it has not already been
         *  done.
         */
        if (!transport_initialized)
           kfile_initialize();

	if (identifier == NULL)
	   return(DefaultTransport);

	for (i = 0; i < num_transport_entries; i++)
	{
	   if (transport_entries[i] != NULL)
	   {
	      if (kstrcmp(identifier, transport_entries[i]->identifier) == 0)
		 return(transport_entries[i]);
	   }
	}
	kinfo(KSYSLIB, "ktransport_routines: Unsupported transport..  Unable "
		       "to find support for the following transport (%s).\n",
		       identifier);
	return(NULL);
}

/*-------------------------------------------------------------
|
|  Routine Name: ktransport_identifier
|
|       Purpose: This function is used to find the transport identifier
|
|         Input: name - the transport string
|        Output: identifier - the transport identifier to be returned
|    Written By: Mark Young
|          Date: June 15, 1995
|
-------------------------------------------------------------*/

char *ktransport_identifier(
   const char *name,
   char *identifier)
{
	char *tmp, temp1[10*KLENGTH], temp2[10*KLENGTH], leftover[10*KLENGTH];

	if (identifier == NULL)
	   identifier = temp1;

	tmp = kstring_cleanup((char *) name, temp2);
	if (ksscanf(tmp, "%*[^:]://%[^:/]", leftover) == 1)
	   kstrcpy(identifier, "remote");
	else if (ksscanf(tmp, "%[^ =]=%[^\n]", identifier, leftover) != 2)
	   identifier[0] = '\0';

	identifier = kstring_cleanup(identifier, identifier);
	return(identifier == temp1 ? kstrdup(identifier) : identifier);
}

/*-------------------------------------------------------------
|
|  Routine Name: ktransport_path(path, token)
|
|       Purpose: This function is used to find the path part of
|	         a transport to be opened. 
|
|         Input: name - the transport string
|        Output: token - the path or command
|    Written By: Mark Young
|          Date: June 15, 1995
|
-------------------------------------------------------------*/

char *ktransport_path(
   const char *path,
   char       *token)
{
	char *tmp, temp1[10*KLENGTH], temp2[10*KLENGTH];

	if (token == NULL)
	   token = temp1;

	tmp = kstring_cleanup((char *) path, temp2);
	if (ksscanf(tmp, "%*[^:]://%*[^/]/%[^\n]", token) != 1 &&
	    ksscanf(tmp, "%*[^ =]=%[^\n]", token) != 1)
	{
	   kstrcpy(token, tmp);
	}
	token = kstring_cleanup(token, token);
	return(token == temp1 ? kstrdup(token) : token);
}

/*-------------------------------------------------------------
|
|  Routine Name: ktransport_command(command, token)
|
|       Purpose: This function is used to find the command part of
|	         the command to be executed
|
|         Input: name - the transport string
|        Output: token - the path or command
|    Written By: Mark Young
|          Date: June 15, 1995
|
-------------------------------------------------------------*/

char *ktransport_command(
   const char *command,
   char *token)
{
	char *tmp, temp1[10*KLENGTH], temp2[10*KLENGTH];

	if (token == NULL)
	   token = temp1;

	tmp = kstring_cleanup((char *) command, temp2);
	if (ksscanf(tmp, "%*[^:]://%*[^/]/%[^\n]", token) != 1)
	   kstrcpy(token, tmp);

	token = kstring_cleanup(token, token);
	return(token == temp1 ? kstrdup(token) : token);
}

/*-------------------------------------------------------------
|
|  Routine Name: ktransport_location(name, location)
|
|       Purpose: This function is used to find the location or
|	         machine in which the user wants to re-direct to
|	         via some transport.
|
|         Input: name - the transport string
|        Output: location - the location to execute the command
|    Written By: Mark Young
|          Date: June 15, 1995
|
-------------------------------------------------------------*/

char *ktransport_location(
   const char *name,
   char       *location)
{
	char temp1[10*KLENGTH], temp2[10*KLENGTH];

	if (location == NULL)
	   location = temp1;

	name = kstring_cleanup((char *)name, temp2);
	if (ksscanf(name, "%*[^:]://%[^:/]", location) != 1)
	{
	   location[0] = '\0';
	   return(NULL);
	}
	return(location == temp1 ? kstrdup(location) : location);
}

/*-------------------------------------------------------------
|
|  Routine Name: ktransport_username(name, username)
|
|       Purpose: This function is used to find the username or
|	         machine in which the user wants to re-direct to
|	         via some transport.
|
|         Input: name - the transport string
|        Output: username - the username to execute the command
|    Written By: Mark Young
|          Date: June 15, 1995
|
-------------------------------------------------------------*/

char *ktransport_username(
   const char *name,
   char *username)
{
        struct passwd *passwd_ent;
        static char *localname = NULL, *lnameptr;
	char *tmp, temp1[10*KLENGTH], temp2[10*KLENGTH];

	if (username == NULL)
	   username = temp1;

	tmp = kstring_cleanup((char *) name, temp2);
	if (ksscanf(tmp, "%*[^:]://%*[^:/]:%[^/]", username) != 1)
	{
	   if (lnameptr == NULL)
           {
	      if ((lnameptr = kgetenv("USER")) != NULL)
		 lnameptr = kstrcpy(localname, lnameptr);
              else if ((passwd_ent = getpwuid(getuid())) != NULL)
                 lnameptr = kstrcpy(localname, passwd_ent->pw_name);
           }
	   kstrcpy(username, localname);
	}
	return(username == temp1 ? kstrdup(username) : username);
}

/*-------------------------------------------------------------
|
|  Routine Name: ktransport_remotecmd(name, remotecmd)
|
|       Purpose: This function is used to find the command to use to
|		 execute the remote command.
|
|         Input: name - the transport string
|        Output: remotecmd - the command to use to execute the remote command
|    Written By: Mark Young
|          Date: June 15, 1995
|
-------------------------------------------------------------*/

char *ktransport_remotecmd(
   const char *name,
   char *remotecmd)
{
	char *tmp, temp1[10*KLENGTH], temp2[10*KLENGTH];

	if (remotecmd == NULL)
	   remotecmd = temp1;

	tmp = kstring_cleanup((char *) name, temp2);
	if (ksscanf(tmp, "%*[^ =]=%[^:]://", remotecmd) != 1 &&
	    ksscanf(tmp, "%[^:]://", remotecmd) != 1)
	   kstrcpy(remotecmd, DefaultRemoteCmd);

	return(remotecmd == temp1 ? kstrdup(remotecmd) : remotecmd);
}

/*-------------------------------------------------------------
|
|  Routine Name: kcommand_to_args(command)
|
|       Purpose: This function is used to find the appropriate daemon
|	         information
|
|         Input: command - the command to parse
|       Returns: args - internal copy of args structure
|    Written By: Mark Young
|          Date: Jul 08, 1992
|
-------------------------------------------------------------*/

static char *temporary_args[MAX_ARGS];
static char temporary_command[10*KLENGTH];

char **kcommand_to_args(
   const char *command)
{
	int  type, num = 0;
	char echar, *cmd, **args = temporary_args;

	/*
	 *  Expand the command to be executed...
	 */
	if ((cmd = kstring_cleanup(command, temporary_command)) == NULL)
	   return(NULL);

	/*
	 *  Parse the cmd string into seperate arguments tokens...
	 */
	if ((type = kcheck_command(cmd)) != 0)
	{
	   if (type == 1)
	   {
	      args[num++] = "/bin/sh";
	      args[num++] = "-cf";
	      args[num++] = cmd;
	   }
	   else
	   {
	      args[num++] = "/bin/csh";
	      args[num++] = "-cf";
	      args[num++] = cmd;
	   }
	}
	else
	{
	   while ((int) *cmd != '\0' && num < MAX_ARGS)
	   {
	      while (isspace(*cmd) && (int) *cmd != '\0')
	         cmd++;

	      if (*cmd != '\0')
	      {
	         if ((int) *cmd == '\'' || (int) *cmd == '"')
	         {
		    echar = *cmd++;
		    args[num++] = (char *) cmd;
		    while ((echar != (int)*cmd) && ((int) *cmd != '\0'))
		    {
		       if ((int) *cmd == '\\' && (int) *(cmd+1) != '\0')
		          cmd += 2;
		       else
		          cmd++;
		    }

		    if (*cmd != '\0')
		       *cmd = ' ';
	         }
		 else
		    args[num++] = cmd;

	         while (!isspace(*cmd) && (int) *cmd != '\0')
	            cmd++;

	         if (*cmd != '\0')
	         {
	            *cmd = '\0';
	            cmd++;
	         }
	      }
	   }
	}
	args[num] = NULL;
	return(args);
}


/************************************************************
*
*  Routine Name: kfile_reopen - re-open a stream khoros transport
*       Purpose: This function is similar to the khoros kfreopen()
*		 call.  The only difference is that kfile_reopen()
*		 re-open the a khoros stream transport as well close
*		 and re-open all of transports that have been kdup'ed
*		 from the existing transport.
*
*		 kfile_reopen() will open a specified file on for an existing
*		 stream.  The existings stream is closed before the new
*		 filename is opened.  This function is typically used to
*		 open a specified file as one of the predefined streams;
*		 such as standard input, standard output, or standard error.
*
*         Input: path - is the string containing the path name to the
*			desired file to be open.  The path name identifies
*		 	the file to be opened, just like the regular UNIX
*		 	freopen() command.
*
*		 type - how the file is to be re-opened.
*	  	 !	"r"  - Open file for only reading
*	  	 !	"w"  - Open file for only writing
*  		 !	"a"  - append file for only writing
*  	 	 !	"A"  - append file for writing; non overwrite
*	  	 !	"r+" - Open file for both reading & writing
*	  	 !	"w+" - Open file for both reading & writing
*	  	 !	"a+" - append file for only writing
*	  	 !	"A+" - append file for writing; non overwrite
*
*		 file - the existing khoros stream transport to be reopened
*			for the specified filename.
*       Returns: returns the kfile structure or NULL upon failure
*    Written By: Mark Young
*          Date: Apr 28, 1993
*************************************************************/

kfile *kfile_reopen(
   char *path,
   char *type,
   kfile *file)
{
	char  *otype;
	int   oflags;
	kfile *temp = file;


	/*
	 *  Go ahead and rewind to the initial dup'ed object
	 */
	while (temp->prev != NULL)
	   temp = temp->prev;

	/*
	 *  Save the old flags and reopen the file with the new type
	 */
	do
	{
	   otype  = temp->type;
	   oflags = temp->flags;
	   temp->type  = type;
	   temp->flags = ktype_to_flags(type);

	   if (kfile_rename(temp, path) == -1)
	   {
	      temp->flags = oflags;
	      temp->type  = otype;
	      continue;
	   }
	   temp->type = kstrdup(temp->type);
	   kfree_and_NULL(otype);

	   temp = temp->next;
	} while (temp != NULL);

	return(file);
}


/**************************************************************
*  
*  Routine Name: kfile_getpermanence - returns whether a file has data
*				       permanence or not
*       Purpose: This function is used to return whether a particular
*	         file or transport identifier is a permanent data
*	         transport (like shared memory) or a connection oriented
*	         protocol like (stream or sockets).
*         Input: path  - path of identifier to inquire about
*	Returns: returns TRUE or FALSE depending whether the transport
*	         is known to support data permanence or -1 upon error
*    Written By: Mark Young
*          Date: Jul 08, 1992
**************************************************************/

int kfile_getpermanence(
   char *path)
{
	TransportInformation *routines;
	char	*identifier, *filename, temp[5*KLENGTH];

	/*
         *  Get the filename component.  Remember the fact that if the
	 *  filename component is either a "-" or "#" this is used to
	 *  designate stdin, stdout, kstderr which definitely does not
	 *  have data permanence.
	 */
        if ((filename = ktransport_path(path, temp)) != NULL)
	{
	   if (kstrcmp(filename, "-") == 0 || kstrcmp(filename, "#") == 0)
	      return(FALSE);
	}

	/*
	 *  According to the transport routines what is the 
	 *  transport data permanence.
	 */
	identifier = ktransport_identifier(path, temp);
	routines = ktransport_routines(identifier);

	if (routines == NULL)
	   return(-1);
	else
	   return(routines->data_permanence);
}


/**************************************************************
*  
*  Routine Name: kfile_getmachtype - gets the machine architecture type
*				     for a file transport id
*       Purpose: This function returns the machine architecture type
*	         for a particular file id.  The machine type is used
*	         in conjunction with the kread_XXXX & kwrite_XXXX routines
*	         which provide data conversion for data independence.
*         Input: id - the file id to be retrieve the machtype, which was opened
*		 earlier with kopen().
*       Returns: the machine type on success, -1 otherwise
*    Written By: Mark Young  
*          Date: Jul 08, 1992
**************************************************************/

int kfile_getmachtype(
   int id)
{
	kfile	*file;


	/*
	 *  Get the file entry associated with the id
	 */
	if ((file = kfile_checkid(id)) == NULL)
	   return(-1);

	/*
	 *  Return the machine type associated 
	 */
	return(file->machtype);
}


/**************************************************************
*  
*  Routine Name: kfile_setmachtype - sets the machine architecture type
*				     for a file transport id
*       Purpose: This function sets the machine architecture type
*	         for a particular file id.  The machine type is used
*	         in conjunction with the kread_XXXX & kwrite_XXXX routines
*	         which provide data conversion for data independence.
*         Input: id - the file id on which to set the machtype, which was opened
*		      earlier with kopen().
*		 machtype - the machine architecture type to set the file id to.
*       Returns: 0 on success, -1 otherwise
*    Written By: Mark Young  
*          Date: Jul 08, 1992
**************************************************************/

int kfile_setmachtype(
   int id,
   int machtype)
{
	kfile	*file;


	/*
	 *  Get the file entry associated with the id
	 */
	if ((file = kfile_checkid(id)) == NULL)
	   return(-1);

	/*
	 *  Set the machine architecture type
	 */
	file->machtype = machtype;
	return(0);
}


/**************************************************************
*  
*  Routine Name: kfile_remotehost - returns whether a host is remote
*       Purpose: This function determines whether or not the machine name
*		 passed in is the hostname or an alias of the current
*		 machine, or if it is the name of another machine
*		 on the network.
*         Input: machine - the machine name we wish to find out about
*       Returns: TRUE (1) if the machine name is a remote name,
*		 FALSE (0) otherwise
*    Written By: Mark Young
*          Date: Jul 08, 1992
**************************************************************/

int kfile_remotehost(
   char *machine)
{
	char *temp, value[5*KLENGTH], temp2[5*KLENGTH];


	if (machine == NULL)
	   return(FALSE);

	kstrcpy(value, machine);
	if ((temp = (char *) kstring_cleanup(value, temp2)) == NULL)
	   return(FALSE);
	if ((temp = (char *) kstring_lower(temp, value)) == NULL)
	   return(FALSE);

	if (kstrcmp(temp, "localhost") == 0)
	{
	   return(FALSE);
	}
	else if (kgethostname(temp2, KLENGTH) == TRUE)
	{
	   if (kstrcmp(temp, temp2) == 0)
	      return(TRUE);
	   else
	      return(FALSE);
	}
	return(TRUE);
}

/************************************************************
*
*  Routine Name: kfile_readdata - read the contents of a khoros transport
*				  into a data array
*       Purpose: This routine reads the contents of a file, using the khoros
*		 transport mechanisms, into an array.  If the data array is
*		 NULL then a data array of suitable size is allocated and
*		 passed back.  An optional "num_read" can be used to find
*		 out the actual number of bytes read.  If "num_read" is
*		 NULL then the parameter is ignored.
*         Input: id  - the khoros transport descriptor
*		 ptr - data to be used if not NULL
*        Output: num - if not NULL the number of bytes read is returned
*       Returns: a pointer to the contents of the file or NULL upon failure
*    Written By: Mark Young  
*          Date: Jan 16, 1993
*  Side Effects: the data returned is allocated and needs to freed using
*		 kfree_and_NULL().
*************************************************************/

kaddr kfile_readdata(
   int     id,
   kaddr   ptr,
   size_t *num)
{
	ssize_t num_bytes;
	ssize_t size = 0, num_read = 0;
	char   *data = (char *) ptr;

	do
	{
	    if (((num_read + ALLOCSIZE) > size) && !ptr)
	    {
	       size += ALLOCSIZE;
	       data = (char *) krealloc((kaddr) data, (size_t)(size+1));
	    }

	    if ((num_bytes = kread(id,&data[num_read],(size_t) ALLOCSIZE)) > 0)
	       num_read += num_bytes;

	} while (num_bytes > 0);

	if (!ptr)
	   data = (char *) krealloc((kaddr) data, (size_t) (num_read+1));
	if (num)
	   *num = num_read;

	/*
	 *  Terminate the data in case it is really an ascii string
	 */
	data[num_read] = '\0';
	return(data);
}


/************************************************************
*
*  Routine Name: kfile_writedata - write the contents of the data to a
*				   khoros transport
*       Purpose: This routine writes the contents of a data array to the
*		 specified khoros transport mechanism.  An optional "num_write"
*		 can be used to specify the actual number of bytes to be
*		 written.  If "num_write" is 0 then the number of bytes
*		 to be written is computed from the data by using kstrlen().
*         Input: id   - the khoros transport descriptor
*		 data - the data array (or string) to be written
*		 num  - if not 0 the number of bytes to be written.
*       Returns: The number of bytes written or -1 upon failure
*    Written By: Mark Young  
*          Date: Jan 16, 1993
*************************************************************/

ssize_t kfile_writedata(
   int    id,
   kaddr  data,
   size_t num)
{
	if (num == 0)
	   num = kstrlen((const char *)data);

	return(kwrite(id, data, num));
}

/************************************************************
*
*  Routine Name: kfile_seddata - string edit from one transport to another
*       Purpose: This routine takes two transport id's, and copies the data
*		 from the source file to the destination file.  It also
*		 uses a variable argument list of strings pairs, where
*		 each pair indication a search pattern and a replacement
*		 pattern.  This list will be used to replace strings in
*		 pattern strings in the input transport with the corresponding
*		 replacement pattern.  Note, the variable argument list MUST
*		 be terminated with a NULL in one of the search pattern slots
*
*                For example, the following is a correct termination:
*
*               ! kfile_seddata(id1, id2, "t1", "t2", FALSE, NULL);
*
*               And this is an incorrect termination:
*
*               ! kfile_seddata(id1, id2, "t1", NULL);
*
*               And this is an incorrect termination:
*
*               ! kfile_seddata(id1, id2, "t1", NULL, FALSE);
*
*               The reason the NULL must be on the search pattern, is
*               so that NULL can be sent in as the replacement string.
*               The replacement mechanism is kstring_replace, and
*               its documentation explains restrictions on search strings
*               and replace strings.
*
*		The regex parameter is used to indicate whether the search
*		and replacement parameters are specified in regular expression
*		form, or are just simply string replacement patterns.  For
*		more information about regular expressions please see the
*		khoros string parsing section of the Khoros Programmers Manual.
*
*		kfile_seddata does not support the "mode" operation as used
*		by ksedfile().  The mode operation is used to how the data
*		should be updated, but it relies on the ability to read
*		as well as write to the destination id (id2).
*
*         Input: id1 - source transport id to copy data from
*                id2 - destination transport id to copy data to
*		 regex   - whether the search and replacement patterns are
*			   regular expressions or not
*                kvalist - list of search patterns and replacement
*                          patterns to be used to modify the data as
*                          it is being copied.
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*    Written By: Mark Young
*          Date: Feb 20, 1994
*   Declaration: int kfile_seddata(
*		 !   int id1,
*		 !   int id2,
*		 !   kvalist)
*************************************************************/
/* ARGSUSED */
int kfile_seddata(
   int id1,
   int id2,
   kvalist)
{
	int  regex;
	kva_list valist;
	char *data, *temp, *istr, *ostr;


	kva_start(valist, id2);
	if ((data = (char *) kfile_readdata(id1, NULL, NULL)) == NULL)
	   return(FALSE);

	while ((istr = kva_arg(valist, char *)) != NULL)
	{
	   ostr = kva_arg(valist, char *);
	   regex = kva_arg(valist, int);
	   if (regex == FALSE)
	      temp = kstring_replace(data, istr, ostr, NULL);
	   else
	      temp = kregex_replace(data, istr, ostr, NULL);

	   kfree_and_NULL(data); data = temp;
	}

	if (kfile_writedata(id2, data, kstrlen(data)) == -1)
	{
	   kfree_and_NULL(data);
	   return(FALSE);
	}
	kfree_and_NULL(data);
	return(TRUE);
}

/************************************************************
*
*  Routine Name: kfile_copydata - copy the contents of a transport id to
*				  another transport id
*       Purpose: This routine copies the contents of a input transport
*		 descriptor to an output transport descriptor.  These
*		 descriptors are specified by the Khoros transport
*		 mechanism.  An optional "num_copy" can be used to
*		 specify the actual number of bytes to be copied.  If
*		 "num_copy" is 0 then all the data from input is written
*		 to the output.
*         Input: id1 - the input khoros transport descriptor
*		 id2 - the output khoros transport descriptor
*		 num - if not 0 the number of bytes to be copied.
*       Returns: the number of bytes copied or -1 upon failure
*    Written By: Mark Young  
*          Date: Jan 20, 1993
*  Side Effects: the data written to the output descriptor will not be flushed
*		 until a kfflush() or kclose() is performed.
*************************************************************/

ssize_t kfile_copydata(
   int    id1,
   int    id2,
   size_t num)
{
	char    data[ALLOCSIZE];
	ssize_t num_copy = 0;
	ssize_t num_bytes;


	do
	{
	    if (num != 0)
	       num_bytes = kmin(num, ALLOCSIZE);
	    else
	       num_bytes = ALLOCSIZE;

	    /*
	     *  Read from id1 at most num_bytes
	     */
	    if ((num_bytes = kread(id1, data, (size_t) num_bytes)) == -1)
	       return(-1);

	    /*
	     *  Write to id2 num_bytes...
	     */
	    if (kwrite(id2, data, (size_t)num_bytes) != (ssize_t) num_bytes)
	       return(-1);

	    if (num_bytes > 0)
	    {
	       num_copy += num_bytes;
	       if (num != 0) num -= num_bytes;
	    }
	} while (num_bytes > 0);

	return(num_copy);
}


/************************************************************
*
*  Routine Name: kfile_comparedata - compare the contents of a transport id to
*				     another transport id
*       Purpose: This routine compare the contents of a input transport
*		 descriptor to an output transport descriptor.  These
*		 descriptors are specified by the Khoros transport
*		 mechanism.  An optional "num_compare" can be used to
*		 specify the actual number of bytes to be compared.  If
*		 "num_compare" is 0 then all the data from the two transports
*		 are compared.
*         Input: id1 - the first khoros transport descriptor
*		 id2 - the second khoros transport descriptor
*		 num - if not 0 the number of bytes to be compared.
*        Output: num_compared - the number of bytes actually compared.
*       Returns: returns an integer less than, equal to, or greater than 0,
*		 according as id1 is lexicographically less than, equal to,
*		 or greater than id2.
*    Written By: Mark Young  
*          Date: Jan 20, 1993
*  Side Effects: data is read from id1 and id2, which means if the transports
*		 do not support data permanence seeking maynot work.
*************************************************************/

int kfile_comparedata(
   int     id1,
   int     id2,
   size_t  num,
   size_t *num_compared)
{
	int     cmp;
	size_t  num_bytes;
	ssize_t i, num1, num2, num_processed = 0;
	char    data1[ALLOCSIZE], data2[ALLOCSIZE];


	do
	{
	    if (num != 0)
	       num_bytes = kmin(num - num_processed, ALLOCSIZE);
	    else
	       num_bytes = ALLOCSIZE;

	    /*
	     *  Read from id1 at most num_bytes
	     */
	    if ((num1 = kread(id1, data1, num_bytes)) == -1)
	    {
	       if (num_compared) *num_compared = num_processed;
	       return(-1);
	    }

	    /*
	     *  Write to id2 num_bytes...
	     */
	    if ((num2 = kread(id2, data2, num_bytes)) == -1)
	    {
	       if (num_compared) *num_compared = num_processed;
	       return(-1);
	    }

	    num_bytes = kmin(num1, num2);
	    if ((cmp = kmemcmp(data1, data2, num_bytes)) != 0)
	    {
	       if (num_compared == NULL)
		  return(cmp);

	       for (i = 0; i < num_bytes; i++, num_processed++)
	       {
	          if (data1[i] != data2[i])
	          {
		     if (num_compared) *num_compared = num_processed;
		     return((int) (data1[i] - data2[i]));
	          }
	       }
	       /* Internal Error! */
	    }
	    else
	       num_processed += num_bytes;

	    if (num1 != num2)
	    {
	       if (num_compared) *num_compared = num_processed;
	       return(-1);
	    }
	} while (num_bytes > 0);

	if (num_compared) *num_compared = num_processed;
	return(0);
}


/************************************************************
*
*  Routine Name: kfile_filename - return the filename associated with
*				  khoros transport id
*       Purpose: This routine returns the filename associated with
*		 a khoros transport descriptor id.  This is the filename
*		 used in originally creating the transport.
*         Input: id - the transport id from which the filename is returned
*       Returns: The filename associated with the id or NULL upon failure
*    Written By: Mark Young  
*          Date: Jan 20, 1993
*  Side Effects: the filename returned is the internal copy of the
*		 kfile transport.  This means that you should not
*		 modify or free the string.
*************************************************************/

char *kfile_filename(
   int id)
{
	kfile *file;

	if ((file = kfile_checkid(id)) == NULL)
	   return(NULL);

	return(file->path);
}


/************************************************************
*
*  Routine Name: kfile_flags - return the flags associated with transport id
*       Purpose: This routine returns the open flags associated with
*		 a khoros transport descriptor id.  This is the flags
*		 used in originally creating/opening of the transport.
*         Input: id - the transport id from which the flags is returned
*       Returns: The flags associated with the id or -1 upon failure
*    Written By: Mark Young  
*          Date: Apr 27, 1993
*************************************************************/

unsigned long kfile_flags(
   int id)
{
	kfile *file;

	if ((file = kfile_checkid(id)) == NULL)
	   return(0L);

	return(file->flags);
}

/************************************************************
*
*  Routine Name: kfile_setstate - adds a flag to the open transport id state
*       Purpose: This routine is used to set a flag within the transport id
*		 state field.
*         Input: file - the transport id from which to set the flag for
*		 flag - the flag to be set (such as KFILE_TEMP)
*       Returns: returns the updated flags associated with the id or -1 upon
*		 failure
*    Written By: Mark Young  
*          Date: Jul 21, 1994
*************************************************************/

unsigned long kfile_setstate(
   kfile *file,
   unsigned long flag)
{
/*
	if ((file = kfile_checkfile(file)) == NULL)
	   return(0L);
 */
	file->state |= flag;
	return(file->state);
}

/************************************************************
*
*  Routine Name: kfile_clrstate - remove a flag from an open transport id state
*       Purpose: This routine is used to clear a flag within the transport id
*		 state field.
*         Input: file - the transport id from which to clear the flag for
*		 flag - the flag to be clear (such as KFILE_TEMP)
*       Returns: The updated flags associated with the id or -1 upon failure
*    Written By: Mark Young  
*          Date: Jul 21, 1994
*************************************************************/

unsigned long kfile_clrstate(
   kfile *file,
   unsigned long flag)
{
/*
	if ((file = kfile_checkfile(file)) == NULL)
	   return(0L);
 */
	file->state &= ~flag;
	return(file->state);
}

/************************************************************
*
*  Routine Name: kfile_getstate - return the current internal stream transport
*			       state
*       Purpose: This routine returns the internal state of a khoros stream
*		 transport.  The state is mask of the following defines:
*
*		 ! 	KFILE_READ    -  whether transport is readable
*		 ! 	KFILE_WRITE   -  whether transport is writeable
*		 ! 	KFILE_RDWR    -  whether transport is readable &
*					 writeable
*		 ! 	KFILE_MYBUF   -  whether transport uses user defined
*					 stream buffer
*		 ! 	KFILE_EOF     -  whether transport is at end of file
*		 ! 	KFILE_ERR     -  whether transport is in an error
*		 ! 	KFILE_PERM    -  whether transport has data permanence
*		 ! 	KFILE_OPEN    -  whether transport is currently opened
*		 ! 	KFILE_DUP     -  whether transport is currently dupped
*		 ! 	KFILE_TEMP    -  whether transport is a temporary
*		 ! 	KFILE_STREAM  -  whether transport is a temporary
*		 ! 	KFILE_LOCK    -  whether transport enforces locking
*         Input: file - the kfile transport to be returned
*       Returns: The type associated with the kfile transport on success,
*		 or NULL upon failure
*    Written By: Mark Young  
*          Date: Apr 27, 1993
*  Side Effects: the type field returned is the internal copy of the
*		 kfile transport.  This means that you should not
*		 modify or free the string.
*************************************************************/

unsigned long int kfile_getstate(
   kfile *file)
{
	if ((file = kfile_checkfile(file)) == NULL)
	   return(FALSE);

	return(file->state);
}

/************************************************************
*
*  Routine Name: kfile_type - return the type flag field used when opening
*			      the transport with kfopen()
*
*       Purpose: This routine returns the type field associated with
*		 a khoros transport.  This is the type fields used
*		 originally when opening the transport with kfopen().
*
*         Input: file - the kfile transport returned earlier by kfopen()
*       Returns: The type associated with the kfile transport on success,
*		 or NULL upon failure
*    Written By: Mark Young  
*          Date: Apr 27, 1993
*  Side Effects: the type field returned is the internal copy of the
*		 kfile transport.  This means that you should not
*		 modify or free the string.
*************************************************************/

char *kfile_type(
   kfile *file)
{
	if ((file = kfile_checkfile(file)) == NULL)
	   return(NULL);

	return(file->type);
}


/************************************************************
*
*  Routine Name: kfile_mode - return the mode associated with transport id
*
*       Purpose: This routine returns the mode associated with
*                a khoros transport descriptor id.  This is the mode
*                used in originally creating/opening of the transport.
*
*         Input: id - the transport id from which the flags is returned
*       Returns: The mode associated with the id or KMODE_INVALID upon failure
*    Written By: Mark Young
*          Date: Nov 15, 1993
*************************************************************/

mode_t kfile_mode(
   int id)
{
	kfile *file;

	if ((file = kfile_checkid(id)) == NULL)
	   return(KMODE_INVALID);

	return(file->mode);
}

/**************************************************************
*  
*  Routine Name: ktransport_list - get the list of supported transports
*       Purpose: This function creates an array of available transports.
*		 Boolean inputs allow the calling routine to selectively mask
*		 in or out local, stream, and permanent transports.
*         Input: local      - a boolean indicating that local transports
*		  	      should be listed
*		 stream     - a boolean indicating that stream transports
*			      should be included in the list
*                permanence - a boolean indicating that data permanent
*		  	      transports should be included in the list
*        Output: num        - the number of entries returned
*	Returns: A pointer to an array of the available transports, or NULL
*		 on error.
*    Written By: Mark Young
*          Date: Jul 08, 1992
*   Declaration: char **ktransport_list(
*		 !    int     local,
*		 !    int     permanence,
*		 !    int     stream,
*		 !    size_t *num)
**************************************************************/
/* ARGSUSED */
char **ktransport_list(
   int     local,
   int     permanence,
   int     stream,
   size_t *num)
{
	size_t	i, j;
	char	**list = NULL, temp[5*KLENGTH];


	for (i = 0, j = 0; i < num_transport_entries; i++)
	{
	   if ((transport_entries[i]->data_permanence && permanence == TRUE) ||
	       (!transport_entries[i]->data_permanence && stream == TRUE))
	    {
	       if (transport_entries[i]->identifier == NULL ||
	           kstrcmp(transport_entries[i]->identifier, "remote") == 0)
		  continue;
	       else if (transport_entries[i]->label == NULL)
		  ksprintf(temp, "%4s", transport_entries[i]->identifier);
	       else
		  ksprintf(temp, "%4s (%s)", transport_entries[i]->identifier,
			transport_entries[i]->label);

	       list = (char **) karray_add(list, KSTRING, kstrdup(temp), j++);
	    }
	}
	*num = j;
	return(list);
}

/**************************************************************
*  
*  Routine Name: ktransport_add - add a new transport to the list of transports
*       Purpose: This function is used to add a new transport to the compiled
*		 list of transports.  The transport is dynamically added to
*		 during runtime, rather than during compile time.
*         Input: transport - the transport to be added
*	Returns: TRUE if the transport was successfully added, FALSE otherwise
*    Written By: Mark Young
*          Date: Jan 14, 1994
**************************************************************/

int ktransport_add(
   TransportInformation *transport)
{
	if (transport && karray_locate(transport_entries, KSTRUCT,
		(kaddr) transport, num_transport_entries) == -1)
	{
	   transport_entries = (TransportInformation **) karray_add(
			transport_entries, KSTRUCT, (kaddr) transport,
			num_transport_entries++);
	}
	else
	   return(FALSE);

	return(TRUE);
}

/**************************************************************
*  
*  Routine Name: ktransport_delete - delete a transport from the list of
*				     all transports
*       Purpose: This function is used to delete a new transport from the
*		 compiled list of transports.  The transport is dynamically
*		 deleted from the list, rather than during compile time.
*         Input: transport - the transport to be deleted
*	Returns: TRUE if the transport was successfully deleted, FALSE otherwise
*    Written By: Mark Young
*          Date: Jan 14, 1994
**************************************************************/

int ktransport_delete(
   TransportInformation *transport)
{
	if (transport && karray_locate(transport_entries, KSTRUCT,
		(kaddr) transport, num_transport_entries) != -1)
	{
	   transport_entries = (TransportInformation **) karray_delete(
			transport_entries, KSTRUCT, (kaddr) transport,
			num_transport_entries--);
	}
	else
	   return(FALSE);

	return(TRUE);
}

/************************************************************
*
*  Routine Name: kflags_to_type - convert kopen() flags to kfopen() type
*				    parameter
*       Purpose: The kflags_to_type() routine is used to convert the
*		 flags used in the kopen() call into their corresponding
*		 equivalent in the type field of the kfopen() call.
*		 The "flags" parameter is converted and stored into the
*		 "type" parameter.  If the "type" parameter is NULL then
*		 space is malloc'ed and returned.  The following is an
*		 example of how to use kflags_to_type():
*
*		 !   type = kflags_to_type(KOPEN_WRONLY | KOPEN_CREAT | KOPEN_TRUNC, NULL);
*
*		 the input flags are converted and result returned to
*		 type will be "w".
*
*                Here is the table as it translates:
*
*                ! a+  = KOPEN_RDWR   | KOPEN_APPEND | KOPEN_CREAT
*                ! w+  = KOPEN_RDWR   | KOPEN_CREAT  | KOPEN_TRUNC
*                ! r+  = KOPEN_RDWR
*                ! a   = KOPEN_WRONLY | KOPEN_APPEND | KOPEN_CREAT
*                ! w   = KOPEN_WRONLY | KOPEN_CREAT  | KOPEN_TRUNC
*                ! r   = KOPEN_RDONLY
*         Input: flags - kopen flags
*        Output: type  - a string to old the kfopen type field.  If NULL then
*			 space is malloc'ed
*       Returns: type is returned if it is not NULL.  Otherwise the malloc'ed
*		 string is returned.  NULL is returned on an error.
*    Written By: Mark Young
*          Date: Apr 27, 1993
*************************************************************/

char *kflags_to_type(
   int  flags,
   char *type)
{
	char	temp[5*KLENGTH];


	/*
	 *
	 *   Here it is.  This is what is all about, well with a few liberties
	 *   thrown in...
	 *   ----------------------------------------------------------------
	 *   a+  = KOPEN_RDWR KOPEN_APPEND KOPEN_CREAT
	 *   w+  = KOPEN_RDWR KOPEN_CREAT KOPEN_TRUNC 
	 *   r+  = KOPEN_RDWR 
	 *   a   = KOPEN_WRONLY KOPEN_APPEND KOPEN_CREAT
	 *   w   = KOPEN_WRONLY KOPEN_CREAT KOPEN_TRUNC
	 *   r   = KOPEN_RDONLY
	 * 
	 */
	if (flags & KOPEN_RDWR)
	{
	   if (flags & KOPEN_APPEND)
	      kstrcpy(temp, "a+");
	   else if (flags & KOPEN_TRUNC || flags & KOPEN_CREAT)
	      kstrcpy(temp, "w+");
	   else
	      kstrcpy(temp, "r+");
	}
	else if (flags & KOPEN_APPEND)
	   kstrcpy(temp, "a");
	else if (flags & KOPEN_WRONLY)
	   kstrcpy(temp, "w");
	else
	   kstrcpy(temp, "r");


	if (type == NULL)
	   type = kstrdup(temp);
	else
	   kstrcpy(type, temp);

	return(type);
}


/************************************************************
*
*  Routine Name: ktype_to_flags - convert kfopen() type to kopen() flags
*
*       Purpose: The ktype_to_flags() routine is used to convert the
*		 type field used in the kfopen() call into their corresponding
*		 equivalent in the flags field of the kopen() call.
*		 The "type" parameter is converted and returned in
*		 kopen flag form.  The following is an example of how
*		 to use ktype_to_flags():
*
*			flags = ktype_to_flags("w");
*
*		 the input flags are converted and result returned to
*		 type will be KOPEN_WRONLY|KOPEN_CREAT|KOPEN_TRUNC.
*
*                Here is the table as it translates:
*
*                ! a+  = KOPEN_RDWR   | KOPEN_APPEND | KOPEN_CREAT
*                ! w+  = KOPEN_RDWR   | KOPEN_CREAT  | KOPEN_TRUNC
*                ! r+  = KOPEN_RDWR
*                ! a   = KOPEN_WRONLY | KOPEN_APPEND | KOPEN_CREAT
*                ! w   = KOPEN_WRONLY | KOPEN_CREAT  | KOPEN_TRUNC
*                ! r   = KOPEN_RDONLY
*         Input: type - the kfopen string field.
*       Returns: the kopen flags on success.  If type is NULL or an invalid
*		 type, a -1 will be returned.
*    Written By: Mark Young
*          Date: Apr 27, 1993
*************************************************************/

int ktype_to_flags(
   const char *type)
{
	int	flags = 0;

	if (!type)
	   return(0);

	switch (type[0])
	{
	   case 'r':
	   case 'R':
		flags = (((int) type[1] == '+') ? KOPEN_RDWR : KOPEN_RDONLY);

		if ((int) type[0] == 'R')
		   flags |= KOPEN_NONBLOCK;

		/*
	 	 *  Extended 
	 	 */
		break;

	   case 'w':
	   case 'W':
	   case 'a':
	   case 'A':
		flags = ((type[1] == '+') ? KOPEN_RDWR : KOPEN_WRONLY) | KOPEN_CREAT;
		if ((int) type[0] == 'W' || (int) type[0] == 'A')
		   flags |= KOPEN_NONBLOCK;

		if ((int) type[0] == 'a' || (int) type[0] == 'A')
		   flags |= KOPEN_APPEND;
		else
		   flags |= KOPEN_TRUNC;
	 	break;

	    default:
		return(0);
	}
	return(flags);
}
