/*
 * Khoros: $Id: kprocess.c,v 1.13.2.1 1997/03/06 23:29:09 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kprocess.c,v 1.13.2.1 1997/03/06 23:29:09 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Unix Sytem Utilities
   >>>>		
   >>>>  Private:
   >>>>                 void()
   >>>>   Public:
   >>>>                 kwait3()
   >>>>                 ksignal()
   >>>>                 ksignal_print()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


/* The following is intended to get SA_NOCLDWAIT defined on
   solaris and osf
*/
#if defined(KOPSYS_SOLARIS)  || defined(KOPSYS_OSF)
#define __EXTENSIONS__
#define _POSIX_C_SOURCE 3
#include <signal.h>
#undef __EXTENSIONS__
#endif


#include "internals.h"	
#include <kutils/ksignal.h>


/************************************************************
*
*  Routine Name: kwait3 - replacement for the system wait3()
*
*       Purpose: This return emulates the bsd wait3() command.
*		 The wait3 process waits until any child has
*		 completed, without hanging (WNOHANG).  It
*		 should be called rather than the system wait3,
*		 because not all systems include a wait3 call.
*
*         Input: options - options used for wait3 (WUNTRACED, WNOHANG)
*
*        Output: status  - the return status of the process
*		 rusage  - the resource usage (for systems that do not
*			   have a wait3 call, this output is not set.)
*                RETURNS: the status of the process.
*
*  Restrictions: Restrictions on data or input as applicable
*    Written By: Bob Weaver
*          Date: Jul 03, 1992 14:06
*      Verified:
*  Side Effects:
* Modifications: Converted from vwait3 in Khoros 1.0 (SJ)
*
*************************************************************/

int kwait3(
   kstatus *status,
   int     options,
   kaddr   rusage)
{
    return(waitpid(-1, status, options));
}

/************************************************************
*
*  Routine Name: ksignal - khoros signal handler
*
*       Purpose: This routine is a replacement for the UNIX signal()
*		 handler.  The problem with UNIX signal() is that
*		 under SVR4 signal() provedes the older, unreliable
*		 signal semantics.
*
*		 The 
*
*         Input: signo  -  the signal number that you wish to install the
*			   handler for.
*                sigfunc - the signal function to be called when the signal
*			   is encountered.
*
*        Output: 
*       Returns: the previously installed signal handler on success, SIG_ERR
*		 on failure
*  Restrictions:
*    Written By: Mark Young & John Salas
*          Date: Mar 24, 1994
*      Verified:
*  Side Effects:
* Modifications: PH 1/15/97: Added special handling for SIGCHLD/SIG_IGN
*
*************************************************************/

void (*ksignal(
   int signo,
   void (*sigfunc)(int)))(int)
{
	struct sigaction act, oact;

	act.sa_handler = (void(*)(int)) sigfunc;
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;
	if (signo == SIGALRM)
	{
#ifdef SA_INTERRUPT
	   act.sa_flags |= SA_INTERRUPT;
#endif
	}
	else
	{
#ifdef SA_RESTART
	   act.sa_flags |= SA_RESTART;
#endif
	}
#ifdef SA_NOCLDWAIT
	if (sigfunc == SIG_IGN  &&  signo == SIGCHLD)
	{
	   /* SIG_IGN on SIGCHLD requires special treatment */
	   act.sa_flags |= SA_NOCLDWAIT;
	}
#endif


	if (sigaction(signo, &act, &oact) < 0)
	   return(SIG_ERR);

	return((void(*)(int))oact.sa_handler);
}

/************************************************************
*
*  Routine Name: ksignal_format - format the actual signal error
*
*       Purpose: This routine actually prints to a string the signal
*		 that occured.
*
*         Input: signo  - the signal to print.
*        Output: string - the string
*       Returns: 
*  Restrictions: 
*    Written By: Mark Young & John Salas
*          Date: Nov 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

char *ksignal_format(
   int   signo,
   char  *string)
{
	char temp[KLENGTH];

	if (string == NULL)
	   string = temp;

	switch (signo)
	{
	   case SIGALRM:
		ksprintf(string,"Alarm has timed out...\n");
		break;

	   case SIGCHLD:
		ksprintf(string,"Child process terminated or stopped...\n");
		break;

	   case SIGCONT:
		ksprintf(string,"Continuing process...\n");
		break;

	   case SIGHUP:
		ksprintf(string,"Hangup encountered...\n");
		break;

	   case SIGKILL:
		ksprintf(string,"Routine killed...\n");
		break;

	   case SIGPIPE:
		ksprintf(string,"Write on a pipe but no process to read "
				"it...\n");
		break;

	   case SIGQUIT:
		ksprintf(string,"Quit character typed on controlling "
			      "keyboard...\n");
		break;

	   case SIGSTOP:
		ksprintf(string,"Routine stopped...\n");
		break;

	   case SIGTSTP:
		ksprintf(string,"Stop special character typed on controlling "
			      "keyboard...\n");
		break;

	   case SIGTTIN:
		ksprintf(string,"Read from controlling terminal atstringted "
			      "by a member of a background process group...\n");
		break;

	   case SIGTTOU:
		ksprintf(string,"Write to controlling terminal atstringted "
			      "by a member of a background process group...\n");
		break;

	   case SIGUSR1:
		ksprintf(string,"Application-defined signal 1...\n");
		break;

	   case SIGUSR2:
		ksprintf(string,"Application-defined signal 2...\n");
		break;

	   case SIGABRT:
		ksprintf(string,"Routine aborted...\n");
		break;

	   case SIGFPE:
		ksprintf(string,"Arithmetic exception...\n");
		break;

	   case SIGILL:
		ksprintf(string,"Illegal instruction...\n");
		break;
#ifdef SIGBUS
	   case SIGBUS:
		ksprintf(string,"Bus error...\n");
		break;
#endif
	   case SIGINT:
		ksprintf(string,"Routine interupted...\n");
		break;

	   case SIGSEGV:
		ksprintf(string,"Segmentation violation...\n");
		break;

	   case SIGTERM:
		ksprintf(string,"Software terminated from signal...\n");
		break;

	   default:
		ksprintf(string,"Unknown signal number (%d).... Check "
			      "signal.3\n", signo);
        }
	return(string == temp ? kstrdup(string) : string);
}
