/*
 * Khoros: $Id: kread.c,v 1.16.2.4 1997/05/24 17:50:14 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kread.c,v 1.16.2.4 1997/05/24 17:50:14 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Transport Machine Independent Reading Utilities
   >>>>		
   >>>>  Private:
   >>>>                 kfread_generic()
   >>>>   Static:
   >>>>                 cleanup_handler()
   >>>>                 save_address()
   >>>>                 read_convert()
   >>>>                 read_pointers()
   >>>>   Public:
   >>>>                 kread_generic()
   >>>>                 kread_array()
   >>>>                 kread_pointer()
   >>>>                 kread_struct()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	

static size_t numaddrs = 0;
static size_t maxaddrs = 0;
static kaddr *addrs = NULL;
static int saveaddrs = FALSE;
static int override_saveaddrs = FALSE;
static size_t numptrs = 0;
static size_t maxptrs = 0;
static kaddr **ptrs = NULL;
static int   *types = NULL;
static int numleft = 0;
 
static size_t convert_size = 0;
static kaddr  convert_data = NULL;

#define FAST_STRUCTURE FALSE


/*-----------------------------------------------------------*/
ssize_t kfread_generic(
   kfile *file,
   kaddr data,
   size_t  num,
   int  type)
{
	return(kread_generic(kfileno(file), data, num, type));
}

/*-----------------------------------------------------------
|
|  Routine Name: cleanup_handler - private routine for cleanup on exit
|
|       Purpose: The cleanup handler routine frees memory on termination
|		 of the program.
|
|    Written By: Mark Young  
|          Date: Mar 5, 1997
| Modifications:
|
------------------------------------------------------------*/
static void cleanup_handler(void)
{
	kfree_and_NULL(addrs);
	kfree_and_NULL(ptrs);
	kfree_and_NULL(types);
 
        numaddrs = 0;
        maxaddrs = 0;
        numptrs = 0;
        maxptrs = 0;
        numleft = 0;
        convert_size = 0;
}

/*-----------------------------------------------------------
|
|  Routine Name: save_address - private routine for saving addresses
|
|       Purpose: The save_address routine is used to save the addresses
|		 so that we can re-connect pointers.
|
|         Input: addr - the address of the pointer that was read
|    Written By: Mark Young  
|          Date: Jan 15, 1997
| Modifications:
|
------------------------------------------------------------*/
static void save_address(
   kaddr addr)
{
	if (numaddrs == maxaddrs)
	{
	   if (maxaddrs == 0)
	      maxaddrs = KLENGTH;
	   else
	      maxaddrs = maxaddrs << 1;

	   addrs  = (kaddr *) krealloc(addrs, maxaddrs * sizeof(kaddr));
	}
	addrs[numaddrs++] = addr;
}

/*-----------------------------------------------------------
|
|  Routine Name: read_convert - private read and conversion routine
|
|       Purpose: The read_convert routine is the grunt routine that
|		 does the actual reading of data as well calls the
|		 appropriate conversion routine. 
|
|         Input: kfile - the device in which we will read the data from
|                addr   - the data array in which we will be storing the
|			 converted data
|                src_esize  - the source element size
|                dest_esize - the destination element size
|                num        - the number of data points to be read and converted
|                converter  - the routine that converts the data for that
|			      routine.
|
|        Output: None
|       Returns: number of converted data points on success, -1 otherwise
|
|    Written By: Mark Young  
|          Date: Jun 14, 1992
| Modifications:
|
------------------------------------------------------------*/
static ssize_t read_convert(
   kfile         *file,
   kaddr         addr,
   size_t        src_esize,
   size_t        dest_esize,
   size_t        num,
   unsigned char *(*converter) PROTO((unsigned char *, unsigned char *,
			unsigned int, unsigned int, size_t)) )
{
	size_t nbytes;
	ssize_t count = 0;
	kaddr  tmp = addr;
	int    override = FALSE;
	unsigned int src_mach, dest_mach;
	
	src_mach  = file->machtype;
	dest_mach = KMACH_LOCAL;
	if (src_esize != dest_esize)
	{
	   if (convert_size < num*src_esize)
	   {
	      convert_size = num*src_esize;
	      convert_data = (kaddr) krealloc(convert_data, convert_size);
	   }
	   tmp = convert_data;
	}

	/*
	 *  If we need to save the pointers being written in order to write
	 *  delayed pointers then save it to the ptr array.
	 */
	if (saveaddrs == TRUE && override_saveaddrs == FALSE)
	   save_address(addr);

	/*
	 *  Read and convert the data.  Byte and Bit don't need to be converted,
	 *  so check to see if we can override.  Note that the Bit data size
	 *  is 0, hence the <= check.
	 */
	if (src_esize <= 1 && dest_esize <= 1)
	   override = TRUE, src_esize = 1;

	nbytes = src_esize * num;
	count = kfile_read(file, (char *) tmp, nbytes)/src_esize;
	if (count > 0 && src_mach != dest_mach && !override)
	{
	   if (!converter((unsigned char *) tmp, (unsigned char *) addr,
			src_mach, dest_mach, (size_t) count))
	      return(0);
	}
	return(count);
}

/*-----------------------------------------------------------
|
|    Routine Name: read_pointers - read a variable array or pointer
|         Purpose: This module is used to read a pointer or a pointer to a
|                  variable sized array.  The size of the array
|                  and the data is stored in the transport.  The
|                  read first reads the number of points,
|                  and then calls kread_generic() to do the actual writing
|                  of the data.  If NULL is passed in then kread_pointer()
|                  will read in 0 for the number of points, which will
|                  then be used by kread_pointer() to return NULL.
|
|      Written By: Mark Young
|            Date: Aug 15, 1995
------------------------------------------------------------*/
static int read_pointers(
   int id)
{
        kaddr *data;
	size_t i = 0, size;
        int   indx, machtype, status = TRUE;

	machtype = kfile_getmachtype(id);
        while (i < numptrs)
	{
           data = ptrs[i];
	   if (numleft < -1)
	   {
	      *data = NULL;
	      i++, numleft++;
	      continue;
	   }

	   override_saveaddrs = TRUE;
	   if (kread_int(id, &indx, (size_t) 1) != 1)
              break;
	   override_saveaddrs = FALSE;

	   if (indx == -1)
	   {
	      /*
	       *  malloc enough space so that we can call kread_generic() to do
	       *  the actual reading.
	       */
	      if (!kstruct_getinfo(types[i], NULL, &size, NULL, NULL,NULL,NULL))
	         size = kmach_sizeof((unsigned int) machtype,
				     (unsigned int) types[i]);

	      if ((*data = kmalloc(size)) == NULL)
	         break;

	      if (kread_generic(id, (kaddr) *data, (size_t) 1, types[i]) != 1)
		 break;
	   }
	   else if (indx < -1)
	   {
	      *data = NULL;
	      numleft = indx+1;
	   }
	   else
	      *data = addrs[indx];

	   i++;
	}

	if (i < numptrs)
	   status = FALSE;

	/*
	 *  Reset global variables and free any temporary memory
	 *  allocated while saving pointers.
	 */
	saveaddrs = FALSE;
        cleanup_handler();
	return(TRUE);
}


/************************************************************
*  
*    Routine Name: kread_generic - read in any data type.
*         Purpose: This module is used to read an array of 
*      	       	   data from a transport.  The data will need 
*      	       	   to contain enough memory to store "num" 
*      	       	   data points of the specified type.
*           Input: id   - the file id to be read which was
*      	       	       	  opened earlier with kopen().
*		   num  - the number of data points to be
*      	       	       	  read into the data array.
*      	       	   type - data type.
*	   Output: data - the kaddr array in which the data will be stored.
*         Returns: the number of data points read
*      Written By: Mark Young
*            Date: Aug 25, 1995
*************************************************************/
ssize_t kread_generic(
   int     id,
   kaddr  data,
   size_t num,
   int    type)
{
	char	*buf;
	kstring *strings;
	int	reset_file = FALSE, record = FALSE;
	int  (*read_struct) PROTO((int, kaddr, int));
	size_t  src, dest, size;
	ssize_t i, snum = -1;

	static int otype = -1;
	static size_t osrc, odest;
	static kfile *file = NULL;


#if FAST_STRUCTURE
        if (!override_saveaddrs && type > 0 && num == 1)
        {
           save_address(data);
           return(1);
        }
#endif
        /*
         *  Get the file entry associated with the id
         */
        if (file == NULL)
	{
	   reset_file = TRUE;
	   if ((file = kfile_checkid(id)) == NULL)
           {
              errno = EBADF;
              return(-1);
           }
	}

	/*
	 *  Get the source and destination element sizes for unsigned longs
	 *  for the local (destination) and remote (source) machine types.
	 */
	if (saveaddrs && type == otype)
	{
	   src  = osrc;
	   dest = odest;
	}
	else if (KMACH_LOCAL != file->machtype)
	{
	   src  = kmach_sizeof(file->machtype, (unsigned int) type);
	   dest = kmach_sizeof(KMACH_LOCAL, (unsigned int) type);
	}
	else
	   src = dest = kmach_sizeof(KMACH_LOCAL, (unsigned int) type);

	/*
	 *  Set the correct converter according to the specified data
	 *  type.
	 */
        switch(type)
	{
       	   case KBIT:
		size = (num+7)/8;
	        snum = read_convert(file, data, src, dest, size, convert_byte);
		if (snum != -1)
		   snum = kmin(size*8, num);
		break;
		
       	   case KBYTE:
	        snum = read_convert(file, data, src, dest, num, convert_byte);
		break;

       	   case KUBYTE:
	        snum = read_convert(file, data, src, dest, num,convert_ubyte);
		break;

       	   case KSHORT:
	        snum = read_convert(file, data, src, dest, num, convert_short);
		break;

       	   case KUSHORT:
	        snum = read_convert(file, data, src, dest, num,convert_ushort);
		break;

       	   case KLONG:
	        snum = read_convert(file, data, src, dest, num, convert_long);
		break;

       	   case KULONG:
	        snum = read_convert(file, data, src, dest, num, convert_ulong);
		break;

       	   case KINT:
	        snum = read_convert(file, data, src, dest, num, convert_int);
		break;

       	   case KUINT:
	        snum = read_convert(file, data, src, dest, num, convert_uint);
		break;

       	   case KFLOAT:
	        snum = read_convert(file, data, src, dest, num, convert_float);
		break;

       	   case KDOUBLE:
	        snum = read_convert(file, data, src, dest, num,convert_double);
		break;

       	   case KCOMPLEX:
	        snum = read_convert(file, data, src, dest,num,convert_complex);
		break;

       	   case KDCOMPLEX:
	        snum = read_convert(file, data, src,dest,num,convert_dcomplex);
		break;

       	   case KSTRING:
		/* call the associated read string routine */
	        strings = (kstring *) data;
		for (i = 0; i < num; i++)
		{
		   strings[i] = kfile_readstring(file, '\0', NULL, 0);
		   if (strings[i] == NULL)
		   {
		      if (reset_file) file = NULL;
		      return((ssize_t)i);
		   }
		}
		snum = (ssize_t) num;
		break;

       	   default:
		/* retrieve the read structure specified by the type */
		if (!kstruct_getinfo(type, NULL, &size, &read_struct, NULL,
				     NULL, NULL) || !read_struct)
		{
		   if (reset_file) file = NULL;
		   return(-1);
		}

	        if (saveaddrs == FALSE)
		   record = saveaddrs = TRUE;

		/* call the associated read struct routine */
		buf = (char *) data;
		for (i = 0; i < num; i++, buf += size)
		{
#if FAST_STRUCTURE
                   if (kread(id, buf, size) != size)
		   {
		      if (reset_file) file = NULL;
                      return((ssize_t)i);
		   }
#endif
		   if (read_struct(id, buf, type) == FALSE)
		   {
		      if (reset_file) file = NULL;
		      return((ssize_t)i);
		   }

		   /* save the pointer in case a pointer is pointing to us */
		   if (override_saveaddrs == FALSE)
		      save_address(buf);
		}
		snum = (ssize_t) num;

	        if (record == TRUE)
		{
		   if (read_pointers(id) == FALSE)
		      snum = -1;
		}
		break;
       	}
	otype = type, osrc = src, odest = dest;
	if (reset_file) file = NULL;
	return(snum);
}

/************************************************************
*  
*    Routine Name: kread_array - read in a variable array
*         Purpose: This module is used to read in a pointer to an
*		   variable sized array.  The size of the array
*		   and the data is stored in the transport.  The
*		   reader first reads the number of points, allocates
*		   enough space for the return data, and then calls
*		   kread_generic() to do the actual reading of the
*		   data.  If NULL is stored then kread_array()
*		   will set the data pointer to NULL and return that
*		   0 data points were read.
*
*		   The "num" argument should be used to represent a
*		   maximum number of data points to be read.  In the
*		   case that num is less than the number of data points
*		   stored, then kread_array() will advance to the
*		   end of the stored data.  This allows for partial
*		   reads of the stored data.
*           Input: id   - the file id to be read which was
*      	       	       	  opened earlier with kopen().
*		   num  - the maximum number of data points to be 
*      	       	       	  read into the data array.
*      	       	   type - data / structure type.
*	   Output: data - the kaddr array in which the data will be stored.
*         Returns: the number of points read or -1 on error
*      Written By: Mark Young
*            Date: Aug 15, 1995
*************************************************************/
ssize_t kread_array(
   int  id,
   kaddr *data,
   size_t num,
   int    type)
{
	size_t  size;
	unsigned int filemax;
	ssize_t maxnum;

        /*
         *  Get the file entry associated with the id
         */
        if (!saveaddrs && kfile_checkid(id) == NULL)
	{
	   errno = EBADF;
	   return(-1);
	}

	if (data == NULL)
	   return(-1);

	/*
	 *  Read from the file the number of data points to follow.  Then
	 *  check to see if the maximum of desired points is less than the
	 *  number of stored points.  If not then we gotta do something
	 *  about it.
	 */
	override_saveaddrs = TRUE;
	if (kread_uint(id, &filemax, (size_t) 1) != 1)
	   return(-1);
	override_saveaddrs = FALSE;

	/*
	 *  Need to increment the address index for back-wards compatibiity.
	 *  The above kread_uint() originally did not use override_saveaddrs
	 *  to indicate that we should not record indexes.  So we end up doing
	 *  a save_address(NULL) even though we aren't really saving anything.
	 */
	save_address(NULL);

	if (num > 0) 
	   num = kmin(filemax, num);
	else
	   num = filemax;

        /* if we are not reading anything, then we are done! */
        if (num == 0)
        {
           *data = NULL;
           return(0);
        }

	/*
	 *  malloc enough space so that we can call kread_generic() to do
	 *  the actual reading.
	 */
	if (!kstruct_getinfo(type, NULL, &size, NULL, NULL, NULL, NULL))
	   size = kmach_sizeof(KMACH_LOCAL, (unsigned int) type);

	if ((*data = kmalloc(size * num)) == NULL)
	   return(-1);

	maxnum = kread_generic(id, *data, num, type);
	return(maxnum);
}

/************************************************************
*  
*    Routine Name: kread_pointer - read in a variable array
*         Purpose: This module is used to read in a pointer to an
*		   variable sized array.  The size of the array
*		   and the data is stored in the transport.  The
*		   reader first reads the number of points, allocates
*		   enough space for the return data, and then calls
*		   kread_generic() to do the actual reading of the
*		   data.  If NULL is stored then kread_pointer()
*		   will set the data pointer to NULL and return that
*		   0 data points were read.
*           Input: id   - the file id to be read which was
*      	       	       	  opened earlier with kopen().
*      	       	   type - data / structure type.
*	   Output: data - the kaddr array in which the data will be stored.
*         Returns: TRUE (1) on success, FALSE (0) otherwise
*      Written By: Mark Young
*            Date: Aug 15, 1995
*************************************************************/
int kread_pointer(
   int   id,
   kaddr *data,
   int   type)
{
        /*
         *  Get the file entry associated with the id
         */
        if (!saveaddrs && kfile_checkid(id) == NULL)
        {
           errno = EBADF;
           return(-1);
        }

	if (data == NULL)
	   return(-1);

	if (numleft < -1)
	{
	   *data = NULL;
	   numleft++;
	   return(TRUE);
	}

	if (numptrs == maxptrs)
	{
	   maxptrs = (maxptrs != 0) ? maxptrs << 1 : KLENGTH;
	   types = (int *) krealloc(types, maxptrs * sizeof(int));
	   ptrs  = (kaddr **) krealloc(ptrs, maxptrs * sizeof(kaddr *));
	}
	types[numptrs] = type;
	ptrs[numptrs++] = data;
	if (saveaddrs == FALSE)
	{
	   saveaddrs = TRUE;
	   if (read_pointers(id) == FALSE)
	      return(FALSE);
	}
	return(TRUE);
}

/************************************************************
*
*    Routine Name: kread_struct - read in a single structure
*         Purpose: This module is used to read in a single structure
*                  from a transport.  Unlike kread_generic() this
*		   routine will malloc the returned memory for a single
*		   structure.
*           Input: id   - the file id to be read which was
*                         opened earlier with kopen().
*                  type - structure type.
*          Output: data - a pointer in which the structures will be stored.
*         Returns: the number of structs read
*      Written By: Mark Young
*            Date: Aug 04, 1995
*************************************************************/

ssize_t kread_struct(
   int    id,
   kaddr *data,
   int    type)
{
	size_t size;

        /*
         *  Get the file entry associated with the id
         */
        if (!saveaddrs && kfile_checkid(id) == NULL)
        {
           errno = EBADF;
           return(-1);
        }

	/*
	 *  malloc enough space so that we can call kread_generic() to do
	 *  the actual reading.
	 */
	if (!kstruct_getinfo(type, NULL, &size, NULL, NULL, NULL, NULL))
	   size = kmach_sizeof(KMACH_LOCAL, (unsigned int) type);

	if ((*data = kmalloc(size)) == NULL)
	   return(-1);

	return(kread_generic(id, *data, (size_t) 1, type));
}
