/*
 * Khoros: $Id: kwrite.c,v 1.14.2.4 1997/05/24 17:50:19 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kwrite.c,v 1.14.2.4 1997/05/24 17:50:19 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $CONVERT/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Transport Machine Independent Writing Utilities
   >>>>		
   >>>>  Private:
   >>>>                 kfwrite_generic()
   >>>>                 kwrite_saveaddrs()
   >>>>   Static:
   >>>>                 write_convert()
   >>>>                 write_pointers()
   >>>>   Public:
   >>>>                 kwrite_generic()
   >>>>                 kwrite_array()
   >>>>                 kwrite_pointer()
   >>>>                 kwrite_struct()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	

static size_t addrindx = 0;
static khash *addrs = NULL;
static int saveaddrs = FALSE;
static int override_saveaddrs = FALSE;
static size_t numptrs = 0;
static size_t maxptrs = 0;
static kaddr **ptrs = NULL;
static int   *types = NULL;

static size_t convert_size = 0;
static kaddr  convert_data = NULL;

static int save_scalar = TRUE;
static int save_struct = TRUE;
static int save_array  = TRUE;

#define FAST_STRUCTURE FALSE

/*-----------------------------------------------------------*/
ssize_t kfwrite_generic(
   kfile *file,
   kaddr  data,
   size_t num,
   int    type)
{
	return(kwrite_generic(kfileno(file), data, num, type));
}

/*-----------------------------------------------------------*/
void kwrite_saveaddrs(
   int scalar_addrs,
   int struct_addrs,
   int array_addrs)
{
	save_scalar = scalar_addrs;
	save_struct = struct_addrs;
	save_array  = array_addrs;
}

/*-----------------------------------------------------------
|
|  Routine Name: cleanup_handler - private routine for cleaning up.
|
|       Purpose: The cleanup handler routine frees memory after writing the
|		 structure.
|
|    Written By: Mark Young  
|          Date: Mar 5, 1997
| Modifications:
|
------------------------------------------------------------*/
static void cleanup_handler(void)
{
        khash_free(addrs, NULL);
        kfree_and_NULL(ptrs);
        kfree_and_NULL(types);
        kfree_and_NULL(convert_data);
 
        addrs = NULL;
        addrindx = 0;
        numptrs = 0;
        maxptrs = 0;
        convert_size = 0;
}

/*-----------------------------------------------------------
|
|  Routine Name: write_convert - private write and conversion routine
|
|       Purpose: The write_convert routine is the grunt routine that
|		 does the actual writing of data as well calls the
|		 appropriate conversion routine. 
|
|         Input: kfile - the device in which we will write the data to
|                addr   - the data array in which we will be storing the
|			 converted data
|                src_esize  - the source element size
|                dest_esize - the destination element size
|                num        - the number of elements to be converted and written
|                converter  - the routine that converts the data for that
|			      routine.
|
|        Output: None
|         Returns: number of converted elements on success, -1 otherwise
|
|    Written By: Mark Young  
|          Date: Jun 14, 1992
| Modifications:
|
------------------------------------------------------------*/
/*ARGSUSED*/
static ssize_t write_convert(
   kfile         *file,
   kaddr         addr,
   size_t        src_esize,
   size_t        dest_esize,
   size_t        num,
   unsigned char *(*converter) PROTO((unsigned char *, unsigned char *,
			unsigned int, unsigned int, size_t)) )
{
	size_t   nbytes;
	ssize_t  count = 0;
	int      override = FALSE;
	unsigned int src_mach, dest_mach;
	
	/*
	 *  Retrieve the source and destination machine types.
	 */
	src_mach  = KMACH_LOCAL;
	dest_mach = file->machtype;
	if (convert_size < num*dest_esize)
	{
	   convert_size = num*dest_esize;
	   convert_data = (kaddr) krealloc(convert_data, convert_size);
	}

	/*
	 *  If we need to save the pointers being written in order to write
	 *  delayed pointers then save it to the addr array.
	 */
	if (saveaddrs == TRUE && override_saveaddrs == FALSE)
	{
	   if ((save_scalar && num == 1) || (save_array && num > 1))
	      khash_add(addrs, addr, (kaddr) addrindx++);
	   else
	      addrindx++;
	}

	/*
	 *  Convert and write the data.  Byte and Bit don't need to be
	 *  converted, so check to see if we can override.  Note that the
	 *  Bit data size is 0, hence the <= check.
	 */
	if (src_esize <= 1 && dest_esize <= 1)
	   override = TRUE, dest_esize = 1;

	if (num > 0 && src_mach != dest_mach && !override)
	{
	   if (!converter((unsigned char *) addr, (unsigned char *)convert_data,
				src_mach, dest_mach, num))
	      return(0);

	   addr = convert_data;
	}
	nbytes = dest_esize * num;
	count = kfile_write(file, (char *) addr, nbytes)/dest_esize;
	return(count);
}

/*-----------------------------------------------------------
|
|    Routine Name: write_pointers - write a variable array or pointer
|         Purpose: This module is used to write a pointer or a pointer to a
|		   variable sized array.  The size of the array
|		   and the data is stored in the transport.  The
|		   writer first writes the number of points,
|		   and then calls kwrite_generic() to do the actual writing
|		   of the data.  If NULL is passed in then kwrite_pointer()
|		   will write out 0 for the number of points, which will
|		   then be used by kwrite_pointer() to return NULL.
|
|      Written By: Mark Young
|            Date: Aug 15, 1995
------------------------------------------------------------*/
static int write_pointers(
   int id)
{
	kaddr *data, tmp;
	int   status = TRUE;
	size_t i = 0;
	ssize_t indx;

	while (i < numptrs)
	{

	   data = ptrs[i];
	   if ((data = ptrs[i]) == NULL)
	      indx = (ssize_t) -1 - types[i];
	   else if (khash_clientdata(addrs, *data, &tmp) == TRUE)
	      indx = (ssize_t) tmp;
	   else
	      indx = -1;

	   override_saveaddrs = TRUE;
	   if (kwrite_int(id, &indx, (ssize_t) 1) != 1)
	      break;
	   override_saveaddrs = FALSE;

	   if (indx == -1)
	   {
	      if (kwrite_generic(id, *data, (ssize_t) 1, types[i]) != 1)
		 break;
	   }
	   i++;
	}

	if (i < numptrs)
	   status = FALSE;

	/*
	 *  Reset global variables and free any temporary memory
	 *  allocated while saving pointers.
	 */
	saveaddrs = FALSE;
	khash_reinit(addrs, NULL);
	addrindx = numptrs = 0;
	return(status);
}


/************************************************************
*  
*    Routine Name: kwrite_generic - write an array in any data type.
*         Purpose: This module is used to write an array of 
*      	       	   data to a transport.
*           Input: id   - the file id to be written to which was
*			  opened earlier with kopen().
*		   data - the kaddr array which holds the data.
*		   num  - the number of data points to be written.
*      	       	   type - data type.
*         Returns: the number of points written or -1 on error
*      Written By: Mark Young
*            Date: Aug 25, 1995
*************************************************************/
ssize_t kwrite_generic(
   int    id,
   kaddr  data,
   size_t num,
   int    type)
{
	char	 *buf;
	kstring  *strings;
	int     (*write_struct) PROTO((int, kaddr, int));
	int	  reset_file = FALSE, record = FALSE, call_cleanup = FALSE;
	size_t    src, dest, size;
	ssize_t   i, snum = -1;

	static int otype = -1;
	static size_t osrc, odest;
	static kfile *file = NULL;


#if FAST_STRUCTURE
        if (!override_saveaddrs && type > 0 && num == 1)
        {
           if ((save_scalar && num == 1) || (save_array && num > 1))
              khash_add(addrs, data, (kaddr) addrindx++);
           else
              addrindx++;

           return(1);
        }
#endif
        /*
         *  Get the file entry associated with the id
         */
        if (file == NULL)
	{
	   reset_file = TRUE;
	   if ((file = kfile_checkid(id)) == NULL)
           {
              errno = EBADF;
              return(-1);
           }
	}

	/*
	 *  Get the source and destination element sizes for unsigned longs
	 *  for the local (destination) and remote (source) machine types.
	 */
	if (saveaddrs && type == otype)
	{
	   src  = osrc;
	   dest = odest;
	}
	else if (KMACH_LOCAL != file->machtype)
	{
	   src  = kmach_sizeof(KMACH_LOCAL, (unsigned int) type);
	   dest = kmach_sizeof(file->machtype, (unsigned int) type);
	}
	else
	   src = dest = kmach_sizeof(KMACH_LOCAL, (unsigned int) type);

	/*
	 *  Set the correct converter according to the specified data
	 *  type.
	 */
        switch(type)
	{
       	   case KBIT:
		size = (num+7)/8;
	        snum = write_convert(file, data, src, dest, size, convert_byte);
		if (snum != -1)
		   snum = kmin(snum*8, num);
		break;
		
       	   case KBYTE:
	        snum = write_convert(file, data, src, dest, num, convert_byte);
		break;

       	   case KUBYTE:
	        snum = write_convert(file, data, src, dest, num, convert_ubyte);
		break;

       	   case KSHORT:
	        snum = write_convert(file, data, src, dest, num, convert_short);
		break;

       	   case KUSHORT:
	        snum = write_convert(file, data, src, dest,num, convert_ushort);
		break;

       	   case KLONG:
	        snum = write_convert(file, data, src, dest, num, convert_long);
		break;

       	   case KULONG:
	        snum = write_convert(file, data, src, dest, num, convert_ulong);
		break;

       	   case KINT:
	        snum = write_convert(file, data, src, dest, num, convert_int);
		break;

       	   case KUINT:
	        snum = write_convert(file, data, src, dest, num, convert_uint);
		break;

       	   case KFLOAT:
	        snum = write_convert(file, data, src, dest, num, convert_float);
		break;

       	   case KDOUBLE:
	        snum = write_convert(file, data, src, dest,num, convert_double);
		break;

       	   case KCOMPLEX:
	        snum = write_convert(file, data, src,dest,num, convert_complex);
		break;

       	   case KDCOMPLEX:
	        snum = write_convert(file, data,src,dest,num, convert_dcomplex);
		break;

       	   case KSTRING:
		/* call the associated write string routine */
	        strings = (kstring *) data;
		for (i = 0; i < num; i++)
		{
		   buf = (strings[i] != NULL) ? strings[i] : "";
		   if (kwrite(id, buf, kstrlen(buf) + 1) == -1)
		   {
		      if (reset_file) file = NULL;
		      return((ssize_t)i);
		   }
		}
		snum = (ssize_t) num;
		break;
		

       	   default:
		/* retrieve the write structure specified by the type */
		if (!kstruct_getinfo(type, NULL, &size, NULL,
		    &write_struct, NULL, NULL) || !write_struct)
		{
		   if (reset_file) file = NULL;
		   return(-1);
		}

	        if (saveaddrs == FALSE)
		{
		   if (addrs == NULL)
		   {
		      call_cleanup = TRUE;
		      addrs = khash_init(KSTRUCT, (size_t) 1013);
		   }
	           record = saveaddrs = TRUE;
		}

		/* call the associated write struct routine */
		buf = (char *) data;
		for (i = 0; i < num; i++, buf += size)
		{
#if FAST_STRUCTURE
                   if (kwrite(id, buf, size) != size)
		   {
		      if (reset_file) file = NULL;
                      return((ssize_t)i);
		   }
#endif
		   if (write_struct(id, buf, type) == FALSE)
		   {
		      if (reset_file) file = NULL;
		      return((ssize_t)i);
		   }

		   /* save the pointer in case a pointer is pointing to us */
		   if (save_struct && override_saveaddrs == FALSE)
		      khash_add(addrs, buf, (kaddr) addrindx++);
		}
		snum = (ssize_t) num;

		if (record == TRUE)
		{
		   if (write_pointers(id) == FALSE)
		      snum = -1;

		   /* cleanup out mess */
		   if (call_cleanup) cleanup_handler();
		}
		break;
       	}
	otype = type, osrc = src, odest = dest;
	if (reset_file) file = NULL;
	return(snum);
}

/************************************************************
*  
*    Routine Name: kwrite_array - write a variable array
*         Purpose: This module is used to write a pointer to an
*		   variable sized array.  The size of the array
*		   and the data is stored in the transport.  The
*		   writer first writes the number of points,
*		   and then calls kwrite_generic() to do the actual writing
*		   of the data.  If NULL is passed in then kwrite_array()
*		   will write out 0 for the number of points, which will
*		   then be used by kwrite_array() to return NULL.
*
*           Input: id   - the file id to be written to which was
*      	       	       	  opened earlier with kopen().
*		   data - the array in which the data will be written from.
*		   num  - the number of data points to be written to the
*			  transport.
*      	       	   type - data / structure type.
*	   Output: 
*         Returns: the number of points written or -1 on error
*      Written By: Mark Young
*            Date: Aug 15, 1995
*************************************************************/
ssize_t kwrite_array(
   int    id,
   kaddr  data,
   size_t num,
   int    type)
{
	ssize_t snum = 0;
	unsigned int size = (unsigned int) num;

        /*
         *  Get the file entry associated with the id
         */
        if (!saveaddrs && kfile_checkid(id) == NULL)
        {
           errno = EBADF;
           return(-1);
        }

	/*
	 *  If the data is NULL, then we need to write out 0, which will
	 *  be interpeted by kwrite_array() as being NULL.
	 */
	if (data == NULL)
	   num = 0;

	/*
	 *  Write to the file the number of data points to follow.
	 */
	override_saveaddrs = TRUE;
	if (kwrite_uint(id, &size, (size_t) 1) != 1)
	   return(-1);
	override_saveaddrs = FALSE;

	/*
	 *  Need to increment the address index for back-wards compatibiity.
	 *  The above kwrite_uint() originally did not use override_saveaddrs
	 *  to indicate that we should not record indexes.  So increment the
	 *  index even though we aren't really saving anything.
	 */
        addrindx++;

	if (num > 0)
	   snum = kwrite_generic(id, data, num, type);

	return(snum);
}

/************************************************************
*  
*    Routine Name: kwrite_pointer - write a variable array
*         Purpose: This module is used to write a pointer to an
*		   variable sized array.  The size of the array
*		   and the data is stored in the transport.  The
*		   writer first writes the number of points,
*		   and then calls kwrite_generic() to do the actual writing
*		   of the data.  If NULL is passed in then kwrite_pointer()
*		   will write out 0 for the number of points, which will
*		   then be used by kread_pointer() to return NULL.
*
*           Input: id   - the file id to be written to which was
*      	       	       	  opened earlier with kopen().
*		   data - the array in which the data will be written from.
*		   num  - the number of data points to be written to the
*			  transport.
*      	       	   type - data / structure type.
*	   Output: 
*         Returns: TRUE (1) on success, FALSE (0) otherwise
*      Written By: Mark Young
*            Date: Aug 15, 1995
*************************************************************/
int kwrite_pointer(
   int   id,
   kaddr *data,
   int   type)
{
        /* Get the file entry associated with the id */
        if (!saveaddrs && kfile_checkid(id) == NULL)
        {
           errno = EBADF;
           return(-1);
        }

	if (data == NULL)
	   return(-1);

	if (*data == NULL)
	{
	   if (numptrs > 0 && ptrs[numptrs-1] == NULL)
	   {
	      types[numptrs-1]++;
	      return(TRUE);
	   }
	   else
	      data = NULL, type = 1;
	}
	if (numptrs == maxptrs)
	{
	   maxptrs = (maxptrs != 0) ? maxptrs << 1 : KLENGTH;
	   types = (int *) krealloc(types, maxptrs * sizeof(int));
	   ptrs  = (kaddr **) krealloc(ptrs, maxptrs * sizeof(kaddr *));
	}
	types[numptrs] = type;
	ptrs[numptrs++] = data;

	if (saveaddrs == FALSE)
	{
	   int call_cleanup = FALSE;

	   if (addrs == NULL)
	   {
	      call_cleanup = TRUE;
	      addrs = khash_init(KSTRUCT, (size_t) 1013);
	   }

	   saveaddrs = TRUE;
	   if (write_pointers(id) == FALSE)
	      return(FALSE);

	   /* cleanup our mess */
	   if (call_cleanup) cleanup_handler();
	}
	return(TRUE);
}

/************************************************************
*  
*    Routine Name: kwrite_struct - write a single structure
*         Purpose: This module is used to write a single structure
*      	       	   to a transport.
*           Input: id   - the file id to write to which was
*      	       	       	  opened earlier with kopen().
*      	       	   type - structure type.
*	   Output: data - a pointer to a single structure to be written.
*         Returns: the number of structs written
*      Written By: Mark Young
*            Date: Aug 04, 1995
*
*************************************************************/
ssize_t kwrite_struct(
   int id,
   kaddr data,
   int type)
{
	ssize_t snum;

	snum = kwrite_generic(id, data, (size_t) 1, type);
	return(snum);
}
