/*
 * Khoros: $Id: program.c,v 1.15.2.4 1997/04/21 21:25:48 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: program.c,v 1.15.2.4 1997/04/21 21:25:48 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Program Utility Routines
   >>>>		
   >>>>   Public:
   >>>>                 kprog_set_argc()
   >>>>                 kprog_set_argv()
   >>>>                 kprog_set_envp()
   >>>>                 kprog_set_program()
   >>>>                 kprog_set_toolbox()
   >>>>                 kprog_get_argc()
   >>>>                 kprog_get_argv()
   >>>>                 kprog_get_envp()
   >>>>                 kprog_get_program()
   >>>>                 kprog_get_toolbox()
   >>>>                 kprog_get_command()
   >>>>                 kkhoros_initialize()
   >>>>                 khoros_init()
   >>>>   Static:
   >>>>                 cleanup_handler()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


static char  program_name[KLENGTH]    = "Unknown_Program_Name";
static char  program_toolbox[KLENGTH] = "BOOTSTRAP";
static char  *program_command = NULL;

static size_t   program_argc   = 0;
static char   **program_argv = NULL;
#if defined(KOPSYS_IRIX)
extern char   **_environ;
#define kenviron _environ
#else
extern char   **environ;
#define kenviron environ
#endif

static void cleanup_handler(kexit_status status, kaddr client_data)
{
   kfree_and_NULL(program_command);
}

/************************************************************
*
*  Routine Name: kprog_set_argc - set the number of commandline parameters
*       Purpose: Sets the number of arguments in the program argv structure.
*	  Input: argc - The current list in which we will be adding the
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*    Written By: Mark Young
*          Date: Sep 26, 1993
*
*************************************************************/

int kprog_set_argc(
   int argc)
{
	if (argc < 0)
	   return(FALSE);

	program_argc = (size_t) argc;
	return(TRUE);
}

/************************************************************
*
*  Routine Name: kprog_set_argv - set the command line argument array
*
*       Purpose: Set the argv list that the program uses.
*	  Input: argv - The array to set the program argv list to.
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*    Written By: Mark Young
*          Date: Sep 26, 1993
*************************************************************/

int kprog_set_argv(
   char *argv[])
{
	if (argv == NULL)
	   return(FALSE);

	program_argv = argv;
	return(TRUE);
}

/************************************************************
*
*  Routine Name: kprog_set_envp - set the number of environment variable
*				  parameters
*       Purpose: Sets the number of environment variable parameters structure
*		 from the program envp structure.
*	  Input: argc - The current list in which we will be adding the
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*    Written By: Mark Young
*          Date: Sep 26, 1993
*************************************************************/

int kprog_set_envp(
   char *envp[])
{
	if (envp == NULL)
	   return(FALSE);

	kenviron = (char **) envp;
	return(TRUE);
}

/************************************************************
*
*  Routine Name: kprog_set_program - set the name of the program
*
*       Purpose: Sets the name of the program.  This is usually the
*		 basename of the first argument in the argv structure.
*
*	  Input: program - The current name of this program
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*    Written By: Mark Young & Steve Kubica
*          Date: Jun 23, 1994
*************************************************************/
int kprog_set_program(
   const char *program)
{
	char *arguments;

	if (program == NULL)
	   return(FALSE);

	kbasename(program, program_name);
	if (program_argv != NULL && program_argc > 1)
	{
	   arguments = karray_to_string(&program_argv[1], KSTRING,
				(size_t) (program_argc-1), " ");
	   program_command = kstring_3cat(program_name, " ", arguments, NULL);
	   kfree_and_NULL(arguments);
	}
	else
	   program_command = kstrdup(program_name);

	return(TRUE);
}

/************************************************************
*
*  Routine Name: kprog_set_toolbox - set the toolbox in which this
*				     software object belongs.
*       Purpose: Sets the name of the toolbox in where the software
*		 belongs.
*
*	  Input: toolbox - toolbox name in which the program exists
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*    Written By: Mark Young
*          Date: Sep 26, 1993
*  Side Effects: the toolbox name is also placed into the environment
*		 variable list so that "$TOOLBOX" will be defined.
*************************************************************/

int kprog_set_toolbox(
   const char *toolbox)
{
	char temp[KLENGTH], *name;


	if (toolbox == NULL)
	   return(FALSE);

	/*
	 *  Stick the program toolbox in to the environment variable list
	 *  so that "$TOOLBOX" is defined.
	 */
	name = kstring_upper(toolbox, NULL);
	(void) ksprintf(temp,"TOOLBOX=$%s", name);
	kfree(name); kputenv(temp);
	kstrcpy(program_toolbox, toolbox);
	return(TRUE);
}

/************************************************************
*
*  Routine Name: kprog_get_argc - get the number of arguments
*				  in the argv structure
*
*       Purpose: Gets the number of arguments in the program argv
*		 structure.
*	  Input: argc - The current list in which we will be adding the
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*    Written By: Mark Young
*          Date: Sep 26, 1993
*************************************************************/

int kprog_get_argc(void)
{
	return(program_argc);
}

/************************************************************
*
*  Routine Name: kprog_get_argv - get the arguments in the argv structure
*       Purpose: returns the array of commandline arguments.
*	  Input: argc - The current list in which we will be adding the
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*  Restrictions: This routine returns a pointer to the internal argv array.
*		 Do not change or free any data in this array, use for
*		 read only purposes only.
*    Written By: Mark Young
*          Date: Sep 26, 1993
*************************************************************/

char **kprog_get_argv(void)
{
	return(program_argv);
}

/************************************************************
*
*  Routine Name: kprog_get_envp - gets the environment variable parameter
*				  structure
*       Purpose: Gets the environment variable parameters structure, which
*		 is the envp.
*       Returns: returns the envp structure or NULL upon failure
*  Restrictions: This routine returns a pointer to the internal environment
*		 array pointer.  Do not change or free any data in this array,
*		 Use as a read only resource only.
*    Written By: Mark Young
*          Date: Sep 26, 1993
*************************************************************/

char **kprog_get_envp(void)
{
	return((char **) kenviron);
}

/************************************************************
*
*  Routine Name: kprog_get_program - gets the name of the program
*
*       Purpose: Gets the name of the program.  This is usually the
*		 basename of the first argument in the argv structure.
*
*	  Input: program - The current name of this program.
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: This routine returns a pointer to the private string
*		 used to store the program variable.  Use this as
*		 a read only resource; do NOT change or free this pointer.
*    Written By: Mark Young
*          Date: Sep 26, 1993
*************************************************************/

char *kprog_get_program(void)
{
	return(program_name);
}

/************************************************************
*
*  Routine Name: kprog_get_toolbox - gets the toolbox in which this
*				     program belongs.
*
*       Purpose: Gets the name of the toolbox in which this program
*		 belongs.
*       Returns: the current toolbox or NULL if not initialized
*  Restrictions: This routine returns a pointer to the private string
*		 used to store the toolbox variable.  Use this as
*		 a read only resource; do NOT change or free this pointer.
*    Written By: Mark Young
*          Date: Sep 26, 1993
*
*************************************************************/

char *kprog_get_toolbox(void)
{
	return(program_toolbox);
}

/************************************************************
*
*  Routine Name: kprog_get_command - gets the command string in which this
*				     program was executed with.
*       Purpose: Gets the command used to originally excute the current program.
*  Restrictions: This routine returns a pointer to the private string
*		 used to store the toolbox variable.  Use this as
*		 a read only resource; do NOT change or free this pointer.
*    Written By: Mark Young & Steve Kubica
*          Date: Jun 23, 1994
*************************************************************/

char *kprog_get_command(void)
{
	return(program_command);
}

/************************************************************
*
*  Routine Name: khoros_initialize - initialize khoros system
*                              (old version for Khoros 2.1p1)
*
*       Purpose: Initializes some khoros global variables used in the
*		 khoros system.  Old version for Khoros 2.1p1 and earlier,
*                this routine will be obsolete in the next release; it is
*		 provided here for backwards compatibility only.
*		 Initializes the following global variables:
*
*         Input: argc    - the number of arguments on the command line
*		 argv    - the list of command line arguments
*		 toolbox - the name of the toolbox the program is 
*		           associated with. 
*    Written By: Mark Young
*          Date: Sep 03, 1994
*
*************************************************************/

void khoros_initialize(
   int  argc,
   char **argv,
   char *toolbox)
{
	kfile *ipc;
	char temp[KLENGTH];

	kutils_init_errors();
	kprog_set_argc(argc);
	kprog_set_argv(argv);
	kexit_handler(cleanup_handler, NULL);

        if (argv != NULL)
	   kprog_set_program(argv[0]);

	kprog_set_toolbox(toolbox);

	/*
	 * Ignore sigpipe which will cause a write or read failure on a
	 * pipe rather than causing the application to abort.
	 */
	ksignal(SIGPIPE, SIG_IGN);
	ksignal(SIGTERM, SIG_DFL);

        if ((ipc = kipc_getipc()) != NULL)
        {
           ksprintf(temp,"%s:%d: initialize process\n", KIPC_PROCESS, getpid());
           kfputs(temp, ipc);
           kfgets(temp, KLENGTH, kstdin);
        }
}

/************************************************************
*
*  Routine Name: khoros_init - initialize khoros system
*                              (new version for Khoros 2.1p2)
*
*       Purpose: Initializes some khoros global variables used in the
*                khoros system.  Initializes the following
*                global variables:
*
*         Input: argc    - the number of arguments on the command line
*                argv    - the list of command line arguments
*                toolbox - the name of the toolbox the program is
*                          associated with.
*		 date    - the product release date
*		 product - the product release name
*		 version - the product version number
*		 major   - the product major release number
*		 minor   - the product minor release number
*		 path    - the path to the application home directory
*
*    Written By: Mark Young, Steve Jorgensen, Danielle Argiro
*          Date: Nov 21, 1996
*
*************************************************************/

void khoros_init(
   int  argc,
   char **argv,
   char *toolbox,
   char *date,
   char *product,
   int  version,
   int  major,
   int  minor,
   char *path)
{
	char temp[KLENGTH];
        char fullpath[KLENGTH];
	char envvar[KLENGTH];

	khoros_initialize(argc, argv, toolbox);

	if (version != PRODUCT_RELEASE_VERSION ||
	    major   != PRODUCT_RELEASE_MAJOR   ||
	    minor   != PRODUCT_RELEASE_MINOR   ||
	    kstrcmp(product, PRODUCT_RELEASE_NAME) != 0)
	{
	   kwarn("kutils", "khoros_init", "\nRuntime Product Mismatch: "
		 "The program '%s' was developed against:\n\n\t%s %d.%d.%d\n\n"
		 "The current system is currently running:\n\n\t%s %d.%d.%d\n\n"
		 "This may cause this application to behave incorrectly!\n\n",
		 kprog_get_program(), product, version, major, minor,
		 PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
		 PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR);
	}

	/*-- need a kputenv defaults for $APPLICATION_HOME and $<APP>_HOME --*/
	kfullpath(path, NULL, fullpath);
	kstring_upper(kprog_get_program(), temp);
	kstring_cat(temp, "_HOME", envvar);
        if (kgetenv(envvar) == NULL)
        {
	   kstring_3cat(envvar, "=", fullpath, temp);
	   kputenv(temp);
	}
	kstring_cat("APPLICATION_HOME=", fullpath, temp);
	kputenv(temp);
}
