/*
 * Khoros: $Id: strings.c,v 1.16.2.4 1997/07/01 00:45:59 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: strings.c,v 1.16.2.4 1997/07/01 00:45:59 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		string routines
   >>>>		
   >>>>   Public:
   >>>>                 kstrstr()
   >>>>                 kstrpbrk()
   >>>>                 kstrtok()
   >>>>                 kstrdup()
   >>>>                 kstrspn()
   >>>>                 kstrcspn()
   >>>>                 kstrcasecmp()
   >>>>                 kstrncasecmp()
   >>>>                 kstrlen()
   >>>>                 kstrcmp()
   >>>>                 kstrncmp()
   >>>>                 kstrchr()
   >>>>                 kstrrchr()
   >>>>                 kstrcpy()
   >>>>                 kstrncpy()
   >>>>                 kstrcat()
   >>>>                 kstrncat()
   >>>>                 kstring_lower()
   >>>>                 kstring_upper()
   >>>>                 kstring_capitalize()
   >>>>                 kstring_replace()
   >>>>                 kstring_seddata()
   >>>>                 kchar_replace()
   >>>>                 kstring_cleanup()
   >>>>                 kstring_format()
   >>>>                 kstring_cat()
   >>>>                 kstring_ncat()
   >>>>                 kstring_3cat()
   >>>>                 kstring_subcmp()
   >>>>                 kstring_copy()
   >>>>                 kstring_ncopy()
   >>>>                 kstring_detab()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	


#define NEEDS_STRDUP
#define NEEDS_STRCASECMP
#define NEEDS_STRNCASECMP

#define nullstring(str)	((str) ? str : "")

/************************************************************
*
*  Routine Name: kstrstr - find a substring within a string
*       Purpose: This function locates the first occurrence of one string
*		 in another.  It is a replacement for the system
*		 library routine strstr().  This routine scans the
*		 input string, 'istr', for the first occurrence
*		 of a substring specified by 'sub_str'.
*         Input: istr    - The string to search.
*		 sub_str - The sub string to look for.
*       Returns: If the sub_str is found in istr, a pointer indicating the
*		 address of the first character of the substring within the
*		 istr string is returned.  If the substring is not
*		 a part of the input string, a NULL is returned.
*
*		 This routine also protects against NULL on both the input and
*		 substring variables.  If either is NULL, the value of NULL is
*		 returned. 
*    Written By: Hideo Noro & Mark Young & Tom Sauer
*          Date: Jul 03, 1992
*************************************************************/

char *kstrstr(
   const char *istr,
   const char *sub_str)
{
#ifdef NEEDS_STRSTR
	char *string;
	int  len;
#endif

	if (istr == NULL || sub_str == NULL)
	{
	   if (istr == NULL)
	      kinfo(KSYSLIB,"kstrstr: NULL input string encountered" );

	   if (sub_str == NULL)
	      kinfo(KSYSLIB,"kstrstr: NULL sub-string encountered" );

	   return(NULL);
	}

	/*
	 *  March thru the string using kstrncmp() to find the sub-string
	 */
#ifdef NEEDS_STRSTR

	len = kstrlen(sub_str);
	string = istr;
	while (*string != '\0')
	{
	   if (kstrncmp(string, sub_str, len) == 0)
	      return(string);
	   else
	      string++;
	}
	return(NULL);
#else
	return(strstr(istr, sub_str));
#endif
}

/************************************************************
*
*  Routine Name: kstrpbrk - find the first occurrence of a character
*			    from a set of chararacters
*       Purpose: This function locates the position of the first occurrence
*		 in the input string of any character from a specified set
*		 of characters.  This routine is a replacement for the system
*		 library routine strpbrk() with some extra protection against
*		 NULL input strings.  It searches the input string, 'istr',
*		 for the first occurrence of any character in a set of
*		 characters, specified by the 'sub_str' string.
*         Input: istr    - The string to search for a character.
*		 sub_str - The string holding the set of characters to
*		 	   look for.
*       Returns: If one of the characters in 'sub_str' is found in 'istr', a
*		 pointer indicating the address of the character it found
*		 in 'istr' is returned.  If no characters in the 'sub_str'
*		 string appear in 'istr', a NULL is returned.
*
*                This routine also protects against NULL on both the input and
*                substring variables.  If either is NULL, the value of NULL is
*                returned.
*    Written By: Mark Young & Steven Jorgensen
*          Date: Jul 03, 1992
*************************************************************/

char *kstrpbrk(
   const char *istr,
   const char *sub_str)
{
#ifdef NEEDS_STRPBRK
	char    *string;
#endif

	if (istr == NULL || sub_str == NULL)
	{
	   if (istr == NULL)
	      kinfo(KSYSLIB,"kstrpbrk: NULL input string encountered" );

	   if (sub_str == NULL)
	      kinfo(KSYSLIB,"kstrpbrk: NULL token string encountered" );

	   return(NULL);
	}

	/*
	 *  March thru the string using kstrchr() to find the sub-string
	 */
#ifdef NEEDS_STRPBRK
	string = istr;
	while (*string != '\0')
	{
	   if (kstrchr(sub_str, *string))
	      return(string);

	   string++;
	}
	return(NULL);
#else
	return(strpbrk(istr, sub_str));
#endif
}

/************************************************************
*
*  Routine Name: kstrtok - find a token within a string
*       Purpose: This function gets the next token, or substring, from the
*		 input string using a set of characters as delimiters.
*		 This routine is a replacement for the system
*		 library routine strtok().  kstrtok() searches
*		 the input string, 'istr', for a token separator, which is
*		 specified in the second string parameter, 'sub_str'.
*		 Then, it returns a pointer to the remaining portion of
*		 the input string after inserting a '\\0' at the token
*		 separator.  If it is called again with a NULL as the input
*		 string, the routine continues to parse the previous string
*		 passed in.
*         Input: istr    - The string to search find tokens in.
*		 sub_str - The string containing the set of character tokens.
*       Returns: A pointer to the first token in 'istr'.
*		 Otherwise, it returns NULL on error, or if the
*		 token string 'sub_str' is NULL.
*    Written By: Mark Young & Steven Jorgensen
*          Date: Jul 03, 1992
*  Side Effects: This routine adds '\\0's to the original string.  It also
*		 has no way to check if the input string has been freed via a
*		 kfree_and_NULL() call between subsequent calls to kstrtok().
*************************************************************/

char *kstrtok(
   char       *istr,
   const char *sub_str)
{
	static char *string = NULL;

	if (istr == NULL && string == NULL)
	   return(NULL);
	else if (istr != NULL)
	   string = istr;

#ifdef NEEDS_STRTOK
	if (istr == NULL)
	   istr = string;
	if ((string = kstrpbrk(string, sub_str)) != NULL)
	{
	   *string++ = '\0';
	   while (*string != '\0' && kstrchr(sub_str, *string))
	      string++;
	}
	return(istr);
#else
	return(strtok(istr, sub_str));
#endif
}


/************************************************************
*
*  Routine Name: kstrdup - return a duplicate of the input string
*       Purpose: This function allocates memory and copies the input string
*		 into that space. This routine is similiar to the system
*		 routine strdup.  It uses kdupalloc() to create
*		 a memory buffer large enough to hold the string passed
*		 into this routine and copyies the input string to the
*		 buffer.  This buffer is then returned to the user. 
*         Input: istr - The string to be duplicated.
*       Returns: A pointer to the duplicated string.  This routine will
*		 return NULL on a memory allocation error, or if the input
*		 string is NULL.
*      Examples: This example initialized the a pointer to be a
*		 memory buffer with a string in it.  This buffer can then be
*		 modified by the rest of the routine.  Since memory
*		 is allocated, a call to kfree_and_NULL() will free the buffer when the
*		 buffer is no longer needed.
*		 .begin code
*		 char *str = kstrdup("this sentence needs a capital t at the start.");
*		 if (str == NULL)
*		 {
*		 !  kerror(NULL, NULL, "Cannot malloc space for the string");
*		 !  return;
*		 }
*		 str[0] = 'T';
*		 kprintf("%s\n", str);
*		 .end code
*		 The resulting print will be 'This sentence needs a capital t at the start.'
*    Written By: Mark Young
*          Date: Nov 28, 1992
*************************************************************/

char *kstrdup(
   const char *istr)
{
#ifdef NEEDS_STRDUP
	return((istr == NULL) ? NULL : (char *)kdupalloc((kaddr) istr,strlen(istr)+1));
#else
	return((istr == NULL) ? NULL : (char *) strdup(istr));
#endif
}


/************************************************************
*
*  Routine Name: kstrspn - return the number of matched characters
*       Purpose: This function returns the length of the input string that
*		 consists completely of a specified list of characters.
*		 This routine is similiar to the system routine strspn().
*		 It counts the number of characters at the start of the
*		 input string that consist entirely of characters from
*		 the 'charset' string.  The count stops at the first
*		 character in the input string that is not in the charset
*		 string.
*         Input: istr    - The string to be scanned.
*		 charset - The string used to determin the character set
*		 	   to use.
*       Returns: The number of characters matched in the input string,
*		 from the 'charset'.  If every character in the input string
*		 is a member of set of characters defined by 'charset', the
*		 length of 'istr' is returned.  If either 'istr'
*		 or 'charset' are NULL, a value of 0 is returned.
*    Written By: Mark Young
*          Date: Nov 28, 1992
*************************************************************/

size_t kstrspn(
   const char *istr,
   const char *charset)
{
#ifdef NEEDS_STRSPN
	register char *str = istr;

	/*
	 *  Sanity check...
	 */
	if (istr == NULL || charset == NULL)
	   return(0);

	while (*str != '\0')
	{
	   if (!kstrchr(charset, *str))
	      break;
	   else str++;
	}
	return((size_t)(str - istr));
#else
	return(!istr || !charset ? 0 : strspn(istr, charset));
#endif
}


/************************************************************
*
*  Routine Name: kstrcspn - return the number of characters not matched
*       Purpose: This function returns the length of a portion of the
*		 input string not matching any of the characters in
*		 specifiec set of characters.  It is similiar
*		 to the system routine strcspn().
*
*		 It counts the number of characters at the start of
*		 the input string that consist entirely of characters not in
*                list of characters specified by 'charset'.  The
*		 count stops at the first character in 'string' that
*		 is in 'charset'.
*         Input: string - The string to be checked.
*		 charset - The set of characters to use for counting.
*       Returns: The number of characters not matched in the input string,
*		 from the 'charset', or 0 if the input string or character
*		 set string are NULL.
*      Examples: The following example uses kstrcspn() to determine
*		 the number of non-vowels that appear before the first
*		 vowel.
*		 .begin code
*		 char *str = "    This is my input string";
*		 int non_vowels = 0;
*
*		 non_vowels = kstrcspn(str, "aeiou");
*		 kprintf("The number of non_vowels = %d\n", non_vowels);
*		 .end code
*		 The result, stored in 'non_vowels', will be 6.
*    Written By: Mark Young
*          Date: Nov 28, 1992
*************************************************************/

size_t kstrcspn(
   const char *istr,
   const char *charset)
{
#ifdef NEEDS_STRCSPN
	register char *str = istr;

	/*
	 *  Sanity check...
	 */
	if (istr == NULL || charset == NULL)
	   return(0);

	while (*str != '\0')
	{
	   if (kstrchr(charset, *str))
	      break;
	   else str++;
	}
	return((size_t)(str - istr));
#else
	return(!istr || !charset ? 0 : strcspn(istr, charset));
#endif
}


/************************************************************
*
*  Routine Name: kstrcasecmp - do a case insensitive string compare
*       Purpose: This function does a case insensitive comparison of two
*		 strings.  It does a character by character comparison
*		 of both input strings, ignoring the case of alphabetic
*		 characters, until the current character for one string is
*		 not equal the current character in the other string.
*		 This continues until the end of one or both of the strings
*		 is reached.  It protects against NULL on the input
*		 strings by replacing NULL pointers with a reference to an
*		 empty string.
*         Input: istr1 - The first input string to compare.
*                istr2 - The second input string to compare.
*       RETURNS: If the two strings are identical, ignoring case differences,
*		 the value of 0 is returned.  If the two strings differ, the
*		 difference of ASCII values between the first character in
*		 each string that differs will be returned.  If the ASCII
*		 value of the differing character in 'istr1' is greater than
*		 the one in 'istr2', then the return value is positive, and
*		 implies that 'istr1' is greater than 'istr2'.  If the
*		 difference between the two ASCII values is negative, the
*		 return value implies that 'istr2' is greater than 'istr1'.
*      Examples: The following example takes two strings that are equal if you
*		 ignore case, and passes them to string compare.
*		 .begin code
*		 char *s1 = "John Doe";
*		 char *s2 = "john DOE";
*		 int   result;
*
*		 result = kstrcasecmp(s1, s2);
*		 kprintf("The result is '%d'\n", result);
*		 .end code
*		 The output that will be printed is 0.
*    Written By: Mark Young and Steven Jorgensen
*          Date: Nov 28, 1992
*************************************************************/

int kstrcasecmp(
   const char *istr1,
   const char *istr2)
{
	istr1 = nullstring(istr1); 
	istr2 = nullstring(istr2); 

#ifdef NEEDS_STRCASECMP
	while (tolower(*istr1) == tolower(*istr2))
	{
	   if (*istr1++ == '\0') return(0);
	   istr2++;
	}
	return(tolower(*istr1) - tolower(*istr2));
#else
	return(strcasecmp(istr1, istr2));
#endif
}


/************************************************************
*
*  Routine Name: kstrncasecmp - do a case insensitive string compare on n
*				characters
*       Purpose: This routine does a case insensitive comparison of two
*		 strings for a specified number of bytes.  This routine is
*		 a replacement for the system strncasecmp().  It does a
*		 character by character case insensitive comparison of
*		 both input strings until the current character for one
*		 string does not equal the current character of the
*		 other string, until the end of the input strings are
*		 reached, or until num characters have been compared.  It
*		 protects against NULL on the input strings by replacing NULL
*		 pointers with a reference to an empty string.
*         Input: istr1 - The first string to compare.
*                istr2 - The second string to compare.
*		 num   - The number of bytes to compare.
*       Returns: If the two strings are identical up to the specified byte,
*		 or end of string the value of 0 is returned.  If the two
*		 strings differ, the ASCII value difference of the first
*		 character that differs in the two strings will be returned.
*		 If the ASCII value of the differing character in 'istr1' is
*		 greater than the one in 'istr2', then the return value is
*		 positive, and implies that 'istr1' is greater than 'istr2'.
*		 If the difference between the two ASCII values is negative,
*		 the return value implies that 'istr2' is greater than 'istr1'.
*      Examples: This example uses strcasecmp twice on the same strings
*		 and different 'num' values.
*		 .begin code
*		 char *s1 = "SAME-DIFFERENT";
*		 char *s2 = "Same-Really Different";
*		 int same, different;
*
*		 same = kstrncasecmp(s1, s2, 5);
*		 different = kstrncasecmp(s1, s2, 10);
*		 printf("same is %d\ndifferent is %d\n", same, different);
*		 .end code
*		 the resulting output will be:
*
*		 ! same is 0
*		 ! different is -14
*
*		 because up to the fifth character they are the same, and
*		 on the sixth character the returned result will
*		 be 'D' - 'R'.
*    Written By: Mark Young and Steven Jorgensen
*          Date: Nov 28, 1992
*************************************************************/

int kstrncasecmp(
   const char *istr1,
   const char *istr2,
   size_t      num)
{
	istr1 = nullstring(istr1); 
	istr2 = nullstring(istr2); 

	if (num <= 0)
	   return(0);
#ifdef NEEDS_STRNCASECMP
	while (--num > 0 && tolower(*istr1) == tolower(*istr2))
	{
	   if (*istr1++ == '\0') return(0);
	   istr2++;
	}
	return(tolower(*istr1) - tolower(*istr2));
#else
	return(strncasecmp(istr1, istr2, num));
#endif
}


/************************************************************
*
*  Routine Name: kstrlen - return the length of a string
*       Purpose: This routine finds the length of a string.
*		 This routine is similiar to the system routine
*		 strlen(). It counts the number of characters in a
*		 string until it reaches a null ('\\0') character.
*		 This routine is better than many standard unix
*		 strlen() routines, because it treats a NULL pointer
*		 as an empty string, which returns a length of zero.
*         Input: istr - The string to get the length of.
*       Returns: The number of characters in the string.  On
*		 a NULL input string, a value of 0 is returned.
*      Examples: This is a very simple example that prints the
*		 length of the input string.
*		 .begin code
*		 char *str = "1234";
*		 kprintf("The length is '%d'\n", kstrlen(str));
*		 .end code
*		 The resulting length is 4, because the null terminating
*		 character is not counted.
*    Written By: Mark Young
*          Date: Nov 28, 1992
*************************************************************/

size_t kstrlen(
   const char *istr)
{
#ifdef NEEDS_STRLEN
	register int i = 0;

	/*
	 *  Sanity check...
	 */
	if (!istr)
	   return(0);

	while (*istr++ != '\0') i++;
	   return(i);
#else
	return(!istr ? 0 : strlen(istr));
#endif
}

/************************************************************
*
*  Routine Name: kstrcmp - compare two strings
*       Purpose: This function does a comparison of two strings. It is
*		 a replacement for the system strcmp().  It does a
*		 character by character case sensitive comparison of both
*		 input strings until the current character for one string
*		 does not equal the current character of the other string,
*		 or until the end of the input strings are reached.  It
*		 protects against NULL on the input strings by replacing
*		 NULL pointers with a reference to an empty string.
*         Input: istr1 - A character pointer to the first string.
*		 istr2 - A character pointer to the second string.
*       Returns: If the two strings are identical, the value of 0 is returned.
*                If the two strings differ, the ASCII value difference of
*                the first character that differs in the two strings will be
*                returned.  If the ASCII value of the differing character in
*                istr1 is greater than the one in istr2, then the return
*                value is positive, and implies that istr1 is greater than
*                istr2.  If the difference between the two ASCII values is
*                negative, the return value implies that istr2 is greater than
*                istr1.
*      Examples: The following example repeats the kstrcasecmp() example.
*		 .begin code
*		 char *s1 = "John Doe";
*		 char *s2 = "john DOE";
*		 int   result;
*
*		 result = kstrcmp(s1, s2);
*		 kprintf("The result is '%d'\n", result);
*		 .end code
*		 The result of this compare is -32 instead of 0,
*		 since that is the result of the difference 'J' - 'j'.
*    Written By: Mark Young
*          Date: Nov 28, 1992
*************************************************************/

int kstrcmp(
   const char *istr1,
   const char *istr2)
{
#ifdef NEEDS_STRCMP
	istr1 = nullstring(istr1); 
	istr2 = nullstring(istr2); 

	while (*istr1 == *istr2)
	{
	   if (*istr1++ == '\0') return(0);
	   istr2++;
	}
	return(*istr1 - *istr2);
#else
	return(istr1 && istr2 ? strcmp(istr1,istr2) : 
		strcmp(nullstring(istr1), nullstring(istr2)));
#endif
}

/************************************************************
*
*  Routine Name: kstrncmp - compare the first n characters of two strings
*       Purpose: This routine is a replacement for the system
*                strcasecmp.  It does a character by character
*                case sensitive comparison of both input strings until
*                the current character for one string does not equal the
*                current character of the other string, until the end of the
*                input strings are reached, or until num characters have been
*                compared.  It protects against NULL on the input strings
*                by replacing NULL pointers with a reference to an empty
*                string.
*         Input: istr1 - A character pointer to the first string.
*		 istr2 - A character pointer to the second string.
*		 num   - The number of characters to compare.
*       Returns: If the two strings are identical or 'num' is less than or
*		 equal to zero, the value of 0 is returned.  If the two
*		 strings differ, the ASCII value difference of the first
*		 character that differs in the two strings will be returned.
*		 If the ASCII value of the differing character in 'istr1' is
*		 greater than the one in 'istr2', then the return value is
*		 positive, and implies that 'istr1' is greater than 'istr2'.
*		 If the difference between the two ASCII values is negative,
*		 the return value implies that 'istr2' is greater than 'istr1'.
*      Examples: This example is similar to the kstrncasecmp() example.
*		 It calls kstrncmp twice on the same two strings with
*		 different values of 'num'.
*		 .begin code
*		 char *s1 = "Same-DIFFERENT";
*		 char *s2 = "Same-different";
*
*		 kprintf("Same is %d\n", kstrncmp(s1, s2, 5));
*		 kprintf("Different is %d\n", kstrncmp(s1, s2, 10));
*		 .end code
*		 The output will be:
*
*		 ! Same is 0
*		 ! Different is -32
*    Written By: Mark Young
*          Date: Nov 28, 1992
*************************************************************/

int kstrncmp(
   const char *istr1,
   const char *istr2,
   size_t      num)
{
	istr1 = nullstring(istr1); 
	istr2 = nullstring(istr2); 

	if (num <= 0)
	   return(0);
#ifdef NEEDS_STRNCMP
	while (--num > 0 && *istr1 == *istr2)
	{
	   if (*istr1++ == '\0') return(0);
	   istr2++;
	}
	return(*istr1 - *istr2);
#else
	return(strncmp(istr1, istr2, (size_t) num));
#endif
}

/************************************************************
*
*  Routine Name: kstrchr - find a character in a string
*       Purpose: This function finds the first occurence of a specified
*		 character in a given string.  This routine is similar
*		 to the system strchr() call.  kstrchr() searches the input
*		 string, 'istr', for the first occurrence of a character,
		 'character'.  The terminating null character is included in
		 the search, and can be the specified character to search for.
*         Input: istr - The string to scan.
*		 character  - The character to look for in 'istr'.
*       Returns: A pointer to the address in the input string where
*		 the 'character' character is located.  NULL is returned
*		 if 'character' is not in the input string, or if the input
*		 string is NULL.
*      Examples: The following example takes a sentence, and uses
*		 kstrchr() to shorten it for use in a different sentence.
*		 .begin code
*		 char *s1 = "Hello World.";
*		 char *result;
*
*		 result = kstrchr(s1, 'W');
*		 kprintf("Goodbye Cruel %s\n", result);
*		 .end code
*		 The output string is 'Goodbye Cruel World.'
*    Written By: Mark Young
*          Date: Nov 28, 1992
*************************************************************/

char *kstrchr(
   const char *istr,
   int         character)
{
	/*
	 *  Sanity check...
	 */
	if (!istr)
	   return(NULL);

#ifdef NEEDS_STRCHR
	while (*istr != character && *istr != '\0')
	   istr++;

	return(*istr == character ? istr : NULL);
#else
	return(strchr(istr, character));
#endif
}

/************************************************************
*
*  Routine Name: kstrrchr - reverse scan a string to find a character
*       Purpose: This function finds the last occurrence of a specified
*		 character in a string and it is similar to the system
*		 call strrchr().  kstsrrchr() searches the input string
*		 from back to front for a character specified by 'character' in
*		 the input string.
*         Input: istr - The input string to scan.
*		 character  - The character to search for.
*       Returns: A pointer to the address in the input string, 'istr', where
*		 the last occurrence of the 'character' character is.  NULL is
*		 returned if 'character' is not in the input string, or if a
*		 NULL input string is provided.
*    Written By: Mark Young
*          Date: Nov 28, 1992
*************************************************************/

char *kstrrchr(
   const char *istr,
   int  character)
{
#ifdef NEEDS_STRRCHR
	register char *strend;
#endif

	/*
	 *  Sanity check...
	 */
	if (istr == NULL)
	   return(NULL);

#ifdef NEEDS_STRRCHR
	strend = istr + kstrlen(istr);
	while (*strend != character && strend != istr)
	   strend--;

	return(*strend == character ? strend : NULL);
#else
	return(strrchr(istr, character));
#endif
}


/************************************************************
*
*  Routine Name: kstrcpy - copy a string
*       Purpose: This function copies one string to another.  This
*		 function is similar to system call strcpy().
*		 kstrcpy() is used used to copy each character in the input
*		 string to the output string.  The terminating null character
*		 of the input string is copied so that the input and output
*		 strings are exact copies.  Thus, 'ostr' is overwritten.
*
*		 Note that the calling routine must ensure that 'ostr'
*		 points to a memory buffer large enough to hold the
*		 copied string and terminating null character.
*		 If the buffer is not large enough, memory will be
*		 overwritten resulting in unpredicable program failure.
*         Input: istr - The string to copy from.
*        Output: ostr - The string to copy into.
*       Returns: A pointer to the copied string 'ostr' after 'istr'
*		 is copied to 'ostr'.  If 'istr' is NULL, 'ostr' is
*		 returned without modification.  If 'ostr' is NULL,
*		 then NULL is returned.
*      Examples: The following example copies strings 's1' to 's2'.
*		 .begin code
*		 char  s2[KLENGTH];
*		 char *s1 = "Hello World.\n";
*		 char *result;
*
*		 result = kstrcpy(s2, s1);
*		 kprintf("%s", result);
*		 .end code
*		 The output is a single line saying 'Hello World', and
*		 the pointer 'result' is pointing to the 's2' array.
*    Written By: Mark Young
*          Date: Nov 28, 1992
*************************************************************/

char *kstrcpy(
   char       *ostr,
   const char *istr)
{
#ifdef NEEDS_STRCPY
	char *string = ostr;

	/*
	 *  Sanity check...
	 */
	if (ostr == NULL)
	   return(NULL);
	else if (istr == NULL)
	{
	   *ostr = '\0';
	   return(ostr);
	}

	/*EMPTY*/
	while ((*ostr++ = *istr++) != '\0');

	return(string);
#else
	return(!ostr ? NULL : (!istr ? *ostr = '\0', ostr: strcpy(ostr,istr)));
#endif
}

/************************************************************
*
*  Routine Name: kstrncpy - copy the first n characters in a string
*       Purpose: This function copies a specified number of characters
*		 from one string to another.  This function is similar to
*		 system call strncpy().  kstrncopy() copies up to num
*		 characters in the input string to the output string.
*
*		 Note that the calling routine must ensure that 'ostr'
*		 points to a memory buffer large enough to hold the
*		 copied string and terminating null character.
*		 If the buffer is not large enough, memory will be
*		 overwritten resulting in unpredicable program failure.
*         Input: istr - The string to copy from.
*		 num   - The number of characters to copy.
*        Output: ostr - The string to copy into.
*       Returns: A pointer to 'ostr' after up to num 'istr' is copied
*		 to 'ostr'.  If 'ostr' is NULL, then NULL is returned.
*		 If 'istr' is NULL or num is less than or equal to
*		 zero, then 'ostr' is returned unchanged.
*    Written By: Mark Young
*          Date: Nov 28, 1992
*************************************************************/

char *kstrncpy(
   char       *ostr,
   const char *istr,
   size_t      num)
{
#ifdef NEEDS_STRNCPY
	char *string = ostr;

	/*
	 *  Sanity check...
	 */
	if (ostr == NULL)
	   return(NULL);

	else if (istr != NULL)
	{
	   /*EMPTY*/
	   while (num-- > 0 && (*ostr++ = *istr++) != '\0');
	}
	while (num-- > 0) *ostr++ = '\0';
	return(string);
#else
        return(!ostr ? NULL :
		strncpy(ostr, !istr ? "" : istr, num));
#endif
}

/************************************************************
*
*  Routine Name: kstrcat - concatenate two strings
*       Purpose: This function concatenates one string to another.
*		 kstrcat appends all the characters from 'istr' to the
*		 end of 'ostr', terminating the resulting string with a null
*		 character.  
*
*		 Note that the calling routine must make sure that 'ostr'
*		 points to a memory buffer large enough to hold the
*		 concatenated string and terminating null character.
*		 If the buffer is not large enough, memory will be
*		 overwritten resulting in unpredicable program failure.
*         Input: ostr - The first string to concatenate.
*		 istr - The second string to concatenate.
*        Output: ostr - The resulting combined output string.
*       Returns: A pointer to 'ostr' after all the characters in 'istr' are
*		 appended onto 'ostr'. It will return NULL if 'ostr' is NULL.
*		 If 'istr' is NULL, 'ostr' is returned unchanged.
*      Examples: The following example contenates two strings 's1' and 's2',
*		 and to generate a well known sentence.
*		 .begin code
*		 char  s1[KLENGTH] = { 'H', 'e', 'l', 'l', 'o' };
*		 char *s2 = " World.\n";
*		 char *result;
*
*		 result = kstrcat(s1, s2);
*		 kprintf("%s", result);
*		 .end code
*		 The output is a single line saying 'Hello World', and
*		 the pointer 'result' is pointing to the 's1' array.
*    Written By: Mark Young
*          Date: Nov 28, 1992
*************************************************************/

char *kstrcat(
   char       *ostr,
   const char *istr)
{
#ifdef NEEDS_STRCAT
	char *string = ostr;

	/*
	 *  Sanity check...
	 */
	if (ostr == NULL)
	   return(NULL);
	else if (istr == NULL)
	{
	   *ostr = '\0';
	   return(ostr);
	}
	ostr += kstrlen(ostr);

	/*EMPTY*/
	while ((*ostr++ = *istr++) != '\0');
	return(string);
#else
	return(!ostr ? NULL : (!istr ? ostr: strcat(ostr,istr)));

#endif
}

/************************************************************
*
*  Routine Name: kstrncat - concatenate up to n characters on a string
*       Purpose: This function concatenates a specified number of characters
*		 one string to another.  This function is similar to
*		 system call strncat().  kstrncat() appends upto 'num'
*		 characters from 'istr' to the end of 'ostr'.
*
*		 Note that the calling routine must make sure that 'ostr'
*		 points to a memory buffer large enough to hold the
*		 concatenated string and terminating null character.
*		 If the buffer is not large enough, memory will be
*		 overwritten resulting in unpredicable program failure.
*         Input: ostr - The base string to concatenate.
*		 istr - The string to concatenate to the base.
*		 num  - The number of characters in 'istr' to concat
*			onto 'ostr'.  If the specified number is larger
*			than the length of istr, then this function stops
*			at the null character.  If 'num' is less than or
*			equal to zero, 'ostr' is left unchanged.
*        Output: ostr - The resulting combined output string.
*       Returns: A pointer to the concatenated string 'ostr' after 'num'
*		 characters of 'istr' are appended onto the end of 'ostr'.
*		 It will return NULL, if 'ostr' is NULL.  If 'istr' is
*		 NULL or num is less than or equal to zero, 'ostr' is
*		 returned unchanged.
*      Examples: This example partially combines two strings into a 
*		 familiar saying.
*		 .begin code
*		 char *s1 = "I'm sorry Dave. ";
*		 char *s2 = "I'm afraid I can't do that.  It is too complicated.";
*		 char result[KLENGTH];
*
*		 kstrcpy(result, s1);
*		 s1 = kstrncat(result, s2, 27);
*		 kprintf("%s\n", s1);
*		 .end code
*		 The output is 'I'm sorry Dave. I'm afraid I can't do that.'
*    Written By: Mark Young
*          Date: Nov 28, 1992
*************************************************************/

char *kstrncat(
   char       *ostr,
   const char *istr,
   size_t      num)
{
#ifdef NEEDS_STRNCAT
	char *string = ostr;

	/*
	 *  Sanity check...
	 */
	if (ostr == NULL)
	   return(NULL);
	else if (istr == NULL || num <= 0)
	{
	   *ostr = '\0';
	   return(ostr);
	}

	/*EMPTY*/
	while (*ostr++ != '\0');
	/*EMPTY*/
	while (num-- > 0 && (*ostr++ = *istr++) != '\0');
	return(string);
#else
        return(!ostr ? NULL : (!istr ? *ostr = '\0', ostr :
			strncat(ostr, istr, num)));
#endif
}


/************************************************************
*
*  Routine Name: kstring_lower - convert a string to lower case.
*       Purpose: This routine performs a character by character
*		 scan for uppercase characters.  When an uppercase
*		 character is found, it calls the function tolower()
*		 to get the lowercase equivalent, and replaces it.
*
*		 Note that if 'ostr' is non-NULL, the calling routine
*		 must ensure that 'ostr' points to a memory buffer
*		 large enough to hold the converted string plus a
*		 null character.  If the buffer is not large enough,
*		 memory will be overwritten resulting in unpredicable
*		 program failure.
*
*         Input: istr - The string to convert to lowercase.
*        Output: ostr - The string that holds the converted string.
*			If it's NULL, it allocates the necessary memory
*			with kmalloc().
*       Returns: The converted string, 'ostr' if that variable is not
*		 NULL, or a pointer to the resulting allocated string.
*		 NULL is returned on an allocation error or if 'istr' is NULL.
*    Written By: Mark Young
*          Date: Jul 13, 1993
*  Side Effects: Allocates the space for the output if ostr is NULL
*************************************************************/

char *kstring_lower(
   const char *istr,
   char       *ostr)
{
	char *string;

	if (!istr)
	{
	   kinfo(KSYSLIB,"kstring_lower: NULL input string encountered" );
	   return(NULL);
	}

	/*
	 *  If the ostr is NULL then kstrdup the input string
	 */
	if (!ostr)
	   ostr = kstrdup(istr);
	else if (istr != ostr)
	   kstrcpy(ostr, istr);

	string = ostr;
	while (*string != '\0')
	{
	   if (isupper(*string))
	      *string = tolower(*string);

	   string++;
	}
	return(ostr);
}

/************************************************************
*
*  Routine Name: kstring_upper - convert a string to upper case.
*       Purpose: This routine performs a character by character
*                scan for lowercase characters.  When an lowercase
*                character is found, it calls the function, toupper(),
*		 to get the uppercase equivalent, and replaces the lower
*		 case character.
*
*		 Note that if 'ostr' is non-NULL, the calling routine
*		 must ensure that 'ostr' points to a memory buffer
*		 large enough to hold the converted string plus a
*		 null character.  If the buffer is not large enough,
*		 memory will be overwritten resulting in unpredicable
*		 program failure.
*
*         Input: istr - The string to convert to upper case.
*        Output: ostr - The string that holds the converted string.
*			If variable is NULL, the routine allocates the 
*			necessary space to hold the result.
*       Returns: The converted string, 'ostr', if it is not NULL, or a
*		 pointer to the resulting allocated string if it is NULL.
*		 NULL is returned if 'istr' is NULL, or if an error occurs.
*    Written By: Mark Young
*          Date: Jul 13, 1993
*  Side Effects: Allocates the space for the output if 'ostr' is NULL
*************************************************************/
char *kstring_upper(
   const char *istr,
   char       *ostr)
{
	char *string;

	if (!istr)
	{
	   kinfo(KSYSLIB,"kstring_upper: NULL input string encountered" );
	   return(NULL);
	}

	/*
	 *  If the ostr is NULL then kstrdup the input string
	 */
	if (!ostr)
	   ostr = kstrdup(istr);
	else if (istr != ostr)
	   kstrcpy(ostr, istr);

	string = ostr;
	while (*string != '\0')
	{
	   if (islower(*string))
	      *string = toupper(*string);

	   string++;
	}
	return(ostr);
}

/************************************************************
*
*  Routine Name: kstring_capitalize - convert a string to its capitalized
*				      equivalent
*       Purpose: This routine checks to see that the first character
*		 of each word is capitalized.  A word is defined to be
*		 something separated by whitespace (i.e. separated by
*		 one or more tabs or spaces).
*
*		 Note that if 'ostr' is non-NULL, the calling routine
*		 must ensure that 'ostr' points to a memory buffer
*		 large enough to hold the capitalized string and terminating
*		 null character.  If the buffer is not large enough,
*		 memory will be overwritten resulting in unpredicable
*		 program failure.
*
*         Input: istr - The string to convert to a capitalized version.
*        Output: ostr - The string that holds the converted string.
*			If it's NULL, it allocates the space necessary.
*       Returns: The converted output string, 'ostr' if it is not NULL, or
*		 a pointer to the resulting allocated string if 'ostr' is
*		 NULL.  NULL is returned if 'istr' is NULL, or an error
*		 occurs.
*    Written By: Mark Young
*          Date: Feb 20, 1994
*  Side Effects: Allocates the space necessary for the output if 'ostr'
*		 is NULL.
*************************************************************/

char *kstring_capitalize(
   const char *istr,
   char       *ostr)
{
	char *string;

	if (!istr)
	{
	   kinfo(KSYSLIB,"kstring_capitalize: NULL input string encountered" );
	   return(NULL);
	}

	/*
	 *  If the ostr is NULL then kstrdup the input string
	 */
	if (!ostr)
	   ostr = kstrdup(istr);
	else if (istr != ostr)
	   kstrcpy(ostr, istr);

	string = ostr;
	while (*string != '\0')
	{
	   if (islower(*string))
	      *string++ = toupper(*string);
	   else
	      string++;

	  /*
	   * STEVE:  Whitespace isn't the only "new word" marker what about
	   * comma period, colon, etc?
	   */
	   while (*string != '\0' && !isspace(*string))
	      string++;

	   while (*string != '\0' && isspace(*string))
	      string++;
	}
	return(ostr);
}

/************************************************************
*
*  Routine Name: kstring_seddata
*       Purpose: This function performs a global change of text on
*                a string based on substring or regular expression 
*                patterns.  
*         Input: str     - source kstring
*                kvalist - NULL terminated list of (search pattern, 
*                          replacement pattern, is_regex) triples 
*                          to be used to modify the data as
*                          it is being copied.
*       Returns: pointer to resulting kstring
*    Written By: Kevin Sullivan
*          Date: Nov 11, 1996
*      Examples: For example, if we call:
*		 .begin code
*		 new_string = kstring_seddata("Welcome to Khoros", 
*				"Khoros", "the Twilight Zone", FALSE,
*				NULL); 
*		 .end code
*		 the result will be:
*		 .begin code
*		 new_string = "Welcome to the Twilight Zone".
*		 .end code
*************************************************************/
 
char *kstring_seddata(
   kstring str,
   kvalist)
{
   int  regex;
   kva_list valist;
   char *data, *temp, *istr, *ostr;
 
 
   kva_start(valist, str);
   if ((data = kstrdup(str)) == NULL)
      return(FALSE);
 
   while ((istr = kva_arg(valist, char *)) != NULL)
   {
      ostr = kva_arg(valist, char *);
      regex = kva_arg(valist, int);
      if (regex == FALSE)
         temp = kstring_replace(data, istr, ostr, NULL);
      else
         temp = kregex_replace(data, istr, ostr, NULL);
 
      kfree_and_NULL(data); data = temp;
   }
 
   return(data);
}

/************************************************************
*
*  Routine Name: kstring_replace - replace one substring with another
*       Purpose: This function performs a global change of text on
*		 the input string.  It returns a string where every
*		 occurrence of the scan string is replaced with the
*		 replacement string.  If 'ostr' is sent in as NULL,
*		 the result will be allocated for you.  If 'ostr'
*		 is provided as non-NULL, the result will be stored in it.
*		 Note that if the latter use is chosen, ostr must
*		 be previously allocated with a sufficient amount of storage
*		 space before this routine is called.
*
*		 Note that if 'ostr' is non-NULL, the calling routine
*		 must ensure that 'ostr' points to a memory buffer
*		 large enough to hold the copied string and terminating
*		 null character.  If the buffer is not large enough,
*		 memory will be overwritten resulting in unpredicable
*		 program failure.
*
*         Input: istr        - The string to be changed.
*		 scan_str    - The search string to be replaced.
*		 replace_str - The string that replaces the search string.
*        Output: ostr        - The string that holds the converted output
*			       string.  If it is NULL, this routine will
*			       allocate the space necessary.
*       Returns: The converted string, 'ostr', if the variable is not NULL, 
*		 or a pointer to the resulting allocated string.
*		 NULL is returned if 'istr' or 'scan_str' are NULL, or if
*		 an error occurs.
*    Written By: Mark Young
*          Date: Jul 03, 1992
*      Examples: For example, if we call:
*		 .begin code
*		 new_string = kstring_replace("Welcome to Khoros", "Khoros",
*					      "the Twilight Zone", NULL);
*		 .end code
*		 the result will be:
*		 .begin code
*		 new_string = "Welcome to the Twilight Zone".
*		 .end code
*  Side Effects: This routine allocates the space for the output if 'ostr'
*		 is NULL.
*************************************************************/

char *kstring_replace(
   const char *istr,
   const char *scan_str,
   const char *replace_str,
   char *ostr)
{
    size_t size = 0;
    size_t ilen, rlen, length, numchar;
    char  *string, *temp = NULL;
    const char *save = istr;

    if (istr == NULL)
      {
	kinfo(KSYSLIB,"kstring_replace: NULL input string encountered" );
	return(NULL);
      }

    if (scan_str == NULL)
      {
	kinfo(KSYSLIB,"kstring_replace: NULL scan string encountered" );
	return(NULL);
      }

    ilen = kstrlen(scan_str);
    rlen = kstrlen(replace_str);
    length = kmax(rlen, KLENGTH);

	/*
	 *  If the ostr is NULL then we need to kallocate temporary
	 *  space for the string.  Another problem case is when the input
	 *  and output strings are the same but the replacement token is
	 *  larger than the input token.  In this case we use a temporary
	 *  array and then kstrcpy the result into the output array, hoping
	 *  that user was smart enough to kmalloc enough space for us.
	 */
    if ((save == ostr && rlen > ilen) || (ostr==NULL))
      {
	size = length;
	if ((temp = (char *) kmalloc(sizeof(char)*size)) == NULL)
	  {
	    kinfo(KSYSLIB,"kstring_replace: Out of memory for temporary "
			  "string");
	    return(NULL);
	  }
	string = temp;
      }
    else
      string = ostr;

    while (*istr != '\0')
      {
	if (temp != NULL)
	  {
	    numchar = string - temp;
	    if (numchar >= (size-rlen-1))
	      {
	        size += length;
	        if ((temp = (char *) krealloc(temp, size)) == NULL)
	          {
		    kinfo(KSYSLIB, "kstring_replace: Out of memory for "
				   "temporary string");
		    return(NULL);
	          }
	       string = temp + numchar;
	      }
	  }

	if (kstrncmp(istr, scan_str, ilen) == 0)
	  {
	    kstrncpy(string, replace_str, rlen);
	    string += rlen;
	    istr += ilen;
	  }
	else
	  *string++ = *istr++;
      }
    *string = '\0';

	/*
	 *  Check to see if the output string should be kmalloc'ed or if
	 *  the input and output strings are the same but we used a temporary
	 *  since the input token was smaller than the replacement token.
	 */
    if (ostr == NULL)
      {
	ostr = kstrdup(temp);
	kfree_and_NULL(temp);
      }
    else if (save == ostr && rlen > ilen)
      {
	kstrcpy(ostr, temp);
	kfree_and_NULL(temp);
      }
    return(ostr);
  }

/************************************************************
*
*  Routine Name: kchar_replace - replace a character with another through a
*				 string
*       Purpose: This function performs a global change of character on
*		 the input string.  It returns a string where every
*		 occurrence of the scan character is replaced with the
*		 replacement character.  If 'ostr' is sent in as NULL,
*		 the result will be allocated with kmalloc() for you.
*		 If 'ostr' is non-NULL, the result will be stored
*		 in 'ostr'.  Note that if 'ostr' is non-NULL, it must
*		 point at a memory buffer with a sufficient amount of
*		 storage space before this routine is called.
*
*		 Note that if 'ostr' is non-NULL, the calling routine
*		 must ensure that 'ostr' points to a memory buffer
*		 large enough to hold the copied string and terminating
*		 null character.  If the buffer is not large enough,
*		 memory will be overwritten resulting in unpredicable
*		 program failure.
*
*         Input: istr         - The string to be changed.
*		 scan_char    - The scan character to be replaced.
*		 replace_char - The character which replaces the scan
*				 character.
*        Output: ostr         - The string that holds the converted string.
*				If it's NULL, the routine allocates the space
*				necessary to hold the result.
*       Returns: The converted string 'ostr' if it is not NULL, or a pointer
*		 to the resulting allocated string if it is NULL.  NULL is
*		 returned on an error, or if 'istr' is NULL.
*    Written By: Mark Young
*          Date: Jul 03, 1992
*      Examples: For example, if we call:
*
*		 .begin code
*		 new_string = kchar_replace("Many fishes", 'f', 'w', NULL);
*		 .end code
*
*		 the result will be:
*		 .begin code
*		 new_string = "Many wishes".
*		 .end code
*  Side Effects: Allocates the space for the output string if 'ostr' is NULL
*************************************************************/

char *kchar_replace(
   const char *istr,
   int         scan_char,
   int         replace_char,
   char       *ostr)
{
	char *string;

	if (istr == NULL)
	{
	   kinfo(KSYSLIB, "kchar_replace: NULL input string encountered");
	   return(NULL);
	}

	/*
	 *  If the ostr is NULL then kstrdup the input string
	 */
	if (ostr == NULL)
	   ostr = kstrdup(istr);
	else if (istr != ostr)
	   kstrcpy(ostr, istr);

	string = ostr;
	while (string != NULL)
	{
	   if ((string = kstrchr(string, scan_char)) != NULL)
	   {
	      *string++ = replace_char;
	      if (*string == '\0') break;
	   }
	}
	return(ostr);
}

/************************************************************
*
*  Routine Name: kstring_cleanup - remove white space from the ends of a string
*       Purpose: This routine checks the beginning and end of the string
*		 for white space (using the function isspace()) and removes
*		 these characters.  White space is defined to be a tab,
*		 space, carriage return, or line feed.
*
*		 Note that if 'ostr' is non-NULL, the calling routine
*		 must ensure that 'ostr' points to a memory buffer
*		 large enough to hold the cleaned string and terminating
*		 null character.  If the buffer is not large enough,
*		 memory will be overwritten resulting in unpredicable
*		 program failure.
*
*         Input: istr - The string to cleanup.
*        Output: ostr - The string that holds the converted string.
*			If it's NULL, it kmallocs the space necessary.
*       Returns: The cleaned string in 'ostr' if it is not NULL, or
*		 a pointer to the resulting allocated string if 'ostr' is
*		 NULL.  NULL is returned if istr is NULL, or an error occurs.
*    Written By: Mark Young
*          Date: Jul 03, 1992
*  Side Effects: Allocates the space for the output string if 'ostr' is NULL
*************************************************************/

char *kstring_cleanup(
   const char *istr,
   char       *ostr)
{
	char *string, *temp;

	if (istr == NULL)
	{
	   if (ostr) ostr[0] = '\0';
	   return(NULL);
	}

	if (ostr == NULL)
	   string = kstrdup(istr);
	else if (istr != ostr)
	   string = kstrcpy(ostr, istr);
	else
	   string = ostr;

	if ((temp = _cleanup_string(string)) == NULL)
	{
	   if (ostr == NULL)
	      kfree(string);
	   else
	      ostr[0] = '\0';

	   return(NULL);
	}
	else if (temp != string)
	   kstrcpy(string, temp);

	return(string);
}

/************************************************************
*
*  Routine Name: kstring_format - Format a string
*       Purpose: This routine formats a string.
*         Input: istr - string to be formatted
*        Output: ostr - string that holds the formatted string.
*			If it's NULL, it kmallocs the space necessary.
*       Returns: ostr if it is not NULL, a pointer to the
*		 resulting kmalloc'ed string if it is NULL.  NULL on
*		 an error.
*  Restrictions: NOTE TO MARK this routine doesn't do anything, and
*		 it returns a bad pointer.. :) (stevej)
*    Written By: Mark Young
*          Date: Jul 03, 1992
*************************************************************/

char *kstring_format(
   const char *istr,
   const char *prepend,
   const char *append,
   int         format_width,
   int         ignore_cr,
   char       *ostr)
{
	char *string, temp[10*KLENGTH];


	temp[0] = '\0';
	if (!ostr)
	   string = kstrcpy(ostr, istr);
	else
	   string = kstrdup(istr);

	return(string);
}

/*****************************************************************
*
*  Routine Name: kstring_cat - concatenate two strings
*       Purpose: This routine concatenates two strings together,
*		 into a third.  If the output parameter is not
*		 NULL, the concatenated string is put into that
*		 string.  Otherwise, the resulting string is allocated
*		 via a call to kmalloc().  In either case, the resulting
*		 string is returned.
*
*		 Note that if 'ostr' is non-NULL, the calling routine
*		 must ensure that 'ostr' points to a memory buffer
*		 large enough to hold the concatenated string and terminating
*		 null character.  If the buffer is not large enough,
*		 memory will be overwritten resulting in unpredicable
*		 program failure.
*
*         Input: istr1 - The first string to concatenate.
*		 istr2 - The second string to concatenate.
*        Output: ostr  - The string that holds the concatenated string.
*                        If it's NULL, the routine allocates the necessary
*			 space.
*       Returns: The concatenated string if 'ostr' is non-NULL, or
*		 the allocated string if 'ostr' is NULL.
*		 On NULL input strings or an error, a NULL is returned.
*  Side Effects: This routine uses kmalloc() to create a string
*		 that holds the result, if 'ostr' is non-NULL. 
*    Written By: Steven Jorgensen & Nick Ruprecht
*          Date: Jul 03, 1992
*************************************************************/
char *kstring_cat(
   const char *istr1,
   const char *istr2,
   char       *ostr)
{
	size_t n1 = kstrlen(istr1), n2 = kstrlen(istr2), nout = n1 + n2;

	if (nout == 0)
	   return (NULL);
	else if (!ostr && (ostr = (char *)kmalloc(nout+1)) == NULL)
	{
	   kinfo(KSYSLIB,"kstring_cat: Cannot kmalloc enough memory for \
resulting string");
	}

	kmemcpy(ostr + n1, (kaddr) istr2, n2);
	kmemcpy(ostr, (kaddr) istr1, n1);

	ostr[nout] = '\0';
	return(ostr);
}

/************************************************************
*
*  Routine Name: kstring_ncat - concatenate two partial strings
*       Purpose: This routine concatenates two partial strings together,
*		 into a third.  If the output parameter, 'ostr', is not
*		 NULL, the concatenated string is put into that
*		 string.  Otherwise, the resulting string is allocated
*		 In either case, the resulting NULL-terminated string is
*		 returned.  The second partial string is added to the end
*		 of the partial string first.  Also, this routine puts a '\\0'
*		 on the resulting string.
*
*		 Note that if 'ostr' is non-NULL, the calling routine
*		 must ensure that 'ostr' points to a memory buffer
*		 large enough to hold the concatenated string and terminating
*		 null character.  If the buffer is not large enough,
*		 memory will be overwritten resulting in unpredicable
*		 program failure.
*
*         Input: istr1 - The first string to concatenate.
*		 istr2 - The second string to concatenate.
*		 num1  - The number of characters from 'istr1' to put in the
*			 final string.
*		 num2  - The number of characters from 'istr2' to put in the
*		         final string.
*        Output: ostr  - The string that holds the concatenated string.
*                        If it's NULL, it allocates the necessary memory.
*       Returns: The concatenated string 'ostr' if the variable is non-NULL,
*		 or the allocated string if 'ostr' is NULL.  If 'istr1'
*		 and 'istr2' are NULL, or if 'num1' and 'num2' are both less
*		 then or equal to zero, or an error occurs, then NULL is
*		 returned.
*
*  Side Effects: This routine uses kmalloc() to allocate a string
*		 that holds the result, if 'ostr' is NULL. 
*    Written By: Steven Jorgensen & Nick Ruprecht
*          Date: Jul 03, 1992
*
*************************************************************/

char *kstring_ncat(
   const char *istr1,
   const char *istr2,
   ssize_t     num1,
   ssize_t     num2,
   char       *ostr)
{
	size_t n1 = kstrlen(istr1), n2 = kstrlen(istr2), nout;

	n1 = (num1 >= 0 && num1 < n1) ? num1 : n1;
	n2 = (num2 >= 0 && num2 < n2) ? num2 : n2;
	nout = n1 + n2;
	if (nout == 0)
	   return (NULL);
	else if ((ostr==NULL) && ((ostr=(char *)kmalloc(nout+1)) == NULL))
	{
	   kinfo(KSYSLIB,"kstring_ncat: Out of memory for resulting string");
	   return(NULL);
	}

	if (n2)
	   kmemcpy(ostr + n1, (kaddr) istr2, n2);
	if (n1)
	   kmemcpy(ostr, (kaddr) istr1, n1);

	ostr[nout] = '\0';
	return(ostr);
}

/************************************************************
*
*  Routine Name: kstring_3cat - concatenate three strings together
*       Purpose: This routine concatenates three strings together,
*		 into a new string.  If the output parameter is not
*		 NULL, the concatenated string is put into that
*		 string.  Otherwise, the resulting string is allocated
*		 via kmalloc(). In either case, the resulting NULL-terminated
*		 string is returned.  The concatenation is as follows;
*		 the second string is added to the end of the first,
*		 and the third string is added to the end of the first two.
*
*		 Note that if 'ostr' is non-NULL, the calling routine
*		 must ensure that 'ostr' points to a memory buffer
*		 large enough to hold the concatenated string and terminating
*		 null character.  If the buffer is not large enough,
*		 memory will be overwritten resulting in unpredicable
*		 program failure.
*
*         Input: istr1 - The first string to concatenate.
*		 istr2 - The second string to concatenate.
*		 istr3 - The third string to concatenate.
*        Output: ostr  - The string that holds the concatenated string.
*                        If it's NULL, it allocates the space necessary
*			 to hold the result.
*       Returns: The concatenated string in 'ostr' if it is non-NULL, or
*		 an allocated string if 'ostr' is NULL.  This routine
*		 returns NULL if 'istr1', 'istr2', and 'istr3' are NULL,
*		 or an error occurs.
*  Side Effects: This routine uses kmalloc() to create a string
*		 that holds the result if 'ostr' is NULL. 
*    Written By: Steven Jorgensen & Nick Ruprecht
*          Date: Jul 03, 1992
*************************************************************/

char *kstring_3cat(
   const char *istr1,
   const char *istr2,
   const char *istr3,
   char       *ostr)
{
	size_t n1 = kstrlen(istr1),
	       n2 = kstrlen(istr2),
	       n3 = kstrlen(istr3),
	       nout = n1 + n2 + n3;

	if (nout == 0)
	   return(NULL);
	else if (!ostr && (ostr = (char *)kmalloc(nout+1)) == NULL)
	{
	   kinfo(KSYSLIB,"kstring_3cat: Out of memory for resulting string");
	}

	kmemmove(ostr + n1 + n2, (kaddr) istr3, n3);
	kmemmove(ostr + n1, (kaddr) istr2, n2);
	if (ostr != istr1)
	   kmemmove(ostr, (kaddr) istr1, n1);

	ostr[nout] = '\0';
	return(ostr);
}

/************************************************************
*
*  Routine Name: kstring_subcmp - compares two sub-strings
*       Purpose: This routine compares two input strings to determine
*		 if 'istr2' is the same string as the beginning of 'istr1'.
*		 In kstring_subcmp() the number of characters compared is
*		 an implied value which is the length of the second input
*		 string 'istr2'.  The input strings are compared, character
*		 by character, until either the end of one of the input
*		 strings is encountered or 'istr1' and 'istr2' no longer
*		 match.
*         Input: istr1 - The base string to be compared.
*		 istr2 - The sub-string to be compared.
*	Returns: If the two strings up to the end of the second string are
*		 identical, the value of 0 is returned.
*                If the two strings differ, the ASCII value difference of
*                the first character that differs in the two strings will be
*                returned.  If the ASCII value of the differing character
*		 in 'istr1' is greater than the one in 'istr2', then the return
*                value is positive, and implies that 'istr1' is greater than
*                'istr2'.  If the difference between the two ASCII values is
*                negative, the return value implies that 'istr2' is greater
*		 than 'istr1'.
*    Written By: Mark Young
*          Date: Aug 26, 1994
*************************************************************/
int kstring_subcmp(
   const char *istr1,
   const char *istr2)
{
	istr1 = nullstring(istr1); 
	istr2 = nullstring(istr2); 

	while (*istr1 == *istr2 && *istr1 != '\0' && *istr2 != '\0')
	   istr1++, istr2++;

	if (*istr2 == '\0')
	   return(0);
	else
	   return(*istr1 - *istr2);
}

/************************************************************
*
*  Routine Name: kstring_copy - copy a string
*       Purpose: This routine copies a string into another string.  If the
*		 output parameter is not NULL, the concatenated string is put
*		 into that string.  Otherwise, the resulting string is
*		 allocated via a call to kmalloc().  In either case, the
*		 resulting string is returned.
*
*		 Note that if 'ostr' is non-NULL, the calling routine
*		 must ensure that 'ostr' points to a memory buffer
*		 large enough to hold the copied string and terminating
*		 null character.  If the buffer is not large enough,
*		 memory will be overwritten resulting in unpredicable
*		 program failure.
*
*         Input: istr - The string to be copied.
*        Output: ostr - The string that holds the copied string.
*                       If 'ostr' is NULL, the routine allocates the
*			space necessary for the result.
*       Returns: If 'ostr' is non-NULL, it is returned.  Otherwise the
*		 allocated string is returned.   If 'istr' is NULL, or
*		 an error occurs, NULL is returned.
*  Side Effects: This routine uses kmalloc() to create a string
*		 that holds the result if 'ostr' is NULL. 
*    Written By: Mark Young
*          Date: Sep 23, 1993
*************************************************************/

char *kstring_copy(
   const char *istr,
   char       *ostr)
{
        if (!istr)
           return(NULL);
	else if (ostr)
	   ostr[0] = '\0';
        else if ((ostr = (char *)kmalloc(kstrlen(istr)+1)) == NULL)
        {
           kinfo(KSYSLIB,"kstring_copy: Out of memory for output string");
           return(NULL);
        }
	kstrcpy(ostr, istr);
        return(ostr);
}

/************************************************************
*
*  Routine Name: kstring_ncopy - copy up to n characters of a string
*       Purpose: This routine concatenates two strings together,
*		 into a third.  If the output parameter is not
*		 NULL, the concatenated string is put into that
*		 string.  Otherwise, the resulting string is allocated
*		 by kmalloc().  In either case, the resulting string is
*		 returned.  Finally, this routine puts a terminating '\\0'
*		 on the end of the new string.
*
*		 Note that if 'ostr' is non-NULL, the calling routine
*		 must ensure that 'ostr' points to a memory buffer
*		 large enough to hold the copied string and terminating
*		 null character.  If the buffer is not large enough,
*		 memory will be overwritten resulting in unpredicable
*		 program failure.
*
*         Input: istr - The string to copy characters from.
*		 num  - The number of characters to copy.
*        Output: ostr - The string that holds the copied string.
*                       If it's NULL, it allocates the space necessary.
*       Returns: The copied string 'ostr', if it is non-NULL, or
*		 the allocated string if 'ostr' is NULL.
*		 NULL is returned if 'istr' is NULL, or if an error occurs.
*  Side Effects: This routine uses kmalloc() to create a string
*		 that holds the result, if 'ostr' is NULL.
*    Written By: Mark Young
*          Date: Sep 23, 1993
*************************************************************/

char *kstring_ncopy(
   const char *istr,
   size_t      num,
   char       *ostr)
{
	/*
	 *  Sanity check...
	 */
	if (!istr)
	{
	   if (ostr) ostr[0] = '\0';
	   return(NULL);
	}
	if (!ostr) ostr = (char *)kmalloc(num+1);

	/*EMPTY*/
	kstrncpy(ostr, istr, num);
	ostr[num] = '\0';
	return(ostr);
}

/************************************************************
*
*  Routine Name: kstring_detab - remove tabs from a string
*       Purpose: This routine converts tab, '\t', characters into
*		 the appropriate number of spaces to make it
*		 fall onto modulo eight boundary.
*
*		 Note that if 'ostr' is non-NULL, the calling routine
*		 must ensure that 'ostr' points to a memory buffer
*		 large enough to hold the detab string and terminating
*		 null character.  If the buffer is not large enough,
*		 memory will be overwritten resulting in unpredicable
*		 program failure.
*
*         Input: istr - The string to remove tabs from.
*        Output: ostr - The string that holds the converted input string.
*                       If 'ostr' is  NULL, it allocates the space necessary
*			to hold the result.
*       Returns: The variable 'ostr' if it is non-NULL, or the allocated
*		 string if 'ostr' is NULL.  If 'istr' is NULL or an error
*		 occurs, NULL is returned.
*
*  Side Effects: This routine uses kmalloc to create a string that holds
*		 the result, if 'ostr' is NULL.
*    Written By: Mark Young
*          Date: Jun 27, 1995
*************************************************************/

char *kstring_detab(
   const char *istr,
   char       *ostr)
{
        size_t i, size, len = 0, buffer_len, tabnum = 0;
        char *tmp, *otmp, *string, *buffer;
 
        /*
         *  Sanity check...
         */
        if (!istr)
           return(NULL);

	/* 
	 * if no output buffer specified, need to figure out worst-case
	 * scenario requirement for size of output string, and allocate
	 * that size of buffer dynamically (DA & JMAS, 6/30/97).
	 */
	if (!ostr)
	{
            string = (char *) istr;
	    while (string != NULL)
	    {
                if ((tmp = kstrchr(string, '\t')) != NULL)
	        {
	           tabnum++;
	           tmp++;
	        }
	        string = tmp;
	    }
	    /* at this point, we know if there aren't any tabs; return if so */
	    if (tabnum == 0) return(kstrdup(istr));

	    /* otherwise, dynamically allocate buffer in which to detab */
	    buffer_len = kstrlen(istr) + (tabnum*8) + 1;
	    buffer = (char *) kcalloc((size_t)1, (buffer_len*sizeof(char)));
            ostr = buffer;
	}

        otmp = ostr;
        string = (char *) istr;
	while (string != NULL)
	{
            if ((tmp = kstrchr(string, '\t')) != NULL)
	    {
	       size = tmp-string; 
	       tmp++;
	    }
            else
               size = (int) kstrlen(string);

	    if (size > 0)
	    {
	       kstrncpy(otmp, string, size);
	       len += size; otmp += size;
	    }

	    if (tmp != NULL)
	    {
	       size = 8 - (len % 8); len += size;
	       for (i = 0; i < size; i++) *otmp++ = ' ';
	    }
	    string = tmp;
	}
	ostr[len] = '\0';
        return(ostr);
}
