/*
 * Khoros: $Id: toolboxes.c,v 1.20.2.6 1997/05/31 18:59:33 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: toolboxes.c,v 1.20.2.6 1997/05/31 18:59:33 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros Toolbox Utilities
   >>>>		
   >>>>  Private:
   >>>>                 ktoolbox_siteinclude()
   >>>>                 ktoolbox_initialize()
   >>>>                 ktoolbox_list()
   >>>>                 ktoolbox_check()
   >>>>                 ktoolbox_path()
   >>>>                 ktoolbox_libpath()
   >>>>                 ktoolbox_binpath()
   >>>>                 ktoolbox_add()
   >>>>                 ktoolbox_delete()
   >>>>   Static:
   >>>>                 ktoolbox_dirinclude()
   >>>>                 ktoolbox_filepaths()
   >>>>                 cleanup_handler()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	

static size_t   numtb = 0;
static char   **toolboxes = NULL;
static char   **toolbox_path = NULL;
static ktoken  *toolbox_tkn = NULL;
static ktoken  *toolbox_ftkn = NULL;
static kbool   *toolbox_globaldir = NULL;
static ssize_t  bs_indx = -1;
static char    *bs_binpath = NULL;
static char    *bs_libpath = NULL;

#ifndef KCONFIG_DEFINE
#define KCONFIG_DEFINE "(undefined)"
#endif


static void cleanup_handler(kexit_status status, kaddr client_data)
{
	karray_free(toolboxes, KSTRING, numtb, NULL);
	karray_free(toolbox_path, KSTRING, numtb, NULL);
	karray_free(toolbox_globaldir, KLOGICAL, numtb, NULL);
	karray_free(toolbox_tkn, KSTRUCT, numtb, NULL);
	karray_free(toolbox_ftkn, KSTRUCT, numtb, NULL);
	toolboxes = NULL;
	toolbox_path = NULL;
	toolbox_globaldir = NULL;
	toolbox_tkn = NULL;
	toolbox_ftkn = NULL;
        kfree_and_NULL(bs_binpath);
        kfree_and_NULL(bs_libpath);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_filepaths - 
|
|       Purpose: returns a color seperated list of paths to toolbox config
|		 files
|
|         Input: 
|        Output: the path to the toolbox config files, if NULL malloc
|       Returns: return the site include path or NULL
|
|    Written By: Mark Young
|          Date: Aug 20, 1995
| Modifications:
|
------------------------------------------------------------*/

static char *ktoolbox_filepaths(
   char *paths)
{
	char *names, temp[KLENGTH];

	names = (char *) kgetenv("KHOROS_TOOLBOX");
	if (kstrlen(names) == 0)
	{
	   ksprintf(temp, "$HOME/.Toolboxes");
	   names = temp;
	}
	paths = (paths == NULL) ? kstrdup(names) : kstrcpy(paths, names);
	return(paths);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_dirinclude - 
|
|       Purpose: The path to the dir include file
|
|         Input: 
|        Output: the path to the dir include, if NULL malloc
|       Returns: return the site include path or NULL
|
|    Written By: Mark Young
|          Date: Aug 20, 1995
| Modifications:
|
------------------------------------------------------------*/

static char *ktoolbox_dirinclude(
   char *path)
{
        char  *karch, *kopsys, *kwidset, filename[KLENGTH];
 
        if ((karch = kgetenv("KARCH")) != NULL)
        {
	   kinfo(KVERBOSE, "KARCH, KOPSYS, and/or KWIDSET is defined.  This is the obsolete method for specifying your system config name, which will be phased out in a future release of Khoros.  The string you specified when running kbuild is what should be used by default.");
           kopsys  = kgetenv("KOPSYS");
           kwidset = kgetenv("KWIDSET");
           if (kopsys == NULL)
              ksprintf(filename, "Dir.%s", karch);
           else if (kwidset == NULL)
              ksprintf(filename, "Dir.%s-%s", karch, kopsys);
           else
              ksprintf(filename, "Dir.%s-%s-%s", karch, kopsys, kwidset);
        }
	else
	   ksprintf(filename, "Dir.%s", KCONFIG_DEFINE);

	path = (path == NULL) ? kstrdup(filename) : kstrcpy(path, filename);
	return(path);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_siteinclude - 
|
|       Purpose: The path to the site include file
|
|         Input: 
|        Output: the path to the site include, if NULL malloc
|       Returns: return the site include path or NULL
|
|    Written By: John MA Salas
|          Date: Nov 13, 1995
| Modifications:
|
------------------------------------------------------------*/

char *ktoolbox_siteinclude(
   const char *tbname,
   char *path)
{
        char *karch, *kopsys, *kwidset, filename[KLENGTH];
        int  location;
 
        location = karray_locate(toolboxes, KSTRING, tbname, numtb);

        if (location == -1)
           return(NULL);

        if ((karch = kgetenv("KARCH")) != NULL)
        {
	   kinfo(KVERBOSE, "KARCH, KOPSYS, and/or KWIDSET is defined.  This is the obsolete method for specifying your system config name, which will be phased out in a future release of Khoros.  The string you specified when running kbuild is what should be used by default.");
           kopsys  = kgetenv("KOPSYS");
           kwidset = kgetenv("KWIDSET");
           if (kopsys == NULL)
              ksprintf(filename, "$%s/config/Site.%s", tbname, karch);
           else if (kwidset == NULL)
              ksprintf(filename, "$%s/config/Site.%s-%s",tbname,karch,kopsys);
           else if (kwidset != NULL)
              ksprintf(filename, "$%s/config/Site.%s-%s-%s", tbname,
                        karch, kopsys, kwidset);
        }
        else
           ksprintf(filename, "$%s/config/Site.%s", tbname, KCONFIG_DEFINE);

        path = (path == NULL) ? kstrdup(filename) : kstrcpy(path, filename);
        return(path);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_initialize - initialize the toolbox routines
|
|       Purpose: This subroutine initializes the global list of
|		 toolboxes used to represent the different toolboxes
|		 currently to be used by the Khoros programs.
|
|         Input:
|        Output:
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Mark Young  
|          Date: Sep 23, 1993
| Modifications:
|
------------------------------------------------------------*/

int ktoolbox_initialize(void)
{
	ssize_t indx;
	kfile  *file;
	ktoken  tbtkn;
	klist  *filelist = NULL;
	int	globaldir = FALSE;
	char   *filenames, *tmp, *tmp1, filename[KLENGTH], temp[KLENGTH],
		tbname[KLENGTH], tbpath[KLENGTH],
		name[KLENGTH], def[KLENGTH], options[KLENGTH], paths[KLENGTH];

	if (toolboxes != NULL)
	   return(TRUE);

	/*
	 *  Add cleanup handler to cleanup this mess...
	 */
	kexit_handler(cleanup_handler, NULL);

	if ((filenames = ktoolbox_filepaths(paths)) == NULL)
	   return(FALSE);

	while (filenames != NULL)
	{
	   if ((tmp = kstrchr(filenames, ':')) != NULL)
	   {
	      kstring_ncopy(filenames, (size_t) (tmp-filenames), filename);
	      tmp++;
	   }
	   else
	      kstrcpy(filename, filenames);

	   filelist = klist_insert(filelist, kstring_to_token(filename),
				  (kaddr) 0, KLIST_TAIL, TRUE);
	
	   filenames = tmp;
	}

	while (filelist != NULL)
	{
	   tmp = ktoken_to_string((ktoken) klist_identifier(filelist));
	   /* KHOROS_TOOLBOX variable cannot be opened */
	   if ((file = kfinput(tmp)) == NULL)
	   {
	      kinfo(KSYSLIB, "ktoolbox_initialize: Unable to open toolbox "
			     "file '%s' for reading.", tmp);
	      filelist = klist_delete(filelist, klist_identifier(filelist));
	      continue;
	   }

	   kfseek(file, (long) klist_clientdata(filelist), 0);

	   while (kfgets(temp, KLENGTH, file) != NULL)
	   {
	      *options = '\0';
	      if (ksscanf(temp,"%[^#:]:%[^;];%[^\n]",tbname,tbpath,options)< 2)
	      {
		 if (kstrncmp(temp, "#include", (size_t) 8) != 0)
		    continue;
		 tmp1 = temp+8;
		 tmp1 = kstring_cleanup(tmp1, tmp1);
		 filelist->client_data = (kaddr) kftell(file);
		 filelist = klist_insert(filelist, kstring_to_token(tmp1),
					 (kaddr) 0, KLIST_HEAD, FALSE);
		 kfclose(file);
		 file = NULL;
		 break;
	      }

	      kstring_cleanup(tbname, tbname);
	      tbtkn = kstring_to_token(tbname);
	      kstring_cleanup(tbpath, tbpath);

	      if (kgetenv(tbname) == NULL)
	      {
	         ksprintf(temp, "%s=%s", tbname, tbpath);
	         kputenv(temp);
	      }

	      /*
	       *  Check to see if this toolbox might be supporting multiple
	       *  toolbox bins.  We know this if there is an options part
	       *  of the toolbox definition (from the khoros toolbox file)
	       */
	      tmp1 = options;
	      globaldir = FALSE;
	      while (ksscanf(tmp1, "%[^=]=%[^,\n]", name, def) == 2)
	      {
	         kstring_cleanup(def, def); kstring_lower(def, def);
		 if (kstrcmp(def, "yes") == 0  || kstrcmp(def, "true") == 0 || 
		     kstrcmp(def, "on") == 0 || kstrcmp(def, "1") == 0)
		 {
		    kstring_cleanup(name, name); kstring_lower(name, name);
		    if (kstrcasecmp(name, "globaldir") == 0)
		       globaldir = TRUE;
		 }

		 if ((tmp1 = kstrchr(tmp1, ',')) != NULL)
		    tmp1++;
		 else
		    break;
	      }

	      if ((indx = karray_locate(toolbox_tkn, KSTRUCT,
				tbtkn, numtb)) != -1)
	      {
	         if (kstrcmp(toolbox_path[indx], tbpath) != 0)
		 {
		    kinfo(KSTANDARD, "Warning toolbox '%s' is defined more "
			  "than once in your Toolboxes file(s).  Overriding "
			  "the original path\n'%s' in toolbox file\n'%s' with\n"
			  "'%s' in toolbox file\n'%s'\n", tbname,
				toolbox_path[indx],
				ktoken_to_string(toolbox_ftkn[indx]), tbpath,
				ktoken_to_string(klist_identifier(filelist)));
		    kfree(toolbox_path[indx]);
		    toolbox_path[indx] = kstrdup(tbpath);
		    toolbox_globaldir[indx] = globaldir;
		    toolbox_ftkn[indx] = klist_identifier(filelist);
	 	 }
	         continue;
	      }

	      /*
	       *  Don't join the boot#strap strings otherwise kcms may
	       *  change it when copying kutils from one toolbox to another.
	       */
	      if (kstrcmp(tbname, "BOOT""STRAP") == 0)
		 bs_indx = numtb;

	      toolboxes = (char **) karray_insert(toolboxes, KSTRING,
				   kstrdup(tbname), numtb, KLIST_TAIL, TRUE);
	      toolbox_tkn = (kaddr *)karray_insert(toolbox_tkn, KSTRUCT, tbtkn,
				   numtb, KLIST_TAIL, TRUE);
	      toolbox_path = (char **) karray_insert(toolbox_path, KSTRING,
				   kstrdup(tbpath), numtb, KLIST_TAIL, TRUE);
	      toolbox_globaldir = (unsigned char *)
				   karray_insert(toolbox_globaldir, KLOGICAL,
				   globaldir, numtb, KLIST_TAIL, TRUE);
	      toolbox_ftkn = (kaddr *) karray_insert(toolbox_ftkn, KSTRUCT,
				   klist_identifier(filelist), numtb++,
				   KLIST_TAIL, TRUE); 
	   }
	   if (file != NULL)
	   {
	      kfclose(file);
	      filelist = klist_delete(filelist, klist_identifier(filelist));
	   }
	}
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_list - create a list of toolbox names
|
|       Purpose: This routine takes the global toolbox list, and
|		 returns an array of toolbox names to the calling
|		 routine.  It also takes a count parameter that
|		 is set to the number of entries in the toolbox array.
|
|         Input: none
|
|        Output: num - The number of toolbox names in the array; 0 on error.
|       Returns: The list of toolbox names if successful, or
|			  NULL on error.
|
|    Written By: Mark Young
|          Date: Sep 23, 1993
| Modifications:
|
------------------------------------------------------------*/

char **ktoolbox_list(
   size_t *num)
{
	ktoolbox_initialize();

	if (num) *num = numtb;
	return(toolboxes);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_check - check to see if the toolbox exists
|				in the toolbox list.
|
|       Purpose: This routine checks to see if a toolbox exists in the
|		 toolbox list.
|         Input: tbname - the toolbox name to check for
|        Output: 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Jul 29, 1994
| Modifications:
|
------------------------------------------------------------*/

int ktoolbox_check(
   const char *tbname)
{
	int  i;

	ktoolbox_initialize();
	for (i = 0; i < numtb; i++)
	{
	   if (kstrcasecmp(tbname, toolboxes[i]) == 0)
	      return(TRUE);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_libpath - get library path for toolbox
|
|       Purpose: This routine returns the library path assoicated
|                with a specific toolbox.  Called by kecho.
|         Input: tbname - the name of the toolbox
|        Output:
|       Returns: corresponding toolbox path, or NULL if not found or on error
|
|    Written By: Kevin Sullivan
|          Date: August 29, 1996
| Modifications:
|
------------------------------------------------------------*/

char *ktoolbox_libpath(
   const char *tbname,
   char       *path)
{
        kfile   *file;
        ssize_t count;
        int     i, status, globaldir = 0;
        char    temp[KLENGTH], filename[KLENGTH], location[KLENGTH],
                *libpath, **scan, *tbpath = NULL;

        ktoolbox_initialize();
        for (i = 0; i < numtb; i++)
        {
           if (kstrcasecmp(tbname, toolboxes[i]) == 0)
           {
              globaldir = toolbox_globaldir[i];
              tbpath    = kstrdup(toolbox_path[i]);
	      break;
           }
        }

        if (tbpath == NULL || !ktoolbox_dirinclude(temp))
           return(NULL);

        libpath = NULL;
        ksprintf(filename, "$%s/config/%s", tbname, temp);
        if ((file = kfinput(filename)) != NULL)
        {
           scan = kparse_file_scan_delimit(file, "^\\s*LibDir", "$",
                        KIGNORE_CASE, "[ \t]", NULL, NULL, &count, &status);

           if (count > 1)
              libpath = kstrdup(scan[1]);

           karray_free(scan, KSTRING, (size_t) count, NULL);
           kfclose(file);
        }
        else if (globaldir)
        {
           if (bs_libpath == NULL)
           {
              if (!ktoolbox_dirinclude(temp))
                 return(NULL);

              kremote_location(kgetenv(tbname), location);
              ksprintf(filename, "$BOOT""STRAP/config/%s", temp);
              kremote_file(location, filename, temp);

              if ((file = kfinput(temp)) != NULL)
              {
                 scan = kparse_file_scan_delimit(file, "^\\s*LibDir", "$",
                        KIGNORE_CASE, "[ \t]", NULL, NULL, &count, &status);

                 if (count > 1)
                    bs_libpath = kremote_exec(location, scan[1], NULL);

                 karray_free(scan, KSTRING, (size_t) count, NULL);
                 kfclose(file);
              }

              if (bs_libpath == NULL)
                 bs_libpath = kfullpath("$BOOTSTRAP/lib", NULL, NULL);
           }
           libpath = kstrdup(bs_libpath);
        }

        /* if no libpath was found then return default */
        if (libpath == NULL)
        {
           ksprintf(temp, "%s/lib", tbpath);
           libpath = kstrdup(temp);
        }

        path = (path == NULL) ? kstrdup(libpath) : kstrcpy(path, libpath);
        kfree(libpath);
        return(path);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_path - get a path entry from a specific toolbox
|
|       Purpose: This subroutine returns the path assoicated with
|                a specific toolbox.  The path is part of the global
|                structure used internally by the khoros routines
|                and should be treated as a read-only resource.
|
|         Input: variable - the name of the toolbox
|       Returns: corresponding toolbox path, or NULL if not found or on error
|    Written By: Mark Young
|          Date: Jul 03, 1992
|
------------------------------------------------------------*/
char *ktoolbox_path(
   const char *variable)
{
	char  cleanup[KLENGTH];

	ktoolbox_initialize();
	if (!kstring_cleanup(variable, cleanup))
	   return(NULL);

	kstring_upper(cleanup, cleanup);
	return(kgetenv(cleanup));
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_binpath - 
|
|       Purpose: Initialize the toolbox bin path.
|
|         Input: tbtkn - the toolbox token
|        Output: the path to the toolbox bin, if NULL malloc
|       Returns: return the toolbox bin path or NULL
|
|    Written By: Mark Young & John Salas
|          Date: Jul 17, 1994
| Modifications:
|
------------------------------------------------------------*/

char *ktoolbox_binpath(
   const char *tbname,
   char       *path)
{
	kfile  *file;
	ssize_t count;
	int    status, i, globaldir = 0;
	char   *binpath, **scan, *tbpath = NULL;
        char   temp[KLENGTH], filename[KLENGTH], location[KLENGTH];
 
        ktoolbox_initialize();
        for (i = 0; i < numtb; i++)
        {
           if (kstrcasecmp(tbname, toolboxes[i]) == 0)
           {
              tbpath    = toolbox_path[i];
              globaldir = toolbox_globaldir[i];
	      break;
           }
        }

	if (tbpath == NULL || !ktoolbox_dirinclude(temp))
	   return(NULL);

	binpath = NULL;
	ksprintf(filename, "$%s/config/%s", tbname, temp);
	if ((file = kfinput(filename)) != NULL)
	{
	   scan = kparse_file_scan_delimit(file, "^\\s*BinDir", "$",
			KIGNORE_CASE, "[ \t]", NULL, NULL, &count, &status);

	   if (count > 1)
	      binpath = kstrdup(scan[1]);

	   karray_free(scan, KSTRING, (size_t) count, NULL);
	   kfclose(file);
	}
	else if (globaldir)
	{
	   if (bs_binpath == NULL)
	   {
	      if (!ktoolbox_dirinclude(temp))
	         return(NULL);

	      kremote_location(kgetenv(tbname), location);
	      ksprintf(filename, "$BOOT""STRAP/config/%s", temp);
	      kremote_file(location, filename, temp);

	      if ((file = kfinput(temp)) != NULL)
	      {
	         scan = kparse_file_scan_delimit(file, "^\\s*BinDir", "$",
			KIGNORE_CASE, "[ \t]", NULL, NULL, &count, &status);

	         if (count > 1)
		    bs_binpath = kremote_exec(location, scan[1], NULL);

	         karray_free(scan, KSTRING, (size_t) count, NULL);
	         kfclose(file);
	      }

	      if (bs_binpath == NULL)
	         bs_binpath = kfullpath("$BOOTSTRAP/bin", NULL, NULL);
	   }
	   binpath = kstrdup(bs_binpath);
	}

	/* define the toolbox binpath so that we don't do this again */
	if (binpath == NULL)
	   ksprintf(temp, "%sBIN=%s/bin", tbname, tbpath);
	else
	   ksprintf(temp, "%sBIN=%s", tbname, binpath);

	kinfo(KDEBUG, "defining binpath: '%s'\n", temp);
	kputenv(temp);

	/* if no binpath was found then return NULL */
	if (binpath == NULL)
	   return(NULL);

	path = (path == NULL) ? kstrdup(binpath) : kstrcpy(path, binpath);
	kfree(binpath);
	return(path);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_add - add a new toolbox to the list
|
|       Purpose: This subroutine adds a new toolbox to the global
|                toolbox list.  This routine takes all the information
|                required to store a Khoros Toolbox entry.
|
|         Input: variable - the name of the toolbox
|		 path     - the path for the top directory of the toolbox
|       Returns: return TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jul 03, 1992
|
------------------------------------------------------------*/
int ktoolbox_add(
   const char *variable,
   const char *path)
{
	kfile   *file;
	char    *filenames, *tmp, **list;
	kstring routine = "ktoolbox_add()";
	ssize_t  i, num;
        int     status, written, first = TRUE;
	char    temp[KLENGTH], filename[KLENGTH];
	char    tbname[KLENGTH], tbvar[KLENGTH], tbpath[KLENGTH],
		paths[KLENGTH];


	if ((filenames = ktoolbox_filepaths(paths)) == NULL)
	   return(FALSE);

	kstring_cleanup(path, tbpath);
	kstring_cleanup(variable, tbvar);
	kstring_upper(tbvar, tbvar);

	while (filenames != NULL)
	{
	  written = FALSE;
	  if ((tmp = kstrchr(filenames, ':')) != NULL)
	  {
	     kstring_ncopy(filenames, (size_t) (tmp - filenames), filename);
	     tmp++;
	  }
	  else kstring_copy(filenames, filename);


	   /* KHOROS_TOOLBOX variable cannot be opened */
	   filenames = tmp;
	   if ((file = kfinput(filename)) != NULL)
	   {
	      list = kparse_file_scan_delimit(file, KPARSE_BOF, KPARSE_EOF,
				 KLITERAL, "\n", NULL, NULL, &num, &status);
	      kfclose(file);
	   }
	   else
	   {
	      num = 0;
	      list = NULL;
	   }

	   for (i = 0; i < num; i++)
	   {
	      if (ksscanf(list[i], "%[^#:]:", tbname) == 1 &&
	          kstrcasecmp(tbname, tbvar) == 0)
	      {
	         written = TRUE;
	         ksprintf(temp, "%s:%s\n", tbvar, tbpath);
		 kfree(list[i]);
		 list[i] = kstrdup(temp);
	      }
	   }

	   /*--------------------------------------------------------------
	    * Only want to change the file if it's the first file and the
	    * toolbox entry is not in it, or if the entry is in the
	    *--------------------------------------------------------------*/
	   if (first || written)
	   {
	      /* KHOROS_TOOLBOX variable cannot be opened */
	      if ((file = kfoutput(filename)) == NULL)
	      {
	         if (!written)
	            kwarn(KUTILS, routine, "Unable to open toolbox file '%s' "
			  "for writing, so could not add reference for toolbox"
			  " `%s'.", filename, tbvar);
	         continue;
	      }
	   
	      for (i = 0; i < num; i++)
	         kfprintf(file, "%s\n", list[i]);
	      if (!written)
	      {
	  	 first = FALSE;
	         kfprintf(file, "%s:%s\n", tbvar, tbpath);
	      }
	      kfclose(file);
	   }
	   karray_free(list, KSTRING, (size_t) num, NULL);
	}

	if (!first)
	{
	   ksprintf(temp, "%s=%s", tbvar, tbpath);
	   kputenv(temp);
	   toolboxes = (char **)karray_insert(toolboxes, KSTRING,
				kstrdup(tbvar), numtb++, KLIST_TAIL, TRUE);
	}
	return(!first);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_delete - delete a toolbox entry from the list
|
|       Purpose: This routine removes a toolbox entry from the
|                global toolbox list.  The toolbox name is taken as
|                an input parameter.
|         Input: variable - the name of the toolbox to delete
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jul 03, 1992
|
------------------------------------------------------------*/
int ktoolbox_delete(
   const char *variable)
{
	kfile   *file;
	char    *filenames, *tmp, **list;
	ssize_t  i, num;
        int     status, mentioned;
	kstring routine = "ktoolbox_delete()";
	char    tbname[KLENGTH], tbvar[KLENGTH], filename[KLENGTH],
		paths[KLENGTH];


	if ((filenames = ktoolbox_filepaths(paths)) == NULL)
	   return(FALSE);

	kstring_cleanup(variable, tbvar);
	kstring_upper(tbvar, tbvar);
	while (filenames != NULL)
	{
	   if ((tmp = kstrchr(filenames, ':')) != NULL)
	   {
	      kstring_ncopy(filenames, (size_t) (tmp - filenames), filename);
	      tmp++;
	   }
	   else kstring_copy(filenames, filename);

	   /* KHOROS_TOOLBOX variable cannot be opened */
	   filenames = tmp;
	   if ((file = kfinput(filename)) == NULL)
	   {
	      kerror(KUTILS, routine, "Unable to open toolbox file '%s' "
				      "for reading.", filename);
	      continue;
	   }

	   list = kparse_file_scan_delimit(file, KPARSE_BOF, KPARSE_EOF,
			KLITERAL, "\n", NULL, NULL, &num, &status);
	   if (list == NULL)
	   {
	      kerror(KUTILS,routine,"Cannot read contents of toolbox file '%s'",
		     filename);
	      continue;
	   }
	   kfclose(file);

	   /*-- is the toolbox mentioned? ----------------------------------*/
	   mentioned = FALSE;
	   for (i = 0; i < num; i++)
	   {
	      if (ksscanf(list[i], "%[^#:]:", tbname) != 1 ||
	          kstrcasecmp(tbname, tbvar) == 0)
	      {
	         mentioned = TRUE;
	      }
	   }

	   if (!mentioned)
	      continue;

	   /* KHOROS_TOOLBOX variable cannot be opened */
	   if ((file = kfoutput(filename)) == NULL)
	   {
	      kwarn(KUTILS, routine, "Unable to write to toolbox file `%s' "
			    "-- reference to toolbox `%s' will remain.",
			    filename, tbvar);
	     continue;
	   }

	   for (i = 0; i < num; i++)
	   {
	      if (ksscanf(list[i], "%[^#:]:", tbname) != 1 ||
	          kstrcasecmp(tbname, tbvar) != 0)
	      {
	         kfprintf(file, "%s\n", list[i]);
	      }
	   }
	   kfclose(file);
	   karray_free(list, KSTRING, (size_t) num, NULL);
	}

	kremenv(tbvar);
	for (i = 0; i < numtb; i++)
	{
	   if (kstrcmp(toolboxes[i], tbvar) == 0)
	   {
	      toolboxes = (char **)karray_delete(toolboxes, KSTRING,
						 toolboxes[i], numtb--);
	      break;
	   }
	}
	return(TRUE);
}
