/*
 * Khoros: $Id: utilities.c,v 1.12.2.4 1997/06/09 23:20:30 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.12.2.4 1997/06/09 23:20:30 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros Utilities
   >>>>		
   >>>>  Private:
   >>>>                 _cleanup_string()
   >>>>                 kexpand_variable()
   >>>>                 kexpand_tilda()
   >>>>                 ktmpnam()
   >>>>                 kcheck_command()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	


/*-----------------------------------------------------------
|
|  Routine Name: _cleanup_string - internal clean up string routine
|
|       Purpose: This function is used to strip the white space
|		 off the beginning and end of a string using
|		 isspace() to detect white space.
|		 Note: this routine changes the input string directly.
|
|         Input: string - the string to clean up
|
|        Output: none
|                RETURNS: the cleaned up string, or NULL on error
|
|    Written By: Mark Young
|          Date: Jul 03, 1992
| Modifications:
|
------------------------------------------------------------*/
char *_cleanup_string(
   register char *string)
{
	register char *temp;


	/*
	 *  Delete any white space from the head and tail of the
	 *  string.
	 */
	while (isspace(*string))
	   string++;

	if (*string == '\0')
	   return(NULL);

	temp = string + kstrlen(string);
	while (isspace(*--temp))
		;
	*(temp+1) = '\0';
	return(string);
}


/*-----------------------------------------------------------
|
|  Routine Name: kexpand_variable - expand environment variables.
|
|       Purpose: This function is used to expand an environment
|		 variable in the supplied string.  The variable
|		 is the environment variable that we are going
|		 to expand in the string.  The variable may contain
|		 a $VARIABLE or just VARIABLE inside the string
|		 or the variable itself.  The other possibility
|		 the string.  This can be done by passing in
|		 NULL for the variable name, in which case any
|		 $VARAIBLE encountered within the string will
|		 be expanded.  (note:  only variables that begin
|		 with a '$' will be expanded since the variable
|		 was not explicitly given).
|
|         Input: string   - the string to be expanded
|        Output: expanded - the memory in which to stores the expanded string
|       Returns: returns the expanded string, or NULL on error.
|    Written By: Mark Young
|          Date: Jul 04, 1995
|
------------------------------------------------------------*/
char *kexpand_variable(
   const char *string,
   char *expanded)
{
	size_t num;
	const char *istr;
	char *tmp, *var, *itmp, *otmp, *ostr, *escape;
	char temp[10*KLENGTH], temp2[10*KLENGTH], variable[KLENGTH],
	     toolbox[KLENGTH];

	if (string == NULL)
	   return(NULL);
	else
	   istr = string;

	if (expanded == NULL)
	   ostr = temp;
	else if (string == expanded)
	   ostr = temp2;
	else
	   ostr = expanded;

	otmp = ostr;
	while (istr != NULL)
	{
	   if ((itmp = kstrchr(istr, '$')) != NULL)
	   {
	      /* copy everything from before the '$' */
	      num = itmp - istr;
              if (num != 0
                 && (escape = itmp) != NULL
                 && *--escape == '\\') /* escape the kgetenv */
              {
                 num--; /* don't copy the '\' */
              }
              else
                 escape = NULL;
	      kstrncpy(otmp, istr, num); otmp += num;

	      /* locate the end of the environment variable */
	      tmp = itmp+1;
	      while (*tmp != '$' && *tmp != '/' && *tmp != '\0')
		 tmp++;

	      num = (size_t) (tmp-(itmp+1));
	      /* copy the environment variable into a temporary string */
	      kstring_ncopy(itmp+1, num, variable);

              if (escape != NULL)
	      {
                 var = NULL;
	      }

	      /* get the variable from the environ list */
	      else if ((var = kgetenv(variable)) == NULL)
	      {
		 /* check to see if it's a toolbox bin variable */
		 if (num > 3 && kstrcasecmp(&variable[num-3], "BIN") == 0)
		 {
		    kstring_ncopy(variable, num-3, toolbox);
		    ktoolbox_binpath(toolbox, toolbox);
		 }
		 /* initialize the toolboxes in case it's a toolbox */
	         else
		    ktoolbox_initialize();

		 /* let's try again */
		 var = kgetenv(kstring_upper(variable, variable));
	      }

	      /*
	       * copy the environment variable into our output string,
	       * otherwise copy the unexpanded input string.
	       */
	      if (var != NULL)
	      {
		 kstrcpy(otmp, var);
		 otmp += kstrlen(var); 
	      }
	      else
	      {
		 kstrncpy(otmp, itmp, (size_t) (tmp-itmp));
		 otmp += tmp-itmp;
	      }
	      itmp = tmp;
	   }
	   else
	      kstrcpy(otmp, istr);

	   istr = itmp;
	}

	/*
	 *  The input and output string is the same, so we better copy our
	 *  expanded output back into our input "string"
	 */
	if (ostr != expanded)
	   return(ostr == temp ? kstrdup(ostr) : kstrcpy(expanded, ostr));
	else
	   return(expanded);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpand_tilda - expand a tilda in a path
|
|       Purpose: This function is used to expand a username.
|		 If the string has a ~username or ~/ or ~ this
|		 routine will expand the string to the full
|		 path of that string
|
|         Input: string - the string to be expanded
|
|        Output: none
|                RETURNS: returns the expanded string, or NULL on error
|
|    Written By: Mark Young
|          Date: Jul 03, 1992
| Modifications:
|
------------------------------------------------------------*/
char *kexpand_tilda(
   const char *string,
   char *expanded)
{
	size_t size;
	const  char *tmp;
	struct passwd *passwd_ent;
	char   *ostr, temp[10*KLENGTH], temp2[10*KLENGTH], *dir = NULL;

	temp[0] = '\0';
	temp2[0] = '\0';

        if (string == NULL)
           return(NULL);
        else if (expanded == NULL)
           ostr = temp;
        else if (string == expanded)
           ostr = temp2;
	else
	   ostr = expanded;

	if (string[0] == '~')
	{
	   if (kstrncmp(string, "~/", (size_t) 2) == 0 ||
	       kstrcmp(string, "~") == 0)
	   {
	      dir = kgetenv("HOME");
	      tmp = &string[1];
	   }
	   else 
	   {
	      if ((tmp = kstrchr(string, '/')) != NULL)
	      {
		 size = tmp - &string[1];
		 kstrncpy(ostr, &string[1], size);
		 ostr[size] = '\0';
	      }
	      else
	         kstrcpy(ostr, &string[1]);

	      if ((passwd_ent = getpwnam(ostr)) != NULL)
		 dir = passwd_ent->pw_dir;
	   }

	   if (dir == NULL)
	   {
	      errno = KINVALID_USER;
	      kerror("kutils", "kfullpath", "Not a valid user (%s)", string);
	      return(NULL);
	   }

	   /* copy the result into the output "ostr" array */
	   kstrcpy(ostr, dir);
	   kstrcat(ostr, tmp);
	}
	else kstrcpy(ostr, string);


        /*
         *  The input and output string is the same, so we better copy our
         *  expanded output back into our input "string"
         */
	if (ostr != expanded)
	   return(ostr == temp ? kstrdup(ostr) : kstrcpy(expanded, ostr));
	else
	   return(expanded);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktmpnam - create a temporary name
|
|       Purpose: This module is a private routine that is used
|		 by the transport library to create a template
|		 filename which is used to help identify the
|		 the application routine.  It uses this template
|		 to create a unique pathname.  If no template
|		 is passed to the routine, TEMPLATE is used.
|
|         Input: ktemplate - template of the file to be created.
|        Output:
|       Returns: returns a string containing a unique temporary filename.
|
|    Written By: Mark Young
|          Date: Jul 03, 1992
| Modifications:
|
------------------------------------------------------------*/
#ifndef P_tmpdir
#define P_tmpdir "/var/tmp"
#endif

char  *ktmpnam(
   const char *ktemplate)
{
	int  i;
	char filename[KLENGTH], basename[KLENGTH], dirname[KLENGTH];

	static pid_t pid = 0;
	static long  cnt = 0;
	static char *tmpdir = NULL;
	static char *dirs[] = { P_tmpdir, ".", "/usr/tmp", "/tmp" };

	if (tmpdir == NULL)
	{
	   pid = getpid();
	   tmpdir = kgetenv("TMPDIR");
	   for (i = 0; i < knumber(dirs) && tmpdir == NULL; i++)
	   {
	      if (kaccess(dirs[i], W_OK) == 0)
		 tmpdir = dirs[i];
	   }

	   if (tmpdir == NULL)
	      tmpdir = "$HOME";
	}

	/*
	 *  Want to seperate the directory (if any) from the actual
	 *  template filename.
	 */
	if (kbasename(ktemplate, basename) == NULL)
	   kstrcpy(basename, TEMPLATE);

	if (!kstrchr(ktemplate, '/') || !kdirname(ktemplate, dirname) ||
	    kaccess(dirname, W_OK) == -1)
	{
	   kstrcpy(dirname, tmpdir);
	}
	ksprintf(filename, "%s/%s%lX.%lx", dirname, basename, (long) pid,cnt++);
        return(kstrdup(filename));
}

/*-----------------------------------------------------------
| 
|  Routine Name: kcheck_command
| 
|       Purpose: Determine if a command string should be run
|                under a shell
| 
|         Input: cmdstr - the command string to check
|        Output: None
|        Return: 0 if no shell needed, 1 if /bin/sh is needed,
|                2 if /bin/csh is needed
|    Written By: Pete Humphrey
|          Date: June 6, 1997
| Modifications: 
| 
------------------------------------------------------------*/
int kcheck_command(char *cmdstr)
{
   char quote_char = 0;         /* keeps track of matching quotes */
   register char *ptr1, *ptr2;
 
   for (ptr1 = cmdstr;  *ptr1;  ptr1 = ptr2)
   {
      register char ch;
 
      /* check for a shell character, a quote mark, or a backslash */
      ptr2 = kstrpbrk(ptr1, "~|;<>*\\\"\'");
      
      if (ptr2 == NULL)
      {
         /* no shell characters, escaped characters, or quotes */
         return 0;
      }
 
      ch = *(ptr2++);
      
      if (ch == '\\')
      {
         /* something is being escaped - skip the next char */
         if (*ptr2) ptr2++;
      }
      else if (ch == '\"'  ||  ch == '\'')
      {
         if (quote_char == ch)
         {
            /* we found a matching quote mark */
            quote_char = 0;
         }
         else
         {
            if (quote_char == 0)
            {
               /* we are starting a new quote */
               quote_char = ch;
            }
         }
      }
      else if (quote_char == 0)
      {
         /* we have found an un-quoted, un-escaped shell character */
         if (kstrchr(cmdstr, '~'))
            return 2;
         else
            return 1;
      }
   }
   return 0;
}
