#!/bin/sh

#
# Khoros: $Id: kbugreport.sh,v 1.10.2.5 1997/06/02 20:14:36 mirage Exp $
#

# Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
# All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.


# # ========================================================================
# #
# # define various unix utilities we're gonna use
# #
# # ========================================================================

# ------------------------------------------------------------------------
# set up some other standard commands.
# eventually we should do something like the above code for echo,
# check for each of them on the path.  Oh for perl!

echo ""
echo " Working - just a few seconds while I gather some info"
echo ""

  tr="tr"
  cp="cp"
  mv="mv -f"
  rm="rm -f"
 awk="awk"
 sed="sed"
 cat="cat"
grep="grep"
strings="strings"
KPAGER=${PAGER:-more}
TMPDIR=${TMPDIR:-/tmp}
email="khoros-bugs@khoral.com"
clr=""
for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
    [ -x "$dir/clear" -a ! -d "$dir/clear" ] && clr="$dir/clear"
done

#
#  Assume a vt100 variant
#
if [ "$clr" = "" ]; then
  clr="echo -c [H[J"
fi
 
# clean up behind us, for example if this script gets interrupted
# ------------------------------------------------------------------------
cleanup() {
   echo ""
   echo "I'll just tidy up any temp files which might be lying around..."
   echo "Ok, bye..."
   $rm $TMPDIR/kbugreport.$$
   $rm $TMPDIR/bugreport.$$
   $rm $TMPDIR/bugheader.$$
   echo bye
   echo ""
   exit 0
}

# if user biffs ctrl-C, call our cleanup() function
trap cleanup 2

# prompt the user, do they want to continue
# be polite if they don't
# ------------------------------------------------------------------------
ConfirmContinue() {
   YesNoPrompt "Do you wish to continue?"
   if [ "$?" -eq "1" ]; then
      cleanup
   fi
}

# Simple continue prompt to allow the printing out of messages and before
# the next prompt and allowing the user to read the message.
# ------------------------------------------------------------------------
ContinuePrompt() {
   echon [Press \<Return\> to continue]
   read resp
}

# ------------------------------------------------------------------------
# let's see if we can use echo -n for prompting ...

# build a $echoN variable and $echoEnd combination that will give
# will allow the echon procedure to work well.
FindEchoN()
{
   # build a list of potential echos, starting with sh built-in
   echoEnd=""
   elist="echo"
   for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
      [ -x "$dir/echo" -a ! -d "$dir/echo" ] && elist="$elist $dir/echo"
   done
 
   # work through list, see if one of them groks -n
   for echo in $elist; do
      result=`$echo -n hello`
      if [ "$result" != "-n hello" ]; then
         echoN="$echo -n"
         break
      fi
   done
   if [ "$echoN" = "" ]; then
      echoN=echo
      echoEnd="\c"
      result=`$echoN "testing"$echoEnd`
      if [ "$result" != "testing" ]; then
         unset echoEnd
      fi
   fi
}

# echo without without return
echon()
{
   $echoN "$* $echoEnd"
}

# this is called every time we change `topic'
newtopic() {
   $clr
}

# prompt the user, to see if we should go on
YesNoPrompt()
{
   if [ "$#" -eq 0 ]; then
           echon "Do you wish to continue? (y/n) [y]:"
   else
      if [ "$#" -eq 2 ]; then
              echon "$1 (y/n) [$2]:"
      else
              echon "$1 (y/n) [y]:"
      fi
   fi
   read resp
   if [ "$#" -eq 2 -a "$resp" = "" ]; then
      resp=$2
   fi
   if [ "$resp" = "n" -o "$resp" = "N" ]; then
           return 1
   fi
   return 0
}
 
# Try to figure out the correct editor to use
FindEditor()
{
        if [ "$EDITOR" != "" ]; then
                return
        fi
 
        dlist=`echo $PATH | $sed -e 's/:/ /g'`
 
        for dir in $dlist; do
                if [ -x "$dir/vi" -a ! -d "$dir/vi" ]; then
                        EDITOR="$dir/vi"
                        return
                fi
        done
 
        for dir in $dlist; do
                if [ -x "$dir/emacs" -a ! -d "$dir/emacs" ]; then
                        EDITOR="$dir/emacs"
                        return
                fi
        done
        for dir in $dlist; do
                if [ -x "$dir/pico" -a ! -d "$dir/pico" ]; then
                        EDITOR="$dir/pico"
                        return
                fi
        done
        EDITOR="none"
}
 
Check4Template()
{
	hastemplate="no"
        if [ -r $HOME/.Khoros/kbug.template ]; then
           cat $HOME/.Khoros/kbug.template > $TMPDIR/bugheader.$$
           hastemplate="yes"
        elif [ -r $HOME/kbug.template ]; then
           cat $HOME/kbug.template > $TMPDIR/bugheader.$$
           hastemplate="yes"
        fi
        if [ "$hastemplate" = "yes" ]; then
	   newtopic
	   echo "This is the kbugreport template you previously saved:"
	   echo ""
	   echo ""
	   cat $TMPDIR/bugheader.$$
	   echo ""
	   echo ""
	   echo "You can create a new template by answering 'n'"
	   echon "Do you wish to use this template now? (y/n) [y] "
	   read resp
           if [ "$resp" = "n" -o "$resp" = "N" -o "$resp" = "no" -o \
           "$resp" = "NO" ]; then
		hastemplate="no"
	   fi
	fi
}

CreateTemplate()
{
	cat $TMPDIR/bugheader.$$
	echo ""
	echo ""
	echo "Do you wish to save the current information in a template file"
	echon "that can be loaded the next time you run kbugreport? (y/n) [y]"
	read resp
        if [ "$resp" = "n" -o "$resp" = "N" -o "$resp" = "no" -o \
           "$resp" = "NO" ]; then
           return 1
        elif [ -d $HOME/.Khoros ]; then
           cp $TMPDIR/bugheader.$$ $HOME/.Khoros/kbug.template
	   if [ $? -ne 0 ]; then
               echo ""
               echo "Can not create template file in $HOME/.Khoros. Template not saved."
               echo ""
	       return 1
            else
               echo ""
               echo "Created template file in $HOME/.Khoros/kbug.template"
               echo ""
            fi
	else
	   cp $TMPDIR/bugheader.$$ $HOME/kbug.template
	   if [ $? -ne 0 ]; then
               echo ""
               echo "Can not create template file in $HOME. Template not saved."
               echo ""
               return 1
            else
               echo ""
               echo "Created template file in $HOME/kbug.template"
               echo ""
            fi
	fi
}

WhichC()
{
        if [ "$cffile" != "NONE" ]; then
	   isgcc="`grep '#define HasGcc[  	]' $cffile | awk '{print $3}'`"
	   if [ "$isgcc" = "YES" ]; then
		cis="gcc"
	   else
		cis="cc"
	   fi
        else
	   cis="cc"
        fi
}

WhichF()
{
        if [ "$cffile" != "NONE" ]; then
	   isf2c="`grep '#define Usef2c[  	]' $cffile | awk '{print $3}'`"
	   if [ "$isf2c" = "YES" ]; then
		fis="f2c"
	   else
	      isg77="`grep '#define Useg77[  	]' $cffile | awk '{print $3}'`"
	      if [ "$isg77" = "YES" ]; then
		fis="g77"
	      else
		fis="Native Fortran"
	      fi
	   fi
        else
	   fis="Native Fortran"
        fi
}

GetShortDesc()
{
	length=1
        until [ "$length" -eq 0 ]; do
		echo ""
		echo "Enter a short description of your bug. You are limited to 75 characters."
		echon "Short Description: "
		read shdesc
		if [ `echo $shdesc | wc -c` -gt 75 ]; then
			echo ""
			echo "You entered more than 75 characters. Our parser"
			echo "will fail. Please enter a shorter description."
			echo ""
		else
			length=0
			echo "Short Description: $shdesc" >> $TMPDIR/bugheader.$$
		fi
	done
}

ReportBug()
{
        $clr
        FindEditor
        val=1
        until [ "$val" -eq 0 ]; do
                if [ "$EDITOR" != "none" ]; then
                        $EDITOR $TMPDIR/bugreport.$$
                else
                        echo "Cannot find an editor to call, Please edit the fil
e 'bugreport_info' in"
                        echo "the current directory and email it to 'khoros-bugs@khoral.com'"
                        $cp $TMPDIR/bugheader.$$ ./bugreport_info
                        cleanup
                fi
 
                $clr
                YesNoPrompt "Are you done editing the Bug Report file?"
                val=$?
        done
}
 
InitiateReport()
{
  echo "             $KHOROSNAME $KHOROSVER Bug Reporting Form" \
  > $TMPDIR/bugheader.$$
  echo "" >> $TMPDIR/bugheader.$$
  echo "" >> $TMPDIR/bugheader.$$
}

FindMailer()
{
        mailer="/usr/ucb/mail"
        if [ -x "$mailer" ]; then
                return
        fi
 
        dlist=`echo $PATH | $sed -e 's/:/ /g'`
 
        for dir in $dlist; do
                if [ -x "$dir/mailx" -a ! -d "$dir/mailx" ]; then
                        mailer="$dir/mailx"
                        return
                fi
        done
 
        for dir in $dlist; do
                if [ -x "$dir/mail" -a ! -d "$dir/mail" ]; then
                        mailer="$dir/mail"
                        return
                fi
        done
        mailer="none"
}
 
MailIt()
{
        FindMailer
        echo ""
	echo "Answering N to the following question will save the bugreport in"
	echo "the current directory so you can FAX, Mail or edit and send the"
	echo "report later. Please do send it to us, we do want to know."
	echo ""
        echo "About to mail the information to '$email'"
        YesNoPrompt "Ready to send it? "
        if [ "$?" -eq 0 ] ; then
                if [ "$mailer" != "none" ]; then
                        $mailer $email <$TMPDIR/kbugreport.$$ >/dev/null 2>&1
                else
                        $cp $TMPDIR/kbugreport.$$ ./kbugreport_info
                        echo ""
                        echo "I cannot find a mailer program that I understand, please"
                        echo "send the file 'kbugreport_info' in the current directory"
                        echo "to 'khoros-bugs@khoral.com'."
                fi
        else
                $cp $TMPDIR/kbugreport.$$ ./kbugreport_info
                echo ""
                echo "Ok exiting kbugreport, but I have left a file in the current"
                echo "directory called kbugreport_info with the information you typed"
                echo "into the bug report.  When you have finalized the bugreport"
                echo "please send the file to 'khoros-bugs@khoral.com' or FAX"
                echo "it to 505-881-3842 or mail it to us at:"
		echo "    Khoral Research"
		echo "    6001 Indian School Rd."
		echo "    Suite 200"
		echo "    Albuquerque, NM 87110   USA"
                echo "Thank You for reporting this bug"

        fi
        $rm $TMPDIR/bugheader.$$
        $rm $TMPDIR/kbugreport.$$
        $rm $TMPDIR/bugreport.$$
}

CountTimes()
{
   newfiletock="$*"
   counttbsfiles=0

   for cktbsfile in $tblist; do
      if [ "$cktbsfile" = "$newfiletock" ]; then
         counttbsfiles=1;
      fi
   done
}

#
# Find kecho
#
Findkecho() {

   foundkecho=0
   kecho=""
   for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
      if [ -x "$dir/kecho" -a ! -d "$dir/kecho" ]; then
         kecho="$dir/kecho"
         foundkecho=1
      fi
   done

}

#
# Find kversion
#
Findkversion() {

   foundkversion=0
   kversion=""
   for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
      if [ -x "$dir/kversion" -a ! -d "$dir/kversion" ]; then
         kversion="$dir/kversion"
         foundkversion=1
      fi
   done

}

FindEchoN
 
PATH=./bin:$PATH:/usr/local/bin:/usr/lang:/usr/bsd:/usr/ucb:/usr/bin:/bin:/usr/etc:/etc:/usr/local/gnu/bin:/usr/bin/X11
export PATH
 
if [ "$KHOROS_TOOLBOX" = "" ]; then
   ktb="$HOME/.Toolboxes"
else
   ktb="$KHOROS_TOOLBOX"
fi

tblist=""
for tbfile in `echo $ktb | $sed -e 's/:/ /g'`; do
   [ -f "$tbfile" ] && tblist="$tblist $tbfile"
done

newlist="$tblist"

while [ X"$newlist" != X ]; do
   for tbfile in $newlist; do

      newfiles=`$grep '^#include' $tbfile | $sed -e 's/^#include[ 	]*\([^ 	][^ 	]*\)/\1/'`
     
      nfiles=""
      for newfile in $newfiles; do
         CountTimes $newfile
         [ $counttbsfiles = 0 -a -f "$newfile" ] && nfiles="$nfiles $newfile"
      done

      for newfile in $nfiles; do
         CountTimes $newfile
         [ $counttbsfiles = 0 -a -f "$newfile" ] && tblist="$tblist $newfile"
      done
   done
   newlist="$nfiles"
done

for tbfile in $tblist; do
   bootdir=`$grep "^BOOTSTRAP" $tbfile | $awk -F\; '{ print $1 }' | \
      $awk -F: '{ print $NF }'` 
   if [ "$bootdir" != "" ]; then
      break
   fi
done
 
if [ "$bootdir" = "" ]; then
   echo ""
   echo "Cannot find the top directory for the BOOTSTRAP Toolbox"
   echo "in the file(s) listed in your KHOROS_TOOLBOX"
   echo "variable.  Please set this variable correctly, or"
   echo "use 'kconfigure' to correctly set up your environment"
   cleanup
fi

Findkecho

if [ $foundkecho = 1 ]; then
   configf=`kecho -echo config-name`
   cffile=$bootdir/config/Site.$configf
else
   configf=NONE
   cffile=$configf
fi

Findkversion

if [ $foundkversion = 1 ]; then
   KHOROSVER="`$kversion -version`"
   KHOROSNAME="`$kversion -name`"
else
   KHOROSVER="Unknown Version"
   KHOROSNAME="Khoros"
fi

Check4Template
if [ "$hastemplate" = "no" ]; then
#
# Did not want to indent all of this next section, so the else is a ways down
#
newtopic
 
  echo "             $KHOROSNAME $KHOROSVER Bug Reporting Form" \

$cat <<"EndFail"

   If this script should fail for any reason and you are unable to get the
   correct bug reporting form - do one of the following:

   1) Use the bug reporting form at out Web Site http://www.khoral.com

   2) E-mail info-request@khoral.com
      Place in the body of the letter the following:
      get khoros bugreport

EndFail

ConfirmContinue

newtopic
  echo "             $KHOROSNAME $KHOROSVER Bug Reporting Form" \

$cat <<"EndIntro"
 
   This script is for reporting bugs you have found in the Khoros System.
   You will be prompted for required information first, and then will be
   able to enter your bug description and a workaround if you have devel-
   oped one. You will then be able to E-mail it to Khoral Research or save
   it to a file to be faxed or dealt with later. 

   Remember to be descriptive as possible including all steps that lead to
   the bug. Include any details that would make your Khoros installation
   different from what might be considered standard.

   Khoral Research appreciates your participation in making a better
   Khoros for the future. All bugs are evaluated and recorded into our
   Bug Tracking system.

   This form is also available at our Web Site. The URL for the form is:

   http://www.khoral.com/forms/khoros-bugs.html

EndIntro

ConfirmContinue

if [ "$?" -eq 1 ]; then
   echo "Ok, bye..."
   echo ""
   cleanup
fi

newtopic

  echo "             $KHOROSNAME $KHOROSVER Bug Reporting Form" \

$cat <<"EndExplan"

    You will be prompted for several answers now. We have done our best
    to calculate the correct default responses (they appear in the [ ]
    brackets). If they appear to be correct just hit return, if not, please
    enter the appropriate information. Hints have been included to assist
    getting the correct information. 

EndExplan
InitiateReport
sysinfo=`$bootdir/objects/script/kbuild/src/config.guess`
tname=`finger $USER | awk -F: '/life/ {print $3}' | sed 's/.//' | head -1`
echon "Your Name [$tname]: "
read name
if [ "$name" = "" ]; then
	name=$tname
fi
echo "Name: $name" >> $TMPDIR/bugheader.$$
echo ""
tumail=$USER@`hostname`
echon "Your E-mail address [$tumail]: "
read umail
if [ "$umail" = "" ]; then
	umail=$tumail
fi
echo "E-mail: $umail" >> $TMPDIR/bugheader.$$
echo "Product: $KHOROSNAME" >> $TMPDIR/bugheader.$$
echo ""
echon "The version of Khoros (Hint: use kversion) [$KHOROSVER]: "
read ver
if [ "$ver" = "" ]; then
  ver=$KHOROSVER
fi
echo "Khoros Version: $ver">> $TMPDIR/bugheader.$$
echo ""
hware=`echo $sysinfo | awk -F- '{print $1}'`
echon "Hardware the bug occured on (Hint: use uname -m) [$hware]: "
read hw
if [ "$hw" = "" ]; then
  hw=$hware
fi
echo "Hardware: $hw" >> $TMPDIR/bugheader.$$
echo ""
osware=`uname -s`
echon "Operating System (Hint: use uname -s) [$osware]: "
read os
if [ "$os" = "" ]; then
  os=$osware
fi
echo "Operating System: $os" >> $TMPDIR/bugheader.$$
echo ""
osver=`uname -r`
echon "Operating System Version (Hint: use uname -r) [$osver]: "
read osv
if [ "$osv" = "" ]; then
  osv=$osver
fi
echo "Operating System Version: $osv" >> $TMPDIR/bugheader.$$
echo ""
WhichC
echon "C compiler used [$cis]: "
read compiler
if [ "$compiler" = "" ]; then
  compiler=$cis
fi
echo ""
WhichF
echon "Fortran compiler used [$fis]: "
read fcompiler
if [ "$fcompiler" = "" ]; then
  fcompiler=$fis
fi
echo "Compiler: $compiler  $fcompiler" >> $TMPDIR/bugheader.$$
echo ""

newtopic

  echo "             $KHOROSNAME $KHOROSVER Bug Reporting Form" \

$cat <<"EndNR"

   The following items are not required but would be appreciated. Even if
   the item does not affect this bug, we can use the data to understand
   what Khoros users are developing with and plan our future releases with
   this data in mind. Thank You!

EndNR
echo ""
echon "Version of Perl used (calculated with perl -v): "
read uperl
echo "Perl Version: $uperl" >> $TMPDIR/bugheader.$$
echo ""
echon "Version of X and patches: "
read xver
echo "X Version: $xver" >> $TMPDIR/bugheader.$$
echo ""
depth="`xdpyinfo | grep 'depths' | awk '{print $NF}'`"
echon "Depth of display (8, 16, 24, etc bit)[$depth]: "
read dis
if [ "$dis" = "" ]; then
  dis=$depth
fi
echo "Display Depth: $dis" >> $TMPDIR/bugheader.$$
echo "" >> $TMPDIR/bugheader.$$
echo ""

newtopic
CreateTemplate
#
# This fi matches the "if [ "$hastemplate" = "no" ]; then" statement way above
# 
fi
  
kdate=`date | awk '{printf "%s-%s-%s\n",$3,$2,$NF}'`
echon "Today's date in this format [$kdate]: "
read udate
if [ "$udate" = "" ]; then
  udate=$kdate
fi
echo "Date: $udate" >> $TMPDIR/bugheader.$$
echo ""

GetShortDesc

echo ""
echo "Enter the program or object you are experiencing the problem with, for"
echon "example - cantata, kadd, etc. : "
read object
if  [ "$object" = "" ]; then
  object=NS
fi
echo "Object: $object" >> $TMPDIR/bugheader.$$
echo ""
echo "Enter the Toolbox the above object is located in, for example - if the above"
echo "object was Cantata, the Toolbox would be DESIGN. You can use the following"
echon "command to help (kfindobj -oname object_from_above): "
read toolbox
if  [ "$toolbox" = "" ]; then
  toolbox=NS
fi
echo "Toolbox: $toolbox" >> $TMPDIR/bugheader.$$
echo ""

newtopic

  echo "             $KHOROSNAME $KHOROSVER Bug Reporting Form" \

$cat <<"EndEdit"
 
    Now you will enter the description of the bug and any suggestions or
    workarounds that you have discovered. After typing your report, end your
    editing session and you will be shown the results of your Bug Report.
    If you are satisfied with the report, answer "Y" to the prompt and the
    entire report will be E-mailed to khoros-bugs@khoral.com. If you answer
    "N", the report will be saved to the current directory. You can than
    mail it to us at:
    Khoral Research
    6001 Indian School Rd.
    Suite 200
    Albuquerque, NM 87110   USA
    or FAX it to us at 505-881-3842.
    Thank You!

EndEdit

echo "######################################################################" \
   >> $TMPDIR/bugheader.$$
echo "                      Bug Report" >> $TMPDIR/bugheader.$$
echo "" >> $TMPDIR/bugheader.$$
ConfirmContinue

GETREPORT=1
while [ $GETREPORT = 1 ]; do
   ReportBug
   if [ ! -f $TMPDIR/bugreport.$$ -o -z $TMPDIR/bugreport.$$ ]; then
      echo ""
      echo " You did not enter the description of the bug."
      ConfirmContinue
   else
      GETREPORT=0
   fi
done

$cat $TMPDIR/bugheader.$$ $TMPDIR/bugreport.$$ > $TMPDIR/kbugreport.$$
$cat $TMPDIR/kbugreport.$$ | $KPAGER
MailIt

