#!/bin/sh

#
#  Khoros: $Id: kbuild.sh,v 1.14.2.9 1997/06/28 23:05:27 mirage Exp $
#

# Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
# All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.


# # ========================================================================
# #
# # define various unix utilities we're gonna use
# #
# # ========================================================================

# ------------------------------------------------------------------------
# set up some other standard commands.
# eventually we should do something like the above code for echo,
# check for each of them on the path.  Oh for perl!

  tr="tr"
  cp="cp"
  mv="mv -f"
  rm="rm -f"
 awk="awk"
 sed="sed"
grep="grep"
strings="strings"
KPAGER=${PAGER:-more}
TMPDIR=${TMPDIR:-/tmp}
KHOROSVER="Khoros 2.2.0.0"

clr=""
for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
    [ -x "$dir/clear" ] && clr="$dir/clear"
done

#
#  Assume a vt100 variant
#
if [ "$clr" = "" ]; then
  clr="echo -c [H[J"
fi
 
# clean up behind us, for example if this script gets interrupted
# ------------------------------------------------------------------------
cleanup() {
   echo ""
   echo "I'll just tidy up any temp files which might be lying around..."
   echo "Ok, bye..."
   $rm $TMPDIR/kbuild.$$
   $rm $TMPDIR/check.$$
   $rm $TMPDIR/check1.$$
   echo bye
   echo ""
   exit 0
}

# if user biffs ctrl-C, call our cleanup() function
trap cleanup 2

# prompt the user, do they want to continue
# be polite if they don't
# ------------------------------------------------------------------------
ConfirmContinue() {
   YesNoPrompt "Do you wish to continue?"
   if [ "$?" -eq "1" ]; then
      cleanup
   fi
}

# Simple continue prompt to allow the printing out of messages and before
# the next prompt and allowing the user to read the message.
# ------------------------------------------------------------------------
ContinuePrompt() {
   echon [Press \<Return\> to continue]
   read resp
}

# ------------------------------------------------------------------------
# let's see if we can use echo -n for prompting ...

# build a $echoN variable and $echoEnd combination that will give
# will allow the echon procedure to work well.
FindEchoN()
{
   # build a list of potential echos, starting with sh built-in
   echoEnd=""
   elist="echo"
   for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
      [ -x "$dir/echo" ] && elist="$elist $dir/echo"
   done
 
   # work through list, see if one of them groks -n
   for echo in $elist; do
      result=`$echo -n hello`
      if [ "$result" != "-n hello" ]; then
         echoN="$echo -n"
         break
      fi
   done
   if [ "$echoN" = "" ]; then
      echoN=echo
      echoEnd="\c"
      result=`$echoN "testing"$echoEnd`
      if [ "$result" != "testing" ]; then
         unset echoEnd
      fi
   fi
}

# echo without without return
echon()
{
   $echoN "$* $echoEnd"
}

# this is called every time we change `topic'
newtopic() {
   $clr
}

#
# Print out the ToolboxInfo files
#
PrintDescription()
{
   keepprinting=1
   while [ $keepprinting = 1 ]; do
      newtopic
      echo ""
      echo "To get the description of the toolbox enter the corresponding number:"
      echo ""
      echo "     1. bootstrap"
      echo "     2. dataserv"
      echo "     3. design"
      echo "     4. datamanip"
      echo "     5. envision"
      echo "     6. geometry"
      echo "     7. image"
      echo "     8. matrix"
      echo "     9. migration"
      echo "     10. support"
      echo "     11. retro"
      echo "     *. Done"
      echo ""
      echon "Enter the number of the corresponding toolbox [Done]: "
      read tbtoprint

      case $tbtoprint in

         1) tbname=bootstrap
            ;;
         2) tbname=dataserv
            ;;
         3) tbname=design
            ;;
         4) tbname=datamanip
            ;;
         5) tbname=envision
            ;;
         6) tbname=geometry
            ;;
         7) tbname=image
            ;; 
         8) tbname=matrix
            ;;
         9) tbname=migration
            ;;
         10) tbname=support
            ;;
         11) tbname=retro
            ;;
         *) keepprinting=0
            ;;
      esac

      if [ $keepprinting = 1 ]; then
         newtopic
         $KPAGER $TOPDIR/$tbname/repos/ToolboxInfo

         ContinuePrompt
      fi
   done
}

#
# Get the global library directory:
#	1. If the globaldir in the Toolboxes file is true, yes, on or 1 any case
#	   do,
#	   a. check for TOOLBOX/config/Dir.$defkconfig
#	   b. check for bootstrap/config/Dir.$defconfig
#	   c. default to bootstrap/lib
#	2. use TOOLBOX/lib
#
GetLibDir()
{
   toolbox="$*"
   toolboxtocheck=`echo $toolbox | $tr "[a-z]" "[A-Z]"`

#
# Determine the name of the file that specifies the LibDir
#
   if [ "$KARCH" = "" ]; then
      dirfile=Dir.$defkconfig
   else
      dirfile=Dir.$KARCH
      if [ "$KOPSYS" != "" ]; then
         dirfile=Dir.$KARCH-$KOPSYS
         if [ "$KWIDSET" != "" ]; then
            dirfile=Dir.$KARCH-$KOPSYS-$KWIDSET
         fi
      fi
   fi
#
# Parse out the setting for globaldir from the $TOPDIR/Toolboxes file
#
   if [ -f $TOPDIR/Toolboxes ]; then
      ans=`$grep "^$toolboxtocheck" $TOPDIR/Toolboxes | $awk -F\; '{print $2}'`

      useglobaldir=false
      for entry in `echo $ans | $sed s/,/\ /g`; do

         case $entry in

            globaldir*) 
                useglobaldir=`echo $entry | $awk -F= '{ print $2 }'`
                break
                ;;
            *)
                useglobaldir=false
                ;;

         esac 
      done

      ans=`$grep "^$toolboxtocheck" $TOPDIR/Toolboxes | $awk -F\; '{ print $1 }'`
      toolboxtopdir=`echo $ans | $awk -F\: '{ print $2 }'`

      if [ "$toolboxtopdir" = "" ]; then 
	 return
      fi

      if [ ! -d "$toolboxtopdir" ]; then
         echo ""
         echo "The directory location for were the toolboxes $toolboxtocheck is located"
         echo "does not exist.  The entry in the $TOPDIR/Toolboxes"
         echo "can be removed since the toolbox does not seem to be installed."
         echo ""
         ContinuePrompt
	 return
      fi
   else
      echo ""
      echo "No $TOPDIR/Toolboxes to use."
      echo "This should not have happened, please mail khoros-bugs@khoral.com."
      echo ""
      cleanup
   fi

#
# Set the library directory in the variable checkdir
#
   checkdir=""
   useglobaldir=`echo $useglobaldir | $tr "[A-Z]" "[a-z]"`


   if [ "$useglobaldir" = "true" -o "$useglobaldir" = "yes" -o "$useglobaldir" = "on" -o "$useglobaldir" = "1" ]; then

      if [ -d $toolboxtopdir/config ]; then
         if [ -f $toolboxtopdir/config/$dirfile ]; then
		    checkdir=`$grep "^LibDir" $toolboxtopdir/config/$dirfile | $awk '{ print $2 }'`
         fi

	 if [ "$checkdir" != "" ]; then
            if [ -d $checkdir ]; then
               return 0
	    else
	       echo ""
	       echo "The library directory"
	       echo ""
               echo "    $checkdir"
	       echo ""
               echo "for $toolbox does not exist as specified in the file"
	       echo ""
	       echo "    $toolboxtopdir/config/$dirfile."
	       echo ""
	       echo "These could be the problems:"
	       echo "   1. The file entry is wrong and needs updating."
	       echo "   2. The file should not exist for this toolbox."
	       echo "Please fix and re-run this script."
	       cleanup
            fi
         fi
#
# Check the bootstrap toolbox
#
      elif [ -d $TOPDIR/bootstrap/config ]; then
         if [ -f $TOPDIR/bootstrap/config/$dirfile ]; then
            checkdir=`$grep "^LibDir" $TOPDIR/bootstrap/config/$dirfile | $awk ' { print $2 }'`
	 fi

	 if [ "$checkdir" != "" ]; then
            if [ -d $checkdir ]; then
               return 0
	    else
	       echo ""
	       echo "The global library directory"
	       echo ""
               echo "    $checkdir"
	       echo ""
               echo "does not exist as specified in the file"
	       echo ""
	       echo "    $TOPDIR/bootstrap/config/$dirfile."
	       echo ""
	       echo "These could be the problems:"
	       echo "   1. The file entry is wrong and needs updating."
	       echo "   2. The file should not exist for this toolbox."
	       echo "   3. Create a $dirfile file in the directory"
	       echo "         $toolboxtopdir/config"
               echo "      to point to the correct location"
	       echo "Please fix and re-run this script."
	       cleanup
            fi
         fi
      fi

      if [ "$checkdir" != "" ]; then
         checkdir=$TOPDIR/bootstrap/lib
         if [ -d $checkdir ]; then
	    return 0
         else
	    echo ""
	    echo "The library directory"
	    echo ""
            echo "    $checkdir"
	    echo ""
            echo "does not exist.  These could be the problems:"
	    echo "   1. The directory was deleted accidently."
	    echo "   2. $KHOROSVER not really installed on your system."
	    echo ""
	    echo "Please fix and re-run this script."
	    cleanup
         fi
      fi
   else
      checkdir=$toolboxtopdir/lib

      if [ -d $checkdir ]; then
	 return 0
      else
	 echo ""
	 echo "The library directory"
	 echo ""
         echo "    $checkdir"
	 echo ""
         echo "for $toolbox does not exist.  These could be the problems:"
	 echo "   1. The directory was deleted accidently."
	 echo "   2. The toolbox is not really installed on the system"
         echo "      and the entry should be deleted from the file"
	 echo ""
	 echo "    $TOPDIR/Toolboxes"
	 echo ""
	 echo "Please fix and re-run this script."
	 cleanup
      fi
   fi

   if [ "$checkdir" = "" ]; then
      echo "The library directory for $toolbox is undefined.  This should not"
      echo "have happened.  These could be the problems:"
      echo "   1. In the $TOPDIR/Toolboxes file"
      echo "      globaldir is set to true, on, yes or 1 and should be set to"
      echo "      false, off, no or 0."
      echo "   2. The $dirfile was deleted from the directory,"
      echo "          $TOPDIR/bootstrap/config"
      if [ "$toolbox" != "bootstrap" ]; then
         echo "      and/or the directory,"
         echo "          $toolboxtopdir/config."
      fi
      echo ""
      echo "Please fix and re-run this script."
      cleanup
   fi
}

#
# Get the global binary directory:
#	1. If the globaldir in the Toolboxes file is true, yes, on or 1 any case
#	   do,
#	   a. check for TOOLBOX/config/Dir.$defkconfig
#	   b. check for bootstrap/config/Dir.$defkconfig
#	   c. default to bootstrap/bin
#	2. use TOOLBOX/bin
#
GetBinDir()
{
   toolbox="$*"
   toolboxtocheck=`echo $toolbox | $tr "[a-z]" "[A-Z]"`

#
# Determine the name of the file that specifies the BinDir
#
   if [ "$KARCH" = "" ]; then
      dirfile=Dir.$defkconfig
   else
      dirfile=Dir.$KARCH
      if [ "$KOPSYS" != "" ]; then
         dirfile=Dir.$KARCH-$KOPSYS
         if [ "$KWIDSET" != "" ]; then
            dirfile=Dir.$KARCH-$KOPSYS-$KWIDSET
         fi
      fi
   fi
#
# Parse out the setting for globaldir from the $TOPDIR/Toolboxes file
#
   if [ -f $TOPDIR/Toolboxes ]; then
      ans=`$grep "^$toolboxtocheck" $TOPDIR/Toolboxes | $awk -F\; '{ print $2 }'`

      useglobaldir=false
      for entry in `echo $ans | $sed s/,/\ /g`; do

         case $entry in

            globaldir*) 
                useglobaldir=`echo $entry | $awk -F= '{ print $2 }'`
                break
                ;;
            *)
                useglobaldir=false
                ;;

         esac 
      done

      ans=`$grep "^$toolboxtocheck" $TOPDIR/Toolboxes | $awk -F\; '{ print $1 }'`
      toolboxtopdir=`echo $ans | $awk -F\: '{ print $2 }'`

      if [ "$toolboxtopdir" = "" ]; then 
	 return
      fi

      if [ ! -d "$toolboxtopdir" ]; then
         echo ""
         echo "The directory location for were the toolboxes $toolboxtocheck is located"
         echo "does not exist.  The entry in the $TOPDIR/Toolboxes"
         echo "can be removed since the toolbox does not seem to be installed."
         echo ""
         ContinuePrompt
	 return
      fi
   else
      echo ""
      echo "No $TOPDIR/Toolboxes to use."
      echo "This should not have happened, please mail khoros-bugs@khoral.com."
      echo ""
      cleanup
   fi

#
# Set the binary directory in the variable checkdir
#
   checkdir=""
   useglobaldir=`echo $useglobaldir | $tr "[A-Z]" "[a-z]"`

   if [ "$useglobaldir" = "true" -o "$useglobaldir" = "yes" -o "$useglobaldir" = "on" -o "$useglobaldir" = "1" ]; then

      if [ -d $toolboxtopdir/config ]; then
         if [ -f $toolboxtopdir/config/$dirfile ]; then
            checkdir=`$grep "^BinDir" $toolboxtopdir/config/$dirfile | $awk '{ print $2 }'`
         fi

	 if [ "$checkdir" != "" ]; then
            if [ -d $checkdir ]; then
               return 0
	    else
	       echo ""
	       echo "The binary directory"
	       echo ""
               echo "    $checkdir"
	       echo ""
               echo "for $toolbox does not exist as specified in the file"
	       echo ""
	       echo "    $toolboxtopdir/config/$dirfile."
	       echo ""
	       echo "These could be the problems:"
	       echo "   1. The file entry is wrong and needs updating."
	       echo "   2. The file should not exist for this toolbox."
	       echo "Please fix and re-run this script."
	       cleanup
            fi
         fi
#
# Check the bootstrap toolbox
#
      elif [ -d $TOPDIR/bootstrap/config ]; then
         if [ -f $TOPDIR/bootstrap/config/$dirfile ]; then
            checkdir=`$grep "^BinDir" $TOPDIR/bootstrap/config/$dirfile | $awk ' { print $2 }'`
         fi

	 if [ "$checkdir" != "" ]; then
            if [ -d $checkdir ]; then
               return 0
	    else
	       echo ""
	       echo "The global binary directory"
	       echo ""
               echo "    $checkdir"
	       echo ""
               echo "does not exist as specified in the file"
	       echo ""
	       echo "    $TOPDIR/bootstrap/config/$dirfile."
	       echo ""
	       echo "These could be the problems:"
	       echo "   1. The file entry is wrong and needs updating."
	       echo "   2. The file should not exist for this toolbox."
	       echo "   3. Create a $dirfile file in the directory"
	       echo "         $toolboxtopdir/config"
               echo "      to point to the correct location."
	       echo "Please fix and re-run this script."
	       cleanup
            fi
         fi
      fi

      if [ "$checkdir" != "" ]; then
         checkdir=$TOPDIR/bootstrap/bin
         if [ -d $checkdir ]; then
	    return 0
         else
	    echo ""
	    echo "The binary directory"
	    echo ""
            echo "    $checkdir"
	    echo ""
            echo "does not exist.  These could be the problems:"
	    echo "   1. The directory was deleted accidently."
	    echo "   2. $KHOROSVER not really installed on your system."
	    echo ""
	    echo "Please fix and re-run this script."
	    cleanup
         fi
      fi
   else
      checkdir=$toolboxtopdir/bin

      if [ -d $checkdir ]; then
	 return 0
      else
	 echo ""
	 echo "The binary directory"
	 echo ""
         echo "    $checkdir"
	 echo ""
         echo "for $toolbox does not exist.  These could be the problems:"
	 echo "   1. The directory was deleted accidently."
	 echo "   2. The toolbox is not really installed on the system"
         echo "      and the entry should be deleted from the file"
	 echo ""
	 echo "    $TOPDIR/Toolboxes"
	 echo ""
	 echo "Please fix and re-run this script."
	 cleanup
      fi
   fi

   if [ "$checkdir" = "" ]; then
      echo "The binary directory for $toolbox is undefined.  This should not"
      echo "have happened.  These could be the problems:"
      echo "   1. In the $TOPDIR/Toolboxes file"
      echo "      globaldir is set to true, on, yes or 1 and should be set to"
      echo "      false, off, no or 0."
      echo "   2. The $dirfile was deleted from the directory,"
      echo "          $TOPDIR/bootstrap/config"
      if [ "$toolbox" != "bootstrap" ]; then
         echo "      and/or the directory,"
         echo "          $toolboxtopdir/config."
      fi
      echo ""
      echo "Please fix and re-run this script."
      cleanup
   fi
}

#
# Check to make sure bootstrap is installed.  Check for all the libraries
# and for kgenmake and kgendepend.
#
Checkbootstrap()
{
   silent=$1

   GetLibDir bootstrap

   for lib in libkclui libkcms libkexpr libkgen libklibdb libklm libktu libku libkvf ; do
      if [ ! -r $checkdir/$lib.so.2.0 -a ! -r $checkdir/$lib.so -a ! -r $checkdir/$lib.a ]; then
         if [ $silent = 0 ]; then
	    echo ""
            echo "Can not find one or more of the libraries that should have been installed"
	    echo "for bootstrap."
	    echo ""
            ContinuePrompt
         fi
         return 0
      fi
   done

   GetBinDir bootstrap

   for binarytock in kgenmake kgendepend; do
      if [ ! -r $checkdir/$binarytock ]; then
         if [ $silent = 0 ]; then
	    echo ""
            echo "Can not find one or more of the binaries that should have been installed"
	    echo "for bootstrap."
	    echo ""
            ContinuePrompt
         fi
         return 0
      fi
   done

   return 1
}

#
# Check to make sure dataserv is installed.  Check for all the libraries.
#
Checkdataserv()
{
   silent=$1


   GetLibDir dataserv

   for lib in libkaps libkapu libkdac libkdms libkfmt libkgm libkog ; do
      if [ ! -r $checkdir/$lib.so.2.0 -a ! -r $checkdir/$lib.so -a ! -r $checkdir/$lib.a ]; then
         if [ $silent = 0 ]; then
	    echo ""
            echo "Can not find one or more of the libraries that should have been installed"
	    echo "for dataserv."
	    echo ""
            ContinuePrompt
         fi
         return 0
      fi
   done

   return 1
}

#
# Check to make sure design is installed.  Check for all the libraries.
#
Checkdesign()
{
   silent=$1

   GetLibDir design

   for lib in libkwid libxvf libxvg libxvl libxvobj libxvs libxvu libxvw ; do
      if [ ! -r $checkdir/$lib.so.2.0 -a ! -r $checkdir/$lib.so -a ! -r $checkdir/$lib.a ]; then
         if [ $silent = 0 ]; then
	    echo ""
            echo "Can not find one or more of the libraries that should have been installed"
	    echo "for design."
	    echo ""
            ContinuePrompt
         fi
         return 0
      fi
   done

   return 1
}

#
# Check to make sure datamanip is installed.  Check for all the libraries.
#
Checkdatamanip()
{
   silent=$1

   GetLibDir datamanip

   for lib in libksegops libkdmanip; do
      if [ ! -r $checkdir/$lib.so.2.0 -a ! -r $checkdir/$lib.so -a ! -r $checkdir/$lib.a ]; then
         if [ $silent = 0 ]; then
	    echo ""
            echo "Can not find one or more of the libraries that should have been installed"
	    echo "for datamanip."
	    echo ""
            ContinuePrompt
         fi
         return 0
      fi
   done

   return 1
}


#
# Get list of binaries for a toolbox, sets the variable binarylist.
#
GetBinaryList()
{
   tbtogetbinaries=$1

   if [ $havekecho = 0 ]; then
      echo ""
      echo "Got an error that should not have happened in GetBinaryList, missing the kecho"
      echo "routine to find all the binaries that are suppose to be installed for $tbtogetbinaries."
      echo "We can continue but you might not have the toolbox $tbname installed correctly"
      echo "which may mean the compiles of the $KHOROSVER system will not be successful."
      echo ""
      echo "Mail khoros-bugs@khoral.com to report this bug."
      echo ""
      ContinuePrompt
      return 0
   fi

   binarylist=""
   for ckobject in `kecho -tb $tbtogetbinaries -echo objects -type kroutine`; do
      binarylist="$binarylist `kecho -tb $tbtogetbinaries -oname $ckobject -echo bname`"
   done

   if [ $tbtogetbinaries != "design" ]; then
      for ckobject in `kecho -tb $tbtogetbinaries  -echo objects -type pane ` ; do 
         binarylist="$binarylist `kecho -tb $tbtogetbinaries -oname $ckobject -echo bname`"
      done
   fi

   for ckobject in `kecho -tb $tbtogetbinaries  -echo objects -type xvroutine`; do
      binarylist="$binarylist `kecho -tb $tbtogetbinaries -oname $ckobject -echo bname`"
   done

   for ckobject in `kecho -tb $tbtogetbinaries  -echo objects -type script`; do
      if [ $ckobject != "khoros_pl" -a $ckobject != "kutils_pl" ]; then
         binarylist="$binarylist `kecho -tb $tbtogetbinaries -oname $ckobject -echo bname`"
      fi
   done

   return 1
}

#
# Get list of libraries for a toolbox, sets the variable librarylist.
#
GetLibraryList()
{
   tbtogetlibraries=$1

   if [ $havekecho = 0 ]; then
      echo ""
      echo "Got an error that should not have happened in GetLibraryList, missing the kecho"
      echo "routine to find all the binaries that are suppose to be installed for $tbtogetlibraries."
      echo "We can continue but you might not have the toolbox $tbname installed correctly"
      echo "which may mean the compiles of the $KHOROSVER system will not be successful."
      echo ""
      echo "Mail khoros-bugs@khoral.com to report this bug."
      echo ""
      ContinuePrompt
      return 0
   fi

   librarylist=""
   for ckobject in `kecho -tb $tbtogetlibraries -echo objects -type library`; do
      librarylist="$librarylist `kecho -tb $tbtogetlibraries -oname $ckobject -echo bname`"
   done

   return 1
}

#
# Check to see if all the binaries installed for a toolbox
#
Checkbinaries()
{
   tbtocheckbin=$1
   silent=$2

   GetBinaryList $tbtocheckbin
   status=$?

#
# Ignore the error for this case.  The users has been informed, but we can
# continue running kbuild in this case so return success.
#
   if [ $status = 0 ]; then
      return 1
   fi

   GetBinDir $tbtocheckbin

   for binarytock in $binarylist ; do
      if [ ! -r $checkdir/$binarytock ]; then
         if [ $silent = 0 ]; then
	    echo ""
            echo "Can not find the binary $binarytock that should have been installed"
	    echo "for $tbtocheckbin."
	    echo ""
            ContinuePrompt
         fi
         return 0
      fi
   done

   return 1
}

#
# Check to see if all the libraries installed for a toolbox
#
Checklibraries()
{
   tbtochecklib=$1
   silent=$2

   GetLibraryList $tbtochecklib
   status=$?

#
# Ignore the error for this case.  The users has been informed, but we can
# continue running kbuild in this case so return success.
#
   if [ $status = 0 ]; then
      return 1
   fi

   GetLibDir $tbtochecklib

   for libtock in $librarylist ; do
      if [ ! -r $checkdir/lib$libtock.so.2.0 -a ! -r $checkdir/lib$libtock.so -a ! -r $checkdir/lib$libtock.a ]; then
         if [ $silent = 0 ]; then
	    echo ""
            echo "Can not find one or more of the libraries that should have been installed"
	    echo "for $tbtochecklib."
	    echo ""
            ContinuePrompt
         fi
         return 0
      fi
   done

   return 1
}

#
# Check to see if a toolbox was installed correctly
#
CheckTB()
{

   tbtocheck=$1

   if [ "$tbtocheck" = "" ]; then
      echo ""
      echo "Got an error that should not have happened in CheckTB, missing the toolbox"
      echo "to check to see if it is installed correctly.  We can continue but you might"
      echo "not have the toolbox $tbname installed correctly."
      echo ""
      echo "Mail khoros-bugs@khoral.com to report this bug."
      echo ""
      ContinuePrompt
      return 1
   fi

   if [ $havekecho = 0 -a "$tbtocheck" != "bootstrap" ]; then
      GetBinDir bootstrap

      if [ ! -r $checkdir/kecho ]; then
	 echo ""
         echo "Can not find the binary kecho that should have been installed"
	 echo "for bootstrap."
	 echo ""
         ContinuePrompt
         return 0
      fi
      havekecho=1
   fi

   if [ "$tbtocheck" = "bootstrap" ]; then
      Checkbootstrap 0
      status=$?
      if [ $status = 0 ]; then
         return 0
      fi

      GetBinDir bootstrap

      for binarytock in conductor ghostwriter kcp kexec kgenimake kgenobj kgentb kman krm kset kgenmanual krename kecho kgenmake kgendepend kgenstruct phantomd kconfigure kgenindex kgenglossary klicense kbugreport kbuild kversion ; do
         if [ ! -r $checkdir/$binarytock ]; then
	    echo ""
            echo "Can not find the binary $binarytock that should have been installed"
	    echo "for bootstrap."
	    echo ""
            ContinuePrompt
            return 0
         fi
      done
      havekecho=1
   elif [ "$tbtocheck" = "dataserv" ]; then
      Checkdataserv 0
      status=$?
      if [ $status = 0 ]; then
         return 0
      fi
      Checkbinaries dataserv 0
      status=$?
      if [ $status = 0 ]; then
         return 0
      fi
   elif [ "$tbtocheck" = "design" ]; then
      Checkdesign 0
      status=$?
      if [ $status = 0 ]; then
         return 0
      fi
      Checkbinaries design 0
      status=$?
      if [ $status = 0 ]; then
         return 0
      fi
   elif [ "$tbtocheck" = "datamanip" ]; then
      Checkdatamanip 0
      status=$?
      if [ $status = 0 ]; then
         return 0
      fi
      Checkbinaries datamanip 0
      status=$?
      if [ $status = 0 ]; then
         return 0
      fi
   else
      Checklibraries $tbtocheck 0
      status=$?
      if [ $status = 0 ]; then
         return 0
      fi
      Checkbinaries $tbtocheck 0
      status=$?
      if [ $status = 0 ]; then
         return 0
      fi
   fi

   return 1
}

#
# Compile the kgenmake program.
#
Compilekgenmake()
{
   newtopic
   if [ -x $bindir/kgenmake ]; then
      echo ""
      echo "It appears that the bootstrap program is compiled and installed."
      echo ""
      YesNoPrompt "Do you wish to rebuild the kgenmake program?" "n"
      if [ "$?" -eq "0" ]; then
         [ -f $bindir/kgenmake ] && $rm $bindir/kgenmake
      else
	 return 0
      fi
   fi

   cd $TOPDIR/bootstrap/objects

   echo ""
   echo "Ok, we're ready to roll."
   echo ""

   echo ""
   echo "The first stage of compilation is the 'bootstrap' program:"
   echo "      kgenmake"
   echo ""
   ConfirmContinue

   echo ""
   echo "Ok, let's try and build the bootstrap program."
   echo "If the bootstrap build fails, then you must determine what"
   echo "happened and run kbuild again:"
   echo ""
   make CC=$ccomp BOOTSTRAPCFLAGS="-DKARCH_$defarch -DKOPSYS_$defopsys -DKCONFIG_DEFINE=\\\\\\\"$defkconfig\\\\\\\"" bootstrap
   if [ -x $bindir/kgenmake ]; then
      echo ""
      echo "The bootstrap program seemed to build and"
      echo "install correctly, so let's go try to build Khoros."
      echo ""
   else
      echo ""
      echo "The bootstrap program did not build correctly."
      echo "Please check the platform configuration file or"
      echo "consult the installation guide and re-run kbuild."
      echo ""
      cleanup
   fi

   ContinuePrompt
}

# Search for the fortran command
SearchForFortran()
{
   if [ "$#" -eq 0 ]; then
      return 1
   fi

   fortrancmd="$*"

   newtopic
   echo ""
   echo "Searching for Fortran command $fortrancmd ..."
   echo ""

#
# Search for fortran in the current path
#
   compfcmd=""
   for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
      [ -x "$dir/$fortrancmd" ] && compfcmd="$dir/$fortrancmd"
   done

#
# If fortran not found, let them re-enter the fortran command or the toolboxes
# to install.  Set HasFortran, Usef2c and Useg77 to NO in the Site.XXX file.
#
   if [ "$compfcmd" = "" ]; then
      echo ""
      echo "Cannot find Fortran compiler $fortrancmd in your path."
      echon "Cannot install the "
      if [ $matrixinst = 1 ]; then
         echon "matrix"
      fi
      
      if [ $matrixinst = 1 -a $geometryinst = 1 ]; then
         echon " and "
      fi

      if [ $geometryinst = 1 ]; then
         echon "geometry"
      fi

      echo "toolbox(es) if you do not have a Fortran compiler."
      echo ""

      cmd='{ if( $1 == "#define" ) { if ($2 == "HasFortran") { if ($3 != "") printf("%s %s 'NO'\n", $1, $2) } else print $0 } else { print $0 } }'

      $awk " $cmd " $SITE_FILE > $SITE_FILE.new
      $mv $SITE_FILE.new $SITE_FILE

      cmd='{ if( $1 == "#define" ) { if ($2 == "Usef2c") { if ($3 != "") printf("%s %s 'NO'\n", $1, $2) } else print $0 } else { print $0 } }'

      $awk " $cmd " $SITE_FILE > $SITE_FILE.new
      $mv $SITE_FILE.new $SITE_FILE

      cmd='{ if( $1 == "#define" ) { if ($2 == "Useg77") { if ($3 != "") printf("%s %s 'NO'\n", $1, $2) } else print $0 } else { print $0 } }'

      $awk " $cmd " $SITE_FILE > $SITE_FILE.new
      $mv $SITE_FILE.new $SITE_FILE

      echo ""
      echon "Do you want to change the name of the fortran compiler? (y/n) [y]: "
      read bot

      if [ "$bot" = "n" -o "$bot" = "N" ]; then
         echo ""
	 echo "Need to re-enter the toolboxes to install."
         echo ""
	 return 2
      else
         newtopic
	 return 1
      fi
   else
#
# If fortran found, set HasFortran to YES and determine if it is f2c or g77
# and set the Site.XXX file accordingly.
#
      echo "Found the Fortran command $compfcmd.  Will configure the file"
      echo ""
      echo "     $SITE_FILE"
      echo ""
      echo "to reflect the fact that you have a Fortran compiler, i.e. changing"
      echo "HasFortran to YES".

      cmd='{ if( $1 == "#define" ) { if ($2 == "HasFortran") { if ($3 != "") printf("%s %s 'YES'\n", $1, $2) } else print $0 } else { print $0 } }'

      $awk " $cmd " $SITE_FILE > $SITE_FILE.new
      $mv $SITE_FILE.new $SITE_FILE

#
# If f2c, set Usef2c to YES in the Site.XXX file.
#
      if [ $fortrancmd = "f2c" -o \( $fortrancmd = "fort77" -a "$kopsysguess" = "linux" \) ]; then
         if [ $fortrancmd = "fort77" -a "$kopsysguess" = "linux" ]; then
            echo "Since, the Fortran command is fort77 (really f2c), will configure the file"
         else
            echo "Since, the Fortran command is f2c, will configure the file"
         fi
         echo ""
         echo "     $SITE_FILE"
         echo ""
         echo "to reflect the fact that you are using the f2c program, i.e. changing"
         echo "Usef2c to YES".
         echo ""

         cmd='{ if( $1 == "#define" ) { if ($2 == "Usef2c") { if ($3 != "") printf("%s %s 'YES'\n", $1, $2) } else print $0 } else { print $0 } }'

         $awk " $cmd " $SITE_FILE > $SITE_FILE.new
         $mv $SITE_FILE.new $SITE_FILE
      else
         echo ""
         echo "Making sure the Fortran command specified in the configuration file"
         echo ""
         echo "     $CONFIG_FILE"
         echo ""
         echo "is $fortrancmd."
         echo ""

         cmd='{ if( $1 == "#" ) { if ($2 == "define") { if ($3 == "FortranCmd") { if( $4 != "" ) printf("%s  %s %s '$fortrancmd'\n", $1, $2, $3) } else print $0 } else print $0 } else { print $0 } }'

         $awk " $cmd " $CONFIG_FILE > $CONFIG_FILE.new
         $mv $CONFIG_FILE.new $CONFIG_FILE

         cmd='{ if( $1 == "#define" ) { if ($2 == "Usef2c") { if ($3 != "") printf("%s %s 'NO'\n", $1, $2) } else print $0 } else { print $0 } }'

         $awk " $cmd " $SITE_FILE > $SITE_FILE.new
         $mv $SITE_FILE.new $SITE_FILE
      fi

#
# Check to make sure the Fortran compiler is not GNU's f77.  The installer
# can name the binary f77.
#
      case `$fortrancmd --version 2>/dev/null` in
	 2*) 
             GNUFORTRAN=1
             ;;
	 *)
             GNUFORTRAN=0
             ;;
      esac 

#
# If GNU's f77, set Useg77 to YES in the Site.XXX file.
#
      if [ $GNUFORTRAN = 1 -o $fortrancmd = "g77" ]; then
	 version=`$fortrancmd --version 2>/dev/null`
         echo "Since the Fortran command is GNU's f77 (ver. $version), will"
         echo "configure the"
         echo ""
         echo "     $SITE_FILE"
         echo ""
         echo "to reflect the fact that you are using GNU's f77 compiler, i.e. changing"
         echo "Useg77 to YES".
         echo ""

         cmd='{ if( $1 == "#define" ) { if ($2 == "Useg77") { if ($3 != "") printf("%s %s 'YES'\n", $1, $2) } else print $0 } else { print $0 } }'

         $awk " $cmd " $SITE_FILE > $SITE_FILE.new
         $mv $SITE_FILE.new $SITE_FILE
      else
         cmd='{ if( $1 == "#define" ) { if ($2 == "Useg77") { if ($3 != "") printf("%s %s 'NO'\n", $1, $2) } else print $0 } else { print $0 } }'

         $awk " $cmd " $SITE_FILE > $SITE_FILE.new
         $mv $SITE_FILE.new $SITE_FILE
      fi
   fi

   ContinuePrompt
   return 0
}

# split config.guess machine-vendor-OS 
SplitGuess()
{
   if [ "$#" -eq 0 ]; then
      karchguess="none"
      kvendor="none"
      kopsysguess="none"
      return 0
   fi

   index=0
   for elements in `echo $1 | $sed -e 's/-/ /g'`; do
      if [ $index -eq 0 ]; then
         karchguess=$elements
      fi

      if [ $index -eq 1 ]; then
         kvendor=$elements
      fi

#
# Clean up the kopsysguess string to get rid of the actual numbers and for
# DGUX the R.  Can not use [0-9.R] as string1 to tr since DGUX does not like
# it.
#
      if [ $index -eq 2 ]; then
         kopsysguess=`echo $elements | $tr -d "0123456789.R"`
      fi

      index=`expr $index + 1`
   done

#
# Make a distinction between the Little Endian DECStation Mips.  All other
# Mips boxes are Big Endian, right ???
#
   if [ "$karchguess" = "mips" -a "$kvendor" = "dec" ]; then
      karchguess="mipsel"
   fi

#
# Just use convex and not c32, c2 and cX.
#
   if [ "$kvendor" = "convex" ]; then
      karchguess="convex"
   fi

#
# No idea what apollo really returns so use this.  I think however the
# early m68k apollo's will not be indicated correctly, but both the
# DN10000 and m68k's are Big Endian IEEE so nothing is broken.
#
   if [ "$kvendor" = "apollo" ]; then
      karchguess="apollo"
   fi

   return 0
}

# Make sure the list of toolboxes has dependencies correct
CheckDependencies()
{
   if [ "$#" -eq 0 ]; then
      echo ""
      echo "No input provided for the CheckDependencies function in kbuild."
      echo "This should not have happened, please mail khoros-bugs@khoral.com."
      echo ""
      cleanup
   fi

   listtbtoinstall="$*"

#
# Set up flags to make sure:
#   a valid toolbox was specified
#   if fortran is needed it is found
#   no unknown toolboxes are specified
#   no duplicate toolboxes are specified
#
   tbspecified=0
   needfortran=0
   unknown=0
   duplicate=0
#
# Set up flags to handle dependencies
#
   bootstrapdep=1
   dataservdep=1
   designdep=1
   datamanipdep=1
   bootstrapinstalled=0
   dataservinstalled=0
   designinstalled=0
   datamanipinstalled=0

#
# Set up flags to make sure there are no duplicates in the listtbtoinstall
# and that all the dependencies are taken care of.
#
   bootstrapinst=0
   dataservinst=0
   designinst=0
   datamanipinst=0
   envisioninst=0
   geometryinst=0
   imageinst=0
   matrixinst=0
   migrationinst=0
   supportinst=0
   retroinst=0

#
# Check on install status of the three required toolboxes bootstrap, dataserv
# and design
#
   Checkbootstrap 1
   bootstrapinstalled=$?
   Checkdataserv 1
   dataservinstalled=$?
   Checkdesign 1
   designinstalled=$?

#
# For each of the toolboxes being installed set its install flag and the
# dependency flags for the toolbox it depends on.  Look at case 5 for
# comments on what is happening for each case.
#
   for tb in $listtbtoinstall; do

      case $tb in

         1) tbspecified=1
            if [ $bootstrapinst = 0 ] ; then
                bootstrapinst=1
            else
                duplicate=1
            fi
	    ;;
         2) tbspecified=1
            if [ $dataservinst = 0 ] ; then
                dataservinst=1
            else
                duplicate=1
            fi
            if [ $bootstrapinst = 0 ]; then
               bootstrapdep=0
            fi
	    ;;
         3) tbspecified=1
            if [ $designinst = 0 ] ; then
                designinst=1
            else
                duplicate=1
            fi
            if [ $bootstrapinst = 0 ]; then
               bootstrapdep=0
            fi
            if [ $dataservinst = 0 ]; then
               dataservdep=0
            fi
	    ;;
         4) tbspecified=1
	    if [ $datamanipinst = 0 ] ; then
               datamanipinst=1
            else
               duplicate=1
            fi
            if [ $bootstrapinst = 0 ]; then
               bootstrapdep=0
            fi
            if [ $dataservinst = 0 ]; then
               dataservdep=0
            fi
            if [ $designinst = 0 ]; then
               designdep=0
            fi
            ;;
         5) tbspecified=1
#
# Determine if the entry has been entered more than once in the list of
# toolboxes to install
#
            if [ $envisioninst = 0 ] ; then
                envisioninst=1
            else
                duplicate=1
            fi
#
# Check the dependencies, i.e. toolboxes this toolbox needs to compile against
# are in the install list before this one.
#
# Make sure that the bootstrap was put on the list before this toolbox
#
            if [ $bootstrapinst = 0 ]; then
               bootstrapdep=0
            fi
#
# Make sure that the dataserv was put on the list before this toolbox
#
            if [ $dataservinst = 0 ]; then
               dataservdep=0
            fi
#
# Make sure that the design was put on the list before this toolbox
#
            if [ $designinst = 0 ]; then
               designdep=0
            fi
#
# Make sure that the datamanip was put on the list before this toolbox
#
            if [ $datamanipinst = 0 ]; then
               datamanipdep=0
            fi
#
# Check on install status of datamanip
#
            Checkdatamanip 1
            datamanipinstalled=$?
            ;;
         6) tbspecified=1
            needfortran=1
            if [ $geometryinst = 0 ] ; then
                geometryinst=1
            else
                duplicate=1
            fi
            if [ $bootstrapinst = 0 ]; then
               bootstrapdep=0
            fi
            if [ $dataservinst = 0 ]; then
               dataservdep=0
            fi
            if [ $designinst = 0 ]; then
               designdep=0
            fi
            ;;
         7) tbspecified=1
            if [ $imageinst = 0 ] ; then
                imageinst=1
            else
                duplicate=1
            fi
            if [ $bootstrapinst = 0 ]; then
               bootstrapdep=0
            fi
            if [ $dataservinst = 0 ]; then
               dataservdep=0
            fi
            if [ $designinst = 0 ]; then
               designdep=0
            fi
            if [ $datamanipinst = 0 ]; then
               datamanipdep=0
            fi
            Checkdatamanip 1
            datamanipinstalled=$?
            ;;
         8) tbspecified=1
            needfortran=1
            if [ $matrixinst = 0 ] ; then
                matrixinst=1
            else
                duplicate=1
            fi
            if [ $bootstrapinst = 0 ]; then
               bootstrapdep=0
            fi
            if [ $dataservinst = 0 ]; then
               dataservdep=0
            fi
            if [ $designinst = 0 ]; then
               designdep=0
            fi
            ;;
         9) tbspecified=1
            if [ $migrationinst = 0 ] ; then
                migrationinst=1
            else
               duplicate=1
            fi
            if [ $bootstrapinst = 0 ]; then
               bootstrapdep=0
            fi
            if [ $dataservinst = 0 ]; then
               dataservdep=0
            fi
            if [ $designinst = 0 ]; then
               designdep=0
            fi
            ;;
        10) tbspecified=1
            if [ $supportinst = 0 ] ; then
                supportinst=1
            else
                duplicate=1
            fi
            if [ $bootstrapinst = 0 ]; then
               bootstrapdep=0
            fi
            if [ $dataservinst = 0 ]; then
               dataservdep=0
            fi
            if [ $designinst = 0 ]; then
               designdep=0
            fi
            ;;
        11) tbspecified=1
            if [ $retroinst = 0 ] ; then
                retroinst=1
            else
                duplicate=1
            fi
            if [ $bootstrapinst = 0 ]; then
               bootstrapdep=0
            fi
            if [ $dataservinst = 0 ]; then
               dataservdep=0
            fi
            if [ $designinst = 0 ]; then
               designdep=0
            fi
            ;;
         *)
            unknown=1
            ;;
      esac

   done

#
# If no toolbox specified or an entry that is not understood.
#
   if [ $unknown = 1 -o $tbspecified = 0 ]; then
      echo ""
      echo "Invalid or unknown entry, please re-enter."
      echo ""
      return 1
   fi

#
# If the same toolbox is entered twice.
#
   if [ $duplicate = 1 ]; then
      echo ""
      echo "Duplicate entries made, please re-enter."
      echo ""
      return 1
   fi

#
# If you need fortran because geometry and matrix being installed.
#
   while [ $needfortran = 1 ]; do
      newtopic
      echo ""
      echo "You have selected the matrix and/or geometry toolbox.   To install "
      echo "these toolboxes you need a Fortran compiler or f2c."
      echo ""
      echon "What is the name of the Fortran compiler to use [$fortrancmd] ?"
      read userfcmd

      if [ "$userfcmd" != "" ]; then
         fortrancmd=`echo $userfcmd`
      fi

      SearchForFortran $fortrancmd
      needfortran=$?
      if [ $needfortran = 2 ]; then
         return 1
      fi
   done

#
# Check the dependencies for the datamanip toolbox.
#
# First, make sure that if it is being installed that it is before those
# toolboxes that depend on it.
#
   if [ $datamanipinst = 1 -a $datamanipdep = 0 ]; then
      echo ""
      echo "datamanip must be installed before envision and/or image,"
      echo "re-ordering list so this will happen."
      echo ""

      newtbtoinstall=`echo $listtbtoinstall | $sed s/4//`
      listtbtoinstall=`echo "4 " $newtbtoinstall`
      ContinuePrompt

#
# Make sure bootstrap, dataserv and design are installed before datamanip if
# the order of the toolboxes being installed has been re-ordered.
#
      if [ $designinst = 1 ]; then
         newtbtoinstall=`echo $listtbtoinstall | $sed s/3//`
         listtbtoinstall=`echo "3 " $newtbtoinstall`
      fi

      if [ $dataservinst = 1 ]; then
         newtbtoinstall=`echo $listtbtoinstall | $sed s/2//`
         listtbtoinstall=`echo "2 " $newtbtoinstall`
      fi

      if [ $bootstrapinst = 1 ]; then
         newtbtoinstall=`echo $listtbtoinstall | $sed s/1//`
         listtbtoinstall=`echo "1 " $newtbtoinstall`
      fi

   elif [ $datamanipinstalled = 0 -a $datamanipinst = 0 -a $datamanipdep = 0 ]; then
#
# Second, if it is not being installed and it has not been installed already.
#
      echo ""
      echo "datamanip must be installed if envision and/or image is to be"
      YesNoPrompt "installed do you want to add datamanip to the list?" "y"
      if [ "$?" -eq 0 ]; then
         newtbtoinstall=`echo "4 " $listtbtoinstall`
         listtbtoinstall=`echo $newtbtoinstall`
#
# Make sure bootstrap, dataserv and design are installed before datamanip if
# the order of the toolboxes being installed has been re-ordered.
#
         if [ $designinst = 1 ]; then
            newtbtoinstall=`echo $listtbtoinstall | $sed s/3//`
            listtbtoinstall=`echo "3 " $newtbtoinstall`
         fi

         if [ $dataservinst = 1 ]; then
            newtbtoinstall=`echo $listtbtoinstall | $sed s/2//`
            listtbtoinstall=`echo "2 " $newtbtoinstall`
         fi

         if [ $bootstrapinst = 1 ]; then
            newtbtoinstall=`echo $listtbtoinstall | $sed s/1//`
            listtbtoinstall=`echo "1 " $newtbtoinstall`
         fi

      else
         echo ""
	 echo "Need to re-enter the toolboxes to install."
         echo ""
         return 1
      fi
   fi

#
# Check the dependencies for the design toolbox.
#
# First, make sure that if it is being installed that it is before those
# toolboxes that depend on it.
#
   if [ $designinst = 1 -a $designdep = 0 ]; then
      echo ""
      echo "design must be installed before any other toolbox except dataserv"
      echo "and bootstrap, re-ordering list so this will happen."
      echo ""

      newtbtoinstall=`echo $listtbtoinstall | $sed s/3//`
      listtbtoinstall=`echo "3 " $newtbtoinstall`
      ContinuePrompt

#
# Make sure bootstrap and dataserv are installed before design if
# the order of the toolboxes being installed has been re-ordered.
#
      if [ $dataservinst = 1 ]; then
         newtbtoinstall=`echo $listtbtoinstall | $sed s/2//`
         listtbtoinstall=`echo "2 " $newtbtoinstall`
      fi

      if [ $bootstrapinst = 1 ]; then
         newtbtoinstall=`echo $listtbtoinstall | $sed s/1//`
         listtbtoinstall=`echo "1 " $newtbtoinstall`
      fi

   elif [ $designinstalled = 0 -a $designinst = 0 -a $designdep = 0 ]; then
#
# Second, if it is not being installed and it has not been installed already.
#
      echo ""
      echo "design must be installed if any other toolbox is to be installed except"
      YesNoPrompt "dataserv and bootstrap, do you want to add design to the list?" "y"
      if [ "$?" -eq 0 ]; then
         newtbtoinstall=`echo "3 " $listtbtoinstall`
         listtbtoinstall=`echo $newtbtoinstall`
#
# Make sure bootstrap and dataserv are installed before design if
# the order of the toolboxes being installed has been re-ordered.
#
         if [ $dataservinst = 1 ]; then
            newtbtoinstall=`echo $listtbtoinstall | $sed s/2//`
            listtbtoinstall=`echo "2 " $newtbtoinstall`
         fi

         if [ $bootstrapinst = 1 ]; then
            newtbtoinstall=`echo $listtbtoinstall | $sed s/1//`
            listtbtoinstall=`echo "1 " $newtbtoinstall`
         fi

      else
         echo ""
	 echo "Need to re-enter the toolboxes to install."
         echo ""
         return 1
      fi
   fi

#
# Check the dependencies for the dataserv toolbox.
#
# First, make sure that if it is being installed that it is before those
# toolboxes that depend on it.
#
   if [ $dataservinst = 1 -a $dataservdep = 0 ]; then
      echo ""
      echo "dataserv must be installed before any other toolbox except bootstrap,"
      echo "re-ordering list so this will happen."
      echo ""

      newtbtoinstall=`echo $listtbtoinstall | $sed s/2//`
      listtbtoinstall=`echo "2 " $newtbtoinstall`
      ContinuePrompt
#
# Make sure bootstrap is installed before design if the order of the
# toolboxes being installed has been re-ordered.
#
      if [ $bootstrapinst = 1 ]; then
         newtbtoinstall=`echo $listtbtoinstall | $sed s/1//`
         listtbtoinstall=`echo "1 " $newtbtoinstall`
      fi

   elif [ $dataservinstalled = 0 -a $dataservinst = 0 -a $dataservdep = 0 ]; then
#
# Second, if it is not being installed and it has not been installed already.
#
      echo ""
      echo "dataserv must be installed if any other toolbox is to be installed except"
      YesNoPrompt "bootstrap, do you want to add dataserv to the list?" "y"
      if [ "$?" -eq 0 ]; then
         newtbtoinstall=`echo "2 " $listtbtoinstall`
         listtbtoinstall=`echo $newtbtoinstall`
#
# Make sure bootstrap is installed before design if the order of the
# toolboxes being installed has been re-ordered.
#
         if [ $bootstrapinst = 1 ]; then
            newtbtoinstall=`echo $listtbtoinstall | $sed s/1//`
            listtbtoinstall=`echo "1 " $newtbtoinstall`
         fi

      else
         echo ""
	 echo "Need to re-enter the toolboxes to install."
         echo ""
         return 1
      fi
   fi

#
# Check the dependencies for the bootstrap toolbox.
#
# First, make sure that if it is being installed that it is before those
# toolboxes that depend on it.
#
   if [ $bootstrapinst = 1 -a $bootstrapdep = 0 ]; then
      echo ""
      echo "bootstrap must be installed before any other toolbox is installed,"
      echo "re-ordering list so this will happen."
      echo ""

      newtbtoinstall=`echo $listtbtoinstall | $sed s/1//`
      listtbtoinstall=`echo "1 " $newtbtoinstall`
      ContinuePrompt
   elif [ $bootstrapinstalled = 0 -a $bootstrapinst = 0 -a $bootstrapdep = 0 ]; then
#
# Second, if it is not being installed and it has not been installed already.
#
      echo ""
      echo "bootstrap must be installed if any other toolbox is to be installed,"
      YesNoPrompt "do you want to add bootstrap to the list?" "y"
      if [ "$?" -eq 0 ]; then
         newtbtoinstall=`echo "1 " $listtbtoinstall`
         listtbtoinstall=`echo $newtbtoinstall`
      else
         echo ""
	 echo "Need to re-enter the toolboxes to install."
         echo ""
         return 1
      fi
   fi

#
# Copy the potentially re-ordered list back to original
#
   tbtoinstall=`echo $listtbtoinstall`

   return 0
}

# # ========================================================================
# #
# # compile Khoros!
# #
# # ========================================================================
CompileKhoros()
{
#
# Get the list of toolboxes to compile.
#
needtbs=1
while [ $needtbs = 1 ]; do
   newtopic
   needtbs=0

   echo "                       Toolbox Selection"
   echo ""
   echo "The first three toolboxes are required by all the others.  So please"
   echo "select them first if they have not already been installed.  If not"
   echo "specified and not installed they will be added.  You may install "
   echo "multiple toolboxes by listing their corresponding numbers, separated"
   echo "by spaces."
   echo ""
   echo "      Toolboxes   Dependent on               Description"
   echo "   1. bootstrap                              Foundation Services"
   echo "   2. dataserv    bootstrap                  Data Services"
   echo "   3. design      bootstrap dataserv         GUI & Viz Services"
   echo "   4. datamanip   bootstrap dataserv design  Data Manipulation Operators"
   echo "   5. envision    datamanip                  Data Visualization Applications"
   echo "   6. geometry    bootstrap dataserv design  3D visualization (need Fortran)"
   echo "   7. image       datamanip                  Multiband Image Operators"
   echo "   8. matrix      bootstrap dataserv design  Matrix Operators (need Fortran)"
   echo "   9. migration   bootstrap dataserv design  Migration Tools K1 to K2 & K2 to K2"
   echo "  10. support     bootstrap dataserv design  Miscellaneous Tools"
   echo "  11. retro       bootstrap dataserv design  Programs from Khoros 1"
   echo "   N. None"
   echo "   A. All"
   echo "   P. Print description of toolbox"
   echo "   Q. Quit kbuild"
   echo ""

   echon "Enter the number(s) of the corresponding toolbox(es) [1 2 3 4 5 10]: "
   read tbtoinstall

   if [ "$tbtoinstall" = "" ]; then
      tbtoinstall="1 2 3 4 5 10"
   fi

   if [ "$tbtoinstall" = "Q" -o "$tbtoinstall" = "q" -o "$tbtoinstall" = "quit" -o "$tbtoinstall" = "Quit" ]; then
      ConfirmContinue
      needtbs=1
   else
      if [ "$tbtoinstall" != "N" -a "$tbtoinstall" != "n" -a "$tbtoinstall" != "None" -a "$tbtoinstall" != "none" -a "$tbtoinstall" != "P" -a "$tbtoinstall" != "p" -a "$tbtoinstall" != "Print" -a "$tbtoinstall" != "print" ]; then
         if [ "$tbtoinstall" = "A" -o "$tbtoinstall" = "a" -o "$tbtoinstall" = "All" -o "$tbtoinstall" = "all" ]; then
            tbtoinstall="1 2 3 4 5 6 7 8 9 10 11"
         fi
#
# Check the list to make sure it is correct with respect to dependecies
#
	 echo ""
         echo "Checking dependencies of the toolboxes requested, this could take a while ..."
         CheckDependencies $tbtoinstall
         checked=$?

         if [ $checked != 0 ]; then
            needtbs=1
            ContinuePrompt
         fi
      fi

#
# Print out ToolboxInfo files if P specified
#
      if [ "$tbtoinstall" = "P" -o "$tbtoinstall" = "p" -o "$tbtoinstall" = "Print" -o "$tbtoinstall" = "print" ]; then
         PrintDescription
         needtbs=1
      elif [ "$tbtoinstall" = "N" -o "$tbtoinstall" = "n" -o "$tbtoinstall" = "None" -o "$tbtoinstall" = "none" ]; then
         echon "No toolboxes to install, is this correct? (y/n) [y]: "
         read bot

         if [ "$bot" = "n" -o "$bot" = "N" ]; then
            needtbs=1
         fi
	 tbtoinstall=""
      elif [ $needtbs != 1 ]; then
#
# Print out toolboxes being installed and make sure these are the ones wanted
#
         newtopic 

         echo "Here are the toolboxes to be installed:"

         for tb in $tbtoinstall; do

            case $tb in

               1)
	          echo "     bootstrap"
                  ;;
               2)
	          echo "     dataserv"
                  ;;
               3)
	          echo "     design"
                  ;;
               4)
	          echo "     datamanip"
                  ;;
               5)
	          echo "     envision"
                  ;;
               6)
	          echo "     geometry"
                  ;;
               7)
	          echo "     image"
                  ;;
               8)
	          echo "     matrix"
                  ;;
               9)
	          echo "     migration"
                  ;;
               10)
	          echo "     support"
                  ;;
               11)
	          echo "     retro"
                  ;;
            esac

         done

         echon "Is this correct? (y/n) [y]: "
         read bot

         if [ "$bot" = "n" -o "$bot" = "N" ]; then
            needtbs=1
         fi

      fi
   fi

done

if [ "$tbtoinstall" = "" ]; then
   return 0
fi

# # ========================================================================
# #
# # Edit the configuration files
# #
# # ========================================================================
newtopic
echo "                        Architecture Configuration File"
echo ""
if [ $CP_SITE = 1 ]; then
   echo ""
   echo "                $SITE_FILE"
   echo ""
   echo "      A template architecture configuration file has been installed"
   echo ""
   echo "      You will need to edit this architecture configuration file"
   echo "      and set the imake symbols to the correct setting for your"
   echo "      architecture"
else
   echo ""
   echo "                $SITE_FILE"
   echo ""
   echo "      A architecture configuration file already exists."
   echo "      If you have not already checked this file for"
   echo "      correctness, you might want to look it over."
fi

echo ""
echo ""
YesNoPrompt "Do you wish to edit the architecture configuration file?"
if [ "$?" -eq "0" ]; then
   EDITOR=${EDITOR:-vi}
   $EDITOR $SITE_FILE
fi


newtopic
echo "                        Compiler Configuration File"
echo ""
if [ $CP_CONFIG = 1 ]; then
   $cp $DIST_CONFIG/$defname.cf $CONFIG_FILE
   echo ""
   echo "      A template platform configuration file has been installed"
   echo ""
   echo "                $CONFIG_FILE"
   echo ""
   echo "      You will need to edit this platform configuration file and"
   echo "      set the imake symbols to the correct setting for your"
   echo "      architecture."
else
   echo ""
   echo "                $CONFIG_FILE"
   echo ""
   echo "      A platform configuration file already exists."
   echo "      If you have not already checked this file for"
   echo "      correctness, you might want to look it over."
fi

echo ""
echo ""
YesNoPrompt "Do you wish to edit the platform configuration file?"
if [ "$?" -eq "0" ]; then
   EDITOR=${EDITOR:-vi}
   $EDITOR $CONFIG_FILE
fi

#
# Compile kgenmake
#
Compilekgenmake

#
# If they want to watch the compiles use tee to send the output to file and
# the terminal.
#
newtopic
echo ""
echon "Do you want to watch the compiles as they proceed? (y/n) [n]:"
read bot

if [ "$bot" = "y" -o "$bot" = "Y" ]; then
   teeoutput=1
else
   teeoutput=0
fi

#
# Give them one last chance to chicken out and not compile Khoros.
#
echo ""
echo ""
echo "This is going to take a while.  Anywhere from 4 to 24 hours depending"
echo "on the number of toolboxes installed and how fast your machine compiles."
echo "Last chance to chicken out before compiling Khoros ..."
ConfirmContinue

#
# Start the compiles.
#
newtopic

#
# Do the rest of the toolboxes.
#
for tb in $tbtoinstall; do
 
   case $tb in

      1) tbname=bootstrap
         ;;
      2) tbname=dataserv
         ;;
      3) tbname=design
         ;;
      4) tbname=datamanip
         ;;
      5) tbname=envision
         ;;
      6) tbname=geometry
         ;;
      7) tbname=image
         ;; 
      8) tbname=matrix
         ;;
      9) tbname=migration
         ;;
     10) tbname=support
         ;;
     11) tbname=retro
         ;;

   esac

   echo "Making $tbname ..."

   if [ -d $TOPDIR/$tbname/objects ]; then
      cd $TOPDIR/$tbname/objects

#
#  Need to do make World for bootstrap all other make install.
#
      if [ $tb -eq 1 ]; then
         if [ $teeoutput -eq 1 ]; then
            echo ""
	    nice kgenmake 2>&1 | tee make.World ; make World 2>&1 | tee -a make.World
	    status=$?
         else
            nice kgenmake > make.World 2>&1 ; make World >> make.World 2>&1
	    status=$?
         fi
      else
         if [ $teeoutput -eq 1 ]; then
            echo ""
            nice kgenmake 2>&1 | tee make.World ; make Makefiles 2>&1 | tee -a make.World ; make install 2>&1 | tee -a make.World
	    status=$?
         else
            nice kgenmake > make.World 2>&1 ; make Makefiles >> make.World 2>&1 ; make install >> make.World 2>&1
	    status=$?
         fi
      fi

#
# Determine if compile failed.  If not, go to next toolbox.
#
      echo ""
      echo "Checking to see if the compile for $tbname was successful ..."
      echo ""
      CheckTB $tbname
      tbcheckstatus=$?

      if [ $tbcheckstatus = 0 ]; then
         if [ $teeoutput -eq 0 ]; then
            newtopic
         fi
         echo ""
         echo "Failed to compile $tbname, no sense in compiling other toolboxes."
	 echo ""
	 echo "Do the following to fix the compile problem:"
	 echo "      % cd $TOPDIR/$tbname/objects"
	 echo "Review the errors in the make.World file and fix the code if needed."
	 echo "Check out the k2_compile_FAQ on http://www.khoral.com or at"
	 echo "ftp.khoral.com:/pub/khoros/FAQ to find out if it is known problem."
	 echo "Once you think the problem is fixed you can re-run kbuild or"
         echo "cd to each TOOLBOX/objects directory and run:"
	 echo "      % kgenmake"
	 echo "      % make Makefiles"
	 echo "      % make install"
	 echo ""
	 cleanup
      fi
      echo ""
      echo "... looks good."
      echo ""
   else
      newtopic
      echo ""
      echo "     $tbname toolbox does not exist."
      echo ""
      cleanup
   fi

   echo "... $tbname made."

done

echo "We are done compiling Khoros."
ContinuePrompt

}

# change the site specific values in the Site.XXX file
SetupManualcf()
{
   newtopic

#
# Get default printer from Site.XXX file.
#
   printer=`$grep PrinterName $SITE_FILE | $awk '{print $3}' | $tr -d "\012"`
   if [ "$printer" = "" ]; then
      printer="lp"
   fi

   echo ""
   echo "Now we need to setup the manual configuration file for your site."
   echo ""
   echon "What is the name of the printer you would like to use? [$printer] "
   read resp

   if [ "$resp" = "" ]; then
      resp=$printer
   fi

#
# Change the default printer.
#
   cmd='{ if( $1 == "#define" ) { if ($2 == "PrinterName") { if ($3 != "") printf("%s %s '$resp'\n", $1, $2) } else print $0 } else { print $0 } }'

   $awk " $cmd " $SITE_FILE > $SITE_FILE.new
   $mv $SITE_FILE.new $SITE_FILE

#
# Find groff, its utility programs and tmac file.
#
   echo "Searching for groff commands in your path ..."
   foundgroff=0
   foundgtroff=0
   foundtroff=0
   foundgtbl=0
   foundtbl=0
   foundgeqn=0
   foundeqn=0
   foundgsoelim=0
   foundsoelim=0

   for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
      if [ $foundgroff = 0 -a -x "$dir/groff" ]; then
         echo "     found groff"
         cmdgroff="groff"
         foundgroff=1
      fi
      if [ $foundgtroff = 0 -a -x "$dir/gtroff" ]; then
         echo "     found gtroff"
         cmdtroff="$dir/gtroff"
         foundgtroff=1
      fi
      if [ $foundgtbl = 0 -a -x "$dir/gtbl" ]; then
         echo "     found gtbl"
         cmdtbl="gtbl"
         foundgtbl=1
      fi
      if [ $foundgeqn = 0 -a -x "$dir/geqn" ]; then
         echo "     found geqn"
         cmdeqn="geqn"
         foundgeqn=1
      fi
      if [ $foundgsoelim = 0 -a -x "$dir/gsoelim" ]; then
         echo "     found gsoelim"
         cmdsoelim="gsoelim"
         foundgsoelim=1
      fi
   done

#
# Check to make sure that when groff was installed that the installer did
# not remove the g from infront of the standard roff utility programs.
#
   printwarn=0
   if [ $foundgtroff = 0 ]; then
      for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
         if [ $foundtroff = 0 -a -x "$dir/troff" ]; then
            echo "     using troff not gtroff (should be fine)"
            cmdtroff="$dir/troff"
            foundtroff=1
            printwarn=1
            break
         fi
      done
   fi

   if [ $foundgtbl = 0 ]; then
      for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
         if [ $foundtbl = 0 -a -x "$dir/tbl" ]; then
            echo "     using tbl not gtbl (should be fine)"
            cmdtbl="tbl"
            foundtbl=1
            printwarn=1
            break
         fi
      done
   fi

   if [ $foundgeqn = 0 ]; then
      for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
         if [ $foundeqn = 0 -a -x "$dir/eqn" ]; then
            echo "     using eqn not geqn (should be fine)"
            cmdeqn="eqn"
            foundeqn=1
            printwarn=1
            break
         fi
      done
   fi

   if [ $foundgsoelim = 0 ]; then
      for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
         if [ $foundsoelim = 0 -a -x "$dir/soelim" ]; then
            echo "     using soelim not gsoelim (should be fine)"
            cmdsoelim="soelim"
            foundsoelim=1
            printwarn=1
            break
         fi
      done
   fi

#
# Search for tmac file, mgs or ms.  Find path to tmac directories used by
# groff by using strings on the troff command.  Gross but since the path is
# hard coded into to binary.  Also, check the GROFF_TMAC_PATH environment
# variable.
#
   usemgs=0
   usems=0
   if [ $foundgtroff = 1 -o $foundtroff = 1 ]; then
      for tmacstuff in `$strings -a $cmdtroff | $grep tmac | $sed 's|:| |g'` `echo $GROFF_TMAC_PATH | $sed 's|:| |g'`; do
         if [ -f "$tmacstuff/tmac.gs" ]; then
            usemgs=1
         elif [ -f "$tmacstuff/tmac.s" ]; then
            usems=1
         fi
      done
   fi

#
# If groff not found print message telling were to get it.
#
   if [ $foundgroff = 0 ]; then
      echo ""
      echo "Can not find groff in your path.  The $KHOROSVER help system requires"
      echo "groff version 1.09+ to work.  You can ftp to ftp.khoral.com and get"
      echo "groff from the directory /pub/dist/gnu or ftp to prep.ai.mit.edu and"
      echo "get groff from the directory /pub/gnu."
      echo ""
      echo "NOTE: You will need a C++ compiler to install groff.  You can use"
      echo "      gcc 2.x or higher which has g++ bundled in.  gcc can be found"
      echo "      at the abve ftp site in the same directories."
      echo ""
   fi

#
# Update the Site.XXX file.
#
   if [ $usemgs = 1 -o $usems = 1 ]; then
      if [ $usemgs = 1 ]; then
         tmacfile="-mgs"
      else
         tmacfile="-ms"
      fi

      cmd='{ if( $1 == "#define" ) { if ($2 == "RoffMacro") { if ($3 != "") printf("%s %s '$tmacfile'\n", $1, $2) } else print $0 } else { print $0 } }'

      $awk " $cmd " $SITE_FILE > $SITE_FILE.new
      $mv $SITE_FILE.new $SITE_FILE
   else
      echo ""
      echo "Can not find the mgs or ms macros on your system.  The help"
      echo "system in Khoros require these macros.  Not sure if the help"
      echo "system will work.  The default macro will be mgs."
      echo ""
   fi

   if [ $foundgtbl = 1 -o $foundtbl = 1 ]; then
      cmd='{ if( $1 == "#define" ) { if ($2 == "TblCmd") { if ($3 != "") printf("%s %s '$cmdtbl'\n", $1, $2) } else print $0 } else { print $0 } }'

      $awk " $cmd " $SITE_FILE > $SITE_FILE.new
      $mv $SITE_FILE.new $SITE_FILE
   fi

   if [ $foundgeqn = 1 -o $foundeqn = 1 ]; then
      cmd='{ if( $1 == "#define" ) { if ($2 == "EqnCmd") { if ($3 != "") printf("%s %s '$cmdeqn' -Tascii\n", $1, $2) } else print $0 } else { print $0 } }'

      $awk " $cmd " $SITE_FILE > $SITE_FILE.new
      $mv $SITE_FILE.new $SITE_FILE
   fi

   if [ $foundgsoelim = 1 -o $foundsoelim = 1 ]; then
      cmd='{ if( $1 == "#define" ) { if ($2 == "SoelimCmd") { if ($3 != "") printf("%s %s '$cmdsoelim'\n", $1, $2) } else print $0 } else { print $0 } }'

      $awk " $cmd " $SITE_FILE > $SITE_FILE.new
      $mv $SITE_FILE.new $SITE_FILE
   fi

   if [ $printwarn = 1 ]; then
      echo ""
      echo "The groff utilities gtroff, gtbl, geqn and/or gsoelim were found,"
      echo "but without the leading g.  This may be fine, but it may mean that"
      echo "groff was not installed correctly.  Run kman and khelp to find out."
      echo "If the man pages and/or help pages do not appear correctly you will"
      echo "want to find the correct versions of the groff utilities on your"
      echo "system.  Add the directory where they are located and then change"
      echo "the $SITE_FILE to use the correct"
      echo "versions of the routines."
   fi

   echo ""
   ContinuePrompt
}

# prompt the user, to see if we should go on
YesNoPrompt()
{
   numargs=$#
   if [ "$numargs" -eq 2 ]; then
      secondarg=$2
   fi
   if [ "$numargs" -eq 0 ]; then
           echon "Do you wish to continue? (y/n) [y]:"
   else
      if [ "$numargs" -eq 2 ]; then
              echon "$1 (y/n) [$2]:"
      else
              echon "$1 (y/n) [y]:"
      fi
   fi
   read resp
   if [ "$numargs" -eq 2 -a "$resp" = "" ]; then
      resp="$secondarg"
   fi
   if [ "$resp" = "n" -o "$resp" = "N" ]; then
           return 1
   fi
   return 0
}

updatetbpath() {
   if [ -f $TOPDIR/Toolboxes ]; then
      cmd="s|$curpath|$TOPDIR|g"
      $sed -e $cmd < $TOPDIR/Toolboxes > $TMPDIR/Toolboxes.$$
      $rm $TOPDIR/Toolboxes
      $cp $TMPDIR/Toolboxes.$$ $TOPDIR/Toolboxes
      $rm $TMPDIR/Toolboxes.$$
   else
      echo ""
      echo "replacing the TOPDIR path in the Toolboxes file"
      echo "with the actual path to the $KHOROSVER Toolbox"
      echo ""
      cmd="s|TOPDIR|$TOPDIR|g"
      $sed -e $cmd < $TOPDIR/bootstrap/repos/Toolboxes > $TMPDIR/Toolboxes.$$
      $cp $TMPDIR/Toolboxes.$$ $TOPDIR/Toolboxes
      $rm $TMPDIR/Toolboxes.$$
   fi
}

if [ ! -d bootstrap -o ! -d bin ]; then
   echo ""
   echo "ERROR: "
   echo "You are apparently not in the $KHOROSVER top level directory."
   echo "Change directory to the top level directory of $KHOROSVER"
   echo "and re-run the kbuild program."
   exit 1
fi

FindEchoN

PATH=./bin:$PATH:/usr/local/bin:/usr/lang:/usr/bsd:/usr/ucb:/usr/bin:/bin:/usr/etc:/etc:/usr/ccs/bin:/usr/local/gnu/bin:/usr/bin/X11:/usr/X11R6/bin
export PATH

#
# This will make sure that the routines GetBinaryList() & GetLibraryList() do
# not run without having first installed kecho from the bootstrap toolbox.
# It is set to 1 in the routine CheckTB() if bootstrap is installed correctly.
#
havekecho=0
checkdir=""

newtopic

cat <<"EndIntro"
                      Khoros 2.2.0.0 Installation

   This script makes sure that your environment is set up correctly and
   helps you install Khoros 2.2.0.0.

   You will be prompted for the Khoros 2.2.0.0 top level directory, operating
   system, machine architecture, multiple architecture support, global
   directories for the binaries & libraries, C compiler, the BOOTSTRAP flags
   necessary to compile kgenmake and the toolboxes in Khoros 2.2.0.0 you wish to
   install.  You will be prompted for the Fortran compiler to use if
   you install a toolbox that requires Fortran.

   The script will then compile the bootstrap program kgenmake, and if
   that stage is successful, will compile the required Khoros toolboxes
   and your choice of other toolboxes in the distribution.  You will be
   given a chance to edit the Architecture Configuration File and the
   Compiler Configuration File before the toolboxes are compiled.  The
   output of the compile for each Khoros toolbox will be logged to the
   file {TOOLBOX}/objects/make.World.

   (If you experience any problems with this script, please report
   them to khoros-bugs@khoral.com)

EndIntro

echo ""
YesNoPrompt "Do you wish to continue the installation?"
if [ "$?" -eq 1 ]; then
   echo "Ok, bye..."
   echo ""
   cleanup
fi

if [ -f Toolboxes ]; then
   TOPDIR=`$grep "^BOOTSTRAP" Toolboxes | $awk -F\; '{ print $1 }' | \
      $awk -F: '{ print $NF }' | $sed 's|/bootstrap||'`
   curpath=$TOPDIR
   copytoolboxesfile=0
else
   TOPDIR=`pwd`
   curpath="$TOPDIR"
   copytoolboxesfile=1
fi

ans="n"
while [ "$ans" = "n" ]; do
   echo ""
   echo "Please enter the full path to the toplevel of $KHOROSVER"
   echon "[$TOPDIR]: "
   read temp
   if [ "$temp" = "" ]; then
      temp=$TOPDIR
   fi

   if [ -d "$temp/bootstrap" ]; then
      TOPDIR=$temp
      ans="y"
      if [ "$curpath" != "$TOPDIR" -o $copytoolboxesfile = 1 ]; then
         updatetbpath
      fi
   elif [ ! -d "$temp" ]; then
      echo ""
      echo not a valid directory $temp\!
      echo ""
   else
      echo ""
      echo bootstrap directory does not exist in $temp\!
      echo $temp can not be the toplevel of $KHOROSVER\!
      echo ""
   fi
done

if [ -f $TOPDIR/bootstrap/repos/db/imprint ]; then
   chmod 666 $TOPDIR/bootstrap/repos/db/imprint
fi

KHOROS_TOOLBOX=$TOPDIR/Toolboxes
DIST_SITE_FILE=$TOPDIR/bootstrap/repos/config/imake_config/Site.def
 DIST_DIR_FILE=$TOPDIR/bootstrap/repos/config/imake_config/Dir.def
   DIST_CONFIG=$TOPDIR/bootstrap/repos/config/imake_config/config
  LOCAL_CONFIG=$TOPDIR/bootstrap/config
export KHOROS_TOOLBOX

echo ""
echo "You are ready to continue."
echo ""
ConfirmContinue

# # ========================================================================
# #
# # guess what OS & machine architecture we are on
# #
# # ========================================================================

guess=`bootstrap/objects/script/kbuild/src/config.guess`
SplitGuess $guess

# # ========================================================================
# #
# # get details of the OS we're building on
# #
# # ========================================================================
newtopic
needkopsys=1
while [ $needkopsys = 1 ]; do
   needkopsys=0
   cat <<"   EndOpSysList"
        What Operatings System (OS) are you building Khoros on?  Enter
        the number corresponding to the OS or type the OS string.

         1. solaris:      Solaris 2.x           (2.4 supported)
         2. irix:         Irix 5.x & 6.x        (5.3 supported)
         3. linux:        Linux 1.2.x & 2.0.x   (2.0.x supported)
         4. osf:          Digital Unix 3.x      (3.2 supported)
         5. sunos:        SunOS 4.1.X           (development)
         6. aix:          AIX 3.2.x & 4.x       (development)
         7. dgux:         DGUX                  (development)
         8. bsdi:         BSDI's BSD/OS         (development)
         9. unicos:       Unicos 8 & 9          (ported)
        10. unicosmk:     Unicos/MK 1.2.530     (ported)
        11. freebsd:      FreeBSD               (development)
        12. ultrix:       Digital Ultrix        (ported)
        13. exemplar:     Convex Exemplar HPUX  (ported)
        14. osf1:         Paragon's OSF1        (ported)
        15. dynix:        Dynix                 (ported)
        16. hpux:         HPUX                  (ported)
        17. netbsd:       NetBSD                (not ported)
        18. convex:       ConvexOS              (not ported)
        19. newsos:       NewsOS                (not ported)
        20. interactive:  Interactive 386/486   (not ported)
        21. Q:            Quit the installation
     
   EndOpSysList
   echon  "Enter the Operatings System [$kopsysguess]: "
   read kopsys

   if [ "$kopsys" = "q" -o "$kopsys" = "Q" -o "$kopsys" = "21" ]; then
      cleanup
   fi

   if [ "$kopsys" = ""  ]; then
      kopsys=$kopsysguess
   fi

   fortrancmd=f77

   case $kopsys in
 
       1)            dynamiclibs=1
		     defopsys="SOLARIS" ;;    # Sparc (Solaris 2.1)
       solaris*)     dynamiclibs=1
		     defopsys="SOLARIS" ;;    # Sparc (Solaris 2.1)
 
       2)            dynamiclibs=1
		     defopsys="IRIX" ;;       # SGI 4.0.4 or 5.2
       irix*)        dynamiclibs=1
		     defopsys="IRIX" ;;       # SGI 4.0.4 or 5.2
 
       3)            dynamiclibs=1
		     defopsys="LINUX" ;;      # Linux
       linux*)       dynamiclibs=1
		     defopsys="LINUX" ;;      # Linux
 
       4)            dynamiclibs=1
		     defopsys="OSF" ;;        # DEC Alpha
       osf*)         dynamiclibs=1
		     defopsys="OSF" ;;        # DEC Alpha
 
       5)            dynamiclibs=1
		     defopsys="SUNOS" ;;      # Sun (SunOS 4.1.X)
       sunos*)       dynamiclibs=1
		     defopsys="SUNOS" ;;      # Sun (SunOS 4.1.X)
 
       6)            dynamiclibs=0
		     fortrancmd=xlf
                     defopsys="AIX" ;;        # IBM RS/6000
       aix*)         dynamiclibs=0
		     fortrancmd=xlf
                     defopsys="AIX" ;;        # IBM RS/6000
 
       7)            dynamiclibs=0
		     defopsys="DGUX" ;;       # Data General
       dgux*)        dynamiclibs=0
		     defopsys="DGUX" ;;       # Data General
 
       8)            dynamiclibs=0
		     defopsys="BSDI" ;;       # BSD/386
       bsdi*)        dynamiclibs=0
		     defopsys="BSDI" ;;       # BSD/386
 
       9)            dynamiclibs=0
		     fortrancmd=cf77
                     defopsys="UNICOS" ;;     # Cray

       10)           dynamiclibs=0
		     fortrancmd=f90
                     defopsys="UNICOSMK" ;;     # Cray T3E
       unicosmk*)    dynamiclibs=0
		     fortrancmd=f90
                     defopsys="UNICOSMK" ;;     # Cray T3E
 
       unicos*)      dynamiclibs=0
		     fortrancmd=cf77
                     defopsys="UNICOS" ;;     # Cray
 
       11)           dynamiclibs=1
		     defopsys="FREEBSD" ;;    # FreeBSD 2.x
       freebsd*)     dynamiclibs=1
		     defopsys="FREEBSD" ;;    # FreeBSD 2.x
 
       12)           dynamiclibs=0
		     defopsys="ULTRIX" ;;     # DEC 3100/5000
       ultrix*)      dynamiclibs=0
		     defopsys="ULTRIX" ;;     # DEC 3100/5000
 
       13)           dynamiclibs=0
		     defopsys="EXEMPLAR" ;;   # HP Convex Exemplar
       exemplar*)    dynamiclibs=0
		     defopsys="EXEMPLAR" ;;   # HP Convex Exemplar

       14)           dynamiclibs=0
		     defopsys="OSF1" ;;       # Paragon
       osf1*)        dynamiclibs=0
		     defopsys="OSF1" ;;       # Paragon
 
       15)           dynamiclibs=0
		     fortrancmd=F77
                     defopsys="DYNIX" ;;      # Dynix
       dynix*)       dynamiclibs=0
		     fortrancmd=F77
                     defopsys="DYNIX" ;;      # Dynix
 
       16)           dynamiclibs=1
		     fortrancmd=fort77
                     defopsys="HPUX" ;;       # HP 9000
       hpux*)        dynamiclibs=1
		     fortrancmd=fort77
                     defopsys="HPUX" ;;       # HP 9000
 
       17)           dynamiclibs=0
		     defopsys="NETBSD" ;;     # NetBSD
       netbsd*)      dynamiclibs=0
		     defopsys="NETBSD" ;;     # NetBSD
 
       18)           dynamiclibs=0
		     fortrancmd=fc
                     defopsys="CONVEX" ;;     # Convex
       convex*)      dynamiclibs=0
		     fortrancmd=fc
                     defopsys="CONVEX" ;;     # Convex
 
       19)           dynamiclibs=0
		     defopsys="NEWSOS" ;;     # Sony
       newsos*)      dynamiclibs=0
		     defopsys="NEWSOS" ;;     # Sony
 
       20)           dynamiclibs=0
		     defopsys="I386" ;;       # 386/486
       interactive*) dynamiclibs=0
		     defopsys="I386" ;;       # 386/486
 
       *)   echo ""
	    echo "You entered an invalid machine type, please re-enter"
	    echo ""
            needkopsys=1
            dynamiclibs=0 ;;

   esac

done

defname=`echo $defopsys | tr "[A-Z]" "[a-z]"`

# # ========================================================================
# #
# # get details of the machine architecture we're building on
# #
# # ========================================================================
newtopic
needkarch=1
while [ $needkarch = 1 ]; do
   needkarch=0
   cat <<"   EndArchList"
        What machine architecture are you building Khoros on?    Enter
        the number corresponding to the architecture or type the architecture
        string.
     
         1. sparc:   Sun SPARCstation (ULTRASparc)
         2. alpha:   Dec Alpha
         3. i386:    386/486 PC, Sequent
         4. i486:    486 PC
         5. i586:    Pentium PC
         6. i686:    Pentium Pro
         7. mips:    SGI Mips, Mips
         8. rs6000:  IBM RS/6000
         9. powerpc: IBM PowerPC
        10. hppa:    HP 9000 (s700, s800), Convex Exemplar
        11. ymp:     Cray
        12. t3e:     Cray T3E
        13. convex:  Convex
        14. i860:    Paragon
        15. ns32k:   Sequent, Encore
        16. m88k:    DG, Luna, Motorola 88K
        17. dap:     CPP-DAP
        18. vax:     Vax
        19. mipsel:  DEC 3100/5000
        20. apollo:  APOLLO DN10000
        21. m68k:    Sun 3, Motorola 68K, HP 9000 (s300, s400), NeXT, Mac, Sony
        22. Q:       Quit the installation
     
   EndArchList
   echon  "Enter the machine architecture [$karchguess]: "
   read karch

   if [ "$karch" = "q" -o "$karch" = "Q" -o "$karch" = "22" ]; then
      cleanup
   fi

   if [ "$karch" = ""  ]; then
      karch=$karchguess
   fi

   case $karch in
 
       1)      defarch="SPARC" ;;
       sparc*) defarch="SPARC" ;;

       2)      defarch="ALPHA" ;;
       alpha*) defarch="ALPHA" ;;
 
       3)         defarch="I80x86" ;;
       4)         defarch="I80x86" ;;
       5)         defarch="I80x86" ;;
       6)         defarch="I80x86" ;;
       i[3456]86*) defarch="I80x86" ;;
 
       7)     defarch="MIPSEB" ;;
       mips*) defarch="MIPSEB" ;;
 
       8)       defarch="RS6000" ;;
       rs6000*) defarch="RS6000" ;;
 
       9)        defarch="RS6000" ;;
       powerpc*) defarch="RS6000" ;;

       10)     defarch="PARISC" ;;
       hppa*) defarch="PARISC" ;;
 
       11)   defarch="CRAY" ;;
       ymp*) defarch="CRAY" ;;
 
       12)   defarch="CRAY_T3E" ;;
       t3e*) defarch="CRAY_T3E" ;;
 
       13)      defarch="CONVEX" ;;
       convex*) defarch="CONVEX" ;;
 
       14)    defarch="I860" ;;
       i860*) defarch="I860" ;;
 
       15)     defarch="NS32000" ;;
       ns32k*) defarch="NS32000" ;;
 
       16)    defarch="M88x00" ;;
       m88k*) defarch="M88x00" ;;
 
       17)    defarch="SPARC" ;;
       dap*)  defarch="SPARC" ;;
 
       18)    defarch="VAX" ;;
       vax*)  defarch="VAX" ;;
 
       19)      defarch="MIPSEL" ;;
       mipsel*) defarch="MIPSEL" ;;
 
       20)      defarch="APOLLO" ;;
       apollo*) defarch="APOLLO" ;;
 
       21)    defarch="MC680x0" ;;
       m68k*) defarch="MC680x0" ;;
 
       *)   echo ""
	    echo "You entered an invalid machine type, please re-enter"
	    echo ""
      needkarch=1 ;;

   esac

done


defarchname=`echo $defarch | tr "[A-Z]" "[a-z]"`

# # ========================================================================
# #
# # to support multiple architectures using a single source tree or klndir
# # need to find out a few things
# #
# # ========================================================================
newtopic
cat <<"EndMULTI"

         Configuration Name

In order to compile Khoros 2.2.0.0 you must specify a configuration name to
be used in the name of the system specific config files.  This name is
used to provide a unique name to the build system for each architecture
you want to build Khoros on.

EndMULTI

needkarch=1
while [ $needkarch = 1 ]; do
   needkarch=0
   echon "What configuration name would you like to use [$defname] ? "
   read resp
   if [ "$resp" = "" ]; then
      defkconfig="$defname"
   else
      defkconfig="$resp"
   fi
done

CONFIG_FILE=$LOCAL_CONFIG/$defkconfig.cf
SITE_FILE=$LOCAL_CONFIG/Site.$defkconfig
DIR_FILE=$LOCAL_CONFIG/Dir.$defkconfig

YesNoPrompt "Do you wish to continue the installation?"
if [ "$?" -eq 1 ]; then
   cleanup
fi

if [ -f $SITE_FILE ]; then
   CP_SITE=0
else
   CP_SITE=1
   [ ! -d $LOCAL_CONFIG ] && mkdir $LOCAL_CONFIG
   $cp $DIST_SITE_FILE $SITE_FILE
fi

#
# Update path to Toolboxes file that is compiled into kutils library.
#
cmd='{ if( $1 == "#define" ) { if ($2 == "KhorosToolboxPath") { if ($3 != "") printf("%s %s '$TOPDIR/Toolboxes'\n", $1, $2) } else print $0 } else { print $0 } }'

$awk " $cmd " $SITE_FILE > $SITE_FILE.new
$mv $SITE_FILE.new $SITE_FILE

if [ -f $CONFIG_FILE ]; then
   CP_CONFIG=0
else
   CP_CONFIG=1
   [ ! -d $LOCAL_CONFIG ] && mkdir $LOCAL_CONFIG
   if [ -f $TOPDIR/bootstrap/repos/config/imake_config/config/$defname.cf ]; then
      $cp $TOPDIR/bootstrap/repos/config/imake_config/config/$defname.cf $CONFIG_FILE
   else
      $cp $TOPDIR/bootstrap/repos/config/imake_config/config/generic.cf $CONFIG_FILE
   fi
fi

# # ========================================================================
# #
# # to support dynamic libraries
# #
# # ========================================================================
if [ $dynamiclibs = 1 ]; then
   newtopic

   echo "         Dynamic Library Support"
   echo ""
   echo "You are running a system that supports dynamic libraries."
   YesNoPrompt "Do you want to create Khoros dynamic libraries?" "y"

   if [ "$?" -eq "0" ]; then
      cmd='{ if( $1 == "#define" ) { if ($2 == "CreateKhorosSharedLibraries") { if ($3 != "") printf("%s %s 'YES'\n", $1, $2) } else print $0 } else { print $0 } }'

      $awk " $cmd " $SITE_FILE > $SITE_FILE.new
      $mv $SITE_FILE.new $SITE_FILE
   else
      cmd='{ if( $1 == "#define" ) { if ($2 == "CreateKhorosSharedLibraries") { if ($3 != "") printf("%s %s 'NO'\n", $1, $2) } else print $0 } else { print $0 } }'

      $awk " $cmd " $SITE_FILE > $SITE_FILE.new
      $mv $SITE_FILE.new $SITE_FILE
   fi
fi


# # ========================================================================
# #
# # to support global binary directory 
# #
# # ========================================================================
newtopic
cat <<"EndGLOBAL"

         Global Binary Directory Support

This option allows you to compile the Khoros 2.2.0.0 system so that all the
binaries are in one directory and the libraries are in an other directory.
For example /usr/local/khoros/bin and /usr/local/khoros/lib, respectively.
If you do not use this option the binaries and libraries for each toolbox
will be in TOOLBOX/bin and TOOLBOX/lib.

EndGLOBAL

YesNoPrompt "Do you want to use global binary directories?" "y"

if [ "$?" -eq "0" ]; then
   globaldir=1
   echo ""
   echo "replacing the globaldir=false in the Toolboxes file with globaldir=true"
   echo ""
   cmd="s|globaldir=false|globaldir=true|g"
   $sed -e $cmd < $TOPDIR/Toolboxes > $TMPDIR/Toolboxes.$$
   $rm $TOPDIR/Toolboxes
   $cp $TMPDIR/Toolboxes.$$ $TOPDIR/Toolboxes
   $rm $TMPDIR/Toolboxes.$$

   ContinuePrompt

# # ========================================================================
# #
# # determine where binaries should be installed
# #
# # ========================================================================

   if [ -f $DIR_FILE ]; then
      temp=`$awk '$1=="BinDir" { print $2 }' $DIR_FILE`
       [ "$temp" = "TOPDIR/bin" ] && temp=$TOPDIR/bin
   else
      [ ! -d $LOCAL_CONFIG ] && mkdir $LOCAL_CONFIG
      $cp $DIST_DIR_FILE $DIR_FILE
      temp=$TOPDIR/bin;
   fi

   needbindir=1
   while [ $needbindir = 1 ]; do
      newtopic
      echo "         Directory for Khoros binaries"
      echo ""
      echo "Enter the directory where you would like Khoros binaries to be installed."
      echo "The file"
      echo ""
      echo "   $DIR_FILE"
      echo ""
      echo "will be updated with the path you specify here."
      echo "Press <Enter> to use the default, given below."
      echo ""
      echon "Bin directory [$temp]: "
      read bindir

#
#  If empty then choose the default presented by $temp
#
      if [ "$bindir" = "" ]; then
         bindir=$temp
      fi

#
#  If the default is the first time installed then (which is defined in the
#  Dir.XXX as TOPDIR/bin) then change it to our current $TOPDIR/bin. Or if
#  they user wants BOOTSTRAP/bin then change it to $TOPDIR/bootstrap/bin
#
      if [ "$bindir" = "TOPDIR/bin" ]; then
         bindir=$TOPDIR/bin
      elif [ "$bindir" = "BOOTSTRAP/bin" ]; then
         bindir=$TOPDIR/bin
      fi

      notdir=0
      if [ -f $bindir ]; then
         notdir=1
      fi
      if [ -b $bindir ]; then
         notdir=1
      fi
      if [ -c $bindir ]; then
         notdir=1
      fi
      if [ -p $bindir ]; then
         notdir=1
      fi

      if [ $notdir = 1 ]; then
         echo ""
         echo "$bindir is not a directory, please re-enter."
         echo ""
	 ContinuePrompt
      elif [ -d $bindir ]; then
         if [ ! -w $bindir ]; then
            echo ""
            echo "$bindir is not a writable directory, please re-enter."
            echo ""
	    ContinuePrompt
         else
            needbindir=0
         fi
      elif [ ! -d $bindir ]; then
         echo ""
	 echo "$bindir does not exist,"
	 echon "do you want me to create it? (y/n) [y] "
	 read resp

         if [ "$resp" = "n" -o "$resp" = "N" ]; then
            echo ""
            echo "Ok, please re-enter."
            echo ""
	    ContinuePrompt
         else
            mkdir $bindir
            if [ $? -ne 0 ]; then
               echo ""
               echo "Can't create $bindir, please re-enter."
               echo ""
	       ContinuePrompt
            else
               echo ""
               echo "Created $bindir"
               echo ""
               needbindir=0
	    fi
         fi
      fi
   done

   echo ""
   echo "Changing the variable 'BinDir' to '$bindir'"
   echo "in the $DIR_FILE"
   echo ""
   cmd='{ if ($1 == "BinDir") { if ($2 != "") printf("%s '$bindir'\n", $1) } else { print $0 } }'

   $awk " $cmd " $DIR_FILE > $DIR_FILE.new
   $mv $DIR_FILE.new $DIR_FILE

   ContinuePrompt

# # ========================================================================
# #
# # Get and Set the directory for the Khoros libraries
# #
# # ========================================================================

   temp=`$awk '$1=="LibDir" { print $2 }' $DIR_FILE`

   needlibdir=1
   while [ $needlibdir = 1 ]; do
      newtopic
      echo ""
      echo "         Directory for Khoros libraries"
      echo ""
      echo "Enter the directory where you want Khoros libraries to be installed."
      echo "The file"
      echo ""
      echo "   $DIR_FILE"
      echo ""
      echo "will be updated with the path you specify here."
      echo "Press <Enter> to use the default, given below."
      echo ""
      if [ "$temp" = "TOPDIR/lib" ]; then
         echon  "Lib directory [$TOPDIR/lib]: "
      else
         echon  "Lib directory [$temp]: "
      fi
      read libdir

#
#  If empty then choose the default presented by $temp
#
      if [ "$libdir" = "" ]; then
         libdir=$temp
      fi

#
#  If the default is the first time installed then (which is defined in the
#  Site.def as TOPDIR/lib) then change it to our current $TOPDIR/lib. Or if
#  they user wants BOOTSTRAP/lib then change it to $TOPDIR/bootstrap/lib
#
      if [ "$libdir" = "TOPDIR/lib" ]; then
         libdir=$TOPDIR/lib
      elif [ "$libdir" = "BOOTSTRAP/lib" ]; then
         libdir=$TOPDIR/lib
      fi

      notdir=0
      if [ -f $libdir ]; then
         notdir=1
      fi
      if [ -b $libdir ]; then
         notdir=1
      fi
      if [ -c $libdir ]; then
         notdir=1
      fi
      if [ -p $libdir ]; then
         notdir=1
      fi

      if [ $notdir = 1 ]; then
         echo ""
         echo "$libdir is not a directory, please re-enter."
         echo ""
	 ContinuePrompt
      elif [ -d $libdir ]; then
         if [ ! -w $libdir ]; then
            echo ""
            echo "$libdir is not a writable directory, please re-enter."
            echo ""
	    ContinuePrompt
         else
            needlibdir=0
         fi
      elif [ ! -d $libdir ]; then
         echo ""
	 echo "$libdir does not exist,"
	 echon "do you want me to create it? (y/n) [y] "
	 read resp

         if [ "$resp" = "n" -o "$resp" = "N" ]; then
            echo ""
            echo "Ok, please re-enter."
            echo ""
	    ContinuePrompt
         else
            mkdir $libdir
            if [ $? -ne 0 ]; then
               echo ""
               echo "Can't create $libdir, please re-enter."
               echo ""
	       ContinuePrompt
            else
               echo ""
               echo "Created $libdir"
               echo ""
               needlibdir=0
	    fi
         fi
      fi
   done

   echo ""
   echo "Changing the variable 'LibDir' to '$libdir'"
   echo "in the $DIR_FILE"
   echo ""

   cmd='{ if ($1 == "LibDir") { if ($2 != "") printf("%s '$libdir'\n", $1) } else { print $0 } }'

   $awk " $cmd " $DIR_FILE > $DIR_FILE.new
   $mv $DIR_FILE.new $DIR_FILE

   ContinuePrompt
else
#
#  Delete the Dir.xxx file if one exists and global dirs not wanted.
#
   globaldir=0
   echo ""
   echo "checking to make sure globaldir=false in the Toolboxes file"
   echo ""
   cmd="s|globaldir=true|globaldir=false|g"
   $sed -e $cmd < $TOPDIR/Toolboxes > $TMPDIR/Toolboxes.$$
   $rm $TOPDIR/Toolboxes
   $cp $TMPDIR/Toolboxes.$$ $TOPDIR/Toolboxes
   $rm $TMPDIR/Toolboxes.$$

   ContinuePrompt

   if [ -f $DIR_FILE ]; then
      $rm $DIR_FILE
   fi

   bindir=$TOPDIR/bootstrap/bin
   libdir=$TOPDIR/bootstrap/lib

fi

PATH=$bindir:$TOPDIR/bin:$PATH
export PATH

if [ "$defopsys" != "UNICOS" ]; then
   if [ "$LD_LIBRARY_PATH" != "" ]; then
      LD_LIBRARY_PATH=$libdir:$LD_LIBRARY_PATH
   else
      LD_LIBRARY_PATH=$libdir
   fi
   export LD_LIBRARY_PATH
fi

# # ========================================================================
# #
# # find out which C compiler they want to use
# #
# # ========================================================================
needcc=1
while [ $needcc = 1 ]; do
   newtopic
   needcc=0
   echo ""
   echo "                       C Compiler"
   echo ""
   echo "Are you using a C-compiler other than 'cc', for example 'gcc' or 'acc'?"
   echo "If you are using a C-compiler other than 'cc', please enter the name of"
   echo "the C-compiler you wish to use.  If the name of the compiler is cc, but"
   echo "it is really gcc, set the HasGcc flag in the file"
   echo ""
   echo "   $SITE_FILE"
   echo ""
   echo "to YES.  If it is gcc 2.x, set HasGcc2 to YES in the same file."
   echo "To determine if the cc is really gcc do cc -v or cc --version."
   echo ""
   echo "NOTE: You will have a chance to edit the"
   echo ""
   echo "         $SITE_FILE"
   echo ""
   echo "      later on in this script."
   echo ""
   echo "NOTE: Whatever C-compiler you use must be ANSI C compliant."
   echo ""
   echon "Enter C-compiler [cc]: "
   read resp

   if [ "$resp" = "" ]; then
      ccomp=cc
   else
      ccomp="$resp"
   fi

   foundcc=""
   for ccdir in `echo $PATH | $sed -e 's/:/ /g'`; do
       if [ -x "$ccdir/$ccomp" ]; then
          foundcc="$ccdir/$ccomp"
          break
       fi
   done
   if [ "$foundcc" = "" ]; then
        echo ""
        echo "The C-compiler $ccomp is not in your path.  Please edit your path"
        echo "in your .cshrc, .bashrc, .profile or .login file to include the"
        echo "path to you C-compiler, and re-run kbuild or answer y below to"
        echo "enter a new C-compiler."
        echo ""
        ConfirmContinue
        needcc=1
   else
        echo ""
        echo "Using the C-complier $ccdir/$ccomp"
        YesNoPrompt "Is this correct?"
        if [ "$?" -eq 1 ]; then
	   needcc=1
        fi
   fi
done

echo ""
echo "Updating $SITE_FILE"
echo ""

#
# If using GNU's C-Compiler gcc
#
arg1=`echo $ccomp|$sed -e 's/[    ].*//'`
#if [ "$arg1" = "gcc" -o "$arg1" = "gcc2" ]; then

notgcc=0
version=`$ccomp --version 2>/dev/null`
case $version in
   2*) 
      GNUGCC2=1
      ;;
   1*)
      GNUGCC2=0
      ;;
   *)
      notgcc=1
      ;;
esac 

if [ $notgcc = 0 ]; then
   CONFIG_FLAGS="$CONFIG_FLAGS -DGCC_DEF"
   echo ""
   if [ $GNUGCC2 ]; then
      echo "Since you are using GNU's C-Compiler changing HasGcc to YES".
   else
      echo "Since you are using GNU's C-Compiler version $version changing HasGcc to YES".
   fi
   echo ""

   cmd='{ if( $1 == "#define" ) { if ($2 == "HasGcc") { if ($3 != "") printf("%s %s 'YES'\n", $1, $2) } else print $0 } else { print $0 } }'

   $awk " $cmd " $SITE_FILE > $SITE_FILE.new
   $mv $SITE_FILE.new $SITE_FILE

   if [ $GNUGCC2 ]; then
      echo ""
      echo "Since you are using GNU's C-Compiler version $version changing HasGcc2 to YES".
      echo ""

      cmd='{ if( $1 == "#define" ) { if ($2 == "HasGcc2") { if ($3 != "") printf("%s %s 'YES'\n", $1, $2) } else print $0 } else { print $0 } }'

      $awk " $cmd " $SITE_FILE > $SITE_FILE.new
      $mv $SITE_FILE.new $SITE_FILE
   fi

   ContinuePrompt
fi

#
# If using CenterLine C-Compiler clcc
#
arg1=`echo $ccomp|$sed -e 's/[    ].*//'`
if [ "$arg1" = "clcc" ]; then
   echo ""
   echo "Since you are using clcc changing HasCenterLineC to YES".
   echo ""

   cmd='{ if( $1 == "#define" ) { if ($2 == "HasCenterLineC") { if ($3 != "") printf("%s %s 'YES'\n", $1, $2) } else print $0 } else { print $0 } }'

   $awk " $cmd " $SITE_FILE > $SITE_FILE.new
   $mv $SITE_FILE.new $SITE_FILE

   ContinuePrompt
fi

#
# If no special compile specified make sure the special compiler flags are
# set to NO (HasGcc, HasGcc2 and HasCenterLineC).
#
arg1=`echo $ccomp|$sed -e 's/[    ].*//'`
if [ $notgcc = 1 -a "$arg1" != "clcc" -a "$arg1" != "gcc" -a "$arg1" != "gcc2" ]; then
   echo ""
   echo "Setting the C compiler parameters".
   echo ""

   cmd='{ if( $1 == "#define" ) { if ($2 == "HasGcc") { if ($3 != "") printf("%s %s 'NO'\n", $1, $2) } else print $0 } else { print $0 } }'

   $awk " $cmd " $SITE_FILE > $SITE_FILE.new
   $mv $SITE_FILE.new $SITE_FILE

   cmd='{ if( $1 == "#define" ) { if ($2 == "HasGcc2") { if ($3 != "") printf("%s %s 'NO'\n", $1, $2) } else print $0 } else { print $0 } }'

   $awk " $cmd " $SITE_FILE > $SITE_FILE.new
   $mv $SITE_FILE.new $SITE_FILE

   cmd='{ if( $1 == "#define" ) { if ($2 == "HasCenterLineC") { if ($3 != "") printf("%s %s 'NO'\n", $1, $2) } else print $0 } else { print $0 } }'

   $awk " $cmd " $SITE_FILE > $SITE_FILE.new
   $mv $SITE_FILE.new $SITE_FILE

   ContinuePrompt
fi

# # ========================================================================
# #
# # Compile Khoros
# #
# # ========================================================================
CompileKhoros

# # ========================================================================
# #
# # Change the site specific values in the Site.XXX file
# #
# # ========================================================================
SetupManualcf

# # ========================================================================
# #
# # Get the environment setup to run Khoros
# #
# # ========================================================================
newtopic
echo ""
echo "Now we need to make sure your environment is set up correctly.  To do so,"
echo "we will run kconfigure."
echo ""
ContinuePrompt

$TOPDIR/bin/kconfigure $TOPDIR

echo ""
echo "NOTE:  All users of Khoros should run kconfigure so they can access the"
echo "       $KHOROSVER software."
echo ""

exit 0
