#!/bin/sh

#
#  Khoros: $Id: kconfigure.sh,v 1.12.2.4 1997/06/11 18:09:45 mirage Exp $
#

# Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
# All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.



#************************************************************
#  Routine Name: kconfigure
#       Purpose: 
#         Input: 
#    Written By: John Salas & Neil Bowers & Tom Sauer & Steven Jorgensen
#          Date: July 22, 1993
# Modifications: Lots for Khoros 2.1 by JMAS 
#************************************************************/

  tr="tr"
  cp="cp"
  mv="mv -f"
  rm="rm -f"
 awk="awk"
 sed="sed"
 cat="cat"
grep="grep"
KPAGER=${PAGER:-more}
TMPDIR=${TMPDIR:-/tmp}

clr=""
for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
    [ -x "$dir/clear" ] && clr="$dir/clear"
done

#
#  Assume a vt100 variant
#
if [ "$clr" = "" ]; then
  clr="echo -c [H[J"
fi

# clean up behind us, for example if this script gets interrupted
# ------------------------------------------------------------------------
cleanup() {
   echo ""
   echo "I'll just tidy up any temp files which might be lying around..."
   echo "Ok, bye..."

   $rm $HOME/.Toolboxes.$$ $HOME/.line.$$ $HOME/.khoros_env.$$
   exit 0
}

# if user biffs ctrl-C, call our cleanup() function
trap cleanup 2

# prompt the user, do they want to continue
# be polite if they don't
# ------------------------------------------------------------------------
ConfirmContinue() {
   echon "Do you wish to continue? (y/n) [y]: "
   read resp
   if [ "$resp" = "n" -o "$resp" = "N" ]; then
      cleanup
   fi
}

# Simple continue prompt to allow the printing out of messages and before
# the next prompt and allowing the user to read the message.
# ------------------------------------------------------------------------
ContinuePrompt() {
   echon [Press \<Return\> to continue]
   read resp
}

# build a $echoN variable and $echoEnd combination that will give
# will allow the echon procedure to work well.
FindEchoN()
{
   # build a list of potential echos, starting with sh built-in
   echoEnd=""
   elist="echo"
   for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
      [ -x "$dir/echo" ] && elist="$elist $dir/echo"
   done
 
   # work through list, see if one of them groks -n
   for echo in $elist; do
      result=`$echo -n hello`
      if [ "$result" != "-n hello" ]; then
         echoN="$echo -n"
         break
      fi
   done
   if [ "$echoN" = "" ]; then
      echoN=echo
      echoEnd="\c"
      result=`$echoN "testing"$echoEnd`
      if [ "$result" != "testing" ]; then
         unset echoEnd
      fi
   fi
}

# this is called every time we change `topic'
newtopic() {
   $clr
}

# echo without without return
echon()
{
   $echoN "$* $echoEnd"
}

#
# Create and check the $HOME/.Toolboxes file
#
CreateDotToolboxes()
{
   newtopic

#
# Create $HOME/.Toolboxes if it does not exist.
#
   if [ ! -f $HOME/.Toolboxes ]; then
      echo ""
      echo "Creating a .Toolboxes file in your home directory."
      echo ""
      echo "# System toolboxes are included below" > $HOME/.Toolboxes
      echo "#include $TOPDIR/Toolboxes" >> $HOME/.Toolboxes
      echo "# Personal Khoros 2 Toolboxes Will be Listed Below" >> $HOME/.Toolboxes
   else
#
#  Check for and remove duplicate entries if a $HOME/.Toolboxes file does exist
#
      echo ""
      echo "We want to check your \$HOME/.Toolboxes for entries contained in the"
      echo "Khoros 2 Toolboxes file.  We recommend that they be deleted since"
      echo "the ones in your \$HOME/.Toolboxes will override the ones in the"
      echo "Khoros 2 Toolboxes file.  This is most likely not what you want to"
      echo "do, but then again maybe it is."
      echo ""
      echon "Do you want to delete these entries? (y/n) [y]: "
      read ans

      if [ "$ans" != "n" -a "$ans" != "N" ]; then
         if [ -f $TOPDIR/Toolboxes ]; then
	    echo "Checking for:"
	    for TB in `$cat $TOPDIR/Toolboxes | $grep -v "^#" | $awk -F: '{ print $1 }'`; do
	       echo "     $TB"
	       $grep -v $TB $HOME/.Toolboxes > $HOME/.Toolboxes.$$
	       $mv $HOME/.Toolboxes.$$ $HOME/.Toolboxes
	    done
	    echo "done."
          fi
      fi
      $grep -v "include $TOPDIR/Toolboxes" $HOME/.Toolboxes > $HOME/.Toolboxes.$$
      $mv $HOME/.Toolboxes.$$ $HOME/.Toolboxes
      $grep -v "System toolboxes are included below" $HOME/.Toolboxes > $HOME/.Toolboxes.$$
      echo "# System toolboxes are included below" > $HOME/.Toolboxes
      echo "#include $TOPDIR/Toolboxes" >> $HOME/.Toolboxes
      $cat $HOME/.Toolboxes.$$ >> $HOME/.Toolboxes
      $rm  $HOME/.Toolboxes.$$
   fi

   echo ""
   echo "Your Khoros Toolbox file has been updated for Khoros 2"
   echo ""
   ContinuePrompt
}

SetKarchValue() {

   if [ -d $TOPDIR/bootstrap/config ]; then
      havedef=0
      havesettings=0
      settings=""
      count=0
      for dirsetting in `ls $TOPDIR/bootstrap/config/Dir.*`; do
         case "$dirsetting" in
	    *.def)
	       echo "A Dir.def file was found in $TOPDIR/bootstrap/config.  This"
	       echo "implies a left over file from an old install of Khoros.  Please"
	       echo "have the person who installed Khoros to remove the file."
	       ContinuePrompt
	       ;;
            *)
               settings="$settings $dirsetting"
               havesettings=1
	       count=`expr $count + 1`
               ;;
         esac
      done

      if [ $havesettings = 0 ]; then
	 echo ""
         echo "No configuration files exist in $TOPDIR/bootstrap/config."
         echo "This implies Khoros has not been installed yet.  Please install"
         echo "Khoros before running kconfigure."
	 echo ""
	 ContinuePrompt
         cleanup
      fi
      if [ $count = 1 ]; then
         KARCH=`echo $dirsetting | $awk -F/ '{print $NF}' | $sed -e s/Dir.//`
         return
      fi
      newtopic
      echo ""
      echo "                Configuration Name of Khoros 2 Installation"
      echo ""
      echo "In order to set up your paths correctly, it is necessary to read"
      echo "one of the system configuration files created when Khoros was"
      echo "installed.  The name of this configuration file was specified by"
      echo "the person who compiled Khoros at your site."
      echo ""
      echo "Here are some possible settings for configuration name I found in"
      echo "$TOPDIR/bootstrap/config."
      echo ""
      echo "Talk to the person that installed Khoros to find out which is"
      echo "correct name, and specify it in the following prompt."
      echo ""

      for dirsetting in $settings; do
         echo `echo $dirsetting | $awk -F/ '{print $NF}' | $sed -e s/Dir.//`
      done

      ans="n"
      while [ "$ans" = "n" ]; do
         echo ""
         echo "Please enter the configuration name of this"
         echon "Khoros 2 installation: "
         read temp

         if [ -f "$TOPDIR/bootstrap/config/Dir.$temp" ]; then
            KARCH=$temp
            ans="y"
         else
	    echo "The name \"$temp\" is not a valid configuration name"
	 fi
      done
   fi
}

ListPossibleKarchs() {

   if [ -d $toolboxtopdir/config -o -d $TOPDIR/bootstrap/config ]; then
      if [ -d $toolboxtopdir/config ]; then
         temptb="$toolboxtopdir"
      else
         temptb="$TOPDIR/bootstrap"
      fi

      havedef=0
      havesettings=0
      settings=""
      for dirsetting in `ls $temptb/config/Dir.*`; do
         case "$dirsetting" in
            *.def) 
               havedef=1
               ;;
            *)
               settings="$settings $dirsetting"
               havesettings=1
               ;;
         esac
      done

      if [ $havesettings = 1 ]; then
         echo ""
         echo "Here are some possible settings for configuration name I found in"
         echo "$temptb/config."
         echo "Talk to the person that installed Khoros to find out which is"
         echo "correct."
         echo ""
         ContinuePrompt

         for dirsetting in $settings; do
            echo `echo $dirsetting | $awk -F/ '{print $NF}' | $sed -e s/Dir.//`
         done
      fi
   fi

}
#
# Get the global library directory:
#	1. If the globaldir in the Toolboxes file is true, yes, on or 1 any case
#	   do,
#	   a. check for TOOLBOX/config/Dir.KARCH-KOPSYS-KWIDSET
#	   b. check for bootstrap/config/Dir.KARCH-KOPSYS-KWIDSET
#	   c. default to bootstrap/lib
#	2. use TOOLBOX/lib
#
GetLibDir()
{
   toolbox="$*"
   toolboxtocheck=`echo $toolbox | $tr "[a-z]" "[A-Z]"`
   checkdir=""

#
# Determine the name of the file that specifies the LibDir
#
   if [ $nokarch = 1 ]; then
      dirfile=Dir.$KARCH
   else
      dirfile=Dir.$KARCH
      if [ "$KOPSYS" != "" ]; then
         dirfile=Dir.$KARCH-$KOPSYS
         if [ "$KWIDSET" != "" ]; then
            dirfile=Dir.$KARCH-$KOPSYS-$KWIDSET
         fi
      fi
   fi
#
# Parse out the setting for globaldir from the $TOPDIR/Toolboxes file
#
   if [ -f $TOPDIR/Toolboxes ]; then
      ans=`$grep "^$toolboxtocheck" $TOPDIR/Toolboxes | $awk -F\; '{ print $2 }'`

      useglobaldir=false
      for entry in `echo $ans | $sed s/,/\ /g`; do

         case $entry in

            globaldir*) 
                useglobaldir=`echo $entry | $awk -F= '{ print $2 }'`
                break
                ;;
            *)
                useglobaldir=false
                ;;

         esac 
      done

      ans=`$grep "^$toolboxtocheck" $TOPDIR/Toolboxes | $awk -F\; '{ print $1 }'`
      toolboxtopdir=`echo $ans | $awk -F\: '{ print $2 }'`
      if [ ! -d $toolboxtopdir ]; then
         echo ""
         echo "The directory location for where the toolboxes $toolboxtocheck is located"
         echo "does not exist.  The entry in the $TOPDIR/Toolboxes"
         echo "can be removed since the toolbox does not seem to be installed."
         echo ""
         ContinuePrompt
	 return
      fi
   else
      echo ""
      echo "No $TOPDIR/Toolboxes to use."
      echo "This should not have happened, please mail khoros-bugs@khoral.com."
      echo ""
      cleanup
   fi

#
# Set the library directory in the variable checkdir
#
   useglobaldir=`echo $useglobaldir | $tr "[A-Z]" "[a-z]"`

   if [ "$useglobaldir" = "true" -o "$useglobaldir" = "yes" -o "$useglobaldir" = "on" -o "$useglobaldir" = "1" ]; then

      if [ -d $toolboxtopdir/config ]; then
         if [ -f $toolboxtopdir/config/$dirfile ]; then
            checkdir=`$grep "^LibDir" $toolboxtopdir/config/$dirfile | $awk '{ print $2 }'`
         fi

	 if [ "$checkdir" != "" ]; then
            if [ -d $checkdir ]; then
               return 0
	    else
	       echo ""
	       echo "The library directory,"
	       echo ""
               echo "    $checkdir"
	       echo ""
               echo "for $toolbox does not exist as specified in the file,"
	       echo ""
	       echo "    $toolboxtopdir/config/$dirfile."
	       echo ""
	       echo "These could be the problems:"
	       echo "   1. The file entry is wrong and needs updating"
	       echo "   2. The file should not exist for this toolbox"
	       echo "   3. Your current setting for KARCH is wrong or is not"
	       echo "      set."
	       echo "Please fix and re-run this script."
               ListPossibleKarchs
	       cleanup
            fi
         fi

#
# Check the bootstrap toolbox
#
      elif [ -d $TOPDIR/bootstrap/config ]; then
         if [ -f $TOPDIR/bootstrap/config/$dirfile ]; then
            checkdir=`$grep "^LibDir" $TOPDIR/bootstrap/config/$dirfile | $awk ' { print $2 }'`
         fi

	 if [ "$checkdir" != "" ]; then
            if [ -d $checkdir ]; then
               return 0
	    else
	       echo ""
	       echo "The global library directory,"
	       echo ""
               echo "    $checkdir"
	       echo ""
               echo "does not exist as specified in the file,"
	       echo ""
	       echo "    $TOPDIR/bootstrap/config/$dirfile."
	       echo ""
	       echo "These could be the problems:"
	       echo "   1. The file entry is wrong and needs updating"
	       echo "   2. The file should not exist for this toolbox"
	       echo "   3. Your current setting for KARCH is wrong or is not"
	       echo "      set."
	       echo "   4. Create a $dirfile file in the directory,"
	       echo "         $toolboxtopdir/config"
               echo "      to point to the correct location"
	       echo "Please fix and re-run this script."
               ListPossibleKarchs
	       cleanup
            fi
         fi
      fi

      if [ "$checkdir" != "" ]; then
         checkdir=$TOPDIR/bootstrap/lib
         if [ -d $checkdir ]; then
	    return 0
         else
	    echo ""
	    echo "The library directory,"
	    echo ""
            echo "    $checkdir"
	    echo ""
            echo "does not exist.  These could be the problems:"
	    echo "   1. The directory was deleted accidently."
	    echo "   2. Khoros 2 not really installed on your system."
	    echo ""
	    echo "Please fix and re-run this script."
	    cleanup
         fi
      fi
   else
      checkdir=$toolboxtopdir/lib

      if [ -d $checkdir ]; then
	 return 0
      else
	 echo ""
	 echo "The library directory,"
	 echo ""
         echo "    $checkdir"
	 echo ""
         echo "for $toolbox does not exist.  These could be the problems:"
	 echo "   1. The directory was deleted accidently."
	 echo "   2. The toolbox is not really installed on the system"
         echo "      and the entry should be deleted from the file,"
	 echo ""
	 echo "    $TOPDIR/Toolboxes"
	 echo ""
	 echo "Please fix and re-run this script."
	 cleanup
      fi
   fi

   if [ "$checkdir" = "" ]; then
      echo ""
      echo "The library directory for $toolbox is undefined.  This should not"
      echo "have happened.  These could be the problems:"
      echo "   1. In the $TOPDIR/Toolboxes file"
      echo "      globaldir is set to true, on, yes or 1 and should be set to"
      echo "      false, off, no or 0."
      echo "   2. The $dirfile was deleted from the directory,"
      echo "          $TOPDIR/bootstrap/config"
      if [ "$toolbox" != "bootstrap" ]; then
         echo "      and/or the directory,"
         echo "          $toolboxtopdir/config."
      fi
      echo "   3. Your current setting for KARCH is wrong or is not set."
      echo ""
      echo "Please fix and re-run this script."

      ListPossibleKarchs
      cleanup
   fi
}

#
# Get the global binary directory:
#       1. If the globaldir in the Toolboxes file is true, yes, on or 1 any case
#          do,
#          a. check for TOOLBOX/config/Dir.KARCH-KOPSYS-KWIDSET
#          b. check for bootstrap/config/Dir.KARCH-KOPSYS-KWIDSET
#          c. default to bootstrap/bin
#       2. use TOOLBOX/bin
#
GetBinDir()
{
   toolbox="$*"
   toolboxtocheck=`echo $toolbox | $tr "[a-z]" "[A-Z]"`
   checkdir=""

#
# Determine the name of the file that specifies the BinDir
#
   if [ $nokarch = 1 ]; then
      dirfile=Dir.$KARCH
   else
      dirfile=Dir.$KARCH
      if [ "$KOPSYS" != "" ]; then
         dirfile=Dir.$KARCH-$KOPSYS
         if [ "$KWIDSET" != "" ]; then
            dirfile=Dir.$KARCH-$KOPSYS-$KWIDSET
         fi
      fi
   fi
#
# Parse out the setting for globaldir from the $TOPDIR/Toolboxes file
#
   if [ -f $TOPDIR/Toolboxes ]; then
      ans=`$grep "^$toolboxtocheck" $TOPDIR/Toolboxes | $awk -F\; '{ print $2 }'`

      useglobaldir=false
      for entry in `echo $ans | $sed s/,/\ /g`; do

         case $entry in

            globaldir*) 
                useglobaldir=`echo $entry | $awk -F= '{ print $2 }'`
                break
                ;;
            *)
                useglobaldir=false
                ;;

         esac 
      done

      ans=`$grep "^$toolboxtocheck" $TOPDIR/Toolboxes | $awk -F\; '{ print $1 }'`
      toolboxtopdir=`echo $ans | $awk -F\: '{ print $2 }'`
      if [ ! -d $toolboxtopdir ]; then
         echo ""
         echo "The directory location for were the toolboxes $toolboxtocheck is located"
         echo "does not exist.  The entry in the $TOPDIR/Toolboxes"
         echo "can be removed since the toolbox does not seem to be installed."
         echo ""
         ContinuePrompt
         return
      fi
   else
      echo ""
      echo "No $TOPDIR/Toolboxes to use."
      echo "This should not have happened, please mail khoros-bugs@khoral.com."
      echo ""
      cleanup
   fi

#
# Set the binary directory in the variable checkdir
#
   useglobaldir=`echo $useglobaldir | $tr "[A-Z]" "[a-z]"`

   if [ "$useglobaldir" = "true" -o "$useglobaldir" = "yes" -o "$useglobaldir" = "on" -o "$useglobaldir" = "1" ]; then

      if [ -d $toolboxtopdir/config ]; then
         if [ -f $toolboxtopdir/config/$dirfile ]; then
            checkdir=`$grep "^BinDir" $toolboxtopdir/config/$dirfile | $awk '{ print $2 }'`
         fi

	 if [ "$checkdir" != "" ]; then
            if [ -d $checkdir ]; then
               return 0
	    else
	       echo ""
	       echo "The binary directory,"
	       echo ""
               echo "    $checkdir"
	       echo ""
               echo "for $toolbox does not exist as specified in the file,"
	       echo ""
	       echo "    $toolboxtopdir/config/$dirfile."
	       echo ""
	       echo "These could be the problems:"
	       echo "   1. The file entry is wrong and needs updating"
	       echo "   2. The file should not exist for this toolbox"
	       echo "   3. Your current setting for KARCH is wrong or is not"
	       echo "      set."
	       echo "Please fix and re-run this script."
               ListPossibleKarchs
	       cleanup
            fi
         fi
#
# Check the bootstrap toolbox
#
      elif [ -d $TOPDIR/bootstrap/config ]; then
         if [ -f $TOPDIR/bootstrap/config/$dirfile ]; then
            checkdir=`$grep "^BinDir" $TOPDIR/bootstrap/config/$dirfile | $awk ' { print $2 }'`
         fi

	 if [ "$checkdir" != "" ]; then
            if [ -d $checkdir ]; then
               return 0
	    else
	       echo ""
	       echo "The global binary directory,"
	       echo ""
               echo "    $checkdir"
	       echo ""
               echo "does not exist as specified in the file,"
	       echo ""
	       echo "    $TOPDIR/bootstrap/config/$dirfile."
	       echo ""
	       echo "These could be the problems:"
	       echo "   1. The file entry is wrong and needs updating"
	       echo "   2. The file should not exist for this toolbox"
	       echo "   3. Your current setting for KARCH is wrong or is not"
	       echo "      set."
	       echo "   4. Create a $dirfile file in the directory,"
	       echo "         $toolboxtopdir/config"
               echo "      to point to the correct location"
	       echo "Please fix and re-run this script."
               ListPossibleKarchs
	       cleanup
            fi
         fi
      fi

      if [ "$checkdir" != "" ]; then
         checkdir=$TOPDIR/bootstrap/bin
         if [ -d $checkdir ]; then
	    return 0
         else
	    echo ""
	    echo "The binary directory,"
	    echo ""
            echo "    $checkdir"
	    echo ""
            echo "does not exist.  These could be the problems:"
	    echo "   1. The directory was deleted accidently."
	    echo "   2. Khoros 2 not really installed on your system."
	    echo ""
	    echo "Please fix and re-run this script."
	    cleanup
         fi
      fi
   else
      checkdir=$toolboxtopdir/bin

      if [ -d $checkdir ]; then
	 return 0
      else
	 echo ""
	 echo "The binary directory,"
	 echo ""
         echo "    $checkdir"
	 echo ""
         echo "for $toolbox does not exist.  These could be the problems:"
	 echo "   1. The directory was deleted accidently."
	 echo "   2. The toolbox is not really installed on the system"
         echo "      and the entry should be deleted from the file,"
	 echo ""
	 echo "    $TOPDIR/Toolboxes"
	 echo ""
	 echo "Please fix and re-run this script."
	 cleanup
      fi
   fi

   if [ "$checkdir" = "" ]; then
      echo ""
      echo "The binary directory for $toolbox is undefined.  This should not"
      echo "have happened.  These could be the problems:"
      echo "   1. In the $TOPDIR/Toolboxes file"
      echo "      globaldir is set to true, on, yes or 1 and should be set to."
      echo "      false, off, no or 0."
      echo "   2. The $dirfile was deleted from the directory,"
      echo "          $TOPDIR/bootstrap/config"
      if [ "$toolbox" != "bootstrap" ]; then
         echo "      and/or the directory,"
         echo "          $toolboxtopdir/config."
      fi
      echo "   3. Your current setting for KARCH is wrong or is not set."
      echo ""
      echo "Please fix and re-run this script."
      ListPossibleKarchs
      cleanup
   fi
}

UpdateLibList()
{
#
# Initialize the list of library directories
#
   if [ "$liblist" = "" ]; then
      liblist=$*
   fi

#
# Search to make sure the current library directory is not already in the list.
#
   doaddtoliblist=0

   for libdirectory in `echo $liblist`; do
      case $libdirectory in
         ${*}*)
		doaddtoliblist=1
	        ;;
             *)
	        ;;
      esac
   done

#
# If not already in the list add it
#
   if [ $doaddtoliblist = 0 ]; then
      liblist=`echo $liblist $*`
   fi

#    echo "$tb libaries located in $*"
}

UpdateBinList()
{
#
# Initialize the list of binary directories
#
   if [ "$binlist" = "" ]; then
      binlist=$*
   fi

#
# Search to make sure the current binary directory is not already in the list.
#
   doaddtobinlist=0

   for bindirectory in `echo $binlist`; do
      case $bindirectory in
         ${*}*)
		doaddtobinlist=1
	        ;;
             *)
	        ;;
      esac
   done

#
# If not already in the list add it
#
   if [ $doaddtobinlist = 0 ]; then
      binlist=`echo $binlist $*`
   fi

   echo "$tb binaries located in $*"
}

#
# Find out the shell the user is using.
#
DetermineShell()
{

   CSHVARIENT=0
   SHVARIENT=0
   DOTFILE=""

#
# See what the SHELL variable is set to
#
   case `echo $SHELL | $awk -F/ '{ print $NF }'` in

       csh) usershell=csh
	    CSHVARIENT=1
	    if [ -f $HOME/.cshrc ]; then
	       DOTFILE=.cshrc
            fi
            ;;
      tcsh) usershell=tcsh
	    CSHVARIENT=1
	    if [ -f $HOME/.tcshrc ]; then
	       DOTFILE=.tcshrc
            else
	       if [ -f $HOME/.cshrc ]; then
	          DOTFILE=.cshrc
               fi
            fi
            ;;
      bash) usershell=bash
	    SHVARIENT=1
	    dotfile=.bashrc
	    if [ -f $HOME/.bash_profile ]; then
	       DOTFILE=.bash_profile
            else
	       if [ -f $HOME/.bash_login ]; then
	          DOTFILE=.bash_login
               else
	          if [ -f $HOME/.profile ]; then
	                DOTFILE=.profile
                  fi
               fi
            fi
            ;;
       zsh) usershell=zsh
	    SHVARIENT=1
	    if [ -f $HOME/.zshrc ]; then
	       DOTFILE=.zshrc
            else
	       if [ -f $HOME/.zprofile ]; then
	          DOTFILE=.zprofile
               else
	          if [ -f $HOME/.zlogin ]; then
	             DOTFILE=.zlogin
                  fi
               fi
            fi
            ;;
       ksh) usershell=ksh
	    SHVARIENT=1
	    DOTFILE=.profile
            ;;
        sh) usershell=sh
	    SHVARIENT=1
	    DOTFILE=.profile
            ;;
#
# If not a known shell ask the user what type of shell variant it is and what
# dot file it uses so that it can be modified.
#
         *) echo "Only understand the following shells:"
	    echo ""
	    echo "         csh, tcsh, bash, zsh, ksh and sh"
	    echo ""

            echon "Do not know this shell $SHELL is it a csh variant? (y/n) [y]: "
            read bot

            if [ "$bot" = "n" -o "$bot" = "N" ]; then
               echon "Do not know this shell $SHELL is it a sh variant? (y/n) [y]: "
               read bot
               if [ "$bot" = "n" -o "$bot" = "N" ]; then
		  echo ""
                  echo "Ok, I am stumped.  You will need to set up under this shell,"
		  echo "your execution path to include the directories where the"
		  echo "Khoros binaries are located."
		  echo ""
		  echo "All we can do is create a $HOME/.Toolboxes."
		  echo ""
		  CreateDotToolboxes
		  cleanup
	       else
                  needdotfile=1
	          while [ $needdotfile = 1 ]; do
                      echo "Name of dotfile to used by $SHELL to set up"
                      echon "environment? [no default]: "
                      read bot

                      if [ "$bot" != "" ]; then
		         echo ""
		         echo "Will use $bot as the dotfile to modify."
		         echo ""
                         needdotfile=0
		      else
		         echo ""
		         echo "Must enter some file, no default, please re-enter."
		         echo ""
                      fi
                  done
		  DOTFILE=$bot
		  SHVARIENT=1
               fi
            else
               needdotfile=1
	       while [ $needdotfile = 1 ]; do
                   echo "Name of dotfile to used by $SHELL to set up"
                   echon "environment? [no default]: "
                   read bot

                   if [ "$bot" != "" ]; then
		      echo ""
		      echo "Will use $bot as the dotfile to modify."
		      echo ""
                      needdotfile=0
		   else
		      echo ""
		      echo "Must enter some file, no default, please re-enter."
		      echo ""
                   fi
               done
	       DOTFILE=$bot
               CSHVARIENT=1
            fi
            ;;
   esac
}

#
# Find out if we need to define a LD_LIBRARY_PATH
#
CheckForLoadLibrary()
{
   os=`uname`
   release=`uname -r`
   if [ $os = "SunOS" -o $os = "IRIX" -o $os = "OSF1" ]; then
      needldpath=0
   else
      needldpath=1
   fi

   for tb in `$cat $TOPDIR/Toolboxes | $grep -v "^#" | $awk -F: '{ print $1 }' | $tr "[A-Z]" "[a-z]"`; do
      GetLibDir $tb
      UpdateLibList $checkdir
   done
   ldlist=`echo $liblist | $sed -e 's/ /:/g'`
}

#
# In case dirname not on system
#
dirname () {
   expr ${1-.}'/' : '\(/\)[^/]*//*$' \
        \| ${1-.}'/' : '\(.*[^/]\)//*[^/][^/]*//*$' \
        \| .
}

FindTopDir() {

   command=$1

   case "$command" in
      /*)
         foundtopdir=`dirname $command`
         foundtopdir=`dirname $foundtopdir`
         ;;
      ../*)
         foundtopdir=`pwd`/`dirname $command`
         foundtopdir=`(cd ${foundtopdir}/.. ; pwd )`
         ;;
      ./*)
         foundtopdir=`dirname $command`
         if [ "$foundtopdir" = "" ]; then
            foundtopdir=`pwd`
         else
            foundtopdir=`pwd`/`expr ${foundtopdir} : '\.\/\(.*\)'`
         fi
         foundtopdir=`(cd ${foundtopdir}/.. ; pwd )`
         ;;
      *)
         foundtopdir=`pwd`
         ;;
   esac

   if [ ! -f "$foundtopdir"/Toolboxes ]; then
      foundtopdir="no default"
   fi
}

#
# Start of Script
#
if [ "$#" = 1 ]; then
   TOPDIR=$1
else
   FindTopDir $0
   TOPDIR=$foundtopdir
fi

DetermineShell

FindEchoN
newtopic

$cat <<EofIntro
 
                                   Khoros 2 
                             User Configuration Script

        This script is used to configure a user's account for the Khoros 2
        software system. This involves the following steps:
                1. Creating a personal Khoros Toolbox file in \$HOME/.Toolboxes
                2. Set up the Khoros environment variables in the appropriate
                   dotfile for your user shell so you can execute Khoros
                   binaries.

        (If you experience any problems with this script, please report
        them to khoros-bugs@khoral.com)

EofIntro

ConfirmContinue

newtopic

ans="n"
echo ""
echo "			Location of Khoros 2 Software"
echo ""
while [ "$ans" = "n" ]; do
   echo ""
   echo "Please enter the full path to the toplevel of Khoros 2"
   echon "[$TOPDIR]: "
   read temp
   if [ "$temp" = "" ]; then
      temp=$TOPDIR
   fi

   if [ -d "$temp/bootstrap" ]; then
      TOPDIR=$temp
      ans="y"
   elif [ "$temp" = "no default" ]; then
      echo ""
      echo there is no default, try again\!
      echo ""
   elif [ ! -d "$temp" ]; then
      echo ""
      echo not a valid directory $temp\!
      echo ""
   else
      echo ""
      echo bootstrap directory does not exist in $temp\!
      echo $temp can not be the toplevel of Khoros 2\!
      echo ""
   fi
done

if [ "$KARCH" = "" ]; then
   nokarch=1
   SetKarchValue
else
   nokarch=0
fi

newtopic
$cat << EofTbFile

				Toolbox File

	Ok, next you need to have a toolbox file.  This contains
	a list of the toolboxes you want to use.  

	Your personal toolbox file will be called .Toolboxes, and
	will live in your home directory.  You refer to the system
	distribution toolboxes by adding a

		#include $TOPDIR/Toolboxes

	to the \$HOME/.Toolboxes file.  If \$HOME/.Toolboxes file does
        not exist one will be created for you.  There should be no
        duplicate entries in the toolbox files.

EofTbFile

ContinuePrompt

CreateDotToolboxes

newtopic

ans="n"
echo ""
echo "          Determining Location of Khoros Binaries"
echo ""

binlist=""

for tb in `$cat $TOPDIR/Toolboxes | $grep -v "^#" | $awk -F: '{ print $1 }' | $tr "[A-Z]" "[a-z]"`; do
   GetBinDir $tb
   UpdateBinList $checkdir
done

ContinuePrompt

# ========================================================================
# 
# Give the user a .khoros_env file, setup their environment
# This file will be sourced from their .cshrc
#
# ========================================================================

newtopic
echo ""
echo "		Setting up Khoros 2 Environment Variables"
echo ""
echo "You need to ensure certain environment variables are set every"
echo "time you log in."

if [ $CSHVARIENT = 1 ]; then
   echo ""
   echo "Since you have a csh variant shell, in this step we will give you a,"
   echo ""
   echo "               $HOME/.khoros_env"
   echo ""
   echo "file, which can be source'd from a specific dotfile.  We will set up the"
   echo ".khoros_env to reflect your local configuration."
   echo ""
else
   echo ""
   echo "Since you have a sh variant shell, in this step we can add for you the"
   echo "the lines to the specific dotfile that will setup your Khoros environment."
   echo ""
fi


CheckForLoadLibrary

ContinuePrompt

if [ $CSHVARIENT = 1 ]; then
   $cat $TOPDIR/bootstrap/repos/dotfiles/khoros_env |  $sed -e s:KHOROSTMPDIR:$TMPDIR:g > $HOME/.khoros_env.$$
   $cat $HOME/.khoros_env.$$ | $sed -e s:BINDIR:"$binlist":g > $HOME/.khoros_env
   $rm $HOME/.khoros_env.$$

   if [ $needldpath = 1 ]; then
$cat <<EofLdLibraryPath >> $HOME/.khoros_env
# Set platform-specific run-time library environment variables for Khoros
if ( \$?LD_LIBRARY_PATH ) then
   setenv LD_LIBRARY_PATH $ldlist:\${LD_LIBRARY_PATH}
else
   setenv LD_LIBRARY_PATH $ldlist
endif
EofLdLibraryPath
   else
$cat <<EofLdLibraryPathComm >> $HOME/.khoros_env
# Set platform-specific run-time library environment variables for Khoros
# if ( \$?LD_LIBRARY_PATH ) then
#    setenv LD_LIBRARY_PATH $ldlist:\${LD_LIBRARY_PATH}
# else
#    setenv LD_LIBRARY_PATH $ldlist
# endif
EofLdLibraryPathComm
   fi

   if [ $nokarch = 0 ]; then
$cat <<EofKarch >> $HOME/.khoros_env
setenv KARCH $KARCH
EofKarch
   fi

   newtopic
   echo "Your .khoros_env file now looks like:"
   echo ""

   if [ -f $TOPDIR/bootstrap/repos/dotfiles/kconfig.csh ]; then
      $cat $TOPDIR/bootstrap/repos/dotfiles/kconfig.csh >> $HOME/.khoros_env
   fi

   $KPAGER $HOME/.khoros_env
   echo ""
   ContinuePrompt

   if [ "$DOTFILE" = "" ]; then
      echo ""
      echo "Can not find a file to add the line source \$HOME/.khoros_env.  To use"
      echo "Khoros you will need to source the $HOME/.khoros_env"
      echo "file by hand.  Or you can talk to you local system manager on which"
      echo "specific dotfile to add the line source \$HOME/.khoros_env."
      echo ""
   elif [ -w "$HOME/$DOTFILE" ]; then
      echo ""
      echo "Do you want me to add the line source \$HOME/.khoros_env to the"
      echon "bottom of your $DOTFILE? (y/n) [y]: "
      read resp

      if [ "$resp" != "n" -a "$resp" != "N" ]; then

         donotadd=0
	 for entry in `$grep source $HOME/$DOTFILE`; do
            case $entry in
               *HOME/.khoros_env*)
		  echo ""
                  echo "Line already exists, not adding."
		  echo ""
		  donotadd=1
                  break
		  ;;
               *)
                  ;;
            esac
         done

	 if [ $donotadd = 0 ]; then
$cat <<EofSource >> $HOME/$DOTFILE
# The following line was added automatically, by Khoros kconfigure script
source \$HOME/.khoros_env
EofSource
         fi

      else
         donotadd=0
	 for entry in `$grep source $HOME/$DOTFILE`; do
            case $entry in
               *HOME/.khoros_env*)
		  donotadd=1
                  break
		  ;;
               *)
                  ;;
            esac
         done

	 if [ $donotadd = 0 ]; then
            echo ""
            echo "This line does not exist in your $DOTFILE.  To use Khoros,"
            echo "you will need to source the $HOME/.khoros_env file by hand"
            echo "or added by hand to your $DOTFILE."
            echo ""
         fi
      fi
   elif [ ! -w "$HOME/$DOTFILE" ]; then
      echo ""
      echo "Can not write to the file $HOME/$DOTFILE.  To use"
      echo "Khoros you will need to source the $HOME/.khoros_env"
      echo "file by hand or add it to one of your environment setup files"
      echo "by hand.  You can talk to you local system manager on how to"
      echo "do this."
      echo ""
   fi
else
   binlistsh=`echo $binlist | sed s/\ /:/g`

   $cat $TOPDIR/bootstrap/repos/dotfiles/khoros_env.sh | $sed -e s%KHOROSTMPDIR%$TMPDIR%g > $HOME/.khoros_env.sh.$$
   $cat $HOME/.khoros_env.sh.$$ | $sed -e s%BINDIR%"$binlistsh"%g > $HOME/.khoros_env.sh
   $rm $HOME/.khoros_env.sh.$$

   if [ $needldpath = 1 ]; then
$cat <<EofLdLibraryPath >> $HOME/.khoros_env.sh
# Set platform-specific run-time library environment variables for Khoros
if [ "\${LD_LIBRARY_PATH}" = "" ]; then
   LD_LIBRARY_PATH=$ldlist
else
   LD_LIBRARY_PATH=$ldlist:\${LD_LIBRARY_PATH}
fi
export LD_LIBRARY_PATH
EofLdLibraryPath
   else
$cat <<EofLdLibraryPathComm >> $HOME/.khoros_env.sh
# Set platform-specific run-time library environment variables for Khoros
# if [ "\${LD_LIBRARY_PATH}" = "" ]; then
#    LD_LIBRARY_PATH=$ldlist
# else
#    LD_LIBRARY_PATH=$ldlist:\${LD_LIBRARY_PATH}
# fi
# export LD_LIBRARY_PATH
EofLdLibraryPathComm
   fi

   if [ $nokarch = 0 ]; then
$cat <<EofKarch >> $HOME/.khoros_env.sh
KARCH=$KARCH
export KARCH
EofKarch
   fi

   if [ -f $TOPDIR/bootstrap/repos/dotfiles/kconfig.sh ]; then
      $cat $TOPDIR/bootstrap/repos/dotfiles/kconfig.sh >> $HOME/.khoros_env.sh
   fi

   newtopic
   echo "Your .khoros_env.sh file now looks like:"
   echo ""
   $KPAGER $HOME/.khoros_env.sh
   echo ""
   ContinuePrompt

   if [ "$DOTFILE" = "" ]; then
      echo ""
      echo "Can not find a file to add the line . \$HOME/.khoros_env.sh .  To use"
      echo "Khoros you will need to source the $HOME/.khoros_env.sh"
      echo "file by hand.  Or you can talk to you local system manager on which"
      echo "specific dotfile to add the line . \$HOME/.khoros_env.sh."
      echo ""
   elif [ -w "$HOME/$DOTFILE" ]; then
      echo ""
      echo "Do you want me to add these lines to the bottom of your"
      echon "$DOTFILE ? (y/n) [y]: "
      read resp

      if [ "$resp" != "n" -a "$resp" != "N" ]; then


         donotadd=0
	 for entry in `$grep khoros_env.sh $HOME/$DOTFILE`; do
            case $entry in
               *HOME/.khoros_env.sh*)
		  echo ""
                  echo "Line already exists, not adding."
		  echo ""
		  donotadd=1
                  break
		  ;;
               *)
                  ;;
            esac
         done

	 if [ $donotadd = 0 ]; then
$cat <<EofSource >> $HOME/$DOTFILE
# The following line was added automatically, by Khoros kconfigure script
. \$HOME/.khoros_env.sh
EofSource
         fi

      else
         donotadd=0
	 for entry in `$grep source $HOME/$DOTFILE`; do
            case $entry in
               *HOME/.khoros_env.sh*)
		  donotadd=1
                  break
		  ;;
               *)
                  ;;
            esac
         done

	 if [ $donotadd = 0 ]; then
            echo ""
            echo "This line does not exist in your $DOTFILE.  To use Khoros,"
            echo "you will need to source the $HOME/.khoros_env.sh file by hand"
            echo "or added by hand to your $DOTFILE."
            echo ""
         fi
      fi
   elif [ ! -w "$HOME/$DOTFILE" ]; then
      echo ""
      echo "Can not write to the file $HOME/$DOTFILE.  To use"
      echo "Khoros you will need to setup the above environment variables"
      echo "by hand or add it to one of your environment setup files"
      echo "by hand.  You can talk to you local system manager on how to"
      echo "do this."
      echo ""
      DOTFILE=""
   fi
fi

ContinuePrompt

newtopic
if [ $CSHVARIENT = 1 ]; then
   echo ""
   echo "Now you should source the .khoros_env to have the your environment"
   echo "setup for you to use Khoros 2 routines.  At your prompt type:"
   echo ""
   echo "      source $HOME/.khoros_env"
   echo ""
else
   echo ""
   echo "Now you should source the .khoros_env.sh to have the your environment"
   echo "setup for you to use Khoros 2 routines.  At your prompt type:"
   echo ""
   echo "      . $HOME/.khoros_env.sh"
   echo ""
fi

exit 0
