#!/bin/sh

#
# Khoros: $Id: kversion.sh,v 1.3.2.4 1997/06/02 20:14:55 mirage Exp $
#

# Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
# All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.


#************************************************************
#
#  Routine Name: kversion - This script will print out the version of Khoros
#                           stuff
#
#       Purpose: This script will print out the version of Khoros,
#		 toolboxes and objects 
#
#         Input:
#
#    Written By: John M. Salas
#          Date: May 29, 1996
# Modifications: Added -date, -name and -version July 22, 1996
#                JMAS and TK
#
#************************************************************/

  tr="tr"
  cp="cp"
  mv="mv -f"
  rm="rm -f"
 awk="awk"
 sed="sed"
 cat="cat"
grep="grep"
strings="strings"
KPAGER=${PAGER:-more}
TMPDIR=${TMPDIR:-/tmp}
clr=""
for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
    [ -x "$dir/clear" -a ! -d "$dir/clear" ] && clr="$dir/clear"
done

#
#  Assume a vt100 variant
#
if [ "$clr" = "" ]; then
  clr="echo -c [H[J"
fi
 
bootdir=""

#
# Find kecho
#
Findkecho() {

   foundkecho=0
   kecho=""
   for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
      if [ -x "$dir/kecho" -a ! -d "$dir/kecho" ]; then
         kecho="$dir/kecho"
         foundkecho=1
      fi
   done

}

CountTimes()
{
   newfiletock="$*"
   counttbsfiles=0

   for cktbsfile in $tblist; do
      if [ "$cktbsfile" = "$newfiletock" ]; then
         counttbsfiles=1;
      fi
   done
}

TryToolboxesFile()
{
   if [ "$KHOROS_TOOLBOX" = "" ]; then
      ktb="$HOME/.Toolboxes"
   else
      ktb="$KHOROS_TOOLBOX"
   fi
   
   tblist=""
   for tbfile in `echo $ktb | $sed -e 's/:/ /g'`; do
      [ -f "$tbfile" ] && tblist="$tblist $tbfile"
   done
   
   newlist="$tblist"
   
   while [ X"$newlist" != X ]; do
      for tbfile in $newlist; do
   
         newfiles=`$grep '^#include' $tbfile | $sed -e 's/^#include[ 	]*\([^ 	][^ 	]*\)/\1/'`
        
         nfiles=""
         for newfile in $newfiles; do
            CountTimes $newfile
            [ $counttbsfiles = 0 -a -f "$newfile" ] && nfiles="$nfiles $newfile"
         done
   
         for newfile in $nfiles; do
            CountTimes $newfile
            [ $counttbsfiles = 0 -a -f "$newfile" ] && tblist="$tblist $newfile"
         done
      done
      newlist="$nfiles"
   done
   
   for tbfile in $tblist; do
      bootdir=`$grep "^BOOTSTRAP" $tbfile | $awk -F\; '{ print $1 }' | \
         $awk -F: '{ print $NF }'` 
      if [ "$bootdir" != "" ]; then
         break
      fi
   done
}
 
Findkecho

if [ $foundkecho = 0 ]; then
   TryToolboxesFile
   if [ "$bootdir" = "" ]; then
      echo ""
      echo "Can not find kecho or bootstrap directory.  Please check to make sure you have"
      echo "a $HOME/.Toolboxes file which is created by kconfigure."
      echo ""
      echo "You can look at the file,"
      echo ""
      echo "   <KHOROS_INSTALL_DIR>/bootstrap/include/bootstrap.h"
      echo ""
      echo "to see the version of Khoros."
      echo ""
      exit 1
   fi

   filetoparse=$bootdir/include/bootstrap.h

   if [ ! -f "$filetoparse" ]; then
      echo ""
      echo "Can not find file with Khoros version information."
      echo ""
      exit 1
   fi

   name=`$grep PRODUCT_RELEASE_NAME $filetoparse | $awk -F\" '{print $2}'`
   date=`$grep PRODUCT_RELEASE_DATE $filetoparse | $awk -F\" '{print $2}'`
   version=`$grep PRODUCT_RELEASE_VERSION $filetoparse | $awk '{print $NF}'`
   major=`$grep PRODUCT_RELEASE_MAJOR $filetoparse | $awk '{print $NF}'`
   minor=`$grep PRODUCT_RELEASE_MINOR $filetoparse | $awk '{print $NF}'`
   patch=`$grep PRODUCT_RELEASE_PATCH $filetoparse | $awk '{print $NF}'`

fi

tb=""
oname=""
versiononly=0
nameonly=0
dateonly=0

while [ x"$1" != x ]; do
    case $1 in
        -tb)
            tb=$2
            shift
            shift
            ;;

        -oname)
            oname=$2
            shift
            shift
            ;;

        -version)
            versiononly=1
            shift
            ;;

        -name)
            nameonly=1
            shift
            ;;

        -date)
            dateonly=1
            shift
            ;;

        -U)
            echo ""
	    echo "Usage for kversion: This script will print out the version of Khoros"
            echo ""
            echo "  [-tb]       (string) toolbox name"
            echo "              (default = {none})"
            echo "  [-oname]    (string) Program or Library Object Name"
            echo "              (default = {none})"
            echo "  [-date]     (flag) Display the date only"
	    echo "              ignored if -tb and/or -oname specified"
            echo "  [-name]     (flag) Display the name only"
	    echo "              ignored if -tb and/or -oname specified"
            echo "  [-version]  (flag) Display the numerical version only"
	    echo "              ignored if -tb and/or -oname specified"
            echo ""
            exit 0

            ;;
        *)
            shift
            ;;
    esac
done

if [ "$oname" != "" -a "$tb" != "" ]; then

   if [ $foundkecho = 1 ]; then
      kecho -echo version -tb $tb -oname $oname
      exit $?
   else
      echo ""
      echo "Could not find kecho so I am guesing the version number is the same as the"
      echo "distribution number:"
      echo ""
      echo "$version.$major.$minor.$patch"
      echo ""
   fi
   exit 0

elif [ "$oname" != "" -a "$tb" = "" ]; then

    echo ""
    echo "Must specify -tb if -oname specified."
    echo ""
    exit 1

elif [ "$tb" != "" ]; then

   if [ $foundkecho = 1 ]; then
      kecho -echo version -tb $tb
      exit $?
   else
      echo ""
      echo "Could not find kecho so I am guesing the version number is the same as the"
      echo "distribution number:"
      echo ""
      echo "$version.$major.$minor.$patch"
      echo ""
   fi
   exit 0

else

    if [ $foundkecho = 1 ]; then
       bootdir=`kecho -echo path -tb bootstrap`
    fi

    filetoparse=$bootdir/include/bootstrap.h

    if [ ! -f "$filetoparse" ]; then
       echo ""
       echo "Can not find file with Khoros version information."
       echo ""
       exit 1
    fi

    name=`$grep PRODUCT_RELEASE_NAME $filetoparse | $awk -F\" '{print $2}'`
    date=`$grep PRODUCT_RELEASE_DATE $filetoparse | $awk -F\" '{print $2}'`
    version=`$grep PRODUCT_RELEASE_VERSION $filetoparse | $awk '{print $NF}'`
    major=`$grep PRODUCT_RELEASE_MAJOR $filetoparse | $awk '{print $NF}'`
    minor=`$grep PRODUCT_RELEASE_MINOR $filetoparse | $awk '{print $NF}'`
    patch=`$grep PRODUCT_RELEASE_PATCH $filetoparse | $awk '{print $NF}'`

    if [ $versiononly = 1 ]; then
       echo "$version.$major.$minor.$patch"
    elif [ $nameonly = 1 ]; then
       echo "$name"
    elif [ $dateonly = 1 ]; then
       echo "$date"
    else
       echo "$name $version.$major.$minor.$patch ($date)"
    fi
fi

exit 0

