 /*
  * Khoros: $Id: kelevation.c,v 1.9.2.2 1997/05/31 18:48:02 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kelevation.c,v 1.9.2.2 1997/05/31 18:48:02 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for kelevation
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "kelevation.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Create Elevation Location Data From Value Plane
|
|       Purpose: main program for kelevation
|
|         Input:
|		char *clui_info->i_file; {Input value data object to use for elevation}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Resulting Output Object with elevation}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		double clui_info->xbegin_double; {Explicit Location Begin for X dimension}
|		int    clui_info->xbegin_flag; {TRUE if -xbegin specified}
|
|		double clui_info->xend_double; {Explicit Location End for X dimension}
|		int    clui_info->xend_flag; {TRUE if -xend specified}
|
|		double clui_info->ybegin_double; {Explicit Location Begin for Y dimension}
|		int    clui_info->ybegin_flag; {TRUE if -ybegin specified}
|
|		double clui_info->yend_double; {Explicit Location End for Y dimension}
|		int    clui_info->yend_flag; {TRUE if -yend specified}
|
|		int clui_info->doff_int; {Depth of Value Plane to transfer}
|		int clui_info->doff_flag; {TRUE if -doff specified}
|
|		int clui_info->toff_int; {Time of Value Plane to transfer}
|		int clui_info->toff_flag; {TRUE if -toff specified}
|
|		int clui_info->eoff_int; {Element of Value Plane to transfer}
|		int clui_info->eoff_flag; {TRUE if -eoff specified}
|
|		int   clui_info->type_list; {Resulting Elevation will be in data type}
|		char *clui_info->type_label;
|		int   clui_info->type_flag; {TRUE if -type specified}
|
|        Output:
|       Returns:
|
|    Written By: Steve Kubica
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
        kobject i, o;
        int     dpos, tpos, epos;
        float   xbeg, ybeg, xend, yend;
        char   *typename;
/* -main_variable_list_end */

	khoros_init(argc, argv, "DATAMANIP", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$DATAMANIP/objects/kroutine/kelevation");
	kexit_handler(kelevation_free_args, NULL);

/* -main_get_args_call */
	kclui_init("DATAMANIP", "kelevation", KGEN_KROUTINE,  &clui_uis_spec,
		   kelevation_usage_additions, kelevation_get_args,
 		   kelevation_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	/*-- open input object --*/
	i = kpds_open_input_object(clui_info->i_file);
	if (i == NULL)
	{
	   kerror("kelevation", "main", "Cannot open input object.");
	   kexit(KEXIT_FAILURE);
	}

	/*-- open output object --*/
	o = kpds_open_output_object(clui_info->o_file);
	if (o == NULL)
	{
	   kerror("kelevation", "main", "Cannot open output object.");
	   kexit(KEXIT_FAILURE);
	}

	/*-- gather relevant inputs into handy variables --*/
	dpos = clui_info->doff_int;
	tpos = clui_info->toff_int;
	epos = clui_info->eoff_int;

	xbeg = clui_info->xbegin_double;
	ybeg = clui_info->ybegin_double;
	xend = clui_info->xend_double;
	yend = clui_info->yend_double;

	typename = clui_info->type_label;
/* -main_before_lib_call_end */

/* -main_library_call */
	lkelevation(i, xbeg, ybeg, xend, yend, dpos, tpos, epos, typename, o);
/* -main_library_call_end */

/* -main_after_lib_call */
        /*-- add history to the output object --*/
        if (!kpds_set_attribute(o, KPDS_HISTORY, kpds_history_string()))
        {
           kerror("kelevation", "main", 
                  "Unable to set history on the destination object");
	   kexit(KEXIT_FAILURE);
        }

	/*-- close objects --*/
	kpds_close_object(i);
	kpds_close_object(o);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: kelevation_usage_additions
| 
|       Purpose: Prints usage additions in kelevation_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kelevation
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void kelevation_usage_additions(void)
{
	kfprintf(kstderr, "\tCreate Elevation Location Data From Value Plane\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: kelevation_free_args
| 
|       Purpose: Frees CLUI struct allocated in kelevation_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kelevation
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
kelevation_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->type_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
