 /*
  * Khoros: $Id: kexprn.c,v 1.4.2.2 1997/05/31 18:48:20 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kexprn.c,v 1.4.2.2 1997/05/31 18:48:20 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for kexprn
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "kexprn.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Output = Function(Input1, Input2)
|
|       Purpose: main program for kexprn
|
|         Input:
|		char *clui_info->i1_file; {input data object 1}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->expr_string; {arithmetic expression}
|		int   clui_info->expr_flag; {TRUE if -expr specified}
|
|		char *clui_info->o_file; {resulting output data object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->i2_file; {input data object 2}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->i3_file; {input data object 3}
|		int   clui_info->i3_flag; {TRUE if -i3 specified}
|
|		char *clui_info->i4_file; {input data object 4}
|		int   clui_info->i4_flag; {TRUE if -i4 specified}
|
|		char *clui_info->i5_file; {input data object 5}
|		int   clui_info->i5_flag; {TRUE if -i5 specified}
|
|		char *clui_info->i6_file; {input data object 6}
|		int   clui_info->i6_flag; {TRUE if -i6 specified}
|
|		int   clui_info->type_list; {Output Data type}
|		char *clui_info->type_label;
|		int   clui_info->type_flag; {TRUE if -type specified}
|
|        Output:
|       Returns:
|
|    Written By: Mark Young
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	int	i, type;
	char	*file[6];
	kobject in[6], dest;
/* -main_variable_list_end */

	khoros_init(argc, argv, "DATAMANIP", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$DATAMANIP/objects/kroutine/kexprn");
	kexit_handler(kexprn_free_args, NULL);

/* -main_get_args_call */
	kclui_init("DATAMANIP", "kexprn", KGEN_KROUTINE,  &clui_uis_spec,
		   kexprn_usage_additions, kexprn_get_args,
 		   kexprn_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	if ((dest = kpds_open_output_object(clui_info->o_file)) == NULL)
	{
	   kerror(NULL, "main", "Failed to open destination object '%s'.",
			clui_info->o_file);
	   kexit(KEXIT_FAILURE);
	}
	file[0] = clui_info->i1_file; file[1] = clui_info->i2_file;
	file[2] = clui_info->i3_file; file[3] = clui_info->i4_file;
	file[4] = clui_info->i5_file; file[5] = clui_info->i6_file;
	for (i = 0; i < 6; i++)
	{
	   if (file[i] != NULL)
	   {
	      if ((in[i] = kpds_open_input_object(file[i])) == NULL)
	      {
		 kerror(NULL, "main", "Failed to open input object '%s'.",
			file[i]);
		 kexit(KEXIT_FAILURE);
	      }
	   }
	   else in[i] = NULL;
	}

	if (!kpds_copy_object_attr(in[0], dest))
	{
	   kerror(NULL, "main", "Unable to copy attributes from input \
'%s' to destination '%s' object.", file[0], clui_info->o_file);
           kexit(KEXIT_FAILURE);
	}

	if (kstrcmp(clui_info->type_label, "Propagate Input Type") != 0 &&
            (type = kdatatype_to_define(clui_info->type_label)) != 0)
	{
	   kpds_set_attribute(dest, KPDS_VALUE_DATA_TYPE, type);
	}
/* -main_before_lib_call_end */

/* -main_library_call */
	if (!kapu_exprn_function(clui_info->expr_string, in, 6, dest))
	{
           kerror(NULL, "main", "Failed to generate output object '%s'.",
			clui_info->o_file);
           kexit(KEXIT_FAILURE);
	}
/* -main_library_call_end */

/* -main_after_lib_call */
        /* add history to the destination object */
        if (!kpds_set_attribute(dest, KPDS_HISTORY, kpds_history_string()))
        {
           kerror(NULL, "main", "Unable to set history on the destination \
'%s' object", clui_info->o_file);
            kexit(KEXIT_FAILURE);
	}

	if (!kpds_copy_remaining_data(in[0], dest))
	{
	   kerror(NULL, "main", "Unable to copy remaining data from input \
'%s' to destination '%s' object.", file[0], clui_info->o_file);
	   kexit(KEXIT_FAILURE);
	}

	for (i = 0;  i < 6; i++)
	   if (in[i] != NULL) kpds_close_object(in[i]);

	kpds_close_object(dest);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: kexprn_usage_additions
| 
|       Purpose: Prints usage additions in kexprn_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kexprn
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void kexprn_usage_additions(void)
{
	kfprintf(kstderr, "\tOutput = Function(Input1, Input2)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: kexprn_free_args
| 
|       Purpose: Frees CLUI struct allocated in kexprn_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kexprn
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
kexprn_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->expr_string);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->i3_file);
		kfree_and_NULL(clui_info->i4_file);
		kfree_and_NULL(clui_info->i5_file);
		kfree_and_NULL(clui_info->i6_file);
		kfree_and_NULL(clui_info->type_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
