 /*
  * Khoros: $Id: kgexprn.c,v 1.4.2.2 1997/05/31 18:51:34 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kgexprn.c,v 1.4.2.2 1997/05/31 18:51:34 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for kgexprn
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "kgexprn.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Generate Object Value Data From Expression
|
|       Purpose: main program for kgexprn
|
|         Input:
|		char *clui_info->expr_string; {function expression}
|		int   clui_info->expr_flag; {TRUE if -expr specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		double clui_info->wbegin_double; {Beginning width value}
|		int    clui_info->wbegin_flag; {TRUE if -wbegin specified}
|
|		double clui_info->wend_double; {Ending width value}
|		int    clui_info->wend_flag; {TRUE if -wend specified}
|
|		int clui_info->wsize_int; {Width size of value data}
|		int clui_info->wsize_flag; {TRUE if -wsize specified}
|
|		double clui_info->hbegin_double; {Beginning height value}
|		int    clui_info->hbegin_flag; {TRUE if -hbegin specified}
|
|		double clui_info->hend_double; {Ending height value}
|		int    clui_info->hend_flag; {TRUE if -hend specified}
|
|		int clui_info->hsize_int; {Height size of value data}
|		int clui_info->hsize_flag; {TRUE if -hsize specified}
|
|		double clui_info->dbegin_double; {Beginning depth value}
|		int    clui_info->dbegin_flag; {TRUE if -dbegin specified}
|
|		double clui_info->dend_double; {Ending depth value}
|		int    clui_info->dend_flag; {TRUE if -dend specified}
|
|		int clui_info->dsize_int; {Depth size of value data}
|		int clui_info->dsize_flag; {TRUE if -dsize specified}
|
|		double clui_info->tbegin_double; {Beginning time value}
|		int    clui_info->tbegin_flag; {TRUE if -tbegin specified}
|
|		double clui_info->tend_double; {Ending time value}
|		int    clui_info->tend_flag; {TRUE if -tend specified}
|
|		int clui_info->tsize_int; {Time size of value data}
|		int clui_info->tsize_flag; {TRUE if -tsize specified}
|
|		double clui_info->ebegin_double; {Beginning elements value}
|		int    clui_info->ebegin_flag; {TRUE if -ebegin specified}
|
|		double clui_info->eend_double; {Ending elements value}
|		int    clui_info->eend_flag; {TRUE if -eend specified}
|
|		int clui_info->esize_int; {Elements size of value data}
|		int clui_info->esize_flag; {TRUE if -esize specified}
|
|		int   clui_info->type_list; {Data type of sinusoidal value data}
|		char *clui_info->type_label;
|		int   clui_info->type_flag; {TRUE if -type specified}
|
|        Output:
|       Returns:
|
|    Written By: Mark Young
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	int	type;
	kobject dest;
/* -main_variable_list_end */

	khoros_init(argc, argv, "DATAMANIP", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$DATAMANIP/objects/kroutine/kgexprn");
	kexit_handler(kgexprn_free_args, NULL);

/* -main_get_args_call */
	kclui_init("DATAMANIP", "kgexprn", KGEN_KROUTINE,  &clui_uis_spec,
		   kgexprn_usage_additions, kgexprn_get_args,
 		   kgexprn_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	if ((dest = kpds_open_output_object(clui_info->o_file)) == NULL)
	{
	   kerror(NULL, "main", "Failed to open output object '%s'.",
			clui_info->o_file);
	   kexit(KEXIT_FAILURE);
	}
	type = kdatatype_to_define(clui_info->type_label);
	kpds_create_value(dest);
	kpds_set_attribute(dest, KPDS_VALUE_DATA_TYPE, type);
/* -main_before_lib_call_end */

/* -main_library_call */
	if (!kapu_exprn_generate(clui_info->expr_string,
		clui_info->wsize_int, clui_info->hsize_int,
		clui_info->dsize_int, clui_info->tsize_int,
		clui_info->esize_int, clui_info->wbegin_double,
		clui_info->hbegin_double, clui_info->dbegin_double,
		clui_info->tbegin_double, clui_info->ebegin_double,
		clui_info->wend_double,   clui_info->hend_double,
		clui_info->dend_double, clui_info->tend_double,
		clui_info->eend_double, TRUE, dest))
	{
	   kerror(NULL, "main", "Failed to generate output object '%s'.",
			clui_info->o_file);
	   kexit(KEXIT_FAILURE);
	}
/* -main_library_call_end */

/* -main_after_lib_call */
	/* add history to the destination object */
	if (!kpds_set_attribute(dest, KPDS_HISTORY, kpds_history_string()))
	{
	   kerror(NULL, "main", "Unable to set history on the destination \
'%s' object", clui_info->o_file);
	    kexit(KEXIT_FAILURE);
	}
	kpds_close_object(dest);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: kgexprn_usage_additions
| 
|       Purpose: Prints usage additions in kgexprn_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kgexprn
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void kgexprn_usage_additions(void)
{
	kfprintf(kstderr, "\tGenerate Object Value Data From Expression\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: kgexprn_free_args
| 
|       Purpose: Frees CLUI struct allocated in kgexprn_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kgexprn
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
kgexprn_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->expr_string);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->type_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
