 /*
  * Khoros: $Id: kimportraw.c,v 1.15.2.2 1997/05/31 18:53:32 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kimportraw.c,v 1.15.2.2 1997/05/31 18:53:32 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for kimportraw
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "kimportraw.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Low level import of raw (binary) data
|
|       Purpose: main program for kimportraw
|
|         Input:
|		char *clui_info->i1_file; {raw (binary) input}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->o_file; {output object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->i2_file; {Object to which the new data should be added}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		int   clui_info->arch_list; {data source architecture}
|		char *clui_info->arch_label;
|		int   clui_info->arch_flag; {TRUE if -arch specified}
|
|		int   clui_info->type_list; {data type of raw data}
|		char *clui_info->type_label;
|		int   clui_info->type_flag; {TRUE if -type specified}
|
|		int clui_info->wsize_int; {Size of width dimension}
|		int clui_info->wsize_flag; {TRUE if -wsize specified}
|
|		int clui_info->hsize_int; {Size of height dimension}
|		int clui_info->hsize_flag; {TRUE if -hsize specified}
|
|		int clui_info->dsize_int; {Size of depth dimension}
|		int clui_info->dsize_flag; {TRUE if -dsize specified}
|
|		int clui_info->tsize_int; {Size of time dimension}
|		int clui_info->tsize_flag; {TRUE if -tsize specified}
|
|		int clui_info->esize_int; {Size of elements dimension}
|		int clui_info->esize_flag; {TRUE if -esize specified}
|
|		int clui_info->skip_int; {skip this number of bytes before reading}
|		int clui_info->skip_flag; {TRUE if -skip specified}
|
|		int   clui_info->segment_list; {destination for incoming data }
|		char *clui_info->segment_label;
|		int   clui_info->segment_flag; {TRUE if -segment specified}
|
|		int   clui_info->color_list; {color space model for data}
|		char *clui_info->color_label;
|		int   clui_info->color_flag; {TRUE if -color specified}
|
|        Output:
|       Returns:
|
|    Written By: Scott Wilson, Jeremy Worley
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	kobject in_obj=NULL,out_obj=NULL,add_obj=NULL;
	int type_list[] = { KNONE, KBIT, KBYTE, KUBYTE, KSHORT, KUSHORT, KINT, 
			    KUINT, KLONG, KULONG, KFLOAT, KDOUBLE, KCOMPLEX, 
			    KDCOMPLEX};
	char *seg_list[] = { "bogus", KDMS_SEGMENT_VALUE, KDMS_SEGMENT_MASK, 
			     KDMS_SEGMENT_MAP, KDMS_SEGMENT_LOCATION, 
			     KDMS_SEGMENT_TIME,"Comment"};
	int col_list[] = {   KNONE, KNONE, KGREY, KRGB, KCMY, KYIQ, KHSV, KHLS,
                             KIHS, KXYZ, KUVW, KUCSUVW, KUCSSOW, KUCSLab,
                             KUCSLuv, KUSERDEFINED };

	int mach_list[] = { KMACH_UNKNOWN,
			    KMACH_LOCAL,
			    KMACH_LITTLE_ENDIAN_IEEE,		/* I80x86  */
			    KMACH_BIG_ENDIAN_IEEE,		/* SPARC   */
			    KMACH_LITTLE_ENDIAN_IEEE,		/* MIPSEL  */
			    KMACH_BIG_ENDIAN_IEEE,		/* IBMRT   */
			    KMACH_BIG_ENDIAN_IEEE,		/* RS6000  */
			    KMACH_BIG_ENDIAN_IEEE,		/* POWERPC */
			    KMACH_BIG_ENDIAN_IEEE, 		/* MC680x0 */
			    KMACH_BIG_ENDIAN_IEEE,		/* M88x00  */
			    KMACH_BIG_ENDIAN_IEEE,		/* MIPSEB  */
			    KMACH_BIG_ENDIAN_IEEE, 		/* PARISC  */
			    KMACH_BIG_ENDIAN_IEEE,		/* CONVEX  */
			    KMACH_BIG_ENDIAN_IEEE,		/* APOLLO  */
			    KMACH_LITTLE_ENDIAN_ALPHA,		/* ALPHA   */ 
			    KMACH_BIG_ENDIAN_CRAY,		/* CYBER   */
			    KMACH_BIG_ENDIAN_CRAY,		/* CRAY    */
			    KMACH_LITTLE_ENDIAN_VAX,		/* VAX     */
			    KMACH_LITTLE_ENDIAN_IEEE};		/* I860    */
/* -main_variable_list_end */

	khoros_init(argc, argv, "DATAMANIP", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$DATAMANIP/objects/kroutine/kimportraw");
	kexit_handler(kimportraw_free_args, NULL);

/* -main_get_args_call */
	kclui_init("DATAMANIP", "kimportraw", KGEN_KROUTINE,  &clui_uis_spec,
		   kimportraw_usage_additions, kimportraw_get_args,
 		   kimportraw_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        /* 
	 * Note special way of opening object that *forces* the data to
         * be accessed as raw data, even if it is one of the supported data
         * types.
	 */
        if ((in_obj = kpds_open_object(clui_info->i1_file, KOBJ_READ|KOBJ_RAW))
	    == KOBJECT_INVALID)
        {
           kerror("kimportraw","main", "Cannot open raw input object.");
           kexit(KEXIT_FAILURE);
        }

        if ((out_obj = kpds_open_output_object(clui_info->o_file))
	    == KOBJECT_INVALID)
        {
           kerror("kimportraw","main", "Cannot open output object.");
           kexit(KEXIT_FAILURE);
        }

        if (clui_info->i2_flag)
	{
	   if ((add_obj = kpds_open_input_object(clui_info->i2_file))
	       == KOBJECT_INVALID)
	   {
	      kerror("kimportraw","main", "Cannot open add object i2.");
	      kexit(KEXIT_FAILURE);
	   }

	   if (!kpds_copy_object_attr(add_obj, out_obj))
	   {
	      kerror("kimportraw","main", 
		     "Unable to copy attributes from add to output object.");
	      kexit(KEXIT_FAILURE);
	   }
	}
/* -main_before_lib_call_end */

/* -main_library_call */
	if (!lkimportraw(in_obj, add_obj, mach_list[clui_info->arch_list],
			  type_list[clui_info->type_list], 
			  seg_list[clui_info->segment_list],
			  clui_info->wsize_int, clui_info->hsize_int,
			  clui_info->dsize_int, clui_info->tsize_int,
			  clui_info->esize_int, clui_info->skip_int,
                          col_list[clui_info->color_list],out_obj))
	   kexit(KEXIT_FAILURE);
/* -main_library_call_end */

/* -main_after_lib_call */
        /* 
	 * add history to the output object 
	 */
        if (!kpds_set_attribute(out_obj, KPDS_HISTORY, kpds_history_string()))
        {
           kerror("kimportraw", "main",
                  "Unable to set history on the destination object");
            kexit(KEXIT_FAILURE);
        }

        if (clui_info->i2_flag)
          {
            if (!kpds_copy_remaining_data(add_obj, out_obj))
              {
               kerror("kimportraw","main",
                  "Unable to copy remaining data from add to output object.");
               kexit(KEXIT_FAILURE);
              }
          }

        if  (clui_info->i2_flag) 
	   kpds_close_object(add_obj);

        kpds_close_object(in_obj);

        kpds_close_object(out_obj);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: kimportraw_usage_additions
| 
|       Purpose: Prints usage additions in kimportraw_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kimportraw
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void kimportraw_usage_additions(void)
{
	kfprintf(kstderr, "\tLow level import of raw (binary) data\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: kimportraw_free_args
| 
|       Purpose: Frees CLUI struct allocated in kimportraw_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kimportraw
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
kimportraw_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->arch_label);
		kfree_and_NULL(clui_info->type_label);
		kfree_and_NULL(clui_info->segment_label);
		kfree_and_NULL(clui_info->color_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
