 /*
  * Khoros: $Id: ktranspose.c,v 1.1.2.2 1997/05/31 19:02:58 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: ktranspose.c,v 1.1.2.2 1997/05/31 19:02:58 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for ktranspose
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "ktranspose.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Transpose Data across Dimensions
|
|       Purpose: main program for ktranspose
|
|         Input:
|		char *clui_info->i_file; {input data object}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {resulting data object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		Loose group:
|		    int clui_info->w_flag; {TRUE if -w specified}
|
|		    int clui_info->h_flag; {TRUE if -h specified}
|
|		    int clui_info->e_flag; {TRUE if -e specified}
|
|		    int clui_info->d_flag; {TRUE if -d specified}
|
|		    int clui_info->t_flag; {TRUE if -t specified}
|
|        Output:
|       Returns:
|
|    Written By: Steve Kubica
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
        kobject in_obj;
        kobject out_obj;
	int     new_order[5];
	int     num_swap = 0;
/* -main_variable_list_end */

	khoros_init(argc, argv, "DATAMANIP", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$DATAMANIP/objects/kroutine/ktranspose");
	kexit_handler(ktranspose_free_args, NULL);

/* -main_get_args_call */
        kclui_init("DATAMANIP", "ktranspose", KGEN_KROUTINE,  &clui_uis_spec,
                   ktranspose_usage_additions, ktranspose_get_args,
                   ktranspose_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        if ((in_obj  = kpds_open_input_object(clui_info->i_file)) 
		== KOBJECT_INVALID)
	{
           kerror("ktranspose","main", "Cannot open input object %s.",
                   clui_info->i_file);
           kexit(KEXIT_FAILURE);
	}
        if ((out_obj = kpds_open_output_object(clui_info->o_file))
		== KOBJECT_INVALID)
        {
           kerror("ktranspose","main", "Cannot open output object %s.",
                   clui_info->o_file);
           kexit(KEXIT_FAILURE);
        }


        if (clui_info->w_flag)
	{
	   new_order[0] = (clui_info->h_flag) ? KHEIGHT   :
	                  (clui_info->d_flag) ? KDEPTH    :
	                  (clui_info->t_flag) ? KTIME     :
	                  (clui_info->e_flag) ? KELEMENTS : KWIDTH;
	   num_swap++;
	}
	else
	   new_order[0] = KWIDTH;

        if (clui_info->h_flag)
	{
	   new_order[1] = (clui_info->w_flag) ? KWIDTH    :
	                  (clui_info->d_flag) ? KDEPTH    :
	                  (clui_info->t_flag) ? KTIME     :
	                  (clui_info->e_flag) ? KELEMENTS : KHEIGHT;
	   num_swap++;
	}
	else
	   new_order[1] = KHEIGHT;

        if (clui_info->d_flag)
	{
	   new_order[2] = (clui_info->w_flag) ? KWIDTH    :
	                  (clui_info->h_flag) ? KHEIGHT   :
	                  (clui_info->t_flag) ? KTIME     :
	                  (clui_info->e_flag) ? KELEMENTS : KDEPTH;
	   num_swap++;
	}
	else
	   new_order[2] = KDEPTH;

        if (clui_info->t_flag)
	{
	   new_order[3] = (clui_info->w_flag) ? KWIDTH    :
	                  (clui_info->h_flag) ? KHEIGHT   :
	                  (clui_info->d_flag) ? KDEPTH    :
	                  (clui_info->e_flag) ? KELEMENTS : KTIME;
	   num_swap++;
	}
	else
	   new_order[3] = KTIME;

        if (clui_info->e_flag)
	{
	   new_order[4] = (clui_info->w_flag) ? KWIDTH    :
	                  (clui_info->h_flag) ? KHEIGHT   :
	                  (clui_info->d_flag) ? KDEPTH    :
	                  (clui_info->t_flag) ? KTIME     : KELEMENTS;
	   num_swap++;
	}
	else
	   new_order[4] = KELEMENTS;

	if (num_swap > 2)
	{
	   kerror("ktranspose","main", 
		  "Only two dimensions can be transposed");
	   kexit(KEXIT_FAILURE);
	}

        if (!kpds_copy_object_attr(in_obj, out_obj))
        {
           kerror("ktranspose","main", 
		"Unable to copy attributes from input to output");
           kexit(KEXIT_FAILURE);
        }
/* -main_before_lib_call_end */

/* -main_library_call */
	if (!lktranspose(in_obj, new_order, out_obj))
        {
           kexit(KEXIT_FAILURE);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
	/* copy rest of input object that was not affected by library */
        if (!kpds_copy_remaining_data(in_obj, out_obj))
        {
           kerror("ktranspose", "main",
                  "Unable to copy remaining data from source to destination.");
           kexit(KEXIT_FAILURE);
        }
 
        /* add history to the output object */
        if (!kpds_set_attribute(out_obj, KPDS_HISTORY, kpds_history_string()))
        {
           kerror("ktranspose", "main",
                  "Unable to set history on the destination object");
           kexit(KEXIT_FAILURE);
        }
	
        kpds_close_object(in_obj);
        kpds_close_object(out_obj);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: ktranspose_usage_additions
| 
|       Purpose: Prints usage additions in ktranspose_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname ktranspose
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void ktranspose_usage_additions(void)
{
	kfprintf(kstderr, "\tTranspose Data across Dimensions\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: ktranspose_free_args
| 
|       Purpose: Frees CLUI struct allocated in ktranspose_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname ktranspose
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
ktranspose_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
