 /*
  * Khoros: $Id: kwiener.c,v 1.14.2.2 1997/05/31 19:03:38 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kwiener.c,v 1.14.2.2 1997/05/31 19:03:38 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for kwiener
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "kwiener.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Wiener Filtering (Restoration) in Fourier Frequency Domain
|
|       Purpose: main program for kwiener
|
|         Input:
|		char *clui_info->i1_file; {source data object (FFT)}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {PSF data object (FFT)}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->o_file; {filtered data object (FFT)}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		double clui_info->w_double; {Wiener parameter}
|		int    clui_info->w_flag; {TRUE if -w specified}
|
|        Output:
|       Returns:
|
|    Written By: Scott Wilson
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
kobject in_obj,psf_obj,out_obj;
/* -main_variable_list_end */

	khoros_init(argc, argv, "DATAMANIP", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$DATAMANIP/objects/kroutine/kwiener");
	kexit_handler(kwiener_free_args, NULL);

/* -main_get_args_call */
	kclui_init("DATAMANIP", "kwiener", KGEN_KROUTINE,  &clui_uis_spec,
		   kwiener_usage_additions, kwiener_get_args,
 		   kwiener_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        if ((in_obj = kpds_open_input_object(clui_info->i1_file))
                == KOBJECT_INVALID)
          {
            kerror("kwiener","main", "Cannot open input object.");
            kexit(KEXIT_FAILURE);
          }
        if ((psf_obj = kpds_open_input_object(clui_info->i2_file))
                == KOBJECT_INVALID)
          {
            kerror("kwiener","main", "Cannot open PSF object.");
            kexit(KEXIT_FAILURE);
          }
        if ((out_obj = kpds_open_output_object(clui_info->o_file))
                == KOBJECT_INVALID)
          {
            kerror("kwiener","main", "Cannot open output object.");
            kexit(KEXIT_FAILURE);
          }
        if (!kpds_copy_object_attr(in_obj, out_obj))
          {
            kerror("kwiener","main",
              "Unable to copy attributes from input to output object.");
            kexit(KEXIT_FAILURE);
          }
/* -main_before_lib_call_end */

/* -main_library_call */
        if (!lkwiener(in_obj,psf_obj,
                      clui_info->w_double,
                      out_obj))
             kexit(KEXIT_FAILURE);
/* -main_library_call_end */

/* -main_after_lib_call */
        /* add history to the output objects and write */
        if (!kpds_set_attribute(out_obj,KPDS_HISTORY,kpds_history_string()))
          {
            kerror("kwiener", "main","Unable to set history on output object"
);
            kexit(KEXIT_FAILURE);
          }
        kpds_close_object(out_obj);
        kpds_close_object(in_obj);
        kpds_close_object(psf_obj);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: kwiener_usage_additions
| 
|       Purpose: Prints usage additions in kwiener_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kwiener
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void kwiener_usage_additions(void)
{
	kfprintf(kstderr, "\tWiener Filtering (Restoration) in Fourier Frequency Domain\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: kwiener_free_args
| 
|       Purpose: Frees CLUI struct allocated in kwiener_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kwiener
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
kwiener_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
