/*
  * Khoros: $Id: utilities.c,v 1.7 1997/01/20 06:15:45 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.7 1997/01/20 06:15:45 mirage Exp $";

#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Utilities for the Data Manipulation routines
   >>>>		
   >>>>  Private:
   >>>>                 return_data_name()
   >>>>                 polymorphic_query()
   >>>>                 polymorphic_create()
   >>>>   Public:
   >>>>                 kcombine_to_complex()
   >>>>                 kdata_insert()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


/*-----------------------------------------------------------
|
|  Routine Name: return_data_name - return the data name
|
|       Purpose: return_data_name looks for the minimum unique 
|                set of characters to identify a data name
|                and returns the abstract data services name
|                for that data.
|
|         Input: str - name passed in 
|
|        Output: 
|
|       Returns: KPDS data name on success, NULL on failure
|
|    Written By: Donna Koechner
|          Date: Mar 02, 1993
| Modifications:
|
------------------------------------------------------------*/

char *return_data_name(
        char *str)
{

   char *lstr = NULL;
   char *return_str = NULL;

   lstr = kstring_lower(str, lstr);

   /* Segment names returned based on the following evaluations:
        value           va
        time            tim
        location        lo
        map             map
        mask            mas  */

   if (kstrstr(lstr, "va") != NULL) 
      return_str = kstring_copy(KDMS_SEGMENT_VALUE,return_str);
   
   else if (kstrstr(lstr, "tim") != NULL) 
      return_str = kstring_copy(KDMS_SEGMENT_TIME,return_str);
   
   else if (kstrstr(lstr, "lo") != NULL) 
      return_str = kstring_copy(KDMS_SEGMENT_LOCATION,return_str);
   
   else if (kstrstr(lstr, "map") != NULL) 
      return_str = kstring_copy(KDMS_SEGMENT_MAP,return_str);
   
   else if (kstrstr(lstr, "mask") != NULL) 
      return_str = kstring_copy(KDMS_SEGMENT_MASK,return_str);

   return(return_str);

}

/*-----------------------------------------------------------
|
|  Routine Name: polymorphic_query - will query to see
|		 if the polymorphic data exists
|
|       Purpose: 
|
|         Input: object - object to query for the data
|		 str    - name passed in 
|
|        Output: 
|
|       Returns: TRUE if it exists, FALSE (0) if it does not exist
|		 or str is not a valid polymorphic data
|
|    Written By: John M. Salas
|          Date: Oct 07, 1993
| Modifications:
|
------------------------------------------------------------*/

int polymorphic_query(
	kobject object,
        char	*str)
{
   char *lstr = NULL;
   lstr = kstring_lower(str, lstr);

   if (kstrstr(lstr, "va") != NULL && kpds_query_value(object) )
   {
      return(TRUE);
   }
   else if (kstrstr(lstr, "map") != NULL && kpds_query_map(object) )
   {
      return(TRUE);
   }
   else if (kstrstr(lstr, "loc") != NULL && kpds_query_location(object) )
   {
      return(TRUE);
   }
   else if (kstrstr(lstr, "time") != NULL && kpds_query_time(object) )
   {
      return(TRUE);
   }
   else if (kstrstr(lstr, "mask") != NULL && kpds_query_mask(object) )
   {
      return(TRUE);
   }

   return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: polymorphic_create - will create
|		 the polymorphic data
|
|       Purpose: 
|
|         Input: object - object to create the data in
|		 str    - name passed in 
|
|        Output: 
|
|       Returns: TRUE if it exists, FALSE (0) if it can not
|		 create or str is not a valid polymorphic data
|
|    Written By: John M. Salas
|          Date: Oct 07, 1993
| Modifications:
|
------------------------------------------------------------*/

int polymorphic_create(
	kobject object,
        char	*str)
{
   char *lstr = NULL;
   lstr = kstring_lower(str, lstr);

   if (kstrstr(lstr, "va") != NULL && !kpds_query_value(object) )
   {
      if ( !kpds_create_value(object) ) return(FALSE);
      return(TRUE);
   }
   else if (kstrstr(lstr, "map") != NULL && !kpds_query_map(object) )
   {
      if ( !kpds_create_map(object) ) return(FALSE);
      return(TRUE);
   }
   else if (kstrstr(lstr, "loc") != NULL && !kpds_query_location(object) )
   {
      if ( !kpds_create_location(object) ) return(FALSE);
      return(TRUE);
   }
   else if (kstrstr(lstr, "time") != NULL && !kpds_query_time(object) )
   {
      if ( !kpds_create_time(object) ) return(FALSE);
      return(TRUE);
   }
   else if (kstrstr(lstr, "mask") != NULL && !kpds_query_mask(object) )
   {
      if ( !kpds_create_mask(object) ) return(FALSE);
      return(TRUE);
   }

   return(FALSE);
}

/************************************************************
*
*  Routine Name: kcombine_to_complex - combines 2 double arrays into a double complex array
*
*       Purpose: kcombine_to_complex takes a real and imaginary array
*                of type double and returns a kdcomplex array.
*
*         Input: 2 double arrays containing the real and imag parts
*
*        Output: a dcomplex array to be split into real & imag parts
*
*  Restrictions:
*    Written By: Ashish Malhotra
*          Date: Nov 02, 1993 14:03
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
 
int kcombine_to_complex(kdcomplex **c, int num, double *r, double *i)
{
   int j;
 
   if ((r==NULL) || (i == NULL))
      return (FALSE);
 
   if (c != NULL && *c == NULL)
   {
      if ((*c = (kdcomplex*)kmalloc((size_t)num * sizeof (kdcomplex))) == NULL)
         return(FALSE);
   }
 
   for (j = 0; j < num; j++)
   {
       (*c)[j] = kdccomp(r[j],i[j]);
   }
 
   return (TRUE);
}

/*************************************************************
*
*  Routine Name: kdata_insert - insert a value in memory 
*
*       Purpose: This function is used to insert a value into
*                memory. A datatype is used to determine how to
*                interpret the real and imaginary values specified.
*
*         Input: data      - point to the data to be filled
*                num       - location to be inserted into
*                datatype  - data type to fill with
*                real      - real part of data to fill with
*                imaginary - imaginary part of data to fill with
*
*        Output:
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*    Written By: Ashish Malhotra
*          Date: Mar 30, 1994
* Modifications:
*
*************************************************************/

int kdata_insert(
   kaddr  data,
   int    num,
   int    datatype,
   double real,
   double imaginary)
{
        int n = num;

        switch (datatype) {
           case KBIT:
                {
                  unsigned char *d = (unsigned char *)data;
                  unsigned char v;
                  unsigned long nbytes = (num >> 3);

                  v = (real == 0.0) ? 0x00 : 0x01;
                  v = ( v << (num % 8));
                  d[nbytes] |= v ;
                }
                return(TRUE);
           case KBYTE:
                {
                  char *d = (char *)data, v = (char)real;
                  d[n] = v;
                }
                return(TRUE);
           case KUBYTE:
                {
                  unsigned char *d = (unsigned char *)data,
                        v = (unsigned char)real;
                  d[n] = v;
                }
                return(TRUE);
            case KSHORT:
                {
                  short *d = (short *)data, v = (short)real;
                  d[n] = v;
                }
                return(TRUE);
           case KUSHORT:
                {
                  unsigned short *d = (unsigned short *)data,
                        v = (unsigned short)real;
                  d[n] = v;
                }
                return(TRUE);
           case  KINT:
                {
                  int *d = (int *)data, v = (int)real;
                  d[n] = v;
                }
                return(TRUE);
           case KUINT:
                {
                  unsigned int *d = (unsigned int *)data,
                        v = (unsigned int)real;
                  d[n] = v;
                }
                return(TRUE);
            case KLONG:
                {
                  long *d = (long *)data, v = (long)real;
                  d[n] = v;
                }
                return(TRUE);
            case KULONG:
                {
                  unsigned long *d = (unsigned long *)data,
                        v = (unsigned long)real;
                  d[n] = v;
                }
                return(TRUE);
            case KFLOAT:
                {
                  float *d = (float *)data;
                  d[n] = real;
                }
                return(TRUE);
            case KDOUBLE:
                {
                  double *d = (double *)data;
                  d[n] = real;
                }
                return(TRUE);
           case KCOMPLEX:
                {
                  kcomplex *d = (kcomplex *)data,
                        v = kccomp((float)real,(float)imaginary);
                  d[n] = v;
                }
                return(TRUE);
           case KDCOMPLEX:
                {
                  kdcomplex *d = (kdcomplex *)data,
                        v = kdccomp(real,imaginary);
                   d[n] = v;
                }
                return(TRUE);
           default:
                return(FALSE);
        }
}

