 /*
  * Khoros: $Id: lkconvert.c,v 1.16.2.3 1997/05/31 19:08:16 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lkconvert.c,v 1.16.2.3 1997/05/31 19:08:16 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for kconvert
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lkconvert
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
static int _kconvert_advance PROTO((int *, int *, int *, int *, int));
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lkconvert - convert, offset, and/or scale data in specified segment
* 
*       Purpose: This routine will convert, offset and/or scale
*		 the segment specified by segment_name of the input 
*		 object source_object and create a new object 
*		 destination_object containing the modified segment 
*		 and all the other segments which will be unmodified.  
*		 Data is scaled first, and then the offset is applied.
*		 Data type conversion is accomplished via casting.  
*		 The data type to convert to will be passed in 
*		 data_type_string and should be one of these values:
*
*                .TS H
*                center tab(:) ;
*                lfB  lfB  lfB
*                l    l    lf(CW) .
*                Data Type:Abbreviation:Data Type
*                =
*                .TH
*                bit              :bi     : KBIT
*                byte             :by     : KBYTE
*                unsigned byte    :un by  : KUBYTE
*                ubyte            :uby    : KUBYTE
*                short            :sh     : KSHORT
*                unsigned short   :un sh  : KUSHORT
*                ushort           :ush    : KUSHORT
*                integer          :in     : KINT
*                unsigned integer :un in  : KUINT
*                uint             :ui     : KUINT
*                long             :lon    : KLONG
*                unsigned long    :un lon : KULONG
*                ulong            :ul     : KULONG
*                float            :fl     : KFLOAT
*                double           :do     : KDOUBLE
*                complex          :co     : KCOMPLEX
*                double complex   :do co  : KDCOMPLEX
*                dcomplex         :dc     : KDCOMPLEX
*                .T&
*                l s s.
*                Propagate Input Type - will not change the data type
*                .TE
*
*		 The scale value is specified by scale and the
*		 offset value by real_offset & imaginary_offset.
*
*         Input: source_object    - the object to get the segment from
*                segment_name     - the segment to convert
*		 data_type_string - data type to convert to
*		 scale            - value to scale the data by
*		 real_offset      - real value to offset data by
*		 imaginary_offset - imaginary value to offset data by
*
*        Output: destination_object - the resulting object
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: 
*    Written By: John M. Salas & Donna Koechner
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int 
lkconvert(
   kobject  source_object,
   char    *segment_name,
   char    *data_type_string,
   double   scale,
   double   real_offset,
   double   imaginary_offset,
   kobject  destination_object)
/* -library_def_end */

/* -library_code */
{
   char    *rtn              = "lkconvert";
   char    *lib              = "ksegops";
   klist   *objlist          = NULL;
   kaddr    data             = NULL;
   kobject  reference_object = NULL;
   int      data_type        = 0;
   int      dim;
   int      i;
   int      dt;
   int      grid;
   int     *begin;
   int     *end;
   int     *size;
   int     *region;


   /*-------------------------------------------------------------------
   | Check parameters passed in.
   -------------------------------------------------------------------*/
   if (kstrcmp(data_type_string, "Propagate Input Type") &&
       (data_type = kdatatype_to_define(data_type_string)) == 0)
   {
      kerror(lib, rtn, "Not a proper data type '%s'.", data_type_string);
      return FALSE;
   }

   /*-------------------------------------------------------------------
   | If no scaling is specified and no data type change is desired
   | the routine is finished.
   -------------------------------------------------------------------*/
   if (scale == 1.0 && real_offset == 0.0 && imaginary_offset == 0.0 &&
       data_type == 0)
   {
      return TRUE;
   }

   /*-------------------------------------------------------------------
   | Initialize kpds - need this to check grid type if location
   | segment is passed in.
   -------------------------------------------------------------------*/
   kpds_init();

   if (kstrcmp(segment_name, KDMS_SEGMENT_LOCATION) == 0)
   {
      KCALL(kpds_get_attribute(source_object, KPDS_LOCATION_GRID, &grid));
      KCALL(kpds_copy_location_attr(source_object, destination_object));
      KCALL(((reference_object = kdms_reference(source_object)) 
		!= KOBJECT_INVALID));
      objlist = klist_add(objlist,reference_object,"KOBJECT");
      if (data_type != 0)
      {
         KCALL(kpds_set_attribute(source_object, 
		KPDS_LOCATION_DATA_TYPE, data_type));
      }
      KCALL(kpds_set_attribute(source_object, KPDS_LOCATION_SCALING, KSCALE));

      KCALL(kpds_set_attribute(source_object, 
		KPDS_LOCATION_SCALE_FACTOR, scale));
      KCALL(kpds_set_attribute(source_object, 
		KPDS_LOCATION_SCALE_OFFSET, real_offset, imaginary_offset));

      KCALL(kpds_copy_location_data(source_object, destination_object, TRUE));

      (void) klist_free(objlist, (void(*)(klist*))lkcall_free);
      return TRUE;
   }

   else
   {
      if (!kdms_query_segment(source_object, segment_name))
      {
         kerror(lib, rtn, "Segment does not exist '%s'.", segment_name);
         return FALSE;
      }

      /*---------------------------------------------------------------
      | Get source and set destination attributes to do offset, scaling, 
      | data type, and optimal regions.
      ----------------------------------------------------------------*/
      KCALL(kdms_get_attribute(source_object, segment_name, KDMS_SIZE, &size));
      KCALL(kdms_get_attribute(source_object, segment_name, 
		KDMS_DIMENSION, &dim));
      KCALL(kdms_get_attribute(source_object, segment_name, 
		KDMS_DATA_TYPE, &dt));
      KCALL(kdms_get_attribute(source_object, segment_name, 
		KDMS_OPTIMAL_REGION_SIZE, &region));

      if (data_type == 0)
         data_type = dt;

      KCALL(kdms_set_attribute(destination_object, segment_name,
		KDMS_DATA_TYPE, data_type));

      /*----------------------------------------------------------------
      | Reference the destination_object and set the data type on
      | reference_object to be the same as the source_object so that
      | when you get the data from the source_object and put it into
      | the reference_object it will be of the same type.  Data Services
      | will convert the data type since the destination_object's
      | data type is the one the user specified.
      ----------------------------------------------------------------*/
      KCALL(((reference_object = kdms_reference(destination_object)) 
		!= KOBJECT_INVALID));
      objlist = klist_add(objlist,reference_object,"KOBJECT");

      if (data_type != dt)
      {
         KCALL(kdms_set_attribute(reference_object, segment_name, 
		KDMS_DATA_TYPE, dt));
      }

      /*----------------------------------------------------------------
      | Set scaling mode on the destination reference if scale and offset
      | specified.
      ----------------------------------------------------------------*/
      if (scale != 1.0 || real_offset != 0.0 || imaginary_offset != 0.0)
      {
         KCALL(kdms_set_attribute(reference_object, segment_name,
		KDMS_SCALING, KSCALE));
      }

      KCALL(kdms_set_attribute(reference_object, segment_name,
		KDMS_SCALE_FACTOR, scale));
      KCALL(kdms_set_attribute(reference_object, segment_name,
		KDMS_SCALE_OFFSET, real_offset, imaginary_offset));

      /*----------------------------------------------------------------
      | Get the data and put it into the reference_object so that the
      | data type conversion and scaling is done.  Since location data
      | can have 3 grid types, need to process it separately.
      -----------------------------------------------------------------*/
   
      if ((begin = (int *)kmalloc((kmax(5, dim) * sizeof(int *))))
         == NULL)
      {
         kerror(lib, rtn, "Could not allocate memory for temp begin variable.");
         (void) klist_free(objlist, (void(*)(klist*))lkcall_free);
         return FALSE;
      }
      objlist = klist_add(objlist, begin, "KMALLOC");
      
      if ((end = (int *)kmalloc((kmax(5, dim) * sizeof(int *))))
         == NULL)
      {
         kerror(lib, rtn, "Could not allocate memory for temp end variable.");
         (void) klist_free(objlist, (void(*)(klist*))lkcall_free);
         return FALSE;
      }
      objlist = klist_add(objlist, end, "KMALLOC");
   
      for (i = 0; i < kmax(5, dim); i++)
      {
         if (i >= dim)
         {
	    begin[i] = 0;
	    end[i] = 1;
         }
         else
         {
	    begin[i] = 0;
	    end[i] = region[i] - 1;
         }
      }
      
      data = NULL;
      do
      {
         if ((data = kdms_get_data(source_object, segment_name, 
		begin, end, data)) == NULL)
         {
            kerror(lib, rtn, "Could not allocate memory for data.");
            (void) klist_free(objlist, (void(*)(klist*))lkcall_free);
            return FALSE;
         }
         if (!kdms_put_data(reference_object, segment_name, begin, end, data))
         {
            kerror(lib, rtn, "Could not put data to destination object.");
	    kfree_and_NULL(data);
            (void) klist_free(objlist, (void(*)(klist*))lkcall_free);
            return FALSE;
         }
      }
      while (_kconvert_advance(begin, end, size, region, dim));

   (void) klist_free(objlist, (void(*)(klist*))lkcall_free);
   kfree_and_NULL(data);
   return TRUE;
   }
}

static int
_kconvert_advance(int *begin, int *end, int *size, int *unit, int dim)
{
   int i;
 
   for (i = 0; i < dim - 1; i++)
   {
      begin[i] += unit[i];
      end[i] += unit[i];
      if (begin[i] < size[i])
         return(TRUE);
      else
      {
         begin[i] = 0;
         end[i] = unit[i] - 1;
      }
   }
   begin[dim - 1] += unit[dim - 1];
   end[dim - 1] += unit[dim - 1];
 
   if (begin[dim - 1] >= size[dim - 1])
      return(FALSE);
 
   return (TRUE);
}
/* -library_code_end */
