 /*
  * Khoros: $Id: lkimportraw.c,v 1.16.2.3 1997/05/31 19:08:31 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lkimportraw.c,v 1.16.2.3 1997/05/31 19:08:31 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for kimportraw
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lkimportraw
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
#include "internals.h"
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lkimportraw - import raw binary data into a data segment
* 
*       Purpose: lkimport_raw can be used to read raw binary data and import
*		 it to a data segment.  lkimportraw will either create an 
*		 object with the new segment in it or append the new 
*		 segment to the optionally supplied add_obj object.
*
*		 Valid arch types are: KMACH_UNKNOWN, KMACH_LOCAL, 
*               	KMACH_LITTLE_ENDIAN_IEEE, KMACH_LITTLE_ENDIAN_VAX,
*               	KMACH_LITTLE_ENDIAN_ALPHA, KMACH_BIG_ENDIAN_IEEE,
*               	KMACH_BIG_ENDIAN_CRAY
*
*		 Valid data types are: KBIT, KBYTE, KUBYTE, KSHORT, KUSHORT, 
*		 KINT, KUINT, KLONG, KULONG, KFLOAT, KDOUBLE, KCOMPLEX,
*		 and KDCOMPLEX
*
*		 Valid color space values are: KNONE, KNONE, KGREY, KRGB, 
*                KCMY, KYIQ, KHSV, KHLS, KIHS, KXYZ, KUVW, KUCSUVW, 
*                KUCSSOW, KUCSLab, KUCSLuv, KUSERDEFINED	
*
*		 The raw data is assumed to be in WHDTE index order.
*
*         Input: raw_obj - raw source data, must have been opened with the KOBJ_RAW flag
*		 add_obj - optional object to import the data into
*		 arch - source architecture of the raw data
*		 type - data type of the raw data
*		 seg - string defining which data segment to insert data into; if this is "Comment" then the data will be placed in the comment field of the output object (only up to the first NULL in the data)
*		 w, h, d, t, e - size of raw data
*		 header_skip - skip this number of bytes before reading the data
*                colorspace - colorspace for the data
*
*        Output: out_obj - resulting destination object
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: 
*    Written By: Scott Wilson, Jeremy Worley
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int 
lkimportraw(
kobject raw_obj,
kobject add_obj,
int arch,
int type,
char *seg,
int w,
int h,
int d,
int t,
int e,
int header_skip,
int colorspace,
kobject out_obj)
/* -library_def_end */

/* -library_code */
  {
    char *lib = "ksegops", *rtn = "lkimportraw";
    klist *objlist=NULL;
    int dim;
    int size[5];
    int order[5];
    char *format=NULL;
    char *actual_seg=NULL;
    int actual_type;

/*
     * sanity check.  if data services recognizes as non-RAW, then 
     * something is amiss;  very likely, the object wasn't opened with 
     * the KOBJ_RAW flag.  Don't allow continuation.
     */
    KCALL(kdms_get_attribute(raw_obj, NULL, KDMS_FORMAT, &format));
    if (kstrcmp(KRAW, format) != 0)
    {
       kerror(lib, rtn, "The raw input object must be accessed in raw "
              "mode; this can be done by using the KOBJ_RAW flag in the "
              "open call. Quitting.");
       return(FALSE);
    }
    
    if (add_obj != KOBJECT_INVALID)
    {
       KCALL((add_obj = kpds_reference_object(add_obj)) != KOBJECT_INVALID);
       objlist = klist_add(objlist,add_obj,"KOBJECT");
    }

    KCALL(kdms_set_attribute(raw_obj,NULL, KDMS_ARCHITECTURE, arch));

/*
     * change the dimensionality, and index order as well, to match the
     * defaults for the desired segment.
     */
    if (kstrcmp(seg, KDMS_SEGMENT_VALUE) == 0)
    {
       size[0] = w;
       size[1] = h;
       size[2] = d;
       size[3] = t;
       size[4] = e;

       order[0] = KWIDTH;
       order[1] = KHEIGHT;
       order[2] = KDEPTH;
       order[3] = KTIME;
       order[4] = KELEMENTS;

       dim = 5;

       actual_seg = seg;
    }
    else if (kstrcmp(seg, KDMS_SEGMENT_MASK) == 0)
    {
       size[0] = w;
       size[1] = h;
       size[2] = d;
       size[3] = t;
       size[4] = e;

       order[0] = KWIDTH;
       order[1] = KHEIGHT;
       order[2] = KDEPTH;
       order[3] = KTIME;
       order[4] = KELEMENTS;

       dim = 5;

       actual_seg = seg;
    }
    else if (kstrcmp(seg, KDMS_SEGMENT_MAP) == 0)
    {
       size[0] = w;
       size[1] = h;
       size[2] = d;
       size[3] = t;
       size[4] = e;

       order[0] = KMAP_WIDTH;
       order[1] = KMAP_HEIGHT;
       order[2] = KMAP_DEPTH;
       order[3] = KMAP_TIME;
       order[4] = KMAP_ELEMENTS;

       dim = 5;

       actual_seg = seg;
    }
    else if (kstrcmp(seg, KDMS_SEGMENT_LOCATION) == 0)
    {
       size[0] = w;
       size[1] = h;
       size[2] = d;
       size[3] = e;

       order[0] = KWIDTH;
       order[1] = KHEIGHT;
       order[2] = KDEPTH;
       order[3] = KDIMENSION;

       dim = 4;

       actual_seg = seg;
    }
    else if (kstrcmp(seg, KDMS_SEGMENT_TIME) == 0)
    {
       size[0] = t;

       order[0] = KTIME;

       dim = 1;

       actual_seg = seg;
    }
    else if (kstrcmp(seg, "Comment") == 0)
    {
       size[0] = w;
       size[1] = h;
       size[2] = d;
       size[3] = t;
       size[4] = e;
 
       order[0] = KWIDTH;
       order[1] = KHEIGHT;
       order[2] = KDEPTH;
       order[3] = KTIME;
       order[4] = KELEMENTS;
 
       dim = 5;
 
       actual_seg = kstring_copy(KDMS_SEGMENT_VALUE,NULL);
    }
    else
    {
       kerror (lib,rtn,
	       "Non-polymorphic or bogus segment '%s' specified.", seg);
       (void)klist_free(objlist, (void(*)(klist*))lkcall_free);
       return (FALSE);
    }

/*
     * Tricky use of a new function.  Rename the segment.
     */
    if (kstrcmp(KDMS_SEGMENT_VALUE, actual_seg) != 0)
       KCALL(kdms_rename_segment(raw_obj, KDMS_SEGMENT_VALUE, actual_seg));

    /* Force the type to be UBYTE if we intend to read a comment */
    if (kstrcmp(seg, "Comment") == 0)
      {
        actual_type = KUBYTE;
      }
    else
      {
        actual_type = type;
      }

    KCALL(kdms_set_attributes(raw_obj, actual_seg, 
                        KDMS_DIMENSION, dim, 
			KDMS_INDEX_ORDER, order, 
                        KDMS_SIZE, size, 
			KDMS_DATA_TYPE, actual_type, 
			NULL));

    if (add_obj != KOBJECT_INVALID)
       KCALL(kdms_copy_object(add_obj, out_obj, TRUE, TRUE));

    KCALL(kdms_set_attribute(raw_obj,NULL,KDMS_RAW_OFFSET,header_skip));

    if (kstrcmp(seg, "Comment") != 0)
      {
        /* Want to read data into the value segment of the output object */
        KCALL(kdms_copy_segment_data(raw_obj, seg, out_obj, seg));
      }
    else
      {
        /* Want to read data into the comment area of the output object */
        int begin[5],end[5],i,nchars;
        char *data=NULL;
        char *oldcomment = NULL, *newcomment = NULL;

        /* Set up to read the whole data area into a linear buffer. */
        for(i=0; i<5; i++) begin[i]=0;
        for(i=0; i<5; i++) end[i]=size[i]-1;
        nchars = size[0]*size[1]*size[2]*size[3]*size[4]+1;

        /* Allocate the buffer and read the data into it */
        data = (char *)kmalloc(nchars*sizeof(char));
        if (data == NULL)
          {
            kerror(lib, rtn, "Unable to obtain comment data buffer");
            (void)klist_free(objlist, (void(*)(klist*))lkcall_free);
            return(FALSE);
          }
        objlist = klist_add(objlist,data,"KMALLOC");
        KCALL(kdms_get_data(raw_obj,actual_seg,begin,end,(kaddr)data));

        /* Make sure that the new data is NULL terminated */
        data[nchars-1]='\0';
        
        /* Get whatever comment is already in the output object and
           append the new one to it before writing it back to the 
           output object. Put a space between the original comment
           and the new one if the old one isn't NULL. */
        KCALL(kpds_get_attribute(out_obj,KPDS_COMMENT,&oldcomment));
        if (oldcomment != NULL) newcomment =
                                  kstring_3cat(oldcomment," ",data,NULL);
        else newcomment = kstring_cat(oldcomment,data,NULL);
        objlist = klist_add(objlist,newcomment,"KMALLOC");
        KCALL(kpds_set_attribute(out_obj,KPDS_COMMENT,newcomment));
      }
    
    KCALL(kcolor_set_attribute(out_obj,KCOLOR_COLORSPACE,colorspace));

    (void)klist_free(objlist, (void(*)(klist*))lkcall_free);

    return TRUE;
  }
/* -library_code_end */
