/*
 * Khoros: $Id: attributes.h,v 1.3 1997/01/20 04:49:43 mirage Exp $
 */
 
 
/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: definitions and prototypes for 
   >>>>		       data services attribute code
   >>>>
   >>>>    Written By: Steve Kubica
   >>>>
   >>>>          Date: October 12, 1995
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kdataccess_attributes_h_
#define _kdataccess_attributes_h_


	/*-------------------------------------*
	|       #include
	---------------------------------------*/

#include <bootstrap.h>

	/*-------------------------------------*
	|       #defines
	---------------------------------------*/

/*-- attribute types --*/
#define KDAS_QUASI_ATTRIBUTE   1
#define KDAS_GENERIC_ATTRIBUTE 2

	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/

   /*----------------------------------------------------------------
   |   kdas_base_attribute
   |   . . . . . . . . . . . . . . . 
   |
   |   This is functionally the attribute base class, and contains 
   |   the characteristics common to both quasi and generic attributes.
   |   The type of attribute is distinguished by the type field.
   |   attributes can be either quasi or generic attributes.  The
   |   name of the attribute is kept here and is in tokenized form 
   |   so it can be efficiently checked.
   \-----------------------------------------------------------------*/
typedef struct _kdas_base_attribute
{
   /*-- attribute type : KDAS_QUASI_ATTRIBUTE or KDAS_GENERIC_ATTRIBUTE --*/
   int    type;		

   /*-- tokenized representation of the attribute --*/
   ktoken attribute;

} kdas_base_attribute;

   /*----------------------------------------------------------------
   |   kdas_generic_attribute
   |   . . . . . . . . . . . . . . . 
   |
   |   This is a generic "brand X" attribute of a specific size, type,
   |   and number of arguments.  Generic attributes can flagged as
   |   permanent implying that they should be stored.  The actual
   |   data for a generic attribute is stored as a single block here.
   \-----------------------------------------------------------------*/
typedef struct _kdas_generic_attribute
{
   /*-- attribute type and token --*/
   int    type;
   ktoken attribute;

   /*-- attribute characteristics --*/
   int datatype;
   int argsize;	
   int numargs;	
   
   /*-- persistence of the attribute --*/
   int permanent;

   /*-- actual storage (tm) of the attribute values --*/
   kaddr value;

} kdas_generic_attribute;

   /*----------------------------------------------------------------
   |   kdas_quasi_attribute
   |   . . . . . . . . . . . . . . . 
   |
   |   This is an attribute which consists of nothing except handler
   |   functions which deal with requests for storage, retrieval,
   |   matching, copying, querying, and printing.  There is not
   |   any actual managed storage space for a quasi attribute.
   \-----------------------------------------------------------------*/
typedef struct _kdas_quasi_attribute 
{
   /*-- attribute type and token --*/
   int    type;
   ktoken attribute;

   int (*get)   PROTO((kobject, ktoken, ktoken, kaddr, kva_list *));
   int (*set)   PROTO((kobject, ktoken, ktoken, kaddr, kva_list *));
   int (*match) PROTO((kobject, kobject, ktoken, ktoken, kaddr, kaddr));
   int (*copy)  PROTO((kobject, kobject, ktoken, ktoken, kaddr, kaddr));
   int (*query) PROTO((kobject, ktoken, ktoken,kaddr,int *,int *,int *,int *));
   int (*print) PROTO((kobject, ktoken, ktoken, kaddr, kfile *));

   /*-- attribute specific client data to pass in for each function --*/
   kaddr    client_data;

} kdas_quasi_attribute;

   /*----------------------------------------------------------------
   |   kdas_attribute
   |   . . . . . . . . . . . . . . . 
   |
   |   This is the data services attribute class, which is really
   |   just a union of a generic and quasi attribute.
   \-----------------------------------------------------------------*/
typedef union _kdas_attribute 
{

   kdas_base_attribute     base;
   kdas_generic_attribute  generic;
   kdas_quasi_attribute    quasi;

} kdas_attribute;

	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/

	/*-------------------------------------*
	|       macros
	---------------------------------------*/

	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

#endif /* _kdataccess_attributes_h_ */
/* Don't add after this point */
