/*
 * Khoros: $Id: data.h,v 1.1 1997/01/23 21:15:34 mirage Exp $
 */
 
 
/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: 
   >>>>
   >>>>    Written By: Steve Kubica
   >>>>
   >>>>          Date: November 28, 1996
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kgeom_data_h_
#define _kgeom_data_h_


	/*-------------------------------------*
	|       #include
	---------------------------------------*/

	/*-------------------------------------*
	|       #defines
	---------------------------------------*/

	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/

/* 
 *    ================================================================== 
 *    calculate the number of points in block of data
 *    ==================================================================
 */

/*-----------------------------------------------------------
|  Routine Name: kgeom_number_locs
|         Input: p - primitive under consideration
|		 o - object for primitive 
|        Output: none
|       Returns: number of location points
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_number_locs(p, o)  \
   (kgeom_number((kgeom_primitive *) p, KPER_VERTEX) * o->location_dim)

/*-----------------------------------------------------------
|  Routine Name: kgeom_number_cols
|         Input: p - primitive under consideration
|		 o - object for primitive
|        Output: none
|       Returns: number of color points
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_number_cols(p, o) \
   (kgeom_number((kgeom_primitive *) p, o->layout) * ((o->has_alpha) ? 4 : 3))

/*-----------------------------------------------------------
|  Routine Name: kgeom_number_norms
|         Input: p - primitive under consideration
|		 o - object for primitive
|        Output: none
|       Returns: number of normal points
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_number_norms(p, o) \
   (kgeom_number((kgeom_primitive *) p, o->layout) * o->location_dim)

/*-----------------------------------------------------------
|  Routine Name: kgeom_number_tcoords
|         Input: p - primitive under consideration
|		 o - object for primitive
|        Output: none
|       Returns: number of tcoord points
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_number_tcoords(p, o)  \
   (kgeom_number((kgeom_primitive *) p, KPER_VERTEX) * o->texture_coord_dim)

/*-----------------------------------------------------------
|  Routine Name: kgeom_number_radii
|         Input: p - type of primitive under consideration
|		 o - object for primitive
|        Output: none
|       Returns: number of radius points
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_number_radii(p, o)  \
   (kgeom_number((kgeom_primitive *)p, KPER_VERTEX))


/* 
 *    ================================================================== 
 *    allocate a block of data
 *    ==================================================================
 */

/*-----------------------------------------------------------
|  Routine Name: kgeom_alloc_locs
|         Input: p - primitive under consideration
|		 o - object for primitive
|        Output: none
|       Returns: allocated block of location data
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_alloc_locs(p, o)  \
        kgeom_alloc(kgeom_number_locs(p, o))

/*-----------------------------------------------------------
|  Routine Name: kgeom_alloc_cols
|         Input: p - primitive under consideration
|		 o - object for primitive
|        Output: none
|       Returns: allocated block of color data
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_alloc_cols(p, o)  \
        kgeom_alloc(kgeom_number_cols(p, o))

/*-----------------------------------------------------------
|  Routine Name: kgeom_alloc_norms
|         Input: p - primitive under consideration
|		 o - object for primitive
|        Output: none
|       Returns: allocated block of normal data
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_alloc_norms(p, o)  \
        kgeom_alloc(kgeom_number_norms(p, o))

/*-----------------------------------------------------------
|  Routine Name: kgeom_alloc_tcoords
|         Input: p - primitive under consideration
|		 o - object for primitive
|        Output: none
|       Returns: allocated block of texture coordinate data
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_alloc_tcoords(p, o)  \
        kgeom_alloc(kgeom_number_tcoords(p, o))

/*-----------------------------------------------------------
|  Routine Name: kgeom_alloc_radii
|         Input: p - primitive under consideration
|		 o - object for primitive
|        Output: none
|       Returns: allocated block of radius data
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_alloc_radii(p, o)  \
        kgeom_alloc(kgeom_number_radii(p, o))

/* 
 *    ================================================================== 
 *    duplicate a block of data
 *    ==================================================================
 */

/*-----------------------------------------------------------
|  Routine Name: kgeom_dup_locs
|         Input: d - data to duplicate
|		 o - object for primitive 
|        Output: none
|       Returns: duplicated block of location data
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_dup_locs(d, p, o)  \
        kgeom_dup(d, kgeom_number_locs(p, o))

/*-----------------------------------------------------------
|  Routine Name: kgeom_dup_cols
|         Input: d - data to duplicate
|                p - primitive under consideration
|		 o - object for primitive
|        Output: none
|       Returns: duplicated block of color data
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_dup_cols(d, p, o)  \
        kgeom_dup(d, kgeom_number_cols(p, o))

/*-----------------------------------------------------------

|  Routine Name: kgeom_dup_norms
|         Input: d - data to duplicate
|                p - primitive under consideration
|		 o - object for primitive
|        Output: none
|       Returns: duplicated block of normal data
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_dup_norms(d, p, o)  \
        kgeom_dup(d, kgeom_number_norms(p, o))

/*-----------------------------------------------------------
|  Routine Name: kgeom_dup_tcoords
|         Input: d - data to duplicate
|                p - primitive under consideration
|		 o - object for primitive
|        Output: none
|       Returns: duplicated block of texture coordinate data
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_dup_tcoords(d, p, o)  \
        kgeom_dup(d, kgeom_number_tcoords(p, o))

/*-----------------------------------------------------------
|  Routine Name: kgeom_dup_radii
|         Input: d - data to duplicate
|                p - primitive under consideration
|		 o - object for primitive
|        Output: none
|       Returns: duplicated block of radius data
|    Written By: Steve Kubica
|          Date: December 23, 1996
------------------------------------------------------------*/
#define kgeom_dup_radius(d, p, o)  \
        kgeom_dup(d, kgeom_number_radii(p, o))

	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/

	/*-------------------------------------*
	|       macros
	---------------------------------------*/

	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

#endif /* _kgeom_primitive_h_ */
/* Don't add after this point */
