/*
 * Khoros: $Id: kgeom.h,v 1.1.2.1 1997/03/18 23:15:09 mirage Exp $
 */
 
/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Definitions and prototypes for library geom
   >>>>
   >>>>       Toolbox: KUBOX
   >>>>
   >>>>    Written By: Steve Kubica
   >>>>
   >>>>          Date: December 18, 1996
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kgeom_h_
#define _kgeom_h_


	/*-------------------------------------*
	|       #include
	---------------------------------------*/

#include "data.h"
#include "primitive.h"
#include "object.h"

	/*-------------------------------------*
	|       #defines
	---------------------------------------*/


/*-------------------------------------------------------------------
|  Object Type Identifiers 
|
|	These are used to identify the type of geometry primitive. 
|       They will be stored in the geometry file.
-------------------------------------------------------------------*/

#define KGEOM_OBJECT                            0
#define KGEOM_QUADMESH                          1
#define KGEOM_OCTMESH                           2
#define KGEOM_TEXTURE2D                         3
#define KGEOM_TEXTURE3D                         4

/* Geometric Primitives */
#define KGEOM_POLYLINE_DISJOINT                 501
#define KGEOM_POLYLINE_CONNECTED                502
#define KGEOM_TRIANGLES_DISJOINT                503
#define KGEOM_TRIANGLES_CONNECTED               504
#define KGEOM_SPHERES                           505     
#define KGEOM_CYLINDERS                         506     
#define KGEOM_DIRECTED_POINTS                   507     

/* Annotation Primitives */
#define KGEOM_TEXT                              550
#define KGEOM_MARKERS                           551
#define KGEOM_POLYGON                           552     
#define KGEOM_RECTANGLES                        553
#define KGEOM_ELLIPSES                          554

/* Camera Primitives */
#define KGEOM_OBJECT_XFORM                      600
#define KGEOM_CAMERA_ORIENTATION                601
#define KGEOM_VR_EVENT                          602

/* View Transform */
#define KGEOM_VIEW_XFORM                        603


/* ======= */

/*
 * defines for the different modeling types
 */
#define KWORLD_SPACE 	0
#define KNDC_SPACE 	1

/*
 * defines for different layouts 
 */
#define KPER_VERTEX 	0
#define KPER_LINE 	1
#define KPER_FACE 	2
#define KPER_CELL 	3

/* 
 * defines for view parameters.
 */
#define KPROJECTION_PERSPECTIVE 0
#define KPROJECTION_PARALLEL    1

/*
 * object transform paramters.
 */
#define KABSOLUTE               0
#define KRELATIVE               1

/* ======= */

/*
 *   Default Values  
 */ 
#define KGEOM_AMBIENT_COLOR_DEFAULT		{0.25, 0.25, 0.25}
#define KGEOM_DIFFUSE_COLOR_DEFAULT           	{0.5, 0.5, 0.5} 
#define KGEOM_COLOR_DEFAULT           	        {1.0, 1.0, 1.0} 
#define KGEOM_LAYOUT_DEFAULT			KPER_VERTEX
#define KGEOM_MODELING_SPACE_DEFAULT           	KWORLD_SPACE
#define KGEOM_OPACITY_DEFAULT           	1.0 
#define KGEOM_SPECULAR_COLOR_DEFAULT		{0.25, 0.25, 0.25} 
#define KGEOM_SPECULAR_EXPONENT_DEFAULT		10.0
#define KGEOM_VISIBLE_DEFAULT          		TRUE 
#define KGEOM_MATRIX_DEFAULT   	                {1, 0, 0, 0, \
                                                 0, 1, 0, 0, \
                                                 0, 0, 1, 0, \
                                                 0, 0, 0, 1}

#define KGEOM_OBJECT_XFORM_RELATIVE_DEFAULT     KRELATIVE
#define KGEOM_OBJECT_XFORM_MATRIX_DEFAULT   	{1, 0, 0, 0, \
                                                 0, 1, 0, 0, \
                                                 0, 0, 1, 0, \
                                                 0, 0, 0, 1}

#define KGEOM_CAMERA_EYE_DEFAULT                {0, 0, 0}
#define KGEOM_CAMERA_AT_DEFAULT                 {0, 0, 1}
#define KGEOM_CAMERA_UP_VECTOR_DEFAULT          {0, 1, 0}
#define KGEOM_CAMERA_FIELD_OF_VIEW_DEFAULT      90.0
#define KGEOM_CAMERA_HITHER_DEFAULT             -1.0
#define KGEOM_CAMERA_YON_DEFAULT                 1.0
#define KGEOM_CAMERA_PROJECTION_DEFAULT         KPROJECTION_PERSPECTIVE
#define KGEOM_CAMERA_IS_STEREO_DEFAULT          FALSE
#define KGEOM_CAMERA_EYE_DISTANCE_DEFAULT       0.0
#define KGEOM_CAMERA_FOCAL_LENGTH_DEFAULT       1.0

#define KGEOM_VIEW_XFORM_MATRIX_DEFAULT   	{1, 0, 0, 0, \
                                                 0, 1, 0, 0, \
                                                 0, 0, 1, 0, \
                                                 0, 0, 0, 1}

#define KGEOM_VR_XFORM_ROTATION_DEFAULT          {0.0, 0.0, 0.0}
#define KGEOM_VR_XFORM_TRANSLATION_DEFAULT       {0.0, 0.0, 0.0}
#define KGEOM_VR_BUTTONS_DEFAULT                 {0, 0, 0, 0, 0, 0, 0, 0, 0}
#define KGEOM_VR_EVENTS_DEFAULT                  FALSE
#define KGEOM_VR_CLOCK_DEFAULT                   0

	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/

/************************************************************
*
*  Routine Name: kgeom_done_writing - close associated file after writing
*
*       Purpose: This function will close the file associated
*                with the geometry object after a write has been 
*                completed.
*
*                In general, the \f(CWkgeom_write_object()\fP function
*                should be used instead of this function. This
*                function has been made public to provide complete
*                flexibility for writing a geometry object.  It should
*                only be used when you wish to write a geometry object
*                primitive-by-primitive.  More information on this is
*                available in the \f(CWkgeom_write_primitive()\fP man
*                page.
*
*                The fid internal to the geometry object specifies the
*                file which will be closed.  In general, this fid is
*                set when the geometry header was written by the
*                \f(CWkgeom_write_header()\fP function.
*
*         Input: object - geometry object to write
*                filename - filename to write header to
*        Output: 
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*
*    Written By: Steve Kubica
*          Date: December 18, 1996
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: int kgeom_done_writing(kgeom_object *object)
*
*************************************************************/
#define kgeom_done_writing(object) \
        kgeom_done_with_file(object)

/************************************************************
*
*  Routine Name: kgeom_done_reading - close associated file after reading
*
*       Purpose: This function will close the file associated
*                with the geometry object after a read has been 
*                completed.
*
*                In general, the \f(CWkgeom_read_object()\fP function
*                should be used instead of this function. This
*                function has been made public to provide complete
*                flexibility for reading a geometry object.  It should
*                only be used when you wish to read a geometry object
*                primitive-by-primitive.  More information on this is
*                available in the \f(CWkgeom_read_primitive()\fP man
*                page.
*
*                The fid internal to the geometry object specifies the
*                file which will be closed.  In general, this fid is
*                set when the geometry header was read by the
*                \f(CWkgeom_read_header()\fP function.
*
*         Input: object - geometry object to write
*                filename - filename to write header to
*        Output: 
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*
*    Written By: Steve Kubica
*          Date: December 18, 1996
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: int kgeom_done_reading(kgeom_object *object)
*
*************************************************************/
#define kgeom_done_reading(object) \
        kgeom_done_with_file(object)


/*------------------------------------------------------------------------
| Attribute:    KPDS_ASSOCIATED_GEOMETRY
| Type:         kgeom_object *
| Default:      KNONE
| Category:     object
| Legal Values: 
| Description:  This attribute can be used to store a geometry object
|		within a data services kobject.  It must be defined
|		for the attribute to exist by calling the
|               kpds_initialize_geometry_attributes function. 
| Access:       
| Persistence:  permanent
------------------------------------------------------------------------*/
#define KPDS_ASSOCIATED_GEOMETRY "associatedGeometry"


/* handy typedefs for traversing xyz, rgb and rgba data */
typedef struct _kvertex {float x; float y; float z;} kvertex;
typedef struct _krgb    {float r; float g; float b;} krgb;
typedef struct _krgba   {float r; float g; float b; float a;} krgba;

	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/

extern int KTYPE_GEOM_OBJECT;

	/*-------------------------------------*
	|       macros
	---------------------------------------*/


	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/* data.c */
int    kgeom_number 	      PROTO((kgeom_primitive *, int));
float *kgeom_alloc  	      PROTO((int));
float *kgeom_dup    	      PROTO((float *, int));
float *kgeom_resize_vectors   PROTO((float *, int, int, int));
float *kgeom_resample_vectors PROTO((float *, int, int, int));


/* object.c */
kgeom_object *kgeom_new_object       		PROTO((void));
void          kgeom_blast_object     		PROTO((kgeom_object *));
kgeom_object *kgeom_copy_object      		PROTO((kgeom_object *, 
						       kgeom_object *));
int           kgeom_write_header     		PROTO((kgeom_object *, 
						       char *));
kgeom_object *kgeom_read_header      		PROTO((char *));
int           kgeom_done_with_file   		PROTO((kgeom_object *));
int           kgeom_write_object                PROTO((kgeom_object *, 
						       char *));
int           kgeom_start_writing_object        PROTO((kgeom_object *));
int           kgeom_write_object_and_primitives PROTO((kgeom_object *));
int           kgeom_finish_writing_object       PROTO((kgeom_object *));
kgeom_object *kgeom_read_object                 PROTO((char *));
int           kgeom_start_reading_object        PROTO((kgeom_object *));
int           kgeom_read_object_and_primitives  PROTO((kgeom_object *));
int           kgeom_finish_reading_object       PROTO((kgeom_object *));

int           	 kgeom_number_primitives PROTO((kgeom_object *));
int           	 kgeom_add_primitive     PROTO((kgeom_object *, 
						kgeom_primitive *));
kgeom_primitive *kgeom_get_primitive     PROTO((kgeom_object *, int));
int              kgeom_remove_primitive  PROTO((kgeom_object *, 
						kgeom_primitive *));
kgeom_object    *kgeom_get_root          PROTO((kgeom_object *));


/* primitive.c */
kgeom_primitive *kgeom_new_primitive    PROTO((int));
int              kgeom_blast_primitive  PROTO((kgeom_primitive *));

int              kgeom_write_primitive  PROTO((kgeom_object *, 
					       kgeom_primitive *));
kgeom_primitive *kgeom_read_primitive   PROTO((kgeom_object *));

kgeom_primitive *kgeom_copy_primitive   PROTO((kgeom_object *, 
					       kgeom_primitive *, 
					       kgeom_primitive *));
/* cast.c */
kgeom_object *kgeom_new_location_dim  PROTO((kgeom_object *, int));
kgeom_object *kgeom_new_layout        PROTO((kgeom_object *, int));
kgeom_object *kgeom_new_has_alpha     PROTO((kgeom_object *, int));

/* octmesh.c */
int kgeom_write_octmesh_header PROTO((kgeom_object *, int, int, int, char *,
				      int, int, int, int));
int kgeom_read_octmesh_header  PROTO((kgeom_object *, int *, int *, int *,
				      char **, int *, int *, int *, int *));
int kgeom_write_octmesh_plane  PROTO((kgeom_object *, int, int, 
				      float *, kubyte *, float *, float *));
int kgeom_read_octmesh_plane   PROTO((kgeom_object *, int, int, 
				      float *, kubyte *, float *, float *));

/* types.c */
int kpds_initialize_geometry_attributes PROTO((void));
int kdefine_geometry_object_type        PROTO((void));

	/* Ghostwriter Prototypes:  Do Not Erase This Comment */


#endif /* _kgeom_h_ */
/* Don't add after this point */
