/*
 * Khoros: $Id: primitive.h,v 1.1.2.1 1997/03/18 23:15:15 mirage Exp $
 */
 
 
/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Geometry Primitive Definition
   >>>>
   >>>>    Written By: Steve Kubica
   >>>>
   >>>>          Date: November 28, 1996
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kgeom_primitive_h_
#define _kgeom_primitive_h_


	/*-------------------------------------*
	|       #include
	---------------------------------------*/

	/*-------------------------------------*
	|       #defines
	---------------------------------------*/

	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/

   /*----------------------------------------------------------------
   |   geometry primitives
   |   . . . . . . . . . . . . . . 
   |   These are the geometry primitives which are storable.  They all
   |   start with an integer type.  They will be unioned into the
   |   kgeom_primitive structure.  
   |
   |   The specifics of the structure can either be accessed by casting 
   |   the kgeom_primitive to the appropriate subtype, or by using the 
   |   name of the union entry for the subtype of interest.
   \-----------------------------------------------------------------*/

/*-- KGEOM_POLYLINE_DISJOINT and KGEOM_POLYLINE_CONNECTED --*/
typedef struct _kgeom_polyline
{
   int  type;
   int  nverts;
   int  linetype;
   int  linewidth;

   float *locs;   float *cols;

} kgeom_polyline;


/*-- KGEOM_TRIANGLES_DISJOINT and KGEOM_TRIANGLES_CONNECTED --*/
typedef struct _kgeom_triangles
{
   int   type;
   int   nverts;
   char *texture;

   float *locs;   float *cols;   float *norms;   float *tcoords;
   
} kgeom_triangles;


/*-- KGEOM_SPHERES --*/
typedef struct _kgeom_spheres
{
   int   type;
   int   nverts;

   float *locs;   float *cols;   float *radii;
   
} kgeom_spheres;


/*-- KGEOM_TEXT --*/
typedef struct _kgeom_text
{
   int   type;
   int   nverts;

   kstring *strings;
   kstring *fonts;
   
   float *locs;   float *offsets;   float *ups;   float *flows;
   float *cols;   float *bgcols;
   
} kgeom_text;


/*-- KGEOM_CYLINDERS --*/
typedef struct _kgeom_cylinders
{
   int   type;
   int   nverts;

   float *locs;   float *cols;   float *rad1;   float *rad2;
   
} kgeom_cylinders;

/*-- KGEOM_DIRECTED_POINTS --*/
typedef struct _kgeom_directed_points
{
   int   type;
   int   nverts;

   float *locs;   float *cols;   float *norms;
   
} kgeom_directed_points;


/*-- KGEOM_MARKERS --*/
typedef struct _kgeom_markers
{
   int   type;
   int   nverts;

   int   markertype;

   float *locs;   float *cols;   float *bgcols;
   
} kgeom_markers;


/*-- KGEOM_POLYGON --*/
typedef struct _kgeom_polygon
{
   int  type;
   int  nverts;
   int  linetype;
   int  linewidth;

   float *locs;   float *cols;   float *bgcol;

} kgeom_polygon;


/*-- KGEOM_RECTANGLES --*/
typedef struct _kgeom_rectangles
{
   int  type;
   int  nverts;
   int  linetype;
   int  linewidth;

   float *locs;   float *cols;   float *bgcols;

} kgeom_rectangles;


/*-- KGEOM_ELLIPSES --*/
typedef struct _kgeom_ellipses
{
   int  type;
   int  nverts;
   int  linetype;
   int  linewidth;

   float *locs;   float *cols;   float *bgcols;   float *rad1;   float *rad2;

} kgeom_ellipses;


/*-- KGEOM_QUADMESH --*/
typedef struct _kgeom_quadmesh
{
   int   type;
   int   width, height;
   char *texture;

   float *locs;   float *cols;   float *norms;   float *tcoords;
   
} kgeom_quadmesh;


/*-- KGEOM_OCTMESH --*/
typedef struct _kgeom_octmesh
{
   int   type;
   int   width, height, depth;
   char *texture;

   float *locs;   kubyte *cols;   float *norms;   float *tcoords;
   
} kgeom_octmesh;


/*-- KGEOM_TEXTURE2D --*/
typedef struct _kgeom_texture2D
{
   int   type;
   int   width, height;

   kubyte *cols;
   
} kgeom_texture2D;


/*-- KGEOM_TEXTURE3D --*/
typedef struct _kgeom_texture3D
{
   int   type;
   int   width, height, depth;

   kubyte *cols;
   
} kgeom_texture3D;


/*-- KGEOM_VIEW_XFORM --*/
typedef struct _kgeom_view_xform
{
   int   type;

   float matrix[16];

} kgeom_view_xform;


/*-- KGEOM_OBJECT_XFORM --*/
typedef struct _kgeom_object_xform
{
   int     type;

   float   matrix[16];
   float   translate[3];
   int     relative;

   kstring name;
   
} kgeom_object_xform;


/*-- KGEOM_CAMERA_ORIENTATION --*/
typedef struct _kgeom_camera_orientation
{
   int   type;

   float eye[3];
   float at[3];
   float up[3];
   
   float field_of_view;
   float hither;
   float yon;
   
   int   projection;
   int   is_stereo;
   
   float eye_distance;
   float focal_length;
   
} kgeom_camera_orientation;


/*-- KGEOM_VR_EVENT --*/
typedef struct _kgeom_vr_event
{
   int   type;

   int   have_button_events;
   int   have_rotations;
   int   have_translations;

   float rotate[3];
   float translate[3];

   int 	 buttons[9];
   int 	 clock;
   
} kgeom_vr_event;


/*-- KGEOM_OBJECT --*/
typedef struct _kgeom_subobject
{
   int type;

   kaddr object;  /*-- can only point to kgeom_object --*/

} kgeom_subobject;


/*-- kgeom_primitive union --*/
typedef union 
{
   int              	    type;
  
   kgeom_polyline   	    lines;
   kgeom_triangles  	    triangles;
   kgeom_spheres    	    spheres;
   kgeom_text       	    text;
   kgeom_cylinders  	    cylinders;
   kgeom_directed_points    directed_points;
   kgeom_markers  	    markers;
   kgeom_polygon  	    polygon;
   kgeom_rectangles  	    rectangles;
   kgeom_ellipses  	    ellipses;
  
   kgeom_quadmesh   	    quadmesh;
   kgeom_octmesh    	    octmesh;
      	
   kgeom_texture2D  	    texture2D;
   kgeom_texture3D  	    texture3D;

   kgeom_view_xform         view_transform;
   kgeom_object_xform       object_transform;
   kgeom_camera_orientation camera_orientation;
   kgeom_vr_event  	    vr_event;

   kgeom_subobject          object;

} kgeom_primitive;


	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/

	/*-------------------------------------*
	|       macros
	---------------------------------------*/

	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

#endif /* _kgeom_primitive_h_ */
/* Don't add after this point */
