 /*
  * Khoros: $Id: Rast.c,v 1.8.2.4 1997/05/29 22:51:12 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Rast.c,v 1.8.2.4 1997/05/29 22:51:12 mirage Exp $";

#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros RAST Data Service Routines
   >>>>		
   >>>>  Private:
   >>>>                 init_rast()
   >>>>                 init_rast()
   >>>>   Static:
   >>>>                 rast_check()
   >>>>                 rast_input()
   >>>>                 rast_output()
   >>>>                 rast_prep()
   >>>>                 rast_destroy()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

#if !defined(KRAST_DEF)

void
init_rast(void) /*-- NULL rast initialization --*/
{
   return;
}

#else

#include "kdatafmt/rast.h"

static int rast_check  	PROTO((int));
static int rast_input  	PROTO((kobject, int, int));
static int rast_output 	PROTO((kobject, int, int));
static int rast_destroy PROTO((kobject));
static int rast_prep    PROTO((kobject));

/*=====================================================================*/

/*-----------------------------------------------------------
|  Routine Name: init_rast
|       Purpose: initialize the sun raster file format
------------------------------------------------------------*/
void
init_rast(void)
{
   kdms_add_file_format("rast", 
			"Sun Rasterfile File Format (rast)",
			rast_check, 
			rast_input, 
			rast_output, 
			rast_destroy,
			rast_prep, 
			NULL, NULL, NULL, NULL, NULL);

   /*-- need some colorspace attributes --*/
   kcolor_init();

   return;
}

/*=====================================================================*/

/*
 *  Internal Resource Structure 
 *  rast  -  Sun Rasterfile File Format (rast)
 */
typedef struct
{
   int  fid;
   rast *image;
}
ResourceStruct;

/****    =============================================================
 ****    Format Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: rast_check - Checks to see if the file is a rasterfile
|
|       Purpose: This function is used to check the first few bytes 
|                of a fid to see if it is a sun rasterfile.
|
|         Input: fid - file id of a file to check
|        Output: none
|	Returns: TRUE if the file is a sun rasterfile
|
|    Written By: Mark Young
|          Date: July 13, 1992
| Modifications:
|
------------------------------------------------------------*/
static int
rast_check(int fid)
{
   int machtype;
   int magic = 0;


   /*-- save current machine type, then set to read sun machine type --*/
   machtype = kfile_getmachtype(fid);
   kfile_setmachtype(fid, KMACH_BIG_ENDIAN_IEEE);

   /*-- read magic identifier --*/
   kread_int(fid, &magic, (size_t)1);

   /*-- restore original machine type --*/
   kfile_setmachtype(fid, machtype);

   /*-- return whether the identifier matches the Rasterfile Magic define --*/
   return (magic == RAS_MAGIC);
}

/*-----------------------------------------------------------
|
|  Routine Name: rast_input - Read a sun rasterfile image
|
|       Purpose: This function is used to read in the image if the
|                supplied data is rast image.  We can simply set up
|		 the header and initialize the data segments.
|
|         Input: object - the rast object to be initialized
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Mark Young
|          Date: July 13, 1992
| Modifications:
|
------------------------------------------------------------*/
/*ARGSUSED */
static int
rast_input(kobject object, int fid, int flags)
{
   ResourceStruct *resources;
   rast *image;

   int datatype = KUBYTE;
   int i;

   int beg[KDAS_MAX_DIM] = {0, 0, 0, 0, 0};
   int end[KDAS_MAX_DIM];
   
   int size [KDAS_MAX_DIM];
   int order[KDAS_MAX_DIM];
   int num = 1;

   kaddr   data, tdata = NULL;
   kubyte *d, *img;
   int     pnum;


   /*-- get the resource structure --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = (ResourceStruct *) kcalloc((size_t)1,
					     sizeof(ResourceStruct))) == NULL)
         return(FALSE);

      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid = fid;

   if ((image = resources->image = rast_fdread(fid)) == NULL)
   {
      /* _kdms_set_error(KDMS_EFMT_FAILURE); */
      return FALSE;
   }

   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (image->ras_width != 0 && image->ras_height != 0)
   {
      /*-- create the value segment --*/
      kpds_create_value(object);

      if (image->ras_depth == 1)
      {
         datatype = KBIT;
         num = 1;


	 /* invert the values since bit 1's are black and bit 0's are white */
	 for (i = 0; i < image->ras_length; i++)
	    image->data[i] = ~image->data[i];
      }
      else
      {
         datatype = KUBYTE;
         num = image->ras_depth >> 3;
      }

      size[0] = image->ras_width;
      size[1] = image->ras_height;
      size[2] = size[3] = 1;
      size[4] = num;
      
      order[0] = KWIDTH;
      order[1] = KHEIGHT;
      order[2] = KDEPTH;
      order[3] = KTIME;
      order[4] = KELEMENTS;

      /*-----------------------------------------------------------------
      |   if the bit depth is 24 or 32, then things could be difficult.
      |   for some unfathomable reason, data can be stored either BGR
      |   or RGB, depending on the type field.  If its 32 bit, then the
      |   first byte of each pixel is padding.
      \----------------------------------------------------------------*/
      switch (num)
      {
	 case 1:
	 {
	    tdata = image->data;  /*-- no transposing necessary --*/
	 } break;

	 case 3:
	 {
	    /*-------------------------------------------------------------
	    | 	if the type is RGB, things are easy,  if not, then the data
	    |   is probably stored BGR, in which case things are a hassle.
	    |   Technically, we gotta check to see if conversion was from 
            |   IFF or TIFF, in which case, things could be different, 
	    |   but this is good enough for now.
	    \------------------------------------------------------------*/
	    if (image->ras_type != RT_FORMAT_RGB)
	    {
	       pnum = size[0] * size[1];

	       if ((data = kmalloc((size_t)(pnum * 3))) == NULL)
		  return FALSE;

	       /*-- swap bgr to be rgb --*/
	       img = (kubyte *) image->data;
	       for (i = 0, d = (kubyte *) data; i < pnum; i++, img += 3)
	       {
		  *d++ = img[2];
		  *d++ = img[1];
		  *d++ = img[0];
	       }
	    }
	    else
	       data = image->data;

	    end[0] = size[0] - 1;
	    end[1] = size[1] - 1;
	    end[2] = size[2] - 1;
	    end[3] = size[3] - 1;
	    end[4] = size[4] - 1;
	    tdata = kpds_transpose_elements_last(data, beg, end, datatype, 5);
	    kfree_and_NULL(data);

	 } break;

	 case 4: /*-- not sure what the fourth value is.. call it x --*/
	 {  
	    pnum = size[0] * size[1];
	    if ((data = kmalloc((size_t)(pnum * 3))) == NULL)
	       return FALSE;

	    if (image->ras_type != RT_FORMAT_RGB)
	    {
	       /*-- swap xbgr to be rgb --*/
	       img = (kubyte *) image->data;
	       for (i = 0, d = (kubyte *) data; i < pnum; i++, img += 4)
	       {
		  *d++ = img[3];
		  *d++ = img[2];
		  *d++ = img[1];
	       }
	    }
	    else
	    {
	       /*-- swap xrgb to be just plane old rgb --*/
	       img = (kubyte *) image->data;
	       for (i = 0, d = (kubyte *) data; i < pnum; i++, img += 4)
	       {
		  *d++ = img[1];
		  *d++ = img[2];
		  *d++ = img[3];
	       }
	    }

	    end[0] = size[0] - 1;
	    end[1] = size[1] - 1;
	    end[2] = size[2] - 1;
	    end[2] = size[3] - 1;
	    end[2] = size[4] - 1;
	    tdata = kpds_transpose_elements_last(data, beg, end, datatype, 5);
	    kfree_and_NULL(data);

	 } break;

	 default:
	    kerror("kdataman", "kdms_open", "Unrecognized pixel depth (%d) "
		   "in Sun Raster file.", image->ras_depth);
      }

      /*-- store all value information --*/
      kdms_set_segment(object, KDMS_SEGMENT_VALUE, 5, size, order,
		       datatype, tdata);

      /*---------------------------------------------------------- 
      |  let data services handle freeing this, since set_segment 
      |  has assigned the data directly. 
      \----------------------------------------------------------*/
      image->data = NULL;
   }

   /*-----------------------------------------------------------------------
   |  map data
   \-----------------------------------------------------------------------*/
   if (image->ras_maptype != RMT_NONE && image->map)
   {
      /*-- create the map segment --*/
      kpds_create_map(object);

      if (image->ras_maptype == RMT_RAW)
         num = 1;
      else
         num = 3;

      size[0] = image->ras_maplength / num;
      size[1] = num;
      size[2] = size[3] = size[4] = 1;

      order[0] = KMAP_HEIGHT;
      order[1] = KMAP_WIDTH;
      order[3] = KMAP_DEPTH;
      order[4] = KMAP_TIME;
      order[2] = KMAP_ELEMENTS;

      /*-- store all map information --*/
      kdms_set_segment(object, KDMS_SEGMENT_MAP, 5, size, order,
		       datatype, image->map);
      /*---------------------------------------------------------- 
      |  let data services handle freeing this, since set_segment 
      |  has assigned the data directly. 
      \----------------------------------------------------------*/
      image->map = NULL;
   }


   /*--------------------------------------------------------------------
   |  if the number of elements is three, or the map width is three,
   |  then set the colorspace model to be RGB so that display programs
   |  interpret this image properly.
   \--------------------------------------------------------------------*/
   if (num == 3)
      kcolor_set_attribute(object, _INTERNAL_COLORSPACE, KRGB);

   return TRUE;
}


/*-----------------------------------------------------------
|
|  Routine Name: rast_output - Closes the rast image
|
|       Purpose: This function is used to close the rast image.  
|
|         Input: object - the rast object to be closed
|        Output: none
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Mark Young
|          Date: July 13, 1992
| Modifications:
|
------------------------------------------------------------*/
/*ARGSUSED */
static int
rast_output(kobject object, int fid, int flags)
{
   ResourceStruct *resources;

   kaddr tdata;
   rast *image = NULL;
   int type;
   int *size;
   int order[KDMS_MAX_DIM];
   int width;
   int height;
   int depth;
   int beg[KDMS_MAX_DIM] = {0, 0, 0, 0, 0};
   int end[KDMS_MAX_DIM];
   int dim;
   int i;
   
   /*-- get the resource structure --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = (ResourceStruct *) kcalloc((size_t)1,
					     sizeof(ResourceStruct))) == NULL)
         return(FALSE);

      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);


   if (resources->image == NULL)
   {
      if ((image = resources->image = rast_create()) == NULL)
      {
         kdas_set_error(KDMS_EFMT_FAILURE);
         return (FALSE);
      }
   }
   else
      image = resources->image;

   /*-----------------------------------------------------------------------
   |  value data -
   \-----------------------------------------------------------------------*/
   if (kpds_query_value(object))
   {
      int rorder[KDMS_MAX_DIM] = {KWIDTH, KHEIGHT, KDEPTH, KTIME, KELEMENTS};

      kdms_get_attribute(object, KDMS_SEGMENT_VALUE, KDMS_DATA_TYPE, &type);

      if (type != KBIT && type != KUBYTE)
	 type = KUBYTE;

      kdms_set_attributes(object, KDMS_SEGMENT_VALUE,
			  KDMS_COUPLING,    KUNCOUPLED,
			  KDMS_INDEX_ORDER, rorder,
			  KDMS_DATA_TYPE,   type,
			  NULL);

      kdms_get_attribute(object, KDMS_SEGMENT_VALUE, KDMS_SIZE, &size);

      end[0] = size[0] - 1;
      end[1] = size[1] - 1;
      end[2] = 0;
      end[3] = 0;
      end[4] = size[4] - 1;
	 
      tdata = kdms_get_data(object, KDMS_SEGMENT_VALUE, beg, end, NULL);

      if (type != KBIT)
      {
	 image->data = (char *) kpds_transpose_elements_first(tdata, beg, end,
							      type, 5);
      }
      else
	 image->data = (char *)tdata;

      image->ras_width  = (int)size[0];
      image->ras_height = (int)size[1];

      if (type == KBIT)
      {
         image->ras_type = RT_STANDARD;
         image->ras_depth = 1;
	 
	 /* invert the values since bit 1's are black and bit 0's are white */
	 for (i = 0; i < (((image->ras_width+7)>>3) * image->ras_height); i++)
	    image->data[i] = ~image->data[i];
      }
      else
      {
         image->ras_type = (size[4] == 3) ? RT_FORMAT_RGB : RT_STANDARD;
         image->ras_depth = (size[4] << 3);
      }
   }
   else
      image->ras_type = RT_STANDARD;

   width  = image->ras_width;
   height = image->ras_height;
   depth  = image->ras_depth >> 3;

   if (image->ras_depth == 1)
   {
      image->ras_length = (width+7)>>3;
      image->ras_length = (image->ras_length + image->ras_length % 2) * height;
   }
   else /*-- must get 16-bit aligned width --*/
      image->ras_length = (width * depth + (width*depth) % 2) * height;

   /*-----------------------------------------------------------------------
   |  map data -
   |  want the map data to be unsigned char, 
   |  and index order to be map_width by map_height
   \-----------------------------------------------------------------------*/
   if (kdms_get_attributes(object, KDMS_SEGMENT_MAP, 
			   KDMS_PHYSICAL_SIZE, &size,
			   KDMS_PHYSICAL_DATA_TYPE, &type,
			   KDMS_PHYSICAL_DIMENSION, &dim,
			   NULL))
   {
      /*
       * set the presentation layer to the physical layer.  Then get the
       * data. 
       */
      order[0] = KMAP_HEIGHT;
      order[1] = KMAP_WIDTH;
      order[2] = KMAP_DEPTH;
      order[3] = KMAP_TIME;
      order[4] = KMAP_ELEMENTS;

      type = KUBYTE;

      /* 
       * note that order is set *after* size...that way the old 
       * index order is honored 
       */
      kdms_set_attributes(object, KDMS_SEGMENT_MAP, 
			  KDMS_COUPLING, KNONE,
			  KDMS_SIZE, size, 
			  KDMS_INDEX_ORDER, order,
			  KDMS_DATA_TYPE, type, 
			  KDMS_INTERPOLATE, KNONE,
			  KDMS_SCALING, KNONE,
			  NULL);
      kdms_get_attribute(object, KDMS_SEGMENT_MAP, KDMS_SIZE, &size);
      
      end[0] = size[0] - 1;
      end[1] = size[1] - 1;
      end[2] = end[3] = end[4] = 0;
      
      image->map = (char *) kdms_get_data(object, KDMS_SEGMENT_MAP, beg, end,
					  NULL);
      
      image->ras_maplength = size[0] * size[1];

      if (size[1] == 1)
         image->ras_maptype = RMT_RAW;
      else
         image->ras_maptype = RMT_EQUAL_RGB;
   }
   else
      image->ras_maptype = RMT_NONE;

   /*-- write the rast structure out to the specified filename --*/
   return rast_fdwrite(fid, image);
}

/*-----------------------------------------------------------
|
|  Routine Name: rast_destroy - Frees an rast image
|
|       Purpose: This function is used to create an rast image.  Which
|                means that we use the current dimension values in order
|                to create a proper rast.
|
|         Input: data - initial sample of data
|                num  - number of bytes in the sample
|
|        Output: Returns: returns TRUE or FALSE if an error occurs
|
|    Written By: Mark Young
|          Date: Jul 13, 1992 15:50
| Modifications:
|
------------------------------------------------------------*/

static int
rast_destroy(kobject object)
{
   ResourceStruct *resources;

   if ((resources = (ResourceStruct *) kdms_glue_get_resources(object))==NULL)
      return TRUE;

   /*-- free the raster structure --*/
   rast_free(resources->image);

   /*-- free the resources --*/
   kfree(resources);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: rast_prep - prepare an object for output to sun raster
|
|       Purpose: This function is used to prepare an object for
|		 output to sun raster.  It will print out information 
|		 about what is not getting stored out to the sun raster
|		 file due to the limitations of the sun raster format.	
|
|         Input: object - the object to be prepared
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: May 18, 1996
|
------------------------------------------------------------*/
/*ARGSUSED2*/
static int
rast_prep(kobject object)
{
   char *warn = NULL;
   int   d, t, e;
   int   type;


   /*-----------------------------------------------------------------------
   |  mask, location, and time data
   \-----------------------------------------------------------------------*/
   if (kpds_query_mask(object))
      warn = kstring_append(warn, "- no support for storing mask "
			    ": mask segment ignored.\n");

   if (kpds_query_location(object))
       warn = kstring_append(warn, "- no support for storing location "
			    ": location segment ignored.\n");

   if (kpds_query_time(object))
       warn = kstring_append(warn, "- no support for storing time "
			    ": time segment ignored.\n");


   /*-- support for checking extraneous attributes coming soon! --*/

   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (kpds_query_value(object))
   {
      /*-- size issue : can only store 2D data plane --*/
      kpds_get_attribute(object, KPDS_VALUE_SIZE, NULL, NULL, &d, &t, &e);

      if (d > 1 || t > 1)
	 warn = kstring_append(warn, "- ignoring depth and time planes of "
			       "value segment.\n");

      /*-- type issue : can only store bit & unsigned byte images --*/
      kpds_get_attribute(object, KPDS_VALUE_DATA_TYPE, &type);

      if (type != KBIT && type != KUBYTE)
	 warn = kstring_append(warn, "- casting value data to unsigned byte "
			       "for storage.\n");
   }

   /*-----------------------------------------------------------------------
   |  map data
   \-----------------------------------------------------------------------*/
   if (kpds_query_map(object))
   {
      /*-- size issue : can only store first wxh map plane --*/
      kpds_get_attribute(object, KPDS_MAP_SIZE, NULL, NULL, &d, &t, &e);

      if (d > 1 || t > 1 || e > 1)
	 warn = kstring_append(warn, "- ignoring depth, time, and element "
			       "planes of map segment.\n");

      /*-- type issue : can only store unsigned byte map --*/
      kpds_get_attribute(object, KPDS_MAP_DATA_TYPE, &type);

      if (type != KUBYTE)
	 warn = kstring_append(warn, "- casting map data to unsigned byte "
			       "for storage.\n");
   }

   /*-- if we have any news to report, report it now --*/
   if (warn)
   {
      kinfo(KSTANDARD, "SUN Raster format output :\n%s", warn);
      kfree_and_NULL(warn);
   }

   return TRUE;
}

#endif /* KRAST_DEF */

/* don`t add after the endif */
