 /*
  * Khoros: $Id: Xwd.c,v 1.8 1997/02/07 21:43:27 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Xwd.c,v 1.8 1997/02/07 21:43:27 mirage Exp $";

#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros XWD Data Service Routines
   >>>>		
   >>>>  Private:
   >>>>                 init_xwd()
   >>>>                 init_xwd()
   >>>>   Static:
   >>>>                 xwd_check()
   >>>>                 xwd_input()
   >>>>                 xwd_output()
   >>>>                 xwd_destroy()
   >>>>                 xwd_prep()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

#if !defined(KXWD_DEF)

void
init_xwd(void) /*-- NULL xwd initialization --*/
{
   return;
}

#else

#include "kdatafmt/xwd.h"

static int xwd_check   PROTO((int));
static int xwd_input   PROTO((kobject, int, int));
static int xwd_output  PROTO((kobject, int, int));
static int xwd_destroy PROTO((kobject));
static int xwd_prep    PROTO((kobject));

/*=====================================================================*/

/*-----------------------------------------------------------
|  Routine Name: init_xwd
|       Purpose: initialize the xwd file format
------------------------------------------------------------*/
void
init_xwd(void)
{
   kdms_add_file_format("xwd", 
			"X11 Window Dump File Format (xwd)",
			xwd_check, 
			xwd_input, 
			xwd_output, 
			xwd_destroy, 
			xwd_prep,
			NULL, NULL, NULL, NULL, NULL);

   /*-- need some colorspace attributes --*/
   kcolor_init();

   return;
}

/*=====================================================================*/

/*
 *  Internal Resource Structure 
 *  xwd  -  X11 Window Dump File Format (xwd)
 */
typedef struct
{
   int  fid;
   xwd *image;
}
ResourceStruct;

/****    =============================================================
 ****    Format Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: xwd_check - Checks to see if the file is an xwd
|
|       Purpose: This function is used to check the first few bytes of
|                the stream to see if it is an xwd.
|
|         Input: fid - file id of a file to check
|        Output: none
|	Returns: TRUE if the file is an xwd
|
|    Written By: Jeremy Worley
|          Date: Mar 14, 1994 16:53
| Modifications:
|
------------------------------------------------------------*/
static int
xwd_check(int fid)
{
   xwd *check;
   
   /*-- seems like the easiest way to do this is to check the header --*/
   if ((check = xwd_readheader(fid)) == NULL)
      return FALSE;

   xwd_free(check);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: xwd_input - Read an xwd image
|
|       Purpose: This function is used to read in the image if the
|                supplied data is an xwd image.  We can simply set up 
|                the header and initialize the data segments.
|
|         Input: object - the xwd object to be initialized
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: March 14, 1994
| Modifications:
|
------------------------------------------------------------*/
/*ARGSUSED */
static int
xwd_input(kobject object, int fid, int flags)
{
   ResourceStruct *resources;
   xwd *image;

   int datatype = KUBYTE;
   int size[KDMS_MAX_DIM];
   int total;
   int i;

   
   /*-- get the resource structure --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = (ResourceStruct *)kcalloc((size_t)1,
					     sizeof(ResourceStruct))) == NULL)
         return FALSE;

      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid = fid;

   if ((image = resources->image = xwd_fdread(fid)) == NULL)
   {
      kdas_set_error(KDMS_EFMT_FAILURE);
      return FALSE;
   }


   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (image->width != 0 && image->height != 0)
   {
      /*-- create the value segment --*/
      kpds_create_value(object);

      if (image->visual_class == TrueColor)
	 datatype = KUBYTE;
      else
	 switch (image->bits_per_pixel)
	 {
	    case 1:
	       datatype = KBIT;
	       break;
	    case 8:
	       datatype = KUBYTE;
	       break;
	    case 16:
	       datatype = KUSHORT;
	       break;
	    case 32:
	       datatype = KUINT;
	       break;
	    default:
	       kdas_set_error(KDMS_EFMT_FAILURE);
	       return FALSE;
	 }

      size[0] = image->bytes_per_line / (image->bits_per_pixel >> 3);
      size[1] = image->pixmap_height;
      size[2] = size[3] = 1;
      size[4] = image->bits_per_pixel >> 3;

      /*-- if we have TrueColor, then reorder to put elements last --*/
      if (size[4] == 4 || size[4] == 3) 
      {
	 kubyte *tdata, *odata = (kubyte *) image->data;
	 kubyte *rd, *gr, *bl;
	 
	 total = size[0] * size[1];
	 
	 tdata = (unsigned char *)kmalloc(sizeof(kubyte *) * total * 3);
	 
	 rd = tdata;  gr = rd + total;  bl = gr + total;

	 if (image->byte_order == LSBFirst)
	    for (i = 0; i < total; i++)
	    {
	       *rd++ = *odata++;
	       *gr++ = *odata++;
	       *bl++ = *odata++;
	       if (size[4] == 4) odata++;
	    }
	 else
	    for (i = 0; i < total; i++)
	    {
	       if (size[4] == 4) odata++;
	       *bl++ = *odata++;
	       *gr++ = *odata++;
	       *rd++ = *odata++;
	    }

	 kfree(image->data);
	 image->data = tdata;

	 size[4] = 3;
      }

      kdms_set_segment(object, KDMS_SEGMENT_VALUE, 5, size, NULL, 
		       datatype, image->data);

      /*---------------------------------------------------------- 
      |  let data services handle freeing this, since set_segment 
      |  has assigned the data directly. 
      \----------------------------------------------------------*/
      image->data = NULL;
   }

   
   /*-----------------------------------------------------------------------
   |  map data
   \-----------------------------------------------------------------------*/
   if (image->ncolors != 0)
   {
      unsigned short *data, *d;

      /*-- create the map segment --*/
      kpds_create_map(object);

      size[0] = 3;
      size[1] = image->ncolors;
      size[2] = size[3] = size[4] = 1;

      data = (unsigned short *) kmalloc(sizeof(unsigned short) * 
					image->ncolors * 3);

      for (i = 0, d = data; i < image->ncolors; i++)
      {
         *d++ = image->maps[i].red;
         *d++ = image->maps[i].green;
         *d++ = image->maps[i].blue;
      }

      kdms_set_segment(object, KDMS_SEGMENT_MAP, 5, size, NULL, KUSHORT, data);
   }

   kcolor_set_attribute(object, _INTERNAL_COLORSPACE, KRGB);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: xwd_output - Closes a xwd image
|
|       Purpose: This function is used to close the xwd image.  
|                It writes out the value data to the xwd format.
|
|         Input: object - the xwd object to be closed
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: March 14, 1994
| Modifications:
|
------------------------------------------------------------*/
#define XWD_MAX_VAL 65535.0

/*ARGSUSED */
static int
xwd_output(kobject object, int fid, int flags)
{
   ResourceStruct *resources;

   unsigned short *data;
   xwd *image;

   int vorder[KDMS_MAX_DIM] = {KELEMENTS, KWIDTH, KHEIGHT, KDEPTH, KTIME};
   int morder[KDMS_MAX_DIM] ={KMAP_WIDTH, KMAP_HEIGHT, KMAP_DEPTH, KMAP_TIME, 
			      KMAP_ELEMENTS};
   int begin[KDMS_MAX_DIM] = {0, 0, 0, 0, 0};
   int end[KDMS_MAX_DIM];
   int *size;
   int *msize;
   
   int type;
   int i;
   int j;
   int e = 1;

   
   /*-- get the resource structure --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = (ResourceStruct *)kcalloc((size_t)1,
					     sizeof(ResourceStruct))) == NULL)
         return(FALSE);

      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   if (resources->image == NULL)
   {
      if ((image = resources->image = xwd_create()) == NULL)
      {
         kdas_set_error(KDMS_EFMT_FAILURE);
         return FALSE;
      }
   }
   else
      image = resources->image;
 
   /*-----------------------------------------------------------------------
   |  value data -
   \-----------------------------------------------------------------------*/
   if (kpds_query_value(object))
   {
      kdms_get_attribute(object, KDMS_SEGMENT_VALUE, KDMS_DATA_TYPE, &type);
      
      /*-- xwd_prep will have guaranteed it is at least kulong --*/
      switch (type)
      {
	 case KBIT:
	 case KBYTE:
	 case KUBYTE:
	    type = KUBYTE;
	    image->pixmap_depth = image->bits_per_pixel = 8;
	    break;
	 case KSHORT:
	 case KUSHORT:
	    image->pixmap_depth = image->bits_per_pixel = 16;
	    type = KUSHORT;
	    break;
	 case KINT:
	 case KUINT:
	    image->pixmap_depth = image->bits_per_pixel = 32;
	    type = KUINT;
	    break;
	 default:
	    image->pixmap_depth = image->bits_per_pixel = 32;
	    type = KULONG;
	    break;
      }
      
      if (!kdms_set_attributes(object, KDMS_SEGMENT_VALUE, 
			       KDMS_INDEX_ORDER, vorder,
			       KDMS_DATA_TYPE,   type, 
			       NULL))
	 return FALSE;
      
      kdms_get_attribute(object, KDMS_SEGMENT_VALUE, KDMS_SIZE, &size);

      e = size[0];
      
      end[0] = size[0] - 1;
      end[1] = size[1] - 1;
      end[2] = size[2] - 1;
      end[3] = end[4] = 0;
      
      image->pixmap_width   = image->width  = size[1];
      image->pixmap_height  = image->height = size[2];
      image->data = kdms_get_data(object, KDMS_SEGMENT_VALUE, 
				  begin, end, NULL);
   }
   
   /*-----------------------------------------------------------------------
   |  map data -
   \-----------------------------------------------------------------------*/
   if (kpds_query_map(object))
   {
      kdms_set_attribute(object, KDMS_SEGMENT_MAP, KDMS_INDEX_ORDER, morder);

      kdms_get_attribute(object, KDMS_SEGMENT_MAP, KDMS_SIZE, &msize);

      if (msize[0] == 1)
      {
	 image->visual_class = GrayScale;
      }
      else
      {
	 if (e == 1)
	 {
	    image->pixmap_depth = 8;
	    image->visual_class = PseudoColor;
	 }
	 else
	 {
	    image->bits_per_rgb = 8 /*24*/;
	    image->pixmap_depth = 24;
	    image->visual_class = DirectColor;
	 }
	 
	 msize[0] = 3;
	 kdms_set_attribute(object, KDMS_SEGMENT_MAP, KDMS_SIZE, msize);
      }

      end[0] = msize[0] - 1;
      end[1] = msize[1] - 1;
      end[2] = msize[2] - 1;
      end[3] = msize[3] - 1;
      end[4] = msize[4] - 1;

      data = (unsigned short *) kdms_get_data(object, KDMS_SEGMENT_MAP, begin,
					      end, NULL);
      image->maps = (xwdcolor *) kmalloc((size_t)msize[1] * sizeof(xwdcolor));

      for (i = 0, j = 0; i < msize[1]; i++)
      {
         image->maps[i].red   = data[j++];
         image->maps[i].green = data[j++];
         image->maps[i].blue  = data[j++];
         image->maps[i].flags = 0xff;
	 image->maps[i].pixel = i;
      }
      image->colormap_entries = image->ncolors = msize[1];
      kfree_and_NULL(data);
   }
   else  /*-- no map : truecolor or grayscale --*/
   {
      image->visual_class = (e == 1) ? GrayScale : TrueColor;
      image->pixmap_depth = (e == 1) ? 8 : 24;
      image->bits_per_rgb = e * 8;

      /*-- convert from lsb to msb --*/
      if (e == 4)
      {
	 int     total = image->pixmap_width * image->pixmap_height;
	 kubyte *mdata, *ldata = (kubyte *) image->data;
	 kubyte *mr, *mg, *mb, *mx;
	 
	 mdata = (kubyte *) kmalloc(sizeof(kubyte *) * total * 4);
      
	 mx = mdata; mb = mx + 1;  mg = mb + 1;  mr = mg + 1;
	 
	 for (i = 0; i < total; i++, mb += 4, mg += 4, mr += 4, mx += 4)
	 {
	    *mr = *ldata++;
	    *mg = *ldata++;
	    *mb = *ldata++;
	    *mx = *ldata++;
	 }

	 kfree(image->data);
	 image->data = mdata;

	 image->blue_mask  = 0xff0000;
	 image->green_mask = 0xff00;
	 image->red_mask   = 0xff;
      }
   }

   image->bits_per_pixel *= e;
   image->bytes_per_line  = image->pixmap_width * (image->bits_per_pixel >> 3);

   /*-- write the xwd structure out to the specified file --*/
   image->pixmap_format    = ZPixmap;
   image->byte_order  	   = MSBFirst;
   image->bitmap_bit_order = MSBFirst;
   image->bitmap_pad  	   = 8 * e;
   image->bitmap_unit 	   = 8 * e;
   image->bits_per_rgb     = 8;
   
   return xwd_fdwrite(fid, image);
}

   
/*-----------------------------------------------------------
|
|  Routine Name: xwd_destroy - Frees an xwd image
|
|       Purpose: This function is used to create an xwd image.  Which
|                means that we use the current dimension values in order
|                to create a proper xwd.
|
|         Input: data - initial sample of data
|                num  - number of bytes in the sample
|
|        Output: Returns: returns TRUE or FALSE if an error occurs
|
|    Written By: Jeremy Worley
|          Date: March 14, 1994
| Modifications:
|
------------------------------------------------------------*/
static int
xwd_destroy(kobject object)
{
   ResourceStruct *resources;

   if ((resources = (ResourceStruct *) kdms_glue_get_resources(object))==NULL)
      return TRUE;

   /*-- free the xwd structure --*/
   xwd_free(resources->image);

   /*-- free the resources --*/
   kfree(resources);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: xwd_prep - prepare an object for output to xwd
|
|       Purpose: This function is used to prepare an object for
|		 output to xwd.  It will print out information 
|		 about what is not getting stored out to the xwd
|		 file due to the limitations of the xwd format.	
|
|         Input: object - the object to be prepared
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: May 18, 1996
|
------------------------------------------------------------*/
/*ARGSUSED2*/
static int
xwd_prep(kobject object)
{
   char *warn = NULL;
   int   d, t, e;
   int   type;


   /*-----------------------------------------------------------------------
   |  mask, location, and time data
   \-----------------------------------------------------------------------*/
   if (kpds_query_mask(object))
      warn = kstring_append(warn, "- no support for storing mask "
			    ": mask segment ignored.\n");

   if (kpds_query_location(object))
       warn = kstring_append(warn, "- no support for storing location "
			    ": location segment ignored.\n");

   if (kpds_query_time(object))
       warn = kstring_append(warn, "- no support for storing time "
			    ": time segment ignored.\n");


   /*-- support for checking extraneous attributes coming soon! --*/

   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (kpds_query_value(object))
   {
      /*-- size issue : can only store 2D data plane --*/
      kpds_get_attribute(object, KPDS_VALUE_SIZE, NULL, NULL, &d, &t, &e);

      if (d > 1 || t > 1)
	 warn = kstring_append(warn, "- ignoring depth and time planes of "
			       "value segment.\n");

      /*-- type issue : can only store bit & unsigned byte images --*/
      kpds_get_attribute(object, KPDS_VALUE_DATA_TYPE, &type);

      if (!kpds_query_map(object))    /*-- true color or grey scale --*/
      {
	 if (type != KBYTE && type != KUBYTE)
	    warn = kstring_append(warn, "- casting value to unsigned byte.\n");

	 kpds_set_attribute(object, KPDS_VALUE_DATA_TYPE, KUBYTE);

	 if (e > 1)   /*-- true color --*/
	    kpds_set_attribute(object, KPDS_VALUE_SIZE, -1, -1, -1, -1, 4);
	 else
	 { 
	    warn = kstring_append(warn, "- creating greyscale colormap.\n");
	    kcolor_set_attribute(object, KCOLOR_MAP_AUTOCOLOR, KGREYSCALE);
	 }
      }
      else /*-- mapped --*/ 
      {
	 if (type > KULONG)
	 {
	    warn = kstring_append(warn, "- casting value to unsigned long.\n");
	    kpds_set_attribute(object, KPDS_VALUE_DATA_TYPE, KULONG);
	 }
      }
   }

   /*-----------------------------------------------------------------------
   |  map data
   \-----------------------------------------------------------------------*/
   if (kpds_query_map(object))
   {
      /*-- size issue : can only store first wxh map plane --*/
      kpds_get_attribute(object, KPDS_MAP_SIZE, NULL, NULL, &d, &t, &e);

      if (d > 1 || t > 1 || e > 1)
	 warn = kstring_append(warn, "- ignoring depth, time, and element "
			       "planes of map segment.\n");

      /*-- type issue : can only store unsigned byte map --*/
      kpds_get_attribute(object, KPDS_MAP_DATA_TYPE, &type);

      if (type != KUSHORT)
      {
	 kpds_set_attributes(object, 
			     KPDS_MAP_DATA_TYPE, KUSHORT, 
			     KPDS_MAP_SCALING,   KNORMALIZE,
			     KPDS_MAP_NORM_MIN,  0.0, 
			     KPDS_MAP_NORM_MAX,  XWD_MAX_VAL,
			     NULL);

	 warn = kstring_append(warn, "- converting map data into an XColor "
			       "array.\n");
      }
   }

   /*-- if we have any news to report, report it now --*/
   if (warn)
   {
      kinfo(KSTANDARD, "XWD format output :\n%s", warn);
      kfree_and_NULL(warn);
   }

   return TRUE;
}

#endif /* KXWD_DEF */

/* don`t add after the endif */
