 /*
  * Khoros: $Id: attributes.c,v 1.3.2.2 1997/05/24 18:34:56 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: attributes.c,v 1.3.2.2 1997/05/24 18:34:56 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Polymophic Model Attributes
   >>>>		
   >>>>  Private:
   >>>>                 kpds_intelligent_vset_attribute()
   >>>>                 kpds_intelligent_vget_attribute()
   >>>>                 kpds_call_attribute_handler()
   >>>>                 kpds_init_attributes()
   >>>>   Static:
   >>>>                 _axis_get()
   >>>>                 _axis_set()
   >>>>                 _complex_convert_get()
   >>>>                 _complex_convert_set()
   >>>>                 _complex_convert_match()
   >>>>                 _complex_convert_copy()
   >>>>                 _complex_convert_query()
   >>>>                 _complex_convert_print()
   >>>>                 _interpolate_get()
   >>>>                 _interpolate_set()
   >>>>                 _interpolate_match()
   >>>>                 _interpolate_copy()
   >>>>                 _interpolate_query()
   >>>>                 _interpolate_print()
   >>>>                 _norm_min_get()
   >>>>                 _norm_min_set()
   >>>>                 _norm_min_match()
   >>>>                 _norm_min_copy()
   >>>>                 _norm_min_query()
   >>>>                 _norm_min_print()
   >>>>                 _norm_max_get()
   >>>>                 _norm_max_set()
   >>>>                 _norm_max_match()
   >>>>                 _norm_max_copy()
   >>>>                 _norm_max_query()
   >>>>                 _norm_max_print()
   >>>>                 _pad_value_get()
   >>>>                 _pad_value_set()
   >>>>                 _pad_value_match()
   >>>>                 _pad_value_copy()
   >>>>                 _pad_value_query()
   >>>>                 _pad_value_print()
   >>>>                 _scale_factor_get()
   >>>>                 _scale_factor_set()
   >>>>                 _scale_factor_match()
   >>>>                 _scale_factor_copy()
   >>>>                 _scale_factor_query()
   >>>>                 _scale_factor_print()
   >>>>                 _scale_offset_get()
   >>>>                 _scale_offset_set()
   >>>>                 _scale_offset_match()
   >>>>                 _scale_offset_copy()
   >>>>                 _scale_offset_query()
   >>>>                 _scale_offset_print()
   >>>>                 _scaling_get()
   >>>>                 _scaling_set()
   >>>>                 _scaling_match()
   >>>>                 _scaling_copy()
   >>>>                 _scaling_query()
   >>>>                 _scaling_print()
   >>>>                 _history_get()
   >>>>                 _history_set()
   >>>>                 _history_match()
   >>>>                 _history_copy()
   >>>>                 _history_query()
   >>>>                 _history_print()
   >>>>                 _minmax_get()
   >>>>                 _increment_size_get()
   >>>>                 _increment_size_set()
   >>>>                 _increment_size_match()
   >>>>                 _increment_size_copy()
   >>>>                 _increment_size_query()
   >>>>                 _increment_size_print()
   >>>>                 _coupling_get()
   >>>>                 _coupling_set()
   >>>>                 _coupling_match()
   >>>>                 _coupling_copy()
   >>>>                 _coupling_query()
   >>>>                 _coupling_print()
   >>>>                 _data_format_get()
   >>>>                 _data_format_set()
   >>>>                 _data_format_match()
   >>>>                 _data_format_copy()
   >>>>                 _data_format_query()
   >>>>                 _data_format_print()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

extern char *pds_value_string;
extern char *pds_mask_string;
extern char *pds_location_string;
extern char *pds_map_string;
extern char *pds_time_string;

extern ktoken pds_value_token;
extern ktoken pds_mask_token;
extern ktoken pds_location_token;
extern ktoken pds_time_token;
extern ktoken pds_map_token;
extern ktoken pds_width_token;
extern ktoken pds_height_token;
extern ktoken pds_depth_token;

/****    =============================================================
 ****    Polymorphic Aware Attribute Access
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kpds_intelligent_vset_attribute
|
|       Purpose: This routine will do a "dms-style" set attribute
|		 on a polymorphic segment.  The segment is specified
|		 using a token, and the attribute is set correctly
|		 according to the current presentation settings.
|
|		 Internally, it will cope with the mapping mode 
|		 and location grid.  Depending on the mapping
|		 mapping mode, this routine will act on the map 
|		 instead of the value.  Depending on the
|		 location grid, this routine will act on either
|		 the uniform, rectilinear, or curvilinear location 
|		 segments.
|
|		 The routine kpds_intelligent_vset_attribute can
|		 be used to do this same thing with an already open
|		 variable argument list. 
|
|         Input: object      - object to set the attribute on
|                segment     - segment to set the attribute on
|                token       - attribute token to set
|        Output: kvalist     - variable argument list containing
|                              pointers to variables with which
|                              to set the attribute's value
|       Returns: TRUE on success, FALSE otherwise
|    Written By: Steve Kubica
|          Date: February 17, 1997
| Modifications:
|
------------------------------------------------------------*/
int 
kpds_intelligent_vset_attribute(
   kobject   object, 
   char     *segment, 
   ktoken    token,
   kva_list *list)
{
   ktoken seg = kstring_to_token(segment);
   char   *attribute = ktoken_to_string(token);
   
   if (seg == pds_value_token)
   {
      /*-- if mapping mode is on and we have a map, then operate on map --*/
      if (kpds_is_mapped(object) && 
	  kdms_query_segment(object, KPDS_SEGMENT_MAP))
	 return kdms_vset_attribute(object, KPDS_SEGMENT_MAP, attribute, list);
      else
	 return kdms_vset_attribute(object, segment, attribute, list);
   }
   else if (seg == pds_location_token)
   {
      switch (kpds_location_grid(object))
      {
         case KUNIFORM:
	    return kdms_vset_attribute(object, KPDS_SEGMENT_UNIFORM, 
				       attribute, list);
	 case KRECTILINEAR:
	    {
	       kva_list wlist = *list;
	       kva_list hlist = *list;
	       kva_list dlist = *list;
	       
	       return (kdms_vset_attribute(object, KPDS_SEGMENT_WIDTH, 
					   attribute, &wlist) &&
		       kdms_vset_attribute(object, KPDS_SEGMENT_HEIGHT, 
					   attribute, &hlist) &&
		       kdms_vset_attribute(object, KPDS_SEGMENT_DEPTH, 
					   attribute, &dlist));
	    }
	 case KCURVILINEAR:
         case KNONE:          /*-- backwards compatibility --*/
	    return kdms_vset_attribute(object, KPDS_SEGMENT_LOCATION, 
				       attribute, list);

	 default: /*-- errno already set --*/
	    return FALSE;
      }
   }

   return kdms_vset_attribute(object, segment, attribute, list);
}

/*-----------------------------------------------------------
|
|  Routine Name: kpds_intelligent_vget_attribute
|
|       Purpose: This routine will do a "dms-style" get attribute
|		 on a polymorphic segment.  The segment is specified
|		 using a token, and the attribute is set on the
|		 correct segment according to the current 
|		 presentation settings.
|
|		 Internally, it will cope with the mapping mode 
|		 and location grid.  Depending on the mapping
|		 mapping mode, this routine will act on the map 
|		 instead of the value.  Depending on the
|		 location grid, this routine will act on either
|		 the uniform, rectilinear, or curvilinear location 
|		 segments.
|
|		 The routine kpds_intelligent_vget_attribute can
|		 be used to do this same thing with an already open
|		 variable argument list. 
|
|         Input: object      - object from which to retrieve the attribute
|                segment     - segment from which to retrieve the attribute
|                attribute   - attribute token to retrieve
|        Output: kvalist     - variable argument list containing
|                              pointers to variables with which
|                              to return the attribute's values
|       Returns: TRUE on success, FALSE otherwise
|    Written By: Steve Kubica
|          Date: February 17, 1997
| Modifications:
|
------------------------------------------------------------*/
int 
kpds_intelligent_vget_attribute(
   kobject   object, 
   char     *segment, 
   ktoken    token,
   kva_list *list)
{
   ktoken seg = kstring_to_token(segment);
   char   *attribute = ktoken_to_string(token);

   if (seg == pds_value_token)
   {
      /*-- if mapping mode is on and we have a map, then operate on map --*/
      if (kpds_is_mapped(object) && 
	  kdms_query_segment(object, KPDS_SEGMENT_MAP))
	 return kdms_vget_attribute(object, KPDS_SEGMENT_MAP, attribute, list);
      else
	 return kdms_vget_attribute(object, segment, attribute, list);
   }
   else if (seg == pds_location_token)
   {
      switch (kpds_location_grid(object))
      {
         case KUNIFORM:
	    return kdms_vget_attribute(object, KPDS_SEGMENT_UNIFORM, 
				       attribute, list);
	 case KRECTILINEAR:
	    {
	       kva_list *wlist = list;
	       kva_list *hlist = list;
	       kva_list *dlist = list;
	       
	       return (kdms_vget_attribute(object, KPDS_SEGMENT_WIDTH, 
					   attribute, wlist) &&
		       kdms_vget_attribute(object, KPDS_SEGMENT_HEIGHT, 
					   attribute, hlist) &&
		       kdms_vget_attribute(object, KPDS_SEGMENT_DEPTH, 
					   attribute, dlist));
	    }
	 case KCURVILINEAR:
         case KNONE:          /*-- backwards compatibility --*/
	    return kdms_vget_attribute(object, KPDS_SEGMENT_LOCATION, 
				       attribute, list);

	 default: /*-- errno already set --*/
	    return FALSE;
      }
   }

   return kdms_vget_attribute(object, segment, attribute, list);
}

/*-----------------------------------------------------------
|
|  Routine Name: kpds_call_attribute_handler
|
|       Purpose: This routine allows you to call a set or get
|                handler for an attribute directly to bypass the 
|		 dms attribute code.  It saves some lookup time,
|		 but that is all.
|
|         Input: func       - the set or get handler to call
|                object     - the object to call with 
|                segment    - the tokenized segment
|                attribute  - the tokenized attribute name
|                clientData - client data to use on the call
|        Output: kvalist    - variable argument list containing
|                             pointers to variables with which
|                             to set the attributes
|       Returns: TRUE on success, FALSE otherwise
|    Written By: Steve Kubica
|          Date: August 14, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kpds_call_attribute_handler(
   int     (*func) PROTO((kobject, ktoken, ktoken, kaddr, kva_list *)),
   kobject object,
   ktoken  segment,
   ktoken  attribute,
   kaddr   client_data,
   kvalist)
{
   kva_list list;
   int status;
   kva_start(list, client_data);
   status = func(object, segment, attribute, client_data, &list);
   kva_end(list);
   return status;
}

/****    =============================================================
 ****    Polymorphic Model Quasi Attribute Methods
 ****    =============================================================
 ****/

/*
 * Centerline's CLCC doesn't honor the ARGSUSED *standard* so this
 * pragma turns off warnings about unused arguments, which are
 * numerous in the next few routines...get the picture Centerline?!? 
 */
#ifdef __CLCC__
#pragma Warning_level (3)
#endif

/*********** ---------------------------------------------------------------
 ***********  KPDS_AXIS_ASSIGNMENT
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _axis_get
|    Written By: Steve Kubica
|          Date: Apr 28, 1994 17:39
------------------------------------------------------------*/
/* ARGSUSED */
static int
_axis_get(kobject object, ktoken segment, ktoken attribute, kaddr client_data,
	  kva_list *list)
{
   int  i;
   int *ord = NULL;
   int *tmp = NULL;


   /* 
    * the current axis assignment will be adequately described by
    * any one of the index orders of the value, mask, and location.  
    * try to get each of these in succession
    */
   if (kdms_query_segment(object, KPDS_SEGMENT_VALUE))
   {
      if (!kdms_get_attribute(object, KPDS_SEGMENT_VALUE, 
			      KDMS_INDEX_ORDER, &ord))
	 return FALSE;
   }
   else if (kdms_query_segment(object, KPDS_SEGMENT_MASK))
   {
      if (!kdms_get_attribute(object, KPDS_SEGMENT_MASK, 
			      KDMS_INDEX_ORDER, &ord))
	 return FALSE;
   }
   else if (kdms_query_segment(object, KPDS_SEGMENT_LOCATION) && 
	    kpds_location_grid(object) == KCURVILINEAR)
   {
      /*-- this is only good for curvilinear location --*/
      if (!kdms_get_attribute(object, KPDS_SEGMENT_LOCATION, 
			      KDMS_INDEX_ORDER, &ord))
	 return FALSE;
   }
   else
      return FALSE; /*-- nothing to get --*/

   /* at this point, the ord array has what we want and we can return it */

   /* 
    *  the first 3 entries of _any_ order array are assignment for
    *  width, height, and, depth 
    */
   for (i = 0; i < 3; i++)	/* return the axis */
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = ord[i];

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _axis_set
|    Written By: Steve Kubica
|          Date: Apr 28, 1994 17:39
------------------------------------------------------------*/
/* ARGSUSED */
static int
_axis_set(kobject object, ktoken segment, ktoken attribute, kaddr client_data,
	  kva_list *list)
{
   int newaxis[3];
   int nord[KDMS_MAX_DIM];

   int  i;
   int *ord = NULL;
   int *pos = NULL;
   int *off = NULL;


   /*-- pull the new axis assignement off the variable argument list --*/
   for (i = 0; i < 3; i++)
      newaxis[i] = kva_arg(*list, int);

   /*-- make sure that these are valid indices to be setting --*/
   if ((newaxis[0]!=KWIDTH && newaxis[0]!=KHEIGHT && newaxis[0]!=KDEPTH) ||
       (newaxis[1]!=KWIDTH && newaxis[1]!=KHEIGHT && newaxis[1]!=KDEPTH) ||
       (newaxis[2]!=KWIDTH && newaxis[2]!=KHEIGHT && newaxis[2]!=KDEPTH))
   {
      /* error = BOGUS AXIS */
      return FALSE;
   }

   /*-- make sure that there are no repeat indices --*/
   if (newaxis[0] == newaxis[1] ||
       newaxis[0] == newaxis[2] ||
       newaxis[1] == newaxis[2])
   {
      /* error = REPEAT AXIS */
      return FALSE;
   }

   /* 
    * the axis assignement will affect the value, mask and location.
    * try to go each of these segments and reassign the axis and
    * reorder their positions and offsets.  
    */

   /*
    *	Reorient the value segment
    */
   if (kdms_query_segment(object, KPDS_SEGMENT_VALUE))
   {
      if (!kdms_get_attribute(object, KPDS_SEGMENT_VALUE,
			      KDMS_INDEX_ORDER, &ord))
	 return FALSE;

      /*-- figure out the new index order --*/
      nord[KWID] = newaxis[KWID];
      nord[KHGT] = newaxis[KHGT];
      nord[KDEP] = newaxis[KDEP];
      nord[KTIM] = KTIME;
      nord[KELM] = KELEMENTS;

      /*-- get internal position and offset arrays and reorder in place --*/
      kpds_get_access_info(object, pds_value_token, NULL, &pos, &off,
			   NULL, NULL);

      /*-- reorder the position and offset --*/
      kdas_array_reorder(pos, 5, ord, nord);
      kdas_array_reorder(off, 5, ord, nord);

      if (!kdms_set_attribute(object, KPDS_SEGMENT_VALUE,
			      KDMS_INDEX_ORDER, nord))
	 return FALSE;
   }

   /*
    *	Reorient the mask segment
    */
   if (kdms_query_segment(object, KPDS_SEGMENT_MASK))
   {
      if (!kdms_get_attribute(object, KPDS_SEGMENT_MASK,
			      KDMS_INDEX_ORDER, &ord))
	 return FALSE;

      /*-- figure out the new index order --*/
      nord[KWID] = newaxis[KWID];
      nord[KHGT] = newaxis[KHGT];
      nord[KDEP] = newaxis[KDEP];
      nord[KTIM] = KTIME;
      nord[KELM] = KELEMENTS;

      /*-- get internal position and offset arrays and reorder in place --*/
      kpds_get_access_info(object, pds_mask_token, NULL, &pos, &off,
			   NULL, NULL);

      /*-- reorder the position and offset --*/
      kdas_array_reorder(pos, 5, ord, nord);
      kdas_array_reorder(off, 5, ord, nord);
      
      if (!kdms_set_attribute(object, KPDS_SEGMENT_MASK,
			       KDMS_INDEX_ORDER, nord))
	 return FALSE;
   }

   /*
    *	Reorient the location segment
    */
   if (kdms_query_segment(object, KPDS_SEGMENT_LOCATION))
   {
      int order[KPDS_LOCATION_DIM] = {KWIDTH, KHEIGHT, KDEPTH, KDIMENSION};
      double x, y, z;
      double nx, ny, nz;

      int grid;
   
      /*-- get the current grid type -- KRECTILINEAR or KCURVILINEAR --*/
      if (!kpds_get_attribute(object, KPDS_LOCATION_GRID, &grid))
	 return(FALSE);

      if (grid == KUNIFORM)
      {
	 /*-- reorder the begin point --*/ 
	 if (!kpds_get_attribute(object, KPDS_LOCATION_BEGIN, &x, &y, &z))
	    return FALSE;

	 nx = ((newaxis[KWID]==KDEPTH)  ? z :
	      ((newaxis[KWID]==KHEIGHT) ? y : x));

	 ny = ((newaxis[KHGT]==KDEPTH)  ? z :
	      ((newaxis[KHGT]==KHEIGHT) ? y : x));

	 nz = ((newaxis[KDEP]==KDEPTH)  ? z :
	      ((newaxis[KDEP]==KHEIGHT) ? y : x));

	 if (!kpds_set_attribute(object, KPDS_LOCATION_BEGIN, nx, ny, nz))
	    return FALSE;

	 /*-- reorder the end point --*/ 
	 if (!kpds_get_attribute(object, KPDS_LOCATION_END, &x, &y, &z))
	    return FALSE;

	 nx = ((newaxis[KWID]==KDEPTH)  ? z :
	      ((newaxis[KWID]==KHEIGHT) ? y : x));

	 ny = ((newaxis[KHGT]==KDEPTH)  ? z :
	      ((newaxis[KHGT]==KHEIGHT) ? y : x));

	 nz = ((newaxis[KDEP]==KDEPTH)  ? z :
	      ((newaxis[KDEP]==KHEIGHT) ? y : x));
	 
	 if (!kpds_set_attribute(object, KPDS_LOCATION_END, nx, ny, nz))
	    return FALSE;

	 /*-- figure out the new index order --*/
	 nord[KWID] = newaxis[KWID];
	 nord[KHGT] = newaxis[KHGT];
	 nord[KDEP] = newaxis[KDEP];
	 nord[KDIM] = KDIMENSION;

	 /*-- get internal position and offset arrays and reorder in place --*/
	 kpds_get_access_info(object, pds_location_token, NULL, &pos, &off,
			      NULL, NULL);
      
	 /*-- reorder the position and offset --*/
	 kdas_array_reorder(pos, 4, order, nord);
	 kdas_array_reorder(off, 4, order, nord);
      }
      else if (grid == KRECTILINEAR)
      {
	 char *names[3] = {"wid", "hgt", "dep"};

	 /*-- rename the width to a temporary name --*/
	 if (!kdms_rename_segment(object, KPDS_SEGMENT_WIDTH, 
				  names[newaxis[KWID]-KWIDTH]))
	    return FALSE;

	 /*-- rename the height to a temporary name --*/
	 if (!kdms_rename_segment(object, KPDS_SEGMENT_HEIGHT, 
				  names[newaxis[KHGT]-KWIDTH]))
	    return FALSE;

	 /*-- rename the depth to a temporary name --*/
	 if (!kdms_rename_segment(object, KPDS_SEGMENT_DEPTH, 
				  names[newaxis[KDEP]-KWIDTH]))
	    return FALSE;

	 /*-- rename all the temporary names to their proper names --*/
	 if (!kdms_rename_segment(object, names[KWID], KPDS_SEGMENT_WIDTH))
	    return FALSE;
	 if (!kdms_rename_segment(object, names[KHGT], KPDS_SEGMENT_HEIGHT))
	    return FALSE;
	 if (!kdms_rename_segment(object, names[KDEP], KPDS_SEGMENT_DEPTH))
	    return FALSE;

	 /*-- figure out the new index order --*/
	 nord[KWID] = newaxis[KWID];
	 nord[KHGT] = newaxis[KHGT];
	 nord[KDEP] = newaxis[KDEP];
	 nord[KDIM] = KDIMENSION;

	 /*-- get internal position and offset arrays and reorder in place --*/
	 kpds_get_access_info(object, pds_location_token, NULL, &pos, &off,
			      NULL, NULL);
      
	 /*-- reorder the position and offset --*/
	 kdas_array_reorder(pos, 4, order, nord);
	 kdas_array_reorder(off, 4, order, nord);

	 return TRUE;
      }
      else if (grid == KCURVILINEAR)
      {
	 /*-- retrieve all the attributes which need adjusting --*/
	 if (!kdms_get_attribute(object, KPDS_SEGMENT_LOCATION,
				 KDMS_INDEX_ORDER,   &ord))
	    return FALSE;

	 /*-- figure out the new index order --*/
	 nord[KWID] = newaxis[KWID];
	 nord[KHGT] = newaxis[KHGT];
	 nord[KDEP] = newaxis[KDEP];
	 nord[KDIM] = KDIMENSION;

	 /*-- get internal position and offset arrays and reorder in place --*/
	 kpds_get_access_info(object, pds_location_token, NULL, &pos, &off,
			      NULL, NULL);
      
	 /*-- reorder the position and offset --*/
	 kdas_array_reorder(pos, 4, ord, nord);
	 kdas_array_reorder(off, 4, ord, nord);

	 if (!kdms_set_attribute(object, KPDS_SEGMENT_LOCATION,
				 KDMS_INDEX_ORDER, nord)) 
	    return FALSE;
      }
      
      return TRUE;
   }

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_COMPLEX_CONVERT
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _complex_convert_set
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_complex_convert_get(kobject object, ktoken association, ktoken attribute,
		     kaddr client_data, kva_list *list)
{
   return kpds_intelligent_vget_attribute(object, (char *) client_data, 
					  KDMS_COMPLEX_CONVERT, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _complex_convert_set
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_complex_convert_set(kobject object, ktoken segment, ktoken attribute, 
		     kaddr client_data, kva_list *list)
{
   return kpds_intelligent_vset_attribute(object, (char *) client_data,
					  KDMS_COMPLEX_CONVERT, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _complex_convert_match
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_complex_convert_match(kobject object1, kobject object2, ktoken segment, 
		       ktoken attribute,kaddr client_data1,kaddr client_data2)
{
   int v1, v2;

   if (!kpds_call_attribute_handler(_complex_convert_get, object1, segment, 
				    attribute, client_data1, &v1) ||
       !kpds_call_attribute_handler(_complex_convert_get, object2, segment, 
				    attribute, client_data2, &v2))
      return FALSE;

   return (v1 == v2);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _complex_convert_copy
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_complex_convert_copy(kobject object1, kobject object2, ktoken segment, 
		      ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   int v;
   
   return (kpds_call_attribute_handler(_complex_convert_get, object1, segment, 
				       attribute, client_data1, &v) &&
	   kpds_call_attribute_handler(_complex_convert_set, object2, segment,
					attribute, client_data2, v));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _complex_convert_query
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_complex_convert_query(kobject object, ktoken segment, ktoken attribute, 
		       kaddr client_data, int *numargs, int *argsize, 
		       int *datatype, int *permanent)
{
   if (!kdms_query_segment(object, (char *)client_data))
      return FALSE;

   if (numargs)   *numargs   = 1;
   if (argsize)   *argsize   = 1;
   if (datatype)  *datatype  = KINT;
   if (permanent) *permanent = FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _complex_convert_print
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_complex_convert_print(kobject object, ktoken segment, ktoken attribute, 
		       kaddr client_data, kfile *outfile)
{
   int v;
   
   if (!kpds_call_attribute_handler(_complex_convert_get, object, segment, 
				    attribute, client_data, &v))
      return FALSE;

   if (outfile)
   {
      switch (v)
      {
	 case KIMAGINARY:
	    kfprintf(outfile, "KIMAGINARY");
	    break;
	 case KMAGNITUDE:
	    kfprintf(outfile, "KMAGNITUDE");
	    break;
	 case KPHASE:
	    kfprintf(outfile, "KPHASE");
	    break;
	 case KREAL:
	    kfprintf(outfile, "KREAL");
	    break;
	 case KLOGMAG:
	    kfprintf(outfile, "KLOGMAG");
	    break;
	 case KLOGMAGP1:
	    kfprintf(outfile, "KLOGMAGP1");
	    break;
	 case KLOGMAGSQ:
	    kfprintf(outfile, "KLOGMAGSQ");
	    break;
	 case KLOGMAGSQP1:
	    kfprintf(outfile, "KLOGMAGSQP1");
	    break;
	 default:
	    kfprintf(outfile, "%d (invalid)", v);
      }
   }

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_INTERPOLATE
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _interpolate_get
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_interpolate_get(kobject object, ktoken assoc, ktoken attr, kaddr client_data,
		 kva_list *list)
{
   return kpds_intelligent_vget_attribute(object, (char *) client_data, 
					  KDMS_INTERPOLATE, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _interpolate_set
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_interpolate_set(kobject object, ktoken segment, ktoken attribute, 
		 kaddr client_data, kva_list *list)
{
   return kpds_intelligent_vset_attribute(object, (char *) client_data,
					  KDMS_INTERPOLATE, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _interpolate_match
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_interpolate_match(kobject object1, kobject object2, ktoken segment, 
		   ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   int i1, i2;

   if (!kpds_call_attribute_handler(_interpolate_get, object1, segment, 
				    attribute, client_data1, &i1) ||
       !kpds_call_attribute_handler(_interpolate_get, object2, segment, 
				    attribute, client_data2, &i2))
      return FALSE;

   return (i1 == i2);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _interpolate_copy
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_interpolate_copy(kobject object1, kobject object2, ktoken segment, 
		  ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   int i;
   
   return (kpds_call_attribute_handler(_interpolate_get, object1, segment, 
				    attribute, client_data1, &i) &&
	   kpds_call_attribute_handler(_interpolate_set, object2, segment, 
				       attribute, client_data2, i));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _interpolate_query
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_interpolate_query(kobject object, ktoken segment, ktoken attribute, 
		   kaddr client_data, int *numargs, int *argsize, 
		   int *datatype, int *permanent)
{
   if (!kdms_query_segment(object, (char *)client_data))
      return FALSE;

   if (numargs)   *numargs   = 1;
   if (argsize)   *argsize   = 1;
   if (datatype)  *datatype  = KINT;
   if (permanent) *permanent = FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _interpolate_print
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_interpolate_print(kobject object, ktoken segment, ktoken attribute, 
		   kaddr client_data, kfile *outfile)
{
   int i;
   
   if (!kpds_call_attribute_handler(_interpolate_get, object, segment, 
				    attribute, client_data, &i))
      return FALSE;


   if (outfile)
   {
      switch (i)
      {
	 case KNONE:
	    kfprintf(outfile, "KNONE");
	    break;
	 case KPAD:
	    kfprintf(outfile, "KPAD");
	    break;
	 case KZERO_ORDER:
	    kfprintf(outfile, "KZERO_ORDER");
	    break;
	 default:
	    kfprintf(outfile, "%d (invalid)", i);
      }
   }

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_NORM_MIN
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _norm_min_get
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_norm_min_get(kobject object, ktoken association, ktoken attribute,
	      kaddr client_data, kva_list *list)
{
   return kpds_intelligent_vget_attribute(object, (char *) client_data, 
					  KDMS_NORM_MIN, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _norm_min_set
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_norm_min_set(kobject object, ktoken assoc, ktoken attr, kaddr client_data,
	      kva_list *list)
{
   return kpds_intelligent_vset_attribute(object, (char *) client_data,
					  KDMS_NORM_MIN, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _norm_min_match
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_norm_min_match(kobject object1, kobject object2, ktoken segment, 
		ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   double v1, v2;

   if (!kpds_call_attribute_handler(_norm_min_get, object1, segment, 
				    attribute, client_data1, &v1) ||
       !kpds_call_attribute_handler(_norm_min_get, object2, segment, 
				    attribute, client_data2, &v2))
      return FALSE;

   return (v1 == v2);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _norm_min_copy
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_norm_min_copy(kobject object1, kobject object2, ktoken segment, 
	       ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   double v;
   
   return (kpds_call_attribute_handler(_norm_min_get, object1, segment, 
				       attribute, client_data1, &v) &&
	   kpds_call_attribute_handler(_norm_min_set, object2, segment, 
				       attribute, client_data2, v));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _norm_min_query
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_norm_min_query(kobject object, ktoken segment, ktoken attribute, 
		kaddr client_data, int *numargs, int *argsize, 
		int *datatype, int *permanent)
{
   if (!kdms_query_segment(object, (char *)client_data))
      return FALSE;

   if (numargs)   *numargs   = 1;
   if (argsize)   *argsize   = 1;
   if (datatype)  *datatype  = KDOUBLE;
   if (permanent) *permanent = FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _norm_min_print
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_norm_min_print(kobject object, ktoken segment, ktoken attribute, 
		kaddr client_data, kfile *outfile)
{
   double v;
   
   if (!kpds_call_attribute_handler(_norm_min_get, object, segment, 
				    attribute, client_data, &v))
      return FALSE;

   if (outfile)
      kfprintf(outfile, "%g", v);

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_NORM_MAX
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _norm_max_get
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_norm_max_get(kobject object, ktoken association, ktoken attribute,
	      kaddr client_data, kva_list *list)
{
   return kpds_intelligent_vget_attribute(object, (char *) client_data, 
					  KDMS_NORM_MAX, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _norm_max_set
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_norm_max_set(kobject object, ktoken assoc, ktoken attr, kaddr client_data,
	      kva_list *list)
{
   return kpds_intelligent_vset_attribute(object, (char *) client_data,
					  KDMS_NORM_MIN, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _norm_max_match
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_norm_max_match(kobject object1, kobject object2, ktoken segment, 
		ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   double v1, v2;

   if (!kpds_call_attribute_handler(_norm_max_get, object1, segment, 
				    attribute, client_data1, &v1) ||
       !kpds_call_attribute_handler(_norm_max_get, object2, segment, 
				    attribute, client_data2, &v2))
      return FALSE;

   return (v1 == v2);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _norm_max_copy
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_norm_max_copy(kobject object1, kobject object2, ktoken segment, 
	       ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   double v;
   
   return (kpds_call_attribute_handler(_norm_max_get, object1, segment, 
				       attribute, client_data1, &v) &&
	   kpds_call_attribute_handler(_norm_max_set, object2, segment, 
				       attribute, client_data2, v));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _norm_max_query
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_norm_max_query(kobject object, ktoken segment, ktoken attribute, 
		kaddr client_data, int *numargs, int *argsize, 
		int *datatype, int *permanent)
{
   if (!kdms_query_segment(object, (char *)client_data))
      return FALSE;

   if (numargs)   *numargs   = 1;
   if (argsize)   *argsize   = 1;
   if (datatype)  *datatype  = KDOUBLE;
   if (permanent) *permanent = FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _norm_max_print
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_norm_max_print(kobject object, ktoken segment, ktoken attribute, 
		kaddr client_data, kfile *outfile)
{
   double v;
   
   if (!kpds_call_attribute_handler(_norm_max_get, object, segment, 
				    attribute, client_data, &v))
      return FALSE;

   if (outfile)
      kfprintf(outfile, "%g", v);

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_PAD_VALUE
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _pad_value_get
|    Written By: Jeremy Worley & John Salas
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_pad_value_get(kobject object, ktoken association, ktoken attribute,
	      kaddr client_data, kva_list *list)
{
   double i;

   if (kstrcmp((char *)client_data, KPDS_SEGMENT_MASK) == 0)
      return kdms_get_attribute(object, KPDS_SEGMENT_MASK, KDMS_PAD_VALUE,
				(double *) kva_arg(*list, double *), &i);
   else
      return kpds_intelligent_vget_attribute(object, (char *) client_data, 
					     KDMS_PAD_VALUE, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _pad_value_set
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_pad_value_set(kobject object, ktoken assoc, ktoken attr, kaddr client_data,
	       kva_list *list)
{
   if (kstrcmp((char *)client_data, KPDS_SEGMENT_MASK) == 0)
      return kdms_set_attributes(object, KPDS_SEGMENT_MASK, KDMS_PAD_VALUE, 
				 (double) kva_arg(*list, double), 0.0, 
				 NULL);
   else
      return kpds_intelligent_vset_attribute(object, (char *) client_data,
					     KDMS_PAD_VALUE, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _pad_value_match
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_pad_value_match(kobject object1, kobject object2, ktoken segment, 
		 ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   double rv1, iv1, rv2, iv2;

   if (!kpds_call_attribute_handler(_norm_max_get, object1, segment, 
				    attribute, client_data1, &rv1, &iv1) ||
       !kpds_call_attribute_handler(_norm_max_get, object2, segment, 
				    attribute, client_data2, &rv2, &iv2))
      return FALSE;

   return ((rv1 == rv2) && (iv1 == iv2));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _pad_value_copy
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_pad_value_copy(kobject object1, kobject object2, ktoken segment, 
		ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   double rv, iv;
   
   return (kpds_call_attribute_handler(_norm_max_get, object1, segment, 
				       attribute, client_data1, &rv, &iv) &&
	   kpds_call_attribute_handler(_norm_max_set, object2, segment, 
				       attribute, client_data2, rv, iv));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _pad_value_query
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_pad_value_query(kobject object, ktoken segment, ktoken attribute, 
		kaddr client_data, int *numargs, int *argsize, 
		int *datatype, int *permanent)
{
   if (!kdms_query_segment(object, (char *)client_data))
      return FALSE;

   if (argsize)   *argsize   = 1;
   if (datatype)  *datatype  = KDOUBLE;
   if (permanent) *permanent = FALSE;
   if (numargs)
      *numargs = (kstrcmp((char *)client_data, KPDS_SEGMENT_MASK) == 0)? 1 : 2;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _pad_value_print
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_pad_value_print(kobject object, ktoken segment, ktoken attribute, 
		kaddr client_data, kfile *outfile)
{
   double rv, iv;
   
   if (!kpds_call_attribute_handler(_norm_max_get, object, segment, 
				    attribute, client_data, &rv, &iv))
      return FALSE;

   if (outfile)
   {
      if (kstrcmp((char *)client_data, KPDS_SEGMENT_MASK) == 0)
	 kfprintf(outfile, "%g", rv);
      else
	 kfprintf(outfile, "(%g, %g)", rv, iv);
   }

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_SCALE_FACTOR
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _scale_factor_get
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_scale_factor_get(kobject object, ktoken association, ktoken attribute,
		  kaddr client_data, kva_list *list)
{
   return kpds_intelligent_vget_attribute(object, (char *) client_data, 
					  KDMS_SCALE_FACTOR, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scale_factor_set
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_scale_factor_set(kobject object, ktoken assoc, ktoken attr, kaddr client_data,
		  kva_list *list)
{
   return kpds_intelligent_vset_attribute(object, (char *) client_data,
					  KDMS_SCALE_FACTOR, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scale_factor_match
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_scale_factor_match(kobject object1, kobject object2, ktoken segment, 
		    ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   double v1, v2;

   if (!kpds_call_attribute_handler(_scale_factor_get, object1, segment, 
				    attribute, client_data1, &v1) ||
       !kpds_call_attribute_handler(_scale_factor_get, object2, segment, 
				    attribute, client_data2, &v2))
      return FALSE;

   return (v1 == v2);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scale_factor_copy
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_scale_factor_copy(kobject object1, kobject object2, ktoken segment, 
		   ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   double v;
   
   return (kpds_call_attribute_handler(_scale_factor_get, object1, segment, 
				       attribute, client_data1, &v) &&
	   kpds_call_attribute_handler(_scale_factor_set, object2, segment, 
				       attribute, client_data2, v));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scale_factor_query
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_scale_factor_query(kobject object, ktoken segment, ktoken attribute, 
		    kaddr client_data, int *numargs, int *argsize, 
		    int *datatype, int *permanent)
{
   if (!kdms_query_segment(object, (char *)client_data))
      return FALSE;

   if (numargs)   *numargs   = 1;
   if (argsize)   *argsize   = 1;
   if (datatype)  *datatype  = KDOUBLE;
   if (permanent) *permanent = FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scale_factor_print
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_scale_factor_print(kobject object, ktoken segment, ktoken attribute, 
		    kaddr client_data, kfile *outfile)
{
   double v;
   
   if (!kpds_call_attribute_handler(_scale_factor_get, object, segment, 
				    attribute, client_data, &v))
      return FALSE;

   if (outfile)
      kfprintf(outfile, "%g", v);

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_SCALE_OFFSET
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _scale_offset_get
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_scale_offset_get(kobject object, ktoken association, ktoken attribute,
		  kaddr client_data, kva_list *list)
{
   return kpds_intelligent_vget_attribute(object, (char *) client_data, 
					  KDMS_SCALE_OFFSET, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scale_offset_set
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_scale_offset_set(kobject object, ktoken assoc, ktoken attr, kaddr client_data,
		  kva_list *list)
{
   return kpds_intelligent_vset_attribute(object, (char *) client_data,
					  KDMS_SCALE_OFFSET, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scale_offset_match
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_scale_offset_match(kobject object1, kobject object2, ktoken segment, 
		    ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   double rv1, rv2, iv1, iv2;

   if (!kpds_call_attribute_handler(_scale_offset_get, object1, segment, 
				    attribute, client_data1, &rv1, &iv1) ||
       !kpds_call_attribute_handler(_scale_offset_get, object2, segment, 
				    attribute, client_data2, &rv2, &iv2))
      return FALSE;

   return ((rv1 == rv2) && (iv1 == iv2));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scale_offset_copy
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_scale_offset_copy(kobject object1, kobject object2, ktoken segment, 
		   ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   double rv, iv;
   
   return (kpds_call_attribute_handler(_scale_offset_get, object1, segment, 
				       attribute, client_data1, &rv, &iv) &&
	   kpds_call_attribute_handler(_scale_offset_set, object2, segment, 
				       attribute, client_data2, rv, iv));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scale_offset_query
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_scale_offset_query(kobject object, ktoken segment, ktoken attribute, 
		    kaddr client_data, int *numargs, int *argsize, 
		    int *datatype, int *permanent)
{
   if (!kdms_query_segment(object, (char *)client_data))
      return FALSE;

   if (numargs)   *numargs   = 2;
   if (argsize)   *argsize   = 1;
   if (datatype)  *datatype  = KDOUBLE;
   if (permanent) *permanent = FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scale_offset_print
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_scale_offset_print(kobject object, ktoken segment, ktoken attribute, 
		    kaddr client_data, kfile *outfile)
{
   double rv, iv;
   
   if (!kpds_call_attribute_handler(_scale_offset_get, object, segment, 
				    attribute, client_data, &rv, &iv))
      return FALSE;

   if (outfile)
      kfprintf(outfile, "(%g, &g)", rv, iv);

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_SCALING
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _scaling_get
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_scaling_get(kobject object, ktoken association, ktoken attribute,
		  kaddr client_data, kva_list *list)
{
   return kpds_intelligent_vget_attribute(object, (char *) client_data, 
					  KDMS_SCALING, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scaling_set
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_scaling_set(kobject object, ktoken assoc, ktoken attr, kaddr client_data,
		  kva_list *list)
{
   return kpds_intelligent_vset_attribute(object, (char *) client_data,
					  KDMS_SCALING, list);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scaling_match
|    Written By: Steve Kubica
|          Date: February 19, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_scaling_match(kobject object1, kobject object2, ktoken segment, 
		    ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   int v1, v2;

   if (!kpds_call_attribute_handler(_scaling_get, object1, segment, 
				    attribute, client_data1, &v1) ||
       !kpds_call_attribute_handler(_scaling_get, object2, segment, 
				    attribute, client_data2, &v2))
      return FALSE;

   return (v1 == v2);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scaling_copy
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_scaling_copy(kobject object1, kobject object2, ktoken segment, 
		   ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   int v;
   
   return (kpds_call_attribute_handler(_scaling_get, object1, segment, 
				       attribute, client_data1, &v) &&
	   kpds_call_attribute_handler(_scaling_set, object2, segment, 
				       attribute, client_data2, v));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scaling_query
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_scaling_query(kobject object, ktoken segment, ktoken attribute, 
	       kaddr client_data, int *numargs, int *argsize, 
	       int *datatype, int *permanent)
{
   if (!kdms_query_segment(object, (char *)client_data))
      return FALSE;

   if (numargs)   *numargs   = 1;
   if (argsize)   *argsize   = 1;
   if (datatype)  *datatype  = KINT;
   if (permanent) *permanent = FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _scaling_print
|    Written By: Steve Kubica
|          Date: February 19, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_scaling_print(kobject object, ktoken segment, ktoken attribute, 
	       kaddr client_data, kfile *outfile)
{
   int v;
   
   if (!kpds_call_attribute_handler(_scaling_get, object, segment, 
				    attribute, client_data, &v))
      return FALSE;

   if (outfile)
      switch (v)
      {
	 case KNONE:
	    kfprintf(outfile, "KNONE");
	    break;
	 case KNORMALIZE:
	    kfprintf(outfile, "KNORMALIZE");
	    break;
	 case KSCALE:
	    kfprintf(outfile, "KPHASE");
	    break;
	 default:
	    kfprintf(outfile, "%d (invalid)", v);
      }

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_HISTORY
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _history_get
|    Written By: Steve Kubica
|          Date: Sep 28, 1993 10:28
------------------------------------------------------------*/
/* ARGSUSED */
static int
_history_get(kobject obj, ktoken assoc, ktoken attr, kaddr clientData,
	     kva_list *list)
{
   char **ret_hist;
   char  *hist;


   if (!kdms_get_attribute(obj, NULL, _INTERNAL_HISTORY, &hist))
      return FALSE;
   
   ret_hist = kva_arg(*list, char **);
   if (ret_hist != NULL) *ret_hist = hist;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _history_set
|    Written By: Steve Kubica
|          Date: Sep 28, 1993 10:28
------------------------------------------------------------*/
/* ARGSUSED */
static int
_history_set(kobject obj, ktoken assoc, ktoken attr, kaddr clientData,
	     kva_list *list)
{
   char *hist;
   char *new_hist;
   int   mode;
   

   hist = kva_arg(*list, char *);

   /* if the history environment variable is set, then use history */
   if (kgetenv("KHOROS_HISTORY"))
   {
      /* get the history mode */
      if (!kdms_get_attribute(obj, NULL, KPDS_HISTORY_MODE, &mode))
	  return FALSE;
   
      /* if the history is NULL, then assume we are zeroing it out */
      if (hist != NULL)
      {
	 /* either append the new history or replace it */
	 if (mode == KAPPEND_HISTORY)
	 {
	    char *old_hist;
	    
	    /* 	get the old history */
	    if (!kdms_get_attribute(obj, NULL, _INTERNAL_HISTORY, &old_hist))
	       return FALSE;
	    
	    /* append the new history to the old -- separated by a '\n' */
	    new_hist = kstring_3cat(old_hist, "\n", hist, NULL);
	 }
	 else
	 {
	    /* simply make our own copy of the history */
	    new_hist = kstring_copy(hist, NULL);
	 }
      }
      else
	 new_hist = NULL;

      /* set the new history */
      if (!kdms_set_attribute(obj, NULL, _INTERNAL_HISTORY, new_hist))
	 return FALSE;

      /* and tidy up the string */
      kfree_and_NULL (new_hist);
   }

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _history_match
|    Written By: Steve Kubica
|          Date: Dec 27, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static int
_history_match(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	       kaddr clientData1, kaddr clientData2)
{
   char *hist1;
   char *hist2;


   if (!kdms_get_attribute(obj1, NULL, _INTERNAL_HISTORY, &hist1))
      return FALSE;

   if (!kdms_get_attribute(obj2, NULL, _INTERNAL_HISTORY, &hist2))
      return FALSE;

   if (kstrcmp(hist1, hist2) == 0)
      return TRUE;
   else
      return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _history_copy
|    Written By: Steve Kubica
|          Date: Apr 28, 1994 17:48
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_history_copy(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	      kaddr clientData1, kaddr clientData2)
{
   char *hist;
   

   if (!kdms_get_attribute(obj1, NULL, _INTERNAL_HISTORY, &hist))
      return FALSE;

   if (!kdms_set_attribute(obj2, NULL, _INTERNAL_HISTORY, hist))
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _history_query
|    Written By: Steve Kubica
|          Date: Mar 30, 1994 14:36
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_history_query(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
	       int *num_args, int *arg_size, int *data_type,
	       int *permanent)
{
   if (num_args)
      *num_args = 1;
   if (arg_size)
      *arg_size = 1;
   if (data_type)
      *data_type = KSTRING;
   if (permanent)
      *permanent = TRUE;

   if (kgetenv("KHOROS_HISTORY"))
      return TRUE;
   else 
      return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _history_print
|    Written By: Steve Kubica
|          Date: Mar 30, 1994 14:36
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_history_print(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
	       kfile * outfile)
{
   char *hist;


   if (!kdms_get_attribute(obj, NULL, _INTERNAL_HISTORY, &hist))
      return FALSE;

   if (outfile)
      kfprintf(outfile, "%s", hist);

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_MINMAX
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _minmax_get
|    Written By: Jeremy Worley & John Salas
|          Date: Sep 28, 1993 10:28
------------------------------------------------------------*/
/* ARGSUSED */
static int
_minmax_get(kobject obj, ktoken assoc, ktoken attr, kaddr clientData,
	     kva_list *list)
{
   double *ret_smin;
   double *ret_smax;
   double  smin = 0.0;
   double  smax = 0.0;
   double  nmin;
   double  nmax;
   kaddr   data = NULL;
   kobject tmp;
   int datatype;
   int w;
   int h;
   int d;
   int t;
   int e;
   int n;
   int j;
   
   /* verify that the segment is present */
   if (!kdms_query_segment(obj, (char *)clientData))
      return FALSE;

   tmp = kdms_reference(obj);

   /* get the current presentation data type */
   kdms_get_attribute(tmp, (char *) clientData, KDMS_DATA_TYPE, &datatype);

   /* can't deal with complex and double complex, so convert to double */
   if (datatype == KCOMPLEX || datatype == KDCOMPLEX)
   {
      datatype = KDOUBLE;
      kdms_set_attribute(tmp, (char *) clientData, KDMS_DATA_TYPE, KDOUBLE);
   }

   if (datatype == KBIT)
   {
      datatype = KUBYTE;
      kdms_set_attribute(tmp, (char *) clientData, KDMS_DATA_TYPE, KUBYTE);
   }

   if (kstrcmp((char *) clientData, KPDS_SEGMENT_VALUE) == 0) 
   {
      kpds_get_attribute(tmp, KPDS_VALUE_OPTIMAL_REGION_SIZE,
			 &w, &h, &d, &t, &e, &n);

      kpds_set_attributes(tmp, 
			  KPDS_VALUE_REGION_SIZE, w, h, d, t, e,
			  KPDS_VALUE_POSITION,    0,0,0,0,0, 
			  NULL);
			     
      for (j = 0; j < n; j++)
      {
	 if ((data = kpds_get_data(tmp, KPDS_VALUE_REGION, data)) == NULL) 
	 {
	    kdms_close(tmp);
	    return FALSE;
	 }

	 kdata_minmax(data, w*h*d*t*e, datatype, &nmin, &nmax);

	 if (j > 0)
	 {
	    smin = kmin(nmin,smin);
	    smax = kmax(nmax,smax);
	 }
	 else
	 {
	    smin = nmin;
	    smax = nmax;
	 }
      }
      
      kfree_and_NULL(data);
   }
   else if (kstrcmp((char *) clientData, KPDS_SEGMENT_MAP) == 0)
   {
      if ((data = kpds_get_data(tmp, KPDS_MAP_REGION, NULL)) == NULL)
      {
	 kdms_close(tmp);
	 return FALSE;
      }

      kpds_get_attribute(tmp, KPDS_MAP_REGION_SIZE, &w, &h, &d, &t, &e);

      kdata_minmax(data, w*h*d*t*e, datatype, &smin, &smax);

      kfree_and_NULL(data);
   }
   else if (kstrcmp((char *) clientData, KPDS_SEGMENT_MASK) == 0)
   {
      if ((data = kpds_get_data(tmp, KPDS_MASK_REGION, NULL)) == NULL)
      {
	 kdms_close(tmp);
	 return FALSE;
      }
      
      kpds_get_attribute(tmp, KPDS_MASK_REGION_SIZE, &w, &h, &d, &t, &e);

      kdata_minmax(data, w*h*d*t*e, datatype, &smin, &smax);

      kfree_and_NULL(data);
   }
   else if (kstrcmp((char *) clientData, KPDS_SEGMENT_LOCATION) == 0)
   {
      int grid;
   
   
      /* get the current grid type -- KRECTILINEAR or KCURVILINEAR */
      if (!kpds_get_attribute(obj, KPDS_LOCATION_GRID, &grid))
	 return(FALSE);
      
      /* 
       *  if the grid type is rectlinear, then we need to get the 
       *  location points from the width, height, and depth segments.
       */ 
      if (grid == KRECTILINEAR) 
      {
	 kaddr data_w = NULL;
	 kaddr data_h = NULL;
	 kaddr data_d = NULL;

	 
	 data_w = kpds_get_data(tmp, KPDS_LOCATION_WIDTH_REGION, NULL);
	 data_h = kpds_get_data(tmp, KPDS_LOCATION_HEIGHT_REGION, NULL);
	 data_d = kpds_get_data(tmp, KPDS_LOCATION_DEPTH_REGION, NULL);
	 
	 if (data_w == NULL || data_h == NULL || data_d)
	 {
	    kdms_close(tmp);
	    return FALSE;
	 }

	 kpds_get_attribute(tmp, KPDS_LOCATION_REGION_SIZE, &w, &h, &d, &e);

	 kdata_minmax(data_w, w, datatype, &nmin, &nmax);
	 smin = kmin(nmin,smin);
	 smax = kmin(nmax,smax);
	 kdata_minmax(data_h, h, datatype, &nmin, &nmax);
	 smin = kmin(nmin,smin);
	 smax = kmin(nmax,smax);
	 kdata_minmax(data_d, d, datatype, &nmin, &nmax);
	 smin = kmin(nmin,smin);
	 smax = kmin(nmax,smax);

	    
	 kfree_and_NULL(data_w);  
	 kfree_and_NULL(data_h);  
	 kfree_and_NULL(data_d);  
      }
      else if (grid == KCURVILINEAR) 
      {
	 if ((data = kpds_get_data(tmp, KPDS_LOCATION_REGION, NULL)) == NULL)
	 {
	    kdms_close(tmp);
	    return FALSE;
	 }
      
	 kpds_get_attribute(tmp, KPDS_LOCATION_REGION_SIZE, &w, &h, &d, &e);

	 kdata_minmax(data, w*h*d*e, datatype, &smin, &smax);
      }

      kfree_and_NULL(data);
   }
   else if (!kstrcmp((char *)clientData, KPDS_SEGMENT_TIME) == 0)
   {
      if ((data = kpds_get_data(tmp, KPDS_TIME_REGION, NULL)) == NULL)
      {
	 kdms_close(tmp);
	 return FALSE;
      }
      
      kpds_get_attribute(tmp, KPDS_TIME_REGION_SIZE, &t);

      kdata_minmax(data, t, datatype, &smin, &smax);

      kfree_and_NULL(data);
   }

   ret_smin = kva_arg(*list, double *);
   ret_smax = kva_arg(*list, double *);

   if (ret_smin != NULL) *ret_smin = smin;
   if (ret_smax != NULL) *ret_smax = smax;

   kdms_close(tmp);

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_COUPLING
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _coupling_get
|    Written By: Steve Kubica
|          Date: April 11, 1996
------------------------------------------------------------*/
/* ARGSUSED */
static int
_coupling_get(kobject obj, ktoken assoc, ktoken attr, kaddr clientData,
	      kva_list *list)
{
   int  coupling = -1;
   int *value;

   /*-- try each of the polymorphic segments to get a value --*/
   if (kpds_query_value(obj))
      kdms_get_attribute(obj, KPDS_SEGMENT_VALUE, KDMS_COUPLING, &coupling);
   else if (kpds_query_mask(obj))
      kdms_get_attribute(obj, KPDS_SEGMENT_LOCATION, KDMS_COUPLING, &coupling);
   else if (kpds_query_location(obj) && 
	    kpds_location_grid(obj) == KCURVILINEAR)
      kdms_get_attribute(obj, KPDS_SEGMENT_LOCATION, KDMS_COUPLING, &coupling);
   else if (kpds_query_time(obj))
      kdms_get_attribute(obj, KPDS_SEGMENT_TIME, KDMS_COUPLING, &coupling);

   if ((value = kva_arg(*list, int *)) != NULL && coupling != -1)
      *value = coupling;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _coupling_set
|    Written By: Steve Kubica
|          Date: April 11, 1996
------------------------------------------------------------*/
/* ARGSUSED */
static int
_coupling_set(kobject obj, ktoken assoc, ktoken attr, kaddr clientData,
	      kva_list *list)
{
   int    coupling;
   int    i, num;
   char **segments;
   
   coupling = kva_arg(*list, int);

   /*-- get a list of all segments and set the coupling on them --*/
   segments = kdms_get_segment_names(obj, &num);

   for (i = 0; i < num; i++)
      kdms_set_attribute(obj, segments[i], KDMS_COUPLING, coupling);

   karray_free(segments, KSTRING, (size_t) num, NULL);
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _coupling_match
|    Written By: Steve Kubica
|          Date: April 11, 1996
------------------------------------------------------------*/
/* ARGSUSED */
static int
_coupling_match(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
		kaddr clientData1, kaddr clientData2)
{
   int coupling1, coupling2;

   kpds_get_attribute(obj1, KPDS_COUPLING, &coupling1);
   kpds_get_attribute(obj2, KPDS_COUPLING, &coupling2);

   return (coupling1 == coupling2);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _coupling_copy
|    Written By: Steve Kubica
|          Date: April 11, 1996
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_coupling_copy(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	       kaddr clientData1, kaddr clientData2)
{
   int coupling;

   kpds_get_attribute(obj1, KPDS_COUPLING, &coupling);
   kpds_set_attribute(obj2, KPDS_COUPLING, coupling);

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _coupling_query
|    Written By: Steve Kubica
|          Date: April 11, 1996
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_coupling_query(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
		int *numargs, int *argsize, int *datatype, int *permanent)
{
   /*-- just fill them out --*/
   if (numargs)
      *numargs = 1;
   if (datatype)
      *datatype = KINT;
   if (permanent)
      *permanent = FALSE;
   if (argsize)
      *argsize = 1;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _increment_size_print
|    Written By: Steve Kubica
|          Date: April 11, 1996
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_coupling_print(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
		kfile *outfile)
{
   int coupling;

   if (!kpds_get_attribute(obj, KPDS_COUPLING, &coupling))
      return FALSE;

   switch (coupling)
   {
      case KUNCOUPLED:
	 kfprintf(outfile,"KUNCOUPLED");
	 break;
      case KCOUPLED:
	 kfprintf(outfile,"KCOUPLED");
	 break;
      case KDEMAND:
	 kfprintf(outfile,"KDEMAND");
	 break;
      default:
	 kfprintf(outfile,"Can not determine coupling (%d)", coupling);
	 break;
   }
   
   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_DATA_FORMAT
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _data_format_get
|    Written By: Tom Robey
|          Date: November 19, 1996
------------------------------------------------------------*/
/* ARGSUSED */
static int
_data_format_get(kobject object,ktoken association,ktoken attribute,
   kaddr clientData,kva_list *list)
{
   int i;
   double min, max;
   int *data_format = NULL;

   i = kpds_get_attribute(object, KPDS_DATA_RANGE, &min, &max);
   if (i == 0)
      return FALSE;
   data_format = kva_arg(*list, int *);
   if (min == 0.0)
   {
      if (max == 1.0)
         *data_format = KFLOAT;
      else if (max == 255.0)
         *data_format = KUBYTE;
      else if (max == 65535.0)
         *data_format = KUSHORT;
      else
         *data_format = KUSERDEFINED;
   }
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _data_format_set
|    Written By: Tom Robey
|          Date: November 19, 1996
------------------------------------------------------------*/
/* ARGSUSED */
static int
_data_format_set(kobject object,ktoken association,ktoken attribute,
   kaddr clientData,kva_list *list)
{
   int i;
   int num_args, arg_size, data_type, permanent;
   int data_format;
   char *lib = "licomposite";
   char *rtn = "_data_format_set";

   i=kdms_query_attribute(object,NULL,KPDS_DATA_RANGE,&num_args,&arg_size,
                          &data_type,&permanent);
   if (i == 0)
   {
      if(!(kdms_create_attribute(object,NULL,KPDS_DATA_RANGE,2,1,KDOUBLE,
            TRUE,TRUE)))
      {
         kerror(lib, rtn, "Failed to create the attribute.");
	 return FALSE;
      }
   }
   data_format = kva_arg(*list, int);
   switch (data_format) {
      case KUBYTE:
         if(!(kpds_set_attribute(object, KPDS_DATA_RANGE, 0.0, 255.0)))
         {
            kerror(lib, rtn, "Failed to set the data range attribute.");
            return FALSE;
         } 
         break;
      case KUSHORT:
         if(!(kpds_set_attribute(object, KPDS_DATA_RANGE, 0.0, 65535.0)))
         {
            kerror(lib, rtn, "Failed to set the data range attribute.");
            return FALSE;
         } 
         break;
      case KFLOAT:
         if(!(kpds_set_attribute(object, KPDS_DATA_RANGE, 0.0, 1.0)))
         {
            kerror(lib, rtn, "Failed to set the data range attribute.");
            return FALSE;
         }

         break;
      default:
         kerror(lib, rtn, "Undefined data format.");
         return FALSE;
   }
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _data_format_match
|    Written By: Tom Robey
|          Date: November 19, 1996
------------------------------------------------------------*/
/* ARGSUSED */
static int
_data_format_match(kobject object1,kobject object2,ktoken association,
   ktoken attribute,kaddr clientData1,kaddr clientData2)
{
   int i;

   i = kdms_match_attribute(object1, object2, NULL, KPDS_DATA_RANGE);
   return i;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _data_format_copy
|    Written By: Tom Robey
|          Date: November 19, 1996
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_data_format_copy(kobject object1,kobject object2,ktoken association,
   ktoken attribute,kaddr clientData1,kaddr clientData2)
{
   int i;

   i = kdms_copy_attribute(object1, object2, NULL, KPDS_DATA_RANGE);
   return i;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _data_format_query
|    Written By: Tom Robey
|          Date: November 19, 1996
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_data_format_query(kobject object,ktoken association,ktoken attribute,
   kaddr clientData,int *num_args,int *arg_size,int *data_type,
   int *permanent)
{
   int i, *i1, *i2, *i3, *i4;
   i = kdms_query_attribute(object, NULL, KPDS_DATA_RANGE, i1, i2, i3, i4);
   if (i == 0)
      return FALSE;
   *num_args = 1;
   *arg_size = 1;
   *data_type = KDOUBLE;
   *permanent = TRUE;
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _data_format_print
|    Written By: Tom Robey
|          Date: November 19, 1996
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_data_format_print(kobject object,ktoken association,ktoken attribute,
   kaddr clientData,kfile *outfile)
{
   int i, min, max;

   i = kpds_get_attribute(object, KPDS_DATA_RANGE, &min, &max);
   if (i == 0)
      return FALSE;
   if (min == 0.0)
   {
      if (max == 1.0)
         kfprintf(outfile, "Data Format: KFLOAT\n");
      else if (max == 255.0)
         kfprintf(outfile, "Data Format: KUBYTE\n");
      else if (max == 65535.0)
         kfprintf(outfile, "Data Format: KUSHORT\n");
      else
         kfprintf(outfile, "Data Format: KUSERDEFINED (%d to %d)\n", min, max);
   }
   return TRUE;
}

/****    =============================================================
 ****    Initialization Routine
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kpds_init_attributes()
|
|       Purpose: 
|
|         Input: none
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley, John Salas, and Steve Kubica
|          Date: Sep 28, 1993 08:10
| Modifications:
|
------------------------------------------------------------*/
void
kpds_init_attributes(void)
{
   kdms_define_attribute(KDMS_OBJECT, KDMS_COMMENT, 1, 1, KSTRING,
			 TRUE, TRUE, NULL);

   kdms_define_attribute(KDMS_OBJECT, KDMS_DATE, 1, 1, KSTRING,
			 TRUE, TRUE, NULL);

   kdms_define_attribute(KDMS_OBJECT, KPDS_MAPPING_MODE, 1, 1, KINT,
			 FALSE, FALSE, KUNMAPPED);

   kdms_define_attribute(KDMS_OBJECT, KPDS_MASKED_VALUE_PRESENTATION, 1, 1, 
			 KINT, TRUE, TRUE, KUSE_ORIGINAL);

   kdms_define_attribute(KDMS_OBJECT, KPDS_MASK_SUBSTITUTE_VALUE, 2, 1, 
			 KDOUBLE, TRUE, TRUE, 1.0, 0.0);

   /* 
    *   axis assignment 
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_AXIS_ASSIGNMENT, NULL,
			       _axis_get, _axis_set, NULL, NULL, NULL, NULL);

   /*
    *   complex convert attributes 
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_VALUE_COMPLEX_CONVERT, pds_value_string,
			       _complex_convert_get,   _complex_convert_set, 
			       _complex_convert_match, _complex_convert_copy, 
			       _complex_convert_query, _complex_convert_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MAP_COMPLEX_CONVERT,pds_map_string,
			       _complex_convert_get,   _complex_convert_set, 
			       _complex_convert_match, _complex_convert_copy, 
			       _complex_convert_query, _complex_convert_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_LOCATION_COMPLEX_CONVERT,
			       pds_location_string,
			       _complex_convert_get,   _complex_convert_set, 
			       _complex_convert_match, _complex_convert_copy, 
			       _complex_convert_query, _complex_convert_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_TIME_COMPLEX_CONVERT, pds_time_string,
			       _complex_convert_get,   _complex_convert_set, 
			       _complex_convert_match, _complex_convert_copy, 
			       _complex_convert_query, _complex_convert_print);

   /*
    *   interpolate attributes 
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MASK_INTERPOLATE, pds_mask_string,
			       _interpolate_get,   _interpolate_set, 
			       _interpolate_match, _interpolate_copy, 
			       _interpolate_query, _interpolate_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_VALUE_INTERPOLATE, pds_value_string,
			       _interpolate_get,   _interpolate_set, 
			       _interpolate_match, _interpolate_copy, 
			       _interpolate_query, _interpolate_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MAP_INTERPOLATE, pds_map_string,
			       _interpolate_get,   _interpolate_set, 
			       _interpolate_match, _interpolate_copy, 
			       _interpolate_query, _interpolate_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_LOCATION_INTERPOLATE, pds_location_string,
			       _interpolate_get,   _interpolate_set, 
			       _interpolate_match, _interpolate_copy, 
			       _interpolate_query, _interpolate_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_TIME_INTERPOLATE, pds_time_string,
			       _interpolate_get,   _interpolate_set, 
			       _interpolate_match, _interpolate_copy, 
			       _interpolate_query, _interpolate_print);

   /*
    *   normalization minimum attributes 
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_VALUE_NORM_MIN, pds_value_string,
			       _norm_min_get,   _norm_min_set, 
			       _norm_min_match, _norm_min_copy, 
			       _norm_min_query, _norm_min_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MAP_NORM_MIN, pds_map_string,
			       _norm_min_get,   _norm_min_set, 
			       _norm_min_match, _norm_min_copy, 
			       _norm_min_query, _norm_min_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_LOCATION_NORM_MIN, pds_location_string,
			       _norm_min_get,   _norm_min_set, 
			       _norm_min_match, _norm_min_copy, 
			       _norm_min_query, _norm_min_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_TIME_NORM_MIN, pds_time_string,
			       _norm_min_get,   _norm_min_set, 
			       _norm_min_match, _norm_min_copy, 
			       _norm_min_query, _norm_min_print);

   /*
    *   normalization maximum attributes 
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_VALUE_NORM_MAX, pds_value_string,
			       _norm_max_get,   _norm_max_set, 
			       _norm_max_match, _norm_max_copy, 
			       _norm_max_query, _norm_max_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MAP_NORM_MAX, pds_map_string,
			       _norm_max_get,   _norm_max_set, 
			       _norm_max_match, _norm_max_copy, 
			       _norm_max_query, _norm_max_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_LOCATION_NORM_MAX, pds_location_string,
			       _norm_max_get,   _norm_max_set, 
			       _norm_max_match, _norm_max_copy, 
			       _norm_max_query, _norm_max_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_TIME_NORM_MAX, pds_time_string,
			       _norm_max_get,   _norm_max_set, 
			       _norm_max_match, _norm_max_copy, 
			       _norm_max_query, _norm_max_print);

   /*
    *   pad value attributes 
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MASK_PAD_VALUE, pds_mask_string,
			       _pad_value_get,   _pad_value_set, 
			       _pad_value_match, _pad_value_copy, 
			       _pad_value_query, _pad_value_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_VALUE_PAD_VALUE, pds_value_string,
			       _pad_value_get,   _pad_value_set, 
			       _pad_value_match, _pad_value_copy, 
			       _pad_value_query, _pad_value_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MAP_PAD_VALUE, pds_map_string,
			       _pad_value_get,   _pad_value_set, 
			       _pad_value_match, _pad_value_copy, 
			       _pad_value_query, _pad_value_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_LOCATION_PAD_VALUE, pds_location_string,
			       _pad_value_get,   _pad_value_set, 
			       _pad_value_match, _pad_value_copy, 
			       _pad_value_query, _pad_value_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_TIME_PAD_VALUE, pds_time_string,
			       _pad_value_get,   _pad_value_set, 
			       _pad_value_match, _pad_value_copy, 
			       _pad_value_query, _pad_value_print);

   /*
    *   scale factor attributes 
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_VALUE_SCALE_FACTOR, pds_value_string,
			       _scale_factor_get,   _scale_factor_set, 
			       _scale_factor_match, _scale_factor_copy, 
			       _scale_factor_query, _scale_factor_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MAP_SCALE_FACTOR, pds_map_string,
			       _scale_factor_get,   _scale_factor_set, 
			       _scale_factor_match, _scale_factor_copy, 
			       _scale_factor_query, _scale_factor_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_LOCATION_SCALE_FACTOR, pds_location_string,
			       _scale_factor_get,   _scale_factor_set, 
			       _scale_factor_match, _scale_factor_copy, 
			       _scale_factor_query, _scale_factor_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_TIME_SCALE_FACTOR, pds_time_string,
			       _scale_factor_get,   _scale_factor_set, 
			       _scale_factor_match, _scale_factor_copy, 
			       _scale_factor_query, _scale_factor_print);

   /*
    *   scale offset attributes 
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_VALUE_SCALE_OFFSET, pds_value_string,
			       _scale_offset_get,   _scale_offset_set, 
			       _scale_offset_match, _scale_offset_copy,
			       _scale_offset_query, _scale_offset_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MAP_SCALE_OFFSET, pds_map_string,
			       _scale_offset_get,   _scale_offset_set, 
			       _scale_offset_match, _scale_offset_copy,
			       _scale_offset_query, _scale_offset_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_LOCATION_SCALE_OFFSET, pds_location_string,
			       _scale_offset_get,   _scale_offset_set, 
			       _scale_offset_match, _scale_offset_copy,
			       _scale_offset_query, _scale_offset_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_TIME_SCALE_OFFSET, pds_time_string,
			       _scale_offset_get,   _scale_offset_set, 
			       _scale_offset_match, _scale_offset_copy,
			       _scale_offset_query, _scale_offset_print);
   /*
    *   scaling attributes 
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_VALUE_SCALING, pds_value_string,
			       _scaling_get,   _scaling_set, 
			       _scaling_match, _scaling_copy, 
			       _scaling_query, _scaling_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MAP_SCALING, pds_map_string,
			       _scaling_get,   _scaling_set, 
			       _scaling_match, _scaling_copy, 
			       _scaling_query, _scaling_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_LOCATION_SCALING, pds_location_string,
			       _scaling_get,   _scaling_set, 
			       _scaling_match, _scaling_copy, 
			       _scaling_query, _scaling_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_TIME_SCALING, pds_time_string,
			       _scaling_get,   _scaling_set, 
			       _scaling_match, _scaling_copy, 
			       _scaling_query, _scaling_print);

   /*
    *   min/max attributes
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_VALUE_MINMAX, pds_value_string,
			       _minmax_get, NULL, NULL, NULL, NULL, NULL);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MAP_MINMAX, pds_map_string,
			       _minmax_get, NULL, NULL, NULL, NULL, NULL);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_LOCATION_MINMAX, pds_location_string,
			       _minmax_get, NULL, NULL, NULL, NULL, NULL);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_TIME_MINMAX, pds_time_string,
			       _minmax_get, NULL, NULL, NULL, NULL, NULL);


   /*
    *   history attributes
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_HISTORY,   NULL,
			       _history_get,   _history_set, 
			       _history_match, _history_copy, 
			       _history_query, _history_print);

   kdms_define_attribute(KDMS_OBJECT, _INTERNAL_HISTORY, 1, 1, KSTRING,
			 TRUE, TRUE, NULL);

   kdms_define_attribute(KDMS_OBJECT, KPDS_HISTORY_MODE, 1, 1, KINT,
			 FALSE, FALSE, KAPPEND_HISTORY);


   /*
    *   coupling
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_COUPLING, NULL,
			       _coupling_get,   _coupling_set, 
			       _coupling_match, _coupling_copy, 
			       _coupling_query, _coupling_print);

   /*
    *   data format attribute
    */
   kdms_define_quasi_attribute(NULL,KPDS_DATA_FORMAT,NULL,
                               _data_format_get,_data_format_set,
                               _data_format_match,_data_format_copy,
                               _data_format_query,_data_format_print);

   /* 
    *   location attributes
    */
   kdms_define_attribute(KDMS_OBJECT, KPDS_POINT_SIZE, 5, 1, KDOUBLE,
			 TRUE, TRUE, 1.0, 1.0, 1.0, 1.0, 1.0);

   kdms_define_attribute(KDMS_OBJECT, KPDS_KERNEL_ORIGIN, 5, 1, KINT,
			 TRUE, TRUE, 0, 0, 0, 0, 0);

   /* 
    *   subobject attribute
    */
   kdms_define_attribute(KDMS_OBJECT, KPDS_SUBOBJECT_POSITION, 5, 1, KINT,
			 TRUE, TRUE, 0, 0, 0, 0, 0);


   /* 
    *   elements first attribute
    */
   kdms_define_attribute(KDMS_OBJECT, KPDS_ELEMENTS_FIRST, 1, 1, KINT,
			 FALSE, FALSE, FALSE);

   return;
}
