 /*
  * Khoros: $Id: colorspace.c,v 1.4.2.6 1997/05/24 18:35:05 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: colorspace.c,v 1.4.2.6 1997/05/24 18:35:05 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for kcltrans
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lkcltrans
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include "colorspace.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lkcolorconvert - The routine lkcolorconvert() is used
*		to perform a colorspace conversion on an input object.
*		Currently, this routine is invoked when the KCOLOR_
*		COLORSPACE attribute is called, however, is setup
*		so that it can be called if needed.  Currently, this
*		routine enforces data ranges which correspond to the
*		converted colormodel.  This means that the output image
* 		may not be in the correct range for viewing.
* 
*       Purpose: This library routine controls which colorspace
*		 transformation functions would be called.  It accepts 
*		 input parameters from the kcltrans kroutine.
*
*         Input: kobject src_obj - This contains the input object.
*                kobject dest_obj - This contains the output image.
*		 int     src_color_model - This integer value represents
*				which colorspace the input object was
*				initially prior to conversion.
*                int     dest_color_model - This integer value represents
*				which colorspace is being converted to.
*
*        Output: Overwrites the input object.
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: Restrictions on data or input as applicable.
*    Written By: A. Christopher Moorman
*          Date: Jan 20, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lkcolorconvert(
   kobject src_obj,
   int src_color_model,
   int dest_color_model)
/* -library_def_end */

/* -library_code */
{
	int type=0;				/* Source obj. datatype.*/
	int w,h,d,t,e;				/* Dimensions of src obj*/
	int rw,rh,rd,rt,re;			/* Optimal region size. */
	int regions;				/* Number of regions.	*/
	int index;				/* Alternate data index.*/
	int i,j;				/* Loop indices.	*/
	int points;				/* Total data points.	*/
	int points_per_plane;			/* Total pointsperplane.*/
	double max_value;			/* Maximum data value.	*/
	double min_value;			/* Minimum data value.	*/
	double p1max, p2max, p3max;		/* Plane X's max value.	*/
	double p1min, p2min, p3min;		/* Plane X's min value.	*/
        char *lib = "kappserv";			/* KCALL library value.	*/
        char *rtn = "colorspace.c";		/* KCALL routine value.	*/
	kobject dest_obj;			/* Temp. output obj.	*/
	kobject src_ref;			/* Src Reference obj.	*/
	double *data=NULL;			/* Data array.		*/
	double *datatemp = NULL;		/* Temp data storage.	*/
	double *plane1 = NULL;			/* Data plane 1.	*/
	double *plane2 = NULL;			/* Data plane 2.	*/
	double *plane3 = NULL;			/* Data plane 3.	*/

               


       /* Check to see if value segment exists within the source object. 
	* If one does not, then the program is aborted.
	*/
        if(!kpds_query_value(src_obj))
        {
	   kset_errno(KINVALID_FILE);
           kerror(lib,rtn,"No value segment exists in the input object!\n");
           return (FALSE);
        }


       /* Check to see if the object contains any location data. Location
	* data is not supported at this time.
	*/
	if(kpds_query_location(src_obj))
	{
	   kerror(lib,rtn,"Location data is not supported.\n");
	   kset_errno(KINVALID_FILE);
           return (FALSE);
	}


       /* Check to see if MASK or TIME data exists in the input object.
	* At this time, this routine does not support the conversion of 
	* time or mask data.
	*/
	if(kpds_query_mask(src_obj) || kpds_query_time(src_obj))
	{
	   kerror(lib,rtn,"Mask and time data are not supported!\n");
	   kset_errno(KINVALID_FILE);
           return (FALSE);
	}


       /* Create a destination object to contain any manipulations.  Once
        * this is complete, the algorithm will replace the source object
        * with the destination object.
        */
        dest_obj = /* kpds_create_object(); */
	   kpds_open_output_object("/tmp/intermediate.kdf");

       /* Check to see if the destination object contains a value
	* segment.  If not, then one is created.
	*/
        if(!kpds_query_value(dest_obj))
           kpds_create_value(dest_obj);


       /* Make a copy of src object to destination object so attributes
        * and data are not lost.
        */
        if (!kpds_copy_object(src_obj, dest_obj))
        {
           kerror("lkcltrans","lkcltrans",
                  "Unable to copy attributes from input to output object.");
           return(FALSE);
        }

       /* Check to see of source objects color model is already the same 
	* as that which was specified by the user.  If they are the same 
	* then the program will just return control back to the calling
	* routine.
	*/
	if(src_color_model == 0) src_color_model = 1;
	if(dest_color_model == src_color_model) 
	   return (TRUE);


       /* Reference objects to avoid any side effects.
        */
        src_ref = kpds_reference_object(src_obj);


       /* Check to see if the input object is a grey image.  If so, then
	* initial adjustments will be done before the conversion can be
	* performed.
	*/
	if(src_color_model == KGREY || src_color_model == KNONE)
	{
	   if(kpds_query_map(src_obj))
	   {
	      kset_errno(KINVALID_FILE);
	      kinfo(KFORCE,"Unable to convert grey map data, Invalid Dims.!\n");
	      return(FALSE);
	   }

           if(!(kpds_get_attribute(src_obj, KPDS_VALUE_SIZE,&w,&h,&d,&t,&e)))
           {
	      kerror(lib,rtn,"Unable to get the value size attribute.\n");
	      return FALSE;
	   }
		
	   if(e < 3) 
	   {
	     /* Set the datatype on the src and destination object to ensure
	      * proper precision during conversion.
	      */
	      if(!(kpds_get_attribute(src_ref, KPDS_VALUE_DATA_TYPE, &type)))
              {
                 kerror(lib,rtn,"Unable to get the value data type on ref.\n");
                 return FALSE;
              }
	      if(!(kpds_set_attribute(src_ref, KPDS_VALUE_DATA_TYPE, KDOUBLE)))
              {
                 kerror(lib,rtn,"Unable to set the value data type on src1.\n");
                 return FALSE;
              }
	      if(!(kpds_set_attribute(dest_obj,KPDS_VALUE_DATA_TYPE, KDOUBLE)))
              {
                 kerror(lib,rtn,"Unable to set the datatype on destination.\n");
                 return FALSE;
              }


	     /* Get the optimal region size from the input object.
	      */
	      if(!(kpds_get_attribute(src_obj, KPDS_VALUE_OPTIMAL_REGION_SIZE,
		 &rw, &rh, &rd, &rt, &re, &regions)))
              {
                 kerror(lib,rtn,"Unable to get the optimal region size.\n");
                 return FALSE;
              }


             /* Set the optimal region size for both the input and output.
              */
              if(!(kpds_set_attribute(src_ref, KPDS_VALUE_REGION_SIZE,
                 rw,rh,rd,rt,re)))
              {
                 kerror(lib,rtn,"Unable to set the value region size.\n");
                 return FALSE;
              }
              if(!(kpds_set_attribute(dest_obj, KPDS_VALUE_REGION_SIZE,
                 rw,rh,rd,rt,re)))
              {
                 kerror(lib,rtn,"Unable to set the value region size.\n");
                 return FALSE;
              }



             /* Initialize the data array, and determine the number of total
              * points, and total points per element plane.
              */
              data = NULL;
              index = rw * rh * rd * rt;
              points = rw * rh * rd * rt * re;

	     /* Temporary storage is needed due to the fact that the original
	      * image is expanded in size and the values missing are created.
	      */
              datatemp = (double *) kcalloc(1, (sizeof(double) * points * 3));

              for(i=0;i<regions;i++)
              {
                 data = (double *) kpds_get_data(src_ref, KPDS_VALUE_REGION,
                   (kaddr)data);

		 for(j=0;j<points*3;j++)
		 {
		    if(j < points) 
			datatemp[j] = data[j];
		    else datatemp[j] = 0.0;
		 }
	      }

	     /* Set the final attributes and write out the data to the 
	      * output object.  This ensures that the grey image has
	      * the proper dimensions to be converted.
	      */
	      if(!(kpds_set_attribute(dest_obj, KPDS_VALUE_SIZE, w,h,d,t,3)))
              {
                 kerror(lib,rtn,"Unable to set the value size on dest. obj.\n");
                 return FALSE;
              }
              if(!(kpds_set_attribute(dest_obj, KPDS_VALUE_REGION_SIZE,
                 rw,rh,rd,rt,3)))
              {
                 kerror(lib,rtn,"Unable to set the value region size.\n");
                 return FALSE;
              }
	      if(!(kpds_put_data(dest_obj, KPDS_VALUE_REGION, datatemp)))
              {
                 kerror(lib,rtn,"Unable to put data to destination.\n");
                 return FALSE;
              }
	      if(!(kpds_set_attribute(dest_obj, KPDS_VALUE_DATA_TYPE, type)))
              {
                 kerror(lib,rtn,"Unable to set the data type on dest. obj.\n");
                 return FALSE;
              }
	      if(!(kpds_copy_object(dest_obj, src_obj)))
              {
                 kerror(lib,rtn,"Unable to copy the source to destination.\n");
                 return FALSE;
              }
	      kfree(datatemp);
	   }
	}

       /* Check to see if a map exists on the image.  If so, then the
	* conversions are performed on the map, rather than the value
	* data.  This will provide improved performance while maintaining
	* data compression.
	*/
	if(kpds_query_map(src_obj))
	{
          /* Get the size of the data type after the mapping was done.
           */
           if(!(kpds_get_attribute(src_obj,KPDS_MAP_SIZE, &w, &h, &d, &t, &e)))
           {
              kerror(lib,rtn,"Unable to get the map size from source obj.\n");
              return FALSE;
           }

	   if(!(kpds_get_attribute(src_ref, KPDS_MAP_DATA_TYPE, &type)))
           {
              kerror(lib,rtn,"Unable to get the map data type from src.\n");
              return FALSE;
           }


 	  /* Modify the datatypes of the source image so that precision
	   * can be maintained.
	   */
           if(!(kpds_set_attribute(src_ref,KPDS_MAP_DATA_TYPE, KDOUBLE)))
           {
              kerror(lib,rtn,"Unable to set the map data type on srcref.\n");
              return FALSE;
           }
           if(!(kpds_set_attribute(dest_obj,KPDS_MAP_DATA_TYPE, KDOUBLE)))
           {
              kerror(lib,rtn,"Unable to set the map data type on dest obj.\n");
              return FALSE;
           }



          /* Get the optimal data region size from the input object.
           */
           if(!(kpds_get_attribute(src_obj, KPDS_MAP_OPTIMAL_REGION_SIZE,
              &rw, &rh, &rd, &rt, &re, &regions)))
           {
              kerror(lib,rtn,"Unable to set the map optimal region size.\n");
              return FALSE;
           }


          /* Set the optimal region size for both the input and output.
           */
           if(!(kpds_set_attribute(src_ref, KPDS_MAP_REGION_SIZE,
                rw,rh,rd,rt,re)))
           {
              kerror(lib,rtn,"Unable to set the map region size on srcref.\n");
              return FALSE;
           }
           if(!(kpds_set_attribute(dest_obj, KPDS_MAP_REGION_SIZE,
                rw,rh,rd,rt,re)))
           {
              kerror(lib,rtn,"Unable to set the map region size on dest.\n");
              return FALSE;
           }



	  /* Set up the data array so that it can contain the rgb data.
	   */
           data = NULL;

	  /* Obtain the number of points based on the optimal region size
	   * obtained in a previous section of code.
	   */
           points = rw * rh * rd * rt * re;
	   index = points_per_plane = rh;

	  /* create arrays to hold the R,G,B values respectively.  Each
	   * value will represent a plane (element dims) of data.
	   */
           plane1 = (double *) kmalloc(points_per_plane * sizeof(double));
           plane2 = (double *) kmalloc(points_per_plane * sizeof(double));
           plane3 = (double *) kmalloc(points_per_plane * sizeof(double));

           for (i=0;i<regions;i++)
           {
              data = (double *) kpds_get_data(src_ref,KPDS_MAP_REGION, data);

             /* Obtain the min and max data values so that scaling and data
              * offset can be performaned.
              */
              if(i == 0) 
	      {
		 max_value = data[0];
		 min_value = data[0];
	         p1max = p2max = p3max = data[0];
                 p1min = p2min = p3min = data[0];
	      }

              for(j=0;j<points-1;j++)
	      {
                 if(max_value < data[j]) max_value = data[j];
                 if(min_value > data[i]) min_value = data[i];
	      }

	      for(j=0;j<points_per_plane;j++)
	      {
		 plane1[j] = data[3*j];
		 plane2[j] = data[3*j+1];
		 plane3[j] = data[3*j+2];
	      }
	      
              for(j=0;j<points_per_plane;j++)
              {
                 if(p1max < plane1[j]) p1max = plane1[j];
                 if(p2max < plane2[j]) p2max = plane2[j];
                 if(p3max < plane3[j]) p3max = plane3[j];
                 if(p1min > plane1[j]) p1min = plane1[j];
                 if(p2min > plane2[j]) p2min = plane2[j];
                 if(p3min > plane3[j]) p3min = plane3[j];
              }
           }

	}

       /* This section will perform the conversion of value data.  This
	* only happens when map data is not present.
	*/
	else
	{
          /* Get the dimensions and data type of the input object.
           */
           if(!(kpds_get_attribute(src_obj, KPDS_VALUE_SIZE,&w,&h,&d,&t,&e)))
           {
              kerror(lib,rtn,"Unable to get the value size on src obj.\n");
              return FALSE;
           }
	   if(!(kpds_get_attribute(src_ref, KPDS_VALUE_DATA_TYPE, &type)))
           {
              kerror(lib,rtn,"Unable to set the value data type.\n");
              return FALSE;
           }
           if(!(kpds_set_attribute(dest_obj, KPDS_VALUE_SIZE,w,h,d,t,e,
		KPDS_VALUE_DATA_TYPE, type)))
           {
              kerror(lib,rtn,"Unable to set the value size on dest. object.\n");
              return FALSE;
           }


          /* Modify input attributes so that data types are set to double
           * for processing.
           */
           if(!(kpds_set_attribute(src_ref,KPDS_VALUE_DATA_TYPE, KDOUBLE)))
           {
              kerror(lib,rtn,"Unable to set the source value data type.\n");
              return FALSE;
           }
           if(!(kpds_set_attribute(dest_obj,KPDS_VALUE_DATA_TYPE, KDOUBLE)))
           {
              kerror(lib,rtn,"Unable to set the destination map data type.\n");
              return FALSE;
           }


          /* Set the KPDS_VALUE_OPTIMAL_REGION_CONSTRAINT so that the
           * optimal region size always divides the data using all of the
           * elements rather than down width and height.  It is setup
           * so that there is a constraint on the element size, and that
           * all other dimensions may be adjusted to allow for a new
           * optimal region size to be obtained.
           */
          if(!(kpds_set_attribute(src_obj,KPDS_VALUE_OPTIMAL_REGION_CONSTRAINT,
              1, 1, 1, 1, e)))
           {
              kerror(lib,rtn,"Unable to set the optimal region size.\n");
              return FALSE;
           }
           if(!(kpds_set_attribute(src_obj, KPDS_VALUE_OPTIMAL_REGION_FLAGS,
              KOPTIMAL_FACTOR, KOPTIMAL_FACTOR, KOPTIMAL_FACTOR,
              KOPTIMAL_FACTOR, KOPTIMAL_FIXED_VALUE)))
           {
              kerror(lib,rtn,"Unable to set the optimal region.\n");
              return FALSE;
           }


          /* Get the optimal data region size from the input object.
           */
           if(!(kpds_get_attribute(src_obj, KPDS_VALUE_OPTIMAL_REGION_SIZE,
              &rw, &rh, &rd, &rt, &re, &regions)))
           {
              kerror(lib,rtn,"Unable to set the optimal region size.\n");
              return FALSE;
           }

          /* Set the optimal region size for both the input and output.
           */
           if(!(kpds_set_attribute(src_ref, KPDS_VALUE_REGION_SIZE,
               rw,rh,rd,rt,re)))
           {
              kerror(lib,rtn,"Unable to set the value region size.\n");
              return FALSE;
           }
           if(!(kpds_set_attribute(dest_obj, KPDS_VALUE_REGION_SIZE,
               rw,rh,rd,rt,re)))
           {
              kerror(lib,rtn,"Unable to set the value region size.\n");
              return FALSE;
           }



          /* Initialize the data array, and determine the number of total 
	   * points, and total points per element plane.
	   */
	   data = NULL;
       	   index = rw * rh * rd * rt;
	   points = rw * rh * rd * rt * re;


          /* This section is needed to determine the min and max values of the
	   * dataset.  It also finds the min and max for each element plane.
	   * This is needed for various conversion routines.  It must be
	   * the max of the entire dataset, not for each region of data that
	   * is read in.
	   */
	   for(j=0;j<regions;j++)
	   {
	      data = (double *) kpds_get_data(src_ref, KPDS_VALUE_REGION,
		   (kaddr)data);

	     /* Set the initial values for each of the min and max variables.
	      */
	      if(j == 0) 
	      {
	         max_value = data[0];
	         min_value = data[0];
	         p1max = p2max = p3max = data[0];
	         p1min = p2min = p3min = data[0];
	      }
	      for(i=0;i<points;i++)
	      {
	         if(max_value < data[i]) max_value = data[i];
	         if(min_value > data[i]) min_value = data[i];
	      }

	     /* Set the pointers of each plane to the begin of each element
	      * plane.  This is needed to find the max values of each plane.
	      */
	      plane1 = data;
	      plane2 = data + index;
	      plane3 = data + 2*index;
	      for(i=0;i<index;i++)
	      {
	         if(p1max < plane1[i]) p1max = plane1[i];
	         if(p2max < plane2[i]) p2max = plane2[i];
	         if(p3max < plane3[i]) p3max = plane3[i];
	         if(p1min > plane1[i]) p1min = plane1[i];
	         if(p2min > plane2[i]) p2min = plane2[i];
	         if(p3min > plane3[i]) p3min = plane3[i];
	      }
	   }
	} 


       /* This section performs the actual conversions on the data,
	* as well as controlling order of multiple conversions.
	*/
	for(j=0;j<regions;j++)
	{
	   if(kpds_query_map(src_obj))
	   {
              data = (double *) kpds_get_data(src_ref,KPDS_MAP_REGION, data);

	      for(i=0;i<points_per_plane;i++)
	      {
		 plane1[i] = data[3*i];
		 plane2[i] = data[3*i+1];
		 plane3[i] = data[3*i+2];
	      }
	   }
	   else
	   {
	      data = (double *) kpds_get_data(src_ref, KPDS_VALUE_REGION,
		   (kaddr)data);

	      plane1 = data;
	      plane2 = data + index;
	      plane3 = data + 2*index;
	   }

          /* Check to see if the input image is not an RGB image.  If it is  
	   * not an RGB image, convert it to RGB first.  Otherwise, skip this
	   * section, and perform an RGB to X colorspace conversion switch 
	   * section.
	   */
	   if(src_color_model != KRGB)
	   {
	      switch(src_color_model)
	      {
	         case KNONE:
		    greyrgb(plane1, plane2, plane3, index, max_value);
		    break;

	         case KGREY:
		    greyrgb(plane1, plane2, plane3, index, max_value);
		    break;

	         case KRGB:
		    break;

	         case KCMY:
		    cmytorgb(plane1, plane2, plane3, index, max_value);
		    break;

	         case KYIQ:
		    yiqtorgb(plane1, plane2, plane3, index, max_value);
		    break;

	         case KHSV:
		    hsvtorgb(plane1, plane2, plane3, index, p1max, p1min,
			p2max, p2min, p3max,p3min);
		    break;

	         case KHLS:
		    hlstorgb(plane1, plane2, plane3, index, p1max, p1min,
			p2max, p2min, p3max, p3min);
		    break;

	         case KIHS:
		    ihstorgb(plane1, plane2, plane3, index);
		    break;

	         case KXYZ:
		    xyztorgb(plane1, plane2, plane3, index);
		    break;

	         case KUVW:
		    uvwtorgb(plane1, plane2, plane3, index);
		    break;

	         case KUCSUVW:
		    ucsuvwtorgb(plane1, plane2, plane3, index);
		    break;

	         case KUCSSOW:
		    ucssowtorgb(plane1, plane2, plane3, index);
		    break;

	         case KUCSLab:
		    ucslabtorgb(plane1, plane2, plane3, index);
		    break;

	         case KUCSLuv:
		    ucsluvtorgb(plane1, plane2, plane3, index);
		    break;

	         case KUSERDEFINED:
	 	    break;

	         default:
		    kset_errno(KINVALID_FILE);
           	    kerror(lib,rtn,"Invalid Colorspace in image!\n");
		    break;
	      }
	   }

	  /* This switch section performs the conversion from RGB images to 
	   * the specified destination colormodel.  If the original image   
	   * was an RGB image, the top switch section was skipped, and      
	   * immediately comes down here.                                   
	   */
           switch(dest_color_model)
	   {
	      case KGREY:	/* RGB to Greyscale */
	         rgbtogrey(plane1, plane2, plane3, index);

                /* For storage reasons, the rest of the element vectors are
		 * cutoff.
	         */
	         if(!(kpds_set_attribute(src_obj, KPDS_VALUE_SIZE, 
			w, h, d, t, 1)))
                 {
                    kerror(lib,rtn,"Unable to set the value size.\n");
                    return FALSE;
                 }
	         break;

	      case KRGB:	/* RGB to RGB Colorspace, No conversion */
	         break;

	      case KCMY:	/* RGB to CMY Colorspace */
	         rgbtocmy(plane1, plane2, plane3, index, max_value);
	         break;

	      case KYIQ:	/* RGB to YIQ Colorspace */
	         rgbtoyiq(plane1,plane2,plane3,index,max_value);
	         break;

	      case KHSV:	/* RGB to HSV Colorspace */	
	         rgbtohsv(plane1,plane2,plane3,index,max_value,min_value);
	         break;

	      case KHLS:	/* RGB to HLS Colorspace */
	         rgbtohls(plane1,plane2,plane3,index,max_value);
	         break;

	      case KIHS:	/* RGB to IHS Colorspace */
	         rgbtoihs(plane1, plane2, plane3, index, max_value);
	         break;

	      case KXYZ:	/* RGB to XYZ Colorspace */
	         rgbtoxyz(plane1, plane2, plane3, index);
	         break;

	      case KUVW:	/* RGB to UVW Colorspace */
	         rgbtouvw(plane1, plane2, plane3, index);
	         break;

	      case KUCSUVW:	/* RGB to UCSUVW Colorspace */
	         rgbtoucsuvw(plane1, plane2, plane3, index);
	         break;

	      case KUCSSOW:	/* RGB to UCSSOW Colorspace */
	         rgbtoucssow(plane1, plane2, plane3, index);
	         break;

	      case KUCSLab:	/* RGB to UCSLab Colorspace */
	         rgbtoucslab(plane1, plane2, plane3, index);
	         break;

              case KUCSLuv:	/* RGB to UCSLuv Colorspace */
	         rgbtoucsluv(plane1, plane2, plane3, index);
	         break;

	      default:	/* User Defined Colorspace */
		 kset_errno(KINVALID_FILE);
           	 kerror(lib,rtn,"Invalid Colorspace in image!\n");
	         break;
	   }


	  /* This section will modify the output image to ensure proper
	   * datatype, data, and size are enforced.
	   */
	   if(kpds_query_map(src_obj))
	   {
	      for(i=0;i<points_per_plane;i++)
	      {
	         data[3*i] = plane1[i];
	         data[3*i+1] = plane2[i];
	         data[3*i+2] = plane3[i];
	      }
              kpds_put_data(dest_obj,KPDS_MAP_REGION, data);
	      kpds_set_attribute(dest_obj, KPDS_MAP_DATA_TYPE,type);
	      kfree(plane1);
	      kfree(plane2);
	      kfree(plane3);
	   }
	   else
	   {
              kpds_put_data(dest_obj,KPDS_VALUE_REGION, data);
	      kpds_set_attribute(dest_obj, KPDS_VALUE_DATA_TYPE,type);
	   }



	}	/* End of region count loop. */


       /* Performing the Cleanup duties 
	*/
	kpds_copy_object(dest_obj, src_obj);
	kpds_close_object(src_obj);
	kpds_close_object(dest_obj);
	return TRUE;
}











/*-----------------------------------------------------------
|
|  Routine Name: function_name - short description of function
|
|       Purpose: This files contains a series of functions which are used
|		 for the colorspace transformations.  These functions 
|		 are used by the kctrans kroutine and lkcltrans library
|		 routine.  A list of the conversions are shown below:
|
|		 Conversions:
|
|			RGB to Greyscale
|			RGB to CMY
|			RGB to YIQ
|			RGB to HSV 
|			RGB to HSL
|			RGB to IHS
|			RGB to XYZ
|			RGB to UVW
|			RGB to UCSUVW
|			RGB to UCSSOW
|			RGB to UCSLab
|			RGB to UCSLub
|			Greyscale to RGB
|			CMY to RGB
|			YIQ to RGB
|			HSV to RGB
|			HLS to RGB
|			IHS to RGB
|			XYZ to RGB
|			UVW to RGB 
|			UCSUVW to RGB
|			UCSSOW to RGB
|			UCSLab to RGB
|			UCSLuv to RGB
|
|
|         Input: kobject src - Kobject which represents the input object.
|                kobject dest - Kobject which represents the output object.
|                int outtype - This value represents the type of colorspace
|		 	       to be applied to the input object.
|
|        Output: argument4 - explanation
|                argument5 - explanation
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: A. Christopher Moorman
|          Date: September 22, 1996
| Modifications:
|
------------------------------------------------------------*/



/* Xcolorspace to RGB conversion routines. */


/* GREYSCALE TO RGB colerspace conversion routine. 
 * This routine performs the color transformation on a grey level image.  
 * It uses a set of generated transformation functions to generate the    
 * RGB values from a grey scale.  Each function is generated from the same
 * equation, however, each function is phase shifted to slightly enhance  
 * certain features.                                                      
 */

int greyrgb(
double *r,
double *b,
double *g,
int points_per_plane,
double max_value)
{

        int j;			/* Loop index variable. 	*/
	int i;
	

       /* This routine performs color transformations on a grey level   
	* image through the usage of generated trasformation functions. 
	* The R,G,B values are each generated from three different      
	* functions, each slightly phase shifted to enhance features.   
	*/
       /* DATA PROCESSING SECTION:
	* The grey level to color transformation used for this routine 
	* is described in the following reference:                     
	* Section 4.6 Gray Level to Color Transformations Pg 239 of    
	* Digital Image Processing, Gonzalez and Woods.                
	*/
	if(max_value == 0) max_value = 0.1;
        for(j=0;j<points_per_plane;j++)
     	{
	   i = (int) r[j];
	   r[j] = kabs(max_value*(sin((i/(max_value/7.3)) - 
	      (0.1*max_value/max_value))));
	   g[j] = kabs(max_value*(sin(((i+1)/(max_value/7.3)) - 
	      (0.3*max_value/max_value))));
	   b[j] = kabs(max_value*(sin(((i+2)/(max_value/7.3)) -
	      (0.8*max_value/max_value))));
	}
	return(TRUE);
}






/* CMY TO RGB colerspace conversion routine. 
 * This routine performs the color transformation on a CMY color image.   
 * It uses the transformation matrix which is representative of the CMY   
 * to RGB converstion.                                                    
 * This transformation matrix was obtained from Digital Image Processing 
 * by Gonzales, on page 228, section 4.6.2 Color Models.                  
 */

int cmytorgb(
double *c,
double *m,
double *y,
int points_per_plane,
double max_value)
{
       						/* Variable Dec:	*/
        int i;					/* Looping index var.	*/
	

       /* This section processes the input data.                       
	* The method used for processing comes from "Digital Image    
	* Processing" by Gonzales, on page 238.   This section is      
	* entitled, "the CMY color model".                             
	* 		[R  = [1	-   [C				   
	*		 G     1	     M    			   
	*		 B]    1]            Y]				 
	*/                                                             
 
       /* The dataset is now converted from the CMY to RGB space. It is
        * important to note that the data is being reference by element
	* first, rather than by R,R....G,G,....B,B... order.
        */
        for(i=0;i<points_per_plane;i++)
        {
           c[i] = c[i] / max_value;
           c[i] = 1.0 - c[i];
 
           m[i] = m[i] / max_value;
           m[i] = 1.0 - m[i];

           y[i] = y[i] / max_value;
           y[i] = 1.0 - y[i];
        }
	return(TRUE);
}





/* This routine performs the YIQ to RGB transformation on an input image.
 * Information on the conversion from the YIQ to RGB Colorspace can be 
 * found in "Digital Image Processing" by Gonzalez and "Fundamentals of	
 * Digital Image Processing" by Jain.
 */

int yiqtorgb(
double *Y,
double *I,
double *Q,
int points_per_plane,
double max_value)
{
        int j;	 				/* Looping index.	*/
	double R,G,B;				/* Temp R,G,B output.	*/


       /* The data is transformed from the YIQ colorspace to the RGB
        * colorspace. This process was obtained from the "Digital
	* Image Processing" book by Gonzalez and the "Fundamentals of
	* Digital Image Processing" by Jain.
        */

       /* The dataset is now converted from YIQ to the RGB Colorspace.
	* The data is manipulated via Y,I,Q pointers.  This allows access
	* to each value individually, without the need to access every data
	* element.
	*/
        for(j=0;j<points_per_plane;j++)
        {
	   if(max_value > 1.0)
	   {
	      Y[j] = Y[j] / max_value;
	      I[j] = I[j] / max_value;
	      Q[j] = Q[j] / max_value;
	   }

           R = 1.0000*Y[j] + 0.9557*I[j] + 0.6199*Q[j];
           G = 1.0000*Y[j] - 0.2716*I[j] - 0.6469*Q[j];
           B = 1.0000*Y[j] - 1.1082*I[j] + 1.7051*Q[j];

           Y[j] = R;
           I[j] = G;
           Q[j] = B;
        }
	return(TRUE);
}







/* This routine performs the HSV to RGB colorspace transformation.  
 * Information on this transformation can be found in "Computer Graphics,
 * Principles and Practice" by Foley, Van Dam, Feiner, and Hughes.
 */

int hsvtorgb(
double *Hue,
double *Sat,
double *Val,
int points_per_plane,
double hmax,
double hmin,
double smax,
double smin,
double vmax,
double vmin)
{
        int j;					/* Looping index var.	*/
	int floor_val=0;			/* floor value of hue.	*/
	double ff=0;				/* Fraction part of hue.*/
	double ptemp=0;
	double q=0;
	double tt=0;				/* Conversion values.	*/
	double r,g,b;				/* Temp R,G,B cals vals	*/
	double hue_value;			/* Temp Hue Value.	*/
	double sat_value;			/* Saturation Value.	*/
	double val_value;			/* Value data.		*/

	double huedesiredrange;			/* Desired Hue range.	*/
	double huerange;			/* Actual hue range.	*/
	double huemfactor=1.0;			/* Hue Scale factor.	*/
	double huesfactor=0.0;			/* Hue offset factor.	*/
	double sdesiredrange;			/* Desired Sat range.	*/
	double srange;				/* Actual Sat range.	*/
	double smfactor=1.0;			/* Sat Scale factor.	*/
	double ssfactor=0.0;			/* Sat offset factor.	*/
	double vdesiredrange;			/* Desired Value range.	*/
	double vrange;				/* Actual Value range.	*/
	double vmfactor=1.0;			/* Value Scale factor.	*/
	double vsfactor=0.0;			/* Value offset factor.	*/


       /* The data is transformed from the HSV colorspace to the RGB
        * colorspace. This process was obtained from "Computer Graphics,
	* Principles and Practice" by Foley, Van Dam, Feiner, and Hughes,
	* on page 592.
        */
        if(hmax > 361.0 || hmin < 0)
        {
           kinfo(KVERBOSE,"The input file is of an incorrect format!\n");
           kinfo(KVERBOSE,"Hue values must be between 0 and 360 degrees!\n");
           kinfo(KVERBOSE,"Attempting to scale data.\n");
           kinfo(KVERBOSE,"NOTE: This could validate invalid hue data.\n");

           huerange = hmax - hmin;
           huedesiredrange = 361.0 - 0.0;
           huemfactor = huedesiredrange/huerange;
           huesfactor = 0.0 - (hmin*huemfactor);
        }
        if(smax > 1.0 || smin < 0.0)
        {
           kinfo(KVERBOSE,"The input file is of an incorrect format!\n");
           kinfo(KVERBOSE,"Saturation values must be between 0 and 1!\n");
           kinfo(KVERBOSE,"Attempting to scale data.\n");

           srange = smax - smin;
           sdesiredrange = 1.0 - 0.0;
           smfactor = sdesiredrange/srange;
           ssfactor = 0.0 - (smin*smfactor);
        }
        if(vmax > 1.0 || vmin < 0.0)
        {
           kinfo(KVERBOSE,"The input file is of an incorrect format!\n");
           kinfo(KVERBOSE,"VALUE values must be between 0 and 1!\n");
           kinfo(KVERBOSE,"Attempting to scale data.\n");

           vrange = vmax - vmin;
           vdesiredrange = 1.0 - 0.0;
           vmfactor = vdesiredrange/vrange;
           vsfactor = 0.0 - (vmin*vmfactor);
        }

       /* The dataset is now converted from RGB to the HSV Colorspace.
	* The data is manipulated via R,G,B pointers.  This allows access 
	* to each value individually.
	*/
        for(j=0;j<points_per_plane;j++)
        {
	   hue_value = (Hue[j] * huemfactor) + huesfactor;
	   sat_value = (Sat[j] * smfactor) + ssfactor;
	   val_value = (Val[j] * vmfactor) + vsfactor;


           if(sat_value == 0)
           {
              if(hue_value > 360.0)              /* UNDEFINED */
              {
                 r = val_value;
                 g = val_value;
                 b = val_value;
              }
              else
              {
                 kinfo(KFORCE,"ERROR: Sat = 0, but hue has a value!\n");
                    kerror("lkcltrans","function.c",
                       "Invalid data, Saturation=0, and hue is valid data.");
              }
           }
           else
           {
              if(hue_value == 360.0) hue_value = 0.0;
              hue_value = hue_value / 60.0;
	      floor_val = (int) hue_value;
              ff = hue_value - floor_val;
              ptemp = val_value * (1.0 - sat_value);
              q = val_value * (1.0 - (sat_value * ff));
              tt = val_value * (1.0 - (sat_value * (1.0 - ff)));
             switch(floor_val)
              {
                 case 0:
                    r = val_value;
                    g = tt;
                    b = ptemp;
                    break;
                 case 1:
                    r = q;
                    g = val_value;
                    b = ptemp;
                    break;
                 case 2:
                    r = ptemp;
                    g = val_value;
                    b = tt;
                    break;
                 case 3:
                    r = ptemp;
                    g = q;
                    b = val_value;
                    break;
                 case 4:
                    r = tt;
                    g = ptemp;
                    b = val_value;
                    break;
                 case 5:
                    r = val_value;
                    g = ptemp;
                    b = q;
                    break;
              }
           }

           Hue[j] = r;
           Sat[j] = g;
           Val[j] = b;
        }
	return(TRUE);
}








int hlstorgb(
double *H,
double *L,
double *S,
int points_per_plane,
double hmax,
double hmin,
double lmax,
double lmin,
double smax,
double smin)
{
        char *lib = "kappserv";			/* KCALL library value.	*/
        char *rtn = "colorspace.c";		/* KCALL routine value.	*/

        int j;					/* Looping index var.	*/
	double hue, sat, light;  		/* Vector H,L,S values. */
	double hue_temp;			/* Modified Hue values.	*/
	double m1,m2;				/* Temporary storage.	*/
	double R,G,B;				/* Temp R,G,B cals vals	*/

	double huedesiredrange;			/* Desired Hue range.	*/
	double huerange;			/* Actual hue range.	*/
	double huemfactor=1.0;			/* Hue Scale factor.	*/
	double huesfactor=0.0;			/* Hue offset factor.	*/
	double sdesiredrange;			/* Desired Sat range.	*/
	double srange;				/* Actual Sat range.	*/
	double smfactor=1.0;			/* Sat Scale factor.	*/
	double ssfactor=0.0;			/* Sat offset factor.	*/
	double ldesiredrange;			/* Desired Lighness Val.*/
	double lrange;				/* Actual Light range.	*/
	double lmfactor=1.0;			/* Light. Scale factor.	*/
	double lsfactor=0.0;			/* Light. offset factor	*/

       /* The data is transformed from the HLS colorspace to the RGB
        * colorspace. This process was obtained from the "Computer Graphics,
	* Principles and Practice", by Foley, Van Damm, Feiner, and Hughes.
        */

       /* It is important to check the input data to ensure that it is
        * a valid data file.  The Hue data should be in the range [0,360].
        * The Saturation and Lightness values should be in the range of
        * [0,1].  If this is not the case, this routine will modify, ie  
	* scale the data so that it is in the proper range.  It is 
	* important to note that this could result in a loss of information.
	* If a user specifies an undefined value for the hue, the scaling 
	* will just assume that it is part of the original data, and treat
	* them as valid data.
        */
	if(hmax > 361.0 || hmin < 0)
        {
           kinfo(KVERBOSE,"The input file is of an incorrect format!\n");
           kinfo(KVERBOSE,"Hue values must be between 0 and 360 degrees!\n");
           kinfo(KVERBOSE,"Attempting to scale data.\n");
           kinfo(KVERBOSE,"NOTE: This could validate invalid hue data.\n");

           huerange = hmax - hmin;
           huedesiredrange = 361.0 - 0.0;
           huemfactor = huedesiredrange/huerange;
           huesfactor = 0.0 - (hmin*huemfactor);
        }
        if(smax > 1.0 || smin < 0.0)
        {
           kinfo(KVERBOSE,"The input file is of an incorrect format!\n");
           kinfo(KVERBOSE,"Saturation values must be between 0 and 1!\n");
           kinfo(KVERBOSE,"Attempting to scale data.\n");

           srange = smax - smin;
           sdesiredrange = 1.0 - 0.0;
           smfactor = sdesiredrange/srange;
           ssfactor = 0.0 - (smin*smfactor);
        }
        if(lmax > 1.0 || lmin < 0.0)
        {
           kinfo(KVERBOSE,"The input file is of an incorrect format!\n");
           kinfo(KVERBOSE,"LIGHTNESS values must be between 0 and 1!\n");
           kinfo(KVERBOSE,"Attempting to scale data.\n");

           lrange = lmax - lmin;
           ldesiredrange = 1.0 - 0.0;
           lmfactor = ldesiredrange/lrange;
           lsfactor = 0.0 - (lmin*lmfactor);
        }


       /* The dataset is now converted from HLS to the RGB Colorspace.
	* A pointer to the H,L, and S values make data manipulation 
	* fairly straight forward.
	*/
        for(j=0;j<points_per_plane;j++)
        {
	   hue = (H[j] * huemfactor) + huesfactor;
	   light = (L[j] * lmfactor) + lsfactor;
	   sat = (S[j] * smfactor) + ssfactor;


           if(light <= 0.5)
              m2 = light*(light + sat);
           else
           {
              m2 = 1 + sat - light * sat;
	   }
           m1 = 2 * light - m2;

           if(sat == 0)
           {
              if(hue == HUE_UNDEFINED)
                 R = G = B = light;
              else
                 kerror(lib,rtn,"ERROR: Sat=0 and Hue is defined!!\n");
           }
           else
           {
                                              /* The Red color */
              hue_temp = hue + 120.0;
              if(hue_temp > 360.0) hue_temp = hue_temp - 360.0;
              if(hue_temp < 0.0) hue_temp = hue_temp + 360.0;
              if(hue_temp > 60.0) R = m1 + (m2 - m1) * hue_temp/60.0;
              else if (hue_temp < 180.0) R = m2;
              else if (hue_temp<240.0) R = m1+(m2-1)*(240-hue_temp)/60.0;
              else R = m1;

                                                /* The Green color */
              hue_temp = hue;
              if(hue_temp > 360.0) hue_temp = hue_temp - 360.0;
              if(hue_temp < 0.0) hue_temp = hue_temp + 360.0;
              if(hue_temp > 60.0) G = m1 + (m2 - m1) * hue_temp/60.0;
              else if (hue_temp < 180.0) G = m2;
              else if (hue_temp<240.0) G = m1+(m2-1)*(240-hue_temp)/60.0;
              else G = m1;


                                                /* The Blue color */
              hue_temp = hue + 120.0;
              if(hue_temp > 360.0) hue_temp = hue_temp - 360.0;
              if(hue_temp < 0.0) hue_temp = hue_temp + 360.0;
              if(hue_temp > 60.0) B = m1 + (m2 - m1) * hue_temp/60.0;
              else if (hue_temp < 180.0) B = m2;
              else if (hue_temp<240.0) B = m1+(m2-1)*(240-hue_temp)/60.0;
              else B = m1;
           }
           H[j] = R;
           L[j] = G;
           S[j] = B;
        }
	return(TRUE);
}








/* This routine performs the transformation from the IHS Colorspace model
 * to the RGB Colorspace model.  Information on this transformation can be
 * found in "Fundamentals of Digital Image Processing" by Jain. 
 */

int ihstorgb(
double *I,
double *H,
double *S,
int points_per_plane)
{
        int j;					/* Looping Index vars.	*/
	double r,g,b;				/* Temp RGB values.	*/
	double hue;				/* Temp I,H,S values.	*/

       /* The data is transformed from the IHS colorspace to the RGB
        * colorspace. This process was obtained from the "Digital
        * Image Processing" book by Gonzalez and the "Fundamentals of
        * Digital Image Processing" by Jain.  Specific information for this
	* routine was found in Gonzales on page 236.
        */
        for(j=0;j<points_per_plane;j++)
        {
	  /* Calculate values for RG sector.	
	   */
	   if( H[j] > 0.0 && H[j] <= 120.0) 
	   {
	      hue = H[j];
	      b = (1.0/3.0) * (1.0 - S[j]);
	      r = (1.0/3.0) * (1.0 + ((S[j] * cos(H[j]))/(cos(60.0 - H[j]))));
	      g = 1.0 - (r + b);
	   }

	  /* Calculate values for GB Sector.
	   */
	   else if( H[j] > 120.0 && H[j] <= 240.0) 
	   {
	      hue = H[j] - 120.0;
	      r = (1.0/3.0) * (1.0 - S[j]);
	      g = (1.0/3.0) * (1.0 + ((S[j] * cos(hue))/(cos(60.0 - hue))));
	      b = 1.0 - (r + g);
	   }

	  /* Calculate values for BR Sector.
	   */
	   else if( H[j] > 240.0 && H[j] <= 360.0)  
	   {
	      hue = H[j] - 240.0;
	      g = (1.0/3.0) * (1.0 - S[j]);
	      b = (1.0/3.0) * (1.0 + ((S[j] * cos(hue))/(cos(60.0 - hue))));
	      r = 1.0 - (g + b);
	   }
	   else 
	   {
	      kinfo(KFORCE,"Invalid range of data, H is HUE_UNDEFINED!\n");
	      I[j] = 0.0;
	      H[j] = 0.0;
	      S[j] = 0.0;
	      continue;
	   }

           I[j] = r;
           H[j] = g;
           S[j] = b;
        }
	return(TRUE);
}





/* This routine performs the transformation from the XYZ colorspace to the 
 * RGB colorspace.  Information on this trasformation can be found in the
 * "Fundamentals of Digital Image Processing" by Jain.
 */

int xyztorgb(
double *X,
double *Y,
double *Z,
int points_per_plane)
{
        int j;					/* Looping index varbles*/
	double R,G,B;				/* Temp R,G,B values.	*/


       /* The data is transformed from the XYZ colorspace to the RGB
        * colorspace. This process was obtained from the "Digital
        * Image Processing" book by Gonzalez and the "Fundamentals of
        * Digital Image Processing" by Jain.
        */
        for(j=0;j<points_per_plane;j++)
        {
           R = 2.3644*X[j] - 0.8958*Y[j] - 0.4677*Z[j];
           G = -0.5148*X[j] + 1.4252*Y[j] + 0.0882*Z[j];
           B = 0.0052*X[j] - 0.0144*Y[j] + 1.0092*Z[j];
 
           X[j] = R;
           Y[j] = G;
           Z[j] = B;
 	}        
	return(TRUE);
}





/* This routine performs the UVW to RGB colorspace transformation.  This 
 * information can be found in the "Fundamentals of Digital Image
 * Processing" by Jain.
 */

int uvwtorgb(
double *U,
double *V,
double *W,
int points_per_plane)
{
        int j;					/* Looping index vars.	*/
	double R,G,B;				/* Temp R,G,B values	*/
	double X, Y, Z;				/* Temp X,Y,Z values.   */


       /* The data is transformed from the UVW colorspace to the RGB
        * colorspace. This process was obtained from the "Digital
        * Image Processing" book by Gonzalez and the "Fundamentals of
        * Digital Image Processing" by Jain.
        */
        for(j=0;j<points_per_plane;j++)
        {
          /* Convert from the UVW colorspace to the XYZ colorspace.
           */
           X = 3.0 * U[j] / 2.0;
           Y = V[j];
           Z = 2.0*W[j] + X - 3.0*Y;
 
          /* Convert from the XYZ colorspace model to the RGB colorspace
           * model.
           */
           R = 2.3644*X - 0.8958*Y - 0.4677*Z;
           G = -0.5148*X + 1.4252*Y + 0.0882*Z;
           B = 0.0052*X - 0.0144*Y + 1.0092*Z;
 
           U[j] = R;
           V[j] = G;
           W[j] = B;
        }
	return(TRUE);
}


int ucsuvwtorgb(
double *U,
double *V,
double *W,
int points_per_plane)
{
        int j;					/* Looping index vars.	*/
	double R,G,B;				/* Temp R,G,B values	*/
	double Rn, Gn, Bn;			/* Temp Rn,Gn,Bn values.*/


       /* The data is transformed from the UVW colorspace to the RGB
        * colorspace. This process was obtained from the "Digital
        * Image Processing" book by Gonzalez and the "Fundamentals of
        * Digital Image Processing" by Jain.
        */
        for(j=0;j<points_per_plane;j++)
        {
          /* Convert to Rn, Gn, Bn NTSC Receiver Primary Colorspace 
           * From the UVW Colorspace.                             
	   */
           Rn = 2.4310*U[j] + 0.3309*V[j] - 0.5758*W[j];   
           Gn = -1.5179*U[j] + 2.0837*V[j] - 0.0569*W[j]; 
           Bn = 1.4399*U[j] - 2.8248*V[j] + 1.8031*W[j];  
  
          /* Convert from the RnGnBn NTSC Receiver Primary Colorspace 
           * To the RGB Spectral Primary Colorspace.                   
	   */
           R = 1.167*Rn - 0.146*Gn - 0.151*Bn;     
           G = 0.114*Rn + 0.753*Gn + 0.159*Bn;
           B = -0.001*Rn + 0.059*Gn + 1.128*Bn;
 
           U[j] = R;
           V[j] = G;
           W[j] = B;
        }
	return(TRUE);
}


/* This routine performs the transformation from the UCSSOW to RGB colorspace.
 * Information on this colormodel and transformation can be found in  
 * "Fundamentals of Digital Image Processing" by Jain.
 */

int ucssowtorgb(
double *S,
double *O,
double *W,
int points_per_plane)
{
        int j;					/* Looping index vars.	*/
	double X,Y,Z;				/* X,Y,Z converted data.*/
	double X0,Y0,Z0;			/* White reference XYZ.	*/
	double U,V,WW;				/* UVW colorspace.	*/
	double u,v,v0,u0;			/* Chromaticity values. */
	double R,G,B;				/* R,G,B Values.	*/
	double tempa, tempval;			/* Temporary storage.	*/


       /* The data is transformed from the UCSSOW colorspace to the RGB
        * colorspace. This process was obtained from the "Digital
        * Image Processing" book by Gonzalez and the "Fundamentals of
        * Digital Image Processing" by Jain.
        *
        * The dataset is now converted from UCSLab to the RGB Colorspace.
        * The data is passed in as arrays to each element(S,O,W), and then
        * manipulated via S,O,W pointers.  This allows access to each
        * value individually, without the need to access every data
        * element.
        */

        for(j=0;j<points_per_plane;j++)
        {
          /* X0,Y0,Z0 can be found for the reference white in the
           * NTSC Receiver Primary system.
           */
           X0 = 0.982;
           Y0 = 1.000;
           Z0 = 1.183;

          /* Solve for U and V values.
           */
           tempval = tan(O[j]);
           tempa = pow(tempval, 2.0);
           U = sqrt((S[j] * S[j]) / (1.0 + tempa));
           V = U*O[j];
           WW = W[j];

          /* Solve for u0 and v0.
           */
           u0 = (4.0 * X0) / (X0 + 15.0*Y0 + 3.0*Z0);
           v0 = (6.0 * Y0) / (X0 + 15.0*Y0 + 3.0*Z0);

          /* solve for u and v.
           */
           u = u0 + (U/13.0*WW);
           v = v0 + (V/13.0*WW);

          /* Solve for XYZ values.
           */
           tempval = (WW + 17.0) / 25.0;
           tempa = pow(tempval, 3.0);
           Y = tempa / 100.0;

           X = U/4.0 * 6.0*Y/V;
           Z = ((4.0*X) - (U*X) - (15.0*Y*U)) / (3*U);

          /* Convert from the XYZ space to the RGB colorspace.
           */
           R = 2.3644*X - 0.8958*Y - 0.4677*Z;
           G = -0.5148*X + 1.4252*Y + 0.0882*Z;
           B = 0.0052*X - 0.0144*Y + 1.0092*Z;

           S[j] = R;
           O[j] = G;
           W[j] = B;
        }
	return(TRUE);
}






/* This routine performs the transformation from the UCSLab to RGB colorspace.
 * Information on this colormodel and transformation can be found in  
 * "Fundamentals of Digital Image Processing" by Jain.
 */


int ucslabtorgb(
double *L,
double *A,
double *B,
int points_per_plane)
{
        int j;					/* Looping index vars.	*/
	double Red, Green, Blue;		/* Temp R,G,B values.	*/
	double X,Y,Z;				/* X,Y,Z converted data.*/
	double X0,Y0,Z0;			/* White reference XYZ.	*/
	double tempa, tempval;			/* Temporary storage.	*/


       /* The data is transformed from the UCSLab colorspace to the RGB
        * colorspace. This process was obtained from the "Digital
        * Image Processing" book by Gonzalez and the "Fundamentals of
        * Digital Image Processing" by Jain.
        */
        for(j=0;j<points_per_plane;j++)
        {
          /* X0,Y0,Z0 can be found for the reference white in the
           * NTSC Receiver Primary system.
           */
           X0 = 0.982;
           Y0 = 1.000;
           Z0 = 1.183;

          /* Convert from the UCSLab to the XYZ colorspace model.
           */
           tempval = (L[j] + 16.0) / 25.0;
           tempa = pow(tempval, 3.0);
           Y = Y0/100.0 * tempa;

           tempval = A[j]/500.0;
           tempa = pow(tempval, 3.0);
           X = X0 * (tempa + (Y/Y0));
 
           tempval = B[j]/200.0;
           tempa = pow(tempval, 3.0);
           Z = Z0 * (tempa - (Y/Y0));

          /* Convert from the XYZ colorspace to the RGB colorspace.
           */
           Red = 2.3644*X - 0.8958*Y - 0.4677*Z;
           Green = -0.5148*X + 1.4252*Y + 0.0882*Z;
           Blue = 0.0052*X - 0.0144*Y + 1.0092*Z;

           L[j] = Red;
           A[j] = Green;
           B[j] = Blue;
        }
	return(TRUE);
}



int ucsluvtorgb(
double *L,
double *U,
double *V,
int points_per_plane)
{
	int i;
	double u0, uprime;			/* u and u0 values.	*/
	double v0, vprime;			/* v and v0 values.	*/
	double X,Y,Z;				/* X,Y,Z values.	*/
	double R,G,B;				/* R,G,B values.	*/
	double tempval,temp;		     	/* Temporary storage.	*/
	double X0,Y0, Z0;			/* Reference XYZ vals.	*/

       /* This routine performs the conversion from the UCSLuv colorspace
	* to the RGB colorspace.  Information on this routine can be 
	* found in the "Computer Graphics, Principles and Practices" by
	* Foley, VanDam, Feiner, and Hughes.
	*/
        X0 = 0.982;
        Y0 = 1.000;
        Z0 = 1.183;
	
	u0 = (4.0 * X0) / (X0 + 15.0*Y0 + 3.0*Z0);
	v0 = (6.0 * Y0) / (X0 + 15.0*Y0 + 3.0*Z0);

	for(i=0;i<points_per_plane;i++)
	{
	   temp = (L[i]/25.0) + 16.0;
	   tempval = pow(temp,(1.0/3.0));
	   Y = Y0 / 100.0 * tempval;

	   uprime = (U[i] / 13.0 * L[i]) + u0;
	   vprime = (V[i] / 13.0 * L[i]) + v0;

	   X = ((6.0*Y/vprime) + (30.0*Y)) / (-2.0 + (12.0/uprime));
	   Z = (4.0*X/uprime) - X - 15.0*Y;

           R = 2.3644*X - 0.8958*Y - 0.4677*Z;
           G = -0.5148*X + 1.4252*Y + 0.0882*Z;
           B = 0.0052*X - 0.0144*Y + 1.0092*Z;

	   L[i] = R;
	   U[i] = G;
	   V[i] = B;
	}
	return(TRUE);
}
 






/* RGB to Xcolorspace conversion routines. */

/* This routine performs the transformation from the RGB colorspace to the 
 * grey scale space.  This routine is actually generated by a created 
 * algorithm, which tries to best fit the data to a grey scale from a
 * RGB scale.  This is specifically for viewing purposes, since there is
 * numerous methods of converting a RGB image to a grey scale.
 */

int rgbtogrey(
double *r,
double *g,
double *b,
int points_per_plane)
{
        int i;					/* Looping index param. */


       /* The dataset is now converted from the RGB to GREY space, and the
	* output written to the destination object. 
	*/
	for(i=0;i<points_per_plane;i++)
	{
	      r[i] = r[i];
	      g[i] = 0.0;	
	      b[i] = 0.0;
	}
	return(TRUE);
}




/* This routine performs the RGB to CMY conversion.  The routine is based on
 * the algorithm discussed in "Digital Image Processing" by Gonzalez.  
 */

int rgbtocmy(
double *r,
double *g,
double *b,
int points_per_plane,
double max_value)
{						
        int i;					/* Index looping vars.	*/

       /* This section processes the input data.                       
        * The method used for processing comes from "Digital Image     
	* Processing" by Gonzales, on page 238.   This section is     
	* entitled, "the CMY color model".                             
	* 		[C  = [1	-   [R				   
	*		 M     1	     G    			   
	*		 Y]    1]            B]				   
	*                                      
	*/ 

       /* The dataset is now converted from the RGB to CMY space, and the
	* output written to the destination object.  It is important to 
	* note that the data is not brought in element first.  That is, 
	* rather thank operating on the data in R,G,B order, the data is
	* read in R,R,....G,G,...B,B....  In this routine it is not that
	* important, however, in other transformations, the R,G,B values
	* must be separated.  This may be done by pointers to the R,G,B
	* sections, or using KPDS_ELEMENTS_FIRST attribute.
	*/
	for(i=0;i<points_per_plane;i++)
	{
	   r[i] = r[i] / max_value;	
	   r[i] = 1.0 - r[i];
	   g[i] = g[i] / max_value;	
	   g[i] = 1.0 - g[i];
	   b[i] = b[i] / max_value;	
	   b[i] = 1.0 - b[i];
	}
	return(TRUE);
}







/* This routine is used to convert from the RGB colorspace model to the YIQ
 * colorspace model.  This method can be found in the "Fundamentals of 
 * Digital Image Processing" by Jain, or the "Digital Image Processing" book
 * by Gonzalez.
 */

int rgbtoyiq(
double *r,
double *g,
double *b,
int index,
double max_value)
{
        int j;					/* Looping index var.	*/
	double Y,I,Q;				/* Temp Y,I,Q cals vals	*/


       /* The data is transformed from the RGB colorspace to the YIQ
        * colorspace. This process was obtained from the "Digital
	* Image Processing" book by Gonzalez and the "Fundamentals of
	* Digital Image Processing" by Jain.
        */
 

       /* The dataset is now converted from RGB to the YIQ Colorspace.
	* The data is read in based on an optimal region size, and then
	* manipulated via R,G,B pointers.  This allows access to each
	* value individually, without the need to access every data
	* element.
	*/
        for(j=0;j<index;j++)
        {
	   r[j] = r[j] / max_value;
  	   g[j] = g[j] / max_value;
	   b[j] = b[j] / max_value;

           Y = 0.299*r[j] + 0.587*g[j] + 0.114*b[j];
           I = 0.596*r[j] - 0.275*g[j] - 0.321*b[j];
           Q = 0.212*r[j] - 0.523*g[j] + 0.311*b[j];
 
           r[j] = Y;
           g[j] = I;
           b[j] = Q;
        }
	return(TRUE);
}





/* This routine performs the RGB to HSV colorspace conversion.  Information
 * on this transformation can be found in "Computer Graphics, Principles
 * and Practice" by Foley, VanDam, Feiner, and Hughes on page 592.
 */

int rgbtohsv(
double *R,
double *G,
double *B,
int points_per_plane,
double max_value,
double min_value)
{
        int j;					/* Looping index var.	*/
	double temp_max=0;			/* Temp R,G,B max val.	*/
	double temp_min=0;			/* Temp R,G,B min val.	*/
	double delta;				/* Range delta value.	*/
	double H,S,V;				/* Temp H,S,V cals vals	*/
	double r,g,b;				/* Normalized r,g,b vals*/

                                                /* Data scaling variable*/
        double datarange, desiredrange;         /* Range values.        */
        double mfactor = 1.0;                   /* Multiplication fact. */
        double sfactor = 0.0;                   /* Offset factor.       */


 
       /* The data is transformed from the RGB colorspace to the HSV
        * colorspace. This process was obtained from the "Digital
        * Image Processing" book by Gonzalez and the "Fundamentals of
	* Digital Image Processing" by Jain.
        */
        if(max_value > 1 || min_value < 0)
        {
           datarange = max_value - min_value;
           desiredrange = 1.0 - 0.0;
           mfactor = desiredrange/datarange;
           sfactor = 0.0 - (min_value*mfactor);
        }

       /* This section will manipulate the individual r,g,b pointers.
	*/
        for(j=0;j<points_per_plane;j++)
        {
	   r = (R[j] * mfactor) + sfactor;
	   g = (G[j] * mfactor) + sfactor;
	   b = (B[j] * mfactor) + sfactor;

	   temp_max = r;
	   temp_min = r;

           if(g > temp_max) temp_max = g;
           if(g < temp_min) temp_min = g;
           if(b > temp_max) temp_max = b;
           if(b < temp_max) temp_min = b;

           V = temp_max;
           if(temp_max != 0.0)
              S = (temp_max - temp_min)/temp_max;
           else
              S = 0.0;
           if(S == 0.0) H = 361.0;	/* UNDEFINED SECTION */
           else
           {
              delta = temp_max - temp_min;
              if(r == temp_max) H = (g-b)/delta;
              else if(g == temp_max) H = 2.0 + (b-r)/delta;
              else if(b == temp_max) H = 4.0 + (r-g)/delta;

             /* Convert the Hue to degrees.
              */
              H = H * 60.0;
              if(H < 0) H = H + 360.0;
           }
           R[j] = H;
           G[j] = S;
           B[j] = V;
        }
	return(TRUE);
}





/* This routine performs the RGB to HLS colorspace conversion.  Information
 * on this transformation can be found in "Computer Graphics, Principles
 * and Practice" by Foley, VanDam, Feiner, and Hughes.
 */

int rgbtohls(
double *R,
double *G,
double *B,
int points_per_plane,
double max_value)
{
        int j;					/* Looping index var.	*/

	double data_max;			/* Max data element.	*/
	double data_min;			/* Min data element.	*/
	double delta;				/* Max-min delta value.	*/
	double H,L,S;				/* Temp H,L,S cals vals	*/
	double r,g,b;				/* Temp r,g,b values.	*/


       /* The data is transformed from the RGB colorspace to the HLS
	* colorspace. This process was obtained from the "Computer Graphics,
	* Principles and Practice", by Foley, Van Damm, Feiner, and Hughes.
        */

       /* The data values are manipulated via pointers to the individual 
	* planes, that is, double *r references the red plane or element 1.
	*/
        for(j=0;j<points_per_plane;j++)
        {
	   r = R[j] / max_value;
	   g = G[j] / max_value;
	   b = B[j] / max_value;

           data_max = r;
           data_min = r;

           if(data_max < g) data_max = g;
           if(data_max < b) data_max = b;
           if(data_min > g) data_min = g;
           if(data_min > b) data_min = b;

          /* Calculate the lightness value.
           */
           L = (data_max + data_min) / 2.0;

          /* Calculate the Saturation value.  First is the Achromatic
           * case, where R=G=B.
           */
           if(data_max == data_min)
           {
              S = 0.0;
              H = HUE_UNDEFINED;
           }
           else
           {
              if(L <= 0.5)
                 S = (data_max - data_min)/(data_max + data_min);
              else
                 S = (data_max - data_min)/(2 - data_max - data_min);

             /* Next, calculate the Hue value.
              */
              delta = data_max - data_min;
              if(r == data_max) H = (g - b)/delta;
              else if(g == data_max) H = 2 + (b - r)/delta;
              else if(b == data_max) H = 4 + (r - g)/delta;

              H = H * 60.0;               /* Convert to degrees */
              if(H < 0.0) H = H + 360.0;
           }
           R[j] = H;
           G[j] = L;
           B[j] = S;
        }
	return(TRUE);
}







/* This routine performs the RGB to IHS colorspace transformation.  This   
 * information can be found in the "Digital Image Processing" book by 
 * Gonzalez and "Fundamentals of Digital Image Processing" by Jain.
 */

int rgbtoihs(
double *R,
double *G,
double *B,
int points_per_plane,
double max_value)
{
        int j;					/* Looping index values.*/
	double minrgb=0;			/* Minimum rgb data val.*/
	double den, num;			/* Temp storage vars.	*/
	double Rn,Gn,Bn;			/* Temp R,G,B values.	*/
	double H,S,I;				/* Temp HSI values.	*/


       /* The data is transformed from the RGB colorspace to the IHS
        * colorspace. This process was obtained from the "Digital
        * Image Processing" book by Gonzalez and the "Fundamentals of
        * Digital Image Processing" by Jain.
        */

       /* The data is manipulated via R,G,B pointers, which represent the
	* address of each plane along the elements dimension.
	*/
        for(j=0;j<points_per_plane;j++)
        {
 	   Rn = R[j] / max_value;
	   Gn = G[j] / max_value;
	   Bn = B[j] / max_value;

	   I = 0.333*(Rn + Gn + Bn);

	  /* Calculate the minimum value of R,G,B.
	   */
	   if(Rn <= Gn) minrgb = Rn;
	   else minrgb = Gn;
	   if(Bn <= minrgb) minrgb = Bn;

	   S = 1 - ((3.0/(Rn + Gn + Bn))*minrgb);

           num = (0.5*((Rn-Gn) + (Rn-Bn)));
           den = pow((((Rn-Gn)*(Rn-Gn)) + ((Rn-Bn)*(Gn-Bn))),0.5);
           H = acos(num/den);

	  /* Convert H to degrees.
	   */
	   H = H * 360.0 / (2.0 * 3.141592654);
	   if(H < 0.0) H = H + 360.0;

           R[j] = I;
           G[j] = H;
           B[j] = S;
        }
	return(TRUE);
}






/* This routine performs the RGB to XYZ colorspace transformation.  This 
 * information can be found in "Digital Image Processing" by Gonzalez and 
 * "Fundamentals of Digital Image Processing" by Jain.
 */

int rgbtoxyz(
double *R,
double *G,
double *B,
int points_per_plane)
{
        int j;					/* Looping index vars.	*/
	double X,Y,Z;				/* Temp X,Y,Z values.	*/


       /* The data is transformed from the RGB colorspace to the XYZ
        * colorspace. This process was obtained from the "Digital
        * Image Processing" book by Gonzalez and the "Fundamentals of
        * Digital Image Processing" by Jain.
        */
        for(j=0;j<points_per_plane;j++)
        {
           X = 0.490*R[j] + 0.310*G[j] + 0.200*B[j];
           Y = 0.177*R[j] + 0.813*G[j] + 0.011*B[j];
           Z = 0.000*R[j] + 0.010*G[j] + 0.990*B[j];
 
           R[j] = X;
           G[j] = Y;
           B[j] = Z;
        }
	return(TRUE);
}




/* This routine performs the trasformation from the RGB colorspace to the UVW
 * colorspace.  Information on this transformation can be found in the  
 * "Fundamentals of Digital Image Processing", by Jain.
 */

int rgbtouvw(
double *R,
double *G,
double *B,
int points_per_plane)
{
        int j;					/* Looping index vars.	*/
	double U,V,W;				/* Temp U,V,W, values.	*/
	double X, Y, Z;				/* X,Y,Z colorspace vals*/


       /* The data is transformed from the RGB colorspace to the UVW
        * colorspace. This process was obtained from the "Digital
        * Image Processing" book by Gonzalez and the "Fundamentals of
        * Digital Image Processing" by Jain, page 67.
        */
        for(j=0;j<points_per_plane;j++)
        {

          /* First, convert from the UVW colorspace to the XYZ colorspace
           * model.
           */
           X = 0.490*R[j] + 0.310*G[j] + 0.200*B[j];
           Y = 0.177*R[j] + 0.813*G[j] + 0.011*B[j];
           Z = 0.000*R[j] + 0.010*G[j] + 0.990*B[j];
 
          /* Next, convert from the XYZ colorspace model to the UVW
           * colorspace model.  Information on both of these transforms
           * can be found in "Fundamentals of Digital Image Processing"
           * by Jain.
           */
           U = 2.0 * X / 3.0;
           V = Y;
           W = (3.0*Y + Z - X) / 2.0;
 
           R[j] = U;
           G[j] = V;
           B[j] = W;
        }
	return(TRUE);
}







/* This routine performs the transformation from the RGB to UCSUVW colorspace.
 * Information on this colormodel and transformation can be found in  
 * "Fundamentals of Digital Image Processing" by Jain.
 */

int rgbtoucsuvw(
double *R,
double *G,
double *B,
int points_per_plane)
{
        int j;					/* Looping index vars.	*/
	double U,V,W;				/* Temp U,V,W, values.	*/
	double Rn, Gn, Bn;			/* Normalized R,G,B.	*/



       /* The data is transformed from the RGB colorspace to the UCSUVW
        * colorspace. This process was obtained from the "Digital
        * Image Processing" book by Gonzalez and the "Fundamentals of
        * Digital Image Processing" by Jain.
        */
        for(j=0;j<points_per_plane;j++)
        {
          /* Convert to Rn, Gn, Bn NTSC Receiver Primary Colorspace 
           * from the RGB Spectral Primary System.                  
	   */
           Rn = 0.8417*R[j] + 0.1561*G[j] + 0.0907*B[j];
           Gn = -0.1298*R[j] + 1.3189*G[j] - 0.2032*B[j];
           Bn = 0.0075*R[j] - 0.0688*G[j] + 0.8972*B[j];
 
          /* Convert from the RnGnBn NTSC Receiver Primary Colorspace 
           * To the UVW colorspace.                                  
	   */
           U = 0.405*Rn + 0.116*Gn + 0.133*Bn;
           V = 0.299*Rn + 0.587*Gn + 0.114*Bn;
           W = 0.145*Rn + 0.827*Gn + 0.627*Bn;

           R[j] = U;
           G[j] = V;
           B[j] = W;
        }
	return(TRUE);
}






/* This routine performs the transformation from the RGB to UCSSOW colorspace.
 * Information on this colormodel and transformation can be found in  
 * "Fundamentals of Digital Image Processing" by Jain.
 */


int rgbtoucssow(
double *R,
double *G,
double *B,
int points_per_plane)
{
        int j;					/* Looping index vars.	*/
	double S,O,W;				/* Temp S,O,W values.	*/
	double X,Y,Z;				/* X,Y,Z converted data.*/
	double X0,Y0,Z0;			/* White reference XYZ.	*/
	double U,V;				/* U,V,W Variables.	*/
	double u,v,u0,v0;			/* Chromaticities values*/
	double tempa, tempval;			/* Temporary storage.	*/


       /* The data is transformed from the RGB colorspace to the UCSSOW
        * colorspace. This process was obtained from the "Digital
        * Image Processing" book by Gonzalez and the "Fundamentals of
        * Digital Image Processing" by Jain.
        */
        for(j=0;j<points_per_plane;j++)
        {
          /* Convert the RGB image to the XYZ colorspace model.
	   */
           X = 0.490*R[j] + 0.310*G[j] + 0.200*B[j];
           Y = 0.177*R[j] + 0.813*G[j] + 0.011*B[j];
           Z = 0.000*R[j] + 0.010*G[j] + 0.990*B[j];

	  /* Get X0,Y0,Z0, the reference for the color white in the 
	   * NTSC Reciever Primary System.
	   */
	   X0 = 0.982;
	   Y0 = 1.000;
	   Z0 = 1.183;

          /* Obtain the values for u0 and v0.
	   */
	   u0 = (4.0 * X0) / (X0 + 15.0*Y0 + 3.0*Z0);
	   v0 = (6.0 * Y0) / (X0 + 15.0*Y0 + 3.0*Z0);
	   u = (4.0 * X) / (X + 15.0*Y + 3.0*Z);
	   v = (6.0 * Y) / (X + 15.0*Y + 3.0*Z);


	  /* Obtain values for U,V,W, the modified UCS system.
	   */
           tempval = (100.0 * Y);
           tempa = pow(tempval, 0.33333);
           W = (25.0 * tempa) - 17.0;
           U = 13.0 * W * (u - u0);
           V = 13.0 * W * (v - v0);

          /* The conversion to the UCSSOW system is now performed.
           */
           tempval = ((U*U) + (V*V));
           S = pow(tempval, 0.5);
           O = atan(V/U);

	  /* Convert the theta value to degrees.
	   */
	   O = O * 360.0 / (2.0 * 3.141592654);
	   if(O < 0.0) O = 360.0 + O;
	
           R[j] = S;
           G[j] = O;
           B[j] = W;
        }
	return(TRUE);
}






/* This routine performs the transformation from the RGB to UCSLAB colorspace.
 * Information on this colormodel and transformation can be found in  
 * "Fundamentals of Digital Image Processing" by Jain.
 */

int rgbtoucslab(
double *R,
double *G,
double *B,
int points_per_plane)
{
        int j;					/* Looping index vars.	*/
	double L,A,Bright;			/* Temp R,A,B values.	*/
	double X,Y,Z;				/* X,Y,Z converted data.*/
	double X0,Y0,Z0;			/* White reference XYZ.	*/
	double tempa, tempb, tempval;		/* Temporary storage.	*/


       /* The data is transformed from the RGB colorspace to the UCSlab
        * colorspace. This process was obtained from the "Digital
        * Image Processing" book by Gonzalez and the "Fundamentals of
        * Digital Image Processing" by Jain.
        */
        for(j=0;j<points_per_plane;j++)
        {
	  /* Convert the RGB image to the XYZ colorspace model.
	   */
           X = 0.490*R[j] + 0.310*G[j] + 0.200*B[j];
           Y = 0.177*R[j] + 0.813*G[j] + 0.011*B[j];
           Z = 0.000*R[j] + 0.010*G[j] + 0.990*B[j];

	  /* Get X0,Y0,Z0, the reference for the color white in the 
	   * NTSC Reciever Primary System.
	   */
	   X0 = 0.982;
	   Y0 = 1.000;
	   Z0 = 1.183;

          /* The conversion to the UCSlab system is now performed.
           */
           tempval = 100.0 * Y / Y0;
           tempa = pow(tempval,0.33333);
           L = 25.0*(tempa) - 16;
 
           tempval = X/X0;
           tempa = pow(tempval,0.33333);
           tempval = Y/Y0;
           tempb = pow(tempval,0.33333);
           A = 500*(tempa - tempb);
 
           tempval = Y/Y0;
           tempa = pow(tempval,0.33333);
           tempval = Z/Z0;
           tempb = pow(tempval,0.33333);
           Bright = 200*(tempa - tempb);
 
           R[j] = L;
           G[j] = A;
           B[j] = Bright;
        }
	return(TRUE);
}




int rgbtoucsluv(
double *R,
double *G,
double *B,
int points_per_plane)
{
	int i;					/* Looping index.	*/
	double X0,Y0,Z0;			/* Reference XYZ values.*/
	double X,Y,Z;				/* X,Y,Z colorspace val.*/
	double tempval;				/* Temp. calc. value.	*/
	double u0, uprime;			/* U and U0 values.	*/
	double v0, vprime;			/* V and V0 values.	*/
	double L,U,V;				/* L,U,V values.	*/


       /* This routine performs the conversion from RGB to the UCSLUV 
	* colorspace.  This information was obtained from "Fundamentals
	* of Digital Image Processing" by Jain, and "Computer Graphics, 
	* principles and practice" by Foley, Van Dam, Feiner, and Hughes.
	*/
	X0 = 0.982;
	Y0 = 1.00;
	Z0 = 1.183;

	u0 = (4.0 * X0) / (X0 + 15.0*Y0 + 3.0*Z0);
	v0 = (6.0 * Y0) / (X0 + 15.0*Y0 + 3.0*Z0);

	for(i=0;i<points_per_plane;i++)
	{
          /* Perform the conversion from the RGB to XYZ colorspace.  These
	   * values are needed for the conversion to LUV.
	   */
           X = 0.490*R[i] + 0.310*G[i] + 0.200*B[i];
           Y = 0.177*R[i] + 0.813*G[i] + 0.011*B[i];
           Z = 0.000*R[i] + 0.010*G[i] + 0.990*B[i];

          /* Obtain values for u, u0, v, and v0.
	   */
           uprime = (4.0 * X) / (X + 15.0*Y + 3.0*Z);
           vprime = (9.0 * Y) / (X + 15.0*Y + 3.0*Z);

          /* Perform the conversion from to the UCSLUV colorspace.
	   */
	   tempval = (100.0 * Y * Y0);
	   L = 25.0 * (pow(tempval, (1.0/3.0))) - 16.0;
	   U = 13.0 * L * (uprime - u0);
	   V = 13.0 * L * (vprime - v0);

	   R[i] = L;
	   G[i] = U;
	   B[i] = V;
	}

	return(TRUE);
}


 





/* -library_code_end */
