 /*
  * Khoros: $Id: info.c,v 1.1 1997/03/04 09:28:53 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: info.c,v 1.1 1997/03/04 09:28:53 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Polymorphic Data Services
   >>>>		
   >>>>  Private:
   >>>>                 kpds_init()
   >>>>   Static:
   >>>>                 _nop_set()
   >>>>                 _nop_copy()
   >>>>                 _line_info_get()
   >>>>                 _line_info_match()
   >>>>                 _line_info_query()
   >>>>                 _line_info_print()
   >>>>                 _plane_info_get()
   >>>>                 _plane_info_match()
   >>>>                 _plane_info_query()
   >>>>                 _plane_info_print()
   >>>>                 _volume_info_get()
   >>>>                 _volume_info_match()
   >>>>                 _volume_info_query()
   >>>>                 _volume_info_print()
   >>>>                 _vector_info_get()
   >>>>                 _vector_info_match()
   >>>>                 _vector_info_query()
   >>>>                 _vector_info_print()
   >>>>                 _region_info_get()
   >>>>                 _region_info_match()
   >>>>                 _region_info_query()
   >>>>                 _region_info_print()
   >>>>                 _optimal_region_constraint_get()
   >>>>                 _optimal_region_constraint_set()
   >>>>                 _optimal_region_constraint_match()
   >>>>                 _optimal_region_constraint_query()
   >>>>                 _optimal_region_constraint_print()
   >>>>                 _optimal_region_flags_get()
   >>>>                 _optimal_region_flags_set()
   >>>>                 _optimal_region_flags_match()
   >>>>                 _optimal_region_flags_query()
   >>>>                 _optimal_region_flags_print()
   >>>>                 _optimal_region_size_get()
   >>>>                 _optimal_region_size_match()
   >>>>                 _optimal_region_size_query()
   >>>>                 _optimal_region_size_print()
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

extern char *pds_mask_string;
extern char *pds_value_string;
extern char *pds_location_string;
extern char *pds_map_string;
extern char *pds_time_string;

/*
 *    ================================================================== 
 *    KPDS Quasi Attribute Methods
 *    ==================================================================
 */

/*-----------------------------------------------------------
|  Routine Name: (static) _nop_set
|       Purpose: This routine is used in lieu of a set function when
|		 the function is considered to be a read only attribute.
|    Written By: Jeremy Worley & John Salas
|          Date: September 28, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static int
_nop_set(kobject object, ktoken segment, ktoken attribute, kaddr client_data, 
	 kva_list * list)
{
   return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _nop_copy
|       Purpose: This routine is used in lieu of a copy function when
|		 the function is considered to be a read only attribute.
|    Written By: Steve Kubica
|          Date: April 4, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_nop_copy(kobject object1, kobject object2, ktoken segment, ktoken attribute,
	  kaddr client_data1, kaddr client_data2)
{
   return FALSE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_LINE_INFO
 *********** --------------------------------------------------------------- */

/*-----------------------------------------------------------
|  Routine Name: (static) _line_info_get
|       Purpose: This routine retrieves the "line information".
|                This information consists of the size in pixels of data
|                points of a line, and the total number of lines.
|    Written By: Jeremy Worley
|          Date: April 1, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_line_info_get(kobject object, ktoken segment, ktoken attribute, 
	       kaddr client_data, kva_list * list)
{
   int 	 *size = NULL, *tmp = NULL;
   int 	  dim, i;
   ktoken seg = kstring_to_token((char *)client_data);

   if (!kpds_get_size(object, seg, &size))
      return FALSE;

   /* get the line length */
   if ((tmp = kva_arg(*list, int *)) != NULL)
         (*tmp) = size[0];	/* a line is always along width */

   dim = kpds_get_dimension(object, seg);

   /* compute the total number of lines */
   if ((tmp = kva_arg(*list, int *)) != NULL)
      for (i = 1, (*tmp) = 1; i < dim; i++)
	 (*tmp) *= size[i];

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _line_info_match
|       Purpose: match the line info attribute
|    Written By: Steve Kubica
|          Date: May 2, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_line_info_match(kobject obj1, kobject obj2, ktoken segment, ktoken attribute,
		 kaddr client_data1, kaddr client_data2)
{
   int 	 *size1 = NULL, *size2 = NULL;
   int 	  tot1 = 1, tot2 = 1;
   int 	  dim, i;
   ktoken seg1 = kstring_to_token((char *) client_data1);
   ktoken seg2 = kstring_to_token((char *) client_data2);
   

   if (!kpds_get_size(obj1, seg1, &size1))
      return FALSE;

   if (!kpds_get_size(obj2, seg2, &size2))
      return FALSE;

   /* line length is always along width */
   if (size1[0] != size2[0])
      return FALSE;

   /* get the dimensionality of the segment */
   dim = kpds_get_dimension(obj1, seg1);

   /* compute the total number of lines */
   for (i = 1, tot1 = 1, tot2 = 1; i < dim; i++)
   {
      tot1 *= size1[i];
      tot2 *= size2[i];
   }

   return (tot1 == tot2);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _line_info_query
|       Purpose: query the line info attribute
|    Written By: 
|          Date: 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_line_info_query(kobject obj, ktoken segment, ktoken attribute, 
		 kaddr client_data, int *num_args, int *arg_size, 
		 int *data_type, int *permanent)
{
   /* if the segment does not exist, then it has no line info */
   if (!kdms_query_segment(obj, (char *) client_data))
      return FALSE;		/* not an error */

   if (num_args)
      *num_args = 2;
   if (arg_size)
      *arg_size = 1;
   if (data_type)
      *data_type = KINT;
   if (permanent)
      *permanent = FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _line_info_print
|       Purpose: print the line info attribute
|    Written By: Steve Kubica
|          Date: May 2, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_line_info_print(kobject obj, ktoken segment, ktoken attribute, 
		 kaddr client_data, kfile *outfile)
{
   int *size = NULL;
   int  tot = 1;
   int  dim;
   int  i;
   ktoken seg = kstring_to_token((char *)client_data);

   if (!kpds_get_size(obj, seg, &size))
      return FALSE;

   dim = kpds_get_dimension(obj, seg);

   /* compute the total number of lines */
   for (i = 1, tot = 1; i < dim; i++)
      tot *= size[i];

   if (outfile)
      kfprintf(outfile, "%d\t%d", size[0], tot);

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_PLANE_INFO
 *********** --------------------------------------------------------------- */

/*-----------------------------------------------------------
|  Routine Name: (static) _plane_info_get
|       Purpose: This routine retrieves the "plane information".
|                This information consists of the size in pixels of data
|                points of a plane, and the total number of planes.
|    Written By: Jeremy Worley
|          Date: April 1, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_plane_info_get(kobject obj, ktoken segment, ktoken attr, kaddr client_data,
		kva_list *list)
{
   int 	 *size = NULL;
   int 	 *tmp = NULL;
   int 	  dim;
   int 	  i;
   ktoken seg = kstring_to_token((char *)client_data);


   /* get the model-based size of this segment */
   if (!kpds_get_size(obj, seg, &size))
      return FALSE;

   /* get the plane size */
   if ((tmp = kva_arg(*list, int *)) != NULL)
         (*tmp) = size[0];	/* a plane is always along width ...  */

   if ((tmp = kva_arg(*list, int *)) != NULL)
         (*tmp) = size[1];	/* ... and height */

   /* get the dimensionality of the segment */
   dim = kpds_get_dimension(obj, seg);

   /* compute the total number of planes */
   if ((tmp = kva_arg(*list, int *)) != NULL)
      for (i = 2, (*tmp) = 1; i < dim; i++)
	 (*tmp) *= size[i];

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _plane_info_match
|       Purpose: match the plane info attribute
|    Written By: Steve Kubica
|          Date: May 2, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_plane_info_match(kobject obj1, kobject obj2, ktoken segment, ktoken attribute,
		  kaddr client_data1, kaddr client_data2)
{
   int 	 *size1 = NULL;
   int 	 *size2 = NULL;
   int 	  tot1 = 1;
   int 	  tot2 = 1;
   int 	  dim;
   int 	  i;
   ktoken seg1 = kstring_to_token((char *)client_data1);
   ktoken seg2 = kstring_to_token((char *)client_data2);


   /* get the model-based size of the first segment */
   if (!kpds_get_size(obj1, seg1, &size1))
      return FALSE;

   /* get the model-based size of the first segment */
   if (!kpds_get_size(obj2, seg2, &size2))
      return FALSE;

   /* plane size is always along width and height */
   if ((size1[0] != size2[0]) || (size1[1] != size2[1]))
      return FALSE;

   /* get the dimensionality of the segment */
   kdms_get_attribute(obj1, (char *) seg1, KDMS_DIMENSION, &dim);

   /* compute the total number of lines */
   for (i = 2, tot1 = 1, tot2 = 1; i < dim; i++)
   {
      tot1 *= size1[i];
      tot2 *= size2[i];
   }

   return (tot1 == tot2);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _plane_info_query
|       Purpose: query the plane info attribute
|    Written By: Steve Kubica
|          Date: May 2, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_plane_info_query(kobject obj, ktoken segment, ktoken attribute, kaddr client_data,
		  int *num_args, int *arg_size, int *data_type,
		  int *permanent)
{
   /* if the segment does not exist, then it has no plane info */
   if (!kdms_query_segment(obj, (char *) client_data))
      return FALSE;		/* not an error */

   if (num_args)
      *num_args = 3;
   if (arg_size)
      *arg_size = 1;
   if (data_type)
      *data_type = KINT;
   if (permanent)
      *permanent = FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _plane_info_print
|       Purpose: print the plane info attribute
|    Written By: Steve Kubica
|          Date: May 2, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_plane_info_print(kobject obj, ktoken segment, ktoken attribute, 
		  kaddr client_data, kfile *outfile)
{
   int 	 *size = NULL;
   int 	  tot = 1;
   int 	  dim;
   int 	  i;
   ktoken seg = kstring_to_token((char *)client_data);


   /* get the model-based size of the first segment */
   if (!kpds_get_size(obj, seg, &size))
      return FALSE;

   /* get the dimensionality of the segment */
   kdms_get_attribute(obj, (char *) seg, KDMS_DIMENSION, &dim);

   /* compute the total number of planes */
   for (i = 2, tot = 1; i < dim; i++)
      tot *= size[i];

   if (outfile)
      kfprintf(outfile, "%d x %d\t%d",
	       size[0], size[1], tot);

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_VOLUME_INFO
 *********** --------------------------------------------------------------- */

/*-----------------------------------------------------------
|  Routine Name: (static) _volume_info_get
|       Purpose: This routine retrieves the "volume information".
|                This information consists of the size in pixels
|                or data points of a volume, and the total number
|                of volumes.
|    Written By: Jeremy Worley
|          Date: April 1, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_volume_info_get(kobject obj, ktoken segment, ktoken attr, kaddr client_data,
		 kva_list *list)
{
   int 	 *size = NULL;
   int 	 *tmp = NULL;
   int 	  dim;
   int 	  i;
   ktoken seg = kstring_to_token((char *)client_data);


   if (!kpds_get_size(obj, seg, &size))
      return FALSE;

   /* get the volume size */
   if ((tmp = kva_arg(*list, int *)) != NULL)
         (*tmp) = size[0];	/* a volume is always along width ...  */

   if ((tmp = kva_arg(*list, int *)) != NULL)
         (*tmp) = size[1];	/* ... and height ... */

   if ((tmp = kva_arg(*list, int *)) != NULL)
         (*tmp) = size[2];	/* ... and depth */

   /* get the dimensionality of the segment */
   dim = kpds_get_dimension(obj, seg);

   /* compute the total number of planes */
   if ((tmp = kva_arg(*list, int *)) != NULL)
      for (i = 3, (*tmp) = 1; i < dim; i++)
	 (*tmp) *= size[i];

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _volume_info_match
|       Purpose: match the volume info attribute
|    Written By: 
|          Date: 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_volume_info_match(kobject obj1, kobject obj2, ktoken segment, 
		   ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   int 	 *size1 = NULL;
   int 	 *size2 = NULL;
   int 	  tot1 = 1;
   int 	  tot2 = 1;
   int 	  dim;
   int 	  i;
   ktoken seg1 = kstring_to_token((char *)client_data1);
   ktoken seg2 = kstring_to_token((char *)client_data2);
   

   if (!kpds_get_size(obj1, seg1, &size1))
      return FALSE;

   if (!kpds_get_size(obj2, seg2, &size2))
      return FALSE;

   /* volume size is always along width, height, and depth */
   if ((size1[0] != size2[0]) || 
       (size1[1] != size2[1]) || 
       (size1[2] != size2[2]))
      return FALSE;

   /* get the dimensionality of the segment */
   dim = kpds_get_dimension(obj1, seg1);

   /* compute the total number of lines */
   for (i = 3, tot1 = 1, tot2 = 1; i < dim; i++)
   {
      tot1 *= size1[i];
      tot2 *= size2[i];
   }

   return (tot1 == tot2);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _volume_info_query
|       Purpose: query the volume info attribute
|    Written By: 
|          Date: 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_volume_info_query(kobject obj, ktoken segment, ktoken attribute, 
		   kaddr client_data, int *num_args, int *arg_size, 
		   int *data_type, int *permanent)
{
   /* if the segment does not exist, then it has no volume info */
   if (!kdms_query_segment(obj, (char *) client_data))
      return FALSE;		/* not an error */

   if (num_args)
      *num_args = 4;
   if (arg_size)
      *arg_size = 1;
   if (data_type)
      *data_type = KINT;
   if (permanent)
      *permanent = FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _volume_info_print
|       Purpose: print the volume info attribute
|    Written By: 
|          Date: 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_volume_info_print(kobject obj, ktoken segment, ktoken attribute, 
		   kaddr client_data, kfile *outfile)
{
   int 	 *size = NULL;
   int 	  tot = 1;
   int 	  dim;
   int 	  i;
   ktoken seg = kstring_to_token((char *)client_data);

   /* get the model-based size of the first segment */
   if (!kpds_get_size(obj, seg, &size))
      return FALSE;

   /* get the dimensionality of the segment */
   dim = kpds_get_dimension(obj, seg);

   /* compute the total number of volumes */
   for (i = 3, tot = 1; i < dim; i++)
      tot *= size[i];

   if (outfile)
      kfprintf(outfile, "%d x %d x %d\t%d",
	       size[0], size[1], size[2], tot);

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_VECTOR_INFO
 *********** --------------------------------------------------------------- */

/*-----------------------------------------------------------
|  Routine Name: (static) _vector_info_get
|       Purpose: This routine retrieves the "vector information".
|                This information consists of the size in pixels
|                or data points of a vector, and the total number
|                of vectors.
|    Written By: Jeremy Worley & John Salas
|          Date: September 28, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static int
_vector_info_get(kobject obj, ktoken segment, ktoken attr, kaddr client_data,
		 kva_list *list)
{
   int   *size = NULL;
   int   *tmp = NULL;
   int    i;
   ktoken seg = kstring_to_token((char *) client_data);   

   /* get the model-based size of this segment */
   if (!kpds_get_size(obj, seg, &size))
      return FALSE;

   /* the vector primitive only exists for the value, mask, and location */
   if ((kstrcmp((char *)client_data, KDMS_SEGMENT_VALUE) == 0) ||
       (kstrcmp((char *)client_data, KDMS_SEGMENT_MASK) == 0))
   {

      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = size[KELM];	/* a vector is along elements */

      /* compute the total number of vectors */
      if ((tmp = kva_arg(*list, int *)) != NULL)
	 for (i = 0, (*tmp) = 1; i < KELM; i++)
	    (*tmp) *= size[i];

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_LOCATION) == 0)
   {

      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = size[KDIM];	/* a vector is along dimension */

      /* compute the total number of vectors */
      if ((tmp = kva_arg(*list, int *)) != NULL)
	 for (i = 0, (*tmp) = 1; i < KDIM; i++)
	    (*tmp) *= size[i];
   }
   else
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _vector_info_match
|       Purpose: match the vector info attribute
|    Written By: 
|          Date: 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_vector_info_match(kobject obj1, kobject obj2, ktoken segment, 
		   ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   int 	 *size1 = NULL;
   int 	 *size2 = NULL;
   int 	  tot1 = 1;
   int 	  tot2 = 1;
   int 	  i;
   ktoken seg1 = kstring_to_token((char *)client_data1);
   ktoken seg2 = kstring_to_token((char *)client_data2);


   /* get the model-based size of the first segment */
   if (!kpds_get_size(obj1, seg1, &size1))
      return FALSE;

   /* get the model-based size of the second segment */
   if (!kpds_get_size(obj2, seg2, &size2))
      return FALSE;

   /* the vector primitive only exists for the value, mask, and location */
   if ((kstrcmp((char *)client_data1, KDMS_SEGMENT_VALUE) == 0) ||
       (kstrcmp((char *)client_data1, KDMS_SEGMENT_MASK) == 0))
   {

      if (size1[KELM] != size2[KELM])
	 return FALSE;

      /* compute the total number of vectors */
      for (i = 0, tot1 = 1, tot2 = 1; i < KELM; i++)
      {
	 tot1 *= size1[i];
	 tot2 *= size2[i];
      }

   }
   else if (kstrcmp((char *)client_data1, KDMS_SEGMENT_LOCATION) == 0)
   {

      if (size1[KDIM] != size2[KDIM])
	 return FALSE;

      /* compute the total number of vectors */
      for (i = 0, tot1 = 1, tot2 = 1; i < KDIM; i++)
      {
	 tot1 *= size1[i];
	 tot2 *= size2[i];
      }

   }
   else
      return FALSE;

   return (tot1 == tot2);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _vector_info_query
|       Purpose: query the vector info attribute
|    Written By: 
|          Date: 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_vector_info_query(kobject obj, ktoken segment, ktoken attribute, 
		   kaddr client_data, int *num_args, int *arg_size,
		   int *data_type, int *permanent)
{
   /* if the segment does not exist, then it has no vector info */
   if (!kdms_query_segment(obj, (char *) client_data))
      return FALSE;		/* not an error */

   /* the vector primitive only exists for the value, mask, and location */
   if ((kstrcmp((char *)client_data, KDMS_SEGMENT_VALUE) != 0) &&
       (kstrcmp((char *)client_data, KDMS_SEGMENT_MASK) != 0) &&
       (kstrcmp((char *)client_data, KDMS_SEGMENT_LOCATION) != 0))
      return FALSE;		/* not really an error either */

   if (num_args)
      *num_args = 2;
   if (arg_size)
      *arg_size = 1;
   if (data_type)
      *data_type = KINT;
   if (permanent)
      *permanent = FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _vector_info_print
|       Purpose: print the vector info attribute
|    Written By: 
|          Date: 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_vector_info_print(kobject obj, ktoken segment, ktoken attribute, 
		   kaddr client_data, kfile *outfile)
{
   int 	 *size = NULL;
   int 	  tot = 1;
   int 	  i;
   ktoken seg = kstring_to_token((char *)client_data);
   
   if (!kpds_get_size(obj, seg, &size))
      return FALSE;

   /* the vector primitive only exists for the value, mask, and location */
   if ((kstrcmp((char *)client_data, KDMS_SEGMENT_VALUE) == 0) ||
       (kstrcmp((char *)client_data, KDMS_SEGMENT_MASK) == 0))
   {
      /* compute the total number of vectors */
      for (i = 0, tot = 1; i < KELM; i++)
	 tot *= size[i];

      if (outfile)
	 kfprintf(outfile, "%d\t%d",
		  size[KELM], tot);

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_LOCATION) == 0)
   {
      /* compute the total number of vectors */
      for (i = 0, tot = 1; i < KDIM; i++)
	 tot *= size[i];

      if (outfile)
	 kfprintf(outfile, "Vector Size = %d \tNumber of Vectors = %d",
		  size[KDIM], tot);
   }
   else
      return FALSE;

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_REGION_INFO
 *********** --------------------------------------------------------------- */

/*-----------------------------------------------------------
|  Routine Name: (static) _region_info_get
|       Purpose: This routine retrieves the "region information".
|                This information consists of the size in pixels
|                or data points of a region, and the total number
|                of vectors.
|    Written By: Jeremy Worley & John Salas
|          Date: September 28, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static int
_region_info_get(kobject obj, ktoken segment, ktoken attr, kaddr client_data,
		 kva_list *list)
{
   int *size = NULL;
   int *tmp = NULL;
   int  tot;
   int  rw;
   int  rh;
   int  rd;
   int  rt;
   int  re;
   ktoken seg = kstring_to_token((char *)client_data);

   if (!kpds_get_size(obj, seg, &size))
      return FALSE;

   /* the region primitive varies for every segment */
   if (kstrcmp((char *)client_data, KDMS_SEGMENT_VALUE) == 0)
   {

      kpds_get_attribute(obj, KPDS_VALUE_REGION_SIZE, &rw, &rh, &rd, &rt, &re);

      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rw;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rh;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rd;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rt;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = re;

      /* calculate and return the number of regions */
      tot = (int)(kceil((float)size[0]/(float)rw) * 
		  kceil((float)size[1]/(float)rh) *
		  kceil((float)size[2]/(float)rd) *
		  kceil((float)size[3]/(float)rt) *
		  kceil((float)size[4]/(float)re));
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = tot;

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_MASK) == 0)
   {

      kpds_get_attribute(obj, KPDS_MASK_REGION_SIZE, &rw, &rh, &rd, &rt, &re);

      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rw;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rh;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rd;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rt;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = re;

      /* calculate and return the number of regions */
      tot = (int)(kceil((float)size[0]/(float)rw) * 
		  kceil((float)size[1]/(float)rh) *
		  kceil((float)size[2]/(float)rd) *
		  kceil((float)size[3]/(float)rt) *
		  kceil((float)size[4]/(float)re));
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = tot;

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_LOCATION) == 0)
   {

      kpds_get_attribute(obj, KPDS_LOCATION_REGION_SIZE, &rw, &rh, &rd, &re);

      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rw;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rh;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rd;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = re;

      /* calculate and return the number of regions */
      tot = (int)(kceil((float)size[0]/(float)rw) * 
		  kceil((float)size[1]/(float)rh) *
		  kceil((float)size[2]/(float)rd) *
		  kceil((float)size[3]/(float)re));
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = tot;

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_TIME) == 0)
   {

      kpds_get_attribute(obj, KPDS_TIME_REGION_SIZE, &rt);

      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rt;

      /* calculate and return the number of regions */
      tot = kceil((float)size[0] / (float)rt);
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = tot;

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_MAP) == 0)
   {

      kpds_get_attribute(obj, KPDS_MAP_REGION_SIZE, &rw, &rh, &rd, &rt, &re);

      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rw;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rh;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rd;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = rt;
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = re;

      /* calculate and return the number of regions */
      tot = (int)(kceil((float)size[0]/(float)rw) * 
		  kceil((float)size[1]/(float)rh) *
		  kceil((float)size[2]/(float)rd) *
		  kceil((float)size[3]/(float)rt) *
		  kceil((float)size[4]/(float)re));
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = tot;

   }
   else
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _region_info_match
|       Purpose: match the region info attribute
|    Written By: 
|          Date: 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_region_info_match(kobject obj1, kobject obj2, ktoken segment, 
		   ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   int *size1 = NULL;
   int *size2 = NULL;
   int  dim;
   int  status;
   int  i;
   ktoken seg1 = kstring_to_token((char *)client_data1);
   ktoken seg2 = kstring_to_token((char *)client_data2);
   

   if (!kpds_get_size(obj1, seg1, &size1))
      return FALSE;

   if (!kpds_get_size(obj2, seg2, &size2))
      return FALSE;

   /* the region primitive varies for every segment */
   if (kstrcmp((char *)client_data1, KDMS_SEGMENT_VALUE) == 0)
      status = kpds_match_attribute(obj1, obj2, KPDS_VALUE_REGION_SIZE);

   else if (kstrcmp((char *)client_data1, KDMS_SEGMENT_MASK) == 0)
      status = kpds_match_attribute(obj1, obj2, KPDS_MASK_REGION_SIZE);

   else if (kstrcmp((char *)client_data1, KDMS_SEGMENT_LOCATION) == 0)
      status = kpds_match_attribute(obj1, obj2, KPDS_LOCATION_REGION_SIZE);

   else if (kstrcmp((char *)client_data1, KDMS_SEGMENT_TIME) == 0)
      status = kpds_match_attribute(obj1, obj2, KPDS_TIME_REGION_SIZE);

   else if (kstrcmp((char *)client_data1, KDMS_SEGMENT_MAP) == 0)
      status = kpds_match_attribute(obj1, obj2, KPDS_MAP_REGION_SIZE);

   else
      return FALSE;

   /* get the dimensionality of the segment */
   dim = kpds_get_dimension(obj1, seg1);

   if (status)	/* regions are same size - so we're cool if the sizes match */
      for (i = 0; i < dim; i++)
	 status &= (size1[i] == size2[i]);

   return status;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _region_info_query
|       Purpose: query the region info attribute
|    Written By: 
|          Date: 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_region_info_query(kobject obj, ktoken segment, ktoken attribute, 
		   kaddr client_data, int *num_args, int *arg_size, 
		   int *data_type, int *permanent)
{
   /* if the segment does not exist, then it has no region info */
   if (!kdms_query_segment(obj, (char *) client_data))
      return FALSE;		/* not an error */

   if (arg_size)
      *arg_size = 1;
   if (data_type)
      *data_type = KINT;
   if (permanent)
      *permanent = FALSE;

   /* the num_args of the region info primitive varies for every segment */
   if (kstrcmp((char *)client_data, KDMS_SEGMENT_VALUE) == 0)
   {
      if (num_args)
	 *num_args = 6;
   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_MASK) == 0)
   {
      if (num_args)
	 *num_args = 6;
   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_LOCATION) == 0)
   {
      if (num_args)
	 *num_args = 5;
   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_TIME) == 0)
   {
      if (num_args)
	 *num_args = 2;
   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_MAP) == 0)
   {
      if (num_args)
	 *num_args = 6;
   }
   else
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _region_info_print
|       Purpose: print the region info attribute
|    Written By: 
|          Date: 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_region_info_print(kobject obj, ktoken segment, ktoken attribute, 
		   kaddr client_data, kfile *outfile)
{
   int rw;
   int rh;
   int rd;
   int rt;
   int re;
   int num;


   /* if the segment does not exist, then it has no region info */
   if (!kdms_query_segment(obj, (char *) client_data))
      return FALSE;		/* not an error */

   /* the region primitive varies for every segment */
   if (kstrcmp((char *)client_data, KDMS_SEGMENT_VALUE) == 0)
   {

      kpds_get_attribute(obj, KPDS_VALUE_REGION_INFO, 
			 &rw, &rh, &rd, &rt, &re, &num);
      if (outfile)
	 kfprintf(outfile, "%d x %d x %d x %d x %d\t"
		  "%d", rw, rh, rd, rt, re, num);

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_MASK) == 0)
   {

      kpds_get_attribute(obj, KPDS_MASK_REGION_INFO, 
			 &rw, &rh, &rd, &rt, &re, &num);
      if (outfile)
	 kfprintf(outfile, "%d x %d x %d x %d x %d\t"
		  "%d", rw, rh, rd, rt, re, num);

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_LOCATION) == 0)
   {

      kpds_get_attribute(obj, KPDS_LOCATION_REGION_INFO,
			 &rw, &rh, &re, &re, &num);
      if (outfile)
	 kfprintf(outfile, "%d x %d x %d x %d\t"
		  "%d", rw, rh, rd, rt, re, num);

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_TIME) == 0)
   {

      kpds_get_attribute(obj, KPDS_TIME_REGION_INFO, &rt, &num);
      if (outfile)
	 kfprintf(outfile, "%d\t"
		  "%d", rt, num);

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_MAP) == 0)
   {

      kpds_get_attribute(obj, KPDS_MAP_REGION_INFO, 
			 &rw, &rh, &re, &rd, &rt, &num);
      if (outfile)
	 kfprintf(outfile, "%d x %d x %d x %d x %d\t"
		  "%d", rw, rh, re, rt, rd, num);
   }
   else
      return FALSE;

   return TRUE;
}


/*********** ---------------------------------------------------------------
 ***********  KPDS_OPTIMAL_REGION_CONSTRAINT
 *********** --------------------------------------------------------------- */

/*-----------------------------------------------------------
|  Routine Name: (static) _optimal_region_constraint_get
|       Purpose: This routine retrieves the "region constraint information".
|    Written By: Jeremy Worley
|          Date: December 8, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_optimal_region_constraint_get(kobject obj, ktoken segment, ktoken attr, 
			       kaddr client_data, kva_list *list)
{
   int *constraint = NULL;
   int dim;
   int i;
   int *tmp;
   
   kdms_get_attributes(obj, (char *)client_data, 
		       _INTERNAL_REGION_CONSTRAINT, &constraint,
		       KDMS_DIMENSION, &dim, NULL);
   
   for (i = 0; i < dim; i++)
      if ((tmp = kva_arg(*list, int *)) != NULL)
	 (*tmp) = constraint[i];

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _optimal_region_constraint_set
|       Purpose: This routine stores the "region constraint information".
|    Written By: Jeremy Worley
|          Date: December 8, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_optimal_region_constraint_set(kobject obj, ktoken segment, ktoken attr, kaddr client_data,
		 kva_list *list)
{
   int *constraint = NULL;
   int dim;
   int i;
   
   if (!kdms_get_attributes(obj, (char *)client_data, 
		       _INTERNAL_REGION_CONSTRAINT, &constraint,
		       KDMS_DIMENSION, &dim, NULL))
      return FALSE;
   
   for (i = 0; i < dim; i++)
      constraint[i] = kva_arg(*list, int);
   
   if (!kdms_set_attribute(obj, (char *)client_data, 
		       _INTERNAL_REGION_CONSTRAINT, constraint))
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _optimal_region_constraint_match
|       Purpose: match the region info attribute
|    Written By: Jeremy Worley 
|          Date: December 8, 1994 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_optimal_region_constraint_match(kobject obj1, kobject obj2, ktoken segment, ktoken attribute,
		   kaddr client_data1, kaddr client_data2)
{
   int *constraint1 = NULL;
   int dim1;
   int *constraint2 = NULL;
   int dim2;
   int i;
   
   kdms_get_attributes(obj1, (char *)client_data1, 
		       _INTERNAL_REGION_CONSTRAINT, &constraint1,
		       KDMS_DIMENSION, &dim1, NULL);

   kdms_get_attributes(obj2, (char *)client_data2, 
		       _INTERNAL_REGION_CONSTRAINT, &constraint2,
		       KDMS_DIMENSION, &dim2, NULL);
 
   if (dim1 != dim2)
      return (FALSE);
   
   for (i = 0; i < dim1; i++)
      if (constraint1[i] != constraint2[i])
	 return(FALSE);
   
   return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _optimal_region_constraint_query
|       Purpose: query the region info attribute
|    Written By: Jeremy Worley 
|          Date: December 8, 1994 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_optimal_region_constraint_query(kobject obj, ktoken segment, ktoken attribute, kaddr client_data,
		   int *num_args, int *arg_size, int *data_type,
		   int *permanent)
{
   /* if the segment does not exist, then it has no region info */
   if (!kdms_query_segment(obj, (char *) client_data))
      return FALSE;		/* not an error */

   if (arg_size)
      *arg_size = 1;
   if (data_type)
      *data_type = KINT;
   if (permanent)
      *permanent = FALSE;

   /* the num_args of the region info primitive varies for every segment */
   if (kstrcmp((char *)client_data, KDMS_SEGMENT_VALUE) == 0)
   {
      if (num_args)
	 *num_args = 5;
   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_MASK) == 0)
   {
      if (num_args)
	 *num_args = 5;
   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_LOCATION) == 0)
   {
      if (num_args)
	 *num_args = 4;
   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_TIME) == 0)
   {
      if (num_args)
	 *num_args = 1;
   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_MAP) == 0)
   {
      if (num_args)
	 *num_args = 5;
   }
   else
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _optimal_region_constraint_print
|       Purpose: print the region info attribute
|    Written By: Jeremy Worley 
|          Date: December 8, 1994 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_optimal_region_constraint_print(kobject obj, ktoken segment, ktoken attribute, kaddr client_data,
		   kfile *outfile)
{
   int rw;
   int rh;
   int rd;
   int rt;
   int re;

   /* if the segment does not exist, then it has no region info */
   if (!kdms_query_segment(obj, (char *) client_data))
      return FALSE;		/* not an error */

   /* the region primitive varies for every segment */
   if (kstrcmp((char *)client_data, KDMS_SEGMENT_VALUE) == 0)
   {

      kpds_get_attribute(obj, KPDS_VALUE_OPTIMAL_REGION_CONSTRAINT, 
			 &rw, &rh, &rd, &rt, &re);
      if (outfile)
	 kfprintf(outfile, "%d x %d x %d x %d x %d\t",
		  rw, rh, rd, rt, re);

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_MASK) == 0)
   {

      kpds_get_attribute(obj, KPDS_MASK_OPTIMAL_REGION_CONSTRAINT, 
			 &rw, &rh, &rd, &rt, &re);
      if (outfile)
	 kfprintf(outfile, "%d x %d x %d x %d x %d\t",
		  rw, rh, rd, rt, re);

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_LOCATION) == 0)
   {

      kpds_get_attribute(obj, KPDS_LOCATION_OPTIMAL_REGION_CONSTRAINT,
			 &rw, &rh, &re, &re);
      if (outfile)
	 kfprintf(outfile, "%d x %d x %d x %d\t",
		  rw, rh, rd, rt, re);

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_TIME) == 0)
   {

      kpds_get_attribute(obj, KPDS_TIME_OPTIMAL_REGION_CONSTRAINT, &rt);
      if (outfile)
	 kfprintf(outfile, "%d\t", rt);

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_MAP) == 0)
   {

      kpds_get_attribute(obj, KPDS_MAP_OPTIMAL_REGION_CONSTRAINT, 
			 &rw, &rh, &re, &rd, &rt);
      if (outfile)
	 kfprintf(outfile, "%d x %d x %d x %d x %d\t"
		  "%d", rw, rh, re, rt, rd);
   }
   else
      return FALSE;

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_OPTIMAL_REGION_FLAGS
 *********** --------------------------------------------------------------- */

/*-----------------------------------------------------------
|  Routine Name: (static) _optimal_region_flags_get
|       Purpose: This routine retrieves the "region flags information".
|    Written By: Jeremy Worley
|          Date: December 8, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_optimal_region_flags_get(kobject obj, ktoken segment, ktoken attr, kaddr client_data,
		 kva_list *list)
{
   int *flags = NULL;
   int dim;
   int i;
   int *tmp;
   
   kdms_get_attributes(obj, (char *)client_data, 
		       _INTERNAL_REGION_FLAGS, &flags,
		       KDMS_DIMENSION, &dim, NULL);
   
   for (i = 0; i < dim; i++)
      if ((tmp = kva_arg(*list, int *)) != NULL)
	 (*tmp) = flags[i];

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _optimal_region_flags_set
|       Purpose: This routine stores the "region flags information".
|    Written By: Jeremy Worley
|          Date: December 8, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_optimal_region_flags_set(kobject obj, ktoken segment, ktoken attr, kaddr client_data,
		 kva_list *list)
{
   int *flags = NULL;
   int dim;
   int i;
   
   if (!kdms_get_attributes(obj, (char *)client_data, 
		       _INTERNAL_REGION_FLAGS, &flags,
		       KDMS_DIMENSION, &dim, NULL))
      return FALSE;
   
   for (i = 0; i < dim; i++)
      flags[i] = kva_arg(*list, int);
   
   if (!kdms_set_attribute(obj, (char *)client_data, 
		       _INTERNAL_REGION_FLAGS, flags))
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _optimal_region_flags_match
|       Purpose: match the region info attribute
|    Written By: Jeremy Worley 
|          Date: December 8, 1994 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_optimal_region_flags_match(kobject obj1, kobject obj2, ktoken segment, ktoken attribute,
		   kaddr client_data1, kaddr client_data2)
{
   int *flags1 = NULL;
   int dim1;
   int *flags2 = NULL;
   int dim2;
   int i;
   
   kdms_get_attributes(obj1, (char *)client_data1, 
		       _INTERNAL_REGION_FLAGS, &flags1,
		       KDMS_DIMENSION, &dim1, NULL);

   kdms_get_attributes(obj2, (char *)client_data2, 
		       _INTERNAL_REGION_FLAGS, &flags2,
		       KDMS_DIMENSION, &dim2, NULL);
 
   if (dim1 != dim2)
      return (FALSE);
   
   for (i = 0; i < dim1; i++)
      if (flags1[i] != flags2[i])
	 return(FALSE);
   
   return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _optimal_region_flags_query
|       Purpose: query the region info attribute
|    Written By: Jeremy Worley 
|          Date: December 8, 1994 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_optimal_region_flags_query(kobject obj, ktoken segment, ktoken attribute, kaddr client_data,
		   int *num_args, int *arg_size, int *data_type,
		   int *permanent)
{
   /* if the segment does not exist, then it has no region info */
   if (!kdms_query_segment(obj, (char *) client_data))
      return FALSE;		/* not an error */

   if (arg_size)
      *arg_size = 1;
   if (data_type)
      *data_type = KINT;
   if (permanent)
      *permanent = FALSE;

   /* the num_args of the region info primitive varies for every segment */
   if (kstrcmp((char *)client_data, KDMS_SEGMENT_VALUE) == 0)
   {
      if (num_args)
	 *num_args = 5;
   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_MASK) == 0)
   {
      if (num_args)
	 *num_args = 5;
   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_LOCATION) == 0)
   {
      if (num_args)
	 *num_args = 4;
   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_TIME) == 0)
   {
      if (num_args)
	 *num_args = 1;
   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_MAP) == 0)
   {
      if (num_args)
	 *num_args = 5;
   }
   else
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _optimal_region_flags_print
|       Purpose: print the region info attribute
|    Written By: Jeremy Worley 
|          Date: December 8, 1994 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_optimal_region_flags_print(kobject obj, ktoken segment, ktoken attribute, kaddr client_data,
		   kfile *outfile)
{
   int rw;
   int rh;
   int rd;
   int rt;
   int re;

   /* if the segment does not exist, then it has no region info */
   if (!kdms_query_segment(obj, (char *) client_data))
      return FALSE;		/* not an error */

   /* the region primitive varies for every segment */
   if (kstrcmp((char *)client_data, KDMS_SEGMENT_VALUE) == 0)
   {

      kpds_get_attribute(obj, KPDS_VALUE_OPTIMAL_REGION_FLAGS, 
			 &rw, &rh, &rd, &rt, &re);
      if (outfile)
	 kfprintf(outfile, "%d, %d, %d, %d, %d\t",
		  rw, rh, rd, rt, re);

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_MASK) == 0)
   {

      kpds_get_attribute(obj, KPDS_MASK_OPTIMAL_REGION_FLAGS, 
			 &rw, &rh, &rd, &rt, &re);
      if (outfile)
	 kfprintf(outfile, "%d, %d, %d, %d, %d\t",
		  rw, rh, rd, rt, re);

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_LOCATION) == 0)
   {

      kpds_get_attribute(obj, KPDS_LOCATION_OPTIMAL_REGION_FLAGS,
			 &rw, &rh, &re, &re);
      if (outfile)
	 kfprintf(outfile, "%d, %d, %d, %d\t",
		  rw, rh, rd, rt, re);

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_TIME) == 0)
   {

      kpds_get_attribute(obj, KPDS_TIME_OPTIMAL_REGION_FLAGS, &rt);
      if (outfile)
	 kfprintf(outfile, "%d\t", rt);

   }
   else if (kstrcmp((char *)client_data, KDMS_SEGMENT_MAP) == 0)
   {

      kpds_get_attribute(obj, KPDS_MAP_OPTIMAL_REGION_FLAGS, 
			 &rw, &rh, &re, &rd, &rt);
      if (outfile)
	 kfprintf(outfile, "%d, %d, %d, %d, %d\t"
		  "%d", rw, rh, re, rt, rd);
   }
   else
      return FALSE;

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_OPTIMAL_REGION_SIZE
 *********** --------------------------------------------------------------- */

/*-----------------------------------------------------------
|  Routine Name: (static) _optimal_region_size_get
|       Purpose: get the optimal region size
|    Written By: Steve Kubica & Jeremy Worley
|          Date: April 6, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_optimal_region_size_get(kobject obj, ktoken segment, ktoken attr, kaddr client_data,
			 kva_list *list)
{
   int *size = NULL;
   int *opt_size = NULL;
   int *constraint = NULL;
   int *flags = NULL;
   int *tmp = NULL;
   int  num_regions;
   int  dim;
   int  i, j, k;
   int  done;
   int  tmp_size[KDMS_MAX_DIM];

   /* if the segment does not exist, then it has no optimum region size */
   if (!kdms_query_segment(obj, (char *) client_data))
      return FALSE;		/* not an error */

   /*-- we must be aware of grid types if we have location data --*/
   if (kstrcmp((char *)client_data, KPDS_SEGMENT_LOCATION) == 0)
   {
      int grid = kpds_location_grid(obj);
      int w,h,d,n;
   
      
      if (grid == KUNIFORM || grid == KRECTILINEAR)
      {
	 /*-- return the whole thing for now --*/
	 kpds_get_attribute(obj, KPDS_LOCATION_SIZE, &w, &h, &d, &n);

	 /*-- first return the region size --*/
	 if ((tmp = kva_arg(*list, int *)) != NULL) (*tmp) = w;
	 if ((tmp = kva_arg(*list, int *)) != NULL) (*tmp) = h;
	 if ((tmp = kva_arg(*list, int *)) != NULL) (*tmp) = d;
	 if ((tmp = kva_arg(*list, int *)) != NULL) (*tmp) = n;
	 
	 /*-- number of regions will be 1 since region size is everything --*/
	 if ((tmp = kva_arg(*list, int *)) != NULL) (*tmp) = 1;
	 
	 return TRUE;
      }

      /*-- KCURVILINEAR data can just fall through to the below code --*/
   }

   /* 
    * get the kdms optimal region size for this region.  note that 
    * this attribute is not mapping aware, but that's ok for right now
    * because it doesn't need to be.   the last element in this array
    * is conveniently going to be the number of optimum regions
    */
   if (!kdms_get_attributes(obj, (char *) client_data,
			    KDMS_OPTIMAL_REGION_SIZE, &opt_size,
			    KDMS_SIZE, &size,
			    _INTERNAL_REGION_CONSTRAINT, &constraint,
			    _INTERNAL_REGION_FLAGS, &flags,
			    KDMS_DIMENSION, &dim, NULL))
      return FALSE;

   /*
    * copy the size because otherwise it'll modify the presentation
    * size if mapping mode is set.
    */
   kmemcpy(&(tmp_size[0]),size, dim * sizeof(int));
   size = &(tmp_size[0]);
   
   /*
    * if the mapping mode is on, then we want to multiply the number of
    * "optimal regions" by the number size of the map_width dimension in
    * the map segment.
    */
   num_regions = opt_size[dim];

   if (kpds_is_mapped(obj))
   {
      if ( (kstrcmp((char *) client_data, KDMS_SEGMENT_VALUE) == 0) ||
	   (kstrcmp((char *) client_data, KDMS_SEGMENT_MASK)  == 0))
      {
	 int *msize;
	 

	 kdms_get_attribute(obj, KDMS_SEGMENT_MAP, KDMS_SIZE, &msize);
	 num_regions *= msize[KMAP_WID];
	 size[KELM] *= msize[KMAP_WID];
      }
   }

   /*
    * ok.  now lets mess with the optimal region size based on the
    * constraints and constraint flags.
    */
   for (i = 0; i < dim; i++)
   {
      if (flags[i] == KOPTIMAL_FACTOR) /* factor */
      {
         /* subtract off what doesn't fit */
	 opt_size[i] -= (opt_size[i] % constraint[i]); 
	 opt_size[i] = kmax(opt_size[i], constraint[i]);
      }
      else /* KOPTIMAL_FIXED_VALUE : fixed value */ 
      {
	 /*-- comment out cause this might make the i+1 optimal size not
	   -- not be a factor of the i+1 size! --*/
#if 0
	 if (i < dim - 1 && constraint[i] < size[i])
	 {
	    opt_size[i+1] *= kmax(1, opt_size[i] / constraint[i]);
	    opt_size[i+1] = kmin(size[i+1], opt_size[i+1]);
	 }
#endif
	 /* 
	  * if our constraint is larger than what we had, then try
	  * to steal some space from a prior dimension.
	  */
	 if (constraint[i] > opt_size[i])
	 {
	    done = FALSE;
	    for (k = constraint[i]; k < 2*constraint[i] && !done; k++)
	       for (j = i-1; j >= 0 && !done; j--)
		  if (opt_size[j] % k == 0)
		     opt_size[j] /= k, done = TRUE;
	 }
	 opt_size[i] = constraint[i];
      }
   }

   /*
    * recompute the number of optimal regions because its been
    * hopelessly munged by the last round of twiddles.
    */
   for (i = 0, num_regions = 1; i < dim; i++)
      num_regions *= (int)(kceil((float)size[i]/(float)opt_size[i]));
   
	       
   /* first return the region size */
   for (i = 0; i < dim; i++)
      if ((tmp = kva_arg(*list, int *)) != NULL)
	    (*tmp) = opt_size[i];

   /* now get the number of those regions */
   if ((tmp = kva_arg(*list, int *)) != NULL)
         (*tmp) = num_regions;

   /*
    * After everything is done, check opt_size to make sure that none
    * of the values are zero.  Do it here so that if plural
    * get_attributes is called, later attributes will get set
    * properly.  Issue a warning message if necessary.
    */
   for (i = 0; i < dim; i++)
      if (opt_size[i] <= 0)
      {
	 kinfo(KSTANDARD,"Computed optimal region is invalid.  This is "
	       "probably due to an uninitialized size attribute or "
	       "illegal optimal region constraints.  Please make sure that "
	       "the size attribute has been properly initialized.\n");
	 return FALSE;
      }
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _optimal_regon_size_match
|       Purpose: match the optimal_region_size attribute
|    Written By: 
|          Date: 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_optimal_region_size_match(kobject obj1, kobject obj2, ktoken segment, ktoken attribute,
			   kaddr client_data1, kaddr client_data2)
{
   return (kdms_match_attribute(obj1, obj2, (char *) client_data1, 
				KDMS_OPTIMAL_REGION_SIZE));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _optimal_region_size_query
|       Purpose: query the optimal_region_size attribute
|    Written By: 
|          Date: 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_optimal_region_size_query(kobject obj, ktoken segment, ktoken attribute,
			   kaddr client_data, int *num_args, int *arg_size,
			   int *data_type, int *permanent)
{
   return (kdms_query_attribute(obj, (char *) client_data, 
				KDMS_OPTIMAL_REGION_SIZE,
				num_args, arg_size, data_type, permanent));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _optimal_region_size_print
|       Purpose: print the optimal_region_size attribute
|    Written By: 
|          Date: 
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_optimal_region_size_print(kobject obj, ktoken segment, ktoken attribute,
			   kaddr client_data, kfile *outfile)
{
   return (kdms_print_attribute(obj, (char *) client_data,
				KDMS_OPTIMAL_REGION_SIZE, outfile));
}


/*
 *    ==================================================================
 *    Initialization Routine
 *    ==================================================================
 */

/*-----------------------------------------------------------
|
|  Routine Name: kpds_init_info_attributes
|
|       Purpose: 
|
|         Input: 
|
|        Output: 
|
|       Returns: nothing
|
|    Written By: Jeremy Worley & John Salas
|          Date: September 28, 1993
| Modifications:
|
------------------------------------------------------------*/
void
kpds_init_info_attributes(void)
{
   static int zeros [KDMS_MAX_DIM] = {0, 0, 0, 0, 0};
   static int ones  [KDMS_MAX_DIM] = {1, 1, 1, 1, 1};

   /* 
    *   region size attributes
    */
   kdms_define_attribute(KDMS_OBJECT, KPDS_MASK_REGION_SIZE, 5, 1, KINT,
			 FALSE, FALSE, 0, 0, 0, 0, 0);

   kdms_define_attribute(KDMS_OBJECT, KPDS_VALUE_REGION_SIZE, 5, 1, KINT,
			 FALSE, FALSE, 0, 0, 0, 0, 0);

   kdms_define_attribute(KDMS_OBJECT, KPDS_MAP_REGION_SIZE, 5, 1, KINT,
			 FALSE, FALSE, 0, 0, 0, 0, 0);

   kdms_define_attribute(KDMS_OBJECT, KPDS_LOCATION_REGION_SIZE, 4, 1, KINT,
			 FALSE, FALSE, 0, 0, 0, 0);

   kdms_define_attribute(KDMS_OBJECT, KPDS_TIME_REGION_SIZE, 1, 1, KINT,
			 FALSE, FALSE, 0);

   /* 
    *  line info attributes
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MASK_LINE_INFO,
			       pds_mask_string,
			       _line_info_get, _nop_set, _line_info_match,
			       _nop_copy, _line_info_query, _line_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_VALUE_LINE_INFO,
			       pds_value_string,
			       _line_info_get, _nop_set, _line_info_match,
			       _nop_copy, _line_info_query, _line_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MAP_LINE_INFO,
			       pds_map_string,
			       _line_info_get, _nop_set, _line_info_match,
			       _nop_copy, _line_info_query, _line_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_LOCATION_LINE_INFO,
			       pds_location_string,
			       _line_info_get, _nop_set, _line_info_match,
			       _nop_copy, _line_info_query, _line_info_print);


   /* 
    *   plane info attributes
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MASK_PLANE_INFO,
			       pds_mask_string,
			       _plane_info_get, _nop_set, _plane_info_match,
			       _nop_copy, _plane_info_query, 
			       _plane_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_VALUE_PLANE_INFO,
			       pds_value_string,
			       _plane_info_get, _nop_set, _plane_info_match,
			       _nop_copy, _plane_info_query, 
			       _plane_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MAP_PLANE_INFO,
			       pds_map_string,
			       _plane_info_get, _nop_set, _plane_info_match,
			       _nop_copy, _plane_info_query, 
			       _plane_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_LOCATION_PLANE_INFO,
			       pds_location_string,
			       _plane_info_get, _nop_set, _plane_info_match,
			       _nop_copy, _plane_info_query, 
			       _plane_info_print);

   /* 
    *   volume info attributes
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MASK_VOLUME_INFO,
			       pds_mask_string,
			       _volume_info_get, _nop_set, _volume_info_match,
			       _nop_copy, _volume_info_query, 
			       _volume_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_VALUE_VOLUME_INFO,
			       pds_value_string,
			       _volume_info_get, _nop_set, _volume_info_match,
			       _nop_copy, _volume_info_query, 
			       _volume_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MAP_VOLUME_INFO,
			       pds_map_string,
			       _volume_info_get, _nop_set, _volume_info_match,
			       _nop_copy, _volume_info_query, 
			       _volume_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_LOCATION_VOLUME_INFO,
			       pds_location_string,
			       _volume_info_get, _nop_set, _volume_info_match,
			       _nop_copy, _volume_info_query, 
			       _volume_info_print);

   /* 
    *   region info attributes
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MASK_REGION_INFO,
			       pds_mask_string,
			       _region_info_get, _nop_set, _region_info_match,
			       _nop_copy, _region_info_query, 
			       _region_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_VALUE_REGION_INFO,
			       pds_value_string,
			       _region_info_get, _nop_set, _region_info_match,
			       _nop_copy, _region_info_query,
			       _region_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MAP_REGION_INFO,
			       pds_map_string,
			       _region_info_get, _nop_set, _region_info_match,
			       _nop_copy, _region_info_query, 
			       _region_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_LOCATION_REGION_INFO,
			       pds_location_string,
			       _region_info_get, _nop_set, _region_info_match,
			       _nop_copy, _region_info_query, 
			       _region_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_TIME_REGION_INFO,
			       pds_time_string,
			       _region_info_get, _nop_set, _region_info_match,
			       _nop_copy, _region_info_query, 
			       _region_info_print);

   /* 
    *   vector info attributes
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MASK_VECTOR_INFO,
			       pds_mask_string,
			       _vector_info_get, _nop_set, _vector_info_match,
			       _nop_copy, _vector_info_query, 
			       _vector_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_VALUE_VECTOR_INFO,
			       pds_value_string,
			       _vector_info_get, _nop_set, _vector_info_match,
			       _nop_copy, _vector_info_query,
			       _vector_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MAP_VECTOR_INFO,
			       pds_map_string,
			       _vector_info_get, _nop_set, _vector_info_match,
			       _nop_copy, _vector_info_query, 
			       _vector_info_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_LOCATION_VECTOR_INFO,
			       pds_location_string,
			       _vector_info_get, _nop_set, _vector_info_match,
			       _nop_copy, _vector_info_query, 
			       _vector_info_print);

   /*
    * optimal_region attributes
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_LOCATION_OPTIMAL_REGION_SIZE,
			       pds_location_string,
			       _optimal_region_size_get, _nop_set,
			       _optimal_region_size_match, _nop_copy,
			       _optimal_region_size_query, 
			       _optimal_region_size_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MAP_OPTIMAL_REGION_SIZE,
			       pds_map_string,
			       _optimal_region_size_get, _nop_set,
			       _optimal_region_size_match, _nop_copy,
			       _optimal_region_size_query, 
			       _optimal_region_size_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MASK_OPTIMAL_REGION_SIZE,
			       pds_mask_string,
			       _optimal_region_size_get, _nop_set,
			       _optimal_region_size_match, _nop_copy,
			       _optimal_region_size_query, 
			       _optimal_region_size_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_TIME_OPTIMAL_REGION_SIZE,
			       pds_time_string,
			       _optimal_region_size_get, _nop_set,
			       _optimal_region_size_match, _nop_copy,
			       _optimal_region_size_query, 
			       _optimal_region_size_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_VALUE_OPTIMAL_REGION_SIZE,
			       pds_value_string,
			       _optimal_region_size_get, _nop_set,
			       _optimal_region_size_match, _nop_copy,
			       _optimal_region_size_query, 
			       _optimal_region_size_print);

   /*
    * optimal_region_constraint attributes
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_LOCATION_OPTIMAL_REGION_CONSTRAINT,
			       pds_location_string,
			       _optimal_region_constraint_get,
			       _optimal_region_constraint_set,
			       _optimal_region_constraint_match, _nop_copy,
			       _optimal_region_constraint_query, 
			       _optimal_region_constraint_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MAP_OPTIMAL_REGION_CONSTRAINT,
			       pds_map_string,
			       _optimal_region_constraint_get,
			       _optimal_region_constraint_set,
			       _optimal_region_constraint_match, _nop_copy,
			       _optimal_region_constraint_query, 
			       _optimal_region_constraint_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MASK_OPTIMAL_REGION_CONSTRAINT,
			       pds_mask_string,
			       _optimal_region_constraint_get,
			       _optimal_region_constraint_set,
			       _optimal_region_constraint_match, _nop_copy,
			       _optimal_region_constraint_query, 
			       _optimal_region_constraint_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_TIME_OPTIMAL_REGION_CONSTRAINT,
			       pds_time_string,
			       _optimal_region_constraint_get,
			       _optimal_region_constraint_set,
			       _optimal_region_constraint_match, _nop_copy,
			       _optimal_region_constraint_query, 
			       _optimal_region_constraint_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_VALUE_OPTIMAL_REGION_CONSTRAINT,
			       pds_value_string,
			       _optimal_region_constraint_get,
			       _optimal_region_constraint_set,
			       _optimal_region_constraint_match, _nop_copy,
			       _optimal_region_constraint_query, 
			       _optimal_region_constraint_print);

   /*
    * optimal_region_flags attributes
    */
   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_LOCATION_OPTIMAL_REGION_FLAGS,
			       pds_location_string,
			       _optimal_region_flags_get,
			       _optimal_region_flags_set,
			       _optimal_region_flags_match, _nop_copy,
			       _optimal_region_flags_query, 
			       _optimal_region_flags_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MAP_OPTIMAL_REGION_FLAGS,
			       pds_map_string,
			       _optimal_region_flags_get,
			       _optimal_region_flags_set,
			       _optimal_region_flags_match, _nop_copy,
			       _optimal_region_flags_query, 
			       _optimal_region_flags_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MASK_OPTIMAL_REGION_FLAGS,
			       pds_mask_string,
			       _optimal_region_flags_get,
			       _optimal_region_flags_set,
			       _optimal_region_flags_match, _nop_copy,
			       _optimal_region_flags_query, 
			       _optimal_region_flags_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_TIME_OPTIMAL_REGION_FLAGS,
			       pds_time_string,
			       _optimal_region_flags_get,
			       _optimal_region_flags_set,
			       _optimal_region_flags_match, _nop_copy,
			       _optimal_region_flags_query, 
			       _optimal_region_flags_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_VALUE_OPTIMAL_REGION_FLAGS,
			       pds_value_string,
			       _optimal_region_flags_get,
			       _optimal_region_flags_set,
			       _optimal_region_flags_match, _nop_copy,
			       _optimal_region_flags_query, 
			       _optimal_region_flags_print);

   /*
    * region constraint internal variables
    */
   kdms_define_attribute(KDMS_ALL_SEGMENTS, _INTERNAL_REGION_CONSTRAINT, 
					    1, 5, KINT, FALSE, FALSE, ones);

   kdms_define_attribute(KDMS_ALL_SEGMENTS, _INTERNAL_REGION_FLAGS, 
					    1, 5, KINT, FALSE, FALSE, zeros);

   /*
    *   histogram attributes 
    */
   kdms_define_attribute(KDMS_OBJECT, KPDS_VALUE_HIST_REGION_SIZE, 5, 1, KINT,
			 FALSE, FALSE, 1, 1, 1, 1, 1);
   kdms_define_attribute(KDMS_OBJECT, KPDS_VALUE_HIST_POSITION, 5, 1, KINT,
			 FALSE, FALSE, 0, 0, 0, 0, 0);
   kdms_define_attribute(KDMS_OBJECT, KPDS_VALUE_HIST_NUMBINS, 1, 1, KINT,
			 FALSE, FALSE, 256);
   kdms_define_attribute(KDMS_OBJECT, KPDS_VALUE_HIST_RANGE, 2, 1, KDOUBLE,
			 FALSE, FALSE, 0.0, 1.0);

   return;
}
