 /*
  * Khoros: $Id: internals.h,v 1.11.2.3 1997/04/08 22:36:54 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Application Services Internals
   >>>>
   >>>>    Written By: Jeremy Worley and Steve Kubica
   >>>>
   >>>>          Date: Oct 07, 1993 13:41
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _internals_h_
#define _internals_h_


	/*-------------------------------------*
	|       #include 
	---------------------------------------*/

#include <dataserv.h>


	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

#define KPDS_SEGMENT_VALUE 	KDMS_SEGMENT_VALUE
#define KPDS_SEGMENT_MASK 	KDMS_SEGMENT_MASK
#define KPDS_SEGMENT_LOCATION 	KDMS_SEGMENT_LOCATION
#define KPDS_SEGMENT_TIME 	KDMS_SEGMENT_TIME 
#define KPDS_SEGMENT_MAP 	KDMS_SEGMENT_MAP

#define KPDS_SEGMENT_WIDTH 	"width"
#define KPDS_SEGMENT_HEIGHT 	"height"
#define KPDS_SEGMENT_DEPTH 	"depth"
#define KPDS_SEGMENT_UNIFORM	"uniform"

#define KPDS_ACCESS_INFO 	      "internalAccessInfo"

#define _STORED_LOCATION_GRID 	      "locationGrid"

#define _INTERNAL_MAP_AUTOCOLOR         "mapAutocolor"
#define _INTERNAL_MAP_OPERATION 	"mapAutocolorOperation"
#define _INTERNAL_COLORSPACE          	"colorspace"
#define _INTERNAL_HISTORY       	"history"

#define _INTERNAL_REGION_CONSTRAINT     "internalRegionConstraint"
#define _INTERNAL_REGION_FLAGS 		"internalRegionFlags"

/* -- uniform and rectilinear location data attributes -- */
#define KPDS_UNIFORM_PRES_LOCATION_SIZE "uniformPresSize"
#define KPDS_UNIFORM_PHYS_LOCATION_SIZE "uniformSize"
#define KPDS_RECTILINEAR_PRES_DIMENSION "rectilinearPresDimension"
#define KPDS_RECTILINEAR_PHYS_DIMENSION "rectilinearDimension"

/* -- polymorphic segment dimensions -- */
#define KPDS_VALUE_DIM     5
#define KPDS_MASK_DIM      5
#define KPDS_LOCATION_DIM  4
#define KPDS_TIME_DIM      1
#define KPDS_MAP_DIM       5

/* -- predefined indices into size, position, and offset arrays -- */
#define KWID  0
#define KHGT  1
#define KDEP  2
#define KTIM  3
#define KELM  4

#define KDIM  3

#define KMAP_WID  0
#define KMAP_HGT  1
#define KMAP_DEP  2
#define KMAP_TIM  3
#define KMAP_ELM  4

	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/

typedef struct _kpds_primitive 
{
   kstring name;

   ktoken  name_token;

   kstring segment;
   ktoken  segment_token; 

   kstring size_attribute; 
   int     usage[KDAS_MAX_DIM];
   kstring region_attribute;

   int    (*get_size)       PROTO((kobject, struct _kpds_primitive *, int **));
   void   (*find_corners)   PROTO((kobject, struct _kpds_primitive *, int *, 
				   int *, int *));
   void   (*offset_corners) PROTO((kobject, struct _kpds_primitive *, int *, 
				   int, int *, int *));

   kaddr  (*get)  	    PROTO((kobject, struct _kpds_primitive *, kaddr));
   int    (*put)  	    PROTO((kobject, struct _kpds_primitive *, kaddr));

} kpds_primitive;

	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/


	/*-------------------------------------*
	|       macros
	---------------------------------------*/

	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/* formats */
void init_ascii PROTO((void));
void init_arf   PROTO((void));
void init_avs   PROTO((void));
void init_eps   PROTO((void));
void init_gdbm  PROTO((void));
void init_pnm   PROTO((void));
void init_rast  PROTO((void));
void init_viff  PROTO((void));
void init_xbm   PROTO((void));
void init_xpm   PROTO((void));
void init_xwd   PROTO((void));


/* attributes.c */
void kpds_init_attributes            PROTO((void));
int  kpds_intelligent_vset_attribute PROTO((kobject, char *, ktoken, 
					    kva_list *));
int  kpds_intelligent_vget_attribute PROTO((kobject, char *, ktoken, 
					    kva_list *));

/* autocolor.c */
void kmap_map_autocolor PROTO((kobject, int));
void kmap_map_operation PROTO((kobject, int));

/* data.c */
void kpds_init_primitive_table PROTO((void));

/* copy.c */
int kaps_copy_through_pres 	PROTO((kobject, kobject, char *));
int kaps_copy_beneath_pres 	PROTO((kobject, kobject, char *));
int kaps_copy_object            PROTO((kobject, kobject, int, int));
int kaps_copy_global_attributes PROTO((kobject, kobject));

/* error.c */
void kpds_init_errors  PROTO((void));

/* global.c */
void kpds_init_segment_tokens PROTO((void));

/* info.c */
void kpds_init_info_attributes PROTO((void));

/* mapping.c */
int   kpds_map_data                 PROTO((int *, int *, int *, kaddr, int *, 
					   int, int, kaddr, double, double));
kaddr kpds_get_mapped_value_data    PROTO((kobject, int *, int *, kaddr));
kaddr kpds_get_mapped_mask_data     PROTO((kobject, int *, int *, kaddr));
kaddr kpds_mask_data                PROTO((kobject, int *, int *, kaddr));
kaddr kpds_get_uniform_data         PROTO((kobject, int *, int *, kaddr));
kaddr kpds_get_rectilinear_data     PROTO((kobject, int *, int *, kaddr));
kaddr kpds_transpose_elements_first PROTO((kaddr, int *, int *, int, int));
kaddr kpds_transpose_elements_last  PROTO((kaddr, int *, int *, int, int));

/* position.c */
int  kpds_get_dimension   PROTO((kobject, ktoken));
int  kpds_get_size        PROTO((kobject, ktoken, int **));
int  kpds_get_order       PROTO((kobject, ktoken, int **));
int  kpds_is_mapped       PROTO((kobject));
int  kpds_is_masked       PROTO((kobject));
int  kpds_location_grid   PROTO((kobject));
int  kpds_elements_first  PROTO((kobject));
void kpds_init_position_attributes PROTO((void));

/* segment initializion */
void kpds_init_value_segment 	PROTO((void));
void kpds_init_mask_segment  	PROTO((void));
void kpds_init_map_segment   	PROTO((void));
void kpds_init_location_segment PROTO((void));
void kpds_init_time_segment     PROTO((void));

/* primitive.c */
void kpds_init_primitive_table PROTO((void));

/* utilities.c */
void  kaps_current_date    PROTO((char *));
char *kstring_append       PROTO((char *, char *));

#endif /* _internals_h_ */
/* Don't add after this point */
