 /*
  * Khoros: $Id: pds_map.c,v 1.11.2.3 1997/05/24 18:35:11 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: pds_map.c,v 1.11.2.3 1997/05/24 18:35:11 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Polymophic Model Data Handlers
   >>>>		
   >>>>  Private:
   >>>>                 kpds_init_map_segment()
   >>>>			kpds_set_map_size()
   >>>>			kpds_get_map_size()
   >>>>   Static:
   >>>>                 _size_get()
   >>>>                 _size_set()
   >>>>                 _size_match()
   >>>>                 _size_copy()
   >>>>                 _size_query()
   >>>>                 _size_print()
   >>>>                 _type_get()
   >>>>                 _type_set()
   >>>>                 _type_match()
   >>>>                 _type_copy()
   >>>>                 _type_query()
   >>>>                 _type_print()
   >>>>                 _create_map()
   >>>>   Public:
   >>>>                 kpds_create_map()
   >>>>                 kpds_query_map()
   >>>>                 kpds_destroy_map()
   >>>>                 kpds_initialize_map()
   >>>>                 kpds_copy_map_data()
   >>>>                 kpds_copy_map_attr()
   >>>>                 kpds_copy_map()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/*-- pds_attrib.c --*/
extern int sync_sizes;

/****    =============================================================
 ****    MASK size routines
 ****    =============================================================
 ****/

int kpds_get_map_size(kobject obj, int *w, int *h, int *d, int *t, int *e)
{
   int *size;

   if (!kdms_query_segment(obj, KPDS_SEGMENT_MAP)) 
      kdas_fail_error(KPDS_EMAP_NONEXIST);

   if (!kdms_get_attribute(obj, KPDS_SEGMENT_MAP, KDMS_SIZE, &size))
      return FALSE; 
    
   if (w) *w = size[KMAP_WID];
   if (h) *h = size[KMAP_HGT];
   if (d) *d = size[KMAP_DEP];
   if (t) *t = size[KMAP_TIM];
   if (e) *e = size[KMAP_ELM];
   
   return TRUE;
}

int kpds_set_map_size(kobject obj, int w, int h, int d, int t, int e)
{
   int  size[KPDS_MAP_DIM] = {-1, -1, -1, -1, -1};
   
   if (!kdms_query_segment(obj, KPDS_SEGMENT_MAP))
      kdas_fail_error(KPDS_EMAP_NONEXIST);

   size[KWID] = w;
   size[KHGT] = h;
   size[KDEP] = d;
   size[KTIM] = t;
   size[KELM] = e;
   
   if (sync_sizes)
   {
      sync_sizes = FALSE;  /*-- avoid calling here again --*/

      /*-- track depth, time, and element sizes only if map size != 1 --*/

      /*--set the shared size of the value segment --*/
      if (kdms_query_segment(obj, KPDS_SEGMENT_VALUE))
	 if (!kpds_set_attribute(obj, KPDS_VALUE_SIZE, -1, -1, 
				 ((d == 1) ? -1 : d), 
				 ((t == 1) ? -1 : t), 
				 ((e == 1) ? -1 : e)))
	    return FALSE;

      /*-- set the shared size of the mask segment --*/
      if (kdms_query_segment(obj, KPDS_SEGMENT_MASK))
	 if (!kpds_set_attribute(obj, KPDS_MASK_SIZE, -1, -1,
				 ((d == 1) ? -1 : d), 
				 ((t == 1) ? -1 : t), 
				 ((e == 1) ? -1 : e)))
	    return FALSE;

      /*-- set the shared size of the location segment --*/
      if (kdms_query_segment(obj, KPDS_SEGMENT_LOCATION))
	 if (!kpds_set_attribute(obj, KPDS_LOCATION_SIZE, -1, -1,
				 ((d == 1) ? -1 : d), -1))
	    return FALSE;
      
      /*-- set the shared size of the time segment --*/
      if (kdms_query_segment(obj, KPDS_SEGMENT_TIME))
	 if (!kpds_set_attribute(obj, KPDS_TIME_SIZE, ((t == 1) ? -1 : t)))
	    return FALSE;

      sync_sizes = TRUE;
   }

   /*-- set the size of the map segment --*/
   if (!kdms_set_attribute(obj, KPDS_SEGMENT_MAP, KDMS_SIZE, size))
      return FALSE;

   return TRUE;
}


/****    =============================================================
 ****    MAP attribute handlers
 ****    =============================================================
 ****/

/*********** ---------------------------------------------------------------
 ***********  KPDS_MAP_SIZE
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _size_get
|       Purpose: get the map size attribute
|    Written By: Steve Kubica and Jeremy Worley
|          Date: April 27, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_size_get(kobject obj, ktoken assoc, ktoken attr, kaddr clientData, 
	  kva_list *list)
{
   int *w = kva_arg(*list, int *);
   int *h = kva_arg(*list, int *);
   int *d = kva_arg(*list, int *);
   int *t = kva_arg(*list, int *);
   int *e = kva_arg(*list, int *);

   return kpds_get_map_size(obj, w, h, d, t, e);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_set
|       Purpose: set the map size attribute
|    Written By: Steve Kubica and Jeremy Worley
|          Date: April 27, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_size_set(kobject obj, ktoken assoc, ktoken attr, kaddr clientData, 
	  kva_list *list)
{
   int  w = kva_arg(*list, int);
   int  h = kva_arg(*list, int);
   int  d = kva_arg(*list, int);
   int  t = kva_arg(*list, int);
   int  e = kva_arg(*list, int);

   return kpds_set_map_size(obj, w, h, d, t, e);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_match
|       Purpose: match the map size attribute
|    Written By: Steve Kubica
|          Date: April 28, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_size_match(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	    kaddr clientData1, kaddr clientData2)
{
   int w1, h1, d1, t1, e1;
   int w2, h2, d2, t2, e2;

   if (!kpds_get_map_size(obj1, &w1, &h1, &d1, &t1, &e1))
      return FALSE;

   if (!kpds_get_map_size(obj2, &w2, &h2, &d2, &t2, &e2))
      return FALSE;

   return ((w1 == w2) && (h1 == h2) && (d1 == d2) && (t1 == t2) && (e1 == e2));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_copy
|       Purpose: copy the map size attribute
|    Written By: Steve Kubica
|          Date: April 28, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_size_copy(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	   kaddr clientData1, kaddr clientData2)
{
   int w, h, d, t, e;

   if (!kpds_get_map_size(obj1, &w, &h, &d, &t, &e))
      return FALSE;

   if (!kpds_set_map_size(obj2, w, h, d, t, e))
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_query
|       Purpose: query the map size attribute
|    Written By: Steve Kubica
|          Date: March 30, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_size_query(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
	    int *numargs, int *argsize, int *datatype, int *permanent)
{
   /*-- pretend that the map doesn't exist --*/
   if (kpds_is_mapped(obj))
      kdas_fail_error(KPDS_EMAPPING_MODE);

   if (!kdms_query_segment(obj, KPDS_SEGMENT_MAP))
      return FALSE;  /* not an error */

   if (numargs)
      *numargs = 5;
   if (argsize)
      *argsize = 1;
   if (datatype)
      *datatype = KINT;
   if (permanent)
      *permanent = TRUE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_print
|       Purpose: print the map size attribute
|    Written By: Steve Kubica
|          Date: March 30, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_size_print(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
	    kfile *outfile)
{
   int w, h, d, t, e;

   /*-- pretend that the map doesn't exist --*/
   if (kpds_is_mapped(obj))
      kdas_fail_error(KPDS_EMAPPING_MODE);

   if (!kpds_get_map_size(obj, &w, &h, &d, &t, &e))
      return FALSE;

   if (outfile)
      kfprintf(outfile, "%d %d %d %d %d", w, h, d, t, e);

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_MAP_DATA_TYPE
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _type_get
|       Purpose: get the type attribute
|    Written By: Jeremy Worley, John Salas, and Steve Kubica
|          Date: April 28, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_type_get(kobject obj, ktoken assoc, ktoken attr, kaddr clientData, 
	  kva_list *list)
{
   int *rtyp = kva_arg(*list, int *);
   int  typ;


   if (!kdms_query_segment(obj, KPDS_SEGMENT_MAP))
      return FALSE;

   if (!kdms_get_attribute(obj, KPDS_SEGMENT_MAP, KDMS_DATA_TYPE, &typ))
      return FALSE;

   if (rtyp) *rtyp = typ;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _type_set
|       Purpose: set the type attribute
|    Written By: Jeremy Worley, John Salas, and Steve Kubica
|          Date: April 28, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_type_set(kobject obj, ktoken assoc, ktoken attr, kaddr clientData, 
	  kva_list *list)
{
   int typ = kva_arg(*list, int);

   if (!kdms_query_segment(obj, KPDS_SEGMENT_MAP))
      return FALSE;

   if (!kdms_set_attribute(obj, KPDS_SEGMENT_MAP, KDMS_DATA_TYPE, typ))
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _type_match
|       Purpose: match the type attribute
|    Written By: Steve Kubica and John Salas
|          Date: April 28, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_type_match(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	    kaddr clientData1, kaddr clientData2)
{
   int typ1;
   int typ2;

   if (!kdms_query_segment(obj1, KPDS_SEGMENT_MAP) ||
       !kdms_query_segment(obj2, KPDS_SEGMENT_MAP))
      return FALSE;

   if (!kdms_get_attribute(obj1, KPDS_SEGMENT_MAP, KDMS_DATA_TYPE, &typ1))
      return FALSE;

   if (!kdms_get_attribute(obj2, KPDS_SEGMENT_MAP, KDMS_DATA_TYPE, &typ2))
      return FALSE;
   
   return (typ1 == typ2);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _type_copy
|       Purpose: copy the type attribute
|    Written By: Steve Kubica
|          Date: April 28, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_type_copy(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	   kaddr clientData1, kaddr clientData2)
{
   int typ;

   if (!kdms_query_segment(obj1, KPDS_SEGMENT_MAP) ||
       !kdms_query_segment(obj2, KPDS_SEGMENT_MAP))
      return FALSE;

   if (!kdms_get_attribute(obj1, KPDS_SEGMENT_MAP, KDMS_DATA_TYPE, &typ))
      return FALSE;

   if (!kdms_set_attribute(obj2, KPDS_SEGMENT_MAP, KDMS_DATA_TYPE, typ))
      return FALSE;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _type_query
|       Purpose: query the type attribute
|    Written By: Steve Kubica
|          Date: March 30, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_type_query(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
	    int *numargs, int *argsize, int *datatype, int *permanent)
{
   if (!kdms_query_segment(obj, KPDS_SEGMENT_MAP))
      return FALSE;

   if (numargs)
      *numargs = 1;
   if (argsize)
      *argsize = 1;
   if (datatype)
      *datatype = KINT;
   if (permanent)
      *permanent = TRUE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _type_print
|       Purpose: print the type attribute
|    Written By: Steve Kubica
|          Date: March 30, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_type_print(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
	    kfile *outfile)
{
   int typ;

   if (!kdms_query_segment(obj, KPDS_SEGMENT_MAP))
      return FALSE;

   if (!kdms_get_attribute(obj, KPDS_SEGMENT_MAP, KDMS_DATA_TYPE, &typ))
      return FALSE;

   if (outfile)
      kfprintf(outfile, "%s (%d)", kdefine_to_datatype(typ), typ);

   return TRUE;
}

/****    =============================================================
 ****    MAP segment handlers
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: (static) _create_map
|
|       Purpose: This routine will create the map segment and
|		 initialize its dimensionality and index order.
|
|         Input: obj     - the obj in which to create the map segment
|		 segment - the name of the map segment.
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica & Jeremy Worley
|          Date: June 1, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static int
_create_map(
   kobject  obj,
   char    *segment)
{
   int order[KPDS_MAP_DIM] = {KMAP_WIDTH, KMAP_HEIGHT, KMAP_DEPTH,
			      KMAP_TIME, KMAP_ELEMENTS};

   if (!kdms_create_segment(obj, KPDS_SEGMENT_MAP))
      return FALSE;
   
   if (!kdms_set_attributes(obj, KPDS_SEGMENT_MAP, 
			    KDMS_DIMENSION, KPDS_MAP_DIM,
			    KDMS_INDEX_ORDER, order, 
			    NULL))
      return FALSE;
   
   return TRUE;
}

/****    =============================================================
 ****    PRIVATE MAP functions
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kpds_init_map_segment()
|
|       Purpose: Define the map segment.
|
|         Input: none
|
|        Output: none
|
|       Returns: none
|
|    Written By: Steve Kubica
|          Date: December 26, 1994
| Modifications:
|
------------------------------------------------------------*/
void
kpds_init_map_segment(void)
{
   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MAP_SIZE, NULL,
			       _size_get,   _size_set, 
			       _size_match, _size_copy, 
			       _size_query, _size_print);


   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MAP_DATA_TYPE, NULL,
			       _type_get,   _type_set, 
			       _type_match, _type_copy, 
			       _type_query, _type_print);

   kdms_define_segment(KPDS_SEGMENT_MAP, _create_map, NULL, NULL, NULL, NULL, 
		       NULL, NULL, NULL, NULL, NULL, NULL);

   return;
}

/****    =============================================================
 ****    PUBLIC MAP functions
 ****    =============================================================
 ****/

/************************************************************
*
*  Routine Name: kpds_create_map - create a map segment within a 
*				   data object.
*
*       Purpose: This function is used to create a map segment
*		 within a specified data object.  The map segment
*		 can not be created if mapping mode is set to map
*		 the value data.
*
*		 It is considered an error to create a map segment
*		 if the object already contains one.
*
*         Input: object  - object in which to create the map segment.
*
*        Output: none
*
*       Returns: TRUE if the map segment was successfully created, 
*		 FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 20, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_create_map(
*		!   kobject object)
*
*************************************************************/
int
kpds_create_map(kobject object)
{
   kpds_init();
   
   if (kpds_is_mapped(object))
      return FALSE;

   return kdms_create_segment(object, KPDS_SEGMENT_MAP);
}

/************************************************************
*
*  Routine Name: kpds_query_map - determine if the map segment
*				  exists in a data object.
*
*       Purpose: This function is used to determine if the map
*		 segment exists in a data object.  If map segment
*		 exists in the specified object, then this function
*		 will return TRUE.  If the object is invalid, or map
*		 data does not exist in the object, then this function
*		 will return FALSE.  
*
*		 When the mapping mode is set to map the data, the
*		 data object will always appear to not contain a map.
*
*         Input: object  - data object to be queried.
*
*        Output: none
*
*       Returns: TRUE if the map segment exists, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 20, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_query_map(
*		!   kobject object)
*
*************************************************************/
int
kpds_query_map(kobject object)
{
   kpds_init();

   if (kpds_is_mapped(object))
      return FALSE;

   return kdms_query_segment(object, KPDS_SEGMENT_MAP);
}

/************************************************************
*
*  Routine Name: kpds_destroy_map - destroy the map segment in a
*				      data object.
*
*       Purpose: This function is used to destroy the map segment
*		 contained within an object.  Once the map segment
*		 has been destroyed, any data or attributes associated
*		 with the map data will be lost forever.  A new
*		 map segment can be created in its place with the
*		 function kpds_create_map.
*
*		 If the mapping mode is set to map the data.
*		 then this function will not destroy the map.  This is
*		 because destroying the map when operating in this
*		 mode would cause an inconsistency in the
*		 interpretation of the value data.  KPDS_MAPPING_MODE
*		 is an attribute which, when set to KMAPPED, causes
*		 the map data to be unmapped into the value data.  For
*		 example, if you are operating on a pseudo-colored
*		 image, and set mapping mode, the result will be that
*		 the value data will appear to be a true-color image.
*
*         Input: object  - object from which to remove the map segment.
*
*        Output: map segment is destroyed
*
*       Returns: TRUE if the map segment is successfully destroyed, 
*		 FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 20, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_destroy_map(
*		!   kobject object)
*
*************************************************************/
int
kpds_destroy_map(kobject object)
{
   kpds_init();
   
   if (kpds_is_mapped(object))
      return FALSE;

   return kdms_destroy_segment(object, KPDS_SEGMENT_MAP);
}

/************************************************************
*
*  Routine Name: kpds_initialize_map - preset map segment to constant
*
*       Purpose: This routine initializes the entire map segment to
*		 a specified value.  The last two arguments are real
*		 and imaginary components of a complex number that is
*		 the value that the data will be initialized to.  The
*		 data type of the map segment at the time this
*		 function is called will determine how the complex
*		 number is used.  A non-complex datatype means that
*		 the real part only will be used.  A
*		 non-floating-point data type means that the integer
*		 part of the real argument will be used.
*
*		 This function will work independent of the mapping
*		 mode of the object.
*
*         Input: object - object to initialize map data on
*		 real   - real part of constant
*		 imag   - imaginary part of constant
*
*        Output: map segment is modified in place
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions:
*    Written By: Jeremy Worley
*          Date: August 3, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_initialize_map(
*                !   kobject object,
*                !   double real,
*                !   double imag)
*
*************************************************************/
int
kpds_initialize_map(
   kobject object, 
   double  real, 
   double  imag)
{
   kpds_init();
   
   return kdms_initialize_segment(object, KPDS_SEGMENT_MAP, real, imag);
}

/************************************************************
*
*  Routine Name: kpds_copy_map_data - copy all map data from
*                                       one object to another object.
*
*       Purpose: This function copies all of the data contained in the
*		 map segment of one object into the map segment
*		 contained in another object.  If the map segment
*		 exists in the destination object, then its data will
*		 be replaced with the data from the source object.  If
*		 the map segment does not exist in the destination
*		 object, it will be created.
*
*		 The map data can be optionally copied through the
*		 presentations of the two data objects.  This implies
*		 that any presentation stages which are normally
*		 invoked during a kpds_get_data on the source object
*		 or a kpds_put_data call on the destination object
*		 will be used for the copy.
*
*		 When copying through the presentation, if the mapping
*		 mode is set to map the data, the map segment will
*		 appear not to be present and this call will fail.
*		 The copy will work normally if not copying through
*		 the presentations.
*
*		 Any of the following presentation stages can be
*		 invoked on either the source object or the
*		 destination object : casting, scaling, normalizing,
*		 padding, and interpolating.
*		 
*		 The following presentation stages can be invoked on
*		 only the source object : axis assignment, position,
*		 and offset.  The resulting copy will be transposed to
*		 be in the default axis ordering.  The resulting copy
*		 will also appear to be shifted by the source position
*		 and offset.
*
*		 These presentation stages are brought into the data
*		 pipeline by setting the appropriate presentation
*		 attributes on the data objects. Please see the
*		 chapter on polymorphic data services in the data
*		 services manual for more information on the various
*		 presentation attributes.
*
*		 The following example may help in visualizing the
*		 process.  The source object has a presentation data
*		 type different from the physical data type and a
*		 presentation size different from the physical size.
*		 The destination object only has a different
*		 presentation and physical data type.
*                .begin code
*		 !              _______________________
*		 !             | _____________________ |
*		 !        _____||______  ---->        ||
*		 !       |interpolating|              ||
*		 !        _____||______          _____||______
*		 !       |___casting___|        |___casting___|
*		 !         ____||_____            ____||_____
*		 !        |           |          |           |
*		 !        |  source   |        	 |destination|
*		 !        |   data    |        	 |   data    |
*		 !        |___________|        	 |___________|
*	         !
*                .end code
*		 In the above example, the data resulting from the
*		 copy will be interpolated to a new size and cast to a
*		 new data type before being copied into the
*		 destination object.  When being copied into the
*		 destination object, the data will be cast yet again
*		 before it finally is stored.
*
*		 If the presentation and physical layers of the
*		 destination object are coupled then the destination
*		 attributes will be ignored and the source
*		 presentation attributes will be propogated to the
*		 destination physical layer.  The copy, in essence,
*		 will be performed only through the source
*		 presentation, with the destination physical and
*		 presentation layers taking on the characteristics of
*		 the source presentation.  By default, output objects
*		 are not coupled, so this behavior is typical.
*		 
*		 The copy need not be performed using the presentation.
*		 If the data is not copied through the presentation,
*		 the destination data will be a direct copy of the
*		 physical source data.  The physical size and data
*		 type of the map segment will be reflected in the
*		 destination data object from the source to the
*		 destination since they describe the physical state of
*		 the data.  Again, if not copying through the presentation,
*		 the copy will ignore the mapping mode of the object.
*
*         Input: object1 - the object that serves as a source for the data.
*
*		 copy_through_presentation - if set to TRUE, the copy will
*			 		     be performed through the
*				  	     presentation of the source 
*					     and destination objects.
*					     if set to FALSE, the copy will
*					     be a direct copy of the physical
*					     data.
*
*        Output: object2 - the object that will serve as a destination for
*			   the copy operation.
*
*       Returns: TRUE if copy was successful, FALSE otherwise
*
*  Restrictions:
*    Written By: Jeremy Worley and Steve Kubica
*          Date: January 2, 1995
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_copy_map_data(
*               !   kobject object1,
*               !   kobject object2,
*		!   int     copy_through_presentation)
*
*************************************************************/
int
kpds_copy_map_data(
   kobject object1,
   kobject object2,
   int     copy_through_presentation)
{
   kpds_init();
   
   if (!copy_through_presentation)
      return kaps_copy_beneath_pres(object1, object2, KPDS_SEGMENT_MAP);
   else
   {
      /*-- make sure mapping mode is off --*/
      if (kpds_is_mapped(object1))
	 return FALSE;

      return kaps_copy_through_pres(object1, object2, KPDS_SEGMENT_MAP);
   }
}

/************************************************************
*
*  Routine Name: kpds_copy_map_attr - copy all map attributes from
*	                                one object to another object.
*
*       Purpose: This function copies all the map attributes from 
*		 one object to another object.
*
*		 If the destination object does not contain a map
*		 segment, then this function will create a map
*		 segment, and initialize its size and data type
*		 attributes to those in the source object.  If the
*		 map data already exists in the destination object,
*		 then the presentation attributes will be set to the
*		 source object's settings.
*
*		 The destination's physical attributes will change
*		 depending on the coupling of the data objects.
*		 If the destination is coupled, then the physical
*		 attributes will be changed as well.  Any data
*		 contained in the destination will be cast, rescaled,
*		 or resized to match its new physical attributes.
*
*		 If the mapping mode is set to map the data.
*		 then this function will not copy the map attributes. 
*
*         Input: source_object - the object that serves as a
*				 source for the attributes.
*
*        Output: destination_object - the object that serves as
*				      a destination for the
*				      operation.
*
*       Returns: TRUE if copy was successful, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 19, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_copy_map_attr(
*		!   kobject object1,
*		!   kobject object2)
*
*************************************************************/
int
kpds_copy_map_attr(
   kobject object1,
   kobject object2)
{
   kpds_init();
   
   if (kpds_is_mapped(object1))
      return FALSE;

   return kdms_copy_segment_attributes(object1, KPDS_SEGMENT_MAP,
				       object2, KPDS_SEGMENT_MAP);
}

/************************************************************
*
*  Routine Name: kpds_copy_map - copy the map segment from
*		                   one object to another.
*
*       Purpose: This function copies all of the attributes and data
*		 contained in the map segment of one object into the
*		 map segment contained in another object.  If the
*		 map segment exists in the destination object, then
*		 its data will be replaced with the data from the
*		 source object.  If the map segment does not exist
*		 in the destination object, it will be created.
*
*		 All map segment attributes will be copied from one
*		 object to the other.  This includes presentation
*		 attributes, such as position and offset.
*
*		 If the mapping mode is set to map the data, the map
*		 segment will appear not to be present and this call
*		 will fail. 
*
*		 The map data can be optionally copied through the
*		 presentations of the two data objects.  This implies
*		 that any presentation stages which are normally
*		 invoked during a kpds_get_data on the source object
*		 or a kpds_put_data call on the destination object
*		 will be used for the copy.
*
*		 Any of the following presentation stages can be
*		 invoked on either the source object or the
*		 destination object : casting, scaling, normalizing,
*		 padding, and interpolating.
*		 
*		 The following presentation stages can be invoked on
*		 only the source object : axis assignment, position,
*		 and offset.  The resulting copy will be transposed to
*		 be in the default axis ordering.  The resulting copy
*		 will also appear to be shifted by the source position
*		 and offset.
*
*		 These presentation stages are brought into the data
*		 pipeline by setting the appropriate presentation
*		 attributes on the data objects. Please see the
*		 chapter on polymorphic data services in the data
*		 services manual for more information on the various
*		 presentation attributes.
*
*		 The following example may help in visualizing the
*		 process.  The source object has a presentation data
*		 type different from the physical data type and a
*		 presentation size different from the physical size.
*		 The destination object only has a different
*		 presentation and physical data type.
*                .begin code
*		 !              _______________________
*		 !             | _____________________ |
*		 !        _____||______  ---->        ||
*		 !       |interpolating|              ||
*		 !        _____||______          _____||______
*		 !       |___casting___|        |___casting___|
*		 !         ____||_____            ____||_____
*		 !        |           |          |           |
*		 !        |  source   |        	 |destination|
*		 !        |   data    |        	 |   data    |
*		 !        |___________|        	 |___________|
*	         !
*                .end code
*		 In the above example, the data resulting from the
*		 copy will be interpolated to a new size and cast to a
*		 new data type before being copied into the
*		 destination object.  When being copied into the
*		 destination object, the data will be cast yet again
*		 before it finally is stored.
*
*		 If the presentation and physical layers of the
*		 destination object are coupled then the destination
*		 attributes will be ignored and the source
*		 presentation attributes will be propogated to the
*		 destination physical layer.  The copy, in essence,
*		 will be performed only through the source
*		 presentation, with the destination physical and
*		 presentation layers taking on the characteristics of
*		 the source presentation.  By default, output objects
*		 are not coupled, so this behavior is typical.
*		 
*		 The copy need not be performed using the presentation.
*		 If the data is not copied through the presentation,
*		 the destination data will be a direct copy of the
*		 physical source data.  The physical size and data
*		 type of the map segment will be reflected in the
*		 destination data object from the source to the
*		 destination since they describe the physical state of
*		 the data.  
*
*		 This function is equivalent to performing successive
*		 calls to kpds_copy_map_attr and kpds_copy_map_data.
*
*         Input: object1 - the object that serves as a source for the map
*			   segment.
*
*		 copy_through_presentation - if set to TRUE, the copy will
*			 		     be performed through the
*				  	     presentation of the source 
*					     and destination objects.
*					     if set to FALSE, the copy will
*					     be a direct copy of the physical
*					     data.
*
*        Output: object2 - the object that will serve as a destination for
*			   the copy operation.
*
*       Returns: TRUE if copy was successful, FALSE otherwise
*
*  Restrictions:
*    Written By: Jeremy Worley & Steve Kubica
*          Date: September 19, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_copy_map(
*               !   kobject object1,
*               !   kobject object2,
*		!   int     copy_through_presentation)
*
*************************************************************/
int
kpds_copy_map(
   kobject object1,
   kobject object2,
   int     copy_through_presentation)
{
   kpds_init();
   
   if (kpds_is_mapped(object1))
      return FALSE;

   return kpds_copy_map_data(object1, object2, copy_through_presentation);
}

