 /*
  * Khoros: $Id: pds_mask.c,v 1.12.2.3 1997/05/24 18:35:14 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: pds_mask.c,v 1.12.2.3 1997/05/24 18:35:14 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Polymophic Model Data Handlers
   >>>>		
   >>>>  Private:
   >>>>                 kpds_init_mask_segment()
   >>>>			kpds_set_mask_size()
   >>>>			kpds_get_mask_size()
   >>>>   Static:
   >>>>                 _get_type()
   >>>>                 _size_get()
   >>>>                 _size_set()
   >>>>                 _size_match()
   >>>>                 _size_copy()
   >>>>                 _size_query()
   >>>>                 _size_print()
   >>>>                 _type_get()
   >>>>                 _type_set()
   >>>>                 _type_match()
   >>>>                 _type_copy()
   >>>>                 _type_query()
   >>>>                 _type_print()
   >>>>                 _create_mask()
   >>>>                 _get_mask_data()
   >>>>                 _put_mask_data()
   >>>>   Public:
   >>>>                 kpds_create_mask()
   >>>>                 kpds_query_mask()
   >>>>                 kpds_destroy_mask()
   >>>>                 kpds_initialize_mask()
   >>>>                 kpds_copy_mask_data()
   >>>>                 kpds_copy_mask_attr()
   >>>>                 kpds_copy_mask()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/*-- pds_attrib.c --*/
extern int sync_sizes;

/****    =============================================================
 ****    MASK size routines
 ****    =============================================================
 ****/

int kpds_get_mask_size(kobject obj, int *w, int *h, int *d, int *t, int *e)
{
   int *size;

   if (!kdms_query_segment(obj, KPDS_SEGMENT_MASK))
      kdas_fail_error(KPDS_EMASK_NONEXIST);

   if (!kdms_get_attribute(obj, KPDS_SEGMENT_MASK, KDMS_SIZE, &size))
      return FALSE; 

   if (w) *w = size[KWID];
   if (h) *h = size[KHGT];
   if (d) *d = size[KDEP];
   if (t) *t = size[KTIM];

   /*-- if there is a map, then map width is element size --*/
   if (kpds_is_mapped(obj) && kdms_query_segment(obj, KPDS_SEGMENT_MAP))
   {
      int map_wid;

      if (!kpds_get_attribute(obj, KPDS_MAP_SIZE, 
			      &map_wid, NULL, NULL, NULL, NULL))
	 return FALSE;

      if (e) *e = size[KELM] * map_wid;
   }
   else
      if (e) *e = size[KELM];

   return TRUE;
}

int kpds_set_mask_size(kobject obj, int w, int h, int d, int t, int e)
{
   int  size[KPDS_MASK_DIM] = {-1, -1, -1, -1, -1};

   if (!kdms_query_segment(obj, KPDS_SEGMENT_MASK))
      kdas_fail_error(KPDS_EMASK_NONEXIST);

   size[KWID] = w;
   size[KHGT] = h;
   size[KDEP] = d;
   size[KTIM] = t;
   size[KELM] = e;

   if (sync_sizes)
   {
      sync_sizes = FALSE;  /*-- avoid calling here again --*/

      /*-- set the shared size of the value segment --*/
      if (kdms_query_segment(obj, KPDS_SEGMENT_VALUE))
	 if (!kpds_set_attribute(obj, KPDS_VALUE_SIZE, w, h, d, t, e))
	    return FALSE;
   
      /*-- set the shared size of the location segment --*/
      if (kdms_query_segment(obj, KPDS_SEGMENT_LOCATION))
	 if (!kpds_set_attribute(obj, KPDS_LOCATION_SIZE, w, h, d, -1))
	    return FALSE;
      
      /*-- set the shared size of the time segment --*/
      if (kdms_query_segment(obj, KPDS_SEGMENT_TIME))
	 if (!kpds_set_attribute(obj, KPDS_TIME_SIZE, t))
	    return FALSE;

      /*-- set the shared size of the map segment --*/
      if (kdms_query_segment(obj, KPDS_SEGMENT_MAP))
      {
	 int *osize;

	 /*-- track size of value depth, time, and element only if previous
	  *    map size is not 1 --  */
	 if (!kdms_get_attribute(obj, KPDS_SEGMENT_MAP, KDMS_SIZE, &osize))
	    return FALSE;

	 if (osize[KMAP_DEP]!=1 || osize[KMAP_TIM]!=1 || osize[KMAP_ELM]!=1)
	 {
	    int msize[KPDS_MAP_DIM] = {-1, -1, -1, -1, -1};
	 
	    msize[KMAP_DEP] = (osize[KMAP_DEP] == 1) ? -1 : d;
	    msize[KMAP_TIM] = (osize[KMAP_TIM] == 1) ? -1 : t;
	    msize[KMAP_ELM] = (osize[KMAP_ELM] == 1) ? -1 : e;
	 
	    if (!kdms_set_attribute(obj, KPDS_SEGMENT_MAP, KDMS_SIZE, msize))
	       return FALSE;
	 }
      }

      /*-- if mapping mode is on, then map width is affected by esize --*/
      if (kpds_is_mapped(obj) && kdms_query_segment(obj, KPDS_SEGMENT_MAP) 
	  && e != -1)  /* e of -1 implies that the esize should not change */
      {
	 int old_e;
      
	 if (!kpds_get_attribute(obj, KPDS_MASK_SIZE, 
				 NULL, NULL, NULL, NULL, &old_e))
	    return FALSE;
      
	 /* only worry about changing the map size if the esize is different */
	 old_e = kmax(old_e, 1);   /*-- don't want esize = 0 --*/
	 if (old_e != e)
	 {
	    if (!kpds_get_attribute(obj, KPDS_MAP_SIZE, 
				    &size[KELM], NULL, NULL, NULL, NULL))
	       return FALSE;
	 
	    /*
	     * if the mask size is mapped, then the current KELEMENTS
	     * size of the mask segment will be divided into the new
	     * KELEMENTS size.  This division will result in the new
	     * size of the KMAP_WIDTH.  The existing KELEMENTS size
	     * will be untouched. 
	     */
	    if (!kpds_set_attribute(obj, KPDS_MAP_SIZE, 
				    size[KELM]/old_e, -1, -1, -1, -1))
	       return FALSE;
	    size[KELM] = old_e;
	 }
      }
      sync_sizes = TRUE;
   }
   
   /*-- set the size of the mask segment --*/
   if (!kdms_set_attribute(obj, KPDS_SEGMENT_MASK, KDMS_SIZE, size))
      return FALSE;

   return TRUE;
}

/****    =============================================================
 ****    MASK utilities
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|  Routine Name: (static) _get_type
|       Purpose: retrieve the mask type
|       Returns: the mask type
|    Written By: Steve Kubica
------------------------------------------------------------*/
static 
int _get_type(kobject obj)
{
   int type;
   
   if (!kdms_get_attribute(obj, KPDS_SEGMENT_MASK, KDMS_DATA_TYPE, &type))
      return FALSE;
   else
      return type;
}

/* 
 *    ================================================================== 
 *    MASK attribute handlers
 *    ==================================================================
 */

/*********** ---------------------------------------------------------------
 ***********  KPDS_MASK_SIZE
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _size_get
|       Purpose: get the mask size attribute
|    Written By: Steve Kubica and Jeremy Worley
|          Date: April 27, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_size_get(kobject obj, ktoken assoc, ktoken attr, kaddr clientData, 
	  kva_list *list)
{
   int *w = kva_arg(*list, int *);
   int *h = kva_arg(*list, int *);
   int *d = kva_arg(*list, int *);
   int *t = kva_arg(*list, int *);
   int *e = kva_arg(*list, int *);

   return kpds_get_mask_size(obj, w, h, d, t, e);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_set
|       Purpose: set the mask size attribute
|    Written By: Steve Kubica and Jeremy Worley
|          Date: April 27, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_size_set(kobject obj, ktoken assoc, ktoken attr, kaddr clientData, 
	  kva_list *list)
{
   int  w = kva_arg(*list, int);
   int  h = kva_arg(*list, int);
   int  d = kva_arg(*list, int);
   int  t = kva_arg(*list, int);
   int  e = kva_arg(*list, int);

   return kpds_set_mask_size(obj, w, h, d, t, e);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_match
|       Purpose: match the mask size attribute
|    Written By: Steve Kubica
|          Date: April 28, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_size_match(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	    kaddr clientData1, kaddr clientData2)
{
   int w1, h1, d1, t1, e1;
   int w2, h2, d2, t2, e2;

   if (!kpds_get_mask_size(obj1, &w1, &h1, &d1, &t1, &e1))
      return FALSE;

   if (!kpds_get_mask_size(obj2, &w2, &h2, &d2, &t2, &e2))
      return FALSE;

   return ((w1 == w2) && (h1 == h2) && (d1 == d2) && (t1 == t2) && (e1 == e2));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_copy
|       Purpose: copy the mask size attribute
|    Written By: Steve Kubica
|          Date: April 28, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_size_copy(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	   kaddr clientData1, kaddr clientData2)
{
   int w, h, d, t, e;

   if (!kpds_get_mask_size(obj1, &w, &h, &d, &t, &e))
      return FALSE;

   if (!kpds_set_mask_size(obj2, w, h, d, t, e))
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_query
|       Purpose: query the mask size attribute
|    Written By: Steve Kubica
|          Date: March 30, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_size_query(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
	    int *numargs, int *argsize, int *datatype, int *permanent)
{
   if (!kdms_query_segment(obj, KPDS_SEGMENT_MASK))
      return FALSE;		/* not an error */

   if (numargs)
      *numargs = 5;
   if (argsize)
      *argsize = 1;
   if (datatype)
      *datatype = KINT;
   if (permanent)
      *permanent = TRUE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_print
|       Purpose: print the mask size attribute
|    Written By: Steve Kubica
|          Date: March 30, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_size_print(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
	    kfile *outfile)
{
   int w, h, d, t, e;

   if (!kpds_get_mask_size(obj, &w, &h, &d, &t, &e))
      return FALSE;

   if (outfile)
      kfprintf(outfile, "%d %d %d %d %d", w, h, d, t, e);

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_MASK_DATA_TYPE
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _type_get
|       Purpose: get the type attribute
|    Written By: Jeremy Worley, John Salas, and Steve Kubica
|          Date: April 28, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_type_get(kobject obj, ktoken assoc, ktoken attr, kaddr clientData, 
	  kva_list *list)
{
   int *rtyp = kva_arg(*list, int *);
   int  typ;


   if (!kdms_query_segment(obj, KPDS_SEGMENT_MASK))
      return FALSE;

   if (!kdms_get_attribute(obj, KPDS_SEGMENT_MASK, KDMS_DATA_TYPE, &typ))
      return FALSE;
   
   if (rtyp) *rtyp = typ;
   
   return TRUE;
}


/*-----------------------------------------------------------
|  Routine Name: (static) _type_set
|       Purpose: set the type attribute
|    Written By: Jeremy Worley, John Salas, and Steve Kubica
|          Date: April 28, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_type_set(kobject obj, ktoken assoc, ktoken attr, kaddr clientData, 
	  kva_list *list)
{
   int typ = kva_arg(*list, int);

   if (!kdms_query_segment(obj, KPDS_SEGMENT_MASK))
      return FALSE;

   if (!kdms_set_attribute(obj, KPDS_SEGMENT_MASK, KDMS_DATA_TYPE, typ))
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _type_match
|       Purpose: match the type attribute
|    Written By: Steve Kubica and John Salas
|          Date: April 28, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_type_match(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	    kaddr clientData1, kaddr clientData2)
{
   int typ1;
   int typ2;

   if (!kdms_query_segment(obj1, KPDS_SEGMENT_MASK) ||
       !kdms_query_segment(obj2, KPDS_SEGMENT_MASK))
      return FALSE;

   if (!kdms_get_attribute(obj1, KPDS_SEGMENT_MASK, KDMS_DATA_TYPE, &typ1))
      return FALSE;

   if (!kdms_get_attribute(obj2, KPDS_SEGMENT_MASK, KDMS_DATA_TYPE, &typ2))
      return FALSE;
   
   return (typ1 == typ2);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _type_copy
|       Purpose: copy the type attribute
|    Written By: Steve Kubica
|          Date: April 28, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_type_copy(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	   kaddr clientData1, kaddr clientData2)
{
   int typ;

   if (!kdms_query_segment(obj1, KPDS_SEGMENT_MASK) ||
       !kdms_query_segment(obj2, KPDS_SEGMENT_MASK))
      return FALSE;

   if (!kdms_get_attribute(obj1, KPDS_SEGMENT_MASK, KDMS_DATA_TYPE, &typ))
      return FALSE;

   if (!kdms_set_attribute(obj2, KPDS_SEGMENT_MASK, KDMS_DATA_TYPE, typ))
      return FALSE;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _type_query
|       Purpose: query the type attribute
|    Written By: Steve Kubica
|          Date: March 30, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_type_query(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
	    int *numargs, int *argsize, int *datatype, int *permanent)
{
   if (!kdms_query_segment(obj, KPDS_SEGMENT_MASK))
      return FALSE;

   if (numargs)
      *numargs = 1;
   if (argsize)
      *argsize = 1;
   if (datatype)
      *datatype = KINT;
   if (permanent)
      *permanent = TRUE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _type_print
|       Purpose: print the type attribute
|    Written By: Steve Kubica
|          Date: March 30, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_type_print(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
	    kfile *outfile)
{
   int typ;

   if (!kdms_query_segment(obj, KPDS_SEGMENT_MASK))
      return FALSE;

   if (!kdms_get_attribute(obj, KPDS_SEGMENT_MASK, KDMS_DATA_TYPE, &typ))
      return FALSE;

   if (outfile)
      kfprintf(outfile, "%s (%d)", kdefine_to_datatype(typ), typ);

   return TRUE;
}

/****    =============================================================
 ****    MASK segment handlers
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: (static) _create_mask
|
|       Purpose: This routine will create the mask segment and
|		 initialize its dimensionality and index order.
|
|         Input: obj     - the obj in which to create the mask segment
|		 segment - the name of the mask segment.
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica & Jeremy Worley
|          Date: June 1, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static int
_create_mask(
   kobject  obj,
   char    *segment)
{
   int  order[KPDS_MASK_DIM] = {KWIDTH, KHEIGHT, KDEPTH, KTIME, KELEMENTS};
   int *vorder = NULL;
   int  oldmode;
   int  w = -1;
   int 	h = -1;
   int 	d = -1;
   int 	t = -1;
   int 	e = -1;


   if (!kdms_create_segment(obj, KPDS_SEGMENT_MASK))
      return FALSE;
   
   if (!kdms_set_attributes(obj, KPDS_SEGMENT_MASK, 
			         KDMS_DIMENSION, KPDS_MASK_DIM,
			    	 KDMS_INDEX_ORDER, order, 
			    	 KDMS_PAD_VALUE, 1.0, 0.0,
			    	 NULL))
      return FALSE;

   /*-- don't want esize to be mapped --*/
   kpds_get_attribute(obj, KPDS_MAPPING_MODE, &oldmode);
   kpds_set_attribute(obj, KPDS_MAPPING_MODE, KUNMAPPED);
      
   /*-- get initial size and index order from existing segments --*/
   if (kdms_query_segment(obj, KPDS_SEGMENT_VALUE))
   {
      kpds_get_attribute(obj, KPDS_VALUE_SIZE, &w, &h, &d, &t, &e);

      kdms_get_attribute(obj, KPDS_SEGMENT_VALUE, KDMS_INDEX_ORDER, &vorder);
      kdms_set_attribute(obj, KPDS_SEGMENT_MASK,  KDMS_INDEX_ORDER, vorder);
   }
   else if (kdms_query_segment(obj, KPDS_SEGMENT_LOCATION))
   {
      kpds_get_attribute(obj, KPDS_LOCATION_SIZE, &w, &h, &d, NULL);

      kdms_get_attribute(obj, KPDS_SEGMENT_LOCATION, KDMS_INDEX_ORDER,&vorder);
      order[0] = vorder[0];
      order[1] = vorder[1];
      order[2] = vorder[2];
      order[3] = KTIME;
      order[4] = KELEMENTS;
      kdms_set_attribute(obj, KPDS_SEGMENT_MASK, KDMS_INDEX_ORDER, order);
   }

   else if (kdms_query_segment(obj, KPDS_SEGMENT_TIME))
      kpds_get_attribute(obj, KPDS_TIME_SIZE, &t);

   else if (kdms_query_segment(obj, KPDS_SEGMENT_MAP))
      kpds_get_attribute(obj, KPDS_MAP_SIZE, NULL, NULL, &d, &t, &e);
   

   /*-- set initial size of the mask segment --*/
   kpds_set_attribute(obj, KPDS_MASK_SIZE, w, h, d, t, e);

   /*-- restore mapping mode --*/
   kpds_set_attribute(obj, KPDS_MAPPING_MODE, oldmode);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _get_mask_data
|
|       Purpose: This routine gets data from the mask segment.
|                It is "map-aware" in that it understands the 
|		 mapping mode and will alter the mask data
|		 size appropirately.
|
|         Input: object  - the object to retrieve data from
|	         segment - segment to retrieve data from
|		 begin   - the begin point for region to get :
|			   if the data is to be mapped, this
|		           begin point represents the mapped begin
|		 end     - the end point for the region to get :
|			   if the data is to be mapped, this
|		           end point represents the mapped end
|
|        Output: data   - pointer to already allocated data,
|			  or NULL if we should perform the allocation
|
|       Returns: pointer to the data being returned on success,
|		 NULL otherwise
|
|    Written By: Steve Kubica
|          Date: April 29, 1994
| Modifications:
|
------------------------------------------------------------*/
static kaddr
_get_mask_data(
   kobject obj,
   char   *segment,
   int    *begin,
   int    *end,
   kaddr   data)
{
   kaddr rdata;

   /*-- verify that mask segment exists --*/
   if (!kdms_query_segment(obj, segment))
      return NULL;
   
   if (kpds_is_mapped(obj))
      rdata = kpds_get_mapped_mask_data(obj, begin, end, data);
   else
      rdata = kdms_get_data(obj, segment, begin, end, data);

   if (kpds_elements_first(obj))
   {
      kaddr ldata;
      
      ldata = kpds_transpose_elements_first(rdata, begin, end, _get_type(obj),
					    KPDS_MASK_DIM);
      return ldata;
   }
   
   return rdata;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _put_mask_data
|
|       Purpose: This routine puts data into the mask segment.
|                It is "map-aware" in that it understands the 
|		 mapping mode. It actually ignores the mapping 
|		 mode, since it is meaningless for a put,
|		 but it know enough to give debug warnings.
|
|         Input: obj - the object to put data into
|	         segment - the segment to put the data into
|		 begin   - the begin point for the put
|		 end     - the end point for the put
|		 data    - the data to be put
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: April 29, 1994
| Modifications:
|
------------------------------------------------------------*/
static int
_put_mask_data(
   kobject obj,
   char   *segment,
   int    *begin,
   int    *end,
   kaddr   data)
{
   int status;
   kaddr pdata = data;
   
   /*-- verify that the mask segment exists --*/
   if (!kdms_query_segment(obj, segment))
      return FALSE;

   /*-- transpose if elements are first --*/
   if (kpds_elements_first(obj))
      pdata = kpds_transpose_elements_last(data, begin, end, _get_type(obj),
					   KPDS_MASK_DIM);

   status = kdms_put_data(obj, segment, begin, end, pdata);
   if (pdata != data) kfree(pdata);
   return status;
}


/****    =============================================================
 ****    PRIVATE MASK functions
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kpds_init_mask_segment()
|
|       Purpose: Define the mask segment.
|
|         Input: none
|
|        Output: none
|
|       Returns: none
|
|    Written By: Steve Kubica
|          Date: December 26, 1994
| Modifications:
|
------------------------------------------------------------*/
void
kpds_init_mask_segment(void)
{
   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_MASK_SIZE, NULL,
			       _size_get,   _size_set, 
			       _size_match, _size_copy, 
			       _size_query, _size_print);

   kdms_define_quasi_attribute(KDMS_OBJECT,KPDS_MASK_DATA_TYPE, NULL,
			       _type_get,   _type_set, 
			       _type_match, _type_copy, 
			       _type_query, _type_print);

   kdms_define_segment(KPDS_SEGMENT_MASK, _create_mask, NULL, NULL, 
		       _get_mask_data, _put_mask_data, NULL, NULL, 
		       NULL, NULL, NULL, NULL);
   return;
}

/****    =============================================================
 ****    PUBLIC MASK functions
 ****    =============================================================
 ****/

/************************************************************
*
*  Routine Name: kpds_create_mask - create a mask segment within a 
*				    data object.
*
*       Purpose: This function is used to create a mask segment
*		 within a specified data object.  The size of the
*		 mask segment will be initialized to match any
*		 the sizes shared with any other polymorphic segments.
*
*		 Note that the pad value for this segment is
*		 initalized to 1.  This is done so that padded data is
*		 marked as valid according to the mask segment.
*
*		 It is considered an error to create a mask segment
*		 if the object already contains one.
*
*         Input: object  - object in which to create the mask segment.
*
*        Output: none
*
*       Returns: TRUE if the mask segment was successfully created, 
*		 FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 20, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_create_mask(
*		!   kobject object)
*
*************************************************************/
int
kpds_create_mask(kobject object)
{
   kpds_init();
   
   return kdms_create_segment(object, KPDS_SEGMENT_MASK);
}

/************************************************************
*
*  Routine Name: kpds_query_mask - determine if the mask segment
*				   exists in a data object.
*
*       Purpose: This function is used to determine if the mask
*		 segment exists in a data object.  If mask segment
*		 exists in the specified object, then this function
*		 will return TRUE.  If the object is invalid, or mask
*		 data does not exist in the object, then this function
*		 will return FALSE.
*
*         Input: object  - data object to be queried.
*
*        Output: none
*
*       Returns: TRUE if the mask segment exists, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 20, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_query_mask(
*		!   kobject object)
*
*************************************************************/
int
kpds_query_mask(kobject object)
{
   kpds_init();
   
   return kdms_query_segment(object, KPDS_SEGMENT_MASK);
}

/************************************************************
*
*  Routine Name: kpds_destroy_mask - destroy the mask segment in a
*				     data object.
*
*       Purpose: This function is used to destroy the mask segment
*		 contained within an object.  Once the mask segment
*		 has been destroyed, any data or attributes associated
*		 with the mask data will be lost forever.  A new
*		 mask segment can be created in its place with the
*		 function kpds_create_mask.
*
*		 If the mask segment does not exist in the specified
*		 object, it is considered to be an error.
*
*         Input: object  - object from which to remove the mask segment.
*
*        Output: mask segment is destroyed
*
*       Returns: TRUE if the mask segment is successfully destroyed, 
*		 FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 20, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_destroy_mask(
*		!   kobject object)
*
*************************************************************/
int
kpds_destroy_mask(kobject object)
{
   kpds_init();
   
   return kdms_destroy_segment(object, KPDS_SEGMENT_MASK);
}

/************************************************************
*
*  Routine Name: kpds_initialize_mask - preset mask segment to constant
*
*       Purpose: This routine initializes the entire mask segment to a
*		 specified mask.  The last argument is a real number
*		 that the mask data will be initialized to.  The data
*		 type of the mask segment at the time this function is
*		 called will determine how the number is used.
*
*         Input: object - object to initialize mask data on
*		 number - constant number to intialize to.
*
*        Output: mask segment is modified in place
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions:
*    Written By: Jeremy Worley
*          Date: August 3, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_initialize_mask(
*                !   kobject object,
*                !   double number)
*
*************************************************************/
int
kpds_initialize_mask(
   kobject object, 
   double number)
{
   int  status = TRUE;
   int  oldmode;

   kpds_init();
   
   /*-- don't want esize to be mapped --*/
   status &= kpds_get_attribute(object, KPDS_MAPPING_MODE, &oldmode);
   status &= kpds_set_attribute(object, KPDS_MAPPING_MODE, KUNMAPPED);
   
   status &= kdms_initialize_segment(object, KPDS_SEGMENT_MASK, number, 0.0);

   /*-- restore mapping mode --*/
   status &= kpds_set_attribute(object, KPDS_MAPPING_MODE, oldmode);

   return status;
}

/************************************************************
*
*  Routine Name: kpds_copy_mask_data - copy all mask data from
*                                      one object to another object.
*
*       Purpose: This function copies all of the data contained in the
*		 mask segment of one object into the mask segment
*		 contained in another object.  If the mask segment
*		 exists in the destination object, then its data will
*		 be replaced with the data from the source object.  If
*		 the mask segment does not exist in the destination
*		 object, it will be created.
*
*		 The mask data can be optionally copied through the
*		 presentations of the two data objects.  This implies
*		 that any presentation stages which are normally
*		 invoked during a kpds_get_data on the source object
*		 or a kpds_put_data call on the destination object
*		 will be used for the copy.
*
*		 Any of the following presentation stages can be
*		 invoked on either the source object or the
*		 destination object : casting, scaling, normalizing,
*		 padding, and interpolating.
*		 
*		 The following presentation stages can be invoked on
*		 only the source object : mapping, axis assignment,
*		 position, and offset.  The resulting copy will be set
*		 to the size it appears to be when mapping mode is set
*		 to mapped, and will be transposed to be in the
*		 default axis ordering.  The resulting copy will also
*		 appear to be shifted by the source position and
*		 offset.
*
*		 These presentation stages are brought into the data
*		 pipeline by setting the appropriate presentation
*		 attributes on the data objects. Please see the
*		 chapter on polymorphic data services in the data
*		 services manual for more information on the various
*		 presentation attributes.
*
*		 The following example may help in visualizing the
*		 process.  The source object has a presentation data
*		 type different from the physical data type, and a
*		 presentation size different from the physical size.
*		 The destination object only has a different
*		 presentation and physical data type.
*                .begin code
*		 !              _______________________
*		 !             | _____________________ |
*		 !        _____||______  ---->        ||
*		 !       |interpolating|              ||
*		 !        _____||______          _____||______
*		 !       |___casting___|        |___casting___|
*		 !         ____||_____            ____||_____
*		 !        |           |          |           |
*		 !        |  source   |        	 |destination|
*		 !        |   data    |        	 |   data    |
*		 !        |___________|        	 |___________|
*	         !
*                .end code
*		 In the above example, the data resulting from the
*		 copy will be interpolated to a new size and cast to a
*		 new data type before being copied into the
*		 destination object.  When being copied into the
*		 destination object, the data will be cast yet again
*		 before it finally is stored.
*
*		 If the presentation and physical layers of the
*		 destination object are coupled then the destination
*		 attributes will be ignored and the source
*		 presentation attributes will be propogated to the
*		 destination physical layer.  The copy, in essence,
*		 will be performed only through the source
*		 presentation, with the destination physical and
*		 presentation layers taking on the characteristics of
*		 the source presentation.  By default, output objects
*		 are not coupled, so this behavior is typical.
*		 
*		 The copy need not be performed using the presentation.
*		 If the data is not copied through the presentation,
*		 the destination data will be a direct copy of the
*		 physical source data.  The physical size and data
*		 type of the mask segment will be reflected in the
*		 destination data object from the source to the
*		 destination since they describe the physical state of
*		 the data.
*
*         Input: object1 - the object that serves as a source for the data.
*
*		 copy_through_presentation - if set to TRUE, the copy will
*			 		     be performed through the
*				  	     presentation of the source 
*					     and destination objects.
*					     if set to FALSE, the copy will
*					     be a direct copy of the physical
*					     data.
*
*        Output: object2 - the object that will serve as a destination for
*			   the copy operation.
*
*       Returns: TRUE if copy was successful, FALSE otherwise
*
*  Restrictions:
*    Written By: Jeremy Worley and Steve Kubica
*          Date: January 2, 1995
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_copy_mask_data(
*               !   kobject object1,
*               !   kobject object2,
*		!   int     copy_through_presentation)
*
*************************************************************/
int
kpds_copy_mask_data(
   kobject object1,
   kobject object2,
   int     copy_through_presentation)
{
   kpds_init();

   if (!copy_through_presentation)
      return kaps_copy_beneath_pres(object1, object2, KPDS_SEGMENT_MASK);
   else
      return kaps_copy_through_pres(object1, object2, KPDS_SEGMENT_MASK);
}

/************************************************************
*
*  Routine Name: kpds_copy_mask_attr - copy all mask attributes from
*	                               one object to another object.
*
*       Purpose: This function copies all the mask attributes from 
*		 one object to another object.
*
*		 If the destination object does not contain a mask
*		 segment, then this function will create a mask
*		 segment, and initialize its size and data type
*		 attributes to those in the source object.  If the
*		 mask data already exists in the destination object,
*		 then the presentation attributes will be set to the
*		 source object's settings.
*
*		 The destination's physical attributes will change
*		 depending on the coupling of the data objects.
*		 If the destination is coupled, then the physical
*		 attributes will be changed as well.  Any data
*		 contained in the destination will be cast, rescaled,
*		 or resized to match its new physical attributes.
*
*         Input: source_object - the object that serves as a
*				 source for the attributes.
*
*        Output: destination_object - the object that serves as
*				      a destination for the
*				      operation.
*
*       Returns: TRUE if copy was successful, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 19, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_copy_mask_attr(
*		!   kobject object1,
*		!   kobject object2)
*
*************************************************************/
int
kpds_copy_mask_attr(
   kobject object1,
   kobject object2)
{
   kpds_init();
   
   return kdms_copy_segment_attributes(object1, KPDS_SEGMENT_MASK,
				       object2, KPDS_SEGMENT_MASK);
}

/************************************************************
*
*  Routine Name: kpds_copy_mask - copy the mask segment from
*		                  one object to another.
*
*       Purpose: This function copies all of the attributes and data
*		 contained in the mask segment of one object into the
*		 mask segment contained in another object.  If the
*		 mask segment exists in the destination object, then
*		 its data will be replaced with the data from the
*		 source object.  If the mask segment does not exist
*		 in the destination object, it will be created.
*
*		 All mask segment attributes will be copied from one
*		 object to the other.  This includes presentation
*		 attributes, such as position and offset.
*
*		 The mask data can be optionally copied through the
*		 presentations of the two data objects.  This implies
*		 that any presentation stages which are normally
*		 invoked during a kpds_get_data on the source object
*		 or a kpds_put_data call on the destination object
*		 will be used for the copy.
*
*		 Any of the following presentation stages can be
*		 invoked on either the source object or the
*		 destination object : casting, scaling, normalizing,
*		 padding, and interpolating.
*		 
*		 The following presentation stages can be invoked on
*		 only the source object : mapping, axis assignment,
*		 position, and offset.  The resulting copy will be
*		 mapped through the map and transposed to be in the
*		 default axis ordering.  The resulting copy will also
*		 appear to be shifted by the source position and
*		 offset.
*
*		 These presentation stages are brought into the data
*		 pipeline by setting the appropriate presentation
*		 attributes on the data objects. Please see the
*		 chapter on polymorphic data services in the data
*		 services manual for more information on the various
*		 presentation attributes.
*
*		 The following example may help in visualizing the
*		 process.  The source object has a presentation data
*		 type different from the physical data type, and a
*		 presentation size different from the physical size.
*		 The destination object only has a different
*		 presentation and physical data type.
*                .begin code
*		 !              _______________________
*		 !             | _____________________ |
*		 !        _____||______  ---->        ||
*		 !       |interpolating|              ||
*		 !        _____||______          _____||______
*		 !       |___casting___|        |___casting___|
*		 !         ____||_____            ____||_____
*		 !        |           |          |           |
*		 !        |  source   |        	 |destination|
*		 !        |   data    |        	 |   data    |
*		 !        |___________|        	 |___________|
*	         !
*                .end code
*		 In the above example, the data resulting from the
*		 copy will be interpolated to a new size and cast to a
*		 new data type before being copied into the
*		 destination object.  When being copied into the
*		 destination object, the data will be cast yet again
*		 before it finally is stored.
*
*		 If the presentation and physical layers of the
*		 destination object are coupled then the destination
*		 attributes will be ignored and the source
*		 presentation attributes will be propogated to the
*		 destination physical layer.  The copy, in essence,
*		 will be performed only through the source
*		 presentation, with the destination physical and
*		 presentation layers taking on the characteristics of
*		 the source presentation.  By default, output objects
*		 are not coupled, so this behavior is typical.
*		 
*		 The copy need not be performed using the presentation.
*		 If the data is not copied through the presentation,
*		 the destination data will be a direct copy of the
*		 physical source data.  The physical size and data
*		 type of the mask segment will be reflected in the
*		 destination data object from the source to the
*		 destination since they describe the physical state of
*		 the data.
*
*		 This function is equivalent to performing successive
*		 calls to kpds_copy_mask_attr and kpds_copy_mask_data.
*
*         Input: object1 - the object that serves as a source for the mask
*			   segment.
*
*		 copy_through_presentation - if set to TRUE, the copy will
*			 		     be performed through the
*				  	     presentation of the source 
*					     and destination objects.
*					     if set to FALSE, the copy will
*					     be a direct copy of the physical
*					     data.
*
*        Output: object2 - the object that will serve as a destination for
*			   the copy operation.
*
*       Returns: TRUE if copy was successful, FALSE otherwise
*
*  Restrictions:
*    Written By: Jeremy Worley & Steve Kubica
*          Date: September 19, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_copy_mask(
*               !   kobject object1,
*               !   kobject object2,
*		!   int     copy_through_presentation)
*
*************************************************************/
int
kpds_copy_mask(
   kobject object1,
   kobject object2,
   int     copy_through_presentation)
{
   kpds_init();
   
   return (kpds_copy_mask_attr(object1, object2) &&
	   kpds_copy_mask_data(object1, object2, copy_through_presentation));
}
