 /*
  * Khoros: $Id: pds_value.c,v 1.12.2.3 1997/05/24 18:35:20 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: pds_value.c,v 1.12.2.3 1997/05/24 18:35:20 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Polymophic Model Data Handlers
   >>>>		
   >>>>  Private:
   >>>>                 kpds_init_value_segment()
   >>>>			kpds_get_value_size()
   >>>>			kpds_set_value_size()
   >>>>   Static:
   >>>>                 _get_type()
   >>>>                 _size_get()
   >>>>                 _size_set()
   >>>>                 _size_match()
   >>>>                 _size_copy()
   >>>>                 _size_query()
   >>>>                 _size_print()
   >>>>                 _type_get()
   >>>>                 _type_set()
   >>>>                 _type_match()
   >>>>                 _type_copy()
   >>>>                 _type_query()
   >>>>                 _type_print()
   >>>>                 _create_value()
   >>>>                 _get_value_data()
   >>>>                 _put_value_data()
   >>>>   Public:
   >>>>                 kpds_create_value()
   >>>>                 kpds_query_value()
   >>>>                 kpds_destroy_value()
   >>>>                 kpds_initialize_value()
   >>>>                 kpds_copy_value_data()
   >>>>                 kpds_copy_value_attr()
   >>>>                 kpds_copy_value()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/*-- pds_attrib.c --*/
extern int sync_sizes;

/****    =============================================================
 ****    VALUE size routines
 ****    =============================================================
 ****/

int kpds_get_value_size(kobject obj, int *w, int *h, int *d, int *t, int *e)
{
   int *size;

   if (!kdms_query_segment(obj, KPDS_SEGMENT_VALUE))
      kdas_fail_error(KPDS_EVALUE_NONEXIST);

   if (!kdms_get_attribute(obj, KPDS_SEGMENT_VALUE, KDMS_SIZE, &size))
      return FALSE; 

   if (w) *w = size[KWID];
   if (h) *h = size[KHGT];
   if (d) *d = size[KDEP];
   if (t) *t = size[KTIM];

   /*-- if there is a map, then map width is element size --*/
   if (kdms_query_segment(obj, KPDS_SEGMENT_MAP) && kpds_is_mapped(obj))
   {
      int map_wid;

      if (!kpds_get_attribute(obj, KPDS_MAP_SIZE, 
			      &map_wid, NULL, NULL, NULL, NULL))
	 return FALSE;

      if (e) *e = size[KELM] * map_wid;
   }
   else
      if (e) *e = size[KELM];

   return TRUE;
}


int kpds_set_value_size(kobject obj,int w, int h, int d, int t, int e)
{
   int size[KDAS_MAX_DIM];
   
   if (!kdms_query_segment(obj, KPDS_SEGMENT_VALUE))
      kdas_fail_error(KPDS_EVALUE_NONEXIST);

   size[KWID] = w;
   size[KHGT] = h;
   size[KDEP] = d;
   size[KTIM] = t;
   size[KELM] = e;

   if (sync_sizes)
   {
      sync_sizes = FALSE;  /*-- avoid calling here again --*/

      /*-- set the shared size of the mask segment --*/
      if (kdms_query_segment(obj, KPDS_SEGMENT_MASK))
	 if (!kpds_set_attribute(obj, KPDS_MASK_SIZE, w, h, d, t, e))
	    return FALSE;
   
      /*-- set the shared size of the location segment --*/
      if (kdms_query_segment(obj, KPDS_SEGMENT_LOCATION))
	 if (!kpds_set_attribute(obj, KPDS_LOCATION_SIZE, w, h, d, -1))
	    return FALSE;
      
      /*-- set the shared size of the time segment --*/
      if (kdms_query_segment(obj, KPDS_SEGMENT_TIME))
	 if (!kpds_set_attribute(obj, KPDS_TIME_SIZE, t))
	    return FALSE;

      /*-- set the shared size of the map segment --*/
      if (kdms_query_segment(obj, KPDS_SEGMENT_MAP))
      {
	 int *osize;

	 /*-- track size of value depth, time, and element only if previous
	  *    map size is not 1 --  */
	 if (!kdms_get_attribute(obj, KPDS_SEGMENT_MAP, KDMS_SIZE, &osize))
	    return FALSE;

	 if (osize[KMAP_DEP]!=1 || osize[KMAP_TIM]!=1 || osize[KMAP_ELM]!=1)
	 {
	    int msize[KPDS_MAP_DIM] = {-1, -1, -1, -1, -1};
	 
	    msize[KMAP_DEP] = (osize[KMAP_DEP] == 1) ? -1 : d;
	    msize[KMAP_TIM] = (osize[KMAP_TIM] == 1) ? -1 : t;
	    msize[KMAP_ELM] = (osize[KMAP_ELM] == 1) ? -1 : e;
	 
	    if (!kdms_set_attribute(obj, KPDS_SEGMENT_MAP, KDMS_SIZE, msize))
	       return FALSE;
	 }
      }

      /*-- if mapping mode is on, then map width is affected by esize --*/
      if (kdms_query_segment(obj, KPDS_SEGMENT_MAP) && kpds_is_mapped(obj) 
	  && e != -1)  /* e of -1 implies that the esize should not change */
      {
	 int *old_s;
	 int  map_w;
	 
 	 /* 
	  *  the new element size will only decompose properly if the
	  *  true value is 1 or the map width is 1 ... otherwise, the
	  *  map size will get screwed up :-(
	  */
	 if (!kdms_get_attribute(obj, KPDS_SEGMENT_VALUE, KDMS_SIZE, &old_s))
	    return FALSE;

	 /*-- if value element size == 1, then map width takes on change --*/
	 if (old_s[KELM] == 1)
	 {
	    if (!kpds_set_attribute(obj, KPDS_MAP_SIZE, e, -1, -1, -1, -1))
	       return FALSE;

	    /*-- the element size will become 1 now --*/
	    size[KELM] = 1;
	 }
	 else  /*-- our only hope is if the map width is 1 --*/
	 {
	    if (!kpds_get_attribute(obj, KPDS_MAP_SIZE, &map_w, 
				    NULL, NULL, NULL, NULL))
	       return FALSE;

	    /*-- if map_w == 1, then value elements can take on change --*/
	    if (map_w != 1)
	       kerror("kappserv", "kpds_set_attribute", 
		      "Unable to alter size of value data with map.  "
		      "Data must be mapped explicitly before "
		      "operation will work.");
	 }
      }
      sync_sizes = TRUE;
   }
   
   /*-	- set the size of the value segment --*/
   if (!kdms_set_attribute(obj, KPDS_SEGMENT_VALUE, KDMS_SIZE, size))
      return FALSE;

   return TRUE;
}





/****    =============================================================
 ****    VALUE utilities
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|  Routine Name: (static) _get_type
|       Purpose: retrieve the value type
|       Returns: the value type
|    Written By: Steve Kubica
------------------------------------------------------------*/
static 
int _get_type(kobject obj)
{
   int type;
   
   if (!kdms_get_attribute(obj, KPDS_SEGMENT_VALUE, KDMS_DATA_TYPE, &type))
      return FALSE;
   else
      return type;
}

/****    =============================================================
 ****    VALUE attribute handlers
 ****    =============================================================
 ****/

/*********** ---------------------------------------------------------------
 ***********  KPDS_VALUE_SIZE
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _size_get
|       Purpose: get the value size attribute
|    Written By: Steve Kubica and Jeremy Worley
|          Date: April 27, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_size_get(kobject obj, ktoken assoc, ktoken attr, kaddr clientData, 
	  kva_list *list)
{
   int *w = kva_arg(*list, int *);
   int *h = kva_arg(*list, int *);
   int *d = kva_arg(*list, int *);
   int *t = kva_arg(*list, int *);
   int *e = kva_arg(*list, int *);

   return kpds_get_value_size(obj, w, h, d, t, e);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_set
|       Purpose: set the value size attribute
|    Written By: Steve Kubica and Jeremy Worley
|          Date: April 27, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_size_set(kobject obj, ktoken assoc, ktoken attr, kaddr clientData, 
	  kva_list *list)
{
   int  w = kva_arg(*list, int);
   int  h = kva_arg(*list, int);
   int  d = kva_arg(*list, int);
   int  t = kva_arg(*list, int);
   int  e = kva_arg(*list, int);

   return kpds_set_value_size(obj, w, h, d, t, e);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_match
|       Purpose: match the value size attribute
|    Written By: Steve Kubica
|          Date: April 28, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_size_match(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	    kaddr clientData1, kaddr clientData2)
{
   int w1, h1, d1, t1, e1;
   int w2, h2, d2, t2, e2;


   if (!kpds_get_value_size(obj1, &w1, &h1, &d1, &t1, &e1))
      return FALSE;

   if (!kpds_get_value_size(obj2, &w2, &h2, &d2, &t2, &e2))
      return FALSE;

   return ((w1 == w2) && (h1 == h2) && (d1 == d2) && (t1 == t2) && (e1 == e2));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_copy
|       Purpose: copy the value size attribute
|    Written By: Steve Kubica
|          Date: April 28, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_size_copy(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	   kaddr clientData1, kaddr clientData2)
{
   int w, h, d, t, e;


   if (!kpds_get_value_size(obj1, &w, &h, &d, &t, &e))
      return FALSE;

   if (!kpds_set_value_size(obj2, w, h, d, t, e))
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_query
|       Purpose: query the value size attribute
|    Written By: Steve Kubica
|          Date: March 30, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_size_query(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
	    int *numargs, int *argsize, int *datatype, int *permanent)
{
   if (!kdms_query_segment(obj, KPDS_SEGMENT_VALUE))
      return FALSE;		/* not an error */

   if (numargs)
      *numargs = 5;
   if (argsize)
      *argsize = 1;
   if (datatype)
      *datatype = KINT;
   if (permanent)
      *permanent = TRUE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _size_print
|       Purpose: print the value size attribute
|    Written By: Steve Kubica
|          Date: March 30, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_size_print(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
	    kfile *outfile)
{
   int w, h, d, t, e;

   if (!kpds_get_value_size(obj, &w, &h, &d, &t, &e))
      return FALSE;

   if (outfile)
      kfprintf(outfile, "%d %d %d %d %d", w, h, d, t, e);

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_VALUE_DATA_TYPE
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _type_get
|       Purpose: get the type attribute
|    Written By: Jeremy Worley, John Salas, and Steve Kubica
|          Date: April 28, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_type_get(kobject obj, ktoken assoc, ktoken attr, kaddr clientData, 
	  kva_list *list)
{
   int *rtyp = kva_arg(*list, int *);
   int  typ;


   if (!kdms_query_segment(obj, KPDS_SEGMENT_VALUE))
      return FALSE;

   if (kdms_query_segment(obj, KPDS_SEGMENT_MAP) && kpds_is_mapped(obj))
   {
      if (!kdms_get_attribute(obj, KPDS_SEGMENT_MAP, KDMS_DATA_TYPE, &typ))
	 return FALSE;
   }
   else
   {
      if (!kdms_get_attribute(obj, KPDS_SEGMENT_VALUE, KDMS_DATA_TYPE, &typ))
	    return FALSE;
   }

   if (rtyp) *rtyp = typ;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _type_set
|       Purpose: set the type attribute
|    Written By: Jeremy Worley, John Salas, and Steve Kubica
|          Date: April 28, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_type_set(kobject obj, ktoken assoc, ktoken attr, kaddr clientData, 
	  kva_list *list)
{
   int typ = kva_arg(*list, int);

   if (!kdms_query_segment(obj, KPDS_SEGMENT_VALUE))
      return FALSE;

   if (kdms_query_segment(obj, KPDS_SEGMENT_MAP) && kpds_is_mapped(obj))
   {
      if (!kdms_set_attribute(obj, KPDS_SEGMENT_MAP, KDMS_DATA_TYPE, typ))
	    return FALSE;
   }
   else
   {
      if (!kdms_set_attribute(obj, KPDS_SEGMENT_VALUE, KDMS_DATA_TYPE, typ))
	    return FALSE;
   }

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _type_match
|       Purpose: match the type attribute
|    Written By: Steve Kubica and John Salas
|          Date: April 28, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_type_match(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	    kaddr clientData1, kaddr clientData2)
{
   int typ1;
   int typ2;

   if (!kdms_query_segment(obj1, KPDS_SEGMENT_VALUE) ||
       !kdms_query_segment(obj2, KPDS_SEGMENT_VALUE))
      return FALSE;

   /*-- use pds to handle mapping mode --*/
   if (!kpds_get_attribute(obj1, KPDS_VALUE_DATA_TYPE, &typ1))
      return FALSE;

   if (!kpds_get_attribute(obj2, KPDS_VALUE_DATA_TYPE, &typ2))
      return FALSE;
   
   return (typ1 == typ2);
}

/*-----------------------------------------------------------
|  Routine Name: (static) _type_copy
|       Purpose: copy the type attribute
|    Written By: Steve Kubica
|          Date: April 28, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_type_copy(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
	   kaddr clientData1, kaddr clientData2)
{
   int typ;

   if (!kdms_query_segment(obj1, KPDS_SEGMENT_VALUE) ||
       !kdms_query_segment(obj2, KPDS_SEGMENT_VALUE))
      return FALSE;

   /*-- use pds to handle mapping mode --*/
   if (!kpds_get_attribute(obj1, KPDS_VALUE_DATA_TYPE, &typ))
      return FALSE;

   if (!kpds_set_attribute(obj2, KPDS_VALUE_DATA_TYPE, typ))
      return FALSE;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _type_query
|       Purpose: query the type attribute
|    Written By: Steve Kubica
|          Date: March 30, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_type_query(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
	    int *numargs, int *argsize, int *datatype, int *permanent)
{
   if (!kdms_query_segment(obj, KPDS_SEGMENT_VALUE))
      return FALSE;

   if (numargs)
      *numargs = 1;
   if (argsize)
      *argsize = 1;
   if (datatype)
      *datatype = KINT;
   if (permanent)
      *permanent = TRUE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _type_print
|       Purpose: print the type attribute
|    Written By: Steve Kubica
|          Date: March 30, 1994
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_type_print(kobject obj, ktoken assoc, ktoken attrib, kaddr clientData,
	    kfile *outfile)
{
   int typ;

   if (!kdms_query_segment(obj, KPDS_SEGMENT_VALUE))
      return FALSE;

   /*-- use pds to handle mapping mode --*/
   if (!kpds_get_attribute(obj, KPDS_VALUE_DATA_TYPE, &typ))
      return FALSE;

   if (outfile)
      kfprintf(outfile, "%s (%d)", kdefine_to_datatype(typ), typ);

   return TRUE;
}

/****    =============================================================
 ****    VALUE segment handlers
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: (static) _create_value
|
|       Purpose: This routine will create the value segment and
|		 initialize its dimensionality and index order.
|
|         Input: obj     - the obj in which to create the value segment
|		 segment - the name of the value segment.
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica & Jeremy Worley
|          Date: June 1, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static int
_create_value(
   kobject  obj,
   char    *segment)
{
   int  order[KPDS_VALUE_DIM] = {KWIDTH, KHEIGHT, KDEPTH, KTIME, KELEMENTS};
   int *vorder = NULL;
   int  oldmode;
   int  w = -1;
   int 	h = -1;
   int 	d = -1;
   int 	t = -1;
   int 	e = -1;


   if (!kdms_create_segment(obj, KPDS_SEGMENT_VALUE))
      return FALSE;
   
   if (!kdms_set_attributes(obj, KPDS_SEGMENT_VALUE, 
			         KDMS_DIMENSION, KPDS_VALUE_DIM,
			    	 KDMS_INDEX_ORDER, order, 
			    	 NULL))
      return FALSE;

   /*-- don't want esize to be mapped --*/
   kpds_get_attribute(obj, KPDS_MAPPING_MODE, &oldmode);
   kpds_set_attribute(obj, KPDS_MAPPING_MODE, KUNMAPPED);
      
   /*-- get initial size and index order from existing segments --*/
   if (kdms_query_segment(obj, KPDS_SEGMENT_MASK))
   {
      kpds_get_attribute(obj, KPDS_MASK_SIZE, &w, &h, &d, &t, &e);

      kdms_get_attribute(obj, KPDS_SEGMENT_MASK, KDMS_INDEX_ORDER, &vorder);
      kdms_set_attribute(obj, KPDS_SEGMENT_VALUE,  KDMS_INDEX_ORDER, vorder);
   }
   else if (kdms_query_segment(obj, KPDS_SEGMENT_LOCATION))
   {
      kpds_get_attribute(obj, KPDS_LOCATION_SIZE, &w, &h, &d, NULL);

      kdms_get_attribute(obj, KPDS_SEGMENT_LOCATION, KDMS_INDEX_ORDER,&vorder);
      order[0] = vorder[0];
      order[1] = vorder[1];
      order[2] = vorder[2];
      order[3] = KTIME;
      order[4] = KELEMENTS;
      kdms_set_attribute(obj, KPDS_SEGMENT_VALUE, KDMS_INDEX_ORDER, order);
   }

   else if (kdms_query_segment(obj, KPDS_SEGMENT_TIME))
      kpds_get_attribute(obj, KPDS_TIME_SIZE, &t);

   else if (kdms_query_segment(obj, KPDS_SEGMENT_MAP))
      kpds_get_attribute(obj, KPDS_MAP_SIZE, NULL, NULL, &d, &t, &e);
   

   /*-- set initial size of the value segment --*/
   kpds_set_attribute(obj, KPDS_VALUE_SIZE, w, h, d, t, e);

   /*-- restore mapping mode --*/
   kpds_set_attribute(obj, KPDS_MAPPING_MODE, oldmode);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _get_value_data
|
|       Purpose: This routine gets data from the value segment.
|                It is "map-aware" in that it understands the 
|		 mapping mode and will map the value data
|		 appropriately.
|
|         Input: object  - the object to retrieve data from
|	         segment - segment to retrieve data from
|		 begin   - the begin point for region to get :
|			   if the data is to be mapped, this
|		           begin point represents the mapped begin
|		 end     - the end point for the region to get :
|			   if the data is to be mapped, this
|		           end point represents the mapped end
|
|        Output: data   - pointer to already allocated data,
|			  or NULL if we should perform the allocation
|
|       Returns: pointer to the data being returned on success,
|		 NULL otherwise
|
|    Written By: Steve Kubica
|          Date: April 29, 1994
| Modifications:
|
------------------------------------------------------------*/
static kaddr
_get_value_data(
   kobject obj,
   char   *segment,
   int    *begin,
   int    *end,
   kaddr   data)
{
   kaddr vdata;
   kaddr rdata;

   /*-- verify that value segment exists --*/
   if (!kdms_query_segment(obj, segment))
      return NULL;
   
   if (kpds_is_mapped(obj))
      vdata = kpds_get_mapped_value_data(obj, begin, end, data);
   else
      vdata = kdms_get_data(obj, segment, begin, end, data);

   if (kpds_is_masked(obj) == KUSE_SUBSTITUTE_VALUE)
      rdata = kpds_mask_data(obj, begin, end, vdata);
   else
      rdata = vdata;

   if (kpds_elements_first(obj))
   {
      kaddr ldata;
      
      ldata = kpds_transpose_elements_first(rdata, begin, end, _get_type(obj),
					    KPDS_VALUE_DIM);
      return ldata;
   }
      
   return rdata;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _put_value_data
|
|       Purpose: This routine puts data into the value segment.
|                It is "map-aware" in that it understands the 
|		 mapping mode. It actually ignores the mapping 
|		 mode, since it is meaningless for a put,
|		 but it know enough to give debug warnings.
|
|         Input: obj - the object to put data into
|	         segment - the segment to put the data into
|		 begin   - the begin point for the put
|		 end     - the end point for the put
|		 data    - the data to be put
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: April 29, 1994
| Modifications:
|
------------------------------------------------------------*/
static int
_put_value_data(
   kobject obj,
   char   *segment,
   int    *begin,
   int    *end,
   kaddr   data)
{
   int status;
   kaddr pdata = data;
   
   /*-- verify that the value segment exists --*/
   if (!kdms_query_segment(obj, segment))
      return FALSE;

   /*-- transpose if elements are first --*/
   if (kpds_elements_first(obj))
      pdata = kpds_transpose_elements_last(data, begin, end, _get_type(obj),
					   KPDS_VALUE_DIM);

   /*-- put the data the same way, ignoring mapping mode --*/
   status = kdms_put_data(obj, segment, begin, end, pdata);
   if (pdata != data) kfree(pdata);
   return status;
}


/****    =============================================================
 ****    PRIVATE VALUE functions
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kpds_init_value_segment()
|
|       Purpose: Define the value segment.
|
|         Input: none
|
|        Output: none
|
|       Returns: none
|
|    Written By: Steve Kubica
|          Date: December 26, 1994
| Modifications:
|
------------------------------------------------------------*/
void
kpds_init_value_segment(void)
{
   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_VALUE_SIZE, NULL,
			       _size_get,   _size_set, 
			       _size_match, _size_copy, 
			       _size_query, _size_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, KPDS_VALUE_DATA_TYPE, NULL,
			       _type_get,   _type_set, 
			       _type_match, _type_copy, 
			       _type_query, _type_print);

   kdms_define_segment(KPDS_SEGMENT_VALUE, _create_value, NULL, NULL,
		       _get_value_data, _put_value_data,
		       NULL, NULL, NULL, NULL, NULL, NULL);
   return;
}

/****    =============================================================
 ****    PUBLIC VALUE functions
 ****    =============================================================
 ****/

/************************************************************
*
*  Routine Name: kpds_create_value - create a value segment within a 
*				     data object.
*
*       Purpose: This function is used to create a value segment
*		 within a specified data object.  The size of the
*		 value segment will be initialized to match any
*		 the sizes shared with any other polymorphic segments.
*
*		 It is considered an error to create a value segment
*		 if the object already contains one.
*
*         Input: object  - object in which to create the value segment.
*
*        Output: none
*
*       Returns: TRUE if the value segment was successfully created, 
*		 FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 20, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_create_value(
*		!   kobject object)
*
*************************************************************/
int
kpds_create_value(kobject object)
{
   kpds_init();
   
   return kdms_create_segment(object, KPDS_SEGMENT_VALUE);
}

/************************************************************
*
*  Routine Name: kpds_query_value - determine if the value segment
*				    exists in a data object.
*
*       Purpose: This function is used to determine if the value
*		 segment exists in a data object.  If value segment
*		 exists in the specified object, then this function
*		 will return TRUE.  If the object is invalid, or value
*		 data does not exist in the object, then this function
*		 will return FALSE.
*
*         Input: object  - data object to be queried.
*
*        Output: none
*
*       Returns: TRUE if the value segment exists, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 20, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_query_value(
*		!   kobject object)
*
*************************************************************/
int
kpds_query_value(kobject object)
{
   kpds_init();

   return kdms_query_segment(object, KPDS_SEGMENT_VALUE);
}

/************************************************************
*
*  Routine Name: kpds_destroy_value - destroy the value segment in a
*				      data object.
*
*       Purpose: This function is used to destroy the value segment
*		 contained within an object.  Once the value segment
*		 has been destroyed, any data or attributes associated
*		 with the value data will be lost forever.  A new
*		 value segment can be created in its place with the
*		 function kpds_create_value.
*
*		 If the value segment does not exist in the specified
*		 object, it is considered to be an error.
*
*         Input: object  - object from which to remove the value segment.
*
*        Output: value segment is destroyed
*
*       Returns: TRUE if the value segment is successfully destroyed, 
*		 FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 20, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_destroy_value(
*		!   kobject object)
*
*************************************************************/
int
kpds_destroy_value(kobject object)
{
   kpds_init();
   
   return kdms_destroy_segment(object, KPDS_SEGMENT_VALUE);
}

/************************************************************
*
*  Routine Name: kpds_initialize_value - preset value segment to constant
*
*       Purpose: This routine initializes the entire value segment to
*		 a specified value.  The last two arguments are real
*		 and imaginary components of a complex number that is
*		 the value that the data will be initialized to.  The
*		 data type of the value segment at the time this
*		 function is called will determine how the complex
*		 number is used.  A non-complex datatype means that
*		 the real part only will be used.  A
*		 non-floating-point data type means that the integer
*		 part of the real argument will be used.
*
*         Input: object - object to initialize value data on
*		 real   - real part of constant
*		 imag   - imaginary part of constant
*
*        Output: value segment is modified in place
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions:
*    Written By: Jeremy Worley
*          Date: August 3, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_initialize_value(
*                !   kobject object,
*                !   double real,
*                !   double imag)
*
*************************************************************/
int
kpds_initialize_value(
   kobject object, 
   double  real, 
   double  imag)
{
   int  status = TRUE;
   int  oldmode;

   kpds_init();
   
   /*-- don't want esize to be mapped --*/
   status &= kpds_get_attribute(object, KPDS_MAPPING_MODE, &oldmode);
   status &= kpds_set_attribute(object, KPDS_MAPPING_MODE, KUNMAPPED);
   
   status &= kdms_initialize_segment(object, KPDS_SEGMENT_VALUE, real, imag);

   /*-- restore mapping mode --*/
   status &= kpds_set_attribute(object, KPDS_MAPPING_MODE, oldmode);

   return status;
}

/************************************************************
*
*  Routine Name: kpds_copy_value_data - copy all value data from
*                                       one object to another object.
*
*       Purpose: This function copies all of the data contained in the
*		 value segment of one object into the value segment
*		 contained in another object.  If the value segment
*		 exists in the destination object, then its data will
*		 be replaced with the data from the source object.  If
*		 the value segment does not exist in the destination
*		 object, it will be created.
*
*		 The value data can be optionally copied through the
*		 presentations of the two data objects.  This implies
*		 that any presentation stages which are normally
*		 invoked during a kpds_get_data on the source object
*		 or a kpds_put_data call on the destination object
*		 will be used for the copy.
*
*		 Any of the following presentation stages can be
*		 invoked on either the source object or the
*		 destination object : casting, scaling, normalizing,
*		 padding, and interpolating.
*		 
*		 The following presentation stages can be invoked on
*		 only the source object : mapping, axis assignment,
*		 position, and offset.  The resulting copy will be
*		 mapped through the map and transposed to be in the
*		 default axis ordering.  The resulting copy will also
*		 appear to be shifted by the source position and
*		 offset.
*
*		 These presentation stages are brought into the data
*		 pipeline by setting the appropriate presentation
*		 attributes on the data objects. Please see the
*		 chapter on polymorphic data services in the data
*		 services manual for more information on the various
*		 presentation attributes.
*
*		 The following example may help in visualizing the
*		 process.  The source object has mapping mode set
*		 to map the data, a presentation data type different
*		 from the physical data type, and a presentation size
*		 different from the physical size.  The destination
*		 object only has a different presentation and physical
*		 data type.
*                .begin code
*		 !              _______________________
*		 !             | _____________________ |
*		 !        _____||______  ---->        ||
*		 !       |___mapping___|              ||
*		 !        _____||______               ||
*		 !       |interpolating|              ||
*		 !        _____||______          _____||______
*		 !       |___casting___|        |___casting___|
*		 !         ____||_____            ____||_____
*		 !        |           |          |           |
*		 !        |  source   |        	 |destination|
*		 !        |   data    |        	 |   data    |
*		 !        |___________|        	 |___________|
*	         !
*                .end code
*		 In the above example, the data resulting from the
*		 copy will be interpolated to a new size, cast to a
*		 new data type, and mapped through the object's map
*		 before being copied into the destination object.
*		 When being copied into the destination object, the
*		 data will be cast yet again before it finally is
*		 stored.
*
*		 If the presentation and physical layers of the
*		 destination object are coupled then the destination
*		 attributes will be ignored and the source
*		 presentation attributes will be propogated to the
*		 destination physical layer.  The copy, in essence,
*		 will be performed only through the source
*		 presentation, with the destination physical and
*		 presentation layers taking on the characteristics of
*		 the source presentation.  By default, output objects
*		 are not coupled, so this behavior is typical.
*		 
*		 The copy need not be performed using the presentation.
*		 If the data is not copied through the presentation,
*		 the destination data will be a direct copy of the
*		 physical source data.  The physical size and data
*		 type of the value segment will be reflected in the
*		 destination data object from the source to the
*		 destination since they describe the physical state of
*		 the data.
*
*         Input: object1 - the object that serves as a source for the data.
*
*		 copy_through_presentation - if set to TRUE, the copy will
*			 		     be performed through the
*				  	     presentation of the source 
*					     and destination objects.
*					     if set to FALSE, the copy will
*					     be a direct copy of the physical
*					     data.
*
*        Output: object2 - the object that will serve as a destination for
*			   the copy operation.
*
*       Returns: TRUE if copy was successful, FALSE otherwise
*
*  Restrictions:
*    Written By: Jeremy Worley and Steve Kubica
*          Date: January 2, 1995
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_copy_value_data(
*               !   kobject object1,
*               !   kobject object2,
*		!   int     copy_through_presentation)
*
*************************************************************/
int
kpds_copy_value_data(
   kobject object1,
   kobject object2,
   int     copy_through_presentation)
{
   kpds_init();
   
   if (!copy_through_presentation)
      return kaps_copy_beneath_pres(object1, object2, KPDS_SEGMENT_VALUE);
   else
      return kaps_copy_through_pres(object1, object2, KPDS_SEGMENT_VALUE);
}

/************************************************************
*
*  Routine Name: kpds_copy_value_attr - copy all value attributes from
*	                                one object to another object.
*
*       Purpose: This function copies all the value attributes from 
*		 one object to another object.
*
*		 If the destination object does not contain a value
*		 segment, then this function will create a value
*		 segment, and initialize its size and data type
*		 attributes to those in the source object.  If the
*		 value data already exists in the destination object,
*		 then the presentation attributes will be set to the
*		 source object's settings.
*
*		 The destination's physical attributes will change
*		 depending on the coupling of the data objects.
*		 If the destination is coupled, then the physical
*		 attributes will be changed as well.  Any data
*		 contained in the destination will be cast, rescaled,
*		 or resized to match its new physical attributes.
*
*         Input: source_object - the object that serves as a
*				 source for the attributes.
*
*        Output: destination_object - the object that serves as
*				      a destination for the
*				      operation.
*
*       Returns: TRUE if copy was successful, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 19, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_copy_value_attr(
*		!   kobject object1,
*		!   kobject object2)
*
*************************************************************/
int
kpds_copy_value_attr(
   kobject object1,
   kobject object2)
{
   kpds_init();
   
   return kdms_copy_segment_attributes(object1, KPDS_SEGMENT_VALUE,
				       object2, KPDS_SEGMENT_VALUE);
}

/************************************************************
*
*  Routine Name: kpds_copy_value - copy the value segment from
*		                   one object to another.
*
*       Purpose: This function copies all of the attributes and data
*		 contained in the value segment of one object into the
*		 value segment contained in another object.  If the
*		 value segment exists in the destination object, then
*		 its data will be replaced with the data from the
*		 source object.  If the value segment does not exist
*		 in the destination object, it will be created.
*
*		 All value segment attributes will be copied from one
*		 object to the other.  This includes presentation
*		 attributes, such as position and offset.
*
*		 The value data can be optionally copied through the
*		 presentations of the two data objects.  This implies
*		 that any presentation stages which are normally
*		 invoked during a kpds_get_data on the source object
*		 or a kpds_put_data call on the destination object
*		 will be used for the copy.
*
*		 Any of the following presentation stages can be
*		 invoked on either the source object or the
*		 destination object : casting, scaling, normalizing,
*		 padding, and interpolating.
*		 
*		 The following presentation stages can be invoked on
*		 only the source object : mapping, axis assignment,
*		 position, and offset.  The resulting copy will be
*		 mapped through the map and transposed to be in the
*		 default axis ordering.  The resulting copy will also
*		 appear to be shifted by the source position and
*		 offset.
*
*		 These presentation stages are brought into the data
*		 pipeline by setting the appropriate presentation
*		 attributes on the data objects. Please see the
*		 chapter on polymorphic data services in the data
*		 services manual for more information on the various
*		 presentation attributes.
*
*		 The following example may help in visualizing the
*		 process.  The source object has mapping mode set
*		 to map the data, a presentation data type different
*		 from the physical data type, and a presentation size
*		 different from the physical size.  The destination
*		 object only has a different presentation and physical
*		 data type.
*                .begin code
*		 !              _______________________
*		 !             | _____________________ |
*		 !        _____||______  ---->        ||
*		 !       |___mapping___|              ||
*		 !        _____||______               ||
*		 !       |interpolating|              ||
*		 !        _____||______          _____||______
*		 !       |___casting___|        |___casting___|
*		 !         ____||_____            ____||_____
*		 !        |           |          |           |
*		 !        |  source   |        	 |destination|
*		 !        |   data    |        	 |   data    |
*		 !        |___________|        	 |___________|
*	         !
*                .end code
*		 In the above example, the data resulting from the
*		 copy will be interpolated to a new size, cast to a
*		 new data type, and mapped through the object's map
*		 before being copied into the destination object.
*		 When being copied into the destination object, the
*		 data will be cast yet again before it finally is
*		 stored.
*
*		 If the presentation and physical layers of the
*		 destination object are coupled then the destination
*		 attributes will be ignored and the source
*		 presentation attributes will be propogated to the
*		 destination physical layer.  The copy, in essence,
*		 will be performed only through the source
*		 presentation, with the destination physical and
*		 presentation layers taking on the characteristics of
*		 the source presentation.  By default, output objects
*		 are not coupled, so this behavior is typical.
*		 
*		 The copy need not be performed using the presentation.
*		 If the data is not copied through the presentation,
*		 the destination data will be a direct copy of the
*		 physical source data.  The physical size and data
*		 type of the value segment will be reflected in the
*		 destination data object from the source to the
*		 destination since they describe the physical state of
*		 the data.
*
*		 This function is equivalent to performing successive
*		 calls to kpds_copy_value_attr and kpds_copy_value_data.
*
*         Input: object1 - the object that serves as a source for the value
*			   segment.
*
*		 copy_through_presentation - if set to TRUE, the copy will
*			 		     be performed through the
*				  	     presentation of the source 
*					     and destination objects.
*					     if set to FALSE, the copy will
*					     be a direct copy of the physical
*					     data.
*
*        Output: object2 - the object that will serve as a destination for
*			   the copy operation.
*
*       Returns: TRUE if copy was successful, FALSE otherwise
*
*  Restrictions:
*    Written By: Jeremy Worley & Steve Kubica
*          Date: September 19, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kpds_copy_value(
*               !   kobject object1,
*               !   kobject object2,
*		!   int     copy_through_presentation)
*
*************************************************************/
int
kpds_copy_value(
   kobject object1,
   kobject object2,
   int     copy_through_presentation)
{
   kpds_init();
   
   return (kpds_copy_value_attr(object1, object2) &&
	   kpds_copy_value_data(object1, object2, copy_through_presentation));
}
