 /*
  * Khoros: $Id: polymorph.c,v 1.16.2.2 1997/03/11 17:15:37 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: polymorph.c,v 1.16.2.2 1997/03/11 17:15:37 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Polymorphic Data Services
   >>>>		
   >>>>  Private:
   >>>>                 kpds_init()
   >>>>                 kpds_get_histogram()
   >>>>                 kpds_get_region()
   >>>>                 kpds_put_region()
   >>>>   Static:
   >>>>                 _kpds_lookup_primitive()
   >>>>                 _nop_set()
   >>>>                 _nop_copy()
   >>>>                 _line_info_get()
   >>>>                 _line_info_match()
   >>>>                 _line_info_query()
   >>>>                 _line_info_print()
   >>>>                 _plane_info_get()
   >>>>                 _plane_info_match()
   >>>>                 _plane_info_query()
   >>>>                 _plane_info_print()
   >>>>                 _volume_info_get()
   >>>>                 _volume_info_match()
   >>>>                 _volume_info_query()
   >>>>                 _volume_info_print()
   >>>>                 _vector_info_get()
   >>>>                 _vector_info_match()
   >>>>                 _vector_info_query()
   >>>>                 _vector_info_print()
   >>>>                 _region_info_get()
   >>>>                 _region_info_match()
   >>>>                 _region_info_query()
   >>>>                 _region_info_print()
   >>>>                 _optimal_region_constraint_get()
   >>>>                 _optimal_region_constraint_set()
   >>>>                 _optimal_region_constraint_match()
   >>>>                 _optimal_region_constraint_query()
   >>>>                 _optimal_region_constraint_print()
   >>>>                 _optimal_region_flags_get()
   >>>>                 _optimal_region_flags_set()
   >>>>                 _optimal_region_flags_match()
   >>>>                 _optimal_region_flags_query()
   >>>>                 _optimal_region_flags_print()
   >>>>                 _optimal_region_size_get()
   >>>>                 _optimal_region_size_match()
   >>>>                 _optimal_region_size_query()
   >>>>                 _optimal_region_size_print()
   >>>>                 _kpds_region_corners()
   >>>>   Public:
   >>>>                 kpds_get_attribute()
   >>>>                 kpds_get_attributes()
   >>>>                 kpds_set_attribute()
   >>>>                 kpds_set_attributes()
   >>>>                 kpds_match_attribute()
   >>>>                 kpds_match_attributes()
   >>>>                 kpds_copy_attribute()
   >>>>                 kpds_copy_attributes()
   >>>>                 kpds_query_attribute()
   >>>>                 kpds_print_attribute()
   >>>>                 kpds_get_data()
   >>>>                 kpds_put_data()
   >>>>                 kpds_open_object()
   >>>>                 kpds_reference_object()
   >>>>                 kpds_copy_global_attr()
   >>>>                 kpds_copy_object_attr()
   >>>>                 kpds_list_output_formats()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

int pds_initialized = FALSE;

extern char *pds_mask_string;
extern char *pds_value_string;
extern char *pds_location_string;
extern char *pds_map_string;
extern char *pds_time_string;

/*
 *    ==================================================================
 *    Initialization Routine
 *    ==================================================================
 */

/*-----------------------------------------------------------
|
|  Routine Name: kpds_init
|
|       Purpose: 
|
|		 We don't need:	KPDS_NAME, KPDS_ARCHITECTURE
|		 because they are #defined to KDMS_*,
|		 have exactly the same arguments, and are
|		 object level attributes.
|
|         Input: 
|
|        Output: 
|
|       Returns: nothing
|
|    Written By: Jeremy Worley & John Salas
|          Date: September 28, 1993
| Modifications:
|
------------------------------------------------------------*/
int
kpds_init(void)

{
   if (pds_initialized)
      return TRUE;
   else
      pds_initialized = TRUE;

   /*-- initialize data managment services first --*/
   kdms_init();

   /*-- initialize data formats --*/
   init_viff();
   init_gdbm();
   init_pnm();
   init_rast();
   init_xbm();
   init_xpm();
   init_xwd();
   init_arf();
   init_ascii();
   init_avs();
   init_eps();

   /*--initalize handy tokenized versions of the polymorphic segment names --*/
   kpds_init_segment_tokens();

   /*-- initialize errnos --*/
   kpds_init_errors();

   /*-- initialize the data segments --*/
   kpds_init_value_segment();
   kpds_init_mask_segment();
   kpds_init_map_segment();
   kpds_init_location_segment();
   kpds_init_time_segment();

   /*-- initialize the primitive table --*/
   kpds_init_primitive_table();

   /*-- initialize attributes --*/
   kpds_init_attributes();
   kpds_init_position_attributes();
   kpds_init_info_attributes();

   /*-------------------------------------------------------------
   |  it is possible to generate errnos with duplicate attribute
   |  definitions.  to prevent the user from seeing this, we'll
   |  re-zero the errno.
   \-------------------------------------------------------------*/
   errno = 0;

   return TRUE;
}

/*
 *    ==================================================================
 *    Public API Routines
 *    ==================================================================
 */

/************************************************************
*
*  Routine Name: kpds_get_attribute - get the value of an attribute
*  		                      from a data object
*
*       Purpose: This routine is used to get the value of
*		 an attribute from a data object.
*
*		 Data Services manages two versions of some of the
*		 attributes associated with each object.  These
*		 attributes are the size and data type.  Internally,
*		 the two versions of these attributes are referred to
*		 as the physical attribute and the presentation
*		 attribute.  Typically, the programmer has access
*		 to only the presentation versions of these attributes.
*		 The physical attributes are set indirectly depending
*		 on the setting of KPDS_COUPLING.  See kpds_get_data
*		 for a description of how the presentation and physical
*		 attributes affect interaction with the data object.
*
*	         Other attributes are classified as either shared or
*	         unshared.  Shared attributes are stored at the
*	         physical layer of the attribute, and thus can be
*	         shared by multiple references of the data object (see
*	         kpds_reference_object for more information about
*	         references).  Unshared attributes, on the other hand,
*	         can only be used by the local object (see
*	         kpds_query_attribute for information on how to
*	         determine whether an attribute is shared or
*	         unshared).
*		
*		 This function accepts an object, the name of the
*		 attribute that is desired and the address of a
*		 location in which to return the value of the
*		 attribute.  Certain KPDS attributes require more than
*		 one argument in order to return the entire attribute.
*		 For example, KPDS_VALUE_SIZE requires five (5)
*		 arguments.  Getting the KPDS_VALUE_DATA_TYPE
*		 attribute might look like this:
*
*		!	kpds_get_attribute(object, KPDS_VALUE_DATA_TYPE, &typ);
*
*		 Getting the KPDS_VALUE_SIZE attribute, is a little
*		 more complex:
*
*		!	kpds_get_attribute(object, KPDS_VALUE_SIZE,
*		!		&w, &h, &d, &t, &e);
*
*		 A complete list of the polymorphic attributes can be
*		 found in Chapter 2 of Programming Services Volume II.
*
*         Input: object    - the object from which to retrieve the
*			     specified attribute.  This must be a
*			     legal kobject that has been opened or
*			     instantiated by an appropriate kpds
*			     function call, such as kpds_open_object
*			     or kpds_reference.
*		 attribute - the name of the attribute to retrieve.
*			     this is a character string that is
*			     the name of an existing attribute in
*			     the object.  There are a large number
*			     of predefined KPDS attributes.  Users
*			     can also create attributes via the
*			     kpds_create_attribute function call.
*
*        Output: va_alist   - a C variable argument list that contains
*			     addresses to locations in which 
*			     KPDS can return the value of the 
*			     specified attribute.
*
*       Returns: TRUE if the attribute was successfully retrieved,
*	 	 FALSE otherwise.
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 30, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration:
*
*************************************************************/
int
kpds_get_attribute(
   kobject object, 
   char   *attribute, 
   kvalist)
{
   kva_list list;
   int status;


   kpds_init();

   kva_start(list, attribute);

   status = kdms_vget_attribute(object, KDMS_OBJECT, attribute, &list);

   kva_end(list);

   return status;
}

/************************************************************
*
*  Routine Name: kpds_get_attributes - get the values of multiple 
*				       attributes from a data object
*
*       Purpose: This function is used to retrieve the values of an 
*		 arbitrary number of attributes from a data object.
*
*		 Data Services manages two versions of some of the
*		 attributes associated with each object.  These
*		 attributes are the size and data type.  Internally,
*		 the two versions of these attributes are referred to
*		 as the physical attribute and the presentation
*		 attribute.  Typically, the programmer has access
*		 to only the presentation versions of these attributes.
*		 The physical attributes are set indirectly depending
*		 on the setting of KPDS_COUPLING.  See kpds_get_data
*		 for a description of how the presentation and physical
*		 attributes affect interaction with the data object.
*
*	         Other attributes are classified as either shared or
*	         unshared.  Shared attributes are stored at the
*	         physical layer of the attribute, and thus can be
*	         shared by multiple references of the data object (see
*	         kpds_reference_object for more information about
*	         references).  Unshared attributes, on the other hand,
*	         can only be used by the local object (see
*	         kpds_query_attribute for information on how to
*	         determine whether an attribute is shared or
*	         unshared).
*		
*		 This function accepts an object followed by a list of
*		 of arguments that alternate between an attribute
*		 name and an address into which the attribute's value
*		 will be stored.  The last argument on this list is
*		 a NULL which serves as a flag that indicates that no
*		 more attributes are present on the list.  
*		 Certain KPDS attributes require more than one address 
*		 in order to return the entire attribute.  For example, 
*	         KPDS_VALUE_SIZE requires five (5) arguments.  For example,
*
*		!	kpds_get_attributes(object,
*		!		KPDS_VALUE_DATA_TYPE, &typ,
*		!		KPDS_VALUE_SIZE, &w, &h, &d, &t, &e,
*		!		NULL);
*
*		 The last argument to this function must be NULL.  If
*		 it is not NULL, then the behavior of this function
*		 will be unpredictable (the NULL argument is used as a
*		 sentinal to indicate the end of the variable argument
*		 list.  If the sentinal is not present, then this
*		 function will continue to attempt to pull arguments
*		 off of the stack, until it finds a NULL).
*
*		 A complete list of the polymorphic attributes can be
*		 found in Chapter 2 of Programming Services Volume II.
*
*         Input: object    - the object from which to retrieve the
*			     specified attribute.  This must be a
*			     legal kobject that has been opened or
*			     instantiated by an appropriate kpds
*			     function call, such as kpds_open_object
*			     or kpds_reference.
*
*        Output: va_alist   - a C variable argument list that contains
*			     a set of attributes, each
*			     followed by addresses to variables
*			     that will be assigned with values
*			     associated with that attribute.
*			     The variable argument list takes the
*			     form:
*
*			     ATTRIBUTE_NAME1, &value1 [, &value2, ...],
*			     ATTRIBUTE_NAME2, &value1,[, &value2, ...],
*			     ..., NULL.
*
*			     The number of value arguments in
*			     the variable argument list for each
*			     attribute depends on the specific
*			     attribute.  For
*			     example, KPDS_VALUE_DATA_TYPE takes only
*			     one value, but KPDS_VALUE_SIZE takes
*			     multiple values.  The NULL at the
*			     end of the variable argument
*			     list serves as a flag indicating
*			     the end of the list to kdms_get_attributes.
*
*       Returns: TRUE if the attribute was successfully retrieved,
*	 	 FALSE otherwise.
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 30, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration:
*
*************************************************************/
int
kpds_get_attributes(
   kobject object,
   kvalist)
{
   kva_list list;
   int status;


   kpds_init();

   kva_start(list, object);

   status = kdms_vget_attributes(object, KDMS_OBJECT, &list);

   kva_end(list);

   return status;
}

/************************************************************
*
*  Routine Name: kpds_set_attribute - set the values of an 
*                                     attribute in a data object
*
*       Purpose: This function is used to assign the value of a
*	 	 attribute to a data object.
*
*		 Data Services manages two versions of some of the
*		 attributes associated with each object.  These
*		 attributes are the size and data type.  Internally,
*		 the two versions of these attributes are referred to
*		 as the physical attribute and the presentation
*		 attribute.  Typically, the programmer has access
*		 to only the presentation versions of these attributes.
*		 The physical attributes are set indirectly depending
*		 on the setting of KPDS_COUPLING.  See kpds_get_data
*		 for a description of how the presentation and physical
*		 attributes affect interaction with the data object.
*
*	         Other attributes are classified as either shared or
*	         unshared.  Shared attributes are stored at the
*	         physical layer of the attribute, and thus can be
*	         shared by multiple references of the data object (see
*	         kpds_reference_object for more information about
*	         references).  Unshared attributes, on the other hand,
*	         can only be used by the local object (see
*	         kpds_query_attribute for information on how to
*	         determine whether an attribute is shared or
*	         unshared).
*		
*		 This function accepts an object, the name of the
*		 attribute that is desired and the value of the
*		 attribute.  Certain KPDS attributes require more than
*		 one value in order to set the entire attribute.  For
*		 example, KPDS_VALUE_SIZE requires five (5) arguments.
*		 For example, setting the KPDS_VALUE_DATA_TYPE
*		 attribute might look like this:
*
*		!	kpds_set_attribute(object, KPDS_VALUE_DATA_TYPE, KFLOAT);
*
*		 Setting the KPDS_VALUE_SIZE attribute, is a little
*		 more complex:
*
*		!	kpds_set_attribute(object, KPDS_VALUE_SIZE,
*		!		100, 100, 1, 1, 1);
*
*		 A complete list of the polymorphic attributes can be
*		 found in Chapter 2 of Programming Services Volume II.
*
*         Input: object    - the object in which to set the
*			     specified attribute.  This must be a
*			     legal kobject that has been opened or
*			     instantiated by an appropriate kpds
*			     function call, such as kpds_open_object
*			     or kpds_reference.
*		 attribute - the name of the attribute to set.
*			     this is a character string that is
*			     the name of an existing attribute in
*			     the object.  There are a large number
*			     of predefined KPDS attributes.  Users
*			     can also create attributes via the
*			     kpds_create_attribute function call.
*		 va_alist   - a C variable argument list that contains
*			     value or values of the specified attribute.
*
*        Output:
*
*       Returns: TRUE if the attribute was successfully set,
*	 	 FALSE otherwise.
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 30, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration:
*
*************************************************************/
int
kpds_set_attribute(
   kobject object, 
   char   *attribute, 
   kvalist)
{
   kva_list list;
   int status;


   kpds_init();

   kva_start(list, attribute);

   status = kdms_vset_attribute(object, KDMS_OBJECT, attribute, &list);

   kva_end(list);

   return status;
}

/************************************************************
*
*  Routine Name: kpds_set_attributes - set the values of multiple
*                                      attributes in a data object.
*
*       Purpose: This function is used to assign the values of an 
*		 arbitrary number of attributes to a data object.
*
*		 Data Services manages two versions of some of the
*		 attributes associated with each object.  These
*		 attributes are the size and data type.  Internally,
*		 the two versions of these attributes are referred to
*		 as the physical attribute and the presentation
*		 attribute.  Typically, the programmer has access
*		 to only the presentation versions of these attributes.
*		 The physical attributes are set indirectly depending
*		 on the setting of KPDS_COUPLING.  See kpds_get_data
*		 for a description of how the presentation and physical
*		 attributes affect interaction with the data object.
*
*	         Other attributes are classified as either shared or
*	         unshared.  Shared attributes are stored at the
*	         physical layer of the attribute, and thus can be
*	         shared by multiple references of the data object (see
*	         kpds_reference_object for more information about
*	         references).  Unshared attributes, on the other hand,
*	         can only be used by the local object (see
*	         kpds_query_attribute for information on how to
*	         determine whether an attribute is shared or
*	         unshared).
*		
*		 This function accepts an object followed by a list of
*		 of arguments that alternate between an attribute
*		 name and values that the specified attribute is to
*		 be set to.  The last argument on this list is
*		 a NULL which serves as a flag that indicates that no
*		 more attributes are present on the list.  
*		 Certain KPDS attributes require more than one value
*		 in order to return the entire attribute.  For example, 
*	         KPDS_VALUE_SIZE requires five (5) arguments.  For example,
*
*		!	kpds_set_attributes(object,
*		!		KPDS_VALUE_DATA_TYPE, KFLOAT,
*		!		KPDS_VALUE_SIZE, 100, 100, 1, 1, 1,
*		!		NULL);
*
*		 The last argument to this function must be NULL.  If
*		 it is not NULL, then the behavior of this function
*		 will be unpredictable (the NULL argument is used as a
*		 sentinal to indicate the end of the variable argument
*		 list.  If the sentinal is not present, then this
*		 function will continue to attempt to pull arguments
*		 off of the stack, until it finds a NULL).
*
*		 A complete list of the polymorphic attributes can be
*		 found in Chapter 2 of Programming Services Volume II.
*
*         Input: object    - the object in which to set the
*			     specified attribute.  This must be a
*			     legal kobject that has been opened or
*			     instantiated by an appropriate kpds
*			     function call, such as kpds_open_object
*			     or kpds_reference.
*
*        Output: va_alist  - a C variable argument list that contains
*			     a set of attributes, each
*			     followed by values will be assigned to
*			     the specified attribute.
*			     The variable argument list takes the
*			     form:
*
*			     ATTRIBUTE_NAME1, value1 [, value2, ...],
*			     ATTRIBUTE_NAME2, value1,[, value2, ...],
*			     ..., NULL.
*
*			     The number of value arguments in
*			     the variable argument list for each
*			     attribute depends on the specific
*			     attribute.  For
*			     example, KPDS_VALUE_DATA_TYPE takes only
*			     one value, but KPDS_VALUE_SIZE takes
*			     multiple values.  The NULL at the
*			     end of the variable argument
*			     list serves as a flag indicating
*			     the end of the list to kdms_set_attributes.
*
*       Returns: TRUE if the attribute was successfully retrieved,
*	 	 FALSE otherwise.
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*          Date: September 30, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration:
*
*************************************************************/
int
kpds_set_attributes(
   kobject object,
   kvalist)
{
   kva_list list;
   int status;


   kpds_init();

   kva_start(list, object);

   status = kdms_vset_attributes(object, KDMS_OBJECT, &list);

   kva_end(list);

   return status;
}

/************************************************************
*
*  Routine Name: kpds_match_attribute - returns TRUE if the
*		 same attribute in two objects match.
*
*       Purpose: 
*		 The purpose of this routine is to allow
*		 the programmer to compare a single in two data
*		 objects.
*
*		 Data Services manages two versions of some of the
*		 attributes associated with each object.  These
*		 attributes are the size and data type.  Internally,
*		 the two versions of these attributes are referred to
*		 as the physical attribute and the presentation
*		 attribute.  Typically, the programmer has access
*		 to only the presentation versions of these attributes.
*		 The physical attributes are set indirectly depending
*		 on the setting of KPDS_COUPLING.  See kpds_get_data
*		 for a description of how the presentation and physical
*		 attributes affect interaction with the data object.
*
*	         Other attributes are classified as either shared or
*	         unshared.  Shared attributes are stored at the
*	         physical layer of the attribute, and thus can be
*	         shared by multiple references of the data object (see
*	         kpds_reference_object for more information about
*	         references).  Unshared attributes, on the other hand,
*	         can only be used by the local object (see
*	         kpds_query_attribute for information on how to
*	         determine whether an attribute is shared or
*	         unshared).
*		
*		 This routine will return TRUE if the specified
*		 attribute has the same value in both of the objects.
*		 This routine will return FALSE if the attribute does
*		 not have the same value in both of of the
*		 objects. kpds_match_attribute will also return FALSE
*		 if the attribute does not exist in either or both of
*		 the objects.
*
*		 A complete list of the polymorphic attributes can be
*		 found in Chapter 2 of Programming Services Volume II.
*
*        Input:   object1 - the first object on
*			    which to match the specified attribute
*		  object2 - the second  object on
*			    which to match the specified attribute
*		  attribute - the attribute that will be
*			      compared in the two objects.
*
*        Output: none
*       Returns: There are three ways for this routine to
*		 return a FALSE:  (1) if the attribute
*		 in the two objects does not match; (2) if
*		 either object does not contain the specified 
*		 attribute; (3) an error condition
*		 resulting from an invalid object or segment.
*		 If none of these three conditions exist, then
*		 this function will return TRUE.
*
*  Restrictions: 
*
*    Written By: Jeremy Worley & John Salas
*          Date: October 18, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kpds_match_attribute(
   kobject object1, 
   kobject object2, 
   char *attr)
{
   kpds_init();
   return kdms_match_attribute(object1, object2, KDMS_OBJECT, attr);
}

/************************************************************
*
*  Routine Name: kpds_match_attributes - returns true if the
*		 list of segment attributes in two
*		 objects match.
*
*       Purpose: 
*		 The purpose of this routine is to allow
*		 the programmer to compare multiple attributes in two
*		 object.
*
*		 Data Services manages two versions of some of the
*		 attributes associated with each object.  These
*		 attributes are the size and data type.  Internally,
*		 the two versions of these attributes are referred to
*		 as the physical attribute and the presentation
*		 attribute.  Typically, the programmer has access
*		 to only the presentation versions of these attributes.
*		 The physical attributes are set indirectly depending
*		 on the setting of KPDS_COUPLING.  See kpds_get_data
*		 for a description of how the presentation and physical
*		 attributes affect interaction with the data object.
*
*	         Other attributes are classified as either shared or
*	         unshared.  Shared attributes are stored at the
*	         physical layer of the attribute, and thus can be
*	         shared by multiple references of the data object (see
*	         kpds_reference_object for more information about
*	         references).  Unshared attributes, on the other hand,
*	         can only be used by the local object (see
*	         kpds_query_attribute for information on how to
*	         determine whether an attribute is shared or
*	         unshared).
*		
*		 This routine will return TRUE if all of the specified
*		 attributes have the same value in the objects.  This
*		 routine will return FALSE if any of the attributes do
*		 not match kpds_match_attributes will also return
*		 FALSE if any of the attributes do not exist in either
*		 or both of the two objects.
*
*		 The last argument to this function must be NULL.  If
*		 it is not NULL, then the behavior of this function
*		 will be unpredictable (the NULL argument is used as a
*		 sentinal to indicate the end of the variable argument
*		 list.  If the sentinal is not present, then this
*		 function will continue to attempt to pull arguments
*		 off of the stack, until it finds a NULL).
*
*		 A complete list of the polymorphic attributes can be
*		 found in Chapter 2 of Programming Services Volume II.
*
*        Input:   object1 - the first  object on
*			    which to match the specified attributes
*		  object2 - the second  object on
*			    which to match the specified attributes
*		  va_alist - variable argument list, that
*			     contains an arbitrarily long
*			     list of attributes followed
*			     a NULL.  It takes the form:
*
*			     ATTRIBUTE_NAME1, ATTRIBUTE_NAME2, ..., NULL
*
*        Output: none
*       Returns: There are three ways for this routine to
*		 return a FALSE:  (1) if any of the attributes
*		 between the two objects do not match; (2) if
*		 either object does not contain one or more of
*		 the specified attributes; (3) an error condition
*		 resulting from an invalid object or segment.
*		 If none of these three conditions exist, then
*		 this function will return TRUE.
*
*  Restrictions: 
*
*    Written By: Jeremy Worley & John Salas
*          Date: October 18, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kpds_match_attributes(
   kobject object1, 
   kobject object2, 
   kvalist)
{
   kva_list list;
   int status;


   kpds_init();

   kva_start(list, object2);

   status = kdms_vmatch_attributes(object1, object2, KDMS_OBJECT, &list);

   kva_end(list);

   return status;
}

/************************************************************
*
*  Routine Name: kpds_copy_attribute - copy an attribute from one
*		 object to another
*
*       Purpose: 
*		 This function is used to copy a single 
*		 attribute from one object to another object.
*
*		 Data Services manages two versions of some of the
*		 attributes associated with each object.  These
*		 attributes are the size and data type.  Internally,
*		 the two versions of these attributes are referred to
*		 as the physical attribute and the presentation
*		 attribute.  Typically, the programmer has access
*		 to only the presentation versions of these attributes.
*		 The physical attributes are set indirectly depending
*		 on the setting of KPDS_COUPLING.  See kpds_get_data
*		 for a description of how the presentation and physical
*		 attributes affect interaction with the data object.
*
*	         Other attributes are classified as either shared or
*	         unshared.  Shared attributes are stored at the
*	         physical layer of the attribute, and thus can be
*	         shared by multiple references of the data object (see
*	         kpds_reference_object for more information about
*	         references).  Unshared attributes, on the other hand,
*	         can only be used by the local object (see
*	         kpds_query_attribute for information on how to
*	         determine whether an attribute is shared or
*	         unshared).
*
*		 This function accepts a source object, a
*		 destination object, and an attribute name.
*		 If the attribute exists in the source object,
*		 then it will be copied to the destination object.
*		 If the attribute does not exist in the source
*		 object, then an error condition is returned.
*		
*		 A complete list of the polymorphic attributes can be
*		 found in Chapter 2 of Programming Services Volume II.
*
*         Input: object1     - the object to copy from
*		 object2     - the object to copy to
*		 attribute   - the attribute to copy
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*
*    Written By: Steve Kubica
*          Date: May 22, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kpds_copy_attribute(
   kobject object1, 
   kobject object2, 
   char   *attribute)
{
   kpds_init();
   return (kdms_copy_attribute(object1, object2, KDMS_OBJECT, attribute));
}

/************************************************************
*
*  Routine Name: kpds_copy_attributes - copy multiple attributes
*		 from one object to another.
*
*       Purpose: 
*		 This function is used to copy multiple
*		 attributes from one object to another.  The
*		 attributes should be provided in a NULL terminated
*		 variable argument list.
*
*		 Data Services manages two versions of some of the
*		 attributes associated with each object.  These
*		 attributes are the size and data type.  Internally,
*		 the two versions of these attributes are referred to
*		 as the physical attribute and the presentation
*		 attribute.  Typically, the programmer has access
*		 to only the presentation versions of these attributes.
*		 The physical attributes are set indirectly depending
*		 on the setting of KPDS_COUPLING.  See kpds_get_data
*		 for a description of how the presentation and physical
*		 attributes affect interaction with the data object.
*
*	         Other attributes are classified as either shared or
*	         unshared.  Shared attributes are stored at the
*	         physical layer of the attribute, and thus can be
*	         shared by multiple references of the data object (see
*	         kpds_reference_object for more information about
*	         references).  Unshared attributes, on the other hand,
*	         can only be used by the local object (see
*	         kpds_query_attribute for information on how to
*	         determine whether an attribute is shared or
*	         unshared).
*
*		 This function accepts a source object, a destination
*		 object, and a list of attribute names.  The last
*		 argument to this function must be NULL.  If it is not
*		 NULL, then the behavior of this function will be
*		 unpredictable (the NULL argument is used as a
*		 sentinal to indicate the end of the variable argument
*		 list.  If the sentinal is not present, then this
*		 function will continue to attempt to pull arguments
*		 off of the stack, until it finds a NULL).  If each of
*		 the attributes exist in the source object, then it
*		 will be copied to the destination object.  If an
*		 attribute does not exist in the source object, then
*		 an error condition is returned.  In the event that an
*		 attribute does not exist, then the remainder of the
*		 attributes on the list will not be copied.
*
*		 A complete list of the polymorphic attributes can be
*		 found in Chapter 2 of Programming Services Volume II.
*
*         Input: object1     - the object to copy from
*		 object2     - the object to copy to
*		 va_alist    - NULL terminated list of attribute names to
*			       copy.
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*
*    Written By: Steve Kubica
*          Date:
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kpds_copy_attributes(
   kobject object1, 
   kobject object2, 
   kvalist)
{
   kva_list list;
   int status;


   kpds_init();

   kva_start(list, object2);

   status = kdms_vcopy_attributes(object1, object2, KDMS_OBJECT, &list);

   kva_end(list);

   return status;
}

/************************************************************
*
*  Routine Name: kpds_query_attribute - get information about an attribute
*
*       Purpose: 
*		 This function is used for two purposes: (1) to
*		 determine the existence of an attribute; and (2) to
*		 obtain the characteristics of the attribute.
*
*		 Data Services manages two versions of some of the
*		 attributes associated with each object.  These
*		 attributes are the size and data type.  Internally,
*		 the two versions of these attributes are referred to
*		 as the physical attribute and the presentation
*		 attribute.  Typically, the programmer has access
*		 to only the presentation versions of these attributes.
*		 The physical attributes are set indirectly depending
*		 on the setting of KPDS_COUPLING.  See kpds_get_data
*		 for a description of how the presentation and physical
*		 attributes affect interaction with the data object.
*
*	         Other attributes are classified as either shared or
*	         unshared.  Shared attributes are stored at the
*	         physical layer of the attribute, and thus can be
*	         shared by multiple references of the data object (see
*	         kpds_reference_object for more information about
*	         references).  Unshared attributes, on the other hand,
*	         can only be used by the local object.
*
*		 The difference between shared and unshared attributes
*		 is abstracted from the user at the PDS level.  The
*		 permanent attributes are generally shared, and the
*		 non-permanent attributes are generally non-shared.
*		 Permanent attributes are attributes that will be stored
*		 as part of an output object when it is written.  Any
*		 attributes that are retrieved when an object is opened
*		 are also permanent attributes.  Non-permanent attributes
*		 exist only while the program that is operating on the
*		 object is executing.
*
*		 The datatype argument indicates what kind of
*		 information is stored in the attribute. Attributes
*		 can be one of the following data types: KBYTE,
*		 KUBYTE, KSHORT, KUSHORT, KINT, KUINT, KLONG, KULONG,
*		 KFLOAT, KDOUBLE, KCOMPLEX, or KDCOMPLEX.  
*
*		 The num_args argument indicates how many arguments
*		 must be passed in an argument list to one of the
*		 attribute functions.
*
*		 The size arguments indicates the number of
*		 units of the data type there are in each argument.
*		 This argument allows arrays of information to be
*		 stored as attributes.
*
*         Input: object      - the object with the attribute
*		 segment     - the segment that the attribute
*			       is stored in.  If this argument
*			       is NULL, then the attribute is
*			       global to the object.
*		 name        - name of the attribute to be
*			       queried.
*		 datatype    - datatype of the attribute
*		 num_args    - number of arguments in this attribute
*		 size        - size of each argument in this
*		 	       attribute.
*		 permanent   - is the attribute stored or
*			       transient?  The return value
*			       will be either TRUE or FALSE
*
*       Returns: TRUE if attribute exists, FALSE otherwise
*
*  Restrictions: 
*
*    Written By: Steve Kubica
*          Date: May 22, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kpds_query_attribute(
   kobject object,
   char   *attribute,
   int    *num_args,
   int    *arg_size,
   int    *data_type,
   int    *permanent)
{
   kpds_init();
   return (kdms_query_attribute(object, NULL, attribute, num_args, arg_size,
				data_type, permanent));
}


/************************************************************
*
*  Routine Name: kpds_print_attribute - print the value of an
*					attribute from a
*					data object.
*
*       Purpose: This function is used to print the value of
*		 an  attribute from a data object to an output
*		 file.
*
*		 This function is typically used by such programs as
*		 kprdata to print out the values of attributes in
*		 an object.
*
*         Input: object     - the object containing the attribute
*		 attribute  - the attribute to print
*		 printfile  - the open kfile to print to
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*
*    Written By: Steve Kubica
*          Date: May 22, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kpds_print_attribute(
   kobject object,
   char   *attribute,
   kfile  *printfile)
{
   kpds_init();
   return (kdms_print_attribute(object, NULL, attribute, printfile));
}

/************************************************************
*
*  Routine Name: kpds_open_object - create an object associated
*                with an input or output transport.
*
*       Purpose: This function is used to
*                instantiate a data object (kobject) that
*                is associated with a permanent file or
*                transport.  If a permanent file is not
*                desired (i.e. the object is going to be
*                used as temporary storage, and will not
*                be used by any other process) then the
*                kpds_create_object function call should be used
*                instead.
*
*                The first argument to this function is
*                the transport or file name.  This argument
*                indicates the name of the transport that
*                is associated with the object.  The transport
*                name can be any legal khoros transport
*                description.  While this will usually be a
*                regular UNIX file name, it is also possible
*                to specify such things as shared memory pointers,
*                memory map pointers, sockets, streams, and
*                even transports on other machines.  For more
*                information on the syntax of a Khoros transport
*                name, refer to the online man page for the
*                Khoros function kopen.
*
*                The second argument to the kpds_open_object function
*                call, flags, is used to provide polymorphic services with
*                specific information about how the object
*                is going to be manipulated.  The flags argument
*                is analogous to kopen's flags argument.
*                The flags argument is constructed by bitwise OR'ing
*                predefined values from the following list:
*
*		.RS
*       	.IP "KOBJ_READ" 10
*				Open an existing file or transport
*                               for reading (input).  By using this flag,
*                               you are indicating that the file or transport
*                               exists and that it contains valid data.
*                               If it does not exist, or the data is not
*                               recognized, then an error message will
*                               be generated and this function will
*                               return KOBJECT_INVALID.
*        
*        	.IP "KOBJ_WRITE" 10
*				Open a file or transport for writing
*                               (output).  By using this flag, you are
*                               indicating that any data that you write
*                               to the object will be stored in the
*                               file or transport specified.
*        
*        	.IP "KOBJ_STREAM" 10
*				If the transport is a stream then polymorphic
*	                        services does not attempt to buffer the
*	                        data.  By setting this value, you are
*	                        indicating that that process reads and/or
*	                        writes the data strictly sequentially from the
*                               object in its native index order, and
*                               it is not necessary to buffer the data
*                               in a temporary transport that can be
*                               accessed randomly.
*        
*        	.IP "KOBJ_RAW" 10
*				When an object is opened, data
*	 			services usually attempts to recognize 
*			        the file format by examining the first 
*		  		part of the file.  By setting this value,
*	 			you will bypass this operation, forcing
*			        the file to be read as raw unformatted
*				data.
*		.RE
*
* 		 These flags can be combined arbitrarily to determine
*		 how the file or transport is opened and subsequently
*		 manipulated.  This is done by bitwise OR'ing any combination
*		 of these options.  For example KOBJ_READ | KOBJ_WRITE
*                will result in a read/write file object.
*                This implies that the file already exists and will be
*                read from using kpds_get_data and written to using
*                kpds_put_data.  When kpds_close is called, the changes
*                that are a result of calls to kpds_put_data will be
*                stored to the file or transport.
*
*                However, if you intend to open an output object, but
*                you need to occasionally read data from it that you
*                have already written, it is not necessary to specify
*                KOBJ_READ (in fact, doing so may result result in an
*                error if the file or transport does not already exist).
*
*                Likewise, it is possible to call kpds_put_data on
*                an input object (one which was opened without the
*                KOBJ_WRITE flag).  If this is done, then subsequent
*                calls to kpds_get_data on a region that has been
*                written to will contain the new data.  However,
* 		 the file or transport that is associated with this input 
* 		 object will not be changed.  Thus, the KOBJ_READ and 
*		 KOBJ_WRITE flags only indicate what operations are allowed 
*		 on the permanent file or transport that is associated with 
*		 the object, not what operations are allowable on the object 
*		 itself.
*
*                If KOBJ_READ is specified, then the Data Services
*                will attempt to recognize the file format automatically.
*                If it fails, then this function will return KOBJECT_INVALID,
*                indicating that it was unable to open the object,
*                unless the KOBJ_RAW flag was also specified,
*                in which case, it will assume that the input file
*                is simply raw data.  The structured file formats
*                that are currently recognized are VIFF (The Khoros
*                2.0 standard file format), Viff (The Khoros 1.0
*                standard file format, which was referred to as VIFF
*                in Khoros 1.0), Pnm (Portable Any Map, which includes
*                PBM, PGM, and PNM), and Sun Raster.
*
*                The default index order will be set to
*                KWIDTH, KHEIGHT, KDEPTH, KTIME and KELEMENTS for
*                the value and mask data,  to KMAP_WIDTH, KMAP_HEIGHT,
*                KMAP_ELEMENTS, KMAP_DEPTH and KMAP_TIME for the
*                map data and KWIDTH, KHEIGHT, KDEPTH and KDIMENSION
*                for the location data.  The only way to get the
*		 index order to reflect the stored index order of
*		 the data is to call kpds_sync_object.  See the
*		 man page for kpds_sync_object for more information.
*
*         Input: name - a string that contains the path name
*                       of a file or transport that will be
*                       associated with the object.
*
*                flags - how the object is to be opened.
*                        A bitwise OR of KOBJ_READ, KOBJ_WRITE,
*                        KOBJ_STREAM and KOBJ_RAW as
*                        described above.
*
*        Output: 
*       Returns: kobject on success, KOBJECT_INVALID upon failure
*
*
*  Restrictions: The KOBJ_RAW flag will have unpredictable
*                results if it is combined with the KOBJ_WRITE flag.
*                This limitation will be removed in a later release of
*                the Khoros 2.0 system.
*
*    Written By: Jeremy Worley & John Salas
*          Date: September 16, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: kobject kpds_open_object(
*		!   char *name,
*		!   int flags)
*
*************************************************************/
kobject
kpds_open_object(
   char *name, 
   int   flags)
{
   kobject tmp;
   char buf[KLENGTH];
   int vorder[5] = {KWIDTH, KHEIGHT, KDEPTH, KTIME, KELEMENTS};
   int morder[5] = {KMAP_WIDTH, KMAP_HEIGHT, KMAP_DEPTH, KMAP_TIME,
		    KMAP_ELEMENTS};
   int lorder[4] = {KWIDTH, KHEIGHT, KDEPTH, KDIMENSION};

   kpds_init();

   tmp = kdms_open(name, flags);

   if (tmp == NULL)
      return KOBJECT_INVALID;

   /*
    * set the date of the object, if it is an output object
    */
   if (flags & KOBJ_WRITE)
   {
      kaps_current_date(buf);
      kdms_set_attribute(tmp, KDMS_OBJECT, KDMS_DATE, buf);
   }

   /*
    * initialize the mask segment's pad value to 1 instead of zero.
    * Also set the index order...see below.
    */
   if (kdms_query_segment(tmp, KDMS_SEGMENT_MASK))
      kdms_set_attributes(tmp, KDMS_SEGMENT_MASK, 
			       KDMS_PAD_VALUE, 1.0, 0.0,
			       KDMS_INDEX_ORDER, vorder, NULL);

   /*
    * This little bit is here to ensure that the proper index order is
    * initialized when reading in a data set.
    */
   if (kpds_query_value(tmp))
      kdms_set_attribute(tmp, KDMS_SEGMENT_VALUE, KDMS_INDEX_ORDER, vorder);

   if (kpds_query_map(tmp))
      kdms_set_attribute(tmp, KDMS_SEGMENT_MAP, KDMS_INDEX_ORDER, morder);

   if (kpds_query_location(tmp))
      kdms_set_attribute(tmp, KDMS_SEGMENT_LOCATION, KDMS_INDEX_ORDER, 
			 lorder);

   return tmp;
}

/************************************************************
*
*  Routine Name: kpds_reference_object - create a reference of a
*		 data object.
*
*       Purpose: 
*		 This function is used to create a reference
*		 object for a data object that can be treated as a second
*		 independent data object under most circumstances.
*		 A referenced object is similar conceptually to a 
*		 symbolic link in a UNIX file system in most 
*		 respects.  For example, getting data from an
*		 input object and a reference of the object will
*		 result in the same data.  Data that is put on an
*		 output object can then be accessed from any of that 
*		 object's references.
*
*		 The similarity ends there.  Once an object is
*		 referenced, the two resulting objects are 
*		 equivalent--there is no way to distinguish which was
*		 the original.  In fact, closing the original does
*		 not in any way affect the reference, and visa-versa.
*
*		 kpds_reference_object creates a new object that has
*		 presentation attributes that are independent
*		 of the original object's presentation attributes.
*                The presentation attributes are UNCOUPLED from
*                the physical attributes, see the description
*                found in Table 4 in Chapter 6 of the the Khoros Programmer's
*                Manual on the KPDS_COUPLING attribute for more
*                information.  The two objects (or more if there
*		 are several calls to kpds_reference_object) share all
*		 physical data.
*
*                The default index order will be set to
*                KWIDTH, KHEIGHT, KDEPTH, KTIME and KELEMENTS for
*                the value and mask data,  to KMAP_WIDTH, KMAP_HEIGHT,
*                KMAP_ELEMENTS, KMAP_DEPTH and KMAP_TIME for the
*                map data and KWIDTH, KHEIGHT, KDEPTH and KDIMENSION
*                for the location data.
*
*         Input: object - the object to be
*		          referenced.
*
*        Output: 
*       Returns: a kobject that is a reference of the input
*		 object on success, KOBJECT_INVALID upon failure
*
*  Restrictions: 
*
*    Written By: Jeremy Worley & John Salas
*          Date: September 16, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: kobject kpds_reference_object(
*		!   kobject object)
*
*************************************************************/
kobject
kpds_reference_object(kobject object)
{
   kpds_init();
   
   return kdms_reference(object);
}

/************************************************************
*
*  Routine Name: kpds_copy_global_attr - copy all global
*		 attributes from one data object to another.
*
*       Purpose: This function copies all the
*		 object level attributes from the source_object
*		 to the destination_object.  None of the segment
*		 related attributes will be copied over.
*
*         Input: source - the object that serves as the
* 				 source for the attributes.
*
*        Output: destination - the object that serves as
*				      the destination for the
*				      operation.
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*    Written By: Steve Kubica
*          Date: August 19, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kpds_copy_global_attr(
   kobject source,
   kobject destination)
{
   kpds_init();

   /* copy over all of the attributes */
   kaps_copy_global_attributes(source, destination);
   return TRUE;
}

/************************************************************
*
*  Routine Name: kpds_copy_object_attr - copy all presentation
*		 attributes from one data object to another.
*
*       Purpose: This function copies all
*                presentation attributes from the source_object
*		 to the destination_object.  This means that all
*		 the attributes of the object will be copied (not
* 		 just parts of the polymorphic data model) as well.  
*		 For example, the source object may contain
*		 attributes that one of the other services (geometry,
*		 numerical, etc.) uses.  These will also be copied.
*		 Segments present in the source object will be created
*                in the destination object if they are not already present.
*
*		 There are three attributes for each data component 
*		 (i.e. Value, Mask, etc.) that are affected by this 
*		 function call in a special way:  KPDS_*_SIZE, 
*		 KPDS_*_DATA_TYPE, and KPDS_*_INDEX_ORDER.  These 
*		 attributes are used to define how the data is stored.
*		 When this function is called, these attributes will
*		 appear to change to the user, but the storage of the
*		 data will only be affected if the KPDS_COUPLING
*		 attribute is set to KCOUPLED.
*
*		 For more information on the behavior of attributes,
*		 please refer to kpds_sync_object, kpds_get_attribute
*		 and kpds_set_attribute.
*
*         Input: source - the object that serves as the
* 				 source for the attributes.
*
*        Output: destination - the object that serves as
*				      the destination for the
*				      operation.
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley & John Salas
*
*          Date: September 19, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kpds_copy_object_attr(
   kobject source,
   kobject destination)
{
   kpds_init();

   /* copy over all of the attributes */
   kaps_copy_object(source, destination, TRUE, FALSE);
   return TRUE;
}


/************************************************************
*
*  Routine Name: kpds_list_output_formats - list all polymorphic output formats
*
*       Purpose: This routine is used to obtain a list of all 
*                output file formats currently supported.  It
*                is guaranteed to include all formats supported
*		 through the polymorphic data model. 
*
*         Input: none
*
*        Output: number - number of formats in the list.
*
*       Returns: an array of strings listing all formats which
*		 support output.
*
*  Restrictions: 
*    Written By: Steve Kubica
*
*          Date: March 22, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
char **
kpds_list_output_formats(int *num)
{
   kpds_init();

   return kdms_support(num, FALSE, TRUE);
}


