 /*
  * Khoros: $Id: position.c,v 1.2.2.2 1997/05/06 22:39:16 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: position.c,v 1.2.2.2 1997/05/06 22:39:16 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>         Polymophic Model Position Attributes
   >>>>         
   >>>>   Public:
   >>>>  Private:
   >>>>			kpds_get_dimension()
   >>>>			kpds_get_size()
   >>>>			kpds_get_order()
   >>>>			kpds_get_access_info()
   >>>>			kpds_is_mapped()
   >>>>			kpds_is_masked()
   >>>>			kpds_location_grid()
   >>>>			kpds_elements_first()
   >>>>			kpds_init_position_attributes()
   >>>>   Static:
   >>>>			_position_get()
   >>>>			_position_set()
   >>>>			_position_match()
   >>>>			_position_copy()
   >>>>			_position_query()
   >>>>			_position_print()
   >>>>			_offset_get()
   >>>>			_offset_set()
   >>>>			_offset_match()
   >>>>			_offset_copy()
   >>>>			_offset_query()
   >>>>			_offset_print()
   >>>>			_increment_size_get()
   >>>>			_increment_size_set()
   >>>>			_increment_size_match()
   >>>>			_increment_size_copy()
   >>>>			_increment_size_query()
   >>>>			_increment_size_print()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

extern char *pds_value_string;
extern char *pds_mask_string;
extern char *pds_location_string;
extern char *pds_map_string;
extern char *pds_time_string;

extern ktoken pds_value_token;
extern ktoken pds_mask_token;
extern ktoken pds_location_token;
extern ktoken pds_time_token;
extern ktoken pds_map_token;
extern ktoken pds_width_token;
extern ktoken pds_height_token;
extern ktoken pds_depth_token;

/*-----------------------------------------------------------
|
|  Routine Name: kpds_get_dimension
|
|       Purpose: Get the dimension of a segment while taking the 
|                mapping mode and location grid into consideration.
|
|         Input: object  - the object to query the segment from
|                segment - the tokenized name of the segment to 
|                          get the dimension of.
|        Output: none
|
|       Returns: the dimension of the segment
|
|    Written By: Steve Kubica
|          Date: February 17, 1997
|
------------------------------------------------------------*/
int 
kpds_get_dimension(
   kobject  object,
   ktoken   segment)
{
   int    dimension;
   
   if (segment == pds_value_token)
      dimension = 5;
   else if (segment == pds_mask_token)
      dimension = 5;
   else if (segment == pds_location_token)
      dimension = 4;
   else if (segment == pds_time_token)
      dimension = 1;
   else if (segment == pds_map_token)
      dimension = 5;
   else if (segment == pds_width_token)
      dimension = 1;
   else if (segment == pds_height_token)
      dimension = 1;
   else if (segment == pds_depth_token)
      dimension = 1;
   else /*-- generic segment case --*/
      if (!kdms_get_attribute(object, (char *) segment, 
			      KDMS_DIMENSION, &dimension))
         return 0;

   return dimension;
}

/*-----------------------------------------------------------
|
|  Routine Name: kpds_get_size
|
|       Purpose: Get the size of a segment while taking the 
|                mapping mode and location grid into consideration.
|
|         Input: object  - the object to query the segment from
|                segment - the tokenized name of the segment to 
|                          get the size of.
|        Output: size    - the size of the segment 
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: February 17, 1997
|
------------------------------------------------------------*/
int 
kpds_get_size(
   kobject object, 
   ktoken  segment, 
   int   **size)
{
   static int isize[KDAS_MAX_DIM];
   kstring    size_attribute;

   size_attribute = (segment == pds_value_token)    ? KPDS_VALUE_SIZE   : 
                    (segment == pds_mask_token)     ? KPDS_MASK_SIZE    : 
                    (segment == pds_location_token) ? KPDS_LOCATION_SIZE: 
                    (segment == pds_time_token)     ? KPDS_TIME_SIZE    : 
                    (segment == pds_map_token)      ? KPDS_MAP_SIZE     : NULL;
      
   if (size_attribute)
   {
      if (!kpds_get_attribute(object, size_attribute, &isize[0], 
			      &isize[1], &isize[2], &isize[3], &isize[4]))
	 return FALSE;

      if (size) *size = isize;
   }
   else
   {
      int *rsize;
      
      if (!kdms_get_attribute(object, (char *)segment, KDMS_SIZE, &rsize))
	 return FALSE;
      
      if (size) *size = rsize;
   }
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kpds_get_order
|
|       Purpose: Get the index order of a segment while taking the 
|                mapping mode and location grid into consideration.
|
|         Input: object  - the object to query the segment from
|                segment - the tokenized name of the segment to 
|                          get the size of.
|        Output: order   - the index order of the segment 
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: February 17, 1997
|
------------------------------------------------------------*/
int 
kpds_get_order(
   kobject object, 
   ktoken  segment, 
   int   **order)
{
   int *rorder;
   
   if (!kdms_get_attribute(object, (char *)segment, KDMS_INDEX_ORDER, &rorder))
      return FALSE;
      
   if (order) *order = rorder;

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kpds_get_access_info
|
|       Purpose: Get the dimensionality, current position, offset,
|		 increment size and increment state of a segment.
|
|                These are all consolidated into a single attribute
|		 to minimize the number of attribute accesses 
|		 needed for a single get data.
|	
|		 If you change the values of the arrays returned
|		 from these, you are effectively setting the attribute.
|		 If this is done carefully, a set should never be 
|		 necessary.
|
|         Input: object  - the object to query the segment from
|                segment - the tokenized name of the segment to 
|                          get the information from 
|
|        Output: dimension - the dimensionality of the segment
|		 position  - the absolute position within the segment
|		 offset    - the relative offset within the segment
|		 increment_size  - internal size to increment by
|		 increment_state - the state of the increment engine
|
|       Returns: TRUE if all is well, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: February 17, 1997
|
------------------------------------------------------------*/
int
kpds_get_access_info(
   kobject  object,
   ktoken   segment,
   int     *dimension,
   int    **position,
   int    **offset,
   int    **increment_size,
   int    **increment_state)
{
   int *access = NULL;
   int  dim = kpds_get_dimension(object, segment);

   /*------------------------------------------------------------
   |  value segment : try this first, since it will be hit
   |                  the most out of all the segments.
   \------------------------------------------------------------*/
   if (segment == pds_value_token)
   {
      if (!kdms_get_attribute(object, (char *)segment, 
			      KPDS_ACCESS_INFO, &access))
         return FALSE;
   }

   /*------------------------------------------------------------
   |  location segment : keep a single access point attribute
   |  	                 and store in the most sensible place for
   |	                 the grid type of interest. 
   \------------------------------------------------------------*/
   else if (segment == pds_location_token)
   {
      switch (kpds_location_grid(object))
      {
         case KUNIFORM :
            if (!kdms_get_attribute(object, KPDS_SEGMENT_UNIFORM,
                                    KPDS_ACCESS_INFO, &access)) 
               return FALSE;
            break;
         case KRECTILINEAR :
            if (!kdms_get_attribute(object, KPDS_SEGMENT_WIDTH,
                                    KPDS_ACCESS_INFO, &access)) 
               return FALSE;
            break;
         case KCURVILINEAR :
         case KNONE:          /*-- backwards compatibility --*/
            if (!kdms_get_attribute(object, KPDS_SEGMENT_LOCATION,
                                    KPDS_ACCESS_INFO, &access))
               return FALSE;

            break;
         default:
            return FALSE;
      }
   }

   /*------------------------------------------------------------
   |   width segments : keep a single access point attribute
   |   height         	and store in the width segment.
   |   depth           	major headache to find corresponding
   |		       	points in dataset.
   \------------------------------------------------------------*/
   else if ((segment == pds_width_token)  ||
            (segment == pds_height_token) ||
            (segment == pds_depth_token))
   {
      int shift;

      if (!kdms_get_attribute(object, KPDS_SEGMENT_WIDTH, KPDS_ACCESS_INFO, 
			      &access))
	 return FALSE;

      /*-- all rectilinear positions reside in single attribute --*/
      shift = (segment == pds_width_token)  ? 0 : 
	      (segment == pds_height_token) ? 1 : 2;

      /*-- get appropriate field from single attribute --*/
      if (dimension)       *dimension       = 1;
      if (position)        *position        = access + shift;
      if (offset)          *offset          = access + 4 + shift;
      if (increment_size)  *increment_size  = access + 8 + shift;
      if (increment_state) *increment_state = access + 12;

      return TRUE;
   }

   /*------------------------------------------------------------
   |   time, mask, and map segments : just get the access point
   \------------------------------------------------------------*/
   else /*-- any other segment will behave nicely --*/
      if (!kdms_get_attribute(object, (char *) segment, 
			      KPDS_ACCESS_INFO, &access)) 
	 return FALSE;

   if (access /* && (dim > 1)*/)
   {
      if (dimension)       *dimension       = dim;
      if (position) 	   *position        = access;
      if (offset)   	   *offset          = access + dim;
      if (increment_size)  *increment_size  = access + dim * 2;
      if (increment_state) *increment_state = access + dim * 3;
   }

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kpds_is_mapped
|
|       Purpose: This utility function returns the mapping mode
|	         It also considers the presence of a map, if there
|		 is no map segment, it will return that mapping mode is off
|		 even though it may really be turned on.
|
|         Input: obj - the object from which to query the mapping mode
|
|        Output: none
|
|       Returns: TRUE (1) if the mapping mode is on, FALSE (0) othewise
|
|    Written By: Steve Kubica
|          Date: Apr 28, 1994 15:15
| Modifications:
|
------------------------------------------------------------*/
int
kpds_is_mapped(kobject obj)
{
   int mapping;

   /* if there is no map, then pretend that mapping mode is off */
   if (!kdms_query_segment(obj, KDMS_SEGMENT_MAP))
      return FALSE;   

   if (!kdms_get_attribute(obj, NULL, KPDS_MAPPING_MODE, &mapping))
      return FALSE;

   /* a sanity check requested by donna */
   if (mapping == TRUE && !kdms_query_segment(obj, KDMS_SEGMENT_VALUE))
      kinfo(KSYSLIB,"Mapping mode has been set, but no value data exists.\n");

   return mapping;
}

/*-----------------------------------------------------------
|
|  Routine Name: kpds_is_masked
|
|       Purpose: This utility function returns the masked value
|		 presentation. It also considers the presence of a mask, 
|		 if there is no mask segment, it will return that the
|		 masked value presentation is set to use the original
|		 even though it may really be set to use replacement values.
|
|         Input: obj - the object from which to query the masked value 
|		       presentation
|
|        Output: none
|
|       Returns: TRUE (1) if the the value is to be masked and a 
|		 mask is present, FALSE (0) othewise
|
|    Written By: Steve Kubica
|          Date: Apr 28, 1994 15:15
| Modifications:
|
------------------------------------------------------------*/
int
kpds_is_masked(kobject obj)
{
   int masking;

   if (!kdms_get_attribute(obj, NULL, KPDS_MASKED_VALUE_PRESENTATION,
			   &masking))
      return KUSE_ORIGINAL;

   /* if there is no mask, then pretend that masking mode is off */
   if (!kdms_query_segment(obj, KDMS_SEGMENT_MASK))
      return KUSE_ORIGINAL;   

   return masking;
}

/*-----------------------------------------------------------
|
|  Routine Name: kpds_location_grid
|
|       Purpose: This utility function returns the location grid.
|
|         Input: obj - the object from which to query the location grid
|
|        Output: none
|
|       Returns: TRUE (1) if the mapping mode is on, FALSE (0) othewise
|
|    Written By: Steve Kubica
|          Date: Apr 28, 1994 15:15
| Modifications:
|
------------------------------------------------------------*/
int
kpds_location_grid(kobject obj)
{
   int grid;

   if (!kdms_get_attribute(obj, NULL, KPDS_LOCATION_GRID, &grid))
      return KNONE;

   return grid;
}


/*-----------------------------------------------------------
|
|  Routine Name: kpds_elements_first
|
|       Purpose: This utility function returns the elements first attribute
|
|         Input: obj - the object from which to query the elements first 
|		       attribute
|
|        Output: none
|
|       Returns: TRUE (1) if the elements first is on, FALSE (0) othewise
|
|    Written By: Steve Kubica
|          Date: Apr 28, 1994 15:15
| Modifications:
|
------------------------------------------------------------*/
int
kpds_elements_first(kobject obj)
{
   int first;

   if (!kdms_get_attribute(obj, NULL, KPDS_ELEMENTS_FIRST, &first))
      return FALSE;

   return first;
}




/*********** ---------------------------------------------------------------
 ***********  KPDS_POSITION
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _position_get
|       Purpose: get the position attribute
|    Written By: Steve Kubica
|          Date: February 18, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_position_get(kobject object, ktoken segment, ktoken attribute, 
	      kaddr client_data, kva_list *list)
{
   int *position = NULL, dimension = 0, *val, i;

   if (kpds_get_access_info(object, kstring_to_token((char *) client_data),
			    &dimension, &position, NULL, NULL, NULL))
   {
      if (dimension == 0 || position == NULL)
	 return FALSE;

      for (i = 0; i < dimension; i++)
	 if ((val = kva_arg(*list, int *)) != NULL) *val = position[i];
   }
   else
      return FALSE;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _position_set
|       Purpose: set the position 
|    Written By: Steve Kubica
|          Date: February 18, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_position_set(kobject object, ktoken segment, ktoken attribute, 
	      kaddr client_data, kva_list *list)
{
   int *position = NULL, *increment_state = NULL, dimension = 0, i;

   if (kpds_get_access_info(object, kstring_to_token((char *) client_data),
			    &dimension, &position, NULL, NULL, 
			    &increment_state))
   {
      if (dimension == 0 || position == NULL)
	 return FALSE;

      /*-- modifying this will set the internal stored array --*/
      for (i = 0; i < dimension; i++)
	 position[i] = kva_arg(*list, int);

      /*-- reinitialize the increment state --*/
      increment_state[0] = 0;
      increment_state[1] = 0;
      increment_state[2] = 0;
   }
   else
      return FALSE;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _position_match
|       Purpose: match the position attribute
|    Written By: Steve Kubica
|          Date: Februrary 18, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_position_match(kobject object1, kobject object2, ktoken segment, 
		ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   int *position1 = NULL, *position2 = NULL, dimension = 0, i;

   if (!kpds_get_access_info(object1, kstring_to_token((char *) client_data1),
			    &dimension, &position1, NULL, NULL, NULL))
      return FALSE;

   if (!kpds_get_access_info(object2, kstring_to_token((char *) client_data2), 
			     NULL, &position2, NULL, NULL, NULL))
      return FALSE;
   
   if (dimension == 0 || position1 == NULL || position2 == NULL)
      return FALSE;

   for (i = 0; i < dimension; i++)
      if (position1[i] != position2[i])
	 return FALSE;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _position_copy
|       Purpose: copy the position attribute
|    Written By: Steve Kubica
|          Date: Februrary 18, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_position_copy(kobject object1, kobject object2, ktoken segment, 
	       ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   int *position1 = NULL, *position2 = NULL, *increment_state = NULL;
   int  dimension = 0, i;

   if (!kpds_get_access_info(object1, kstring_to_token((char *) client_data1), 
			     &dimension, &position1, NULL, NULL, NULL))
      return FALSE;

   if (!kpds_get_access_info(object2, kstring_to_token((char *) client_data2), 
			     NULL, &position2, NULL, NULL, &increment_state))
      return FALSE;
   
   if (dimension == 0 || position1 == NULL || position2 == NULL)
      return FALSE;

   /*-- modifying this will set the internal stored array --*/
   for (i = 0; i < dimension; i++)
      position2[i] = position1[i];
   
   /*-- reinitialize the increment state --*/
   increment_state[0] = 0;
   increment_state[1] = 0;
   increment_state[2] = 0;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _position_query
|       Purpose: query the position attribute
|    Written By: Steve Kubica
|          Date: Februrary 18, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_position_query(kobject object, ktoken segment, ktoken attribute, 
		kaddr client_data, int *numargs, int *argsize, 
		int *datatype, int *permanent)
{
   int dimension = 0;
   
   if (!kpds_get_access_info(object, kstring_to_token((char *) client_data), 
			     &dimension, NULL, NULL, NULL, NULL))
      return FALSE;

   if (numargs)   *numargs   = dimension;
   if (argsize)   *argsize   = 1;
   if (datatype)  *datatype  = KINT;
   if (permanent) *permanent = FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _position_print
|       Purpose: print the position attribute
|    Written By: Steve Kubica
|          Date: Februrary 18, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_position_print(kobject object, ktoken segment, ktoken attribute, 
		kaddr client_data, kfile *outfile)
{
   int *position = NULL, dimension = 0, i;

   if (!kpds_get_access_info(object, kstring_to_token((char *) client_data), 
			     &dimension, &position, NULL, NULL, NULL))
      return FALSE;
   
   if (dimension == 0 || position == NULL)
      return FALSE;

   if (outfile)
      for (i = 0; i < dimension; i++)
	 kfprintf(outfile, " %d ", position[i]);

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_OFFSET
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _offset_get
|       Purpose: get the offset attribute
|    Written By: Steve Kubica
|          Date: February 18, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_offset_get(kobject object, ktoken segment, ktoken attribute, 
	    kaddr client_data, kva_list *list)
{
   int *offset = NULL, dimension = 0, *val, i;

   if (kpds_get_access_info(object, kstring_to_token((char *) client_data), 
			    &dimension, NULL, &offset, NULL, NULL))
   {
      if (dimension == 0 || offset == NULL)
	 return FALSE;

      for (i = 0; i < dimension; i++)
	 if ((val = kva_arg(*list, int *)) != NULL) *val = offset[i];
   }
   else
      return FALSE;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _offset_set
|       Purpose: set the offset 
|    Written By: Steve Kubica
|          Date: February 18, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_offset_set(kobject object, ktoken segment, ktoken attribute, 
	      kaddr client_data, kva_list *list)
{
   int *offset = NULL, dimension = 0, i;

   if (kpds_get_access_info(object, kstring_to_token((char *) client_data), 
			    &dimension, NULL, &offset, NULL, NULL))
   {
      if (dimension == 0 || offset == NULL)
	 return FALSE;

      /*-- modifying this will set the internal stored array --*/
      for (i = 0; i < dimension; i++)
	 offset[i] = kva_arg(*list, int);
   }
   else
      return FALSE;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _offset_match
|       Purpose: match the offset attribute
|    Written By: Steve Kubica
|          Date: Februrary 18, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_offset_match(kobject object1, kobject object2, ktoken segment, 
		ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   int *offset1 = NULL, *offset2 = NULL, dimension = 0, i;

   if (!kpds_get_access_info(object1, kstring_to_token((char *) client_data1),
			     &dimension, NULL, &offset1, NULL, NULL))
      return FALSE;

   if (!kpds_get_access_info(object2, kstring_to_token((char *) client_data2), 
			     NULL, NULL, &offset2, NULL, NULL))
      return FALSE;
   
   if (dimension == 0 || offset1 == NULL || offset2 == NULL)
      return FALSE;

   for (i = 0; i < dimension; i++)
      if (offset1[i] != offset2[i])
	 return FALSE;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _offset_copy
|       Purpose: copy the offset attribute
|    Written By: Steve Kubica
|          Date: Februrary 18, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_offset_copy(kobject object1, kobject object2, ktoken segment, 
	       ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   int *offset1 = NULL, *offset2 = NULL;
   int  dimension = 0, i;

   if (!kpds_get_access_info(object1, kstring_to_token((char *) client_data1),
			     &dimension, NULL, &offset1, NULL, NULL))
      return FALSE;

   if (!kpds_get_access_info(object2, kstring_to_token((char *) client_data2),
			     NULL, NULL, &offset2, NULL, NULL))
      return FALSE;
   
   if (dimension == 0 || offset1 == NULL || offset2 == NULL)
      return FALSE;

   /*-- modifying this will set the internal stored array --*/
   for (i = 0; i < dimension; i++)
      offset2[i] = offset1[i];
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _offset_query
|       Purpose: query the offset attribute
|    Written By: Steve Kubica
|          Date: Februrary 18, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_offset_query(kobject object, ktoken segment, ktoken attribute, 
		kaddr client_data, int *numargs, int *argsize, 
		int *datatype, int *permanent)
{
   int dimension = 0;
   
   if (!kpds_get_access_info(object, kstring_to_token((char *) client_data), 
			     &dimension, NULL, NULL, NULL, NULL))
      return FALSE;

   if (numargs)   *numargs   = dimension;
   if (argsize)   *argsize   = 1;
   if (datatype)  *datatype  = KINT;
   if (permanent) *permanent = FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _offset_print
|       Purpose: print the offset attribute
|    Written By: Steve Kubica
|          Date: Februrary 18, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_offset_print(kobject object, ktoken segment, ktoken attribute, 
		kaddr client_data, kfile *outfile)
{
   int *offset = NULL, dimension = 0, i;

   if (!kpds_get_access_info(object, kstring_to_token((char *) client_data), 
			     &dimension, NULL, &offset, NULL, NULL))
      return FALSE;
   
   if (dimension == 0 || offset == NULL)
      return FALSE;

   if (outfile)
      for (i = 0; i < dimension; i++)
	 kfprintf(outfile, " %d ", offset[i]);

   return TRUE;
}

/*********** ---------------------------------------------------------------
 ***********  KPDS_INCREMENT_SIZE
 *********** ---------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: (static) _increment_size_get
|       Purpose: get the increment_size attribute
|    Written By: Steve Kubica
|          Date: Februrary 18, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_increment_size_get(kobject object, ktoken segment, ktoken attribute, 
		    kaddr client_data, kva_list *list)
{
   int *isize = NULL, dimension = 0, *val, *size, *rorder, *order, i;
   int 	vorder[5] = {KWIDTH, KHEIGHT, KDEPTH, KTIME, KELEMENTS};
   int 	lorder[4] = {KWIDTH, KHEIGHT, KDEPTH, KDIMENSION};
   int 	morder[5] = {KMAP_WIDTH, KMAP_HEIGHT, KMAP_DEPTH, KMAP_TIME, 
		     KMAP_ELEMENTS};
   int 	torder[1] = {0};

   ktoken seg = kstring_to_token((char *) client_data);

   if (kpds_get_access_info(object, seg, &dimension, NULL, NULL, &isize, NULL))
   {
      kpds_get_size(object, seg, &size);
      kpds_get_order(object, seg, &order);

      if (dimension == 0 || isize == NULL || size == NULL || order == NULL)
	 return FALSE;

      /*-- must return numbers in expected polymorphic order --*/
      if (kstrcmp((char *)client_data, KPDS_SEGMENT_VALUE) == 0)
	 rorder = vorder;
      else if (kstrcmp((char *)client_data, KPDS_SEGMENT_MASK) == 0)
	 rorder = vorder;
      else if (kstrcmp((char *)client_data, KPDS_SEGMENT_LOCATION) == 0)
	 rorder = lorder;
      else if (kstrcmp((char *)client_data, KPDS_SEGMENT_MAP) == 0)
	 rorder = morder;
      else if (kstrcmp((char *)client_data, KPDS_SEGMENT_TIME) == 0)
	 rorder = torder;
      else
	 rorder = order; /*-- just use the order that's there --*/
	 
      for (i = 0; i < dimension; i++)
	 if ((val = kva_arg(*list, int *)) != NULL) 
	    *val = (isize[i] > 0) ? isize[i] : kdas_get_order(size, order, 
							      dimension, 
							      rorder[i], 1);
   }
   else
      return FALSE;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _increment_size_set
|       Purpose: set the increment_size 
|    Written By: Steve Kubica
|          Date: Februrary 18, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_increment_size_set(kobject object, ktoken segment, ktoken attribute, 
		    kaddr client_data, kva_list *list)
{
   int *isize = NULL, dimension = 0, *rorder, *order, i;
   int  new_isize[KDAS_MAX_DIM];
   
   int 	vorder[5] = {KWIDTH, KHEIGHT, KDEPTH, KTIME, KELEMENTS};
   int 	lorder[4] = {KWIDTH, KHEIGHT, KDEPTH, KDIMENSION};
   int 	morder[5] = {KMAP_WIDTH, KMAP_HEIGHT, KMAP_DEPTH, KMAP_TIME, 
		     KMAP_ELEMENTS};
   int 	torder[1] = {KTIME};

   ktoken seg = kstring_to_token((char *)client_data);

   if (kpds_get_access_info(object, seg, &dimension, NULL, NULL, &isize, NULL))
   {
      kpds_get_order(object, seg, &order);

      if (dimension == 0 || isize == NULL || order == NULL)
	 return FALSE;

      /*-- must set numbers in expected polymorphic order --*/
      if (kstrcmp((char *)client_data, KPDS_SEGMENT_VALUE) == 0)
	 rorder = vorder;
      else if (kstrcmp((char *)client_data, KPDS_SEGMENT_MASK) == 0)
	 rorder = vorder;
      else if (kstrcmp((char *)client_data, KPDS_SEGMENT_LOCATION) == 0)
	 rorder = lorder;
      else if (kstrcmp((char *)client_data, KPDS_SEGMENT_TIME) == 0)
	 rorder = torder;
      else if (kstrcmp((char *)client_data, KPDS_SEGMENT_MAP) == 0)
	 rorder = morder;
      else
	 rorder = order; /*-- just use the order that's there --*/
	 
      for (i = 0; i < dimension; i++)
	 new_isize[i] = kva_arg(*list, int);
      
      kdas_array_reorder(new_isize, dimension, rorder, order);

      for (i = 0; i < dimension; i++)
	 isize[i] = new_isize[i];
   }
   else
      return FALSE;

return TRUE;
}
   
/*-----------------------------------------------------------
|  Routine Name: (static) _increment_size_match
|       Purpose: match the increment_size attribute
|    Written By: Steve Kubica
|          Date: Februrary 18, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static int
_increment_size_match(kobject object1, kobject object2, ktoken segment, 
		      ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   int *isize1 = NULL, *isize2 = NULL, dimension = 0, i;

   if (!kpds_get_access_info(object1, (char *) client_data1, &dimension, 
			     NULL, NULL, &isize1, NULL))
      return FALSE;

   if (!kpds_get_access_info(object2, (char *) client_data2, NULL, 
			     NULL, NULL, &isize2, NULL))
      return FALSE;
   
   if (dimension == 0 || isize1 == NULL || isize2 == NULL)
      return FALSE;

   for (i = 0; i < dimension; i++)
      if (isize1[i] != isize2[i])
	 return FALSE;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _increment_size_copy
|       Purpose: copy the increment_size attribute
|    Written By: Steve Kubica
|          Date: Februrary 18, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_increment_size_copy(kobject object1, kobject object2, ktoken segment, 
		     ktoken attribute, kaddr client_data1, kaddr client_data2)
{
   int *isize1 = NULL, *isize2 = NULL;
   int  dimension = 0, i;

   if (!kpds_get_access_info(object1, (char *) client_data1, &dimension, 
			     NULL, NULL, &isize1, NULL))
      return FALSE;

   if (!kpds_get_access_info(object2, (char *) client_data2, NULL, 
			     NULL, NULL, &isize2, NULL))
      return FALSE;
   
   if (dimension == 0 || isize1 == NULL || isize2 == NULL)
      return FALSE;

   /*-- modifying this will set the internal stored array --*/
   for (i = 0; i < dimension; i++)
      isize2[i] = isize1[i];

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _increment_size_query
|       Purpose: query the increment_size attribute
|    Written By: Steve Kubica
|          Date: Februrary 18, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_increment_size_query(kobject object, ktoken segment, ktoken attribute, 
		      kaddr client_data, int *numargs, int *argsize, 
		      int *datatype, int *permanent)
{
   int dimension = 0;
   
   if (!kpds_get_access_info(object, (char *) client_data, &dimension, 
			     NULL, NULL, NULL, NULL))
      return FALSE;

   if (numargs)   *numargs   = dimension;
   if (argsize)   *argsize   = 1;
   if (datatype)  *datatype  = KINT;
   if (permanent) *permanent = FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _increment_size_print
|       Purpose: print the increment_size attribute
|    Written By: Steve Kubica
|          Date: Februrary 18, 1997
-----------------------------------------------------------*/
/* ARGSUSED */
static int
_increment_size_print(kobject object, ktoken segment, ktoken attribute, 
		      kaddr client_data, kfile *outfile)
{
   int *isize = NULL, dimension = 0, *size, *rorder, *order, i;
   int 	vorder[5] = {KWIDTH, KHEIGHT, KDEPTH, KTIME, KELEMENTS};
   int 	lorder[4] = {KWIDTH, KHEIGHT, KDEPTH, KDIMENSION};
   int 	morder[5] = {KMAP_WIDTH, KMAP_HEIGHT, KMAP_DEPTH, KMAP_TIME, 
		     KMAP_ELEMENTS};
   int 	torder[1] = {KTIME};

   ktoken seg = kstring_to_token((char *) client_data);

   if (!kpds_get_access_info(object, seg, &dimension, NULL, NULL, &isize,NULL))
      return FALSE;
   
   kpds_get_size(object, seg, &size);
   kpds_get_order(object, seg, &order);

   if (dimension == 0 || isize == NULL || size == NULL || order == NULL)
      return FALSE;

   /*-- must return numbers in expected polymorphic order --*/
   if (kstrcmp((char *)client_data, KPDS_SEGMENT_VALUE) == 0)
      rorder = vorder;
   else if (kstrcmp((char *)client_data, KPDS_SEGMENT_MASK) == 0)
      rorder = vorder;
   else if (kstrcmp((char *)client_data, KPDS_SEGMENT_LOCATION) == 0)
      rorder = lorder;
   else if (kstrcmp((char *)client_data, KPDS_SEGMENT_TIME) == 0)
      rorder = torder;
   else if (kstrcmp((char *)client_data, KPDS_SEGMENT_MAP) == 0)
      rorder = morder;
   else
      rorder = order; /*-- just use the order that's there --*/
   
   if (outfile)
      for (i = 0; i < dimension; i++)
	 kfprintf(outfile, " %d ", 
		  (isize[i] > 0) ? isize[i] : kdas_get_order(size, order, 
							     dimension, 
							     rorder[i], 1));

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kpds_init_position_attributes
|
|       Purpose: This initializes all the position, offset, 
|		 and other related attributes.
|
|         Input: none
|
|        Output: none
|
|       Returns: none
|
|    Written By: Steve Kubica
|          Date: Apr 28, 1994 15:15
| Modifications:
|
------------------------------------------------------------*/
void
kpds_init_position_attributes(void)
{
   static int zero_array[20] = {0, 0, 0, 0, 0,
				0, 0, 0, 0, 0, 
				0, 0, 0, 0, 0, 
				0, 0, 0, 0, 0};
   /*-- position --*/
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_VALUE_POSITION, pds_value_string,
			       _position_get,   _position_set, 
			       _position_match, _position_copy, 
			       _position_query, _position_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MASK_POSITION, pds_mask_string,
			       _position_get,   _position_set, 
			       _position_match, _position_copy, 
			       _position_query, _position_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MAP_POSITION, pds_map_string,
			       _position_get,   _position_set, 
			       _position_match, _position_copy, 
			       _position_query, _position_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_LOCATION_POSITION, pds_location_string,
			       _position_get,   _position_set, 
			       _position_match, _position_copy, 
			       _position_query, _position_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_TIME_POSITION, pds_time_string,
			       _position_get,   _position_set, 
			       _position_match, _position_copy, 
			       _position_query, _position_print);

   /*-- offsets --*/
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_VALUE_OFFSET, pds_value_string,
			       _offset_get,   _offset_set, 
			       _offset_match, _offset_copy, 
			       _offset_query, _offset_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MASK_OFFSET, pds_mask_string,
			       _offset_get,   _offset_set, 
			       _offset_match, _offset_copy, 
			       _offset_query, _offset_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MAP_OFFSET, pds_map_string,
			       _offset_get,   _offset_set, 
			       _offset_match, _offset_copy, 
			       _offset_query, _offset_print);
   
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_LOCATION_OFFSET, pds_location_string,
			       _offset_get,   _offset_set, 
			       _offset_match, _offset_copy, 
			       _offset_query, _offset_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_TIME_OFFSET, pds_time_string,
			       _offset_get,   _offset_set, 
			       _offset_match, _offset_copy, 
			       _offset_query, _offset_print);
   

   /*-- increment_size attributes --*/
   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_VALUE_INCREMENT_SIZE, pds_value_string,
			       _increment_size_get,   _increment_size_set, 
			       _increment_size_match, _increment_size_copy, 
			       _increment_size_query, _increment_size_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MASK_INCREMENT_SIZE, pds_mask_string,
			       _increment_size_get,   _increment_size_set, 
			       _increment_size_match, _increment_size_copy, 
			       _increment_size_query, _increment_size_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_MAP_INCREMENT_SIZE, pds_map_string,
			       _increment_size_get,   _increment_size_set, 
			       _increment_size_match, _increment_size_copy, 
			       _increment_size_query, _increment_size_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_LOCATION_INCREMENT_SIZE, 
			       pds_location_string,
			       _increment_size_get,   _increment_size_set, 
			       _increment_size_match, _increment_size_copy, 
			       _increment_size_query, _increment_size_print);

   kdms_define_quasi_attribute(KDMS_OBJECT, 
			       KPDS_TIME_INCREMENT_SIZE, pds_time_string,
			       _increment_size_get,   _increment_size_set, 
			       _increment_size_match, _increment_size_copy, 
			       _increment_size_query, _increment_size_print);

   /*-- internal position, offset, and increment size information --*/
   kdms_define_attribute(KPDS_SEGMENT_VALUE, KPDS_ACCESS_INFO, 1, 18, KINT,
			 FALSE, FALSE, zero_array);

   kdms_define_attribute(KPDS_SEGMENT_MASK, KPDS_ACCESS_INFO, 1, 18, KINT,
			 FALSE, FALSE, zero_array);

   kdms_define_attribute(KPDS_SEGMENT_MAP, KPDS_ACCESS_INFO, 1, 18, KINT,
			 FALSE, FALSE, zero_array);

   kdms_define_attribute(KPDS_SEGMENT_TIME, KPDS_ACCESS_INFO, 1, 6, KINT,
			 FALSE, FALSE, zero_array);

   kdms_define_attribute(KPDS_SEGMENT_LOCATION, KPDS_ACCESS_INFO, 1, 15, KINT,
			 FALSE, FALSE, zero_array);
   kdms_define_attribute(KPDS_SEGMENT_WIDTH, KPDS_ACCESS_INFO, 1, 15, KINT,
			 FALSE, FALSE, zero_array);
   kdms_define_attribute(KPDS_SEGMENT_UNIFORM, KPDS_ACCESS_INFO, 1, 15, KINT,
			 FALSE, FALSE, zero_array);

   return;
}

