 /*
  * Khoros: $Id: primitive.c,v 1.1.2.1 1997/05/24 18:35:23 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: primitive.c,v 1.1.2.1 1997/05/24 18:35:23 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		  Polymophic Model Data Handlers
   >>>>		
   >>>>  Private:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static int   lookup_primitive  	 PROTO((char *));
static int   get_size  	         PROTO((kobject, kpds_primitive *, int **));
static int   get_location_size 	 PROTO((kobject, kpds_primitive *, int **));
static void  find_corners      	 PROTO((kobject, kpds_primitive *, int *, 
					int *, int *));
static void  find_region_corners PROTO((kobject, kpds_primitive *, int *, 
					int *, int *));
static void  find_location_region_corners PROTO((kobject, kpds_primitive *, 
						 int *, int *, int *));
static int   increment_position  PROTO((int *, int *, int *, int));
static void  advance             PROTO((int *, int *, int *, int));
static void  advance_and_update  PROTO((kobject, kpds_primitive *, int *, int, 
					int *, int *));
static kaddr get_primitive PROTO((kobject, kpds_primitive *, kaddr));
static int   put_primitive PROTO((kobject, kpds_primitive *, kaddr));
static kaddr get_histogram PROTO((kobject, kpds_primitive *, kaddr));

/****    =============================================================
 ****    Polymorphic Data Serivces Primitive Table
 ****    =============================================================
 ****/

kpds_primitive kpds_primitives[] = 
{
   
   {KPDS_VALUE_POINT,  0,  KPDS_SEGMENT_VALUE, 0, KPDS_VALUE_SIZE, 
    {0, 0, 0, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive },
   {KPDS_VALUE_VECTOR, 0,  KPDS_SEGMENT_VALUE, 0, KPDS_VALUE_SIZE, 
    {0, 0, 0, 0, 1},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_VALUE_LINE, 0,    KPDS_SEGMENT_VALUE, 0, KPDS_VALUE_SIZE, 
    {1, 0, 0, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_VALUE_PLANE, 0,   KPDS_SEGMENT_VALUE, 0, KPDS_VALUE_SIZE, 
    {1, 1, 0, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_VALUE_VOLUME, 0,  KPDS_SEGMENT_VALUE, 0, KPDS_VALUE_SIZE, 
    {1, 1, 1, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_VALUE_ALL, 0,     KPDS_SEGMENT_VALUE, 0, KPDS_VALUE_SIZE, 
    {1, 1, 1, 1, 1},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_VALUE_REGION, 0,  KPDS_SEGMENT_VALUE, 0, KPDS_VALUE_SIZE, 
    {0, 0, 0, 0, 0},       KPDS_VALUE_REGION_SIZE, get_size, 
                           find_region_corners, advance_and_update, 
                           get_primitive, put_primitive},

   {KPDS_MASK_POINT, 0,    KPDS_SEGMENT_MASK, 0, KPDS_MASK_SIZE, 
    {0, 0, 0, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_MASK_VECTOR, 0,   KPDS_SEGMENT_MASK, 0, KPDS_MASK_SIZE, 
    {0, 0, 0, 0, 1},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_MASK_LINE, 0,     KPDS_SEGMENT_MASK, 0, KPDS_MASK_SIZE, 
    {1, 0, 0, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_MASK_PLANE, 0,    KPDS_SEGMENT_MASK, 0, KPDS_MASK_SIZE, 
    {1, 1, 0, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_MASK_VOLUME, 0,   KPDS_SEGMENT_MASK, 0, KPDS_MASK_SIZE, 
    {1, 1, 1, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_MASK_ALL, 0,      KPDS_SEGMENT_MASK, 0, KPDS_MASK_SIZE, 
    {1, 1, 1, 1, 1},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_MASK_REGION, 0,   KPDS_SEGMENT_MASK, 0, KPDS_MASK_SIZE, 
    {0, 0, 0, 0, 0},       KPDS_MASK_REGION_SIZE, get_size, 
                           find_region_corners, advance_and_update, 
                           get_primitive, put_primitive},

   {KPDS_LOCATION_POINT,0, KPDS_SEGMENT_LOCATION, 0, KPDS_LOCATION_SIZE, 
    {0, 0, 0, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_LOCATION_VECTOR,0,KPDS_SEGMENT_LOCATION, 0, KPDS_LOCATION_SIZE, 
    {0, 0, 0, 1, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_LOCATION_LINE, 0, KPDS_SEGMENT_LOCATION, 0, KPDS_LOCATION_SIZE, 
    {1, 0, 0, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_LOCATION_PLANE,0, KPDS_SEGMENT_LOCATION, 0, KPDS_LOCATION_SIZE, 
    {1, 1, 0, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_LOCATION_VOLUME,0,KPDS_SEGMENT_LOCATION, 0, KPDS_LOCATION_SIZE, 
    {1, 1, 1, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_LOCATION_ALL, 0,  KPDS_SEGMENT_LOCATION, 0, KPDS_LOCATION_SIZE, 
    {1, 1, 1, 1, 1},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_LOCATION_REGION,0,KPDS_SEGMENT_LOCATION, 0, KPDS_LOCATION_SIZE, 
    {0, 0, 0, 0, 0},       KPDS_LOCATION_REGION_SIZE, get_size, 
                           find_region_corners, advance_and_update, 
                           get_primitive, put_primitive},

   {KPDS_MAP_POINT, 0,     KPDS_SEGMENT_MAP, 0, KPDS_MAP_SIZE, 
    {0, 0, 0, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_MAP_VECTOR, 0,    KPDS_SEGMENT_MAP, 0, KPDS_MAP_SIZE, 
    {1, 0, 0, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_MAP_LINE, 0,      KPDS_SEGMENT_MAP, 0, KPDS_MAP_SIZE, 
    {1, 0, 0, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_MAP_PLANE, 0,     KPDS_SEGMENT_MAP, 0, KPDS_MAP_SIZE, 
    {1, 1, 0, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_MAP_VOLUME, 0,    KPDS_SEGMENT_MAP, 0, KPDS_MAP_SIZE, 
    {1, 1, 1, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_MAP_ALL, 0,       KPDS_SEGMENT_MAP, 0, KPDS_MAP_SIZE, 
    {1, 1, 1, 1, 1},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_MAP_REGION, 0,    KPDS_SEGMENT_MAP, 0, KPDS_MAP_SIZE, 
    {0, 0, 0, 0, 0},       KPDS_MAP_REGION_SIZE, get_size, 
                           find_region_corners, advance_and_update, 
                           get_primitive, put_primitive},

   {KPDS_TIME_POINT, 0,    KPDS_SEGMENT_TIME, 0, KPDS_TIME_SIZE, 
    {0, 0, 0, 0, 0},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_TIME_ALL, 0,      KPDS_SEGMENT_TIME, 0, KPDS_TIME_SIZE, 
    {1, 1, 1, 1, 1},       NULL, get_size, find_corners, 
                           advance_and_update, get_primitive, put_primitive},
   {KPDS_TIME_REGION, 0,   KPDS_SEGMENT_TIME, 0, KPDS_TIME_SIZE, 
    {0, 0, 0, 0, 0},       KPDS_TIME_REGION_SIZE, get_size, 
                           find_region_corners, advance_and_update, 
                           get_primitive, put_primitive},

   {KPDS_LOCATION_WIDTH_POINT, 0, KPDS_SEGMENT_WIDTH, 0, NULL, 
    {0, 0, 0, 0, 0},       NULL, get_location_size, 
                           find_corners, advance_and_update, 
                           get_primitive, put_primitive},
   {KPDS_LOCATION_WIDTH_ALL, 0, KPDS_SEGMENT_WIDTH, 0, NULL, 
    {1, 1, 1, 1, 1},       NULL, get_location_size, 
                           find_corners, advance_and_update, 
                           get_primitive, put_primitive},
   {KPDS_LOCATION_WIDTH_REGION, 0,   KPDS_SEGMENT_WIDTH, 0, NULL, 
    {0, 0, 0, 0, 0},       KPDS_LOCATION_REGION_SIZE, get_location_size, 
                           find_location_region_corners, advance_and_update, 
                           get_primitive, put_primitive},

   {KPDS_LOCATION_HEIGHT_POINT, 0, KPDS_SEGMENT_HEIGHT, 0, NULL, 
    {0, 0, 0, 0, 0},       NULL, get_location_size, 
                           find_corners, advance_and_update, 
                           get_primitive, put_primitive},
   {KPDS_LOCATION_HEIGHT_ALL, 0, KPDS_SEGMENT_HEIGHT, 0, NULL, 
    {1, 1, 1, 1, 1},       NULL, get_location_size, 
                           find_corners, advance_and_update, 
                           get_primitive, put_primitive},
   {KPDS_LOCATION_HEIGHT_REGION, 0,   KPDS_SEGMENT_HEIGHT, 0, NULL, 
    {0, 0, 0, 0, 0},       KPDS_LOCATION_REGION_SIZE, get_location_size, 
                           find_location_region_corners, advance_and_update, 
                           get_primitive, put_primitive},

   {KPDS_LOCATION_DEPTH_POINT, 0, KPDS_SEGMENT_DEPTH, 0, NULL, 
    {0, 0, 0, 0, 0},       NULL, get_location_size, 
                           find_corners, advance_and_update, 
                           get_primitive, put_primitive},
   {KPDS_LOCATION_DEPTH_ALL, 0, KPDS_SEGMENT_DEPTH, 0, NULL, 
    {1, 1, 1, 1, 1},       NULL, get_location_size, 
                           find_corners, advance_and_update, 
                           get_primitive, put_primitive},
   {KPDS_LOCATION_DEPTH_REGION, 0,   KPDS_SEGMENT_DEPTH, 0, NULL, 
    {0, 0, 0, 0, 0},       KPDS_LOCATION_REGION_SIZE, get_location_size, 
                           find_location_region_corners, advance_and_update, 
                           get_primitive, put_primitive},

   {KPDS_VALUE_HISTOGRAM, 0, KPDS_SEGMENT_VALUE, 0, NULL,
    {0, 0, 0, 0, 0},      NULL, NULL, NULL, NULL, get_histogram, NULL}

};

int kpds_num_primitives = sizeof(kpds_primitives) / sizeof(kpds_primitives[0]);

/****    =============================================================
 ****    Public Data Access Routines 
 ****    =============================================================
 ****/

/************************************************************
*
*  Routine Name: kpds_get_data - retrieve data from
*		 a segment in a data object.
*
*       Purpose: 
*		 This function is used to retrieve a unit of 
*		 data, which is referred to as a "primitive", 
*		 from a data object.
*
*		 The first argument is the data object that
*		 will be accessed in order to retrieve the
*		 data.  
*
*		 The second argument is the data primitive that is 
*		 desired.  Data services understands a number of
*		 primitives that allow access to different parts of
*		 a data set.  Except for the vector primitives
*		 (KPDS_VALUE_VECTOR, KPDS_MAP_VECTOR, KPDS_TIME_VECTOR,
*		 KPDS_LOCATION_VECTOR), the 
*		 interpretation of all of these primitives depends on 
*		 the attribute index order primitives (KPDS_VALUE_INDEX_ORDER,
*		 KPDS_MAP_INDEX_ORDER, etc).
*		 The index order attributes determine how data is
*		 retrieved and stored.  Successive calls to kpds_get_data
*		 cause an automatic increment of the position of each
*		 part of the data set.  How the position is incremented depends
*		 on the primitive that is being retrieved and the index
*		 order.  For example, if a KPDS_VALUE_LINE is being retrieved,
*		 and the index order is KWIDTH, KHEIGHT, KDEPTH, ...,
*		 then the line will be defined in the width direction
*		 and successive lines can be indexed by incrementing
*		 the height dimension until it wraps around, then
*		 incrementing the depth dimension, and so on.  Below
*		 is a list of the types of primitives that are available:
*		 
*		.RS
*	 	.IP "point" 8
*			  specifies that a single value will be returned.
*	 		  Successive calls to kpds_get_data will result
*	 		  in adjacent points being returned, as 
*	 		  described above.  An example of a point primitive
*	 		  is the KPDS_VALUE_POINT primitive.
*	 		  
*	 	.IP "line" 8
*			  specifies that a one-dimensional unit of data 
*	 		  will be returned.  The direction in which
*	 		  a line is defined is given by the first (or
*	 		  lowest order) value of the index order.  In
*	 		  the above example, KWIDTH was the first value
*	 		  in the index order attribute.  An example of a 
*	 		  line primitive is the KPDS_VALUE_LINE primitive.
*	 	
*	 	.IP "plane" 8
*			  specifies that a two-dimensional unit of data
*	 		  will be returned.  The plane is defined along
*	 		  the two lowest order dimensions in the index
*	 		  order for the segment specified.  An example of a 
*	 		  plane primitive is the KPDS_VALUE_PLANE primitive.
*	 
*	 	.IP "volume" 8
*			  specifies that a three-dimensional unit of data
*	 		  will be returned.  The volume is defined along
*	 		  the three lowest order dimensions in the
*	 		  index order for the segment specified.   An example 
*	 		  of a volume primitive is the KPDS_VALUE_VOLUME 
*	 		  primitive.
*	 
*	 	.IP "region" 8
*			  specifies that a n-dimensional unit of data
*	 		  will be returned.  The n varies from segment
*	 		  to segment and is based on the dimensionality
*	 		  of the segment being accessed.  For example,
*	 		  Data Services considers the Value segment
*	 		  to be a five-dimensional segment, so n is five
*	 		  for the value segment.  A region front, upper
*	 		  corner is specified by the current position
*	 		  (for the Value segment, this is a five-tuple).
*	 		  The size of the region is given by the 
*	 		  attribute KPDS_REGION_SIZE, which must be
*	 		  set prior to using this primitive.  An example of a 
*	 		  region primitive is the KPDS_VALUE_REGION primitive.
*	 		  
*	 	.IP "all" 8
*			  specifies that all data for the specified segment
*	 		  should be retrieved and returned.  An example of an
*	 		  all primitive is the KPDS_VALUE_ALL primitive.
*	 
*	 	.IP "vector" 8
*			 specifies that a one-dimensional unit of data
*	 		  will be returned.  The dimension in which
*	 		  this unit of data is defined is fixed for each
*	 		  segment.  Thus, regardless of the index order,
*	 		  the same dimension will be used to obtain
*	 		  the vector of data.  The only thing that is
*	 		  impacted by index order when retrieving vectors
*	 		  is the actual order in which the vectors 
*	 		  in a data set are retrieved due to the
*	 		  auto-incrementing of position.  An example of a 
*	 		  vector primitive is the KPDS_VALUE_VECTOR primitive.
*	 		  For the five predefined part of a data set, each has
*	 		  a different dimension that specifies a vector.
*	 		  Below is a list of the data components and their
*	 		  associated vector definition:
*		
*	 	.TS
*		center;
*		c c.
*				\fBSegment\fP	\fBVector Definition\fP
*				_
*	 			VALUE	KELEMENTS
*	 			MASK	KELEMENTS
*	 			MAP	KWIDTH
*	 			LOCATION	KDIMENSION
*	 			TIME	KTIME
*		.TE
*		.RE
*
*                The third argument, "data", serves as both
*                an input and an output argument.  As input,
*                it dictates whether kpds_get_data must
*                allocate space sufficient for the operation.
*                If the argument is NULL, then memory will
*                be allocated to store the data primitive
*                requested.  A pointer to that memory will
*                be returned.  If this argument is not NULL,
*                then kpds_get_data assumes that the "data"
*                argument is a pointer to a sufficient
*                amount of memory with the correct
*                dimensionality for the primitive (no memory
*                allocation occurs).  In this case, if this
*                routine returns successfully, then the
*                return value is the pointer "data".
*
*         Input: object    - the  object that
*			     will serve as a source for the
*			     data requested.
*		 primitive - a description of the unit of
*                            data desired.
*                data      - a pointer to the region of
*                            memory that will serve as a
*                            destination for the data. If
*                            this value is NULL, then
*                            sufficient space for this
*                            operation will be allocated
*                            for this operation.  The data
*                            type kaddr is used because
*                            it indicates a generic data
*                            pointer.
*
*        Output: If "data" is not initially NULL, then the memory
*		 that it points to will be filled with the data
*		 that is being requested.
*       Returns: If "data" is not initially NULL, then the data
*                space pointed to by "data" will be returned on
*                success.  If the "data" argument is NULL, then
*                a new pointer to the requested data will be
*                returned.  Unsuccessful calls to this routine
*                are indicated by a return value of NULL.
*
*  Restrictions: This routine assumes that if the argument
*                "data" is not NULL, then it contains the
*                appropriate amount of memory with the
*                appropriate dimensionality for the
*                requested primitive.
*    Written By: Jeremy Worley & John Salas
*          Date: October 18, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
kaddr
kpds_get_data(
   kobject object,
   char   *primitive,
   kaddr   data)
{
   int prim;

   kpds_init();

   if ((prim = lookup_primitive(primitive)) < 0)
      return FALSE;	/* errno already set */

   return kpds_primitives[prim].get(object, &kpds_primitives[prim], data);
}


/************************************************************
*
*  Routine Name: kpds_put_data - store a unit of data
*		 to a segment in a data object.
*
*       Purpose: 
*		 This function is used to store data in a data 
*		 object.
*
*		 The first argument is the data object that
*		 will be accessed in order to put the data.  
*
*		 The second argument is the data primitive that is 
*		 to be written.  Data services understands a number of
*		 primitives that allow access to different parts of
*		 a data set.  Except for the vector primitives
*		 (KPDS_VALUE_VECTOR, KPDS_MAP_VECTOR, KPDS_TIME_VECTOR,
*		 KPDS_LOCATION_VECTOR), the 
*		 interpretation of all of these primitives depends on 
*		 the attribute index order primitives (KPDS_VALUE_INDEX_ORDER,
*		 KPDS_MAP_INDEX_ORDER, etc).
*		 The index order attributes determine how data is
*		 retrieved and stored.  Successive calls to kpds_get_data
*		 cause an automatic increment of the position of each
*		 part of the data set.  How the position is incremented depends
*		 on the primitive that is being retrieved and the index
*		 order.  For example, if a KPDS_VALUE_LINE is being stored,
*		 and the index order is KWIDTH, KHEIGHT, KDEPTH, ...,
*		 then the line will be defined in the width direction
*		 and successive lines can be indexed by incrementing
*		 the height dimension until it wraps around, then
*		 incrementing the depth dimension, and so on.  Below
*		 is a list of the types of primitives that are available:
*		
*		.RS
*	 	.IP "point" 8
*			  specifies that a single value will be stored.
*	 		  Successive calls to kpds_put_data will result
*	 		  in adjacent points being stored, as 
*	 		  described above.  An example of a point primitive
*	 		  is the KPDS_VALUE_POINT primitive.
*	 		  
*	 	.IP "line" 8
*			  specifies that a one-dimensional unit of data 
*	 		  will be returned.  The direction in which
*	 		  a line is defined is given by the first (or
*	 		  lowest order) value of the index order.  In
*	 		  the above example, KWIDTH was the first value
*	 		  in the index order attribute.  An example of a 
*	 		  line primitive is the KPDS_VALUE_LINE primitive.
*	 	
*	 	.IP "plane" 8
*			  specifies that a two-dimensional unit of data
*	 		  will be returned.  The plane is defined along
*	 		  the two lowest order dimensions in the index
*	 		  order for the segment specified.  An example of a 
*	 		  plane primitive is the KPDS_VALUE_PLANE primitive.
*	 
*	 	.IP "volume" 8
*			  specifies that a three-dimensional unit of data
*	 		  will be returned.  The volume is defined along
*	 		  the three lowest order dimensions in the
*	 		  index order for the segment specified.   An example 
*	 		  of a volume primitive is the KPDS_VALUE_VOLUME 
*	 		  primitive.
*	 
*	 	.IP "region" 8
*			  specifies that a n-dimensional unit of data
*	 		  will be returned.  The n varies from segment
*	 		  to segment and is based on the dimensionality
*	 		  of the segment being accessed.  For example,
*	 		  Data Services considers the Value segment
*	 		  to be a five-dimensional segment, so n is five
*	 		  for the value segment.  A region front, upper
*	 		  corner is specified by the current position
*	 		  (for the Value segment, this is a five-tuple).
*	 		  The size of the region is given by the 
*	 		  attribute KPDS_REGION_SIZE, which must be
*	 		  set prior to using this primitive.  An example of a 
*	 		  region primitive is the KPDS_VALUE_REGION primitive.
*	 		  
*	 	.IP "all" 8
*			  specifies that all data for the specified segment
*	 		  should be retrieved and returned.  An example of an
*	 		  all primitive is the KPDS_VALUE_ALL primitive.
*	 
*	 	.IP "vector" 8
*			  specifies that a one-dimensional unit of data
*	 		  will be returned.  The dimension in which
*	 		  this unit of data is defined is fixed for each
*	 		  segment.  Thus, regardless of the index order,
*	 		  the same dimension will be used to obtain
*	 		  the vector of data.  The only thing that is
*	 		  impacted by index order when retrieving vectors
*	 		  is the actual order in which the vectors 
*	 		  in a data set are retrieved due to the
*	 		  auto-incrementing of position.  An example of a 
*	 		  vector primitive is the KPDS_VALUE_VECTOR primitive.
*	 		  For the five predefined part of a data set, each has
*	 		  a different dimension that specifies a vector.
*	 		  Below is a list of the data components and their
*	 		  associated vector definition:
*	 
*	 	.TS
*		center;
*		c c.
*				\fBSegment\fP	\fBVector Definition\fP
*				_
*	 			VALUE	KELEMENTS
*	 			MASK	KELEMENTS
*	 			MAP	KWIDTH
*	 			LOCATION	KDIMENSION
*	 			TIME	KTIME
*		.TE
*		.RE
*
*		The fourth argument is the data to be written.
*		This must be a non-NULL pointer to valid data of
*		the appropriate size (as defined, for example, 
*		by the KPDS_VALUE_SIZE attribute)
*		and data type (as defined, for example,  by the 
*		KPDS_VALUE_DATA_TYPE attribute).
*
*         Input: object    - the data object that
*			     will serve as a destination for
*			     the data.
*		 primitive - a description of the unit of
*                            data in the argument "data".
*                data      - a pointer to the region of
*                            memory that will serve as a
*                            source for the data.
*        Output:
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions:
*    Written By: Jeremy Worley & John Salas
*          Date: November 23, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kpds_put_data(
   kobject object,
   char   *primitive,
   kaddr   data)
{
   int prim;

   kpds_init();

   if ((prim = lookup_primitive(primitive)) < 0)
      return FALSE;	/* errno already set */

   if (data == NULL)
      return FALSE;
     
   return kpds_primitives[prim].put(object, &kpds_primitives[prim], data);
}

/****    =============================================================
 ****    Primitive Internal Access Routines
 ****    =============================================================
 ****/

void
kpds_init_primitive_table(void)
{
   int i;
   
   /*-- tokenize the primitive table --*/
   for (i = 0; i < kpds_num_primitives; i++)
   {
      kpds_primitives[i].name_token = 
	 kstring_to_token(kpds_primitives[i].name);
      kpds_primitives[i].segment_token =
	 kstring_to_token(kpds_primitives[i].segment);
   }
   return;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) lookup_primitive
|
|       Purpose: This routine returns the index into the 
|                primitive table for a given primitive
|                name.  If the primitive is not found, 
|                then an errno is set.
|
|         Input: prim - the name of the primitive to look up
|
|        Output: none
|
|       Returns: the index into the primitive table on success, 
|		 -1 otherwise.
|
|    Written By: Jeremy Worley & John Salas
|          Date: September 30, 1993
| Modifications:
|
------------------------------------------------------------*/
static int
lookup_primitive(char *primitive)
{
   int i;
   ktoken name_token = kstring_to_token(primitive);

   for (i = 0; i < kpds_num_primitives; i++)
      if (kpds_primitives[i].name_token == name_token)
	 return i;

   kset_errno(KPDS_EINVALID_PRIMITIVE);

   return -1;
}


/*
 *  find the size of a segment from a primitive's point of view
 */ 
static int 
get_size(
   kobject         object, 
   kpds_primitive *prim, 
   int           **size)
{
   static int isize[KDAS_MAX_DIM];
   
   /*-- don't bypass defined attribute! --*/ 
   if (!kpds_get_attribute(object, prim->size_attribute, &isize[0], 
			   &isize[1], &isize[2], &isize[3], &isize[4]))
      return FALSE;

   if (size) *size = isize;

   return TRUE;
}

/*
 *  find size of rectilinear location segment 
 */ 
static int 
get_location_size(
   kobject         object, 
   kpds_primitive *prim, 
   int           **size)
{
   int w, h, d, e;
   static int isize[1];
   
   /*-- don't bypass defined attribute! --*/ 
   if (!kpds_get_attribute(object, KPDS_LOCATION_SIZE, &w, &h, &d, &e))
      return FALSE;

   if (kstrcmp(prim->segment, KPDS_SEGMENT_WIDTH) == 0)
      isize[0] = w;
   else if (kstrcmp(prim->segment, KPDS_SEGMENT_HEIGHT) == 0)
      isize[0] = h;
   else if (kstrcmp(prim->segment, KPDS_SEGMENT_DEPTH) == 0)
      isize[0] = d;

   if (size) *size = isize;

   return TRUE;
}

/*
 *  find the corners for a given primitive 
 */ 
static void
find_corners(
   kobject         object,
   kpds_primitive *prim,
   int            *size,
   int            *begin,
   int            *end)
{
   int i;
   for (i = 0; i < KDAS_MAX_DIM; i++)
      begin[i] = 0, end[i] = (prim->usage[i]==1) ? (size[i]-1) : 0;
}

/*
 *  find the corners for a region
 */ 
static void
find_region_corners(
   kobject         object,
   kpds_primitive *prim,
   int            *size,
   int            *begin,
   int            *end)
{
   int i;
   
   kmemset((kaddr) begin, 0, (size_t) KDAS_MAX_DIM * sizeof(int));
   kpds_get_attribute(object, prim->region_attribute, 
		      &end[0], &end[1], &end[2], &end[3], &end[4]);

   /*-- subtract one from size to get end point --*/
   for (i = 0; i < KDAS_MAX_DIM; i++) end[i]--;
   
   return;
}

/*
 *  find the corners for a region of rectilinear location
 */ 
static void
find_location_region_corners(
   kobject         object,
   kpds_primitive *prim,
   int            *size,
   int            *begin,
   int            *end)
{
   int w, h, d, e;
   
   kmemset((kaddr) begin, 0, (size_t) KDAS_MAX_DIM * sizeof(int));
   kpds_get_attribute(object, KPDS_LOCATION_REGION_SIZE, &w, &h, &d, &e);

   if (kstrcmp(prim->segment, KPDS_SEGMENT_WIDTH) == 0)
      end[0] = w - 1;
   else if (kstrcmp(prim->segment, KPDS_SEGMENT_HEIGHT) == 0)
      end[0] = h - 1;
   else if (kstrcmp(prim->segment, KPDS_SEGMENT_DEPTH) == 0)
      end[0] = d - 1;

   return;
}

/*-----------------------------------------------------------
|
|  Routine Name: increment_position
|
|       Purpose: This routine determines whether or not to 
|                increment a position.  This routine also
|                updates the increment state of the 
|		 presentation in question.
|
|         Input: a,b,c - three autoincrement state variables.
|                rd    - TRUE if operation is a read, FALSE
|                        if operation is write
|
|        Output: a,b,c - updated in place
|
|       Returns: TRUE if position should be incremented, FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: January 2, 1993
| Modifications:
|
------------------------------------------------------------*/
static int
increment_position(int *a, int *b, int *c, int rd)
{
   char ta, tb, q;

   /*-- execute one cycle of the state machine --*/
   q = (*a && !*b) || (!rd && *c) || (rd && *b && !*c);
   ta = (rd && *a) || (rd && *c) || (*b && !*c);
   tb = (rd && !*c) || (*a && !*b) || (!rd && *c);

   /*-- update the state --*/
   *a = (int) ta;
   *b = (int) tb;
   *c = !rd;

   /*-- return whether or not to increment --*/
   return ((int) q);
}


/*
 *  advance a position by unit within size
 */ 
static void
advance(
   int *position,
   int *size,
   int *unit,
   int  dim)
{
   int i;
   
   for (i = 0; i < dim - 1; i++)
   {
      position[i] += unit[i];

      if (position[i] < size[i])
         return;
      else
	 position[i] = 0;
   }

   position[dim - 1] += unit[dim - 1];

   if (position[dim - 1] >= size[dim - 1])
      position[dim - 1] = 0;

   /* return; end of the data */

   return;
}

/*
 *  advance a position and update the corresponding corner markers
 */ 
static void
advance_and_update(
   kobject         object,
   kpds_primitive *prim,
   int            *size,
   int             get_operation, 
   int            *begin,
   int            *end)
{
   ktoken seg = prim->segment_token;
   int  dimension;
   int *pos;
   int *off;
   int *isize;
   int *istate;
   int  nsize[KDAS_MAX_DIM];
   int  unit[KDAS_MAX_DIM];
   int  i, total_offset;
   
   kpds_get_access_info(object, seg, &dimension, &pos, &off, &isize, &istate);
   
   /*-------------------------------------------------------------------
   |  increment position first 
   \------------------------------------------------------------------*/
   for (i = 0; i < dimension; i++)
   {
      nsize[i] = (isize[i] > 0) ? isize[i] : size[i];
      unit[i]  = end[i] - begin[i] + 1;
   }
   
   /*-- this will modify the internal position & increment state --*/
   if (increment_position(&istate[0], &istate[1], &istate[2], get_operation))
      advance(pos, nsize, unit, dimension);

   /*-------------------------------------------------------------------
   |  move begin & end point to those indicated by the position & offset
   \------------------------------------------------------------------*/
   for (i = 0; i < dimension; i++)
   {
      if (prim->usage[i] == 0)
      {                         /* zero's indicate what to use */
         total_offset = 0;
         total_offset += (pos != NULL) ? pos[i] : 0;
         total_offset += (off != NULL) ? off[i] : 0;

         begin[i] += total_offset;
         end[i]   += total_offset;
      }
   }
   
   return;
}

/*
 *  get a primitive
 */ 
static kaddr
get_primitive(
   kobject         object,
   kpds_primitive *prim,
   kaddr           data)
{
   static int begin[KDAS_MAX_DIM];
   static int end  [KDAS_MAX_DIM];
   int       *size;

   if (!prim->get_size(object, prim, &size))
      return NULL;

   prim->find_corners  (object, prim, size, begin, end);
   prim->offset_corners(object, prim, size, TRUE, begin, end);

   return kdms_get_data(object, prim->segment, begin, end, data);
}

/*
 *  put a primitive
 */ 
static int
put_primitive(
   kobject         object,
   kpds_primitive *prim,
   kaddr           data)
{
   static int begin[KDAS_MAX_DIM];
   static int end  [KDAS_MAX_DIM];
   int       *size;

   if (!prim->get_size(object, prim, &size))
      return FALSE;

   prim->find_corners  (object, prim, size, begin, end);
   prim->offset_corners(object, prim, size, FALSE, begin, end);

   return kdms_put_data(object, prim->segment, begin, end, data);
}


/*-----------------------------------------------------------
|
|  Routine Name: get_histogram
|
|       Purpose: This is the data retrieval routine for the
|		 KPDS_VALUE_HISTOGRAM primitive.
|
|         Input: 
|
|        Output: 
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley & John Salas
|          Date: September 30, 1993
| Modifications:
|
------------------------------------------------------------*/
static kaddr
get_histogram(
   kobject         object,
   kpds_primitive *prim,
   kaddr           hist)
{
   int     i;
   int     beg[5], end[5];

   int     hr[5];    /*-- histogram region size --*/
   int     or[5];    /*-- optimal region size --*/

   int     hend[5];  /*-- end of histogram extent --*/

   int    *size;

   int     dimension, datatype;
   int     numbins;
   int     numpts;

   double  rmin, rmax, binwidth;
   kaddr   data = NULL;
   unsigned int *ihist;


   kdms_get_attributes(object, prim->segment,
		       KDMS_DIMENSION, &dimension,
		       KDMS_DATA_TYPE, &datatype,
		       KDMS_SIZE,      &size, 
		       NULL);

   kdms_get_attributes(object, KDMS_OBJECT,
		       KPDS_VALUE_HIST_POSITION, 
		          &beg[0], &beg[1], &beg[2], &beg[3], &beg[4],
		       KPDS_VALUE_HIST_REGION_SIZE, 
		          &hr[0], &hr[1], &hr[2], &hr[3], &hr[4],
		       KPDS_VALUE_HIST_NUMBINS, &numbins,
		       KPDS_VALUE_HIST_RANGE,   &rmin, &rmax,
		       NULL);

   /*-- the histogram region size indicates the overall histogram area --*/
   for (i = 0; i < dimension; i++)
      hend[i] = beg[i] + hr[i];  /*-- compute 'absolute' end of region --*/
   
   /*-- break the histogram region size up into optimal regions --*/
   kdas_optimal_region_size(dimension, hr, datatype, or);

   /*-- calculate the number of points in this optimal region --*/
   for (i = 0, numpts = 1; i < dimension; i++)
      numpts *= or[i];

   if (hist == NULL)
      hist = (unsigned int *) kcalloc((size_t)numbins, sizeof(unsigned int));
   else
      kmemset(hist, (int)(numbins * sizeof(unsigned int)), (size_t)0);
   
   ihist = (unsigned int *) hist;

   binwidth = (double) numbins / (rmax - rmin);

   /*-- loop over the optimal regions --*/
   do 
   {
      /*-- begin will be incremented at the end of each loop --*/

      /*-- calculate the end of this optimal region for latest begin --*/
      for (i = 0; i < dimension; i++)
	 end[i] = beg[i] + or[i];
      
      data = kdms_get_data(object, prim->segment, beg, end, data);


      switch (datatype)
      {
	 case KBYTE:
	    {
	       char *dta  = (char *) data;
	       char  dmin = (char)   rmin;

	       for (i = 0; i < numpts; i++)
		  ihist[(int)(kmin(numbins - 1, 
			  kmax(0, ((dta[i] - dmin) * binwidth))))] += 1;
	    }
	    break;
	 case KUBYTE:
	    {
	       unsigned char *dta  = (unsigned char *) data;
	       unsigned char  dmin = (unsigned char)   rmin;

	       for (i = 0; i < numpts; i++)
		  ihist[(int)(kmin((int)(numbins - 1),
	  (int)kmax(0, (int)((signed)(dta[i] - dmin) * binwidth))))] += 1;
	    }
	    break;
	 case KSHORT:
	    {
	       short *dta  = (short *) data;
	       short  dmin = (short)   rmin;
	
	       for (i = 0; i < numpts; i++)
		  ihist[(int)(kmin(numbins - 1,
			  kmax(0, ((dta[i] - dmin) * binwidth))))] += 1;
	    }
	    break;
	 case KUSHORT:
	    {
	       unsigned short *dta  = (unsigned short *) data;
	       unsigned short  dmin = (unsigned short)   rmin;

	       for (i = 0; i < numpts; i++)
		  ihist[(int)(kmin((int)(numbins - 1),
	      (int)kmax(0, (int)((signed)(dta[i] - dmin) * binwidth))))] += 1;
	    }
	    break;
	 case KINT:
	    {
	       int *dta  = (int *) data;
	       int  dmin = (int)   rmin;

	       for (i = 0; i < numpts; i++)
		  ihist[(int)(kmin(numbins - 1,
			 kmax(0, ((dta[i] - dmin) * binwidth))))] += 1;
	    }
	    break;
	 case KUINT:
	    {
	       unsigned int *dta  = (unsigned int *) data;
	       unsigned int  dmin = (unsigned int)   rmin;

	       for (i = 0; i < numpts; i++)
		  ihist[(int)(kmin(numbins - 1,
				   ((dta[i] - dmin) * binwidth)))] += 1;
	    }
	    break;
	 case KLONG:
	    {
	       long *dta  = (long *) data;
	       long  dmin = (long)   rmin;

	       for (i = 0; i < numpts; i++)
		  ihist[(int)(kmin(numbins - 1, 
			   kmax(0, ((dta[i] - dmin) * binwidth))))] += 1;
	    }
	    break;
	 case KULONG:
	    {
	       unsigned long *dta  = (unsigned long *) data;
	       unsigned long  dmin = (unsigned long)   rmin;

	       for (i = 0; i < numpts; i++)
		  ihist[(int)(kmin(numbins - 1, 
				   ((dta[i] - dmin) * binwidth)))] += 1;
	    }
	    break;
	 case KFLOAT:
	    {
	       float *dta  = (float *) data;
	       float  dmin = (float)   rmin;

	       for (i = 0; i < numpts; i++)
		  ihist[(int)(kmin(numbins - 1,
			   kmax(0, ((dta[i] - dmin) * binwidth))))] += 1;
	    }
	    break;
	 case KDOUBLE:
	    {
	       double *dta  = (double *) data;
	       double  dmin = (double)   rmin;

	       for (i = 0; i < numpts; i++)
		  ihist[(int)(kmin(numbins - 1,
			   kmax(0, ((dta[i] - dmin) * binwidth))))] += 1;
	    }
	    break;
      }

   } while (kdas_advance_position(dimension, hend, or, beg));

   kfree_and_NULL(data);

   return hist;
}
