 /*
  * Khoros: $Id: attr.c,v 1.8.2.4 1997/05/24 18:35:42 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: attr.c,v 1.8.2.4 1997/05/24 18:35:42 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Data Services Attribute Code
   >>>>		
   >>>>  Private:
   >>>>                 _kdas_vargs_to_attribute_data()
   >>>>                 _kdas_attribute_data_to_vargs()
   >>>>                 kdas_attr_generic_construct()
   >>>>                 kdas_attr_quasi_construct()
   >>>>                 kdas_attr_destruct()
   >>>>                 kdas_attr_get()
   >>>>                 kdas_attr_set()
   >>>>                 kdas_attr_match()
   >>>>                 kdas_attr_copy()
   >>>>                 kdas_attr_query()
   >>>>                 kdas_attr_print()
   >>>>                 kdas_attribute_list_free()
   >>>>                 kdas_list_string_locate()
   >>>>                 kdas_attr_setmode()
   >>>>   Static:
   >>>>                 _kdas_copy_attribute_data()
   >>>>                 _kdas_generic_get()
   >>>>                 _kdas_generic_set()
   >>>>                 _kdas_generic_match()
   >>>>                 _kdas_generic_copy()
   >>>>                 _kdas_generic_query()
   >>>>                 _kdas_generic_print()
   >>>>                 _kdas_undefined_get()
   >>>>                 _kdas_undefined_set()
   >>>>                 _kdas_undefined_match()
   >>>>                 _kdas_undefined_copy()
   >>>>                 _kdas_undefined_query()
   >>>>                 _kdas_undefined_print()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static int dupdee = TRUE;

/****    =============================================================
 ****    Attribute Utility Routines 
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: _kdas_vargs_to_attribute_data 
|
|       Purpose: This routine reads a variable argument list of 
|                data and combines that data into a single block 
|                of data.  The variable argument list will consist
|                of a given number of arguments, where each argument
|                consists of a given argument size of elements where
|                each element is of a given data type.  The data
|                is pulled off of the variable argument list and
|                combined into a single data block.
|
|                Note that if the argument size is equal to one, then
|                the data is simply assigned.  If the argument size
|                is greater than one, then the data is copied.
|
|                This data block can be decomposed and returned via 
|                variable argument pointers using the call
|                kdas_attribute_data_to_vargs().
|
|		 The data type of KSTRUCT is assumed to refer to
|		 a generic data pointer (kaddr), since there is
|		 no pointer data type.
|
|         Input: datatype - data type of attribute data
|                argsize  - argument size of each attribute arguyment
|                numargs  - number of arguments in attribute data 
|
|                list     - a variable argument list containing 
|                           data in the form :
|
|                              value1 [, value2, ...]
|
|                           The list should consist of "numarg" arguments,
|                           where each argument is of "argsize" size of
|                           "datatype" data type.
|
|        Output: dest     - pointer to pointer of where to return
|                           block of attribute data
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley and Steve Kubica
|          Date: March 17, 1994
| Modifications:
|
------------------------------------------------------------*/
int
_kdas_vargs_to_attribute_data(
   kaddr     *dest,
   int        datatype,
   int        argsize,
   int        numargs,
   kva_list  *list)
{
   int i, j, k;
   size_t size;
   int status = TRUE;

   if (dest == NULL)
      return FALSE;

   /*-- if the size or number of arguments is zero, then return --*/
   if (argsize == 0 || numargs == 0)
   {
      kinfo(KDEBUG, "_kdas_vargs_to_attribute_data : erroneous input\n"
            "   argsize = %d\n   numargs = %d\n", argsize, numargs);
      return FALSE;
   }

   /*-- if destination is NULL, then we must allocate space --*/
   if (*dest == NULL)
      if ((*dest = (kaddr) kcalloc((size_t)(numargs * argsize),
				   (size_t)kdata_size(datatype))) == NULL)
      {
	 kset_errno(KMEMORY_ALLOCATION);
	 return FALSE;
      }

   /*-- array arguments are handled generically (argsize > 1) --*/
   if (argsize > 1)
   {
      if (datatype == KSTRING) /*-- strings are a special case --*/
      {
	 kstring *stmp;
	 kstring *tmp = (kstring *) * dest;
	 for (i = 0, k = 0; i < numargs; i++)
	 {
	    stmp = kva_arg(*list, kstring *);
	    for (j = 0; j < argsize; j++, k++)
	    {
	       kfree(tmp[k]);
	       tmp[k] = kstring_copy(stmp[j], NULL);
	    }
	 }
	 return TRUE;
      }

      /*-- structures are also a special case --*/
      else if (kstruct_getinfo(datatype, NULL, &size, NULL, NULL, NULL, NULL))
      {
	 char *cdest = (char *) *dest;
	 for (i = 0; i < numargs; i++)
	 {
	    char *csrc  = kva_arg(*list, char *);
	    if (csrc == NULL)
	       for (j = 0; j < argsize; j++)
	       {
		  if (!dupdee)
		     memcpy(cdest,csrc,size);
		  else
                     kstruct_duplicate(csrc, datatype, cdest); 

		  cdest += ((int)size) * argsize;
		  csrc  += ((int)size) * argsize;
	       }
	 }
      }
      else /*-- regular old data type --*/
      {
	 char *dbyte = (char *)*dest;
	 for (i = 0; i < numargs; i++)
	    kmemcpy(dbyte + i * argsize * kdata_size(datatype),
		    kva_arg(*list, char *),
		    (size_t)(argsize * kdata_size(datatype)));
	 return TRUE;
      }
   }

   /*-- for when argsize == 1 --*/
   switch (datatype)
   {
      case KBYTE:
         {
            signed char *tmp = (signed char *)*dest;
            for (i = 0; i < numargs; i++)
               tmp[i] = (signed char)kva_arg(*list, int);
         }
         break;

      case KUBYTE:
         {
            unsigned char *tmp = (unsigned char *)*dest;
            for (i = 0; i < numargs; i++)
               tmp[i] = (unsigned char)kva_arg(*list, unsigned int);
         }
         break;

      case KSHORT:
         {
            short *tmp = (short *)*dest;
            for (i = 0; i < numargs; i++)
               tmp[i] = (short)kva_arg(*list, int);
         }
         break;

      case KUSHORT:
         {
            unsigned short *tmp = (unsigned short *)*dest;
            for (i = 0; i < numargs; i++)
               tmp[i] = (unsigned short)kva_arg(*list, unsigned int);
         }
         break;

      case KINT:
         {
            int *tmp = (int *)*dest;
            for (i = 0; i < numargs; i++)
               tmp[i] = kva_arg(*list, int);
         }
         break;

      case KUINT:
         {
            unsigned int *tmp = (unsigned int *)*dest;
            for (i = 0; i < numargs; i++)
               tmp[i] = kva_arg(*list, unsigned int);
         }
         break;

      case KLONG:
         {
            long *tmp = (long *)*dest;
            for (i = 0; i < numargs; i++)
               tmp[i] = kva_arg(*list, long);
         }
         break;

      case KULONG:
         {
            unsigned long *tmp = (unsigned long *)*dest;
            for (i = 0; i < numargs; i++)
               tmp[i] = kva_arg(*list, unsigned long);
         }
         break;

      case KFLOAT:
         {
            float *tmp = (float *)*dest;
            for (i = 0; i < numargs; i++)
               tmp[i] = (float)kva_arg(*list, double);
         }
         break;

      case KDOUBLE:
         {
            double *tmp = (double *)*dest;
            for (i = 0; i < numargs; i++)
               tmp[i] = kva_arg(*list, double);
         }
         break;

      case KCOMPLEX:
         {
            kcomplex *tmp = (kcomplex *) * dest;
            for (i = 0; i < numargs; i++)
               tmp[i] = kva_arg(*list, kcomplex);
         }
         break;

      case KDCOMPLEX:
         {
            kdcomplex *tmp = (kdcomplex *) * dest;
            for (i = 0; i < numargs; i++)
               tmp[i] = kva_arg(*list, kdcomplex);
         }
         break;

      case KSTRING:
         {
            kstring *tmp = (kstring *) * dest;
            for (i = 0; i < numargs; i++)
               tmp[i] = kstring_copy(kva_arg(*list, kstring), NULL);
         }
         break;

      case KSTRUCT:
         {
            kaddr *tmp = (kaddr *) * dest;
            for (i = 0; i < numargs; i++)
               tmp[i] = kva_arg(*list, kaddr);
         }
         break;

      default:         
	 if (kstruct_getinfo(datatype, NULL, &size, NULL, NULL, NULL, NULL))
         {
            char *tmp = (char *) *dest; /* kaddr */
	    
            for (i = 0; i < numargs; i++, tmp += (int)size*argsize)
            {
               kaddr ptr = kva_arg(*list, kaddr);
               if (ptr != NULL)
               {
                  if (!dupdee)
                     memcpy(tmp,ptr,size);
                  else
                     kstruct_duplicate(ptr, datatype, tmp);
               }
	       else
		  kfree_and_NULL(*dest);
            }
         }
         else
	 {
	    kset_errno(KDAS_EATTR_UNKNOWN_DATATYPE);
	    return FALSE;
	 }
   }
   return status;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdas_attribute_data_to_vargs
|
|       Purpose: This routine decomposes a block of attribute data
|                into its consituent parts and returns them via a
|                variable argument list of pointers to data.
|                The variable argument list will consist of a given
|                number of arguments, where each argument is a pointer
|                to data of a given argument size of elements where
|                each element is of a given data type.  
|
|                Note that if the argument size is equal to one, then
|                the data is simply returned via the given data pointer.  
|                If the argument size is greater than one, then a pointer 
|                to the internal copy of the data is returned.  The user
|                must be careful not to change or free this data.
|
|                This data can be composed into a data block from
|                the variable argument pointers using the call
|                _kdas_vargs_to_attribute_data().
|
|		 The data type of KSTRUCT is assumed to refer to
|		 a generic data pointer (kaddr), since there is
|		 no pointer data type.
|
|         Input: src      - pointer to attribute data 
|                datatype - data type of attribute data
|                argsize  - argument size of each attribute arguyment
|                numargs  - number of arguments in attribute data 
|
|        Output: list     - a variable argument list containing 
|                           data in the form :
|
|                              &value1 [, &value2, ...]
|
|                           The list should consist of "numarg" arguments,
|                           where each argument is a pointer to "argsize" 
|                           size of "datatype" data type.
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley and Steve Kubica
|          Date: March 17, 1994
| Modifications:
|
------------------------------------------------------------*/
int
_kdas_attribute_data_to_vargs(
   kaddr     src,
   int       datatype,
   int       argsize,
   int       numargs,
   kva_list *list)
{
   kaddr *entity;
   int    i;
   size_t size;

   /*-- if the size or number of arguments is zero, then return --*/
   if (argsize == 0 || numargs == 0)
   {
      kinfo(KDEBUG, "_kdas_attribute_data_to_vargs : erroneous input\n"
            "   argsize = %d\n   numargs = %d\n", argsize, numargs);
      return FALSE;
   }

   /*-- if the data is null, then the attribute has not yet been set --*/
   if (src == NULL)
   {
      kset_errno(KDAS_EATTR_UNSET);
      return FALSE;
   }

   /*-- array arguments are handled generically (argsize > 1) --*/
   if (argsize > 1)
   {
      char *sbyte = (char *)src;
      for (i = 0; i < numargs; i++)
      {
         entity = kva_arg(*list, kaddr *);
         if (entity != NULL)
            *entity = (kaddr) (sbyte + i * argsize * kdata_size(datatype));
      }
      return TRUE;
   }

   /*-- case for when argsize == 1 --*/
   switch (datatype)
   {
      case KBYTE:
         {
	    signed char *tmp;
	    for (i = 0; i < numargs; i++)
	    {
	       tmp = (kva_arg(*list, signed char *));
	       if (tmp)
		  *tmp = (signed char)((signed char *)src)[i];
	    }
	 }
	 break;

      case KUBYTE:
	 {
	    unsigned char *tmp;
	    for (i = 0; i < numargs; i++)
	    {
	       tmp = (kva_arg(*list, unsigned char *));
	       if (tmp)
		  *tmp = (unsigned char)((unsigned char *)src)[i];
	    }
	 }
	 break;

      case KSHORT:
	 {
	    short *tmp;
	    for (i = 0; i < numargs; i++)
	    {
	       tmp = (kva_arg(*list, short *));
	       if (tmp)
		  *tmp = (short)((short *)src)[i];
	    }
	 }
	 break;

      case KUSHORT:
	 {
	    unsigned short *tmp;
	    for (i = 0; i < numargs; i++)
	    {
	       tmp = (kva_arg(*list, unsigned short *));
	       if (tmp)
		  *tmp = (unsigned short)((unsigned short *)src)[i];
	    }
	 }
	 break;

      case KINT:
	 {
	    int *tmp;
	    for (i = 0; i < numargs; i++)
	    {
	       tmp = (kva_arg(*list, int *));
	       if (tmp)
		  *tmp = (int)((int *)src)[i];
	    }
	 }
	 break;

      case KUINT:
	 {
	    unsigned int *tmp;
	    for (i = 0; i < numargs; i++)
	    {
	       tmp = (kva_arg(*list, unsigned int *));
	       if (tmp)
		  *tmp = (unsigned int)((unsigned int *)src)[i];
	    }
	 }
	 break;

      case KLONG:
	 {
	    long *tmp;
	    for (i = 0; i < numargs; i++)
	    {
	       tmp = (kva_arg(*list, long *));
	       if (tmp)
		  *tmp = (long)((long *)src)[i];
	    }
	 }
	 break;

      case KULONG:
	 {
	    unsigned long *tmp;
	    for (i = 0; i < numargs; i++)
	    {
	       tmp = (kva_arg(*list, unsigned long *));
	       if (tmp)
		  *tmp = (unsigned long)((unsigned long *)src)[i];
	    }
	 }
	 break;

      case KFLOAT:
	 {
	    float *tmp;
	    for (i = 0; i < numargs; i++)
	    {
	       tmp = (kva_arg(*list, float *));
	       if (tmp)
		  *tmp = (float)((float *)src)[i];
	    }
	 }
	 break;

      case KDOUBLE:
	 {
	    double *tmp;
	    for (i = 0; i < numargs; i++)
	    {
	       tmp = (kva_arg(*list, double *));
	       if (tmp)
		  *tmp = (double)((double *)src)[i];
	    }
	 }
	 break;

      case KCOMPLEX:
	 {
	    kcomplex *tmp = (kcomplex *) src;
	    kcomplex *tmp2;
	    for (i = 0; i < numargs; i++)
	    {
	       tmp2 = (kva_arg(*list, kcomplex *));
	       if (tmp2)
		  *tmp2 = tmp[i];
	    }
	 }
	 break;

      case KDCOMPLEX:
	 {
	    kdcomplex *tmp = (kdcomplex *) src;
	    kdcomplex *tmp2;
	    for (i = 0; i < numargs; i++)
	    {
	       tmp2 = (kva_arg(*list, kdcomplex *));
	       if (tmp2)
		  *tmp2 = tmp[i];
	    }
	 }
	 break;

      case KSTRING:
	 {
	    for (i = 0; i < numargs; i++)
	       *(kva_arg(*list, kstring *)) = ((kstring *) src)[i];
	 }
	 break;

      case KSTRUCT:
	 {
	    kaddr *tmp;
	    for (i = 0; i < numargs; i++)
	    {
	       tmp = (kva_arg(*list, kaddr *));
	       if (tmp)
		  *tmp = (kaddr)((kaddr *)src)[i];
	    }
	 }
	 break;

      default:
         if (kstruct_getinfo(datatype, NULL, &size, NULL, NULL, NULL, NULL))
         {
            kaddr *tmp;
            char  *srctmp = (char *) src;
            
            for (i = 0; i < numargs; i++, srctmp += (int)size*argsize)
            {
               tmp = (kva_arg(*list, kaddr *));
               if (tmp)
                  *tmp = (kaddr)srctmp;
            }
         }
         else
	 {
	    kset_errno(KDAS_EATTR_UNKNOWN_DATATYPE);
	    return FALSE;
	 }
   }
   return TRUE;
}


/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdas_copy_attribute_data
|
|       Purpose: This routine copies a block of attribute data from
|                one pointer to another.  If the destination pointer
|                points to NULL, then space is allocated for the
|                resulting copy.  The space is otherwise assumed to 
|                be of a sufficient size to hold the copy.  This size
|                is determined by multiplying the given data type by
|                the given argument size by the given number of 
|                arguments.
|
|         Input: src      - pointer to src attribute data
|                datatype - data type of attribute data to copy
|                argsize  - argument size of attribute data to copy
|                numargs  - number of arguments in attribute data to copy
|
|        Output: dest     - pointer to pointer of where to put copied data
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley and Steve Kubica
|          Date: March 17, 1994
| Modifications:
|
------------------------------------------------------------*/
static int
_kdas_copy_attribute_data(
   kaddr  src,
   kaddr *dest,
   int    datatype,
   int    argsize,
   int    numargs)
{
   int i;
   size_t size;
   

   if (src == NULL || dest == NULL)
      return FALSE;

   /*-- if the size or number of arguments is zero, then return --*/
   if (argsize == 0 || numargs == 0)
   {
      kinfo(KDEBUG, "_kdas_copy_attribute_data : erroneous input\n"
	    "   argsize = %d\n   numargs = %d\n", argsize, numargs);
      return FALSE;
   }

   /*--------------------------------------------------------------------
   |  if the destination data pointer is null, then we must allocate 
   |  space for this thing.  otherwise assume space there is enough
   |  (it is necessary to use calloc because we've gotta realloc if its
   |  a string and the krealloc is smart enough to detect the NULL)
   \--------------------------------------------------------------------*/
   if (*dest == NULL)
      if ((*dest = (kaddr) kcalloc((size_t)(numargs * argsize *
					    kdata_size(datatype)), 
				   (size_t) 1)) == NULL)
      {
	 kset_errno(KMEMORY_ALLOCATION);
	 return FALSE;
      }

   /*-- do the copy --*/
   if (datatype == KSTRING)
   {
      kstring  *sbyte = (kstring *)  src;  /*-- indexable version of src  --*/
      kstring **dbyte = (kstring **) dest; /*-- indexable version of dest --*/

      for (i = 0; i < numargs * argsize; i++)
      {
	 /* kfree((*dbyte)[i]);  
		w/o this we get a memory leak, with we get a FMR ... 
		choose lesser of two evils */
	 (*dbyte)[i] = kstring_copy(sbyte[i], NULL);
      }
   }
   else if (kstruct_getinfo(datatype, NULL, &size, NULL, NULL, NULL, NULL))
   {
     char *csrc  = (char *)  src;
     char *cdest = (char *) *dest;
     for (i = 0; i < argsize; i++)
     {
	kstruct_duplicate(csrc, datatype, cdest); 
	cdest += (int)size * argsize;
	csrc  += (int)size * argsize;
     }
   }
   else
      kmemcpy((*dest), src, (size_t)(argsize*numargs*kdata_size(datatype)));

   return TRUE;
}

/****    =============================================================
 ****    Generic Attribute Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|  Routine Name: (static) _kdas_generic_get
|       Purpose: perform a generic attribute retrieval
|    Written By: Jeremy Worley
|          Date: September 23, 1993
------------------------------------------------------------*/
static int
_kdas_generic_get(
   kdas_generic_attribute *atr,
   kva_list               *list)
{
   int status;

   if (atr == NULL)
   {
      kinfo(KDEBUG, "_kdas_generic_get : NULL attribute provided\n");
      return FALSE;
   }

   status = _kdas_attribute_data_to_vargs(atr->value, atr->datatype,
					  atr->argsize, atr->numargs, list);
   return status;
}


/*-----------------------------------------------------------
|  Routine Name: (static) _kdas_generic_set
|       Purpose: perform a generic attribute set
|    Written By: Jeremy Worley
|          Date: September 23, 1993
------------------------------------------------------------*/
static int
_kdas_generic_set(
   kdas_generic_attribute *atr,
   kva_list               *list)
{
   int status;

   /*-- sanity check --*/
   if (atr == NULL)
   {
      kinfo(KDEBUG, "_kdas_generic_set : NULL attribute provided\n");
      return FALSE;
   }

   status = _kdas_vargs_to_attribute_data(&(atr->value), atr->datatype,
					  atr->argsize, atr->numargs, list);
   return status;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _kdas_generic_match
|       Purpose: perform a generic attribute match
|    Written By: Jeremy Worley
|          Date: September 23, 1993
------------------------------------------------------------*/
static int
_kdas_generic_match(
   kdas_generic_attribute *atr1,
   kdas_generic_attribute *atr2)
{
   int i;
   size_t size;
   

   if (atr1 == NULL || atr2 == NULL)
   {
      kinfo(KDEBUG, "_kdas_generic_match : NULL attribute provided\n");
      return FALSE;
   }

   /*-- do the attributes have the same characteristics? --*/
   if ((atr1->datatype != atr2->datatype) ||
       (atr1->argsize  != atr2->argsize)  ||
       (atr1->numargs  != atr2->numargs))
      return FALSE;		/*-- not an error --*/

   if (atr1->datatype == KSTRING)
   {
      kstring *c1data = (kstring *) atr1->value;
      kstring *c2data = (kstring *) atr2->value;
      for (i = 0; i < atr1->numargs; i++)
	 if (kstrcmp(c1data[i], c2data[i]) != 0)
	    return FALSE;	/*-- not an error --*/
   }
   else if (kstruct_getinfo(atr1->datatype, NULL, &size, 
                            NULL, NULL, NULL, NULL))
   {
     char *c1 = (char *) atr1->value;
     char *c2 = (char *) atr2->value;
     for (i = 0; i < atr1->argsize; i++)
     {
	if (!kstruct_compare(c1, c2, atr1->datatype))
	   return FALSE;
	c1 += (int)size * atr1->argsize;
	c2 += (int)size * atr2->argsize; 
     }
   }
   else
   {
      if (kmemcmp(atr1->value, atr2->value, 
		  (size_t)(atr1->argsize * atr1->numargs *
			   kdata_size(atr1->datatype))) != 0)
	 return FALSE;	/*-- not an error --*/
   }

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _kdas_generic_copy
|       Purpose: perform a generic attribute copy
|    Written By: Jeremy Worley
|          Date: September 23, 1993
------------------------------------------------------------*/
static int
_kdas_generic_copy(
   kdas_generic_attribute *atr1,
   kdas_generic_attribute *atr2)
{
   int status;

   if (atr1 == NULL || atr2 == NULL)
   {
      kinfo(KDEBUG, "_kdas_generic_copy : null attribute provided\n");
      return FALSE;
   }

   /*-- only copy if both attributes have the same characteristics --*/
   if ((atr1->datatype != atr2->datatype) ||
       (atr1->argsize  != atr2->argsize)  ||
       (atr1->numargs  != atr2->numargs))
   {
      kset_errno(KDAS_EATTR_COPY_FAILED);
      kinfo(KDEBUG, "_kdas_generic_copy : "
	    "attribute 1 : "
	    "data type = %s, argument size = %d, number of arguments = %d\n"
	    "attribute 2 : "
	    "data type = %s, argument size = %d, number of arguments = %d\n",
	    kdefine_to_datatype(atr1->datatype), atr1->argsize, atr1->numargs,
	    kdefine_to_datatype(atr2->datatype), atr2->argsize, atr2->numargs);
      return FALSE;
   }

   status = _kdas_copy_attribute_data(atr1->value, &atr2->value,
				      atr1->datatype, atr1->argsize,
				      atr1->numargs);
   return status;
}


/*-----------------------------------------------------------
|  Routine Name: (static) _kdas_generic_query
|       Purpose: perform a generic attribute query
|    Written By: Jeremy Worley and Steve Kubica
|          Date: September 23, 1993
------------------------------------------------------------*/
static int
_kdas_generic_query(
   kdas_generic_attribute *atr,
   int                    *numargs,
   int                    *argsize,
   int                    *datatype,
   int                    *permanent)
{
   if (atr == NULL)
   {
      kinfo(KDEBUG, "_kdas_generic_query : null attribute provided\n");
      return FALSE;
   }

   if (numargs)
      *numargs = atr->numargs;

   if (argsize)
      *argsize = atr->argsize;

   if (datatype)
      *datatype = atr->datatype;

   if (permanent)
      *permanent = atr->permanent;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _kdas_generic_print
|       Purpose: perform a generic attribute print
|    Written By: Steve Kubica
|          Date: March 21, 1994
------------------------------------------------------------*/
static int
_kdas_generic_print(
   kdas_generic_attribute *atr,
   kfile                  *outfile)
{
   int   i, j;
   char *name;
   int   status = TRUE;

   if (atr == NULL || atr->value == NULL)
   {
      kinfo(KDEBUG, "_kdas_generic_print : NULL attribute provided\n");
      return FALSE;
   }

   /*-- if the size or number of arguments is zero, then return --*/
   if (atr->argsize == 0 || atr->numargs == 0)
   {
      kinfo(KDEBUG, "_kdas_generic_print : erroneous input\n"
	    "   argsize = %d\n   numargs = %d\n", atr->argsize, atr->numargs);
      return FALSE;
   }

   switch (atr->datatype)
   {
      case KBYTE:
	 {
	    signed char *tmp = (signed char *)atr->value;
	    for (i = 0; i < atr->numargs; i++)
	    {
	       for (j = 0; j < atr->argsize; j++)
		  kfprintf(outfile, " %d ", tmp[j + i * atr->argsize]);
	       if (i < atr->numargs - 1)
		  kfprintf(outfile, ",");
	    }
	 }
	 break;

      case KUBYTE:
	 {
	    unsigned char *tmp = (unsigned char *)atr->value;
	    for (i = 0; i < atr->numargs; i++)
	    {
	       for (j = 0; j < atr->argsize; j++)
		  kfprintf(outfile, " %d ", tmp[j + i * atr->argsize]);
	       if (i < atr->numargs - 1)
		  kfprintf(outfile, ",");
	    }
	 }
	 break;

      case KSHORT:
	 {
	    short *tmp = (short *)atr->value;
	    for (i = 0; i < atr->numargs; i++)
	    {
	       for (j = 0; j < atr->argsize; j++)
		  kfprintf(outfile, " %d ", tmp[j + i * atr->argsize]);
	       if (i < atr->numargs - 1)
		  kfprintf(outfile, ",");
	    }
	 }
	 break;

      case KUSHORT:
	 {
	    unsigned short *tmp = (unsigned short *)atr->value;
	    for (i = 0; i < atr->numargs; i++)
	    {
	       for (j = 0; j < atr->argsize; j++)
		  kfprintf(outfile, " %d ", tmp[j + i * atr->argsize]);
	       if (i < atr->numargs - 1)
		  kfprintf(outfile, ",");
	    }
	 }
	 break;

      case KINT:
	 {
	    int *tmp = (int *)atr->value;
	    for (i = 0; i < atr->numargs; i++)
	    {
	       for (j = 0; j < atr->argsize; j++)
		  kfprintf(outfile, " %d ", tmp[j + i * atr->argsize]);
	       if (i < atr->numargs - 1)
		  kfprintf(outfile, ",");
	    }
	 }
	 break;

      case KUINT:
	 {
	    unsigned int *tmp = (unsigned int *)atr->value;
	    for (i = 0; i < atr->numargs; i++)
	    {
	       for (j = 0; j < atr->argsize; j++)
		  kfprintf(outfile, " %d ", tmp[j + i * atr->argsize]);
	       if (i < atr->numargs - 1)
		  kfprintf(outfile, ",");
	    }
	 }
	 break;

      case KLONG:
	 {
	    long *tmp = (long *)atr->value;
	    for (i = 0; i < atr->numargs; i++)
	    {
	       for (j = 0; j < atr->argsize; j++)
		  kfprintf(outfile, " %d ", tmp[j + i * atr->argsize]);
	       if (i < atr->numargs - 1)
		  kfprintf(outfile, ",");
	    }
	 }
	 break;

      case KULONG:
	 {
	    unsigned long *tmp = (unsigned long *)atr->value;
	    for (i = 0; i < atr->numargs; i++)
	    {
	       for (j = 0; j < atr->argsize; j++)
		  kfprintf(outfile, " %d ", tmp[j + i * atr->argsize]);
	       if (i < atr->numargs - 1)
		  kfprintf(outfile, ",");
	    }
	 }
	 break;

      case KFLOAT:
	 {
	    float *tmp = (float *)atr->value;
	    for (i = 0; i < atr->numargs; i++)
	    {
	       for (j = 0; j < atr->argsize; j++)
		  kfprintf(outfile, " %g ", tmp[j + i * atr->argsize]);
	       if (i < atr->numargs - 1)
		  kfprintf(outfile, ",");
	    }
	 }
	 break;

      case KDOUBLE:
	 {
	    double *tmp = (double *)atr->value;
	    for (i = 0; i < atr->numargs; i++)
	    {
	       for (j = 0; j < atr->argsize; j++)
		  kfprintf(outfile, " %g ", tmp[j + i * atr->argsize]);
	       if (i < atr->numargs - 1)
		  kfprintf(outfile, ",");
	    }
	 }
	 break;

      case KCOMPLEX:
	 {
	    kcomplex *tmp = (kcomplex *) atr->value;
	    for (i = 0; i < atr->numargs; i++)
	    {
	       for (j = 0; j < atr->argsize; j++)
		  kfprintf(outfile, " ( %g, %g ) ",
			   (tmp[j + i * atr->argsize]).r,
			   (tmp[j + i * atr->argsize]).i);
	       if (i < atr->numargs - 1)
		  kfprintf(outfile, ",");
	    }
	 }
	 break;

      case KDCOMPLEX:
	 {
	    kdcomplex *tmp = (kdcomplex *) atr->value;
	    for (i = 0; i < atr->numargs; i++)
	    {
	       for (j = 0; j < atr->argsize; j++)
		  kfprintf(outfile, " ( %g, %g ) ",
			   (tmp[j + i * atr->argsize]).r,
			   (tmp[j + i * atr->argsize]).i);
	       if (i < atr->numargs - 1)
		  kfprintf(outfile, ",");
	    }
	 }
	 break;

      case KSTRING:
	 {
	    kstring *tmp = (kstring *) atr->value;
	    for (i = 0; i < atr->numargs; i++)
	    {
	       for (j = 0; j < atr->argsize; j++)
		  if (tmp[j + i * atr->argsize] != NULL)
		     kfprintf(outfile, " %s ", tmp[j + i * atr->argsize]);
		  else 
		     kfprintf(outfile, " (null) ");
		  
	       if (i < atr->numargs - 1)
		  kfprintf(outfile, ",");
	    }
	 }
	 break;

      default:
         if (kstruct_getinfo(atr->datatype,&name,NULL,NULL,NULL,NULL,NULL))
         {
            if (name != NULL)
               kfprintf(outfile, "(structure %s)", name );
            else
               kfprintf(outfile, "(unnamed structure type)");
            
            return TRUE;
         }
         else
	 {
	    kset_errno(KDAS_EATTR_UNKNOWN_DATATYPE);
	    return FALSE;
	 }
   }

   return status;
}

/****    =============================================================
 ****    Quasi Attribute Methods
 ****    =============================================================
 ****/

/*
 * Centerline's CLCC doesn't honor the ARGSUSED *standard* so
 * this pragma turns off warnings about unused arguments, which
 * are numerous in the next few routines
 */
#ifdef __CLCC__
#pragma Warning_level (3)
#endif

/*-----------------------------------------------------------
|  Routine Name: (static) _kdas_undefined_get
|       Purpose: used if a quasi-attribute get method is undefined
|    Written By: Steve Kubica
|          Date: March 21, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_kdas_undefined_get(kobject obj, ktoken assoc, ktoken attrib, 
		    kaddr client_data, kva_list * list)
{
   return FALSE;
}


/*-----------------------------------------------------------
|  Routine Name: (static) _kdas_undefined_set
|       Purpose: used if a quasi-attribute put method is undefined
|    Written By: Steve Kubica
|          Date: March 21, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_kdas_undefined_set(kobject obj, ktoken assoc, ktoken attrib, 
		    kaddr client_data, kva_list * list)
{
   return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _kdas_undefined_match
|       Purpose: used if a quasi-attribute match method is undefined
|    Written By: Steve Kubica
|          Date: March 21, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_kdas_undefined_match(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
		      kaddr client_data1, kaddr client_data2)
{
   return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _kdas_undefined_copy
|       Purpose: used if a quasi-attribute copy method is undefined
|    Written By: Steve Kubica
|          Date: March 21, 1994
| Modifications:
------------------------------------------------------------*/
/* ARGSUSED */
static int
_kdas_undefined_copy(kobject obj1, kobject obj2, ktoken assoc, ktoken attrib,
		     kaddr client_data1, kaddr client_data2)
{
   return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _kdas_query_undefined
|       Purpose: used if a quasi-attribute query method is undefined
|    Written By: Steve Kubica
|          Date: March 21, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_kdas_undefined_query(kobject obj, ktoken assoc, ktoken attrib, 
		      kaddr client_data, int *numargs, int *argsize, 
		      int *datatype, int *permanent)
{
   return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _kdas_undefined_print
|       Purpose: used if a quasi-attribute print method is undefined
|    Written By: Steve Kubica
|          Date: March 21, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static int
_kdas_undefined_print(kobject obj, ktoken assoc, ktoken attrib, 
		      kaddr client_data, kfile *outfile)
{
   return FALSE;
}

/*
 * Centerline's CLCC doesn't honor the ARGSUSED *standard* so
 * this pragma turns back on warnings about unused arguments, which
 * occur in the above service routines.
 */
#ifdef __CLCC__
#pragma Warning_level (4)
#endif

/****    =============================================================
 ****    Attribute Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kdas_attr_generic_construct
|
|       Purpose: This routine will construct a generic-type
|                attribute according to the given arguments.
|
|		 The attribute is distinguished by the attribute
|		 identifier, which is simply a tokenized
|		 version of the attribute name, such as "size".
|
|		 A default data block may be provided for
|		 initializing the value of the attribute. If
|		 a default value is provided, a copy of that
|		 value will be made and assigned to the attribute.
|		 If no default is provided (NULL), then the
|		 attribute value will be initialized to be NULL.
|
|                Note that the constructors for each attribute
|                type are implemented as separate functions 
|                because of their wildly different function parameters.
|
|         Input: attribute - attribute identifier 
|		 datatype  - data type of the attribute
|  		 argsize   - size of each attribute argument
|		 numargs   - number of arguments in attribute
| 		 permanent - is this attribute permanent
|		 def       - default values to use when
|                            initializing the attribute value.
|        Output: none
|
|       Returns: a pointer to the constructed attribute on success,
|		 NULL otherwise
|
|    Written By: Steve Kubica
|          Date: March 17, 1994
| Modifications:
|
------------------------------------------------------------*/
kdas_attribute *
kdas_attr_generic_construct(
   ktoken attribute,
   int    datatype,
   int 	  argsize,
   int 	  numargs,
   int 	  permanent,
   kaddr  def)
{
   kdas_attribute *atr = NULL;

   /*-- allocate a new attribute structure --*/
   if ((atr = (kdas_attribute *) kmalloc(sizeof(kdas_attribute))) == NULL)
   {
      kset_errno(KMEMORY_ALLOCATION);
      return FALSE;
   }

   /*-- assign the base information --*/
   atr->generic.type      = KDAS_GENERIC_ATTRIBUTE;
   atr->generic.attribute = attribute;

   /*-- fill out the generic attribute information --*/
   atr->generic.datatype  = datatype;
   atr->generic.argsize   = argsize;
   atr->generic.numargs   = numargs;
   atr->generic.permanent = permanent;
   atr->generic.value     = NULL;

   /*-- assign the default data (if present) to the attributes value --*/
   if (def)
      _kdas_copy_attribute_data(def, &(atr->generic.value),
				datatype, argsize, numargs);

   return atr;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_attr_quasi_construct
|
|       Purpose: This routine will construct a quasi-type
|                attribute according to the given arguments.
|
|		 The attribute is distinguished by the attribute
|		 identifier, which is simply a tokenized
|		 version of the attribute name, such as "size".
|
|		 Functions are provided for each of the attribute's
|		 methods. If NULL is passed in for any of the
|		 functions, an undefined function will be used.
|
|		 Client data may be given to the attribute.
|		 This client data will be passed into each
|		 of the given attribute methods.  Note that
|		 a copy of this client data is _not_ made,
|		 so the address of a static variable must be
|		 provided.
|		 
|                Note that the constructors for each attribute
|                type are implemented as separate functions 
|                because of their wildly different function parameters.
|
|         Input: attribute    - attribute name
|		 get()        - get function to use
|  		 set()        - set function to use
|		 match()      - match function to use
| 		 copy()       - copy function to use 
|		 query()      - query function to use 
|		 print()      - print function to use 
|		 client_data  - client data to pass into the attribute's
|			        methods.
|        Output: none
|
|       Returns: a pointer to the constructed attribute on success,
|		 NULL otherwise
|
|    Written By: Steve Kubica
|          Date: March 21, 1994
| Modifications:
|
------------------------------------------------------------*/
kdas_attribute *
kdas_attr_quasi_construct(
   char *attribute,
   int (*get)   PROTO((kobject, ktoken, ktoken, kaddr, kva_list *)),
   int (*set)   PROTO((kobject, ktoken, ktoken, kaddr, kva_list *)),
   int (*match) PROTO((kobject, kobject, ktoken, ktoken, kaddr, kaddr)),
   int (*copy)  PROTO((kobject, kobject, ktoken, ktoken, kaddr, kaddr)),
   int (*query) PROTO((kobject, ktoken, ktoken,kaddr,int *,int *,int *,int *)),
   int (*print) PROTO((kobject, ktoken, ktoken, kaddr, kfile *)),
   kaddr client_data)
{
   kdas_attribute *atr = NULL;

   /*-- allocate a new attribute structure --*/
   if ((atr = (kdas_attribute *) kmalloc(sizeof(kdas_attribute))) == NULL)
   {
      kset_errno(KMEMORY_ALLOCATION);
      return FALSE;
   }

   /*-- assign the base information --*/
   atr->quasi.type = KDAS_QUASI_ATTRIBUTE;
   
   /*-- tokenized version for handy reference --*/
   atr->quasi.attribute = kstring_to_token(attribute);

   /*-- fill out the generic attribute methods --*/
   atr->quasi.get   = (get)   ? get   : _kdas_undefined_get;
   atr->quasi.set   = (set)   ? set   : _kdas_undefined_set;
   atr->quasi.match = (match) ? match : _kdas_undefined_match;
   atr->quasi.copy  = (copy)  ? copy  : _kdas_undefined_copy;
   atr->quasi.query = (query) ? query : _kdas_undefined_query;
   atr->quasi.print = (print) ? print : _kdas_undefined_print;

   /*-- initialize the object and client_data to be NULL --*/
   atr->quasi.client_data = client_data;

   return atr;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_attr_destruct
|
|       Purpose: Destroy an attribute.  This function can
|		 destroy both generic and quasi attributes.
|		 All internal attribute data, as well as 
|		 the attribute itself, will be freed.
|
|         Input: atr - the attribute to free
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: March 17, 1994
| Modifications:
|
------------------------------------------------------------*/
void
kdas_attr_destruct(kdas_attribute *atr)
{
   int i;
   size_t size;
   

   /*-- trivial free --*/
   if (atr == NULL)
      return;

   /*-- this is only case where attribute contains something to free --*/  
   if (atr->base.type == KDAS_GENERIC_ATTRIBUTE)
   {
      if (atr->generic.datatype == KSTRING)
      {
	 kstring *cdata = (kstring *) atr->generic.value;
	 for (i = 0; i < atr->generic.argsize * atr->generic.numargs; i++)
	    kfree(cdata[i]);

         kfree(atr->generic.value);
      }
      else if (kstruct_getinfo(atr->generic.datatype, NULL, &size, 
                               NULL, NULL, NULL, NULL))
      {
         char *c1  = (char *) atr->generic.value;

         for (i = 0; i < atr->generic.argsize; i++) 
	 {
	    kstruct_free(c1, atr->generic.datatype, FALSE);
	    c1 += (int) size * atr->generic.argsize;
	 }
	 kfree_and_NULL(atr->generic.value);
      }
      else
         kfree_and_NULL(atr->generic.value);
   }
   
   /*-- finally, free the attribute --*/
   atr->base.type = -1; /*-- sanity --*/

   kfree(atr);

   return;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_attr_get
|
|       Purpose: Get an attribute.  This function can
|		 get both generic and quasi attributes.
|		 It takes an open variable argument list
|		 as the source of variables to use in 
|		 returning the attribute values.
|
|         Input: atr     - the attribute to get
|		 object  - object for quasi calls
|		 segment - tokenized segment for quasi calls
|		 va_list - a variable argument list containing 
|		           variables in which to return the
|			   values of the attribute
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: March 17, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_attr_get(kdas_attribute *atr, kobject object, ktoken segment, 
	      kva_list *list)
{
   int status = TRUE;
   
   if (atr->base.type == KDAS_GENERIC_ATTRIBUTE)
      status = _kdas_generic_get(&(atr->generic), list);

   else if (atr->base.type == KDAS_QUASI_ATTRIBUTE)
      status = atr->quasi.get(object, segment, atr->quasi.attribute,
			      atr->quasi.client_data, list);
   else
      status = FALSE;

   if (!status)
      kset_errno(KDAS_EATTR_GET_FAILED);

   return status;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_attr_set
|
|       Purpose: Set an attribute.  This function can
|		 set both generic and quasi attributes.
|		 It takes an open variable argument list
|		 as the source of data to use in setting
|		 the attribute values.
|
|         Input: atr     - the attribute to set
|		 object  - object for quasi calls
|		 segment - tokenized segment for quasi calls
|		 va_list - a variable argument list containing 
|		           values of the attribute
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: March 17, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_attr_set(kdas_attribute *atr, kobject object, ktoken segment, 
	      kva_list *list)
{
   int status = TRUE;
   
   if (atr->base.type == KDAS_GENERIC_ATTRIBUTE)
      status = _kdas_generic_set(&(atr->generic), list);

   else if (atr->base.type == KDAS_QUASI_ATTRIBUTE)
      status = atr->quasi.set(object, segment, atr->quasi.attribute,
			      atr->quasi.client_data, list);
   else
      status = FALSE;

   if (!status)
      kset_errno(KDAS_EATTR_SET_FAILED);

   return status;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_attr_match
|
|       Purpose: Match an attribute.  This function can
|		 match both generic and quasi attributes.
|
|         Input: atr1 	 - the first attribute
|		 atr2 	 - the second attribute to compare against 
|		      	   the first one 
|		 object1 - object for quasi calls
|		 object2 - object for quasi calls
|		 segment - tokenized segment for quasi calls
|        Output: none
|
|       Returns: TRUE if attributes match, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: March 17, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_attr_match(kdas_attribute *atr1, kdas_attribute *atr2, 
		kobject object1, kobject object2, ktoken segment)
{
   int status = TRUE;
   
   if (atr1->base.type == KDAS_GENERIC_ATTRIBUTE)
      status = _kdas_generic_match(&(atr1->generic), &(atr2->generic));

   else if (atr1->base.type == KDAS_QUASI_ATTRIBUTE)
      status = atr1->quasi.match(object1, object2, segment, 
				 atr1->quasi.attribute,
				 atr1->quasi.client_data, 
				 atr2->quasi.client_data);

   return status;   /* FALSE is not an error */
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_attr_copy
|
|       Purpose: Copy an attribute.  This function can
|		 copy both generic and quasi attributes.
|
|         Input: atr1 	 - the first attribute
|		 atr2 	 - the second attribute in which to copy
|		      	   the first one 
|		 object1 - object for quasi calls
|		 object2 - object for quasi calls
|		 segment - tokenized segment for quasi calls
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: March 17, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_attr_copy(kdas_attribute *atr1, kdas_attribute *atr2, 
	       kobject object1, kobject object2, ktoken segment)
{
   int status = TRUE;
   
   if (atr1->base.type == KDAS_GENERIC_ATTRIBUTE)
      status = _kdas_generic_copy(&(atr1->generic), &(atr2->generic));

   else if (atr1->base.type == KDAS_QUASI_ATTRIBUTE)
      status = atr1->quasi.copy(object1, object2, segment,
				atr1->quasi.attribute,
				atr1->quasi.client_data,
				atr2->quasi.client_data);
   if (!status)
      kset_errno(KDAS_EATTR_COPY_FAILED);

   return status;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_attr_query
|
|       Purpose: Query the characteristics of an attribute.  
|		 This function can query both generic and 
|		 quasi attributes.
|
|         Input: atr       - the attribute to query
|		 object    - object for quasi calls
|		 segment   - tokenized segment for quasi calls
|
|        Output: numargs   - number of arguments
|		 argsize   - size of each argument
|		 datatype  - datatype of each argument
|		 permanent - whether the attribute is permanent or not
|
|       Returns: TRUE if attribute exists, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: March 17, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_attr_query(kdas_attribute *atr, kobject object, ktoken segment, 
		int *numargs, int *argsize, int *datatype, int *permanent)
{
   int status = TRUE;
   
   if (atr->base.type == KDAS_GENERIC_ATTRIBUTE)
      status = _kdas_generic_query(&(atr->generic), numargs, argsize,
				   datatype, permanent);

   else if (atr->base.type == KDAS_QUASI_ATTRIBUTE)
      status = atr->quasi.query(object, segment, atr->quasi.attribute, 
				atr->quasi.client_data,
				numargs, argsize, datatype, permanent);
   if (!status)
      kset_errno(KDAS_EATTR_QUERY_FAILED);

   return status;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_attr_print
|
|       Purpose: Print the value of an attribute.  
|		 This function can print both generic and 
|		 quasi attributes.
|
|         Input: atr     - the attribute to print
|		 object  - object for quasi calls
|		 segment - tokenized segment for quasi calls
|		 outfile - the file to print to
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: March 17, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_attr_print(kdas_attribute *atr, kobject object, ktoken segment, 
		kfile *outfile)
{
   int status = TRUE;
   
   if (atr->base.type == KDAS_GENERIC_ATTRIBUTE)
      status = _kdas_generic_print(&(atr->generic), outfile);

   else if (atr->base.type == KDAS_QUASI_ATTRIBUTE)
      status = atr->quasi.print(object, segment, atr->quasi.attribute, 
				atr->quasi.client_data, outfile);
   if (!status)
      kset_errno(KDAS_EATTR_PRINT_FAILED);

   return status;
}

/****    =============================================================
 ****    Attribute List Utility Routine 
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kdas_attribute_list_free
|
|       Purpose: Free an attribute contained in the node
|		 of a linked list structure.
|
|         Input: node - list node to free
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: February 29, 1996
| Modifications:
|
------------------------------------------------------------*/
void
kdas_attribute_list_free(klist *node)
{
   kdas_attribute *atr;
   
   if (node)
   {
      /*-- don't free the identifier, only the client_data --*/
      atr = (kdas_attribute *) node->client_data;
      kdas_attr_destruct(atr);
   }

   return;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_list_string_locate
|
|       Purpose: 
|
|         Input: 
|
|        Output: none
|
|       Returns: 
|
|    Written By: Steve Kubica
|          Date: March 13, 1996
| Modifications:
|
------------------------------------------------------------*/
klist *
kdas_list_string_locate(klist *list, char *attribute)
{
   klist *l;

   if (list == NULL)
      return NULL;
   
   for (l = list; l != NULL; l = klist_next(l))
      if (strcmp((char *) l->identifier, attribute) == 0)
	 return l;

   return NULL;
}


/*-----------------------------------------------------------
|
|  Routine Name: kdas_attr_setmode
|
|       Purpose: 
|
|         Input: 
|
|        Output: none
|
|       Returns: 
|
|    Written By: John M Salas & Jeremy Worley
|          Date: Nov 19, 1996
| Modifications:
|
------------------------------------------------------------*/
void
kdas_attr_setmode(int setmode)
{
    dupdee = setmode;
}
