/*
  * Khoros: $Id: buffer.c,v 1.4 1997/01/20 04:56:26 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: buffer.c,v 1.4 1997/01/20 04:56:26 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Data Services Buffer Management Code
   >>>>		
   >>>>  Private:
   >>>>                 kdas_adjust_buffer_size()
   >>>>                 kdas_free_all_buffers()
   >>>>   Static:
   >>>>                 _kdas_free_buffer()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/****    =============================================================
 ****    List of All Buffers Used By Data Services
 ****    =============================================================
 ****/

static klist *kdas_buffer_list = NULL;

/****    =============================================================
 ****    Buffer Size Management
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kdas_adjust_buffer_size
|
|       Purpose: This is a simple routine which checks out
|		 the size of a buffer and see if it will 
|		 hold the requested region size. If not
|		 then it reallocates the buffer up to 
|		 accomodate the new size.
|
|		 If NULL is passed in as the buffer, it
|		 is assumed that the buffer has not been
|		 allocated before, and it is allocated
|		 for you.  It is added to a list of all
|		 buffers which have been allocated for the
|		 session.  These can be removed with a call
|		 to kdas_free_all_buffers().
|
|         Input: buffer      - the buffer to adjust
|		 dimension   - dimensionality of what we want to store
|		 size        - size array of how much we want to store
|		 datatype    - datatype of what we want to store
|
|        Output: none
|
|       Returns: the reallocated buffer
|
|    Written By: Steve Kubica & Jeremy Worley 
|          Date: October 15, 1995
| Modifications:
|
------------------------------------------------------------*/
kdas_buffer *
kdas_adjust_buffer_size(
   kdas_buffer *buffer, 
   int  	dimension,
   int         *size,
   int  	datatype)
{
   kdas_buffer *rbuffer = NULL;
   int    overall_size;
   int    i;
   

   /*-- compute the overall size as a product of size and datatype --*/
   for (i = 0, overall_size = 1; i < dimension; i++)
      overall_size *= size[i];

   overall_size *= kdata_size(datatype);

   /*-- if buffer has never been allocated before, then add to the list --*/
   if (buffer == NULL)
   {
      rbuffer = (kdas_buffer *) kcalloc((size_t)1, sizeof(kdas_buffer));
      kdas_buffer_list = klist_add(kdas_buffer_list, rbuffer, NULL);
   }
   else  /*-- already on our list --*/
      rbuffer = buffer;

   /*-- increase the buffer size if we need to --*/
   if (overall_size > rbuffer->size)
   {
      rbuffer->data = krealloc(rbuffer->data, (size_t) overall_size);
      rbuffer->size = overall_size;
   }

   return rbuffer;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_free_all_buffers
|
|       Purpose: This routine will free all buffers
|		 which have ever been allocated in data
|		 services.
|
|         Input: none
|
|        Output: none
|
|       Returns: nothing
|
|    Written By: Steve Kubica
|          Date: January 7, 1996
| Modifications:
|
------------------------------------------------------------*/

/*-- static utility routine for freeing list element --*/
static void _kdas_free_buffer(klist *list)
{
   if (list && list->identifier)
   {
      kdas_buffer *buffer = (kdas_buffer *) list->identifier;
      kfree(buffer->data);
   }
   return;
}

void
kdas_free_all_buffers(void)
{
   klist_free(kdas_buffer_list, _kdas_free_buffer);
}
