 /*
  * Khoros: $Id: Raw.c,v 1.16.2.2 1997/03/11 17:16:12 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Raw.c,v 1.16.2.2 1997/03/11 17:16:12 mirage Exp $";

#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros RAW Data Service Routines
   >>>>		
   >>>>  Private:
   >>>>                 init_raw()
   >>>>   Static:
   >>>>                 raw_input()
   >>>>                 raw_output()
   >>>>                 raw_destroy()
   >>>>                 raw_prep()
   >>>>                 raw_locate_segment()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static int raw_input   	      PROTO((kobject, int, int));
static int raw_output  	      PROTO((kobject, int, int));
static int raw_destroy 	      PROTO((kobject));
static int raw_prep 	      PROTO((kobject));
static int raw_locate_segment PROTO((kobject, char *));

/*=====================================================================*/

/*-----------------------------------------------------------
|  Routine Name: init_raw
|       Purpose: initialize the raw file format
------------------------------------------------------------*/
void
init_raw(void)
{
   kdms_add_file_format("raw", 
			"Raw data format",
			NULL,       /*-- no check --*/
			raw_input, 
			raw_output, 
			raw_destroy, 
			raw_prep,
			raw_locate_segment, NULL, NULL, NULL, NULL);

   kdms_define_attribute(KDMS_OBJECT, KDMS_RAW_OFFSET, 1, 1, KINT, 
			 FALSE, TRUE, 0);
   return;
}


/*=====================================================================*/

/*
 *  Internal Resource Structure 
 *  raw  -  Basic Raw Data Format
 */
typedef struct
{
   int    fid;			/* monkey food I  */
   int    flags;		/* monkey food II */
}
ResourceStruct;

/****    =============================================================
 ****    Format Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: raw_input - Read a raw image
|
|       Purpose: This function is used to read in the image if the
|                supplied data is raw image.  The data was checked
|                by the raw_check() routine in which the raw
|                identifier indicated that the data is a valid raw.
|                Since the data was valid we can assume simply read
|                the header in and initialize that data segments.
|
|         Input: object - the raw object to be initialized
|
|        Output: Returns: returns TRUE or FALSE if an error occurs
|
|    Written By: Jeremy Worley
|          Date: Jul 13, 1992 15:50
| Modifications:
|
------------------------------------------------------------*/
static int
raw_input(kobject object, int fid, int flags)
{
   ResourceStruct *resources;

   if (!kdms_initialized(object))
   {
      if ((resources = (ResourceStruct *) kcalloc((size_t)1,
		sizeof(ResourceStruct))) == NULL)
      {
         return(FALSE);
      }
      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid   = fid;
   resources->flags = flags;

   kdms_create_segment(object, KDMS_SEGMENT_VALUE);
   kdms_set_attribute(object, KDMS_SEGMENT_VALUE, KDMS_DIMENSION, 5);
   
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: raw_output - Closes the raw image
|
|       Purpose: This function is used to close the raw image.  Which
|                means if the image was modified we then re-write image
|                back out.
|
|         Input: object - the raw object to be closed
|
|        Output: Returns: returns TRUE or FALSE if an error occurs
|
|    Written By: Jeremy Worley
|          Date: Jul 13, 1992 15:50
| Modifications:
|
------------------------------------------------------------*/
static int
raw_output(kobject object, int fid, int flags)
{
   ResourceStruct *resources;

   if (!kdms_initialized(object))
   {
      if ((resources = (ResourceStruct *) kcalloc((size_t)1,
		sizeof(ResourceStruct))) == NULL)
      {
         return(FALSE);
      }
      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid   = fid;
   resources->flags = flags;

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: raw_destroy - Frees an raw image
|
|       Purpose: This function is used to free the resources used
|		 by the raw format glue.
|
|         Input: object - object with resources to free
|        Output: none
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: July 13, 1992
| Modifications:
|
------------------------------------------------------------*/
static int
raw_destroy(kobject object)
{
   ResourceStruct *resources;

   resources = (ResourceStruct *) kdms_glue_get_resources(object);
   if (resources == NULL)
      return TRUE;

   kfree(resources);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: raw_prep - prepare an object for output to raw
|
|       Purpose: This function is used to prepare an object for
|		 output to a raw datafile.  It essentially will print 
|		 out information about what segments are getting dumped
|		 to the file.
|
|         Input: object - the object to be prepared
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: May 18, 1996
|
------------------------------------------------------------*/

static char *_kstring_append(char *str1, char *str2)
{
   int len1 = kstrlen(str1), len2 = kstrlen(str2);
   str1 = (char *) krealloc(str1, sizeof(char) * (len1+len2));
   return kstrcat(str1, str2);
}

/*ARGSUSED2*/
static int
raw_prep(kobject object)
{
   char *warn = NULL;

   /*-----------------------------------------------------------------------
   |  map, mask, location, and time data
   \-----------------------------------------------------------------------*/
   if (kdms_query_segment(object, KDMS_SEGMENT_MAP))
      warn = _kstring_append(warn, "- no support for storing map "
			    ": map segment ignored.\n");

   if (kdms_query_segment(object, KDMS_SEGMENT_MASK))
      warn = _kstring_append(warn, "- no support for storing mask "
			    ": mask segment ignored.\n");

   if (kdms_query_segment(object, KDMS_SEGMENT_LOCATION))
      warn = _kstring_append(warn, "- no support for storing location "
			    ": location segment ignored.\n");

   if (kdms_query_segment(object, KDMS_SEGMENT_TIME))
      warn = _kstring_append(warn, "- no support for storing time "
			    ": time segment ignored.\n");

   /*-- if we have any news to report, report it now --*/
   if (warn)
   {
      kinfo(KSTANDARD, "Raw format output :\n%s", warn);
      kfree_and_NULL(warn);
   }

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: raw_locate_segment - locate the segment
|
|       Purpose: 
|
|         Input: 
|        Output: none
|       Returns: returns the file fid fseeked to the beginning 
|		 of the segment
|
|    Written By: Jeremy Worley
|          Date: Jul 13, 1992 15:50
| Modifications:
|
------------------------------------------------------------*/
static int
raw_locate_segment(kobject object, char *segment)
{
   ResourceStruct *resources;
   int            offset;

   resources = (ResourceStruct *) kdms_glue_get_resources(object);

   /*-- only support value segment for writing --*/
   if ((resources->flags && KOBJ_WRITE) &&
       (kstrcmp(segment, KDMS_SEGMENT_VALUE) != 0))
      return -1;

   /*-- move the fid up by offset --*/
   kdms_get_attribute(object, NULL, KDMS_RAW_OFFSET, &offset);
   klseek(resources->fid, (off_t) offset, KSEEK_SET);

   return resources->fid;
}
