 /*
  * Khoros: $Id: internals.h,v 1.10 1997/01/20 05:03:08 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Internal Data Services Declaration
   >>>>
   >>>>    Written By: Mark Young, Jeremy Worley, Steve Kubica
   >>>>
   >>>>          Date: July 13, 1992
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _internals_h_
#define _internals_h_

 /*-------------------------------------*
 |       #include
 ---------------------------------------*/

#include <dataserv.h>


 /*-------------------------------------*
 |       #defines
 ---------------------------------------*/


 /*-------------------------------------*
 |       typedefs
 ---------------------------------------*/

/*
 * this is for sanity because nearly all dbx (not gdb) get totally
 * baffled and insist on resolving to the wrong kobject.
 */
typedef struct _kobject _data_object;


   /*----------------------------------------------------------------
   |   kdms_physical structure
   |   . . . . . . . . . . . . . . 
   |
   |   This structure represents the base, physical layer of a 
   |   data services object.   All object-level physical resources
   |   are kept here.  Note this does not include the physical 
   |   segments themselves, those are accessed only through the
   |   presentations at the presentation layer.  
   \-----------------------------------------------------------------*/
typedef struct _kdms_physical 
{
   char *filename;    /*-- the *real* filename of the file --*/

   int   fid;         /*-- open fid of the file, opened with below flags --*/
   int   flags;       /*-- flags used when opening the object --*/
   int   temporary;   /*-- TRUE if file is a temporary file */

   int   creating;    /*-- TRUE while the object is being opened --*/

   /*-- close hook : do this prior to closing the object --*/
   int (*close_hook) PROTO((kobject));

   /*-- format information for the format this object is using --*/
   kdas_format  *format;

   /*-- format specific resources --*/
   kaddr resources;

   /*-- user defined object level physical attributes --*/
   klist *attributes;

   /*-- callback list --*/
   klist *callbacks;

   /*-- reference object list --*/
   klist *references;

} kdms_physical;

   /*----------------------------------------------------------------
   |   kobject structure
   |   . . . . . . . . . . . . . . 
   |
   |   This is the internal structure of a data object.  It is only
   |   only used interally by data services, and is not intended for
   |   public consumption.   It represents the presentation layer of
   |   a data object.
   \-----------------------------------------------------------------*/
struct _kobject
{
   int type;  /*-- kobject identifier --*/

   kdms_physical *phys;

   /*-- user defined object level presentation attributes --*/
   klist *attributes;

   /*-- segment presentations --*/
   klist *presentations;
};


   /*----------------------------------------------------------------
   |   kcallback structure
   |   . . . . . . . . . . . . . . 
   |
   |   This structure is used to manage the information in a data 
   |   services callback.
   \-----------------------------------------------------------------*/
typedef struct _kcallback 
{
   kobject  object;
   ktoken   segment;
   ktoken   type;
   kaddr    clientData;

   union 
   {
      void (*data) PROTO((kobject, char *, kaddr, kdms_callback *));
      void (*object) PROTO((kobject, char *, kaddr));
   }
   callback;

} kcallback;


   /*----------------------------------------------------------------
   |   kdms_quasi_segment structure
   |   . . . . . . . . . . . . . . 
   |
   |   This is the structure for quasi segments, which simply contain
   |   function stubs to functions which can be used to "emulate"
   |   the behavior of an actual segment.
   \-----------------------------------------------------------------*/
typedef struct _kdms_quasi_segment {

   char *segment;   /*-- segment name --*/

   int lock;        /*-- lock down the definition if it is in use --*/

   int (*create)    PROTO((kobject, char *));
   int (*destroy)   PROTO((kobject, char *));
   int (*query)     PROTO((kobject, char *));

   kaddr (*get_data) PROTO((kobject, char *, int *, int *, kaddr));
   int   (*put_data) PROTO((kobject, char *, int *, int *, kaddr));

   int (*get_atr)   PROTO((kobject, char *, char *, kva_list *));
   int (*set_atr)   PROTO((kobject, char *, char *, kva_list *));
   int (*match_atr) PROTO((kobject, kobject, char *, char *));
   int (*copy_atr)  PROTO((kobject, kobject, char *, char *));
   int (*query_atr) PROTO((kobject, char *, char *,int *,int *,int *,int *));
   int (*print_atr) PROTO((kobject, char *, char *, kfile *));
   
} kdms_quasi_segment;


 /*-------------------------------------*
 |       global variable declarations
 ---------------------------------------*/

 /*-------------------------------------*
 |       macros
 ---------------------------------------*/

 /*-------------------------------------*
 |       routine definitions
 ---------------------------------------*/

/*-- Kdf.c --*/
void init_kdf PROTO((void));

/*-- Raw.c --*/
void init_raw PROTO((void));
 
/*-- attributes.c --*/
void kdms_free_attribute_defines      PROTO((void));
int  kdms_copy_attribute_list         PROTO((kobject, kobject, ktoken,
					     klist *, klist **));

/*-- callback.c --*/
void kdms_disable_callbacks 	      PROTO((void));
void kdms_enable_callbacks  	      PROTO((void));
int  kdms_process_data_callbacks      PROTO((kobject, kdas_presentation *, 
					     int *, int *, char *));
int  kdms_process_object_callbacks    PROTO((kobject, char *));
int  kdms_delete_callbacks_for_object PROTO((kobject, kdas_presentation *));

/*-- dms_attrib.c --*/
int  kdms_init_attributes             PROTO((void));

/*-- errors.c --*/
void kdms_init_errors                 PROTO((void));

/*-- glue.c --*/
kdas_format *kdms_identify_format     PROTO((int, int));
kdas_format *kdms_locate_format       PROTO((char *));
void         kdms_free_formats        PROTO((void));

/*-- object.c --*/
kobject kdms_object_construct 	      PROTO((int));
void    kdms_object_destruct  	      PROTO((kobject));
kobject kdms_object_open      	      PROTO((char *, int));
kobject kdms_object_reopen    	      PROTO((kobject, char *, int));
kobject kdms_object_reference 	      PROTO((kobject));
int     kdms_object_close     	      PROTO((kobject));
kobject kdms_object_locate    	      PROTO((char *));
int     kdms_object_update_references PROTO((kobject, char *));
void    kdms_close_all_objects        PROTO((kexit_status));

/*-- segment.c --*/
kdas_presentation *kdms_segment_create    PROTO((kobject, char *));
kdas_presentation *kdms_segment_reference PROTO((kdas_presentation *));
kdas_presentation *kdms_get_segment_info  PROTO((kobject, char *));
kdas_presentation *kdms_get_segtoken_info PROTO((kobject, ktoken));
int kdms_segment_sync       PROTO((kdas_presentation *, int));
int kdms_segment_copy_data  PROTO((kdas_presentation *, kdas_presentation *));
int kdms_segment_destruct   PROTO((kobject, char *));
int kdms_add_segment        PROTO((kobject, kdas_presentation *));
int kdms_destroy_segments   PROTO((kobject));
int kdms_reference_segments PROTO((kobject, kobject));
int kdms_sync_segments      PROTO((kobject, int));
int kdms_write_segments     PROTO((kobject));
int kdms_segment_copy_physical_attributes PROTO((kdas_presentation *, 
						 kdas_presentation *, int));

/*-- segue.c --*/
kdms_quasi_segment *kdms_locate_segment_define PROTO((char *));
void                kdms_free_segment_defines  PROTO((void));

/*-- utilities.c --*/
int kdms_legal_size     PROTO((int *, int));
int kdms_legal_datatype PROTO((int));
int kdms_legal_order    PROTO((int *, int));

#endif	/* _internals_h_ */

/* Don't add after this point */
