/*
  * Khoros: $Id: segue.c,v 1.10 1997/01/20 05:03:13 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: segue.c,v 1.10 1997/01/20 05:03:13 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Data Services Quasi-Segment Support
   >>>>		
   >>>>  Private:
   >>>>                 kdms_locate_segment_define()
   >>>>                 kdms_free_segment_defines()
   >>>>   Static:
   >>>>                 _kdms_quasi_segment_construct()
   >>>>                 _kdms_quasi_segment_destruct()
   >>>>                 _kdms_add_segment_define()
   >>>>                 _kdms_delete_segment_define()
   >>>>   Public:
   >>>>                 kdms_define_segment()
   >>>>                 kdms_undefine_segment()
   >>>>                 kdms_query_segment_definition()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static kdms_quasi_segment *_kdms_quasi_segment_construct PROTO((char   *,
   int 	 (*) PROTO((kobject, char *)),
   int 	 (*) PROTO((kobject, char *)),
   int 	 (*) PROTO((kobject, char *)),
   kaddr (*) PROTO((kobject, char *, int *, int *, kaddr)),
   int   (*) PROTO((kobject, char *, int *, int *, kaddr)),
   int   (*) PROTO((kobject, char *, char *, kva_list *)),
   int   (*) PROTO((kobject, char *, char *, kva_list *)),
   int   (*) PROTO((kobject, kobject, char *, char *)),
   int   (*) PROTO((kobject, kobject, char *, char *)),
   int   (*) PROTO((kobject, char *, char *,int *,int *,int *,int *)),
   int   (*) PROTO((kobject, char *, char *, kfile *)) ));

static int  _kdms_add_segment_define      PROTO((kdms_quasi_segment *));
static int  _kdms_delete_segment_define    PROTO((char *));

/****    =============================================================
 ****    Segment Definition List
 ****    =============================================================
 ****/

   /*----------------------------------------------------------------
   |   segment definition list
   |   . . . . . . . . . . . . . . 
   |
   |   This used to be a hash, but the hash had way too much overhead
   |   for the ten or so segments which ever get defined.
   \-----------------------------------------------------------------*/

kdms_quasi_segment **segment_defines;
int                  num_segment_defines;

/****    =============================================================
 ****    Public Routines
 ****    =============================================================
 ****/

/************************************************************
*
*  Routine Name: kdms_define_segment - define a segment
*
*       Purpose: This routine is used to define alternate segment
* 		 handler functions for a session.  A segment
* 		 definition is used to provide alternate functions for
* 		 creating, destroying, querying, and referencing,
* 		 segments.  Alternate functions can also be provided
* 		 for getting and putting data, as well as getting,
* 		 setting, matching, copying, querying, and printing
* 		 that segment's attributes.  Segments do not need
*		 to be defined.  This function just provides a
*		 convenient method for encapsulating alternate
*		 functionality for a segment name beneath the
*		 existing data services api.
*
*		 While in one of these alternate functions, the
*		 definition is "locked" so that further kdms calls 
*		 which access the defined segment will _not_ go 
*		 through the defined segment handler functions.  
*		 Once out of the handler function, the definition 
*		 will be "unlocked".
*
*	 	 This provides the ability to imbed any needed 
*		 initialization steps into the creation of a given
*		 segment.  For example, the create handler for the
*		 "value" segment will be called on a
*		 kdms_create_segment(obj, "value") call.  Within this
*		 handler, another kdms_create_segment(obj, "value")
*		 call will not call the create handler becuse the
*		 definition is locked, but instead will create the
*		 "value" segment, just as it would have done had the
*		 "value" segment not been defined.  Additional
*		 initialization steps, for example, setting the
*		 dimensionality of the "value" segment, can be easily
*		 added to the handler after the internal create call.
*
*		 If a handler function for a particular action is 
*		 not provided, then data services will behave as if
*		 the segment has never been defined.
*		 
*		 This segment definition can be undefined with the 
*		 kdms_undefine_segment call.
*
*         Input: segment    - string name of the segment being defined.
*
*   		 create         - create handler function for this segment.
*				  This function will be invoked whenever a
*				  kdms_create_segment function is called
*				  with the defined segment.
*
*       !The create handler declaration is of the form :
*       !
*       !   int create_handler(
*       !       kobject   object,
*       !       char     *segment)
*
*				  The object and segment are passed from
*				  the calling create function.  The return
*			    	  value of the handler will be propogated
*				  back up to the return value of the calling
*				  create function.
*
*   		 destroy        - destroy handler function for this segment.
*				  This function will be invoked whenever a
*				  kdms_destroy_segment function is called
*				  with the defined segment.
*
*       !The destroy handler declaration is of the form :
*       !
*       !   int destroy_handler(
*       !       kobject   object,
*       !       char     *segment)
*
*				  The object and segment are passed from
*				  the calling destroy function.  The return
*			    	  value of the handler will be propogated
*				  back up to the return value of the calling
*				  destroy function.
*
*   		 query          - query handler function for this segment.
*				  This function will be invoked whenever a
*				  kdms_query_segment function is called
*				  with the defined segment.
*
*       !The query handler declaration is of the form :
*       !
*       !   int query_handler(
*       !       kobject   object,
*       !       char     *segment)
*
*				  The object and segment are passed from
*				  the calling query function.  The return
*			    	  value of the handler will be propogated
*				  back up to the return value of the calling
*				  query function.
*
*   		 get_data       - get_data handler function for this segment.
*				  This function will be invoked whenever a
*				  kdms_get_data function is called
*				  with the defined segment.
*
*       !The get_data declaration is of the form :
*       !
*       !   kaddr get_data_handler(
*       !       kobject   object,
*       !       char     *segment,
*	!	int      *begin,
*	!	int      *end
*	!       int       kaddr)
*
*				  The object, segment, begin corner, end
*				  corner, and data pointer are passed from
*				  the calling get_data function.  The return
*			    	  data pointer from the handler will be 
*				  propogated back up to the return value 
*				  of the calling get_data function.
*
*   		 put_data       - put_data handler function for this segment.
*				  This function will be invoked whenever a
*				  kdms_put_data function is called
*				  with the defined segment.
*
*       !The put_data declaration is of the form :
*       !
*       !   int put_data_handler(
*       !       kobject   object,
*       !       char     *segment,
*	!	int      *begin,
*	!	int      *end
*	!       int       kaddr)
*
*				  The object, segment, begin corner, end
*				  corner, and data pointer are passed from
*				  the calling put_data function.  The return
*			    	  value from the handler will be propogated 
*			 	  back up to the return value of the calling
*				  put_data function.
*
*   		 get_atr        - get attribute handler function for this 
*				  segment. This function will be
*				  invoked whenever a kdms_get_attribute,
*				  kdms_get_attributes, kdms_vget_attribute,
*				  or kdms_vget_attributes function is
*				  called with the defined segment.
*
*       !The get attribute handler declaration is of the form :
*       !
*       !   int get_attribute_handler(
*       !       kobject   object,
*       !       char     *segment,
*       !       char     *attribute,
*       !       kva_list *list)
*
*                                 The object, segment, and attribute are 
*				  passed from the calling get function. The
*                                 variable argument list is an already
*                                 opened varargs list which contains the
*                                 pointers with which to return the 
*                                 attribute data.  The variable arguments
*                                 can be pulled off with the kva_arg()
*                                 function.  The list will be closed after
*                                 this handler returns.  The return value
*				  of this handler will also be propogated
*			          up to be the return value of the calling
*				  get function.
*
*		 set_atr        - set attribute handler function for this
*			          segment. This function will be
*			          invoked whenever a kdms_set_attribute,
*			          kdms_set_attributes, kdms_vset_attribute, 
*			          or kdms_vset_attributes function is
*			          called with the defined segment.
*
*	!The set attribute handler declaration is of the form :
*       !
*	!   int set_attribute_handler(
*	!       kobject   object,
*       !       char     *segment,
*       !       char     *attribute,
*       !       kva_list *list)
*
*                                 The object, segment, and attribute are 
*				  passed from the calling set function. The
*                                 variable argument list is an already
*                                 opened varargs list which contains the
*                                 data with which to set the attribute
*                                 data.  The variable arguments
*                                 can be pulled off with the kva_arg()
*                                 function.  The list will be closed after
*                                 this handler returns.  The return value
*                                 of this handler will also be propogated
*                                 up to be the return value of the calling
*                                 set function.
*
*		 match_atr      - match attribute handler function for this
*			          segment. This function will be invoked 
*				  whenever a kdms_match_attribute, 
*				  kdms_match_attributes, or 
*			          kdms_vmatch_attributes
*				  function is called with the defined segment.
*
*	!The match attribute handler declaration is of the form :
*       !
*	!   int match_attribute_handler(
*	!       kobject   object1,
*	!       kobject   object2,
*       !       char     *segment,
*       !       char     *attribute)
*
*                                 The object1, object2, segment, and
*				  attribute arguments are passed
*				  from the calling match function. 
*				  The return value of this
*			          handler will be propogated up to be
*			          the return value of the calling
*			          match function. 
*
*		 copy_atr       - copy attribute handler function for this
*			          segment. This function will be invoked 
*				  whenever a kdms_copy_attribute, 
*				  kdms_copy_attributes, or 
*			          kdms_vcopy_attributes
*				  function is called with the defined segment.
*
*	!The copy attribute handler declaration is of the form :
*       !
*	!   int copy_attribute_handler(
*	!       kobject   object1,
*	!       kobject   object2,
*       !       char     *segment,
*       !       char     *attribute)
*
*                                 The object1, object2, segment, and
*				  attribute arguments are passed
*				  from the calling copy function. 
*                                 copy function. The return value of this
*			          handler will be propogated up to be
*			          the return value of the calling
*			          copy function.  
*
*		 query_atr      - query attribute handler function for this
*			          segment.  This function will be invoked 
*				  whenever a kdms_query_attribute function is 
*				  called with the defined segment.
*
*	!The query attribute handler declaration is of the form :
*       !
*	!   int query_attribute_handler(
*	!       kobject   object,
*       !       char     *segment,
*       !       char     *attribute,
*       !       int      *num_args,
*       !       int      *arg_size,
*       !       int      *data_type,
*       !       int      *permanent)
*
*                                 The object, segment, and attribute are 
*			          passed in from the calling query
*			          function.  The association is the
*			          tokenized representation of the
*			          segment argument from the query
*			          function.  The num_args argument can
*			          be used to return the expected
*			          number of arguments for this
*			          attribute.  The arg_size argument
*			          can be used to return the expected
*			          argument size for this attribute.
*			          The data_type argument can be used
*			          to return the expected data type for
*			          this attribute.  The permanent
*			          argument can be used to indicate if
*			          this attribute is stored or not.
*			          Note that it is up to the programmer
*			          to ensure that the values which the
*			          handler returns for any of those
*			          arguments matches what is going to
*			          be processed in the get and set
*			          handlers.  The return value of this
*			          handler will be propogated up to be
*			          the return value of the calling
*			          query function.  In general, a value
*			          of TRUE is interpreted to mean
*			          that the attribute 'exists'.
*
*		 print_atr      - print attribute handler function for this 
*			          segment. This function will be invoked 
*				  whenever a kdms_print_attribute function is 
*				  called with the defined segment.
*
*	!The print attribute handler declaration is of the form :
*       !
*	!   int print_attribute_handler(
*	!       kobject   object,
*       !       char     *segment,
*       !       char     *attribute,
*       !       kfile    *outfile)
*
*                                 The object, segment, attribute, and
*                                 outfile is passed in from the
*                                 calling print function.  The return
*                                 value of this handler will be
*                                 propogated up to be the return value
*                                 of the calling print function.
*        Output: none
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*    Written By: Steve Kubica
*          Date: December 22, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kdms_define_segment(
*                !      char        *segment,
*   		 !      kfunc_int   *create,
*   		 !      kfunc_int   *destroy,
*   		 !      kfunc_int   *query,
*   		 !      kfunc_kaddr *get_data,
*   		 !      kfunc_int   *put_data,
*   		 !      kfunc_int   *get_atr,
*   		 !      kfunc_int   *set_atr,
*   		 !      kfunc_int   *match_atr_atrnn,
*   		 !      kfunc_int   *copy,_atr
*   		 !      kfunc_int   *query,
*		 !      kfunc_int   *print)
*
*************************************************************/
int
kdms_define_segment(
   char  *segment,
   int 	 (*create)    PROTO((kobject, char *)),
   int 	 (*destroy)   PROTO((kobject, char *)),
   int 	 (*query)     PROTO((kobject, char *)),
   kaddr (*get_data)  PROTO((kobject, char *, int *, int *, kaddr)),
   int   (*put_data)  PROTO((kobject, char *, int *, int *, kaddr)),
   int   (*get_atr)   PROTO((kobject, char *, char *, kva_list *)),
   int   (*set_atr)   PROTO((kobject, char *, char *, kva_list *)),
   int   (*match_atr) PROTO((kobject, kobject, char *, char *)),
   int   (*copy_atr)  PROTO((kobject, kobject, char *, char *)),
   int   (*query_atr) PROTO((kobject, char *, char *,int *,int *,int *,int *)),
   int   (*print_atr) PROTO((kobject, char *, char *, kfile *)) )
{
   kdms_quasi_segment *defin = NULL;
   kdms_quasi_segment *old_defin = NULL;

   KDMS_INIT;

   old_defin = kdms_locate_segment_define(segment);
   if (old_defin != NULL)
   {
      kinfo(KDEBUG, "kdms_define_segment: attempt to redefine the " 
	            "segment '%s'.", segment);
      return FALSE;
   }

   defin = _kdms_quasi_segment_construct(segment, create, destroy, query,
					 get_data, put_data, 
					 get_atr, set_atr, match_atr, 
					 copy_atr, query_atr, print_atr);
   
   return _kdms_add_segment_define(defin);
}

/************************************************************
*
*  Routine Name: kdms_undefine_segment - undefine a defined segment
*
*       Purpose: This function will remove a segment definition 
*		 from the segment definition list.  
*
*         Input: segment - the name of the segment to undefine
*
*        Output: none
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*    Written By: Steve Kubica
*          Date: December 22, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kdms_undefine_segment(
*                !      char *segment)
*
*************************************************************/
int
kdms_undefine_segment(
   char *segment)
{
   KDMS_INIT;

   return _kdms_delete_segment_define(segment);
}

/************************************************************
*
*  Routine Name: kdms_query_segment_definition - determines if a segment
*					         is defined.
*
*       Purpose: This function will check to see if a segment is
*		 defined on the segment definition list.
*
*         Input: segment - the name of the segment to query for
*
*        Output: none
*
*       Returns: TRUE if segment is defined, FALSE otherwise
*
*  Restrictions: 
*    Written By: Steve Kubica
*          Date: March 25, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kdms_is_defined_attribute(
*                !      char *segment)
*
*************************************************************/
int
kdms_query_segment_definition(char *segment)
{
   KDMS_INIT;

   if (kdms_locate_segment_define(segment) == NULL)
      return FALSE;
   else
      return TRUE;
}

/****    =============================================================
 ****    Segment Definition Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_quasi_segment_construct
|
|       Purpose: This routine will construct a segment definition
|                according to the given arguments.
|
|		 The segment definition is distinguished by its
|		 segment identifier, which is simply a tokenized
|		 version of the segment name.
|
|		 This segment definition is used to provide
|		 alternate functions for creating, destroying,
|	 	 querying, and referencing, segments.  Alternate
|		 functions can also be provided for getting and
|		 putting data, as well as getting, setting, matching,
|		 copying, querying, and printing that segment's
|		 attributes. 
|
|		 While in one of these alternate functions, the
|		 definition is "locked" so that further kdms calls 
|		 which need to access the segment will _not_ go 
|		 through your handler functions.  Once out of your
|		 handler routine, the definition will be "unlocked".
|
|	 	 This gives you the ability to imbed any needed 
|		 initialization steps into the creation of a given segment.  
|		 For example, the create handler for the "value" segment 
|		 will be called on a kdms_create_segment(obj, "value") call.  
|		 Within this handler, we can again call 
|		 kdms_create_segment("value"), which, because the 
|		 definition is locked, will not call our handler function, 
|		 but instead will create the "value" segment, just as it 
|		 would have done has we not defined it. Inside the create 
|		 handler, after the kdms_create_segment call, we can 
|		 then set the proper dimensionality and index order.
|
|		 If a handler function for a particular action is 
|		 not provided, then data services will simply fall back 
|		 on the action's "default" behavior.
|
|		 The data service functions and their corresponding
|		 handler functions which you can provide are :
|
|		 !   create    - kdms_create_segment
|		 !   destroy   - kdms_destroy_segment
|		 !   query     - kdms_query_segment
|		 !   get_data  - kdms_get_data
|		 !   put_data  - kdms_put_data
|		 !   get       - kdms_set_attribute(s)
|		 !   set       - kdms_set_attribute(s)
|		 !   match     - kdms_match_attribute(s)
|		 !   copy      - kdms_copy_attribute(s)
|		 !   query     - kdms_query_attribute
|		 !   print     - kdms_print_attribute
|
|         Input: segment     - segment identifier 
|		 create()    - create function
|		 destroy()   - destroy function
|		 query()     - query function
|		 get_data()  - get data function
|		 put_data()  - put data function
|		 get_atr()   - attribute get function
|  		 set_atr()   - attribute set function
|		 match_atr() - attribute match function
| 		 copy_atr()  - attribute copy function
|		 query_atr() - attribute query function
|		 print_atr() - attribute print function
|
|        Output: none
|
|       Returns: a pointer to the constructed quasi segment  
|		 on success, NULL otherwise
|
|    Written By: Steve Kubica
|          Date: December 22, 1994
| Modifications:
|
------------------------------------------------------------*/
static kdms_quasi_segment *
_kdms_quasi_segment_construct(
   char   *segment,
   int 	 (*create)    PROTO((kobject, char *)),
   int 	 (*destroy)   PROTO((kobject, char *)),
   int 	 (*query)     PROTO((kobject, char *)),
   kaddr (*get_data)  PROTO((kobject, char *, int *, int *, kaddr)),
   int   (*put_data)  PROTO((kobject, char *, int *, int *, kaddr)),
   int   (*get_atr)   PROTO((kobject, char *, char *, kva_list *)),
   int   (*set_atr)   PROTO((kobject, char *, char *, kva_list *)),
   int   (*match_atr) PROTO((kobject, kobject, char *, char *)),
   int   (*copy_atr)  PROTO((kobject, kobject, char *, char *)),
   int   (*query_atr) PROTO((kobject, char *, char *,int *,int *,int *,int *)),
   int   (*print_atr) PROTO((kobject, char *, char *, kfile *)) )
{
   kdms_quasi_segment *defin;

   /*-- allocate a new segment definition structure --*/
   if ((defin = (kdms_quasi_segment *) 
	kmalloc(sizeof(kdms_quasi_segment))) == NULL)
   {
      kdas_set_error(KMEMORY_ALLOCATION);
      return NULL;
   }

   /*-- fill out the segment definition structure --*/
   defin->segment   = kstrdup(segment);
   defin->lock      = FALSE;

   defin->create    = create;
   defin->destroy   = destroy;
   defin->query     = query;
   defin->get_data  = get_data;
   defin->put_data  = put_data;
   defin->get_atr   = get_atr;
   defin->set_atr   = set_atr;
   defin->match_atr = match_atr;
   defin->copy_atr  = copy_atr;
   defin->query_atr = query_atr;
   defin->print_atr = print_atr;

   return defin;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_quasi_segment_destruct
|
|       Purpose: Destroy a segment definition.  
|
|         Input: defin - the segment definition to free
|
|        Output: none
|
|       Returns: none
|
|    Written By: Steve Kubica
|          Date: Dec 22, 1994 10:47
| Modifications:
|
------------------------------------------------------------*/
static void
_kdms_quasi_segment_destruct(kdms_quasi_segment *defin)
{
   /*-- trivial free --*/
   if (defin == NULL)
      return;

   /*-- free the segment name --*/
   kfree(defin->segment);

   /*-- free the segment definition --*/
   kfree(defin);

   return;
}

/****    =============================================================
 ****    Segment Definition List Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_add_segment_define
|
|       Purpose: Given a quasi segment definition, add
|		 it to the list of segment defines.
|
|         Input: define - quasi segment definition to add
|			  to the list.	
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: December 22, 1994
| Modifications:
|
------------------------------------------------------------*/
static int _kdms_add_segment_define(kdms_quasi_segment *define)
{
   segment_defines = (kdms_quasi_segment **) krealloc((kaddr) segment_defines, 
		     sizeof(kdms_quasi_segment *) * (num_segment_defines + 1));

   if (segment_defines == NULL)
      return FALSE;

   segment_defines[num_segment_defines] = define;
   num_segment_defines++;
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_delete_segment_define
|
|       Purpose: Given a segment name, delete its definition.
|
|         Input: segment - name of segment whose definition 
|			   we want to delete.
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: December 22, 1994
| Modifications:
|
------------------------------------------------------------*/
static int _kdms_delete_segment_define(char *segment)
{
   int i, j;
   
   if (segment == NULL)
      return FALSE;

   for (i = 0; i < num_segment_defines; i++)
   {
      if (strcmp(segment, segment_defines[i]->segment) == 0) 
      {
	 /*-- delete it --*/
	 _kdms_quasi_segment_destruct(segment_defines[i]);
	 
	 /*-- shrink down the list --*/
	 num_segment_defines--;
	 for (j = i; j < num_segment_defines; j++)
	    segment_defines[j] = segment_defines[j+1];
	 segment_defines = (kdms_quasi_segment **) 
	         krealloc((kaddr) segment_defines, 
			  sizeof(kdms_quasi_segment *) * num_segment_defines);
	 return TRUE;
      }
   }

   return FALSE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdms_locate_segment_define
|
|       Purpose: Given a segment identifier, locate the 
|                segment in the segment definition list 
|		 and return it.  It will not return a 
|		 segment if it has been locked.
|
|         Input: segment - identifier of the segment to search for
|
|        Output: none
|
|       Returns: the segment definition if it is on the list, 
|	  	 NULL otherwise
|
|    Written By: Steve Kubica
|          Date: December 22, 1994
| Modifications:
|
------------------------------------------------------------*/
kdms_quasi_segment *
kdms_locate_segment_define(
   char *segment)
{
   int i;

   if (segment == NULL)
      return NULL;

   for (i = 0; i < num_segment_defines; i++)
      if (strcmp(segment, segment_defines[i]->segment) == 0 &&
	  segment_defines[i]->lock == FALSE) 
	 return segment_defines[i];

   return NULL;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdms_free_segment_defines
|
|       Purpose: Free up the static segment definition list.
|		 All the definitions are freed and then the list
|		 itself is freed.
|
|         Input: none
|
|        Output: none
|
|       Returns: nothing
|
|    Written By: Steve Kubica
|          Date: March 21, 1994
| Modifications:
|
------------------------------------------------------------*/
void
kdms_free_segment_defines(void)
{
   int i;

   for (i = 0; i < num_segment_defines; i++)
      _kdms_quasi_segment_destruct(segment_defines[i]);
   
   kfree(segment_defines);

   return;
}
