/*
  * Khoros: $Id: utility.c,v 1.8 1997/01/20 05:03:15 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utility.c,v 1.8 1997/01/20 05:03:15 mirage Exp $";

#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Data Services Utilities
   >>>>		
   >>>>  Private:
   >>>>                 kdms_legal_size()
   >>>>                 kdms_legal_datatype()
   >>>>                 kdms_legal_order()
   >>>>                 _kdms_get_order()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/****    =============================================================
 ****    Utility Routines
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kdms_legal_size 
|
|       Purpose: This routine determines if an array of size
|		 dimension has been initialized.  If the
|		 array has not been allocated or has no
|		 dimensionality, then it is assumed to 
|		 be uninitialized.  If the array is allocated,
|		 but contains the value '-1', then it is
|		 considered to be uninitialized.
|
|         Input: array     - array to check
|		 dimension - dimensionality of the array 
|
|        Output: none
|
|       Returns: TRUE (1) if initialized, FALSE (0) otherwise
|
|    Written By: Steve Kubica
|          Date: October 16, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdms_legal_size(int *size, int dimension)
{
   int i;
   
   /*-- check for obvious non-initialization --*/
   if (size == NULL || dimension <= 0)
      return FALSE;
   
   for (i = 0; i < dimension; i++)
      if (size[i] == -1)
	 return FALSE;
   
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdms_legal_datatype 
|
|       Purpose: This routine determines if a datatype is
|		 legal or has been defined.
|
|         Input: datatype  - datatype to check
|
|        Output: none
|
|       Returns: TRUE (1) if legal, FALSE (0) otherwise
|
|    Written By: Jeremy Worley
|          Date: May 20, 1993
| Modifications:
|
------------------------------------------------------------*/
int
kdms_legal_datatype(int datatype)
{
   switch (datatype)
   {
      case KBIT:
      case KBYTE:
      case KUBYTE:
      case KSHORT:
      case KUSHORT:
      case KLONG:
      case KULONG:
      case KINT:
      case KUINT:
      case KFLOAT:
      case KDOUBLE:
      case KCOMPLEX:
      case KDCOMPLEX:
      case KSTRUCT:
	 return TRUE;
	 
      default:
	 return FALSE;
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: kdms_legal_order
|
|       Purpose: Determines if the index order of the specified
|                segment is legal.
|
|         Input: order     - order to check
|                dimension - dimensionality of the index order
|
|        Output: none
|
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Jeremy Worley
|          Date: May 20, 1993
| Modifications:
|
------------------------------------------------------------*/
int
kdms_legal_order(int *order, int dimension)
{
   int i, j;

   /*----------------------------------------------------------------------
   |  check all combinations to make sure that there is no way that two of
   |  the index orders are the same.
   \----------------------------------------------------------------------*/
   for (i = 0; i < dimension - 1; i++)
      for (j = i + 1; j < dimension; j++)
         if ((order[i] == order[j] && order[i] != KNONE) || order[i] == -1)
            return FALSE;     /* not an error */
	 
   if (dimension == 1 && order[i] == -1)
      return FALSE;
	 
   /*-- they be legal --*/
   return TRUE;
}



int
_kdms_get_order(int direction, int *cur_order, int *info, int dim, int def)
{
   int i;

   for (i = 0; i < dim; i++)
      if (direction == cur_order[i])
         return (info[i]);

   return (def);
}

