/*
 * Khoros: $Id: internals.h,v 1.1.2.1 1997/03/18 23:18:53 mirage Exp $
 */
 
/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose:	Private include file for library `geom'.
   >>>>			This is (should be) included by all source
   >>>>			files comprising the library.  This file includes
   >>>>			the public include file for the library:
   >>>>				<kgeom/kgeom.h>
   >>>>
   >>>>       Toolbox: $GLASS
   >>>>
   >>>>    Written By: Steve Kubica
   >>>>
   >>>>          Date: December 24, 1996
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#ifndef _kgeom_internals_h_
#define _kgeom_internals_h_



	/*-------------------------------------*
	|	#includes
	\--------------------------------------*/

#include <dataserv.h>

	/*-------------------------------------*
	|	#defines
	\--------------------------------------*/

/*
 *    ================================================================== 
 *    reading and writing check macro
 *    ==================================================================
 */

#define CK(exprn) s &= (exprn > 0) 

#define LOCS    kgeom_number_locs(primitive, object)
#define COLS    kgeom_number_cols(primitive, object)
#define NORMS   kgeom_number_norms(primitive, object)
#define TCOORDS kgeom_number_tcoords(primitive, object)
#define RADII   kgeom_number_radii(primitive, object)

/*
 *    ================================================================== 
 *    writing macros for primitive data
 *    ==================================================================
 */

#define WRITE_FIELD(FIELD, NUMBER)     \
   has = (p->FIELD != NULL);         \
   CK(kwrite_ubyte(fid, &has, (size_t) 1));   \
   if (has)                          \
      CK(kwrite_float(fid, p->FIELD, (size_t) NUMBER));

#define WRITE_LOCS    WRITE_FIELD(locs,    LOCS)
#define WRITE_COLS    WRITE_FIELD(cols,    COLS)
#define WRITE_NORMS   WRITE_FIELD(norms,   NORMS)
#define WRITE_TCOORDS WRITE_FIELD(tcoords, TCOORDS)
#define WRITE_RADII   WRITE_FIELD(radii,   RADII)

/*-- more obscure data fields used by some of the primitives --*/
#define WRITE_BGCOLS  WRITE_FIELD(bgcols,  COLS)
#define WRITE_OFFS    WRITE_FIELD(offsets, LOCS)
#define WRITE_UPS     WRITE_FIELD(ups,     LOCS)
#define WRITE_FLOWS   WRITE_FIELD(flows,   LOCS)
#define WRITE_RAD1    WRITE_FIELD(rad1,    RADII)
#define WRITE_RAD2    WRITE_FIELD(rad2,    RADII)

/*
 *    ================================================================== 
 *    reading macros for primitive data
 *    ==================================================================
 */

#define READ_FIELD(FIELD, NUMBER)  \
   CK(kread_ubyte(fid, &has, (size_t) 1));  \
   if (has)                        \
   {                               \
      int n = NUMBER;              \
                                   \
      if ((p->FIELD = kgeom_alloc((int) n)) == NULL) \
	 return FALSE;                         \
     \
      CK(kread_float(fid, p->FIELD, (size_t) n)); \
   } \
   else p->FIELD = NULL;

#define READ_LOCS    READ_FIELD(locs,  	 LOCS)
#define READ_COLS    READ_FIELD(cols,  	 COLS)
#define READ_NORMS   READ_FIELD(norms, 	 NORMS)
#define READ_TCOORDS READ_FIELD(tcoords, TCOORDS)
#define READ_RADII   READ_FIELD(radii,   RADII)

/*-- more obscure data fields used by some of the primitives --*/
#define READ_BGCOLS  READ_FIELD(bgcols,  COLS)
#define READ_OFFS    READ_FIELD(offsets, LOCS)
#define READ_UPS     READ_FIELD(ups,     LOCS)
#define READ_FLOWS   READ_FIELD(flows,   LOCS)
#define READ_RAD1    READ_FIELD(rad1,    RADII)
#define READ_RAD2    READ_FIELD(rad2,    RADII)

/*
 *    ================================================================== 
 *    copying macros for primitive data
 *    ==================================================================
 */

#define COPY_FIELD(FIELD, NUMBER) \
   np->FIELD = kgeom_dup(op->FIELD, (int) NUMBER); \
   if (np->FIELD == NULL && op->FIELD != NULL) \
      return FALSE;

#define COPY_LOCS    COPY_FIELD(locs,  	 LOCS)
#define COPY_COLS    COPY_FIELD(cols,  	 COLS)
#define COPY_NORMS   COPY_FIELD(norms, 	 NORMS)
#define COPY_TCOORDS COPY_FIELD(tcoords, TCOORDS)
#define COPY_RADII   COPY_FIELD(radii,   RADII)

/*-- more obscure data fields used by some of the primitives --*/
#define COPY_BGCOLS  COPY_FIELD(bgcols,  COLS)
#define COPY_OFFS    COPY_FIELD(offsets, LOCS)
#define COPY_UPS     COPY_FIELD(ups,     LOCS)
#define COPY_FLOWS   COPY_FIELD(flows,   LOCS)
#define COPY_RAD1    COPY_FIELD(rad1,    RADII)
#define COPY_RAD2    COPY_FIELD(rad2,    RADII)

	/*-------------------------------------*
	|	typedefs
	\--------------------------------------*/


	/*-------------------------------------*
	|	global variable declarations
	\--------------------------------------*/


	/*-------------------------------------*
	|	macros
	\--------------------------------------*/


	/*-------------------------------------*
	|	routine definitions
	\--------------------------------------*/

/*-- cast.c --*/
void kgeom_new_primitive_location_dim PROTO((kgeom_object *,kgeom_primitive *,
					     int));
void kgeom_new_primitive_layout       PROTO((kgeom_object *,kgeom_primitive *,
					     int));
void kgeom_new_primitive_has_alpha    PROTO((kgeom_object *,kgeom_primitive *,
					     int));

/*-- primitive.c --*/
int primitive_function_index PROTO((int));

#endif /* _kgeom_internals_h_ */
/* Don't add after this point */
