 /*
  * Khoros: $Id: octmesh.c,v 1.1.2.1 1997/03/18 23:18:59 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: octmesh.c,v 1.1.2.1 1997/03/18 23:18:59 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		   Geometry Services Object Code
   >>>>
   >>>>   Public:	
   >>>>			kgeom_write_octmesh_header()
   >>>>			kgeom_write_octmesh_plane()
   >>>>
   >>>>			kgeom_read_octmesh_header()
   >>>>			kgeom_read_octmesh_plane()
   >>>>  Private:
   >>>>   Static:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static kubyte version = 1;

/*-----------------------------------------------------------
|
|  Routine Name: kgeom_write_octmesh_header - start writing an octmesh
|
|       Purpose: This routine is used for incrementally writing out
|		 an octmesh a plane at a time rather than all at once.
|		 
|		 This routine writes out the header information from
|		 the octmesh primitive, after which each plane of the 
|		 octmesh data can be written out using the function 
|		 kgeom_write_octmesh_plane(). 
|
|		 It is kept semi-private for now, since it is not clear
|		 if this is the best way to handle this problem.
|	
|         Input: object   - geometry object to write header for
|                filename - the filename to write header to
|
|        Output: none
|
|       Returns: TRUE on success, FALSE on failure
|
|    Written By: Steve Kubica
|          Date: December 18, 1996
| Modifications:
|
------------------------------------------------------------*/
int
kgeom_write_octmesh_header(
   kgeom_object  *object,
   int   width,
   int   height,
   int   depth,
   char *texture,
   int   has_locs,
   int   has_cols,
   int   has_norms,
   int   has_tcoords)
{
   kubyte has;
   int fid = object->fid;
   int s = TRUE;
   int type = KGEOM_OCTMESH;
   
   CK(kwrite_int   (fid, &type, (size_t) 1));
   CK(kwrite_ubyte (fid, &version, (size_t) 1));

   CK(kwrite_int   (fid, &width, (size_t) 1));
   CK(kwrite_int   (fid, &height, (size_t) 1));
   CK(kwrite_int   (fid, &depth, (size_t) 1));

   /*-- write the texture name if we have one --*/
   has = (texture != NULL);
   CK(kwrite_ubyte(fid, &has, (size_t) 1));
   if (has)
      CK(kwrite_string(fid, &texture, (size_t) 1));

   has = (kubyte) has_locs;     CK(kwrite_ubyte(fid, &has, (size_t) 1));
   has = (kubyte) has_cols;     CK(kwrite_ubyte(fid, &has, (size_t) 1));
   has = (kubyte) has_norms;    CK(kwrite_ubyte(fid, &has, (size_t) 1));
   has = (kubyte) has_tcoords;  CK(kwrite_ubyte(fid, &has, (size_t) 1));

   return s;
}

/*-----------------------------------------------------------
|
|  Routine Name: kgeom_read_octmesh_header - start reading an octmesh
|
|       Purpose: This routine is used for incrementally reading in
|		 an octmesh a plane at a time rather than all at once.
|		 
|		 This routine reads in the header information from
|		 the octmesh primitive; each plane of the octmesh data 
|		 can then be read in using the function 
|		 kgeom_read_octmesh_plane(). 
|
|		 It is kept semi-private for now, since it is not clear
|		 if this is the best way to handle this problem.
|	
|         Input: object   - geometry object to write header for
|                filename - the filename to write header to
|
|        Output: none
|
|       Returns: TRUE on success, FALSE on failure
|
|    Written By: Steve Kubica
|          Date: December 18, 1996
| Modifications:
|
------------------------------------------------------------*/
int
kgeom_read_octmesh_header(
   kgeom_object  *object,
   int    *width,
   int    *height,
   int    *depth,
   char  **texture,
   int    *has_locs,
   int    *has_cols,
   int    *has_norms,
   int    *has_tcoords)
{
   kubyte has, v;
   int fid = object->fid;
   int s = TRUE;
   char *texture_name;
   
   /*-- read the version (unused) --*/
   CK(kread_ubyte (fid, &v, (size_t) 1));
   CK(kread_int   (fid, width, (size_t) 1));
   CK(kread_int   (fid, height, (size_t) 1));
   CK(kread_int   (fid, depth, (size_t) 1));

   /*-- read the texture name if we have one --*/
   CK(kread_ubyte(fid, &has, (size_t) 1));
   if (has)
      CK(kread_string(fid, &texture_name, (size_t) 1));
   else 
      texture_name = NULL;
   
   if (texture) *texture = texture_name;

   CK(kread_ubyte(fid, &has, (size_t) 1));   
   if (has_locs) *has_locs = (int) has;

   CK(kread_ubyte(fid, &has, (size_t) 1));
   if (has_cols) *has_cols = (int) has;

   CK(kread_ubyte(fid, &has, (size_t) 1));
   if (has_norms) *has_norms = (int) has;

   CK(kread_ubyte(fid, &has, (size_t) 1));
   if (has_tcoords) *has_tcoords = (int) has;

   return s;
}

/*-----------------------------------------------------------
|
|  Routine Name: kgeom_write_octmesh_plane - write one plane of an octmesh
|
|       Purpose: This routine is used for incrementally writing out
|		 an octmesh a plane at a time rather than all at once.
|		 
|		 This routine writes out a single plane of octmesh data
|		 which consists of location, colors, normals and texture
|		 coordinates.  All planes of an octmesh must contain
|		 the same components, ie if the first plane contains
|		 color data, then all planes must contain color data.
|
|		 It is kept semi-private for now, since it is not clear
|		 if this is the best way to handle this problem.
|	
|         Input: object   - geometry object to write header for
|                filename - the filename to write header to
|
|        Output: none
|
|       Returns: TRUE on success, FALSE on failure
|
|    Written By: Steve Kubica
|          Date: December 18, 1996
| Modifications:
|
------------------------------------------------------------*/
int
kgeom_write_octmesh_plane(
   kgeom_object  *object,
   int            width,
   int            height,
   float         *locs,
   kubyte        *cols,
   float         *norms,
   float         *tcoords)
{
   int fid = object->fid;
   int s = TRUE;
   size_t plane_size = width * height;
   size_t l, c, t;

   l = plane_size * object->location_dim;
   c = plane_size * ((object->has_alpha) ? 4 : 3);
   t = plane_size * object->texture_coord_dim;

   if (locs != NULL)    CK(kwrite_float(fid, locs, l));
   if (cols != NULL)    CK(kwrite_ubyte(fid, cols, c));
   if (norms != NULL)   CK(kwrite_float(fid, norms, l));
   if (tcoords != NULL) CK(kwrite_float(fid, tcoords, t));

   return s;
}

/*-----------------------------------------------------------
|
|  Routine Name: kgeom_read_octmesh_plane - read one plane of an octmesh
|
|       Purpose: This routine is used for incrementally reading in
|		 an octmesh a plane at a time rather than all at once.
|		 
|		 This routine reads in a single plane of octmesh data
|		 which consists of location, colors, normals and texture
|		 coordinates.
|
|		 It is kept semi-private for now, since it is not clear
|		 if this is the best way to handle this problem.
|	
|         Input: object   - geometry object to write header for
|                filename - the filename to write header to
|
|        Output: none
|
|       Returns: TRUE on success, FALSE on failure
|
|    Written By: Steve Kubica
|          Date: December 18, 1996
| Modifications:
|
------------------------------------------------------------*/
int
kgeom_read_octmesh_plane(
   kgeom_object  *object,
   int            width,
   int            height,
   float         *locs,
   kubyte        *cols,
   float         *norms,
   float         *tcoords)
{
   int fid = object->fid;
   int s = TRUE;
   size_t plane_size = width * height;
   size_t l, c, t;

   l = plane_size * object->location_dim;
   c = plane_size * ((object->has_alpha) ? 4 : 3);
   t = plane_size * object->texture_coord_dim;

   if (locs != NULL)    CK(kread_float(fid, locs, l));
   if (cols != NULL)    CK(kread_ubyte(fid, cols, c));
   if (norms != NULL)   CK(kread_float(fid, norms, l));
   if (tcoords != NULL) CK(kread_float(fid, tcoords, t));

   return s;
}
