 /*
  * Khoros: $Id: prim_copy.c,v 1.1.2.3 1997/03/18 23:19:06 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: prim_copy.c,v 1.1.2.3 1997/03/18 23:19:06 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Geometry Services Primitive Code
   >>>>
   >>>>   Public:	
   >>>>			kgeom_copy_primitive()
   >>>>  Private:
   >>>>   Static:
   >>>>			_copy_polyline()
   >>>>			_copy_triangles()
   >>>>			_copy_spheres()
   >>>>			_copy_text()
   >>>>			_copy_cylinders()
   >>>>			_copy_directed_points()
   >>>>			_copy_markers()
   >>>>			_copy_polygon()
   >>>>			_copy_rectangles()
   >>>>			_copy_ellipses()
   >>>>			_copy_mesh()
   >>>>			_copy_texture()
   >>>>			_copy_view_xform()
   >>>>			_copy_object_xform()
   >>>>			_copy_camera_orientation()
   >>>>			_copy_vr_event()
   >>>>			_copy_object()
   >>>>			_copy_invalid()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static int _copy_polyline   	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_triangles  	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_spheres    	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_text       	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_cylinders  	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_directed_points     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_markers  	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_polygon  	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_rectangles 	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_ellipses 	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_mesh       	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_texture    	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_view_xform   	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_object_xform 	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_camera_orientation  PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_vr_event 	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_object     	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));
static int _copy_invalid    	     PROTO((kgeom_object *,kgeom_primitive *,
					    kgeom_primitive *));

static int (*copy_function[]) PROTO((kgeom_object *, kgeom_primitive *,
				     kgeom_primitive *)) =
{
   _copy_invalid,
   _copy_polyline,
   _copy_triangles,
   _copy_spheres,
   _copy_text,
   _copy_cylinders,
   _copy_directed_points,
   _copy_markers,
   _copy_polygon,
   _copy_rectangles,
   _copy_ellipses,
   _copy_mesh,
   _copy_texture,
   _copy_view_xform,
   _copy_object_xform,
   _copy_camera_orientation,
   _copy_vr_event,
   _copy_object
};

/************************************************************
*
*  Routine Name: kgeom_copy_primitive - copy a geometry primitive
*
*       Purpose: This routine will copy a given primitive and all
*		 associated data into another primitive structure.  If
*		 another primitive structure is not provided for the
*		 destination of the copy, a new one will be
*		 constructed.
*	
*		 The object presentation is used to determine how much
*		 data is present in the primitive, so make certain the
*		 has_alpha, layout, location_dim, and other fields
*		 correctly reflect the data in the primitive.  The
*		 copied primitive will not be added to the object, so
*		 you are free to destroy it or add it to another
*		 object.
*       
*         Input: object - object to which original primitive belongs
*		 primitive - primitive to copy
*        Output: new_primitive - a pointer to geometry primitive that will
*			         serve as a destination for the copy. If
*			      	 NULL, then a new destination primitive will
*			      	 be allocated.
*       Returns: copy of primitive
*
*  Restrictions: 
*
*    Written By: Steve Kubica
*          Date: December 18, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
kgeom_primitive *
kgeom_copy_primitive(
   kgeom_object    *object, 
   kgeom_primitive *primitive,
   kgeom_primitive *new_primitive)
{
   if (object == NULL)
   {
      kset_errno(KNULL_PARAMETER);
      kinfo(KVERBOSE, 
	    "An object is needed in order to determine how much data is "
	     "present in the primitive.");
      return NULL;
   }

   if (primitive == NULL)
      return NULL;

   if (new_primitive == NULL)
      new_primitive = kgeom_new_primitive(primitive->type);
   else
      new_primitive->type = primitive->type;
   
   copy_function[primitive_function_index(primitive->type)](object, primitive, 
							    new_primitive);
   return new_primitive;
}

/*
 *    ================================================================== 
 *    primitive functions
 *    ==================================================================
 */

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_polyline
------------------------------------------------------------*/
static int _copy_polyline(kgeom_object *object, 
			  kgeom_primitive *primitive, 
			  kgeom_primitive *new_primitive)
{
   kgeom_polyline *op = (kgeom_polyline *) primitive;
   kgeom_polyline *np = (kgeom_polyline *) new_primitive;

   np->nverts    = op->nverts;
   np->linetype  = op->linetype;
   np->linewidth = op->linewidth;

   COPY_LOCS;  COPY_COLS;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_triangles
------------------------------------------------------------*/
static int _copy_triangles(kgeom_object *object, 
			   kgeom_primitive *primitive, 
			   kgeom_primitive *new_primitive)
{
   kgeom_triangles *op = (kgeom_triangles *) primitive;
   kgeom_triangles *np = (kgeom_triangles *) new_primitive;

   np->nverts  = op->nverts;
   np->texture = kstrdup(op->texture);

   COPY_LOCS;  COPY_COLS;  COPY_NORMS;  COPY_TCOORDS;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_spheres
------------------------------------------------------------*/
static int _copy_spheres(kgeom_object *object, 
			 kgeom_primitive *primitive, 
			 kgeom_primitive *new_primitive)
{
   kgeom_spheres *op = (kgeom_spheres *) primitive;
   kgeom_spheres *np = (kgeom_spheres *) new_primitive;

   np->nverts = op->nverts;

   COPY_LOCS;  COPY_COLS;  COPY_RADII; 

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_text
------------------------------------------------------------*/
static int _copy_text(kgeom_object *object, 
		      kgeom_primitive *primitive, 
		      kgeom_primitive *new_primitive)
{
   kgeom_text *op = (kgeom_text *) primitive;
   kgeom_text *np = (kgeom_text *) new_primitive;

   np->nverts  = op->nverts;
   np->fonts   = (char **) karray_copy(op->fonts, KSTRING, (size_t) op->nverts, TRUE);
   np->strings = (char **) karray_copy(op->strings, KSTRING, (size_t) op->nverts, TRUE);

   COPY_LOCS; COPY_OFFS; COPY_UPS; COPY_FLOWS; COPY_COLS; COPY_BGCOLS;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_cylinders
------------------------------------------------------------*/
static int _copy_cylinders(kgeom_object *object, 
			   kgeom_primitive *primitive, 
			   kgeom_primitive *new_primitive)
{
   kgeom_cylinders *op = (kgeom_cylinders *) primitive;
   kgeom_cylinders *np = (kgeom_cylinders *) new_primitive;

   np->nverts  = op->nverts;

   COPY_LOCS; COPY_COLS; COPY_RAD1; COPY_RAD2;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_directed_pointes
------------------------------------------------------------*/
static int _copy_directed_points(kgeom_object *object, 
				 kgeom_primitive *primitive, 
				 kgeom_primitive *new_primitive)
{
   kgeom_directed_points *op = (kgeom_directed_points *) primitive;
   kgeom_directed_points *np = (kgeom_directed_points *) new_primitive;

   np->nverts  = op->nverts;

   COPY_LOCS; COPY_COLS; COPY_NORMS;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_markers
------------------------------------------------------------*/
static int _copy_markers(kgeom_object *object, 
			 kgeom_primitive *primitive, 
			 kgeom_primitive *new_primitive)
{
   kgeom_markers *op = (kgeom_markers *) primitive;
   kgeom_markers *np = (kgeom_markers *) new_primitive;

   np->nverts     = op->nverts;
   np->markertype = op->markertype;

   COPY_LOCS; COPY_COLS; COPY_BGCOLS;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_polygon
------------------------------------------------------------*/
static int _copy_polygon(kgeom_object *object, 
			 kgeom_primitive *primitive, 
			 kgeom_primitive *new_primitive)
{
   kgeom_polygon *op = (kgeom_polygon *) primitive;
   kgeom_polygon *np = (kgeom_polygon *) new_primitive;

   np->nverts    = op->nverts;
   np->linetype  = op->linetype;
   np->linewidth = op->linewidth;

   COPY_LOCS;  COPY_COLS; 

   /*-- strange single bg-color case --*/
   np->bgcol = (float *) kdupalloc(op->bgcol, sizeof(float)*((object->has_alpha)? 4 : 3));
   if (np->bgcol == NULL && op->bgcol != NULL)
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_rectangles
------------------------------------------------------------*/
static int _copy_rectangles(kgeom_object *object, 
			    kgeom_primitive *primitive, 
			    kgeom_primitive *new_primitive)
{
   kgeom_rectangles *op = (kgeom_rectangles *) primitive;
   kgeom_rectangles *np = (kgeom_rectangles *) new_primitive;

   np->nverts    = op->nverts;
   np->linetype  = op->linetype;
   np->linewidth = op->linewidth;

   COPY_LOCS;  COPY_COLS;  COPY_BGCOLS;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_ellipses
------------------------------------------------------------*/
static int _copy_ellipses(kgeom_object *object, 
			  kgeom_primitive *primitive, 
			  kgeom_primitive *new_primitive)
{
   kgeom_ellipses *op = (kgeom_ellipses *) primitive;
   kgeom_ellipses *np = (kgeom_ellipses *) new_primitive;

   np->nverts    = op->nverts;
   np->linetype  = op->linetype;
   np->linewidth = op->linewidth;

   COPY_LOCS;  COPY_COLS;  COPY_BGCOLS;  COPY_RAD1;  COPY_RAD2;
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_mesh
------------------------------------------------------------*/
static int _copy_mesh(kgeom_object *object, 
		      kgeom_primitive *primitive, 
		      kgeom_primitive *new_primitive)
{
   if (primitive->type == KGEOM_QUADMESH)
   {
      kgeom_quadmesh *op = (kgeom_quadmesh *) primitive;
      kgeom_quadmesh *np = (kgeom_quadmesh *) new_primitive;

      np->width   = op->width;
      np->height  = op->height;
      np->texture = kstrdup(op->texture);

      COPY_LOCS;    COPY_COLS;    COPY_NORMS;  COPY_TCOORDS;
   }
   else /*-- KGEOM_OCTMESH --*/
   {
      kgeom_octmesh *op = (kgeom_octmesh *) primitive;
      kgeom_octmesh *np = (kgeom_octmesh *) new_primitive;

      np->width   = op->width;
      np->height  = op->height;
      np->depth   = op->depth;
      np->texture = kstrdup(op->texture);

      COPY_LOCS;  /* COPY_COLS; */  

      /*-- can't use COPY_COLS since this is a ubyte field --*/
      np->cols = (unsigned char *) kdupalloc(op->cols, 
			   sizeof(kubyte)*np->width*np->height*np->depth);
      if (np->cols == NULL && op->cols != NULL)
	 return FALSE;
      
      COPY_NORMS;  COPY_TCOORDS;
   }
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_texture
------------------------------------------------------------*/
static int _copy_texture(kgeom_object *object, 
			 kgeom_primitive *primitive, 
			 kgeom_primitive *new_primitive)
{
   if (primitive->type == KGEOM_TEXTURE2D)
   {
      kgeom_texture2D *op = (kgeom_texture2D *) primitive;
      kgeom_texture2D *np = (kgeom_texture2D *) new_primitive;

      np->width  = op->width;
      np->height = op->height;

      /*-- can't use COPY_COLS since this is a ubyte field --*/
      np->cols = (unsigned char *) kdupalloc(op->cols, sizeof(kubyte) * np->width * np->height);
      if (np->cols == NULL && op->cols != NULL)
	 return FALSE;
   }
   else /*-- KGEOM_TEXTURE3D --*/
   {
      kgeom_texture3D *op = (kgeom_texture3D *) primitive;
      kgeom_texture3D *np = (kgeom_texture3D *) new_primitive;

      np->width  = op->width;
      np->height = op->height;
      np->depth  = op->depth;

      /*-- can't use COPY_COLS since this is a ubyte field --*/
      np->cols = (unsigned char *) kdupalloc(op->cols, sizeof(kubyte) * 
			   np->width * np->height * np->depth);
      if (np->cols == NULL && op->cols != NULL)
	 return FALSE;
   }
   
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_view_xform
------------------------------------------------------------*/
static int _copy_view_xform(kgeom_object *object, 
			    kgeom_primitive *primitive,
			    kgeom_primitive *new_primitive)
{
   kmemcpy((kaddr)new_primitive, (kaddr)primitive, sizeof(kgeom_object_xform));
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_object_xform
------------------------------------------------------------*/
static int _copy_object_xform(kgeom_object *object, 
			      kgeom_primitive *primitive,
			      kgeom_primitive *new_primitive)
{
   kmemcpy((kaddr)new_primitive, (kaddr)primitive, sizeof(kgeom_object_xform));
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_camera_orientation
------------------------------------------------------------*/
static int _copy_camera_orientation(kgeom_object *object, 
				    kgeom_primitive *primitive,
				    kgeom_primitive *new_primitive)
{
   kmemcpy((kaddr)new_primitive, (kaddr)primitive, 
	   sizeof(kgeom_camera_orientation));
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_vr_event
------------------------------------------------------------*/
static int _copy_vr_event(kgeom_object *object, 
			  kgeom_primitive *primitive,
			  kgeom_primitive *new_primitive)
{
   kmemcpy((kaddr)new_primitive, (kaddr)primitive, sizeof(kgeom_vr_event));
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_object
------------------------------------------------------------*/
static int _copy_object(kgeom_object *object, 
			kgeom_primitive *primitive,
			kgeom_primitive *new_primitive)
{
   kgeom_subobject *obj     = (kgeom_subobject *) primitive;
   kgeom_subobject *new_obj = (kgeom_subobject *) new_primitive;

   new_obj->object = (kaddr) kgeom_copy_object((kgeom_object *) obj->object, 
				           (kgeom_object *) new_obj->object);
   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _copy_invalid
------------------------------------------------------------*/
static int _copy_invalid(kgeom_object *object, 
			 kgeom_primitive *primitive,
			 kgeom_primitive *new_primitive)
{
   kerror("kgeom", "kgeom_copy_primitive", "invalid primitive type");
   return FALSE;
}

