 /*
  * Khoros: $Id: prim_read.c,v 1.1.2.3 1997/03/18 23:19:10 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: prim_read.c,v 1.1.2.3 1997/03/18 23:19:10 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Geometry Services Primitive Code
   >>>>
   >>>>   Public:	
   >>>>			kgeom_read_primitive()
   >>>>  Private:
   >>>>   Static:
   >>>>			_read_polyline()
   >>>>			_read_triangles()
   >>>>			_read_spheres()
   >>>>			_read_text()
   >>>>			_read_cylinders()
   >>>>			_read_directed_points()
   >>>>			_read_markers()
   >>>>			_read_polygon()
   >>>>			_read_rectangles()
   >>>>			_read_ellipses()
   >>>>			_read_mesh()
   >>>>			_read_texture()
   >>>>			_read_view_xform()
   >>>>			_read_object_xform()
   >>>>			_read_camera_orientation()
   >>>>			_read_vr_event()
   >>>>			_read_object()
   >>>>			_read_invalid()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static int _read_polyline   	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_triangles  	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_spheres    	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_text       	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_cylinders  	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_directed_points     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_markers  	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_polygon  	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_rectangles 	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_ellipses 	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_mesh       	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_texture    	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_view_xform   	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_object_xform 	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_camera_orientation  PROTO((kgeom_object *,kgeom_primitive *));
static int _read_vr_event 	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_object     	     PROTO((kgeom_object *,kgeom_primitive *));
static int _read_invalid    	     PROTO((kgeom_object *,kgeom_primitive *));

static int (*read_function[]) PROTO((kgeom_object *, kgeom_primitive *)) =
{
   _read_invalid,
   _read_polyline,
   _read_triangles,
   _read_spheres,
   _read_text,
   _read_cylinders,
   _read_directed_points,
   _read_markers,
   _read_polygon,
   _read_rectangles,
   _read_ellipses,
   _read_mesh,
   _read_texture,
   _read_view_xform,
   _read_object_xform,
   _read_camera_orientation,
   _read_vr_event,
   _read_object
};

/************************************************************
*
*  Routine Name: kgeom_read_primitive - read a geometry primitive
*
*       Purpose: This routine will read a given primitive and all
*		 associated data from the file indicated by the provided
*		 object.  A new primitive will be constructed for
*		 this.
*	
*		 In general, the \f(CWkgeom_write_object()\fP function
*		 should be used instead of this function. This
*		 function has been made public to provide complete
*		 flexibility for reading a geometry object.  It should
*		 only be used when you wish to read a geometry object
*		 primitive-by-primitive.
*
*		 If you wish to use this function to read in a
*		 primitive at a time, you should follow this sequence
*		 to reading the object:
*
*  !		\f(CWobject = kgeom_read_header("filename");\fP
*
*		 If this has returned a valid object, then the object
*		 itself can be read.  The next call will read in all
*		 the specific information
*		 
*  !		\f(CWkgeom_start_reading_object(object);\fP
*
*		 At this point, you can start reading in individual
*		 primitives using the \f(CWkgeom_read_primitive()\fP
*		 function.  The file to be read from is determined
*		 from the open fid inside the object. This function
*		 will return NULL if there are no more primitives to
*		 read.
*		 
*		 After all primitives have been read, finish with the
*		 following calls :
*
*  !		\f(CWkgeom_finish_reading_object(object);\fP
* 
*  !		\f(CWkgeom_done_reading(object);\fP
*
*		 The object presentation is used to determine how much
*		 data is present in the primitive.  The call to
*		 \f(CWkgeom_start_reading_object()\fP should have
*		 initialized these fields appropriately. The primitive
*		 will not be added to the object, so you are free to
*		 destroy it or add it to a different object if you
*		 choose.
*       
*         Input: object - object to which primitive will belong 
*        Output: none
*       Returns: new primitive with primitive data from file
*
*  Restrictions: 
*
*    Written By: Steve Kubica
*          Date: December 18, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
kgeom_primitive *
kgeom_read_primitive(kgeom_object *object) 
{
   kgeom_primitive *primitive = NULL;
   int              type;
   
   if (object == NULL || object->fid == -1)
   {
      kerror("kgeom", "kgeom_read_primitive", 
	     "NULL object or object with unopen file");
      return FALSE;
   }

   /*-- get the type and construct a new primitive of that type --*/ 
   kread_int(object->fid, &type, (size_t) 1);

   if (type == -1)
      return NULL;

   if ((primitive = kgeom_new_primitive(type)) == NULL)
      return NULL;

   if (!read_function[primitive_function_index(primitive->type)](object, 
								 primitive))
   {
      kgeom_blast_primitive(primitive);
      return NULL;
   }

   return primitive;
}

/*
 *    ================================================================== 
 *    primitive functions
 *    ==================================================================
 */

/*-----------------------------------------------------------
|  Routine Name: (static) _read_polyline
------------------------------------------------------------*/
static int _read_polyline(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_polyline *p = (kgeom_polyline *) primitive;
   kubyte has, v;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte(fid, &v, (size_t) 1));
   CK(kread_int  (fid, &p->nverts, (size_t) 1));
   CK(kread_int  (fid, &p->linetype, (size_t) 1));
   CK(kread_int  (fid, &p->linewidth, (size_t) 1));

   READ_LOCS;  READ_COLS;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_triangles
------------------------------------------------------------*/
static int _read_triangles(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_triangles *p = (kgeom_triangles *) primitive;
   kubyte has, v;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte (fid, &v, (size_t) 1));
   CK(kread_int   (fid, &p->nverts, (size_t) 1));

   CK(kread_ubyte(fid, &has, (size_t) 1));
   if (has)
      CK(kread_string(fid, &p->texture, (size_t) 1));
   else 
      p->texture = NULL;

   READ_LOCS;  READ_COLS;  READ_NORMS;  READ_TCOORDS;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_spheres
------------------------------------------------------------*/
static int _read_spheres(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_spheres *p = (kgeom_spheres *) primitive;
   kubyte has, v;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte (fid, &v, (size_t) 1));
   CK(kread_int   (fid, &p->nverts, (size_t) 1));

   READ_LOCS;  READ_COLS;  READ_RADII;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_text
------------------------------------------------------------*/
static int _read_text(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_text *p = (kgeom_text *) primitive;
   kubyte has, v;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte (fid, &v, (size_t) 1));
   CK(kread_int   (fid, &p->nverts, (size_t) 1));

   CK(kread_ubyte(fid, &has, (size_t) 1));
   if (has)
   {
      if ((p->strings = (kstring *) kcalloc((size_t) p->nverts, 
					    sizeof(kstring))) == NULL)
	 return FALSE;
      CK(kread_string(fid, p->strings, (size_t) p->nverts));
   }
   else p->strings = NULL;

   CK(kread_ubyte(fid, &has, (size_t) 1));
   if (has)
   {
      if ((p->fonts = (kstring *) kcalloc((size_t) p->nverts, 
					  sizeof(kstring))) == NULL)
	 return FALSE;
      CK(kread_string(fid, p->fonts, (size_t) p->nverts));
   }
   else p->fonts = NULL;

   READ_LOCS; READ_OFFS; READ_UPS; READ_FLOWS; READ_COLS; READ_BGCOLS;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_cylinders
------------------------------------------------------------*/
static int _read_cylinders(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_cylinders *p = (kgeom_cylinders *) primitive;
   kubyte has, v;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte (fid, &v, (size_t) 1));
   CK(kread_int   (fid, &p->nverts, (size_t) 1));

   READ_LOCS; READ_COLS; READ_RAD1; READ_RAD2;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_directed_points
------------------------------------------------------------*/
static int _read_directed_points(kgeom_object *object, 
				 kgeom_primitive *primitive)
{
   kgeom_directed_points *p = (kgeom_directed_points *) primitive;
   kubyte has, v;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte (fid, &v, (size_t) 1));
   CK(kread_int   (fid, &p->nverts, (size_t) 1));

   READ_LOCS;  READ_COLS;  READ_NORMS;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_markers
------------------------------------------------------------*/
static int _read_markers(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_markers *p = (kgeom_markers *) primitive;
   kubyte has, v;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte (fid, &v, (size_t) 1));
   CK(kread_int   (fid, &p->nverts, (size_t) 1));
   CK(kread_int   (fid, &p->markertype, (size_t) 1));

   READ_LOCS;  READ_COLS;  READ_BGCOLS;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_polygon
------------------------------------------------------------*/
static int _read_polygon(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_polygon *p = (kgeom_polygon *) primitive;
   kubyte has, v;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte (fid, &v, (size_t) 1));
   CK(kread_int   (fid, &p->nverts, (size_t) 1));
   CK(kread_int   (fid, &p->linetype, (size_t) 1));
   CK(kread_int   (fid, &p->linewidth, (size_t) 1));

   READ_LOCS;  READ_COLS;  

   /*-- strange single bg-color case --*/
   CK(kread_ubyte(fid, &has, (size_t) 1));
   if (has)
   {
      if ((p->bgcol = (float *) kcalloc((size_t) ((object->has_alpha)? 4 : 3),
					sizeof(float))) == NULL)
	 return FALSE;
      CK(kread_float(fid, p->bgcol, (size_t) ((object->has_alpha)? 4 : 3)));
   }
   else p->bgcol = NULL;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_rectangles
------------------------------------------------------------*/
static int _read_rectangles(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_rectangles *p = (kgeom_rectangles *) primitive;
   kubyte has, v;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte (fid, &v, (size_t) 1));
   CK(kread_int   (fid, &p->nverts, (size_t) 1));
   CK(kread_int   (fid, &p->linetype, (size_t) 1));
   CK(kread_int   (fid, &p->linewidth, (size_t) 1));

   READ_LOCS;  READ_COLS;  READ_BGCOLS;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_ellipses
------------------------------------------------------------*/
static int _read_ellipses(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_ellipses *p = (kgeom_ellipses *) primitive;
   kubyte has, v;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte (fid, &v, (size_t) 1));
   CK(kread_int   (fid, &p->nverts, (size_t) 1));
   CK(kread_int   (fid, &p->linetype, (size_t) 1));
   CK(kread_int   (fid, &p->linewidth, (size_t) 1));

   READ_LOCS;  READ_COLS;  READ_BGCOLS;  READ_RAD1;  READ_RAD2;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_mesh
------------------------------------------------------------*/
static int _read_mesh(kgeom_object *object, kgeom_primitive *primitive)
{
   kubyte has, v;
   int fid = object->fid;
   int s = TRUE;

   if (primitive->type == KGEOM_QUADMESH)
   {
      kgeom_quadmesh *p = (kgeom_quadmesh *) primitive;

      /*-- read the version (unused) --*/
      CK(kread_ubyte (fid, &v, (size_t) 1));
      CK(kread_int   (fid, &p->width, (size_t) 1));
      CK(kread_int   (fid, &p->height, (size_t) 1));

      /*-- read the texture name if we have one --*/
      CK(kread_ubyte(fid, &has, (size_t) 1));
      if (has)
	 CK(kread_string(fid, &p->texture, (size_t) 1));
      else 
	 p->texture = NULL;

       READ_LOCS;  READ_COLS;  READ_NORMS;  READ_TCOORDS;
   }
   else /*-- KGEOM_OCTMESH --*/
   {
      kgeom_octmesh *p = (kgeom_octmesh *) primitive;
      float  *locs, *norms, *tcoords;
      kubyte *cols;
      int     has_locs, has_cols, has_norms, has_tcoords;
      size_t  plane_size, l, c, t;
      int     i;

      CK(kgeom_read_octmesh_header(object, &p->width, &p->height, &p->depth,
				   &p->texture, &has_locs, &has_cols, 
				   &has_norms, &has_tcoords));

      plane_size = (size_t) p->width * p->height;

      l = plane_size * object->location_dim;
      c = plane_size * ((object->has_alpha) ? 4 : 3);
      t = plane_size * object->texture_coord_dim;

      p->locs    = (has_locs)    ? (float *) kmalloc(l*p->depth*sizeof(float)) : NULL;
      p->cols    = (has_cols)    ? (kubyte *)kmalloc(c*p->depth*sizeof(kubyte)): NULL;
      p->norms   = (has_norms)   ? (float *) kmalloc(l*p->depth*sizeof(float)) : NULL;
      p->tcoords = (has_tcoords) ? (float *) kmalloc(t*p->depth*sizeof(float)) : NULL;
      
      locs = p->locs; cols = p->cols; norms = p->norms; tcoords = p->tcoords;

      for (i = 0; i < p->depth; i++)
      {
	 CK(kgeom_read_octmesh_plane(object, p->width, p->height,
				     locs, cols, norms, tcoords));
	 if (has_locs)    locs    += l;
	 if (has_cols)    cols    += c;
	 if (has_norms)   norms   += l;
	 if (has_tcoords) tcoords += t;
      }
   }

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_texture
------------------------------------------------------------*/
static int _read_texture(kgeom_object *object, kgeom_primitive *primitive)
{
   kubyte has, v;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte (fid, &v, (size_t) 1));

   if (primitive->type == KGEOM_TEXTURE2D)
   {
      kgeom_texture2D *p = (kgeom_texture2D *) primitive;

      CK(kread_int   (fid, &p->width, (size_t) 1));
      CK(kread_int   (fid, &p->height, (size_t) 1));

      /*-- can't use READ_COLS since this is a ubyte field --*/
      CK(kread_ubyte(fid, &has, (size_t) 1));
      if (has)
      {
         int n = p->width*p->height * ((object->has_alpha) ? 4 : 3);
         if ((p->cols = (unsigned char *) kmalloc((size_t) n * sizeof(kubyte))) == NULL)
            return FALSE;
         CK(kread_ubyte(fid, p->cols, (size_t) n));
      }
      else p->cols = NULL;
   }
   else /*-- KGEOM_TEXTURE3D --*/
   {
      kgeom_texture3D *p = (kgeom_texture3D *) primitive;

      CK(kread_int   (fid, &p->width, (size_t) 1));
      CK(kread_int   (fid, &p->height, (size_t) 1));
      CK(kread_int   (fid, &p->depth, (size_t) 1));

      /*-- can't use READ_COLS since this is a ubyte field --*/
      CK(kread_ubyte(fid, &has, (size_t)1));
      if (has)
      {
         int n = p->width*p->height*p->depth * ((object->has_alpha) ? 4 : 3);
         if ((p->cols = (unsigned char *) kmalloc((size_t) n * sizeof(kubyte))) == NULL)
            return FALSE;                         \
         CK(kread_ubyte(fid, p->cols, (size_t) n));
      }
      else p->cols = NULL;
   }

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_view_xform
------------------------------------------------------------*/
static int _read_view_xform(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_view_xform *p = (kgeom_view_xform *) primitive;
   kubyte v;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte(fid, &v, (size_t) 1));
   CK(kread_float(fid,  p->matrix, (size_t) 16));

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_object_xform
------------------------------------------------------------*/
static int _read_object_xform(kgeom_object *object, 
			      kgeom_primitive *primitive)
{
   kgeom_object_xform *p = (kgeom_object_xform *) primitive;
   kubyte v, has;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte (fid, &v, (size_t) 1));
   CK(kread_float (fid,  p->matrix, (size_t) 16));
   CK(kread_float (fid,  p->translate, (size_t) 3));
   CK(kread_int   (fid, &p->relative, (size_t) 1));

   CK(kread_ubyte(fid, &has, (size_t)1));
   if (has)
      CK(kread_string(fid, &p->name, (size_t) 1));
   else 
      p->name = NULL;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_camera_orientation
------------------------------------------------------------*/
static int _read_camera_orientation(kgeom_object *object, 
				    kgeom_primitive *primitive)
{
   kgeom_camera_orientation *p = (kgeom_camera_orientation *) primitive;
   kubyte v;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte (fid, &v, (size_t) 1));
   CK(kread_float (fid,  p->eye, (size_t) 3));
   CK(kread_float (fid,  p->at, (size_t) 3));
   CK(kread_float (fid,  p->up, (size_t) 3));
   CK(kread_float (fid, &p->field_of_view, (size_t) 1));
   CK(kread_float (fid, &p->hither, (size_t) 1));
   CK(kread_float (fid, &p->yon, (size_t) 1));
   CK(kread_int   (fid, &p->projection, (size_t) 1));
   CK(kread_int   (fid, &p->is_stereo, (size_t) 1));
   CK(kread_float (fid, &p->eye_distance, (size_t) 1));
   CK(kread_float (fid, &p->focal_length, (size_t) 1));

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_vr_event
------------------------------------------------------------*/
static int _read_vr_event(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_vr_event *p = (kgeom_vr_event *) primitive;
   kubyte v;
   int fid = object->fid;
   int s = TRUE;

   /*-- read the version (unused) --*/
   CK(kread_ubyte(fid, &v, (size_t) 1));

   CK(kread_int  (fid, &p->have_button_events, (size_t) 3));
   CK(kread_int  (fid, &p->have_rotations, (size_t) 3));
   CK(kread_int  (fid, &p->have_translations, (size_t) 3));

   CK(kread_float(fid,  p->rotate, (size_t) 3));
   CK(kread_float(fid,  p->translate, (size_t) 3));
   CK(kread_int  (fid,  p->buttons, (size_t) 9));
   CK(kread_int  (fid, &p->clock, (size_t) 1));

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_object
------------------------------------------------------------*/
static int _read_object(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_subobject *p = (kgeom_subobject *) primitive;
   
   if ((p->object = (kaddr) kgeom_new_object()) == NULL)
      return FALSE;

   if (!kgeom_read_object_and_primitives((kgeom_object *) p->object))
   {
      kgeom_blast_object((kgeom_object *)p->object);
      return FALSE;
   }

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _read_invalid
------------------------------------------------------------*/
static int _read_invalid(kgeom_object *object, kgeom_primitive *primitive)
{
   kerror("kgeom", "kgeom_read_primitive", "invalid primitive type");
   return FALSE;
}
