 /*
  * Khoros: $Id: prim_write.c,v 1.1.2.1 1997/03/18 23:19:14 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: prim_write.c,v 1.1.2.1 1997/03/18 23:19:14 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Geometry Services Primitive Writing Code
   >>>>
   >>>>   Public:	
   >>>>			kgeom_write_primitive()
   >>>>  Private:
   >>>>   Static:
   >>>>			_write_polyline()
   >>>>			_write_triangles()
   >>>>			_write_spheres()
   >>>>			_write_text()
   >>>>			_write_cylinders()
   >>>>			_write_directed_points()
   >>>>			_write_markers()
   >>>>			_write_polygon()
   >>>>			_write_rectangles()
   >>>>			_write_ellipses()
   >>>>			_write_mesh()
   >>>>			_write_texture()
   >>>>			_write_view_xform()
   >>>>			_write_object_xform()
   >>>>			_write_camera_orientation()
   >>>>			_write_vr_event()
   >>>>			_write_object()
   >>>>			_write_invalid()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static kubyte version = 1;

static int _write_polyline  	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_triangles 	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_spheres   	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_text      	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_cylinders 	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_directed_points    PROTO((kgeom_object *,kgeom_primitive *));
static int _write_markers 	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_polygon 	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_rectangles 	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_ellipses 	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_mesh      	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_texture   	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_view_xform   	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_object_xform	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_camera_orientation PROTO((kgeom_object *,kgeom_primitive *));
static int _write_vr_event	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_object    	     PROTO((kgeom_object *,kgeom_primitive *));
static int _write_invalid   	     PROTO((kgeom_object *,kgeom_primitive *));

static int (*write_function[]) PROTO((kgeom_object *, kgeom_primitive *)) =
{
   _write_invalid,
   _write_polyline,
   _write_triangles,
   _write_spheres,
   _write_text,
   _write_cylinders,
   _write_directed_points,
   _write_markers,
   _write_polygon,
   _write_rectangles,
   _write_ellipses,
   _write_mesh,
   _write_texture,
   _write_view_xform,
   _write_object_xform,
   _write_camera_orientation,
   _write_vr_event,
   _write_object
};

/************************************************************
*
*  Routine Name: kgeom_write_primitive - write a geometry primitive
*
*       Purpose: This routine will write a given primitive and all
*		 associated data to the file indicated by the provided
*		 object.
*
*		 In general, the \f(CWkgeom_write_object()\fP function
*		 should be used instead of this function. This
*		 function has been made public to provide complete
*		 flexibility for writing a geometry object.  It should
*		 only be used when you wish to write a geometry object
*		 primitive-by-primitive.  
*
*		 If you wish to use this function to write out a primitive
*		 at a time, you should follow this sequence to write out
*		 the the object:
*
*  !		\f(CWkgeom_write_header(object, "filename");\fP
*		 
*  !		\f(CWkgeom_start_writing_object(object);\fP
*
*		 At this point, you can start writing out individual
*		 primitives using the \f(CWkgeom_write_primitive()\fP
*		 function.  The file to be written to is determined
*		 from the open fid inside the object.
*
*		 When you are done writing primitives, finish with the
*		 following calls :
*
*  !		\f(CWkgeom_finish_writing_object(object);\fP
* 
*  !		\f(CWkgeom_done_writing(object);\fP
*	
*		 The object presentation is used to determine how much
*		 data is present in the primitive, so make sure the
*		 has_alpha, layout, location_dim, and other fields are
*		 set correctly.  These should be valid before the call
*		 to \f(CWkgeom_start_writing_object() \fP is made.
*		 The primitive does not have to be present on the
*		 object's primitive list to be written.
*       
*         Input: object - object to which primitive belongs 
*		 primitive - primitive to write
*        Output: none
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*
*    Written By: Steve Kubica
*          Date: December 1, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kgeom_write_primitive(
   kgeom_object    *object, 
   kgeom_primitive *primitive)
{
   if (object == NULL || object->fid == -1)
   {
      kerror("kgeom", "kgeom_write_primitive", 
	     "NULL object or object with unopen file");
      return FALSE;
   }

   if (primitive == NULL)
   {
      kset_errno(KNULL_PARAMETER);
      return FALSE;
   }

   return write_function[primitive_function_index(primitive->type)](object, 
								    primitive);
}

/*
 *    ================================================================== 
 *    primitive functions
 *    ==================================================================
 */

/*-----------------------------------------------------------
|  Routine Name: (static) _write_polyline
------------------------------------------------------------*/
static int _write_polyline(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_polyline *p = (kgeom_polyline *) primitive;
   kubyte has;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int  (fid, &p->type, (size_t) 1));
   CK(kwrite_ubyte(fid, &version, (size_t) 1));
   CK(kwrite_int  (fid, &p->nverts, (size_t) 1));
   CK(kwrite_int  (fid, &p->linetype, (size_t) 1));
   CK(kwrite_int  (fid, &p->linewidth, (size_t) 1));

   WRITE_LOCS;  WRITE_COLS;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_triangles
------------------------------------------------------------*/
static int _write_triangles(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_triangles *p = (kgeom_triangles *) primitive;
   kubyte has;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int   (fid, &p->type, (size_t) 1));
   CK(kwrite_ubyte (fid, &version, (size_t) 1));
   CK(kwrite_int   (fid, &p->nverts, (size_t) 1));

   has = (p->texture != NULL);
   CK(kwrite_ubyte(fid, &has, (size_t)1));
   if (has)
      CK(kwrite_string(fid, &p->texture, (size_t) 1));

   WRITE_LOCS;  WRITE_COLS;  WRITE_NORMS;  WRITE_TCOORDS;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_spheres
------------------------------------------------------------*/
static int _write_spheres(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_spheres *p = (kgeom_spheres *) primitive;
   kubyte has;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int   (fid, &p->type, (size_t) 1));
   CK(kwrite_ubyte (fid, &version, (size_t) 1));
   CK(kwrite_int   (fid, &p->nverts, (size_t) 1));

   WRITE_LOCS;  WRITE_COLS;  WRITE_RADII;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_text
------------------------------------------------------------*/
static int _write_text(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_text *p = (kgeom_text *) primitive;
   kubyte has;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int   (fid, &p->type, (size_t) 1));
   CK(kwrite_ubyte (fid, &version, (size_t) 1));
   CK(kwrite_int   (fid, &p->nverts, (size_t) 1));

   has = (p->strings != NULL);
   CK(kwrite_ubyte(fid, &has, (size_t) 1));
   if (has)                          \
      CK(kwrite_string(fid, p->strings, (size_t) p->nverts));

   has = (p->fonts != NULL);
   CK(kwrite_ubyte(fid, &has, (size_t) 1));
   if (has)                          \
      CK(kwrite_string(fid, p->fonts, (size_t) p->nverts));
   
   WRITE_LOCS; WRITE_OFFS; WRITE_UPS; WRITE_FLOWS; WRITE_COLS; WRITE_BGCOLS;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_cylinders
------------------------------------------------------------*/
static int _write_cylinders(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_cylinders *p = (kgeom_cylinders *) primitive;
   kubyte has;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int   (fid, &p->type, (size_t) 1));
   CK(kwrite_ubyte (fid, &version, (size_t) 1));
   CK(kwrite_int   (fid, &p->nverts, (size_t) 1));

   WRITE_LOCS; WRITE_COLS; WRITE_RAD1; WRITE_RAD2;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_directed_points
------------------------------------------------------------*/
static int _write_directed_points(kgeom_object *object, 
				  kgeom_primitive *primitive)
{
   kgeom_directed_points *p = (kgeom_directed_points *) primitive;
   kubyte has;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int   (fid, &p->type, (size_t) 1));
   CK(kwrite_ubyte (fid, &version, (size_t) 1));
   CK(kwrite_int   (fid, &p->nverts, (size_t) 1));

   WRITE_LOCS;  WRITE_COLS;  WRITE_NORMS;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_markers
------------------------------------------------------------*/
static int _write_markers(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_markers *p = (kgeom_markers *) primitive;
   kubyte has;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int   (fid, &p->type, (size_t) 1));
   CK(kwrite_ubyte (fid, &version, (size_t) 1));
   CK(kwrite_int   (fid, &p->nverts, (size_t) 1));
   CK(kwrite_int   (fid, &p->markertype, (size_t) 1));

   WRITE_LOCS;  WRITE_COLS;  WRITE_BGCOLS;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_polygon
------------------------------------------------------------*/
static int _write_polygon(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_polygon *p = (kgeom_polygon *) primitive;
   kubyte has;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int   (fid, &p->type, (size_t) 1));
   CK(kwrite_ubyte (fid, &version, (size_t) 1));
   CK(kwrite_int   (fid, &p->nverts, (size_t) 1));
   CK(kwrite_int   (fid, &p->linetype, (size_t) 1));
   CK(kwrite_int   (fid, &p->linewidth, (size_t) 1));

   WRITE_LOCS;  WRITE_COLS;

   /*-- strange single bg-color case --*/
   has = (p->bgcol != NULL);
   CK(kwrite_ubyte(fid, &has, (size_t) 1));
   if (has)                          \
      CK(kwrite_float(fid, p->bgcol, (size_t) ((object->has_alpha) ? 4 : 3)));

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_rectangles
------------------------------------------------------------*/
static int _write_rectangles(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_rectangles *p = (kgeom_rectangles *) primitive;
   kubyte has;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int   (fid, &p->type, (size_t) 1));
   CK(kwrite_ubyte (fid, &version, (size_t) 1));
   CK(kwrite_int   (fid, &p->nverts, (size_t) 1));
   CK(kwrite_int   (fid, &p->linetype, (size_t) 1));
   CK(kwrite_int   (fid, &p->linewidth, (size_t) 1));

   WRITE_LOCS;  WRITE_COLS;  WRITE_BGCOLS;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_ellipses
------------------------------------------------------------*/
static int _write_ellipses(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_ellipses *p = (kgeom_ellipses *) primitive;
   kubyte has;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int   (fid, &p->type, (size_t) 1));
   CK(kwrite_ubyte (fid, &version, (size_t) 1));
   CK(kwrite_int   (fid, &p->nverts, (size_t) 1));
   CK(kwrite_int   (fid, &p->linetype, (size_t) 1));
   CK(kwrite_int   (fid, &p->linewidth, (size_t) 1));

   WRITE_LOCS;  WRITE_COLS;  WRITE_BGCOLS;  WRITE_RAD1;  WRITE_RAD2;

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_mesh
------------------------------------------------------------*/
static int _write_mesh(kgeom_object *object, kgeom_primitive *primitive)
{
   kubyte has;
   int fid = object->fid;
   int s = TRUE;

   if (primitive->type == KGEOM_QUADMESH)
   {
      kgeom_quadmesh *p = (kgeom_quadmesh *) primitive;

      CK(kwrite_int   (fid, &primitive->type, (size_t) 1));
      CK(kwrite_ubyte (fid, &version, (size_t) 1));
      CK(kwrite_int   (fid, &p->width, (size_t) 1));
      CK(kwrite_int   (fid, &p->height, (size_t) 1));

      /*-- write the texture name if we have one --*/
      has = (p->texture != NULL);
      CK(kwrite_ubyte(fid, &has, (size_t)1));
      if (has)
	 CK(kwrite_string(fid, &p->texture, (size_t) 1));

      WRITE_LOCS;   WRITE_COLS;  WRITE_NORMS;  WRITE_TCOORDS;
   }
   else /*-- KGEOM_OCTMESH --*/
   {
      kgeom_octmesh *p = (kgeom_octmesh *) primitive;
      float  *locs, *norms, *tcoords;
      kubyte *cols;
      int     has_locs, has_cols, has_norms, has_tcoords;
      size_t  plane_size, l, c, t;
      int     i;

      has_locs = (locs != NULL);   has_norms   = (norms != NULL);   
      has_cols = (cols != NULL);   has_tcoords = (tcoords != NULL);

      /*-- octmesh data is interleaved so it can be incrementally written --*/
      CK(kgeom_write_octmesh_header(object, p->width, p->height, p->depth, 
				    p->texture, has_locs, has_cols, 
				    has_norms, has_tcoords));

      plane_size = (size_t) p->width * p->height;

      l = plane_size * (size_t) object->location_dim;
      c = plane_size * (size_t) ((object->has_alpha) ? 4 : 3);
      t = plane_size * (size_t) object->texture_coord_dim;

      locs = p->locs; cols = p->cols; norms = p->norms; tcoords = p->tcoords;

      for (i = 0; i < p->depth; i++)
      {
	 CK(kgeom_write_octmesh_plane(object, p->width, p->height,
				      locs, cols, norms, tcoords));
	 if (has_locs)    locs    += l;
	 if (has_cols)    cols    += c;
	 if (has_norms)   norms   += l;
	 if (has_tcoords) tcoords += t;
      }
   }

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_texture
------------------------------------------------------------*/
static int _write_texture(kgeom_object *object, kgeom_primitive *primitive)
{
   kubyte has;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int   (fid, &primitive->type, (size_t) 1));
   CK(kwrite_ubyte (fid, &version, (size_t) 1));

   if (primitive->type == KGEOM_TEXTURE2D)
   {
      kgeom_texture2D *p = (kgeom_texture2D *) primitive;

      CK(kwrite_int   (fid, &p->width, (size_t) 1));
      CK(kwrite_int   (fid, &p->height, (size_t) 1));

      /*-- cant use WRITE_COLS since this is a ubyte field --*/
      has = (p->cols != NULL);
      CK(kwrite_ubyte(fid, &has, (size_t) 1));
      if (has)
      {
	 int n = p->width * p->height * ((object->has_alpha) ? 4 : 3);
         CK(kwrite_ubyte(fid, p->cols, (size_t) n));
      }
   }
   else /*-- KGEOM_TEXTURE3D --*/
   {
      kgeom_texture3D *p = (kgeom_texture3D *) primitive;

      CK(kwrite_int   (fid, &p->width, (size_t) 1));
      CK(kwrite_int   (fid, &p->height, (size_t) 1));
      CK(kwrite_int   (fid, &p->depth, (size_t) 1));

      /*-- cant use WRITE_COLS since this is a ubyte field --*/
      has = (p->cols != NULL);
      CK(kwrite_ubyte(fid, &has, (size_t) 1));
      if (has)
      {
	 int n = p->width*p->height*p->depth*((object->has_alpha) ? 4 : 3);
         CK(kwrite_ubyte(fid, p->cols, (size_t) n));
      }
   }

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_view_xform
------------------------------------------------------------*/
static int _write_view_xform(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_view_xform *p = (kgeom_view_xform *) primitive;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int  (fid, &p->type, (size_t) 1));
   CK(kwrite_ubyte(fid, &version, (size_t) 1));
   CK(kwrite_float(fid,  p->matrix, (size_t) 16));

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_object_xform
------------------------------------------------------------*/
static int _write_object_xform(kgeom_object *object,
			       kgeom_primitive *primitive)
{
   kgeom_object_xform *p = (kgeom_object_xform *) primitive;
   kubyte has;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int   (fid, &p->type, (size_t) 1));
   CK(kwrite_ubyte (fid, &version, (size_t) 1));
   CK(kwrite_float (fid,  p->matrix, (size_t) 16));
   CK(kwrite_float (fid,  p->translate, (size_t) 3));
   CK(kwrite_int   (fid, &p->relative, (size_t) 1));

   has = (p->name != NULL);
   CK(kwrite_ubyte(fid, &has, (size_t) 1));
   if (has)
      CK(kwrite_string(fid, &p->name, (size_t) 1));

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_camera_orientation
------------------------------------------------------------*/
static int _write_camera_orientation(kgeom_object *object,
				     kgeom_primitive *primitive)
{
   kgeom_camera_orientation *p = (kgeom_camera_orientation *) primitive;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int   (fid, &p->type, (size_t) 1));
   CK(kwrite_ubyte (fid, &version, (size_t) 1));
   CK(kwrite_float (fid,  p->eye, (size_t) 3));
   CK(kwrite_float (fid,  p->at, (size_t) 3));
   CK(kwrite_float (fid,  p->up, (size_t) 3));
   CK(kwrite_float (fid, &p->field_of_view, (size_t) 1));
   CK(kwrite_float (fid, &p->hither, (size_t) 1));
   CK(kwrite_float (fid, &p->yon, (size_t) 1));
   CK(kwrite_int   (fid, &p->projection, (size_t) 1));
   CK(kwrite_int   (fid, &p->is_stereo, (size_t) 1));
   CK(kwrite_float (fid, &p->eye_distance, (size_t) 1));
   CK(kwrite_float (fid, &p->focal_length, (size_t) 1));

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_vr_event
------------------------------------------------------------*/
static int _write_vr_event(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_vr_event *p = (kgeom_vr_event *) primitive;
   int fid = object->fid;
   int s = TRUE;

   CK(kwrite_int  (fid, &p->type, (size_t) 1));
   CK(kwrite_ubyte(fid, &version, (size_t) 1));

   CK(kwrite_int  (fid, &p->have_button_events, (size_t) 3));
   CK(kwrite_int  (fid, &p->have_rotations, (size_t) 3));
   CK(kwrite_int  (fid, &p->have_translations, (size_t) 3));

   CK(kwrite_float(fid,  p->rotate, (size_t) 3));
   CK(kwrite_float(fid,  p->translate, (size_t) 3));
   CK(kwrite_int  (fid,  p->buttons, (size_t) 9));
   CK(kwrite_int  (fid, &p->clock, (size_t) 1));

   return s;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_object
------------------------------------------------------------*/
static int _write_object(kgeom_object *object, kgeom_primitive *primitive)
{
   kgeom_subobject *p = (kgeom_subobject *) primitive;

   /*-- make sure our fid is being carried down --*/
   ((kgeom_object *)p->object)->fid = object->fid;
   
   return kgeom_write_object_and_primitives(((kgeom_object *)p->object));
}

/*-----------------------------------------------------------
|  Routine Name: (static) _write_invalid
------------------------------------------------------------*/
static int _write_invalid(kgeom_object *object, kgeom_primitive *primitive)
{
   kerror("kgeom", "kgeom_write_primitive", "invalid primitive type");
   return FALSE;
}
