/*
  * Khoros: $Id: Label.h,v 1.3.2.5 1997/05/04 16:18:17 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: #defines and #includes for creating a
   >>>>		       graphical label.
   >>>>
   >>>>    Written By: Mark Young
   >>>>
   >>>>          Date: Aug 25, 1994
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kwidgets_Label_h_
#define _kwidgets_Label_h_

	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

/*
 *  Khoros Option Defines for XvwLabelGadget
 */
/*------------------------------------------------------------------------
| Attribute:	XVW_LABEL
| Description:	The label used for the label object.
| Type:		char *
| Default:	NULL	
| Legal Values:	any printable text
| Resource:	label
------------------------------------------------------------------------*/
#define XVW_LABEL               "label"

/*------------------------------------------------------------------------
| Attribute:	XVW_LABEL_JUSTIFY
| Description:	Specifies whether the label on the label object
|		is to be left justified, middle justified, or right justified.
| Type:		int
| Default:	.symbol index KLABEL_JUSTIFY_CENTER
| Legal Values:	.symbol index KLABEL_JUSTIFY_LEFT
|		.symbol index KLABEL_JUSTIFY_CENTER
|		.symbol index KLABEL_JUSTIFY_RIGHT
| Resource:	labelJustify
------------------------------------------------------------------------*/
#define XVW_LABEL_JUSTIFY	"labelJustify"

/* 
 *   how to define justification of labels
 */
#define KLABEL_JUSTIFY_LEFT    KJUSTIFY_LEFT
#define KLABEL_JUSTIFY_RIGHT   KJUSTIFY_RIGHT
#define KLABEL_JUSTIFY_CENTER  KJUSTIFY_CENTER

/*
 * private attributes used only by Text object, not for public consumption
 */
#define XVW_LABEL_CURSOR_POS	   "labelCursorPos"
#define XVW_LABEL_CUT_CURSOR_POS1  "labelCutCursorPos1"
#define XVW_LABEL_CUT_CURSOR_POS2  "labelCutCursorPos2"
#define XVW_LABEL_DISPLAY_CURSOR   "labelDisplayCursor"
#define XVW_LABEL_SELECTABLE       "labelSelectable"
#define XVW_LABEL_CURSOR_MOTION    "labelCursorMotion"
#define XVW_LABEL_CURSOR_XLOC      "labelCursorXLoc"
#define XVW_LABEL_CURSOR_YLOC      "labelCursorYLoc"
#define XVW_LABEL_PASTE_BUFFER     "labelPasteBuffer"
#define XVW_LABEL_LINES            "labelLines"
#define XVW_LABEL_NUMLINES         "labelNumLines"
#define XVW_LABEL_MULTILINE        "labelMultiline"
#define XVW_LABEL_YDIST	           "labelYdist"

/*------------------------------------------------------------------------
| Attribute:	XVW_LABEL_EMPHASIZE
| Description:  This attribute causes the label to be drawn twice, giving 
|		it a 3D, emphasized effect.
| Type:         int
| Default:      FALSE
| Legal Values: TRUE/FALSE
| Resource:	labelEmphasize
------------------------------------------------------------------------*/
#define XVW_LABEL_EMPHASIZE		"labelEmphasize"

/*------------------------------------------------------------------------
| Attribute:	XVW_LABEL_EMPHASIZECOLOR
| Description:  When
|		.symbol index XVW_LABEL_EMPHASIZE
|		is set to TRUE, this is the color in which the label
|		is emphasized.
| Type:         char *
| Default:      the background color
| Legal Values: any valid color name: see /usr/lib/X11/rgb.txt or Section
|               7.1.1 of \fIThe XLib Programming Manual\fP by Adrian Nye
| Resource:	labelEmphasizeColor
------------------------------------------------------------------------*/
#define XVW_LABEL_EMPHASIZECOLOR	"labelEmphasizeColor"

/*------------------------------------------------------------------------
| Attribute:    XVW_LABEL_EMPHASIZEPIXEL
| Description:  When
|               .symbol index XVW_LABEL_EMPHASIZE
|               is set to TRUE, this is the pixel value associated with
|		the color in which the label is emphasized.
| Type:         unsigned long
| Default:      pixel associated with the background color
| Legal Values: any valid pixel value: see Section 7.3 and 7.4
|               of \fIThe XLib Programming Manual\fP by Adrian Nye
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_LABEL_EMPHASIZEPIXEL         "labelEmphasizePixel"

/*------------------------------------------------------------------------
| Attribute:	XVW_FORCE_REDISPLAY
| Description:  This attribute of the manager gadget is inherited by the
|		label object;  it is emphasized here because it 
|		can be used by the label object to achieve smoother, 
|		faster update of the label.  It is especially useful when
|		the label is updated very rapidly, as for example 
|		in an event handler installed on button motion. 
| Type:         int
| Default:      FALSE
| Legal Values: TRUE/FALSE
| Resource:	forceRedisplay
------------------------------------------------------------------------*/

/*
 *  Define the Class name, by which people should be externally referencing
 *  the Label Object by.
 */
#define LabelGadgetClass	xvwLabelGadgetClass

    /*-------------------------------------*
    |       global variable declarations
    ---------------------------------------*/

/*
 *  XToolkit Widget Declaration LabelGadgetClass
 */
extern   WidgetClass     LabelGadgetClass;


	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/*
 *  Khoros Function Definition for Label Object
 */
xvobject xvw_create_label PROTO((xvobject, char *));


#endif /* _kwidgets_Label_h_ */
/* Don't add after this point */
