 /*
  * Khoros: $Id: List.h,v 1.4 1997/01/25 20:00:15 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Definitions for List xvobject
   >>>>
   >>>>     Written By: Mark Young & Danielle Argiro
   >>>>
   >>>>          Date: Jun 10, 1996
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvwidgets_List_h_
#define _xvwidgets_List_h_

        /*-------------------------------------*
        |       #defines
        ---------------------------------------*/


/*------------------------------------------------------------------------
| Attribute:	XVW_LIST_STRINGS
|
| Description:	This attribute is the array of strings representing the items
|		that are displayed by the list object.  Note that when setting 
|		this attribute, you MUST set the 
|		.symbol index XVW_LIST_SIZE 
|		attribute 
|		at the same time to indicate the number of strings in the array!
|		Similarly, if getting this attribute, you should get the
|		.symbol index XVW_LIST_SIZE 
|		attribute as well to tell you the number
|		of strings in the returned array.
|
| Type:		kstring *
| Default:	NULL
| Legal Values:	Array of strings, of size specified by 
|		.symbol index XVW_LIST_SIZE.
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_LIST_STRINGS  "listStrings"

/* this will be obsolete soon */
#define XVW_LIST_LIST  "listStrings"

/*------------------------------------------------------------------------
| Attribute:	XVW_LIST_SIZE	
|
| Description:	This attribute specifies size of the array of strings that are 
|		displayed by the list object. Note that when setting 
|               this attribute, you should set the 
|               .symbol index XVW_LIST_STRINGS
|		attribute
|               at the same time to indicate the strings in the array!
| Type:		int
| Default:	0
| Legal Values:	size of the string array being displayed 
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_LIST_SIZE	  "listSize"

/*------------------------------------------------------------------------
| Attribute:    XVW_LIST_INDEX
|
| Description:  This attribute specifies the selected item in the list by
|		index. Getting this attribute returns the index of the selected 
|		item in the list, or -1 if no item is selected. Setting 
|		this attribute causes the item with the specified index to 
|		be selected, or if set to -1 causes the list to have no item
|		selected .
| Type:         int
| Default:      -1
| Legal Values: -1 for no string selected; otherwise, 
|		the index of the string that is selected (0 to 
|		.symbol index XVW_LIST_SIZE 
|		- 1)
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_LIST_INDEX     "listIndex"

/*------------------------------------------------------------------------
| Attribute:    XVW_LIST_STRING
|
| Description:  This attribute specifies the selected item in the list by
|               string. Getting this attribute returns the string corresponding
|		to the selected item of the list, or NULL if no item is 
|		selected. Setting this attribute causes the selected item 
|		in the list to have its string updated to this new value.
|		Setting this attribute to NULL causes the list to have no
|		item selected.
| Type:         kstring
| Default:      NULL
| Legal Values: NULL for no item selected; otherwise,
|               the string of the item that is selected
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_LIST_STRING     "listString"

/*------------------------------------------------------------------------
| Attribute:    XVW_LIST_ADD
|
| Description:  This action attribute adds the specified string to the
|		end of the list and automatically increments
|		.symbol index XVW_LIST_SIZE .
|		The displayed list is automatically updated to display
|		the new string.
| Type:         char *
| Default:      N/A
| Legal Values: the string to be added to the list
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_LIST_ADD     "listAdd"

/*------------------------------------------------------------------------
| Attribute:    XVW_LIST_DELETE
|
| Description:  This action attribute deletes the specified string from the
|               list and automatically decrements
|               .symbol index XVW_LIST_SIZE .
|		The displayed list is automatically updated so that the old
|               string is no longer displayed.
| Type:         char *
| Default:      N/A
| Legal Values: the string to be deleted from the list
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_LIST_DELETE     "listDelete"


/*------------------------------------------------------------------------
| Attribute:    XVW_LIST_DELETEALL
|
| Description:  This action attribute deletes all strings from the
|               list and automatically sets
|               .symbol index XVW_LIST_SIZE 
|               to 0 and
|		.symbol index XVW_LIST_INDEX
|		to -1. The displayed list is emptied.
| Type:         int
| Default:      N/A
| Legal Values: TRUE
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_LIST_DELETEALL     "listDeleteAll"


/*------------------------------------------------------------------------
| Attribute:    XVW_LIST_ITEM_SELECT
|
| Description:	\fIxvw_add_callback()\fP may be used to install a callback 
|		on the list object which will be fired when the user clicks 
|		\fIonce\fP on an item in the list.  
|		.sp
|		When calling 
|		\fIxvw_add_callback()\fP, pass this attribute directly, as in
|               .begin code
|               xvw_add_callback(listobj, 
|			         XVW_LIST_ITEM_SELECT,
|                                list_cb, client_data);
|               .end code
| Type:		kfunc_void
| Default:	NULL
| Legal Values: callback function, in the form:
|               .begin code
|               void callback_function
|                  xvobject object,
|                  kaddr client_data,
|                  kaddr call_data)
|               .end code
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_LIST_ITEM_SELECT    "listItemSelect"

/*------------------------------------------------------------------------
| Attribute:    XVW_LIST_ITEM_ACTION
|
| Description:	\fIxvw_add_callback()\fP may be used to install a callback 
|               on the list object which will be fired when the user clicks 
|               \fItwice, rapidly\fP on an item in the list.  
|               .sp
|               When calling
|               \fIxvw_add_callback()\fP, pass this attribute directly, as in
|               .begin code
|               xvw_add_callback(listobj, 
|				 XVW_LIST_ITEM_ACTION,
|                                list_cb, client_data);
|		.end code
|
| Type:		kfunc_void
| Default:	NULL
| Legal Values:	callback function, in the form:
|               .begin code
|               void callback_function
|                  xvobject object,
|                  kaddr client_data,
|                  kaddr call_data)
|               .end code
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_LIST_ITEM_ACTION    "listItemAction"

/*------------------------------------------------------------------------
| Attribute:    XVW_LIST_HIGHLT_ELEM
|
| Description:	This \fIaction\fP attribute may be used to highlight an 
|		element in the list.  Provide the index of the element 
|		to be highlighted. If the element is already highlighted
|               when the action attribute is used, nothing will happen.
|		Remember that indexing begins at 0.
|
| Type:		int
| Default:	N/A
| Legal Values: 0 <= value <= 
|		.symbol index XVW_LIST_SIZE
|		-1
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_LIST_HIGHLT_ELEM    "listHighlightElem"

/*------------------------------------------------------------------------
| Attribute:    XVW_LIST_UNHIGHLT_ELEM
|
| Description:	This \fIaction\fP attribute may be used to un-highlight an 
|               element in the list.  Provide the index of the element 
|               to be un-highlighted.  If the element is not highlighted
|		when the action attribute is used, nothing will happen.
|		Remember that indexing begins at 0.
|
| Type:		int
| Default:	N/A
| Legal Values:	0 <= value <= 
|               .symbol index XVW_LIST_SIZE
|               -1
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_LIST_UNHIGHLT_ELEM  "listUnhighlightElem"

/*
 *  Define the Class name, by which people should be externally referencing
 *  the List Object by.
#define ListWidgetClass       xvwListWidgetClass
 */

    /*-------------------------------------*
    |       global variable declarations
    ---------------------------------------*/

/*
 *  XToolkit Widget Declaration ListWidgetClass
 */
extern   WidgetClass     xvwListWidgetClass;

    /*-------------------------------------*
    |       typedefs
    ---------------------------------------*/

/*
 *  The list structure returned as callback data from the list widget
 */
typedef struct _xvw_list_struct {
	char   *string;
	int    list_index;
	int    action;
} xvw_list_struct;

    /*-------------------------------------*
    |       routine definitions
    ---------------------------------------*/

/*
 *  Khoros Function Definition for List Widget
 */
xvobject xvw_create_list       PROTO((xvobject, char *));
xvobject xvw_retrieve_list     PROTO((xvobject));
void     xvw_change_list       PROTO((xvobject, char **, size_t, int));


#endif /* _xvwidgets_List_h_ */
/* Do not add after this line */
