/*
  * Khoros: $Id: Pixmap.h,v 1.3 1997/01/20 05:10:28 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Attributes of the Pixmap object
   >>>>
   >>>>    Written By: Mark Young & Danielle Argiro
   >>>>
   >>>>          Date: Aug 25, 1994
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kwidgets_Pixmap_h_
#define _kwidgets_Pixmap_h_

	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

/*------------------------------------------------------------------------
| Attribute:    XVW_PIXMAP
|
| Description:  This is the pixmap (or bitmap) that appears in the pixmap
|               object.  Candidates for the value of this attribute may
|		be created with the use of \fIXCreatePixmap()\fP;  see
|		\fIThe Xlib Reference Manual\fP by O'Reilly and Associates.
|		Note that this attribute is mutually exclusive with
|               .symbol index XVW_PIXMAP_FILENAME ;
|               specify one or the other, not both.
|
| Type:         Pixmap
| Default:      NULL
| Legal Values: Valid Pixmap structure
| Resource:     pixmap
------------------------------------------------------------------------*/
#define XVW_PIXMAP              "pixmap"

/*------------------------------------------------------------------------
| Attribute:	XVW_PIXMAP_FILENAME
|
| Description:  This is the name of the file defining the pixmap. 
|
| Type:         char *
| Default:      NULL
| Legal Values: The full path to a valid xpm or xbm file, defining
|               the desired pixmap.  Note that the path may contain
|               $TOOLBOX.
| Resource:	pixmapFilename
------------------------------------------------------------------------*/
#define XVW_PIXMAP_FILENAME	 	"pixmapFilename"

/*------------------------------------------------------------------------
| Attribute:    XVW_PIXMAP_JUSTIFY
|
| Description:  Specifies how the pixmap is justified within the pixmap
|		object.  The pixmap can be set to be
|               left justified, middle justified, or right justified.
|               Note that the same values are used as for
|               .symbol index XVW_LABEL_JUSTIFY .
| Type:         int
| Default:      .symbol index KPIXMAP_JUSTIFY_CENTER
| Legal Values: .symbol index KPIXMAP_JUSTIFY_LEFT
|               .symbol index KPIXMAP_JUSTIFY_CENTER
|               .symbol index KPIXMAP_JUSTIFY_RIGHT
| Resource:     pixmapJustify
------------------------------------------------------------------------*/
#define XVW_PIXMAP_JUSTIFY      "pixmapJustify"

/*
 *   how to define justification of pixmaps
 *   pixmap label widgets)
 */
#define KPIXMAP_JUSTIFY_LEFT    KJUSTIFY_LEFT
#define KPIXMAP_JUSTIFY_RIGHT   KJUSTIFY_RIGHT
#define KPIXMAP_JUSTIFY_CENTER  KJUSTIFY_CENTER

/*------------------------------------------------------------------------
| Attribute:	XVW_PIXMAP_MASK
|
| Description:  Pixmaps are defined by a rectangular grid of values.  For 
|		implementing non-rectangular pixmaps, a bitmap is used to 
|		indicate which portions of the rectangular pixmap should 
|		appear, and which portions should not be displayed. This 
|		bitmap is referred to as a \fImask\fP.  The bitmap must be 
|		defined in an xbm file, and the bitmap should be of the same 
|		size as the pixmap being displayed.  Anywhere a 1 appears in 
|		the bitmap, the value of the pixmap at that point is displayed; 
|		anywhere a 0 appears in the bitmap, the value of the pixmap 
|		at that point will not be displayed.  Thus, a circular pixmap
|		can be implemented with a pixmap/bitmap mask pair:  the pixmap
|		will define the picture in the circle, while the bitmap mask
|		has all 1's within the boundaries of the circle, and all 0's
|		outside the circle bounds.
|
|               Candidates for the bitmap may be created with 
|               \fIXCreateBitmapFromData()\fP;  see \fIThe Xlib Reference 
|		Manual\fP by O'Reilly and Associates.  Note that this 
|		attribute is
|               mutually exclusive with
|               .symbol index XVW_PIXMAP_MASKNAME ;
|               specify one or the other, not both.
|
| Type:         Pixmap
| Default:      NULL
| Legal Values: Valid Pixmap structure
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_PIXMAP_MASK	 		"pixmapMask"

/*------------------------------------------------------------------------
| Attribute:	XVW_PIXMAP_MASKNAME
| Description:  This is the file defining the bitmap mask used with
|		non-rectangular pixmaps;  see 
|		.symbol index XVW_PIXMAP_MASK
|		for more details. 
| Type:         char *
| Default:      NULL
| Legal Values: The full path to the xbm file defining the mask;
|		Note that the path may contain $TOOLBOX.
| Resource:	pixmapMaskname
------------------------------------------------------------------------*/
#define XVW_PIXMAP_MASKNAME		"pixmapMaskname"


/*
 *  Define the Class name, by which people should be externally referencing
 *  the Pixmap Object by.
 */
#define PixmapGadgetClass	xvwPixmapGadgetClass

    /*-------------------------------------*
    |       global variable declarations
    ---------------------------------------*/

/*
 *  XToolkit Widget Declaration PixmapGadgetClass
 */
extern   WidgetClass     PixmapGadgetClass;


	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/*
 *  Khoros Function Definition for Pixmap Object
 */
xvobject xvw_create_pixmap PROTO((xvobject, char *));


#endif /* _kwidgets_Pixmap_h_ */
/* Don't add after this point */
