 /*
  * Khoros: $Id: Text.h,v 1.4.2.2 1997/05/02 18:59:22 mirage Exp $
  */
 
 
/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */
 
 
/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Defines for the Text Widget
   >>>>
   >>>>     Written By: Mark Young & Danielle Argiro & John Salas
   >>>>
   >>>>          Date: Jan 22, 1997
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */
 
#ifndef _xvwidgets_Text_h_
#define _xvwidgets_Text_h_
 
        /*-------------------------------------*
        |       #defines
        ---------------------------------------*/
 


/*------------------------------------------------------------------------
| Attribute:	XVW_TEXT_STRING
|
| Description:	The string that is displayed in the text object. It is mutually
|		exclusive with
|		.symbol index XVW_TEXT_FILENAME ;
|		use one or the other, not both simultaneously.
|
| Type:		char *
| Default:	NULL
| Legal Values:	any printable text
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_TEXT_STRING          "textString"

/*------------------------------------------------------------------------
| Attribute:    XVW_TEXT_FILENAME
|
| Description:  The name of the file whose contents is displayed in
|		the text object.  It is mutually exclusive with
|		.symbol index XVW_TEXT_STRING ;
|		use one or the other, not both simultaneously.
|
| Type:         char *
| Default:      NULL
| Legal Values: name of any existing file with readable text 
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_TEXT_FILENAME         "textFilename"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXT_INSERT_POS
|
| Description:	The location of the cursor in the string, where each number
|		represents a character.  For example, if 
|		.symbol index XVW_TEXT_INSERT_POS
|		was set to 5, that would mean that the cursor appeared 5
|		letters into the string.
|
| Type:		int
| Default:	0
| Legal Values:	0 <= value <= length of currently displayed text
| Resource:	TextLabel.labelCursorPos
------------------------------------------------------------------------*/
#define XVW_TEXT_INSERT_POS      "TextLabel.labelCursorPos"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXT_EDIT_TYPE
|
| Description:	The text in the text object may be read-only, or read-write.  
|		Specify 
|		.symbol index KTEXT_READ
|		for the former, or
|		.symbol index KTEXT_EDIT
| 		for the latter.
|
| Type:		int
| Default:	.symbol index KTEXT_EDIT
| Legal Values:	.symbol index KTEXT_READ
|		.symbol index KTEXT_EDIT
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_TEXT_EDIT_TYPE       "textEditType"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXT_LEFT_MARGIN
|
| Description:	The number of pixels from the leftmost edge of the text
|		object that the string will be displayed.
|
| Type:		unsigned short
| Default:	0
| Legal Values:	values >= 0
| Resource:	textLeftMargin
------------------------------------------------------------------------*/
#define XVW_TEXT_LEFT_MARGIN    "textLeftMargin"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXT_WRAP
|
| Description:	Used with multi-line text objects only, this attribute controls 
|               if and how text is wrapped when 
|		it is longer than the text object's width.  The text may 
|		be left unwrapped, it may wrap on a line which may divide
|		words, or attempt to wrap on words which will leave words
|		intact.
|		.sp
|		.symbol index KTEXT_WRAP_NEVER -
|		don't wrap text
|		.br
|		.symbol index  KTEXT_WRAP_LINE -
|		wrap text on lines (may break up words) 
|		.br
|		.symbol index KTEXT_WRAP_WORD -
|		wrap text on words (attempt to not break up words).
|		With single-line text objects, this attribute is ignored;
|		wrapping is never done.
|
| Type:		int
| Default:	.symbol index KTEXT_WRAP_NEVER
| Legal Values:	.symbol index KTEXT_WRAP_WORD
|		.symbol index KTEXT_WRAP_NEVER
|		.symbol index KTEXT_WRAP_LINE
| Resource:	textWrap
------------------------------------------------------------------------*/
#define XVW_TEXT_WRAP           "textWrap"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXT_MULTILINE
|
| Description:	Control whether or not more than one line of text can
|		be displayed.  Set to TRUE if you want to display more 
|		than one line of text.
|
| Type:		int
| Default:	FALSE
| Legal Values:	TRUE/FALSE
| Resource:	textMultiline
------------------------------------------------------------------------*/
#define XVW_TEXT_MULTILINE      "textMultiline"


/*------------------------------------------------------------------------
| Attribute:    XVW_TEXT_ENTER_STRING
|
| Description:  If desired, \fIxvw_add_callback()\fP may be used to install
|               a callback on the text object text object which will be fired
|		when the user enters <return> in the text window.  
|
|	        This can only be used with single-line text objects, not with 
|	        multi-line text objects (as <return> in a multi-line text object
|		must be reserved to indicate a carriage return.  
|
|		The callback should be installed as follows:
|
|               .begin code
|               xvw_add_callback(text_object, XVW_TEXT_ENTER_STRING,
|                                callback, client_data);
|               .end code
|		
|		The string that was entered will be sent to the callback
|		as the call_data.  Before using, the string must be cast
|		to its proper data type, as in:
|
|		kstring string = (kstring) call_data;
|
| Type:         kfunc_void
| Default:      NULL
| Legal Values: callback function, in the form:
|               .begin code
|                void callback_function
|                    xvobject object,
|                    kaddr client_data,
|                    kaddr call_data)
|               .end code
| Resource:   N/A
------------------------------------------------------------------------*/
#define XVW_TEXT_ENTER_STRING        "textEnterString"

/* 
 *   how to define whether to wrap (on text widgets) 
 */
#define KTEXT_WRAP_WORD     0
#define KTEXT_WRAP_LINE     1
#define KTEXT_WRAP_NEVER    2

/* 
 *   how to define edit type (for text widgets) 
 */
#define  KTEXT_READ    0
#define  KTEXT_EDIT    1

    /*-------------------------------------*
    |       global variable declarations
    ---------------------------------------*/

/*
 *  XToolkit Widget Declaration TextWidgetClass
 */
extern   WidgetClass     xvwTextWidgetClass;

        /*-------------------------------------*
        |       routine definitions
        ---------------------------------------*/

/*
 *  Khoros Function Definition for Text Widget
 */
xvobject xvw_create_text              PROTO((xvobject, char *));


#endif /* _xvwidgets_Text_h_ */
/* Do not add after this line */
