 /*
  * Khoros: $Id: xvforms.h,v 1.6.6.2 1997/04/29 22:26:58 mirage Exp $
  */



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>                                                          <<<<
   >>>>       Purpose: Main include file for xvforms library     <<<<
   >>>>                                                          <<<<
   >>>>    Written By: Danielle Argiro                           <<<<
   >>>>                                                          <<<<
   >>>>          Date: Aug 5, 1995                               <<<<
   >>>>                                                          <<<<
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvforms_xvforms_h_
#define _xvforms_xvforms_h_

        /*-------------------------------------*
        |       #include
        ---------------------------------------*/

#include <xvforms/xvformsP.h>


        /*-------------------------------------*
        |       #defines
        ---------------------------------------*/

/*
 * definitions of general xvforms attributes
 */
#include <xvforms/xvgen.h>

/*
 * definitions of attributes for integer selections
 */
#include <xvforms/xvint.h>

/*
 * definitions of attributes for float selections
 */
#include <xvforms/xvflt.h>

/*
 * definitions of attributes for double selections
 */
#include <xvforms/xvdbl.h>

/*
 * definitions of attributes for inputfile & outputfile selections
 */
#include <xvforms/xvfile.h>

/*
 * definitions of attributes for string selections
 */
#include <xvforms/xvstr.h>

/*
 * definitions of attributes for logical selections
 */
#include <xvforms/xvlogic.h>

/*
 * definitions of attributes for toggle selections
 */
#include <xvforms/xvtoggle.h>

/*
 * definitions of attributes for cycle selections
 */
#include <xvforms/xvcycle.h>

/*
 * definitions of attributes for list selections
 */
#include <xvforms/xvlist.h>

/*
 * definitions of attributes for displaying subforms/panes
 */
#include <xvforms/xvdisp.h>

/*
 * definitions of attributes for printing UIS's
 */
#include <xvforms/xvprint.h>

/*
 * definitions of miscellaneous attributes
 */
#include <xvforms/xvmisc.h>


/*
 * #defines for extra calls
 */

#define XVF_CALL_FIRST               1
#define XVF_CALL_LAST                2
#define XVF_CALL_SUBSTITUTE          3

/*
 * #defines for GUI item elements
 */
#define XVF_BACKPLANE    1
#define XVF_OPT_OBJ      2
#define XVF_BUTTON_OBJ   3
#define XVF_LABEL_OBJ    4
#define XVF_TEXT_OBJ     5
#define XVF_SCROLL_OBJ   6
#define XVF_PIXMAP_OBJ   7

/*
 * #defines for controlling use of menuforms
 */
#define XVF_NO_EDIT      -1
#define XVF_PARTIAL_EDIT  0
#define XVF_FULL_EDIT     1

#define KFORM_ENTIRE_FORM   1
#define KFORM_FULL_SUBFORM  2
#define KFORM_PANE_ONLY     3

        /*-------------------------------------*
        |       typedefs
        ---------------------------------------*/

        /*-------------------------------------*
        |       global variable declarations
        ---------------------------------------*/

        /*-------------------------------------*
        |       macros
        ---------------------------------------*/


/************************************************************
*
*  Routine Name: xvf_set_attribute - set a single attribute of a GUI item
*
*       Purpose: sets a single attribute for a GUI item
*
*         Input: kformstruct - GUI item for which to set the attribute
*                attribute   - the attribute name
*                value       - the attribute value
*
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: Value provided must be of the expected data type
*    Written By: Mark Young & Danielle Argiro
*          Date: Feb 1, 1994 
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: xvf_set_attribute(
*                !   kform_struct kformstruct,
*                !   char         *attribute,
*                !   data         value)
*
*************************************************************/

#define xvf_set_attribute(kformstruct, attribute, value) \
                xvf_set_attributes(kformstruct, attribute, value, NULL)


/************************************************************
*
*  Routine Name: xvf_get_attribute - get a single attribute of a GUI item
*
*       Purpose: gets a single attribute for a GUI item
*
*         Input: kformstruct - GUI item for which to get the attribute
*                attribute   - the attribute name
*                value       - pointer to the the attribute value
*
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: Value pointer passed in must be of the expected data type
*    Written By: Mark Young & Danielle Argiro 
*          Date: Feb 1, 1994 
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: xvf_get_attribute(
*                !   kform_struct kformstruct,
*                !   char         *attribute,
*                !   data         *value)
*
*************************************************************/

#define xvf_get_attribute(kformstruct, attribute, value) \
                xvf_get_attributes(kformstruct, attribute, value, NULL)

        /*-------------------------------------*
        |       routine definitions
        ---------------------------------------*/

/* attributes.c */
int xvf_set_attributes  PROTO((kform_struct *, kvalist));
int xvf_get_attributes  PROTO((kform_struct *, kvalist));
int xvf_vset_attributes PROTO((kform_struct *, kva_list));
int xvf_vget_attributes PROTO((kform_struct *, kva_list));

/* attrib_cb.c */
int xvf_add_gui_callback    PROTO((kform_struct *, char *,
				   void (*)(kform_struct *,char *,kaddr,kaddr),
				   kaddr));
int xvf_remove_gui_callback PROTO((kform_struct *, char *,
				   void (*)(kform_struct *,char *,kaddr,kaddr),
				   kaddr));

/* init.c */
void xvf_initialize PROTO((void));

/* clear.c */
void xvf_clear_selections PROTO((kform *));

/* createtree.c */
kform *xvf_create_form PROTO((char *, int, void(*)(kform *, ksubform *, kaddr),
			      kaddr, int, int, int));

/* runform.c */
void xvf_run_form PROTO((void));

/* change.c */
int xvf_change_gui PROTO((kform_struct *, int, char *));

/* extra.c */
int xvf_add_extra_call    PROTO((kform_struct *, void(*)(kaddr), kaddr, int));
int xvf_remove_extra_call PROTO((kform_struct *, void(*)(kaddr), kaddr));

/* destroy.c */
void xvf_destroy_allforms PROTO((void));
void xvf_destroy_form     PROTO((kform *));
void xvf_destroy_subform  PROTO((ksubform *));

/* get.c */
xvobject      xvf_get_xvobject    PROTO((kform_struct *, int, int));
kform_struct *xvf_get_kformstruct PROTO((kform *, xvobject));


	/* Ghostwriter Prototypes:  Do Not Erase This Comment */

#endif /* _xvforms_xvforms_h_ */
/* Don't add after this line */
