/*
  * Khoros: $Id: Color.h,v 1.7 1997/01/20 05:11:26 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: #defines and #includes for subclassing a
   >>>>		       Color widget.
   >>>>
   >>>>    Written By: Mark Young
   >>>>
   >>>>          Date: Nov 16, 1992 16:20
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvisual_Color_h_
#define _xvisual_Color_h_

	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

/*
 *  Attribute definitions for Color Visual Object
 */

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_BEGIN
|
| Description:  Use this attribute to specify the beginning of the range of 
|               color indices to be allocated, if that range is to be limited 
|               to something other than the full number of indices in the 
|               colormap.
|
| Type:         int
| Default:      0
| Legal Values: 0 to height of colormap
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_BEGIN		 "colorBegin"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_END
|
| Description:  Use this attribute to specify the end of the range of 
|               color indices to be allocated, if that range is to be limited 
|               to something other than the full number of indices in the 
|               colormap.
|
| Type:         int
| Default:      0
| Legal Values: 0 to height of colormap
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_END		 "colorEnd"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_COLORS
| Description:  This \fIread only\fP attribute can be used to obtain
|		the XColor array representing the map data for the displayed
|		data object.  Red, green, and blue values are normalized 
|		between 0 and 65,536.  Pixel values reflect those that were
|		allocated in order to display the colormap.
| Type:         XColor *
| Default:      NULL
| Legal Values:  
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_COLORS	 "colorColors"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_NUMCOLORS
| Description:  This \fIread only\fP attribute is used to obtain the size 
|		of the XColor array returned by
|		.symbol index XVW_COLOR_COLORS .
| Type:         int
| Default:      0
| Legal Values: value > 0
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_NUMCOLORS	 "colorNumcolors"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_COLOROBJ
| Description:  This is the data object containing the colormap to be
|		displayed.  Note that this attribute is mutually exclusive
|		with 
|		.symbol index XVW_COLOR_COLORFILE ;
|		set one or the other, not both. 
| Type:         kobject
| Default:      NULL
| Legal Values: valid data object
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_COLOROBJ	 "colorColorobj"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_COLORFILE
|
| Description:  The name of the file containing the colormap to be displayed.
|               Note that this attribute is mutually exclusive with 
|               .symbol index XVW_COLOR_COLOROBJ ; 
|		set one or the other, not both.
|
| Type:         char *
| Default:      NULL
| Legal Values: name of file w/ colormap
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_COLORFILE	 "colorColorfile"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_RELOAD
|
| Description:  Use of this \fIaction attribute\fP causes the data to be
|               re-read from the associated data object;  the data is then
|               redisplayed.
|               .sp
|               Use of this attribute is useful when displaying data with
|               a large number of image values.  When displaying such data 
|		on screens that are less than 24 bit, the limitation
|               of the number of colorcells available for allocation on the
|               screen can cause difficulties in realistically displaying
|               the colors in the data.
|               .sp
|               If the data being displayed has more values than can be
|               accomodated by the screen, then some data values must be
|               mapped to the same pixel as other data values.
|               .sp
|               Consider the situation of a large image that cannot
|               be displayed in its entirety, and furthermore containing a
|               much greater number of image values than can be displayed on
|               the current screen, \fInot all of which appear in every region
|               of the image\fP.  The displayed result will be most acceptable
|               for the region of the image was loaded first, since the image
|               values in that region of the image will have the greatest chance
|               of getting unique pixel values allocated for them.  However,
|               other regions of the image with a large number of data values
|               that differ from the values of the first region may suffer from
|               the effect of having a large number of data values all being
|               mapped to the same pixel value.  Such situations can be
|               rectified by allowing the currently region to be re-read from
|               scratch, causing the color allocation process to be redone, and
|               the current region being allowed to commandeer all the available
|               pixel values.
|
| Type:         int
| Default:      N/A
| Legal Values: TRUE (action attribute)
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_RELOAD	 "colorReload"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_CALLBACK
|
| Description:  If desired, you may install a callback that will be fired
|               when the colormap is changed.  The colormap may be changed
|               by an autocolor procedure or a colormap operation being 
|               applied, a pseudocoloring operation, a change in the map 
|               data of the data object being displayed, or any other
|               modification of the colormap.
|
| Type:         kfunc_void
| Default:      NULL
| Legal Values: callback function, in the form:
|               .begin code
|               void callback_function
|                  xvobject object,
|                  kaddr client_data,
|                  kaddr call_data)
|               .end code
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_CALLBACK	 "colorCallback"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_NORM_TYPE
|
| Description:  Before colors are displayed by the color class, normalization 
|		must be done in order to ensure that the pixel values of the 
|		image fall within a visible range.  Normalization can be one 
|		of two types: local or global.
|		For local normalization ( 
|		.symbol index KCOLOR_NORM_LOCAL ,
|		or "Local" on the menuform), the normalization
|		procedure is performed locally using the maximum and minimum 
|		values of each of the red, green, and blue map columns.  For 
|		example, if the minimum and maximum values in the red column 
|		are 0 to 1, and the minimum and maximum values in the
|		green column are 100 to 200, the red column will be normalized 
|		such that the value 0 stays 0, but the 1 value becomes 255, 
|		while the green column will be normalized such that the 100 
|		value becomes 0, and the 200 value becomes 255.
|		For global normalization (
|		.symbol index KCOLOR_NORM_GLOBAL , 
|		or "Global" on the menuform), the normalization procedure is
|		performed globally over all of the three columns displayed as 
|		red, green, and blue.  Thus, with global normalization, if the 
|		minimum and maximum values over all the red, green, and blue
|		columns are 1 to 200, all three map columns will be normalized 
|		such that 1 becomes 0 and 200 becomes 255.
|
| Type:         int
| Default:      .symbol index KCOLOR_NORM_LOCAL
| Legal Values: .symbol index KCOLOR_NORM_GLOBAL
|               .symbol index KCOLOR_NORM_LOCAL
| Resource:     colorNormType
------------------------------------------------------------------------*/
#define XVW_COLOR_NORM_TYPE      "colorNormType" 

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_NORM_UBYTE
|
| Description:  If desired, you may normalize unsigned byte data.  Normally
|	        all data types are normalized, but under certain conditions
|		(especially when comparing images together) you don't want
|		to perform normalization.
|
| Type:         int
| Default:      TRUE
| Legal Values: TRUE/FALSE
| Resource:     colorNormUbyte
------------------------------------------------------------------------*/
#define XVW_COLOR_NORM_UBYTE      "colorNormUbyte" 

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_NORM_METHOD
|
| Description:  When normalization is to be performed, this attribute defines 
|		the algorithm which is used in normalization.  Normalization 
|               options allow different emphasis on how the data is displayed.
|
|	        The default, 
|               .symbol index KCOLOR_NORM_BESTGUESS 
|               tries to choose the correct normalization by looking at the 
|               data to be displayed.  If the data is floating point and 
|               between 0 and 1 (inclusive), the normalization chooses a minimum
|               of 0 and a maximum of 1 and scales accordingly.
|               Otherwise, if all the data is between 0 and 255
|               (inclusive), normalization chooses a minimum of 0 and a
|               maximum of 255.  Note that global normalization will always
|               be used in these cases.  If neither of these conditions is met, 
|               then
|               .symbol index KCOLOR_NORM_MAXCOLORS 
|               will be used for the normalization.
|
|               Using
|               .symbol index KCOLOR_NORM_NONE 
|               allows normalization to be turned off.  This option assumes
|               the data is already scaled between 0 and 65535.
|               
|               For a simple normalization
|		of values to within displayable intensity bounds, use 
|		.symbol index KCOLOR_NORM_MAXCOLORS 
|		Alternatively, images can have contrast increased or reduced 
|		using on of the standard deviation normalizations.
|		.symbol index KCOLOR_NORM_1STDDEV 
|		will normalize values to within 1 standard deviation,
|		.symbol index KCOLOR_NORM_2STDDEV 
|		will normalize values to within 2 standard deviations, and
|		.symbol index KCOLOR_NORM_3STDDEV 
|		will normalize values to within 3 standard deviations.
|
| Type:         int
| Default:      .symbol index KCOLOR_NORM_MAXCOLORS
| Legal Values: .symbol index KCOLOR_NORM_MAXCOLORS
|               .symbol index KCOLOR_NORM_1STDDEV
|               .symbol index KCOLOR_NORM_2STDDEV
|               .symbol index KCOLOR_NORM_3STDDEV
|               .symbol index KCOLOR_NORM_BESTGUESS
|               .symbol index KCOLOR_NORM_NONE
| Resource:     colorNormMethod
------------------------------------------------------------------------*/
#define XVW_COLOR_NORM_METHOD    "colorNormMethod"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_RED_FUNCTION
| Description: 	With images having colormaps made up of more than three columns,
|		such as those produced by clustering algorithms, it is often 
|		informative to be able to apply a function to the values in 
|		those map columns in order to define the values that will be 
|		displayed as each of the red, green, and blue columns.  For 
|		example, in an image having 6 map columns, you might define the
|		red intensity of pixel 10 as (map column 2)[10]+ (map column 3)
|		[10]/ (map column 4) [10].  In this case, the
|		.symbol index XVW_COLOR_RED_FUNCTION 
|		attribute may be set to the string defining the function
|		to be applied in order to produce the values that will be used 
|		as red.  Functions must have \fIonly\fP variable \fIM\fP, 
|		where \fIM\fP stands for "map column"; following each \fIM\fP 
|		must be a number \fIstarting at zero\fP and ranging to \fIN-1\fP
|		where \fIN\fP is the number of map columns available in the 
|		image.  For example, a valid function for red might be:
|		.br
|		(M2 - M3)/(M2 + M3)
|		.br
|		or
|		.br
|		(M0 + M1 + M2)/(M3 - M4)
|		.br
|		Equation evaluation follows the standard rules of precedence; 
|		evaluation proceeds from left to right, and use of parentheses 
|		is fully supported.
|		.sp
|		In a more simple use, the 
|		.symbol index XVW_COLOR_RED_FUNCTION 
|		may also be the variable
|		representing the map column which is to be used to specify the 
|		red values of the pixels in the image.  For example, if you 
|		wanted the fourth map column to define the red values of the 
|		pixels in the image, (remember that map column numbering 
|		begins at 0), you could set 
|		.symbol index XVW_COLOR_RED_FUNCTION to:
|		.br
|		M3
|		.br
|		You can also set the 
|		.symbol index XVW_COLOR_RED_FUNCTION 
|		to a constant, if desired.  For example, if you wanted the red
|		values of all the pixels in the image to be 200, you could set 
|		.symbol index XVW_COLOR_RED_FUNCTION 
|		to:
|		.br
|		200
|		.br
|		By default, the 
|		.symbol index XVW_COLOR_RED_FUNCTION 
|		attribute is set to \fIM0\fP, or the first map column;  thus, 
|		it produces the results that you would normally expect
|		for an image with a colormap where no function was being 
|		applied.
|
|    Type:      char *
| Default:      "M0"
| Legal Values: see description
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_RED_FUNCTION   "colorRedFunction"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_GREEN_FUNCTION
| Description:  This is the function defining the green value of each pixel in 
|		the image.  By default, the 
|		.symbol noindex XVW_COLOR_GREEN_FUNCTION 
|		attribute is set to \fIM1\fP, or the second map column;  thus, 
|		it produces the results that you would normally expect
|		for an image with a colormap, where no function was being 
|		applied.  See the explanation for 
|		.symbol noindex XVW_COLOR_RED_FUNCTION 
|		for more details.
| Type:         char *
| Default:      "M1"
| Legal Values: see description
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_GREEN_FUNCTION "colorGreenFunction"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_BLUE_FUNCTION
| Description:  This is the function defining the blue value of each pixel in 
|		the image.  By default, the 
|		.symbol noindex XVW_COLOR_BLUE_FUNCTION 
|		attribute is set to \fIM2\fP, or the third map column;  thus, 
|		it produces the results that you would normally expect
|		for an image with a colormap, where no function was being 
|		applied.  See the explanation for 
|		.symbol noindex XVW_COLOR_RED_FUNCTION 
|		for more details.
| Type:         char *
| Default:      "M2"
| Legal Values: see description
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_BLUE_FUNCTION  "colorBlueFunction"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_ALLOC_POLICY
|
| Description:  A \fIvisual\fP describes the characteristics of a virtual
|               colormap that is created for use with an application on a 
|               particular screen.  For a complete explanation of visuals, 
|               you are referred to Chapters 7.2, 7.3, and 7.4 of \fIThe 
|               XLib Programming Manual\fP, by Adrian Nye, published by
|               O'Reilly & Associates.  Depending on your particular 
|               workstation, you may have a DirectColor, GreyScale, 
|               PseudoColor, StaticColor, StaticGrey, or TrueColor visual.  
|               The former three visual types support two types of colorcells: 
|               \fIread only\fP and \fIread/write\fP.  The color in a read 
|               only cell is set once by one application, and from then on 
|               can be shared by multiple applications, but not changed.  A
|               read/write color cell can have its color changed at any time 
|               by the application that allocated it; however, it cannot be 
|               shared by other applications.  The latter three visual types 
|               are limited to read only colorcells.
|               .sp
|               By default, the color class will allocate color cells as read 
|               only.  If you set the
|               .symbol index XVW_COLOR_ALLOC_POLICY to
|               .symbol index KCOLOR_ALLOC_READWRITE ,
|               and the user is on a workstation that has a visual of type 
|               DirectColor, GreyScale, or PseudoColor (\f(CW% xdpyinfo\fP 
|               will indicate the type of visual on a workstation) the 
|               read/write method of color allocation will be used.  
|               .sp
|               There are tradeoffs that must be taken into account when 
|               setting the color allocation method;  again, you are referred 
|               to \fIThe XLib Programming Manual\fP for more information.
|               .sp
|               To summarize very briefly, the color class will be able to 
|               update the colors that are displayed in a subclassed object
|               such as an image or zoom object \fImuch\fP faster when a 
|               read/write color allocation method is used.  However, since 
|               color cells will not be shared with other applications, this 
|               may result in the annoying "techno-flashing" phenomenon where 
|               the colormaps of spectrum and any other applications that 
|               happen to be running at the same time are installed and 
|               de-installed as the pointer is moved in and out of the 
|               application's GUI.  Thus, if this application is the only 
|               application requiring a large number of colors that the user
|               happens to be running, use of the read/write color allocation 
|               scheme is recommended.  Technoflashing will increasingly become
|               a problem as the application has to compete with other 
|               applications for color.  Technoflashing is minimized by 
|               leaving an application with the read-only color allocation 
|               default behavior.  However, colors will be slower to refresh,  
|               particularly the colors of classes with a large number of 
|               clusters assigned.
|               .sp
|               The recommended solution is to use a logical command line 
|               argument for applications, which will allow the user to
|               change the value of the
|               .symbol index XVW_COLOR_ALLOC_POLICY
|               attribute.  At this time, it cannot be set via the app-defaults
|               file.
|             
| Type:         int
| Default:      .symbol index KCOLOR_ALLOC_READONLY
| Legal Values: .symbol index KCOLOR_ALLOC_READONLY
|               .symbol index KCOLOR_ALLOC_READWRITE
| Resource:     N/A
-----------------------------------------------------------------------*/
#define XVW_COLOR_ALLOC_POLICY   "colorAllocPolicy"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_PRIVATE_CMAP
|
| Description:  To summarize very briefly, the color class will be able to 
|               allocate more colors if a private colormap is created.  This
|               allows for a better representation.  However, since 
|               colormaps will not be shared with other applications, this 
|               may result in the annoying "techno-flashing" phenomenon where 
|               the colormaps of spectrum and any other applications that 
|               happen to be running at the same time are installed and 
|               de-installed as the pointer is moved in and out of the 
|               application's GUI.  Thus, if this application wants as many
|		colors to be allocated as possible, use of the private colormap
|		allocation scheme is recommended.  Technoflashing will
|		increasingly become a problem as the application has to
|		compete with other applications for color.  Technoflashing
|		is minimized by leaving an application with the private
|		colormap set to FALSE, since the default colormap will be
|		used.  However, colors displayed may not truly reflect their
|               real values.
|               .sp
|               The recommended solution is to use a logical command line 
|               argument for applications, which will allow the user to
|               change the value of the
|               .symbol index XVW_COLOR_PRIVATE_CMAP
|               attribute or set thru the app-defaults.
| Type:         int
| Default:      FALSE
| Legal Values: TRUE/FALSE
| Resource:     colorPrivateCmap
-----------------------------------------------------------------------*/
#define XVW_COLOR_PRIVATE_CMAP   "colorPrivateCmap"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_RED_MAPCOL
| Description:  An image with a simple colormap has three map columns 
|		associated with it, where the pixels in the image are used to 
|		index into the map columns; the first map column defines the 
|		red values, the second map column defines the green values, 
|		and the third map column defines the blue values.  In this way, 
|		the color for each pixel in the image is defined.   Some images,
|		however, have more than three map columns; for example, output 
|		images produced by clustering algorithms may have many map 
|		columns, none of which is necessarily associated only with 
|		red, green, or blue values.  When multiple map columns are 
|		present, it may be useful to view any of the map columns as 
|		red, green, or blue.
|		.sp
|		Thus, the 
|		.symbol noindex XVW_COLOR_RED_MAPCOL 
|		attribute specifies the index of the map column 
|		that is to be used to specify the red values for each of the 
|		pixels in the image.  Valid values are from \fI0\fP to
|		\fIN-1\fP, where \fIN\fP is the number of map columns 
|		contained with the image.
|		.sp
|		By default, 
|		.symbol noindex XVW_COLOR_RED_MAPCOL 
|		is set to 0, indicating the first map column;
|		thus, it produces the results that you would normally expect 
|		for an RGB image where only three map columns were present.
|		.sp
|		Note that setting 
|		.symbol noindex XVW_COLOR_RED_MAPCOL 
|		to the integer representing the desired
|		map column is the same as setting the 
|		.symbol index XVW_COLOR_RED_FUNCTION 
|		attribute to the variable \fIMx\fP, where \fIx\fP is the index 
|		of the desired map column.  For example, setting 
|		.symbol index XVW_COLOR_RED_MAPCOL 
|		to 5 is the same as setting 
|		.symbol noindex XVW_COLOR_RED_FUNCTION 
|		to "M5".
| Type:         int
| Default:      0
| Legal Values: valid map column index
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_RED_MAPCOL    "colorRedMapcol"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_GREEN_MAPCOL
| Description:  This attribute controls which map column is used to specify 
|		the green values of the pixels in the image;  see 
|		.symbol noindex XVW_COLOR_RED_MAPCOL 
|		for more details.
| Type:         int
| Default:      0
| Legal Values: valid map column index
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_GREEN_MAPCOL  "colorGreenMapcol"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_BLUE_MAPCOL
| Description:  This attribute controls which map column is used to specify 
|		the blue values of the pixels in the image;  see 
|		.symbol noindex XVW_COLOR_RED_MAPCOL 
|		for more details.
| Type:         int
| Default:      0
| Legal Values: valid map column index
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_BLUE_MAPCOL   "colorBlueMapcol"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_SAVEMAP
| Description:  This \fIaction attribute\fP allows you to save the colormap
|		associated with the displayed data object \fIby itself\fP;
|		that is, the data object written out will not contain
|		value, mask, time, or location data, but only the map data. 
| Type:         char *
| Default:      NULL 
| Legal Values: valid filename 
| Resource: 	N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_SAVEMAP   	"colorSavemap"

/*
 * this is a private attribute currently being used by the Image object
 */
#define XVW_COLOR_CURRENTMAP       "colorCurrentmap"


/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_CHANGE_RED_MAPCOL
|
| Description:  This is an \fIaction\fP attribute; i.e, it may only be used
|               with \fIxvw_set_attribute(s)()\fP.  Provide TRUE as the value.
|               .sp
|               This attribute provides a convenient, interactive front end 
|               that an application may use to allow the user to set the
|               .symbol index XVW_COLOR_RED_MAPCOL
|               attribute.  It will query the number of map columns associated 
|               with the image, and present the user with a pop-up list of the 
|               map columns available.  The user will be allowed to choose one 
|               of the map columns from the list;  then,
|               .symbol index XVW_COLOR_RED_MAPCOL
|               will be automatically set according to the choice made by the 
|               user.
|
| Type:         int
| Default:      N/A (action attribute)
| Legal Values: TRUE
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_CHANGE_RED_MAPCOL    "colorChangeRedMapcol"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_CHANGE_GREEN_MAPCOL
|
| Description:  This is an \fIaction\fP attribute; i.e, it may only be used 
|               with \fIxvw_set_attribute(s)()\fP.  Provide TRUE as the value. 
|               See
|               .symbol noindex XVW_COLOR_RED_MAPCOL
|               for more details.
|
| Type:         int
| Default:      N/A  (action attribute)
| Legal Values: TRUE
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_CHANGE_GREEN_MAPCOL  "colorChangeGreenMapcol"

/*------------------------------------------------------------------------
| Attribute:    XVW_COLOR_CHANGE_BLUE_MAPCOL
|
| Description:  This is an \fIaction\fP attribute; i.e, it may only be used 
|               with \fIxvw_set_attribute(s)()\fP.  Provide TRUE as the value. 
|               See
|               .symbol noindex XVW_COLOR_RED_MAPCOL
|               for more details.
|
| Type:         int
| Default:      N/A (action attribute)
| Legal Values: TRUE
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_COLOR_CHANGE_BLUE_MAPCOL   "colorChangeBlueMapcol"


/*
 *  values for XVW_COLOR_NORM_TYPE
 */
#define KCOLOR_NORM_GLOBAL	1
#define KCOLOR_NORM_LOCAL	2

/*
 *  values for XVW_COLOR_NORM_METHOD
 */
#define KCOLOR_NORM_MAXCOLORS 	1
#define KCOLOR_NORM_1STDDEV     2
#define KCOLOR_NORM_2STDDEV     3
#define KCOLOR_NORM_3STDDEV     4
#define KCOLOR_NORM_BESTGUESS   5
#define KCOLOR_NORM_NONE        6

/*
 *  values for XVW_COLOR_ALLOC_POLICY
 */
#define KCOLOR_ALLOC_READONLY     1
#define KCOLOR_ALLOC_READWRITE    2

/*
 *  Define the Class name, by which people should be externally referencing
 *  the Color Object by.
 */
#define ColorWidgetClass	xvwColorWidgetClass
#define ColorGadgetClass	xvwColorGadgetClass

    /*-------------------------------------*
    |       global variable declarations
    ---------------------------------------*/

/*
 *  XToolkit WidgetClass Declaration ColorGadgetClass
 */
extern   WidgetClass	ColorGadgetClass;

/*
 *  XToolkit WidgetClass Declaration ColorWidgetClass
 */
extern   WidgetClass	ColorWidgetClass;


#endif /* _xvisual_Color_h_ */
/* Don't add after this point */
