/*
  * Khoros: $Id: Indicator.h,v 1.5 1997/01/20 05:11:51 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: #defines and #includes for creating a
   >>>>		       indicator annotation.
   >>>>
   >>>>    Written By: John Salas & Mark Young
   >>>>
   >>>>          Date: Feb 05, 1993 13:46
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvisual_Indicator_h_
#define _xvisual_Indicator_h_

	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

/*
 *  Attribute definitions for Indicator Visual Object
 */

/*------------------------------------------------------------------------
| Attribute:    XVW_INDICATOR_CONSTRAINT
| Description:  Indicates whether x and/or y values are to be displayed
|		along with the indicator.  It is set specifying one of the 
|		allowed values or by OR'ing the x and y values to do both.  The 
|		.symbol index KINDICATOR_CONSTRAINT_X
|		value is used to specify that the indicator should be 
|		constrained in the horizontal direction; which means that no 
|		X position indication will be given.  The 
|		.symbol index KINDICATOR_CONSTRAINT_Y 
|		value is used to specify that the indicator should be 
|		constrained in the vertical direction; which means that
|		no Y position indication will be given.  
|		.symbol index KINDICATOR_CONSTRAINT_NONE ,
|		the default value, specifies that both vertical and horizontal 
|		indication will be given.
| Type:         int    
| Default:      .symbol index KINDICATOR_CONSTRAINT_NONE
| Legal Values: .symbol index KINDICATOR_CONSTRAINT_NONE
|               .symbol index KINDICATOR_CONSTRAINT_X
|               .symbol index KINDICATOR_CONSTRAINT_Y
| Resource:     indicatorConstraint
------------------------------------------------------------------------*/
#define XVW_INDICATOR_CONSTRAINT	"indicatorConstraint"

/*------------------------------------------------------------------------
| Attribute:    XVW_INDICATOR_XPOS
| Description:  This read-only attribute returns the stringvalue visual object
|               that displays the X position of the indicator.If the
|               .symbol noindex XVW_INDICATOR_SHOW_YPOS 
|               attribute is set to FALSE, the returned xvobject will be NULL.
| Type:         xvobject
| Default:      N/A  (read only)
| Legal Values: string xvobject
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_INDICATOR_XPOS		"indicatorXpos"

/*------------------------------------------------------------------------
| Attribute:    XVW_INDICATOR_YPOS
| Description:  This read-only attribute returns the stringvalue visual object
|               that displays the Y position of the indicator.  If the
|               .symbol noindex XVW_INDICATOR_SHOW_XPOS 
|               attribute is set to FALSE, the returned xvobject will be NULL.
| Type:         xvobject
| Default:      N/A (read only)
| Legal Values: string xvobject
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_INDICATOR_YPOS		"indicatorYpos"

/*------------------------------------------------------------------------
| Attribute:    XVW_INDICATOR_XPOS_VALUE
| Description:  This attribute is the x position of the indicator in world 
|		coordinates.
| Type:         double
| Default:      KMAXFLOAT
| Legal Values: any double world coordinate value
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_INDICATOR_XPOS_VALUE	"indicatorXposValue"

/*------------------------------------------------------------------------
| Attribute:    XVW_INDICATOR_YPOS_VALUE
| Description:  This attribute is the y position of the indicator in world 
|               coordinates.
| Type:         double
| Default:      KMAXFLOAT
| Legal Values: any double world coordinate value
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_INDICATOR_YPOS_VALUE	"indicatorYposValue"

/*------------------------------------------------------------------------
| Attribute:    XVW_INDICATOR_LINE
| Description: 	This attribute indicates whether a x and/or y line should be 
|		displayed along with the indicator.  It is set specifying one 
|		of the allowed values or by OR'ing the x and y values to do 
|		both.  The 
|		.symbol index KINDICATOR_HORIZONTAL
|		value specifies that a horizontal line should be displayed, 
|		spanning the width of the parent object.  The
|		.symbol index KINDICATOR_VERTICAL
|		value specifies that a vertical line should be displayed, 
|		spanning the height of the parent object.
| Type:         int
| Default:      .symbol index KINDICATOR_LINE_NONE
| Legal Values: .symbol index KINDICATOR_LINE_NONE
|               .symbol index KINDICATOR_LINE_VERTICAL
|               .symbol index KINDICATOR_LINE_HORIZONTAL
|               .symbol index KINDICATOR_LINE_BOTH
| Resource:     indicatorLine
------------------------------------------------------------------------*/
#define XVW_INDICATOR_LINE     		"indicatorLine"

/*------------------------------------------------------------------------
| Attribute:    XVW_INDICATOR_SHOW_XPOS
| Description:  Indicates whether the indicator should display its X position.
| Type:         int
| Default:      TRUE
| Legal Values: TRUE/FALSE
| Resource:     indicatorShowXpos
------------------------------------------------------------------------*/
#define XVW_INDICATOR_SHOW_XPOS     	"indicatorShowXpos"

/*------------------------------------------------------------------------
| Attribute:    XVW_INDICATOR_SHOW_YPOS
| Description:  Indicates whether the indicator should display its Y position.
| Type:         int
| Default:      TRUE
| Legal Values: TRUE/FALSE
| Resource:     indicatorShowYpos
------------------------------------------------------------------------*/
#define XVW_INDICATOR_SHOW_YPOS     	"indicatorShowYpos"

/*
 *  Value definitions for Indicator Visual Object
 */
#define KINDICATOR_CONSTRAINT_NONE	(0L)
#define KINDICATOR_CONSTRAINT_X		(1L << 0)
#define KINDICATOR_CONSTRAINT_Y		(1L << 1)

#define KINDICATOR_LINE_NONE		0
#define KINDICATOR_LINE_VERTICAL	1
#define KINDICATOR_LINE_HORIZONTAL	2
#define KINDICATOR_LINE_BOTH		3

/*
 *  Define the Class name, by which people should be externally referencing
 *  the Indicator Object by.
 */
#define IndicatorGadgetClass	xvwIndicatorGadgetClass

    /*-------------------------------------*
    |       global variable declarations
    ---------------------------------------*/

/*
 *  XToolkit WidgetClass Declaration IndicatorGadgetClass
 */
extern   WidgetClass     IndicatorGadgetClass;


    /*-------------------------------------*
    |       routine definitions
    ---------------------------------------*/

/*
 *  Function definitions for Indicator Visual Object
 */
xvobject xvw_create_indicator PROTO((xvobject, char *));


#endif /* _xvisual_Indicator_h_ */
/* Don't add after this point */
