/*
  * Khoros: $Id: Palette.h,v 1.6 1997/01/20 05:11:57 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: #defines and #includes for creating a
   >>>>		       Palette widget.
   >>>>
   >>>>    Written By: Mark Young
   >>>>
   >>>>          Date: Nov 16, 1992 16:20
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvisual_Palette_h_
#define _xvisual_Palette_h_

    /*-------------------------------------*
    |       #defines 
    ---------------------------------------*/

/*
 *  Attribute definitions for Palette Visual Object
 */

/*------------------------------------------------------------------------
| Attribute:    XVW_PALETTE_CALLBACK
|
| Description:  If desired, \fIxvw_add_callback()\fP or 
|               \fIxvw_insert_callback()\fPmay be used to install
|               a callback on the palette object which will be fired when
|               the user selects a color cell from the palette object.
|               When calling \fIxvw_add_callback()\fP, pass this attribute
|               directly, as in
|
|               .begin code
|               xvw_add_callback(palette_object, XVW_PALETTE_CALLBACK,
|                                palette_callback, client_data);
|               .end code
|
|		The index of the colorcell that was selected by the user
|		is passed into the callback as the call_data; this integer
|		\fIpointer\fP must be cast correctly before use. 
|		It is the responsibility of the palette callback to keep a
|		list of the color cells that have been selected (if this
|		information will be needed).  The callback must also set the 
|		attributes XVW_PALETTE_ADD to select (stipple) a color cell
|		and XVW_PALETTE_DELETE to un-select (un-stipple) a color cell.
|		The following code segment gives an example of a callback
|		that might be installed on a palette object:
|
|               .begin code
|		static void test_callback(
|		   xvobject palette,
|		   kaddr    client_data,
|		   kaddr    call_data)
|		{
|		    static int *indexes = NULL;
|		    static int index_num = 0;
|		
|		    int *indx = (int *) call_data;
|		
|		    if (karray_locate(indexes, KINT, *indx, index_num) == -1)
|                   {
|            		xvw_set_attribute(palette, XVW_PALETTE_ADD, *indx);
|            		kfprintf(kstderr, "adding cell %d to cell list\n",
|                                 *indx);
|            		indexes = karray_add(indexes, KINT, *indx, index_num++);
|
|        	    }
|        	    else
|        	    {
|            	    	xvw_set_attribute(palette, XVW_PALETTE_DELETE, *indx);
|            	        kfprintf(kstderr, "deleting cell %d from cell list\n",
|                     		 *indx);
|            		indexes = karray_delete(indexes, KINT, *indx, index_num--);
|         	    }
|		}
|
| Type:         kfunc_void
| Default:      NULL
| Legal Values: callback function, in the form:
|               .begin code
|               void palette_callback
|                   xvobject object,
|                   kaddr client_data,
|                   kaddr call_data)
|               .end code
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PALETTE_CALLBACK	"paletteCallback"

/*------------------------------------------------------------------------
| Attribute:    XVW_PALETTE_TYPE
| Description:  This attribute specifies how the palette will be displayed; 
|		it may be one of:
|		.symbol index PALETTE_TYPE_COLORBAR
|		("Color Bar" on the menuform)
|		for a linearly increasing color bar.
|		.br
|		.symbol index KPALETTE_TYPE_COLORCELL
|		("Color Cell" on the menuform)
|		for a palette of rectangular color cells.
|		.br
|		or
|		.br
|		.symbol index PALETTE_TYPE_COLORWHEEL
|		("Color Wheel" on the menuform)
|		for a pie chart of colors.
|
| Type:         int
| Default:      .symbol index KPALETTE_TYPE_COLORBAR
| Legal Values: .symbol index KPALETTE_TYPE_COLORBAR
|               .symbol index KPALETTE_TYPE_COLORCELL
|               .symbol index KPALETTE_TYPE_COLORWHEEL
| Resource:     paletteType
------------------------------------------------------------------------*/
#define XVW_PALETTE_TYPE	"paletteType"

/*------------------------------------------------------------------------
| Attribute:    XVW_PALETTE_INDEXLIST
|
| Description:  This \fIread-only\fP attribute allows you to obtain the
|               palette list (ie, those data values (pixels) in the palette
|		that are currently highlighted).  By default, there is no
|		palette list (ie, no pixels in the palette are highlighted).
|
| Type:         int *
| Default:      N/A
| Legal Values: integer array of image pixel values
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PALETTE_INDEXLIST         "paletteIndexlist"

/*------------------------------------------------------------------------
| Attribute:    XVW_PALETTE_INDEXNUM
|
| Description:  This is \fIread-only\fP attribute allows you to obtain the
|               number of image data values (pixels) currently highlighted
|               in the palette object. Note that this is the size
|               of the array returned by
|               .symbol noindex XVW_PALETTE_INDEXLIST .
|
| Type:         int
| Default:      N/A
| Legal Values: size of the array specified by
|               .symbol index XVW_PALETTE_INDEXLIST
|
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PALETTE_INDEXNUM          "paletteIndexnum"

/*------------------------------------------------------------------------
| Attribute:    XVW_PALETTE_ADD
|
| Description:  Use of this \fIwrite-only\fP attribute adds the specified
|               image data value (pixel) to \fIpalette list\fP (the list
|               of pixels highlighted in the palette).
|
| Type:         int
| Default:      N/A
| Legal Values: valid pixel value of the displayed data object
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PALETTE_ADD               "paletteAdd"

/*------------------------------------------------------------------------
| Attribute:    XVW_PALETTE_DELETE
|
| Description:  Use of this \fIwrite-only\fP attribute deletes
|               the image data value (pixel) from the \fIpalette list\fP
|               (the list of pixels highlighted in the palette).
|
| Type:         int
| Default:      N/A (action attribute)
| Legal Values: valid pixel value of the displayed data object
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PALETTE_DELETE            "paletteDelete"

/*------------------------------------------------------------------------
| Attribute:    XVW_PALETTE_CLEAR
|
| Description:  This \fIaction\fP attribute clears the \fIpalette list\fP
|               so that no pixels are highlighted in the palette.
|
| Type:         int
| Default:      N/A (action attribute)
| Legal Values: TRUE
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PALETTE_CLEAR             "paletteClear"


/*
 *  Palette type for the controlling how the palette should be displayed.
 *  The choices are rectangular, gridded, spherical...
 */
#define KPALETTE_TYPE_COLORBAR	 1
#define KPALETTE_TYPE_COLORCELL	 2
#define KPALETTE_TYPE_COLORWHEEL 3

/*
 *  Define the Class name, by which people should be externally referencing
 *  the Palette Object by.
 */
#define PaletteWidgetClass	xvwPaletteWidgetClass

    /*-------------------------------------*
    |       global variable declarations
    ---------------------------------------*/

/*
 *  XToolkit Widget Declaration for PaletteWidgetClass
 */
extern   WidgetClass     PaletteWidgetClass;


    /*-------------------------------------*
    |       routine definitions
    ---------------------------------------*/

/*
 *  Function definitions for Palette Visual Object
 */
xvobject xvw_create_palette  PROTO((xvobject, char *));


#endif /* _xvisual_Palette_h_ */
/* Don't add after this point */
