/*
  * Khoros: $Id: PrintMapVal.h,v 1.5 1997/01/20 05:12:11 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: #defines and #includes for creating a
   >>>>		       PrintMapVal widget.
   >>>>
   >>>>    Written By: Mark Young & Danielle Argiro
   >>>>
   >>>>          Date: Jun 26, 1994
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvisual_PrintMapVal_h_
#define _xvisual_PrintMapVal_h_

	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

/*
 *  Attribute definitions for PrintMapVal Visual Object
 */

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTMAPVAL_WIDTH
| Description:  Specifies the number of map column values that should be
|		displayed in the horizontal direction on the printmapval 
|		display.
| Type:         int
| Default:      7
| Legal Values: 1 <= width <= image width
| Resource:     printmapvalWidth
------------------------------------------------------------------------*/
#define XVW_PRINTMAPVAL_WIDTH		"printmapvalWidth"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTMAPVAL_HEIGHT
| Description:  Specifies the number of map column values that should be
|		displayed in the vertical direction on the printmapval 
|		display.
| Type:         int
| Default:      9
| Legal Values: 1 <= height <= image height
| Resource:     printmapvalHeight
------------------------------------------------------------------------*/
#define XVW_PRINTMAPVAL_HEIGHT		"printmapvalHeight"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTMAPVAL_XPOSITION
| Description:  This is the X position in the image corresponding to the
|		map value that appears in the upper left hand corner of the
|		printmapval grid.
| Type:         int 
| Default:      0 
| Legal Values: 0 - image width 
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PRINTMAPVAL_XPOSITION	"printmapvalXposition"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTMAPVAL_YPOSITION
| Description:  This is the Y position in the image corresponding to the 
|               map value that appears in the upper left hand corner of the
|               printmapval grid.
| Type:         int
| Default:      0 
| Legal Values: 0 - image height 
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PRINTMAPVAL_YPOSITION	"printmapvalYposition"

/*
 * this attribute will allow the user to edit the map value directly.
 * it is not supported as of yet.
 */
#define XVW_PRINTMAPVAL_EDITABLE	"printmapvalEditable" 

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTMAPVAL_OBJECT
|
| Description: 	This is the data object (kobject) containing the image for 
|		which map values are to be displayed.
|               Note that this attribute is mutually exclusive with
|               .symbol index XVW_PRINTMAPVAL_FILENAME ;
|               use one or the other, not both.
|
| Type:         kobject
| Default:      NULL
| Legal Values: valid data object
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PRINTMAPVAL_OBJECT		"printmapvalObject"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTMAPVAL_FILENAME
|
| Description:  The file containing the data object for which the map values 
|		are to be displayed.
|               Note that this attribute is mutually exclusive with
|               .symbol index XVW_PRINTMAPVAL_OBJECT ;
|               use one or the other, not both.
|
| Type:         char *
| Default:      NULL
| Legal Values: valid input filename
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PRINTMAPVAL_FILENAME	"printmapvalFilename"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTMAPVAL_SHOWCOLOR
|
| Description: 	This attribute indicates whether or not the color of the pixel 
|		under the pointer should be displayed as the background color 
|		for the labelstring object in which the map values are 
|		displayed.  When set to FALSE, the background will be black.
|
| Type:         int
| Default:      FALSE
| Legal Values: TRUE/FALSE
| Resource:     printmapvalShowcolor
------------------------------------------------------------------------*/
#define XVW_PRINTMAPVAL_SHOWCOLOR	"printmapvalShowcolor"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTMAPVAL_UPDATEMODE
|
| Description:  This attribute indicates whether the printmapval display is to 
|		be updated continuously as the pointer moves across the image, 
|		or not until the button is clicked at a particular location 
|		in the image.  Values include:
|		.symbol index KPRINTMAPVAL_UM_CONTINUOUS , 
|		("Continuous" on the menuform), or
|		.symbol index KPRINTMAPVAL_UM_BUTTONPRESS
|		("ButtonPress" on the menuform)
|
| Type:         int
| Default:      .symbol index KPRINTMAPVAL_UM_CONTINUOUS
| Legal Values: .symbol index KPRINTMAPVAL_UM_CONTINUOUS
|               .symbol index KPRINTMAPVAL_UM_BUTTONPRESS
| Resource:     printmapvalUpdatemode
------------------------------------------------------------------------*/
#define XVW_PRINTMAPVAL_UPDATEMODE	"printmapvalUpdatemode"

/*------------------------------------------------------------------------
| Attribute:   	XVW_PRINTMAPVAL_POLICY
|
| Description: 	This attribute indicates whether the printmapval object is 
|		to display the values from the colormap that is being used to 
|		display the data object (ie, the RGB values that may have been 
|		converted and/or normalized), or the values from the actual map
|		segment of the data object being displayed (ie, the map values 
|		before they are converted and/or normalized for use in defining
|		the colors that appear on the screen).  Values include:
|		.symbol index KPRINTMAPVAL_DISPLAYEDVALUES
|		("Values Associated With Displayed Color" on the menuform)
|		.br
|		.symbol index KPRINTMAPVAL_MAPDATAVALUES
|		("Actual Values from Map Data" on the menuform)
|
| Type:         int
| Default:      .symbol index KPRINTMAPVAL_DISPLAYEDVALUES
| Legal Values: .symbol index KPRINTMAPVAL_DISPLAYEDVALUES
|               .symbol index KPRINTMAPVAL_MAPDATAVALUES
| Resource:     printmapvalPolicy
------------------------------------------------------------------------*/
#define XVW_PRINTMAPVAL_POLICY	"printmapvalPolicy"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTMAPVAL_CLIPOBJ
|
| Description:  A clip mask is used to obscure portions of the displayed map 
|		values; if desired, this data object (kobject) attribute can be 
|		used to set a clip mask. When a clip mask is used 
|		with the printmapval object, the only map values that will be
|               displayed by the printmapval object will be those that 
|		correspond to a value data of (1) in the clip mask.  Map values
|		corresponding to a value of (0) in the clip mask will not be 
|		displayed. Note that ONLY images of data type bit can
|               be used as clip masks; if the input image to be used as a clip
|               mask is not of type bit, it will be internally converted to
|               type bit, without prompting or warning, before it is used.
|               Note that this conversion will not affect the original file in
|               any way.  If a clip mask currently being used is no longer
|               desired, set this attribute back to NULL.
|               Note that this attribute is mutually exclusive with
|               .symbol index XVW_PRINTMAPVAL_CLIPFILE ;
|               use one or the other, not both.
|
| Type:         kobject
| Default:      NULL
| Legal Values: valid data object defining clip maks
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PRINTMAPVAL_CLIPOBJ               "printmapvalClipobj"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTMAPVAL_CLIPFILE
| Description:  The name of a file containing a clip mask may be specified
|               using this attribute; the default is NULL. Note that this
|               attribute is mutually exclusive with
|               .symbol index XVW_PRINTPIXEL_CLIPOBJ ;
|               use one or the other, not both.  See the explanation of
|               .symbol noindex XVW_PRINTPIXEL_CLIPOBJ
|               for more details on the clip mask.
| Type:         char *
| Default:      NULL
| Legal Values: valid name of input file containing clip mask
| Resource:     N/A
-----------------------------------------------------------------------*/
#define XVW_PRINTMAPVAL_CLIPFILE              "printmapvalClipfile"




/*
 *  Value definitions for XVW_PRINTMAPVAL_UPDATEMODE
 */
#define KPRINTMAPVAL_UM_CONTINUOUS      0
#define KPRINTMAPVAL_UM_BUTTON_PRESS    1

/*
 *  Value definitions for XVW_PRINTMAPVAL_POLICY
 */
#define KPRINTMAPVAL_DISPLAYEDVALUES  1
#define KPRINTMAPVAL_MAPDATAVALUES    2

/*
 *  Define the Class name, by which people should be externally referencing
 *  the PrintMapVal Object by.
 */
#define PrintMapValWidgetClass	xvwPrintMapValWidgetClass

	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/

/*
 *  XToolkit Widget Declaration for PrintMapValWidgetClass
 */
extern   WidgetClass     PrintMapValWidgetClass;


	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/*
 *  Function definitions for PrintMapVal Visual Object
 */
xvobject xvw_create_printmapval  PROTO((xvobject, char *));


#endif /* _xvisual_PrintMapVal_h_ */
/* Don't add after this point */
