/*
  * Khoros: $Id: PrintPixel.h,v 1.6 1997/01/20 05:12:14 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: #defines and #includes for creating a
   >>>>		       PrintPixel widget.
   >>>>
   >>>>    Written By: Mark Young
   >>>>
   >>>>          Date: Nov 16, 1992 16:20
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvisual_PrintPixel_h_
#define _xvisual_PrintPixel_h_

	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

/*
 *  Attribute definitions for PrintPixel Visual Object
 */

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTPIXEL_WIDTH
| Description:  Specifies the number of pixel values that should be
|		displayed in the horizontal direction on the printpixel display.
| Type:         int
| Default:      7
| Legal Values: 1 <= width <= image width
| Resource:     printpixelWidth
------------------------------------------------------------------------*/

#define XVW_PRINTPIXEL_WIDTH		"printpixelWidth"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTPIXEL_HEIGHT
| Description:  Specifies the number of pixel values that should be
|		displayed in the vertical direction on the printpixel display.
| Type:         int
| Default:      9
| Legal Values: 1 <= height <= image height
| Resource:     printpixelHeight
------------------------------------------------------------------------*/
#define XVW_PRINTPIXEL_HEIGHT		"printpixelHeight"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTPIXEL_XPOSITION
| Description:  This is the X position in the image corresponding to the
|               map value that appears in the upper left hand corner of the
|               printmapval grid.
| Type:         int
| Default:      0
| Legal Values: 0 - image width
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PRINTPIXEL_XPOSITION       "printpixelXposition"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTPIXEL_YPOSITION
| Description:  This is the Y position in the image corresponding to the
|               map value that appears in the upper left hand corner of the
|               printmapval grid.
| Type:         int
| Default:      0
| Legal Values: 0 - image height
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PRINTPIXEL_YPOSITION       "printpixelYposition"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTPIXEL_DEPTH_OFFSET
|
| Description:  When the input data object has depth > 1, this attribute
|               may be used to specify the depth offset at which the
|               pixel values are to be extracted (the printpixel object
|		always operates on a (width x height) region).
|
| Type:         int
| Default:      0
| Legal Values: 0 < value < depth of data object value segment
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PRINTPIXEL_DEPTH_OFFSET            "printpixelDepthOffset"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTPIXEL_TIME_OFFSET
|
| Description:  When the input data object has time > 1, this attribute
|               may be used to specify the time offset at which the
|               pixel values are to be extracted (the printpixel object
|               always operates on a (width x height) region).
|
| Type:         int
| Default:      0
| Legal Values: value < time size of data object value segment
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PRINTPIXEL_TIME_OFFSET            "printpixelTimeOffset"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTPIXEL_ELEMENTS_OFFSET
|
| Description:  When the input data object has elements > 1, this attribute
|               may be used to specify the elements offset at which the
|               pixel values are to be extracted (the printpixel object
|               always operates on a (width x height) region).
|
| Type:         int
| Default:      0
| Legal Values: value < number of elements in data object value segment
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PRINTPIXEL_ELEMENTS_OFFSET            "printpixelElementsOffset"

/*
 * this attribute will allow the user to edit the pixel value directly.
 * it is not supported as of yet.
 */
#define XVW_PRINTPIXEL_EDITABLE		"printpixelEditable"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTPIXEL_OBJECT
|
| Description:  This is the data object (kobject) containing the image for 
|		which pixel values are to be displayed.
|               Note that this attribute is mutually exclusive with
|               .symbol index XVW_PRINTPIXEL_FILENAME ;
|               use one or the other, not both.
|
| Type:         kobject
| Default:      NULL
| Legal Values: valid data object
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PRINTPIXEL_OBJECT		"printpixelObject"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTPIXEL_FILENAME
|
| Description:  The file containing the data object for which the pixel 
|		values are to be displayed.
|                Note that this attribute is mutually exclusive with
|               .symbol index XVW_PRINTPIXEL_OBJECT ;
|               use one or the other, not both.
|
| Type:         char *
| Default:      NULL
| Legal Values: valid input filename
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PRINTPIXEL_FILENAME		"printpixelFilename"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTPIXEL_SHOWCOLOR
| Description:  Indicates whether or not the color of the pixel under the
|		pointer should be displayed as the background color for the 
|		label object in which the pixel values are displayed.  
|		When set to FALSE, the background will be black.
| Type:         int
| Default:      FALSE
| Legal Values: TRUE/FALSE
| Resource:     printpixelShowcolor
------------------------------------------------------------------------*/
#define XVW_PRINTPIXEL_SHOWCOLOR	"printpixelShowcolor"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTPIXEL_UPDATEMODE
| Description:	Indicates whether the printmapval display is to be updated
|		continuously as the pointer moves across the image, or not 
|		until the button is clicked at a particular location in the 
|		image.
| Type:         int
| Default:      .symbol index KPRINTPIXEL_UM_CONTINUOUS
| Legal Values: .symbol index KPRINTPIXEL_UM_CONTINUOUS
|               .symbol index KPRINTPIXEL_UM_BUTTONPRESS
| Resource:     printpixelUpdatemode
-----------------------------------------------------------------------*/
#define XVW_PRINTPIXEL_UPDATEMODE	"printpixelUpdatemode"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTPIXEL_CLIPOBJ
|
| Description:  A clip mask is used to obscure portions of the displayed pixels;
|               if desired, this data object (kobject) attribute can be used
|               to set a clip mask for the data. When a clip mask is used with
|		the printpixel object, the only pixel values that will be 
|		displayed by the printpixel object will be those that correspond
|		to a pixel value of (1) in the clip mask.  Pixels corresponding
|		to a value of (0) in the clip mask will not be displayed.
|               Note that ONLY images of data type bit can
|               be used as clip masks; if the input image to be used as a clip
|               mask is not of type bit, it will be internally converted to
|               type bit, without prompting or warning, before it is used.
|               Note that this conversion will not affect the original file in
|               any way.  If a clip mask currently being used is no longer
|               desired, set this attribute back to NULL.
|               Note that this attribute is mutually exclusive with
|               .symbol index XVW_PRINTPIXEL_CLIPFILE ;
|               use one or the other, not both.
|
| Type:         kobject
| Default:      NULL
| Legal Values: valid data object defining clip maks
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PRINTPIXEL_CLIPOBJ               "printpixelClipobj"

/*------------------------------------------------------------------------
| Attribute:    XVW_PRINTPIXEL_CLIPFILE
| Description:  The name of a file containing a clip mask may be specified
|               using this attribute; the default is NULL. Note that this
|               attribute is mutually exclusive with
|               .symbol index XVW_PRINTPIXEL_CLIPOBJ ;
|               use one or the other, not both.  See the explanation of
|               .symbol noindex XVW_PRINTPIXEL_CLIPOBJ
|               for more details on the clip mask.
| Type:         char *
| Default:      NULL
| Legal Values: valid name of input file containing clip mask
| Resource:     N/A
-----------------------------------------------------------------------*/
#define XVW_PRINTPIXEL_CLIPFILE              "printpixelClipfile"


/*
 *  Value definitions for PrintPixel Visual Object
 */
#define KPRINTPIXEL_UM_CONTINUOUS      0
#define KPRINTPIXEL_UM_BUTTON_PRESS    1

/*
 *  Define the Class name, by which people should be externally referencing
 *  the PrintPixel Object by.
 */
#define PrintPixelWidgetClass	xvwPrintPixelWidgetClass

	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/

/*
 *  XToolkit Widget Declaration for PrintPixelWidgetClass
 */
extern   WidgetClass     PrintPixelWidgetClass;


	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/*
 *  Function definitions for PrintPixel Visual Object
 */
xvobject xvw_create_printpixel  PROTO((xvobject, char *));


#endif /* _xvisual_PrintPixel_h_ */
/* Don't add after this point */
