/*
  * Khoros: $Id: String.h,v 1.7 1997/01/20 05:12:22 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: #defines and #includes for creating a
   >>>>		       graphical string annotation.
   >>>>
   >>>>    Written By: John Salas and Mark Young
   >>>>
   >>>>          Date: Aug 05, 1992 14:42
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvisual_String_h_
#define _xvisual_String_h_

	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

/*
 *  Attribute definitions for String Visual Object
 */

/*------------------------------------------------------------------------
| Attribute:    XVW_STRING_XPLACEMENT
| Description:  This double value specifies the x location of the lower 
|		left hand corner of the string in world coordinates.
| Type:         double
| Default:      0.0
| Legal Values: any double value
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_STRING_XPLACEMENT	   "stringXplacement"

/*------------------------------------------------------------------------
| Attribute:    XVW_STRING_YPLACEMENT
| Description:  This double value specifies the y location of the lower 
|               left hand corner of the string in world coordinates.
| Type:         double
| Default:      0.0
| Legal Values: any double value
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_STRING_YPLACEMENT	   "stringYplacement"

/*------------------------------------------------------------------------
| Attribute:    XVW_STRING_JUSTIFICATION
| Description:  This attribute specifies the justification of the string 
|		\fIwith respect to the width and height of the string object\fP.
|		Thus, if the string object has its string set to "hello", with 
|		a width of 5 and a height of 1, the justification will have 
|		little (if any) effect, as the string has no room to "move" 
|		within its boundaries.  However, if the width and height are 
|		set to \fIlarger\fP than what is necessary to display the 
|		string, the justification will take effect.  Justification can 
|		be set to any of the following values:
|		.br
|		.symbol index KSTRING_JUSTIFY_CENTER 
|		("Center" on the menuform)
|		.br
|		.symbol index KSTRING_JUSTIFY_TOP 
|		("Top" on the menuform)
|		.br
|		.symbol index KSTRING_JUSTIFY_BOTTOM 
|		("Bottom" on the menuform)
|		.br
|		.symbol index KSTRING_JUSTIFY_LEFT 
|		("Left" on the menuform)
|		.br
|		.symbol index KSTRING_JUSTIFY_RIGHT 
|		("Right" on the menuform)
|		.br
|		.symbol index KSTRING_JUSTIFY_TOPRIGHT 
|		("TopRight" on the menuform)
|		.br
|		.symbol index KSTRING_JUSTIFY_TOPLEFT 
|		("TopLeft" on the menuform)
|		.br
|		.symbol index KSTRING_JUSTIFY_BOTTOMRIGHT 
|		("BottomRight" on the menuform)
|		.br
|		.symbol index KSTRING_JUSTIFY_BOTTOMLEFT 
|		("BottomLeft" on the menuform)
| Type:         int
| Default:      .symbol index KSTRING_JUSTIFY_LEFT
| Legal Values: .symbol index KSTRING_JUSTIFY_CENTER
|               .symbol index KSTRING_JUSTIFY_TOP
|               .symbol index KSTRING_JUSTIFY_BOTTOM
|               .symbol index KSTRING_JUSTIFY_LEFT
|               .symbol index KSTRING_JUSTIFY_RIGHT
|               .symbol index KSTRING_JUSTIFY_TOPRIGHT
|               .symbol index KSTRING_JUSTIFY_TOPLEFT
|               .symbol index KSTRING_JUSTIFY_BOTTOMRIGHT
|               .symbol index KSTRING_JUSTIFY_BOTTOMLEFT
| Resource:     stringJustification
-----------------------------------------------------------------------*/
#define XVW_STRING_JUSTIFICATION   "stringJustification"

/*------------------------------------------------------------------------
| Attribute:    XVW_STRING_STRING
| Description:  This is the string to display in the string object.
| Type:         char *
| Default:      NULL
| Legal Values: any printable string
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_STRING_STRING	   "stringString"

/*------------------------------------------------------------------------
| Attribute:    XVW_STRING_STYLE
| Description:  The string visual object supports three styles: plain, 
|		emphasized, and embossed.  When this attribute is set to 
|		.symbol index KSTRING_STYLE_PLAIN ,
|		the string is drawn once, in the foreground color.
|		When this attribute is set to
|		.symbol index KSTRING_STYLE_EMPHASIZE , 
|		the string is emphasized. This means it is double-drawn using 
|		both the foreground and highlight colors;  this causes it to 
|		take on a 3D, or "emphasized" effect.  
|		When this attribute is set to 
|		.symbol index KSTRING_STYLE_EMBOSSED_IN ,
|		the string will appear embossed "into" the background; both
|		the foreground and highlight colors are used in embossing.
|		.symbol index KSTRING_STYLE_EMBOSSED_OUT 
|		also causes the string to appear embossed, but the string
|		will seem to come "out of" rather than "into" the background.
| Type:		int
| Default:      .symbol index KSTRING_STYLE_PLAIN
| Legal Values: .symbol index KSTRING_STYLE_PLAIN
|		.symbol index KSTRING_STYLE_EMPHASIZE
|		.symbol index KSTRING_STYLE_EMBOSSED_IN
|		.symbol index KSTRING_STYLE_EMBOSSED_OUT
| Resource:     stringEmphasize
------------------------------------------------------------------------*/
#define XVW_STRING_STYLE	   "stringStyle"

/*------------------------------------------------------------------------
| Attribute:    XVW_STRING_HIGHLIGHT_COLOR
| Description:  When the 
|		.symbol index XVW_STRING_STYLE
|		attribute is used to set the string style to
|               .symbol index KSTRING_STYLE_EMPHASIZE ,
|               .symbol index KSTRING_STYLE_EMBOSSED_IN ,
|		or
|               .symbol index KSTRING_STYLE_EMBOSSED_OUT ,
|		this attribute determines the additional color to be used
|		in the process of drawing the emphasized or embossed string.
| Type:         char *
| Default:      default bg color
| Legal Values: any valid color name: see /usr/lib/X11/rgb.txt or Section
|               7.1.1 of \fIThe XLib Programming Manual\fP by Adrian Nye
| Resource:     stringHighlightColor
------------------------------------------------------------------------*/
#define XVW_STRING_HIGHLIGHT_COLOR  "stringHighlightColor"

/*------------------------------------------------------------------------
| Attribute:    XVW_STRING_HIGHLIGHT_PIXEL
|
| Description:  The pixel value that defines the desired highlight color.
|
| Type:         unsigned long
| Default:      default bg pixel (XtDefaultBackground)
| Legal Values: any valid pixel value: see Section 7.3 and 7.4
|               of \fIThe XLib Programming Manual\fP by Adrian Nye
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_STRING_HIGHLIGHT_PIXEL              "stringHighlightPixel"



/*
 * Value definitions for String Visual Object
 */

/* text string justification */
#define KSTRING_JUSTIFY_CENTER        1
#define KSTRING_JUSTIFY_TOP           2  
#define KSTRING_JUSTIFY_BOTTOM        3  
#define KSTRING_JUSTIFY_LEFT          4
#define KSTRING_JUSTIFY_RIGHT         5  
#define KSTRING_JUSTIFY_TOPRIGHT      6 
#define KSTRING_JUSTIFY_TOPLEFT       7  
#define KSTRING_JUSTIFY_BOTTOMRIGHT   8  
#define KSTRING_JUSTIFY_BOTTOMLEFT    9  

/* string style */
#define KSTRING_STYLE_PLAIN        1
#define KSTRING_STYLE_EMPHASIZED   2
#define KSTRING_STYLE_EMBOSSED_IN  3
#define KSTRING_STYLE_EMBOSSED_OUT 4

/*
 *  Define the Class name, by which people should be externally referencing
 *  the String Object by.
 */
#define StringGadgetClass	xvwStringGadgetClass

	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/

/*
 *  XToolkit Widget Declaration for StringGadgetClass
 */
extern   WidgetClass     StringGadgetClass;


	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/*
 *  Function definitions for String Visual Object
 */
xvobject xvw_create_string PROTO((xvobject, char *));


#endif /* _xvisual_String_h_ */
/* Don't add after this point */
