 /*
  * Khoros: $Id: FinderList.h,v 1.7 1997/01/20 05:13:02 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Definitions for Khoros FinderList Object
   >>>>
   >>>>    Written By: Mark Young
   >>>>
   >>>>          Date: Jul 02, 1994
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvlang_FinderList_h_
#define _xvlang_FinderList_h_

        /*-------------------------------------*
        |       #defines
        ---------------------------------------*/


/*------------------------------------------------------------------------
| Attribute:    XVW_FINDERLIST_EXPRESSION
| Description:  The regular expression used within the finder list.  The
|		regular expression which uses the following syntax.
|
|		 .             Match any single character except newline
|
|		*             Match the preceding character or range
|		              of characters 0 or more times.  The
|		              matching includes items within a [...].
|		
|		[...] or [^..]  Matches any one character contained within
|		                the brackets. If the first character after
|		                the '[' is the ']', then it is included in
|		                the characters to match.  If the first
|		                character after the '[' is a '^', then it
|		                will match all characters NOT included in
|		                the [].  The '-' will indicate a range of
|		                characters.  For example, [a-z] specifies
|		                all characters between and including the
|		                ascii values 'a' and 'z'.  If the '-'
|		                follows the '[' or is right before the ']'
|		                then it is interpreted literally.
|		
|		^             If this is the first character of the
|		              regular expression, it matches the beginning
|		              of the line.
|		
|		$             If this is the last character of the
|		              regular expression, it matches the end of
|		              the line.
|		
|		\\            This escapes the meaning of a special character.
|
| Type:         char *
| Default:      NULL
| Legal Values: any regular expression
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_FINDERLIST_EXPRESSION  "finderlistExpression"

/*------------------------------------------------------------------------
| Attribute:    XVW_FINDERLIST_SELECT
|
| Description:  If desired,  \fIxvw_add_callback()\fP may be used to install
|		a callback on the finderlist object which will be fired each 
|		time the user selects an operator from the finderlist using a 
|		single button click.
|               When calling \fIxvw_add_callback()\fP, pass this attribute
|               directly, as in
|               .begin code
|               xvw_add_callback(finderlist, XVW_FINDERLIST_SELECT,
|                                finderlist_cb, client_data);
|               .end code
|               An \fIxvw_program_info\fP structure will be passed into
|		the callback as the \fIcall_data\fP; this structure consists
|		of the information necessary to create or print information
|		about the operator that was selected from the finder list.
|
|		The xvw_program_info() structure is defined as follows:
|		.begin code
|		typedef struct
|		{
|	            int  type;         { type of glyph; one of: 
|					 XVL_OPERATOR    (normal operator),
|				         XVL_PROCEDURE   (procedure),
|				         XVL_LOOP        (loop), or 
|				         XVL_CONDITIONAL (conditional) }
|	            char *tbname;      {  toolbox name       }
|	            char *oname;       {  object name        }
|	            char *category;    {  category           }
|	            char *subcategory; {  subcategory        }
|	            char *icon_name;   {  icon name          }
|	            char *panefile;    {  filename of pane file  }
|     	            char *short_desc;  {  short description  }
|	            char *workspace;   {  saved workspace file from which 
|	                                  glyph originated (if applicable)  }
|		} xvw_program_info;
|
|		Inside the callback, the call_data must be cast to the
|		xvw_program_info typedef before use, as follows:
|		.begin code
|		xvw_program_info *program_info = (xvw_program_info *) call_data;
|		.end code
|
| Type:         kfunc_void
| Default:      NULL
| Legal Values: N/A
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_FINDERLIST_SELECT	   "finderlistSelect"

/*------------------------------------------------------------------------
| Attribute:    XVW_FINDERLIST_CREATE
|
| Description:  If desired, a callback may be installed on the finderlist
|               object that will be fired each time the user doubleclicks
|		on an operator in the finderlist.  It may also be fired by
|		the application, using the
|		.symbol index XVW_FINDERLIST_OPEN
|		\fIaction attribute\fP.
|               When calling \fIxvw_add_callback()\fP, pass this attribute
|               directly, as in
|               .begin code
|               xvw_add_callback(finderlist, XVW_FINDERLIST_CREATE,
|                                finderlist_cb, client_data);
|               .end code
|               An \fIxvw_program_info\fP structure will be passed into
|               the callback as the \fIcall_data\fP; this structure consists
|               of the information necessary to create or print information
|               about the operator that was selected from the finder list.
|		See
|		.symbol index XVW_FINDERLIST_SELECT
|		for an explanation of the xvw_program_info structure.
|
| Type:         kfunc_void
| Default:      NULL
| Legal Values: N/A
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_FINDERLIST_CREATE	   "finderlistCreate"

/*------------------------------------------------------------------------
| Attribute:    XVW_FINDERLIST_OPEN
| Description:  This \fIaction attribute\fP causes the
|		.symbol index XVW_FINDERLIST_CREATE
|		to be fired with the operator that is currently selected 
|		in the finderlist.  See the explanation of
|		.symbol index XVW_FINDERLIST_CREATE for more details .
| Type:         int
| Default:      N/A
| Legal Values: TRUE
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_FINDERLIST_OPEN	   "finderlistOpen"

/*
 *  Define the Class name, by which people should be externally referencing
 *  the FinderList Object by.
 */
#define FinderListWidgetClass	xvwFinderListWidgetClass

        /*-------------------------------------*
        |       global variable declarations
        ---------------------------------------*/

/*
 *  XToolkit WidgetClass Declaration FinderListWidgetClass
 */
extern   WidgetClass     FinderListWidgetClass;


        /*-------------------------------------*
        |       routine definitions
        ---------------------------------------*/

/*
 *  Khoros Function Definition for FinderList Object
 */
xvobject xvw_create_finderlist PROTO((xvobject, char *));


#endif /* _xvlang_FinderList_h_ */
/* Do not add after this line */
