/*
  * Khoros: $Id: TextDisplay.h,v 1.5.2.1 1997/05/04 16:27:44 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Attributes for Text Display Object
   >>>>
   >>>>    Written By: Mark Young & Danielle Argiro
   >>>>
   >>>>          Date: Aug 25, 1994
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvobjects_TextDisplay_h_
#define _xvobjects_TextDisplay_h_


	/*-------------------------------------*
	|       #defines
	---------------------------------------*/

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXTDISPLAY_ASCII
|
| Description:  The XFontStruct structure defining the Ascii font to be 
|		used with "normal" text appearing in the text display object.  
|		The XFontStruct value for setting this attribute can be 
|		obtained with \fIXLoadQueryFont()\fP; a list of XFontStruct 
|		candidates may be obtained with \fIXListFontsWithInfo()\fP. 
|		For more information on these Xlib routines, see Section 6.2 
|		of \fIThe Xlib Programming Manual\fP by Adrian Nye.  Note that 
|		this attribute is mutually exclusive with
|		.symbol index XVW_TEXTDISPLAY_ASCII_FONTNAME ;
|		use one or the other, not both.
|
| Type:         XFontStruct
| Default:      see description
| Legal Values: see description
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_TEXTDISPLAY_ASCII		    "textdisplayAscii"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXTDISPLAY_ASCII_FONTNAME
|
| Description:  This attribute specifies the name of the Ascii font to be used
|		with "normal" text appearing in the text display object.  
|		The default font used for roman is:
|		.br
|		6x13
|		.br
|		Available fonts for a particular workstation can be obtained
|               from the command line with \f(CW%xlsfonts\fP, or from an
|               application program with the \fIXListFonts()\fP XLib routine.  
|
| Type:         char *
| Default:      see description 
| Legal Values: see description 
| Resource:	textdisplayAsciiFontname
------------------------------------------------------------------------*/
#define XVW_TEXTDISPLAY_ASCII_FONTNAME	    "textdisplayAsciiFontname"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXTDISPLAY_ROMAN
|
| Description:  The XFontStruct structure defining the Roman font to be 
|		used with "normal" text appearing in the text display object.  
|		The XFontStruct value for setting this attribute can be 
|		obtained with \fIXLoadQueryFont()\fP; a list of XFontStruct 
|		candidates may be obtained with \fIXListFontsWithInfo()\fP. 
|		For more information on these Xlib routines, see Section 6.2 
|		of \fIThe Xlib Programming Manual\fP by Adrian Nye.  Note that 
|		this attribute is mutually exclusive with
|		.symbol index XVW_TEXTDISPLAY_ROMAN_FONTNAME ;
|		use one or the other, not both.
|
| Type:         XFontStruct
| Default:      see description
| Legal Values: see description
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_TEXTDISPLAY_ROMAN		    "textdisplayRoman"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXTDISPLAY_ROMAN_FONTNAME
|
| Description:  This attribute specifies the name of the Roman font to be used
|		with "normal" text appearing in the text display object.  
|		The default font used for roman is:
|		.br
|		*-new century schoolbook-medium-r-normal--12-120-*
|		.br
|		Available fonts for a particular workstation can be obtained
|               from the command line with \f(CW%xlsfonts\fP, or from an
|               application program with the \fIXListFonts()\fP XLib routine.  
|
| Type:         char *
| Default:      see description 
| Legal Values: see description 
| Resource:	textdisplayRomanFontname
------------------------------------------------------------------------*/
#define XVW_TEXTDISPLAY_ROMAN_FONTNAME	    "textdisplayRomanFontname"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXTDISPLAY_ROFF
|
| Description:  If the file to appear in the text display object has \fIroff\fP
|		formatting commands, setting this attribute to TRUE will cause
|		those formatting commands to be interpreted and the text
|		formatted as specified.  If FALSE, the file to be displayed
|		is considered to be a plain ascii text file, and the text is
|		displayed exactly as it appears in the file.
|
| Type:         int
| Default:      TRUE 
| Legal Values: TRUE/FALSE 
| Resource:	textdisplayRoff
------------------------------------------------------------------------*/
#define XVW_TEXTDISPLAY_ROFF		"textdisplayRoff"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXTDISPLAY_INDENT
|
| Description:  The number of pixels to indent into the text display object
|		before printing text. 
|
| Type:         int
| Default:      10 
| Legal Values: values > 0 
| Resource:	textdisplayIndent
------------------------------------------------------------------------*/
#define XVW_TEXTDISPLAY_INDENT		"textdisplayIndent"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXTDISPLAY_FILE
|
| Description:  This is the open stream to the file to be displayed
|		in the text display object. Note that this attribute
|		is mutually exclusive with
|		.symbol index XVW_TEXTDISPLAY_FILENAME ;
|		use one or the other, not both.
|
| Type:         kfile
| Default:      NULL 
| Legal Values: A \fIkfile\fP pointer returned by \fIkfopen()\fP or 
|		another Khoros open data transport routine 
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_TEXTDISPLAY_FILE		"textdisplayFile"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXTDISPLAY_FILENAME
|
| Description:  This is the name of the file to be displayed in the 
|		display object. 
|
| Type:         char * 
| Default:      NULL 
| Legal Values: valid name of input file 
| Resource:	textdisplayFilename
------------------------------------------------------------------------*/
#define XVW_TEXTDISPLAY_FILENAME	"textdisplayFilename"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXTDISPLAY_CALLBACK
|
| Description:  If desired,  \fIxvw_add_callback()\fP may be used to install
|               a callback on the text display object that will be fired when 
|		the user clicks on a particular word that appears in the text 
|		being displayed.  
|		.br
|               When calling \fIxvw_add_callback()\fP, pass this attribute
|               directly, as in
|               .begin code
|               xvw_add_callback(textdisplayobj, XVW_TEXTDISPLAY_CALLBACK,
|                                textdisplay_cb, client_data);
|               .end code
|		.br
|		When this callback attribute is used, either the 
|		.symbol index XVW_TEXTDISPLAY_WORDCOLOR 
|		or the
|		.symbol index XVW_TEXTDISPLAY_WORDPIXEL
|		attribute should be used to set the color of the word to
|		something other than the rest of the text.  This will
|		indicate to the user that they can click on the word to
|		make something happen (what will happen is, of course, 
|		determined by the callback).
|		.br
|		In addition, it is also good to set either the
|		.symbol index XVW_TEXTDISPLAY_HIGHLIGHTCOLOR 
|               or the
|               .symbol index XVW_TEXTDISPLAY_HIGHLIGHTPIXEL
|		attribute to specify the color in which the word will
|		appear when it is actually clicked on.
|		.br
|		Note that the word that the user clicked on will be
|		passed as a string in the \fIcall_data\fP;  to access the
|		string inside the callback, it must first cast to a string,
|		as in:
|		\f(CWchar *word = (char *) calldata\fP
|
| Type:         kfunc_void
| Default:      NULL
| Legal Values: callback function, in the form:
|		.begin code
|		void callback_function
|		   xvobject object,
|		   kaddr client_data,
|		   kaddr call_data)
|		.end code
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_TEXTDISPLAY_CALLBACK	"textdisplayCallback"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXTDISPLAY_ADDTEXT
|
| Description:  This \fIaction attribute\fP adds text to the
|		text display.  The text is appended to the existing
|		list of text and is appropriately displayed after
|		being compiled.  The text passed via the \fIaction attribute\fP
|		is duplicated.
|
| Type:         char * 
| Default:      N/A 
| Legal Values: Any multiline text which is displayable
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_TEXTDISPLAY_ADDTEXT		"textdisplayAddtext"

/*------------------------------------------------------------------------
| Attribute:	XVW_TEXTDISPLAY_CLEARTEXT
|
| Description:  This \fIaction attribute\fP deletes all text from the
|		text display.
|		All text will be deleted from the text display list,
|		which will result in the text display defaulting to it's
|		initial state.
|
| Type:         int 
| Default:      N/A 
| Legal Values: TRUE 
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_TEXTDISPLAY_CLEARTEXT	"textdisplayCleartext"


/*
 *  Define the Class name, by which people should be externally referencing
 *  the TextDisplay Object by.
 */
#define TextDisplayWidgetClass	xvwTextDisplayWidgetClass


        /*-------------------------------------*
        |       global variable declarations
        ---------------------------------------*/

/*
 *  XToolkit WidgetClass Declaration TextDisplayWidgetClass
 */
extern   WidgetClass     TextDisplayWidgetClass;


        /*-------------------------------------*
        |       routine definitions
        ---------------------------------------*/

/*
 *  Khoros Function Definition for TextDisplay Object
 */
xvobject xvw_create_textdisplay PROTO((xvobject, char *));


#endif /* _xvobjects_TextDisplay_h_ */
/* Don't add after this point */
