 /*
  * Khoros: $Id: xvwidgets.h,v 1.21.2.3 1997/04/29 22:27:09 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: general #includes, #defines, and structures
   >>>>		       for all xvwidgets
   >>>>
   >>>>     Written By: Mark Young & Danielle Argiro & John Salas
   >>>>
   >>>>          Date: Aug 25, 1994
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvwidgets_xvwidgets_h_
#define _xvwidgets_xvwidgets_h_


	/*-------------------------------------*
	|       #include 
	---------------------------------------*/

#include <xvwidgets/attributes.h>
#include <xvwidgets/GenGeometry.h>
#include <xvwidgets/GenCharGeometry.h>
#include <xvwidgets/GenMisc.h>
#include <xvwidgets/Shell.h>


	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/

/*
 *  Declare a xvclass currently to be the WidgetClass structure defined by Xt.
 */
typedef WidgetClass xvclass;

        /*-------------------------------------*
        |       global variable declarations
        ---------------------------------------*/

	/*-------------------------------------*
	|       macros
	---------------------------------------*/

/************************************************************
*
*  Routine Name: xvw_getextent - get a rectangular extent
*
*       Purpose: Gets a rectangular extent for the given object.  This is
*		 a simplier interface to the xvw_getextent_rectangle()
*		 routine.
*
*         Input: object - the object in which to get extents, if NULL then
*                         from the RootWindow.
*        Output: x - the starting x position
*                y - the starting y position
*                w - the width of the extent
*                h - the height of the extent
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: 
*    Written By: Mark Young
*          Date: Jan 02, 1995
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: xvw_getextent(
*                !   xvobject object,
*                !   int      erase,
*                !   int      *x,
*                !   int      *y,
*                !   int      *w,
*                !   int      *h)
*
*************************************************************/
 
#define xvw_getextent(object, x, y, w, h) \
                xvw_getextent_rectangle(object, TRUE, x, y, w, h)

	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/* action.c */
void xvw_add_action        PROTO((xvobject, char *, 
				  void (*)(xvobject, kaddr, XEvent *),
				  kaddr, int));
void xvw_remove_action     PROTO((xvobject, char *,
				  void (*)(xvobject, kaddr, XEvent *),
				  kaddr));
void xvw_destroy_actions   PROTO((xvobject));

/* attributes.c */
int xvw_vset_attributes     PROTO((xvobject, kva_list));
int xvw_vget_attributes     PROTO((xvobject, kva_list));
int xvw_set_attributes      PROTO((xvobject, kvalist));
int xvw_get_attributes      PROTO((xvobject, kvalist));
int xvw_format_attribute    PROTO((xvobject, char *, char *, kvalist));
int xvw_init_attributes     PROTO((xvclass, xvattribute *, int,
				   xvattribute *, int, char *));
int xvw_define_attributes   PROTO((xvclass, kvalist));
int xvw_define_constraints  PROTO((xvclass, kvalist));

/* busy.c */
void xvw_busy		 PROTO((xvobject, int));

/* callback.c */
void xvw_add_callback    PROTO((xvobject, char *,
				void(*)(xvobject, kaddr, kaddr), kaddr));
void xvw_insert_callback PROTO((xvobject, char *, int,
				void(*)(xvobject, kaddr, kaddr), kaddr));
void xvw_remove_callback PROTO((xvobject, char *,
				void(*)(xvobject, kaddr, kaddr), kaddr));
void xvw_call_callback   PROTO((xvobject, char *));
void xvw_destroy_callbacks PROTO((xvobject));

/* converter.c */
void xvw_init_converters PROTO((void));

/* event.c */
void xvw_insert_event  	   PROTO((xvobject, unsigned long,
			          void (*)(xvobject, kaddr, XEvent *, int *),
			          kaddr, int));
void xvw_add_event    	   PROTO((xvobject, unsigned long,
			          void (*)(xvobject, kaddr, XEvent *, int *),
				  kaddr));
void xvw_remove_event 	   PROTO((xvobject, unsigned long,
			          void (*)(xvobject, kaddr, XEvent *, int *),
			          kaddr));
void xvw_destroy_events	   PROTO((xvobject));

/* fid.c */
void xvw_add_detectfid    PROTO((xvobject, int,
				void (*)(xvobject, int, kaddr), kaddr));
void xvw_remove_detectfid PROTO((xvobject, int,
				void (*)(xvobject, int, kaddr), kaddr));
void xvw_destroy_detectfids  PROTO((xvobject));

/* file.c */
void xvw_add_detectfile    PROTO((xvobject, char *, double,
				  int(*)(xvobject, char *, kaddr), kaddr));
void xvw_remove_detectfile   PROTO((xvobject, char *,
				  int(*)(xvobject, char *, kaddr), kaddr));
void xvw_destroy_detectfiles PROTO((xvobject));
void xvw_call_detectfile     PROTO((xvobject, char *));


/* general.c */
Window xvw_rootwindow    PROTO((xvobject));
xvobject xvw_toplevel    PROTO((xvobject));
Visual *xvw_visual       PROTO((xvobject));
Colormap xvw_colormap    PROTO((xvobject));

Widget xvw_widget        PROTO((xvobject));
Window xvw_window        PROTO((xvobject));
char *xvw_name           PROTO((xvobject));
xvclass xvw_class	 PROTO((xvobject));
xvobject xvw_object      PROTO((Widget));
xvobject xvw_create      PROTO((xvobject, int, int, String, xvclass));
int xvw_recreate         PROTO((xvobject, xvclass));
int xvw_reparent         PROTO((xvobject, xvobject));
xvobject xvw_duplicate   PROTO((xvobject));
xvobject xvw_parent      PROTO((xvobject));
void xvw_destroy         PROTO((xvobject));
void xvw_place		 PROTO((xvobject, xvobject, int, int));
void xvw_unmap           PROTO((xvobject));
void xvw_map             PROTO((xvobject));
void xvw_unmanage        PROTO((xvobject));
void xvw_manage          PROTO((xvobject));
void xvw_raise           PROTO((xvobject));
void xvw_lower           PROTO((xvobject));
void xvw_realize         PROTO((xvobject));
void xvw_unrealize       PROTO((xvobject));
void xvw_sensitive       PROTO((xvobject, int));
void xvw_refresh         PROTO((xvobject));
int xvw_geometry         PROTO((xvobject, int *, int *, unsigned int *,
                                unsigned int *, unsigned int *));
xvobject *xvw_children   PROTO((xvobject, xvclass, size_t *));
int xvw_numchildren	 PROTO((xvobject, xvclass));
int xvw_translate_coords PROTO((xvobject, int, int, int *, int *));

int xvw_check_realized   PROTO((xvobject));
int xvw_check_sensitive  PROTO((xvobject));
int xvw_check_mapped     PROTO((xvobject));
int xvw_check_visible    PROTO((xvobject));
int xvw_check_managed    PROTO((xvobject));
int xvw_check_menuactive PROTO((xvobject));
int xvw_check_menuexist  PROTO((xvobject));
int xvw_check_toplevel   PROTO((xvobject));
int xvw_check_localhost  PROTO((xvobject));
int xvw_check_subclass   PROTO((xvobject, xvclass));

/* initialize.c */
Display *xvw_initialize     PROTO((void (*)(void)));
Display *xvw_display        PROTO((xvobject));
XtAppContext xvw_appcontext PROTO((xvobject));
Screen *xvw_screen	    PROTO((xvobject));
int xvw_screennum	    PROTO((xvobject));
XFontStruct *xvw_font	    PROTO((xvobject));

/* menu.c */
int xvw_get_menuinfo     PROTO((xvobject, char *, kaddr));
int xvw_set_menuinfo     PROTO((xvobject, char *, kaddr));
void xvw_initinfo_menu   PROTO((kaddr (*)(xvobject, char *),
				int   (*)(xvobject, kform *),
				int   (*)(xvobject, kform *),
				int   (*)(xvobject, kform *)));
void xvw_update_menu     PROTO((xvobject));
void xvw_activate_menu   PROTO((xvobject));
void xvw_inactivate_menu PROTO((xvobject));
int xvw_activated_menu   PROTO((xvobject));

/* process.c */
pid_t xvw_spawn PROTO((char *));
int xvw_system  PROTO((char *));
int xvw_sleep   PROTO((double));
int xvw_waitpid PROTO((pid_t));
int xvw_check_process PROTO((pid_t, int *));

pid_t xvw_fork PROTO((char *,
		void (*)PROTO((int, kaddr)),
		void (*)PROTO((kfile *, kaddr, char *)),
		void (*)PROTO((kfile *, char *, char *, kaddr)),
		kaddr));

/* resources.c */
void xvw_load_resources PROTO((char *));

/* signal.c */
int xvw_kill_process PROTO((pid_t));
int xvw_remove_handler PROTO((pid_t));

pid_t xvw_add_handler  PROTO((void (*)PROTO((int, kaddr)),
		void (*)PROTO((kfile *, kaddr, char *)),
		void (*)PROTO((kfile *, char *, char *, kaddr)),
		kaddr, kfile *));
int xvw_setinfo_handler PROTO((pid_t,
		void (*)PROTO((int, kaddr)),
		void (*)PROTO((kfile *, kaddr, char *)),
		void (*)PROTO((kfile *, char *, char *, kaddr)),
		kaddr, kfile *));
int xvw_getinfo_handler PROTO((pid_t,
		void (**)PROTO((int, kaddr)),
		void (**)PROTO((kfile *, kaddr, char *)),
		void (**)PROTO((kfile *, char *, char *, kaddr)),
		kaddr *, kfile **));

/* timeout.c */
void xvw_add_timeout      PROTO((xvobject, double,
				void (*)(xvobject, kaddr, int *), kaddr));
void xvw_remove_timeout   PROTO((xvobject,
				void (*)(xvobject, kaddr, int *), kaddr));
void xvw_destroy_timeouts PROTO((xvobject));

/* dispatch.c */
void xvw_process_event   PROTO((void));
int  xvw_sync            PROTO((int));
int  xvw_clean_event_list PROTO((xvobject));

/* chargeom.c */
int  xvw_recompute_charfont PROTO((xvobject, int *, int *));

/* toplevel.c */
void xvw_add_toplevel          PROTO((xvobject));
void xvw_remove_toplevel       PROTO((xvobject));
void xvw_set_toplevel          PROTO((xvobject, xvobject));
xvobject xvw_get_toplevel      PROTO((xvobject));
xvobject *xvw_getall_toplevels PROTO((size_t *));

/* utilities.c */
Pixel    xvw_allocate_color PROTO((xvobject, char *));
Colormap xvw_copy_colormap  PROTO((xvobject));
void     xvw_cache_pixmap   PROTO((char *, char **));
char   **xvw_locate_pixmap  PROTO((char *));

/* wmprotocol.c */
void xvw_add_protocol    PROTO((xvobject, char *, 
				void (*)(xvobject, kaddr), kaddr));
void xvw_remove_protocol PROTO((xvobject, char *, 
				void (*)(xvobject, kaddr), kaddr));


/* extent.c */
XPoint *xvw_getextent_polyline	 PROTO((xvobject, int, int *));
XPoint *xvw_getextent_freehand 	 PROTO((xvobject, int, int *));
int xvw_getextent_line		 PROTO((xvobject, int, int *, int *, int *,
					int *));
int xvw_getextent_circle	 PROTO((xvobject, int, int *, int *,
					unsigned int *));
int xvw_getextent_rectangle	 PROTO((xvobject, int, int *, int *,
					unsigned int *, unsigned int *));
int xvw_getextent_ellipse	 PROTO((xvobject, int, int *, int *,
					unsigned int *, unsigned int *));


/*
 *   Routines to control direct manipulation forms
 */
/* menuform.c */
void xvf_init_menuform PROTO((void));

#endif /* _xvwidgets_xvwidgets_h_ */
/* Don't add after this point */
