/*
  * Khoros: $Id: Label.c,v 1.19.2.19 1997/05/24 19:01:50 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Label.c,v 1.19.2.19 1997/05/24 19:01:50 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Label Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Redisplay()
   >>>>                 Destroy()
   >>>>                 SetValues()
   >>>>                 RecomputeSize()
   >>>>                 EventHandler()
   >>>>                 ConvertProc()
   >>>>                 LoseOwnershipProc()
   >>>>                 TransferDoneProc()
   >>>>                 RequestorCallback()
   >>>>   Public:
   >>>>                 xvw_create_label()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include <kwidgets/LabelP.h>
#include <X11/Xatom.h>

static void ClassInitialize   PROTO((void));
static void Initialize	      PROTO((Widget, Widget, ArgList, Cardinal *));
static void Redisplay	      PROTO((Widget, XEvent *, Region));
static void Destroy	      PROTO((Widget));
static Boolean SetValues      PROTO((Widget, Widget, Widget, ArgList,
					Cardinal *));
static void RecomputeSize     PROTO((Widget));
static void EventHandler      PROTO((xvobject, kaddr, XEvent *, int *));
static Boolean ConvertProc    PROTO((Widget, Atom *, Atom *, Atom *,
				     XtPointer *, unsigned long *, int *));
static void LoseOwnershipProc PROTO((Widget, Atom *));

static void RequestorCallback PROTO((Widget, XtPointer, Atom *, Atom *,
                                     XtPointer, unsigned long *, int *));
static void TransferDoneProc  PROTO((Widget, Atom *, Atom *));

static void LabelSelectTimeout PROTO((kaddr, XtIntervalId *));
static void UpdateCutBufferContents PROTO((XvwLabelGadget));


/*
 *  The current label owning the cut_buffer and it's contents have to
 *  be statically defined since Xt's selection mechanism doesn't support
 *  gadgets. :(
 */
static Widget cut_buffer_label = NULL;
static char   *cut_buffer_contents = NULL;

/*--------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_LABEL,		    NULL,                     XtRString,    NULL}, 
{XVW_LABEL_LINES,           NULL,                     XtRPointer,   NULL},
{XVW_LABEL_NUMLINES,        NULL,                     XtRInt,       NULL},
{XVW_LABEL_CURSOR_POS,      NULL,                     XtRInt,       NULL},
{XVW_LABEL_CUT_CURSOR_POS1, NULL,                     XtRInt,       NULL},
{XVW_LABEL_CUT_CURSOR_POS2, NULL,                     XtRInt,       NULL},
{XVW_LABEL_CURSOR_XLOC,     NULL,                     XtRInt,       NULL},
{XVW_LABEL_CURSOR_YLOC,     NULL,                     XtRInt,       NULL},
{XVW_LABEL_SELECTABLE,      NULL,                     XtRInt,       XtRBoolean},
{XVW_LABEL_DISPLAY_CURSOR,  NULL,                     XtRInt,       XtRBoolean},
{XVW_LABEL_EMPHASIZE,	    NULL,                     XtRInt,       XtRBoolean},
{XVW_LABEL_EMPHASIZEPIXEL,  NULL,                     XtRPixel,     NULL}, 
{XVW_LABEL_EMPHASIZECOLOR,  XVW_LABEL_EMPHASIZEPIXEL, XtRString,    XtRPixel}, 
{XVW_LABEL_JUSTIFY,	    NULL,                     XtRInt,       NULL}, 
{XVW_LABEL_MULTILINE,	    NULL,                     XtRInt,       XtRBoolean},
{XVW_LABEL_YDIST, 	    NULL,                     XtRInt,       NULL},

{XVW_LABEL_CURSOR_MOTION,   NULL,  XtRCallback, NULL},
{XVW_LABEL_PASTE_BUFFER,    NULL,  XtRCallback, NULL},

};

/*--------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwLabelGadgetRec, label.field)

static XtResource resources[] = { 
{XVW_LABEL, NULL, XtRString, sizeof(String),
      offset(string), XtRImmediate, (XtPointer) ""},
{XVW_LABEL_LINES, NULL, XtRPointer, sizeof(XtPointer),
      offset(lines), XtRImmediate, (XtPointer) NULL},
{XVW_LABEL_NUMLINES, NULL, XtRInt, sizeof(int),
      offset(numlines), XtRImmediate, (XtPointer) 0},
{XVW_LABEL_CURSOR_POS, NULL, XtRInt, sizeof(int),
      offset(cursor_pos), XtRImmediate, (XtPointer) -1},
{XVW_LABEL_CUT_CURSOR_POS1, NULL, XtRInt, sizeof(int),
      offset(cut_cursor_pos1), XtRImmediate, (XtPointer) -1},
{XVW_LABEL_CUT_CURSOR_POS2, NULL, XtRInt, sizeof(int),
      offset(cut_cursor_pos2), XtRImmediate, (XtPointer) -1},
{XVW_LABEL_CURSOR_XLOC, NULL, XtRInt, sizeof(int),
      offset(cursor_xloc), XtRImmediate, (XtPointer) 0},
{XVW_LABEL_CURSOR_YLOC, NULL, XtRInt, sizeof(int),
      offset(cursor_yloc), XtRImmediate, (XtPointer) 0},
{XVW_LABEL_SELECTABLE, NULL, XtRBoolean, sizeof(Boolean),
      offset(selectable), XtRImmediate, (XtPointer) FALSE},
{XVW_LABEL_DISPLAY_CURSOR, NULL, XtRBoolean, sizeof(Boolean),
      offset(display_cursor), XtRImmediate, (XtPointer) FALSE},
{XVW_LABEL_EMPHASIZE, NULL, XtRBoolean, sizeof(Boolean),
      offset(emphasize), XtRImmediate, (XtPointer) FALSE},
{XVW_LABEL_EMPHASIZEPIXEL, NULL, XtRPixel, sizeof(Pixel),
      offset(pixel), XtRString, (XtPointer) "red"},
{XVW_LABEL_JUSTIFY, NULL, XtRInt, sizeof(int),
      offset(justify), XtRImmediate, (XtPointer) KLABEL_JUSTIFY_LEFT},
{XVW_LABEL_MULTILINE, NULL, XtRBoolean, sizeof(Boolean),
      offset(multiline), XtRImmediate, (XtPointer) TRUE},
{XVW_LABEL_YDIST, NULL, XtRInt, sizeof(int),
      offset(ydist), XtRImmediate, (XtPointer) 0},
{XVW_LABEL_CURSOR_MOTION, NULL, XtRCallback, sizeof(XtCallbackList),
      offset(cursor_callback), XtRCallback, (XtPointer) NULL},
{XVW_LABEL_PASTE_BUFFER, NULL, XtRCallback, sizeof(XtCallbackList),
      offset(paste_callback), XtRCallback, (XtPointer) NULL},
};
#undef offset


#define SUPERCLASS (&xvwManagerGadgetClassRec)

XvwLabelGadgetClassRec xvwLabelGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Label",				/* class_name		  */
    sizeof(XvwLabelGadgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    NULL,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    FALSE,				/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    XtInheritPicking,			/* pick object proc       */
    XtInheritEraseSel,			/* erase selected proc    */
    XtInheritRefreshSel,		/* refresh selection proc */
    NULL,				/* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwLabelGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwLabelGadgetClass for public consumption
 */
WidgetClass xvwLabelGadgetClass = (WidgetClass) &xvwLabelGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of label object class has been created.
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young
|          Date: Jan 10, 1997
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
	kwidgets_cache_pixmaps();
        xvw_init_attributes(xvwLabelGadgetClass, attributes,
                XtNumber(attributes), NULL, 0,
                "$DESIGN/objects/library/kwidgets/uis/Label.pane");
	xvw_load_resources("$DESIGN/objects/library/kwidgets/app-defaults/Label");
}


/*-----------------------------------------------------------
|
|  Routine Name: Initialize 
|
|       Purpose: This method will set up the initial values 
|                for the fields of a label object instance.
|                The width, height, ascent, and descent are
|		 all initialized to sane values. 
|
|         Input: request - not used 
|                new1     - object instance after initialization, with
|    Written By: Mark Young
|          Date: Jan 10, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwLabelGadget nobj = (XvwLabelGadget) new1;
	unsigned long mask = ButtonPressMask | ButtonMotionMask |
			         ButtonReleaseMask;

	/*
	 *  Compute the position of the object in device coordinates
	 */
	nobj->label.lines           = NULL;
	nobj->label.numlines        = 0;
	nobj->label.string_width    = 0;
	nobj->label.string_height   = 0;
	nobj->label.string_ascent   = 0;
	nobj->label.string_descent  = 0;
	nobj->label.cursor_pos      = 0;
	nobj->label.cursor_xloc     = 0;
	nobj->label.cursor_yloc     = 0;
	nobj->label.cursor_x        = 0;
	nobj->label.cursor_y        = 0;
	nobj->label.x_indx1         = 0;
	nobj->label.x_indx2         = 0;
	nobj->label.y_indx1         = 0;
	nobj->label.y_indx2         = 0;
	nobj->label.cut_cursor_pos1 = 0;
	nobj->label.cut_cursor_pos2 = 0;
	nobj->label.multiline       = TRUE;
	nobj->label.cursor_callback = NULL;
	nobj->label.paste_callback  = NULL;
	nobj->label.select_id       = (XtIntervalId) 0;
	nobj->label.select_count    = 0;
	nobj->label.select_active   = FALSE;


	/*
	 *  For support of multiline strings, compile string specified
	 *  at line breaks into an array of text lines.
	 */
	nobj->label.string = kstrdup(nobj->label.string);
	LabelCompileString(new1);
	RecomputeSize(new1);

	if (nobj->label.selectable)
	    xvw_insert_event(xvw_object(new1), mask, EventHandler, 
			     NULL, KLIST_TAIL);
}


/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the label object in response
|                to an expose event. 
|
|         Input: widget - the label object that was exposed 
|                event  - the event that caused the redraw
|                region - the region that was exposed 
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 10, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	/*
	 *  Call the ManagerObj to refresh the background, border, etc
	 */
	(*xvwLabelGadgetClass->core_class.superclass->core_class.expose)
                (widget, event, region);

	/*
	 * utility routine redraws the label contents
	 */
	LabelDrawContents(widget);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: This method will free the label string.
|
|         Input: widget - the widget being destroyed
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 10, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	XvwLabelGadget xobj = (XvwLabelGadget) widget;

	/* if the label being destroyed owns the cut buffer, free it */
	if (widget == cut_buffer_label)
	{
           XtDisownSelection(XtParent(widget), XA_PRIMARY,
                     XtLastTimestampProcessed(XtDisplay(cut_buffer_label)));
	   cut_buffer_label = NULL;
	   kfree_and_NULL(cut_buffer_contents);
	}

	/* remove the select timeout if one is currently set */
        if (xobj->label.select_id != 0)
            XtRemoveTimeOut(xobj->label.select_id);

	karray_free(xobj->label.lines, KSTRING, 
		    (size_t) xobj->label.numlines, NULL);
	kfree_and_NULL(xobj->label.string);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: If the the justify or postion of the 
|		 label as changed, recalculate the new position. 
|
|         Input: current - the object containing current settings 
|                request - the object containing requested settings
|                new1     - the object processed through all set values methods 
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young, John Salas & Danielle Argiro
|          Date: Jan 10, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwLabelGadget cobj = (XvwLabelGadget) current;
	XvwLabelGadget nobj = (XvwLabelGadget) new1;
 
	Boolean redisplay = FALSE;


	if (cobj->label.justify   != nobj->label.justify   ||
	    cobj->label.emphasize != nobj->label.emphasize ||
	    cobj->manager.font	  != nobj->manager.font)
	{
	    redisplay = TRUE; 
	    RecomputeSize(new1);
	}

	if (cobj->label.cut_cursor_pos1 != nobj->label.cut_cursor_pos1 ||
	    cobj->label.cut_cursor_pos2 != nobj->label.cut_cursor_pos2)
	{
	   /* cut cursor pos can be > length of string (to include newline) */
	   int length = kstrlen(nobj->label.string);
	   if (nobj->label.cut_cursor_pos1 < 0)
	       nobj->label.cut_cursor_pos1 = 0;
	   else if (nobj->label.cut_cursor_pos1 > length)
	       nobj->label.cut_cursor_pos1 = length;

	   if (nobj->label.cut_cursor_pos2 < 0)
	       nobj->label.cut_cursor_pos2 = 0;
	   else if (nobj->label.cut_cursor_pos2 > length)
	       nobj->label.cut_cursor_pos2 = length;

	   if (cobj->label.cut_cursor_pos1 != cobj->label.cut_cursor_pos2 ||
	       nobj->label.cut_cursor_pos1 != nobj->label.cut_cursor_pos2)
	   {
	      /* refresh contents of the label */
	      redisplay = TRUE;
	   }
	}

	/*
	 *  New array of text lines (attribute available to text widget only)
	 */
	if (cobj->label.lines != nobj->label.lines)
        {
	    int    i; 
	    size_t size;

            /* 
	     * Copy these new text lines into allocated private strings,
	     * before freeing the previously allocated private strings
	     */
            nobj->label.lines = (char **) kdupalloc(nobj->label.lines, 
  				        nobj->label.numlines * sizeof (char *));
            for (i = 0; i < nobj->label.numlines; i++)
                nobj->label.lines[i] = kstrdup(nobj->label.lines[i]);

	    if (nobj->label.y_indx2 > nobj->label.numlines-1)
		nobj->label.y_indx2 = nobj->label.numlines-1;
	    if (nobj->label.cursor_yloc > nobj->label.numlines-1)
		nobj->label.cursor_yloc = nobj->label.numlines-1;

	    /*
	     * Now, have to cat lines together into nobj->label.string since 
	     * the whole string, not the compiled lines, is used for cut & paste
	     */
	    kfree_and_NULL(cobj->label.string);
	    for (i = 0, size = 0; i < nobj->label.numlines; i++)
		size += (size_t) (kstrlen(nobj->label.lines[i])+1);

	    nobj->label.string = (char *) kcalloc((size_t) 1,size*sizeof(char));
	    ConcatenateLines(nobj->label.lines, nobj->label.numlines,
			     nobj->label.string);

            /* erase cursor */
            LabelDrawCursor(current, cobj->manager.background_gc);

	    /* new lines imply a new size */
            RecomputeSize(new1);

	    /* refresh contents of the label */
	    redisplay = TRUE;
        }

	/*
	 *  New Label String
	 */
	else if (cobj->label.string != nobj->label.string)
	{
	   /*
	    *  Free the previously allocated private string, and make a
	    *  private copy of the new string
	    */
	   kfree_and_NULL(cobj->label.string);
	   nobj->label.string = kstring_detab(nobj->label.string, NULL);

	   /*
	    *  For support of multiline strings, compile string specified
	    *  at line breaks into an array of text lines.
	    */
	   nobj->label.numlines = 0;
	   nobj->label.lines = NULL;
	   LabelCompileString(new1);

	   if (nobj->label.y_indx2 > nobj->label.numlines-1)
	       nobj->label.y_indx2 = nobj->label.numlines-1;
	   if (nobj->label.cursor_yloc > nobj->label.numlines-1)
	       nobj->label.cursor_yloc = nobj->label.numlines-1;

           /* erase cursor */
           LabelDrawCursor(current, cobj->manager.background_gc);

	   /* new lines imply a new size */
	   RecomputeSize(new1);

	   /* refresh contents of the label */
	   redisplay = TRUE;
	}

	/*
	 *  instead of simply setting redisplay to TRUE (easy but inefficient)
	 *  it is better to do the erasing/drawing of the cursor and text line
	 *  directly.
	 */
	if (cobj->label.display_cursor != nobj->label.display_cursor)
	{
	   /* draw cursor in current location */
	   if (nobj->label.display_cursor == TRUE)
	      LabelDrawCursor(new1, nobj->manager.foreground_gc);
	   else /* erase cursor */
	       LabelDrawCursor(current, cobj->manager.background_gc);
	}

	/*
	 *  New X and/or Y location of cursor; 
	 *
	 *  slight kludge: using value of -1 to say "move xloc to end of line"
	 *                                       or "move yloc to end of file"
	 *
         *  a cursor xloc of 0 puts the cursor before the first character in
	 *  column 1; a cursor xloc of kstrlen(line) puts it after the last
	 *  of line.
	 *
         *    0123456789
         *   ------------
	 * 0 |ABC DEF GH  \ 
	 * 1 |IJKL         ) Y loc 2, X loc 4 => the "Q" in cursor position 19
	 * 2 |MNOPQR S    /
 	 */
	if (cobj->label.cursor_xloc != nobj->label.cursor_xloc ||
	    cobj->label.cursor_yloc != nobj->label.cursor_yloc)
	{
	    int cursor_pos = 0, cursor_yloc, cursor_xloc; 
	
	    cursor_xloc = nobj->label.cursor_xloc;
	    cursor_yloc = nobj->label.cursor_yloc;

	    /*
	     * protect against going less than 0 or > numlines.
	     */
	    if (cursor_yloc == -1 || cursor_yloc > nobj->label.numlines-1)
		cursor_yloc = nobj->label.numlines-1;
	    if (cursor_yloc < 0) 
		cursor_yloc = 0;

            /*
             * if the cursor_xloc is -1, go to the end of the line.  Or if
             * cursor_xloc is out of bounds reset to 0 or end of the line.
             */
	    if (nobj->label.lines == NULL)
	        cursor_xloc = 0;
	    else if (cursor_xloc == -1 ||
		     cursor_xloc > kstrlen(nobj->label.lines[cursor_yloc]))
	       cursor_xloc = kstrlen(nobj->label.lines[cursor_yloc]);
	    else if (cursor_xloc < 0)
	       cursor_xloc = 0;

            /*
             * compute cursor position given our updated x&y location.
             */
	    LabelDetermineCursorPos(nobj->label.lines, nobj->label.numlines,
                                    cursor_xloc, cursor_yloc, &cursor_pos);

            /*
             * update cursor position, x & y location.
             */
            nobj->label.cursor_pos  = cursor_pos;
            nobj->label.cursor_xloc = cursor_xloc;
            nobj->label.cursor_yloc = cursor_yloc;

	    /*
	     *  instead of simply setting redisplay to TRUE (easy but 
	     *  inefficient) it is better to do the erasing/drawing of the 
	     *  cursor and text line directly.
	     */
	    if (cobj->label.cursor_pos != nobj->label.cursor_pos)
	    {
	        /* erase the old cursor */
	        LabelDrawCursor(current, cobj->manager.background_gc);
                LabelCalculatePixelCursorXY(new1, nobj->label.cursor_xloc,
                                   nobj->label.cursor_yloc);
                /*
              	 * Check to see if a cursor position change caused the
              	 * the contents of the text being displayed to change.  If
             	 * so then we need to redisplay the entire contents, rather
             	 * than just redisplaying the cursor.
             	 */
            	if (cobj->label.y_indx1 != nobj->label.y_indx1 ||
                    cobj->label.y_indx2 != nobj->label.y_indx2 ||
                    cobj->label.x_indx1 != nobj->label.x_indx1 ||
                    cobj->label.x_indx2 != nobj->label.x_indx2)
            	{
                    redisplay = TRUE;
            	}

                /*
              	 *  instead of simply setting redisplay to TRUE (easy but
               	 *  inefficient) it is better to do the erasing/drawing of the
             	 *  cursor and text line directly.
             	 */
            	else LabelDrawCursor(new1, nobj->manager.foreground_gc);
	    }
	}

	if (cobj->label.selectable != nobj->label.selectable)
	{
	    unsigned long mask = ButtonPressMask | ButtonMotionMask |
			         ButtonReleaseMask;
	    if (nobj->label.selectable == TRUE)
 	        xvw_insert_event(xvw_object(new1), mask, 
				 EventHandler, NULL, KLIST_TAIL);
	    else xvw_remove_event(xvw_object(new1), mask, 
				  EventHandler, NULL);
	}

        /*
         *  New cursor position: have to calculate new X and Y locations
         *
         *    0123456789
         *   ------------
         * 0 |ABC DEF GH  \
         * 1 |IJKL         ) The "L" in cursor position 13 => Y loc 1, X loc 1
         * 2 |MNOPQR S    /
         */
        if (cobj->label.cursor_pos != nobj->label.cursor_pos)
        {
	    LabelDetermineCursorXYLoc(nobj->label.lines, 
				      nobj->label.numlines,
				      nobj->label.cursor_pos,
				      &nobj->label.cursor_xloc,
				      &nobj->label.cursor_yloc,
				      &nobj->label.cursor_pos);

	    /* erase the old cursor */
	    LabelDrawCursor(current, cobj->manager.background_gc);

            LabelCalculatePixelCursorXY(new1, nobj->label.cursor_xloc,
                                   nobj->label.cursor_yloc);

            /*
             * Check to see if a cursor position change caused the
             * the contents of the text being displayed to change.  If
             * so then we need to redisplay the entire contents, rather
             * than just redisplaying the cursor.
             */
            if (cobj->label.y_indx1 != nobj->label.y_indx1 ||
                cobj->label.y_indx2 != nobj->label.y_indx2 ||
                cobj->label.x_indx1 != nobj->label.x_indx1 ||
                cobj->label.x_indx2 != nobj->label.x_indx2)
            {
                redisplay = TRUE;
            }
	    /*
             *  instead of simply setting redisplay to TRUE (easy but 
             *  inefficient) it is better to do the erasing/drawing of the 
             *  cursor and text line directly.
             */
	    else LabelDrawCursor(new1, nobj->manager.foreground_gc);
        }

	/*
	 *  If we we need to redisplay the contents of the Label then call
	 *  LabelRefreshContents directly rather than call the manager force
	 *  redisplay routine.  This is more efficient and avoids unwanted
	 *  flicker when redrawing the label.
	 */
	if (redisplay && nobj->label.selectable)
	{
	    LabelRefreshContents(current, new1);
	    redisplay = FALSE;
	}
	else if (redisplay && nobj->manager.force_redisplay)
	    ManagerObjForceRedisplay(current, new1, TRUE, &redisplay);

	if (cobj->label.lines != nobj->label.lines)
	{
	    karray_free(cobj->label.lines, KSTRING,
                    (size_t) cobj->label.numlines, NULL);
	}
	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: RecomputeSize
|
|       Purpose: Given the current position recompute the size.
|
|         Input: widget - the label object 
|    Written By: Mark Young
|          Date: Jan 10, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RecomputeSize(
   Widget widget)
{
	XvwLabelGadget xobj = (XvwLabelGadget) widget;

	String *lines;
	int i, numlines, len, ascent, descent, width, buffer, tmp_width;
        int ydist = xobj->label.multiline ? xobj->label.ydist : 0;
        XtWidgetGeometry temp;
	
	if (xobj->label.string == NULL &&
	    xobj->label.lines == NULL) return; 

	/*
	 *  Get the extent of the string
	 */
	if (xobj->manager.font != NULL)
	{
	   lines = xobj->label.lines;
	   numlines = xobj->label.numlines;
	   ascent  = xobj->manager.font->max_bounds.ascent;
	   descent = xobj->manager.font->max_bounds.descent;
	   xobj->label.string_ascent  = ascent;
	   xobj->label.string_descent = descent;
	   xobj->label.string_height  = (ascent + descent + ydist) * numlines;

	   for (i = 0, width = 0; i < numlines; i++)
	   {
	      if ((len = kstrlen(lines[i])) == 0)
		  continue;

	      tmp_width = XTextWidth(xobj->manager.font, lines[i], len);
	      if (tmp_width > width) width = tmp_width;
	   }
	   xobj->label.string_width = width;
	}
	else
	{
	   xobj->label.string_width   =
	   xobj->label.string_height  =
	   xobj->label.string_ascent  =
	   xobj->label.string_descent = 0;
	}

	/*
	 *  Request the new size....
	 */
	buffer = xobj->manager.shadow_thickness + 1;

        temp.request_mode = CWWidth | CWHeight;
        temp.width  = (Dimension) xobj->label.string_width  + 2*buffer;
        temp.height = (Dimension) xobj->label.string_height + 2*buffer;
        (void) XtMakeGeometryRequest(widget, &temp, &temp);
}

/*-----------------------------------------------------------
|
|  Routine Name: EventHandler
|
|       Purpose: Handles ButtonPress events for the Label object:
|		 Button1 Press: moves cursor to new position
|		 Button2 Press: pastes from the cut buffer
|
|         Input: label       - the label object
|                client_data - not used
|                event       - the button press event
|       Returns:
|    Written By: Danielle Argiro
|          Date: January 20, 1997
|
------------------------------------------------------------*/

/* ARGSUSED */
static void EventHandler(
   xvobject label,
   kaddr    client_data,
   XEvent   *event,
   int      *dispatch)
{
	XvwLabelGadget xobj = (XvwLabelGadget) (xvw_widget(label));
	int select_count = xobj->label.select_count;

	if (event->type == ButtonPress)
	{
	    int cursor_pos;

	    /*
	     *  Need to select_active to true in order to start selection of
	     *  text.
	     */
	    xobj->label.select_active = TRUE;
	    if (xobj->label.select_id != 0)
	    {
	        XtRemoveTimeOut(xobj->label.select_id);
	        xobj->label.select_id = 0;
	        xobj->label.select_count = 0;
	    }

	   /*
            *  from the (x,y) pixel location given, calculate values for
            *  xloc_pos and yloc_pos, which indicate the character x & y 
	    *  position of the pointer, eg character 8 of line 4
            */
           (void) LabelCalculateCharCursorXY(xvw_widget(label),event->xbutton.x,
			     event->xbutton.y, &xobj->label.cursor_xloc,
			     &xobj->label.cursor_yloc);
 
           /*
            *  now that we know the character position of the pointer,
            *  figure out which character it was overall (eg, character 34).
            */
            LabelDetermineCursorPos(xobj->label.lines, xobj->label.numlines,
                                    xobj->label.cursor_xloc,
				    xobj->label.cursor_yloc, 
				    &cursor_pos);
	    xvw_set_attribute(label, XVW_LABEL_CURSOR_POS, cursor_pos);
	    LabelResetCutCursorPosition(label);

	    if (event->xbutton.button == 1)
	    {
		XtAppContext  app_context;
                unsigned long clicktime;
 
                app_context    = XtWidgetToApplicationContext((Widget)xobj);
                clicktime = (unsigned long) 
			    (XtGetMultiClickTime(xvw_display(label)));
		xobj->label.select_id = XtAppAddTimeOut(app_context, clicktime, 
					   LabelSelectTimeout, xobj);
		xobj->label.select_count = select_count + 1;
		return;
	    }
	}
	else if (event->type == ButtonRelease)
	{
	    /*
	     * Use of button 2 when there is a callback installed for cut buffer
	     * paste: call utility to paste buffer and fire callbacks
	     */
	    if (event->xbutton.button == 2)
	    {
	        if (xobj->label.paste_callback != NULL)
		    LabelPasteBuffer(label);
	    }
	    else UpdateCutBufferContents(xobj);   

	    /* turn text selective active off */
	    xobj->label.select_active = FALSE;
	}
	else /* event type was ButtonMotion */
	{
	    int char_x, char_y, cut_cursor_pos2, off_the_end;


	    /*
	     *  If select_active is not TRUE then we didn't start selection
	     *  by ButtonPress.  So simply ignore this event.
	     */
	    if (xobj->label.select_active == FALSE)
	        return;

            off_the_end = LabelCalculateCharCursorXY(xvw_widget(label), 
						     event->xmotion.x, 
					event->xmotion.y, &char_x, &char_y);
	    LabelDetermineCursorPos(xobj->label.lines, xobj->label.numlines,
				    char_x, char_y, &cut_cursor_pos2);

	    /*
	     * allow cut cursor position to be greater than the length of the 
	     * string, to indicate when the user highlights to the end of the 
	     * line, saying that they want the newline
	     */
	    if (off_the_end && xobj->label.cut_cursor_pos1 != cut_cursor_pos2)
	    {
	       if (cut_cursor_pos2 < kstrlen(xobj->label.string) - 1)
	          cut_cursor_pos2++;
	    }
	    xvw_set_attribute(label, XVW_LABEL_CUT_CURSOR_POS2,cut_cursor_pos2);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: ConvertProc
|
|       Purpose: Converts data to the format specified by the
|		 requestor (RequestorCallback) and prepares
|		 it for transfer.  Since this is for support of
|		 the label widget, it simply needs to set *value_return
|		 to the string pointer and length_return to the
|		 length of the string.
|
|         Input: xobj          - the label widget
|		 selection     -
|		 target        -
|		 type_return   - 
|		 value_return  - 
|		 length_return -
|		 format_return -
|                
|    Written By: Danielle Argiro & Mark Young
|          Date: April 21, 1997
|
------------------------------------------------------------*/

static Boolean ConvertProc(
    Widget    widget,
    Atom      *selection,
    Atom      *target,
    Atom      *type_return,
    XtPointer *value_return,
    unsigned long *length_return,
    int       *format_return)
{

	if (*target != XA_STRING) 
	    return (FALSE);

	*format_return = 8;
	*type_return   = XA_STRING;
	*length_return = kstrlen(cut_buffer_contents);
	*value_return  = cut_buffer_contents;

	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: LoseOwnershipProc
|
|       Purpose: If the selection is lost, either because it timed out
|		 or because the user made a different selection, this 
|		 action handler is invoked.  This function simply 
|		 resets the cut cursor position to the cursor position before 
|		 redrawing the label contents (this makes highlighting go away).
|
|         Input: parent - have to install lose ownership on the parent of the
|			  label since Xt does not support use of XtOwnSelection
|			  on a gadget.
|		 selection - XA_PRIMARY
|
|    Written By: Danielle Argiro & Mark Young
|          Date: April 21, 1997
|
------------------------------------------------------------*/
 
static void LoseOwnershipProc(
    Widget parent,
    Atom   *selection)
{
	XvwLabelGadget xobj;

	kfree_and_NULL(cut_buffer_contents);
	if (cut_buffer_label == NULL) return;

	xobj = (XvwLabelGadget) cut_buffer_label;
	cut_buffer_label = NULL;
	xobj->label.cut_cursor_pos1 = xobj->label.cursor_pos;
	xobj->label.cut_cursor_pos2 = xobj->label.cursor_pos;
	LabelDrawContents((Widget) xobj);
}

/*-----------------------------------------------------------
|
|  Routine Name: TransferDoneProc
|
|       Purpose: This dummy routine must be defined so that
|		 Xt does not free the data that we stored in the cut buffer.
|
|         Input: parent - have to install lose ownership on the parent of the
|			  label since Xt does not support use of XtOwnSelection
|			  on a gadget.
|		 selection - XA_PRIMARY
|
|    Written By: Danielle Argiro & Mark Young
|          Date: April 21, 1997
|
------------------------------------------------------------*/
 
static void TransferDoneProc(
    Widget parent,
    Atom   *selection,
    Atom   *target)
{
	return;
}

/*-----------------------------------------------------------
|
|  Routine Name: RequestorCallback
|
|       Purpose: Pastes the selection from the cut buffer to 
|		 the label widget
|
|         Input: parent - have to install lose ownership on the parent of the
|			  label since Xt does not support use of XtOwnSelection
|			  on a gadget.
|		 selection - XA_PRIMARY
|		 type      - XA_STRING
|	         value     - a pointer to the string selection to be pasted
|		 length    - the length of the selection
|		 format    - the format of the selection
|
|    Written By: Danielle Argiro & Mark Young
|          Date: April 21, 1997
|
------------------------------------------------------------*/
 
static void RequestorCallback(
    Widget    parent,
    XtPointer client_data,
    Atom      *selection,
    Atom      *type,
    XtPointer value,
    unsigned long *length,
    int       *format)
{
	xvobject label   = (xvobject) client_data;
	char     *tmp, *buffer = (char *) value;

        /*
         * nothing in the cut buffer; never mind
         */
        if (buffer == NULL) return;
 
        /*
         * fire callbacks on XVW_LABEL_PASTE_BUFFER with the contents
         * of the cut buffer as the call_data
         */
	tmp = kstring_ncopy(buffer, *length, NULL);
        XtCallCallbacks(xvw_widget(label), XVW_LABEL_PASTE_BUFFER, tmp);
	kfree(tmp);

	/*
	 * redraw the label with new pasted text in it
	 */
	LabelDrawContents(xvw_widget(label));
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelSelectTimeout
|
|       Purpose: Contains the code that handles text selection
|
|         Input: client_data - used to pass in the label widget
|                id          - timeout id
|    Written By: Danielle Argiro & Mark Young
|          Date: April 22, 1997
|
------------------------------------------------------------*/
static void LabelSelectTimeout(
    kaddr client_data,
    XtIntervalId *id)
{
        XvwLabelGadget xobj  = (XvwLabelGadget) client_data;
	int  cut_cursor_pos1 = xobj->label.cut_cursor_pos1;
	int  cut_cursor_pos2 = xobj->label.cut_cursor_pos2;
	int  cursor_pos      = xobj->label.cursor_pos;
	char *string         = xobj->label.string;
	int  length          = kstrlen(xobj->label.string);
	int  pos, is_space;
	xvobject label = xvw_object((Widget) xobj);

	/*
	 * if there was only one button click, only need to move
	 * the cursor (and that was already done in EventHandler)
	 */
	if (xobj->label.select_count == 1) 
	{
            XtCallCallbacks((Widget) xobj, XVW_LABEL_CURSOR_MOTION,
                        &xobj->label.cursor_pos);
	    cut_cursor_pos1 = cut_cursor_pos2 = xobj->label.cursor_pos;
	}

	/*
	 * two button clicks: highlight a word
	 */
	else if (xobj->label.select_count == 2)
	{
	    pos = cursor_pos;
	    is_space = isspace(string[pos]);
	    while (pos-1 >= 0 && isspace(string[pos-1]) == is_space) 
		pos--;
	    cut_cursor_pos1 = pos;
	    
	    pos = cursor_pos;
	    while (isspace(string[pos]) == is_space && pos < length)         
		pos++;
	    cut_cursor_pos2 = pos;
	}

	/*
         * three button clicks: highlight a line
         */
	else if (xobj->label.select_count == 3)
	{
	    pos = cursor_pos;
	    if (string[pos] == '\n') pos--;
	    while (pos-1 >=0 && string[pos-1] != '\n') 
		pos--;
	    cut_cursor_pos1 = pos;
            
	    pos = cursor_pos;
	    while (string[pos] != '\n' && pos < length) 
		pos++;
	    pos++;  /* last increment to include new line */
	    cut_cursor_pos2 = pos;
	}

	/*
         * four button clicks: highlight all the available text
         */
	else /* xobj->label.select_count == 4 or higher */
	{
	    cut_cursor_pos1 = 0;
	    cut_cursor_pos2 = length;
	}

	/*
	 * Note here we have to make two separate calls to set the cursor_pos
	 * and cut_cursor_pos attributes (which force highlighting when they
	 * differ) because the code for setting cursor_pos will reset the
	 * value of cut_cursor_pos otherwise.
	 */
	xvw_set_attribute(label, XVW_LABEL_CURSOR_POS,      cut_cursor_pos1);
	xvw_set_attribute(label, XVW_LABEL_CUT_CURSOR_POS1, cut_cursor_pos1);
	xvw_set_attribute(label, XVW_LABEL_CUT_CURSOR_POS2, cut_cursor_pos2);

	/*
	 *  reset timeout fields now that operation is completed
	 */
        xobj->label.select_id      = (XtIntervalId) 0;
        xobj->label.select_count   = 0;
	UpdateCutBufferContents(xobj);
}

/*-----------------------------------------------------------
|
|  Routine Name: UpdateCutBufferContents
|
|       Purpose: Decides whether or not the internal cut buffer contents
|                and the internal label widget associated with the cut buffer
|		 need to be updated. If so, it updates them accordingly and 
|		 calls Xt to own or disown the selection appropriately.  
|	 	 Note that "selection" refers to the contents of the cut buffer.
|		 Conditions where cut_buffer_contents need to be updated are:
|
|		 1) when there is text highlighted (ie, size != 0)
|		 2) when there is NOT text highlighted (ie, size == 0)
|		    but we currently own the selection 
|		    (ie, cut_buffer_label = widget)
|
|         Input: xobj - the label widget
|
|    Written By: Danielle Argiro & Mark Young
|          Date: April 23, 1997
|
------------------------------------------------------------*/
 
static void UpdateCutBufferContents(
    XvwLabelGadget xobj)
{
        char *buffer;
        int   min_pos, max_pos, size;
        Widget widget = (Widget) xobj;
 
        min_pos = kmin(xobj->label.cut_cursor_pos1, 
		       xobj->label.cut_cursor_pos2);
        max_pos = kmax(xobj->label.cut_cursor_pos1, 
		       xobj->label.cut_cursor_pos2);
        size = max_pos - min_pos;

	/* 
	 * if there is no text highlighted and this label widget does not 
	 * own the selection, then there is nothing to do - just return
	 */
        if (size == 0 && cut_buffer_label != widget) return;

        /*
         * there is text highlighted
         *
         * 1) call XtOwnSelection so that this label object will now own the
         *    selection (and any other highlighted text in this application
         *    or another application will be un-highlighted, since
         *    XtDisownSelection will get called either by the next code segment
         *    or by the other application, respectively)
         * 2) need to free the previous cut buffer, since *if* we already own
         *    the selection, LoseOwnershipProc will *not* be called to free
         *    the memory.
         * 3) alloc and set global cut_buffer_contents, set cut_buffer_label
         */
        if (size > 0)
        {
            XtOwnSelection(XtParent(widget), XA_PRIMARY,
                           XtLastTimestampProcessed(XtDisplay(widget)),
                           ConvertProc, LoseOwnershipProc, TransferDoneProc);
            kfree(cut_buffer_contents);
            buffer = kstring_ncopy(&xobj->label.string[min_pos],
				   (size_t) size, NULL);
            cut_buffer_contents = buffer;
            cut_buffer_label = widget;
        }

#if 0
	/*
	 * there is NO text highlighted, but we own this selection. 
	 * 
	 * To clear the cut buffer, we would have to do the following code.	
	 * 1) call XtDisownSelection to release the selection (the contents
	 *    of the cut buffer) to another label widget in this application
	 *    or to another application
	 * 2) set global cut_buffer_contents to NULL, reset cut_buffer_label
	 *
	 * HOWEVER THIS CODE IS IFDEF'D OUT ON PURPOSE because we want to leave
	 * the contents of the cut buffer intact, even though there is no
	 * text highlighted.  We ifdef'd this code out rather than deleting it
	 * in case we ever need to change our minds, we don't have to re-write
	 * it.
	 *
	 * PLEASE DON'T DELETE IT.
	 */
	else
	{
            XtDisownSelection(XtParent(widget), XA_PRIMARY,
                              XtLastTimestampProcessed(XtDisplay(widget)));
            cut_buffer_label = NULL;
            kfree_and_NULL(cut_buffer_contents);
	}
#endif

}

/*-----------------------------------------------------------
|
|  Routine Name: LabelPasteBuffer
|
|       Purpose: Pastes the cut buffer using XtGetSelectionValue
|		 to call the RequestorCallback.
|
|         Input: label - the label object
|    Written By: Danielle Argiro & Mark Young
|          Date: April 30, 1997
|
------------------------------------------------------------*/
void LabelPasteBuffer(
    xvobject label)
{
	Widget widget = xvw_widget(label);

	XtGetSelectionValue(XtParent(widget),
		XA_PRIMARY, XA_STRING, RequestorCallback, label, 
		 XtLastTimestampProcessed(XtDisplay(widget)));
}

/************************************************************
*
*  Routine Name: xvw_create_label - create a label object
*
*       Purpose: A label object is for the display of non-editable text
*                on Khoros graphical user interfaces.  If desired, a label 
*                object may also display a pixmap, in addition to the text.
*                The label object supports multiline text;
*                simply include "\n" in the text to indicate the
*                end of a line.  Label text and pixmaps may be left,
*                center, or right justified.  
*
*		 There are three major differences between a \fIlabel\fP 
*                object and a \fItext\fP object.  Firstly, a label object is 
*                a gadget; ie, it has no window of its own, but uses the 
*                window of its parent.  A text object, on the other hand, is a 
*                widget; it has a dedicated window of its own.  Secondly, the
*                label object supports the display of non-editable text only; 
*                the text displayed by a text object can be edited by the user 
*                unless it is explicitly handled by the application via the
*		 paste callback, irregardless if a cursor is displayed.
*		 Thirdly, the label object is
*                implemented as part of Khoros GUI services; the text object,
*		 use the label as a subpart in order to provide editability.
*		 For most non-editable text display uses, a label object is 
*                preferable, and has the advantage that it is much faster to 
*                refresh than a text object.   
*
*	         The difference between a \fIlabel\fP object and a \fIstring\fP
*                object is that the label object is meant for display of text
*                on graphical user interfaces, while the string object is an
*                annotation, meant for display of text on images, area objects,
*                and so on.  
*
*         Input: parent - parent of the label object; NULL will 
*			  cause a default toplevel to be created automatically
*                name   - a name for this particular instance of the object
*                         (for use in app-defaults files, etc)
*        Output:
*	Returns: The label object on success, NULL on failure
*
*  Restrictions: 
*    Written By: Mark Young
*  	   Date: Apr 30, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_label(
   xvobject parent,
   char     *name)
{
	xvobject  object;


	/*
	 *  Call the xvw_create() routine to do the actual creation.
	 */
	object = xvw_create(parent, FALSE, TRUE, name, LabelGadgetClass);
	return(object);
}
