 /*
  * Khoros: $Id
  */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id";
#endif
 
 
/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>           Label Widget Utility Routines
   >>>>
   >>>>	Private:  
   >>>>           LabelEraseLine()
   >>>>           LabelDrawLine()
   >>>>           LabelDrawContents()
   >>>>           LabelRefreshContents()
   >>>>           LabelDrawHighlight()
   >>>>           LabelDrawCursor()
   >>>>           LabelFindXOffset()
   >>>>           LabelFindYOffset()
   >>>>           LabelCompileString()
   >>>>           LabelDetermineCursorPos()
   >>>>           LabelDetermineCursorXYLoc()
   >>>>           LabelCalculatePixelCursorXY()
   >>>>           LabelCalculateCharCursorXY()
   >>>>           LabelPageDown()
   >>>>           LabelPageUp()
   >>>>           LabelResetCutCursorPosition()
   >>>>           LabelEnsureCursorVisibility()
   >>>>	Static:
   >>>>	Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <kwidgets/LabelP.h>
#include <kwidgets/ManagerP.h>

/*-----------------------------------------------------------
|
|  Routine Name: LabelEraseLine
|
|       Purpose: Utility routine erases a single line of the label
|		 that is currently displayed.
|
|         Input: widget - the label object that needs to be redrawn
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 10, 1997
|
------------------------------------------------------------*/

void LabelEraseLine(
    Widget widget,
    int    yindx)
{
	XvwLabelGadget xobj = (XvwLabelGadget) widget;

        int x, y, yoffset, label_width;

        Window  window   = XtWindow(widget);
        Display *display = XtDisplay(widget);
        int     line_height = (int) (xobj->label.string_ascent +
                                     xobj->label.string_descent +
				     xobj->label.ydist);

	/*
	 *  If the object has not been realized then return
	 */
	if (xobj->manager.realized == FALSE)
	    return;

	/*
 	 * draw the lines of text in the Label widget which are visible
	 */
	yoffset = LabelFindYOffset(widget);
	x = xobj->rectobj.x + xobj->manager.shadow_thickness;
	y = (yindx - xobj->label.y_indx1)*line_height + yoffset;
	label_width = xobj->rectobj.width - 2*xobj->manager.shadow_thickness;

	XFillRectangle(display, window, xobj->manager.background_gc, x,
		   y - xobj->label.string_ascent, (unsigned) label_width+1,
		   (unsigned) line_height+1);
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelDrawLine
|
|       Purpose: Utility routine redraws a single line of the label
|		 that is currently displayed.
|
|         Input: widget - the label object that needs to be redrawn
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 10, 1997
|
------------------------------------------------------------*/

void LabelDrawLine(
    Widget widget,
    int    yindx,
    int    start_xloc,
    int    start_yloc,
    int    end_xloc,
    int    end_yloc)
{
	XvwLabelGadget xobj = (XvwLabelGadget) widget;

	GC	draw_gc, fill_gc;
	char    *tmp, *line = NULL;
        Window  window   = XtWindow(widget);
        Display *display = XtDisplay(widget);
        int     x, y, yoffset, xoffset, length, tmplen, label_width;

        int     ydist    = xobj->label.multiline ? xobj->label.ydist : 0;
        int     line_height = (int) (xobj->label.string_ascent +
                                     xobj->label.string_descent + ydist);

	/*
	 *  If the object has not been realized then return
	 */
	if (xobj->manager.realized == FALSE || yindx >= xobj->label.numlines)
	    return;

	if (yindx < xobj->label.y_indx1 || yindx > xobj->label.y_indx2)
	    return;

	/*
 	 * draw the lines of text in the Label widget which are visible
	 */
	line = xobj->label.lines[yindx];
	length = kstrlen(line);
	xoffset = LabelFindXOffset(widget, line);
	yoffset = LabelFindYOffset(widget);

	x = xobj->rectobj.x;
	y = (yindx - xobj->label.y_indx1)*line_height + yoffset;
	label_width = xobj->rectobj.width - (xoffset - x);

	/*
	 * find segment of line to be drawn (realizing that entire 
	 * text line may exceed the width of the window
	 */
	if (xobj->label.x_indx1 < length)
	{
            tmp = &line[xobj->label.x_indx1];
	    tmplen = length - xobj->label.x_indx1;;
	}
	else if (length-1 < xobj->label.x_indx1)
	{
            tmp = "";
	    tmplen = 0;
	}
	else
	{
	    tmp = line;
	    tmplen = length;
	}

	/*
	 * If we have the situation where this line is to be partially 
	 * highlighted, call LabelDrawHighlight to do the work.
	 */
	if ((yindx == start_yloc || yindx == end_yloc) &&
	     xobj->label.cut_cursor_pos1 != xobj->label.cut_cursor_pos2)
	{
	    LabelDrawHighlight(widget, tmp, xoffset, y, yindx, start_xloc,
				   start_yloc, end_xloc, end_yloc);
	}
	/*
	 * Otherwise, either the whole string is highlighted (filled
	 * with background and drawn in foreground) or highlighted (filled
	 * with foreground and drqawn in background). 
	 */
	else
	{
	    /* entire line is highlighted; fill w/ fg, draw w/ bg */
	    if (yindx > start_yloc && yindx < end_yloc  &&
		xobj->label.cut_cursor_pos1 != xobj->label.cut_cursor_pos2)
	    {
		draw_gc = xobj->manager.background_gc;
		fill_gc = xobj->manager.foreground_gc;
	    }
	    /* entire line is not highlighted; fill w/ bg, draw w/ fg */
	    else
	    {
		fill_gc = xobj->manager.background_gc;
		draw_gc = xobj->manager.foreground_gc;
	    }

	    /* 
	     * if line is selectable, go ahead and fill. (if line is not
	     * selectable, there is no need to fill, let the manager object's
	     * expose method take care of it.
	     */ 
	    if (xobj->label.selectable)
	    {
	        XFillRectangle(display, window, fill_gc, xoffset,
			       y - xobj->label.string_ascent, 
			       (unsigned) label_width+1,
			       (unsigned) line_height+1);
	    }

	    /*
	     *  Now check to see if we need to draw the label emphasized
	     */
	    if (xobj->label.emphasize)
	    {
	        /* Finally, draw the label in the fg color specified earlier */
	        XSetForeground(display, draw_gc, xobj->label.pixel);
	        XDrawString(display, window, draw_gc, xoffset+2, y+2+ydist/2,
			    tmp, tmplen);
	        if (draw_gc == xobj->manager.foreground_gc)
	            XSetForeground(display, draw_gc, xobj->manager.foreground);
		else
	            XSetForeground(display, draw_gc, xobj->manager.background);
	    }

	    /* Finally, draw the label in the fg color specified earlier */
	    XDrawString(display, window, draw_gc, xoffset,y+ydist/2,tmp,tmplen);
	}
 
	/* draw the cursor */
	if (xobj->label.cursor_yloc == yindx)
	    LabelDrawCursor(widget, xobj->manager.foreground_gc);
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelDrawContents
|
|       Purpose: Utility routine redraws the contents of the label
|		 that is currently displayed.
|
|         Input: widget - the label object that needs to be redrawn
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 10, 1997
|
------------------------------------------------------------*/

void LabelDrawContents(
    Widget widget)
{
	XvwLabelGadget xobj = (XvwLabelGadget) widget;

        int i, start_highlt, end_highlt, start_xloc = 0;
	int start_yloc = 0, end_xloc = 0, end_yloc = 0;

	/*
	 *  If the object has not been realized then return
	 */
	if (xobj->manager.realized == FALSE) return;

	/*
	 *  calculate xobj->label.cursor_x, xobj->label.cursor_y,
	 *  as well as column and line offset indices 
	 *  xobj->label.x_indx1, xobj->label.x_indx2, 
	 *  xobj->label.y_indx1 and xobj->label.y_indx2
	 */
        LabelCalculatePixelCursorXY(widget, xobj->label.cursor_xloc,
                                   xobj->label.cursor_yloc);
 
        if (xobj->label.cut_cursor_pos1 != xobj->label.cut_cursor_pos2)
        {
            start_highlt = kmin(xobj->label.cut_cursor_pos1,
                                xobj->label.cut_cursor_pos2);
            end_highlt   = kmax(xobj->label.cut_cursor_pos1,
                                xobj->label.cut_cursor_pos2);
            LabelDetermineCursorXYLoc(xobj->label.lines, xobj->label.numlines,
                         start_highlt, &start_xloc, &start_yloc, &start_highlt);
            LabelDetermineCursorXYLoc(xobj->label.lines, xobj->label.numlines,
                         end_highlt, &end_xloc, &end_yloc, &end_highlt);
        }

	/*
 	 * draw the lines of text in the Label widget which are visible
	 */
	for (i = xobj->label.y_indx1; i <= xobj->label.y_indx2; i++)
	   LabelDrawLine(widget, i, start_xloc, start_yloc, end_xloc, end_yloc);

	/*
	 *  the following code is really a kludge, but it is the easiest and
	 *  most efficient way to deal with the case where the text widget 
	 *  parent is not as wide as the text being displayed by the label
	 *  object, and the label over-writes the shadow border of the text
	 *  widget.  to get around this, we break the wall of the manager,
	 *  can call ManagerDrawBorder() to draw the text widget's shadow
	 *  border again (so that it doesn't *look* as though any over-writing
	 *  ever went on).
	 */
	if (XtIsSubclass(XtParent(widget), ManagerWidgetClass))
	{
	   Widget parent = XtParent(widget);
	   XvwManagerWidget xwid = (XvwManagerWidget) parent;

           ManagerDrawBorder(parent, NULL, xwid->manager.top_shadow_gc,
                xwid->manager.bottom_shadow_gc, xwid->manager.shadow_type,
                xwid->manager.shadow_thickness, 0, 0, (unsigned int)
                parent->core.width, (unsigned int) parent->core.height);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelRefreshContents
|
|       Purpose: Utility routine refreshes only the changed lines 
|		 between the current and new label.
|
|         Input: current - the currently displayed label widget
|                new1    - the updated label widget needed refreshing
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 10, 1997
|
------------------------------------------------------------*/

void LabelRefreshContents(
    Widget current,
    Widget new1)
{
	XvwLabelGadget cobj = (XvwLabelGadget) current;
	XvwLabelGadget nobj = (XvwLabelGadget) new1;

	char **clines, **nlines;
        int  i, min_pos, max_pos;
	int  cxstart = 0, cxend = 0, nxstart = 0, nxend = 0;
	int  cystart = 0, cyend = 0, nystart = 0, nyend = 0;

	/*
	 *  If the object has not been realized then return
	 */
	if (nobj->manager.realized == FALSE) return;

	/*
	 *  calculate nobj->label.cursor_x, nobj->label.cursor_y,
	 *  as well as column and line offset indices 
	 *  nobj->label.x_indx1, nobj->label.x_indx2, 
	 *  nobj->label.y_indx1 and nobj->label.y_indx2
	 */
        LabelCalculatePixelCursorXY(new1, nobj->label.cursor_xloc,
                                    nobj->label.cursor_yloc);
 
	/*
	 *  compute the current highlight bounding box
	 */
        min_pos = kmin(cobj->label.cut_cursor_pos1,cobj->label.cut_cursor_pos2);
        max_pos = kmax(cobj->label.cut_cursor_pos1,cobj->label.cut_cursor_pos2);
        LabelDetermineCursorXYLoc(cobj->label.lines, cobj->label.numlines,
                         min_pos, &cxstart, &cystart, &min_pos);
        LabelDetermineCursorXYLoc(cobj->label.lines, cobj->label.numlines,
                         max_pos, &cxend, &cyend, &max_pos);

	/*
	 *  compute the new highlight bounding box
	 */
        min_pos = kmin(nobj->label.cut_cursor_pos1,nobj->label.cut_cursor_pos2);
        max_pos = kmax(nobj->label.cut_cursor_pos1,nobj->label.cut_cursor_pos2);
        LabelDetermineCursorXYLoc(nobj->label.lines, nobj->label.numlines,
                         min_pos, &nxstart, &nystart, &min_pos);
        LabelDetermineCursorXYLoc(nobj->label.lines, nobj->label.numlines,
                         max_pos, &nxend, &nyend, &max_pos);

	/*
 	 * in case the new object is smaller then erase the extra lines from
	 * old object.
	 */
	for (i = nobj->label.numlines; i < cobj->label.numlines; i++)
	   LabelEraseLine(current, i);

	/*
 	 * if the viewable portion of the label has shrunk then erase the
	 * lines that were previously visible.
	 */
	for (i = nobj->label.y_indx2+1; i <= cobj->label.y_indx2; i++)
	   LabelEraseLine(current, i);

	/*
 	 * draw the lines of text in the Label widget which are visible
	 * and have text changes or highlight changes
	 */
	clines = cobj->label.lines; nlines = nobj->label.lines;
	for (i = nobj->label.y_indx1; i <= nobj->label.y_indx2; i++)
	{
	   /*
	    *  if  our beginning x&y drawing indx (x_indx1 or y_indx1) have
	    *  changed or if the contents of the line we are displaying has
	    *  changed then redisplay the line, otherwise leave it alone.
	    */
	   if (cobj->label.x_indx1 != nobj->label.x_indx1 ||
	       cobj->label.y_indx1 != nobj->label.y_indx1 ||
	       i >= cobj->label.numlines || kstrcmp(clines[i], nlines[i]))
	   {
	       LabelDrawLine(new1, i, nxstart, nystart, nxend, nyend);
	   }
	}

	/*
	 *  If we have any highlight changes then refresh the lines.
	 *  We check this by seeing if the starting or ending y points
	 *  have changed.  If so then refresh those lines, otherwise
	 *  if start or end x points have changed just refresh the one line.
	 */
	if (cystart != nystart)
	{
	    /* case 1: change to the Y start highlight position */
	    for (i = kmin(cystart, nystart); i <= kmax(cystart, nystart); i++)
	       LabelDrawLine(new1, i, nxstart, nystart, nxend, nyend);
	}
	    /* case 2: change to the X start highlight position */
	else if (cxstart != nxstart)
	    LabelDrawLine(new1, nystart, nxstart, nystart, nxend, nyend);

	if (cyend != nyend)
	{
	    /* case 3: change to the Y end highlight position */
	    for (i = kmin(cyend, nyend); i <= kmax(cyend, nyend); i++)
	       LabelDrawLine(new1, i, nxstart, nystart, nxend, nyend);
	}
	    /* case 4: change to the X end highlight position */
	else if (cxend != nxend)
	    LabelDrawLine(new1, nyend, nxstart, nystart, nxend, nyend);

	/*
	 *  the following code is really a kludge, but it is the easiest and
	 *  most efficient way to deal with the case where the text widget 
	 *  parent is not as wide as the text being displayed by the label
	 *  object, and the label over-writes the shadow border of the text
	 *  widget.  to get around this, we break the wall of the manager,
	 *  can call ManagerDrawBorder() to draw the text widget's shadow
	 *  border again (so that it doesn't *look* as though any over-writing
	 *  ever went on).
	 */
	if (XtIsSubclass(XtParent(new1), ManagerWidgetClass))
	{
	   Widget parent = XtParent(new1);
	   XvwManagerWidget xwid = (XvwManagerWidget) parent;

           ManagerDrawBorder(parent, NULL, xwid->manager.top_shadow_gc,
                xwid->manager.bottom_shadow_gc, xwid->manager.shadow_type,
                xwid->manager.shadow_thickness, 0, 0, (unsigned int)
                parent->core.width, (unsigned int) parent->core.height);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelDrawHighlight
|
|       Purpose: This utility routine draws highlighted text for the
|		 Label widget.  Used by the Redisplay() method of the
|		 Label widget, it highlights text by reversing the foreground
|		 and background colors; the highlighted text indicates to
|		 the user what is in the cut buffer for cut-and-paste purposes
|		 (applicable only when the Label widget is being by the Text
|		 widget). 
|
|         Input: widget - the label object that was exposed
|                event  - the event that caused the redraw
|                region - the region that was exposed
|    Written By: Danielle Argiro & Mark Young!
|          Date: April 9, 1997
|
------------------------------------------------------------*/

void LabelDrawHighlight(
    Widget widget,
    char *line,
    int x,
    int y,
    int yloc,
    int start_xloc,
    int start_yloc,
    int end_xloc,
    int end_yloc)
{
	XvwLabelGadget xobj = (XvwLabelGadget) widget;

        char *tmp = line;
        int  len = kstrlen(tmp), incr_width = 0;

        Window  window   = XtWindow(widget);
        Display *display = XtDisplay(widget);
        int  ydist       = xobj->label.multiline ? xobj->label.ydist : 0;
        int  ascent      = (int) xobj->label.string_ascent;
        int  line_height = (int) xobj->label.string_ascent +
                                 xobj->label.string_descent + ydist;
        /*
         * if we are offsetting into the window with x_indx1, need
         * to take that into account before we start drawing the string
         */
        start_xloc -= xobj->label.x_indx1;
        end_xloc   -= xobj->label.x_indx1;
 
        /* this is just a sanity check */
        end_xloc   = kmin(end_xloc, len);
        start_xloc = kmin(start_xloc, len);
 
        /*
         * ready to start drawing current line
         */
        if (yloc == start_yloc && start_xloc > 0)
        {
            /*
             * the place to begin highlighting is past the first character
             * of the string, so draw first part of string in regular fg
             */
            incr_width = XTextWidth(xobj->manager.font, tmp, start_xloc);
            XFillRectangle(display, window, xobj->manager.background_gc,
                            x, y-ascent, (unsigned) incr_width,
                           (unsigned) line_height+1);
            XDrawString(display, window, xobj->manager.foreground_gc,
                        x, y+ydist/2, tmp, start_xloc);
 
            tmp += start_xloc; len -= start_xloc;
            if (start_yloc == end_yloc)
               end_xloc -= start_xloc;
        }
 
        /*
         *  this whole line needs to be highlighted
         */
        if (yloc != end_yloc)
        {
            XFillRectangle(display, window, xobj->manager.foreground_gc,
                         x+incr_width, y-ascent,
                         (unsigned) (xobj->rectobj.width - incr_width - x) + 2,
                         (unsigned) line_height+1);
            XDrawString(display, window, xobj->manager.background_gc,
                        x+incr_width, y+ydist/2, tmp, len);
            incr_width += XTextWidth(xobj->manager.font, tmp, len);
        }
        else
        {
            /*
             * need to highlight a text segment, which may or may not reach
             * all the way to the end of the string.
             */
            if (end_xloc > 0)
            {
                int fill_width;
 
                fill_width = XTextWidth(xobj->manager.font, tmp, end_xloc);
                XFillRectangle(display, window, xobj->manager.foreground_gc,
                            x+incr_width, y-ascent,
                            (unsigned) fill_width, (unsigned) line_height+1);
                XDrawString(display, window, xobj->manager.background_gc,
                            x+incr_width, y+ydist/2, tmp, end_xloc);
                incr_width += fill_width; tmp += end_xloc; len -= end_xloc;
            }

            /*
             * if we are done highlighting on this line and there are still
             * characters left in the string, draw the in the regular fg color
             */
            if (len >= 0)
            {
                XFillRectangle(display, window, xobj->manager.background_gc,
                          x+incr_width, y - ascent,
                          (unsigned) (xobj->rectobj.width - incr_width - x) + 2,
                          (unsigned) line_height+1);
                XDrawString(display, window, xobj->manager.foreground_gc,
                            x+incr_width, y+ydist/2, tmp, len);
            }
            else
            {
                XFillRectangle(display, window, xobj->manager.foreground_gc,
                          x+incr_width, y - ascent,
                          (unsigned) (xobj->rectobj.width - incr_width - x) + 2,
                          (unsigned) line_height+1);
            }
        }
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelDrawCursor
|
|       Purpose: Utility routine redraws the contents of the label
|		 that is currently displayed.
|
|         Input: widget - the label object that needs to be redrawn
|		 erase  - TRUE  if we are erasing the cursor,
|		          FALSE if we are drawing it.
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 10, 1997
|
------------------------------------------------------------*/

void LabelDrawCursor(
    Widget widget,
    GC gc)
{
	XvwLabelGadget xobj = (XvwLabelGadget) widget;
        int char_height = (int) xobj->label.string_ascent+
                                xobj->label.string_descent;
        int ydist       = xobj->label.multiline ? xobj->label.ydist : 0;
	int cursor_x, cursor_y; 
	Display *display = XtDisplay(widget);
	Window  window = XtWindow(widget);
	XSegment segments[3];

	/*
	 *  if label is not editable, we never draw a cursor
	 */
	if (!xobj->label.selectable || !xobj->manager.realized ||
	    !xobj->label.display_cursor)
	   return;

	cursor_x = xobj->label.cursor_x - 1;
	cursor_y = xobj->label.cursor_y + ydist/2 + 2;

	/* horiz line on top */
	segments[0].x1 = cursor_x-2;      
	segments[0].y1 = cursor_y - char_height;
        segments[0].x2 = cursor_x+2;        
	segments[0].y2 = cursor_y - char_height;

	/* horiz line on bottom */
	segments[1].x1 = cursor_x-2;
        segments[1].y1 = cursor_y;
	segments[1].x2 = cursor_x+2;
        segments[1].y2 = cursor_y;

	/* vertical line */
	segments[2].x1 = cursor_x;
        segments[2].x2 = cursor_x;
	segments[2].y1 = cursor_y-char_height;
	segments[2].y2 = cursor_y;

	/* draw cursor using segments */
	XDrawSegments(display, window, gc, segments, 3);
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelFindXOffset
|
|       Purpose: This utility routine finds the x offset to be
|		 used in drawing the label. The x offset is needed
|		 in order to do center and right justification; if
|		 we only did left justification, this wouldn't be needed.
|
|         Input: widget - the label widget to find the x offset
|	         line   - the line to compute the x offset for
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 10, 1997
|
------------------------------------------------------------*/
int LabelFindXOffset(
    Widget widget,
    char *line)
{
	XvwLabelGadget xobj = (XvwLabelGadget) widget;

	char *tmp;
	int  xoffset, length, thickness, label_width, string_width;

        length    = kstrlen(line); 
	thickness = xobj->manager.shadow_thickness + 1;
	label_width = xobj->rectobj.width - 2*thickness - 4;

	/* if line is NULL, still need a sample string to calculate offset,
	   just using an "A", cause why not */
	if (length == 0) 
	{
	    tmp = "A";
	    length = 1;
	}
	else
	    tmp = line;

        string_width = XTextWidth(xobj->manager.font, tmp, length);

	/* 
	 * default case (KLABEL_JUSTIFY_LEFT) has an xoffset of 2 pixels 
	 * to the right of the left edge (add shadow thickness so you don't
	 * over-write the border
	 */
	xoffset = xobj->rectobj.x + thickness + 2;
        if (string_width < label_width)
        {
            if (xobj->label.justify == KLABEL_JUSTIFY_CENTER)
                xoffset += (label_width - string_width)/2;
            else if (xobj->label.justify == KLABEL_JUSTIFY_RIGHT)
                xoffset += label_width - string_width;
        }
	return(xoffset);
}
  
/*-----------------------------------------------------------
|
|  Routine Name: LabelFindYOffset
|
|       Purpose: This utility routine finds the y offset to be
|                used in drawing the label. The y offset is needed
|		 to compensate for the height of the first line
|		 of text in the window.  The height of a text line
|		 is simply the maximum height of a letter in the 
|		 font being used.
|
|         Input: widget - the label widget to find the y offset
|    Written By: Mark Young & Danielle Argiro
|          Date: Feb 24, 1997
|
------------------------------------------------------------*/
int LabelFindYOffset(
    Widget widget)
{
        XvwLabelGadget xobj = (XvwLabelGadget) widget;

        int thickness, yoffset;

        thickness = xobj->manager.shadow_thickness + 1;
        yoffset = xobj->rectobj.y + thickness + xobj->label.string_ascent;
        return(yoffset);
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelCompileString
|
|       Purpose: This routine actually the single string that was
|		 handed to the label widget via the XVW_LABEL_STRING
|		 attribute string into a multiline detab'ed string
|		 so that what the label widget actually displays is
|		 an array of text lines.
|
|         Input: widget - the label object
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Apr 10, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
void LabelCompileString(
   Widget widget)
{
	XvwLabelGadget xobj = (XvwLabelGadget) widget;

	size_t size, numlines = 0;
	char   *tmp = NULL, *string, *ptr, **lines = NULL;

	string = xobj->label.string;
	do
	{
	    /* look for new line to do a break */
	    if ((ptr = kstrchr(string, '\n')) != NULL)
	    {
	       size = ptr-string;
	       ptr++;
	    }
	    else
	       size = kstrlen(string);

	    /* make a local copy */
	    tmp = kstring_ncopy(string, size, NULL);

	    /* if there are any tabs then call kstring_detab() */
	    if (kstrchr(tmp, '\t') != NULL)
	    {
	       char *detab = kstring_detab(tmp, NULL);
	       kfree(tmp); tmp = detab;
	    }

	    /* add the line to our lines array and reset string to ptr */
	    string = ptr;
	    lines = (char **) karray_insert(lines, KSTRING, tmp, numlines++,
					    KLIST_TAIL, TRUE);
	} while (string != NULL);

	/*
	 *  Free the old lines before saving the new ones
	 */
	karray_free(xobj->label.lines, KSTRING, (size_t) xobj->label.numlines,
		    NULL);

	/*
	 * Save the text and number of lines...
	 */
	xobj->label.lines    = lines;
	xobj->label.numlines = numlines;
	xobj->label.y_indx2 = kmax(0, numlines-1);
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelDetermineCursorPos
|
|       Purpose: Given an (x,y) location for the cursor in characters,
|                eg, character 4 of line 8, figures out what the overall
|		 cursor location is in characters - eg, character 54.
|		 (functionality opposite of LabelDetermineCursorXYLoc)
|
|         Input: lines      - the compiled lines of label being displayed
|                numlines   - the number of lines
|                xloc       - x location of cursor (in characters)
|                yloc       - y location of cursor (in characters)
|        Output: cursor_pos - the overall character position.
|
|    Written By: Danielle Argiro & Mark Young
|          Date: April 8, 1997
|
------------------------------------------------------------*/
void LabelDetermineCursorPos(
   char **lines,
   int  numlines,
   int  xloc,
   int  yloc,
   int  *cursor_pos)
{
	int i;

	if (lines == NULL)
	{
	    *cursor_pos = 0;
	    return;
	}

	if (yloc >= numlines)
   	    yloc = numlines-1;

        /*
         *  from 0, add up the string lengths (+1 for \n) 
	 *  of all of the text lines.
         */
        *cursor_pos = 0;
        for (i = 0; i < yloc; i++)
           *cursor_pos += kstrlen(lines[i])+1;

	/* bounds check x location */
	if (xloc >= kstrlen(lines[i]))
   	    xloc = kstrlen(lines[i]);

        *cursor_pos += xloc;
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelDetermineCursorXYLoc
|
|       Purpose: Given a cursor location in characters - eg, character 54,
|		 figures the (x,y) location for the cursor in characters
|                eg, character 4 of line 8.
|		 (functionality opposite of LabelDetermineCursorPos)
|
|	 	 If the cursor position passed in results in an x or y location 
|		 that is past the bounds of the text lines (to the right or the 
|		 bottom) the xloc and/or yloc will be re-adjusted so that they
|		 stay within the bounds of the text lines. In this case, the
|		 new_cursor_pos output will differ from the cursor_pos input,
|		 indicating that the input cursor position had better be 
|		 changed to match it.  Usually, the check and assignment is
|		 abbreviated by passing in the same argument for both.
|
|         Input: lines      - the compiled lines of label being displayed
|		 numlines   - number of lines
|                cursor_pos - overall cursor position (in characters)
|
|        Output: xloc       - x location of cursor (in characters)
|                yloc       - y location of cursor (in characters)
|                new_cursor_pos - in the case that cursor position was past
|                             the end of text lines, outputs the cursor
|			      position that reflects the 
|
|    Written By: Mark Young & Danielle Argiro
|          Date: April 9, 1997
|
------------------------------------------------------------*/
void LabelDetermineCursorXYLoc(
   char **lines,
   int  numlines,
   int  cursor_pos,
   int  *xloc,
   int  *yloc,
   int  *new_cursor_pos)
{
	int i, len, pos = 0;

	/* sanity / quick check */
	if (cursor_pos <= 0)
	{
	   *xloc = *yloc = *new_cursor_pos = 0;
	   return;
	}

	/* look for y location */
        for (i = 0; i < numlines-1; i++)
        {
            len = kstrlen(lines[i])+1;
	    if (pos+len > cursor_pos)
	        break;
	    else pos += len;
        }
	*yloc = i;

	/* 
	 * pos is the sum of all the characters in all previous lines,
	 * excluding the current line, so subtracting that from the cursor
	 * position gives us the x location of the cursor
	 */
	*xloc = cursor_pos - pos;
	*new_cursor_pos = cursor_pos;

	/* prevent x location from going off the end of the line */
	len = kstrlen(lines[*yloc]);
	if (*xloc > len)
	{
	   *new_cursor_pos -= *xloc-len;
	   *xloc = len;
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelCalculatePixelCursorXY
|
|       Purpose: Given an (x,y) location for the cursor in characters,
|                eg, character 4 of line 8, figures out what that location 
|		 is in pixels - eg, (88, 104).  Stores the values as
|                as xobj->label.cursor_x and xobj->label.cursor_y
|                respectively.  If the character position of the cursor
|		 would put the cursor outside the parent window, then
|		 xobj->label.x_indx1 and xobj->label.y_indx1 are
|		 set to indicate how many characters to offset in each
|		 direction before displaying the label. 
|
|		 Note that if the number of characters in a line is
|		 greater than 0, then the number of x cursor positions
|		 in that line is the number of characters in that line.
|		 For example, if we have:
|		 abc
|		 The cursor position can be 0 (before the "a"), 
|		 1, 2, or 3 (after the "c").
|
|         Input: widget   - the label object
|                xloc     - x location of cursor (in characters)
|                yloc     - y location of cursor (in characters)
|
|    Written By: Danielle Argiro & Mark Young
|          Date: February 19, 1997
|
------------------------------------------------------------*/

void LabelCalculatePixelCursorXY(
   Widget widget,
   int    xloc,
   int    yloc)
{
   	XvwLabelGadget xobj = (XvwLabelGadget) widget;
	int  length, xoffset, yoffset, thickness, max_viewable_lines; 
	int  char_width, line_height, core_width, core_height, tmplen;
        int  ydist = xobj->label.multiline ? xobj->label.ydist : 0;

        char *tmp, *line = NULL;

	/*
	 * if the label gadget is not realized, none of this makes sense
	 * as it all depends on the core geometry values being set.
	 */
	if (!xobj->manager.realized) return;

	/*
	 * stats needed for calculations
	 */
	if (xobj->label.lines != NULL)
            line = xobj->label.lines[yloc];

	length = kstrlen(line);

        core_width  = (int) (XtParent((Widget)xobj))->core.width;
        core_height = (int) (XtParent((Widget)xobj))->core.height;
        line_height = (int) (xobj->label.string_ascent+
                             xobj->label.string_descent + ydist);
	if (length > 0)
	    char_width = XTextWidth(xobj->manager.font, line, length)/length;
	else char_width  = (int) xobj->manager.font->max_bounds.width;

        /* offset according to justification */
        xoffset = LabelFindXOffset(widget, line);

        /* offset according to character height & shadow thickness */
	yoffset = LabelFindYOffset(widget);

        /* 
	 * calculate what the DC X cursor position would be at the end of the
	 * string, based on the size of the text line and the font, and 
	 * assuming that it all fits in the parent window (which it may not)
	 */
	xobj->label.cursor_x = xoffset;

	/* if the line length is zero, cursor position is justif. offset only */
        if (length == 0)
        {
            xobj->label.x_indx1 = 0;
        }

	/* if the xloc > linelength, cursor position is at end of string */
        else if (xloc >= length)
	{
	    if (xobj->label.x_indx1 >= length)
		xobj->label.x_indx1 = kmax(0, length-1);

	    tmplen = length - xobj->label.x_indx1;
	    tmp = &line[xobj->label.x_indx1];
            xobj->label.cursor_x += XTextWidth(xobj->manager.font, tmp, tmplen);
	}

	/* otherwise, get DC cursor X position from size of string at xloc */
        else 
        {
	    if (xobj->label.x_indx1 > xloc)
	        xobj->label.x_indx1 = xloc;

	    tmplen = xloc - xobj->label.x_indx1;
	    tmp = &line[xobj->label.x_indx1];
	    xobj->label.cursor_x += XTextWidth(xobj->manager.font, tmp, tmplen);
        }

	/*
	 * now, see if that DC X cursor position is too far to the right to
	 * fit in the window.  if it is, have to find the minimum x_indx1
	 * index to offset into the string so that the desired cursor position
	 * is in the window, and then reset the DC X cursor position.
	 */
        if (xobj->label.cursor_x >= core_width-char_width)
        {
	    int i = 0, text_width = xobj->label.cursor_x;
	    while (text_width >= core_width-char_width && (xloc-i >= 0))
	    {
                tmp = &line[i];
		tmplen = xloc - i;
                text_width = XTextWidth(xobj->manager.font,tmp, tmplen)+xoffset;
		i++;
	    }
            xobj->label.x_indx1 = i-1;
	    xobj->label.cursor_x = text_width;
        }

	/* 
	 * or, if the DC X cursor position is less than 0, then have to 
	 * decrement the x_indx1 index that controls the segment of the line 
	 * that appears in the window; this will cause the line segment to 
	 * one more character from the beginning of text line  (note, this
	 * case only comes from cursor movement via arrow keys in the text
	 * widget, so we don't have to worry about decrementing by more than 1).
	 */
        else if (xobj->label.cursor_x < 0)
	{  
	    if (xobj->label.x_indx1 > 0)
	        xobj->label.x_indx1--;
	    xobj->label.cursor_x = xoffset;
	}

	/* 
	 * or, if desired x location is less than offset, pin it at the offset
	 */
        else if (xobj->label.cursor_x <= xoffset)
	{  
	    xobj->label.cursor_x = xoffset;
	}

        /*
         * if the y location exceeds max # lines that the window
         * can accommodate, then have to adjust index that controls
         * the number of text lines that appear in the window
         */
	thickness    = xobj->manager.shadow_thickness + 1;
        max_viewable_lines  = (int) (core_height-2*thickness)/line_height;
        if (yloc  >= max_viewable_lines + xobj->label.y_indx1)
        {
	    xobj->label.y_indx1 = yloc + 1 - max_viewable_lines;
	    xobj->label.cursor_y = line_height * (yloc - xobj->label.y_indx1) +
				   yoffset;
        }

        /*
         * or, if desired y location is less than 0, then have to adjust the
         * index that controls the number of lines that appear in the
         * window to have more lines from beginning of text 
         */
        else if (yloc - xobj->label.y_indx1 <= 0)
        {
            xobj->label.y_indx1 = yloc;
            xobj->label.cursor_y = yoffset;
        }

	/*
	 * otherwise, just set cursor_y position, taking into account that
	 * we may be offsetting into the array of lines with y_indx1.
	 */
        else
	{
	    xobj->label.cursor_y = line_height * (yloc - xobj->label.y_indx1) +
				   yoffset;
	}

	/*
	 * also set y_indx2, which is the index of the last line to fit
	 * in the window; this value is used in Redisplay().
	 */
        xobj->label.y_indx2 = kmin(xobj->label.numlines-1,
                                   xobj->label.y_indx1+max_viewable_lines-1);

        if (xobj->label.y_indx2 < 0) xobj->label.y_indx2 = 0;
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelCalculateCharCursorXY
|
|       Purpose: Given an (x,y) location for the cursor in pixels,
|		 figures out what that location is in characters -
|		 eg, character 4 of line 8. These values are stored
|		 in char_x and char_y respectively.
|
|         Input: widget - the label widget
|                x      - x location of cursor (in pixels)
|                y      - y location of cursor (in pixels) 
|        Output: char_x - x location of cursor (in characters)
|                char_y - y location of cursor (in characters)
|	Returns: TRUE if (x,y) passed in was off the end of the line
|		 FALSE otherwise
|
|    Written By: Danielle Argiro & Mark Young
|          Date: Mar 05, 1997
|
------------------------------------------------------------*/
int LabelCalculateCharCursorXY(
	Widget widget,
        int x,
        int y,
        int *char_x,
        int *char_y)
{
   	XvwLabelGadget xobj = (XvwLabelGadget) widget;
	int numlines = xobj->label.numlines;
	int off_the_end = FALSE, i, xloc, yloc, xoffset, length; 
	int yoffset, temp_width, save_width, line_height, tmplen;
        int ydist = xobj->label.multiline ? xobj->label.ydist : 0;

	char *tmp, *line = NULL;

	/*
	 * First, figure out y location of button press in characters.
	 * This is not too bad.
	 * 
	 * 1. get the height of one line in the label, and using that, 
	 *    it is easy to calculate the X position of the cursor
	 *    in characters rather than pixels. Note you have to look at
	 *    the manager font structure, not xobj->label.string_ascent or
	 *     xobj->label.string_descent which might still be 0.  
	 * 2. bounds-check 
	 */
        yoffset = xobj->rectobj.y + xobj->manager.shadow_thickness + 1;
	line_height = xobj->label.string_ascent + 
		      xobj->label.string_descent + ydist;
	yloc = (y - yoffset)/line_height + xobj->label.y_indx1;
	if (yloc >= numlines) yloc = numlines-1;
	else if (yloc < 0) yloc = 0;

	/*
	 * stats needed for calculations
	 */
	if (xobj->label.lines != NULL)
            line = xobj->label.lines[yloc];

	length = kstrlen(line);

	/*
	 * 2. next, get the width of the line itself so we can 
	 * 3. figure the character width, and using that 
	 * 4. decide where the button press was in terms of characters, and
	 * 5. bounds-check
	 */
	if (length == 0)
	   xloc = 0;
	else
	{
	    if (xobj->label.x_indx1 >= length)
                xobj->label.x_indx1 = kmax(0, length-1);

	    tmp = &line[xobj->label.x_indx1];
	    tmplen = length - xobj->label.x_indx1;
  	    xoffset = LabelFindXOffset(widget, line);
	    temp_width = XTextWidth(xobj->manager.font, tmp, tmplen) + xoffset;

	    if (x <= xoffset)
                xloc = xobj->label.x_indx1;
	    else if (x >= temp_width)
	    {
                xloc = length+1;
		off_the_end = TRUE;
	    }
	    else
            {
	       save_width = xloc = 0;
	       for (i = xobj->label.x_indx1+1; i < length; i++)
	       {
                   tmp = &line[xobj->label.x_indx1];
		   tmplen = i - xobj->label.x_indx1;
		   temp_width = XTextWidth(xobj->manager.font, tmp, tmplen) +
					   xoffset;

		   if (x >= save_width && x <= temp_width)
		   {
		       /*
		        * if x is within the first half of the character,
			* move the cursor to the left character, otherwise
			* move it to the right character.
		        */
		       if ((x-save_width) <= (temp_width-save_width)/2)
			  xloc = i - 1;
		       else
			  xloc = i;
		       break;
		   }
		   save_width = temp_width;
		}
	        if (i == length) xloc = i;
            }
	}
	*char_x = xloc;
	*char_y = yloc;

	return(off_the_end);
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelPageDown
|
|       Purpose: Utility routine changes y_indx1 and y_indx2 
|		 offsets of the Label to move down by one page
|
|         Input: label - the label object
|    Written By: Danielle Argiro & Mark Young
|          Date: April 27, 1997
|
------------------------------------------------------------*/

void LabelPageDown(
   xvobject label)
{
   	XvwLabelGadget xobj = (XvwLabelGadget) xvw_widget(label);
	int diff;

	/*
	 * case where label is already paged down, or there are not
	 * enough text lines to page
	 */
	if (xobj->label.y_indx2 == xobj->label.numlines-1) 
	    return;

	/*
	 *  page down by difference in number of lines
	 */
	diff = xobj->label.y_indx2 - xobj->label.y_indx1;
	xobj->label.y_indx1 = xobj->label.y_indx2;
	xobj->label.y_indx2 += diff;

	/* bounds check */
	if (xobj->label.y_indx2 > xobj->label.numlines-1)
	    xobj->label.y_indx2 = xobj->label.numlines-1;
	
	/* change cursor y location */
	xobj->label.cursor_yloc += diff;
	if (xobj->label.cursor_yloc > xobj->label.numlines-1)
	    xobj->label.cursor_yloc =  xobj->label.numlines-1;

	/* update overall cursor position from new cursor y location */
	LabelDetermineCursorPos(xobj->label.lines, xobj->label.numlines,
   			xobj->label.cursor_xloc, xobj->label.cursor_yloc,
   			&xobj->label.cursor_pos);

	/* clear the window and redisplay */
	xvw_refresh(label);
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelPageUp
|
|       Purpose: Utility routine changes y_indx1 and y_indx2 
|                offsets of the Label to move up by one page
|
|         Input: label - the label object
|    Written By: Danielle Argiro & Mark Young
|          Date: April 27, 1997
|
------------------------------------------------------------*/

void LabelPageUp(
   xvobject label)
{
        XvwLabelGadget xobj = (XvwLabelGadget) xvw_widget(label);
	int thickness    = xobj->manager.shadow_thickness + 1;
        int ydist        = xobj->label.multiline ? xobj->label.ydist : 0;
        int line_height  = (int) (xobj->label.string_ascent +
                                  xobj->label.string_descent + ydist);
	int core_height  = (int) (XtParent((Widget)xobj))->core.height;
        int max_viewable_lines  = (int) (core_height-2*thickness)/line_height;
 
        /*
         * case where label is already paged up, or there are not
         * enough text lines to page
         */
        if (xobj->label.y_indx1 == 0)
            return;
 
        /*
         *  page up by number of lines you can see in the window
         */
        xobj->label.y_indx2 -= max_viewable_lines-1;
        xobj->label.y_indx1 -= max_viewable_lines-1;
 
        /* bounds check */
        if (xobj->label.y_indx1 < 0)
	{
            xobj->label.y_indx1 = 0;
            xobj->label.y_indx2 = kmin(xobj->label.numlines-1, 
				       max_viewable_lines-1);
	}
 
        /* change cursor y location */
        xobj->label.cursor_yloc -= max_viewable_lines-1;
        if (xobj->label.cursor_yloc < 0)
            xobj->label.cursor_yloc =  0;
 
	/* update overall cursor position from new cursor y location */
	LabelDetermineCursorPos(xobj->label.lines, xobj->label.numlines,
   			xobj->label.cursor_xloc, xobj->label.cursor_yloc,
   			&xobj->label.cursor_pos);

        /* clear the window and redisplay */
        xvw_refresh(label);
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelResetCutCursorPosition
|
|       Purpose: Resets both the cut cursor position 1 and cut cursor
|		 position 2 to the cursor position.  This causes any
|		 highlighted text to get unhighlighted; since we highlight
|		 from cut cursor position 1 to cut cursor position 2, there
|		 is nothing highlighted when they are the same.
|
|         Input: label - the label object
|    Written By: Danielle Argiro & Mark Young
|          Date: April 27, 1997
|
------------------------------------------------------------*/

void LabelResetCutCursorPosition(
    xvobject label)
{
	XvwLabelGadget xobj = (XvwLabelGadget) xvw_widget(label);
	int cursor_pos = xobj->label.cursor_pos;

        xvw_set_attributes(label, 
                           XVW_LABEL_CUT_CURSOR_POS1, cursor_pos,
                           XVW_LABEL_CUT_CURSOR_POS2, cursor_pos,
                           NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: LabelEnsureCursorVisibility
|
|       Purpose: Make sure that the cursor is actually visible by
|		 resetting the xloc and/or yloc within the viewable
|		 bounds of the display.
|
|         Input: label - the label object
|    Written By: Mark Young & Danielle Argiro
|          Date: Mar 2, 1997
|
------------------------------------------------------------*/

void LabelEnsureCursorVisibility(
    xvobject label,
    xvobject reference)
{
	XvwLabelGadget xobj = (XvwLabelGadget) xvw_widget(label);

	int xloc = xobj->label.cursor_xloc;
	int yloc = xobj->label.cursor_yloc;

	if (xloc < xobj->label.x_indx1)
	    xloc = xobj->label.x_indx1;

	if (yloc < xobj->label.y_indx1)
	    yloc = xobj->label.y_indx1;
	else if (yloc > xobj->label.y_indx2)
	    yloc = xobj->label.y_indx2;

	if (reference != NULL)
	{
	   int height;

	   xvw_get_attribute(reference, XVW_HEIGHT, &height);
	   
	   if ((int) (xobj->rectobj.y+xobj->rectobj.height) < height)
	      yloc = 0;
#if 0
	   else if (yloc > height)
	      yloc = height-1;
#endif
	}
	xvw_set_attributes(label,
		   XVW_LABEL_CURSOR_XLOC, xloc,
		   XVW_LABEL_CURSOR_YLOC, yloc,
		   NULL);
}
