/*
  * Khoros: $Id: List.c,v 1.4.2.6 1997/06/26 18:13:41 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: List.c,v 1.4.2.6 1997/06/26 18:13:41 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros List Widget
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Destroy()
   >>>>                 SetValues()
   >>>>
   >>>>                 ListLabelSelect()
   >>>>                 ListLabelUnselect()
   >>>>
   >>>>                 ListSetUtils()
   >>>>                 ListResizeHandler()
   >>>>                 ListEventHandler()
   >>>>                 ListActionHandler()
   >>>>                 ListSelectTimeout()
   >>>>   Public:
   >>>>                 xvw_create_list()
   >>>>                 xvw_change_list()
   >>>>                 xvw_retrieve_list()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include <kwidgets/ListP.h>


static void     ClassInitialize PROTO((void));
static void     Initialize	PROTO((Widget, Widget, ArgList, Cardinal *));
static void     Destroy		PROTO((Widget));
static Boolean	SetValues       PROTO((Widget, Widget, Widget, ArgList,
				       Cardinal *));
static void ListLabelSelect     PROTO((XvwListWidget, int));
static void ListLabelUnselect   PROTO((XvwListWidget, int));
static int  ListSetUtils        PROTO((xvobject, char *, kaddr));
static void ListResizeHandler   PROTO((xvobject, kaddr, XEvent *, int *));
static void ListEventHandler    PROTO((xvobject, kaddr, XEvent *, int *));
static void ListActionHandler   PROTO((Widget, XEvent *, String *, Cardinal));
static void ListSelectTimeout   PROTO((kaddr, XtIntervalId *));


/*-------------------------------------------------------
|
|   Action Handler
|
--------------------------------------------------------*/

static char defaultTranslations[] =
    "<Key>Up:           Action(IncrementUp)           \n\
     <Key>Down:         Action(IncrementDown)";

static XtActionsRec actionsList[] =
{
  {"Action", (XtActionProc) ListActionHandler}
};

/*-------------------------------------------------------------------*
|
|   Full class attributes
|
--------------------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_LIST_STRINGS,    	NULL,  XtRPointer,  NULL},
{XVW_LIST_SIZE,    	NULL,  XtRInt,	    NULL},
{XVW_LIST_INDEX,    	NULL,  XtRInt,	    NULL},
{XVW_LIST_STRING,    	NULL,  XtRPointer,  NULL},
{XVW_LIST_ITEM_SELECT,  NULL,  XtRCallback, NULL},
{XVW_LIST_ITEM_ACTION,  NULL,  XtRCallback, NULL},
};


/*-------------------------------------------------------------------*
|
|   Full class record constant
|
--------------------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwListWidgetRec, list.field)

static XtResource resources[] = { 

{XVW_LIST_STRINGS, NULL, XtRPointer, sizeof(XtPointer),
      offset(strings), XtRImmediate, (XtPointer) NULL},
{XVW_LIST_SIZE, NULL, XtRInt, sizeof(int),
      offset(size), XtRImmediate, (XtPointer) 0},
{XVW_LIST_INDEX, NULL, XtRInt, sizeof(int),
      offset(indx), XtRImmediate, (XtPointer) -1},
{XVW_LIST_STRING, NULL, XtRPointer, sizeof(XtPointer),
      offset(string), XtRImmediate, (XtPointer) NULL},
{XVW_LIST_ITEM_SELECT, NULL, XtRCallback, sizeof(XtCallbackList),
      offset(select_callback), XtRCallback, (XtPointer) NULL},
{XVW_LIST_ITEM_ACTION, NULL, XtRCallback, sizeof(XtCallbackList),
      offset(action_callback), XtRCallback, (XtPointer) NULL},


{XVW_VP_FORCE_HORIZ, NULL, XtRBoolean, sizeof(Boolean),
	XtOffsetOf(XvwViewportWidgetRec, viewport.force_horizontal),
        XtRImmediate, (XtPointer) FALSE},
{XVW_VP_FORCE_VERT, NULL, XtRBoolean, sizeof(Boolean),
	XtOffsetOf(XvwViewportWidgetRec, viewport.force_vertical),
        XtRImmediate, (XtPointer) FALSE},
{XVW_VP_ALLOW_VERT, NULL, XtRBoolean, sizeof(Boolean),
	XtOffsetOf(XvwViewportWidgetRec, viewport.allow_vertical),
        XtRImmediate, (XtPointer) TRUE},
{XVW_VP_ALLOW_HORIZ, NULL, XtRBoolean, sizeof(Boolean),
	XtOffsetOf(XvwViewportWidgetRec, viewport.allow_horizontal),
        XtRImmediate, (XtPointer) TRUE},
{XVW_VP_USE_RIGHT, NULL, XtRBoolean, sizeof(Boolean),
	XtOffsetOf(XvwViewportWidgetRec, viewport.use_right),
        XtRImmediate, (XtPointer) TRUE},
};
#undef offset


/*-------------------------------------------------------------------*
|
|   Class Declaration for List Widget
|
--------------------------------------------------------------------*/

#define superclass (&xvwViewportWidgetClassRec)

XvwListWidgetClassRec xvwListWidgetClassRec =
{
  {
    (WidgetClass) superclass,		/* superclass		  */	
    "List",				/* class_name		  */
    sizeof(XvwListWidgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    actionsList,			/* actions		  */
    XtNumber(actionsList),		/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    defaultTranslations,		/* tm_table		  */
    XtInheritQueryGeometry,		/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwListWidgetConstraintsRec),  /* constraint_size	  */
    NULL,				    /* initialize	  */
    NULL,				    /* destroy		  */
    NULL,				    /* set_values	  */
    NULL,				    /* extension	  */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			    /* pre-layout routine     */
    XtInheritLayout,		    	    /* post-layout routine    */
    XtInheritChangeSel,			    /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  },  /* XvwManagerWidgetClass fields initialization */
  {
    NULL,  		                    /* extension          */
  },  /* XvwViewportWidgetClass fields initialization */
  {
    NULL,  		                    /* extension          */
  },  /* XvwListWidgetClass fields initialization */
};
#undef superclass

  /* for public consumption */
WidgetClass xvwListWidgetClass = (WidgetClass) &xvwListWidgetClassRec;


/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for Class and Constraint Declarations
|
--------------------------------------------------------------------*/

#undef kwidget
#undef kwidgetclass
#undef kconstraint

#define kwidget(widget)	     (XvwListWidget) (widget)
#define kwidgetclass(widget) (XvwListWidgetClass) (widget)->core.widget_class
#define kconstraint(widget)  (XvwListWidgetConstraints) (widget)->core.constraints


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an
|                instance of a XvwListWidget class has been created.
|                This is where the attributes for the class are
|		 initialized. 
|
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 15, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
	kwidgets_cache_pixmaps();
	xvw_init_attributes(xvwListWidgetClass, attributes,
		XtNumber(attributes), NULL, 0, NULL);
	xvw_load_resources("$DESIGN/objects/library/kwidgets/app-defaults/List");

	xvw_define_attributes(xvwListWidgetClass,
		XVW_LIST_HIGHLT_ELEM,   XtRInt,    ListSetUtils, NULL,
		XVW_LIST_UNHIGHLT_ELEM, XtRInt,    ListSetUtils, NULL,
		XVW_LIST_ADD,           XtRString, ListSetUtils, NULL,
		XVW_LIST_DELETE,        XtRString, ListSetUtils, NULL,
		XVW_LIST_DELETEALL,     XtRString, ListSetUtils, NULL,
		NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial list
|                for the list widget instance.
|
|         Input: request - not used
|                new1     - widget instance after initialization, with
|                          initial image
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 15, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwListWidget xwid = kwidget(new1);

	xwid->list.id      = 0;
	xwid->list.indx    = -1;
	xwid->list.string  = NULL;
	xwid->list.strings = NULL;
	xwid->list.label = xvw_create_label(xvw_object(new1), "Label");
	xvw_set_attributes(xwid->list.label,
     		XVW_LABEL_CURSOR_POS,     0,
     		XVW_LABEL_YDIST,          9,
		XVW_LABEL_MULTILINE,      TRUE,
		XVW_LABEL_DISPLAY_CURSOR, FALSE,
		XVW_LABEL_SELECTABLE,     FALSE,
     		XVW_SHADOW_TYPE,          KSHADOW_NONE,
     		XVW_BORDER_WIDTH,         0,
		NULL);
	xvw_add_event(xwid->list.label, ButtonPressMask, ListEventHandler,new1);
	xvw_add_event(xwid->viewport.plane, StructureNotifyMask,
			ListResizeHandler, (XtPointer) xwid);
	xvw_add_event(xwid->viewport.clip, StructureNotifyMask,
			ListResizeHandler, (XtPointer) xwid);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: Free memory associated with list object,
|		 remove pending timeout (if any)
|
|         Input: widget - the list widget being destroyed
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 15, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
        XvwListWidget xwid = kwidget(widget);
 
	karray_free(xwid->list.strings, KSTRING, 
                    (size_t) xwid->list.size, NULL);

	if (xwid->list.id != 0)
           XtRemoveTimeOut(xwid->list.id);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the public values
|                of a XvwListWidget instance.  The public values
|                which can be changed are all related to the display
|		 of the list.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new1     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 15, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues (
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num)
{
	XvwListWidget cxwid = kwidget(current);
	XvwListWidget nxwid = kwidget(new1);

	if (cxwid->list.strings != nxwid->list.strings)
	{ 
	    int i;

	    /* update the list label with the new strings */
	    xvw_set_attributes(nxwid->list.label,
			XVW_LABEL_LINES, nxwid->list.strings,
			XVW_LABEL_NUMLINES, nxwid->list.size,
			NULL);

	   /* make copies of strings */
	   nxwid->list.strings = (char **) kdupalloc(nxwid->list.strings,
				nxwid->list.size * sizeof (char *));
	   for (i = 0; i < nxwid->list.size; i++)
	       nxwid->list.strings[i] = kstrdup(nxwid->list.strings[i]);

	   karray_free(cxwid->list.strings, KSTRING, 
                    (size_t) cxwid->list.size, NULL);
	}

	if (cxwid->list.indx != nxwid->list.indx)
	{
	    if (nxwid->list.indx < -1 ||
		nxwid->list.indx >= nxwid->list.size)
	    {
		kerror(KWIDGETS, "List SetValues",
		       "Invalid list index specified, %d. "
		       "Legal values include -1 (for no list item selected), "
		       "and 0 to %d, as %d was specified as the list size",
		        nxwid->list.indx, nxwid->list.size - 1, 
			nxwid->list.size);
	    }
	    else if (nxwid->list.indx == -1)
	    {
	        ListLabelUnselect(nxwid, cxwid->list.indx);
		nxwid->list.string = NULL;
	    }
	    else 
	    {
	        ListLabelUnselect(nxwid, cxwid->list.indx);
	        ListLabelSelect(nxwid, nxwid->list.indx);
		nxwid->list.string = nxwid->list.strings[nxwid->list.indx];
	    }
	}

	if (cxwid->list.string != nxwid->list.string)
	{
	    int indx;

	    if (nxwid->list.string == NULL)
	    {
	        ListLabelUnselect(nxwid, cxwid->list.indx);
	        nxwid->list.indx = -1;
	    }
	    else
	    {
		indx = karray_locate(nxwid->list.strings, KSTRING,
				     nxwid->list.string, nxwid->list.size);
		if (indx == -1)
		{
		    kerror(KWIDGETS, "List SetValues",
		           "Invalid list string specified, %s. Legal values "
		           "include NULL (for no list item selected), or any"
		           "string in the array specified.",
		            nxwid->list.string);
		    nxwid->list.string = cxwid->list.string;
		}
	        else if (nxwid->list.string == NULL)
		{
                    ListLabelUnselect(nxwid, cxwid->list.indx);
                    nxwid->list.indx = -1;
		}
	       else
               {
                    ListLabelUnselect(nxwid, cxwid->list.indx);
                    ListLabelSelect(nxwid, indx);
		    nxwid->list.indx = indx;
               }

	    }
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ListResizeHandler
|
|       Purpose: Make sure the label spans the list so that when we
|		 select an element from the list the entire line is
|		 selected.
|
|         Input:
|    Written By: Mark Young
|          Date: May 21, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ListResizeHandler(
   xvobject object,
   kaddr    client_data,
   XEvent   *event,
   int      *dispatch)
{
}

/*-----------------------------------------------------------
|
|  Routine Name: ListLabelSelect
|
|       Purpose: Utility routine selects a label (specified by
|		 index) in the list widget. It does checks to be
|		 sure that index is within range, and simply
|		 returns if the index is out of range.
|
|         Input: xwid - the list widget
|                indx - index of the list item (label) to select
|
|    Written By: Danielle Argiro & Mark Young
|          Date: Jan 15, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static void ListLabelSelect(
   XvwListWidget xwid,
   int           indx)
{
	int len, pos;
	xvobject object = xvw_object((Widget) xwid);

	/* bounds checkng */
        if (indx < 0 || indx >= xwid->list.size)
	    return;

	/* set the cursor to the character 0 of the selected item. */
	xvw_set_attributes(xwid->list.label,
		XVW_LABEL_CURSOR_YLOC, indx,
		XVW_LABEL_CURSOR_XLOC, 0,
		NULL);

	/* get the cursor position and len of the selected item. */
	len = kstrlen(xwid->list.strings[indx])+1;
	xvw_get_attribute(xwid->list.label, XVW_LABEL_CURSOR_POS, &pos);

	/* set the highlighted region of text to be selected item */
	xvw_set_attributes(xwid->list.label,
		XVW_LABEL_CUT_CURSOR_POS1, pos,
		XVW_LABEL_CUT_CURSOR_POS2, pos+len,
		NULL);
	LabelEnsureCursorVisibility(xwid->list.label, object);
}

/*-----------------------------------------------------------
|
|  Routine Name: ListLabelUnselect
|
|       Purpose: Utility routine un-selects a label in the list widget
|
|         Input: xwid - the list widget
|                indx - index of the list item (label) to un-select
|
|    Written By: Danielle Argiro & Mark Young
|          Date: Jan 15, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static void ListLabelUnselect(
   XvwListWidget xwid,
   int           indx)
{
	/* unselect string by calling */
	LabelResetCutCursorPosition(xwid->list.label);
}

/*-----------------------------------------------------------
|  
|  Routine Name: ListSetUtils - set list utilities
|  
|       Purpose: Resource attribute for highlighting, unhightlighting, etc
|		 elements within the list.
|  
|         Input: list - the list object in which we are to select the item
|                attribute - the XVW_LIST_HIGHLT_ELEM, XVW_LIST_UNHIGHLT_ELEM
|                calldata - the item to be selected
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 15, 1997
------------------------------------------------------------*/
/* ARGSUSED */          
static int ListSetUtils(
   xvobject list,     
   char     *attribute, 
   kaddr    calldata)   
{
	XvwListWidget xwid = (XvwListWidget) xvw_widget(list);


	if (kstrcmp(attribute, XVW_LIST_HIGHLT_ELEM) == 0)
	{
	    int indx = *(int *) calldata;
	    ListLabelSelect(xwid, indx);
	    xwid->list.indx = indx;
	    xwid->list.string = xwid->list.strings[indx];
	}
	else if (kstrcmp(attribute, XVW_LIST_UNHIGHLT_ELEM) == 0)
	{
	    int indx = *(int *) calldata;
	    ListLabelUnselect(xwid, indx);
	    xwid->list.indx = -1;
	    xwid->list.string = NULL;
	}
        else if (kstrcmp(attribute, XVW_LIST_ADD) == 0)
        {
	    kstring string = *(kstring *) calldata;
	
	    xwid->list.strings = (char **) karray_add(xwid->list.strings,
				KSTRING, kstrdup(string), xwid->list.size++);

            /* update the list label with the new strings */
            xvw_set_attributes(xwid->list.label,
                        XVW_LABEL_LINES, xwid->list.strings,
                        XVW_LABEL_NUMLINES, xwid->list.size,
                        NULL);
        }
        else if (kstrcmp(attribute, XVW_LIST_DELETE) == 0)
        {
            kstring string = *(kstring *) calldata;
	    int     indx;

            indx = (int) karray_locate(xwid->list.strings, KSTRING,
					string, xwid->list.size);
	    if (indx == -1)
	    {
		kerror(KWIDGETS, "ListSetUtils XVW_LIST_DELETE",
		       "Unable to locate string %s in list for deletion",
		       string);
		return(FALSE);
            }
            xwid->list.strings = (char **) karray_delete(xwid->list.strings,
					KSTRING, string, xwid->list.size);

	    xwid->list.size--;
	    if (xwid->list.indx == indx)
		xwid->list.indx = -1;
	    else if (xwid->list.indx > indx) 
		xwid->list.indx--;

	    if (xwid->list.indx != -1) 
		xwid->list.string = xwid->list.strings[xwid->list.indx];
	    else xwid->list.string = NULL;

            /* update the list label with the new strings */
            xvw_set_attributes(xwid->list.label,
                        XVW_LABEL_LINES, xwid->list.strings,
                        XVW_LABEL_NUMLINES, xwid->list.size,
                        NULL);
        }

	/*
	 *  delete all elements within list
	 */
        else if (kstrcmp(attribute, XVW_LIST_DELETEALL) == 0)
        {
	    karray_free(xwid->list.strings, KSTRING, 
		    (size_t) xwid->list.size, NULL);
	    xwid->list.indx = -1;
	    xwid->list.size = 0;

            /* update the list label with the new strings */
            xvw_set_attributes(xwid->list.label,
                        XVW_LABEL_LINES, xwid->list.strings,
                        XVW_LABEL_NUMLINES, xwid->list.size,
                        NULL);
        }
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ListEventHandler 
|
|       Purpose: Handles events for List widget:
|		 Button Press - selects the item
|		 Enter Notify - emphasizes the string
|		 Leave Notify - de-emphasizes the string
|
|         Input: label       - the label sub-part of the list object 
|                client_data - passes in the index of the label
|                event       - the X event button press, enter / leave notify
|
|    Written By: Danielle Argiro & Mark Young
|          Date: Jan 15, 1997
------------------------------------------------------------*/
/* ARGSUSED */
static void ListEventHandler(
   xvobject label,
   kaddr    client_data,
   XEvent   *event,
   int      *dispatch)
{
	Widget widget = (Widget) client_data;
	XvwListWidget xwid = kwidget(widget);
	xvw_list_struct list_struct;

	*dispatch = FALSE;

	/*
	 *  Button Press implies List Item Select or List Item Action
	 */
	if (event->type == ButtonPress)
	{
	    int xindx, yindx;

	    /*
	     * Checking to see if this is a double click:
	     * 1) index != -1  (it would've been set below)
	     * 2) timeout has been installed and not removed
	     *    (didn't have time to get to timeout before getting
             *     the next button press event)
	     * 3) this label is the same as the one from the last buttonpress
	     */
	    LabelCalculateCharCursorXY(xvw_widget(xwid->list.label),
			event->xbutton.x, event->xbutton.y, &xindx, &yindx);

	    if (yindx == -1)
	        return;

	    if (xwid->list.indx != -1 && xwid->list.id != 0 &&
		yindx == xwid->list.indx)
	    {
		/*
 	         * this was indeed a double click, meaning it's time to
	         * handle a List Action.  Ie, set the fields in the list 
		 * structure, remove timeout that was added last time around, 
		 * fire callbacks for XVW_LIST_ITEM_ACTION, and return.
	         */
                list_struct.string = xwid->list.strings[xwid->list.indx];
                list_struct.list_index = xwid->list.indx;
                list_struct.action = TRUE;

		XtRemoveTimeOut(xwid->list.id);
		xwid->list.id = 0;
                XtCallCallbacks((Widget) xwid, XVW_LIST_ITEM_ACTION,
                                    &list_struct);
		return;
	    }

	    /* 
	     * Remaining code for not-double-clicks:
	     *
	     * 1) if there was previously a label selected, and that one 
             *    is not the same as the one we're on now, un-highlight old
	     *    label.  in other words, we're not supporting the ability to 
	     *    unselect a label once you've selected it.
	     */
	    if (xwid->list.indx != -1 && yindx != xwid->list.indx)
	        ListLabelUnselect(xwid, xwid->list.indx); 

            /* 
	     * 2) search to find which label is selected. 
	     *    then, set the index and highlight the label
	     */
	    xwid->list.indx = yindx;
	    xwid->list.string = xwid->list.strings[xwid->list.indx];

	    ListLabelSelect(xwid, xwid->list.indx);

	    /*
	     * 3) At this point, we are about ready to select this
	     *    item from the list, but we can't since we don't know if the
	     *    second click in a double click is coming in a couple nano-
	     *    seconds from now.  So instead, we install a timeout that
	     *    contains the code to deal with list item selection, and
	     *    let it fire after 1.5*multiclicktime (we use the 1.5 factor
	     *    so you don't have to be lightning fast with the mouse).
	     *    ps. by this time, index should NOT be -1, the if check is 
	     *    just because i'm paranoid.  
	     */
	    if (xwid->list.indx != -1)
	    { 
	       XtAppContext  app_context;
	       unsigned long multiclicktime;
		   
	       app_context    = XtWidgetToApplicationContext(widget);
	       multiclicktime = (unsigned long) 
			        (XtGetMultiClickTime(XtDisplay(widget)) * 1.5);
	       xwid->list.id = XtAppAddTimeOut(app_context, multiclicktime,
                                                    ListSelectTimeout, xwid);
	    }
	    
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: ListSelectTimeout
|
|       Purpose: Contains the code that handles List Select 
|		 (fires callbacks with filled-out call_data
|		 on single button press events).
|
|         Input: client_data - used to pass in the list widget
|                id          - timeout id
|    Written By: Danielle Argiro & Mark Young
|          Date: Jan 15, 1997
|
------------------------------------------------------------*/
static void ListSelectTimeout(
    kaddr client_data,
    XtIntervalId *id)
{
	XvwListWidget  xwid = (XvwListWidget) client_data;
	xvw_list_struct list_struct;

	xwid->list.id = (XtIntervalId) 0;

	list_struct.string     = xwid->list.strings[xwid->list.indx];
        list_struct.list_index = xwid->list.indx;
        list_struct.action     = FALSE;

        XtCallCallbacks((Widget) xwid, XVW_LIST_ITEM_SELECT, &list_struct);
}

/*-----------------------------------------------------------
|
|  Routine Name: ListActionHandler
|
|       Purpose: Handles key press navigation of the list, with
|		 <^> moving up an element in the list
|		 <V> moving down an element in the list
|
|         Input: widget - the wonderous selection in question
|                event  - the event that triggered this mess.
|    Written By: Danielle Argiro & Mark Young
|          Date: Jan 15, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ListActionHandler(
   Widget   widget,
   XEvent   *event,
   String   *params,
   Cardinal num_params)
{
	XvwListWidget xwid = kwidget(widget);

 
        /*
         *  Sanity check to make sure the list is not empty...
         */
        if (xwid->list.size == 0)
            return;

        if (kstrcmp(params[0], "IncrementUp") == 0)
	{
            ListLabelUnselect(xwid, xwid->list.indx);

	    if (xwid->list.indx == -1)
		xwid->list.indx = xwid->list.size;

	    xwid->list.indx--;
	    if (xwid->list.indx == -1) 
		xwid->list.indx = 0;

	    xwid->list.string = xwid->list.strings[xwid->list.indx];
            ListLabelSelect(xwid, xwid->list.indx);
	}
	else if (kstrcmp(params[0], "IncrementDown") == 0)
        {
            ListLabelUnselect(xwid, xwid->list.indx);

	    if (xwid->list.indx == -1)
                xwid->list.indx = -1;

            xwid->list.indx++;
            if (xwid->list.indx > xwid->list.size - 1)
		xwid->list.indx = xwid->list.size - 1;

	    xwid->list.string = xwid->list.strings[xwid->list.indx];
            ListLabelSelect(xwid, xwid->list.indx);
        }
}


/************************************************************
*
*  Routine Name: xvw_create_list - create list GUI object
*
*       Purpose: The \fIlist\fP GUI object presents the user with a set of
*                strings inside a scrolled viewport. A callback can be
*                installed on the list object which will be fired when
*                the user selects an item from the list.
*
*                A list GUI object is really a \fIcompound\fP object;
*                that is, it is made up of two objects: the viewport and the
*                "actual list", where the viewport contains the list and
*                provides scrolling capabilities, and the "actual list" is
*                inside the viewport and contains the list contents.
*
*                When you create a list, the GUI object returned is the
*                viewport object; this object should be referenced when
*                setting geometry, relative offset, and so on (in other words,
*                when setting or getting any attribute not having to do
*                \fIdirectly\fP with the contents of the list, where those
*                particular attributes are called out below).
*
*                The "actual list" part of the object is used when changing
*                the contents of the list, setting the size of the list, and
*                highlighting or unhighlighting elements of the list.  The
*                "actual list" part of the compound list object can be obtained
*                with a call to xvw_retrieve_list().
*
*                When the Athena widget set is used, the list object is
*                instantiated as a xfwfScrolledListWidget.  When the Olit
*                widget set is used, the list object is instantiated as a
*                scrollingListWidget.  When the Motif widget set is used,
*                the list object is instantiated as a ScrolledList.
*
*         Input: parent - parent of the list object
*                name   - a name for this particular instance of the list object
*                         (for use in app-defaults files, etc)
*
*        Output: 
*       Returns: The list object on success, NULL on failure
*
*  Restrictions:
*    Written By: Mark Young & Danielle Argiro
*          Date: Jun 15, 1996
*      Verified: 
*  Side Effects:
* Modifications:
*   Declaration: xvobject xvw_create_list(
*                !   xvobject parent,
*                !   char   *name)
*
*******************************************************************/

xvobject xvw_create_list(
   xvobject parent,
   char     *name)
{
	xvobject object;

	object = xvw_create(parent, TRUE, TRUE, name, xvwListWidgetClass);
	return(object);
}

/************************************************************
*
*  Routine Name: xvw_change_list - change the contents of a list object
*
*       Purpose: Changes the contents of a list object while it is displayed
*
*         Input: object - the list object to have contents changed
*                list   - new contents of list object
*                size   - size of the new list (# entries)
*                resize - TRUE if list object should try to resize
*                         itself after the change, FALSE otherwise.
*  Restrictions: The list object must have been previously
*                 created with a call to xvw_create_list()
*    Written By: Danielle Argiro
*          Date: Apr 9, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
************************************************************/

void xvw_change_list(
   xvobject listobj,
   char   **list,
   size_t size,
   int    resize)
{
	xvw_set_attribute(listobj, XVW_VP_YOFFSET, 0);
	xvw_set_attribute(listobj, XVW_LIST_INDEX, -1);
	xvw_set_attributes(listobj,
		XVW_LIST_SIZE,    size,
		XVW_LIST_STRINGS, list,
		NULL);
}

/************************************************************
*
*  Routine Name: xvw_retrieve_list - get the actual list component of a
*                                    compound list object
*
*       Purpose: Returns the actual list object component of
*                the compound list object returned by xvw_create_list().
*         Input: list - the compound list object for which the 
*                       actual list component is desired
*        Output: 
*       Returns: The actual list object on success, NULL on failure
*  Restrictions:
*    Written By: Danielle Argiro
*          Date: April 21, 1993
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: xvobject xvw_retrieve_list(
*                !   xvobject listobj)
*
*******************************************************************/

xvobject xvw_retrieve_list(
   xvobject listobj)
{
	return(listobj);
}
