/*
  * Khoros: $Id: ManagerObj.c,v 1.15.2.3 1997/04/30 22:33:39 mirage Exp $
  */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: ManagerObj.c,v 1.15.2.3 1997/04/30 22:33:39 mirage Exp $";
#endif
 
 
/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Manager Gadget Subclass
   >>>>  Private:
   >>>>                 ManagerObjForceRedisplay()
   >>>>                 ManagerObjChangeGeometry()
   >>>>                 ManagerObjRealize()
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 ClassPartInitialize()
   >>>>                 Initialize()
   >>>>                 Redisplay()
   >>>>                 Realize()
   >>>>                 SetValues()
   >>>>                 Destroy()
   >>>>                 CopyAttributeFromParent()
   >>>>                 GetGC()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <kwidgets/ManagerObjP.h>

static void ClassInitialize PROTO((void));
static void ClassPartInitialize PROTO((WidgetClass));
static void Initialize      PROTO((Widget, Widget, ArgList, Cardinal *));
static void Redisplay	    PROTO((Widget, XEvent *, Region));
static void Realize         PROTO((Widget));
static void Destroy         PROTO((Widget));
static Boolean SetValues    PROTO((Widget, Widget, Widget, ArgList,Cardinal *));

static void CopyAttributeFromParent PROTO((Widget, int, XrmValue *));
static void GetGC           PROTO((Widget, Widget));

/*-------------------------------------------------------------------*
|
|   Full class attribute
|
--------------------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_FONT,                NULL, XtRFontStruct, NULL},
{XVW_MAPPED,	          NULL, XtRInt,	    XtRBoolean},
{XVW_FORCE_REDISPLAY,     NULL, XtRInt,	    XtRBoolean},
{XVW_SHADOW_TYPE,     	  NULL, XtRInt,	    NULL},
{XVW_SHADOW_THICKNESS,	  NULL, XtRInt,	    XtRDimension},
{XVW_TOP_SHADOW_PIXEL,    NULL, XtRPixel,   NULL},
{XVW_BOTTOM_SHADOW_PIXEL, NULL, XtRPixel,   NULL},
{XVW_TOP_SHADOW_COLOR,    XVW_TOP_SHADOW_PIXEL,    XtRString, XtRPixel},
{XVW_BOTTOM_SHADOW_COLOR, XVW_BOTTOM_SHADOW_PIXEL, XtRString, XtRPixel},
};


/*-------------------------------------------------------------------*
|
|   Full class record constant
|
--------------------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwManagerGadgetRec, manager.field)

static XtResource resources[] = { 
{XVW_MAPPED, NULL, XtRBoolean, sizeof(Boolean),
      offset(mapped), XtRImmediate, (XtPointer) TRUE},
{XVW_FORCE_REDISPLAY, NULL, XtRBoolean, sizeof(Boolean),
      offset(force_redisplay), XtRImmediate, (XtPointer) FALSE},
{XVW_FOREGROUND_PIXEL, NULL, XtRPixel, sizeof(Pixel),
      offset(foreground), XtRString, "#000000"},
{XVW_SHADOW_THICKNESS, NULL, XtRDimension, sizeof(Dimension),
	offset(shadow_thickness), XtRImmediate, (XtPointer) 2},
{XVW_SHADOW_TYPE, NULL, XtRInt, sizeof(int),
	offset(shadow_type), XtRImmediate, (XtPointer) KSHADOW_NONE},
{XVW_TOP_SHADOW_PIXEL, NULL, XtRPixel, sizeof(Pixel),
      offset(top_shadow), XtRString, "#eeeeee"},
{XVW_BOTTOM_SHADOW_PIXEL, NULL, XtRPixel, sizeof(Pixel),
      offset(bottom_shadow), XtRString, "#747474"},
{XVW_FONT, "fontList", XtRFontStruct, sizeof(XFontStruct *),
      offset(font), XtRString, "6x13"},

{XVW_BACKGROUND_PIXEL, NULL, XtRPixel, sizeof(Pixel),
      offset(background),  XtRString, "#c4c4c4"},
{XVW_BORDER_PIXEL, XtNborder, XtRPixel, sizeof(Pixel),
      offset(border_pixel),  XtRString, "#00005a"},
{XVW_DEPTH, NULL, XtRInt, sizeof(int),
      offset(depth), XtRCallProc, (XtPointer) CopyAttributeFromParent},
{XVW_COLORMAP, NULL, XtRColormap, sizeof(Colormap),
      offset(colormap), XtRCallProc, (XtPointer) CopyAttributeFromParent},
{XVW_BACKGROUND_PIXMAP, XtNbackgroundPixmap, XtRPixmap, sizeof(Pixmap),
      offset(background_pixmap), XtRImmediate, (XtPointer) XtUnspecifiedPixmap},
{XtNborderPixmap, NULL, XtRPixmap, sizeof(Pixmap),
      offset(border_pixmap), XtRImmediate, (XtPointer) XtUnspecifiedPixmap},
{XtNtranslations, NULL, XtRTranslationTable, sizeof(XtTranslations),
      offset(tm.translations), XtRTranslationTable, (XtPointer) NULL},
{XtNaccelerators, NULL, XtRAcceleratorTable, sizeof(XtTranslations),
      offset(accelerators), XtRTranslationTable, (XtPointer) NULL},

{XVW_BORDER_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwManagerGadgetRec, rectobj.border_width),
        XtRImmediate, (XtPointer) 0},
};
#undef offset


/*-------------------------------------------------------------------*
|
|   Class Declaration for Manager Gadget
|
--------------------------------------------------------------------*/

#define superclass (&rectObjClassRec)

XvwManagerGadgetClassRec xvwManagerGadgetClassRec =
{
  {
    (WidgetClass) superclass,		/* superclass		  */	
    "ManagerObj",			/* class_name		  */
    sizeof(XvwManagerGadgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    ClassPartInitialize,		/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    (XtProc) Realize,			/* not used - obj	  */
    NULL,				/* not used - obj	  */
    0,					/* not used - obj	  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* not used - obj	  */
    FALSE,				/* not used - obj	  */
    FALSE,				/* not used - obj    	  */
    FALSE,				/* not used - obj	  */
    Destroy,				/* destroy		  */
    NULL,				/* not used - obj	  */
    Redisplay,				/* not used - obj	  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* not used - obj	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* not used - obj	  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* not used - obj	  */
    NULL,				/* not used - obj	  */
    NULL,				/* not used - obj	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    NULL, 		                /* pick object proc    	  */
    DefaultEraseSelection,   		/* erase selected proc    */
    DefaultRefreshSelection,		/* refresh selection proc */
    NULL,				/* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
};
#undef superclass

  /* for public consumption */
WidgetClass xvwManagerGadgetClass = (WidgetClass) &xvwManagerGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an
|		 instance of a managerGadget class has been 
|		 created.  This is where the attributes for
|		 the class are initialized.
|
|    Written By: Mark Young
|          Date: Aug 19, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
	kwidgets_cache_pixmaps();
	xvw_init_attributes(xvwManagerGadgetClass, attributes,
		XtNumber(attributes), NULL, 0, NULL);
	xvw_load_resources("$DESIGN/objects/library/kwidgets/app-defaults/ManagerObj");
}

/*-----------------------------------------------------------
|
|  Routine Name: ClassPartInitialize
|
|       Purpose: This method is called when the first time an
|                instance of XvwManagerGadgetClass or a subclass of
|                the XvwManagerGadgetClass is created.  It handles the
|                inheritence of the picking, erasing, and refresh
|		 selection routines to the subclass. 
|
|         Input: wclass - the widget class  
|    Written By: Mark Young
|          Date: Aug 19, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassPartInitialize(
   WidgetClass wclass)
{
        XvwManagerGadgetClass mclass = (XvwManagerGadgetClass) wclass;
        XvwManagerGadgetClass sclass = (XvwManagerGadgetClass)
                                        wclass->core_class.superclass;

	if (wclass != xvwManagerGadgetClass)
	{
           if (mclass->manager_class.pick_selection == XtInheritPicking)
	   {
              mclass->manager_class.pick_selection =
			sclass->manager_class.pick_selection;
	   }

           if (mclass->manager_class.erase_selection == XtInheritEraseSel)
	   {
              mclass->manager_class.erase_selection =
			sclass->manager_class.erase_selection;
	   }

           if (mclass->manager_class.refresh_selection == XtInheritRefreshSel)
	   {
              mclass->manager_class.refresh_selection =
			sclass->manager_class.refresh_selection;
	   }

           if (mclass->manager_class.change_geometry == XtInheritChangeGeometry)
           {
              mclass->manager_class.change_geometry =
                        sclass->manager_class.change_geometry;
           }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This will initialize the gc for a new XvwManagerGadget.
|                
|         Input: request - not used 
|                new1 - widget instance after initialization, with gc initialized
|    Written By: Mark Young
|          Date: Aug 19, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget request,
   Widget new1,
   ArgList args,
   Cardinal *num_args)
{
	XvwManagerGadget xobj = (XvwManagerGadget) new1;
	Widget parent = XtParent(new1);

	xobj->manager.window = parent->core.window;
        xobj->manager.screen = parent->core.screen;
        xobj->manager.name = XrmNameToString(new1->core.xrm_name);
        xobj->manager.tm.translations = NULL;
	xobj->manager.realized = FALSE;

	xobj->manager.tile = None;
	xobj->manager.foreground_gc    = NULL;
	xobj->manager.background_gc    = NULL;
	xobj->manager.border_gc        = NULL;
	xobj->manager.top_shadow_gc    = NULL;
	xobj->manager.bottom_shadow_gc = NULL;
	xobj->rectobj.border_width     = 0;
	xobj->rectobj.width  = kmax(xobj->rectobj.width,  1);
	xobj->rectobj.height = kmax(xobj->rectobj.height, 1);
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the ManagerObj object in response
|                to an expose event.
|
|         Input: widget - the manager object that was exposed 
|                event  - the event that caused the redraw
|                region - the region that was exposed 
|    Written By: Mark Young
|          Date: Apr 10, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	XvwManagerGadget xobj = (XvwManagerGadget) widget;

        GC      gc;
	int      x, y;
	unsigned int w, h, bw;
	Widget  parent   = XtParent(widget);
        Window  window   = XtWindow(widget);
        Display *display = XtDisplay(widget);


	/*
	 *  Check to see if we aren't realized.  If so then Realize ourselves
	 *  first.
	 */
	if (!xobj->manager.realized)
	{
	   ManagerObjRealize(XtClass(widget), widget);
	   window = XtWindow(widget);
	}

	/*
	 *  Get current coordinates.
	 */
	x  = xobj->rectobj.x;
	y  = xobj->rectobj.y;
	bw = xobj->rectobj.border_width;
	w  = xobj->rectobj.width;
	h  = xobj->rectobj.height;

	if (parent->core.background_pixel  != xobj->manager.background ||
	   (parent->core.background_pixmap != xobj->manager.background_pixmap &&
	    xobj->manager.background_pixmap != XtUnspecifiedPixmap))
	{
	   gc = xobj->manager.background_gc;
	   XFillRectangle(display, window, gc, x, y, w, h);
	}

	/*
         *  Draw the background "border" rectangle.
         */
	if (bw > 0)
	{
	   gc = xobj->manager.border_gc;
           XDrawRectangle(display, window, gc, x, y, w+bw, h+bw);
	   x += kmax(1, bw/2); y += kmax(1, bw/2);
	}

	if (xobj->manager.shadow_thickness > 0)
	{
	   ManagerDrawBorder(widget, region, xobj->manager.top_shadow_gc,
		xobj->manager.bottom_shadow_gc, xobj->manager.shadow_type,
	        xobj->manager.shadow_thickness, x, y, w, h);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: Realize
|
|       Purpose: This routine is superfluous since an
|		 object is not realized by itself.  Superfluous
|		 this!  You ignorant fool!
|
|         Input: widget - the object being realized
|    Written By: Mark Young
|          Date: Oct 19, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Realize(
   Widget widget)
{
        XvwManagerGadget xobj = (XvwManagerGadget) widget;


	/*
	 *  Set the window to our parent's now that we are being realized.
	 */
	widget->core.window = widget->core.parent->core.window;

	/*
	 *  Refresh the if object if force_redisplay is set.
	 */
	GetGC(NULL, widget);
	if (xobj->manager.force_redisplay)
	   ManagerObjForceRedisplay(widget, widget, FALSE, NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues 
|
|       Purpose: This method is used to set the public attributes
|		 of a XvwManagerGadget.  These attributes are
|		 the font, foreground, and background colors.
|
|         Input: current - the object containing current settings
|                request - the object containing requested settings
|                new1     - the object processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Aug 19, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwManagerGadget cobj = (XvwManagerGadget) current;
	XvwManagerGadget nobj = (XvwManagerGadget) new1;
	Boolean redisplay = FALSE, clear_area = FALSE;


	if (nobj->manager.font == NULL)
	   nobj->manager.font = cobj->manager.font;

	if (cobj->manager.font	        != nobj->manager.font 	       ||
            cobj->manager.foreground    != nobj->manager.foreground    ||
	    cobj->manager.background    != nobj->manager.background    ||
	    cobj->manager.border_pixel  != nobj->manager.border_pixel  ||
	    cobj->manager.top_shadow    != nobj->manager.top_shadow    ||
	    cobj->manager.bottom_shadow != nobj->manager.bottom_shadow ||
	    cobj->rectobj.border_width  != nobj->rectobj.border_width  ||
	    XtIsSensitive(current)      != XtIsSensitive(new1))
	{
	   GetGC(current, new1);
	   if (cobj->rectobj.border_width != nobj->rectobj.border_width ||
	       XtIsSensitive(current)     != XtIsSensitive(new1))
	   {
	      redisplay = TRUE, clear_area = TRUE;
	   }

	   if (cobj->manager.background != nobj->manager.background)
	   {
	      Widget parent = new1->core.parent;

	      redisplay = TRUE;
	      if (parent->core.background_pixel == nobj->manager.background)
		 clear_area = TRUE;
	      else
		 clear_area = FALSE;
	   }
	}

	if (cobj->rectobj.x      != nobj->rectobj.x            ||
            cobj->rectobj.y      != nobj->rectobj.y            ||
            cobj->rectobj.width  != nobj->rectobj.width        ||
            cobj->rectobj.height != nobj->rectobj.height       ||
	    cobj->rectobj.border_width != nobj->rectobj.border_width)
	{
	   ManagerObjChangeGeometry(new1->core.widget_class, new1);
	}

	if (nobj->manager.force_redisplay && redisplay == TRUE)
	   ManagerObjForceRedisplay(current, new1, clear_area, &redisplay);

	if (cobj->manager.mapped != nobj->manager.mapped)
	   redisplay = TRUE, clear_area = TRUE;

	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy 
|
|       Purpose: This method will deallocate the gc used 
|                by the XvwManagerGadget before it is destroyed. 
|
|         Input: widget - the widget being destroyed
|    Written By: Mark Young
|          Date: Aug 19, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	XvwManagerGadget xobj = (XvwManagerGadget) widget;

        if (xobj->manager.foreground_gc)
           XtReleaseGC(widget, xobj->manager.foreground_gc);
        if (xobj->manager.background_gc)
           XtReleaseGC(widget, xobj->manager.background_gc);
        if (xobj->manager.border_gc)
           XtReleaseGC(widget, xobj->manager.border_gc);
        if (xobj->manager.top_shadow_gc)
           XtReleaseGC(widget, xobj->manager.top_shadow_gc);
        if (xobj->manager.bottom_shadow_gc)
           XtReleaseGC(widget, xobj->manager.bottom_shadow_gc);
	if (xobj->manager.tile != None)
	   XmuReleaseStippledPixmap(XtScreen(widget), xobj->manager.tile);
}

/*-----------------------------------------------------------
|
|  Routine Name: CopyAttributeFromParent - copy attribute from parent
|
|    Written By: Mark Young
|          Date: May 04, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void CopyAttributeFromParent(
    Widget      widget,
    int         offset,
    XrmValue    *value)
{
	value->addr = (((char *) widget->core.parent) + offset);
}

/*-----------------------------------------------------------
|
|  Routine Name: GetGC
|
|       Purpose: This routine will set up a new gc
|		 for a given XvwManagerGadget.  The new gc
|		 will inherit the the foreground and background
|		 colors as well as the font from the  
|		 XvwManagerGadget.
|                
|         Input: widget - the XvwManagerGadget being assigned a gc 
|    Written By: Mark Young
|          Date: Aug 19, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void GetGC(
   Widget current,
   Widget new1)
{
	XvwManagerGadget nobj = (XvwManagerGadget) new1;
	XvwManagerGadget cobj = (XvwManagerGadget) current;

	XGCValues values;
	unsigned long mask;

#define changed(field) (cobj == NULL || cobj->field != nobj->field)

	/*
	 * Get the general foreground/background/font GC
	 */
	if (changed(manager.foreground) == TRUE ||
	    changed(manager.background) == TRUE ||
	    changed(manager.font)	== TRUE ||
            XtIsSensitive(current)      != XtIsSensitive(new1))
	{
	   if (nobj->manager.foreground_gc)
	      XtReleaseGC(new1, nobj->manager.foreground_gc);

	   mask = GCForeground | GCBackground | GCLineWidth;
	   values.line_width = 0;
           values.foreground = nobj->manager.foreground;
           values.background = nobj->manager.background;
           if (nobj->manager.font != NULL)
	   {
	      mask |= GCFont;
	      values.font = nobj->manager.font->fid;
	   }

           if (XtIsSensitive(new1) == FALSE)
	   {
              if (nobj->manager.tile != None)
		 XmuReleaseStippledPixmap(XtScreen(new1), nobj->manager.tile);

	      nobj->manager.tile = XmuCreateStippledPixmap(XtScreen(new1),
		   nobj->manager.foreground, nobj->manager.background,
                   new1->core.depth);

	      mask |= GCTile | GCFillStyle;
	      values.fill_style = FillTiled;
	      values.tile = nobj->manager.tile;
	   }
           nobj->manager.foreground_gc = XtGetGC(new1, mask, &values);
	}

        /*
         *  Get the top shadow GC
         */
	if (changed(manager.top_shadow) == TRUE)
	{
           if (nobj->manager.top_shadow_gc)
              XtReleaseGC(new1, nobj->manager.top_shadow_gc);

	   mask = GCForeground | GCLineWidth;
	   values.line_width = 0;
           values.foreground = nobj->manager.top_shadow;
           nobj->manager.top_shadow_gc = XtGetGC(new1, mask, &values);
	}

	/*
	 * Get the bottom shadow GC
	 */
	if (changed(manager.bottom_shadow) == TRUE)
	{
           if (nobj->manager.bottom_shadow_gc)
              XtReleaseGC(new1, nobj->manager.bottom_shadow_gc);

	   mask = GCForeground | GCLineWidth;
	   values.line_width = 0;
           values.foreground = nobj->manager.bottom_shadow;
           nobj->manager.bottom_shadow_gc = XtGetGC(new1, mask, &values);
	}

	/*
	 * Get the background GC
	 */
	if (changed(manager.background) == TRUE ||
	    changed(manager.foreground)	== TRUE ||
	    changed(manager.font)	== TRUE)
	{
           if (nobj->manager.background_gc)
              XtReleaseGC(new1, nobj->manager.background_gc);

	   mask = GCForeground | GCBackground | GCLineWidth;
	   values.line_width = 0;
           values.foreground = nobj->manager.background;
           values.background = nobj->manager.foreground;
           if (nobj->manager.font != NULL)
           {
              mask |= GCFont;
              values.font = nobj->manager.font->fid;
           }
           nobj->manager.background_gc = XtGetGC(new1, mask, &values);
	}

	/*
	 *  Get the border GC
	 */
	if (changed(manager.border_pixel) == TRUE ||
	    changed(rectobj.border_width) == TRUE)
	{
           if (nobj->manager.border_gc)
              XtReleaseGC(new1, nobj->manager.border_gc);

	   mask = GCForeground | GCLineWidth;
	   values.line_width = nobj->rectobj.border_width;
           values.foreground = nobj->manager.border_pixel;
           nobj->manager.border_gc = XtGetGC(new1, mask, &values);
	}
#undef changed
}

/*-----------------------------------------------------------
|
|  Routine Name: ManagerObjForceRedisplay - 
|
|       Purpose: For the manager object, if the force redisplay is set
|                then we want to call the refresh directly rather than
|		 letting Xt do this via an expose proc.
|
|         Input: current - the current manager object
|		 new1     - the new manager object that is forcing a redisplay
|       Returns: 0 if we dispatched the redisplay, 1 if we need Xt to do it.
|    Written By: Mark Young
|          Date: Jun 05, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
void ManagerObjForceRedisplay(
   Widget  current,
   Widget  new1,
   int     clear_area,
   Boolean *redisplay)
{
        XvwManagerGadget cobj = (XvwManagerGadget) current;
        XvwManagerGadget nobj = (XvwManagerGadget) new1;

        XEvent  event;
        Region  region;
	WidgetClass wclass;
	int     x, y, w, h;


	if (redisplay)
	   *redisplay = FALSE;

	if (XtWindow(new1) == 0 || nobj->manager.mapped == FALSE)
	   return;

	if (cobj->rectobj.x      != nobj->rectobj.x	 ||
            cobj->rectobj.y      != nobj->rectobj.y	 || 
            cobj->rectobj.width  != nobj->rectobj.width  ||
            cobj->rectobj.height != nobj->rectobj.height)
	{
#if 0
	   XtWidgetGeometry temp;

	   temp.request_mode = CWWidth | CWHeight | CWX | CWY;
	   temp.x = nobj->rectobj.x;
	   temp.y = nobj->rectobj.y;
	   temp.width  = nobj->rectobj.width;
	   temp.height = nobj->rectobj.height;
	   (void) XtMakeGeometryRequest(current, &temp, &temp);
#else
	   ManagerObjChangeGeometry(new1->core.widget_class, new1);
#endif
	}
	else if (clear_area)
	{
	   x = cobj->rectobj.x; y = cobj->rectobj.y;
	   w = cobj->rectobj.width; h = cobj->rectobj.height;
	   XClearArea(XtDisplay(new1), XtWindow(new1), x, y, 
		      (unsigned) w, (unsigned) h, FALSE);
	}

	if (nobj->manager.realized == FALSE)
	   ManagerObjRealize(XtClass(new1), new1);

	/*
	 *  Now let's see whether we have to call the object's redisplay
	 *  method.  This is done by actually calling it directly, rather
	 *  than having Xt do an expose...
	 */
	wclass = XtClass(new1);
	if (wclass->core_class.expose != NULL)
	{
	   /*
	    *  Create an expose region for the object that is the same that Xt
	    *  would create.
	    */
	   event.type = Expose;
           event.xexpose.x = event.xexpose.y = 0;
           event.xexpose.width  = new1->core.width;
           event.xexpose.height = new1->core.height;

	   region = XCreateRegion();
           XtAddExposureToRegion(&event, region);

	   /*
	    *  Call the class's redisplay method, since it ain't NULL
	    */
	   (*wclass->core_class.expose) (new1, &event, region);
	   XDestroyRegion(region);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: ManagerObjChangeGeometry 
|
|       Purpose:
|
|         Input: wclass -
|                widget -
|    Written By: Mark Young
|          Date: Aug 19, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
void ManagerObjChangeGeometry(
   WidgetClass wclass,
   Widget      widget)
{
        XvwManagerGadgetClass mclass = (XvwManagerGadgetClass) wclass;

	if (wclass != xvwManagerGadgetClass && wclass->core_class.superclass)
	   ManagerObjChangeGeometry(wclass->core_class.superclass, widget);

	if (mclass->manager_class.change_geometry != NULL)
	   (*mclass->manager_class.change_geometry)(widget);
}

/*-----------------------------------------------------------
|
|  Routine Name: ManagerObjRealize 
|
|       Purpose:
|
|         Input: wclass -
|                widget -
|    Written By: Mark Young
|          Date: Aug 19, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
void ManagerObjRealize(
   WidgetClass wclass,
   Widget      widget)
{
	XvwManagerGadget xobj;

	if (wclass != objectClass && wclass->core_class.superclass)
	   ManagerObjRealize(wclass->core_class.superclass, widget);
	else
	{
	   xobj = (XvwManagerGadget) widget;
	   xobj->manager.realized = TRUE;
	}

	if (wclass->core_class.realize != NULL)
	   (*(wclass->core_class.realize))(widget, NULL, NULL);
}
