 /*
  * Khoros: $Id: Menu.c,v 1.17.2.1 1997/04/28 22:18:33 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Menu.c,v 1.17.2.1 1997/04/28 22:18:33 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

#include "copyright.h"          /* Khoros copyright */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros Menu Widget
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 ConstraintInitialize()
   >>>>                 Initialize()
   >>>>			SetValues()
   >>>>   Public:
   >>>>                 xvw_create_menu()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <kwidgets/MenuP.h>
#include <kwidgets/LabelP.h>
#include <kwidgets/ButtonP.h>

static void ClassInitialize	 PROTO((void));
static void Initialize           PROTO((Widget, Widget, ArgList, Cardinal *));
static void ConstraintInitialize PROTO((Widget, Widget, ArgList, Cardinal *));
static Boolean SetValues  PROTO((Widget, Widget, Widget, ArgList, Cardinal *));

static void Layout	         PROTO((Widget));

/*-------------------------------------------------------------------*
|
|   Full class attributes
|
--------------------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_MENU_TEAR_OFF, NULL,  XtRInt, NULL},
};


/*-------------------------------------------------------------------*
|
|   Full class record constant
|
--------------------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwMenuWidgetRec, menu.field)

static XtResource resources[] = { 
{XVW_MENU_TEAR_OFF, NULL, XtRBoolean, sizeof(Boolean),
      offset(tear_off), XtRImmediate, (XtPointer) FALSE},
{XVW_BUFFER_DIST, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwMenuWidgetRec, manager.buffer_dist),
	XtRImmediate, (XtPointer) 0},
{XVW_HORIZ_DIST, NULL, XtRPosition, sizeof(Position),
        XtOffsetOf(XvwMenuWidgetRec, manager.horiz_offset),
	XtRImmediate, (XtPointer) 0},
{XVW_VERT_DIST, NULL, XtRPosition, sizeof(Position),
        XtOffsetOf(XvwMenuWidgetRec, manager.vert_offset),
	XtRImmediate, (XtPointer)  0},
};
#undef offset


/*-------------------------------------------------------------------*
|
|   Class Declaration for Menu Widget
|
--------------------------------------------------------------------*/

#define superclass (&xvwManagerWidgetClassRec)

XvwMenuWidgetClassRec xvwMenuWidgetClassRec =
{
  {
    (WidgetClass) superclass,		/* superclass		  */	
    "Menu",				/* class_name		  */
    sizeof(XvwMenuWidgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    XtInheritQueryGeometry,		/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwMenuWidgetConstraintsRec),    /* constraint_size	  */
    ConstraintInitialize,		    /* initialize	  */
    NULL,				    /* destroy		  */
    NULL,				    /* set_values	  */
    NULL,				    /* extension	  */
  },  /* ConstraintClass fields initialization */
  {
    Layout,				    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    XtInheritChangeSel,                     /* change selected proc   */
    XtInheritEraseSel,                      /* erase selected proc    */
    XtInheritRefreshSel,                    /* refresh selection proc */
    XtInheritResize,                        /* resize                 */
    XtInheritGeometryManager,               /* geometry_manager       */
  }, /* XvwManagerWidgetClass fields initialization */
  {
    NULL,  		                    /* extension          */
  },  /* XvwMenuWidgetClass fields initialization */
};
#undef superclass

  /* for public consumption */
WidgetClass xvwMenuWidgetClass = (WidgetClass) &xvwMenuWidgetClassRec;


/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for widget definition
|
--------------------------------------------------------------------*/

#define kwidget(widget)	     (XvwMenuWidget) (widget)
#define kconstraint(widget)  (XvwMenuWidgetConstraints) (widget)->core.constraints

/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an
|                instance of a XvwMenuWidget class has been created.
|                This is where the attributes for the class are
|		 initialized. 
|
|    Written By: Mark Young
|          Date: Apr 23, 1996
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
	kwidgets_cache_pixmaps();
	xvw_init_attributes(xvwMenuWidgetClass, attributes,
		XtNumber(attributes), NULL, 0, NULL);
	xvw_load_resources("$DESIGN/objects/library/kwidgets/app-defaults/Menu");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|	Purpose: This method will set up the initial menu widget
|		 instance.
|
|	  Input: request - not used
|                new1     - widget instance after initialization
|    Written By: Mark Young
|          Date: Apr 23, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwMenuWidget xwid = kwidget(new1);

	xwid->menu.active = NULL;
}

/*-----------------------------------------------------------
|
|  Routine Name: ConstraintInitialize
|
|	Purpose: Constraint initialize.
|
|	  Input: request - not used
|                new1     - widget instance after initialization
|    Written By: Mark Young
|          Date: Apr 23, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ConstraintInitialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	xvobject object = xvw_object(new1);

	if (xvw_check_subclass(object, InputOnlyWidgetClass))
	   return;

	/*
	 *  specify border width of 0, background pixel, and 
	 *  relative layout of this child.
	 */
	if (xvw_check_subclass(object, ButtonWidgetClass))
	{
	   xvw_set_attributes(object,
		XVW_BORDER_WIDTH,	  0,
		XVW_BUTTON_FORCE_ARMED,   TRUE,
		XVW_BUTTON_SELECT_TYPE,   KSHADOW_OUT,
		XVW_BUTTON_UNSELECT_TYPE, KSHADOW_OFF,
		NULL);
	}
	else
	   xvw_set_attribute(object, XVW_BORDER_WIDTH, 0);
}

/*-----------------------------------------------------------
|
|  Routine Name: Layout
|
|       Purpose: This routine will layout the children of the Menu.  
|
|         Input: widget - the menu widget 
|    Written By: Mark Young
|          Date: Jun 3, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Layout(
   Widget widget)
{
	XvwMenuWidget xwid = (XvwMenuWidget) widget;

	Widget   child;
	XvwMenuWidgetConstraints kman;
	unsigned int i, ypos, max_width = 0;

	/* Don't Layout if there are no children */
	if (xwid->composite.num_children == 0)
	   return;

	for (i = 0; i < xwid->composite.num_children; i++)
	{
	   kman = kconstraint(xwid->composite.children[i]);
	   if (kman->manager.tacking & KMANAGER_TACK_RIGHT)
	      continue;

	   child = xwid->composite.children[i];
	   max_width = kmax(child->core.width, max_width);
	}

	ypos = xwid->manager.buffer_dist;
	if (xwid->manager.shadow_type != KSHADOW_NONE)
	   ypos += xwid->manager.shadow_thickness;

	for (i = 0; i < xwid->composite.num_children; i++)
	{
	   child = xwid->composite.children[i];
	   
	   kman = kconstraint(xwid->composite.children[i]);
	   if (kman->manager.tacking & KMANAGER_TACK_RIGHT)
	   {
	      if (ypos != child->core.y)
		 xvw_set_attribute(xvw_object(child), XVW_YPOSITION, ypos);
	   }
	   else if (max_width != child->core.width || ypos != child->core.y)
	   {
	      xvw_set_attributes(xvw_object(child),
			XVW_WIDTH, max_width,
			XVW_YPOSITION, ypos,
			NULL);
	   }
	   ypos += child->core.height;
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: Determine what the widget should do by
|		 comparing the current public settable values 
|		 versus the new settings.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new1     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Apr 23, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
       XvwMenuWidget cxwid = (XvwMenuWidget) current;
       XvwMenuWidget nxwid = (XvwMenuWidget) new1;

       if (cxwid->menu.tear_off != nxwid->menu.tear_off)
       {
#if 0
	  if (nxwid->menu.tear_off)
	     xvw_manage(nxwid->menu.separator);
	  else
	     xvw_unmanage(nxwid->menu.separator);
#endif
       }
       return(FALSE);
}


/************************************************************
*
*  Routine Name: xvw_create_menu - create a toolbox menu object
*
*  Routine Name: xvw_create_menu - create menu GUI object
*
*       Purpose: The \fImenu\fP GUI object is the menu which is associated
*		 with a button 	n which the user can click to pull down a menu
*		 of items;  holding the mouse button down, the user may select
*		 an item from the menu.  Items in the menu may be \fIbutton\fP
*		 objects or \fIlabel\fP objects.  Button object items in the
*		 menu will be highlighted when the user moves the pointer over
*		 them. If the mouse button is released with the pointer over
*		 a button in the menu when that button has a callback installed,*		 the button callback will be fired normally.
*
*		 It is important to understand that the \fImenubutton\fP GUI 
*		 object is a \fIcompound\fP object.  That is, there are actually
*		 two objects created: the menubutton and the menu itself, where 
*		 the menubutton is the button that appears on the GUI,
*		 and the menu is what pops up when the user clicks on the 
*		 menubutton.
*
*		 When you create a menubutton object, the GUI object returned 
*		 is the \fIbutton\fP; this object should be referenced when 
*		 setting geometry, relative offset, and so on.  Attributes 
*		 such as label, width, and height may also be set on the 
*		 menubutton.
*
*		 However, in order to add buttons and labels to the pulldown 
*		 menu, you will to call \fIxvw_create_button()\fP and
*		 \fIxvw_create_label()\fP with the \fImenu\fP (not the 
*		 \fImenubutton\fP) as the parent.  The actual menu component
*		 of the compound menubutton object can be obtained with: 
*	      \f(CWxvw_get_attribute(menubutton, XVW_MENUBUTTON_MENU, &menu);\fP
*
*         Input: parent - parent of the menu object
*                name   - a name for this particular instance of the menu
*                         object (for use in app-defaults files, etc)
*
*        Output:
*       Returns: The menu GUI object on success, NULL on failure.
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 23, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*******************************************************************/

xvobject xvw_create_menu(
   xvobject parent,
   char     *name)
{
	xvobject object, popup;

	popup = xvw_create_popup_shell(parent, name);
	object = xvw_create(popup, TRUE, TRUE, name, MenuWidgetClass);
	return(object);
}
