 /*
  * Khoros: $Id: Scrollbar.c,v 1.11.2.7 1997/05/24 19:02:05 mirage Exp $
  */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Scrollbar.c,v 1.11.2.7 1997/05/24 19:02:05 mirage Exp $";
#endif
 
 
/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>                Scrollbar Widget Routines
   >>>>
   >>>>	Private:
   >>>>			ClassInitialize()
   >>>>			Initialize()
   >>>>			Resize()
   >>>>                 Destroy()
   >>>>			SetValues()
   >>>>
   >>>>			ActionHandler()
   >>>>			ContMotionHandler()
   >>>>			IncrMotionHandler()
   >>>>
   >>>>			IncrementScrollbar()
   >>>>			SetNewSliderLocation()
   >>>>			SetNewSliderSize()
   >>>>	Public:
   >>>>			xvw_create_scrollbar()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <kwidgets/ScrollbarP.h>

static void    ClassInitialize PROTO((void));
static void    Initialize      PROTO((Widget, Widget, ArgList,Cardinal *));
static void    Resize          PROTO((Widget));
static void    Destroy	       PROTO((Widget));
static Boolean SetValues       PROTO((Widget, Widget, Widget, 
                                      ArgList, Cardinal *));

static void ActionHandler        PROTO((Widget, XEvent *, String *, Cardinal));
static void ContMotionHandler    PROTO((xvobject, kaddr, XEvent *, int *));
static void IncrMotionHandler    PROTO((xvobject, kaddr, XEvent *, int *));

static void IncrementScrollbar   PROTO((xvobject, int));
static void SetNewSliderLocation PROTO((xvobject));
static void SetNewSliderSize     PROTO((xvobject));
static void ScrollbarIncrTimeout PROTO((kaddr, XtIntervalId *));



#define SB_UP        1
#define SB_DOWN      2
#define SB_LEFT      1
#define SB_RIGHT     2
#define SB_MAXIMUM   3
#define SB_MINIMUM   4
/*-------------------------------------------------------
|
|   Action Handler
|
--------------------------------------------------------*/

static char defaultTranslations[] =
    "<Key>Up:           Action(IncrementUp)           \n\
     <Key>Down:         Action(IncrementDown)         \n\
     <Key>Left:         Action(IncrementLeft)         \n\
     <Key>Right:        Action(IncrementRight)        \n\
     Ctrl<Key>Up:       Action(PageUp)                \n\
     Ctrl<Key>Down:     Action(PageDown)              \n\
     Ctrl<Key>Left:     Action(PageLeft)              \n\
     Ctrl<Key>Right:    Action(PageRight)             \n\
     <Key>Home:         Action(Minimum)               \n\
     Ctrl<Key>Home:     Action(Minimum)               \n\
     <Key>End:          Action(Maximum)               \n\
     Ctrl<Key>End:      Action(Maximum)";

static XtActionsRec actionsList[] =
{
  {"Action", (XtActionProc) ActionHandler}
};


/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_SCROLL_MIN,   NULL,  XtRDouble, NULL},
{XVW_SCROLL_MAX,   NULL,  XtRDouble, NULL},
{XVW_SCROLL_VALUE, NULL,  XtRDouble, NULL},
{XVW_SCROLL_INCR,  NULL,  XtRDouble, NULL},
{XVW_ORIENTATION,  NULL,  XtRInt, NULL},

{XVW_SCROLL_CONT_MOTION,  NULL,  XtRCallback, NULL},
{XVW_SCROLL_INCR_MOTION,  NULL,  XtRCallback, NULL},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwScrollbarWidgetRec, scrollbar.field)

static XtResource resources[] = { 

{XVW_SCROLL_MIN, NULL, XtRDouble, sizeof(double),
      offset(minval), XtRString, (XtPointer) "0.0"},
{XVW_SCROLL_MAX, NULL, XtRDouble, sizeof(double),
      offset(maxval), XtRString, (XtPointer) "1.0"},
{XVW_SCROLL_VALUE, NULL, XtRDouble, sizeof(double),
      offset(value), XtRString, (XtPointer) "0.0"},
{XVW_SCROLL_INCR, NULL, XtRDouble, sizeof(double),
      offset(increment), XtRString, (XtPointer) "0.1"},
{XVW_ORIENTATION, NULL, XtRInt, sizeof(int),
      offset(orientation), XtRImmediate, (XtPointer) KSCROLLBAR_ORIENT_HORIZ},
{XVW_SCROLL_CONT_MOTION, NULL, XtRCallback, sizeof(XtCallbackList),
      offset(cont_callback), XtRCallback, (XtPointer) NULL},
{XVW_SCROLL_INCR_MOTION, NULL, XtRCallback, sizeof(XtCallbackList),
      offset(incr_callback), XtRCallback, (XtPointer) NULL},

{XVW_BACKGROUND, NULL, XtRPixel, sizeof(Pixel),
        XtOffsetOf(XvwScrollbarWidgetRec, core.background_pixel),
        XtRString, (XtPointer) "#979797"},
{XVW_BORDER_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwScrollbarWidgetRec, core.border_width),
        XtRImmediate, (XtPointer) 1},
{XVW_BUFFER_DIST, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwScrollbarWidgetRec, manager.buffer_dist),
	XtRImmediate, (XtPointer) 0},
{XVW_SHADOW_TYPE, NULL, XtRInt, sizeof(int),
	XtOffsetOf(XvwScrollbarWidgetRec, manager.shadow_type),
	XtRImmediate, (XtPointer) KSHADOW_IN},
};
#undef offset


#define SUPERCLASS (&xvwManagerWidgetClassRec)

XvwScrollbarWidgetClassRec xvwScrollbarWidgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Scrollbar",			/* class_name		  */
    sizeof(XvwScrollbarWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    actionsList,		 	/* actions		  */
    XtNumber(actionsList),		/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    defaultTranslations,		/* tm_table		  */
    NULL,				/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwScrollbarWidgetConstraintsRec),  /* constraint_size    */
    NULL,				    /* initialize         */
    NULL,                                   /* destroy            */
    NULL,				    /* set_values	  */
    NULL,                                   /* extension          */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    XtInheritChangeSel,			    /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    Resize,			    	    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  }, /* XvwManagerWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwScrollbarWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwScrollbarWidgetClass for public consumption
 */
WidgetClass xvwScrollbarWidgetClass = (WidgetClass) &xvwScrollbarWidgetClassRec;

/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for Class and Constraint Declarations
|
--------------------------------------------------------------------*/
 
#define kwidget(widget)      (XvwScrollbarWidget) (widget)
#define kwidgetclass(widget) (XvwScrollbarWidgetClass) (widget)->core.widget_class
#define kconstraint(widget)  (XvwScrollbarWidgetConstraints) (widget)->core.constraints

/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of a xvwScrollbarWidgetClass has been created. 
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young
|          Date: June 10, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
	kwidgets_cache_pixmaps();
        xvw_init_attributes(xvwScrollbarWidgetClass, attributes,
		XtNumber(attributes), NULL, 0,
		"$DESIGN/objects/library/kwidgets/uis/Scrollbar.pane");
	xvw_load_resources("$DESIGN/objects/library/kwidgets/app-defaults/Scrollbar");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial values
|                for the fields of a label object instance.       
|                The width, height, ascent, and descent are       
|                all initialized to sane values.
|
|         Input: request - not used     
|                new1     - object instance after initialization, with
|    Written By: Mark Young & Danielle Argiro
|          Date: Dec 27, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
        XvwScrollbarWidget xwid = (XvwScrollbarWidget) new1;
	xvobject object = xvw_object(new1);


	xwid->scrollbar.incr_direction = 0;
	xwid->scrollbar.incr_id = (XtIntervalId) 0;

#if 0
	/*
	 *  Create buttons to increment & decrement scrollbar
	 */
	xwid->scrollbar.incr = xvw_create_button(object, "Incr");
	xvw_set_attributes(xwid->scrollbar.incr,
		XVW_LEFT_OF,        NULL,
		XVW_TACK_EDGE,      KMANAGER_TACK_VERT,
		XVW_BORDER_WIDTH,   0,
		XVW_MINIMUM_WIDTH,  12,
		XVW_MINIMUM_HEIGHT, 12,
		NULL);
	xwid->scrollbar.decr = xvw_create_button(object, "Decr");
	xvw_set_attributes(xwid->scrollbar.decr,
		XVW_RIGHT_OF,       NULL,
		XVW_TACK_EDGE,      KMANAGER_TACK_VERT,
		XVW_BORDER_WIDTH,   0,
		XVW_MINIMUM_WIDTH,  12,
		XVW_MINIMUM_HEIGHT, 12,
		NULL);
#endif

        /*
         *  Create the slider as a button subpart on the slider backplane 
         */
	xwid->scrollbar.slider = xvw_create_button(object, "Slider");
	xvw_set_attributes(xwid->scrollbar.slider,
   		XVW_TACK_EDGE,            KMANAGER_TACK_VERT, 
 		XVW_BUTTON_SELECT_TYPE,   KSHADOW_OUT,
		XVW_BUTTON_UNSELECT_TYPE, KSHADOW_OUT, 
		XVW_MINIMUM_WIDTH,        12,
		XVW_MINIMUM_HEIGHT,       12,
		XVW_BORDER_WIDTH,         0,
		NULL);
	xvw_insert_event(xwid->scrollbar.slider,
		ButtonPressMask|ButtonReleaseMask|ButtonMotionMask,
		ContMotionHandler, new1, KLIST_TAIL);
	xvw_insert_event(object, ButtonPressMask|ButtonReleaseMask,
		IncrMotionHandler, xwid->scrollbar.slider, KLIST_TAIL);

}

/*-----------------------------------------------------------
|
|  Routine Name: Resize
|
|       Purpose: Sets the location of the scrollbar slider 
|		 when scrollbar is resized.
|
|         Input: widget - the scrollbar object being resized
|    Written By: Danielle Argiro
|          Date: Jan 6, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Resize(
   Widget   widget)
{
	SetNewSliderSize(xvw_object(widget));
	SetNewSliderLocation(xvw_object(widget));
	(*xvwScrollbarWidgetClass->core_class.superclass->core_class.resize)
                (widget);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: This method will remove the incrment timeout (if set)
|
|         Input: widget - the widget being destroyed
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 10, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	XvwScrollbarWidget xwid = (XvwScrollbarWidget) widget;

	/* remove the increment timeout if one is currently set */
        if (xwid->scrollbar.incr_id != 0)
            XtRemoveTimeOut(xwid->scrollbar.incr_id);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: Determine what the widget should do by
|		 comparing the current public settable values 
|		 versus the new settings.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new1     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: Dec 27, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwScrollbarWidget cxwid = (XvwScrollbarWidget) current;
	XvwScrollbarWidget nxwid = (XvwScrollbarWidget) new1;

	/*
	 *  change orientation from horizontal to vertical or vice versa
	 */
	if (cxwid->scrollbar.orientation != nxwid->scrollbar.orientation)
	{
	   
	   if (nxwid->scrollbar.orientation == KSCROLLBAR_ORIENT_HORIZ)
	      xvw_set_attribute(nxwid->scrollbar.slider,
			XVW_TACK_EDGE, KMANAGER_TACK_VERT);
	   else
	      xvw_set_attribute(nxwid->scrollbar.slider,
			XVW_TACK_EDGE, KMANAGER_TACK_HORIZ);
	}

        /*
         *  change in range or increment value => change in slider size 
         */
        if (cxwid->scrollbar.maxval    != nxwid->scrollbar.maxval    ||
	    cxwid->scrollbar.minval    != nxwid->scrollbar.minval    ||
	    cxwid->scrollbar.increment != nxwid->scrollbar.increment ||
 	    cxwid->scrollbar.value     != nxwid->scrollbar.value )
	{
	   if (nxwid->scrollbar.value > nxwid->scrollbar.maxval)
	       nxwid->scrollbar.value = nxwid->scrollbar.maxval;
	   else if (nxwid->scrollbar.value < nxwid->scrollbar.minval)
	       nxwid->scrollbar.value = nxwid->scrollbar.minval;

	   SetNewSliderSize(xvw_object(new1));
	   SetNewSliderLocation(xvw_object(new1));
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ActionHandler
|
|       Purpose: Notify the user of the joyous selection that have
|                so dedicately selected, or are about to select.
|
|         Input: widget - the wonderous selection in question
|                event  - the event that triggered this mess.
|    Written By: Mark Young & Danielle Argiro
|          Date: Mar 10, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ActionHandler(
   Widget   widget,
   XEvent   *event,
   String   *params,
   Cardinal num_params)
{
        if (kstrcmp(params[0], "IncrementLeft") == 0)
            IncrementScrollbar(xvw_object(widget), SB_LEFT);
        else if (kstrcmp(params[0], "IncrementRight") == 0)
            IncrementScrollbar(xvw_object(widget), SB_RIGHT);
        else if (kstrcmp(params[0], "IncrementUp") == 0)
            IncrementScrollbar(xvw_object(widget), SB_UP);
        else if (kstrcmp(params[0], "IncrementDown") == 0)
            IncrementScrollbar(xvw_object(widget), SB_DOWN);
        else if (kstrcmp(params[0], "Minimum") == 0)
            IncrementScrollbar(xvw_object(widget), SB_MINIMUM);
        else if (kstrcmp(params[0], "Maximum") == 0)
            IncrementScrollbar(xvw_object(widget), SB_MAXIMUM);
}

/*-----------------------------------------------------------
|
|  Routine Name: ContMotionHandler
|
|       Purpose: Handles continuous movement of the scrollbar slider (thumb).
|
|         Input: slider      - the slider sub-part (button object) 
|                              of the scrollbar
|                client_data - passes in the scrollbar widget
|                event       - ButtonMotion, ButtonPress, or ButtonRelease
|                dispatch    - whether to continue dispatching the event
|                 
|    Written By: Danielle Argiro & Mark Young
|          Date: Dec 27, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ContMotionHandler(
   xvobject slider,
   kaddr    client_data,
   XEvent   *event,
   int      *dispatch)
{
	double     value;
	int        xpos, ypos, slider_size, core_size, thickness;
        int        new_location, current_location, event_location;
	Widget     widget = (Widget) client_data;
        XvwScrollbarWidget xwid = (XvwScrollbarWidget) kwidget(widget);

	/*
	 *  Translate the event coordinates from the slider WC coordinate
	 *  system to the scrollbar's WC coordinate system.
	 */
	xvw_translate_coords(xvw_object(widget), 0, 0, &xpos, &ypos);
        if (xwid->scrollbar.orientation == KSCROLLBAR_ORIENT_HORIZ)
        {
	    xvw_get_attributes(slider, 
	    		       XVW_XPOSITION, &current_location,
			       XVW_WIDTH,     &slider_size,
			       NULL);
	    event_location = event->xbutton.x_root - xpos;
	    core_size = (int) xwid->core.width;
	}
	else
	{
            xvw_get_attributes(slider,
                               XVW_YPOSITION, &current_location,
                               XVW_HEIGHT,    &slider_size,
                               NULL);
	    event_location = event->xbutton.y_root - ypos;
            core_size = (int) xwid->core.height;
	}
	thickness = xwid->manager.shadow_thickness;
	core_size -= 2*thickness;
	new_location = event_location - (int) slider_size/2 - thickness;

	/* pin at min and max at either end of scrollbar */
        if (new_location < 0)
            new_location = 0;
        else if (new_location > (core_size - slider_size))
            new_location = core_size - slider_size;

	/* move slider to new position */
	if (xwid->scrollbar.orientation == KSCROLLBAR_ORIENT_HORIZ)
            xvw_set_attribute(slider, XVW_XPOSITION, new_location+thickness);
	else xvw_set_attribute(slider, XVW_YPOSITION, new_location+thickness);

	/* 
	 * value = location*((maxval-minval)/(sb_size-slider_size) + minval
	 */
	value = new_location*(xwid->scrollbar.maxval-xwid->scrollbar.minval)/
	        (core_size-slider_size) + xwid->scrollbar.minval;

	/* if nothing has changed then simply return */
	if (value == xwid->scrollbar.value)
	   return;

	xwid->scrollbar.value = value;
	XtCallCallbacks((Widget) xwid, XVW_SCROLL_CONT_MOTION, &value);
}

/*-----------------------------------------------------------
|
|  Routine Name: IncrMotionHandler
|
|       Purpose: Event handler that responds to incremental 
|                movement of the scrollbar slider.
|
|         Input: scrollbar   - the scrollbar object
|                client_data - passes in the slider sub-part (button object)
|			       of the scrollbar object
|		 event       - the ButtonPress or ButtonRelease event
|                dispatch    - whether to continue dispatching the event
|    Written By: Danielle Argiro & Mark Young
|          Date: Jan 3, 1996
|
------------------------------------------------------------*/

static void IncrMotionHandler(
   xvobject scrollbar,
   kaddr    client_data,
   XEvent   *event,
   int      *dispatch)
{
        Widget   widget = xvw_widget(scrollbar);
        XvwScrollbarWidget xwid   = (XvwScrollbarWidget) kwidget(widget);

	int      current_location, event_location;
	xvobject slider = xwid->scrollbar.slider;

	if (event->type == ButtonPress)
	{
            XtAppContext app_context = XtWidgetToApplicationContext(widget);

	    if (xwid->scrollbar.orientation == KSCROLLBAR_ORIENT_HORIZ)
            {
                xvw_get_attribute(slider, XVW_XPOSITION, &current_location);
	        event_location = event->xbutton.x;
	    }
	    else
	    {
                xvw_get_attribute(slider, XVW_YPOSITION, &current_location);
                event_location = event->xbutton.y;
	    }

	    if (event_location > current_location)
		xwid->scrollbar.incr_direction = SB_DOWN;
	    else
		xwid->scrollbar.incr_direction = SB_UP;

	    IncrementScrollbar(scrollbar, xwid->scrollbar.incr_direction);
	    xwid->scrollbar.incr_id = XtAppAddTimeOut(app_context,
			(unsigned long) 250, ScrollbarIncrTimeout, xwid);
	}
	else
	{
	    /* remove the increment timeout if one is currently set */
            if (xwid->scrollbar.incr_id != 0)
	    {
                XtRemoveTimeOut(xwid->scrollbar.incr_id);
		xwid->scrollbar.incr_id = (XtIntervalId) 0;
	    }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: IncrementScrollbar
|
|       Purpose: Increments the scrollbar slider.
|
|         Input: scrollbar   - the scrollbar object
|                direction   - SB_UP or SB_DOWN, SB_LEFT or SB_RIGHT
|    Written By: Danielle Argiro & Mark Young
|          Date: Jan 3, 1996
|
------------------------------------------------------------*/
static void IncrementScrollbar(
   xvobject scrollbar,
   int      direction)
{
        Widget widget = xvw_widget(scrollbar);
        XvwScrollbarWidget xwid = (XvwScrollbarWidget) kwidget(widget);

	double value = xwid->scrollbar.value;

	if (direction == SB_UP) /* or direction == SB_LEFT */
            value -= xwid->scrollbar.increment;
	else if (direction == SB_DOWN) /* or direction == SB_RIGHT */
            value += xwid->scrollbar.increment;
	else if (direction == SB_MAXIMUM)
	    value = xwid->scrollbar.maxval;
	else if (direction == SB_MINIMUM) 
	    value = xwid->scrollbar.minval;

	if (value < xwid->scrollbar.minval)
	    value = xwid->scrollbar.minval;
	else if (value > xwid->scrollbar.maxval)
	    value = xwid->scrollbar.maxval;

	if (value == xwid->scrollbar.value)
	    return;

	xwid->scrollbar.value = value;
	SetNewSliderLocation(scrollbar);
	XtCallCallbacks((Widget) xwid, XVW_SCROLL_INCR_MOTION, &value);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetNewSliderLocation
|
|       Purpose: Sets the location of the slider according 
|                to a (new) scrollbar value.
|
|         Input: scrollbar   - the scrollbar object
|
|    Written By: Danielle Argiro
|          Date: Jan 6, 1996
|
------------------------------------------------------------*/
static void SetNewSliderLocation(
   xvobject scrollbar)
{
        int      new_location, slider_size, core_size, thickness;

        Widget             widget = xvw_widget(scrollbar);
        XvwScrollbarWidget xwid   = (XvwScrollbarWidget) kwidget(widget);
        xvobject           slider = xwid->scrollbar.slider;

        if (xwid->scrollbar.orientation == KSCROLLBAR_ORIENT_HORIZ)
        {
            xvw_get_attribute(slider, XVW_WIDTH, &slider_size);
	    core_size = xwid->core.width;
	}
	else
	{
            xvw_get_attribute(slider, XVW_HEIGHT, &slider_size);
            core_size = xwid->core.height;
	}

	/* 
	 * location = (value-minval)*(sb_size-slider_size)/(maxval-minval)] 
	 */
	thickness    = xwid->manager.shadow_thickness;
	new_location = ((xwid->scrollbar.value - xwid->scrollbar.minval) *
		        (core_size-slider_size))/
		        (xwid->scrollbar.maxval - xwid->scrollbar.minval);

	/* pin at min and max at either end of scrollbar */
        if (new_location < thickness)
            new_location = thickness;
        else if (new_location > (core_size - slider_size - thickness))
            new_location = core_size - slider_size - thickness;

	/* move slider to new position */
	if (xwid->scrollbar.orientation == KSCROLLBAR_ORIENT_HORIZ)
             xvw_set_attribute(slider, XVW_XPOSITION, new_location);
	else xvw_set_attribute(slider, XVW_YPOSITION, new_location);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetNewSliderSize
|
|       Purpose: Sets the size of the slider according
|                to (new) scrollbar max/min values or
|                (new) scrollbar increment value.
|
|         Input: scrollbar   - the scrollbar object
|    Written By: Danielle Argiro & Mark Young
|          Date: Jan 6, 1996
|
------------------------------------------------------------*/
static void SetNewSliderSize(
   xvobject scrollbar)
{
	int slider_size, core_size, new_slider_size;
        Widget             widget = xvw_widget(scrollbar);
	XvwScrollbarWidget xwid   = (XvwScrollbarWidget) widget;

        /*
         *  for horizontal scrollbar, need to change width;
         *  for vertical scrollbar, need to change height
         */
        if (xwid->scrollbar.orientation == KSCROLLBAR_ORIENT_HORIZ)
        {
            xvw_get_attribute(xwid->scrollbar.slider, XVW_WIDTH,  &slider_size);
            core_size = xwid->core.width - 2*xwid->manager.shadow_thickness;
        }
        else
        {
            xvw_get_attribute(xwid->scrollbar.slider,XVW_HEIGHT, &slider_size);
            core_size = xwid->core.height - 2*xwid->manager.shadow_thickness;
        }

        new_slider_size = (int) (((xwid->scrollbar.increment)/
                                  (xwid->scrollbar.maxval -
                                   xwid->scrollbar.minval)) *
				   core_size);

        if (xwid->scrollbar.orientation == KSCROLLBAR_ORIENT_HORIZ)
            xvw_set_attribute(xwid->scrollbar.slider, 
			      XVW_WIDTH, new_slider_size);
        else xvw_set_attribute(xwid->scrollbar.slider, 
			       XVW_HEIGHT, new_slider_size);
}

/*-----------------------------------------------------------
|
|  Routine Name: ScrollbarIncrTimeout
|
|       Purpose: Timeout for auto-incrementing the scrollbar.  This
|		 timeout will increment the scrollbar according to the
|		 increment direction, then re-install the timer
|		 to repeat the auto-increment operation.  This is
|		 repeated until the user releases the button.  The
|		 actual XtRemoveTimeout() is done in the IncrMotionHandler()
|		 event handler.
|
|         Input: client_data - used to pass in the scrollbar widget
|                id          - timeout id
|    Written By: Mark Young & Danielle Argiro
|          Date: April 22, 1997
|
------------------------------------------------------------*/
static void ScrollbarIncrTimeout(
    kaddr client_data,
    XtIntervalId *id)
{
	Widget widget = (Widget) client_data;
        XvwScrollbarWidget xwid = (XvwScrollbarWidget) widget;

        XtAppContext app_context = XtWidgetToApplicationContext(widget);

	/*
	 *  Perform the actual incrementing of the scrollbar according to
	 *  the increment direction.
	 */
	IncrementScrollbar(xvw_object(widget), xwid->scrollbar.incr_direction);

	/*
	 *  Add the timeout to perform the operation again after the
	 *  50 ms.  This is repeated until the user releases the button.
	 */
	xwid->scrollbar.incr_id = XtAppAddTimeOut(app_context,
		(unsigned long) 50, ScrollbarIncrTimeout, xwid);
}

/************************************************************
*
*  Routine Name: xvw_create_scrollbar - create scrollbar GUI object
*
*       Purpose: The \fIscrollbar\fP GUI object allows users to reposition
*		 data that is too large to fit in a viewing window.  Typically,
*		 the scrollbar is used by other objects that serve as a 
*		 backplane for a large number of subordinate objects 
*		 that will not fit within the viewing area. For example,
*		 the TextDisplay object has a built-in scrollbar which allows 
*		 the user to scroll through the text being displayed, and the
*		 List object will display a scrollbar if the number of items
*		 in the list exceeds the space provided by the height of the
*		 List object.  The Viewport object also has built-in scrollbars
*		 to allow the user to shift the area of view, as does the
*		 Canvas object.
*
*		 The Scrollbar consists of a long, thin area called the
*		 scrollbar trough, within which is a smaller, movable button
*		 called the slider.  Scrollbars may be horizontal or vertical.
*		 Horizontal scrollbars have the minimum value at the top and
*		 the maximum value at the bottom; vertical scrollbars have the
*		 minimum value at the left and the maximum value at the right.
*
*		 With a horizontal scrollbar, clicking in the scrollbar trough 
*		 to the right of the slider will cause the scrollbar to 
*		 increment in value; clicking in the scrollbar trough to the
*		 left of the slider will cause the scrollbar to decrement in
*		 value.  Similarly with a vertical scrollbar, clicking above
*		 the slider will decrement the value while clicking below the
*		 slider will increment the value.  The slider itself can be
*		 dragged back and forth (with a horizontal scrollbar) or up and
*		 down (with a vertical scrollbar) to change the value.  
*
*                A callback can be installed on the scrollbar for 
*                incremental movement; this callbeack will be fired when the 
*		 user clicks in the scrollbar trough (after the scrollbar 
*		 value is incremented or decremented).  The same callback or
*		 another callback may be installed on the scrollbar for 
*		 continuous movement; this callback will be fired when the
*		 user moves the slider of the scrollbar (after the scrollbar
*		 value is updated). 
*
*         Input: parent - parent of the scrollbar object
*                name   - a name for this particular instance of the scrollbar
*                         object (for use in app-defaults files, etc)
*
*        Output:
*       Returns: The Scrollbar object on success, NULL on failure. 
*    Written By: Mark Young & Danielle Argiro
*  	   Date: June 15, 1996
*   Declaration: xvobject xvw_create_scrollbar(
*                !   xvobject parent,
*                !   char   *name)
*
*************************************************************/

xvobject xvw_create_scrollbar(
   xvobject parent,
   char     *name)
{
	xvobject  object;


	/*
	 *  Call the xvw_create() routine to do the actual creation.
	 */
	object = xvw_create(parent, FALSE, TRUE, name, xvwScrollbarWidgetClass);
	return(object);
}
