/*
  * Khoros: $Id: Separator.c,v 1.3.2.1 1997/04/28 22:18:45 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Separator Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Redisplay()
   >>>>                 SetValues()
   >>>>   Public:
   >>>>                 xvw_create_separator()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <kwidgets/SeparatorP.h>

static void ClassInitialize PROTO((void));
static void Initialize      PROTO((Widget, Widget, ArgList, Cardinal *));
static void Redisplay	    PROTO((Widget, XEvent *, Region));
static Boolean SetValues    PROTO((Widget, Widget, Widget, ArgList,Cardinal *));


/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_SEPARATOR_TYPE,     NULL,     XtRInt,     NULL},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwSeparatorGadgetRec, separator.field)

static XtResource resources[] = { 
{XVW_SEPARATOR_TYPE, NULL, XtRInt, sizeof(int), 
      offset(type), XtRImmediate, (XtPointer) KSEPARATOR_TYPE_SOLID},
{XVW_SEPARATOR_TYPE, NULL, XtRInt, sizeof(int), 
      offset(orientation), XtRImmediate,
      (XtPointer) KSEPARATOR_ORIENTATION_HORIZ},

{XVW_SHADOW_TYPE, NULL, XtRInt, sizeof(int),
	XtOffsetOf(XvwSeparatorGadgetRec, manager.shadow_type),
	XtRImmediate, (XtPointer) KSHADOW_OFF},
{XVW_BORDER_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwSeparatorGadgetRec, rectobj.border_width),
        XtRImmediate, (XtPointer) 0},
};
#undef offset


#define SUPERCLASS (&xvwManagerGadgetClassRec)

XvwSeparatorGadgetClassRec xvwSeparatorGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Separator",			/* class_name		  */
    sizeof(XvwSeparatorGadgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    NULL,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    XtInheritPicking,			/* pick object proc       */
    XtInheritEraseSel,                  /* erase selected proc    */
    XtInheritRefreshSel,	        /* refresh selection proc */
    NULL,			        /* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwSeparatorGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwSeparatorGadgetClass for public consumption
 */
WidgetClass xvwSeparatorGadgetClass = (WidgetClass) &xvwSeparatorGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of separator object class has been created.
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young
|          Date: Mar 03, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
	kwidgets_cache_pixmaps();
	xvw_init_attributes(xvwSeparatorGadgetClass, attributes,
		XtNumber(attributes), NULL, 0, 
		"$DESIGN/objects/library/kwidgets/uis/Separator.pane");
	xvw_load_resources("$DESIGN/objects/library/kwidgets/app-defaults/Separator");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This will initialize the separator object.
|
|         Input: request - not used
|                new1     - widget instance after initialization
|    Written By: Mark Young
|          Date: Mar 03, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget request,
   Widget new1,
   ArgList args,
   Cardinal *num_args)
{
	XvwSeparatorGadget xobj = (XvwSeparatorGadget) new1;

	xvobject object = xvw_object(new1);
	if (xobj->separator.orientation == KSEPARATOR_ORIENTATION_HORIZ)
	{
	   xvw_set_attributes(object,
		XVW_TACK_EDGE, KMANAGER_TACK_HORIZ,
		XVW_MINIMUM_WIDTH,  8,
		XVW_MINIMUM_HEIGHT, 8,
		NULL);
	}
	else
	{
	   xvw_set_attributes(object,
		XVW_TACK_EDGE, KMANAGER_TACK_VERT,
		XVW_MINIMUM_WIDTH,  8,
		XVW_MINIMUM_HEIGHT, 8,
		NULL);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the separator object in response
|                to an expose event. 
|
|         Input: widget - the separator object that was exposed 
|                event  - the event that caused the redraw
|                region - the region that was exposed 
|    Written By: Mark Young
|          Date: Mar 03, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	XvwSeparatorGadget xobj = (XvwSeparatorGadget) widget;

	Window  window;
	Display *display;
	GC      gc1, gc2;
	int     x, y, w, h;

	/*
	 *  Call the ManagerObj to do the widget thang (make sure we are
	 *  properly realized, etc)
	 */
	(*xvwSeparatorGadgetClass->core_class.superclass->core_class.expose)
                (widget, event, region);

	if (xobj->separator.type == KSEPARATOR_TYPE_NONE)
	   return;

	x = widget->core.x;
	y = widget->core.y;
	w = widget->core.width;
	h = widget->core.height;
	window  = XtWindow(widget);
	display = XtDisplay(widget);
	gc1 = xobj->manager.top_shadow_gc;
	gc2 = xobj->manager.bottom_shadow_gc;
	if (xobj->separator.type == KSEPARATOR_ORIENTATION_HORIZ)
	{
	   x += widget->core.border_width;
	   y += h/2 + widget->core.border_width - 1;
	   XDrawLine(display, window, gc1, x, y+1, x+w, y+1);
	   XDrawLine(display, window, gc2, x, y,   x+w, y);
	}
	else
	{
	   y += widget->core.border_width;
	   x += w/2 + widget->core.border_width - 1;
	   XDrawLine(display, window, gc1, x+1, y, x+1, y+h);
	   XDrawLine(display, window, gc2, x,   y, x,   y+h);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: If the the size or postion of the separator has changed, 
|		 recalculate the size and position of the separator.
|
|         Input: current - the object containing current settings 
|                request - the object containing requested settings
|                new1     - the object processed through all set values methods 
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Mar 03, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwSeparatorGadget cobj = (XvwSeparatorGadget) current;
	XvwSeparatorGadget nobj = (XvwSeparatorGadget) new1;

	if (cobj->separator.type != nobj->separator.type)
	{
	   xvobject object = xvw_object(new1);
	   if (nobj->separator.type == KSEPARATOR_ORIENTATION_HORIZ)
	      xvw_set_attribute(object, XVW_TACK_EDGE, KMANAGER_TACK_HORIZ);
	   else
	      xvw_set_attribute(object, XVW_TACK_EDGE, KMANAGER_TACK_VERT);
	}
	return(FALSE);
}

/************************************************************
*
*  Routine Name: xvw_create_separator - create a separator object
*
*       Purpose: A separator object simply supports display of a 
*		 bitmap or separator, as defined by an xbm or xpm file.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the object
*                         (for use in app-defaults files, etc)
*
*        Output:
*       Returns: The separator object on success, NULL on failure
*  Restrictions: 
*    Written By: Mark Young 
*          Date: Mar 03, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_separator(
   xvobject parent,
   char   *name)
{
	xvobject  object;


	/*
	 *  Create the line object
	 */
	object = xvw_create(parent, FALSE, TRUE, name, SeparatorGadgetClass);
	if (object != NULL)
	{
	   xvw_set_attributes(object,
                XVW_MENUABLE,     TRUE,     /* menuable       */
                XVW_RESIZABLE,    FALSE,    /* resizable      */
                XVW_SELECTABLE,   TRUE,     /* selectable     */
		NULL);
	}
	return(object);
}
