 /*
  * Khoros: $Id: Text.c,v 1.6.2.17 1997/06/26 18:11:50 mirage Exp $
  */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Text.c,v 1.6.2.17 1997/06/26 18:11:50 mirage Exp $";

#endif
 
 
/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>                Text Widget Routines
   >>>>
   >>>>	Private:
   >>>>			ClassInitialize()
   >>>>			Initialize()
   >>>>			Realize()
   >>>>			Destroy()
   >>>>                 Resize()
   >>>>			SetValues()
   >>>>			ActionHandler()
   >>>>			TextProcessKeyPress()
   >>>>			TextSetString()
   >>>>			TextInsertString()
   >>>>			TextProcessAction()
   >>>>			TextPasteCallback()
   >>>>	Public:
   >>>>			xvw_create_text()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <X11/keysym.h>
#include <kwidgets/TextP.h>

static void ClassInitialize PROTO((void));
static void Initialize      PROTO((Widget, Widget, ArgList,Cardinal *));
static void Realize         PROTO((Widget, XtValueMask *,
                                        XSetWindowAttributes *));
static void Destroy         PROTO((Widget));
static void Resize          PROTO((Widget));

static Boolean SetValues   PROTO((Widget, Widget, Widget, ArgList, Cardinal *));

static void ActionHandler     PROTO((Widget, XEvent *, String *, Cardinal *));
static void TextProcessKeyPress    PROTO((xvobject, XEvent *));
static int  TextSetString          PROTO((xvobject, char *, kaddr));
static void TextInsertString       PROTO((XvwTextWidget, char *, size_t));
static int  TextProcessAction      PROTO((xvobject, XEvent *, String));
static void TextPasteCallback      PROTO((xvobject, kaddr, kaddr));
static void TextBlinkCursorTimeout PROTO((kaddr, XtIntervalId *));


/*-------------------------------------------------------
|
|   Action Handler
|
--------------------------------------------------------*/

/*
 * NOTE: be sure to list default translations in order of
 * most to least restrictive - ie, if there will be Ctrl<Key>Up and
 * <Key>Up defined, be sure to list Ctrl<Key>Up first.
 *
 * IMPORTANT: If you change this table, you must update the table of actions in 
 * ~/manual/manual/ProgServices3/PS3.3.kwidgets/text_keybindings.sec
 *
 * Note: Also want to implement:
 *       Ctrl<Key>k: DeleteToEndOfLine
 *       Ctrl<Key>y: PasteText
 *       Ctrl<Key>w: DeleteText
 */
static char defaultTranslations[] =
"\
Ctrl<Key>Up:        Action(MoveToBeginParagraph)			     \n\
Ctrl<Key>Down:      Action(MoveToNextParagraph)				     \n\
Ctrl<Key>Left:      Action(MoveToBeginWord)				     \n\
Ctrl<Key>Right:     Action(MoveToNextWord)				     \n\
Ctrl<Key>Home:      Action(MoveToBeginFile)				     \n\
Ctrl<Key>End:       Action(MoveToEndFile)				     \n\
Ctrl<Key>Insert:    Action(PasteBuffer)					     \n\
\
Meta<Key>b:         Action(MoveToBeginWord)			             \n\
Meta<Key>f:         Action(MoveToNextWord)  				     \n\
Ctrl<Key>b:         Action(MoveLeftOneCharacter)			     \n\
Ctrl<Key>f:         Action(MoveRightOneCharacter)			     \n\
Ctrl<Key>a:         Action(MoveToBeginLine)				     \n\
Ctrl<Key>e:         Action(MoveToEndLine)				     \n\
Ctrl<Key>n:         Action(MoveDownOneLine)				     \n\
Ctrl<Key>p:         Action(MoveUpOneLine)				     \n\
Meta<Key>v:         Action(MoveUpOnePage)				     \n\
Ctrl<Key>v:         Action(MoveDownOnePage)				     \n\
Ctrl<Key>y:         Action(PasteBuffer)					     \n\
\
<Key>Prior:         Action(MoveUpOnePage)				     \n\
<Key>Next:          Action(MoveDownOnePage)				     \n\
<Key>Up:            Action(MoveUpOneLine)				     \n\
<Key>Down:          Action(MoveDownOneLine)				     \n\
<Key>Left:          Action(MoveLeftOneCharacter)			     \n\
<Key>Right:         Action(MoveRightOneCharacter)			     \n\
<Key>Home:          Action(MoveToBeginLine)				     \n\
<Key>End:           Action(MoveToEndLine)				     \n\
\
<KeyDown>:          Action(AddText)					     \n\
<EnterNotify>:      Action(FocusIn)					     \n\
<LeaveNotify>:      Action(FocusOut)					     \n\
";

static XtActionsRec actionsList[] =
{
  {"Action", (XtActionProc) ActionHandler},
};

/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{"TextLabel",           NULL,             XtRObject,   NULL},
{XVW_TEXT_STRING,       NULL,             XtRString,   NULL},
{XVW_TEXT_FILENAME,     NULL,             XtRString,   NULL},
{XVW_TEXT_EDIT_TYPE,    NULL,             XtRInt,      NULL},
{XVW_TEXT_WRAP,         NULL,             XtRInt,      NULL},
{XVW_TEXT_MULTILINE,    NULL,             XtRInt,      XtRBoolean},
{XVW_TEXT_LEFT_MARGIN,  XVW_BUFFER_DIST,  XtRInt,      XtRDimension},
{XVW_TEXT_ENTER_STRING, NULL,             XtRCallback, NULL},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwTextWidgetRec, text.field)

static XtResource resources[] = { 

{"TextLabel", NULL, XtRObject, sizeof(XtPointer),
      offset(text), XtRImmediate, (XtPointer) NULL},
{XVW_TEXT_STRING, NULL, XtRString, sizeof(String),
      offset(string), XtRImmediate, (XtPointer) NULL},
{XVW_TEXT_FILENAME, NULL, XtRString, sizeof(String),
      offset(filename), XtRImmediate, (XtPointer) NULL},
{XVW_TEXT_EDIT_TYPE, NULL, XtRInt, sizeof(int),
      offset(edit_type), XtRImmediate, (XtPointer) KTEXT_EDIT},
{XVW_TEXT_WRAP, NULL, XtRInt, sizeof(int),
      offset(wrap_type), XtRImmediate, (XtPointer) KTEXT_WRAP_NEVER},
{XVW_TEXT_MULTILINE, NULL, XtRBoolean, sizeof(Boolean),
      offset(multiline), XtRImmediate, (XtPointer) FALSE},
{XVW_TEXT_ENTER_STRING, NULL, XtRCallback, sizeof(XtCallbackList),
      offset(enter_callback), XtRCallback, (XtPointer) NULL},

{XVW_BORDER_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwTextWidgetRec, core.border_width),
        XtRImmediate, (XtPointer) 1},
{XVW_BUFFER_DIST, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwTextWidgetRec, manager.buffer_dist),
	XtRImmediate, (XtPointer) 0},
{XVW_SHADOW_TYPE, NULL, XtRInt, sizeof(int),
	XtOffsetOf(XvwTextWidgetRec, manager.shadow_type),
	XtRImmediate, (XtPointer) KSHADOW_IN},
{XVW_CURSOR, NULL, XtRCursor, sizeof(Cursor),
      XtOffsetOf(XvwTextWidgetRec, manager.cursor), XtRString,
      (XtPointer) "xterm"},
};
#undef offset


#define SUPERCLASS (&xvwManagerWidgetClassRec)

XvwTextWidgetClassRec xvwTextWidgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Text",				/* class_name		  */
    sizeof(XvwTextWidgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    Realize,				/* realize		  */
    actionsList,			/* actions		  */
    XtNumber(actionsList),		/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    defaultTranslations,		/* tm_table		  */
    NULL,				/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwTextWidgetConstraintsRec),  /* constraint_size    */
    NULL,				    /* initialize         */
    NULL,                                   /* destroy            */
    NULL,				    /* set_values	  */
    NULL,                                   /* extension          */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    XtInheritChangeSel,			    /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    Resize,			    	    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  }, /* XvwManagerWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwTextWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwTextWidgetClass for public consumption
 */
WidgetClass xvwTextWidgetClass = (WidgetClass) &xvwTextWidgetClassRec;

/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for Class and Constraint Declarations
|
--------------------------------------------------------------------*/
 
#define kwidget(widget)      (XvwTextWidget) (widget)
#define kwidgetclass(widget) (XvwTextWidgetClass) (widget)->core.widget_class
#define kconstraint(widget)  (XvwTextWidgetConstraints) (widget)->core.constraints

/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of a xvwTextWidgetClass has been created. 
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young & Danielle Argiro
|          Date: January 19, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
	kwidgets_cache_pixmaps();
        xvw_init_attributes(xvwTextWidgetClass, attributes,
		XtNumber(attributes), NULL, 0,
		"$DESIGN/objects/library/kwidgets/uis/Text.pane");
	xvw_load_resources("$DESIGN/objects/library/kwidgets/app-defaults/Text");

        xvw_define_attributes(xvwTextWidgetClass,
                XVW_TEXT_STRING,   XtRString,    TextSetString,  NULL,
                NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial values
|                for the fields of a label object instance.       
|                The width, height, ascent, and descent are       
|                all initialized to sane values.
|
|         Input: request - not used     
|                new1     - object instance after initialization, with
|    Written By: Mark Young & Danielle Argiro
|          Date: January 19, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
        XvwTextWidget xwid = (XvwTextWidget) new1;
	xvobject object = xvw_object(new1);
	unsigned long mask;

        /*
         *  Create and initialize label sub-part.  The label is used
         *  to display the actual text portion of the Text object.
         */
	xwid->text.text = xvw_create_label(object, "TextLabel");
	xvw_set_attributes(xwid->text.text,
     		XVW_TACK_EDGE,            KMANAGER_TACK_ALL,
		XVW_LABEL_MULTILINE,      FALSE,
		XVW_LABEL_DISPLAY_CURSOR, TRUE,
		XVW_LABEL_SELECTABLE,     TRUE,
     		XVW_SHADOW_TYPE,          KSHADOW_NONE,
     		XVW_BORDER_WIDTH,         0,
		NULL);

	xvw_set_attributes(object,
		 	   XVW_CHAR_MIN_WIDTH,       1.0,
	                   XVW_CHAR_MIN_HEIGHT,      1.0,
                           XVW_CHAR_MAX_HEIGHT,      1.0,
			   NULL);

	mask = ButtonPressMask | EnterWindowMask | LeaveWindowMask;
        xvw_add_callback(xwid->text.text, XVW_LABEL_PASTE_BUFFER,
                         TextPasteCallback, xwid);

        xvw_set_attribute(xwid->text.text, XVW_LABEL_SELECTABLE, TRUE);

	xwid->text.alloc_size = KLENGTH;
	xwid->text.string = (char*) kmalloc(xwid->text.alloc_size*sizeof(char));
	xwid->text.string[0] = '\0';
	xwid->text.blink_id = 0;
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: Frees memory associated with text widget when
|		 the text widget is destroyed.
|
|         Input: widget - the text widget being destroyed
|    Written By: Mark Young & Danielle Argiro
|          Date: January 19, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	XvwTextWidget xwid = kwidget(widget);

	kfree(xwid->text.string);
}

/*-----------------------------------------------------------
|
|  Routine Name: Realize
|
|       Purpose: When the window for the text widget is first created,
|		 the window width is determined; at that time,
|		 text wrapping needs to occur (if wrapping is being done).
|
|         Input: widget - the text object being realized
|                valuemask  - determines the window attributes being passed in
|                attributes - the list of window attributes to be set
|    Written By: Danielle Argiro
|          Date: March 14, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Realize(
   Widget               widget,
   XtValueMask          *valuemask,
   XSetWindowAttributes *winattrib)
{
        XvwTextWidget xwid = kwidget(widget);
	int dummy;


	/* call superclass's method for realization */
        (*xvwTextWidgetClass->core_class.superclass->core_class.realize)
                (widget, valuemask, winattrib);

        /*
	 * since this is the first time we have width and height, 	
	 * need to wrap lines if necessary. 
         */
        if (xwid->text.wrap_type != KTEXT_WRAP_NEVER)
	    SetLabelLines(xwid, &dummy);
}

/*-----------------------------------------------------------
|
|  Routine Name: Resize
|
|       Purpose: Method called on resize to refresh text widget
|         Input:
|    Written By: Danielle Argiro
|          Date: Jan 16, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Resize(
   Widget widget)
{
        XvwTextWidget xwid = kwidget(widget);
	int dummy;

	/* call superclass's method for resizing */
        (*xvwTextWidgetClass->core_class.superclass->core_class.resize)
                (widget);

        /*
	 * need to wrap lines based on change of size if necessary. 
         */
	if (xwid->text.wrap_type != KTEXT_WRAP_NEVER)
	    SetLabelLines(xwid, &dummy);

}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: Determine what the widget should do by
|		 comparing the current public settable values 
|		 versus the new settings.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new1     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: January 19, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwTextWidget cxwid = (XvwTextWidget) current;
	XvwTextWidget nxwid = (XvwTextWidget) new1;
	int redisplay = FALSE;

        if (cxwid->text.multiline != nxwid->text.multiline)
        {
            if (nxwid->text.multiline)
	    {
                xvw_set_attribute(xvw_object(new1), 
				  XVW_MAXIMUM_HEIGHT, ~0L);
		xvw_set_attribute(nxwid->text.text,
				  XVW_LABEL_MULTILINE, TRUE);
	    }
            else
            {
                xvw_set_attribute(xvw_object(new1), 
				  XVW_CHAR_MAX_HEIGHT, 1.0);
		xvw_set_attribute(nxwid->text.text,
				  XVW_LABEL_MULTILINE, TRUE);
                nxwid->text.wrap_type = KTEXT_WRAP_NEVER;
            }
            redisplay = TRUE;
        }

	if (cxwid->text.filename != nxwid->text.filename)
        {
	    char *buffer = NULL;
	    buffer = (char *)kreadfile(nxwid->text.filename, NULL);
	    if (buffer != NULL)
		xvw_set_attribute(xvw_object(new1), XVW_TEXT_STRING, buffer);
	    else kerror(KWIDGETS, "SetValues", 
		       "Can't read text from file %s\n", nxwid->text.filename);
	    nxwid->text.orig_text_end = kstrlen(buffer)-1;
	    kfree(buffer);
	}

        if (cxwid->text.wrap_type != nxwid->text.wrap_type)
        {
            redisplay = TRUE;
        }

        if (cxwid->manager.font != nxwid->manager.font)
	{
	    xvw_set_attribute(nxwid->text.text, XVW_FONT, nxwid->manager.font);
	}

	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: ActionHandler
|
|       Purpose: Handles key press navigation of the list, with:
|
|          Up arrow:    moves cursor one line up (leaving x location alone)
|          Down arrow:  moves cursor one line down (leaving x location alone)
|          Left arrow:  moves cursor one character to the left
|                       (if cursor has to move up a line, x location
|                       will be reset to 0)
|          Right arrow: moves cursor one character to the right
|                       (if cursor has to move down a line, x location
|                       will be reset to 0)
|          Home:        moves to beginning of current line
|          End:         moves to end of current line
|
|          Prior (Page Up):    moves back (up) one page
|          Next (Page Down):   moves forward (down) one page
|
|          <Ctrl> Up Arrow:    moves to beginning of current paragraph
|                              (or to previous paragraph if we're already at
|                              the beginning of the paragraph)
|          <Ctrl> Down Arrow:  moves to beginning of next paragraph
|          <Ctrl> Left Arrow:  moves to beginning of current word
|          <Ctrl> Right Arrow: moves to beginning of next word
|          <Ctrl> Home:        moves to beginning of available text
|          <Ctrl> End:         moves to end of available text
|
|          <Ctrl> Prior (Page Up)
|                - or -
|          <Alt>  Left Arrow:  moves left one page
|
|          <Ctrl> Next (Page Down)
|               - or -
|          <Alt> Right Arrow:  moves left one page
|
|         Input: widget - the Text widget
|                event  - event that triggered the action handler
|		 params - passes in string saying what to do
|    Written By: Mark Young & Danielle Argiro
|          Date: January 19, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ActionHandler(
   Widget   widget,
   XEvent   *event,
   String   *params,
   Cardinal *num_params)
{
	int i;
	xvobject object = xvw_object(widget);

        /*
         * Process the different actions by calling the TextProcessAction
	 * routines.
         */
	for (i = 0; i < *num_params; i++)
	{
	   if (TextProcessAction(object, event, params[i]) == FALSE)
	      return;
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: TextProcessAction
|
|       Purpose: Handles key press navigation of the list, with:
|
|          Up arrow:    moves cursor one line up (leaving x location alone)
|          Down arrow:  moves cursor one line down (leaving x location alone)
|          Left arrow:  moves cursor one character to the left
|                       (if cursor has to move up a line, x location
|                       will be reset to 0)
|          Right arrow: moves cursor one character to the right
|                       (if cursor has to move down a line, x location
|                       will be reset to 0)
|          Home:        moves to beginning of current line
|          End:         moves to beginning of current line
|
|          Prior (Page Up):    moves back (up) one page
|          Next (Page Down):   moves forward (down) one page
|
|          <Ctrl> Up Arrow:    moves to beginning of current paragraph
|                              (or to next paragraph up if we're already at
|                              the beginning of the paragraph)
|          <Ctrl> Down Arrow:  moves to beginning of next paragraph
|          <Ctrl> Left Arrow:  moves to beginning of current word
|          <Ctrl> Right Arrow: moves to beginning of next word
|          <Ctrl> Home:        moves to beginning of available text
|          <Ctrl> End:         moves to end of available text
|
|          <Ctrl> Prior (Page Up)
|                - or -
|          <Alt>  Left Arrow:  moves left one page
|
|          <Ctrl> Next (Page Down)
|               - or -
|          <Alt> Right Arrow:  moves left one page
|
|         Input: object - the Text object
|                event  - event that triggered the action handler
|		 params - passes in string saying what to do
|    Written By: Mark Young & Danielle Argiro
|          Date: January 19, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static int TextProcessAction(
   xvobject object,
   XEvent   *event,
   String   action)
{
        XvwTextWidget xwid = (XvwTextWidget) xvw_widget(object);
	Widget widget = (Widget) xwid;
	int    cursor_pos;
	char   *string = xwid->text.string;
	static Pixel  save_top_shadow, save_bottom_shadow;

        /*
         * Get the current cursor position
         */
	xvw_get_attribute(xwid->text.text, XVW_LABEL_CURSOR_POS, &cursor_pos);

	/*
	 * Key press: add text located at the current cursor position
	 */
	if (kstrcmp(action, "AddText") == 0)
	{
	    if (event->type != KeyPress && event->type != KeyRelease)
	    {
	       kerror(KWIDGETS, "TextProcessAction", "AddText directive can "
				"only be associated with KeyPress or "
				"KeyRelease events");
	       return(FALSE);
	    }
	    else
	        TextProcessKeyPress(object, event);
	}

        /*
         * Up arrow: moves cursor one line up (leaving x location alone)
         */
	else if (kstrcmp(action, "MoveUpOneLine") == 0)
	{
            int cursor_yloc;
            xvw_get_attribute(xwid->text.text, XVW_LABEL_CURSOR_YLOC,
			      &cursor_yloc);
            cursor_yloc--;
	    cursor_yloc = kmax(cursor_yloc, 0);
            xvw_set_attribute(xwid->text.text,
                              XVW_LABEL_CURSOR_YLOC, cursor_yloc);
	    LabelResetCutCursorPosition(xwid->text.text);
	
	}

        /*
         * Down arrow: moves cursor one line down (leaving x location alone)
         */
	else if (kstrcmp(action, "MoveDownOneLine") == 0)
	{
            int cursor_yloc;
            xvw_get_attribute(xwid->text.text, XVW_LABEL_CURSOR_YLOC, 
			      &cursor_yloc);
            cursor_yloc++;
	    cursor_yloc = kmin(cursor_yloc, kstrlen(string));
            xvw_set_attribute(xwid->text.text, 
			       XVW_LABEL_CURSOR_YLOC, cursor_yloc);
	    LabelResetCutCursorPosition(xwid->text.text);
	}

	/*
	 * Left arrow: moves cursor one character to the left
	 *             (if cursor has to move up a line, x location 
 	 *             will be reset to 0)
	 */
	else if (kstrcmp(action, "MoveLeftOneCharacter") == 0)
	{
	    cursor_pos--;
	    xvw_set_attribute(xwid->text.text, XVW_LABEL_CURSOR_POS,cursor_pos);
	    LabelResetCutCursorPosition(xwid->text.text);
	}

	/*
	 * Right arrow: moves cursor one character to the right
	 *              (if cursor has to move down a line, x location 
	 *              will be reset to 0)
	 */
	else if (kstrcmp(action, "MoveRightOneCharacter") == 0)
	{
            cursor_pos++;
	    xvw_set_attribute(xwid->text.text, XVW_LABEL_CURSOR_POS,cursor_pos);
	    LabelResetCutCursorPosition(xwid->text.text);
	}

	/*
	 * Home: moves to beginning of current line
	 */
	else if (kstrcmp(action, "MoveToBeginLine") == 0)
	{
	    xvw_set_attribute(xwid->text.text, XVW_LABEL_CURSOR_XLOC, 0);
	    LabelResetCutCursorPosition(xwid->text.text);
	}

	/*
	 * End: moves to beginning of current line
	 */
	else if (kstrcmp(action, "MoveToEndLine") == 0)
	{
            xvw_set_attribute(xwid->text.text, XVW_LABEL_CURSOR_XLOC, -1);
	    LabelResetCutCursorPosition(xwid->text.text);
	}

	/*
	 * Prior (Page Up): moves back (up) one page
	 */
	else if (kstrcmp(action, "MoveUpOnePage") == 0)
	{
	    LabelPageUp(xwid->text.text);
	    LabelResetCutCursorPosition(xwid->text.text);
	}

	/*
	 * Next (Page Down): moves forward (down) one page
	 */
	else if (kstrcmp(action, "MoveDownOnePage") == 0)
	{
	    LabelPageDown(xwid->text.text);
	    LabelResetCutCursorPosition(xwid->text.text);
	}

	/*
	 * <Ctrl> Up Arrow:  moves to beginning of current paragraph 
	 *                   (or to next paragraph up if we're already at 
	 *                   the beginning of the paragraph)
	 */
	else if (kstrcmp(action, "MoveToBeginParagraph") == 0)
	{
            int count = 0, pos;

            /*
             * if there is white space before the cursor, skip it.  this
             * handles the case where we are already at the beginning of the
             * current paragraph, and we need to move to the end of the previous
             * paragraph before backing up to the previous word's beginning
             */
	    while (cursor_pos-1 >= 0 && isspace(string[cursor_pos-1]))
                cursor_pos--;
            /*
             * skip everything until we've seen two newlines with
             * nothing intervening except whitespace.
             */
	    pos = cursor_pos;
            while (pos >= 0)
            {
                if (!isspace(string[pos]))
                {
                   if (count >= 2) break;
                   count = 0;
		   cursor_pos = pos;
                }
                else if (string[pos] == '\n') count++;
                pos--;
            }
            xvw_set_attribute(xwid->text.text, XVW_LABEL_CURSOR_POS,cursor_pos);
	    LabelResetCutCursorPosition(xwid->text.text);
	}

	/*
	 * <Ctrl> Down Arrow: moves to beginning of next paragraph
	 */
	else if (kstrcmp(action, "MoveToNextParagraph") == 0)
	{
	    int count = 0, length = kstrlen(xwid->text.string);

            /*
             * if the cursor is on white space, rewind to the end of the
	     * previous paragraph before starting.
             */
            while (cursor_pos > 0 && isspace(string[cursor_pos]))
                cursor_pos--;

	    /* 
             * skip everything until we've seen two newlines with
	     * nothing intervening except whitespace.
	     */
	    while (cursor_pos < length)
	    {
	        if (!isspace(string[cursor_pos]))
		{
		   if (count >= 2) break;
		   count = 0;
		}
		else if (string[cursor_pos] == '\n') count++;
		cursor_pos++;
	    }
	    xvw_set_attribute(xwid->text.text, XVW_LABEL_CURSOR_POS,cursor_pos);
	    LabelResetCutCursorPosition(xwid->text.text);
	}

	/*
	 * <Ctrl> Left Arrow: moves to beginning of current word
	 */
	else if (kstrcmp(action, "MoveToBeginWord") == 0)
	{
            /*
             * if there is white space before the cursor, skip it.  this 
	     * handles the case where we are already at the beginning of the 
	     * current word, and we need to move to the end of the previous 
	     * word before backing up to the previous word's beginning
             */
	    while (cursor_pos-1 >= 0 && isspace(string[cursor_pos-1]))
		cursor_pos--;
	    
	    /* back up past all letters to beginning of word */
	    while (cursor_pos-1 >= 0 && !isspace(string[cursor_pos-1]))
		cursor_pos--;

            /* reset cursor position */
	    xvw_set_attribute(xwid->text.text, XVW_LABEL_CURSOR_POS,cursor_pos);
	    LabelResetCutCursorPosition(xwid->text.text);
	}

	/*
	 * <Ctrl> Right Arrow: moves to beginning of next word
	 */
	else if (kstrcmp(action, "MoveToNextWord") == 0)
	{
	    int length = kstrlen(xwid->text.string);

	    /* move forward past all letters to end of word */
	    while (cursor_pos < length && !isspace(string[cursor_pos]))
		cursor_pos++;

	    /* skip white space until next word */
	    while (cursor_pos < length && isspace(string[cursor_pos]))
		cursor_pos++;

	    /* reset cursor position */
	    xvw_set_attribute(xwid->text.text, XVW_LABEL_CURSOR_POS,cursor_pos);
	    LabelResetCutCursorPosition(xwid->text.text);
	}

        /*
         * <Ctrl> Home: moves to beginning of available text
         */
	else if (kstrcmp(action, "MoveToBeginFile") == 0)
	{
	   /* simply set cursor position to 0 */
            xvw_set_attribute(xwid->text.text, XVW_LABEL_CURSOR_POS, 0);
	    LabelResetCutCursorPosition(xwid->text.text);
	}

        /*
         * <Ctrl> End: moves to end of available text
         */
	else if (kstrcmp(action, "MoveToEndFile") == 0)
	{
	   /* simply set cursor position to end of available text */
	    xvw_set_attribute(xwid->text.text, XVW_LABEL_CURSOR_POS, 
			     kstrlen(xwid->text.string));
	    LabelResetCutCursorPosition(xwid->text.text);
	}

        /*
         *  EnterNotify: Puts focus in text widget
         */
        else if (kstrcmp(action, "FocusIn") == 0)
        {
	    /* 
	     * if the timeout that causes the cursor to blink is not already
	     * installed, install it.  Then change the top and bottom shadow
	     * color to be the same as the border color (effectively thickening
	     * the border)
	     */
            XtAppContext app_context = XtWidgetToApplicationContext(widget);
	    if (xwid->text.blink_id == 0)
	    {
                xwid->text.blink_id = XtAppAddTimeOut(app_context,
					(unsigned long) 500,
                                        TextBlinkCursorTimeout, xwid);
	        xvw_get_attributes(xvw_object(widget),
                               XVW_TOP_SHADOW_PIXEL,    &save_top_shadow,
                               XVW_BOTTOM_SHADOW_PIXEL, &save_bottom_shadow,
                               NULL);

  	        xvw_set_attributes(xvw_object(widget), 
			       XVW_TOP_SHADOW_PIXEL,    xwid->core.border_pixel,
			       XVW_BOTTOM_SHADOW_PIXEL, xwid->core.border_pixel,
		 	       NULL);
	    }

        }

        /*
         *  LeaveNotify: Takes focus out of text widget
         */
        else if (kstrcmp(action, "FocusOut") == 0)
        {
	    /*
	     * remove the timeout that causes the cursor to blink, make sure
	     * that the cursor is displayed, and restore the top shadow and
	     * bottom shadow colors (putting the border back to normal)
	     */
	    xvw_set_attribute(xwid->text.text, XVW_LABEL_DISPLAY_CURSOR, TRUE);
	    if (xwid->text.blink_id != 0)
	    {
	        XtRemoveTimeOut(xwid->text.blink_id);
	        xwid->text.blink_id = 0;
	    }
	    xvw_set_attributes(xvw_object(widget),
                               XVW_TOP_SHADOW_PIXEL,    save_top_shadow,
                               XVW_BOTTOM_SHADOW_PIXEL, save_bottom_shadow,
                               NULL);
        }

        /*
         *  {Ctrl} Insert: Pastes contents of cut buffer at cursor location
         */
        else if (kstrcmp(action, "PasteBuffer") == 0)
        {
	    /* get contents of cut buffer from label widget */
	    LabelPasteBuffer(xwid->text.text);
        }

	else
	{
	    kerror(KWIDGETS, "TextProcessAction", 
                   "Unknown action directive '%s'", action);
	    return(FALSE);
	}

	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: TextBlinkCursorTimeout
|
|       Purpose: Contains the code that makes the cursor blink
|
|         Input: client_data - used to pass in the Text widget
|                id          - timeout id
|    Written By: Danielle Argiro & Mark Young
|          Date: April 24, 1997
|
------------------------------------------------------------*/
static void TextBlinkCursorTimeout(
    kaddr client_data,
    XtIntervalId *id)
{
	int display_cursor;
        XvwTextWidget xwid = (XvwTextWidget) client_data;
        XtAppContext  app_context = XtWidgetToApplicationContext((Widget)xwid);
	
	xvw_get_attribute(xwid->text.text, XVW_LABEL_DISPLAY_CURSOR,
			  &display_cursor);
	display_cursor = !display_cursor;
	xvw_set_attribute(xwid->text.text, XVW_LABEL_DISPLAY_CURSOR,
                          display_cursor);

        xwid->text.blink_id = XtAppAddTimeOut(app_context, (unsigned long) 500,
                                              TextBlinkCursorTimeout, xwid);
}

/*-----------------------------------------------------------
|
|  Routine Name: TextPasteCallback
|
|       Purpose: Text widget callback to handles a buttonpress event 
|		 in the label object when there is text to paste
|		 from the cut buffer.
|
|         Input: object       - the Text object
|                client_data  - not used
|                call_data    - passes in the cut buffer
|    Written By: Danielle Argiro & Mark Young
|          Date: January 20, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void TextPasteCallback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
 	char *tmp, *buffer = (char *) call_data;
        XvwTextWidget xwid = (XvwTextWidget) client_data;
	
	/*
	 * if we have a non-editable text widget, this is not allowed!
	 */
	if (xwid->text.edit_type == KTEXT_READ)
	{
            XBell(XtDisplay((Widget)xwid), 0);
            return;
	}

	/* 
	 * if we have a single-line text widget and what we are
	 * about to paste has newlines in it, take the newlines out
	 */
	if (!xwid->text.multiline && (kstrstr(buffer, "\n") != NULL))
	{
	    tmp = kstring_seddata(buffer, "\n", " ", FALSE, NULL);

	    /*
	     * insert text from cut buffer into text widget
	     */
	    TextInsertString(xwid, tmp, kstrlen(tmp));
	    kfree(tmp);
	}
	else
	{
	    /*
	     * insert text from cut buffer into text widget
	     */
	    TextInsertString(xwid, buffer, kstrlen(buffer));
	}
}


/*-----------------------------------------------------------
|
|  Routine Name: TextProcessKeyPress
|
|       Purpose: When a key is pressed in the text widget,
|		 this utility routine processes the key press event;
|		 it inserts a character at the insert position,
|		 or deletes a character from the insert position.
|
|         Input: object - the text object
|                event  - the key press event 
|    Written By: Mark Young & Danielle Argiro
|          Date: January 19, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void TextProcessKeyPress(
   xvobject object,
   XEvent   *event)
{
        XvwTextWidget xwid = (XvwTextWidget) xvw_widget(object);

	KeySym    keysym;
	char      temp[KLENGTH]; 
	size_t    i, len, pos, length, size;
	char      *string; 
        XKeyEvent *xkey = (XKeyEvent *) &(event->xkey);
        int       dummy, cursor_pos, cut_cursor_pos1, cut_cursor_pos2;


	/* get the characters that they typed in */
        len = (size_t) XLookupString(xkey, temp, KLENGTH, &keysym, NULL);
	if (len <= (size_t) 0) return;

	/* substitute spaces for unprintable characters */
	for (i = 0; i < len; i++)
	{
	    if (!isprint(temp[i])   && keysym != XK_Delete    && 
	 	keysym != XK_Tab    && keysym != XK_BackSpace && 
		keysym != XK_Return && keysym != XK_KP_Enter)
	    {
	       XBell(xvw_display(object), 0);
	       return;
	    }
	}
	length = (size_t) kstrlen(xwid->text.string)+1;

	/*
	 *  Refuse to let user modify text if edit mode is read-only, or edit 
	 *  mode is append and they are trying to change the original text
	 */
        if (xwid->text.edit_type == KTEXT_READ)
	{
	    XBell(xvw_display(object), 0);
	    return;
	}

	/*
	 *  Special cases <delete> and <backspace>:
	 *  Erase the character at the insert position.
	 */
	if (keysym == XK_Delete || keysym == XK_BackSpace)
	{
	   /* 
	    * if cut cursor position 1 is not equal to cut cursor position 2, 
	    * that implies that there is highlighted text to delete, as 
	    * opposed to simply deleting one character at the cursor position
	    */
	   xvw_get_attributes(xwid->text.text, 
			      XVW_LABEL_CURSOR_POS,      &cursor_pos,
			      XVW_LABEL_CUT_CURSOR_POS1, &cut_cursor_pos1,
			      XVW_LABEL_CUT_CURSOR_POS2, &cut_cursor_pos2,
			      NULL);
	   if (cursor_pos == 0 &&  cut_cursor_pos1 == cut_cursor_pos2)
	   {
	      XBell(xvw_display(object), 0);
	      return;
	   }

	   /*
	    * here we figure out how many characters to delete
	    */
	   if (cut_cursor_pos1 != cut_cursor_pos2)
	   {
	        size = kabs(cut_cursor_pos1 - cut_cursor_pos2);
	        pos = kmax(cut_cursor_pos1, cut_cursor_pos2);
	   }
	   else 
	   {
		size = 1;
		pos = cursor_pos;
		cursor_pos--;
	   }

	   /* 
	    * from the insertion position, scoot the rest of the string back
	    * enough to accommodate the new characters.
	    */
	   string = xwid->text.string;
	   if (length > pos)
	      kmemmove(&string[pos-size], &string[pos], length-pos);

	   /*
	    * need to set both cursor position & cut cursor position so that
	    * if we deleted highlighted text
	    */
	   xvw_set_attributes(xwid->text.text, 
			      XVW_LABEL_CURSOR_POS,      cursor_pos,
			      XVW_LABEL_CUT_CURSOR_POS1, cursor_pos,
			      XVW_LABEL_CUT_CURSOR_POS2, cursor_pos,
			      NULL);
           /*
            * create array of text lines from text string,
            * send array of text lines to label object to be displayed,
            * update text string from array of detabbed text lines
            */
           SetLabelLines(xwid, &dummy);
	}

	/*
	 *  Special case <return>:
	 *
	 *  1. If there is a callback installed on XVW_TEXT_ENTER_STRING,
	 *     then fire the callback.
	 *  2. Otherwise, if this is a single-line text object then ignore
	 *     the <return>
	 *  3. Or if it's a multi-line text object, then replace whatever wierd
	 *     character XLookupString() gives us with the more sensible '\n'
	 *     and insert the '\n' into the text. 
	 */
	else if (keysym == XK_Return || keysym == XK_KP_Enter)
	{
	    if (xwid->text.enter_callback != NULL)
	    {
	        XtCallCallbacks((Widget) xwid, XVW_TEXT_ENTER_STRING,
                                 &xwid->text.string);
	    }
	    else if (xwid->text.multiline == TRUE && keysym == XK_Return)
	    {
	        temp[0] = '\n';
	        TextInsertString(xwid, temp, (size_t) 1);
	    }
	}

	/*
	 *  Otherwise it's a "regular" character - just insert it at
	 *  the insert position.
	 */
	else
	{
	    TextInsertString(xwid, temp, len);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: TextInsertString
|
|       Purpose: Utility routine inserts text at the current
|		 insert position.
|
|         Input: xwid   - the Text widget
|		 string - text to insert
|		 len    - length of string
|    Written By: Mark Young & Danielle Argiro
|          Date: January 20, 1997
|
------------------------------------------------------------*/

static void TextInsertString(
    XvwTextWidget xwid,
    char          *buffer,
    size_t        len)
{
	size_t pos, length;
	int    additional_charnum, cursor_pos, size;
	int    cut_cursor_pos1, cut_cursor_pos2; 

	/* sanity check */
	if (len <= (size_t) 0) return;

        /*
	 * this code supports replacement of highlighted text as well as
	 * plain insertion of text.  Therefore if cut_cursor_pos is not equal
	 * to cursor_pos, that means there is text that must be deleted.
         */
	xvw_get_attributes(xwid->text.text, 
			   XVW_LABEL_CURSOR_POS,      &cursor_pos,
			   XVW_LABEL_CUT_CURSOR_POS1, &cut_cursor_pos1,
			   XVW_LABEL_CUT_CURSOR_POS2, &cut_cursor_pos2,
			   NULL);

	length = kstrlen(xwid->text.string)+1;

        /*
         * here, if there was highlighted text, have to delete it first 
	 * before the insertion opration. 
         */
        if (cut_cursor_pos1 != cut_cursor_pos2)
        {
            size = kabs(cut_cursor_pos1 - cut_cursor_pos2);
            pos  = kmax(cut_cursor_pos1, cut_cursor_pos2);

            /*
             * from the insertion position, scoot the rest of the string back
             * enough to accommodate the new characters.
             */
	    if (length > pos)
               kmemmove(&xwid->text.string[pos-size], 
		     &xwid->text.string[pos], length-pos);
	    cursor_pos = pos-size;
        }

	/*
         * from the insertion position, scoot the rest of the string back
         * enough to accommodate the new characters.
         */
        pos = (size_t) cursor_pos;
        if (!CheckStringBufferSize(xwid, (int)(pos+len+length))) return;

	if (length > pos)
           kmemmove(&xwid->text.string[pos+len], 
		 &xwid->text.string[pos], length-pos);

        /* now, insert the new characters at the insert position */
        kmemcpy(&xwid->text.string[pos], buffer, len);

        /* now, insert the new characters at the insert position */
        cursor_pos += (int) len;

        /* 
         * create array of text lines from text string,
         * send array of text lines to label object to be displayed,
         * update text string from array of detabbed text lines
         */
	SetLabelLines(xwid, &additional_charnum);

	/* 
	 * compensate insert position for inserted text that may have grown
	 * in size when it was de-tabbed 
	 */
	cursor_pos += additional_charnum;

        /*
         * need to set both cursor position & cut cursor position so that if we 
	 * replaced highlighted text, highlighted portion will un-highlight
         */
        xvw_set_attributes(xwid->text.text,
                           XVW_LABEL_CURSOR_POS,      cursor_pos,
                           XVW_LABEL_CUT_CURSOR_POS1, cursor_pos,
                           XVW_LABEL_CUT_CURSOR_POS2, cursor_pos,
                           NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: TextSetString
|
|       Purpose: Routine to handle action attribute 
|		 XVW_TEXT_STRING.  This is an action attribute
|		 rather than a normal attribute because we
|		 need to handle dynamic allocation of memory.
|
|         Input: object    - the Text attribute
|		 attribute - XVW_TEXT_STRING ("textString")
|		 calldata  - passes in the new string
|    Written By: Mark Young & Danielle Argiro
|          Date: January 19, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static int TextSetString(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
        XvwTextWidget xwid = (XvwTextWidget) xvw_widget(object);
	char *string = *((char **) calldata);
	int  cursor_pos, dummy;

	/* make sure xwid->text.string is big enough to accept new string */
        if (!CheckStringBufferSize(xwid, (int) kstrlen(string)))
            return(FALSE);

	kstrcpy(xwid->text.string, string);
	
	/* 
	 * create array of text lines from text string,
	 * send array of text lines to label object to be displayed,
	 * update text string from array of detabbed text lines
	 */
	SetLabelLines(xwid, &dummy);

	/* for single-line text, set insert position to the end */
	if (!xwid->text.multiline)
	{
 	    cursor_pos = kstrlen(xwid->text.string);
            xvw_set_attribute(xwid->text.text, XVW_LABEL_CURSOR_POS, 
			      cursor_pos);
	}

	/* mark end of original text for use w/ edit type KTEXT_APPEND */
	xwid->text.orig_text_end = kstrlen(xwid->text.string)-1;
	return(TRUE);
}


/************************************************************
*
*  Routine Name: xvw_create_text - create text GUI object
*
*       Purpose: The \fItext\fP object allows the display and editing
*		 of text.
*
*         Input: parent - parent of the text object
*                name   - a name for this particular instance of the text
*                         object (for use in app-defaults files, etc)
*
*        Output:
*       Returns: The text GUI object on success, NULL on failure. 
*  Restrictions: 
*    Written By: Mark Young & Danielle Argiro
*  	   Date: Jun 15, 1996
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: xvobject xvw_create_text(
*                !   xvobject parent,
*                !   char   *name)
*
*************************************************************/

xvobject xvw_create_text(
   xvobject parent,
   char     *name)
{
	xvobject  object;


	/*
	 *  Call the xvw_create() routine to do the actual creation.
	 */
	object = xvw_create(parent, FALSE, TRUE, name, xvwTextWidgetClass);
	return(object);
}
