 /*
  * Khoros: $Id
  */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id";
#endif
 
 
/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>          Text & Label Widget Utility Routines
   >>>>
   >>>>	Private:  
   >>>>           CheckStringBufferSize()
   >>>>           SetLabelLines()
   >>>>           ConcatenateLines()
   >>>>           DeTabString()
   >>>>	Static:
   >>>>           SetTextLinesNoWrap()
   >>>>           SetTextLinesWrapWord)
   >>>>           SetTextLinesWrapLine()
   >>>>
   >>>>	Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <kwidgets/TextP.h>

static char **SetTextLinesWrapWord PROTO((XvwTextWidget, char *, int *));
static char **SetTextLinesNoWrap   PROTO((XvwTextWidget, char *, int *));
static char **SetTextLinesWrapLine PROTO((XvwTextWidget, char *, int *));

/*-----------------------------------------------------------
|
|  Routine Name: CheckStringBufferSize
|
|       Purpose: The buffer for the string object is allocated
|                in chunks of KLENGTH.  This routine checks to
|                see if the size of the string that is about to
|                be set into the text widget is larger than the
|                currently allocated buffer, and if so it reallocs
|                the string to the next increment of KLENGTH.
|                After calling this routine, you are assured that
|                the buffer is large enough to accomodate the new
|                string.
|
|         Input: xwid - the Text widget
|                len  - length of the string that needs to be stored
|    Written By: Mark Young & Danielle Argiro
|          Date: Mar 19, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
int CheckStringBufferSize(
   XvwTextWidget xwid,
   int len)
{
        if (xwid->text.alloc_size > len)
           return(TRUE);

        xwid->text.alloc_size = ((len/KLENGTH) + 1)*KLENGTH;
        if ((xwid->text.string = (char *)krealloc(xwid->text.string,
                        xwid->text.alloc_size * sizeof(char))) == NULL)
           return(FALSE);
        else
           return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetLabelLines
|
|       Purpose: Routine to translate text string that came from
|                XVW_TEXT_STRING attribute into an array of de-tabbed
|		 strings that can be sent to the label object with
|		 XVW_LABEL_LINES.  Then, the array of de-tabbed strings
|		 is concatenated together and stored back in
|		 text string so that things don't become confused.
|
|         Input: xwid - the text widget
|	 Output: additional_charnum - number of additional characters
|			              in string that came from replacing
|				      tabs with spaces
|    Written By: Danielle Argiro
|          Date: March 11, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
void SetLabelLines(
	XvwTextWidget xwid,
	int *additional_charnum)
{
	int  i, size = 0, numlines, add_chars;
	char *detabbed;
	char **lines;

	*additional_charnum = 0;

	/*
	 * simple case where there is a NULL or empty string, no need to
	 * do any of this...
	 */
	if (kstrlen(xwid->text.string) == 0)
	{
            xvw_set_attribute(xwid->text.text, XVW_LABEL, xwid->text.string);
	    return;
	}

        /* 
	 * turn text string into array of text strings; if the widget isn't
	 * realized, wrapping is set to KTEXT_WRAP_NEVER, or it's a 
	 * single-line text widget, then we don't want to wrap
	 */
	if (!xwid->text.multiline ||
	    !xvw_check_realized(xvw_object((Widget)xwid)) ||
	    xwid->text.wrap_type == KTEXT_WRAP_NEVER)
            lines = SetTextLinesNoWrap(xwid, xwid->text.string, &numlines);
	else if (xwid->text.wrap_type == KTEXT_WRAP_WORD)
            lines = SetTextLinesWrapWord(xwid, xwid->text.string, &numlines);
	else /* xwid->text.wrap_type == KTEXT_WRAP_LINE */
	    lines = SetTextLinesWrapLine(xwid, xwid->text.string, &numlines);

#if 0
	kfprintf(kstderr, "\n\n");
	for (i = 0; i < numlines; i++)
    	    kfprintf(kstderr, "lines[%d] = '%s'\n", i, lines[i]);
#endif

        /* de-tab the array of text strings */
        for (i = 0; i < numlines; i++)
        {
            detabbed = DeTabString(lines[i], &add_chars);
            kfree(lines[i]);
            lines[i] = detabbed;
	    size += kstrlen(lines[i]);
	    *additional_charnum += add_chars;
        }

        /* make sure xwid->text.string is big enough to accept new string */
        if (!CheckStringBufferSize(xwid, (int) kstrlen(xwid->text.string) +
						       *additional_charnum))
	{
	      karray_free(lines, KSTRING, (size_t) numlines, NULL);
              return;
	}

	/* now have to update text string from detabbed text lines */
	ConcatenateLines(lines, numlines, xwid->text.string);
#if 0
kfprintf(kstderr, "xwid->text.string: '%s'\n", xwid->text.string);
#endif

	/* set the text into the label object subpart of the text objecgt */
        xvw_set_attributes(xwid->text.text, 
			   XVW_LABEL_NUMLINES, numlines,
			   XVW_LABEL_LINES, lines,
			   NULL);

	karray_free(lines, KSTRING, (size_t) numlines, NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: ConcatenateLines
|
|       Purpose: Concatenates array of lines into a single string
|
|         Input: 
|        Output: 
|    Written By: Danielle Argiro & Mark Young
|          Date: April 8, 1997
|
------------------------------------------------------------*/
void ConcatenateLines(
    char **lines, 
    int  numlines, 
    char *string)
{
        int i, len, indx = 0;

	if (numlines == 0)
	    return;

        for (i = 0; i < numlines-1; i++)
        {
	    if ((len = kstrlen(lines[i])) > 0)
	        kstrncpy(&string[indx], lines[i], (size_t) len);

	    string[indx+len] = '\n';
            indx += len+1;
        }
        kstrcpy(&string[indx], lines[i]);
}

/*-----------------------------------------------------------
|
|  Routine Name: DeTabString
|
|       Purpose: Replaces tabs in a string with spaces 
|
|         Input: string    - the string to be detabbed
|        Output: add_chars - tells you how many additional characters
|			     were added to the string in the process
|			     (ie, the difference between the size of the 
|			      original string with tabs and the new string
|			      with spaces).
|    Written By: Danielle Argiro & Mark Young
|          Date: March 11, 1997
|
------------------------------------------------------------*/
char *DeTabString(
   char *string, 
   int  *add_chars)
{
	char *tmp = string;
	char *detabbed = NULL;
	int  size, i, dtlength = 0;

	*add_chars = 0;
	while (tmp != NULL)
	{
	    tmp = kstrchr(string, '\t');

	    if (!tmp) size = (int) kstrlen(string);
            else size = tmp-string;

	    detabbed = (char *) krealloc(detabbed, (dtlength+size+1) 
				         * sizeof(char));
	    kstring_ncopy(string, (size_t) size, &detabbed[dtlength]); 
	    dtlength += size;

            if (tmp != NULL)
            {
                size = 8 - (dtlength % 8);
	       
	        detabbed = (char *) krealloc(detabbed, (dtlength+size+1) * 
			                    sizeof(char));
                for (i = 0; i < size; i++)
                   detabbed[dtlength++] = ' ';
	        tmp++;

	     	*add_chars += size-1;
            }
	    string = tmp;
	}
        detabbed[dtlength] = '\0'; 
	return(detabbed);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetTextLinesNoWrap
|
|       Purpose: Creates array of text lines from text string,
|		 where each text line will be broken at \n.
|		 No word wrapping or line wrapping is performed.
|
|		 If text lines extend past the width of the text widget
|		 window, users have to use arrow keys to scroll the
|		 text so that they can see the letters of the text lines.
|
|                Empty lines are used to represent \n's by themselves, while
|                \n's at the end of a line are represented by line breaks.
|                For example, the string, "50 days in the sun\n\nvacation fun\n"
|                would be translated into:
|                       lines[0] = "50 days in the sun"
|                       lines[1] = ""
|                       lines[2] = "vacation fun"
|
|         Input: xwid     - the Text widget
|        Output: numlines - number of lines in returned array
|	Returns: lines    - returns array of text lines
|
|    Written By: Danielle Argiro & Mark Young
|          Date: March 19, 1997
|
------------------------------------------------------------*/
static char **SetTextLinesNoWrap(
   XvwTextWidget xwid,
   char *string,
   int  *numlines)
{
	char *tmp, *tmp2, *line;
	int  size;
	char **lines = NULL;
	size_t nlines;

	/*
	 *  The algorithm used for creating array of text lines is as follows: 
	 *
	 *  1. keep a pointer to the head of the text being worked on 
	 *     as the wrapping algorithm iterates, called 'string'.
	 *  2. look for \n at which to break the line,
	 *  3. set 'tmp' pointer to the next character after the last character
	 *     in the desired line segment and 'size' to the number of 
	 *     characters in the desired line segment
	 *  4. using 'string' and 'size', copy the desired line segment into
	 *     the lines array.
	 *  5. update 'string' to point at the next character after the
	 *     recently added line segment, using 'tmp', and repeat until
	 *     there are no more characters left in the string.
	 */
	nlines = 0; tmp = NULL;
        do
        {
	    /* 
	     * look for \n at which to break the line; set 'tmp' pointer to 
             * end of line segment where we want to break it, and 'size' to
             * the number of chars in the line segment that will be broken
	     */ 
	    tmp2 = NULL;
	    tmp = kstrpbrk(string, "\r\n"); 
	    if (!tmp)
                size = (int) kstrlen(string);
            else size = tmp-string;

	    /*
	     * copy desired line segment of determined 'size' from 'string',
	     * and store it in the lines array. update line count, and increment
	     * tmp ptr past whitespace if necessary.
	     */
            line = kstring_ncopy(string, (size_t) size, NULL); 
            if (kstrlen(tmp) == 0 || *tmp == '\n' || *tmp == '\r')
            {
               lines = (char **) karray_insert(lines, KSTRING, line,
                                     nlines, KLIST_TAIL, TRUE);
               nlines++;

	       /* 
	        * tmp2 allows us to snag \n's at the end of the string, and
	        * make sure that an empty line is correctly added to the lines
	        * array.  see comments in SetTextLinesWrapWord().
	        */
	       tmp2 = kstrpbrk(tmp, "\n\r");
               if (kstrlen(tmp) > 0) tmp++;
            }
	
	    /* update 'string' pointer past current line segment, and repeat */
            string = tmp;

        } while (kstrlen(string) > 0 || tmp2 != NULL);

	*numlines = (int) nlines;
	return(lines);
}


/*-----------------------------------------------------------
|
|  Routine Name: SetTextLinesWrapWord
|
|       Purpose: This routine creates the array of text line segments
|                that are sent to the Label object for display.
|
|                It wraps text at words according to the width of the text 
|                widget; lines will be broken at \n or at the last word
|                that can fit in the window, whichever comes first.
|                If a text line consists of one long word that will not
|                fit in the window, the string will not be wrapped, and the
|                user will have to use the arrow keys to scroll the word
|                in order to see all the characters in it.
|
|                Empty lines are used to represent \n's by themselves, while
|                \n's at the end of a line are represented by line breaks.
|                For example, the string, "\nwe are all here\nand it's ok\n"
|                would be translated into:
|                       lines[0] = ""
|                       lines[1] = "we are all here"
|                       lines[2] = "and it's ok"
|                       lines[3] = ""
|
|         Input: xwid     - the Text widget
|        Output: numlines - number of lines in returned array
|       Returns: lines    - returns array of text lines
|
|    Written By: Danielle Argiro & Mark Young
|          Date: March 11, 1997
|
------------------------------------------------------------*/
static char **SetTextLinesWrapWord(
   XvwTextWidget xwid,
   char *string,
   int  *numlines)
{
        char *tmp, *tmp2, *wrapline, *line;
        int  i, size, linewidth;
        char **lines = NULL;
        size_t nlines;
        int  thickness = xwid->manager.shadow_thickness + 1;
        int  width = xwid->core.width - 2*thickness;
	int  char_width = (int) xwid->manager.font->max_bounds.width;

	/*
	 * The algorithm used for creating array of text lines that wrap
	 * at word breaks is the same as the one where no wrapping is 
	 * done (see SetTextLinesNoWrap) except that we look for \n at which to 
         * break the line, or the last word that will fit without extending 
         * outside the window, whichever comes first.
	 */
	nlines = 0; tmp = NULL;
        do
        {
	    /* 
	     * make copy of remainder of string; a copy is necessary since
	     * we'll need to put in \0's experimentally to see how long
	     * the line can get before the window width is exceeded.
	     * 'wrapline' will contain the desired line segment, broken
	     * after the last word that can fit without going out of window
	     */
	    if (tmp == NULL) tmp = string;
	    wrapline = kstring_copy(tmp, NULL);

	    /* 
	     * if the first thing in the string is <cr>, that's it - we are
	     * ready to go. no need to wrap or spend time at any of this.
	     */
	    if (wrapline[0] == '\n' || wrapline[0] == '\r')
	    {
		size = 0;
	    }
	    else
	    {
	        /*
	         *  scan first for a \n to break at.  if there is one, that
	         *  line segment might fit in the window as it is.
	         */
	        tmp2 = kstrpbrk(wrapline, "\r\n");
            	if (tmp2) 
	    	{
	            size = kstrlen(tmp2);
	            wrapline[kstrlen(wrapline)-size] = '\0';
	    	}

		/*
	         *  now get line width of line segment so far.
	         */
	        linewidth = (int) XTextWidth(xwid->manager.font, 
				             wrapline, (int) kstrlen(wrapline));
	
	        /*
	         * while line segment is too big, scan backwards in it for a
	         * space at which to break. if there is a space, put \0 in at 
	         * that location, and try again. if there is no space at which
	         * to break, the size will have to be the too-wide line segment;
	         * to give up, set linewidth to 0 to stop iteration
	         */
	        while (linewidth > width-char_width)
	        {
		    tmp2 = kstrrchr(wrapline, ' ');
		    if (!tmp2) 
		    {
		        size = (int) kstrlen(wrapline);
	                linewidth = 0;
		    }
		    else 
	            {
		        size = kstrlen(tmp2); 
		        wrapline[kstrlen(wrapline)-size] = '\0';
	                linewidth = (int) XTextWidth(xwid->manager.font, 
				            wrapline, (int) kstrlen(wrapline));
	            }
	        }

	        /*
	         *  'size' is now the size of the desired line segment, but now 
                 *  have to get the 'tmp' ptr pointing to the that location in 
	         *  the original string, 
	         */
	        tmp = string;
	        size = kstrlen(wrapline);
	        for (i = 0; i < size; i++) tmp++;
	    }

	    kfree(wrapline);

	    /*
	     * copy desired line segment of determined 'size' from 'string',
	     * and store it in the lines array. update line count, and increment
	     * tmp ptr past whitespace if necessary.  note that if we were
	     * pointing to a \n, this will enter a line of size 0 in the array.
	     */
            line = kstring_ncopy(string, (size_t) size, NULL); 
            lines = (char **)karray_insert(lines, KSTRING, line,
                                     nlines, KLIST_TAIL, TRUE);
            nlines++;

	    /*
	     * nasty little special case where if the string *ends* in \n,
	     * we need to iterate one more time to enter the desired empty
	     * line in the array, and the way this loop is structured 
	     * (incrementing tmp past \n's and spaces in the next code seg), 
	     * it won't go the last time unless you force it. tmp2 does this.
	     */
	    tmp2 = kstrpbrk(tmp, "\n\r "); 

	    /*
	     * increment past the space or the \n at the end of a line, if
	     * the line ended in one
	     */
	    if (*tmp == '\n' || *tmp == '\r' || *tmp == ' ')
		tmp++;

	    /* update 'string' pointer past current line segment, and repeat */
            string = tmp;

        } while (kstrlen(string) > 0 || tmp2 != NULL);

	*numlines = (int) nlines;

	return(lines);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetTextLinesWrapLine
|
|       Purpose: This routine creates the array of text line segments
|		 that are sent to the Label object for display.
|
|		 It wraps text at lines according to the width of the text 
|		 widget; lines will be broken at \n or at the last character 
|                that can fit in the window, whichever comes first.
|
|		 Empty lines are used to represent \n's by themselves, while
|		 \n's at the end of a line are represented by line breaks.
|		 For example, the string, "1234567890123\n34343243434\n" 
|		 would be translated into:
|		 	lines[0] = "1234567890123
|		 	lines[1] = "34343243434
|		 	lines[2] = ""
|
|         Input: xwid     - the Text widget
|        Output: numlines - number of lines in returned array
|       Returns: lines    - returns array of text lines
|
|    Written By: Danielle Argiro & Mark Young
|          Date: March 11, 1997
|
------------------------------------------------------------*/
static char **SetTextLinesWrapLine(
   XvwTextWidget xwid,
   char *string,
   int  *numlines)
{
        char *tmp, *tmp2, *wrapline, *line;
        int  i, size, linewidth;
        char **lines = NULL;
        size_t nlines;
        int  thickness = xwid->manager.shadow_thickness + 1;
        int  width = xwid->core.width - 2*thickness;
	int  char_width = (int) xwid->manager.font->max_bounds.width;

	/*
	 * The algorithm used for creating array of text lines that wrap
	 * at the last character that will fit on a line is the same as the 
	 * one where wrapping is done at word breaks (see SetTextLinesWrapWord) 
	 * except that we look for \n at which to 
         * break the line, or the last word that will fit without extending 
         * outside the window, whichever comes first.
	 */
	nlines = 0; tmp = NULL;
        do
        {
	    /* wrapline is the copy of the string that we can mess with */
	    if (tmp == NULL) tmp = string;
	    wrapline = kstring_copy(tmp, NULL);

	    /* look for a new line to break at if there is one */
            tmp2 = kstrpbrk(wrapline, "\r\n");
            if (tmp2)
            {
                size = kstrlen(tmp2);
                wrapline[kstrlen(wrapline)-size] = '\0';
            }

	    /* then go backwards from end until string fits in window */
            i = kstrlen(wrapline);
	    linewidth = width;
            while (linewidth > width - char_width)
            {
	  	wrapline[i] = '\0';
                linewidth = (int) XTextWidth(xwid->manager.font, wrapline, 
					     (int) kstrlen(wrapline));
                i--;
            }

	    /*
	     * set 'tmp' to next character in string following last character
	     * in the desired line segment, set 'size' to the size of the 
	     * desired text line segment that we'll add to the lines array 
	     */
	    tmp = string;
	    size = kstrlen(wrapline);
	    for (i = 0; i < size; i++)
	        tmp++;
	    kfree(wrapline);

	    /* allocate line and add it to the array */
            line = kstring_ncopy(string, (size_t) size, NULL); 
            lines = (char **) karray_insert(lines, KSTRING, line, nlines, 
			          KLIST_TAIL, TRUE);
            nlines++;

	    /*
             * special case for if entire text string ends in a \n, need to 
	     * iterate one more time to enter the necessary empty line in the 
	     * array; use of tmp2 forces the last iteration.
             */
            tmp2 = kstrpbrk(tmp, "\n\r");

            /* increment past \n at end of line, if the line ended in one */
            if (*tmp == '\n') tmp++;

	    /* move string ptr down to tmp, and continue */
            string = tmp;

        } while (kstrlen(string) > 0 || tmp2 != NULL);

	*numlines = (int) nlines;
	return(lines);
}
