/*
 * Khoros: $Id: wid_util.c,v 1.16.2.5 1997/05/24 19:02:43 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: wid_util.c,v 1.16.2.5 1997/05/24 19:02:43 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		General GUI Element Creation Routines         <<<<
   >>>>  Private:
   >>>>                 xvf_create_formback()
   >>>>                 xvf_create_selback()
   >>>>                 xvf_create_menubutton()
   >>>>                 xvf_create_button()
   >>>>                 xvf_create_menu_button()
   >>>>                 xvf_create_optional_button()
   >>>>                 xvf_create_boolean_button()
   >>>>                 xvf_create_label()
   >>>>                 xvf_create_menu_label()
   >>>>                 xvf_create_pseudo_label()
   >>>>                 xvf_create_text()
   >>>>                 xvf_create_cr_pixmap()
   >>>>                 xvf_create_scroll()
   >>>>                 xvf_get_pixmap_filename()
   >>>>   Static:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/*------------------------------------------------------------
|
|  Routine Name: xvf_create_formback()
|
|       Purpose: Creates a backplane for general use.
|
|	  Input: parent    - parent of the backplane object
|		 minwidth  - minimum width of backplane
|		 minheight - minimum height of backplane
|		 name      - name to reference backplane object
|        Output: none
|	Returns: the backpane object
|    Written By: Danielle Argiro
|          Date: Apr 08, 1992
| Modifications: 
|
-----------------------------------------------------------------*/   

xvobject xvf_create_formback(
   xvobject parent,
   double   minwidth,
   double   minheight,
   char    *name)
{
	xvobject back;

	back = xvw_create_manager(parent, name);
	xvw_set_attributes(back,
		XVW_CHAR_XSNAP,	      1.0,	      /* xchar snapto	    */
		XVW_CHAR_YSNAP,	      0.5,	      /* ychar snapto	    */
	        XVW_CHAR_WIDTH,       minwidth,
	        XVW_CHAR_HEIGHT,      minheight,
		XVW_CURSORNAME,      "left_ptr",
                XVW_MENUABLE,         TRUE,    
                XVW_SELECTABLE,       TRUE,    
		XVW_BUFFER_DIST,      5,
		NULL);

	xvw_insert_callback(back, XVW_GEOMETRY_CALLBACK, FALSE,
			    xvf_inform_geometry, NULL);
	return(back);
}

/*------------------------------------------------------------
|
|  Routine Name: xvf_create_selback()
|
|       Purpose: Creates a backplane for a selection.
|
| 	  Input: parent - parent of the backplane object
|		 x      - x location (in characters) of object
|		 y      - y location (in characters) of object
|		 name   - name to reference backplane object
|
|        Output: Returns the backpane object
|    Written By: Danielle Argiro
|          Date: Apr 08, 1992
| Modifications:
|
------------------------------------------------------------------*/

xvobject xvf_create_selback(
   xvobject parent,
   double  x,
   double  y,
   double  width,
   double  height,
   char   *name,
   int    dist)
{
	xvobject back;
	int      tacking = KMANAGER_TACK_NONE;
 
	back = xvw_create_manager(parent, name);

	if (width == -1.0)
	{
	    tacking |= KMANAGER_TACK_RIGHT;
	    width = 1.0;
	}

	if (height == -1.0)
	{
	    tacking |= KMANAGER_TACK_BOTTOM;
	    height = 1.0;
	}

	xvw_set_attributes(back, 
                XVW_BORDER_WIDTH,     0,              /* no border          */
                XVW_CHAR_XPOS,        x,	      /* set x dist         */
                XVW_CHAR_YPOS,        y,              /* set y dist         */
		XVW_CHAR_XSNAP,	      1.0,	      /* xchar snapto	    */
		XVW_CHAR_YSNAP,	      0.5,	      /* ychar snapto	    */
                XVW_RESIZABLE,        TRUE,           /* resizable          */
                XVW_CURSORNAME,      "left_ptr",      /* cursor             */
                XVW_DEF_VERT_DIST,    0,              /* dist bwn childs    */
                XVW_MENUABLE,         TRUE,           /* menuable           */
                XVW_SELECTABLE,       TRUE,           /* selectable         */
		XVW_BUFFER_DIST,      dist,	      /* buffer distance    */
		XVW_TACK_EDGE,        tacking,        /* tacking            */
		XVW_CHAR_WIDTH,       width,          /* width              */ 
		XVW_CHAR_HEIGHT,      height,         /* height             */ 
                NULL);

	xvw_insert_callback(back, XVW_GEOMETRY_CALLBACK, FALSE,
			    xvf_inform_geometry, NULL);
	return(back);
}

/*-------------------------------------------------------------
|
|  Routine Name: xvf_create_menubutton()
|
|       Purpose: Creates a menubutton object (with menu)
|
|	  Input: parent - parent of the backplane object
|                offset - object for menu to be right of (if applic).
|		 x      - x location (in characters) of object
|		 y      - y location (in characters) of object
|		 width  - width      (in characters) of object
|		 height - height     (in characters) of object
|		 label  - label to display on button object
|		 name   - name to reference backplane object
|        Output: Returns the button object
|    Written By: Danielle Argiro
|          Date: Apr 08, 1992
| Modifications: 
|
------------------------------------------------------------------*/

xvobject xvf_create_menubutton(
   xvobject parent,
   xvobject offset,
   double  x,
   double  y,
   double  width,
   double  height,
   char   *label,
   char   *name,
   char   **button_names,
   int    button_num,
   xvobject **menu_buttons)
{
	int      i;
	xvobject menu; 
	xvobject menubutton;

	/*
 	 *  first, create button which will bring up the menu.
	 *  using a copy of label is a memory leak, but must be
         *  left that way or menu button will not come up under Athena.
         *  set width, height, min height.
	 */
	if (kstrlen(label) > (int) width)
	    label[(int)width] = '\0';

	menubutton = xvw_create_menubutton(parent, name);
	if (parent != menubar)
	{
	   xvw_set_attributes(menubutton, 
		XVW_LABEL,            label,       /* set label      */
		XVW_CHAR_WIDTH,       width,       /* set width      */
		XVW_CHAR_HEIGHT,      height,      /* set height     */
		XVW_CHAR_MIN_HEIGHT,  1.0,         /* minimum height */
		NULL);
	}
	else xvw_set_attribute(menubutton, XVW_LABEL, label);

	/* if offset was provided, move menubutton to right of offset */
	if (offset != NULL)
	     xvw_set_attribute(menubutton, XVW_RIGHT_OF, offset);

	/* otherwise, use (x,y) position */
	else if (parent != menubar)
	{
	    xvw_set_attributes(menubutton, 
		XVW_CHAR_XPOS,      x,   	  /* set x dist */
		XVW_CHAR_YPOS,      y,            /* set y dist */
		NULL);
	}

	/* don't need to alloc array of menu buttons if there aren't any yet */
	if (button_num == 0) 
	    return(menubutton);

	/* allocate array of menu buttons for use within the pulldown menu */
	*menu_buttons = (xvobject *) kcalloc((size_t) 1, 
				button_num*sizeof(xvobject));
	if (*menu_buttons == NULL)
	{
	    kerror("xvforms", "xvf_create_menubutton",
		   "Unable to allocate array of buttons for display on menu");
	    return(NULL);
	}
	xvw_get_attribute(menubutton, XVW_MENUBUTTON_MENU, &menu);

	for (i = 0; i < button_num; i++)
	    (*menu_buttons)[i] = xvf_create_menu_button(menu, button_names[i], 
						        button_names[i]);

	return(menubutton);
}
			         
/*-------------------------------------------------------------
|
|  Routine Name: xvf_create_button()
|
|       Purpose: Creates a square or rectangular button object
|
|	  Input: parent - parent of the backplane object
|		 x      - x location (in characters) of object
|		 y      - y location (in characters) of object
|		 width  - width      (in characters) of object
|		 height - height     (in characters) of object
|		 label  - label to display on button object
|		 name   - name to reference backplane object
|        Output: Returns the button object
|    Written By: Danielle Argiro
|          Date: Apr 08, 1992
| Modifications: 
|
------------------------------------------------------------------*/

xvobject xvf_create_button(
   xvobject parent,
   double  x,
   double  y,
   double  width,
   double  height,
   char   *label,
   char   *name)
{
	xvobject button;
	char     *pixmap_filename;

	button = xvw_create_button(parent, name);

        pixmap_filename = xvf_get_pixmap_filename(label);
        if (pixmap_filename != NULL)
	    xvw_set_attribute(button, XVW_PIXMAP_FILENAME, pixmap_filename);
	else xvw_set_attribute(button, XVW_LABEL, label);

	xvw_set_attributes(button, 
		XVW_CHAR_MIN_HEIGHT,  1.0,      /* minimum height */
		XVW_CHAR_WIDTH,       width,    /* set width      */
		XVW_CHAR_HEIGHT,      height,   /* set height     */
		XVW_CHAR_XPOS,        x,        /* set x pos      */
		XVW_CHAR_YPOS,        y,        /* set y pos      */
		XVW_BUFFER_DIST,      0,
		NULL);

	return(button);
}

/*-------------------------------------------------------------
|
|  Routine Name: xvf_create_menu_button()
|
|       Purpose: Creates a button object for use on a menu.
|
|	  Input: parent - parent of the backplane object
|		 label  - label to display on button object
|		 name   - name to reference backplane object
|        Output: Returns the button object
|    Written By: Danielle Argiro
|          Date: Apr 08, 1992
| Modifications: 
|
------------------------------------------------------------------*/

xvobject xvf_create_menu_button(
   xvobject parent,
   char   *label,
   char   *name)
{
	xvobject button;

	button = xvw_create_button(parent, name);
	xvw_set_attributes(button, 
			   XVW_LABEL,         label,
			   XVW_LABEL_JUSTIFY, KLABEL_JUSTIFY_LEFT,
			   NULL);
	return(button);
}


/*-------------------------------------------------------------
|
|  Routine Name: xvf_create_optional_button()
|
|       Purpose: Creates a little square button object 
|		 used for indicating if selections are optional.  
|
|	  Input: parent - parent of the backplane object
|        Output: Returns the button object
|    Written By: Danielle Argiro
|          Date: Apr 08, 1992
| Modifications: 
|
------------------------------------------------------------------*/

xvobject xvf_create_optional_button(
   xvobject parent)
{
	xvobject  opt_button;

	opt_button = xvw_create_button(parent, "opt_button");
	xvw_set_attributes(opt_button,
			   XVW_LABEL,            "",  /* blank label      */
			   XVW_CHAR_WIDTH,       2.0,  /* char width       */
                           XVW_CHAR_HEIGHT,      1.0,  /* char height      */
			   XVW_CHAR_MIN_WIDTH,   2.0,  /* minimum width    */
                           XVW_CHAR_MIN_HEIGHT,  1.0,  /* minimum height   */
			   XVW_CHAR_MAX_WIDTH,   2.0,  /* maximum width    */
                           XVW_CHAR_MAX_HEIGHT,  1.0,  /* maximum height   */
			   XVW_YPOSITION,	 0,
			   NULL);
	return(opt_button);
}

/*-------------------------------------------------------------
|
|  Routine Name: xvf_create_boolean_button()
|
|       Purpose: Creates a round button object 
|
| 	  Input: parent - parent of the backplane object
|		 width  - width  (in characters) of object
|		 height - height (in characters) of object
|		 offset - object to left of boolean object
|		 title  - title of boolean object
|	         name   - name of boolean object
|        Output: Returns the button object
|    Written By: Danielle Argiro
|          Date: Apr 08, 1992
| Modifications: 
|
------------------------------------------------------------------*/

xvobject xvf_create_boolean_button(
   xvobject parent,
   double    width,
   double    height,
   xvobject offset,
   char     *title,
   char     *name)
{
	xvobject  button;
	char      *pixmap_filename;

	if (kstrlen(title) > (int) width)
	    title[(int)width] = '\0';

        if ((pixmap_filename = xvf_get_pixmap_filename(title)) == NULL)
        {
            button = xvw_create_button(parent, name);
            xvw_set_attribute(button, XVW_LABEL, title);
        }
        else
        {
            button = xvw_create_pixmap(parent, name);
            xvw_set_attribute(button, XVW_PIXMAP_FILENAME, pixmap_filename);
        }

	xvw_set_attributes(button, 
	         XVW_CHAR_WIDTH,       width,          /* set width        */
	         XVW_CHAR_HEIGHT,      height,         /* set height       */
		 XVW_CHAR_MIN_HEIGHT,  1.0,            /* minimum height   */
	         XVW_RIGHT_OF,         offset,         /* set offset       */
                 XVW_TACK_EDGE,        KMANAGER_TACK_ALL,/* grow vert & horiz */
		 XVW_BUFFER_DIST,      0,
	         NULL);

	return(button);
}

/*-------------------------------------------------------------
|
|  Routine Name: xvf_create_label()
|
|       Purpose: Creates a label object 
|
|	  Input: parent - parent of the backplane object
|		 x      - x offset (in characters) to label
|		 y      - y offset (in characters) to label
|		 offset - object to Left of label
|		 label  - label to display in label object
|		 name   - name to reference label object
|		 left_justify - flag TRUE if label is to be left justified
|				     FALSE if label is to be centered
|		 menuable - TRUE if user should be able to modify the 
|				label directly.
|        Output: Returns the label object
|    Written By: Danielle Argiro
|          Date: Apr 08, 1992
| Modifications: 
|
------------------------------------------------------------------*/

xvobject xvf_create_label(
   xvobject parent,
   double    x,
   double    y,
   xvobject offset,
   char     *label_string,
   char     *name,
   int      left_justify,
   int      selectable)
{
	xvobject  label;
	char      *pixmap_filename;

	if (label_string == NULL)
	    label_string = kstrdup("");

        if ((pixmap_filename = xvf_get_pixmap_filename(label_string)) == NULL)
        {
            label = xvw_create_label(parent, name);
            xvw_set_attribute(label, XVW_LABEL, label_string);
        }
        else
        {
            label = xvw_create_pixmap(parent, name);
            xvw_set_attribute(label, XVW_PIXMAP_FILENAME, pixmap_filename);
	    kfree(pixmap_filename);
        }

	/* create the label object */
	xvw_set_attributes(label, 
	         XVW_BORDER_WIDTH,     0,        /* no border        */
		 XVW_CHAR_MIN_HEIGHT,  1.0,      /* minimum height   */
	         NULL);

	/* if geometry is specified, set the position of the label using it */
	if ((x != 0.0) || (y != 0.0))
	{
	    xvw_set_attributes(label, 
	         XVW_CHAR_XPOS,        x,        /* set x dist       */
	         XVW_CHAR_YPOS,        y,        /* set y dist       */
		 NULL);
	}

	/* if offset is specified, set the position of the label using it */
	else if (offset != NULL)
	    xvw_set_attribute(label, XVW_RIGHT_OF, offset);

	if (selectable)
	    xvw_set_attribute(label, XVW_SELECTABLE, TRUE);

	return(label);
}

/*-------------------------------------------------------------
|
|  Routine Name: xvf_create_menu_label()
|
|       Purpose: Creates a label object for use on a menu.
|
|	  Input: parent - parent of the backplane object
|		 label  - label to display in label object
|		 name   - name to reference label object
|        Output: Returns the label object
|    Written By: Danielle Argiro
|          Date: Apr 08, 1992
* Modifications: 
|
------------------------------------------------------------------*/

xvobject xvf_create_menu_label(
   xvobject parent,
   char   *label,
   char   *name)
{
	xvobject  menu_label;

	if (kstrlen(label) > 0)
	{
	    menu_label = xvw_create_label(parent, name);
	    xvw_set_attribute(menu_label, XVW_LABEL, label);
	}
	else menu_label = xvw_create_separator(parent, name);

	return(menu_label);
}


/*-------------------------------------------------------------
|
|  Routine Name: xvf_create_pseudo_label()
|
|       Purpose: Creates a button object of type that is used both
|
|                to label an KUIS_INPUTFILE or KUIS_OUTPUTFILE selection and
|                to bring up the browser.
|	  Input: parent - parent of the backplane object
|		 width  - width      (in characters) of label
|		 height - height     (in characters) of label
|		 offset - object to Left of label
|		 label  - label to display in label object
|		 left_justify - TRUE if label is to be left justified
|				 FALSE if label is to be centered
|        Output: Returns the label object
|    Written By: Danielle Argiro
|          Date: Apr 08, 1992
| Modifications: 
|
------------------------------------------------------------------*/

xvobject xvf_create_pseudo_label(
   xvobject parent,
   double    width,
   double    height,
   xvobject offset,
   char     *label_string)
{
	xvobject  pseudo_label;
	char      *pixmap_filename;

	if (kstrlen(label_string) > (int) width)
	    label_string[(int)width] = '\0';

        pseudo_label = xvw_create_button(parent, "browser_button");
	pixmap_filename = xvf_get_pixmap_filename(label_string);
        if (pixmap_filename != NULL)
            xvw_set_attribute(pseudo_label, XVW_PIXMAP_FILENAME, 
			      pixmap_filename);
	else xvw_set_attributes(pseudo_label, 
			XVW_LABEL,       label_string,
			XVW_CHAR_WIDTH,  width,          /* set width      */
                        XVW_CHAR_HEIGHT, height,         /* set height     */
			NULL);

	xvw_set_attribute(pseudo_label, XVW_BUFFER_DIST, 0);

	/* if offset was provided, move pseudo label to right of offset */
	if (offset != NULL)
	     xvw_set_attribute(pseudo_label, XVW_RIGHT_OF, offset);

	/* otherwise, use (x,y) position of (0,0) */
	else xvw_set_attributes(pseudo_label, 
		XVW_XPOSITION,      0,   	  /* set x dist */
		XVW_YPOSITION,      0,            /* set y dist */
		NULL);
	   
	return(pseudo_label);
}

/*-------------------------------------------------------------
|
|  Routine Name: xvf_create_text()
|
|       Purpose: Creates a text object for entering strings and values.
|
|	  Input: parent      - parent of the text object
|		 width       - width  (in characters) of text object
|		 height      - height (in characters) of text object
|		 offset      - object to Left of text object
|		 def_string  - default string to display in text object
|		 spec        - flag TRUE for special case if called by
|			       xvf_create_string_sel w/ geometry height > 1.
|			       will set special attributes to scroll, etc.
|        Output: none
|       Returns: The text object
|    Written By: Danielle Argiro
|          Date: Apr 08, 1992
| Modifications: 
|
------------------------------------------------------------------*/

xvobject xvf_create_text(
   xvobject       parent,
   double          width,
   double          height,
   xvobject       offset,
   char           *def_string,
   int            spec)
{
	xvobject  text_obj;

	text_obj =  xvw_create_text(parent, "ascii");

	if ((height > 1.0) || (height == -1.0))
	    xvw_set_attributes(text_obj, 
			       XVW_TEXT_MULTILINE, TRUE,
			       XVW_TEXT_WRAP,      KTEXT_WRAP_NEVER,
			       NULL);
	else xvw_set_attribute(parent, XVW_CHAR_MAX_HEIGHT, 1.0);

	if (width < 2.0) width = 2.0;

	if (!kstrcmp(def_string," "))
		def_string = "";

	xvw_set_attributes(text_obj, 
		XVW_YPOSITION,       0,           /* y position              */
                XVW_CHAR_WIDTH,      width,       /* width                   */
		XVW_CHAR_MIN_WIDTH,  3.0,         /* minimum width           */
		XVW_CHAR_MIN_HEIGHT, 1.0,         /* minimum height          */
                XVW_RIGHT_OF,        offset,      /* offset                  */
                XVW_ABOVE,           NULL,        /* offset                  */
                XVW_BELOW,           NULL,        /* offset                  */
          	XVW_TACK_EDGE,	     KMANAGER_TACK_ALL,/* grow vert & horiz  */
                XVW_CURSORNAME,      "xterm",     /* cursor                  */
		XVW_SELECTABLE,       TRUE,
		XVW_TEXT_STRING,     def_string,  /* default string          */
		NULL);

#if 0
	if (spec)
	    xvw_set_attribute(text_obj, XVW_TEXT_SCROLL_VERT, 
			      KSCROLLBAR_IFNEEDED);
#endif

	return(text_obj);
}


/*-------------------------------------------------------------
|
|  Routine Name: xvf_create_cr_pixmap()
|
|       Purpose: Creates a small, square label object containing 
|                a pixmap that represents a carriage return.
|
|   	  Input: parent - parent of the backplane object
|		 y      - y location (in characters) of object
|		 offset - object to Left of pixmap object
|        Output: Returns the pixmap object
|    Written By: Danielle Argiro
|          Date: Apr 08, 1992
| Modifications: 
|
------------------------------------------------------------------*/

#define LIVE_PIXMAP "$DESIGN/objects/library/xvforms/pixmaps/lightning.xpm"

xvobject xvf_create_cr_pixmap(
   xvobject parent)
{
	xvobject    pixmap_object;

	pixmap_object = xvw_create_pixmap(parent, "LivePixmap");
	xvw_set_attributes(pixmap_object, 
                XVW_BORDER_WIDTH,      0,      
                XVW_LEFT_OF,           NULL,	
                XVW_BELOW,	       NULL,	 
                XVW_PIXMAP_FILENAME,   LIVE_PIXMAP,
		NULL);
	return(pixmap_object);
}


/*-------------------------------------------------------------
|
|  Routine Name:  xvf_create_scroll()
|
|       Purpose: Creates a scroll bar object.
|
|	  Input: parent - parent of the backplane object
|		 offset - object to Left of scroll bar
|		 name   - name to reference scroll object by 
|
|        Output: Returns the pixmap object
|    Written By: Danielle Argiro
|          Date: Apr 08, 1992
| Modifications: 
|
------------------------------------------------------------------*/

xvobject  xvf_create_scroll(
   xvobject parent,
   xvobject offset,
   char     *name)
{
	xvobject  scroll;

	scroll = xvw_create_scrollbar(parent, name);
	xvw_set_attributes(scroll, 
		XVW_ORIENTATION,      KSCROLLBAR_ORIENT_HORIZ,   /* horiz    */
                XVW_CHAR_MIN_HEIGHT,  1.0,                /* min height      */
                XVW_CHAR_MAX_HEIGHT,  1.0,                /* min height      */
                XVW_YPOSITION,        0,	          /* set y pos       */
                XVW_RIGHT_OF,         offset,	          /* set offset      */
                XVW_TACK_EDGE,        KMANAGER_TACK_HORIZ,/* grow horiz.     */
		NULL);
	return(scroll);
}

/*-------------------------------------------------------------
|
|  Routine Name: xvf_get_pixmap_filename()
|
|       Purpose: The forms support specification of a pixmap
|                for use in a label or button with the syntax
|                pixmap: {fullpath to pixmap}.
|                This utility routine sees if that syntax is relevant,
|                and returns the fullpath to the pixmap if it is.
|
|         Input: label_string - string which may have the syntax
|             "pixmap: {fullpath to pixmap}", eg
|             "pixmap: $ENVISION/objects/xvroutine/editimage/misc/editimage.xpm"
|                or may be simply a label, such as "Options Menu".
|
|        Output: Returns the fullpath to pixmap if syntax is followed,
|                NULL otherwise
|    Written By: Danielle Argiro
|          Date: Apr 08, 1996
| Modifications:
|
------------------------------------------------------------------*/

char *xvf_get_pixmap_filename(
    char *label_string)
{
	char filename[KLENGTH];
	
	if (kstrncmp(label_string, "pixmap:", (size_t) 7) != 0)
	    return(NULL);

	if (ksscanf(label_string, "pixmap:%s", filename) == 1)
	{
#if 1
	    if (kaccess(filename, R_OK) == 0)
	        return(kstrdup(filename));
	    else 
	    {
	        kerror("xvforms", "get_pixmap_filename",
		       "You have an entry, '%s', that is attempting to specify "
		       "a pixmap, but the file does not exist", filename);
	        return(NULL);
	    }
#else
	    return(kstrdup(filename));
#endif
	}
	else return(NULL);
}
