/*
 * Khoros: $Id: draw_2D.c,v 1.15.2.1 1997/05/24 19:02:58 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: draw_2D.c,v 1.15.2.1 1997/05/24 19:02:58 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		2D Drawing  Utilities
   >>>>		
   >>>>  Private:
   >>>>                 X2D_draw_polyline()
   >>>>                 X2D_draw_discrete()
   >>>>                 X2D_draw_bargraph()
   >>>>                 X2D_draw_histogram()
   >>>>                 X2D_draw_polymarker()
   >>>>                 X2D_draw_linemarker()
   >>>>                 X2D_draw_segments()
   >>>>                 X2D_draw_line()
   >>>>                 X2D_draw_marker()
   >>>>                 X2D_draw_rectangle()
   >>>>                 X2D_draw_circle()
   >>>>                 X2D_draw_ellipse()
   >>>>                 X2D_fill_rectangle()
   >>>>                 X2D_fill_circle()
   >>>>                 X2D_fill_ellipse()
   >>>>                 X2D_fill_polyline()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "graphics.h"


/*-----------------------------------------------------------
|
|  Routine Name: X2D_draw_polyline
|
|       Purpose: Draws a polygon at the world coordinate location specified
|
|         Input: id	- xvgraphics structure ID
|		 coords	- array of world coordinate points
|		 size	- number of points
|		 fg	- color of polyline
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X2D_draw_polyline (
   XVGraphicsID   id,
   Coord          *coords,
   int            size,
   XColor         *fg)
{
	X3DGraphics	*graphics;
	Coord		*ncoords;
	XSegment	*segments;
	int		i, nsize, seg_num;
	Colormap        colormap;

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_draw_polyline", "Unknown graphics id %d\n", id);
	   return;
	}

	if ((ncoords = _X2D_clip_lines(graphics, coords, size, &nsize)) == NULL)
	   return;

	seg_num = nsize/2;
	if ((segments = (XSegment *) kmalloc ((size_t) seg_num * 
			sizeof (XSegment))) == NULL)
	{
	   kerror (NULL, "X2D_draw_polyline", "Not enough memory\n");
	   return;
	}

	if (!(_X2D_convert_wc_to_dc_seg(graphics, ncoords, segments, nsize)))
	{
	   kfree_and_NULL(segments);
	   return;
	}

	colormap = X3D_get_colormap(graphics->id);

	for (i = 0; i < seg_num; i++)
	{
	    _X3D_set_fgcolor(graphics, colormap, ncoords[i*2].d, fg);
	    X3D_draw[graphics->device].segments(graphics, &segments[i], 1);
	}

	if (coords != ncoords)
	   kfree_and_NULL(ncoords);
	kfree_and_NULL(segments);
}

/*-----------------------------------------------------------
|
|  Routine Name: X2D_draw_discrete
|
|       Purpose: Draws a discrete plot
|
|         Input: id	- xvgraphics structure ID
|		 coords	- array of world coordinate points
|		 size	- number of points
|		 fg	- color of discrete plot
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X2D_draw_discrete (
   XVGraphicsID id,
   Coord        *coords,
   int          size,
   XColor       *fg)
{
	X3DGraphics	*graphics;
	XPoint		*points;

	int		baseline;
	Real		xmin, ymin;
	int		i, nsize, clip_code;
	Coord		zero_coord, *ncoords;
	Colormap        colormap;

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_draw_discrete", "Unknown graphics id %d\n", id);
	   return;
	}

	if (graphics->wc_min.y < 0  && graphics->wc_max.y > 0)
	{
	   zero_coord.x = 0.0;
	   zero_coord.y = 0.0;
	   if (!(X2D_convert_point_wc_to_dc(id, zero_coord, &xmin, &ymin)))
	      return;
	}
	else
	{
	   if (!(X2D_convert_point_wc_to_dc(id, graphics->wc_min, 
					    &xmin, &ymin)))
	      return;
	}
	baseline = (ymin + 0.5);

	clip_code = X2D_TOP | X2D_BOTTOM;
	if ((ncoords = _X2D_clip_points(graphics, coords, size, 
				        clip_code, &nsize)) == NULL)
	{
	   return;
	}

	if ((points = (XPoint *) kmalloc ((size_t) nsize * sizeof(XPoint)))
		    == NULL)
	{
	   kerror (NULL, "X2D_draw_discrete", "Not enough memory\n");
	   return;
	}

	if (!(_X2D_convert_wc_to_dc(graphics, ncoords, points, nsize)))
	{
	   kfree_and_NULL(points);
	   return;
	}

	colormap = X3D_get_colormap(graphics->id);
	for (i = 0; i < nsize; i++)
	{
	    _X3D_set_fgcolor(graphics, colormap, ncoords[i].d, fg);
	    X3D_draw[graphics->device].line(graphics, points[i].x,
			baseline, points[i].x, points[i].y);
	}
	if (coords != ncoords)
	   kfree_and_NULL(ncoords);
        kfree_and_NULL(points);
}

/*-----------------------------------------------------------
|
|  Routine Name: X2D_draw_bargraph
|
|       Purpose: Draws a bar graph
|
|         Input: id	- xvgraphics structure ID
|		 coords	- array of world coordinate points
|		 size	- number of points
|		 fg	- color of bargraph
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X2D_draw_bargraph (
   XVGraphicsID   id,
   Coord  *coords,
   int    size,
   XColor *fg)
{
	X3DGraphics	*graphics;
	int		nsize;
	XPoint		*points;
	Coord		*ncoords;

	int		i, x, y;
	int		bottom, left, right;
	Real            xmin, xmax, ymin, ymax;
	unsigned int	width, height;
	Colormap        colormap;

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_draw_bargraph", "Unknown graphics id %d\n", id);
	   return;
	}
	colormap = X3D_get_colormap(graphics->id);

	if(!(X2D_convert_point_wc_to_dc (id, graphics->wc_min, &xmin, &ymin)))
	   return;

	if(!(X2D_convert_point_wc_to_dc (id, graphics->wc_max, &xmax, &ymax)))
	   return;

	left   = xmin + 0.5;
	right  = xmax + 0.5;
	bottom = ymin + 0.5;

	if ((ncoords = _X2D_clip_rectangles(graphics, coords, size, &nsize))
		== NULL)
	   return;

	if ((points = (XPoint *) kmalloc ((size_t) nsize * sizeof(XPoint)))
		== NULL)
	{
	   kerror (NULL, "X2D_draw_bargraph", "Not enough memory\n");
	   return;
	}

	if(!(_X2D_convert_wc_to_dc(graphics, ncoords, points, nsize)))
	{
	   kfree_and_NULL(points);
	   return;
	}

	/*
	 *  Initialize the width and the first x position.
	 */
	if (nsize == 1)
	   width = kabs(right-left)/2;
	else width = kabs(points[0].x - points[1].x);

	x = points[0].x - (width/2.0 + 0.5);
	for (i = 0; i < nsize; i++)
	{
	    _X3D_set_fgcolor(graphics, colormap, ncoords[i].d, fg);
	   if (i+1 != nsize)
	      width = kabs(points[i].x - points[i+1].x);

	   y = points[i].y;
	   height = kabs(bottom - y);
	   X3D_draw[graphics->device].rectangle(graphics, x, y, width, height);
	   x += width;
	}

	if (coords != ncoords)
	   kfree_and_NULL(ncoords);
        kfree_and_NULL(points);
}

/*-----------------------------------------------------------
|
|  Routine Name: X2D_draw_histogram
|
|       Purpose: Draws a histogram
|
|         Input: id	- xvgraphics structure ID
|		 coords	- array of world coordinate points
|		 size	- number of points
|		 wcmin	- world coordinate minimum (x,y,z)
|		 wcmax	- world coordinate maximum (x,y,z)
|		 num	- NOT USED
|		 fg	- color of histogram
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void X2D_draw_histogram (
   XVGraphicsID id,
   Coord        *coords,
   int          size,
   Coord        wcmin,
   Coord        wcmax,
   int          num,
   XColor       *fg)
{
	X3DGraphics	*graphics;
	int		nsize;
	XPoint		*points;
	Coord		*ncoords;

	int		i, x, y;
	int		baseline;
	Real		xmin, ymin;
	unsigned int	width, height;


	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_draw_histogram", "Unknown graphics id %d\n", id);
	   return;
	}

	if(!(X2D_convert_point_wc_to_dc (id, graphics->wc_min, &xmin, &ymin)))
	   return;

	baseline = ymin;

	if ((ncoords = _X2D_clip_rectangles(graphics, coords, size, &nsize))
		== NULL)
	   return;

	if ((points = (XPoint *) kmalloc ((size_t) nsize * sizeof (XPoint)))
		== NULL)
	{
	   kerror (NULL, "X2D_draw_histogram", "Not enough memory\n");
	   return;
	}

	if(!(_X2D_convert_wc_to_dc(graphics, ncoords, points, nsize)))
	{
	   kfree_and_NULL(points);
	   return;
	}

	X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);

	for (i = 1; i <= nsize; i++)
	{
	    width = kabs((points[i].x - points[i+1].x));
	    x = points[i].x;
	    y = points[i].y;
	    height = kabs(baseline - y);
	    X3D_draw[graphics->device].rectangle(graphics, x, y, width, height);
	}

	if (coords != ncoords)
	   kfree_and_NULL(ncoords);
        kfree_and_NULL(points);
}

/*-----------------------------------------------------------
|
|  Routine Name: X2D_draw_polymarker
|
|       Purpose: Draws a polymarker plot
|
|         Input: id	      - xvgraphics structure ID
|		 coords	      - array of world coordinate points
|		 size	      - number of points
|		 marker_scale - amount by which to scale markers
|		 marker_type  - type of marker to display
|		 fg	      - color of polymarker
|
|        Output: none
|
|       Returns: nothing
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X2D_draw_polymarker (
   XVGraphicsID id,
   Coord        *coords,
   int          size,
   int          marker_scale,
   int          marker_type,
   XColor       *fg)
{
	X3DGraphics	*graphics;
	int             nsize, clip_code;
	XPoint          *points;
	Coord           *ncoords;

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_draw_polymarker", "Unknown graphics id %d\n", id);
	   return;
	}
        clip_code = X2D_CENTER;
        if ((ncoords = _X2D_clip_points(graphics, coords, size, clip_code,
                                        &nsize)) == NULL)
        {
           return;
        }

        if ((points = (XPoint *) kmalloc ((size_t) nsize * sizeof (XPoint)))
                == NULL)
        {
           kerror (NULL, "X2D_draw_polymarker", "Not enough memory\n");
           return;
        }

        if (!(_X2D_convert_wc_to_dc(graphics, ncoords, points, nsize)))
        {
           kfree_and_NULL(points);
           return;
        }
	_X2D_draw_markers(id, ncoords, points, nsize, 
		          marker_scale, marker_type, fg);

        if (coords != ncoords)
           kfree_and_NULL(ncoords);
        kfree_and_NULL(points);
}

/*-----------------------------------------------------------
|
|  Routine Name: X2D_draw_linemarker
|
|       Purpose: Draws a linemarker plot
|
|         Input: id	      - xvgraphics structure ID
|		 coords	      - array of world coordinate points
|		 size	      - number of points
|		 marker_scale - amount by which to scale markers
|		 marker_type  - type of marker to display
|		 fg	      - color of linemarker
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X2D_draw_linemarker (
   XVGraphicsID id,
   Coord        *coords,
   int          size,
   int          marker_scale,
   int          marker_type,
   XColor       *fg)
{
	if (_X3D_get_graphics(id) == NULL)
	{
	   kerror (NULL, "X2D_draw_linemarker", "Unknown graphics id %d\n", id);
	   return;
        }
	X2D_draw_polymarker (id, coords, size, marker_scale, marker_type, fg);
	X2D_draw_polyline (id, coords, size, fg);
}

/*-----------------------------------------------------------
|
|  Routine Name: X2D_draw_segments
|
|       Purpose: Draws a series of line segments
|
|         Input: id	- xvgraphics structure ID
|		 coords	- array of world coordinate points
|		 size	- number of points
|		 fg	- color of segment
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X2D_draw_segments (
   XVGraphicsID id,
   Coord        *coords,
   int          size,
   XColor       *fg)
{
	X3DGraphics	*graphics;
	Coord		*ncoords;
	XSegment	*segments;
	int		seg_num, nsize;


	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_draw_segments", "Unknown graphics id %d\n", id);
	   return;
	}

	if ((ncoords = _X2D_clip_segments(graphics, coords, size, &nsize))
		== NULL)
	   return;

	seg_num = nsize/2;
	if ((segments = (XSegment *) kmalloc ((size_t) seg_num * 
			sizeof (XSegment))) == NULL)
	{
	   kerror (NULL, "X2D_draw_segments", "Not enough memory\n");
	   return;
	}

	if (!(_X2D_convert_wc_to_dc_seg(graphics, ncoords, segments, nsize)))
	{
	   kfree_and_NULL(segments);
	   return;
	}

	X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);
	X3D_draw[graphics->device].segments(graphics, segments, seg_num);

	if (coords != ncoords)
	   kfree_and_NULL(ncoords);
        kfree_and_NULL(segments);
}

/*-----------------------------------------------------------
|
|  Routine Name: X2D_draw_line
|
|       Purpose: Draws a line from the first coordinate to the second
|
|         Input: id     - xvgraphics structure ID
|		 coord1 - the beginning world coordinate point
|		 coord2 - the end world coordinate point
|		 fg	- color of line
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X2D_draw_line (
   XVGraphicsID id,
   Coord        coord1,
   Coord        coord2,
   XColor       *fg)
{
	X3DGraphics	*graphics;
	int		nsize;
	XPoint		points[2];
	Coord		coords[2], *ncoords;

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_draw_line", "Unknown graphics id %d\n", id);
	   return;
	}

	coords[0] = coord1;
	coords[1] = coord2;
	if ((ncoords = _X2D_clip_lines(graphics, coords, 2, &nsize)) == NULL)
	   return;

	if (!(_X2D_convert_wc_to_dc(graphics, ncoords, points, nsize)))
	   return;

	X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);
	X3D_draw[graphics->device].line(graphics, points[0].x,
		points[0].y, points[1].x, points[1].y);

	if (coords != ncoords)
	   kfree_and_NULL(ncoords);
}

/*-----------------------------------------------------------
|
|  Routine Name: X2D_draw_marker
|
|       Purpose: Draws a marker
|
|         Input: id	      - xvgraphics structure ID
|		 coord	      - single coordinate marker point
|		 marker_scale - amount by which to scale marker
|		 marker_type  - type of marker to display
|		 fg	      - color of marker
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X2D_draw_marker (
   XVGraphicsID id,
   Coord        coord,
   int          marker_scale,
   int          marker_type,
   XColor       *fg)
{
	X3DGraphics	*graphics;
	int             nsize, clip_code;
	XPoint          *points;
	Coord           *ncoords;

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_draw_marker", "Unknown graphics id %d\n", id);
	   return;
	}

        clip_code = X2D_CENTER;
        if ((ncoords = _X2D_clip_points(graphics, &coord, 1, clip_code,
                                        &nsize)) == NULL)
        {
           return;
        }

        if ((points = (XPoint *) kmalloc ((size_t) nsize * sizeof (XPoint)))
                == NULL)
        {
           kerror (NULL, "X2D_draw_polymarker", "Not enough memory\n");
           return;
        }

        if (!(_X2D_convert_wc_to_dc(graphics, ncoords, points, nsize)))
        {
           kfree_and_NULL(points);
           return;
        }
	_X2D_draw_markers(id, ncoords, points, 1, 
		          marker_scale, marker_type, fg);

        kfree_and_NULL(points);
}

/*-----------------------------------------------------------
|
|  Routine Name: X2D_draw_rectangle
|
|       Purpose: Draws a rectangle at the world coordinate location specified
|
|         Input: id	- xvgraphics structure ID
|		 coord	- the world coordinate upper left edge of rectangle 
|		 width	- the world coordinate width of rectangle
|		 height	- the world coordinate height of rectangle
|		 fg	- color of rectangle
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X2D_draw_rectangle (
   XVGraphicsID   id,
   Coord  coord,
   Real   width,
   Real   height,
   XColor *fg)
{
	X3DGraphics	*graphics;
	XPoint		points[2];
	Coord		coords[2];
	int		x, y;
	unsigned int	w, h;


	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_draw_rectangle", "Unknown graphics id %d\n", id);
	   return;
	}
	coords[0] = coord;
	coords[1].x = coord.x + width;
	coords[1].y = coord.y + height;

	if (!(_X2D_convert_wc_to_dc(graphics, coords, points, 2)))
	   return;

	x = kmin(points[0].x, points[1].x);
	y = kmin(points[0].y, points[1].y);
	w = kabs(points[1].x - points[0].x);
	h = kabs(points[1].y - points[0].y);
	X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);
	X3D_draw[graphics->device].rectangle(graphics, x, y, w, h);
}

/*-----------------------------------------------------------
|
|  Routine Name: X2D_draw_circle
|
|       Purpose: Draws a circle at the world coordinate location specified
|
|         Input: id	- xvgraphics structure ID
|		 coord	- the world coordinate center of the circle
|		 radius - the world coordinate radius of the circle
|		 fg	- color of circle
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X2D_draw_circle (
   XVGraphicsID   id,
   Coord  coord,
   Real   radius,
   XColor *fg)
{
	X3DGraphics	*graphics;
	XPoint		points[2];
	Coord		coords[2];

	int		x, y;
	unsigned int	w, h;


	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_draw_circle", "Unknown graphics id %d\n", id);
	   return;
	}

	coords[0].x = coord.x - radius;
	coords[0].y = coord.y - radius;
	coords[1].x = coord.x + radius;
	coords[1].y = coord.y + radius;

	if (!(_X2D_convert_wc_to_dc(graphics, coords, points, 2)))
	   return;

	if ( points[0].x < points[1].x )
	{
	   x = (points[1].x - points[0].x)/2.0 + points[0].x;
	}
	else
	{
	   x = (points[0].x - points[1].x)/2.0 + points[1].x;
	}

	if ( points[0].y < points[1].y )
	{
	   y = (points[1].y - points[0].y)/2.0 + points[0].y;
	}
	else
	{
	   y = (points[0].y - points[1].y)/2.0 + points[1].y;
	}
	w = kabs(points[1].x - points[0].x);
	h = kabs(points[1].y - points[0].y);
	X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);
	X3D_draw[graphics->device].arc(graphics, x, y, w, h, 0, 360);

}

/*-----------------------------------------------------------
|
|  Routine Name: X2D_draw_ellipse
|
|       Purpose: Draws an ellipse at the world coordinate location specified
|
|         Input: id	- xvgraphics structure ID
|		 coord	- the world coordinate center of the ellipse
|		 width  - the world coordinate width of the ellipse
|		 height - the world coordinate height of the ellipse
|		 fg	- color of ellipse
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: January 8, 1996
| Modifications:
|
------------------------------------------------------------*/

void X2D_draw_ellipse (
   XVGraphicsID   id,
   Coord  coord,
   Real   width,
   Real   height,
   XColor *fg)
{
	X3DGraphics	*graphics;
	XPoint		points[2];
	Coord		coords[2];
	int		x, y;
	unsigned int	w, h;


	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_draw_ellipse", "Unknown graphics id %d\n", id);
	   return;
	}

	coords[0].x = coord.x - width/2.0;
	coords[0].y = coord.y - height/2.0;
	coords[1].x = coord.x + width/2.0;
	coords[1].y = coord.y + height/2.0;

	if (!(_X2D_convert_wc_to_dc(graphics, coords, points, 2)))
	   return;

	if ( points[0].x < points[1].x )
	{
	   x = (points[1].x - points[0].x)/2.0 + points[0].x;
	}
	else
	{
	   x = (points[0].x - points[1].x)/2.0 + points[1].x;
	}

	if ( points[0].y < points[1].y )
	{
	   y = (points[1].y - points[0].y)/2.0 + points[0].y;
	}
	else
	{
	   y = (points[0].y - points[1].y)/2.0 + points[1].y;
	}
	w = kabs(points[1].x - points[0].x);
	h = kabs(points[1].y - points[0].y);
	X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);
	X3D_draw[graphics->device].arc(graphics, x, y, w, h, 0, 360);
}

/*-----------------------------------------------------------
|
|  Routine Name: X2D_fill_rectangle
|
|       Purpose: Draws a filled rectangle at the world coordinate 
|		 location specified
|
|         Input: id	- xvgraphics structure ID
|		 coord	- the beginning world coordinate point
|		 width	- the world coordinate width of rectangle
|		 height	- the world coordinate height of rectangle
|		 fg	- color of rectangle
|		 bg	- fill color of rectangle
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X2D_fill_rectangle(
   XVGraphicsID  id,
   Coord         coord,
   Real          width,
   Real          height,
   XColor        *fg,
   XColor        *bg)
{
	X3DGraphics	*graphics;
	XPoint		points[2];
	Coord		coords[2];
	int		x, y;
	unsigned int	w, h;


	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_fill_rectangle", "Unknown graphics id %d\n", id);
	   return;
	}

	coords[0]   = coord;
	coords[1].x = coord.x + width;
	coords[1].y = coord.y + height;

	if (!(_X2D_convert_wc_to_dc(graphics, coords, points, 2)))
	   return;

	x = kmin(points[0].x, points[1].x);
	y = kmin(points[0].y, points[1].y);
	w = kabs(points[1].x - points[0].x);
	h = kabs(points[1].y - points[0].y);
	X3D_draw[graphics->device].fill_color(graphics, bg, FALSE);
	X3D_draw[graphics->device].fill_rectangle(graphics, x, y, w, h);

	if (graphics->line_width != KLINE_NONE)
	{
	   X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);
	   X3D_draw[graphics->device].rectangle(graphics, x, y, w, h);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: X2D_fill_circle
|
|       Purpose: Draws a filled circle at the world coordinate
|                location specified
|
|	  Input: id	- xvgraphics structure ID
|		 coord	- the world coordinate center of the circle
|		 radius	- the world coordinate radius of the circle
|		 fg	- color of circle
|		 bg	- fill color of circle
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X2D_fill_circle(
   XVGraphicsID id,
   Coord        coord,
   Real         radius,
   XColor       *fg,
   XColor       *bg)
{
	X3DGraphics	*graphics;
	XPoint		points[2];
	Coord		coords[2];
	int		x, y;
	unsigned int	w, h;


	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_fill_circle", "Unknown graphics id %d\n", id);
	   return;
	}

	coords[0].x = coord.x - radius;
	coords[0].y = coord.y - radius;
	coords[1].x = coord.x + radius;
	coords[1].y = coord.y + radius;

	if (!(_X2D_convert_wc_to_dc(graphics, coords, points, 2)))
	   return;

	if ( points[0].x < points[1].x )
	{
	   x = (points[1].x - points[0].x)/2.0 + points[0].x;
	}
	else
	{
	   x = (points[0].x - points[1].x)/2.0 + points[1].x;
	}

	if ( points[0].y < points[1].y )
	{
	   y = (points[1].y - points[0].y)/2.0 + points[0].y;
	}
	else
	{
	   y = (points[0].y - points[1].y)/2.0 + points[1].y;
	}
	w = kabs(points[1].x - points[0].x);
	h = kabs(points[1].y - points[0].y);

	X3D_draw[graphics->device].fill_color(graphics, bg, FALSE);
	X3D_draw[graphics->device].fill_arc(graphics, x, y, w, h, 0, 360);
	if (graphics->line_width != KLINE_NONE)
	{
	   X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);
	   X3D_draw[graphics->device].arc(graphics, x, y, w, h, 0, 360);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: X2D_fill_ellipse
|
|       Purpose: Draws a filled ellipse at the world coordinate
|                location specified
|
|	  Input: id	- xvgraphics structure ID
|		 coord	- the world coordinate center of the ellipse
|		 width	- the world coordinate width of the ellipse
|		 height	- the world coordinate height of the ellipse
|		 fg	- color of ellipse
|		 bg	- fill color of ellipse
|
|    Written By: Danielle Argiro  & Steve Kubica
|          Date: January 8, 1996
| Modifications:
|
------------------------------------------------------------*/

void X2D_fill_ellipse(
   XVGraphicsID id,
   Coord        coord,
   Real         width,
   Real         height,
   XColor       *fg,
   XColor       *bg)
{
	X3DGraphics	*graphics;
	XPoint		points[2];
	Coord		coords[2];
	int		x, y;
	unsigned int	w, h;


	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_fill_ellipse", "Unknown graphics id %d\n", id);
	   return;
	}

	coords[0].x = coord.x - width/2.0;
	coords[0].y = coord.y - height/2.0;
	coords[1].x = coord.x + width/2.0;
	coords[1].y = coord.y + height/2.0;

	if (!(_X2D_convert_wc_to_dc(graphics, coords, points, 2)))
	   return;

	if ( points[0].x < points[1].x )
	{
	   x = (points[1].x - points[0].x)/2.0 + points[0].x;
	}
	else
	{
	   x = (points[0].x - points[1].x)/2.0 + points[1].x;
	}

	if ( points[0].y < points[1].y )
	{
	   y = (points[1].y - points[0].y)/2.0 + points[0].y;
	}
	else
	{
	   y = (points[0].y - points[1].y)/2.0 + points[1].y;
	}
	w = kabs(points[1].x - points[0].x);
	h = kabs(points[1].y - points[0].y);

	X3D_draw[graphics->device].fill_color(graphics, bg, FALSE);
	X3D_draw[graphics->device].fill_arc(graphics, x, y, w, h, 0, 360);
	if (graphics->line_width != KLINE_NONE)
	{
	   X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);
	   X3D_draw[graphics->device].arc(graphics, x, y, w, h, 0, 360);
	}
}


/*-----------------------------------------------------------
|
|  Routine Name: X2D_fill_polyline
|
|       Purpose: Draws a filled polyline shape
|
|         Input: id	- xvgraphics structure ID
|		 coords	- array of world coordinate points
|		 size	- number of points
|		 fg	- color of polygon
|		 bg	- color of polygon
|
|    Written By: Mark Young, Danielle Argiro, Steve Kubica
|          Date: Feb 13, 1996
|
------------------------------------------------------------*/

void X2D_fill_polyline (
   XVGraphicsID id,
   Coord        *coords,
   int          size,
   XColor       *fg,
   XColor       *bg)
{
	X3DGraphics	*graphics;
	int		nsize;
	XPoint		*points;
	Coord		*ncoords;
	Colormap        colormap;

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X2D_draw_polygon", "Unknown graphics id %d\n", id);
	   return;
	}

	if ((ncoords = _X2D_clip_polygon(graphics, coords, size, &nsize))
		== NULL)
	   return;

	if ((points = (XPoint *) kmalloc ((unsigned) (nsize+1) * sizeof (XPoint)))
		== NULL)
	{
	   kerror (NULL, "X2D_draw_polygon", "Not enough memory\n");
	   return;
	}

	if (!(_X2D_convert_wc_to_dc(graphics, ncoords, points, nsize)))
	{
	   kfree_and_NULL(points);
	   return;
	}

	points[nsize] = points[0];

        colormap = X3D_get_colormap(graphics->id);

	X3D_draw[graphics->device].fill_color(graphics, bg, FALSE);
	X3D_draw[graphics->device].fill_polygon(graphics, points, nsize,
			Convex, CoordModeOrigin);

	_X3D_set_fgcolor(graphics, colormap, 
			 (Index) KGRAPHICS_UNINITIALIZED, fg);
	X2D_draw_polyline(id, coords, size, fg);

	if (coords != ncoords)
	   kfree_and_NULL(ncoords);
        kfree_and_NULL(points);
}

