/*
 * Khoros: $Id: draw_3D.c,v 1.15 1997/01/20 05:18:45 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: draw_3D.c,v 1.15 1997/01/20 05:18:45 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		3D Drawing Utilities
   >>>>		
   >>>>  Private:
   >>>>                 X3D_draw_polyline()
   >>>>                 X3D_draw_lineplot()
   >>>>                 X3D_draw_wireframe()
   >>>>                 X3D_draw_impulse()
   >>>>                 X3D_draw_polymarker()
   >>>>                 X3D_draw_colormarker()
   >>>>                 X3D_draw_polygon()
   >>>>                 X3D_draw_segments()
   >>>>                 X3D_draw_line()
   >>>>                 X3D_draw_rectangle()
   >>>>                 X3D_draw_mesh()
   >>>>                 X3D_draw_horizon()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "graphics.h"


/*-----------------------------------------------------------
|
|  Routine Name: X3D_draw_polyline
|
|       Purpose: Draws a 3D polyline plot
|
|         Input: id     - X3D graphics structure ID
|                coords - array of world coordinate points
|                size   - number of points
|                fg     - color of polyline
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X3D_draw_polyline (
   XVGraphicsID id,
   Coord        *coords,
   int          size,
   XColor       *fg)
{
	X3DGraphics	*graphics;
	Coord		*ncoords;
	XSegment	*segments;
	int		i, nsize, seg_num;
	Colormap        colormap;

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X3D_draw_polyline", "Unknown graphics id %d\n", id);
	   return;
	}

	if ((ncoords = _X3D_clip_lines(graphics, coords, size, NULL, &nsize))
		== NULL)
	   return;

	seg_num = nsize/2;
        if ((segments = (XSegment *) kmalloc ((size_t) nsize *
                         sizeof(XSegment))) == NULL)
        {
	   kerror (NULL, "X3D_draw_polyline", "Not enough memory\n");
           return;
        }
	if (!(_X3D_convert_wc_to_dc_seg(graphics, graphics->matrix2, ncoords,
			                segments, nsize)))
	{
           kfree_and_NULL(segments);
	   return;
	}

	colormap = X3D_get_colormap(graphics->id);

        for (i = 0; i < seg_num; i++)
        {
            _X3D_set_fgcolor(graphics, colormap, ncoords[i*2].d, fg);
            X3D_draw[graphics->device].segments(graphics, &segments[i], 1);
        }

	if (ncoords != coords) 
	    kfree_and_NULL(ncoords);
        kfree_and_NULL(segments);
}

/*-----------------------------------------------------------
|
|  Routine Name: X3D_draw_lineplot
|
|       Purpose: Draws a 3D line plot
|
|         Input: id       - X3D graphics structure ID
|		 coords	  - array of world coordinate points
|		 row_size - number of points in each polyline
|		 size	  - number of points
|		 fg	  - color of lines
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X3D_draw_lineplot(
   XVGraphicsID id,
   Coord        *coords,
   int          row_size,
   int          size,
   XColor       *fg)
{
	X3DGraphics	*graphics;
	XSegment	*segments;
	Coord		*ncoords;
	int		i, j, k; 
	int             col_size, nsize, seg_num;
	Colormap        colormap;

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X3D_draw_lineplot", "Unknown graphics id %d\n", id);
	   return;
	}

        /*
	 *  see how many segments are to be plotted.  If there is only one
	 *  segment, then plot the whole thing.
	 */
	if (row_size == 1)
	   row_size = size; 

	col_size = size/row_size;

	if ((ncoords = (Coord *) kmalloc((size_t) row_size*2*sizeof(Coord)))
		== NULL)
	{
	   kerror (NULL, "X3D_draw_lineplot", "Not enough memory\n");
	   return;
	}

	if ((segments = (XSegment *) kmalloc(row_size * sizeof(XSegment)))
		== NULL)
	{
	   kerror (NULL, "X3D_draw_lineplot", "Not enough memory\n");
	   return;
	}

	/*
	 *  Initialize the current segment size
	 */
	X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);

        /* 
	 * for each segment, plot a separate curve 
	 */
	colormap = X3D_get_colormap(graphics->id);
	for (i = 0; i < col_size; i++)
        {
	   j = i * row_size;
	   if (!_X3D_clip_lines(graphics, &coords[j], row_size,
				ncoords, &nsize))
	   {
	      continue;
	   }
	   seg_num = nsize/2;

	   if (!(_X3D_convert_wc_to_dc_seg(graphics, graphics->matrix2, 
					   ncoords, segments, nsize)))
	   {
	      kfree_and_NULL(segments);
	      return;
	   }
	   /* for each segment of curve, set color */
	   for (k = 0; k < seg_num; k++)
	   {
	  	_X3D_set_fgcolor(graphics, colormap, ncoords[k*2].d, fg);
	        X3D_draw[graphics->device].segments(graphics, &segments[k], 1);
	   }

	}
        kfree_and_NULL(ncoords);
	kfree_and_NULL(segments);
}

/*-----------------------------------------------------------
|
|  Routine Name: X3D_draw_wireframe
|
|       Purpose: Connects a series of polylines to make a 3D
|		 plot in a mesh pattern. This is done by drawing
|		 both lines along both the columns and rows.
|
|         Input: id	  - X3D graphics structure ID
|		 coords	  - array of world coordinate points
|		 row_size - number of points in each polyline
|		 size	  - number of points
|		 fg	  - color of wire frame
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X3D_draw_wireframe(
   XVGraphicsID id,
   Coord        *coords,
   int          row_size,
   int          size,
   XColor       *fg)
{
	X3DGraphics	*graphics;
	XSegment	*segments;
	Coord		*columns, *ncoords;
	int		i, j, k, l; 
	int             col_size, nsize, seg_num, seg_size;
	Colormap        colormap;

	col_size = size/row_size;
	if (col_size == 1 || row_size == 1)
	{
	   X3D_draw_lineplot(id, coords, row_size, size, fg);
	   return;
	}

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X3D_draw_wireframe", "Unknown graphics id %d\n", id);
	   return;
	}

	if ((columns = (Coord *) kmalloc((size_t)col_size*sizeof(Coord)))
		== NULL)
	{
	   kerror (NULL, "X3D_draw_wireframe", "Not enough memory\n");
	   return;
	}

	seg_size = kmax(col_size, row_size);
	if ((segments = (XSegment *) kmalloc(seg_size * sizeof(XSegment)))
	                      == NULL)
	{
	   kerror (NULL, "X3D_draw_wireframe", "Not enough memory\n");
	   return;
	}

	if ((ncoords = (Coord *) kmalloc(seg_size*2 * sizeof(Coord))) == NULL)
	{
	   kerror (NULL, "X3D_draw_wireframe", "Not enough memory\n");
	   kfree(segments);
	   return;
	}

        /*
	 *  Draw the rows first
	 */
	colormap = X3D_get_colormap(graphics->id);
	for (i = 0; i < col_size; i++)
        {
	   j = i * row_size;
	   if (!_X3D_clip_lines(graphics, &coords[j], row_size,ncoords, &nsize))
	      continue;

	   seg_num = nsize/2;
	   if (!(_X3D_convert_wc_to_dc_seg(graphics, graphics->matrix2, ncoords,
		segments, nsize)))
	   {
	      kfree(segments);
	      kfree(ncoords);
	      return;
	   }
	   for (k = 0; k < seg_num; k++)
	   {
	      _X3D_set_fgcolor(graphics, colormap, ncoords[k*2].d, fg);
	      X3D_draw[graphics->device].segments(graphics, &segments[k], 1);
	   }
	}

        /*
	 *  Draw the cols last
	 */
	for (i = 0; i < row_size; i++)
        {
	    for (j = 0; j < col_size; j++)
	    {
	        l = j * row_size + i;
		columns[j] = coords[l];
	    }

	   if (!_X3D_clip_lines(graphics, columns, col_size, ncoords, &nsize))
	      continue;

	   seg_num = nsize/2;
	   if (!(_X3D_convert_wc_to_dc_seg(graphics, graphics->matrix2, ncoords,
		segments, nsize)))
	   {
	      kfree(segments);
	      kfree(ncoords);
	      return;
	   }
	   for (k = 0; k < seg_num; k++)
           {
              _X3D_set_fgcolor(graphics, colormap, ncoords[k*2].d, fg);
	      X3D_draw[graphics->device].segments(graphics, &segments[k], 1);
	   }
	}
        kfree(segments);
        kfree(columns);
        kfree(ncoords);
}

/*-----------------------------------------------------------
|
|  Routine Name: X3D_draw_impulse
|
|       Purpose: Draws a 3D impulse plot
|
|         Input: id	- X3D graphics structure ID
|		 coords	- array of world coordinate points
|		 size	- number of points
|		 fg	- color of impulse
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X3D_draw_impulse (
   XVGraphicsID id,
   Coord        *coords,
   int          size,
   XColor       *fg)
{
	X3DGraphics	*graphics;
        XSegment        *segments;
	Coord		*impulse, *ncoords;
	Real		base_value;
	int		i, j, seg_num, nsize;
	Colormap        colormap;


	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X3D_draw_impulse", "Unknown graphics id %d\n", id);
	   return;
	}

	if ((impulse = (Coord *) kmalloc ((size_t) size*2 * sizeof(Coord)))
		== NULL)
	{
	   kerror (NULL, "X3D_draw_impulse", "Not enough memory\n");
	   return;
	}

	if ((kmin(graphics->wc_min.z, graphics->wc_max.z) < 0.0) && 
            (kmax(graphics->wc_min.z, graphics->wc_max.z) > 0.0))
	   base_value = 0.0;
	else
	   base_value = kmin(graphics->wc_min.z, graphics->wc_max.z);

	i = j = 0;
	while (i < size)
	{
	   impulse[j] = impulse[j+1] = coords[i++];
	   impulse[j+1].z = base_value;
	   j += 2;
	}

	if ((ncoords = (Coord *) kmalloc(size*2 * sizeof(Coord))) == NULL)
	{
	   kerror (NULL, "X3D_draw_impulse", "Not enough memory\n");
	   return;
	}

	if (!_X3D_clip_segments(graphics, impulse, 2*size, ncoords, &nsize))
	{
	   kfree_and_NULL(impulse);
	   kfree_and_NULL(ncoords);
	   return;
	}

	seg_num = nsize/2;
	if ((segments = (XSegment *) kmalloc ((size_t) nsize * 
			 sizeof(XSegment))) == NULL)
	{
	   kerror (NULL, "X3D_draw_impulse", "Not enough memory\n");
	   kfree_and_NULL(impulse);
	   kfree_and_NULL(segments);
	   kfree_and_NULL(ncoords);
	   return;
	}


	if (!(_X3D_convert_wc_to_dc_seg(graphics, graphics->matrix2, ncoords,
			segments, nsize)))
	{
	   kfree_and_NULL(impulse);
	   kfree_and_NULL(segments);
	   kfree_and_NULL(ncoords);
	   return;
	}
	colormap = X3D_get_colormap(graphics->id);
	for (i = 0; i < seg_num; i++)
	{
	    _X3D_set_fgcolor(graphics, colormap, impulse[i].d, fg);
	    X3D_draw[graphics->device].segments(graphics, &segments[i], 1);
	}

	kfree_and_NULL(impulse);
	kfree_and_NULL(segments);
	kfree_and_NULL(ncoords);
}

/*-----------------------------------------------------------
|
|  Routine Name: X3D_draw_polymarker
|
|       Purpose: Draws a 3D polymarker plot
|
|         Input: id           - X3D graphics structure ID
|                coords       - array of world coordinate points
|                size         - number of points
|                marker_scale - amount by which to scale markers
|                marker_type  - type of marker to display
|                fg           - color of polymarker
|
|    Written By: Mark Young
|          Date:
| Modifications:
|
------------------------------------------------------------*/

void X3D_draw_polymarker (
   XVGraphicsID id,
   Coord        *coords,
   int          size,
   int          marker_scale,
   int          marker_type,
   XColor       *fg)
{
	X3DGraphics	*graphics;
        XPoint          *points;
        Coord           *ncoords;
        int             nsize, clip_code;

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X3D_draw_polymarker", "Unknown graphics id %d\n", id);
	   return;
	}

        clip_code = X2D_CENTER;
        if ((ncoords = _X3D_clip_points(graphics, coords, size, clip_code,
                                        NULL, &nsize)) == NULL)
        {
           return;
        }

        if ((points = (XPoint *) kmalloc ((size_t) nsize * sizeof (XPoint)))
                == NULL)
        {
           kfprintf (kstderr,"X3D_draw_marker:\n");
           kfprintf (kstderr,"\t not enough memory\n");
           return;
        }

        if(!(_X3D_convert_wc_to_dc(graphics, graphics->matrix2, ncoords, points,
                        nsize)))
        {
           kfree(points);
           return;
        }

	_X2D_draw_markers(id, ncoords, points, nsize, 
		          marker_scale, marker_type, fg);

        if (coords != ncoords)
           kfree(ncoords);
        kfree(points);
}

/*-----------------------------------------------------------
|
|  Routine Name: X3D_draw_colormarker
|
|       Purpose: Draws a 3D colormarker plot
|
|         Input: id           - X3D graphics structure ID
|                coords       - array of world coordinate points
|                size         - number of points
|                marker_scale - amount by which to scale markers
|                marker_type  - type of marker to display
|		 colors	      - color array
|		 num   	      - size of color array
|
|    Written By: Mark Young
|          Date:
| Modifications:
|
------------------------------------------------------------*/

void X3D_draw_colormarker (
   XVGraphicsID id,
   Coord        *coords,
   int          size,
   int          marker_scale,
   int          marker_type,
   XColor       *colors,
   int          num,
   XColor       *plotcolor)
{
/*
	XColor		*fg;
	X3DGraphics	*graphics;
	Coord		*ncoords;
	int		nsize, clip_code;


	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X3D_draw_colormarker", 
		   "Unknown graphics id %d\n", id);
	   return;
	}

	clip_code = X2D_CENTER;
	if ((ncoords = _X3D_clip_points(graphics, coords, size, clip_code,
		NULL, &nsize)) == NULL)
	{
	   return;
	}

	_X2D_draw_markers(id, ncoords, points, nsize, 
			  marker_scale, marker_type, fg);

	if (coords != ncoords)
	   kfree_and_NULL(ncoords);
*/
}

/*-----------------------------------------------------------
|
|  Routine Name: X3D_draw_polygon
|
|       Purpose: Draws a 3D polygon
|
|         Input: id     - X3D graphics structure ID
|                coords - array of world coordinate points
|                size   - number of points
|                fg     - color of polygon
|                bg     - color of polygon's faces
|
|    Written By: Mark Young
|          Date:
| Modifications:
|
------------------------------------------------------------*/

void X3D_draw_polygon (
   XVGraphicsID id,
   Coord        *coords,
   int          size,
   XColor       *fg,
   XColor       *bg)
{
	X3DGraphics	*graphics;
	int		nsize;
	Coord		*ncoords;
	XPoint		*points;

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X3D_draw_polygon", "Unknown graphics id %d\n", id);
	   return;
	}

	if ((ncoords = _X3D_clip_polygon(graphics, coords, size, NULL,&nsize))
		== NULL)
	   return;

	if ((points = (XPoint *) kmalloc ((size_t)(nsize+1)*sizeof (XPoint)))
		== NULL)
	{
	   kerror (NULL, "X3D_draw_polygon", "Not enough memory\n");
	   return;
	}

	if (!(_X3D_convert_wc_to_dc(graphics, graphics->matrix2, ncoords,
			points, nsize)))
	{
	   kfree_and_NULL(points);
	   return;
	}

        /*
	 *  Fill the desired polygon.
	 */
	X3D_draw[graphics->device].fill_color(graphics, bg, FALSE);
	X3D_draw[graphics->device].fill_polygon(graphics, points, nsize,
			Complex, CoordModeOrigin);

	if (points[0].x != points[nsize-1].x || points[0].y !=
		points[nsize-1].y)
	{
	   points[nsize++] = points[0];
	}

        /*
	 *  Draw the desired polyline outline.
	 */
	X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);
	X3D_draw[graphics->device].lines(graphics, points, nsize,
			CoordModeOrigin);

	if (ncoords != coords)
	   kfree_and_NULL(ncoords);
        kfree_and_NULL(points);
}

/*-----------------------------------------------------------
|
|  Routine Name: X3D_draw_segments
|
|       Purpose: Draws a series of 3D line segments
|
|         Input: id     - X3D graphics structure ID
|                coords - array of world coordinate points
|                size   - number of points
|                fg     - color of segments
|
|    Written By: Mark Young
|          Date:
| Modifications:
|
------------------------------------------------------------*/

void X3D_draw_segments (
   XVGraphicsID id,
   Coord        *coords,
   int          size,
   XColor       *fg)
{
        X3DGraphics     *graphics;
	Coord		*ncoords;
        XSegment        *segments;
        int             num, nsize;


        if ((graphics = _X3D_get_graphics(id)) == NULL)
        {
	   kerror (NULL, "X3D_draw_segments", "Unknown graphics id %d\n", id);
           return;
        }

	if ((ncoords = _X3D_clip_segments(graphics, coords, size,NULL,&nsize))
		== NULL)
	   return;

        num = nsize/2;
        if ((segments = (XSegment *) kmalloc ((size_t) num *
                         sizeof (XSegment))) == NULL)
        {
	   kerror (NULL, "X3D_draw_segments", "Not enough memory\n");
           return;
        }

        if(!(_X3D_convert_wc_to_dc_seg(graphics, graphics->matrix2, ncoords,
			segments, nsize)))
	{
	   kfree_and_NULL(segments);
	   return;
	}
	X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);
        X3D_draw[graphics->device].segments(graphics, segments, num);

	if (ncoords != coords)
	   kfree_and_NULL(ncoords);
        kfree_and_NULL(segments);
}

/*-----------------------------------------------------------
|
|  Routine Name: X3D_draw_line
|
|       Purpose: Draws a line in 3D space
|
|         Input: id     - X3D graphics structure ID
|                coord1 - the beginning world coordinate point
|                coord2 - the end world coordinate point
|                fg     - color of line
|
|    Written By: Mark Young
|          Date:
| Modifications:
|
------------------------------------------------------------*/

void X3D_draw_line (
   XVGraphicsID id,
   Coord        coord1,
   Coord        coord2,
   XColor       *fg)
{
        X3DGraphics     *graphics;
        XPoint          pt1, pt2;

        if ((graphics = _X3D_get_graphics(id)) == NULL)
        {
	   kerror (NULL, "X3D_draw_line", "Unknown graphics id %d\n", id);
           return;
        }

        if(!(_X3D_convert_wc_to_dc(graphics, graphics->matrix, &coord1,
			&pt1, 1)))
	   return;
	
        if(!(_X3D_convert_wc_to_dc(graphics, graphics->matrix, &coord2,
			&pt2, 1)))
	   return;

	X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);
        X3D_draw[graphics->device].line(graphics, pt1.x, pt1.y, pt2.x, pt2.y);
}

/*-----------------------------------------------------------
|
|  Routine Name: X3D_draw_rectangle
|
|       Purpose: Draws a rectangle in 3D space
|
|         Input: id     - X3D graphics structure ID
|                coord  - the world coordinate upper left edge of rectangle
|                width  - the world coordinate width (x direction) of rectangle
|                height - the world coordinate height (z direction) of rectangle
|                depth  - the world coordinate depth (y direction) of rectangle
|                fg     - color of rectangle
|
|    Written By: Mark Young
|          Date:
| Modifications:
|
------------------------------------------------------------*/

void X3D_draw_rectangle (
   XVGraphicsID id,
   Coord        coord,
   Real         width,
   Real         height,
   Real         depth,
   XColor       *fg)
{
	X3DGraphics	*graphics;
	Coord		coords[5];
	XPoint		point[5];


	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X3D_draw_rectangle", "Unknown graphics id %d\n", id);
	   return;
	}

	coords[0] = coords[4] = coord;

	coords[1] = coords[0];
	coords[1].x += width;
	coords[1].y += depth;

	coords[2] = coords[1];
	coords[2].z += height;

	coords[3] = coords[1];
	coords[3].z += height;

	if(!(_X2D_convert_wc_to_dc(graphics, coords, point, 5)))
	   return;

	X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);
	X3D_draw[graphics->device].lines(graphics, point, 5, CoordModeOrigin);
}

/*-----------------------------------------------------------
|
|  Routine Name: X3D_draw_mesh
|
|       Purpose: Draws a mesh plot
|
|         Input: id        - X3D graphics structure ID
|		 coords    - array of world coordinate points
|		 row_size  - number of points in each polyline
|		 size      - number of points
|		 fg        - color of mesh
|		 bg        - color of mesh face
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X3D_draw_mesh(
   XVGraphicsID id,
   Coord        *coords,
   int          row_size,
   int          size,
   XColor       *fg,
   XColor       *bg)
{
        X3DGraphics     *graphics;
	int		nsize;
	XPoint		points[10];
	Coord		ncoords[10], polygon[5];
	Colormap        colormap;

	int	 corner;
	register int indx;
	int	 row, col, num_row, num_col, i, j;

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X3D_draw_mesh", "Unknown graphics id %d\n", id);
	   return;
	}
	
	num_col = row_size;
	num_row = size/num_col;
	row = num_row -1;
	col = num_col -1;

	/*  Compute which corner to start drawing from */
	corner = _X3D_find_corner(graphics, &coords[0], &coords[row_size-1],
			&coords[size-row_size], &coords[size-1]);

	colormap = X3D_get_colormap(graphics->id);
	if (corner == BACKLEFT)
	{
	    for (i = 0; i < col; i++)
	    {
	        for (j = row; j > 0; j--)
		{
		    indx = j * num_col + i;
		    polygon[0] =
		    polygon[4] = coords[indx];

		    polygon[1] = coords[++indx];

		    indx -= num_col;
		    polygon[2] = coords[indx];

		    polygon[3] = coords[--indx];

		    if (!_X3D_clip_polygon(graphics, polygon,5,ncoords, &nsize))
		       continue;

		    if (!(_X3D_convert_wc_to_dc(graphics, graphics->matrix2,
			ncoords, points, nsize)))
		       continue;

		    _X3D_set_bgcolor(graphics, colormap, ncoords[0].d, bg);
		    X3D_draw[graphics->device].fill_polygon(graphics, points,
				nsize, Nonconvex, CoordModeOrigin);
		   _X3D_set_fgcolor(graphics, colormap, ncoords[0].d, fg);
		   X3D_draw[graphics->device].lines(graphics, points, nsize,
			CoordModeOrigin);
		}
	    }
	}
	else if (corner == BACKRIGHT)
	{
	    for (j = row; j > 0; j--)
	    {
	        for (i = col; i > 0; i--)
		{
	            indx = j * num_col + i;
		    polygon[0] =
		    polygon[4] = coords[indx];

		    polygon[1] = coords[--indx];

		    indx -= num_col;
		    polygon[2] = coords[indx];

		    polygon[3] = coords[++indx];

		    if (!_X3D_clip_polygon(graphics, polygon,5,ncoords, &nsize))
		       continue;

		    if (!(_X3D_convert_wc_to_dc(graphics, graphics->matrix2,
			ncoords, points, nsize)))
		       continue;

		    /*
		     *  Fill the desired polygon outline
		     */
		    _X3D_set_bgcolor(graphics, colormap, ncoords[0].d, bg);
		    X3D_draw[graphics->device].fill_polygon(graphics, points,
				nsize, Nonconvex, CoordModeOrigin);

		    /*
		     *  Draw the desired polyline outline.
		     */
		   _X3D_set_fgcolor(graphics, colormap, ncoords[0].d, fg);
		   X3D_draw[graphics->device].lines(graphics, points, nsize,
			CoordModeOrigin);
		}
	    }
	}
	else if (corner == FRONTRIGHT)
	{
	    for (i = col; i > 0; i--)
	    {
	        for (j = 0; j < row; j++)
		{
	            indx = j * num_col + i;
		    polygon[0] =
		    polygon[4] = coords[indx];

		    polygon[1] = coords[--indx];

		    indx += num_col;
		    polygon[2] = coords[indx];

		    polygon[3] = coords[++indx];

		    if (!_X3D_clip_polygon(graphics, polygon,5,ncoords, &nsize))
		       continue;

		    if (!(_X3D_convert_wc_to_dc(graphics, graphics->matrix2,
			ncoords, points, nsize)))
		       continue;

		    /*
		     *  Fill the desired polygon outline
		     */
		    _X3D_set_bgcolor(graphics, colormap, ncoords[0].d, bg);
		    X3D_draw[graphics->device].fill_polygon(graphics, points,
				nsize, Nonconvex, CoordModeOrigin);

		    /*
		     *  Draw the desired polyline outline.
		     */
		   _X3D_set_fgcolor(graphics, colormap, ncoords[0].d, fg);
		   X3D_draw[graphics->device].lines(graphics, points, nsize,
			CoordModeOrigin);
		}
	    }
	}
	else if (corner == FRONTLEFT)
	{
	    for (j = 0; j < row; j++)
	    {
	        for (i = 0; i < col; i++)
		{
	            indx = j * num_col + i;
		    polygon[0] =
		    polygon[4] = coords[indx];

		    polygon[1] = coords[++indx];

		    indx += num_col;
		    polygon[2] = coords[indx];

		    polygon[3] = coords[--indx];

		    if (!_X3D_clip_polygon(graphics, polygon,5,ncoords, &nsize))
		       continue;

		    if (!(_X3D_convert_wc_to_dc(graphics, graphics->matrix2,
			ncoords, points, nsize)))
		       continue;

		    /*
		     *  Fill the desired polygon outline
		     */
		    _X3D_set_bgcolor(graphics, colormap, ncoords[0].d, bg);
		    X3D_draw[graphics->device].fill_polygon(graphics, points,
				nsize, Nonconvex, CoordModeOrigin);

		    /*
		     *  Draw the desired polyline outline.
		     */
		   _X3D_set_fgcolor(graphics, colormap, ncoords[0].d, fg);
		   X3D_draw[graphics->device].lines(graphics, points, nsize,
			CoordModeOrigin);
		}
	    }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: X3D_draw_horizon
|
|       Purpose: Draws a horizon plot.  This is done by finding
|		 opposite side of the plot and drawing each row
|		 from back to front.  Painter's algorithm is used
|		 to fill the row, thus obscuring the previous row.
|		 This is repeated until all rows have been drawn.
|
|		 Note:  We say row, but we also mean column.  We draw
|			rows or columns depending on which side of the
|			plot is furthest from the viewer.  Thus if the
|			back left corner is furthest then we would draw
|			each row, but if the front left corner is the
|			furthest then we draw the each column.
|
|         Input: id        - X3D graphics structure ID
|		 coords    - array of world coordinate points
|		 row_size  - number of points in each polyline
|		 size      - number of points
|		 fg	   - color of horizon
|		 bg	   - color of horizon surface
|		 wcmin	   - the minimum extent of the plot
|		 wcmax	   - the maximum extent of the plot
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void X3D_draw_horizon(
   XVGraphicsID id,
   Coord        *coords,
   int          rows,
   int          size,
   XColor       *fg,
   XColor       *bg,
   Coord        wcmin,
   Coord        wcmax)
{
        X3DGraphics     *graphics;
	register XPoint	*points;
	register Coord	*horizon, *ncoords;
	Colormap colormap;

	int	 corner;
	register int i, j, m;
	int	 nsize, cols, rsize = rows +2;

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "X3D_draw_horizon", "Unknown graphics id %d\n", id);
	   return;
	}
	

	colormap = X3D_get_colormap(graphics->id);
	if ((horizon = (Coord *) kmalloc((size_t) rsize * sizeof(Coord)))
		== NULL)
	{
	   kerror (NULL, "X3D_draw_horizon", "Not enough memory\n");
	   return;
	}

	if ((ncoords = (Coord *) kmalloc((size_t) 2*rsize * sizeof(Coord)))
		== NULL)
	{
	   kerror (NULL, "X3D_draw_horizon", "Not enough memory\n");
	   kfree_and_NULL(horizon); return;
	}

	if ((points = (XPoint *) kmalloc((size_t) 2*rsize * sizeof(XPoint)))
		== NULL)
	{
	   kerror (NULL, "X3D_draw_horizon", "Not enough memory\n");
	   kfree_and_NULL(horizon); kfree_and_NULL(ncoords);
	   return;
	}

	/*  Compute which corner is closest to you */
	corner = _X3D_find_corner(graphics, &coords[0], &coords[rows-1],
			&coords[size-rows], &coords[size-1]);
	cols = size/rows;

	if (corner == BACKLEFT)
	{
	   horizon[0] = horizon[rows +1] = wcmin;
	   horizon[rows +1].x = wcmax.x;

	   for (i = 0; i < cols; i++)
	   {
	      m = (cols -i -1) * rows;
	      horizon[0].y = coords[m].y;
	      horizon[0].d = coords[m].d;

	      for (j = 1; j <= rows; j++)
		 horizon[j] = coords[m++];
	      horizon[rows +1].y = horizon[rows].y;

	      if (!_X3D_clip_polygon(graphics, horizon, rsize, ncoords, &nsize))
		 continue;

	      if (!(_X3D_convert_wc_to_dc(graphics, graphics->matrix2, ncoords,
			points, nsize)))
		 continue;

	      /*
	       *  Fill the desired polygon outline
	       */
              _X3D_set_bgcolor(graphics, colormap, ncoords[0].d, bg);
	      X3D_draw[graphics->device].fill_polygon(graphics, points, nsize,
				Nonconvex, CoordModeOrigin);

	      /*
	       *  Draw the desired polyline outline.
	       */
	      _X3D_set_fgcolor(graphics, colormap, ncoords[0].d, fg);
	      X3D_draw[graphics->device].lines(graphics, points, nsize,
			CoordModeOrigin);
	   }
	}
	else if (corner == FRONTLEFT)
	{
	   horizon[0] = horizon[rows +1] = wcmin;
	   horizon[rows +1].x = wcmax.x;

	   for (i = 0; i < cols; i++)
	   {
	      m = i * rows;
	      horizon[0].y = coords[m].y;
	      horizon[0].d = coords[m].d;

	      for (j = 1; j <= rows; j++)
		 horizon[j] = coords[m++];
	      horizon[rows +1].y = horizon[rows].y;

	      if (!_X3D_clip_polygon(graphics, horizon, rsize, ncoords, &nsize))
		 continue;

	      if (!(_X3D_convert_wc_to_dc(graphics, graphics->matrix2, ncoords,
			points, nsize)))
		 continue;

	      /*
	       *  Fill the desired polygon outline
	       */
              _X3D_set_bgcolor(graphics, colormap, ncoords[0].d, bg);
	      X3D_draw[graphics->device].fill_polygon(graphics, points, nsize,
				Nonconvex, CoordModeOrigin);

	      /*
	       *  Draw the desired polyline outline.
	       */
	      _X3D_set_fgcolor(graphics, colormap, ncoords[0].d, fg);
	      X3D_draw[graphics->device].lines(graphics, points, nsize,
			CoordModeOrigin);
	   }
	}
	else if (corner == BACKRIGHT)
	{
	   horizon[0] = horizon[rows +1] = wcmin;
	   horizon[0].x = wcmax.x;

	   for (i = 0; i < cols; i++)
	   {
	      m = (cols -i) * rows -1;
	      horizon[0].y = coords[m].y;
	      horizon[0].d = coords[m].d;

	      for (j = 1; j <= rows; j++)
		 horizon[j] = coords[m--];
	      horizon[rows +1].y = horizon[rows].y;

	      if (!_X3D_clip_polygon(graphics, horizon, rsize, ncoords, &nsize))
		 continue;

	      if (!(_X3D_convert_wc_to_dc(graphics, graphics->matrix2, ncoords,
			points, nsize)))
		 continue;

	      /*
	       *  Fill the desired polygon outline
	       */
              _X3D_set_bgcolor(graphics, colormap, ncoords[0].d, bg);
	      X3D_draw[graphics->device].fill_polygon(graphics, points, nsize,
				Nonconvex, CoordModeOrigin);

	      /*
	       *  Draw the desired polyline outline.
	       */
	      _X3D_set_fgcolor(graphics, colormap, ncoords[0].d, fg);
	      X3D_draw[graphics->device].lines(graphics, points, nsize,
			CoordModeOrigin);
	   }
	}
	else if (corner == FRONTRIGHT)
	{
	   horizon[0] = horizon[rows +1] = wcmin;
	   horizon[0].x = wcmax.x;

	   for (i = 0; i < cols; i++)
	   {
	      m = (i +1) * rows -1;
	      horizon[0].y = coords[m].y;
	      horizon[0].d = coords[m].d;

	      for (j = 1; j <= rows; j++)
		 horizon[j] = coords[m--];
	      horizon[rows +1].y = horizon[rows].y;

	      if (!_X3D_clip_polygon(graphics, horizon, rsize, ncoords, &nsize))
		 continue;

	      if (!(_X3D_convert_wc_to_dc(graphics, graphics->matrix2, ncoords,
			points, nsize)))
		 continue;

	      /*
	       *  Fill the desired polygon outline
	       */
              _X3D_set_bgcolor(graphics, colormap, ncoords[0].d, bg);
	      X3D_draw[graphics->device].fill_polygon(graphics, points, nsize,
				Nonconvex, CoordModeOrigin);

	      /*
	       *  Draw the desired polyline outline.
	       */
	      _X3D_set_fgcolor(graphics, colormap, ncoords[0].d, fg);
	      X3D_draw[graphics->device].lines(graphics, points, nsize,
			CoordModeOrigin);
	   }
	}
	kfree_and_NULL(points); kfree_and_NULL(horizon); kfree_and_NULL(ncoords);
}
