/*
  * Khoros: $Id: Ellipse.c,v 1.10.2.2 1997/04/29 22:30:03 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Ellipse.c,v 1.10.2.2 1997/04/29 22:30:03 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Ellipse Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Redisplay()
   >>>>                 SetValues()
   >>>>                 RecomputePosition()
   >>>>                 ChangeGeometry()
   >>>>                 PickGadget()
   >>>>                 EraseSelection()
   >>>>                 RefreshSelection()
   >>>>   Public:
   >>>>                 xvw_create_ellipse()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include <xvisual/EllipseP.h>

static void ClassInitialize	 PROTO((void));
static void Initialize           PROTO((Widget, Widget, ArgList, Cardinal *));
static void Redisplay		 PROTO((Widget, XEvent *, Region));
static Boolean SetValues	 PROTO((Widget, Widget, Widget, ArgList,
					Cardinal *));
static void RecomputePosition	 PROTO((Widget));
static void ChangeGeometry	 PROTO((Widget));
static int  PickGadget		 PROTO((Widget, int, int));

static void EraseSelection       PROTO((Widget));
static void RefreshSelection     PROTO((Widget));



/*--------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_ELLIPSE_XCENTER,   NULL,    XtRDouble,    NULL},
{XVW_ELLIPSE_YCENTER,   NULL,    XtRDouble,    NULL},
{XVW_ELLIPSE_WIDTH,     NULL,    XtRDouble,    NULL},
{XVW_ELLIPSE_HEIGHT,    NULL,    XtRDouble,    NULL},
};

/*--------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwEllipseGadgetRec, ellipse.field)

static XtResource resources[] = { 
{XVW_ELLIPSE_XCENTER, NULL, XtRDouble, sizeof(double), 
      offset(center.x), XtRString, (XtPointer) "0.0"},
{XVW_ELLIPSE_YCENTER, NULL, XtRDouble, sizeof(double), 
      offset(center.y), XtRString, (XtPointer) "0.0"},
{XVW_ELLIPSE_WIDTH,   NULL, XtRDouble, sizeof(double), 
      offset(width), XtRString, (XtPointer) "1.0"},
{XVW_ELLIPSE_HEIGHT,   NULL, XtRDouble, sizeof(double), 
      offset(height), XtRString, (XtPointer) "0.75"},

{XVW_FOREGROUND_PIXEL, NULL, XtRPixel, sizeof(Pixel),
        XtOffsetOf(XvwEllipseGadgetRec, manager.foreground),
        XtRString, (XtPointer) "#ffff00"},
};
#undef offset


#define SUPERCLASS (&xvwGraphicsGadgetClassRec)

XvwEllipseGadgetClassRec xvwEllipseGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Ellipse",			        /* class_name		  */
    sizeof(XvwEllipseGadgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    NULL,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),			/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    FALSE,				/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    PickGadget,				/* pick object proc       */
    EraseSelection,   			/* erase selected proc    */
    RefreshSelection,			/* refresh selection proc */
    ChangeGeometry,			/* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    RecomputePosition,			/* recompute position proc    */
  },  /* XvwGraphicsGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwEllipseGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwEllipseGadgetClass for public consumption
 */
WidgetClass xvwEllipseGadgetClass = (WidgetClass) &xvwEllipseGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of ellipse object class has been created.
|                It will initialize all the class attributes. 
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: January 8, 1996
| Modifications:
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
        xvw_init_attributes(xvwEllipseGadgetClass, attributes,
		      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/Ellipse.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Ellipse");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial values
|                for all the fields for a ellipse gadget instance.
|
|         Input: request - widget instance with requested public settings
|                new1     - widget instance after initialization, with
|                          private settings initialized and public
|                          settings verified
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 8, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwEllipseGadget xobj = (XvwEllipseGadget) new1;

	xobj->graphics.old_x      = new1->core.x;
	xobj->graphics.old_y      = new1->core.y;
	xobj->graphics.old_width  = new1->core.width;
	xobj->graphics.old_height = new1->core.height;
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the ellipse object in response
|                to an expose event. 
|
|         Input: widget - the ellipse object that was exposed 
|                event  - the event that caused the redraw
|                region - the region that was exposed 
|
|        Output: None
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 8, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	XvwEllipseGadget xobj = (XvwEllipseGadget) widget;

	XVGraphicsID id;
	XColor       *fg, *bg;

	id = xobj->graphics.id;
	fg = &xobj->graphics.fg;
	bg = &xobj->graphics.bg;

	if (xobj->graphics.filled == FALSE)
	   X2D_draw_ellipse(id, xobj->ellipse.center,
			    (Real) xobj->ellipse.width, 
			    (Real) xobj->ellipse.height, fg);
	else
	   X2D_fill_ellipse(id, xobj->ellipse.center,
			    (Real) xobj->ellipse.width, 
			    (Real) xobj->ellipse.height, fg, bg);

}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: If the width, height or the position of the ellipse 
|		 has changed, recalculate the size and position
|		 of the ellipse.
|
|         Input: current - the object containing current settings 
|                request - the object containing requested settings
|                new1     - the object processed through all set values methods 
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 8, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues (
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwEllipseGadget cobj = (XvwEllipseGadget) current;
	XvwEllipseGadget nobj = (XvwEllipseGadget) new1;


	if (cobj->ellipse.center.x != nobj->ellipse.center.x ||
	    cobj->ellipse.center.y != nobj->ellipse.center.y ||
	    cobj->ellipse.width    != nobj->ellipse.width    ||
	    cobj->ellipse.height   != nobj->ellipse.height)
	{
	   if (nobj->ellipse.width  <= 0.0 || 
	       nobj->ellipse.height <= 0.0)
	   {
	      kinfo(KFORCE, "Ellipse Object: The width and height must be > 0.0, setting back to %g x %g.\n", cobj->ellipse.width, cobj->ellipse.height);
	      nobj->ellipse.width  = cobj->ellipse.width;
	      nobj->ellipse.height = cobj->ellipse.height;
	   }
	   GraphicsRecomputePosition(new1, FALSE);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: RecomputePosition
|
|       Purpose: Given a new center, width, and height, recompute
|		 the new device coordinate position of the ellipse. 
|
|         Input: widget - the ellipse object  
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 8, 1996
|
------------------------------------------------------------*/

static void RecomputePosition(
   Widget widget)
{
	XvwEllipseGadget xobj = (XvwEllipseGadget) widget;

	Coord  coord;
	Real   xpos1, xpos2, ypos1, ypos2;
	XVGraphicsID id = xobj->graphics.id;

	coord = xobj->ellipse.center;
	coord.x -= xobj->ellipse.width/2.0;
	coord.y -= xobj->ellipse.height/2.0;
	X2D_convert_point_wc_to_dc(id, coord, &xpos1, &ypos1);

	coord = xobj->ellipse.center;
	coord.x += xobj->ellipse.width/2.0;
	coord.y += xobj->ellipse.height/2.0;
	X2D_convert_point_wc_to_dc(id, coord, &xpos2, &ypos2);

	xobj->graphics.old_x      = widget->core.x;
        xobj->graphics.old_y      = widget->core.y;
        xobj->graphics.old_width  = widget->core.width;
        xobj->graphics.old_height = widget->core.height;

	widget->core.x      = (Position) kmin(xpos1, xpos2);
        widget->core.y      = (Position) kmin(ypos1, ypos2);
        widget->core.width  = (Dimension) kabs(xpos2 - xpos1);
        widget->core.height = (Dimension) kabs(ypos2 - ypos1);
}

/*-----------------------------------------------------------
|
|  Routine Name: ChangeGeometry
|
|       Purpose: Given a new device coordinate description of the
|		 ellipse, recompute the world coordinate center
|		 and radius.
|
|         Input: widget - the ellipse object
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 8, 1996
|
------------------------------------------------------------*/

static void ChangeGeometry(
   Widget widget)
{
	XvwEllipseGadget xobj = (XvwEllipseGadget) widget;
	Coord coord1, coord2;
	Real  xpos, ypos;
	Position width, height;

	XVGraphicsID id = xobj->graphics.id;

	/*
         * Resize procedure: compute the new world coordinate width and height.
	 * RecomputePosition is called to update the device coordinates, 
	 * since in this case DC change => WC change => DC change.  This is not
	 * typical, (usally in ChangeGeometry, DC change => WC change, period)
	 * but is necessary to keep center of ellipse at the same point 
	 * during the resize procedure.
         */
	if (xobj->graphics.pick_status == PICK_RESIZE)
	{
	    width = widget->core.width;
	    height = widget->core.height;
	   
	    /*-- compute the new world coordinate radius --*/
	    X2D_convert_point_dc_to_wc(id, 0.0, 0.0, &coord1);
            X2D_convert_point_dc_to_wc(id, (Real)width, (Real)height, &coord2);
            xobj->ellipse.width  = kabs(coord2.x - coord1.x);
            xobj->ellipse.height = kabs(coord2.y - coord1.y);


	    /*-- Update core.x, core.y DC's --*/
	    RecomputePosition(widget); 
	}

	/*
	 * Move procedure: 
	 * maintain size of ellipse, update center point
	 */
	else if (xobj->graphics.pick_status == PICK_MOVE)
	{
	    /*-- compute world coordinate center point --*/
	    xpos = (Real) (widget->core.x + (Real)widget->core.width/2.0);
            ypos = (Real) (widget->core.y + (Real)widget->core.height/2.0);
            X2D_convert_point_dc_to_wc(id, xpos, ypos,
                                       &xobj->ellipse.center);
	    RecomputePosition(widget);  
	}

	/*
	 * Display procedure:
	 * get placement & radius from DC's
	 */
	else
	{
            /*-- compute world coordinate center point --*/
            xpos = (Real) (widget->core.x + (Real)widget->core.width/2.0);
            ypos = (Real) (widget->core.y + (Real)widget->core.height/2.0);
            X2D_convert_point_dc_to_wc(id, xpos, ypos,
                                       &xobj->ellipse.center);

            /*-- compute the new world coordinate width & height  --*/
            X2D_convert_point_dc_to_wc(id, 0.0, 0.0, &coord1);
            X2D_convert_point_dc_to_wc(id, 
				       (Real)widget->core.width,
				       (Real)widget->core.height,
				       &coord2);

            xobj->ellipse.width  = kabs(coord2.x - coord1.x);
            xobj->ellipse.height = kabs(coord2.y - coord1.y);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: PickGadget
|
|       Purpose: Given a ellipse object and an attempted picking location,
|		 see if the ellipse has been picked.  Note whether the 
|		 ellipse was picked on a corner marker (implying a resize)
|		 or on the ellipse radius (implying a move).
|
|         Input: widget - the ellipse object we are checking 
|                x      - the x device coordinate that the pick attempt was at 
|                y      - the y device coordinate that the pick attempt was at
|
|       Returns: TRUE (1) if object was successfully picked, FALSE (0) otherwise
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 8, 1996
|
------------------------------------------------------------*/

static int PickGadget(
   Widget widget,
   int    x,
   int    y)
{
        XvwEllipseGadget xobj = (XvwEllipseGadget) widget;

	/*
         *  ellipse was picked on one of its corners markers;
         */
	if (xvw_pick_cornermarkers(widget, x, y))
	{
	    xobj->graphics.pick_status = PICK_RESIZE;
	    return(TRUE);
	}

	/*
	 *  ellipse was picked on one of its side markers  
 	 */
	else if (xvw_pick_sidemarkers(widget, x, y))
	{
	    xobj->graphics.pick_status = PICK_RESIZE;
	    return(TRUE);
	}

        /*
	 *  ellipse was picked elsewhere; consider it picked for moving
         */
	else
        {
           xobj->graphics.pick_status = PICK_MOVE;
           return(TRUE);
        }

}

/*-----------------------------------------------------------
|
|  Routine Name: EraseSelection
|
|       Purpose: Erase the side and corner markers on the ellipse
|		 which had indicated that the ellipse had been selected
|
|         Input: widget - the ellipse whose corner markers we are erasing
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 8, 1996
|
------------------------------------------------------------*/

/* ARGSUSED */
static void EraseSelection(
    Widget widget)
{
	XvwGraphicsGadgetClassRec *graphics_class;

        /*
         * call utility routine to refresh the side markers around ellipse
         */
        xvw_erase_sidemarkers(widget);

	/* 
	 * call the EraseSelection method of the Graphics Class
	 * to erase the corner markers around the ellipse
	 */
	graphics_class = (XvwGraphicsGadgetClassRec *) 
		         (xvwEllipseGadgetClass->core_class.superclass);
	(*graphics_class->manager_class.erase_selection)(widget);
}

/*-----------------------------------------------------------
|
|  Routine Name: RefreshSelection
|
|       Purpose: Draws little side and corner markers on the ellipse
|                to indicate that the ellipse has been selected
|
|         Input: widget - ellipse whose side & corner markers we are refreshing
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 8, 1996
| Modifications:
|
------------------------------------------------------------*/

/* ARGSUSED */
static void RefreshSelection(
    Widget widget)
{
        XvwEllipseGadget xwid = (XvwEllipseGadget) widget;
	XvwGraphicsGadgetClassRec *graphics_class;
	GC gc = xwid->manager.foreground_gc;

	/*
	 * call utility routine to refresh the side markers around ellipse
	 */
	xvw_draw_sidemarkers(widget, gc);

	/* 
	 * call the RefreshSelection method of the Graphics Class
	 * to refresh the corner markers around the ellipse
	 */
	graphics_class = (XvwGraphicsGadgetClassRec *) 
		         (xvwEllipseGadgetClass->core_class.superclass);
	(*graphics_class->manager_class.refresh_selection)(widget);
}


/************************************************************
*
*  Routine Name: xvw_create_ellipse - create a ellipse object.
*
*       Purpose: An ellipse visual object supports the display of an ellipse.
*
*                The (x, y) location of the ellipse center, its width and height
*                may be specified in world coordinates, where the
*                world coordinate range is dictated by the "controlling" visual
*                object. By default, the parent of the ellipse object is its
*                controlling visual object; this may be changed using the
*                XVW_GRAPHICS_ATTACH attribute.
*
*                Alternatively, the size and location of the ellipse may be
*                specified using device coordinates. When device coordinates
*                are used, they are with respect to the bounding box surrounding
*                the ellipse; specify the (x,y) location of the upper left hand
*                corner of the bounding box, as well as the width and height
*                of the bounding box.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically.
*                name   - a name for this particular instance of the ellipse
*                         object (for use in app-defaults files, etc)
*
*        Output: 
*	Returns: The ellipse xvobject on success, NULL on failure
*  Restrictions:
*    Written By: Danielle Argiro & Steve Kubica
*          Date: January 8, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_ellipse(
   xvobject parent,
   char     *name)
{
	xvobject  object;


	/*
	 *  Create the ellipse object
	 */
	object = xvw_create(parent, FALSE, TRUE, name, EllipseGadgetClass);
	return(object);
}
