/*
  * Khoros: $Id: PanIcon.c,v 1.21.2.3 1997/05/24 19:03:36 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: PanIcon.c,v 1.21.2.3 1997/05/24 19:03:36 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		PanIcon Widget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Redisplay()
   >>>>                 SetValues()
   >>>>                 ChangePanIconPosition()
   >>>>                 UpdatePosition()
   >>>>                 PanIconCallback()
   >>>>   Public:
   >>>>                 xvw_create_panicon()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/PanIconP.h>


static void ClassInitialize PROTO((void));
static void Initialize	    PROTO((Widget, Widget, ArgList, Cardinal *));
static Boolean SetValues    PROTO((Widget, Widget, Widget, ArgList,
				   Cardinal *));
static void Redisplay	    PROTO((Widget, XEvent *, Region));
static void UpdatePosition  PROTO((xvobject, kaddr, XEvent *, int *));
static void PanIconCallback PROTO((kobject, char *, Widget, kdms_callback *));
static void ChangePanIconPosition PROTO((Widget, int));

#undef kwidget
#undef kwidgetclass

#define kwidget(widget)      (XvwPanIconWidget) (widget)
#define kwidgetclass(widget) (XvwPanIconWidgetClass) (widget)->core.widget_class

/*--------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_PANICON_SIZE,	NULL,	XtRInt,		XtRDimension},
{XVW_PANICON_XPOS,	NULL,	XtRInt,		XtRPosition},
{XVW_PANICON_YPOS,	NULL,	XtRInt,		XtRPosition},
{XVW_PANICON_WIDTH,	NULL,	XtRInt,		XtRDimension},
{XVW_PANICON_HEIGHT,	NULL,	XtRInt,		XtRDimension},
{XVW_PANICON_CALLBACK,	NULL,	XtRCallback,	NULL},
};

/*--------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwPanIconWidgetRec, panicon.field)

static XtResource resources[] = { 
{XVW_PANICON_SIZE, NULL, XtRDimension, sizeof(Dimension),
      offset(size), XtRImmediate, (XtPointer) 100},
{XVW_PANICON_XPOS, NULL, XtRPosition, sizeof(Position),
      offset(xpos), XtRImmediate, (XtPointer) 0},
{XVW_PANICON_YPOS, NULL, XtRPosition, sizeof(Position),
      offset(ypos), XtRImmediate, (XtPointer) 0},
{XVW_PANICON_WIDTH, NULL, XtRDimension, sizeof(Dimension),
      offset(width), XtRImmediate, (XtPointer) 100},
{XVW_PANICON_HEIGHT, NULL, XtRDimension, sizeof(Dimension),
      offset(height), XtRImmediate, (XtPointer) 100},
{XVW_PANICON_CALLBACK, NULL, XtRCallback, sizeof(XtCallbackList),
      offset(update_callback), XtRCallback, (XtPointer) NULL},

{NULL, NULL, XtRPixel, sizeof(Pixel),
      offset(black), XtRString, (XtPointer) "#000000"},
{NULL, NULL, XtRPixel, sizeof(Pixel),
      offset(white), XtRString, (XtPointer) "#ffffff"},
{XVW_IMAGE_BACKING, NULL, XtRBoolean, sizeof(Boolean),
        XtOffsetOf(XvwImageWidgetRec, image.backing),
        XtRImmediate, (XtPointer) TRUE},
{NULL, NULL, XtRInt, sizeof(int),
        XtOffsetOf(XvwImageWidgetRec, image.reload_type),
        XtRImmediate, (XtPointer) PANICON_OBJECT_RELOAD},
};
#undef offset


#define SUPERCLASS (&xvwImageWidgetClassRec)

XvwPanIconWidgetClassRec xvwPanIconWidgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "PanIcon",				/* class_name		  */
    sizeof(XvwPanIconWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),			/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,	            		/* subresources       */
    0,					/* subresources_count */
    sizeof(XvwManagerWidgetConstraintsRec),/* constraint_size    */
    NULL,				/* initialize         */
    NULL,                               /* destroy            */
    NULL,				/* set_values         */
    NULL,                               /* extension          */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			/* pre-layout routine     */
    XtInheritLayout,			/* post-layout routine    */
    XtInheritChangeSel,                 /* change selected proc   */
    XtInheritEraseSel,                  /* erase selected proc    */
    XtInheritRefreshSel,                /* refresh selection proc */
    XtInheritResize,                    /* resize                 */
    XtInheritGeometryManager,           /* geometry_manager       */
  }, /* XvwManagerWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwGraphicsWidgetClass fields initialization */
  {
    XtInheritColorReload,		/* reload color proc             */
  },  /* XvwColorWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwImageWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwPanIconWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwPanIconWidgetClass for public consumption
 */
WidgetClass xvwPanIconWidgetClass = (WidgetClass) &xvwPanIconWidgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of panicon widget class has been created.
|                It will initialize all the class attributes.
|
|    Written By: Mark Young and John M. Salas
|          Date: Aug 15, 1992
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
        xvw_init_attributes(xvwPanIconWidgetClass, attributes,
		      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/PanIcon.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/PanIcon");
}


/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial image
|		 for an image widget instance. 
|
|         Input: request - not used
|		 new1	 - widget instance after initialization, with 
|		           initial image
|    Written By: Mark Young
|          Date: Mon Jun 29
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwPanIconWidget xwid = kwidget(new1);

	XColor color;

	/*
	 *  Initialize the factor to a scale of 1
	 */
	xwid->panicon.factor = 1.0;
	xwid->panicon.max_width  = 0;
	xwid->panicon.max_height = 0;
	xvw_add_event(xvw_object(new1), ButtonMotionMask | ButtonPressMask |
		      ButtonReleaseMask, UpdatePosition, NULL);

	color.flags = DoRed | DoGreen | DoBlue;

	color.red = color.green = color.blue = 0;
	if (XAllocColor(XtDisplay(new1), new1->core.colormap, &color) != 0)
	   xwid->panicon.black = color.pixel;

	color.red = color.green = color.blue = 65535;
	if (XAllocColor(XtDisplay(new1), new1->core.colormap, &color) != 0)
	   xwid->panicon.white = color.pixel;
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the rectangle within the
|		 PanIcon.
|
|         Input: widget - widget data structure for panicon
|		 event	- the event that caused the redraw
|		 region	- the region that was exposed
|    Written By: Mark Young
|          Date: Jun 26, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	(*xvwPanIconWidgetClass->core_class.superclass->core_class.expose)
                (widget, event, region);
	ChangePanIconPosition(widget, FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the public values
|                of a panicon widget instance.   
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new1     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Oct 24, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwPanIconWidget cxwid = (XvwPanIconWidget) current;
	XvwPanIconWidget nxwid = (XvwPanIconWidget) new1;

	double factor;
	int w, h, d, t, e, redisplay = FALSE;


	if (cxwid->image.iobject != nxwid->image.iobject)
	{
	   kpds_add_value_callback(nxwid->image.iobject, KPDS_CALLBACK_ACCESS,
			(kfunc_void) PanIconCallback, (kaddr) new1);
	}

	if (nxwid->image.iobject != NULL &&
	   (cxwid->image.iobject != nxwid->image.iobject  ||
	    cxwid->panicon.size  != nxwid->panicon.size))
	{
	   /*
	    *  Get the value data demsionality
	    */
	   kpds_get_attribute(nxwid->image.iobject, KPDS_VALUE_SIZE,
			&w, &h, &d, &t, &e);

	   factor = nxwid->panicon.size;
	   factor = kmin(factor/w, factor/h);
	   w *= factor; h *= factor;
	   w = kmax(w, 1); h = kmax(h, 1);

           /*
            *  Max it out...
            */
           nxwid->panicon.max_width =
           nxwid->manager.max_width = nxwid->core.width  = w;
           nxwid->panicon.max_height =
           nxwid->manager.max_height = nxwid->core.height = h;

	   nxwid->panicon.factor = factor; 
	   xvw_set_attributes(xvw_object(new1),
		XVW_PREFERRED_WIDTH,  nxwid->core.width,
		XVW_PREFERRED_HEIGHT, nxwid->core.height,
		NULL);
	   kpds_set_attributes(nxwid->image.iobject,
		KPDS_VALUE_SIZE, w, h, d, t, e,
		KPDS_VALUE_INTERPOLATE, KZERO_ORDER,
		NULL);

	   /*
	    *  In case there is a mask segment...
	    */
	   kpds_set_attributes(nxwid->image.iobject,
		KPDS_MASK_SIZE, w, h, d, t, e,
		KPDS_MASK_INTERPOLATE, KZERO_ORDER,
		NULL);

	   /*
	    *   we really need to delete this kludge, but if we do then data
	    *   services will core dump...  Jun 06, 1994 (MY)
	    */
	   if (xvw_check_subclass(xvw_object(XtParent(new1)),
				  xvwManagerWidgetClass))
	   {
	      XvwManagerWidgetConstraints kman = (XvwManagerWidgetConstraints)
							new1->core.constraints;
	      kman->manager.max_width  = w;
	      kman->manager.max_height = h;
	   }
	}

	if (cxwid->panicon.xpos   !=  nxwid->panicon.xpos  ||
	    cxwid->panicon.ypos   !=  nxwid->panicon.ypos  ||
	    cxwid->panicon.width  !=  nxwid->panicon.width  ||
	    cxwid->panicon.height !=  nxwid->panicon.height)
	{
	   ChangePanIconPosition(new1, TRUE);
	}
	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: ChangePanIconPosition
|
|       Purpose: changes the PanIcon position of the panner in the image object
|                so that if there is other devices that have any access
|                callbacks we will know it.
|
|         Input: widget    - panicon widget
|                update_position - whether to trigger an update
|    Written By: Mark Young
|          Date: Jun 26, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void ChangePanIconPosition(
   Widget widget,
   int    update_position)
{
        XvwPanIconWidget xwid = (XvwPanIconWidget) widget;
	double value;
	int    x, y, w, h, d, t, e, xtmp, ytmp, type, reload, update;

	/*
	 *  Sanity check to make sure the PanIcon is realized.
	 */
	if (!XtIsRealized(widget)) return;

	kpds_get_attributes(xwid->image.iobject,
		KPDS_VALUE_DATA_TYPE, &type,
		KPDS_VALUE_POSITION, &w, &h, &d, &t, &e,
		NULL);

	x = xwid->panicon.xpos   * xwid->panicon.factor; xtmp = x;
	y = xwid->panicon.ypos   * xwid->panicon.factor; ytmp = y;
	w = xwid->panicon.width  * xwid->panicon.factor;
	h = xwid->panicon.height * xwid->panicon.factor;
	if (x < 0)
	   x = 0;
	else if ((x + w + 1) >= xwid->panicon.max_width)
	   x = xwid->panicon.max_width - w - 1;
	if (y < 0)
	   y = 0;
	else if ((y + h + 1) >= xwid->panicon.max_height)
	   y = xwid->panicon.max_height - h - 1;

	if (x != xtmp) xwid->panicon.xpos = x/xwid->panicon.factor;
	if (y != ytmp) xwid->panicon.ypos = y/xwid->panicon.factor;

	/*
	 *  Clear the previous rectangle.
	 */
	XClearArea(XtDisplay(widget), XtWindow(widget), 0, 0, 0, 0, FALSE);

	/*
	 *  Draw the background "black" rectangles.
	 */
	XDrawRectangle(XtDisplay(widget), XtWindow(widget),
		xwid->manager.foreground_gc, x-1, y-1,
		(unsigned int) w+2, (unsigned int) h+2);
	XDrawRectangle(XtDisplay(widget), XtWindow(widget),
		xwid->manager.foreground_gc, x+1, y+1,
		(unsigned int) w-2, (unsigned int) h-2);

	/*
	 *  Now draw the foreground "white" rectangle.
	 */
	XDrawRectangle(XtDisplay(widget), XtWindow(widget),
		xwid->manager.top_shadow_gc, x, y,
		(unsigned int) w, (unsigned int) h);

	if (update_position)
	{
	   kpds_set_attributes(xwid->image.iobject,
		KPDS_VALUE_DATA_TYPE, KDOUBLE,
		KPDS_VALUE_POSITION, x, y, d, t, e,
		NULL);

	   xvw_get_type(xwid->image.iobject, &reload, &update);
	   xvw_set_type(xwid->image.iobject, PANICON_OBJECT_RELOAD,
			PANICON_POSITION_UPDATE);
	   kpds_get_data(xwid->image.iobject, KPDS_VALUE_POINT, &value);
	   kpds_set_attribute(xwid->image.iobject, KPDS_VALUE_DATA_TYPE, type);
	   xvw_set_type(xwid->image.iobject, reload, update);
	   XtCallCallbacks(widget, XVW_PANICON_CALLBACK, NULL);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: UpdatePosition
|
|       Purpose: updates the position of the panner in the image object
|                so that if there is other devices that have any access
|                access callbacks we will know it.
|
|         Input: widget      - image widget
|                client_data - client data (unused)
|                event       - type of event that triggered us
|        Output: dispatch    - whether to continue dispatching this event
|    Written By: Mark Young
|          Date: Aug 19, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void UpdatePosition(
   xvobject object,
   kaddr    client_data,
   XEvent   *event,
   int      *dispatch)
{
	int x, y, xpos, ypos, update = FALSE;
	XvwPanIconWidget xwid = (XvwPanIconWidget) xvw_widget(object);

 
	if (event->type == MotionNotify)
	{
	   x = event->xmotion.x; y = event->xmotion.y;
	   if (event->xmotion.state == 0) return;
	}
	else if (event->type == ButtonPress || event->type == ButtonRelease)
	{
	   x = event->xbutton.x; y = event->xbutton.y;
	   if (event->type == ButtonRelease)
	      update = TRUE;
	}
	else
	   return;

	xpos = x / xwid->panicon.factor - ((int) xwid->panicon.width)/2;
	ypos = y / xwid->panicon.factor - ((int) xwid->panicon.height)/2;
	xwid->panicon.xpos = xpos;
	xwid->panicon.ypos = ypos;
	ChangePanIconPosition((Widget) xwid, update);
}

/*-----------------------------------------------------------
|
|  Routine Name: PanIconCallback
|
|       Purpose: This redisplays the position of the object being tracked
|                 
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 17, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void PanIconCallback(
   kobject object,
   char    *segment,
   Widget  widget,
   kdms_callback *cb)
{
	XvwPanIconWidget xwid = (XvwPanIconWidget) widget;

	int x, y, w, h, d, reload, update;
	xvobject panicon = xvw_object(widget);


	/*
	 *  Make sure we aren't here because of ourself.  In other words, we
	 *  don't want to have the PanIcon update when kpds_get_data() is
	 *  performmed by the PanIcon, only other visual objects, such as
	 *  Image objects.
	 */
	if (object == cb->object)
	   return;

	xvw_get_type(object, &reload, &update);
	if (reload != IMAGE_OBJECT_RELOAD || update != IMAGE_REFRESH_UPDATE)
	   return;

	kpds_get_attribute(cb->object, KPDS_VALUE_SIZE, &w, &h, NULL,NULL,NULL);
	if (cb->begin.w == 0 && cb->begin.h == 0 &&
	    (w-10) <= (cb->end.w - cb->begin.w) &&
	    (h-10) <= (cb->end.h - cb->begin.h))
	{
	   xvw_unmap(panicon);
	   return;
	}
	else
	{
	   xvw_manage(panicon);
	   xvw_map(panicon);
	}
	x = cb->begin.w / xwid->panicon.factor;
	y = cb->begin.h / xwid->panicon.factor;
	w = (cb->end.w - cb->begin.w) / xwid->panicon.factor;
	h = (cb->end.h - cb->begin.h) / xwid->panicon.factor;
	d = cb->begin.d;

	xvw_set_attributes(xvw_object(widget),
		XVW_IMAGE_BANDNUM,  d,
		XVW_PANICON_XPOS,   x,
		XVW_PANICON_YPOS,   y,
		XVW_PANICON_WIDTH,  w,
		XVW_PANICON_HEIGHT, h,
		NULL);
}


/************************************************************
*
*  Routine Name: xvw_create_panicon - create a panicon object
*
*       Purpose: A panicon object provides a mechanism with which to roam 
*                around in an image which is too large to be accomodated
*                by the image object in which it is displayed.  Thus,
*                the panicon object must be used in conjunction 
*                with an image object on which it will pan.  
*
*                Note that the 
*                \f(CWXVW_IMAGE_IMAGEOBJ\fP
*                attribute is used to specify the data object on which 
*                the panicon should pan; this should be the same data 
*                object as the one displayed in the image object.
*
*                The panicon object will \fInot\fP be mapped unless
*                it is needed;  a created panicon object will remain
*                "invisible" if the image being displayed 
*                is small enough to fit within the image object.  
*                Thus, if an application using an image object expects
*                to \fIever\fP receive a request to display an image
*                which is larger than the image object size, a pan
*                icon object should \fIalways\fP be created;  it will
*                not appear unless it is necessary.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the panicon
*                         object (for use in app-defaults files, etc)
*
*        Output: 
*       Returns: Returns the panicon object, or NULL upon failure
*  Restrictions:
*    Written By: Mark Young
*          Date: Oct 08, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_panicon(
   xvobject parent,
   char     *name)
{
	xvobject object;

	object = xvw_create(parent, FALSE, FALSE, name, PanIconWidgetClass);
	xvw_unmap(object);
	if (object != NULL)
	{
	   xvw_set_attributes(object,
                XVW_MENUABLE,     TRUE,      /* menuable       */
                XVW_RESIZABLE,    FALSE,     /* resizable      */
                XVW_SELECTABLE,   TRUE,      /* selectable     */
		NULL);
	}
	return(object);
}
