/*
  * Khoros: $Id: Pseudo.c,v 1.19.6.5 1997/05/24 19:03:47 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Pseudo.c,v 1.19.6.5 1997/05/24 19:03:47 mirage Exp $";
#endif


/*
 * Copyright (C) 1993
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Pseudo Widget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Destroy()
   >>>>                 SetValues()
   >>>>                 ValueCallback()
   >>>>                 UpdateValue()
   >>>>                 PaletteCallback()
   >>>>                 ResetCallback()
   >>>>                 ChangePseudoIndex()
   >>>>                 SelectCallback()
   >>>>   Public:
   >>>>                 xvw_create_pseudo()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/PseudoP.h>
#include <xvisual/Palette.h>


static void ClassInitialize PROTO((void));
static void Initialize	    PROTO((Widget, Widget, ArgList, Cardinal *));
static Boolean SetValues    PROTO((Widget, Widget, Widget, ArgList,
				   Cardinal *));
static void Destroy         PROTO((Widget));
static void ValueCallback     PROTO((xvobject, kaddr, kaddr));
static void UpdateValue       PROTO((XvwPseudoWidget, xvobject, int));
static void PaletteCallback   PROTO((xvobject, kaddr, kaddr));
static void ResetCallback     PROTO((xvobject, kaddr, kaddr));
static int  ChangePseudoIndex PROTO((xvobject, char *, kaddr));
static void SelectCallback PROTO((kobject, char *, Widget, kdms_callback *));

#undef kwidget
#undef kwidgetclass
#undef kconstraint

#define kwidget(widget)      (XvwPseudoWidget) (widget)
#define kwidgetclass(widget) (XvwPseudoWidgetClass) (widget)->core.widget_class
#define kconstraint(widget)  (XvwPseudoWidgetConstraints) (widget)->core.constraints

/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_PSEUDO_REDVAL,       NULL, XtRInt, NULL},
{XVW_PSEUDO_GREENVAL,     NULL, XtRInt, NULL},
{XVW_PSEUDO_BLUEVAL,      NULL, XtRInt, NULL},
{XVW_PSEUDO_ALPHAVAL,     NULL, XtRInt, NULL},
{XVW_PSEUDO_USE_ALPHA,    NULL, XtRInt, XtRBoolean},
{XVW_PSEUDO_INDEX1,       NULL, XtRInt, NULL},
{XVW_PSEUDO_INDEX2,       NULL, XtRInt, NULL},
{XVW_PSEUDO_INDEXLIST,    NULL, XtRPointer, NULL},
{XVW_PSEUDO_INDEXNUM,     NULL, XtRInt, NULL},
{XVW_PSEUDO_SHOW_PALETTE, NULL, XtRInt, XtRBoolean},
{XVW_PSEUDO_UPDATE_ONADD, NULL, XtRInt, XtRBoolean},
{XVW_PSEUDO_CALLBACK,     NULL, XtRCallback, NULL},
{XVW_PSEUDO_PALETTE_OBJECT, NULL, XtRPointer, NULL},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwPseudoWidgetRec, pseudo.field)

static XtResource resources[] = { 
{XVW_PSEUDO_INDEX1, NULL, XtRInt, sizeof(int),
      offset(index1), XtRImmediate, (XtPointer) -1},
{XVW_PSEUDO_INDEX2, NULL, XtRInt, sizeof(int),
      offset(index2), XtRImmediate, (XtPointer) -1},
{XVW_PSEUDO_INDEXLIST, NULL, XtRPointer, sizeof(XtPointer),
      offset(indexes), XtRImmediate, (XtPointer) NULL},
{XVW_PSEUDO_INDEXNUM, NULL, XtRInt, sizeof(int),
      offset(num), XtRImmediate, (XtPointer) 0},
{XVW_PSEUDO_REDVAL, NULL, XtRInt, sizeof(int),
      offset(redval), XtRImmediate, (XtPointer) 0},
{XVW_PSEUDO_GREENVAL, NULL, XtRInt, sizeof(int),
      offset(greenval), XtRImmediate, (XtPointer) 0},
{XVW_PSEUDO_BLUEVAL, NULL, XtRInt, sizeof(int),
      offset(blueval), XtRImmediate, (XtPointer) 0},
{XVW_PSEUDO_ALPHAVAL, NULL, XtRInt, sizeof(int),
      offset(alphaval), XtRString, (XtPointer) 0},
{XVW_PSEUDO_USE_ALPHA, NULL, XtRBoolean, sizeof(Boolean),
      offset(use_alpha), XtRImmediate, (XtPointer) FALSE},
{XVW_PSEUDO_SHOW_PALETTE, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_palette), XtRImmediate, (XtPointer) TRUE},
{XVW_PSEUDO_UPDATE_ONADD, NULL, XtRBoolean, sizeof(Boolean),
      offset(update_onadd), XtRImmediate, (XtPointer) FALSE},
{XVW_PSEUDO_CALLBACK, NULL, XtRCallback, sizeof(XtCallbackList),
      offset(pseudo_callbacks), XtRCallback, (XtPointer) NULL},
{XVW_PSEUDO_PALETTE_OBJECT, NULL, XtRObject, sizeof(xvobject),
        offset(palette), XtRImmediate, (XtPointer) NULL},

{XVW_SHADOW_TYPE, NULL, XtRInt, sizeof(int),
        XtOffsetOf(XvwPseudoWidgetRec, manager.shadow_type),
        XtRImmediate, (XtPointer) KSHADOW_OUT},
{XVW_PREFERRED_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwManagerWidgetRec, manager.preferred_width),
        XtRImmediate, (XtPointer) 300},
{XVW_PREFERRED_HEIGHT, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwManagerWidgetRec, manager.preferred_height),
        XtRImmediate, (XtPointer) 175},
{XVW_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(CoreRec, core.width),
        XtRImmediate, (XtPointer) 300},
{XVW_HEIGHT, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(CoreRec, core.height),
        XtRImmediate, (XtPointer) 175},
};
#undef offset


#define SUPERCLASS (&xvwColorWidgetClassRec)

XvwPseudoWidgetClassRec xvwPseudoWidgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Pseudo",				/* class_name		  */
    sizeof(XvwPseudoWidgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    XtInheritTranslations,		/* tm_table		  */
    NULL,				/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,	            		/* subresources       */
    0,					/* subresources_count */
    sizeof(XvwManagerWidgetConstraintsRec),/* constraint_size    */
    NULL,				/* initialize         */
    NULL,                               /* destroy            */
    NULL,				/* set_values         */
    NULL,                               /* extension          */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			/* pre-layout routine     */
    XtInheritLayout,			/* post-layout routine    */
    XtInheritChangeSel,		        /* change selected proc   */
    XtInheritEraseSel,			/* erase selected proc    */
    XtInheritRefreshSel,		/* refresh selection proc */
    XtInheritResize,			/* resize		  */
    XtInheritGeometryManager,  		/* geometry_manager	  */
  }, /* XvwManagerWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwGraphicsWidgetClass fields initialization */
  {
    XtInheritColorReload,		/* reload color proc             */
  },  /* XvwColorWidgetClass fields initialization */
  {
    NULL,                               /* field not used    */
  },  /* XvwPseudoWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwPseudoWidgetClass for public consumption
 */
WidgetClass xvwPseudoWidgetClass = (WidgetClass) &xvwPseudoWidgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of pseudo widget class has been created.
|                It will initialize all the class attributes.
|
|    Written By: Mark Young 
|          Date: Nov 17, 1992
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
        xvw_init_attributes(xvwPseudoWidgetClass, attributes,
		      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/Pseudo.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Pseudo");

	xvw_define_attributes(xvwPseudoWidgetClass,
	  XVW_PSEUDO_ADD,    XtRInt, ChangePseudoIndex, NULL,
	  XVW_PSEUDO_DELETE, XtRInt, ChangePseudoIndex, NULL,
	  XVW_PSEUDO_CLEAR,  XtRInt, ChangePseudoIndex, NULL,
	  NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|	Purpose: This method will set up the initial pseudo color
|                widget instance.
|
|	  Input: request - not used
|                new1     - widget instance after initialization
|    Written By: Mark Young
|          Date: May 26, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwPseudoWidget xwid = kwidget(new1);

	xvobject blue_offset; 
	xvobject object = xvw_object(new1);


	/*
	 *  Create the palette with the reset button
	 */
	xwid->pseudo.palette = xvw_create_palette(object, "palette");
	xwid->pseudo.reset   = xvw_create_button(object, "reset");

	/*
	 * create double objects with which to set Red, Green, and Blue
	 */
	xwid->pseudo.red    = xvw_create_integer(object, "red");
	xwid->pseudo.blue   = xvw_create_integer(object, "blue");
	xwid->pseudo.green  = xvw_create_integer(object, "green");

	if (xwid->pseudo.use_alpha)
	   blue_offset = xwid->pseudo.alpha = xvw_create_integer(object,"alpha");
	else blue_offset = NULL;


	/*
	 *  Initialize the palette to extend the width of the palette
	 */
	if (xwid->pseudo.show_palette)
	{
	   xvw_set_attributes(xwid->pseudo.reset,
		              XVW_BELOW,   NULL,
		              XVW_LEFT_OF, NULL,
			      XVW_LABEL,  "Reset Range",
			      NULL);
	   xvw_add_callback(xwid->pseudo.reset, XVW_BUTTON_SELECT,
			    ResetCallback, xwid);

	   xvw_set_attributes(xwid->pseudo.palette,
		XVW_BELOW,		xwid->pseudo.reset,
		XVW_ABOVE,		xwid->pseudo.red,
		XVW_TACK_EDGE,		KMANAGER_TACK_ALL,
		XVW_PALETTE_TYPE,       KPALETTE_TYPE_COLORCELL, 
		NULL);
	}

        /*
         *  Make sure the scrollbars extends the height of the
         *  pseudo widget.
         */
        xvw_set_attributes(xwid->pseudo.red,
		XVW_BACKGROUND_COLOR,	"#ff0000",
		XVW_ABOVE,	        xwid->pseudo.green,
		XVW_RIGHT_OF,	        NULL,
		XVW_LEFT_OF,	        NULL,
		XVW_TACK_EDGE,	        KMANAGER_TACK_HORIZ,
		XVW_CHAR_HEIGHT,        1.0,
		XVW_INTEGER_LABEL,      "Red:  ",
		XVW_INTEGER_VALUE,      0,
		XVW_INTEGER_MINVALUE,   0,
		XVW_INTEGER_MAXVALUE,   255,
		NULL);

        xvw_set_attributes(xwid->pseudo.green,
		XVW_BACKGROUND_COLOR,	"#00ff00",
		XVW_ABOVE,	        xwid->pseudo.blue,
		XVW_RIGHT_OF,	        NULL,
		XVW_LEFT_OF,	        NULL,
		XVW_TACK_EDGE,	        KMANAGER_TACK_HORIZ,
                XVW_INTEGER_LABEL,      "Green:",
		XVW_CHAR_HEIGHT,        1.0,
                XVW_INTEGER_VALUE,      0,
                XVW_INTEGER_MINVALUE,   0,
                XVW_INTEGER_MAXVALUE,   255,
		NULL);

        xvw_set_attributes(xwid->pseudo.blue,
		XVW_BACKGROUND_COLOR,	"#0000ff",
		XVW_ABOVE,	        blue_offset,
		XVW_RIGHT_OF,	        NULL,
		XVW_LEFT_OF,	        NULL,
		XVW_TACK_EDGE,	        KMANAGER_TACK_HORIZ,
		XVW_CHAR_HEIGHT,        1.0,
                XVW_INTEGER_LABEL,      "Blue: ",
                XVW_INTEGER_VALUE,      0,
                XVW_INTEGER_MINVALUE,   0,
                XVW_INTEGER_MAXVALUE,   255,
		NULL);

	if (xwid->pseudo.use_alpha)
           xvw_set_attributes(xwid->pseudo.alpha,
		XVW_ABOVE,	        NULL,
		XVW_RIGHT_OF,	        NULL,
		XVW_LEFT_OF,	        NULL,
		XVW_TACK_EDGE,	        KMANAGER_TACK_HORIZ,
		XVW_CHAR_HEIGHT,        1.0,
                XVW_INTEGER_LABEL,      "Alpha:",
                XVW_INTEGER_VALUE,      0,
                XVW_INTEGER_MINVALUE,   0,
                XVW_INTEGER_MAXVALUE,   255,
		NULL);
	/*
	 *  Add the callbacks for the red, green, and blue integer objects 
	 *  that will be used to change the color palette.
	 */
	xvw_insert_callback(xwid->pseudo.red, XVW_INTEGER_CALLBACK,
			 FALSE, ValueCallback, (XtPointer) xwid);
	xvw_insert_callback(xwid->pseudo.green, XVW_INTEGER_CALLBACK,
			 FALSE, ValueCallback, (XtPointer) xwid);
	xvw_insert_callback(xwid->pseudo.blue, XVW_INTEGER_CALLBACK,
			 FALSE, ValueCallback, (XtPointer) xwid);

	if (xwid->pseudo.use_alpha)
	   xvw_insert_callback(xwid->pseudo.alpha, XVW_INTEGER_CALLBACK,
	   	         FALSE, ValueCallback, (XtPointer) xwid);

	xvw_insert_callback(xwid->pseudo.palette, XVW_PALETTE_CALLBACK,
			 FALSE, PaletteCallback, (XtPointer) xwid);

	if (!xwid->pseudo.show_palette)
	{
	   xvw_unmanage(xwid->pseudo.palette);
	   xvw_unmanage(xwid->pseudo.reset);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: Frees memory associated with Pseudo object
|
|         Input: widget - the pseudo widget being destroyed
|    Written By: Danielle Argiro
|          Date: Sept 21, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
        XvwPseudoWidget xobj = (XvwPseudoWidget) widget;

        karray_free(xobj->pseudo.indexes, KINT, 
		    (size_t) xobj->pseudo.num, NULL);
}



/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: Determine what the widget should do by
|		 comparing the current values verse the
|		 new settings.
|
|         Input: current - the widget containing current settings 
|		 request - the widget containing requested settings
|		 new1	 - the widget processed through all set values methods 
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 17, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	int redisplay = FALSE;
	XvwPseudoWidget cxwid = kwidget(current);
	XvwPseudoWidget nxwid = kwidget(new1);


	/*
	 *  Change the Scrollbar, and it's values, for the appropriate
	 *  red, green, or blue components.
	 */
	if (cxwid->pseudo.redval != nxwid->pseudo.redval)
	   UpdateValue(nxwid, nxwid->pseudo.red, nxwid->pseudo.redval);

	if (cxwid->pseudo.greenval != nxwid->pseudo.greenval)
	   UpdateValue(nxwid, nxwid->pseudo.green, nxwid->pseudo.greenval);

	if (cxwid->pseudo.blueval != nxwid->pseudo.blueval)
	   UpdateValue(nxwid, nxwid->pseudo.blue, nxwid->pseudo.blueval);

	if (cxwid->pseudo.alphaval != nxwid->pseudo.alphaval)
	   UpdateValue(nxwid, nxwid->pseudo.alpha, nxwid->pseudo.alphaval);

	if (cxwid->color.object != nxwid->color.object)
	{
	   xvw_set_attribute(nxwid->pseudo.palette, XVW_COLOR_COLOROBJ,
			nxwid->color.object);
	   kpds_add_value_callback(nxwid->color.object, KPDS_CALLBACK_ACCESS,
			(kfunc_void) SelectCallback, (kaddr) new1);
	}

	if (cxwid->pseudo.show_palette != nxwid->pseudo.show_palette)
	{
	   xvobject palette = nxwid->pseudo.palette;
	   xvobject reset = nxwid->pseudo.reset;

	   if (!nxwid->pseudo.show_palette)
	   {
	      xvw_unmanage(palette);
	      xvw_unmanage(reset);
	   }
	   else
	   {
	      xvw_manage(palette);
	      xvw_manage(reset);
	   }
	   redisplay = TRUE;
	}

	if (cxwid->color.alloc_policy != nxwid->color.alloc_policy)
	{
	   xvw_set_attribute(nxwid->pseudo.palette, XVW_COLOR_ALLOC_POLICY,
			     nxwid->color.alloc_policy);
	}
	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: ValueCallback
|
|       Purpose: This routine will change the (R, G, B, or Alpha) 
|                component according to the new int value provided.
|
|         Input: object      - the scrollbar being manipulated
|		 client_data - the pseudo widget
|		 call_data   - int representing the scroll position
|    Written By: Mark Young 
|          Date: Jun 06, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ValueCallback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	XvwPseudoWidget   xwid = kwidget(client_data);
	xvw_pseudo_struct pseudo;
	int               value;

	/*
	 *  The movement is really a double that represents the new
	 *  value of the (RGB) component.
	 */
	pseudo.object = object;
	xvw_get_attribute(object, XVW_INTEGER_VALUE, &value);
	pseudo.value   = value;

	if (object == xwid->pseudo.red)
	   pseudo.type = KPSEUDO_RED;
	else if (object == xwid->pseudo.green)
	   pseudo.type = KPSEUDO_GREEN;
	else if (object == xwid->pseudo.blue)
	   pseudo.type = KPSEUDO_BLUE;
	else if (object == xwid->pseudo.alpha)
	   pseudo.type = KPSEUDO_ALPHA;

	UpdateValue(xwid, object, (int) value);
	XtCallCallbackList((Widget) xwid, xwid->pseudo.pseudo_callbacks,
			(XtPointer) &pseudo);

	UpdateValue(xwid, object, (int) value);
}

/*-----------------------------------------------------------
|
|  Routine Name: UpdateValue
|
|       Purpose: This routine will update the (R, G, B, or Alpha)
|                value displayed according to the new value.
|
|         Input: xwid	- the pseudo widget
|		 object - the scrollbar being manipulated
|		 value  - the new scrollbar value
|    Written By: Mark Young 
|          Date: Jun 06, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void UpdateValue(
   XvwPseudoWidget xwid,
   xvobject	   object,
   int		   value)
{
	int    i, num, indx, pos;
	static int active = FALSE;
	Widget widget = (Widget) xwid;

	/*
	 *  Make sure we aren't in the middle of callback...
	 */
	if (active)
	   return;
	else
	   active = TRUE;

	/*
	 *  The movement is really a double that represents the new
	 *  value of the (RGB) component.
	 */
	if (object == xwid->pseudo.red)
	{
	   if (xwid->pseudo.redval == value)
	   {
	      active = FALSE;
	      return;
	   }
	   pos   = 0;
	   xwid->pseudo.redval = value;
	}
	else if (object == xwid->pseudo.green)
	{
	   if (xwid->pseudo.greenval == value)
	   {
	      active = FALSE;
	      return;
	   }
	   pos   = 1;
	   xwid->pseudo.greenval = value;
	}
	else if (object == xwid->pseudo.blue)
	{
	   if (xwid->pseudo.blueval == value)
	   {
	      active = FALSE;
	      return;
	   }
	   pos   = 2;
	   xwid->pseudo.blueval = value;
	}
	else if (object == xwid->pseudo.alpha)
	{
	   if (xwid->pseudo.alphaval == value)
	   {
	      active = FALSE;
	      return;
	   }
	   pos   = 2;
	   xwid->pseudo.alphaval = value;
	}
	else
	{
	   active = FALSE;
	   return;
	}
	xvw_set_attribute(object, XVW_INTEGER_VALUE, value);

	/*
         *  If we have indexes to update and a valid pseudocolor object,
	 *  then go ahead and update the data object's mapdata with our new
	 *  pseudo color value(s).
	 */
	if (xwid->pseudo.indexes != NULL && xwid->color.object != NULL)
	{
	   XColor *xcolors = ColorGetXColors(widget);

	    /*
	     *  Loop thru all the pixel values in the pseudo color list 
	     *  and put the new pseudo color value.
	     */
	    num = value << 8;
	    for (i = 0; i < xwid->pseudo.num; i++)
	    {
	        indx = xwid->pseudo.indexes[i];

		if (pos == 0)
		   xcolors[indx].red = num;
		else if (pos == 1)
		   xcolors[indx].green = num;
		else if (pos == 2)
		   xcolors[indx].blue = num;

		ColorStore(widget, indx, NULL);
	        kinfo(KDEBUG,"pseudo: %d,%d: %g", pos, indx, value);
	    }
	}
	active = FALSE;
}

/*-----------------------------------------------------------
|
|  Routine Name: PaletteCallback
|
|       Purpose: This routine will be called when the palette callback is
|		 activated.
|
|         Input: object      - which palette being manipulated
|		 client_data - the palette widget
|		 call_data   - int representing the index
|    Written By: Mark Young 
|          Date: Jun 06, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void PaletteCallback(
   xvobject palette,
   kaddr    client_data,
   kaddr    call_data)
{
	XvwPseudoWidget xwid = kwidget(client_data);
	xvobject object = (xvobject) client_data;
	int indx = *((int *) call_data);

	size_t num   = xwid->pseudo.num; 
	int *indexes = xwid->pseudo.indexes;


	if (karray_locate(indexes, KINT, indx, num) == -1)
	   xvw_set_attribute(object, XVW_PSEUDO_ADD, indx);
	else
	   xvw_set_attribute(object, XVW_PSEUDO_DELETE, indx);
}

/*-----------------------------------------------------------
|
|  Routine Name: ResetCallback
|
|       Purpose: This routine will be called when the 
|		 user clicks on the "Reset" button.
|
|         Input: object      - the reset button
|                client_data - the pseudo widget
|                call_data   - not used
|    Written By: Danielle
|          Date: Jun 07, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ResetCallback(
   xvobject button,
   kaddr    client_data,
   kaddr    call_data)
{
        xvobject object = (xvobject) client_data;

        xvw_set_attribute(object, XVW_PSEUDO_CLEAR, TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ChangePseudoIndex
|
|       Purpose: ChangePseudoIndex is used to set the indexes associated
|		 with the pseudo widget.
|
|         Input: object    - the object in which we will be getting the
|			     filename
|		 attribute - the old object to be closed
|		 calldata  - the filename to be set
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 21, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static int ChangePseudoIndex(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	XvwPseudoWidget xwid = (XvwPseudoWidget) xvw_widget(object);
	int indx = *((int *) calldata);

	XColor *xcolors;
	Widget widget = (Widget) xwid;
	int    val;
	size_t  num = xwid->pseudo.num; 
	int    *indexes = xwid->pseudo.indexes;


	if (kstrcmp(attribute, XVW_PSEUDO_ADD) == 0)
	{
	   if (karray_locate(indexes, KINT, indx, num) == -1)
	   {
	      if (xwid->pseudo.update_onadd && indexes != NULL)
	      {
		 xcolors = ColorGetXColors(widget);
		 ColorStore(widget, indx, &xcolors[indexes[0]]);
	      }
	      else if (!indexes && (xcolors = ColorGetXColors(widget)) != NULL)
	      {
		 val = (int) ((int) xcolors[indx].red >> 8);
		 UpdateValue(xwid, xwid->pseudo.red, val);

		 val = (double) ((int) xcolors[indx].green >> 8);
		 UpdateValue(xwid, xwid->pseudo.green, val);

		 val = (double) ((int) xcolors[indx].blue >> 8);
		 UpdateValue(xwid, xwid->pseudo.blue, val);
	      }
	      indexes = (int *)karray_insert(indexes, KINT, indx, num++,
				      KLIST_TAIL, TRUE);
	      xvw_set_attribute(xwid->pseudo.palette, XVW_PALETTE_ADD, indx);
	   }
	}
        else if (kstrcmp(attribute, XVW_PSEUDO_DELETE) == 0)
	{
	   if (karray_locate(indexes, KINT, indx, num) != -1)
	   {
	      indexes = (int *)karray_delete(indexes, KINT, indx, num--);
	      xvw_set_attribute(xwid->pseudo.palette, XVW_PALETTE_DELETE, indx);
	   }
	}
        else if (kstrcmp(attribute, XVW_PSEUDO_CLEAR) == 0)
	{
	   kfree_and_NULL(indexes);
	   indexes = NULL; num = 0;
	   xvw_set_attribute(xwid->pseudo.palette, XVW_PALETTE_CLEAR, TRUE);
	   xwid->pseudo.index1 = -1;
	   xwid->pseudo.index2 = -1;
	}
	else
	{
	   kerror(XVISUAL, "ChangePseudoIndex", "invalid attribute %s",
			attribute);
	   return(FALSE);
	}
	xwid->pseudo.num = num;
	xwid->pseudo.indexes = indexes;
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: SelectCallback
|
|       Purpose: This adds the selected value
|                 
|         Input: 
|        Output:
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Jun 17, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void SelectCallback(
   kobject object,
   char    *segment,
   Widget  widget,
   kdms_callback *cb)
{
	int update;
	xvw_get_type(object, NULL, &update);
	if (update != BUTTONPRESS_EVENT_UPDATE)
	   return;
}


/************************************************************
*
*  Routine Name: xvw_create_pseudo - create a pseudo xvobject
*
*       Purpose: The psuedocolor visual object provides a method of changing 
*                the colormap of another visual object.  It displays the colors 
*                associated with the pixel values of the displayed object
*                (as a color palette, colorcell grid, or color wheel).  Under
*		 the color display is a set of three scrollbars, representing
*		 Red, Green, and Blue.
*
*		 Select pixels for color change from the color
*		 palette, colorcell grid, or color wheel. Next, 
*                move the thumbs of the three scrollbars underneath to change 
*		 the red, green, and blue values defining the color in which
*		 the selected pixels appear.  If desired, specific values for
*		 red, green, and blue may be typed directly into the text
*		 parameter boxes that appear to the right of the scrollbars.
*		 A "Reset" button at the top of the pseudocolor object
*		 provides a convenient way to un-select previously selected
*		 pixels.
*
*		 The pixels to be changed are selected from the colorcell grid
*		 (or color palette, or color wheel).  Pixels may be selected
*		 for color change using the following methods.  These 
*		 descriptions assume use of the colorcell grid, but pixel 
*		 selection for the color palette and color wheel work similarly.
*		 Pixels may be selected individually, or in ranges.  Note that
*		 \fIall\fP selected pixels on the colorcell grid will have their
*		 color changed when you set the red, green, and blue values.
*		 The "Reset" button un-selects \fIall\fP selected pixels
*		 in the colorcell grid.
*		 .sp
*		 1) To select a single pixel for color change, click on
*		    the color cell representing the desired pixel.  The 
*		    color cell will appear intented and stippled when it
*		    is selected.  To unselect the pixel, click the mouse
*		    button on it again, or use the "Reset" button.
*		 .sp
*		 2) To select a range of pixels for color change, click
*		    on a pixel at the end of the range to be changed.  Holding
*		    the button down, move the mouse to the desired end point;
*		    the range may be moved back and forth as long as the mouse
*		    button is held down. Releasing the mouse button sets the
*		    range. Multiple ranges may be selected by repeating this
*		    process on previously unselected pixels.  Portions of a 
*		    selected range may be unselected by repeating on previously
*		    selected pixels.  To unselect the range, repeat the 
*		    selection process, or use the "Reset" button.
*
*		 The 
*                \f(CWXVW_COLOR_COLOROBJ\fP
*                attribute is used to specify the data object whose colormap 
*                the psuedocolor object will be used to modify.
*
*                Note that pseudocolor should not be done on RGB images when
*                the calling application is displayed on an 8-bit screen.  
*                This is because a mapping transformation is used in order 
*                to convert the RGB image into an 8-bit representation for 
*                visual display on an 8-bit screen.
*
*                Because of this transformation, pseudocoloring an RGB image on 
*                an 8-bit screen will not make sense;  any changes to the 
*                colormap made via thresholding would alter the 24-bit to 
*                8-bit transformation (a 3-3-2 RGB map is used), rather than 
*                the RGB values themselves, producing an unexpected result.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the pseudo
*                         object (for use in app-defaults files, etc)
*        Output: 
*	Returns: The pseudocolor object on success, NULL otherwise
*  Restrictions:
*    Written By: Mark Young
*          Date: Nov 17, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_pseudo(
   xvobject parent,
   char     *name)
{
	xvobject object;


	/*
	 *  Call the xvw_create() routine to do the actual creation.
	 */
	object = xvw_create(parent, FALSE, TRUE, name, PseudoWidgetClass);
	return(object);
}
