/*
  * Khoros: $Id: String.c,v 1.18.2.2 1997/04/29 22:30:34 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: String.c,v 1.18.2.2 1997/04/29 22:30:34 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		String Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Destroy()
   >>>>                 Redisplay()
   >>>>                 SetValues()
   >>>>                 RecomputePosition()
   >>>>                 ChangeGeometry()
   >>>>                 CompileString()
   >>>>                 RecomputeSize()
   >>>>   Public:
   >>>>                 xvw_create_string()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include <xvisual/StringP.h>


static void ClassInitialize	 PROTO((void));
static void Initialize		 PROTO((Widget, Widget, ArgList, Cardinal *));
static void Destroy   		 PROTO((Widget));
static void Redisplay		 PROTO((Widget, XEvent *, Region));
static Boolean SetValues	 PROTO((Widget, Widget, Widget, ArgList,
					Cardinal *));
static void RecomputePosition	 PROTO((Widget));
static void ChangeGeometry	 PROTO((Widget));
static void CompileString	 PROTO((Widget));
static void RecomputeSize	 PROTO((Widget));


/*--------------------------------------------------------
|
|   Full class attributes

|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_STRING_XPLACEMENT,         NULL,    XtRDouble,    NULL}, 
{XVW_STRING_YPLACEMENT,         NULL,    XtRDouble,    NULL}, 
{XVW_STRING_STRING,             NULL,    XtRString,    NULL}, 
{XVW_STRING_STYLE,		NULL,    XtRInt,       NULL}, 
{XVW_STRING_JUSTIFICATION,      NULL,    XtRInt,       NULL}, 
{XVW_STRING_HIGHLIGHT_COLOR,   XVW_STRING_HIGHLIGHT_PIXEL, XtRString, XtRPixel},
{XVW_STRING_HIGHLIGHT_PIXEL,   NULL,    XtRPixel,     NULL},
};

/*--------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwStringGadgetRec, string.field)

static XtResource resources[] = { 
{XVW_STRING_XPLACEMENT, NULL, XtRDouble, sizeof(double), 
      offset(placement.x), XtRString, (XtPointer) "0.0"},
{XVW_STRING_YPLACEMENT, NULL, XtRDouble, sizeof(double), 
      offset(placement.y), XtRString, (XtPointer) "0.0"},
{XVW_STRING_JUSTIFICATION, NULL, XtRInt, sizeof(int), 
      offset(justification), XtRImmediate, (XtPointer) KSTRING_JUSTIFY_LEFT},
{XVW_STRING_STRING, NULL, XtRString, sizeof(String),
      offset(string), XtRImmediate, (XtPointer) NULL},
{XVW_STRING_STYLE, NULL, XtRInt, sizeof(int),
      offset(style), XtRImmediate, (XtPointer) KSTRING_STYLE_PLAIN},
{XVW_STRING_STRING, NULL, XtRString, sizeof(String),
      offset(string), XtRImmediate, (XtPointer) NULL},
{XVW_STRING_HIGHLIGHT_PIXEL, XVW_STRING_HIGHLIGHT_COLOR,XtRPixel,sizeof(Pixel),
      offset(highlight_color.pixel), XtRString, XtDefaultBackground},

{XVW_FOREGROUND_PIXEL, NULL, XtRPixel, sizeof(Pixel),
        XtOffsetOf(XvwStringGadgetRec, manager.foreground),
        XtRString, (XtPointer) "#ffff00"},
{XVW_BACKGROUND_PIXEL, NULL, XtRPixel, sizeof(Pixel),
        XtOffsetOf(XvwStringGadgetRec, manager.background),
        XtRString, (XtPointer) "#000000"},
};
#undef offset


#define SUPERCLASS (&xvwGraphicsGadgetClassRec)

XvwStringGadgetClassRec xvwStringGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "String",				/* class_name		  */
    sizeof(XvwStringGadgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    NULL,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),			/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    FALSE,				/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    XtInheritPicking,			/* pick object proc       */
    XtInheritEraseSel,			/* erase selected proc    */
    XtInheritRefreshSel,		/* refresh selection proc */
    ChangeGeometry,			/* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    RecomputePosition,                  /* recompute position proc    */
  },  /* XvwGraphicsGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwStringGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwStringGadgetClass for public consumption
 */
WidgetClass xvwStringGadgetClass = (WidgetClass) &xvwStringGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of string object class has been created.
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
        xvw_init_attributes(xvwStringGadgetClass, attributes,
	 	      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/String.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/String");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize 
|
|       Purpose: This method will set up the initial values 
|                for the fields of a string object instance.
|                The width, height, ascent, and placement are
|		 all initialized to sane values. 
|
|         Input: request - not used 
|                new1     - object instance after initialization, with
|                          date initialized 
|    Written By: Mark Young
|          Date: Aug 03, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwStringGadget nobj = (XvwStringGadget) new1;
	Colormap colormap;

	/*
	 *  Compute the position of the object in device coordinates
	 */
	nobj->string.lines = NULL;
	nobj->string.numlines = 0;
	nobj->string.string_width   =
	nobj->string.string_height  = 0;
	nobj->string.string_ascent  = 0;
	nobj->string.string_descent = 0;
        kmemset((kaddr) &nobj->string.placement, 0, sizeof(Coord));

	colormap = nobj->object.parent->core.colormap;
	XQueryColor(XtDisplay(new1), colormap, &nobj->string.highlight_color);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: This routine will free the memory associated
|		 with the string before the string is destroyed
|
|         Input: widget - the string object being destroyed
|        Output:
|    Written By: Danielle Argiro & Mark Young
|          Date: Feb 22, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
        XvwStringGadget xobj = (XvwStringGadget) widget;

	/* free memory associated with String object */
	kfree_and_NULL(xobj->string.string);
	karray_free(xobj->string.lines, KSTRING, 
		    (size_t) xobj->string.numlines, NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the string object in response
|                to an expose event. 
|
|         Input: widget - the string object that was exposed 
|                event  - the event that caused the redraw
|                region - the region that was exposed 
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	XvwStringGadget xobj = (XvwStringGadget) widget;

	static Pixel blackpixel = 0;

	unsigned int w, h;
	int  i, x, y, length;

	GC gc = xobj->manager.foreground_gc;
	int numlines     = xobj->string.numlines;
	String  *lines   = xobj->string.lines;
	Display *display = XtDisplay(widget);
	Window  window   = XtWindowOfObject(widget);
	int     justify  = xobj->string.justification;

	/*
	 *  Allocate us a black pixel, if we don't have one yet.
	 */
	if (blackpixel == 0)
	   blackpixel = xvw_allocate_color(xvw_object(widget), "black");

	/*
	 *  Check to see if this if there is a string to draw
	 */
	if (kstrlen(xobj->string.string) == 0 ||  
            !xvw_check_visible(xvw_object(widget)))
	   return;

	/*
	 *  Compute position using the string's current position
	 */
	x = widget->core.x;
	y = widget->core.y;
	w = widget->core.width;
	h = widget->core.height;

	/*
	 *  If string annotation is filled, fill extent with background color
	 */
	if (xobj->graphics.filled)
	{
	   XSetForeground(display, gc, xobj->manager.background);
	   XFillRectangle(display, window, gc, x, y, w, h);
	   XSetForeground(display, gc, xobj->manager.foreground);
	}

	/*
	 * Now, adjust position of string according to the justification
	 */
	if (justify == KSTRING_JUSTIFY_CENTER ||
	    justify == KSTRING_JUSTIFY_BOTTOM ||
	    justify == KSTRING_JUSTIFY_TOP)
	   x += (w/2) - xobj->string.string_width/2;
	else if (justify == KSTRING_JUSTIFY_RIGHT ||
		 justify == KSTRING_JUSTIFY_TOPRIGHT ||
		 justify == KSTRING_JUSTIFY_BOTTOMRIGHT)
	   x += w - xobj->string.string_width;

	if (justify == KSTRING_JUSTIFY_CENTER ||
	    justify == KSTRING_JUSTIFY_LEFT ||
	    justify == KSTRING_JUSTIFY_RIGHT)
	   y += (h/2) - xobj->string.string_height/2;
	else if (justify == KSTRING_JUSTIFY_BOTTOM ||
		 justify == KSTRING_JUSTIFY_BOTTOMLEFT ||
		 justify == KSTRING_JUSTIFY_BOTTOMRIGHT)
	   y += h - xobj->string.string_height;

	y += xobj->string.string_ascent;
	for (i = 0; i < numlines; i++)
	{
	   length = kstrlen(lines[i]);
	   if (xobj->string.style == KSTRING_STYLE_EMPHASIZED)
	   {
	      XSetForeground(display, gc, xobj->string.highlight_color.pixel);
	      XDrawString(display, window, gc, x+1, y, lines[i], length);
	      XSetForeground(display, gc, xobj->manager.foreground);
	      XDrawString(display, window, gc, x, y, lines[i], length);
	   }
	   else if (xobj->string.style == KSTRING_STYLE_EMBOSSED_IN)
	   {
	      XSetForeground(display, gc, blackpixel);
	      XDrawString(display, window, gc, x,   y,   lines[i], length);
	      XDrawString(display, window, gc, x,   y+1, lines[i], length);
	      XSetForeground(display, gc, xobj->string.highlight_color.pixel);
	      XDrawString(display, window, gc, x+2, y+2, lines[i], length);
	      XDrawString(display, window, gc, x+2, y+1, lines[i], length);
	      XSetForeground(display, gc, xobj->manager.foreground);
	      XDrawString(display, window, gc, x+1, y+1, lines[i], length);
	   }
	   else if (xobj->string.style == KSTRING_STYLE_EMBOSSED_OUT)
	   {
	      XSetForeground(display, gc, xobj->string.highlight_color.pixel);
	      XDrawString(display, window, gc, x,   y,  lines[i], length);
	      XDrawString(display, window, gc, x,   y+1, lines[i], length);
	      XSetForeground(display, gc, blackpixel);
	      XDrawString(display, window, gc, x+2, y+2, lines[i], length);
	      XDrawString(display, window, gc, x+2, y+1, lines[i], length);
	      XSetForeground(display, gc, xobj->manager.foreground);
	      XDrawString(display, window, gc, x+1, y+1, lines[i], length);
	   }
           else
	     XDrawString(display, window, gc, x, y, lines[i], length);

	   y += xobj->string.string_ascent+xobj->string.string_descent;
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: If the the justification or postion of the 
|		 string as changed, recalculate the new position. 
|
|         Input: current - the object containing current settings 
|                request - the object containing requested settings
|                new1     - the object processed through all set values methods 
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Aug 03, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwStringGadget cobj = (XvwStringGadget) current;
	XvwStringGadget nobj = (XvwStringGadget) new1;

	Boolean clear_area, redisplay = FALSE;

	if (!GraphicsCheckCoord(cobj->string.placement,nobj->string.placement))
	   GraphicsRecomputePosition(new1, FALSE);

	if (cobj->manager.font != nobj->manager.font)
	{
	   RecomputeSize(new1);
	   redisplay = TRUE;
	}

	if (cobj->string.justification != nobj->string.justification)
	{
	   RecomputeSize(new1);
	   redisplay = TRUE;
	}

	if (cobj->string.style != nobj->string.style)
	{
	   RecomputeSize(new1);
	   redisplay = TRUE;
	}

	if (cobj->string.highlight_color.pixel != 
            nobj->string.highlight_color.pixel)
        {
	   redisplay = TRUE;
	}

	if (cobj->string.string != nobj->string.string)
	{
	   /*
	    *  Free the previously allocated private string
	    */
	   kfree_and_NULL(cobj->string.string);
	   nobj->string.string = kstrdup(nobj->string.string);
	   CompileString(new1);
	   RecomputeSize(new1);
	   redisplay = TRUE;
	}

	if (redisplay && nobj->manager.force_redisplay)
	{
	   clear_area = !nobj->graphics.filled;
	   ManagerObjForceRedisplay(current, new1, clear_area, &redisplay);
	}

	return(redisplay && nobj->manager.mapped);
}

/*-----------------------------------------------------------
|
|  Routine Name: RecomputePosition
|
|       Purpose: Given a new world coordinate string location 
|                recompute the new device coordinate position 
|		 of the string. 
|
|         Input: widget - the string object 
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: January 8, 1996
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RecomputePosition(
   Widget widget)
{
	XvwStringGadget xobj = (XvwStringGadget) widget;

	Real      x, y;

	X2D_convert_point_wc_to_dc(xobj->graphics.id,
 		                   xobj->string.placement, &x, &y);
	RecomputeSize(widget);

	widget->core.x = (Position) x;
        widget->core.y = (Position) y;
        widget->core.width  = widget->core.width;
        widget->core.height = widget->core.height;
}

/*-----------------------------------------------------------
|
|  Routine Name: ChangeGeometry
|
|       Purpose: Given a new device coordinate description of the
|                string, recompute the world coordinate position of 
|                the string.
|
|         Input: widget - the string object
|    Written By: Mark Young
|          Date: Aug 03, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ChangeGeometry(
   Widget widget)
{
	XvwStringGadget xobj = (XvwStringGadget) widget;

	Real         x, y;
	XVGraphicsID id = xobj->graphics.id;

	x = (Real) widget->core.x;
	y = (Real) widget->core.y;

	X2D_convert_point_dc_to_wc(id, x, y, &xobj->string.placement);
}

/*-----------------------------------------------------------
|
|  Routine Name: CompileString - this routine compiles the string
|
|       Purpose: This routine actually compiles the string into a
|		 multiline detab'ed string.
|
|         Input: widget - the string gadet
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 24, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void CompileString(
   Widget widget)
{
	XvwStringGadget xobj = (XvwStringGadget) widget;

	int    i, size, len = 0;
	size_t numlines = 0;
	char   temp[KLENGTH], *string, *tmp, **lines = NULL;

	string = xobj->string.string;
	do
	{
	    tmp = kstrpbrk(string, "\t\r\n");
	    if (!tmp)
	       size = kstrlen(string);
	    else
	       size = tmp-string;

	    kstring_ncopy(string, (size_t) size, &temp[len]); len += size;
	    if (tmp && *tmp == '\t')
	    {
	       size = 8 - (len % 8);
	       for (i = 0; i < size; i++)
		   temp[len++] = ' ';
	       temp[len] = '\0'; tmp++;
	    }
	    else if (!tmp || *tmp == '\n' || *tmp == '\r')
	    {
	       len = 0; if (tmp) tmp++;
	       lines = (char **)karray_insert(lines, KSTRING, kstrdup(temp),
				     numlines++, KLIST_TAIL, TRUE);
	    }
	    string = tmp;
	} while (string != NULL);

	/*
	 * Save the text and number of lines...
	 */
	karray_free(xobj->string.lines, KSTRING, 
		    (size_t) xobj->string.numlines, NULL);
	xobj->string.lines    = lines;
	xobj->string.numlines = numlines;
}

/*-----------------------------------------------------------
|
|  Routine Name: RecomputeSize
|
|       Purpose: Given a new world coordinate position,  
|                recompute the new device coordinate position 
|		 of the string. 
|
|         Input: widget - the string object 
|    Written By: Mark Young
|          Date: Aug 03, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RecomputeSize(
   Widget widget)
{
	XvwStringGadget xobj = (XvwStringGadget) widget;

	String *lines;
	int i, numlines, len, ascent, descent, width;


	/*
	 *  Get the extent of the string
	 */
	len = kstrlen(xobj->string.string);
	if (len > 0 && xobj->manager.font != NULL)
	{
	   lines = xobj->string.lines;
	   numlines = xobj->string.numlines;
	   ascent  = xobj->manager.font->max_bounds.ascent;
	   descent = xobj->manager.font->max_bounds.descent;
	   xobj->string.string_ascent  = ascent;
	   xobj->string.string_descent = descent;
	   xobj->string.string_height  = (ascent + descent) * numlines;

	   for (i = 0, width = 0; i < numlines; i++)
	   {
	      len = XTextWidth(xobj->manager.font, lines[i], 
			       (int) kstrlen(lines[i]));
	      if (len > width) width = len;
	   }
	   xobj->string.string_width = width;
	}
	else
	{
	   xobj->string.string_width   = 1;
	   xobj->string.string_height  = 1;
	   xobj->string.string_ascent  = 0;
	   xobj->string.string_descent = 0;
	}
	widget->core.width  = (Dimension) xobj->string.string_width;
	widget->core.height = (Dimension) xobj->string.string_height;
}

/************************************************************
*
*  Routine Name: xvw_create_string - create a string annotation
*
*       Purpose: A string visual object is used to display a
*		 (non-editable) string of characters.  The string
*		 specified may be of any length, and of multiple
*		 lines.  Carriage returns may be specified with "\n". 
*
*                The location of the upper left corner of the the string 
*		 may be specified in world coordinates, where the world 
*		 coordinate range is dictated by the "controlling" visual
*		 object. By default, the parent of the string object is its 
*		 controlling visual object; this may be changed using the 
*		 XVW_GRAPHICS_ATTACH attribute.
*
*                Alternatively, the location of the string may be
*                specified using device coordinates. As when using world 
*		 coordinates, specify the upper left hand corner of the
*		 string.
*
*                The amount of space used by the string cannot be explicitly 
*		 set by the application;  this is automatically calculated 
*		 according to the font that is used.  Specification of
*                a width and height simply provides a buffer of space around the
*                string;  note that justification will have no
*                effect unless the width and height are specified to be larger
*                than what is actually needed by the string.
*                Width and height may be specified in characters or in pixels.
*
*                In addition to font specification, the string object also supports
*                multiple styles in which the string may
*                appear: available styles include plain, emphasized, embossed in,
*                and embossed out.
*
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the string
*                         object (for use in app-defaults files, etc)
*        Output:
*	Returns: The string object on success, NULL on failure
*  Restrictions:
*    Written By: Mark Young
*  	   Date: Apr 10, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_string(
   xvobject parent,
   char     *name)
{
	xvobject  object;


	/*
	 *  Call the xvw_create() routine to do the actual creation.
	 */
	object = xvw_create(parent, FALSE, TRUE, name, StringGadgetClass);
	if (object != NULL)
	{
	   xvw_set_attributes(object,
		XVW_MENUABLE,     TRUE,     /* menuable       */
		XVW_RESIZABLE,    FALSE,    /* resizable      */
		XVW_SELECTABLE,   TRUE,     /* selectable     */
		NULL);
	}
	return(object);
}
