/*
  * Khoros: $Id: Zoom.c,v 1.20.2.1 1997/04/29 22:30:39 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Zoom.c,v 1.20.2.1 1997/04/29 22:30:39 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Zoom Widget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Redisplay()
   >>>>                 DrawZoomCursor()
   >>>>                 SetValues()
   >>>>                 ZoomCallback()
   >>>>   Public:
   >>>>                 xvw_create_zoom()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include <xvisual/ZoomP.h>

static void ClassInitialize PROTO((void));
static void DrawZoomCursor  PROTO((Widget, int, int));
static void Initialize      PROTO((Widget, Widget, ArgList, Cardinal *));
static void Redisplay       PROTO((Widget, XEvent *, Region));
static Boolean SetValues  PROTO((Widget, Widget, Widget, ArgList, Cardinal *));
static void ZoomCallback  PROTO((kobject, char *, Widget, kdms_callback *));

#define DefaultZoomFactor "8.0"

/*--------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_ZOOM_FACTOR,	     NULL,    XtRDouble,   NULL},
{XVW_ZOOM_LOCATIONMARKER,    NULL,    XtRInt,      NULL},
{XVW_ZOOM_UPDATEMODE,	     NULL,    XtRInt,      NULL},
{XVW_ZOOM_INTERPOLATE,	     NULL,    XtRInt,      NULL},
{XVW_ZOOM_XPOSITION,	     NULL,    XtRInt,      NULL},
{XVW_ZOOM_YPOSITION,	     NULL,    XtRInt,      NULL},
};

/*--------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwZoomWidgetRec, zoom.field)

static XtResource resources[] = { 
{XVW_ZOOM_FACTOR, NULL, XtRDouble, sizeof(double),
      offset(zoomfactor), XtRString, (XtPointer) DefaultZoomFactor},
{XVW_ZOOM_LOCATIONMARKER, NULL, XtRInt, sizeof(int),
      offset(location_marker), XtRImmediate, (XtPointer) KZOOM_LM_BOX},
{XVW_ZOOM_UPDATEMODE, NULL, XtRInt, sizeof(int),
      offset(update_mode), XtRImmediate, (XtPointer) KZOOM_UM_CONTINUOUS},
{XVW_ZOOM_INTERPOLATE, NULL, XtRInt, sizeof(int),
      offset(interpolate), XtRImmediate, (XtPointer) KZERO_ORDER},
{XVW_ZOOM_XPOSITION, NULL, XtRInt, sizeof(int),
      offset(xposition), XtRImmediate, (XtPointer) 0},
{XVW_ZOOM_YPOSITION, NULL, XtRInt, sizeof(int),
      offset(yposition), XtRImmediate, (XtPointer) 0},

{XVW_IMAGE_BACKING, NULL, XtRBoolean, sizeof(Boolean),
      XtOffsetOf(XvwImageWidgetRec, image.backing),
      XtRImmediate, (XtPointer) FALSE},
{XVW_IMAGE_XMAGNIFY, NULL, XtRDouble, sizeof(double),
      XtOffsetOf(XvwImageWidgetRec, image.xmag),
      XtRString, (XtPointer) DefaultZoomFactor},
{XVW_IMAGE_YMAGNIFY, NULL, XtRDouble, sizeof(double),
      XtOffsetOf(XvwImageWidgetRec, image.ymag),
      XtRString, (XtPointer) DefaultZoomFactor},
{NULL, NULL, XtRInt, sizeof(int),
      XtOffsetOf(XvwImageWidgetRec, image.reload_type),
      XtRImmediate, (XtPointer) ZOOM_OBJECT_RELOAD},

{NULL, NULL, XtRPixel, sizeof(Pixel),
      offset(black), XtRString, (XtPointer) "#000000"},
{NULL, NULL, XtRPixel, sizeof(Pixel),
      offset(white), XtRString, (XtPointer) "#ffffff"},
};
#undef offset


#define SUPERCLASS (&xvwImageWidgetClassRec)

XvwZoomWidgetClassRec xvwZoomWidgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Zoom",				/* class_name		  */
    sizeof(XvwZoomWidgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),			/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,	  		        /* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,	            		/* subresources       */
    0,					/* subresources_count */
    sizeof(XvwManagerWidgetConstraintsRec),/* constraint_size    */
    NULL,				/* initialize         */
    NULL,                               /* destroy            */
    NULL,				/* set_values         */
    NULL,                               /* extension          */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			/* pre-layout routine     */
    XtInheritLayout,			/* post-layout routine    */
    XtInheritChangeSel,                 /* change selected proc   */
    XtInheritEraseSel,                  /* erase selected proc    */
    XtInheritRefreshSel,                /* refresh selection proc */
    XtInheritResize,                    /* resize                 */
    XtInheritGeometryManager,           /* geometry_manager       */
  }, /* XvwManagerWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwGraphicsWidgetClass fields initialization */
  {
    XtInheritColorReload,		/* reload color proc             */
  },  /* XvwColorWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwImageWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwZoomWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwZoomWidgetClass for public consumption
 */
WidgetClass xvwZoomWidgetClass = (WidgetClass) &xvwZoomWidgetClassRec;

#undef  kwidget
#define kwidget(widget)      (XvwZoomWidget) (widget)

/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of zoom widget class has been created.
|                It will initialize all the class attributes.
|
|         Input: None 
|
|        Output: None
|
|    Written By: Mark Young and John M. Salas
|          Date: Aug 15, 1992 12:31
| Modifications:
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
        xvw_init_attributes(xvwZoomWidgetClass, attributes,
		      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/Zoom.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Zoom");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial image
|                for an image widget instance.
|
|         Input: request - not used
|                new1     - widget instance after initialization, with
|                          initial image
|    Written By: Mark Young & Danielle Argiro
|          Date: May 16, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
        XvwZoomWidget xwid = kwidget(new1);
        XColor    color;

        /* allocate black specially */
        color.flags = DoRed | DoGreen | DoBlue;
        color.red = color.green = color.blue = 0;
        if (XAllocColor(XtDisplay(new1), new1->core.colormap, &color) != 0)
           xwid->zoom.black = color.pixel;

        /* allocate white specially */
        color.flags = DoRed | DoGreen | DoBlue;
        color.red = color.green = color.blue = 65535;
        if (XAllocColor(XtDisplay(new1), new1->core.colormap, &color) != 0)
           xwid->zoom.white = color.pixel;
}


/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This method draws the zoom cursor after the 
|		 zoomed image has been redisplayed by the Image
|		 class method.
|
|         Input: widget - the zoom widget
|                event  - the event that caused the exposure
|                region - the region that was exposed
|    Written By: Danielle Argiro
|          Date: July 5, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	int x, y;
	XvwZoomWidget xwid = (XvwZoomWidget) widget;


	/*
	 *  Check to see if we are visible.  If not then return...
	 */
	if (!XtWindow(widget) || !xvw_check_visible(xvw_object(widget)))
	   return;

	(*xvwZoomWidgetClass->core_class.superclass->core_class.expose)
                (widget, event, region);

        /*
         * Find center of enlarged pixel at which cursor is to be drawn: 
	 *
	 * this is the position that the cursor is in the non-magnified image 
	 * multiplied by the zoom factor, minus the amount that the zoom window 
         * is offset into the magnified image, plus half the amount of the 
	 * zoom factor (to get position into the middle of the enlarged pixel)
         */
	x = (int) (xwid->zoom.xposition * xwid->zoom.zoomfactor -
                   xwid->image.xoffset + xwid->zoom.zoomfactor/2.0);
	y = (int) (xwid->zoom.yposition * xwid->zoom.zoomfactor -
                   xwid->image.yoffset + xwid->zoom.zoomfactor/2.0);
 
        /*
         * Draw zoom cursor around center of enlarged pixel
         */
        DrawZoomCursor(widget, x, y);
}

/*-----------------------------------------------------------
|
|  Routine Name: DrawZoomCursor
|
|       Purpose: This utility routine draws the zoom cursor
|		 (a cursor in the zoom window which represents 
|		 the "focus" of the zoom window in the image).
|
|         Input: widget - the zoom widget
|		 w, h   - (x,y) location in zoom window (device 
|		          coordinates) at which to draw the cursor
|        Output: 
|       Returns:
|    Written By: Danielle Argiro
|          Date: July 5, 1995
|
------------------------------------------------------------*/

static void DrawZoomCursor(
   Widget widget,
   int    w,
   int    h)
{
	XvwZoomWidget xwid = (XvwZoomWidget) widget;
	Position      x, y; 
	GC            gc = xwid->manager.foreground_gc;
	Window        window   = xvw_window(xvw_object(widget));
	Display       *display = xvw_display(xvw_object(widget));
	int           factor = (int) xwid->zoom.zoomfactor;

	if (factor < 4) factor = 4;

	x = (Position) w;
	y = (Position) h;

	/*
	 * draw cursor as cross
	 */
	if (xwid->zoom.location_marker == KZOOM_LM_CROSS)
	{
    	    XSetForeground(display, gc, xwid->zoom.white);
    	    XDrawLine(display, window, gc, x-1, y-factor/2, x-1, y+factor/2);
    	    XDrawLine(display, window, gc, x+1, y-factor/2, x+1, y+factor/2);
    	    XDrawLine(display, window, gc, x-factor/2, y-1, x+factor/2, y-1);
    	    XDrawLine(display, window, gc, x-factor/2, y+1, x+factor/2, y+1);
    	    XSetForeground(display, gc, xwid->zoom.black);
    	    XDrawLine(display, window, gc, x, y-factor/2, x, y+factor/2);
    	    XDrawLine(display, window, gc, x-factor/2, y, x+factor/2, y);
	}

	/*
	 * draw cursor as box, same size as enlarged pixel
	 */
	else if (xwid->zoom.location_marker == KZOOM_LM_BOX)
	{
	    XSetForeground(display, gc, xwid->zoom.white);
            XDrawRectangle(display, window, gc, x-factor/2, 
			   y-factor/2, (unsigned) factor, (unsigned) factor);
            XSetForeground(display, gc, xwid->zoom.black);
            XDrawRectangle(display, window, gc, x-factor/2+1, y-factor/2+1, 
			   (unsigned) factor-2, (unsigned) factor-2);
	}

	/*
	 * draw cursor as circle, same size as enlarged pixel
	 */
	else if (xwid->zoom.location_marker == KZOOM_LM_CIRCLE ||
		 xwid->zoom.location_marker == KZOOM_LM_DOT)
        {
	    if (xwid->zoom.location_marker == KZOOM_LM_DOT)
		factor = 4;

            XSetForeground(display, gc, xwid->zoom.white);
            XDrawArc(display, window, gc, x-factor/2, y-factor/2, 
		     (unsigned) factor, (unsigned) factor, 360*64, 360*64);
            XSetForeground(display, gc, xwid->zoom.black);
            XDrawArc(display, window, gc, x-factor/2+1, y-factor/2+1, 
		     (unsigned) factor-2, (unsigned) factor-2, 360*64, 360*64);
        }
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the public values
|                of a zoom widget instance.   
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new1     - the widget processed through all set values methods
|
|        Output: None
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Oct 24, 1992 23:59
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwZoomWidget cxwid = (XvwZoomWidget) current;
	XvwZoomWidget nxwid = (XvwZoomWidget) new1;

	int  w, h, d, t, e, *order, *size, xoff, yoff, redisplay = FALSE;
        Real xmin, xmax, ymin, ymax;

	if (cxwid->image.iobject != nxwid->image.iobject)
	{
	   kpds_add_value_callback(nxwid->image.iobject, KPDS_CALLBACK_ACCESS,
				   (kfunc_void) ZoomCallback, (kaddr) new1);
	}

	/*
	 * changing the image object or the zoom factor
	 */
	if (nxwid->image.iobject != NULL &&
	    (cxwid->image.iobject   != nxwid->image.iobject  ||
	     cxwid->zoom.zoomfactor != nxwid->zoom.zoomfactor))
	{
	   if (nxwid->zoom.zoomfactor <= 0.0)
	   {
	      kwarn("design", "ZoomClass", 
		      "Attempt to set zoom factor <= 0; resetting to 0.001"); 
	      nxwid->zoom.zoomfactor = 0.001;
	   }

	   if (cxwid->zoom.zoomfactor != nxwid->zoom.zoomfactor)
	   {
	      nxwid->image.resize = TRUE;
	      nxwid->image.xmag = nxwid->zoom.zoomfactor;
	      nxwid->image.ymag = nxwid->zoom.zoomfactor;
	      redisplay = TRUE;
	   }

	   /*
	    *  Get the value data demsionality
	    */
	   kdms_get_attributes(nxwid->image.iobject, KDMS_SEGMENT_VALUE,
			KDMS_PHYSICAL_SIZE, &size,
			KDMS_PHYSICAL_INDEX_ORDER, &order,
			NULL);

	   w = _kdms_get_order(KWIDTH,    order, size, 5, 0);
	   h = _kdms_get_order(KHEIGHT,   order, size, 5, 0);
	   d = _kdms_get_order(KDEPTH,    order, size, 5, 0);
	   t = _kdms_get_order(KTIME,     order, size, 5, 0);
	   e = _kdms_get_order(KELEMENTS, order, size, 5, 0);
	   w *= nxwid->zoom.zoomfactor; h *= nxwid->zoom.zoomfactor;
	   nxwid->image.wsize = w; nxwid->image.hsize = h;
	   kpds_set_attributes(nxwid->image.iobject,
		KPDS_VALUE_SIZE,        w, h, d, t, e,
		KPDS_VALUE_INTERPOLATE, nxwid->zoom.interpolate,
		NULL);

           xmin = (Real) nxwid->image.xoffset * nxwid->zoom.zoomfactor;
           ymin = (Real) nxwid->image.yoffset * nxwid->zoom.zoomfactor;
           xmax = (Real) (nxwid->image.xoffset+nxwid->image.wsize)*
				nxwid->zoom.zoomfactor;
           ymax = (Real) (nxwid->image.yoffset+nxwid->image.hsize)*
				nxwid->zoom.zoomfactor;
           xvw_set_attributes(xvw_object(new1),
                      XVW_GRAPHICS_WCMIN_X, xmin,
                      XVW_GRAPHICS_WCMIN_Y, ymax - 1,
                      XVW_GRAPHICS_WCMAX_X, xmax - 1,
                      XVW_GRAPHICS_WCMAX_Y, ymin,
                      NULL);

	   /*
	    *  In case there is a mask segment...
	    */
	   kpds_set_attribute(nxwid->image.iobject, KPDS_MASK_INTERPOLATE,
			nxwid->zoom.interpolate);

	   if (cxwid->image.iobject == nxwid->image.iobject)
	   {
	      xoff = kmax(nxwid->image.xposition * nxwid->zoom.zoomfactor, 0);
	      yoff = kmax(nxwid->image.yposition * nxwid->zoom.zoomfactor, 0);

	      xvw_set_attributes(xvw_object(new1),
		  XVW_IMAGE_XOFFSET, xoff,
		  XVW_IMAGE_YOFFSET, yoff,
		  XVW_IMAGE_RELOAD,  TRUE,
		  NULL);
	   }
	}

	/*
	 *  changing x, y position in image at which zoom window is focused
	 */
	if (cxwid->zoom.xposition != nxwid->zoom.xposition ||
	    cxwid->zoom.yposition != nxwid->zoom.yposition)
	{
	    int xoffset, yoffset;
	    xoffset = (int) nxwid->zoom.xposition * nxwid->zoom.zoomfactor;
	    yoffset = (int) nxwid->zoom.yposition * nxwid->zoom.zoomfactor;
            xvw_set_attributes(xvw_object(new1),
                               XVW_IMAGE_XOFFSET, xoffset,
                               XVW_IMAGE_YOFFSET, yoffset,
                               XVW_IMAGE_RELOAD,  TRUE,
                               NULL);
	    redisplay = TRUE;
	}
      

	/*
	 *  changing clip mask
	 */
	if (cxwid->image.cobject != nxwid->image.cobject)
	{
	   kpds_set_attribute(nxwid->image.cobject,
		KPDS_VALUE_INTERPOLATE, nxwid->zoom.interpolate);
	}

	/*
	 *  changing interpolation mode
	 */
	if (cxwid->zoom.interpolate != nxwid->zoom.interpolate)
	{
	   kpds_set_attribute(nxwid->image.iobject,
		KPDS_VALUE_INTERPOLATE, nxwid->zoom.interpolate);

	   /*
	    *  In case there is a mask segment...
	    */
	   kpds_set_attribute(nxwid->image.iobject,
		KPDS_MASK_INTERPOLATE,  nxwid->zoom.interpolate);
	   redisplay = TRUE;
	}

        if (cxwid->zoom.location_marker != nxwid->zoom.location_marker)
        {
           redisplay = TRUE;
        }

	return(redisplay);
}


/*-----------------------------------------------------------
|
|  Routine Name: ZoomCallback
|
|       Purpose: This redisplays the position of the object being tracked
|                 
|         Input: 
|        Output:
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Nov 17, 1992 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ZoomCallback(
   kobject object,
   char    *segment,
   Widget  widget,
   kdms_callback *cb)
{
	XvwZoomWidget xwid = (XvwZoomWidget) widget;

	int w, h, d, update, reload;

	/*
	 *  Make sure we aren't here because of ourself
	 */
	if (object == cb->object)
	   return;

	xvw_get_type(object, &reload, &update);
	if ((xwid->zoom.update_mode == KZOOM_UM_CONTINUOUS &&
	                     update != MOTION_EVENT_UPDATE) ||
	    (xwid->zoom.update_mode == KZOOM_UM_BUTTON_PRESS &&
	                     update != BUTTONPRESS_EVENT_UPDATE) ||
	                     reload == ZOOM_OBJECT_RELOAD)
	{
	   return;
	}

	/*
	 * this is the point in the zoom window that corresponds to the
 	 * point in the image where the user clicked (the focus pixel)
	 */
	w = (cb->end.w - cb->begin.w)/2 + cb->begin.w;
	h = (cb->end.h - cb->begin.h)/2 + cb->begin.h;
	d = cb->begin.d;
	xwid->zoom.xposition = (int)(w/xwid->zoom.zoomfactor);
	xwid->zoom.yposition = (int)(h/xwid->zoom.zoomfactor);

	/*
	 * backing off up and to the left by 1/2 the size of the zoom window,
	 * so that focus pixel appears in the middle of the zoom window
	 * rather than at the upper right hand corner
	 */
	w -= (((int) xwid->core.width)  - xwid->zoom.zoomfactor)/2;
	h -= (((int) xwid->core.height) - xwid->zoom.zoomfactor)/2;
	w = kmax(w, 0); h = kmax(h, 0);
	xvw_set_attributes(xvw_object(widget),
	    XVW_IMAGE_XOFFSET, w,
	    XVW_IMAGE_YOFFSET, h,
	    XVW_IMAGE_REDISPLAY, TRUE, 
	    NULL);
	xvw_set_type(object, reload, update);
}


/************************************************************
*
*  Routine Name: xvw_create_zoom - create a zoom object
*
*       Purpose: The zoom object provides a zoom window which is
*                used to zoom in on an image.  It was designed for
*                use in conjunction with an image object.
*
*                The zoom factor can be set as desired. The zoom object
*                may be set to update on either Button Press or 
*                Pointer Motion events.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the zoom
*                         object (for use in app-defaults files, etc)
*        Output: 
*	Returns: The zoom object on success, NULL on failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Oct 08, 1992 15:37
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_zoom(
   xvobject parent,
   char     *name)
{
	xvobject object;

	object = xvw_create(parent, FALSE, TRUE, name, ZoomWidgetClass);
	return(object);
}
