/*
  * Khoros: $Id: annotations.c,v 1.9 1997/01/20 05:21:59 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: annotations.c,v 1.9 1997/01/20 05:21:59 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		General Utility Routines
   >>>>		
   >>>>  Private:
   >>>>                 xvw_save_annotations()
   >>>>                 xvw_restore_annotations()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/CircleP.h>
#include <xvisual/DateP.h>
#include <xvisual/LineP.h>
#include <xvisual/MarkerP.h>
#include <xvisual/RectangleP.h>
#include <xvisual/StringP.h>
#include <xvisual/StringValueP.h>


/*-----------------------------------------------------------
|
|  Routine Name: xvw_save_annotations - saves the annotations
|                                       of a particular parent
|
|       Purpose: Saves the annotations of a particular parent
|                into a file
|         Input: object - visual object that is the parent of the
|                         annotations to be saved
|                file   - open stream to file to which annotations
|                         will be saved
|
|    Written By: Mark Young
|          Date:
| Modifications:
|
------------------------------------------------------------*/

int xvw_save_annotations(
   xvobject object,
   kfile    *file)
{
	int    i, num;
	Widget *children;

	xvw_get_attributes(object,
		XVW_NUM_CHILDREN, &num,
		XVW_CHILDREN,     &children,
		NULL);

	kfprintf(file, "AnnotationsBegin\n");
	for (i = 0; i < num; i++)
	{
	   if (XtClass(children[i]) == CircleGadgetClass)
	   {
	      kfprintf(file, "circle\n");
	   }
	   else if (XtClass(children[i]) == DateGadgetClass)
	   {
	      kfprintf(file, "date\n");
	   }
	   else if (XtClass(children[i]) == LineGadgetClass)
	   {
	      kfprintf(file, "line\n");
	   }
	   else if (XtClass(children[i]) == MarkerGadgetClass)
	   {
	      kfprintf(file, "marker\n");
	   }
	   else if (XtClass(children[i]) == RectangleGadgetClass)
	   {
	      kfprintf(file, "rectangle\n");
	   }
	   else if (XtClass(children[i]) == StringGadgetClass)
	   {
	      kfprintf(file, "string\n");
	   }
	   else if (XtClass(children[i]) == StringValueGadgetClass)
	   {
	      kfprintf(file, "stringvalue\n");
	   }
	}
	kfprintf(file, "AnnotationsEnd\n");
	return(TRUE);
}


/*-----------------------------------------------------------
|
|  Routine Name: xvw_restore_annotations - restores annotations
|                                       to a particular parent
|
|       Purpose: Restores the annotations to a particular parent
|                from a file
|         Input: object - visual object that will be the parent of the
|                         annotations to be restored
|                file   - open stream to file from which annotations
|                         will be read
|
|    Written By: Mark Young
|          Date:
| Modifications:
|
------------------------------------------------------------*/

int xvw_restore_annotations(
   xvobject object,
   kfile    *file)
{
#if 0
	char   temp[KLENGTH];
	while (!kfeof(file))
	{
	   if (kstrcmp(temp, "circle") == 0)
	   {
	   }
	   else if (kstrcmp(temp, "cursor") == 0)
	   {
	   }
	   else if (kstrcmp(temp, "date") == 0)
	   {
	   }
	   else if (kstrcmp(temp, "line") == 0)
	   {
	   }
	   else if (kstrcmp(temp, "marker") == 0)
	   {
	   }
	   else if (kstrcmp(temp, "raster") == 0)
	   {
	   }
	   else if (kstrcmp(temp, "rectangle") == 0)
	   {
	   }
	   else if (kstrcmp(temp, "string") == 0)
	   {
	   }
	   else if (kstrcmp(temp, "stringvalue") == 0)
	   {
	   }
	   else if (kstrcmp(temp, "text") == 0)
	   {
	   }
	}
#endif
	return(TRUE);
}
