 /*
  * Khoros: $Id: Port.c,v 1.58.2.4 1997/05/24 19:04:21 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Port.c,v 1.58.2.4 1997/05/24 19:04:21 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros Port Gadget
   >>>>  Private:
   >>>>                 PortGetInfo()
   >>>>                 PortCreateConnection()
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 SetValues()
   >>>>                 Destroy()
   >>>>                 PickGadget()
   >>>>                 PortReset()
   >>>>                 PortHandler()
   >>>>                 ConnectionHandler()
   >>>>                 IportRemoveConnection()
   >>>>                 ModifiedCallback()
   >>>>   Public:
   >>>>                 xvw_create_port()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvlang/PortP.h>
#include <xvlang/NodeP.h>
#include <xvlang/GlyphP.h>

static void     ClassInitialize PROTO((void));
static void	Initialize	PROTO((Widget, Widget, ArgList, Cardinal *));
static Boolean	SetValues       PROTO((Widget, Widget, Widget, ArgList,
				       Cardinal *));
static void     Destroy		PROTO((Widget));

static int  PickGadget		  PROTO((Widget, int, int));
static int  PortReset	          PROTO((xvobject, char *, kaddr));
static int  PortDeleteConnection  PROTO((xvobject, char *, kaddr));
static void PortHandler           PROTO((xvobject, kaddr, XEvent *, int *));
static void ConnectionHandler     PROTO((xvobject, kaddr, XEvent *, int *));
static void IportRemoveConnection PROTO((xvobject, kaddr, kaddr));
static void OportRemoveConnection PROTO((xvobject, kaddr, kaddr));
static void ModifiedCallback      PROTO((xvobject, kaddr, kaddr));
static void UpdatePixmap	  PROTO((Widget));
static void UpdateOptSel	  PROTO((Widget, int));
static void UpdateFilename	  PROTO((Widget, int));
static void UpdateGroupOptsel     PROTO((kform_struct *, int));
static void CopyFilename	  PROTO((Widget));


static klist *portlist = NULL;
static Widget iportobj = NULL, oportobj = NULL;

/*-------------------------------------------------------------------*
|
|   Full class attributes
|
--------------------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_PORT_SELECTION,	  NULL,	  XtRPointer,  NULL},
{XVW_PORT_DAV,		  NULL,	  XtRInt,      XtRBoolean},
{XVW_PORT_SELECTED,	  NULL,	  XtRInt,      XtRBoolean},
{XVW_PORT_MODIFIED,	  NULL,	  XtRInt,      XtRBoolean},
{XVW_PORT_TYPE,		  NULL,	  XtRInt,      NULL},
{XVW_PORT_DIRECTION,	  NULL,	  XtRInt,      NULL},
{XVW_PORT_CONNECTION_PARENT,NULL, XtRObject,   XtRWidget},
{XVW_PORT_FILENAME,	  NULL,   XtRString,   NULL},
{XVW_PORT_DAV_CALLBACK,	  NULL,   XtRCallback, NULL},
{XVW_PORT_SELECTED_CALLBACK,NULL, XtRCallback, NULL},
{XVW_PORT_MODIFIED_CALLBACK,NULL, XtRCallback, NULL},
{XVW_PORT_FILENAME_CALLBACK,NULL, XtRCallback, NULL},
{XVW_NODE_REMOTE_LOCATION,  NULL, XtRString,   NULL},

{XVW_PORT_DAV_PIXEL, NULL, XtRPixel, NULL},
{XVW_PORT_SELECTED_PIXEL,  NULL, XtRPixel, NULL},
{XVW_PORT_CONTROL_PIXEL,   NULL, XtRPixel, NULL},
{XVW_PORT_PARAMETER_PIXEL, NULL, XtRPixel, NULL},
{XVW_PORT_OPTIONAL_DATA_PIXEL,  NULL, XtRPixel, NULL},
{XVW_PORT_REQUIRED_DATA_PIXEL,  NULL, XtRPixel, NULL},
};


/*-------------------------------------------------------------------*
|
|   Full class record constant
|
--------------------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwPortGadgetRec, port.field)

static XtResource resources[] = { 
{XVW_PORT_SELECTION, NULL, XtRPointer, sizeof(XtPointer),
        offset(selection), XtRImmediate, (XtPointer) NULL},
{XVW_PORT_DAV, NULL, XtRBoolean, sizeof(Boolean),
        offset(dav), XtRImmediate, (XtPointer) FALSE},
{XVW_PORT_DAV_CALLBACK, NULL, XtRCallback, sizeof(XtPointer),
      offset(dav_callback), XtRCallback, (XtPointer) NULL},
{XVW_PORT_SELECTED, NULL, XtRBoolean, sizeof(Boolean),
        offset(selected), XtRImmediate, (XtPointer) FALSE},
{XVW_PORT_SELECTED_CALLBACK, NULL, XtRCallback, sizeof(XtPointer),
      offset(selected_callback), XtRCallback, (XtPointer) NULL},
{XVW_PORT_MODIFIED, NULL, XtRBoolean, sizeof(Boolean),
        offset(modified), XtRImmediate, (XtPointer) FALSE},
{XVW_PORT_MODIFIED_CALLBACK, NULL, XtRCallback, sizeof(XtPointer),
      offset(modified_callback), XtRCallback, (XtPointer) NULL},
{XVW_PORT_TYPE, NULL, XtRInt, sizeof(int),
        offset(type), XtRImmediate, (XtPointer) KPORT_UNKNOWN},
{XVW_PORT_DIRECTION, NULL, XtRInt, sizeof(int),
        offset(direction), XtRImmediate, (XtPointer) KPORT_UNKNOWN},
{XVW_PORT_CONNECTION_PARENT, NULL, XtRWidget, sizeof(Widget),
        offset(connection_parent), XtRImmediate, (XtPointer) NULL},
{XVW_PORT_FILENAME, NULL, XtRString, sizeof(String),
      offset(filename), XtRImmediate, (XtPointer) NULL},
{XVW_PORT_FILENAME_CALLBACK, NULL, XtRCallback, sizeof(XtPointer),
      offset(filename_callback), XtRCallback, (XtPointer) NULL},
{XVW_NODE_REMOTE_LOCATION, NULL, XtRString, sizeof(String),
        offset(machine), XtRImmediate, (XtPointer) "localhost"},

{XVW_PORT_CONTROL_PIXEL, NULL, XtRPixel, sizeof(Pixel),
        offset(control_pixel), XtRString, (XtPointer) "#979797"},
{XVW_PORT_PARAMETER_PIXEL, NULL, XtRPixel, sizeof(Pixel),
        offset(parameter_pixel), XtRString, (XtPointer) "#ffffff"},
{XVW_PORT_OPTIONAL_DATA_PIXEL, NULL, XtRPixel, sizeof(Pixel),
        offset(optional_pixel), XtRString, (XtPointer) "#0000ff"},
{XVW_PORT_REQUIRED_DATA_PIXEL, NULL, XtRPixel, sizeof(Pixel),
        offset(required_pixel), XtRString, (XtPointer) "#ffff00"},
{XVW_PORT_SELECTED_PIXEL, NULL, XtRPixel, sizeof(Pixel),
        offset(selected_pixel), XtRString, (XtPointer) "#000000"},
{XVW_PORT_DAV_PIXEL, NULL, XtRPixel, sizeof(Pixel),
        offset(dav_pixel), XtRString, (XtPointer) "#00ff00"},

{XVW_FORCE_REDISPLAY, NULL, XtRBoolean, sizeof(Boolean),
	XtOffsetOf(XvwPortGadgetRec, manager.force_redisplay),
	XtRImmediate, (XtPointer) TRUE},
{XVW_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwPortGadgetRec, rectobj.width),
        XtRImmediate, (XtPointer) 10},
{XVW_HEIGHT, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwPortGadgetRec, rectobj.height),
        XtRImmediate, (XtPointer) 8},

};
#undef offset


/*-------------------------------------------------------------------*
|
|   Class Declaration for Port Gadget
|
--------------------------------------------------------------------*/

#define superclass (&xvwManagerGadgetClassRec)

XvwPortGadgetClassRec xvwPortGadgetClassRec =
{
  {
    (WidgetClass) superclass,		/* superclass		  */	
    "Port",			/* class_name		  */
    sizeof(XvwPortGadgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    NULL,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    PickGadget,                             /* pick object proc       */
    XtInheritEraseSel,                      /* erase selected proc    */
    XtInheritRefreshSel,                    /* refresh selection proc */
    NULL,                		    /* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    NULL,  		                    /* extension          */
  },  /* XvwPortGadgetClass fields initialization */
};
#undef superclass

  /* for public consumption */
WidgetClass xvwPortGadgetClass = (WidgetClass) &xvwPortGadgetClassRec;


/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for Class and Constraint Declarations
|
--------------------------------------------------------------------*/

#undef kwidget
#undef kwidgetclass
#undef kconstraint

#define kwidget(widget)	     (XvwPortGadget) (widget)
#define kwidgetclass(widget) (XvwPortGadgetClass) (widget)->core.widget_class
#define kconstraint(widget)  (XvwPortGadgetConstraints) (widget)->core.constraints


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an
|                instance of a XvwPortGadget class has been created.
|                This is where the attributes for the class are
|		 initialized. 
|
|    Written By: Mark Young
|          Date: Nov 22, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
	xvlang_cache_pixmaps();
	xvw_init_attributes(xvwPortGadgetClass, attributes,
		XtNumber(attributes), NULL, 0, NULL);
	xvw_load_resources("$DESIGN/objects/library/xvlang/app-defaults/Port");
	xvw_define_attributes(xvwPortGadgetClass,
            XVW_PORT_RESET,             XtRInt,    PortReset,             NULL,
            XVW_PORT_DELETE_CONNECTION, XtRObject, PortDeleteConnection,  NULL,
            NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|	Purpose: This method will set up the initial port widget instance.
|
|	  Input: request - not used
|                new1     - widget instance after initialization
|    Written By: Mark Young
|          Date: Nov 22, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	unsigned long mask;
	XvwPortGadget xobj = kwidget(new1);
	xvobject port = xvw_object(new1);

	/*
	 *  Initialize the ports to be empty
	 */
	xobj->port.num = 0;
	xobj->port.kludge      = FALSE;
	xobj->port.temporary   = FALSE;
	xobj->port.iports      = NULL;
	xobj->port.oports      = NULL;
	xobj->port.connections = NULL;
	xobj->port.machine     = kstrdup(xobj->port.machine);
	mask = ButtonPressMask | ButtonReleaseMask |
	       EnterWindowMask | LeaveWindowMask;
	xvw_insert_event(port, mask, PortHandler, (kaddr) port, KLIST_HEAD);
	portlist = klist_add(portlist, (kaddr) new1, NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the public values
|                of a XvwPortGadget instance.  The public values
|                which can be changed are all related to the display
|		 of the port.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new1     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 22, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num)
{
	XvwPortGadget cxobj = kwidget(current);
	XvwPortGadget nxobj = kwidget(new1);

	char *filename;
	int  i, selected, modified, dav;

	if (cxobj->port.type            != nxobj->port.type ||
	    cxobj->port.direction       != nxobj->port.direction ||
	    cxobj->port.dav_pixel       != nxobj->port.dav_pixel ||
	    cxobj->port.control_pixel   != nxobj->port.control_pixel ||
	    cxobj->port.parameter_pixel != nxobj->port.parameter_pixel ||
	    cxobj->port.selected_pixel  != nxobj->port.selected_pixel ||
	    cxobj->port.optional_pixel  != nxobj->port.optional_pixel ||
	    cxobj->port.required_pixel  != nxobj->port.required_pixel)
	{
           UpdatePixmap(new1);
	}

	if (cxobj->port.filename != nxobj->port.filename)
	{
	   nxobj->port.filename = kstrdup(nxobj->port.filename);
	   UpdateFilename(new1, TRUE);
	   kfree_and_NULL(cxobj->port.filename);
	}

        if (cxobj->port.machine != nxobj->port.machine)
        {
           kfree(cxobj->port.machine);
           nxobj->port.machine = kstring_cleanup(nxobj->port.machine, NULL);
           if (nxobj->port.machine == NULL)
              nxobj->port.machine = kstrdup("localhost");

	   CopyFilename(new1);
        }

	if (cxobj->port.dav != nxobj->port.dav)
	{
	   if (nxobj->port.dav && !nxobj->port.kludge)
	   {
	      nxobj->port.kludge = TRUE;
	      UpdateOptSel(new1, TRUE);
	   }
	   UpdatePixmap(new1);
	   CopyFilename(new1);
	}

	if (cxobj->port.selected != nxobj->port.selected)
	{
	   UpdatePixmap(new1);
	   UpdateOptSel(new1, TRUE);
	}

	if (cxobj->port.selection != nxobj->port.selection)
	{
	   UpdateOptSel(new1, FALSE);
	   xvf_set_attribute(nxobj->port.selection, XVF_FILE_CHECK, FALSE);
	}

	if (cxobj->port.dav != nxobj->port.dav &&
	    nxobj->port.direction == KPORT_OUTPUT)
	{
	   Widget w = XtParent((Widget)nxobj);
	   char *gname = w->core.name;
	   char *fn = nxobj->port.filename;
	   if (XtIsSubclass(w, GlyphWidgetClass))
	   {
	      XvwGlyphWidget nwid = (XvwGlyphWidget) w;
	      gname = nwid->node.name;
	   }
	   if (fn == NULL) fn = "nofile";
	   xvl_trace(NULL, XVLANG_TRACE_PROCS,
		     "DAV is ON for output port of %s (%s)\n",
		     gname,
		     fn);
	   for (i = 0; i < nxobj->port.num; i++)
	   {
	      xvobject connection = nxobj->port.connections[i];
	      if (nxobj->port.type == KPORT_DATAFLOW)
	      {
		 xvw_set_attribute(connection, XVW_FOREGROUND, nxobj->port.dav ?
			nxobj->port.dav_pixel : nxobj->port.required_pixel);
	      }

	      if (nxobj->port.dav == TRUE)
	      {
		 xvl_trace(NULL, XVLANG_TRACE_PROCS,
			   "Setting DAV ON for iports[%d]\n", i);
	         xvw_set_attributes(nxobj->port.iports[i],
			XVW_PORT_DAV, TRUE,
			XVW_PORT_MODIFIED, TRUE,
			NULL);
	      }
	      else
	         xvw_set_attribute(nxobj->port.iports[i], XVW_PORT_DAV, FALSE);
	   }
	}

	if (cxobj->port.selected != nxobj->port.selected)
	{
	   selected = nxobj->port.selected;
	   XtCallCallbacks(new1, XVW_PORT_SELECTED_CALLBACK, &selected);
	}

	if (cxobj->port.dav != nxobj->port.dav)
	{
	   dav = nxobj->port.dav;
	   if (nxobj->port.direction != KPORT_OUTPUT)
	   {
	      Widget w = XtParent((Widget)nxobj);
	      char *gname = w->core.name;
	      char *fn = nxobj->port.filename;
	      if (XtIsSubclass(w, GlyphWidgetClass))
	      {
		 XvwGlyphWidget nwid = (XvwGlyphWidget) w;
		 gname = nwid->node.name;
	      }
	      if (fn == NULL) fn = "nofile";
	      xvl_trace(NULL, XVLANG_TRACE_PROCS,
			"DAV is %s for input port of %s (%s)\n",
			dav ? "ON" : "OFF",
			gname,
			fn);
	   }
	   XtCallCallbacks(new1, XVW_PORT_DAV_CALLBACK,(XtPointer) &dav);
	}

	if (cxobj->port.modified != nxobj->port.modified)
	{
	   Widget w = XtParent((Widget)nxobj);
	   char *gname = w->core.name;
	   if (XtIsSubclass(w, GlyphWidgetClass))
	   {
	      XvwGlyphWidget nwid = (XvwGlyphWidget) w;
	      gname = nwid->node.name;
	   }
	   modified = nxobj->port.modified;
	   xvl_trace(NULL, XVLANG_TRACE_PROCS,
		     "%s port of %s (%s) modified is %s\n",
		     (nxobj->port.direction == KPORT_OUTPUT) ? "Output" : "Input",
		     gname,
		     nxobj->port.filename,
		     modified ? "TRUE" : "FALSE");
	   XtCallCallbacks(new1, XVW_PORT_MODIFIED_CALLBACK, &modified);
	}

	if (cxobj->port.filename != nxobj->port.filename)
	{
	   filename = nxobj->port.filename;
	   XtCallCallbacks(new1, XVW_PORT_FILENAME_CALLBACK, &filename);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: PickGadget
|
|       Purpose: Given a connection object and an attempted picking location,
|                see if the port has been picked.
|
|         Input: widget - the connection object we are checking
|                x      - the x device coordinate that the pick attempt was at
|                y      - the y device coordinate that the pick attempt was at
|       Returns: TRUE (1) if object was successfully picked, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jun 29, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static int PickGadget(
   Widget widget,
   int    x,
   int    y)
{
        XvwPortGadget xobj = (XvwPortGadget) widget;
        int xpos, ypos, width, height;

        xpos  = xobj->rectobj.x;     ypos = xobj->rectobj.y;
        width = xobj->rectobj.width; height = xobj->rectobj.height;
        if (xpos <= x && x <= xpos+width && ypos <= y && y <= ypos+height)
           return(TRUE);
        else
           return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: destroy the connections...
|
|         Input: widget - the image widget being destroyed 
|    Written By: Mark Young
|          Date: Nov 21, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	XvwPortGadget xobj = kwidget(widget);

	size_t   i;
	Widget   parent;

	/*
	 *  Destroy all the connections and free the memory...
	 */
	parent = xobj->port.connection_parent;
	if (!parent->core.being_destroyed)
	{
	   for (i = 0; i < xobj->port.num; i++)
	   {
              if (xobj->port.direction == KPORT_OUTPUT)
	      {
                 xvw_remove_callback(xobj->port.iports[i],
		   	   XVW_PORT_MODIFIED_CALLBACK, ModifiedCallback, xobj);
                 xvw_remove_callback(xobj->port.iports[i],
		 	   XVW_DESTROY, IportRemoveConnection, xobj);
	         xvw_destroy(xobj->port.connections[i]);
	      }
	      else if (xobj->port.direction == KPORT_INPUT)
	      {
                 xvw_remove_callback(xobj->port.oports[i],
                          XVW_DESTROY, OportRemoveConnection, xobj);
	      }
	   }
	}
	kfree_and_NULL(xobj->port.iports);
	kfree_and_NULL(xobj->port.oports);
	kfree_and_NULL(xobj->port.connections);

	if (xobj->port.type == KPORT_DATAFLOW &&
	    xobj->port.direction == KPORT_OUTPUT)
	{
	   if (xobj->port.temporary == TRUE)
	      kunlink(xobj->port.filename);
	}
	kfree(xobj->port.filename);
	kfree(xobj->port.machine);
	portlist = klist_delete(portlist, (kaddr) widget);

	if (widget == iportobj)
	   iportobj = NULL;
	else if (widget == oportobj)
	   oportobj = NULL;
}

/*-----------------------------------------------------------
|
|  Routine Name: PortReset
|
|       Purpose: PortReset is used to update the port from the specified
|		 selection.
|
|         Input: port  - the port object in which was modified
|                attribute - the modified attribute
|                calldata  - the filename to be set
|       Returns: TRUE (1) if successful, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 21, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static int PortReset(
   xvobject port,
   char     *attribute,
   kaddr    calldata)
{
	Widget widget = xvw_widget(port);
	XvwPortGadget xobj = kwidget(widget);

	if (xobj->port.selection == NULL)
	   return(TRUE);

	UpdateFilename(widget, FALSE);
	UpdateOptSel(widget, FALSE);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: PortDeleteConnection
|
|       Purpose: PortDeleteConnection is used to delete the connection 
|                between an output port and the input port that is passed in. 
|
|         Input: port      - output port object from which connection originates
|                attribute - the modified attribute
|                calldata  - the input port to which the output port is 
|                            currently connected
|       Returns: TRUE (1) if successful, FALSE (0) otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: July 1, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static int PortDeleteConnection(
   xvobject oport,
   char     *attribute,
   kaddr    calldata)
{
	xvobject      iport = *((xvobject *) calldata);
        XvwPortGadget oxobj = (XvwPortGadget) xvw_widget(oport);
	XvwPortGadget ixobj = (XvwPortGadget) xvw_widget(iport);

        xvw_remove_callback(iport, XVW_PORT_MODIFIED_CALLBACK,
                            ModifiedCallback, oxobj);
        xvw_remove_callback(iport, XVW_DESTROY,
                            IportRemoveConnection, oxobj);
        xvw_remove_callback(oport, XVW_DESTROY,
                            OportRemoveConnection, ixobj);
        IportRemoveConnection(iport, oxobj, NULL);
        OportRemoveConnection(oport, ixobj, NULL);

	return(TRUE);
}


/*-----------------------------------------------------------
|
|  Routine Name: PortHandler
|
|       Purpose: This routine is called when the input or output
|		 of an io port is activated.
|         Input: object      - remote button being activated
|                client_data - the glyph widget
|                event       - event the activated us
|		 dispatch    - whether to continue dispatch the event
|    Written By: Mark Young
|          Date: Nov 22, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void PortHandler(
   xvobject object,
   kaddr    client_data,
   XEvent   *event,
   int	    *dispatch)
{
	xvobject port = (xvobject) client_data;

	Widget widget = xvw_widget(port);
	Widget itemp = iportobj, otemp = oportobj;
	XvwPortGadget xobj1, xobj2, xobj = (XvwPortGadget) kwidget(widget);

#if 0
	static int xpos = 0, ypos = 0;
#endif

	if (event->type == ButtonPress || event->type == ButtonRelease)
	   *dispatch = FALSE;

#if 0
	if (event->type != ButtonRelease ||
	    kabs(xpos - event->xbutton.x_root) > (int) widget->core.width ||
	    kabs(ypos - event->xbutton.y_root) > (int) widget->core.height)
	{
	   xpos = event->xbutton.x_root; ypos = event->xbutton.y_root;
	   return;
	}
#endif

	if (event->type == EnterNotify)
	{
	   char temp[KLENGTH], *tmp = NULL;

	   if (xobj->port.type == KPORT_CONTROLFLOW)
	   {
	      if (xobj->port.direction == KPORT_INPUT)
	         tmp = kstrdup("Input Control Connection Port");
	      else
	         tmp = kstrdup("Output Control Connection Port");
	   }
	   else if (xobj->port.selection != NULL)
	      xvf_get_attribute(xobj->port.selection, XVF_TITLE, &tmp);
	   else
	      tmp = kstrdup("<none>");

	   ksprintf(temp, "%s: (%s) (%s)", tmp,
		xobj->port.dav ? "data available" : "data unavailable",
		xobj->port.modified ? "modified" : "unmodified");
	   kannounce(XVLANG, "PortHandler", temp); kfree(tmp);
	   return;
	}
	else if (event->type == LeaveNotify)
	{
	   kannounce(XVLANG, "PortHandler", " ");
	   return;
	}
	else if (event->type == ButtonPress)
	{
	   return;
	}

	/*
	 *  Check to see if the object is an input or output
	 */
	if (xobj->port.direction == KPORT_INPUT)
	{
	   iportobj = (iportobj == widget) ? NULL : widget;
	   UpdatePixmap(iportobj); UpdatePixmap(itemp);
	}
	else if (xobj->port.direction == KPORT_OUTPUT)
	{
	   oportobj = (oportobj == widget) ? NULL : widget;
	   UpdatePixmap(oportobj); UpdatePixmap(otemp);
	}

	/*
	 *  If either both input or output port are set then create the
	 *  connection
	 */
	xobj1 = kwidget(iportobj); xobj2 = kwidget(oportobj);
	if (xobj1 && xobj2 && xobj1->port.type == xobj2->port.type &&
	    xobj1->port.connection_parent == xobj2->port.connection_parent)
	{
	   itemp = iportobj; otemp = oportobj;
	   PortCreateConnection(xvw_object(iportobj), xvw_object(oportobj));
	   iportobj = oportobj = NULL; UpdatePixmap(itemp); UpdatePixmap(otemp);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: ConnectionHandler
|
|       Purpose: This routine is called when the input or output
|		 of an io port is activated.
|         Input: object      - the connection object
|                client_data - passes in the output port
|                event       - event the activated us
|		 dispatch    - whether to continue dispatch the event
|    Written By: Mark Young
|          Date: Nov 22, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ConnectionHandler(
   xvobject connection,
   kaddr    client_data,
   XEvent   *event,
   int	    *dispatch)
{
	xvobject oport = (xvobject) client_data;

	int    i, size;
	Widget widget = xvw_widget(oport);
	XvwPortGadget xobj = (XvwPortGadget) kwidget(widget);
	char   *filename, **list, temp[KLENGTH], identifier[KLENGTH];
	static int working = FALSE;
	static size_t num;

	static char *choice1[] =
	{
		"Delete Connection",
		"Connection Options",
		"Save Data to File",
		"",
	};
	static char *choice2[] =
        {
                "Delete Connection",
                "Connection Options",
        };
	static char *transports[] = 
	{
                "Standard Unix Files (file)",
                "Memory Mapped Files (mmap)",
                "Shared Memory Files (shm)",
                "Standard Stream FIFO (stream)",
                "Unix Domain Socket (socket)",
	};

	if (event->type == ButtonPress || event->type == ButtonRelease)
	   *dispatch = FALSE;

	if (working) return;

	working = TRUE;

#if 0
	static int xpos = 0, ypos = 0;
	if (event->type != ButtonRelease ||
	    kabs(xpos - event->xbutton.x_root) > (int) widget->core.width ||
	    kabs(ypos - event->xbutton.y_root) > (int) widget->core.height)
	{
	   xpos = event->xbutton.x_root; ypos = event->xbutton.y_root;
	   working = FALSE;
	   return;
	}
#endif

	if (event->type == LeaveNotify)
	{
	   kannounce(XVLANG, "ConnectionHandler", " ");
	   working = FALSE;
	   return;
	}
	else if (event->type == EnterNotify)
	{
	   char     *tmp = NULL;

	   if (xobj->port.filename != NULL)
	      tmp = xobj->port.filename;
	   else if (xobj->port.type == KPORT_CONTROLFLOW)
	      tmp = "Control Flow Connection";
	   else if (xobj->port.type == KPORT_PARAMETER)
	      tmp = "Parameter Connection";
	   else
	      tmp = "Untyped Connection";

	   kannounce(XVLANG, "Connection", "%s", tmp);
	   working = FALSE;
	   return;
	}
	else if (event->type == ButtonPress)
	{
	   working = FALSE;
	   return;
	}

	if (xobj->port.temporary)
	{
	   /* substituted hardwired list above for ktransport_list since
	      sockets and pipes aren't working (DA 11/7/96
	   if (!transports)
	      transports = ktransport_list(TRUE, TRUE, TRUE, &num);  */

	   num = knumber(transports);
	   list = (char **)karray_merge(choice1, transports, KSTRING,
			       knumber(choice1), num, FALSE);
	   size = num + knumber(choice1);
	}
	else
	{
	   if (xobj->port.filename != NULL)
	   {
	      list = (char **)karray_copy(choice1, KSTRING, 
			         knumber(choice1), FALSE);
	      size = knumber(choice1);
	   }
	   else
	   {
	      list = (char **)karray_copy(choice2, KSTRING, 
			         knumber(choice2), FALSE);
	      size = knumber(choice2);
	   }
	}

	if (xobj->port.filename != NULL)
	    ksprintf(temp, "%s", xobj->port.filename);
	else ksprintf(temp, "%s", "   ");

	if (!kchoose(KFORCE, list, size, -1, NULL, &i, temp))
	   i = -1;
	else i--;

	if (i != -1)
	{
	   kinfo(KDEBUG, "entry '%s' selected for filename '%s'\n", list[i],
			xobj->port.filename);
	
	   /* 
	    * First connection menu item: delete the connection 
	    */
	   if (i == 0)
	   {
	      xvobject iport;
	      xvw_get_attribute(connection, XVW_CONNECTION_END, &iport);
	      xvw_set_attribute(oport, XVW_PORT_DELETE_CONNECTION, iport);
	   }
	
	   /*
	    * Second connection menu item: display connection menuform
	    */
	   else if (i == 1)
	   {
	      xvw_activate_menu(connection);
	   }

	   /*
	    * Third connection menu item: write data to file
	    */
	   else if (i == 2)
	   {
	      char   *prompt = "Filename:", *answer = temp;
	     
	      /* prompt for file from user... */
	      if (xvu_query_wait("Please enter filename", &prompt, "Ok",
		    &answer, 1, 55) == TRUE && kstrlen(answer) > 0 &&
	          kcopyfile(xobj->port.filename, answer) == -1)
	      {
		 kinfo(KSTANDARD, "Unable to copy existing data from file '%s' "
			"to the new transport '%s'", xobj->port.filename,
			answer);
	      }
	   }
	
	   /*
	    * Fourth connection menu item: change transport
            */
	   else if (i >= knumber(choice1) && xobj->port.temporary)
	   {
	      i -= knumber(choice1);
	      if (ksscanf(transports[i], "%*[^(](%[^)])", identifier) != 1)
	      {
		 kinfo(KSTANDARD, "Unable to get identifer from the transport "
			"list '%s'", transports[i]); kfree(list);
                 working = FALSE;
		 return;
	      }
	      filename = ktempnam(identifier, "io");
	      xvw_set_attributes(oport,
			XVW_PORT_DAV, FALSE,
			XVW_PORT_FILENAME, filename,
			NULL);
	   }
	}
	kfree_and_NULL(list);
        working = FALSE;
}

/*-----------------------------------------------------------
|
|  Routine Name: IportRemoveConnection
|
|       Purpose: This routine is called when the iport is destroyed
|         Input: connection - the connection to be destroyed.
|    Written By: Mark Young
|          Date: Nov 22, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void IportRemoveConnection(
   xvobject iport,
   kaddr    client_data,
   kaddr    call_data)
{
	int      indx;
	xvobject connection;

	XvwPortGadget oxobj = (XvwPortGadget) client_data;

	if ((indx = karray_locate(oxobj->port.iports, KSTRUCT,
		                  iport, (size_t) oxobj->port.num)) != -1)
	{
	   connection = oxobj->port.connections[indx];
	   oxobj->port.connections = (xvobject *)karray_delete(oxobj->port.connections,
		KSTRUCT, connection, oxobj->port.num);
	   oxobj->port.iports = (xvobject *)karray_delete(oxobj->port.iports,
		KSTRUCT, iport, oxobj->port.num--);
	   xvw_destroy(connection);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: OportRemoveConnection
|
|       Purpose: This routine is called when the oport is destroyed
|         Input: connection - the connection to be destroyed.
|    Written By: Mark Young
|          Date: Nov 22, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void OportRemoveConnection(
   xvobject oport,
   kaddr    client_data,
   kaddr    call_data)
{
        int      indx;
        xvobject connection, iport;

        XvwPortGadget ixobj = (XvwPortGadget) client_data;

        if ((indx = karray_locate(ixobj->port.oports, KSTRUCT,
                                  oport, (size_t) ixobj->port.num)) != -1)
        {
           connection = ixobj->port.connections[indx];
           ixobj->port.connections = (xvobject *)karray_delete(ixobj->port.connections,
                KSTRUCT, connection, ixobj->port.num);
           ixobj->port.oports = (xvobject *)karray_delete(ixobj->port.oports,
                KSTRUCT, oport, ixobj->port.num--);

	    if (ixobj->object.being_destroyed != TRUE)
	    {
	       iport = xvw_object((Widget) ixobj);
               xvw_set_attributes(iport,
                       XVW_PORT_DAV, FALSE,
                       XVW_PORT_FILENAME, NULL,
                       NULL);
	       ixobj->port.selected = FALSE;
	       UpdateOptSel((Widget) ixobj, TRUE);
	       UpdatePixmap((Widget) ixobj);
	    }
        }
}


/*-----------------------------------------------------------
|
|  Routine Name: UpdatePixmap - update the pixel being displayed
|
|       Purpose: Update the pixel 
|         Input: port - the port object
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Mar 30, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void UpdatePixmap(
   Widget widget)
{
	Pixel pixel;
	XvwPortGadget xobj = kwidget(widget);

	if (widget == NULL)
	   return;
	else if (widget == iportobj || widget == oportobj)
	   pixel = xobj->port.selected_pixel;
	else if (xobj->port.type == KPORT_DATAFLOW &&
	         xobj->port.dav && xobj->port.selected)
	   pixel = xobj->port.dav_pixel;
	else if (xobj->port.type == KPORT_CONTROLFLOW)
	   pixel = xobj->port.control_pixel;
	else if (xobj->port.type == KPORT_PARAMETER)
	   pixel = xobj->port.parameter_pixel;
	else if (xobj->port.type == KPORT_DATAFLOW && xobj->port.selected)
	   pixel = xobj->port.required_pixel;
	else if (xobj->port.type == KPORT_DATAFLOW && !xobj->port.selected)
	   pixel = xobj->port.optional_pixel;
	else
	   return;

	if (pixel != xobj->manager.background)
	   xvw_set_attribute(xvw_object(widget), XVW_BACKGROUND, pixel);
}

/*-----------------------------------------------------------
|
|  Routine Name: UpdateOptSel - update optionally selected port
|
|       Purpose: Update the optionally selected port 
|         Input: xobj - the port object
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: May 22, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void UpdateOptSel(
   Widget widget,
   int    update)
{
	XvwPortGadget xobj = kwidget(widget);

	kform_struct *selection;
	int optional, selected, orig = xobj->port.selected;

	if ((selection = xobj->port.selection) != NULL)
	{
	   xvf_get_attributes(xobj->port.selection,
		XVF_OPTIONAL, &optional,
		XVF_OPTSEL, &selected,
		NULL);

	   if (optional == FALSE)
	   {
	      xobj->port.selected = TRUE;
	   }
	   else if (selected == 0 || selected == 1)
	   {
	      if (update)
	      {
                 /* selection is not an ME, MI, or loose group member */
	         if (selection->Selptr->back_group == NULL)
	            xvf_set_attribute(selection,XVF_OPTSEL,xobj->port.selected);

                 /* selection is a group member */
	         else if ((xobj->port.selected && selected != 1) ||
			  (!xobj->port.selected && selected != 0))
		    UpdateGroupOptsel(selection, xobj->port.selected);
	      }
	      else
	         xobj->port.selected = selected;
	   }
	   else if (selected == 2 || selected == 3)
	   {
	      if (update)
	      {
	         if (xobj->port.direction == KPORT_INPUT)
		    selected = xobj->port.kludge ? 2 : 3,
		    xobj->port.dav = selected == 2;

	         /* selection is not an ME, MI, or loose group member */
	         if (selection->Selptr->back_group == NULL)
		    xvf_set_attribute(selection, XVF_OPTSEL, selected);

	         /* selection is a group member */
	         else if ((xobj->port.selected && selected != 2) ||
			  (!xobj->port.selected && selected != 3))
		    UpdateGroupOptsel(selection, selected);
	      }
	      xobj->port.selected = (selected == 2);
	   }
	}

	if (xobj->port.selected && xobj->port.type == KPORT_DATAFLOW &&
	    xobj->port.direction == KPORT_OUTPUT && !xobj->port.filename)
	{
	   char *tmp;

	   /*
	    *  If no filename exists create a temporary file for the output
	    *  connection.
	    */
	   xobj->port.temporary = TRUE;
	   tmp = ktempnam(kgetenv("KHOROS_TEMPFILE"), "io");
	   xvw_set_attributes(xvw_object(widget),
		XVW_PORT_DAV, FALSE,
		XVW_PORT_FILENAME, tmp,
		NULL);
	   kfree(tmp);
	}

	if (orig != xobj->port.selected)
	   UpdatePixmap(widget);
}

/*-----------------------------------------------------------
|
|  Routine Name: UpdateGroupOptsel - an Overcome Kforms Limitations Routine
|
|       Purpose: xvforms / kforms does not support the turning OFF of
|                an ME, MI, or Loose group member (as the user has
|                no way to do this).  However, xvlang needs to turn OFF
|                a group member when an input connection to a group toggle
|                member is deleted.  This routine was written to overcome
|                this limitation.
|         Input: kformstruct - the kformstruct associated with the
|                              selection that got turned off
|                selected    - flag that gets turned on or off;  if the
|                              selected field is ON, the routine just
|                              returns, since kforms / xvforms handles that
|                              case just fine.
|       Returns: 
|    Written By: Danielle Argiro
|          Date: June 5, 1996
|
------------------------------------------------------------*/

static void UpdateGroupOptsel(
   kform_struct *kformstruct,
   int          selected)
{
	kselection *selection, *group;
	int done = FALSE;

	/*
	 * turning selection ON -- don't have to do this by hand,
	 * since kforms/xvforms handles that case just fine: just
	 * set XVF_OPTSEL normally 
	 */
	if (selected == 1 || selected == 2) 
	{
	    xvf_set_attribute(kformstruct, XVF_OPTSEL, selected);
	    return;
	}

	/*
	 * this selection is not a member of a group -- this
	 * routine is not relevant in that case
	 */
	selection = kformstruct->Selptr;
	if (selection->back_group == NULL) return;

	/*
	 * this selection is the member of an ME group - simply
	 * turn on the next group member that we can find that
	 * is not this selection, and also not another inputfile selection.
	 */
	if (selection->back_group->type == KUIS_MUTEXCL ||
	    selection->back_group->type == KUIS_GROUP)
	{
	   group = selection->back_group->group_next;
	   while (group != NULL && !done)
	   {
		if (group != selection && group->type != KUIS_INPUTFILE)
		{
		    if (group->type == KUIS_MUTINCL ||
                        group->type == KUIS_GROUP)
			UpdateGroupOptsel(group->group_next->back_kformstruct, 
				          selected == 0 ? 1:2);
		    else xvf_set_attribute(group->back_kformstruct, XVF_OPTSEL, 
				      selected == 0 ? 1:2);
		    done = TRUE;
	        }
		group = group->next;
	   }
	  
	   /* 
	    * no luck finding an alternate group member to select;
	    * go ahead and just set XVF_OPTSEL, knowing that this
	    * will still exhibit the bug of the inputfile selection with the
	    * deleted connection having it's optional box still on,
	    * but no filename inside
	    */
	   if (group == NULL && !done)
	       xvf_set_attribute(kformstruct, XVF_OPTSEL, selected);
	}

}

/*-----------------------------------------------------------
|
|  Routine Name: UpdateFilename - update filename associated with the port
|
|       Purpose: Update the filename associated with the port
|         Input: 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: May 22, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void UpdateFilename(
   Widget widget,
   int    update)
{
	XvwPortGadget xobj = kwidget(widget);

	int  i;
	kform_struct *selection;
	xvobject port = xvw_object(widget);
	char *name, *filename, temp[KLENGTH];


	if ((selection = xobj->port.selection) == NULL)
	   return;

	xvf_get_attribute(selection, XVF_FILE_NAME, &name);
	filename = kstring_cleanup(name, temp); kfree(name);
	if (kstrcmp(xobj->port.filename, filename) == 0)
	   return;

	if (update == FALSE)
	{
	   if (xobj->port.temporary == TRUE)
              kunlink(xobj->port.filename);

	   kfree(xobj->port.filename);
	   xobj->port.filename = kstrdup(filename);
	   xobj->port.temporary = FALSE;
	}
	else
	{
	   filename = xobj->port.filename;
	   xvf_set_attribute(selection, XVF_LITERAL, filename);
	}

	if (xobj->port.direction == KPORT_OUTPUT)
	{
	   for (i = 0; i < xobj->port.num; i++)
	   {
	      xvobject iport = xobj->port.iports[i];
	      if (filename != NULL)
	         xvw_set_attribute(iport, XVW_PORT_FILENAME, filename);
	      else
	         xvw_set_attribute(port, XVW_PORT_DELETE_CONNECTION, iport);
	   }
	}
	else if (xobj->port.direction == KPORT_INPUT && update == FALSE)
	{
	   for (i = 0; i < xobj->port.num; i++)
	   {
	      xvobject oport = xobj->port.oports[i];
	      xvw_set_attribute(oport, XVW_PORT_DELETE_CONNECTION, port);
	   }

	   if (filename != NULL)
	      xvw_set_attribute(port, XVW_PORT_DAV, TRUE);
	}

	if (update == FALSE)
	{
	   if (filename != NULL)
	      xvw_set_attribute(port, XVW_PORT_SELECTED, TRUE);
	   else
	      xvw_set_attributes(port, 
				 XVW_PORT_SELECTED, FALSE,
			         XVW_PORT_DAV,     FALSE,
				 NULL);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: CopyFilename - copies the data (if needed) from remote
|				machine.
|
|       Purpose: Copies the data from a remote connetions...
|         Input: widget - the port to copy data for
|    Written By: Mark Young
|          Date: Jul 2, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void CopyFilename(
   Widget widget)
{
	XvwPortGadget ixobj = kwidget(widget);


	if (ixobj->port.direction != KPORT_INPUT || ixobj->port.dav == FALSE)
	   return;

#if 0
        XvwPortGadget oxobj;
	int  i;
	char *machine, *filename, infile[KLENGTH], outfile[KLENGTH];

	for (i = 0; i < ixobj->port.num; i++)
	{
	   oxobj = (XvwPortGadget) xvw_widget(ixobj->port.oports[i]);
	   if (oxobj->port.temporary == TRUE &&
	       kstrcmp(ixobj->port.machine, oxobj->port.machine) != 0)
	   {
	      kremote_file(oxobj->port.machine, oxobj->port.filename, infile);
	      kremote_file(ixobj->port.machine, ixobj->port.filename, outfile);
	      kcopyfile(infile, outfile);
	   }
	}
#endif
}

/*-----------------------------------------------------------
|
|  Routine Name: ModifiedCallback - routine for maintaining modified callback
|
|       Purpose: This routine is used to maintain the modified callback
|         Input: port - the input port
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jun 14, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ModifiedCallback(
   xvobject port,
   kaddr    client_data,
   kaddr    call_data)
{
	int modified = *((int *) call_data);
	XvwPortGadget xobj = (XvwPortGadget) client_data;

	int i;

	if (modified == FALSE)
	{
	   for (i = 0; i < xobj->port.num; i++)
	   {
	      xvobject iport, connection = xobj->port.connections[i];
	      xvw_get_attribute(connection, XVW_CONNECTION_END, &iport);
	      xvw_get_attribute(iport, XVW_PORT_MODIFIED, &modified);
	      if (modified == TRUE)
	         break;
	   }
	}

	/*
	 *  If the status hasn't changed then just return.
	 */
	if (modified == xobj->port.modified)
	   return;

	/*
	 *  Update the status an call the modified callbacks.
	 */
	xobj->port.modified = modified;
	XtCallCallbacks((Widget) xobj, XVW_PORT_MODIFIED_CALLBACK, &modified);
}

/*-----------------------------------------------------------
|
|  Routine Name: PortGetInfo - routine for getting port information
|
|       Purpose: This routine is called when we want to get port information
|         Input: port - the port
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Dec 12, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
int PortGetInfo(
   xvobject port,
   char     **varname,
   char     **filename,
   int	    *temporary,
   int	    *selected,
   int      *direction,
   int      *type,
   int      *num,
   xvobject **connections)
{
	XvwPortGadget xobj = (XvwPortGadget) xvw_widget(port);


	if (num) *num = xobj->port.num;
	if (temporary) *temporary = xobj->port.temporary;
	if (type)      *type      = xobj->port.type;
	if (selected)  *selected  = xobj->port.selected;
	if (direction) *direction = xobj->port.direction;
	if (filename)  *filename  = xobj->port.filename;
	if (connections) *connections = xobj->port.connections;

	if (varname && xobj->port.type == KPORT_CONTROLFLOW)
	{
	   if (xobj->port.direction == KPORT_INPUT)
	      *varname = kstrdup("in");
	   else
	      *varname = kstrdup("out");
	}
	else if (varname && xobj->port.type == KPORT_DATAFLOW)
	{
	   *varname = NULL;
	   if (xobj->port.selection)
	   {
	      xvf_get_attribute(xobj->port.selection, XVF_VARIABLE, varname);
	      *varname = kstring_cleanup(*varname, *varname);
	   }
	   if (*varname == NULL) *varname = kstrdup("");
	}
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: PortCreateConnection - routine for creating connections
|					between two ports
|
|       Purpose: This routine is called when we want to create a connection
|         Input: iport - the input port
|                oport - the output port
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Dec 12, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
int PortCreateConnection(
   xvobject iport,
   xvobject oport)
{
	/*
	 * ixobj is the input port on the connection end.
	 * oxobj is the output port on the connection begin.
	 */
	XvwPortGadget ixobj = (XvwPortGadget) xvw_widget(iport);
	XvwPortGadget oxobj = (XvwPortGadget) xvw_widget(oport);

	int      i;
	unsigned long mask;
	xvobject connection, parent;

	if ((parent = xvw_object(oxobj->port.connection_parent)) == NULL)
	   parent = xvw_parent(oport);

	if (ixobj->port.type == KPORT_DATAFLOW)
	   connection = xvw_create_connection(parent, "PortConnection");
	else if (ixobj->port.type == KPORT_CONTROLFLOW)
	   connection = xvw_create_connection(parent, "ControlConnection");
	else if (ixobj->port.type == KPORT_PARAMETER)
	   connection = xvw_create_connection(parent, "ParameterConnection");
	else
	   connection = xvw_create_connection(parent, "Connection");

	xvw_set_attributes(connection,
		XVW_CONNECTION_END,   iport,
		XVW_CONNECTION_BEGIN, oport,
		NULL);
        xvw_insert_callback(iport, XVW_PORT_MODIFIED_CALLBACK, FALSE,
			ModifiedCallback, oxobj);
        xvw_insert_callback(iport, XVW_DESTROY, FALSE,
			IportRemoveConnection, oxobj);
        xvw_insert_callback(oport, XVW_DESTROY, FALSE,
			OportRemoveConnection, ixobj);

	mask = ButtonPressMask | ButtonReleaseMask |
	       EnterWindowMask | LeaveWindowMask;
	xvw_insert_event(connection, mask, ConnectionHandler,oport, KLIST_HEAD);

	/*
	 *  Here is where we prevent multiple input connections...
	 */
	if (ixobj->port.num > 0)
	{
	    for (i = ixobj->port.num-1; i >= 0; i--)
		xvw_set_attribute(ixobj->port.oports[i], 
		                  XVW_PORT_DELETE_CONNECTION, iport);
	}
	/*
	 *  Here we prevent multiple output connections for non-permanent
	 *  transports
	 */
	xvw_set_attribute(oport, XVW_PORT_SELECTED, TRUE);
	if (kfile_getpermanence(oxobj->port.filename) == FALSE)
	{
	    for (i = oxobj->port.num-1; i >= 0; i--)
		xvw_set_attribute(oport, XVW_PORT_DELETE_CONNECTION, 
			          oxobj->port.iports[i]);
	}


	oxobj->port.iports = (xvobject *)karray_add(oxobj->port.iports,
		KSTRUCT, iport, oxobj->port.num);
	ixobj->port.oports = (xvobject *)karray_add(ixobj->port.oports,
		KSTRUCT, oport, ixobj->port.num);
	oxobj->port.connections = (xvobject *)karray_add(oxobj->port.connections,
		KSTRUCT, connection, oxobj->port.num++);
	ixobj->port.connections = (xvobject *)karray_add(ixobj->port.connections,
		KSTRUCT, connection, ixobj->port.num++);

	xvw_set_attributes(iport,
		XVW_PORT_DAV, oxobj->port.dav,
		XVW_PORT_FILENAME, oxobj->port.filename,
		XVW_PORT_SELECTED, TRUE,
		NULL);

	if (oxobj->port.type == KPORT_DATAFLOW)
	{
	   xvw_set_attribute(connection, XVW_FOREGROUND, oxobj->port.dav ?
			oxobj->port.dav_pixel : oxobj->port.required_pixel);
	}
	return(TRUE);
}


/************************************************************
*
*  Routine Name: xvw_create_port - create a port object
*
*       Purpose: The purpose of the Port object is to provide a general
*		 mechanism for representing a "port" within a visual program.
*		 Typically these are used by the glyph to represent input
*		 and output connections within the visual program.  Normally
*		 the port is created by other visual programming components,
*		 such as the Node object, but is provided so that other
*		 styles of visual programs can be created.
*
*		 The Port object is really available to reduce the complexity
*		 of the application programmer in dealing with the user
*		 interactions of connecting Glyphs together.  It also
*		 manages the complexity of representing the connections
*		 and the UIS that is used to represent the Port.  So
*		 if the user updates the filename represented by the
*		 Port, and the Port is an output, then the contains
*		 of the Ports connected to the output, are updated
*		 appropriately.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - the name with which to reference the object
*        Output: None
*       Returns: The port object on success, NULL on failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Nov 22, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*******************************************************************/

xvobject xvw_create_port(
   xvobject parent,
   char     *name)
{
	xvobject object;


	object = xvw_create(parent, FALSE, TRUE, name, PortGadgetClass);
	return(object);
}
