 /*
  * Khoros: $Id: ToolboxMenu.c,v 1.25.2.2 1997/05/24 19:04:27 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: ToolboxMenu.c,v 1.25.2.2 1997/05/24 19:04:27 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

#include "copyright.h"          /* Khoros copyright */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros ToolboxMenu Widget
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Realize()
   >>>>                 ConstraintInitialize()
   >>>>                 Destroy()
   >>>>                 SubcatCallback()
   >>>>                 PaneCallback()
   >>>>                 MenuCallback()
   >>>>   Public:
   >>>>                 xvw_create_toolboxmenu()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvlang/ToolboxMenuP.h>

static void ClassInitialize	 PROTO((void));
static void Initialize		 PROTO((Widget, Widget, ArgList, Cardinal *));
static void Realize      PROTO((Widget, XtValueMask *, XSetWindowAttributes *));
static void ConstraintInitialize PROTO((Widget, Widget, ArgList, Cardinal *));
static void Destroy		 PROTO((Widget));
static int  CompositeInsertPosition PROTO((Widget));

static void SubcatCallback	PROTO((xvobject, kaddr, kaddr));
static void PaneCallback	PROTO((xvobject, kaddr, kaddr));
static void MenuCallback	PROTO((xvobject, kaddr, kaddr));


#define ToolboxMenuName	  "Toplevel ToolboxMenu"

/*-------------------------------------------------------------------*
|
|   Full class attributes
|
--------------------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_TOOLBOXMENU_CREATE,    NULL,   XtRCallback, NULL},
};


/*-------------------------------------------------------------------*
|
|   Full class record constant
|
--------------------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwToolboxMenuWidgetRec, toolboxmenu.field)

static XtResource resources[] = { 
{XVW_TOOLBOXMENU_CREATE, NULL, XtRCallback, sizeof(XtPointer),
      offset(menu_callback), XtRCallback, (XtPointer) NULL},

{XVW_BUFFER_DIST, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwToolboxMenuWidgetRec, manager.buffer_dist),
        XtRImmediate, (XtPointer) 0},
{XVW_DEF_HORIZ_DIST, NULL, XtRPosition, sizeof(Position),
        XtOffsetOf(XvwToolboxMenuWidgetRec, manager.horiz_offset),
        XtRImmediate, (XtPointer) 10},
{XVW_DEF_VERT_DIST, NULL, XtRPosition, sizeof(Position),
        XtOffsetOf(XvwToolboxMenuWidgetRec, manager.vert_offset),
        XtRImmediate, (XtPointer) 3},
{XVW_ROWCOL_PACKING, NULL, XtRInt, sizeof(int),
        XtOffsetOf(XvwToolboxMenuWidgetRec, rowcol.packing),
        XtRImmediate, (XtPointer) KROWCOL_PACK_TIGHT},
{XtNinsertPosition, XtCInsertPosition, XtRFunction, sizeof(XtOrderProc),
        XtOffsetOf(XvwToolboxMenuWidgetRec, composite.insert_position),
        XtRImmediate, (XtPointer) CompositeInsertPosition},
};
#undef offset


/*-------------------------------------------------------------------*
|
|   Class Declaration for ToolboxMenu Widget
|
--------------------------------------------------------------------*/

#define superclass (&xvwRowColWidgetClassRec)

XvwToolboxMenuWidgetClassRec xvwToolboxMenuWidgetClassRec =
{
  {
    (WidgetClass) superclass,		/* superclass		  */	
    "ToolboxMenu",			/* class_name		  */
    sizeof(XvwToolboxMenuWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    Realize,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    NULL,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    XtInheritQueryGeometry,		/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwToolboxMenuWidgetConstraintsRec),  /* constraint_size	  */
    ConstraintInitialize,		    /* initialize	  */
    NULL,				    /* destroy		  */
    NULL,				    /* set_values	  */
    NULL,				    /* extension	  */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,                        /* pre-layout routine     */
    XtInheritLayout,                        /* post-layout routine    */
    XtInheritChangeSel,			    /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  },  /* XvwManagerWidgetClass fields initialization */
  {
    NULL,  		                    /* extension          */
  },  /* XvwRowColWidgetClass fields initialization */
  {
    NULL,  		                    /* extension          */
  },  /* XvwToolboxMenuWidgetClass fields initialization */
};
#undef superclass

  /* for public consumption */
WidgetClass xvwToolboxMenuWidgetClass = (WidgetClass) &xvwToolboxMenuWidgetClassRec;


/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for widget definition
|
--------------------------------------------------------------------*/

#define kwidget(widget)	     (XvwToolboxMenuWidget) (widget)


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an
|                instance of a XvwToolboxMenuWidget class has been created.
|                This is where the attributes for the class are
|		 initialized. 
|
|    Written By: Mark Young
|          Date: Jul 02, 1994
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
	xvlang_cache_pixmaps();
	xvw_init_attributes(xvwToolboxMenuWidgetClass, attributes,
		XtNumber(attributes), NULL, 0, NULL);
	xvw_load_resources("$DESIGN/objects/library/xvlang/app-defaults/ToolboxMenu");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|	Purpose: This method will set up the initial toolboxmenu widget
|		 instance.
|
|	  Input: request - not used
|                new1     - widget instance after initialization
|    Written By: Mark Young
|          Date: Jul 02, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwToolboxMenuWidget xwid = kwidget(new1);

	/*
	 *  Set the kludge count to 0.  Don't understand?  Check it out in
	 *  insert position routine below.
	 */
	xwid->toolboxmenu.count = 0;
	xwid->toolboxmenu.menulist = NULL;
}

/*-----------------------------------------------------------
|
|  Routine Name: Realize
|
|       Purpose: This method will bootstrap the ToolboxList on realization
|                rather than initialization.  This should hopefully be
|		 faster.
|
|         Input: widget     - the widget to realize
|                valuemask  - determines the window attributes being passed in
|                attributes - the list of window attributes to be set
|    Written By: Mark Young
|          Date: Jun 4, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Realize(
   Widget               widget,
   XtValueMask          *valuemask,
   XSetWindowAttributes *attrib)
{
	xvobject object = xvw_object(widget);

	klist    *category;
	char     *name;
	xvobject button, menu, menubutton;

	kannounce(XVLANG, "Initialize", "Creating Walking Menus for toolboxes");
	menubutton = xvw_create_menubutton(object, ToolboxMenuName);
	xvw_set_attribute(menubutton, XVW_LABEL, "Glyphs");

        xvl_get_contents(&category, NULL);
        xvw_get_attribute(menubutton, XVW_MENUBUTTON_MENU, &menu);
        while (category != NULL)
        {
	   name = ktoken_to_string((ktoken) category->identifier);
	   button = xvw_create_menubutton(menu, name);
	   xvw_set_attribute(button, XVW_LABEL, name);
	   xvw_add_callback(button, XVW_BUTTON_SELECT, SubcatCallback, widget);

           category = klist_next(category);
        }
	kannounce(XVLANG, "Initialize", "");

	/*
	 *  Now do the important thing.  Realize the Node so that we can
	 *  get on with it.
	 */
        (*xvwToolboxMenuWidgetClass->core_class.superclass->core_class.realize)
                (widget, valuemask, attrib);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: destroy the XvwToolboxMenuWidget instance...
|         Input: widget - the toolboxmenu widget being destroyed 
|    Written By: Mark Young
|          Date: Feb 17, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	XvwToolboxMenuWidget xwid = kwidget(widget);

	klist_free(xwid->toolboxmenu.menulist, NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: ConstraintInitialize
|
|	Purpose: Constraint initialize.
|
|	  Input: request - not used
|                new1     - widget instance after initialization
|    Written By: Mark Young
|          Date: Jul 02, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ConstraintInitialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwToolboxMenuWidget xwid = kwidget(XtParent(request));
	xvobject object    = xvw_object(new1);

	xvw_set_attributes(object,
		XVW_BACKGROUND,   xwid->core.background_pixel,
		XVW_BORDER_WIDTH, 0,
		NULL);

	if (xvw_check_subclass(object, ButtonWidgetClass))
	{
	   xvw_set_attributes(object,
		XVW_BUTTON_SELECT_TYPE,   KSHADOW_OUT,
		XVW_BUTTON_UNSELECT_TYPE, KSHADOW_OFF,
		NULL);
	}

	if (kstrncasecmp(XtName(new1), "Help", (size_t) 4) == 0)
	{
	   xvw_set_attributes(object,
		XVW_LEFT_OF, NULL,
		XVW_ROWCOL_ACTIVE, FALSE,
		XVW_YPOSITION, xwid->manager.shadow_thickness,
		NULL);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: CompositeInsertPosition
|
|	Purpose: Composite insert position.
|
|	  Input: child - the child to be inserted.
|    Written By: Mark Young
|          Date: Jul 02, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static int CompositeInsertPosition(
   Widget   child)
{
	XvwToolboxMenuWidget xwid = kwidget(XtParent(child));


	if (kstrcasecmp(XtName(child), ToolboxMenuName) == 0 ||
	    kstrcasecmp(XtName(child), "Help") == 0)
	{
	   xwid->toolboxmenu.count++;
	   return(xwid->composite.num_children);
	}
	else
	   return(xwid->composite.num_children - xwid->toolboxmenu.count);
}

/*-----------------------------------------------------------
|
|  Routine Name: SubcatCallback
|
|       Purpose: This routine will create the subcategory menu when
|                activated.
|
|         Input: object      - which button being manipulated
|                client_data - the button widget
|                call_data   - int representing the index
|    Written By: Mark Young
|          Date: Jul 02, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void SubcatCallback(
   xvobject menubutton,
   kaddr    client_data,
   kaddr    call_data)
{
	Widget   widget = (Widget) client_data;

	xvobject menu, button;
	char     *name, *catname;
	klist    *category, *subcategory;

        xvw_get_attribute(menubutton, XVW_MENUBUTTON_MENU, &menu);
	catname = xvw_name(menu);

	/* remove the callback so that we don't recreate this menu again */
	xvw_remove_callback(menubutton,XVW_BUTTON_SELECT,SubcatCallback,widget);
 
        /*  Need to locate the correct SubCategoryList */
        xvl_get_contents(&category, NULL);
        category = klist_locate(category, kstring_to_token(catname));
        subcategory = (klist *) klist_clientdata(category);

	while (subcategory != NULL)
	{
	   name = ktoken_to_string((ktoken) subcategory->identifier);
	   button = xvw_create_menubutton(menu, name);
	   xvw_set_attribute(button, XVW_LABEL, name);
	   xvw_add_callback(button, XVW_BUTTON_SELECT, PaneCallback, widget);

           subcategory = klist_next(subcategory);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: PaneCallback
|
|       Purpose: This routine will create the pane menu when activated.
|
|         Input: object      - which button being manipulated
|                client_data - the button widget
|                call_data   - int representing the index
|    Written By: Mark Young
|          Date: Jul 02, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void PaneCallback(
   xvobject menubutton,
   kaddr    client_data,
   kaddr    call_data)
{
	Widget   widget = (Widget) client_data;
	XvwToolboxMenuWidget xwid = kwidget(widget);

	xvobject menu, button, parent;
	char     *catname, *subname, **info;
	klist    *category, *subcategory, *panelist, *menulist;


	parent = xvw_parent(menubutton);
	xvw_get_attribute(menubutton, XVW_MENUBUTTON_MENU, &menu);
	catname = xvw_name(parent); subname = xvw_name(menu);
	menulist = xwid->toolboxmenu.menulist;

	/* remove the callback so that we don't recreate this menu again */
	xvw_remove_callback(menubutton, XVW_BUTTON_SELECT, PaneCallback,widget);


	/*  need to locate the correct PaneList */
        xvl_get_contents(&category, NULL);
	category = klist_locate(category, kstring_to_token(catname));
	subcategory = (klist *) category->client_data;
	subcategory = klist_locate(subcategory, kstring_to_token(subname));
	panelist = (klist *) subcategory->client_data;

	while (panelist != NULL)
	{
	   info = (char **) klist_clientdata(panelist);
	   button = xvw_create_button(menu, info[0]);
	   xvw_set_attribute(button, XVW_LABEL, info[0]);
	   xvw_add_callback(button, XVW_BUTTON_SELECT, MenuCallback, widget);
	   menulist = klist_insert(menulist, button, panelist, KLIST_HEAD,TRUE);

	   panelist = klist_next(panelist);
	}
	xwid->toolboxmenu.menulist = menulist;
}

/*-----------------------------------------------------------
|
|  Routine Name: MenuCallback
|
|       Purpose: This routine will be called when the menu callback is
|                activated.
|
|         Input: object      - which button being manipulated
|                client_data - the button widget
|                call_data   - int representing the index
|    Written By: Mark Young
|          Date: Jul 02, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void MenuCallback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	Widget   widget = (Widget) client_data;
	XvwToolboxMenuWidget xwid = kwidget(widget);

	xvw_program_info program_info;
	klist *entry = klist_locate(xwid->toolboxmenu.menulist, object);
	klist *panelist = (klist *) klist_clientdata(entry);


	/*
	 *  Go ahead and load the program info structure
	 */
	if (!widget || !xvl_load_contents(panelist, &program_info))
	   return;

	XtCallCallbacks(widget, XVW_TOOLBOXMENU_CREATE, &program_info);
}


/************************************************************
*
*  Routine Name: xvw_create_toolboxmenu - create a toolbox menu object
*
*       Purpose: The toolbox menu object allows the user to select an 
*		 operator according to its category/subcategory/operator 
*		 name.  In cantata, the toolbox menu object is used
*		 as the first method for glyph creation; the 
*		 Creates a toolbox menu object, that consists of three
*		 menu objects.  The first menu displays the different
*		 categories.  The second menu displays the sub-categories
*		 for the selected category.  And finally the third
*		 displays the operators for the selected sub-category.
*
*		 Two callbacks are provided which can be used to notify
*		 the programmer when the user selects or activates an operator
*		 from the menu.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - the name with which to reference the object
*        Output: None
*       Returns: The toolboxmenu object on success, NULL on failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Jul 02, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*******************************************************************/

xvobject xvw_create_toolboxmenu(
   xvobject parent,
   char     *name)
{
	xvobject object;


	object = xvw_create(parent, FALSE, TRUE, name, ToolboxMenuWidgetClass);
	return(object);
}
