 /*
  * Khoros: $Id: WorkRestore.c,v 1.23.2.2 1997/05/24 19:04:30 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: WorkRestore.c,v 1.23.2.2 1997/05/24 19:04:30 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>	              Khoros Workspace Widget Utilities
   >>>>   Static:
   >>>>			RestoreExtConnection()
   >>>>			RestoreConnection()
   >>>>  Private:
   >>>>			WorkspaceRestore()
   >>>>   Public:
   >>>>	
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvlang/WorkspaceP.h>
#include <xvlang/GlyphP.h>

extern xvobject clipboard;

/*-----------------------------------------------------------
|
|  Routine Name: RestoreExtConnection - Restore external connections
|
|       Purpose: This routine actually
|
|         Input: file  - the file the information is suppose to be read from
|		 line  - the information just read
|		 xoffset,yoffset - initial x & y offset
|		 nodes - the list of restore nodes
|        Output: ident - the node indentifier
|    Written By: Mark Young and Danielle Argiro
|          Date: Jun 24, 1996
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RestoreExtConnection(
    kfile    *file,
    xvobject parent,
    char     *line,
    int	     xoffset,
    int	     yoffset,
    klist    *nodes)
{
	int      i, ident, type;
	char label[KLENGTH];
	XvwGlyphWidget xwid;
	klist        *entry;
	xvobject      procedure;
	kform_struct *selection;
	xvobject object, extport, iport = NULL, oport = NULL;
	
	/* 
	 * restore address of the port on the outside glyph to which
	 * we need to make the connection
	 */
	kstrcpy(label, "");
	if (ksscanf(line, "ExtConnection:%[^:]:%p:%d:%d:", 
		     label, &object, &ident, &type) != 4)
	   return;

	/* locate procedure associated w/ ext. connection */
	if ((procedure = ProcedureLocate(xvw_widget(parent))) == NULL)
	   return;

	/* get entry for external port out of node list */
	if ((entry = klist_locate(nodes, (kaddr) ident)) == NULL)
	   return;

	/* the external port itself was saved as the clientdata of list entry */
	extport = (xvobject) klist_clientdata(entry);

	/* 
	 * get the selection that is associated with the external port
	 * (the same selection that is on the GUI of the procedure)
	 */
	xvw_get_attribute(extport, XVW_EXTPORT_SELECTION, &selection);

	/*
	 * search procedure glyph inputs and outputs for that selection
 	 */
	GlyphUpdateForm(xvw_widget(procedure));
	xwid = (XvwGlyphWidget) xvw_widget(procedure);
	if (type == KPORT_CONTROLFLOW)
	{
	    if (kstrcmp(label, "in") == 0)
	    {
                oport = object;
	        iport = xwid->node.cflow_in;
	    }
	    else 
	    {
                iport = object;
	        oport = xwid->node.cflow_out;
	    }
	}
	else if (kstrcmp(label, "in") == 0)
	{
	   oport = object;
	   for (i = 0; i < xwid->node.isize; i++)
	   {
	      if (xwid->glyph.isels[i] == selection)
	      {
		 iport = xwid->node.iports[i];
	         break;
	      }
	   }
	}
	else
	{
	   iport = object;
	   for (i = 0; i < xwid->node.osize; i++)
	   {
	      if (xwid->glyph.osels[i] == selection)
	      {
		 oport = xwid->node.oports[i];
	         break;
	      }
	   }
	}

	/*
	 *  connect port on procedure glyph to the original outside glyph's port
	 */
	if (iport && oport)
	   PortCreateConnection(iport, oport);
}

/*-----------------------------------------------------------
|
|  Routine Name: RestoreConnection - Restore the node connection
|
|       Purpose: This routine does the actually restoring a connection for
|		 a node.
|
|         Input: file  - the file the information is suppose to be read from
|		 line  - the information just read
|		 xoffset,yoffset - initial x & y offset
|		 nodes - the list of restore nodes
|        Output: ident - the node indentifier
|    Written By: Mark Young
|          Date: Aug 26, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RestoreConnection(
    kfile    *file,
    xvobject parent,
    char     *line,
    int	     xoffset,
    int	     yoffset,
    klist    *nodes)
{
	klist    *ilist, *olist;
	int      inum,   onum, val;
	xvobject inode, onode, iport, oport;
	char     ivar[KLENGTH], ovar[KLENGTH], identifier[KLENGTH],
		 type[KLENGTH];

	kstrcpy(identifier, ""); kstrcpy(type, "");
	if (ksscanf(line, "%[^:]:%d:%d:", identifier, &onum, &inum) != 3 &&
	    ksscanf(line, "%[^:]:%d:%[^:]:%d:%[^:]", identifier,
			&onum, ovar, &inum, ivar) != 5 &&
	    ksscanf(line, "%[^:]:%[^:]:%d:%[^:]:%d:%[^:]", identifier,
			type, &onum, ovar, &inum, ivar) != 6)
	{
	   return;
	}

	if (!kstring_subcmp(identifier, "Control"))
	{
	   val = KPORT_CONTROLFLOW;
	   kstrcpy(ivar, "in"); kstrcpy(ovar, "out");
	}
	else if (!kstrcmp(type, "data"))
	{
	   val = KPORT_DATAFLOW;
	}
	else if (!kstrcmp(type, "parameter"))
	   val = KPORT_PARAMETER;
	else if (!kstrcmp(type, "control"))
	   val = KPORT_CONTROLFLOW;
	else if (!kstring_subcmp(identifier, "Node"))
	   val = KPORT_DATAFLOW;
	   
	if ((ilist = klist_locate(nodes, (kaddr) inum)) != NULL)
	   inode = (xvobject) klist_clientdata(ilist);
	else inode = NULL;

	if ((olist = klist_locate(nodes, (kaddr) onum)) != NULL)
	   onode = (xvobject) klist_clientdata(olist);
	else onode = NULL;

	iport = xvl_locate_port(inode, val, ivar, NULL);
	oport = xvl_locate_port(onode, val, ovar, NULL);
	if (iport && oport)
	   PortCreateConnection(iport, oport);
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceRestore
|
|       Purpose: WorkspaceRestore is used to restore the node's within a
|		 workspace.
|
|         Input: widget - the workspace widget to restore to
|		 file   - the open file descriptor in which to read the
|			  workspace from
|		 xoffset - the x offset to add to each node
|		 yoffset - the y offset to add to each node
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Nov 30, 1993
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int WorkspaceRestore(
   Widget widget,
   kfile  *file,
   int    xoffset,
   int    yoffset)
{
	XvwWorkspaceWidget xwid = (XvwWorkspaceWidget) widget;

	double   version = -1.0;
	klist    *list, *nodes = NULL;
	xvobject workspace, node, object;
	char     line[10*KLENGTH], error[KLENGTH];
	int      ident, add_selected = FALSE, num, i = 0;

	static   int count = 0;


        /*
         *  If there are children in the workspace then go ahead and unselect
         *  them and then set a flag so that we can restore the workspace
         *  selected.
         */
	count++;
	workspace = xvw_object(widget);
	object = xwid->viewport.plane;
        xvw_get_attribute(object, XVW_NUM_CHILDREN, &num);
        if (num > 0 || workspace == clipboard)
        {
           xvw_set_attribute(object, XVW_SELECT_DELETE_ALL, TRUE);
           add_selected = TRUE;
        }

	while (kfgets(line, 10*KLENGTH, file) != NULL)
	{
	   if (line[0] == '#' || line[0] == '\0' ||
	       kstring_cleanup(line, line) == NULL)
	      continue;

	   node = NULL; i++;
	   if (kstring_subcmp(line, "Glyph:") == 0 ||
	       kstring_subcmp(line, "Conditional:") == 0 ||
	       kstring_subcmp(line, "ProcedureBegin:") == 0 ||
	       kstring_subcmp(line, "LoopBegin:") == 0 ||
	       kstring_subcmp(line, "ExtPort:") == 0)
	   {
	      node = NodeRestore(workspace, file, line, xoffset,yoffset,&ident);
	   }
	   else if (kstring_subcmp(line, "WorkspaceBegin:") == 0)
	   {
	      (void) ksscanf(line, "WorkspaceBegin: %lg", &version);
	      if (version > WORKSPACE_VERSION && count == 1)
	      {
	         kerror(XVLANG, "WorkspaceRestore", "Invalid Workspace. "
		        "'%s' appears to be a newer workspace than what is "
		        "expected. The workspace you are trying to restore "
		        "is version %g, but %s %d.%d expects a workspace "
		        "version of %g.",
			kfile_filename(kfileno(file)), version, 
                        kprog_get_program(),  PRODUCT_RELEASE_VERSION,
			PRODUCT_RELEASE_MAJOR, WORKSPACE_VERSION);
	      }
	   }
	   else if (kstring_subcmp(line, "NodeConnection:") == 0 ||
		    kstring_subcmp(line, "ControlConnection:") == 0 ||
		    kstring_subcmp(line, "Connection:") == 0)
	   {
	      RestoreConnection(file, workspace, line, xoffset,
				yoffset, nodes);
	   }
	   else if (kstring_subcmp(line, "ExtConnection:") == 0)
	   {
	      RestoreExtConnection(file, workspace, line, xoffset,
				yoffset, nodes);
	   }
	   else if (kstring_subcmp(line, "WorkspaceEnd:") == 0 ||
		    kstring_subcmp(line, "ProcedureEnd:") == 0 ||
		    kstring_subcmp(line, "LoopEnd:") == 0)
	   {
	      break;
	   }
	   else if (kstring_subcmp(line, "VariablesBegin:") == 0)
	   {
	      long expr_id = (long) workspace;

	      while (kfgets(line, 10*KLENGTH, file) != NULL)
	      {
		 if (kstring_cleanup(line, line) == NULL || line[0] == '#')
		    continue;

		 if (kstring_subcmp(line, "VariablesEnd:") != 0)
		    kexpr_evaluate_generic(expr_id, line, KSTRING, NULL,error);
		 else
		    break;
	      }
	      xvl_expr_call_callback(expr_id, XVL_EXPR_UPDATE);
	   }
	   else if (kstring_subcmp(line, "AnnotationsBegin") == 0)
	   {
	      xvw_restore_annotations(object, file);
	   }
	   else i--;

	   if (node != NULL)
	      nodes = klist_add(nodes, (kaddr) ident, node);
 	}

	if (add_selected == TRUE)
	{
	   for (list = nodes; list != NULL;  list = klist_next(list))
	   {
	       node = (xvobject) klist_clientdata(list);
	       xvw_set_attribute(object, XVW_SELECT_ADD, node);
	   }
	}
	klist_free(nodes, NULL);

	count--;
	if (i == 0 && count == 0)
	{
	   kerror(XVLANG, "WorkspaceRestore", "Invalid Workspace.  Sorry, but "
		  "'%s' does not appear to be a valid workspace.  Please "
		  "verify that this is a valid Khoros workspace and try "
		  "again.", kfile_filename(kfileno(file)));
	}
	else if (version < WORKSPACE_VERSION && count == 0)
	{
	   if (kprompt(KSTANDARD, "Update", "Cancel", 1, "The workspace '%s' "
	       "appears to be older than the workspace version (%g) that is "
	       "expected by Khoros %d.%d. Do you wish update the workspace "
	       "to version %g at this time?",
		      kfile_filename(kfileno(file)), WORKSPACE_VERSION,
		      PRODUCT_RELEASE_VERSION, PRODUCT_RELEASE_MAJOR,
		      WORKSPACE_VERSION) == TRUE)
           {
              xvw_set_attribute(workspace, XVW_WORKSPACE_SAVEALL,
				kfile_filename(kfileno(file)));
           } 
	}
	return(TRUE);
}
