 /*
  * Khoros: $Id: Workspace.c,v 1.35.2.4 1997/04/29 22:31:58 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Workspace.c,v 1.35.2.4 1997/04/29 22:31:58 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>	              Khoros Workspace Widget
   >>>>  Private:
   >>>>   Static:
   >>>> 		ClassInitialize()
   >>>> 		Initialize()
   >>>> 		ConstraintInitialize()
   >>>> 		SetValues()
   >>>> 		SetValuesHook()
   >>>> 		Destroy()
   >>>> 		WorkspaceGUIFormCallback()
   >>>>   Public:
   >>>>			xvw_create_workspace()
   >>>>	
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvlang/WorkspaceP.h>

#define XK_LATIN1
#include <X11/keysymdef.h>

static void ClassInitialize        PROTO((void));
static void Initialize             PROTO((Widget, Widget, ArgList, Cardinal *));
static void ConstraintInitialize   PROTO((Widget, Widget, ArgList, Cardinal *));
static Boolean SetValues	   PROTO((Widget, Widget, Widget, ArgList,
					  Cardinal *));
static Boolean SetValuesHook	   PROTO((Widget, ArgList, Cardinal *));
static void    Destroy		   PROTO((Widget));

static void WorkspaceGUIFormCallback  PROTO((kform *, ksubform *, kaddr));

extern xvobject clipboard;

extern klist *procedurelist;

/*-------------------------------------------------------------------*
|
|   Full class attributes
|
--------------------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_WORKSPACE_WKSPGUI,		NULL,   XtRPointer,  NULL},
{XVW_WORKSPACE_FILENAME,	NULL,   XtRString,   NULL},
{XVW_WORKSPACE_RUN,	        NULL,   XtRInt,      XtRBoolean},
{XVW_WORKSPACE_SCHEDULE,	NULL,   XtRInt,      XtRBoolean},
{XVW_WORKSPACE_STOP,	        NULL,   XtRInt,      NULL},
{XVW_WORKSPACE_RUNMODE,	        NULL,   XtRInt,      NULL},
{XVW_WORKSPACE_STATUS,          NULL,   XtRInt,      XtRBoolean},
{XVW_WORKSPACE_RUN_CALLBACK,    NULL,   XtRCallback, NULL},
{XVW_WORKSPACE_ACTIVE_CALLBACK, NULL,   XtRCallback, NULL},

{XVW_GLYPH_ECHO,	 NULL,   XtRInt,     NULL},
{XVW_GLYPH_REPORTING,	 NULL,   XtRInt,     XtRBoolean},
{XVW_NODE_PLACEMENT,     NULL,   XtRInt,     XtRBoolean},
{XVW_NODE_LEAVE,         NULL,   XtRInt,     XtRBoolean},
{XVW_NODE_REMOTE_ENABLE, NULL,   XtRInt,     XtRBoolean},
{XVW_NODE_REMOTE_LOCATION, NULL, XtRString,  NULL},
{XVW_NODE_SIZE,          NULL,   XtRInt,     NULL},
{XVW_CONNECTION_TYPE,    NULL,   XtRInt,     NULL},
};


/*-------------------------------------------------------------------*
|
|   Full class record constant
|
--------------------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwWorkspaceWidgetRec, workspace.field)

static XtResource resources[] = { 
{XVW_WORKSPACE_WKSPGUI, NULL, XtRPointer, sizeof(XtPointer),
     offset(wkspgui), XtRImmediate, (XtPointer) NULL},
{XVW_WORKSPACE_FILENAME, NULL, XtRString, sizeof(String),
     offset(filename), XtRImmediate, (XtPointer) NULL},
{XVW_WORKSPACE_RUN_CALLBACK, NULL, XtRCallback, sizeof(XtPointer),
     offset(run_callback), XtRImmediate, (XtPointer) NULL},
{XVW_WORKSPACE_ACTIVE_CALLBACK, NULL, XtRCallback, sizeof(XtPointer),
     offset(active_callback), XtRImmediate, (XtPointer) NULL},
{XVW_WORKSPACE_RUN, NULL, XtRBoolean, sizeof(Boolean),
     offset(running), XtRImmediate, (XtPointer) FALSE},
{XVW_WORKSPACE_SCHEDULE, NULL, XtRBoolean, sizeof(Boolean),
     offset(scheduling), XtRImmediate, (XtPointer) FALSE},
{XVW_WORKSPACE_RUNMODE, NULL, XtRInt, sizeof(int),
     offset(runmode), XtRImmediate, (XtPointer) KWORKSPACE_RUNMODE_ONCE},
{XVW_WORKSPACE_STATUS, NULL, XtRBoolean, sizeof(Boolean),
        offset(status), XtRImmediate, (XtPointer) 0},

{XVW_GLYPH_ECHO, NULL, XtRInt, sizeof(int),
     offset(glyph_echo), XtRImmediate, (XtPointer) KGLYPH_ECHO_TO_CONSOLE},
{XVW_GLYPH_REPORTING, NULL, XtRBoolean, sizeof(Boolean),
     offset(glyph_reporting), XtRImmediate, (XtPointer) TRUE},
{XVW_NODE_PLACEMENT, NULL, XtRBoolean, sizeof(Boolean),
     offset(icon_placement), XtRImmediate, (XtPointer) TRUE},
{XVW_NODE_LEAVE, NULL, XtRBoolean, sizeof(Boolean),
     offset(icon_leave), XtRImmediate, (XtPointer) TRUE},
{XVW_NODE_SIZE, NULL, XtRBoolean, sizeof(Boolean),
     offset(icon_size), XtRImmediate, (XtPointer) KNODE_SIZE_LARGE},
{XVW_NODE_REMOTE_ENABLE, NULL, XtRBoolean, sizeof(Boolean),
     offset(icon_remote_enable), XtRImmediate, (XtPointer) FALSE},
{XVW_NODE_REMOTE_LOCATION, NULL, XtRString, sizeof(String),
     offset(icon_remote_location), XtRImmediate, (XtPointer) NULL},
{XVW_CONNECTION_TYPE, NULL, XtRInt, sizeof(int), 
     offset(connection_type), XtRImmediate,
	(XtPointer) KCONNECTION_TYPE_MANHATTAN},

{XVW_BACKGROUND, NULL, XtRPixel, sizeof(Pixel),
        XtOffsetOf(XvwWorkspaceWidgetRec, core.background_pixel),
        XtRString, (XtPointer) "#979797"},
{XVW_FOREGROUND_PIXEL, NULL, XtRPixel, sizeof(Pixel),
        XtOffsetOf(XvwWorkspaceWidgetRec, manager.foreground_pixel),
        XtRString, (XtPointer) "#eeeeee"},
{XVW_CANVAS_PIXEL, NULL, XtRPixel, sizeof(Pixel),
        XtOffsetOf(XvwWorkspaceWidgetRec, manager.grid_pixel),
        XtRString, (XtPointer) "#454545"},
{XVW_CANVAS_GRID, NULL, XtRInt, sizeof(int),
        XtOffsetOf(XvwWorkspaceWidgetRec, manager.gridtype),
        XtRImmediate, (XtPointer) KMANAGER_GRID_OFF},
{XVW_CANVAS_GRIDSIZE, NULL, XtRInt, sizeof(int),
        XtOffsetOf(XvwWorkspaceWidgetRec, manager.gridsize),
        XtRImmediate, (XtPointer) 20},
};
#undef offset


/*-------------------------------------------------------------------*
|
|   Class Declaration for Workspace Widget
|
--------------------------------------------------------------------*/

#define superclass (&xvwCanvasWidgetClassRec)

XvwWorkspaceWidgetClassRec xvwWorkspaceWidgetClassRec =
{
  {
    (WidgetClass) superclass,		/* superclass		  */	
    "Workspace",			/* class_name		  */
    sizeof(XvwWorkspaceWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    SetValuesHook,			/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    XtInheritQueryGeometry,		/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwWorkspaceWidgetConstraintsRec),  /* constraint_size */
    ConstraintInitialize,		    /* initialize	  */
    NULL,		   		    /* destroy		  */
    NULL,				    /* set_values	  */
    NULL,				    /* extension	  */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    XtInheritChangeSel,			    /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  },  /* XvwManagerWidgetClass fields initialization */
  {
    NULL,  		                    /* extension          */
  },  /* XvwViewportWidgetClass fields initialization */
  {
    NULL,  		                    /* extension          */
  },  /* XvwCanvasWidgetClass fields initialization */
  {
    NULL,  		                    /* save workspace     */
    NULL,  		                    /* restore workspace  */
    NULL,  		                    /* extension          */
  },  /* XvwWorkspaceWidgetClass fields initialization */
};
#undef superclass

  /* for public consumption */
WidgetClass xvwWorkspaceWidgetClass = (WidgetClass) &xvwWorkspaceWidgetClassRec;


/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for Class and Constraint Declarations
|
--------------------------------------------------------------------*/

#define kwidget(widget)	    (XvwWorkspaceWidget) (widget)


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an
|                instance of a XvwWorkspaceWidget class has been created.
|                This is where the attributes for the class are
|		 initialized. 
|
|    Written By: Mark Young
|          Date: Oct 19, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
	xvlang_cache_pixmaps();
	xvw_init_attributes(xvwWorkspaceWidgetClass, attributes,
		XtNumber(attributes), NULL, 0, 
		"$DESIGN/objects/library/xvlang/uis/Workspace.pane");
	xvw_load_resources("$DESIGN/objects/library/xvlang/app-defaults/Workspace");

	xvw_define_attributes(xvwWorkspaceWidgetClass,
		XVW_CANVAS_DUPLICATE, XtRInt, WorkspaceActions, NULL,
		XVW_CANVAS_UNDO,      XtRInt, WorkspaceActions, NULL,
		XVW_CANVAS_DELETE,    XtRInt, WorkspaceActions, NULL,
		XVW_CANVAS_COPY,      XtRInt, WorkspaceActions, NULL,
		XVW_CANVAS_PASTE,     XtRInt, WorkspaceActions, NULL,
		XVW_CANVAS_CUT,       XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_RUN,    XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_STOP,   XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_STEP,   XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_RESET,  XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_REDRAW, XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_CLEAR,  XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_CHECK,  XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_INFO,   XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_SAVE,   XtRString, WorkspaceActions, NULL,
		XVW_WORKSPACE_SAVEALL,XtRString, WorkspaceActions, NULL,
		XVW_WORKSPACE_RESTORE,XtRString, WorkspaceActions, NULL,
		XVW_WORKSPACE_NAME,   XtRString, WorkspaceActions,WorkspaceInfo,
		XVW_WORKSPACE_CREATE_PROCEDURE,  XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_CREATE_WHILELOOP,  XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_CREATE_FORLOOP,    XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_CREATE_COUNTLOOP,  XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_EXPORT_SELECTIONS, XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_SHOW_WKSPGUI,  XtRInt, WorkspaceActions, NULL,
		XVW_WORKSPACE_SAVE_WKSPGUI, XtRString, WorkspaceActions,NULL,
		XVW_WORKSPACE_SHOW_CLIPBOARD, XtRInt, WorkspaceActions,NULL,
		NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method is called the first time an
|                instance of a XvwWorkspaceWidget class has been created.
|                This is where the attributes for the class are
|                initialized.
|
|    Written By: Mark Young
|          Date: Oct 19, 1993
|
------------------------------------------------------------*/

#define FORMFILE "$DESIGN/objects/library/xvlang/uis/wkspgui.subform"

/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwWorkspaceWidget xwid = kwidget(new1);
	xvobject object = xvw_object(new1);

	kguide  *guide;

	/*
	 *  Initialize the indicator that the workspace is running to FALSE
	 */ 
	xwid->workspace.saved   = NULL;

	/*
	 *  Create our wkspgui and substitute the run button to call
	 *  WorkspaceRun() in order to run a workspace.
	 */
	if ((xwid->workspace.wkspgui = kvf_create_form(FORMFILE, KNONE,
			WorkspaceGUIFormCallback, NULL)) != NULL)
	{
	   xwid->workspace.wkspgui->editable = XVF_FULL_EDIT;
	   guide = kvf_search_sel_guide(xwid->workspace.wkspgui->subform);
	   if (guide != NULL)
	   {
              xwid->workspace.selections = kvf_get_selection_kformstructs(
			guide->pane, KNONE, &xwid->workspace.num_selections);
	   }
	}
	xwid->workspace.created_form = TRUE;
	xwid->workspace.wkspgui_initialized = FALSE;

	xwid->viewport.override = TRUE;
	if (kstrcmp(xvw_name(object), "Clipboard") == 0)
	   WorkspaceClipboardCreate(new1);
	else if (xvw_check_subclass(xvw_parent(object), WorkspaceWidgetClass))
	   WorkspaceProcedureCreate(new1);
	else xwid->workspace.titlebar = NULL;

	xwid->viewport.override = FALSE;

	xwid->workspace.expression_id = (long) object;
}

/*-----------------------------------------------------------
|
|  Routine Name: ConstraintInitialize
|
|       Purpose: When a new child of the canvas is created,
|		 tell the child who its parent is. 
|                
|         Input: request  - widget child with requested constraints
|                new1      - widget child with actual constraints
|		 args     - not used
|		 num_args - not used
|    Written By: Mark Young
|          Date: Nov 26, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ConstraintInitialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num)
{
	XvwWorkspaceWidget xwid = kwidget(XtParent(request));
	xvobject object    = xvw_object(new1);
	xvobject workspace = xvw_object((Widget) xwid);


	if (xvw_check_subclass(object, GlyphWidgetClass) == TRUE)
	{
	   xvw_set_attributes(object,
		XVW_GLYPH_EXPRESSION_ID,     workspace,
		XVW_GLYPH_CONNECTION_PARENT, workspace,
		XVW_GLYPH_WKSPGUI, xwid->workspace.wkspgui,
		XVW_GLYPH_ECHO, xwid->workspace.glyph_echo,
		XVW_GLYPH_REPORTING, xwid->workspace.glyph_reporting,
		NULL);
	}

	if (xvw_check_subclass(object, NodeWidgetClass) == TRUE)
	{
	   xvw_insert_callback(object, XVW_NODE_RUN_CALLBACK, FALSE,
                        WorkspaceGlyphRunCallback, workspace);
	   xvw_insert_callback(object, XVW_NODE_MODIFIED_CALLBACK, FALSE,
                        WorkspaceGlyphModifiedCallback, workspace);
	   xvw_set_attributes(object,
		XVW_NODE_PLACEMENT, xwid->workspace.icon_placement,
		XVW_NODE_LEAVE,     xwid->workspace.icon_leave,
		XVW_NODE_SIZE,      xwid->workspace.icon_size,
		XVW_NODE_REMOTE_ENABLE,  xwid->workspace.icon_remote_enable,
		XVW_NODE_REMOTE_LOCATION, xwid->workspace.icon_remote_location,
		NULL);
	}

	if (xvw_check_subclass(object, ConnectionGadgetClass) == TRUE)
	{
	   xvw_set_attributes(object,
		XVW_CONNECTION_TYPE, xwid->workspace.connection_type,
		NULL);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the public values
|                of a XvwWorkspaceWidget instance.  The public values
|                which can be changed are all related to the display
|		 of the workspace.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new1     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Aug 19, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num)
{
	XvwWorkspaceWidget cxwid = kwidget(current);
	XvwWorkspaceWidget nxwid = kwidget(new1);


	if (cxwid->workspace.filename != nxwid->workspace.filename)
	{
	   kfree_and_NULL(cxwid->workspace.filename);
	   nxwid->workspace.filename = kstrdup(nxwid->workspace.filename);
	}

	if (cxwid->workspace.wkspgui != nxwid->workspace.wkspgui)
	   nxwid->workspace.created_form = FALSE;

	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValuesHook
|
|       Purpose: This method is called to set the public values
|                of the childred attributes.
|         Input: widget   - the object containing the subpart
|                           to set the values on
|                args     - the list of args to set
|                num_args - the number of args being set
|    Written By: Mark Young
|          Date: Dec 21, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValuesHook(
   Widget   widget,
   ArgList  args,
   Cardinal *num_args)
{
	int i; 
        kfile *file;
	char  *date;
	Cardinal num;
	Arg temp[MAX_ARGS];
	XvwWorkspaceWidget xwid = kwidget(widget);
	xvobject object         = xwid->viewport.plane;
	XvwManagerWidget xman   = (XvwManagerWidget) xvw_widget(object);


	for (i = num = 0; i < *num_args; i++)
	{
	   if (kstrcmp(args[i].name, XVW_NODE_PLACEMENT) == 0     ||
	       kstrcmp(args[i].name, XVW_NODE_LEAVE) == 0         ||
	       kstrcmp(args[i].name, XVW_NODE_SIZE) == 0         ||
	       kstrcmp(args[i].name, XVW_NODE_REMOTE_ENABLE) == 0 ||
	       kstrcmp(args[i].name, XVW_NODE_REMOTE_LOCATION) == 0 ||
	       kstrcmp(args[i].name, XVW_GLYPH_ECHO) == 0 ||
	       kstrcmp(args[i].name, XVW_GLYPH_REPORTING) == 0 ||
	       kstrcmp(args[i].name, XVW_CONNECTION_TYPE) == 0)
	   {
	      temp[num++] = args[i];
	   }
	}

	if (num > 0)
	{
	   for (i = 0; i < xman->composite.num_children; i++)
	      XtSetValues(xman->composite.children[i], temp, num);
	}


	if (xwid->workspace.glyph_echo == KGLYPH_ECHO_TO_LOG ||
	    xwid->workspace.glyph_echo == KGLYPH_ECHO_TO_CONSOLE_AND_LOG)
	{
            file = xvl_get_logfile();
            if (file != NULL)
            {
                date = kcms_get_date();
	        kfprintf(file, 
		         "\n#======= Cantata Log Session Start, %s ==========\n",
		         date);
	        kfclose(file);
	    }
	}

	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: destroy the workspace...
|         Input: widget - the workspace widget being destroyed 
|    Written By: Mark Young
|          Date: Nov 8, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	XvwWorkspaceWidget xwid = kwidget(widget);
	klist *entry;

	entry = klist_locate(procedurelist, widget);
        if (entry != NULL)
            procedurelist = klist_delete(procedurelist, widget);

	if (xwid->workspace.wkspgui != NULL && xwid->workspace.created_form)
	   xvf_destroy_form(xwid->workspace.wkspgui);

	kfree_and_NULL(xwid->workspace.filename);
	kfree_and_NULL(xwid->workspace.selections);
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceGUIFormCallback - form callback routine
|
|
|       Purpose: This routine is used to process the user's form
|                changes to the workspace GUI.   
|
|         Input: object - object to be used to build the form
|    Written By: Mark Young & Danielle Argiro
|          Date: Nov 26, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void WorkspaceGUIFormCallback(
   kform    *form,
   ksubform *subform,
   kaddr    client_data)
{
	int	   i, num;
        kform_list *entry;
        kselection *selection, **extensions;


	/*
	 *  Make sure closing the form or subform doesn't destroy our form...
	 */
	if (form->quit)
	{
	   form->quit = subform->quit = 0;
	}

	/*
	 *  at this time, workspace GUI can only have a single pane
	 */
        if (!(kvf_check_for_single_pane(form, "WorkspaceGUIFormCallback")))
	    return;

	/*
	 *  go through all selections on workspace GUI, firing the callbacks
	 *  associated with the glyph form extensions of the selections
         *  on the workspace GUI.
	 */
	selection = form->subform->guide->pane->sel_list;
	while (selection != NULL)
	{
	    if (selection->extensions == NULL || selection->modified == FALSE)
	    {
	       selection = selection->next;
	       continue;
	    }
	    selection->modified = FALSE;

	    num = selection->extension_num;
	    extensions = selection->extensions;
	    for (i = 0; i < num; i++)
	    {
               entry = kvf_get_entry((kaddr) extensions[i]->back_form);
	       if (entry != NULL)
               {
                  if (entry->routine != NULL)
                     (void) entry->routine(extensions[i]->back_form, 
					   extensions[i]->back_subform,
                                           entry->client_data);
               }
            }
        }
}


/************************************************************
*
*  Routine Name: xvw_create_workspace - create a workspace object
*
*       Purpose: The workspace visual programming object provides a canvas 
*		 in which other visual programming objects may be placed 
*		 and combined into a visual program.  The workspace object 
*		 supports the saving, restoring, execution, and management 
*		 of visual programs.  It also serves as a visual programming
*		 editor. 
*
*		 The workspace object does not directly support visual
*		 programming, but rather provides a canvas in which other
*		 visual programming components can be combined in which to
*		 create a visual program.  Its role is similar to that
*		 of the manager object; however, instead of allowing the
*		 creation and management of GUI objects, the workspace object
*		 allows the creation and management of visual programming
*		 objects.
*
*                The workspace object allows (1) specification of an internal
*                routine to be used for saving a network, and (2) specification
*                of an internal routine to be used for restoring a network.
*
*                The workspace object can be used to create new visual
*                programming models other than the one used in \fHcantata\fP;
*                all that is necessary is to provide different routines for
*                saving and restoring of networks.
*  
*                Moreover, the workspace object has the capability to
*                manipulate any other visual programming object that is
*                subclassed from the node object, such as glyphs, conditionals,
*                loops, and procedures.  However, if the visual programming
*                toolkit is extended to include other visual programming
*                objects that are subclassed from the node object, they will
*                automatically be usable in a workspace object.
*  
*                As a visual programming editor, the workspace object comes
*                with a full suite of capabilities including running
*                a network, stopping a network, single stepping through
*                a network, resetting the network, redrawing the network,
*                clearing the network, checking the network, and providing
*                information about the network.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - the name with which to reference the object
*        Output: None
*       Returns: The workspace object on success, NULL on failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Jan 14, 1993 16:47       
*      Verified:
*  Side Effects:
* Modifications:
*
*******************************************************************/

xvobject xvw_create_workspace(
   xvobject parent,
   char     *name)
{
        xvobject workspace;
        XvwWorkspaceWidget xwid;

        workspace = xvw_create(parent, FALSE, TRUE, name, WorkspaceWidgetClass);
        xvw_set_attribute(workspace, XVW_SELECTABLE, TRUE);  

        xwid = (XvwWorkspaceWidget) xvw_widget(workspace); 
        return(workspace);
}
