 /*
  * Khoros: $Id: utilities.c,v 1.28.2.2 1997/05/24 19:04:49 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.28.2.2 1997/05/24 19:04:49 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		General Utilities
   >>>>		
   >>>>  Private:
   >>>>                 xvl_locate_port()
   >>>>                 xvl_copy_pane()
   >>>>                 xvl_copy_selection()
   >>>>                 xvl_create_selection()
   >>>>                 xvl_excluded_selection()
   >>>>                 xvl_delete_selections()
   >>>>                 xvl_export_selections()
   >>>>                 xvl_report()
   >>>>                 xvl_trace()
   >>>>                 xvl_set_trace_flags()
   >>>>                 xvl_get_logfile()
   >>>>   Static:
   >>>>                 set_unique_variable()
   >>>>                 link_extensions()
   >>>>                 select_group_members()
   >>>>                 create_external_connection()
   >>>>                 create_external_connection()
   >>>>                 delete_selection()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include <xvlang/GlyphP.h>
#include <xvlang/ExtPortP.h>


static void delete_selection PROTO((xvobject *, size_t, kform_struct *));

static int xvlangTrace = 0;

/*-----------------------------------------------------------
|
|  Routine Name: select_group_members
|
|       Purpose: Used by xvl_export_selections() and
|		 xvl_delete_selections() to select (put 
|		 brackets around) the group members that are
|		 about to be exported or deleted with the one
|		 that was chosen by the user.
|
|         Input: group_start - ultimate group start
|                select_members - TRUE to select group members, FALSE otherwise
|        Output:
|       Returns: TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: May 10, 1995
|
------------------------------------------------------------*/
static void select_group_members(
    kselection *group_start,
    int        select_members)
{
	kselection *selection;
	selection = group_start;
	while (selection != NULL)
	{
	    if (selection->group_next != NULL)
		select_group_members(selection->group_next, select_members);
	    else if (select_members)
		xvw_set_attribute(selection->back_control->back, 
				  XVW_SELECT_ADD, selection->back);
	    else xvw_set_attribute(selection->back_control->back, 
				   XVW_SELECT_DELETE, selection->back);
	    selection = selection->next;
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: link_extensions
|
|       Purpose: Links an exported selection on the pane of a glyph to
|		 its counterpart on the workspace GUI using the extension 
|		 field of the kselection structure.  This causes their values
|		 to be linked together, so that when the user changes
|		 the value of the variable on the pane of the glyph, the
|		 selection on the workspace GUI has its value updated,
|		 and vice versa. 
|
|         Input: export_sel    - selection exported from the pane of the glyph
|		 new_selection - newly created selection on the workspace GUI
|        Output:
|       Returns: TRUE on success, FALSE on failure
|    Written By: Mark Young & Danielle Argiro
|          Date: May 03, 1995
|
------------------------------------------------------------*/

static void link_extensions(
    kselection *export_sel,
    kselection *new_selection)
{
	kselection *export_member;
	kselection *new_member;

	/*
	 *  Create the extension connection between the selection on the
	 *  glyph's pane and the selection on the workspace GUI.
	 */
	kvf_link_extension(export_sel->back_kformstruct, 
		  	   new_selection->back_kformstruct);

	/*
	 *  Selection is part of a group; need to create 
	 *  extensions for the respective group members.
	 */
 	if (export_sel->type == KUIS_MUTEXCL ||
            export_sel->type == KUIS_MUTINCL ||
            export_sel->type == KUIS_GROUP)
	{
	    export_member = export_sel->group_next;
	    new_member = new_selection->group_next;
	    while (new_member != NULL)
	    {
		link_extensions(export_member, new_member);
		new_member    = new_member->next;
		export_member = export_member->next;
	    }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: set_unique_variable
|
|       Purpose: Goes through all selections in the selection list
|		 of the workspace GUI and makes sure that the newly
|		 created selection's variable name doesn't collide
|		 with the variable name of an existing selection.
|                If its variable, for example, is "i1", and there is
|		 already an "i1" on the workspace GUI, then we change
|		 the variable name to "i1_2".  However, if there is already 
|		 an "i1_2", then we must name it "i1_3".
|
|         Input: new_selection - newly duplicated selection for
|			         which to ensure that variable name is unique
|        Output: 
|       Returns: TRUE on success, FALSE on failure
|    Written By: Mark Young & Danielle Argiro
|          Date: May 03, 1995
|
------------------------------------------------------------*/

static int set_unique_variable(
    kselection *new_selection)
{
	int        duplicate;
	kselection *selptr;
	char       *variable;
	char       temp[KLENGTH];

	/*
	 * variable name provided: just use it.
	 */
	selptr = new_selection->back_control->sel_list;

        duplicate = 0;
        while (selptr != NULL)
        {
            if (selptr != new_selection)
            {
                if (selptr->var_token == new_selection->var_token)
                       duplicate++;
                else
                {
                   ksprintf(temp, "%s_",
                            ktoken_to_string(new_selection->var_token));
                   if (kstrncmp(ktoken_to_string(selptr->var_token), temp,
                                                 kstrlen(temp)) == 0)
                       duplicate++;
                }
            }
            selptr = selptr->next;
        }

        if (duplicate > 0) 
        {
	    if (!(kvf_get_attribute(new_selection->back_kformstruct,
                                     KVF_VARIABLE, &variable)))
		return(FALSE);

            ksprintf(temp, "%s_%d", variable, duplicate+1);
	    kfree_and_NULL(variable);
            if (!(kvf_set_attribute(new_selection->back_kformstruct,
                                    KVF_VARIABLE, temp)))
		return(FALSE);

        }
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: create_external_connection - creates external connection
|
|       Purpose: This routine creates the external connection within
|                procedures to represent connections to the outside world.
|       
|         Input: workspace - the workspace in which the connection should be
|                            made
|                glyph     - the glyph in which connection is being made for
|                kformstruct - the form structure the connection is being
|                              made for.
|    Written By: Mark Young
|          Date: Mar 07, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void create_external_connection(
  xvobject workspace,
  xvobject glyph,
  kform_struct *old_sel,
  kform_struct *new_sel)
{
	int      x, y, xpos, ypos;
	xvobject extport, iport, oport, procedure;
	char	 *varname, name[KLENGTH];

	/*
	 * create external port in workspace
	 */
	if ((extport = xvw_create_extport(workspace, "extport")) == NULL)
	   return;

	kvf_get_attribute(new_sel, KVF_VARIABLE, &varname);
	xvw_translate_coords(workspace, 0, 0, &x, &y);

	/*
	 * figure out location and name for the external port
	 */
	if (new_sel->flag == KUIS_INPUTFILE || new_sel->flag == KUIS_STDIN)
	{
	   iport = xvl_locate_port(glyph, KPORT_DATAFLOW, NULL, old_sel);
	   oport = xvl_locate_port(extport, KPORT_DATAFLOW, NULL, NULL);
	   xvw_translate_coords(iport, -x, -y, &xpos, &ypos); xpos -= 100;
	   ksprintf(name, "External Input (%s)", varname);
	}
	else
	{
	   iport = xvl_locate_port(extport, KPORT_DATAFLOW, NULL, NULL);
	   oport = xvl_locate_port(glyph, KPORT_DATAFLOW, NULL, old_sel);
	   xvw_translate_coords(oport, -x, -y, &xpos, &ypos); xpos += 100;
	   ksprintf(name, "External Output (%s)", varname);
	}

	/*
	 * set location & name of external port; associate external port
	 * with the GUI selection on workspace GUI
	 */
	xvw_set_attributes(extport,
		XVW_XPOSITION, xpos,
		XVW_YPOSITION, ypos,
		XVW_NODE_NAME, name,
		XVW_EXTPORT_SELECTION, new_sel,
		NULL);

	/*
	 *  create I/O ports on procedure glyph corresponding to
	 *  the external connections that we just created.
	 */
	procedure = ProcedureLocate(xvw_widget(workspace));
	if (procedure != NULL)
	    GlyphUpdateForm(xvw_widget(procedure));

	if (iport && oport)
	   PortCreateConnection(iport, oport);

	return;
}


/*-----------------------------------------------------------
|
|  Routine Name: xvl_locate_port - locate the port
|
|       Purpose: This routine locates the desired port in order restore the
|		 connection.
|
|         Input: node - the node in which to locate the port
|		 varname - the variable name to be located
|    Written By: Mark Young
|          Date: Aug 12, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
xvobject xvl_locate_port(
   xvobject     node,
   int		type,
   char         *varname,
   kform_struct *selection)
{
        XvwGlyphWidget   xglyph;
        XvwExtPortWidget xextport;
 
        int  i, status;
        char *var = NULL;
 
	if (xvw_check_subclass(node, GlyphWidgetClass) == TRUE)
	{
	   xglyph = (XvwGlyphWidget) xvw_widget(node);

	   if (type == KPORT_CONTROLFLOW)
	   {
	      if (kstrcmp(varname, "in") == 0)
		 return(xglyph->node.cflow_in);
	      else if (kstrcmp(varname, "out") == 0)
		 return(xglyph->node.cflow_out);
	   }

           /* Scan for the desired port along the input ports...  */
           for (i = 0; i < xglyph->node.isize; i++)
           {
	       if (xglyph->glyph.isels[i] == selection)
		  return(xglyph->node.iports[i]);

               xvf_get_attribute(xglyph->glyph.isels[i], XVF_VARIABLE, &var);
               status = kstrcmp(var, varname); kfree(var);
	       if (status == 0)
                  return(xglyph->node.iports[i]);
           }
 
           /* Scan for the desired port along the output ports...  */
           for (i = 0; i < xglyph->node.osize; i++)
           {
	       if (xglyph->glyph.osels[i] == selection)
		  return(xglyph->node.oports[i]);

               xvf_get_attribute(xglyph->glyph.osels[i], XVF_VARIABLE, &var);
               status = kstrcmp(var, varname); kfree(var);
	       if (status == 0)
                  return(xglyph->node.oports[i]);
           }
	}
	else if (xvw_check_subclass(node, ExtPortWidgetClass) == TRUE)
	{
	   xextport = (XvwExtPortWidget) xvw_widget(node);

	   return(xextport->extport.port);
	}
        return(NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_copy_pane
|
|       Purpose: This routine exports the selections on a glyph to the
|                workspace GUI. We go through the selections of the pane,
|                skipping over those we don't want to export, and copying
|                the others to the workspace GUI.
|
|         Input: workspace  - the workspace associated with the wkspgui
|                glyph      - the glyph in which the exported selection is
|                             associated with 
|                paneform  - the form tree representing the pane of
|                            the glyph containing selections to export
|    Written By: Mark Young & Danielle Argiro
|          Date: May 03, 1995
|
------------------------------------------------------------*/
/*ARGSUSED*/
void xvl_copy_pane(
   xvobject workspace,
   xvobject glyph)
{
        kselection *selection;
	kform      *paneform = NULL;

	xvw_get_attribute(glyph, XVW_GLYPH_FORM, &paneform);
	if (!paneform)
	    return;
        if (!kvf_check_for_single_pane(paneform, "xvl_copy_pane"))
            return;

        /*
         *  Don't want the Options button (already have one). 
         *  Don't want any of the buttons from the Options button.
         *  Don't want any help, quit, routine, buttons.
         *  Don't want blank selections.
	 *  Copy all other selections from glyph's pane to wksp GUI.
         */
        selection = paneform->subform->guide->pane->sel_list;
        while (selection != NULL)
        {
           if (!xvl_excluded_selection(selection))
               xvl_copy_selection(workspace, glyph, selection, NULL);

           selection = selection->next;
        }

}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_copy_selection
|
|       Purpose: Creates a new GUI selection on the workspace GUI
|                modelled after the selection the glyph that it comes from.
|
|         Input: workspace  - the workspace associated with the wkspgui
|		 glyph      - the glyph in which the exported selection is
|			      associated with
|                export_sel - GUI selection to export to workspace GUI
|                varname    - the new variable name of the selection
|			      (if NULL, will make sure that name is unique).
|    Written By: Mark Young & Danielle Argiro
|          Date: May 03, 1995
|
------------------------------------------------------------*/
/*ARGSUSED*/
void xvl_copy_selection(
   xvobject   workspace,
   xvobject   glyph,
   kselection *export_sel,
   char	      *varname)
{
	int        i;
	float      y, height;
	kcontrol   *wksp_pane;
	kselection *new_selection;
	kform	   *wkspgui = NULL;

        /*
         * only supporting workspace GUIs w/ single panes on a single subforms
         */
	xvw_get_attribute(workspace, XVW_WORKSPACE_WKSPGUI, &wkspgui);
	if (!wkspgui)
	   return;
        if (!kvf_check_for_single_pane(wkspgui, "xvl_copy_selection"))
           return;

        /*
         *  Need to make sure that the selection has not already been
         *  exported to form.  We only want the selection to be linked
         *  once for each form, otherwise we may end up with a selection
         *  being exported into the workspace gui multiple times.
         */
        wksp_pane = wkspgui->subform->guide->pane;
        for (i = 0; i < export_sel->extension_num; i++)
        {
           if (export_sel->extensions[i]->back_control == wksp_pane)
              return;
        }
	
	/*
	 *  export the selection by copying it to the wksp GUI form tree
	 */
	new_selection = kvf_copy_selection(wksp_pane, export_sel, NULL, NULL);

	/*
	 * zero out the (x,y) position of the new selection so 
         * kvf_calculate_y_position won't use *it's* y position & height when 
         * calculating the desired y position  (but save height for restoration)
	 */
	if ((new_selection->type != KUIS_MUTEXCL) &&
            (new_selection->type != KUIS_MUTINCL) &&
            (new_selection->type != KUIS_GROUP))
	{
	    kvf_get_attribute(new_selection->back_kformstruct,
			      KVF_HEIGHT, &height);
	    kvf_set_attributes(new_selection->back_kformstruct,
			       KVF_Y,      0.0,
			       KVF_HEIGHT, 0.0,
			       NULL);
	    /*
	     * set the y position of the selection in the workspace GUI
	     */
	    y = kvf_calculate_y_position(wksp_pane);
	    kvf_set_attributes(new_selection->back_kformstruct,
			      KVF_X,      0.0,
			      KVF_Y,      kmax(y, 1.5),
			      KVF_HEIGHT, height,
			      NULL);
	}

        /*
         * (physically) create the new selection on the wksp GUI backplane
         */
        if (wksp_pane->back != NULL)
	{
            if (!(xvf_create_selection(new_selection, wksp_pane->back,
				       wksp_pane->type)))
		return;

	    if ((new_selection->opt_selected != 0) &&
                (new_selection->opt_object != NULL))
                xvw_set_attribute(new_selection->opt_object, 
				   XVW_BUTTON_SELECTED, TRUE);

            if (new_selection->toggle_next != NULL)
                xvf_highlight_toggle(new_selection);

            if (new_selection->group_next != NULL)
                xvf_highlight_group(new_selection->group_next);
	}

	/*
	 *  Make the new selection has a unique variable name
	 *  (except for -C, -B, -K, which don't use variable names)
	 */
	if (new_selection->type != KUIS_MUTEXCL &&
	    new_selection->type != KUIS_MUTINCL &&
	    new_selection->type != KUIS_GROUP)
	{
	    if (varname != NULL)
	    {
	        if (!(kvf_set_attribute(new_selection->back_kformstruct, 
			                KVF_VARIABLE, varname)))
	            kerror("xvlang", "xvl_copy_selection",
                           "Internal error: can't set variable for selection");
	    }
	    else 
	    {
	       if (!(set_unique_variable(new_selection)))
                    kerror("xvlang", "xvl_copy_selection",
                           "Internal error: can't set variable for selection");
	    }
	}

        if (workspace != NULL && glyph != NULL && 
            (export_sel->type == KUIS_INPUTFILE  ||
             export_sel->type == KUIS_OUTPUTFILE ||
             export_sel->type == KUIS_STDIN      ||
             export_sel->type == KUIS_STDOUT))
	{
            create_external_connection(workspace, glyph,  
            		export_sel->back_kformstruct,
			new_selection->back_kformstruct);
	}
	else
	{
	    /* Link extension for single selection, or for group */
	    link_extensions(export_sel, new_selection); 
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_create_selection
|
|       Purpose: Creates a new GUI input or output selection on the
|		 workspace GUI
|
|         Input: form    - pointer to the workspace GUI form tree
|                type    - whether to create an input or output
|                varname    - the new variable name of the selection
|			      (if NULL, will make sure that name is unique).
|    Written By: Mark Young & Danielle Argiro
|          Date: May 03, 1995
|
------------------------------------------------------------*/
/*ARGSUSED*/
kform_struct *xvl_create_selection(
   kform  *form,
   int	  type,
    char  *varname)
{
        kcontrol   *pane;
	float      y, height;
	kselection *selection;
	char	   line[KLENGTH];

        /*
         * only supporting workspace GUIs w/ single panes on a single subforms
         */
        if (!(kvf_check_for_single_pane(form, "xvl_create_selection")))
            return(NULL);
	else
	   pane = form->subform->guide->pane;

	/*
	 *  export the selection by copying it to the wksp GUI form tree
	 */
	if (type == KUIS_INPUTFILE)
	   kstrcpy(line, "-I 1 0 0 1 0 1 -1x1+0+0 ' ' 'InputFile ' 'ifile' i1");
	else
	   kstrcpy(line, "-O 1 0 0 1 0 1 -1x1+0+0 ' ' 'OutputFile' 'ofile' o1");

	selection = kvf_create_new_selection(pane, line);
	if (selection == NULL)
	   return(NULL);

	/*
	 * zero out the (x,y) position of the new selection so 
         * kvf_calculate_y_position won't use *it's* y position & height when 
         * calculating the desired y position  (but save height for restoration)
	 */
	kvf_get_attribute(selection->back_kformstruct, KVF_HEIGHT, &height);
	kvf_set_attributes(selection->back_kformstruct,
			       KVF_Y,      0.0,
			       KVF_HEIGHT, 0.0,
			       NULL);
	/*
	 * set the y position of the selection in the workspace GUI
	 */
	y = kvf_calculate_y_position(pane);
	kvf_set_attributes(selection->back_kformstruct,
			      KVF_X,      0.0,
			      KVF_Y,      kmax(y, 1.5),
			      KVF_HEIGHT, height,
			      NULL);

        /*
         * (physically) create the new selection on the wksp GUI backplane
         */
        if (pane->back != NULL)
	{
            if (!(xvf_create_selection(selection, pane->back, pane->type)))
		return(NULL);

	    if ((selection->opt_selected != 0) &&
                (selection->opt_object != NULL))
                xvw_set_attribute(selection->opt_object, 
				  XVW_BUTTON_SELECTED, TRUE);
	}

	/*
	 *  Make the new selection has a unique variable name
	 */
	if (varname != NULL)
	{
	    if (!(kvf_set_attribute(selection->back_kformstruct, 
			                KVF_VARIABLE, varname)))
	        kerror("xvlang", "xvl_create_selection",
                           "Internal error: can't set variable for selection");
	}
	else if (!(set_unique_variable(selection)))
	{
            kerror("xvlang", "xvl_create_selection",
                           "Internal error: can't set variable for selection");
	}
	return(selection->back_kformstruct);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_excluded_selection
|
|       Purpose: Certain selections on the workspace GUI are
|		 excluded from operations such as deletion,
|		 comparisions to exported selections on glyph panes,
|		 etc.  This utility routine determines whether the
|		 selection is one of these:
|
|	         1 - the "Options" button
|		 2 - a button on the "Options" pulldown menu
|		 3 - a "Help", "Quit", or "Run" button
|		 4 - a blank selection
|
|         Input: selection - workspace GUI selection to examine
|        Output:
|       Returns: TRUE if the selection is one of the ones listed above,
|		 FALSE otherwise.
|    Written By: Mark Young & Danielle Argiro
|          Date: May 03, 1995
|
------------------------------------------------------------*/

int xvl_excluded_selection(
    kselection *selection)
{
	ktoken opt_token = kstring_to_token("_gui_options");

	/*
         *  found the options button
         */
        if ((selection->type == KUIS_STARTSUBMENU) &&
             (selection->var_token == opt_token))
            return(TRUE);

	/*
         * found button in the options pulldown menu
         */
        else if ((selection->back_submenu != NULL) &&
                 (selection->back_submenu->var_token == opt_token))
            return(TRUE);

	/*
         * found help, quit, routine, blanks
         */
        else if ((selection->type == KUIS_HELP) ||
                 (selection->type == KUIS_QUIT) ||
                 (selection->type == KUIS_BLANK) ||
                 (selection->type == KUIS_ROUTINE))
	    return(TRUE);

	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_delete_selections
|
|       Purpose: This utility routine deletes the selected
|		 selections from the workspace GUI. It is called
|		 from WorkspaceGUIActions() when the user selects 
|		 the selections they don't want from the workspace
|		 GUI and then uses the workspace GUI "Options" pulldown
|		 menu to "Delete Selections".
|
|		 First, the backplanes of the selections are obtained
|		 from the workspace GUI.  Then, the selections are
|		 deleted.  Special treatment is necessary for selections
|		 that are part of a group or a nested group.
|
|         Input: selection - workspace GUI selection to examine
|        Output:
|       Returns: 
|    Written By: Mark Young & Danielle Argiro
|          Date: May 03, 1995
|
------------------------------------------------------------*/
void xvl_delete_selections(
    xvobject workspace)
{
        kcontrol     *pane;
        char         *title;
	xvobject     *selbacks, *children;
        kselection   *back_group;
        kform_struct *kformstruct;
	kform	     *wkspgui = NULL;
        int          i, status, num_sels = 0; 
	size_t       child_num;
        xvobject     procedure;

        /*
         * only supporting workspace GUIs w/ single panes on a single subforms
         */
	xvw_get_attribute(workspace, XVW_WORKSPACE_WKSPGUI, &wkspgui);
	if (wkspgui == NULL)
	   return;
        if (!kvf_check_for_single_pane(wkspgui, "xvl_delete_selections"))
           return;

        /*
         * get selected selections off workspace GUI
         */
	pane = wkspgui->subform->guide->pane;
        xvw_get_attributes(pane->back,
                XVW_SELECTIONS,     &selbacks,
                XVW_NUM_SELECTIONS, &num_sels,
                NULL);

        children = WorkspaceChildren(workspace, FALSE, &child_num);

	/*
	 * have to delete each selected selection from wksp GUI
	 */
        for (i = 0; i < num_sels; i++)
        {
            /* get kformstruct representing selection indicated by user */
            kformstruct = xvf_get_kformstruct(wkspgui, selbacks[i]);
            if (kformstruct != NULL)
            {
		/*
                 * if they have indicated a selection that is part of a group,
                 * have to delete the whole group. Take into account that
                 * groups may be nested.
                 */
                if (kformstruct->Selptr->back_group != NULL)
                {
                    back_group = kformstruct->Selptr->back_group;
                    if (back_group->back_group != NULL)
                        back_group = back_group->back_group;

		    select_group_members(back_group, TRUE);
                    kvf_get_attribute(kformstruct, KVF_TITLE, &title);
                    status = kprompt(KSTANDARD, NULL, NULL, 0, "Selection '%s' is part of a group; deleting this selection will also cause all other group members to be deleted.  Continue with deletion of selection '%s' and group members?",title, title);
                    if (status == TRUE)
		       delete_selection(children, child_num, 
					back_group->back_kformstruct);
		    else select_group_members(back_group, FALSE);
                }
                /* selection not part of group; delete it directly */
                else delete_selection(children, child_num, kformstruct);
            }
	}

	procedure = ProcedureLocate(xvw_widget(workspace));
	if (procedure != NULL)
	    GlyphUpdateForm(xvw_widget(procedure));
}

static void delete_selection(
    xvobject     *children,
    size_t        child_num,
    kform_struct *kformstruct)
{
	int j;
	kform_struct *extport_sel;

	for (j = 0; j < child_num; j++)
        {
	    if (xvw_check_subclass(children[j], ExtPortWidgetClass) == TRUE)
	    {
		xvw_get_attribute(children[j], XVW_EXTPORT_SELECTION, 
			          &extport_sel);
	        if (extport_sel == kformstruct)
		{
		    xvw_set_attribute(children[j], XVW_EXTPORT_SELECTION, NULL);
		     xvw_destroy(children[j]);
		}
	    }
	}
	xvf_set_attribute(kformstruct, XVF_DELETE, TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_export_selections
|
|       Purpose: This utility routine exports the selected
|                selections from the glyph's pane to the workspace GUI. 
|		 It is called from OptionsActions() when the user selects
|                the selections they want to export to the workspace
|                GUI and then uses the glyph's "Options" pulldown
|                menu to "Export Selections".
|
|                First, the backplanes of the selections are obtained
|                from the glyph's pane.  Then, the selections are
|                exported.  Special treatment is necessary for selections
|                that are part of a group or a nested group.
|
|         Input: workspace  - the workspace associated with the wkspgui
|                glyph      - the glyph in which the exported selection is
|                             associated with
|    Written By: Mark Young & Danielle Argiro
|          Date: May 03, 1995
|
------------------------------------------------------------*/
void xvl_export_selections(
    xvobject workspace,
    xvobject glyph)
{
        kcontrol     *pane, *wksp_pane;
        char         *title;
        xvobject     *selbacks;
        kselection   *back_group;
        kform_struct *kformstruct;
	kform        *paneform = NULL, *wkspgui = NULL;
        int          i, j, status, already_there, num_sels = 0;

        /*
         * only supporting workspace GUIs w/ single panes on a single subforms
         */
	xvw_get_attribute(glyph, XVW_GLYPH_FORM, &paneform);
	xvw_get_attribute(workspace, XVW_WORKSPACE_WKSPGUI, &wkspgui);
	if (!wkspgui || !paneform)
	   return;
        if (!kvf_check_for_single_pane(wkspgui, "xvl_copy_selection"))
           return;

        /*
         * get selected selections off glyph's pane
         */
	pane = paneform->subform->guide->pane;
	xvw_get_attributes(pane->back,
                XVW_SELECTIONS,     &selbacks,
                XVW_NUM_SELECTIONS, &num_sels,
                NULL);
	/*
	 *  user error: nothing selected
	 */
	if (num_sels == 0)
	{
	    kerror("xvlang", "xvl_export_selections",
		   "No selections on glyph's GUI selected for exporting");
	    return;
	}

        /*
         * have to export each selected selection to wksp GUI
         */
	for (i = 0; i < num_sels; i++)
	{
	     already_there = FALSE;
	     kformstruct = xvf_get_kformstruct(paneform, selbacks[i]);
             if (kformstruct != NULL)
             {
		/*
                 *  Make sure that selection has not already been exported
         	 */
        	 wksp_pane = wkspgui->subform->guide->pane;
        	 for (j = 0; j < kformstruct->Selptr->extension_num; j++)
        	 {
           	    if (kformstruct->Selptr->extensions[j]->back_control ==
		        wksp_pane) already_there = TRUE;
        	 }

                /*
                 * if they have indicated a selection that is part of a group,
                 * have to delete the whole group. Take into account that
                 * groups may be nested.
                 */
                if ((kformstruct->Selptr->back_group != NULL) && 
		    (!already_there))
                {
                    back_group = kformstruct->Selptr->back_group;
                    if (back_group->back_group != NULL)
                        back_group = back_group->back_group;

		    select_group_members(back_group, TRUE);
                    kvf_get_attribute(kformstruct, KVF_TITLE, &title);
                    status = kprompt(KSTANDARD, NULL, NULL, 0, "Selection '%s' is part of a group; exporting this selection will also cause all other group members to be exported.  Continue with exporting of selection '%s' and group members?", title, title);
                    if (status == TRUE)
		        xvl_copy_selection(workspace, glyph, back_group, NULL);
		    else select_group_members(back_group, FALSE);
                }
                /* selection not part of group; delete it directly */
                else if (!already_there)
		   xvl_copy_selection(workspace,glyph,kformstruct->Selptr,NULL);
             }
        }
        xvw_set_attribute(pane->back, XVW_SELECT_DELETE_ALL, TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_enable_reporting
|
|       Purpose: This enables or disable checkpoint reporting.
|
|    Written By: Mark Young
|          Date: May 21, 1996
|
------------------------------------------------------------*/
static int enable_reporting = FALSE;

void xvl_enable_reporting(
    int enable)
{
	enable_reporting = enable;
}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_report
|
|       Purpose: This utility routine is used to report status or check
|                points.  This is used for debugging, but also by routines
|                that want to monitor checkpoints for testsuite purposes.
|
|    Written By: Mark Young
|          Date: May 21, 1996
|
------------------------------------------------------------*/
void xvl_report(
    int  checkpoint,
    char *format,
    kvalist)
{
	kva_list args;

	if (enable_reporting == FALSE)
	   return;

	kva_start(args, format);
	if (checkpoint)
	{
	   kfprintf(kstderr, "CheckPoint: ");
	   kvfprintf(kstderr, format, args);
	}
	else
	   kvfprintf(kstderr, format, args);

	kva_end(args);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_trace
|
|       Purpose: This utility routine is used to log information
|                to the standard khoros log file.
|
|         Input: widget   the Glyph Widget that controls logging.
|                         if NULL, we will log anyway
|                tflags   prints info if (tflags & xvlangTrace) != 0
|                format   printf-style format string
|
|    Written By: Pete Humphrey
|          Date: Feb 24, 1997
|
------------------------------------------------------------*/
void xvl_trace(
    Widget widget,
    int tflags,
    char *format,
    kvalist)
{
   kva_list args;
   kfile *file;
   int consoleFlag = 0, logFlag = 0;

   if ((tflags & xvlangTrace) == 0) return;
   
   if (widget == NULL) logFlag = 1;
   else if (XtIsSubclass(widget, GlyphWidgetClass))
   {
      XvwGlyphWidget xwid = (XvwGlyphWidget) widget;
      if (xwid->glyph.echo == KGLYPH_ECHO_TO_LOG  ||
	  xwid->glyph.echo == KGLYPH_ECHO_TO_CONSOLE_AND_LOG)
      {
	 logFlag = 1;
      }
      if (xwid->glyph.echo == KGLYPH_ECHO_TO_CONSOLE  ||
	  xwid->glyph.echo == KGLYPH_ECHO_TO_CONSOLE_AND_LOG)
      {
	 consoleFlag = 1;
      }
   }

   logFlag = consoleFlag = 1;

   if (logFlag)
   {
      file = xvl_get_logfile();
      if (file == NULL)
	 return;
	 
      kva_start(args, format);
      kvfprintf(file, format, args);
      kfclose(file);
      kva_end(args);
   }

   if (consoleFlag)
   {
      kva_start(args, format);
      kvprintf(format, args);
      kva_end(args);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_set_trace_flags
|
|       Purpose: This utility sets trace flags used by xvl_trace.
|
|         Input: flags    flag values (a bitmap)
|
|    Written By: Pete Humphrey
|          Date: Feb 24, 1997
|
------------------------------------------------------------*/

void xvl_set_trace_flags(int flags)
{
   xvlangTrace = flags;
}


/*-----------------------------------------------------------
|
|  Routine Name: xvl_set_extport_davs
|
|       Purpose: When a loop or a procedure is finished executing,
|		 we need to set the data availables on the output
|		 nodes of the loop or procedure glyph, according to
|		 whether or not the corresponding external output ports
|		 inside the loop or procedure have their data availables
|		 set.  This routine accomplishes that task.
|
|    Written By: Danielle Argiro
|          Date: July 2, 1996
|
------------------------------------------------------------*/

void xvl_set_extport_davs(
    xvobject workspace,
    xvobject *oports,
    int      oport_num)
{
	xvobject     plane, extport_port; 
	xvobject     *extports;
        int          i, j, dav;
        size_t       extport_num;
        kform_struct *port_sel, *extport_sel;
	
	xvw_get_attribute(workspace, XVW_VP_PLANE_OBJECT, &plane);
        extports = xvw_children(plane, ExtPortWidgetClass, &extport_num);

        for (i = 0; i < oport_num; i++)
	{
            xvw_get_attribute(oports[i], XVW_PORT_SELECTION, &port_sel);
            for (j = 0; j < extport_num; j++)
            {
		xvw_get_attribute(extports[j], XVW_EXTPORT_SELECTION,
                                  &extport_sel);
                if (port_sel->u.selptr == extport_sel->u.selptr)
		{
                    xvw_get_attribute(extports[j], XVW_EXTPORT_PORT_OBJECT, 
				      &extport_port);
		    xvw_get_attribute(extport_port, XVW_PORT_DAV, &dav);
                    xvw_set_attribute(oports[i], XVW_PORT_DAV, dav);
		}
            }
	}
	kfree(extports);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_clear_extport_davs
|
|       Purpose: When a loop or a procedure is finished executing,
|                we need to set the data availables on the output
|                nodes of the loop or procedure glyph, according to
|                whether or not the corresponding external output ports
|                inside the loop or procedure have their data availables
|                set.  This routine accomplishes that task.
|
|    Written By: Danielle Argiro
|          Date: July 2, 1996
|
------------------------------------------------------------*/

void xvl_clear_extport_davs(
    xvobject workspace,
    xvobject *oports,
    int      oport_num)
{
        xvobject     plane, extport_port;
        xvobject     *extports;
        int          i, j;
        size_t       extport_num;
        kform_struct *port_sel, *extport_sel;

        xvw_get_attribute(workspace, XVW_VP_PLANE_OBJECT, &plane);
        extports = xvw_children(plane, ExtPortWidgetClass, &extport_num);

        for (i = 0; i < oport_num; i++)
        {
            xvw_get_attribute(oports[i], XVW_PORT_SELECTION, &port_sel);
            for (j = 0; j < extport_num; j++)
            {
                xvw_get_attribute(extports[j], XVW_EXTPORT_SELECTION,
                                  &extport_sel);
                if (port_sel->u.selptr == extport_sel->u.selptr)
                {
                    xvw_get_attribute(extports[j], XVW_EXTPORT_PORT_OBJECT,
                                      &extport_port);
                    xvw_set_attribute(oports[i], XVW_PORT_DAV, FALSE);
                }
            }
        }
}

/************************************************************
*
*  Routine Name: xvl_get_logfile - opens log file specified by KHOROS_LOG
*
*       Purpose: Utility routine that opens the log file specified
*                by KHOROS_LOG for echoing glyph execution
*
*       Returns: Open stream to log file on success, NULL if KHOROS_LOG
*                is not set or if the file specified cannot be opened.
*
*  Restrictions:
*    Written By: Danielle Argiro
*          Date: Sept 24, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*******************************************************************/



kfile *xvl_get_logfile(void)
{
	static char *log_filename = NULL;
	static int  khoros_log_set = TRUE;
        static char filepath[KLENGTH];
	kfile *file = NULL;

	/* if log_filename was set to "none", that means previous failure */
	if (khoros_log_set == FALSE)
	    return(NULL);

	/* if log_filename is NULL, still need to open the file */
        if (log_filename == NULL)
	{
	    filepath[0] = '\0';
            if ((log_filename = kgetenv("KHOROS_LOG")) == NULL)
	    {
                kerror(XVLANG, "xvl_get_logfile",
                       "The KHOROS_LOG environment variable (used to specify "
		       "the filename of the log file) is not set; therefore, "
                       "glyph execution log cannot be written out. Please "
	               "exit cantata, set KHOROS_LOG, and try again.");
		khoros_log_set = FALSE;
	        return(NULL);
	    }
            else (void) kfullpath(log_filename, NULL, filepath);
	}
	file = kfopen(filepath, "a+");
        if (file == NULL)
        {
	    kerror(XVLANG, "xvl_get_logfile",
                   "Cannot write to file '%s', specified as log file "
                   "by the KHOROS_LOG environment variable; therefore, "
                   "glyph execution log cannot be written out. Please "
	           "exit cantata, reset KHOROS_LOG, and try again.",
                   log_filename);
	    khoros_log_set = FALSE;
        }
	return(file);
}


/*-----------------------------------------------------------
|
|  Routine Name: xvl_propagate_wksp_attributes
|
|       Purpose: This routine obtains the settings of attributes
|                for the canvas grid, glyphs, workspace and command
|                bar from a "superior" workspace (for a new area,
|                this is the default workspace; for a procedure, loop,
|                etc, this is the main workspace) and initializes the
|                same attributes of a newly-created workspace to the
|                same values.
|         Input: None
|        Output: None
|    Written By: Danielle Argiro
|          Date: Sept 26, 1996
| Modifications:
|
------------------------------------------------------------*/

void xvl_propagate_wksp_attributes(
    xvobject ref_workspace,
    xvobject new_workspace)
{
        char *gridcolor, *backgroundcolor, *backpixmaps;
        int   griddisplay, gridsize, connecttype, connectredraw;
        int   glyphplacement, glyphdisplay, glyphreport, remoteexec, echoexec;

        xvw_get_attributes(ref_workspace,
                   XVW_CANVAS_GRID,              &griddisplay,
                   XVW_FOREGROUND_COLOR,         &gridcolor,
                   XVW_CANVAS_GRIDSIZE,          &gridsize,
                   XVW_CONNECTION_TYPE,          &connecttype,
                   XVW_REFRESH_CHILDREN_ON_EDIT, &connectredraw,
                   XVW_NODE_PLACEMENT,           &glyphplacement,
                   XVW_NODE_LEAVE,               &glyphdisplay,
                   XVW_GLYPH_REPORTING,          &glyphreport,
                   XVW_NODE_REMOTE_ENABLE,       &remoteexec,
                   XVW_GLYPH_ECHO,               &echoexec,
                   XVW_BACKGROUND_COLOR,         &backgroundcolor,
                   XVW_BACKGROUND_PIXMAPFILE,    &backpixmaps,
                   NULL);

        xvw_set_attributes(new_workspace,
                   XVW_CANVAS_GRID,              griddisplay,
                   XVW_FOREGROUND_COLOR,         gridcolor,
                   XVW_CANVAS_GRIDSIZE,          gridsize,
                   XVW_CONNECTION_TYPE,          connecttype,
                   XVW_REFRESH_CHILDREN_ON_EDIT, connectredraw,
                   XVW_NODE_PLACEMENT,           glyphplacement,
                   XVW_NODE_LEAVE,               glyphdisplay,
                   XVW_GLYPH_REPORTING,          glyphreport,
                   XVW_NODE_REMOTE_ENABLE,       remoteexec,
                   XVW_GLYPH_ECHO,               echoexec,
                   XVW_BACKGROUND_COLOR,         backgroundcolor,
                   NULL);

        if (backpixmaps != NULL)
             xvw_set_attribute(new_workspace, XVW_BACKGROUND_PIXMAPFILE,
                               backpixmaps);
}


