/*
  * Khoros: $Id: Browser.c,v 1.28.2.2 1997/04/29 22:32:36 mirage Exp $
  */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Browser.c,v 1.28.2.2 1997/04/29 22:32:36 mirage Exp $";
#endif
 
 
/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros Browser GUI Widget
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Destroy()
   >>>>                 SetValues()
   >>>>                 BrowserCallback()
   >>>>                 UpdateBrowserList()
   >>>>                 UpdateDirectoryList()
   >>>>                 ChangeBrowserMode()
   >>>>                 BrowserDown()
   >>>>   Public:
   >>>>                 xvw_create_browser()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvobjects/BrowserP.h>

static void Initialize	   PROTO((Widget, Widget, ArgList, Cardinal *));
static void Destroy	   PROTO((Widget));
static Boolean SetValues   PROTO((Widget, Widget, Widget, ArgList, Cardinal *));
static void ClassInitialize     PROTO((void));
static void BrowserCallback     PROTO((xvobject, kaddr, kaddr));
static void UpdateBrowserList   PROTO((xvobject));
static void ChangeBrowserMode   PROTO((xvobject, int, char *));
static int  UpdateDirectoryList PROTO((xvobject, char *, kaddr));
static void BrowserDown         PROTO((XvwBrowserWidget, xvobject));

static char **alias_array = NULL;
static size_t num_aliases  = 0;


/*-------------------------------------------------------------------*
|
|   Define Default Pixmap Paths
|
--------------------------------------------------------------------*/
 
#define DIR_PIXMAP "$DESIGN/objects/library/xvobjects/pixmaps/dir.xpm"
#define ALIASES_PIXMAP "$DESIGN/objects/library/xvobjects/pixmaps/aliases.xpm"

/*-------------------------------------------------------------------*
|
|   Full class attributes
|
--------------------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_BROWSER_DIRECTORY,       NULL,   XtRString,   NULL},
{XVW_BROWSER_FILTER,          NULL,   XtRString,   NULL},
{XVW_BROWSER_CALLBACK,	      NULL,   XtRCallback, NULL},
{XVW_BROWSER_DESTROY_ON_QUIT, NULL,   XtRInt,      XtRBoolean},

{XVW_BROWSER_DIRECTORY_PIXMAP,	   NULL,   XtRPixmap,   NULL},
{XVW_BROWSER_ALIASES_PIXMAP,	   NULL,   XtRPixmap,   NULL},
{XVW_BROWSER_DIRECTORY_PIXMAPFILE, XVW_BROWSER_DIRECTORY_PIXMAP,
		XtRString, XtRPixmap},
{XVW_BROWSER_ALIASES_PIXMAPFILE, XVW_BROWSER_ALIASES_PIXMAP,
		XtRString, XtRPixmap},
};


/*-------------------------------------------------------------------*
|
|   Full class record constant
|
--------------------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwBrowserWidgetRec, browser.field)

static XtResource resources[] = { 
{XVW_BROWSER_DIRECTORY, NULL, XtRString, sizeof(String),
        offset(directory), XtRImmediate, (XtPointer) NULL},
{XVW_BROWSER_FILTER, NULL, XtRString, sizeof(String),
        offset(filter_string), XtRImmediate, (XtPointer) NULL},
{XVW_BROWSER_CALLBACK, NULL, XtRCallback, sizeof(XtPointer),
      offset(browser_callback), XtRCallback, (XtPointer) NULL},
{XVW_BROWSER_DESTROY_ON_QUIT, NULL, XtRBoolean, sizeof(Boolean),
      offset(destroy_on_quit), XtRImmediate, (XtPointer) TRUE},
{XVW_BROWSER_DIRECTORY_PIXMAP, XVW_BROWSER_DIRECTORY_PIXMAPFILE, XtRPixmap, 
      sizeof(Pixmap), offset(directory_pixmap), XtRString,
      (XtPointer) DIR_PIXMAP},
{XVW_BROWSER_ALIASES_PIXMAP, XVW_BROWSER_ALIASES_PIXMAPFILE, XtRPixmap, 
      sizeof(Pixmap), offset(aliases_pixmap), XtRString,
      (XtPointer) ALIASES_PIXMAP},

{XVW_PREFERRED_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwBrowserWidgetRec, manager.preferred_width),
	XtRImmediate, (XtPointer) 300 },
{XVW_PREFERRED_HEIGHT, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwBrowserWidgetRec, manager.preferred_height),
	XtRImmediate, (XtPointer) 400},
};
#undef offset


/*-------------------------------------------------------------------*
|
|   Class Declaration for Browser Widget
|
--------------------------------------------------------------------*/

#define superclass (&xvwManagerWidgetClassRec)

XvwBrowserWidgetClassRec xvwBrowserWidgetClassRec =
{
  {
    (WidgetClass) superclass,		/* superclass		  */	
    "Browser",			/* class_name		  */
    sizeof(XvwBrowserWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    XtInheritQueryGeometry,		/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,				/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwBrowserWidgetConstraintsRec),  /* constraint_size	  */
    NULL,				    /* initialize	  */
    NULL,				    /* destroy		  */
    NULL,				    /* set_values	  */
    NULL,				    /* extension	  */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    XtInheritChangeSel,			    /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  },  /* XvwManagerWidgetClass fields initialization */
  {
    NULL,  		                    /* extension          */
  },  /* XvwBrowserWidgetClass fields initialization */
};
#undef superclass

  /* for public consumption */
WidgetClass xvwBrowserWidgetClass = (WidgetClass) &xvwBrowserWidgetClassRec;


/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for widget definition
|
--------------------------------------------------------------------*/

#undef  kwidget
#define kwidget(widget)	     (XvwBrowserWidget) (widget)


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an
|                instance of a XvwBrowserWidget class has been created.
|                This is where the attributes for the class are
|		 initialized. 
|
|    Written By: Mark Young
|          Date: Nov 01, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
        xvobjects_cache_pixmaps();
	xvw_init_attributes(xvwBrowserWidgetClass, attributes,
		XtNumber(attributes), NULL, 0, NULL);
	xvw_load_resources("$DESIGN/objects/library/xvobjects/app-defaults/Browser");
}


/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|	Purpose: This method will set up the initial browser widget instance.
|
|	  Input: request - not used
|                new1     - widget instance after initialization
|    Written By: Mark Young
|          Date: Nov 01, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwBrowserWidget xwid = kwidget(new1);

	xvobject separator;
	char     temp[KLENGTH];
	xvobject object = xvw_object(new1);

	xwid->browser.pixmap = xvw_create_pixmap(object, "pixmap");
	xvw_set_attributes(xwid->browser.pixmap,
		XVW_PIXMAP,	      xwid->browser.directory_pixmap,
		XVW_RIGHT_OF,	      NULL,
		XVW_BELOW,	      NULL,
		NULL);
	xwid->browser.button = xvw_create_button(object, "button");
	xvw_set_attributes(xwid->browser.button,
		XVW_LEFT_OF,	      NULL,
		XVW_RIGHT_OF,	      NULL,
		XVW_ABOVE,	      xwid->browser.pixmap,
		XVW_BELOW,	      xwid->browser.pixmap,
		XVW_LABEL_JUSTIFY,    KLABEL_JUSTIFY_LEFT,
		XVW_LABEL,	      "Directory Browser",
		NULL);

	xwid->browser.select = xvw_create_button(object, "select");
	xwid->browser.cancel = xvw_create_button(object, "quit");
	xwid->browser.help = xvw_create_button(object, "help");
	separator = xvw_create_separator(object, "separator");
	xvw_set_attributes(separator, 
		XVW_VERT_DIST,	      8,
	        XVW_ABOVE,	      xwid->browser.cancel,
		NULL);

	xwid->browser.filename = xvw_create_outputfile(object, "filename");
	xvw_set_attributes(xwid->browser.filename,
		XVW_VERT_DIST,	      8,
		XVW_OUTPUTFILE_DISPLAY_BUTTON,  FALSE,
		XVW_TACK_EDGE,	      KMANAGER_TACK_HORIZ,
		XVW_ABOVE,	      separator,
		NULL);

	xwid->browser.label = xvw_create_label(object, "label");	
	xvw_set_attributes(xwid->browser.label,
                XVW_TACK_EDGE,        KMANAGER_TACK_HORIZ,
                XVW_LABEL,            "Directory / Filename",
                XVW_ABOVE,            xwid->browser.filename,
                NULL);
	
	xwid->browser.list = xvw_create_list(object, "list");
	xvw_set_attributes(xwid->browser.list,
		XVW_VERT_DIST,	      8,
	        XVW_BELOW,            xwid->browser.pixmap,
		XVW_ABOVE,	      xwid->browser.label,
		XVW_TACK_EDGE,	      KMANAGER_TACK_ALL,
		NULL);
	xwid->browser.list = xvw_retrieve_list(xwid->browser.list);

	xvw_set_attributes(xwid->browser.select,
		XVW_VERT_DIST,	      8,
		XVW_LABEL,	      "Ok",
		XVW_CHAR_WIDTH,       7.0,
		XVW_CHAR_HEIGHT,      1.25,
		XVW_RIGHT_OF,         NULL,
		XVW_LEFT_OF,          xwid->browser.cancel,
		XVW_ABOVE,	      NULL,
		NULL);
	xvw_set_attributes(xwid->browser.cancel,
		XVW_VERT_DIST,	      8,
		XVW_LABEL,	      "Cancel",
		XVW_CHAR_WIDTH,       7.0,
		XVW_CHAR_HEIGHT,      1.25,
		XVW_LEFT_OF,          NULL,
		XVW_RIGHT_OF,         NULL,
		XVW_ABOVE,	      NULL,
		NULL);
	xvw_set_attributes(xwid->browser.help,
		XVW_VERT_DIST,	      8,
		XVW_LABEL,	      "Help",
		XVW_CHAR_WIDTH,       7.0,
		XVW_CHAR_HEIGHT,      1.25,
		XVW_LEFT_OF,          NULL,
		XVW_RIGHT_OF,         xwid->browser.cancel,
		XVW_ABOVE,	      NULL,
		NULL);

	xvw_add_callback(xwid->browser.select, XVW_BUTTON_SELECT,
			BrowserCallback, object);
	xvw_add_callback(xwid->browser.cancel, XVW_BUTTON_SELECT,
			BrowserCallback, object);
	xvw_add_callback(xwid->browser.help, XVW_BUTTON_SELECT,
			BrowserCallback, object);
	xvw_add_callback(xwid->browser.list, XVW_LIST_ITEM_ACTION,
			BrowserCallback, object);

        xvw_insert_callback(xwid->browser.list, XVW_LIST_ITEM_SELECT,
                        FALSE, BrowserCallback, object);

	xvw_add_callback(xwid->browser.filename, XVW_OUTPUTFILE_CALLBACK,
			BrowserCallback, object);
	if (xwid->browser.directory != NULL)
	    xvw_add_detectfile(xvw_object(new1), xwid->browser.directory, 1.0, 
	 		       UpdateDirectoryList, NULL);
	/* 
	 * BAD!!  BrowserCallback is written to take a (xvobject, kaddr, kaddr)
	 * and xvw_add_event takes (xvobject, kaddr, XEvent *, int *)
	 * shouldn't use the same routine for both.. Casting it for now --
	 * Steve Jorgensen, Apr 25, 1997
	 */
	xvw_add_event(xwid->browser.pixmap, ButtonReleaseMask,
		(void (*)(xvobject, kaddr, XEvent *, int*))BrowserCallback,
		      object);
	xvw_add_callback(xwid->browser.button, XVW_BUTTON_SELECT,
			BrowserCallback, object);

	xwid->browser.aliases = FALSE;
	xwid->browser.old_directory = NULL;
	xwid->browser.directory = kstrdup(xwid->browser.directory);

	if (kexpandpath("./", NULL, temp))
	    xvw_set_attribute(object, XVW_BROWSER_DIRECTORY, temp);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: This method will free memory allocated by the
|		 Browser object before it is destroyed.
|
|         Input: widget - the widget being destroyed
|    Written By: Danielle Argiro
|          Date: March 14, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	XvwBrowserWidget xwid = kwidget(widget);

	kfree(xwid->browser.directory);
	kfree(xwid->browser.filter_string);
}


/*-----------------------------------------------------------
|
|  Routine Name: SetValues 
|
|       Purpose: This method is used to set the public values 
|                of a XvwBrowserWidget instance.  The public values
|		 which can be changed are the current directory and
|		 the filter being used.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new1     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Dec 13, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwBrowserWidget cxwid = kwidget(current);
	XvwBrowserWidget nxwid = kwidget(new1);

	if (cxwid->browser.directory != nxwid->browser.directory)
	{
	   char *directory;
	   char temp[KLENGTH];

	   if  ((!nxwid->browser.aliases) && 
	        ((nxwid->browser.directory == NULL)|| 
	         (kstrlen(nxwid->browser.directory) == 0)))
	      nxwid->browser.directory = "./";

	   /*
	    *  We want to change the browser mode if the new directory
	    *  being entered contains a ':' and we are in "directory" mode.
	    *  Or if it doesn't contain a ':' and we are in "alias" mode.
	    *  Finally, if none of these conditions exist, then go ahead and
	    *  free the previously supplied directory name.  Note: in the
	    *  first two cases the old directory name is saved, so you better
	    *  not free it!
	    */
	   directory = nxwid->browser.directory;
	   nxwid->browser.directory = cxwid->browser.directory;
	   if (nxwid->browser.aliases == FALSE && kstrchr(directory, ':'))
	      ChangeBrowserMode(xvw_object(new1), TRUE, NULL);
	   else if (nxwid->browser.aliases == TRUE && kstrchr(directory, '/'))
	      ChangeBrowserMode(xvw_object(new1), FALSE, NULL);
	   else
	      kfree_and_NULL(cxwid->browser.directory);

	    nxwid->browser.directory = kstring_cleanup(directory, NULL);

	   /*
	    * enter new directory name in outputfile selection of browser
	    */
 	    kstring_cleanup(directory, temp);
	    if ((nxwid->browser.aliases == FALSE) && 
	        (kstrlen(directory) != 0) && kexpandpath(directory, NULL, temp))
	       xvw_set_attribute(nxwid->browser.filename, 
			         XVW_OUTPUTFILE_FILENAME, temp);
	    else xvw_set_attribute(nxwid->browser.filename, 
                                 XVW_OUTPUTFILE_FILENAME, temp);

	   /*
	    * take UpdateDirectoryList() input handler off old directory,
	    * and install it on the new directory
	    */
	   if (cxwid->browser.directory != NULL)
	       xvw_remove_detectfile(xvw_object(new1),cxwid->browser.directory,
			             UpdateDirectoryList, NULL);
	   if (nxwid->browser.directory != NULL)
	       xvw_add_detectfile(xvw_object(new1), nxwid->browser.directory, 
			          1.0,  UpdateDirectoryList, NULL);
		
	   /*
	    * update the contents of the browser list according to new directory
	    */
	   UpdateBrowserList(xvw_object(new1));
	}

	if (cxwid->browser.filter_string != nxwid->browser.filter_string)
	{
	   kfree_and_NULL(cxwid->browser.filter_string);
	   nxwid->browser.filter_string =
			kstring_cleanup(nxwid->browser.filter_string, NULL);
	   UpdateBrowserList(xvw_object(new1));
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: BrowserCallback
|
|       Purpose: This routine will change the text and update the
|	 	 scrollbar according to the value paased in.
|
|         Input: object      - the object which is calling us
|		 call_data   - none
|		 client_data - the browser gui object
|    Written By: Mark Young 
|          Date: Jun 06, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void BrowserCallback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject browser = (xvobject) client_data;
	XvwBrowserWidget xwid = (XvwBrowserWidget) xvw_widget(browser);

        struct stat buf;
	int    len, aliases = xwid->browser.aliases;
	char   *directory = xwid->browser.directory;
	char   temp[KLENGTH], fullpath[KLENGTH], dir[KLENGTH], *name;
	static int working = FALSE;

	temp[0] = '\0';
	if (object == xwid->browser.list)
	{
	   xvw_list_struct *list = (xvw_list_struct *) call_data; 
	   if (working) return;
	   working = TRUE;
	   if (!xwid->browser.aliases)
	   {
	      /* In directory mode and we have a file, need to strip file off
	         the directory before appending our new selection */
              if (kfullpath(directory, NULL, temp) &&
                  (stat(temp, &buf) == -1 || !S_ISDIR(buf.st_mode)))
                 directory = kdirname(directory, temp);
	   }
	   directory = kstrcpy(dir, directory);

	   if (kstrcmp("../", list->string) == 0)
	   {
	      len = kstrlen(directory);
	      if (aliases == TRUE && len > 0)
	      {
		 if (directory[len-1] == ':')
		    directory[len-1] = '\0';

	         if ((name = kstrrchr(directory, ':')) != NULL)
	         {
		    name[1] = '\0';
		    kstrcpy(temp, directory);
	         }
	         else temp[0] = '\0';
	      }
	      else if (aliases == FALSE)
	      {
		 if (len > 0 && directory[len-1] != '/')
		    ksprintf(temp, "%s/%s", directory, list->string);
		 else if (directory[len-1] == '/')
		    ksprintf(temp, "%s%s", directory, list->string);
		 else
		    kstrcpy(temp, list->string);

	         kexpandpath(temp, NULL, temp);
		 if (kstrcmp(temp, "/") != 0) kstrcat(temp, "/");
	      }
	   }
	   else
	   {
	       if ((len = kstrlen(directory)) == 0)
	          kstrcpy(temp, list->string);
	       else if (aliases == FALSE && directory[len-1] != '/')
	         ksprintf(temp, "%s/%s", directory, list->string);
	       else if (aliases == TRUE  && directory[len-1] != ':')
	         ksprintf(temp, "%s:%s", directory, list->string);
	       else
	         ksprintf(temp, "%s%s", directory, list->string);
	   }

           if (list->action == TRUE &&
	       kfullpath(temp, NULL, fullpath) &&
	       stat(fullpath, &buf) != -1 && !S_ISDIR(buf.st_mode))
	   {
	      name = kstring_cleanup(temp, temp);
	      XtCallCallbacks(xvw_widget(browser), XVW_BROWSER_CALLBACK, &name);
	      BrowserDown(xwid, browser);
	   }
	   else 
	   {
               if ((kstrlen(temp) >  0) && (!xwid->browser.aliases))
	       {
                    kexpandpath(temp, NULL, fullpath);
                    xvw_set_attribute(xwid->browser.filename,
                                      XVW_OUTPUTFILE_FILENAME, fullpath);
	       }
	       else xvw_set_attribute(xwid->browser.filename,
                                      XVW_OUTPUTFILE_FILENAME, temp);

	       if (list->action == TRUE) 
	          xvw_set_attribute(browser, XVW_BROWSER_DIRECTORY, temp);
	   }
	   working = FALSE;
	}
	else if (object == xwid->browser.select)
	{
	   xvw_get_attribute(xwid->browser.filename, 
			     XVW_OUTPUTFILE_FILENAME, &name);

	   if (kfullpath(name, NULL, fullpath) &&
               stat(fullpath, &buf) != -1 && !S_ISDIR(buf.st_mode))
	   {
	      XtCallCallbacks(xvw_widget(browser), XVW_BROWSER_CALLBACK, &name);
	      BrowserDown(xwid, browser);
	   }
	   else
	      xvw_set_attribute(browser, XVW_BROWSER_DIRECTORY, name);
	}
	else if (object == xwid->browser.filename)
	{
	   name = *((char **) call_data); 

	   /* changing from directory mode to aliases mode */
	   if (xwid->browser.aliases == FALSE && kstrchr(name, ':'))
	      ChangeBrowserMode(browser, TRUE, temp);

	   /* changing from aliases mode to directory mode */
	   else if (xwid->browser.aliases == TRUE && kstrchr(name, '/'))
	      ChangeBrowserMode(browser, FALSE, temp);

	   /* user entered name of an existing file as directory/filepath */
	   else if (kfullpath(name, NULL, temp) && 
               stat(temp, &buf) != -1 && S_ISREG(buf.st_mode))
	   {
	      XtCallCallbacks(xvw_widget(browser), XVW_BROWSER_CALLBACK, &name);
	      BrowserDown(xwid, browser);
	   }
		
	   /* user entered existing directory */ 
	   else if (kfullpath(name, NULL, temp) &&
                   stat(temp, &buf) != -1 && S_ISDIR(buf.st_mode))
	       xvw_set_attribute(browser, XVW_BROWSER_DIRECTORY, name);

	   /* user entered something else: must be meant for output? */
	   else
	   {
	      XtCallCallbacks(xvw_widget(browser), XVW_BROWSER_CALLBACK, &name);
	      BrowserDown(xwid, browser);
	   }

	}
	else if (object == xwid->browser.help)
	{
	   xvobject help;

	   help = xvw_create_help(NULL, "Online Help");
           xvw_set_attributes(help,
                XVW_HELP_TITLE, "Online Help for Browser Object",
                XVW_HELP_FILENAME,
			"$DESIGN/objects/library/xvutils/help/Browser.hlp",
                NULL);
	}
	else if (object == xwid->browser.cancel)
	{
           name = NULL;
           XtCallCallbacks(xvw_widget(browser), XVW_BROWSER_CALLBACK, &name);
           BrowserDown(xwid, browser);
	}
	else if (object == xwid->browser.pixmap ||
		 object == xwid->browser.button)
	{
	   if (xwid->browser.aliases == FALSE)
	      ChangeBrowserMode(browser, TRUE, temp);
           else ChangeBrowserMode(browser, FALSE, temp);

	   xvw_set_attribute(browser, XVW_BROWSER_DIRECTORY, temp); 
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: UpdateBrowserList
|
|       Purpose: This routine will update the browser list according
|                to current directory.
|
|         Input: browser - the browser object to update
|    Written By: Mark Young
|          Date: Dec 13, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void UpdateBrowserList(
   xvobject browser)
{
	XvwBrowserWidget xwid = (XvwBrowserWidget) xvw_widget(browser);

	int  i, j, dirsize; 
	size_t size = 0;
	char **list = NULL, temp[KLENGTH], *end, *begin,
	     *directory = xwid->browser.directory;


	/*
	 *  Update the directory list
	 */
	if (xwid->browser.aliases == FALSE)
	{
	   UpdateDirectoryList(browser, directory, NULL);
	}
	else
	{
	   if (alias_array == NULL)
	      alias_array = kalias_list(&num_aliases);

	   if ((end = kstrrchr(directory, ':')) != NULL)
	      dirsize = (end - directory) + 1;
	   else
	      dirsize = kstrlen(directory);

	   list = (char **)karray_add(NULL, KSTRING, kstrdup("../"), size++);
	   for (i = 0; i < num_aliases; i++)
	   {
	       if (kstring_subcmp(alias_array[i], directory) == 0)
	       {
		  begin = &alias_array[i][dirsize];
		  if ((end = kstrchr(begin, ':')) != NULL)
		     kstring_ncopy(begin, (size_t)((end-begin)+1), temp);
		  else
		     kstrcpy(temp, begin);

	          for (j = 0; j < size; j++)
		     if (kstrcmp(list[j], temp) == 0) break;

	          if (j == size)
		     list = (char **)karray_add(list, KSTRING, kstrdup(temp), size++);
	       }
	   }
	   xvw_change_list(xwid->browser.list, list, size, FALSE);
	   karray_free(list, KSTRING, size, NULL);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: UpdateDirectoryList
|
|       Purpose: This input handler is installed on the current directory
|		 when the browser is in 'directory' mode;  every second it 
|		 updates the contents of the browser list according to the
|		 current contents of the directory, so that the browser list 
|		 is kept up to date if the directory contents change.
|
|		 This routine is also called directly from UpdateBrowserList(),
|		 to update the browser contents when the directory is changed.
|
|         Input: browser - the browser object to update
|    Written By: Mark Young & Danielle Argiro
|          Date: March 14, 1996
|
------------------------------------------------------------*/
static int UpdateDirectoryList(
    xvobject browser,
    char    *dirname,
    kaddr    client_data)
{
	XvwBrowserWidget xwid = (XvwBrowserWidget) xvw_widget(browser);

	struct stat buf;
	size_t num, size = 0;
	char   temp[KLENGTH];
	char   *filter = xwid->browser.filter_string;
	char   *directory = xwid->browser.directory;
	char   **browserlist, **tmplist2, **dirlist, **filelist; 

	
	/* browser in aliases mode - don't need to do anything */
	if (xwid->browser.aliases == TRUE) return(TRUE);

	if (xwid->browser.aliases == TRUE) return(TRUE);

        if (kfullpath(directory, NULL, temp) &&
               (stat(temp, &buf) == -1 || !S_ISDIR(buf.st_mode)))
        {
              kdirname(directory, temp);
        }

	/*
         *  List the directories first...
         */
        dirlist = karray_dirlist(NULL, temp, NULL, KDIR, TRUE, &num);
        tmplist2 = (char **)karray_merge(NULL, dirlist, KSTRING, size, num, TRUE);
	karray_free(dirlist,  KSTRING, num, NULL);
        size += num; 

        /*
         *  Now list the files...
         */
        filelist = karray_dirlist(NULL, temp, filter, KFILE, TRUE, &num);
        browserlist = (char **)karray_merge(tmplist2, filelist, KSTRING, size, num,TRUE);
	karray_free(filelist, KSTRING, num, NULL);
	karray_free(tmplist2, KSTRING, size, NULL);
        size += num;

	/*
	 *  Change the contents of the browser list w/ the files+directories
	 */
	xvw_change_list(xwid->browser.list, browserlist, size, FALSE);

	karray_free(browserlist, KSTRING, size, NULL);
	return(TRUE);
}


/*-----------------------------------------------------------
|
|  Routine Name: ChangeBrowserMode
|
|       Purpose: This routine will change the browser mode from aliases
|		 to directory and back again.  It's responsibility is to
|		 update all the internal and visual cues.
|
|         Input: browser - the browser object to update.
|		 aliases - whether to update to aliases or directory mode.
|	 Output: temp    - the directory to update the browser list to.
|    Written By: Mark Young
|          Date: Jan 26, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ChangeBrowserMode(
   xvobject browser,
   int	    aliases,
   char	    *temp)
{
	XvwBrowserWidget xwid = (XvwBrowserWidget) xvw_widget(browser);

	Pixmap pixmap;
	char   *label, *button;

	if (aliases == TRUE)
	{
	   button  = "Aliases Browser";
           label  = "Directory : Alias";
	   pixmap = xwid->browser.aliases_pixmap;
	}
	else
	{
	   button = "Directory Browser";
           label  = "Directory / Filename";
	   pixmap = xwid->browser.directory_pixmap;
	}
	xwid->browser.aliases = aliases;
	if (temp) kstrcpy(temp, xwid->browser.old_directory);
	kfree(xwid->browser.old_directory);
	xwid->browser.old_directory = xwid->browser.directory;
	xwid->browser.directory = NULL;

	xvw_set_attribute(xwid->browser.label,  XVW_LABEL,  label);
	xvw_set_attribute(xwid->browser.button, XVW_LABEL,  button);
	xvw_set_attribute(xwid->browser.pixmap, XVW_PIXMAP, pixmap);
}

/*-----------------------------------------------------------
|
|  Routine Name: BrowserDown
|
|       Purpose: Pops down the browser; destroys the browser if
|		 the XVW_BROWSER_DESTROY_ON_QUIT attribute is set
|		 to true, simply unmaps it otherwise
|
|         Input: xwid    - the Browser widget
|		 browser - the browser object
|        Output: 
|    Written By: Danielle Argiro
|          Date: Feb 13, 1997
|
------------------------------------------------------------*/
/* ARGSUSED */
static void BrowserDown(
   XvwBrowserWidget xwid, 
   xvobject browser)
{
	if (xwid->browser.destroy_on_quit)
	{
            xvw_destroy(browser);
	}
        else 
	{
	    xvw_set_attribute(xwid->browser.list, XVW_LIST_INDEX, -1);
	    xvw_unmap(xvw_toplevel(browser));
	}
}

/************************************************************
*
*  Routine Name: xvw_create_browser - create a browser GUI object
*
*       Purpose: The browser GUI object
*                allows the user to enter an input file.  It features a 
*                list object in which the user may select a file, or a
*		 text box in which the user may type in the filename;  
*                the filename is registered when the user hits <cr> or
*		 selects a file from the browser list.  
*                
*                The browser GUI object consists of a manager object 
*                with six children: two button object, a list object, 
*                a label object, a textinput object, and a textinput
*		 object.
*
*                A callback can be installed on the browser object, which
*                will be fired when the user enters a new filename.
*
*         Input: parent - parent of the browser object; NULL will cause
*                         a default toplevel to be created automatically
*                name   - a name for this particular instance of the object
*                         (for use in app-defaults files, etc)
*
*        Output:
*       Returns: The browser object on success, NULL on failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Nov 01, 1993
*      Verified: 
*  Side Effects:
* Modifications:
*
*******************************************************************/

xvobject xvw_create_browser(
   xvobject parent,
   char     *name)
{
	xvobject object;


	object = xvw_create(parent, TRUE, TRUE, name, BrowserWidgetClass);
	return(object);
}
