 /*
  * Khoros: $Id: Notebook.c,v 1.7.2.1 1997/04/29 22:32:53 mirage Exp $
  */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Notebook.c,v 1.7.2.1 1997/04/29 22:32:53 mirage Exp $";
#endif
 
 
/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Notebook Widget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 ConstraintInitialize()
   >>>>                 SetValues()
   >>>>                 ConstraintDestroy()
   >>>>                 ChangeSelected()
   >>>>                 ChangeHandler()
   >>>>   Public:
   >>>>                 xvw_create_notebook()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvobjects/NotebookP.h>

static void ClassInitialize      PROTO((void));
static void Initialize		 PROTO((Widget, Widget, ArgList, Cardinal *));
static void ConstraintInitialize PROTO((Widget, Widget, ArgList, Cardinal *));
static void ConstraintDestroy    PROTO((Widget));
static Boolean SetValues         PROTO((Widget, Widget, Widget, ArgList,
				   Cardinal *));
static void ChangeSelected PROTO((Widget, Widget));
static void ChangeHandler  PROTO((xvobject, kaddr, kaddr));


/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_NOTEBOOK_BUFFER_SIZE,	    NULL,    XtRInt,	   NULL},
{XVW_NOTEBOOK_CALLBACK,	            NULL,    XtRCallback,  NULL},
{XVW_NOTEBOOK_SELECTED_CHILD,	    NULL,    XtRObject,    XtRWidget},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwNotebookWidgetRec, notebook.field)

static XtResource resources[] = { 
{XVW_NOTEBOOK_BUFFER_SIZE, NULL, XtRInt, sizeof(int),
      offset(buffer_size), XtRImmediate, (XtPointer) 5},
{XVW_NOTEBOOK_CALLBACK, NULL, XtRCallback, sizeof(XtPointer),
      offset(notebook_callback), XtRCallback, (XtPointer) NULL},
{XVW_NOTEBOOK_SELECTED_CHILD, NULL, XtRWidget, sizeof(Widget),
      offset(selected), XtRImmediate, (XtPointer) NULL},

{XVW_SHADOW_TYPE, NULL, XtRInt, sizeof(int),
        XtOffsetOf(XvwNotebookWidgetRec, manager.shadow_type),
        XtRImmediate, (XtPointer) KSHADOW_OUT},
};
#undef offset


#define SUPERCLASS (&xvwManagerWidgetClassRec)

XvwNotebookWidgetClassRec xvwNotebookWidgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Notebook",				/* class_name		  */
    sizeof(XvwNotebookWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    XtInheritTranslations,		/* tm_table		  */
    NULL,				/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,				/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,			            /* subresources       */
    0,					    /* subresources_count */
    sizeof(XvwManagerWidgetConstraintsRec), /* constraint_size    */
    ConstraintInitialize,		    /* initialize         */
    ConstraintDestroy,                      /* destroy            */
    NULL,				    /* set_values         */
    NULL,                                   /* extension          */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    XtInheritChangeSel,			    /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  }, /* XvwManagerWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwNotebookWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwNotebookWidgetClass for public consumption
 */
WidgetClass xvwNotebookWidgetClass = (WidgetClass) &xvwNotebookWidgetClassRec;

/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for Class and Constraint Declarations
|
--------------------------------------------------------------------*/

#define kwidget(widget)      (XvwNotebookWidget) (widget)


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of a xvwNotebookWidgetClass has been created. 
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young
|          Date: Apr 15, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
        xvobjects_cache_pixmaps();
        xvw_init_attributes(xvwNotebookWidgetClass, attributes,
		XtNumber(attributes), NULL, 0, 
		"$DESIGN/objects/library/xvobjects/uis/Notebook.pane");
	xvw_load_resources("$DESIGN/objects/library/xvobjects/app-defaults/Notebook");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will create all the subparts to the
|		 notebook widget.  They consist of a horizontal &
|		 vertical rowcol area to manage the tab indexes.
|	
|         Input: request  - the requested viewport
|		 new1      - the viewport widget 
|                args     - not used 
|                nun_args - not used 
|    Written By: Mark Young
|          Date: Apr 15, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwNotebookWidget xwid   = kwidget(new1);

	/*
	 * Initialize the tabs to be empty.
	 */
	xwid->notebook.tabs     = NULL;
	xwid->notebook.override = FALSE;
}

/*-----------------------------------------------------------
|
|  Routine Name: ConstraintInitialize
|
|       Purpose: This method is used to overide the translations
|                of a child of the XvwManagerWidget and to
|                install the accelerators from the XvwManagerWidget
|                into the child.
|
|         Input: request - widget child with requested constraints
|                new1     - widget child with overidden translations and
|                          installed accelerators
|    Written By: Mark Young
|          Date: Apr 15, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ConstraintInitialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwNotebookWidget xwid = (XvwNotebookWidget) XtParent(new1);

	if (!xwid->notebook.override && XtIsSubclass(new1, ManagerWidgetClass))
	{
	   klist    *tail;
	   xvobject tab, last, selected = xvw_object(new1);

	   tail = klist_tail(xwid->notebook.tabs);
	   last = (xvobject) klist_clientdata(tail);

	   xwid->notebook.override = TRUE;
	   tab = xvw_create_button(xvw_parent(selected), "tab");
	   xvw_set_attributes(tab,
		XVW_HORIZ_DIST, 0,
		XVW_BELOW,     NULL,
		XVW_RIGHT_OF,  last,
		XVW_LABEL,     XtName(new1),
		XVW_CHAR_HEIGHT, 1.2,
		NULL);
	   xwid->notebook.override = FALSE;
	   xvw_set_attributes(selected,
		XVW_TACK_EDGE, KMANAGER_TACK_ALL,
		XVW_VERT_DIST, 0,
		XVW_BELOW, tab,
		NULL);

	   xwid->notebook.tabs = klist_add(xwid->notebook.tabs, new1, tab);
	   xvw_insert_callback(tab, XVW_BUTTON_SELECT, FALSE,
			ChangeHandler, xwid);
	   ChangeSelected((Widget) xwid, new1);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: Determine what the widget should do by
|		 comparing the current public settable values 
|		 versus the new settings.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new1     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Apr 15, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
       XvwNotebookWidget cxwid = (XvwNotebookWidget) current;
       XvwNotebookWidget nxwid = (XvwNotebookWidget) new1;


       if (cxwid->notebook.selected != nxwid->notebook.selected)
       {
	  if (nxwid->notebook.selected != NULL)
	     ChangeSelected(new1, nxwid->notebook.selected);
       }
     
       if (cxwid->notebook.buffer_size != nxwid->notebook.buffer_size)
	  ManagerRequestLayout(new1, FALSE);

       return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ConstraintDestroy
|
|       Purpose: This method is used to change the selected area to the
|		 first area in the list, if the child being destroyed is
|		 the currently selected area.
|
|         Input: child   - child widget to be deleted
|    Written By: Mark Young
|          Date: Apr 15, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ConstraintDestroy(
   Widget child)
{
	XvwNotebookWidget xwid = (XvwNotebookWidget) XtParent(child);

	klist    *entry, *tmp;
	xvobject tab, prev, next, selected = NULL;

	if (xwid->core.being_destroyed == TRUE)
	   return;

	/*
	 *  Destroy the tab and the entry within the notebook tab list.
	 */
	if ((entry = klist_locate(xwid->notebook.tabs, child)) == NULL)
	   return;

	/*
	 *  This section of code serves two purposes.  First is to get the
	 *  next and prev tabs in the list, since we need to set the next
	 *  tab to be right of the prev tab.  The second purpose is to set
	 *  a new "selected" if the one being deleted is currently "selected".
	 */
	tmp = klist_next(entry);
	next = (xvobject) klist_clientdata(tmp);
	if (tmp != NULL)
	   selected = xvw_object((Widget) klist_identifier(tmp));

	tmp = klist_prev(entry);
	prev = (xvobject) klist_clientdata(tmp);
	if (tmp != NULL && selected == NULL)
	   selected = xvw_object((Widget) klist_identifier(tmp));

	/* if "next" tab is not NULL set it to be right of the "prev" tab */
	if (next != NULL) xvw_set_attribute(next, XVW_RIGHT_OF, prev);

	/*
	 *  See if the child was the selected workspace.
	 */
	tab = (xvobject) klist_clientdata(entry); xvw_destroy(tab);
	xwid->notebook.tabs = klist_delete(xwid->notebook.tabs, child);
	if (xwid->notebook.selected == child)
	{
	   ChangeSelected((Widget) xwid, xvw_widget(selected));
	   XtCallCallbacks((Widget) xwid, XVW_NOTEBOOK_CALLBACK, &selected);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: ChangeHandler 
|
|       Purpose: Handler to call ChangeSelected and also invoke
|		 the inform callback.
|
|         Input: object - the newly selected object
|		 client_data  - the event
|		 event  - the event
|    Written By: Mark Young
|          Date: Apr 15, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ChangeHandler(
   xvobject tab,
   kaddr    client_data,
   kaddr    call_data)
{
	XvwNotebookWidget xwid = (XvwNotebookWidget) client_data;

	klist    *entry;
	Widget   child;
	xvobject selected;

	entry = klist_locate_clientdata(xwid->notebook.tabs, tab);
	child = (Widget) klist_identifier(entry);

	selected = xvw_object(child);
	XtCallCallbacks((Widget) xwid, XVW_NOTEBOOK_CALLBACK, &selected);
	ChangeSelected((Widget) xwid, child);
}

/*-----------------------------------------------------------
|
|  Routine Name: ChangeSelected 
|
|       Purpose: When the selected child attribute
|		 has been changed, this routine will update
|		 the child so it "looks" selected by puttin
|		 a border around it.
|
|         Input: object - the newly selected object
|    Written By: Mark Young
|          Date: Apr 15, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ChangeSelected(
   Widget   widget,
   Widget   child)
{
	XvwNotebookWidget xwid = (XvwNotebookWidget) widget;

	xvobject tab;
	klist    *entry;

	if (xwid->notebook.selected != NULL)
	{
	   xvw_unmap(xvw_object(xwid->notebook.selected));

	   if ((entry = (klist *) klist_locate(xwid->notebook.tabs,
				xwid->notebook.selected)) != NULL)
	   {
	      tab = (xvobject) klist_clientdata(entry);
	      xvw_set_attribute(tab, XVW_BUTTON_SELECTED, FALSE);
	   }
	}
	xwid->notebook.selected = child;

	if (child == NULL)
	   return;

	xvw_map(xvw_object(child));

	entry = (klist *) klist_locate(xwid->notebook.tabs, child);
	tab = (xvobject) klist_clientdata(entry);
	xvw_set_attribute(tab, XVW_BUTTON_SELECTED, TRUE);
	if (klist_size(xwid->notebook.tabs) > 1)
	{
	   xvw_manage(tab);
	   xvw_manage((xvobject) klist_clientdata(xwid->notebook.tabs));
	}
	else
	   xvw_unmanage(tab);
}


/************************************************************
*
*  Routine Name: xvw_create_notebook - create a notebook object
*
*       Purpose: The notebook object is designed for doing notebook of 
*		 objects that are subclassed from the Manager widget.  
*		 Only objects subclassed from the manager widget (\fInot\fP
*		 the manager object) may be created as children of the
*		 notebook object.  Such objects include area objects, 
*		 image objects, zoom objects, viewport objects, and so on.  
*		 See Chapter 1, "Introduction", of the Programming Services 
*		 Manual, Volume 3, for diagrams depicting the objects that 
*		 are subclassed from the Manager widget.  
*
*		 The notebook object allows you to do quick and easy
*		 notebook when a variety of such objects share a common
*		 backplane.  You may specify the number of objects
*		 that should appear in a single row;  relative notebook
*		 specifications are not needed.  
*
*		 The notebook object is especially effective when laying
*		 out objects \fIof the same type\fP, as it will preserve
*		 proportionality between the objects.  It is often used
*		 in applications such as \fHxprism\fP, where the user
*		 will be interactively creating new objects to display
*		 data.  The application need not implement special
*		 code to do appropriate notebook of new objects as they
*		 are created, since the notebook object does it 
*		 automatically according to initial specifications.   
*
*         Input: parent - the parent of the notebook object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the object
*                         (for use in app-defaults files, etc)
*
*        Output:
*	Returns: The notebook object on success, NULL on failure
*  Restrictions:
*    Written By: Mark Young
*          Date: Apr 15, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_notebook(
   xvobject parent,
   char     *name)
{
	xvobject object;

	/*
	 *  Call the xvw_create() routine to do the actual creation.
	 */
	object = xvw_create(parent, FALSE, TRUE, name, NotebookWidgetClass);
	return(object);
}
