 /*
  * Khoros: $Id: TextDisplay.c,v 1.24.2.6 1997/06/04 16:47:38 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: TextDisplay.c,v 1.24.2.6 1997/06/04 16:47:38 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros TextDisplay Widget
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Destroy()
   >>>>                 SetValues()
   >>>>                 SetFilename()
   >>>>                 GetFilename()
   >>>>                 ChangeTextDisplay()
   >>>>                 AddText()
   >>>>                 ClearText()
   >>>>                 ReadText()
   >>>>   Public:
   >>>>                 xvw_create_textdisplay()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include <xvobjects/TextDisplayP.h>


static void     ClassInitialize PROTO((void));
static void     Initialize	PROTO((Widget, Widget, ArgList, Cardinal *));
static void     Destroy		PROTO((Widget));
static Boolean	SetValues       PROTO((Widget, Widget, Widget, ArgList,
				       Cardinal *));

static int  SetFilename		PROTO((xvobject, char *, kaddr));
static int  GetFilename		PROTO((xvobject, char *, kaddr));
static int  ChangeTextDisplay	PROTO((xvobject, char *, kaddr));

static void AddText   PROTO((XvwTextDisplayWidget, char *));
static void ClearText PROTO((XvwTextDisplayWidget));
static void ReadText  PROTO((XvwTextDisplayWidget));

#define MANPAGE_ASCII  "6x13"
#define MANPAGE_ROMAN  "-adobe-helvetica-medium-r-normal--12-*-iso8859-1"

/*-------------------------------------------------------------------*
|
|   Full class attributes
|
--------------------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_TEXTDISPLAY_ROFF,    NULL,  XtRInt,	 XtRBoolean},
{XVW_TEXTDISPLAY_INDENT,  NULL,  XtRInt,	 XtRDimension},
{XVW_TEXTDISPLAY_FILE,	  NULL,  XtRPointer,     NULL},
{XVW_TEXTDISPLAY_ASCII,   NULL,  XtRFontStruct,  NULL},
{XVW_TEXTDISPLAY_ROMAN,   NULL,  XtRFontStruct,  NULL},
{XVW_TEXTDISPLAY_CALLBACK,NULL,  XtRCallback,    NULL},
{XVW_TEXTDISPLAY_ASCII_FONTNAME, XVW_TEXTDISPLAY_ASCII,XtRString,XtRFontStruct},
{XVW_TEXTDISPLAY_ROMAN_FONTNAME, XVW_TEXTDISPLAY_ROMAN,XtRString,XtRFontStruct},
{XVW_FONT,      XVW_TEXTDISPLAY_ROMAN,  XtRFontStruct,  NULL},
{XVW_FONTNAME,  XVW_TEXTDISPLAY_ROMAN,  XtRString,      XtRFontStruct},
};


/*-------------------------------------------------------------------*
|
|   Full class record constant
|
--------------------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwTextDisplayWidgetRec, textdisplay.field)

static XtResource resources[] = { 
{XVW_TEXTDISPLAY_ASCII, XVW_TEXTDISPLAY_ASCII_FONTNAME, XtRFontStruct, 
        sizeof(XFontStruct *), offset(ascii), XtRString, MANPAGE_ASCII},
{XVW_TEXTDISPLAY_ROMAN, XVW_TEXTDISPLAY_ROMAN_FONTNAME, XtRFontStruct, 
        sizeof(XFontStruct *), offset(roman), XtRString, MANPAGE_ROMAN},
{XVW_TEXTDISPLAY_ROFF, NULL, XtRBoolean, 
        sizeof(Boolean), offset(roff), XtRImmediate, (XtPointer) FALSE},
{XVW_TEXTDISPLAY_INDENT, NULL, XtRDimension, 
        sizeof(Dimension), offset(indent), XtRImmediate, (XtPointer) 10},
{XVW_TEXTDISPLAY_FILE, NULL, XtRPointer,   
        sizeof(XtPointer), offset(file), XtRImmediate, (XtPointer) NULL},
{XVW_TEXTDISPLAY_CALLBACK, NULL, XtRCallback, sizeof(XtPointer),
      offset(textdisplay_callbacks), XtRCallback, (XtPointer) NULL},

{XVW_VP_FORCE_HORIZ, NULL, XtRBoolean, sizeof(Boolean),
	XtOffsetOf(XvwViewportWidgetRec, viewport.force_horizontal),
        XtRImmediate, (XtPointer) FALSE},
{XVW_VP_FORCE_VERT, NULL, XtRBoolean, sizeof(Boolean),
	XtOffsetOf(XvwViewportWidgetRec, viewport.force_vertical),
        XtRImmediate, (XtPointer) FALSE},
{XVW_VP_ALLOW_VERT, NULL, XtRBoolean, sizeof(Boolean),
	XtOffsetOf(XvwViewportWidgetRec, viewport.allow_vertical),
        XtRImmediate, (XtPointer) TRUE},
{XVW_VP_ALLOW_HORIZ, NULL, XtRBoolean, sizeof(Boolean),
	XtOffsetOf(XvwViewportWidgetRec, viewport.allow_horizontal),
        XtRImmediate, (XtPointer) TRUE},
{XVW_VP_USE_RIGHT, NULL, XtRBoolean, sizeof(Boolean),
	XtOffsetOf(XvwViewportWidgetRec, viewport.use_right),
        XtRImmediate, (XtPointer) TRUE},
};
#undef offset


/*-------------------------------------------------------------------*
|
|   Class Declaration for TextDisplay Widget
|
--------------------------------------------------------------------*/

#define superclass (&xvwViewportWidgetClassRec)

XvwTextDisplayWidgetClassRec xvwTextDisplayWidgetClassRec =
{
  {
    (WidgetClass) superclass,		/* superclass		  */	
    "TextDisplay",			/* class_name		  */
    sizeof(XvwTextDisplayWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    XtInheritQueryGeometry,		/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwTextDisplayWidgetConstraintsRec),  /* constraint_size	  */
    NULL,				    /* initialize	  */
    NULL,				    /* destroy		  */
    NULL,				    /* set_values	  */
    NULL,				    /* extension	  */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    XtInheritChangeSel,			    /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  },  /* XvwManagerWidgetClass fields initialization */
  {
    NULL,  		                    /* extension          */
  },  /* XvwTextDisplayWidgetClass fields initialization */
  {
    NULL,  		                    /* extension          */
  },  /* XvwTextDisplayWidgetClass fields initialization */
};
#undef superclass

  /* for public consumption */
WidgetClass xvwTextDisplayWidgetClass = (WidgetClass) &xvwTextDisplayWidgetClassRec;


/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for Class and Constraint Declarations
|
--------------------------------------------------------------------*/

#undef  kwidget
#define kwidget(widget)	(XvwTextDisplayWidget) (widget)


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an
|                instance of a XvwTextDisplayWidget class has been created.
|                This is where the attributes for the class are
|		 initialized. 
|
|    Written By: Mark Young
|          Date: Nov 01, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
        xvobjects_cache_pixmaps();
	xvw_init_attributes(xvwTextDisplayWidgetClass, attributes,
		XtNumber(attributes), NULL, 0, NULL);
	xvw_load_resources("$DESIGN/objects/library/xvobjects/app-defaults/TextDisplay");

	xvw_define_attributes(xvwTextDisplayWidgetClass,
		XVW_TEXTDISPLAY_ADDTEXT,    XtRString, ChangeTextDisplay, NULL,
		XVW_TEXTDISPLAY_CLEARTEXT,  XtRInt,    ChangeTextDisplay, NULL,
		XVW_TEXTDISPLAY_FILENAME, XtRString, SetFilename, GetFilename,
		NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial textdisplay
|                for the textdisplay widget instance.
|
|         Input: request - not used
|                new1    - widget instance after initialization, with
|                          initial image
|    Written By: Mark Young
|          Date: Nov 01, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num_args)
{
	XvwTextDisplayWidget xwid = kwidget(new1);

	XFontStruct *font;

	/*
	 *  Set the font according to whether we are in roff mode or not.
	 */
	if (xwid->textdisplay.roff == TRUE)
	   font = xwid->textdisplay.roman;
	else
	   font = xwid->textdisplay.ascii;

	/*
	 *  Create the label in which to display the text
	 */
	xwid->textdisplay.label = xvw_create_label(xvw_object(new1), "Label");
        xvw_set_attributes(xwid->textdisplay.label,
                XVW_LABEL_CURSOR_POS,     0,
                XVW_LABEL_MULTILINE,      TRUE,
                XVW_LABEL_DISPLAY_CURSOR, FALSE,
                XVW_LABEL_SELECTABLE,     TRUE,
                XVW_SHADOW_TYPE,          KSHADOW_NONE,
                XVW_BORDER_WIDTH,         0,
		XVW_FONT,		  font,
                NULL);

	/*
	 *  Initialize the notifywindow to NULL
	 */
	xwid->textdisplay.notifywindow = NULL;

	/*
	 *  Change the cursor on the text window to be the xterm
	 */
	xvw_set_attribute(xwid->viewport.plane, XVW_CURSORNAME, "xterm");

	/*
	 *  Initialize the text to NULL.
	 */
	xwid->textdisplay.text = NULL;
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: Close the bold, italic, symbol, helvetica, and roman font.
|
|         Input: widget - the textdisplay widget being destroyed
|    Written By: Mark Young
|          Date: Nov 01, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
        XvwTextDisplayWidget xwid = kwidget(widget);
 
	/*
	 *  If the textdisplay file is open, then close it...
	 */
	if (xwid->textdisplay.file != NULL)
	   kfclose(xwid->textdisplay.file);

	kfree(xwid->textdisplay.text);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the public values
|                of a XvwTextDisplayWidget instance.  The public values
|                which can be changed are all related to the display
|		 of the textdisplay.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new1     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 01, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues (
   Widget   current,
   Widget   request,
   Widget   new1,
   ArgList  args,
   Cardinal *num)
{
	XvwTextDisplayWidget cxwid = kwidget(current);
	XvwTextDisplayWidget nxwid = kwidget(new1);

	long int pos;

	if (cxwid->textdisplay.file != nxwid->textdisplay.file)
	{
	   if (cxwid->textdisplay.file != NULL)
	      kfclose(cxwid->textdisplay.file);

	   ClearText(nxwid);
	   pos = kftell(nxwid->textdisplay.file);
	   ReadText(nxwid);
	   kfseek(nxwid->textdisplay.file, pos, KSEEK_SET);
	   nxwid->textdisplay.file = NULL;
	}

        if (cxwid->textdisplay.roff != nxwid->textdisplay.roff)
        {
	   XFontStruct *font;

	   if (nxwid->textdisplay.roff == TRUE)
	      font = nxwid->textdisplay.roman;
	   else
	      font = nxwid->textdisplay.ascii;

	   xvw_set_attribute(nxwid->textdisplay.label, XVW_FONT, font);
        }

        if (cxwid->textdisplay.indent != nxwid->textdisplay.indent)
        {
        }
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetFilename - sets the filename to be displayed
|
|       Purpose: Open and display the textdisplay file.
|
|         Input: xvobject - the textdisplay object
|                attribute - the attribute XVW_TEXTDISPLAY_FILENAME
|                calldata  - the name of the filename to be displayed
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 01, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static int SetFilename(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	char **string = (char **) calldata;
	kfile *file;

	if ((file = kfopen(*string, "r")) == NULL)
	{
	   kerror(XVOBJECTS, "SetFilename", "can't open input file '%s'",
		  *string);
	   return(FALSE);
	}
	return(xvw_set_attribute(object, XVW_TEXTDISPLAY_FILE, file));
}

/*-----------------------------------------------------------
|
|  Routine Name: GetFilename - gets the filename currently being displayed
|
|       Purpose: Get the name of the file being displayed.
|
|         Input: xvobject - the textdisplay object
|                attribute - the attribute XVW_TEXTDISPLAY_FILENAME
|        Output: calldata  - the name of the filename being displayed
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 01, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static int GetFilename(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	char **string = (char **) calldata;
	XvwTextDisplayWidget xwid = (XvwTextDisplayWidget) xvw_widget(object);


	if ((*string = kfile_filename(kfileno(xwid->textdisplay.file))) == NULL)
	   return(FALSE);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ChangeTextDisplay - changes the current list of words
|
|       Purpose: This routine is used to manipulate the list of words
|		 that we perform the textdisplay callback
|
|         Input: xvobject - the textdisplay object
|		 attribute - the attribute which is one of:
|				XVW_TEXTDISPLAY_ADDWORD
|				XVW_TEXTDISPLAY_DELETEWORD
|				XVW_TEXTDISPLAY_CLEARWORD
|				XVW_TEXTDISPLAY_ADDTEXT
|				XVW_TEXTDISPLAY_CLEARTEXT
|		 calldata - the word to be added, deleted, or to clear the list
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 01, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static int ChangeTextDisplay(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
        XvwTextDisplayWidget xwid = (XvwTextDisplayWidget) xvw_widget(object);
 

	if (kstrcmp(attribute, XVW_TEXTDISPLAY_ADDTEXT) == 0)
	{
	   char *text = *((char **) calldata);

	   /*
	    *  Add Text and reset the cursor position to the end of the file.
	    */
	   AddText(xwid, text);
	}
	else if (kstrcmp(attribute, XVW_TEXTDISPLAY_CLEARTEXT) == 0)
	{
	   ClearText(xwid);
	}
	else
	{
	   kerror(XVOBJECTS, "ChangeTextDisplay", "invalid attribute %s",
			attribute);
	   return(FALSE);
	}
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: AddText - 
|
|       Purpose: This routine adds to the contents given the supplied
|		 text.
|
|         Input: xwid - the text display widget
|    Written By: Mark Young
|          Date: Nov 01, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void AddText(
   XvwTextDisplayWidget xwid,
   char  *text)
{
	int  length;
	char *newtext, *tmp;

	static size_t  bufsiz = 0;
	static char *buffer = NULL;

	/*
	 *  Sanity check
	 */
	if (text == NULL)
	   return;

        /*
         * Go through the file and cleanup the code by deleting any roff
         * characters.
         */
	if (xwid->textdisplay.roff == TRUE)
	{
	   int indx = 0;

	   while (text != NULL)
	   {
	      if ((tmp = kstrchr(text, '')) != NULL)
	      {
	         length = (tmp-text)-1;
		 tmp++;
	      }
	      else
		 length = kstrlen(text);

	      /*
	       *  Better make sure we got enough temporary space to copy the
	       *  string to.  We actually need length bytes plus one for the
	       *  roff character, plus one final byte to terminate the
	       *  temporary buffer.
	       */
	      while (indx+length+1 >= bufsiz)
	      {
		 bufsiz += KLENGTH;
		 buffer = (char *) krealloc(buffer, bufsiz);
	      }

	      if (length > 0)
	      {
	         kstrncpy(&buffer[indx], text, (size_t) length);
		 indx += length;
	      }

	      /*
	       * if tmp is not NULL then this means we found a roff
	       * character.  What we need to is store it and advance
	       * tmp
	       */
	      if (tmp != NULL)
		 buffer[indx++] = *(tmp++);

	      /* advance the text string */
	      text = tmp;
	   }
	   buffer[indx] = '\0';
	   newtext = kstring_cat(xwid->textdisplay.text, buffer, NULL);
	}
	else
	   newtext = kstring_cat(xwid->textdisplay.text, text, NULL);

	/*
	 *  Append text to our current textdisplay "text".
	 */
	kfree(xwid->textdisplay.text);
	xwid->textdisplay.text = newtext;
 
	/*
	 * Update the label...
	 */
	xvw_set_attribute(xwid->textdisplay.label, XVW_LABEL, newtext);
}

/*-----------------------------------------------------------
|
|  Routine Name: ClearText - 
|
|       Purpose: This routine clears the contents from the text display
|		 object.
|
|         Input: xwid - the text display widget
|    Written By: Mark Young
|          Date: Nov 13, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClearText(
   XvwTextDisplayWidget xwid)
{
	kfree_and_NULL(xwid->textdisplay.text);
	xvw_set_attribute(xwid->textdisplay.label, XVW_LABEL, NULL);
	xvw_set_attributes(xvw_object((Widget) xwid),
		XVW_VP_XOFFSET, 0,
		XVW_VP_YOFFSET, 0,
		NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: ReadText - 
|
|       Purpose: This routine reads the contents from the khoros transport
|		 file into the textdisplay.
|
|         Input: xwid - the text display widget
|    Written By: Mark Young
|          Date: Nov 01, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ReadText(
   XvwTextDisplayWidget xwid)
{
	int   fid;
	char  *text, *data, *tmp;


	fid = kfileno(xwid->textdisplay.file);
	xvw_set_attribute(xwid->viewport.vertical, XVW_SCROLL_VALUE, 0.0);

	if (xwid->textdisplay.notifywindow == NULL)
	{
	   xvobject object = xvw_object((Widget) xwid);

	   xwid->viewport.override = TRUE;
	   xwid->textdisplay.notifywindow = xvw_create_notifywindow(object,
		"notifywindow");
	   xvw_set_attributes(xwid->textdisplay.notifywindow,
		XVW_NOTIFYWINDOW_VISIBLE, FALSE,
		XVW_ABOVE,    NULL,
		XVW_BELOW,    NULL,
		XVW_LEFT_OF,  NULL,
		XVW_RIGHT_OF, NULL,
		NULL);
	   xwid->viewport.override = FALSE;
	}
	xvw_set_attributes(xwid->textdisplay.notifywindow,
		XVW_NOTIFYWINDOW_VISIBLE, TRUE,
		XVW_NOTIFYWINDOW_MESSAGE, "Formatting Page....",
		NULL);
	/*
	 * Copy the file into memory. 
	 */
	if ((text = (char *)kfile_readdata(fid, NULL, NULL)) == NULL)
	{
	   kerror(XVOBJECTS, "TextDisplay",
		  "Failure to read data from the file '%s'.",
		  kfile_filename(fid));
	}
	xvw_set_attribute(xwid->textdisplay.notifywindow,
		XVW_NOTIFYWINDOW_VISIBLE, FALSE);

	/*
	 *  MY/JS: Cleanup the string for multiple newlines.  We need to
	 *	   do this since .bp add bogus amounts of newlines.
	 */
	if (xwid->textdisplay.roff == TRUE)
	{
	   data = kstring_seddata(text, "\n\n+", "\n\n", TRUE, NULL);
	   kfree(text); text = data;
	}

	data = text;
	while (*data == '\n' && *data != '\0')
	   data++;

	if (data != text)
	   data--;

	if (*data != '\0')
	{
	   tmp = data + kstrlen(data) - 1;
	   while (*tmp == '\n' && *tmp != '\0')
	      tmp--;
	   tmp[1] = '\0';
	}
	AddText(xwid, data);
	kfree_and_NULL(text);
}


/************************************************************
*
*  Routine Name: xvw_create_textdisplay - create a textdisplay object
*
*       Purpose: The textdisplay object supports display of text files.
*
*		 For files that were output from \fIroff\fP interpreters,
*		 the textdisplay object will format the text as specified
*		 by the control characters produced by the \fIroff\fP 
*		 interpreter.  In other words, the textdisplay object cannot 
*		 format files with \fIroff\fP commands still in them;  rather, 
*		 it supports display of files that were output by a \fIroff\fP
*		 interpreter such as \fInroff\fP or \fIgroff\fP, after such
*		 a program was already run on the file with the \fIroff\fP
*		 commands in it.  
*
*		 Of course, the textdisplay object also supports display
*		 of plain ascii text files.
*
*		 The textdisplay object supports specification of five 
*		 different fonts, the roman font that is used with "normal" 
*		 text, as well as bold, italic, and helvetica fonts for emphasis
*		 and a symbol font for use with equations.
*
*	         The textdisplay object can also be used to support
*		 hypertext-type functionality.  The textdisplay object
*		 is capable of keeping a special list of words that 
*		 might appear in the text;  such "special" words will
*		 appear in a different color from the rest of the text.
*		 A callback can be installed on the textdisplay object 
*		 which will be fired when the user clicks on a word that
*		 is part of the textdisplay list of "special" words.
*		 Thus, the application can use this capability to display
*		 a new text file determined by the word selected by the user.
*
*         Input: parent - parent of the textdisplay object; NULL will cause
*                         a default toplevel to be created automatically
*                name   - a name for this particular instance of the object
*                         (for use in app-defaults files, etc)
*
*        Output: 
*       Returns: The textdisplay object on success, NULL on failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Nov 01, 1993
*      Verified: 
*  Side Effects:
* Modifications:
*
*******************************************************************/

xvobject xvw_create_textdisplay(
   xvobject parent,
   char     *name)
{
	xvobject object;

	object = xvw_create(parent, TRUE, TRUE, name, TextDisplayWidgetClass);
	return(object);
}
