 /*
  * Khoros: $Id: prompt.c,v 1.16.2.1 1997/04/29 22:36:32 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: prompt.c,v 1.16.2.1 1997/04/29 22:36:32 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		        Prompt Dialog Box
   >>>>
   >>>>  Private:
   >>>>                 xvu_prompt()
   >>>>                 xvu_create_prompt()
   >>>>   Static:
   >>>>                 xvu_prompt_yes()
   >>>>                 xvu_prompt_no()
   >>>>   Public:
   >>>>                 xvu_prompt_wait()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	

static int  xvu_prompt_done  = FALSE;
static int  xvu_prompt_sig;

static void xvu_prompt_yes PROTO((xvobject, kaddr, kaddr));
static void xvu_prompt_no PROTO((xvobject, kaddr, kaddr));

#define PROMPT_PIXMAP "$DESIGN/objects/library/xvutils/pixmaps/prompt.xpm"

/*-----------------------------------------------------------
|
|  Routine Name: xvu_prompt
|
|       Purpose: xvu_prompt produces standardized prompting messages for
|                X-based Khoros library routines and X based Khoros 
|                applications.  It should be set up as the prompt
|		 handler for all X-based promptings.
|
|         Input: yes_response  - prompt to put in the yes button
|		 no_response   - prompt to put in the no button
|		 default_val   - default value
|                message       - grammatically correct, clear explanation of
|                                the error that occurred
|        Output: none
|    Written By: Danielle Argiro and Steven Jorgensen
|          Date: Jul 29, 1992 18:39
|      Verified:
|  Side Effects:
| Modifications:
|
------------------------------------------------------------*/
int xvu_prompt(
   char *yes_response,
   char *no_response,
   int  default_val,
   char *message)
{
	int	status;

	if (message == NULL)
	{
	    errno = KNULL_PARAMETER;
	    kerror("xvutils", "xvu_prompt", 
		   "Passing NULL message to xvu_prompt");
	    return(default_val);
	}
	status = xvu_prompt_wait(message, "Please Respond", yes_response,
			       no_response);
	return(status);
}

/************************************************************
*
*  Routine Name: xvu_prompt_wait - pop up prompting object (2 choices); wait for response
*
*       Purpose: Creates and maps a pop-up prompting object which has 
*                two buttons representing two choices. The user must 
*                click on one of the buttons;  a status representing 
*                which of the two buttons was chosen will be returned 
*                to the application.  The user must click on one of the 
*                two buttons before control will be returned to the 
*                application program.
*
*         Input: prompting  - prompting string 
*                label      - short label for top of prompting object;
*			      NULL will produce default of "Choose One".
*		 yes_label  - label for button representing value of 1 (TRUE);
*                             NULL will produce default of "Yes".
*                no_label  - label for second representing value of 0 (FALSE);
*                            NULL will produce default of "No".
*        Output:
*	Returns:  1 (TRUE)  if user selected button with yes_label (Yes) 
*                 0 (FALSE) if user selected button with no_label (No) 
*
*  Restrictions: 
*    Written By: Danielle Argiro & Stephanie Hallett
*          Date: Jul 9, 1992 13:51
*      Verified:
*  Side Effects:
* Modifications: Converted from Khoros 1.0 (DA)
*
*************************************************************/

int xvu_prompt_wait(
   char *prompting,
   char *label,
   char *yes_label,
   char *no_label)
{
        knotify handler;
	xvu_prompt_done = FALSE;

	handler = kset_prompthandler(NULL);
	if (!(xvu_create_prompt(prompting, label, yes_label, no_label)))
	    return(FALSE);

	(void) kset_prompthandler(handler);
	while (!xvu_prompt_done)
	    xvw_process_event(); 

	return(xvu_prompt_sig);
}

/*-------------------------------------------------------------
|
|  Routine Name: xvu_create_prompt()
|
|       Purpose: Creates the prompting object. 
|
|         Input: prompt_string  - the prompting message 
|                prompt_label   - short label for top of object
|                                 NULL will give default of "Choose One"
|		 yes_labe   l   - label for acknowledgement button returning 1
|                                 NULL will give default of "Yes" 
|		 no_label       - label for acknowledgement button returning 0
|                                 NULL will give default of "Stop"
|
|        Output: Returns TRUE on success, FALSE on failure
|    Written By: Danielle Argiro 
|          Date: April 23, 1996
| Modifications:
|
-------------------------------------------------------------*/

#define TEXT_WIDTH  35
#define TEXT_HEIGHT 10

xvobject yes_button; 	/* button to select "yes" (1) */
xvobject no_button; 	/* button to select "no"  (0) */

int xvu_create_prompt(
   char *prompt_string,
   char *prompt_label,
   char *yes_label,
   char *no_label)
{
	xvobject back;	
	xvobject label;
	xvobject mesg;	
	xvobject pixmap;	
	xvobject separator;
	float    width, height, label_width, button_width;
	char     label_string[KLENGTH]; 
	char     yes_string[KLENGTH]; 
	char     no_string[KLENGTH];
	char     name[KLENGTH];

	xvu_initialize();

	/*
	 * Provide default labels if none provided
	 */
	if (prompt_label == NULL)
	    ksprintf(label_string, "Please Respond");
	else ksprintf(label_string, "%s", prompt_label);
	label_width = (float) kmax(kstrlen(label_string)+1, 5);

        if (yes_label == NULL)
            ksprintf(yes_string, "Yes");
        else ksprintf(yes_string, "%s", yes_label);
	button_width = (float) kmax((float) kstrlen(yes_string), 7.0);

        if (no_label == NULL)
            ksprintf(no_string, "No");
        else ksprintf(no_string, "%s", no_label);
	button_width = (float) kmax((float) button_width, 
				    (float) kstrlen(no_string));
	

	width =  (float) kmin(kmax3(TEXT_WIDTH, xvu_figure_width(prompt_string),
                  (int) (2.0*button_width + label_width)), 60);

	/* 
	 * create the backplane object 
	 */
	back = xvw_create(NULL, TRUE, TRUE, "prompt_dialog", 
		          ManagerWidgetClass);

	(void) ksprintf(name, "Prompt Dialog for %s", kprog_get_program());
	xvw_set_attribute(xvw_toplevel(back), XVW_SHELL_TITLE, name);

	/* 
         * Create "Yes" and "No" buttons at bottom
	 */
	no_button = xvw_create_button(back, "No");
	yes_button = xvw_create_button(back, "Yes");

	xvw_add_callback(no_button, XVW_BUTTON_SELECT, xvu_prompt_no, back);
	xvw_add_callback(yes_button, XVW_BUTTON_SELECT, xvu_prompt_yes, back);

        xvw_set_attributes(no_button,
                XVW_LABEL,         no_string,
                XVW_CHAR_WIDTH,    button_width,
                XVW_CHAR_HEIGHT,   1.25,
                XVW_ABOVE,         NULL,
                XVW_LEFT_OF,       NULL,
                XVW_RIGHT_OF,      yes_button,
                XVW_VERT_DIST,     8,
                NULL);

        xvw_set_attributes(yes_button,
                XVW_LABEL,        yes_string,
                XVW_CHAR_WIDTH,   button_width,
                XVW_CHAR_HEIGHT,  1.25,
                XVW_ABOVE,        NULL,
                XVW_LEFT_OF,      no_button,
                XVW_RIGHT_OF,     NULL,
                XVW_VERT_DIST,     8,
                NULL);

        /*
         *  Separator divides acknowledgement buttons from other stuff
         */
        separator = xvw_create_separator(back, "separator");
        xvw_set_attributes(separator,
                           XVW_ABOVE,     yes_button,
                           XVW_VERT_DIST, 8,
                           NULL);

	label = xvw_create_label(back, "label");
	pixmap = xvw_create_pixmap(back, "pixmap");

	/* 
	 * create text object w/ prompting mesg
	 */
	height = (float) xvu_figure_height(prompt_string, (int) width) + 4.0;
	if (height <= TEXT_HEIGHT)
	{
	   xvw_set_attributes(back,
                XVW_CHAR_WIDTH,   width,                
                XVW_CHAR_HEIGHT,  (double) TEXT_HEIGHT,
		NULL);

           mesg = xvw_create_text(back, "prompt_mesg");
           xvw_set_attributes(mesg,
		XVW_TEXT_MULTILINE,  TRUE,	          
		XVW_TEXT_STRING,     prompt_string,       
                XVW_TEXT_EDIT_TYPE,  KTEXT_READ,           
                XVW_TEXT_WRAP,       KTEXT_WRAP_WORD,       
		XVW_TACK_EDGE,	     KMANAGER_TACK_ALL,
		XVW_ABOVE,	     separator,
		XVW_BELOW,	     label,
		XVW_LEFT_OF,         NULL,
		XVW_RIGHT_OF,        NULL,
	        XVW_VERT_DIST,       16,
	        XVW_HORIZ_DIST,      8,
                NULL);
	}
	else
	{
	   xvw_set_attributes(back,
                XVW_CHAR_WIDTH,       width,             
                XVW_CHAR_HEIGHT,      kmin(height, 20.0),
		NULL);

           mesg = xvw_create_textdisplay(back, "prompt_mesg");
           xvw_set_attributes(mesg,
		XVW_TEXTDISPLAY_ADDTEXT, prompt_string,  
		XVW_TACK_EDGE,	         KMANAGER_TACK_ALL,
		XVW_ABOVE,	         separator,
		XVW_BELOW,	         pixmap,
		XVW_LEFT_OF,	         NULL,
		XVW_RIGHT_OF,	         NULL,
	        XVW_VERT_DIST,           8,
	        XVW_HORIZ_DIST,          8,
                NULL);
	}

	
        xvw_set_attributes(pixmap,
                XVW_PIXMAP_FILENAME, PROMPT_PIXMAP,
                XVW_BELOW,           NULL,
                XVW_RIGHT_OF,        NULL,
                XVW_VERT_DIST,       8,
                XVW_HORIZ_DIST,      8,
                NULL);

	/* 
	 * create the label object 
	 */
	xvw_set_attributes(label,
		XVW_LABEL,            label_string,   
		XVW_BELOW,            pixmap,  	   
		XVW_RIGHT_OF,         NULL,  	  
		XVW_LEFT_OF,          NULL,  	 
		XVW_ABOVE,            pixmap,  	 
		XVW_LABEL_JUSTIFY,    KLABEL_JUSTIFY_CENTER, 
		XVW_VERT_DIST,        8,
                NULL);

	xvw_add_protocol(back, "WM_DELETE_WINDOW",
                      (void (*)(xvobject, kaddr)) xvu_prompt_yes, back);
	xvw_place(back, yes_button, 0, 0);

	return(TRUE);
}

/*-------------------------------------------------------------
|
|  Routine Name: xvu_prompt_yes()
|
|       Purpose: Destroys prompting object when the user clicks on
|                the "Yes" button; returns status of 1.
|
|         Input: object      - the "Yes" acknowledgement button
|                client_data - stores the toplevel object
|
|        Output: none
|    Written By: Danielle Argiro
|          Date: April 23, 1996
| Modifications: 
|
-------------------------------------------------------------*/
/* ARGSUSED */
static void xvu_prompt_yes(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject toplevel = (xvobject) client_data;

	xvw_remove_callback(yes_button, XVW_BUTTON_SELECT, 
		            xvu_prompt_yes, client_data);
	xvw_remove_callback(no_button, XVW_BUTTON_SELECT, 
		            xvu_prompt_no, client_data);
	xvw_remove_protocol(toplevel, "WM_DELETE_WINDOW", 
                     (void (*)(xvobject, kaddr)) xvu_prompt_yes, toplevel);
	xvw_unmap(toplevel);
	xvw_destroy(toplevel);
	xvu_prompt_done = TRUE;
	xvu_prompt_sig = 1;

}

/*-------------------------------------------------------------
|
|  Routine Name: xvu_prompt_no()
|
|       Purpose: Destroys prompting object when the user clicks on
|                the "No" button; returns status of 0.
|
|         Input: object      - the "No" acknowledgement button
|                client_data - stores the toplevel object
|
|        Output: none
|    Written By: Danielle Argiro
|          Date: April 23, 1996
| Modifications: 
|
-------------------------------------------------------------*/
/* ARGSUSED */
static void xvu_prompt_no(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject toplevel  = (xvobject) client_data;

	xvw_remove_callback(yes_button, XVW_BUTTON_SELECT, 
		            xvu_prompt_yes, client_data);
	xvw_remove_callback(no_button, XVW_BUTTON_SELECT, 
		            xvu_prompt_no, client_data);
	xvw_unmap(toplevel);
	xvw_destroy(toplevel);
	xvu_prompt_done = TRUE;
	xvu_prompt_sig = 0;

}
