 /*
  * Khoros: $Id: query.c,v 1.21.2.1 1997/04/29 22:36:33 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: query.c,v 1.21.2.1 1997/04/29 22:36:33 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		        Query Dialog Box
   >>>>
   >>>>  Private:
   >>>>                 xvu_create_query()
   >>>>   Static:
   >>>>                 quit_query()
   >>>>                 cancel_query()
   >>>>   Public:
   >>>>                 xvu_query_wait()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	

static int  xvu_query_done  = FALSE;
static int  xvu_query_sig;

static void quit_query   PROTO((xvobject, kaddr, kaddr));
static void cancel_query PROTO((xvobject, kaddr, kaddr));

typedef struct _Query_Info {
        xvobject toplevel;
        xvobject back;
	xvobject ok_button;
	xvobject cancel_button;
	xvobject *text;
	char     **answers;
	int      num;
	} Query_Info;

#define QUERY_PIXMAP "$DESIGN/objects/library/xvutils/pixmaps/query.xpm"

/************************************************************
*
*  Routine Name: xvu_query_wait - pop up prompt widget; wait for response
*
*       Purpose: Creates a pop-up query object in order to prompt
*                more responses from the user; xvu_query_wait may
*                be used to obtain a set of strings, floats, integers,
*                or responses of mixed types from the user.
*
*		 Will not return control to application program until
*		 the user selects "Ok" or "Cancel".
*
*                The prompts[] and answers[] arrays must be of the same
*                size, where that size is specified as 'num_prompts'.
*
*                The prompts[] array must be completely filled out 
*                with 'num_prompts' strings giving appropriate prompt 
*                for each response desired.
*		
*		 The answers[] array must be dynamically allocated, and
*                each element of the string array should be initialized
*                to contain the string representation of the default (if any).
*                Strings should be passed in as NULL if no default is 
*                appropriate).  The defaults will appear in the text objects 
*                when the query object is popped up; when the user clicks on 
*                "Ok", the contents of the answers[] array will be freed if 
*                non-NULL, and the responses of the user will be substituted 
*                in their place.  
*
*                When the caller is prompting for float, int, or other 
*                non-string responses, it is expected that the caller will
*                convert the responses returned in the answers[] array to
*                their appropriately typed counterparts before use.
*
*		 The 'size' argument is provided to ensure that the text
*                objects are physically large enough to accomodate the
*                user's response.
*
*                
*         Input: top_label    - label to appear at top of query object
*		 prompts[]    - array of prompts
*                button       - label for acknowledgement button;
*                               NULL will give default of "Ok"
*		 answers[]    - array of strings in which to return the
*				user's responses, one for each of the prompts.
*                               Default values may be provided here if desired.
*		 num_prompts  - size of prompts[] and answers[] arrays
*		 size	      - size (in characters) to make each text 
*                               object for the users' responses
*
*        Output: answers[] - set of strings provided by the caller; 
*                            defaults will be returned when the user does not
*                            change a string;  NULL will be returned if the
*                            user did not provide a string and no default was
*                            provided.
*
*	Returns:       1 if user selected "Ok"
*                      0 if user selected "Cancel"
*
*
*  Restrictions: 
*
*    Written By: Danielle Argiro
*          Date: May 19, 1994
*      Verified:
*  Side Effects: The strings in the answers[] array will be freed if non-NULL,
*                and the responses of the user substituted.  If the user
*                leaves the default in place, that will be returned;  if the
*                user blanks out a string, NULL is returned. 
*
* Modifications:
*
*************************************************************/

int  xvu_query_wait(
   char *top_label,
   char *prompts[],
   char *button,
   char *answers[],
   int  num_prompts,
   int  size)
{
	xvobject toplevel;

	if (answers == NULL)
	{
	    kerror("xvutils", "xvu_query_wait",
		   "Invalid call to xvu_query_wait with answers array = NULL; "
		   "you must provide an allocated array of strings of size "
		   "num_prompts.  For no defaults, strings should be "
		   "initialized to \"\".");
	    return(FALSE);
	}
	xvu_query_done = FALSE;
	xvw_busy(NULL, TRUE);
	toplevel  = xvu_create_query(top_label, prompts, button, answers,
		                     num_prompts, size);
	if (toplevel == NULL)
	    return(FALSE);

	while(!xvu_query_done)
	    xvw_process_event();  

	return(xvu_query_sig);
}


/*-----------------------------------------------------------
|
|  Routine Name: xvu_create_query()
|
|       Purpose: Creates a query object. 
|         Input: top_label    - label to appear at top of query object
|                prompts[]    - array of prompts
|                button       - label for acknowledgement button;
|                               NULL will give default of "Ok"
|                answers[]    - array of strings in which to return the
|                               user's responses, one for each of the prompts.
|                               Default values may be provided here if desired.
|                num_prompts  - size of prompts[] and answers[] arrays
|                size         - size (in characters) to make each text
|                               object for the users' responses
|
|        Output: Returns the query object's toplevel 
|    Written By: Danielle Argiro
|          Date: Jul 9, 1992
| Modifications: Converted from Khoros 1.0 (DA)
|
-------------------------------------------------------------*/

xvobject xvu_create_query(
   char *top_label,
   char **prompts,
   char *button,
   char **answers,
   int  num_prompts,
   int  size)
{
	xvobject   offset;    
	xvobject   separator; 
	xvobject   label; 
	xvobject   pixmap; 
	int        i; 
	float      width, height, prompt_width = 0.0;
	char       name[KLENGTH];
	Query_Info *query_info;
	
	xvu_initialize();

	if (num_prompts < 1)
	{
	    errno = KINVALID_PARAMETER;
	    kerror("xvutils", "xvu_create_query", "'num_prompts' must be > 0");
	    return(FALSE);
	}

	query_info = (Query_Info *) (kcalloc ((size_t) 1, sizeof(Query_Info)));

	/*
	 *  allocate space for answers array, text & label arrays
	 */
	if (answers == NULL)
	    answers = (char **) kcalloc((size_t) 1, num_prompts*sizeof(char *));
	for (i = 0; i < num_prompts; i++)
	{
	    if (answers[i] == NULL)
		answers[i] = (char *) kcalloc((size_t) 1, 125*sizeof(char));
	}
	query_info->text  = (xvobject *) kmalloc(num_prompts*sizeof(xvobject)); 

	/*
	 * determine length of longest prompt
	 */
	prompt_width = 0.0;
	if (prompts != NULL)
	{
	    for (i = 0; i < num_prompts; i++)
	    {
	        if ((prompts[i] != NULL) && 
		    (kstrlen(prompts[i]) > prompt_width))
	 	     prompt_width = kstrlen(prompts[i]);
	    }
	    prompt_width += 1.0;
	}

	/*
	 * create the query box's toplevel object, and add it
	 * to the list of toplevels that is used with journal playback.
	 */
	query_info->back = xvw_create(NULL, TRUE, TRUE, "query_dialog", 
		          	ManagerWidgetClass);
        query_info->toplevel = xvw_toplevel(query_info->back);

	(void) ksprintf(name, "Query Dialog for %s", kprog_get_program());
	xvw_set_attribute(query_info->toplevel, XVW_SHELL_TITLE, name);


	/* 
	 * create Ok & Cancel buttons at bottom
	 */
	query_info->ok_button    = xvw_create_button(query_info->back, "ok");
	query_info->cancel_button = xvw_create_button(query_info->back, "cancel");

        xvw_set_attributes(query_info->ok_button,
		XVW_LABEL,            "Ok",   
		XVW_RIGHT_OF,          NULL,   
		XVW_LEFT_OF,           query_info->cancel_button,   
                XVW_ABOVE,             NULL,  
		XVW_CHAR_WIDTH,        7.0,  
                XVW_CHAR_HEIGHT,       1.25,  
                XVW_VERT_DIST,         8,   
                NULL);

        xvw_set_attributes(query_info->cancel_button,
		XVW_LABEL,            "Cancel", 
                XVW_LEFT_OF,           NULL,
		XVW_RIGHT_OF,          query_info->ok_button,   
                XVW_ABOVE,             NULL,     
		XVW_CHAR_WIDTH,        7.0, 
                XVW_CHAR_HEIGHT,       1.25,	  
                XVW_VERT_DIST,         8,   
                NULL);

	/*
	 *  Separator divides acknowledgement buttons from other stuff
	 */
	separator = xvw_create_separator(query_info->back, "separator");
	xvw_set_attributes(separator, 
			   XVW_ABOVE,     query_info->ok_button,
			   XVW_VERT_DIST, 8,
			   NULL);

        /*
         *  create the set of prompt & answer object pairs
         */
        offset = separator;
        for (i = num_prompts-1; i >=0; i--)
        {
            /*
             * create text object
             */
            ksprintf(name, "%d_text", i);
            query_info->text[i] = xvw_create_textinput(query_info->back, name);
            xvw_set_attributes(query_info->text[i],
                XVW_ABOVE,            offset,   
                XVW_LEFT_OF,          NULL,     
                XVW_HORIZ_DIST,       8,
		XVW_TACK_EDGE,        KMANAGER_TACK_HORIZ,
		XVW_TEXTINPUT_LABEL,  prompts ? prompts[i] : NULL,
	        XVW_TEXTINPUT_TEXT,   answers ? answers[i] : NULL,
                NULL);
           offset = query_info->text[i];
	   if (i == num_prompts-1)
	      xvw_set_attribute(query_info->text[i], XVW_VERT_DIST, 8);  
        }

	pixmap = xvw_create_pixmap(query_info->back, "pixmap");
	xvw_set_attributes(pixmap,
                XVW_PIXMAP_FILENAME, QUERY_PIXMAP,
                XVW_BELOW,           NULL,
                XVW_RIGHT_OF,        NULL,
                XVW_ABOVE,           offset,
                XVW_VERT_DIST,       8,
                XVW_HORIZ_DIST,      8,
                NULL);

	/*
	 *  
	 */
	label = xvw_create_label(query_info->back, "label");
        xvw_set_attributes(label,
		XVW_LABEL,            top_label,          
                XVW_BELOW,            pixmap,       
                XVW_ABOVE,            pixmap,       
                XVW_RIGHT_OF,         pixmap,             
                XVW_LEFT_OF,          NULL,             
                NULL);

	height  = (float) (num_prompts+4);
	width   = (float) kmax(size+prompt_width, kstrlen(top_label));
	xvw_set_attributes(query_info->back, 
			   XVW_CHAR_HEIGHT,  height, 
			   XVW_CHAR_WIDTH,   width,
			   NULL);

	query_info->answers = answers;
	query_info->num     = num_prompts;

	xvw_add_callback(query_info->ok_button, XVW_BUTTON_SELECT,
			quit_query, query_info);
	xvw_add_callback(query_info->cancel_button, XVW_BUTTON_SELECT,
			cancel_query, query_info);
	xvw_add_protocol(query_info->toplevel, "WM_DELETE_WINDOW", 
                        (void (*)(xvobject, kaddr)) cancel_query, query_info);

	xvw_place(query_info->toplevel, NULL, 0, 0);
	return(query_info->toplevel);
}

/*-----------------------------------------------------------
|
|  Routine Name: quit_query()
|
|       Purpose: Quits the query object
|         Input: object      - the "ok" object
|                client_data - not used
|        Output: none
|          Date: Jul 9, 1992
|    Written By: Danielle Argiro 
| Modifications: Converted from Khoros 1.0 (DA)
|
-------------------------------------------------------------*/
/* ARGSUSED */
static void quit_query(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	int  i;
	char *response;

	Query_Info *query_info = (Query_Info *) client_data;

	for (i = 0; i < query_info->num; i++)
	{
	    xvw_get_attribute(query_info->text[i], XVW_TEXTINPUT_TEXT, 
			      &response);
	    if (kstrlen(response) == 0)
	       query_info->answers[i] = NULL;
	    else ksprintf(query_info->answers[i], "%s", response);
	}
	xvw_remove_callback(query_info->ok_button, XVW_BUTTON_SELECT,
			    quit_query, query_info);
	xvw_remove_callback(query_info->cancel_button, XVW_BUTTON_SELECT,
			    cancel_query, query_info);

	kfree_and_NULL(query_info);
	xvu_query_done = TRUE;
        xvw_unmap(xvw_toplevel(object));
        xvw_destroy(xvw_toplevel(object));
	xvw_busy(NULL, FALSE);
	xvu_query_sig = 1;

}

/*-----------------------------------------------------------
|
|  Routine Name: cancel_query()
|
|       Purpose: Cancels the query object
|         Input: object      - the "cancel" object
|                client_data - not used
|                call_data   - not used
|        Output: none
|          Date: July 9, 1992
|    Written By: Danielle Argiro 
| Modifications: Converted from Khoros 1.0 (DA)
|
-------------------------------------------------------------*/
/* ARGSUSED */
static void cancel_query(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	Query_Info *query_info = (Query_Info *) client_data;

	xvw_remove_protocol(query_info->toplevel, "WM_DELETE_WINDOW", 
               (void (*)(xvobject, kaddr)) cancel_query, query_info->toplevel);
	xvw_remove_callback(query_info->ok_button, XVW_BUTTON_SELECT,
			    quit_query, query_info);
	xvw_remove_callback(query_info->cancel_button, XVW_BUTTON_SELECT,
			    cancel_query, query_info);
	xvu_query_done = TRUE;

	kfree_and_NULL(query_info);
        xvw_unmap(xvw_toplevel(object));
        xvw_destroy(xvw_toplevel(object));
	xvw_busy(NULL, FALSE);
	xvu_query_sig = 0;
}
