 /*
  * Khoros: $Id: save.c,v 1.9.2.1 1997/04/29 22:36:36 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: save.c,v 1.9.2.1 1997/04/29 22:36:36 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>                 Save Dialog Box
   >>>>  Private:
   >>>>                 xvu_create_save()
   >>>>   Static:
   >>>>                 xvu_save_yes()
   >>>>                 xvu_save_cancel()
   >>>>   Public:
   >>>>                 xvu_save_wait()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	

static int xvu_save_done  = FALSE;
static int xvu_save_signal;

static void xvu_save_accept    PROTO((xvobject, kaddr, kaddr));
static void xvu_save_discard   PROTO((xvobject, kaddr, kaddr));
static void xvu_save_cancel    PROTO((xvobject, kaddr, kaddr));


#define SAVE_PIXMAP "$DESIGN/objects/library/xvutils/pixmaps/save.xpm"

/*-----------------------------------------------------------------
|
|  Routine Name: xvu_save
|
|       Purpose: xvu_save produces standardized prompting for asking the
|		 user if they wish to save an application.  It will be set
|		 up as the save handler for ksave in all X-based Khoros
|		 routines by xvf_initialize().
|
|         Input: message   - grammatically correct, clear prompt asking
|                            the user if they wish to save the application.
|        Output:
|    Written By: Mark Young
|          Date: Apr 17, 1996
| Modifications:
|
--------------------------------------------------------------*/

int xvu_save(
   char *message)
{
	knotify handler;

	handler = kset_savehandler(NULL);
	xvu_save_wait(message, NULL, NULL, NULL, NULL);

	(void) kset_savehandler(handler);
        return(TRUE);
}

/************************************************************
*
*  Routine Name: xvu_save_wait - pop up a save message; wait for response
*
*       Purpose: Creates and maps a pop-up save object which looks like:
*.begin code
*                       -----------------------------------
*                       |                                 |
*                       |     Save Changes to {file}?     |
*                       |                                 |      
*                       - - - - - - - - - - - - - - - - - -
*                       | -------- ----------- ---------- |      
*                       | | Save | | Discard | | Cancel | |      
*                       | -------- ----------- ---------- |
*                       -----------------------------------
*.end code
*                This routine will block input to the application until
*                it is acknowledged by the user.  The user must click on 
*                "Save", "Discard" or "Cancel" before control will be 
*                returned to the application program.
*
*                Note that this utility does *not* actually save
*                any changes to the file, or write to the file specified
*                in any way.  It simply prompts the user and returns
*                a status depending on which button the user clicked.
*
*        Input:   string        - Optional string for first line of
*                                 label; if non-NULL, will replace
*                                 "Save Changes to"
*
*                 filename      - Name of file to which changes will be
*                                 saved (subsequently, by the application).
*
*		  save_label    - Optional label for button which returns 
*                                 value of 2; if non-NULL, will replace label
*                                 of "Save" on first button 
*
*		  discard_label - Optional label for button which returns 
*                                 value of 1; if non-NULL, will replace label
*                                 of "Discard" on first button 
*
*		  cancel_label - Optional label for button which returns 
*                                value of 0; if non-NULL, will replace label
*                                of "Cancel" on first button
*
*        Output:
*	Returns:  2 if user selected "Save" (first) button
*                 1 if user selected "Discard" (second)
*		  0 if user selected "Cancel" (third) button
*
*  Restrictions: 
*    Written By: Danielle Argiro  
*          Date: April 17, 1996
*      Verified:
*  Side Effects:
* Modifications: 
*
*************************************************************/

int xvu_save_wait(
   char *string,
   char *filename,
   char *save_label,
   char *discard_label,
   char *cancel_label)
{
	xvu_save_done = FALSE;

	if (!(xvu_create_save(string, filename, save_label, 
			      discard_label, cancel_label)))
	    return(FALSE);

	while(!xvu_save_done)
	    xvw_process_event(); 

	return(xvu_save_signal);
}


/*------------------------------------------------------------
|
|  Routine Name: xvu_create_save()
|
|       Purpose: This utility routine actually creates the save dialog box:
|
|                       -----------------------------------
|                       |                                 |
|                       |     Save Changes to {file}?     |
|                       |                                 |
|                       - - - - - - - - - - - - - - - - - -
|                       | -------- ----------- ---------- |
|                       | | Save | | Discard | | Cancel | |
|                       | -------- ----------- ---------- |
|                       -----------------------------------
|
|        Input:   string        - Optional string for first line of
|                                 label; if non-NULL, will replace
|                                 "Save Changes to"
|
|                 filename      - Name of file to which changes will be
|                                 saved (subsequently, by the application).
|
|                 save_label    - Optional label for button which returns
|                                 value of 2; if non-NULL, will replace label
|                                 of "Save" on first button
|
|                 discard_label - Optional label for button which returns
|                                 value of 1; if non-NULL, will replace label
|                                 of "Discard" on first button
|
|                 cancel_label - Optional label for button which returns
|                                value of 0; if non-NULL, will replace label
|                                of "Cancel" on first button
|
|        Output: Returns TRUE on success, FALSE on failure
|          Date: April 17, 1996
|    Written By: Danielle Argiro
| Modifications: 
|
-------------------------------------------------------------*/

int xvu_create_save(
   char *string,
   char *filename,
   char *save_label,
   char *discard_label,
   char *cancel_label)
{
	xvobject back;        /* backplane for entire save dialog box       */
	xvobject label;       /* label "Save Changes to"                    */
	xvobject separator;   /* separator to create visual seperation      */
	xvobject save;        /* "Save" button                              */
	xvobject discard;     /* "Discard" button                           */
	xvobject cancel;      /* "Cancel" button                            */
	xvobject pixmap;

	char     temp[KLENGTH];
	char     name[KLENGTH];

	xvu_initialize();

        /* create the backplane object */
	back = xvw_create(NULL, TRUE, TRUE, "save_dialog", ManagerWidgetClass);
	xvw_set_attributes(back,
			   XVW_CHAR_WIDTH,  32.0,
			   XVW_CHAR_HEIGHT, 5.0,
			   NULL);

	(void) ksprintf(name, "Save Dialog for %s", kprog_get_program());
	xvw_set_attribute(xvw_toplevel(back), XVW_SHELL_TITLE, name);

        /* set strings to appear as labels on buttons */
        if (save_label == NULL)
           ksprintf(temp, "Save");
        else ksprintf(temp, "%s", save_label);

        /* create the "Save" button (returns 2) */
        save = xvw_create_button(back, "save_button");

        /* create the "Discard" button (returns 1) */
        discard = xvw_create_button(back, "discard_button");

        /* create the "Cancel" button (returns 0) */
        cancel = xvw_create_button(back, "cancel_button");

        xvw_set_attributes(save,
                           XVW_LABEL,       temp,
                           XVW_CHAR_WIDTH,  7.0,
                           XVW_CHAR_HEIGHT, 1.25,
                           XVW_RIGHT_OF,    NULL,
                           XVW_LEFT_OF,     discard,
                           XVW_ABOVE,       NULL,
                           XVW_VERT_DIST,   8,
                           NULL);
        xvw_add_callback(save, XVW_BUTTON_SELECT, xvu_save_accept, back);

        if (discard_label == NULL)
           ksprintf(temp, "Discard");
        else ksprintf(temp, "%s", discard_label);

        xvw_set_attributes(discard,
                           XVW_LABEL,       temp,
                           XVW_CHAR_WIDTH,  7.0,
                           XVW_CHAR_HEIGHT, 1.25,
                           XVW_RIGHT_OF,    NULL,
                           XVW_LEFT_OF,     NULL,
                           XVW_ABOVE,       NULL,
                           XVW_VERT_DIST,   8,
                           NULL);
        xvw_add_callback(discard, XVW_BUTTON_SELECT, xvu_save_discard, back);

        if (cancel_label == NULL)
           ksprintf(temp, "Cancel");
        else ksprintf(temp, "%s", cancel_label);

        xvw_set_attributes(cancel,
                           XVW_LABEL,       temp,
                           XVW_CHAR_WIDTH,  7.0,
                           XVW_CHAR_HEIGHT, 1.25,
                           XVW_LEFT_OF,     NULL,
                           XVW_RIGHT_OF,    discard,
                           XVW_ABOVE,       NULL,
                           XVW_VERT_DIST,   8,
                           NULL);
        xvw_add_callback(cancel, XVW_BUTTON_SELECT, xvu_save_cancel, back);

	separator = xvw_create_separator(back, "separator");
	xvw_set_attributes(separator, 
			   XVW_ABOVE,	  cancel,
			   XVW_VERT_DIST, 8,
			   NULL);

	/* set strings to appear as labels on the save object */
	if ((string == NULL) && (filename == NULL))
	    ksprintf(temp, "Save Changes?");
	else if ((string == NULL) && (filename != NULL))
 	    ksprintf(temp, "Save Changes to %s?", filename);
	else if ((string != NULL) && (filename == NULL))
 	    ksprintf(temp, "%s?", string);
	else if ((string != NULL) && (filename != NULL))
 	    ksprintf(temp, "%s %s?", string, filename);

        pixmap = xvw_create_pixmap(back, "pixmap");
        xvw_set_attributes(pixmap,
                XVW_PIXMAP_FILENAME, SAVE_PIXMAP,
                XVW_BELOW,           NULL,
                XVW_RIGHT_OF,        NULL,
                XVW_ABOVE,           separator,
                XVW_VERT_DIST,       8,
                XVW_HORIZ_DIST,      8,
                NULL);

	/* create the first label */
        label = xvw_create_label(back, "label");
        xvw_set_attributes(label,
		           XVW_LABEL,                 temp,  
                           XVW_BELOW,                 NULL,   
                           XVW_ABOVE,                 separator,   
                           XVW_LEFT_OF,               NULL,   
                           XVW_RIGHT_OF,              NULL,   
                           XVW_LABEL_JUSTIFY,         KLABEL_JUSTIFY_CENTER,   
			   XVW_TACK_EDGE,	      KMANAGER_TACK_ALL,
			   XVW_VERT_DIST,	      8,
                           NULL);

        /* Selecting "delete" with window manager will equal hitting "Cancel" */
	xvw_add_protocol(back, "WM_DELETE_WINDOW",
                         (void (*)(xvobject, kaddr)) xvu_save_cancel, back);

	/* Map and place the save dialog box */
	xvw_place(back, save, 0, 0);

	/* good job, eh? */
	return(TRUE);
}

/*------------------------------------------------------------
|
|  Routine Name: xvu_save_accept()
|
|       Purpose: Destroys save dialog box when the user clicks on
|                the "Save" button; returns status of 2.
|
|         Input: object      - the "Save" acknowledgement button
|                client_data - passes in the backplane
|                call_data   - unused
|        Output: 
|          Date: April 17, 1996         
|    Written By: Danielle Argiro
| Modifications: 
|
-------------------------------------------------------------*/
/* ARGSUSED */
static void xvu_save_accept(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject back = (xvobject) client_data;

	xvw_unmap(back);
	xvw_destroy(back);
	xvu_save_done = TRUE;
	xvu_save_signal = 2;
}

/*------------------------------------------------------------
|
|  Routine Name: xvu_save_discard()
|
|       Purpose: Destroys save dialog box when the user clicks on
|                the "Discard" button; returns status of 1.
|
|         Input: object      - the "Discard" acknowledgement button
|                client_data - passes in the backplane
|                call_data   - unused
|        Output:
|          Date: April 17, 1996
|    Written By: Danielle Argiro
| Modifications:
|
-------------------------------------------------------------*/
/* ARGSUSED */
static void xvu_save_discard(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
        xvobject back = (xvobject) client_data;

        xvw_unmap(back);
        xvw_destroy(back);
        xvu_save_done = TRUE;
        xvu_save_signal = 1;
}


/*------------------------------------------------------------
|
|  Routine Name: xvu_save_cancel()
|
|       Purpose: Destroys save dialog box when the user clicks on
|                the "Cancel" button; returns status of 0.
|
|         Input: object      - the "Cancel" acknowledgement button
|                client_data - passes in the backplane
|                call_data   - unused
|        Output: 
|          Date: April 17, 1996 
|    Written By: Danielle Argiro
| Modifications: 
|
-------------------------------------------------------------*/
/* ARGSUSED */
static void xvu_save_cancel(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject back = (xvobject) client_data;

	xvw_unmap(back);
	xvw_destroy(back);
	xvu_save_done = TRUE;
	xvu_save_signal = 0;
}

