 /*
  * Khoros: $Id: converter.c,v 1.14.2.3 1997/05/24 19:05:43 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: converter.c,v 1.14.2.3 1997/05/24 19:05:43 mirage Exp $";
#endif

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Conversion routines for xvwidgets
   >>>>		
   >>>>  Private:
   >>>>                 xvw_init_converters()
   >>>>   Static:
   >>>>                 cvtFontToString()
   >>>>                 cvtStringToFont()
   >>>>                 cvtFontStructToString()
   >>>>                 cvtStringToFontStruct()
   >>>>
   >>>>                 cvtPixelToString()
   >>>>                 cvtStringToPixel()
   >>>>
   >>>>                 cvtPixmapToFilename()
   >>>>                 cvtFilenameToPixmap()
   >>>>                 cvtPixmapMaskToFilename()
   >>>>                 cvtFilenameToPixmapMask()
   >>>>
   >>>>                 cvtStringToCursor()
   >>>>
   >>>>                 cvtBooleanToInt()
   >>>>                 cvtBooleanToShort()
   >>>>                 cvtBooleanToString()
   >>>>                 cvtShortToString()
   >>>>                 cvtIntToString()
   >>>>                 cvtFloatToString()
   >>>>                 cvtDoubleToString()
   >>>>                 cvtStringToDouble()
   >>>>                 cvtIntToShort()
   >>>>                 cvtShortToInt()
   >>>>                 cvtIntToUShort()
   >>>>                 cvtUShortToInt()
   >>>>                 cvtDoubleToFloat()
   >>>>                 cvtFloatToDouble()
   >>>>                 cvtObjectToWidget()
   >>>>                 cvtWidgetToObject()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <X11/Xatom.h>
#include <kdatafmt/xpm.h>

#define cvtDone(type, value, toVal) \
{                                                       \
    if (toVal->addr != NULL) {                          \
	if (toVal->size < sizeof(type)) {               \
	    toVal->size = sizeof(type);                 \
	    return(FALSE);                              \
	}                                               \
	*((type*)(toVal->addr)) = (type) (value);       \
    }                                                   \
    else {                                              \
	static type static_val;                         \
	static_val = (type) (value);                    \
	toVal->addr = (char *)&static_val;              \
    }                                                   \
    toVal->size = sizeof(type);                         \
}

static XtConvertArgRec convertArgs[] = {
    {XtWidgetBaseOffset, (XtPointer)XtOffsetOf(WidgetRec, core.self),
     sizeof(Widget)},
    {XtWidgetBaseOffset, (XtPointer)XtOffsetOf(WidgetRec, core.screen),
     sizeof(Screen *)},
    {XtWidgetBaseOffset, (XtPointer)XtOffsetOf(WidgetRec, core.colormap),
     sizeof(Colormap)},
};

/*-------------------------------------------------------------------*
|
|   Font Conversion
|
--------------------------------------------------------------------*/

static klist *fontlist = NULL;

/*-----------------------------------------------------------
|  Routine Name: cvtFontToString
|       Purpose: Convert a font to a string.
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtFontToString(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	klist	    *temp;
	Atom	    name, value;
	XFontStruct *fontstruct;
	char	    *fontname = NULL;
	Font *font = (Font *) fromVal->addr;


	if ((temp = klist_locate(fontlist, (kaddr) *font)) != NULL)
	   fontname = ktoken_to_string((ktoken) temp->client_data);
	else if ((fontstruct = XQueryFont(display, *font)) != NULL)
	{
	   if ((name = XInternAtom(display, "FONTNAME", FALSE)) != None &&
	        XGetFontProperty(fontstruct, name, &value) == TRUE)
	           fontname = XGetAtomName(display, value);
	}

	if (fontname == NULL)
	{
	   cvtDone(String, "Unknown font", toVal);
	   return(FALSE);
	}
	cvtDone(String, fontname, toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtStringToFont
|       Purpose: Convert a string to a font.
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtStringToFont(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	ktoken	    token;
	klist	    *list;
	XFontStruct *fontstruct = NULL;
	char	    fontname[KLENGTH], *string = (String) fromVal->addr;


        if (!kstring_cleanup(string, fontname))
           return(FALSE);

	/*
	 *  Go ahead and see if the fontname is already cached in our
	 *  static fontlist.
	 */
	if ((token = kstring_to_token(fontname)) == NULL)
	   return(FALSE);

	if ((list = klist_locate_clientdata(fontlist, (kaddr) token)) != NULL)
	{
	   fontstruct = (XFontStruct *) klist_identifier(list);
	   cvtDone(Font, fontstruct->fid, toVal);
	   return(TRUE);
	}

        if ((fontstruct = XLoadQueryFont(display, fontname)) == NULL)
	{
	   XrmName xrm_name[2];
	   XrmClass xrm_class[2];
	   XrmRepresentation rep_type;
	   XrmValue value;

	   /*
	    *  Error on default font...
	    */
	   if (kstrcmp(fontname,"XtDefaultFont") != 0)
	   {
	      kerror("xvwidgets", "cvtStringToFont", "Error! Cannot load font "
		     "'%s'.  Loading default font 'xtDefaultFont'", fontname);
	   }

	   /*
	    *  Try to find default font
	    */
	   kstrcpy(fontname, "XtDefaultFont");
	   xrm_name[0] = XrmStringToName ("xtDefaultFont");
	   xrm_name[1] = (XrmClass) NULL;
	   xrm_class[0] = XrmStringToClass ("XtDefaultFont");
	   xrm_class[1] = (XrmClass) NULL;
	   if (XrmQGetResource(XtDatabase(display), xrm_name, xrm_class,
                            &rep_type, &value))
	   {
              if (rep_type == XtQstring && (fontstruct = XLoadQueryFont(display,
			(char *) value.addr)) == None)
		 kinfo(KSTANDARD,"Unable to load default font 'XtDefaultFont'");
	      else if (rep_type == XtQfont)
		fontstruct = XQueryFont(display, *((Font *) value.addr));
	      else if (rep_type == XtQfontStruct)
		fontstruct = ((XFontStruct*)value.addr);
           }
	   else
	      kinfo(KXVLIB, "cvtStringToFont: couldn't find 'xtDefaultFont' "
			    "to be loaded.");

	   kstrcpy(fontname, "-*-*-*-R-*-*-*-120-*-*-*-*-ISO8859-1");
	   if (fontstruct == NULL && (fontstruct = XLoadQueryFont(display,
		fontname)) == NULL)
	   {
	      kinfo(KSTANDARD, "cvtStringToFont: could not find "
				"'XtDefaultFont' to be loaded.");
	   }
	   fontlist = klist_insert(fontlist, (kaddr) fontstruct, (kaddr) token,
				KLIST_HEAD, TRUE);
	   token = kstring_to_token(fontname);
        }

	if (fontstruct != NULL)
	   fontlist = klist_insert(fontlist, (kaddr) fontstruct, (kaddr) token,
				KLIST_HEAD, TRUE);
	else
	{
	   kerror("xvwidgets", "cvtStringToFont", "Error!  Unable to find any "
		  "reasonable font to be loaded.  Unpredictable behavior may "
		  "occur!");
	   return(FALSE);
	}
	cvtDone(Font, fontstruct->fid, toVal);
	return(TRUE);
}

/*-------------------------------------------------------------------*
|
|   FontStruct Conversion
|
--------------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: cvtFontStructToString
|       Purpose: Convert a font structure into a string.
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtFontStructToString(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	klist	    *temp;
	Atom	    name, value;
	char	    *fontname = NULL;
	XFontStruct *fontstruct = *((XFontStruct **) fromVal->addr);


	if (fontstruct == NULL)
	   return(FALSE);

	if ((temp = klist_locate(fontlist, (kaddr) fontstruct)) != NULL)
	   fontname = ktoken_to_string((ktoken) klist_clientdata(temp));
	else if ((name = XInternAtom(display, "FONTNAME", FALSE)) != None &&
	        XGetFontProperty(fontstruct, name, &value) == TRUE)
	   fontname = XGetAtomName(display, value);

	if (fontname == NULL)
	{
	   cvtDone(String, "Unknown font", toVal);
	   return(FALSE);
	}
	cvtDone(String, fontname, toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtStringToFontStruct - 
|       Purpose: Convert a string into a font structure.  
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtStringToFontStruct(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	ktoken      token;
	klist	    *list;
	XFontStruct *fontstruct = NULL;
	char	    fontname[KLENGTH], *string = (String) fromVal->addr;


        if (!kstring_cleanup(string, fontname))
           return(FALSE);

	/*
	 *  Go ahead and see if the fontname is already cached in our
	 *  static fontlist.
	 */
	if ((token = kstring_to_token(fontname)) == NULL)
	   return(FALSE);

	if ((list = klist_locate_clientdata(fontlist, (kaddr) token)) != NULL)
	{
	   cvtDone(XFontStruct *, klist_identifier(list), toVal);
	   return(TRUE);
	}

	if ((fontstruct = XLoadQueryFont(display, fontname)) == NULL)
	{
	   XrmName xrm_name[2];
	   XrmClass xrm_class[2];
	   XrmRepresentation rep_type;
	   XrmValue value;

	   /*
	    *  Error on default font...
	    */
	   if (kstrcmp(fontname, "XtDefaultFont"))
	   {
	      kerror("xvwidgets", "cvtStringToFontStruct", "Error! Cannot "
		     "load font '%s'.  Loading default font 'xtDefaultFont'",
		     fontname);
	   }

	   /*
	    *  Try to find default font
	    */
	   kstrcpy(fontname, "xtDefaultFont");
	   xrm_name[0] = XrmStringToName ("xtDefaultFont");
	   xrm_name[1] = (XrmClass) NULL;
	   xrm_class[0] = XrmStringToClass ("XtDefaultFont");
	   xrm_class[1] = (XrmClass) NULL;
	   if (XrmQGetResource(XtDatabase(display), xrm_name, xrm_class,
                            &rep_type, &value))
	   {
              if (rep_type == XtQstring && (fontstruct = XLoadQueryFont(display,
			(char *) value.addr)) == NULL)
		 kinfo(KSTANDARD,"Unable to load default font 'xtDefaultFont'");
	      else if (rep_type == XtQfont)
                fontstruct = XQueryFont(display, *((Font *) value.addr));
              else if (rep_type == XtQfontStruct)
                fontstruct = ((XFontStruct*)value.addr);
           }
	   else
	      kinfo(KXVLIB, "cvtStringToFontStruct: could not find "
			    "'XtDefaultFont' to be loaded.");

	   kstrcpy(fontname, "-*-*-*-R-*-*-*-120-*-*-*-*-ISO8859-1");
	   if (fontstruct == NULL && (fontstruct = XLoadQueryFont(display,
		fontname)) == NULL)
	   {
	      kinfo(KSTANDARD, "cvtStringToFontStruct: could not find "
			       "'XtDefaultFont' to be loaded.");
	   }
	   fontlist = klist_insert(fontlist, (kaddr) fontstruct, (kaddr) token,
				KLIST_HEAD, TRUE);
	   token = kstring_to_token(fontname);
        }

	if (fontstruct != NULL)
	   fontlist = klist_insert(fontlist, (kaddr) fontstruct, (kaddr) token,
				KLIST_HEAD, TRUE);
	else
	{
	   kerror("xvwidgets", "cvtStringToFontStruct", "Error!  Unable "
		  "to find any reasonable font to be loaded.  Unpredictable "
		  "behavior may occur!");
	   return(FALSE);
	}
	cvtDone(XFontStruct *, fontstruct, toVal);
	return(TRUE);
}


/*-------------------------------------------------------------------*
|
|   Color conversion
|
--------------------------------------------------------------------*/
 
klist *colorlist = NULL;

/*-----------------------------------------------------------
|  Routine Name: cvtPixelToString
|       Purpose: Convert a pixel to a string. 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtPixelToString(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	klist	 *temp;
	Widget   widget;
	XColor   xcolor;
	Screen   *screen;
	Colormap colormap;
	int	 red, green, blue;
	char	 *string = NULL, color[KLENGTH];


	if (*num_args != 3)
	{
	   kinfo(KSTANDARD, "Pixel to kstring conversion needs the standard "
			    "converter arguments");
	   return(FALSE);
	}
	widget   = *((Widget *) args[0].addr);
	screen   = *((Screen **)   args[1].addr);
	colormap = *((Colormap *)  args[2].addr);
	xcolor.pixel = (*((Pixel *) fromVal->addr));

	if (colormap == DefaultColormap(display, XScreenNumberOfScreen(screen)))
	{
	   klist *temp;

	   if ((temp = klist_locate(colorlist, (kaddr) xcolor.pixel)) != NULL)
	      string = ktoken_to_string((ktoken) temp->client_data);
	   else if (xcolor.pixel == WhitePixelOfScreen(screen))
	      string = XtDefaultBackground;
	   else if (xcolor.pixel == BlackPixelOfScreen(screen))
	      string = XtDefaultForeground;
	}

	if (string == NULL)
	{
	   XQueryColor(display, colormap, &xcolor);

	   red   = xcolor.red >> 8;
	   green = xcolor.green >> 8;
	   blue  = xcolor.blue >> 8;
	   (void) ksprintf(color,"#%02x%02x%02x", red, green, blue);
	   string = kstrdup(color);
	}
	cvtDone(String, string, toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtStringToPixel
|       Purpose: Convert a string to a pixel.
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtStringToPixel(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	Pixel    pixel;
	Widget	 widget;
	Screen   *screen;
	Colormap colormap;
	char	 colorname[KLENGTH], *string = (String) fromVal->addr;


	if (*num_args != 3)
	{
	   kinfo(KSTANDARD, "String to Pixel conversion needs the standard "
			    "converter arguments");
	   return(FALSE);
	}
	widget   = *((Widget *) args[0].addr);
	screen   = *((Screen **)   args[1].addr);
	colormap = *((Colormap *)  args[2].addr);

	if (!kstring_cleanup(string, colorname))
	   return(FALSE);

	if ((pixel = xvw_widget_color(widget, colorname)) == (Pixel) ~0L)
	{
	   cvtDone(Pixel, pixel, toVal);
	   return(FALSE);
	}

	if (colormap == DefaultColormap(display, XScreenNumberOfScreen(screen)))
	{
	   colorlist = klist_insert(colorlist, (kaddr) pixel,
				(kaddr) kstring_to_token(colorname),
				KLIST_HEAD, TRUE);
	}
	cvtDone(Pixel, pixel, toVal);
	return(TRUE);
}


/*-------------------------------------------------------------------*
|
|   Pixmap and Mask conversion
|
--------------------------------------------------------------------*/
 
static klist *masklist = NULL;
static klist *pixmaplist = NULL;
static khash *colormap_cache = NULL;

static int cachePixmap(
   char   *name,
   Widget widget,
   Pixmap pixmap,
   Pixmap mask)
{
        Pixmap entry[2];
	khash *pixmap_cache = NULL;
        ktoken token = kstring_to_token(name);
        unsigned long colormap = widget->core.colormap;

	masklist   = klist_insert(masklist, (kaddr) mask, (kaddr) token,
                                KLIST_HEAD, TRUE);
	pixmaplist = klist_insert(pixmaplist, (kaddr) pixmap, (kaddr) token,
                                KLIST_HEAD, TRUE);

	/*
	 *  Initialize the pixmap colormap hash...
	 */
	if (colormap_cache == NULL)
	   colormap_cache = khash_init(KULONG, 10);

	/*
	 *  Locate the existing colormap hash.  If an entry doesn't exist
	 *  then create one.
	 */
        if (khash_clientdata(colormap_cache, colormap, &pixmap_cache) == FALSE)
	{
	   pixmap_cache = khash_init(KSTRUCT, 100);
	   khash_add(colormap_cache, colormap, pixmap_cache);
	}

	/*
	 *  Add the pixmap and mask entry to the colormap hash
	 */
	entry[0] = pixmap; entry[1] = mask;
	khash_add(pixmap_cache, token, kdupalloc(entry, sizeof(entry)));
	return(FALSE);
}

static int lookupPixmap(
   char   *name,
   Widget widget,
   Pixmap *pixmap,
   Pixmap *mask)
{
	Pixmap *entry = NULL;
	khash *pixmap_cache = NULL;
        ktoken token = kstring_to_token(name);
	unsigned long colormap = widget->core.colormap;

	/*
	 *  Lookup the pixmap, but only the one associated with the
	 *  widget's colormap.  This way we don't return a pixmap which
	 *  is not valid for the desired visual or colormap.
	 */
	if (khash_clientdata(colormap_cache, colormap,&pixmap_cache) == FALSE ||
	    khash_clientdata(pixmap_cache, token, &entry) == FALSE)
	{
	   if (pixmap) *pixmap = NULL;
	   if (mask)   *mask   = NULL;
	   return(FALSE);
	}
	if (pixmap) *pixmap = entry[0];
	if (mask)   *mask   = entry[1];
	return(TRUE);
}

/*
 *  Reverse name lookup for the pixmap.
 */
static char *pixmapName(
   Pixmap pixmap)
{
	klist  *temp;
	char   *string;

	if ((temp = klist_locate(pixmaplist, (kaddr) pixmap)) == NULL)
	   return(NULL);

        string = ktoken_to_string((ktoken) temp->client_data);
	return(string);
}

/*
 *  Reverse name lookup for the mask pixmap.
 */
static char *maskName(
   Pixmap mask)
{
	klist  *temp;
	char   *string;

	if ((temp = klist_locate(masklist, (kaddr) mask)) == NULL)
	   return(NULL);

        string = ktoken_to_string((ktoken) temp->client_data);
	return(string);
}

/*-----------------------------------------------------------
|  Routine Name: cvtPixmapToFilename
|       Purpose: Convert a pixmap to a string. 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtPixmapToFilename(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	Pixmap pixmap = (*((Pixmap *) fromVal->addr));

	char *string;

	if (pixmap == None)
	   string = "None";
	else if (pixmap == ParentRelative)
	   string = "ParentRelative";
	else if (pixmap == XtUnspecifiedPixmap)
	   string = "Unspecified";
	else if ((string = pixmapName(pixmap)) == NULL)
	{
	   cvtDone(String, NULL,  toVal);
	   return(FALSE);
	}
	cvtDone(String, string,  toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtFilenameToPixmap
|       Purpose: Convert a string to a pixmap.
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtFilenameToPixmap(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	klist    *list;
	Widget   widget;
	Screen   *screen;
	Window   rootwindow;
	unsigned char *data;
	Pixmap   bitmap, pixmap, mask = None;
	char	 pixmapfile[KLENGTH], **pixmapdef;
	char	 *string = (String) fromVal->addr;
	int      i, x, y, screennum, need_mask = FALSE;
	unsigned int width, height, border_width, depth;

	Pixel	 fg, bg;
	XColor	 xcolor;
	xpm	 *image;
	Visual   *visual;
	GC	 gc, gc_mask;
	XImage   *ximage, *xmask;

	if (*num_args != 3)
	{
	   kinfo(KSTANDARD, "String to Pixmap conversion needs the standard "
			    "converter arguments");
	   cvtDone(Pixmap, None, toVal);
	   return(FALSE);
	}
	widget     = *((Widget *) args[0].addr);
	screen     = *((Screen **) args[1].addr);
	screennum  = XScreenNumberOfScreen(screen);
	fg	   = BlackPixelOfScreen(screen);
	bg	   = WhitePixelOfScreen(screen);
	visual     = DefaultVisual(display, screennum);
	rootwindow = RootWindow(display, screennum);

	if (kstrcmp(string, "None") == 0)
	{
	   cvtDone(Pixmap, None, toVal);
	   return(TRUE);
	}
	else if (kstrcmp(string, "Unspecified") == 0)
	{
	   cvtDone(Pixmap, XtUnspecifiedPixmap, toVal);
	   return(TRUE);
	}
	else if (kstrcmp(string, "ParentRelative") == 0)
	{
	   cvtDone(Pixmap, ParentRelative, toVal);
	   return(TRUE);
	}
	else if (lookupPixmap(string, widget, &pixmap, NULL) == TRUE)
	{
	   cvtDone(Pixmap, pixmap, toVal);
	   return(TRUE);
	}

	/*
	 *  Build the pixmap from scratch...
	 */
	bg = widget->core.background_pixel;
	if ((pixmapdef = xvw_locate_pixmap(string)) == NULL ||
	    (image = xpm_parse(pixmapdef)) == NULL)
	{
	   image = xpm_read(string);
	}

	if (image != NULL)
	{
	   data  = (unsigned char *) image->data;
	   width = image->width; height = image->height;

	   pixmap = XCreatePixmap(display, rootwindow, width, height,
			widget->core.depth);
	   ximage = XCreateImage(display, visual, widget->core.depth, ZPixmap,
			0, NULL, width, height, 8, 0);
	   ximage->data = (char *) kmalloc((size_t) (ximage->bytes_per_line *
					height));

	   gc = XCreateGC(display, pixmap, (unsigned long) 0, NULL);  
	   for (i = 0; i < width*height; i++, data++)
	   {
	      x = i % width; y = i / width;
	      if (image->maps[*data].flags == 0xff)
	      {
		 need_mask = TRUE;
		 image->maps[*data].flags = 1;
		 image->maps[*data].pixel = bg;
	      }
	      else if (!image->maps[*data].flags)
	      {
		 xcolor.red   = image->maps[*data].red;
		 xcolor.green = image->maps[*data].green;
		 xcolor.blue  = image->maps[*data].blue;
		 xcolor.flags = DoRed | DoGreen | DoBlue;
		 XAllocColor(display, widget->core.colormap, &xcolor);
		 image->maps[*data].flags = (int) xcolor.flags;
		 image->maps[*data].pixel = xcolor.pixel;
	      }
	      XPutPixel(ximage, x, y, image->maps[*data].pixel);
	   }
	   XPutImage(display, pixmap, gc, ximage, 0, 0, 0, 0, width, height);
	   XDestroyImage(ximage); XFreeGC(display, gc);

	   /*
	    *  Mask stuff
	    */
	   if (need_mask == TRUE)
	   {
	      mask  = XCreatePixmap(display, rootwindow, width, height, 1);
	      xmask = XCreateImage(display, visual, 1, XYPixmap, 0, NULL,
				   width, height, 8, 0);
	      xmask->data = (char *) kcalloc((size_t) 1, (size_t) 
				(xmask->bytes_per_line * height));
	      gc_mask = XCreateGC(display, mask, (unsigned long) 0, NULL);  
	      data = (unsigned char *) image->data;

	      for (i = 0; i < width*height; i++, data++)
	      {
	         if (image->maps[*data].flags != 1)
	            XPutPixel(xmask, (int) (i%width), (int) (i/width),
			      (unsigned long) 1);
	      }
	      XPutImage(display, mask, gc_mask, xmask,  0,0,0,0, width, height);
	      XDestroyImage(xmask); XFreeGC(display, gc_mask);
	   }
	   xpm_free(image);
	}
	else if (kfullpath(string, NULL, pixmapfile) &&
		 (bitmap = XmuLocateBitmapFile(screen, pixmapfile, NULL, 0,
			NULL, NULL, NULL, NULL)) != None)
	{
	   XGetGeometry(display, bitmap, &rootwindow, &x, &y, 
		     &width, &height, &border_width, &depth);
	   pixmap = XmuCreatePixmapFromBitmap (display,
			rootwindow, bitmap, width, height, 
			widget->core.depth, fg, bg);
	}
	else
	{
	   kerror("xvwidgets", "cvtFilenameToPixmap", "Error!  Unable to "
		  "locate the pixmap file '%s'.", string);
	   mask = pixmap = None;
	}
	cachePixmap(string, widget, pixmap, mask);
	cvtDone(Pixmap, pixmap, toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtPixmapMaskToFilename
|       Purpose: Convert a pixmap mask to a filename. 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jul 19, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtPixmapMaskToFilename(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	Pixmap mask = (*((Pixmap *) fromVal->addr));

	char *string;

	if (mask == None)
	   string = "None";
	else if (mask == ParentRelative)
	   string = "ParentRelative";
	else if (mask == XtUnspecifiedPixmap)
	   string = "Unspecified";
	else if ((string = maskName(mask)) == NULL)
	{
	   cvtDone(String, NULL, toVal);
	   return(FALSE);
	}
	cvtDone(String, string,  toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtFilenameToPixmapMask
|       Purpose: Convert a filename to a pixmap mask.
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jul 19, 1994
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtFilenameToPixmapMask(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	char *string = (String) fromVal->addr;

	Pixmap mask;
	Widget widget;

        if (*num_args != 3)
        {
           kinfo(KSTANDARD, "String to PixmapMask conversion needs the "
			    "standard converter arguments");
           cvtDone(Pixmap, None, toVal);
           return(FALSE);
        }
	widget = *((Widget *) args[0].addr);

	if (kstrcmp(string, "None") == 0)
	   mask = None;
	else if (kstrcmp(string, "ParentRelative") == 0)
	   mask = ParentRelative;
	else if (kstrcmp(string, "Unspecified") == 0)
	   mask = XtUnspecifiedPixmap;
	else if (lookupPixmap(string, widget, NULL, &mask) == FALSE)
	{
	   cvtDone(Pixmap, None, toVal);
	   return(FALSE);
	}
        cvtDone(Pixmap, mask, toVal);
        return(TRUE);
}

/*-------------------------------------------------------------------*
|
|   Cursor conversion
|
--------------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: cvtStringToCursor
|       Purpose: Convert a kstring to a Cursor. 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jul 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtStringToCursor(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	Cardinal num;
	Cursor   cursor;
        char     filename[KLENGTH], temp[KLENGTH];
	String	 string = (String) fromVal->addr;
	XrmValue ret_val;


	if (kfullpath((char *) string, NULL, filename) != NULL)
	{
	   if (kaccess(filename, R_OK) != 0 &&
	       kstrcmp(kdirname(filename, temp), ".") != 0)
	   {
	      kstrcpy(filename, "cross");
	   }
	   fromVal->addr = (char *) filename;
	}
	num = 1;
	ret_val.size = 0; ret_val.addr = NULL;
	XmuCvtStringToCursor(&args[1], &num, fromVal, &ret_val);

	cursor = *((Cursor *) ret_val.addr);
	fromVal->addr = (char *) string;
	cvtDone(Cursor, cursor, toVal);
	return(TRUE);
}

/*-------------------------------------------------------------------*
|
|   Boolean/Short/Int/Long/String conversion
|
--------------------------------------------------------------------*/

/*-----------------------------------------------------------
|  Routine Name: cvtBooleanToInt
|       Purpose: Convert a boolean to an int.
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtBooleanToInt(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	cvtDone(int, (*(Boolean *)fromVal->addr) != 0, toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtBooleanToShort
|       Purpose: Convert a boolean to a short. 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtBooleanToShort(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	cvtDone(short, ((*(Boolean *) fromVal->addr) != 0), toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtBooleanToString
|       Purpose: Convert a boolean to a string.
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtBooleanToString(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	char	*temp;

	if ((*(Boolean *) fromVal->addr) != 0)
	   temp = "1";
	else
	   temp = "0";

	cvtDone(String, kstrdup(temp), toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtShortToString
|       Purpose: Convert a short to a string. 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtShortToString(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	char	temp[KLENGTH];

	(void) ksprintf(temp, "%d", (*(short *) fromVal->addr));
	cvtDone(String, kstrdup(temp), toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtIntToString
|       Purpose: Convert an int to a string.
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtIntToString(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	char	temp[KLENGTH];

	(void) ksprintf(temp, "%d", (*(int *) fromVal->addr));
	cvtDone(String, kstrdup(temp), toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtFloatToString
|       Purpose: Convert a float to a string.
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Oct 24, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtFloatToString(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	char	temp[KLENGTH];

	(void) ksprintf(temp, "%g", (*(float *) fromVal->addr));
	cvtDone(String, kstrdup(temp), toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtDoubleToString
|       Purpose: Convert a double to a string.
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Oct 24, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtDoubleToString(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	char	temp[KLENGTH];

	(void) ksprintf(temp, "%g", (*(double *) fromVal->addr));
	cvtDone(String, kstrdup(temp), toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtStringToDouble
|       Purpose: Convert a string to double.
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Oct 24, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtStringToDouble(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	double num;

	num = strtod(fromVal->addr, NULL);
	cvtDone(double, num, toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtIntToShort
|       Purpose: Convert a int to short
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtIntToShort(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	cvtDone(short, (*(int *) fromVal->addr), toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtShortToInt
|       Purpose: Convert a int to a short. 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtShortToInt(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	cvtDone(int, (*(short *) fromVal->addr), toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtIntToUShort
|       Purpose: Convert a int to short
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtIntToUShort(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	cvtDone(unsigned short, (*(int *) fromVal->addr), toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtUShortToInt
|       Purpose: Convert a int to a short. 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtUShortToInt(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	cvtDone(int, (*(unsigned short *) fromVal->addr), toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtDoubleToFloat
|       Purpose: Convert a double to a float. 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtDoubleToFloat(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	cvtDone(float, (*(double *) fromVal->addr), toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtFloatToDouble
|       Purpose: Convert a float to a double. 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtFloatToDouble(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	cvtDone(double, (*(float *) fromVal->addr), toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtObjectToWidget
|       Purpose: Convert an xvobject to a widget
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young and John Salas
|          Date: Aug 18, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtObjectToWidget(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	cvtDone(Widget, xvw_widget(*(xvobject *) fromVal->addr), toVal);
	return(TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: cvtWidgetToObject
|       Purpose: Convert a widget to an xvobject.
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Feb 16, 1993
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean cvtWidgetToObject(
   Display     *display,
   XrmValuePtr args,
   Cardinal    *num_args,
   XrmValuePtr fromVal,
   XrmValuePtr toVal,
   XtPointer   *closure_ret)
{
	cvtDone(xvobject, xvw_object(*(Widget *) fromVal->addr), toVal);
	return(TRUE);
}

/*------------------------------------------------------------
|  Routine Name: xvw_init_converters
|       Purpose: Initialize all the type converters.  This routine
|		 is called from xvw_initialize routine.
|          Date: Aug 15, 1993
|    Written By: Mark Young and John Salas
------------------------------------------------------------*/
/* ARGSUSED */
void xvw_init_converters(void)
{
	XtAppContext  app_context;

	XtQstring     = XrmStringToQuark(XtRString);
	XtQfont       = XrmStringToQuark(XtRFont);
	XtQfontStruct = XrmStringToQuark(XtRFontStruct);
	XtQpixmap     = XrmStringToQuark(XtRPixmap);
	XtQdouble     = XrmStringToQuark(XtRDouble);
	XtQfloat      = XrmStringToQuark(XtRFloat);

	app_context   = xvw_appcontext(NULL);

	/*
	 *  String (to & from) Pixel
	 */
	XtAppSetTypeConverter(app_context, XtRPixel, XtRString,
		cvtPixelToString, convertArgs, (Cardinal) knumber(convertArgs),
		XtCacheByDisplay, NULL);
	XtAppSetTypeConverter(app_context, XtRString, XtRPixel,
		cvtStringToPixel, convertArgs, (Cardinal) knumber(convertArgs),
		XtCacheByDisplay, NULL);

	/*
	 *  Pixmap/Bitmap (to & from) filename
	 */
	XtAppSetTypeConverter(app_context, XtRString, XtRPixmap,
		cvtFilenameToPixmap, convertArgs,
		(Cardinal) knumber(convertArgs), XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRPixmap, XtRString,
		cvtPixmapToFilename, NULL, 0, XtCacheNone, NULL);

	/*
	 *  Pixmap Mask (to & from) filename
	 */
	XtAppSetTypeConverter(app_context, XtRString, XtRPixmapMask,
		cvtFilenameToPixmapMask, convertArgs,
		(Cardinal) knumber(convertArgs), XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRPixmapMask, XtRString,
		cvtPixmapMaskToFilename, NULL, 0, XtCacheNone, NULL);

	/*
	 *  Font/Fontstruct (to & from) font name
	 */
	XtAppSetTypeConverter(app_context, XtRFont, XtRString,
		cvtFontToString, NULL, 0, XtCacheByDisplay, NULL);
	XtAppSetTypeConverter(app_context, XtRString, XtRFont,
		cvtStringToFont, NULL, 0, XtCacheByDisplay, NULL);
	XtAppSetTypeConverter(app_context, XtRFontStruct, XtRString,
		cvtFontStructToString, NULL, 0, XtCacheByDisplay, NULL);
	XtAppSetTypeConverter(app_context, XtRString, XtRFontStruct,
		cvtStringToFontStruct, NULL, 0, XtCacheByDisplay, NULL);

	/*
	 *  Int/Boolean/Short/String/Float/Double
	 */
	XtAppSetTypeConverter(app_context, XtRBoolean, XtRInt,
		cvtBooleanToInt, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRBoolean, XtRShort,
		cvtBooleanToShort, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRBoolean, XtRString,
		cvtBooleanToString, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRBoolean, XtRString,
		cvtBooleanToString, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRShort, XtRString,
		cvtShortToString, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRInt, XtRString,
		cvtIntToString, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRFloat, XtRString,
		cvtFloatToString, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRDouble, XtRString,
		cvtDoubleToString, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRShort, XtRInt,
		cvtShortToInt, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRDimension, XtRInt,
		cvtUShortToInt, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRPosition, XtRInt,
		cvtShortToInt, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRInt, XtRShort,
		cvtIntToShort, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRInt, XtRDimension,
		cvtIntToUShort, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRInt, XtRPosition,
		cvtIntToShort, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRDouble, XtRFloat,
		cvtDoubleToFloat, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRFloat, XtRDouble,
		cvtFloatToDouble, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRObject, XtRWidget,
		cvtObjectToWidget, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRWidget, XtRObject,
		cvtWidgetToObject, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRObjectList, XtRWidgetList,
		cvtObjectToWidget, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRWidgetList, XtRObjectList,
		cvtWidgetToObject, NULL, 0, XtCacheNone, NULL);
	XtAppSetTypeConverter(app_context, XtRString, XtRDouble,
		cvtStringToDouble, NULL, 0, XtCacheNone, NULL);

	/*
	 *  Cursor (to & from) string
	 */
	XtAppSetTypeConverter(app_context, XtRString, XtRCursor,
		cvtStringToCursor, convertArgs, 
	 	(Cardinal) knumber(convertArgs), XtCacheByDisplay, NULL);
}
