 /*
  * Khoros: $Id: dispatch.c,v 1.11.2.2 1997/05/24 19:05:46 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: dispatch.c,v 1.11.2.2 1997/05/24 19:05:46 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Dispatch Event Utilities
   >>>>		
   >>>>  Private:
   >>>>                 xvw_sync()
   >>>>                 xvw_process_event()
   >>>>   Static:
   >>>>                 process_event()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


/*--------------------------------------------------------------
|
|  Routine Name: process_event
|
|       Purpose: Reads an event and then dispatches it.  If
|                no events remain and block is set to FALSE, then
|                FALSE is returned, indicating that no more events remain.
|                In addition, will record the event the event if 
|                a journal recording is going on.
|
|         Input: block - whether we should block ("wait") until
|                        an event arrives.
|
|        Output: none
|	Returns: Returns TRUE if more events are pending; otherwise,
|                FALSE is returned.  Note: if block is set to TRUE
|                then we will always return TRUE.
|
|    Written By: Mark Young
|          Date: Apr 07, 1995
| Modifications:
|
------------------------------------------------------------------*/

static int process_event(
   int block)
{
        XtInputMask  mask;
	XEvent       event;
	XtAppContext context = xvw_appcontext(NULL);

        /*
	 * If there is a pending event, and it is an X event
	 * (as opposed to XtInputEvents, XtTimerEvents, etc), then
	 * call journal record and then process event.
         */
	mask = XtAppPending(context);
        if ((mask & XtIMXEvent) || (mask == 0 && block == TRUE))
	{
	   if (XtAppPeekEvent(context, &event))
	   {
	      xvw_record_journal(&event);
	      mask = XtIMXEvent;
	   }
	   else
	      mask = XtIMAlternateInput | XtIMTimer;
	}

	/*
	 * Otherwise, if there is a pending event that is not an
	 * X event, just process it.
	 */
	if (mask != 0)
	   XtAppProcessEvent(context, mask);

	/*
	 * return TRUE if there was an X Event to process, FALSE otherwise
	 */
        return((mask & XtIMXEvent) ? TRUE : FALSE);
}

/*--------------------------------------------------------------
|
|  Routine Name: xvw_sync
|
|       Purpose: Gathers and dispatches XEvents until none are
|                left.  The purpose is to remove all pending events 
|                by either processing them or discarding them from 
|                the queue.  This is done by calling XSync(display, TRUE) 
|                if all pending events are to be discarded, or by 
|                repeatedly calling process_event() with block set to FALSE.
|
|         Input: discard - whether the pending events should be
|                          processed or simply removed from the
|                          queue.
|    Written By: Mark Young 
|          Date: Jul 08, 1993
|
------------------------------------------------------------------*/

int xvw_sync(
   int discard)
{
        int events_processed = FALSE;

        if (discard == FALSE)
	{
           while (process_event(FALSE))
	      events_processed = TRUE;
	}
        else
	   XSync(xvw_display(NULL), TRUE);

	return(events_processed);
}

/*--------------------------------------------------------------
|
|  Routine Name: xvw_process_event
|
|       Purpose: Gathers and dispatches XEvents.
|    Written By: Mark Young
|          Date: Jul 16, 1993
|
------------------------------------------------------------------*/

void xvw_process_event(void)
{
	(void) process_event(TRUE);
}
