 /*
  * Khoros: $Id: initialize.c,v 1.15.2.4 1997/05/24 19:05:48 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: initialize.c,v 1.15.2.4 1997/05/24 19:05:48 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		xvobject Initialization Routine
   >>>>		
   >>>>  Private:
   >>>>   Static:
   >>>>                 exit_xvwidgets()
   >>>>   Public:
   >>>>                 xvw_initialize()
   >>>>                 xvw_display()
   >>>>                 xvw_appcontext()
   >>>>                 xvw_screen()
   >>>>                 xvw_screennum()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <X11/Vendor.h>


static Display      *display              = NULL;
static Screen       *screen               = NULL;
static int          screennum             = 0;
static XtAppContext appcontext            = NULL;


/*
 *  Fallback Resources used for initialization
 */
static kstring fallback_resources[] = {
        "*input:           True",
	"*quit*background: #974528",
	"*quit*foreground: #ffffff",
	"*help*background: #5a9700",
	"*help*foreground: #ffffff",
	"*license*background: #457497",
	"*license*foreground: #ffffff",
	"*_gui_options*background: #5a5a97",
	"*_gui_options*foreground: #ffffff",
	"*browser_button*background: #979797",
        NULL,
};

/*-----------------------------------------------------------
|
|  Routine Name: (static) exit_xvwidgets
|
|       Purpose: This routine is used to cleanup xvwidgets upon
|                exiting of the application.  Currently the only
|                thing it does is to annoy people.
|
|         Input: status - the exit status
|                client_data - the data we installed with the kexit_handler
|                              handler
|
|        Output:
|       Returns:
|    Written By: Mark Young
|          Date: Oct 31, 1993
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void exit_xvwidgets(
   kexit_status   status,
   kaddr          client_data)
{
	Display *disp = (Display *) client_data;

	if (kget_notify() == KHOSTILE				  &&
	    kaccess("/usr/demo/SOUND/sounds/flush.au", R_OK) == 0 &&
	    kaccess("/dev/audio", W_OK) == 0			  &&
	    xvw_check_localhost(NULL))
	{
	   kcopyfile("/usr/demo/SOUND/sounds/flush.au", "/dev/audio");
	}
	kset_errorhandler(NULL);
        kset_warnhandler(NULL);
        kset_prompthandler(NULL);
        kset_infohandler(NULL);
        kset_announcehandler(NULL);
        kset_choosehandler(NULL);
	if (disp) XtCloseDisplay(disp);
	kinfo(KDEBUG, "exiting....");
}


/************************************************************
*
*  Routine Name: xvw_initialize - initialize the xvwidgets library
*
*       Purpose: Opens the display, sets the widget set of choice,
*                and initializes the xvwidgets library prior to use.
*                A call to this routine \fImust\fP be made before any other
*                calls to routines in GUI & Visualization services
*                are made.
*
*         Input: menu_handler - specifies the menu handling routines. 
*                               May be one of:
*                ! XVW_MENUS_XVFORMS -  Enable use of xvforms as internal menus
*                ! XVW_MENUS_NONE    -  Disable internal menus
*        Output:
*       Returns: The newly opened display on success, NULL on failure
*
*  Restrictions: 
*    Written By: Mark Young & Danielle Argiro
*          Date: Jun 17, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*******************************************************************/
Display *xvw_initialize(
   void (*menu_handler)(void))
{
	int  argc, indx1, indx2;
        char path[KLENGTH], toolbox[KLENGTH];
        char **argv, *tbname, *program, *playback = NULL, *record = NULL;

	/*
	 *   *****  general initializations  *****
	 */
	xvw_toplevels       = NULL;

	/*
	 *  Sanity check for now...  Make sure that we don't allow the
	 *  programmer to re-initialize the display.
	 */
	if (display)
	   return(display);

	/* null out the announce handler */
        kset_announcehandler(NULL);

        /* initialize the toolkit */
        XtToolkitInitialize();

        /* create an application context */
        appcontext = XtCreateApplicationContext();

        /*
	 * in case there was a problem creating application context,
         * set fallback resources
	 */
        XtAppSetFallbackResources(appcontext, fallback_resources);

        /* open the display */
	argc = kprog_get_argc();
	argv = kprog_get_argv();
        display = XtOpenDisplay(appcontext, NULL, kprog_get_program(),
                                "Khoros", NULL, (Cardinal) 0,
                                &argc, argv);
        if (display == NULL)
        {
	    kerror("xvwidgets", "xvw_initialize", 
		   "Unable to open display... '%s'", XDisplayName(NULL));
            return(NULL);
        }

	indx1 = karray_locate(argv, KSTRING, "-record", argc);
	indx2 = karray_locate(argv, KSTRING, "-playback", argc);
	if (indx1 != -1)
	{
	   record = argv[indx1+1];
	   argv[indx1] = argv[indx1+1] = NULL; argc -= 2;
	}
	if (indx2 != -1)
	{
	   playback = argv[indx2+1];
	   argv[indx2] = argv[indx2+1] = NULL; argc -= 2;
	}
	kprog_set_argc(argc);
	kprog_set_argv(argv);

	/*
	 *  Initialize the journal playback and record files if they
	 *  have been specified on the command line.
	 */
	if (playback || record)
	   xvw_start_journal(playback, record);

#if 0
	XtInitializeWidgetClass(vendorShellWidgetClass);
	XtInitializeWidgetClass(ManagerWidgetClass);
#endif
        screennum = XDefaultScreen(display);
        screen = XDefaultScreenOfDisplay(display);

	/*
	 *  Install our own X error handlers and Xt converters.  An exit
	 *  handler is also installed in order to shutdown the display.
	 */
	xvw_init_errors();
	xvw_init_converters();
	xvw_init_handlers();
	kexit_handler(exit_xvwidgets, (kaddr) display);

	/*
	 *  Check the command line arguments and make sure that there are
	 *  no left-over command line.
	 */
	if (!kclui_check_args())
	   kexit(KEXIT_FAILURE);

	if (menu_handler != XVW_MENUS_NONE)
	    (*menu_handler)();

        program = kprog_get_program();
        tbname  = kprog_get_toolbox();
        if (program != NULL && tbname != NULL)
        {
           kstring_upper(tbname, toolbox);
           ksprintf(path, "$%s/objects/xvroutine/%s/app-defaults/%s",
                        toolbox, program, program);
           xvw_load_resources(path);
        }

        if (!khoros_imprint())
           kexit(KEXIT_FAILURE);

	return(display);
}

/************************************************************
*
*  Routine Name: xvw_display - returns the display associated with a object
*
*       Purpose: Returns the display associated with the object.
*		 Note that this is the GUI & Visualization
*                services equivalent of \fIXtDisplay()\fP.
*		 If object is NULL, returns the default display.
*
*         Input: object - object for which to identify the display
*
*        Output: none
*	Returns: The display associated with the object or gadget
*                on success, NULL on failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Jun 15, 1993
*      Verified: 
*  Side Effects:
* Modifications:
*
*******************************************************************/

Display *xvw_display(
   xvobject object)
{
	Widget widget = xvw_widget(object);

	if (!object)
	   return(display);

	return(XtDisplayOfObject(widget));
}

/************************************************************
*
*  Routine Name: xvw_appcontext - return the application context associated
*				  with a object
*
*       Purpose: Returns the application context associated with the
*		 object.    Note that this is the GUI & Visualization
*                services equivalent of \fIXtWidgetToApplicationContext()\fP.
*		 If object is NULL, returns the default application context.
*
*         Input:  object - object for which to get the application context
*        Output: 
*       Returns: The application context associated with the object 
*		 on success, NULL on failure
*
*  Restrictions: Restrictions on data or input as applicable
*    Written By: Mark Young
*          Date: Jan 21, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

XtAppContext xvw_appcontext(
   xvobject object)
{
	Widget widget = xvw_widget(object);

	if (!object)
	   return(appcontext);

	return(XtWidgetToApplicationContext(widget));
}

/************************************************************
*
*  Routine Name: xvw_screen - return the screen associated with a object
*
*       Purpose: Returns the display screen associated with the
*		 object.  Note that this is the GUI & Visualization
*                services equivalent of \fIXtScreen()\fP.
*		 If object is NULL, returns the default screen.
*
*         Input:  object - object for which to get the screen
*        Output: 
*       Returns: The Screen associated with the object 
*		 on success, NULL on failure
*
*  Restrictions: Restrictions on data or input as applicable
*    Written By: Mark Young
*          Date: Jan 21, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

Screen *xvw_screen(
   xvobject object)
{
	Widget widget = xvw_widget(object);

	if (!object)
	   return(screen);

	return(XtScreenOfObject(widget));
}

/************************************************************
*
*  Routine Name: xvw_screennum - return the screen number associated 
*                                with an object
*
*       Purpose: Returns the screen index number associated with the
*		 object.  Note that this is the GUI & Visualization
*		 services equivalent of \fIXScreenNumberOfScreen()\fP.
*		 If object is NULL, returns the default screen number.
*
*         Input:  object - object for which to get the screen number
*        Output: 
*       Returns: The screen number associated with the object
*		 on success, -1 on failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Jan 21, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

int xvw_screennum(
   xvobject object)
{
	Widget widget = xvw_widget(object);

	Screen *tmp;

	if (!object)
	   return(screennum);

	if ((tmp = XtScreenOfObject(widget)) != NULL)
	   return(XScreenNumberOfScreen(tmp));
	else
	   return(-1);
}
