/*
 * Khoros: $Id: journal.c,v 1.3.2.2 1997/05/24 19:05:52 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: journal.c,v 1.3.2.2 1997/05/24 19:05:52 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Journal Playback Routines
   >>>> 
   >>>>	Private: 
   >>>>		   xvw_record_journal()
   >>>>		   xvw_start_journal()
   >>>>		   xvw_stop_journal()
   >>>>	Static: 
   >>>>		   lookup_event()
   >>>>		   read_event()
   >>>>		   write_event()
   >>>>		   name_to_xvobject()
   >>>>		   xvobject_to_name()
   >>>>		   file_playback()
   >>>>		   input_playback()
   >>>>	Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

kfile *record_file  = NULL;
kfile *playback_file = NULL;
XtInputId InputId = (XtInputId) 0;

/*------------------------------------------------------------
|
|  Routine Name: lookup_event
|
|       Purpose: This routine will return the event name for the given
|		 event type.
|
|        Output: type - the event type to be looked up
|	Returns: internal string representing the event or NULL
|    Written By: Mark Young
|          Date: Jul 9, 1992 
|
--------------------------------------------------------------*/
static char *lookup_event(
   int type)
{
	/*
	 *  Return the name associated with the event type.
	 */
	switch (type)
	{
	   case KeyPress:	  return("KeyPress");
	   case KeyRelease:	  return("KeyRelease");
	   case ButtonPress:	  return("ButtonPress");
	   case ButtonRelease:	  return("ButtonRelease");
	   case MotionNotify:	  return("MotionNotify");
	   case EnterNotify:	  return("EnterNotify");
	   case LeaveNotify:	  return("LeaveNotify");
	   case FocusIn:	  return("FocusIn");
	   case FocusOut:	  return("FocusOut");
	   case KeymapNotify:	  return("KeymapNotify");
	   case Expose:		  return("Expose");
	   case GraphicsExpose:	  return("GraphicsExpose");
	   case NoExpose:	  return("NoExpose");
	   case VisibilityNotify: return("VisibilityNotify");
	   case CreateNotify:	  return("CreateNotify");
	   case UnmapNotify:	  return("UnmapNotify");
	   case MapNotify:	  return("MapNotify");
	   case ReparentNotify:	  return("ReparentNotify");
	   case ConfigureNotify:  return("ConfigureNotify");
	   case GravityNotify:	  return("GravityNotify");
	   case DestroyNotify:	  return("DestroyNotify");
	   case MapRequest:	  return("MapRequest");
	   case ConfigureRequest: return("ConfigureRequest");
	   case ResizeRequest:	  return("ResizeRequest");
	   case CirculateNotify:  return("CirculateNotify");
	   case CirculateRequest: return("CirculateRequest");
	   case PropertyNotify:	  return("PropertyNotify");
	   case SelectionClear:	  return("SelectionClear");
	   case SelectionRequest: return("SelectionRequest");
	   case SelectionNotify:  return("SelectionNotify");
	   case ColormapNotify:	  return("ColormapNotify");
	   case ClientMessage:	  return("ClientMessage");
	   default:		  return("Unknown!");
	}
}

/*------------------------------------------------------------
|
|  Routine Name: read_event
|
|       Purpose: This routine will read in an event from
|		 a given file.
|
|        Output: event - the event being read 
|	Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jul 9, 1992 
|
--------------------------------------------------------------*/
static int read_event(
   XEvent *event)
{
	unsigned long tval;
	int fid = kfileno(playback_file);
	size_t size = 1;

	/*
	 *  Zero out the event structure then read in the xany fields
	 *  which consist of the Event "type", "serial", and "send_event"
	 *  fields.
	 */
	kmemset((char *) event, 0, sizeof(XEvent));
	kread_int(fid, &event->xany.type, size);
	kread_ulong(fid, &event->xany.serial, size);
	kread_int(fid, &event->xany.send_event, size);

	/*
	 *  After reading the general "xany" information, we now need to
	 *  read the event specific information.  To do this we switch
	 *  on the event type in order to read the correct info.
	 */
	switch (event->type)
	{
	   case KeyPress:
	   case KeyRelease:
		kread_ulong(fid, &tval, size); event->xkey.time = tval;
		kread_int(fid, &event->xkey.x, size);
		kread_int(fid, &event->xkey.y, size);
		kread_int(fid, &event->xkey.x_root, size);
		kread_int(fid, &event->xkey.y_root, size);
		kread_uint(fid, &event->xkey.state, size);
		kread_uint(fid, &event->xkey.keycode, size);
		kread_int(fid, &event->xkey.same_screen, size);
		break;
	   case ButtonPress:
	   case ButtonRelease:
		kread_ulong(fid, &tval, size); event->xbutton.time = tval;
		kread_int(fid, &event->xbutton.x, size);
		kread_int(fid, &event->xbutton.y, size);
		kread_int(fid, &event->xbutton.x_root, size);
		kread_int(fid, &event->xbutton.y_root, size);
		kread_uint(fid, &event->xbutton.state, size);
		kread_uint(fid, &event->xbutton.button, size);
		kread_int(fid, &event->xbutton.same_screen, size);
		break;
	   case MotionNotify:
		kread_ulong(fid, &tval, size); event->xmotion.time = tval;
		kread_int(fid, &event->xmotion.x, size);
		kread_int(fid, &event->xmotion.y, size);
		kread_int(fid, &event->xmotion.x_root, size);
		kread_int(fid, &event->xmotion.y_root, size);
		kread_uint(fid, &event->xmotion.state, size);
		kread_byte(fid, &event->xmotion.is_hint, size);
		kread_int(fid, &event->xbutton.same_screen, size);
		break;
	   case EnterNotify:
	   case LeaveNotify:
		kread_ulong(fid, &tval, size); event->xcrossing.time = tval;
		kread_int(fid, &event->xcrossing.x, size);
		kread_int(fid, &event->xcrossing.y, size);
		kread_int(fid, &event->xcrossing.x_root, size);
		kread_int(fid, &event->xcrossing.y_root, size);
		kread_int(fid, &event->xcrossing.mode, size);
		kread_int(fid, &event->xcrossing.detail, size);
		kread_int(fid, &event->xcrossing.same_screen, size);
		kread_int(fid, &event->xcrossing.focus, size);
		kread_uint(fid, &event->xcrossing.state, size);
		break;
	   case FocusIn:
	   case FocusOut:
		kread_int(fid, &event->xfocus.mode, size);
		kread_int(fid, &event->xfocus.detail, size);
		break;
	   case KeymapNotify:
		kread_byte(fid, event->xkeymap.key_vector, (size_t) 32);
		break;
	   case Expose:
		kread_int(fid, &event->xexpose.x, size);
		kread_int(fid, &event->xexpose.y, size);
		kread_int(fid, &event->xexpose.width, size);
		kread_int(fid, &event->xexpose.height, size);
		kread_int(fid, &event->xexpose.count, size);
		break;
	   case GraphicsExpose:
		kread_int(fid, &event->xgraphicsexpose.x, size);
		kread_int(fid, &event->xgraphicsexpose.y, size);
		kread_int(fid, &event->xgraphicsexpose.width, size);
		kread_int(fid, &event->xgraphicsexpose.height, size);
		kread_int(fid, &event->xgraphicsexpose.count, size);
		kread_int(fid, &event->xgraphicsexpose.major_code, size);
		kread_int(fid, &event->xgraphicsexpose.minor_code, size);
		break;
	   case NoExpose:
		kread_int(fid, &event->xnoexpose.major_code, size);
		kread_int(fid, &event->xnoexpose.minor_code, size);
		break;
	   case VisibilityNotify:
		kread_int(fid, &event->xvisibility.state, size);
		break;
	   case CreateNotify:
		kread_int(fid, &event->xcreatewindow.x, size);
		kread_int(fid, &event->xcreatewindow.y, size);
		kread_int(fid, &event->xcreatewindow.width, size);
		kread_int(fid, &event->xcreatewindow.height, size);
		kread_int(fid, &event->xcreatewindow.border_width, size);
		kread_int(fid, &event->xcreatewindow.override_redirect, size);
		break;
	   case UnmapNotify:
		kread_int(fid, &event->xunmap.from_configure, size);
		break;
	   case MapNotify:
		kread_int(fid, &event->xmap.override_redirect, size);
		break;
	   case ReparentNotify:
		kread_int(fid, &event->xreparent.x, size);
		kread_int(fid, &event->xreparent.y, size);
		kread_int(fid, &event->xreparent.override_redirect, size);
		break;
	   case ConfigureNotify:
		kread_int(fid, &event->xconfigure.x, size);
		kread_int(fid, &event->xconfigure.y, size);
		kread_int(fid, &event->xconfigure.width, size);
		kread_int(fid, &event->xconfigure.height, size);
		kread_int(fid, &event->xconfigure.border_width, size);
		kread_int(fid, &event->xconfigure.override_redirect, size);
		break;
	   case GravityNotify:
		kread_int(fid, &event->xgravity.x, size);
		kread_int(fid, &event->xgravity.y, size);
		break;
	   case DestroyNotify:
		/*  do nothing */
		break;
	   case MapRequest:
		/*  do nothing */
		break;
	   case ConfigureRequest:
		kfprintf(kstderr,"play ConfigureRequest\n");
		break;
	   case ResizeRequest:
		kfprintf(kstderr,"play ResizeRequest\n");
		break;
	   case CirculateNotify:
		kfprintf(kstderr,"play CirculateNotify\n");
		break;
	   case CirculateRequest:
		kfprintf(kstderr,"play CirculateRequest\n");
		break;
	   case PropertyNotify:
		kfprintf(kstderr,"play PropertyNotify\n");
		break;
	   case SelectionClear:
		kfprintf(kstderr,"play SelectionClear\n");
		break;
	   case SelectionRequest:
		kfprintf(kstderr,"play SelectionRequest\n");
		break;
	   case SelectionNotify:
		kfprintf(kstderr,"play SelectionNotify\n");
		break;
	   case ColormapNotify:
		kfprintf(kstderr,"play ColormapNotify\n");
		break;
	   case ClientMessage:
		kfprintf(kstderr,"play ClientMessage\n");
		break;
	}

	/*
	 *  Key codes are not portable since ever keyboard maps different
	 *  keys to different properties.  So lookup the XKeysym associated
	 *  with the stored string.
	 */
        if (event->type == KeyPress || event->type == KeyRelease)
        {
	   KeySym  keysym;
	   KeyCode keycode;
	   char    temp[KLENGTH];

	   kfscanf(playback_file, "%[^\n]\n", temp);
	   keysym = XStringToKeysym(temp);
	   if ((keycode = XKeysymToKeycode(xvw_display(NULL), keysym)) == 0)
	   {
	      if (keysym == XK_BackSpace)
	         keycode = XKeysymToKeycode(xvw_display(NULL), XK_Delete);
	      else if (keysym == XK_Delete)
	         keycode = XKeysymToKeycode(xvw_display(NULL), XK_BackSpace);
	   }
	   event->xkey.keycode = keycode;
        }
	return(TRUE);
}

/*------------------------------------------------------------
|
|  Routine Name: write_event
|
|       Purpose: This routine will write a given event out to 
|		 a given file. 
|
|         Input: event - the event to write
|	Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jul 9, 1992 
|
--------------------------------------------------------------*/
static int write_event(
   XEvent *event)
{
	size_t size = 1;
	unsigned long tval;
	int fid = kfileno(record_file);

	/*
	 *  Write out the xany fields which consist of the Event "type",
	 *  "serial", and "send_event" fields.
	 */
	kwrite_int(fid, &event->xany.type, size);
	kwrite_ulong(fid, &event->xany.serial, size);
	kwrite_int(fid, &event->xany.send_event, size);

	/*
	 *  After writing the general "xany" information, we now need to
	 *  write the event specific information.  To do this we switch
	 *  on the event type in order to write the correct info.
	 */
	switch (event->type)
	{
	   case KeyPress:
	   case KeyRelease:
		tval = event->xkey.time; kwrite_ulong(fid, &tval, size);
		kwrite_int(fid, &event->xkey.x, size);
		kwrite_int(fid, &event->xkey.y, size);
		kwrite_int(fid, &event->xkey.x_root, size);
		kwrite_int(fid, &event->xkey.y_root, size);
		kwrite_uint(fid, &event->xkey.state, size);
		kwrite_uint(fid, &event->xkey.keycode, size);
		kwrite_int(fid, &event->xkey.same_screen, size);
		break;
	   case ButtonPress:
	   case ButtonRelease:
		tval = event->xbutton.time; kwrite_ulong(fid, &tval, size);
		kwrite_int(fid, &event->xbutton.x, size);
		kwrite_int(fid, &event->xbutton.y, size);
		kwrite_int(fid, &event->xbutton.x_root, size);
		kwrite_int(fid, &event->xbutton.y_root, size);
		kwrite_uint(fid, &event->xbutton.state, size);
		kwrite_uint(fid, &event->xbutton.button, size);
		kwrite_int(fid, &event->xbutton.same_screen, size);
		break;
	   case MotionNotify:
		tval = event->xmotion.time; kwrite_ulong(fid, &tval, size);
		kwrite_int(fid, &event->xmotion.x, size);
		kwrite_int(fid, &event->xmotion.y, size);
		kwrite_int(fid, &event->xmotion.x_root, size);
		kwrite_int(fid, &event->xmotion.y_root, size);
		kwrite_uint(fid, &event->xmotion.state, size);
		kwrite_byte(fid, &event->xmotion.is_hint, size);
		kwrite_int(fid, &event->xbutton.same_screen, size);
		break;
	   case EnterNotify:
	   case LeaveNotify:
		tval = event->xcrossing.time; kwrite_ulong(fid, &tval, size);
		kwrite_int(fid, &event->xcrossing.x, size);
		kwrite_int(fid, &event->xcrossing.y, size);
		kwrite_int(fid, &event->xcrossing.x_root, size);
		kwrite_int(fid, &event->xcrossing.y_root, size);
		kwrite_int(fid, &event->xcrossing.mode, size);
		kwrite_int(fid, &event->xcrossing.detail, size);
		kwrite_int(fid, &event->xcrossing.same_screen, size);
		kwrite_int(fid, &event->xcrossing.focus, size);
		kwrite_uint(fid, &event->xcrossing.state, size);
		break;
	   case FocusIn:
	   case FocusOut:
		kwrite_int(fid, &event->xfocus.mode, size);
		kwrite_int(fid, &event->xfocus.detail, size);
		break;
	   case KeymapNotify:
		kwrite_byte(fid, event->xkeymap.key_vector, (size_t) 32);
		break;
	   case Expose:
		kwrite_int(fid, &event->xexpose.x, size);
		kwrite_int(fid, &event->xexpose.y, size);
		kwrite_int(fid, &event->xexpose.width, size);
		kwrite_int(fid, &event->xexpose.height, size);
		kwrite_int(fid, &event->xexpose.count, size);
		break;
	   case GraphicsExpose:
		kwrite_int(fid, &event->xgraphicsexpose.x, size);
		kwrite_int(fid, &event->xgraphicsexpose.y, size);
		kwrite_int(fid, &event->xgraphicsexpose.width, size);
		kwrite_int(fid, &event->xgraphicsexpose.height, size);
		kwrite_int(fid, &event->xgraphicsexpose.count, size);
		kwrite_int(fid, &event->xgraphicsexpose.major_code, size);
		kwrite_int(fid, &event->xgraphicsexpose.minor_code, size);
		break;
	   case NoExpose:
		kwrite_int(fid, &event->xnoexpose.major_code, size);
		kwrite_int(fid, &event->xnoexpose.minor_code, size);
		break;
	   case VisibilityNotify:
		kwrite_int(fid, &event->xvisibility.state, size);
		break;
	   case CreateNotify:
		kwrite_int(fid, &event->xcreatewindow.x, size);
		kwrite_int(fid, &event->xcreatewindow.y, size);
		kwrite_int(fid, &event->xcreatewindow.width, size);
		kwrite_int(fid, &event->xcreatewindow.height, size);
		kwrite_int(fid, &event->xcreatewindow.border_width, size);
		kwrite_int(fid, &event->xcreatewindow.override_redirect, size);
		break;
	   case UnmapNotify:
		kwrite_int(fid, &event->xunmap.from_configure, size);
		break;
	   case MapNotify:
		kwrite_int(fid, &event->xmap.override_redirect, size);
		break;
	   case ReparentNotify:
		kwrite_int(fid, &event->xreparent.x, size);
		kwrite_int(fid, &event->xreparent.y, size);
		kwrite_int(fid, &event->xreparent.override_redirect, size);
		break;
	   case ConfigureNotify:
		kwrite_int(fid, &event->xconfigure.x, size);
		kwrite_int(fid, &event->xconfigure.y, size);
		kwrite_int(fid, &event->xconfigure.width, size);
		kwrite_int(fid, &event->xconfigure.height, size);
		kwrite_int(fid, &event->xconfigure.border_width, size);
		kwrite_int(fid, &event->xconfigure.override_redirect, size);
		break;
	   case GravityNotify:
		kwrite_int(fid, &event->xgravity.x, size);
		kwrite_int(fid, &event->xgravity.y, size);
		break;
	   case DestroyNotify:
		/*  do nothing */
		break;
	   case MapRequest:
		/*  do nothing */
		break;
	   case ConfigureRequest:
		kfprintf(kstderr,"record ConfigureRequest\n");
		break;
	   case ResizeRequest:
		kfprintf(kstderr,"record ResizeRequest\n");
		break;
	   case CirculateNotify:
		kfprintf(kstderr,"record CirculateNotify\n");
		break;
	   case CirculateRequest:
		kfprintf(kstderr,"record CirculateRequest\n");
		break;
	   case PropertyNotify:
		kfprintf(kstderr,"record PropertyNotify\n");
		break;
	   case SelectionClear:
		kfprintf(kstderr,"record SelectionClear\n");
		break;
	   case SelectionRequest:
		kfprintf(kstderr,"record SelectionRequest\n");
		break;
	   case SelectionNotify:
		kfprintf(kstderr,"record SelectionNotify\n");
		break;
	   case ColormapNotify:
		kfprintf(kstderr,"record ColormapNotify\n");
		break;
	   case ClientMessage:
		kfprintf(kstderr,"record ClientMessage\n");
		break;
	}

	/*
	 *  Key codes are not portable since ever keyboard maps different
	 *  keys to different properties.  So lookup the string associated
	 *  with the XKeysym.  We then do the opposite in the read_event()
	 *  routine.
	 */
        if (event->type == KeyPress || event->type == KeyRelease)
        {
	   KeySym keysym;
	   char   *string, temp[KLENGTH];

           (void) XLookupString(&event->xkey, temp, KLENGTH, &keysym, NULL);
	   string = XKeysymToString(keysym);
	   kfprintf(record_file, "%s\n", !string ? "" : string);
        }
	return(TRUE);
}

/*------------------------------------------------------------
|
|  Routine Name: xvobject_to_name
|
|       Purpose: This routine takes an object and returns it's full
|                name, made up of its full ancestor list.
|
|                example: great-grandparent.grandparent.parent.child
|
|         Input: object - the object to build the full name for
|	 Output: name   - the character array to store the name in
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: Jul 9, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static int xvobject_to_name(
   xvobject object,
   char     *name)
{
	size_t   num, indx;
	char     buffer[KLENGTH];
	xvobject *children, parent = xvw_parent(object);

	/*
	 *  If the parent is not NULL then recursively call ourselves
	 *  until we reach the top-most object.  We then recursively
	 *  build our name.
	 */
	if (parent != NULL)
	{
	   if (!xvobject_to_name(parent, name))
	      return(FALSE);

	   children = xvw_children(parent, NULL, &num);
	}
	else
	{
	   name[0] = '\0';
	   children = xvw_getall_toplevels(&num);
	}

	if ((indx = karray_locate(children, KSTRUCT, object, num)) == -1)
	{
           kfprintf(kstderr, "\njournal_record:\n\n");
           kfprintf(kstderr, "Couldn't locate object to build name '%s'\ninto "
			     "journal record file.  Discarding event!\n",
			     xvw_name(object));
	   return(FALSE);
	}
	kfree(children);

	ksprintf(buffer, "%d:%s:", indx, xvw_name(object));
	kstrcat(name, buffer);
	return(TRUE);
}

/*------------------------------------------------------------
|
|  Routine Name: name_to_xvobject
|
|       Purpose: This routine takes a full name and returns the associated
|                widget.
|
|                example: great-grandparent.grandparent.parent.child
|
|         Input: name - the full name string of the widget
|       Returns: The widget on success, NULL otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: Jul 9, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static xvobject name_to_xvobject(
   char   *name)
{
	size_t   num;
	int      indx;
	xvobject object, *children;
	char     temp[KLENGTH], remainder[KLENGTH];

	object = NULL;
	while (name != NULL)
	{
	   if (ksscanf(name, " %d:%[^:]:%[^\n]", &indx, temp, remainder) != 3)
	   {
	      kfprintf(kstderr,"\njournal_playback: ");
              kfprintf(kstderr,"Error parsing name from playback file.\n\n"
			     "Unable to parse the name from the string '%s'\n"
			     "Discarding playing back event!\n", name);
              return(NULL);
	   }

	   if (object == NULL)
	      children = xvw_getall_toplevels(&num);
	   else
	      children = xvw_children(object, NULL, &num);

	   if (indx < 0 || indx >= num)
	   {
	      kfprintf(kstderr,"\njournal_playback: ");
              kfprintf(kstderr,"Invalid index retrieved from playback file.\n\n"
			     "Parse index '%d' from the string '%s',\n"
			     "but no such index exists.  Discarding playing "
			     "back event!\n", indx, name);
	      kfree(children);
              return(NULL);
	   }
	   object = children[indx]; kfree(children);

	   /* verify the stored name with the retrieved object.  */
	   if (kstrcmp(temp, xvw_name(object)) != 0)
	   {
	      kfprintf(kstderr,"\njournal_playback: ");
              kfprintf(kstderr,"Object name retrieved from playback file.\n\n"
			     "Parsed name '%s' from the string '%s',\n"
			     "but the corresponding object has the name '%s'."
			     "Discarding playing back event!\n", temp, name,
			     xvw_name(object));
              return(NULL);
	   }
	   name = kstring_cleanup(remainder, remainder);
	}
	return(object);
}

/*------------------------------------------------------------
|
|  Routine Name: file_playback
|
|       Purpose: This routine is used as the driver to play back
|		 the events out of a file.  First we check to
|		 see if there is an event to be dispatched (if
|		 the data is not NULL).
|
|		 After dispatching the previous event, we read the
|		 next event from the file.  Associated with this event 
|                is the elapsed time since the last event.  With this, we
|                compute the amount of time we need to wait before
|		 playing back this event.  We then add a timeout to
|		 call us back after that elapsed time.  At that time
|		 we will dispatch the event and get the next event to
|		 played back.
|
|         Input: data  - the data event to be played back, NULL if no event.
|       	 id    - the id for this timeout.
|    Written By: Mark Young and Danielle Argiro
|          Date: Jul 9, 1992 
|
------------------------------------------------------------*/
/* ARGSUSED */
static void file_playback(
   XtPointer    data,
   XtIntervalId *id)
{
	char *widname = (char *) data;

	XEvent event;
	xvobject object;
	int    elapsed, dispatch_event = FALSE;
	char   name[10*KLENGTH], buffer[10*KLENGTH], type[KLENGTH];

	/*
	 *  If there is an event then dispatch it before getting the next
	 *  one from file_playback().
	 */
	if (widname != NULL && read_event(&event) &&
	    (object = name_to_xvobject(widname)) != NULL)
	{
	   event.xany.display = xvw_display(object);
	   event.xany.window  = xvw_window(object);
	   dispatch_event = TRUE;
	}
	kfree_and_NULL(widname);

	/*
	 *  Get the elapsed time for the next event from playback_file.
	 *  If it fails then add a timeout to calls us back immediately,
	 *  otherwise call us back with the original elapsed time .vs. the
	 *  current elapsed.  Which we will then dispatch the event at the
	 *  correct amount of elapsed time.  Also parse the name which we
	 *  will use to trigger the dispatching of event on playback.
	 */
        if (kfgets(buffer, 10*KLENGTH, playback_file) != NULL &&
	    ksscanf(buffer, " %d %s '%[^']'", &elapsed, type, name) == 3)
	{
	   XtAppAddTimeOut(xvw_appcontext(NULL), (unsigned long) elapsed, 
			   file_playback, kstrdup(name));
        }
	/* make sure we ain't at the end of the journal file */
	else if (kfeof(playback_file) == TRUE)
	{
	   kinfo(KSTANDARD, "End of playback for journal file. Will continue "
		 "normally.");
	   kfclose(playback_file); playback_file = NULL;
	}
	else
	{
           kfprintf(kstderr, "journal_playback:\n");
           kfprintf(kstderr, "Couldn't parse elapsed time and name from "
			     "journal playback file.\n");
	   XtAppAddTimeOut(xvw_appcontext(NULL), (unsigned long) 50,
			   file_playback, NULL);
	}

	/* so if we need to dispatch the event, dispatch it at this time */
	if (dispatch_event == TRUE)
	   XtDispatchEvent(&event);
}

/*------------------------------------------------------------
|
|  Routine Name: input_playback
|
|       Purpose: This routine is used as the driver to play back
|		 the events from the input stream.  The difference
|		 between input_playback() and file_playback() is that
|		 the input callback is triggered when a new event
|		 arrives and is then dispatched immediately, rather
|		 than waiting for some duration.
|
|        Output: data  - the data event to be played back, NULL if no event.
|       	 id    - the id for this timeout.
|    Written By: Mark Young and Danielle Argiro
|          Date: Jul 9, 1992 
|
------------------------------------------------------------*/
/* ARGSUSED */
static void input_playback(
   XtPointer data,
   int	     *fid,
   XtInputId *id)
{
	XEvent   event;
	int      elapsed;
	xvobject object;
	char     name[10*KLENGTH], buffer[10*KLENGTH], type[KLENGTH];

	/*
	 *  If there is an event then dispatch it before getting the next
	 *  one from file_playback().
	 */
	if (kfeof(playback_file) == TRUE)
	{
	   kinfo(KSTANDARD, "End of playback for journal file. Will continue "
		 "normally.");
	   kfclose(playback_file); playback_file = NULL;
	   XtRemoveInput(InputId); InputId = 0;
	   return;
	}
	/*
	 *  Get the elapsed time for the next event from playback_file.
	 *  If it fails then error, otherwise read the event and dispatch
	 *  the event immediately.  Also parse the name which we will use
	 *  to trigger the dispatching of event on playback.
	 */
        else if (kfgets(buffer, 10*KLENGTH, playback_file) == NULL ||
	         ksscanf(buffer, " %d %s '%[^']'", &elapsed, type, name) != 3)
	{
           kfprintf(kstderr, "journal_playback:\n");
           kfprintf(kstderr, "Couldn't parse elapsed time and name from "
			     "journal playback file.\n");
	}
	else if (read_event(&event) && (object=name_to_xvobject(name)) != NULL)
	{
	   event.xany.display = xvw_display(object);
	   event.xany.window  = xvw_window(object);
	   XtDispatchEvent(&event);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_record_journal
|
|       Purpose: Records an event
|                or journal record.
|
|         Input: event - the event to be recorded
|    Written By: Mark Young & Danielle Argiro
|          Date: Jul 9, 1992 
|
------------------------------------------------------------*/
void xvw_record_journal(
   XEvent *event)
{
	Time     ttime;
	Widget   widget;
	xvobject object;
	int      elapsed;
	Window   window;
	Display  *display;
	char     name[10*KLENGTH], *type;

	static Time otime = 0;

	/*
	 *  Check to see if we are journal recording.  If not then just return.
	 */
	if (record_file == NULL)
	   return;

	/*
	 *  Just for now weed the following events...
	 */
        if (event->type == DestroyNotify  || event->type == Expose ||
	    event->type == GraphicsExpose || event->type == NoExpose ||
            event->xany.send_event == TRUE)
        {
           return;
        }

	/*  get the display and window from the event */
	window  = event->xany.window;
	display = event->xany.display;

        /*
         *  Compute the elapsed time for recording of events.  This will
	 *  be used to delay the amount of time in which the journal playback
	 *  routine delays before dispatching the playback event.
         */
        ttime = XtLastTimestampProcessed(display);
        if (otime == 0 || ttime == 0)
           elapsed = 0;
        else
           elapsed = ttime - otime;

	/*
	 *  Get the widget associated with the event.  If no widget is
	 *  registered then simply return.
	 */
	if ((widget = XtWindowToWidget(display, window)) == NULL ||
	    (object = xvw_check_widget(widget)) == NULL)
	{
	   return;
	}

	/*
	 *  Now that we have the object, we need to construct a "full"
	 *  name in which to store with the event.  We will use this name
	 *  and the elapsed time to play back the event.
	 */
	if ((type = lookup_event(event->type)) == NULL ||
	     xvobject_to_name(object, name) == FALSE)
	{
	   return;
	}

	/*
	 *  Write a journal playback header which is simply the elapsed time
	 *  and "full" name.  ie)
	 *
	 *  10 '<num>:top_name:<num>:parent_name:<num>:button_name:' KeyPress
	 *  <recorded event structure>
	 */
        if (kfprintf(record_file, "%5d %16s '%s'\n", elapsed, type, name) == 0)
	   return;

	
	/*
	 *  Fix these event up before writing them out.
	 */
        if (event->type == ConfigureNotify && !XtParent(widget))
        {
	   int    x, y;
	   Window dummy;

           XTranslateCoordinates(display, window,  XRootWindow(display,
                            xvw_screennum(object)), 0, 0, &x, &y, &dummy);
           event->xconfigure.x = x;
           event->xconfigure.y = y;
        }
        else if (event->type == MotionNotify && event->xmotion.is_hint)
        {
	   Window dummy;
	   unsigned int mask;
	   int    x, y, rx, ry;

           XQueryPointer(display, window, &dummy, &dummy, &rx,&ry,&x,&y, &mask);
           event->xmotion.x = x;
           event->xmotion.y = y;
           event->xmotion.x_root = rx;
           event->xmotion.y_root = ry;
           event->xmotion.is_hint = FALSE;
        }

	/*
	 *  Write the actual event structure to be played back.
	 */
	write_event(event);
	otime = ttime;
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_start_journal
|
|       Purpose: Initializes the application for journal playback 
|                or journal record.
|
|         Input: none
|    Written By: Mark Young & Danielle Argiro
|          Date: Jul 9, 1992 
|
------------------------------------------------------------*/
void xvw_start_journal(
    char *playback,
    char *record)
{
	unsigned char type;

	/*
	 *  Check to see if the user wants to start journal playback.
	 */
	if (playback != NULL)
	{
	   if ((playback_file = kfopen(playback, "r")) != NULL)
	   {
	      kfread(&type, (size_t) 1, (size_t) 1, playback_file);
	      kfile_setmachtype(kfileno(playback_file), type);

	      if (kfile_ispermanent(playback_file))
	      {
	         XtAppAddTimeOut(xvw_appcontext(NULL), (unsigned long) 0, 
			      file_playback, NULL);
	      }
	      else
	      {
		 int ifid = 0;

		 kgetdescriptors(playback_file, &ifid, NULL);
	         InputId = XtAppAddInput(xvw_appcontext(NULL), ifid,
			(XtPointer) XtInputReadMask, input_playback, NULL);
	      }
	   }
	   else
	   {
	      kfprintf(kstderr, "xvw_start_journal:  Error! Unable to open "
		     "journal playback file '%s'\n", playback);
	   }
	}

	/*
	 *  Check to see if the user wants to start a journal recording.
	 */
	if (record != NULL)
	{
	   if ((record_file = kfopen(record, "w")) != NULL)
	   {
	      type = kfile_getmachtype(kfileno(record_file));
	      kfwrite(&type, (size_t) 1, (size_t) 1, record_file);
	   }
	   else
	   {
	      kfprintf(kstderr, "xvw_start_journal:  Error! Unable to open "
		     "journal record file '%s'\n", record);
	   }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_stop_journal
|
|       Purpose: Stops any journal record/playback by closing the
|		 record_file and/or playback_file.
|
|    Written By: Mark Young & Danielle Argiro
|          Date: May 3, 1997
|
------------------------------------------------------------*/
void xvw_stop_journal(void)
{
	/*
	 *  Check to see if the journal record is active.
	 */
	if (record_file != NULL)
	{
	   kfclose(record_file);
	   record_file = NULL;
	}

	/*
	 *  Check to see if the journal playback is active.
	 */
	if (playback_file != NULL)
	{
	   kfclose(playback_file);
	   playback_file = NULL;

	   /*
	    *  Remove the input handler if journal playback was using the
	    *  input playback routine!
	    */
	   if (InputId != 0)
	   {
	      XtRemoveInput(InputId);
	      InputId = 0;
	   }
	}
}
