 /*
  * Khoros: $Id: process.c,v 1.19.2.1 1997/05/04 16:20:04 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: process.c,v 1.19.2.1 1997/05/04 16:20:04 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>  Private:
   >>>>                 xvw_sleep()
   >>>>                 xvw_waitpid()
   >>>>                 xvw_fork()
   >>>>                 xvw_spawn()
   >>>>                 xvw_system()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/*-----------------------------------------------------------
|
|  Routine Name: xvw_sleep
|
|       Purpose: The purpose of this routine is to sleep
|                for the desired number of seconds or until
|                there is input pending from the X server
|		 ie) XtPending().
|
|         Input: amount -  integer corresponding to the process
|    Written By: Mark Young
|          Date: Nov 15, 1993
|
------------------------------------------------------------*/
static int timer_done = FALSE;

/*ARGSUSED*/
static void timeout_handler(
   XtPointer     client_data, 
   XtIntervalId  *id)
{
	timer_done = TRUE;
}

int xvw_sleep(
   double amount)
{
	unsigned long elapsed = (unsigned long) (amount * 1000);

	timer_done = FALSE;
	XtAppAddTimeOut(xvw_appcontext(NULL), elapsed, timeout_handler, NULL);

	while (timer_done == FALSE)
	   xvw_process_event();

	return(TRUE);
}

/*-------------------------------------------------------------
|
|  Routine Name: xvw_waitpid -
|
|      Purpose:  The purpose of this routine is to wait until
|		 a certain process is done while still processing
|		 certain X Events.
|
|        Input:  pid -  integer corresponding to the process
|			we wish to check.
|       Output:  returns 0 upon suceesful completion of process
|		 pid, or the status if the process abnormally aborted
|   Written By:   Mark Young
|
-----------------------------------------------------------*/
/*ARGSUSED*/
int xvw_waitpid(
   pid_t pid)
{
	int status = 0;

	/*
	 *  Check to see if the process is done.  If not then sleep for
	 *  1 second, then check again.  Repeat the process until the process
	 *  is done.
	 */
	do {
	    xvw_sleep(1.0);
	} while (waitpid(pid, &status, WNOHANG) != -1);

	/*
	 *  Race thru the entry list looking for our pid, if it's in the
	 *  list then return the status, otherwise return NULL.
	 */
	(void) xvw_remove_handler(pid);
	return(status);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_fork
|
|       Purpose: forks a child.  The child executes a routine based 
|                on the command, while the parent continues doing 
|                cantata programming stuff.
|
|         Input: command     -
|                routine     - 
|                client_data -
|    Written By: Mark Young
|          Date: Jun 15, 1993
|
------------------------------------------------------------*/
pid_t xvw_fork(
   char *command,
   void (*dispatch) PROTO((int, kaddr)),
   void (*message) PROTO((kfile *, kaddr, char *)),
   void (*process) PROTO((kfile *, char *, char *, kaddr)),
   kaddr client_data)
{
	pid_t pid;
	kfile *file;
	char  *type;

	/*
	 *  Create the process by using kpopen().  The "Z" says that we
	 *  want to be able to write to the process as well as read from
	 *  the child's standard output and standard error messages.  It
	 *  also says to enable subprocess IPC.
	 */
	if (process != NULL)
	   type = "Z";
	else
	   type = "r+";

	if ((file = kpopen(command, type)) == NULL)
	{
	   char *temp = kfullpath(command, NULL, NULL);
	   kerror("xvwidgets", "xvw_fork", 
		  "Cannot execute the following command:\n'%s'", temp);
	   kfree(temp);
	   return(-1);
	}
	pid = xvw_add_handler(dispatch, message, process, client_data, file);
	return(pid);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_system
|
|       Purpose: The child executes the routine in foreground based on the
|                command passed in.  Call xvw_fork to execute
|                the routine then wait until the process
|                finishes before returning.
|
|         Input: command 
|       Returns: none
|    Written By: Mark Young
|          Date: Jun 15, 1993
|
------------------------------------------------------------*/
pid_t xvw_spawn(
   char *command)
{
	pid_t pid;
	kfile *file;

	/*
	 *  Create the process by using kpopen().
	 */
	if ((file = kpopen(command, "r+")) == NULL)
	{
	   char *temp = kfullpath(command, NULL, NULL);
	   kerror("xvwidgets", "xvw_spawn", "Error! Cannot execute the "
		  "following command\n\n'%s'.", temp);
	   kfree(temp);
	   return(-1);
	}
	pid = xvw_add_handler(NULL, NULL, NULL, NULL, file);
	return(pid);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_system
|
|       Purpose: The child executes the routine in foreground based on the
|                command passed in.  Call xvw_fork to execute
|                the routine then wait until the process
|                finishes before returning.
|
|         Input: command 
|       Returns: none
|    Written By: Mark Young
|          Date: Jun 15, 1993
|
------------------------------------------------------------*/

int xvw_system(
   char *command)
{
	int pid, status;

	/*
	 *  Create the process by using kspawn().
	 */
	if ((pid = kspawn(command)) == -1)
	{
	   char *temp = kfullpath(command, NULL, NULL);
	   kerror("xwidgets", "xvw_system", "Error! Cannot execute the "
		"following command\n\n'%s'.", temp);
	   kfree(temp);
	   return(-1);
	}
	status = xvw_waitpid(pid);
	return(status);
}
