 /*
  * Khoros: $Id: shell.c,v 1.13.6.4 1997/05/04 16:20:05 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: shell.c,v 1.13.6.4 1997/05/04 16:20:05 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Common xvobject Creation and Utilities
   >>>>		
   >>>>   Static:
   >>>>                 set_position()
   >>>>                 delete_toplevel()
   >>>>                 initialize_attributes()
   >>>>   Public:
   >>>>                 xvw_create_application_shell()
   >>>>                 xvw_create_transient_shell()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


static xvattribute shell_attributes[] = {
{XVW_SHELL_INPUT,       XtNinput,            XtRBoolean,    NULL},
{XVW_SHELL_TITLE,       XtNtitle,            XtRString,     NULL},
{XVW_SHELL_X,           XtNx,                XtRPosition,   NULL},
{XVW_SHELL_Y,           XtNy,                XtRPosition,   NULL},
{XVW_SHELL_RESIZE,      XtNallowShellResize, XtRBoolean,    NULL},
{XVW_SHELL_ICON_NAME,   XtNiconName,         XtRString,     NULL},
{XVW_SHELL_ICON_WINDOW, XtNiconWindow,       XtRWindow,     NULL},
{XVW_SHELL_ICON_PIXMAP, XtNiconPixmap,       XtRPixmap,     NULL},
{XVW_SHELL_ICON_PIXMAPFILE, XtNiconPixmap,   XtRString,     XtRPixmap},
{XVW_SHELL_ICON_MASK,     XtNiconMask,       XtRPixmap,     NULL},
{XVW_SHELL_ICON_MASKFILE, XtNiconMask,       XtRString,     XtRPixmap},
{XVW_SHELL_WIN_GRAVITY,   XtNwinGravity,     XtRInt,        NULL},
{XVW_SHELL_INITIAL_STATE, XtNinitialState,   XtRInt,        NULL},
{XVW_CHILDREN,            NULL,              XtRWidgetList, NULL},
{XVW_NUM_CHILDREN,        NULL,              XtRInt,        NULL},
};

/*-----------------------------------------------------------
|
|  Routine Name: set_position - set the toplevel position..
|
|       Purpose: This sets the toplevel position.
|
|         Input: toplevel - the toplevel object
|                attribute - the attribute (shell xpositon or shell yposition)
|                calldata - the value to be set
|
|    Written By: Mark Young
|          Date: Jul 12, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static int set_position(
    xvobject toplevel,
    char     *attribute,
    kaddr    calldata)
{
	int      i;
	Position x, y;
	Arg	 args[10];
	char	 temp[KLENGTH];
	Widget   widget = xvw_widget(toplevel);

        i = 0;
        XtSetArg(args[i], XtNx, &x);           i++;
        XtSetArg(args[i], XtNy, &y);           i++;
	XtGetValues(widget, args, (Cardinal) i);

	if (kstrcmp(attribute, XVW_SHELL_X) == 0)
	   x = *((int *) calldata);
	else if (kstrcmp(attribute, XVW_SHELL_Y) == 0)
	   y = *((int *) calldata);
	else
	   return(FALSE);

	/*
	 *  Set it the geometry way to force a position mapping...
	 */
	ksprintf(temp, "%+d%+d", x, y);

        i = 0;
        XtSetArg(args[i], XtNx, x);           i++;
        XtSetArg(args[i], XtNy, y);           i++;
        XtSetArg(args[i], XtNgeometry, kstrdup(temp)); i++;
	XtSetValues(widget, args, (Cardinal) i);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: delete_toplevel - delete window callback
|
|       Purpose: This routine destroys the shell since a delete
|		 window has been called.  When a "WM_DELETE_WINDOW"
|		 is detected then we simply destroy the toplevel
|		 rather than default, which is to exit the program.
|		 If the programmer wishes to do something else then
|		 they can use xvw_add_protocol().
|
|         Input: object      - the shell object to be deleted
|                client_data - unused
|    Written By: Mark Young
|          Date: Jun 28, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void delete_toplevel(
   xvobject toplevel,
   kaddr  client_data)
{
	xvw_unmap(toplevel);
}

/*-----------------------------------------------------------
|
|  Routine Name: initialize_attributes - initializes attributes for all shells
|
|       Purpose: This routine initializes attributes for all shell classes.
|    Written By: Mark Young
|          Date: May 1, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void initialize_attributes(void)
{
	static int initialized = FALSE;

	if (initialized == TRUE)
	   return;

	initialized = TRUE;
	xvw_init_attributes(applicationShellWidgetClass, shell_attributes,
			       (int) knumber(shell_attributes), NULL, 0, NULL);
	xvw_init_attributes(transientShellWidgetClass, shell_attributes,
			       (int) knumber(shell_attributes), NULL, 0, NULL);
	xvw_init_attributes(overrideShellWidgetClass, shell_attributes,
			       (int) knumber(shell_attributes), NULL, 0, NULL);
	xvw_define_attributes(applicationShellWidgetClass,
	      XVW_SHELL_X, XtRInt, set_position, NULL,
	      XVW_SHELL_Y, XtRInt, set_position, NULL,
	      NULL);
}

/************************************************************
*
*  Routine Name: xvw_create_application_shell() - create an application 
*                                                 shell object
*
*       Purpose: Creates an application shell object to serve as
*                a toplevel for an application; the toplevel serves
*                as a mediating device between the application and
*                the window manager.  This call allows the application 
*                to have several independant windows, which is the case 
*                with xvroutines having multiple subforms.  The routine
*                also supports the creation of toplevels on potentially 
*                different screens, which applies when a Khoros application 
*                distributes its user interface with the concert program.  
*                Both the display and screen are optional arguments;  if 
*                either of these are set to NULL, then the default display 
*                and default screen will be used.
*
*         Input: name    - name for the application shell object
*		 display - the X Display structure; pass NULL to use the
*			   default display
*                screen  - the X Screen structure; pass NULL to use the
*			   default screen
*
*        Output: 
*	Returns: returns the application shell object on success,
*		 NULL on failure
*
*  Restrictions:
*    Written By: Danielle Argiro & Mark Young
*          Date: Apr 9, 1993
*      Verified: 
*  Side Effects:
* Modifications:
*   Declaration: xvobject xvw_create_application_shell(
*                !   char    *name,
*                !   Display *display,
*                !   Screen  *screen)
*
*******************************************************************/

xvobject xvw_create_application_shell(
   char    *name,
   Display *display,
   Screen  *screen)
{
        Widget   shell;
	xvobject object;
	char     **argv;
        int      i, argc;
        Arg      args[MAX_ARGS];

	/*
	 *  The screen and display are optional since we have already
	 *  initialized the display.  Therefore if NULL then the defaults
	 *  are used.
	 */
	if (display == NULL) display = xvw_display(NULL);
	if (screen  == NULL) screen  = DefaultScreenOfDisplay(display);

	/* get command-line arguments */
	argc = kprog_get_argc();
	argv = kprog_get_argv();

        i = 0;
        XtSetArg(args[i], XtNscreen, screen);           i++;
        XtSetArg(args[i], XtNargc, argc);               i++;
        XtSetArg(args[i], XtNargv, argv);               i++;
        XtSetArg(args[i], XtNinput, TRUE);              i++;
        XtSetArg(args[i], XtNallowShellResize, TRUE);   i++;
        shell = XtAppCreateShell(name, kprog_get_program(), 
				 applicationShellWidgetClass,
				 display, args, (Cardinal) i);

	/* add toplevel to xvw_toplevels list so that when jp is used,
           that list is searched first */
	initialize_attributes();
	object = xvw_object(shell);
	xvw_add_protocol(object, "WM_DELETE_WINDOW", delete_toplevel, NULL);
	object->id = XtAppAddTimeOut(xvw_appcontext(object), (unsigned long) 0,
				xvw_manage_handler, (XtPointer) object);
	xvw_add_toplevel(object);
        return(object);
}

/************************************************************
*
*  Routine Name: xvw_create_transient_shell() - create a transient shell object
*
*       Purpose: Creates an transient shell object to serve as
*                the toplevel for a popup object.  Both the display and 
*                the screen are optional arguments;  if either is set to 
*                NULL, then the default display and default screen are used.
*
*         Input: name    - name for the transient shell object
*                display - the X Display structure; pass NULL to use the
*			   default display
*                screen  - the X Screen structure; pass NULL to use the
*			   default screen
*        Output: 
*	Returns: the transient shell object on success,
*		 NULL on failure
*
*  Restrictions:
*    Written By: Danielle Argiro & Mark Young
*          Date: Apr 9, 1993
*      Verified: 
*  Side Effects:
* Modifications:
*   Declaration: xvobject xvw_create_transient_shell(
*                !   char    *name,
*                !   Display *display,
*                !   Screen  *screen)
*
*******************************************************************/

xvobject xvw_create_transient_shell(
   char    *name,
   Display *display,
   Screen  *screen)
{
	int	 i;
	Widget	 shell;
	xvobject object;
	Arg	 args[MAX_ARGS];

	/*
	 *  The screen and display are optional since we have already
	 *  initialized the display.  Therefore if NULL then the defaults
	 *  are used.
	 */
	if (display == NULL) display = xvw_display(NULL);
	if (screen  == NULL) screen  = DefaultScreenOfDisplay(display);

	i = 0;
	XtSetArg(args[i], XtNscreen,       screen);        i++;
	XtSetArg(args[i], XtNinput,        TRUE);          i++;
	XtSetArg(args[i], XtNallowShellResize, TRUE);      i++;
	XtSetArg(args[i], XtNwindowGroup,  xvw_rootwindow(NULL)); i++;
	shell = XtAppCreateShell(name, kprog_get_program(), 
				 transientShellWidgetClass,
				 display, args, (Cardinal) i);

	/* add toplevel to xvw_toplevels list so that when jp is used,
           that list is searched first */
	initialize_attributes();
	object = xvw_object(shell);
	xvw_add_protocol(object, "WM_DELETE_WINDOW", delete_toplevel, NULL);
	object->id = XtAppAddTimeOut(xvw_appcontext(object), (unsigned long) 0,
				xvw_manage_handler, (XtPointer) object);
	xvw_add_toplevel(object);
        return(object);
}

/************************************************************
*
*  Routine Name: xvw_create_popup_shell() - create a popup shell object
*
*       Purpose: Creates a popup shell object to serve as
*                the toplevel for a popup object.  Both the display and 
*                the screen are optional arguments;  if either is set to 
*                NULL, then the default display and default screen are used.
*
*         Input: name    - name for the transient shell object
*                display - the X Display structure; pass NULL to use the
*			   default display
*                screen  - the X Screen structure; pass NULL to use the
*			   default screen
*        Output: 
*	Returns: the popup shell object on success, NULL on failure
*
*  Restrictions:
*    Written By: Mark Young & Danielle Argiro
*          Date: May 1, 1996
*      Verified: 
*  Side Effects:
* Modifications:
*   Declaration: xvobject xvw_create_popup_shell(
*                !   char    *name,
*                !   Display *display,
*                !   Screen  *screen)
*
*******************************************************************/

xvobject xvw_create_popup_shell(
   xvobject parent,
   char    *name)
{
	int	 i;
	Widget	 shell;
	xvobject object;
	Arg	 args[MAX_ARGS];

	/*
	 *  The screen and display are optional since we have already
	 *  initialized the display.  Therefore if NULL then the defaults
	 *  are used.
	 */
	i = 0;
	XtSetArg(args[i], XtNinput,            TRUE);      i++;
	XtSetArg(args[i], XtNallowShellResize, TRUE);      i++;
	XtSetArg(args[i], XtNsaveUnder,  TRUE);		   i++;
	XtSetArg(args[i], XtNwindowGroup,  xvw_rootwindow(NULL)); i++;
	shell = XtCreatePopupShell(name, overrideShellWidgetClass,
			 xvw_widget(parent), args, (Cardinal) i);

	/* add toplevel to xvw_toplevels list so that when jp is used,
           that list is searched first */
	initialize_attributes();
	object = xvw_object(shell);
        return(object);
}
