 /*
  * Khoros: $Id: toplevel.c,v 1.9 1997/01/20 05:33:26 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: toplevel.c,v 1.9 1997/01/20 05:33:26 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Private Utility Routines
   >>>>		
   >>>>  Private:
   >>>>                 xvw_add_toplevel()
   >>>>                 xvw_remove_toplevel()
   >>>>                 xvw_set_toplevel()
   >>>>                 xvw_get_toplevel()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


/*--------------------------------------------------------------
|
|  Routine Name: xvw_add_toplevel - add a toplevel object
|
|       Purpose: Every time a new toplevel object is created by one
|		 of the routines in xvutils, it must be added to the
|		 xvw_toplevels list so that when journal playback is used,
|		 this list is searched first for toplevel objects.  This makes
|		 journal playback more reliable and efficient.
|
|         Input: toplevel - the toplevel object created by
|			    xvw_create_application_shell(),
|			    xvw_create_transient_shell().
|        Output:
|	Returns:
|    Written By: Mark Young
|          Date: Oct 20, 1993
| Modifications: 
|
----------------------------------------------------------------*/

void xvw_add_toplevel(
   xvobject toplevel)
{
	/*
	 *  the node to destroy is first in the list
	 */
	xvw_toplevels = klist_add(xvw_toplevels, toplevel, NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_remove_toplevel - remove a toplevel object
|
|       Purpose: Every time a toplevel object is destroyed by one of the
|                exiting callbacks in xvutils, we must delete it  from the
|                xvw_toplevels list.  The toplevel object was created by the
|                xvw_create_application_shell(), xvw_create_transient_shell().
|                The toplevel is now being destroyed by the destroy callback
|                that requires that we delete it out of the toplevel list.
|
|         Input: toplevel - the toplevel remove created by
|			    xvw_create_application_shell(),
|			    xvw_create_transient_shell().
|
|        Output:
|       Returns:
|    Written By: Mark Young
|          Date: Oct 20, 1993
| Modifications:
|
------------------------------------------------------------*/

void xvw_remove_toplevel(
   xvobject toplevel)
{
        /*
         *  the node to destroy is first in the list
         */
        xvw_toplevels = klist_delete(xvw_toplevels, toplevel);

	/*
	 *  make sure the toplevel isn't busy, since the busy support won't
	 *  maintain this anymore.
	 */
	if (toplevel->busy != NULL)
	   xvw_set_attribute(toplevel->busy, XVW_BUSY, FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_set_toplevel - allows the setting and unsetting of an
|				    objects toplevel
|
|       Purpose: This routine can be used to set and unset the objects
|                toplevel.  This can be useful for objects like the
|		 TitleBar which may want to temporarily reassign the
|		 toplevel according to whether the object is maximized
|		 or minimized.
|
|         Input: object - the object in which to set the toplevel
|                toplevel - the toplevel or NULL for the object
|    Written By: Mark Young
|          Date: Apr 05, 1996
|
------------------------------------------------------------*/
void xvw_set_toplevel(
   xvobject object,
   xvobject toplevel)
{
	object->toplevel = toplevel;
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_get_toplevel - allows the getting of an objects toplevel
|
|       Purpose: This routine can be used to get the objects
|                toplevel.  This can be useful for objects like the
|		 TitleBar which may want to temporarily reassign the
|		 toplevel according to whether the object is maximized
|		 or minimized.
|
|         Input: object - the object in which to get the toplevel
|    Written By: Mark Young
|          Date: Apr 05, 1996
|
------------------------------------------------------------*/
xvobject xvw_get_toplevel(
   xvobject object)
{
	return(object->toplevel);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_getall_toplevels - gets all the toplevel objects
|
|       Purpose: This routine can be used to get all the toplevel objects.
|                This can be useful for routines like xvforms which needs
|		 to destroy all active toplevels before returning from
|		 xvf_run_form().
|
|    Written By: Mark Young
|          Date: Apr 05, 1996
|
------------------------------------------------------------*/
xvobject *xvw_getall_toplevels(
   size_t *num)
{
	xvobject *objects;

	objects = (xvobject *) klist_to_array(xvw_toplevels, num);
	return(objects);
}
