 /*
  * Khoros: $Id: cantata.c,v 1.27.2.3 1997/05/31 18:31:46 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: cantata.c,v 1.27.2.3 1997/05/31 18:31:46 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for cantata
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "cantata.h"

clui_info_struct *clui_info = NULL;

gui_info_struct *gui_info;

/*-----------------------------------------------------------
|
|  Routine Name: main() - The visual programming language for Khoros
|
|       Purpose: main program for cantata
|
|         Input:
|		int clui_info->mini_flag; {TRUE if -mini specified}
|
|		int clui_info->tbmenus_logic; {Create Cantata Pulldown Toolbox Menus}
|		int clui_info->tbmenus_flag; {TRUE if -tbmenus specified}
|
|		int clui_info->commandbar_logic; {Create Cantata Command Bar Menus}
|		int clui_info->commandbar_flag; {TRUE if -commandbar specified}
|
|		int clui_info->console_logic; {Create Cantata Console}
|		int clui_info->console_flag; {TRUE if -console specified}
|
|		Optional M.E. group:
|		    char *clui_info->wksp_file; {file for restoring cantata workspace}
|		    int   clui_info->wksp_flag; {TRUE if -wksp specified}
|
|		    M.I. group:
|			char *clui_info->tb_string; {the toolbox in which the encapsulated workspace}
|			int   clui_info->tb_flag; {TRUE if -tb specified}
|
|			char *clui_info->oname_string; {The encapsulated workspace object name}
|			int   clui_info->oname_flag; {TRUE if -oname specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

	kform  *form;            /* form tree representing UIS        */
	char   *form_fullpath;   /* for use when [-form] is specified */
/* -main_variable_list */
/* -main_variable_list_end */

	khoros_init(argc, argv, "DESIGN", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$DESIGN/objects/xvroutine/cantata");
	kexit_handler(cantata_free_args, NULL);

/* -main_get_args_call */
	kclui_init("DESIGN", "cantata", KGEN_XVROUTINE,  &clui_uis_spec,
		   cantata_usage_additions, cantata_get_args,
 		   cantata_free_args);
 	xvf_set_pulldown_parent_class(ToolboxMenuWidgetClass);
/* -main_get_args_call_end */

	/*
	 * Initialize xvwidgets lib
	 */
	if (!(xvw_initialize(XVW_MENUS_XVFORMS)))
	{
	     kinfo(KSTANDARD, "Could not open connection to server");
	     kexit(KEXIT_FAILURE);
	}

	/*
	 * Allocate the GUI Info structure 
	 */
	gui_info = (gui_info_struct *)
				kcalloc ((size_t) 1, sizeof(gui_info_struct));
	/*
	 * Create the forms for cantata 
	 */
	if (clui_info->form_flag)
	{
	    form_fullpath   = kfullpath(clui_info->form_file, NULL, NULL);
	    form = xvf_create_form(form_fullpath, KNONE, run_cantata, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	    kfree_and_NULL(form_fullpath);
	}
	else
	{
	    form = xvf_build_form(&gui_uis_spec, KNONE, run_cantata, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	}
	if (form == NULL)
	   kexit(KEXIT_FAILURE);

	/*
	 * Initialize the GUI Info structure 
	 */
	if (!(_xvf_init_cantata(form, gui_info)))
	    kexit(KEXIT_FAILURE);

/* -main_before_lib_call */
	/*
	 *  Busy the toplevel while we build things..
	 */
	xvf_add_extra_call(gui_info->quit_struct, cantata_exit, NULL,
			   XVF_CALL_SUBSTITUTE);
	initialize_cantata();
	cantata_form = form;

	/* 
	 * instead of just setting XVW_NOTIFYWINDOW_VISIBLE to false,
	 * we put a timeout, so that the notify window won't go down
	 * until the forms are mapped
	 */
	xvw_add_timeout(NULL, 0.1, notify_unmap, NULL);
/* -main_before_lib_call_end */

/* -main_library_call */
	xvf_run_form();
/* -main_library_call_end */

/* -main_after_lib_call */
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: cantata_usage_additions
| 
|       Purpose: Prints usage additions in cantata_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname cantata
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void cantata_usage_additions(void)
{
	kfprintf(kstderr, "\tThe visual programming language for Khoros\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: cantata_free_args
| 
|       Purpose: Frees CLUI struct allocated in cantata_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname cantata
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
cantata_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->form_file);
		kfree_and_NULL(clui_info->wksp_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
