 /*
  * Khoros: $Id: do_dist.c,v 1.15.2.1 1997/04/29 22:41:06 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_dist.c,v 1.15.2.1 1997/04/29 22:41:06 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for pane dist
   >>>> 
   >>>>  Private: 
   >>>> 	update_distcomp_info
   >>>> 	dist_remote_exec
   >>>> 	dist_start_daemon
   >>>> 	dist_stop_daemon
   >>>> 	dist_delete_remote
   >>>> 	dist_add_name
   >>>> 
   >>>>   Static: 
   >>>> 	set_distcomp_activate
   >>>> 	strip_status_string
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "cantata.h"

static char *on_str  = "<daemon running>";
static char *off_str = "<no daemon present>";


static void set_distcomp_activate PROTO((distcomp_dist *, int));
static void strip_status_string   PROTO((char *, char *));


/*-----------------------------------------------------------
|
|  Routine Name: dist_remote_exec
|
|       Purpose: Do routine which is called when
|                  toggle selection remote_exec is used
|
|         Input: dist_info - ptr to PaneInfo struct for dist pane
|
|        Output: None
|    Written By: Danielle Argiro
|          Date: June 27, 1996
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void dist_remote_exec(
     distcomp_dist *dist_info)
{
        if (dist_info->remote_exec_num == 1)
	{
           xvw_set_attribute(main_workspace, XVW_NODE_REMOTE_ENABLE, TRUE);
	   set_distcomp_activate(gui_info->distcomp->dist, TRUE);
	}
        else
	{
           xvw_set_attribute(main_workspace, XVW_NODE_REMOTE_ENABLE, FALSE);
	   set_distcomp_activate(gui_info->distcomp->dist, FALSE);
	}

}

/*-----------------------------------------------------------
|
|  Routine Name: update_distcomp_info
|
|       Purpose: extra call on distributed computing subform. 
|
|         Input: client_data - textdisplay object on the dist. computing subform
|
|    Written By: Mark Young & Danielle Argiro
|          Date: June 27, 1996
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void update_distcomp_info(
    kaddr    client_data)
{
	int    i, num, enabled; 
	size_t host_num;
	char   **machines;
	char   **machine_list;
	char   temp[KLENGTH];

	/*
	 * get list of remote machines for distributed computing
	 */
	machines = khost_list(&host_num);

	num = (int) host_num;
	xvf_set_attribute(gui_info->distcomp->dist->remote_hosts_struct,
                          XVF_LIST_SIZE,  num);

	if (num == 0) return;

	machine_list = (char **) kcalloc((size_t) num, sizeof(char *));

	for (i = 0; i < num; i++)
    	{
	    if (kremote_running(machines[i]))
	  	ksprintf(temp,  "%s %s", machines[i], on_str);
	    else ksprintf(temp, "%s %s", machines[i], off_str);
	    machine_list[i] = kstrdup(temp);
    	}
	xvf_set_attributes(gui_info->distcomp->dist->remote_hosts_struct,
			   XVF_LIST_SIZE,     num,
			   XVF_LIST_CONTENTS, machine_list,
			   NULL);

	xvw_get_attribute(main_workspace, XVW_NODE_REMOTE_ENABLE, &enabled);
	set_distcomp_activate(gui_info->distcomp->dist, enabled);
}

/*-----------------------------------------------------------
| 
|  Routine Name: dist_start_daemon
| 
|       Purpose: Do routine which is called when
|                  pane action button start_daemon is used
| 
|         Input: dist_info - ptr to PaneInfo struct for dist pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: June 27, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void dist_start_daemon(
     distcomp_dist *dist_info)
{
	char    location[KLENGTH];
	char    user[KLENGTH];
	char	remote_host[KLENGTH];
	char	temp[KLENGTH];
	char	command[KLENGTH];

	if (kstrlen(dist_info->remote_hosts_label) == 0)
	{
            kerror(NULL, "dist_start_daemon", "You must select a remote host "
			 "from the list on which to start daemon.");
            kinfo(KHOSTILE, "idiot!");
            return;
	}


	/*
         * get remote host, location and user for future reference in messages
	 */
	strip_status_string(dist_info->remote_hosts_label, remote_host);
	(void) khost_location(remote_host, location);
	(void) khost_username(remote_host, user);
        (void) khost_remotecmd(remote_host, command);

	/*
	 * protection against starting multiple daemons	
	 */
	if (kremote_running(remote_host))
	{
	    kinfo(KSTANDARD, "There is already a daemon started on remote machine %s, user %s", location, user);
	    return;
	}

	/*
	 * start the daemon
	 */
	if (!kremote_start(remote_host))
	{
            kerror(NULL, "dist_start_daemon",
                   "Host '%s' may not be valid, user '%s' may not be valid, "
		   "the command '%s' may not be valid, or there is no .rhosts "
                   "file allowing remote access.", location, user, command);
            return;
	}

	/* 
	 * print out a nice message 
	 */
	kinfo(KSTANDARD, "Daemon started on remote machine %s, user %s",
              location, user);

	/*
	 * update Available Remote Hosts list with new status
	 */
	ksprintf(temp, "%s %s", remote_host, on_str);
	kfree(dist_info->remote_hosts_label);
	dist_info->remote_hosts_label = kstrdup(temp);
	xvf_set_attribute(dist_info->remote_hosts_struct,
		          XVF_LIST_LABEL, temp);

	/*
	 * In case Remote Execution is still set to Disabled, automatically
	 * enable it.
	 */
        xvw_set_attribute(main_workspace, XVW_NODE_REMOTE_ENABLE, TRUE);
        dist_info->remote_exec_num = 1;
	ksprintf(temp, "%d", dist_info->remote_exec_num);
	xvf_set_attribute(dist_info->remote_exec_struct, XVF_TOGGLE_VAL, temp);
}


/*-----------------------------------------------------------
| 
|  Routine Name: dist_stop_daemon
| 
|       Purpose: Do routine which is called when
|                  pane action button stop_daemon is used
| 
|         Input: dist_info - ptr to PaneInfo struct for dist pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: June 27, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void dist_stop_daemon(
     distcomp_dist *dist_info)
{
        char    location[KLENGTH];
        char    user[KLENGTH];
        char    remote_host[KLENGTH];
        char    temp[KLENGTH];

        if (kstrlen(dist_info->remote_hosts_label) == 0)
        {
            kerror(NULL, "dist_start_daemon", "You must select a remote host from the list on which to stop daemon.");
            kinfo(KHOSTILE, "idiot!");
            return;
        }

        /*
         * get remote host, location and user for future reference in messages
         */
	strip_status_string(dist_info->remote_hosts_label, remote_host);
        (void) khost_location(remote_host, location);
        (void) khost_username(remote_host, user);

        /*
         * can't stop daemon if there isn't one running
         */
        if (!kremote_running(remote_host))
        {
            kinfo(KSTANDARD, "No daemon is currently running on remote machine %s, user %s", location, user);
            return;
        }

        /*
         * stop the daemon
         */
        kremote_stop(remote_host);

        /*
         * print out a nice message
         */
        kinfo(KSTANDARD, "Daemon stopped on remote machine %s, user %s",
              location, user);

        /*
         * update Available Remote Hosts list with new status
         */
        ksprintf(temp, "%s %s", remote_host, off_str);
	kfree(dist_info->remote_hosts_label);
	dist_info->remote_hosts_label = kstrdup(temp);
        xvf_set_attribute(dist_info->remote_hosts_struct,
                          XVF_LIST_LABEL, temp);

}


/*-----------------------------------------------------------
| 
|  Routine Name: dist_delete_remote
| 
|       Purpose: Do routine which is called when
|                  pane action button delete_remote is used
| 
|         Input: dist_info - ptr to PaneInfo struct for dist pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: June 27, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void dist_delete_remote(
     distcomp_dist *dist_info)
{
        char    remote_host[KLENGTH];
        char    list_string[KLENGTH];
        char  **list_contents;
        int     i, size;

        if (kstrlen(dist_info->remote_hosts_label) == 0)
        {
            kerror(NULL, "dist_start_daemon", "You must select a remote host from the list to delete.");
            kinfo(KHOSTILE, "idiot!");
            return;
        }
	ksprintf(list_string, "%s", dist_info->remote_hosts_label);
	strip_status_string(dist_info->remote_hosts_label, remote_host);

	/*
	 * let them chicken out if they like
	 */
        if (kprompt(KSTANDARD, "Yes", "No", 1, "Are you sure you wish to "
                    "delete remote host '%s' from the list?", 
	            remote_host) == FALSE)
            return;

        /*
         * stop the daemon if it was running
         */
	if (kremote_running(remote_host))
            kremote_stop(remote_host);

	/*
	 *  delete remote host from list on GUI: this is a pain in the butt
	 *  sorry, but XVF_LIST_DELETE will cause an unaligned access on OSF,
	 *  so we have to do a workaround...
	 */
        xvf_get_attributes(dist_info->remote_hosts_struct,
                           XVF_LIST_CONTENTS, &list_contents,
                           XVF_LIST_SIZE,     &size,
                           NULL);

        for (i = 0; i < size; i++)
        {
            if (kstrcmp(list_contents[i], dist_info->remote_hosts_label) == 0)
            {
                list_contents = (char **) karray_delete(list_contents, KSTRING,
                                              list_contents[i], (size_t) size);
                size--;
                break;
            }
        }
	if (size == 0) list_contents = NULL;
        xvf_set_attributes(dist_info->remote_hosts_struct,
                           XVF_LIST_SIZE,     size,
                           XVF_LIST_CONTENTS, list_contents,
                           NULL);

	/*
	 * finally, actually delete the host from the remote hosts list
	 */
	khost_delete(remote_host);
}


/*-----------------------------------------------------------
| 
|  Routine Name: dist_add_name
| 
|       Purpose: Do routine which is called when
|                  string selection add_name is used
| 
|         Input: dist_info - ptr to PaneInfo struct for dist pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: June 27, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void dist_add_name(
     distcomp_dist *dist_info)
{
        char remote_host[KLENGTH];
        char machine[KLENGTH];
        char user[KLENGTH];
        char command[KLENGTH];
        char temp[KLENGTH];
	char **remote_host_list;
	int  i, remote_host_num;
 
        /*
         * check to make sure they've provided a valid host name
         */
	if (ksscanf(dist_info->add_name, "%[^:]:%[^:]:%s", 
		    machine, user, command) != 3)
	{
	    kerror(NULL, "dist_add_name",
              "String specified is not a valid remote machine specification. "
	      "Remote hosts must be specified with syntax 'machine:user:cmd', "
	      "as in 'brandy:paula:rsh'.");
            return;
	}

        if (!(kremote_check(dist_info->add_name)))
        {
            kerror(NULL, "dist_add_name",
                   "String specified, '%s', is not a valid remote machine.",
                   dist_info->add_name);
            return;
        }
 
	/*
	 * check to make sure the host name isn't a duplicate entry.
	 */
	xvf_get_attributes(dist_info->remote_hosts_struct, 
			  XVF_LIST_CONTENTS, &remote_host_list,
			  XVF_LIST_SIZE,     &remote_host_num,
			  NULL);

	for (i = 0; i < remote_host_num; i++)
	{
	    strip_status_string(remote_host_list[i], remote_host);
	    if (kstrcmp(dist_info->add_name, remote_host) == 0)
	    {
               kinfo(KSTANDARD, "Machine '%s' is already "
	             "in the remote hosts list", dist_info->add_name);
	       return;
	    }
	}

        /*
         *  add remote machine to host list
         */
	if (!kremote_start(dist_info->add_name))
	{
            kerror(NULL, "dist_add_name",
                   "Host '%s' may not be valid, user '%s' may not be valid, "
		   "the command '%s' may not be valid, or there is no .rhosts "
                   "file allowing remote access.", machine, user, command);
            return;
	}
	khost_add(dist_info->add_name);
	
        /*
         *  add remote machine to GUI displaylist - again working around
	 *  OSF crash when using XVF_LIST_ADD...
         */
	ksprintf(temp, "%s %s", dist_info->add_name, on_str);
	remote_host_list = (char **) karray_add(remote_host_list, KSTRING, 
				      kstrdup(temp), (size_t)remote_host_num);
	remote_host_num++;

	if (remote_host_num == 0) remote_host_list = NULL;
        xvf_set_attributes(dist_info->remote_hosts_struct,
                           XVF_LIST_SIZE,     remote_host_num,
                           XVF_LIST_CONTENTS, remote_host_list,
                           NULL);
}

/*-----------------------------------------------------------
| 
|  Routine Name: dist_save
| 
|       Purpose: Do routine which is called when
|                  pane action button save is used
| 
|         Input: dist_info - ptr to PaneInfo struct for dist pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Jul 01, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void dist_save(
     distcomp_dist *dist_info)
{
	char *filename;

	if ((filename = kgetenv("KHOROS_HOSTS")) == NULL)
	    filename = "$HOME/.khoros_hosts";
	if (koverwrite(KSTANDARD, filename))
	{
	    khost_save(filename);
	    kinfo(KSTANDARD, "Remote host list saved to file '%s'\n", filename);
	}

}


/*-----------------------------------------------------------
|
|  Routine Name: set_distcomp_activate
|
|       Purpose: This utility routine activates or deactivates
|                selections on the Remote Execution subform
|                according to whether remote execution is enabled or disabled
|
|         Input: dist_info - ptr to PaneInfo struct for dist pane
|                enable    - TRUE if remote execution is enabled, 
|                            FALSE otherwise
|        Output: None
|    Written By: Danielle Argiro
|          Date: Jul 01, 1996
| Modifications:
|
------------------------------------------------------------*/
static void set_distcomp_activate(
    distcomp_dist *dist_info,
    int            enable)
{
       xvf_set_attribute(dist_info->remote_hosts_struct,  XVF_ACTIVATE, enable);
       xvf_set_attribute(dist_info->add_name_struct,      XVF_ACTIVATE, enable);
       xvf_set_attribute(dist_info->delete_remote_struct, XVF_ACTIVATE, enable);
       xvf_set_attribute(dist_info->start_daemon_struct,  XVF_ACTIVATE, enable);
       xvf_set_attribute(dist_info->stop_daemon_struct,   XVF_ACTIVATE, enable);
       xvf_set_attribute(dist_info->save_struct,          XVF_ACTIVATE, enable);
}

/*-----------------------------------------------------------
|
|  Routine Name: strip_status_string
|
|       Purpose: The item from Available Remote Hosts display list selection
|                will look like:
|                      "brandy:mirage:rsh <no daemon present>" or
|                      "brandy:mirage:rsh <daemon started>"
|                so to get the "plain" remote host string we need to strip 
|                off the "<...>" comment before using the string
|
|         Input: status_string - "brandy:mirage:rsh <no daemon present>"
|        Output: strip_string  - "brandy:mirage:rsh" (must be allocated)
|    Written By: Danielle Argiro
|          Date: Jul 02, 1996
| Modifications:
|
------------------------------------------------------------*/
static void strip_status_string(
    char *status_string,
    char *strip_string)
{
	char *end;

        (void) kstring_cleanup(status_string, strip_string);
        end = kstrstr(strip_string, " <");
        if (end != NULL) *end = '\0';
}
