 /*
  * Khoros: $Id: do_variables.c,v 1.12.2.3 1997/06/26 18:14:42 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_variables.c,v 1.12.2.3 1997/06/26 18:14:42 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for pane variables
   >>>> 
   >>>>  Private: 
   >>>> 	variables_scope
   >>>> 	variables_varval
   >>>> 	variables_deletevar
   >>>> 	variables_expression
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "cantata.h"

/*-----------------------------------------------------------
| 
|  Routine Name: variables_scope
| 
|       Purpose: Do routine which is called when
|                  displaylist selection scope is used
| 
|         Input: variables_info - ptr to PaneInfo struct for variables pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Dec 13, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void variables_scope(
     var_variables *variables_info)
{
	xvobject wksp;


	if (workspace_list == NULL)
	   return;

	/*
	 *  Get the desired xvobject from the workspace_list, but
	 *  subtract 1, since the list starts at 1, not 0.
	 */
	wksp = workspace_list[variables_info->scope-1];
	update_varscope_displaylist(wksp);
}

/*-----------------------------------------------------------
| 
|  Routine Name: variables_varorig
| 
|       Purpose: Do routine which is called when
|                  displaylist selection varorig is used
| 
|         Input: variables_info - ptr to PaneInfo struct for variables pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Jul 25, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void variables_varorig(
     var_variables *variables_info)
{
	xvf_set_attribute(variables_info->expression_struct,
			  XVF_STRING_VAL, variables_info->varorig_label);
}

/*-----------------------------------------------------------
| 
|  Routine Name: variables_varval
| 
|       Purpose: Do routine which is called when
|                  displaylist selection varval is used
| 
|         Input: variables_info - ptr to PaneInfo struct for variables pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Dec 13, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void variables_varval(
     var_variables *variables_info)
{
	xvf_set_attribute(variables_info->expression_struct,
			  XVF_STRING_VAL, variables_info->varval_label);
}

/*-----------------------------------------------------------
| 
|  Routine Name: variables_deletevar
| 
|       Purpose: Do routine which is called when
|                  pane action button delete is used
| 
|         Input: variables_info - ptr to PaneInfo struct for variables pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Dec 13, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void variables_deletevar(
     var_variables *variables_info)
{
	long		expr_id;
	char            **variable_names;
	xvw_list_struct **list_return;
	int             i, num = 0;
	char            *prompt = "Select variable(s) to delete";
	char            *label  = "Variables";


	expr_id = (long) selected_workspace;

        /* get the variables defined in the scope of the selected workspace */
        variable_names = kexpr_variables_list(expr_id, 0, &num);

	/*
	 * no variables to delete 
	 */
	if (variable_names == NULL || num == 0)
	{
	    kerror(NULL, "variables_deletevar", "No variables to delete");
	    return;
	}

	/* 
	 * only one variable to delete 
 	 */
	if (num == 1)
	{
	    /* user changed their mind; return */
	    if (!(kprompt(KSTANDARD, NULL, NULL, 0, "Delete variable %s?",
		  variable_names[0]))) return;
	   
	    /* delete the variable from the selected workspace */
	    else kexpr_variable_delete(expr_id, variable_names[0]);
	}

	/* 
	 * many variables that could be deleted: display list of 
	 * variables defined in selected workspace and let them choose,
	 * then delete all chosen variables
	 */
	else 
	{
	    list_return = xvu_run_list_multsel_wait(variable_names, (size_t) 
					num, prompt, label, FALSE, FALSE, &num);
	
	    for (i = 0; i < num; i++)
	    {
	        /* delete specified variables from selected workspace */
	        kexpr_variable_delete(expr_id, 	list_return[i]->string);
	    }
	}

	/* update displaylist of variables & values */
	update_varval_displaylist();
	update_vardef_displaylist();
}
/*-----------------------------------------------------------
| 
|  Routine Name: variables_expression
| 
|       Purpose: Do routine which is called when
|                  string selection expression is used
| 
|         Input: variables_info - ptr to PaneInfo struct for variables pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 19, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void variables_expression(
     var_variables *variables_info)
{
	/* PUT YOUR CODE HERE ! */
	long	 expr_id;
        char     value[KLENGTH];
	
	expr_id = (long) selected_workspace;

	kexpr_set_define_mode(TRUE); /* change variable definition */
        if (kexpr_evaluate_string(expr_id, variables_info->expression,
				  value, NULL))
        {
           xvf_set_attribute(variables_info->blank2_struct, XVF_TITLE, value);
	   update_varval_displaylist();
	   update_vardef_displaylist();
        }
	else
           xvf_set_attribute(variables_info->blank2_struct, XVF_TITLE, NULL);
	kexpr_set_define_mode(FALSE);
}
