 /*
  * Khoros: $Id: utilities.c,v 1.30.2.4 1997/05/24 19:06:49 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.30.2.4 1997/05/24 19:06:49 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Routines For Dealing w/ Cantata Form
   >>>> 
   >>>>  Private: 
   >>>>			create_glyph()
   >>>>			status_callback()
   >>>>			update_status()
   >>>>			create_workspace()
   >>>>			change_workspace()
   >>>>			save_workspace()
   >>>>			restore_workspace()
   >>>>			change_procedure()
   >>>>
   >>>>			update_varval_displaylist()
   >>>>			update_vardef_displaylist()
   >>>>			update_varscope_displaylist()
   >>>>			
   >>>>			get_workspace_list()
   >>>>			
   >>>>			update_variables_subform_cb()
   >>>>			
   >>>>			init_pref_values()
   >>>>			init_menu_icons()
   >>>>			init_encap_wksp()
   >>>>			
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "cantata.h"

static void get_workspace_list PROTO((xvobject, char ***, xvobject **,
				      size_t *, size_t *, char *));

/*-----------------------------------------------------------
|
|  Routine Name: perform_action
|
|       Purpose: Performs the desired cantata action by calling the associated
|                action handler.
|         Input: object - the subcategory action
|		 client_data - the pane list
|		 event - the event that invoked the action handler
|        Output:
|       Returns:
|    Written By: Mark Young
|          Date: Apr 26, 1997
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void perform_action(
   xvobject object,
   kaddr    client_data,
   XEvent   *event)
{
	void (*routine)(kaddr) = (void (*)(kaddr)) client_data;

	if (!routine)
	   return;

	routine(NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: create_glyph
|
|       Purpose: Creates a operator according to the selection from the 
|                pulldown menu.
|         Input: object - the subcategory action
|		 client_data - the pane list
|		 call_data   - none
|        Output:
|       Returns:
|    Written By: Mark Young
|          Date: Oct 11, 1993
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void create_glyph(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject  node;
	xvw_program_info *info = (xvw_program_info *) call_data;

	if (!info)
	   return;

	if (info->type == XVL_PROCEDURE)
	{
	   node = xvw_create_procedure(active_workspace, "procedure");
	   xvw_set_attributes(node,
                XVW_NODE_NAME,      info->icon_name,
                XVW_GLYPH_TBNAME,   info->tbname,
                XVW_GLYPH_ONAME,    info->oname,
		XVW_PROCEDURE_RESTORE, info->workspace,
                NULL);
	}
	else if (info->type == XVL_CONDITIONAL)
	{
	   node = xvw_create_conditional(active_workspace, "conditional");
	   xvw_set_attributes(node, 
		XVW_NODE_NAME,	    info->icon_name,
		XVW_GLYPH_FORMFILE, info->panefile,
		XVW_GLYPH_TBNAME,   info->tbname,
		XVW_GLYPH_ONAME,    info->oname,
		NULL);
	}
	else if (info->type == XVL_LOOP)
	{
	   node = xvw_create_loop(active_workspace, "loop");
	   xvw_set_attributes(node, 
		XVW_NODE_NAME,	    info->icon_name,
		XVW_GLYPH_FORMFILE, info->panefile,
		XVW_GLYPH_TBNAME,   info->tbname,
		XVW_GLYPH_ONAME,    info->oname,
		NULL);
	}
	else
	{
	   node = xvw_create_glyph(active_workspace, "glyph");
	   xvw_set_attributes(node, 
		XVW_NODE_NAME,	    info->icon_name,
		XVW_GLYPH_FORMFILE, info->panefile,
		XVW_GLYPH_TBNAME,   info->tbname,
		XVW_GLYPH_ONAME,    info->oname,
		NULL);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: status_callback
|
|       Purpose: Updates the description from the currently selected
|		 form struct.  The description is updated
|		 
|         Input: object - the toolboxlist object
|		 client_data - the description kformstruct
|		 call_data   - the program object information
|        Output:
|       Returns:
|    Written By: Mark Young
|          Date: Jul 11, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void status_callback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	char title[KLENGTH];
	xvw_program_info *info = (xvw_program_info *) call_data;

	ksprintf(title, "%s - %s", info->oname, info->short_desc ?
			info->short_desc: "(unknown short description)");
	if (statusobj)  xvw_set_attribute(statusobj,  XVW_TEXT_STRING, title);
}

/*-----------------------------------------------------------
|
|  Routine Name: update_status
|
|       Purpose: Updates the status from the currently status info message.
|
|         Input: toolbox - the toolbox in which the message was issued
|		 program - the program in which the message was issued
|		 library - the library in which the message was issued
|		 routine - the routine in which the message was issued
|		 message - the message in which the message was issued
|        Output:
|       Returns:
|    Written By: Mark Young
|          Date: Jul 11, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int update_status(
   char *toolbox,
   char *program,
   char *library,
   char *routine,
   char *message)
{
        if (kstrlen(message) > 0)
	   message = kstring_cleanup(message, message);

	if (statusobj)  xvw_set_attribute(statusobj, XVW_TEXT_STRING, message);
	if (tooltips && xvw_check_managed(tooltips))
	    xvw_set_attribute(tooltips, XVW_TOOLTIPS_MESSAGE, message);

        return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: create_workspace
|
|       Purpose: This routine creates a new workspace within the notebook.
|
|    Written By: Mark Young
|          Date: Dec 08, 1994
|
|  Changes:
|    PH: set workspace attribute
|
------------------------------------------------------------*/
/* ARGSUSED */
void create_workspace(void)
{
	static int i = 1;
	char name[KLENGTH];

	/*
	 *  Create the main workspace which is used to build our visual program
	 */
	ksprintf(name, "Area%d", i++);
	main_workspace = xvw_create_workspace(notebook, name);
	xvw_set_attributes(main_workspace,
		XVW_CANVAS_WIDTH,  3000,
		XVW_CANVAS_HEIGHT, 3000,
		XVW_WORKSPACE_RUNMODE, KWORKSPACE_RUNMODE_CONTINUOUS,
		NULL);
	xvw_set_attribute(commandbar, XVW_COMMANDBAR_WORKSPACE, main_workspace);

	active_workspace   = main_workspace;
	selected_workspace = main_workspace;
	xvw_add_callback(main_workspace, XVW_WORKSPACE_ACTIVE_CALLBACK,
			 change_procedure, active_indicator);
	update_varscope_displaylist(main_workspace);
}

/*-----------------------------------------------------------
|
|  Routine Name: change_workspace
|
|       Purpose: This routine changes the currently selected main workspace
|		 within the notebook object.  We use this info to know which
|		 main workspace the user is currently working.
|
|         Input: object - the notebook object
|		 client_data - not used
|		 call_data   - the newly selected main workspace
|       Returns:
|    Written By: Mark Young
|          Date: Dec 08, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
void change_workspace(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject selected = *((xvobject *) call_data);

	/*
	 *  Make the newly selected object our main workspace object.
	 */
	main_workspace   = selected;
	active_workspace = selected;
	xvw_set_attribute(commandbar, XVW_COMMANDBAR_WORKSPACE, main_workspace);
	update_varscope_displaylist(main_workspace);
}

/*-----------------------------------------------------------
|
|  Routine Name: save_workspace
|
|       Purpose: saves a workspace into the supplied filename
|         Input: filename - the filename to save the workspace into
|		 saveall  - whether to save all glyphs or just the
|			    selected ones.
|        Output:
|       Returns:
|    Written By: Mark Young
|          Date: Oct 11, 1993
| Modifications:
|
------------------------------------------------------------*/

void save_workspace(
   char *filename,
   int  saveall)
{
	/*
	 *  Do the actual restoring of the workspace...
	 */
	if (saveall)
	   xvw_set_attribute(active_workspace, XVW_WORKSPACE_SAVEALL, filename);
	else
	   xvw_set_attribute(active_workspace, XVW_WORKSPACE_SAVE, filename);
}

/*-----------------------------------------------------------
|
|  Routine Name: restore_workspace
|
|       Purpose: restore a workspace into the supplied filename
|         Input: filename - the filename to restore the workspace from
|        Output:
|       Returns:
|    Written By: Mark Young
|          Date: Oct 11, 1993
| Modifications:
|
------------------------------------------------------------*/

void restore_workspace(
   char *filename)
{
	xvw_set_attribute(active_workspace, XVW_WORKSPACE_RESTORE, filename);
}

/*-----------------------------------------------------------
|
|  Routine Name: change_procedure
|
|       Purpose: This routine changes the currently selected procedure within
|                the main workspace object.  We use this info to know which
|		 procedure the user is currently working.
|
|         Input: object - the main workspace object
|		 client_data - the active workspace indicator
|		 call_data   - the newly selected procedure
|       Returns:
|    Written By: Mark Young
|          Date: Dec 08, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
void change_procedure(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject selected = *((xvobject *) call_data);

	xvobject plane;
	char temp[KLENGTH], *name;


	/*
	 *  Now build a name heirarchy in which we can show the active
	 *  workspace.
	 */
	ksprintf(temp, "%s", xvw_name(object));
	while (object != selected)
	{
	   kstrcat(temp, ".");

	   plane = selected;
	   while (xvw_parent(plane) != object)
	   {
	      if ((plane = xvw_parent(plane)) == NULL)
		 return;
	   }
	   object = plane;
	   xvw_get_attribute(object, XVW_WORKSPACE_NAME, &name);
	   kstrcat(temp, name);
	}
	xvw_set_attribute(active_indicator, XVW_TEXT_STRING, temp);

	/*
	 *  Finally make the newly selected object our working workspace object.
	 */
	active_workspace = selected;
	update_varscope_displaylist(active_workspace);
}

/*-----------------------------------------------------------
|
|  Routine Name: update_varval_displaylist
|
|       Purpose: Updates the list of variables & their values
|                on the "Variable Values" displaylist selection 
|                of the "Variables" subform, according to the variables 
|                currently defined in the scope of the selected workspace.
|         Input: 
|        Output:
|       Returns:
|    Written By: Danielle Argiro, Mark Young,
|          Date: Dec 13, 1994
| Modifications:
|
------------------------------------------------------------*/

void update_varval_displaylist(void)
{
	int  num;
	long expr_id;
	char **variable_entries;

	expr_id = (long) selected_workspace;

	/* get the variables defined in the scope of the current workspace */
        variable_entries = kexpr_variables_list(expr_id, 1, &num);

        /* set the "Variables Defined By:" displaylist to display wksp names */
        xvf_set_attributes(gui_info->var->variables->varval_struct,
                           XVF_LIST_SIZE,     num,
                           XVF_LIST_CONTENTS, variable_entries,
                           NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: update_vardef_displaylist
|
|       Purpose: Updates the list of variables & their definitions
|                on the "Variable Definitions" displaylist selection
|                of the "Variables" subform, according to the variables
|                currently defined in the scope of the selected workspace.
|         Input: 
|        Output:
|       Returns:
|    Written By: Danielle Argiro, Mark Young, John Salas
|          Date: July 13, 1995
| Modifications:
|
------------------------------------------------------------*/

void update_vardef_displaylist(void)
{
	int  num;
	long expr_id;
        char **variable_defs;

	expr_id = (long) selected_workspace;

        /* get the variables defined in the scope of the current workspace */
        variable_defs = kexpr_variables_def(expr_id, &num);

        /* set the "Variable Definitions:" displaylist to display wksp names */
        xvf_set_attributes(gui_info->var->variables->varorig_struct,
                           XVF_LIST_SIZE,     num,
                           XVF_LIST_CONTENTS, variable_defs,
                           NULL);
}


/*-----------------------------------------------------------
|
|  Routine Name: update_varscope_displaylist
|
|       Purpose: Updates the list of workspace names on the 
|                "Scope Defined By:" displaylist selection 
|                of the "Variables" subform, according to the 
|                names of the main workspace & any sub-workspaces
|                currently defined in the scope of the current workspace.
|
|         Input: wksp - name of workspace for which to update variables
|        Output:
|       Returns:
|    Written By: Danielle Argiro, Mark Young
|          Date: Dec 13, 1994
| Modifications:
|
------------------------------------------------------------*/

void update_varscope_displaylist(
    xvobject wksp)
{
	size_t num = 0, indx = 0;
	char *tmp, **workspace_names = NULL;

	selected_workspace = wksp;
	kfree_and_NULL(workspace_list);

	if (main_workspace == NULL)
	   return;

	/*
	 *  Add main workspace as the first item in the list.
	 */
	xvw_get_attribute(main_workspace, XVW_WORKSPACE_NAME, &tmp);
	tmp = kstring_cleanup(tmp, NULL);
	workspace_names = (char **) karray_add(NULL, KSTRING, tmp, num);
	workspace_list  = (xvobject *) karray_add(NULL, KSTRUCT,
						  main_workspace, num++);

	/*
	 *  Add in any other workspaces to the list.  Set the default index
	 *  to be that of the main workspace.
	 */
	get_workspace_list(main_workspace, &workspace_names, &workspace_list,
			   &indx, &num, "");

        /* set the "scope" displaylist to display workspace names */
        xvf_set_attributes(gui_info->var->variables->scope_struct,
                           XVF_LIST_SIZE,     (int) num,
                           XVF_LIST_CONTENTS, workspace_names,
                           XVF_LIST_INDEX,    (int) indx,
                           NULL);
	karray_free(workspace_names, KSTRING, num, NULL);

	/*
	 * update list of variables & their values on 
	 * "Variables Defined" displaylist
	 */
	update_varval_displaylist();
	update_vardef_displaylist();
}

/*-----------------------------------------------------------
|
|  Routine Name: get_workspace_list
|
|       Purpose: Creates the list of workspace names and objects that
|		 are children of the workspace passed in.  The names and
|		 objects are *appended* to the appropriate arrays.
|
|         Input: parent_wksp     - (parent) workspace
|		 names         - partial list of workspace names
|		 list          - partial list of workspace objects
|		 num           - size of partial list, or 0 if list is NULL
|                indent        - string of whitespace to use w/ indentation
|        Output: num           - size of new list
|		 indx          - index of selected workspace
|       Returns: none
|    Written By: Mark Young & Danielle Argiro
|          Date: Jul 26, 1995
| Modifications:
|
------------------------------------------------------------*/

static void get_workspace_list(
    xvobject parent_wksp,
    char     ***names,
    xvobject **list,
    size_t   *indx,
    size_t   *num,
    char     *indent)
{
	char      temp[KLENGTH];
	char      name[KLENGTH];
	char      space[KLENGTH];
	char     *subwksp_name;
	xvobject *subwksps = NULL;
	int       i; 
	size_t    subwksp_num;

        /* get all subworkspaces of main workspace */
        subwksps = xvw_children(parent_wksp, WorkspaceWidgetClass,
			        &subwksp_num);

	/* need indent to be local */
	kstring_cat(indent, " ", space);

	/* add in names of subworkspaces */
        for (i = 0; i < subwksp_num; i++)
        {
            if (selected_workspace == subwksps[i])
	       *indx = *num;

            xvw_get_attribute(subwksps[i], XVW_WORKSPACE_NAME, &subwksp_name);
	    kstring_cleanup(subwksp_name, temp);
	    ksprintf(name, "%s%s", space, temp);
            *list  = (xvobject *) karray_insert(*list, KSTRUCT, subwksps[i],
				   *num, KLIST_TAIL, TRUE);
            *names = (char **) karray_insert(*names, KSTRING, kstrdup(name),
				   (*num)++, KLIST_TAIL, TRUE);
	    get_workspace_list(subwksps[i], names, list, indx, num, space);
        }
	kfree_and_NULL(subwksps);
}

/*-----------------------------------------------------------
|
|  Routine Name: update_variables_subform_cb
|
|       Purpose: This callback is installed using xvl_expr_add_callback(),
|		 and is fired when things happen to the workspace that
|		 will affect the status of variables.  For example, when
|		 a procedure is created, the varscope list needs to have
|		 the name of the procedure added to it.  Another example
|		 is when the user changes the name of a count loop or a
|		 procedure, and the varscope list has to have the corresponding
|		 entry also change its name.
|
|         Input: object    - object that initiated change in variable status.
|		 operation - one of: XVL_EXPR_UPDATE (variable definitions or
|			     current values updated), XVL_EXPR_ADD, (new wksp
|			     added, varscope list needs entry added) or 
|			     XVL_EXPR_DELETE (old wksp deleted, varscope list
|		             needs entry deleted).
|        Output:
|       Returns:
|    Written By: Danielle Argiro
|          Date: Dec 13, 1994
| Modifications:
|
------------------------------------------------------------*/
void update_variables_subform_cb(
    xvobject object,
    int   operation,
    kaddr client_data)
{
	if (operation == XVL_EXPR_UPDATE)
	{
	   update_varval_displaylist();
	   update_vardef_displaylist();
	}
	else 
	{
	   update_varscope_displaylist(selected_workspace);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: init_pref_values
|
|       Purpose: This routine obtains the settings of attributes
|                for the canvas grid, glyphs, workspace and command
|                bar from the main workspace and initializes the
|                variables which save these original values for later
|                restoration when the "Restore Defaults" button of
|                the Preferences subform is used.  This method is preferable
|                to hardwiring defaults, since this way the user is allowed
|                to set values in their app-defaults files and have those
|                values restored.
|         Input: None
|        Output: None
|    Written By: Danielle Argiro
|          Date: June 18, 1996
| Modifications:
|
------------------------------------------------------------*/

void init_pref_values(void)
{
        /*
         *  Translate canvas grid attributes into values on Canvas Grid pane
         */
	xvw_get_attributes(main_workspace, 
		   XVW_CANVAS_GRID,      &pref_def_griddisplay,
		   XVW_FOREGROUND_COLOR, &pref_def_gridcolor,
		   XVW_CANVAS_GRIDSIZE,  &pref_def_gridsize,
		   NULL);

        /*
         *  Translate workspace glyph attributes into values on Glyph pane
         */
	xvw_get_attributes(main_workspace, 
		   XVW_CONNECTION_TYPE,          &pref_def_connecttype,
		   XVW_REFRESH_CHILDREN_ON_EDIT, &pref_def_connectredraw,
		   XVW_NODE_PLACEMENT,           &pref_def_glyphplacement,
		   XVW_NODE_LEAVE,               &pref_def_glyphdisplay,
		   XVW_GLYPH_REPORTING,          &pref_def_glyphreport,
		   XVW_NODE_REMOTE_ENABLE,       &pref_def_remoteexec,
		   NULL);

        /*
         *  Translate workspace attributes into values on Workspace pane
         */
        xvw_get_attributes(main_workspace,
		   XVW_GLYPH_ECHO,            &pref_def_echoexec,
		   XVW_BACKGROUND_COLOR,      &pref_def_backgroundcolor,
	           XVW_BACKGROUND_PIXMAPFILE, &pref_def_backpixmaps, 
		   NULL);

        /*
         *  Translate commandbar attributes into values on Command Bar pane
         */
	xvw_get_attributes(commandbar,
		  XVW_COMMANDBAR_SHOW_RUN,         &pref_def_runwksp,
		  XVW_COMMANDBAR_SHOW_STOP,        &pref_def_stopwksp,
		  XVW_COMMANDBAR_SHOW_RUNMODE,     &pref_def_runmode,
		  XVW_COMMANDBAR_SHOW_RESET,       &pref_def_resetwksp,
		  XVW_COMMANDBAR_SHOW_CLEAR,       &pref_def_clearwksp,
		  XVW_COMMANDBAR_SHOW_CHECK,       &pref_def_checkwksp,
		  XVW_COMMANDBAR_SHOW_REDRAW,      &pref_def_redrawwksp,
		  XVW_COMMANDBAR_SHOW_INFO,        &pref_def_wkspinfo,
		  XVW_COMMANDBAR_SHOW_CUT,         &pref_def_cut,
		  XVW_COMMANDBAR_SHOW_COPY,        &pref_def_copy,
		  XVW_COMMANDBAR_SHOW_PASTE,       &pref_def_paste,
		  XVW_COMMANDBAR_SHOW_CLIPBOARD,   &pref_def_showclipboard,
		  XVW_COMMANDBAR_SHOW_DUPLICATE,   &pref_def_duplicate,
		  XVW_COMMANDBAR_SHOW_SELECTALL,   &pref_def_selectall,
		  XVW_COMMANDBAR_SHOW_UNSELECTALL, &pref_def_unselectall,
		  XVW_COMMANDBAR_SHOW_DELETE,      &pref_def_delete,
		  XVW_COMMANDBAR_SHOW_UNDO,        &pref_def_undodelete,
		  XVW_COMMANDBAR_SHOW_PROCEDURE,   &pref_def_createprocedure,
		  XVW_COMMANDBAR_SHOW_COUNTLOOP,   &pref_def_createcount,
		  XVW_COMMANDBAR_SHOW_WHILELOOP,   &pref_def_createwhile,
		  XVW_COMMANDBAR_SHOW_OPEN,        &pref_def_openwksp,
		  XVW_COMMANDBAR_SHOW_SAVE,        &pref_def_savewksp,
		  XVW_COMMANDBAR_SHOW_SAVEAS,      &pref_def_savewkspas,
		  NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: init_menu_icons
|
|       Purpose: This routine initializes the menu items such that the
|		 small icons appear next to each menu item.
|
|         Input: None
|        Output: None
|       Returns:
|    Written By: Becky Bishop
|          Date: May 2, 1996
| Modifications:
|
------------------------------------------------------------*/
void init_menu_icons(void)
{
	char       temp[KLENGTH];
	char       *icondir;
	xvobject   new_button, open_button, save_button, saveas_button;
	xvobject   close_button, exit_button, cut_button, copy_button;
	xvobject   paste_button, clip_button, dup_button, delete_button;
	xvobject   undo_button, selectall_button, unselect_button, find_button;
	xvobject   run_button, stop_button, reset_button;
	xvobject   clear_button, check_button, redraw_button, info_button;
	xvobject   var_button, distrib_button, pref_button, cbar_button;
	xvobject   cons_button, size_button, proc_button, count_button;
	xvobject   while_button, tips_button;

	icondir = "$DESIGN/objects/library/xvlang/pixmaps/commandbar/small";


	new_button =
		xvf_get_xvobject(gui_info->create_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/new.xpm", icondir);
	xvw_set_attributes(new_button, 
				XVW_PIXMAP_FILENAME, temp,
			        XVW_ATEXT,           "Ctrl+N", 
			        NULL);
	xvw_add_action(gui_parent, "<Ctrl>n", perform_action,
		       (kaddr) cantata_create, -1);

	open_button =
		xvf_get_xvobject(gui_info->open_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/open.xpm", icondir);
	xvw_set_attributes(open_button, 
				XVW_PIXMAP_FILENAME, temp,
			        XVW_ATEXT,	     "Ctrl+O",
			        NULL);
	xvw_add_action(gui_parent, "<Ctrl>o", perform_action,
		       (kaddr) cantata_open, -1);

	save_button =
		xvf_get_xvobject(gui_info->resave_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/save.xpm", icondir);
	xvw_set_attributes(save_button, 
				XVW_PIXMAP_FILENAME, temp,
				XVW_ATEXT,           "Ctrl+S", 
			 	NULL);
	xvw_add_action(gui_parent, "<Ctrl>s", perform_action,
		       (kaddr) cantata_resave, -1);

	saveas_button =
		xvf_get_xvobject(gui_info->save_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/saveas.xpm", icondir);
	xvw_set_attributes(saveas_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	close_button =
		xvf_get_xvobject(gui_info->close_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/close.xpm", icondir);
	xvw_set_attributes(close_button, 
				XVW_PIXMAP_FILENAME, temp,
				XVW_ATEXT,           "Ctrl+W", 
				NULL);
	xvw_add_action(gui_parent, "<Ctrl>w", perform_action,
		       (kaddr) cantata_close, -1);

	exit_button =
		xvf_get_xvobject(gui_info->quit_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/exit.xpm", icondir);
	xvw_set_attributes(exit_button, 
				XVW_PIXMAP_FILENAME, temp,
				XVW_ATEXT,           "Ctrl+Q", 
				NULL);
	xvw_add_action(gui_parent, "<Ctrl>q", perform_action,
		       (kaddr) cantata_exit, -1);

	cut_button =
		xvf_get_xvobject(gui_info->cut_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/cut.xpm", icondir);
	xvw_set_attributes(cut_button, 
				XVW_PIXMAP_FILENAME, temp,
			        XVW_ATEXT,	   "Ctrl+X", 
			        NULL);
	xvw_add_action(gui_parent, "<Ctrl>x", perform_action,
		       (kaddr) cantata_cut, -1);

	copy_button =
		xvf_get_xvobject(gui_info->copy_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/copy.xpm", icondir);
	xvw_set_attributes(copy_button, 
				XVW_PIXMAP_FILENAME, temp,
				XVW_ATEXT,           "Ctrl+C", 
				NULL);
	xvw_add_action(gui_parent, "<Ctrl>c", perform_action,
		       (kaddr) cantata_copy, -1);

	paste_button =
		xvf_get_xvobject(gui_info->paste_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/paste.xpm", icondir);
	xvw_set_attributes(paste_button, 
				XVW_PIXMAP_FILENAME, temp,
				XVW_ATEXT,           "Ctrl+V", 
			 	NULL);
	xvw_add_action(gui_parent, "<Ctrl>v", perform_action,
		       (kaddr) cantata_paste, -1);

	clip_button =
	    xvf_get_xvobject(gui_info->clipboard_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/showclip.xpm", icondir);
	xvw_set_attributes(clip_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	dup_button =
	    xvf_get_xvobject(gui_info->duplicate_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/dupe.xpm", icondir);
	xvw_set_attributes(dup_button, 
				XVW_PIXMAP_FILENAME, temp,
			        XVW_ATEXT,           "Ctrl+D",
			        NULL);
	xvw_add_action(gui_parent, "<Ctrl>d", perform_action,
		       (kaddr) cantata_duplicate, -1);

	delete_button =
		xvf_get_xvobject(gui_info->del_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/delete.xpm", icondir);
	xvw_set_attributes(delete_button, 
				XVW_PIXMAP_FILENAME, temp,
				XVW_ATEXT,           "Ctrl+B",
				NULL);
	xvw_add_action(gui_parent, "<Ctrl>b", perform_action,
		       (kaddr) cantata_del, -1);

	undo_button =
		xvf_get_xvobject(gui_info->undo_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/undelete.xpm", icondir);
	xvw_set_attributes(undo_button, 
				XVW_PIXMAP_FILENAME, temp,
				XVW_ATEXT,           "Ctrl+Z",
				NULL);
	xvw_add_action(gui_parent, "<Ctrl>z", perform_action,
		       (kaddr) cantata_undo, -1);

	selectall_button =
	      xvf_get_xvobject(gui_info->select_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/select.xpm", icondir);
	xvw_set_attributes(selectall_button, 
				XVW_PIXMAP_FILENAME, temp,
				XVW_ATEXT,           "Ctrl+A",
			        NULL);
	xvw_add_action(gui_parent, "<Ctrl>a", perform_action,
		       (kaddr) cantata_select, -1);

	unselect_button =
	    xvf_get_xvobject(gui_info->unselect_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/unselect.xpm", icondir);
	xvw_set_attributes(unselect_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	find_button =
		xvf_get_xvobject(gui_info->find_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/cut.xpm", icondir);
	xvw_set_attributes(find_button, 
				XVW_PIXMAP_FILENAME, temp,
				NULL);

	run_button =
		xvf_get_xvobject(gui_info->run_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/run.xpm", icondir);
	xvw_set_attributes(run_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	stop_button =
		xvf_get_xvobject(gui_info->stop_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/stop.xpm", icondir);
	xvw_set_attributes(stop_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	reset_button =
		xvf_get_xvobject(gui_info->reset_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/reset.xpm", icondir);
	xvw_set_attributes(reset_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	clear_button =
		xvf_get_xvobject(gui_info->clear_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/clear.xpm", icondir);
	xvw_set_attributes(clear_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	check_button =
		xvf_get_xvobject(gui_info->check_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/check.xpm", icondir);
	xvw_set_attributes(check_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	redraw_button =
		xvf_get_xvobject(gui_info->redraw_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/redraw.xpm", icondir);
	xvw_set_attributes(redraw_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	info_button =
		xvf_get_xvobject(gui_info->info_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/info.xpm", icondir);
	xvw_set_attributes(info_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	var_button =
		xvf_get_xvobject(gui_info->var_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/variable.xpm", icondir);
	xvw_set_attributes(var_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	distrib_button =
	   xvf_get_xvobject(gui_info->distcomp_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/distrib.xpm", icondir);
	xvw_set_attributes(distrib_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	pref_button =
	   xvf_get_xvobject(gui_info->pref_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/prefs.xpm", icondir);
	xvw_set_attributes(pref_button, 
				XVW_PIXMAP_FILENAME, temp,                      
                                NULL);

	cbar_button =
	   xvf_get_xvobject(gui_info->commandbar_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/hidecbar.xpm", icondir);
	xvw_set_attributes(cbar_button, 
				XVW_PIXMAP_FILENAME, temp,                      
                                NULL);

	cons_button =
	   xvf_get_xvobject(gui_info->console_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/hidecons.xpm", icondir);
	xvw_set_attributes(cons_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	size_button =
	   xvf_get_xvobject(gui_info->size_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/small.xpm", icondir);
	xvw_set_attributes(size_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	proc_button = xvf_get_xvobject(
		gui_info->create_procedure_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/proc.xpm", icondir);
	xvw_set_attributes(proc_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	count_button = xvf_get_xvobject(
		gui_info->create_countloop_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/count.xpm", icondir);
	xvw_set_attributes(count_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);
 
	while_button = xvf_get_xvobject(
		gui_info->create_whileloop_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/while.xpm", icondir);
	xvw_set_attributes(while_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);

	tips_button = xvf_get_xvobject(
		gui_info->tooltips_struct, XVF_BUTTON_OBJ, TRUE);
	ksprintf(temp, "%s/showtips.xpm", icondir);
	xvw_set_attributes(tips_button, 
				XVW_PIXMAP_FILENAME, temp,
                                NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: init_encap_wksp
|
|       Purpose: routine which is called when initializing the toolbox
|                list
|
|         Input: program_info - ptr to PaneInfo struct for program pane
|
|        Output: None
|    Written By: Mark Young
|          Date: Feb 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void init_encap_wksp(
     kaddr client_data)
{
        size_t  i, num;
        klist   *list = NULL;
        char    **tmp, *name;
        kform_struct *formstruct;

	encapsulate_program *program_info = (encapsulate_program *) client_data;

        /*
         *  get array of toolbox names
         */
        if ((tmp = kcms_query_toolboxes(&num)) == NULL)
        {
           kinfo(KSTANDARD, "Warning: No toolboxes accessable via cantata");
           return;
        }

        /*
         *  Now update the toolbox display list object.  This will allow
         *  the user to browse existing toolboxes in which to create their
         *  cantata workspace program.
         */
	for (i = 0; i < num; i++)
	    kstring_lower(tmp[i], tmp[i]);

        tmp = (char **) karray_sort(tmp, KSTRING, num, TRUE);
        formstruct = program_info->toolboxes_struct;
        xvf_set_attribute(formstruct, XVF_LIST_SIZE, num);
        xvf_set_attribute(formstruct, XVF_LIST_CONTENTS, tmp);
        karray_free(tmp, KSTRING, num, NULL);

        num = 0; tmp = NULL;
        xvl_get_contents(&list, NULL);
        while (list != NULL)
        {
           name = ktoken_to_string(list->identifier);
           tmp  = (char **) karray_add(tmp, KSTRING, name, num++);
           list = klist_next(list);
        }
        formstruct = program_info->category_struct;
        xvf_set_attribute(formstruct, XVF_LIST_SIZE, num);
        xvf_set_attribute(formstruct, XVF_LIST_CONTENTS, tmp);
        kfree(tmp);

        xvf_remove_extra_call(gui_info->encapsulate_struct,
                init_encap_wksp, (kaddr) program_info);
}

void notify_unmap(
     xvobject object,
     kaddr    client_data,
     int      *stop_timer)
{
        xvw_set_attribute(notifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
	*stop_timer = TRUE;
}
