/* 
 * Khoros: $Id: utilities.c,v 1.16.2.3 1997/04/29 22:43:41 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.16.2.3 1997/04/29 22:43:41 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            craftsman utility routines
   >>>>
   >>>>  Private:
   >>>>         verify_lib_list() - Verify the library routine specified
   >>>>         change_lib_list() - Change the library string list
   >>>>         update_categories() - Update category and subcategory lists
   >>>>         our_announce_handler - kinfo() handler for craftsman
   >>>>         update_keywords - update list of keywords and selected keyword
   >>>>         keylist_to_array - convert klist of keys to array of kstrings
   >>>>         update_categories() - Update category and subcategory lists
   >>>>         merge_category_strings - merge toolbox (sub)category strings
   >>>>         enable_in_cantata - enable/disable the IN CANTATA selections
   >>>>         remove_deps() - remove dependencies from depname array
   >>>>         update_adddeplist() - update Add Dependency list
   >>>>         update_dependencies - update list of dependencies
   >>>>
   >>>>   Static:
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "craftsman.h"

/*-----------------------------------------------------------
| Routine Name:	verify_lib_list() - Verify the library routine specified
|
| Purpose:	This routine is called to verify the library routine specified
|		to associate a kroutine with.  Note that the library must
|		already exist.  If the library specified does not exist,
|		then print an error message and return FALSE.   Also, the
|		object specified must be a library object.
|
| Written By:	Neil Bowers
| Date:		3-jun-93
------------------------------------------------------------*/
int
verify_lib_list(
   kstring  library)
{
   int  status = FALSE;


   status = (library != NULL && kstring_cleanup(library, library) != NULL);

   if (!status)
      kerror(NULL, "Verify lib list",
	     "A library name was not specified.\n"
	     "If you wish to create a library routine, you must "
	     "specify an existing library for the library routine");

   return status;
}


/*-----------------------------------------------------------
| Routine Name:	change_lib_list() - Change the library string list
|
| Purpose:	This routine is called to change the library string list.
|		It changes the list by querying kcms for all the existing
|		library objects. 
|
| Written By:	Neil Bowers
| Date:		17-mar-94
------------------------------------------------------------*/
void
change_lib_list(void)
{
   klist *  list;
   char **  array;
   int      count  = 0;


   if ((list = kcms_get_objects(cmsToolbox, KCMS_LIBRARY)) != NULL)
   {
      array = kcms_list_get_attribute(list, KCMS_NAME, &count);

      xvf_set_attributes(paneKroutine->lib_list_struct,
			 XVF_LIST_SIZE,     count,
			 XVF_LIST_CONTENTS, array,
			 NULL);
   }
}

/*-----------------------------------------------------------
| Routine Name:	update_categories() - Update category and subcategory lists
|
| Purpose:	This routine is called to gather the category and subcategory
|		lists.  This is done by querying kcms for existing
|		category/subcategory information.
|
| Written By:	Becky Bishop 
| Date:		Thu Jun  3 10:40:43 MDT 1993
------------------------------------------------------------*/
void
update_categories(void)
{
   char          **cat_list;
   unsigned long  *otypes;
   char          **subcat_list;
   char          **programs;
   char          **panefile;
   char          **workspaces;
   char          **tmp_cat;
   char          **tmp_subcat;
   char          **icons;
   int            *incantata;
   char          **short_descriptions;
   size_t          num_programs;
   int             i, j;
   size_t          tmp_cnt;
   size_t          num_cat;
   size_t          num_subcat;


   /*-- if global categories & subcategories selected, then do nothing --*/
   if (gui_info->preferences->types->cattype_num == 2)
      return;

   /*-- (sub) categories from current toolbox, but no toolbox selected --*/
   if (cmsToolbox == NULL)
      return;

   cat_list = NULL;
   subcat_list = NULL;

   kcms_get_attribute(cmsToolbox, KCMS_TB_CATEGORYINFO,
		      &programs, &otypes, &icons, &tmp_cat, &tmp_subcat,
		      &panefile, &workspaces, &incantata,
		      &short_descriptions, &num_programs);

   if (num_programs == 0)
      return;

   num_cat = num_programs;
   num_subcat = num_programs;

   cat_list = (char **)karray_add(NULL, KSTRING, tmp_cat[0], (size_t) 0);
   tmp_cnt = 0;
   for (i = 0; i < num_cat; i++)
   {
      if (tmp_cat[i] == NULL || !kstrcmp(tmp_cat[i], ""))
	 continue;
      j = 0;
      while (j < tmp_cnt && kstrcmp(cat_list[j], tmp_cat[i]))
	 j++;
      if (j == tmp_cnt)
      {
	 cat_list = (char **)karray_add(cat_list, KSTRING, tmp_cat[i], tmp_cnt);
	 tmp_cnt++;
      }
   }
   num_cat = tmp_cnt;
   xvf_set_attributes(gui_info->template1->category_struct,
		      XVF_LIST_SIZE,     (int) num_cat,
		      XVF_LIST_CONTENTS, cat_list,
		      NULL);
   xvf_set_attributes(paneObjAttrs->category_struct,
		      XVF_LIST_SIZE,     (int) num_cat,
		      XVF_LIST_CONTENTS, cat_list,
		      NULL);

   subcat_list = (char **)karray_add(NULL, KSTRING, tmp_subcat[0], (size_t) 0);
   tmp_cnt = 0;
   for (i = 0; i < num_subcat; i++)
   {
      if (tmp_subcat[i] == NULL || !kstrcmp(tmp_subcat[i], ""))
	 continue;
      j = 0;
      while (j < tmp_cnt && kstrcmp(subcat_list[j], tmp_subcat[i]))
	 j++;
      if (j == tmp_cnt)
      {
	 subcat_list = (char **)karray_add(subcat_list, KSTRING, tmp_subcat[i], tmp_cnt);
	 tmp_cnt++;
      }
   }
   num_subcat = tmp_cnt;
   xvf_set_attributes(gui_info->template1->subcategory_struct,
		      XVF_LIST_SIZE,     (int) num_subcat,
		      XVF_LIST_CONTENTS, subcat_list,
		      NULL);
   xvf_set_attributes(paneObjAttrs->subcategory_struct,
		      XVF_LIST_SIZE,     (int) num_subcat,
		      XVF_LIST_CONTENTS, subcat_list,
		      NULL);
}

/*-----------------------------------------------------------
| Routine Name:	our_announce_handler - kinfo() handler for craftsman
|
| Purpose:	This routine catches any strings displayed using kinfo()
|		and displays them in the notify-window popup.
|		This means the user will see messages from the code
|		generation library and similar.
|
| Input:	message	- the message to display
| Returns:	TRUE if successful, FALSE otherwise.
| Written By:	Neil Bowers
| Date:		17-mar-94
------------------------------------------------------------*/
int
our_announce_handler(
   kstring  toolbox,
   kstring  object,
   kstring  library,
   kstring  routine,
   kstring  message)
{
   int result;

   result = xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_MESSAGE, message);
   xvw_sync(FALSE);
   return result;
}

/*-----------------------------------------------------------
| Routine Name:	update_keywords - update list of keywords and selected keyword
|
| Purpose:	This routine is called after the list of keywords associated
|		with a toolbox or software object has changed. The routine
|		take the object reference, and an optional keyword which
|		should be auto-selected. If NULL, then no word is selected
|		in the list.
|
| Input:	object - a toolbox or software object
|		selected - an optional string which contains a keyword
|			   which we want auto-selected in the display list.
| Returns:	void
| Written By:	Neil Bowers
| Date:		17-mar-94
------------------------------------------------------------*/
void
update_keywords(
   kobject  object,
   kstring  selected)
{
   int            cmstype;
   klist         *list;
   kstring       *keywords;
   size_t         nkeys;
   kform_struct  *list_struct;
   int            i;


   if (!kcms_get_attribute(object, KCMS_TYPE, &cmstype))
      return;

   if (cmstype == KOBJ_CMSTB)
   {
      if (!kcms_get_attribute(object, KCMS_TB_KEYWORDS, &list))
	 return;
      list_struct = gui_info->tbattrs->keywords->keylist_struct;
   }
   else /*-- assume it's a software object --*/
   {
      if (!kcms_get_attribute(object, KCMS_CMOBJ_KEYWORDS, &list))
	 return;
      list_struct = gui_info->obj_attributes->obj_keywords->keylist_struct;
   }

   if (list == NULL)
      xvf_set_attributes(list_struct,
			 XVF_LIST_SIZE,     0,
			 XVF_LIST_CONTENTS, NULL,
			 NULL);
   else if ((keywords = keylist_to_array(list, &nkeys)) != NULL)
   {
      keywords = (char **)karray_sort(keywords, KSTRING, nkeys, TRUE);
      xvf_set_attributes(list_struct,
			 XVF_LIST_SIZE,     (int) nkeys,
			 XVF_LIST_CONTENTS, keywords,
			 NULL);

      /*-- if a selected item was specified, select it ----------------*/
      if (selected != NULL)
      {
	 i=0;
	 while (i < nkeys && kstrcmp(selected, keywords[i]) != 0)
	    i++;
	 if (i < nkeys)
	 {
	    xvf_set_attribute(list_struct, XVF_LIST_VAL, i);
	    if (cmstype == KOBJ_CMSTB)
	    {
	       kfree(gui_info->tbattrs->keywords->keylist_label);
	       gui_info->tbattrs->keywords->keylist_label = kstrdup(selected);
	       gui_info->tbattrs->keywords->keylist = i;
	    }
	    else
	    {
	       kfree(gui_info->obj_attributes->obj_keywords->keylist_label);
	       gui_info->obj_attributes->obj_keywords->keylist_label =
		  kstrdup(selected);
	       gui_info->obj_attributes->obj_keywords->keylist = i;
	    }
	 }
      }
   }
}

/*-----------------------------------------------------------
| Routine Name:	keylist_to_array - convert klist of keys to array of kstrings
|
| Purpose:	This should be removed, and calls to this function
|		replaced with calls to klist_to_array().
|		Though I think this was done this way since we want to
|		strdup the strings in the array, because we're passing
|		them into xvforms?
|
| Written By:	Neil Bowers
| Date:		16-mar-1995
------------------------------------------------------------*/
kstring *
keylist_to_array(
   klist  *keylist,
   size_t *count)
{
   size_t    i;
   kstring  *array;
   klist    *list;
   kstring   keyword;


   *count = klist_size(keylist);
   array = (kstring *)kmalloc(*count * sizeof(kstring));
   if (array == NULL)
      return NULL;

   for (i=0, list = keylist; list != NULL; list = klist_next(list))
   {
      keyword  = (kstring)klist_clientdata(list);
      array[i] = kstrdup(keyword);
      i++;
   }

   return array;
}

/*-----------------------------------------------------------
| Routine Name:	update_categories() - Update category and subcategory lists
|
| Purpose:	This routine is called to gather the category and subcategory
|		lists.  This is done by querying kcms for existing
|		category/subcategory information.
|
| Written By:	Neil Bowers
| Date:		16-mar-1995
------------------------------------------------------------*/
int
craftsman_load_categories(
   kobject  toolbox)
{
   char  **tmp_cat;
   char  **tmp_subcat;
   size_t  num_programs;
   int     i;
   int     j;


   if (!kcms_get_attribute(toolbox,
			   KCMS_TB_CATEGORYINFO,
			   NULL, NULL, NULL, &tmp_cat, &tmp_subcat,
			   NULL, NULL, NULL, NULL, &num_programs))
      return FALSE;

   for (i = 0; i < num_programs; i++)
   {
      /*-- check i'th category ----------------------------------------*/
      if (tmp_cat[i] != NULL && kstrlen(tmp_cat[i]) > 0)
      {
	 j = 0;
	 while (j < nCategories && kstrcmp(pstrCategories[j], tmp_cat[i]))
	    j++;
	 if (j == nCategories)
	 {
	    pstrCategories = (char **)karray_add(pstrCategories, KSTRING,
				    kstrdup(tmp_cat[i]), (size_t) nCategories);
	    nCategories++;
	 }
      }

      /*-- check i'th category ----------------------------------------*/
      if (tmp_subcat[i] != NULL && kstrlen(tmp_subcat[i]) > 0)
      {
	 j = 0;
	 while (j < nSubcategories
		&& kstrcmp(pstrSubcategories[j], tmp_subcat[i]))
	 {
	    j++;
	 }
	 if (j == nSubcategories)
	 {
	    pstrSubcategories = (char **)karray_add(pstrSubcategories, KSTRING,
					   kstrdup(tmp_subcat[i]),
					   (size_t) nSubcategories);
	    nSubcategories++;
	 }
      }
   }

   return TRUE;
}

/*-----------------------------------------------------------
| Routine Name:	merge_category_strings - merge toolbox (sub)category strings
|
| Purpose:	Set category and subcategory strings on the various
|		selections in the Craftsman GUI.
|
| Written By:	Neil Bowers
| Date:		16-mar-1995
------------------------------------------------------------*/
void
merge_category_strings(
   kstring  category,
   kstring  subcategory)
{
   int  j = 0;


   if (category != NULL)
   {
      while (j < nCategories && kstrcmp(pstrCategories[j], category))
	 j++;
      if (j == nCategories)
      {
	 pstrCategories = (char **)karray_add(pstrCategories, KSTRING,
				     kstrdup(category), (size_t) nCategories);
	 nCategories++;
	 pstrCategories = (char **)karray_sort(pstrCategories, KSTRING, 
				      (size_t) nCategories, FALSE);
	 xvf_set_attributes(gui_info->template1->category_struct,
			    XVF_LIST_SIZE,     nCategories,
			    XVF_LIST_CONTENTS, pstrCategories,
			    NULL);
	 xvf_set_attributes(paneObjAttrs->category_struct,
			    XVF_LIST_SIZE,     nCategories,
			    XVF_LIST_CONTENTS, pstrCategories,
			    NULL);
      }
   }

   if (subcategory != NULL)
   {
      while (j < nSubcategories
	     && kstrcmp(pstrSubcategories[j], subcategory))
	 j++;
      if (j == nSubcategories)
      {
	 pstrSubcategories = (char **)karray_add(pstrSubcategories, KSTRING,
					kstrdup(subcategory),
					(size_t) nSubcategories);
	 nSubcategories++;
	 pstrSubcategories = (char **)karray_sort(pstrSubcategories, KSTRING,
					 (size_t) nSubcategories, FALSE);
	 xvf_set_attributes(gui_info->template1->subcategory_struct,
			    XVF_LIST_SIZE,     nSubcategories,
			    XVF_LIST_CONTENTS, pstrSubcategories,
			    NULL);
	 xvf_set_attributes(paneObjAttrs->subcategory_struct,
			    XVF_LIST_SIZE,     nSubcategories,
			    XVF_LIST_CONTENTS, pstrSubcategories,
			    NULL);
      }
   }
}

/*-----------------------------------------------------------
| Routine Name:	enable_in_cantata - enable/disable the IN CANTATA selections
|
| Purpose:	Enable or disable the various selections in the GUI
|		which let you set whether an object should appear in Cantata.
|
| Written By:	Neil Bowers
| Date:		11-May-96
------------------------------------------------------------*/
void
enable_in_cantata(
   int  in_cantata)
{
   xvf_set_attribute(paneObjAttrs->install_in_cantata_struct,
		     XVF_ACTIVATE, in_cantata);
   xvf_set_attribute(gui_info->template1->kroutine->incantata_struct,
		     XVF_ACTIVATE, in_cantata);
   xvf_set_attribute(gui_info->template1->xvroutine->incantata_struct,
		     XVF_ACTIVATE, in_cantata);
   xvf_set_attribute(gui_info->template1->pane->incantata_struct,
		     XVF_ACTIVATE, in_cantata);
   xvf_set_attribute(gui_info->template1->script->incantata_struct,
		     XVF_ACTIVATE, in_cantata);
}
/*-----------------------------------------------------------
| Routine Name: remove_deps() - remove dependencies from depname array
|
| Purpose:      This routine is called from update_adddeplist() to update
|               the list of toolboxes on the Add New Dependency list on
|               the Toolbox Attributes pane so that you can't add yourself
|               or any box you already depend on as a new dependency.
|
| Input:        tbname - name of toolbox whose dependencies are to be removed
|               depname_array - array from which to remove dependencies
|               counter - size of array
| Output:       depname_array - array with dependencies removed
|               counter - resulting size of array
|
| Returns:      depname_array
|
| Written By:   Kevin Sullivan
| Date:         31-jan-97
------------------------------------------------------------*/
kstring *
remove_deps(
   kstring     tbname,
   kstring    *depname_array,
   size_t     *counter
   )
{
   kobject     tbobj = kcms_open_toolbox(tbname);
   klist      *deplist = NULL;
   kstring     depname;
 
   if (tbobj != NULL)
      if (kcms_get_attribute(tbobj, KCMS_TB_DEPENDENCIES, &deplist))
         for (;
              deplist != NULL;
              deplist = klist_next(deplist))
         {
            depname = (kstring)klist_clientdata(deplist);
            if (karray_locate(depname_array, KSTRING, depname, *counter) != -1)
            {
               depname_array = (char **)karray_delete(depname_array, KSTRING,
                                             depname, *counter);
               (*counter)--;
            }
            depname_array = remove_deps(depname, depname_array, counter);
         }
   return depname_array;
}
/*-----------------------------------------------------------
| Routine Name: update_adddeplist() - update Add Dependency list
|
| Purpose:      This routine is called from select_toolbox(), 
|               dependencies_delete(), and dependencies_adddep()
|               to update the Add New Dependency list
| Written By:   Kevin Sullivan
| Date:         31-jan-97
------------------------------------------------------------*/
void
update_adddeplist(void)
{
   char      tblower[KLENGTH];
   size_t    tb_depcount = (size_t)countToolboxes;
   kstring   tbname;
   kstring  *tb_depname_array = NULL;

   if ((cmsToolbox != NULL)
       && (kcms_get_attribute(cmsToolbox, KCMS_NAME, &tbname)))
   {
      /*-- update the pulldown for new dependency tb on dependencies -----*/
      kstring_lower(tbname, tblower);
      tb_depname_array = (char **)karray_copy(toolbox_names, KSTRING,
                                     (size_t)countToolboxes, TRUE);
      if (karray_locate(tb_depname_array, KSTRING, tblower, tb_depcount) != -1)
      {
         tb_depname_array = (char **)karray_delete(tb_depname_array, KSTRING,
                                            tblower, tb_depcount);
         tb_depcount--;
      }
      tb_depname_array = remove_deps(tblower, tb_depname_array, &tb_depcount);
      xvf_set_attributes(gui_info->tbattrs->dependencies->adddep_struct,
                         XVF_LIST_SIZE,     tb_depcount,
                         XVF_LIST_CONTENTS, tb_depname_array,
                         NULL);
   }
}

/*-----------------------------------------------------------
| Routine Name: update_dependencies - update list of dependencies 
|               and selected dependency
|
| Purpose:      This routine is called after the list of dependencies associated
|               with a toolbox has changed. The routine
|               take the object reference, and an optional dependency which
|               should be auto-selected. If NULL, then no word is selected
|               in the list.
|
| Input:        object - a toolbox
|               selected - an optional string which contains a dependency
|                          which we want auto-selected in the display list.
| Returns:      void
| Written By:   Kevin Sullivan
| Date:         24-jan-97
------------------------------------------------------------*/
void
update_dependencies(
   kobject  object,
   kstring  selected)
{
   klist         *list;
   kstring       *dependencies;
   size_t         ndeps;
   kform_struct  *list_struct;
   int            i;

   if (!kcms_get_attribute(object, KCMS_TB_DEPENDENCIES, &list))
      return;
   list_struct = gui_info->tbattrs->dependencies->deplist_struct;

   if (list == NULL)
      xvf_set_attributes(list_struct,
                         XVF_LIST_SIZE,     0,
                         XVF_LIST_CONTENTS, NULL,
                         NULL);
   else if ((dependencies = keylist_to_array(list, &ndeps)) != NULL)
   {
      xvf_set_attributes(list_struct,
                         XVF_LIST_SIZE,     (int) ndeps,
                         XVF_LIST_CONTENTS, dependencies,
                         NULL);

      /*-- if a selected item was specified, select it ----------------*/
      if (selected != NULL)
      {
         i=0;
         while (i < ndeps && kstrcmp(selected, dependencies[i]) != 0)
            i++;

         if (i < ndeps)
         {
            xvf_set_attribute(list_struct, XVF_LIST_VAL, i+1);
            kfree(gui_info->tbattrs->dependencies->deplist_label);
            gui_info->tbattrs->dependencies->deplist_label = kstrdup(selected);
            gui_info->tbattrs->dependencies->deplist = i;
         }
      }
      else
      {
         xvf_set_attribute(list_struct, XVF_LIST_VAL, 0);
         gui_info->tbattrs->dependencies->deplist = -1;
      }
   }
}
