 /*
  * Khoros: $Id: guise.c,v 1.15.2.2 1997/05/31 18:33:40 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: guise.c,v 1.15.2.2 1997/05/31 18:33:40 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for guise
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "guise.h"

clui_info_struct *clui_info = NULL;

gui_info_struct *gui_info;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Direct manipulation GUI design tool
|
|       Purpose: main program for guise
|
|         Input:
|		int clui_info->force_flag; {TRUE if -force specified}
|
|		char *clui_info->o_file; {output UIS file}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		Optional M.E. group:
|		    char *clui_info->i_file; {initial UIS file}
|		    int   clui_info->i_flag; {TRUE if -i specified}
|
|		    M.I. group:
|			char *clui_info->tb_string; {toolbox}
|			int   clui_info->tb_flag; {TRUE if -tb specified}
|
|			char *clui_info->oname_string; {object name}
|			int   clui_info->oname_flag; {TRUE if -oname specified}
|
|			char *clui_info->uis_string; {UIS file to edit (do not include path)}
|			int   clui_info->uis_flag; {TRUE if -uis specified}
|
|        Output:
|       Returns:
|
|    Written By: Danielle Argiro
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

	kform  *form;            /* form tree representing UIS        */
	char   *form_fullpath;   /* for use when [-form] is specified */
/* -main_variable_list */
        char    fullpath[KLENGTH];
/* -main_variable_list_end */

	khoros_init(argc, argv, "DESIGN", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$DESIGN/objects/xvroutine/guise");
	kexit_handler(guise_free_args, NULL);

/* -main_get_args_call */
	kclui_init("DESIGN", "guise", KGEN_XVROUTINE,  &clui_uis_spec,
		   guise_usage_additions, guise_get_args,
 		   guise_free_args);
/* -main_get_args_call_end */

	/*
	 * Initialize xvwidgets lib
	 */
	if (!(xvw_initialize(XVW_MENUS_XVFORMS)))
	{
	     kinfo(KSTANDARD, "Could not open connection to server");
	     kexit(KEXIT_FAILURE);
	}

	/*
	 * Allocate the GUI Info structure 
	 */
	gui_info = (gui_info_struct *)
				kcalloc ((size_t) 1, sizeof(gui_info_struct));
	/*
	 * Create the forms for guise 
	 */
	if (clui_info->form_flag)
	{
	    form_fullpath   = kfullpath(clui_info->form_file, NULL, NULL);
	    form = xvf_create_form(form_fullpath, KNONE, run_guise, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	    kfree_and_NULL(form_fullpath);
	}
	else
	{
	    form = xvf_build_form(&gui_uis_spec, KNONE, run_guise, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	}
	if (form == NULL)
	   kexit(KEXIT_FAILURE);

	/*
	 * Initialize the GUI Info structure 
	 */
	if (!(_xvf_init_guise(form, gui_info)))
	    kexit(KEXIT_FAILURE);

/* -main_before_lib_call */
        guise_initialize();
/* -main_before_lib_call_end */

/* -main_library_call */
	xvf_run_form();
/* -main_library_call_end */

/* -main_after_lib_call */
	if (kfullpath(output_filename, kgetenv("TMPDIR"), fullpath) != NULL)
            kunlink(fullpath);

        kfree_and_NULL(output_filename);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: guise_usage_additions
| 
|       Purpose: Prints usage additions in guise_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname guise
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void guise_usage_additions(void)
{
	kfprintf(kstderr, "\tDirect manipulation GUI design tool\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: guise_free_args
| 
|       Purpose: Frees CLUI struct allocated in guise_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname guise
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
guise_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->form_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
