 /*
  * Khoros: $Id: preview.c,v 1.17.2.4 1997/05/31 18:34:37 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: preview.c,v 1.17.2.4 1997/05/31 18:34:37 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for preview
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "preview.h"

clui_info_struct *clui_info = NULL;

gui_info_struct *gui_info;

/*-----------------------------------------------------------
|
|  Routine Name: main() - GUI display tool
|
|       Purpose: main program for preview
|
|         Input:
|		char *clui_info->i_file; {input UIS file}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		int clui_info->noform_flag; {TRUE if -noform specified}
|
|        Output:
|       Returns:
|
|    Written By: John Rasure, Mark Young
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

	kform  *form;            /* form tree representing UIS        */
	char   *form_fullpath;   /* for use when [-form] is specified */
/* -main_variable_list */
/* -main_variable_list_end */

	khoros_init(argc, argv, "DESIGN", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$DESIGN/objects/xvroutine/preview");
	kexit_handler(preview_free_args, NULL);

/* -main_get_args_call */
	kclui_init("DESIGN", "preview", KGEN_XVROUTINE,  &clui_uis_spec,
		   preview_usage_additions, preview_get_args,
 		   preview_free_args);
/* -main_get_args_call_end */

	/*
	 * Initialize xvwidgets lib
	 */
	if (!(xvw_initialize(XVW_MENUS_XVFORMS)))
	{
	     kinfo(KSTANDARD, "Could not open connection to server");
	     kexit(KEXIT_FAILURE);
	}

	/*
	 * Allocate the GUI Info structure 
	 */
	gui_info = (gui_info_struct *)
				kcalloc ((size_t) 1, sizeof(gui_info_struct));
	/*
	 * Create the forms for preview 
	 */
	if (clui_info->form_flag)
	{
	    form_fullpath   = kfullpath(clui_info->form_file, NULL, NULL);
	    form = xvf_create_form(form_fullpath, KNONE, run_preview, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	    kfree_and_NULL(form_fullpath);
	}
	else
	{
	    form = xvf_build_form(&gui_uis_spec, KNONE, run_preview, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	}
	if (form == NULL)
	   kexit(KEXIT_FAILURE);

	/*
	 * Initialize the GUI Info structure 
	 */
	if (!(_xvf_init_preview(form, gui_info)))
	    kexit(KEXIT_FAILURE);

/* -main_before_lib_call */
        if (clui_info->noform_flag)
        {
            if (!clui_info->i_flag)
            {
                kerror(NULL, "main", "When specifying the [-noform] flag, "
                      "you must give an input UIS file using the [-i] option");
                kexit(KEXIT_FAILURE);
            }
            kfree_and_NULL(gui_info);
        }
        if (clui_info->i_flag)
        {
            if (gui_info != NULL && gui_info->show->view->i_struct != NULL)
            {
                xvf_set_attribute(gui_info->show->view->i_struct,
                                  XVF_FILE_NAME, clui_info->i_file);
                kfree_and_NULL(gui_info->show->view->i);
                gui_info->show->view->i = kstrdup(clui_info->i_file);
            }

            display_form = xvf_create_form(clui_info->i_file, KNONE,
                                           run_display, gui_info,
                                           -1, -1, XVF_FULL_EDIT);
            if (display_form != NULL)
                xvw_add_detectfile(NULL, clui_info->i_file, PREVIEW_DELAY,
                                   update_displayed_uisfile, NULL);
        }
/* -main_before_lib_call_end */

/* -main_library_call */
	xvf_run_form();
/* -main_library_call_end */

/* -main_after_lib_call */
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: preview_usage_additions
| 
|       Purpose: Prints usage additions in preview_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname preview
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void preview_usage_additions(void)
{
	kfprintf(kstderr, "\tGUI display tool\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: preview_free_args
| 
|       Purpose: Frees CLUI struct allocated in preview_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname preview
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
preview_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->form_file);
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
