/*
  * Khoros: $Id: create.c,v 1.16.2.7 1997/05/24 19:08:23 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: create.c,v 1.16.2.7 1997/05/24 19:08:23 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>         Functionality routines for running a workspace
   >>>>
   >>>>  Private:
   >>>>          quit_cb
   >>>>          workspace_cb
   >>>>          create_workspace
   >>>>          update_status
   >>>>   Static:
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "runwksp.h"

xvobject status = NULL; 
static int update_status PROTO((char *, char *, char *, char *, char *));


/*-----------------------------------------------------------
|
|  Routine Name: quit_cb - routine for quit callback
|
|       Purpose: This is the quit workspace callback routine
|         Input: quit        - the quit button
|                client_data - the workspace object to quit
|                call_data   - unused
|    Written By: Mark Young
|          Date: Jul 14, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void quit_cb(
    xvobject object,
    kaddr client_data,
    kaddr call_data)
{
	xvobject toplevel = (xvobject) client_data;
	xvw_destroy(toplevel);
}

/*-----------------------------------------------------------
|
|  Routine Name: workspace_cb - routine for workspace callback
|
|       Purpose: This is the workspace run callback routine
|         Input: workspace   - the workspace
|                client_data - the workspace object to quit
|                call_data   - unused
|    Written By: Mark Young
|          Date: Sep 05, 1994
| Modifications:
|
------------------------------------------------------------*/
void quit_timeout(
   XtPointer     client_data,
   XtIntervalId  *id)
{
	xvobject toplevel = (xvobject) client_data;

	xvw_destroy(toplevel);
}

/* ARGSUSED */
static void workspace_cb(
    xvobject object,
    kaddr client_data,
    kaddr call_data)
{
	int running = *((int *) call_data);
	xvobject toplevel = (xvobject) client_data;

	if (clui_info->silent_flag == FALSE)
	   kfprintf(kstderr,"runwksp: Workspace running status = %d\n",running);

	if (running != -1)
	   return;

	XtAppAddTimeOut(xvw_appcontext(object), (unsigned long) 1000, 
			quit_timeout, toplevel);
}

/*-----------------------------------------------------------
|
|  Routine Name: create_workspace()
|
|       Purpose: Creates the display workspace for running a compiled
|	         "workspace" (pane) object.
|
|         Input: 
|        Output:
|       Returns:
|    Written By: Mark Young
|          Date: Sep 08, 1993
| Modifications:
|
------------------------------------------------------------*/
void create_workspace(void)
{
	kform    *form = NULL;
	kobject  toolbox, program, obj;
        kselection *close_sel, *sel_list;
	char	 *tbname, *oname, *wksp_file = NULL;
	xvobject toplevel; 
	xvobject parent; 
	xvobject workspace; 
	xvobject commandbar = NULL; 
	xvobject quit = NULL;
	xvobject pixmap = NULL;
	char *pixmapfile = "$DESIGN/objects/xvroutine/runwksp/pixmaps/runwksp.xpm";

	/*
	 *  Create the parent in which the workspace will be placed
	 */
	if (clui_info->rootwindow_flag)
	{
	   toplevel = xvw_create_rootwindow("Runwksp", NULL, NULL);
           parent = xvw_create_manager(toplevel, "Run Workspace Program");
	   xvw_set_attribute(parent, XVW_TACK_EDGE, KMANAGER_TACK_ALL);
	}
	else
	{
           toplevel = parent = xvw_create_manager(NULL,"Run Workspace Program");
	   xvw_set_attributes(parent,
                XVW_WIDTH,  750,
                XVW_HEIGHT, 600,
		NULL);
	}

	if (clui_info->hidden_flag == FALSE)
	{
	    /*
	     *  Create pixmap for logo & label for top of GUI
	     */
	    pixmap =  xvw_create_pixmap(parent, "pixmap");
            xvw_set_attributes(pixmap, 
		XVW_PIXMAP_FILENAME, pixmapfile,
		XVW_BELOW,           NULL,
		NULL);

           commandbar = xvw_create_commandbar(parent, "CommandBar");
           quit = xvw_create_button(parent, "Quit");

	   /*
	    *  Create the commandbar
	    */
           xvw_set_attributes(commandbar,
                XVW_TACK_EDGE, KMANAGER_TACK_HORIZ,
		XVW_LEFT_OF,   quit,
		XVW_RIGHT_OF,  pixmap,
		XVW_BELOW,     NULL,
                NULL);

	   xvw_set_attributes(quit,
                XVW_LABEL,      "Quit",
                XVW_LEFT_OF,     NULL,
                XVW_ABOVE,       commandbar,
                XVW_BELOW,       commandbar,
		XVW_TACK_EDGE,   KMANAGER_TACK_VERT,
                XVW_CHAR_WIDTH,  6.0,
                NULL);

	   xvw_set_attributes(quit,
                XVW_LABEL,      "Quit",
                XVW_LEFT_OF,     NULL,
                XVW_ABOVE,       commandbar,
                XVW_BELOW,       commandbar,
		XVW_TACK_EDGE,   KMANAGER_TACK_VERT,
                XVW_CHAR_WIDTH,  6.0,
                NULL);

	    /*
	     *  Create the status window
	     */
	   status = xvw_create_text(parent, "Status");
           xvw_set_attributes(status,
                XVW_BORDER_WIDTH,    1,
                XVW_TACK_EDGE,       KMANAGER_TACK_HORIZ,
                XVW_ABOVE,           NULL,
                XVW_CHAR_HEIGHT,     1.0,
                XVW_TEXT_EDIT_TYPE,  KTEXT_READ,
                NULL);
	   kset_announcehandler((knotify)update_status);
	}

	if (clui_info->report_flag)
	   xvl_enable_reporting(TRUE);

	/*
	 *  Create the workspace
	 */
        workspace = xvw_create_workspace(parent, "Workspace");
        xvw_set_attributes(workspace,
                XVW_BELOW,         commandbar,
		XVW_ABOVE,	   status,
		XVW_CANVAS_WIDTH,  3000,
		XVW_CANVAS_HEIGHT, 3000,
		XVW_TACK_EDGE,     KMANAGER_TACK_ALL,
		XVW_GLYPH_REPORTING, (clui_info->hidden_flag == FALSE),
		XVW_GLYPH_ECHO,      (clui_info->silent_flag == FALSE),
		XVW_WORKSPACE_RUNMODE, KWORKSPACE_RUNMODE_CONTINUOUS,
                NULL);

	if (clui_info->hidden_flag == FALSE)
	   xvw_set_attribute(commandbar, XVW_COMMANDBAR_WORKSPACE, workspace);

        if (clui_info->wksp_file)
        {
           wksp_file = clui_info->wksp_file;
        }
        else if (clui_info->tb_string && clui_info->oname_string)
        {
           tbname = clui_info->tb_string;
           oname  = clui_info->oname_string;
	   toolbox = kcms_open_toolbox(tbname);
	   program = kcms_open_cmobj(toolbox, oname);
           if (toolbox == NULL || program == NULL)
           {
              (void) kerror(NULL, "main()", "Cannot open %s '%s'.",
                        (!toolbox) ? "toolbox" : "program",
                        (!toolbox) ? clui_info->tb_string :
                        clui_info->oname_string);
              kexit(KEXIT_FAILURE);
           }
           if (kcms_get_attribute(program, KCMS_CMOBJ_WORKSPACE, &obj) && obj)
              kcms_get_attribute(obj, KCMS_PATH, &wksp_file);
 
           kcms_close(program); kcms_close(toolbox);
        }

	if (clui_info->hidden_flag == FALSE)
	   xvw_add_callback(quit, XVW_BUTTON_SELECT, quit_cb, toplevel);
	else
	   xvw_unmanage(toplevel);

	if (wksp_file != NULL)
	   xvw_set_attribute(workspace, XVW_WORKSPACE_RESTORE, wksp_file);
 
	if (clui_info->args_flag)
	{
	   xvw_get_attribute(workspace, XVW_WORKSPACE_WKSPGUI, &form);
	   if (form != NULL && clui_info->args_string != NULL)
	   {
	      char *tmp;
	      if (clui_info->args_string[0] == '-')
              {
                  tmp = kstring_cat(" ", clui_info->args_string, NULL);
                  xvf_modify_form_from_cmd(form, tmp); kfree_and_NULL(tmp);
              }
	      else xvf_modify_form_from_cmd(form, clui_info->args_string);
	   }
	}

	if (clui_info->showgui_flag == TRUE)
	{
	   xvw_set_attribute(workspace, XVW_WORKSPACE_SHOW_WKSPGUI, TRUE);
           if (clui_info->hidden_flag == TRUE)
           {
              xvw_get_attribute(workspace, XVW_WORKSPACE_WKSPGUI, &form);
              sel_list = form->subform->guide->pane->sel_list;
              close_sel = kvf_find_specified_selection(sel_list,KUIS_QUIT,NULL);
              xvw_add_callback(close_sel->value_object, XVW_BUTTON_SELECT,
                                quit_cb, toplevel);
           }
	}
	else if (clui_info->hidden_flag == TRUE)
	{
           xvw_add_callback(workspace, XVW_WORKSPACE_RUN_CALLBACK,
		workspace_cb, toplevel);
	   xvw_set_attribute(workspace, XVW_WORKSPACE_RUN, TRUE);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: update_status
|
|       Purpose: Updates the status box from the currently status info message.
|
|         Input: toolbox - the toolbox in which the message was issued
|                program - the program in which the message was issued
|                library - the library in which the message was issued
|                routine - the routine in which the message was issued
|                message - the message in which the message was issued
|        Output:
|       Returns:
|    Written By: Mark Young
|          Date: Jul 11, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static int update_status(
   char *toolbox,
   char *program,
   char *library,
   char *routine,
   char *message)
{
        if (status) xvw_set_attribute(status, XVW_TEXT_STRING, message);
        return(TRUE);
}
