/*
 * Khoros: $Id: do_manager.c,v 1.9.2.3 1997/05/24 19:08:40 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_manager.c,v 1.9.2.3 1997/05/24 19:08:40 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>             help_xvregister()
   >>>>             license_xvregister()
   >>>>             select_newsletter_button()
   >>>>             select_upgrade_button()
   >>>>             select_mailinglist_button()
   >>>>             select_org_aerospace_button()
   >>>>             select_org_engineering_button()
   >>>>             select_org_computer_button()
   >>>>             select_org_education_button()
   >>>>             select_org_medical_button()
   >>>>             select_org_manufacturing_button()
   >>>>             select_org_military_button()
   >>>>             select_org_transportation_button()
   >>>>             select_org_other_button()
   >>>>             select_func_educator_button()
   >>>>             select_func_engineer_button()
   >>>>             select_func_mis_button()
   >>>>             select_func_rnd_button()
   >>>>             select_func_other_button()
   >>>>             select_use_visualization_button()
   >>>>             select_use_prototyping_button()
   >>>>             select_use_swd_button()
   >>>>             select_use_rnd_button()
   >>>>             select_use_other_button()
   >>>>             select_register_button()
   >>>>             select_noregister_button()
   >>>>  Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "xvregister.h"

/*-----------------------------------------------------------
|
|  Routine Name: help_xvregister - Callback used to display on line help
|
|       Purpose: A callback is placed on the help button when it is
|                first created.  This callback causes the application
|                to display the on line help page.
|
|         Input: help_button - The GUI object which the callback is installed
|                client_data - Pointer to private data passed by the application
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void help_xvregister(xvobject help_button, kaddr client_data, kaddr call_data)
{
   char help_cmd[KLENGTH];
   char *help_file = "$DESIGN/objects/xvroutine/xvregister/help/xvregister.hlp";

   ksprintf(help_cmd, "khelp -i %s", help_file);
   if (kaccess(help_file, R_OK) == -1)
      kerror("xvregister", "help_xvregister", 
	     "Cannot access help file '%s'", help_file);
   else
   {
      if (kspawn(help_cmd) == -1)
         kerror("xvregister", "help_xvregister", 
	        "Unable to spawn of 'khelp' to display help page");
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: license_xvregister - Callback used to display license agreement
|
|       Purpose: A callback is placed on the license button when it is
|                first created.  This callback causes the application
|                to display the Khoral Research License agreement
|
|         Input: license_button - The GUI object which the callback is installed
|                client_data - Pointer to private data passed by the application
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void license_xvregister(xvobject license_button, 
		       kaddr client_data, kaddr call_data)
{
   char *tbpath;
   char *license_path;
   char license_cmd[KLENGTH];

   tbpath = ktoolbox_path("bootstrap");
   license_path = kstring_cat(tbpath, "/repos/license/License", NULL);
   ksprintf(license_cmd, "khelp -i %s", license_path);

   if (kaccess(license_path, F_OK) == -1)
      kerror("xvregister", "license_xvregister", 
	     "Cannot access license file '%s'", license_path);
   
   else
   {
      if (kspawn(license_cmd) == -1)
         kerror("xvregister", "license_xvregister", 
	        "Unable to spawn of 'klicense' to display license page");
   }
   kfree(license_path);
}

/*-----------------------------------------------------------
|
|  Routine Name: select_newsletter_button - Callback used to change the
|					    label on the newsletter button
|
|       Purpose: A callback is placed on the newsletter button when it is
|                first created.  This callback causes the title on the
|                button to toggle between "Yes" and "No".
|
|         Input: newsletter_button - The GUI object which the callback is 
|				     installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_newsletter_button(
	xvobject newsletter_button, 
	kaddr client_data, 
	kaddr call_data)
{
   char	*button_label;

   xvw_get_attribute(xvreg_gui_info->newsletter, XVW_LABEL, &button_label);
   if (kstrcmp(button_label, "Yes") == 0)
      xvw_set_attributes(xvreg_gui_info->newsletter, 
		         XVW_LABEL, "No",
			 XVW_LABEL_JUSTIFY,    KLABEL_JUSTIFY_CENTER,
		         NULL);
   else
      xvw_set_attributes(xvreg_gui_info->newsletter, 
		        XVW_LABEL, "Yes",
			 XVW_LABEL_JUSTIFY,    KLABEL_JUSTIFY_CENTER,
		         NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: select_upgrade_button - Callback used to change the
|				         label on the upgrade button
|
|       Purpose: A callback is placed on the upgrade button when it is
|                first created.  This callback causes the title on the
|                button to toggle between "Yes" and "No".
|
|         Input: upgrade_button - The GUI object which the callback is 
|			          installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_upgrade_button(
	xvobject upgrade_button, 
	kaddr client_data, 
	kaddr call_data)
{
   char	*button_label;

   xvw_get_attribute(xvreg_gui_info->upgrade, XVW_LABEL, &button_label);
   if (kstrcmp(button_label, "Yes") == 0)
      xvw_set_attributes(xvreg_gui_info->upgrade, 
		         XVW_LABEL, "No",
			 XVW_LABEL_JUSTIFY,    KLABEL_JUSTIFY_CENTER,
		         NULL);
   else
      xvw_set_attributes(xvreg_gui_info->upgrade, 
		        XVW_LABEL, "Yes",
			 XVW_LABEL_JUSTIFY,    KLABEL_JUSTIFY_CENTER,
		         NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: select_mailinglist_button - Callback used to change the
|				             label on the mailinglist button
|
|       Purpose: A callback is placed on the mailinglist button when it is
|                first created.  This callback causes the title on the
|                button to toggle between "Yes" and "No".
|
|         Input: mailinglist_button - The GUI object which the callback is 
|			              installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_mailinglist_button(
	xvobject mailinglist_button, 
	kaddr client_data, 
	kaddr call_data)
{
   char	*button_label;

   xvw_get_attribute(xvreg_gui_info->mailinglist, XVW_LABEL, &button_label);
   if (kstrcmp(button_label, "Yes") == 0)
      xvw_set_attributes(xvreg_gui_info->mailinglist, 
		         XVW_LABEL, "No",
			 XVW_LABEL_JUSTIFY,    KLABEL_JUSTIFY_CENTER,
		         NULL);
   else
      xvw_set_attributes(xvreg_gui_info->mailinglist, 
		        XVW_LABEL, "Yes",
			 XVW_LABEL_JUSTIFY,    KLABEL_JUSTIFY_CENTER,
		         NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: select_org_aerospace_button - Callback used to change the
|				               highlighting on the aerospace 
|					       toggle button
|
|       Purpose: A callback is placed on the aerospace toggle button when it is
|                first created.  This callback causes the button highlighting
|                to reflect whether the button was selected or de-selected
|
|         Input: org_aerospace_button - The GUI object which the callback is 
|			                installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_org_aerospace_button(
	xvobject org_aerospace_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->org_aerospace.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->org_aerospace.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_aerospace.toggle_button,
			 XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->org_aerospace.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->org_aerospace.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_aerospace.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_org_engineering_button - Callback used to change the
|				                 highlighting on the 
|						 engineering toggle button
|
|       Purpose: A callback is placed on the engineering toggle button when it 
|	         is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: org_engineering_button - The GUI object which the callback is 
|			                  installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_org_engineering_button(
	xvobject org_engineering_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->org_engineering.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->org_engineering.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_engineering.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->org_engineering.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->org_engineering.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_engineering.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_org_computer_button - Callback used to change the
|			                      highlighting on the computer 
|					      toggle button
|
|       Purpose: A callback is placed on the computer toggle button when it 
|	         is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: org_computer_button - The GUI object which the callback is 
|			               installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_org_computer_button(
	xvobject org_computer_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->org_computer.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->org_computer.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_computer.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->org_computer.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->org_computer.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_computer.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_org_education_button - Callback used to change the
|			                       highlighting on the education 
|					       toggle button
|
|       Purpose: A callback is placed on the education toggle button when it 
|	         is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: org_education_button - The GUI object which the callback is 
|			                installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_org_education_button(
	xvobject org_education_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->org_education.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->org_education.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_education.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->org_education.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->org_education.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_education.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_org_gis_button - Callback used to change the
|                                      highlighting on the gis 
|                                      toggle button
|
|       Purpose: A callback is placed on the gis toggle button when it 
|              is first created.  This callback causes the button 
|              highlighting to reflect whether the button was selected or 
|              de-selected
|
|         Input: org_gis_button - The GUI object which the callback is 
|                                     installed
|                client_data - Pointer to private data passed by the 
|                            application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Tom Konerth
|          Date: December 18, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_org_gis_button(
      xvobject org_gis_button, 
      kaddr client_data, 
      kaddr call_data)
{
   if (xvreg_gui_info->org_gis.toggle_button_highlighted == 
              TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->org_gis.toggle_button_highlighted = 
              TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_gis.toggle_button,
                       XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->org_gis.toggle_button_highlighted ==
              TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->org_gis.toggle_button_highlighted = 
              TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_gis.toggle_button,
                         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_org_medical_button - Callback used to change the
|		                             highlighting on the medical 
|					     toggle button
|
|       Purpose: A callback is placed on the medical toggle button when it 
|	         is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: org_medical_button - The GUI object which the callback is 
|			              installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_org_medical_button(
	xvobject org_medical_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->org_medical.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->org_medical.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_medical.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->org_medical.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->org_medical.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_medical.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_org_manufacturing_button - Callback used to change the
|		                                   highlighting on the 
|				                   manufacturing toggle button
|
|       Purpose: A callback is placed on the manufacturing toggle button when 
|		 it is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: org_manufacturing_button - The GUI object which the callback 
|					    is installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_org_manufacturing_button(
	xvobject org_manufacturing_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->org_manufacturing.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->org_manufacturing.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_manufacturing.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->org_manufacturing.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->org_manufacturing.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_manufacturing.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_org_military_button - Callback used to change the
|		                              highlighting on the 
|		                              military toggle button
|
|       Purpose: A callback is placed on the military toggle button when 
|		 it is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: org_military_button - The GUI object which the callback is 
|			               installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_org_military_button(
	xvobject org_military_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->org_military.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->org_military.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_military.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->org_military.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->org_military.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_military.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_org_transportation_button - Callback used to change 
|						    the highlighting on the 
|		                                    transportation toggle 
|						    button
|
|       Purpose: A callback is placed on the transportation toggle button when 
|		 it is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: org_transportation_button - The GUI object which the callback 
|					     is installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_org_transportation_button(
	xvobject org_transportation_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->org_transportation.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->org_transportation.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_transportation.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->org_transportation.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->org_transportation.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_transportation.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_org_other_button - Callback used to change the
|		                           highlighting on the 
|		                           org_other toggle button
|
|       Purpose: A callback is placed on the org_other toggle button when 
|		 it is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: org_other_button - The GUI object which the callback is 
|			            installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_org_other_button(
	xvobject org_other_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->org_other.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->org_other.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_other.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->org_other.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->org_other.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->org_other.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_func_educator_button - Callback used to change the
|		                               highlighting on the 
|		                               educator toggle button
|
|       Purpose: A callback is placed on the educator toggle button when 
|		 it is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: func_educator_button - The GUI object which the callback is 
|			                installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_func_educator_button(
	xvobject func_educator_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->func_educator.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->func_educator.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->func_educator.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->func_educator.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->func_educator.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->func_educator.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_func_engineer_button - Callback used to change the
|		                               highlighting on the 
|		                               engineer toggle button
|
|       Purpose: A callback is placed on the engineer toggle button when 
|		 it is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: func_engineer_button - The GUI object which the callback is 
|			                installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_func_engineer_button(
	xvobject func_engineer_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->func_engineer.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->func_engineer.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->func_engineer.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->func_engineer.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->func_engineer.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->func_engineer.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_func_mis_button - Callback used to change the
|		                               highlighting on the 
|		                               mis toggle button
|
|       Purpose: A callback is placed on the mis toggle button when 
|		 it is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: func_mis_button - The GUI object which the callback is 
|			                installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_func_mis_button(
	xvobject func_mis_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->func_mis.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->func_mis.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->func_mis.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->func_mis.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->func_mis.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->func_mis.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_func_rnd_button - Callback used to change the
|		                          highlighting on the 
|		                          rnd toggle button
|
|       Purpose: A callback is placed on the rnd toggle button when 
|		 it is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: func_rnd_button - The GUI object which the callback is 
|			                installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_func_rnd_button(
	xvobject func_rnd_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->func_rnd.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->func_rnd.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->func_rnd.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->func_rnd.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->func_rnd.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->func_rnd.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_func_other_button - Callback used to change the
|		                            highlighting on the 
|		                            func_other toggle button
|
|       Purpose: A callback is placed on the func_other toggle button when 
|		 it is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: func_other_button - The GUI object which the callback is 
|			             installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_func_other_button(
	xvobject func_other_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->func_other.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->func_other.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->func_other.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->func_other.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->func_other.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->func_other.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_use_visualization_button - Callback used to change the
|		                                   highlighting on the 
|		                                   visualization toggle button
|
|       Purpose: A callback is placed on the visualization toggle button when 
|		 it is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: use_visualization_button - The GUI object which the callback 
|					    is installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_use_visualization_button(
	xvobject use_visualization_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->use_visualization.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->use_visualization.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->use_visualization.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->use_visualization.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->use_visualization.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->use_visualization.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_use_prototyping_button - Callback used to change the
|		                                 highlighting on the 
|		                                 prototyping toggle button
|
|       Purpose: A callback is placed on the prototyping toggle button when 
|		 it is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: use_prototyping_button - The GUI object which the callback 
|				          is installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_use_prototyping_button(
	xvobject use_prototyping_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->use_prototyping.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->use_prototyping.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->use_prototyping.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->use_prototyping.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->use_prototyping.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->use_prototyping.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_use_swd_button - Callback used to change the
|	                                 highlighting on the 
|	                                 swd toggle button
|
|       Purpose: A callback is placed on the swd toggle button when 
|		 it is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: use_swd_button - The GUI object which the callback 
|			          is installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_use_swd_button(
	xvobject use_swd_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->use_swd.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->use_swd.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->use_swd.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->use_swd.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->use_swd.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->use_swd.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_use_rnd_button - Callback used to change the
|	                                 highlighting on the 
|	                                 rnd toggle button
|
|       Purpose: A callback is placed on the rnd toggle button when 
|		 it is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: use_rnd_button - The GUI object which the callback 
|			          is installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_use_rnd_button(
	xvobject use_rnd_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->use_rnd.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->use_rnd.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->use_rnd.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->use_rnd.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->use_rnd.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->use_rnd.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_use_other_button - Callback used to change the
|	                                   highlighting on the 
|	                                   use_other toggle button
|
|       Purpose: A callback is placed on the use_other toggle button when 
|		 it is first created.  This callback causes the button 
|		 highlighting to reflect whether the button was selected or 
|		 de-selected
|
|         Input: use_other_button - The GUI object which the callback 
|			            is installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_use_other_button(
	xvobject use_other_button, 
	kaddr client_data, 
	kaddr call_data)
{
   if (xvreg_gui_info->use_other.toggle_button_highlighted == 
		TOGGLE_BUTTON_SELECTED)
   {
      xvreg_gui_info->use_other.toggle_button_highlighted = 
		TOGGLE_BUTTON_NOT_SELECTED;
      xvw_set_attribute(xvreg_gui_info->use_other.toggle_button,
		         XVW_BUTTON_SELECTED, FALSE);
   }
   else if (xvreg_gui_info->use_other.toggle_button_highlighted ==
		TOGGLE_BUTTON_NOT_SELECTED)
   {
      xvreg_gui_info->use_other.toggle_button_highlighted = 
		TOGGLE_BUTTON_SELECTED;
      xvw_set_attribute(xvreg_gui_info->use_other.toggle_button,
		         XVW_BUTTON_SELECTED, TRUE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_register_button - Callback used to collect the
|	         registration information and prompt the user to email
|	         the information or save it to a file.
|
|       Purpose: A callback is placed on the register button when it is
|		 first created.  This callback causes all registration
|		 information to be collected and written to a temporary
|		 file.  It then queries the user to either email the
|		 the registration or save it to a file in the registree's
|		 home directory.
|
|         Input: register_button - The GUI object which the callback 
|			           is installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_register_button(
	xvobject register_button, 
	kaddr client_data, 
	kaddr call_data)
{
   print_fields_struct  *fields_to_print;
   int			gui_status;
   int			submit_status = FALSE;
   int			license_status = FALSE;
   int			registration_status = FALSE;
   int			submit_reg_method;
   char			*temporary_file = NULL;
   int			xvregister_status;
   int			agreed_to_license = FALSE;
   int      		set_registration;
   int      		set_license = UNLICENSED;

   fields_to_print = extract_gui();
   gui_status = fields_to_print->status;

   if (gui_status == NO_NAME_STRING)
   {
      kerror("xvregister", "select_register_button",
             "You did not specify the 'Name' field which is required for registration.  Required fields include:  \n\n\tName, Street Address, City\n\tState/Province, Zip/Postal Code, Country");
      return;
   }
   if (gui_status == NO_ADDRESS_STRING)
   {
      kerror("xvregister", "select_register_button",
             "You did not specify the 'Address' field which is required for registration.  Required fields include:  \n\n\tName, Street Address, City\n\tState/Province, Zip/Postal Code, Country");
      return;
   }
   if (gui_status == NO_CITY_STRING)
   {
      kerror("xvregister", "select_register_button",
             "You did not specify the 'City' field which is required for registration.  Required fields include:  \n\n\tName, Street Address, City\n\tState/Province, Zip/Postal Code, Country");
      return;
   }
   if (gui_status == NO_STATE_STRING)
   {
      kerror("xvregister", "select_register_button",
             "You did not specify the 'State' field which is required for registration.  Required fields include:  \n\n\tName, Street Address, City\n\tState/Province, Zip/Postal Code, Country");
      return;
   }
   if (gui_status == NO_ZIP_STRING)
   {
      kerror("xvregister", "select_register_button",
             "You did not specify the 'Zip' field which is required for registration.  Required fields include:  \n\n\tName, Street Address, City\n\tState/Province, Zip/Postal Code, Country");
      return;
   }
   if (gui_status == NO_COUNTRY_STRING)
   {
      kerror("xvregister", "select_register_button",
             "You did not specify the 'Country' field which is required for registration.  Required fields include:  \n\n\tName, Street Address, City\n\tState/Province, Zip/Postal Code, Country");
      return;
   }
   if (gui_status == NO_ORG_PLEASE_SPECIFY  || 
       gui_status == NO_FUNC_PLEASE_SPECIFY ||
       gui_status == NO_USE_PLEASE_SPECIFY)
   {
      kerror("xvregister", "select_register_button",
             "When selecting 'Other', you must specify something descriptive");
      return;
   }

   if ((temporary_file = print_fields_to_file(fields_to_print)) == NULL)
   {
      kfree(fields_to_print);
      return;
   }
   else
   {
      if (verify_khoros_dotfile() == -1)
      {
         license_status = FALSE;
         registration_status = FALSE;
      }
      else
      {
         registration_status = verify_registration();
	 license_status = verify_licensed();
      }

      if (registration_status == TRUE && license_status == TRUE)
      {
         if ( registration_status == TRUE && license_status == TRUE)
           kinfo(KFORCE, "Your version of Khoros has already been registered " 
			 "and licensed.  Thank you!!");
         kexit(KEXIT_SUCCESS);
      }
      else if (registration_status == TRUE && license_status != TRUE)
      {
        kinfo(KFORCE, "Your version of Khoros has already been registered "
                         "but has not been licensed."); 
      }
   
      if (registration_status != TRUE)
      {
         temporary_file = cleanup_temp_file(temporary_file);
         submit_reg_method = prompt_submit_reg();
         if (submit_reg_method == 1)
            submit_status = email_registration(temporary_file);
         else if (submit_reg_method == 2)
            submit_status = save_registration(temporary_file);

         if (submit_status == FALSE)
         {
            kfree(fields_to_print);
            return;
         }
      }

      if (license_status != TRUE)
      {
         agreed_to_license = show_license();
      }

      if (registration_status != TRUE)
         set_registration = REGISTERED;
      else 
         set_registration = KEEP_OLD_REG_FIELD;

      if (license_status != TRUE && agreed_to_license == TRUE)
         set_license = LICENSED;
      else if (license_status == TRUE || license_status == NOT_CURRENT_VERSION)
         set_license = KEEP_OLD_LIC_FIELD;
      else if (license_status    != TRUE 	        && 
	       license_status    != NOT_CURRENT_VERSION &&
	       agreed_to_license == FALSE)
         set_license = UNLICENSED;

      xvregister_status = 
		update_khoros_dotfile(set_license, set_registration);

      kfree(fields_to_print);

      if (inform_user(xvregister_status))
         kexit(KEXIT_SUCCESS);
      else
	 kexit(KEXIT_FAILURE);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: select_noregister_button - Callback used to update
|	         the users ~/.khoros dotfile to reflect that he/she
|	         is not registered.
|
|       Purpose: A callback is placed on the no register button when it is
|		 first created.  This callback causes the users .khoros
|		 dot file to be updated and reflect that the user is
|		 is not registered.
|
|         Input: register_button - The GUI object which the callback 
|			           is installed
|                client_data - Pointer to private data passed by the 
|			       application (not used)
|                call_data   - Data passed by the GUI object (not used)
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 8, 1996
| Modifications:
|
------------------------------------------------------------*/
void select_noregister_button(
	xvobject register_button, 
	kaddr client_data, 
	kaddr call_data)
{
   int		registration_status = FALSE;
   int		license_status = FALSE;
   int		agreed_to_license = FALSE;
   int		set_registration;
   int		set_license = UNLICENSED;
   int		xvregister_status = FALSE;

   if (verify_khoros_dotfile() == -1)
   {
      license_status = FALSE;
      registration_status = FALSE;
   }
   else
   {
      registration_status = verify_registration();
      license_status = verify_licensed();
   }

   if (registration_status == NOT_CURRENT_VERSION || 
       registration_status == TRUE)
      set_registration = KEEP_OLD_REG_FIELD;
   else
      set_registration = UNREGISTERED;

   if (license_status != TRUE)
   {
      agreed_to_license = show_license();
   }

   if ( registration_status == TRUE && license_status == TRUE)
   {
      kinfo(KFORCE, "Your version of Khoros has already been registered " 
		    "and licensed.  Thank you!!");
      kexit(KEXIT_SUCCESS);
   }

   if ((license_status == NOT_CURRENT_VERSION &&
        agreed_to_license == FALSE)           || 
        license_status == TRUE)
      set_license = KEEP_OLD_LIC_FIELD;
   else if (license_status != TRUE && agreed_to_license == TRUE)
      set_license = LICENSED;
   else if (license_status != TRUE && license_status != NOT_CURRENT_VERSION &&
	    agreed_to_license == FALSE)
      set_license = UNLICENSED;

   xvregister_status = 
	update_khoros_dotfile(set_license, set_registration);

   if (license_status == TRUE)
      set_license = LICENSED;

   if (inform_user(xvregister_status))
      kexit(KEXIT_SUCCESS);
   else
      kexit(KEXIT_FAILURE);
}

