 /*
  * Khoros: $Id: animate.c,v 1.19.2.2 1997/05/31 18:52:22 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: animate.c,v 1.19.2.2 1997/05/31 18:52:22 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for animate
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "animate.h"

clui_info_struct *clui_info = NULL;

gui_info_struct *gui_info;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Interactive Animation of Image Sequence
|
|       Purpose: main program for animate
|
|         Input:
|		Optional M.E. group:
|		    char *clui_info->i_file; {File containing multiple image frames}
|		    int   clui_info->i_flag; {TRUE if -i specified}
|
|		    char *clui_info->base_string; {Basename for a set of image frames in different files}
|		    int   clui_info->base_flag; {TRUE if -base specified}
|
|		char *clui_info->cmap_file; {File containing alternate colormap}
|		int   clui_info->cmap_flag; {TRUE if -cmap specified}
|
|		char *clui_info->clip_file; {File containing clip mask}
|		int   clui_info->clip_flag; {TRUE if -clip specified}
|
|		float clui_info->speed_float; {animation speed}
|		int   clui_info->speed_flag; {TRUE if -speed specified}
|
|		int clui_info->priv_logic; {Use private colormap?}
|		int clui_info->priv_flag; {TRUE if -priv specified}
|
|		int   clui_info->alloc_cycle; {Color allocation policy}
|		char *clui_info->alloc_label;
|		int   clui_info->alloc_flag; {TRUE if -alloc specified}
|
|		int clui_info->x_int; {X location for GUI autoplacement}
|		int clui_info->x_flag; {TRUE if -x specified}
|
|		int clui_info->y_int; {Y location for GUI autoplacement}
|		int clui_info->y_flag; {TRUE if -y specified}
|
|		float clui_info->update_float; {How often to check input file for modification}
|		int   clui_info->update_flag; {TRUE if -update specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

	kform  *form;            /* form tree representing UIS        */
	char   *form_fullpath;   /* for use when [-form] is specified */
/* -main_variable_list */
	xvobject parent, offset;
/* -main_variable_list_end */

	khoros_init(argc, argv, "ENVISION", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$ENVISION/objects/xvroutine/animate");
	kexit_handler(animate_free_args, NULL);

/* -main_get_args_call */
	kclui_init("ENVISION", "animate", KGEN_XVROUTINE,  &clui_uis_spec,
		   animate_usage_additions, animate_get_args,
 		   animate_free_args);
/* -main_get_args_call_end */

	/*
	 * Initialize xvwidgets lib
	 */
	if (!(xvw_initialize(XVW_MENUS_XVFORMS)))
	{
	     kinfo(KSTANDARD, "Could not open connection to server");
	     kexit(KEXIT_FAILURE);
	}

	/*
	 * Allocate the GUI Info structure 
	 */
	gui_info = (gui_info_struct *)
				kcalloc ((size_t) 1, sizeof(gui_info_struct));
	/*
	 * Create the forms for animate 
	 */
	if (clui_info->form_flag)
	{
	    form_fullpath   = kfullpath(clui_info->form_file, NULL, NULL);
	    form = xvf_create_form(form_fullpath, KNONE, run_animate, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	    kfree_and_NULL(form_fullpath);
	}
	else
	{
	    form = xvf_build_form(&gui_uis_spec, KNONE, run_animate, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	}
	if (form == NULL)
	   kexit(KEXIT_FAILURE);

	/*
	 * Initialize the GUI Info structure 
	 */
	if (!(_xvf_init_animate(form, gui_info)))
	    kexit(KEXIT_FAILURE);

/* -main_before_lib_call */
	/*
	 *  Create the animate object
	 */
	animate = xvw_create_animate(gui_info->object, "animate");
	xvw_set_attributes(animate,
		XVW_BORDER_WIDTH, 0,
		XVW_RIGHT_OF, NULL,
		XVW_BELOW, NULL,
		NULL);

	/*
	 *  Create the panicon object
	 */
	offset = xvf_get_xvobject(gui_info->options_struct,XVF_BUTTON_OBJ,TRUE);
	parent = xvf_get_xvobject(gui_info->animate_struct, XVF_BACKPLANE,TRUE);

	panicon = xvw_create_panicon(parent, "panicon");
	xvw_set_attributes(panicon,
		XVW_BELOW,      offset,
		XVW_RIGHT_OF,   NULL,
		XVW_VERT_DIST,  15,
		XVW_HORIZ_DIST, 15,
		NULL);

	framelabel = xvw_create_label(gui_info->object, "label");
	xvw_set_attributes(framelabel,
		XVW_RIGHT_OF, NULL,
		XVW_BELOW,    animate,
		XVW_LABEL,    " Animation Frame Number: ",
		NULL);

	framenum = xvw_create_label(gui_info->object, "framenum");
	xvw_set_attributes(framenum,
		XVW_LABEL,    "0",
		XVW_RIGHT_OF, framelabel,
		XVW_BELOW,    animate,
		NULL);

        position = xvw_create_position(gui_info->object, "position");
	xvw_set_attributes(position,
                XVW_BELOW,                framenum,
                XVW_LEFT_OF,              NULL,
                XVW_RIGHT_OF,             NULL,
                XVW_TACK_EDGE,            KMANAGER_TACK_HORIZ,
                NULL);

	xvw_insert_callback(animate, XVW_ANIMATE_CALLBACK, FALSE,
		update_framenum, (kaddr) framenum);

	startup_from_clui();
/* -main_before_lib_call_end */

/* -main_library_call */
	xvf_run_form();
/* -main_library_call_end */

/* -main_after_lib_call */
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: animate_usage_additions
| 
|       Purpose: Prints usage additions in animate_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname animate
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void animate_usage_additions(void)
{
	kfprintf(kstderr, "\tInteractive Animation of Image Sequence\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: animate_free_args
| 
|       Purpose: Frees CLUI struct allocated in animate_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname animate
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
animate_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->form_file);
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->cmap_file);
		kfree_and_NULL(clui_info->clip_file);
		kfree_and_NULL(clui_info->alloc_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
