/*
  * Khoros: $Id: do_file.c,v 1.10.2.2 1997/05/24 19:05:16 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_file.c,v 1.10.2.2 1997/05/24 19:05:16 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for pane files
   >>>> 
   >>>>  Private: 
   >>>> 	input_newimage
   >>>> 	input_newbasename
   >>>> 	file_i
   >>>> 	file_b
   >>>> 	file_c
   >>>> 	file_cmap
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "animate.h"

/*-----------------------------------------------------------
|
|  Routine Name: input_newimage
|
|       Purpose: Displays a new image; updates all of the zoom,
|                position, printpixel, printmapval, LUT, pseudocolor,
|                and thresholding displays to reflect the new image data.
|
|         Input: filename - ptr to PaneInfo struct for file pane
|        Output:
|    Written By: Mark Young & Danielle Argiro
|          Date: May 26, 1994
| Modifications:
|
------------------------------------------------------------*/
void input_newimage(
     char *filename)
{
        /*
         *  get the data object associated w/ the file just specified
         */
        if ((current_image = kpds_open_object(filename, KOBJ_READ)) == NULL)
        {
           /*
            *  print an error indicating that we failed to open the object
            */
           kerror(NULL, "input_newimage",
                  "Unable to open specified data file '%s'", filename);
	   kinfo(KHOSTILE, "idiot.");
           return;
        }

        if (numimages == 0)
           init_colormap_display(current_image);

        /*
         *  go ahead and add the object to our image list
         */
        images = (kobject *)karray_add(images, KSTRUCT, current_image, (size_t) numimages);
	numimages++;

        /*
         *  set all animate displays to have their objects associated
         *  with the data object associated with the file just read in,
         *  so that we are animating, etc all on the same info.  (currently
	 *  this is only
         */
	xvw_set_attribute(animate,  XVW_IMAGE_IMAGEOBJ,  current_image);
	xvw_set_attribute(panicon,  XVW_IMAGE_IMAGEOBJ,  current_image);
	xvw_set_attribute(position, XVW_POSITION_OBJECT, current_image);
        update_animate_list(numimages-1);

	/*
	 *  set image to have the normalization type, normalization method
	 *  specified on Display pane
	 */
	xvw_set_attribute(animate, XVW_COLOR_NORM_TYPE,
                          gui_info->options->display->norm_type);
	xvw_set_attribute(animate, XVW_COLOR_NORM_METHOD,
                          gui_info->options->display->norm_method_val);
}

/*-----------------------------------------------------------
|
|  Routine Name: input_newbasename
|
|       Purpose: Displays a new sequence; updates all of the auxillary
|                displays to reflect the new sequence.
|
|         Input: name - path to the base name of the sequence to animated
|        Output:
|    Written By: Mark Young
|          Date: Jan 23, 1995
| Modifications:
|
------------------------------------------------------------*/
void input_newbasename(
     char *name)
{
	char	**file;
	int	i;
	size_t  num;
	kobject sequence, tmpobj, result, append[2];
	char    *routine = "input_newbasename";

	/*
	 *  Get all the files in the directory with the specified basename
	 */
	if ((file = karray_dirlist(name, NULL, NULL, KPATH | KFILE,
			           TRUE, &num)) == NULL)
	{
	   kerror(NULL, routine,"No files found using the base name of '%s'.",
		  name);
           kinfo(KHOSTILE, "think!");
	   return;
	}

	/*
	 *  need to assume at least two frames for a sequence
	 */
	if (num < 2)
	{
	    kerror(NULL, routine, "There must be at least two images with "
                   "basename '%s' for the command line option [-base %s] "
                   "to work.", name, name);
	    return;
	}

	/*
	 * create output data object to contain appended sequence
	 */
 	result = kpds_create_object();
	if (result == NULL)
	{
	    kerror(NULL, "file_b", "kpds_create_object() failed to create a "
                      "temporary output object in which to build an output "
                      "animation sequence.");
            kinfo(KHOSTILE, "But it's probably YOUR fault");
	    return;
	}

	/*
	 *  start with the first frame
	 */
	if ((tmpobj = kpds_open_input_object(file[0])) == NULL)
        {
            kerror(NULL, routine, "Failed to open input object '%s'.",
                   file[0]);
	    return;
        }

	/*
	 *  for each frame 1 - N, open the data object with that frame,
	 *  append it onto the data object containing all the previous frames,
	 *  close the data object with that frame.
	 */
	sequence = tmpobj;
	for (i = 1; i < num; i++)
	{
	   if ((tmpobj = kpds_open_input_object(file[i])) == NULL)
	   {
	      kerror(NULL, routine, "Failed to open input object '%s'.",
		     file[i]);
	      kinfo(KHOSTILE, "sorry, charlie!");
	   }

	   append[0] = sequence;
	   append[1] = tmpobj;
	   if ((kapu_append(append, 2, FALSE, FALSE, FALSE, TRUE,
			    FALSE, KAPU_USE_COMMON_MAP, result)) == FALSE)
	   {
	       kerror(NULL, routine, "kapu_append() failed to append basename "
                      "images together in order to create an output animation "
                      "sequence");
               kinfo(KHOSTILE, "But it's probably YOUR fault");
	       kpds_close_object(tmpobj);
	       return;
	   }
	   kpds_close_object(tmpobj);
	   sequence = result;
	}
	karray_free(file, KSTRUCT, num, NULL);
        current_image = sequence;

	/*
	 * initialize colormap
	 */
        if (numimages == 0)
            init_colormap_display(current_image);

        /*
         *  add the object to the image list
         */
        images = (kobject *)karray_add(images, KSTRUCT, current_image, (size_t) numimages);
        numimages++;

        /*
         *  set all animate displays to have their objects associated
         *  with the data object associated with the file just read in,
         *  so that we are animating, etc all on the same info.  (currently
         *  this is only
         */
        xvw_set_attribute(animate,  XVW_IMAGE_IMAGEOBJ, current_image);
        xvw_set_attribute(panicon,  XVW_IMAGE_IMAGEOBJ, current_image);
	xvw_set_attribute(position, XVW_POSITION_OBJECT, current_image);

        update_animate_list(numimages-1);
}

/*-----------------------------------------------------------
| 
|  Routine Name: file_i
| 
|       Purpose: Do routine which is called when
|                  input selection i is used
| 
|         Input: file_info - ptr to PaneInfo struct for files pane
| 
|        Output: None
|    Written By: Mark Young & Danielle Argiro
|          Date: Aug 02, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void file_i(
     File_file *file_info)
{
	static char *old_filename;
        static int count = 0;

        if (count == 0)
            old_filename = kstrdup(clui_info->i_file);

	if (file_info->i == NULL)
        {
            kerror(NULL, "file_i", "You must provide a filename");
	    kinfo(KHOSTILE, "doofus");
            return;
        }
	if (clui_info->update_float > 0.0)
        {
            xvw_remove_detectfile(animate, old_filename, 
				  detectfile_handler, NULL);
            xvw_add_detectfile(animate, file_info->i,
                               clui_info->update_float, 
			       detectfile_handler, NULL);
        }
        input_newimage(file_info->i);

        kfree_and_NULL(old_filename);
        old_filename = kstrdup(file_info->i);
        count++;
}


/*-----------------------------------------------------------
| 
|  Routine Name: file_b
| 
|       Purpose: Do routine which is called when
|                  string selection b is used
| 
|         Input: file_info - ptr to PaneInfo struct for files pane
| 
|        Output: None
|    Written By: Mark Young & Danielle Argiro
|          Date: Aug 02, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void file_b(
     File_file *file_info)
{
	if (file_info->b == NULL)
	{
	    kerror(NULL, "file_b", "You must provide a basename");
	    kinfo(KHOSTILE, "use yer HEAD");
	    return;
	}
	input_newbasename(file_info->b);
}


/*-----------------------------------------------------------
| 
|  Routine Name: file_clip
| 
|       Purpose: Do routine which is called when
|                  input selection clip is used
| 
|         Input: file_info - ptr to PaneInfo struct for files pane
| 
|        Output: None
|    Written By: Mark Young & Danielle Argiro
|          Date: Aug 02, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void file_clip(
     File_file *file_info)
{
	/* note: allow filename of NULL to clear out clip mask */
	if (kstrlen(file_info->clip) > 0) 
            xvw_set_attribute(animate, XVW_IMAGE_CLIPFILE, file_info->clip);
	else xvw_set_attribute(animate, XVW_IMAGE_CLIPFILE, NULL);
}

/*-----------------------------------------------------------
| 
|  Routine Name: file_cmap
| 
|       Purpose: Do routine which is called when
|                  input selection cmap is used
| 
|         Input: file_info - ptr to PaneInfo struct for files pane
| 
|        Output: None
|    Written By: Mark Young & Danielle Argiro
|          Date: Aug 29, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void file_cmap(
     File_file *file_info)
{
	kobject data_object;

        if (kstrlen(file_info->cmap) == 0)
        {
	    xvw_get_attribute(animate, XVW_IMAGE_IMAGEOBJ, &data_object);
	    if (data_object != NULL)
		xvw_set_attribute(animate, XVW_COLOR_COLOROBJ, data_object);
        }
	else xvw_set_attribute(animate, XVW_COLOR_COLORFILE, file_info->cmap);
}


/*-----------------------------------------------------------
| 
|  Routine Name: file_o
| 
|       Purpose: Do routine which is called when
|                  output selection o is used
| 
|         Input: file_info - ptr to PaneInfo struct for files pane
| 
|        Output: None
|    Written By: Mark Young & Danielle Argiro
|          Date: Apr 18, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void file_o(
     File_file *file_info)
{
        kobject object, out;

        if (file_info->o == NULL)
        {
            kerror(NULL, "file_out_file",
                  "You must provide a name for the output file!");
            kinfo(KHOSTILE, "Wasn't that *completely* clear already?");
            return;
        }

        /*
         *  open an output object for writing
         */
        if ((out = kpds_open_object(file_info->o, KOBJ_WRITE)) == NULL)
           return;

        /*
         *  get the data object associated with the image being displayed
         */
        xvw_get_attribute(animate, XVW_IMAGE_IMAGEOBJ, &object);

        /*
         *  copy the attributes & the data of the displayed image
         *  to the output image
         */
        kpds_copy_value_attr(object, out);
        kpds_copy_map_attr(object, out);
        kpds_copy_value_data(object, out, FALSE);
        kpds_copy_map_data(object, out, FALSE);

        /*
         *  that's all... close the output object
         */
        kpds_close_object(out);

        kinfo(KSTANDARD, "Done writing output file to %s", file_info->o);

}


