/*
  * Khoros: $Id: create.c,v 1.11 1997/01/20 06:46:52 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: create.c,v 1.11 1997/01/20 06:46:52 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>         Functionality routines for editcmap
   >>>>
   >>>>  Private: create_cmap_workspace
   >>>>
   >>>>   Static: save_callback
   >>>>           options_callback
   >>>>           quit_handler
   >>>>           quit_callback
   >>>>           help_callback
   >>>>           detectfile_handler
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "editcmap.h"

kobject  value_object      = NULL;
xvobject visual_object     = NULL;
int      colormap_modified = FALSE;

#define USE_PSEUDO 1
#define USE_THRESH 2
#define USE_LUT    3

static void save_callback        PROTO((xvobject, kaddr, kaddr));
static void save_needed_callback PROTO((xvobject, kaddr, kaddr));
static void quit_handler         PROTO((xvobject, kaddr, XEvent *));
static void quit_callback        PROTO((xvobject, kaddr, kaddr));
static void help_callback        PROTO((xvobject, kaddr, kaddr));
static void options_callback     PROTO((xvobject, kaddr, kaddr));
static int  detectfile_handler        PROTO((xvobject, char *, kaddr));


/*-----------------------------------------------------------
|
|  Routine Name: create_cmap_workspace()
|
|       Purpose: Creates the display workspace for editcmap
|         Input:
|        Output:
|       Returns:
|    Written By: Mark Young
|          Date: Sep 08, 1993
| Modifications:
|
------------------------------------------------------------*/
void create_cmap_workspace(void)
{
	char     *infile;
	char     temp[KLENGTH]; 
	xvobject parent, options, license, help, quit, save, palette;

        /*
         *  create parent object
         */
	infile = clui_info->i_file;

        ksprintf(temp, "Editcmap: %s", infile);
        parent = xvw_create_manager(NULL, temp);
        xvw_set_attributes(parent,
                           XVW_WIDTH,  410,
                           XVW_HEIGHT, 270,
                           NULL);
 
	if (clui_info->pseudo_flag)
	    xvw_set_attribute(parent, XVW_CHAR_HEIGHT, 12.0);
	else if (clui_info->thresh_flag)
	    xvw_set_attribute(parent, XVW_CHAR_HEIGHT, 8.0);
	
        /*
         * create Options button which will bring up internal menuform
         */
        options = xvw_create_button(parent, "options");
        xvw_set_attributes(options,
                XVW_BELOW,        NULL,
                XVW_RIGHT_OF,     NULL,
                XVW_LABEL,        "Options",
                NULL);

        /*
         *  create save button
         */
        save = xvw_create_button(parent, "save");
        xvw_set_attributes(save,
                XVW_RIGHT_OF,    options,
                XVW_LABEL,      "Save",
                XVW_CHAR_WIDTH,  5.0,
                NULL);
        xvw_add_callback(save, XVW_BUTTON_SELECT, save_callback, NULL);

        /*
         *  create quit button
         */
        quit = xvw_create_button(parent, "quit");
        xvw_set_attributes(quit,
                XVW_BELOW,        NULL,
                XVW_LEFT_OF,      NULL,
                XVW_LABEL,        "Quit",
                XVW_CHAR_WIDTH,   5.0,
                NULL);
	xvw_add_callback(quit, XVW_BUTTON_SELECT, quit_callback, NULL);

        /*
         *  create help button
         */
        help = xvw_create_button(parent, "help");
        xvw_set_attributes(help,
                XVW_LEFT_OF,     quit,
                XVW_LABEL,      "Help",
                XVW_CHAR_WIDTH,  5.0,
                NULL);
        xvw_add_callback(help, XVW_BUTTON_SELECT, help_callback, NULL);

        /*
         *  create license button
         */
        license = xvw_create_button(parent, "license");
        xvw_set_attributes(license,
                XVW_LEFT_OF,     help,
                XVW_LABEL,      "License",
                XVW_CHAR_WIDTH,  7.0,
                NULL);
        xvw_add_callback(license, XVW_BUTTON_SELECT, help_callback, NULL);

        if (clui_info->i_file)
	   value_object = kpds_open_object(clui_info->i_file, KOBJ_READ);

        if (clui_info->x_flag || clui_info->y_flag)
        {
           xvobject toplevel = xvw_toplevel(parent);
 
           if (clui_info->x_flag)
              xvw_set_attribute(toplevel, XVW_SHELL_X, clui_info->x_int);
           if (clui_info->y_flag)
              xvw_set_attribute(toplevel, XVW_SHELL_Y, clui_info->y_int);
        }

	/*
	 *  [-pseudo]: create pseudocolor object
	 */
        if (clui_info->pseudo_flag)
        {
	   if (!(visual_object = xvw_create_pseudo(parent, "pseudo")))
	   {
	       kerror(NULL, "create_cmap_workspace",
		      "Sorry, can't create pseudo");
	       kexit(KEXIT_FAILURE);
	   }
	   /*-- kludge : force correct initial size --*/
	   xvw_set_attributes(visual_object, 
			XVW_WIDTH,  400,
			XVW_HEIGHT, 260,
			NULL);
	   xvw_set_attribute(visual_object, XVW_TACK_EDGE, KMANAGER_TACK_ALL);
	   xvw_insert_callback(visual_object, XVW_PSEUDO_CALLBACK, FALSE, 
			       save_needed_callback, save);
	   xvw_get_attribute(visual_object, XVW_PSEUDO_PALETTE_OBJECT, 
			     &palette);
        }

	/*
	 *  [-thresh]: create thresholding object
	 */
       else if (clui_info->thresh_flag)
       {
	   if (!(visual_object = xvw_create_threshold(parent, "threshold")))
	   {
	       kerror(NULL, "create_cmap_workspace",
		      "Sorry, can't create threshold");
	       kexit(KEXIT_FAILURE);
           }
	   /*-- kludge : force correct initial size --*/
	   xvw_set_attributes(visual_object, 
			XVW_WIDTH,  400,
			XVW_HEIGHT, 165,
			NULL);
	   xvw_set_attribute(visual_object, XVW_TACK_EDGE, KMANAGER_TACK_ALL);
           xvw_insert_callback(visual_object, XVW_THRESHOLD_CALLBACK, FALSE, 
			       save_needed_callback, save);
	   xvw_get_attribute(visual_object, XVW_THRESHOLD_PALETTE_OBJECT, 
			     &palette);
        }

        xvw_add_callback(options, XVW_BUTTON_SELECT, options_callback, 
			 visual_object);
	xvw_add_action(visual_object, "<Key>q", quit_handler, NULL, TRUE);

	xvw_set_attribute(visual_object, XVW_BELOW, options);

	/*
	 * NOTE! Setting XVW_COLOR_COLORFILE on the palette sub-part of
	 *       the pseudo or threshold object rather than the pseudo
	 *       or threshold object itself is a kludge. This is to
	 *       fix the bug where  pseudo and threshold objects are using
	 *       the default visual, rather than using the visual that 
	 *       matches the data (which might be 24 bit).  Since all the
	 *       color object share a common structure, we now have the
	 *       problem that some visuals may be 8-bit and some 24-bit.
	 *       We should fix this so the XVW_COLOR_COLORFILE can be
	 *       set directly on the threshold or pseudo object.
	 */
	xvw_set_attribute(palette,     XVW_COLOR_COLORFILE, infile);
	xvw_set_attribute(visual_object, XVW_COLOR_COLORFILE, infile);

	/*
	 *  Set private colormap if requested
	 */
	if (clui_info->priv_flag)
            xvw_set_attribute(visual_object, XVW_COLOR_PRIVATE_CMAP, 
			      clui_info->priv_logic);
	/*
	 * set the visual used by the displayed image according to CLUI value
	 */
	if (clui_info->alloc_flag)
            xvw_set_attribute(visual_object, XVW_COLOR_ALLOC_POLICY,
			      clui_info->alloc_cycle);

	/*
	 *  Install input handler on file detect in case file changes.
	 */
	if (clui_info->update_float > 0.0)
        {
           xvw_add_detectfile(visual_object, infile, clui_info->update_float, 
			      detectfile_handler, save);
        }

}

/*-----------------------------------------------------------
|
|  Routine Name: save_callback
|
|       Purpose: Callback to updates output file
|                according to current colormap
|
|         Input: save_button - the save button
|                client_data - not used
|                call_data   - not used
|        Output:
|       Returns:
|    Written By: Mark Young & Danielle Argiro
|          Date: Apr 21, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void save_callback(
   xvobject save_button,
   kaddr    client_data,
   kaddr    call_data)
{
        char    *outfile = clui_info->o_file;
	kobject color_object, cmap_object, out_object;
	char    **answers = NULL;
	char    *label  = "No filename specified with [-o]; please enter one now";
	char    **prompts = NULL;
	
	/*
	 * if no outfile specified on cmd line, get it now
	 */
	if (clui_info->o_file == NULL)
	{
	    answers = (char **) kcalloc((size_t)1, sizeof (char *));
	    prompts = (char **) kcalloc((size_t)1, sizeof (char *));
            prompts[0] = kstrdup("Filename:");
	    answers[0] = '\0';
            if (!(xvu_query_wait(label, prompts, "ok", answers, 1, 30)))
            {
                karray_free(answers, KSTRING, (size_t) 1, NULL);
                karray_free(prompts, KSTRING, (size_t) 1, NULL);
                return;
            }
	    outfile = kstrdup(answers[0]);
	    karray_free(answers, KSTRING, (size_t) 1, NULL);
	    karray_free(prompts, KSTRING, (size_t) 1, NULL);
	    if (outfile == NULL) return;
	    clui_info->o_file = kstrdup(outfile);
	}

	/*
	 *  whether or not we will be writing out only the colormap,
	 *  or both the colormap and other data object info, 
	 *  begin by writing out only the colormap to the output file
	 */
        if (!(xvw_set_attribute(visual_object, XVW_COLOR_SAVEMAP, outfile)))
        {
               kerror(NULL, "save_callback", 
	    	      "Could't save colormap to file '%s'", outfile);
	       return;
        }

	/*
	 * writing out both the colormap and other data object info
	 */
	if (!clui_info->cmap_only_flag)
	{
	    /* read the cmap information back in again */
	    cmap_object = kpds_open_object(outfile, KOBJ_READ);
	    if (cmap_object == NULL)
            {
               kerror(NULL, "save_callback",
                      "Couldn't read output file '%s'", outfile);
               return;
            }

	    /* get other data object info from pseudo or thresh color object */
	    xvw_get_attribute(visual_object, XVW_COLOR_COLOROBJ, &color_object);

	    /* open an output object for writing */
	    out_object = kpds_open_object(outfile, KOBJ_WRITE);
            if (out_object == NULL)
	    {
               kerror(NULL, "save_callback", 
                      "Could't open output object in file '%s'", outfile);
               return;
	    }

	    /* copy data object attributes from color object to output */
	    kpds_copy_object_attr(color_object, out_object);

	    /* copy map attributes and map segmenmt from cmap object */
	    kpds_copy_map_attr(cmap_object, out_object);
	    kpds_copy_map_data(cmap_object, out_object, FALSE);

	    /* copy remaining data from color object */
	    kpds_copy_remaining_data(color_object, out_object);

            kpds_close_object(out_object);
	}

	if (colormap_modified)
	    xvw_set_attribute(save_button, XVW_LABEL, "Save");
	colormap_modified = FALSE;
}

/*-----------------------------------------------------------
|
|  Routine Name: save_needed_callback
|
|       Purpose: Callback fired when pseudocolor or threshold scrollbars move,
|		 updates "Save" button to say "Save (Needed)" (if necessary).
|		 Use of global flag simplifies implementation.
|
|         Input: object      - the save button
|                client_data - not used
|                call_data   - not used
|        Output:
|       Returns:
|    Written By: Mark Young & Danielle Argiro
|          Date: Apr 21, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void save_needed_callback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject save_button = (xvobject) client_data;

	/* button already reads, "Save (Needed)" */
	if (colormap_modified) return;

	xvw_set_attribute(save_button, XVW_LABEL, "Save (Needed)");
	colormap_modified = TRUE;
	
}


/*-----------------------------------------------------------
|
|  Routine Name: quit_handler()
|
|       Purpose: Action handler quits the editcmap program when
|                user hits 'Q' or 'q'.
|
|         Input: object      - the lut, pseudo, or threshold object
|                client_data - not used
|                event       - the 'q' or 'Q' keypress event
|        Output:
|       Returns:
|    Written By: Danielle Argiro
|          Date: Nov 3, 1993
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void quit_handler(
   xvobject object,
   kaddr    client_data,
   XEvent   *event)
{
        xvw_destroy(object);
        kexit(KEXIT_SUCCESS);
}

/*-----------------------------------------------------------
|
|  Routine Name: quit_callback()
|
|       Purpose: Callback quits when user hits quit button
|
|         Input: object      - the image, plot, or other object
|                client_data - not used
|                call_data   - not used
|        Output: None
|       Returns: None
|    Written By: Danielle Argiro
|          Date: March 6, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void quit_callback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
        xvw_destroy(xvw_parent(object));
        kexit(KEXIT_SUCCESS);
}



/*-----------------------------------------------------------
|
|  Routine Name: help_callback()
|
|       Purpose: Callback displays appropriate help page
|                according to the type of visual display being used
|
|         Input: object      - the quit button
|                client_data - not used
|                call_data   - not used
|        Output: None
|       Returns: None
|    Written By: Danielle Argiro
|          Date: March 7, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void help_callback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
        static xvobject help = NULL;
	char *label, *filename;
	static char *last_filename = NULL;
	static char *lfile = "$BOOTSTRAP/repos/license/License";
	static char *hfile = "$ENVISION/objects/xvroutine/editcmap/help/editcmap.hlp";
	
	xvw_get_attribute(object, XVW_LABEL, &label);
	if (kstrcmp(label, "License") == 0)
	     filename = lfile;
	else filename = hfile;

	if (help == NULL)
	{
            help = xvw_create_help(NULL, "Online Help");
            xvw_set_attributes(help,
                               XVW_HELP_TITLE,          "Online Help",
                               XVW_HELP_FILENAME,        filename,
                               XVW_HELP_DESTROY_ON_QUIT, FALSE,
                               NULL);
	}
	else if (!xvw_check_mapped(help))
	     xvw_map(help);
	else xvw_raise(help);

	if (last_filename != filename)
	    xvw_set_attribute(help, XVW_HELP_FILENAME, filename);

	last_filename = filename;
}


/*-----------------------------------------------------------
|
|  Routine Name: options_callback()
|
|       Purpose: Callback for the Options button which will
|                display the internal menuform for the 
|		 pseudocolor or threshold object.
|
|         Input: object      - the options button
|                client_data - not used
|                call_data   - the image, plot, or other object
|        Output: None
|       Returns: None
|    Written By: Mark Young
|          Date: Aug 19, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void options_callback(
    xvobject object,
    kaddr    client_data,
    kaddr    call_data)
{
        xvobject visual_obj = (xvobject) client_data;
 
        xvw_activate_menu(visual_obj);
}

/*-----------------------------------------------------------
|
|  Routine Name: detectfile_handler
|
|       Purpose: This is the input handler that is called when
|                the input file containing the colormap is updated.
|
|         Input: object      - the pseudocolor or threshold object
|		 filename    - name of the file containing colormap
|		 client_data - passes in the save button
|        Output: none
|    Written By: Danielle Argiro
|          Date: June 16, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static int detectfile_handler(
   xvobject object,
   char     *filename,
   kaddr    client_data)
{
        kobject  data_object;
	xvobject save_button = (xvobject) client_data;

        if (kaccess(filename, R_OK) == -1)
           return(TRUE);

        /*
         * get data object from the modified file
         */
        if ((data_object = kpds_open_object(filename, KOBJ_READ)) != NULL)
        {
           /* update displayed pseudocolor or threshold object */
	   xvw_set_attribute(visual_object, XVW_COLOR_COLOROBJ, data_object);
           kpds_close_object(data_object);

	   /* new file hasn't had colormap changed yet */
	   if (colormap_modified)
               xvw_set_attribute(save_button, XVW_LABEL, "Save");
           colormap_modified = FALSE;

	   /* clear if its a pseudocolor object */
	   if (clui_info->pseudo_flag)
		xvw_set_attribute(object, XVW_PSEUDO_CLEAR, TRUE);
        }
        return(TRUE);
}
