/*
  * Khoros: $Id: do_file.c,v 1.9 1997/01/20 06:47:13 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_file.c,v 1.9 1997/01/20 06:47:13 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for pane file
   >>>> 
   >>>>  Private: 
   >>>> 	file_i
   >>>> 	file_clip
   >>>> 	file_cmap
   >>>> 
   >>>> 	file_o
   >>>> 	file_out_cmap
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */



#include "editimage.h"

/*-----------------------------------------------------------
| 
|  Routine Name: file_i
| 
|       Purpose: Called when input selection i is used
| 
|         Input: file_info - ptr to PaneInfo struct for file pane;
|                Inputs a new image to be displayed.
|        Output: 
|    Written By: Mark Young & Danielle Argiro
|          Date: May 26, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void file_i(
     File_file *file_info)
{
	static char *old_filename;
	static int count = 0;

	if (count == 0)
	    old_filename = kstrdup(clui_info->i_file);

	if (file_info->i == NULL)
	{
	    kerror(NULL, "file_i", "You must provide a filename!");
	    kinfo(KHOSTILE, "Feeling particularly slow today?");
	    return;
	}

	/*
         *  get the data object associated w/ the file just specified
         */
        if ((current_image = kpds_open_object(file_info->i, KOBJ_READ)) == NULL)
        {
           /*
            *  print an error indicating that we failed to open the object
            */
           kerror(NULL, "edimg_input_newimage",
                  "Unable to open specified data file '%s'", file_info->i);
           return;
        }

	if (clui_info->update_float > 0.0)
        {
	   xvw_remove_detectfile(image, old_filename, detectfile_handler, NULL);
           xvw_add_detectfile(image, file_info->i,
                              clui_info->update_float, detectfile_handler, NULL);
        }

	/* 
	 *  simply call the routine to input a new image 
	 */
	edimg_input_newimage();

	kfree_and_NULL(old_filename);
	old_filename = kstrdup(file_info->i);

	count++;
}

/*-----------------------------------------------------------
|
|  Routine Name: file_clip
|
|       Purpose: Do routine which is called when
|                  input selection c is used
|
|         Input: file_info - ptr to PaneInfo struct for file pane
|
|        Output: None
|    Written By: Danielle Argiro
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void file_clip(
     File_file *file_info)
{
	set_clipmask(file_info->clip);
}

/*-----------------------------------------------------------
|
|  Routine Name: file_cmap
|
|       Purpose: Do routine which is called when
|                  input selection cmap is used
|
|         Input: file_info - ptr to PaneInfo struct for file pane
|
|        Output: None
|    Written By:
|          Date: Aug 29, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void file_cmap(
     File_file *file_info)
{
        if (file_info->cmap == NULL)
        {
            kerror(NULL, "file_cmap", "You must provide a filename");
            return;
        }
	set_alternate_colormap(file_info->cmap);
}


/*-----------------------------------------------------------
| 
|  Routine Name: file_o
| 
|       Purpose: Called when output selection o is used;
|                Writes the displayed image to the specified output file.
| 
|         Input: file_info - ptr to PaneInfo struct for file pane
|        Output: 
|    Written By: Mark Young & Danielle Argiro
|          Date: May 26, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void file_o(
     File_file *file_info)
{
	kobject iobject, out;

	if (file_info->o == NULL)
        {
            kerror(NULL, "file_out_file", 
		  "You must provide a name for the output file!");
            kinfo(KHOSTILE, "Wasn't that *completely* clear already?");
            return;
        }

	if (file_info->format_label == NULL)
	{
	    kerror(NULL, "file_out_file",
                  "Please select a format for the output file.");
            kinfo(KHOSTILE, "What, I'm supposed to read your mind?");
            return;
	}

        if ((out = kpds_open_object(file_info->o, KOBJ_WRITE)) == NULL)
        {
            kerror(NULL, "file_o", "Unable to open output file '%s'.", 
		   file_info->o);
	    return;
        }

	if (!kpds_set_attribute(out, KPDS_FORMAT, file_info->format_label))
        {
           kerror(NULL, "capture_capture",
                  "Unable to set format (%s) on destination object",
                  file_info->format_label);
           return;
        }

	xvw_get_attribute(image, XVW_IMAGE_IMAGEOBJ, &iobject);

	/*
         *  copy attributes & data of displayed image to output image
         */
        kdms_copy_object(iobject, out, TRUE, TRUE);

        /*
         *  copy current colormap to output image
         */
        xvw_set_attribute(image, XVW_COLOR_CURRENTMAP, out);

        /*
         *  that's all... close the output object
         */
        kpds_close_object(out);

	kinfo(KSTANDARD, "Image written to %s file '%s'\n",
	      file_info->format_label, file_info->o);
}

/*-----------------------------------------------------------
| 
|  Routine Name: file_out_cmap
| 
|       Purpose: Do routine which is called when
|                  output selection out_cmap is used
| 
|         Input: file_info - ptr to PaneInfo struct for file pane
| 
|        Output: None
|    Written By: 
|          Date: Nov 15, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void file_out_cmap(
     File_file *file_info)
{
	if (file_info->out_cmap == NULL)
	{
	    kerror(NULL, "file_out_cmap", "You must provide a filename");
	    kinfo(KHOSTILE, "Idiot!");
	    return;
	}
	xvw_set_attribute(image, XVW_COLOR_SAVEMAP, file_info->out_cmap);
}

