/*
  * Khoros: $Id: do_roi.c,v 1.12.2.3 1997/05/24 19:05:37 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_roi.c,v 1.12.2.3 1997/05/24 19:05:37 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for pane roi
   >>>> 
   >>>>  Private: 
   >>>> 	roi_o
   >>>> 	roi_shape
   >>>> 	roi_extract_roi
   >>>> 	roi_insert_roi
   >>>> 	roi_policy
   >>>> 	roi_mult
   >>>> 	roi_disp
   >>>> 	roi_mode
   >>>> 	roi_pres
   >>>> 
   >>>> 	extract_roi
   >>>> 	display_extracted_roi
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "editimage.h"

static kobject extract_roi        PROTO((options_roi *));
static void display_extracted_roi PROTO((kobject, int));
static void delete_roi_toplevel   PROTO((xvobject, kaddr));

/*-----------------------------------------------------------
|
|  Routine Name: roi_o
|
|       Purpose: Do routine which is called when
|                  output selection o is used
|
|         Input: roi_info - ptr to PaneInfo struct for roi pane
|
|        Output: None
|    Written By:
|          Date: Jan 13, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void roi_o(
     options_roi *roi_info)
{
	kobject object;
	File_file *file_info = gui_info->File->file;

	/*
	 * outputting ROI displayed as image
	 */
	if (roi_info->pres == KIMAGE_ROI_IMAGE)
        {
	    if (disp_image == NULL)
	    {
		kerror(NULL, "file_o", "No ROI to output; use the EXTRACT button to extract an ROI to output");
		return;
	    }
	    if (!(xvw_get_attribute(disp_image, XVW_IMAGE_IMAGEOBJ, &object)))
                return;
	}
	/*
         * outputting ROI displayed as signal
         */
        else if (roi_info->pres == KIMAGE_ROI_SIGNAL)
        {
	    if (disp_plot2d == NULL)
            {
                kerror(NULL, "file_o", "No ROI to output; use the EXTRACT button to extract an ROI to output");
                return;
            }
            if (!(xvw_get_attribute(disp_plot2d, XVW_PLOT2D_PLOTOBJ, &object)))
                return;
        }

	/*
         * outputting ROI displayed as surface
         */
        else if (roi_info->pres == KIMAGE_ROI_SURFACE)
        {
	    if (disp_plot3d == NULL)
	    {
		kerror(NULL, "file_o", "No ROI to output; use the EXTRACT button to extract an ROI to output");
		return;
	    }
            if (!(xvw_get_attribute(disp_plot3d, XVW_PLOT3D_PLOTOBJ, &object)))
                return;
        }

	if (output_image(object, roi_info->o, file_info->format_label))
            kinfo(KSTANDARD, "Displayed ROI written to %s output file %s",
                  file_info->format_label, roi_info->o);
        else kerror(NULL, "file_o", "Unable to write displayed ROI to %s output file '%s'",
                  file_info->format_label, roi_info->o);


}

/*-----------------------------------------------------------
| 
|  Routine Name: roi_shape
| 
|       Purpose: Do routine which is called when
|                  toggle selection shape is used
| 
|         Input: roi_info - ptr to PaneInfo struct for roi pane
| 
|        Output: None
|    Written By: 
|          Date: Dec 16, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void roi_shape(
     options_roi *roi_info)
{
	xvw_set_attribute(image, XVW_IMAGE_ROI_SHAPE, roi_info->shape_val);

	adjust_gui_from_roi_shape(NULL);
}


/*-----------------------------------------------------------
| 
|  Routine Name: roi_extract_roi
| 
|       Purpose: Do routine which is called when
|                  output selection extract is used
| 
|         Input: roi_info - ptr to PaneInfo struct for roi pane
| 
|        Output: None
|    Written By: 
|          Date: Dec 16, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void roi_extract_roi(
     options_roi *roi_info)
{
	kobject  out_object, roi_object;

        /*
         *  open an output object for writing
         */
	if (!roi_info->disp && roi_info->o == NULL)
	{
	    kerror(NULL, "roi_extract_roi", "No output file is specified, "
	           "and Display Extracted ROI is set to False; nowhere to "
		   "extract an ROI!  Please specify output file or set "
		   "Display Extracted ROI to True and try again...");
	    return;
	}

        if (roi_info->o != NULL)
            out_object = kpds_open_output_object(roi_info->o);
        else out_object = kpds_create_object();
        if (out_object == NULL)
        {
           kerror(NULL, "roi_extract_roi",
                  "Failed to create output object");
           return;
        }

        /*
         *  set output file format
         */
        kpds_set_attribute(out_object, KPDS_FORMAT, 
			   gui_info->File->file->format_label);


	/*
	 * extract ROI using mouse (interactive) or keyboard (noninteractive)
	 */
	roi_object = extract_roi(roi_info);
	if (roi_object == NULL) return;

        /*
         *  copy the attributes & the data of the ROI to the output object
         */
        (void) kpds_copy_object(roi_object, out_object);

        /*
         *  go ahead and add the extracted ROI file to our image list
         */
        images = (kobject *)karray_add(images, KSTRUCT, roi_object, (size_t) numimages++);
        edimg_update_image_list(numimages-1);

        /*
         *  we'll be displaying extracted ROI, but have not yet created
         *  the toplevel ROI display object.
         */
        if ((roi_info->disp) && (disp_roi == NULL))
        {
           /*
            *  create manager to be the parent to display extracted ROI
            */
            disp_roi = xvw_create_manager(NULL, "disp_roi");
	    xvw_add_protocol(xvw_toplevel(disp_roi), "WM_DELETE_WINDOW",
                             delete_roi_toplevel, NULL);

           /*
            *  for ROI shape of signal, want an area, axis, & plot object
            */
            disp_area = xvw_create_area(disp_roi, "disp_area");
            xvw_set_attributes(disp_area,
                	       XVW_MINIMUM_WIDTH,  350,
                               XVW_MINIMUM_HEIGHT, 350,
		               XVW_TACK_EDGE,      KMANAGER_TACK_ALL,
                               NULL);
            /*
             * for all other ROI shapes, want an image object
             */
            disp_image = xvw_create_image(disp_roi, "disp_image");
        }

	/*
	 *  Display the extracted ROI as a signal, image, or surface,
	 *  depending on the ROI presentation specified.
	 */
        if (disp_roi != NULL)
	    display_extracted_roi(roi_object, roi_info->pres);

        /*
         *  that's all... close the output object & temp ROI object
         */
        kpds_close_object(out_object);

	if (roi_info->o != NULL)
	    kinfo(KSTANDARD, "Done extracting ROI to output file '%s'",
                  roi_info->o);

}


/*-----------------------------------------------------------
| 
|  Routine Name: roi_insert_roi
| 
|       Purpose: Do routine which is called when
|                  input selection insert is used
| 
|         Input: roi_info - ptr to PaneInfo struct for roi pane
| 
|        Output: None
|    Written By: 
|          Date: Dec 16, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void roi_insert_roi(
     options_roi *roi_info)
{
	/* PUT YOUR CODE HERE ! */
}


/*-----------------------------------------------------------
| 
|  Routine Name: roi_policy
| 
|       Purpose: Do routine which is called when
|                  list selection policy is used
| 
|         Input: roi_info - ptr to PaneInfo struct for roi pane
| 
|        Output: None
|    Written By: 
|          Date: Jan 04, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void roi_policy(
     options_roi *roi_info)
{
	if ((roi_info->policy == KIMAGE_ROI_OUTSIDE) &&
	    (roi_info->pres == KIMAGE_ROI_SURFACE))
	{
	    kerror(NULL, "roi_policy", "It doesn't make sense to set ROI policy to 'Outside Shape' when the ROI Presentation type is 'Surface'; resetting ROI policy to 'Inside Shape'");
            xvf_set_attribute(roi_info->policy_struct, XVF_LIST_VAL, 
			      KIMAGE_ROI_INSIDE); 
	    roi_info->policy = KIMAGE_ROI_INSIDE;
            return;
        }
	else xvw_set_attribute(image, XVW_IMAGE_ROI_POLICY, roi_info->policy);
}


/*-----------------------------------------------------------
| 
|  Routine Name: roi_mult
| 
|       Purpose: Do routine which is called when
|                  logical selection mult is used
| 
|         Input: roi_info - ptr to PaneInfo struct for roi pane
| 
|        Output: None
|    Written By: 
|          Date: Jan 04, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void roi_mult(
     options_roi *roi_info)
{
	/* 
	 * it doesn't make sense to extract multiple images w/ signal
	 * or surface presentation 
	 */
        if ((roi_info->pres != KIMAGE_ROI_IMAGE) &&
            (roi_info->mult == TRUE))
        {
            kerror(NULL, "roi_mult", "Cannot extract multiple images when ROI presentation type is not set to 'Image'");
            xvf_set_attribute(roi_info->mult_struct, XVF_LOGIC_VAL, FALSE);
            roi_info->mult = FALSE;
	    return;
        }
	xvw_set_attribute(image, XVW_IMAGE_ROI_MULTIPLE, roi_info->mult);
	if (roi_info->mult)
	{
	    xvf_set_attribute(roi_info->mode_struct, XVF_TOGGLE_NUM, 1);
	    roi_info->mode_val =  roi_info->mode_num = EXTRACT_W_MOUSE;
            xvf_set_attribute(roi_info->mode_struct->Selptr->toggle_next->next->back_kformstruct,
                             XVF_ACTIVATE, FALSE);
        }
        else
        {
            xvf_set_attribute(roi_info->mode_struct->Selptr->toggle_next->next->back_kformstruct,
                             XVF_ACTIVATE, TRUE);
        }

	adjust_gui_from_roi_mode();
}

/*-----------------------------------------------------------
|
|  Routine Name: roi_disp
|
|       Purpose: Do routine which is called when
|                  logical selection disp is used
|
|         Input: roi_info - ptr to PaneInfo struct for roi pane
|
|        Output: None
|    Written By:
|          Date: Jan 04, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void roi_disp(
     options_roi *roi_info)
{
        if (disp_roi == NULL)
           return;

        /*
         * unmap/map toplevel according to if the user wants the toplevel
         * displayed
         */
        if (!roi_info->disp)
            xvw_unmap(xvw_toplevel(disp_roi));
        else
            xvw_map(xvw_toplevel(disp_roi));
}


/*-----------------------------------------------------------
| 
|  Routine Name: roi_mode
| 
|       Purpose: Do routine which is called when
|                  toggle selection mode is used
| 
|         Input: roi_info - ptr to PaneInfo struct for roi pane
| 
|        Output: None
|    Written By: 
|          Date: Jan 05, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void roi_mode(
     options_roi *roi_info)
{
	adjust_gui_from_roi_mode();
}

/*-----------------------------------------------------------
| 
|  Routine Name: roi_pres
| 
|       Purpose: Do routine which is called when
|                  list selection pres is used
| 
|         Input: roi_info - ptr to PaneInfo struct for roi pane
| 
|        Output: None
|    Written By: 
|          Date: Jan 05, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void roi_pres(
     options_roi *roi_info)
{
	/* 
	 * it doesn't make sense to display signal or surface
	 * if extracting multiple images
	 */
        if ((roi_info->pres != KIMAGE_ROI_IMAGE) &&
            (roi_info->mult == TRUE))
        {
            kerror(NULL, "roi_pres", "Cannot set ROI presentation type to 'Signal' or 'Surface' unless 'Extract Multiple ROIs' is set to 'FALSE'");
            xvf_set_attribute(roi_info->pres_struct, XVF_LIST_VAL,
                              KIMAGE_ROI_IMAGE);
            roi_info->pres = KIMAGE_ROI_IMAGE;
        }

	/*
	 * doesn't make sense to display surface if extracting outside image
	 */
	if ((roi_info->pres != KIMAGE_ROI_IMAGE) &&
            (roi_info->policy == KIMAGE_ROI_OUTSIDE))
        {
            kerror(NULL, "roi_pres", "Cannot set ROI presentation type to 'Signal' or 'Surface' unless 'ROI Policy' is set to 'Inside Shape' or 'Shape Outline'");
            xvf_set_attribute(roi_info->pres_struct, XVF_LIST_VAL,
                              KIMAGE_ROI_IMAGE);
            roi_info->pres = KIMAGE_ROI_IMAGE;
        }
        else xvw_set_attribute(image, XVW_IMAGE_ROI_PRESENTATION, 
				roi_info->pres);
}


/*-----------------------------------------------------------
|
|  Routine Name: extract_roi
|
|       Purpose: Does interactive (mouse) ROI extraction or
|                non-interactive (keyboard) ROI extraction.
|                When ROI extraction is done interactively with the 
|                mouse, xvw_get_attribute() is used; when ROI 
|                extraction is done non-interactively with the 
|                keyboard, xvw_getroi_xxx() is used.  REMEMBER
|                that this routine is duplicated in extractor.
|
|         Input: roi_info - ptr to PaneInfo struct for roi pane
|
|        Output: None
|    Written By: Danielle Argiro
|          Date: Jan 05, 1995
| Modifications:
|
------------------------------------------------------------*/
static kobject extract_roi(
    options_roi *roi_info)
{
	int     x, y, policy;
	kobject roi_object;

        xvw_get_attribute(image, XVW_IMAGE_ROI_POLICY, &policy);

        /*
         * Insertion/Extraction Mode set to Mouse Specification (interactive)
         * simply set the XVW_IMAGE_ROI attribute on the image object
         */
        if (roi_info->mode_val == EXTRACT_W_MOUSE)
        {
            /*
             *  let image object do the interactive ROI extraction,
             *  it'll pass back a temp kobject.
             */
            xvw_get_attribute(image, XVW_IMAGE_ROI, &roi_object);

        }
        /*
         * Insertion/Extraction Mode set to Keyboard Specification 
	 * (non-interactive); set arguments & call xvw_get_specified_roi()
         */
        else
        {
            /* keyboard extraction of a rectangle */
            if (roi_info->shape_val == KIMAGE_ROI_RECTANGLE)
	    {
		x = roi_info->rect_x; y = roi_info->rect_y;
	    }

            else if (roi_info->shape_val == KIMAGE_ROI_CIRCLE)
            {
                x = roi_info->circle_x; y = roi_info->circle_y;
	    }
            else if (roi_info->shape_val == KIMAGE_ROI_ELLIPSE)
            {
                x = roi_info->ellipse_x; y = roi_info->ellipse_y;
            }
            else if (roi_info->shape_val == KIMAGE_ROI_LINE)
            {
		 x = roi_info->line_x1; y = roi_info->line_x2;
            }
            else /* (roi_info->shape_val == KIMAGE_ROI_POLYLINE) ||
                    (roi_info->shape_val == KIMAGE_ROI_FREEHAND) */
            {
                kerror(NULL, "roi_extract_roi",
                       "Sorry, but keyboard (noninteractive) extraction of polyline and freehand ROIs are not supported");
                return(NULL);
            }

            roi_object = xvw_get_specified_roi(image, roi_info->shape_val,
                                   1, policy, x, y,
                                  (unsigned) roi_info->rect_width,
                                  (unsigned) roi_info->rect_height,
                                  (unsigned) roi_info->circle_radius,
                                  (unsigned) roi_info->ellipse_a,
                                  (unsigned) roi_info->ellipse_b,
                                  roi_info->line_x2, roi_info->line_y2);
        }
	return(roi_object);
}

/*-----------------------------------------------------------
|
|  Routine Name: display_extracted_roi
|
|       Purpose: Displays the extracted ROI depending on whether 
|                the ROI Presentation is set to signal, image, or surface.
|                REMEMBER that this routine is replicated both in extractor
|                and in editimage.
|
|         Input: roi_object - The kobject associated w/ extracted roi
|		 pres       - KIMAGE_ROI_SIGNAL, 
|                             KIMAGE_ROI_IMAGE or 
|                             KIMAGE_ROI_SURFACE
|        Output:
|       Returns:
|    Written By: Danielle Argiro
|          Date: Jan 06, 1995
| Modifications:
|
------------------------------------------------------------*/

static void display_extracted_roi(
    kobject roi_object,
    int     pres)
{
	/* display extracted ROI in 2D plot object */
        if (pres == KIMAGE_ROI_SIGNAL)
        {
	    xvw_unmanage(disp_image);
            xvw_manage(disp_area);
            if (disp_plot3d != NULL)
            {
		xvw_destroy(disp_plot3d);
                disp_plot3d = NULL;
            }
            if (disp_axis2d == NULL)
            {
                disp_axis2d = xvw_create_axis2d(disp_area, "disp_axis2d");
                xvw_set_attributes(disp_axis2d, 
				   XVW_GRAPHICS_ATTACH, disp_axis2d, 
                                   XVW_GRAPHICS_VIEWPORT_MIN_X, 0.2,
                                   XVW_GRAPHICS_VIEWPORT_MIN_Y, 0.2,
                                   XVW_GRAPHICS_VIEWPORT_MAX_X, 0.9,
                                   XVW_GRAPHICS_VIEWPORT_MAX_Y, 0.9,
				   NULL);
                disp_plot2d = xvw_create_plot2d(disp_area, "disp_plot2d");
                xvw_set_attribute(disp_plot2d, XVW_GRAPHICS_ATTACH, 
				  disp_axis2d);
            }
            xvw_set_attribute(disp_plot2d, XVW_PLOT2D_PLOTOBJ, roi_object);
	    xvw_set_attribute(disp_area, XVW_AREA_TITLE_STRING, "Signal");
        }

        /* display extracted ROI in image object */
        else if (pres == KIMAGE_ROI_IMAGE)
        {
            xvw_unmanage(disp_area);
            xvw_manage(disp_image);
            if (disp_axis2d != NULL)
            {
                xvw_destroy(disp_axis2d);
                xvw_destroy(disp_plot2d);
                disp_axis2d = NULL;
                disp_plot2d = NULL;
            }
            if (disp_plot3d != NULL)
            {
                xvw_destroy(disp_plot3d);
                disp_plot3d = NULL;
            }
            xvw_set_attribute(disp_image, XVW_IMAGE_IMAGEOBJ, roi_object);
	}

        /* display extracted ROI in 3D plot object */
        else if (pres == KIMAGE_ROI_SURFACE)
        {
	    xvw_unmanage(disp_image);
            xvw_manage(disp_area);
            if (disp_axis2d != NULL)
            {
		xvw_destroy(disp_axis2d);
                xvw_destroy(disp_plot2d);
                disp_axis2d = NULL;
                disp_plot2d = NULL;
            }
            if (disp_plot3d == NULL)
            {
                disp_plot3d = xvw_create_plot3d(disp_area, "disp_plot3d");
                xvw_set_attributes(disp_plot3d,
                                   XVW_PLOT3D_PLOTOBJ,  roi_object,
                                   XVW_PLOT3D_PLOTTYPE, KPLOT3D_MESH,
                                   NULL);
            }
            else xvw_set_attribute(disp_plot3d, XVW_PLOT3D_PLOTOBJ,
                                   roi_object);
	    xvw_set_attribute(disp_area, XVW_AREA_TITLE_STRING, "Surface");
        }

	else 
	{
	    kerror(NULL, "display_extracted_roi",
		   "Unexpected presentation type!");
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: output_image
|
|       Purpose:
|
|         Input: object   - data object to output
|                outfile  - name of output file to write out
|                format   - format for output file.
|
|        Output:
|    Written By: Danielle
|          Date: July 2, 1995
| Modifications:
|
------------------------------------------------------------*/
int output_image(
    kobject  object,
    char     *outfile,
    char     *format)
{
        kobject out;

        /*
         *  open an output object for writing
         */
        if ((out = kpds_open_object(outfile, KOBJ_WRITE)) == NULL)
            return(FALSE);

        /*
         *  set output file format
         */
        if (!(kpds_set_attribute(out, KPDS_FORMAT, format)))
            return(FALSE);

        /*
         *  copy the attributes & the data of the displayed image
         *  to the output image
         */
        kpds_copy_global_attr(object, out);
        kpds_copy_value_attr(object, out);
        kpds_copy_map_attr(object, out);
        kpds_copy_mask_attr(object, out);
        kpds_copy_value_data(object, out, FALSE);
        kpds_copy_map_data(object, out, FALSE);
        kpds_copy_mask_data(object, out, FALSE);
        kcolor_copy_attributes(object, out, KCOLOR_COLORSPACE,
                              KCOLOR_HAS_ALPHA, NULL);

        /*
         *  that's all... close the output object
         */
        kpds_close_object(out);

        return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: delete_roi_toplevel
|
|       Purpose: This window manager protocol handler is installed
|                on the toplevel that displays the extracted ROI;
|                so that if the user uses the window manager to
|                destroy the window, the "disp_roi" variable is
|                properly set to NULL so it will be re-created the
|                next time (and no seg fault will occur)
|
|         Input: object      - the toplevel object
|                client_data - not used
|                client_data - not used
|        Output: 
|    Written By: Danielle Argiro
|          Date: Apri 19, 1996
| Modifications:
|
------------------------------------------------------------*/

static void delete_roi_toplevel(
    xvobject object,
    kaddr    client_data)
{
        xvw_destroy(object);
        disp_roi = NULL;
}

