/*
  * Khoros: $Id: utilities.c,v 1.13.2.3 1997/05/24 19:05:40 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.13.2.3 1997/05/24 19:05:40 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Utility routines for editimage
   >>>> 
   >>>>  Private: 
   >>>> 	edimg_input_newimage
   >>>> 	edimg_startup_from_clui
   >>>> 	edimg_update_image_list
   >>>> 	edimg_update_band_list
   >>>> 	adjust_gui_from_roi_shape
   >>>> 	adjust_gui_from_roi_mode
   >>>> 	set_current_image_object
   >>>> 	set_alternate_colormap
   >>>> 	set_clipmask
   >>>> 	detectfile_handler
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */



#include "editimage.h"


/*-----------------------------------------------------------
| 
|  Routine Name: edimg_input_newimage
| 
|       Purpose: Displays a new image; updates all of the zoom,
|                position, printpixel, printmapval, pseudocolor,
|		 and thresholding displays to reflect the new image data.
| 
|         Input: filename - ptr to PaneInfo struct for file pane
|        Output: 
|    Written By: Mark Young & Danielle Argiro
|          Date: May 26, 1994 
| Modifications: 
| 
------------------------------------------------------------*/
void edimg_input_newimage(void)
{
	int      e = 1, has_alpha, colorspace, depth, visible;
	char     *comment_string, *name;

	xvw_get_attribute(notifier, XVW_NOTIFYWINDOW_VISIBLE, &visible);
	kpds_get_attribute(current_image, KPDS_NAME, &name);
        xvw_format_attribute(notifier, XVW_NOTIFYWINDOW_MESSAGE,
  		             "Loading image '%s'", name);
	if (!visible)
	    xvw_set_attribute(notifier, XVW_NOTIFYWINDOW_VISIBLE, TRUE);

	/*
	 *  set all editimage displays to have their objects associated
         *  with the data object associated with the file just read in,
         *  so that we are displaying, zooming, etc all on the same info.
	 */
        set_current_image_object();


	xvw_set_attribute(printmapval, XVW_PRINTMAPVAL_POLICY, 
			  gui_info->MapValues->mapvals->policy);

	if (numimages == 0)
	   edimg_init_autocolor_display(current_image);

	/*
	 *  go ahead and add the object to our image list
	 */
	images = (kobject *)karray_add(images, KSTRUCT, current_image, 
			    (size_t) numimages++);
	edimg_update_image_list(numimages-1);

	/* 
         * set comment displayed in string on "Comment" pane of Options subform 
         */
	if (kpds_get_attribute(current_image, KPDS_COMMENT, &comment_string))
        {
	   xvf_set_attribute(gui_info->options->comment->comment_string_struct,
			     XVF_STRING_VAL, comment_string);
	   kfree_and_NULL(gui_info->options->comment->comment_string);
           gui_info->options->comment->comment_string = kstrdup(comment_string);
        }

	/*
	 * if an Autocolor Procedure or Colormap Operation was installed
	 * on the last image, it's not going to be installed on the new
	 * image, so reflect that on the GUI.
	 */
	xvw_set_attribute(autocolor_list, XVW_LIST_INDEX, KORIGINAL);
	xvw_set_attribute(cmap_op_list, XVW_LIST_INDEX, -1);

	/*
	 * If the Info pane of Options subform is open, update info displayed.
	 * Otherwise, info will get updated when the user selects the Info 
	 * guide button of the Options subform, since get_image_info was also
	 * added as a callback to the guidebutton with xvf_add_extra_call().
	 */
	if (gui_info->options->information_selected)
            get_image_info(NULL);

	/*
	 *  If any pixels were selected from pseudocolor display,
	 *  need to clear them.
	 */
	xvw_set_attribute(pseudo, XVW_PSEUDO_CLEAR, TRUE);

	/*
	 *  if the incoming image is an RGB image, and we are on an 8-bit
	 *  screen, disable thresholding, pseudocolor, and colormap operations.
	 *  otherwise, re-enable thresholding, pseudocolor, and colormap stuff
	 */
	kpds_get_attribute(current_image, KPDS_VALUE_SIZE, 
			   NULL, NULL, NULL, NULL, &e);
	kcolor_get_attributes(current_image,
                              KCOLOR_HAS_ALPHA, &has_alpha,
                              KCOLOR_COLORSPACE, &colorspace,
                              NULL);
	xvw_get_attribute(image, XVW_DEPTH, &depth);

	/* all this nastiness to ask is it RGB on 8-bit */
	if (depth == 8 && colorspace == KRGB &&
           (e == 3 || (e == 4 && has_alpha)))
	{
	    xvf_set_attribute(gui_info->Thresh_struct, XVF_ACTIVATE, FALSE);
	    if (gui_info->Thresh_selected)
		   xvf_set_attribute(gui_info->Thresh_struct, 
			             XVF_DISPLAY_SUBFORM, FALSE);

	    xvf_set_attribute(gui_info->Pseudo_struct, XVF_ACTIVATE, FALSE);
            if (gui_info->Pseudo_selected)
                   xvf_set_attribute(gui_info->Pseudo_struct,
                                     XVF_DISPLAY_SUBFORM, FALSE);

	    xvf_set_attribute(gui_info->options->colormap_struct,
			      XVF_ACTIVATE, FALSE);
	    if (gui_info->options->colormap_selected)
	           xvf_set_attribute(gui_info->options->capture_struct,
			             XVF_DISPLAY_PANE, TRUE);

	}
	else 
	{
	    xvf_set_attribute(gui_info->Thresh_struct, XVF_ACTIVATE, TRUE);
	    xvf_set_attribute(gui_info->Pseudo_struct, XVF_ACTIVATE, TRUE);
	    xvf_set_attribute(gui_info->options->colormap_struct, XVF_ACTIVATE, 
		              TRUE);
	}
	xvw_set_attribute(notifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
}

/*-----------------------------------------------------------
| 
|  Routine Name: edimg_startup_from_clui
| 
|       Purpose: On startup, makes all changes necessary
|                to properly react to CLUI arguments specified
| 
|         Input: 
|        Output: 
|    Written By: Danielle Argiro
|          Date: Aug 03, 1994 
| Modifications: 
| 
------------------------------------------------------------*/
void edimg_startup_from_clui(void)
{
	char     **format_list;
	int      format_num;

        /*
         * get list of file format types to display on the Files subform
         */
        format_list = kpds_list_output_formats(&format_num);
        xvf_set_attributes(gui_info->File->file->format_struct,
                           XVF_LIST_SIZE,     format_num,
                           XVF_LIST_CONTENTS, format_list,
                           XVF_LIST_INDEX,    0,
                           NULL);
	xvf_set_attributes(gui_info->options->capture->format_struct,
                           XVF_LIST_SIZE,     format_num,
                           XVF_LIST_CONTENTS, format_list,
                           XVF_LIST_INDEX,    0,
                           NULL);

	kfree_and_NULL(gui_info->File->file->format_label);
	kfree_and_NULL(gui_info->options->capture->format_label);
	gui_info->File->file->format_label = kstrdup(format_list[0]);
	gui_info->options->capture->format_label = kstrdup(format_list[0]);
	gui_info->File->file->format = 0;
	gui_info->options->capture->format = 0;

	/*
	 *  input file provided on cmd line: display input file
	 */
	if (clui_info->i_file != NULL) 
	{
	    /*
             *  get the data object associated w/ the file just specified
             */
            if ((current_image = kpds_open_object(clui_info->i_file, 
						  KOBJ_READ)) == NULL)
            {
               /*
                *  print an error indicating that we failed to open the object
                */
               kerror(NULL, "edimg_startup_from_clui",
                      "Unable to open specified data file '%s'", 
		      clui_info->i_file);
               return;
            }
	    edimg_input_newimage();
	    if (clui_info->update_float > 0.0)
            {
                xvw_add_detectfile(image, clui_info->i_file,
                             clui_info->update_float, detectfile_handler, NULL);
            }
	    xvf_set_attribute(gui_info->File->file->i_struct,
			      XVF_FILE_NAME, clui_info->i_file);
	    kfree_and_NULL(gui_info->File->file->i);
	    gui_info->File->file->i = kstrdup(clui_info->i_file);
	}

	/*
	 *  clip mask provided on cmd line: use clip mask
	 */
	if (clui_info->clip_file != NULL) 
	{
	    set_clipmask(clui_info->clip_file);
	    xvf_set_attribute(gui_info->File->file->clip_struct,
                              XVF_FILE_NAME, clui_info->clip_file);
            kfree_and_NULL(gui_info->File->file->clip);
            gui_info->File->file->clip = kstrdup(clui_info->clip_file);
	}

	/*
	 * name of cmap provided on cmd line for use of separate file for cmap
	 */
        if (clui_info->cmap_flag)
        {
	    set_alternate_colormap(clui_info->cmap_file);

            xvf_set_attribute(gui_info->File->file->cmap_struct,
                              XVF_FILE_NAME, clui_info->cmap_file);
            kfree_and_NULL(gui_info->File->file->cmap);
            gui_info->File->file->cmap = kstrdup(clui_info->cmap_file);
        }

	/*
         *  Time interval at which the input file will be checked for
	 *  modification
	 */
        if (clui_info->update_flag)
        {
        }

	/*
	 * install private/default colormap if requested
	 */
	if (clui_info->priv_flag)
	{
	    xvw_set_attribute(image, XVW_COLOR_PRIVATE_CMAP, 
			      clui_info->priv_logic);
	    gui_info->options->display->priv = clui_info->priv_logic;

	    xvf_set_attribute(gui_info->options->display->priv_struct,
			      XVF_LOGIC_VAL, clui_info->priv_logic);
	}

	/*
	 * set the alloc policy according to CLUI value
	 */
	if (clui_info->alloc_flag)
        {
            xvw_set_attribute(image, XVW_COLOR_ALLOC_POLICY,
			      clui_info->alloc_cycle);
	    gui_info->options->display->alloc = clui_info->alloc_cycle;
            xvf_set_attribute(gui_info->options->display->alloc_struct,
                              XVF_LIST_VAL, clui_info->alloc_cycle);
        }

	/*
	 * adjust ROI pane according to initial ROI shape type.
	 */
	xvf_add_extra_call(gui_info->options->roi_struct, 
		           adjust_gui_from_roi_shape, NULL, XVF_CALL_LAST);

	/*
	 * want Info pane to be updated with image info, but not till 
	 * they use the "Info" guide button.
	 */
	xvf_add_extra_call(gui_info->options->information_struct,
                           get_image_info, NULL, XVF_CALL_FIRST);

}

/*-----------------------------------------------------------
| 
|  Routine Name: edimg_update_image_list
| 
|       Purpose: Do routine which are called to update the image and
|		 band lists.
|         Input:
|        Output:
|    Written By: Mark Young
|          Date: Jul 03, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void edimg_update_image_list(
   int image_index)
{
	int  i;
	char **list, *name;
	kform_struct *kformstruct;
	options_lists *lists_info = gui_info->options->lists;

	kformstruct = lists_info->image_list_struct;
	for (i = 0, list = NULL; i < numimages; i++)
	{
	   name = NULL;
	   kpds_get_attribute(images[i], KPDS_NAME, &name);
	   if (!name) name = "(NULL)";
	   list = (char **)karray_insert(list, KSTRING, name, (size_t) i, 
			        KLIST_TAIL, TRUE);
	}
	edimg_update_band_list();
	xvf_set_attribute(kformstruct, XVF_LIST_SIZE, numimages);
	xvf_set_attribute(kformstruct, XVF_LIST_CONTENTS, list);
	xvf_set_attribute(kformstruct, XVF_LIST_INDEX, image_index);
	kfree_and_NULL(list);
}

/*-----------------------------------------------------------
| 
|  Routine Name: edimg_update_band_list
| 
|       Purpose: Do routine which are called to update the band list.
|         Input:
|        Output:
|    Written By: Mark Young
|          Date: Jul 03, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void edimg_update_band_list(void)
{
	options_lists *lists_info = gui_info->options->lists;

	int  i, indx, num;
	kform_struct *kformstruct;
	char **list, temp[KLENGTH];


	xvw_get_attribute(image, XVW_IMAGE_BANDNUM, &indx);
	xvw_get_attribute(image, XVW_IMAGE_BAND_MAXNUM, &num); num++;
	kformstruct = lists_info->band_list_struct;
	for (i = 0, list = NULL; i < num; i++)
	{
	   ksprintf(temp,"Image Band %d", i);
	   list = (char **)karray_add(list, KSTRING, kstrdup(temp), (size_t) i);
	}
	xvf_set_attribute(kformstruct, XVF_LIST_SIZE, num);
	xvf_set_attribute(kformstruct, XVF_LIST_CONTENTS, list);
	xvf_set_attribute(kformstruct, XVF_LIST_INDEX, indx);
	karray_free(list, KSTRING, (size_t) num, NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: adjust_gui_from_roi_shape
|
|       Purpose: This routine appropriately maps and unmaps the float
|                selections that are used for keyboard ROI extraction
|                depending on the ROI shape.  REMEMBER that this routine
|                is duplicated in extractor.
|                
|         Input: clientdata - not used
|        Output: None
|    Written By: Danielle Argiro
|          Date: Jan 04, 1995
| Modifications:
|
------------------------------------------------------------*/
void adjust_gui_from_roi_shape(kaddr clientdata)
{
	options_roi *roi_info = gui_info->options->roi;

	/*
	 *  note i am not doing this by the book, but the quick & dirty way
	 */
	if (roi_info->shape_val == KIMAGE_ROI_RECTANGLE)
	{
	   xvw_unmanage(roi_info->line_x1_struct->Selptr->back);
	   xvw_unmanage(roi_info->line_x2_struct->Selptr->back);
	   xvw_unmanage(roi_info->line_y1_struct->Selptr->back);
	   xvw_unmanage(roi_info->line_y2_struct->Selptr->back);
           xvw_unmanage(roi_info->circle_radius_struct->Selptr->back);
           xvw_unmanage(roi_info->circle_x_struct->Selptr->back);
           xvw_unmanage(roi_info->circle_y_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_x_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_y_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_a_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_b_struct->Selptr->back);

	   xvw_manage(roi_info->rect_width_struct->Selptr->back);
	   xvw_manage(roi_info->rect_height_struct->Selptr->back);
	   xvw_manage(roi_info->rect_x_struct->Selptr->back);
	   xvw_manage(roi_info->rect_y_struct->Selptr->back);
	}
	else if (roi_info->shape_val == KIMAGE_ROI_CIRCLE)
	{
	   xvw_unmanage(roi_info->rect_width_struct->Selptr->back);
	   xvw_unmanage(roi_info->rect_height_struct->Selptr->back);
	   xvw_unmanage(roi_info->rect_x_struct->Selptr->back);
	   xvw_unmanage(roi_info->rect_y_struct->Selptr->back);
           xvw_unmanage(roi_info->line_x1_struct->Selptr->back);
           xvw_unmanage(roi_info->line_x2_struct->Selptr->back);
           xvw_unmanage(roi_info->line_y1_struct->Selptr->back);
           xvw_unmanage(roi_info->line_y2_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_x_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_y_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_a_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_b_struct->Selptr->back);

	   xvw_manage(roi_info->circle_radius_struct->Selptr->back);
	   xvw_manage(roi_info->circle_x_struct->Selptr->back);
	   xvw_manage(roi_info->circle_y_struct->Selptr->back);
	}

        else if (roi_info->shape_val == KIMAGE_ROI_ELLIPSE)
        {
           xvw_unmanage(roi_info->rect_width_struct->Selptr->back);
           xvw_unmanage(roi_info->rect_height_struct->Selptr->back);
           xvw_unmanage(roi_info->rect_x_struct->Selptr->back);
           xvw_unmanage(roi_info->rect_y_struct->Selptr->back);
           xvw_unmanage(roi_info->line_x1_struct->Selptr->back);
           xvw_unmanage(roi_info->line_x2_struct->Selptr->back);
           xvw_unmanage(roi_info->line_y1_struct->Selptr->back);
           xvw_unmanage(roi_info->line_y2_struct->Selptr->back);
           xvw_unmanage(roi_info->circle_radius_struct->Selptr->back);
           xvw_unmanage(roi_info->circle_x_struct->Selptr->back);
           xvw_unmanage(roi_info->circle_y_struct->Selptr->back);

           xvw_manage(roi_info->ellipse_x_struct->Selptr->back);
           xvw_manage(roi_info->ellipse_y_struct->Selptr->back);
           xvw_manage(roi_info->ellipse_a_struct->Selptr->back);
           xvw_manage(roi_info->ellipse_b_struct->Selptr->back);
        }

        else if ((roi_info->shape_val == KIMAGE_ROI_POLYLINE) ||
		 (roi_info->shape_val == KIMAGE_ROI_FREEHAND))
        {
           xvw_unmanage(roi_info->line_x1_struct->Selptr->back);
           xvw_unmanage(roi_info->line_x2_struct->Selptr->back);
           xvw_unmanage(roi_info->line_y1_struct->Selptr->back);
           xvw_unmanage(roi_info->line_y2_struct->Selptr->back);
           xvw_unmanage(roi_info->circle_radius_struct->Selptr->back);
           xvw_unmanage(roi_info->circle_x_struct->Selptr->back);
           xvw_unmanage(roi_info->circle_y_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_x_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_y_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_a_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_b_struct->Selptr->back);
           xvw_unmanage(roi_info->rect_width_struct->Selptr->back);
           xvw_unmanage(roi_info->rect_height_struct->Selptr->back);
           xvw_unmanage(roi_info->rect_x_struct->Selptr->back);
           xvw_unmanage(roi_info->rect_y_struct->Selptr->back);
        }
        else if (roi_info->shape_val == KIMAGE_ROI_LINE) 
        {
           xvw_unmanage(roi_info->circle_radius_struct->Selptr->back);
           xvw_unmanage(roi_info->circle_x_struct->Selptr->back);
           xvw_unmanage(roi_info->circle_y_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_x_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_y_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_a_struct->Selptr->back);
           xvw_unmanage(roi_info->ellipse_b_struct->Selptr->back);
           xvw_unmanage(roi_info->rect_width_struct->Selptr->back);
           xvw_unmanage(roi_info->rect_height_struct->Selptr->back);
           xvw_unmanage(roi_info->rect_x_struct->Selptr->back);
           xvw_unmanage(roi_info->rect_y_struct->Selptr->back);

           xvw_manage(roi_info->line_x1_struct->Selptr->back);
           xvw_manage(roi_info->line_x2_struct->Selptr->back);
           xvw_manage(roi_info->line_y1_struct->Selptr->back);
           xvw_manage(roi_info->line_y2_struct->Selptr->back);
        }

	/*
	 * trick to prevent user from using the keyboard for polygons or
	 * freehand drawing; allow them to use it for anything else
	 */
	if ((roi_info->shape_val == KIMAGE_ROI_POLYLINE) ||
	    (roi_info->shape_val == KIMAGE_ROI_FREEHAND))
	{
	   xvf_set_attribute(roi_info->mode_struct, XVF_TOGGLE_NUM, 1);
	   xvf_set_attribute(roi_info->mode_struct->Selptr->toggle_next->next->back_kformstruct,
			     XVF_ACTIVATE, FALSE);
	   roi_info->mode_val = 1;
           roi_info->mode_num = 1;
	}
	else 
	{
	   xvf_set_attribute(roi_info->mode_struct->Selptr->toggle_next->next->back_kformstruct,
			     XVF_ACTIVATE, TRUE);
	}

	adjust_gui_from_roi_mode();
}

/*-----------------------------------------------------------
|
|  Routine Name: adjust_gui_from_roi_mode
|
|       Purpose: This routine appropriately sensitizes and un-sensitizes
|                the keyboard-ROI-extraction parameters based on whether
|                the "mode" parameter is set to mouse (interactive) or
|                keyboard (non-interactive) ROI extraction. REMEMBER
|		 that this routine is duplicated in extractor.
|         Input: 
|        Output:
|    Written By: Danielle Argiro
|          Date: Jan 04, 1995
| Modifications:
|
------------------------------------------------------------*/

void adjust_gui_from_roi_mode(void)
{
	int set, shape;
	options_roi *roi_info = gui_info->options->roi;

	if (roi_info->mode_val == 1)
	   set = FALSE;
	else set = TRUE;

	shape = roi_info->shape_val;

	if  (shape == KIMAGE_ROI_LINE) 
        {
            xvw_sensitive(roi_info->line_x1_struct->Selptr->back, set);
            xvw_sensitive(roi_info->line_x2_struct->Selptr->back, set);
            xvw_sensitive(roi_info->line_y1_struct->Selptr->back, set);
            xvw_sensitive(roi_info->line_y2_struct->Selptr->back, set);
        }
	else if (shape == KIMAGE_ROI_ELLIPSE)
	{
	    xvw_sensitive(roi_info->ellipse_x_struct->Selptr->back, set);
            xvw_sensitive(roi_info->ellipse_y_struct->Selptr->back, set);
            xvw_sensitive(roi_info->ellipse_a_struct->Selptr->back, set);
            xvw_sensitive(roi_info->ellipse_b_struct->Selptr->back, set);
	}
	else if (shape == KIMAGE_ROI_CIRCLE)
	{
	    xvw_sensitive(roi_info->circle_radius_struct->Selptr->back, set);
            xvw_sensitive(roi_info->circle_x_struct->Selptr->back, set);
            xvw_sensitive(roi_info->circle_y_struct->Selptr->back, set);
	}
	else if (shape == KIMAGE_ROI_RECTANGLE)
	{
            xvw_sensitive(roi_info->rect_width_struct->Selptr->back, set);
            xvw_sensitive(roi_info->rect_height_struct->Selptr->back, set);
            xvw_sensitive(roi_info->rect_x_struct->Selptr->back, set);
            xvw_sensitive(roi_info->rect_y_struct->Selptr->back, set);
        }
}

/*-----------------------------------------------------------
|
|  Routine Name: set_current_image_object
|
|       Purpose: For incoming images, sets the image object on 
|                the image, zoom, panicon, position, printpixel,
|                printmapval, pseudo, and threshold objects. 
|         Input: none
|        Output: none
|    Written By: Danielle Argiro
|          Date: April 17, 1995
| Modifications:
|
------------------------------------------------------------*/

void set_current_image_object(void)
{
        xvw_set_attribute(image,       XVW_IMAGE_IMAGEOBJ,     current_image);
        xvw_set_attribute(zoom,        XVW_IMAGE_IMAGEOBJ,     current_image);
        xvw_set_attribute(panicon,     XVW_IMAGE_IMAGEOBJ,     current_image);
        xvw_set_attribute(position,    XVW_POSITION_OBJECT,    current_image);
        xvw_set_attribute(printpixel,  XVW_PRINTPIXEL_OBJECT,  current_image);
        xvw_set_attribute(printmapval, XVW_PRINTMAPVAL_OBJECT, current_image);
        xvw_set_attribute(pseudo,      XVW_COLOR_COLOROBJ,     current_image);
        xvw_set_attribute(threshold,   XVW_COLOR_COLOROBJ,     current_image);
}

/*-----------------------------------------------------------
|
|  Routine Name: set_alternate_colormap
|
|       Purpose: For incoming colormaps, sets the color object on
|                the image, zoom, panicon, printpixel,
|                printmapval, pseudo, and threshold objects.
|         Input: filename - name of file w/ colormap
|        Output: none
|    Written By: Danielle Argiro
|          Date: April 17, 1995
| Modifications:
|
------------------------------------------------------------*/

void set_alternate_colormap(
   char *filename)
{
	if (kstrlen(filename) == 0)
	    xvw_get_attribute(image, XVW_IMAGE_IMAGEFILE, &filename);

        if (image != NULL)
            xvw_set_attribute(image, XVW_COLOR_COLORFILE, filename);
        if (zoom != NULL)
            xvw_set_attribute(zoom, XVW_COLOR_COLORFILE, filename);
        if (panicon != NULL)
            xvw_set_attribute(panicon, XVW_COLOR_COLORFILE, filename);
        if (printpixel != NULL)
            xvw_set_attribute(printpixel, XVW_COLOR_COLORFILE, filename);
        if (printmapval != NULL)
            xvw_set_attribute(printmapval, XVW_COLOR_COLORFILE, filename);
        if (pseudo != NULL)
            xvw_set_attribute(pseudo, XVW_COLOR_COLORFILE, filename);
        if (threshold != NULL)
            xvw_set_attribute(threshold, XVW_COLOR_COLORFILE, filename);
}

/*-----------------------------------------------------------
|
|  Routine Name: set_clipmask
|
|       Purpose: For incoming clip masks, sets the clip object on
|                the image, zoom, panicon, printpixel, and printmapval 
|		 objects.
|         Input: filename - name of clip file
|        Output: none
|    Written By: Danielle Argiro
|          Date: Aug 4, 1995
| Modifications:
|
------------------------------------------------------------*/

void set_clipmask(
    char *filename)
{
	if (kstrlen(filename) == 0) filename = NULL;

        if (image != NULL)
            xvw_set_attribute(image, XVW_IMAGE_CLIPFILE, filename);
        if (zoom != NULL)
            xvw_set_attribute(zoom,    XVW_IMAGE_CLIPFILE, filename);
        if (panicon != NULL)
            xvw_set_attribute(panicon, XVW_IMAGE_CLIPFILE, filename);
        if (printpixel != NULL)
            xvw_set_attribute(printpixel, XVW_PRINTPIXEL_CLIPFILE, filename);
        if (printmapval != NULL)
            xvw_set_attribute(printmapval, XVW_PRINTMAPVAL_CLIPFILE, filename);
}

/*-----------------------------------------------------------
|
|  Routine Name: detectfile_handler
|
|       Purpose: This is the input handler that is called when
|                the input file containing the image is modified.
|         Input: object      - the image object
|		 filename    - name of file containing image being displayed
|		 client_data - not used
|        Output: none
|    Written By: Danielle Argiro
|          Date: June 16, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int detectfile_handler(
   xvobject object,
   char     *filename,
   kaddr    client_data)
{
        kobject data_object;

        if (kaccess(filename, R_OK) == -1)
           return(TRUE);

        /*
         * get data object from the modified file
         */
        if ((data_object = kpds_open_object(filename, KOBJ_READ)) != NULL)
        {
	   current_image = data_object;
	   edimg_input_newimage();
        }
        return(TRUE);
}

