/*
  * Khoros: $Id: do_pane.c,v 1.10.2.2 1997/04/30 22:38:05 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_pane.c,v 1.10.2.2 1997/04/30 22:38:05 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for pane pane
   >>>> 
   >>>>  Private: 
   >>>> 	pane_i
   >>>> 	pane_o
   >>>> 	pane_policy
   >>>> 	pane_shape
   >>>> 	pane_disp
   >>>> 	pane_mult
   >>>> 	pane_pres
   >>>> 	pane_mode
   >>>> 	pane_extract
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "extractor.h"

xvobject image;
xvobject position;
xvobject panicon;

xvobject disp_roi;
xvobject disp_area;
xvobject disp_axis2d;
xvobject disp_plot2d;
xvobject disp_plot3d;
xvobject disp_image;

/*-----------------------------------------------------------
| 
|  Routine Name: pane_i
| 
|       Purpose: Do routine which is called when
|                  input selection i is used
| 
|         Input: pane_info - ptr to PaneInfo struct for pane pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Dec 19, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pane_i(
     subform_pane *pane_info)
{
	static char *old_filename;
	static int  count = 0;

	if (count == 0)
	    old_filename = kstrdup(clui_info->i_file);

	if (kstrlen(pane_info->i) == 0)
	{
	    kerror(NULL, "pane_i", "Please provide an input filename!");
	    kinfo(KHOSTILE, "idiot!");
	    return;
	}

	xvw_set_attribute(image,    XVW_IMAGE_IMAGEFILE,   pane_info->i);
	xvw_set_attribute(panicon,  XVW_IMAGE_IMAGEFILE,   pane_info->i);
	xvw_set_attribute(position, XVW_POSITION_FILENAME, pane_info->i);

	/*
         *  Remove old input handler; install new input handler on 
	 *  file detect in case file gets modified.
         */
        if (clui_info->update_float > 0.0)
        {
	   xvw_remove_detectfile(image, old_filename, detectfile_handler, NULL);
           xvw_add_detectfile(image, pane_info->i, clui_info->update_float, 
			      detectfile_handler, NULL);
	   kfree_and_NULL(old_filename);
	   old_filename = kstrdup(pane_info->i);
        }
	count++;
}

/*-----------------------------------------------------------
|
|  Routine Name: pane_o
|
|       Purpose: Do routine which is called when
|                  output selection o is used
|
|         Input: pane_info - ptr to PaneInfo struct for pane pane
|
|        Output: None
|    Written By: Danielle Argiro
|          Date: Jan 04, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void pane_o(
     subform_pane *pane_info)
{
	int     dummy;
	kobject roi_object = NULL, out_object;

        if (disp_plot2d != NULL || disp_image != NULL || disp_plot3d != NULL)
        {

            /* open an output object for writing */
            out_object = kpds_open_object(pane_info->o, KOBJ_WRITE);
            if (out_object == NULL)
            {
               kerror(NULL, "main", "Unable to open output file '%s'",
		      (pane_info->o == NULL) ? "(none)" : pane_info->o);
               kexit(KEXIT_FAILURE);
            }

            if (pane_info->pres == KIMAGE_ROI_SIGNAL && disp_plot2d != NULL)
            {
	        xvw_get_attribute(disp_plot2d, XVW_PLOT2D_PLOTOBJ, &roi_object);
            }
            else if (pane_info->pres == KIMAGE_ROI_IMAGE && disp_image != NULL)
            {
	        xvw_get_attribute(disp_image, XVW_IMAGE_IMAGEOBJ, &roi_object);
            }
            else if (pane_info->pres == KIMAGE_ROI_SURFACE &&
		     disp_plot3d != NULL)
            {
	        xvw_get_attribute(disp_plot3d, XVW_PLOT3D_PLOTOBJ, &roi_object);
            }

            if (roi_object != NULL)
            {
	        /*
         	 *  copy the attributes & the data of the displayed image
         	 *  to the output image.  NOTE: for some reason, if you use
	         *  kpds_copy_object here, there will be a bug with output
	         *  of floating point ROI's getting bogus negative values
	         *  in the value segment; this will also happen w/ RGB images.
                 */
                 kpds_copy_global_attr(roi_object, out_object);
                 kpds_copy_value_attr(roi_object, out_object);
                 kpds_copy_map_attr(roi_object, out_object);
                 kpds_copy_mask_attr(roi_object, out_object);
                 kpds_copy_value_data(roi_object, out_object, FALSE);
                 kpds_copy_map_data(roi_object, out_object, FALSE);
                 kpds_copy_mask_data(roi_object, out_object, FALSE);
                 kcolor_copy_attributes(roi_object, out_object, 
					KCOLOR_COLORSPACE, KCOLOR_HAS_ALPHA, 
					NULL);
                /*
                 *  that's all... close the output object & temp ROI object
                 */
                kpds_close_object(out_object);
                kpds_close_object(roi_object);
                return;
            }
            else kpds_close_object(out_object);
        }

	get_roi(image, NULL, NULL, &dummy);
}


/*-----------------------------------------------------------
| 
|  Routine Name: pane_policy
| 
|       Purpose: Do routine which is called when
|                  list selection policy is used
| 
|         Input: pane_info - ptr to PaneInfo struct for pane pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Dec 19, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pane_policy(
     subform_pane *pane_info)
{
	if ((pane_info->policy == KIMAGE_ROI_OUTSIDE) &&
            (pane_info->pres == KIMAGE_ROI_SURFACE))
        {
            kerror(NULL, "roi_policy", "It doesn't make sense to set ROI policy to 'Outside Shape' when the ROI Presentation type is 'Surface'; resetting ROI policy to 'Inside Shape'");
            xvf_set_attribute(pane_info->policy_struct, XVF_LIST_VAL,
                              KIMAGE_ROI_INSIDE);
            pane_info->policy = KIMAGE_ROI_INSIDE;
            return;
        }
        else xvw_set_attribute(image, XVW_IMAGE_ROI_POLICY, pane_info->policy);
}

/*-----------------------------------------------------------
| 
|  Routine Name: pane_shape
| 
|       Purpose: Do routine which is called when
|                  toggle selection shape is used
| 
|         Input: pane_info - ptr to PaneInfo struct for pane pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Dec 19, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pane_shape(
     subform_pane *pane_info)
{
	xvw_set_attribute(image, XVW_IMAGE_ROI_SHAPE, pane_info->shape_val);

	adjust_gui_from_roi_shape(NULL);
}

/*-----------------------------------------------------------
| 
|  Routine Name: pane_disp
| 
|       Purpose: Do routine which is called when
|                  logical selection disp is used
| 
|         Input: pane_info - ptr to PaneInfo struct for pane pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Dec 20, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pane_disp(
     subform_pane *pane_info)
{
	if (disp_roi == NULL)
	   return;

	/* 
	 * unmap/map toplevel according to if the user wants the toplevel
	 * displayed
	 */
	if (!pane_info->disp)
	    xvw_unmap(xvw_toplevel(disp_roi));
	else
	    xvw_map(xvw_toplevel(disp_roi));
}


/*-----------------------------------------------------------
| 
|  Routine Name: pane_mult
| 
|       Purpose: Do routine which is called when
|                  logical selection mult is used
| 
|         Input: pane_info - ptr to PaneInfo struct for pane pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Jan 04, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pane_mult(
     subform_pane *pane_info)
{
	if ((pane_info->pres != KIMAGE_ROI_IMAGE) && 
	    (pane_info->mult == TRUE))
	{
	    kerror(NULL, "pane_mult", "Cannot extract multiple images when ROI presentation type is not set to 'Image'");
	    xvf_set_attribute(pane_info->mult_struct, XVF_LOGIC_VAL, FALSE);
	    pane_info->mult = FALSE;
	}
	else xvw_set_attribute(image, XVW_IMAGE_ROI_MULTIPLE, pane_info->mult);
}


/*-----------------------------------------------------------
| 
|  Routine Name: pane_pres
| 
|       Purpose: Do routine which is called when
|                  list selection pres is used
| 
|         Input: pane_info - ptr to PaneInfo struct for pane pane
| 
|        Output: None
|    Written By: 
|          Date: Jan 05, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pane_pres(
     subform_pane *pane_info)
{
        /*
         * it doesn't make sense to display signal or surface
         * if extracting multiple images
         */
	if ((pane_info->pres != KIMAGE_ROI_IMAGE) &&
            (pane_info->mult == TRUE))
        {
            kerror(NULL, "pane_mult", "Cannot set ROI presentation type to 'Signal' or 'Surface' unless 'Extract Multiple ROIs' is set to 'FALSE'");
            xvf_set_attribute(pane_info->pres_struct, XVF_LIST_VAL, 
 			      KIMAGE_ROI_IMAGE);
            pane_info->pres = KIMAGE_ROI_IMAGE;
        }

        /*
         * doesn't make sense to display surface if extracting outside image
         */
        if ((pane_info->pres != KIMAGE_ROI_IMAGE) &&
            (pane_info->policy == KIMAGE_ROI_OUTSIDE))
        {
            kerror(NULL, "roi_pres", "Cannot set ROI presentation type to 'Signal' or 'Surface' unless 'ROI Policy' is set to 'Inside Shape' or 'Shape Outline'");
            xvf_set_attribute(pane_info->pres_struct, XVF_LIST_VAL,
                              KIMAGE_ROI_IMAGE);
            pane_info->pres = KIMAGE_ROI_IMAGE;
        }
	else xvw_set_attribute(image, XVW_IMAGE_ROI_PRESENTATION, pane_info->pres);
}


/*-----------------------------------------------------------
| 
|  Routine Name: pane_mode
| 
|       Purpose: Do routine which is called when
|                  toggle selection mode is used
| 
|         Input: pane_info - ptr to PaneInfo struct for pane pane
| 
|        Output: None
|    Written By: 
|          Date: Jan 06, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pane_mode(
     subform_pane *pane_info)
{
        adjust_gui_from_roi_mode();
	if (pane_info->mode_val == EXTRACT_W_MOUSE)
             xvw_add_event(image, ButtonPressMask, get_roi, NULL);
	else xvw_remove_event(image, ButtonPressMask, get_roi, NULL);
}


/*-----------------------------------------------------------
| 
|  Routine Name: pane_extract
| 
|       Purpose: Do routine which is called when
|                  pane action button extract is used
| 
|         Input: pane_info - ptr to PaneInfo struct for pane pane
| 
|        Output: None
|    Written By: 
|          Date: Jan 06, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pane_extract(
     subform_pane *pane_info)
{
	int dummy;
	get_roi(image, NULL, NULL, &dummy);
}


