 /*
  * Khoros: $Id: extractor.c,v 1.14.2.2 1997/05/31 18:53:33 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: extractor.c,v 1.14.2.2 1997/05/31 18:53:33 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for extractor
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "extractor.h"

clui_info_struct *clui_info = NULL;

gui_info_struct *gui_info;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Interactive image Region of Interest (ROI) extraction
|
|       Purpose: main program for extractor
|
|         Input:
|		char *clui_info->i_file; {File containing image from which to extract ROI}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Output image in which to save ROI}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int   clui_info->policy_list; {ROI extraction policy}
|		char *clui_info->policy_label;
|		int   clui_info->policy_flag; {TRUE if -policy specified}
|
|		int clui_info->disp_logic; {Display extracted ROI in separate window?}
|		int clui_info->disp_flag; {TRUE if -disp specified}
|
|		int clui_info->mult_logic; {Extract multiple ROIs from the same window?}
|		int clui_info->mult_flag; {TRUE if -mult specified}
|
|		int   clui_info->pres_list; {ROI presentation}
|		char *clui_info->pres_label;
|		int   clui_info->pres_flag; {TRUE if -pres specified}
|
|		int clui_info->shape_toggle; {Desired shape for ROI}
|		int clui_info->shape_flag; {TRUE if -shape specified}
|
|		int clui_info->priv_logic; {Use private colormap?}
|		int clui_info->priv_flag; {TRUE if -priv specified}
|
|		int   clui_info->alloc_cycle; {Color allocation policy}
|		char *clui_info->alloc_label;
|		int   clui_info->alloc_flag; {TRUE if -alloc specified}
|
|		int clui_info->x_int; {X location for GUI autoplacement}
|		int clui_info->x_flag; {TRUE if -x specified}
|
|		int clui_info->y_int; {Y location for GUI autoplacement}
|		int clui_info->y_flag; {TRUE if -y specified}
|
|		float clui_info->update_float; {How often to check input file for modification}
|		int   clui_info->update_flag; {TRUE if -update specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

	kform  *form;            /* form tree representing UIS        */
	char   *form_fullpath;   /* for use when [-form] is specified */
/* -main_variable_list */
/* -main_variable_list_end */

	khoros_init(argc, argv, "ENVISION", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$ENVISION/objects/xvroutine/extractor");
	kexit_handler(extractor_free_args, NULL);

/* -main_get_args_call */
	kclui_init("ENVISION", "extractor", KGEN_XVROUTINE,  &clui_uis_spec,
		   extractor_usage_additions, extractor_get_args,
 		   extractor_free_args);
/* -main_get_args_call_end */

	/*
	 * Initialize xvwidgets lib
	 */
	if (!(xvw_initialize(XVW_MENUS_XVFORMS)))
	{
	     kinfo(KSTANDARD, "Could not open connection to server");
	     kexit(KEXIT_FAILURE);
	}

	/*
	 * Allocate the GUI Info structure 
	 */
	gui_info = (gui_info_struct *)
				kcalloc ((size_t) 1, sizeof(gui_info_struct));
	/*
	 * Create the forms for extractor 
	 */
	if (clui_info->form_flag)
	{
	    form_fullpath   = kfullpath(clui_info->form_file, NULL, NULL);
	    form = xvf_create_form(form_fullpath, KNONE, run_form, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	    kfree_and_NULL(form_fullpath);
	}
	else
	{
	    form = xvf_build_form(&gui_uis_spec, KNONE, run_form, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	}
	if (form == NULL)
	   kexit(KEXIT_FAILURE);

	/*
	 * Initialize the GUI Info structure 
	 */
	if (!(_xvf_init_form(form, gui_info)))
	    kexit(KEXIT_FAILURE);

/* -main_before_lib_call */
	/*
	 *  create the image object
	 */
	image    = xvw_create_image(gui_info->wksp, "image");
	xvw_set_attribute(image, XVW_IMAGE_IMAGEFILE, clui_info->i_file);
	
	if (clui_info->priv_flag)
	    xvw_set_attribute(image, XVW_COLOR_PRIVATE_CMAP, 
			      clui_info->priv_logic);

	/*
	 *  create the position object underneath the image object
	 */
	position = xvw_create_position(gui_info->wksp, "position");
	xvw_set_attributes(position,
			   XVW_BELOW,             image,
			   XVW_LEFT_OF,           NULL,
			   XVW_RIGHT_OF,          NULL,
                           XVW_POSITION_FILENAME, clui_info->i_file,
			   NULL);

	panicon = xvw_create_panicon(gui_info->wksp, "panicon");
        xvw_set_attributes(panicon,
                XVW_BELOW,           NULL,
                XVW_RIGHT_OF,        NULL,
                XVW_IMAGE_IMAGEFILE, clui_info->i_file,
                NULL);
	/*
	 *  adjust GUI according to CLUI arguments, and start
	 */
	startup_from_clui();
/* -main_before_lib_call_end */

/* -main_library_call */
	xvf_run_form();
/* -main_library_call_end */

/* -main_after_lib_call */
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: extractor_usage_additions
| 
|       Purpose: Prints usage additions in extractor_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname extractor
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void extractor_usage_additions(void)
{
	kfprintf(kstderr, "\tInteractive image Region of Interest (ROI) extraction\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: extractor_free_args
| 
|       Purpose: Frees CLUI struct allocated in extractor_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname extractor
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
extractor_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->form_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->policy_label);
		kfree_and_NULL(clui_info->pres_label);
		kfree_and_NULL(clui_info->alloc_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
