/*
  * Khoros: $Id: utilities.c,v 1.7 1997/01/20 06:48:14 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.7 1997/01/20 06:48:14 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>         Functionality routines for getimage
   >>>>
   >>>>  Private:
   >>>>          capture_image()
   >>>>   Static:
   >>>>          find_named_object()
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "getimage.h"

/*-----------------------------------------------------------
|
|  Routine Name: capture_image()
|
|       Purpose: captures an image
|
|         Input: 
|        Output: 
|       Returns: 
|    Written By: Mark Young & Danielle Argiro
|          Date: Apr 03, 1995
| Modifications:
|
------------------------------------------------------------*/
void capture_image(void)
{
	kobject  image;
	xvobject object = NULL;
	char     *format = NULL;
	int      rubberband = FALSE;
	char     *wname = NULL, *fstring = NULL;


	/*
	 * open output object
	 */
	if ((image = kpds_open_object(clui_info->o_file, KOBJ_WRITE)) == NULL)
	{
	   kerror("getimage", "main", "unable to open output file",
		  clui_info->o_file);
	   kexit(KEXIT_FAILURE);
	}

	/*
	 * set file format
	 */
        if      (clui_info->ascii_flag)   
		 format = KASCII,  
		 fstring = "in ascii format";
        else if (clui_info->avs_flag)     
		format = KAVS,    
		fstring = "Application Visualization System (AVS) format";
        else if (clui_info->xbm_flag)     
                 format = KXBM,    
		 fstring = "X11 Bitmap (XBM) format";
        else if (clui_info->pnm_flag)     
                 format = KPNM,    
		 fstring = "Portable Any Map (PNM) format";
        else if (clui_info->viff_flag)    
                 format = KVIFF,   
		 fstring = "VIFF (Khoros 1.0) format (xvimage)";
        else if (clui_info->eps_flag)     
                 format = KEPS,    
		 fstring = "in Encapsulated PostScript (EPS) Format";
        else if (clui_info->xwd_flag)     
                 format = KXWD,    
		 fstring = "X11 Window Dump (XWD) format";
        else if (clui_info->raw_flag)     
                 format = KRAW,    
		 fstring = "Raw (headerless) format";
        else if (clui_info->xpm_flag)     
                 format = KXPM,    
		 fstring = "X Window Pixmap (XPM) format";
        else if (clui_info->rast_flag)    
                 format = KRASTER, 
		 fstring = "Sun Raster format";
        else format = KDF,
             fstring = "KDF (Khoros 2.1) Format";

	if (!kpds_set_attribute(image, KPDS_FORMAT, format))
        {
           kerror("kformats", "main",
                  "Unable to set format (%s) on destination object", format);
           kfree_and_NULL(format);
           kexit(KEXIT_FAILURE);
        }

	/*
	 * if object is NULL, need them to click on the window to save.
	 */
	if (clui_info->win_flag)
	{
           /*
            * Get the entire window rather than a portion or subimage of one.
            */
	   kfprintf(kstderr,
		    "\nPlease click on the window to be captured.\n\n");
	}

	/*
	 *  Get the object depending on whether they specified the name of
	 *  window or the entire screen.  This will be passed to xvw_getimage,
	 *  which will do the actual work of capturing the window's image.
	 */
	if (clui_info->screen_flag == TRUE)
	   object = xvw_getobject(NULL, xvw_rootwindow(NULL), NULL);
	else
	{
	   if (clui_info->wname_flag) wname = clui_info->wname_string;
	   object = xvw_getobject(NULL, (Window) KNONE, wname);
	}

	/*
	 *  rubberbanding a portion of the screen or the specified window
	 */
	if (!clui_info->all_flag)
	{
           /*
            * Get a portion or subimage of the window/screen rather than
            * entire one.
            */
           kfprintf(kstderr,"\nPlease rubberband the portion of the screen \nto be captured as a %s file.\n\n", fstring);
	   rubberband = TRUE;
	}

	/*
	 *  Now that we have opened the output data object, call xvw_getimage
	 *  to do the actual work of getting the screen dump.  We pass in
	 *  object for the xvobject (the first parameter), since this will
	 *  be the object in which we will be getting the image for.
	 */
	if (!xvw_getimage(object, rubberband, clui_info->decor_flag, image))
	{
	   kpds_close_object(image);
	   kunlink(clui_info->o_file);
	   kexit(KEXIT_FAILURE);
	}
	kpds_close_object(image);
}
