/*
  * Khoros: $Id: class.c,v 1.8.2.1 1997/05/24 19:06:25 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: class.c,v 1.8.2.1 1997/05/24 19:06:25 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>      Utility Routines for Shuffling Classes
   >>>>
   >>>>  Private:
   >>>>           spc_delete_or_empty
   >>>>           spc_hide_or_show
   >>>>           spc_catch_all
   >>>>           spc_transfer_classes
   >>>>           spc_refresh_class_colors
   >>>>           spc_empty_class
   >>>>           spc_hide_or_show_class
   >>>>   Static:
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "spectrum.h"


/*-----------------------------------------------------------
|
|  Routine Name: spc_delete_or_empty
|
|       Purpose: Prompts the user for the class(es) to be
|                either deleted or emptied, and either deletes
|                or empties them, depending on flag.
|
|         Input: flag - one of SPC_DELETE or SPC_EMPTY
|        Output: none
|       Returns: TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: June 13, 1993
| Modifications: 
|
------------------------------------------------------------*/
int spc_delete_or_empty(
     int flag)
{
        char            **classnames; 
        char            *error_mesg_none, *prompt, *label;
	char            temp[KLENGTH], temp2[KLENGTH];
        int             i, num, class_flag;
	size_t          size; 
        LegendEntry     *legend_ptr;
        xvw_list_struct **list_return;

        char *error_mesgND = "No classes to delete",
             *error_mesgNE = "No classes to empty",
             *promptD1     = "Select classes to delete",
             *promptE1     = "Select classes to empty",
             *promptD2     = "Delete class(es): ",
             *promptE2     = "Empty class(es): ",
             *labelD       = "Current classes",
             *labelE       = "Current Classes w/ Clusters";

        /* delete */
        if (flag == SPC_DELETE)
        {
            error_mesg_none = error_mesgND;
            prompt = promptD1;
            label  = labelD;
            class_flag = SPC_ALL;
        }

        /* empty */
        else if (flag == SPC_EMPTY)
        {
            error_mesg_none = error_mesgNE;
            prompt = promptE1;
            label  = labelE;
            class_flag = SPC_NOTEMPTY;
        }

	/* internal error: bogus flag */
	else 
	{
	    errno = SPC_INTERNAL;
	    kerror(NULL, "spc_delete_or_empty",
		   "Call to spc_delete_or_empty() with bogus flag");
	    kinfo(KHOSTILE, "The authors of this code were brain damaged.");
	    return(FALSE);
	}

	/* user error: bogus attempt to hide or show */
        if (spc_legend_list == NULL)
        {
	    errno = SPC_USER_ERROR;
            kerror(NULL, "spc_delete_or_empty", error_mesg_none);
	    kinfo(KHOSTILE, "Use your brain (if you have one).");
            return(FALSE);
        }
        classnames = 
	    spc_create_strings_class_list("ALL of the Classes Below", 
					   class_flag, &size, NULL, 
					   (size_t) 0);

	/*
         * internal error: since we added "All Classes" to the list, there
         * should ALWAYS be at least that one entry
         */
	if (size == 0)
	{
	    errno = SPC_INTERNAL;
            kerror(NULL, "spc_delete_or_empty", "Empty class list");
            return(FALSE);
	}

	/*
         * If there was one thing in the list (ie, "All Classes"), that implies
         * that there are no classes to empty or delete
         */
	else if (size == 1)
	{
            errno = SPC_USER_ERROR;
	    kerror(NULL, "spc_delete_or_empty", error_mesg_none);
	    return(FALSE);
	}

	/*
         * only one class; delete or empty the one immediately
         */
        else if (size == 2)
        {
            legend_ptr = spc_find_legend_from_classname(classnames[1]);
	    if (legend_ptr == NULL)
	    {
                errno = SPC_INTERNAL;
	        kerror(NULL, "spc_delete_or_empty", "Can't find legend ptr");
                return(FALSE);
	    }
            if (flag == SPC_DELETE)
	    {
                ksprintf(temp, "Delete class '%s'?", 
			 legend_ptr->classname);
                if (!(kprompt(KSTANDARD, "Ok", "Cancel", 0, temp)))
                    return(FALSE);
                spc_delete_legend_entry(legend_ptr);
	    }
            else if (flag == SPC_EMPTY)
	    {
                ksprintf(temp, "Empty class '%s'?", 
			 legend_ptr->classname);
                if (!(kprompt(KSTANDARD, "Ok", "Cancel", 0, temp)))
                    return(FALSE);
                spc_empty_class(legend_ptr);
	    }
        }
        else
        {
            /*
             * prompt for class(es) including option to do them all
             */
            list_return = xvu_run_list_multsel_wait(classnames, size, prompt,
					            label, FALSE, FALSE, &num);
            if (list_return == NULL) return(FALSE);

	    /*
	     * complicated code to print a nice prompt
	     */
            if (flag == SPC_DELETE)     
		ksprintf(temp2, promptD2);
            else if (flag == SPC_EMPTY) 
		ksprintf(temp2, promptE2);
            for (i = 0; i < num; i++)
            {
                ksprintf(temp, "%s", list_return[i]->string);
                if (i < num-1) 
		    kstrcat(temp, ", ");
                kstrcat(temp2, temp);
            }
            kstrcat(temp2, "?");
            if (!(kprompt(KSTANDARD, "Ok", "Cancel", 1, temp2)))
                return(FALSE);

            /*
             * user wants to delete/empty all at once
             */
            if (kstrcmp(list_return[0]->string,"ALL of the Classes Below")== 0)
            {
                for (i = 1; i < size; i++)
                {
                    legend_ptr = spc_find_legend_from_classname(classnames[i]);
		    if (flag == SPC_DELETE)
                        spc_delete_legend_entry(legend_ptr);
                    else if (flag == SPC_EMPTY)
                        spc_empty_class(legend_ptr);
                }
            }
            /*
             * user wants to delete/empty specific ones
             */
	    else
	    {
                for (i = 0; i < num; i++)
                {
                    legend_ptr = 
			spc_find_legend_from_classname(list_return[i]->string);
                    if (flag == SPC_DELETE)
                        spc_delete_legend_entry(legend_ptr);
                    else if (flag == SPC_EMPTY)
                        spc_empty_class(legend_ptr);
                }
	    }
            karray_free(list_return, KSTRUCT, (size_t) num, NULL);
        }

        karray_free(classnames, KSTRING, size, NULL);
	return(TRUE);
}



/*-----------------------------------------------------------
|
|  Routine Name: spc_hide_or_show
|
|       Purpose: Prompts the user for the class(es) to be
|                either hidden or shown, and either hides
|                or shows them, depending on flag.
|
|         Input: flag - one of SPC_HIDDEN or SPC_DISPLAYED
|        Output: none
|       Returns: TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: June 13, 1993
| Modifications: 
|
------------------------------------------------------------*/
int spc_hide_or_show(
    int flag)
{
        int    i, num, class_flag;
	size_t size; 
        LegendEntry *legend_ptr;
        xvw_list_struct **list_return;
        char **classnames;
        char *error_mesg_none, *error_mesg_all, *prompt, *label;
        char *error_mesgNH = "No classes to hide.",
             *error_mesgNS = "No classes to show.",
             *error_mesgAH = "All classes already hidden.",
             *error_mesgAS = "All classes already shown.",
             *promptH = "Select classes to hide",
             *promptS = "Select classes to display",
             *labelH  = "Currently Displayed Classes",
             *labelS  = "Currently Hidden Classes";

	/* hiding the class */
        if (flag == SPC_HIDDEN)
        {
            error_mesg_none = error_mesgNH;
            error_mesg_all  = error_mesgAH;
            prompt = promptH;
            label  = labelH;
            class_flag = SPC_DISPLAYED;
        }

	/* showing the class */
        else if (flag == SPC_DISPLAYED)
        {
            error_mesg_none = error_mesgNS;
            error_mesg_all  = error_mesgAS;
            prompt = promptS;
            label  = labelS;
            class_flag = SPC_HIDDEN;
        }

	/* internal error: bogus flag */
        else
        {
	    errno = SPC_INTERNAL;
            kerror(NULL, "spc_hide_or_show",
                   "Call to spc_hide_or_show() with bogus flag");
	    kinfo(KHOSTILE, "The authors of this code were terminally confused.");
            return(FALSE);
        }

	/* user error: bogus attempt to hide or show */
        if (spc_legend_list == NULL)
        {
	    errno = SPC_USER_ERROR;
            kerror(NULL, "spc_hide_or_show", error_mesg_none);
	    kinfo(KHOSTILE, "You obviously don't have a clue what's going on.");
            return(FALSE);
        }

	/* create list of classes that can be hidden or shown */
        classnames = 
	    spc_create_strings_class_list("ALL of the Classes Below", 
	 				   class_flag, &size, NULL,
					   (size_t) 0);
	/* 
	 * internal error: since we added "All Classes" to the list, there
	 * should ALWAYS be at least that one entry
	 */
        if (size == 0)
        {
            errno = SPC_INTERNAL;
            kerror(NULL, "spc_hide_or_show", "Empty class list");
            return(FALSE);
        }

	/* 
	 * If there was one thing in the list (ie, "All Classes"), that implies 
         * that there are classes, but they are all hidden (if user is trying 
         * to hide) or all shown (if user is trying to show)
         */
	else if (size == 1)
	{
            errno = SPC_USER_ERROR;
            kerror(NULL, "spc_hide_or_show", error_mesg_all);
            kinfo(KHOSTILE, "Perhaps you should go home and try again when you're not so stupid.");
	    return(FALSE);
	}

	/* 
	 * only one class; hide or show the one immediately 
	 */
        else if (size == 2)
        {
            legend_ptr = spc_find_legend_from_classname(classnames[1]);
	    spc_hide_or_show_class(legend_ptr, flag);
        }

	/*
	 * more than one class to potentially be hidden or shown
	 */
        else
        {
	    /*
	     * prompt for class(es) including option to do them all
	     */
            list_return = xvu_run_list_multsel_wait(classnames, size, prompt, 
						    label, FALSE, FALSE, &num);
            if (list_return == NULL) return(FALSE);

	    /*
	     * user wants to hide/show all at once
	     */
	    if (kstrcmp(list_return[0]->string, "ALL of the Classes Below") == 0)
	    {
                for (i = 1; i < size; i++)
                {
                    legend_ptr = spc_find_legend_from_classname(classnames[i]);
	            spc_hide_or_show_class(legend_ptr, flag);
                }
	    }

	    /*
	     * user wants to hide/show specific classes
	     */
	    else
	    {
                for (i = 0; i < num; i++)
                {
                    legend_ptr = spc_find_legend_from_classname(list_return[i]->string);
	            spc_hide_or_show_class(legend_ptr, flag);
                }
            }
            karray_free(list_return, KSTRUCT, (size_t) num, NULL);
	}
        karray_free(classnames, KSTRING, (size_t) size, NULL);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: spc_catch_all
|
|       Purpose: dumps all unassigned clusters into a "catch-all" class.
|
|         Input: none
|        Output: none
|       Returns: TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: June 13, 1993
| Modifications: 
|
------------------------------------------------------------*/


int spc_catch_all(void)
{
        int          i, found = FALSE;
	char         prompt[KLENGTH];
        LegendEntry *legend_ptr = NULL;


	/*	
	 *  make sure there is at least one cluster to add to a catch-all class
	 */
	i = 0;
	while ((i < spc_map_rownum) && (!found))
	{
	   if (spc_legend_lookup[i] == NULL) found = TRUE;
	   i++;
	}
	if (!found)
	{
	    errno = SPC_USER_ERROR;
	    kerror(NULL, "spc_catch_all", 
		   "All clusters have already been assigned");
	    return(TRUE);
	}

	/*
	 *  establish appropriate prompt based on whether class exists yet
	 */
	legend_ptr = spc_find_legend_from_classname("Catch-All Class");
	if (legend_ptr == NULL)
	   ksprintf(prompt, "Are you sure you want to dump all remaining clusters into a Catch-All Class (to be created)?");
	else  ksprintf(prompt, "Are you sure you want to dump all remaining clusters into the (existing) Catch-All Class?");
 
	/*
	 *  make sure they really want to do this 
	 */
        if (!(kprompt(KSTANDARD, "Yes", "Cancel", 1, prompt)))
            return(FALSE);

	/*
	 *  create the catch-all class
	 */
	if (legend_ptr == NULL)
            legend_ptr = spc_add_legend_entry(-1, 1, 1, 1, "Catch-All Class",
					      NULL, 0, SPC_LGD_CREATED);
	/*
	 *  set the current class to the new catch-all class
	 */
	spc_set_current_class(legend_ptr);

	/*
	 *  dump all unassigned clusters into catch-all class
	 */
        for (i = 0; i < spc_map_rownum; i++)
        {
            if (spc_legend_lookup[i] == NULL)
                spc_add_cluster_to_class(legend_ptr, i);
        }
	
	/*
	 *  update class contents display
	 */
	spc_update_all_classcontents_labels();

	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: spc_transfer_classes
|
|       Purpose: Transfers the chosen class(es) into another class.
|         Input: none
|        Output: none
|       Returns: TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: June 13, 1993
| Modifications: 
|
------------------------------------------------------------*/
int spc_transfer_classes(void)
{
        int             i, j, k, transfer_size, src_num; 
	size_t          src_size, dest_size;
        int             *transfer_clusters, total_clusters;
        char            **classnames, **destinations, **exclude_list = NULL;
	char            *dest_string = NULL;
        LegendEntry     *legend_ptr;
        xvw_list_struct **source_list_return;

        char *prompt1 = "Select Source Class(s)";
        char *label1  = "Current Class Categories";
        char *prompt2 = "Select Destination Class";

        if (spc_legend_list == NULL)
        {
            errno = SPC_USER_ERROR;
            kerror(NULL, "spc_transfer_classes",
		   "No classes to transfer");
	    kinfo(KHOSTILE, "Only intelligent users should really attempt to use this application.");
            return(FALSE);
        }

        classnames = spc_create_strings_class_list(NULL, SPC_NOTEMPTY, 
						   &src_size, NULL, (size_t) 0);
        if (src_size == 1)
        {
            errno = SPC_USER_ERROR;
            kerror(NULL, "spc_transfer_classes",
		   "Cannot perform transfer with only one class");
	    kinfo(KHOSTILE, "Are you familiar with the word, 'transfer'?  Can you pronounce it?.");
            return(FALSE);
        }

        /*
         *  create list of source classes to choose from (as many)
         */
        source_list_return = 
		xvu_run_list_multsel_wait(classnames, (size_t) src_size, 
					  prompt1, label1, FALSE, FALSE, 
					  &src_num);
        if (source_list_return == NULL) return(FALSE);

        /*
         *  exclude source classes from choices of destination classes
         */
        for (i = 0; i < src_num; i++)
        {
            exclude_list = (char **) 
			    krealloc(exclude_list, (i+1)*sizeof(char *));
            exclude_list[i] = kstrdup(source_list_return[i]->string);
        }

        /*
         *  create list of destination classes to choose from
         *  (only one, excluding source classes chosen)
         */
        destinations = spc_create_strings_class_list(NULL, SPC_ALL, &dest_size,
                                                     exclude_list, 
						     (size_t) src_num);
        (void) xvu_choose_wait(prompt2, destinations, (size_t) dest_size, 
			       -1, FALSE, &dest_string);
        if (dest_string == NULL) return(FALSE);

        /*
         *  collect clusters to be transferred from list of source classes
         *  save clusters in an integer array
         */
        k = 0;
        transfer_clusters = (int *) kmalloc(sizeof(int));  
	transfer_size = 1;
        for (i = 0; i < src_num; i++)
        {
            legend_ptr = 
		spc_find_legend_from_classname(source_list_return[i]->string);

            transfer_clusters =  (int *) krealloc(transfer_clusters,
                                 (transfer_size+legend_ptr->clusternum) *
                                 sizeof(int));
            transfer_size += legend_ptr->clusternum;

            total_clusters = legend_ptr->clusternum;
            for (j = 0; j < total_clusters; j++)
            {
               transfer_clusters[k] = legend_ptr->clusters[0];
               spc_delete_cluster_from_class(legend_ptr, transfer_clusters[k++]);
            }
        }

        /*
         *  get clusters from integer array, and add them into chosen class
         */
        legend_ptr = spc_find_legend_from_classname(dest_string);
        for (i = 0; i < k; i++)
            spc_add_cluster_to_class(legend_ptr, transfer_clusters[i]);

	/*
	 *  set the current class to the destination class
	 */
	spc_set_current_class(legend_ptr);

	/*
	 *  update the "Class Contents" display
	 */
	spc_update_all_classcontents_labels();

        karray_free(classnames,        KSTRING, (size_t) src_size,  NULL);
        karray_free(exclude_list,      KSTRING, (size_t) src_num,   NULL);
        karray_free(destinations,      KSTRING, (size_t) dest_size, NULL);
        karray_free(source_list_return,KSTRUCT, (size_t) src_size,  NULL);

	return(TRUE);
}


/*-----------------------------------------------------------
|
|  Routine Name: spc_empty_class
|
|       Purpose: Empties the class represented by the legend_ptr passed in
|
|         Input: legend_ptr - node representing class to be emptied
|        Output: none
|       Returns: TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: June 13, 1993
| Modifications: 
|
------------------------------------------------------------*/
int spc_empty_class(
    LegendEntry *legend_ptr)
{
        int i, total_clusters;

        total_clusters = legend_ptr->clusternum;
        for (i = total_clusters-1; i >= 0; i--)
	{
	    if (!(spc_delete_cluster_from_class(legend_ptr, 
				legend_ptr->clusters[i])))
		return(FALSE);
	}
	legend_ptr->orig_type = SPC_LGD_CREATED;
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: spc_refresh_class_colors
|
|       Purpose: After the colormap of the image has changed because
|                map columns were changed, a function of map columns
|                was applied, or whatever, this routine refreshes the
|                colors of the currently assigned classes.
|
|         Input: none
|        Output: none
|       Returns: TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: Nov 3, 1993
| Modifications:
|
------------------------------------------------------------*/

void spc_refresh_class_colors(void)
{
        LegendEntry *legend_ptr;

        legend_ptr = spc_legend_list;
        while (legend_ptr != NULL)
        {
	    if (!legend_ptr->hidden)
                xvw_set_attribute(legend_ptr->colorbox,
                                  XVW_COLORCELL_UPDATE,  TRUE);
            legend_ptr = legend_ptr->next;
        }
        xvw_set_attributes(spc_scatter->plot,
                           XVW_PLOT2D_PLOTSIZE,      spc_unique_values,
                           XVW_PLOT2D_POINTS,        spc_scatter->plotdata,
                           XVW_GRAPHICS_RESET_WORLD, FALSE,
			   NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: spc_hide_or_show_class
|
|       Purpose: Given a legend pointer, hides or shows the class.
|         Input: legend_ptr - pointer to desired class
|                flag       - SPC_HIDDEN to hide, SPC_DISPLAYED to show.
|        Output: none
|       Returns: TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: Feb 16, 1994
| Modifications:
|
------------------------------------------------------------*/
void spc_hide_or_show_class(
    LegendEntry *legend_ptr,
    int         flag)
{
	/* sanity check */
	if (legend_ptr == NULL)
	    return;

	/*
	 * hide class by restoring original color
	 */
        if (flag == SPC_HIDDEN)
        {
	    legend_ptr->hidden = TRUE;
	    xvw_set_attribute(legend_ptr->colorbox,
                              XVW_COLORCELL_RESTORE, TRUE);
        }

	/*
	 * show class by activating class colors
	 */
        else
        {
	    legend_ptr->hidden = FALSE;
            xvw_set_attributes(legend_ptr->colorbox,
                               XVW_COLORCELL_REDVAL,   legend_ptr->redval,
                               XVW_COLORCELL_GREENVAL, legend_ptr->greenval,
                               XVW_COLORCELL_BLUEVAL,  legend_ptr->blueval,
                               NULL);
        }
}
