/*
  * Khoros: $Id: do_legend.c,v 1.5 1997/01/20 06:48:54 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_legend.c,v 1.5 1997/01/20 06:48:54 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for pane legend
   >>>> 
   >>>>  Private: 
   >>>> 	legend_delete
   >>>> 	legend_empty
   >>>> 	legend_show
   >>>> 	legend_hide
   >>>> 	legend_catch_all
   >>>> 	legend_transfer
   >>>> 	legend_hide_all
   >>>> 	legend_show_all
   >>>> 	legend_empty_all
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */



#include "spectrum.h"

/*-----------------------------------------------------------
| 
|  Routine Name: legend_delete
| 
|       Purpose: Do routine which is called when
|                  pane action button delete is used
| 
|         Input: legend_info - ptr to PaneInfo struct for legend pane
|        Output: None
|    Written By: Danielle Argiro
|          Date: May 11, 1993
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void legend_delete(
     Legend_legend *legend_info)
{
	/* sanity check */
	if (spc_image == NULL)
	{
	    errno = SPC_USER_ERROR;
	    kerror(NULL, "legend_delete",
		   "You must display an image w/ a legend before deleting classes!");
	    kinfo(KHOSTILE, "What a bonehead... ");
	    return;
	}
	spc_delete_or_empty(SPC_DELETE);
}


/*-----------------------------------------------------------
| 
|  Routine Name: legend_empty
| 
|       Purpose: Do routine which is called when
|                  pane action button empty is used
| 
|         Input: legend_info - ptr to PaneInfo struct for legend pane
|        Output: None
|    Written By: Danielle Argiro
|          Date: May 11, 1993
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void legend_empty(
     Legend_legend *legend_info)
{
	/* sanity check */
	if (spc_image == NULL)
	{
	    errno = SPC_USER_ERROR;
	    kerror(NULL, "legend_empty",
		   "You must display an image w/ a legend before emptying classes!");
	    kinfo(KHOSTILE, "Get a life!");
	    return;
	}
	spc_delete_or_empty(SPC_EMPTY);
}


/*-----------------------------------------------------------
| 
|  Routine Name: legend_show
| 
|       Purpose: Do routine which is called when
|                  pane action button show is used
| 
|         Input: legend_info - ptr to PaneInfo struct for legend pane
|        Output: None
|    Written By: Danielle Argiro
|          Date: May 11, 1993
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void legend_show(
     Legend_legend *legend_info)
{
	/* sanity check */
	if (spc_image == NULL)
	{
	    errno = SPC_USER_ERROR;
	    kerror(NULL, "legend_show",
		   "You must display an image w/ a legend before displaying classes!");
	    kinfo(KHOSTILE, "Does thickheadedness run in your family, or is it just you?");
	    return;
	}
	spc_hide_or_show(SPC_DISPLAYED);
}


/*-----------------------------------------------------------
| 
|  Routine Name: legend_hide
| 
|       Purpose: Do routine which is called when
|                  pane action button hide is used
| 
|         Input: legend_info - ptr to PaneInfo struct for legend pane
|        Output: None
|    Written By: Danielle Argiro
|          Date: May 11, 1993
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void legend_hide(
     Legend_legend *legend_info)
{
        /* sanity check */
        if (spc_image == NULL)
        {
	    errno = SPC_USER_ERROR;
            kerror(NULL, "legend_hide",
                   "You must display an image w/ a legend before hiding classes!");
	    kinfo(KHOSTILE, "I hope you're not actually getting paid to do this.");
            return;
        }

	spc_hide_or_show(SPC_HIDDEN);
}


/*-----------------------------------------------------------
| 
|  Routine Name: legend_catch_all
| 
|       Purpose: Do routine which is called when
|                  pane action button catch_all is used
| 
|         Input: legend_info - ptr to PaneInfo struct for legend pane
|        Output: None
|    Written By: Danielle Argiro
|          Date: May 11, 1993
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void legend_catch_all(
     Legend_legend *legend_info)
{
	/* sanity check */
        if (spc_image == NULL)
        {
	    errno = SPC_USER_ERROR;
            kerror(NULL, "legend_hide",
                   "You must display an image w/ a legend before creating a catch-all class!");
	    kinfo(KHOSTILE, "Ever heard the term, 'luser'?  Think about it...");
            return;
        }
	spc_catch_all();
}


/*-----------------------------------------------------------
| 
|  Routine Name: legend_transfer
| 
|       Purpose: Do routine which is called when
|                  pane action button transfer is used
| 
|         Input: legend_info - ptr to PaneInfo struct for legend pane
|        Output: None
|    Written By: Danielle Argiro
|          Date: May 11, 1993
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void legend_transfer(
     Legend_legend *legend_info)
{
	/* sanity check */
        if (spc_image == NULL)
        {
	    errno = SPC_USER_ERROR;
            kerror(NULL, "legend_hide",
                   "You must display an image w/ a legend before transferring classes!");
	    kinfo(KHOSTILE, "For goodness sake!  Use your brain!");
            return;
        }
	spc_transfer_classes();
}

/*-----------------------------------------------------------
| 
|  Routine Name: legend_hide_all
| 
|       Purpose: Do routine which is called when
|                  pane action button hide_all is used
| 
|         Input: legend_info - ptr to PaneInfo struct for legend pane
| 
|        Output: None
|    Written By: 
|          Date: Oct 18, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void legend_hide_all(
     Legend_legend *legend_info)
{
        LegendEntry *legend_ptr;

        /* user error: bogus attempt to hide */
        if (spc_legend_list == NULL)
        {
            errno = SPC_USER_ERROR;
            kerror(NULL, "legend_hide_all", "No classes to hide.");
            kinfo(KHOSTILE, "If I were you, I would certainly hide!");
            return;
        }

	/* hide all classes */
        legend_ptr = spc_legend_list;
        while (legend_ptr != NULL)
        {
            spc_hide_or_show_class(legend_ptr, SPC_HIDDEN);
            legend_ptr = legend_ptr->next;
        }
}


/*-----------------------------------------------------------
| 
|  Routine Name: legend_show_all
| 
|       Purpose: Do routine which is called when
|                  pane action button show_all is used
| 
|         Input: legend_info - ptr to PaneInfo struct for legend pane
| 
|        Output: None
|    Written By: 
|          Date: Oct 18, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void legend_show_all(
     Legend_legend *legend_info)
{
	LegendEntry *legend_ptr;

        /* user error: bogus attempt to show */
        if (spc_legend_list == NULL)
        {
            errno = SPC_USER_ERROR;
            kerror(NULL, "legend_show_all", "No classes to show.");
            kinfo(KHOSTILE, "If I were you, I wouldn't bother to show!");
            return;
        }

	/* show all classes */
        legend_ptr = spc_legend_list;
        while (legend_ptr != NULL)
        {
            spc_hide_or_show_class(legend_ptr, SPC_DISPLAYED);
            legend_ptr = legend_ptr->next;
        }
}


/*-----------------------------------------------------------
| 
|  Routine Name: legend_empty_all
| 
|       Purpose: Do routine which is called when
|                  pane action button empty_all is used
| 
|         Input: legend_info - ptr to PaneInfo struct for legend pane
| 
|        Output: None
|    Written By: 
|          Date: Oct 18, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void legend_empty_all(
     Legend_legend *legend_info)
{
	LegendEntry *legend_ptr;

        /* user error: bogus attempt to empty */
        if (spc_legend_list == NULL)
        {
            errno = SPC_USER_ERROR;
            kerror(NULL, "legend_show_all", "No classes to empty.");
            kinfo(KHOSTILE, "It's your head that's empty");
            return;
        }

	if (!(kprompt(KSTANDARD, "Yes", "Cancel", 0, 
		      "Are you SURE you want to empty all classes???")))
           return;


	legend_ptr = spc_legend_list;
	while (legend_ptr != NULL)
	{
	    spc_empty_class(legend_ptr);
	    legend_ptr = legend_ptr->next;
	}
}


