/*
  * Khoros: $Id: do_spectrum.c,v 1.15.2.2 1997/07/01 00:39:20 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_spectrum.c,v 1.15.2.2 1997/07/01 00:39:20 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for master spectrum
   >>>> 
   >>>>  Private: 
   >>>>          spectrum_add_class
   >>>>          spectrum_delete_class
   >>>>          spectrum_select
   >>>>          spectrum_add_cluster_manually
   >>>>          spectrum_delete_cluster_manually
   >>>>          spectrum_reassign
   >>>>          spectrum_trans
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "spectrum.h"

/*-----------------------------------------------------------
| 
|  Routine Name: spectrum_add_class
| 
|       Purpose: Do routine which is called when
|                  pane action button add_class is used
| 
|         Input: master_info - ptr to FormInfo struct for spectrum
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Nov 24, 1993
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void spectrum_add_class(
     gui_info_struct *master_info)
{
	int          i;
	LegendEntry *legend_ptr;

        /*
         * make sure they've named the last class
         * before letting them start a new one
         */
        legend_ptr = spc_legend_list;
        while (legend_ptr != NULL)
            legend_ptr = legend_ptr->next;

	/* 
	 * add the legend entry that represents the class
	 */
        legend_ptr = spc_add_legend_entry(-1, 0, 0, 0, "New Class",
                                          NULL, 0, SPC_USER_CREATED);

	spc_set_current_class(legend_ptr);

kfprintf(kstderr, "Classes\n\n");
legend_ptr = spc_legend_list;
while (legend_ptr != NULL)
{
   kfprintf(kstderr, "Class %d -- %s\n", legend_ptr->class1, legend_ptr->classname);
   kfprintf(kstderr, "            clusters:\n");
   for (i = 0; i < legend_ptr->clusternum; i++)
       kfprintf(kstderr,"                       %d\n", legend_ptr->clusters[i]);
   legend_ptr = legend_ptr->next;
}
kfprintf(kstderr, "\n\n");
}


/*-----------------------------------------------------------
| 
|  Routine Name: spectrum_delete_class
| 
|       Purpose: Do routine which is called when
|                  pane action button delete_class is used
| 
|         Input: master_info - ptr to FormInfo struct for spectrum
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Nov 24, 1993
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void spectrum_delete_class(
     gui_info_struct *master_info)
{
	if (spc_image == NULL) 
	{
	    errno = SPC_USER_ERROR;
	    kerror(NULL, "spectrum_delete_class",
		   "You must display an image w/ classes before deleting classes!");
	    kinfo(KHOSTILE, "Don't you have any common sense AT ALL?");
	    return;
	}

	spc_delete_or_empty(SPC_DELETE);

}


/*-----------------------------------------------------------
| 
|  Routine Name: spectrum_select
| 
|       Purpose: Do routine which is called when
|                  pane action button select is used
| 
|         Input: master_info - ptr to FormInfo struct for spectrum
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Nov 24, 1993
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void spectrum_select(
     gui_info_struct *master_info)
{
        size_t          listsize; 
        char            **classnames;
        char            prompt[KLENGTH];
        LegendEntry     *legend_ptr;
        char            *class_return = NULL;

        if (spc_legend_list == NULL)
        {
	    errno = SPC_USER_ERROR;
            kerror(NULL, "spectrum_select", "No classes created, from which to select.");
	    kinfo(KHOSTILE, "Obviously!  (What a pinhead).");
            return;
        }

        /*
         *  set up prompts w/ old map column names entered
         */
        if (gui_info->cluster_operation_val == SPC_ADD)
            ksprintf(prompt, "Select Class To Which to Add Clusters:");
        else ksprintf(prompt, "Select Class From Which to Delete Clusters:");

        /*
         * prompt to see which class they want
         */
        classnames = spc_create_strings_class_list(NULL, SPC_ALL, &listsize,
						   NULL, (size_t) 0);
        (void) xvu_choose_wait(prompt, classnames, (int) listsize, -1, FALSE,
			       &class_return);

        /*
         *  find current class from user's choice of list items and
         *  set the current class to the class selected
         */
	if (class_return != NULL)
	{
          legend_ptr = spc_find_legend_from_classname(class_return);
          spc_set_current_class(legend_ptr);
	  kfree(class_return);
	}
}

/*-----------------------------------------------------------
| 
|  Routine Name: spectrum_add_cluster_manually
| 
|       Purpose: Do routine which is called when
|                  pane action button add_cluster_manually is used
| 
|         Input: master_info - ptr to FormInfo struct for spectrum
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Jul 15, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
#define QUERY_NUM 10

void spectrum_add_cluster_manually(
     gui_info_struct *master_info)
{
	int  i, j, cluster, garbage;
	char temp[KLENGTH];
	char **answers;

	if ((spc_current_class == NULL) || (spc_legend_lookup == NULL))
	{
	    errno = SPC_USER_ERROR;
	    kerror(NULL, "spectrum_add_cluster_manually",
		   "There is no current class to which to add clusters!");
	    kinfo(KHOSTILE, "Use your head, will you?");
	    return;
	}

	answers  = (char **) kcalloc((size_t) QUERY_NUM, sizeof(char *));
	for (i = 0; i < QUERY_NUM; i++)
	    answers[i] = (char *) kcalloc((size_t) 1, KLENGTH * sizeof(char));

	/*
         *  prompt for new names
         */
	ksprintf(temp, "Enter Clusters To be Added to Class %s", 
		 spc_current_class->classname);
        if (!(xvu_query_wait(temp, NULL, "ok", answers, QUERY_NUM, 15)))
	{
	    karray_free(answers, KSTRING, (size_t) QUERY_NUM, NULL);
	    return;
	}

	for (i = 0; i < QUERY_NUM; i++)
        {
	    kstring_cleanup(answers[i], temp);
	    if (kstrlen(temp) == 0) continue;
	    else 
	    {
		garbage = FALSE;
		for (j = 0; j < kstrlen(temp); j++)
		    if (!isdigit(temp[j])) garbage = TRUE;
	    }
	    if (!garbage)
	    {
	        cluster = atoi(temp);
	        if (cluster < 0 || cluster > spc_map_rownum)
		{
		    kerror(NULL, "spectrum_add_cluster_manually",
                           "%d is not a valid cluster number; "
			   "cluster numbers range from 0 to %d.",
			   cluster, spc_map_rownum);
	        }
		else
	        {
	            if (spc_legend_lookup[cluster] != NULL)
		        kinfo(KSTANDARD, "Cluster number %d has already been "
			                 "assigned to class %s", cluster, 
			                 spc_legend_lookup[cluster]->classname);
 	            else if (spc_add_cluster_to_class(spc_current_class, 
					              cluster))
                    {
                         kfprintf(kstderr,
                             "Adding cluster %d to current class '%s' \n",
                             cluster, spc_legend_lookup[cluster]->classname);
                    }

                    /* if this is the first cluster we add to a class, need to
                       set the pseudocolor list */
                    if ((spc_current_class->clusters != NULL) &&
                        (spc_current_class->clusters[0] == cluster))
                        spc_set_pseudo_color(spc_current_class);
	        }
	    }
	    else
	    {
	        kerror(NULL, "spectrum_add_cluster_manually",
	               "Ignoring bogus entry '%s'\n", temp);
	    }
	}
}


/*-----------------------------------------------------------
| 
|  Routine Name: spectrum_delete_cluster_manually
| 
|       Purpose: Do routine which is called when
|                  pane action button delete_cluster_manually is used
| 
|         Input: master_info - ptr to FormInfo struct for spectrum
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Jul 15, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void spectrum_delete_cluster_manually(
     gui_info_struct *master_info)
{
	int    i, cluster, size, delete_cluster_number; 
	char   temp[KLENGTH];
	char  *prompt = "Choose cluster(s) to be deleted from class";
        char  *label  = "Current clusters";
        char **cluster_list;
	xvw_list_struct **delete_list_return = NULL;

	/*
	 *  No classes from which to delete clusters
	 */
        if ((spc_current_class == NULL) || (spc_legend_lookup == NULL))
        {
	    errno = SPC_USER_ERROR;
            kerror(NULL, "spectrum_delete_cluster_manually",
                   "There is no current class to which to delete clusters!");
            kinfo(KHOSTILE, "Use your head, will you?");
            return;
        }

        /*
         * no clusters to delete from the class
         */
        if (spc_current_class->clusternum == 0)
        {
            errno = SPC_USER_ERROR;
            kerror(NULL, "spectrum_delete_cluster_manually", 
		   "No clusters in current class to delete!");
            return;
        }

	/*
	 * only one cluster in the class to possibly be deleted
	 */
	else if (spc_current_class->clusternum == 1)
        {
            ksprintf(temp, "Delete cluster %d from class %s?",
                     spc_current_class->clusters[0], 
		     spc_current_class->classname);
            if (!(kprompt(KSTANDARD, "Ok", "Cancel", 0, temp)))
               return;
            spc_delete_cluster_from_class(spc_current_class, 
					  spc_current_class->clusters[0]);
	    return;
        }

	/*
	 *  more than one cluster that could be deleted from the class
	 */
	cluster_list = 
		spc_create_strings_cluster_list(spc_current_class->clusters, 
				        (size_t) spc_current_class->clusternum);

	/*
	 *  put up list of clusters to choose from
	 */
	ksprintf(temp, "%s %s", prompt, spc_current_class->classname);
	delete_list_return =
		xvu_run_list_multsel_wait(cluster_list,
                                     (size_t) spc_current_class->clusternum+1,
                                     temp, label, FALSE, FALSE,
                                     &delete_cluster_number);
	if (delete_list_return == NULL) return;

	/*
         * user wants to delete all clusters at once
         */
        if (kstrcmp(delete_list_return[0]->string,"ALL Clusters")== 0)
        {
	    size = spc_current_class->clusternum;
            for (i = size-1; i >= 0; i--)
            {
                if (spc_delete_cluster_from_class(spc_current_class, 
					spc_current_class->clusters[i]))
                {
                    kfprintf(kstderr,
                             "Deleting ALL clusters from current class '%s' \n",
                             spc_current_class->classname);
                }
            }
	    return;
        }

	/*
	 *  delete each of the clusters specified
	 */
        for (i = 0; i < delete_cluster_number; i++)
        {
            ksscanf(delete_list_return[i]->string, "Cluster Number %d", 
		    &cluster);

            if (spc_delete_cluster_from_class(spc_current_class, cluster))
            {
                kfprintf(kstderr,
                         "Deleting cluster %d from current class '%s' \n",
                         cluster, spc_current_class->classname);
            }

            /* if this is the first cluster we add to a class, need to
                   set the pseudocolor list */
            if ((spc_current_class->clusters != NULL) &&
                (spc_current_class->clusters[0] == cluster))
                spc_set_pseudo_color(spc_current_class);
        }
	spc_update_classcontents_label(spc_current_class);
}


/*-----------------------------------------------------------
| 
|  Routine Name: spectrum_reassign
| 
|       Purpose: Do routine which is called when
|                  pane action button reassign is used
| 
|         Input: master_info - ptr to FormInfo struct for spectrum
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Feb 14, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void spectrum_reassign(
     gui_info_struct *master_info)
{
	int     *unique_values; 
	int     i, j, w, h, d, t, e, found; 
	size_t  offset; 
	size_t  dest_size; 
	int     dest_cluster, all_clusters = FALSE;
	size_t  unique_cluster_number; 
	int     reassign_cluster_number;
	int     roi_x, roi_y, xoffset, yoffset;
	int     *image_values = NULL, *mask_values = NULL;
	kobject tmp_roi, dest_object;
	char    **destinations = NULL, **cluster_list;
	char    *prompt = "Choose cluster(s) to be reassigned to new class";
	char    *label  = "Unique cluster numbers appearing in polygon";
	char    *dest_prompt = "Choose new class for clusters";
	LegendEntry *legend_ptr;
	int     *reassign_clusters = NULL, reassign_from_zoom = FALSE;
	char    *dest_string_return = NULL;

	xvw_list_struct **clust_list_return = NULL;

	if (gui_info->Zoom->zoom->zoommode == KZOOM_UM_BUTTON_PRESS)
	   reassign_from_zoom = TRUE;

	/*
	 * create list of classes having at least one cluster assigned, 
	 * that can be potential destinations for re-assigned clusters.
	 * if none exist, might as well return immediately.
	 */
	destinations = spc_create_strings_class_list(NULL, SPC_NOTEMPTY,
				                     &dest_size, NULL, 
						     (size_t) 0);
	if (dest_size == 0)
	{
	    errno = SPC_USER_ERROR;
	    kerror(NULL, "spectrum_reassign", "There are currently no classes with at least one cluster assigned, and therefore no classes to which to reassign clusters.");
	    return;
	}

	/*
	 * get ROI from zoom window
	 */
	if (reassign_from_zoom)
	{
	    xvf_set_attribute(gui_info->Zoom->zoom->quit_struct, 
			      XVF_ACTIVATE, FALSE);
	    xvw_set_attribute(zoom_workspace, XVW_IMAGE_ROI_SHAPE,
			      KIMAGE_ROI_POLYLINE);
	    xvw_get_attribute(zoom_workspace, XVW_IMAGE_ROI, &tmp_roi);
	    xvw_get_attribute(zoom_workspace, XVW_IMAGE_XPOSITION, &xoffset);
	    xvw_get_attribute(zoom_workspace, XVW_IMAGE_YPOSITION, &yoffset);

	}
	/*
	 * get ROI from main spectrum workspace
	 */
	else
	{
            xvw_set_attribute(spc_display->workspace, XVW_IMAGE_ROI_SHAPE,
                              KIMAGE_ROI_POLYLINE);
            xvw_get_attribute(spc_display->workspace, XVW_IMAGE_ROI, &tmp_roi);
	    xoffset = 0; yoffset = 0;
	}
	if (tmp_roi == NULL) 
	{
	     kerror(NULL, "spectrum_reassign", 
		    "Incomplete polygon; aborting cluster reassignment");
	     if (reassign_from_zoom)
                  xvf_set_attribute(gui_info->Zoom->zoom->quit_struct,
                                  XVF_ACTIVATE, TRUE);
	     return;
	}

	/*
	 * from the ROI, get an integer array of unique clusters
	 */
	unique_values = spc_find_unique_values_from_polygon(tmp_roi, 
							&unique_cluster_number);
	if (unique_values == NULL) return;

	/*
	 * from the integer array of unique clusters, get an array of 
	 * strings containing "Cluster Number %d" for prompting the user.
	 */
	cluster_list = spc_create_strings_cluster_list(unique_values, 
						       unique_cluster_number);
	kfree(unique_values);

	/*
	 * less than one unique cluster: some sort of bizarre error
	 */
	if (unique_cluster_number < 1)
	{
	    errno = SPC_USER_ERROR;
	    kerror(NULL, "spectrum_reassign", 
		   "Incomplete polygon with no clusters inside");
	    if (reassign_from_zoom)
                xvf_set_attribute(gui_info->Zoom->zoom->quit_struct,
                                  XVF_ACTIVATE, TRUE);
	    return;
	}
	/*
	 *  just one unique cluster: don't bother prompting
	 */
	else if (unique_cluster_number == 1)
	    reassign_cluster_number = 1;
	/*
	 *  more than one unique cluster: prompt for which ones to reassign
	 */
	else 
	{
	    offset = 0;
	    if (unique_cluster_number > 1) offset = 1;
	    clust_list_return = xvu_run_list_multsel_wait(cluster_list, 
						 unique_cluster_number +offset, 
						 prompt, label, FALSE, FALSE, 
					         &reassign_cluster_number);
	    /* no clusters chosen for re-assignment; return */
            if (clust_list_return == NULL) 
	    {
	        if (reassign_from_zoom)
                    xvf_set_attribute(gui_info->Zoom->zoom->quit_struct,
                                      XVF_ACTIVATE, TRUE);
	        return;
	    }
	}


	/* 
	 * based on user's choice from displayed list of cluster numbers,
	 * construct array of clusters for reference later.
	 */
	if ((reassign_cluster_number == 1) && (unique_cluster_number == 1))
	{
	    /* there was only one cluster in question to begin with */
	    reassign_clusters = (int *) kmalloc(sizeof(int));
	    ksscanf(cluster_list[0], "Cluster Number %d",
                    &reassign_clusters[0]);
	}
	else
	{
	    for (i = 0; i < reassign_cluster_number; i++)
	    {
		if (kstrcmp(clust_list_return[i]->string, "ALL Clusters") == 0)
		    all_clusters = TRUE;
	    }
	    if (all_clusters)
	    {
		reassign_clusters = (int *) 
				kmalloc((size_t) unique_cluster_number *
						    sizeof(int));
		/* all available clusters to be reassigned */
		j = 0;
                for (i = 1; i < unique_cluster_number+1; i++)
                    ksscanf(cluster_list[i], "Cluster Number %d",
                            &reassign_clusters[i-1]);
		reassign_cluster_number = unique_cluster_number;
	    }
	    else if (reassign_cluster_number == 1)
	    {
		reassign_clusters = (int *) kmalloc(sizeof(int));
	        ksscanf(clust_list_return[0]->string, "Cluster Number %d",
                        &reassign_clusters[0]);
	    }
	    else 
	    { 
	        /* more than one cluster possible, more than one chosen */
		reassign_clusters = (int *) kmalloc((size_t) reassign_cluster_number *
                                                    sizeof(int));
	        for (i = 0; i < reassign_cluster_number; i++)
	            ksscanf(clust_list_return[i]->string, "Cluster Number %d",
                            &reassign_clusters[i]);
	    }
	}
	karray_free(cluster_list, KSTRING, unique_cluster_number, NULL);

	/*
	 * prompt user for choice of possible destination classes.
	 */
	(void) xvu_choose_wait(dest_prompt, destinations, (int) dest_size, 
                               -1, FALSE, &dest_string_return);
        if (dest_string_return == NULL) 
	{
	    if (reassign_from_zoom)
                xvf_set_attribute(gui_info->Zoom->zoom->quit_struct,
                                  XVF_ACTIVATE, TRUE);
	    return;
	}

	/*
	 * get the chosen destination class from it's name;  reassigned 
	 * clusters will take on the cluster value of whichever cluster
	 * happens to appear first in the new class.
	 */
	xvw_busy(NULL, TRUE);
	legend_ptr = spc_find_legend_from_classname(dest_string_return);
	dest_cluster = legend_ptr->clusters[0];

	/*
	 * get all the value & mask data in a single region from the ROI.
	 */
        kpds_set_attribute(tmp_roi, KPDS_VALUE_DATA_TYPE,   KINT);
        kpds_get_attribute(tmp_roi, KPDS_VALUE_SIZE,       &w, &h, &d, &t, &e);
	kpds_set_attribute(tmp_roi, KPDS_VALUE_REGION_SIZE, w, h, d, t, e);
	kpds_set_attribute(tmp_roi, KPDS_VALUE_POSITION,    0, 0, 0, 0, 0);

	image_values = (int *) kpds_get_data(tmp_roi, KPDS_VALUE_REGION, NULL);
	if (image_values == NULL)
	{
	    kerror(NULL, "spectrum_reassign", "Unable to get data from specified region of interest; possible lack of swap space; aborting reassign clusters operation...");
	    return;
	}

	kpds_set_attribute(tmp_roi, KPDS_MASK_REGION_SIZE,  w, h, d, t, e);
        kpds_set_attribute(tmp_roi, KPDS_MASK_DATA_TYPE,  KINT);
	kpds_set_attribute(tmp_roi, KPDS_MASK_POSITION,  0, 0, 0, 0, 0);

	mask_values  = (int *) kpds_get_data(tmp_roi, KPDS_MASK_REGION,  NULL);
	if (mask_values == NULL)
	{
	    kerror(NULL, "spectrum_reassign", "Unable to get mask values from specified region of interest; possible lack of swap space; aborting reassign clusters operation...");
	    return;
	}

/*
 * Uncomment below to verify ROI mask & image values
 */
#if 0
kfprintf(kstderr, "Before:\n");
for (i = 0; i < w*h; i++)
{
   if ((i % w == 0) && (i > 0))
       kfprintf(kstderr, "\n");
   kfprintf(kstderr, "%3d ", mask_values[i]);
}
kfprintf(kstderr, "\n\n");

for (i = 0; i < w*h; i++)
{
   if ((i % w == 0) && (i > 0))
       kfprintf(kstderr, "\n");
   kfprintf(kstderr, "%3d ", image_values[i]);
}
kfprintf(kstderr, "\n\n");
#endif



	/*
	 * go through the entire ROI, replacing image data values with
	 * the new cluster value whenever appropriate.
	 */
	for (i = 0; i < w*h*d*t*e; i++)
	{
	    /* do not replace value if it was masked out (not in the polygon) */
	    if (mask_values[i] != 0)
	    {
		/* do not replace value if it was not selected from the list */
		found = karray_locate(reassign_clusters, KINT,
				      image_values[i],
				      (size_t) reassign_cluster_number);

		/* replace unmasked, selected values with new cluster value */
		if (found != -1) 
		     image_values[i] = dest_cluster;
	    }
	}
	kfree_and_NULL(reassign_clusters);

/*
 * Uncomment below to verify ROI mask & image values
 */
#if 0
kfprintf(kstderr, "After:\n");
for (i = 0; i < w*h; i++)
{
   if ((i % w == 0) && (i > 0))
       kfprintf(kstderr, "\n");
   kfprintf(kstderr, "%3d ", mask_values[i]);
}
kfprintf(kstderr, "\n\n");

for (i = 0; i < w*h; i++)
{
   if ((i % w == 0) && (i > 0))
       kfprintf(kstderr, "\n");
   kfprintf(kstderr, "%3d ", image_values[i]);
}
kfprintf(kstderr, "\n\n");
#endif

	/*
	 *  put the region with the modified image data values 
	 *  back into the original image in its original location.
	 */
        kpds_get_attribute(tmp_roi, KPDS_SUBOBJECT_POSITION,
                           &roi_x, &roi_y, NULL, NULL, NULL);
	/*
	 *  put modified data back into image 
	 */
	if (reassign_from_zoom)
	   xvw_get_attribute(zoom_workspace, XVW_IMAGE_IMAGEOBJ, &dest_object);
	else xvw_get_attribute(spc_display->workspace, XVW_IMAGE_IMAGEOBJ, 
			       &dest_object);
	
        kpds_set_attribute(dest_object, KPDS_VALUE_DATA_TYPE, KINT);
	kpds_set_attribute(dest_object, KPDS_VALUE_POSITION, 
			   roi_x, roi_y, 0, 0, 0);
	kpds_set_attribute(dest_object, KPDS_VALUE_REGION_SIZE, w, h, d, t, e);
	if (!kpds_put_data(dest_object, KPDS_VALUE_REGION, image_values))
	{
	    kerror(NULL, "spectrum_reassign", "Unable to change cluster numbers in specified region of interest; possible lack of swap space; aborting reassign clusters operation...");
	    return;
	}

	xvw_busy(NULL, FALSE);

	if (reassign_from_zoom)
	    xvf_set_attribute(gui_info->Zoom->zoom->quit_struct, 
			      XVF_ACTIVATE, TRUE);

	/* cleanup */
	kfree_and_NULL(image_values); 
	kfree_and_NULL(mask_values);
}

/*-----------------------------------------------------------
| 
|  Routine Name: spectrum_trans
| 
|       Purpose: Do routine which is called when
|                  pane action button trans is used
| 
|         Input: master_info - ptr to FormInfo struct for spectrum
| 
|        Output: None
|    Written By: 
|          Date: Sep 28, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void spectrum_trans(
     gui_info_struct *master_info)
{
	int         cluster, i, garbage; 
	size_t      size;
	char        **prompts, **answers, **classnames;
	LegendEntry *legend_ptr;
	char        *old_classname = "none";
	char        *new_classname = NULL;
	char        temp[KLENGTH];

	if (spc_legend_list == NULL)
	{
	    errno = SPC_USER_ERROR;
            kerror(NULL, "spectrum_trans", "No classes to which clusters can be transferred!");
            kinfo(KHOSTILE, "idiot");
	    return;
	}

	prompts = (char **) kcalloc((size_t) 1, sizeof(char *));
        answers = (char **) kcalloc((size_t) 1, sizeof(char *));

	prompts[0] = kstrdup("Cluster Number:");
	answers[0] = (char *) kcalloc((size_t) 1, KLENGTH * sizeof(char));

	/*
         *  prompt for cluster to transfer
         */
	if (!(xvu_query_wait("Specify Cluster to Transfer",
                               prompts, "ok", answers, 1, 25)))
	{
	    kfree_and_NULL(prompts[0]); kfree_and_NULL(answers[0]);
            kfree_and_NULL(answers);    kfree_and_NULL(prompts);
	    return;
	}

	/*
	 * errorcheck user's entry of cluster number
	 */
        kstring_cleanup(answers[0], temp);
        garbage = FALSE;
        for (i = 0; i < kstrlen(temp); i++)
            if (!isdigit(temp[i])) garbage = TRUE;
        if (garbage)
	{
	    kerror(NULL, "spectrum_trans", 
		   "Bogus entry for cluster number, '%s'", temp);
	    kfree_and_NULL(prompts[0]);
            kfree_and_NULL(answers[0]);
            kfree_and_NULL(answers); kfree_and_NULL(prompts);
	    return;
	}

	cluster = atoi(answers[0]);
	kfree_and_NULL(prompts[0]);
	kfree_and_NULL(answers[0]);
	kfree_and_NULL(answers); kfree_and_NULL(prompts);


	/*
	 *  bogus cluster number
	 */
	if ((cluster < 0) || (cluster > spc_map_rownum))
	{
	    errno = SPC_USER_ERROR;
	    kerror(NULL, "spectrum_trans", "Invalid cluster number specified; cluster number must be between 0 and %d\n", spc_map_rownum);
	    return;
	}

	/*
	 *  cluster not currently assigned
	 */
	if (spc_classes[cluster] == -1)
	{
	    errno = SPC_USER_ERROR;
	    kwarn(NULL, "spectrum_trans", "Cluster number %d does not currently belong to a class... but we'll 'transfer' it to a class anyway.", cluster);
	}

	/*
	 * prompt for destination class
	 */
	classnames = spc_create_strings_class_list(NULL, SPC_ALL, &size, 
						   NULL, (size_t) 0);
	(void) xvu_choose_wait("Select Destination Class", classnames, 
				(int) size, 1, FALSE, &new_classname);
        if (new_classname == NULL) return;

	/*
	 * delete the cluster from the class that it currently belongs to
	 */
	if (spc_legend_lookup[cluster] != NULL)
	{
	    old_classname = spc_legend_lookup[cluster]->classname;
	    spc_delete_cluster_from_class(spc_legend_lookup[cluster], cluster);
	}

	/*
	 * get legend pointer of new class name provided, 
	 * and add cluster to the new class
	 */
	legend_ptr = spc_find_legend_from_classname(new_classname);
	if (legend_ptr == NULL)
	{
	    errno = SPC_INTERNAL;
	    kerror(NULL, "spectrum_trans", "Legend ptr NULL");
	    return;
	}
	spc_add_cluster_to_class(legend_ptr, cluster);
        kfprintf(kstderr,
                 "Transferring cluster %d from class '%s' to class '%s' \n",
                 cluster, old_classname, spc_legend_lookup[cluster]->classname);
	spc_update_all_classcontents_labels();
}

/*-----------------------------------------------------------
| 
|  Routine Name: spectrum_autoclass
| 
|       Purpose: Do routine which is called when
|                  pane action button autoclass is used
| 
|         Input: master_info - ptr to FormInfo struct for spectrum
| 
|        Output: None
|    Written By: 
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void spectrum_autoclass(
     gui_info_struct *master_info)
{
	kinfo(KSTANDARD, "Sorry, this feature has been disabled");
}


/*-----------------------------------------------------------
| 
|  Routine Name: spectrum_cluster_operation
| 
|       Purpose: Do routine which is called when
|                  toggle selection cluster_operation is used
| 
|         Input: master_info - ptr to FormInfo struct for spectrum
| 
|        Output: None
|    Written By: 
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void spectrum_cluster_operation(
     gui_info_struct *master_info)
{
	/* PUT YOUR CODE HERE ! */
}


