 /*
  * Khoros: $Id: xprism.c,v 1.14.2.2 1997/05/31 18:55:15 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: xprism.c,v 1.14.2.2 1997/05/31 18:55:15 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for xprism
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "xprism.h"

clui_info_struct *clui_info = NULL;

gui_info_struct *gui_info;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Interactive 2D & 3D Plotting
|
|       Purpose: main program for xprism
|
|         Input:
|		char *clui_info->i1_file; {Input plot 1}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {Input plot 2}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->i3_file; {Input plot 3}
|		int   clui_info->i3_flag; {TRUE if -i3 specified}
|
|		char *clui_info->i4_file; {Input plot 4}
|		int   clui_info->i4_flag; {TRUE if -i4 specified}
|
|		float clui_info->update_float; {How often to check input file for modification}
|		int   clui_info->update_flag; {TRUE if -update specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

	kform  *form;            /* form tree representing UIS        */
	char   *form_fullpath;   /* for use when [-form] is specified */
/* -main_variable_list */
/* -main_variable_list_end */

	khoros_init(argc, argv, "ENVISION", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$ENVISION/objects/xvroutine/xprism");
	kexit_handler(xprism_free_args, NULL);

/* -main_get_args_call */
	kclui_init("ENVISION", "xprism", KGEN_XVROUTINE,  &clui_uis_spec,
		   xprism_usage_additions, xprism_get_args,
 		   xprism_free_args);
/* -main_get_args_call_end */

	/*
	 * Initialize xvwidgets lib
	 */
	if (!(xvw_initialize(XVW_MENUS_XVFORMS)))
	{
	     kinfo(KSTANDARD, "Could not open connection to server");
	     kexit(KEXIT_FAILURE);
	}

	/*
	 * Allocate the GUI Info structure 
	 */
	gui_info = (gui_info_struct *)
				kcalloc ((size_t) 1, sizeof(gui_info_struct));
	/*
	 * Create the forms for xprism 
	 */
	if (clui_info->form_flag)
	{
	    form_fullpath   = kfullpath(clui_info->form_file, NULL, NULL);
	    form = xvf_create_form(form_fullpath, KNONE, run_xprism, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	    kfree_and_NULL(form_fullpath);
	}
	else
	{
	    form = xvf_build_form(&gui_uis_spec, KNONE, run_xprism, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	}
	if (form == NULL)
	   kexit(KEXIT_FAILURE);

	/*
	 * Initialize the GUI Info structure 
	 */
	if (!(_xvf_init_xprism(form, gui_info)))
	    kexit(KEXIT_FAILURE);

/* -main_before_lib_call */
	/*
	 * set up function pane GUI according to 2D/3D
	 */
	set_function_pane_gui();

	/*
	 * need to set area_list, plot_list to NULL on quit
	 */
	area_list = gui_info->Options->options->area_list_struct;
	plot_list = gui_info->Options->options->plot_list_struct;
	xvf_add_extra_call(gui_info->quit_struct, quit_cleanup,
			   NULL, XVF_CALL_FIRST);

	/*
	 *  set up tacking
	 */
        xvw_set_attribute(gui_info->workspace, XVW_TACK_EDGE,
                          KMANAGER_TACK_BOTTOM | KMANAGER_TACK_RIGHT);
	xvw_set_attributes(xvw_parent(gui_info->workspace),
                           XVW_MINIMUM_WIDTH,  525,
                           XVW_MINIMUM_HEIGHT, 260,
                           NULL);
	/*
	 *  create layout object to be parent of area objects;
	 *  install callback to change the area.
	 */
	layout = xvw_create_layout(gui_info->workspace, "layout");
	xvw_set_attributes(layout,
		           XVW_TACK_EDGE,    KMANAGER_TACK_ALL,
		           XVW_BORDER_WIDTH, 0,
		           NULL);
	xvw_insert_callback(layout, XVW_LAYOUT_CALLBACK,
			    FALSE, change_area, NULL);

	
	/*
	 * input files from CLUI if necessary
	 */
	if (clui_info->i1_file) input_file(clui_info->i1_file);
	if (clui_info->i2_file) input_file(clui_info->i2_file);
	if (clui_info->i3_file) input_file(clui_info->i3_file);
	if (clui_info->i4_file) input_file(clui_info->i4_file);
/* -main_before_lib_call_end */

/* -main_library_call */
	xvf_run_form();
/* -main_library_call_end */

/* -main_after_lib_call */
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: xprism_usage_additions
| 
|       Purpose: Prints usage additions in xprism_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname xprism
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void xprism_usage_additions(void)
{
	kfprintf(kstderr, "\tInteractive 2D & 3D Plotting\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: xprism_free_args
| 
|       Purpose: Frees CLUI struct allocated in xprism_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname xprism
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
xprism_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->form_file);
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->i3_file);
		kfree_and_NULL(clui_info->i4_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
