/*
 * Khoros: $Id: vrgeometry.h,v 1.3.2.1 1997/04/07 22:52:08 mirage Exp $
 */
 

#ifndef _vrgeometry_h_
#define _vrgeometry_h_




/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 
/*
 * Khoros: $Id: vrgeometry.h,v 1.3.2.1 1997/04/07 22:52:08 mirage Exp $
 */
 



	/*-------------------------------------*
	|       #include
	---------------------------------------*/


	/*-------------------------------------*
	|       #defines
	---------------------------------------*/
#define MAXBUTTONS 9
#define BUTTON1 0
#define BUTTON2 1
#define BUTTON3 2
#define BUTTON4 3
#define BUTTON5 4
#define BUTTON6 5
#define BUTTON7 6
#define BUTTON8 7
#define BUTTON9 8



	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/

typedef struct vrevent
{
  float trans[3];		/* in order of x,y,z */
  float rot[3];			/* in order of x,y,z */
  int button_events[MAXBUTTONS];
  int have_any_button_events;
  int have_any_rotations,have_any_translations;
  int clock;		/* might find a reason to use this */
} vrevent;

typedef struct camera_struct
{
    float eye[3],at[3],up[3];
    int projection;
    float fov,hither,yon;
    int is_stereo;
    float eye_sep,focal_len;
} camera_struct;

	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/


	/*-------------------------------------*
	|       macros
	---------------------------------------*/

#define VR_CLEAR_ALL_FLAGS(e) \
{ \
  (e)->have_any_button_events = 0; \
  (e)->have_any_rotations = 0; \
  (e)->have_any_translations = 0; \
  (e)->trans[0] = (e)->trans[1] = (e)->trans[2] = 0; \
  (e)->rot[0] = (e)->rot[1] = (e)->rot[2] = 0; \
  (e)->clock = -1; \
}

/* this macro clears out all the buttons in a vrevent structure */
#define VR_CLEAR_ALL_BUTTONS(e) \
{ \
    int i; \
    for (i=0;i<MAXBUTTONS;i++) \
	(e)->button_events[i] = 0; \
    (e)->have_any_button_events = 0; \
}

/* this macro sets the "have a rotation" flag to the desired value */

#define VR_SET_ROTATIONS_FLAG(flag,e) \
{ \
    (e)->have_any_rotations = flag; \
}				     

/* set the rotation vector in the vr event structure */
#define VR_SET_ROTATION_VEC(v,e) \
{ \
  (e)->rot[0] = v[0];\
  (e)->rot[1] = v[1];\
  (e)->rot[2] = v[2];\
}

/* this macro sets the "have a translation" flag to the desired value */

#define VR_SET_TRANSLATIONS_FLAG(flag,e) \
{ \
    (e)->have_any_translations = flag; \
}				     

/* set the translation vector in the vr event structure */
#define VR_SET_TRANSLATION_VEC(v,e) \
{ \
  (e)->trans[0] = v[0];\
  (e)->trans[1] = v[1];\
  (e)->trans[2] = v[2];\
}

/* stuff for buttons */
#define VR_SET_BUTTONS_FLAG(flag,e) \
{ \
  (e)->have_any_button_events = flag; \
}				     

#define VR_SET_BUTTON(index,flag,e) \
{ \
  (e)->button_events[index] = flag; \
}

#define VR_GET_BUTTON(index,e,i) \
{ \
  i = (e)->button_events[index]; \
}

	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

int encode_vrevent PROTO((vrevent *, kgeom_object *));
int decode_vrevent PROTO((vrevent *, kgeom_object *));
int extract_camera_info PROTO((camera_struct *, kgeom_object *));
int extract_model_matrix PROTO((float **, kgeom_object *, char **));
int extract_model_translation PROTO((float **, kgeom_object *, char **));
int extract_view_matrix PROTO((float **, kgeom_object *));
int build_relative_object_xform PROTO((float *, float *, float *, vrevent *));
int write_relative_object_xform PROTO((float *, kgeom_object *, char *));
int write_relative_camera_xform PROTO((kgeom_object *, camera_struct *));
int build_relative_camera_xform PROTO((camera_struct *, vrevent *, camera_struct *, float *));
int vr_search_for_named_object PROTO((char *name, kgeom_object *obj));
int extract_named_model_matrix PROTO((float **model, kgeom_object *obj,char *name));
int extract_named_model_translation PROTO((float **trans,kgeom_object *obj,char *name));

	/* Ghostwriter Prototypes:  Do Not Erase This Comment */

#endif /* _vrgeometry_h_ */
/* Don't add after this point */
