 /*
  * Khoros: $Id: g3daxis.c,v 1.8.2.2 1997/05/31 18:49:26 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: g3daxis.c,v 1.8.2.2 1997/05/31 18:49:26 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for g3daxis
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "g3daxis.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Generates a 3d axis geometry object from data.
|
|       Purpose: main program for g3daxis
|
|         Input:
|		char *clui_info->i_file; {Data file containing 3d location data.}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Geometry object containing a 3d axis.}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		Required M.E. group:
|		    int clui_info->do_auto_extents_logic; {Indicates if the extents of the axes should be computed from the input location data.}
|		    int clui_info->do_auto_extents_flag; {TRUE if -do_auto_extents specified}
|
|		    M.I. group:
|			float clui_info->xmin_float; {Minimum X coordinate for user-specified extents.}
|			int   clui_info->xmin_flag; {TRUE if -xmin specified}
|
|			float clui_info->xmax_float; {Maximum X coordinate for user-specified extents.}
|			int   clui_info->xmax_flag; {TRUE if -xmax specified}
|
|			float clui_info->ymin_float; {Minimum Y coordinate for user-specified extents.}
|			int   clui_info->ymin_flag; {TRUE if -ymin specified}
|
|			float clui_info->ymax_float; {Maximum Y coordinate for user-specified extents.}
|			int   clui_info->ymax_flag; {TRUE if -ymax specified}
|
|			float clui_info->zmin_float; {Minimum Z coordinate for user-specified extents.}
|			int   clui_info->zmin_flag; {TRUE if -zmin specified}
|
|			float clui_info->zmax_float; {Maximum Z coordinate for user-specified extents.}
|			int   clui_info->zmax_flag; {TRUE if -zmax specified}
|
|		char *clui_info->name_string; {Name used as the internal object name.}
|		int   clui_info->name_flag; {TRUE if -name specified}
|
|		M.I. group:
|		    int clui_info->do_ticks_logic; {Will cause tick marks to be generated along each axis.}
|		    int clui_info->do_ticks_flag; {TRUE if -do_ticks specified}
|
|		    int clui_info->tdensity_int; {Indicates the number of tick marks/text along each axis.}
|		    int clui_info->tdensity_flag; {TRUE if -tdensity specified}
|
|		    int clui_info->do_labels_logic; {Will cause text to appear in the geometry.}
|		    int clui_info->do_labels_flag; {TRUE if -do_labels specified}
|
|		    char *clui_info->sprintf_format_string; {Raw text formatting string used when labels are sprintf'ed.  NOTE: the user's input string is NOT checked for validity.  The format string should be valid for floating point values, eg: %g, %6.4f, etc..}
|		    int   clui_info->sprintf_format_flag; {TRUE if -sprintf_format specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
    kobject input;
    kgeom_object *output;
    int do_autoextents,do_ticks,do_labels,tick_density;
    float extents[6];
    char *name;
    char *format_string;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/g3daxis");
	kexit_handler(g3daxis_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "g3daxis", KGEN_KROUTINE,  &clui_uis_spec,
		   g3daxis_usage_additions, g3daxis_get_args,
 		   g3daxis_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    /* open objects */
    input = kpds_open_input_object(clui_info->i_file);
    output = kgeom_new_object();

    /* gather input parms. */
    do_ticks = clui_info->do_ticks_logic;
    do_labels = clui_info->do_labels_logic;
    tick_density = clui_info->tdensity_int;
    name = clui_info->name_string;
    do_autoextents = clui_info->do_auto_extents_logic;
    format_string = clui_info->sprintf_format_string;
    
    if (do_autoextents == 0)
    {
	extents[0] = clui_info->xmin_float;
	extents[1] = clui_info->ymin_float;
	extents[2] = clui_info->zmin_float;
	
	extents[3] = clui_info->xmax_float;
	extents[4] = clui_info->ymax_float;
	extents[5] = clui_info->zmax_float;
    }
/* -main_before_lib_call_end */

/* -main_library_call */
	lg3daxis(input,output,do_ticks,do_labels,do_autoextents,
		 tick_density,name,extents,format_string);
/* -main_library_call_end */

/* -main_after_lib_call */
    kpds_close_object(input);
    kgeom_write_object(output, clui_info->o_file);
    kgeom_blast_object(output);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: g3daxis_usage_additions
| 
|       Purpose: Prints usage additions in g3daxis_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname g3daxis
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void g3daxis_usage_additions(void)
{
	kfprintf(kstderr, "\tGenerates a 3d axis geometry object from data.\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: g3daxis_free_args
| 
|       Purpose: Frees CLUI struct allocated in g3daxis_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname g3daxis
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
g3daxis_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->name_string);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
