 /*
  * Khoros: $Id: gcmaplegend.c,v 1.11.2.2 1997/05/31 18:50:05 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gcmaplegend.c,v 1.11.2.2 1997/05/31 18:50:05 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for gcmaplegend
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "gcmaplegend.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Construct Geometry Representing a Color Legend
|
|       Purpose: main program for gcmaplegend
|
|         Input:
|		char *clui_info->i_file; {input vis colormap}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output geometry object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->name_string; {color legend}
|		int   clui_info->name_flag; {TRUE if -name specified}
|
|		int clui_info->orient_toggle; {legend orientation}
|		int clui_info->orient_flag; {TRUE if -orient specified}
|
|		int clui_info->reverse_logic; {reverse colors?}
|		int clui_info->reverse_flag; {TRUE if -reverse specified}
|
|		int clui_info->outline_logic; {make outline?}
|		int clui_info->outline_flag; {TRUE if -outline specified}
|
|		float clui_info->length_float; {length of legend}
|		int   clui_info->length_flag; {TRUE if -length specified}
|
|		float clui_info->thickness_float; {thickness of legend}
|		int   clui_info->thickness_flag; {TRUE if -thickness specified}
|
|		float clui_info->xpos_float; {x position}
|		int   clui_info->xpos_flag; {TRUE if -xpos specified}
|
|		float clui_info->ypos_float; {y position}
|		int   clui_info->ypos_flag; {TRUE if -ypos specified}
|
|		int clui_info->do_text_labels_logic; {Boolean value indicating whether or not text labels will be drawn.}
|		int clui_info->do_text_labels_flag; {TRUE if -do_text_labels specified}
|
|		int clui_info->num_labels_int; {Number of subdivisions along color legend for text labels.}
|		int clui_info->num_labels_flag; {TRUE if -num_labels specified}
|
|		char *clui_info->sprintf_format_string; {C-style formatting string.  No error checking is perfomed.  Use %g for most stuff.}
|		int   clui_info->sprintf_format_flag; {TRUE if -sprintf_format specified}
|
|		float clui_info->text_offset_float; {Offset of text in either the horizontal or vertical dimension, depending upon the orientation of the legend.}
|		int   clui_info->text_offset_flag; {TRUE if -text_offset specified}
|
|		int clui_info->do_title_logic; {Boolean indicating whether or not to generate a title to go along with the legend.}
|		int clui_info->do_title_flag; {TRUE if -do_title specified}
|
|		char *clui_info->title_string_string; {The text for the title.}
|		int   clui_info->title_string_flag; {TRUE if -title_string specified}
|
|		float clui_info->title_x_offset_float; {X-Offset of title relative to the origin of the legend.}
|		int   clui_info->title_x_offset_flag; {TRUE if -title_x_offset specified}
|
|		float clui_info->title_y_offset_float; {Y-offset of title relative to legend origin.}
|		int   clui_info->title_y_offset_flag; {TRUE if -title_y_offset specified}
|
|		int   clui_info->title_font_size_list; {Choices for point size for the title.}
|		char *clui_info->title_font_size_label;
|		int   clui_info->title_font_size_flag; {TRUE if -title_font_size specified}
|
|		int   clui_info->title_font_family_list; {List of available font families.  Based upon those commonly available on X servers.  This works reliably only in the software renderer.}
|		char *clui_info->title_font_family_label;
|		int   clui_info->title_font_family_flag; {TRUE if -title_font_family specified}
|
|        Output:
|       Returns:
|
|    Written By: Wes Bethel
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
    kobject input;
    kgeom_object *output;
    char *name;
    int reverse,outline,horiz_vert;
    float length, thickness;
    float xpos, ypos;
    char *sprintf_format;
    int do_labels,num_labels;
    int do_title;
    float title_x_offset,title_y_offset;
    char *title_string;
    int title_font_size;
    char title_font_family[64];
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/gcmaplegend");
	kexit_handler(gcmaplegend_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "gcmaplegend", KGEN_KROUTINE,  &clui_uis_spec,
		   gcmaplegend_usage_additions, gcmaplegend_get_args,
 		   gcmaplegend_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    /* gather the parameters for the library call. */
    
    input = kpds_open_object(clui_info->i_file,KOBJ_READ);

    output = kgeom_new_object();
    
    name = clui_info->name_string;
    reverse = clui_info->reverse_logic;
    outline = clui_info->outline_logic;
    length = clui_info->length_float;
    thickness = clui_info->thickness_float;
    horiz_vert = clui_info->orient_toggle;
    xpos = clui_info->xpos_float;
    ypos = clui_info->ypos_float;

    sprintf_format = clui_info->sprintf_format_string;
    do_labels = clui_info->do_text_labels_logic;
    num_labels = clui_info->num_labels_int;

    do_title = clui_info->do_title_logic;
    title_x_offset = clui_info->title_x_offset_float;
    title_y_offset = clui_info->title_y_offset_float;
    title_string = clui_info->title_string_string;

    /* WARNING!  the following statement assumes that in the font size
     list of the UI that the strings composing the list are of the
     form "dd some_text" where "dd" is a size on points. */
    ksscanf(clui_info->title_font_size_label,"%d",&title_font_size);

    kstrcpy(title_font_family,clui_info->title_font_family_label);
/* -main_before_lib_call_end */

/* -main_library_call */
    lgcmaplegend(input,output,name,reverse,outline,horiz_vert,&length,&thickness,&xpos,&ypos,do_labels,num_labels,sprintf_format,do_title,title_string,&title_x_offset,&title_y_offset,title_font_size,title_font_family);
/* -main_library_call_end */

/* -main_after_lib_call */
    kpds_close_object(input);
    kgeom_write_object(output, clui_info->o_file);
    kgeom_blast_object(output);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: gcmaplegend_usage_additions
| 
|       Purpose: Prints usage additions in gcmaplegend_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gcmaplegend
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void gcmaplegend_usage_additions(void)
{
	kfprintf(kstderr, "\tConstruct Geometry Representing a Color Legend\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: gcmaplegend_free_args
| 
|       Purpose: Frees CLUI struct allocated in gcmaplegend_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gcmaplegend
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
gcmaplegend_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->name_string);
		kfree_and_NULL(clui_info->sprintf_format_string);
		kfree_and_NULL(clui_info->title_string_string);
		kfree_and_NULL(clui_info->title_font_size_label);
		kfree_and_NULL(clui_info->title_font_family_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
