 /*
  * Khoros: $Id: gcreate2dtex.c,v 1.8.2.2 1997/05/31 18:50:46 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gcreate2dtex.c,v 1.8.2.2 1997/05/31 18:50:46 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for gcreate2dtex
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "gcreate2dtex.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Create a 2d texture from an image.
|
|       Purpose: main program for gcreate2dtex
|
|         Input:
|		char *clui_info->i_file; {First Input data object}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Resulting output data object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int   clui_info->tex_filter_list; {Control over the method used in filtering the input image when making a texture.}
|		char *clui_info->tex_filter_label;
|		int   clui_info->tex_filter_flag; {TRUE if -tex_filter specified}
|
|		float clui_info->umin_float; {Minimum u-coordinate value associated with origin of image data.}
|		int   clui_info->umin_flag; {TRUE if -umin specified}
|
|		float clui_info->vmin_float; {Mininum v-coordinate assigned to the origin of the texture.}
|		int   clui_info->vmin_flag; {TRUE if -vmin specified}
|
|		float clui_info->umax_float; {Maximum u-coordinate to use for the non-origin corner of the texture image.}
|		int   clui_info->umax_flag; {TRUE if -umax specified}
|
|		float clui_info->vmax_float; {Maximum v-coordinate to use for the non-origin corner of the texture image.}
|		int   clui_info->vmax_flag; {TRUE if -vmax specified}
|
|        Output:
|       Returns:
|
|    Written By: wes
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	kobject inobj;
	kgeom_object *outobj;
	float tc[4];
	int method;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/gcreate2dtex");
	kexit_handler(gcreate2dtex_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "gcreate2dtex", KGEN_KROUTINE,  &clui_uis_spec,
		   gcreate2dtex_usage_additions, gcreate2dtex_get_args,
 		   gcreate2dtex_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	inobj = kpds_open_object(clui_info->i_file,KOBJ_READ);
	outobj = kgeom_new_object();
	
	tc[0] = clui_info->umin_float;
	tc[1] = clui_info->vmin_float;
	tc[2] = clui_info->umax_float;
	tc[3] = clui_info->vmax_float;

	method = clui_info->tex_filter_list - 1;
/* -main_before_lib_call_end */

/* -main_library_call */
	lgcreate2dtex(inobj,outobj,tc,method);
/* -main_library_call_end */

/* -main_after_lib_call */
	kpds_close_object(inobj);
	kgeom_write_object(outobj, clui_info->o_file);
	kgeom_blast_object(outobj);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: gcreate2dtex_usage_additions
| 
|       Purpose: Prints usage additions in gcreate2dtex_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gcreate2dtex
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void gcreate2dtex_usage_additions(void)
{
	kfprintf(kstderr, "\tCreate a 2d texture from an image.\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: gcreate2dtex_free_args
| 
|       Purpose: Frees CLUI struct allocated in gcreate2dtex_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gcreate2dtex
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
gcreate2dtex_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->tex_filter_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
