 /*
  * Khoros: $Id: gcuberille.c,v 1.11.2.3 1997/05/31 18:51:06 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gcuberille.c,v 1.11.2.3 1997/05/31 18:51:06 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for gcuberille
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "gcuberille.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Volume Visualization using Geometric Icons.
|
|       Purpose: main program for gcuberille
|
|         Input:
|		char *clui_info->i_file; {input 3d data object}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->cmap_file; {input vis colormap}
|		int   clui_info->cmap_flag; {TRUE if -cmap specified}
|
|		char *clui_info->o_file; {Resulting output data object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		float clui_info->shrink_float; {ratio of cuberille size to grid block size.}
|		int   clui_info->shrink_flag; {TRUE if -shrink specified}
|
|		char *clui_info->name_string; {object name}
|		int   clui_info->name_flag; {TRUE if -name specified}
|
|        Output:
|       Returns:
|
|    Written By: Wes Bethel
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
    kobject inobj, cmapobj;
    kgeom_object *outobj;
    float shrinkage;
    char *name;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/gcuberille");
	kexit_handler(gcuberille_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "gcuberille", KGEN_KROUTINE,  &clui_uis_spec,
		   gcuberille_usage_additions, gcuberille_get_args,
 		   gcuberille_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    inobj = kpds_open_object(clui_info->i_file,KOBJ_READ);
    cmapobj = kpds_open_object(clui_info->cmap_file,KOBJ_READ);
    outobj = kgeom_new_object();

    shrinkage = clui_info->shrink_float;
    name = clui_info->name_string;

    kgeom_write_header(outobj, clui_info->o_file);
/* -main_before_lib_call_end */

/* -main_library_call */
    lgcuberille(inobj,cmapobj,outobj,&shrinkage,name);
/* -main_library_call_end */

/* -main_after_lib_call */
    kpds_close_object(inobj);
    kpds_close_object(cmapobj);

    kgeom_finish_writing_object(outobj);
    kgeom_done_writing(outobj);

    kgeom_blast_object(outobj);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: gcuberille_usage_additions
| 
|       Purpose: Prints usage additions in gcuberille_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gcuberille
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void gcuberille_usage_additions(void)
{
	kfprintf(kstderr, "\tVolume Visualization using Geometric Icons.\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: gcuberille_free_args
| 
|       Purpose: Frees CLUI struct allocated in gcuberille_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gcuberille
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
gcuberille_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->cmap_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->name_string);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
