 /*
  * Khoros: $Id: gextents.c,v 1.12.2.3 1997/05/31 18:51:23 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gextents.c,v 1.12.2.3 1997/05/31 18:51:23 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for gextents
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "gextents.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Generate Geometry Representing Extents Around Data
|
|       Purpose: main program for gextents
|
|         Input:
|		char *clui_info->i_file; {input data object}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output geometry object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->show_bounds_logic; {If true (default), will display the wireframe bounding box of the object.}
|		int clui_info->show_bounds_flag; {TRUE if -show_bounds specified}
|
|		int clui_info->color_toggle; {If 1, the lines will be colored white, else they will use the red, green, cyan color scheme described above}
|		int clui_info->color_flag; {TRUE if -color specified}
|
|		int clui_info->type_toggle; {Select between one of the two box types}
|		int clui_info->type_flag; {TRUE if -type specified}
|
|		char *clui_info->name_string; {The name selected here will identify the extents object in the renderer}
|		int   clui_info->name_flag; {TRUE if -name specified}
|
|		int clui_info->u_min_logic; {If true, will cause a two D wire mesh object to be built at u=0.}
|		int clui_info->u_min_flag; {TRUE if -u_min specified}
|
|		int clui_info->u_max_logic; {If true, will cause a 2d wire mesh object at u=umax to be build.}
|		int clui_info->u_max_flag; {TRUE if -u_max specified}
|
|		int clui_info->v_min_logic; {If true, will cause a 2d wire mesh object at v=0 to be built.}
|		int clui_info->v_min_flag; {TRUE if -v_min specified}
|
|		int clui_info->v_max_logic; {If true, will cause a 2d wire mesh object at v=1 to be built.}
|		int clui_info->v_max_flag; {TRUE if -v_max specified}
|
|		int clui_info->w_min_logic; {If true, will case a 2d wire mesh object to be built at W=0.}
|		int clui_info->w_min_flag; {TRUE if -w_min specified}
|
|		int clui_info->w_max_logic; {If true, will cause a 2d wire mesh object to be built at w=1.}
|		int clui_info->w_max_flag; {TRUE if -w_max specified}
|
|		int   clui_info->lod_list; {Controls how much detail is produced.  This variable provides control over the amount of geometry gneerated.}
|		char *clui_info->lod_label;
|		int   clui_info->lod_flag; {TRUE if -lod specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
    kobject input;
    kgeom_object *output;
    int style,type;
    char *objname;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/gextents");
	kexit_handler(gextents_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "gextents", KGEN_KROUTINE,  &clui_uis_spec,
		   gextents_usage_additions, gextents_get_args,
 		   gextents_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    /* open the objects. */
    input = kpds_open_input_object(clui_info->i_file);
    output = kgeom_new_object();

    /* grab parms from structure. */
    style = clui_info->color_toggle;
    type = clui_info->type_toggle;
    objname = clui_info->name_string;
/* -main_before_lib_call_end */

/* -main_library_call */
    make_extents_box(input,output,style,objname,type,
		     clui_info->show_bounds_logic,
		     clui_info->u_min_logic,clui_info->u_max_logic,
		     clui_info->v_min_logic,clui_info->v_max_logic,
		     clui_info->w_min_logic,clui_info->w_max_logic);
/* -main_library_call_end */

/* -main_after_lib_call */
    /* close the objects */
    kpds_close_object(input);
    kgeom_write_object(output, clui_info->o_file);
    kgeom_blast_object(output);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: gextents_usage_additions
| 
|       Purpose: Prints usage additions in gextents_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gextents
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void gextents_usage_additions(void)
{
	kfprintf(kstderr, "\tGenerate Geometry Representing Extents Around Data\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: gextents_free_args
| 
|       Purpose: Frees CLUI struct allocated in gextents_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gextents
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
gextents_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->name_string);
		kfree_and_NULL(clui_info->lod_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
