 /*
  * Khoros: $Id: gflipnorms.c,v 1.11.2.2 1997/05/31 18:51:43 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gflipnorms.c,v 1.11.2.2 1997/05/31 18:51:43 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for gflipnorms
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "gflipnorms.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Flip all Normals in a Geometry Object.
|
|       Purpose: main program for gflipnorms
|
|         Input:
|		char *clui_info->i_file; {input object containing normals}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {resulting output data object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	kgeom_object    *geom = NULL;
	kgeom_primitive *prim = NULL;

	float *norms = NULL;
	int    num_norms = 0;
	int    num, i, j;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/gflipnorms");
	kexit_handler(gflipnorms_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "gflipnorms", KGEN_KROUTINE,  &clui_uis_spec,
		   gflipnorms_usage_additions, gflipnorms_get_args,
 		   gflipnorms_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	geom = kgeom_read_object(clui_info->i_file);
	
	if (geom == NULL)
	{
	   kerror(NULL, "gflipnorms", "Unable to open input %s",
		  clui_info->i_file);

	   kexit(KEXIT_FAILURE);
	}
/* -main_before_lib_call_end */

/* -main_library_call */
	/*-- loop over all primitives with normals --*/
	num = kgeom_number_primitives(geom);
	
	for (i = 0; i < num; i++)
	{
	   prim = kgeom_get_primitive(geom, i);
	   
	   switch (prim->type)
	   {
	      case KGEOM_TRIANGLES_DISJOINT :
		 {
		    kgeom_triangles *p = (kgeom_triangles *) prim;
		    norms = p->norms;
		    num_norms = (geom->layout == KPER_VERTEX) ? p->nverts 
		                                              : p->nverts/3;
		 } break;
		 
	      case KGEOM_TRIANGLES_CONNECTED :
		 {
		    kgeom_triangles *p = (kgeom_triangles *) prim;
		    norms = p->norms;
		    num_norms = (geom->layout == KPER_VERTEX) ? p->nverts 
		                                              : p->nverts-1;
		 } break;

	      case KGEOM_DIRECTED_POINTS :
		 {
		    kgeom_directed_points *p = (kgeom_directed_points *) prim;
		    norms = p->norms;
		    num_norms = p->nverts;
		 } break;

	      case KGEOM_QUADMESH :
		 {
		    kgeom_quadmesh *p = (kgeom_quadmesh *) prim;
		    norms = p->norms;
		    
		    if (geom->layout == KPER_VERTEX)
		       num_norms = p->width * p->height;
		    else
		       num_norms = (p->width-1) * (p->height-1);
		 } break;

	      case KGEOM_OCTMESH :
		 {
		    kgeom_octmesh *p = (kgeom_octmesh *) prim;
		    norms = p->norms;
		    
		    if (geom->layout == KPER_VERTEX)
		       num_norms = p->width * p->height * p->depth;
		    else
		       num_norms = (p->width-1) * (p->height-1) * (p->depth-1);
		 } break;
	      
	      default :  /*-- no other primitives have normals --*/
		 norms = NULL;
		 num_norms = 0;
	   }

	   /*-- flip the normals for this primitive --*/
	   for (j = 0; j < num_norms * geom->location_dim; j++)
	      *norms++ *= -1;  /* really naive way to flip.. hope it works */ 
	}
/* -main_library_call_end */

/* -main_after_lib_call */
	/*-- write the geometry out to the output --*/   
	if (!kgeom_write_object(geom, clui_info->o_file))
	{
	   kerror(NULL, "gflipnorms", "Unable to open output %s",
		  clui_info->o_file);
	   kexit(KEXIT_FAILURE);
	}
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: gflipnorms_usage_additions
| 
|       Purpose: Prints usage additions in gflipnorms_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gflipnorms
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void gflipnorms_usage_additions(void)
{
	kfprintf(kstderr, "\tFlip all Normals in a Geometry Object.\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: gflipnorms_free_args
| 
|       Purpose: Frees CLUI struct allocated in gflipnorms_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gflipnorms
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
gflipnorms_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
