 /*
  * Khoros: $Id: ghogs.c,v 1.5.2.3 1997/05/31 18:52:38 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: ghogs.c,v 1.5.2.3 1997/05/31 18:52:38 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for ghogs
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "ghogs.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Generate hedgehog icons at each grid node of a vector field.
|
|       Purpose: main program for ghogs
|
|         Input:
|		char *clui_info->i_file; {Input data object containing vector value data.}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Resulting output data object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int   clui_info->icon_choice_list; {List of icons used for geometric icon.}
|		char *clui_info->icon_choice_label;
|		int   clui_info->icon_choice_flag; {TRUE if -icon_choice specified}
|
|		float clui_info->icon_scale_float; {Used to scale the size of the icon.}
|		int   clui_info->icon_scale_flag; {TRUE if -icon_scale specified}
|
|		int   clui_info->linear_log_list; {Used to control if scaling is done using linear or log10 basis.}
|		char *clui_info->linear_log_label;
|		int   clui_info->linear_log_flag; {TRUE if -linear_log specified}
|
|		int clui_info->icon_smooth_int; {Controls how finely procedural icons are tesselated.}
|		int clui_info->icon_smooth_flag; {TRUE if -icon_smooth specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	kobject input;
	kgeom_object *output;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/ghogs");
	kexit_handler(ghogs_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "ghogs", KGEN_KROUTINE,  &clui_uis_spec,
		   ghogs_usage_additions, ghogs_get_args,
 		   ghogs_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	input = kpds_open_object(clui_info->i_file,KOBJ_READ);

	output = kgeom_new_object();

	kgeom_write_header(output, clui_info->o_file);

	kgeom_start_writing_object(output);
/* -main_before_lib_call_end */

/* -main_library_call */
	lghogs(input,output,clui_info);
/* -main_library_call_end */

/* -main_after_lib_call */
	kpds_close_object(input);

	kgeom_finish_writing_object(output);
	kgeom_done_writing(output);
	kgeom_blast_object(output);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: ghogs_usage_additions
| 
|       Purpose: Prints usage additions in ghogs_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname ghogs
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void ghogs_usage_additions(void)
{
	kfprintf(kstderr, "\tGenerate hedgehog icons at each grid node of a vector field.\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: ghogs_free_args
| 
|       Purpose: Frees CLUI struct allocated in ghogs_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname ghogs
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
ghogs_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->icon_choice_label);
		kfree_and_NULL(clui_info->linear_log_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
