 /*
  * Khoros: $Id: gimpfacet.c,v 1.11.2.2 1997/05/31 18:52:57 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gimpfacet.c,v 1.11.2.2 1997/05/31 18:52:57 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for gimpfacet
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "gimpfacet.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Import FACET data
|
|       Purpose: main program for gimpfacet
|
|         Input:
|		char *clui_info->i_file; {input facet file}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output geometry object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->name_string; {object name}
|		int   clui_info->name_flag; {TRUE if -name specified}
|
|        Output:
|       Returns:
|
|    Written By: Steve Kubica
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	kfile  *in;
	kgeom_object *out;
	char   *name;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/gimpfacet");
	kexit_handler(gimpfacet_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "gimpfacet", KGEN_KROUTINE,  &clui_uis_spec,
		   gimpfacet_usage_additions, gimpfacet_get_args,
 		   gimpfacet_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	if ((in = kfopen(clui_info->i_file, "r")) == NULL)
	{
	   kerror(NULL, "gimpfacet", 
		  "Unable to open input '%s' for reading",
		  (clui_info->i_file == NULL) ? "NULL" : clui_info->i_file);
	   kexit(KEXIT_FAILURE);
	}

	if ((out = kgeom_new_object()) == NULL)
	{
	   kerror(NULL, "gimpfacet", 
		  "Unable to allocate space for output geometry");
	   kexit(KEXIT_FAILURE);
	}

	if (clui_info->name_flag)
	   name = clui_info->name_string;
	else
	   name = "facets";
/* -main_before_lib_call_end */

/* -main_library_call */
	lgimpfacet(in, name, out);
/* -main_library_call_end */

/* -main_after_lib_call */
	kfclose(in);

	if (!kgeom_write_object(out,clui_info->o_file))
	{
	   kerror(NULL, "gimpfacet", 
		  "Unable to open '%s' for writing",
		  (clui_info->o_file == NULL) ? "NULL" : clui_info->o_file);
	   kexit(KEXIT_FAILURE);
	}
	
	kgeom_blast_object(out);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: gimpfacet_usage_additions
| 
|       Purpose: Prints usage additions in gimpfacet_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gimpfacet
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void gimpfacet_usage_additions(void)
{
	kfprintf(kstderr, "\tImport FACET data\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: gimpfacet_free_args
| 
|       Purpose: Frees CLUI struct allocated in gimpfacet_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gimpfacet
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
gimpfacet_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->name_string);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
