 /*
  * Khoros: $Id: gisosurface.c,v 1.13.2.2 1997/05/31 18:53:14 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gisosurface.c,v 1.13.2.2 1997/05/31 18:53:14 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for gisosurface
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "gisosurface.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Generate surface of constant value from 3d data.
|
|       Purpose: main program for gisosurface
|
|         Input:
|		char *clui_info->i_file; {input 3d data object}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		float clui_info->l_float; {contouring level}
|		int   clui_info->l_flag; {TRUE if -l specified}
|
|		char *clui_info->o_file; {output geometry object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->normals_logic; {compute normals?}
|		int clui_info->normals_flag; {TRUE if -normals specified}
|
|		int clui_info->flip_logic; {reverse normals?}
|		int clui_info->flip_flag; {TRUE if -flip specified}
|
|		char *clui_info->name_string; {object name}
|		int   clui_info->name_flag; {TRUE if -name specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
    kobject inobj;
    kgeom_object *outobj;
    double threshold;
    int flip_flag,do_normals;
    char *name;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/gisosurface");
	kexit_handler(gisosurface_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "gisosurface", KGEN_KROUTINE,  &clui_uis_spec,
		   gisosurface_usage_additions, gisosurface_get_args,
 		   gisosurface_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    /**
     * we need to get the input, output objects, and the
     * level to pass to the iso_main() call (below).
     **/
    inobj = kpds_open_input_object(clui_info->i_file);
    outobj = kgeom_new_object();

    kgeom_write_header(outobj, clui_info->o_file);

    kgeom_start_writing_object(outobj);

    threshold = clui_info->l_float;
    flip_flag = clui_info->flip_logic;
    do_normals = clui_info->normals_logic;
    name = clui_info->name_string;
/* -main_before_lib_call_end */

/* -main_library_call */
	iso_main(inobj,outobj,threshold,do_normals,flip_flag,name);
/* -main_library_call_end */

/* -main_after_lib_call */
    kpds_close_object(inobj);

    kgeom_finish_writing_object(outobj);
    kgeom_done_writing(outobj);
    
    kgeom_blast_object(outobj);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: gisosurface_usage_additions
| 
|       Purpose: Prints usage additions in gisosurface_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gisosurface
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void gisosurface_usage_additions(void)
{
	kfprintf(kstderr, "\tGenerate surface of constant value from 3d data.\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: gisosurface_free_args
| 
|       Purpose: Frees CLUI struct allocated in gisosurface_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gisosurface
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
gisosurface_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->name_string);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
