 /*
  * Khoros: $Id: glinecolor.c,v 1.11.2.3 1997/05/31 18:53:33 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: glinecolor.c,v 1.11.2.3 1997/05/31 18:53:33 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for glinecolor
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "glinecolor.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Create Polylines from ASCII XYZ data and a Vis Cmap
|
|       Purpose: main program for glinecolor
|
|         Input:
|		char *clui_info->i_file; {input ASCII data containing polyline points}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->cmap_file; {Input vis colormap}
|		int   clui_info->cmap_flag; {TRUE if -cmap specified}
|
|		char *clui_info->o_file; {output geometry object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->name_string; {object name}
|		int   clui_info->name_flag; {TRUE if -name specified}
|
|		int clui_info->close_logic; {connect first and last points in list of vertices.}
|		int clui_info->close_flag; {TRUE if -close specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	kgeom_object *outobj;
	kobject vismap;
	kfile *inobj;
	char *name;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/glinecolor");
	kexit_handler(glinecolor_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "glinecolor", KGEN_KROUTINE,  &clui_uis_spec,
		   glinecolor_usage_additions, glinecolor_get_args,
 		   glinecolor_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	inobj = kfopen(clui_info->i_file,"r");
	if (inobj == NULL)
	{
	    kerror(NULL,"main","can't open the requested input file: <%s>",clui_info->i_file);
	    kexit(KEXIT_FAILURE);
	}
	vismap = kpds_open_object(clui_info->cmap_file,KOBJ_READ);
	outobj = kgeom_new_object();

	/* Color is specified with alpha in "lglinecolor", Terry Ligocki */
	outobj->has_alpha = TRUE;

	name = clui_info->name_string;
/* -main_before_lib_call_end */

/* -main_library_call */
	lglinecolor(inobj,vismap,outobj,name);
/* -main_library_call_end */

/* -main_after_lib_call */
	kfclose(inobj);
	kpds_close_object(vismap);
	kgeom_write_object(outobj, clui_info->o_file);
	kgeom_blast_object(outobj);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: glinecolor_usage_additions
| 
|       Purpose: Prints usage additions in glinecolor_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname glinecolor
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void glinecolor_usage_additions(void)
{
	kfprintf(kstderr, "\tCreate Polylines from ASCII XYZ data and a Vis Cmap\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: glinecolor_free_args
| 
|       Purpose: Frees CLUI struct allocated in glinecolor_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname glinecolor
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
glinecolor_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->cmap_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->name_string);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
