 /*
  * Khoros: $Id: goctmesh.c,v 1.11.2.2 1997/05/31 18:54:27 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: goctmesh.c,v 1.11.2.2 1997/05/31 18:54:27 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for goctmesh
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "goctmesh.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Create an Octmesh Geometry Object from 3D Data
|
|       Purpose: main program for goctmesh
|
|         Input:
|		char *clui_info->i_file; {input 3d data for octmesh}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output geometry octmesh}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->cmap_file; {input vis colormap}
|		int   clui_info->cmap_flag; {TRUE if -cmap specified}
|
|		int clui_info->do_normals_logic; {Flag indicating whether or not normals should be generated.  Normals are computed from the value data, and are required for voxel-dot shading.}
|		int clui_info->do_normals_flag; {TRUE if -do_normals specified}
|
|		int clui_info->flip_normals_logic; {Causes normals to be flipped or not.}
|		int clui_info->flip_normals_flag; {TRUE if -flip_normals specified}
|
|		char *clui_info->name_string; {object name}
|		int   clui_info->name_flag; {TRUE if -name specified}
|
|        Output:
|       Returns:
|
|    Written By: Yo Momma
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	kobject inobj,cmapobj;
	kgeom_object *outobj;
	int have_mapfile;
	char *name;
	int normals_flag;
	int flip_normals;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/goctmesh");
	kexit_handler(goctmesh_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "goctmesh", KGEN_KROUTINE,  &clui_uis_spec,
		   goctmesh_usage_additions, goctmesh_get_args,
 		   goctmesh_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	inobj = kpds_open_object(clui_info->i_file,KOBJ_READ);

	outobj = kgeom_new_object();

	kgeom_write_header(outobj, clui_info->o_file);

	if (clui_info->cmap_flag)
        {
	    cmapobj = kpds_open_object(clui_info->cmap_file,KOBJ_READ);
	    have_mapfile = TRUE;
	}
	else
	    have_mapfile = FALSE;

	name = clui_info->name_string;
	normals_flag = clui_info->do_normals_logic;
	flip_normals = clui_info->flip_normals_logic;
/* -main_before_lib_call_end */

/* -main_library_call */
	lgoctmesh(inobj,have_mapfile,cmapobj,outobj,name,normals_flag,
		  flip_normals);
/* -main_library_call_end */

/* -main_after_lib_call */
	kpds_close_object(inobj);

	kgeom_done_writing(outobj);
	kgeom_blast_object(outobj);
	
	if (have_mapfile == TRUE)
	    kpds_close_object(cmapobj);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: goctmesh_usage_additions
| 
|       Purpose: Prints usage additions in goctmesh_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname goctmesh
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void goctmesh_usage_additions(void)
{
	kfprintf(kstderr, "\tCreate an Octmesh Geometry Object from 3D Data\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: goctmesh_free_args
| 
|       Purpose: Frees CLUI struct allocated in goctmesh_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname goctmesh
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
goctmesh_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->cmap_file);
		kfree_and_NULL(clui_info->name_string);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
