
/*
 * Copyright (c) 1995-1997,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gprgeom_util.c,v 1.7.2.1 1997/04/07 22:52:19 mirage Exp $";
#endif

#include <geometry.h>
#include "gprgeom.h"

#define NUM_PRIMS 13

static void print_polyline_dis  PROTO((kgeom_object *, kgeom_primitive *, 
				       int, int, int, int, int, kfile *));
static void print_polyline_con  PROTO((kgeom_object *, kgeom_primitive *, 
				       int, int, int, int, int, kfile *));
static void print_triangles_dis PROTO((kgeom_object *, kgeom_primitive *, 
				       int, int, int, int, int, kfile *));
static void print_triangles_con PROTO((kgeom_object *, kgeom_primitive *, 
				       int, int, int, int, int, kfile *));
static void print_spheres       PROTO((kgeom_object *, kgeom_primitive *, 
				       int, int, int, int, int, kfile *));
static void print_qmesh         PROTO((kgeom_object *, kgeom_primitive *, 
				       int, int, int, int, int, kfile *));
static void print_omesh         PROTO((kgeom_object *, kgeom_primitive *, 
				       int, int, int, int, int, kfile *));
static void print_texture2d     PROTO((kgeom_object *, kgeom_primitive *, 
				       int, int, int, int, int, kfile *));
static void print_text          PROTO((kgeom_object *, kgeom_primitive *, 
				       int, int, int, int, int, kfile *));
static void print_obj_xform     PROTO((kgeom_object *, kgeom_primitive *, 
				       int, int, int, int, int, kfile *));
static void print_view_xform    PROTO((kgeom_object *, kgeom_primitive *, 
				       int, int, int, int, int, kfile *));
static void print_camera_stuff  PROTO((kgeom_object *, kgeom_primitive *, 
				       int, int, int, int, int, kfile *));
static void print_vr_event      PROTO((kgeom_object *, kgeom_primitive *, 
				       int, int, int, int, int, kfile *));
static void print_unknown       PROTO((kgeom_object *, kgeom_primitive *, 
				       int, int, int, int, int, kfile *));

static void (*prtfuncs[NUM_PRIMS+1]) PROTO((kgeom_object *, kgeom_primitive *,
					    int, int, int, int, int, 
					    kfile *)) =
{
    print_polyline_dis,
    print_polyline_con,
    print_triangles_dis,
    print_triangles_con,
    print_spheres,
    print_qmesh,
    print_omesh,
    print_texture2d,
    print_text,
    print_obj_xform,
    print_view_xform,
    print_camera_stuff,
    print_vr_event,
    print_unknown
};

static void print_vectors PROTO((float *, int, int, char *, char *, kfile *));
static void print_uvectors PROTO((kubyte *, int, int, char *, char *, kfile *));
static void print_att PROTO((kgeom_object *, kfile *));

static char *layout_string PROTO((int));

/*-----------------------------------------------------------
| 
|  Routine Name: kprgeom_util()
| 
|       Purpose: routine to print out information contained
|                in an object containing geometry information.
| 
|         Input: geom - the geometry object in question
| 
|        Output: bunch of text 
|    Written By: wes
|          Date: May 20, 1993
| Modifications:
|                Wed Jan 10 14:14:20 PST 1996  add support for text
| 
------------------------------------------------------------*/
void
kprgeom_util(
   kgeom_object *geom,
   kfile        *output,
   int           do_att,
   int           do_loc,
   int           do_color,
   int           do_norm_radii,
   int           do_texture)
{
   kgeom_primitive *prim;
   int num_prims;
   int func_index;
   int i;

   if (do_att)
      print_att(geom,output);

   if ((num_prims = kgeom_number_primitives(geom)) == 0)
      kfprintf(output,"\tObject contains no primitives\n");

   for (i=0;i<num_prims;i++)
   {
      if (prim = kgeom_get_primitive(geom, i))
      {
	 switch (prim->type)
	 {
	    case KGEOM_POLYLINE_DISJOINT:
	       func_index = 0;
	       break;
	    case KGEOM_POLYLINE_CONNECTED:
	       func_index = 1;
	       break;
	    case KGEOM_TRIANGLES_DISJOINT:
	       func_index = 2;
	       break;
	    case KGEOM_TRIANGLES_CONNECTED:
	       func_index = 3;
	       break;
	    case KGEOM_SPHERES:
	       func_index = 4;
	       break;
	    case KGEOM_QUADMESH:
	       func_index = 5;
	       break;
	    case KGEOM_OCTMESH:
	       func_index = 6;
	       break;
	    case KGEOM_TEXTURE2D:
	       func_index = 7;
	       break;
	    case KGEOM_TEXT:
	       func_index = 8;
	       break;
	    case KGEOM_OBJECT_XFORM:
	       func_index = 9;
	       break;
	    case KGEOM_VIEW_XFORM:
	       func_index = 10;
	       break;
	    case KGEOM_CAMERA_ORIENTATION:
	       func_index = 11;
	       break;
	     case KGEOM_VR_EVENT:
	       func_index = 12;
	       break;
	    default:
	       func_index = NUM_PRIMS;
	       break;
	 }
	 (*prtfuncs[func_index])(geom,prim,i,
				 do_loc,do_color,do_norm_radii,do_texture,
				 output);
      }
      else
	 kfprintf(output,"\tStrange NULL primitive at position %d\n", i);
   }
   
   return;
}

static void
print_text(
   kgeom_object    *geom,
   kgeom_primitive *prim_type,
   int pos,
   int do_loc,
   int do_color,
   int do_norm_radii,
   int do_texture,
   kfile *o)
{
   kgeom_text *prim = (kgeom_text *) prim_type;
   int nverts = prim->nverts;
   int dim = geom->location_dim;
   int i;
   
   kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_TEXT <%d>.\n",pos,prim->type);
   kfprintf(o,"\tand has %d strings\n",nverts);

   if (prim->strings == NULL)
      kfprintf(o,"\tString data is NOT present.\n");
   else
   {
      if (prim->fonts == NULL)
	 for (i=0;i<nverts;i++)
	    kfprintf(o,"String %d:\t>%s<\n",i,prim->strings[i]);
      else
	 for (i=0;i<nverts;i++)
	    kfprintf(o,"String %d:\t>%s<\tFont:\t>%s<\n",i,prim->strings[i],
		     ((prim->fonts[i] == NULL) ? "no font specified" 
	                                       : prim->fonts[i]));
   }

   kfprintf(o,"\t\tLocation data %s present.\n",
	    (prim->locs) ? "is" : "is NOT");
   kfprintf(o,"\t\tColor data %s present.\n",
	    (prim->cols) ? "is" : "is NOT");
   kfprintf(o,"\t\tBackground color data %s present.\n",
	    (prim->bgcols) ? "is" : "is NOT");
   kfprintf(o,"\t\tOffset data %s present.\n",
	    (prim->offsets) ? "is" : "is NOT");
   kfprintf(o,"\t\tUp data %s present.\n",
	    (prim->ups) ? "is" : "is NOT");
   kfprintf(o,"\t\tFlow data %s present.\n",
	    (prim->flows) ? "is" : "is NOT");
   
   if (prim->locs == NULL)
      kfprintf(o,"\tLocation data is NOT present.\n");

   if (prim->cols == NULL)
      kfprintf(o,"\tColor data is NOT present.\n");

   if (do_loc && prim->locs)
      print_vectors(prim->locs,nverts,dim,"Vertices","index\tX\tY\tZ",o);

   if (do_color && prim->cols)
   {
      if (geom->has_alpha)
	 print_vectors(prim->cols,nverts,4,"Colors","index\tR\tG\tB\a",o);
      else
	 print_vectors(prim->cols,nverts,3,"Colors","index\tR\tG\tB",o);
   }
   
   return;
}

static void
print_polyline_dis(
   kgeom_object    *geom,
   kgeom_primitive *prim_type,
   int pos,
   int do_loc,
   int do_color,
   int do_norm_radii,
   int do_texture,
   kfile *o)
{
   kgeom_polyline *prim = (kgeom_polyline *) prim_type;
   int nverts = prim->nverts;
   int dim = geom->location_dim;
   char *layout = layout_string(geom->layout);

   kfprintf(o,"\n------- PRIMITIVE %d is KPOLYLINE_DIS <%d>.\n",
	    pos,prim->type);
   kfprintf(o,"\tand has %d vertices (%d lines).\n",nverts,nverts/2);

   if (prim->locs == NULL)
      kfprintf(o,"\t\tLocation data is NOT present. Bad Primitive.\n");

   if (prim->cols == NULL)
      kfprintf(o,"\t\tColor data is NOT present.\n");
   else
      kfprintf(o,"\t\tColor data is present on a %s basis.\n", layout);

   if (do_loc && prim->locs)
      print_vectors(prim->locs,nverts,dim,"Vertices","index\tX\tY\tZ",o);
   
   if (do_color && prim->cols)
   {
      int ncolors = (geom->layout == KPER_VERTEX) ? nverts : nverts/2;
      
      if (geom->has_alpha)
	 print_vectors(prim->cols,ncolors,4,"Colors","index\tR\tG\tB\tA",o);
      else
	 print_vectors(prim->cols,ncolors,3,"Colors","index\tR\tG\tB",o);
   }
   
   return;
}

static void
print_polyline_con(
   kgeom_object    *geom, 
   kgeom_primitive *prim_type,
   int pos,
   int do_loc,
   int do_color,
   int do_norm_radii,
   int do_texture,
   kfile *o)
{
   kgeom_polyline *prim = (kgeom_polyline *) prim_type;
   int nverts = prim->nverts;
   int dim = geom->location_dim;
   char *layout = layout_string(geom->layout);

   kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_POLYLINE_CONNECTED <%d>\n",
	    pos,prim->type);
   kfprintf(o,"\tand has %d vertices (%d lines).\n",nverts,nverts-1);

   if (prim->locs == NULL)
      kfprintf(o,"\t\tLocation data is NOT present. Bad Primitive.\n");

   if (prim->cols == NULL)
      kfprintf(o,"\t\tColor data is NOT present.\n");
   else
      kfprintf(o,"\t\tColor data is present on a %s basis.\n",layout);
    
   if (do_loc && prim->locs)
      print_vectors(prim->locs,nverts,dim,"Vertices","index\tX\tY\tZ",o);
    
   if (do_color && prim->cols)
   {
      int ncolors = (geom->layout == KPER_VERTEX) ? nverts : nverts-1;
      
      if (geom->has_alpha)
	 print_vectors(prim->cols,ncolors,4,"Colors","index\tR\tG\tB\tA",o);
      else
	 print_vectors(prim->cols,ncolors,3,"Colors","index\tR\tG\tB",o);
   }

   return;
}

static void
print_spheres(
   kgeom_object    *geom,
   kgeom_primitive *prim_type,
   int pos,
   int do_loc,
   int do_color,
   int do_norm_radii,
   int do_texture,
   kfile *o)
{
   kgeom_spheres *prim = (kgeom_spheres *) prim_type;
   int nverts = prim->nverts;
   int dim = geom->location_dim;
   
   kfprintf(o,"\n------- PRIMITIVE %d is KSPHERES <%d>\n",pos,prim->type);
   kfprintf(o,"\tand has %d spheres.\n",nverts);


   if (prim->locs == NULL)
      kfprintf(o,"\t\tLocation data is NOT present. Bad Primitive.\n");

   kfprintf(o,"\t\tColor data %s present.\n",(prim->cols) ? "is" : "is NOT");
   kfprintf(o,"\t\tRadius data %s present.\n",(prim->radii) ? "is" : "is NOT");

   if (do_loc && prim->locs)
      print_vectors(prim->locs,nverts,dim,"Centers","index\tX\tY\tZ",o);

   if (do_color && prim->cols)
   {
      if (geom->has_alpha)
	 print_vectors(prim->cols,nverts,4,"Colors","index\tR\tG\tB\tA",o);
      else
	 print_vectors(prim->cols,nverts,3,"Colors","index\tR\tG\tB",o);
   }
   
   if (do_norm_radii && prim->radii)
      print_vectors(prim->radii,nverts,1,"Radii","index\tRadius",o);

   return;
}

static void
print_triangles_dis(
   kgeom_object    *geom,
   kgeom_primitive *prim_type,
   int pos,
   int do_loc,
   int do_color,
   int do_norm_radii,
   int do_texture,
   kfile *o)
{
   kgeom_triangles *prim = (kgeom_triangles *) prim_type;
   int nverts = prim->nverts;
   int dim = geom->location_dim;
   char *layout = layout_string(geom->layout);
   
   kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_TRIANGLES_DISJOINT <%d>\n",
	    pos,prim->type);
   kfprintf(o,"\tand has %d vertices (%d triangles).\n",nverts,nverts/3);

   if (prim->locs == NULL)
      kfprintf(o,"\t\tLocation data is NOT present. Bad Primitive.\n");

   if (prim->cols == NULL)
      kfprintf(o,"\t\tColor data is NOT present.\n");
   else
      kfprintf(o,"\t\tColor data is present on a %s basis.\n",layout);
   
   if (prim->norms == NULL)
      kfprintf(o,"\t\tNormal data is NOT present.\n");
   else
      kfprintf(o,"\t\tNormal data is present on a %s basis.\n",layout);
   
   kfprintf(o,"\t\tTexture coordinate data %s present.\n",
	    (prim->tcoords) ? "is" : "is NOT");

   if (prim->texture)
      kfprintf(o,"\t\tTexture transport assigned <%s>\n", prim->texture);


   if (do_loc && prim->locs != NULL)
      print_vectors(prim->locs,nverts,dim,"Vertices","index\tX\tY\tZ",o);

   if (do_norm_radii && prim->norms != NULL)
       print_vectors(prim->norms,
		     (geom->layout == KPER_VERTEX) ? nverts : nverts/3,
		     dim,"Normals","index\tX\tY\tZ",o);
   
   if (do_color && prim->cols)
   {
      int ncolors = (geom->layout == KPER_VERTEX) ? nverts : nverts/3;
      
      if (geom->has_alpha)
	 print_vectors(prim->cols,ncolors,4,"Colors","index\tR\tG\tB\tA",o);
      else
	 print_vectors(prim->cols,ncolors,3,"Colors","index\tR\tG\tB",o);
   }

   if (do_texture && prim->tcoords)
      kfprintf(o,"\tTexture Coordinates are not yet supported.\n");

   return;
}

static void
print_triangles_con(
   kgeom_object    *geom,
   kgeom_primitive *prim_type,
   int pos,
   int do_loc,
   int do_color,
   int do_norm_radii,
   int do_texture,
   kfile *o)
{ 
   kgeom_triangles *prim = (kgeom_triangles *) prim_type;
   int nverts = prim->nverts;
   int dim = geom->location_dim;
   char *layout = layout_string(geom->layout);

   kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_TRIANGLES_CONNECTED <%d>\n",
	    pos,prim->type);
   kfprintf(o,"\tand has %d vertices (%d triangles).\n",nverts,nverts-2);
   
   if (prim->locs == NULL)
      kfprintf(o,"\t\tLocation data is NOT present. Bad Primitive.\n");
   
   if (prim->cols == NULL)
      kfprintf(o,"\t\tColor data is NOT present.\n");
   else
      kfprintf(o,"\t\tColor data is present on a %s basis.\n",layout);
   
   if (prim->norms == NULL)
      kfprintf(o,"\t\tNormal data is NOT present.\n");
   else
      kfprintf(o,"\t\tNormal data is present on a %s basis.\n",layout);
   
   kfprintf(o,"\t\tTexture coordinate data %s present.\n",
	    (prim->tcoords) ? "is" : "is NOT");

   if (prim->texture)
      kfprintf(o,"\t\tTexture transport assigned <%s>\n", prim->texture);


   if (do_loc && prim->locs != NULL)
      print_vectors(prim->locs,nverts,dim,"Vertices","index\tX\tY\tZ",o);

   if (do_norm_radii && prim->norms != NULL)
       print_vectors(prim->norms,
		     (geom->layout == KPER_VERTEX) ? nverts : nverts-2,
		     dim,"Normals","index\tX\tY\tZ",o);
   
   if (do_color && prim->cols)
   {
      int ncolors = (geom->layout == KPER_VERTEX) ? nverts : nverts-2;
      
      if (geom->has_alpha)
	 print_vectors(prim->cols,ncolors,4,"Colors","index\tR\tG\tB\tA",o);
      else
	 print_vectors(prim->cols,ncolors,3,"Colors","index\tR\tG\tB",o);
   }

   if (do_texture && prim->tcoords)
      /* this assumes per-vertex layout of tcoords */
      print_vectors(prim->tcoords,nverts,geom->texture_coord_dim,
		    "Texture Coords",
	    (geom->texture_coord_dim==2)?"index\tU\tV":"index\tU\tV\tW",o);

   return;
}

static void
print_qmesh(
   kgeom_object    *geom, 
   kgeom_primitive *prim_type,
   int pos,
   int do_loc,
   int do_color,
   int do_norm_radii,
   int do_texture,
   kfile *o)
{
   kgeom_quadmesh *prim = (kgeom_quadmesh *) prim_type;
   int usize = prim->width;
   int vsize = prim->height;
   int dim   = geom->location_dim;
   char *layout = layout_string(geom->layout);
    
   kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_QUADMESH <%d>\n",
	    pos,prim->type);
   kfprintf(o,"\twhich has %d by %d vertices in U and V.\n",usize,vsize);


   if (prim->locs == NULL)
      kfprintf(o,"\t\tLocation data is NOT present. Bad Primitive.\n");
   
   if (prim->cols == NULL)
      kfprintf(o,"\t\tColor data is NOT present.\n");
   else
      kfprintf(o,"\t\tColor data is present on a %s basis.\n",layout);
   
   if (prim->norms == NULL)
      kfprintf(o,"\t\tNormal data is NOT present.\n");
   else
      kfprintf(o,"\t\tNormal data is present on a %s basis.\n",layout);
   
   kfprintf(o,"\t\tTexture coordinate data %s present.\n",
	    (prim->tcoords) ? "is" : "is NOT");

   if (prim->texture)
      kfprintf(o,"\t\tTexture transport assigned <%s>\n", prim->texture);


   if (do_loc && prim->locs != NULL)
      print_vectors(prim->locs,usize*vsize,dim,"Vertices","index\tX\tY\tZ",o);

   if (do_norm_radii && prim->norms != NULL)
   {
      int nnorms = (geom->layout == KPER_VERTEX) ? usize*vsize 
	                                         : (usize-1) * (vsize-1);
      print_vectors(prim->norms,nnorms,dim,"Normals","index\tX\tY\tZ",o);
   }
   
   if (do_color && prim->cols)
   {
      int ncolors = (geom->layout == KPER_VERTEX) ? usize*vsize 
	                                          : (usize-1) * (vsize-1);
      
      if (geom->has_alpha)
	 print_vectors(prim->cols,ncolors,4,"Colors","index\tR\tG\tB\tA",o);
      else
	 print_vectors(prim->cols,ncolors,3,"Colors","index\tR\tG\tB",o);
   }

   if (do_texture && prim->tcoords)
      /* this assumes per-vertex layout of tcoords */
      print_vectors(prim->tcoords,usize*vsize,geom->texture_coord_dim,
		    "Texture Coords",
	    (geom->texture_coord_dim==2)?"index\tU\tV":"index\tU\tV\tW",o);
   return;
}

static void
print_omesh(
   kgeom_object    *geom, 
   kgeom_primitive *prim_type,
   int pos,
   int do_loc,
   int do_color,
   int do_norm_radii,
   int do_texture,
   kfile *o)
{
   kgeom_octmesh *prim = (kgeom_octmesh *) prim_type;
   int usize = prim->width;
   int vsize = prim->height;
   int wsize = prim->depth;
   int dim   = geom->location_dim;
   char *layout = layout_string(geom->layout);
    
   kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_OCTMESH <%d>\n",
	    pos,prim->type);
   kfprintf(o,"\twhich has %d by %d by %d vertices in U, V, and W.\n",
	    usize,vsize, wsize);

   if (prim->locs == NULL)
      kfprintf(o,"\t\tLocation data is NOT present. Bad Primitive.\n");
   
   if (prim->cols == NULL)
      kfprintf(o,"\t\tColor data is NOT present.\n");
   else
      kfprintf(o,"\t\tColor data is present on a %s basis.\n",layout);
   
   if (prim->norms == NULL)
      kfprintf(o,"\t\tNormal data is NOT present.\n");
   else
      kfprintf(o,"\t\tNormal data is present on a %s basis.\n",layout);
   
   kfprintf(o,"\t\tTexture coordinate data %s present.\n",
	    (prim->tcoords) ? "is" : "is NOT");

   if (prim->texture)
      kfprintf(o,"\t\tTexture transport assigned <%s>\n", prim->texture);


   if (do_loc && prim->locs != NULL)
      print_vectors(prim->locs,usize*vsize*wsize,dim,
		    "Vertices","index\tX\tY\tZ",o);

   if (do_norm_radii && prim->norms != NULL)
   {
      int nnorms = (geom->layout == KPER_VERTEX) ? usize*vsize*wsize 
	                                      : (usize-1)*(vsize-1)*(wsize-1);
      print_vectors(prim->norms,nnorms,dim,"Normals","index\tX\tY\tZ",o);
   }
   
   if (do_color && prim->cols)
   {
      int ncolors = (geom->layout == KPER_VERTEX) ? usize*vsize*wsize 
	                                      : (usize-1)*(vsize-1)*(wsize-1);
      if (geom->has_alpha)
	 print_uvectors(prim->cols,ncolors,4,"Colors","index\tR\tG\tB\tA",o);
      else
	 print_uvectors(prim->cols,ncolors,3,"Colors","index\tR\tG\tB",o);
   }

   if (do_texture && prim->tcoords)
      /* this assumes per-vertex layout of tcoords */
      print_vectors(prim->tcoords,usize*vsize*wsize,geom->texture_coord_dim,
		    "Texture Coords",
	    (geom->texture_coord_dim==2)?"index\tU\tV":"index\tU\tV\tW",o);
   return;
}

static void
print_texture2d(
   kgeom_object    *geom, 
   kgeom_primitive *prim_type,
   int pos,
   int do_loc,
   int do_color,
   int do_norm_radii,
   int do_texture,
   kfile *o)
{
   kgeom_texture2D *prim = (kgeom_texture2D *) prim_type;
   int usize = prim->width;
   int vsize = prim->height;
    
   kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_TEXTURE2D <%d>\n",
	    pos, prim->type);
   kfprintf(o,"\twhich has %d by %d vertices in U and V.\n",usize,vsize);

   if (prim->cols == NULL)
      kfprintf(o,"\t\tColor data is NOT present.  Terrible Texture.\n");
   else
      kfprintf(o,"\t\tColor data is present.\n");

   if (do_color && prim->cols)
   {
      int ncolors = usize * vsize;
      
      if (geom->has_alpha)
	 print_uvectors(prim->cols,ncolors,4,"Colors","index\tR\tG\tB\tA",o);
      else
	 print_uvectors(prim->cols,ncolors,3,"Colors","index\tR\tG\tB",o);
   }
   
   return;
}

static void
print_unknown(
   kgeom_object    *geom, 
   kgeom_primitive *prim_type,
   int pos,
   int do_loc,
   int do_color,
   int do_norm_radii,
   int do_texture,
   kfile *o)
{
   kfprintf(o,"\tUnrecognized primitive in position #%d is of type %d.\n",
	    pos,prim_type->type);
}

static void
print_obj_xform(
   kgeom_object    *geom, 
   kgeom_primitive *prim_type,
   int pos,
   int do_loc,
   int do_color,
   int do_norm_radii,
   int do_texture,
   kfile *o)
{
   kgeom_object_xform *prim = (kgeom_object_xform *) prim_type;
   int i;

   kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_OBJECT_XFORM <%d>\n",
	    pos,prim->type);
    
   kfprintf(o,"\tObject Name: <%s>\n",prim->name);
   kfprintf(o,"\tRelative value is <%s> \n",
	    (prim->relative == KABSOLUTE) ? "KABSOLUTE" : "KRELATIVE");

   kfprintf(o,"\tTranslate Vector:\t%g\t%g\t%g\n",
	    prim->translate[0],prim->translate[1],prim->translate[2]);

   kfprintf(o,"\tMatrix:\n");
   for (i=0;i<16;i++)
   {
      kfprintf(o,"%f ",prim->matrix[i]);
      if (((i+1) % 4) == 0)
	 kfprintf(o,"\n");
   }

   return;
}

static void
print_view_xform(
   kgeom_object    *geom,
   kgeom_primitive *prim_type,
   int pos,
   int do_loc,
   int do_color,
   int do_norm_radii,
   int do_texture,
   kfile *o)
{
   kgeom_view_xform *prim = (kgeom_view_xform *) prim_type;
   int i;
   
   kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_VIEW_XFORM <%d>\n",
	    pos,prim->type);
    
   kfprintf(o,"\tMatrix:\n");
   for (i=0;i<16;i++)
   {
      kfprintf(o,"%f ",prim->matrix[i]);
      if (((i+1) % 4) == 0)
	 kfprintf(o,"\n");
   }

   return;
}

static void
print_camera_stuff(
   kgeom_object    *geom, 
   kgeom_primitive *prim_type,
   int pos,
   int do_loc,
   int do_color,
   int do_norm_radii,
   int do_texture,
   kfile *o)
{
   kgeom_camera_orientation *prim = (kgeom_camera_orientation *) prim_type;

   kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_CAMERA_ORIENTATAION <%d>\n",
	    pos,prim->type);
    
   kfprintf(o,"\tCamera Info\n");
   kfprintf(o,"\t\tEye:\t%g %g %g \n",prim->eye[0],prim->eye[1],prim->eye[2]);
   kfprintf(o,"\t\tLookAt:\t%g %g %g \n",prim->at[0],prim->at[1],prim->at[2]);
   kfprintf(o,"\t\tUp:\t%g %g %g \n",prim->up[0],prim->up[1],prim->up[2]);
   kfprintf(o,"\t\tHither/Yon\t %g %g\n",prim->hither,prim->yon);
   kfprintf(o,"\t\tFOV\t%g \n",prim->field_of_view);
   kfprintf(o,"\t\tProjection: %s \n",(prim->projection == KPROJECTION_PERSPECTIVE)?"KPROJECTION_PERSPECTIVE":"KPROJECTION_PARALLEL");

   if (prim->is_stereo)
   {
      kfprintf(o,"\tStereo Camera\n");
      kfprintf(o,"\t\tEye Distance:\t%g\n",prim->eye_distance);
      kfprintf(o,"\t\tFocal Length:\t%g\n",prim->focal_length);
   }

   return;
}

static void
print_vr_event(
   kgeom_object    *geom, 
   kgeom_primitive *prim_type,
   int pos,
   int do_loc,
   int do_color,
   int do_norm_radii,
   int do_texture,
   kfile *o)
{
   kgeom_vr_event *prim = (kgeom_vr_event *) prim_type;

   kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_VR_EVENT <%d>\n",
	    pos,prim->type);

   kfprintf(o," have_button_events?\t\t%d \n",prim->have_button_events);
   kfprintf(o," \tbuttons: %d %d %d %d %d %d %d %d %d \n",prim->buttons[0],prim->buttons[1],prim->buttons[2],prim->buttons[3],prim->buttons[4],prim->buttons[5],prim->buttons[6],prim->buttons[7],prim->buttons[8]);
   
   kfprintf(o," have_rotations?    \t\t%d \n",prim->have_rotations);
   kfprintf(o," \trotations: \n");
   kfprintf(o," \t\t%g\t%g\t%g\n",prim->rotate[0],prim->rotate[1],prim->rotate[2]);
   
   kfprintf(o," have_translations? \t\t%d \n",prim->have_translations);
   kfprintf(o," \ttranslations: \n");
   kfprintf(o," \t\t%g\t%g\t%g\n",prim->translate[0],prim->translate[1],prim->translate[2]);

   kfprintf(o," clock: %d \n",prim->clock);
   return;
}


static void
print_vectors(
   float *v,
   int    n,
   int    length,
   char  *string,
   char  *hdr_string,
   kfile *f)
{
   int i,j,offset;
    
   kfprintf(f,"\n\t%s\n",string);
   kfprintf(f,"%s\n",hdr_string);
   offset = 0;
   for (i=0;i<n;i++)
   {
      kfprintf(f,"%d",i);
      for (j=0;j<length;j++)
	 kfprintf(f,"\t%g ",v[offset++]);
      kfprintf(f,"\n");
   }
   return;
}

static void
print_uvectors(
   kubyte *v,
   int    n,
   int    length,
   char  *string,
   char  *hdr_string,
   kfile *f)
{
   int i,j,offset;
    
   kfprintf(f,"\n\t%s\n",string);
   kfprintf(f,"%s\n",hdr_string);
   offset = 0;
   for (i=0;i<n;i++)
   {
      kfprintf(f,"%d",i);
      for (j=0;j<length;j++)
	 kfprintf(f,"\t%d ",v[offset++]);
      kfprintf(f,"\n");
   }
   return;
}

static void
print_att(
   kgeom_object *geom,
   kfile *f)
{
    /**
      * this routine prints all the object-level attributes.
    **/
   int i, j;
   
   if (geom->name == NULL)
      kfprintf(f,"\tObject name is : <NULL>\n",geom->name);
   else
      kfprintf(f,"\tObject name is : <%s>\n",geom->name);
   
   if (geom->has_bounding_box)
   {
      kfprintf(f,"\tBounding box: %g,%g,%g <-> %g,%g,%g \n",
	       geom->min[0],geom->min[1],geom->min[2],
	       geom->max[0],geom->max[1],geom->max[2]);
   }
   else
      kfprintf(f,"\tThis object has NO bounding box.\n");
   
   if (geom->has_center)
      kfprintf(f,"\tCenter point: %g,%g,%g \n",
	       geom->center[0],geom->center[1],geom->center[2]);
   else
      kfprintf(f,"\tThis object has NO center point.\n");
   
   if (geom->has_matrix)
   {
      float *mx = geom->matrix;
      kfprintf(f,"\tObject Transformation Matrix:\n");
      for (j=0;j<4;j++)
      {
	 kfprintf(f,"\t");
	 for (i=0;i<4;i++)
	    kfprintf(f,"\t%g",*mx++);
	 kfprintf(f,"\n");
      }
   }
   else
      kfprintf(f,"\tThis object has NO transformation matrix.\n");
   
   
   /* deal with colors, etc.*/
   if (geom->has_color)
   {
      if (geom->has_alpha)
	 kfprintf(f,"\tObject color:\t%g %g %g %g\n",
		  geom->color[0],geom->color[1],geom->color[2], geom->opacity);
      else
	 kfprintf(f,"\tObject color:\t%g %g %g\n",
		  geom->color[0],geom->color[1],geom->color[2]);
   }
   
   kfprintf(f,"\tAmbient color:\t%g %g %g\n",
	    geom->ambient_color[0],
	    geom->ambient_color[1],
	    geom->ambient_color[2]);
   kfprintf(f,"\tDiffuse color:\t%g %g %g\n",
	    geom->diffuse_color[0],
	    geom->diffuse_color[1],
	    geom->diffuse_color[2]);
   kfprintf(f,"\tSpecular color:\t%g %g %g\n",
	    geom->specular_color[0],
	    geom->specular_color[1],
	    geom->specular_color[2]);
   kfprintf(f,"\tSpecular exponent:\t%g\n", geom->specular_exponent);
   
   
   kfprintf(f,"\tLayout is <%s>\n",layout_string(geom->layout));
   kfprintf(f,"\tModeling Space is <%s>\n",
	    (geom->modeling_space == KWORLD_SPACE) ? "KWORLD_SPACE" 
	                                           : "KNDC_SPACE");
   
   kfprintf(f,"\tLocation dimensionality is %d\n", geom->location_dim);
   kfprintf(f,"\tObject is %s\n", (geom->visible) ?"visible" :"NOT visible");
   
   return;
}

char *layout_string(int layout)
{
   switch (layout)
   {
      case KPER_VERTEX :
	 return ("KPER_VERTEX");
      case KPER_LINE :
	 return ("KPER_LINE");
      case KPER_FACE :
	 return ("KPER_FACE");
      default :
	 return ("unknown");
   }
}

