 /*
  * Khoros: $Id: grake.c,v 1.7.2.4 1997/05/31 18:55:11 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: grake.c,v 1.7.2.4 1997/05/31 18:55:11 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for grake
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "grake.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Create a computational rake.
|
|       Purpose: main program for grake
|
|         Input:
|		char *clui_info->i_file; {This data is that from which the rakes location data will be computed.}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {The sample points.}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->orake_geom_file; {Contains a geometry icon for the rake.}
|		int   clui_info->orake_geom_flag; {TRUE if -orake_geom specified}
|
|		char *clui_info->i1_file; {input file}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		int clui_info->nsamples_int; {Number of samples to compute.}
|		int clui_info->nsamples_flag; {TRUE if -nsamples specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	kobject rake_obj,data_obj;
	kgeom_object *in_xform_object;
	vertex_3d orig[2],new1[2];
	vertex_3d *p1,*p2;
	int do_new_output;
	float last_model[16],this_model[16];
	float last_trans[3],this_trans[3];
	int i;
	matrix4x4 m;
	int first_time=1;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/grake");
	kexit_handler(grake_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "grake", KGEN_KROUTINE,  &clui_uis_spec,
		   grake_usage_additions, grake_get_args,
 		   grake_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
#if 0
	for (i=0;i<16;i++)
	    last_model[i] = this_model[i] = 0.;
	last_trans[0] = last_trans[1] = last_trans[2] = 0.;
	
	/* open the source file. */
	data_obj = kpds_open_object(clui_info->i_file,KOBJ_READ);
	compute_original_endpoints(orig,data_obj);

	/* the dirty trick here is that we send the geometry
	 down the pipe only once. */
	create_geometry_icon(orig,clui_info->orake_geom_file);

#endif
	
	identity_4x4(&m);
	
	while (knegotiate_continuous_run())
	{
/* -main_before_lib_call_end */

/* -main_library_call */
  	    if (first_time || (knegotiate_check(clui_info->i_file)) &&
	        (clui_info->i_flag)) /* new velocity data */
	    {

		for (i=0;i<16;i++)
	    	last_model[i] = this_model[i] = 0.;
		last_trans[0] = last_trans[1] = last_trans[2] = 0.;
	
		/* open the source file. */
		data_obj = kpds_open_object(clui_info->i_file,KOBJ_READ);
		compute_original_endpoints(orig,data_obj);

		/* the dirty trick here is that we send the geometry
	 	down the pipe only once. */
		create_geometry_icon(orig,clui_info->orake_geom_file);
	    }
	    first_time = 0;

	    
	    do_new_output = 1;
	    
	    if ((knegotiate_check(clui_info->i1_file)) &&
		(clui_info->i1_flag)) /* new xform? */
	    {
		float *view=NULL,*model=NULL;
		float *trans=NULL;
		char *name=NULL;
		int have_new_matrix,have_new_trans;

		have_new_matrix = have_new_trans = 0;
		
		in_xform_object = kgeom_read_object(clui_info->i1_file);
		/* see if the rake_icon is present in the input
		 xform file.  if not, then we don't have to do anything */

		if (vr_search_for_named_object("rake_icon",in_xform_object) == 0)
		    do_new_output = 0;
		else
		{
		    /* see if this model xform is same as last.  skip doing
		       new output if so. */
		
		    extract_named_model_matrix(&model,in_xform_object,"rake_icon");
		    extract_named_model_translation(&trans,in_xform_object,"rake_icon");

		    kmemcpy(this_model,model,sizeof(float)*16);
		    for (i=0;i<16;i++)
			if (last_model[i] != this_model[i])
			    break;
		    if (i == 16)
			have_new_matrix = 0;
		    else
		    {
			have_new_matrix = 1;
			for (i=0;i<16;i++)
			    m.m[i/4][i%4] = this_model[i];
			kmemcpy(last_model,this_model,sizeof(float)*16);
		    }
		    for (i=0;i<3;i++)
			if (last_trans[i] != trans[i])
			    break;
		    if (i == 3)
			have_new_trans = 0;
		    else
		    {
			have_new_trans = 1;
			kmemcpy(last_trans,trans,sizeof(float)*3);
		    }
		    if (have_new_trans || have_new_matrix)
			do_new_output = 1;
		    else
			do_new_output = 0;
		}

		extract_view_matrix(&view,in_xform_object);
		kgeom_blast_object(in_xform_object);
	    }
/* -main_library_call_end */

/* -main_after_lib_call */
	    if (do_new_output == 1)
	    {	
		xform_points(orig,&m,last_trans,new1,2);
/*		create_geometry_icon(new1,clui_info->orake_geom_file); */
		create_samples(new1,clui_info->nsamples_int,clui_info->o_file);
	    }

/*	    kmemcpy(last_model,this_model,sizeof(float)*16); */
	}
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: grake_usage_additions
| 
|       Purpose: Prints usage additions in grake_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname grake
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void grake_usage_additions(void)
{
	kfprintf(kstderr, "\tCreate a computational rake.\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: grake_free_args
| 
|       Purpose: Frees CLUI struct allocated in grake_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname grake
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
grake_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->orake_geom_file);
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
