/*
 * Khoros: $Id: work.c,v 1.5.2.2 1997/05/02 15:36:52 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: work.c,v 1.5.2.2 1997/05/02 15:36:52 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>             _static_routines()
   >>>>  Public:
   >>>>             public_routines()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "grake.h"


/*-----------------------------------------------------------
|
|  Routine Name: function_name - short description of function
|
|       Purpose: This should be a complete description that anyone
|                could understand;  it should have acceptable grammar
|                and correct spelling.
|
|         Input: argument1 - explanation
|                argument2 - explanation
|                argument3 - explanation
|
|        Output: argument4 - explanation
|                argument5 - explanation
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By:
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

int
compute_original_endpoints(vertex_3d *orig,
			   kobject data_obj)
{
    vertex_3d min,max;
    float tloc;
    
    lget_minmax_points(data_obj,&min,&max);

    orig[0] = min;
    orig[1] = max;

    tloc = min.v[2] + (max.v[2]-min.v[2])*0.5;
    orig[0].v[2] = orig[1].v[2] = tloc;
    
    tloc = min.v[1] + (max.v[1]-min.v[1])*0.5;
    orig[0].v[1] = orig[1].v[1] = tloc;
    return(CHILL);
}

int
lget_minmax_points(kobject o,
		   vertex_3d *min,
		   vertex_3d *max)
{
    int i,j,k,l;
    float *plane=NULL;
    int dw,dh,dd,dt,de;
    
    for (i=0;i<3;i++)
    {
        min->v[i] = KGEOM_BIGNUM;
        max->v[i] = KGEOM_SMALLNUM;
    }

    kpds_get_attribute(o,KPDS_VALUE_SIZE,&dw,&dh,&dd,&dt,&de);
    
    /* -- assert that we want floating point location (x,y,z) data -- */
    kpds_set_attributes(o,
                        KPDS_LOCATION_DATA_TYPE, KFLOAT,
                        KPDS_LOCATION_SIZE, -1, -1, -1, 3, NULL);

    for (k=0;k<dd;k++)
    {
	for (l=0;l<3;l++)
	{
	    /* -- look for min/max points for plane k -- */
	    kpds_set_attribute(o, KPDS_LOCATION_POSITION, 0, 0, k, l);
	    plane = (float *)kpds_get_data(o, KPDS_LOCATION_PLANE, plane);
	    for (j=0;j<dh*dw;j++)
	    {
		if (plane[j] > max->v[l])
		    max->v[l] = plane[j];
		if (plane[j] < min->v[l])
		    min->v[l] = plane[j];
	    }
	}
    }
    kfree(plane);

    return(CHILL);
}

int
create_geometry_icon(vertex_3d *orig,
		     char *fname)
{
    kgeom_object *k;
    int i;
    kgeom_polyline p;

    k = kgeom_new_object();
    k->name = kstrdup("rake_icon");

    p.type = KGEOM_POLYLINE_DISJOINT;
    p.nverts = 2;
    p.locs = (float *) orig;
    p.cols = NULL;
    kgeom_add_primitive(k, (kgeom_primitive *) &p);

    k->has_bounding_box = TRUE;
    k->has_center       = TRUE;
    
    kmemcpy(k->min, (float *) &(orig->v[0]), sizeof(float)*3);
    kmemcpy(k->min, (float *) &(orig[1].v[0]), sizeof(float)*3);
    
    for (i=0;i<3;i++)
        k->center[i] = 0.5 * (orig[0].v[i] + orig[1].v[i]) ;

    kgeom_write_object(k, fname);
    kgeom_blast_object(k);
    return(CHILL);
}


int
create_samples(vertex_3d *orig,
	       int n,
	       char *fname)
{
    kobject k;
    vertex_3d *w;
    float dt;
    vertex_3d p,dp;
    int i,j;

    k = kpds_open_object(fname,KOBJ_WRITE);

    w = (vertex_3d *)kmalloc(sizeof(vertex_3d)*n);

    dt = 1./(float)(n-1);

    p = orig[0];

    for (i=0;i<3;i++)
	dp.v[i] = (orig[1].v[i] - orig[0].v[i]) * dt;

    for (i=0;i<n;i++)
    {
	w[i] = p;
	for (j=0;j<3;j++)
	    p.v[j] += dp.v[j];
    }
    

    kpds_create_location(k);
    kpds_set_attributes(k,KPDS_LOCATION_SIZE,n,1,1,3,
			KPDS_LOCATION_DATA_TYPE,KFLOAT,
			KPDS_ELEMENTS_FIRST,TRUE,NULL);

    kpds_put_data(k,KPDS_LOCATION_ALL,w);

    kpds_close_object(k);

    kfree(w);
    return(CHILL);
}

int
xform_points(vertex_3d *s,
	     matrix4x4 *m,
	     float *t,
	     vertex_3d *d,
	     int n)
{
    int i,j;
    
    for (i=0;i<n;i++)		/* loop over points */
    {
	for (j=0;j<3;j++)	/* loop over coords of output vector */
	{
	    d[i].v[j] = s[i].v[0]*m->m[0][j] +
		s[i].v[1]*m->m[1][j] + s[i].v[2]*m->m[2][j] + m->m[3][j] + t[j];
	}
    }
    return(CHILL);
}
