 /*
  * Khoros: $Id: gresample.c,v 1.1.2.3 1997/05/31 18:55:27 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gresample.c,v 1.1.2.3 1997/05/31 18:55:27 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for gresample
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "gresample.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - First-order resizing of value segment.
|
|       Purpose: main program for gresample
|
|         Input:
|		char *clui_info->i_file; {First Input data object}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Resulting output data object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		Required M.E. group:
|		    M.I. group:
|			float clui_info->w_abs_float; {The size of the width dimension will be set to this amount, unless you specify zero, which means the output will be the same size as the input.}
|			int   clui_info->w_abs_flag; {TRUE if -w_abs specified}
|
|			float clui_info->h_abs_float; {The size of the height dimension will be set to this amount, unless you specify zero, which means the output will be the same size as the input.}
|			int   clui_info->h_abs_flag; {TRUE if -h_abs specified}
|
|			float clui_info->d_abs_float; {The size of the depth dimension will be set to this amount unless you specify zero, which means the output will be the same size as the input}
|			int   clui_info->d_abs_flag; {TRUE if -d_abs specified}
|
|			float clui_info->t_abs_float; {The size of the time dimension will be set to this value (unless you specify a zero, which means the output will be the same size as the input).}
|			int   clui_info->t_abs_flag; {TRUE if -t_abs specified}
|
|		    M.I. group:
|			float clui_info->w_float; {Width dimension will be scaled by this amount.}
|			int   clui_info->w_flag; {TRUE if -w specified}
|
|			float clui_info->h_float; {Height dimension will be scaled by this amount.}
|			int   clui_info->h_flag; {TRUE if -h specified}
|
|			float clui_info->d_float; {Depth dimension will be scaled by this amount.}
|			int   clui_info->d_flag; {TRUE if -d specified}
|
|			float clui_info->t_float; {Time dimension will be scaled by this amount.}
|			int   clui_info->t_flag; {TRUE if -t specified}
|
|		int   clui_info->cmode_list; {Will choose between either a data-services processing mode (slow, but for large objects) or an in-memory-mode (fast) for smaller objects.}
|		char *clui_info->cmode_label;
|		int   clui_info->cmode_flag; {TRUE if -cmode specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	kobject in,out;
	float w,h,d,t;
	int sw,sh,sd,st,se;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/gresample");
	kexit_handler(gresample_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "gresample", KGEN_KROUTINE,  &clui_uis_spec,
		   gresample_usage_additions, gresample_get_args,
 		   gresample_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	in = kpds_open_object(clui_info->i_file,KOBJ_READ);
	kpds_get_attribute(in,KPDS_VALUE_SIZE,&sw,&sh,&sd,&st,&se);
	
	out = kpds_open_object(clui_info->o_file,KOBJ_WRITE);

	if (clui_info->w_flag)  /* doing relative scale factors, compute
				 size of the new image in pixels. */
	{
	    w = clui_info->w_float * sw;
	    h = clui_info->h_float * sh;
	    d = clui_info->d_float * sd;
	    t = clui_info->t_float * st;
	}
	else			/* doing absolute size */
	{
	    w = clui_info->w_abs_float;
	    if (w == 0.)	/* use the source size */
	        w = sw;
	    
	    h = clui_info->h_abs_float;
	    if (h == 0.)
	        h = sh;
	    
	    d = clui_info->d_abs_float;
	    if (d == 0.)
	        d = sd;
	    
	    t = clui_info->t_abs_float;
	    if (t == 0.)
	        t = st;
	}
/* -main_before_lib_call_end */

/* -main_library_call */
	lgresample_config(in,out);

	switch (clui_info->cmode_list)
	{
	case 1:
	    lgresample_value_using_dataservices(in,out,&w,&h,&d,&t);
	    break;
	case 2:
	    lgresample_value_using_memory(in,out,&w,&h,&d,&t);
	    break;
	}
/* -main_library_call_end */

/* -main_after_lib_call */
	kpds_close_object(in);
	kpds_close_object(out);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: gresample_usage_additions
| 
|       Purpose: Prints usage additions in gresample_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gresample
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void gresample_usage_additions(void)
{
	kfprintf(kstderr, "\tFirst-order resizing of value segment.\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: gresample_free_args
| 
|       Purpose: Frees CLUI struct allocated in gresample_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gresample
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
gresample_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->cmode_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
