
/*
 * Copyright (c) 1997,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 
/*
 * Khoros: $Id: work.c,v 1.1.2.1 1997/05/02 15:36:56 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: work.c,v 1.1.2.1 1997/05/02 15:36:56 mirage Exp $";
#endif


#include "gresample.h"

/*static void lerp PROTO((float *s,float *d,int sn,int dn)); */

static void grab_times PROTO((float *dst,float *src,int source_width,int source_height,int source_depth,int source_time, int source_elements,int at_width, int at_height,  int at_depth,int at_elements,int num));

static void grab_column PROTO((float *dst,float *src, int source_width,int source_height, int source_depth, int source_time, int source_elements, int at_width, int at_depth,int at_time,int at_elements,int num));

static void stuff_column PROTO((float *src,float *dst,int dest_width,int dest_height,int dest_time,int dest_depth,int dest_elements, int at_width,int at_depth,int at_time,int at_elements,int num));

static void grab_depths PROTO((float *dst,float *src,int source_width,int source_height,int source_depth,int source_time, int source_elements,int at_width,int at_height,int at_time,int at_elements,int num));

static void stuff_depths PROTO((float *src,float *dst,int dest_width,int dest_height,int dest_time,int dest_depth, int dest_elements,int at_width, int at_height, int at_time, int at_elements,int num));

static void stuff_times PROTO((float *src,float *dst,int dest_width,int dest_height, int dest_time,int dest_depth,int dest_elements,int at_width,int at_height,int at_depth,int at_elements, int num));

int
lgresample_config(kobject in,
		  kobject out)
{

    /* this is highly questionable..but will work ok for uniform
     location data types. */
    
    if (kpds_query_location(in))
	kpds_copy_location(in,out,FALSE);

    /* meaningful only if the data type is preserved in the output object.
      for now, we bash everything to KFLOATs. */
    
    if (kpds_query_map(in))
	kpds_copy_map(in,out,FALSE);

    kcolor_copy_attributes(in,out,
			   KCOLOR_COLORSPACE,
			   KCOLOR_MAP_OPERATION,
			   NULL);
    
    /* the mask segment from the input object is now invalid in the
       output object.  for now, ignore it.  later, interpolate it.*/

    kpds_create_value(out);
    kpds_copy_value_attr(in,out);
    return(CHILL);
}

int
lgresample_value_using_dataservices(kobject in,
				    kobject out,
				    float *dest_w, /* size in pixels. */
				    float *dest_h,
				    float *dest_d,
				    float *dest_t)
{
    int sw,sh,sd,st,se;		/* dimensions of source object */
    int dw,dh,dd,dt;		/* dimensions of dest object */
    int i,j,k,l,e;		/* misc loop indices */
    kobject tmp;		/* object for temporary storage */
    int tw,th,td,tt;		/* dimensions of temp object  */
    float *fs,*fd,*pstat;		/* misc work  */

    /* grab the size of the input object. */
    kpds_get_attribute(in,KPDS_VALUE_SIZE,&sw,&sh,&sd,&st,&se);

    /* set presentation attributes */
    kpds_set_attribute(out,KPDS_VALUE_DATA_TYPE,KFLOAT);
    kpds_set_attribute(in,KPDS_VALUE_DATA_TYPE,KFLOAT);

    dw = *dest_w;
    dh = *dest_h;
    dd = *dest_d;
    dt = *dest_t;

    kpds_set_attribute(out,KPDS_VALUE_SIZE,dw,dh,dd,dt,se);

    /* configure a temporary object for holding intermediate results. */

    tmp = kpds_create_object();
    tw = MAX(dw,sw);
    th = MAX(dh,sh);
    td = MAX(dd,sd);
    tt = MAX(dt,st);
    
    kpds_create_value(tmp);
    kpds_set_attributes(tmp,KPDS_VALUE_SIZE,tw,th,td,tt,se,
			KPDS_VALUE_DATA_TYPE,KFLOAT,NULL);
    


    /* first set of linear ops is over the width dimension. we do all
     rows, which means we loop over time,depth & height.  this set of
     ops is executed even if w==1 because we have to get data moved
     into the tmp object for further processing. */

    fs = (float *)kmalloc(sizeof(float)*tw);
    fd = (float *)kmalloc(sizeof(float)*tw);
    pstat = NULL;
    for (e=0;e<se;e++)		/* loop over source #elements */
    {
	for (k=0;k<st;k++)	/* loop over source time */
	{
	    for (l=0;l<sd;l++)	/* loop over source depth */
	    {
		for (j=0;j<sh;j++) /* loop over source HEIGHT */
		{
		    /* set the VALUE position pointer on the input object. */
		    kpds_set_attribute(in,KPDS_VALUE_POSITION,0,j,l,k,e);
		    /* grab a row's worth of data */
		    pstat = (float *)kpds_get_data(in,KPDS_VALUE_LINE,fs);

		    /* lerp the data */
		    kmemset(fd,0,sizeof(float)*tw); /* temporary */
	/*	    kmemcpy(fd,fs,sizeof(float)*sw); */
		    lerp(fs,fd,sw,dw);

		    /* stuff the data into the temp object */
		    kpds_set_attribute(tmp,KPDS_VALUE_POSITION,0,j,l,k,e);
		    kpds_put_data(tmp,KPDS_VALUE_LINE,fd);
		}
	    }
	}
    }
    kfree(fs);
    kfree(fd);

    /* next comes height.  if h == 1, we can just skip this step.
       we'll loop over (source) time, (source) elements and dest width. */
    
    fs = (float *)kmalloc(sizeof(float)*th);
    fd = (float *)kmalloc(sizeof(float)*th);
    
    if (dh != sh)
    {
	for (e=0;e<se;e++)		/* loop over source #elements */
	{
	    for (k=0;k<st;k++)	/* loop over source time */
	    {
		for (l=0;l<sd;l++)	/* loop over source depth */
		{
		    for (j=0;j<dw;j++) /* loop over dest width */
		    {
			/* set the VALUE position pointer on the tmp object. */

			kpds_set_attribute(tmp,KPDS_VALUE_REGION_SIZE,1,sh,1,1,1);
			kpds_set_attribute(tmp,KPDS_VALUE_POSITION,j,0,l,k,e);

			/* grab a row's worth of data */
			pstat = (float *)kpds_get_data(tmp,KPDS_VALUE_REGION,fs);

			/* lerp the data */
			kmemset(fd,0,sizeof(float)*th); /* temporary */
			lerp(fs,fd,sh,dh); 

			/* stuff the data into the temp object */
			kpds_set_attribute(tmp,KPDS_VALUE_REGION_SIZE,1,dh,1,1,1);
			kpds_set_attribute(tmp,KPDS_VALUE_POSITION,j,0,l,k,e);
			kpds_put_data(tmp,KPDS_VALUE_REGION,fd);
		    }
		}
	    }
	}
    }
    
    kfree(fs);
    kfree(fd);
    
    /* next comes depth.  if dh == sh, we can just skip this step.
       we'll loop over (source) time, (source) elements, width & height.
       we lerp over depth. */
    
    fs = (float *)kmalloc(sizeof(float)*sd);
    fd = (float *)kmalloc(sizeof(float)*td);
    
    if (dd != sd)
    {
	fprintf(stderr," this code has not been tested - lerp over depth \n");
	
	for (e=0;e<se;e++)		/* loop over source #elements */
	{
	    for (k=0;k<st;k++)	/* loop over source time */
	    {
		for (l=0;l<dh;l++)	/* loop over dest height */
		{
		    for (j=0;j<dw;j++) /* loop over dest width */
		    {
			/* set the VALUE position pointer on the tmp object. */

			kpds_set_attribute(tmp,KPDS_VALUE_REGION_SIZE,1,1,sd,1,1);
			kpds_set_attribute(tmp,KPDS_VALUE_POSITION,j,l,0,k,e);

			/* grab a row's worth of data */
			pstat = (float *)kpds_get_data(tmp,KPDS_VALUE_REGION,fs);

			/* lerp the data */
			kmemset(fd,0,sizeof(float)*td); /* temporary */
			lerp(fs,fd,sd,dd); 

			/* stuff the data into the temp object */
			kpds_set_attribute(tmp,KPDS_VALUE_REGION_SIZE,1,1,dd,1,1);
			kpds_set_attribute(tmp,KPDS_VALUE_POSITION,j,l,0,k,e);
			kpds_put_data(tmp,KPDS_VALUE_REGION,fd);
		    }
		}
	    }
	}
    }
    
    

    kfree(fs);
    kfree(fd);
    
    /* finally comes time.  if dt == st, we can just skip this step.
       we'll loop over (source) elements, and dest width,dest height &
       dest depth.  lerping is over time. */
    
    fs = (float *)kmalloc(sizeof(float)*tt);
    fd = (float *)kmalloc(sizeof(float)*tt);
    
    if (dt != st)
    {
	fprintf(stderr," this code has not been tested - lerp over time \n");
	
	for (e=0;e<se;e++)		/* loop over source #elements */
	{
	    for (k=0;k<st;k++)	/* loop over dest depth */
	    {
		for (l=0;l<dh;l++)	/* loop over dest height */
		{
		    for (j=0;j<dw;j++) /* loop over dest width */
		    {
			/* set the VALUE position pointer on the tmp object. */

			kpds_set_attribute(tmp,KPDS_VALUE_REGION_SIZE,1,1,1,st,1);
			kpds_set_attribute(tmp,KPDS_VALUE_POSITION,j,l,k,0,e);

			/* grab a row's worth of data */
			pstat = (float *)kpds_get_data(tmp,KPDS_VALUE_REGION,fs);

			/* lerp the data */
			kmemset(fd,0,sizeof(float)*tt); /* temporary */
			lerp(fs,fd,st,dt); 

			/* stuff the data into the temp object */
			kpds_set_attribute(tmp,KPDS_VALUE_REGION_SIZE,1,1,1,dt,1);
			kpds_set_attribute(tmp,KPDS_VALUE_POSITION,j,l,k,0,e);
			kpds_put_data(tmp,KPDS_VALUE_REGION,fd);
		    }
		}
	    }
	}
    }


    /* after all the lerping is done, copy the value data from
       the tmp object over to the dest object. */

    kpds_set_attributes(tmp,KPDS_VALUE_POSITION,0,0,0,0,0,
			KPDS_VALUE_REGION_SIZE,dw,dh,dd,dt,se,NULL);
    kpds_set_attribute(out,KPDS_VALUE_POSITION,0,0,0,0,0);
    kpds_copy_value_data(tmp,out,FALSE);
    kpds_set_attributes(out,KPDS_VALUE_SIZE,dw,dh,dd,dt,se,NULL);

    
    /* adios */
    return(CHILL);
}

int
lgresample_value_using_memory(kobject in,
			      kobject out,
			      float *dest_w, /* size in pixels */
			      float *dest_h,
			      float *dest_d,
			      float *dest_t)
{
    int sw,sh,sd,st,se;		/* dimensions of source object */
    int dw,dh,dd,dt;		/* dimensions of dest object */
    float *fs,*fd,*pstat;		/* misc work  */
    float *core,*src;
    float *tmp;
    int save_in_format;
    int size_in[5];
    float size_out[5];

    /* grab dimensions of input object. */
    kpds_get_attribute(in,KPDS_VALUE_SIZE,&sw,&sh,&sd,&st,&se);
    size_in[0] = sw;
    size_in[1] = sh;
    size_in[2] = sd;
    size_in[3] = st;
    size_in[4] = se;

    /* malloc holding area for input data. */
    src = (float *)kmalloc(sizeof(float)*sw*sh*sd*st*se); /* ouch! */

    /* presentation attributes of input*/
    kpds_get_attribute(in,KPDS_VALUE_DATA_TYPE,&save_in_format);
    kpds_set_attribute(in,KPDS_VALUE_DATA_TYPE,KFLOAT);
    kpds_set_attribute(in,KPDS_VALUE_POSITION,0,0,0,0,0);
    
    /* grab all the input data. */
    pstat = (float *)kpds_get_data(in,KPDS_VALUE_ALL,src);

    /* how big is the output object? */
    size_out[0] = dw = *dest_w;
    size_out[1] = dh = *dest_h;
    size_out[2] = dd = *dest_d;
    size_out[3] = dt = *dest_t;
    size_out[4] = size_in[4];

    /* grab space to hold computed output */
    tmp = (float *)kmalloc(sizeof(float)*dw*dh*dd*dt*se); /* ouch! */

    /* do the interpolation */
    lresample_memory(src,tmp,size_in, size_out,FALSE) ;

    /* set up the output object */
    kpds_set_attribute(out,KPDS_VALUE_DATA_TYPE,KFLOAT);
    kpds_set_attribute(out,KPDS_VALUE_SIZE,dw,dh,dd,dt,se);
    kpds_set_attribute(out,KPDS_VALUE_POSITION,0,0,0,0,0);

    /* and stuff the data */
    kpds_put_data(out,KPDS_VALUE_ALL,tmp);
    
    /* restore format so output data matches input data */
    kpds_set_attribute(out,KPDS_VALUE_DATA_TYPE,save_in_format);

    /* free up memory */
    kfree(tmp);
    
    /* adios */
    return(CHILL);
}

#if 0

static void
lerp(float *s,			/* source pixels */
     float *d,			/* dest pixels */
     int sn,			/* source size */
     int dn)			/* dest size */
{
    register int i,index,indexp1;
    register double t,dt;
    register double frac1,frac2;
    register double floor_t;
    
    /* we assume that *d has all been initialized to 0.0 */
    if (sn == dn)
	kmemcpy(d,s,sizeof(float)*sn);
    else if (dn > sn)
    {
	dt = (float)sn/(float)dn;
	t = 0;
	for (i=0;i<dn;i++,t+=dt)
	{
	    floor_t = kfloor(t);
	    
	    index = (int)floor_t;
	    indexp1 = (index+1 >= sn) ? index : index+1;
	    
	    frac2 = t - floor_t; /* portion of next pixel */
	    frac1 = 1. - frac2; /* portion of this pixel */
	    d[i] = frac1 * s[index] + frac2 * s[indexp1];
	}
    }
    else			/* sn > dn */
    {
	int dindex;
	
	t = 0.;
	dt = (double)dn/(double)sn;
	for (i=0;i<sn;i++,t+=dt)
	{
	    index = kfloor(t);
	    dindex = kfloor(t+dt);
	    if (dindex == index) /* the source pixel fits entirely within
				  the dest pixel.*/
		d[dindex] += s[i]*dt;
	    else		/* the source pixel covers parts of two
				 dest pixels */
	    {
		frac1 = 1. - (t - kfloor(t));
		frac2 = (t+dt) - kfloor(t+dt);
		d[index] += frac1 * s[i];
		d[index+1] += frac2 * s[i];
	    }
	}
    }
}

#endif

static void
grab_column(float *dst,		/* a linear array of floats */
	    float *src,		/* a big 5d array of floats */
	    int source_width,	/* width size of src */
	    int source_height,	/* height size of src */
	    int source_depth,	/* depth size of src */
	    int source_time,	/* time size of src */
	    int source_elements, /* elements size of src */
	    int at_width,	/* where to grab a column from */
	    int at_depth,	/* at what depth */
	    int at_time,	/* at what time */
	    int at_elements,	/* at what elements */
	    int num)		/* how big is column? */
{
    int i;
    float *s;
    int wh_size,whd_size,whdt_size;

    wh_size = source_width * source_height;
    whd_size = wh_size * source_depth;
    whdt_size = whd_size * source_time;

    s = src + at_width + at_depth * wh_size + at_time * whd_size + at_elements * whdt_size;

    for (i=0;i<num;i++,s += source_width)
	*dst++ = *s;
}

static void
stuff_column(float *src,
	     float *dst,
	     int dest_width,
	     int dest_height,
	     int dest_time,
	     int dest_depth,
	     int dest_elements,
	     int at_width,
	     int at_depth,
	     int at_time,
	     int at_elements,
	     int num)
{
    int i;
    float *d;
    int wh_size,whd_size,whdt_size;

    wh_size = dest_width * dest_height;
    whd_size = wh_size * dest_depth;
    whdt_size = whd_size * dest_time;

    d = dst + at_width + at_depth * wh_size + at_time * whd_size + at_elements * whdt_size;
    for (i=0;i<num;i++,d += dest_width)
	*d = *src++;
}


static void
grab_depths(float *dst,		/* a linear array of floats */
	    float *src,		/* a big 5d array of floats */
	    int source_width,	/* width size of src */
	    int source_height,	/* height size of src */
	    int source_depth,	/* depth size of src */
	    int source_time,	/* time size of src */
	    int source_elements, /* elements size of src */
	    int at_width,	/* where to grab a column from */
	    int at_height,	/* at what height */
	    int at_time,	/* at what time */
	    int at_elements,	/* at what elements */
	    int num)		/* how many depths to grab? */
{
    int i;
    float *s;
    int wh_size,whd_size,whdt_size;

    wh_size = source_width * source_height;
    whd_size = wh_size * source_depth;
    whdt_size = whd_size * source_time;

    s = src + at_width + at_height*source_width + at_time * whd_size + at_elements * whdt_size;

    for (i=0;i<num;i++,s += wh_size)
	*dst++ = *s;
}

static void
stuff_depths(float *src,
	     float *dst,
	     int dest_width,
	     int dest_height,
	     int dest_time,
	     int dest_depth,
	     int dest_elements,
	     int at_width,
	     int at_height,
	     int at_time,
	     int at_elements,
	     int num)
{
    int i;
    float *d;
    int wh_size,whd_size,whdt_size;

    wh_size = dest_width * dest_height;
    whd_size = wh_size * dest_depth;
    whdt_size = whd_size * dest_time;

    d = dst + at_width + at_height * dest_width + at_time * whd_size + at_elements * whdt_size;
    for (i=0;i<num;i++,d += wh_size)
	*d = *src++;
}


static void
grab_times(float *dst,		/* a linear array of floats */
	   float *src,		/* a big 5d array of floats */
	   int source_width,	/* width size of src */
	   int source_height,	/* height size of src */
	   int source_depth,	/* depth size of src */
	   int source_time,	/* time size of src */
	   int source_elements, /* elements size of src */
	   int at_width,	/* where to grab a column from */
	   int at_height,	/* at what height */
	   int at_depth,	/* at what depth */
	   int at_elements,	/* at what elements */
	   int num)		/* how many depths to grab? */
{
    int i;
    float *s;
    int wh_size,whd_size,whdt_size;

    wh_size = source_width * source_height;
    whd_size = wh_size * source_depth;
    whdt_size = whd_size * source_time;

    s = src + at_width + at_height*source_width + at_depth * wh_size + at_elements * whdt_size;

    for (i=0;i<num;i++,s += whd_size)
	*dst++ = *s;
}

static void
stuff_times(float *src,
	    float *dst,
	    int dest_width,
	    int dest_height,
	    int dest_time,
	    int dest_depth,
	    int dest_elements,
	    int at_width,
	    int at_height,
	    int at_depth,
	    int at_elements,
	    int num)
{
    int i;
    float *d;
    int wh_size,whd_size,whdt_size;

    wh_size = dest_width * dest_height;
    whd_size = wh_size * dest_depth;
    whdt_size = whd_size * dest_time;

    d = dst + at_width + at_height * dest_width + at_depth * wh_size + at_elements * whdt_size;
    for (i=0;i<num;i++,d += whd_size)
	*d = *src++;
}

