 /*
  * Khoros: $Id: gspheres.c,v 1.11.2.2 1997/05/31 18:55:46 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gspheres.c,v 1.11.2.2 1997/05/31 18:55:46 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for gspheres
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "gspheres.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Produce Spheres Whose Radii are Determined from Data
|
|       Purpose: main program for gspheres
|
|         Input:
|		char *clui_info->i_file; {input data object }
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output geometry object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->cmap_file; {input vis colormap}
|		int   clui_info->cmap_flag; {TRUE if -cmap specified}
|
|		char *clui_info->name_string; {object name}
|		int   clui_info->name_flag; {TRUE if -name specified}
|
|		float clui_info->scale_float; {scale factor for radii}
|		int   clui_info->scale_flag; {TRUE if -scale specified}
|
|        Output:
|       Returns:
|
|    Written By: wes
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
    kobject datafile,mapfile;
    kgeom_object *outfile;
    int have_mapfile;
    float radius_scale;
    int w,h,d,dims,e,t;
    char *name;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/gspheres");
	kexit_handler(gspheres_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "gspheres", KGEN_KROUTINE,  &clui_uis_spec,
		   gspheres_usage_additions, gspheres_get_args,
 		   gspheres_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    datafile = kpds_open_object(clui_info->i_file,KOBJ_READ);
    outfile = kgeom_new_object();

    if (clui_info->cmap_flag)
    {
	mapfile = kpds_open_object(clui_info->cmap_file,KOBJ_READ);
	have_mapfile = TRUE;
    }
    else
	have_mapfile = FALSE;
    radius_scale = clui_info->scale_float;
    name = clui_info->name_string;
/* -main_before_lib_call_end */

/* -main_library_call */
    lgspheres(datafile,mapfile,outfile,have_mapfile,&radius_scale,name);
/* -main_library_call_end */

/* -main_after_lib_call */
    kpds_close_object(datafile);
    if (have_mapfile)
	kpds_close_object(mapfile);
    kgeom_write_object(outfile, clui_info->o_file);
    kgeom_blast_object(outfile);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: gspheres_usage_additions
| 
|       Purpose: Prints usage additions in gspheres_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gspheres
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void gspheres_usage_additions(void)
{
	kfprintf(kstderr, "\tProduce Spheres Whose Radii are Determined from Data\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: gspheres_free_args
| 
|       Purpose: Frees CLUI struct allocated in gspheres_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gspheres
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
gspheres_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->cmap_file);
		kfree_and_NULL(clui_info->name_string);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
