 /*
  * Khoros: $Id: gstreamlines.c,v 1.11.2.2 1997/05/31 18:56:05 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gstreamlines.c,v 1.11.2.2 1997/05/31 18:56:05 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for gstreamlines
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "gstreamlines.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Compute Streamlines through 3-Component Vector/Flow Data.
|
|       Purpose: main program for gstreamlines
|
|         Input:
|		char *clui_info->i_file; {Data object containing flow vectors.  Must have 3 computational dimensions and 3 coordinate dimensions.}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->rake_file; {List of x,y,z points used for seeding the streamlines computation.}
|		int   clui_info->rake_flag; {TRUE if -rake specified}
|
|		char *clui_info->o_file; {Geometry representing streamlines.}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->name_string; {Internal name of geometry object.}
|		int   clui_info->name_flag; {TRUE if -name specified}
|
|		float clui_info->step_float; {Controls granularity of integration.  Make smaller for better approximation.}
|		int   clui_info->step_flag; {TRUE if -step specified}
|
|		int clui_info->nsteps_int; {Number of integration steps to compute.}
|		int clui_info->nsteps_flag; {TRUE if -nsteps specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
    streamlines_struct sstruct;
    int status;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/gstreamlines");
	kexit_handler(gstreamlines_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "gstreamlines", KGEN_KROUTINE,  &clui_uis_spec,
		   gstreamlines_usage_additions, gstreamlines_get_args,
 		   gstreamlines_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	sstruct.vectordata = kpds_open_input_object(clui_info->i_file);
	status = sanity_check_vectordata(&sstruct);
	if (status == WHACKED)
	{
	    kerror("GEOMETRY","gstreamlines","There vector data is ill configured - this routine wants w,h,d > 1, t==1, e==3, Location-dims==3.  You gave me something different.. Aborting.");
	    exit(KEXIT_FAILURE);
	}
	init_spacetree(&sstruct);
	kpds_close_object(sstruct.vectordata);

	while (knegotiate_continuous_run())
	{
	    /* we're expecting the rake file to change. */
	    if (knegotiate_check(clui_info->rake_file) == FALSE)
		continue;
	    
	    sstruct.rakeobj = kpds_open_input_object(clui_info->rake_file);

	    /* do sanity checks on the input objects */
	    status = sanity_check_rake(&sstruct);
	    if (status == WHACKED)
	    {
		kerror("GEOMETRY","gstreamlines","There rake data is ill configured - this routine wants Location-dims==3.  You gave me something different.. Aborting.");
		exit(KEXIT_FAILURE);
	    }

	    sstruct.step_size = clui_info->step_float;
	    sstruct.num_steps = clui_info->nsteps_int;
	    sstruct.object_name = clui_info->name_string;

	    sstruct.streams_geom = kgeom_new_object();
/* -main_before_lib_call_end */

/* -main_library_call */
	    lgstreamlines(&sstruct);
/* -main_library_call_end */

/* -main_after_lib_call */
	    kpds_close_object(sstruct.rakeobj);
	    kgeom_write_object(sstruct.streams_geom, clui_info->o_file);
	    kgeom_blast_object(sstruct.streams_geom);
	} /* end continuous run. */
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: gstreamlines_usage_additions
| 
|       Purpose: Prints usage additions in gstreamlines_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gstreamlines
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void gstreamlines_usage_additions(void)
{
	kfprintf(kstderr, "\tCompute Streamlines through 3-Component Vector/Flow Data.\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: gstreamlines_free_args
| 
|       Purpose: Frees CLUI struct allocated in gstreamlines_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gstreamlines
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
gstreamlines_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->rake_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->name_string);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
