 /*
  * Khoros: $Id: gstreamlines.h,v 1.9.2.3 1997/05/31 18:56:09 mirage Exp $
  */

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>>       Purpose: Include file for gstreamlines
   >>>> 
   >>>>    Written By: 
   >>>> 
   >>>> Modifications: 
   >>>> 
   >>>>          Date: July 1, 1997
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#ifndef _gstreamlines_h_
#define _gstreamlines_h_

        /*-------------------------------------*
         |       #includes
        ---------------------------------------*/

#include <geometry.h>

/* -include_includes */
#include "otree.h"
/* -include_includes_end */


        /*-------------------------------------*
         |       #defines
        ---------------------------------------*/

/* -include_defines */
/* -include_defines_end */

        /*-------------------------------------*
         |       typedefs
        ---------------------------------------*/

typedef struct _clui_info_struct {

/*
 *  Data object containing flow vectors.  Must have 3 computational dimensions and 3 coordinate dimensions. (required infile)
 */
char	*i_file;	/* Data object containing flow vectors.  Must have 3 computational dimensions and 3 coordinate dimensions. FILENAME */
int	 i_flag;	/* Data object containing flow vectors.  Must have 3 computational dimensions and 3 coordinate dimensions. FLAG */

/*
 *  List of x,y,z points used for seeding the streamlines computation. (required infile)
 */
char	*rake_file;	/* List of x,y,z points used for seeding the streamlines computation. FILENAME */
int	 rake_flag;	/* List of x,y,z points used for seeding the streamlines computation. FLAG */

/*
 *  Geometry representing streamlines. (required outfile)
 */
char	*o_file;	/* Geometry representing streamlines. FILENAME */
int	 o_flag;	/* Geometry representing streamlines. FLAG */

/*
 *  Internal name of geometry object. (optional string)
 */
char	*name_string;	/* Internal name of geometry object. STRING */
int	 name_flag;   	/* Internal name of geometry object. FLAG */

/*
 *  Controls granularity of integration.  Make smaller for better approximation. (optional float)
 */
float	 step_float;	/* Controls granularity of integration.  Make smaller for better approximation. FLOAT */
int	 step_flag;   	/* Controls granularity of integration.  Make smaller for better approximation. FLAG */

/*
 *  Number of integration steps to compute. (optional integer)
 */
int	 nsteps_int; 	/* Number of integration steps to compute. INT */
int	 nsteps_flag;	/* Number of integration steps to compute. FLAG */

} clui_info_struct;

/* -include_typedefs */
/* the following structure is used to package up parms for computing
 the streamlines.*/
typedef struct streamlines_struct
{
    kobject vectordata;		/* source vector data */
    kobject rakeobj;		/* seed points for computation */
    kgeom_object *streams_geom;	/* output geometry object */
    int ijkdims[3];	/* size of computational space for vectordata */
				/* note - we may want to add a fourth
				 computational dimension for unsteady flows
				 at a later point in time. */

    int num_streams;		/* number of streamlines to be computed */
    
    float step_size;		/* size of integration step */
    int num_steps;		/* number of integration steps */
    char *object_name;
} streamlines_struct;

typedef struct stream
{
    vertex seed;
    int npoints;
    int grid_start[3];          /* i,j,k grid block at beginning */
    int grid_current[3];        /* i,j,k grid block of last point */
    vertex *path;               /* list of points in streamline */
    float *time;                /* time-stamp values at each point in path[]*/
} stream;
/* -include_typedefs_end */


        /*-------------------------------------*
         |       global variable declarations
        ---------------------------------------*/

extern clui_info_struct *clui_info;
extern uis_struct clui_uis_spec;
/* -include_variables */
/* -include_variables_end */


        /*-------------------------------------*
         |       macros
        ---------------------------------------*/

/* -include_macros */
/* -include_macros_end */

        /*-------------------------------------*
         |       routine definitions
        ---------------------------------------*/

int main PROTO((int, char **));
void gstreamlines_get_args PROTO((kaddr));
void gstreamlines_usage_additions PROTO((void));
void gstreamlines_free_args PROTO((kexit_status, kaddr));

/* -include_routines */
int lgstreamlines PROTO((streamlines_struct *)); /* temporary */

void init_spacetree PROTO((streamlines_struct *s));
int sanity_check_vectordata PROTO((streamlines_struct *));
int sanity_check_rake PROTO((streamlines_struct *));
int lgstreamlines PROTO((streamlines_struct *));
int malloc_river PROTO((stream **, int));
int free_river PROTO((stream *, int));
int do_streamline PROTO((stream *, streamlines_struct *, float *, float **));


int output_streamlines_geometry PROTO((stream *, streamlines_struct *));
int init_output_geom_stuff PROTO((streamlines_struct *));

int init_otree PROTO((streamlines_struct *, qtree *, float **));
int find_grid_ijk PROTO((vertex *, int *, qtree *));
int print_qtree PROTO((qtree *, FILE *, int));
int qtree_find_point PROTO((vertex *, qtree *));
int is_contained PROTO((vertex *, vertex *, vertex *));
int add_to_hit_stack PROTO((qtree *));
int look_at_columns PROTO((vertex *, float *, float *, float *, int *));
/* -include_routines_end */

#endif
