
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 
/*
 * Khoros: $Id: streams.c,v 1.5.2.2 1997/05/02 15:37:10 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: streams.c,v 1.5.2.2 1997/05/02 15:37:10 mirage Exp $";
#endif

#include "gstreamlines.h"

#include "otree.h"


static stream *river=NULL;
static qtree tree;
static int last_nstreams=0;	/* in anticipation of continuous-run.. */
static int num_prims=1;

static float *vdata_coords[3];

static int init_streams PROTO((streamlines_struct *s,qtree *tree,stream *river,float **vdata_coords));

static int init_vdata PROTO((streamlines_struct *s));
static int init_stream_struct PROTO((stream **, streamlines_struct *, int));
static void compute_streamlines PROTO((stream *, streamlines_struct *, float **));

static void compute_corner_weights PROTO((vertex *p,int *ijk,double *w,float *xp, float *yp,float *zp,int isize,int ijsize));
static void load_vector_data PROTO((vertex *, int *, double *, float *, int, int ));

int
sanity_check_vectordata(streamlines_struct *ss)
{
    int status=CHILL;
    int w,h,d,t,e;
    kobject s;

    s = ss->vectordata;
    kpds_get_attribute(s,KPDS_VALUE_SIZE,&w,&h,&d,&t,&e);
    if ((e != 3) || (t != 1) ||
	!((w > 1) && (h > 1) && (d > 1)))
    {
	status = WHACKED;
    }
    ss->ijkdims[0] = w;
    ss->ijkdims[1] = h;
    ss->ijkdims[2] = d;


    kpds_set_attribute(s,KPDS_VALUE_DATA_TYPE,KFLOAT);
    kpds_set_attribute(s,KPDS_ELEMENTS_FIRST,TRUE);
    
    if (kpds_query_location(s))
    {
	kpds_get_attribute(s,KPDS_LOCATION_SIZE,NULL,NULL,NULL,&d);
	if (d != 3)
	    status = WHACKED;
    }
    else
    {
	kpds_set_attribute(s, KPDS_LOCATION_GRID, KUNIFORM);
	kpds_create_location(s);
	kpds_set_attributes(s,
			    KPDS_LOCATION_BEGIN, 0.,0.,0.,
			    KPDS_LOCATION_END,(double)w,
			    (double)h, (double)d, NULL);
       
    }
    return(status);
}

int
sanity_check_rake(streamlines_struct *ss)
{
    int status=CHILL;
    int w,h,d,dim;
    kobject s;

    s = ss->rakeobj;
    if (kpds_query_location(s))
    {
	kpds_get_attribute(s,KPDS_LOCATION_SIZE,&w,&h,&d,&dim);
	if (dim != 3)
	    status = WHACKED;

	ss->num_streams = w*h*d;
    }
    else
	status = WHACKED;

    /* set misc presentation attributes */

    kpds_set_attribute(s,KPDS_LOCATION_DATA_TYPE,KFLOAT);

    return(status);
}

void
init_spacetree(streamlines_struct *s){

    init_otree(s,&tree,vdata_coords);
    init_vdata(s);
}


int
lgstreamlines(streamlines_struct *s)
{

    init_stream_struct(&river,s,last_nstreams);

    /* build the space subdividing tree.  when continuous run happens,
     we'll need to call this routine ONLY when the topology of the
     grid with vector data changes...not when the rake object changes. */
    
/*    init_otree(s,&tree,vdata_coords); */

    init_streams(s,&tree,river,vdata_coords);

    compute_streamlines(river,s,vdata_coords);

    init_output_geom_stuff(s);
    output_streamlines_geometry(river,s);

    last_nstreams = s->num_streams; /* in anticipation of continuous run */

#if 0
    kfree(vdata_coords[0]);	/* won't want to do this in continuous run
				 except for when we get a new grid of vector
				 data to work on. */
#endif
    return(CHILL);
}


int
init_stream_struct(stream **river,
		   streamlines_struct *ss,
		   int last_nstreams)
{
    stream *t;
    float *xpts=NULL,*ypts=NULL,*zpts=NULL;
    int npoints,i;
    kobject rake;

    /* assume that *nstreams has current # of streams, and that
     this number is valid. */

    if (*river != NULL)
	free_river(*river,last_nstreams);

    npoints = ss->num_streams;
    malloc_river(river,npoints);

    t = *river;

    xpts = (float *)kmalloc(sizeof(float)*npoints*3);
    ypts = xpts + npoints;
    zpts = ypts + npoints;

    rake = ss->rakeobj;
    
    kpds_set_attribute(rake,KPDS_LOCATION_POSITION,0,0,0,0);
    kpds_get_data(rake,KPDS_LOCATION_VOLUME,xpts);

    kpds_set_attribute(rake,KPDS_LOCATION_POSITION,0,0,0,1);
    kpds_get_data(rake,KPDS_LOCATION_VOLUME,ypts);
    
    kpds_set_attribute(rake,KPDS_LOCATION_POSITION,0,0,0,2);
    kpds_get_data(rake,KPDS_LOCATION_VOLUME,zpts);
    
    for (i=0;i<npoints;i++)
    {
	t[i].npoints = 0;
	t[i].seed.x = xpts[i];
	t[i].seed.y = ypts[i];
	t[i].seed.z = zpts[i];
    }

    kfree(xpts);
    return(CHILL);
}

int
malloc_river(stream **r,int n)
{
    int i;
    
    *r = (stream *)kmalloc(sizeof(stream)*n);

    for (i=0;i<n;i++)
    {
	(*r)[i].path = (vertex *)kmalloc(sizeof(vertex)*MAXPATHLENGTH);
	(*r)[i].time = (float *)kmalloc(sizeof(float)*MAXPATHLENGTH);

    }
    return(CHILL);
}
int
free_river(stream *r,int n)
{
    int i;
    for (i=0;i<n;i++)
    {
	kfree(r[i].path);
	kfree(r[i].time);
    }
    kfree(r);
    return(CHILL);
}

static int
init_streams(streamlines_struct *s,
	     qtree *tree,
	     stream *river,
	     float **vdata_coords)
{
    int i;
    
    /* first, find the index of the grid block containing each of
     the seed points for the streamline */

    for (i=0;i<s->num_streams;i++)
    {
	find_grid_ijk(&(river->seed),(river->grid_start),tree);
	river++;
    }
    return(CHILL);
}

static float *source_vdata=NULL;

static int
init_vdata(streamlines_struct *s)
{
    /* in this routine, we read all of the velocity data. */
    kobject t;
    /* note: in this ve5rsion, we read in ALL of the vector data
     at once.  an experiment might be to use data services to grab
     points of data only whne they're needed.  the percentage of
     data that we actually look at is small. */

    t = s->vectordata;

    kpds_set_attribute(t,KPDS_ELEMENTS_FIRST,TRUE);
    
    
    source_vdata = NULL;
    source_vdata = (float *)kpds_get_data(t,KPDS_VALUE_ALL,source_vdata);

    if (source_vdata == NULL)
    {
	kerror("GEOMETRY","gstreamlines","can't malloc enough space to hold all of the vector data.");
	return(WHACKED);
    }

}

static void
compute_streamlines(stream *river,
		    streamlines_struct *s,
		    float **vdata_coords)
{
    int i;

    for (i=0;i<s->num_streams;i++)
	do_streamline(river+i,s,source_vdata,vdata_coords);


}

int
do_streamline(stream *r,
	      streamlines_struct *ss,
	      float *source_vdata,
	      float **vdata_coords)
{
    vertex *path,vdata,vdata_next;
    float *time_path;
    int next_grid[3];
    int status,i;
    int ijk[3];		/* index of grid cell. */
    vertex p_next;
    vertex p_this;
    double C,s;
    float *t;
    double weights[8];
    int nsteps;
    float *xvpts,*yvpts,*zvpts;
    int vipts,vijpts;
    
    double p_mag;

    xvpts = vdata_coords[0];
    yvpts = vdata_coords[1];
    zvpts = vdata_coords[2];

    C = ss->step_size;
    nsteps = ss->num_steps;

    path = r->path;
    time_path = r->time;
    *time_path++ = 0.;
    
    VCOPY(path,&(r->seed));	/* copy over seed point. */
    VCOPY(&p_this,path);
    
    r->npoints = 1;		/* initialize # in path to 1... */
    IVCOPY((r->grid_current),(r->grid_start));
    find_grid_ijk(&p_this,ijk,&tree);
    if ((ijk[0] == -1) || (ijk[1] == -1) || (ijk[2] == -1))
	return(1);		/* went outside the grid...stop computing here. */
	
    /* p_this is a point in space.  we want the vector data at
     the grid cell (interpolated or point sampled) at the cell that
     contains this point. */
    vipts = ss->ijkdims[0];
    vijpts = vipts * ss->ijkdims[1];

    compute_corner_weights(&p_this,ijk,weights,xvpts,yvpts,zvpts,
			   vipts,vijpts);
    load_vector_data(&vdata,ijk,weights,source_vdata,vipts,vijpts);

    for (i=0;i<nsteps;i++)
    {
	/* note: we should use some interpolated version of p_this.
	 for now, we're doing a way cheap approximation. */
	
	p_mag = vdata.x*vdata.x + vdata.y*vdata.y + vdata.z*vdata.z;
	p_mag = ksqrt(p_mag);
	/* normalize the vector field.. */
	if (p_mag < EPSILON)
	    vdata.x = vdata.y = vdata.z = 0.;
	else
	{
	    vdata.x /= p_mag;
	    vdata.y /= p_mag;
	    vdata.z /= p_mag;
	}
	
	s = p_mag * C;
	
	p_next.x = p_this.x + s * vdata.x;
	p_next.y = p_this.y + s * vdata.y;
	p_next.z = p_this.z + s * vdata.z;

	/* to do the second order runge-kutta, we need to look at the
	 data at the new grid point. */
	find_grid_ijk(&p_next,ijk,&tree);
	if ((ijk[0] == -1) || (ijk[1] == -1) || (ijk[2] == -1))
	    break;		/* went outside the grid... */
	
	compute_corner_weights(&p_next,ijk,weights,xvpts,yvpts,zvpts,
			       vipts,vijpts);
	load_vector_data(&vdata_next,ijk,weights,source_vdata,vipts,vijpts);
	
	p_mag = vdata_next.x*vdata_next.x + vdata_next.y*vdata_next.y +
	    vdata_next.z*vdata_next.z;
	p_mag = ksqrt(p_mag);
	/* normalize the vector field.. */
	if (p_mag < EPSILON)
	    vdata_next.x = vdata_next.y = vdata_next.z = 0.;
	else
	{
	    vdata_next.x /= p_mag;
	    vdata_next.y /= p_mag;
	    vdata_next.z /= p_mag;
	}

	vdata.x = s * (vdata.x + vdata_next.x)*0.5;
	vdata.y = s * (vdata.y + vdata_next.y)*0.5;
	vdata.z = s * (vdata.z + vdata_next.z)*0.5;

	p_next.x = p_this.x + vdata.x;
	p_next.y = p_this.y + vdata.y;
	p_next.z = p_this.z + vdata.z;

	r->npoints += 1;
	path++;
	VCOPY(path,&p_next);
	*time_path++ = C;	/* delta in time. */

	if (i < nsteps-1)
        {
	    VCOPY(&p_this,&p_next);
	    /* find out where i am now */
	    find_grid_ijk(&p_this,ijk,&tree);
	    
	    /* check to see if the point went outside the grid.. */
	    if ((ijk[0] == -1) || (ijk[1] == -1) || (ijk[2] == -1))
		break;

	    compute_corner_weights(&p_this,ijk,weights,xvpts,yvpts,zvpts,
				   vipts,vijpts);
	    load_vector_data(&vdata,ijk,weights,source_vdata,vipts,vijpts);
	}
    }
}

static void
compute_corner_weights(vertex *p,
		       int *ijk,
		       double *w,
		       float *xp,
		       float *yp,
		       float *zp,
		       int isize,
		       int ijsize)
{
    /* given a point "p", which is (supposedly) contained by the
     grid cell ijk, compute the offsets from each corner.  these are
     in the range 0..1, and place them in d[]. */

    double d[8],dinv[8],dinv_sum;
    vertex t[8],s[8];
    int i,j;

    j = ijk[0] + ijk[1]*isize + ijk[2]*ijsize;
    for (i=0;i<2;i++)
    {
	s[i].x = xp[i+j];
	s[i].y = yp[i+j];
	s[i].z = zp[i+j];
    }

    for (i=0;i<2;i++)
    {
	s[i+2].x = xp[i+j+isize];
	s[i+2].y = yp[i+j+isize];
	s[i+2].z = zp[i+j+isize];
    }
    
    for (i=0;i<2;i++)
    {
	s[i+4].x = xp[i+j+ijsize];
	s[i+4].y = yp[i+j+ijsize];
	s[i+4].z = zp[i+j+ijsize];
    }

    for (i=0;i<2;i++)
    {
	s[i+6].x = xp[i+j+ijsize+isize];
	s[i+6].y = yp[i+j+ijsize+isize];
	s[i+6].z = zp[i+j+ijsize+isize];
    }


    for (i=0;i<8;i++)		/* compute distance from point p to
				 each corner of the cell */
    {
	t[i].x = p->x - s[i].x;
	t[i].y = p->y - s[i].y;
	t[i].z = p->z - s[i].z;
    }

    for (i=0;i<8;i++)
	d[i] = t[i].x*t[i].x + t[i].y*t[i].y + t[i].z*t[i].z;

    for (i=0;i<8;i++)
	d[i] = ksqrt(d[i]);

    for (i=0;i<8;i++)
    {
	if (d[i] < EPSILON)
	    dinv[i] = 1.0;
	else
	    dinv[i] = 1./(d[i]*d[i]);
    }

    dinv_sum = 0.;
    for (i=0;i<8;i++)
	dinv_sum += dinv[i];

    for (i=0;i<8;i++)		/* the contributing weight of each corner */
	w[i] = dinv[i] / dinv_sum;
}

static void
load_vector_data(vertex *v,
		 int *ijk,
		 double *w,
		 float *data,
		 int isize,
		 int ijsize)
{
    int i,j;
    float *t;
    
    t = data + ijk[0]*3 + ijk[1]*3*isize + ijk[2]*3*ijsize;

    v->x = v->y = v->z = 0.;

    for (i=0;i<2;i++)
    {
	v->x += *(t+i*3) * w[i];
	v->y += *(t+1+i*3) * w[i];
	v->z += *(t+2+i*3) * w[i];
    }

    j = isize*3;
    for (i=0;i<2;i++)
    {
	v->x += *(t+i*3+j) * w[i+2];
	v->y += *(t+1+i*3+j) * w[i+2];
	v->z += *(t+2+i*3+j) * w[i+2];
    }
    
    j = ijsize*3;
    for (i=0;i<2;i++)
    {
	v->x += *(t+i*3+j) * w[i+4];
	v->y += *(t+1+i*3+j) * w[i+4];
	v->z += *(t+2+i*3+j) * w[i+4];
    }
    
    j = ijsize*3 + isize*3;
    for (i=0;i<2;i++)
    {
	v->x += *(t+i*3+j) * w[i+6];
	v->y += *(t+1+i*3+j) * w[i+6];
	v->z += *(t+2+i*3+j) * w[i+6];
    }
    
}

int
output_streamlines_geometry(stream *river,
			    streamlines_struct *s)
{
    int i,j,offset;
    float fbuf[MAXPATHLENGTH*3];
    int index;

    /* this is the routine that takes the streamline paths and creates
     a bunch of polyline objects. */
    num_prims = 1;
    
    /* build streamlines objects.. */

    for (i=0;i<s->num_streams;i++)
    {
	/* note: there is the potential for a primitive to
	 be created which has zero points in it.  supposedly,
	 geometry services has been fixed to accomodate this.
	 12 nov 96, wes */
	
       kgeom_polyline *p = (kgeom_polyline *) 
	  kgeom_new_primitive(KGEOM_POLYLINE_CONNECTED);
       
       p->nverts = river[i].npoints;
       p->locs = kgeom_dup_locs((float *)river[i].path, p, s->streams_geom);
       
       kgeom_add_primitive(s->streams_geom, (kgeom_primitive *) p);
    }
    return(CHILL);
}

int
init_output_geom_stuff(streamlines_struct *s)
{
   s->streams_geom->name = kstrdup("streamlines");
   return(CHILL);
}
