 /*
  * Khoros: $Id: gtransform.c,v 1.8.2.3 1997/05/31 18:56:27 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gtransform.c,v 1.8.2.3 1997/05/31 18:56:27 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for gtransform
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "gtransform.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - geometric transformation of geometry objects.
|
|       Purpose: main program for gtransform
|
|         Input:
|		char *clui_info->i_file; {Input Geometry Object }
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Output Geometry Object }
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		float clui_info->xscale_float; {Scale to be applied to X coordinates of vertices.}
|		int   clui_info->xscale_flag; {TRUE if -xscale specified}
|
|		float clui_info->xtrans_float; {Translation to be applied to X coordinates of vertices.}
|		int   clui_info->xtrans_flag; {TRUE if -xtrans specified}
|
|		float clui_info->yscale_float; {Scale to be applied to Y coordinates of vertices.}
|		int   clui_info->yscale_flag; {TRUE if -yscale specified}
|
|		float clui_info->ytrans_float; {Translation to be applied to Y coordinates of vertices.}
|		int   clui_info->ytrans_flag; {TRUE if -ytrans specified}
|
|		float clui_info->zscale_float; {Scale to be applied to Z coordinates of vertices.}
|		int   clui_info->zscale_flag; {TRUE if -zscale specified}
|
|		float clui_info->ztrans_float; {Translation to be applied to Z coordinates of vertices.}
|		int   clui_info->ztrans_flag; {TRUE if -ztrans specified}
|
|		int clui_info->do_normals_logic; {Toggle to enable or prohibit transformation of the normals, if present.}
|		int clui_info->do_normals_flag; {TRUE if -do_normals specified}
|
|		int clui_info->parallel_int; {indicates how many nodes will be used in a threaded operation.}
|		int clui_info->parallel_flag; {TRUE if -parallel specified}
|
|        Output:
|       Returns:
|
|    Written By: wes
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	kgeom_object *geom;
	int status;
	char *name=NULL;
	float scale[3];
	float trans[3];
	int do_normals;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/gtransform");
	kexit_handler(gtransform_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "gtransform", KGEN_KROUTINE,  &clui_uis_spec,
		   gtransform_usage_additions, gtransform_get_args,
 		   gtransform_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	geom = kgeom_read_object(clui_info->i_file);
	
	if (geom == NULL)
	{
	    kerror(NULL,"gtubeness","Invalid geometry input %s\n",
		   clui_info->i_file);
	    kexit(KEXIT_FAILURE);
	}

	scale[0] = clui_info->xscale_float;
	scale[1] = clui_info->yscale_float;
	scale[2] = clui_info->zscale_float;

	trans[0] = clui_info->xtrans_float;
	trans[1] = clui_info->ytrans_float;
	trans[2] = clui_info->ztrans_float;

	do_normals = clui_info->do_normals_logic;
	
	/* initialize the threads mechanism... */
#ifdef _USE_THREADS
	init_gthreads(clui_info->parallel_int);
#endif
/* -main_before_lib_call_end */

/* -main_library_call */
	lgtransform(geom,scale,trans,do_normals);
/* -main_library_call_end */

/* -main_after_lib_call */
	kgeom_write_object(geom, clui_info->o_file);
	kgeom_blast_object(geom);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: gtransform_usage_additions
| 
|       Purpose: Prints usage additions in gtransform_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gtransform
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void gtransform_usage_additions(void)
{
	kfprintf(kstderr, "\tgeometric transformation of geometry objects.\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: gtransform_free_args
| 
|       Purpose: Frees CLUI struct allocated in gtransform_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gtransform
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
gtransform_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
