 /*
  * Khoros: $Id: gtubeness.c,v 1.11.2.3 1997/05/31 18:56:43 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gtubeness.c,v 1.11.2.3 1997/05/31 18:56:43 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for gtubeness
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "gtubeness.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Replace Lines with Tesselated Cylinders
|
|       Purpose: main program for gtubeness
|
|         Input:
|		char *clui_info->i_file; {Input geometry object.}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Tube-ified geometry object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		float clui_info->scale_float; {A scalar which indicates the radius of the tube.}
|		int   clui_info->scale_flag; {TRUE if -scale specified}
|
|		int clui_info->subdivisions_int; {The number of sides for each tube will be 2^(subdivisions - 1).}
|		int clui_info->subdivisions_flag; {TRUE if -subdivisions specified}
|
|		int   clui_info->cap_style_list; {A list of choices for possible methods for capping the tubes.}
|		char *clui_info->cap_style_label;
|		int   clui_info->cap_style_flag; {TRUE if -cap_style specified}
|
|        Output:
|       Returns:
|
|    Written By: wes
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	kgeom_object *geom;
	int status;
	char *name=NULL;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/kroutine/gtubeness");
	kexit_handler(gtubeness_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "gtubeness", KGEN_KROUTINE,  &clui_uis_spec,
		   gtubeness_usage_additions, gtubeness_get_args,
 		   gtubeness_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	geom = kgeom_read_object(clui_info->i_file);
	if (geom == NULL)
	{
	    kerror(NULL,"gtubeness","Invalid geometry file %s.\n",
		   clui_info->i_file);
	    kexit(KEXIT_FAILURE);
	}

	/* start writing the object back out.. we'll write out the 
	   primitives, one-by-one, replacing the lines with tesselated
	   cylinders made of triangles */
	if (!kgeom_write_header(geom, clui_info->o_file))
	{
	    kerror(NULL,"gtubeness","Unable to open output file %s.\n",
		   clui_info->o_file);
	    kexit(KEXIT_FAILURE);
	}
	   
	kgeom_start_writing_object(geom);
/* -main_before_lib_call_end */

/* -main_library_call */
	lgtubeness(geom,clui_info->scale_float,
		   clui_info->subdivisions_int,
		   clui_info->cap_style_list);
/* -main_library_call_end */

/* -main_after_lib_call */
	kgeom_finish_writing_object(geom);
	kgeom_done_writing(geom);
	kgeom_blast_object(geom);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: gtubeness_usage_additions
| 
|       Purpose: Prints usage additions in gtubeness_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gtubeness
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void gtubeness_usage_additions(void)
{
	kfprintf(kstderr, "\tReplace Lines with Tesselated Cylinders\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: gtubeness_free_args
| 
|       Purpose: Frees CLUI struct allocated in gtubeness_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gtubeness
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
gtubeness_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->cap_style_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
