
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gthreads.c,v 1.2 1997/01/23 01:11:02 mirage Exp $";
#endif

/* #include "internals.h" */
#ifdef _USE_THREADS

static int loc_concurrency=0;


#if (defined(__sun) && defined(__sparc))
/* SOLARIS Code */

#define _REENTRANT

#include <stdio.h>
#undef _POSIX_SOURCE
#undef _POSIX_C_SOURCE
#undef _XOPEN_SOURCE

#include <sys/thread.h>

#include "geometry/gthreads.h"

int
init_gthreads (int requested_concurrency)
{
    int ncpus_conf,ncpus_online;
    int concurrency_level;

    ncpus_conf = sysconf(_SC_NPROCESSORS_CONF);
    ncpus_online = sysconf(_SC_NPROCESSORS_ONLN);

    fprintf(stderr,"\tThere are %d cpus configured, and\n\t%d cpus online.\n",ncpus_conf,ncpus_online);

    if (requested_concurrency < 1)
	concurrency_level = ncpus_online;
    else
	concurrency_level = requested_concurrency;
    loc_concurrency = concurrency_level;

    if (loc_concurrency > MAX_CONCURRENCY)
    {
	fprintf(stderr,"\tERROR!  The requested level of concurrency, %d, exceeds that of the maximum allowable.  Please adjust gthreads.h and recompile.\n",loc_concurrency);
	exit(1);
    }
    
    fprintf(stderr,"\tThis program will run with %d-way parallelism.\n",concurrency_level);
    thr_setconcurrency(concurrency_level);
}

int
get_loc_concurrency(void)
{
    return(loc_concurrency);
}

int
spin_threads(void (*func)(),
	     void *args,
	     int args_size_in_bytes)
{
    int i;
    void *addr;

    addr = args;
    for (i=0;i<loc_concurrency;i++)
    {
	addr = (void *)((char *)args + args_size_in_bytes*i);
	if (thr_create(NULL,0,func,addr,0,NULL))
	{
	    fprintf(stderr,"\tERROR! thr_create fails.  Bye! \n");
	    perror("spin threads");
	    exit(1);
	}
    }
}

int
join_threads()
{
    int i,id;
    
    for (i=0;i<loc_concurrency;i++)
	thr_join(NULL,&id,NULL);
}


#endif
/* end solaris code */

#endif
