 /*
  * Khoros: $Id: lg3daxis.c,v 1.9.2.2 1997/05/31 18:58:32 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lg3daxis.c,v 1.9.2.2 1997/05/31 18:58:32 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for g3daxis
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lg3daxis
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
static int lcreate_base_axes PROTO((kgeom_object *,float *,int *));
static int lcreate_axis_labels PROTO((kgeom_object *,float *,int *,int,char *));
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lg3daxis - short description of lg3daxis()
* 
*       Purpose: This should be a complete description that anyone
*                could understand;  it should have acceptable grammar
*                and correct spelling.
*
*         Input: argument1 - explanation
*                argument2 - explanation
*                argument3 - explanation
*
*        Output: argument4 - explanation
*                argument5 - explanation
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: Restrictions on data or input as applicable
*    Written By: 
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int
lg3daxis(kobject input,
	 kgeom_object *output,
	 int do_ticks,
	 int do_labels,
	 int do_autoextents,
	 int tick_density,
	 char *name,
	 float *extents,
	 char *format_string)
/* -library_def_end */

/* -library_code */
{
    float loc_extents[6];
    int w,h,d,dim;
    int nprims=1;

    /* get the coordinate info used in building the axes */
    if (do_autoextents)
    {
       if (!kpds_query_location(input))
       {
	  kerror("GEOMETRY","lg3daxis","Input must contain location data over which to calculate the extent");
	  return(WHACKED);
       }

       kpds_get_attribute(input,KPDS_LOCATION_SIZE,&w,&h,&d,&dim);
       if (dim != 3)
       {
	  kerror("GEOMETRY","lg3daxis","Dimensionality of location data must be 3. The dimensionality of this data is %d.",dim);
	    return(WHACKED);
       }
       find_location_minmax(input,w,h,d,dim,(vertex *)loc_extents,
			    (vertex *)(loc_extents+3));
    }
    else
       kmemcpy(loc_extents,extents,sizeof(float)*6);


    /* set up the output object */
    output->name = kstrdup(name);

    /* create the axes themselves */
    lcreate_base_axes(output,loc_extents,&nprims);

    /* create the text that goes with it */
    if (do_labels && do_ticks)
       lcreate_axis_labels(output,loc_extents,&nprims,tick_density,
			   format_string);

    return TRUE;
}

#define MAXSTRINGLENGTH 32

static int
lcreate_axis_labels(kgeom_object *output,
		    float *loc_extents,
		    int *nprims,
		    int nticks,
		    char *format_string)
{
    kgeom_text *text = NULL;
    char **strings,*work;
    vertex *locs;

    /* total number of strings is:
        3 axes * (nticks-1 strings per axis) + 1 (the origin label)     */
    int nstrings,i,index=0;
    float axis,daxis;
    char fstring[64];

    nstrings = 3 * (nticks-1) + 1;
    
    strings = (char **)kmalloc(sizeof(char *)*nstrings);
    work    = (char *)kmalloc(sizeof(char)*nstrings*MAXSTRINGLENGTH);
    kmemset(work,0,(size_t)nstrings*MAXSTRINGLENGTH);
    
    for (i=0;i<nstrings;i++)
	strings[i] = work+MAXSTRINGLENGTH*i;
    locs = (vertex *)kmalloc(sizeof(vertex)*nstrings);

    /* the first string is at the origin. */
    kmemset(fstring,0,64);
    sprintf(fstring,"(%s,%s,%s)",format_string,format_string,format_string);
    sprintf(strings[index],fstring,loc_extents[0],loc_extents[1],loc_extents[2]);
    locs[index].x = loc_extents[0];
    locs[index].y = loc_extents[1];
    locs[index].z = loc_extents[2];
    index++;

    /* do the X axis */
    kmemset(fstring,0,64);
    sprintf(fstring,"%s",format_string);
    daxis = (loc_extents[3]-loc_extents[0])/(nticks - 1);
    axis = loc_extents[0] + daxis;
    for (i=0;i<nticks-1;i++)
    {
	sprintf(strings[index],format_string,axis);
	locs[index].x = axis;
	locs[index].y = loc_extents[1];
	locs[index].z = loc_extents[2];
	index++;
	axis += daxis;
    }

    /* do the Y axis */
    daxis = (loc_extents[4]-loc_extents[1])/(nticks - 1);
    axis = loc_extents[1] + daxis;
    for (i=0;i<nticks-1;i++)
    {
	sprintf(strings[index],format_string,axis);
	locs[index].x = loc_extents[0];
	locs[index].y = axis;
	locs[index].z = loc_extents[2];
	index++;
	axis += daxis;
    }

    /* do the Z axis */
    daxis = (loc_extents[5]-loc_extents[2])/(nticks - 1);
    axis = loc_extents[2] + daxis;
    for (i=0;i<nticks-1;i++)
    {
	sprintf(strings[index],format_string,axis);
	locs[index].x = loc_extents[0];
	locs[index].y = loc_extents[1];
	locs[index].z = axis;
	index++;
	axis += daxis;
    }

    /* stuff the data */
    text = (kgeom_text *) kgeom_new_primitive(KGEOM_TEXT);
    text->nverts  = nstrings;
    text->strings = strings;
    text->locs    = (float *) locs;
    kgeom_add_primitive(output, (kgeom_primitive *) text);

    return TRUE; 
}

surface_colors_rgb red = {1.,0.,0.};
surface_colors_rgb green = {0.,1.,0.};
surface_colors_rgb cyan = {0.,1.,1.};

static int
lcreate_base_axes(kgeom_object *output,
		  float *loc_extents,
		  int *nprims)
{
    /* in this routine, we create a primitive which has 3 disjoint line
       segments. the convention we use here is that the X axis will be colored
       red, the Y axis will be colored green, the Z axis will be colored cyan*/

    kgeom_polyline *lines = NULL;
    vertex *lverts;
    surface_colors_rgb *lcolors;
    int i;

    lverts = (vertex *) kmalloc(sizeof(vertex) * 6);
    lcolors = (surface_colors_rgb *) kmalloc(sizeof(surface_colors_rgb) * 6);

    /* copy the origin over 3 times */
    for (i=0;i<3;i++)
	VCOPY(lverts+(i*2),loc_extents);

    /* end of 1st line is xmax,ymin,zmin */
    VCOPY(lverts+1,loc_extents);
    lverts[1].x = loc_extents[3];

    /* end of 2nd line segment is xmin,ymax,zmin */
    VCOPY(lverts+3,loc_extents);
    lverts[3].y = loc_extents[4];

    /* end of 3rd line segment is xmin,ymin,zmax */
    VCOPY(lverts+5,loc_extents);
    lverts[5].z = loc_extents[5];

    /* now, set the colors.. */

    for (i=0;i<2;i++)
	VCOPY(lcolors+i,&red);
    
    for (i=2;i<4;i++)
	VCOPY(lcolors+i,&green);
    
    for (i=4;i<6;i++)
	VCOPY(lcolors+i,&cyan);

    lines = (kgeom_polyline *) kgeom_new_primitive(KGEOM_POLYLINE_DISJOINT);
    lines->nverts = 6;
    lines->locs = (float *) lverts;
    lines->cols = (float *) lcolors;
    kgeom_add_primitive(output, (kgeom_primitive *) lines);

    return TRUE;
}
/* -library_code_end */
