 /*
  * Khoros: $Id: lgcmaplegend.c,v 1.13.2.3 1997/05/31 18:58:36 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lgcmaplegend.c,v 1.13.2.3 1997/05/31 18:58:36 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for gcmaplegend
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lgcmaplegend
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lgcmaplegend - generate a color legend
* 
*       Purpose: This function is used to contruct geometry
*		 representing a color legend showing the colors
*		 contained in a vis colormap.  The length and
*		 thickness of the legend can be controlled, along with
*		 its 2d position in space.  The legend can be oriented
*		 either vertically or horizontally.  An outline around
*		 the distinct colors in the legend may be optionally
*		 included.  The colors in the legend can be reversed.
*
*         Input: input      - input data object containing vis colormap
*	         name       - object name to assign to the output geometry
*		 reverse    - TRUE if a reversed legend should be generated
*		 outline    - TRUE if the legend should have an outline
*		 horiz_vert - 1 if a horizontal legend should be generated
*		              anything else implies a vertical legend.
*		 length     - length of the legend
*		 thickness  - thickness of the legend
*		 xpos       - x position for the legend
*		 ypos       - y position for the legend
*
*        Output: output - data object to contain color legend geometry
*
*       Returns: CHILL (1) on success, WHACKED (0) on failure
*  Restrictions: 
*    Written By: Wes Bethel
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: *
* Wed Mar  6 14:01:05 PST 1996  -
*     this routine now generates a colormap legend using 2-space coordinates.
*     it should be noted that downstream modules, unless they explicitly
*     check for this attribute, will be given 3space coordinates with a
*     z-value of 0.0.
****************************************************************/
/* -library_def */
int lgcmaplegend (kobject       input,
		  kgeom_object *output,
		  char *name,
		  int reverse,
		  int outline,
		  int horiz_vert,
		  float *length,
		  float *thickness,
		  float *xpos,
		  float *ypos,
		  int do_labels,
		  int num_labels,
		  char *sprintf_format,
		  int do_title,
		  char *title_string,
		  float *title_x_offset,
		  float *title_y_offset,
		  int title_font_size,
		  char *title_font_family)
/* -library_def_end */

/* -library_code */
{
    kgeom_triangles *tri;
    kgeom_polyline  *lines;
    kgeom_text      *text;
   
    int ntriangles, nverts, i, vindex, cindex, j;
    vis_cmap map;
    vertex_3d *verts,*colors=NULL;
    float *coords;
    vertex_3d v0,v1,dv;

    int result;
    float zval = 0.4; /* arbitrary zval for the colorbar */

    /* make call to put the map into a structure. */
    result = lkobj_to_vismap(input,&map);
    if (result == WHACKED)
        return(WHACKED);

    /* set up stuff for output object */
    ntriangles = map.nentries * 2;
    nverts = ntriangles+2;

    /* set up object */
    output->name = kstrdup(name);
    output->has_alpha = FALSE; 
    output->layout = KPER_FACE;

    /* set attribute saying that we're working with 2-space coordinates */
    output->location_dim = 2;
    
    /* set modeling space to ndc -- the coords map to -1..1 in x/y for
       facilitating placing the color legend on the screen, irrespective
       of what's happening in the world coord system */
/* not yet implemented in rmonster.
    output->modeling_space = KNDC_SPACE;
*/

    verts = (vertex_3d *)kmalloc((unsigned)(sizeof(vertex_3d)*nverts));
    coords = (float *)kmalloc(sizeof(float)*nverts*2);
    colors = (vertex_3d *)kmalloc((unsigned)(sizeof(vertex_3d)*nverts));

    if (horiz_vert == 1)  /* horiz. */
    {
        v0.v[0] = *xpos;
	v0.v[1] = *ypos;
	v0.v[2] = zval;

	v1.v[0] = *xpos;
	v1.v[1] = *ypos + *thickness;
	v1.v[2] = zval;
	
	dv.v[0] = (*length)/(map.nentries);
	dv.v[1] = dv.v[2] = 0.;
    }
    else /* vert. */
    {
        v0.v[0] = *xpos;
	v0.v[1] = *ypos;
	v0.v[2] = zval;

	v1.v[0] = *xpos + *thickness;
	v1.v[1] = *ypos;
	v1.v[2] = zval;

	dv.v[1] = (*length)/(map.nentries);
	dv.v[0] = dv.v[2] = 0.;
    }

    /* compute the triangle vertices */
    vindex = 0;
    /* changed map.nentries+2 to map.nentries+1
     *
     *	logic for doing this :   (map.nentries+2)/2 = map.nentries+1
     *
     *							// Steve K 
     */
    for (i=0;i<(map.nentries+1);i++)
    {
	/* leave in the use of 3d coords for purposes of computing
	 a bounding box.  */
	
        memcpy((char *)(verts+vindex),(char *)&v0,sizeof(vertex_3d)); 
	coords[vindex*2] = v0.v[0];
	coords[vindex*2+1] = v0.v[1];
	vindex++;
	
	memcpy((char *)(verts+vindex),(char *)&v1,sizeof(vertex_3d));
	coords[vindex*2] = v1.v[0];
	coords[vindex*2+1] = v1.v[1];
	vindex++;

	for (j=0;j<3;j++)
	{
	    v0.v[j] += dv.v[j];
	    v1.v[j] += dv.v[j];
	}
    }

    /* deal with colors. */

    memset((char *)colors,0,sizeof(vertex_3d)*ntriangles);
    cindex = 0;
    for (i=0;i<map.nentries;i++)
    {
        colors[cindex].v[0] = map.r[i];
	colors[cindex].v[1] = map.g[i];
	colors[cindex].v[2] = map.b[i];
	cindex++;

        colors[cindex].v[0] = map.r[i];
	colors[cindex].v[1] = map.g[i];
	colors[cindex].v[2] = map.b[i];
	cindex++;
    }
    
    tri = (kgeom_triangles *) kgeom_new_primitive(KGEOM_TRIANGLES_CONNECTED);
    tri->nverts = nverts;
    tri->locs   = (float *) coords;
    tri->cols   = (float *) colors;
    kgeom_add_primitive(output, (kgeom_primitive *) tri);

#if 0
    /* take out bounding box stuff. */
    /**
      * call library routine to compute the bounding box.
    **/
    output->has_bounding_box = TRUE;
    lcompute_bbox(verts,nverts,&output->min,&output->max);

    output->has_center = TRUE;
    for (i=0;i<3;i++)
	output->center[i] = output->min[i] + 
	   (output->max[i] - output->min[i])*0.5;
#endif

    /* if requested, construct the outline */
    if (outline)
    {
	float *overts;

	/* 4 corners + closing vertex*/
	overts = (float *) kmalloc(sizeof(float)*10);
	
	overts[0] = *xpos;
	overts[1] = *ypos;

	overts[3] = *ypos;
	if (horiz_vert == 1)	/* horizontal */
	    overts[2] = *xpos + *length;
	else
	    overts[2] = *xpos + *thickness;

	overts[4] = overts[2];
	if (horiz_vert == 1)
	    overts[5] = *ypos + *thickness;
	else
	    overts[5] = *ypos + *length;

	overts[6] = *xpos;
	overts[7] = overts[5];

	overts[8] = overts[0];
	overts[9] = overts[1];

	lines = (kgeom_polyline *) 
	        kgeom_new_primitive(KGEOM_POLYLINE_CONNECTED);
	lines->nverts = 5;
	lines->locs = overts;
	kgeom_add_primitive(output, (kgeom_primitive *) lines);
    }

    if (do_labels)
    {
	char **strings,*work;
	float *lverts,dx,dy,offset;
	float fbracket,dfbracket;
#define MAXSTRINGLENGTH 256
	offset = 0.1;		/* make this a parameter! */
	
	strings = (char **)kmalloc(sizeof(char *)*num_labels);
	work = (char *)kmalloc(sizeof(char)*num_labels*MAXSTRINGLENGTH);
	for (i=0;i<num_labels;i++)
	   strings[i] = work+MAXSTRINGLENGTH*i;
	kmemset(work,0,(size_t)num_labels*MAXSTRINGLENGTH);
	lverts = (float *)kmalloc(sizeof(float)*2*num_labels);

	/* first, build the locations for each text string */

	lverts[0] = *xpos + ((horiz_vert == 1) ? 0. : (*thickness + offset)) ;
	lverts[1] = *ypos + ((horiz_vert == 1) ? -offset : 0) ;

	dx = (horiz_vert == 1) ? (*length / (num_labels - 1)) : 0.;
	dy = (horiz_vert == 1) ? 0. : (*length / (num_labels - 1));

	for (i=1;i<num_labels;i++)
	{
	    lverts[i*2] = lverts[(i-1)*2] + dx;
	    lverts[i*2+1] = lverts[(i-1)*2+1] + dy;
	}

	/* now, do the actual labels. */

	fbracket = 0.;
	dfbracket = (map.nentries / (num_labels - 1));
	for (i=0;i<num_labels;i++, fbracket += dfbracket)
	   ksprintf(strings[i],sprintf_format,map.bracket[(int)fbracket]);

	text = (kgeom_text *) kgeom_new_primitive(KGEOM_TEXT);
	text->nverts  = num_labels;
	text->strings = (kstring *) karray_copy(strings, KSTRING,
						num_labels, TRUE);
	text->locs    = lverts;
	kgeom_add_primitive(output, (kgeom_primitive *) text);

	kfree(work);
	kfree(strings);
    }

    if (do_title)
    {
	vertex_3d vert[2];
	char **title,**font_strings,*work,*work2;

	title        = (kstring *) kmalloc(sizeof(kstring) * 2);
	font_strings = (kstring *) kmalloc(sizeof(kstring) * 2);

	vert[0].v[0] = *xpos + *title_x_offset;
	vert[0].v[1] = *ypos + *title_y_offset;
	vert[0].v[2] = 0.;

	vert[1] = vert[0];
	
	work = (char *)kmalloc(sizeof(char)*MAXSTRINGLENGTH*2);
	work2 = (char *)kmalloc(sizeof(char)*MAXSTRINGLENGTH*2);
	font_strings[0] = work;
	font_strings[1] = work;
	
	kstrcpy(work2,title_string);
	title[0] = work2;
	title[1] = work2 + MAXSTRINGLENGTH;

	kstrcpy(work2+MAXSTRINGLENGTH,"."); /* a dummy string */
	
	kmemset(work,0,MAXSTRINGLENGTH);

	/* build a string that X can understand */
	ksprintf(work,"*%s*medium*-r-*%d*",title_font_family,title_font_size);

	text = (kgeom_text *) kgeom_new_primitive(KGEOM_TEXT);
	text->nverts  = 1;
	text->strings = (kstring *) karray_copy(title, KSTRING, 2, TRUE);
	text->fonts   = (kstring *) karray_copy(font_strings, KSTRING, 2, TRUE);
	text->locs    = kgeom_dup_locs((float *)vert, lines, output);
	kgeom_add_primitive(output, (kgeom_primitive *) text);

	kfree(title);
	kfree(font_strings);

	kfree(work);
	kfree(work2);
    }

    /* we never set this on the object, so it's okay to free */
    kfree(verts);
    
    return(CHILL);
}
/* -library_code_end */
