 /*
  * Khoros: $Id: lgcreate2dtex.c,v 1.9.2.3 1997/05/31 18:58:40 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lgcreate2dtex.c,v 1.9.2.3 1997/05/31 18:58:40 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for gcreate2dtex
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lgcreate2dtex
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lgcreate2dtex - take an "image" and create a 2d texture for use by renderers,etc. in the GEOMETRY toolbox.
* 
*       Purpose: provide a routine which will convert an arbitrary
*        image into a 2d "texture"
*
*         Input: kobject inobj - contains a source image which will be
*                  used as the texture.
*
*        Output: kobject outobj - will contain a geometry services texture
*                  which can be bound to an object with gbindtexture.
*
*        Parms:  float tc[4] - contains texture coordinates that will
*                  be associated with the min/max corners of the texture.
*                  these coordinates, along with multiple textures, can
*                  be used to implement tiled textures.
*                int method - indicates what kind, if any, of filtering
*                  will be applied to the source image.
*
*   september 1995 NOTES:
*   1. at present, we only do a straight copy of the source texture data.
*      in the future, we will implement MIPmapping of textures.  some
*      work is needed on the data services infrastructure before this
*      will funciton properly.
*   2. it seems to make sense to put tiled textures along the depth axis.
*      this is not implemented.  furthermore, if it were, then the tc[]
*      array would need to be two-dimensional as we need a set of coordinates
*      for each tile in the set.
*   3. the stuff in the tc[] array is not used at all right now.  this is
*      because there is no corresponding attribute in the geom services
*      infrastructure.  the default behavior in rmonster will be to use
*      a coordinate range of 0..1 for the texture (effectively disabling
*      the use of tiled textures).
*
*       Returns: CHILL on success, WHACKED otherwise
*
*  Restrictions: Restrictions on data or input as applicable
*    Written By: wes
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lgcreate2dtex(kobject inobj,
		  kgeom_object *outobj,
		  float *tc,
		  int method)
/* -library_def_end */

/* -library_code */
{
    int has_alpha,size;
    int vw,vh,vd,vt,ve;
    unsigned char *ival;
    kgeom_texture2D *texture = NULL;
    
    /* configuration check on input object */
    kpds_get_attribute(inobj,KPDS_VALUE_SIZE,&vw,&vh,&vd,&vt,&ve);

    /* check for d==1, t==1 */
    if ((vd != 1) || (vt != 1))
    {
	kerror("GEOMETRY","lcreate2dtex","The input image has d!=1 or t!=1. Please supply a 2d image as input.");
	return(WHACKED);
    }

    if (vw == 1 || vh == 1)
    {
	kerror("GEOMETRY","lcreate2dtex","The input image must have width and height greater than 1.");
	return(WHACKED);
    }

    
    kcolor_get_attribute(inobj,KCOLOR_HAS_ALPHA,&has_alpha);

    kpds_set_attribute(inobj,KPDS_MAPPING_MODE,KMAPPED); 

    /* set it up so we get RGB or RGBa KUBYTE vectors with values 0.0-255.0 */
    size = (has_alpha) ? 4 : 3;
    kpds_set_attributes(inobj,
			KPDS_VALUE_SIZE,-1,-1,-1,-1,size,
			KPDS_VALUE_DATA_TYPE,KUBYTE,
			KPDS_ELEMENTS_FIRST,TRUE,
			KPDS_VALUE_NORM_MIN, 0.0, 
			KPDS_VALUE_NORM_MAX, 255.0, 
			NULL);

    ival = (unsigned char *)kpds_get_data(inobj,KPDS_VALUE_ALL,NULL);
    
    /* configure output object */
    outobj->has_alpha = has_alpha;
    
    texture = (kgeom_texture2D *) kgeom_new_primitive(KGEOM_TEXTURE2D);
    texture->width  = vw;
    texture->height = vh;
    texture->cols   = (kubyte *) ival;

    kgeom_add_primitive(outobj, (kgeom_primitive *)texture);

    return(CHILL);
}

/****************************************************************
* 
*  Routine Name: lgisa2dtex - determine if the object is a 2d texture
* 
*       Purpose: a short function for checking to see if the given object
*                is a "2d texture"
*
*         Input: kobject inobj - input object, will be sanity-checked
*                to determine if it's contents are a 2d texture.
*
*        Output: none
*
*       Returns: TRUE if the object is a 2d texture, FALSE otherwise
*
*  Restrictions: 
*    Written By: wes
*          Date: Sep 07, 1995
*      Verified: 
*  Side Effects: 
* Modifications: 
****************************************************************/

int
lgisa2dtex(kgeom_object *geom)
{
    kgeom_primitive *prim = NULL;
    int num_prims, i;

    num_prims = kgeom_number_primitives(geom);
    
    for (i=0;i<num_prims;i++)
    {
       prim = kgeom_get_primitive(geom, i);

       if (prim->type == KGEOM_TEXTURE2D ||
           prim->type == KGEOM_TEXTURE3D)
          return TRUE;
    }
    return FALSE;
}
/* -library_code_end */
