 /*
  * Khoros: $Id: lglinecolor.c,v 1.13.2.3 1997/05/31 18:58:47 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lglinecolor.c,v 1.13.2.3 1997/05/31 18:58:47 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for glinecolor
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lglinecolor
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
#include "geometry/vis_cmap.h"
static void get_colored_segments	PROTO((int, kfile *, kgeom_object *, 
					       int, vertex_3d *, vertex_3d *,
		     			       vis_cmap *));
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lglinecolor - *
* 
*       Purpose: This should be a complete description that anyone
*                could understand;  it should have acceptable grammar
*                and correct spelling.
*
*         Input: argument1 - explanation
*                argument2 - explanation
*                argument3 - explanation
*
*        Output: argument4 - explanation
*                argument5 - explanation
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: Restrictions on data or input as applicable
*    Written By: 
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lglinecolor(kfile *infile,
		kobject vis_cmap_obj,
		kgeom_object *outobj,
		char *name)
/* -library_def_end */

/* -library_code */
{
    int nsegs,seq_no;
    int i,result;
    vertex_3d bmin,bmax,center;
    vis_cmap lmap;

    /* try to convert the vis_cmap_obj to a vis_map */
    result = lkobj_to_vismap(vis_cmap_obj,&lmap);
    if (result == WHACKED)  /* assume the library routne barked at the user */
	return(WHACKED);
    
    /* set up the output object */
    outobj->name = kstrdup(name);

    kfscanf(infile,"%d",&nsegs);
    seq_no = 0;

    for (i=0;i<3;i++)
    {
	bmin.v[i] = KGEOM_BIGNUM;
	bmax.v[i] = KGEOM_SMALLNUM;
    }
    
    while (!kfeof(infile))
    {
	get_colored_segments(nsegs,infile,outobj,seq_no,&bmin,&bmax,&lmap);
	kfscanf(infile,"%d",&nsegs);
	seq_no++;
    }

    for (i=0;i<3;i++)
	center.v[i] = bmin.v[i] + (bmax.v[i] - bmin.v[i]) * 0.5;

    outobj->has_bounding_box = TRUE;
    outobj->min[0] = bmin.v[0];
    outobj->min[1] = bmin.v[1];
    outobj->min[2] = bmin.v[2];
    outobj->max[0] = bmax.v[0];
    outobj->max[1] = bmax.v[1];
    outobj->max[2] = bmax.v[2];

    outobj->has_center = TRUE;
    outobj->center[0] = center.v[0];
    outobj->center[1] = center.v[1];
    outobj->center[2] = center.v[2];

    return TRUE;
}

static void
get_colored_segments(int nverts,
		     kfile *infile,
		     kgeom_object *outobj,
		     int prim_no,
		     vertex_3d *min,
		     vertex_3d *max,
		     vis_cmap *lmap)
{
    int i,j;
    vertex_3d *v,*v1;
    float *c,*c1;
    float scalar;
    int index;
    kgeom_polyline *lines;

    v = v1 = (vertex_3d *)kmalloc((size_t)(sizeof(vertex_3d)*nverts));
    c = c1 = (float *)kmalloc((size_t)(sizeof(float)*nverts*4));
    
    for (i=0;i<nverts;i++,v++,c+=4)
    {
	kfscanf(infile,"%f %f %f %f",&(v->v[0]),&(v->v[1]),&(v->v[2]),&scalar);
	for (j=0;j<3;j++)
	{
	    if (v->v[j] > max->v[j])
		max->v[j] = v->v[j];
	    if (v->v[j] < min->v[j])
		min->v[j] = v->v[j];
	}

	index = lget_map_index(lmap,&scalar);
	lget_map_rgba(lmap,index,c,c+1,c+2,c+3);
    }
    
    lines = (kgeom_polyline *) kgeom_new_primitive(KGEOM_POLYLINE_CONNECTED);
    lines->nverts = nverts;
    lines->locs = (float *) v1;
    lines->cols = (float *) c1;
    kgeom_add_primitive(outobj, (kgeom_primitive *) lines);
}
/* -library_code_end */
