 /*
  * Khoros: $Id: lgoctmesh.c,v 1.14.2.4 1997/05/31 18:58:53 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lgoctmesh.c,v 1.14.2.4 1997/05/31 18:58:53 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for goctmesh
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lgoctmesh
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
#include "internals.h"

static int ok_octmesh_input PROTO(( kobject, int * ));
static int make_oct_mesh PROTO(( kobject, kgeom_object *, int *, vis_cmap *, int,int,int));
static void malloc_slice PROTO((octmesh_slice *,int,int,int));
/* static void free_slice PROTO((octmesh_slice *)); */
static void load_slice PROTO((octmesh_slice *, int,int,int,kobject));
static void compute_normal PROTO((vertex *,int,int,int,int,int,octmesh_slice *,octmesh_slice *,octmesh_slice *,octmesh_slice *, int));
static void compute_slice_normals PROTO((octmesh_slice *,octmesh_slice *,octmesh_slice *,int,int,int));
static void dump_slice PROTO((kgeom_object *,octmesh_slice *,int,int,int,int,vertex *,int));
int colorize_slice PROTO((octmesh_slice *, vis_cmap *, int, int));
int compute_map_indices PROTO((octmesh_slice *, vis_cmap *, int, int));
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lgoctmesh - *
* 
*       Purpose: to create an octmesh geom object from 3D data.
*                the geom may be optionally colorized with
*                a colormap, whose presence is optional.
*
*         Input: inobj - an opened kobject containing the source data
*                cmapobj - an opened kobject containing a visualization cmap
*                name - a char string containing the name for the geom object
*
*        Output: outobj - an opened kobject containing the geom object
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: Restrictions on data or input as applicable
*    Written By: Yo Momma
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: do not drive heavy machinery while operating this software
*      Examples: 
* Modifications: *
* jan 16 - changes for looping through the input data set by plane,
*          rather than trying to process the whole thing at once.
*          add code to generate normals.

Mon Oct 28 09:45:06 PST 1996 -
   various enhancements to produce more compact octmesh files.
   specifically, when the input data consists of scalar data,
   we will convert it to unsigned byte format, thus achieving
   some space savings.

   we will try really hard to write an octmesh object which conforms
   to the following attributes:
      1. floating point 3space location data.
      2(a). byte, scalar color information
      2(b). rgba colormap (NOT a vis colormap)
      3. floating point 3dim "normals" which represent gradient in
         the original floating point input scalar data.


   only if the input data contains 4-component (or 3-component) color
   information at each vertex will this format of data be written.

Sun Jan 19 1997 : Steve K
   new geometry services has no support for colormap : octmesh will 
   contain float rgba colors.
****************************************************************/
/* -library_def */
int lgoctmesh(kobject inobj,
	      int have_mapfile,
	      kobject cmapobj,
	      kgeom_object *outobj,
	      char *name,
	      int do_normals,
	      int flip_normals)
/* -library_def_end */

/* -library_code */
{
    vis_cmap cmap;
    int i,num_dims,dims[5],input_elements;

    /** first, verify that the input object is processable **/

    /** at present, we want 3 dimensional data and 3d (or no) coords and
      will barf if we get anything else **/

    if (ok_octmesh_input(inobj,&input_elements) == WHACKED)
	return(WHACKED);

    /* next, convert the colormap if it's present to a vis_cmap */
    if (have_mapfile)
    {
	if (lkobj_to_vismap(cmapobj,&cmap) == WHACKED)
        {
	    errno = KINVALID_DATATYPE;
	    kerror("GEOMETRY","loctmesh","The object supplied for a vis colormap is invalid. \n");
	    return(WHACKED);
	}

	if (cmap.nentries > 256)
	{
	    kerror("GEOMETRY","loctmesh"," please use a vis colormap that has 256 or fewer entries.  you've given me one with %d entries. \n",cmap.nentries);
	    return(WHACKED);
	}
	
	if (input_elements != 1)
	{
	    kerror("GEOMETRY","loctmesh"," I can't make an octmesh with a colormap and non-scalar input data. Aborting.");
	    return(WHACKED);
	}
    }

    /* set up input object */
    kpds_set_attribute(inobj,KPDS_LOCATION_DATA_TYPE,KFLOAT);
    kpds_set_attribute(inobj,KPDS_VALUE_DATA_TYPE,KFLOAT);

#if 0
    /* color data */
    if (input_elements == 1)
    {
	/* set up for byte output color data */
	kgeom_set_attribute(outobj,KGEOM_OBJECT,KGEOM_COLOR_DATA_TYPE,KUBYTE);
	kgeom_set_attribute(outobj,KGEOM_OBJECT,KGEOM_COLORSPACE,KRGB);
    }
    else
    {
	/* set up for floating pt multi-component output color */
	kgeom_set_attribute(outobj,KGEOM_OBJECT,KGEOM_COLOR_DATA_TYPE,KFLOAT);
	kgeom_set_attribute(outobj,KGEOM_OBJECT,KGEOM_COLORSPACE,KRGB);
    }
#endif

    /* figure out if we are workign with 2 or 3d data. */
    kpds_get_attribute(inobj,KPDS_VALUE_SIZE,dims,dims+1,dims+2,
		       dims+3,dims+4);
	
    /* in the following loop, we`ll scan through the sizes associated
       with each value segment dimension (EXCEPT for elements) and
       count if the value is greater than 1 */

    num_dims=0;
    for (i=0;i<4;i++)
	if (dims[i] > 1)
	    num_dims++;

    /* if num_dims==3, then we'll (try to) make an octmesh.  otherwise,
       we'll barf. */

    /* set up the output object */
    outobj->name      = kstrdup(name);
    outobj->layout    = KPER_VERTEX;
    outobj->has_alpha = TRUE;
    kgeom_start_writing_object(outobj);

    if (num_dims == 3)
	make_oct_mesh(inobj,outobj,dims,&cmap,have_mapfile,do_normals,flip_normals); 
    else
    {
	errno = KINVALID_DATATYPE;
	kerror("GEOMETRY","loctmesh","There must be 3 dimmensions of value data to make an octmesh.  You have given me %d\n",num_dims);
	return(WHACKED);
    }    

    kgeom_finish_writing_object(outobj);

    return(CHILL);
}

static int
ok_octmesh_input(kobject inobj,int *input_elements)
{
    int lw,lh,ld,ldims;
    int vw,vh,vd,ve,vt;

    if (kpds_query_location(inobj))
    {
       kpds_get_attribute(inobj,KPDS_LOCATION_SIZE,&lw,&lh,&ld,&ldims);
    
       if (ldims != 3)
       {
	  errno = KINVALID_DATATYPE;
	  kerror("GEOMETRY","loctmesh","The location data must have dimensions=3.  The input file has dimensions=%d.\n",ldims);
	  return(WHACKED);
       }
    }
    else
    {
       errno = KINVALID_DATATYPE;
       kerror("GEOMETRY","loctmesh","The input file must have explicit location data\n");
       return(WHACKED);
    }

    /* set up a happy presentation for the location data */
    kpds_set_attribute(inobj, KPDS_LOCATION_DATA_TYPE, KFLOAT);

    kpds_get_attribute(inobj,KPDS_VALUE_SIZE,&vw,&vh,&vd,&vt,&ve);
    *input_elements = ve;
    if (ve != 1)  /* it's not scalar data .. */
    {
	if (ve != 4) /* can't possibly be anything but RGBA data */
	{
	    errno = KINVALID_DATATYPE;
	    kerror("GEOMETRY","loctmesh","This datafile has elements = %d.  Only elements=1 or elements=4 (interpreted as RGBA) are supported.");
	    return(WHACKED);
	}
    }

    /* set up a happy presentation for the value data */
    kpds_set_attribute(inobj, KPDS_VALUE_DATA_TYPE, KFLOAT);

    return(CHILL);
}

static int
make_oct_mesh(kobject kin,
	      kgeom_object *kout,
	      int *dims,
	      vis_cmap *cmap,
	      int use_map,
	      int do_normals,
	      int flip_normals)
{
    int i,iw;
    int usize,vsize,wsize;
    octmesh_slice *slice_prev,*slice0,*slice1,*slice_next,*slice_temp;
    int have_colors,have_normals;
    vertex *work_verts;

    /* temp */
    have_colors = use_map;
    have_normals = 1;

    /* Mon Oct 28 10:18:12 PST 1996 - below, we're going to assume that if
     "use_map" is true upon input that we're going to use the ubyte output
     color format.  */

    /* assign values to usize,vsize and wsize.  from the calling routine,
       dims[] holds the size of the value data.  i'm assuming that
       there are at three and only three values in this array that are > 1.
       such an assumption eliminates a lot of boring error checking
       code, but the below code will seg fault if this is not the case */
    
    for (i=0;i<4;i++)
	if (dims[i] != 1)
	    break;
    
    usize = dims[i];
    for (i=i+1;i<4;i++)
	if (dims[i] != 1)
	    break;
    vsize = dims[i];
    
    for (i=i+1;i<4;i++)
	if (dims[i] != 1)
	    break;
    wsize = dims[i];

    /* malloc a work area for vertices.  this will be used by the
     dump_slice() routine as it reformats data for output to dataservices */
    work_verts = (vertex *)kmalloc(sizeof(vertex)*usize*vsize);

    /* write out the octmesh header */
    /* this mesh will have no texture or texture coords */
    kgeom_write_octmesh_header(kout, usize, vsize, wsize, NULL,
			       TRUE, TRUE, have_normals, FALSE);

    /* grab the location data from the input. this code below assumes
     3d location data for both input and output, present on a per-grid
     point basis (curvilinear).  this needs to be enhanced to accomodate
     varying grid types. */

    slice_prev = (octmesh_slice *)kmalloc(sizeof(octmesh_slice));
    slice0 = (octmesh_slice *)kmalloc(sizeof(octmesh_slice));
    slice1 = (octmesh_slice *)kmalloc(sizeof(octmesh_slice));
    slice_next = (octmesh_slice *)kmalloc(sizeof(octmesh_slice));
    
    malloc_slice(slice_prev,usize,vsize,use_map);
    malloc_slice(slice0,usize,vsize,use_map);
    malloc_slice(slice1,usize,vsize,use_map);
    malloc_slice(slice_next,usize,vsize,use_map);

    load_slice(slice_prev,0,usize,vsize,kin);
    load_slice(slice0,0,usize,vsize,kin);
    load_slice(slice1,1,usize,vsize,kin);

    if (wsize == 2)
	load_slice(slice_next,1,usize,vsize,kin);
    else
	load_slice(slice_next,2,usize,vsize,kin);

    for (iw=0;iw<wsize;iw++)
    {
	/* process the slice */
	fprintf(stderr," processing slice %d. \n",iw);

#if 0
	if (use_map)
	    colorize_slice(slice0,cmap,usize,vsize);
#endif
	if (use_map)
	    compute_map_indices(slice0,cmap,usize,vsize);

	/* always compute slice normals */
	compute_slice_normals(slice_prev,slice0,slice1,usize,vsize,flip_normals);
	/* dump the slice */
	dump_slice(kout,slice0,usize,vsize,use_map,
		   have_normals,work_verts,iw);

	/* play pointer cames, switching around current, previous
	 and next slice pointers. */
	slice_temp = slice_prev;
	slice_prev = slice0;
	slice0 = slice1;
	slice1 = slice_next;
	slice_next = slice_temp;

	if (iw < wsize-3)
	    load_slice(slice_next,iw+3,usize,vsize,kin);
    }

    kfree(work_verts);
    return(CHILL);
}

#if 0
void
lcompute_3dgrad(vertex_3d *dg_slice0,
		int usize,
		int vsize,
		float *d_slice0,
		float *d_slice1,
		float *d_slice2,
		vertex_3d *c_slice0,
		vertex_3d *c_slice1,
		vertex_3d *c_slice2,
		vertex_3d *work,
		int flip)
{
    /**
      * PARMS:
      *
      * dg_slice0 - destination array for the gradients
      * usize,vsize - represent the width and height of a slice of data
      * d_slice0,d_slice1,d_slice2 - three "slices" of data
      * c_slice0,c_slice1,c_slice2 - three "slices" of coordinate information
      * work - a scratch area of size usize*vsize*3 for use by this routine
    **/

    /**
      * the "gradient" is computed as a function of both the data
      * to be contoured, and the space in which it exists.
      *
      * without considering the space, the data gradient is:
      *
      * G.x = data(x+1,y,z) - data(x-1,y,z)
      * G.y = data(x,y+1,z) - data(x,y-1,z)
      * G.z = data(x,y,z+1) - data(x,y,z-1)
      *
      * There is a similar gradient computed for each of the
      * x,y, and z axes of the coordinate space:
      *
      * GX.x = coords(x+1,y,z).x - coords(x-1,y,z).x
      * GX.y = coords(x+1,y,z).y - coords(x-1,y,z).y
      * GX.z = coords(x+1,y,z).z - coords(x-1,y,z).z
      *
      * and similarly for GY and GZ.
      *
      * the "final" gradient is computed as a function of the data
      * gradient and the spatial gradient.   this will provide the
      * correct gradient in curvilinear grids:
      *
      * grad.x = G.x*GX.x + G.x*GX.y + G.x*GX.z
      * grad.y = G.y*GY.x + G.y*GY.y + G.y*GY.z
      * grad.z = G.z*GZ.x + G.z*GZ.y + G.z*GZ.z
    **/
    
    register float *d0,*d1;
    register vertex_3d *g;
    register double t;
    register vertex_3d *c0,*c1;
    register vertex_3d *sx,*sy,*sz;
    double flip_sign;

    int i,j;

    if (flip)
	flip_sign= -1.;
    else
	flip_sign = 1.;
    
    /* do z gradient first */
    
    d0 = d_slice0;
    d1 = d_slice2;
    g = dg_slice0;
    
    for (j=0;j<vsize;j++)
    {
	for (i=0;i<usize;i++)
	{
	    g->v[2] = (*d1 - *d0) * 0.5;
	    d1++;
	    d0++;
	    g++;
	}
    }

    /* next, do X gradients */

    for (j=0;j<vsize;j++)
    {
	g = dg_slice0 + j*usize;
	d0 = d_slice1 + j*usize;

	g->v[0] = (*(d0+1) - *d0) * 0.5;
	d0++;
	g++;
	
	for (i=1;i<usize-1;i++)
        {
	    g->v[0] = (*(d0+1) - *(d0-1)) * 0.5;
	    d0++;
	    g++;
	}
	g->v[0] = (*(d0) - *(d0-1)) * 0.5;
    }

    /* finally, do the Y gradients. */

    d0 = d_slice1;
    d1 = d0 + usize;
    g = dg_slice0;

    /* do first row. */
    for (i=0;i<usize;i++)
    {
	g->v[1] = (*d1 - *d0) * 0.5;
	d1++;
	d0++;
	g++;
    }

    /* do subseqeunt rows (except the last one */
    d0 = d_slice1;
    d1 = d0 + usize*2; /* skip forward two rows. */
	
    for (j=1;j<vsize-1;j++)
    {
	for (i=0;i<usize;i++)
	{
	    g->v[1] = (*d1 - *d0) * 0.5;
	    d1++;
	    d0++;
	    g++;
	}
    }

    d1 = d0 + usize;
    /* do last row. */
    for (i=0;i<usize;i++)
    {
	g->v[1] = (*d1 - *d0) * 0.5;
	d1++;
	d0++;
	g++;
    }

    /* now compute the gradients in space */
    
    /* do the Z coords first. */
    c0 = c_slice0;
    c1 = c_slice2;
    g = work+usize*vsize*2;
    
    for (j=0;j<vsize;j++)
    {
	for (i=0;i<usize;i++)
	{
	    g->v[0] = (c1->v[0] - c0->v[0]);
	    g->v[1] = (c1->v[1] - c0->v[1]);
	    g->v[2] = (c1->v[2] - c0->v[2]);
	    c1++;
	    c0++;
	    g++;
	}
    }

    /* next, do the Y coords. */

    c0 = c_slice1;
    c1 = c0 + usize;
    g = work + usize*vsize;

    /* do first row. */
    for (i=0;i<usize;i++)
    {
	g->v[0] = (c1->v[0] - c0->v[0]);
	g->v[1] = (c1->v[1] - c0->v[1]);
	g->v[2] = (c1->v[2] - c0->v[2]);
	c1++;
	c0++;
	g++;
    }

    /* do subseqeunt rows (except the last one */
    c0 = c_slice1;
    c1 = c0 + usize*2; /* skip forward two rows. */
	
    for (j=1;j<vsize-1;j++)
    {
	for (i=0;i<usize;i++)
	{
	    g->v[0] = (c1->v[0] - c0->v[0]);
	    g->v[1] = (c1->v[1] - c0->v[1]);
	    g->v[2] = (c1->v[2] - c0->v[2]);
	    d1++;
	    d0++;
	    g++;
	}
    }

    c1 = c0 + usize;
    /* do last row. */
    for (i=0;i<usize;i++)
    {
	g->v[0] = (c1->v[0] - c0->v[0]);
	g->v[1] = (c1->v[1] - c0->v[1]);
	g->v[2] = (c1->v[2] - c0->v[2]);
	d1++;
	d0++;
	g++;
    }

    /* next, do X gradients */

    for (j=0;j<vsize;j++)
    {
	g = work + j*usize;
	c0 = c_slice1 + j*usize;

	g->v[0] = ((c0+1)->v[0] - c0->v[0]);
	g->v[1] = ((c0+1)->v[1] - c0->v[1]);
	g->v[2] = ((c0+1)->v[2] - c0->v[2]);
	c0++;
	g++;
	
	for (i=1;i<usize-1;i++)
        {
	    g->v[0] = ((c0+1)->v[0] - (c0-1)->v[0]);
	    g->v[1] = ((c0+1)->v[1] - (c0-1)->v[1]);
	    g->v[2] = ((c0+1)->v[2] - (c0-1)->v[2]);
	    c0++;
	    g++;
	}
	g->v[0] = (c0->v[0] - (c0-1)->v[0]);
	g->v[1] = (c0->v[1] - (c0-1)->v[1]);
	g->v[2] = (c0->v[2] - (c0-1)->v[2]);
    }

    /* compute the produce of data and spatial gradients */
    sx = work;
    sy = sx + usize*vsize;
    sz = sy + usize*vsize;

    g = dg_slice0;

    for (j=0;j<vsize;j++)
    {
	for (i=0;i<usize;i++)
	{
	    g->v[0] = g->v[0]*sx->v[0] + g->v[0]*sx->v[1] + g->v[0]*sx->v[2];
	    g->v[1] = g->v[1]*sy->v[0] + g->v[1]*sy->v[1] + g->v[1]*sy->v[2];
	    g->v[2] = g->v[2]*sz->v[0] + g->v[2]*sz->v[1] + g->v[2]*sz->v[2];
	}
    }
    

    /* now, normalize them all */
    g = dg_slice0;
    for (j=0;j<vsize;j++)
	for (i=0;i<usize;i++)
	{
	    t = g->v[0]*g->v[0] + g->v[1]*g->v[1] + g->v[2]*g->v[2];
	    if (t != 0.)
	    {
		t = ksqrt(t);
		t = flip_sign/t;
		g->v[0] *= t;
		g->v[1] *= t;
		g->v[2] *= t;
	    }
	    g++;
	}
}
#endif



static void
malloc_slice(octmesh_slice *slicep,
	     int u,
	     int v,
	     int use_map)
{
    int i,t;
    vertex *n2;
    float *d2;
    float *fx,*fy,*fz;
    surface_colors_rgba *rgba;
#if 0
    byte_surface_colors_index *b; /* doesn't work quite right.. */
#endif
    byte_surface_colors_rgba *b;

    slicep->usize = u;
    slicep->vsize = v;

    if (use_map)
    {
	b = (byte_surface_colors_rgba *)kmalloc(sizeof(byte_surface_colors_rgba)*u*v);
	slicep->colors.urgba = (byte_surface_colors_rgba **)kmalloc(sizeof(byte_surface_colors_rgba *)*v);
#if 0
	b = (byte_surface_colors_index *)kmalloc(sizeof(byte_surface_colors_index)*u*v);
	slicep->colors.bindex = (byte_surface_colors_index **)kmalloc(sizeof(byte_surface_colors_index *)*v);
#endif
    }
    else
    {
        rgba = (surface_colors_rgba *)kmalloc(sizeof(surface_colors_rgba)*u*v);
	slicep->colors.rgba = (surface_colors_rgba **)kmalloc(sizeof(surface_colors_rgba *)*v);
    }

    n2 = (vertex *)kmalloc(sizeof(vertex)*u*v);
    slicep->normals.normals_v = (vertex **)kmalloc(sizeof(vertex *)*v);

    fx = (float *)kmalloc(sizeof(float)*u*v);
    slicep->xpts = (float **)kmalloc(sizeof(float *)*v);

    fy = (float *)kmalloc(sizeof(float)*u*v);
    slicep->ypts = (float **)kmalloc(sizeof(float *)*v);

    fz = (float *)kmalloc(sizeof(float)*u*v);
    slicep->zpts = (float **)kmalloc(sizeof(float *)*v);

    d2 = (float *)kmalloc(sizeof(float)*u*v);
    slicep->data = (float **)kmalloc(sizeof(float *)*v);
    
    for (i=0,t=0;i<v;i++,t+=u)
    {
	if (use_map)
/*	    slicep->colors.bindex[i] = b + t;*/
	    slicep->colors.urgba[i] = b+t;
	else
	    slicep->colors.rgba[i] = rgba + t;
	
	slicep->normals.normals_v[i] = n2 + t; 
	slicep->xpts[i] = fx + t;
	slicep->ypts[i] = fy + t;
	slicep->zpts[i] = fz + t;
	slicep->data[i] = d2 + t;
#if 0
	slicep->mask[i] = m2 + t;
#endif
    }
}

#if 0
static void
free_slice(slicep)
octmesh_slice *slicep;
{
    kfree(slicep->colors.rgba[0]);
    kfree(slicep->normals.normals_v[0]);
    kfree(slicep->xpts[0]);
    kfree(slicep->ypts[0]);
    kfree(slicep->zpts[0]);
    kfree(slicep->data[0]);

    kfree(slicep->colors.rgba);
    kfree(slicep->normals.normals_v); 
    kfree(slicep->xpts);
    kfree(slicep->ypts);
    kfree(slicep->zpts);
    kfree(slicep->data);
}
#endif

static void
load_slice(octmesh_slice *slicep,
	   int w,
	   int u,
	   int v,
	   kobject input)
{
    /* grab a plane of data */
    kpds_set_attribute(input,KPDS_VALUE_POSITION,0,0,w,0,0);
    kpds_get_data(input,KPDS_VALUE_PLANE,(float *)&(slicep->data[0][0]));

    /* grab a plane of coordinates */
    kpds_set_attribute(input,KPDS_LOCATION_POSITION,0,0,w,0);
    kpds_get_data(input,KPDS_LOCATION_PLANE,(float *)&(slicep->xpts[0][0]));
    
    kpds_set_attribute(input,KPDS_LOCATION_POSITION,0,0,w,1);
    kpds_get_data(input,KPDS_LOCATION_PLANE,(float *)&(slicep->ypts[0][0]));

    kpds_set_attribute(input,KPDS_LOCATION_POSITION,0,0,w,2);
    kpds_get_data(input,KPDS_LOCATION_PLANE,(float *)&(slicep->zpts[0][0]));
}

static void
compute_slice_normals(octmesh_slice *prev,
		      octmesh_slice *this1,
		      octmesh_slice *next,
		      int usize,
		      int vsize,
		      int flip_normals)
{
   int i,j;

   for (j=0;j<vsize;j++)
      for (i=0;i<usize;i++)
	{
	   compute_normal(&(this1->normals.normals_v[j][i]),
			  i,j,0,usize,vsize,prev,this1,next,NULL,
			  flip_normals);
	}
}


static void
compute_normal(vertex *norm,
	       int ui,
	       int vi,
	       int wi,
	       int usize,
	       int vsize,
	       octmesh_slice *slice_prev,
	       octmesh_slice *slice0,
	       octmesh_slice *slice1,
	       octmesh_slice *slice_next,
	       int flip_normals)
{
    int i_minus,i_plus,j_minus,j_plus;
    vertex data,geom;
    octmesh_slice *sp0,*sp1,*sp2;
    double mag;
    double sign;
    
    if (wi == 0)
    {
	sp0 = slice_prev;
	sp1 = slice0;
	sp2 = slice1;
    }
    else
    {
	sp0 = slice0;
	sp1 = slice1;
	sp2 = slice_next;
    }

    i_plus = (ui==usize-1) ? usize-1 : ui+1;
    i_minus = (ui == 0) ? 0 : ui-1;
    j_plus = (vi == vsize-1) ? vsize-1 : vi+1;
    j_minus = (vi == 0) ? 0 : vi-1;

    data.x = sp1->data[vi][i_plus] - sp1->data[vi][i_minus];
    data.y = sp1->data[j_plus][ui] - sp1->data[j_minus][ui];
    data.z = sp2->data[vi][ui] - sp0->data[vi][ui];
    
    geom.x = sp1->xpts[vi][i_plus] - sp1->xpts[vi][i_minus];
    geom.y = sp1->ypts[j_plus][ui] - sp1->ypts[j_minus][ui];
    geom.z = sp2->zpts[vi][ui] - sp0->zpts[vi][ui];

    if (flip_normals)
	sign = -1.;
    else
	sign = 1.;
    
    if (geom.x != 0.)
	norm->x = data.x / geom.x;
    else
	norm->x = 0.;

    if (geom.y != 0.)
	norm->y = data.y / geom.y;
    else
	norm->y = 0.;

    if (geom.z != 0.)
	norm->z = data.z / geom.z;
    else
	norm->z = 0.;

    mag = norm->x * norm->x + norm->y * norm->y + norm->z * norm->z;
    if (mag != 0.)
    {
	mag = ksqrt(mag);
	mag = 1./mag;
	norm->x *= mag * sign;
	norm->y *= mag * sign;
	norm->z *= mag * sign;
    }
}

static void
dump_slice(kgeom_object *kobj,
	   octmesh_slice *s,
	   int usize,
	   int vsize,
	   int use_map,
	   int have_normals,
	   vertex *w,
	   int dval)
{
    register float **rx,**ry,**rz;
    register int i,j;
    register vertex *rw;


    rw = w;
    rx = s->xpts;
    ry = s->ypts;
    rz = s->zpts;
    
    /* reformat the vertex information for output */
    for (j=0;j<vsize;j++)
    {
	for (i=0;i<usize;i++)
	{
	    rw->x = rx[j][i];
	    rw->y = ry[j][i];
	    rw->z = rz[j][i];

	    rw++;
	}
    }

    kgeom_write_octmesh_plane(kobj, usize, vsize, (float *) w, 
			      (kubyte *)&(s->colors.urgba[0][0]),
			      (float *)&(s->normals.normals_v[0][0]),
			      NULL);
#if 0

    kgeom_set_attribute(kobj,KGEOM_OCTMESH,KGEOM_OCTMESH_LOCATION_POSITION,0,0,dval);
    kgeom_put_data(kobj,KGEOM_OCTMESH_LOCATION_PLANE,w);

    /* stuff the normals */
    if (have_normals)
    {
	kgeom_set_attribute(kobj,KGEOM_OCTMESH,KGEOM_OCTMESH_NORMAL_POSITION,
			    0,0,dval);
	kgeom_put_data(kobj,KGEOM_OCTMESH_NORMAL_PLANE,
		       (float *)&(s->normals.normals_v[0][0]));
    }

    /* stuff the colors */
    if (use_map)
    {
	/* output colors are colormap-indexed. */
	kgeom_set_attribute(kobj,KGEOM_OCTMESH,KGEOM_OCTMESH_COLOR_POSITION,
			    0,0,dval);
	kgeom_put_data(kobj,KGEOM_OCTMESH_COLOR_PLANE,
		       &(s->colors.bindex[0][0]));
    }
    else
    {
	/* output colors are floating point RGBA tuples. */
	kgeom_set_attribute(kobj,KGEOM_OCTMESH,KGEOM_OCTMESH_COLOR_POSITION,
			    0,0,dval);
	kgeom_put_data(kobj,KGEOM_OCTMESH_COLOR_PLANE,
		       (float *)&(s->colors.rgba[0][0]));
    }
#endif
}

int
colorize_slice(octmesh_slice *slice,
	       vis_cmap *cmap,
	       int usize,
	       int vsize)
{
    register vis_cmap *rcmap;
    register float *rdata;
    register float *r,*g,*b,*a;
    register int stride=4;
    int i,j,index;

    rcmap = cmap;
    rdata = slice->data[0];
    r = &(slice->colors.rgba[0][0].r);
    g = &(slice->colors.rgba[0][0].g);
    b = &(slice->colors.rgba[0][0].b);
    a = &(slice->colors.rgba[0][0].a);
    
    for (j=0;j<vsize;j++)
    {
	for (i=0;i<usize;i++)
	{
	    index = lget_map_index(rcmap,rdata);
	    lget_map_rgba(cmap,index,r,g,b,a);

	    r += stride;
	    g += stride;
	    b += stride;
	    a += stride;
	    rdata++;
	}
    }

    return TRUE;
}

int
compute_map_indices(octmesh_slice *slice,
		    vis_cmap *cmap,
		    int usize,
		    int vsize)
{
    register vis_cmap *rcmap;
    register float *rdata;
    register byte_surface_colors_rgba *bubba;

    int i,j,index;
    float r,g,b,a;

    rcmap = cmap;
    rdata = slice->data[0];

    bubba = &(slice->colors.urgba[0][0]);
    
    for (j=0;j<vsize;j++)
    {
	for (i=0;i<usize;i++)
	{
	    /* note: here was have the potential for overflow if the
	     size of the colormap is > 255. this condition should be
	     caught as part of the setup for processing for this kroutine. */
	    
	    index = lget_map_index(rcmap,rdata);
	    lget_map_rgba(cmap,index,&r,&g,&b,&a);
	    bubba->r = (unsigned char)(r*255.0);
	    bubba->g = (unsigned char)(g*255.0);
	    bubba->b = (unsigned char)(b*255.0);
	    bubba->a = (unsigned char)(a*255.0);
	    bubba++;

#if 0
	    r += stride;
	    g += stride;
	    b += stride;
	    a += stride;
#endif
	    rdata++;
	}
    }

    return TRUE;
}
/* -library_code_end */
