 /*
  * Khoros: $Id: lgtubeness.c,v 1.13.2.3 1997/05/31 18:59:02 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lgtubeness.c,v 1.13.2.3 1997/05/31 18:59:02 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for gtubeness
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lgtubeness
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
int build_scaled_unit_circle PROTO((float *, float *, int , float *));
int build_tube PROTO((vertex_3d *, vertex_3d *, int, vertex_3d *, vertex_3d *,
		      float *, float *, float *_colors, int, int, float *));

int transform_yaxis_to_abc PROTO((vertex_3d *, matrix4x4 *));
int make_strip PROTO((vertex_3d *, vertex_3d *, vertex_3d *, int));
int load_colors PROTO((float *, float *, int, int, int));
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lgtubeness - replace lines with tubes.
* 
*       Purpose: A Geometry filter which will replaces all line segments
*                in a geometry object with tubes.  Non-line-segment objects
*                will be passed to the output object unchanged.
*
*         Input: geom - the input geometry object.
*                float scale - radius of all output tubes.
*                int subs - a value controlling the number of sides for
*                    each tube.  the range of this value is 2<=N<=6, and
*                    the number of sides on each tube is computed as:
*                    2<<(N-1).
*                int cap_style - indicates how the tubes should be capped.
*	             1 - no cap, a tube without ends
*                    2 - flat caps, no mitering (not working)
*		     3 - round caps, spheres at the ends
*
*        Output: geom - the input geometry object with lines replaced
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: Restrictions on data or input as applicable
*    Written By: wes
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: note the relationship between the cap_style variable
*                and the list of styles in the .pane file.
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lgtubeness(kgeom_object *geom,
	       float scale,
	       int subs,
	       int cap_style)
/* -library_def_end */

/* -library_code */
{
    int p, i, num_prims, status;
    int nsides_per_tube,ntriangles_per_tube,nverts_per_tube;
    vertex_3d bmin,bmax,center,lbmin,lbmax;
    vertex_3d *work_verts,*work_norms;
    float *ctable,*stable;
    float *work_colors;

    num_prims = kgeom_number_primitives(geom);

    /*-- start with the old bounding box --*/
    for (i=0;i<3;i++)
    {
       bmin.v[i] = geom->min[i];  /* was KGEOM_BIGNUM; */
       bmax.v[i] = geom->max[i];  /* was -1. * KGEOM_BIGNUM; */
    }

    nsides_per_tube = 2<<(subs-1);
    ntriangles_per_tube = nsides_per_tube * 2;
    nverts_per_tube = ntriangles_per_tube + 2;

    work_verts = (vertex_3d *)kmalloc((unsigned)sizeof(vertex_3d)*nverts_per_tube);
    work_norms = (vertex_3d *)kmalloc((unsigned)sizeof(vertex_3d)*nverts_per_tube);

    if (geom->has_alpha)
	work_colors = (float *)kmalloc((unsigned)sizeof(float)*nverts_per_tube*4);
    else
	work_colors = (float *)kmalloc((unsigned)sizeof(float)*nverts_per_tube*3);

    ctable = (float *)kmalloc((unsigned)sizeof(float)*nsides_per_tube);
    stable = (float *)kmalloc((unsigned)sizeof(float)*nsides_per_tube);

    build_scaled_unit_circle(ctable,stable,nsides_per_tube,&scale);
    
    for (p=0;p<num_prims;p++)
    {
       kgeom_primitive *prim = kgeom_get_primitive(geom, p);

       switch (prim->type)
       {
	  case KGEOM_POLYLINE_DISJOINT:
	     {
		kgeom_polyline *lines = (kgeom_polyline *) prim;
		int in_nverts,in_segs;
		int has_color,next_color_step;
		float *c1;
		int color_stride;
		float *verts=NULL,*colors=NULL;
		vertex_3d *v1,*v2,*s;

		in_nverts = lines->nverts;
		verts  = lines->locs;
		colors = lines->cols;
		
		in_segs = in_nverts/2;

		s = (vertex_3d *)verts;
		v1 = s;
		v2 = v1+1;

		if (colors == NULL)
		{
		    has_color = 0;
		    color_stride = 0;
		    c1 = NULL;
		}
		else
		{
		    has_color = 1;
		    c1 = colors;
		    if (geom->has_alpha)
			color_stride = 4;
		    else
			color_stride = 3;
		}
		next_color_step = (geom->layout == KPER_VERTEX) ?
		    color_stride * 2 : color_stride;
		
		for (i=0;i<in_segs;i++,v1+=2,v2+=2,c1+=next_color_step)
		{
		    status = build_tube(v1,v2,nsides_per_tube,work_verts,
					work_norms,ctable,stable,c1,
					color_stride,geom->layout,work_colors);
		    
		    if (status == CHILL)
		    {
		       kgeom_triangles tri;
		       
		       tri.type    = KGEOM_TRIANGLES_CONNECTED;
		       tri.nverts  = nverts_per_tube;
		       tri.texture = NULL;
		       tri.locs    = (float *) work_verts;
		       tri.norms   = (float *) work_norms;

		       if (has_color)
			  tri.cols = (float *) work_colors;
		       else
			  tri.cols = NULL;

		       kgeom_write_primitive(geom, (kgeom_primitive *)&tri);

		       lcompute_bbox((vertex_3d *)work_verts,nverts_per_tube,
				     &lbmin,&lbmax);
		       lunion_boxes(&lbmin,&lbmax,&bmin,&bmax,&bmin,&bmax);
		    }
		}

		if (cap_style == 3) {
		       kgeom_primitive *prim;
		       int j;
		       float *radius;

		       prim = kgeom_new_primitive(KGEOM_SPHERES);

		       radius = (float *)kmalloc(in_nverts*sizeof(*radius));

		       for (j = 0; j < in_nverts; j++) {
			 radius[j] = scale;
		       }

		       prim->spheres.nverts = in_nverts;
		       prim->spheres.locs = verts;
		       prim->spheres.cols = colors;
		       prim->spheres.radii = radius;

		       kgeom_write_primitive(geom,prim);

		       kfree(radius);
		}
	     }
	     break;
	  case KGEOM_POLYLINE_CONNECTED:
	     {
		kgeom_polyline *lines = (kgeom_polyline *) prim;
		int in_nverts,in_segs;
		int has_color;
		float *verts=NULL,*colors=NULL;
		vertex_3d *v1,*v2,*s;
		float *c1;
		int color_stride;

		in_nverts = lines->nverts;
		verts  = lines->locs;
		colors = lines->cols;
		
		in_segs = in_nverts-1;

		s = (vertex_3d *)verts;
		v1 = s;
		v2 = v1+1;

		if (colors == NULL)
		{
		    c1 = NULL;
		    has_color = 0;
		    color_stride = 0;
		}
		else
		{
		    has_color = 1;
		    if (geom->has_alpha)
			color_stride = 4;
		    else
			color_stride = 3;
		    c1 = colors;
		}
		
		for (i=0;i<in_segs;i++,v1=v2,v2+=1, c1+=color_stride)
		{
		    status = build_tube(v1,v2,nsides_per_tube,work_verts,
					work_norms,ctable,stable,c1,
					color_stride,geom->layout,work_colors);
		    
		    if (status == CHILL)
		    {
		       kgeom_triangles tri;

		       tri.type    = KGEOM_TRIANGLES_CONNECTED;
		       tri.nverts  = nverts_per_tube;
		       tri.texture = NULL;
		       tri.locs    = (float *) work_verts;
		       tri.norms   = (float *) work_norms;

		       if (has_color)
			  tri.cols = (float *) work_colors;
		       else
			  tri.cols = NULL;

		       kgeom_write_primitive(geom, (kgeom_primitive *)&tri);

		       lcompute_bbox((vertex_3d *)work_verts,nverts_per_tube,
				     &lbmin,&lbmax);
		       lunion_boxes(&lbmin,&lbmax,&bmin,&bmax,&bmin,&bmax);
		    }
		}

		if (cap_style == 3) {
		       kgeom_primitive *prim;
		       int j;
		       float *radius;

		       prim = kgeom_new_primitive(KGEOM_SPHERES);

		       radius = (float *)kmalloc(in_nverts*sizeof(*radius));

		       for (j = 0; j < in_nverts; j++) {
			 radius[j] = scale;
		       }

		       prim->spheres.nverts = in_nverts;
		       prim->spheres.locs = verts;
		       prim->spheres.cols = colors;
		       prim->spheres.radii = radius;

		       kgeom_write_primitive(geom,prim);

		       kfree(radius);
		}
	     }

	     break;
	  default:
	     /*-- just write out the primitive we have --*/

	     kgeom_write_primitive(geom, prim);
	     break;
	}
    }

    if (geom->has_bounding_box)
    {
       kmemcpy(geom->min, &bmin.v[0], sizeof(float)*3);
       kmemcpy(geom->max, &bmax.v[0], sizeof(float)*3);
    }

    if (geom->has_center)
       for (i=0;i<3;i++)
	  geom->center[i] = bmin.v[i] + 0.5 * (bmax.v[i] - bmin.v[i]);
    
    kfree(work_verts);
    kfree(work_norms);
    kfree(ctable);
    kfree(stable);
	
    return TRUE;
}

int
build_tube(vertex_3d *v1,
	   vertex_3d *v2,
	   int nsides_per_tube,
	   vertex_3d *work_verts,
	   vertex_3d *work_norms,
	   float *ctable,
	   float *stable,
	   float *in_colors,
	   int color_stride,
	   int layout,
	   float *work_colors)
{
    vertex_3d *e1,*e2,*n1,*n2;
    matrix4x4 m;
    vertex_3d unitvec,dirvec;
    double mag;
    int i;
    int status;

    e1 = (vertex_3d *)kmalloc((unsigned)sizeof(vertex_3d)*nsides_per_tube);
    e2 = (vertex_3d *)kmalloc((unsigned)sizeof(vertex_3d)*nsides_per_tube);
    n1 = (vertex_3d *)kmalloc((unsigned)sizeof(vertex_3d)*nsides_per_tube);
    n2 = (vertex_3d *)kmalloc((unsigned)sizeof(vertex_3d)*nsides_per_tube);
    
    /* compute the unit direction vector of the tube. */
    for (i=0;i<3;i++)
	dirvec.v[i] = v2->v[i] - v1->v[i];

    mag = 0.;
    for (i=0;i<3;i++)
	mag += dirvec.v[i]*dirvec.v[i];
    mag = ksqrt(mag);

    /* if the height of the tube is zero, bail out */
    if (mag != 0.)
    {
	mag = 1./mag;
	for (i=0;i<3;i++)
	    unitvec.v[i] = dirvec.v[i]* mag;

	/* compute the transformation that will allow us to take points
	   which lie in the x-z plane and transform them into a plane
	   with a normal that corresponds to the direction of the tube. */
	
	transform_yaxis_to_abc(&unitvec,&m);

	/* load up some points, the scaled unit circle from the precomputed
	   sin/cos tables, into a vertex array.  we'll have a circle that
	   lies in the x-z plane. */
	 
	for (i=0;i<nsides_per_tube;i++)
	{
	    e1[i].v[0] = ctable[i];
	    e1[i].v[1] = 0.;
	    e1[i].v[2] = stable[i];
	}

	/* rotate the circle so that it lies in a plane whose normal is
	   the direction of the tube. */
	
	npoint_xfrm((KGEOM_VERTEX_TYPE *)&(e1[0].v[0]),
		    &m,
		    (KGEOM_VERTEX_TYPE *)&(e2[0].v[0]),
		    nsides_per_tube);

	kmemcpy(n1,e2,(unsigned)(sizeof(vertex_3d)*nsides_per_tube));
	kmemcpy(n2,e2,(unsigned)(sizeof(vertex_3d)*nsides_per_tube));

	/* e2 now has the rotated circle.  translate this circle to
	   the point in space of one end of the tube; v1. put the
	   results into e1. */
	
	npoint_add((KGEOM_VERTEX_TYPE *)&(e2[0].v[0]),
		   (KGEOM_VERTEX_TYPE *)&(v1->v[0]),
		   (KGEOM_VERTEX_TYPE *)&(e1[0].v[0]),
		   nsides_per_tube);

	/* translate e2 to the other end of the tube. */
	npoint_add((KGEOM_VERTEX_TYPE *)&(e2[0].v[0]),
		   (KGEOM_VERTEX_TYPE *)&(v2->v[0]),
		   (KGEOM_VERTEX_TYPE *)&(e2[0].v[0]),
		   nsides_per_tube);

	/* ok, now we have two circle's worth of points which
	   form the ends of the tube.  now, load up the destination
	   arrays with points in the right order so as to correspond
	   to t-strips. */

	make_strip(e1,e2,work_verts,nsides_per_tube);
	make_strip(n1,n2,work_norms,nsides_per_tube);

	if (in_colors != NULL)
	    load_colors(work_colors,in_colors,nsides_per_tube,layout,
			color_stride);
	
	/* build_cylinder_surface(e1,e2,nsides_per_tube,work_verts,c1,c2,
			       colortable,n1,n2,work_norms);  */
	
	status = CHILL;
    }
    else
    {
	/* a degenerate condition */
	status = WHACKED;
    }

    kfree(e1);
    kfree(e2);
    kfree(n1);
    kfree(n2);
    return(status);
}

#define MY_PI 3.1415927
int
build_scaled_unit_circle(float *ctable,
			 float *stable,
			 int n,
			 float *scale)
{
    int i;
    double dscale,t,dt,t2;

    dscale = *scale;
    
    dt = (2. * MY_PI)/n;
    t = 0;
    for (i=0;i<n;i++,t+=dt)
    {
	t2 = kcos(t);
	ctable[i] = t2 * dscale;
	t2 = ksin(t);
	stable[i] = t2 * dscale;
    }
    return(CHILL);
}

int
transform_yaxis_to_abc(vertex_3d *abc,matrix4x4 *m)
{
    /**
      * this routine will compute a transformation matrix M which will allow
      * one to transform points which lie in the x-z plane into a plane
      * with the normal "abc".
    **/

       /**
      * the basic idea is that we have a unit circle (scaled to the
      * appropriate radius) which lies in the XZ plane.  this plane
      * has a normal of [0,1,0].  we want to figure out how to rotate
      * the plane containing this circle into the plane which has a
      * normal, which is the same as the direction formed by the
      * input vector.
      *
      * the derivation for the rotations goes something like this:
      *
      * we compute the unit vector from the input vector, and call
      * it (A,B,C).  take the cross product of this vector and [0,1,0]
      * and we end up with (C,0,-A).  the cross product forms the
      * axis of rotation.  the angle of rotation about this axis is
      * the dot product of [A,B,C] and [0,1,0]
      *
      * after some hand waving, we end up with the following transformation
      * matrix (thanks go to Terry Ligocki of LBL for his help):
 
      |                                              |
      |  A^2 * B + C^2                 A*B*C - A*C   |
      |  -------------        A        -----------   |
      |    A^2 + C^2                    A^2 + C^2    |
      |                                              |
T =   |       -A              B            -C        |
      |                                              |
      |   A*B*C - A*C                 B * C^2 + A^2  |
      |   -----------         C       -------------  |
      |    A^2 + C^2                    A^2 + C^2    |
      |                                              |
 
      This is assuming that a point, p, is a column vector and the transformed
      point is T*p (i.e. the matrix multiples on the left).
 
      *
    **/

    double a2c2;
    double m11,m12,m13,m31,m32,m33;
    
    identity_4x4(m);

    a2c2 = abc->v[0]*abc->v[0] + abc->v[2]*abc->v[2];

    if (a2c2 == 0.)  /* a degenerate condition; make an identity matrix */
    {
        m11 = 1.;
        m12 = 0.;
        m13 = 0.;
 
        m31 = 0.;
        m32 = 0.;
        m33 = 1.;
 
    }
    else
    {
        a2c2 = 1./a2c2;
 
/*        m11 = tp.x*tp.x*tp.y + tp.z*tp.z; */
	m11 = abc->v[0]*abc->v[0]*abc->v[1] + abc->v[2]*abc->v[2];
        m11 *= a2c2;
 
/*        m12 = -1. * tp.x; */
	m12 = -1. * abc->v[0];
 
/*        m13 = tp.x * tp.y * tp.z - tp.x * tp.z; */
/* Changed line below to fix typo, Terry Ligocki:
 *	m13 = abc->v[0] * abc->v[1] * abc->v[2] - abc->v[2]*abc->v[2];
 */
	m13 = abc->v[0] * abc->v[1] * abc->v[2] - abc->v[0]*abc->v[2];
	
        m13 *= a2c2;
 
        m31 = m13;
 
/*        m32 = -1. * tp.z; */
	m32 = -1. * abc->v[2];
 
/*        m33 = tp.y * tp.z * tp.z + tp.x * tp.x; */
	m33 = abc->v[1] * abc->v[2] * abc->v[2] + abc->v[0] * abc->v[0];
        m33 *= a2c2;
    }

    m->m[0][0] = m11;
    m->m[0][1] = m12;
    m->m[0][2] = m13;
    m->m[2][0] = m31;
    m->m[2][1] = m32;
    m->m[2][2] = m33;
   
    return(CHILL);
}

int
make_strip(vertex_3d *s1,
	   vertex_3d *s2,
	   vertex_3d *d,
	   int n)
{
    int offset,i;
    /**
      * we're given two arrays of things, s1 and s2, each of which is
      * N long, and we want to scramble them into a new array D, which
      * better be N*2+2 long, so as to make a triangle strip from s1 and s2.
    **/
    offset = 0;
    for (i=0;i<n;i++)
    {
	kmemcpy(d+offset,s2+i,(unsigned)sizeof(vertex_3d));
	offset++;
	kmemcpy(d+offset,s1+i,(unsigned)sizeof(vertex_3d));
	offset++;
    }
    kmemcpy(d+offset,s2,(unsigned)sizeof(vertex_3d));
    offset++;
    kmemcpy(d+offset,s1,(unsigned)sizeof(vertex_3d));

    return(CHILL);
}
	
int
load_colors(float *d,
	    float *s,
	    int n,
	    int layout,
	    int stride)
{
    int offset,i;
    float even[4],odd[4];
    unsigned int tsize;

    tsize = sizeof(float)*stride;

    offset = 0;

    kmemcpy(even,s,tsize);
    if (layout == KPER_VERTEX)
	kmemcpy(odd,s+stride,tsize);
    else
	kmemcpy(odd,even,tsize);
    
    for (i=0;i<n;i++)
    {
	kmemcpy(d+offset,even,tsize);
	offset += stride;
	kmemcpy(d+offset,odd,tsize);
	offset += stride;
    }
    kmemcpy(d+offset,even,tsize);
    offset += stride;
    kmemcpy(d+offset,odd,tsize);

    return(CHILL);
}
/* -library_code_end */
