
/*
 * Copyright (c) 1995-1997,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 
/*
 * Khoros: $Id: lresample.c,v 1.1.2.1 1997/06/28 23:26:59 mirage Exp $
 * created: Fri Apr  4 13:56:56 PST 1997
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lresample.c,v 1.1.2.1 1997/06/28 23:26:59 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */



#include "internals.h"

/* static void lerp PROTO((float *s,float *d,int sn,int dn)); */

static void grab_column PROTO((float *dst,float *src, int source_width,int source_height, int source_depth, int source_time, int source_elements, int at_width, int at_depth,int at_time,int at_elements,int num));

static void stuff_column PROTO((float *src,float *dst,int dest_width,int dest_height,int dest_time,int dest_depth,int dest_elements, int at_width,int at_depth,int at_time,int at_elements,int num));


static void grab_depths PROTO((float *dst,float *src,int source_width,int source_height,int source_depth,int source_time, int source_elements,int at_width,int at_height,int at_time,int at_elements,int num));

static void stuff_depths PROTO((float *src,float *dst,int dest_width,int dest_height,int dest_time,int dest_depth, int dest_elements,int at_width, int at_height, int at_time, int at_elements,int num));

static void grab_times PROTO((float *dst,float *src,int source_width,int source_height,int source_depth,int source_time, int source_elements,int at_width, int at_height,  int at_depth,int at_elements,int num));

static void stuff_times PROTO((float *src,float *dst,int dest_width,int dest_height, int dest_time,int dest_depth,int dest_elements,int at_width,int at_height,int at_depth,int at_elements, int num));

int
lresample_memory(float *src,
		 float *dest,
		 int size_in[5], /* w,h,d,t,e */
		 float size_out[5],/* w,h,d,t (same e as in) */
		 int elements_first) 
{
    /*
     * this routine will do a first-order resampling of one 5-d
     * array into another 5-d array.  the sizes of each are given
     * as arrays (parameters).  each array is stored as a flat
     * array...it's really not a 5d array.
     *
     * on input:
     *
     * src[] points to the floating point input data. it's a
     *       flattened 5d array.
     * dest[] points to a user-supplied array which serves as the
     *       landing zone for the computed resampling. it had better
     *       be size_out[0]*size_out[1]*..*size_out[4] in size.
     * size_in[] - integer array giving the size of each dimension of input.
     * size_out[] - floating point array giving the size of each
     *       dimension of output.
     * elements_first - use this to indicate if the data is stored
     *       RRRGGGBBB (elements_first == TRUE) or RGBRGB
     *       (elements_first == FALSE)
     */
    
    int sw,sh,sd,st,se;		/* dimensions of source object */
    int dw,dh,dd,dt;		/* dimensions of dest object */
    int i,j,k,l,e;		/* misc loop indices */
    int tw,th,td,tt;		/* dimensions of temp object  */
    float *fs,*fd,*pstat;		/* misc work  */
    float *core;
    int src_index,dest_index;
    int twh_size,twhd_size,twhdt_size;
    float *tmp;
    int tmp_index;
    int save_in_format;
    float *new_src;

    sw = size_in[0];
    sh = size_in[1];
    sd = size_in[2];
    st = size_in[3];
    se = size_in[4];

    dw = size_out[0];
    dh = size_out[1];
    dd = size_out[2];
    dt = size_out[3];

    /* grab a bunch of memory for doing all the work */

    tw = MAX(dw,sw);
    th = MAX(dh,sh);
    td = MAX(dd,sd);
    tt = MAX(dt,st);

    twh_size= tw*th;
    twhd_size = twh_size * td;
    twhdt_size = twhd_size * tt;

    core = (float *)kmalloc(sizeof(float)*tw*th*td*tt*se); /* ouch! */
    kmemset(core,0,sizeof(float)*tw*th*td*tt*se);

    src_index = 0;
    dest_index = 0;

    /* if elements_first is true, unscramble into a holding array (more
     memory) such that elements_first becomes false. */
    if (elements_first == TRUE)
    {
	int w,h,d,t,e;
	int index=0;
	int base;
	int nw,nwh,ndwh,ntdwh;
	int stride;

	nw = sw*se;
	nwh = nw * sh;
	ndwh = nwh * sd;
	ntdwh = ndwh * st;

	new_src = (float *)kmalloc(sizeof(float)*sw*sh*sd*st*se); /* ouch! */
	stride = se;
	for (e=0;e<se;e++)
        {
	    for (t=0;t<st;t++)
	    {
		for (d=0;d<sd;d++)
		{
		    for (h=0;h<sh;h++)
		    {
			base = t * ndwh + d * nwh + h * nw + e;
			for (w=0;w<sw;w++)
			{
			    new_src[index++] =
				src[base+w*stride];
			}
		    }
		}
	    }
	}
	src = new_src;
    }

    fd = (float *)kmalloc(sizeof(float)*tw);
    for (e=0;e<se;e++)		/* loop over source #elements */
    {
	for (k=0;k<st;k++)	/* loop over source time */
	{
	    for (l=0;l<sd;l++)	/* loop over source depth */
	    {
		dest_index = e*twhdt_size + k*twhd_size + l*twh_size;
		for (j=0;j<sh;j++) /* loop over source HEIGHT */
		{
		    fs = src + src_index;
		    kmemset(fd,0,sizeof(float)*tw); /* temporary */
		    lerp(fs,fd,sw,dw);
		    kmemcpy(core+dest_index,fd,sizeof(float)*dw); 
		    src_index += sw;
		    dest_index += tw;
		}
	    }
	}
    }
    kfree(fd);
    /* kfree(fs); don't free -- this is pointer into other space */
    
    if (elements_first == TRUE)
    {
	/* we can release the memory now.  it's contents have been
	 copied into "core". */
	kfree(new_src);
    }

    /* next comes height.  if h == 1, we can just skip this step.
       we'll loop over (source) time, (source) elements and dest width. */

    if (dh != sh)
    {
	fd = (float *)kmalloc(sizeof(float)*dh);
	fs = (float *)kmalloc(sizeof(float)*sh);
	for (e=0;e<se;e++)		/* loop over source #elements */
	{
	    for (k=0;k<st;k++)	/* loop over source time */
	    {
		for (l=0;l<sd;l++)	/* loop over source depth */
		{
		    for (j=0;j<dw;j++) /* loop over dest width */
		    {
			/* grab a row's worth of data */
			grab_column(fs,core,tw,th,td,tt,se,j,l,k,e,sh);
			
			/* lerp the data */
			kmemset(fd,0,sizeof(float)*dh); /* temporary */
			lerp(fs,fd,sh,dh); 

			stuff_column(fd,core,tw,th,td,tt,se,j,l,k,e,dh);
		    }
		}
	    }
	}
	kfree(fd);
	kfree(fs);
    }
    

    /* next comes depth.  if d == 1, we can just skip this step.
       we'll loop over (source) time, (source) elements and dest width. */

    if (dd != sd)
    {
	fd = (float *)kmalloc(sizeof(float)*dd);
	fs = (float *)kmalloc(sizeof(float)*sd);
	for (e=0;e<se;e++)		/* loop over source #elements */
	{
	    for (k=0;k<st;k++)	/* loop over source time */
	    {
		for (l=0;l<dh;l++)	/* loop over dest height */
		{
		    for (j=0;j<dw;j++) /* loop over dest width */
		    {
			/* grab a row's worth of data */
			grab_depths(fs,core,tw,th,td,tt,se,j,l,k,e,sd);
			
			/* lerp the data */
			kmemset(fd,0,sizeof(float)*dd); /* temporary */
			lerp(fs,fd,sd,dd); 

			stuff_depths(fd,core,tw,th,td,tt,se,j,l,k,e,dd);
		    }
		}
	    }
	}
	kfree(fd);
	kfree(fs);
    }

    /* next comes time.  if t==1 we can skip this step */
    if (dt != st)
    {
	fprintf(stderr," warning: lerp over time in fast mode not yet tested. \n");
	fd = (float *)kmalloc(sizeof(float)*dt);
	fs = (float *)kmalloc(sizeof(float)*st);
	for (e=0;e<se;e++)		/* loop over source #elements */
	{
	    for (k=0;k<dd;k++)	/* loop over dest time */
	    {
		for (l=0;l<dh;l++)	/* loop over dest height */
		{
		    for (j=0;j<dw;j++) /* loop over dest width */
		    {
			/* grab a row's worth of data */
			grab_times(fs,core,tw,th,td,tt,se,j,l,k,e,st);
			
			/* lerp the data */
			kmemset(fd,0,sizeof(float)*dt);
			lerp(fs,fd,st,dt); 

			stuff_times(fd,core,tw,th,td,tt,se,j,l,k,e,dt);
		    }
		}
	    }
	}
	kfree(fd);
	kfree(fs);
    }

    
    /* ok, now we need to create one chunk of memory which will
     hold everything for one big xfer to data services.*/

    tmp = (float *)kmalloc(sizeof(float)*dw*dh*dd*dt*se);
    tmp_index = 0;

    for (e=0;e<se;e++)		/* loop over source #elements */
    {
	for (k=0;k<dt;k++)	/* loop over dest time */
	{
	    for (l=0;l<dd;l++)	/* loop over dest depth */
	    {
		src_index = e*twhdt_size + k*twhd_size + l*twh_size;
		for (j=0;j<dh;j++) /* loop over source HEIGHT */
		{
		    /* copy rows from the big memory are to output xfer
		     memory area. */
		    
		    kmemcpy(tmp+tmp_index,core+src_index,sizeof(float)*dw);
		    tmp_index += dw;
		    src_index += tw;
		}
	    }
	}
    }
    

    /* if elements_first is true, we have to scramble the data (which at
       this point is in elements_first == FALSE format) back into
       element_first == TRUE format. */
    if (elements_first == TRUE)
    {
	register float *rd;
	int w,h,d,t,e;
	int index=0,index2;
	int src_per_row,base;
	int nw,nwh,ndwh,ntdwh;

	nw = dw;
	nwh = dw * dh;
	ndwh = nwh * dd;
	ntdwh = ndwh * dt;

	src_per_row = se*sw;
	src = tmp;
	
        rd = new_src = (float *)kmalloc(sizeof(float)*dw*dh*dd*dt*se);

	for (t=0;t<dt;t++)
	{
	    for (d=0;d<dd;d++)
	    {
		for (h=0;h<dh;h++)
		{
		    index = t*ndwh + d*nwh + h*nw;
		    
		    for (w=0;w<dw;w++)
		    {
			index2 = index + w;
			for (e=0;e<se;e++,index2+=ntdwh)
			    *rd++ = src[index2];
			    
		    }
		}
	    }
	}
	tmp = new_src;
    }

    /* copy stuff back into the dest ptr */
    kmemcpy(dest,tmp,sizeof(float)*(dw*dh*dt*dt*se));

    if (elements_first == TRUE)
	 kfree(new_src);
    
    kfree(core);
    /* kfree(tmp); -- this is a fmr sometimes */
    
    /* adios */
    return(CHILL);
}

void
lerp(float *s,			/* source pixels */
     float *d,			/* dest pixels */
     int sn,			/* source size */
     int dn)			/* dest size */
{
    register int i,index,indexp1;
    register double t,dt;
    register double frac1,frac2;
    register double floor_t;
    
    /* we assume that *d has all been initialized to 0.0 */
    if (sn == dn)
	kmemcpy(d,s,sizeof(float)*sn);
    else if (dn > sn)
    {
	dt = (float)sn/(float)dn;
	t = 0;
	for (i=0;i<dn;i++,t+=dt)
	{
	    floor_t = kfloor(t);
	    
	    index = (int)floor_t;
	    indexp1 = (index+1 >= sn) ? index : index+1;
	    
	    frac2 = t - floor_t; /* portion of next pixel */
	    frac1 = 1. - frac2; /* portion of this pixel */
	    d[i] = frac1 * s[index] + frac2 * s[indexp1];
	}
    }
    else			/* sn > dn */
    {
	int dindex;
	
	t = 0.;
	dt = (double)dn/(double)sn;
	for (i=0;i<sn;i++,t+=dt)
	{
	    index = kfloor(t);
	    dindex = kfloor(t+dt);
	    if (dindex == index) /* the source pixel fits entirely within
				  the dest pixel.*/
		d[dindex] += s[i]*dt;
	    else		/* the source pixel covers parts of two
				 dest pixels */
	    {
		frac1 = 1. - (t - kfloor(t));
		frac2 = (t+dt) - kfloor(t+dt);
		d[index] += frac1 * s[i];
		d[index+1] += frac2 * s[i];
	    }
	}
    }
}

static void
grab_column(float *dst,		/* a linear array of floats */
	    float *src,		/* a big 5d array of floats */
	    int source_width,	/* width size of src */
	    int source_height,	/* height size of src */
	    int source_depth,	/* depth size of src */
	    int source_time,	/* time size of src */
	    int source_elements, /* elements size of src */
	    int at_width,	/* where to grab a column from */
	    int at_depth,	/* at what depth */
	    int at_time,	/* at what time */
	    int at_elements,	/* at what elements */
	    int num)		/* how big is column? */
{
    int i;
    float *s;
    int wh_size,whd_size,whdt_size;

    wh_size = source_width * source_height;
    whd_size = wh_size * source_depth;
    whdt_size = whd_size * source_time;

    s = src + at_width + at_depth * wh_size + at_time * whd_size + at_elements * whdt_size;

    for (i=0;i<num;i++,s += source_width)
	*dst++ = *s;
}

static void
stuff_column(float *src,
	     float *dst,
	     int dest_width,
	     int dest_height,
	     int dest_time,
	     int dest_depth,
	     int dest_elements,
	     int at_width,
	     int at_depth,
	     int at_time,
	     int at_elements,
	     int num)
{
    int i;
    float *d;
    int wh_size,whd_size,whdt_size;

    wh_size = dest_width * dest_height;
    whd_size = wh_size * dest_depth;
    whdt_size = whd_size * dest_time;

    d = dst + at_width + at_depth * wh_size + at_time * whd_size + at_elements * whdt_size;
    for (i=0;i<num;i++,d += dest_width)
	*d = *src++;
}
static void
grab_depths(float *dst,		/* a linear array of floats */
	    float *src,		/* a big 5d array of floats */
	    int source_width,	/* width size of src */
	    int source_height,	/* height size of src */
	    int source_depth,	/* depth size of src */
	    int source_time,	/* time size of src */
	    int source_elements, /* elements size of src */
	    int at_width,	/* where to grab a column from */
	    int at_height,	/* at what height */
	    int at_time,	/* at what time */
	    int at_elements,	/* at what elements */
	    int num)		/* how many depths to grab? */
{
    int i;
    float *s;
    int wh_size,whd_size,whdt_size;

    wh_size = source_width * source_height;
    whd_size = wh_size * source_depth;
    whdt_size = whd_size * source_time;

    s = src + at_width + at_height*source_width + at_time * whd_size + at_elements * whdt_size;

    for (i=0;i<num;i++,s += wh_size)
	*dst++ = *s;
}

static void
stuff_depths(float *src,
	     float *dst,
	     int dest_width,
	     int dest_height,
	     int dest_time,
	     int dest_depth,
	     int dest_elements,
	     int at_width,
	     int at_height,
	     int at_time,
	     int at_elements,
	     int num)
{
    int i;
    float *d;
    int wh_size,whd_size,whdt_size;

    wh_size = dest_width * dest_height;
    whd_size = wh_size * dest_depth;
    whdt_size = whd_size * dest_time;

    d = dst + at_width + at_height * dest_width + at_time * whd_size + at_elements * whdt_size;
    for (i=0;i<num;i++,d += wh_size)
	*d = *src++;
}


static void
grab_times(float *dst,		/* a linear array of floats */
	   float *src,		/* a big 5d array of floats */
	   int source_width,	/* width size of src */
	   int source_height,	/* height size of src */
	   int source_depth,	/* depth size of src */
	   int source_time,	/* time size of src */
	   int source_elements, /* elements size of src */
	   int at_width,	/* where to grab a column from */
	   int at_height,	/* at what height */
	   int at_depth,	/* at what depth */
	   int at_elements,	/* at what elements */
	   int num)		/* how many depths to grab? */
{
    int i;
    float *s;
    int wh_size,whd_size,whdt_size;

    wh_size = source_width * source_height;
    whd_size = wh_size * source_depth;
    whdt_size = whd_size * source_time;

    s = src + at_width + at_height*source_width + at_depth * wh_size + at_elements * whdt_size;

    for (i=0;i<num;i++,s += whd_size)
	*dst++ = *s;
}

static void
stuff_times(float *src,
	    float *dst,
	    int dest_width,
	    int dest_height,
	    int dest_time,
	    int dest_depth,
	    int dest_elements,
	    int at_width,
	    int at_height,
	    int at_depth,
	    int at_elements,
	    int num)
{
    int i;
    float *d;
    int wh_size,whd_size,whdt_size;

    wh_size = dest_width * dest_height;
    whd_size = wh_size * dest_depth;
    whdt_size = whd_size * dest_time;

    d = dst + at_width + at_height * dest_width + at_depth * wh_size + at_elements * whdt_size;
    for (i=0;i<num;i++,d += whd_size)
	*d = *src++;
}
