
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 *
 * Sun Mar 23 20:27:34 PST 1997
 * - added support for transformations of children-of-root objects. look
 *   in build_relative_object_xform(). the "model" matrix is now being honored.
 *   it is up to the calling routine to make sure that the "model" matrix
 *   is set to the transformation matrix of the object-to-be-transformed's
 *   PARENT.  if we're transforming the "root" object, the parent matrix
 *   should be set to the identity matrix (ie, no parent).
 */
 
/*
 * Khoros: $Id: vr_utils.c,v 1.4.2.2 1997/05/02 16:12:33 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vr_utils.c,v 1.4.2.2 1997/05/02 16:12:33 mirage Exp $";
#endif


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vrlibrary
   >>>> 
   >>>>	Private: 
   >>>>	Static: 
   >>>>	Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <geometry.h>
#include "geometry/rmatrix.h"
#include "vrgeometry/vrgeometry.h"

static int build_view_orientation_matrix PROTO((camera_struct *,matrix4x4 *));


int
encode_vrevent(vrevent *event,
	       kgeom_object *vo)
{
  kgeom_vr_event *vr_event = (kgeom_vr_event *) 
                    kgeom_new_primitive(KGEOM_VR_EVENT);

  vr_event->have_button_events = event->have_any_button_events;
  vr_event->have_rotations     = event->have_any_rotations;
  vr_event->have_translations  = event->have_any_translations;

  kmemcpy(vr_event->buttons,   event->button_events, (sizeof(int)*9));
  kmemcpy(vr_event->translate, event->trans, (sizeof(float)*3));
  kmemcpy(vr_event->rotate,    event->rot,   (sizeof(float)*3));
  vr_event->clock = event->clock;

  kgeom_add_primitive(vo, (kgeom_primitive *) vr_event);
  return(CHILL);
}

int
decode_vrevent(vrevent *event,
		kgeom_object *vo)
{
  int done = FALSE;
  int rstat = WHACKED;
  int i,num,*list;
  float *r,*t;
  int *b;

  num = kgeom_number_primitives(vo);
  
  i = 0;
  while (i < num && !done)
  {
     kgeom_primitive *prim = kgeom_get_primitive(vo, i);
     
     /* we're going to look only at the 1st vr event primitive */
     if (prim->type == KGEOM_VR_EVENT)
     {
	kgeom_vr_event *vr = (kgeom_vr_event *) prim;

	event->have_any_button_events = vr->have_button_events;
	event->have_any_rotations     = vr->have_rotations;
	event->have_any_translations  = vr->have_translations;
	event->clock                  = vr->clock;

	kmemcpy(event->rot,vr->rotate,sizeof(float)*3);
	kmemcpy(event->trans,vr->translate,sizeof(float)*3);
	kmemcpy(event->button_events,vr->buttons,sizeof(int)*9);
	
	rstat = CHILL;
	done = TRUE;
     }
     i++;
  }

  if (!done)
  {
     fprintf(stderr," didn't get a vr event..\n");
     rstat = WHACKED;
  }

  return(rstat);
}

int
extract_camera_info(camera_struct *c,
		    kgeom_object *obj)
{
    /* try to find a KGEOM_CAMERA_ORIENTATION prim in this object.
       input is a kojbect.  if we find such a prim, we'll copy the
       KGEOM_CAMERA_ORIENTATAION attributes to *c as output and return CHILL
       as the status. */

  int done = FALSE;
  int status=WHACKED;
  int num,*list,i;
  float *eye=NULL,*at=NULL,*up=NULL;
  float hither,yon,fov,eye_sep,focal_len;
  int projection,is_stereo;

  num = kgeom_number_primitives(obj);
  
  i = 0;
  while (i < num && !done)
  {
     kgeom_primitive *prim = kgeom_get_primitive(obj, i);
     
     /* we're going to look only at the 1st camera orientation primitive */
     if (prim->type == KGEOM_CAMERA_ORIENTATION)
     {
	kgeom_camera_orientation *kc = (kgeom_camera_orientation *) prim;

	kmemcpy(c->eye, kc->eye, sizeof(float)*3);
	kmemcpy(c->at,  kc->at,  sizeof(float)*3);
	kmemcpy(c->up,  kc->up,  sizeof(float)*3);
	
	c->fov    = kc->field_of_view;
	c->hither = kc->hither;
	c->yon    = kc->yon;
	c->projection = kc->projection;
	c->is_stereo  = kc->is_stereo;
	c->eye_sep    = kc->eye_distance;
	c->focal_len  = kc->focal_length;
	
	status = CHILL;
	done = TRUE;
     }
     i++;
  }

  if (!done)
     status = WHACKED;

  return(status);
      
}

int
extract_model_matrix(float **model,
		     kgeom_object *obj,
  		     char **name)

{
  /* try to find a KGEOM_OBJECT_XFORM prim in this object.
   input is a kojbect.  if we find such a prim, we'll copy the
   KGEOM_OBJECT_XFORM attribute to *model as output and return CHILL
   as the status. */
  int done=FALSE;
  int status=WHACKED;
  int num,*list,i;
  int relative;

  num = kgeom_number_primitives(obj);

  i = 0;
  while (i < num && !done)
  {
     kgeom_primitive *prim = kgeom_get_primitive(obj, i);
     
     /* we're going to look only at the 1st object orientation primitive */
     if (prim->type == KGEOM_OBJECT_XFORM)
     {
	kgeom_object_xform *xf = (kgeom_object_xform *) prim;

	*model = xf->matrix;
	*name = xf->name;

	relative = xf->relative;
	 /* we'll ignore name and relative for now.  we assume that
	  relative is KABSOLUTE */

	status = CHILL;
	done = TRUE;
     }
     i++;
  }
  return(status);
}


int
extract_model_translation(float **trans,
		     kgeom_object *obj,
  		     char **name)

{
  /* try to find a KGEOM_OBJECT_XFORM prim in this object.
   input is a kojbect.  if we find such a prim, we'll copy the
   KGEOM_OBJECT_XFORM_TRANSLATE attribute to *model as output and return CHILL
   as the status. */
  int done=FALSE;
  int status=WHACKED;
  int num,*list,i;
  int relative;

  num = kgeom_number_primitives(obj);

  i = 0;
  while (i < num && !done)
  {
     kgeom_primitive *prim = kgeom_get_primitive(obj, i);
     
     /* we're going to look only at the 1st camera orientation primitive */
     if (prim->type == KGEOM_OBJECT_XFORM)
     {
	kgeom_object_xform *xf = (kgeom_object_xform *) prim;

	*trans = xf->translate;
	*name = xf->name;

	relative = xf->relative;
	 /* we'll ignore name and relative for now.  we assume that
	  relative is KABSOLUTE */

	status = CHILL;
	done = TRUE;
     }
     i++;
  }
  return(status);
}


int
extract_view_matrix(float **view,
		    kgeom_object *obj)
{
  /* try to find a KGEOM_VIEW_XFORM prim in this object.
   input is a kojbect.  if we find such a prim, we'll copy the
   KGEOM_VIEW_XFORM attribute to *model as output and return CHILL
   as the status. */

  int done=FALSE;
  int status=WHACKED;
  int num,*list,i;
  int relative;
  char *name;

  num = kgeom_number_primitives(obj);

  i = 0;
  while (i < num && !done)
  {
     kgeom_primitive *prim = kgeom_get_primitive(obj, i);
     
     /* we're going to look only at the 1st camera orientation primitive */
     if (prim->type == KGEOM_VIEW_XFORM)
     {
	kgeom_view_xform *vf = (kgeom_view_xform *) prim;

	*view = vf->matrix;

	status = CHILL;
	done = TRUE;
     }
     i++;
  }
  return(status);
}

int
build_relative_object_xform(float *r,
			    float *model,
			    float *view,
			    vrevent *e)
{
  /* take as input model, view and a vr event and construct a
   transformation matrix which represents a relative transformation
   in the model coordinate system. */
  matrix4x4 m,v,mv,mvinverse,trans,rot,composite,scale;
  matrix4x4 vinverse,minverse;
  vector4 et,er,t;

  /* compute the inverse of the model*view transformation.. */
  kmemcpy(&m,model,sizeof(matrix4x4));
  kmemcpy(&v,view,sizeof(matrix4x4));

  /* ignore the translational component of each of the matrices. */
  v.m[3][0] = v.m[3][1] = v.m[3][2] = 0.;
  m.m[3][0] = m.m[3][1] = m.m[3][2] = 0.;
  
  mmul_4x4(&m,&v,&mv);
  matrix_inverse(&mv,&mvinverse);
  
  t.v[0] = e->trans[0];
  t.v[1] = e->trans[1];
  t.v[2] = e->trans[2];
  t.v[3] = 0;			/* ignore homogeneous coords */

  /* the translational component of the relative world transformation
   is the vrevent translation, in "user" space, mapped back into world
   space vis a vis the model*view inverse matrix. */
  point_xfrm(&t,&mvinverse,&et);
  
  identity_4x4(&trans);
  trans.m[3][0] = et.v[0];
  trans.m[3][1] = et.v[1];
  trans.m[3][2] = et.v[2];

  /* now, construct the appropriate rotation matrix. */
  identity_4x4(&rot);

      /**
      * in building the rot matrix, we'll apply the rotations in order of
      * decreasing magnitude.  in other words, if the user is tweaking hard
      * on a y-axis rotation, and just barely on x & z rotations, we'll apply
      * the y-axis rotation first, then the x & z will be done in decreasing
      * order of magnitude.
      *
      * what we want is to figure out how much we have to rotate the object
      * in object space; angle size in each of the axes, in order to achieve
      * the user-specified rotation.  the problem is that the user specifies
      * the rotation in NPC space, yet we rotate in object space.
      *
      * a vector is formed in which the X component is the X axis rotation
      * component from the device, etc.  we transform this vector through
      * the inverse view/object transformation matrices.  this gives us
      * rotation angles about the principal axes in modeling space.
    **/

  {
    int order[3];  /* boolean to indicate order.  order[0]=0 means x axis
		      is first, order[0]=1, x axis is 2nd, order[0]=2,
		      x axis is third. etc. */
    double fx,fy,fz,t,c,s,xr,yr,zr,tt;
    matrix4x4 rx,ry,rz,loc_inverse,temp,loc_obj_inverse;
    matrix4x4 *mp[3];
    vector4 r1,r2;
    int i;
    
    r1.v[0] = e->rot[0];	/* x-rotation from device */
    r1.v[1] = e->rot[1];	/* y-rotation from device */
    r1.v[2] = e->rot[2]; /*z rotation from device. */
    r1.v[3] = 0;

    point_xfrm(&r1,&mvinverse,&r2); /* xform rot vector
						   thru invs.*/
    xr = r2.v[0];
    yr = r2.v[1];
    zr = r2.v[2];

    tt = r2.v[0]; 
    fx = fabs(tt);
    tt = r2.v[1]; 
    fy = fabs(tt);
    tt = r2.v[2]; 
    fz = fabs(tt);

    mp[0] = mp[1] = mp[2] = NULL;

    order[0] = ((fx >= fy) && (fx >= fz)) ? 0 :
	(((fx >= fy) || (fx >= fz)) ? 1 : 2);
    order[2] = ((fz >= fy) && (fz >= fx)) ? 0 :
	(((fz >= fy) || (fz >= fx)) ? 1 : 2);
    order[1] = ((fy >= fx) && (fy >= fz)) ? 0 :
	(((fy >= fx) || (fy >= fz)) ? 1 : 2);

    identity_4x4(&rx);
    tt = xr;			/* assume device events are in radians  */
    c = kcos(tt);
    s = ksin(tt);
    rx.m[1][1] = rx.m[2][2] = c;
    rx.m[1][2] = s;
    rx.m[2][1] = -1. * s;
    
    identity_4x4(&ry);
    tt = yr;			/* assume device events are in radians */
    c = kcos(tt);
    s = ksin(tt);
    ry.m[0][0] = ry.m[2][2] = c;
    ry.m[0][2] = -1 * s;
    ry.m[2][0] = s;
    
    identity_4x4(&rz);
    tt = zr;			/* assume device events are in radians */
    c = kcos(tt);
    s = ksin(tt);
    rz.m[0][0] = rz.m[1][1] = c;
    rz.m[0][1] = s;
    rz.m[1][0] = -1. * s;
    
    identity_4x4(&rot);

    if (fx != 0)
	mp[order[0]] = &rx;
    if (fy != 0)
	mp[order[1]] = &ry;
    if (fz != 0)
	mp[order[2]] = &rz;

    for (i=0;i<3;i++)
    {
	if (mp[i] != NULL)
	    mmul_4x4(&rot,mp[i],&rot);
    }
  }


  /* ok, done with constructing the rotation matrix.  now composite
   the translation and rotation matrices together. the rotations are
   applied first, then the translations.*/

  mmul_4x4(&rot,&trans,&composite);

  kmemcpy(r,&composite,sizeof(matrix4x4));
  return(CHILL);

}

int
write_relative_object_xform(float *m,
			    kgeom_object *obj,
			    char *name)
{
  /* take the transformation matrix M and write it into the
   kgeom_object obj.  */

  kgeom_object_xform *oxform = (kgeom_object_xform *) 
                    kgeom_new_primitive(KGEOM_OBJECT_XFORM);

  oxform->relative = KRELATIVE;
  oxform->name = kstrdup(name);
  kmemcpy(oxform->matrix, m, sizeof(float) * 16);

  kgeom_add_primitive(obj, (kgeom_primitive *) oxform);
  return(CHILL);
}

int
write_relative_camera_xform(kgeom_object *obj,
			    camera_struct *c)
{
  /* take the info in the camera struct and cram it into the kgeom_object
     using geometry services. */

  kgeom_camera_orientation *kc = (kgeom_camera_orientation *) 
                    kgeom_new_primitive(KGEOM_CAMERA_ORIENTATION);

  kc->hither = c->hither;
  kc->yon    = c->yon;
  kc->projection   = c->projection;
  kc->is_stereo    = c->is_stereo;
  kc->eye_distance = c->eye_sep;
  kc->focal_length = c->focal_len;
  kc->field_of_view = c->fov;
  kmemcpy(kc->eye, c->eye, sizeof(float) * 3);
  kmemcpy(kc->at,  c->at,  sizeof(float) * 3);
  kmemcpy(kc->up,  c->up,  sizeof(float) * 3);

  kgeom_add_primitive(obj, (kgeom_primitive *) kc);
  return(CHILL);
}
#define TWOPI 6.2831853

int
build_relative_camera_xform(camera_struct *old,
			    vrevent *e,
			    camera_struct *new1,
			    float *fview)
{
    int i;
    double ctheta,stheta,cbeta,sbeta,cgamma,sgamma,t;
    matrix4x4 rz,ry,rx,composite,cum_matrix;
    vertex_3d ttrans,trot;

    /* catch the misc parameters that don't get explicitly set below.. */
    kmemcpy(new1,old,sizeof(camera_struct));

    /* construct a view orientation matrix for the view parms in old */
    build_view_orientation_matrix(old,&cum_matrix);

    e->rot[2] *= -1.;
    vert_xfrm((vertex_3d *) e->rot,&cum_matrix,&trot);

    /* fview not used... */
    
    t = trot.v[2];
    if (t > TWOPI)
	t = t-TWOPI;
    else if (t < -TWOPI)
	t = t+TWOPI;
    ctheta = kcos(t);
    stheta = ksin(t);

    identity_4x4(&rz);
    rz.m[0][0] = rz.m[1][1] = ctheta;
    rz.m[0][1] = stheta;
    rz.m[1][0] = -1. * stheta;

    t = trot.v[1];
    if (t > TWOPI)
	t = t-TWOPI;
    else if (t < -TWOPI)
	t = t+TWOPI;
    
    cbeta = kcos(t);
    sbeta = ksin(t);

    identity_4x4(&ry);
    ry.m[0][0] = ry.m[2][2] = cbeta;
    ry.m[0][2] = -1. * sbeta;
    ry.m[2][0] = sbeta;

    t = trot.v[0];
    if (t > TWOPI)
	t = t-TWOPI;
    else if (t < -TWOPI)
	t = t+TWOPI;
    
    cgamma = kcos(t);
    sgamma = ksin(t);

    identity_4x4(&rx);
    rx.m[1][1] = rx.m[2][2] = cgamma;
    rx.m[1][2] = sgamma;
    rx.m[2][1] = -1. * sgamma;

    identity_4x4(&composite);

    mmul_4x4(&composite,&rz,&composite);
    mmul_4x4(&composite,&ry,&composite);
    mmul_4x4(&composite,&rx,&composite);

    mmul_4x4(&cum_matrix,&composite,&composite);
    
    kmemcpy(&cum_matrix,&composite,sizeof(float)*16);

    /* the new eye point is a function of the old eye point, the
     orientation matrix constructed from the various viewing parameters
     and the translational component of the vr event. */

    e->trans[2] *= -1.;
    vert_xfrm((vertex_3d *) e->trans,&composite,&ttrans);

    for (i=0;i<3;i++)
        new1->eye[i] += ttrans.v[i];

    /* the new at point is a function of the current eye point, plus the
     Z axis of the current rotation matrix. */

    for (i=0;i<3;i++)
	new1->at[i] = new1->eye[i] - (composite.m[2][i] * new1->focal_len);

    /* the up vector is the y axis of the new view orientation matrix */
    for (i=0;i<3;i++)
	new1->up[i] = composite.m[1][i];

    return(CHILL);
}

static int
build_view_orientation_matrix(camera_struct *c,
			      matrix4x4 *m)
{
    vertex_3d v_y,v_z,v_x;
    int i;
    
    identity_4x4(m);

    /**
      * while the up vector is simply the Y axis of the 
      * rotation matrix.
    **/
    vert3norm((vertex_3d *)c->up,&v_y);
    for (i=0;i<3;i++)
	m->m[1][i] = v_y.v[i];

    /**
      * the Z axis is the eye-at vector.
    **/
    vert3diff((vertex_3d *)c->eye,(vertex_3d *)c->at,&v_z);
    vert3norm(&v_z,&v_z);
    for (i=0;i<3;i++)
	m->m[2][i] = v_z.v[i];
    
    /**
      * the X axis is the cross product of the eye-at vector with the
      * up vector.
    **/
    vert3cross(&v_y,&v_z,&v_x);
    for (i=0;i<3;i++)
	m->m[0][i] = v_x.v[i];

    return(CHILL);
}


int
vr_search_for_named_object(char *name,
			   kgeom_object *obj)
{
    /* this routine will look through all the prims of the input
     object.  if there are ANY transformation events which match
     that object, return a 1.  otherwise, return a zero. */
  /* try to find a KGEOM_OBJECT_XFORM prim in this object.
   input is a kojbect.  if we find such a prim, we'll copy the
   KGEOM_OBJECT_XFORM attribute to *model as output and return CHILL
   as the status. */
  int done=FALSE;
  int status=WHACKED;
  int num,*list,i;
  int relative;

  num = kgeom_number_primitives(obj);

  i = 0;
  while (i < num && !done)
  {
     kgeom_primitive *prim = kgeom_get_primitive(obj, i);
     
     /* we're going to look only at the 1st object orientation primitive */
     if (prim->type == KGEOM_OBJECT_XFORM)
     {
	kgeom_object_xform *xf = (kgeom_object_xform *) prim;

	if (strcmp(name,xf->name) == 0)
	    return(1);
     }
     i++;
  }
  return(0);
}

int
extract_named_model_matrix(float **model,
			   kgeom_object *obj,
			   char *name)
{
    /* scan through the list of prims looking for a KGEOM_OBJECT_XFORM
     for the named object. if we find one, return the matrix in *model
     and return a 1.  otherwise, return a zero. */
  int num,i;

  num = kgeom_number_primitives(obj);

  for (i = 0;i<num;i++)
  {
     kgeom_primitive *prim = kgeom_get_primitive(obj, i);
     
     /* we're going to look only at the 1st object orientation primitive */
     if (prim->type == KGEOM_OBJECT_XFORM)
     {
	kgeom_object_xform *xf = (kgeom_object_xform *) prim;

	if (strcmp(name,xf->name) == 0)
	{
	    *model = xf->matrix;
	    return(1);
	}
     }
  }
  return(0);
}

int
extract_named_model_translation(float **trans,
				kgeom_object *obj,
				char *name)
{
    /* scan through the list of prims looking for a KGEOM_OBJECT_XFORM
     for the named object. if we find one, return the translation component
     in *trans and return a 1.  otherwise, return a zero. */
  int num,i;

  num = kgeom_number_primitives(obj);

  for (i = 0;i<num;i++)
  {
     kgeom_primitive *prim = kgeom_get_primitive(obj, i);
     
     /* we're going to look only at the 1st object orientation primitive */
     if (prim->type == KGEOM_OBJECT_XFORM)
     {
	kgeom_object_xform *xf = (kgeom_object_xform *) prim;

	if (strcmp(name,xf->name) == 0)
	{
	    *trans = xf->translate;
	    return(1);
	}
     }
  }
  return(0);
}
