 /*
  * Khoros: $Id: form_init.c,v 1.3.2.2 1997/05/31 18:59:34 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: form_init.c,v 1.3.2.2 1997/05/31 18:59:34 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	FormInfo Initialization Routines
   >>>> 
   >>>>  Private: 
   >>>> 	_xvf_init_form
   >>>> 	_xvf_free_form
   >>>> 
   >>>>   Static: 
   >>>> 	_xvf_init_form_warning
   >>>> 	_xvf_init_form_error
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "oslicer.h"

static void _xvf_init_form_warning PROTO((char *));
static void _xvf_init_form_error   PROTO((char *, char *));

/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_init_form
| 
|       Purpose: Initializes the GUI Information (gui_info_struct) structure
| 
|         Input: forminfo - ptr to _xvf_init_form_gui_struct
|                form     - pointer to the form tree
| 
|        Output: none
|    Written By: conductor
|          Date: July 1, 1997
| Modifications:  
| 
------------------------------------------------------------*/
int _xvf_init_form(

    kform *form, 
    gui_info_struct *forminfo)
{
	ksubform   *subform;
	kcontrol   *pane;
	kselection *selection;
	/*
	 * initializing 'form' Form Info structure
	 */

	forminfo->subform_struct =
	    kvf_create_struct_from_subformname(form, "form.subform");
	if (forminfo->subform_struct == NULL)
	{
	    _xvf_init_form_error("subform 'subform' on form 'form'", 
			    "form_subform");
	    return(FALSE);
	}

	/*
	 * initializing 'form->subform' Subform Info structure
	 */

	forminfo->subform = (form_subform *)
				kcalloc((size_t) 1, sizeof(form_subform));
	subform = forminfo->subform_struct->Subformptr;

	/*
	 * initializing 'form->subform->pane' Pane Info structure
	 */

	forminfo->subform->pane = (subform_pane *)
				kcalloc((size_t)1, sizeof(subform_pane));
	forminfo->subform->pane_struct =
	    kvf_create_struct_from_panename(subform, "form.subform.pane");
	if (forminfo->subform->pane_struct == NULL)
	{
	    _xvf_init_form_error("pane 'pane' on subform 'subform'", "subform_pane");
	    return(FALSE);
	}
	pane = forminfo->subform->pane_struct->Controlptr;
	forminfo->subform->pane->license_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.license");
	if (forminfo->subform->pane->license_struct == NULL)
		    _xvf_init_form_warning("unknown selection 'license' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->help_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.help");
	if (forminfo->subform->pane->help_struct == NULL)
		    _xvf_init_form_warning("unknown selection 'help' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->quit_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.quit");
	if (forminfo->subform->pane->quit_struct == NULL)
		    _xvf_init_form_warning("quit button 'quit' on pane 'pane' of subform 'subform'");
	forminfo->subform->pane->quit = FALSE;

	forminfo->subform->pane->slice_axis_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.slice_axis");
	if (forminfo->subform->pane->slice_axis_struct != NULL)
	{
	    selection = forminfo->subform->pane->slice_axis_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->slice_axis_struct, KVF_LIST_VAL,
	                    &(forminfo->subform->pane->slice_axis));
	    kvf_get_attribute(forminfo->subform->pane->slice_axis_struct, KVF_LIST_LABEL,
	                    &(forminfo->subform->pane->slice_axis_label));
	}
	else
	    _xvf_init_form_warning("list selection 'slice_axis' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->slice_axis_selected = FALSE;

	forminfo->subform->pane->slice_number_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.slice_number");
	if (forminfo->subform->pane->slice_number_struct != NULL)
	{
	    selection = forminfo->subform->pane->slice_number_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->slice_number_struct, KVF_INT_DEF,
	                    &(forminfo->subform->pane->slice_number));
	}
	else
	    _xvf_init_form_warning("integer selection 'slice_number' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->slice_number_selected = FALSE;


	return(TRUE);

}

/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_init_form_warning
| 
|       Purpose: Prints warning when code is out of sync with UIS file
| 
|         Input: string - item that cannot be found
| 
|        Output: none
|    Written By: conductor
|          Date: July 1, 1997
| Modifications:  
| 
------------------------------------------------------------*/
static void _xvf_init_form_warning(
    char *string)
{
	kfprintf(kstderr, "Warning: cannot find %s.\n", string);
	kfprintf(kstderr, "         UIS file out of sync with code\n");
	kfprintf(kstderr, "         generated by conductor in 'form_init.c'.\n");
}


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_init_form_error
| 
|       Purpose: Prints error when code is irreconcilable with UIS file
| 
|         Input: string1 - item that cannot be found
|                string2 - name of FormInfo structure
| 
|        Output: none
|    Written By: conductor
|          Date: July 1, 1997
| Modifications:  
| 
------------------------------------------------------------*/
static void _xvf_init_form_error(
    char *string1,
    char *string2)
{
	kfprintf(kstderr, "ERROR: cannot find %s.\n", string1);
	kfprintf(kstderr, "         UIS file out of sync with code\n");
	kfprintf(kstderr, "         generated by conductor in 'form_init.c'.\n");
	kfprintf(kstderr, "         Unable to initialize '%s' structure as defined in form_info.h; Aborting.\n", string2);
}


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_free_form
| 
|       Purpose: Frees the gui_info_struct
| 
|         Input: forminfo - ptr to _xvf_free_form_gui_struct
| 
|        Output: none
|    Written By: conductor
|          Date: July 1, 1997
| Modifications:  
| 
------------------------------------------------------------*/
void _xvf_free_form(
    gui_info_struct *forminfo)
{
	if (forminfo->subform->pane->license_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->license_struct);
	if (forminfo->subform->pane->help_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->help_struct);
	if (forminfo->subform->pane->quit_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->quit_struct);
	if (forminfo->subform->pane->slice_axis_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->slice_axis_struct);
	kfree_and_NULL(forminfo->subform->pane->slice_axis_label);
	if (forminfo->subform->pane->slice_number_struct != NULL)
	     kfree_and_NULL(forminfo->subform->pane->slice_number_struct);
	kfree_and_NULL(forminfo->subform->pane);
	if (forminfo->subform->pane_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane_struct);


	if (forminfo->subform_struct != NULL)
	    kfree_and_NULL(forminfo->subform_struct);
	kfree_and_NULL(forminfo->subform);
	kfree_and_NULL(forminfo);
}
