 /*
  * Khoros: $Id: oslicer.c,v 1.6.2.3 1997/05/31 18:59:36 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: oslicer.c,v 1.6.2.3 1997/05/31 18:59:36 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for oslicer
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "oslicer.h"

clui_info_struct *clui_info = NULL;

gui_info_struct *gui_info;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Interactive Slicing Tool.
|
|       Purpose: main program for oslicer
|
|         Input:
|		char *clui_info->i_file; {First Input data object}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Resulting output data object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

	kform  *form;            /* form tree representing UIS        */
	char   *form_fullpath;   /* for use when [-form] is specified */
/* -main_variable_list */
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/xvroutine/oslicer");
	kexit_handler(oslicer_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "oslicer", KGEN_XVROUTINE,  &clui_uis_spec,
		   oslicer_usage_additions, oslicer_get_args,
 		   oslicer_free_args);
/* -main_get_args_call_end */

	/*
	 * Initialize xvwidgets lib
	 */
	if (!(xvw_initialize(XVW_MENUS_XVFORMS)))
	{
	     kinfo(KSTANDARD, "Could not open connection to server");
	     kexit(KEXIT_FAILURE);
	}

	/*
	 * Allocate the GUI Info structure 
	 */
	gui_info = (gui_info_struct *)
				kcalloc ((size_t) 1, sizeof(gui_info_struct));
	/*
	 * Create the forms for oslicer 
	 */
	if (clui_info->form_flag)
	{
	    form_fullpath   = kfullpath(clui_info->form_file, NULL, NULL);
	    form = xvf_create_form(form_fullpath, KNONE, run_form, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	    kfree_and_NULL(form_fullpath);
	}
	else
	{
	    form = xvf_build_form(&gui_uis_spec, KNONE, run_form, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	}
	if (form == NULL)
	   kexit(KEXIT_FAILURE);

	/*
	 * Initialize the GUI Info structure 
	 */
	if (!(_xvf_init_form(form, gui_info)))
	    kexit(KEXIT_FAILURE);

/* -main_before_lib_call */
	/* open the objects. */
	open_new_input_object(clui_info->i_file);
	register_output_fname(clui_info->o_file);

	xvw_add_detectfile(NULL,clui_info->i_file,(float)0.5,
			   reread_input,NULL);

	/* configure the various pieces of the ui. */
	config_gui(gui_info->subform->pane);
/* -main_before_lib_call_end */

/* -main_library_call */
	xvf_run_form();
/* -main_library_call_end */

/* -main_after_lib_call */
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: oslicer_usage_additions
| 
|       Purpose: Prints usage additions in oslicer_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname oslicer
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void oslicer_usage_additions(void)
{
	kfprintf(kstderr, "\tInteractive Slicing Tool.\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: oslicer_free_args
| 
|       Purpose: Frees CLUI struct allocated in oslicer_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname oslicer
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
oslicer_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->form_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
