/*
 * Khoros: #RCSID#
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: #RCSID#";
#endif

/*
 * Copyright (C) 1996, The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/copyright.lbl.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>             _static_routines()
   >>>>  Public:
   >>>>             public_routines()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "oslicer.h"

static kobject input,output;

typedef struct do_slice_struct
{
    int isize[5],ipos[5];
    int loc_size[4];
    int osize[5];
    int slice_axis;
    int slice_number;
    int has_loc;
} do_slice_struct;

static do_slice_struct s;
static char outputfname[512];
static int map_gagged=0;	/* only till we finish writing code.. */

static char loc_fname[KLENGTH];
static void samurai PROTO(());
static int config_output_object PROTO((int w,int h,int d,int t,int e));

int
reread_input(xvobject obj, char *call_data, kaddr client_data)
{
    extern gui_info_struct *gui_info;
    fprintf(stderr," rereading input \n");
    open_new_input_object(loc_fname);

    config_gui(gui_info->subform->pane);
    /* we're going to assume for now that the new input object has
    the same dims, etc as the output object.  what we should do is reconfig
    the gui (examining limits and such) before slicing. */

    samurai();
    return(TRUE);
}

int
open_new_input_object(char *fname)
{
    while ((input = kpds_open_object(fname,KOBJ_READ)) == KOBJECT_INVALID)
	fprintf(stderr," looping while trying to open the output object.\n");
    kstrcpy(loc_fname,fname);
    
    /* -- ensure that we have location data on the input  //SK -- */
    if (!kpds_query_location(input))
    {
       int wid,hgt,dep;
       
       /* -- we know from the khoros_init_input that we have value -- */
       kpds_get_attribute(input,KPDS_VALUE_SIZE,&wid,&hgt,&dep,NULL,NULL);
       
       /* we are going to set the uniform location min and max to be from 
	  (0,0,0) <-> (wid,hgt,dep).   The rest of the routine will 
	  process as if we had regular curvilinear location data present */

       /* -- create uniform location data -- */
       kpds_set_attribute(input, KPDS_LOCATION_GRID, KUNIFORM);
       kpds_create_location(input);
       kpds_set_attributes(input, 
			   KPDS_LOCATION_BEGIN, 0.,0.,0.,
			   KPDS_LOCATION_END,(double)wid,
			   (double)hgt, (double)dep, NULL);
       
    }

    return(CHILL);
}

int
register_output_fname(char *fname)
{
    kstrcpy(outputfname,fname);
    return(CHILL);
}

static int
open_new_output_object()
{
    output = kpds_open_object(outputfname,KOBJ_WRITE);
    config_output_object(s.osize[0],s.osize[1],s.osize[2],
			 s.osize[3],s.osize[4]);
    return(CHILL);
}

static int
config_output_object(int w,
		     int h,
		     int d,
		     int t,
		     int e)
{
    kpds_create_value(output);
    
    kpds_copy_attribute(input,output,KPDS_VALUE_DATA_TYPE);
    kpds_set_attribute(output,KPDS_VALUE_SIZE,
		       w,h,d,t,e);
    s.osize[0] = w;
    s.osize[1] = h;
    s.osize[2] = d;
    s.osize[3] = t;
    s.osize[4] = e;

    if (kpds_query_location(input))
    {
	int dims,i,j,k;
	
	s.has_loc = 1;
	kpds_create_location(output);
	kpds_copy_attribute(input,output,KPDS_LOCATION_DATA_TYPE);
	kpds_get_attribute(input,KPDS_LOCATION_SIZE,&i,&j,&k,&dims);
	s.loc_size[0] = i;
	s.loc_size[1] = j;
	s.loc_size[2] = k;
	s.loc_size[3] = dims;
	kpds_set_attribute(output,KPDS_LOCATION_SIZE,w,h,d,dims);
    }
    else
	s.has_loc = 0;

    if (kpds_query_map(input))
    {
	if (!map_gagged)
	{
	    kerror("GEOMETRY","slicers"," we don't support slicing of the map segment. this message will not be repeated....");
	    map_gagged = 1;
	}
    }
    return(CHILL);
}

int
config_gui(subform_pane *p)
{
    int w,h,d,t,e;
    int which_axis;
    int lower,upper;
    int n;

    /* set the limit of the slider based upon the current value
     of the "which axis" parameter. */

    kpds_get_attribute(input,KPDS_VALUE_SIZE,&w,&h,&d,&t,&e);
    
    s.osize[0] = s.isize[0] = w;
    s.osize[1] = s.isize[1] = h;
    s.osize[2] = s.isize[2] = d;
    s.osize[3] = s.isize[3] = t;
    s.osize[4] = s.isize[4] = e;

    s.slice_number = n = p->slice_number;

    lower = 0;

    kmemset(s.ipos,0,sizeof(int)*5);
    switch (p->slice_axis)
    {
    case 1:			/* width */
	upper = w-1;		/* set upper limit.. */
	s.osize[0] = 1;		/* we're going from N-d to (N-1)-d */
	s.ipos[0] = n;		/* location of slice plane */
	break;
    case 2:			/* height */
	upper = h-1;
	s.osize[1] = 1;
	s.ipos[1] = n;
	break;
    case 3:			/* depth */
	upper = d-1;
	s.osize[2] = 1;
	s.ipos[2] = n;
	break;
    case 4:			/* time */
	upper = t-1;
	s.osize[3] = 1;
	s.ipos[3] = n;
	break;
    case 5:			/* elements */
	upper = e-1;
	s.osize[3] = 1;
	s.ipos[4] = n;
	break;
    }


    if (which_axis != p->slice_axis)
    {
	xvf_set_attribute(p->slice_number_struct,
			  XVF_INT_LOWER,
			  lower);
    
	xvf_set_attribute(p->slice_number_struct,
			  XVF_INT_UPPER,
			  upper);
    }

    which_axis = s.slice_axis = p->slice_axis;

    samurai();
    return(CHILL);
}

static void
samurai()
{
    /* this is the routine that does the slicing. */

    void *p;

    open_new_output_object();

    kpds_set_attribute(input,KPDS_VALUE_POSITION,s.ipos[0],s.ipos[1],
		       s.ipos[2],s.ipos[3],s.ipos[4]);
    kpds_set_attribute(input,KPDS_VALUE_REGION_SIZE,s.osize[0],s.osize[1],
		       s.osize[2],s.osize[3],s.osize[4]);

    p = kpds_get_data(input,KPDS_VALUE_REGION,NULL);
    kpds_put_data(output,KPDS_VALUE_ALL,p);

    if (s.has_loc)
    {
	kpds_set_attribute(input,KPDS_LOCATION_POSITION,s.ipos[0],s.ipos[1],
			   s.ipos[2],0);
	kpds_set_attribute(input,KPDS_LOCATION_REGION_SIZE,s.osize[0],s.osize[1],
			   s.osize[2],s.loc_size[3]);

	p = kpds_get_data(input,KPDS_LOCATION_REGION,NULL);
	kpds_put_data(output,KPDS_LOCATION_ALL,p);

    }
    
    while (!kpds_close_object(output))
	fprintf(stderr,"looping while closing the kobject...\n");

    
}

