 /*
  * Khoros: $Id: do_pane.c,v 1.2 1997/05/02 06:37:28 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_pane.c,v 1.2 1997/05/02 06:37:28 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	$GEOMETRY/objects/xvroutine/pdb2geom/src/do_pane.c
   >>>> 
   >>>>  Private: 
   >>>> 	pane_i
   >>>> 	pane_o
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "pdb2geom.h"

STATE interface;

static COLOR colors[] = { { "Red",		1.000, 0.000, 0.000 },
		          { "Orange",		1.000, 0.300, 0.000 },
		          { "Yellow",		1.000, 1.000, 0.000 },
		          { "Green",		0.000, 1.000, 0.000 },
		          { "Cyan",		0.000, 1.000, 1.000 },
		          { "Blue",		0.000, 0.000, 1.000 },
		          { "Violet",		0.400, 0.050, 0.800 },
		          { "Magenta",		1.000, 0.000, 1.000 },
		          { "Tan",		0.800, 0.250, 0.150 },
		          { "Brown",		0.500, 0.100, 0.000 },
		          { "White",		1.000, 1.000, 1.000 },
		          { "Gray",		0.500, 0.500, 0.500 },
		        };

void setup_list(kform_struct *list, LIST *cur) {
	int num;
	LIST *scan;
	char **names;

	xvf_set_attribute(list,XVF_LIST_DELETEALL,TRUE);

	num = 0;
	for (scan = cur; scan != NULL; scan = scan->next) {
		num++;
	}

	xvf_set_attribute(list,XVF_LIST_SIZE,num);
	xvf_set_attribute(list,XVF_LIST_START,1);

	names = (char **)kmalloc(num*sizeof(*names));

	num = 0;
	for (scan = cur; scan != NULL; scan = scan->next) {
		names[num] = scan->name;
		num++;
	}

	xvf_set_attribute(list,XVF_LIST_CONTENTS,names);

	kfree_and_NULL(names);

	xvf_set_attribute(list,XVF_LIST_VAL,1);
}

void setup_template(subform_pane *pane_info, int type, TEMPLATE *templ) {
	templ->residue_name = NULL;
	templ->residue_num = -1;
	templ->chain_name = NULL;
	templ->name = NULL;
	templ->num = -1;

	if (pane_info->res_name_optsel == TRUE) {
		templ->residue_name = pane_info->res_name_label;
	}

	if (pane_info->res_num_optsel == TRUE) {
		templ->residue_num = pane_info->res_num;
	}

	if (pane_info->chain_name_optsel == TRUE) {
		templ->chain_name = pane_info->chain_name_label;
	}

	if (pane_info->name_optsel == TRUE) {
		templ->name = pane_info->name_label;
	}

	if (pane_info->num_optsel == TRUE) {
		templ->num = pane_info->num;
	}

	templ->type = type;
}

void do_all(STATE *interface) {
	do_read(interface);
	do_display(interface);
}

void do_read(STATE *interface) {
	kfile *in;
	subform_pane *pane;
	int size;

	in = kfopen(interface->in,"r");

	freePDB();
	readPDB(in);

	kfclose(in);

	pane = gui_info->subform->pane;

	setup_list(pane->res_name_struct,residue_names);
	setup_list(pane->chain_name_struct,chain_names);
	setup_list(pane->name_struct,names);
}

void do_display(STATE *interface) {
	kgeom_object *out;

	out = kgeom_new_object();

	out->name = kstrdup("PDB");

	kgeom_write_header(out,interface->out);
	kgeom_start_writing_object(out);

	display(out,interface->rep);

	kgeom_finish_writing_object(out);
	kgeom_done_writing(out);

	kgeom_blast_object(out);
}

void do_match(TEMPLATE *templ, subform_pane *pane_info) {
	int count;
	char info[1024];

	count = matchPDB(templ);
	ksprintf(info,"%9d",count);

	switch (templ->type) {
	case COL:
		xvf_set_attribute(pane_info->colored_struct,XVF_STRING_DEF,info);
		xvf_set_attribute(pane_info->colored_struct,XVF_STRING_VAL,info);

		kfree_and_NULL(pane_info->colored);
		pane_info->colored = kstrdup(info);
		break;

	case DISP:
		xvf_set_attribute(pane_info->displayed_struct,XVF_STRING_DEF,info);
		xvf_set_attribute(pane_info->displayed_struct,XVF_STRING_VAL,info);

		kfree_and_NULL(pane_info->displayed);
		pane_info->displayed = kstrdup(info);
		break;
	}
}

/*-----------------------------------------------------------
| 
|  Routine Name: pane_i
| 
|       Purpose: Do routine which is called when
|                  input selection i is used
| 
|         Input: pane_info - ptr to PaneInfo struct for pane pane
| 
|        Output: None
|    Written By: 
|          Date: Mar 06, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pane_i(
     subform_pane *pane_info)
{
	kfree_and_NULL(interface.in);
	interface.in = kstrdup(pane_info->i);

	do_all(&interface);
}


/*-----------------------------------------------------------
| 
|  Routine Name: pane_o
| 
|       Purpose: Do routine which is called when
|                  output selection o is used
| 
|         Input: pane_info - ptr to PaneInfo struct for pane pane
| 
|        Output: None
|    Written By: 
|          Date: Mar 06, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pane_o(
     subform_pane *pane_info)
{
	kfree_and_NULL(interface.out);
	interface.out = kstrdup(pane_info->o);

	do_display(&interface);
}
/*-----------------------------------------------------------
| 
|  Routine Name: pane_color_now
| 
|       Purpose: Do routine which is called when
|                  pane action button color_now is used
| 
|         Input: pane_info - ptr to PaneInfo struct for pane pane
| 
|        Output: None
|    Written By: 
|          Date: Mar 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pane_color_now(
     subform_pane *pane_info)
{
	TEMPLATE templ;
	float r,g,b;
	int i,ncolors;

	setup_template(pane_info,COL,&templ);
	ncolors = sizeof(colors) / sizeof(colors[0]);

	for (i = 0; i < ncolors; i++) {
		if (kstrcmp(pane_info->color_label,colors[i].name) == 0) {
			r = colors[i].r;
			g = colors[i].g;
			b = colors[i].b;

			break;
		}
	}

	if (i == ncolors) {
		if (kstrcmp(pane_info->color_label,"RGB") == 0) {
			r = pane_info->red;
			g = pane_info->green;
			b = pane_info->blue;
		} else {
			kfprintf(kstderr,"Unknown color, %s; using white\n",
				pane_info->color_label);
			r = 1.0; g = 1.0; b = 1.0;
		}
	}

	do_match(&templ,pane_info);
	colorPDB(r,g,b);
	do_display(&interface);
}


/*-----------------------------------------------------------
| 
|  Routine Name: pane_display_now
| 
|       Purpose: Do routine which is called when
|                  pane action button display_now is used
| 
|         Input: pane_info - ptr to PaneInfo struct for pane pane
| 
|        Output: None
|    Written By: 
|          Date: Mar 14, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pane_display_now(
     subform_pane *pane_info)
{
	TEMPLATE templ;

	setup_template(pane_info,DISP,&templ);

	do_match(&templ,pane_info);
	do_display(&interface);
}


/*-----------------------------------------------------------
| 
|  Routine Name: pane_radius
| 
|       Purpose: Do routine which is called when
|                  float selection radius is used
| 
|         Input: pane_info - ptr to PaneInfo struct for pane pane
| 
|        Output: None
|    Written By: 
|          Date: Mar 24, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pane_radius(
     subform_pane *pane_info)
{
	TEMPLATE templ;
	float radius;

	setup_template(pane_info,COL,&templ);

	radius = pane_info->radius;

	do_match(&templ,pane_info);
	radiusPDB(radius);
	do_display(&interface);
}


/*-----------------------------------------------------------
| 
|  Routine Name: pane_rep
| 
|       Purpose: Do routine which is called when
|                  list selection rep is used
| 
|         Input: pane_info - ptr to PaneInfo struct for pane pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 28, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pane_rep(
     subform_pane *pane_info)
{
	kfree_and_NULL(interface.rep);
	interface.rep = kstrdup(pane_info->rep_label);

	do_display(&interface);
}
