 /*
  * Khoros: $Id: form_init.c,v 1.1.2.1 1997/06/07 20:37:23 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: form_init.c,v 1.1.2.1 1997/06/07 20:37:23 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	FormInfo Initialization Routines
   >>>> 
   >>>>  Private: 
   >>>> 	_xvf_init_form
   >>>> 	_xvf_free_form
   >>>> 
   >>>>   Static: 
   >>>> 	_xvf_init_form_warning
   >>>> 	_xvf_init_form_error
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "pdb2geom.h"

static void _xvf_init_form_warning PROTO((char *));
static void _xvf_init_form_error   PROTO((char *, char *));

/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_init_form
| 
|       Purpose: Initializes the GUI Information (gui_info_struct) structure
| 
|         Input: forminfo - ptr to _xvf_init_form_gui_struct
|                form     - pointer to the form tree
| 
|        Output: none
|    Written By: conductor
|          Date: July 1, 1997
| Modifications:  
| 
------------------------------------------------------------*/
int _xvf_init_form(

    kform *form, 
    gui_info_struct *forminfo)
{
	ksubform   *subform;
	kcontrol   *pane;
	kselection *selection;
	/*
	 * initializing 'form' Form Info structure
	 */

	forminfo->subform_struct =
	    kvf_create_struct_from_subformname(form, "form.subform");
	if (forminfo->subform_struct == NULL)
	{
	    _xvf_init_form_error("subform 'subform' on form 'form'", 
			    "form_subform");
	    return(FALSE);
	}

	/*
	 * initializing 'form->subform' Subform Info structure
	 */

	forminfo->subform = (form_subform *)
				kcalloc((size_t) 1, sizeof(form_subform));
	subform = forminfo->subform_struct->Subformptr;

	/*
	 * initializing 'form->subform->pane' Pane Info structure
	 */

	forminfo->subform->pane = (subform_pane *)
				kcalloc((size_t)1, sizeof(subform_pane));
	forminfo->subform->pane_struct =
	    kvf_create_struct_from_panename(subform, "form.subform.pane");
	if (forminfo->subform->pane_struct == NULL)
	{
	    _xvf_init_form_error("pane 'pane' on subform 'subform'", "subform_pane");
	    return(FALSE);
	}
	pane = forminfo->subform->pane_struct->Controlptr;
	forminfo->subform->pane->license_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.license");
	if (forminfo->subform->pane->license_struct == NULL)
		    _xvf_init_form_warning("unknown selection 'license' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->help_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.help");
	if (forminfo->subform->pane->help_struct == NULL)
		    _xvf_init_form_warning("unknown selection 'help' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->quit_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.quit");
	if (forminfo->subform->pane->quit_struct == NULL)
		    _xvf_init_form_warning("quit button 'quit' on pane 'pane' of subform 'subform'");
	forminfo->subform->pane->quit = FALSE;

	forminfo->subform->pane->i_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.i");
	if (forminfo->subform->pane->i_struct != NULL)
	{
	    selection = forminfo->subform->pane->i_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->i_struct, KVF_FILE_DEF,
	                    &(forminfo->subform->pane->i));
	}
	else
	    _xvf_init_form_warning("input selection 'i' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->i_selected = FALSE;

	forminfo->subform->pane->o_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.o");
	if (forminfo->subform->pane->o_struct != NULL)
	{
	    selection = forminfo->subform->pane->o_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->o_struct, KVF_FILE_DEF,
	                    &(forminfo->subform->pane->o));
	}
	else
	    _xvf_init_form_warning("output selection 'o' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->o_selected = FALSE;

	forminfo->subform->pane->rep_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.rep");
	if (forminfo->subform->pane->rep_struct != NULL)
	{
	    selection = forminfo->subform->pane->rep_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->rep_struct, KVF_LIST_VAL,
	                    &(forminfo->subform->pane->rep));
	    kvf_get_attribute(forminfo->subform->pane->rep_struct, KVF_LIST_LABEL,
	                    &(forminfo->subform->pane->rep_label));
	}
	else
	    _xvf_init_form_warning("list selection 'rep' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->rep_selected = FALSE;

	forminfo->subform->pane->res_name_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.res_name");
	if (forminfo->subform->pane->res_name_struct != NULL)
	{
	    selection = forminfo->subform->pane->res_name_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->res_name_struct, KVF_LIST_VAL,
	                    &(forminfo->subform->pane->res_name));
	    kvf_get_attribute(forminfo->subform->pane->res_name_struct, KVF_LIST_LABEL,
	                    &(forminfo->subform->pane->res_name_label));
	}
	else
	    _xvf_init_form_warning("list selection 'res_name' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->displayed_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.displayed");
	if (forminfo->subform->pane->displayed_struct != NULL)
	{
	    selection = forminfo->subform->pane->displayed_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->displayed_struct, KVF_STRING_DEF,
	                    &(forminfo->subform->pane->displayed));
	}
	else
	    _xvf_init_form_warning("string selection 'displayed' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->displayed_optsel = TRUE;

	forminfo->subform->pane->display_now_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.display_now");
	if (forminfo->subform->pane->display_now_struct == NULL)
	    _xvf_init_form_warning("pane action button 'display_now' on pane 'pane' of subform 'subform'");
	forminfo->subform->pane->display_now = FALSE;
	selection = forminfo->subform->pane->display_now_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->subform->pane->res_num_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.res_num");
	if (forminfo->subform->pane->res_num_struct != NULL)
	{
	    selection = forminfo->subform->pane->res_num_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->res_num_struct, KVF_INT_DEF,
	                    &(forminfo->subform->pane->res_num));
	}
	else
	    _xvf_init_form_warning("integer selection 'res_num' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->res_num_optsel = FALSE;

	forminfo->subform->pane->colored_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.colored");
	if (forminfo->subform->pane->colored_struct != NULL)
	{
	    selection = forminfo->subform->pane->colored_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->colored_struct, KVF_STRING_DEF,
	                    &(forminfo->subform->pane->colored));
	}
	else
	    _xvf_init_form_warning("string selection 'colored' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->colored_optsel = TRUE;

	forminfo->subform->pane->radius_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.radius");
	if (forminfo->subform->pane->radius_struct != NULL)
	{
	    selection = forminfo->subform->pane->radius_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->radius_struct, KVF_FLOAT_DEF,
	                    &(forminfo->subform->pane->radius));
	}
	else
	    _xvf_init_form_warning("float selection 'radius' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->radius_selected = FALSE;

	forminfo->subform->pane->chain_name_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.chain_name");
	if (forminfo->subform->pane->chain_name_struct != NULL)
	{
	    selection = forminfo->subform->pane->chain_name_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->chain_name_struct, KVF_LIST_VAL,
	                    &(forminfo->subform->pane->chain_name));
	    kvf_get_attribute(forminfo->subform->pane->chain_name_struct, KVF_LIST_LABEL,
	                    &(forminfo->subform->pane->chain_name_label));
	}
	else
	    _xvf_init_form_warning("list selection 'chain_name' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->color_now_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.color_now");
	if (forminfo->subform->pane->color_now_struct == NULL)
	    _xvf_init_form_warning("pane action button 'color_now' on pane 'pane' of subform 'subform'");
	forminfo->subform->pane->color_now = FALSE;
	selection = forminfo->subform->pane->color_now_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->subform->pane->name_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.name");
	if (forminfo->subform->pane->name_struct != NULL)
	{
	    selection = forminfo->subform->pane->name_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->name_struct, KVF_LIST_VAL,
	                    &(forminfo->subform->pane->name));
	    kvf_get_attribute(forminfo->subform->pane->name_struct, KVF_LIST_LABEL,
	                    &(forminfo->subform->pane->name_label));
	}
	else
	    _xvf_init_form_warning("list selection 'name' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->num_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.num");
	if (forminfo->subform->pane->num_struct != NULL)
	{
	    selection = forminfo->subform->pane->num_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->num_struct, KVF_INT_DEF,
	                    &(forminfo->subform->pane->num));
	}
	else
	    _xvf_init_form_warning("integer selection 'num' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->num_optsel = FALSE;

	forminfo->subform->pane->color_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.color");
	if (forminfo->subform->pane->color_struct != NULL)
	{
	    selection = forminfo->subform->pane->color_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->color_struct, KVF_LIST_VAL,
	                    &(forminfo->subform->pane->color));
	    kvf_get_attribute(forminfo->subform->pane->color_struct, KVF_LIST_LABEL,
	                    &(forminfo->subform->pane->color_label));
	}
	else
	    _xvf_init_form_warning("list selection 'color' on pane 'pane' of subform 'subform'");

	forminfo->subform->pane->red_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.red");
	if (forminfo->subform->pane->red_struct != NULL)
	{
	    selection = forminfo->subform->pane->red_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->red_struct, KVF_FLOAT_DEF,
	                    &(forminfo->subform->pane->red));
	}
	else
	    _xvf_init_form_warning("float selection 'red' on pane 'pane' of subform 'subform'");


	forminfo->subform->pane->green_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.green");
	if (forminfo->subform->pane->green_struct != NULL)
	{
	    selection = forminfo->subform->pane->green_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->green_struct, KVF_FLOAT_DEF,
	                    &(forminfo->subform->pane->green));
	}
	else
	    _xvf_init_form_warning("float selection 'green' on pane 'pane' of subform 'subform'");


	forminfo->subform->pane->blue_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.blue");
	if (forminfo->subform->pane->blue_struct != NULL)
	{
	    selection = forminfo->subform->pane->blue_struct->Selptr;
	    kvf_get_attribute(forminfo->subform->pane->blue_struct, KVF_FLOAT_DEF,
	                    &(forminfo->subform->pane->blue));
	}
	else
	    _xvf_init_form_warning("float selection 'blue' on pane 'pane' of subform 'subform'");


	forminfo->subform->pane->blank1_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.blank1");
	if (forminfo->subform->pane->blank1_struct == NULL)
	    _xvf_init_form_warning("blank selection 'blank1' on pane 'pane' of subform 'subform'");
	forminfo->subform->pane->blank2_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.blank2");
	if (forminfo->subform->pane->blank2_struct == NULL)
	    _xvf_init_form_warning("blank selection 'blank2' on pane 'pane' of subform 'subform'");
	forminfo->subform->pane->blank3_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "form.subform.pane.blank3");
	if (forminfo->subform->pane->blank3_struct == NULL)
	    _xvf_init_form_warning("blank selection 'blank3' on pane 'pane' of subform 'subform'");

	return(TRUE);

}

/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_init_form_warning
| 
|       Purpose: Prints warning when code is out of sync with UIS file
| 
|         Input: string - item that cannot be found
| 
|        Output: none
|    Written By: conductor
|          Date: July 1, 1997
| Modifications:  
| 
------------------------------------------------------------*/
static void _xvf_init_form_warning(
    char *string)
{
	kfprintf(kstderr, "Warning: cannot find %s.\n", string);
	kfprintf(kstderr, "         UIS file out of sync with code\n");
	kfprintf(kstderr, "         generated by conductor in 'form_init.c'.\n");
}


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_init_form_error
| 
|       Purpose: Prints error when code is irreconcilable with UIS file
| 
|         Input: string1 - item that cannot be found
|                string2 - name of FormInfo structure
| 
|        Output: none
|    Written By: conductor
|          Date: July 1, 1997
| Modifications:  
| 
------------------------------------------------------------*/
static void _xvf_init_form_error(
    char *string1,
    char *string2)
{
	kfprintf(kstderr, "ERROR: cannot find %s.\n", string1);
	kfprintf(kstderr, "         UIS file out of sync with code\n");
	kfprintf(kstderr, "         generated by conductor in 'form_init.c'.\n");
	kfprintf(kstderr, "         Unable to initialize '%s' structure as defined in form_info.h; Aborting.\n", string2);
}


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_free_form
| 
|       Purpose: Frees the gui_info_struct
| 
|         Input: forminfo - ptr to _xvf_free_form_gui_struct
| 
|        Output: none
|    Written By: conductor
|          Date: July 1, 1997
| Modifications:  
| 
------------------------------------------------------------*/
void _xvf_free_form(
    gui_info_struct *forminfo)
{
	if (forminfo->subform->pane->license_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->license_struct);
	if (forminfo->subform->pane->help_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->help_struct);
	if (forminfo->subform->pane->quit_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->quit_struct);
	kfree_and_NULL(forminfo->subform->pane->i);
	if (forminfo->subform->pane->i_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->i_struct);
	kfree_and_NULL(forminfo->subform->pane->o);
	if (forminfo->subform->pane->o_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->o_struct);
	if (forminfo->subform->pane->rep_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->rep_struct);
	kfree_and_NULL(forminfo->subform->pane->rep_label);
	if (forminfo->subform->pane->res_name_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->res_name_struct);
	kfree_and_NULL(forminfo->subform->pane->res_name_label);
	kfree_and_NULL(forminfo->subform->pane->displayed);
	if (forminfo->subform->pane->displayed_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->displayed_struct);
	if (forminfo->subform->pane->display_now_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->display_now_struct);
	if (forminfo->subform->pane->res_num_struct != NULL)
	     kfree_and_NULL(forminfo->subform->pane->res_num_struct);
	kfree_and_NULL(forminfo->subform->pane->colored);
	if (forminfo->subform->pane->colored_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->colored_struct);
	if (forminfo->subform->pane->radius_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->radius_struct);
	if (forminfo->subform->pane->chain_name_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->chain_name_struct);
	kfree_and_NULL(forminfo->subform->pane->chain_name_label);
	if (forminfo->subform->pane->color_now_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->color_now_struct);
	if (forminfo->subform->pane->name_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->name_struct);
	kfree_and_NULL(forminfo->subform->pane->name_label);
	if (forminfo->subform->pane->num_struct != NULL)
	     kfree_and_NULL(forminfo->subform->pane->num_struct);
	if (forminfo->subform->pane->color_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->color_struct);
	kfree_and_NULL(forminfo->subform->pane->color_label);
	if (forminfo->subform->pane->red_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->red_struct);
	if (forminfo->subform->pane->green_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->green_struct);
	if (forminfo->subform->pane->blue_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->blue_struct);
	if (forminfo->subform->pane->blank1_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->blank1_struct);
	if (forminfo->subform->pane->blank2_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->blank2_struct);
	if (forminfo->subform->pane->blank3_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane->blank3_struct);
	kfree_and_NULL(forminfo->subform->pane);
	if (forminfo->subform->pane_struct != NULL)
	    kfree_and_NULL(forminfo->subform->pane_struct);


	if (forminfo->subform_struct != NULL)
	    kfree_and_NULL(forminfo->subform_struct);
	kfree_and_NULL(forminfo->subform);
	kfree_and_NULL(forminfo);
}
