/*
 * Khoros: $Id: io.c,v 1.2 1997/05/02 06:37:29 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: io.c,v 1.2 1997/05/02 06:37:29 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>             _static_routines()
   >>>>  Public:
   >>>>             public_routines()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "pdb2geom.h"

/*-----------------------------------------------------------
|
|  Routine Name: function_name - short description of function
|
|       Purpose: This should be a complete description that anyone
|                could understand;  it should have acceptable grammar
|                and correct spelling.
|
|         Input: argument1 - explanation
|                argument2 - explanation
|                argument3 - explanation
|
|        Output: argument4 - explanation
|                argument5 - explanation
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By:
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

float atomic_table[][4] = {
				C_rad,		0.0, 1.0, 0.0,
				O_rad,		1.0, 0.0, 0.0,
				N_rad,		0.0, 0.0, 1.0,
				H_rad,		1.0, 1.0, 1.0,
				S_rad,		1.0, 1.0, 0.0,
				P_rad,		1.0, 1.0, 0.5,
				Cl_rad,		1.0, 1.0, 0.5,
				I_rad,		0.5, 1.0, 0.5,
				F_rad,		0.0, 1.0, 1.0,
				Other_rad,	1.0, 0.0, 1.0,
		          };

LIST *names = NULL;
LIST *chain_names = NULL;
LIST *residue_names = NULL;

DATA molecule;

void free_names(LIST **cur) {
	LIST *scan;
	LIST *temp;

	for (scan = *cur; scan != NULL; scan = temp) {
		kfree_and_NULL(scan->name);
		temp = scan->next;

		kfree_and_NULL(scan);
	}

	*cur = NULL;
}

void add_name(LIST **cur, char *name) {
	LIST *temp;

	for (temp = *cur; temp != NULL; temp = temp->next) {
		if (kstrcmp(temp->name,name) == 0) {
			break;
		}
	}

	if (temp == NULL) {
		temp = (LIST *)kmalloc(sizeof(*temp));

		temp->name = kstrdup(name);
		temp->next = *cur;
		*cur = temp;
	}
}

int compare_names(const void *e1, const void *e2) {
	return(kstrcmp((*(LIST **)e1)->name,(*(LIST **)e2)->name));
}

void sort_names(LIST **cur) {
	LIST *scan;
	int i,num;
	LIST **temp;

	num = 0;
	for (scan = *cur; scan != NULL; scan = scan->next) {
		num++;
	}

	temp = (LIST **)kmalloc(num*sizeof(*temp));

	num = 0;
	for (scan = *cur; scan != NULL; scan = scan->next) {
		temp[num] = scan;
		num++;
	}

	qsort(temp,num,sizeof(*temp),compare_names);
		
	*cur = temp[0];
	for (i = 0; i < num-1; i++) {
		temp[i]->next = temp[i+1];
	}
	temp[i]->next = NULL;

	kfree_and_NULL(temp);
}

void freePDB(void) {
	int i,num;

	num = molecule.numatoms;

	for (i = 0; i < num; i++) {
		kfree_and_NULL(molecule.atom[i].name);
		kfree_and_NULL(molecule.atom[i].chain_name);
		kfree_and_NULL(molecule.atom[i].residue_name);
	}

	kfree_and_NULL(molecule.atom);
	kfree_and_NULL(molecule.bond);

	free_names(&names);
	free_names(&chain_names);
	free_names(&residue_names);
}

void readPDB(kfile *in) {
	int i;
	int num,num_block;
	char line[MAXLINE];
	char atom[MAXNAME];
	int atom_num;
	char atom_name[MAXNAME];
	char residue_name[MAXNAME];
	char chain_id[MAXNAME];
	int residue_num;
	float x,y,z;
	LIST *scan;
	TEMPLATE templ;

	num = 0;
	num_block = NUMBLOCK;
	molecule.atom = NULL;

	while (kfgets(line,sizeof(line),in) != NULL) {
		atom[0] = '\0';
		atom_num = 0;
		atom_name[0] = '\0';
		residue_name[0] = '\0';
		residue_num = 0;
		chain_id[0] = '\0';

		ksscanf(&line[ 0],"%6s",atom);
		ksscanf(&line[ 6],"%d",&atom_num);
		ksscanf(&line[12],"%4s",atom_name);
		ksscanf(&line[17],"%3s",residue_name);
		ksscanf(&line[21],"%1s",chain_id);
		ksscanf(&line[22],"%d",&residue_num);
		ksscanf(&line[30],"%g",&x);
		ksscanf(&line[38],"%g",&y);
		ksscanf(&line[46],"%g",&z);

		if (strcmp(atom,"ATOM") == 0 ||
		    strcmp(atom,"atom") == 0 ||
		    strcmp(atom,"HETATM") == 0 ||
		    strcmp(atom,"hetatm") == 0) {
			char a;

			if (num_block == NUMBLOCK) {
				molecule.atom = (ATOM *)krealloc(molecule.atom,(num+NUMBLOCK)*sizeof(*(molecule.atom)));

				if (molecule.atom == NULL) {
					kfprintf(kstderr,"Unable to allocate memory for %d atoms\n",num+NUMBLOCK);
					kexit(KEXIT_FAILURE);
				}

				num_block = 0;
			}

			molecule.atom[num].match[DISP] = TRUE;
			molecule.atom[num].match[COL] = TRUE;

			if (atom_name[1] == 'L' ||
			    atom_name[1] == 'l') {
				atom_name[2] = '\0';
			} else {
				atom_name[1] = '\0';
			}

			molecule.atom[num].name = kstrdup(atom_name);
			molecule.atom[num].num = atom_num;
			add_name(&names,atom_name);

			molecule.atom[num].chain_name = kstrdup(chain_id);
			add_name(&chain_names,chain_id);

			molecule.atom[num].residue_name = kstrdup(residue_name);
			molecule.atom[num].residue_num = residue_num;
			add_name(&residue_names,residue_name);

			molecule.atom[num].pos[0] = x;
			molecule.atom[num].pos[1] = y;
			molecule.atom[num].pos[2] = z;

			a = molecule.atom[num].name[0];
			if (islower(a)) a = toupper(a);

			switch (a) {
			case 'C':
				a = molecule.atom[num].name[1];
				if (islower(a)) a = toupper(a);

				if (a == 'L') atom_num = Cl;
				else	      atom_num = C;

				break;

			case 'O':
				atom_num = O; break;

			case 'N':
				atom_num = N; break;

			case 'H':
				atom_num = H; break;

			case 'S':
				atom_num = S; break;

			case 'P':
				atom_num = P; break;

			case 'I':
				atom_num = I; break;

			case 'F':
				atom_num = F; break;

			default:
				atom_num = Other; break;
			}

			molecule.atom[num].num = atom_num;

			molecule.atom[num].radius = atomic_table[atom_num][0];

			molecule.atom[num].color[0] = atomic_table[atom_num][1];
			molecule.atom[num].color[1] = atomic_table[atom_num][2];
			molecule.atom[num].color[2] = atomic_table[atom_num][3];

			num++;
			num_block++;
		}
	}

	molecule.numatoms = num;

	make_bonds();

	sort_names(&names);
	sort_names(&chain_names);
	sort_names(&residue_names);
}

void make_bonds(void) {
	int i,j;
	int numatoms;
	int numbonds;
	int numbonds_this_atom;
	int num_block;
	float dx,dy,dxy,dz;
	double dhypot;
	double a1,a2;

	numatoms = molecule.numatoms;

	numbonds = 0;
	num_block = NUMBLOCK;
	molecule.bond = NULL;

	for (i = numatoms-1; i > 0; i--) {
	    numbonds_this_atom = 0;

	    for (j = i-1; j >= 0; j--) {

		/*
		* The outer loop index 'i' is AFTER the inner loop 'j': 'i'
		* leads 'j' in the list: since hydrogens traditionally follow
		* the heavy atom they're bonded to, this makes it easy to quit
		* bonding to hydrogens after one bond is made by breaking out of
		* the 'j' loop when 'i' is a hydrogen and we make a bond to it.
		* Working backwards like this makes it easy to find the heavy
		* atom that came 'just before' the Hydrogen. mp 
		*/
		if (num_block == NUMBLOCK) {
			molecule.bond = (BOND *)krealloc(molecule.bond,(numbonds+NUMBLOCK)*sizeof(*(molecule.bond)));

			if (molecule.bond == NULL) {
				kfprintf(kstderr,"Unable to allocate memory for %d bonds\n",numbonds+NUMBLOCK);
				kexit(KEXIT_FAILURE);
			}

			num_block = 0;
		}

		/* never bond hydrogens to each other... */
		if (molecule.atom[i].num == H && molecule.atom[j].num == H)
			continue;

		dx = molecule.atom[i].pos[0] - molecule.atom[j].pos[0];
		dx = ABS(dx);
		if (dx > BOND_THRESHOLD) continue;

		dy = molecule.atom[i].pos[1] - molecule.atom[j].pos[1];
		dy = ABS(dy);
		if (dy > BOND_THRESHOLD) continue;

		a1 = dx;
		a2 = dy;
		dhypot = khypot(a1,a2);
		dxy = dhypot;
		if (dxy > BOND_THRESHOLD) continue;

		dz = molecule.atom[i].pos[2] - molecule.atom[j].pos[2];
		dz = ABS(dz);
		if (dz > BOND_THRESHOLD) continue;

		a1 = dxy;
		a2 = dz;
		dhypot = khypot(a1,a2);
		if (dhypot > BOND_THRESHOLD) continue;

		molecule.bond[numbonds].end[0] = j;
		molecule.bond[numbonds].end[1] = i;

		numbonds++;
		num_block++;
		numbonds_this_atom++;

		if (molecule.atom[i].num == H) break; /* only one bond per H */
	    }
	}

	molecule.numbonds = numbonds;
}

int matchPDB(TEMPLATE *templ) {
	char *name;
	int num;
	char *chain_name;
	char *residue_name;
	int residue_num;
	int type;
	int i,n;
	int count;

	n = molecule.numatoms;

	name = templ->name;
	num = templ->num;
	chain_name = templ->chain_name;
	residue_name = templ->residue_name;
	residue_num = templ->residue_num;

	type = templ->type;

	count = 0;
	for (i = 0; i < n; i++) {
		if (
(name == NULL         || kstrcmp(name,molecule.atom[i].name) == 0               ) &&
(num == -1            || num == molecule.atom[i].num                            ) &&
(chain_name == NULL   || kstrcmp(chain_name,molecule.atom[i].chain_name) == 0   ) &&
(residue_name == NULL || kstrcmp(residue_name,molecule.atom[i].residue_name) == 0) &&
(residue_num == -1    || residue_num == molecule.atom[i].residue_num            )
		   ) {
			molecule.atom[i].match[type] = TRUE;
			count++;
		} else {
			molecule.atom[i].match[type] = FALSE;
		}
	}

	return(count);
}

void colorPDB(float r, float g, float b) {
	int i,n;

	n = molecule.numatoms;

	for (i = 0; i < n; i++) {
		if (molecule.atom[i].match[COL] == TRUE) {
			molecule.atom[i].color[0] = r;
			molecule.atom[i].color[1] = g;
			molecule.atom[i].color[2] = b;
		}
	}
}

void radiusPDB(float radius) {
	int i,n;

	n = molecule.numatoms;

	for (i = 0; i < n; i++) {
		if (molecule.atom[i].match[COL] == TRUE) {
			molecule.atom[i].radius = radius;
		}
	}
}

void display(kgeom_object *out, char *display) {
	if (kstrcmp(display,"Sticks") == 0) {
		atomic_lines(out);
	} else
	if (kstrcmp(display,"Balls") == 0) {
		atomic_spheres(out,1.0);
	} else
	if (kstrcmp(display,"Sticks & Balls") == 0) {
		atomic_lines(out);
		atomic_spheres(out,0.75);
	} else {
		kfprintf(kstderr,"Unknown display type: %s\n",display);
	}
}
