/*
 * Khoros: $Id: lines.c,v 1.1 1997/05/02 02:33:13 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lines.c,v 1.1 1997/05/02 02:33:13 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>             _static_routines()
   >>>>  Public:
   >>>>             public_routines()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "pdb2geom.h"


/*-----------------------------------------------------------
|
|  Routine Name: function_name - short description of function
|
|       Purpose: This should be a complete description that anyone
|                could understand;  it should have acceptable grammar
|                and correct spelling.
|
|         Input: argument1 - explanation
|                argument2 - explanation
|                argument3 - explanation
|
|        Output: argument4 - explanation
|                argument5 - explanation
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By:
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void pack_write_lines(char *match, float *points, float *colors,
		      int num, kgeom_object *out) {
	int i,i3,cur,cur3;
	kgeom_primitive *prim;

	cur = 0;
	cur3 = 0;
	for (i = 0, i3 = 0; i < num; i++, i3 += 3) {
		if (match[i] == TRUE) {
			colors[cur3+0] = colors[i3+0];
			colors[cur3+1] = colors[i3+1];
			colors[cur3+2] = colors[i3+2];

			points[cur3+0] = points[i3+0];
			points[cur3+1] = points[i3+1];
			points[cur3+2] = points[i3+2];

			cur++;
			cur3 += 3;
		}
	}

	if (cur != 0) {
		prim = kgeom_new_primitive(KGEOM_POLYLINE_DISJOINT);

		prim->lines.nverts = cur;
		prim->lines.locs = points;
		prim->lines.cols = colors;

		kgeom_write_primitive(out,prim);
	}
}

void atomic_lines(kgeom_object *out) {
	int i,num;
	int a1,a2;
	int cur,cur3;
	int count;
	char *match;
	float *points;
	float *colors;

	num = molecule.numbonds;

	match = (char *)kmalloc(2*num*sizeof(*match));
	if (match == NULL) {
		kfprintf(kstderr,"Unable to allocate enough memory to draw lines\n");
		kexit(KEXIT_FAILURE);
	}

	points = (float *)kmalloc(2*num*3*sizeof(*points));
	if (points == NULL) {
		kfprintf(kstderr,"Unable to allocate enough memory to draw lines\n");
		kexit(KEXIT_FAILURE);
	}

	colors = (float *)kmalloc(2*num*3*sizeof(*colors));
	if (colors == NULL) {
		kfprintf(kstderr,"Unable to allocate enough memory to draw lines\n");
		kexit(KEXIT_FAILURE);
	}

	cur = 0;
	cur3 = 0;
	for (i = 0; i < num; i++) {
		a1 = molecule.bond[i].end[0];
		a2 = molecule.bond[i].end[1];

		match[cur] = molecule.atom[a1].match[DISP];

		colors[cur3+0] = molecule.atom[a1].color[0];
		colors[cur3+1] = molecule.atom[a1].color[1];
		colors[cur3+2] = molecule.atom[a1].color[2];

		points[cur3+0] = molecule.atom[a1].pos[0];
		points[cur3+1] = molecule.atom[a1].pos[1];
		points[cur3+2] = molecule.atom[a1].pos[2];
		cur++;
		cur3 += 3;

		match[cur] = molecule.atom[a1].match[DISP];

		colors[cur3+0] = molecule.atom[a1].color[0];
		colors[cur3+1] = molecule.atom[a1].color[1];
		colors[cur3+2] = molecule.atom[a1].color[2];

		points[cur3+0] = (molecule.atom[a1].pos[0] + molecule.atom[a2].pos[0]) * 0.5;
		points[cur3+1] = (molecule.atom[a1].pos[1] + molecule.atom[a2].pos[1]) * 0.5;
		points[cur3+2] = (molecule.atom[a1].pos[2] + molecule.atom[a2].pos[2]) * 0.5;
		cur++;
		cur3 += 3;
	}

	pack_write_lines(match,points,colors,cur,out);

	cur = 0;
	cur3 = 0;
	for (i = 0; i < num; i++) {
		a1 = molecule.bond[i].end[0];
		a2 = molecule.bond[i].end[1];

		match[cur] = molecule.atom[a2].match[DISP];

		colors[cur3+0] = molecule.atom[a2].color[0];
		colors[cur3+1] = molecule.atom[a2].color[1];
		colors[cur3+2] = molecule.atom[a2].color[2];

		points[cur3+0] = molecule.atom[a2].pos[0];
		points[cur3+1] = molecule.atom[a2].pos[1];
		points[cur3+2] = molecule.atom[a2].pos[2];
		cur++;
		cur3 += 3;

		match[cur] = molecule.atom[a2].match[DISP];

		colors[cur3+0] = molecule.atom[a2].color[0];
		colors[cur3+1] = molecule.atom[a2].color[1];
		colors[cur3+2] = molecule.atom[a2].color[2];

		points[cur3+0] = (molecule.atom[a1].pos[0] + molecule.atom[a2].pos[0]) * 0.5;
		points[cur3+1] = (molecule.atom[a1].pos[1] + molecule.atom[a2].pos[1]) * 0.5;
		points[cur3+2] = (molecule.atom[a1].pos[2] + molecule.atom[a2].pos[2]) * 0.5;
		cur++;
		cur3 += 3;
	}

	pack_write_lines(match,points,colors,cur,out);

	kfree_and_NULL(match);
	kfree_and_NULL(points);
	kfree_and_NULL(colors);
}
