 /*
  * Khoros: $Id: pdb2geom.c,v 1.1.2.1 1997/06/07 20:36:19 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: pdb2geom.c,v 1.1.2.1 1997/06/07 20:36:19 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for pdb2geom
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "pdb2geom.h"

clui_info_struct *clui_info = NULL;

gui_info_struct *gui_info;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Generate geometry from PDB models
|
|       Purpose: main program for pdb2geom
|
|         Input:
|		char *clui_info->i_file; {PDB description}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Geometry representing the PDB}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int   clui_info->rep_list; {How to represent the PDB}
|		char *clui_info->rep_label;
|		int   clui_info->rep_flag; {TRUE if -rep specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

	kform  *form;            /* form tree representing UIS        */
	char   *form_fullpath;   /* for use when [-form] is specified */
/* -main_variable_list */
	kfile *in;
	kgeom_object *out;
/* -main_variable_list_end */

	khoros_init(argc, argv, "GEOMETRY", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$GEOMETRY/objects/xvroutine/pdb2geom");
	kexit_handler(pdb2geom_free_args, NULL);

/* -main_get_args_call */
	kclui_init("GEOMETRY", "pdb2geom", KGEN_XVROUTINE,  &clui_uis_spec,
		   pdb2geom_usage_additions, pdb2geom_get_args,
 		   pdb2geom_free_args);
/* -main_get_args_call_end */

	/*
	 * Initialize xvwidgets lib
	 */
	if (!(xvw_initialize(XVW_MENUS_XVFORMS)))
	{
	     kinfo(KSTANDARD, "Could not open connection to server");
	     kexit(KEXIT_FAILURE);
	}

	/*
	 * Allocate the GUI Info structure 
	 */
	gui_info = (gui_info_struct *)
				kcalloc ((size_t) 1, sizeof(gui_info_struct));
	/*
	 * Create the forms for pdb2geom 
	 */
	if (clui_info->form_flag)
	{
	    form_fullpath   = kfullpath(clui_info->form_file, NULL, NULL);
	    form = xvf_create_form(form_fullpath, KNONE, run_form, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	    kfree_and_NULL(form_fullpath);
	}
	else
	{
	    form = xvf_build_form(&gui_uis_spec, KNONE, run_form, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	}
	if (form == NULL)
	   kexit(KEXIT_FAILURE);

	/*
	 * Initialize the GUI Info structure 
	 */
	if (!(_xvf_init_form(form, gui_info)))
	    kexit(KEXIT_FAILURE);

/* -main_before_lib_call */
	xvf_set_attribute(gui_info->subform->pane->i_struct,XVF_FILE_NAME,
			  clui_info->i_file);
	gui_info->subform->pane->i = kstrdup(clui_info->i_file);

	xvf_set_attribute(gui_info->subform->pane->o_struct,XVF_FILE_NAME,
			  clui_info->o_file);
	gui_info->subform->pane->o = kstrdup(clui_info->o_file);

	xvf_set_attribute(gui_info->subform->pane->rep_struct,XVF_LIST_VAL,
			  clui_info->rep_list);
	gui_info->subform->pane->rep = clui_info->rep_list;
	gui_info->subform->pane->rep_label = kstrdup(clui_info->rep_label);

	interface.in = kstrdup(clui_info->i_file);
	interface.out = kstrdup(clui_info->o_file);
	interface.rep = kstrdup(clui_info->rep_label);

	do_all(&interface);
/* -main_before_lib_call_end */

/* -main_library_call */
	xvf_run_form();
/* -main_library_call_end */

/* -main_after_lib_call */
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: pdb2geom_usage_additions
| 
|       Purpose: Prints usage additions in pdb2geom_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname pdb2geom
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void pdb2geom_usage_additions(void)
{
	kfprintf(kstderr, "\tGenerate geometry from PDB models\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: pdb2geom_free_args
| 
|       Purpose: Frees CLUI struct allocated in pdb2geom_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname pdb2geom
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
pdb2geom_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->rep_label);
		kfree_and_NULL(clui_info->form_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
