 /*
  * Khoros: $Id: pdb2geom.h,v 1.1.2.1 1997/06/07 20:36:20 mirage Exp $
  */

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>>       Purpose: Include file for pdb2geom
   >>>> 
   >>>>    Written By: 
   >>>> 
   >>>> Modifications: 
   >>>> 
   >>>>          Date: July 1, 1997
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#ifndef _pdb2geom_h_
#define _pdb2geom_h_

        /*-------------------------------------*
         |       #includes
        ---------------------------------------*/

#include <geometry.h>
#include <xvinclude.h>
#include "form_info.h"

/* -include_includes */
#include <malloc.h>
/* -include_includes_end */


        /*-------------------------------------*
         |       #defines
        ---------------------------------------*/

/* -include_defines */
#define MAXLINE		8192
#define MAXNAME		1024

#define NUMBLOCK 1000

#define DISP		0
#define COL		1

#define C	0
#define O	1
#define N	2
#define H	3
#define S	4
#define P	5
#define Cl	6
#define I	7
#define F	8
#define Other	9

#define C_rad 	0.77 
#define O_rad 	0.6
#define N_rad 	0.7 
#define H_rad 	0.37 
#define S_rad 	1.0
#define P_rad 	1.1       
#define Cl_rad 	0.99      
#define I_rad 	1.33
#define F_rad 	0.64
#define Other_rad  1.5

#define BOND_THRESHOLD	1.85

#define ABS(x)	(((x) >= 0) ? (x) : -(x))
/* -include_defines_end */

        /*-------------------------------------*
         |       typedefs
        ---------------------------------------*/

typedef struct _clui_info_struct {

/*
 *  PDB description (required infile)
 */
char	*i_file;	/* PDB description FILENAME */
int	 i_flag;	/* PDB description FLAG */

/*
 *  Geometry representing the PDB (required outfile)
 */
char	*o_file;	/* Geometry representing the PDB FILENAME */
int	 o_flag;	/* Geometry representing the PDB FLAG */

/*
 *  How to represent the PDB (Required list)
 */
int	 rep_list;	/* How to represent the PDB LIST INDEX */
char	 *rep_label;	/* How to represent the PDB LIST LABEL */
int	 rep_flag;	/* How to represent the PDB FLAG */

/*
 *  Alternate UIS file (optional infile)
 */
char	*form_file;	/* Alternate UIS file FILENAME */
int	 form_flag;	/* Alternate UIS file FLAG */

/*
 *  (x,y) position for automatic GUI placement */

int	 x_int;		/* x position INT */
int	 x_flag;	/* x position FLAG */

int	 y_int;		/* y position INT */
int	 y_flag;	/* y position FLAG */

} clui_info_struct;

/* -include_typedefs */
typedef struct {
	char *in;
	char *out;
	char *rep;
} STATE;

typedef struct {
	char *name;
	float r,g,b;
} COLOR;

typedef struct list LIST;

struct list {
	char *name;
	LIST *next;
};

typedef struct {
	char match[2];
	char *name;
	int num;
	char *chain_name;
	char *residue_name;
	int residue_num;
	float pos[3];
	float color[3];
	float radius;
} ATOM;

typedef struct {
	int end[2];
} BOND;

typedef struct {
	char *name;
	int num;
	char *chain_name;
	char *residue_name;
	int residue_num;
	int type;
} TEMPLATE;

typedef struct {
	int numatoms;
	ATOM *atom;
	int numbonds;
	BOND *bond;
} DATA;
/* -include_typedefs_end */


        /*-------------------------------------*
         |       global variable declarations
        ---------------------------------------*/

extern clui_info_struct *clui_info;
extern uis_struct clui_uis_spec;
extern gui_info_struct *gui_info;

extern uis_struct gui_uis_spec;
/* -include_variables */
extern STATE interface;

extern LIST *names;
extern LIST *chain_names;
extern LIST *residue_names;

extern DATA molecule;
/* -include_variables_end */


        /*-------------------------------------*
         |       macros
        ---------------------------------------*/

/* -include_macros */
/* -include_macros_end */

        /*-------------------------------------*
         |       routine definitions
        ---------------------------------------*/

int main PROTO((int, char **));
void pdb2geom_get_args PROTO((kaddr));
void pdb2geom_usage_additions PROTO((void));
void pdb2geom_free_args PROTO((kexit_status, kaddr));

/* -include_routines */
void setup_list PROTO((kform_struct *, LIST *));
void setup_template PROTO((subform_pane *, int, TEMPLATE *));
void do_all PROTO((STATE *));
void do_read PROTO((STATE *));
void do_display PROTO((STATE *));
void do_match PROTO((TEMPLATE *, subform_pane *));

void free_names PROTO((LIST **));
void add_name PROTO((LIST **, char *));
int compare_names PROTO((const void *, const void *));
void sort_names PROTO((LIST **));

void freePDB PROTO((void));
void readPDB PROTO((kfile *));
void make_bonds PROTO((void));
int matchPDB PROTO((TEMPLATE *));
void colorPDB PROTO((float, float, float));
void radiusPDB PROTO((float));

void display PROTO((kgeom_object *, char *));

void pack_write_lines PROTO((char *, float *, float *, int, kgeom_object *));
void atomic_lines PROTO((kgeom_object *));

void pack_write_spheres PROTO((char *, float *, float *, float *, int, kgeom_object *));
void atomic_spheres PROTO((kgeom_object *, float));
/* -include_routines_end */

#endif
