/*
 * Khoros: $Id: spheres.c,v 1.1 1997/05/02 02:33:16 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: spheres.c,v 1.1 1997/05/02 02:33:16 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>             _static_routines()
   >>>>  Public:
   >>>>             public_routines()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "pdb2geom.h"


/*-----------------------------------------------------------
|
|  Routine Name: function_name - short description of function
|
|       Purpose: This should be a complete description that anyone
|                could understand;  it should have acceptable grammar
|                and correct spelling.
|
|         Input: argument1 - explanation
|                argument2 - explanation
|                argument3 - explanation
|
|        Output: argument4 - explanation
|                argument5 - explanation
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By:
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void pack_write_spheres(char *match, float *points, float *colors,
		        float *radii, int num, kgeom_object *out) {
	int i,i3;
	int cur,cur3;
	kgeom_primitive *prim;

	cur = 0;
	cur3 = 0;
	for (i = 0, i3 = 0; i < num; i++, i3 += 3) {
		if (match[i] == TRUE) {
			colors[cur3+0] = colors[i3+0];
			colors[cur3+1] = colors[i3+1];
			colors[cur3+2] = colors[i3+2];

			points[cur3+0] = points[i3+0];
			points[cur3+1] = points[i3+1];
			points[cur3+2] = points[i3+2];

			radii[cur] = radii[i];
			cur++;
			cur3 += 3;
		}
	}

	if (cur != 0) {
		prim = kgeom_new_primitive(KGEOM_SPHERES);

		prim->spheres.nverts = cur;
		prim->spheres.locs = points;
		prim->spheres.cols = colors;
		prim->spheres.radii = radii;

		kgeom_write_primitive(out,prim);
	}
}

void atomic_spheres(kgeom_object *out, float scale) {
	int i,num;
	int cur,cur3;
	int count;
	char *match;
	float *points;
	float *colors;
	float *radii;

	num = molecule.numatoms;

	match = (char *)kmalloc(num*sizeof(*match));
	if (match == NULL) {
		kfprintf(kstderr,"Unable to allocate enough memory to draw balls\n");
		kexit(KEXIT_FAILURE);
	}

	points = (float *)kmalloc(num*3*sizeof(*points));
	if (points == NULL) {
		kfprintf(kstderr,"Unable to allocate enough memory to draw balls\n");
		kexit(KEXIT_FAILURE);
	}

	colors = (float *)kmalloc(num*3*sizeof(*colors));
	if (colors == NULL) {
		kfprintf(kstderr,"Unable to allocate enough memory to draw balls\n");
		kexit(KEXIT_FAILURE);
	}

	radii = (float *)kmalloc(num*sizeof(*radii));
	if (radii == NULL) {
		kfprintf(kstderr,"Unable to allocate enough memory to draw balls\n");
		kexit(KEXIT_FAILURE);
	}

	cur = 0;
	cur3 = 0;
	for (i = 0; i < num; i++) {
		match[cur] = molecule.atom[i].match[DISP];

		colors[cur3+0] = molecule.atom[i].color[0];
		colors[cur3+1] = molecule.atom[i].color[1];
		colors[cur3+2] = molecule.atom[i].color[2];

		points[cur3+0] = molecule.atom[i].pos[0];
		points[cur3+1] = molecule.atom[i].pos[1];
		points[cur3+2] = molecule.atom[i].pos[2];

		radii[cur] = scale * molecule.atom[i].radius;
		cur++;
		cur3 += 3;
	}

	pack_write_spheres(match,points,colors,radii,cur,out);

	kfree_and_NULL(match);
	kfree_and_NULL(points);
	kfree_and_NULL(colors);
	kfree_and_NULL(radii);
}
