
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 

#ifndef __camera_h_
#define __camera_h_

#include "geometry/geom.h"

#define CAMERA_TYPE_MONOSCOPIC 0
#define CAMERA_TYPE_STEREOSCOPIC 1
#define CYCLOPS_POINT 0
#define LEFT_EYE_POINT 1
#define RIGHT_EYE_POINT 2

int set_projection PROTO((int));
int get_projection PROTO((int *));

int set_eye PROTO((vertex_3d *));
int get_eye PROTO((vertex_3d *));

int set_at PROTO((vertex_3d *));
int get_at PROTO((vertex_3d *));

int set_up PROTO((vertex_3d *));
int get_up PROTO((vertex_3d *));

int set_fov PROTO((float));
int get_fov PROTO((float *));

int set_user_hither PROTO((float));
int get_user_hither PROTO((float *));

int set_render_hither PROTO((float));
int get_render_hither PROTO((float *));

int set_user_yon PROTO((float));
int get_user_yon PROTO((float *));

int set_render_yon PROTO((float));
int get_render_yon PROTO((float *));

int set_zmin PROTO((float));
int get_zmin PROTO((float *));

int set_zproj PROTO((float));
int get_zproj PROTO((float *));

int get_view_xform PROTO((matrix4x4 *));
int set_view_xform PROTO((matrix4x4 *));

int compute_view_xform PROTO((void));

int set_world_to_view_scale PROTO((float, float));
int get_world_to_view_scale PROTO((float *, float *));

int set_hither_clip_flag PROTO((int));
int get_hither_clip_flag PROTO((int *));

int set_yon_clip_flag PROTO((int));
int get_yon_clip_flag PROTO((int *));

int compute_perspective_projection PROTO((matrix4x4 *, vertex_3d *, vertex_3d *,vertex_3d *,float *,float *,float *,float *, float *));
int compute_parallelprojection PROTO((matrix4x4 *, vertex_3d *, vertex_3d *,vertex_3d *,float *,float *,float *,float *, float *));

int get_view_rotation_matrix PROTO((matrix4x4 *));

int set_normalized_view PROTO((void));
int set_startup_default_camera_parms PROTO((void));

int software_set_camera PROTO((void));

/* stereo stuff */
int set_camera_type PROTO((int));
int get_camera_type PROTO((void));
int set_focal_length PROTO((float));
int get_focal_length PROTO((float *));
int set_eye_separation PROTO((float));
int get_eye_separation PROTO((float *));
int set_left_eye PROTO((vertex_3d *));
int get_left_eye PROTO((vertex_3d *));
int set_right_eye PROTO((vertex_3d *));
int get_right_eye PROTO((vertex_3d *));
int set_leftright_flip PROTO((int));
int get_leftright_flip PROTO((void));
int set_focal_point PROTO((vertex_3d *));
int get_focal_point PROTO((vertex_3d *));
int compute_stereo_camera_parms PROTO((void));


#endif
