
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 
 /*
  * Khoros: $Id: do_Cameras.c,v 1.1 1997/04/19 00:14:45 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_Cameras.c,v 1.1 1997/04/19 00:14:45 mirage Exp $";
#endif

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for pane Cameras
   >>>> 
   >>>>  Private: 
   >>>> 	Cameras_projection_cycle
   >>>> 	Cameras_eye_x
   >>>> 	Cameras_up_x
   >>>> 	Cameras_at_x
   >>>> 	Cameras_eye_y
   >>>> 	Cameras_up_y
   >>>> 	Cameras_at_y
   >>>> 	Cameras_eye_z
   >>>> 	Cameras_up_z
   >>>> 	Cameras_at_z
   >>>> 	Cameras_hither_clip_form_loc
   >>>> 	Cameras_yon_clip_form_loc
   >>>> 	Cameras_back_clip_form_on_off
   >>>> 	Cameras_fov_input
   >>>> 	Cameras_aspect
   >>>> 	Cameras_normalize_cam
   >>>> 	Cameras_reset_camera
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "render.h"

/**
  * camera.h has routine protos for accessing the camera parms.
**/
#include "camera.h"

/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_projection_cycle
| 
|       Purpose: Do routine which is called when
|                  cycle selection projection_cycle is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_projection_cycle(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
    int proj;
    proj = Cameras_info->projection_cycle;
    /* range check?? */
    set_projection(proj);
    compute_view_xform(); /* recompute and update internal view matrix. */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_eye_x
| 
|       Purpose: Do routine which is called when
|                  float selection eye_x is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_eye_x(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
    vertex_3d t;
    get_eye(&t);
    t.v[0] = Cameras_info->eye_x;
    set_eye(&t);
    compute_view_xform(); /* recompute and update internal view matrix. */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_up_x
| 
|       Purpose: Do routine which is called when
|                  float selection up_x is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_up_x(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
    vertex_3d t;
    get_up(&t);
    t.v[0] = Cameras_info->up_x;
    set_up(&t);
    compute_view_xform(); /* recompute and update internal view matrix. */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_at_x
| 
|       Purpose: Do routine which is called when
|                  float selection at_x is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_at_x(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
    vertex_3d t;
    get_at(&t);
    t.v[0] = Cameras_info->at_x;
    set_at(&t);
    compute_view_xform(); /* recompute and update internal view matrix. */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_eye_y
| 
|       Purpose: Do routine which is called when
|                  float selection eye_y is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_eye_y(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
    vertex_3d t;
    get_eye(&t);
    t.v[1] = Cameras_info->eye_y;
    set_eye(&t);
    compute_view_xform(); /* recompute and update internal view matrix. */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_up_y
| 
|       Purpose: Do routine which is called when
|                  float selection up_y is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_up_y(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
    vertex_3d t;
    get_up(&t);
    t.v[1] = Cameras_info->up_y;
    set_up(&t);
    compute_view_xform(); /* recompute and update internal view matrix. */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_at_y
| 
|       Purpose: Do routine which is called when
|                  float selection at_y is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_at_y(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
    vertex_3d t;
    get_at(&t);
    t.v[1] = Cameras_info->at_y;
    set_at(&t);
    compute_view_xform(); /* recompute and update internal view matrix. */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_eye_z
| 
|       Purpose: Do routine which is called when
|                  float selection eye_z is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_eye_z(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
    vertex_3d t;
    get_eye(&t);
    t.v[2] = Cameras_info->eye_z;
    set_eye(&t);
    compute_view_xform(); /* recompute and update internal view matrix. */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_up_z
| 
|       Purpose: Do routine which is called when
|                  float selection up_z is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_up_z(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
    vertex_3d t;
    get_up(&t);
    t.v[2] = Cameras_info->up_z;
    set_up(&t);
    compute_view_xform(); /* recompute and update internal view matrix. */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_at_z
| 
|       Purpose: Do routine which is called when
|                  float selection at_z is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_at_z(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
    vertex_3d t;
    get_at(&t);
    t.v[2] = Cameras_info->at_z;
    set_at(&t);
    compute_view_xform(); /* recompute and update internal view matrix. */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_hither_clip_form_loc
| 
|       Purpose: Do routine which is called when
|                  float selection hither_clip_form_loc is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_hither_clip_form_loc(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */

    float hither;

    hither = Cameras_info->hither_clip_form_loc;
    set_user_hither(hither);
    compute_view_xform(); /* recompute and update internal view matrix. */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_yon_clip_form_loc
| 
|       Purpose: Do routine which is called when
|                  float selection yon_clip_form_loc is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_yon_clip_form_loc(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
    float yon;
    yon = Cameras_info->yon_clip_form_loc;
    set_user_yon(yon);
    compute_view_xform(); /* recompute and update internal view matrix. */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_back_clip_form_on_off
| 
|       Purpose: Do routine which is called when
|                  logical selection back_clip_form_on_off is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_back_clip_form_on_off(
     Render_Cameras *Cameras_info)
{
    int flag;
	/* PUT YOUR CODE HERE ! */
    flag = Cameras_info->back_clip_form_on_off;
    set_yon_clip_flag(flag);
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_fov_input
| 
|       Purpose: Do routine which is called when
|                  float selection fov_input is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_fov_input(
     Render_Cameras *Cameras_info)
{
    /* PUT YOUR CODE HERE ! */
    float fov;
    fov = Cameras_info->fov_input;
    set_fov(fov);
    compute_view_xform(); /* recompute and update internal view matrix. */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_aspect
| 
|       Purpose: Do routine which is called when
|                  float selection aspect is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_aspect(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_normalize_cam
| 
|       Purpose: Do routine which is called when
|                  pane action button normalize_cam is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_normalize_cam(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
    /* if we got here, then we want to reset the view parms. */
    set_normalized_view();
    update_cameras_pane_GUI(Cameras_info);
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_reset_camera
| 
|       Purpose: Do routine which is called when
|                  pane action button reset_camera is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_reset_camera(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
    set_startup_default_camera_parms();
    update_cameras_pane_GUI(Cameras_info);
}

/*-----------------------------------------------------------
| 
|  Routine Name: update_cameras_pane_GUI
| 
|       Purpose: called to force update of GUI to reflect
|                  current renderer camera parms.
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: wes
|          Date: Jun 19, 1993
| Modifications:
|
|     Notes    :  there is a lot of dependency between this
|                 routine and the .form file.  if the .form
|                 file gets changed, then this routine will need
|                 to get changed (probably) also.
| 
------------------------------------------------------------*/
void
update_cameras_pane_GUI(Render_Cameras *ci)
{
    int proj,yon_clip,i;
    float fov,t;
    vertex_3d v;
    char temp[KLENGTH];

    /* set the projection thingy.  in the form file, perspective==0,
     and parallel == 1 */
    get_projection(&proj);
    xvf_set_attribute(ci->projection_cycle_struct,
                   XVF_CYCLE_VAL,
                   proj);
    ci->projection_cycle = proj;

    /* set the FOV thingy */
    get_fov(&fov);
    sprintf(temp, "%g", fov);
    xvf_set_attribute(ci->fov_input_struct,
                   XVF_LITERAL, temp);
    ci->fov_input = fov;

    /* set the eye thingy */
    get_eye(&v);
    sprintf(temp, "%g", v.v[0]);
    xvf_set_attribute(ci->eye_x_struct,
                   XVF_LITERAL, temp);
    sprintf(temp, "%g", v.v[1]);
    xvf_set_attribute(ci->eye_y_struct,
                   XVF_LITERAL, temp);
    sprintf(temp, "%g", v.v[2]);
    xvf_set_attribute(ci->eye_z_struct,
                   XVF_LITERAL, temp);
    ci->eye_x = v.v[0];
    ci->eye_y = v.v[1];
    ci->eye_z = v.v[2];

    /* set the at_thingy */
    get_at(&v);
    sprintf(temp, "%g", v.v[0]);
    xvf_set_attribute(ci->at_x_struct,
                   XVF_LITERAL, temp);
    sprintf(temp, "%g", v.v[1]);
    xvf_set_attribute(ci->at_y_struct,
                   XVF_LITERAL, temp);
    sprintf(temp, "%g", v.v[2]);
    xvf_set_attribute(ci->at_z_struct,
                   XVF_LITERAL, temp);
    ci->at_x = v.v[0];
    ci->at_y = v.v[1];
    ci->at_z = v.v[2];

    /* set the up thingy */
    get_up(&v);
    sprintf(temp, "%g", v.v[0]);
    xvf_set_attribute(ci->up_x_struct,
                   XVF_LITERAL, temp);
    sprintf(temp, "%g", v.v[1]);
    xvf_set_attribute(ci->up_y_struct,
                   XVF_LITERAL, temp);
    sprintf(temp, "%g", v.v[2]);
    xvf_set_attribute(ci->up_z_struct,
                   XVF_LITERAL, temp);
    ci->up_x = v.v[0];
    ci->up_y = v.v[1];
    ci->up_z = v.v[2];

    /* set the hither thingy */
    get_user_hither(&t);
    sprintf(temp, "%g", t);
    xvf_set_attribute(ci->hither_clip_form_loc_struct,
                      XVF_LITERAL, temp);
    ci->hither_clip_form_loc = t;

    /* set the yon thingy */
    get_user_yon(&t);
    sprintf(temp, "%g", t);
    xvf_set_attribute(ci->yon_clip_form_loc_struct,
                      XVF_LITERAL, temp);
    ci->yon_clip_form_loc = t;

    /* set the yon clip flag */
    get_yon_clip_flag(&yon_clip);
    xvf_set_attribute(ci->back_clip_form_on_off_struct,
                      XVF_LOGIC_VAL,
                      yon_clip);

    /* set stereo viewing parameters. */
    
    get_eye_separation(&t);
    sprintf(temp, "%g", t);
    xvf_set_attribute(ci->eye_separation_struct,
                      XVF_LITERAL, temp);
    ci->eye_separation = t;

    get_focal_length(&t);
    sprintf(temp, "%g", t);
    xvf_set_attribute(ci->focal_length_struct,
                      XVF_LITERAL, temp);
    ci->focal_length = t;

    i = get_leftright_flip();
    xvf_set_attribute(ci->lr_flip_struct,
		      XVF_LOGIC_VAL,i);
    ci->lr_flip = i;

    /* build the VOF table */
    {
	int n;
	char **strings;

	n = get_num_vofs();
	strings = get_vof_strings();

	xvf_set_attribute(ci->do_vof_struct,
			  XVF_LIST_DELETEALL,NULL);
	xvf_set_attributes(ci->do_vof_struct,
			   XVF_LIST_SIZE,n,
			   XVF_LIST_CONTENTS,strings,
			   NULL);
    }

    /* are we stereo-capable? */
    {
	int i;
	i = is_active_renderer_stereo_capable();
	if (i == TRUE)
	{
	    /* activate all the stereo parms */
	    xvf_set_attribute(ci->camera_type_struct,
			      XVF_ACTIVATE,TRUE);
	    i = get_camera_type();
	    xvf_set_attribute(ci->camera_type_struct,
			      XVF_LIST_VAL,i+1);
	    ci->camera_type = i+1;
	    xvf_set_attribute(ci->eye_separation_struct,
			      XVF_ACTIVATE,TRUE);
	    xvf_set_attribute(ci->focal_length_struct,
			      XVF_ACTIVATE,TRUE);
	    xvf_set_attribute(ci->lr_flip_struct,
			      XVF_ACTIVATE,TRUE);
	    xvf_set_attribute(ci->do_vof_struct,
			      XVF_ACTIVATE,TRUE);
	    i = get_current_vof();
	    xvf_set_attribute(ci->do_vof_struct,
			      XVF_LIST_VAL,i+1);
	    ci->do_vof = i+1;

	    /* at some point, we'll need to add stuff for VOF
	     formats for machines other than SGI  6 sep 96, wes*/

	}
	else
	{
	    /* disable all stereo parms. */
	    
	    xvf_set_attribute(ci->camera_type_struct,
			      XVF_ACTIVATE,FALSE);

	    /* also, we won't need to change VOF's */
	    xvf_set_attribute(ci->eye_separation_struct,
			      XVF_ACTIVATE,FALSE);
	    xvf_set_attribute(ci->focal_length_struct,
			      XVF_ACTIVATE,FALSE);
	    xvf_set_attribute(ci->lr_flip_struct,
			      XVF_ACTIVATE,FALSE);
	    xvf_set_attribute(ci->do_vof_struct,
			      XVF_ACTIVATE,FALSE);


	}
    }
	    

    
}

/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_eye_separation
| 
|       Purpose: Do routine which is called when
|                  float selection eye_separation is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Jul 23, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_eye_separation(
     Render_Cameras *ci)
{
    /* PUT YOUR CODE HERE ! */
    set_eye_separation(ci->eye_separation);
    renderer_invoke_func();
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_focal_length
| 
|       Purpose: Do routine which is called when
|                  float selection focal_length is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Jul 23, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_focal_length(
     Render_Cameras *ci)
{
    /* PUT YOUR CODE HERE ! */
    set_focal_length(ci->focal_length);
    renderer_invoke_func();
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_camera_type
| 
|       Purpose: Do routine which is called when
|                  list selection camera_type is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Jul 23, 1996
| Modifications: 
| Todo: add checks to ensure that we don't change to a stereo
|       camera when the active renderer doesn't support it??
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_camera_type(
     Render_Cameras *ci)
{
    /* PUT YOUR CODE HERE ! */

    if (ci->camera_type == 1)	/* mono camera */
    {
	/* deactivate eye sep & focal length items, l/r flip. */
	xvf_set_attribute(ci->eye_separation_struct,
			  XVF_ACTIVATE,FALSE);
	xvf_set_attribute(ci->focal_length_struct,
			  XVF_ACTIVATE,FALSE);
	xvf_set_attribute(ci->lr_flip_struct,
			  XVF_ACTIVATE,FALSE);

	if (get_camera_type() != CAMERA_TYPE_MONOSCOPIC)
	{
	    set_camera_type(CAMERA_TYPE_MONOSCOPIC);
	    renderer_shutdown_func();
	    renderer_init_func();
	}
    }
    else if (ci->camera_type == 2) /* stereo camera */
    {
	/* activate eye sep & focal length items. */
	xvf_set_attribute(ci->eye_separation_struct,
			  XVF_ACTIVATE,TRUE);
	xvf_set_attribute(ci->focal_length_struct,
			  XVF_ACTIVATE,TRUE);
	xvf_set_attribute(ci->lr_flip_struct,
			  XVF_ACTIVATE,TRUE);

	set_focal_length((float)(ci->focal_length));
	set_eye_separation((float)(ci->eye_separation));
	if (get_camera_type() != CAMERA_TYPE_STEREOSCOPIC)
	{
	    set_camera_type(CAMERA_TYPE_STEREOSCOPIC);
	    renderer_shutdown_func();
	    renderer_init_func();
	    renderer_invoke_func();
	}
	
    }
    else
	kerror("GEOMETRY","render"," Bogus camera type value of %d.",ci->camera_type);
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_lr_flip
| 
|       Purpose: Do routine which is called when
|                  logical selection lr_flip is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Jul 24, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_lr_flip(
     Render_Cameras *ci)
{
    /* PUT YOUR CODE HERE ! */
    set_leftright_flip(ci->lr_flip);
    renderer_invoke_func();
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_do_vof
| 
|       Purpose: Do routine which is called when
|                  list selection do_vof is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Aug 21, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_do_vof(
     Render_Cameras *Cameras_info)
{
    int old_vof,new_vof;
	/* PUT YOUR CODE HERE ! */
    /* what we're going to do here is first tell render that we want
     to have a new video format, then call the routine that will execute
     that change-video-format command. */

    old_vof = get_current_vof();
    new_vof = Cameras_info->do_vof-1; /* the uis counts from 1, not zero.. */

    if (old_vof != new_vof)
    {
	set_current_vof(new_vof);
	change_vof(new_vof);
    }
    
#if 0
    char ircombine_string[256];

    kmemset(ircombine_string,0,256);
    ksprintf(ircombine_string,"/usr/gfx/ircombine -destination active -channel 0 format=%s",Cameras_info->do_vof_label);
    system(ircombine_string);
#endif
}


/*-----------------------------------------------------------
| 
|  Routine Name: Cameras_multisample
| 
|       Purpose: Do routine which is called when
|                  logical selection multisample is used
| 
|         Input: Cameras_info - ptr to PaneInfo struct for Cameras pane
| 
|        Output: None
|    Written By: 
|          Date: Aug 21, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Cameras_multisample(
     Render_Cameras *Cameras_info)
{
	/* PUT YOUR CODE HERE ! */
}


