
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 
 /*
  * Khoros: $Id: do_Imaging.c,v 1.1 1997/04/19 00:14:46 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_Imaging.c,v 1.1 1997/04/19 00:14:46 mirage Exp $";
#endif

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for pane Imaging
   >>>> 
   >>>>  Private: 
   >>>> 	Imaging_image_width
   >>>> 	Imaging_image_height
   >>>> 	Imaging_image_display_choice
   >>>> 	Imaging_red_background
   >>>> 	Imaging_green_background
   >>>> 	Imaging_blue_background
   >>>> 	Imaging_renderer_choice
   >>>> 	Imaging_transparency_choice
   >>>> 	Imaging_invoke_renderer
   >>>> 	Imaging_antialias
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "render.h"
#include "imaging.h"
#include "texture.h"

/*-----------------------------------------------------------
| 
|  Routine Name: Imaging_image_width
| 
|       Purpose: Do routine which is called when
|                  integer selection image_width is used
| 
|         Input: Imaging_info - ptr to PaneInfo struct for Imaging pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: may 12 1996 - use a call to a more generic routine for
|                handling image resize operations.
| 
------------------------------------------------------------*/
void Imaging_image_width(
     Render_Imaging *Imaging_info)
{
    int oldw,oldh;
    get_image_dims(&oldw,&oldh);
    renderer_resize_func(Imaging_info->image_width,oldh);
#if 0
    change_image_width(Imaging_info->image_width);
#endif
}


/*-----------------------------------------------------------
| 
|  Routine Name: Imaging_image_height
| 
|       Purpose: Do routine which is called when
|                  integer selection image_height is used
| 
|         Input: Imaging_info - ptr to PaneInfo struct for Imaging pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: may 12 1996 - use a call to a more generic routine for
|                handling image resize operations.
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Imaging_image_height(
     Render_Imaging *Imaging_info)
{
	/* PUT YOUR CODE HERE ! */
    int oldw,oldh;
    get_image_dims(&oldw,&oldh);
    renderer_resize_func(oldw,Imaging_info->image_height);
#if 0    
    change_image_height(Imaging_info->image_height);
#endif
}


/*-----------------------------------------------------------
| 
|  Routine Name: Imaging_image_display_choice
| 
|       Purpose: Do routine which is called when
|                  list selection image_display_choice is used
| 
|         Input: Imaging_info - ptr to PaneInfo struct for Imaging pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Imaging_image_display_choice(
     Render_Imaging *Imaging_info)
{
	/* PUT YOUR CODE HERE ! */
    int new_val;
    int w,h;
    /* new_val = Imaging_info->image_display_choice - 1; */
    new_val = Imaging_info->image_display_choice;
    set_image_display_choice(new_val);

#if 0
    /* in order to have the new change take effect immediately, be
     displayed properly, etc, we'll call the routine which handles
     an image resize event.  this process is really too heavy, but
     it works reliably. */
    get_image_dims(&w,&h);
    change_image_dims(w,h);

    /* -- sad if geometry takes long to render, but better then befuddled
       users staring at the screen waiting -- */
#endif
    renderer_shutdown_func();
    renderer_init_func();
    renderer_invoke_func();
}


/*-----------------------------------------------------------
| 
|  Routine Name: Imaging_red_background
| 
|       Purpose: Do routine which is called when
|                  float selection red_background is used
| 
|         Input: Imaging_info - ptr to PaneInfo struct for Imaging pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Imaging_red_background(
     Render_Imaging *Imaging_info)
{
	/* PUT YOUR CODE HERE ! */
    float r,g,b,t;
    get_background_color(&r,&g,&b);
    t = Imaging_info->red_background;
    set_background_color(t,g,b);
}


/*-----------------------------------------------------------
| 
|  Routine Name: Imaging_green_background
| 
|       Purpose: Do routine which is called when
|                  float selection green_background is used
| 
|         Input: Imaging_info - ptr to PaneInfo struct for Imaging pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Imaging_green_background(
     Render_Imaging *Imaging_info)
{
	/* PUT YOUR CODE HERE ! */
    float r,g,b;
    get_background_color(&r,&g,&b);
    g = Imaging_info->green_background;
    set_background_color(r,g,b);
}


/*-----------------------------------------------------------
| 
|  Routine Name: Imaging_blue_background
| 
|       Purpose: Do routine which is called when
|                  float selection blue_background is used
| 
|         Input: Imaging_info - ptr to PaneInfo struct for Imaging pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Imaging_blue_background(
     Render_Imaging *Imaging_info)
{
	/* PUT YOUR CODE HERE ! */
    float r,g,b;
    get_background_color(&r,&g,&b);
    b = Imaging_info->blue_background;
    set_background_color(r,g,b);
}


/*-----------------------------------------------------------
| 
|  Routine Name: Imaging_renderer_choice
| 
|       Purpose: Do routine which is called when
|                  list selection renderer_choice is used
| 
|         Input: Imaging_info - ptr to PaneInfo struct for Imaging pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Imaging_renderer_choice(
     Render_Imaging *Imaging_info)
{
	/* PUT YOUR CODE HERE ! */
    int i,active;
    i = Imaging_info->renderer_choice - 1;

    /* check to see if the new renderer choice is different from the
       currently active renderer. */
    active = get_active_renderer();

    /* if so, shut down the old one */
    if (i != active)
    {
        renderer_shutdown_func();

        /* and fire up the new one */
        set_active_renderer(i);
        renderer_init_func();

	
	/* need to update cameras pane - hack - note that we're using
	 the global variable gui_info !! */
	update_cameras_pane_GUI(gui_info->Render->Cameras);
/*      init_imaging_pane(Imaging_info); */
    }
}


/*-----------------------------------------------------------
| 
|  Routine Name: Imaging_transparency_choice
| 
|       Purpose: Do routine which is called when
|                  list selection transparency_choice is used
| 
|         Input: Imaging_info - ptr to PaneInfo struct for Imaging pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Imaging_transparency_choice(
     Render_Imaging *Imaging_info)
{
	/* PUT YOUR CODE HERE ! */
}


/*-----------------------------------------------------------
| 
|  Routine Name: Imaging_invoke_renderer
| 
|       Purpose: Do routine which is called when
|                  logical selection invoke_renderer is used
| 
|         Input: Imaging_info - ptr to PaneInfo struct for Imaging pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Imaging_invoke_renderer(
     Render_Imaging *Imaging_info)
{
    /* PUT YOUR CODE HERE ! */
    /* note: 0 means deferred mode, while 1 means immediate mode */
    
    set_render_deferred_mode(Imaging_info->invoke_renderer);
}


/*-----------------------------------------------------------
| 
|  Routine Name: Imaging_antialias
| 
|       Purpose: Do routine which is called when
|                  logical selection antialias is used
| 
|         Input: Imaging_info - ptr to PaneInfo struct for Imaging pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Imaging_antialias(
     Render_Imaging *Imaging_info)
{
	/* PUT YOUR CODE HERE ! */
}

init_imaging_pane(Render_Imaging *im)
{
    float r,g,b;
    double d;
    int display_mode,w,h;
    int n,i, depth;
    int texture2d_index;
    renderer_struct **rndr;

    get_background_color(&r,&g,&b);

    /* set RGB background sliders to init values. */
    im->red_background = r;
    xvf_set_attributes(im->red_background_struct,
		       XVF_FLOAT_VAL,r,
		       XVF_FLOAT_DEF,r,NULL);

    im->green_background = g;
    xvf_set_attribute(im->green_background_struct,
                   XVF_FLOAT_VAL,g);

    im->blue_background = b;
    xvf_set_attribute(im->blue_background_struct,
                   XVF_FLOAT_VAL,b);

    /**
      * set the renderer choices UI list.
    **/

    get_all_renderers_info(&n,&rndr);

    /* we'll assume (naively) for now that the default renderer is
       in r[0], and set up the UI to reflect that fact. */

    xvf_set_attribute(im->renderer_choice_struct,
                      XVF_LIST_DELETEALL,
                      TRUE);  /* wipe the list clean */

    for (i=0;i<n;i++)
        xvf_set_attribute(im->renderer_choice_struct,
                          XVF_LIST_ADD,
                          rndr[i]->renderer_name);

    if (n > 1)
        xvf_set_attribute(im->renderer_choice_struct,
                          XVF_ACTIVATE,TRUE);
    else
        xvf_set_attribute(im->renderer_choice_struct,
                          XVF_ACTIVATE,FALSE);

    /* set image display mode for the current active renderer  */

    n = get_active_renderer();
    xvf_set_attribute(im->renderer_choice_struct,
		      XVF_LIST_INDEX,n);

    xvf_set_attribute(im->image_display_choice_struct,
                      XVF_LIST_DELETEALL,
                      TRUE);  /* wipe the list clean */

    get_display_depth(&depth);

    /* -- add all the displays, but don't add 24 bit if we can't do it -- */
    for (i=0;i<rndr[n]->ndisplay_options;i++)
    {
       if (i == s24_DISPLAY && depth != 24) 
       {
	  xvf_set_attribute(im->image_display_choice_struct,
			    XVF_LIST_ADD, "DISPLAY DOESN'T SUPPORT 24 BIT");
	  i++;
       }
       
       xvf_set_attribute(im->image_display_choice_struct,
			       XVF_LIST_ADD,
			 rndr[n]->display_options_txt[i]);
    }


    if (n == 0)			/* assume for now that hardware renderers will
				 use only their default display mode. */
    {
	get_image_display_choice(&display_mode);

	if (display_mode > 0)
	{
	    xvf_set_attribute(im->image_display_choice_struct,
			      XVF_LIST_VAL, 
			      display_mode);

	    im->image_display_choice = display_mode;
	}
       xvf_set_attribute(im->ogl_sphere_subdivisions_struct,
			 XVF_ACTIVATE,FALSE);
    }

#if defined(__OGL_H) && defined(_USE_GFX_HARDWARE)
    else
    {
    /* if we're using OpenGL, then enable the sphere subdivisions thing. */
    
       xvf_set_attribute(im->ogl_sphere_subdivisions_struct,
			 XVF_ACTIVATE,TRUE);

       xvf_set_attribute(im->ogl_sphere_subdivisions_struct,
			 XVF_INT_VAL,
			 get_sphere_subdivisions());
    }
#endif

    get_image_dims(&w,&h);
    xvf_set_attribute(im->image_width_struct,
                      XVF_INT_VAL,
                      w);

    xvf_set_attribute(im->image_height_struct,
                      XVF_INT_VAL,
                      h);

    xvf_set_attribute(im->invoke_renderer_struct,
                      XVF_ACTIVATE,TRUE);
    n = get_render_deferred_mode();
    xvf_set_attribute(im->invoke_renderer_struct,
		      XVF_LOGIC_VAL,n);

    /* disable various things that aren't ready yet. */

    xvf_set_attribute(im->transparency_choice_struct,
                      XVF_ACTIVATE,FALSE);

    xvf_set_attribute(im->antialias_struct,
                      XVF_ACTIVATE,FALSE);

    texture2d_index = get_global_2dtexturing_function_index() + 1;

    xvf_set_attribute(im->texture_method_2d_struct,
		      XVF_LIST_VAL, 
		      texture2d_index);

   return(CHILL);
}

int
repaint_imaging_pane(void)
{
    Render_Imaging *t;

    /* a good way to break this code is to rearrange the GUI for render. */
    t = gui_info->Render->Imaging;
    init_imaging_pane(t);
    return(CHILL);
}
/*-----------------------------------------------------------
| 
|  Routine Name: Imaging_texture_method_2d
| 
|       Purpose: Do routine which is called when
|                  list selection texture_method_2d is used
| 
|         Input: Imaging_info - ptr to PaneInfo struct for Imaging pane
| 
|        Output: None
|    Written By: 
|          Date: Sep 06, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Imaging_texture_method_2d(
     Render_Imaging *Imaging_info)
{
	/* PUT YOUR CODE HERE ! */
    int index;
    index = Imaging_info->texture_method_2d - 1;
    set_global_2dtexturing_function_index(index);
}


/*-----------------------------------------------------------
| 
|  Routine Name: Imaging_ogl_sphere_subdivisions
| 
|       Purpose: Do routine which is called when
|                  integer selection ogl_sphere_subdivisions is used
| 
|         Input: Imaging_info - ptr to PaneInfo struct for Imaging pane
| 
|        Output: None
|    Written By: 
|          Date: Mar 18, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Imaging_ogl_sphere_subdivisions(
     Render_Imaging *Imaging_info)
{
#if defined(__OGL_H) && defined(_USE_GFX_HARDWARE)
    set_sphere_subdivisions(Imaging_info->ogl_sphere_subdivisions);
#endif
}


