
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 
 /*
  * Khoros: $Id: do_Output.c,v 1.1 1997/04/19 00:14:53 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_Output.c,v 1.1 1997/04/19 00:14:53 mirage Exp $";
#endif

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for pane Output
   >>>> 
   >>>>  Private: 
   >>>> 	Output_output_image_fname
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "render.h"
#include "vrxfrm.h"
#include "scene.h"

/*-----------------------------------------------------------
| 
|  Routine Name: Output_output_image_fname
| 
|       Purpose: Do routine which is called when
|                  output selection output_image_fname is used
| 
|         Input: Output_info - ptr to PaneInfo struct for Output pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 11, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Output_output_image_fname(
     Render_Output *Output_info)
{
    /* PUT YOUR CODE HERE ! */
    char *fname;

    fname = Output_info->output_image_fname;
    
    if ((fname == NULL) || (fname[0] == '\0'))
	set_do_named_output(FALSE);
    else
	set_do_named_output(TRUE);
	
    set_output_transport_name(Output_info->output_image_fname);
}


void
init_output_pane_GUI(Render_Output *ri)
{
    int have_output_image_fname;
    int have_output_vr_events;
    char s[256];

    get_do_named_output(&have_output_image_fname);
    
    if (have_output_image_fname == TRUE)
    {
	get_output_transport_name(s);
        xvf_set_attribute(ri->output_image_fname_struct,
                          XVF_FILE_NAME,
                          s);
        xvf_set_attribute(ri->output_image_fname_struct,
                          XVF_ACTIVATE,
                          TRUE);
        xvf_set_attribute(ri->output_image_fname_struct,
                          XVF_OPTSEL,
                          TRUE);
    }
	
    have_output_vr_events = have_named_vr_events();
    if (have_output_vr_events == TRUE)
    {
	get_vr_transport_name(s);
        xvf_set_attribute(ri->oxfrm_struct,
			  XVF_FILE_NAME,
			  s);
        xvf_set_attribute(ri->oxfrm_struct,
                          XVF_ACTIVATE,
                          TRUE);
        xvf_set_attribute(ri->oxfrm_struct,
                          XVF_OPTSEL,
                          TRUE);
    }

    if (have_output_scene_filename() == TRUE)
/*    if (s[0] != '\0') */
    {
	get_output_scene_filename(s);
        xvf_set_attribute(ri->oscene_struct,
			  XVF_FILE_NAME,
			  s);
        xvf_set_attribute(ri->oscene_struct,
                          XVF_ACTIVATE,
                          TRUE);
        xvf_set_attribute(ri->oscene_struct,
                          XVF_OPTSEL,
                          TRUE);
    }
}


/*-----------------------------------------------------------
| 
|  Routine Name: Output_oxfrm
| 
|       Purpose: Do routine which is called when
|                  output selection oxfrm is used
| 
|         Input: Output_info - ptr to PaneInfo struct for Output pane
| 
|        Output: None
|    Written By: 
|          Date: May 22, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Output_oxfrm(
     Render_Output *Output_info)
{
  /* PUT YOUR CODE HERE ! */
  init_xform_output(Output_info->oxfrm);
}


/*-----------------------------------------------------------
| 
|  Routine Name: Output_write_scene_file
| 
|       Purpose: Do routine which is called when
|                  pane action button write_scene_file is used
| 
|         Input: Output_info - ptr to PaneInfo struct for Output pane
| 
|        Output: None
|    Written By: 
|          Date: Sep 05, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Output_write_scene_file(
     Render_Output *Output_info)
{
	/* PUT YOUR CODE HERE ! */
    write_scene_file();
}


/*-----------------------------------------------------------
| 
|  Routine Name: Output_oscene
| 
|       Purpose: Do routine which is called when
|                  output selection oscene is used
| 
|         Input: Output_info - ptr to PaneInfo struct for Output pane
| 
|        Output: None
|    Written By: 
|          Date: Sep 05, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void Output_oscene(
     Render_Output *Output_info)
{
	/* PUT YOUR CODE HERE ! */
    set_output_scene_filename(Output_info->oscene);
}


