 /*
  * Khoros: $Id: form_info.c,v 1.1.2.2 1997/05/31 19:00:13 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: form_info.c,v 1.1.2.2 1997/05/31 19:00:13 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	FormInfo Extraction Routines
   >>>> 
   >>>>  Private: 
   >>>> 	_xvf_get_Master
   >>>> 
   >>>>   Static: 
   >>>> 	_xvf_get_Render
   >>>> 	_xvf_get_Objects
   >>>> 	_xvf_get_Cameras
   >>>> 	_xvf_get_Lights
   >>>> 	_xvf_get_Imaging
   >>>> 	_xvf_get_Transformations
   >>>> 	_xvf_get_Input
   >>>> 	_xvf_get_Output
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "render.h"


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_get_Master
| 
|       Purpose: Extracts information from GUI
| 
|         Input: forminfo - pointer to gui_info_struct
| 
|        Output: none
|    Written By: conductor -oname render
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void _xvf_get_Master(
     gui_info_struct *forminfo)
{
	ksubform   *subform;
	if (forminfo->Render_struct != NULL)
	{
	    subform = forminfo->Render_struct->Subformptr;
	    kvf_get_attribute(subform->back_kformstruct, KVF_SELECTED,
	                    &(forminfo->Render_selected));
	    _xvf_get_Render(forminfo->Render);

	}

}

/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_get_Render
| 
|       Purpose: Extracts information from Render subform
| 
|         Input: subforminfo - pointer to Master_Render structure
| 
|        Output: none
|    Written By: conductor -oname render
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void _xvf_get_Render(
     Master_Render *subforminfo)
{
	kselection *selection;
	kcontrol   *pane;
	if (subforminfo->quit_struct != NULL)
	{
	    selection = subforminfo->quit_struct->Selptr;
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(subforminfo->quit));

	}

	if (subforminfo->refresh_struct != NULL)
	{
	    selection = subforminfo->refresh_struct->Selptr;
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(subforminfo->refresh));

	}

	if (subforminfo->Objects_struct != NULL)
	{
	    pane = subforminfo->Objects_struct->Controlptr;
	    kvf_get_attribute(pane->back_kformstruct, KVF_SELECTED,
	                    &(subforminfo->Objects_selected));
	    _xvf_get_Objects(subforminfo->Objects);
	}

	if (subforminfo->Cameras_struct != NULL)
	{
	    pane = subforminfo->Cameras_struct->Controlptr;
	    kvf_get_attribute(pane->back_kformstruct, KVF_SELECTED,
	                    &(subforminfo->Cameras_selected));
	    _xvf_get_Cameras(subforminfo->Cameras);
	}

	if (subforminfo->Lights_struct != NULL)
	{
	    pane = subforminfo->Lights_struct->Controlptr;
	    kvf_get_attribute(pane->back_kformstruct, KVF_SELECTED,
	                    &(subforminfo->Lights_selected));
	    _xvf_get_Lights(subforminfo->Lights);
	}

	if (subforminfo->Imaging_struct != NULL)
	{
	    pane = subforminfo->Imaging_struct->Controlptr;
	    kvf_get_attribute(pane->back_kformstruct, KVF_SELECTED,
	                    &(subforminfo->Imaging_selected));
	    _xvf_get_Imaging(subforminfo->Imaging);
	}

	if (subforminfo->Transformations_struct != NULL)
	{
	    pane = subforminfo->Transformations_struct->Controlptr;
	    kvf_get_attribute(pane->back_kformstruct, KVF_SELECTED,
	                    &(subforminfo->Transformations_selected));
	    _xvf_get_Transformations(subforminfo->Transformations);
	}

	if (subforminfo->Input_struct != NULL)
	{
	    pane = subforminfo->Input_struct->Controlptr;
	    kvf_get_attribute(pane->back_kformstruct, KVF_SELECTED,
	                    &(subforminfo->Input_selected));
	    _xvf_get_Input(subforminfo->Input);
	}

	if (subforminfo->Output_struct != NULL)
	{
	    pane = subforminfo->Output_struct->Controlptr;
	    kvf_get_attribute(pane->back_kformstruct, KVF_SELECTED,
	                    &(subforminfo->Output_selected));
	    _xvf_get_Output(subforminfo->Output);
	}


}



/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_get_Objects
| 
|       Purpose: Extracts information from Objects pane
| 
|         Input: paneinfo - pointer to Render_Objects structure
| 
|        Output: none
|    Written By: conductor -oname render
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void _xvf_get_Objects(
     Render_Objects *paneinfo)
{
	kselection *selection;

	/*
	 * extracting 'Master->Render->Objects' Pane Info structure
	 */

	if (paneinfo->current_object_struct != NULL)
	{
	    selection = paneinfo->current_object_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_VAL,
	                    &(paneinfo->current_object));
	    if (paneinfo->current_object_label != NULL) kfree_and_NULL(paneinfo->current_object_label);
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_LABEL,
	                    &(paneinfo->current_object_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->current_object_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->current_object_optsel));
	    }

	}

	if (paneinfo->object_shader_struct != NULL)
	{
	    selection = paneinfo->object_shader_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_VAL,
	                    &(paneinfo->object_shader));
	    if (paneinfo->object_shader_label != NULL) kfree_and_NULL(paneinfo->object_shader_label);
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_LABEL,
	                    &(paneinfo->object_shader_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->object_shader_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->object_shader_optsel));
	    }

	}

	if (paneinfo->post_obj_struct != NULL)
	{
	   selection = paneinfo->post_obj_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_VAL,
	                    &(paneinfo->post_obj));
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_LABEL,
	                    &(paneinfo->post_obj_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->post_obj_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->post_obj_optsel));
	    }

	}

	if (paneinfo->obj_red_struct != NULL)
	{
	    selection = paneinfo->obj_red_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->obj_red));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->obj_red_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->obj_red_optsel));
	    }

	}

	if (paneinfo->obj_green_struct != NULL)
	{
	    selection = paneinfo->obj_green_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->obj_green));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->obj_green_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->obj_green_optsel));
	    }

	}

	if (paneinfo->obj_blue_struct != NULL)
	{
	    selection = paneinfo->obj_blue_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->obj_blue));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->obj_blue_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->obj_blue_optsel));
	    }

	}

	if (paneinfo->obj_alpha_struct != NULL)
	{
	    selection = paneinfo->obj_alpha_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->obj_alpha));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->obj_alpha_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->obj_alpha_optsel));
	    }

	}

	if (paneinfo->obj_ambient_struct != NULL)
	{
	    selection = paneinfo->obj_ambient_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->obj_ambient));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->obj_ambient_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->obj_ambient_optsel));
	    }

	}

	if (paneinfo->obj_diffuse_struct != NULL)
	{
	    selection = paneinfo->obj_diffuse_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->obj_diffuse));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->obj_diffuse_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->obj_diffuse_optsel));
	    }

	}

	if (paneinfo->obj_spec_struct != NULL)
	{
	    selection = paneinfo->obj_spec_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->obj_spec));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->obj_spec_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->obj_spec_optsel));
	    }

	}

	if (paneinfo->obj_spec_exp_struct != NULL)
	{
	    selection = paneinfo->obj_spec_exp_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->obj_spec_exp));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->obj_spec_exp_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->obj_spec_exp_optsel));
	    }

	}

	if (paneinfo->obj_spec_metal_struct != NULL)
	{
	    selection = paneinfo->obj_spec_metal_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->obj_spec_metal));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->obj_spec_metal_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->obj_spec_metal_optsel));
	    }

	}

	if (paneinfo->object_info_struct != NULL)
	{
	    selection = paneinfo->object_info_struct->Selptr;
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->object_info));

	}

	if (paneinfo->action1_struct != NULL)
	{
	    selection = paneinfo->action1_struct->Selptr;
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->action1));

	}

}


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_get_Cameras
| 
|       Purpose: Extracts information from Cameras pane
| 
|         Input: paneinfo - pointer to Render_Cameras structure
| 
|        Output: none
|    Written By: conductor -oname render
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void _xvf_get_Cameras(
     Render_Cameras *paneinfo)
{
	kselection *selection;

	/*
	 * extracting 'Master->Render->Cameras' Pane Info structure
	 */

	if (paneinfo->projection_cycle_struct != NULL)
	{
	    selection = paneinfo->projection_cycle_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_CYCLE_VAL,
	                    &(paneinfo->projection_cycle));
	    if (paneinfo->projection_cycle_label != NULL) kfree_and_NULL(paneinfo->projection_cycle_label);
	    kvf_get_attribute(selection->back_kformstruct, KVF_CYCLE_LABEL,
	                    &(paneinfo->projection_cycle_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->projection_cycle_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->projection_cycle_optsel));
	    }

	}

	if (paneinfo->eye_x_struct != NULL)
	{
	    selection = paneinfo->eye_x_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->eye_x));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->eye_x_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->eye_x_optsel));
	    }

	}

	if (paneinfo->up_x_struct != NULL)
	{
	    selection = paneinfo->up_x_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->up_x));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->up_x_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->up_x_optsel));
	    }

	}

	if (paneinfo->at_x_struct != NULL)
	{
	    selection = paneinfo->at_x_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->at_x));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->at_x_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->at_x_optsel));
	    }

	}

	if (paneinfo->eye_y_struct != NULL)
	{
	    selection = paneinfo->eye_y_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->eye_y));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->eye_y_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->eye_y_optsel));
	    }

	}

	if (paneinfo->up_y_struct != NULL)
	{
	    selection = paneinfo->up_y_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->up_y));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->up_y_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->up_y_optsel));
	    }

	}

	if (paneinfo->at_y_struct != NULL)
	{
	    selection = paneinfo->at_y_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->at_y));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->at_y_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->at_y_optsel));
	    }

	}

	if (paneinfo->eye_z_struct != NULL)
	{
	    selection = paneinfo->eye_z_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->eye_z));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->eye_z_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->eye_z_optsel));
	    }

	}

	if (paneinfo->up_z_struct != NULL)
	{
	    selection = paneinfo->up_z_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->up_z));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->up_z_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->up_z_optsel));
	    }

	}

	if (paneinfo->at_z_struct != NULL)
	{
	    selection = paneinfo->at_z_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->at_z));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->at_z_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->at_z_optsel));
	    }

	}

	if (paneinfo->hither_clip_form_loc_struct != NULL)
	{
	    selection = paneinfo->hither_clip_form_loc_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->hither_clip_form_loc));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->hither_clip_form_loc_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->hither_clip_form_loc_optsel));
	    }

	}

	if (paneinfo->yon_clip_form_loc_struct != NULL)
	{
	    selection = paneinfo->yon_clip_form_loc_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->yon_clip_form_loc));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->yon_clip_form_loc_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->yon_clip_form_loc_optsel));
	    }

	}

	if (paneinfo->back_clip_form_on_off_struct != NULL)
	{
	   selection = paneinfo->back_clip_form_on_off_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_VAL,
	                    &(paneinfo->back_clip_form_on_off));
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_LABEL,
	                    &(paneinfo->back_clip_form_on_off_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->back_clip_form_on_off_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->back_clip_form_on_off_optsel));
	    }

	}

	if (paneinfo->fov_input_struct != NULL)
	{
	    selection = paneinfo->fov_input_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->fov_input));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->fov_input_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->fov_input_optsel));
	    }

	}

	if (paneinfo->aspect_struct != NULL)
	{
	    selection = paneinfo->aspect_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->aspect));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->aspect_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->aspect_optsel));
	    }

	}

	if (paneinfo->normalize_cam_struct != NULL)
	{
	    selection = paneinfo->normalize_cam_struct->Selptr;
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->normalize_cam));

	}

	if (paneinfo->reset_camera_struct != NULL)
	{
	    selection = paneinfo->reset_camera_struct->Selptr;
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->reset_camera));

	}

	if (paneinfo->camera_type_struct != NULL)
	{
	    selection = paneinfo->camera_type_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_VAL,
	                    &(paneinfo->camera_type));
	    if (paneinfo->camera_type_label != NULL) kfree_and_NULL(paneinfo->camera_type_label);
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_LABEL,
	                    &(paneinfo->camera_type_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->camera_type_selected));
	    }

	}

	if (paneinfo->eye_separation_struct != NULL)
	{
	    selection = paneinfo->eye_separation_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->eye_separation));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->eye_separation_selected));
	    }

	}

	if (paneinfo->focal_length_struct != NULL)
	{
	    selection = paneinfo->focal_length_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->focal_length));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->focal_length_selected));
	    }

	}

	if (paneinfo->lr_flip_struct != NULL)
	{
	   selection = paneinfo->lr_flip_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_VAL,
	                    &(paneinfo->lr_flip));
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_LABEL,
	                    &(paneinfo->lr_flip_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->lr_flip_selected));
	    }

	}

	if (paneinfo->multisample_struct != NULL)
	{
	   selection = paneinfo->multisample_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_VAL,
	                    &(paneinfo->multisample));
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_LABEL,
	                    &(paneinfo->multisample_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->multisample_selected));
	    }

	}

	if (paneinfo->do_vof_struct != NULL)
	{
	    selection = paneinfo->do_vof_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_VAL,
	                    &(paneinfo->do_vof));
	    if (paneinfo->do_vof_label != NULL) kfree_and_NULL(paneinfo->do_vof_label);
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_LABEL,
	                    &(paneinfo->do_vof_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->do_vof_selected));
	    }

	}

}


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_get_Lights
| 
|       Purpose: Extracts information from Lights pane
| 
|         Input: paneinfo - pointer to Render_Lights structure
| 
|        Output: none
|    Written By: conductor -oname render
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void _xvf_get_Lights(
     Render_Lights *paneinfo)
{
	kselection *selection;

	/*
	 * extracting 'Master->Render->Lights' Pane Info structure
	 */

	if (paneinfo->light_list_struct != NULL)
	{
	    selection = paneinfo->light_list_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_VAL,
	                    &(paneinfo->light_list));
	    if (paneinfo->light_list_label != NULL) kfree_and_NULL(paneinfo->light_list_label);
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_LABEL,
	                    &(paneinfo->light_list_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->light_list_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->light_list_optsel));
	    }

	}

	if (paneinfo->light_onoff_struct != NULL)
	{
	   selection = paneinfo->light_onoff_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_VAL,
	                    &(paneinfo->light_onoff));
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_LABEL,
	                    &(paneinfo->light_onoff_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->light_onoff_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->light_onoff_optsel));
	    }

	}

	if (paneinfo->light_type_list_struct != NULL)
	{
	    selection = paneinfo->light_type_list_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_VAL,
	                    &(paneinfo->light_type_list));
	    if (paneinfo->light_type_list_label != NULL) kfree_and_NULL(paneinfo->light_type_list_label);
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_LABEL,
	                    &(paneinfo->light_type_list_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->light_type_list_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->light_type_list_optsel));
	    }

	}

	if (paneinfo->light_position_string_struct != NULL)
	{
	    selection = paneinfo->light_position_string_struct->Selptr;
	    if (selection->modified)
	    {
	    if (paneinfo->light_position_string != NULL) kfree_and_NULL(paneinfo->light_position_string);
	    kvf_get_attribute(selection->back_kformstruct, KVF_STRING_VAL,
	                    &(paneinfo->light_position_string));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->light_position_string_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->light_position_string_optsel));
	    }

	}

	if (paneinfo->light_direction_string_struct != NULL)
	{
	    selection = paneinfo->light_direction_string_struct->Selptr;
	    if (selection->modified)
	    {
	    if (paneinfo->light_direction_string != NULL) kfree_and_NULL(paneinfo->light_direction_string);
	    kvf_get_attribute(selection->back_kformstruct, KVF_STRING_VAL,
	                    &(paneinfo->light_direction_string));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->light_direction_string_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->light_direction_string_optsel));
	    }

	}

	if (paneinfo->current_light_color_string_struct != NULL)
	{
	    selection = paneinfo->current_light_color_string_struct->Selptr;
	    if (selection->modified)
	    {
	    if (paneinfo->current_light_color_string != NULL) kfree_and_NULL(paneinfo->current_light_color_string);
	    kvf_get_attribute(selection->back_kformstruct, KVF_STRING_VAL,
	                    &(paneinfo->current_light_color_string));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->current_light_color_string_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->current_light_color_string_optsel));
	    }

	}

	if (paneinfo->ambient_light_color_string_struct != NULL)
	{
	    selection = paneinfo->ambient_light_color_string_struct->Selptr;
	    if (selection->modified)
	    {
	    if (paneinfo->ambient_light_color_string != NULL) kfree_and_NULL(paneinfo->ambient_light_color_string);
	    kvf_get_attribute(selection->back_kformstruct, KVF_STRING_VAL,
	                    &(paneinfo->ambient_light_color_string));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->ambient_light_color_string_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->ambient_light_color_string_optsel));
	    }

	}

}


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_get_Imaging
| 
|       Purpose: Extracts information from Imaging pane
| 
|         Input: paneinfo - pointer to Render_Imaging structure
| 
|        Output: none
|    Written By: conductor -oname render
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void _xvf_get_Imaging(
     Render_Imaging *paneinfo)
{
	kselection *selection;

	/*
	 * extracting 'Master->Render->Imaging' Pane Info structure
	 */

	if (paneinfo->image_width_struct != NULL)
	{
	    selection = paneinfo->image_width_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_INT_VAL,
	                    &(paneinfo->image_width));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->image_width_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->image_width_optsel));
	    }
	}

	if (paneinfo->image_height_struct != NULL)
	{
	    selection = paneinfo->image_height_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_INT_VAL,
	                    &(paneinfo->image_height));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->image_height_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->image_height_optsel));
	    }
	}

	if (paneinfo->image_display_choice_struct != NULL)
	{
	    selection = paneinfo->image_display_choice_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_VAL,
	                    &(paneinfo->image_display_choice));
	    if (paneinfo->image_display_choice_label != NULL) kfree_and_NULL(paneinfo->image_display_choice_label);
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_LABEL,
	                    &(paneinfo->image_display_choice_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->image_display_choice_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->image_display_choice_optsel));
	    }

	}

	if (paneinfo->red_background_struct != NULL)
	{
	    selection = paneinfo->red_background_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->red_background));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->red_background_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->red_background_optsel));
	    }

	}

	if (paneinfo->green_background_struct != NULL)
	{
	    selection = paneinfo->green_background_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->green_background));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->green_background_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->green_background_optsel));
	    }

	}

	if (paneinfo->blue_background_struct != NULL)
	{
	    selection = paneinfo->blue_background_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->blue_background));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->blue_background_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->blue_background_optsel));
	    }

	}

	if (paneinfo->renderer_choice_struct != NULL)
	{
	    selection = paneinfo->renderer_choice_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_VAL,
	                    &(paneinfo->renderer_choice));
	    if (paneinfo->renderer_choice_label != NULL) kfree_and_NULL(paneinfo->renderer_choice_label);
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_LABEL,
	                    &(paneinfo->renderer_choice_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->renderer_choice_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->renderer_choice_optsel));
	    }

	}

	if (paneinfo->invoke_renderer_struct != NULL)
	{
	   selection = paneinfo->invoke_renderer_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_VAL,
	                    &(paneinfo->invoke_renderer));
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_LABEL,
	                    &(paneinfo->invoke_renderer_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->invoke_renderer_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->invoke_renderer_optsel));
	    }

	}

	if (paneinfo->transparency_choice_struct != NULL)
	{
	    selection = paneinfo->transparency_choice_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_VAL,
	                    &(paneinfo->transparency_choice));
	    if (paneinfo->transparency_choice_label != NULL) kfree_and_NULL(paneinfo->transparency_choice_label);
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_LABEL,
	                    &(paneinfo->transparency_choice_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->transparency_choice_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->transparency_choice_optsel));
	    }

	}

	if (paneinfo->texture_method_2d_struct != NULL)
	{
	    selection = paneinfo->texture_method_2d_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_VAL,
	                    &(paneinfo->texture_method_2d));
	    if (paneinfo->texture_method_2d_label != NULL) kfree_and_NULL(paneinfo->texture_method_2d_label);
	    kvf_get_attribute(selection->back_kformstruct, KVF_LIST_LABEL,
	                    &(paneinfo->texture_method_2d_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->texture_method_2d_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->texture_method_2d_optsel));
	    }

	}

	if (paneinfo->antialias_struct != NULL)
	{
	   selection = paneinfo->antialias_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_VAL,
	                    &(paneinfo->antialias));
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_LABEL,
	                    &(paneinfo->antialias_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->antialias_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->antialias_optsel));
	    }

	}

	if (paneinfo->ogl_sphere_subdivisions_struct != NULL)
	{
	    selection = paneinfo->ogl_sphere_subdivisions_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_INT_VAL,
	                    &(paneinfo->ogl_sphere_subdivisions));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->ogl_sphere_subdivisions_selected));
	    }
	}

}


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_get_Transformations
| 
|       Purpose: Extracts information from Transformations pane
| 
|         Input: paneinfo - pointer to Render_Transformations structure
| 
|        Output: none
|    Written By: conductor -oname render
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void _xvf_get_Transformations(
     Render_Transformations *paneinfo)
{
	kselection *selection;

	/*
	 * extracting 'Master->Render->Transformations' Pane Info structure
	 */

	if (paneinfo->x_rotate_struct != NULL)
	{
	    selection = paneinfo->x_rotate_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->x_rotate));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->x_rotate_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->x_rotate_optsel));
	    }

	}

	if (paneinfo->y_rotate_struct != NULL)
	{
	    selection = paneinfo->y_rotate_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->y_rotate));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->y_rotate_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->y_rotate_optsel));
	    }

	}

	if (paneinfo->z_rotate_struct != NULL)
	{
	    selection = paneinfo->z_rotate_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_FLOAT_VAL,
	                    &(paneinfo->z_rotate));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->z_rotate_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->z_rotate_optsel));
	    }

	}

	if (paneinfo->center_point_string_struct != NULL)
	{
	    selection = paneinfo->center_point_string_struct->Selptr;
	    if (selection->modified)
	    {
	    if (paneinfo->center_point_string != NULL) kfree_and_NULL(paneinfo->center_point_string);
	    kvf_get_attribute(selection->back_kformstruct, KVF_STRING_VAL,
	                    &(paneinfo->center_point_string));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->center_point_string_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->center_point_string_optsel));
	    }

	}

	if (paneinfo->translate_vector_string_struct != NULL)
	{
	    selection = paneinfo->translate_vector_string_struct->Selptr;
	    if (selection->modified)
	    {
	    if (paneinfo->translate_vector_string != NULL) kfree_and_NULL(paneinfo->translate_vector_string);
	    kvf_get_attribute(selection->back_kformstruct, KVF_STRING_VAL,
	                    &(paneinfo->translate_vector_string));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->translate_vector_string_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->translate_vector_string_optsel));
	    }

	}

	if (paneinfo->xyz_scale_string_struct != NULL)
	{
	    selection = paneinfo->xyz_scale_string_struct->Selptr;
	    if (selection->modified)
	    {
	    if (paneinfo->xyz_scale_string != NULL) kfree_and_NULL(paneinfo->xyz_scale_string);
	    kvf_get_attribute(selection->back_kformstruct, KVF_STRING_VAL,
	                    &(paneinfo->xyz_scale_string));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->xyz_scale_string_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->xyz_scale_string_optsel));
	    }

	}

	if (paneinfo->transformation_mode_struct != NULL)
	{
	   selection = paneinfo->transformation_mode_struct->Selptr;
	    if (selection->modified)
	    {
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_VAL,
	                    &(paneinfo->transformation_mode));
	    kvf_get_attribute(selection->back_kformstruct, KVF_LOGIC_LABEL,
	                    &(paneinfo->transformation_mode_label));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->transformation_mode_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->transformation_mode_optsel));
	    }

	}

}


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_get_Input
| 
|       Purpose: Extracts information from Input pane
| 
|         Input: paneinfo - pointer to Render_Input structure
| 
|        Output: none
|    Written By: conductor -oname render
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void _xvf_get_Input(
     Render_Input *paneinfo)
{
	kselection *selection;

	/*
	 * extracting 'Master->Render->Input' Pane Info structure
	 */

	if (paneinfo->input_object_1_struct != NULL)
	{
	    selection = paneinfo->input_object_1_struct->Selptr;
	    if (selection->modified)
	    {
		if (paneinfo->input_object_1 != NULL) kfree_and_NULL(paneinfo->input_object_1);
		kvf_get_attribute(selection->back_kformstruct, KVF_FILE_NAME,
		                  &(paneinfo->input_object_1));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->input_object_1_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->input_object_1_optsel));
	    }
	}

	if (paneinfo->input_object_2_struct != NULL)
	{
	    selection = paneinfo->input_object_2_struct->Selptr;
	    if (selection->modified)
	    {
		if (paneinfo->input_object_2 != NULL) kfree_and_NULL(paneinfo->input_object_2);
		kvf_get_attribute(selection->back_kformstruct, KVF_FILE_NAME,
		                  &(paneinfo->input_object_2));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->input_object_2_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->input_object_2_optsel));
	    }
	}

	if (paneinfo->input_object_3_struct != NULL)
	{
	    selection = paneinfo->input_object_3_struct->Selptr;
	    if (selection->modified)
	    {
		if (paneinfo->input_object_3 != NULL) kfree_and_NULL(paneinfo->input_object_3);
		kvf_get_attribute(selection->back_kformstruct, KVF_FILE_NAME,
		                  &(paneinfo->input_object_3));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->input_object_3_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->input_object_3_optsel));
	    }
	}

	if (paneinfo->input_object_4_struct != NULL)
	{
	    selection = paneinfo->input_object_4_struct->Selptr;
	    if (selection->modified)
	    {
		if (paneinfo->input_object_4 != NULL) kfree_and_NULL(paneinfo->input_object_4);
		kvf_get_attribute(selection->back_kformstruct, KVF_FILE_NAME,
		                  &(paneinfo->input_object_4));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->input_object_4_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->input_object_4_optsel));
	    }
	}

	if (paneinfo->input_object_5_struct != NULL)
	{
	    selection = paneinfo->input_object_5_struct->Selptr;
	    if (selection->modified)
	    {
		if (paneinfo->input_object_5 != NULL) kfree_and_NULL(paneinfo->input_object_5);
		kvf_get_attribute(selection->back_kformstruct, KVF_FILE_NAME,
		                  &(paneinfo->input_object_5));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->input_object_5_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->input_object_5_optsel));
	    }
	}

	if (paneinfo->input_object_6_struct != NULL)
	{
	    selection = paneinfo->input_object_6_struct->Selptr;
	    if (selection->modified)
	    {
		if (paneinfo->input_object_6 != NULL) kfree_and_NULL(paneinfo->input_object_6);
		kvf_get_attribute(selection->back_kformstruct, KVF_FILE_NAME,
		                  &(paneinfo->input_object_6));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->input_object_6_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->input_object_6_optsel));
	    }
	}

	if (paneinfo->input_object_7_struct != NULL)
	{
	    selection = paneinfo->input_object_7_struct->Selptr;
	    if (selection->modified)
	    {
		if (paneinfo->input_object_7 != NULL) kfree_and_NULL(paneinfo->input_object_7);
		kvf_get_attribute(selection->back_kformstruct, KVF_FILE_NAME,
		                  &(paneinfo->input_object_7));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->input_object_7_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->input_object_7_optsel));
	    }
	}

}


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_get_Output
| 
|       Purpose: Extracts information from Output pane
| 
|         Input: paneinfo - pointer to Render_Output structure
| 
|        Output: none
|    Written By: conductor -oname render
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void _xvf_get_Output(
     Render_Output *paneinfo)
{
	kselection *selection;

	/*
	 * extracting 'Master->Render->Output' Pane Info structure
	 */

	if (paneinfo->output_image_fname_struct != NULL)
	{
	    selection = paneinfo->output_image_fname_struct->Selptr;
	    if (selection->modified)
	    {
	    if (paneinfo->output_image_fname != NULL) kfree_and_NULL(paneinfo->output_image_fname);
	    kvf_get_attribute(selection->back_kformstruct, KVF_FILE_NAME,
	                    &(paneinfo->output_image_fname));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->output_image_fname_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->output_image_fname_optsel));
	    }
	}

	if (paneinfo->oxfrm_struct != NULL)
	{
	    selection = paneinfo->oxfrm_struct->Selptr;
	    if (selection->modified)
	    {
	    if (paneinfo->oxfrm != NULL) kfree_and_NULL(paneinfo->oxfrm);
	    kvf_get_attribute(selection->back_kformstruct, KVF_FILE_NAME,
	                    &(paneinfo->oxfrm));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->oxfrm_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->oxfrm_optsel));
	    }
	}

	if (paneinfo->oscene_struct != NULL)
	{
	    selection = paneinfo->oscene_struct->Selptr;
	    if (selection->modified)
	    {
	    if (paneinfo->oscene != NULL) kfree_and_NULL(paneinfo->oscene);
	    kvf_get_attribute(selection->back_kformstruct, KVF_FILE_NAME,
	                    &(paneinfo->oscene));
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->oscene_selected));
	    kvf_get_attribute(selection->back_kformstruct, KVF_OPTSEL,
	                    &(paneinfo->oscene_optsel));
	    }
	}

	if (paneinfo->write_scene_file_struct != NULL)
	{
	    selection = paneinfo->write_scene_file_struct->Selptr;
	    kvf_get_attribute(selection->back_kformstruct, KVF_SELECTED,
	                    &(paneinfo->write_scene_file));

	}

}


