 /*
  * Khoros: $Id: form_init.c,v 1.1.2.2 1997/05/31 19:00:19 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: form_init.c,v 1.1.2.2 1997/05/31 19:00:19 mirage Exp $";
#endif

/*
 * Copyright (C) 1996,1997 , The Regents of the University of California.
 * All rights reserved.  See $GEOMETRY/repos/license/LBL_Copyright.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	FormInfo Initialization Routines
   >>>> 
   >>>>  Private: 
   >>>> 	_xvf_init_Master
   >>>> 	_xvf_free_Master
   >>>> 
   >>>>   Static: 
   >>>> 	_xvf_init_Master_warning
   >>>> 	_xvf_init_Master_error
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "render.h"

static void _xvf_init_Master_warning PROTO((char *));
static void _xvf_init_Master_error   PROTO((char *, char *));

/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_init_Master
| 
|       Purpose: Initializes the GUI Information (gui_info_struct) structure
| 
|         Input: forminfo - ptr to _xvf_init_Master_gui_struct
|                form     - pointer to the form tree
| 
|        Output: none
|    Written By: conductor
|          Date: July 1, 1997
| Modifications:  
| 
------------------------------------------------------------*/
int _xvf_init_Master(

    kform *form, 
    gui_info_struct *forminfo)
{
	ksubform   *subform;
	kcontrol   *pane;
	kselection *selection;
	kcontrol   *guidepane;
	/*
	 * initializing 'Master' Form Info structure
	 */

	forminfo->Render_struct =
	    kvf_create_struct_from_subformname(form, "Master.Render");
	if (forminfo->Render_struct == NULL)
	{
	    _xvf_init_Master_error("subform 'Render' on form 'Master'", 
			    "Master_Render");
	    return(FALSE);
	}

	/*
	 * initializing 'Master->Render' Subform Info structure
	 */

	forminfo->Render = (Master_Render *)
				kcalloc((size_t) 1, sizeof(Master_Render));
	subform = forminfo->Render_struct->Subformptr;
	guidepane = subform->guidepane;
	forminfo->Render->license_struct =
	    kvf_create_struct_from_selname(guidepane->sel_list, "Master.Render.license");
	if (forminfo->Render->license_struct == NULL)
		    _xvf_init_Master_warning("unknown selection 'license' on subform 'Render'");

	forminfo->Render->help_struct =
	    kvf_create_struct_from_selname(guidepane->sel_list, "Master.Render.help");
	if (forminfo->Render->help_struct == NULL)
		    _xvf_init_Master_warning("unknown selection 'help' on subform 'Render'");

	forminfo->Render->quit_struct =
	    kvf_create_struct_from_selname(guidepane->sel_list, "Master.Render.quit");
	if (forminfo->Render->quit_struct == NULL)
		    _xvf_init_Master_warning("quit button 'quit' on subform 'Render'");
	forminfo->Render->quit = FALSE;

	forminfo->Render->refresh_struct =
	    kvf_create_struct_from_selname(guidepane->sel_list, "Master.Render.refresh");
	if (forminfo->Render->refresh_struct == NULL)
	    _xvf_init_Master_warning("pane action button 'refresh' on subform 'Render'");
	forminfo->Render->refresh = FALSE;
	selection = forminfo->Render->refresh_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->Render->b0_struct =
	    kvf_create_struct_from_selname(guidepane->sel_list, "Master.Render.b0");
	if (forminfo->Render->b0_struct == NULL)
	    _xvf_init_Master_warning("blank selection 'b0' on subform 'Render'");

	/*
	 * initializing 'Master->Render->Objects' Pane Info structure
	 */

	forminfo->Render->Objects = (Render_Objects *)
				kcalloc((size_t)1, sizeof(Render_Objects));
	forminfo->Render->Objects_struct =
	    kvf_create_struct_from_panename(subform, "Master.Render.Objects");
	if (forminfo->Render->Objects_struct == NULL)
	{
	    _xvf_init_Master_error("pane 'Objects' on subform 'Render'", "Render_Objects");
	    return(FALSE);
	}
	pane = forminfo->Render->Objects_struct->Controlptr;
	forminfo->Render->Objects->current_object_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.current_object");
	if (forminfo->Render->Objects->current_object_struct != NULL)
	{
	    selection = forminfo->Render->Objects->current_object_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Objects->current_object_struct, KVF_LIST_VAL,
	                    &(forminfo->Render->Objects->current_object));
	    kvf_get_attribute(forminfo->Render->Objects->current_object_struct, KVF_LIST_LABEL,
	                    &(forminfo->Render->Objects->current_object_label));
	}
	else
	    _xvf_init_Master_warning("list selection 'current_object' on pane 'Objects' of subform 'Render'");

	forminfo->Render->Objects->current_object_selected = FALSE;

	forminfo->Render->Objects->object_shader_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.object_shader");
	if (forminfo->Render->Objects->object_shader_struct != NULL)
	{
	    selection = forminfo->Render->Objects->object_shader_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Objects->object_shader_struct, KVF_LIST_VAL,
	                    &(forminfo->Render->Objects->object_shader));
	    kvf_get_attribute(forminfo->Render->Objects->object_shader_struct, KVF_LIST_LABEL,
	                    &(forminfo->Render->Objects->object_shader_label));
	}
	else
	    _xvf_init_Master_warning("list selection 'object_shader' on pane 'Objects' of subform 'Render'");

	forminfo->Render->Objects->object_shader_selected = FALSE;

	forminfo->Render->Objects->post_obj_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.post_obj");
	if (forminfo->Render->Objects->post_obj_struct != NULL)
	{
	    selection = forminfo->Render->Objects->post_obj_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Objects->post_obj_struct, KVF_LOGIC_DEF,
	                    &(forminfo->Render->Objects->post_obj));
	    kvf_get_attribute(forminfo->Render->Objects->post_obj_struct, KVF_LOGIC_LABEL,
	                    &(forminfo->Render->Objects->post_obj_label));
	}
	else
	    _xvf_init_Master_warning("logical selection 'post_obj' on pane 'Objects' of subform 'Render'");

	forminfo->Render->Objects->post_obj_selected = FALSE;

	forminfo->Render->Objects->obj_red_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.obj_red");
	if (forminfo->Render->Objects->obj_red_struct != NULL)
	{
	    selection = forminfo->Render->Objects->obj_red_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Objects->obj_red_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Objects->obj_red));
	}
	else
	    _xvf_init_Master_warning("float selection 'obj_red' on pane 'Objects' of subform 'Render'");

	forminfo->Render->Objects->obj_red_selected = FALSE;
	forminfo->Render->Objects->obj_red_optsel = TRUE;

	forminfo->Render->Objects->obj_green_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.obj_green");
	if (forminfo->Render->Objects->obj_green_struct != NULL)
	{
	    selection = forminfo->Render->Objects->obj_green_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Objects->obj_green_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Objects->obj_green));
	}
	else
	    _xvf_init_Master_warning("float selection 'obj_green' on pane 'Objects' of subform 'Render'");

	forminfo->Render->Objects->obj_green_selected = FALSE;
	forminfo->Render->Objects->obj_green_optsel = TRUE;

	forminfo->Render->Objects->obj_blue_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.obj_blue");
	if (forminfo->Render->Objects->obj_blue_struct != NULL)
	{
	    selection = forminfo->Render->Objects->obj_blue_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Objects->obj_blue_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Objects->obj_blue));
	}
	else
	    _xvf_init_Master_warning("float selection 'obj_blue' on pane 'Objects' of subform 'Render'");

	forminfo->Render->Objects->obj_blue_selected = FALSE;
	forminfo->Render->Objects->obj_blue_optsel = TRUE;

	forminfo->Render->Objects->obj_alpha_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.obj_alpha");
	if (forminfo->Render->Objects->obj_alpha_struct != NULL)
	{
	    selection = forminfo->Render->Objects->obj_alpha_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Objects->obj_alpha_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Objects->obj_alpha));
	}
	else
	    _xvf_init_Master_warning("float selection 'obj_alpha' on pane 'Objects' of subform 'Render'");

	forminfo->Render->Objects->obj_alpha_selected = FALSE;
	forminfo->Render->Objects->obj_alpha_optsel = TRUE;

	forminfo->Render->Objects->obj_ambient_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.obj_ambient");
	if (forminfo->Render->Objects->obj_ambient_struct != NULL)
	{
	    selection = forminfo->Render->Objects->obj_ambient_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Objects->obj_ambient_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Objects->obj_ambient));
	}
	else
	    _xvf_init_Master_warning("float selection 'obj_ambient' on pane 'Objects' of subform 'Render'");

	forminfo->Render->Objects->obj_ambient_selected = FALSE;
	forminfo->Render->Objects->obj_ambient_optsel = TRUE;

	forminfo->Render->Objects->obj_diffuse_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.obj_diffuse");
	if (forminfo->Render->Objects->obj_diffuse_struct != NULL)
	{
	    selection = forminfo->Render->Objects->obj_diffuse_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Objects->obj_diffuse_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Objects->obj_diffuse));
	}
	else
	    _xvf_init_Master_warning("float selection 'obj_diffuse' on pane 'Objects' of subform 'Render'");

	forminfo->Render->Objects->obj_diffuse_selected = FALSE;
	forminfo->Render->Objects->obj_diffuse_optsel = TRUE;

	forminfo->Render->Objects->obj_spec_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.obj_spec");
	if (forminfo->Render->Objects->obj_spec_struct != NULL)
	{
	    selection = forminfo->Render->Objects->obj_spec_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Objects->obj_spec_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Objects->obj_spec));
	}
	else
	    _xvf_init_Master_warning("float selection 'obj_spec' on pane 'Objects' of subform 'Render'");

	forminfo->Render->Objects->obj_spec_selected = FALSE;
	forminfo->Render->Objects->obj_spec_optsel = TRUE;

	forminfo->Render->Objects->obj_spec_exp_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.obj_spec_exp");
	if (forminfo->Render->Objects->obj_spec_exp_struct != NULL)
	{
	    selection = forminfo->Render->Objects->obj_spec_exp_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Objects->obj_spec_exp_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Objects->obj_spec_exp));
	}
	else
	    _xvf_init_Master_warning("float selection 'obj_spec_exp' on pane 'Objects' of subform 'Render'");

	forminfo->Render->Objects->obj_spec_exp_selected = FALSE;
	forminfo->Render->Objects->obj_spec_exp_optsel = TRUE;

	forminfo->Render->Objects->obj_spec_metal_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.obj_spec_metal");
	if (forminfo->Render->Objects->obj_spec_metal_struct != NULL)
	{
	    selection = forminfo->Render->Objects->obj_spec_metal_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Objects->obj_spec_metal_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Objects->obj_spec_metal));
	}
	else
	    _xvf_init_Master_warning("float selection 'obj_spec_metal' on pane 'Objects' of subform 'Render'");

	forminfo->Render->Objects->obj_spec_metal_selected = FALSE;
	forminfo->Render->Objects->obj_spec_metal_optsel = TRUE;

	forminfo->Render->Objects->object_info_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.object_info");
	if (forminfo->Render->Objects->object_info_struct == NULL)
	    _xvf_init_Master_warning("pane action button 'object_info' on pane 'Objects' of subform 'Render'");
	forminfo->Render->Objects->object_info = FALSE;
	selection = forminfo->Render->Objects->object_info_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->Render->Objects->action1_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.action1");
	if (forminfo->Render->Objects->action1_struct == NULL)
	    _xvf_init_Master_warning("pane action button 'action1' on pane 'Objects' of subform 'Render'");
	forminfo->Render->Objects->action1 = FALSE;
	selection = forminfo->Render->Objects->action1_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->Render->Objects->object_properties_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Objects.object_properties");
	if (forminfo->Render->Objects->object_properties_struct == NULL)
	    _xvf_init_Master_warning("blank selection 'object_properties' on pane 'Objects' of subform 'Render'");

	/*
	 * initializing 'Master->Render->Cameras' Pane Info structure
	 */

	forminfo->Render->Cameras = (Render_Cameras *)
				kcalloc((size_t)1, sizeof(Render_Cameras));
	forminfo->Render->Cameras_struct =
	    kvf_create_struct_from_panename(subform, "Master.Render.Cameras");
	if (forminfo->Render->Cameras_struct == NULL)
	{
	    _xvf_init_Master_error("pane 'Cameras' on subform 'Render'", "Render_Cameras");
	    return(FALSE);
	}
	pane = forminfo->Render->Cameras_struct->Controlptr;
	forminfo->Render->Cameras->projection_cycle_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.projection_cycle");
	if (forminfo->Render->Cameras->projection_cycle_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->projection_cycle_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->projection_cycle_struct, KVF_CYCLE_VAL,
	                    &(forminfo->Render->Cameras->projection_cycle));
	    kvf_get_attribute(forminfo->Render->Cameras->projection_cycle_struct, KVF_CYCLE_LABEL,
	                    &(forminfo->Render->Cameras->projection_cycle_label));
	}
	else
	    _xvf_init_Master_warning("cycle selection 'projection_cycle' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->projection_cycle_selected = FALSE;

	forminfo->Render->Cameras->eye_x_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.eye_x");
	if (forminfo->Render->Cameras->eye_x_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->eye_x_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->eye_x_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->eye_x));
	}
	else
	    _xvf_init_Master_warning("float selection 'eye_x' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->eye_x_selected = FALSE;
	forminfo->Render->Cameras->eye_x_optsel = TRUE;

	forminfo->Render->Cameras->up_x_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.up_x");
	if (forminfo->Render->Cameras->up_x_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->up_x_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->up_x_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->up_x));
	}
	else
	    _xvf_init_Master_warning("float selection 'up_x' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->up_x_selected = FALSE;
	forminfo->Render->Cameras->up_x_optsel = TRUE;

	forminfo->Render->Cameras->at_x_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.at_x");
	if (forminfo->Render->Cameras->at_x_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->at_x_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->at_x_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->at_x));
	}
	else
	    _xvf_init_Master_warning("float selection 'at_x' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->at_x_selected = FALSE;
	forminfo->Render->Cameras->at_x_optsel = TRUE;

	forminfo->Render->Cameras->eye_y_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.eye_y");
	if (forminfo->Render->Cameras->eye_y_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->eye_y_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->eye_y_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->eye_y));
	}
	else
	    _xvf_init_Master_warning("float selection 'eye_y' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->eye_y_selected = FALSE;
	forminfo->Render->Cameras->eye_y_optsel = TRUE;

	forminfo->Render->Cameras->up_y_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.up_y");
	if (forminfo->Render->Cameras->up_y_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->up_y_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->up_y_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->up_y));
	}
	else
	    _xvf_init_Master_warning("float selection 'up_y' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->up_y_selected = FALSE;
	forminfo->Render->Cameras->up_y_optsel = TRUE;

	forminfo->Render->Cameras->at_y_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.at_y");
	if (forminfo->Render->Cameras->at_y_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->at_y_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->at_y_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->at_y));
	}
	else
	    _xvf_init_Master_warning("float selection 'at_y' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->at_y_selected = FALSE;
	forminfo->Render->Cameras->at_y_optsel = TRUE;

	forminfo->Render->Cameras->eye_z_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.eye_z");
	if (forminfo->Render->Cameras->eye_z_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->eye_z_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->eye_z_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->eye_z));
	}
	else
	    _xvf_init_Master_warning("float selection 'eye_z' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->eye_z_selected = FALSE;
	forminfo->Render->Cameras->eye_z_optsel = TRUE;

	forminfo->Render->Cameras->up_z_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.up_z");
	if (forminfo->Render->Cameras->up_z_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->up_z_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->up_z_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->up_z));
	}
	else
	    _xvf_init_Master_warning("float selection 'up_z' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->up_z_selected = FALSE;
	forminfo->Render->Cameras->up_z_optsel = TRUE;

	forminfo->Render->Cameras->at_z_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.at_z");
	if (forminfo->Render->Cameras->at_z_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->at_z_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->at_z_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->at_z));
	}
	else
	    _xvf_init_Master_warning("float selection 'at_z' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->at_z_selected = FALSE;
	forminfo->Render->Cameras->at_z_optsel = TRUE;

	forminfo->Render->Cameras->hither_clip_form_loc_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.hither_clip_form_loc");
	if (forminfo->Render->Cameras->hither_clip_form_loc_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->hither_clip_form_loc_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->hither_clip_form_loc_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->hither_clip_form_loc));
	}
	else
	    _xvf_init_Master_warning("float selection 'hither_clip_form_loc' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->hither_clip_form_loc_selected = FALSE;
	forminfo->Render->Cameras->hither_clip_form_loc_optsel = TRUE;

	forminfo->Render->Cameras->yon_clip_form_loc_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.yon_clip_form_loc");
	if (forminfo->Render->Cameras->yon_clip_form_loc_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->yon_clip_form_loc_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->yon_clip_form_loc_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->yon_clip_form_loc));
	}
	else
	    _xvf_init_Master_warning("float selection 'yon_clip_form_loc' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->yon_clip_form_loc_selected = FALSE;
	forminfo->Render->Cameras->yon_clip_form_loc_optsel = TRUE;

	forminfo->Render->Cameras->back_clip_form_on_off_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.back_clip_form_on_off");
	if (forminfo->Render->Cameras->back_clip_form_on_off_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->back_clip_form_on_off_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->back_clip_form_on_off_struct, KVF_LOGIC_DEF,
	                    &(forminfo->Render->Cameras->back_clip_form_on_off));
	    kvf_get_attribute(forminfo->Render->Cameras->back_clip_form_on_off_struct, KVF_LOGIC_LABEL,
	                    &(forminfo->Render->Cameras->back_clip_form_on_off_label));
	}
	else
	    _xvf_init_Master_warning("logical selection 'back_clip_form_on_off' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->back_clip_form_on_off_selected = FALSE;

	forminfo->Render->Cameras->fov_input_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.fov_input");
	if (forminfo->Render->Cameras->fov_input_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->fov_input_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->fov_input_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->fov_input));
	}
	else
	    _xvf_init_Master_warning("float selection 'fov_input' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->fov_input_selected = FALSE;
	forminfo->Render->Cameras->fov_input_optsel = TRUE;

	forminfo->Render->Cameras->aspect_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.aspect");
	if (forminfo->Render->Cameras->aspect_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->aspect_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->aspect_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->aspect));
	}
	else
	    _xvf_init_Master_warning("float selection 'aspect' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->aspect_selected = FALSE;
	forminfo->Render->Cameras->aspect_optsel = TRUE;

	forminfo->Render->Cameras->normalize_cam_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.normalize_cam");
	if (forminfo->Render->Cameras->normalize_cam_struct == NULL)
	    _xvf_init_Master_warning("pane action button 'normalize_cam' on pane 'Cameras' of subform 'Render'");
	forminfo->Render->Cameras->normalize_cam = FALSE;
	selection = forminfo->Render->Cameras->normalize_cam_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->Render->Cameras->reset_camera_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.reset_camera");
	if (forminfo->Render->Cameras->reset_camera_struct == NULL)
	    _xvf_init_Master_warning("pane action button 'reset_camera' on pane 'Cameras' of subform 'Render'");
	forminfo->Render->Cameras->reset_camera = FALSE;
	selection = forminfo->Render->Cameras->reset_camera_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->Render->Cameras->camera_type_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.camera_type");
	if (forminfo->Render->Cameras->camera_type_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->camera_type_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->camera_type_struct, KVF_LIST_VAL,
	                    &(forminfo->Render->Cameras->camera_type));
	    kvf_get_attribute(forminfo->Render->Cameras->camera_type_struct, KVF_LIST_LABEL,
	                    &(forminfo->Render->Cameras->camera_type_label));
	}
	else
	    _xvf_init_Master_warning("list selection 'camera_type' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->camera_type_selected = FALSE;

	forminfo->Render->Cameras->eye_separation_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.eye_separation");
	if (forminfo->Render->Cameras->eye_separation_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->eye_separation_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->eye_separation_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->eye_separation));
	}
	else
	    _xvf_init_Master_warning("float selection 'eye_separation' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->eye_separation_selected = FALSE;

	forminfo->Render->Cameras->focal_length_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.focal_length");
	if (forminfo->Render->Cameras->focal_length_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->focal_length_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->focal_length_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Cameras->focal_length));
	}
	else
	    _xvf_init_Master_warning("float selection 'focal_length' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->focal_length_selected = FALSE;

	forminfo->Render->Cameras->lr_flip_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.lr_flip");
	if (forminfo->Render->Cameras->lr_flip_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->lr_flip_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->lr_flip_struct, KVF_LOGIC_DEF,
	                    &(forminfo->Render->Cameras->lr_flip));
	    kvf_get_attribute(forminfo->Render->Cameras->lr_flip_struct, KVF_LOGIC_LABEL,
	                    &(forminfo->Render->Cameras->lr_flip_label));
	}
	else
	    _xvf_init_Master_warning("logical selection 'lr_flip' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->lr_flip_selected = FALSE;

	forminfo->Render->Cameras->multisample_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.multisample");
	if (forminfo->Render->Cameras->multisample_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->multisample_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->multisample_struct, KVF_LOGIC_DEF,
	                    &(forminfo->Render->Cameras->multisample));
	    kvf_get_attribute(forminfo->Render->Cameras->multisample_struct, KVF_LOGIC_LABEL,
	                    &(forminfo->Render->Cameras->multisample_label));
	}
	else
	    _xvf_init_Master_warning("logical selection 'multisample' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->multisample_selected = FALSE;

	forminfo->Render->Cameras->do_vof_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.do_vof");
	if (forminfo->Render->Cameras->do_vof_struct != NULL)
	{
	    selection = forminfo->Render->Cameras->do_vof_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Cameras->do_vof_struct, KVF_LIST_VAL,
	                    &(forminfo->Render->Cameras->do_vof));
	    kvf_get_attribute(forminfo->Render->Cameras->do_vof_struct, KVF_LIST_LABEL,
	                    &(forminfo->Render->Cameras->do_vof_label));
	}
	else
	    _xvf_init_Master_warning("list selection 'do_vof' on pane 'Cameras' of subform 'Render'");

	forminfo->Render->Cameras->do_vof_selected = FALSE;

	forminfo->Render->Cameras->normalize_camera_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.normalize_camera");
	if (forminfo->Render->Cameras->normalize_camera_struct == NULL)
	    _xvf_init_Master_warning("blank selection 'normalize_camera' on pane 'Cameras' of subform 'Render'");
	forminfo->Render->Cameras->blank1_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.blank1");
	if (forminfo->Render->Cameras->blank1_struct == NULL)
	    _xvf_init_Master_warning("blank selection 'blank1' on pane 'Cameras' of subform 'Render'");
	forminfo->Render->Cameras->blank2_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.blank2");
	if (forminfo->Render->Cameras->blank2_struct == NULL)
	    _xvf_init_Master_warning("blank selection 'blank2' on pane 'Cameras' of subform 'Render'");
	forminfo->Render->Cameras->blank3_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.blank3");
	if (forminfo->Render->Cameras->blank3_struct == NULL)
	    _xvf_init_Master_warning("blank selection 'blank3' on pane 'Cameras' of subform 'Render'");
	forminfo->Render->Cameras->blank4_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Cameras.blank4");
	if (forminfo->Render->Cameras->blank4_struct == NULL)
	    _xvf_init_Master_warning("blank selection 'blank4' on pane 'Cameras' of subform 'Render'");

	/*
	 * initializing 'Master->Render->Lights' Pane Info structure
	 */

	forminfo->Render->Lights = (Render_Lights *)
				kcalloc((size_t)1, sizeof(Render_Lights));
	forminfo->Render->Lights_struct =
	    kvf_create_struct_from_panename(subform, "Master.Render.Lights");
	if (forminfo->Render->Lights_struct == NULL)
	{
	    _xvf_init_Master_error("pane 'Lights' on subform 'Render'", "Render_Lights");
	    return(FALSE);
	}
	pane = forminfo->Render->Lights_struct->Controlptr;
	forminfo->Render->Lights->light_list_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Lights.light_list");
	if (forminfo->Render->Lights->light_list_struct != NULL)
	{
	    selection = forminfo->Render->Lights->light_list_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Lights->light_list_struct, KVF_LIST_VAL,
	                    &(forminfo->Render->Lights->light_list));
	    kvf_get_attribute(forminfo->Render->Lights->light_list_struct, KVF_LIST_LABEL,
	                    &(forminfo->Render->Lights->light_list_label));
	}
	else
	    _xvf_init_Master_warning("list selection 'light_list' on pane 'Lights' of subform 'Render'");

	forminfo->Render->Lights->light_list_selected = FALSE;

	forminfo->Render->Lights->light_onoff_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Lights.light_onoff");
	if (forminfo->Render->Lights->light_onoff_struct != NULL)
	{
	    selection = forminfo->Render->Lights->light_onoff_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Lights->light_onoff_struct, KVF_LOGIC_DEF,
	                    &(forminfo->Render->Lights->light_onoff));
	    kvf_get_attribute(forminfo->Render->Lights->light_onoff_struct, KVF_LOGIC_LABEL,
	                    &(forminfo->Render->Lights->light_onoff_label));
	}
	else
	    _xvf_init_Master_warning("logical selection 'light_onoff' on pane 'Lights' of subform 'Render'");

	forminfo->Render->Lights->light_onoff_selected = FALSE;

	forminfo->Render->Lights->light_type_list_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Lights.light_type_list");
	if (forminfo->Render->Lights->light_type_list_struct != NULL)
	{
	    selection = forminfo->Render->Lights->light_type_list_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Lights->light_type_list_struct, KVF_LIST_VAL,
	                    &(forminfo->Render->Lights->light_type_list));
	    kvf_get_attribute(forminfo->Render->Lights->light_type_list_struct, KVF_LIST_LABEL,
	                    &(forminfo->Render->Lights->light_type_list_label));
	}
	else
	    _xvf_init_Master_warning("list selection 'light_type_list' on pane 'Lights' of subform 'Render'");

	forminfo->Render->Lights->light_type_list_selected = FALSE;

	forminfo->Render->Lights->light_position_string_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Lights.light_position_string");
	if (forminfo->Render->Lights->light_position_string_struct != NULL)
	{
	    selection = forminfo->Render->Lights->light_position_string_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Lights->light_position_string_struct, KVF_STRING_DEF,
	                    &(forminfo->Render->Lights->light_position_string));
	}
	else
	    _xvf_init_Master_warning("string selection 'light_position_string' on pane 'Lights' of subform 'Render'");

	forminfo->Render->Lights->light_position_string_selected = FALSE;
	forminfo->Render->Lights->light_position_string_optsel = TRUE;

	forminfo->Render->Lights->light_direction_string_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Lights.light_direction_string");
	if (forminfo->Render->Lights->light_direction_string_struct != NULL)
	{
	    selection = forminfo->Render->Lights->light_direction_string_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Lights->light_direction_string_struct, KVF_STRING_DEF,
	                    &(forminfo->Render->Lights->light_direction_string));
	}
	else
	    _xvf_init_Master_warning("string selection 'light_direction_string' on pane 'Lights' of subform 'Render'");

	forminfo->Render->Lights->light_direction_string_selected = FALSE;
	forminfo->Render->Lights->light_direction_string_optsel = TRUE;

	forminfo->Render->Lights->current_light_color_string_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Lights.current_light_color_string");
	if (forminfo->Render->Lights->current_light_color_string_struct != NULL)
	{
	    selection = forminfo->Render->Lights->current_light_color_string_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Lights->current_light_color_string_struct, KVF_STRING_DEF,
	                    &(forminfo->Render->Lights->current_light_color_string));
	}
	else
	    _xvf_init_Master_warning("string selection 'current_light_color_string' on pane 'Lights' of subform 'Render'");

	forminfo->Render->Lights->current_light_color_string_selected = FALSE;
	forminfo->Render->Lights->current_light_color_string_optsel = TRUE;

	forminfo->Render->Lights->ambient_light_color_string_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Lights.ambient_light_color_string");
	if (forminfo->Render->Lights->ambient_light_color_string_struct != NULL)
	{
	    selection = forminfo->Render->Lights->ambient_light_color_string_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Lights->ambient_light_color_string_struct, KVF_STRING_DEF,
	                    &(forminfo->Render->Lights->ambient_light_color_string));
	}
	else
	    _xvf_init_Master_warning("string selection 'ambient_light_color_string' on pane 'Lights' of subform 'Render'");

	forminfo->Render->Lights->ambient_light_color_string_selected = FALSE;
	forminfo->Render->Lights->ambient_light_color_string_optsel = TRUE;


	/*
	 * initializing 'Master->Render->Imaging' Pane Info structure
	 */

	forminfo->Render->Imaging = (Render_Imaging *)
				kcalloc((size_t)1, sizeof(Render_Imaging));
	forminfo->Render->Imaging_struct =
	    kvf_create_struct_from_panename(subform, "Master.Render.Imaging");
	if (forminfo->Render->Imaging_struct == NULL)
	{
	    _xvf_init_Master_error("pane 'Imaging' on subform 'Render'", "Render_Imaging");
	    return(FALSE);
	}
	pane = forminfo->Render->Imaging_struct->Controlptr;
	forminfo->Render->Imaging->image_width_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Imaging.image_width");
	if (forminfo->Render->Imaging->image_width_struct != NULL)
	{
	    selection = forminfo->Render->Imaging->image_width_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Imaging->image_width_struct, KVF_INT_DEF,
	                    &(forminfo->Render->Imaging->image_width));
	}
	else
	    _xvf_init_Master_warning("integer selection 'image_width' on pane 'Imaging' of subform 'Render'");

	forminfo->Render->Imaging->image_width_selected = FALSE;
	forminfo->Render->Imaging->image_width_optsel = TRUE;

	forminfo->Render->Imaging->image_height_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Imaging.image_height");
	if (forminfo->Render->Imaging->image_height_struct != NULL)
	{
	    selection = forminfo->Render->Imaging->image_height_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Imaging->image_height_struct, KVF_INT_DEF,
	                    &(forminfo->Render->Imaging->image_height));
	}
	else
	    _xvf_init_Master_warning("integer selection 'image_height' on pane 'Imaging' of subform 'Render'");

	forminfo->Render->Imaging->image_height_selected = FALSE;
	forminfo->Render->Imaging->image_height_optsel = TRUE;

	forminfo->Render->Imaging->image_display_choice_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Imaging.image_display_choice");
	if (forminfo->Render->Imaging->image_display_choice_struct != NULL)
	{
	    selection = forminfo->Render->Imaging->image_display_choice_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Imaging->image_display_choice_struct, KVF_LIST_VAL,
	                    &(forminfo->Render->Imaging->image_display_choice));
	    kvf_get_attribute(forminfo->Render->Imaging->image_display_choice_struct, KVF_LIST_LABEL,
	                    &(forminfo->Render->Imaging->image_display_choice_label));
	}
	else
	    _xvf_init_Master_warning("list selection 'image_display_choice' on pane 'Imaging' of subform 'Render'");

	forminfo->Render->Imaging->image_display_choice_selected = FALSE;

	forminfo->Render->Imaging->red_background_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Imaging.red_background");
	if (forminfo->Render->Imaging->red_background_struct != NULL)
	{
	    selection = forminfo->Render->Imaging->red_background_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Imaging->red_background_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Imaging->red_background));
	}
	else
	    _xvf_init_Master_warning("float selection 'red_background' on pane 'Imaging' of subform 'Render'");

	forminfo->Render->Imaging->red_background_selected = FALSE;
	forminfo->Render->Imaging->red_background_optsel = TRUE;

	forminfo->Render->Imaging->green_background_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Imaging.green_background");
	if (forminfo->Render->Imaging->green_background_struct != NULL)
	{
	    selection = forminfo->Render->Imaging->green_background_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Imaging->green_background_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Imaging->green_background));
	}
	else
	    _xvf_init_Master_warning("float selection 'green_background' on pane 'Imaging' of subform 'Render'");

	forminfo->Render->Imaging->green_background_selected = FALSE;
	forminfo->Render->Imaging->green_background_optsel = TRUE;

	forminfo->Render->Imaging->blue_background_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Imaging.blue_background");
	if (forminfo->Render->Imaging->blue_background_struct != NULL)
	{
	    selection = forminfo->Render->Imaging->blue_background_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Imaging->blue_background_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Imaging->blue_background));
	}
	else
	    _xvf_init_Master_warning("float selection 'blue_background' on pane 'Imaging' of subform 'Render'");

	forminfo->Render->Imaging->blue_background_selected = FALSE;
	forminfo->Render->Imaging->blue_background_optsel = TRUE;

	forminfo->Render->Imaging->renderer_choice_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Imaging.renderer_choice");
	if (forminfo->Render->Imaging->renderer_choice_struct != NULL)
	{
	    selection = forminfo->Render->Imaging->renderer_choice_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Imaging->renderer_choice_struct, KVF_LIST_VAL,
	                    &(forminfo->Render->Imaging->renderer_choice));
	    kvf_get_attribute(forminfo->Render->Imaging->renderer_choice_struct, KVF_LIST_LABEL,
	                    &(forminfo->Render->Imaging->renderer_choice_label));
	}
	else
	    _xvf_init_Master_warning("list selection 'renderer_choice' on pane 'Imaging' of subform 'Render'");

	forminfo->Render->Imaging->renderer_choice_selected = FALSE;

	forminfo->Render->Imaging->invoke_renderer_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Imaging.invoke_renderer");
	if (forminfo->Render->Imaging->invoke_renderer_struct != NULL)
	{
	    selection = forminfo->Render->Imaging->invoke_renderer_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Imaging->invoke_renderer_struct, KVF_LOGIC_DEF,
	                    &(forminfo->Render->Imaging->invoke_renderer));
	    kvf_get_attribute(forminfo->Render->Imaging->invoke_renderer_struct, KVF_LOGIC_LABEL,
	                    &(forminfo->Render->Imaging->invoke_renderer_label));
	}
	else
	    _xvf_init_Master_warning("logical selection 'invoke_renderer' on pane 'Imaging' of subform 'Render'");

	forminfo->Render->Imaging->invoke_renderer_selected = FALSE;

	forminfo->Render->Imaging->transparency_choice_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Imaging.transparency_choice");
	if (forminfo->Render->Imaging->transparency_choice_struct != NULL)
	{
	    selection = forminfo->Render->Imaging->transparency_choice_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Imaging->transparency_choice_struct, KVF_LIST_VAL,
	                    &(forminfo->Render->Imaging->transparency_choice));
	    kvf_get_attribute(forminfo->Render->Imaging->transparency_choice_struct, KVF_LIST_LABEL,
	                    &(forminfo->Render->Imaging->transparency_choice_label));
	}
	else
	    _xvf_init_Master_warning("list selection 'transparency_choice' on pane 'Imaging' of subform 'Render'");

	forminfo->Render->Imaging->transparency_choice_selected = FALSE;

	forminfo->Render->Imaging->texture_method_2d_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Imaging.texture_method_2d");
	if (forminfo->Render->Imaging->texture_method_2d_struct != NULL)
	{
	    selection = forminfo->Render->Imaging->texture_method_2d_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Imaging->texture_method_2d_struct, KVF_LIST_VAL,
	                    &(forminfo->Render->Imaging->texture_method_2d));
	    kvf_get_attribute(forminfo->Render->Imaging->texture_method_2d_struct, KVF_LIST_LABEL,
	                    &(forminfo->Render->Imaging->texture_method_2d_label));
	}
	else
	    _xvf_init_Master_warning("list selection 'texture_method_2d' on pane 'Imaging' of subform 'Render'");

	forminfo->Render->Imaging->texture_method_2d_selected = FALSE;

	forminfo->Render->Imaging->antialias_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Imaging.antialias");
	if (forminfo->Render->Imaging->antialias_struct != NULL)
	{
	    selection = forminfo->Render->Imaging->antialias_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Imaging->antialias_struct, KVF_LOGIC_DEF,
	                    &(forminfo->Render->Imaging->antialias));
	    kvf_get_attribute(forminfo->Render->Imaging->antialias_struct, KVF_LOGIC_LABEL,
	                    &(forminfo->Render->Imaging->antialias_label));
	}
	else
	    _xvf_init_Master_warning("logical selection 'antialias' on pane 'Imaging' of subform 'Render'");

	forminfo->Render->Imaging->antialias_selected = FALSE;

	forminfo->Render->Imaging->render_controls_label_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Imaging.render_controls_label");
	if (forminfo->Render->Imaging->render_controls_label_struct == NULL)
	    _xvf_init_Master_warning("blank selection 'render_controls_label' on pane 'Imaging' of subform 'Render'");
	forminfo->Render->Imaging->blank1_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Imaging.blank1");
	if (forminfo->Render->Imaging->blank1_struct == NULL)
	    _xvf_init_Master_warning("blank selection 'blank1' on pane 'Imaging' of subform 'Render'");
	forminfo->Render->Imaging->ogl_sphere_subdivisions_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Imaging.ogl_sphere_subdivisions");
	if (forminfo->Render->Imaging->ogl_sphere_subdivisions_struct != NULL)
	{
	    selection = forminfo->Render->Imaging->ogl_sphere_subdivisions_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Imaging->ogl_sphere_subdivisions_struct, KVF_INT_DEF,
	                    &(forminfo->Render->Imaging->ogl_sphere_subdivisions));
	}
	else
	    _xvf_init_Master_warning("integer selection 'ogl_sphere_subdivisions' on pane 'Imaging' of subform 'Render'");

	forminfo->Render->Imaging->ogl_sphere_subdivisions_selected = FALSE;


	/*
	 * initializing 'Master->Render->Transformations' Pane Info structure
	 */

	forminfo->Render->Transformations = (Render_Transformations *)
				kcalloc((size_t)1, sizeof(Render_Transformations));
	forminfo->Render->Transformations_struct =
	    kvf_create_struct_from_panename(subform, "Master.Render.Transformations");
	if (forminfo->Render->Transformations_struct == NULL)
	{
	    _xvf_init_Master_error("pane 'Transformations' on subform 'Render'", "Render_Transformations");
	    return(FALSE);
	}
	pane = forminfo->Render->Transformations_struct->Controlptr;
	forminfo->Render->Transformations->x_rotate_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Transformations.x_rotate");
	if (forminfo->Render->Transformations->x_rotate_struct != NULL)
	{
	    selection = forminfo->Render->Transformations->x_rotate_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Transformations->x_rotate_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Transformations->x_rotate));
	}
	else
	    _xvf_init_Master_warning("float selection 'x_rotate' on pane 'Transformations' of subform 'Render'");

	forminfo->Render->Transformations->x_rotate_selected = FALSE;
	forminfo->Render->Transformations->x_rotate_optsel = TRUE;

	forminfo->Render->Transformations->y_rotate_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Transformations.y_rotate");
	if (forminfo->Render->Transformations->y_rotate_struct != NULL)
	{
	    selection = forminfo->Render->Transformations->y_rotate_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Transformations->y_rotate_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Transformations->y_rotate));
	}
	else
	    _xvf_init_Master_warning("float selection 'y_rotate' on pane 'Transformations' of subform 'Render'");

	forminfo->Render->Transformations->y_rotate_selected = FALSE;
	forminfo->Render->Transformations->y_rotate_optsel = TRUE;

	forminfo->Render->Transformations->z_rotate_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Transformations.z_rotate");
	if (forminfo->Render->Transformations->z_rotate_struct != NULL)
	{
	    selection = forminfo->Render->Transformations->z_rotate_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Transformations->z_rotate_struct, KVF_FLOAT_DEF,
	                    &(forminfo->Render->Transformations->z_rotate));
	}
	else
	    _xvf_init_Master_warning("float selection 'z_rotate' on pane 'Transformations' of subform 'Render'");

	forminfo->Render->Transformations->z_rotate_selected = FALSE;
	forminfo->Render->Transformations->z_rotate_optsel = TRUE;

	forminfo->Render->Transformations->center_point_string_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Transformations.center_point_string");
	if (forminfo->Render->Transformations->center_point_string_struct != NULL)
	{
	    selection = forminfo->Render->Transformations->center_point_string_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Transformations->center_point_string_struct, KVF_STRING_DEF,
	                    &(forminfo->Render->Transformations->center_point_string));
	}
	else
	    _xvf_init_Master_warning("string selection 'center_point_string' on pane 'Transformations' of subform 'Render'");

	forminfo->Render->Transformations->center_point_string_selected = FALSE;
	forminfo->Render->Transformations->center_point_string_optsel = TRUE;

	forminfo->Render->Transformations->translate_vector_string_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Transformations.translate_vector_string");
	if (forminfo->Render->Transformations->translate_vector_string_struct != NULL)
	{
	    selection = forminfo->Render->Transformations->translate_vector_string_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Transformations->translate_vector_string_struct, KVF_STRING_DEF,
	                    &(forminfo->Render->Transformations->translate_vector_string));
	}
	else
	    _xvf_init_Master_warning("string selection 'translate_vector_string' on pane 'Transformations' of subform 'Render'");

	forminfo->Render->Transformations->translate_vector_string_selected = FALSE;
	forminfo->Render->Transformations->translate_vector_string_optsel = TRUE;

	forminfo->Render->Transformations->xyz_scale_string_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Transformations.xyz_scale_string");
	if (forminfo->Render->Transformations->xyz_scale_string_struct != NULL)
	{
	    selection = forminfo->Render->Transformations->xyz_scale_string_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Transformations->xyz_scale_string_struct, KVF_STRING_DEF,
	                    &(forminfo->Render->Transformations->xyz_scale_string));
	}
	else
	    _xvf_init_Master_warning("string selection 'xyz_scale_string' on pane 'Transformations' of subform 'Render'");

	forminfo->Render->Transformations->xyz_scale_string_selected = FALSE;
	forminfo->Render->Transformations->xyz_scale_string_optsel = TRUE;

	forminfo->Render->Transformations->transformation_mode_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Transformations.transformation_mode");
	if (forminfo->Render->Transformations->transformation_mode_struct != NULL)
	{
	    selection = forminfo->Render->Transformations->transformation_mode_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Transformations->transformation_mode_struct, KVF_LOGIC_DEF,
	                    &(forminfo->Render->Transformations->transformation_mode));
	    kvf_get_attribute(forminfo->Render->Transformations->transformation_mode_struct, KVF_LOGIC_LABEL,
	                    &(forminfo->Render->Transformations->transformation_mode_label));
	}
	else
	    _xvf_init_Master_warning("logical selection 'transformation_mode' on pane 'Transformations' of subform 'Render'");

	forminfo->Render->Transformations->transformation_mode_selected = FALSE;


	/*
	 * initializing 'Master->Render->Input' Pane Info structure
	 */

	forminfo->Render->Input = (Render_Input *)
				kcalloc((size_t)1, sizeof(Render_Input));
	forminfo->Render->Input_struct =
	    kvf_create_struct_from_panename(subform, "Master.Render.Input");
	if (forminfo->Render->Input_struct == NULL)
	{
	    _xvf_init_Master_error("pane 'Input' on subform 'Render'", "Render_Input");
	    return(FALSE);
	}
	pane = forminfo->Render->Input_struct->Controlptr;
	forminfo->Render->Input->input_object_1_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Input.input_object_1");
	if (forminfo->Render->Input->input_object_1_struct != NULL)
	{
	    selection = forminfo->Render->Input->input_object_1_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Input->input_object_1_struct, KVF_FILE_DEF,
	                    &(forminfo->Render->Input->input_object_1));
	}
	else
	    _xvf_init_Master_warning("input selection 'input_object_1' on pane 'Input' of subform 'Render'");

	forminfo->Render->Input->input_object_1_selected = FALSE;
	forminfo->Render->Input->input_object_1_optsel = FALSE;

	forminfo->Render->Input->input_object_2_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Input.input_object_2");
	if (forminfo->Render->Input->input_object_2_struct != NULL)
	{
	    selection = forminfo->Render->Input->input_object_2_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Input->input_object_2_struct, KVF_FILE_DEF,
	                    &(forminfo->Render->Input->input_object_2));
	}
	else
	    _xvf_init_Master_warning("input selection 'input_object_2' on pane 'Input' of subform 'Render'");

	forminfo->Render->Input->input_object_2_selected = FALSE;
	forminfo->Render->Input->input_object_2_optsel = FALSE;

	forminfo->Render->Input->input_object_3_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Input.input_object_3");
	if (forminfo->Render->Input->input_object_3_struct != NULL)
	{
	    selection = forminfo->Render->Input->input_object_3_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Input->input_object_3_struct, KVF_FILE_DEF,
	                    &(forminfo->Render->Input->input_object_3));
	}
	else
	    _xvf_init_Master_warning("input selection 'input_object_3' on pane 'Input' of subform 'Render'");

	forminfo->Render->Input->input_object_3_selected = FALSE;
	forminfo->Render->Input->input_object_3_optsel = FALSE;

	forminfo->Render->Input->input_object_4_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Input.input_object_4");
	if (forminfo->Render->Input->input_object_4_struct != NULL)
	{
	    selection = forminfo->Render->Input->input_object_4_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Input->input_object_4_struct, KVF_FILE_DEF,
	                    &(forminfo->Render->Input->input_object_4));
	}
	else
	    _xvf_init_Master_warning("input selection 'input_object_4' on pane 'Input' of subform 'Render'");

	forminfo->Render->Input->input_object_4_selected = FALSE;
	forminfo->Render->Input->input_object_4_optsel = FALSE;

	forminfo->Render->Input->input_object_5_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Input.input_object_5");
	if (forminfo->Render->Input->input_object_5_struct != NULL)
	{
	    selection = forminfo->Render->Input->input_object_5_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Input->input_object_5_struct, KVF_FILE_DEF,
	                    &(forminfo->Render->Input->input_object_5));
	}
	else
	    _xvf_init_Master_warning("input selection 'input_object_5' on pane 'Input' of subform 'Render'");

	forminfo->Render->Input->input_object_5_selected = FALSE;
	forminfo->Render->Input->input_object_5_optsel = FALSE;

	forminfo->Render->Input->input_object_6_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Input.input_object_6");
	if (forminfo->Render->Input->input_object_6_struct != NULL)
	{
	    selection = forminfo->Render->Input->input_object_6_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Input->input_object_6_struct, KVF_FILE_DEF,
	                    &(forminfo->Render->Input->input_object_6));
	}
	else
	    _xvf_init_Master_warning("input selection 'input_object_6' on pane 'Input' of subform 'Render'");

	forminfo->Render->Input->input_object_6_selected = FALSE;
	forminfo->Render->Input->input_object_6_optsel = FALSE;

	forminfo->Render->Input->input_object_7_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Input.input_object_7");
	if (forminfo->Render->Input->input_object_7_struct != NULL)
	{
	    selection = forminfo->Render->Input->input_object_7_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Input->input_object_7_struct, KVF_FILE_DEF,
	                    &(forminfo->Render->Input->input_object_7));
	}
	else
	    _xvf_init_Master_warning("input selection 'input_object_7' on pane 'Input' of subform 'Render'");

	forminfo->Render->Input->input_object_7_selected = FALSE;
	forminfo->Render->Input->input_object_7_optsel = FALSE;


	/*
	 * initializing 'Master->Render->Output' Pane Info structure
	 */

	forminfo->Render->Output = (Render_Output *)
				kcalloc((size_t)1, sizeof(Render_Output));
	forminfo->Render->Output_struct =
	    kvf_create_struct_from_panename(subform, "Master.Render.Output");
	if (forminfo->Render->Output_struct == NULL)
	{
	    _xvf_init_Master_error("pane 'Output' on subform 'Render'", "Render_Output");
	    return(FALSE);
	}
	pane = forminfo->Render->Output_struct->Controlptr;
	forminfo->Render->Output->output_image_fname_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Output.output_image_fname");
	if (forminfo->Render->Output->output_image_fname_struct != NULL)
	{
	    selection = forminfo->Render->Output->output_image_fname_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Output->output_image_fname_struct, KVF_FILE_DEF,
	                    &(forminfo->Render->Output->output_image_fname));
	}
	else
	    _xvf_init_Master_warning("output selection 'output_image_fname' on pane 'Output' of subform 'Render'");

	forminfo->Render->Output->output_image_fname_selected = FALSE;
	forminfo->Render->Output->output_image_fname_optsel = FALSE;

	forminfo->Render->Output->oxfrm_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Output.oxfrm");
	if (forminfo->Render->Output->oxfrm_struct != NULL)
	{
	    selection = forminfo->Render->Output->oxfrm_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Output->oxfrm_struct, KVF_FILE_DEF,
	                    &(forminfo->Render->Output->oxfrm));
	}
	else
	    _xvf_init_Master_warning("output selection 'oxfrm' on pane 'Output' of subform 'Render'");

	forminfo->Render->Output->oxfrm_selected = FALSE;
	forminfo->Render->Output->oxfrm_optsel = FALSE;

	forminfo->Render->Output->oscene_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Output.oscene");
	if (forminfo->Render->Output->oscene_struct != NULL)
	{
	    selection = forminfo->Render->Output->oscene_struct->Selptr;
	    kvf_get_attribute(forminfo->Render->Output->oscene_struct, KVF_FILE_DEF,
	                    &(forminfo->Render->Output->oscene));
	}
	else
	    _xvf_init_Master_warning("output selection 'oscene' on pane 'Output' of subform 'Render'");

	forminfo->Render->Output->oscene_selected = FALSE;
	forminfo->Render->Output->oscene_optsel = TRUE;

	forminfo->Render->Output->write_scene_file_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "Master.Render.Output.write_scene_file");
	if (forminfo->Render->Output->write_scene_file_struct == NULL)
	    _xvf_init_Master_warning("pane action button 'write_scene_file' on pane 'Output' of subform 'Render'");
	forminfo->Render->Output->write_scene_file = FALSE;
	selection = forminfo->Render->Output->write_scene_file_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	return(TRUE);

}

/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_init_Master_warning
| 
|       Purpose: Prints warning when code is out of sync with UIS file
| 
|         Input: string - item that cannot be found
| 
|        Output: none
|    Written By: conductor
|          Date: July 1, 1997
| Modifications:  
| 
------------------------------------------------------------*/
static void _xvf_init_Master_warning(
    char *string)
{
	kfprintf(kstderr, "Warning: cannot find %s.\n", string);
	kfprintf(kstderr, "         UIS file out of sync with code\n");
	kfprintf(kstderr, "         generated by conductor in 'form_init.c'.\n");
}


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_init_Master_error
| 
|       Purpose: Prints error when code is irreconcilable with UIS file
| 
|         Input: string1 - item that cannot be found
|                string2 - name of FormInfo structure
| 
|        Output: none
|    Written By: conductor
|          Date: July 1, 1997
| Modifications:  
| 
------------------------------------------------------------*/
static void _xvf_init_Master_error(
    char *string1,
    char *string2)
{
	kfprintf(kstderr, "ERROR: cannot find %s.\n", string1);
	kfprintf(kstderr, "         UIS file out of sync with code\n");
	kfprintf(kstderr, "         generated by conductor in 'form_init.c'.\n");
	kfprintf(kstderr, "         Unable to initialize '%s' structure as defined in form_info.h; Aborting.\n", string2);
}


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_free_Master
| 
|       Purpose: Frees the gui_info_struct
| 
|         Input: forminfo - ptr to _xvf_free_Master_gui_struct
| 
|        Output: none
|    Written By: conductor
|          Date: July 1, 1997
| Modifications:  
| 
------------------------------------------------------------*/
void _xvf_free_Master(
    gui_info_struct *forminfo)
{
	if (forminfo->Render->license_struct != NULL)
	    kfree_and_NULL(forminfo->Render->license_struct);
	if (forminfo->Render->help_struct != NULL)
	    kfree_and_NULL(forminfo->Render->help_struct);
	if (forminfo->Render->quit_struct != NULL)
	    kfree_and_NULL(forminfo->Render->quit_struct);
	if (forminfo->Render->refresh_struct != NULL)
	    kfree_and_NULL(forminfo->Render->refresh_struct);
	if (forminfo->Render->b0_struct != NULL)
	    kfree_and_NULL(forminfo->Render->b0_struct);
	if (forminfo->Render->Objects->current_object_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->current_object_struct);
	kfree_and_NULL(forminfo->Render->Objects->current_object_label);
	if (forminfo->Render->Objects->object_shader_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->object_shader_struct);
	kfree_and_NULL(forminfo->Render->Objects->object_shader_label);
	if (forminfo->Render->Objects->post_obj_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->post_obj_struct);
	kfree_and_NULL(forminfo->Render->Objects->post_obj_label);
	if (forminfo->Render->Objects->obj_red_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->obj_red_struct);
	if (forminfo->Render->Objects->obj_green_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->obj_green_struct);
	if (forminfo->Render->Objects->obj_blue_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->obj_blue_struct);
	if (forminfo->Render->Objects->obj_alpha_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->obj_alpha_struct);
	if (forminfo->Render->Objects->obj_ambient_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->obj_ambient_struct);
	if (forminfo->Render->Objects->obj_diffuse_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->obj_diffuse_struct);
	if (forminfo->Render->Objects->obj_spec_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->obj_spec_struct);
	if (forminfo->Render->Objects->obj_spec_exp_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->obj_spec_exp_struct);
	if (forminfo->Render->Objects->obj_spec_metal_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->obj_spec_metal_struct);
	if (forminfo->Render->Objects->object_info_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->object_info_struct);
	if (forminfo->Render->Objects->action1_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->action1_struct);
	if (forminfo->Render->Objects->object_properties_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects->object_properties_struct);
	kfree_and_NULL(forminfo->Render->Objects);
	if (forminfo->Render->Objects_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Objects_struct);

	if (forminfo->Render->Cameras->projection_cycle_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->projection_cycle_struct);
	kfree_and_NULL(forminfo->Render->Cameras->projection_cycle_label);
	if (forminfo->Render->Cameras->eye_x_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->eye_x_struct);
	if (forminfo->Render->Cameras->up_x_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->up_x_struct);
	if (forminfo->Render->Cameras->at_x_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->at_x_struct);
	if (forminfo->Render->Cameras->eye_y_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->eye_y_struct);
	if (forminfo->Render->Cameras->up_y_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->up_y_struct);
	if (forminfo->Render->Cameras->at_y_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->at_y_struct);
	if (forminfo->Render->Cameras->eye_z_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->eye_z_struct);
	if (forminfo->Render->Cameras->up_z_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->up_z_struct);
	if (forminfo->Render->Cameras->at_z_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->at_z_struct);
	if (forminfo->Render->Cameras->hither_clip_form_loc_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->hither_clip_form_loc_struct);
	if (forminfo->Render->Cameras->yon_clip_form_loc_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->yon_clip_form_loc_struct);
	if (forminfo->Render->Cameras->back_clip_form_on_off_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->back_clip_form_on_off_struct);
	kfree_and_NULL(forminfo->Render->Cameras->back_clip_form_on_off_label);
	if (forminfo->Render->Cameras->fov_input_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->fov_input_struct);
	if (forminfo->Render->Cameras->aspect_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->aspect_struct);
	if (forminfo->Render->Cameras->normalize_cam_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->normalize_cam_struct);
	if (forminfo->Render->Cameras->reset_camera_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->reset_camera_struct);
	if (forminfo->Render->Cameras->camera_type_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->camera_type_struct);
	kfree_and_NULL(forminfo->Render->Cameras->camera_type_label);
	if (forminfo->Render->Cameras->eye_separation_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->eye_separation_struct);
	if (forminfo->Render->Cameras->focal_length_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->focal_length_struct);
	if (forminfo->Render->Cameras->lr_flip_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->lr_flip_struct);
	kfree_and_NULL(forminfo->Render->Cameras->lr_flip_label);
	if (forminfo->Render->Cameras->multisample_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->multisample_struct);
	kfree_and_NULL(forminfo->Render->Cameras->multisample_label);
	if (forminfo->Render->Cameras->do_vof_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->do_vof_struct);
	kfree_and_NULL(forminfo->Render->Cameras->do_vof_label);
	if (forminfo->Render->Cameras->normalize_camera_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->normalize_camera_struct);
	if (forminfo->Render->Cameras->blank1_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->blank1_struct);
	if (forminfo->Render->Cameras->blank2_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->blank2_struct);
	if (forminfo->Render->Cameras->blank3_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->blank3_struct);
	if (forminfo->Render->Cameras->blank4_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras->blank4_struct);
	kfree_and_NULL(forminfo->Render->Cameras);
	if (forminfo->Render->Cameras_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Cameras_struct);

	if (forminfo->Render->Lights->light_list_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Lights->light_list_struct);
	kfree_and_NULL(forminfo->Render->Lights->light_list_label);
	if (forminfo->Render->Lights->light_onoff_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Lights->light_onoff_struct);
	kfree_and_NULL(forminfo->Render->Lights->light_onoff_label);
	if (forminfo->Render->Lights->light_type_list_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Lights->light_type_list_struct);
	kfree_and_NULL(forminfo->Render->Lights->light_type_list_label);
	kfree_and_NULL(forminfo->Render->Lights->light_position_string);
	if (forminfo->Render->Lights->light_position_string_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Lights->light_position_string_struct);
	kfree_and_NULL(forminfo->Render->Lights->light_direction_string);
	if (forminfo->Render->Lights->light_direction_string_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Lights->light_direction_string_struct);
	kfree_and_NULL(forminfo->Render->Lights->current_light_color_string);
	if (forminfo->Render->Lights->current_light_color_string_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Lights->current_light_color_string_struct);
	kfree_and_NULL(forminfo->Render->Lights->ambient_light_color_string);
	if (forminfo->Render->Lights->ambient_light_color_string_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Lights->ambient_light_color_string_struct);
	kfree_and_NULL(forminfo->Render->Lights);
	if (forminfo->Render->Lights_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Lights_struct);

	if (forminfo->Render->Imaging->image_width_struct != NULL)
	     kfree_and_NULL(forminfo->Render->Imaging->image_width_struct);
	if (forminfo->Render->Imaging->image_height_struct != NULL)
	     kfree_and_NULL(forminfo->Render->Imaging->image_height_struct);
	if (forminfo->Render->Imaging->image_display_choice_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Imaging->image_display_choice_struct);
	kfree_and_NULL(forminfo->Render->Imaging->image_display_choice_label);
	if (forminfo->Render->Imaging->red_background_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Imaging->red_background_struct);
	if (forminfo->Render->Imaging->green_background_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Imaging->green_background_struct);
	if (forminfo->Render->Imaging->blue_background_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Imaging->blue_background_struct);
	if (forminfo->Render->Imaging->renderer_choice_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Imaging->renderer_choice_struct);
	kfree_and_NULL(forminfo->Render->Imaging->renderer_choice_label);
	if (forminfo->Render->Imaging->invoke_renderer_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Imaging->invoke_renderer_struct);
	kfree_and_NULL(forminfo->Render->Imaging->invoke_renderer_label);
	if (forminfo->Render->Imaging->transparency_choice_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Imaging->transparency_choice_struct);
	kfree_and_NULL(forminfo->Render->Imaging->transparency_choice_label);
	if (forminfo->Render->Imaging->texture_method_2d_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Imaging->texture_method_2d_struct);
	kfree_and_NULL(forminfo->Render->Imaging->texture_method_2d_label);
	if (forminfo->Render->Imaging->antialias_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Imaging->antialias_struct);
	kfree_and_NULL(forminfo->Render->Imaging->antialias_label);
	if (forminfo->Render->Imaging->render_controls_label_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Imaging->render_controls_label_struct);
	if (forminfo->Render->Imaging->blank1_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Imaging->blank1_struct);
	if (forminfo->Render->Imaging->ogl_sphere_subdivisions_struct != NULL)
	     kfree_and_NULL(forminfo->Render->Imaging->ogl_sphere_subdivisions_struct);
	kfree_and_NULL(forminfo->Render->Imaging);
	if (forminfo->Render->Imaging_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Imaging_struct);

	if (forminfo->Render->Transformations->x_rotate_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Transformations->x_rotate_struct);
	if (forminfo->Render->Transformations->y_rotate_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Transformations->y_rotate_struct);
	if (forminfo->Render->Transformations->z_rotate_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Transformations->z_rotate_struct);
	kfree_and_NULL(forminfo->Render->Transformations->center_point_string);
	if (forminfo->Render->Transformations->center_point_string_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Transformations->center_point_string_struct);
	kfree_and_NULL(forminfo->Render->Transformations->translate_vector_string);
	if (forminfo->Render->Transformations->translate_vector_string_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Transformations->translate_vector_string_struct);
	kfree_and_NULL(forminfo->Render->Transformations->xyz_scale_string);
	if (forminfo->Render->Transformations->xyz_scale_string_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Transformations->xyz_scale_string_struct);
	if (forminfo->Render->Transformations->transformation_mode_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Transformations->transformation_mode_struct);
	kfree_and_NULL(forminfo->Render->Transformations->transformation_mode_label);
	kfree_and_NULL(forminfo->Render->Transformations);
	if (forminfo->Render->Transformations_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Transformations_struct);

	kfree_and_NULL(forminfo->Render->Input->input_object_1);
	if (forminfo->Render->Input->input_object_1_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Input->input_object_1_struct);
	kfree_and_NULL(forminfo->Render->Input->input_object_2);
	if (forminfo->Render->Input->input_object_2_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Input->input_object_2_struct);
	kfree_and_NULL(forminfo->Render->Input->input_object_3);
	if (forminfo->Render->Input->input_object_3_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Input->input_object_3_struct);
	kfree_and_NULL(forminfo->Render->Input->input_object_4);
	if (forminfo->Render->Input->input_object_4_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Input->input_object_4_struct);
	kfree_and_NULL(forminfo->Render->Input->input_object_5);
	if (forminfo->Render->Input->input_object_5_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Input->input_object_5_struct);
	kfree_and_NULL(forminfo->Render->Input->input_object_6);
	if (forminfo->Render->Input->input_object_6_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Input->input_object_6_struct);
	kfree_and_NULL(forminfo->Render->Input->input_object_7);
	if (forminfo->Render->Input->input_object_7_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Input->input_object_7_struct);
	kfree_and_NULL(forminfo->Render->Input);
	if (forminfo->Render->Input_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Input_struct);

	kfree_and_NULL(forminfo->Render->Output->output_image_fname);
	if (forminfo->Render->Output->output_image_fname_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Output->output_image_fname_struct);
	kfree_and_NULL(forminfo->Render->Output->oxfrm);
	if (forminfo->Render->Output->oxfrm_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Output->oxfrm_struct);
	kfree_and_NULL(forminfo->Render->Output->oscene);
	if (forminfo->Render->Output->oscene_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Output->oscene_struct);
	if (forminfo->Render->Output->write_scene_file_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Output->write_scene_file_struct);
	kfree_and_NULL(forminfo->Render->Output);
	if (forminfo->Render->Output_struct != NULL)
	    kfree_and_NULL(forminfo->Render->Output_struct);


	if (forminfo->Render_struct != NULL)
	    kfree_and_NULL(forminfo->Render_struct);
	kfree_and_NULL(forminfo->Render);
	kfree_and_NULL(forminfo);
}
