
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 

#ifndef __imaging_h_
#define __imaging_h_

#ifdef __GL_GL_H__
#define PROTO(s) s
#else
#include "xvinclude.h"
#include "xvisual/xvisual.h"
#endif

#define MAX_RENDERER_NAME_STR_LENGTH 64
#define SOFTWARE_RENDERER 0
#define NATIVE_RENDERER 1

/* these define's correspond to the strings in render.form */
#define RENDER_DEFERRED 0
#define RENDER_IMMEDIATE 1

#define SINGLE_BUFFERED 0
#define DOUBLE_BUFFERED 1

/**
  * this struct hold info about the capabilities on the output side of
  * a renderer.
**/

typedef struct screen_stuff
{
    Display *display;
    Window window;
    int screen;
} screen_stuff;

typedef struct renderer_struct
{
    int ndisplay_options;
    int selected_display_option; /* 14 march 95 - at some point we may want
				  to have per-renderer display choices, but
				  this seems kinda weird right now.  at the
				  moment, all renderers will dispaly the same
				  way, as much as is possible */
    int display_options[8];
    char *display_options_txt[8];
    int renderer_active;
    char renderer_name[MAX_RENDERER_NAME_STR_LENGTH];
    int is_hardware_renderer;	/* boolean to indicate if this is a hardware
				 or software renderer */

    int stereo_capable;

    /* routines which build and tear down the renderer-specific
       display mechanism. */
    int (*init_output_func)(int, int, int,int, int, int, kstring);
    int (*shutdown_output_func)(void);

    /* this gets called when the user resizes the display window */
    int (*window_resize_func)(int,int);

    /* current display w,h */
    int width,height;

    /* renderer invocation function */
    int (*render_func)(void);

    /* renderer-specific camera function */
    int (*set_camera_func)();

    /* refresh action func */
    int (*refresh_func)();
    

} renderer_struct;

#define s332_DISPLAY 0
#define s24_DISPLAY 1
#define sQUANT_DISPLAY 2
#define sDITHER_DISPLAY 3
#define sNO_DISPLAY 4

static char s332_STRING[]={"3-3-2 TrueColor (8bit display)"};
static char s24BIT_STRING[]={"24-Bit TrueColor (24bit display)"};
static char sQUANT_STRING[]={"Color Quantized Image (8bit display)"};
static char sDITHER_STRING[]={"Dithered Image (8bit display)"};
static char sNODISPLAY_STRING[]={"No Display"};

#define DEFAULT_BACKGROUND_RED 0.25
#define DEFAULT_BACKGROUND_GREEN 0.25
#define DEFAULT_BACKGROUND_BLUE 0.4

int set_background_color PROTO((float, float, float));
int get_background_color PROTO((float *, float *, float *));

int clear_framebuffer PROTO((void));

int read_dot PROTO((int,int,float *,float *,float *));
int write_dot PROTO((int, int, float, float, float));

int get_image_display_choice PROTO((int *));
int set_image_display_choice PROTO((int));

int get_display_depth PROTO((int *));
int set_display_depth PROTO((int));

int change_image_width PROTO((int));
int change_image_height PROTO((int));

int repaint_imaging_pane PROTO((void));

#ifdef __GL_GL_H__
#else
void render_create_gc PROTO((xvobject, kaddr, XEvent *, int *));
void render_handler PROTO((xvobject , kaddr,XEvent *, int  *));
#endif

int have_graphics_hardware PROTO((renderer_struct *,screen_stuff *));
/** we really should have a machine.h..... **/
int find_gfx_hardware PROTO((char **,int, char *, char *, Display *, int));
void identify_native_renderer PROTO((void));
void identify_software_renderer PROTO((void));

int get_all_renderers_info PROTO((int *n,renderer_struct ***r));

int is_active_renderer_stereo_capable PROTO(());
int set_active_renderer PROTO((int));
int get_active_renderer PROTO((void));

int set_requested_renderer PROTO((int));
int get_requested_renderer PROTO((void));

void renderer_init_func PROTO((void));
void renderer_shutdown_func PROTO((void));
void renderer_invoke_func PROTO((void));
void renderer_resize_func PROTO((int,int));
void renderer_refresh_func PROTO((void));

void set_render_deferred_mode PROTO((int mode));
int get_render_deferred_mode PROTO(());
int get_sphere_subdivisions PROTO(());
int set_sphere_subdivisions PROTO((int new1));


#endif
