/*
 * Khoros: $Id: init.c,v 1.1 1997/04/19 00:15:06 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: init.c,v 1.1 1997/04/19 00:15:06 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>             _static_routines()
   >>>>  Public:
   >>>>             public_routines()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "render.h"
#include "init.h"
#include "imaging.h"


/*-----------------------------------------------------------
|
|  Routine Name: function_name - short description of function
|
|       Purpose: This should be a complete description that anyone
|                could understand;  it should have acceptable grammar
|                and correct spelling.
|
|         Input: argument1 - explanation
|                argument2 - explanation
|                argument3 - explanation
|
|        Output: argument4 - explanation
|                argument5 - explanation
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By:
|          Date: 
| Modifications:
|
------------------------------------------------------------*/


int
init_from_clui (clui_info_struct *clui_info)
{
    /* initialize textureing - must be done before objects are read in.*/
    init_texturing();
    
    /**
     * deal with opening input object(s), building the internal
     * render tree, adding file detects.
     **/
    init_object_list(clui_info);

    set_image_dims(clui_info->wsize_int,clui_info->hsize_int);
    set_image_display_choice(clui_info->view_list);
        
    
    /**
     * initialize the output mechanism.
     **/
    set_do_named_output(clui_info->o_flag);
    if (clui_info->o_flag)
	set_output_transport_name(clui_info->o_file);

#ifdef _USE_GFX_HARDWARE
    if (clui_info->use_hardware_logic)
	set_requested_renderer(NATIVE_RENDERER);

#if 0
	select_hardware_renderer(); /* hack for this version... 
					   in imaging.c */
#endif
    
    else
#endif
	set_requested_renderer(SOFTWARE_RENDERER);
    
    init_camera(clui_info);
    
    /* initialize the output vrxfrm field if necessary */
    if (clui_info->oxfrm_file)
	init_xform_output(clui_info->oxfrm_file);

    if (clui_info->iscene_flag)
	set_input_scene_filename(clui_info->iscene_file);

    if (clui_info->oscene_flag)
	set_output_scene_filename(clui_info->oscene_file);
    return(CHILL);
}

int
internal_init()
{
    int i;
    

    identify_software_renderer();
    identify_native_renderer();

    i = get_requested_renderer();
#ifdef _USE_GFX_HARDWARE
    if (i == NATIVE_RENDERER)
	select_hardware_renderer(); /* hack for this version... 
					   in imaging.c */
    else
#endif
	set_active_renderer(SOFTWARE_RENDERER);

    /**
     * initialize the lights database
     **/
    init_lights();

    /* read the scene file, if present. */
    {
	char s[256];
	s[0] = 0;
	get_input_scene_filename(s);
	if (s[0] != '\0')
	    read_scene_file();

    }

    {
	int n;
	n = get_current_vof();

	/* we're going to assume that the "default" vof is index 0.
	 this would correspond to 1280x1024@72hz on the SGI.  if
	 we detect a current vof of zero, we're not going to reset
	 the VOF - we assume that the machine is in this state when
	 we get here, and that we need to change only if it's non-zero.*/

	if (n > 0)
	    change_vof(n);
    }
    
    renderer_init_func();
    return(CHILL);
}

int
init_gui_from_internal (gui_info_struct *gui_info)
{
    /**
     * first, update the "objects" pane.  the init function
     * is different than the "update" function.  the transformations
     * call must precede the object list call. 
     **/
    init_transformations(gui_info->Render->Transformations);
    init_object_list_GUI(gui_info->Render->Objects);  
    init_input_pane_GUI(clui_info,gui_info->Render->Input);
    
    init_imaging_pane(gui_info->Render->Imaging);

    update_cameras_pane_GUI(gui_info->Render->Cameras);

    init_lights_pane(gui_info->Render->Lights);

    init_output_pane_GUI(gui_info->Render->Output);
    return(CHILL);
}
