
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 


#ifndef __light_h_
#define __light_h_

#define LIGHT_ON 1
#define LIGHT_OFF 0

#define NLIGHT_TYPES 4

#define DIRECTIONAL_LIGHT 0
#define BIDIRECTIONAL_LIGHT 1
#define POINT_LIGHT 2
#define AMBIENT_LIGHT 3

static char *light_type_strings[] =
{
    "Directional",
    "Bidirectional",
    "Point Light",
    "Ambient"
};

/**
  * NOTE: render.form has a line which determines the number of
  * lights.  the UIS list of lights will be reconscructed based
  * upon the following #define.
**/
#define MAX_LIGHTS 3

typedef struct light_struct
{
    int light_type;        /* one of the supported types. */
    vertex_3d position;    /* required for point and area sources. */
    vertex_3d direction;   /* required for directional lights */
    float rgb[3];          /* light color */
    int light_on;          /* toggle on/off */
} light_struct;

void init_lights PROTO((void));

int set_current_light PROTO((int));
int get_current_light PROTO((int *));

int set_light_color PROTO((int, float, float, float));
int get_light_color PROTO((int, float *, float *, float *));

int set_light_onoff PROTO((int, int));
int get_light_onoff PROTO((int, int *));

int set_light_type PROTO((int, int));
int get_light_type PROTO((int, int *));

int set_light_direction PROTO((int, vertex_3d *));
int get_light_direction PROTO((int, vertex_3d *));

int set_light_position PROTO((int, vertex_3d *));
int get_light_position PROTO((int, vertex_3d *));

int get_ambient_light_color PROTO((vertex_3d *));
int set_ambient_light_color PROTO((vertex_3d *));

int get_num_lights PROTO((int *));
int set_num_lights PROTO((int));

#endif
