
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 *
 * september 1996 - add stuff to support video-output-format changes in
 *        a somewhat organized manner.  each system is different, so all
 *        the different machines' code is corralled here.  yee-haw!
 */
 


/*
 * Khoros: $Id: machine.c,v 1.1.2.1 1997/05/02 16:47:11 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: machine.c,v 1.1.2.1 1997/05/02 16:47:11 mirage Exp $";
#endif

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  this file contains arch specific code.
   >>>>
   >>>>
   >>>>
   >>>>
   >>>>  Static:
   >>>>             _static_routines()
   >>>>  Public:
   >>>>             public_routines()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "render.h"
#include "imaging.h"


static char NATIVE_RENDERER_NAMES[] =
#ifdef __OGL_H
{"OpenGL"};
#else 
{"none"};
#endif

/* misc include files */
#ifdef _USE_GFX_HARDWARE

#ifdef __OGL_H
/* none needed here */
#endif

#endif


int
find_gfx_hardware (char **exts,
		   int nexts,
		   char *vendor,
		   char *native_renderer_name,
		   Display *display,
		   int screen)
{
    /* the purpose of this routine is to determine if there's any
     graphics hardware out there that we can support. */
    int status;

    status = FALSE;
#ifdef _USE_GFX_HARDWARE

#ifdef __OGL_H
    {
	extern int ogl_min_config();
	
	status = ogl_min_config(); /* in ogl.c */
	if (status)
	{
	    kstrcpy(native_renderer_name,NATIVE_RENDERER_NAMES);
	    status = TRUE;
	}
    }
#else
    /* IRIX GL code nuked 4/30/96 */
    /* solaris  XGL code nuked 3/12/96 */
#endif
	
#endif
    return(status);  /* don't know how to do anything else right now. */

}


/*
 * the code below is machine specific for video output formats.  the
 * underlying assumption is that an actual VOF change is done by
 * issuing a command of the form:
 *     system("<change_vof_command_for_this_machine> some_vof_string parms");
 * and that the specific command and parameters will vary from machine to
 * machine.
 *
 * routines:
 *
 * init_vof(void) - builds a table of valid VOF strings for this machine.
 *        
 */

static int num_vofs=0;
static int svof = 0;

#ifdef KOPSYS_IRIX
/**
  * the following formats and system calls run on IR systems.
**/
#define NUM_VOFS 5
static char *vof_strings[NUM_VOFS]=
{
    "1280x1024_72.vfo",
    "1024x768_96s.vfo",
    "640x480_120s.vfo",
    "1280x1024_120s.vfo",
    "Puff-the-Magic-Dragon"
};

static char vof_change_incantation[]={"/usr/gfx/ircombine -destination active -channel 0 format="};

#if 0
/* these vof's work on non-IR systems. */
#define NUM_VOFS 2
static char *vof_strings[NUM_VOFS]=
{
    "1280x1024_72",
    "1024x768_96s"
};

/* use this one for non-IR SGI's.  make sure that the VOF's you request
  are actually present.  also, i've seen instances where the requested VOF's
  are present and work, but the monitor can't handle the format, so the
  monitor just shuts off. */
static char vof_change_incantation[]={"/usr/gfx/setmon -n "};

#endif


char
**get_vof_strings()
{
    return(vof_strings);
}

int
change_vof(int vof_index)
{
    char t[256];

    ksprintf(t,"%s%s",vof_change_incantation,vof_strings[vof_index]);
    system(t);
}

int
init_vof()
{
}
#else

#define NUM_VOFS 0

int
change_vof(int vof_index)
{
    /* not supported */
    return(WHACKED);
}

int
init_vof()
{
    return(WHACKED);
}

char
**get_vof_strings()
{
    return(NULL);
}
#endif


int
get_num_vofs(void)
{
    return(NUM_VOFS);
}

int
set_current_vof(int new1)
{
    /* note: we don't actually change video output formats here. */
    if ( (new1 < 0) || (new1 >= NUM_VOFS))
	return(WHACKED);
    
    svof = new1;
    return(CHILL);
}

int
get_current_vof()
{
    return(svof);
}
