
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 

/***********************************************************************

  Tue Mar 18 10:50:31 PST 1997 - folded in hacks made on DC machine
  which implement OpenGL display lists, but for disjoint triangle
  primitives ONLY.  the intent is to get some performance..the largest
  objects we have are disjoint triangle objects (eg, isosurfaces).
  this is not a general-purpose solution but works for now.  

  Wed Nov 13 09:53:30 PST 1996 - added support for 3d texture mapping
  for oct meshes.  presently, the 3d texture extension to OpenGL is
  supported only on SGI boxes.  ifdef's were added so that this code
  will compile using the Mesa OpenGL emulator.
  
  outstanding issues:

  Mon Mar 18 15:00:06 PST 1996 - opengl's idea of the illumination
  equation is really poorly implemented...all of the code herein has
  to explicitly attentuate vertex/face rgb by each of the diffuse,
  ambient and specular reflectance coefficients.  open gl should have
  provided for these single scalar values as parameters explicitly,
  thereby saving a lot of trouble.  in this rev of the code, the
  malloc's and multiplies are done within the primitive routines.
  at a later date, we will want to optimize.

  Thu May  2 10:52:40 PDT 1996 - transparency doesn't work quite right...
  multiple overlapping transparent surfaces will not be rendered properly.
  the fix will be to sort all transparent objects back-to-front and then
  render them in that order.  till then, the bug will appear when one
  transparent surface is drawn underneath, ie, further away from the viewer,
  one that is closer to the viewer.  this bug WILL NOT BE FIXED for 2.1.
  note that depth sorting is done for octmeshes.


***********************************************************************/


/*
 * Khoros: $Id: ogl.c,v 1.1.2.2 1997/05/02 22:18:23 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: ogl.c,v 1.1.2.2 1997/05/02 22:18:23 mirage Exp $";
#endif

#if defined(__OGL_H) && defined(_USE_GFX_HARDWARE)

#include "render.h"
#include "camera.h"
#include "lights.h"
#include "shader.h"
#include "texture.h"
#include "render_mouse.h"
#include "vrxfrm.h"
#include <GL/glx.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <unistd.h>

static Display *dpy;
static XVisualInfo *vi;

/* for valid attributes, see the glXChooseVisual man page.
 by default, we try to open a truecolor RGB, doublebuffered visual.
 preliminary tests showed that, when running on an SGI and displaying
 onto a sun-ultra running solaris 5.5 with a prelim release of GLX,
 only a double-buffered visual was available. test conducted march 96 */
static int monoattributeList[]={GLX_RGBA,GLX_DOUBLEBUFFER,
				GLX_DEPTH_SIZE,1,None};
static int stereoattributeList[]={GLX_RGBA,GLX_DOUBLEBUFFER,
				      GLX_STEREO,
				      GLX_DEPTH_SIZE,1,None};
static GLXContext cx;
static Colormap cmap;
static XSetWindowAttributes swa;


static char ogl24BIT_STRING[]={"24-Bit RGB OpenGL"};

int ogl_init_func PROTO((int, int,int,int,int,int,char *));
int ogl_clear_window PROTO((void));
int ogl_shutdown_func PROTO((void));
int ogl_camera_func PROTO((void));
int ogl_render_func PROTO((GLenum,int));
int ogl_render_master PROTO((void));
int ogl_window_resize_func PROTO((int,int));
static int ogl_surfprop_func PROTO((renderobj *r));

int ogl_omesh_texture PROTO((render_prim *, renderobj *, int, matrix4x4 *));

static void ogl_render_2d_stuff PROTO((renderobj *, matrix4x4 *, int, int, int, void (*)(const GLfloat *)));
static void ogl_render_opaque_subtree PROTO((renderobj *, matrix4x4 *, int, int, int,  void (*)(const GLfloat *)));
static void ogl_render_transparent_subtree PROTO((renderobj *, matrix4x4 *, int, int, int, void (*)(const GLfloat *)));

static void ogl_build_and_init_textures PROTO((void));
static int build_ogl_texture PROTO((render_texture_table *rmt));
static int map_to_ogl_renderprims PROTO((int ktype));
static int ogl_lights_func PROTO((void));
static int tstrip_to_disjoint_v PROTO((float *s,float *d,int n, int size));
static int tstrip_to_disjoint_f PROTO((float *s,float *d,int n,int size));
static int dump_disjoint_triangles_to_ogl PROTO((vertex_3d *v,vertex_3d *norms,float *fcolors,int color_size,float *ftcoords,int tc_size,int nverts, renderobj *,void (*)(const GLfloat *)));
int expand_rgb_to_rgba PROTO((float *,float *,int,float *));

static void ogl_noprim PROTO((render_prim *,int,int,vertex_3d *,vertex_3d *,vertex_3d *, float *,int,int,renderobj *, void (*)(const GLfloat *), matrix4x4 *));

static void ogl_polylin_dis PROTO((render_prim *,int,int,vertex_3d *,vertex_3d *,vertex_3d *,	float *,int,int,renderobj *,void (*)(const GLfloat *), matrix4x4 *));

static void ogl_polylin_con PROTO((render_prim *,int,int,vertex_3d *,vertex_3d *,vertex_3d *,	float *,int,int,renderobj *,void (*)(const GLfloat *), matrix4x4 *));

static void ogl_polytri_dis PROTO((render_prim *,int,int,vertex_3d *,vertex_3d *,vertex_3d *,	float *,int,int,renderobj *,void (*)(const GLfloat *), matrix4x4 *));

static void ogl_polytri_con PROTO((render_prim *,int,int,vertex_3d *,vertex_3d *,vertex_3d *,	float *,int,int,renderobj *,void (*)(const GLfloat *), matrix4x4 *));

static void ogl_qmesh PROTO((render_prim *,int,int,vertex_3d *,vertex_3d *,vertex_3d *, float *,int,int,renderobj *,void (*)(const GLfloat *), matrix4x4 *));

static void ogl_sphere PROTO((render_prim *,int,int,vertex_3d *,vertex_3d *,vertex_3d *, float *,int,int,renderobj *,void (*)(const GLfloat *), matrix4x4 *));

static void ogl_text PROTO((render_prim *,int,int,vertex_3d *,vertex_3d *,vertex_3d *, float *,int,int,renderobj *,void (*)(const GLfloat *), matrix4x4 *));

static void ogl_omesh PROTO((render_prim *,int,int,vertex_3d *,vertex_3d *,vertex_3d *, float *,int,int,renderobj *,void (*)(const GLfloat *), matrix4x4 *));

#if 0

/* the following prim routines need to be written. */

static void ogl_polyhedron PROTO((render_prim *,int,int,vertex_3d *,vertex_3d *,vertex_3d *, float *,int,int,renderobj *));

static void ogl_dirpoint PROTO((render_prim *,int,int,vertex_3d *,vertex_3d *,vertex_3d *, float *,int,int,renderobj *));

#endif

#define NO_PRIM 0
#define POLYLIN_DIS 1
#define POLYLIN_CON 2
#define POLYTRI_DIS 3
#define POLYTRI_CON 4
#define QMESH 5
#define SPHERE 6
#define TEXT 7
#define OMESH 8
#define PHEDRON 9
#define DIRPT 10
static void (*ogl_primfuncs[]) PROTO((render_prim *,
				      int,int,
				      vertex_3d *,vertex_3d *,vertex_3d *,
				      float *,
				      int,int,
				      renderobj *,
				      void (*)(const GLfloat *),
				      matrix4x4 *)) =
{
    ogl_noprim,
    ogl_polylin_dis,
    ogl_polylin_con,
    ogl_polytri_dis,
    ogl_polytri_con,
    ogl_qmesh,
    ogl_sphere,
    ogl_text,
    ogl_omesh,
    ogl_noprim,
    ogl_noprim

};

static void c_no_op PROTO((float *));
static void gl_no_op PROTO((const GLfloat *));

/***
   * a separate texture structure is needed for OpenGL due to the following
   * restrictions imposed by OpenGL:
   *
   * 1. the texture size must be a power of 2,
   * 2. the texture is going to be loaded into the graphics pipe often -
   *    there is no notion of caching a bunch of textures.
** */
typedef struct
{
    int ncomponents;		/* number of channels in image */
    int constrained_dims[3];	/* dimensions of base level texture */
    int mipmap_levels;		/* number of mipmap levels (may be 0) */
    GLenum format;

    unsigned char **the_texture;	/* the texture itself. the mipmaps
					 are stored in the 2nd dimension. the
					 length of the 2nd dimension is
					 mipmap_levels. */
#if 0
    /* not sure what i had in mind with these.  delete them? */
    void (*load_texture_func)(); /* routine to load the texture. */
    void (*free_texture_func)(); /* frees the texture.  */
#endif
    
} ogl_texture_struct;

/**
  * as a substitute for texture caching, the following pointer is used
  * to keep track of the last texture loaded to OpenGl at render time.
  * we'll check  this each time we're about to load a texture, and if
  * the texture is already loaded, we won't load it again.  otherwise,
  * we'll reload it and update the pointer.
 **/

static ogl_texture_struct *currently_loaded_ogl_texture=NULL;
static ogl_texture_struct *malloc_ogl_texture PROTO((void));
static void flush_ogl_textures PROTO((void));
static void free_ogl_texture_struct PROTO((ogl_texture_struct *o));
static void compute_voxel_reorient_matrix PROTO((matrix4x4 *v,matrix4x4 *m,vertex_3d *eye,vertex_3d *at));

/* transport stuff- some duplication of code in output.c used by the
 software renderer.*/
static kobject output_ogl_kimage;
static unsigned char *ogl_pixel_buf=NULL;

static int sphere_subdivisions=6;

int
set_sphere_subdivisions(int new1)
{
    sphere_subdivisions = new1;
    return(CHILL);
}

int
get_sphere_subdivisions()
{
    return(sphere_subdivisions);
}


static int
ogl_check_if_texture_is_loaded(ogl_texture_struct *new1)
{
    /* this routine just checks to see if the named texture is
     the same as the last one loaded. it returns 1 if so, 0 otherwise. */
    
    if (new1 == currently_loaded_ogl_texture)
	return(1);
    else
	return(0);
}

static int
ogl_activate_texture(ogl_texture_struct *new1)
{
    GLenum format;
    
    if (ogl_check_if_texture_is_loaded(new1) == 0) /* not active, gotta load it */
    {
	currently_loaded_ogl_texture = new1;
	
	glPixelStorei(GL_UNPACK_ALIGNMENT,1);
	glTexImage2D(GL_TEXTURE_2D,0,new1->ncomponents,
		     new1->constrained_dims[0],new1->constrained_dims[1],0,
		     new1->format,GL_UNSIGNED_BYTE,
		     (const GLvoid *)(new1->the_texture[0]));

    }
    return(CHILL);
}


int
ogl_min_config()
/* the purpose of this routine is to determine if $DISPLAY is ogl-capable */
{

    int status = 0;
    Window win;
    
    /* get connection */
    dpy = XOpenDisplay(NULL);
    if (dpy == NULL)
	return(0);

    if (!glXQueryExtension(dpy,NULL,NULL))
	return(0);	/* no opengl/glx extensions present on dpy */
    /* get appropriate visual - as an alternate, we could do an
     XQueryExtension() and search for GLX in there. */
    if (get_camera_type() == CAMERA_TYPE_MONOSCOPIC)
	vi = glXChooseVisual(dpy,DefaultScreen(dpy),monoattributeList);
    else			/* assume stereo */
	vi = glXChooseVisual(dpy,DefaultScreen(dpy),stereoattributeList);
    if (vi == NULL)
	return(0);


    cx = glXCreateContext(dpy,vi,0,GL_TRUE);
#if 0
    /* use this for testing purposes */
    cx = NULL;			/* use indirect context */
#endif
    if (cx == NULL)
    {
	/* couldn't create a direct rendering context.  try to create an
	   indirect  rendering context. */
	cx = glXCreateContext(dpy,vi,0,GL_FALSE);
	if (cx == NULL)		/* we're hosed..there's no open gl here. */
	    return(0);
    }
	
/*    cmap = XCreateColormap(dpy,RootWindow(dpy,vi->screen),vi->visual,AllocNone); */

    glXDestroyContext(dpy,cx);
    XCloseDisplay(dpy);		/* don't need it...will reopen later.. */
    return(1);
}


int
init_native_renderer(renderer_struct *r)
{
    r->init_output_func = ogl_init_func;
    r->render_func = ogl_render_master;
    r->shutdown_output_func = ogl_shutdown_func;
    r->window_resize_func = ogl_window_resize_func;
    r->refresh_func = ogl_render_master; /* a refresh is a complete redraw */

    r->set_camera_func = ogl_camera_func;
    
    /* we may want to try to support something other than 24bit
       images at some point.  hmmmm.   12 mar 96 */
    r->ndisplay_options = 1;
    r->selected_display_option = 0;
    r->display_options_txt[0] = ogl24BIT_STRING;
    r->is_hardware_renderer = 1; /* depends on what hardware's
				    out there, huh? */
    {
	/* check to see if we're stereo capable.  */
	
	dpy = XOpenDisplay(NULL);
	if (dpy == NULL)
	    return(0);

	if (!glXQueryExtension(dpy,NULL,NULL))
	    return(0);	/* no opengl/glx extensions present on dpy */
	/* get appropriate visual - as an alternate, we could do an
	   XQueryExtension() and search for GLX in there. */
	
	vi = glXChooseVisual(dpy,DefaultScreen(dpy),stereoattributeList);
	r->stereo_capable =  (vi == NULL) ? FALSE:TRUE;
    }
    return(1);
}

static Display *D;
static int S;
static Window W;
static xvobject xvimage=NULL;

/** debug! **/
static Window dW;
static XEvent event;

static Bool WaitForNotify(Display *d,
			  XEvent *e,
			  char *arg)
{
    return((e->type == MapNotify) && (e->xmap.window == (Window)arg));
}

/** end debug **/


int
ogl_init_func(int width,
	      int height,
	      int xpos,
	      int ypos,
	      int img_disp_choice,
	      int do_named_output,
	      char *name)
{

    char transport_name[512];
    int image_display_choice;
    float fr,fg,fb;
    int i;
    unsigned long valuemask;

    xvimage = xvw_create_image(NULL, "OpenGL-Render");
    D = dpy = xvw_display(xvimage);
    
    /* get connection */
    if (dpy == NULL)
	return(WHACKED);
    XSync(dpy,0);
    
#if 0
    W = xvw_window(xvimage);
    if ((xpos >= 0) && (ypos >= 0))
	XMoveWindow(D,xvw_window(xvw_parent(xvimage)),xpos,ypos);
    XSync(D,0); 
#endif
    /* get screen */
    S = xvw_screennum(xvimage);
    W = xvw_window(xvimage);

    /* check here for stereo.  this could be done better. */
    if (get_camera_type() == CAMERA_TYPE_STEREOSCOPIC)
    {
	vi = glXChooseVisual(dpy,S,stereoattributeList);
	if (vi == NULL)		/* this opengl is not stereo capable.. */
	{
	    /* error recovery consists of trying to grab a mono camera. */
	    vi = glXChooseVisual(dpy,S,monoattributeList);
	    if (vi != NULL)
		set_camera_type(CAMERA_TYPE_MONOSCOPIC);
	}
    }
    else
	vi = glXChooseVisual(dpy,S,monoattributeList);
    if (vi == NULL)
	return(WHACKED);

    xvw_set_attribute(xvimage, XVW_MENUABLE, FALSE);
    xvw_set_attribute(xvimage, XVW_VISUAL, vi->visual);
#if 0
    /* this is untested? */
    xvw_set_attribute(xvimage, XVW_IMAGE_BACKING, TRUE); /* test this.. */
#endif

    /* -- don't want it creatin one of those pan-icon thingies -- */
    xvw_set_attribute(xvimage, XVW_MAXIMUM_WIDTH,  10000);
    xvw_set_attribute(xvimage, XVW_MAXIMUM_HEIGHT, 10000);

    /* -- set (get) the depth of the image -- */
    xvw_set_attributes(xvimage,XVW_WIDTH,width,XVW_HEIGHT,height,NULL);

    cx = glXCreateContext(D,vi,0,GL_TRUE);
    if (cx == NULL)
    {
	/* couldn't create a rendering context.  try to create an indirect
	 rendering context. */
	cx = glXCreateContext(dpy,vi,0,GL_FALSE);
	if (cx == NULL)		/* we're hosed..there's no open gl here. */
	    return(WHACKED);
    }
    /* hope it's a 24bit window...proceed to open the gl */

    xvw_realize(xvw_toplevel(xvimage)); 
    xvw_realize(xvimage); 
    xvw_map(xvimage);
    W = xvw_window(xvimage);

    if ((xpos >= 0) && (ypos >= 0))
	XMoveWindow(D,xvw_window(xvw_parent(xvimage)),xpos,ypos);

    XSync(D,0);


    valuemask = CWBackingStore; 
    kmemset(&swa,0,sizeof(XSetWindowAttributes));
    /*    swa.backing_store = Always; */
    swa.backing_store = NotUseful;
    XChangeWindowAttributes(D,W,valuemask,&swa);

    XSync(D,0);
    
    glXMakeCurrent(D,W,cx);

    /* hack stuff ... diagnostics...check the value of i to see how
     many bits of depth are being used.  this call is unnecessary unless
     you are just curious. */
/*    glXGetConfig(D,vi,GLX_DEPTH_SIZE,&i); */

    /* Need to call this, but how & why? */
    /* XSetWMColormapWindows();*/

    set_up_render_mouse_button_definitions(xvimage,NULL);
    
    XSync(D,0); 
    ogl_clear_window();
    glXSwapBuffers(D,W);
    XSync(D,0);
    ogl_clear_window();
    glXSwapBuffers(D,W);
    

    /* disable removal of backfacing polygons. this should be an option? */
    glDisable(GL_CULL_FACE);
    glMatrixMode(GL_MODELVIEW);
    
    /* turn on the zbuffer */
    glEnable(GL_DEPTH_TEST);

    glEnable(GL_NORMALIZE);
    glFrontFace (GL_CW);

    /* -- build GL style textures from the render textures (if any) -- */
    ogl_build_and_init_textures();

    if (do_named_output)
    {
	ogl_pixel_buf = (unsigned char *)kmalloc(sizeof(unsigned char)*width*height*3);
    }

    glFinish();
    return(CHILL);
}


int
ogl_window_resize_func(int new_width,
		       int new_height)
{
    int w,h,img_display_choice,do_named_flag;
    int xpos,ypos;
    char transport_name[256];
    
    ogl_shutdown_func();	/* shutdown opengl using the old config */

    get_render_image_position(&xpos,&ypos);
    set_image_dims(new_width,new_height);
    get_image_display_choice(&img_display_choice);
    get_output_transport_name(transport_name);
    get_do_named_output(&do_named_flag);

    compute_view_xform();


    return(ogl_init_func(new_width,new_height,xpos,ypos,
			 img_display_choice,do_named_flag,transport_name)); 

}

int
ogl_clear_window(void)
{
    
    float bg[3];
    GLclampf red, green, blue, alpha;
    
    get_background_color(bg,bg+1,bg+2);

    glClearColor(bg[0],bg[1],bg[2],(float)1.0);
    glClearDepth((GLclampd)1.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    /* need to write zbuffer clearing code */
    return(CHILL);
}

int
ogl_shutdown_func(void)
{
    int do_named_transport;
    
    glXDestroyContext(D,cx);
    remove_xvevent_callbacks(xvimage);
    xvw_destroy(xvimage);
    xvimage = NULL;

    /* need to finish this off.... */

    /* flush out the opengl textures... this cleans out the hardware
     texture table in the render_texture_table structure.  it
     would be more efficient to write a warm_shutdown_function whereby
     the textures aren't flushed then re-read whenever we do a
     window resize.  but this works for now, and the delays don't affect
     raw rendering speed. */

    flush_ogl_textures();

    /* kill off the transport */

    get_do_named_output(&do_named_transport);
    if (do_named_transport)
    {
	kfree(ogl_pixel_buf);
/*	kpds_close_object(output_ogl_kimage); */
    }
    return(CHILL);
}

int
ogl_camera_func(void)
{
    vertex_3d l_eye,l_at,l_up,y_ref,eye_at_vector;
    float l_fov,l_hither,l_yon,l_zmin,l_zproj,dot;
    int l_proj;
    double ddot;
    double mag;
    int i;
    int width, height;
    
    float near,far,aspect;
    
    get_image_dims(&width,&height);
    glViewport(0,0,width,height);
    
    get_eye(&l_eye);
    get_at(&l_at);
    get_up(&l_up);
    get_fov(&l_fov);

    get_user_hither(&l_hither); 
    get_user_yon(&l_yon);

    get_projection(&l_proj);

    for (i=0;i<3;i++)
	eye_at_vector.v[i] = l_eye.v[i] - l_at.v[i];

    vertex_mag(&eye_at_vector,&mag);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    near = l_hither * mag;
    far = l_yon * mag;

    if (l_proj == KPROJECTION_PERSPECTIVE)
    {
	aspect = 1.0;		/* for now */
	aspect = (float)width/(float)height;
	gluPerspective((GLdouble)l_fov,
		       (GLdouble)aspect,
		       (GLdouble)near,
		       (GLdouble)far);
	gluLookAt((GLdouble)(l_eye.v[0]),
		  (GLdouble)(l_eye.v[1]),
		  (GLdouble)(l_eye.v[2]),
		  (GLdouble)(l_at.v[0]),
		  (GLdouble)(l_at.v[1]),
		  (GLdouble)(l_at.v[2]),
		  (GLdouble)(l_up.v[0]),
		  (GLdouble)(l_up.v[1]),
		  (GLdouble)(l_up.v[2]));
    }
    else
    {
	double window_width,window_height;

	/* this code doesn't do quite the "right" thing. 23 july 96, wes*/
	window_width = mag* 0.5;
	window_height = mag * /* DEGREES_TO_RADIANS(l_fov) *  */ 0.5;
	glOrtho(-window_width, window_width,
		-window_height,window_height,	(GLdouble)near,(GLdouble)far); 

	gluLookAt((GLdouble)(l_eye.v[0]),
		  (GLdouble)(l_eye.v[1]),
		  (GLdouble)(l_eye.v[2]),
		  (GLdouble)(l_at.v[0]),
		  (GLdouble)(l_at.v[1]),
		  (GLdouble)(l_at.v[2]),
		  (GLdouble)(l_up.v[0]),
		  (GLdouble)(l_up.v[1]),
		  (GLdouble)(l_up.v[2]));
    }
    glMatrixMode(GL_MODELVIEW);

    
/*    glDepthRange((GLclampd)near,(GLclampd)far); */
    glDepthRange((GLclampd)0.0001,(GLclampd)10000.0);
    return(CHILL);
}



int
ogl_render_master(void)
{
    int ctype,lrflip;
    vertex_3d save_eye, lefteye,righteye;
    vertex_3d save_lookat, focalpoint;

    /* the following routine is necessary because there's a "clear to
     black" event in the queue somewhere, and it should be processed
     prior to rendering anything. */
    
    XSync(D,0);

    ctype = get_camera_type();

    if (ctype == CAMERA_TYPE_MONOSCOPIC)
    {
	ogl_render_func(GL_BACK,1);
	glXSwapBuffers(D,W);
#if 0
	glXSwapBuffers(D,dW);
#endif
    }
    else if (ctype == CAMERA_TYPE_STEREOSCOPIC)
    {
	get_eye(&save_eye);
	get_at(&save_lookat);
	
	get_left_eye(&lefteye);
	get_right_eye(&righteye);
	get_focal_point(&focalpoint);
	lrflip = get_leftright_flip();
	
	/* set left eye */
	set_eye(&lefteye);
	set_at(&focalpoint);

	if (lrflip)
	    ogl_render_func(GL_BACK_RIGHT,1);
	else
	    ogl_render_func(GL_BACK_LEFT,1);
	glFlush();

	/* set right eye */
	set_eye(&righteye);
	if (lrflip)
	    ogl_render_func(GL_BACK_LEFT,0);
	else
	    ogl_render_func(GL_BACK_RIGHT,0);
	    

#if 0
	glXSwapBuffers(D,dW);
#endif
	glXSwapBuffers(D,W); 

	set_eye(&save_eye);
	set_at(&save_lookat);
    }

    return(CHILL);
}
			

int
ogl_render_func(GLenum destbuffer,int dump_toggle)
{
    matrix4x4 I;
    matrix4x4 xfrm;
    int w,h;
    int projection;
    renderobj *r;
    int have_transparent_objs;
    int do_named_transport;

    glDrawBuffer(destbuffer);
    
    get_image_dims(&w,&h);
    
    /* initialize the output transport, if present */
    get_do_named_output(&do_named_transport);
    if (do_named_transport && dump_toggle)
    {
	char tname[512];

	get_output_transport_name(tname);
	output_ogl_kimage = kpds_open_object(tname,KOBJ_WRITE);
	/*  -- tell image widget to interpret data as RGB with no alpha -- */
	kcolor_set_attribute(output_ogl_kimage,KCOLOR_COLORSPACE,KRGB);
	kcolor_set_attribute(output_ogl_kimage,KCOLOR_HAS_ALPHA,FALSE);

	/* set up value segment */
	kpds_create_value(output_ogl_kimage);
	kpds_set_attribute(output_ogl_kimage,KPDS_VALUE_SIZE,w,h,1,1,3);
	kpds_set_attribute(output_ogl_kimage,KPDS_VALUE_DATA_TYPE,KUBYTE);
	kpds_set_attribute(output_ogl_kimage,KPDS_ELEMENTS_FIRST,TRUE);

	glPixelStorei(GL_PACK_ALIGNMENT,1);
    }
    
    /* first step in rendering is to clear the window */
    ogl_clear_window();

    /* set up the camera transformation */
    ogl_camera_func();

    /* set up the lights and stuff */
    ogl_lights_func();

    /* render the opaque objects */
    identity_4x4(&xfrm);
    get_projection(&projection);
    r = get_root_object();

    have_transparent_objs = have_any_transparent_objs(r);

    /* push down stack one level - this spot is used for all render
     transforms */
    glMatrixMode(GL_MODELVIEW);
    identity_4x4(&I);

    /* render the opaque objects first */

    ogl_render_opaque_subtree(r,&xfrm,w,h,projection,glVertex3fv);

    /* then render the transparent objects, if any */
    if (have_transparent_objs)
	ogl_render_transparent_subtree(r,&xfrm,w,h,projection,
			      glVertex3fv);

    ogl_render_2d_stuff(r,&xfrm,w,h,projection,glVertex2fv);

    /* final step after rendering everything is to flush out
     all the rendering calls, then do a swapbuffers call. */
/*    glFlush();*/
    glFinish();

    if (dump_toggle)
	dump_xform_output();

    /* dump pixels to the output transport, if present.. */

    get_do_named_output(&do_named_transport);
    if (do_named_transport && dump_toggle)
    {
	int j,offset=0;
	
	/* the following line of code reads the entire framebuffer.  it's
	 flipped in Y, so the loop below reads a row at a time. a desired
	 enhancement is to do a put_data on each row, so we don't need this
	 potentially huge buffer hanging around. */
	
/*	glReadPixels(0,0,w,h,GL_RGB,GL_UNSIGNED_BYTE,ogl_pixel_buf); */

	offset = w*h*3 - w*3;
	for (j=0;j<h;j++,offset -=(3*w))
	{
	    /* grab a row... */
	    glReadPixels(0,j,w,1,GL_RGB,GL_UNSIGNED_BYTE,ogl_pixel_buf+offset);
	}
	kpds_put_data(output_ogl_kimage,KPDS_VALUE_ALL,ogl_pixel_buf);
	kpds_close_object(output_ogl_kimage);
    }

    return(CHILL);
}

static void
ogl_render_transparent_subtree(renderobj *r,
			       matrix4x4 *parent_transform,
			       int w,
			       int h,
			       int projection,
			       void (*vertexfunc)(const GLfloat *))
{
    int status;
    int display_status,opaque_status;
    int nprims,i;
    matrix4x4 current_transform;
    render_prim *p;
    int prim_type;
    matrix4x4 savematrix;
    matrix4x4 minusC,SR,C,T,child_transform;

    glEnable(GL_BLEND);		/* enable color blending */
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);	/* set up color blending equation */

    /* build the transformation matrix for this object:
     * M = -CSRCT, where
     *    C = object's center point,
     *    SR = scale and rotate components.
     *    T = translate vector.
     **/

    identity_4x4(&minusC);
    identity_4x4(&SR);
    identity_4x4(&C);
    identity_4x4(&T);
	
    identity_4x4(&child_transform);
	
    minusC.m[3][0] = -1. * r->center.v[0];
    minusC.m[3][1] = -1. * r->center.v[1];
    minusC.m[3][2] = -1. * r->center.v[2];
	
    matrix_4x4copy(&SR,&(r->xfrm));
	    
    C.m[3][0] = r->center.v[0];
    C.m[3][1] = r->center.v[1];
    C.m[3][2] = r->center.v[2];

    for (i=0;i<3;i++)
	T.m[3][i] = r->translate_vect.v[i];
	
    mmul_4x4(&minusC,&SR,&child_transform);
    mmul_4x4(&child_transform,&C,&child_transform);
    mmul_4x4(&child_transform,&T,&child_transform);

    /* premultiply this matrix with the parent's
       transformation. */
    
    mmul_4x4(&child_transform,parent_transform,&current_transform);

    if ((r->nchildren == 0) && (r->verts_nspace == 3)) /* a leaf node */
    {
	get_object_display_status(r,&display_status);
	opaque_status = is_opaque(r);
	if ((display_status == TRUE) && (opaque_status == FALSE))
	{
    
	    glLoadMatrixf(&current_transform.m[0][0]); 

	    nprims = r->nprims;
	    for (i=0;i<nprims;i++)
	    {
		p = r->prims+i;

		/* create space for the vertices: one copy for the
		   object space (transformed) vertices, one for the
		   view space vertices. */
		prim_type = map_to_ogl_renderprims(p->type);
		(*ogl_primfuncs[prim_type])(p,w,h,NULL,NULL,NULL,NULL,
					    projection,opaque_status,r,
					    vertexfunc,&current_transform);
	    }
	}
    }
    else
    {
	for (i=0;i<r->nchildren;i++)
	{
	    ogl_render_transparent_subtree(r->children[i],&current_transform,
					   w,h,projection,vertexfunc);
	}
    }
    glDisable(GL_BLEND);		/* enable color blending */
}

static void
ogl_render_opaque_subtree(renderobj *r,
			  matrix4x4 *parent_transform,
			  int w,
			  int h,
			  int projection,
			  void (*vertexfunc)(const GLfloat *))
{
    int status;
    int display_status,opaque_status;
    int nprims,i;
    matrix4x4 current_transform;
    render_prim *p;
    int prim_type;
    matrix4x4 minusC,SR,C,T,child_transform;

    /* build the transformation matrix for this object:
     * M = -CSRCT, where
     *    C = object's center point,
     *    SR = scale and rotate components.
     *    T = translate vector.
     **/

    identity_4x4(&minusC);
    identity_4x4(&SR);
    identity_4x4(&C);
    identity_4x4(&T);
	
    identity_4x4(&child_transform);
	
    minusC.m[3][0] = -1. * r->center.v[0];
    minusC.m[3][1] = -1. * r->center.v[1];
    minusC.m[3][2] = -1. * r->center.v[2];
	
    matrix_4x4copy(&SR,&(r->xfrm));
	    
    C.m[3][0] = r->center.v[0];
    C.m[3][1] = r->center.v[1];
    C.m[3][2] = r->center.v[2];

    for (i=0;i<3;i++)
	T.m[3][i] = r->translate_vect.v[i];
	
    mmul_4x4(&minusC,&SR,&child_transform);
    mmul_4x4(&child_transform,&C,&child_transform);
    mmul_4x4(&child_transform,&T,&child_transform);

    /* premultiply this matrix with the parent's
       transformation. */
    
    mmul_4x4(&child_transform,parent_transform,&current_transform);

    if ((r->nchildren == 0) && (r->verts_nspace == 3)) /* a leaf node */
    {
	get_object_display_status(r,&display_status);
	opaque_status = is_opaque(r);
	if ((display_status == TRUE) && (opaque_status == TRUE))
	{
	    glLoadMatrixf(&current_transform.m[0][0]); 

	    nprims = r->nprims;
	    for (i=0;i<nprims;i++)
	    {
		p = r->prims+i;
		
		prim_type = map_to_ogl_renderprims(p->type);
		(*ogl_primfuncs[prim_type])(p,w,h,
					    NULL,NULL,NULL,NULL,
					    projection,opaque_status,
					    r,vertexfunc,&current_transform);
		    
	    }
	}
    }
    else
    {
	for (i=0;i<r->nchildren;i++)
	{
	    ogl_render_opaque_subtree(r->children[i],&current_transform,
				      w,h,projection,vertexfunc);
	}
    }
}

static
int map_to_ogl_renderprims(int ktype)
{
    /* convert from khoros primitve type to one of the render
       supported types. */
    int val;
    
    switch (ktype)
    {
    case KGEOM_POLYLINE_DISJOINT:
	val = POLYLIN_DIS;
	break;
    case KGEOM_POLYLINE_CONNECTED:
	val = POLYLIN_CON;
	break;
    case KGEOM_TRIANGLES_DISJOINT:
	val = POLYTRI_DIS;
	break;
    case KGEOM_TRIANGLES_CONNECTED:
	val = POLYTRI_CON;
	break;
    case KGEOM_SPHERES:
	val = SPHERE;
	break;
    case KGEOM_QUADMESH:
	val = QMESH;
	break;
    case KGEOM_OCTMESH:
	val = OMESH;
	break;

    case KGEOM_TEXT:
	val = TEXT;
	break;

	/* everything else isn't supported yet. */
    default:
	val = NO_PRIM;
	break;
    }
    return(val);
}

static void
ogl_noprim(render_prim *p,
	   int xs,int ys,
	   vertex_3d *obj_space_verts,
	   vertex_3d *view_space_verts,
	   vertex_3d *norms,
	   float *unused,
	   int projection,
	   int opaque_status,
	   renderobj *r,
	   void (*vertexfunc)(const GLfloat *),
	   matrix4x4 *m)
{
}

static void
ogl_polylin_dis(render_prim *p,
		int xs,int ys,
		vertex_3d *obj_space_verts,
		vertex_3d *view_space_verts,
		vertex_3d *norms,
		float *unused,
		int projection,
		int opaque_status,
		renderobj *r,
		void (*vertexfunc) PROTO((const GLfloat *)),
		matrix4x4 *m)
{
    vertex_3d *v;
    int i;
    float body_color[4];
    void (*colorfunc_ptr) PROTO((const GLfloat *));
    unsigned char *a_ptr;
    unsigned int stride_a,stride_b;

    glDisable(GL_LIGHTING);	/* turn off lighting */
    glEnable(GL_DEPTH_TEST);
    
    v = p->verts;
    
    if (p->ncolors == 0)
    {
	/* if there's no list of colors, set the "global" line color, as
	   well as set a dummy color function pointer (a no-op) */
	body_color[0] = r->colors.r;
	body_color[1] = r->colors.g;
	body_color[2] = r->colors.b;
	body_color[3] = r->colors.a;
	glColor4fv(body_color);

	colorfunc_ptr = gl_no_op;
	a_ptr = NULL;
	stride_a = stride_b = 0;
    }
    else
    {
	if (p->rgba_color_list)
	{
	    colorfunc_ptr = glColor4fv;
	    a_ptr = (unsigned char *)(p->rgba_color_list);
	    stride_b = sizeof (surface_colors_rgba);
	}
	else
	{
	    colorfunc_ptr = glColor3fv;
	    a_ptr = (unsigned char *)(p->rgb_color_list);
	    stride_b = sizeof(surface_colors_rgb);
	}
	if (r->layout == KPER_VERTEX)
	    stride_a = stride_b;
	else
	    stride_a = 0;
    }

    glBegin(GL_LINES);
    
    /* GL_LINES  Treates each pair of vertices as an independent line segment.
       Vertices 2n-1 and 2n define line n.  N/2 lines are drawn.  */
    for (i=0; i<p->nverts/2; i++) 
    {

	(*colorfunc_ptr)((const GLfloat *)a_ptr);
	a_ptr += stride_a;
	
	(*vertexfunc)((const GLfloat *)v);
	v++;

	(*colorfunc_ptr)((const GLfloat *)a_ptr);
	a_ptr += stride_b;
      
	(*vertexfunc)((const GLfloat *)v);
	v++;

    }
    glEnd();
}

static void
ogl_polylin_con(render_prim *p,
		int xs, int ys,
		vertex_3d *obj_space_verts,
		vertex_3d *view_space_verts,
		vertex_3d *norms,
		float *unused,
		int projection,
		int opaque_status,
		renderobj *r,
		void (*vertexfunc) PROTO((const GLfloat *)),
		matrix4x4 *m)
{
    register int i,dup;
    vertex_3d *v;
    float body_color[4];
    float *fcolors;
    unsigned int stride_a;
    void (*colorfunc_ptr) PROTO((const GLfloat *));

    glDisable(GL_LIGHTING);	/* turn off lighting */
    glEnable(GL_DEPTH_TEST);
    
    v = p->verts;

    dup = 0;
    if (p->ncolors == 0)
    {
	/* if there's no list of colors, set the "global" line color, as
	   well as set a dummy color function pointer (a no-op) */
	body_color[0] = r->colors.r;
	body_color[1] = r->colors.g;
	body_color[2] = r->colors.b;
	body_color[3] = r->colors.a;
	glColor4fv(body_color);

	colorfunc_ptr = gl_no_op;
	fcolors = NULL;
	stride_a = 0;
    }
    else
    {
	if (p->rgba_color_list)
	{
	    colorfunc_ptr = glColor4fv;
	    fcolors = (float *)(p->rgba_color_list);
	    stride_a = 4;
	}
	else
	{
	    colorfunc_ptr = glColor3fv;
	    fcolors = (float *)(p->rgb_color_list);
	    stride_a = 3;
	}
	if (r->layout == KPER_VERTEX)
	    dup = 0;
	else
	    dup = 1;
    }

    glBegin(GL_LINE_STRIP);

    /* spit out first vertex location / color */
    (*colorfunc_ptr)(fcolors);
    (*vertexfunc)((const GLfloat *)v);
    v++;

    for (i=1;i< ((dup == 1) ? (p->nverts-1) : (p->nverts));i++,v++)
    {
	/* double the vertex - load next vertex with the last color
	 if necessary due to color layout of per-segment */
	if (dup)
	    (*vertexfunc)((const GLfloat *)v);

	/* load the next vertex.  */
	fcolors += stride_a;
	(*colorfunc_ptr)(fcolors);
	(*vertexfunc)((const GLfloat *)v);
	
    }
    if (dup)
	(*vertexfunc)((const GLfloat *)v);
	

    glEnd();
}

static void
c_no_op(float *foo )
{
}

static void
gl_no_op(const GLfloat *foo )
{
}

static int
ogl_lights_func(void)
{
    vertex_3d amb_color,lcolor,lposition,ldirection;
    GLfloat list[32];
    int ilist=0;
    int projection;
    int gl_lindex=0;
    int gl_lindex_base;
    int i,nlights,onoff,ltype,imaxlights;
    float maxlights;

    GLfloat position[4],direction[4],color[4],zero[4];

    zero[0] = 0.;
    zero[1] = 0.;
    zero[2] = 0.;
    zero[3] = 1.0;

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glGetFloatv(GL_MAX_LIGHTS,&maxlights);
    imaxlights = maxlights;

    /* builds lights, lighting model, activates bindings. */

    /* GL provides for 8 lights.  the constants in gl.h define
     LIGHT0, LIGHT1... LIGHT7.  at present (august 1995) these are
     in sequential, ascending order.  the code below reflects this fact.
     a quick sanity check is provided at the front to verify that this is
     the case.  if not (for some reason if the constants are redefined)
     it will be reported. */

    if (GL_LIGHT0+1 != GL_LIGHT1)	/* if not sequential */
	kerror(NULL,"render","GL light definitions are no longer sequential.  the code in gl.c:gl_lights_func needs to be updated.",NULL);		/* report the error */

    get_projection(&projection);

    get_ambient_light_color(&amb_color);

    color[0] = amb_color.v[0];	/* red */
    color[1] = amb_color.v[1];	/* green */
    color[2] = amb_color.v[2];	/* blue */
    color[3] = 1.0;		/* alpha */

    glLightModelfv(GL_LIGHT_MODEL_AMBIENT,color);

    /* first, turn off all the GL lights before we start turning them
     back on again.  note that the following code makes the assumption that
     GL_LIGHT1 is #defined as GL_LIGHT0+1 (which it is, currently) */

    for (i=0;i<imaxlights;i++)	/* from gl.h */
	glDisable(GL_LIGHT0+i);	/* turn 'em off */

    get_num_lights(&nlights);

    for (i=0;i<nlights;i++)
    {
	get_light_onoff(i,&onoff);
	if (onoff == LIGHT_ON)
	{
	    if (gl_lindex == imaxlights)
	    {
		kerror(NULL,"render","max # of lights permitted in GL has been exceeded..skipping and continuing..",NULL);
		continue;
	    }

	    get_light_type(i,&ltype);
	    get_light_color(i,&(lcolor.v[0]),&(lcolor.v[1]),&(lcolor.v[2]));

	    color[0] = lcolor.v[0];
	    color[1] = lcolor.v[1];
	    color[2] = lcolor.v[2];
	    color[3] = 1.0;
	    
	    switch (ltype)
	    {
	    case DIRECTIONAL_LIGHT:
		get_light_direction(i,&ldirection);
		direction[0] = ldirection.v[0];
		direction[1] = ldirection.v[1];
		direction[2] = ldirection.v[2];
		direction[3] = 0.;

		glLightfv(GL_LIGHT0+gl_lindex,GL_AMBIENT,zero);
		glLightfv(GL_LIGHT0+gl_lindex,GL_DIFFUSE,color);
		glLightfv(GL_LIGHT0+gl_lindex,GL_SPECULAR,color);
		glLightfv(GL_LIGHT0+gl_lindex,GL_POSITION,direction);
		glEnable(GL_LIGHT0+gl_lindex);
		gl_lindex++;
		
		break;
	    case BIDIRECTIONAL_LIGHT:

		/* according to the GL manual, explicit two-sided lighting
		 is not available on all systems.  we'll create two
		 directional light sources..*/
		get_light_direction(i,&ldirection);
		direction[0] = ldirection.v[0];
		direction[1] = ldirection.v[1];
		direction[2] = ldirection.v[2];
		direction[3] = 0.;

		glLightfv(GL_LIGHT0+gl_lindex,GL_AMBIENT,zero);
		glLightfv(GL_LIGHT0+gl_lindex,GL_DIFFUSE,color);
		glLightfv(GL_LIGHT0+gl_lindex,GL_SPECULAR,color);
		glLightfv(GL_LIGHT0+gl_lindex,GL_POSITION,direction);
		glEnable(GL_LIGHT0+gl_lindex);
		gl_lindex++;

		direction[0] = ldirection.v[0] * -1.;
		direction[1] = ldirection.v[1] * -1.;
		direction[2] = ldirection.v[2] * -1.;

		glLightfv(GL_LIGHT0+gl_lindex,GL_AMBIENT,zero);
		glLightfv(GL_LIGHT0+gl_lindex,GL_DIFFUSE,color);
		glLightfv(GL_LIGHT0+gl_lindex,GL_SPECULAR,color);
		glLightfv(GL_LIGHT0+gl_lindex,GL_POSITION,direction);
		glEnable(GL_LIGHT0+gl_lindex);
		gl_lindex++;

		break;
		
	    case POINT_LIGHT:
		get_light_position(i,&lposition);
		
		direction[0] = lposition.v[0];
		direction[1] = lposition.v[1];
		direction[2] = lposition.v[2];
		direction[3] = 1.;

		glLightfv(GL_LIGHT0+gl_lindex,GL_AMBIENT,zero);
		glLightfv(GL_LIGHT0+gl_lindex,GL_DIFFUSE,color);
		glLightfv(GL_LIGHT0+gl_lindex,GL_SPECULAR,color);
		glLightfv(GL_LIGHT0+gl_lindex,GL_POSITION,direction);
		glEnable(GL_LIGHT0+gl_lindex);
		gl_lindex++;
		break;
	    default:
		break;
	    }
	}
    }
    ilist = 0;
    list[ilist++] = GL_LIGHT_MODEL_LOCAL_VIEWER;
    list[ilist++] = 1.;
    glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER,list);
    return(CHILL);
}
static void
ogl_polytri_dis(render_prim *p,
		int xs,int ys,
		vertex_3d *obj_space_verts,
		vertex_3d *view_space_verts,
		vertex_3d *norms,
		float *unused,
		int projection,
		int opaque_status,
		renderobj *r,
		void (*vertexfunc)(const GLfloat *),
		matrix4x4 *m)
{
    float *fn,*fc;
    float body_color[4];
    int cincr;
    int nverts;
    float *wcolors=NULL,*wnormals=NULL;

    /* set polygon rendering mode */
    glPolygonMode(GL_FRONT,GL_FILL);

    /* enable the lighting */
    
    ogl_surfprop_func(r);	/* set common surface properties */

    if (p->has_texture)
    {
#if 0
	/* texturing for disjoint polytri's UNTESTED 18 dec 95  wes */
	int tindex;
	tindex = get_texture_index(p->texture_transport_name);
	if (tindex >= 0)
	{
	    texbind(TX_TEXTURE_0,tindex+1);
	    tevbind(TV_ENV0,tindex+1);
	}
        else
	{
	    /* it is an error condition of the texture is not present in
	       the texture table*/
	    kerror("GEOMETRY","render","No such texture is registered in the internal texture table.  This is a softare error---please file a  bug report.");
	}
#endif
    }


    /* deal with colors. */
    /* unfinished business... we need to scale the vertex rgb values
     by the diffuse reflectance coefficient. */
    nverts = p->nverts;
    if (p->ncolors == 0)	/* no color list - set global color */
    {

	body_color[0] = r->colors.r;
	body_color[1] = r->colors.g;
	body_color[2] = r->colors.b;
	body_color[3] = r->colors.a;
	cincr = 0;
#if 0
	glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

#endif
	glColor4fv(body_color);

	fc = NULL;
    }
    else
    {
	float *fsrc;
	if (p->rgb_color_list)
	{
	    cincr = 3;
	    fsrc = (float *)(p->rgb_color_list);
	}
	else			/* assume rgba colors */
	{
	    cincr = 4;
	    fsrc = (float *)(p->rgba_color_list);
	}
	if (nverts != p->ncolors) /* convert from perface to per-vertex */
	{
	    fc = wcolors = (float *)kmalloc(sizeof(float)*nverts*cincr);
	    /* per-face colors for disjoint polytri's UNTESTED!!!
	       18 dec 95 wes */
	    tstrip_to_disjoint_f(fsrc,wcolors,nverts,cincr);
	}
	else
	    fc = fsrc;
    }
    
    /*  deal with normals */
    if (p->nthings == 0)
	fn = NULL;
    else
    {
	float *fsrc;
	if (p->nthings == nverts)
	    fn = &(p->prim_specifics.normals[0].v[0]);
	else			/* convert from per-face to per-vertex */
	{
	    fn = wnormals = (float *)(kmalloc(sizeof(float)*nverts*3));
	    fsrc = (float *)(p->prim_specifics.normals);
	    tstrip_to_disjoint_f(fsrc,wnormals,nverts,3);
	}
    }
    
    dump_disjoint_triangles_to_ogl(p->verts,
				   (vertex_3d *)fn,fc,cincr,
				   p->tcoords,p->tcoords_dimensions,
				   nverts,r,vertexfunc);

    /* turn off texture mapping if it was on */
#if 0
    if (p->has_texture)
	texbind(TX_TEXTURE_0,0);
#endif

    glDisable(GL_COLOR_MATERIAL);

   if (wcolors != NULL)
	kfree(wcolors);
    if (wnormals != NULL)
	kfree(wnormals);
    
}

static int
dump_disjoint_triangles_to_ogl(vertex_3d *v,
			       vertex_3d *norms,
			       float *fcolors,
			       int color_size,
			       float *ftcoords,
			       int tc_size,
			       int nverts,
			       renderobj *r,
			       void (*vertexfunc) PROTO((const GLfloat *)))
{

    /* specialized function to dump disjoint triangles to GL. */
#if 0
    register float *rv,*rn,*rc,*rt;
    register int i,cincr,tincr;
#endif
    float *rv,*rn,*rc,*rt;
    int i,cincr,tincr;
    float *tcolors = NULL;
        
    void (*colorfunc_ptr) PROTO((const GLfloat *));
    void (*tc_ptr) PROTO((const GLfloat *));
    void (*norm_ptr) PROTO((const GLfloat *));

    if (norms)
	norm_ptr = glNormal3fv;
    else
	norm_ptr = gl_no_op;

#if 0
    if (ftcoords)
    {
	if (tc_size == 2)
	    tc_ptr = t2f;
	else if (tc_size == 3)
	    tc_ptr = t3f;
				/* else error */
    }
    else
	tc_ptr = c_no_op;
#endif

/*    ogl_surfprop_func(r); */

    rv = &(v->v[0]);
    rn = &(norms->v[0]);
    cincr = color_size;
    rt = ftcoords;
    tincr = tc_size;
    
	/* indicate to open gl that all the vertex colors coming
	 down the pipe affect diffuse reflectance. */

    if (fcolors)
    {
	register int i;
	register float *fp_dst,amb,diff,alpha,*fp_src;
	    
	glColorMaterial(GL_FRONT, GL_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
      

	/* it REALLY sucks that i have to scale the RGB values
	   by the diffuse attenuation factor prior to giving these
	   values to open gl.  the open gl spec should have allowed
	   for this kind of thing...for crying out loud, it's
	   even in the 1979 version of newman & sproull's book. */
	    
	rc = fp_dst = tcolors = (float *)kmalloc(sizeof(float)*color_size*nverts);
	amb = r->attribs.amb;
	diff = r->attribs.diff;
	fp_src = fcolors;

	if (color_size==3)
	{
	    for (i=0;i<nverts*color_size;i++,fp_src++)
		*fp_dst++ = *fp_src * diff;
	    colorfunc_ptr = glColor3fv;
	}
	else		/* assume csize==4 */
	{
	    alpha = r->colors.a;
	    for (i=0;i<nverts;i++)
	    {
		*fp_dst++ = *fp_src++ * diff;
		*fp_dst++ = *fp_src++ * diff;
		*fp_dst++ = *fp_src++ * diff;
		/*		    *fp_dst++ = *fp_src++; */
		*fp_dst++ = *fp_src++ * alpha;
	    }
	    colorfunc_ptr = glColor4fv;
	}
    }
    else
    {
	colorfunc_ptr = gl_no_op;
	rc = NULL;
    }


    glBegin(GL_TRIANGLES);
    for (i=0;i<nverts;i++)
    {
	(*colorfunc_ptr)(rc);
	(*norm_ptr)(rn);
	(*vertexfunc)(rv);
#if 0
	(*tc_ptr)(rt);
	rt += tincr;
#endif

	rv += 3;
	rc+= cincr;
	rn += 3;
    }
    glEnd();
    if (tcolors)
	kfree(tcolors);
    return(CHILL);
}

static int
tstrip_to_disjoint_v(float *s,
		     float *d,
		     int n,
		     int size_s)
{
    /* copies from array S into array D using the assumption that
     the S entries are arranged 0,1,2,3,... and that they will end
     up in D as if 0,1,2,1,2,3,2,3,4.... */
    
    register float *rs,*rd;
    register int i,nbytes,index;

    rs = s;
    rd = d;
    nbytes = size_s * sizeof(float);
    
    for (i=0;i<n;i++,rd+=size_s)
    {
	index = size_s * (i/3 + i%3);
	kmemcpy(rd,rs+index,nbytes);
    }
    return(CHILL);
}
static int
tstrip_to_disjoint_f(float *s,
		     float *d,
		     int n,
		     int size_s)
    /* copies from array S into array D using the assumption that
     the S entries are arranged 0,1,2,3,... and that they will end
     up in D as if 0,0,0,1,1,1,2,2,2,... */
{    
    register float *rs,*rd;
    register int i,nbytes,index;

    rs = s;
    rd = d;
    nbytes = size_s * sizeof(float);
    
    for (i=0;i<n;i++,rd+= size_s)
    {
	index = size_s * (i/3);
	kmemcpy(rd,rs+index,nbytes);
    }
    return(CHILL);
}
static int
ogl_surfprop_func(renderobj *r)
{
    GLfloat gamb[4],gdiff[4],gspec[4],gspecexp;

    gamb[0] = r->attribs.amb * r->colors.r;
    gamb[1] = r->attribs.amb * r->colors.g;
    gamb[2] = r->attribs.amb * r->colors.b;
    gamb[3] = r->colors.a;

    gdiff[0] = r->attribs.diff * r->colors.r;
    gdiff[1] = r->attribs.diff * r->colors.g;
    gdiff[2] = r->attribs.diff * r->colors.b;
    gdiff[3] = r->colors.a;

    /* hack...we'll set the color of the highlight to white, no
     matter what.  this isn't correct, but we can't do any better in opengl*/
    gspec[0] = gspec[1] = gspec[2] = r->attribs.spec;
    gspec[3] = MAX(r->attribs.spec,r->colors.a);

    gspecexp = r->attribs.spec_exp;

    glMaterialfv(GL_FRONT,GL_AMBIENT,gamb);
    glMaterialfv(GL_FRONT,GL_DIFFUSE,gdiff);
    glMaterialfv(GL_FRONT,GL_SPECULAR,gspec);
    glMaterialf(GL_FRONT,GL_SHININESS,gspecexp);
    
    switch(r->shader)
    {
    case KSHADER_NO_LIGHT:
	glDisable(GL_LIGHTING);
	break;
    case KSHADER_FLAT:
	glEnable(GL_LIGHTING);
	glShadeModel(GL_FLAT);	/* sort of.. */
	break;
    case KSHADER_GOUROUD:
    default:
	glEnable(GL_LIGHTING);
	glShadeModel(GL_SMOOTH);
	break;
    }
    return(CHILL);
}

static void
ogl_polytri_con(render_prim *p,
		int xs,int ys,
		vertex_3d *obj_space_verts,
		vertex_3d *view_space_verts,
		vertex_3d *norms,
		float *unused,
		int projection,
		int opaque_status,
		renderobj *r,
		void (*vertexfunc) PROTO((const GLfloat *)),
		matrix4x4 *m)
{
    int i;
    float *fv=NULL,*fn=NULL,*ft=NULL;
    float *tcolors=NULL,*fc=NULL,*tcolors2=NULL;
    float body_color[4];
    void (*colorfunc_ptr) PROTO((const GLfloat *));	/* ptr to color function */
    void (*tfunc_ptr) PROTO((const GLfloat *));	/* ptr to texture map function */
    unsigned char *a_ptr=NULL;
    int stride_a=0,stride_b=0;
    int stride_tc=0;
    int csize;
    int funky_colors=0,funky_normals=0;

    int do_disjoint=0;

    /* first, figure out if we need to change representation from
     t-strip to disjoint triangles.  this is necessary if either per-face
     normals or colors are specified. it is necessary because since each
     vertex is shared by up to three triangles, any particular vertex may
     have up to 3 different normals or colors depending upon which triangle
     it belongs to.  yeah, it's ugly.  life's tough. */

    if ((p->nverts != p->ncolors) && (p->ncolors != 0)) /* per face colors */
	funky_colors = do_disjoint = 1;

    if ((p->nverts != p->nthings) && (p->nthings != 0)) /* per face normals */
	funky_normals = do_disjoint = 1;

    
    /* set polygon rendering mode */
    glPolygonMode(GL_FRONT,GL_FILL);
#if 0
    glEnable(GL_LIGHTING);	/* activate lighting */
#endif
    ogl_surfprop_func(r);	/* set common surface properties */

    fv = &(p->verts[0].v[0]);

    /* assume texture coords are per vertex. */
    /* do the texture bind operation */
    if (p->has_texture)
    {
	int tindex;
	ogl_texture_struct *t;
	
	tindex = get_texture_index(p->texture_transport_name);
	t = (ogl_texture_struct *)get_hardware_texture_ptr(tindex);
	
	if (t != NULL)
	{
	    ogl_activate_texture(t);
	    glEnable(GL_TEXTURE_2D);
	    /* set other flags? */
#if 0
	    texbind(TX_TEXTURE_0,tindex+1);
	    tevbind(TV_ENV0,tindex+1);
#endif
	}
        else
	{
	    /* it is an error condition of the texture is not present in
	       the texture table*/
	    kerror("GEOMETRY","render","ogl_polytri_con: No hardware texture is registered in the internal texture table.  This is a software error---please file a  bug report.");
	}
    }
    
    if (do_disjoint)
    {
	/* here, we're going to bust things up into the equivalent of
	 disjoint triangles.  it's an excercise in copying memory around.
	 then we'll call a routine which is optimized for dumping this kind
	 of thing to GL . */
	
	vertex_3d *wnormals;
	float *fcolors=NULL,*fsrc;
	float *ftcoords=NULL;
	vertex_3d *wverts;
	int nverts,color_size;
	nverts = (p->nverts-2)*3; /* # of faces in tstrip, 3 verts per face */
	
	wverts = (vertex_3d *)kmalloc(sizeof(vertex_3d)*nverts);
	wnormals = (vertex_3d *)kmalloc(sizeof(vertex_3d)*nverts);

	fprintf(stderr," this branch of code should not be executed due to changes in object.c.  please file a bug report - include the words 'do disjoint' \n");

	if (p->ncolors != 0)
	{
	    if (p->rgb_color_list)
	    {
		fsrc = &(p->rgb_color_list[0].r);
		color_size =3;
	    }
	    else		/* assume rgba */
	    {
		fsrc = &(p->rgba_color_list[0].r);
		color_size = 4;
	    }
	    fcolors = (float *)kmalloc(sizeof(float)*nverts*color_size);
	    if (funky_colors)	/* convert from per-face to per vertex */
		tstrip_to_disjoint_f(fsrc,fcolors,nverts,color_size);
	    else
		tstrip_to_disjoint_v(fsrc,fcolors,nverts,color_size);
	}
	else			/* set global object color in GL now */
	{
	    float body_color[4];
	    body_color[0] = r->colors.r;
	    body_color[1] = r->colors.g;
	    body_color[2] = r->colors.b;
	    body_color[3] = r->colors.a;

	    /* the global body color is set already by ogl_surfprop_func() */
	    
	    glColor4fv(body_color);
	}

	if (p->tcoords)
	{
	    ftcoords = (float *)kmalloc(sizeof(float)*nverts*(p->tcoords_dimensions));
	    /* assume per vertex tcoords */
	    tstrip_to_disjoint_v(p->tcoords,ftcoords,nverts,p->tcoords_dimensions);
	}

	tstrip_to_disjoint_v((float *)(p->verts),
			     (float *)wverts,nverts,3);  /* the vertices */
	/* assume we have normals.  the code will seg fault if we don't. */
	if (funky_normals)
	    tstrip_to_disjoint_f((float *)(p->prim_specifics.normals),
				 (float *)wnormals,
				 nverts,3);
	else
	    tstrip_to_disjoint_v((float *)(p->prim_specifics.normals),
				 (float *)wnormals,
				 nverts,3);

	dump_disjoint_triangles_to_ogl(wverts,wnormals,fcolors,color_size,
				      ftcoords,p->tcoords_dimensions,nverts,r,
				       vertexfunc);
	    

	/* all done */
	kfree(wverts);
	kfree(wnormals);
	if (fcolors)
	    kfree(fcolors);
	if (ftcoords)
	    kfree(ftcoords);

	return;
    } /* end code dealing with disjoint representation of tstrips. */

    /* if we got here, we can assume per-vertex everything. */


    if (p->has_texture)
    {
	/* NOTE!!!! we're assuming per-vertex texture coordinates.  */
	
	if (p->tcoords_dimensions == 2)
	{
	    tfunc_ptr = glTexCoord2fv;
	    stride_tc = 2;	/* stride over 2 floats to next vertex */
	}
	else if (p->tcoords_dimensions == 3)
	{
	    tfunc_ptr = glTexCoord3fv;
	    stride_tc = 3;	/* stride over 3 floats to next vertex */
	}
	ft = p->tcoords;
    }
    else
    {
	tfunc_ptr = gl_no_op;
	ft = NULL;
    }

    if (p->ncolors == 0) 
    {
	colorfunc_ptr = gl_no_op;
	a_ptr = NULL;
	stride_a = 0;
    }
    else
    {
	register float *fp_src;

	/* for now, assume per vertex */
	if (p->rgba_color_list) 
	{
	    colorfunc_ptr = glColor4fv;
	    fp_src = (float *)(p->rgba_color_list);
	    csize = 4;
	}
	else
	{
	    colorfunc_ptr = glColor3fv;
	    fp_src = (float *)(p->rgb_color_list);
	    csize = 3;
	}

	/* indicate to open gl that all the vertex colors coming
	 down the pipe affect diffuse reflectance. */
	
	glColorMaterial(GL_FRONT, GL_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
      
	{

	    /* it REALLY sucks that i have to scale the RGB values
	     by the diffuse attenuation factor prior to giving these
	     values to open gl.  the open gl spec should have allowed
	     for this kind of thing...for crying out loud, it's
	     even in the 1979 version of newman & sproull's book. */
	    
	    register int i;
	    register float *fp_dst,amb,diff,alpha;
	    
	    fc = fp_dst = tcolors = (float *)kmalloc(sizeof(float)*csize*p->ncolors);
	    amb = r->attribs.amb;
	    diff = r->attribs.diff;

	    if (csize==3)
	    {
		for (i=0;i<p->ncolors*csize;i++,fp_src++)
		    *fp_dst++ = *fp_src * diff;
	    }
	    else		/* assume csize==4 */
	    {
		alpha = r->colors.a;
		for (i=0;i<p->ncolors;i++)
		{
		    *fp_dst++ = *fp_src++ * diff;
		    *fp_dst++ = *fp_src++ * diff;
		    *fp_dst++ = *fp_src++ * diff;
/*		    *fp_dst++ = *fp_src++; */
		    *fp_dst++ = *fp_src++ * alpha;
		}
	    }
	}
    }
    
    /* assume normals are per vertex. */

    fn = &(p->prim_specifics.normals[0].v[0]);

    glBegin(GL_TRIANGLE_STRIP);
    
    for (i = 0; i < p->nverts; i++) 
    {
	(*colorfunc_ptr)(fc);
	fc += csize; 

	(*tfunc_ptr)(ft);	/* do the texture coords of the vertex */

	glNormal3fv(fn);		/* do the normal for the vertex */
	(*vertexfunc)(fv);		/* do the vertex */

	/* update pointers */
	fn += 3;
	fv += 3;
	ft += stride_tc;
    }

    glEnd();

    /* turn off texture mapping if it was on */

    if (p->has_texture)
    {
	glDisable(GL_TEXTURE_2D);
#if 0
	texbind(TX_TEXTURE_0,0);
#endif
    }

    if (tcolors)
	kfree(tcolors);
    if (tcolors2)
	kfree(tcolors2);

    glDisable(GL_COLOR_MATERIAL);
}

static void
ogl_qmesh(render_prim *p,
	  int xs, int ys,
	  vertex_3d *obj_space_verts,
	  vertex_3d *view_space_verts,
	  vertex_3d *norms,
	  float *unused,
	  int projection,
	  int opaque_status,
	  renderobj *r,
	  void (*vertexfunc) PROTO((const GLfloat *)),
	  matrix4x4 *m)
{
    int iu,iv,usize,vsize;
    vertex_3d *vrow0,*vrow1;
    vertex_3d *nrow0,*nrow1;
    float *crow0,*crow1;
    float *trow0,*trow1;
    float *tcolors=NULL,*tcolors2=NULL;
    void (*normalfunc_ptr) PROTO((const GLfloat *));
    void (*colorfunc_ptr) PROTO((const GLfloat *));
    void (*texfunc_ptr) PROTO((const GLfloat *));
    int csize,tsize;

    /* NOTE: 18 dec 1995, wes.  at present, we support only per-vertex
     everything for qmeshes. to implement per-face normals/vertices will
     require playing some games with GL : doubling up vertices where the
     location stays the same but the color/normal info changes. */
    
    glPolygonMode(GL_FRONT,GL_FILL); 
    glEnable(GL_LIGHTING);	/* activate lighting */
    
    ogl_surfprop_func(r);

    /* do the texture bind operation */
    if (p->has_texture)
    {
	int tindex;
	ogl_texture_struct *t;
	
	tindex = get_texture_index(p->texture_transport_name);
	t = (ogl_texture_struct *)get_hardware_texture_ptr(tindex);

	if (t != NULL)
	{
	    ogl_activate_texture(t);
	    glEnable(GL_TEXTURE_2D);
	}
        else
	{
	    /* it is an error condition of the texture is not present in
	       the texture table*/
	    kerror("GEOMETRY","render","ogl_qmesh: No hardware texture is registered in the internal texture table.  This is a software error---please file a  bug report.");
	}
    }

    usize = p->mesh_dims[0];
    vsize = p->mesh_dims[1];

    /* set up for handling normals */
    if (p->nthings == 0)	/* no normals - theoretically this shouldn't
				 happen - render will compute vertex normals
				 if they're not supplied.  the absence of
				 normals will be supported here cuz it's easy*/
    {
	normalfunc_ptr = gl_no_op;
	nrow0 = nrow1 = NULL;
    }
    else
    {
	normalfunc_ptr = glNormal3fv;
	nrow0 = p->prim_specifics.normals;
	nrow1 = nrow0 + usize;
    }

    /* set up for handling colors */
    if (p->ncolors == 0)
    {
	float body_color[4];
	
	colorfunc_ptr = gl_no_op;
	crow0 = crow1 = NULL;
	csize = 0;
    }
    else
    {
	register float *fp_src;
	
	if (p->rgb_color_list)
	{
	    if (opaque_status == 0) /* have to create an alpha component */
	    {
		csize = 4;
		colorfunc_ptr = glColor4fv;
		tcolors2 = (float *)kmalloc(sizeof(float)*csize*p->ncolors);
		expand_rgb_to_rgba((float *)(p->rgb_color_list),tcolors2,p->ncolors,&(r->colors.a)); 
		fp_src = tcolors2;
	    }
	    else
	    {
		csize = 3;
		colorfunc_ptr = glColor3fv;
		fp_src = (float *)(p->rgb_color_list);
	    }
	}
	else			/* assume rgba_color_list defined */
	{
	    fp_src = (float *)(p->rgba_color_list);
	    csize = 4;
	    colorfunc_ptr = glColor4fv;
	}
	    /* it REALLY sucks that i have to scale the RGB values
	     by the diffuse attenuation factor prior to giving these
	     values to open gl.  the open gl spec should have allowed
	     for this kind of thing...for crying out loud, it's
	     even in the 1979 version of newman & sproull's book. */
	{
	    register int i;
	    register float *fp_dst,amb,diff,alpha;
	    
	    fp_dst = tcolors = (float *)kmalloc(sizeof(float)*csize*p->ncolors);
	    amb = r->attribs.amb;
	    diff = r->attribs.diff;

	    /* fix to account for alpha */
	    if (csize == 3)
	    {
		for (i=0;i<p->ncolors*csize;i++,fp_src++)
		    *fp_dst++ = *fp_src * diff;
	    }
	    else		/* assume csize == 4 */
	    {
		alpha = r->colors.a;
		for (i=0;i<p->ncolors;i++)
		{
		    *fp_dst++ = *fp_src++ * diff;
		    *fp_dst++ = *fp_src++ * diff;
		    *fp_dst++ = *fp_src++ * diff;
		    *fp_dst++ = *fp_src++ * alpha; 
		    
		}
	    }
	}
	
	/* indicate to open gl that all the vertex colors coming
	 down the pipe affect diffuse reflectance. */
	
	glColorMaterial(GL_FRONT, GL_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
	
	crow0 = tcolors;
	crow1 = crow0 + csize * usize; 
    }

    /* set up for handling texture coords */
    if (p->tcoords == 0)
    {
	texfunc_ptr = gl_no_op;
	trow0 = trow1 = NULL;
	tsize = 0;
    }
    else
    {
	if (p->tcoords_dimensions == 2)
	    tsize = 2;
	else
	    tsize = 3;

	texfunc_ptr = (tsize == 2) ? glTexCoord2fv : glTexCoord3fv;
	trow0 = p->tcoords;
	trow1 = trow0 + usize*tsize;
    }

    vrow0 = p->verts;
    vrow1 = vrow0 + usize;

    for (iv=0;iv<vsize-1;iv++)
    {
	glBegin(GL_QUAD_STRIP);
	for (iu=0;iu<usize;iu++)
	{
	    (*texfunc_ptr)(trow0+iu*tsize); 
	    (*colorfunc_ptr)(crow0+iu*csize);
	    (*normalfunc_ptr)((float *)(nrow0+iu));
	    (*vertexfunc)((float *)(vrow0+iu));

	    (*texfunc_ptr)(trow1+iu*tsize); 
	    (*colorfunc_ptr)(crow1+iu*csize);
	    (*normalfunc_ptr)((float *)(nrow1+iu));
	    (*vertexfunc)((float *)(vrow1+iu)); 
	}
	glEnd();
	
	vrow0 = vrow1;
	vrow1 = vrow0 + usize;

	if (nrow0)
	{
	    nrow0 = nrow1;
	    nrow1 = nrow0 + usize;
	}
	
	if (crow0)
	{
	    crow0 = crow1;
	    crow1 = crow0+usize*csize;
	}

	if (trow0)
	{
	    trow0 = trow1;
	    trow1 = trow0 + usize*tsize;
	}

    }

    
    /* turn off texture mapping if it was on */

    if (p->has_texture)
	glDisable(GL_TEXTURE_2D);

    if (tcolors)
	kfree(tcolors);
    if (tcolors2)
	kfree(tcolors2);
}

static void
ogl_sphere(render_prim *p,
	   int xs, int ys,
	   vertex_3d *obj_space_verts,
	   vertex_3d *view_space_verts,
	   vertex_3d *norms,
	   float *unused,
	   int projection,
	   int opaque_status,
	   renderobj *r,
	   void (*vertexfunc)(const GLfloat *),
	   matrix4x4 *m)
{
    int i;
    GLUquadricObj *ogl_sphere;
    vertex_3d *v;
    float *radius,dummy_radius;
    GLint slices,stacks;
    int r_incr;
    float *fp_dst,*tcolors = NULL,*tcolors2=NULL,*fc;
    int csize;
    void (*colorfunc_ptr) PROTO((const GLfloat *));

    colorfunc_ptr = NULL;
    
    /* this version of "hardware spheres" isn't as fast as i'd like.
     my hope is that a display list will run faster than the code below.
     i get about 2 fps with 2600 spheres on a re2.  blech. */
    
    glPolygonMode(GL_FRONT,GL_FILL);
    glEnable(GL_LIGHTING);	/* activate lighting */
    ogl_surfprop_func(r);
    
    /* create a base quadric object which will be borgified into a sphere */
    ogl_sphere = gluNewQuadric();

    /* set the orientation of the normals to point outward. */
    gluQuadricOrientation(ogl_sphere,GLU_OUTSIDE);

    /* set per-vertex normals */
    gluQuadricNormals(ogl_sphere,GLU_SMOOTH);

    v = p->verts;

    if (p->prim_specifics.radii != NULL)
    {
	radius = p->prim_specifics.radii;
	r_incr = 1;
	slices= stacks = sphere_subdivisions;
    }
    else
    {
	dummy_radius = 0.;
	radius = &dummy_radius;
	r_incr = 0;
	slices = stacks = 1;
    }

    if (p->ncolors == 0)
    {
	float gdiff[4];
	glColorMaterial(GL_FRONT, GL_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	gdiff[0] = r->attribs.diff * r->colors.r;
	gdiff[1] = r->attribs.diff * r->colors.g;
	gdiff[2] = r->attribs.diff * r->colors.b;
	gdiff[3] = r->colors.a;
	glColor4fv(gdiff);
    }
    else
    {
	register float *fp_src;
	
	if (p->rgb_color_list)
	{
	    if (opaque_status == 0) /* have to create an alpha component */
	    {
		csize = 4;
		colorfunc_ptr = glColor4fv;
		tcolors2 = (float *)kmalloc(sizeof(float)*csize*p->ncolors);
		expand_rgb_to_rgba((float *)(p->rgb_color_list),tcolors2,p->ncolors,&(r->colors.a)); 
		fp_src = tcolors2;
	    }
	    else
	    {
		csize = 3;
		colorfunc_ptr = glColor3fv;
		fp_src = (float *)(p->rgb_color_list);
	    }
	}
	else			/* assume rgba_color_list defined */
	{
	    fp_src = (float *)(p->rgba_color_list);
	    csize = 4;
	    colorfunc_ptr = glColor4fv;
	}
	    /* it REALLY sucks that i have to scale the RGB values
	     by the diffuse attenuation factor prior to giving these
	     values to open gl.  the open gl spec should have allowed
	     for this kind of thing...for crying out loud, it's
	     even in the 1979 version of newman & sproull's book. */
	{
	    register int i;
	    register float *fp_dst,amb,diff,alpha;
	    
	    fc = fp_dst = tcolors = (float *)kmalloc(sizeof(float)*csize*p->ncolors);
	    amb = r->attribs.amb;
	    diff = r->attribs.diff;

	    /* fix to account for alpha */
	    if (csize == 3)
	    {
		for (i=0;i<p->ncolors*csize;i++,fp_src++)
		    *fp_dst++ = *fp_src * diff;
	    }
	    else		/* assume csize == 4 */
	    {
		alpha = r->colors.a;
		for (i=0;i<p->ncolors;i++)
		{
		    *fp_dst++ = *fp_src++ * diff;
		    *fp_dst++ = *fp_src++ * diff;
		    *fp_dst++ = *fp_src++ * diff;
		    *fp_dst++ = *fp_src++ * alpha; 
		    
		}
	    }
	}
	
	/* indicate to open gl that all the vertex colors coming
	 down the pipe affect diffuse reflectance. */
	
	glColorMaterial(GL_FRONT, GL_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
	
    }
	    
    for (i=0;i<p->nverts;i++,v++)
    {
	glPushMatrix();
	glTranslatef(v->v[0],v->v[1],v->v[2]);
	/* if p->ncolors == 0, we had no colorfunc.. //SK */
	if (colorfunc_ptr != NULL) (*colorfunc_ptr)(fc);
	fc += csize;
	gluSphere(ogl_sphere,(double)(*radius),slices,stacks);
	glPopMatrix();

	if (r_incr)
	    radius++;
    }
    
    gluDeleteQuadric(ogl_sphere);
    
    if (tcolors)
	kfree(tcolors);
    if (tcolors2)
	kfree(tcolors2);
}


static void
ogl_text(render_prim *p,
	 int xs, int ys,
	 vertex_3d *obj_space_verts,
	 vertex_3d *view_space_verts,
	 vertex_3d *norms,
	 float *unused,
	 int projection,
	 int opaque_status,
	 renderobj *r,
	 void (*vertexfunc)(const GLfloat *),
	 matrix4x4 *m)
{
    int i;
    vertex_3d *v;
    float *scolor;
    render_text_struct *rmt;
    float rgba[4];

    /* note that text is always opaque.  this is a feature of this port. ;-) */
    
    glDisable(GL_LIGHTING);	/* turn off lighting */
    glEnable(GL_DEPTH_TEST);	/* activate z-buffering */

    v = p->verts;

    glPixelStorei(GL_UNPACK_ALIGNMENT,1);
    
    rmt = p->prim_specifics.text;

    rgba[0] = r->colors.r;
    rgba[1] = r->colors.g;
    rgba[2] = r->colors.b;
    rgba[3] = r->colors.a;

    for (i=0;i<p->nverts;i++,v++,rmt++)
    {
	/* color here */
	if (rmt->fg_color != NULL)
	{
	    rgba[0] = rmt->fg_color->r;
	    rgba[1] = rmt->fg_color->g;
	    rgba[2] = rmt->fg_color->b;
	    rgba[3] = r->colors.a;
	}
	glColor4fv(rgba);

	/* position */
	glRasterPos3fv((float *)v); /* used instead of vertexfunc */

	glBitmap((GLsizei)(rmt->r_width),
		 (GLsizei)(rmt->r_height),
		 (GLfloat) 0., (GLfloat)0.,
		 (GLfloat) 0., (GLfloat)0.,
		 (const GLubyte *)(rmt->bitmap_mask[0]));
    }
}    

int
expand_rgb_to_rgba(float *rgb,
		   float *rgba,
		   int n,
		   float *alpha)
{
    int i;
    for (i=0;i<n;i++)
    {
	*rgba++ = *rgb++;
	*rgba++ = *rgb++;
	*rgba++ = *rgb++;
	*rgba++ = *alpha;
    }
    return(CHILL);
}

typedef struct zdepth
{
    float zval;
    int index;
} zdepth;

build_depths_list(zdepth *d,
		  int n,
		  render_prim *p,
		  matrix4x4 *m)
{
    /*
       in this routine,we're going to construct a list of z coordinates
     from the source primitive.  the list of z-coordinates is computed from
     the source primitive by doing a vector/matrix multiply through the
     current modelview matrix so that the z-coordinate is a properly
     transformed z-coordinate in viewspace.

     the results will typically be sorted later on so that transparent
     objects can be rendered back-to-front so that compositing happens
     correctly.
     */
    
    int i;
    vertex_3d *v;
    float m02,m12,m22,m32;

    m02 = m->m[0][2];
    m12 = m->m[1][2];
    m22 = m->m[2][2];
    m32 = m->m[2][3];		/* actually, the translational part of the
				 matrix is constant for a given matrix, so we
				 won't even bother with this extra FLOP. */

    v = p->verts;
    
    for (i=0;i<n;i++,v++)
    {
/*	d[i].zval = v->v[0] * m02 + v->v[1] * m12 + v->v[2] * m22 + m32; */
	d[i].zval = v->v[0] * m02 + v->v[1] * m12 + v->v[2] * m22;
	d[i].index = i;
    }
    return(CHILL);
}

int
depthcompare(zdepth *d1,zdepth *d2)
{
    if (d1->zval < d2->zval)
	return(1);
    else
	return(-1);
}




/* following are coordinates for a template splat.  it will be
 rendered as six triangles. */
#define IDEAL_SPLAT_NSIDES 6
static float ideal_splat_coords[IDEAL_SPLAT_NSIDES+2][3] =
{
  {0.,0.,0.},			/* the origin */
    {1.,0.,0},
    {0.5,0.8660254,0.},
    {-0.5,0.8660254,0.},
    {-1.,0.,0},
    {-0.5,-0.8660254,0.},
    {0.5,-0.8660254,0.},
    {1.,0.,0}
};

int
ogl_omesh_voxel_dot(render_prim *p,
		    renderobj *r,
		    zdepth *depths,
		    int nvoxels)
{
    register int i,index;
    register float *vwork,*nwork;
    unsigned char *cwork;
    void (*colorfunc)(unsigned char *);

    /* for the code below, we assume that p has normals and colors
     on a per-vertex basis.  the colors must consist of rgba values.
     if this isn't the case, the code will die.
     3 may 96, wes */

    /* indicate to open gl that all the vertex colors coming
       down the pipe affect diffuse reflectance. */

    /* 29 oct 96 - all colors in octmesh are assumed to be unsigned bytes. */
    if (p->byte_rgba_color_list)
	colorfunc = (void (*)(unsigned char *))glColor4ubv;
    else
	colorfunc = (void (*)(unsigned char *))c_no_op;
	
    glColorMaterial(GL_FRONT, GL_DIFFUSE);
    glEnable(GL_COLOR_MATERIAL);
	
    glPointSize(1.0);
    glBegin(GL_POINTS);

    
    for (i=0;i<nvoxels;i++)
    {
	index = depths[i].index;
	vwork = (float *)(p->verts + index);
	if (p->byte_rgba_color_list)
	    cwork = (unsigned char *)(p->byte_rgba_color_list + index);
	else
	    cwork = NULL;
	
	if (p->prim_specifics.normals)
	    nwork = (float *)(p->prim_specifics.normals + index);
	else
	    nwork = NULL;

/*	glColor4fv(cwork); */
	(*colorfunc)(cwork);
	if (nwork)
	    glNormal3fv(nwork); 
	glVertex3fv(vwork);
    }

    glEnd();
    return(CHILL);
}

int
ogl_omesh_voxel_splat_h(render_prim *p,
			renderobj *r,
			zdepth *depths,
			int nvoxels,
			matrix4x4 *m,
			matrix4x4 *model)
{
    register int i,index,j;
    register float *vwork,*cwork,*nwork;
    float v[IDEAL_SPLAT_NSIDES+2][3];
    vertex_3d v4[IDEAL_SPLAT_NSIDES+2];
    matrix4x4 scale,translate,composite;
    matrix4x4 model_inverse,rotate;

    float temp_radius = 0.5;

    /* for the code below, we assume that p has normals and colors
     on a per-vertex basis.  the colors must consist of rgba values.
     if this isn't the case, the code will die.
     9 may 96, wes */

    identity_4x4(&scale);
    identity_4x4(&translate);

#if 0
    matrix_inverse(model,&model_inverse);
    mmul_4x4(m,&model_inverse,&rotate);
#endif
    matrix_4x4copy(&rotate,m);
    /* indicate to open gl that all the vertex colors coming
       down the pipe affect diffuse reflectance. */
	
    glColorMaterial(GL_FRONT, GL_DIFFUSE);
    glEnable(GL_COLOR_MATERIAL);
	
    for (i=0;i<nvoxels;i++)
    {
	
	index = depths[i].index;
	vwork = (float *)(p->verts + index);

	/* set the color & normals for the entire splat */
	cwork = (float *)(p->rgba_color_list + index);
	nwork = (float *)(p->prim_specifics.normals + index);
#if 0
	glColor4fv(cwork);
	glNormal3fv(nwork);
#endif

	temp_radius = p->oct_mesh_radii[index];
	scale.m[0][0] = scale.m[1][1] = scale.m[2][2] = temp_radius;  
	translate.m[3][0] = vwork[0];
	translate.m[3][1] = vwork[1];
	translate.m[3][2] = vwork[2];
/*	mmul_4x4(&rotate,&scale,&composite); */
	mmul_4x4(&scale,&rotate,&composite); 
	mmul_4x4(&composite,&translate,&composite);
	
	npoint_xfrm((float *)ideal_splat_coords,&composite,(float *)v,IDEAL_SPLAT_NSIDES+2);

#if 0
	glPushMatrix(); 
	glLoadMatrixf(&(work1.m[0][0]));
#endif
	glBegin(GL_TRIANGLE_FAN);
	
	glColor4fv(cwork);
	glNormal3fv(nwork);
	
	for (j=0;j<IDEAL_SPLAT_NSIDES+2;j++)
	  glVertex3fv(v[j]); 
	  /*	  glVertex3fv(ideal_splat_coords[j]); */

	
	glEnd();
#if 0
	glPopMatrix();
#endif
	
    }
    return(CHILL);
}


static void
ogl_omesh(render_prim *p,
	  int xs,int ys,
	  vertex_3d *obj_space_verts,
	  vertex_3d *view_space_verts,
	  vertex_3d *norms,
	  float *unused,
	  int projection,
	  int opaque_status,
	  renderobj *r,
	  void (*vertexfunc)(const GLfloat *),
	  matrix4x4 *modelmatrix)
{
    zdepth *depths=NULL;
    int nvoxels;
    matrix4x4 model,projection_m,m,voxel_reorient;
    int (*cfunc)(zdepth *, zdepth *) = depthcompare;
    vertex_3d eye,at;
    
    glEnable(GL_LIGHTING);	/* activate lighting */
    ogl_surfprop_func(r);


    /* stuff to be done regardless of shader type : we're going to malloc
     a chunk of memory to use for storing transformed z coords, as well as
     an array of indices.  then we're going to sort the z's, back-to-front,
     and render the voxels in that order. */

    nvoxels = p->mesh_dims[0] * p->mesh_dims[1] * p->mesh_dims[2];

    glGetFloatv(GL_MODELVIEW_MATRIX,&(model.m[0][0]));
    glGetFloatv(GL_PROJECTION_MATRIX,&(projection_m.m[0][0]));
    mmul_4x4(&model,&projection_m,&m);
    
    if (r->shader != KSHADER_VOXEL_3D_TEXTURE)
    {
	depths = (zdepth *)kmalloc(sizeof(zdepth)*nvoxels);

	/* now, build an array that contains the transformed z-coord
	   for each voxel center. */
	build_depths_list(depths,nvoxels,p,&m);

	/* now sort those puppies into ascending-z order.  we will use
	   this list to render voxels back-to-front */
	qsort((void *)depths,nvoxels,sizeof(zdepth),
	       (int (*)(const void *, const void *))cfunc);

	/* compute a new matrix which will re-orient the voxel template
	   (whose normal is 0,0,1) so that it faces the viewer, and push
	   it onto the matrix stack. */
	identity_4x4(&voxel_reorient);
    
	get_eye(&eye);
	get_at(&at);
	
	compute_voxel_reorient_matrix(&voxel_reorient,modelmatrix,&eye,&at);
    }
    
    /* now invoke the requested shader. */
    switch(r->shader)
    {

    case KSHADER_VOXEL_DOT:
	ogl_omesh_voxel_dot(p,r,depths,nvoxels);
	break;
    case KSHADER_SPLAT_HOMOGENEOUS:
	ogl_omesh_voxel_splat_h(p,r,depths,nvoxels,&voxel_reorient,&model);
	break;
    case KSHADER_VOXEL_3D_TEXTURE:
	ogl_omesh_texture(p,r,nvoxels,&m);
	break;
    }
#if 0
    /* pop off the voxel-reorientation matrix. */
    glPopMatrix();
#endif

    if (depths != NULL)
	kfree(depths);
}

static void
ogl_render_2d_stuff(renderobj *r,
		    matrix4x4 *model,
		    int w,
		    int h,
		    int projection,
		    void (*vertexfunc)(const GLfloat *))
{
    /*
       * this routine manages the rendering for all the 2d stuff.  it is
       * a lot like the opaque and transparent 3d rendering routines, but
       * all the matrix stuff and transformations has been ripped out.  it
       * is not possible to control placement of 2d stuff in the viewport
       * interactively.  we make no assumptions about the opaque status of
       * 2d object -they may be transparent and should be rendered correctly.
       *
       * 13 may 1996, wes
     */

    int display_status,opaque_status;
    int nprims,i;
    render_prim *p;
    vertex_3d *v;
    vertex_3d *obj_space,*v2,*view_space;
    int prim_type;
    matrix4x4 dummy;

    /*  */
    identity_4x4(&dummy);

    /* the 2d viewport ranges from -1/1 in both X & Y */
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);
    
    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity();

    /* turn off the Z buffer tests */
    glDisable(GL_DEPTH_TEST);
    
    if ((r->nchildren == 0) && (r->verts_nspace == 2)) /* a leaf node */
    {
	get_object_display_status(r,&display_status);
	opaque_status = is_opaque(r);
	if (display_status == TRUE)
	{
	    nprims = r->nprims;
	    for (i=0;i<nprims;i++)
	    {
		p = r->prims+i;
		v = p->verts;
		obj_space = view_space = NULL;
		
		prim_type = map_to_ogl_renderprims(p->type);
		(*ogl_primfuncs[prim_type])(p,w,h,NULL,NULL,
					    NULL,NULL,KPROJECTION_PARALLEL,
					    opaque_status,r,vertexfunc,&dummy);
		    
	    }
	}
    }
    else
    {
	for (i=0;i<r->nchildren;i++)
	    ogl_render_2d_stuff(r->children[i],NULL,w,h,projection,vertexfunc);
    }
    /* turn it back on.. */
    glEnable(GL_DEPTH_TEST);
}

static void
ogl_build_and_init_textures(void)
{
    int i,n;
    render_texture_table *rmt;
 
    n = get_num_render_textures();
    for (i=0;i<n;i++)
    {
        /* build a OpenGL style texture from an render texture. */
        rmt = get_render_texture_table(i);
        build_ogl_texture(rmt);
    }
 
    /* define a texturing environment to GL */
    glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
}

static int
build_ogl_texture(render_texture_table *rmt)
{
    ogl_texture_struct *ogt;
    render_texture *rt;

    ogt = malloc_ogl_texture();

    rt = rmt->rmt;
    /* assign constrained dimensions dimensions */
    {
	int ow,oh,od;		/* original texture width, height, depth */
	int cw,ch,cd;
	int i;

	ow = rt->dimensions[0];
	oh = rt->dimensions[1];
	if (rt->dims == 3)
	    od = rt->dimensions[2];
	else
	    od = 0;

	/* comptue constrained dimensions.  we'll round DOWN to the
	 next nearest power of 2. */

	i = 0;
	while (ow >  0)
	{
	    ow = ow >> 1;
	    i++;
	}
	cw = 1 << (i-1);	/* change to "i" for rounding UP to next
				 nearest power of 2. */
	
	i = 0;
	while (oh >  0)
	{
	    oh = oh >> 1;
	    i++;
	}
	ch = 1 << (i-1);	/* change to "i" for rounding UP to next
				 nearest power of 2. */

	if (od > 0)
	{
	    i = 0;
	    while (od >  0)
	    {
		od = od >> 1;
		i++;
	    }
	    cd = 1 << (i-1);	/* change to "i" for rounding UP to next
				 nearest power of 2. */
	}
	else
	    cd = 0;
	
	ogt->constrained_dims[0] = cw;
	ogt->constrained_dims[1] = ch;
	ogt->constrained_dims[2] = cd;
    }

    /* for now, we only will have the base level texture, and no mipmaps.
     when we decide to compute mipmaps, we'll have to change the
     following code*/

    ogt->mipmap_levels = 0;	/* means only base level texture */

				/* malloc space for texture pointers */
    ogt->the_texture = (unsigned char **)kmalloc(sizeof(unsigned char *)*
						 (ogt->mipmap_levels + 1));

				/* malloc space for base level texture */
    ogt->the_texture[0] = (unsigned char *)kmalloc(sizeof(unsigned char) *
						   ogt->constrained_dims[0] *
						   ogt->constrained_dims[1] *
						   (ogt->constrained_dims[2] == 0) ? 1 : ogt->constrained_dims[2]);

    /* reload the texture from the named transport using data services
     to resize it. */

    load_constrained_size_texture(rmt->tname,ogt->constrained_dims,
				  rmt->rmt->dims,
				  &(ogt->the_texture[0]));

    if (rt->ncomponents == 3)	/* assume RGB */
	ogt->format = GL_RGB;
    else			/* assume RGBA */
	ogt->format = GL_RGBA;
    ogt->ncomponents = rt->ncomponents;

    /* assign texture loading (from ktransport) function */

    rmt->hardware_texture = (void *)ogt;
    return(CHILL);
}

static ogl_texture_struct *
malloc_ogl_texture(void)
{
    ogl_texture_struct *t;

    t = (ogl_texture_struct *)kmalloc(sizeof(ogl_texture_struct));
    return(t);
}

static void
free_ogl_texture_struct(ogl_texture_struct *o)
{
    int i;

    for (i=0;i< (o->mipmap_levels)+1;i++)
    {
	/* free up the various resolutions of textures */
	kfree(o->the_texture[i]);
    }
    /* clear it out to zero - makes me feel better */
    kmemset(o,0,sizeof(ogl_texture_struct));
}

static void
flush_ogl_textures(void)
{
    /* in this routine, we will free up all pointers, memory etc
     associated with the opengl textures. */
    int i,n;
    render_texture_table *r;
    ogl_texture_struct *o;

    n = get_num_render_textures();

    for (i=0;i<n;i++)
    {
	r = get_render_texture_table(i);
	if (r != NULL)
	{
	    o = (ogl_texture_struct *)(r->hardware_texture);

	    /* free all the stuff in the struct */
	    free_ogl_texture_struct(o);

	    /* the free the struct itself */
	    kfree(o);
	}
	/* else: error!! */
    }
}

/***
  * once again, terry ligocki comes through for us:
  * The transformation matrix for taking a plane normal to (0,0,1) to a plane
  * normal to (A,B,C) assuming (A,B,C) is a unit vector is:
  * /                                             \
  * |  A^2 * C + B^2     A*B*C - A*B              |
  * |  -------------     -----------       A      |
  * |    A^2 + B^2        A^2 + B^2               |
  * |                                             |
  * |   A*B*C - A*B     B^2 * C + A^2             |
  * |   -----------     -------------      B      | = T
  * |    A^2 + B^2        A^2 + B^2               |
  * |                                             |
  * |                                             |
  * |        -A              -B            C      |
  * |                                             |
  * \                                             /
  * the vector is postmultiplied, ie T*p
**/

static void
compute_voxel_reorient_matrix(matrix4x4 *v, /* the resulting matrix */
			      matrix4x4 *m, /* the model matrix */
			      vertex_3d *eye, /* world coord of eye */
			      vertex_3d *at) /* world coord of look-at */
{
    vertex_3d n1,normal;
    double mag;
    float A,B,C;
    matrix4x4 w,w2;
    double A2B2;

    identity_4x4(&w);

    npoints_diff(eye,at,&n1,1); /* vector from look-at point to eye */
    
    /* invert the world-view transformation matrix. */
    kfmatrix_inverse((float *)&(m->m[0][0]), 4, (float *)&(w2.m[0][0]));

    /* now, transform the to-eye vector through this inverse. */
#if 0
    identity_4x4(&w2);		/* HACK HACK HACK */
#endif
    npoint_xfrm((float *)&n1,&w2,(float *)&normal,1);
    
    vertex_mag(&normal,&mag);	/* unitized */
#if 0
    normal.v[0] = w2.m[0][2];
    normal.v[1] = w2.m[1][2];
    normal.v[2] = w2.m[2][2];
    
    vertex_mag(&normal,&mag);	/* unitized */
#endif
    mag = 1./mag;
    normal.v[0] *= mag;
    normal.v[1] *= mag;
    normal.v[2] *= mag;

    /*
       we're assuming that the voxel center is at 0,0,0, has a normal
       of 0,0,1, and we wish to reorient the voxel so that it's new
       normal is A,B,C.

       this scheme will work always in orthographic projections, but
       may produce visual artifacts when using perspective projection
       and extreme close-up views.
     */

    A = normal.v[0];
    B = normal.v[1];
    C = normal.v[2];

    /* what we want is actually the transpose of the above */


    A2B2 = (A*A + B*B);
    if (A2B2 == 0.)
    {
	A2B2 = 1.0;
	identity_4x4(v);
	return;
    }

    w.m[0][0] = (A*A * C + B*B) / A2B2;
    w.m[1][1] = (B*B * C + A*A) / A2B2;
    w.m[0][1] = w.m[1][0] = (A*B*C - A*B) / A2B2;
    
    w.m[2][2] = C;

    w.m[0][2] = -A;
    w.m[2][0] = A;

    w.m[1][2] = -B;
    w.m[2][1] = B;

    /*    matrix_4x4copy(&w,v); */
    matrix_4x4copy(v,&w);

    /*  */

    /*    mmul_4x4(&w2,&w,v); */
    /*    mmul_4x4(&w,v,v); */

    /*    identity_4x4(&flip_y);
    flip_y.m[1][1] = -1; */


}

typedef struct corner_struct
{
    vertex_3d v;	/* the geometric coordinates */
    int p[3];		/* the parameteric value in u/v/w */
} corner;

int
cornercompare(corner *d1,corner *d2)
{
    if (d1->v.v[2] < d2->v.v[2])
	return(1);
    else
	return(-1);
}

int
ogl_omesh_texture(render_prim *p,
		  renderobj *r,
		  int nvoxels,
		  matrix4x4 *m)
{
    int isize,ijsize,ijksize,tobackrow;
    int ustart,vstart,wstart;
    int i,skip1,skip2,to_skip,to_scan;
    
    isize= p->mesh_dims[0];
    ijsize = isize * p->mesh_dims[1];
    ijksize = ijsize * p->mesh_dims[2];
    tobackrow = ijksize - ijsize;
    /* matrix m has the model*view transform. */
    {
	/* figure out which corner of the (presumably rectangular) volume
	 is furthest away from us. */
	
	int i,j,k;
	corner c[8],*scan[3],*origin;
	int (*cfunc)(const void *, const void *) = (int (*)(const void *, const void *))cornercompare;
	vertex_3d varray[8],v2[8];

	/* load up the "corner" points thingies with values
	 which correspond to the parametric extents of the octmesh. */
	
	c[0].p[0] = c[0].p[1] = c[0].p[2] = 0; /* u=v=w = 0 */
	c[1].p[0] = 1; c[1].p[1] = c[1].p[2] = 0; /* u=1, v=w = 0 */
	c[2].p[0] = c[2].p[1] = 1; c[2].p[2] = 0; /* u=v=1, w = 0 */
	c[3].p[0] = 0; c[3].p[1] = 1; c[3].p[2] = 0; /* u=0, v=1, w = 0 */
	
	c[4].p[0] = c[4].p[1] = 0; c[4].p[2] = 1; /* u=v= 0, w = 1 */
	c[5].p[0] = 1; c[5].p[1] = 0; c[5].p[2] = 1; /* u=1, v=0, w = 0 */
	c[6].p[0] = c[6].p[1] = 1; c[6].p[2] = 1; /* u=v=1, w = 1 */
	c[7].p[0] = 0; c[7].p[1] = 1; c[7].p[2] = 1; /* u=0, v=1, w = 1 */

	/* load up an array of vertices with the corners of the
	 octmesh.  note that we're assuming a uniform grid type, and
	 that we're loading them up in the same order as we have
	 assigned parametric extents in the previous few lines of code. */

	kmemcpy(varray,p->verts,sizeof(vertex_3d));
	kmemcpy(varray+1,p->verts+isize-1,sizeof(vertex_3d));
	kmemcpy(varray+2,p->verts+ijsize-1,sizeof(vertex_3d));
	kmemcpy(varray+3,p->verts+ijsize-isize,sizeof(vertex_3d));

	kmemcpy(varray+4,p->verts+tobackrow,sizeof(vertex_3d));
	kmemcpy(varray+5,p->verts+tobackrow+isize-1,sizeof(vertex_3d));
	kmemcpy(varray+6,p->verts+tobackrow+ijsize-1,sizeof(vertex_3d));
	kmemcpy(varray+7,p->verts+tobackrow+ijsize-isize,sizeof(vertex_3d));

	/* transform the vertices through the model*view transformation  */

	npoint_xfrm((float *)varray,m,(float *)v2,8);

	/* stuff the vertices into the corner points structure */
	
	for (i=0;i<8;i++)
	    kmemcpy(&(c[i].v),v2+i,sizeof(vertex_3d));

	/* and sort them so that the first corner point will have the
	 largest Z-coordinate */
	
	qsort((void *)c,8,sizeof(corner),cfunc);

	/* pick the furthest point away from us as the beginning point */
	
	ustart = c[0].p[0];
	vstart = c[0].p[1];
	wstart = c[0].p[2];
	origin = c;
	scan[0] = NULL;
	scan[1] = NULL;
	scan[2] = NULL;

	/* we want to choose which set of planes to scan out according
	 to the axis which has the largest vector dot product with
	 the eye point (which is at 0,0,1).  so, we first have to
	 find the respective corner points which are different in only
	 one parametric direction from the furthest-away point. */


	k = 0;
	for (i=1;i<8;i++)
	{
	    int num_diff;
	    for (j=0,num_diff=0;j<3;j++)
	    {
		num_diff += !(origin->p[j] == c[i].p[j]);
	    }
	    if (num_diff == 1)
		scan[k++] = c+i;
	}

	/* at this point, the corner which is the furthest away from
	 us is in *origin.  the corner points in scan[] have the following
	 characteristics: 1. they are in decreasing Z order, and 2. they
	 differ in one and only one parametric dimension from the origin.

	 we will now compute direction vectors from origin to each of
	 the corners.  */


	for (i=0;i<3;i++)
	{
	    for (j=0;j<3;j++)
		v2[i].v[j] = scan[i]->v.v[j] - origin->v.v[j];
	}
	nnormal_unit(v2,v2,3);	/* normalize the vectors */
	for (i=0;i<3;i++)
	    v2[i].v[2] = (float)kfabs((double)v2[i].v[2]);

	/* now, figure out which vector has the largest amount of change
	   in Z. this corresponds to a dot product with the to-eye vector */

	i = 0;
	if (v2[1].v[2] > v2[2].v[2])
	    i = 1;
	if (v2[2].v[2] > v2[i].v[2])
	    i = 2;

	for (j=0;j<3;j++)
	{
	    if (!(origin->p[j] == scan[i]->p[j]))
		break;
	}
	to_scan = j;
	

	/* at this point, "i" has an index which will let us determine
	 which parametric direction to scan convert the octmesh planes. */
#if 0
	/* figure out which set of plane's we're going to render. the
	 code below will choose those sets of planes... */
	for (i=0;i<3;i++)
	{
	    if (c[0].p[i] != c[1].p[i])
	    {
		skip1 = i;
		break;
	    }
	}
	for (i=0;i<3;i++)
	{
	    if ((c[0].p[i] != c[2].p[i]) && (i != skip1))
	    {
		skip2 = i;
		break;
	    }
	}
#endif
    }

    /* we're going to assume that the 3d texture extension is available
     on this platform. */

    /* temp placeholder just to get 3d texturing working. */
    glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);

/* #ifndef MESA */
#if !defined (MESA) && defined (GL_TEXTURE_3D_EXT)

    glTexParameterf(GL_TEXTURE_3D_EXT,GL_TEXTURE_WRAP_S,GL_CLAMP);
    glTexParameterf(GL_TEXTURE_3D_EXT,GL_TEXTURE_WRAP_T,GL_CLAMP);
    glTexParameterf(GL_TEXTURE_3D_EXT,GL_TEXTURE_WRAP_R_EXT,GL_CLAMP);
    glTexParameterf(GL_TEXTURE_3D_EXT,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
    glTexParameterf(GL_TEXTURE_3D_EXT,GL_TEXTURE_MIN_FILTER,GL_NEAREST);

    glEnable(GL_BLEND);		/* enable color blending */
/*    glDisable(GL_DEPTH_TEST); */
/*    glDisable(GL_COLOR_MATERIAL); */
/*    glPolygonMode(GL_FRONT,GL_FILL);*/
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);	/* set up color blending equation */
    
    glPixelStorei(GL_UNPACK_ALIGNMENT,1);
    glTexImage3DEXT(GL_TEXTURE_3D_EXT,
		    0,
		    GL_RGBA,
		    p->mesh_dims[0],p->mesh_dims[1],p->mesh_dims[2],
		    0,
		    GL_RGBA,
		    GL_UNSIGNED_BYTE,
		    (const GLvoid *)(p->byte_rgba_color_list));
#ifdef KOPSYS_IRIX
    if (glGetError() == GL_TEXTURE_TOO_LARGE_EXT)
    {
	fprintf(stderr," texture is too large.  hahhahahaha! \n");
    }
#endif
    glEnable(GL_TEXTURE_3D_EXT);


    /* hack!! turn off lighting */
    glDisable(GL_LIGHTING);

    /* send some stuff down the pipe */

    /* hack #1.  send down a bunch of quadmeshes corresponding to s=0..w-1 */
    /* hack assumption: it's a uniform field and we can do an entire
     w-slice with a single quad.*/

    {
	int iw,w_incr;
	float tw,dtw;
	vertex_3d *v;
	float one,zero;

	one = 1.;
	zero = 0.;


	if (to_scan == 2)
/*	if ((skip1 != 2) && (skip2 != 2))*/
	{

	    if (wstart == 0)
	    {
		tw=0.;
		iw = 0;
		w_incr = 1;
		dtw = 1./(p->mesh_dims[2] - 1);
	    }
	    else
	    {
		tw = 1.;
		iw = p->mesh_dims[2]-1;
		w_incr = -1;
		dtw = -1./(p->mesh_dims[2] - 1);
	    }

	    /* do rows of constant w */
	    
	    for (i=0;i<p->mesh_dims[2];i++,iw+=w_incr,tw+=dtw)  
	    {

		v = p->verts + ijsize*iw;
		
		glBegin(GL_QUADS);

		glTexCoord3f(zero,zero,tw);  
		glVertex3fv((float *)v);

		glTexCoord3f(one,zero,tw); 
		glVertex3fv((float *)(v+isize-1));
		
		glTexCoord3f(one,one,tw);  
		glVertex3fv((float *)(v+ijsize-1));
	    
		glTexCoord3f(zero,one,tw);  
		glVertex3fv((float *)(v+ijsize-isize));
	    

		glEnd();
	    }

	    /* do rows of constant v */
	}

	if (to_scan == 1)
/*	if ((skip1 != 1) && (skip2 != 1)) */
	{

	    if (vstart == 0)
	    {
		tw=0.;
		dtw = 1./(p->mesh_dims[1] - 1);
		iw = 0;
		w_incr = 1;
	    }
	    else
	    {
		tw=1.;
		dtw = -1./(p->mesh_dims[1] - 1);
		iw = p->mesh_dims[1]-1;
		w_incr = -1;
	    }


	    for (i=0;i<p->mesh_dims[1];i++,iw+=w_incr,tw+=dtw)   
	    {	

		v = p->verts + isize*iw;
	    
		glBegin(GL_QUADS); 

		glTexCoord3f(zero,tw,zero); 
		glVertex3fv((float *)v);

		glTexCoord3f(one,tw,zero); 
		glVertex3fv((float *)(v+isize-1));

		glTexCoord3f(one,tw,one); 
		glVertex3fv((float *)(v+isize-1+tobackrow ));
	    
		glTexCoord3f(zero,tw,one);
		glVertex3fv((float *)(v+tobackrow));
	    

		glEnd();
	    }
	}

	if (to_scan == 0)
/*	if ((skip1 != 0) && (skip2 != 0)) */
	{

	    /* do rows of constant u */
	    if (ustart == 0)
	    {
		tw = 0;
		dtw = 1./(p->mesh_dims[0] - 1);
		iw = 0;
		w_incr = 1;
	    }
	    else
	    {
		tw = 1.;
		dtw = -1./(p->mesh_dims[0] - 1);
		iw = p->mesh_dims[0] - 1;
		w_incr = -1;
	    }

	    for (i=0;i<p->mesh_dims[0];i++,iw+=w_incr,tw+=dtw)
	    {
		v = p->verts + iw;
		glBegin(GL_QUADS);

		glTexCoord3f(tw,zero,zero);
		glVertex3fv((float *)v);

		glTexCoord3f(tw,one,zero);
		glVertex3fv((float *)(v+ijsize-isize));
		
		glTexCoord3f(tw,one,one);
		glVertex3fv((float *)(v+ijsize-isize+tobackrow));

		glTexCoord3f(tw,zero,one);
		glVertex3fv((float *)(v+tobackrow));
		glEnd();
	    }
	}
    }

    /* ok, all done.  turn off 3d texturing. */

    glDisable(GL_TEXTURE_3D_EXT);
    glEnable(GL_DEPTH_TEST);
#endif  /* MESA */
    return(CHILL);
}
#endif
