
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 
/*
 * Khoros: $Id: quantize.c,v 1.1 1997/04/19 00:15:23 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: quantize.c,v 1.1 1997/04/19 00:15:23 mirage Exp $";
#endif

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>             _static_routines()
   >>>>  Public:
   >>>>             public_routines()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "render.h"


/***************************************************************/
/* The following code based on code from the 'pbmplus' package */
/* written by Jef Poskanzer                                    */
/***************************************************************/


/* ppmquant.c - quantize the colors in a pixmap down to a specified number
**
** Copyright (C) 1989, 1991 by Jef Poskanzer.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
*/

#define DEBUG 0

#undef max
#define max(a,b) ((a) > (b) ? (a) : (b))
#undef min
#define min(a,b) ((a) < (b) ? (a) : (b))
#undef abs
#define abs(a) ((a) >= 0 ? (a) : -(a))
#undef odd
#define odd(n) ((n) & 1)

typedef unsigned char pixval;

#define PPM_MAXMAXVAL 255
typedef struct { pixval r, g, b; } pixel;

#define PPM_GETR(p) ((p).r)
#define PPM_GETG(p) ((p).g)
#define PPM_GETB(p) ((p).b)

#define PPM_ASSIGN(p,red,grn,blu) \
  { (p).r = (red); (p).g = (grn); (p).b = (blu); }

#define PPM_EQUAL(p,q) ( (p).r == (q).r && (p).g == (q).g && (p).b == (q).b )


/* Color scaling macro -- to make writing ppmtowhatever easier. */

#define PPM_DEPTH(newp,p,oldmaxval,newmaxval) \
    PPM_ASSIGN( (newp), \
	        (int) PPM_GETR(p) * (int)(newmaxval) / (int)(oldmaxval), \
	        (int) PPM_GETG(p) * (int)(newmaxval) / (int)(oldmaxval), \
	        (int) PPM_GETB(p) * (int)(newmaxval) / (int)(oldmaxval) )


/* Luminance macro. */

/* 
 * #define PPM_LUMIN(p) \
 *   ( 0.299 * PPM_GETR(p) + 0.587 * PPM_GETG(p) + 0.114 * PPM_GETB(p) )
 */

/* Luminance macro, using only integer ops.  Returns an int (*256)  JHB */
#define PPM_LUMIN(p) \
  ( 77 * PPM_GETR(p) + 150 * PPM_GETG(p) + 29 * PPM_GETB(p) )

/* Color histogram stuff. */

typedef struct colorhist_item* colorhist_vector;
struct colorhist_item { pixel color;
			int value;
		      };

typedef struct colorhist_list_item* colorhist_list;
struct colorhist_list_item { struct colorhist_item ch;
			     colorhist_list next;
			   };

typedef colorhist_list* colorhash_table;

typedef struct box* box_vector;
struct box {
  int index;
  int colors;
  int sum;
};


#define MAXCOLORS 32767
#define CLUSTER_MAXVAL 63

#define LARGE_LUM
#define REP_AVERAGE_PIXELS

#define FS_SCALE 1024

#define HASH_SIZE 6553

#define ppm_hashpixel(p) ((((int) PPM_GETR(p) * 33023 +    \
			    (int) PPM_GETG(p) * 30013 +    \
			    (int) PPM_GETB(p) * 27011) & 0x7fffffff)   \
			  % HASH_SIZE)



/*** function defs ***/

#ifdef __STDC__
  static colorhist_vector mediancut(colorhist_vector, int, int, int, int);
#if 0
  static int              redcompare  (colorhist_vector, colorhist_vector);
  static int              greencompare(colorhist_vector, colorhist_vector);
  static int              bluecompare (colorhist_vector, colorhist_vector);
  static int              sumcompare  (box_vector, box_vector);
#endif
  int              redcompare  (const void *, const void *);
  int              greencompare(const void *, const void *);
  int              bluecompare (const void *, const void *);
  int              sumcompare  (const void *, const void *);

  static colorhist_vector ppm_computecolorhist(pixel **, int,int,int,int *);
  static colorhash_table  ppm_computecolorhash(pixel **, int,int,int,int *);
  static colorhist_vector ppm_colorhashtocolorhist(colorhash_table, int);
  static colorhash_table  ppm_alloccolorhash(void);
  static void             ppm_freecolorhist(colorhist_vector);
  static void             ppm_freecolorhash(colorhash_table);
#else
  static colorhist_vector mediancut();
  static int              redcompare(), greencompare(), bluecompare();
  static int              sumcompare();
  static colorhist_vector ppm_computecolorhist(), ppm_colorhashtocolorhist();
  static colorhash_table  ppm_computecolorhash(), ppm_alloccolorhash();
  static void             ppm_freecolorhist(), ppm_freecolorhash();
#endif


/****************************************************************************/

void
number_colors(pixel **f, int rows, int cols)
{
  int row, col;
  unsigned char lastr, lastg, lastb;
  pixel *pP;

  lastr = (**f).r;
  lastg = (**f).g;
  lastb = (**f).b;

  for (row=0; row<rows; row++) {
    for (col=0, pP=f[row]; col<cols; col++, pP++) {
      if ((lastr != pP->r) ||
	  (lastg != pP->g) ||
	  (lastb != pP->b)) {
	printf("There are more than one color in the image.\n");
	return;
      }
    }
  }
  printf("There really is only one color in the image.\n");
}

/*-----------------------------------------------------------
|
|  Routine Name: ppmquant
|
|       Purpose: Quantize an image from 24 bits to 8 bits.  This
|                routine uses Paul Heckbert's algorithm (Proceedings
|                of SIGGRAPH, 1982) to find the best 256 color colormap
|                to represent this image, and then maps the colors of
|                the image to indices into this colormap.
|
|         Input: pic24 - The 24 bit source image, stored as RGB triplets
|                  of floating point numbers.
|                cols - The number of columns (width) of the image.
|                rows - The number of rows (height) of the image.
|                rmap,gmap,bmap - the colormap
|
|*        Output: Modifies the global colormap and byte image buffer.
|                
|       Returns: true or false.
|                
|
|    Written By: Jef Poskanzer
|          Date: 1989
| Modifications: 1992 - Converted from main routine into a function for XV
|                  by John Bradley.
|                Jan 26, 1995 - Kevin Campbell
|                  Added khoros-style comment block for this function.
|                  Changed the input image from a byte image to a float 
|                    image for render.
|                  Changed newcolors from an argument to a variable to 
|                    force it to be 256.
|
------------------------------------------------------------*/
int 
ppmquant(float *pic24, int cols, int rows, unsigned char *rmap,
	 unsigned char *gmap, unsigned char *bmap, unsigned char *bimg_buffer)
{
  pixel**          pixels;
  register pixel*  pP; 
  int              row;
  register int     col, limitcol;
  pixval           maxval, newmaxval;
  int              colors;
  register int     index;
  colorhist_vector chv, colormap;
  colorhash_table  cht;
  int              i;
  unsigned char    *picptr;
  static char      *fn = "ppmquant()";
  int newcolors;

  index = 0;
  maxval = 255;
  newcolors = 256;

  /*
   *  reformat 24-bit pic24 image (3 floats per pixel) into 2-dimensional
   *  array of pixel structures
   */

  if (DEBUG) fprintf(stderr,"%s: remapping to ppm-style internal fmt\n", fn);
  
  pixels = (pixel **) malloc(rows * sizeof(pixel *));
  if (!pixels) {
    kerror(NULL, "ppmquant", "couldn't allocate 'pixels' array");
    return FALSE;
  }
  for (row=0; row<rows; row++) {
    pixels[row] = (pixel *) malloc(cols * sizeof(pixel));
    if (!pixels[row]) {
      kerror(NULL, "ppmquant", "couldn't allocate a row of pixels array");
      return FALSE;
    }

    for (col=0, pP=pixels[row]; col<cols; col++, pP++) {
      pP->r = (unsigned char)((*pic24++)*255.0);
      pP->g = (unsigned char)((*pic24++)*255.0);
      pP->b = (unsigned char)((*pic24++)*255.0);
    }
  }
  if (DEBUG) fprintf(stderr,"%s: done format remapping\n", fn);

  /*
   *  attempt to make a histogram of the colors, unclustered.
   *  If at first we don't succeed, lower maxval to increase color
   *  coherence and try again.  This will eventually terminate, with
   *  maxval at worst 15, since 32^3 is approximately MAXCOLORS.
   */

  for ( ; ; ) {
    if (DEBUG) fprintf(stderr, "%s:  making histogram\n", fn);

    chv = ppm_computecolorhist(pixels, cols, rows, MAXCOLORS, &colors);
    if (chv != (colorhist_vector) 0) break;
    
    if (DEBUG) fprintf(stderr, "%s: too many colors!\n", fn);
    newmaxval = maxval / 2;
    if (DEBUG) fprintf(stderr, "%s: rescaling colors (maxval=%d) %s\n",
		       fn, newmaxval, "to improve clustering");

    for (row=0; row<rows; ++row)
      for (col=0, pP=pixels[row]; col<cols; ++col, ++pP)
	PPM_DEPTH( *pP, *pP, maxval, newmaxval );
    maxval = newmaxval;
  }

  if (DEBUG) fprintf(stderr,"%s: %d colors found\n", fn, colors);



  /*
   * Step 3: apply median-cut to histogram, making the new colormap.
   */

  if (DEBUG) fprintf(stderr, "%s: choosing %d colors\n", fn, newcolors);
  colormap = mediancut(chv, colors, rows * cols, maxval, newcolors);
  ppm_freecolorhist(chv);



  /*
   *  Step 4: map the colors in the image to their closest match in the
   *  new colormap, and write 'em out.
   */

  if (DEBUG) fprintf(stderr,"%s: mapping image to new colors\n", fn);
  cht = ppm_alloccolorhash();

  picptr = bimg_buffer;
  for (row = 0;  row < rows;  ++row) {
    col = 0;  limitcol = cols;  pP = pixels[row];

/*    if ((row & 0x1f) == 0) WaitCursor(); */
    do {
      int hash;
      colorhist_list chl;

      /* Check hash table to see if we have already matched this color. */

      hash = ppm_hashpixel(*pP);
      for (chl = cht[hash];  chl;  chl = chl->next)
	if (PPM_EQUAL(chl->ch.color, *pP)) {index = chl->ch.value; break;}

      if (!chl /*index = -1*/) {/* No; search colormap for closest match. */
	register int i, r1, g1, b1, r2, g2, b2;
	register long dist, newdist;

	r1 = PPM_GETR( *pP );
	g1 = PPM_GETG( *pP );
	b1 = PPM_GETB( *pP );
	dist = 2000000000;

	for (i=0; i<newcolors; i++) {
	  r2 = PPM_GETR( colormap[i].color );
	  g2 = PPM_GETG( colormap[i].color );
	  b2 = PPM_GETB( colormap[i].color );

	  newdist = ( r1 - r2 ) * ( r1 - r2 ) +
	            ( g1 - g2 ) * ( g1 - g2 ) +
	            ( b1 - b2 ) * ( b1 - b2 );

	  if (newdist<dist) { index = i;  dist = newdist; }
	}

	hash = ppm_hashpixel(*pP);
	chl = (colorhist_list) malloc(sizeof(struct colorhist_list_item));
	if (!chl) {
	  kerror(NULL, "ppmquant", "ran out of memory adding to hash table");
	  return FALSE;
	}

	chl->ch.color = *pP;
	chl->ch.value = index;
	chl->next = cht[hash];
	cht[hash] = chl;
      }

      *picptr++ = index;

      ++col;
      ++pP;
    }
    while (col != limitcol);
  }

  /* rescale the colormap and load the XV colormap */
  for (i=0; i<newcolors; i++) {
    PPM_DEPTH(colormap[i].color, colormap[i].color, maxval, 255);
    rmap[i] = PPM_GETR( colormap[i].color );
    gmap[i] = PPM_GETG( colormap[i].color );
    bmap[i] = PPM_GETB( colormap[i].color );
  }

  /* free the pixels array */
  for (i=0; i<rows; i++) free(pixels[i]);
  free(pixels);

  /* free cht and colormap */
  ppm_freecolorhist(colormap);
  ppm_freecolorhash(cht);

  return 0;
}



/*
** Here is the fun part, the median-cut colormap generator.  This is based
** on Paul Heckbert's paper "Color Image Quantization for Frame Buffer
** Display", SIGGRAPH '82 Proceedings, page 297.
*/

/**********************************/
#if 0
static int redcompare(colorhist_vector ch1, colorhist_vector ch2)
{
  return (int) PPM_GETR( ch1->color ) - (int) PPM_GETR( ch2->color );
}

/**********************************/
static int greencompare(colorhist_vector ch1, colorhist_vector ch2)
{
  return (int) PPM_GETG( ch1->color ) - (int) PPM_GETG( ch2->color );
}

/**********************************/
static int bluecompare(colorhist_vector ch1, colorhist_vector ch2)
{
  return (int) PPM_GETB( ch1->color ) - (int) PPM_GETB( ch2->color );
}
static int sumcompare(box_vector b1, box_vector b2 )
{
  return b2->sum - b1->sum;
}

#endif
int redcompare(const void  *cv_ch1, const void *cv_ch2)
{
    colorhist_vector ch1,ch2;
    ch1 = (colorhist_vector)cv_ch1;
    ch2 = (colorhist_vector)cv_ch2;
  return (int) PPM_GETR( ch1->color ) - (int) PPM_GETR( ch2->color );
}

/**********************************/
int greencompare(const void *cv_ch1,const void *cv_ch2)
{
    colorhist_vector ch1,ch2;
    ch1 = (colorhist_vector) cv_ch1;
    ch2 = (colorhist_vector) cv_ch2;
  return (int) PPM_GETG( ch1->color ) - (int) PPM_GETG( ch2->color );
}

/**********************************/
int bluecompare(const void *cv_ch1,const void *cv_ch2)
{
    colorhist_vector ch1,ch2;
    ch1 = (colorhist_vector) cv_ch1;
    ch2 = (colorhist_vector) cv_ch2;
  return (int) PPM_GETB( ch1->color ) - (int) PPM_GETB( ch2->color );
}
/**********************************/
int sumcompare(const void *cv_b1, const void *cv_b2 )
{
    box_vector b1,b2;
    b1 = (box_vector)cv_b1;
    b2 = (box_vector)cv_b2;
  return b2->sum - b1->sum;
}

/****************************************************************************/
static colorhist_vector mediancut(
				  colorhist_vector chv,
				  int colors, 
				  int sum,
				  int maxval,
				  int newcolors)
{
  colorhist_vector colormap;
  box_vector bv;
  register int bi, i;
  int boxes;

  bv = (box_vector) malloc(sizeof(struct box) * newcolors);
  colormap = (colorhist_vector) 
             malloc(sizeof(struct colorhist_item) * newcolors );

  if (!bv || !colormap) {
    kerror(NULL, "mediancut", "unable to malloc in mediancut()");
    return NULL;
  }

  for (i=0; i<newcolors; i++)
    PPM_ASSIGN(colormap[i].color, 0, 0, 0);

  /*
   *  Set up the initial box.
   */
  bv[0].index = 0;
  bv[0].colors = colors;
  bv[0].sum = sum;
  boxes = 1;


  /*
   ** Main loop: split boxes until we have enough.
   */

  while ( boxes < newcolors ) {
    register int indx, clrs;
    int sm;
    register int minr, maxr, ming, maxg, minb, maxb, v;
    int halfsum, lowersum;

    /*
     ** Find the first splittable box.
     */
    for (bi=0; bv[bi].colors<2 && bi<boxes; bi++) ;
    if (bi == boxes) break;	/* ran out of colors! */

    indx = bv[bi].index;
    clrs = bv[bi].colors;
    sm = bv[bi].sum;

    /*
     ** Go through the box finding the minimum and maximum of each
     ** component - the boundaries of the box.
     */
    minr = maxr = PPM_GETR( chv[indx].color );
    ming = maxg = PPM_GETG( chv[indx].color );
    minb = maxb = PPM_GETB( chv[indx].color );

    for (i=1; i<clrs; i++) {
      v = PPM_GETR( chv[indx + i].color );
      if (v < minr) minr = v;
      if (v > maxr) maxr = v;

      v = PPM_GETG( chv[indx + i].color );
      if (v < ming) ming = v;
      if (v > maxg) maxg = v;

      v = PPM_GETB( chv[indx + i].color );
      if (v < minb) minb = v;
      if (v > maxb) maxb = v;
    }

    /*
     ** Find the largest dimension, and sort by that component.  I have
     ** included two methods for determining the "largest" dimension;
     ** first by simply comparing the range in RGB space, and second
     ** by transforming into luminosities before the comparison.  You
     ** can switch which method is used by switching the commenting on
     ** the LARGE_ defines at the beginning of this source file.
     */
    {
      /* LARGE_LUM version */

      pixel p;
      int rl, gl, bl;

      PPM_ASSIGN(p, maxr - minr, 0, 0);
      rl = PPM_LUMIN(p);

      PPM_ASSIGN(p, 0, maxg - ming, 0);
      gl = PPM_LUMIN(p);

      PPM_ASSIGN(p, 0, 0, maxb - minb);
      bl = PPM_LUMIN(p);

      if (rl >= gl && rl >= bl)
	qsort( (void*) &(chv[indx]), (size_t)clrs, sizeof(struct colorhist_item),
	      redcompare );
      else if (gl >= bl)
	qsort( (void *) &(chv[indx]), clrs, sizeof(struct colorhist_item),
	      greencompare );
      else qsort( (void *) &(chv[indx]), clrs, sizeof(struct colorhist_item),
		 bluecompare );
    }

    /*
     ** Now find the median based on the counts, so that about half the
     ** pixels (not colors, pixels) are in each subdivision.
     */
    lowersum = chv[indx].value;
    halfsum = sm / 2;
    for (i=1; i<clrs-1; i++) {
      if (lowersum >= halfsum) break;
      lowersum += chv[indx + i].value;
    }

    /*
     ** Split the box, and sort to bring the biggest boxes to the top.
     */
    bv[bi].colors = i;
    bv[bi].sum = lowersum;
    bv[boxes].index = indx + i;
    bv[boxes].colors = clrs - i;
    bv[boxes].sum = sm - lowersum;
    ++boxes;
    qsort( (char*) bv, boxes, sizeof(struct box), sumcompare );
  }  /* while (boxes ... */

  /*
   ** Ok, we've got enough boxes.  Now choose a representative color for
   ** each box.  There are a number of possible ways to make this choice.
   ** One would be to choose the center of the box; this ignores any structure
   ** within the boxes.  Another method would be to average all the colors in
   ** the box - this is the method specified in Heckbert's paper.  A third
   ** method is to average all the pixels in the box.  You can switch which
   ** method is used by switching the commenting on the REP_ defines at
   ** the beginning of this source file.
   */
  
  for (bi=0; bi<boxes; bi++) {
    /* REP_AVERAGE_PIXELS version */
    register int indx = bv[bi].index;
    register int clrs = bv[bi].colors;
    register long r = 0, g = 0, b = 0, sum = 0;

    for (i=0; i<clrs; i++) {
      r += PPM_GETR( chv[indx + i].color ) * chv[indx + i].value;
      g += PPM_GETG( chv[indx + i].color ) * chv[indx + i].value;
      b += PPM_GETB( chv[indx + i].color ) * chv[indx + i].value;
      sum += chv[indx + i].value;
    }

    r = r / sum;  if (r>maxval) r = maxval;	/* avoid math errors */
    g = g / sum;  if (g>maxval) g = maxval;
    b = b / sum;  if (b>maxval) b = maxval;

    PPM_ASSIGN( colormap[bi].color, r, g, b );
  }

  free(bv);
  return colormap;
}




/****************************************************************************/
static colorhist_vector 
  ppm_computecolorhist(
		       pixel** pixels,
		       int cols, 
		       int rows, 
		       int maxcolors,
		       int* colorsP)
{
  colorhash_table cht;
  colorhist_vector chv;

  cht = ppm_computecolorhash(pixels, cols, rows, maxcolors, colorsP);
  if (!cht) return (colorhist_vector) 0;

  chv = ppm_colorhashtocolorhist(cht, maxcolors);
  ppm_freecolorhash(cht);
  return chv;
}


/****************************************************************************/
static colorhash_table ppm_computecolorhash(
					    pixel** pixels,
					    int cols, 
					    int rows, 
					    int maxcolors,
					    int* colorsP)
{
  colorhash_table cht;
  register pixel* pP;
  colorhist_list chl;
  int col, row, hash;

  cht = ppm_alloccolorhash( );
  *colorsP = 0;

  /* Go through the entire image, building a hash table of colors. */
  for (row=0; row<rows; row++)
    for (col=0, pP=pixels[row];  col<cols;  col++, pP++) {
      hash = ppm_hashpixel(*pP);

      for (chl = cht[hash]; chl != (colorhist_list) 0; chl = chl->next)
	if (PPM_EQUAL(chl->ch.color, *pP)) break;
      
      if (chl != (colorhist_list) 0) ++(chl->ch.value);
      else {
	if ((*colorsP)++ > maxcolors) {
	  ppm_freecolorhash(cht);
	  return (colorhash_table) 0;
	}
	
	chl = (colorhist_list) malloc(sizeof(struct colorhist_list_item));
	if (!chl) {
	  kerror(NULL, "ppm_computecolorhash", 
		 "ran out of memory computing hash table");
	  return NULL;
	}

	chl->ch.color = *pP;
	chl->ch.value = 1;
	chl->next = cht[hash];
	cht[hash] = chl;
      }
    }
  
  return cht;
}


/****************************************************************************/
static colorhash_table ppm_alloccolorhash()
{
  colorhash_table cht;
  int i;

  cht = (colorhash_table) malloc( HASH_SIZE * sizeof(colorhist_list) );
  if (!cht) {
    kerror(NULL, "ppm_alloccolorhash",
	   "ran out of memory allocating hash table");
    return NULL;
  }

  for (i=0; i<HASH_SIZE; i++ )
    cht[i] = (colorhist_list) 0;

  return cht;
}


/****************************************************************************/
static colorhist_vector ppm_colorhashtocolorhist(
						 colorhash_table cht,
						 int maxcolors)
{
  colorhist_vector chv;
  colorhist_list chl;
  int i, j;

  /* Now collate the hash table into a simple colorhist array. */
  chv = (colorhist_vector) malloc( maxcolors * sizeof(struct colorhist_item) );

  /* (Leave room for expansion by caller.) */
  if (!chv) {
    kerror(NULL, "ppm_colorhashtocolorhist", 
	   "ran out of memory generating histogram");
    return NULL;
  }

  /* Loop through the hash table. */
  j = 0;
  for (i=0; i<HASH_SIZE; i++)
    for (chl = cht[i];  chl != (colorhist_list) 0;  chl = chl->next) {
      /* Add the new entry. */
      chv[j] = chl->ch;
      ++j;
    }

  return chv;
}


/****************************************************************************/
static void ppm_freecolorhist(colorhist_vector chv)
{
  free( (char*) chv );
}


/****************************************************************************/
static void ppm_freecolorhash(colorhash_table cht)
{
  int i;
  colorhist_list chl, chlnext;

  for (i=0; i<HASH_SIZE; i++)
    for (chl = cht[i];  chl != (colorhist_list) 0; chl = chlnext) {
      chlnext = chl->next;
      free( (char*) chl );
    }

  free( (char*) cht );
}

