
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
/*********************************************************************
 

    Sun Jan  1 17:50:35 PST 1995 - changes to show_box() routine so that
    the line clipping is done to the XOR box presented to the user.

    Fri Dec 29 11:17:34 PST 1995 - implementation of renderer deferral mode.

       buglet - "blinking" z-rotate box is due to an inconsistency with
         X drawing lines into the "window" whereas GL is rendering into
	 a double buffer which is getting swapped around.  this doesn't
	 bother me too much...but should be straightened out sometime.

    Wed Aug 28 13:19:08 PDT 1996 - do something about exposure events.

*********************************************************************/


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

#include <design.h>
#include <geometry.h>

#include "render.h"
#include "imaging.h"
#include "zbuffer.h"
#include "sort_trans.h"
#include "render_objs.h"
#include "render_mouse.h"
#include "render_cursors.h"
#include "geometry/rmatrix.h"
#include "camera.h"
#include "imaging.h"

#include "xvinclude.h"
#include "xvisual/xvisual.h"

#include <X11/cursorfont.h>

static xvobject render_xvobject;

/**
  * the following should be ____
**/
static double DELTA_SCALE = 50.;
/*#define DELTA_SCALE 50.0 */


Display *render_display=NULL;
static Window render_window=0,render_parent_window=0;
static Screen *render_screen;
static GC render_gc_equiv;
static Cursor zencursor,translate_cursor,xy_rotate_cursor,
    z_rotate_cursor,scale_cursor;
static int current_rotate_mode;
static double z_rotate_portal_radius;
#define Z_ROTATE 0
#define XY_ROTATE 1

static int cursor_x= -1, cursor_y = -1;
static int virgin;

static int render_deferred_mode = 0;

/**
  * routines for building a temporary object used to represent, in
  * a minimal wireframe representation, objects which will be transformed
  * interactively by the user.
  *
  * the bounding box consists of 8 vertices.  they are ordered thus:

                              max
          7-------------------6
	 /|                  /|
	3-------------------2 |
	| |                 | |
	| 4-----------------|-5
	|/                  |/
	0------------------ 1
      min
 *       
**/

void get_parent_xfrm PROTO((matrix4x4 *));
void get_parent_xfrm_inverse PROTO((matrix4x4 *));
void sync_root_box PROTO((void));

static renderobj *ref_obj;
static vertex_3d box_verts3d[8];
static vertex_3d box_center,trans_vect,trans_vect_orig;
static vertex_3d bv[8];
static matrix4x4 view,model,comp,comp_inverse,I,view_rotations_inverse;
static matrix4x4 parent_xfrm,parent_xfrm_inverse,view_rotations;
static matrix4x4 parent_xfrm_nt,parent_xfrm_inverse_nt; /* no translation */
static matrix4x4 parent_view,parent_view_inverse;
static matrix4x4 m_ry,m_rx,m_rz;
static double dscale;
static int imagew,imageh;
KGEOM_VERTEX_TYPE dx,dy,dz,xscale,yscale,xd,yd,rx,ry,rz;
static int projection;
static int zbox_x_min,zbox_x_max,zbox_y_min,zbox_y_max,zbox_width,zbox_height,
    zbox_x_center,zbox_y_center;

/* the following paramter controls the size of the rotate circle
   that appears on the screen when you rotate the object.  the
   value 0.1666 is 1/6.  this is the distance from the left and
   right side of the screen (or top and bottom, depending upon which
   is the smallest dimension) which contains the circle.  in other
   words, if this value is 1/6, then the radius of the rotate circle
   is 1/3 the size of the smallest image dimension.  if you changed it
   to 0.25, the circle would be smaller, if you changed it to 0.10, the
   circle would be larger.  to me, 1/6 seems aesthetically pleasing. */

#define UL_ZROTATE_CIRCLE 0.1666

static void
build_outline_object(void)
{
    vertex_3d *mn,*mx;
    
    ref_obj = get_current_object();

    mn = &(ref_obj->bmin);
    mx = &(ref_obj->bmax);
    
    VCOPY(box_verts3d,mn);  /* do the min vertex. */
    VCOPY(box_verts3d+6,mx); /* do the max vertex. */

    VCOPY(box_verts3d+1,mn);
    box_verts3d[1].v[0] = mx->v[0];

    VCOPY(box_verts3d+2,mx);
    box_verts3d[2].v[2] = mn->v[2];

    VCOPY(box_verts3d+3,mn);
    box_verts3d[3].v[1] = mx->v[1];

    VCOPY(box_verts3d+4,mn);
    box_verts3d[4].v[2] = mx->v[2];

    VCOPY(box_verts3d+5,mx);
    box_verts3d[5].v[1] = mn->v[1];

    VCOPY(box_verts3d+7,mx);
    box_verts3d[7].v[0] = mn->v[0];

    matrix_4x4copy(&model,&(ref_obj->xfrm)); 

    get_parent_xfrm(&parent_xfrm);
    matrix_4x4copy(&parent_xfrm_nt,&parent_xfrm);

    /* make a version of the parent transformation matrix but with
     no translational component. this matrix will be used in computing
     the transformation of the object.  the version with the translational
     component will be used in moving the XOR box around on the screen. */
    parent_xfrm_nt.m[3][0] = parent_xfrm_nt.m[3][1] = parent_xfrm_nt.m[3][2] = 0.;
    matrix_inverse(&parent_xfrm_nt,&parent_xfrm_inverse_nt);
    
    get_parent_xfrm_inverse(&parent_xfrm_inverse); 

    VCOPY(&trans_vect,&(ref_obj->translate_vect));
    VCOPY(&trans_vect_orig,&trans_vect);
    VCOPY(&box_center,&(ref_obj->center));
}

static void
set_up_xfrms(void)
{
    vertex_3d eye,at,flength_vec;
    double flength,dfov,t;
    float fov;
    int i,min_dim;
    
    identity_4x4(&I);
    
    get_view_xform(&view);
    get_image_dims(&imagew,&imageh);
    mmul_4x4(&model,&view,&comp);
    matrix_inverse(&comp,&comp_inverse);
    get_view_rotation_matrix(&view_rotations);
    matrix_inverse(&view_rotations,&view_rotations_inverse);

    mmul_4x4(&parent_xfrm_nt,&view_rotations,&parent_view);
    matrix_inverse(&parent_view,&parent_view_inverse);

    /**
      * compute a scale factor which is applied to mouse motions and
      * used in computing translational components.  the goal here is to
      * enable a mapping such that a 1 pixel change in the mouse position
      * results in a one pixel change in the object if it lies in the
      * projection plane.  objects closer to the viewer than the projection
      * plane will move more than this, objects further away will move
      * less than this amount.
    **/
    get_eye(&eye);
    get_at(&at);
    get_fov(&fov);

    for (i=0;i<3;i++)
	flength_vec.v[i] = eye.v[i] - at.v[i];

    flength = 0.;
    for (i=0;i<3;i++)
	flength += flength_vec.v[i]*flength_vec.v[i];
    flength = ksqrt(flength);

    dfov = fov * 0.5;
    dfov = DEGREES_TO_RADIANS(dfov);
    t = ktan(dfov);

    DELTA_SCALE = t * 0.5 * MAX(imagew,imageh);
    DELTA_SCALE = flength/DELTA_SCALE;

    dx = DELTA_SCALE;
    dy = DELTA_SCALE;

    /**
      * for the Z component, we will scale mouse motions so that the
      * user can move the object either +/-Z in an amount equal to the
      * half the focal length of the camera.
    **/

    dz = flength*0.25;

    /**
      * for x/y rotations, we want to have 2pi radians worth of
      * rotation across the body of the screen.
    **/
    ry = 2.*PI/(double)imagew;
    rx = 2.*PI/(double)imageh;

    xd = xscale = ((KGEOM_VERTEX_TYPE)imagew - 0.001)/2.0;
    yd = yscale = ((KGEOM_VERTEX_TYPE)imageh - 0.001)/2.0;

    /**
      * for scaling, we will permit the user to double the size
      * of the object when moving the mouse halfway across the
      * screen in X.
    **/
    dscale = 2./(double)imagew;

    min_dim = MIN(imagew,imageh);
    
    zbox_x_center = imagew >> 1;
    zbox_y_center = imageh >> 1;
    
    dscale = 2./(double)min_dim;
    
    zbox_x_min = zbox_x_center - (0.5 - UL_ZROTATE_CIRCLE) * min_dim;
    zbox_x_max = zbox_x_center + (0.5 - UL_ZROTATE_CIRCLE) * min_dim;
    
    zbox_y_min = zbox_y_center - (0.5 - UL_ZROTATE_CIRCLE) * min_dim;
    
    zbox_y_max = zbox_y_center + (0.5 - UL_ZROTATE_CIRCLE) * min_dim;


    zbox_width = zbox_x_max - zbox_x_min;
    zbox_height = zbox_y_max - zbox_y_min;

    z_rotate_portal_radius = (zbox_width > zbox_height) ? zbox_height : zbox_width;
    z_rotate_portal_radius = z_rotate_portal_radius * 0.5;

    get_projection(&projection);
}

static void
iso_scale_box(int idel)
{
    matrix4x4 t,t2;
    double acc;

    acc = (double)idel * dscale;

    identity_4x4(&t);
    t.m[0][0] += acc;
    t.m[1][1] += acc;
    t.m[2][2] += acc;

    mmul_4x4(&t,&model,&t2);
    matrix_4x4copy(&model,&t2);
}

static int
get_rotate_mode(int x,int y)
{
    /* figure out if we're inside or outside of the "circle" formed
       by the Z rotate portal.*/
    double d;
    int dx,dy;

    dx = x - zbox_x_center;
    dy = y - zbox_y_center;

    d = (double)(dx * dx) + (double)(dy * dy);
    d = ksqrt(d);
    
    if (d > z_rotate_portal_radius)
	return(Z_ROTATE);
    else
	return(XY_ROTATE);
}


static void
rotate_box_y(int delx)
{
    double st,ct;
    double a;
    matrix4x4 t,t2;
    
    identity_4x4(&t);
    
    a = -1. * (double)delx * ry;
    st = sin(a);
    ct = cos(a);

    t.m[0][0] = t.m[2][2] = ct;
    t.m[0][2] = st;
    t.m[2][0] = -1. * st;
#if 0
    mmul_4x4(&view_rotations,&t,&t); 
    mmul_4x4(&t,&view_rotations_inverse,&t); 
#endif
    
    mmul_4x4(&parent_view,&t,&t); 
    mmul_4x4(&t,&parent_view_inverse,&t); 
    
    mmul_4x4(&model,&t,&t2); 
    matrix_4x4copy(&m_ry,&t2);
    matrix_4x4copy(&model,&t2);
    mmul_4x4(&t2,&view,&comp);
}

static void
rotate_box_z(int delx,int dely,int newx, int newy)
{
    double st,ct;
    double a1,a2;
    matrix4x4 t,t2;
    int sign,ox,oy;

    /* figure out if positive or negative z axis rotation.
       the scheme we'll use is to look at the z component of a
       vector cross product, to determine if the center of the
       screen is on the "right" or "left" of the current mouse trajectory.
     **/

    ox = zbox_x_center - newx;
    oy = zbox_y_center - newy;
    sign = delx * oy -	dely * ox;
    if (sign > 0)
	sign = -1;
    else if (sign < 0)
	sign = 1;
    else  /* huh? */
	sign = 0;
    
    identity_4x4(&t);

    /* compute theta of rotation - this is dependent upon the
       number of pixels the cursor has moved since last time, as well
       as the distance from the center of the screen.
    */

    a1 = ksqrt((double)(delx*delx) + (double)(dely*dely));
    a2 = ksqrt((double)(ox * ox) + (double)(oy * oy));
    a1 = katan(a1/a2);
    a1 = FABS(a1);
    a1 *= (double)sign;

    st = ksin(a1);
    ct = kcos(a1);

    t.m[0][0] = t.m[1][1] = ct;
    t.m[0][1] = st;
    t.m[1][0] = -1. * st;

#if 0
    mmul_4x4(&view_rotations,&t,&t); 
    mmul_4x4(&t,&view_rotations_inverse,&t);
#endif
    mmul_4x4(&parent_view,&t,&t);
    mmul_4x4(&t,&parent_view_inverse,&t);
    
    mmul_4x4(&model,&t,&t2);

    matrix_4x4copy(&m_rz,&t2);
    matrix_4x4copy(&model,&t2);
    mmul_4x4(&t2,&view,&comp);
}

static void
rotate_box_x(int delx)
{
    double st,ct;
    double a;
    matrix4x4 t,t2;
    
    identity_4x4(&t);
    
    a = (double)delx * rx;
    st = sin(a);
    ct = cos(a);

    t.m[1][1] = t.m[2][2] = ct;
    t.m[1][2] = st * -1.;
    t.m[2][1] = st;

#if 0    
    mmul_4x4(&view_rotations,&t,&t);
    mmul_4x4(&t,&view_rotations_inverse,&t);
#endif
    mmul_4x4(&parent_view,&t,&t);
    mmul_4x4(&t,&parent_view_inverse,&t);
    
    mmul_4x4(&model,&t,&t2);
    matrix_4x4copy(&m_rx,&t2);
    matrix_4x4copy(&model,&t2);
    mmul_4x4(&t2,&view,&comp);
}

static void
translate_box_xy(int delx,
		 int dely)
{
    double ddx,ddy,ddz;
    matrix4x4 *m;

/*    m = &view_rotations_inverse; */
    m = &parent_view_inverse;

    ddx = delx * m->m[0][0] + dely * m->m[1][0];
    ddy = delx * m->m[0][1] + dely * m->m[1][1];
    ddz = delx * m->m[0][2] + dely * m->m[1][2];

    trans_vect.v[0] += dx * ddx;
    trans_vect.v[1] += dy * ddy;
    trans_vect.v[2] += dx * ddz;
}

static void
translate_box_z(int delx,
		int dely)
{
    int delta;
    double ddx,ddy,ddz;
    matrix4x4 *m;

    m = &parent_view_inverse;

    delta = delx + dely;
    
    ddx = delta * m->m[2][0];
    ddy = delta * m->m[2][1];
    ddz = delta * m->m[2][2];

    trans_vect.v[0] += dx * ddx;
    trans_vect.v[1] += dx * ddy;
    trans_vect.v[2] += dx * ddz;
    
/*    trans_vect.v[2] += dx * delta; */
}

static void
show_z_rotate_box(void)
{
    XDrawArc(render_display,render_window,render_gc_equiv,
	     zbox_x_min,zbox_y_min,
	     (zbox_x_max - zbox_x_min),(zbox_y_max-zbox_y_min),
	     0,360*64);
}

static void
show_box(void)
{
    int i;
    float zmin; /* we should move this to a static variable for performance,
		   cuz we don't need to obtain this value on each entry into
		   this routine. typically, zmin isn't going to change while
		   an interactive transformation is happening.  1/1/95 wes*/
    int clipped[12];  /* boolean visible/not visible flags */
    vertex_3d larray[12][2];
    int ixarray[12][2],iyarray[12][2];

    get_zmin(&zmin);

    geom_xform_points(&model,&trans_vect,&box_center,box_verts3d,bv,
		      &parent_xfrm,NULL,8);
		      
    npoint_xfrm(&(bv[0].v[0]),&view,&(bv[0].v[0]),8);

    /* set up for clipping.  here, we have to construct an array of
       line segments */

    for (i=0;i<4;i++)
    {
        VCOPY(&(larray[i][0]),&(bv[i]));
	VCOPY(&(larray[i][1]),&(bv[(i+1)%4]));
    }
    for (i=0;i<4;i++)
    {
        VCOPY(&(larray[i+4][0]),&(bv[i+4]));
	if (i != 3)
	    VCOPY(&(larray[i+4][1]),&(bv[i+5]));
        else
	    VCOPY(&(larray[i+4][1]),&(bv[4]));
    }
    for (i=8;i<12;i++)
    {
        VCOPY(&(larray[i][0]),&(bv[i%8]));
	VCOPY(&(larray[i][1]),&(bv[i%8+4]));
    }

    /* now clip the array of segments */
    for (i=0;i<12;i++)
    {
        if (projection == KPROJECTION_PERSPECTIVE)
        {
	    clipped[i] = clip_line_persp(&(larray[i][0].v[0]),
					 &(larray[i][0].v[1]),
					 &(larray[i][0].v[2]),
					 &(larray[i][1].v[0]),
					 &(larray[i][1].v[1]),
					 &(larray[i][1].v[2]),
					 &zmin,1,0);
        }
	else
	{
	    clipped[i] = clip_line_parallel(&(larray[i][0].v[0]),
					    &(larray[i][0].v[1]),
					    &(larray[i][0].v[2]),
					    &(larray[i][1].v[0]),
					    &(larray[i][1].v[1]),
					    &(larray[i][1].v[2]),
					    &zmin,1,0);
	}
    }

    /* do the perspective divide and map to screen space */
    for (i=0;i<12;i++)
    {
        if (clipped[i] == CHILL)
	{
	    if (projection == KPROJECTION_PERSPECTIVE)
	    {
	      larray[i][0].v[0] = larray[i][0].v[0]/larray[i][0].v[2];
	      larray[i][0].v[1] = larray[i][0].v[1]/larray[i][0].v[2];

	      larray[i][1].v[0] = larray[i][1].v[0]/larray[i][1].v[2];
	      larray[i][1].v[1] = larray[i][1].v[1]/larray[i][1].v[2];
	    }

	    larray[i][0].v[0] = larray[i][0].v[0] * xscale + xd;
	    larray[i][0].v[1] = larray[i][0].v[1] * yscale + yd;
	    larray[i][1].v[0] = larray[i][1].v[0] * xscale + xd;
	    larray[i][1].v[1] = larray[i][1].v[1] * yscale + yd;
	}
    }

    for (i=0;i<12;i++)
    {
        if (clipped[i] == CHILL)
	{
	    ixarray[i][0] = (int)larray[i][0].v[0];
	    ixarray[i][1] = (int)larray[i][1].v[0];

	    iyarray[i][0] = imageh - (int)(larray[i][0].v[1]) - 1;
	    iyarray[i][1] = imageh - (int)(larray[i][1].v[1]) - 1;
	}
    }


    /* draw a bunch of lines */
    for (i=0;i<12;i++)
    {
        if (clipped[i] == CHILL)
	    XDrawLine(render_display, render_window, render_gc_equiv, 
		      ixarray[i][0], iyarray[i][0],ixarray[i][1], iyarray[i][1]);
    }
}


/***
  * routines which setup things for interactive rendering, ie,
  * setting up callbacks for button presses, etc.
**/

void
render_resize(xvobject obj, kaddr cliendData,
		XEvent *event, int *dispatch)
{
    unsigned int newx,newy;
    int oldw,oldh;

    /** using the debugger, it seems that the event type ConfigureNotify
      is used to signal a size change of the image widget, so we'll key on
      that event type.  we'll also check existing image dimensions to
      see if there is any change, given that i'm anticipating that
      ConfigureNotify could be used to signal things other than just a
      size change request.  1/19/95, wes. **/
    
    if (event->type == ConfigureNotify)
    {
	newx = event->xconfigure.width;
	newy = event->xconfigure.height;

	get_image_dims(&oldw,&oldh);

	if ((oldw != newx) || (oldh != newy))
	{
	    oldw = newx;
	    oldh = newy;  /* use int instead of unsigned int */
	    renderer_resize_func(newx,newy);
	    /* -- sad if geometry takes long to render, but better
	       then befuddled users staring at the screen waiting -- */
	    renderer_invoke_func();
	}
    }
    /* repaint imaging pane GUI to reflect new dims. */
    repaint_imaging_pane();

}

void
render_expose(xvobject obj, kaddr clientData, 
		XEvent *event, int  *dispatch)
{
    if (event->xexpose.count == 0)
    {
	renderer_refresh_func();
    }
}
/*
 *   Event handler for initializing gc_equiv
 * 	this will be called on expose events
 *  	(There's probably a more elegant way to do this - Mark will know)
 */
void
render_create_gc(xvobject obj, kaddr clientData, 
		   XEvent *event, int  *dispatch) 
{
    Pixmap zenmap,zenmask;
    XColor fg,bg;
    XGCValues	  values;	
    unsigned long  mask;

    render_display = xvw_display(obj);
    render_window  = xvw_window(obj);
    render_parent_window = xvw_window(xvw_parent(obj));
    render_screen = xvw_screen(obj);
    render_xvobject = obj;

    mask = GCLineWidth | GCFunction | GCSubwindowMode ;
    values.line_width = 0;
    values.function = GXinvert;  /* (NOT src) XOR dst */
    
    values.foreground = (unsigned long)(-1);
    values.background = (unsigned long)(0);

    values.subwindow_mode = IncludeInferiors;
    render_gc_equiv = XCreateGC(render_display, render_window,
				  mask, &values);

    /* Okay, we've initialzed gc_equiv - we don't have to do this again */
    xvw_remove_event(obj, ExposureMask, render_create_gc, NULL);

    /* add in an event action that will cause the window to get
     refreshed on an expose event.  this is primarily for the
     benefit of the OpenGL implementation..which doesn't do backing
     store properly.  8/28/96 wes .*/
    xvw_add_event(obj,ExposureMask,render_expose,NULL);
   
    /**
     * build cursor definitions
     **/
    translate_cursor = XCreateFontCursor(render_display,XC_diamond_cross);
    xy_rotate_cursor = XCreateFontCursor(render_display,XC_fleur);
    z_rotate_cursor = XCreateFontCursor(render_display,XC_exchange);
    scale_cursor = XCreateFontCursor(render_display,XC_sizing);
    zenmap = XCreateBitmapFromData(render_display,render_window,
				   (char *)zen_bits,17,17);
    zenmask = XCreateBitmapFromData(render_display,render_window,
				    (char *)zen_mask,17,17);
	
    fg.pixel = XWhitePixel(render_display,XDefaultScreen(render_display));
    fg.flags = bg.flags = DoRed | DoGreen | DoBlue;
    fg.red = fg.green = fg.blue = -1;
    bg.red = bg.green = bg.blue = 0;
    bg.pixel = XBlackPixel(render_display,XDefaultScreen(render_display));
    
    zencursor = XCreatePixmapCursor(render_display,zenmap,zenmask,&fg,&bg,1,1);

}


/**
  * action handlers for render.
**/

void
btn3_down_func(xvobject o,
	       kaddr client_data,
	       XEvent *event)
{
    XDefineCursor(render_display,render_window,translate_cursor);

    xvw_get_attribute(o, XVW_IMAGE_XPOSITION, &cursor_x);
    xvw_get_attribute(o, XVW_IMAGE_YPOSITION, &cursor_y);

    virgin = 1;
    build_outline_object();
    set_up_xfrms();

    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();		/* show the XOR box. */

}


void
btn3_motion_func(xvobject o,
	       kaddr client_data,
	       XEvent *event)
{
    int newx,newy;
    int delx,dely;


    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();		/* un-show the XOR box. */
    
    xvw_get_attribute(o, XVW_IMAGE_XPOSITION, &newx);
    xvw_get_attribute(o, XVW_IMAGE_YPOSITION, &newy);

    virgin = 0;
    
    delx = newx - cursor_x;
    dely = cursor_y - newy;

    translate_box_xy(delx,dely);

    cursor_x = newx;
    cursor_y = newy;

    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();		/* show the XOR box. */
    else			/* assume immediate mode */
    {
	int i;
	
	for (i=0;i<3;i++)
	    ref_obj->translate_vect.v[i] = trans_vect.v[i];
	renderer_invoke_func();
    }

}
void
btn3_up_func(xvobject o,
	       kaddr client_data,
	       XEvent *event)
{
    int i;
    
    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();		/* unshow the xor box */
    
    XUndefineCursor(render_display,render_window);
    XSync(render_display,0);

    if (virgin == 0)
    {
	if (render_deferred_mode == RENDER_DEFERRED)
	{
	    for (i=0;i<3;i++)
		trans_vect.v[i] = trans_vect.v[i] - trans_vect_orig.v[i];
	    for (i=0;i<3;i++)
		ref_obj->translate_vect.v[i] += trans_vect.v[i];
	}

	set_current_object(ref_obj);
	
	sync_root_box();
	
	renderer_invoke_func();
    }
}

void
shift_btn3_down_func(xvobject o,
		     kaddr client_data,
		     XEvent *event)
{
    XDefineCursor(render_display,render_window,translate_cursor);

    xvw_get_attribute(o, XVW_IMAGE_XPOSITION, &cursor_x);
    xvw_get_attribute(o, XVW_IMAGE_YPOSITION, &cursor_y);

    virgin = 1;

    build_outline_object();
    set_up_xfrms();
    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();		/* show the xor box */
}


void
shift_btn3_motion_func(xvobject o,
		       kaddr client_data,
		       XEvent *event)
{
    int newx,newy;
    int delx,dely;

    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();		/* unshow the xor box */
    
    xvw_get_attribute(o, XVW_IMAGE_XPOSITION, &newx);
    xvw_get_attribute(o, XVW_IMAGE_YPOSITION, &newy);

    delx = newx - cursor_x;
    dely = cursor_y - newy;

    virgin = 0;

    translate_box_z(delx,dely);

    cursor_x = newx;
    cursor_y = newy;

    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();		/* show the xor box */
    else
    {
	int i;
	
	for (i=0;i<3;i++)
	    ref_obj->translate_vect.v[i] = trans_vect.v[i];
	renderer_invoke_func();
    }
}

void
shift_btn2_down_func(xvobject o,
		     kaddr client_data,
		     XEvent *event)
{
    XDefineCursor(render_display,render_window,scale_cursor);

    xvw_get_attribute(o, XVW_IMAGE_XPOSITION, &cursor_x);
    xvw_get_attribute(o, XVW_IMAGE_YPOSITION, &cursor_y);

    virgin = 1;

    build_outline_object();
    set_up_xfrms();
    
    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();		/* show the xor box */
}


void
shift_btn2_motion_func(xvobject o,
		       kaddr client_data,
		       XEvent *event)
{
    int newx,newy;
    int delx;

    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();		/* show the xor box */

    
    xvw_get_attribute(o, XVW_IMAGE_XPOSITION, &newx);
    xvw_get_attribute(o, XVW_IMAGE_YPOSITION, &newy);

    delx = newx - cursor_x;

    virgin = 0;

    iso_scale_box(delx);

    cursor_x = newx;
    cursor_y = newy;

    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();		/* show the xor box */
    else
    {
	matrix_4x4copy(&(ref_obj->xfrm),&model);
	renderer_invoke_func();
    }
}


void
btn2_down_func(xvobject o,
	       kaddr client_data,
	       XEvent *event)
{
    xvw_get_attribute(o, XVW_IMAGE_XPOSITION, &cursor_x);
    xvw_get_attribute(o, XVW_IMAGE_YPOSITION, &cursor_y);

    virgin = 1;

    build_outline_object();
    set_up_xfrms();
    
    show_z_rotate_box();

    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();		/* show the xor box */

    current_rotate_mode = get_rotate_mode(cursor_x,cursor_y);

    if (current_rotate_mode == Z_ROTATE)
	XDefineCursor(render_display,render_window,z_rotate_cursor);
    else
	XDefineCursor(render_display,render_window,xy_rotate_cursor);
	
}


void
btn2_motion_func(xvobject o,
	       kaddr client_data,
	       XEvent *event)
{
    int delx,dely,newx,newy;
    int rmode;

    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();
    
    xvw_get_attribute(o, XVW_IMAGE_XPOSITION, &newx);
    xvw_get_attribute(o, XVW_IMAGE_YPOSITION, &newy);

    delx = newx - cursor_x;
    dely = cursor_y - newy;

    virgin = 0;

    rmode = get_rotate_mode(newx,newy);

    if (rmode != current_rotate_mode) /* cursor change */
    {
	XUndefineCursor(render_display,render_window);
	if (rmode == Z_ROTATE)
	    XDefineCursor(render_display,render_window,z_rotate_cursor);
	else
	    XDefineCursor(render_display,render_window,xy_rotate_cursor);
	current_rotate_mode = rmode;
    }

    /* here, we're going to figure out if we're inside or outside the
       Z-rotate portal.  if inside, we'll rotate about x/y axes, otherwise,
       we'll rotate about the Z axis.  we might have to adjust the cursor
       to reflect the change. */

    if (rmode == Z_ROTATE)
	rotate_box_z(delx,-dely,newx,newy);
    else
    {
	rotate_box_y(delx);
	rotate_box_x(dely);
    }

    cursor_x = newx;
    cursor_y = newy;

    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();
    else
    {
	matrix_4x4copy(&(ref_obj->xfrm),&model);
	renderer_invoke_func();

	show_z_rotate_box();
    }
}
void
btn2_up_func(xvobject o,
	       kaddr client_data,
	       XEvent *event)
{
    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();
    
    show_z_rotate_box();
    
    XUndefineCursor(render_display,render_window);
    XSync(render_display,0);


    /* now, premultiply all the accumulated rotations and stuff
       back into the object transformation matrix. */

    if (virgin == 0)
    {
	matrix_4x4copy(&(ref_obj->xfrm),&model);

	set_current_object(ref_obj);
	sync_root_box();
    
	renderer_invoke_func();
    }
}


void
shift_btn2_up_func(xvobject o,
		   kaddr client_data,
		   XEvent *event)
{
    if (render_deferred_mode == RENDER_DEFERRED)
	show_box();
    
    XUndefineCursor(render_display,render_window);

    XSync(render_display,0);

    /* now, premultiply all the accumulated rotations and stuff
       back into the object transformation matrix. */

    if (virgin == 0)
    {
	matrix_4x4copy(&(ref_obj->xfrm),&model);

	set_current_object(ref_obj);
	sync_root_box();
    
        renderer_invoke_func();
    }
}

void
left_key_func(xvobject o,
	      kaddr client_data,
	      XEvent *event)
{
    double st,ct;
    double a;
    matrix4x4 t,t2;
    float f;
#ifdef DEBUG    
    fprintf(stderr,"left key pressed\n");
#endif
    ref_obj = get_current_object();
    matrix_4x4copy(&model,&(ref_obj->xfrm)); 
    get_parent_xfrm(&parent_xfrm);
    get_parent_xfrm_inverse(&parent_xfrm_inverse);

    set_up_xfrms();

    get_yrot_relative(&f);
    
    identity_4x4(&t);
    
    a = DEGREES_TO_RADIANS(f);
    st = ksin(a);
    ct = kcos(a);

    t.m[0][0] = t.m[2][2] = ct;
    t.m[0][2] = st;
    t.m[2][0] = -1. * st;

    mmul_4x4(&view_rotations,&t,&t);
    mmul_4x4(&t,&view_rotations_inverse,&t);
    mmul_4x4(&model,&t,&t2);
    matrix_4x4copy(&m_rx,&t2);
    matrix_4x4copy(&model,&t2);
    mmul_4x4(&t2,&view,&comp);
    
    matrix_4x4copy(&(ref_obj->xfrm),&model);
    set_current_object(ref_obj);
    sync_root_box();
    
    renderer_invoke_func();
}

void
right_key_func(xvobject o,
	       kaddr client_data,
	       XEvent *event)
{
    double st,ct;
    double a;
    matrix4x4 t,t2;
    float f;
#ifdef DEBUG    
    fprintf(stderr,"right key pressed\n");
#endif
    ref_obj = get_current_object();
    matrix_4x4copy(&model,&(ref_obj->xfrm)); 
    get_parent_xfrm(&parent_xfrm);
    get_parent_xfrm_inverse(&parent_xfrm_inverse);

    set_up_xfrms();

    get_yrot_relative(&f);
    f *= -1.;
    
    identity_4x4(&t);
    
    a = DEGREES_TO_RADIANS(f);
    st = ksin(a);
    ct = kcos(a);

    t.m[0][0] = t.m[2][2] = ct;
    t.m[0][2] = st;
    t.m[2][0] = -1. * st;

    mmul_4x4(&view_rotations,&t,&t);
    mmul_4x4(&t,&view_rotations_inverse,&t);
    mmul_4x4(&model,&t,&t2);
    matrix_4x4copy(&m_rx,&t2);
    matrix_4x4copy(&model,&t2);
    mmul_4x4(&t2,&view,&comp);
    
    matrix_4x4copy(&(ref_obj->xfrm),&model);
    set_current_object(ref_obj);
    sync_root_box();
    
    renderer_invoke_func();
}

void
up_key_func(xvobject o,
	    kaddr client_data,
	    XEvent *event)
{
    /* effect a positive rotation about the X-axis */
    double st,ct;
    double a;
    matrix4x4 t,t2;
    float f;
#ifdef DEBUG    
    fprintf(stderr,"up key pressed\n");
#endif    
    ref_obj = get_current_object();
    matrix_4x4copy(&model,&(ref_obj->xfrm)); 
    get_parent_xfrm(&parent_xfrm);
    get_parent_xfrm_inverse(&parent_xfrm_inverse);

    set_up_xfrms();

    get_xrot_relative(&f);
    
    identity_4x4(&t);
    
    a = DEGREES_TO_RADIANS(f);
    st = sin(a);
    ct = cos(a);

    t.m[1][1] = t.m[2][2] = ct;
    t.m[1][2] = st * -1.;
    t.m[2][1] = st;

    
    mmul_4x4(&view_rotations,&t,&t);
    mmul_4x4(&t,&view_rotations_inverse,&t);
    mmul_4x4(&model,&t,&t2);
    matrix_4x4copy(&m_rx,&t2);
    matrix_4x4copy(&model,&t2);
    mmul_4x4(&t2,&view,&comp);
    
    matrix_4x4copy(&(ref_obj->xfrm),&model);
    set_current_object(ref_obj);
    sync_root_box();
    
    renderer_invoke_func();
}

void
down_key_func(xvobject o,
	      kaddr client_data,
	      XEvent *event)
{
    /* effect a negative rotation about the X axis */
    double st,ct;
    double a;
    matrix4x4 t,t2;
    float f;
#ifdef DEBUG    
    fprintf(stderr,"down key pressed\n");
#endif    
    ref_obj = get_current_object();
    matrix_4x4copy(&model,&(ref_obj->xfrm)); 
    get_parent_xfrm(&parent_xfrm);
    get_parent_xfrm_inverse(&parent_xfrm_inverse);

    set_up_xfrms();

    get_xrot_relative(&f);
    f *= -1.;
    
    identity_4x4(&t);
    
    a = DEGREES_TO_RADIANS(f);
    st = sin(a);
    ct = cos(a);

    t.m[1][1] = t.m[2][2] = ct;
    t.m[1][2] = st * -1.;
    t.m[2][1] = st;

    
    mmul_4x4(&view_rotations,&t,&t);
    mmul_4x4(&t,&view_rotations_inverse,&t);
    mmul_4x4(&model,&t,&t2);
    matrix_4x4copy(&m_rx,&t2);
    matrix_4x4copy(&model,&t2);
    mmul_4x4(&t2,&view,&comp);
    
    matrix_4x4copy(&(ref_obj->xfrm),&model);
    set_current_object(ref_obj);
    sync_root_box();
    
    renderer_invoke_func();
}

void
page_up_key_func(xvobject o,
		 kaddr client_data,
		 XEvent *event)
{
    /* effect a positive rotation about the Z-axis */
    double st,ct;
    double a;
    matrix4x4 t,t2;
    float f;
#ifdef DEBUG    
    fprintf(stderr,"page up key pressed\n");
#endif    

    ref_obj = get_current_object();
    matrix_4x4copy(&model,&(ref_obj->xfrm)); 
    get_parent_xfrm(&parent_xfrm);
    get_parent_xfrm_inverse(&parent_xfrm_inverse);

    set_up_xfrms();

    get_zrot_relative(&f);
    
    identity_4x4(&t);
    
    a = DEGREES_TO_RADIANS(f);
    st = sin(a);
    ct = cos(a);

    st = ksin(a);
    ct = kcos(a);

    t.m[0][0] = t.m[1][1] = ct;
    t.m[0][1] = st;
    t.m[1][0] = -1. * st;

    mmul_4x4(&view_rotations,&t,&t);
    mmul_4x4(&t,&view_rotations_inverse,&t);
    mmul_4x4(&model,&t,&t2);
    matrix_4x4copy(&m_rx,&t2);
    matrix_4x4copy(&model,&t2);
    mmul_4x4(&t2,&view,&comp);
    
    matrix_4x4copy(&(ref_obj->xfrm),&model);
    set_current_object(ref_obj);
    sync_root_box();
    
    renderer_invoke_func();

}

void
page_down_key_func(xvobject o,
		   kaddr client_data,
		   XEvent *event)
{
    /* effect a positive rotation about the Z-axis */
    double st,ct;
    double a;
    matrix4x4 t,t2;
    float f;
#ifdef DEBUG    
    fprintf(stderr,"page down key pressed\n");
#endif    

    ref_obj = get_current_object();
    matrix_4x4copy(&model,&(ref_obj->xfrm)); 
    get_parent_xfrm(&parent_xfrm);
    get_parent_xfrm_inverse(&parent_xfrm_inverse);

    set_up_xfrms();

    get_zrot_relative(&f);
    f *= -1.;
    
    identity_4x4(&t);
    
    a = DEGREES_TO_RADIANS(f);
    st = sin(a);
    ct = cos(a);

    st = ksin(a);
    ct = kcos(a);

    t.m[0][0] = t.m[1][1] = ct;
    t.m[0][1] = st;
    t.m[1][0] = -1. * st;

    mmul_4x4(&view_rotations,&t,&t);
    mmul_4x4(&t,&view_rotations_inverse,&t);
    mmul_4x4(&model,&t,&t2);
    matrix_4x4copy(&m_rx,&t2);
    matrix_4x4copy(&model,&t2);
    mmul_4x4(&t2,&view,&comp);
    
    matrix_4x4copy(&(ref_obj->xfrm),&model);
    set_current_object(ref_obj);
    sync_root_box();
    
    renderer_invoke_func();
}

static float cache_eye_separation;
static int toggle_position=CHILL;

/* the following two routines permit the getting and setting of
 the cache values.  note that the intent of the toggle and cache
 eye separation is to provide means whereby the user can switch,
 while in stereo display mode and using a stereo VOF, immediately
 between a true stereo image and a faked mono image for the purposes
 of demonstrating the difference between stereo and mono imaging.
 the following two routines are used in the scene file reading and
 writing */
int
get_toggle_cache(int *toggle, float *cache_eye_sep)
{
    *toggle = toggle_position;
    *cache_eye_sep = cache_eye_separation;
    return(CHILL);
}

int
set_toggle_cache(int toggle, float *cache_eye_sep)
{
    toggle_position = toggle;
    cache_eye_separation = *cache_eye_sep;
    return(CHILL);
}

void
stereo_mono_toggle_func(xvobject o,
			kaddr client_data,
			XEvent *event)
{

    if (get_camera_type() == CAMERA_TYPE_STEREOSCOPIC)
    {
	if (toggle_position == CHILL)
	{
	    /* assume "real" eye separation is non-zero, and that
	     the user wants to "fake" a mono display by temporarily
	     setting the eye separation to zero.  so, we'll grab the
	     eye separation, cache it, set the current one to zero,
	     ask the stereo view parms to be recomputed, and the scene
	     to be rerendered.*/
	    get_eye_separation(&cache_eye_separation);
	    set_eye_separation((float)0.);
	    toggle_position = WHACKED;
	}
	else
	{
	    /* reset the eye separation back to whatever it
	     was before we zero'd it out. */
	    set_eye_separation(cache_eye_separation);
	    toggle_position = CHILL;
	}
	renderer_invoke_func();
    }
}

void
render_configurerequest(xvobject obj, kaddr clientData, 
		   XEvent *event, int  *dispatch)
{
    int x,y;

    if ((render_display == NULL) || (event->type != ConfigureNotify))
	return;
    get_render_image_position(&x,&y);
    wheres_render_window(&x,&y);
    set_render_image_position(x,y);
    
}


void
set_up_render_mouse_button_definitions(xvobject xvimage,
					 kobject output_kimage)
{
    /**
      * here, we assign routines to various mouse buttons.
      *
      * we will use the right button for translations, the middle
      * button for rotations, and the left button for picks.
      *
      * action handlers are established for:
      *     right button down, right button motion, right button up -
      *          produce translations in the image plane
      *     <shift>+right button - produces translations perpindicular
      *          to the image plane
      *
      *     middle button - rotations.  which axis depends upon
      *          where the cursor is on the screen and which way
      *          the mouse moves.
      *
      *     shift+middle = isometric scaling
      *   
    **/

    xvw_add_event(xvimage, ExposureMask, render_create_gc, NULL);
    xvw_add_event(xvimage, StructureNotifyMask, render_resize,NULL);

    /* the following event is added to the parent so that we can
     capture window-movement events.  */
    xvw_add_event(xvw_parent(xvimage), StructureNotifyMask, render_configurerequest,NULL);
/*    xvw_add_event(xvimage, ConfigureRequestMask, render_configrequest,NULL); */
/*    xvw_add_event(xvimage, VisibilityChangeMask, render_vischange,NULL);*/

    /* right button definitions: down, motion, up */
    xvw_add_action(xvimage,"<Btn3Down>",btn3_down_func,NULL,TRUE);
    xvw_add_action(xvimage,"<Btn3Motion>",btn3_motion_func,NULL,TRUE);
    xvw_add_action(xvimage,"<Btn3Up>",btn3_up_func,NULL,TRUE);
    
    /* shift_right button definitions: down, motion, up.  note that
     the same routines for init and completion are used here as in
     plain ole btn3_down.  only the shift_btn3_motion func is different */
    
    xvw_add_action(xvimage,"Shift<Btn3Down>",btn3_down_func,NULL,TRUE); 
    xvw_add_action(xvimage,"Shift<Btn3Motion>",shift_btn3_motion_func,NULL,TRUE);
    xvw_add_action(xvimage,"Shift<Btn3Up>",btn3_up_func,NULL,TRUE); 
    
    /* middle button definitions: down, motion, up - rotations*/
    xvw_add_action(xvimage,"<Btn2Down>",btn2_down_func,NULL,TRUE);
    xvw_add_action(xvimage,"<Btn2Motion>",btn2_motion_func,NULL,TRUE);
    xvw_add_action(xvimage,"<Btn2Up>",btn2_up_func,NULL,TRUE);

    /* keyboard actions */

    xvw_add_action(xvimage,"<Key>Left",left_key_func,NULL,TRUE);
    xvw_add_action(xvimage,"<Key>Right",right_key_func,NULL,TRUE);
    xvw_add_action(xvimage,"<Key>Up",up_key_func,NULL,TRUE);
    xvw_add_action(xvimage,"<Key>Down",down_key_func,NULL,TRUE);
    xvw_add_action(xvimage,"<Key>Page_Up",page_up_key_func,NULL,TRUE);
    xvw_add_action(xvimage,"<Key>Page_Down",page_down_key_func,NULL,TRUE);

    /* -- silly ole X, the above "standard" keycodes only work on sun -- */

    /* -- define some other key sequences  -- */

    /* -- (x,X) for X  (y,Y) for Y  (z,Z) for Z -- */
    /* -- chosen cuz they are ez to recall -- */
    xvw_add_action(xvimage,"<Key>x",left_key_func,NULL,TRUE);
    xvw_add_action(xvimage,"<Key>X",right_key_func,NULL,TRUE);
    xvw_add_action(xvimage,"<Key>y",up_key_func,NULL,TRUE);
    xvw_add_action(xvimage,"<Key>Y",down_key_func,NULL,TRUE);
    xvw_add_action(xvimage,"<Key>z",page_up_key_func,NULL,TRUE);
    xvw_add_action(xvimage,"<Key>Z",page_down_key_func,NULL,TRUE);

    /* shift-middle button definitions: scaling */

    /* first, delete old actions.. */
#if 0
    xvw_remove_action(xvimage,"Shift<Btn2Down>",btn2_down_func,NULL);
    xvw_remove_action(xvimage,"Shift<Btn2Motion>",btn2_motion_func,NULL);
    xvw_remove_action(xvimage,"Shift<Btn2Up>",btn2_up_func,NULL);
#endif
    
    xvw_add_action(xvimage,"Shift<Btn2Down>",shift_btn2_down_func,NULL,TRUE);
    xvw_add_action(xvimage,"Shift<Btn2Motion>",shift_btn2_motion_func,NULL,TRUE);
    xvw_add_action(xvimage,"Shift<Btn2Up>",shift_btn2_up_func,NULL,TRUE);

    /* temp hack. this is for alternating between stereo & mono quickly. */
    xvw_add_action(xvimage,"Ctrl<Btn1Down>",stereo_mono_toggle_func,NULL,TRUE);

}

void
remove_xvevent_callbacks(xvobject xvimage)
{
    xvw_remove_event(xvimage, ExposureMask, render_create_gc, NULL);
    xvw_remove_event(xvimage, StructureNotifyMask, render_resize,NULL);

    /* the following event is added to the parent so that we can
     capture window-movement events.  */
    xvw_remove_event(xvw_parent(xvimage), StructureNotifyMask, render_configurerequest,NULL);
}

void
show_render_cursor()
{
   if (render_display == NULL)
      return;

    xvw_busy(render_xvobject,FALSE);
    XDefineCursor(render_display,render_window,zencursor);
    XFlush(render_display);

}

void
clear_render_cursor()
{
   if (render_display == NULL)
      return;

    XUndefineCursor(render_display,render_window);
    XFlush(render_display);

}

static float x_rot_relative,y_rot_relative,z_rot_relative;

void
set_xrot_relative(float f)
{
    x_rot_relative = f;
}
void
set_yrot_relative(float f)
{
    y_rot_relative = f;
}
void
set_zrot_relative(float f)
{
    z_rot_relative = f;
}
void
get_xrot_relative(float *f)
{
    *f = x_rot_relative;
}
void
get_yrot_relative(float *f)
{
    *f = y_rot_relative;
}
void
get_zrot_relative(float *f)
{
    *f = z_rot_relative;
}

void
set_render_deferred_mode(int mode)
{
    render_deferred_mode = mode;
}

int
get_render_deferred_mode()
{
    return(render_deferred_mode);
}

static int render_image_xposition= -1;
static int render_image_yposition = -1;

int
set_render_image_position(int x,int y)
{
    render_image_xposition = x;
    render_image_yposition = y;
    return(CHILL);
}

int
get_render_image_position(int *x, int *y)
{
    *x = render_image_xposition;
    *y = render_image_yposition;
    return(CHILL);
}

#if 0
int
move_render_window(int newx,
		     int newy,
		     Window w)
{
    /* this actually moves the parent of the render window,
     which will do the right thing. */
    Window parent;
    parent = xvw_window(xvw_parent(render_xvobject));

    if (w != NULL)
    {
	XMoveWindow(render_display,w,newx,newy);
	return(CHILL);
    }
	
    if (parent == NULL)
	return(WHACKED);
    XMoveWindow(render_display,parent,newx,newy);
#if 0
    xvobject toplevel;
    toplevel = xvw_toplevel(render_xvobject);
    xvw_set_attribute(toplevel, XVW_SHELL_X, newx);
    xvw_set_attribute(toplevel, XVW_SHELL_Y, newy);
    
#endif
}
#endif
int
wheres_render_window(int *xpos,
		       int *ypos)
{
    XWindowAttributes wa;
    Window parent;
    Window root_return,parent_return;
    Window *children_return;
    unsigned int nchildren_return;
    int status;
    
/*    parent = xvw_window(xvw_parent(render_xvobject)); */
/*    render_parent_window; */
    status = XQueryTree(render_display,render_parent_window,
			&root_return,&parent_return,
			&children_return,&nchildren_return);
    
    if (status == 0)
	return TRUE;

    XGetWindowAttributes(render_display,parent_return,&wa);
    *xpos = wa.x;
    *ypos = wa.y;

    if (nchildren_return)
	XFree(children_return);

    return TRUE;
}
