/*
 * Khoros: $Id: scene.c,v 1.1.2.1 1997/05/02 16:47:23 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: scene.c,v 1.1.2.1 1997/05/02 16:47:23 mirage Exp $";
#endif



/*
 * Copyright (c) 1996-1997,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 *
 * Fall 1996 - created
 *
 * Tue Mar 18 13:08:12 PST 1997 - version change from 1.0 to 1.0.1 of
 *    the scenefile.  added the imaging_sphere_subdivisions tag.  wes
 */
 


#include "render.h"
#include "camera.h"
#include "render_objs.h"
#include "scene.h"
#include "lights.h"

static char output_scene_filename[256],input_scene_filename[256];

static int s_have_input_scene_filename=FALSE;
static int s_have_output_scene_filename=FALSE;

static int lookup_token PROTO((char *));

#define NUM_TAGS 24

static char comment_tag[]={"#"};

static char image_size_tag[]={"image_size"};
static char image_background_color_tag[]={"image_background_color"};
static char imaging_renderer_tag[] = {"imaging_renderer"};
static char imaging_defer_tag[] = {"imaging_render_defer_mode"};
static char image_location_tag[]={"image_location"};
static char imaging_sphere_subdivisions_tag[]={"imaging_sphere_subdivisions"};

static char camera_fov_tag[]={"camera_fov"};
static char camera_type_tag[]={"camera_type"};
static char camera_eye_tag[]={"camera_eye"};
static char camera_at_tag[]={"camera_lookat"};
static char camera_up_tag[]={"camera_up"};
static char camera_stereo_tag[]={"camera_stereo"};
static char camera_vof_tag[]={"camera_vof"};
static char camera_stereo_cache_tag[]={"camera_cache"};
static char camera_clip_tag[]={"camera_clip"};

static char object_color_tag[]={"object_color"};
static char object_matrix_tag[]={"object_matrix"};
static char object_translate_tag[]={"object_translate"};
static char object_center_tag[]={"object_center"};
static char object_properties_tag[]={"object_properties"};
static char object_shader_tag[]={"object_shader"};

static char light_ambient_color_tag[]={"light_ambient_color"};
static char light_source_tag[]={"light_source"};


static char *tags[]=
{
    comment_tag,
    image_size_tag,
    image_background_color_tag,
    imaging_renderer_tag,
    imaging_defer_tag,
    image_location_tag,
    imaging_sphere_subdivisions_tag,
    
    camera_fov_tag,
    camera_type_tag,
    camera_eye_tag,
    camera_at_tag,
    camera_up_tag,
    camera_stereo_tag,
    camera_vof_tag,
    camera_stereo_cache_tag,
    camera_clip_tag,
    
    object_color_tag,
    object_matrix_tag,
    object_translate_tag,
    object_center_tag,
    object_properties_tag,
    object_shader_tag,
    
    light_ambient_color_tag,
    light_source_tag
};

/* the following defines are used to specify the parameter list for
 all functions called for each tag.  the first define sets up the
 function prototypes, and could be used in the procedure itself.  i
 chose to use the second define so that emacs would be tricked into
 highliting the C code correctly.  wes, 10 sep 96*/
#define PARMLIST() (PROTO(int line_no))
#define PLIST() int line_no

static int comment_func PARMLIST();
static int image_size_func PARMLIST();
static int image_background_color_func PARMLIST();
static int imaging_renderer_func PARMLIST();
static int imaging_defer_func PARMLIST();
static int image_location_func PARMLIST();
static int imaging_sphere_subdivisions_func PARMLIST();

static int camera_fov_func PARMLIST();
static int camera_type_func PARMLIST();
static int camera_eye_func PARMLIST();
static int camera_at_func PARMLIST();
static int camera_up_func PARMLIST();
static int camera_stereo_func PARMLIST();
static int camera_vof_func PARMLIST();
static int camera_stereo_cache_func PARMLIST();
static int camera_clip_func PARMLIST();

static int object_color_func PARMLIST();
static int object_matrix_func PARMLIST();
static int object_translate_func PARMLIST();
static int object_center_func PARMLIST();
static int object_properties_func PARMLIST();
static int object_shader_func PARMLIST();

static int light_ambient_color_func PARMLIST();
static int light_source_func PARMLIST();

static int unknown_func PARMLIST();

static int (*parsefunc[])(int) =
{
    comment_func,
    image_size_func,
    image_background_color_func,
    imaging_renderer_func,
    imaging_defer_func,
    image_location_func,
    imaging_sphere_subdivisions_func,
    
    camera_fov_func,
    camera_type_func,
    camera_eye_func,
    camera_at_func,
    camera_up_func,
    camera_stereo_func,
    camera_vof_func,
    camera_stereo_cache_func,
    camera_clip_func,
    
    object_color_func,
    object_matrix_func,
    object_translate_func,
    object_center_func,
    object_properties_func,
    object_shader_func,
    
    light_ambient_color_func,
    light_source_func,
    unknown_func
};

static void dump_object_info PROTO((kfile *,renderobj *));

static char scenefile_id[]={"Render-Scene-File 1.0.1"};

int
have_input_scene_filename()
{
    return(s_have_input_scene_filename);
}

int
have_output_scene_filename()
{
    return(s_have_output_scene_filename);
}

int
set_output_scene_filename(char *s)
{
    kstrcpy(output_scene_filename,s);
    s_have_output_scene_filename = TRUE;
    return(CHILL);
}

int
get_output_scene_filename (char *s)
{
    kstrcpy(s,output_scene_filename);
    return(CHILL);
}

int
set_input_scene_filename(char *s)
{
    kstrcpy(input_scene_filename,s);
    s_have_input_scene_filename = TRUE;
    return(CHILL);
}

int
get_input_scene_filename (char *s)
{
    kstrcpy(s,input_scene_filename);
    return(CHILL);
}

int
write_scene_file()
{
    kfile *f;
    char s[256];
    kstring date;
    int i;

    f = kfopen(output_scene_filename,"w");
    if (f == NULL)
    {
	kerror("GEOMETRY","render","I can't open the scene file named <%s>.  Skipping the writing of the scene file. ",output_scene_filename);
	return(WHACKED);
    }

    /* write a comment at the top of the file indicating the
     creation time. */
    kfprintf(f,"%s \n",comment_tag);
    kfprintf(f,"%s %s \n",comment_tag,scenefile_id);
    date = kcms_get_date();
    kfprintf(f,"%s %s \n",comment_tag,date);

    /* dump imaging parameters */
    {
	int width,height;	/* for image size */
	int xpos,ypos;		/* for image location */
	float br,bg,bb;		/* for background color */
	int renderer,d_mode;

	kfprintf(f,"%s \n%s Imaging Parameters\n%s \n",comment_tag,comment_tag,comment_tag);
	
	get_image_dims(&width,&height);
	kfprintf(f,"%s Image Dimensions \n",comment_tag);
	kfprintf(f,"%s %d %d \n",image_size_tag,width,height);

	get_background_color(&br,&bg,&bb);
	kfprintf(f,"%s Image Background Color - R G B in range 0..1 \n",comment_tag);
	kfprintf(f,"%s %g %g %g \n",image_background_color_tag,br,bg,bb);

	renderer = get_active_renderer();
	kfprintf(f,"%s %d \n",imaging_renderer_tag,renderer);

	d_mode = get_render_deferred_mode();
	kfprintf(f,"%s Render deferral mode: 0 == deferred, 1 == immediate.\n",comment_tag);
	kfprintf(f,"%s %d \n",imaging_defer_tag,d_mode);

	wheres_render_window(&xpos,&ypos);
	kfprintf(f,"%s %d %d \n",image_location_tag,xpos,ypos);

#if defined(__OGL_H) && defined(_USE_GFX_HARDWARE)
	kfprintf(f,"%s %d \n",imaging_sphere_subdivisions_tag,get_sphere_subdivisions());
#else
	/* is 6 a reasonable default?   //SK */
	kfprintf(f,"%s %d \n",imaging_sphere_subdivisions_tag,6);
#endif
	
    }

    /* dump camera parms */
    {
	float fov,h,y;
	int m,s,type;
	vertex_3d v;
	
	kfprintf(f,"%s \n%s Camera Parameters\n%s \n",comment_tag,comment_tag,comment_tag);

	get_fov(&fov);
	kfprintf(f,"%s Camera Field of View in degrees.\n",comment_tag);
	kfprintf(f,"%s %g \n",camera_fov_tag,fov);

	get_eye(&v);
	kfprintf(f,"%s Camera Eye point in world coords.\n",comment_tag);
	kfprintf(f,"%s %g %g %g \n",camera_eye_tag,v.v[0],v.v[1],v.v[2]);

	get_at(&v);
	kfprintf(f,"%s Camera Look-At point in world coords.\n",comment_tag);
	kfprintf(f,"%s %g %g %g \n",camera_at_tag,v.v[0],v.v[1],v.v[2]);

	get_up(&v);
	kfprintf(f,"%s Camera Up Vector in world coords.\n",comment_tag);
	kfprintf(f,"%s %g %g %g \n",camera_up_tag,v.v[0],v.v[1],v.v[2]);

	m = CAMERA_TYPE_MONOSCOPIC;
	s = CAMERA_TYPE_STEREOSCOPIC;
	type = get_camera_type();
	kfprintf(f,"%s Camera Type: %d=mono or %d=stereo\n",comment_tag,m,s);
	kfprintf(f,"%s %d \n",camera_type_tag,type);

	/* do stereo specific stuff. note: we're going to write out the
	 stereo parameters even if the current camera is not stereo. */
	{
	    float eye_sep, focal_length;
	    int lr_flip,toggle;

	    get_eye_separation(&eye_sep);
	    get_focal_length(&focal_length);
	    lr_flip = get_leftright_flip();
	    
	    kfprintf(f,"%s Stereo camera parameters: eye_separation focal_length left_right_flip \n",comment_tag);
	    kfprintf(f,"%s %g %g %d \n",camera_stereo_tag,eye_sep,focal_length,lr_flip);

	    get_toggle_cache(&toggle,&eye_sep);
	    kfprintf(f,"%s The following line contains internal variables which are NOT user-servicable. \n",comment_tag);
	    kfprintf(f,"%s %d %g \n",camera_stereo_cache_tag,toggle,eye_sep);
	}

	{
	    /* dump out the VOF */
	    int n;
	    char **strings;
	    n = get_num_vofs();
	    if (n > 0)
	    {
		n = get_current_vof();
		kfprintf(f,"%s The following VOF line is highly machine dependant. \n",comment_tag);
		kfprintf(f,"%s %d \n",camera_vof_tag,n);
	    }
	}

	/* dump clipping parms */
	get_user_hither(&h);
	get_user_yon(&y);
	kfprintf(f,"%s %g %g \n",camera_clip_tag,h,y);
    }

    /* dump object info */
    {
	renderobj *r;
	/* traverse the object tree, and write out stuff for each object */
	
	kfprintf(f,"%s \n%s Object Parameters\n%s \n",comment_tag,comment_tag,comment_tag);

	r = get_root_object();
	dump_object_info(f,r);
    }

    /* dump lights info */
    {
	vertex_3d c;
	int i,n,status;
	light_struct l;
	
	kfprintf(f,"%s \n%s Lighting Parameters\n%s \n",comment_tag,comment_tag,comment_tag);

	get_ambient_light_color(&c);
	kfprintf(f,"%s Ambient Light Color, [r,g,b] in [0..1] \n",comment_tag);
	kfprintf(f,"%s %g %g %g \n",light_ambient_color_tag,c.v[0],c.v[1],c.v[2]);

	status = get_num_lights(&n);

	kfprintf(f,"%s Following is a record for each light source.  It's format is: \n",comment_tag);
	kfprintf(f,"%s LIGHT_NUMBER LIGHT_TYPE LIGHT_ON/OFF LIGHT_POSITION LIGHT_DIRECTION LIGHT_COLOR \n",comment_tag);
	
	for (i=0;i<n;i++)
	{
	    /* gather all the parms for this light */
	    get_light_color(i,&(l.rgb[0]),&(l.rgb[1]),&(l.rgb[2]));
	    get_light_onoff(i,&(l.light_on));
	    get_light_type(i,&(l.light_type));
	    get_light_direction(i,&(l.direction));
	    get_light_position(i,&(l.position));

	    /* write one long-ass record for all this stuff. */
	    kfprintf(f,"%s %d %d %d %g %g %g  %g %g %g  %g %g %g \n",light_source_tag,i,l.light_type,l.light_on,l.position.v[0],l.position.v[1],l.position.v[2],l.direction.v[0],l.direction.v[1],l.direction.v[2],l.rgb[0],l.rgb[1],l.rgb[2]);
	}
    }

    kfclose(f);
    return(CHILL);
}

static char delims[]={" \"\n"};
static char name_delims[]={"\""};

int
read_scene_file()
{
    kfile *f;
    char s[256];
    char *token;
    int index;
    int line_no=1;

    f = kfopen(input_scene_filename,"r");
    if (f == NULL)
    {
	kerror("GEOMETRY","render","I can't open the scene file named <%s>.  Skipping the reading of the scene file. ",input_scene_filename);
	return(WHACKED);
    }
    while (!kfeof(f))
    {
	kmemset(s,0,256);
	kfgets(s,256,f);
	if (kfeof(f))
	    break;
/*	kfprintf(stderr,"%s",s); */
	token = kstrtok(s,delims);
	index = lookup_token(token);
	(*parsefunc[index])(line_no);
	line_no++;
    }
    return(CHILL);
}

static int
lookup_token(char *t)
{
    int i;

    for (i=0;i<NUM_TAGS;i++)
    {
	if (kstrncmp(t,tags[i],kstrlen(tags[i])) == 0)
	    break;
    }
    return(i);
}

static int
comment_func (PLIST())
{
    int i;
    i=0;
    return(CHILL);
}

static int
imaging_renderer_func (PLIST())
{
    char *t;
    int nr,n;
    renderer_struct **r;
    int i;

    t = kstrtok(NULL,delims);
    ksscanf(t,"%d",&nr);

    /* do a  quick sanity check */
    get_all_renderers_info(&n,&r);
    if ((nr < 0) || (nr > n))
    {
	kerror("GEOMETRY","render"," While reading the scene file, a renderer was requested which is not available.  Skipping this line and using the default renderer. \n");
	return(CHILL);
    }
    set_active_renderer(nr);
    return(CHILL);

}

static int
imaging_defer_func (PLIST())
{
    char *t;
    int m;

    t = kstrtok(NULL,delims);
    ksscanf(t,"%d",&m);
    set_render_deferred_mode(m);
    return(CHILL);
}

static int
image_location_func (PLIST())
{
    char *t;
    int x,y;

    t = kstrtok(NULL,delims);
    ksscanf(t,"%d",&x);
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%d",&y);

    set_render_image_position(x,y);
    return(CHILL);
}

static int
image_size_func (PLIST())
{
    int i;
    char *sx,*sy;
    int w,h;
    
    sx = kstrtok(NULL,delims);
    sy = kstrtok(NULL,delims);
    ksscanf(sx,"%d",&w);
    ksscanf(sy,"%d",&h);
    set_image_dims(w,h);
    return(CHILL);
}

static int
image_background_color_func (PLIST())
{
    int i;
    char *sr,*sg,*sb;
    float r,g,b;
    
    sr = kstrtok(NULL,delims);
    sg = kstrtok(NULL,delims);
    sb = kstrtok(NULL,delims);
    ksscanf(sr,"%g",&r);
    ksscanf(sg,"%g",&g);
    ksscanf(sb,"%g",&b);
    set_background_color(r,g,b);
    return(CHILL);
}

static int
imaging_sphere_subdivisions_func (PLIST())
{
    int i;
    char *t;
    int s;
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%d",&s);
#if defined(__OGL_H) && defined(_USE_GFX_HARDWARE)
    set_sphere_subdivisions(s);
#endif
    return(CHILL);
}

static int
camera_fov_func(PLIST())
{
    float fov;
    char *sfov;

    sfov = kstrtok(NULL,delims);
    ksscanf(sfov,"%g",&fov);
    set_fov(fov);
    compute_view_xform();
    return(CHILL);
}

static int
camera_vof_func(PLIST())
{
    char *t;
    int n;

    t = kstrtok(NULL,delims);
    ksscanf(t,"%d",&n);
    set_current_vof(n);
    return(CHILL);
}

static int
camera_eye_func(PLIST())
{
    vertex_3d v;
    char *t;

    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(v.v[0]));
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(v.v[1]));
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(v.v[2]));

    set_eye(&v);
    compute_view_xform();
    return(CHILL);
}

static int
camera_at_func(PLIST())
{
    vertex_3d v;
    char *t;

    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(v.v[0]));
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(v.v[1]));
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(v.v[2]));

    set_at(&v);
    compute_view_xform();
    return(CHILL);
}

static int
camera_up_func(PLIST())
{
    vertex_3d v;
    char *t;

    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(v.v[0]));
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(v.v[1]));
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(v.v[2]));

    set_up(&v);
    compute_view_xform();
    return(CHILL);
}

static int
camera_type_func(PLIST())
{
    char *t;
    int type;

    t = kstrtok(NULL,delims);
    ksscanf(t,"%d",&type);
    set_camera_type(type);
    return(CHILL);
}

static int
camera_stereo_func(PLIST())
{
    char *t;
    int lr_flip;
    float eye_sep,focal_length;

/*	    kfprintf(f,"%s Stereo camera parameters: eye_separation focal_length left_right_flip \n",comment_tag); */
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&eye_sep);

    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&focal_length);

    t = kstrtok(NULL,delims);
    ksscanf(t,"%d",&lr_flip);

    set_focal_length(focal_length);
    set_eye_separation(eye_sep);
    set_leftright_flip(lr_flip);
    compute_stereo_camera_parms();
    return(CHILL);

}

static int
camera_stereo_cache_func(PLIST())
{
    char *t;
    int toggle;
    float eye_sep;

/*	    kfprintf(f,"%s Stereo camera parameters: eye_separation focal_length left_right_flip \n",comment_tag); */
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%d",&toggle);

    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&eye_sep);

    set_toggle_cache(toggle, &eye_sep);
    compute_stereo_camera_parms();
    return(CHILL);
}

static int
camera_clip_func(PLIST())
{
    char *t;
    float h,y;

    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&h);

    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&y);

    set_user_hither(h);
    set_user_yon(y);
    return(CHILL);
}

static int
object_center_func(PLIST())
{
    char *t;
    renderobj *robj;
    int status;

    t = kstrtok(NULL,name_delims);	/* grab the next thing...should be
				 an object name. */
    status = find_renderobj (t, &robj);
    if (status == WHACKED)
    {
	kerror("GEOMETRY","render", "while reading the scene file, the object named %s was referenced but i can't find it.  skipping this line of the scene file and continuing...",t);
	return(1);
    }
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(robj->center.v[0]));
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(robj->center.v[1]));
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(robj->center.v[2]));
    return(CHILL);
}

static int
object_matrix_func(PLIST())
{
    char *t;
    renderobj *robj;
    int i,j;
    int status;

    t = kstrtok(NULL,name_delims);	/* grab the next thing...should be
				 an object name. */
    status = find_renderobj (t, &robj);
    if (status == WHACKED)
    {
	kerror("GEOMETRY","render", "while reading the scene file, the object named %s was referenced but i can't find it.  skipping this line of the scene file and continuing...",t);
	return(1);
    }
    for (j=0;j<4;j++)
	for (i=0;i<4;i++)
	{
	    t = kstrtok(NULL,delims);
	    ksscanf(t,"%g",&(robj->xfrm.m[j][i]));
	}
    return(CHILL);
}

static int
object_translate_func(PLIST())
{
    char *t;
    renderobj *robj;
    int status;

    t = kstrtok(NULL,name_delims);	/* grab the next thing...should be
				 an object name. */
    status = find_renderobj (t, &robj);
    if (status == WHACKED)
    {
	kerror("GEOMETRY","render", "while reading the scene file, the object named %s was referenced but i can't find it.  skipping this line of the scene file and continuing...",t);
	return(1);
    }

    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(robj->translate_vect.v[0]));
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(robj->translate_vect.v[1]));
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(robj->translate_vect.v[2]));
    return(CHILL);
}

static int
object_shader_func(PLIST())
{
    char *t;
    renderobj *robj;
    int status;

    t = kstrtok(NULL,name_delims);	/* grab the next thing...should be
				 an object name. */
    status = find_renderobj (t, &robj);
    if (status == WHACKED)
    {
	kerror("GEOMETRY","render", "while reading the scene file, the object named %s was referenced but i can't find it.  skipping this line of the scene file and continuing...",t);
	return(1);
    }
    t = kstrtok(NULL,delims);
    ksscanf(t,"%d",&(robj->shader));
    return(CHILL);
}

static int
object_properties_func(PLIST())
{
    char *t;
    surface_attribs a;
    renderobj *robj;
    int status;

    t = kstrtok(NULL,name_delims);	/* grab the next thing...should be
				 an object name. */
    status = find_renderobj (t, &robj);
    if (status == WHACKED)
    {
	kerror("GEOMETRY","render", "while reading the scene file, the object named %s was referenced but i can't find it.  skipping this line of the scene file and continuing...",t);
	return(1);
    }

    /* get ambient */
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(a.amb));
    
    /* get diffuse */
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(a.diff));
    
    /* get specular coefficient */
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(a.spec));
    
    /* get specular exponent */
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(a.spec_exp));
    
    /* get specular metal */
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(a.spec_metal));

    kmemcpy(&(robj->attribs),&a,sizeof(surface_attribs));

    return(CHILL);
}

static int
object_color_func(PLIST())
{
    char *t;
    renderobj *robj;
    int status;
    float r,g,b,a;
    

    t = kstrtok(NULL,name_delims);	/* grab the next thing...should be
				 an object name. */
    status = find_renderobj (t, &robj);
    if (status == WHACKED)
    {
	kerror("GEOMETRY","render", "while reading the scene file, the object named %s was referenced but i can't find it.  skipping this line of the scene file and continuing...",t);
	return(1);
    }
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&r);
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&g);
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&b);
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&a);

    set_object_rgba(robj,r,g,b,a);
    return(CHILL);
}

static int
unknown_func (PLIST())
{
    kerror("GEOMETRY","render","While reading the scene file, a bogus line was encountered on line %d. Skipping it and continuing. \n",line_no);
    return(CHILL);
}

static void
dump_object_info (kfile *f,
		  renderobj *r)
{
    int i,j;
    char name[KLENGTH];
    /* first, dump out this object's info... */

    /* dump the object color */
    kmemset(name,0,KLENGTH);
    name[0] = '"';
    kstrcat(name,r->name);
    name[kstrlen(name)] = '"';

    /* dump object surface properties */
    kfprintf(f,"%s Object Properties are: ambient diffuse specular_coefficient specular_exponent specular_metal \n",comment_tag);
    kfprintf(f,"%s %s %g %g %g %g %g \n",object_properties_tag,name,r->attribs.amb,r->attribs.diff,r->attribs.spec,r->attribs.spec_exp,r->attribs.spec_metal);
    
    kfprintf(f,"%s %s %g %g %g %g \n",object_color_tag,name,r->colors.r,
	     r->colors.g,r->colors.b,r->colors.a);

    /* dump the object shader */
    kfprintf(f,"%s %s %d \n",object_shader_tag,name,r->shader);
    
    /* dump the object matrix */
    kfprintf(f,"%s %s ",object_matrix_tag,name);
    for (j=0;j<4;j++)
	for (i=0;i<4;i++)
	    kfprintf(f,"%g ",r->xfrm.m[j][i]);
    kfprintf(f,"\n");

    /* dump the object translation vector */
    kfprintf(f,"%s %s %g %g %g \n",object_translate_tag,name,r->translate_vect.v[0],r->translate_vect.v[1],r->translate_vect.v[2]);

    /* dump the object center point */
    kfprintf(f,"%s %s %g %g %g \n",object_center_tag,name,r->center.v[0],r->center.v[1],r->center.v[2]);

    /* then do all it's children recursively */
    for (i=0;i<r->nchildren;i++)
	dump_object_info(f,r->children[i]);

}

static int
light_ambient_color_func (PLIST())
{
    char *t;
    vertex_3d c;

    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(c.v[0]));
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(c.v[1]));
    
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(c.v[2]));

    set_ambient_light_color(&c);
    return(CHILL);
}

static int
light_source_func(PLIST())
{
    char *t;
    int n,type,on;
    vertex_3d dir,pos;
    float rgb[3];
#if 0
    kfprintf(f,"%s LIGHT_NUMBER LIGHT_TYPE LIGHT_ON/OFF LIGHT_POSITION LIGHT_DIRECTION LIGHT_COLOR \n",comment_tag);
#endif

    /* light number */
    t = kstrtok(NULL,delims);
    ksscanf(t,"%d",&n);

    /* light type */
    t = kstrtok(NULL,delims);
    ksscanf(t,"%d",&type);
    
    /* light on? */
    t = kstrtok(NULL,delims);
    ksscanf(t,"%d",&on);

    /* light position */
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(pos.v[0]));
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(pos.v[1]));
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(pos.v[2]));

    /* light direction */
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(dir.v[0]));
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(dir.v[1]));
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",&(dir.v[2]));

    /* light color */
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",rgb);
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",rgb+1);
    t = kstrtok(NULL,delims);
    ksscanf(t,"%g",rgb+2);

    /* ok, now set the light in the renderer to reflect all this stuff. */
    
    set_light_color(n,rgb[0],rgb[1],rgb[2]);
    set_light_onoff(n,on);
    set_light_type(n,type);
    set_light_direction(n,&(dir));
    set_light_position(n,&(pos));
    return(CHILL);
}
