
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 
/***********************************************************************


  Tue Apr  2 13:51:02 PST 1996, wes - cobbled on a bitmap_mask to the render
  text structure.  the bitmap mask is an equivalent representation of the
  X-rendered string to the "mask" (which is 8-bit), and is used in
  OpenGL.

***********************************************************************/

#include "render.h"
#include "render_objs.h"
#include "zbuffer.h"

unsigned char **rm_build_text_mask PROTO((char *string,char *font,render_text_struct *));
unsigned char **rm_malloc_2d_byte_buffer PROTO((int width,int height));
int free_2d_byte_buffer PROTO((char **c));

/* --  text primitive setup routines for the render software renderer -- */


unsigned char **
rm_malloc_2d_byte_buffer(int width,
			 int height)
{
    /* malloc and return a 2d byte array. */
    int i,j;
    unsigned char **c2;
    unsigned char *c;

    c = (unsigned char *)kmalloc(sizeof(unsigned char)*width*height);
    c2 = (unsigned char **)kmalloc(sizeof(unsigned char *)*height);

    for (i=0;i<height;i++)
	c2[i] = c+(i*width);

    return(c2);
}

int
rm_free_2d_byte_buffer(char **c)
{
    /* free a 2d byte buffer.  it is assumed that this buffer has
     been malloc'd by malloc_2d_byte_buffer(). */
    
    kfree(c[0]);
    return(CHILL);
}

/* these "fallback fonts" are used if the user-specified font is bogus
 or doesn't exist. these fontnames follow the X font naming convention. */

#define NFALLBACK_FONTS 3
char fallback_fonts[NFALLBACK_FONTS][80] =
{
    {"*-times-*-r-*14*"},	/* try to grab a 14 point times-roman font */
    {"*times-roman*"},		/* try to grab a plain old times-roman font */
    {"fixed"}			/* grab a really ugly font */
};

#define DEPTH 8

#define SET_BIT(a,j,i) (a[j][i/8] |=  (1 << (7-(i%8))));

unsigned char **
build_text_mask(char *string,
		char *font,
		render_text_struct *rmt)
{
    /* modified 2 april 1996.  will also set the bitmap_mask field within
     the rmt structure. */
    
    Display *render_display; /* assume it's been XOpenDisplay'd */
    XFontStruct *xfs;
    XGCValues values;
    Pixmap    pixmap;
    GC        gc;
    int computed_width,computed_height,computed_base;
    XImage	*image;
    unsigned char **render_mask;
    int bitmap_width_in_bytes;
    unsigned char **bitmap_mask;

    render_display = XOpenDisplay(NULL);
    
    /* first, load a font...maybe... */
    if (font == NULL)
	xfs = NULL;
    else
	xfs = XLoadQueryFont(render_display,font);
    
    if (xfs == NULL)	/* try to go for a fallback font */
    {
	int i;
#if 0	
	fprintf(stderr," the asked-for font, <%s>, doesn't exist.  searching in fallback fonts...\n",font);
#endif	    
	for (i=0;i<NFALLBACK_FONTS;i++)
	{
#if 0
	    fprintf(stderr," trying the font: <%s> \n",fallback_fonts[i]);
#endif
	    xfs = XLoadQueryFont(render_display,fallback_fonts[i]);
	    if (xfs != NULL)
		break;
	}
        if (i == NFALLBACK_FONTS)
	{
	    fprintf(stderr," it's not your lucky day. i can't find a font and am going to exit now.  bye! ..\n");
	    exit(-1);
	}
    }
    values.font = xfs->fid;

    /* second, figure out how big the pixmap will need to be to
     contain the rendered text. */

    {
	int direction_return,font_ascent_return,font_descent_return;
	XCharStruct overall_return;
	    
	XQueryTextExtents(render_display,
			  values.font,
			  string,
			  strlen(string),
			  &direction_return,
			  &font_ascent_return,
			  &font_descent_return,
			  &overall_return);

	computed_width = overall_return.width;
	computed_height = overall_return.ascent + overall_return.descent;
	computed_base = overall_return.ascent;

	rmt->r_width = computed_width;
	rmt->r_height = computed_height;
		
    }

    /* X malloc the pixmap. */
    pixmap = XCreatePixmap(render_display, DefaultRootWindow(render_display),
			   computed_width,computed_height,DEPTH); 
#if 0
    values.foreground = WhitePixel(render_display, DefaultScreen(render_display));
    values.background = BlackPixel(render_display, DefaultScreen(render_display));
#endif
    values.foreground = 1;
    values.background = 0;
	
    gc = XCreateGC(render_display, pixmap, GCForeground | GCBackground | GCFont,
		   &values);
    XDrawImageString(render_display, pixmap, gc, 0, computed_base,string,
		     strlen(string));
    image = XGetImage(render_display,pixmap,0,0,computed_width,computed_height,
		      255,ZPixmap);

    render_mask = rm_malloc_2d_byte_buffer(computed_width,computed_height);

    bitmap_width_in_bytes = (computed_width / 8) +
	((computed_width % 8 == 0) ? 0 : 1);
    bitmap_mask = rm_malloc_2d_byte_buffer(bitmap_width_in_bytes,
					   computed_height);
    rmt->bitmap_mask = bitmap_mask;
    kmemset(bitmap_mask[0],0,bitmap_width_in_bytes*computed_height);
    
    /* now grab the pixels from the XImage.  there's probably a more
     efficient way to do this, but since we're not doing it very often,
     i haven't optimized it. */
    {
	int i,j;
	unsigned long p;
	
	for (j=0;j<computed_height;j++)
	    for (i=0;i<computed_width;i++)
	    {
		p = XGetPixel(image,i,j);
		if (p)
		{
		    render_mask[j][i] = (unsigned char)0xff;
		    SET_BIT(bitmap_mask,computed_height-j-1,i);
		}
		else
		    render_mask[j][i] = 0;
		/* don't need to explicitly clear the bit..already  */
#if 0
		render_mask[j][i] = (unsigned char)p;
#endif
	    }
    }

    XFreePixmap(render_display, pixmap);

    return(render_mask);
}
		

