
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 *
 *
 * Mon Mar 24 18:54:27 PST 1997 - reorder the output primitives in the
 *  dump_xform_output() thingamabopper so that the root object's transformation
 *  matrix is first, and the currently selected object is second. if the
 *  currently selected object is "root" then we only dump one matrix.
 */
 

/*
 * Khoros: $Id: vrxfrm.c,v 1.1 1997/04/19 00:15:57 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vrxfrm.c,v 1.1 1997/04/19 00:15:57 mirage Exp $";
#endif


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>             _static_routines()
   >>>>  Public:
   >>>>             public_routines()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "render.h"
#include "render_objs.h"
#include "vrxfrm.h"

#include <math.h>

static char vr_output_transport_name[256];
static int initted=0;

int
init_xform_output(char *transport_name)
{
  if ((transport_name == NULL) || (transport_name[0] == '\0'))
    initted = 0;
  else
  {
    kstrcpy(vr_output_transport_name,transport_name);
    initted = 1;
  }
  return(CHILL);
}

int
have_named_vr_events()
{
    if (initted)
	return(TRUE);
    else
	return(FALSE);
}

int
set_vr_transport_name (char *s)
{
    kstrcpy(vr_output_transport_name,s);
    initted = 1;
    return(CHILL);
}

int
get_vr_transport_name (char *s)
{
    kstrcpy(s,vr_output_transport_name);
    return(CHILL);
}

int
dump_xform_output(void)
{
    kgeom_object *o;
    matrix4x4 view,model;
    renderobj *current_obj;
    char test[KLENGTH];
    float *tvect;
    kgeom_object_xform *object_xform;
    kgeom_view_xform   *view_xform;
    kgeom_camera_orientation *camera_orient;
    
    /*
     *
     * Modified 
     * Mon Mar 24 18:54:27 PST 1997 : new order is
     * 1. the transformation of the root object
     * 2. the transformations of the currently selected object, if it's
     *    not the root object.
     * 1. the rotation matrix associated with the current camera settings.
     *    NOTE: no translational or scaling values are present in the
     *          camera transformation matrix.
     * 3. camera position & orientation parameters.
     */


    if (initted == 0 || !knegotiate_check(vr_output_transport_name))
	return(1);

    /* open the output object... */
    o = kgeom_new_object();

    /* dump the xforms associated with the root object. */
    {
	renderobj *r;
	/* the current object isn't "root".  dump some more stuff. */
	matrix4x4 minusC,C;
	identity_4x4(&minusC);
	identity_4x4(&C);

	object_xform = (kgeom_object_xform *)
	    kgeom_new_primitive(KGEOM_OBJECT_XFORM);


	r = get_root_object();
	kmemcpy(test,r->name,kstrlen(r->name));
	test[kstrlen(r->name)] = '\0';

	minusC.m[3][0] = -1. * r->center.v[0];
	minusC.m[3][1] = -1. * r->center.v[1];
	minusC.m[3][2] = -1. * r->center.v[2];

	C.m[3][0] = r->center.v[0];
	C.m[3][1] = r->center.v[1];
	C.m[3][2] = r->center.v[2];

	mmul_4x4(&minusC,&(r->xfrm),&model);
	mmul_4x4(&model,&C,&model);

	tvect = &(r->translate_vect.v[0]);
	
	object_xform->name = kstrdup(test);
	object_xform->relative = KRELATIVE;
	kmemcpy(object_xform->matrix, (float *) &model, sizeof(float)*16);
	kmemcpy(object_xform->translate, tvect, sizeof(float)*3);

	kgeom_add_primitive(o, (kgeom_primitive *)object_xform);
    }

      /* dump the xforms associated with the currently selected object,
	 if it's not the root object. */
    {
	/* Thu Oct 31 15:04:42 PST 1996 - the obj->xfrm matrix doesn't
	   contain any translational components.  the matrix which we output
	   will have it's translational components just written into the
	   bottom row in columns 0..2. */
	
	current_obj = get_current_object();

	kmemcpy(test,current_obj->name,strlen(current_obj->name));
	test[strlen(current_obj->name)] = '\0';

	if (kstrcmp(test,"root") != 0)
	{
	    /* the matrix that we're going to dump out is constructed
	       from:
	       1. C - the object's "center" point,
	       2. SR - the scale & rotation applied to the object
	       by computing:
	       M = -CSRC
	       */

	    matrix4x4 minusC,C;
	    identity_4x4(&minusC);
	    identity_4x4(&C);

	    object_xform = (kgeom_object_xform *)
		kgeom_new_primitive(KGEOM_OBJECT_XFORM);
 
	    minusC.m[3][0] = -1. * current_obj->center.v[0];
	    minusC.m[3][1] = -1. * current_obj->center.v[1];
	    minusC.m[3][2] = -1. * current_obj->center.v[2];

	    C.m[3][0] = current_obj->center.v[0];
	    C.m[3][1] = current_obj->center.v[1];
	    C.m[3][2] = current_obj->center.v[2];

	    mmul_4x4(&minusC,&(current_obj->xfrm),&model);
	    mmul_4x4(&model,&C,&model);

	    tvect = &(current_obj->translate_vect.v[0]);

	    kmemcpy(test,current_obj->name,strlen(current_obj->name));
	    test[strlen(current_obj->name)] = '\0';

	    object_xform->name = kstrdup(test);
	    object_xform->relative = KRELATIVE;
	    kmemcpy(object_xform->matrix, (float *) &model, sizeof(float)*16);
	    kmemcpy(object_xform->translate, tvect, sizeof(float)*3);
	    
	    kgeom_add_primitive(o, (kgeom_primitive *)object_xform);
	}
    }


  /* next, do the transformation matrix associated with the camera */

    view_xform = (kgeom_view_xform *) 
	kgeom_new_primitive(KGEOM_VIEW_XFORM);

    get_view_rotation_matrix(&view);
    kmemcpy(view_xform->matrix, (float *) &view, sizeof(float)*16);
    kgeom_add_primitive(o, (kgeom_primitive *)view_xform);

    {
	/* do the camera stuff.. */
	vertex_3d leye,lat,lup;
	float fov;
	int projection;
	float hither,yon;
	float focal_len;
	int i;
      
	get_eye(&leye);
	get_at(&lat);
	get_up(&lup);
	get_fov(&fov);
	get_projection(&projection);
	get_user_yon(&yon);
	get_user_hither(&hither);

	focal_len = 0;
	for (i=0;i<3;i++)
	    focal_len += ((leye.v[i]-lat.v[i])*(leye.v[i]-lat.v[i]));
	focal_len = (float)ksqrt((double)focal_len);
      
	camera_orient = (kgeom_camera_orientation *) 
	    kgeom_new_primitive(KGEOM_CAMERA_ORIENTATION);

	camera_orient->field_of_view = fov;
	camera_orient->hither        = hither;
	camera_orient->yon           = yon;
	camera_orient->projection    = projection;
	camera_orient->is_stereo     = FALSE; /* someday .. */
	camera_orient->eye_distance  = 10.0;
	camera_orient->focal_length  = focal_len;
	kmemcpy(camera_orient->eye, (float *) &(leye.v[0]), sizeof(float)*3);
	kmemcpy(camera_orient->at,  (float *) &(lat.v[0]), sizeof(float)*3);
	kmemcpy(camera_orient->up,  (float *) &(lup.v[0]), sizeof(float)*3);

	kgeom_add_primitive(o, (kgeom_primitive *)camera_orient);
    }

    /*fprintf(stderr,"stuffing vroutput...\n");*/
    kgeom_write_object(o, vr_output_transport_name);
    kgeom_blast_object(o);
}

int
vr_process_object(char *obj_file)
{
    kgeom_object *o;
    int i,num;
    int rstat = WHACKED;

    /* this routine will return WHACKED if no net changes occured.  a
     value of CHILL corresponds roughly to a need to rerender. */

    o = kgeom_read_object(obj_file);

    num = kgeom_number_primitives(o);

    for (i=0;i<num;i++)
    {
        kgeom_primitive *prim = kgeom_get_primitive(o, i);
       
        if (prim->type == KGEOM_OBJECT_XFORM)
	{
	    float *matrix=NULL;
	    int relative;
	    char *name=NULL;
	    renderobj *rmobj;
	    kgeom_object_xform *object_xform = (kgeom_object_xform *) prim;

	    matrix   = object_xform->matrix;
	    relative = object_xform->relative;
	    name     = object_xform->name;
#if 0
	    /* isnan() generates a compiler warning on solaris. i'm
	     commenting this code out because it has never been executed
	     except during initial phases of development when cantata
	     would cause garbage to be sent in as a vr event. this
	     has been cleared up.  if you suspect garbage in the
	     vr event stream, uncomment this code and recompile, ignoring
	     the compile warnings.  the code works...*/
	    {
		for (i=0;i<16;i++)
		    if (isnan(matrix[i]))
			break;
		if (i != 16)
	        {
		    kerror("GEOMETRY","render","render: NaN detected in input vr event. Skipping this event and continuing.\n");
		    return(WHACKED);
		}
	    }
#endif

	    if ((name == NULL) || (name[0] == '\0'))
	        rmobj = get_root_object();
	    else
	        find_renderobj(name, &rmobj);
	    
	    if (rmobj == NULL)
	    {
	        kerror("GEOMETRY","render", "i can't find the object named <%s> to which i would have applied a KGEOM_OBJECT_XFORM operation.  skipping the request and continuing. ",name);
	    }
	    else
	    {
	        if (relative == KABSOLUTE)
		    kmemcpy(&(rmobj->xfrm),matrix,sizeof(matrix4x4));
		else
		{
		    mmul_4x4(&(rmobj->xfrm),(matrix4x4 *)matrix,
			     &(rmobj->xfrm));
#if 0
		    {
			int i,j,offset=0;
			fprintf(stderr," matrix \n");
			for (j=0;j<4;j++)
			{
			    for (i=0;i<4;i++)
				fprintf(stderr," %g ",matrix[offset++]);
			    fprintf(stderr," \n");
			}
		    }
#endif
		}
		rstat = CHILL;
	    }
	    
	}
	else if (prim->type == KGEOM_CAMERA_ORIENTATION)
	{
	    /* read in new camera parameters. */
	    float fov,hither,yon;
	    int projection;
	    int is_stereo;
	    float eye_sep,foc_len;
	    int relative;
	    float *eye=NULL,*at=NULL,*up=NULL;
	    kgeom_camera_orientation *camera_orient = 
	       (kgeom_camera_orientation *) prim;
   
	    eye = camera_orient->eye;
	    at  = camera_orient->at;
	    up  = camera_orient->up;
	    fov = camera_orient->field_of_view;
	    hither = camera_orient->hither;
	    yon    = camera_orient->yon;
	    projection = camera_orient->projection;
	    is_stereo  = camera_orient->is_stereo;
	    eye_sep = camera_orient->eye_distance;
	    foc_len = camera_orient->focal_length;

	    /* there is no notion of a "relative" camera operation. some
	     other module, namely gvr_xform_cam, deals with all that crap.
	     so, we just copy the values striaght into the guts of render. */
	    
	    set_eye((vertex_3d *)eye);
	    set_at((vertex_3d *)at);
	    set_up((vertex_3d *)up);
	    set_fov(fov);
	    set_projection(projection);
	    set_user_yon(yon);
	    set_user_hither(hither);

	    compute_view_xform();
	    rstat = CHILL;
	}
	else if (prim->type == KGEOM_VIEW_XFORM)
	  /* in anticipation of this working someday */
	  kerror("GEOMETRY","render","vr_process_object can't deal with a KGEOM_VIEW_XFORM operation.");
    }
    kgeom_blast_object(o);
    return(rstat);
}
