 /*
  * Khoros: $Id: icolorspace.c,v 1.3.2.3 1997/05/31 19:10:09 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: icolorspace.c,v 1.3.2.3 1997/05/31 19:10:09 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for icolorspace
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "icolorspace.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Performs the actual data conversion of a image colorspace.
|
|       Purpose: main program for icolorspace
|
|         Input:
|		char *clui_info->i_file; {First Input data object}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		int clui_info->type_toggle; {Destination colorspace model.}
|		int clui_info->type_flag; {TRUE if -type specified}
|
|		int   clui_info->outputtype_list; {List of possible output datatypes.}
|		char *clui_info->outputtype_label;
|		int   clui_info->outputtype_flag; {TRUE if -outputtype specified}
|
|		char *clui_info->o_file; {Resulting output data object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->set_internal_only_flag; {TRUE if -set_internal_only specified}
|
|        Output:
|       Returns:
|
|    Written By: A. Christopher Moorman
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	kobject srcobj;			/* Khoros Input object.		*/
	kobject destobj;		/* Khoros Output object.	*/
	int dest_color_model;		/* Destination colormodel.	*/
	int typeconv; 			/* Destination output datatype	*/
/* -main_variable_list_end */

	khoros_init(argc, argv, "IMAGE", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$IMAGE/objects/kroutine/icolorspace");
	kexit_handler(icolorspace_free_args, NULL);

/* -main_get_args_call */
	kclui_init("IMAGE", "icolorspace", KGEN_KROUTINE,  &clui_uis_spec,
		   icolorspace_usage_additions, icolorspace_get_args,
 		   icolorspace_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
       /* Open the input object, report an error if object cannot be opened.
        */
        if (!(srcobj = kpds_open_input_object(clui_info->i_file)))
        {
	   kset_errno(KINVALID_FILE);
           kerror("kaps", "kcltrans",
                  "Failed to open input object '%s'.", clui_info->i_file);
           kexit(KEXIT_FAILURE);
        }


       /* Open the output object, report an error if object cannot be opened.
        */
        if (!(destobj = kpds_open_output_object(clui_info->o_file)))
        {
           kpds_close_object(srcobj);
	   kset_errno(KINVALID_FILE);
           kerror("kaps", "kcltrans",
                  "Failed to open output object '%s'.", clui_info->o_file);
           kexit(KEXIT_FAILURE);
        }

       /* Make a copy of the source object to the destination object. 
	*/
	kpds_copy_object(srcobj, destobj);


       /* Check to see if an output datattype has been specified.  If one
	* was, then set the output datatype to that value.
	*/
	if(clui_info->outputtype_flag)
	{
           if(clui_info->outputtype_list > 1)
           {
              typeconv = kpow(2,(clui_info->outputtype_list-2));
              kpds_set_attribute(destobj,KPDS_VALUE_DATA_TYPE,typeconv);
	      if(kpds_query_map(destobj))
                kpds_set_attribute(destobj,KPDS_MAP_DATA_TYPE,typeconv);
           }
	}


       /* This section performs a series of attribute conversion to modify
	* the colormodel.  If the user wishes to convert only the type 
	* without modifying the data, then a toggle would be provided
	* to leave the data untouched.
	*/
	if(clui_info->set_internal_only_flag)
	{
	   if(clui_info->type_toggle <= KUCSUVW)
	      dest_color_model = clui_info->type_toggle;
	   else
	      dest_color_model = clui_info->type_toggle + 1;

	   kdms_set_attribute(destobj, NULL, "colorspace", 
		dest_color_model);
	}
	else
	{
	   if(clui_info->type_toggle <= KUCSUVW)
	      dest_color_model = clui_info->type_toggle;
	   else
	      dest_color_model = clui_info->type_toggle + 1;

           kpds_set_attribute(destobj, KCOLOR_COLORSPACE, dest_color_model);
	}
/* -main_before_lib_call_end */

/* -main_library_call */
/* -main_library_call_end */

/* -main_after_lib_call */
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: icolorspace_usage_additions
| 
|       Purpose: Prints usage additions in icolorspace_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname icolorspace
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void icolorspace_usage_additions(void)
{
	kfprintf(kstderr, "\tPerforms the actual data conversion of a image colorspace.\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: icolorspace_free_args
| 
|       Purpose: Frees CLUI struct allocated in icolorspace_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname icolorspace
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
icolorspace_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->outputtype_label);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
