 /*
  * Khoros: $Id: idissolve.c,v 1.4.2.2 1997/05/31 19:11:19 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: idissolve.c,v 1.4.2.2 1997/05/31 19:11:19 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for idissolve
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "idissolve.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - no short description specified
|
|       Purpose: main program for idissolve
|
|         Input:
|		char *clui_info->i_file; {First Input data object}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Resulting output data object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		float clui_info->scale_float; {float}
|		int   clui_info->scale_flag; {TRUE if -scale specified}
|
|        Output:
|       Returns:
|
|    Written By: Thomas H. Robey
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
   kobject src_obj=NULL, dst_obj=NULL;
   double factor = 1.0;
/* -main_variable_list_end */

	khoros_init(argc, argv, "IMAGE", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$IMAGE/objects/kroutine/idissolve");
	kexit_handler(idissolve_free_args, NULL);

/* -main_get_args_call */
	kclui_init("IMAGE", "idissolve", KGEN_KROUTINE,  &clui_uis_spec,
		   idissolve_usage_additions, idissolve_get_args,
 		   idissolve_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
   if ((src_obj=kpds_open_input_object(clui_info->i_file)) == KOBJECT_INVALID)
   {
      kerror(NULL,"idissolve","Cannot open input object %s",clui_info->i_file);
      kexit(KEXIT_FAILURE);
   }
   if ((dst_obj=kpds_open_output_object(clui_info->o_file)) == KOBJECT_INVALID)
   {
      kerror(NULL,"idissolve","Cannot open output object %s.",
             clui_info->o_file);
      kexit(KEXIT_FAILURE);
   }
   factor = (double)clui_info->scale_float;
/* -main_before_lib_call_end */

/* -main_library_call */
   if (!(lidissolve(src_obj, dst_obj, factor)))
   {
      kpds_close_object(src_obj);
      kpds_close_object(dst_obj);
      kexit(KEXIT_FAILURE);
   }
/* -main_library_call_end */

/* -main_after_lib_call */
   kpds_close_object(src_obj);
   kpds_close_object(dst_obj);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: idissolve_usage_additions
| 
|       Purpose: Prints usage additions in idissolve_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname idissolve
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void idissolve_usage_additions(void)
{
	kfprintf(kstderr, "\tno short description specified\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: idissolve_free_args
| 
|       Purpose: Frees CLUI struct allocated in idissolve_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname idissolve
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
idissolve_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
