 /*
  * Khoros: $Id: licomposite.c,v 1.2.2.2 1997/05/24 20:06:07 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: licomposite.c,v 1.2.2.2 1997/05/24 20:06:07 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for icomposite
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	licomposite
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: licomposite - Compositing routine for images
* 
*       Purpose: This library function implements a variety of 
*                compositing routines for combining images as 
*                detailed by Thomas Porter and Tom Duff in 
*                "Compositing Digital Images", Computer Graphics,
*                ACM, Vol. 18, 1984.
*
*         Input: src1_obj - First input image
*                src2_obj - Second input image
*                flag - indicates the type of compositing
*                   aPLUSb - A, B where the other do not exist,
*                            and both A and B where both exist.
*                            (default)
*                   aOVERb - A, B where the other do not exist,
*                            and A where both exist.
*                   bOVERa - A, B where the other do not exist,
*                            and B where both exist.
*                   aINb   - A where B exists.
*                   bINa   - B where A exists.
*                   aOUTb  - A where B doesn't exist.
*                   aATOPb - B where A doesn't exist, A where
*                            B exists.
*                   bATOPa - A where B doesn't exist, B where
*                            A exists.
*                   aXORb  - A, B where the other do not exist.
*
*        Output: dst_obj - Output image
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: If an input image does not have an alpha
*                channel, this routine will create one with
*                value of 1 (appropriate for a background 
*                image).  Input images are required to have 
*                either four channels or three if no alpha
*                channel is supplied.  Masked or gated images 
*                currently are not supported.
*    Written By: Thomas H. Robey
*          Date: Nov 04, 1996
*      Verified: 
*  Side Effects: Output is scaled between 0 and 1.  If the 
*                datatype of the first source is double the
*                datatype of the destination will be the same.
*                Otherwise the datatype of the destination will
*                be float in order to hold the scaled output.
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int 
licomposite(kobject src1_obj, kobject src2_obj, kobject dst_obj, int flag)
/* -library_def_end */

/* -library_code */
{
   int i, j, k, datatype, truth, color, n=3;
   int data_format;
   int w1size, h1size, d1size, t1size, e1size;
   int w2size, h2size, d2size, t2size, e2size;
   int width, height, depth, timeaxis, elements, num_rgns, index;
   int process_map,process_value,process_mask,map_type,value_type,mask_type;
   double off1, off2, factor;
   char *lib = "licomposite";
   char *rtn = "licomposite";
   kobject dst_ref=NULL, alpha_obj=NULL;
   double alphaA, alphaB, alpha;
   double *data1=NULL, *data2=NULL;
   klist *objlist = NULL;

   /* Check input */

   if (src1_obj==NULL || src2_obj==NULL)
   {
      kerror(lib, rtn, "Invalid source object.");
      return FALSE;
   }

   /* Check for grey scale */

   KCALL(kpds_get_attribute(src1_obj, KCOLOR_COLORSPACE, &color));
   if (color==KNONE || color==KGREY)
      n = 1;
   KCALL(kpds_get_attribute(src2_obj, KCOLOR_COLORSPACE, &j));
   if ((n!=1 && j!=color) || (n==1 && (j!=KNONE && j!=KGREY)))
   {
      kerror(lib, rtn, "Source objects have different colorspaces.");
      return FALSE;
   }

   /* Set up the source objects */
 
   if (kpds_query_map(src1_obj))
   {
      KCALL(kpds_set_attribute(src1_obj, KPDS_MAPPING_MODE, KMAPPED));
   }
   if (kpds_query_map(src2_obj))
   {
      KCALL(kpds_set_attribute(src2_obj, KPDS_MAPPING_MODE, KMAPPED));
   }
   KCALL(kpds_get_attributes(src1_obj,KPDS_VALUE_DATA_TYPE,&datatype,
         KPDS_VALUE_SIZE,&w1size,&h1size,&d1size,&t1size,&e1size,NULL));
   KCALL(kpds_get_attribute(src2_obj,KPDS_VALUE_SIZE,&w2size,&h2size,
         &d2size,&t2size,&e2size));
   KCALL(kpds_get_attribute(src1_obj,KCOLOR_HAS_ALPHA,&truth));
   if (truth != TRUE)
   {
      if (e1size != n)
      {
         kerror(lib,rtn,
                "Invalid number of data channels in first source object %d.",
                e1size);
         return FALSE;
      }
      KCALL((alpha_obj = kpds_create_object()) != KOBJECT_INVALID);
      if (!licreate_alpha(src1_obj, NULL, alpha_obj, 1.0))
      {
         kfree(alpha_obj);
         (void) klist_free(objlist, (void (*)(klist*))lkcall_free);
         kerror(lib, rtn, "Error creating alpha channel.");
         return FALSE;
      }
      src1_obj = alpha_obj;
      e1size = n + 1;
   }
   else
   {
      if (e1size != n+1)
      {
         kerror(lib,rtn,
                "Invalid number of data channels in first source object %d.",
                e1size);
         return FALSE;
      }
   }
   KCALL(kpds_get_attribute(src2_obj, KCOLOR_HAS_ALPHA, &truth));
   if (truth != TRUE)
   {
      if (e2size != n)
      {
         kerror(lib,rtn,
                "Invalid number of data channels in second source object %d.",
                e2size);
         return FALSE;
      }
      KCALL((alpha_obj = kpds_create_object()) != KOBJECT_INVALID);
      if (!licreate_alpha(src2_obj, NULL, alpha_obj, 1.0))
      {
         kfree(alpha_obj);
         (void) klist_free(objlist, (void (*)(klist*))lkcall_free);
         kerror(lib, rtn, "Error creating alpha channel.");
         return FALSE;
      }
      src2_obj = alpha_obj;
      e2size = n + 1;
   }
   else
   {
      if (e2size != n+1)
      {
         kerror(lib,rtn,
                "Invalid number of data channels in second source object %d.",
                e2size);
         return FALSE;
      }
   }

   /* Check data format attribute */

   KCALL(kpds_set_attribute(src1_obj, KPDS_VALUE_DATA_TYPE, KDOUBLE));
   KCALL(kpds_set_attribute(src2_obj, KPDS_VALUE_DATA_TYPE, KDOUBLE));
   i=kpds_get_attribute(src1_obj, KPDS_DATA_FORMAT, &data_format);
   if (i==0 || (i==1 && data_format!=KFLOAT))
   {
      KCALL(kapu_scale_data(&src1_obj, 0.0, 1.0));
   }
   i=kpds_get_attribute(src2_obj, KPDS_DATA_FORMAT, &data_format);
   if (i==0 || (i==1 && data_format!=KFLOAT))
   {
      KCALL(kapu_scale_data(&src2_obj, 0.0, 1.0));
   }

   /* Upgrade objects.  Must reverse scaling for pad value */
   
   KCALL(kpds_get_attributes(src1_obj,KPDS_VALUE_SCALE_OFFSET,&off1,&off2,
         KPDS_VALUE_SCALE_FACTOR,&factor,NULL));
   if (off1 != 0.0)
   {
      KCALL(kpds_set_attribute(src1_obj,KPDS_VALUE_PAD_VALUE,-off1/factor,0.0));
   }
   KCALL(kpds_get_attributes(src2_obj,KPDS_VALUE_SCALE_OFFSET,&off1,&off2,
         KPDS_VALUE_SCALE_FACTOR,&factor,NULL));
   if (off1 != 0.0)
   {
      KCALL(kpds_set_attribute(src2_obj,KPDS_VALUE_PAD_VALUE,-off1/factor,0.0));
   }
   KCALL(kapu_upgrade_objects(&process_map,&process_value,&process_mask,
         &map_type,&value_type,&mask_type,TRUE,FALSE,TRUE,TRUE,2,
         src1_obj,src2_obj));
   /* Get size to allow destination objects to be set up */
   KCALL(kpds_get_attribute(src1_obj,KPDS_VALUE_SIZE,&w1size,&h1size,
         &d1size,&t1size,&e1size));

   /* Set up optimal region size for processing */

   KCALL(kpds_set_attributes(src1_obj,
         KPDS_VALUE_OPTIMAL_REGION_CONSTRAINT,1,1,1,1,e1size,
         KPDS_VALUE_OPTIMAL_REGION_FLAGS,KOPTIMAL_FACTOR,KOPTIMAL_FACTOR,
         KOPTIMAL_FACTOR,KOPTIMAL_FACTOR,KOPTIMAL_FIXED_VALUE,NULL));
   KCALL(kpds_get_attribute(src1_obj,KPDS_VALUE_OPTIMAL_REGION_SIZE,
         &width,&height,&depth,&timeaxis,&elements,&num_rgns));
   KCALL(kpds_set_attribute(src1_obj,KPDS_VALUE_REGION_SIZE,width,
         height,depth,timeaxis,elements));
   KCALL(kpds_set_attribute(src2_obj,KPDS_VALUE_REGION_SIZE,width,
         height,depth,timeaxis,elements));
 
   /* Set up destination object */
 
   if (!kpds_query_value(dst_obj))
   {
      KCALL(kpds_create_value(dst_obj));
   }
   if (datatype != KDOUBLE)
      datatype = KFLOAT;
   KCALL(kpds_set_attributes(dst_obj,KPDS_VALUE_DATA_TYPE,datatype,
         KPDS_DATA_FORMAT,KFLOAT,
         KPDS_VALUE_SIZE,w1size,h1size,d1size,t1size,n+1,
         KCOLOR_COLORSPACE,color,
         KCOLOR_HAS_ALPHA,TRUE,NULL));
 
   /* Set up a destination reference object of type double */
 
   KCALL((dst_ref=kpds_reference_object(dst_obj)) != KOBJECT_INVALID);
   objlist = klist_add(objlist, dst_ref, "KOBJECT");
   KCALL(kpds_set_attributes(dst_ref,KPDS_VALUE_DATA_TYPE,KDOUBLE,
         KPDS_VALUE_REGION_SIZE,width,height,depth,timeaxis,elements,NULL));
 
   /* Process the first n channels */
 
   index = width * height * depth * timeaxis;
   for (i=0; i<num_rgns; i++)
   {
      KCALL((data1=(double *)kpds_get_data(src1_obj,KPDS_VALUE_REGION,data1))
         != NULL);
      KCALL((data2=(double *)kpds_get_data(src2_obj,KPDS_VALUE_REGION,data2))
         != NULL);

      /* Perform composite */

      switch (flag)
      {
         case aOVERb:
            for (j=0; j<index; j++)
            {
               alphaA = data1[j+n*index];
               alphaB = data2[j+n*index];
               alpha = alphaA + alphaB * (1.0 - alphaA);
               for (k=0; k<n; k++)
               {
                  if (alpha == 0)
                     data1[j+k*index] = 0.0;
                  else
                     data1[j+k*index]=(alphaA*data1[j+k*index]+
                        alphaB*(1.0-alphaA)*data2[j+k*index])/alpha;
               }
               data1[j+n*index] = alpha;
            }
            break;
         case bOVERa:
            for (j=0; j<index; j++)
            {
               alphaA = data1[j+n*index];
               alphaB = data2[j+n*index];
               alpha = alphaA * (1.0 - alphaB) + alphaB;
               for (k=0; k<n; k++)
               {
                  if (alpha == 0.0)
                      data1[j+k*index] = 0.0;
                  else
                     data1[j+k*index]=(alphaA*(1.0-alphaB)*data1[j+k*index]+
                        alphaB*data2[j+k*index])/alpha;
               }
               data1[j+n*index] = alpha;
            }
            break;
         case aINb:
            for (j=0; j<index; j++)
            {
               alphaA = data1[j+n*index];
               alphaB = data2[j+n*index];
               alpha = alphaA * alphaB;
               for (k=0; k<n; k++)
               {
                  if (alpha == 0.0)
                     data1[j+k*index] = 0.0;
               /* else
                     data1[j+k*index] = data1[j+k*index]; */
               }
               data1[j+n*index] = alpha;
            }
            break;
         case bINa:
            for (j=0; j<index; j++)
            {
               alphaA = data1[j+n*index];
               alphaB = data2[j+n*index];
               alpha = alphaB * alphaA;
               for (k=0; k<n; k++)
               {
                  if (alpha == 0.0)
                     data1[j+k*index] = 0.0;
                  else
                     data1[j+k*index] = data2[j+k*index];
               }
               data1[j+n*index] = alpha;
            }
            break;
         case aOUTb:
            for (j=0; j<index; j++)
            {
               alphaA = data1[j+n*index];
               alphaB = data2[j+n*index];
               alpha = alphaA * (1.0 - alphaB);
               for (k=0; k<n; k++)
               {
                  if (alpha == 0.0)
                     data1[j+k*index] = 0.0;
                  else
                     data1[j+k*index] = data1[j+k*index];
               }
               data1[j+n*index] = alpha;
            }
            break;
         case bOUTa:
            for (j=0; j<index; j++)
            {
               alphaA = data1[j+n*index];
               alphaB = data2[j+n*index];
               alpha = alphaB * (1.0 - alphaA);
               for (k=0; k<n; k++)
               {
                  if (alpha == 0.0)
                     data1[j+k*index] = 0.0;
                  else
                     data1[j+k*index] = data2[j+k*index];
               }
               data1[j+n*index] = alpha;
            }
            break;
         case aATOPb:
            for (j=0; j<index; j++)
            {
               alphaA = data1[j+n*index];
               alphaB = data2[j+n*index];
               alpha = alphaA * alphaB + alphaB * (1.0 - alphaA);
               for (k=0; k<n; k++)
               {
                  if (alpha == 0.0)
                     data1[j+k*index] = 0.0;
                  else
                     data1[j+k*index]=(alphaA*alphaB*data1[j+k*index]+
                        alphaB*(1.0-alphaA)*data2[j+k*index])/alpha;
               }
               data1[j+n*index] = alpha;
            }
            break;
         case bATOPa:
            for (j=0; j<index; j++)
            {
               alphaA = data1[j+n*index];
               alphaB = data2[j+n*index];
               alpha = alphaA * (1.0 - alphaB) + alphaB * alphaA;
               for (k=0; k<n; k++)
               {
                  if (alpha == 0.0)
                     data1[j+k*index] = 0.0;
                  else
                     data1[j+k*index]=(alphaA*(1.0-alphaB)*data1[j+k*index]+
                        alphaB*alphaA*data2[j+k*index])/alpha;
               }
               data1[j+n*index] = alpha;
            }
            break;
         case aXORb:
            for (j=0; j<index; j++)
            {
               alphaA = data1[j+n*index];
               alphaB = data2[j+n*index];
               alpha = alphaA * (1.0 - alphaB) + alphaB * (1.0 - alphaA);
               for (k=0; k<n; k++)
               {
                  if (alpha == 0.0)
                     data1[j+k*index] = 0.0;
                  else
                     data1[j+k*index]=(alphaA*(1.0-alphaB)*data1[j+k*index]+
                        alphaB*(1.0-alphaA)*data2[j+k*index])/alpha;
               }
               data1[j+n*index] = alpha;
            }
            break;
         case aPLUSb:
         default:
            for (j=0; j<index; j++)
            {
               alphaA = data1[j+n*index];
               alphaB = data2[j+n*index];
               alpha = alphaA + alphaB;
               for (k=0; k<n; k++)
               {
                  if (alpha == 0.0)
                     data1[j+k*index] = 0.0;
                  else
                     data1[j+k*index]=(alphaA*data1[j+k*index]+
                        alphaB*data2[j+k*index])/alpha;
               }
               data1[j+n*index] = alpha;
            }
      }

      KCALL(kpds_put_data(dst_ref, KPDS_VALUE_REGION, data1));
   }
 
   /* Set up for exit */
 
   kfree(data1);
   kfree(data2);
   (void)klist_free(objlist, (void (*)(klist*))lkcall_free);
   return TRUE;
}
/* -library_code_end */
