 /*
  * Khoros: $Id: licreate_alpha.c,v 1.2.2.1 1997/04/24 22:17:56 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: licreate_alpha.c,v 1.2.2.1 1997/04/24 22:17:56 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for icreate_alpha
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	licreate_alpha
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: licreate_alpha - Creates an alpha channel
* 
*       Purpose: This function creates an alpha channel for an
*                image using either a one channel source or a 
*                constant.
*
*         Input: src_obj - Source image
*                matte_obj - Matte or alpha channel
*                real - constant channel if matte_obj is not
*                       specified.
*
*        Output: dst_obj - Output image
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: If a constant alpha is supplied it should be
*                between 0 and 1 and will be scaled to match the
*                input image.
*    Written By: Thomas H. Robey
*          Date: Oct 28, 1996
*      Verified: 
*  Side Effects: If the input image is mapped and the alpha channel is
*                specified by a file, the output image is not mapped.
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int 
licreate_alpha(kobject src_obj, kobject matte_obj, kobject dst_obj,
               double real)
/* -library_def_end */

/* -library_code */
{
   int i, j, k, datatype, color, size, region_size, n=3;
   int num_args, arg_size, data_type, permanent;
   int wsize, hsize, dsize, tsize, esize;
   int width, height, depth, timeaxis, elements, num_rgns;
   int m_wsize, m_hsize, m_dsize, m_tsize, m_esize;
   char *lib = "licomposite";
   char *rtn = "licreate_alpha";
   kobject dst_ref = NULL;
   double min, max;
   double *data = NULL;
   klist *objlist = NULL;

   /* Check input */

   if (matte_obj==NULL && (real<0 || real >1.0))
   {
      kerror(lib, rtn, "Invalid alpha channel.");
      return FALSE;
   }
   if (src_obj==NULL || dst_obj==NULL)
   {
      kerror(lib, rtn, "Invalid source or destination object.");
      return FALSE;
   }

   /* Check for grey scale */

   KCALL(kpds_get_attribute(src_obj, KCOLOR_COLORSPACE, &color));
   if (color==KNONE || color==KGREY)
      n = 1;

   /* Check if map can be operated on directly */

   if (kpds_query_map(src_obj) && matte_obj==NULL)
   {
      /* Set up the source object */
 
      KCALL(kpds_get_attribute(src_obj,KPDS_MAP_SIZE,&wsize,&hsize,
            &dsize,&tsize,&esize));
      if (wsize != n)
      {
         kerror(lib, rtn, "Invalid map width in input object %d.", wsize);
         return FALSE;
      }
      KCALL(kpds_copy_object_attr(src_obj, dst_obj));
      KCALL(kpds_set_attributes(src_obj,KPDS_MAP_DATA_TYPE,KDOUBLE,
            KPDS_MAP_REGION_SIZE,1,hsize,dsize,tsize,esize,NULL));

      /* Set up destination object */
 
      KCALL(kpds_set_attributes(dst_obj,KCOLOR_HAS_ALPHA,TRUE,
            KPDS_MAP_SIZE,wsize+1,hsize,dsize,tsize,esize,NULL));
      KCALL((dst_ref=kpds_reference_object(dst_obj)) != KOBJECT_INVALID);
      objlist = klist_add(objlist, dst_ref, "KOBJECT");
      KCALL(kpds_set_attributes(dst_ref,KPDS_MAP_DATA_TYPE,KDOUBLE,
            KPDS_MAP_REGION_SIZE,1,hsize,dsize,tsize,esize,NULL));

      i=kdms_query_attribute(dst_obj,NULL,KPDS_DATA_RANGE,&num_args,&arg_size,
                             &data_type,&permanent);
      if (i == 0)
      {
         KCALL(kdms_create_attribute(dst_obj,NULL,KPDS_DATA_RANGE,2,1,KDOUBLE,
               TRUE,TRUE));
      }
      else
         i = kpds_get_attribute(src_obj, KPDS_DATA_RANGE, &min, &max);
      if (i == 0)
      {
         KCALL(kapu_minmax(src_obj, KPDS_MAP_ALL, &min, &max));
      }
      KCALL(kpds_set_attribute(dst_obj, KPDS_DATA_RANGE, min, max));

      /* Copy the first n channels */
 
      for (i=0; i<n; i++)
      {
         KCALL((data=(double *)kpds_get_data(src_obj,KPDS_MAP_REGION,data))
            != NULL);
         KCALL(kpds_put_data(dst_ref, KPDS_MAP_REGION, data));
      }

      /* create alpha channel */

      for (i=0; i<hsize*dsize*tsize*esize; i++)
         data[i] = real * (max - min) + min;
      KCALL(kpds_put_data(dst_ref, KPDS_MAP_REGION, data));
      KCALL(kpds_copy_remaining_data(src_obj, dst_ref));
   }
   else
   {
 
      /* Set up the source object */
 
      if (kpds_query_map(src_obj))
         KCALL(kpds_set_attribute(src_obj, KPDS_MAPPING_MODE, KMAPPED));
      KCALL(kpds_get_attributes(src_obj,KPDS_VALUE_DATA_TYPE,&datatype,
            KPDS_VALUE_SIZE,&wsize,&hsize,&dsize,&tsize,&esize,NULL));
      if (esize != n)
      {
         kerror(lib, rtn, "Invalid number of data channels in source object.");
         return FALSE;
      }
      KCALL(kpds_set_attribute(src_obj,KPDS_VALUE_DATA_TYPE,KDOUBLE));

      /* Set up destination object */
 
      if (!kpds_query_value(dst_obj))
         KCALL(kpds_create_value(dst_obj));
      KCALL(kpds_set_attributes(dst_obj,KPDS_VALUE_DATA_TYPE,datatype,
            KPDS_VALUE_SIZE,wsize,hsize,dsize,tsize,esize+1,
            KCOLOR_COLORSPACE,color,
            KCOLOR_HAS_ALPHA,TRUE,NULL));

      i=kdms_query_attribute(dst_obj,NULL,KPDS_DATA_RANGE,&num_args,&arg_size,
                             &data_type,&permanent);
      if (i == 0)
      {
         KCALL(kdms_create_attribute(dst_obj,NULL,KPDS_DATA_RANGE,2,1,KDOUBLE,
               TRUE,TRUE));
      }
      else
         i = kpds_get_attribute(src_obj, KPDS_DATA_RANGE, &min, &max);
      if (i == 0)
      {
         KCALL(kapu_minmax(src_obj, KPDS_VALUE_ALL, &min, &max));
      }
      KCALL(kpds_set_attribute(dst_obj, KPDS_DATA_RANGE, min, max));

      /* Set up a destination reference object of type double */

      KCALL((dst_ref=kpds_reference_object(dst_obj)) != KOBJECT_INVALID);
      objlist = klist_add(objlist, dst_ref, "KOBJECT");
      KCALL(kpds_set_attributes(dst_ref,KPDS_VALUE_DATA_TYPE,KDOUBLE,
            KPDS_VALUE_OPTIMAL_REGION_CONSTRAINT,1,1,1,1,1,
            KPDS_VALUE_OPTIMAL_REGION_FLAGS,KOPTIMAL_FACTOR,
            KOPTIMAL_FACTOR,KOPTIMAL_FACTOR,KOPTIMAL_FACTOR,
            KOPTIMAL_FIXED_VALUE,NULL));
      KCALL(kpds_get_attribute(dst_ref,KPDS_VALUE_OPTIMAL_REGION_SIZE,
            &width,&height,&depth,&timeaxis,&elements,&num_rgns));
      KCALL(kpds_set_attribute(src_obj,KPDS_VALUE_REGION_SIZE,width,
            height,depth,timeaxis,elements));
      KCALL(kpds_set_attribute(dst_ref,KPDS_VALUE_REGION_SIZE,width,
            height,depth,timeaxis,elements));

      /* Check to see if an object is to be used to supply the alpha channel */
 
      if (matte_obj != NULL)
      {
         KCALL(kpds_get_attribute(matte_obj,KPDS_VALUE_SIZE,&m_wsize,&m_hsize,
               &m_dsize,&m_tsize,&m_esize));
         if (m_wsize!=wsize || m_hsize!=hsize || m_dsize!=dsize || 
            m_tsize!=tsize || m_esize!=1)
         {
            (void)klist_free(objlist, (void (*)(klist*))lkcall_free);
            kerror(lib, rtn, "Invalid matte object size.");
            return FALSE;
         }
         KCALL(kpds_set_attributes(matte_obj,KPDS_VALUE_DATA_TYPE,KDOUBLE,
               KPDS_VALUE_REGION_SIZE,width,height,depth,timeaxis,elements,
               NULL));
      }
 
      /* Copy the first n channels */
 
      region_size = width * height * depth * timeaxis;
      size = wsize * hsize * dsize * tsize;
      for (j=0; j<n+1; j++)
      {
         k = 0;
         while (k < size)
         {
            if (j < n)
            {
               KCALL((data=(double *)kpds_get_data(src_obj,KPDS_VALUE_REGION,
                  data)) != NULL);
            }
            else
            {
               /*------------------------------------------------------------
               | Create the alpha channel.  If object is specified for alpha 
               | channel, use it.  Otherwise use the (scaled) floating point 
               | factor.
               |-----------------------------------------------------------*/
               if (matte_obj != NULL)
               {
                  KCALL((data=(double *)kpds_get_data(matte_obj,
                     KPDS_VALUE_REGION,data)) != NULL);
               }
               else
               {
                  for (i=0; i<region_size; i++)
                     data[i] = real * (max - min) + min;
               }
            }
            k += region_size;
            KCALL(kpds_put_data(dst_ref, KPDS_VALUE_REGION, data));
         }
      }
   }
 
   /* Set up for exit */
 
   kfree(data);
   (void)klist_free(objlist,(void (*)(klist*))lkcall_free);
   return TRUE;
}
/* -library_code_end */
