 /*
  * Khoros: $Id: lidarken.c,v 1.2.2.1 1997/04/23 22:08:17 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lidarken.c,v 1.2.2.1 1997/04/23 22:08:17 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for idarken
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lidarken
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lidarken - Darkens or brightens an image.
* 
*       Purpose: This library routine darkens or brightens an 
*                image.  Scaling factors less than 1.0 darken
*                an image while those greater than 1.0 are
*                brightened.  The first three channels of image 
*                data are scaled; the alpha channel is unchanged.
*                If KPDS_DATA_RANGE is not defined it will be
*                defined.
*
*         Input: src_obj - Input image
*                factor - Scaling factor
*
*        Output: dst_obj - Output image
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: The scaling factor must be positive.  The input
*                image must have at least three channels.
*    Written By: Thomas H. Robey
*          Date: Oct 28, 1996
*      Verified: 
*  Side Effects: If a factor greater than 1.0 is supplied, the 
*                output is not clipped.
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int 
lidarken(kobject src_obj, kobject dst_obj, double factor)
/* -library_def_end */

/* -library_code */
{
   int i, j, k, region_size, size, color, alpha, m, n=3;
   int width, height, depth, timeaxis, elements, num_rgns;
   int wsize, hsize, dsize, tsize, esize;
   int num_args, arg_size, data_type, permanent;
   char *lib = "licomposite";
   char *rtn = "lidarken";
   double min, max;
   double *data = NULL;
   kobject dst_ref = NULL;
   klist *objlist = NULL;

   /* Check input */

   if (factor < 0.0)
   {
      kerror(lib, rtn, "Negative scaling factor %g.", factor);
      return FALSE;
   }

   /* Check for grey scale and alpha channel */

   KCALL(kpds_get_attributes(src_obj,KCOLOR_COLORSPACE,&color,
         KCOLOR_HAS_ALPHA,&alpha,NULL));
   if (color==KNONE || color==KGREY)
      n = 1;
   if (alpha == TRUE)
      m = n +1;
   else
      m = n;
   
   /* Check to see if map can operated on directly */
 
   if (kpds_query_map(src_obj))
   {
      /* Set up the source object */

      KCALL(kpds_get_attribute(src_obj,KPDS_MAP_SIZE,&wsize,&hsize,&dsize,
            &tsize,&esize));
      KCALL(kpds_copy_object_attr(src_obj, dst_obj));
      KCALL(kpds_set_attributes(src_obj,KPDS_MAP_DATA_TYPE,KDOUBLE,
            KPDS_MAP_REGION_SIZE,1,hsize,dsize,tsize,esize,NULL));

      /* Set up destination object */

      KCALL((dst_ref = kpds_reference_object(dst_obj)) != KOBJECT_INVALID);
      objlist = klist_add(objlist, dst_ref, "KOBJECT");
      KCALL(kpds_set_attributes(dst_ref, KPDS_MAP_DATA_TYPE, KDOUBLE,
            KPDS_MAP_REGION_SIZE,1,hsize,dsize,tsize,esize,NULL));

      i=kdms_query_attribute(dst_obj,NULL,KPDS_DATA_RANGE,&num_args,&arg_size,
                             &data_type,&permanent);
      if (i == 0)
      {
         KCALL(kdms_create_attribute(dst_obj,NULL,KPDS_DATA_RANGE,2,1,KDOUBLE,
               TRUE,TRUE));
      }
      else
         i = kpds_get_attribute(src_obj, KPDS_DATA_RANGE, &min, &max);
      if (i == 0)
      {
         KCALL(kapu_minmax(src_obj, KPDS_MAP_ALL, &min, &max));
      }
      KCALL(kpds_set_attribute(dst_obj, KPDS_DATA_RANGE, min, max));

      /* Scale the first n channels */

      for (i=0; i<m; i++)
      {
         KCALL((data = (double *)kpds_get_data(src_obj,KPDS_MAP_REGION,data)) != NULL);
         if (i < n)
            for (j=0; j<hsize*dsize*tsize*esize; j++)
               data[j] *= factor;
         KCALL(kpds_put_data(dst_ref, KPDS_MAP_REGION, data));
      }
      KCALL(kpds_copy_remaining_data(src_obj, dst_ref));
   }

   /* Now, the unmapped case */

   else
   {
 
      /* Set up the source object */

      KCALL(kpds_get_attribute(src_obj,KPDS_VALUE_SIZE,&wsize,&hsize,&dsize,
            &tsize,&esize));
      KCALL(kpds_copy_object_attr(src_obj, dst_obj));
      KCALL(kpds_set_attribute(src_obj, KPDS_VALUE_DATA_TYPE, KDOUBLE));

      /* Set up a destination object */

      KCALL((dst_ref = kpds_reference_object(dst_obj)) != KOBJECT_INVALID);
      objlist = klist_add(objlist, dst_ref, "KOBJECT");
      KCALL(kpds_set_attributes(dst_ref,KPDS_VALUE_DATA_TYPE,KDOUBLE,
            KPDS_VALUE_OPTIMAL_REGION_CONSTRAINT,1,1,1,1,1,
            KPDS_VALUE_OPTIMAL_REGION_FLAGS,KOPTIMAL_FACTOR,
            KOPTIMAL_FACTOR,KOPTIMAL_FACTOR,KOPTIMAL_FACTOR,
            KOPTIMAL_FIXED_VALUE,NULL));
      KCALL(kpds_get_attribute(dst_ref,KPDS_VALUE_OPTIMAL_REGION_SIZE,
            &width,&height,&depth,&timeaxis,&elements,&num_rgns));
      KCALL(kpds_set_attributes(src_obj,KPDS_VALUE_DATA_TYPE,KDOUBLE,
            KPDS_VALUE_REGION_SIZE,width,height,depth,timeaxis,elements,NULL));
      KCALL(kpds_set_attribute(dst_ref,KPDS_VALUE_REGION_SIZE,width,
            height,depth,timeaxis,elements));

      i=kdms_query_attribute(dst_obj,NULL,KPDS_DATA_RANGE,&num_args,&arg_size,
                             &data_type,&permanent);
      if (i == 0)
      {
         KCALL(kdms_create_attribute(dst_obj,NULL,KPDS_DATA_RANGE,2,1,KDOUBLE,
               TRUE,TRUE));
      }
      else
         i = kpds_get_attribute(src_obj, KPDS_DATA_RANGE, &min, &max);
      if (i == 0)
      {
         KCALL(kapu_minmax(src_obj, KPDS_VALUE_ALL, &min, &max));
      }
      KCALL(kpds_set_attribute(dst_obj, KPDS_DATA_RANGE, min, max));

      /* Process the first n channels */

      region_size = width * height * depth * timeaxis;
      size = wsize * hsize * dsize * tsize;
      for (j=0; j<m; j++)
      {
         k = 0;
         while (k < size)
         {
            KCALL((data=(double *)kpds_get_data(src_obj,KPDS_VALUE_REGION,data))
               != NULL);
            k += region_size;
            if (j < n)
               for (i=0; i<region_size; i++)
                  data[i] *= factor;
            KCALL(kpds_put_data(dst_ref, KPDS_VALUE_REGION, data));
         }
      }
   }

   /* Set up for exit */

   kfree(data);
   (void)klist_free(objlist, (void(*)(klist *))lkcall_free);
   return TRUE;

}
/* -library_code_end */
