 /*
  * Khoros: $Id: lidissolve.c,v 1.2.2.1 1997/04/24 22:17:57 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lidissolve.c,v 1.2.2.1 1997/04/24 22:17:57 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for idissolve
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lidissolve
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lidissolve - Dissolves an image.
* 
*       Purpose: This library routine dissolves or reverse the 
*                dissolution of an image.  The alpha channel is
*                scaled by the scaling factor.  Scaling factors
*                less than 1.0 will dissolve an image while 
*                factors greater than 1.0 will reverse 
*                dissolution.  If KPDS_DATA_RANGE is not defined
*                it will be defined.
*
*         Input: src_obj - Input image
*                factor - Scaling factor
*
*        Output: dst_obj - Output image
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: The scaling facor must be positive.  If the input
*                image does not have an alpha channel, an alpha
*                channel of 1.0 will be created (and then scaled).
*                The input image must have four channels (three if
*                no alpha channel is supplied).
*    Written By: Thomas H. Robey
*          Date: Oct 28, 1996
*      Verified: 
*  Side Effects: If a factor greater than one is supplied, the
*                output alpha is not clipped.
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int 
lidissolve(kobject src_obj, kobject dst_obj, double factor)
/* -library_def_end */

/* -library_code */
{
   int i, j, k, region_size, size, truth, color, n=3;
   int num_args, arg_size, data_type, permanent;
   int wsize, hsize, dsize, tsize, esize;
   int width, height, depth, timeaxis, elements, num_rgns;
   double *data = NULL;
   char *lib = "licomposite";
   char *rtn = "lidissolve";
   double min, max;
   klist *objlist = NULL;
   kobject dst_ref=NULL, alpha_obj=NULL;

   /* Check input */

   if (factor < 0.0)
   {
      kerror(lib, rtn, "Negative scaling factor %g.", factor);
      return FALSE;
   }

   /* Check for grey scale */

   KCALL(kpds_get_attribute(src_obj, KCOLOR_COLORSPACE, &color));
   if (color==KNONE || color==KGREY)
      n = 1;

   if (kpds_query_map(src_obj))
   {
      /* Set up the source object */

      KCALL(kpds_get_attribute(src_obj,KPDS_MAP_SIZE,&wsize,&hsize,&dsize,
            &tsize,&esize));
      KCALL(kpds_get_attribute(src_obj, KCOLOR_HAS_ALPHA, &truth));
      if (truth != TRUE)
      {
         if (wsize != n)
         {
            kerror(lib, rtn, "Invalid map width in input object %d.", wsize);
            return FALSE;
         }
         KCALL((alpha_obj = kpds_create_object()) != KOBJECT_INVALID);
         objlist = klist_add(objlist, alpha_obj, "KOBJECT");
         if (!licreate_alpha(src_obj, NULL, alpha_obj, 1.0))
         {
            (void)klist_free(objlist, (void (*)(klist*))lkcall_free);
            kerror(lib, rtn, "Error creating alpha channel.");
            return FALSE;
         }
         src_obj = alpha_obj;
         wsize = n + 1;
      }
      else
      {
         if (wsize != n+1)
         {
            kerror(lib, rtn, "Invalid map width in input object %d.", wsize);
            return FALSE;
         }
      }
      KCALL(kpds_copy_object_attr(src_obj, dst_obj));
      KCALL(kpds_set_attributes(src_obj,KPDS_MAP_DATA_TYPE,KDOUBLE,
            KPDS_MAP_REGION_SIZE,1,hsize,dsize,tsize,esize,NULL));

      /* Set up destination object */

      KCALL((dst_ref=kpds_reference_object(dst_obj)) != KOBJECT_INVALID);
      objlist = klist_add(objlist, dst_ref, "KOBJECT");
      KCALL(kpds_set_attributes(dst_ref,KPDS_MAP_DATA_TYPE,KDOUBLE,
            KPDS_MAP_REGION_SIZE,1,hsize,dsize,tsize,esize,NULL));

      i=kdms_query_attribute(dst_obj,NULL,KPDS_DATA_RANGE,&num_args,&arg_size,
                             &data_type,&permanent);
      if (i == 0)
      {
         KCALL(kdms_create_attribute(dst_obj,NULL,KPDS_DATA_RANGE,2,1,KDOUBLE,
               TRUE,TRUE));
      }
      else
         i = kpds_get_attribute(src_obj, KPDS_DATA_RANGE, &min, &max);
      if (i == 0)
      {
         KCALL(kapu_minmax(src_obj, KPDS_MAP_ALL, &min, &max));
      }
      KCALL(kpds_set_attribute(dst_obj, KPDS_DATA_RANGE, min, max));

      /* Copy first n channels */

      for (i=0; i<n; i++)
      {
         KCALL((data=(double *)kpds_get_data(src_obj,KPDS_MAP_REGION,data))
               != NULL);
         KCALL(kpds_put_data(dst_ref, KPDS_MAP_REGION, data));
      }

      /* Scale the alpha channel */

      KCALL((data=(double *)kpds_get_data(src_obj,KPDS_MAP_REGION,data)) 
            != NULL);
      for (i=0; i<hsize*dsize*tsize*esize; i++)
         data[i] *= factor;
      KCALL(kpds_put_data(dst_ref, KPDS_MAP_REGION, data));
      KCALL(kpds_copy_remaining_data(src_obj, dst_ref));
   }
   else
   {

      /* Set up the source object */

      KCALL(kpds_get_attributes(src_obj,KCOLOR_HAS_ALPHA,&truth,
            KPDS_VALUE_SIZE,&wsize,&hsize,&dsize,&tsize,&esize,NULL));
      if (truth != TRUE)
      {
         if (esize != n)
         {
            kerror(lib,rtn,"Invalid number of data channels in source object.");
            return FALSE;
         }
         KCALL((alpha_obj = kpds_create_object()) != KOBJECT_INVALID);
         objlist = klist_add(objlist, alpha_obj, "KOBJECT");
         if (!licreate_alpha(src_obj, NULL, alpha_obj, 1.0))
         {
            (void)klist_free(objlist, (void (*)(klist*))lkcall_free);
            kerror(lib, rtn, "Error creating alpha channel.");
            return FALSE;
         }
         src_obj = alpha_obj;
         esize = n + 1;
      }
      else
      {
         if (esize != n+1)
         {
            kerror(lib,rtn,"Invalid number of data channels in source object.");
            return FALSE;
         }
      }
      KCALL(kpds_copy_object_attr(src_obj, dst_obj));

      /* Set up a destination object */

      KCALL((dst_ref=kpds_reference_object(dst_obj)) != KOBJECT_INVALID);
      objlist = klist_add(objlist, dst_ref, "KOBJECT");
      KCALL(kpds_set_attributes(dst_ref,KPDS_VALUE_DATA_TYPE,KDOUBLE,
            KPDS_VALUE_OPTIMAL_REGION_CONSTRAINT,1,1,1,1,1,
            KPDS_VALUE_OPTIMAL_REGION_FLAGS,KOPTIMAL_FACTOR,
            KOPTIMAL_FACTOR,KOPTIMAL_FACTOR,KOPTIMAL_FACTOR,
            KOPTIMAL_FIXED_VALUE,NULL));
      KCALL(kpds_get_attribute(dst_ref,KPDS_VALUE_OPTIMAL_REGION_SIZE,
            &width,&height,&depth,&timeaxis,&elements,&num_rgns));
      KCALL(kpds_set_attributes(src_obj,KPDS_VALUE_DATA_TYPE,KDOUBLE,
            KPDS_VALUE_REGION_SIZE,width,height,depth,timeaxis,elements,NULL));
      KCALL(kpds_set_attribute(dst_ref,KPDS_VALUE_REGION_SIZE,width,
            height,depth,timeaxis,elements));

      i=kdms_query_attribute(dst_obj,NULL,KPDS_DATA_RANGE,&num_args,&arg_size,
                             &data_type,&permanent);
      if (i == 0)
      {
         KCALL(kdms_create_attribute(dst_obj,NULL,KPDS_DATA_RANGE,2,1,KDOUBLE,
               TRUE,TRUE));
      }
      else
         i = kpds_get_attribute(src_obj, KPDS_DATA_RANGE, &min, &max);
      if (i == 0)
      {
         KCALL(kapu_minmax(src_obj, KPDS_VALUE_ALL, &min, &max));
      }
      KCALL(kpds_set_attribute(dst_obj, KPDS_DATA_RANGE, min, max));

      /* Copy the first n channels and scale the alpha channel */

      region_size = width * height * depth * timeaxis;
      size = wsize * hsize * dsize * tsize;
      for (j=0; j<n+1; j++)
      {
         k = 0;
         while (k < size)
         {
            KCALL((data=(double *)kpds_get_data(src_obj,KPDS_VALUE_REGION,data))
               != NULL);
            k += region_size;
            if (j == n)
               for (i=0; i<region_size; i++)
                  data[i] *= factor;
            KCALL(kpds_put_data(dst_ref, KPDS_VALUE_REGION, data));
         }
      }
   }

   /* Set up for exit */

   kfree(data);
   (void)klist_free(objlist, (void (*)(klist*))lkcall_free);
   return TRUE;
}
/* -library_code_end */
