 /*
  * Khoros: $Id: meigen.c,v 1.18.2.2 1997/05/31 19:35:35 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: meigen.c,v 1.18.2.2 1997/05/31 19:35:35 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for meigen
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "meigen.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Compute Eigenvectors and Eigenvalues of a Square Matrix
|
|       Purpose: main program for meigen
|
|         Input:
|		char *clui_info->i_file; {Input Matrix}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o1_file; {Eigenvector output matrix}
|		int   clui_info->o1_flag; {TRUE if -o1 specified}
|
|		char *clui_info->o2_file; {Eigenvalue output matrix}
|		int   clui_info->o2_flag; {TRUE if -o2 specified}
|
|        Output:
|       Returns:
|
|    Written By: Scott Wilson
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
kobject in_obj = NULL;
kobject evec_obj = NULL;
kobject eval_obj = NULL;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MATRIX", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MATRIX/objects/kroutine/meigen");
	kexit_handler(meigen_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MATRIX", "meigen", KGEN_KROUTINE,  &clui_uis_spec,
		   meigen_usage_additions, meigen_get_args,
 		   meigen_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        if ((in_obj = kpds_open_input_object(clui_info->i_file))
                == KOBJECT_INVALID)
        {
           kerror("meigen","main", "Cannot open input object.");
           kexit(KEXIT_FAILURE);
        }

        if (clui_info->o1_flag)
          {
            if ((evec_obj = kpds_open_output_object(clui_info->o1_file))
                    == KOBJECT_INVALID)
              {
                kerror("meigen","main", "Cannot open output object.");
                kexit(KEXIT_FAILURE);
              }
            if (!kpds_copy_object_attr(in_obj, evec_obj))
              {
                kerror("meigen","main",
                  "Unable to copy attributes from input to output object.");
                kexit(KEXIT_FAILURE);
              }
          }
        if (clui_info->o2_flag)
          {
            if ((eval_obj = kpds_open_output_object(clui_info->o2_file))
                == KOBJECT_INVALID)
              {
                 kerror("meigen","main", "Cannot open output object.");
                 kexit(KEXIT_FAILURE);
              }
            if (!kpds_copy_object_attr(in_obj, eval_obj))
              {
                kerror("meigen","main",
                  "Unable to copy attributes from input to output object.");
                kexit(KEXIT_FAILURE);
              }
          }
/* -main_before_lib_call_end */

/* -main_library_call */
        if (!lmeigen(in_obj,evec_obj,eval_obj)) kexit(KEXIT_FAILURE);
/* -main_library_call_end */

/* -main_after_lib_call */
        /* add history to the output objects and write */
        if (clui_info->o1_flag)
          {
            if (!kpds_set_attribute(evec_obj,KPDS_HISTORY,kpds_history_string()))
              {
                kerror("meigen", "main","Unable to set history on output object");
                kexit(KEXIT_FAILURE);
              }
            kpds_close_object(evec_obj);
          }
        if (clui_info->o2_flag)
          {
            if (!kpds_set_attribute(eval_obj,KPDS_HISTORY,kpds_history_string()))
              {
                kerror("meigen", "main","Unable to set history on output object");
                kexit(KEXIT_FAILURE);
              }
            kpds_close_object(eval_obj);
          }
        kpds_close_object(in_obj);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: meigen_usage_additions
| 
|       Purpose: Prints usage additions in meigen_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname meigen
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void meigen_usage_additions(void)
{
	kfprintf(kstderr, "\tCompute Eigenvectors and Eigenvalues of a Square Matrix\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: meigen_free_args
| 
|       Purpose: Frees CLUI struct allocated in meigen_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname meigen
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
meigen_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o1_file);
		kfree_and_NULL(clui_info->o2_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
