 /*
  * Khoros: $Id: mlse.c,v 1.14.2.2 1997/05/31 19:37:49 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: mlse.c,v 1.14.2.2 1997/05/31 19:37:49 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for mlse
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "mlse.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Least Squares Solution to System of Linear Equations
|
|       Purpose: main program for mlse
|
|         Input:
|		char *clui_info->i1_file; {A matrix}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {b vector (or B matrix)}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->o_file; {solution vector (or matrix)}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: Scott Wilson
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
kobject sys_obj = NULL;
kobject b_obj = NULL;
kobject sol_obj = NULL;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MATRIX", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MATRIX/objects/kroutine/mlse");
	kexit_handler(mlse_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MATRIX", "mlse", KGEN_KROUTINE,  &clui_uis_spec,
		   mlse_usage_additions, mlse_get_args,
 		   mlse_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        if ((sys_obj = kpds_open_input_object(clui_info->i1_file))
                == KOBJECT_INVALID)
        {
           kerror("mlse","main", "Cannot open input object.");
           kexit(KEXIT_FAILURE);
        }
        if ((b_obj = kpds_open_input_object(clui_info->i2_file))
                == KOBJECT_INVALID)
        {
           kerror("mlse","main", "Cannot open input object.");
           kexit(KEXIT_FAILURE);
        }

        if ((sol_obj = kpds_open_output_object(clui_info->o_file))
                == KOBJECT_INVALID)
          {
            kerror("mlse","main", "Cannot open output object.");
            kexit(KEXIT_FAILURE);
          }
        if (!kpds_copy_object_attr(sys_obj, sol_obj))
          {
            kerror("mlse","main",
              "Unable to copy attributes from input to output object.");
            kexit(KEXIT_FAILURE);
          }
/* -main_before_lib_call_end */

/* -main_library_call */
        if (!lmlse(sys_obj,b_obj,sol_obj)) kexit(KEXIT_FAILURE);
/* -main_library_call_end */

/* -main_after_lib_call */
        /* add history to the output objects and write */
        if (!kpds_set_attribute(sol_obj,KPDS_HISTORY,kpds_history_string()))
          {
            kerror("mlse", "main","Unable to set history on output object");
            kexit(KEXIT_FAILURE);
          }
        kpds_close_object(sol_obj);
        kpds_close_object(sys_obj);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: mlse_usage_additions
| 
|       Purpose: Prints usage additions in mlse_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname mlse
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void mlse_usage_additions(void)
{
	kfprintf(kstderr, "\tLeast Squares Solution to System of Linear Equations\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: mlse_free_args
| 
|       Purpose: Frees CLUI struct allocated in mlse_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname mlse
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
mlse_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
