 /*
  * Khoros: $Id: mpow.c,v 1.14.2.2 1997/05/31 19:39:06 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: mpow.c,v 1.14.2.2 1997/05/31 19:39:06 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for mpow
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "mpow.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Compute the Positive Integral Power of a Square Matrix
|
|       Purpose: main program for mpow
|
|         Input:
|		char *clui_info->i_file; {Input matrix object}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		int clui_info->pow_int; {power to which the matrix should be raised (>=0)}
|		int clui_info->pow_flag; {TRUE if -pow specified}
|
|		char *clui_info->o_file; {Output matrix object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: Scott Wilson
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
kobject in_obj = NULL;
kobject out_obj = NULL;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MATRIX", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MATRIX/objects/kroutine/mpow");
	kexit_handler(mpow_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MATRIX", "mpow", KGEN_KROUTINE,  &clui_uis_spec,
		   mpow_usage_additions, mpow_get_args,
 		   mpow_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        if ((in_obj = kpds_open_input_object(clui_info->i_file))
                == KOBJECT_INVALID)
        {
           kerror("mpow","main", "Cannot open input object.");
           kexit(KEXIT_FAILURE);
        }
       if ((out_obj = kpds_open_output_object(clui_info->o_file))
            == KOBJECT_INVALID)
         {
           kerror("mpow","main", "Cannot open output object.");
           kexit(KEXIT_FAILURE);
         }
       if (!kpds_copy_object_attr(in_obj, out_obj))
         {
           kerror("mpow","main",
             "Unable to copy attributes from input object to output object.");
           kexit(KEXIT_FAILURE);
         }
/* -main_before_lib_call_end */

/* -main_library_call */
        if (!lmpow(in_obj,clui_info->pow_int,out_obj)) kexit(KEXIT_FAILURE);
/* -main_library_call_end */

/* -main_after_lib_call */
        /* add history to the output object and write */
        if (!kpds_set_attribute(out_obj,KPDS_HISTORY,kpds_history_string()))
          {
            kerror("mpow", "main","Unable to set history on output object");
            kexit(KEXIT_FAILURE);
          }
        kpds_close_object(out_obj);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: mpow_usage_additions
| 
|       Purpose: Prints usage additions in mpow_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname mpow
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void mpow_usage_additions(void)
{
	kfprintf(kstderr, "\tCompute the Positive Integral Power of a Square Matrix\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: mpow_free_args
| 
|       Purpose: Frees CLUI struct allocated in mpow_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname mpow
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
mpow_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
