 /*
  * Khoros: $Id: kgeom_update.c,v 1.1.2.2 1997/05/31 19:34:04 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kgeom_update.c,v 1.1.2.2 1997/05/31 19:34:04 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for kgeom_update
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "kgeom_update.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Update Old Geometry Format to New
|
|       Purpose: main program for kgeom_update
|
|         Input:
|		char *clui_info->i_file; {Geometry data in old format}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Geometry data in new format}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	kobject       old_geom;
	kgeom_object *new_geom;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MIGRATION", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MIGRATION/objects/kroutine/kgeom_update");
	kexit_handler(kgeom_update_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MIGRATION", "kgeom_update", KGEN_KROUTINE,  &clui_uis_spec,
		   kgeom_update_usage_additions, kgeom_update_get_args,
 		   kgeom_update_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	if (!(old_geom = kgeom_open_input_object(clui_info->i_file)))
	{
	   kerror("MIGRATION", "kgeom_update", 
		  "Unable to open input file %s for conversion", 
		  clui_info->i_file);
	   kexit(KEXIT_FAILURE);
	}

	if (!(new_geom = kgeom_new_object()))
	{
	   kerror("MIGRATION", "kgeom_update", 
		  "Unable to allocate output geometry object");
	   kexit(KEXIT_FAILURE);
	}
/* -main_before_lib_call_end */

/* -main_library_call */
	if (!transfer_attributes(old_geom, new_geom))
	{
	   kerror("MIGRATION", "kgeom_update", 
		  "Unable to transfer attribute data from input to output");
	   kexit(KEXIT_FAILURE);
	}
	   
	if (!transfer_primitives(old_geom, new_geom))
	{
	   kerror("MIGRATION", "kgeom_update", 
		  "Unable to transfer primitive data from input to output");
	   kexit(KEXIT_FAILURE);
	}
/* -main_library_call_end */

/* -main_after_lib_call */
	if (!kgeom_write_object(new_geom, clui_info->o_file))
	{
	   kerror("MIGRATION", "kgeom_update", 
		  "Unable to write output file %s", clui_info->o_file);
	   kexit(KEXIT_FAILURE);
	}
	
	kgeom_blast_object(new_geom);
	kgeom_close_object(old_geom);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: kgeom_update_usage_additions
| 
|       Purpose: Prints usage additions in kgeom_update_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kgeom_update
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void kgeom_update_usage_additions(void)
{
	kfprintf(kstderr, "\tUpdate Old Geometry Format to New\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: kgeom_update_free_args
| 
|       Purpose: Frees CLUI struct allocated in kgeom_update_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kgeom_update
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
kgeom_update_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
