/*
 * Khoros: $Id: update.c,v 1.1 1997/03/16 22:41:08 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: update.c,v 1.1 1997/03/16 22:41:08 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>             _static_routines()
   >>>>  Public:
   >>>>             public_routines()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "kgeom_update.h"


/*-----------------------------------------------------------
|
|  Routine Name: function_name - short description of function
|
|       Purpose: This should be a complete description that anyone
|                could understand;  it should have acceptable grammar
|                and correct spelling.
|
|         Input: argument1 - explanation
|                argument2 - explanation
|                argument3 - explanation
|
|        Output: argument4 - explanation
|                argument5 - explanation
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By:
|          Date: 
| Modifications:
|
------------------------------------------------------------*/
int
transfer_attributes(
   kobject       old_geom,
   kgeom_object *new_geom)
{
   double ambient;
   double diffuse;
   double s_coef, s_exp;
   double opacity;

   float *matrix;
   float *min, *max;
   float *center;
   float *color;

   int    has_alpha;

   if (!kgeom_get_attributes(old_geom, KGEOM_OBJECT,
			     KGEOM_AMBIENT, &ambient,
			     KGEOM_DIFFUSE, &diffuse,
			     KGEOM_SPECULAR_COEFFICIENT, &s_coef,
			     KGEOM_SPECULAR_EXPONENT,    &s_exp,
			     NULL))
      return FALSE;

   new_geom->ambient_color[0] = (float) ambient;
   new_geom->ambient_color[1] = (float) ambient;
   new_geom->ambient_color[2] = (float) ambient;

   new_geom->diffuse_color[0] = (float) diffuse;
   new_geom->diffuse_color[1] = (float) diffuse;
   new_geom->diffuse_color[2] = (float) diffuse;

   new_geom->specular_color[0] = (float) s_coef;
   new_geom->specular_color[1] = (float) s_coef;
   new_geom->specular_color[2] = (float) s_coef;

   new_geom->specular_exponent = (float) s_exp;
   
   if (!kgeom_get_attributes(old_geom, KGEOM_OBJECT, 
			KGEOM_MODELING_SPACE, 	  &new_geom->modeling_space,
			KGEOM_VISIBLE,        	  &new_geom->visible, 
			KGEOM_LAYOUT,         	  &new_geom->layout,
			KGEOM_LOCATION_SIZE,   	  &new_geom->location_dim,
			KGEOM_TEXTURE_COORD_SIZE, &new_geom->texture_coord_dim,
			NULL))
      return FALSE;

   if (kgeom_get_attribute(old_geom, KGEOM_OBJECT, 
			   KGEOM_BOUNDING_BOX, &min, &max))
   {
      new_geom->has_bounding_box = TRUE;
      kmemcpy(new_geom->min, min, (size_t) 3 * sizeof(float));
      kmemcpy(new_geom->max, max, (size_t) 3 * sizeof(float));
   }

   if (kgeom_get_attribute(old_geom, KGEOM_OBJECT, KGEOM_CENTER, &center))
   {
      new_geom->has_center = TRUE;
      kmemcpy(new_geom->center, center, (size_t) 3 * sizeof(float));
   }

   if (kgeom_get_attribute(old_geom, KGEOM_OBJECT, KGEOM_MATRIX, &matrix))
   {
      new_geom->has_matrix = TRUE;
      kmemcpy(new_geom->matrix, matrix, (size_t) 16 * sizeof(float));
   }

   if (kgeom_get_attribute(old_geom, KGEOM_OBJECT, KGEOM_COLOR, &color))
   {
      new_geom->has_color = TRUE;
      kmemcpy(new_geom->color, color, (size_t) 3 * sizeof(float));
   }
   
   if (!kgeom_get_attributes(old_geom, KGEOM_OBJECT, 
			     KGEOM_HAS_ALPHA, &has_alpha,
			     KGEOM_OPACITY,   &opacity,
			     NULL))
      return FALSE;
   
   new_geom->has_alpha = (kubyte) has_alpha;
   new_geom->opacity   = (float)  opacity;

   return TRUE;
}


/*-----------------------------------------------------------
|
|  Routine Name: function_name - short description of function
|
|       Purpose: This should be a complete description that anyone
|                could understand;  it should have acceptable grammar
|                and correct spelling.
|
|         Input: argument1 - explanation
|                argument2 - explanation
|                argument3 - explanation
|
|        Output: argument4 - explanation
|                argument5 - explanation
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By:
|          Date: 
| Modifications:
|
------------------------------------------------------------*/
int
transfer_primitives(
   kobject       old_geom,
   kgeom_object *new_geom)
{
   int  i, num;
   int *primlist;
   
   if (!kgeom_get_attributes(old_geom, KGEOM_OBJECT, 
			     KGEOM_NUMBER_PRIMITIVES, &num,
			     KGEOM_PRIMITIVE_LIST,    &primlist,
			     NULL))
      return FALSE;
   
   /*-- this will handle most cases --*/
   kgeom_set_attributes(old_geom, KGEOM_OBJECT,
			KGEOM_LOCATION_DATA_TYPE,      KFLOAT,
			KGEOM_COLOR_DATA_TYPE,         KFLOAT,
			KGEOM_NORMAL_DATA_TYPE,        KFLOAT,
			KGEOM_RADIUS_DATA_TYPE,        KFLOAT,
			KGEOM_TEXTURE_COORD_DATA_TYPE, KFLOAT,
			NULL);

   for (i = 0; i < num; i++)
   {
      kgeom_primitive *p = kgeom_new_primitive(primlist[i]);
      
      kgeom_set_attribute(old_geom, KGEOM_OBJECT, 
			  KGEOM_PRIMITIVE_POSITION, i);

      switch (primlist[i])
      {
	 case KGEOM_POLYLINE_DISJOINT :
	 case KGEOM_POLYLINE_CONNECTED :
	    {
	       kgeom_get_attributes(old_geom, primlist[i],
				    KGEOM_NUMBER_VERTICES, &p->lines.nverts,
				    KGEOM_LINE_TYPE,       &p->lines.linetype,
				    KGEOM_LINE_WIDTH,      &p->lines.linewidth,
				    NULL);
	       
	       kgeom_get_data(old_geom, primlist[i],
			      &p->lines.locs, 
			      &p->lines.cols);
	    } break;

	 case KGEOM_TRIANGLES_DISJOINT :
	 case KGEOM_TRIANGLES_CONNECTED :
	    {
	       kgeom_get_attributes(old_geom, primlist[i],
				 KGEOM_NUMBER_VERTICES,&p->triangles.nverts,
				 NULL);
	       
	       kgeom_get_data(old_geom, primlist[i],
			      &p->triangles.locs, 
			      &p->triangles.cols,
			      &p->triangles.norms,
			      &p->triangles.tcoords);
	    } break;

	 case KGEOM_SPHERES :
	    {
	       kgeom_get_attributes(old_geom, primlist[i],
				    KGEOM_NUMBER_VERTICES, &p->spheres.nverts,
				    NULL);
	       
	       kgeom_get_data(old_geom, primlist[i],
			      &p->spheres.locs, 
			      &p->spheres.cols,
			      &p->spheres.radii);
	    } break;
	 
	 case KGEOM_TEXT :
	    {
	       kgeom_get_attributes(old_geom, primlist[i],
				    KGEOM_NUMBER_VERTICES, &p->text.nverts,
				    NULL);
	       
	       kgeom_get_data(old_geom, primlist[i],
			      &p->text.strings, 
			      &p->text.fonts,
			      &p->text.locs,
			      &p->text.offsets,
			      &p->text.ups,
			      &p->text.flows,
			      &p->text.cols,
			      &p->text.bgcols);
	    } break;

	 case KGEOM_CYLINDERS :
	    {
	       kgeom_get_attributes(old_geom, primlist[i],
			    KGEOM_NUMBER_VERTICES, &p->cylinders.nverts,
			    NULL);
	       
	       kgeom_get_data(old_geom, primlist[i],
			      &p->cylinders.locs, 
			      &p->cylinders.cols,
			      &p->cylinders.rad1,
			      &p->cylinders.rad2);
	    } break;

	 case KGEOM_DIRECTED_POINTS :
	    {
	       kgeom_get_attributes(old_geom, primlist[i],
			    KGEOM_NUMBER_VERTICES, &p->directed_points.nverts,
			    NULL);
	       
	       kgeom_get_data(old_geom, primlist[i],
			      &p->directed_points.locs, 
			      &p->directed_points.cols,
			      &p->directed_points.norms);
	    } break;

	 case KGEOM_MARKERS :
	    {
	       kgeom_get_attributes(old_geom, primlist[i],
			    KGEOM_NUMBER_VERTICES, &p->markers.nverts,
			    KGEOM_MARKER_TYPE,     &p->markers.markertype,
			    NULL);
	       
	       kgeom_get_data(old_geom, primlist[i],
			      &p->markers.locs, 
			      &p->markers.cols,
			      &p->markers.bgcols);
	    } break;

	 case KGEOM_POLYGON :
	    {
	       kgeom_get_attributes(old_geom, primlist[i],
				  KGEOM_NUMBER_VERTICES, &p->polygon.nverts,
				  KGEOM_LINE_TYPE,       &p->polygon.linetype,
				  KGEOM_LINE_WIDTH,      &p->polygon.linewidth,
				  NULL);
	       
	       kgeom_get_data(old_geom, primlist[i],
			      &p->polygon.locs, 
			      &p->polygon.cols,
			      &p->polygon.bgcol);
	    } break;

	 case KGEOM_RECTANGLES :
	    {
	       kgeom_get_attributes(old_geom, primlist[i],
			KGEOM_NUMBER_VERTICES, &p->rectangles.nverts,
			KGEOM_LINE_TYPE,       &p->rectangles.linetype,
			KGEOM_LINE_WIDTH,      &p->rectangles.linewidth,
			NULL);
	       
	       kgeom_get_data(old_geom, primlist[i],
			      &p->rectangles.locs, 
			      &p->rectangles.cols,
			      &p->rectangles.bgcols);
	    } break;

	 case KGEOM_ELLIPSES :
	    {
	       kgeom_get_attributes(old_geom, primlist[i],
			KGEOM_NUMBER_VERTICES, &p->ellipses.nverts,
			KGEOM_LINE_TYPE,       &p->ellipses.linetype,
			KGEOM_LINE_WIDTH,      &p->ellipses.linewidth,
			NULL);
	       
	       kgeom_get_data(old_geom, primlist[i],
			      &p->ellipses.locs, 
			      &p->ellipses.cols,
			      &p->ellipses.bgcols,
			      &p->ellipses.rad1,
			      &p->ellipses.rad2);
	    } break;

	 case KGEOM_QUADMESH :
	    {
	       kgeom_get_attributes(old_geom, primlist[i],
				    KGEOM_QUADMESH_SIZE, 
				    &p->quadmesh.width,
				    &p->quadmesh.height,
				    KGEOM_TEXTURE, &p->quadmesh.texture,
				    NULL);

	       kgeom_get_data(old_geom, primlist[i],
			      &p->quadmesh.locs, 
			      &p->quadmesh.cols,
			      &p->quadmesh.norms,
			      &p->quadmesh.tcoords);
	    } break;

	 case KGEOM_OCTMESH :
	    {
	       kgeom_get_attributes(old_geom, primlist[i],
				    KGEOM_OCTMESH_SIZE, 
				    &p->octmesh.width,
				    &p->octmesh.height,
				    &p->octmesh.depth,
				    KGEOM_TEXTURE, &p->octmesh.texture,
				    NULL);

	       kgeom_set_attribute(old_geom, KGEOM_OBJECT, 
				   KGEOM_COLOR_DATA_TYPE, KUBYTE);

	       kgeom_get_data(old_geom, primlist[i],
			      &p->octmesh.locs, 
			      &p->octmesh.cols,
			      &p->octmesh.norms,
			      &p->octmesh.tcoords);

	       kgeom_set_attribute(old_geom, KGEOM_OBJECT, 
				   KGEOM_COLOR_DATA_TYPE, KFLOAT);
	    } break;

	 case KGEOM_TEXTURE2D :
	    {
	       kgeom_get_attributes(old_geom, primlist[i],
				    KGEOM_TEXTURE2D_SIZE, 
				    &p->texture2D.width,
				    &p->texture2D.height,
				    NULL);

	       kgeom_set_attribute(old_geom, KGEOM_OBJECT, 
				   KGEOM_COLOR_DATA_TYPE, KUBYTE);

	       kgeom_get_data(old_geom, primlist[i],
			      &p->texture2D.cols);

	       kgeom_set_attribute(old_geom, KGEOM_OBJECT, 
				   KGEOM_COLOR_DATA_TYPE, KFLOAT);

	    } break;

	 case KGEOM_TEXTURE3D :
	    {
	       kgeom_get_attributes(old_geom, primlist[i],
				    KGEOM_TEXTURE3D_SIZE, 
				    &p->texture3D.width,
				    &p->texture3D.height,
				    &p->texture3D.depth,
				    NULL);

	       kgeom_set_attribute(old_geom, KGEOM_OBJECT, 
				   KGEOM_COLOR_DATA_TYPE, KUBYTE);

	       kgeom_get_data(old_geom, primlist[i],
			      &p->texture3D.cols);

	       kgeom_set_attribute(old_geom, KGEOM_OBJECT, 
				   KGEOM_COLOR_DATA_TYPE, KFLOAT);
	    } break;

	 case KGEOM_VIEW_XFORM :
	    {
	       kprintf("encountered primitive KGEOM_VIEW_XFORM : "
		       "not supported for conversion\n");
	    } break;

	 case KGEOM_OBJECT_XFORM :
	    {
	       kprintf("encountered primitive KGEOM_OBJECT_XFORM : "
		       "not supported for conversion\n");
	    } break;

	 case KGEOM_CAMERA_ORIENTATION :
	    {
	       kprintf("encountered primitive KGEOM_CAMERA_ORIENTATION : "
		       "not supported for conversion\n");
	    } break;

	 case KGEOM_VR_EVENT :
	    {
	       kprintf("encountered primitive KGEOM_VR_EVENT : "
		       "not supported for conversion\n");
	    } break;

	 default:
	    {
	       kprintf("encountered unknown primitive type <%d>\n", 
		       primlist[i]);
	    } break;
      }

      kgeom_add_primitive(new_geom, p);
   }
   
   return TRUE;
}

