/*
  * Khoros: $Id: updateuis.c,v 1.8 1997/01/20 07:23:58 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: updateuis.c,v 1.8 1997/01/20 07:23:58 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

#include "uisupdate.h"

/*-----------------------------------------------------------
| 
|  Routine Name: update_uis
| 
|       Purpose: Updates the UIS file
| 
|         Input: filename - old UIS file to be updated
|        Output: 
|       Returns: TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: May 11, 1993
| Modifications: 
| 
------------------------------------------------------------*/
static int use_count PROTO((ktoken, ktoken *, int));

int update_uis(
    char *filename,
    int  force)
{
	char       temp[KLENGTH];
	char       *startsubform_line, *startpane_line;
	int        flag, i;
	int        last_subform_index = 0, last_guide_index = 0;
	int        blank_count = 0, help_count = 0;
	int        doing_toggle = 0, toggle_count = 0, toggle_def = 0,var_count;
	ktoken     var_token; 
	int        used_subform_var_count = 0, clean_lines = TRUE;
	ktoken     *used_subform_vars;
	kform      *form;
	Line_Info  line_info, tmp_line_info;
	char       *routine = "update_uis";
	uis_struct *uis_spec;
	size_t     *file_lookup;
	size_t     *line_lookup;
	char       **filenames;
	char       **uis;

        uis_spec = kvf_read_uis_file(filename);
        if (uis_spec == NULL) return(FALSE);

	uis         = uis_spec->uis;
	line_lookup = uis_spec->line_lookup;
	file_lookup = uis_spec->file_lookup;
	filenames   = uis_spec->filenames;

	if (kstrcmp(uis[0], "# Khoros Image Processing System Workspace") == 0)
	    clean_lines = FALSE;

	if ((used_subform_vars = (ktoken *) kmalloc(sizeof(ktoken))) == NULL)
        {
	    kerror(NULL, routine, "Can't alloc array of used subform names "
                   "-- updated UIS file may contain subform button lines with "
                   "identical variable names that will have to be modified "
                   "by hand.");
        }

	kvf_clear_line_info(&line_info);

	for (i = 0; i < uis_spec->linenum; i++)
	{
	     flag = kvf_get_line_type(uis[i]);
	     switch(flag)
	     {
		case KUIS_STARTFORM: 
		     if (!(kvf_oldparse_startform_line(uis[i], &line_info))) 
			return(FALSE);
		     line_info.width  = 20.0;
		     line_info.height = 5.0;
		     kvf_deparse_startform_line(&line_info, &uis[i]);
		     break;

		case KUIS_STARTSUBFORM: 
		     if (!(kvf_oldparse_startsubform_line(uis[i], &line_info))) 
			return(FALSE);
		     line_info.width  = 20.0;
		     line_info.height = 5.0;
		     kvf_deparse_startsubform_line(&line_info, &uis[i]);
		     break;

		case KUIS_PSEUDOSUBFORM: 
		     if (!(kvf_oldparse_psuedosubform_line(uis[i], &line_info)))
			return(FALSE);
		     kvf_deparse_subformbutton_line(&line_info, &uis[i]);
		     break;

		case KUIS_STARTPANE: 
		     if (!(kvf_oldparse_startpane_line(uis[i], &line_info))) 
			return(FALSE);
		     line_info.selected = FALSE;
		     kvf_deparse_startpane_line(&line_info, &uis[i]);
		     break;

		case KUIS_BLANK: 
		     kfree_and_NULL(line_info.variable);
		     if (kvf_oldparse_blank_line(uis[i], &line_info))
		     {
		         ksprintf(temp, "blank%d", blank_count++);
		         if (line_info.variable == NULL)
		             line_info.variable = kstrdup(temp);
		         kvf_deparse_blank_line(&line_info, &uis[i]);
		         kvf_free_line_info_strings(&line_info);
		     }
		     else
		     {
			kerror(NULL, routine,
			       "Unable to update Blank on line %d of %s",
			       line_lookup[i], filenames[file_lookup[i]]); 
		     }
		     break;

		case KUIS_HELP: 
		     kfree_and_NULL(line_info.variable);
		     if (kvf_oldparse_help_line(uis[i], &line_info))
		     {
			 if (help_count == 0)
		             ksprintf(temp, "help", help_count++);
		         else ksprintf(temp, "help%d", help_count++);
		         if (line_info.variable == NULL)
		             line_info.variable = kstrdup(temp);
			 line_info.height = 1.0;
			 line_info.width  = (float) (kstrlen(line_info.title)+2.0);
		         kvf_deparse_help_line(&line_info, &uis[i]);
		         kvf_free_line_info_strings(&line_info);
		     }
		     else
		     {
			kerror(NULL, "update_uis",
			       "Unable to update Help on line %d of %s",
			       line_lookup[i], filenames[file_lookup[i]]); 
		     }
		     break;

		case KUIS_QUIT:
		     if (kvf_oldparse_quit_line(uis[i], &line_info))
                     {
                         line_info.height = 1.0;
                         line_info.width  = 6.0;
                         kvf_deparse_quit_line(&line_info, &uis[i]);
                         kvf_free_line_info_strings(&line_info);
                     }
                     else
                     {
                        kerror(NULL, routine,
                               "Unable to update Quit on line %d of %s",
                               line_lookup[i], filenames[file_lookup[i]]);
                     }
		     break;


		case KUIS_TOGGLE:
		     doing_toggle = TRUE;
		     toggle_count = 0;
		     if (kvf_oldparse_toggle_line(uis[i], &line_info))
			toggle_def = line_info.toggle_def;
		     else
                     {
                        kerror(NULL, routine,
                               "Unable to update %s on line %d of %s",
                               kvf_ascii_typeflag(flag),
                               line_lookup[i], filenames[file_lookup[i]]);
                     }
		     break;

		case KUIS_INTEGER:
		case KUIS_FLOAT:
		case KUIS_CYCLE:
		case KUIS_LIST:
		case KUIS_STARTSUBMENU:
		case KUIS_INPUTFILE:
		case KUIS_OUTPUTFILE:
		case KUIS_LOGICAL:
		case KUIS_STRING:
		case KUIS_FLAG:
		case KUIS_STRINGLIST:
		case KUIS_WORKSPACE:
		     if (kvf_gen_oldparse(uis[i], &line_info))
		     {
		         if (flag == KUIS_CYCLE || flag == KUIS_LIST 
			     || flag == KUIS_STRINGLIST) 
			 {
		             line_info.int_val = 1;
			     if (flag != KUIS_STRINGLIST)
			         line_info.list_val += 1;
			 }
		         if (flag == KUIS_INTEGER || flag == KUIS_FLOAT) 
			 {
		             line_info.special   = 1;
		             line_info.precision = 0;
			 }
			 if (doing_toggle) 
			 {
			     toggle_count++;
			     if (toggle_count == toggle_def)
			         line_info.opt_sel = 1;
			     else line_info.opt_sel = 0;
			     line_info.optional = 1;
			 }
			 line_info.write = clean_lines;

			 if (!line_info.optional)
			     line_info.opt_sel = 1;
		         kvf_gen_deparse(&line_info, &uis[i]);
	             }
		     else
		     {
			kerror(NULL, routine,
			       "Unable to update %s on line %d of %s",
			       kvf_ascii_typeflag(flag),
			       line_lookup[i], filenames[file_lookup[i]]); 
		     }
		     break;

		case KUIS_END:
		     if (doing_toggle) doing_toggle = FALSE;
		     break;
	     }

	}

	kvf_clear_line_info(&tmp_line_info);
	for (i = 0; i < uis_spec->linenum; i++)
        {
             flag = kvf_get_line_type(uis[i]);
             switch(flag)
             {
		case KUIS_SUBFORMBUTTON:
	             if (last_subform_index < i) 
			last_subform_index = i;
		     kvf_oldparse_subformbutton_line(uis[i], &line_info);
		     startsubform_line = kvf_oldfind_matching_M_line(uis_spec, 
					               &last_subform_index, i);
		     if (startsubform_line == NULL)
			return(FALSE);
		     kvf_oldparse_startsubform_line(startsubform_line, 
						    &tmp_line_info);

		     var_token = kstring_to_token(tmp_line_info.variable);
		     if ((var_count = use_count(var_token, used_subform_vars, 
				                used_subform_var_count)) > 0)
			sprintf(temp, "%s_%d", tmp_line_info.variable, 
					       var_count+1);
		     else sprintf(temp, "%s", tmp_line_info.variable);

		     if ((used_subform_vars = (ktoken *) 
			 krealloc(used_subform_vars, 
			 (used_subform_var_count+1) * sizeof(ktoken))) == NULL)
		     {
			kerror(NULL, routine, "Can't alloc array of used "
                               "subform names -- updated UIS file may contain "
                               "subform button lines with identical variable "
                               "names that will have to be modified by hand.");
		     }
		     if (used_subform_vars != NULL)
		          used_subform_vars[used_subform_var_count++] = 
								var_token;
		 
		     line_info.variable = kstrdup(temp);
		     kvf_deparse_subformbutton_line(&line_info, &uis[i]);
		     tmp_line_info.variable = kstrdup(temp);
		     kvf_deparse_startsubform_line(&tmp_line_info, 
					   &uis[last_subform_index]);
                     last_subform_index++;
		     break;
		     

		case KUIS_GUIDEBUTTON:
		     kvf_oldparse_guide_line(uis[i], &line_info);
	             if (last_guide_index < i) 
			last_guide_index = i;
		     startpane_line = kvf_oldfind_matching_P_line(uis_spec, 
					&last_guide_index, i);
		     if (startpane_line == NULL)
			return(FALSE);
		     kvf_oldparse_startpane_line(startpane_line, 
						 &tmp_line_info);

                     line_info.variable = kstrdup(tmp_line_info.variable);
		     kvf_deparse_guide_line(&line_info, &uis[i]);
                     last_guide_index++;
		     break;
		     
	     }
	}


	form = kvf_build_form(uis_spec, KNONE, NULL, NULL);
	if (form == NULL)
	{
	    kerror(NULL, routine,  "Cannot create updated UIS file due "
                   "to syntax error(s) in original UIS file(s).");
	    return(FALSE);
	}
	kvf_print_UIS_file(form, filename, force, TRUE);

	return(TRUE);
}

static int use_count(
     ktoken var_token,
     ktoken *used_token_array,
     int used_token_array_size)
{
	int i, count;

	if (used_token_array == NULL) return (0);

	count = 0;
	for (i = 0; i < used_token_array_size; i++)
	{
	    if (used_token_array[i] == var_token)
		count++;
	}
	return(count);
}

/*----------------------------------------------------------
|
| Routine Name: kvf_oldfind_matching_M_line
|
|      Purpose: A problem results when we must name the (-d) subform button
|               according to the name on the [-M] line, which hasn't
|               been read in yet.  This routine finds the (-M) line to match
|               the (-d) line, to solve that problem.
|
|         Input: uis_spec           - UIS structure
|                last_subform_index - for efficiency's sake, points
|                                     to the last [-M] line found, so we
|                                     don't have to search the entire UIS again
|                sfb_index          - index into the UIS of current -d line
|
|        Output: Returns the -M line on success, NULL on failure.
|          Date: Apr 10, 1992
|    Written By: Danielle Argiro
|
---------------------------------------------------------------*/

char *kvf_oldfind_matching_M_line(
   uis_struct *uis_spec,
   int        *last_subform_index,
   int         sfb_index)
{
        int       flag;
        Line_Info line_info;
        char      *control_line;
	char      *routine = "kvf_oldfind_matching_M_line";

	char      **uis        = uis_spec->uis;
	size_t    *line_lookup = uis_spec->line_lookup;
	size_t    *file_lookup = uis_spec->file_lookup;
	char      **filenames   = uis_spec->filenames;

        flag = kvf_get_line_type(uis[*last_subform_index]);
        if (flag == -1) return(NULL);

        while (flag != KUIS_STARTSUBFORM)
        {
            (*last_subform_index)++;
            if (uis[*last_subform_index] == NULL)
            {
	        errno = KUIS_SYNTAX;
                kerror(NULL, routine, "Cannot find [-M] line that matches the "
                       "SubformButton [-d] line on line %d of %s;  aborting "
                       "form creation.", line_lookup[sfb_index], 
                        filenames[file_lookup[sfb_index]]);
                return(NULL);
            }
            flag = kvf_get_line_type(uis[*last_subform_index]);
            if (flag == -1) return(NULL);
        }
        kvf_clear_line_info(&line_info);
        if (!(kvf_parse_startsubform_line(uis[*last_subform_index],
                                          &line_info)))
        {
	     errno = KUIS_SYNTAX;
             kerror(NULL, routine, "Cannot find [-M] line that matches the "
                    "SubformButton [-d] line on line %d of file %s;  aborting "
                    "form creation", line_lookup[sfb_index], 
                     filenames[file_lookup[sfb_index]]);
             return(NULL);
        }
        else control_line = uis[*last_subform_index];

        kvf_free_line_info_strings(&line_info);

        return(control_line);
}

/*------------------------------------------------------------
|
| Routine Name: kvf_oldfind_matching_P_line
|
|      Purpose: A problem results when we must name the (-g) guide button
|               according to the name on the [-P] line, which hasn't
|               been read in yet.  This routine finds the (-P) line to match
|               the (-g) line, to solve that problem.
|
|         Input: uis_spec          - UIS structure
|                last_guide_index  - for efficiency's sake, points
|                                    to the last [-P] line found, so we
|                                    don't have to search the entire UIS again
|                gb_index          - index into the UIS of (-g) line
|
|        Output: Returns the -P line on success, NULL on failure.
|          Date: Apr 10, 1992
|    Written By: Danielle Argiro
|
---------------------------------------------------------------*/

char *kvf_oldfind_matching_P_line(
   uis_struct *uis_spec,
   int        *last_guide_index,
   int        gb_index)
{
        int       flag;
        char      *control_line;
        Line_Info line_info;
        char      *routine = "kvf_oldfind_matching_M_line";

        char      **uis        = uis_spec->uis;
        size_t    *line_lookup = uis_spec->line_lookup;
        size_t    *file_lookup = uis_spec->file_lookup;
        char      **filenames   = uis_spec->filenames;


        flag = kvf_get_line_type(uis[*last_guide_index]);
        if (flag == -1) return(NULL);

        while (flag != KUIS_STARTPANE)
        {
            (*last_guide_index)++;
            if (uis[*last_guide_index] == NULL)
            {
		errno = KUIS_SYNTAX;
                kerror(NULL, routine, "Cannot find [-P] line that matches the "
                       "GuideButton [-g] line on line %d of %s;  aborting form "
                       "creation.", line_lookup[gb_index], 
                       filenames[file_lookup[gb_index]]);
                return(NULL);
            }
            flag = kvf_get_line_type(uis[*last_guide_index]);
            if (flag == -1) return(NULL);
        }
        kvf_clear_line_info(&line_info);
        if (!(kvf_parse_startpane_line(uis[*last_guide_index], &line_info)))
        {
	     errno = KUIS_SYNTAX;
             kerror(NULL, routine, "Cannot find [-P] line that matches the "
                    "GuideButton [-g] line on line %d of %s;  aborting form "
                    "creation", line_lookup[gb_index], 
                    filenames[file_lookup[gb_index]]);
             return(NULL);
        }
        else control_line = uis[*last_guide_index];

        kvf_free_line_info_strings(&line_info);
        return(control_line);
}

