 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vchghomot
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vchghomot.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Minima Imposition
|
|       Purpose: main program for vchghomot
|
|         Input:
|		char *clui_info->i1_file; {Image to be transformed}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {Image with reference points}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->str_file; {Structuring Element which defines the connectivity used }
|		int   clui_info->str_flag; {TRUE if -str specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: Junior Barrera
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage *img1, *img2, *img3, *img4, *images[3];
int num_images;
mat_str b;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vchghomot");
	kexit_handler(vchghomot_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vchghomot", KGEN_KROUTINE,  &clui_uis_spec,
		   vchghomot_usage_additions, vchghomot_get_args,
 		   vchghomot_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        num_images = 0;         /* initialize counter */

        img1 = readimage(clui_info->i1_file);
        if (img1 == NULL)
          kexit(1);                    
        images[num_images++] = img1;

        img2 = readimage(clui_info->i2_file);
        if (img2 == NULL)
          kexit(1);                    
        images[num_images++] = img2;

        CHECKINPUT(program, img1, img2);  /* Verify input images */

        read_str(clui_info->str_file, &b);
/*
        if (clui_info->m_flag) {
          img3 = readimage(clui_info->m_file);
          images[num_images++] = img3;
          if (img3 == NULL)
            kexit(1);
          CHECKINPUT(program, img1, img3); 
        }
*/
        if (img1->data_storage_type == VFF_TYP_BIT) {
          kfprintf(kstderr, "vhomot: BIT not defined");
          kexit(1);
        }

        img4 = createimage(img1->col_size, img1->row_size, img1->data_storage_type, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);  

        if (img4 == NULL) {
          kfprintf(kstderr, "vhomot: createimage Failed\n");
          kexit(1);
        }
/* -main_before_lib_call_end */

/* -main_library_call */
if (!lvhomot(img1, img2, &b, img3, 0/*clui_info->m_flag*/, img4)) {
          (void) kfprintf(kstderr, "lvhomot Failed\n");
          kexit(1);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
(void) copyheader(img1, img4);
        copymapdata(img1, &img4, 1);
        (void) writeimage(clui_info->o_file, img4);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vchghomot_usage_additions
| 
|       Purpose: Prints usage additions in vchghomot_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vchghomot
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vchghomot_usage_additions(void)
{
	kfprintf(kstderr, "\tMinima Imposition\n");

/* -usage_additions */
kfprintf(kstderr, "\tOptional fourth input is a masking image.\n");
        kfprintf(kstderr, "\t\n");
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vchghomot_free_args
| 
|       Purpose: Frees CLUI struct allocated in vchghomot_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vchghomot
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vchghomot_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->str_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
