 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vclorec
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vclorec.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Sup-Reconstruction
|
|       Purpose: main program for vclorec
|
|         Input:
|		char *clui_info->i1_file; {first input image}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {second input image}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->str_file; {Structuring Element}
|		int   clui_info->str_flag; {TRUE if -str specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: Adriano Nagelschmidt Rodrigues
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage *img1, *img2, *img3, *images[3];
xvimage *img11, *img22, *img33;
int num_images, up_type;
unsigned long border;
mat_str b;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vclorec");
	kexit_handler(vclorec_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vclorec", KGEN_KROUTINE,  &clui_uis_spec,
		   vclorec_usage_additions, vclorec_get_args,
 		   vclorec_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        num_images = 0;         /* initialize counter */

        img1 = readimage(clui_info->i1_file);
        if (img1 == NULL)
          kexit(1);                    
        images[num_images++] = img1;

        img2 = readimage(clui_info->i2_file);
        if (img2 == NULL)
          kexit(1);                    
        images[num_images++] = img2;

        read_str(clui_info->str_file, &b);
/*
        if (clui_info->m_flag) {
          img3 = readimage(clui_info->m_file);
          images[num_images++] = img3;
        }

        if ((clui_info->m_flag) && (img3 == NULL)) kexit(1);         

        if (clui_info->m_flag)  { 
          CHECKINPUT(program, img1, img3);  
        }
*/         
        CHECKINPUT(program, img1, img2);      /* Verify input images */

        up_type = -1;    /* call lvcast1 with auto upconvert */

                         /*  Call lvcast1 to upconvert data types */
        if (!lvcast1(images, num_images, up_type)) {
          (void) kfprintf(kstderr, "ERROR: images could not be cast to the same type\n");
          kexit(1);
        }

       if (img1->data_storage_type == VFF_TYP_BIT) {
         img11 = format(img1, &border);
         img22 = format(img2, &border);
/*
         if (clui_info->m_flag)
           img33 = format(img3, &border);
*/
       }
       else {
         img11 = img1;
         img22 = img2;
         img33 = img3;
       }
/* -main_before_lib_call_end */

/* -main_library_call */
if (!lvclosrec(img11, img22, &b, img33, 0/*clui_info->m_flag*/, border, img22)) {
          (void) kfprintf(kstderr, "lvclosrec Failed\n");
          kexit(1);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
if (img1->data_storage_type == VFF_TYP_BIT)
          (void) unformat(img22, img2);
        (void) writeimage(clui_info->o_file, img2);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vclorec_usage_additions
| 
|       Purpose: Prints usage additions in vclorec_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vclorec
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vclorec_usage_additions(void)
{
	kfprintf(kstderr, "\tSup-Reconstruction\n");

/* -usage_additions */
kfprintf(kstderr, "\tOptional fourth input is a masking image.\n");
        kfprintf(kstderr, "\t\n");
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vclorec_free_args
| 
|       Purpose: Frees CLUI struct allocated in vclorec_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vclorec
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vclorec_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->str_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
