 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vdil
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vdil.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Conditionally Translation Invariant Dilation
|
|       Purpose: main program for vdil
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->str_file; {Structuring Element}
|		int   clui_info->str_flag; {TRUE if -str specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: Junior Barrera & Roberto Lotufo & Ulisses Braga
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage *img1, *img2, *img3, *images[2];
xvimage *img11, *img22, *img33;
int num_images, up_type;
unsigned long border;
mat_str b;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vdil");
	kexit_handler(vdil_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vdil", KGEN_KROUTINE,  &clui_uis_spec,
		   vdil_usage_additions, vdil_get_args,
 		   vdil_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        num_images = 0;         /* initialize counter */

        img1 = readimage(clui_info->i_file);
        if (img1 == NULL){
	  kerror(NULL, "vdil","cannot open %s",clui_info->i_file);
	  kexit(1);
	}
        images[num_images++] = img1;

        read_str(clui_info->str_file, &b);
/* remove mask input , ral 12nov94
        if (clui_info->m_flag) {
           img2= readimage(clui_info->m_file);
           images[num_images++] = img2;
        }
        if ((clui_info->m_flag) && (img2 == NULL)) kexit(1);         
        if (clui_info->m_flag)  { 
            CHECKINPUT(program, img1, img2);
        }
*/
         
        up_type = -1;    /* call lvcast1 with auto upconvert */

                         /*  Call lvcast1 to upconvert data types */
        if (! lvcast1(images, num_images, up_type)) {
           (void) kfprintf(kstderr, "ERROR: images could not be cast to the same type\n");
           kexit(1);
        }

        img3 = createimage(img1->col_size, img1->row_size, img1->data_storage_type, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);

       if (img3 == NULL) {
         (void) kfprintf(kstderr, "createimage Failed\n");
          kexit(1);
       }

       if (img1->data_storage_type == VFF_TYP_BIT) {
         img11 = format(img1, &border);
/*
         if (clui_info->m_flag)
           img22 = format(img2, &border);
*/
         img33 = format(img3, &border);
       }
       else {
         img11 = img1;
         img22 = img2;
         img33 = img3;
       }
/* -main_before_lib_call_end */

/* -main_library_call */
if (!lvdil(img11, &b, img22, 0 /* clui_info->m_flag */, border, img33)) {
          (void) kfprintf(kstderr, "lvdil Failed\n");
          kexit(1);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
if (img1->data_storage_type == VFF_TYP_BIT)
          (void) unformat(img33, img3);
        (void) copyheader(img1, img3);
        copymapdata(img1, &img3, 1);
        (void) writeimage(clui_info->o_file, img3);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vdil_usage_additions
| 
|       Purpose: Prints usage additions in vdil_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vdil
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vdil_usage_additions(void)
{
	kfprintf(kstderr, "\tConditionally Translation Invariant Dilation\n");

/* -usage_additions */
kfprintf(kstderr, "\tOptional third input is a masking image.\n");
        kfprintf(kstderr, "\t\n");
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vdil_free_args
| 
|       Purpose: Frees CLUI struct allocated in vdil_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vdil
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vdil_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->str_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
