 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vdilexpan
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vdilexpan.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Expansion by Dilation
|
|       Purpose: main program for vdilexpan
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->str_file; {Structuring Element}
|		int   clui_info->str_flag; {TRUE if -str specified}
|
|		int clui_info->vs_int; {expansion scale factor}
|		int clui_info->vs_flag; {TRUE if -vs specified}
|
|		int clui_info->hs_int; {expansion scale factor}
|		int clui_info->hs_flag; {TRUE if -hs specified}
|
|		int clui_info->voff_int; {expansion scale factor}
|		int clui_info->voff_flag; {TRUE if -voff specified}
|
|		int clui_info->hoff_int; {expansion scale factor}
|		int clui_info->hoff_flag; {TRUE if -hoff specified}
|
|        Output:
|       Returns:
|
|    Written By: Marco Antonio Carvalho Pessoa
Gerald Jean Francis Banon (for last revision)
Sao Jose dos Campos, SP, Brazil, November, 9th, 94
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage *img1, *img2, *img3, *images[2];
xvimage *img11, *img22, *img33;
int num_images, up_type;
unsigned long border;
mat_str b;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vdilexpan");
	kexit_handler(vdilexpan_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vdilexpan", KGEN_KROUTINE,  &clui_uis_spec,
		   vdilexpan_usage_additions, vdilexpan_get_args,
 		   vdilexpan_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        num_images = 0;         /* initialize counter */

        img1 = readimage(clui_info->i_file);
        if (img1 == NULL)
           kexit(1);                    
        images[num_images++] = img1;

        if (clui_info->str_flag)
           read_str(clui_info->str_file, &b);
/* 
        if (clui_info->m_flag) {
           img2= readimage(clui_info->m_file);
           images[num_images++] = img2;
        }

        if ((clui_info->m_flag) && (img2 == NULL)) kexit(1);         

        if (clui_info->m_flag)  { 
           CHECKINPUT(program, img1, img2);  
        }
*/
        up_type = - 1;    /* call lvcast1 with auto upconvert */

                         /*  Call lvcast1 to upconvert data types */
        if (!lvcast1(images, num_images, up_type)) {
           (void) kfprintf(kstderr, "ERROR: images could not be cast to the same type\n");
          kexit(1);
        }
        
        img3 = createimage(clui_info->vs_int*img1->col_size, clui_info->hs_int*img1->row_size, img1->data_storage_type, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);

       if (img3 == NULL) {
          (void) kfprintf(kstderr, "\n\nScale Factor probably too large\n");
          kexit(1);
       }

       if (img1->data_storage_type == VFF_TYP_BIT) {
          img11 = format(img1, &border);
/*          if (clui_info->m_flag)
          img22 = format(img2, &border);
*/
          img33 = format(img3, &border);
       }
       else {
            img11 = img1;
/*
            img22 = img2;
*/
            img33 = img3;
       }
/* -main_before_lib_call_end */

/* -main_library_call */
if (!lvdilexpan(img11, &b, img22, clui_info->vs_int, clui_info->hs_int, clui_info->voff_int, clui_info->hoff_int, clui_info->str_flag, 0/*clui_info->m_flag*/, border, img33)) {
           (void) kfprintf(kstderr, "lvdilexpan failed\n");
           kexit(1);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
if (img1->data_storage_type == VFF_TYP_BIT)
           (void) unformat(img33, img3);
        copymapdata(img1, &img3, 1);
        (void) writeimage(clui_info->o_file, img3);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vdilexpan_usage_additions
| 
|       Purpose: Prints usage additions in vdilexpan_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vdilexpan
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vdilexpan_usage_additions(void)
{
	kfprintf(kstderr, "\tExpansion by Dilation\n");

/* -usage_additions */
kfprintf(kstderr, "\tOptional third input is a masking image.\n");
        kfprintf(kstderr, "\t\n");
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vdilexpan_free_args
| 
|       Purpose: Frees CLUI struct allocated in vdilexpan_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vdilexpan
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vdilexpan_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->str_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
