 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vdiskstr
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vdiskstr.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Generates a disk structuring element
|
|       Purpose: main program for vdiskstr
|
|         Input:
|		char *clui_info->o_file; {output resulting struc file definition}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->d_int; {diameter of the disk to be created}
|		int clui_info->d_flag; {TRUE if -d specified}
|
|		int clui_info->dim_logic; {2D/3D dimension}
|		int clui_info->dim_flag; {TRUE if -dim specified}
|
|		int clui_info->p_logic; {disk type}
|		int clui_info->p_flag; {TRUE if -p specified}
|
|		int clui_info->off_int; {value to be added to nonplanar elements}
|		int clui_info->off_flag; {TRUE if -off specified}
|
|		char *clui_info->t_toggle; {lets user choose which kind of disk is to be created}
|		int   clui_info->t_flag; {TRUE if -t specified}
|
|		char *clui_info->trigger_file; {trigger input}
|		int   clui_info->trigger_flag; {TRUE if -trigger specified}
|
|        Output:
|       Returns:
|
|    Written By: Ulisses Braga
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
int        type,offset,dim;
    char       *met;
    mat_str    ker;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vdiskstr");
	kexit_handler(vdiskstr_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vdiskstr", KGEN_KROUTINE,  &clui_uis_spec,
		   vdiskstr_usage_additions, vdiskstr_get_args,
 		   vdiskstr_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    if (clui_info->p_flag)
      type = clui_info->p_logic; /* Desired type */
    else
      type = 0;               /* Planar, default */  

    if (clui_info->off_flag && type)
      offset = clui_info->off_int;
    else
      offset = 0;

    if (clui_info->dim_flag)
      dim = clui_info->dim_logic;
    else
      dim = 0;               /* 2-D, default */  
 
    if (clui_info->t_flag)
      met = clui_info->t_toggle; /* Desired metric */
    else
      met = "euclidean";   /* Euclidean, default */
/* -main_before_lib_call_end */

/* -main_library_call */
if(! lvgdisk(&ker,clui_info->d_int,type,offset,dim,met))
    {
        (void) kfprintf(kstderr, "vgdisk: lvgdisk Failed\n");
        kexit(1);
    }
/* -main_library_call_end */

/* -main_after_lib_call */
write_str(clui_info->o_file,&ker);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vdiskstr_usage_additions
| 
|       Purpose: Prints usage additions in vdiskstr_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vdiskstr
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vdiskstr_usage_additions(void)
{
	kfprintf(kstderr, "\tGenerates a disk structuring element\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vdiskstr_free_args
| 
|       Purpose: Frees CLUI struct allocated in vdiskstr_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vdiskstr
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vdiskstr_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->t_toggle);
		kfree_and_NULL(clui_info->trigger_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
