 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vdist
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vdist.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Distance Transform of a binary image
|
|       Purpose: main program for vdist
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		int clui_info->cv_toggle; {dist. function}
|		int clui_info->cv_flag; {TRUE if -cv specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: Adriano Nagelschmidt Rodrigues
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage *img1, *img2, *images[1];
xvimage *img11, *img22;
int num_images, up_type;
unsigned long border;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vdist");
	kexit_handler(vdist_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vdist", KGEN_KROUTINE,  &clui_uis_spec,
		   vdist_usage_additions, vdist_get_args,
 		   vdist_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        num_images = 0;         /* initialize counter */

        img1 = readimage(clui_info->i_file);
        if (img1 == NULL)
            kexit(1);                    
        images[num_images++] = img1;

        img2 = createimage(img1->col_size, img1->row_size, VFF_TYP_2_BYTE, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);

       if (img2 == NULL) {
         (void) kfprintf(kstderr, "createimage Failed\n");
         kexit(1);
       }
/* -main_before_lib_call_end */

/* -main_library_call */
if (!lvdist(img1, clui_info->cv_toggle, img2)) {
          (void) kfprintf(kstderr, "lvdist Failed\n");
          kexit(1);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
(void) writeimage(clui_info->o_file, img2);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vdist_usage_additions
| 
|       Purpose: Prints usage additions in vdist_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vdist
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vdist_usage_additions(void)
{
	kfprintf(kstderr, "\tDistance Transform of a binary image\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vdist_free_args
| 
|       Purpose: Frees CLUI struct allocated in vdist_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vdist
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vdist_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
