 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vedgeoff
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vedgeoff.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Remove Objects Touching the Frame of the Image Window
|
|       Purpose: main program for vedgeoff
|
|         Input:
|		char *clui_info->i_file; {Input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->str_file; {Structuring Element}
|		int   clui_info->str_flag; {TRUE if -str specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: Junior Barrera
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage *img1, *img2, *images[1];
xvimage *img11, *img22;
int num_images;
unsigned long border;
mat_str b;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vedgeoff");
	kexit_handler(vedgeoff_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vedgeoff", KGEN_KROUTINE,  &clui_uis_spec,
		   vedgeoff_usage_additions, vedgeoff_get_args,
 		   vedgeoff_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
      num_images = 0;         /* initialize counter */

      img1 = readimage(clui_info->i_file);
      if (img1 == NULL)
        kexit(1);                    
      images[num_images++] = img1;

      read_str(clui_info->str_file, &b);
 
      img2 = createimage(img1->col_size, img1->row_size, img1->data_storage_type, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);  

      if (img2 == NULL) {
         kfprintf(kstderr, "lvedgeoff: createimage Failed\n");
         kexit(1);
      }    
   
      if (img1->data_storage_type == VFF_TYP_BIT) {
        img11 = format(img1, &border);
        img22 = format(img2, &border);
      }
      else {
        img11 = img1;
        img22 = img2;
      }
/* -main_before_lib_call_end */

/* -main_library_call */
if (!lvedgeoff(img11, &b, border, img22)) {
        (void) kfprintf(kstderr, "lvedgeoff Failed\n");
        kexit(1);
      }
/* -main_library_call_end */

/* -main_after_lib_call */
if (img1->data_storage_type == VFF_TYP_BIT)
        (void) unformat(img22, img2);
     (void) copyheader(img1, img2);
     copymapdata(img1, &img2, 1);
     (void) writeimage(clui_info->o_file, img2);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vedgeoff_usage_additions
| 
|       Purpose: Prints usage additions in vedgeoff_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vedgeoff
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vedgeoff_usage_additions(void)
{
	kfprintf(kstderr, "\tRemove Objects Touching the Frame of the Image Window\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vedgeoff_free_args
| 
|       Purpose: Frees CLUI struct allocated in vedgeoff_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vedgeoff
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vedgeoff_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->str_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
