 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vequal
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vequal.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Informs if two images are equal
|
|       Purpose: main program for vequal
|
|         Input:
|		char *clui_info->i1_file; {input file}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->v_string; {variable}
|		int   clui_info->v_flag; {TRUE if -v specified}
|
|		char *clui_info->o_file; {The first input file}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		Required M.E. group:
|		    char *clui_info->i2_file; {input file}
|		    int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		    int clui_info->c_int; {constant value}
|		    int clui_info->c_flag; {TRUE if -c specified}
|
|        Output:
|       Returns:
|
|    Written By: Junior Barrera
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage *img1, *img2, *images[2];
xvimage *img11, *img22;
int value;
int num_images, up_type;
unsigned long border;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vequal");
	kexit_handler(vequal_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vequal", KGEN_KROUTINE,  &clui_uis_spec,
		   vequal_usage_additions, vequal_get_args,
 		   vequal_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        num_images = 0;       /* initialize counter */

        img1 = readimage(clui_info->i1_file);
        if (img1 == NULL) {
          kfprintf(kstderr,"vequal: readimage failed for input image 1.\n");
          kexit(1);
        }
   
       images[num_images++] = img1;
        
       if (clui_info->i2_flag) {
         img2 = readimage(clui_info->i2_file);
         if (img2 == NULL) {
           kfprintf(kstderr,"vequal: readimage failed for input image 2.\n");
           kexit(1);
         }
         images[num_images++] = img2;
         CHECKINPUT(program, img1, img2);   /* verifies input images */
       }

        up_type = -1;    /* call lvcast1 with auto upconvert */

                         /*  Call lvcast1 to upconvert data types */
        if (!lvcast1(images, num_images, up_type)) {
          (void) kfprintf(kstderr, "ERROR: images could not be cast to the same type\n");
          kexit(1);
        }

       if (img1->data_storage_type == VFF_TYP_BIT) {
         img11 = format(img1, &border);
         if (clui_info->i2_flag)
           img22 = format(img2, &border);
       }
       else {
         img11 = img1;
         img22 = img2;
       }
/* -main_before_lib_call_end */

/* -main_library_call */
if (!lvequal(img11, img22, clui_info->i2_flag, clui_info->c_int, &value)) {
          (void) kfprintf(kstderr, "vequal: lvequal Failed\n");
          kexit(1);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
if (img1->data_storage_type == VFF_TYP_BIT)
     (void) unformat(img11, img1);
   (void) writeimage(clui_info->o_file, img1);
   kprintf("%s = %d\n", clui_info->v_string, value);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vequal_usage_additions
| 
|       Purpose: Prints usage additions in vequal_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vequal
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vequal_usage_additions(void)
{
	kfprintf(kstderr, "\tInforms if two images are equal\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vequal_free_args
| 
|       Purpose: Frees CLUI struct allocated in vequal_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vequal
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vequal_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->v_string);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
