 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vless
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vless.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Informs if one image is less than the other
|
|       Purpose: main program for vless
|
|         Input:
|		Required M.E. group:
|		    char *clui_info->i1_file; {Upper level}
|		    int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		    int clui_info->u_int; {upper level}
|		    int clui_info->u_flag; {TRUE if -u specified}
|
|		char *clui_info->v_string; {variable}
|		int   clui_info->v_flag; {TRUE if -v specified}
|
|		char *clui_info->o_file; {The first input image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		Required M.E. group:
|		    char *clui_info->i2_file; {Lower level}
|		    int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		    int clui_info->l_int; {lower level}
|		    int clui_info->l_flag; {TRUE if -l specified}
|
|        Output:
|       Returns:
|
|    Written By: Adriano Nagelschmidt Rodrigues
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage *img1, *img2, *images[2];
xvimage *img11, *img22;
int num_images, up_type;
int value;
unsigned long border;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vless");
	kexit_handler(vless_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vless", KGEN_KROUTINE,  &clui_uis_spec,
		   vless_usage_additions, vless_get_args,
 		   vless_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        num_images = 0;         /* initialize counter */

	if (clui_info->i1_flag) {
          img1 = readimage(clui_info->i1_file);                 
          images[num_images++] = img1;
          if (img1 == NULL)
            kexit(1);
        }
 
	if (clui_info->i2_flag) {
          img2 = readimage(clui_info->i2_file);                 
          images[num_images++] = img2;
          if (img2 == NULL)
            kexit(1);
        }
        
        if ((clui_info->i1_flag) && (clui_info->i2_flag)) {
          CHECKINPUT(program, img1, img2);
        }

        up_type = -1;    /* call lvcast1 with auto upconvert */

                         /*  Call lvcast1 to upconvert data types */
        if (!lvcast1(images, num_images, up_type)) {
          (void) kfprintf(kstderr, "ERROR: images could not be cast to the same type\n");
          kexit(1);
        }

        if (img1->data_storage_type == VFF_TYP_BIT) {
          if (clui_info->i1_flag)
            img11 = format(img1, &border);
          if (clui_info->i2_flag)
            img22 = format(img2, &border);
        }
        else {
          img11 = img1;
          img22 = img2;
        }
/* -main_before_lib_call_end */

/* -main_library_call */
if (!lvless(img11, clui_info->i1_flag, clui_info->u_int, img22, clui_info->i2_flag, clui_info->l_int, &value)) {
          (void) kfprintf(kstderr, "lvless Failed\n");
          kexit(1);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
if (img1->data_storage_type == VFF_TYP_BIT)
          (void) unformat(img11, img1);
        (void) writeimage(clui_info->o_file, img1);
         kprintf("%s = %d\n", clui_info->v_string, value);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vless_usage_additions
| 
|       Purpose: Prints usage additions in vless_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vless
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vless_usage_additions(void)
{
	kfprintf(kstderr, "\tInforms if one image is less than the other\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vless_free_args
| 
|       Purpose: Frees CLUI struct allocated in vless_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vless
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vless_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->v_string);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
