 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vlocmax
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vlocmax.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Regional Max
|
|       Purpose: main program for vlocmax
|
|         Input:
|		char *clui_info->i_file; {Input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->str_file; {Structuring Element}
|		int   clui_info->str_flag; {TRUE if -str specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: Junior Barrera
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage *img1, *img2, *img3, *images[2];
int num_images;
mat_str b;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vlocmax");
	kexit_handler(vlocmax_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vlocmax", KGEN_KROUTINE,  &clui_uis_spec,
		   vlocmax_usage_additions, vlocmax_get_args,
 		   vlocmax_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        num_images = 0;         /* initialize counter */

        img1 = readimage(clui_info->i_file);
        if (img1 == NULL)
          kexit(1);                    
        images[num_images++] = img1;

        read_str(clui_info->str_file, &b);
/*
        if (clui_info->m_flag) {
          img2 = readimage(clui_info->m_file);
          images[num_images++] = img2;
          if (img2 == NULL)
            kexit(1);
          CHECKINPUT(program, img1, img2); 
        }
*/
        if (img1->data_storage_type == VFF_TYP_BIT) {
          kfprintf(kstderr, "vlocmax: BIT not defined");
          kexit(1);
        }

        img3 = createimage(img1->col_size, img1->row_size, img1->data_storage_type, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);  

        if (img3 == NULL) {
          kfprintf(kstderr, "vlocmax: createimage Failed\n");
          kexit(1);
        }
/* -main_before_lib_call_end */

/* -main_library_call */
if (!lvlocmax(img1, &b, img2, 0/*clui_info->m_flag*/, img3)) {
          (void) kfprintf(kstderr, "lvlocmax Failed\n");
          kexit(1);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
(void) copyheader(img1, img3);
        copymapdata(img1, &img3, 1);
        (void) writeimage(clui_info->o_file, img3);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vlocmax_usage_additions
| 
|       Purpose: Prints usage additions in vlocmax_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vlocmax
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vlocmax_usage_additions(void)
{
	kfprintf(kstderr, "\tRegional Max\n");

/* -usage_additions */
kfprintf(kstderr, "\tOptional fourth input is a masking image.\n");
        kfprintf(kstderr, "\t\n");
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vlocmax_free_args
| 
|       Purpose: Frees CLUI struct allocated in vlocmax_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vlocmax
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vlocmax_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->str_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
