 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vnconddil
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vnconddil.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - N times (with respect to the composition) the conditional dilation
|
|       Purpose: main program for vnconddil
|
|         Input:
|		char *clui_info->i1_file; {first input image}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {second input image}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->str_file; {Structuring Element}
|		int   clui_info->str_flag; {TRUE if -str specified}
|
|		int clui_info->n_int; {number of cond. dilations}
|		int clui_info->n_flag; {TRUE if -n specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: Adriano Nagelschmidt Rodrigues
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage *img1, *img2, *img3, *img4, *images[3];
xvimage *img11, *img22, *img33, *img44;
int num_images, up_type;
unsigned long border;
mat_str b;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vnconddil");
	kexit_handler(vnconddil_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vnconddil", KGEN_KROUTINE,  &clui_uis_spec,
		   vnconddil_usage_additions, vnconddil_get_args,
 		   vnconddil_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        num_images = 0;         /* initialize counter */

        img1 = readimage(clui_info->i1_file);
        if (img1 == NULL)
          kexit(1);                    
        images[num_images++] = img1;     
        
        img2 = readimage(clui_info->i2_file);
        if (img2 == NULL)
          kexit(1);  
        images[num_images++] = img2;

        read_str(clui_info->str_file, &b);   
/*
        if (clui_info->m_flag) {
          img3 = readimage(clui_info->m_file);
          images[num_images++] = img3;
        }

        if ((clui_info->m_flag) && (img2 == NULL)) kexit(1);         

        if (clui_info->m_flag)  { 
          CHECKINPUT(program, img1, img3); 
        }
*/                
        CHECKINPUT(program, img1, img2);      /* Verify input images */
         
        up_type = -1;    /* call lvcast1 with auto upconvert */

                         /*  Call lvcast1 to upconvert data types */
        if (!lvcast1(images, num_images, up_type)) {
          (void) kfprintf(kstderr, "ERROR: images could not be cast to the same type\n");
          kexit(1);
        }

        img4 = createimage(img1->col_size, img1->row_size, img1->data_storage_type, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);

       if (img4 == NULL) {
         (void) kfprintf(kstderr, "createimage Failed\n");
         kexit(1);
       }

       if (img1->data_storage_type == VFF_TYP_BIT) {
         img11 = format(img1, &border);
         img22 = format(img2, &border);
/*
         if (clui_info->m_flag)
           img33 = format(img3, &border);
*/
         img44 = format(img4, &border);
       }
       else {
         img11 = img1;
         img22 = img2;
         img33 = img3;
         img44 = img4;
       }
/* -main_before_lib_call_end */

/* -main_library_call */
if (!lvncdil(img11, img22, &b, clui_info->n_int, img33, 0/*clui_info->m_flag*/, border, img44)) {
          (void) kfprintf(kstderr, "lvncdil Failed\n");
          kexit(1);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
if (img1->data_storage_type == VFF_TYP_BIT)
          (void) unformat(img44, img4);
        (void) copyheader(img1, img4);
        copymapdata(img1, &img4, 1);
        (void) writeimage(clui_info->o_file, img4);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vnconddil_usage_additions
| 
|       Purpose: Prints usage additions in vnconddil_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vnconddil
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vnconddil_usage_additions(void)
{
	kfprintf(kstderr, "\tN times (with respect to the composition) the conditional\n");
	kfprintf(kstderr, "\tdilation\n");

/* -usage_additions */
kfprintf(kstderr, "\tOptional fourth input is a masking image.\n");
        kfprintf(kstderr, "\t\n");
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vnconddil_free_args
| 
|       Purpose: Frees CLUI struct allocated in vnconddil_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vnconddil
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vnconddil_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->str_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
