 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vopenseg
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vopenseg.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Opening by a straight line segment
|
|       Purpose: main program for vopenseg
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		int clui_info->alfa_int; {angle of the line segment from the horizontal}
|		int clui_info->alfa_flag; {TRUE if -alfa specified}
|
|		int clui_info->length_int; {length of the line segment}
|		int clui_info->length_flag; {TRUE if -length specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: Adriano Nagelschmidt Rodrigues,
Using Guy Pilchen's veroseg as a template.
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage       *img_in,
                        *img_out;
   xvimage       *img11, 
                        *img22;
   int                  alfa,
                        length;
   unsigned long        border;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vopenseg");
	kexit_handler(vopenseg_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vopenseg", KGEN_KROUTINE,  &clui_uis_spec,
		   vopenseg_usage_additions, vopenseg_get_args,
 		   vopenseg_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        img_in = readimage(clui_info->i_file);
        if (img_in == NULL)
            kexit(1);

        alfa = clui_info->alfa_int;
        length = clui_info->length_int;

        img_out = createimage(  img_in->col_size, 
                                img_in->row_size, 
                                img_in->data_storage_type, 
                                img_in->num_of_images,
                                img_in->num_data_bands, 
                                img_in->comment, 
                                img_in->map_row_size, 
                                img_in->map_col_size, 
                                img_in->map_scheme,
                                img_in->map_storage_type, 
                                img_in->location_type, 
                                img_in->location_dim);

       if (img_out == NULL) {
         (void) kfprintf(kstderr, "vopenseg: createimage Failed\n");
          kexit(1);
       }

       if (img_in->data_storage_type == VFF_TYP_BIT) {
         img11 = format(img_in, &border);
         img22 = format(img_out, &border);
       }
       else {
         img11 = img_in;
         img22 = img_out;
       }
/* -main_before_lib_call_end */

/* -main_library_call */
if (!lvopenseg(img11, alfa, length, border, img22)) {
          (void) kfprintf(kstderr, "lvopenseg Failed\n");
          kexit(1);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
if (img_in->data_storage_type == VFF_TYP_BIT)
          (void) unformat(img22, img_out);
       (void) copyheader(img_in, img_out);
       copymapdata(img_in, &img_out, 1);
       (void) writeimage(clui_info->o_file, img_out);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vopenseg_usage_additions
| 
|       Purpose: Prints usage additions in vopenseg_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vopenseg
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vopenseg_usage_additions(void)
{
	kfprintf(kstderr, "\tOpening by a straight line segment\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vopenseg_free_args
| 
|       Purpose: Frees CLUI struct allocated in vopenseg_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vopenseg
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vopenseg_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
