 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vrotstr
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vrotstr.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Rotates a structuring element
|
|       Purpose: main program for vrotstr
|
|         Input:
|		char *clui_info->str_file; {input Morphological Kernel}
|		int   clui_info->str_flag; {TRUE if -str specified}
|
|		int clui_info->rot_int; {the step of rotation value}
|		int clui_info->rot_flag; {TRUE if -rot specified}
|
|		int clui_info->d_logic; {rotation direction}
|		int clui_info->d_flag; {TRUE if -d specified}
|
|		char *clui_info->o_file; {output morphological kernel}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: Junior Barrera & Adriano Nagelschmidt Rodrigues
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
mat_str b_in, b_out;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vrotstr");
	kexit_handler(vrotstr_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vrotstr", KGEN_KROUTINE,  &clui_uis_spec,
		   vrotstr_usage_additions, vrotstr_get_args,
 		   vrotstr_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
   if (!read_str(clui_info->str_file, &b_in) ) {
      (void) kfprintf(kstderr, "vrotstr: read_str Failed\n");
      kexit(1);
   }
/* -main_before_lib_call_end */

/* -main_library_call */
if(! lvrotstr(&b_in, clui_info->rot_int, clui_info->d_logic, &b_out) )   {
        (void) kfprintf(kstderr, "vrotstr: lvrotstr Failed\n");
        kexit(1);
    }
/* -main_library_call_end */

/* -main_after_lib_call */
if (! write_str(clui_info->o_file, &b_out) ) {
   (void) kfprintf(kstderr, "vrotstr: write_str Failed\n");
   kexit(1);
}
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vrotstr_usage_additions
| 
|       Purpose: Prints usage additions in vrotstr_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vrotstr
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vrotstr_usage_additions(void)
{
	kfprintf(kstderr, "\tRotates a structuring element\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vrotstr_free_args
| 
|       Purpose: Frees CLUI struct allocated in vrotstr_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vrotstr
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vrotstr_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->str_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
