 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vsubm
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vsubm.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Subtraction of Two Images
|
|       Purpose: main program for vsubm
|
|         Input:
|		char *clui_info->i1_file; {first input image}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		Required M.E. group:
|		    char *clui_info->i2_file; {second input image}
|		    int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		    int clui_info->c_int; {sub constant}
|		    int clui_info->c_flag; {TRUE if -c specified}
|
|        Output:
|       Returns:
|
|    Written By: Junior Barrera
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage *img1, *img2, *img3, *images[3];
    xvimage *img11, *img22, *img33;
    int  num_images, up_type;
    unsigned long border;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vsubm");
	kexit_handler(vsubm_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vsubm", KGEN_KROUTINE,  &clui_uis_spec,
		   vsubm_usage_additions, vsubm_get_args,
 		   vsubm_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    num_images = 0;           /* initialize image counter */

    img1 = readimage(clui_info->i1_file);
    if (img1 == NULL)
        kexit(1);                   /* Quit if bad image */
    images[num_images++] = img1;

    if (clui_info->i2_flag) {
      img2 = readimage(clui_info->i2_file);
      if (img2 == NULL)
          kexit(1);                   /* Quit if bad image */
      images[num_images++] = img2;
      CHECKINPUT(program, img1, img2);
    }
/* remove mask input, ral 12nov94
    if (clui_info->m_flag) {
       img3 = readimage(clui_info->m_file);
       images[num_images++] = img3;
    }
    if (clui_info->m_flag && (img3 == NULL)) kexit(1);
    if (clui_info->m_flag) {
      MATCHINPUT(program,img1,img3);
    }
*/
    up_type = -1;        /* call lvcast1 with auto upconvert set */

    if (! lvcast1(images, num_images, up_type)) {
      (void) kfprintf(kstderr, "ERROR: images could not be cast to the same type\n");
      kexit(1);
    }

    if (img1->data_storage_type == VFF_TYP_BIT) {
      img11 = format(img1, &border);
      if (clui_info->i2_flag)
        img22 = format(img2, &border);
/*
      if (clui_info->m_flag)
        img33 = format(img3, &border);
*/
    }
    else {
      img11 = img1;
      img22 = img2;
      img33 = img3;
    }
/* -main_before_lib_call_end */

/* -main_library_call */
if (! lvsubm(img11, img22, clui_info->i2_flag, clui_info->c_int, img33, 0 /* clui_info->m_flag */, img11)) {
      (void) kfprintf(kstderr, "lvsubm Failed\n");
      kexit(1);
    }
/* -main_library_call_end */

/* -main_after_lib_call */
if (img1->data_storage_type == VFF_TYP_BIT)
      (void) unformat(img11, img1);
    writeimage(clui_info->o_file, img1);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vsubm_usage_additions
| 
|       Purpose: Prints usage additions in vsubm_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vsubm
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vsubm_usage_additions(void)
{
	kfprintf(kstderr, "\tSubtraction of Two Images\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vsubm_free_args
| 
|       Purpose: Frees CLUI struct allocated in vsubm_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vsubm
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vsubm_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
