 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vthreshm
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vthreshm.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Adaptive Thresholding of an Image
|
|       Purpose: main program for vthreshm
|
|         Input:
|		char *clui_info->i1_file; {Image to be compared}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		Required M.E. group:
|		    char *clui_info->i2_file; {Lower threshold level}
|		    int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		    int clui_info->l_int; {lower thresh. level}
|		    int clui_info->l_flag; {TRUE if -l specified}
|
|		Required M.E. group:
|		    char *clui_info->i3_file; {Upper thres. level}
|		    int   clui_info->i3_flag; {TRUE if -i3 specified}
|
|		    int clui_info->u_int; {upper thresh. level}
|		    int clui_info->u_flag; {TRUE if -u specified}
|
|        Output:
|       Returns:
|
|    Written By: Roberto Hirata Jr. & Adriano Nagelschmidt Rodrigues
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage *img1, *img2, *img3, *img4, *imgout, *images[4];
int num_images;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vthreshm");
	kexit_handler(vthreshm_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vthreshm", KGEN_KROUTINE,  &clui_uis_spec,
		   vthreshm_usage_additions, vthreshm_get_args,
 		   vthreshm_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        num_images = 0;         /* initialize counter */

        img1 = readimage(clui_info->i1_file);
        if (img1 == NULL)
            kexit(1);                    
        images[num_images++] = img1;

	if (clui_info->i2_flag) {
          img2 = readimage(clui_info->i2_file);                 
          images[num_images++] = img2;
          CHECKINPUT(program, img1, img2);
        }
        if ((clui_info->i2_flag) && (img2 == NULL))
          kexit(1);

	if (clui_info->i3_flag) {
          img3 = readimage(clui_info->i3_file);                 
          images[num_images++] = img3;
          CHECKINPUT(program, img1, img3);
        }
        if ((clui_info->i3_flag) && (img3 == NULL))
          kexit(1);

/* remove mask input (lotufo, 12nov94)
        if (clui_info->m_flag) {
          img4 = readimage(clui_info->m_file);
          images[num_images++] = img4;
          CHECKINPUT(program, img1, img4);  
        }
        if ((clui_info->m_flag) && (img4 == NULL)) kexit(1);
*/
	if (img1->data_storage_type == VFF_TYP_2_BYTE) {
          imgout = createimage(img1->col_size, img1->row_size, VFF_TYP_1_BYTE, img1->num_of_images, img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme, img1->map_storage_type, img1->location_type, img1->location_dim);
          if (imgout == NULL) {
            (void) kfprintf(kstderr, "createimage Failed\n");
            kexit(1);
          }
        }
        else
          imgout = img1;
/* -main_before_lib_call_end */

/* -main_library_call */
if (!lvthreshad(img1, img2, clui_info->i2_flag, clui_info->l_int, img3, clui_info->i3_flag, clui_info->u_int, img4, 0 /*clui_info->m_flag */, imgout)) {
          (void) kfprintf(kstderr, "lvthreshad Failed\n");
          kexit(1);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
(void) writeimage(clui_info->o_file, imgout);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vthreshm_usage_additions
| 
|       Purpose: Prints usage additions in vthreshm_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vthreshm
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vthreshm_usage_additions(void)
{
	kfprintf(kstderr, "\tAdaptive Thresholding of an Image\n");

/* -usage_additions */
kfprintf(kstderr, "\tOptional fourth input is a masking image.\n");
        kfprintf(kstderr, "\t\n");
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vthreshm_free_args
| 
|       Purpose: Frees CLUI struct allocated in vthreshm_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vthreshm
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vthreshm_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->i3_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
