 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vtoggle
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vtoggle.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Toggle of Two Images
|
|       Purpose: main program for vtoggle
|
|         Input:
|		char *clui_info->i1_file; {first input image}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {second input image}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->i3_file; {third input image}
|		int   clui_info->i3_flag; {TRUE if -i3 specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: Junior Barrera
|          Date: May 12, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage *img1, *img2, *img3, *img4, *images[4];
int num_images, up_type;
/* -main_variable_list_end */

	khoros_init(argc, argv, "MMACH", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$MMACH/objects/kroutine/vtoggle");
	kexit_handler(vtoggle_free_args, NULL);

/* -main_get_args_call */
	kclui_init("MMACH", "vtoggle", KGEN_KROUTINE,  &clui_uis_spec,
		   vtoggle_usage_additions, vtoggle_get_args,
 		   vtoggle_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        num_images = 0;         /* initialize counter */

        img1 = readimage(clui_info->i1_file);
        if (img1 == NULL)
          kexit(1);                    
        images[num_images++] = img1;

        img2 = readimage(clui_info->i2_file);
        if (img2 == NULL)
          kexit(1);                      
        images[num_images++] = img2;

        img3 = readimage(clui_info->i3_file);
        if (img3 == NULL)
          kexit(1);                      
        images[num_images++] = img3;

        CHECKINPUT(program, img1, img2);      /* Verify input images */
        MATCHINPUT(program, img1, img3);      /* Verify input images */

/* remove of mask input, ral 12nov94
        if (clui_info->m_flag) {
          img4 = readimage(clui_info->m_file);
          images[num_images++] = img4;
        }
        if ((clui_info->m_flag) && (img4 == NULL)) kexit(1);         
        if(clui_info->m_flag)  { 
            MATCHINPUT(program, img1, img4);
        }
*/
        up_type = -1;    /* call lvcast1 with auto upconvert */

                         /*  Call lvcast1 to upconvert data types */
        if (! lvcast1(images, num_images, up_type)) {
          (void) kfprintf(kstderr, "ERROR: images could not be cast to the same type\n");
          kexit(1);
        }
/* -main_before_lib_call_end */

/* -main_library_call */
if (! lvtoggle(img1, img2, img3, img4, 0 /*clui_info->m_flag*/, img1)) {
          (void) kfprintf(kstderr, "lvtoggle Failed\n");
          kexit(1);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
(void) writeimage(clui_info->o_file, img1);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vtoggle_usage_additions
| 
|       Purpose: Prints usage additions in vtoggle_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vtoggle
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vtoggle_usage_additions(void)
{
	kfprintf(kstderr, "\tToggle of Two Images\n");

/* -usage_additions */
kfprintf(kstderr, "\tOptional fourth input is a masking image.\n");
        kfprintf(kstderr, "\t\n");
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vtoggle_free_args
| 
|       Purpose: Frees CLUI struct allocated in vtoggle_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vtoggle
|          Date: May 12, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vtoggle_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->i3_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
