 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vchghomot
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvchghomot
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvchghomot - Change the homotopy of an image.
* 
*       Purpose: .I vhomot
will change the homotopy of an image, based on a reference image.
img1 is the input images; img2 is the reference image; b is a structuring element; img3 is the gating image; mflg is a flag. mflg should be set to 1 if the gating image is to be used.
A non-zero mask pixel in img2 enables the output pixel to contain 
the result. A zero mask pixel in img2 just transfers the value 
of the corresponding pixel in img1 to the output pixel.
.LP
The input images and the optional mask image must be of the same size, data type
and have the same number of data bands. The image img2 is also used to
return the result.
*         Input: .IP "img1" 15  
input image
.Ip "img2" 15
reference image
.IP "b" 15
structuring element 
.IP "img3" 15
gating mask image
.IP "mflg" 15
a flag set (equal 1) if gating image mask available
.IP
Note that the input image and the optional mask image must be of the same data type and size.
*        Output: .IP "img4" 15
holds the result of the change of homotopy operation.  The output data
type is the same as the input data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvhomot
can be defined for all data types supported by Khoros (except bit), but at the moment it has been implemented just for the unsigned char types.
*    Written By: Junior Barrera
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: Adriano Nagelschmidt Rodrigues: port to new toolbox version.
****************************************************************/
/* -library_def */
int
lvhomot(xvimage *img1, xvimage *img2, mat_str *b, xvimage *img3, int mflg, xvimage *img4)
/* -library_def_end */

/* -library_code */
{
  xvimage *img5;

  img5 = createimage(img1->col_size, img1->row_size, img1->data_storage_type, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);

  if (img5 == NULL) {
    kfprintf(kstderr, "lvhomot: createimage Failed\n");
    return(0);
  } 

  if (!copyimagedata1(img1, img5))
    return(0);

  if (!lvsup(img5, NULL, 0, 1, img3, mflg, img5))
    return(0);
  if (!lvinf(img5, NULL, 0, 254, img3, mflg, img5))
    return(0);
  if (!lvinv(img2, img3, mflg, img4))
    return(0);
  if (!lvinf(img5, img4, 1, 0, img3, mflg, img5))
    return(0);
  if (!lvclosrec(img5, img4, b, img3, mflg, 0, img4))
    return(0);
      
  freeimage(img5);     
  return(1);  
}
/* -library_code_end */
