 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vclohole
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvclohole
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvclohole - Closes the holes of the objects without changing their contours.
* 
*       Purpose: .I vclohole
will close the holes of the objects without changing their contours.
img1 is the input images; b is a structuring element;
img2 is the output image.
*         Input: .IP "img1" 15  
input image
.IP "b" 15
structuring element
.IP "border" 15
mask to the last pixel of a row (used in formatted BIT images)
*        Output: .IP "img2" 15
holds the result of the close holes operation. The output data
type is the same as the input data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvclohole
can be defined for all data types supported by Khoros, but at the moment it has been implemented just for the bit and unsigned char types.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Junior Barrera
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: Adriano Nagelschmidt Rodrigues: port to new toolbox version.
****************************************************************/
/* -library_def */
int
lvclohole(xvimage *img1, mat_str *b, unsigned long border, xvimage *img2)
/* -library_def_end */

/* -library_code */
{
  xvimage *img3;
  int value;

  img3 = createimage(img1->col_size, img1->row_size, img1->data_storage_type, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);  

  if (img3 == NULL) {
    kfprintf(kstderr, "lvclohole: createimage Failed\n");
    return(0);
  }   
 
  if (!copyimagedata1(img1, img3))     return(0);
  if (!lvwind(img2, border))           return(0);
  if (!lvinv(img3, NULL, 0, img3))     return(0);
  if (!lvopenrec(img3, img2, b, NULL, 0, border, img2))    return(0); 
  if (!lvinv(img2, NULL, 0, img2))    return(0);

  return(1);
}
/* -library_code_end */
