 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vclorec
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvclorec
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvclorec - Do the closing by reconstruction of an image by a set structuring element, using a conditional image, with an optional operation gating mask.
* 
*       Purpose: .I lvclosrec
will do the closing by reconstruction of an image by a set structuring element.
img1 and b are, respectively, the input image and the structuring element;
img2 is the conditional image, img3 is the gating image, and mflg should be set to 1 if the gating image is to be used. img4 is the output image. Note that img2 can be used for both the input and the resulting output xvimage structure, in order to save space.
A non-zero mask pixel in img3 enables the output pixel to contain 
the closing by reconstruction. A zero mask pixel in img3 just transfers the value  of the corresponding pixel in img1 to the output pixel.
.LP
The input, output and mask images must be of the same size, data type, and have 
the same number of data bands.
*         Input: .IP "img1" 15  
first input image 
.IP "img2" 15  
second input image 
.IP "b" 15
set structuring element
.IP "img3" 15
gating mask image
.IP "mflg" 15
a flag set (equal 1) if gating image mask available
.IP "border" 15
mask to the last pixel of a row (used in formatted BIT images)
.IP
Note that the input and mask images must be of the same data type and size.
*        Output: .IP "img4" 15
holds the result of the closing by reconstruction operation.  The output data
type is the same as the input data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvclosrec
can be defined for all data types supported by Khoros, but at the moment it has been implemented just for the bit and unsigned char types.
The structuring elements are subsets of the 3x3 matrix and the origin is always at the center of this matrix.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Adriano Nagelschmidt Rodrigues
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int
lvclosrec(xvimage *img1, xvimage *img2, mat_str *b_mat, xvimage *img3, int mflg, unsigned long border, xvimage *img4)
/* -library_def_end */

/* -library_code */
{
   xvimage *img5;
   int value;

   img5 = createimage(img1->col_size, img1->row_size, img1->data_storage_type, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);

   if (img5 == NULL) {
     (void) kfprintf(kstderr, "lvclosrec: createimage Failed\n");
     return(0);
   }

   if (!copyimagedata1(img2, img5))
     return(0);

   value = 0;
   while (value == 0) {
     if (!lvcero(img5, img1, b_mat, img3, mflg, border, img4))
       return(0);
     if (!lvcero(img4, img1, b_mat, img3, mflg, border, img5))
       return(0);      
     if (!lvequal(img4, img5, 1, 0, &value))
       return(0);
   } 

   freeimage(img5);
   return(1);
}
/* -library_code_end */
