 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vdist
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvdist
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
#define MAXVAL 32767
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvdist - Do the distance function of a binary image.
* 
*       Purpose: .I lvdist
Will do the distance function of a binary image.
img1 is the input image (of the VIFF_TYP_1_BYTE data type), img2 is the output image (of the VFF_TYP_2_BYTE data type). cv is an integer parameter that indicates wich distance function is desired (1 for euclidean, 4 for 4-connectivity City Bl. or 8 for 8-connectivity City Bl).
*         Input: .IP "img1" 15  
input xvimage structure of the VIFF_TYP_1_BYTE data type.
.IP "cv" 15
chosen distance function. cv values can be 1 (for euclidean dist. func.), 4 (for 4-connect. CB dist. func) or 8 (for 8-connect. CB dist. func.).
.IP
Note that img1 should be a binary image of the BYTE1 data type.
*        Output: .IP "img2" 15
holds the distance operation. The output image should be of the VIFF_TYP_2_BYTE data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvdist
The input and output images must be of the same size. The input and output image data types should be, respectively, VFF_TYP_1_BYTE and VFF_TYP_2_BYTE.
*    Written By: Adriano Nagelschmidt Rodrigues
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int
lvdist(xvimage *img1, int cv, xvimage *img2)
/* -library_def_end */

/* -library_code */
{
  int nc, nr, k, r;
  int d[8], flag;
  register int i, j, l, C1x, C1y, C2x, C2y;
  item_str p_num;
  queue_str q;
  unsigned char *c1;
  short *s1, *Jx, *Jy;
  xvimage *img3, *img4;


  nc = img1->row_size;
  nr = img1->col_size;
  k = nc * nr;
  c1 = (unsigned char *)(img1->imagedata); 
  s1 = (short *)(img2->imagedata);

  switch (img1->data_storage_type) {
    case VFF_TYP_BIT :
      kfprintf(kstderr,"lvdist: BIT not implemented\n");
      return(0);
      break;

    case VFF_TYP_1_BYTE :
      switch (cv) {
        case 1 :                 /*** Euclidean distance function ***/

          img3 = createimage(img1->col_size, img1->row_size, VFF_TYP_2_BYTE, img1->num_of_images, img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme, img1->map_storage_type, img1->location_type, img1->location_dim);

          if (img3 == NULL) {
            kfprintf(kstderr, "lvdist: createimage Failed\n");
            return(0);
          }

          img4 = createimage(img1->col_size, img1->row_size, VFF_TYP_2_BYTE, img1->num_of_images, img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme, img1->map_storage_type, img1->location_type, img1->location_dim);

          if (img4 == NULL) {
            kfprintf(kstderr, "lvdist: createimage Failed\n");
            return(0);
          }

          Jx = (short *)(img3->imagedata);  /* vector images */
          Jy = (short *)(img4->imagedata);

          for (i = 0; i < k; i++)           /* initialize images */
            if (c1[i] == 255)
              Jy[i] = Jx[i] = MAXVAL;
            else
              Jy[i] = Jx[i] = 0;

          for (i = 0; i < nc; i++) {        /* initialize frames */
            Jy[i] = Jx[i] = MAXVAL;
            Jy[i + k - nc] = Jx[i + k - nc] = MAXVAL;
          }
          for (i = 0; i < k; i += nc) {
            Jy[i] = Jx[i] = MAXVAL;
            Jy[i + nc - 1] = Jx[i + nc - 1] = MAXVAL;
          }
         
          for (i = 0; i < nr; i++) {   /* 1st loop of Danielson's alg. */
            r = i * nc;
            if (i != 0)   
              for (j = 1; j < nc; j++) {   /* skip first line and column */
                l = r + j;
                C1x = Jx[l];
                C1y = Jy[l];
                C2x = Jx[l - nc];
                C2y = Jy[l - nc] - 1;
                if (ksqr(C1x) + ksqr(C1y) > ksqr(C2x) + ksqr(C2y)) {
                  C1x = C2x;
                  C1y = C2y;
                }
                C2x = Jx[l - 1] + 1;
                C2y = Jy[l - 1];
                if (ksqr(C1x) + ksqr(C1y) > ksqr(C2x) + ksqr(C2y)) {
                  Jx[l] = C2x;
                  Jy[l] = C2y;
                }
                else {
                  Jx[l] = C1x;
                  Jy[l] = C1y;
                }
              }
            for (j = nc - 2; j >= 0; j--) {   /* skip last column */
              l = r + j;
              C1x = Jx[l];
              C1y = Jy[l];
              C2x = Jx[l + 1] - 1;
              C2y = Jy[l + 1];
              if (ksqr(C1x) + ksqr(C1y) > ksqr(C2x) + ksqr(C2y)) {
                Jx[l] = C2x;
                Jy[l] = C2y;
              }
            }
          }

          for (i = nr - 1; i >= 0; i--) {   /* 2nd loop of Danielson's alg. */
            r = i * nc;
            if (i != nr - 1)   
              for (j = nc - 2; j >= 0; j--) {   /* skip last line and column */
                l = r + j;
                C1x = Jx[l];
                C1y = Jy[l];
                C2x = Jx[l + nc];
                C2y = Jy[l + nc] + 1;
                if (ksqr(C1x) + ksqr(C1y) > ksqr(C2x) + ksqr(C2y)) {
                  C1x = C2x;
                  C1y = C2y;
                }
                C2x = Jx[l + 1] - 1;
                C2y = Jy[l + 1];
                if (ksqr(C1x) + ksqr(C1y) > ksqr(C2x) + ksqr(C2y)) {
                  Jx[l] = C2x;
                  Jy[l] = C2y;
                }
                else {
                  Jx[l] = C1x;
                  Jy[l] = C1y;
                }
              }
            for (j = 1; j < nc; j++) {   /* skip first column */
              l = r + j;
              C1x = Jx[l];
              C1y = Jy[l];
              C2x = Jx[l - 1] + 1;
              C2y = Jy[l - 1];
              if (ksqr(C1x) + ksqr(C1y) > ksqr(C2x) + ksqr(C2y)) {
                Jx[l] = C2x;
                Jy[l] = C2y;
              }
            }
          }

          for (i = 0; i < k; i++) {   /* output image receives vectors' norm */
            C1x = Jx[i];
            C1y = Jy[i];
            s1[i] = sqrt((double)(ksqr(C1x) + ksqr(C1y)));
          }          

          freeimage(img3);
          freeimage(img4);
          return(1);
          break;

        case 4 :              /*** 4 or 8 connectivity dist. function ***/
        case 8 :

          d[0] = -nc;          /* neighborhood pixels */
          d[1] = 1;
          d[2] = nc;
          d[3] = -1;
          d[4] = 1 - nc;
          d[5] = 1 + nc;
          d[6] = nc - 1;
          d[7] = -1 - nc; 

          for (i = 0; i < k; i++)   /* Copy original BYTE1 to BYTE2 image */
            if (c1[i] == 255)
              s1[i] = MAXVAL;
            else
              s1[i] = 0;

          for (i = 0; i < nc; i++) {      /* Write 0 to image frame */
            s1[i] = 0;
            s1[i + k - nc] = 0;
          }
          for (i = 0; i < k; i += nc) {
            s1[i] = 0;
            s1[i + nc - 1] = 0;
          }

          queue_init(&q);            /* initialize pixels' queue */

          for (i = 0; i < k; i++) {                /* find border pixels */
            j = 0;
            flag = 0;
            if (s1[i] == MAXVAL)
              while ((!flag) && (j < cv))
                if (s1[i + d[j++]] == 0) {
                  p_num.address = i;
                  queue_add(&q, &p_num);
                  s1[i] = 1;
                  flag = 1;
                }
          }

          while (!queue_empty(&q)) {              /* inner layers */
            queue_first(&q, &p_num);
            i = p_num.address;
            for (j = 0; j < cv; j++) {
              l = i + d[j];
              if (s1[l] == MAXVAL) {
                s1[l] = s1[i] + 1;
                p_num.address = l;
                queue_add(&q, &p_num); 
              }
            }
          } 

          return(1);
          break;

        default :
          kfprintf(kstderr,"lvdist: illegal value on parameter cv\n");
          return(0);
          break;
      }
 
    case VFF_TYP_2_BYTE :
      kfprintf(kstderr,"lvdist: BYTE2 not implemented\n");
      return(0);
      break;

    case VFF_TYP_4_BYTE :
      kfprintf(kstderr,"lvdist: BYTE4 not implemented\n");
      return(0);
      break;

    case VFF_TYP_FLOAT :
      kfprintf(kstderr,"lvdist: FLOAT not implemented\n");
      return(0);
      break;

    case VFF_TYP_COMPLEX :
      kfprintf(kstderr,"lvdist: COMPLEX not implemented\n");
      return(0);
      break;

    case VFF_TYP_DOUBLE :
      kfprintf(kstderr,"lvdist: DOUBLE not implemented\n");
      return(0);
      break;

    case VFF_TYP_DCOMPLEX :
      kfprintf(kstderr,"lvdist: COMPLEXD not implemented\n");
      return(0);
      break;

    default:
      kfprintf(kstderr,"lvdist: Unknown data storage type, %d\n",img1->data_storage_type);
      return(0);
      break;
  }
}
/* -library_code_end */
