 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vedgeoff
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvedgeoff
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvedgeoff - Eliminates the objects that hit the window.
* 
*       Purpose: .I vedgeoff
will eliminate the objects that hit the window.
img1 is the input images; b is a structuring element;
img2 is the output image.
*         Input: .IP "img1" 15  
input image
.IP "b" 15
structuring element
.IP "border" 15
mask to the last pixel of a row (used in formatted BIT images)
*        Output: .IP "img2" 15
holds the result of the edge off operation. The output data
type is the same as the input data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvedgeoff
can be defined for all data types supported by Khoros, but at the moment it has been implemented just for the bit and unsigned char types.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Junior Barrera
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: Adriano Nagelschmidt Rodrigues: port to new toolbox version.
****************************************************************/
/* -library_def */
int
lvedgeoff(xvimage *img1, mat_str *b, unsigned long border, xvimage *img2)
/* -library_def_end */

/* -library_code */
{
  int value;

  if (!lvwind(img2, border))
    return(0);
  if (!lvopenrec(img1, img2, b, NULL, 0, border, img2))
    return(0); 
  if (!lvsubm(img1, img2, 1, 0, NULL, 0, img2))
    return(0);

  return(1);
}
/* -library_code_end */
