 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vequal
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvequal
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvequal - Compares two images, if they are equal affects a cantata variable with one, otherwise affects it with zero.
* 
*       Purpose: .I lvequal
compares two imput images pixel by pixel, if they are equal affects a cantata variable with one, otherwise affects it with zero.
*         Input: .IP "image1" 15
xvimage structure
.IP "image2" 15
xvimage structure
.IP "img2_flg" 15
a flag set if img2 is present
.IP "c" 15
constant to compare
*        Output: .IP "value" 15
an int value (0 or 1).
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvequal can be defined for all data types supported by khoros, but at the moment it is implemented only for bit and unsigned char images.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Junior Barrera
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: Adriano Nagelschmidt Rodrigues, Luiz Fernando Vieira: port to new toolbox, constant image option.
****************************************************************/
/* -library_def */
int
lvequal(xvimage *img1, xvimage *img2, int img2_flg, int c, int *value)
/* -library_def_end */

/* -library_code */
{    
    unsigned char *c1, *c2;                /* auxiliar pointers */
    int i, j, col32, row;
    int nr, nc, tnp;                               /* total number of pixels */

    unsigned long *l1, *l2, aux;
      
    if ((!img2_flg) && (c > 255 || c < 0)) {
      kfprintf(kstderr, "lvequal: illegal constant.\n");
      return(0);
    }
    
    switch (img1->data_storage_type)
      {
        case VFF_TYP_BIT :
          l1 = (unsigned long *)(img1->imagedata);
          l2 = (unsigned long *)(img2->imagedata);
          nc = img1->row_size;
          nr = img1->col_size;
          col32 = (nc + 31) >> 5;
          tnp = nr * col32;
          *value = 1;
          if (img2_flg) {
            for (i = 0; i < tnp; i++) 
              if (*(l1++) != *(l2++)) {
                *value = 0;
                 return(1);
              }
          }
          else {
            if (!c) {
              for (i = 0; i < tnp; i++)
                if (*(l1++) != 0) {
                  *value = 0;
                  return(1);
                }
            }
            else {
              aux = 0xFFFFFFFFL;
              aux >>= (nc % 32);
              row = col32 - 1;
              for (j = 0; j < nr; j++) {
                l1[row] |= aux;
                row += col32;
              }
              aux = 0xFFFFFFFFL;
              for (i = 0; i < tnp; i++)
                if (*(l1++) != aux) {
                  *value = 0;
                  return(1);
                }
            }
          }   
          return(1);            
          break;

        case VFF_TYP_1_BYTE :
          tnp = img1->row_size * img1->col_size * img1->num_data_bands;
          c1 = (unsigned char *)(img1->imagedata);
          c2 = (unsigned char *)(img2->imagedata);
          *value = 1;
          if (img2_flg) { 
            for (i = 0; i < tnp; i++) 
              if (*(c1++) != *(c2++)) {
                *value = 0;
                return(1);
              }
          }
          else
            for (i = 0; i < tnp; i++)
              if (*(c1++) != c) {
                *value = 0;
                return(1);
              }   
          return(1);
          break;
 
        case VFF_TYP_2_BYTE :
          kfprintf(kstderr,"lvequal: BYTE2 not implemented\n");
          return(0);
          break;

        case VFF_TYP_4_BYTE :
          kfprintf(kstderr,"lvequal: BYTE4 not implemented\n");
          return(0);
          break;

        case VFF_TYP_FLOAT :
          kfprintf(kstderr,"lvequal: FLOAT not implemented\n");
          return(0);
          break;

        case VFF_TYP_COMPLEX :
          kfprintf(kstderr,"lvequal: COMPLEX not implemented\n");
          return(0);
          break;

        case VFF_TYP_DOUBLE :
          kfprintf(kstderr,"lvequal: DOUBLE not implemented\n");
          return(0);
          break;

        case VFF_TYP_DCOMPLEX :
          kfprintf(kstderr,"lvequal: COMPLEXD not implemented\n");
          return(0);
          break;

        default:
          kfprintf(kstderr,"lvequal: Unknown data storage type, %d\n",img1->data_storage_type);
          return(0);
          break;
      }
}
/* -library_code_end */
