 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vextintersec
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvextintersec
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvextintersec - Do the infimum of multiple images pixel by pixel with an operation gating mask.
* 
*       Purpose: .I vminf
will do the infimum of multiple (up to 6) images pixel by pixel.
img1 and img2 are the required input images; img3, img4, img5 and img6 are optional input images. img7 is the gating image, and mflg should be set to 1 if the gating image is to be used. img8 is the output image.
A non-zero mask pixel in img7 enables the output pixel to contain 
the infimum. A zero mask pixel in img7 just transfers the value 
of the corresponding pixel in img1 to the output pixel.
.LP
The input images must be of the same size, data type, and have 
the same number of data bands, as must the optional mask and output images.
*         Input: .IP "img1" 15  
first xvimage structure
.IP "img2" 15
second xvimage structure
.IP "img3" 15  
optional third xvimage structure
.IP "i3_flg" 15
a flag set if the third image is available
.IP "img4" 15
optional fourth xvimage structure
.IP "i4_flg" 15
a flag set if the fourth image is available
.IP "img5" 15  
optional fifth xvimage structure
.IP "i5_flg" 15
a flag set if the fifth image is available
.IP "img6" 15
optional sixth xvimage structure
.IP "i6_flg" 15
a flag set if the sixth image is available
.IP "img7" 15
gating mask image
.IP "mflg" 15
a flag set (equal 1) if gating image mask available
.IP
Note that all inputs must be of the same data type and size.
*        Output: .IP "img8" 15
holds the result of the inf operation.  The output data
type is the same as the input data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvminf
can be defined for all data types supported by Khoros, but at the moment it has been implemented just for the bit and unsigned char types.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Adriano Nagelschmidt Rodrigues
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int
lvminf(xvimage *img1, xvimage *img2, xvimage *img3, int i3_flg, xvimage *img4, int i4_flg, xvimage *img5, int i5_flg, xvimage *img6, int i6_flg, xvimage *img7, int mflg, xvimage *img8)
/* -library_def_end */

/* -library_code */
{
  if (!lvinf(img1, img2, 1, 0, img7, mflg, img8))
    return(0);
  if (i3_flg)
    if (!lvinf(img8, img3, 1, 0, img7, mflg, img8))
      return(0);
  if (i4_flg)
    if (!lvinf(img8, img4, 1, 0, img7, mflg, img8))
      return(0);
  if (i5_flg)
    if (!lvinf(img8, img5, 1, 0, img7, mflg, img8))
      return(0);
  if (i6_flg)
    if (!lvinf(img8, img6, 1, 0, img7, mflg, img8))
      return(0);

  return(1);
}
/* -library_code_end */
