 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vframe
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvframe
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvframe - Builds a window.
* 
*       Purpose: .I lvwind
will build a window of the dimensions of the input image.
img1 is an input image; img2 is the output image.
*         Input: .IP "img1" 15  
 xvimage structure
*        Output: .IP "img1" 15
holds the result of the window operation.  The output data
type is the same as the input data type.
.IP "border" 15
mask to the last pixel of a row (used in formatted BIT images)
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvwind
can be defined for all data types supported by Khoros, but at the moment it has been implemented just for the bit and unsigned char types.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Adriano Nagelschmidt Rodrigues
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int
lvwind(xvimage *img1, long unsigned int border)
/* -library_def_end */

/* -library_code */
{
 int j, i, k, col32, row;
 int nr, nc, nb;
 unsigned char *c1, *ref;
 unsigned long *l1;

 switch (img1->data_storage_type) {
   case VFF_TYP_BIT:
     nr = img1->col_size;
     if ((img1->row_size < 65) || (img1->col_size < 3)) {
       kfprintf(kstderr,"lvwind: BIT images should be at least 65 pixels wide and 3 pixels high.\n"); 
       return(0);
     }            
     col32 = (img1->row_size + 31) >> 5;                       
     l1 = (unsigned long *)(img1->imagedata);
     for (j = 0; j < nr; j++) {
       l1[j * col32] =  0x80000000L;
       l1[j * col32 + col32 - 1] = border;
     }
     for (j = 0; j < col32; j++) {
       l1[j] = -1;
       l1[(nr - 1) * col32 + j] = -1;
     }
     for (i = 1; i < nr - 1; i++) {
       row = i * col32;
       for (j = 1; j < col32 - 1; j ++)
         l1[row + j] = 0;
     }
     return(1);     
     break;

   case VFF_TYP_1_BYTE :
     nr = img1->col_size;     
     nc = img1->row_size;
     nb = img1->num_data_bands;                       
     c1 = (unsigned char *)(img1->imagedata);
     for (k = 0; k < nb; k++) {
       ref = c1 + nr * nc * k;
       for (j = 1; j < nc - 1; j++) {
         *(ref + j) = 255;
         *(ref + (nr - 1) * nc + j) = 255;
       }
       for (j = 0; j < nr; j++) {
         *(ref + j * nc) =  255;
         *(ref + j * nc + nc - 1) = 255;
       }
       for (i = 1; i < nr - 1; i++) {
         row = i * nc;
         for (j = 1; j < nc - 1; j ++)
           *(ref + row + j) = 0;
       }
     }      
     return(1);
     break;

   case VFF_TYP_2_BYTE :
     kfprintf(kstderr,"lvinf: BYTE2 not implemented\n");
     return(0);
     break;

   case VFF_TYP_4_BYTE :
     kfprintf(kstderr,"lvinf: BYTE4 not implemented\n");
     return(0);
     break;

   case VFF_TYP_FLOAT :
     kfprintf(kstderr,"lvinf: FLOAT not implemented\n");
     return(0);
     break;

   case VFF_TYP_COMPLEX :
     kfprintf(kstderr,"lvinf: COMPLEX not implemented\n");
     return(0);
     break;

   case VFF_TYP_DOUBLE :
     kfprintf(kstderr,"lvinf: DOUBLE not implemented\n");
     return(0);
     break;

   case VFF_TYP_DCOMPLEX :
     kfprintf(kstderr,"lvinf: COMPLEXD not implemented\n");
     return(0);
     break;

   default:
     kfprintf(kstderr,"lvinf: Unknown data storage type, %d\n",img1->data_storage_type);
     return(0);
     break;
  }
}
/* -library_code_end */
