 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vicanon
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvicanon
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvicanon - Do the n-canonical dual transformation of an image, with an operation gating mask.
* 
*       Purpose: .I lvncanond
will do the n-canonical dual transformation of an image by two set structuring elements.
img1, b1 and b2 are, respectively, the input image and the structuring elements; r is the struct. el. rotation step. img2 is the gating image, and mflg should be set to 1 if the gating image is to be used. img3 is the output image.
A non-zero mask pixel in img2 enables the output pixel to contain 
the n-canonical dual transformation. A zero mask pixel in img2 just transfers the value 
of the corresponding pixel in img1 to the output pixel.
.LP
The input and mask images must be of the same size, data type, and have 
the same number of data bands.
*         Input: .IP "img1" 15  
input image 
.IP "b1" 15
first set structuring element
.IP "b2" 15
second set structuring element
.IP "r" 15
struct. element rotation step
.IP "img2" 15
gating mask image
.IP "mflg" 15
a flag set (equal 1) if gating image mask available
.IP "border" 15
mask to the last pixel of a row (used in formatted BIT images)
.IP
Note that the input and mask images must be of the same data type and size.
*        Output: .IP "img3" 15
holds the result of the n-canonical dual transformation operation.  The output data
type is the same as the input data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvncanond
can be defined for all data types supported by Khoros, but at the moment it has been implemented just for the bit and unsigned char types.
The structuring elements are subsets of the 3x3 matrix and the origin is always at the center of this matrix.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Adriano Nagelschmidt Rodrigues
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int
lvncanond(xvimage *img1, mat_str *b1, mat_str *b2, int r, xvimage *img2, int mflg, unsigned long border, xvimage *img3)
/* -library_def_end */

/* -library_code */
{
   xvimage *img4;
   mat_str b1_rot, b2_rot;
   int i;
 
   img4 = createimage(img1->col_size, img1->row_size, img1->data_storage_type, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);

   if (img4 == NULL) {
     (void) kfprintf(kstderr, "lvncanond: createimage Failed\n");
     return(0);
   }

   if (!setimage(img3))
     return(0);

   for (i = 0; i < 8; i += r) {
     if (!lvrotstr(b1, i, 1, &b1_rot))
       return(0);
     if (!lvrotstr(b2, i, 1, &b2_rot))
       return(0);
     if (!lvinfgen(img1, &b1_rot, &b2_rot, img2, mflg, border, img4))
       return(0);
     if (!lvinf(img3, img4, 1, 0, img2, mflg, img3))
       return(0);
   }

   freeimage(img4);
   return(1);
}
/* -library_code_end */
