 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for viff2str
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lviff2str
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lviff2str - Converts a VIFF image to a structuring element file
* 
*       Purpose: .I viff2str
Converts an input Khoros image to a \fBmorphological kernel\fP structure.
.LP
If the kernel type is not supplied, Planar will be assumed.
*         Input: .IP
.br
img  - the VIFF image structure.
.br
type - a flag wich specifies planar (0) or non-planar (1) kernels
*        Output: .IP
.br
ker  - the kernel output structure
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: 
*    Written By: Ulisses Braga
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int
lviff2str (xvimage *img, mat_str *ker, int type)
/* -library_def_end */

/* -library_code */
{
    int           i,j,k,nr,nc,nb,size;
    int           l1,l2,l3,l4,logic;
    char          *program = "lviff2str";
    unsigned char *ppi;
    short         *pni;

    /* Error checking: */
    proper_num_images(program,img,1,TRUE);  
    proper_map_enable(program,img,VFF_MAP_OPTIONAL,TRUE); 

    nr   = img->col_size;
    nc   = img->row_size;
    nb   = img->num_data_bands;
    size = nr*nc*nb;

    switch(type) {
    case 0: /* PLANAR */
      if (!lvconvert(img,VFF_TYP_1_BYTE,TRUE,FALSE,1.0,0.0,FALSE)) {
        kfprintf(kstderr,"%s: lvconvert Failed.\n",program); 
        return(0); 
        } 
      ppi = (unsigned char *) img->imagedata;

      if ((nr<=3) && (nc<=3) && (nb==1)) {
        /* This should become a MAT structuring element: */
        ker->tag = MM_MAT;
        l1 = (nr!=1) && (nc!=1);
        l2 = (nr==1) && (nc!=1);
        l3 = (nr!=1) && (nc==1);
        l4 = (nr==1) && (nc==1);
        for (i=0; i<3; i++)
          for (j=0; j<3; j++) {
            logic = (i<nr && j<nc && l1) || (i==1 && j<nc && l2) ||
                    (i<nr && j==1 && l3) || (i==1 && j==1 && l4);
          ker->x[j][i] = (logic) ? *ppi++ : 0;
          /* OBS: MAT format is the transposed of Khoros format */
            }
        }

      else { 
        /* Let's stick with the IMG structuring element: */
        ker->tag = MM_IMG;
        ker->m = nr;
        ker->n = nc;
        ker->p = nb;
        ker->dat = (short *) kmalloc(size*sizeof(short)); 
        for (i=0; i<size; i++)
          ker->dat[i] = (*ppi++) ? 0 : -1;
        }
      break;

      case 1: /* NONPLANAR */
        /* Convert data: */
        if (!lvconvert(img,VFF_TYP_2_BYTE,FALSE,FALSE,0.0,0.0,FALSE)) {
          kfprintf(kstderr,"%s: lvconvert Failed.\n",program); 
          return(0); 
          } 
        pni = (short *) img->imagedata;
        ker->tag = MM_IMG;
        ker->m = nr;
        ker->n = nc;
        ker->p = nb;
        ker->dat = (short *) kmalloc(size*sizeof(short)); 
        for (i=0; i<size; i++)
          ker->dat[i] = (short) *pni++;
        break;
      }

    return(1); /* Success */
}
/* -library_code_end */
