 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vintersec
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvintersec
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvintersec - Do the inf of two images pixel by pixel with an operation gating mask.
* 
*       Purpose: .I vinf
will do the inf of two images pixel by pixel.
img1 and img2 are the input images; img3 is the gating image, and mflg should be set to 1 if the gating image is to be used. img4 is the output image.  An optional parameter (that will represent a constant image) can be entered instead of img2. Note that img1 or img2 can be used for both the input and the resulting output xvimage structure, in order to save space.
A non-zero mask pixel in img3 enables the output pixel to contain 
the inf. A zero mask pixel in img3 just transfers the value 
of the corresponding pixel in img1 to the output pixel.
.LP
The two input images must be of the same size, data type, and have 
the same number of data bands, as must the optional mask and output images.
*         Input: .IP "img1" 15  
first xvimage structure
.IP "img2" 15
second xvimage structure
.IP "img2_flg"
a flag set if img2 is available
.IP "c"
int parameter to be used if img2 is not present
.IP "img3" 15
gating mask image
.IP "mflg" 15
a flag set (equal 1) if gating image mask available
.IP
Note that all inputs must be of the same data type and size.
*        Output: .IP "img4" 15
holds the result of the inf operation.  The output data
type is the same as the input data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvinf
can be defined for all data types supported by Khoros, but at the moment it has been implemented just for the bit and unsigned char types.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Junior Barrera
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: Port to new toolbox version and constant image option added by Adriano Nagelschmidt Rodrigues and Luiz Fernando Vieira
****************************************************************/
/* -library_def */
int
lvinf(xvimage *img1, xvimage *img2, int img2_flg, int c, xvimage *img3, int mflg, xvimage *img4)
/* -library_def_end */

/* -library_code */
{
 int j, col32, k;
 unsigned char *c1, *c2, *c3, *c4;
 unsigned long *l1, *l2, *l3, *l4;

 if ((!img2_flg) && (c > 255 || c < 0)) {
   kfprintf(kstderr, "lvaddm: illegal constant value.\n");
   return(0);
 }

 switch (img1->data_storage_type) {
   case VFF_TYP_BIT:
      if (!img2_flg) {
        kfprintf(kstderr, "lvinf: cannot compare constant and BIT image.\n");
        return(0);
      }
      col32 = (img1->row_size + 31) >> 5;
      k = img1->col_size * col32;
      l1 = (unsigned long *)(img1->imagedata);
      l2 = (unsigned long *)(img2->imagedata);
      l4 = (unsigned long *)(img4->imagedata);
      if (!mflg)
        for (j=0; j<k; j++) {
          *l4 = *l1 & *l2;
          l1++;
          l2++;
          l4++;
        }
      else {
        l3 = (unsigned long *)(img3->imagedata);
        for (j=0; j<k; j++) {
          *l4 = ((*l1 & *l2) & *l3) | (*l1 & (~(*l3)));
          l1++;
          l2++;
          l3++;
          l4++;
        }
      }
      return(1);     
     break;

   case VFF_TYP_1_BYTE : 
     k = img1->row_size * img1->col_size * img1->num_data_bands;
     c1 = (unsigned char *)(img1->imagedata);
     if (img2_flg)
       c2 = (unsigned char *)(img2->imagedata);
     c4 = (unsigned char *)(img4->imagedata);
      if (!mflg) {
        if (img2_flg)         
          for (j=0; j<k; j++) {
            if (*c1 <= *c2)
              *c4 = *c1;
            else 
              *c4 = *c2;
            c1++;
            c2++;
            c4++; 
          }
        else
          for (j=0; j<k; j++) {
            if (*c1 <= c)
              *c4 = *c1;
            else
              *c4 = c;
            c1++;
            c4++;
          }
      }               
      else {
        c3 = (unsigned char *)(img3->imagedata);
        if (img2_flg)
          for (j=0; j<k; j++) {
            if (*c3) 
              if (*c1 <= *c2)
                *c4 = *c1;
              else 
                *c4 = *c2;  
            else
              *c4 = *c1;
            c1++;
            c2++;
            c3++;
            c4++;
          }
        else
          for (j=0; j<k; j++) {
            if (*c3)
              if (*c1 <= c)
                *c4 = *c1;
              else
                *c4 = c;
            else
              *c4 = *c1;
            c1++;
            c3++;
            c4++;
         }
      }
     return(1);
     break;

   case VFF_TYP_2_BYTE :
     kfprintf(kstderr,"lvinf: BYTE2 not implemented\n");
     return(0);
     break;

   case VFF_TYP_4_BYTE :
     kfprintf(kstderr,"lvinf: BYTE4 not implemented\n");
     return(0);
     break;

   case VFF_TYP_FLOAT :
     kfprintf(kstderr,"lvinf: FLOAT not implemented\n");
     return(0);
     break;

   case VFF_TYP_COMPLEX :
     kfprintf(kstderr,"lvinf: COMPLEX not implemented\n");
     return(0);
     break;

   case VFF_TYP_DOUBLE :
     kfprintf(kstderr,"lvinf: DOUBLE not implemented\n");
     return(0);
     break;

   case VFF_TYP_DCOMPLEX :
     kfprintf(kstderr,"lvinf: COMPLEXD not implemented\n");
     return(0);
     break;

   default:
     kfprintf(kstderr,"lvinf: Unknown data storage type, %d\n",img1->data_storage_type);
     return(0);
     break;
  }
}
/* -library_code_end */
