 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vlabelm
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvlabelm
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
#define MAXVAL 32767
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvlabelm - Do the labeling of a binary image.
* 
*       Purpose: .I lvlabelexp
Will do the labeling of a binary image.
img1 is the input image (of the VIFF_TYP_1_BYTE data type), img2 is the output image (of the VFF_TYP_2_BYTE data type). cv is an integer parameter that indicates wich connectivity is desired (4 for 4-connectivity or 8 for 8-connectivity).
*         Input: .IP "img1" 15  
input xvimage structure of the VIFF_TYP_1_BYTE data type.
.IP "cv" 15
chosen connectivity: 4 or 8.
.IP
Note that img1 should be a binary image of the BYTE1 data type.
*        Output: .IP "img2" 15
holds the labeling operation. The output image should be of the VIFF_TYP_2_BYTE data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvlabelexp
The input and output images must be of the same size. The input and output image data types should be, respectively, VFF_TYP_1_BYTE and VFF_TYP_2_BYTE.
*    Written By: Adriano Nagelschmidt Rodrigues
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int
lvlabelexp(xvimage *img1, int cv, xvimage *img2)
/* -library_def_end */

/* -library_code */
{
  int nc, nr, k, r;
  int d[8], label, flag;
  register int i, j, l;
  item_str p_num;
  queue_str q;
  unsigned char *c1;
  short *s1;

  if ((cv != 4) && (cv != 8)) {
    kfprintf(kstderr,"lvlabelexp: illegal value on parameter cv\n");
    return(0);
  }

  switch (img1->data_storage_type) {
    case VFF_TYP_BIT :
      kfprintf(kstderr,"lvlabelexp: BIT not implemented\n");
      return(0);
      break;

    case VFF_TYP_1_BYTE :
      nc = img1->row_size;
      nr = img1->col_size;
      k = nc * nr;
      c1 = (unsigned char *)(img1->imagedata); 
      s1 = (short *)(img2->imagedata);
      label = 1;

      d[0] = -nc;          /* neighborhood pixels */
      d[1] = 1;
      d[2] = nc;
      d[3] = -1;
      d[4] = 1 - nc;
      d[5] = 1 + nc;
      d[6] = nc - 1;
      d[7] = -1 - nc; 

      for (i = 0; i < k; i++)   /* Copy original BYTE1 to BYTE2 image */
        if (c1[i] == 255)
          s1[i] = MAXVAL;
        else
          s1[i] = 0;

      for (i = 0; i < nc; i++) {      /* Write 0 to image frame */
        s1[i] = 0;
        s1[i + k - nc] = 0;
      }
      for (i = 0; i < k; i += nc) {
        s1[i] = 0;
        s1[i + nc - 1] = 0;
      }

      queue_init(&q);            /* initialize pixels' queue */

      for (i = 0; i < k; i++)     /* labeling */
        if (s1[i] == MAXVAL) {
          p_num.address = i;
          queue_add(&q, &p_num);
          s1[i] = label;
          while (!queue_empty(&q)) {
            queue_first(&q, &p_num);
            j = p_num.address;
            for (r = 0; r < cv; r++) {
              l = j + d[r];
              if (s1[l] == MAXVAL) {
                p_num.address = l;
                queue_add(&q, &p_num);
                s1[l] = label;
              }
            }
          }
          label++;
        }

      return(1);
      break;
 
    case VFF_TYP_2_BYTE :
      kfprintf(kstderr,"lvlabelexp: BYTE2 not implemented\n");
      return(0);
      break;

    case VFF_TYP_4_BYTE :
      kfprintf(kstderr,"lvlabelexp: BYTE4 not implemented\n");
      return(0);
      break;

    case VFF_TYP_FLOAT :
      kfprintf(kstderr,"lvlabelexp: FLOAT not implemented\n");
      return(0);
      break;

    case VFF_TYP_COMPLEX :
      kfprintf(kstderr,"lvlabelexp: COMPLEX not implemented\n");
      return(0);
      break;

    case VFF_TYP_DOUBLE :
      kfprintf(kstderr,"lvlabelexp: DOUBLE not implemented\n");
      return(0);
      break;

    case VFF_TYP_DCOMPLEX :
      kfprintf(kstderr,"lvlabelexp: COMPLEXD not implemented\n");
      return(0);
      break;

    default:
      kfprintf(kstderr,"lvlabelexp: Unknown data storage type, %d\n",img1->data_storage_type);
      return(0);
      break;
  }
}
/* -library_code_end */
