 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vless
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvless
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvless - Check if an image is always equal or less than another image.
* 
*       Purpose: .I lvless
compares two images pixel by pixel and informs if img2 is always equal or less than img1, in wich case it returns a variable set to one, or set to zero otherwise. A constant image (that can be entered in the place of img1 or img2) can be specified by an integer parameter.
.LP
.I lvless
returns a value of one upon success and zero on failure.
*         Input: .IP "img1" 15  
This is the first image.
.IP "i1_flg" 15
A flag set (equal 1) if img1 is a parameter.
.IP "u" 15
This is an int value for a constant image. 
.IP "img2" 15
This is the second image.
.IP "i2_flg" 15
A flag set (equal 1) if img2 is a parameter.
.IP "l" 15
This is an int value for a constant image.
.IP
Note that all inputs must be of the same data type and size.
*        Output: .IP "value"
an int value (0 or 1); set if img1 >= img2 for every pixel;
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvless can be defined for all data types supported by khoros, but at the moment it is implemented only for bit and unsigned char images.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Adriano Nagelschmidt Rodrigues
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int
lvless(xvimage *img1, int i1_flg, int u, xvimage *img2, int i2_flg, int l, int *value) 
/* -library_def_end */

/* -library_code */
{
  int j, k;
  unsigned char *c1, *c2;
  unsigned long *l1, *l2;
  int col32, nr, nc, tnp;
 
  switch (img1->data_storage_type) {
    case VFF_TYP_BIT :
      l1 = (unsigned long *)(img1->imagedata);
      l2 = (unsigned long *)(img2->imagedata);
      nc = img1->row_size;
      nr = img1->col_size;
      col32 = (nc + 31) >> 5;
      tnp = nr * col32;
      *value = 1;
      if (!i1_flg || !i2_flg) {
        kfprintf(kstderr, "lvless: no constant values allowed with BIT image.\n");
        return(0);
      }
      for (j = 0; j < tnp; j++) {
        if ((*l2 | *l1) != *l1)
          *value = 0;
        l1++;
        l2++;
      }
      break;

    case VFF_TYP_1_BYTE : 
      k = img1->row_size * img1->col_size * img1->num_data_bands;
      *value = 1;
      if ((l < 0 || l > 255) || (u < 0 || u > 255)) {
        kfprintf(kstderr, "lvless: illegal constant value.\n");
        return(0);
      }
      if (i1_flg && i2_flg) {
        c1 = (unsigned char *)(img1->imagedata);
        c2 = (unsigned char *)(img2->imagedata);
        for (j=0; j<k; j++)             
          if (*(c2++) > *(c1++)) {
            *value = 0;
            return(1);
          }
      }
      if (i1_flg && !i2_flg) {
        c1 = (unsigned char *)(img1->imagedata);
        for (j=0; j<k; j++)             
          if (l > *(c1++)) {
            *value = 0;
            return(1);
          }
      }
      if (!i1_flg && i2_flg) {
        c2 = (unsigned char *)(img2->imagedata);
        for (j=0; j<k; j++)             
          if (*(c2++) > u) {
            *value = 0;
            return(1);
          }
      }
      if (!i1_flg && !i2_flg)
        if (l > u)
          *value = 0;
      return(1);
      break;
 
    case VFF_TYP_2_BYTE :
      kfprintf(kstderr,"lvless: BYTE2 not implemented\n");
      return(0);
      break;

    case VFF_TYP_4_BYTE :
      kfprintf(kstderr,"lvless: BYTE4 not implemented\n");
      return(0);
      break;

    case VFF_TYP_FLOAT :
      kfprintf(kstderr,"lvless: FLOAT not implemented\n");
      return(0);
      break;

    case VFF_TYP_COMPLEX :
      kfprintf(kstderr,"lvless: COMPLEX not implemented\n");
      return(0);
      break;

    case VFF_TYP_DOUBLE :
      kfprintf(kstderr,"lvless: DOUBLE not implemented\n");
      return(0);
      break;

    case VFF_TYP_DCOMPLEX :
      kfprintf(kstderr,"lvless: COMPLEXD not implemented\n");
      return(0);
      break;

    default:
      kfprintf(kstderr,"lvless: Unknown data storage type, %d\n",img1->data_storage_type);
      return(0);
      break;
  }
}
/* -library_code_end */
