 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vneg
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvneg
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvneg - Do the inversion of an image pixel by pixel, with an operation gating mask.
* 
*       Purpose: .I lvinv
will do the inversion of an image pixel by pixel.
img1 is an input image; img2 is the gating image, and mflg should be set to 1 if the gating image is to be used. img3 is the output image. Note that img1 can be used for both the input and the resulting output xvimage structure, in order to save space.
A non-zero mask pixel in img2 enables the output pixel to contain 
the inversion. A zero mask pixel in img2 just transfers the value 
of the corresponding pixel in img1 to the output pixel.
.LP
The input, output and the optional mask images must be of the same size, 
data type, and have the same number of data bands.
*         Input: .IP "img1" 15  
first xvimage structure
.IP "img2" 15
gating mask image
.IP "mflg" 15
a flag set (equal 1) if gating image mask available
.IP
Note that all inputs must be of the same data type and size.
*        Output: .IP "img3" 15
holds the result of the inversion operation.  The output data
type is the same as the input data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvinv
can be defined for all data types supported by Khoros, but at the moment it has been implemented just for the bit and unsigned char types.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Junior Barrera
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: General modifications made by Luiz Fernando Vieira, Adriano Nagelschmidt Rodrigues
****************************************************************/
/* -library_def */
int
lvinv(xvimage *img1, xvimage *img2, int mflg, xvimage *img3)
/* -library_def_end */

/* -library_code */
{
  int    j, col32, k;
  unsigned char *c1, *c2, *c3;
  unsigned long *l1, *l2, *l3;

  switch (img1->data_storage_type) {
    case VFF_TYP_BIT :
      col32 = (img1->row_size + 31) >> 5;
      k = img1->col_size * col32;
      l1 = (unsigned long *)(img1->imagedata);
      l3 = (unsigned long *)(img3->imagedata);
      if (!mflg) 
        for(j=0; j<k; j++) {
          *l3 = ~(*l1);
           l1++;
           l3++;
        }
      else {
        l2 = (unsigned long *)(img2->imagedata);
        for (j=0; j<k; j++) {
          *l3 = (~(*l1) & *l2) | (*l1 & ~(*l2));
          l1++;
          l2++;
          l3++;
        }
      }
      return(1);
      break;

    case VFF_TYP_1_BYTE : 
      k = img1->row_size * img1->col_size * img1->num_data_bands;
      c1 = (unsigned char *)(img1->imagedata); 
      c3 = (unsigned char *)(img3->imagedata);    
      if (!mflg) 
        for (j=0; j<k; j++) {
          *c3 = 255 - *c1;
           c1++;
           c3++;
        }
      else {
        c2 = (unsigned char *)(img2->imagedata);
        for (j=0; j<k; j++) {
          if (*c2++) 
            *c3 = 255 - *c1;  
          else
            *c3 = *c1;
          c1++;
          c3++;
        }       
      }       
      return(1);
      break;
 
    case VFF_TYP_2_BYTE :
      kfprintf(kstderr,"lvinv: BYTE2 not implemented\n");
      return(0);
      break;

    case VFF_TYP_4_BYTE :
      kfprintf(kstderr,"lvinv: BYTE4 not implemented\n");
      return(0);
      break;

    case VFF_TYP_FLOAT :
      kfprintf(kstderr,"lvinv: FLOAT not implemented\n");
      return(0);
      break;

    case VFF_TYP_COMPLEX :
      kfprintf(kstderr,"lvinv: COMPLEX not implemented\n");
      return(0);
      break;

    case VFF_TYP_DOUBLE :
      kfprintf(kstderr,"lvinv: DOUBLE not implemented\n");
      return(0);
      break;

    case VFF_TYP_DCOMPLEX :
      kfprintf(kstderr,"lvinv: COMPLEXD not implemented\n");
      return(0);
      break;

    default:
      kfprintf(kstderr,"lvinv: Unknown data storage type, %d\n",img1->data_storage_type);
      return(0);
      break;
  }
}
/* -library_code_end */
