 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vnopen
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvnopen
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvnopen - N successive openings by the same structuring element.
* 
*       Purpose: .I vnopen
will do n successive openings by the same structuring element.
img1 is the input image; b is the structuring element; N is the number of openings performed; img2 is the gating image; img3 is the output image. mflg 
is a flag. mflg should be set to 1 if the gating image is to be used.
A non-zero mask pixel in img2 enables the output pixel to contain 
the result. A zero mask pixel in img2 just transfers the value 
of the corresponding pixel in img1 to the output pixel.
.LP
The input image and the optional mask image must be of the same size, data type, and have the same number of data bands.
*         Input: .IP "img1" 15  
input image
.IP "str" 15
structuring element
.IP "n" 15
number of successive openings
.IP "img2" 15
gating mask image
.IP "mflg" 15
a flag set (equal 1) if gating image mask available
.IP "border" 15
mask to the last pixel of a row (used in formatted BIT images)
.IP
Note that the input and mask images must be of the same data type and size.
*        Output: .IP "img3" 15
holds the result of the n successive openings. The output data
type is the same as the input data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvnopen
can be defined for all data types supported by Khoros, but at the moment it has been implemented just for the bit and unsigned char types.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Adriano Nagelschmidt Rodrigues
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int
lvnkopen(xvimage *img1, mat_str *b, int n, xvimage *img2, int mflg, unsigned long border, xvimage *img3)
/* -library_def_end */

/* -library_code */
{
  xvimage *img4;
        
  img4 = createimage(img1->col_size, img1->row_size, img1->data_storage_type, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);

  if (img4 == NULL) {
    (void) kfprintf(kstderr, "lvnopen: createimage Failed\n");
    return(0);
  }
         
  if (n>0) {
    if (!lvnero(img1, b, n, img2, mflg, border, img4))
      return(0);
    if (!lvndil(img4, b, n, img2, mflg, border, img3))
      return(0);
  }
  else {
    kfprintf(kstderr, "lvnopen: N must be greater than 0\n");
    return(0);
  }

  freeimage(img4);
  return(1);
}
/* -library_code_end */
