 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vopenseg
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvopenseg
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
#include <math.h>
#include <values.h>
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvopenseg - Opening by a straight line segment.
* 
*       Purpose: .I vopenseg
will do an opening of the input image by a straight line segment.
.LP
the input image is "img_in"
the straight line segment is defined by :
   - its length : "length"
   - its angle from the horizontal direction "alfa"
the output image will be "img_out".
.LP
"im_trans" and "img_trav" are template images.
*         Input: .IP "img_in" 15  
input image
.IP "alfa" 15
angle of the straight line segment from the horizontal
.IP "length" 15
length of the straight line segment
.IP "border" 15
mask to the last pixel of a row (used in formatted BIT images)
*        Output: .IP "img_out" 15
holds the result of the opening. The output data
type is the same as the input data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvopenseg
can be defined for all data types supported by Khoros, but at the moment it has been implemented just for the bit and unsigned char types.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Adriano Nagelschmidt Rodrigues,
Using Guy Pilchen's veroseg as a template.
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int
lvopenseg(xvimage *img_in, int alfa, int length, unsigned long border, xvimage *img_out)
/* -library_def_end */

/* -library_code */
{
   xvimage *img_aux;

   img_aux = createimage(img_in->col_size, img_in->row_size, img_in->data_storage_type, img_in->num_of_images, img_in->num_data_bands, img_in->comment, img_in->map_row_size, img_in->map_col_size, img_in->map_scheme,
img_in->map_storage_type, img_in->location_type, img_in->location_dim);

   if (img_aux == NULL) {
     (void) kfprintf(kstderr, "lvopenseg: createimage Failed\n");
     return(0);
   }

   if (!lveroseg(img_in, alfa, length, border, img_aux))
     return(0);
   if (!lvdilseg(img_aux, alfa, length, border, img_out))
     return(0);

   freeimage(img_aux);
   return(1);
}
/* -library_code_end */
