 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vrotstr
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvrotstr
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvrotstr - Rotates a structuring element.
* 
*       Purpose: .I vrotstr
rotates a structuring element. Clockwise or counterclockwise direction can be specified. The step can range from 0 to 7 (writes to a file).
*         Input: .IP "b_in" 15
input structuring element (mat_str object)
.IP "step" 15
rotation step
.IP "d" 15
a flag set if clockwise struct. el. rotation
*        Output: .IP "b_out" 15
output structuring element (mat_str object)
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: 
*    Written By: Junior Barrera & Adriano Nagelschmidt Rodrigues
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: Adriano Nagelschmidt Rodrigues: added generic struct. el. rotation.
****************************************************************/
/* -library_def */
int
lvrotstr (mat_str *b_in, int step, int d, mat_str *b_out)
/* -library_def_end */

/* -library_code */
{
   int i, j, k, rr, nc, nr, nb;
   short *s1, *s2;

   if ((step < 0) || (step > 7)) {
     kfprintf(kstderr, "lvrotstr: rotation step out of range\n");
     return(0);
   }

   if (!d && (step != 0))
     step = 8 - step;

   b_out->m = b_in->m;
   b_out->n = b_in->n;
   b_out->p = b_in->p;
   b_out->tag = b_in->tag;

   switch (b_in->tag) {
      
      /* Generic struct. el. rotation */
      case MM_IMG:

         if (step != 4) {
           kfprintf(kstderr, "lvrotstr: the only valid rot. step with generic struct. el. is 4\n");
           return(0);
         }

         kfree_and_NULL(b_out->dat);
  
         nc = b_in->n;
         nr = b_in->m;
         nb = b_in->p;
         k = nr*nc;

         s2 = (short *)kmalloc(k*nb*sizeof(short));
         if (s2 == NULL) { 
           kfprintf(kstderr,"lvrotstr: Could not allocate enough memory!\n");
           return(0);
         }

         s1 = b_in->dat;
         for (j = 0; j < nb; j++) {
           rr = j*k;
           for (i = 0; i < k; i++)
             s2[k - 1 - i + rr] = s1[i + rr];
         }
         b_out->dat = s2;
         break;


      case MM_MAT:
        switch (step) {
 
          case 0:
             b_out->x[2][2] = b_in->x[2][2];
             b_out->x[2][1] = b_in->x[2][1];
             b_out->x[2][0] = b_in->x[2][0]; 
             b_out->x[1][2] = b_in->x[1][2];
             b_out->x[1][1] = b_in->x[1][1];
             b_out->x[1][0] = b_in->x[1][0];
             b_out->x[0][2] = b_in->x[0][2];
             b_out->x[0][1] = b_in->x[0][1];
             b_out->x[0][0] = b_in->x[0][0];
             break;

          case 1: 
             b_out->x[2][2] = b_in->x[2][1];
             b_out->x[2][1] = b_in->x[2][0];
             b_out->x[2][0] = b_in->x[1][0]; 
             b_out->x[1][2] = b_in->x[2][2];
             b_out->x[1][1] = b_in->x[1][1];
             b_out->x[1][0] = b_in->x[0][0];
             b_out->x[0][2] = b_in->x[1][2];
             b_out->x[0][1] = b_in->x[0][2];
             b_out->x[0][0] = b_in->x[0][1];
             break;

          case 2:
             b_out->x[2][2] = b_in->x[2][0];
             b_out->x[2][1] = b_in->x[1][0];
             b_out->x[2][0] = b_in->x[0][0]; 
             b_out->x[1][2] = b_in->x[2][1];
             b_out->x[1][1] = b_in->x[1][1];
             b_out->x[1][0] = b_in->x[0][1];
             b_out->x[0][2] = b_in->x[2][2];
             b_out->x[0][1] = b_in->x[1][2];
             b_out->x[0][0] = b_in->x[0][2];
             break;

          case 3: 
             b_out->x[2][2] = b_in->x[1][0];
             b_out->x[2][1] = b_in->x[0][0];
             b_out->x[2][0] = b_in->x[0][1]; 
             b_out->x[1][2] = b_in->x[2][0];
             b_out->x[1][1] = b_in->x[1][1];
             b_out->x[1][0] = b_in->x[0][2];
             b_out->x[0][2] = b_in->x[2][1];
             b_out->x[0][1] = b_in->x[2][2];
             b_out->x[0][0] = b_in->x[1][2];
             break;

          case 4: 
             b_out->x[2][2] = b_in->x[0][0];
             b_out->x[2][1] = b_in->x[0][1];
             b_out->x[2][0] = b_in->x[0][2]; 
             b_out->x[1][2] = b_in->x[1][0];
             b_out->x[1][1] = b_in->x[1][1];
             b_out->x[1][0] = b_in->x[1][2];
             b_out->x[0][2] = b_in->x[2][0];
             b_out->x[0][1] = b_in->x[2][1];
             b_out->x[0][0] = b_in->x[2][2];
             break;

          case 5:
             b_out->x[2][2] = b_in->x[0][1];
             b_out->x[2][1] = b_in->x[0][2];
             b_out->x[2][0] = b_in->x[1][2]; 
             b_out->x[1][2] = b_in->x[0][0];
             b_out->x[1][1] = b_in->x[1][1];
             b_out->x[1][0] = b_in->x[2][2];
             b_out->x[0][2] = b_in->x[1][0];
             b_out->x[0][1] = b_in->x[2][0];
             b_out->x[0][0] = b_in->x[2][1];
             break;

          case 6:
             b_out->x[2][2] = b_in->x[0][2]; 
             b_out->x[2][1] = b_in->x[1][2];
             b_out->x[2][0] = b_in->x[2][2]; 
             b_out->x[1][2] = b_in->x[0][1];
             b_out->x[1][1] = b_in->x[1][1];
             b_out->x[1][0] = b_in->x[2][1];
             b_out->x[0][2] = b_in->x[0][0];
             b_out->x[0][1] = b_in->x[1][0];
             b_out->x[0][0] = b_in->x[2][0];
             break;

          case 7: 
             b_out->x[2][2] = b_in->x[1][2]; 
             b_out->x[2][1] = b_in->x[2][2];
             b_out->x[2][0] = b_in->x[2][1]; 
             b_out->x[1][2] = b_in->x[0][2];
             b_out->x[1][1] = b_in->x[1][1];
             b_out->x[1][0] = b_in->x[2][0];
             b_out->x[0][2] = b_in->x[0][1];
             b_out->x[0][1] = b_in->x[0][0];
             b_out->x[0][0] = b_in->x[1][0];
             break;
 
        }
        break;
   }
   return(1);

}
/* -library_code_end */
