 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vsmoothseg
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvsmoothseg
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvsmoothseg - Do the smooth segmentation, with an operation gating mask.
* 
*       Purpose: .I lvsmooth
will do the smooth segmentation by a set of structuring elements.
img1, b1, b2 and b3 are, respectively, the input image and the structuring elements;
img2 is the conditional image, img3 is the gating image, and mflg should be set to 1 if the gating image is to be used. img4 is the output image.
A non-zero mask pixel in img3 enables the output pixel to contain 
the conditional skeleton by thinning. A zero mask pixel in img3 just transfers the value of the corresponding pixel in img1 to the output pixel.
.LP
The input and mask images must be of the same size, data type, and have 
the same number of data bands.
*         Input: .IP "img1" 15  
first image
.IP "img2" 15
second image 
.IP "b1" 15
first set structuring element
.IP "b2" 15
second set structuring element
.IP "b3" 15
third set structuring element
.IP "img3" 15
gating mask image
.IP "mflg" 15
a flag set (equal 1) if gating image mask available
.IP "border" 15
mask to the last pixel of a row (used in formatted BIT images)
.IP
Note that the input and mask images must be of the same data type and size.
*        Output: .IP "img4" 15
holds the result of the conditional skeleton by thinning operation.  The output data
type is the same as the input data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvsmooth
can be defined for all data types supported by Khoros, but at the moment it has been implemented just for the bit and unsigned char types.
The structuring elements are subsets of the 3x3 matrix and the origin is always at the center of this matrix.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Adriano Nagelschmidt Rodrigues
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int
lvsmooth(xvimage *img1, xvimage *img2, mat_str *b1, mat_str *b2, mat_str *b3, xvimage *img3, int mflg, unsigned long border, xvimage *img4)
/* -library_def_end */

/* -library_code */
{
  int value, n, i;
  xvimage *img5, *img6;

  img5 = createimage(img1->col_size, img1->row_size, img1->data_storage_type, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);

  if (img5 == NULL) {
    (void) kfprintf(kstderr, "lvsmooth: createimage Failed\n");
    return(0);
  }

  img6 = createimage(img1->col_size, img1->row_size, img1->data_storage_type, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);

  if (img6 == NULL) {
    (void) kfprintf(kstderr, "lvsmooth: createimage Failed\n");
    return(0);
  }

 if (!copyimagedata1(img1, img4))
   return(0);

  value = 1;
  n = 0;

  while (value == 1) { 
    if (!lvero(img4, b1, img3, mflg, border, img5))
      return(0);
    if (!lvless(img5, 1, 0, img2, 1, 0, &value))
      return(0);
    if (value)
      n++;
    if (!lvero(img5, b1, img3, mflg, border, img4))
      return(0);
    if (!lvless(img4, 1, 0, img2, 1, 0, &value))
      return(0);
    if (value)
      n++;
  }

  if (!copyimagedata1(img2, img5))
    return(0);
  
  for (i = n; i > 0; i--) {
    if (!lvnero(img1, b1, i, img3, mflg, border, img6))
      return(0);
    if (!lvcskthick(img5, img6, b2, b3, 1, 1, img3, mflg, border, img4))
      return(0);
    if (!copyimagedata1(img4, img5))
      return(0);
  }

  if (!lvcskthick(img5, img1, b2, b3, 1, 1, img3, mflg, border, img4))
      return(0);

  freeimage(img5);
  freeimage(img6);
  return(1);
}
/* -library_code_end */
