 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vsubm
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvsubm
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvsubm - Compute the difference (subtract) between two images pixel by pixel.
* 
*       Purpose: .I lvsubm
subtracts two images pixel by pixel with a operation gating mask. The second image can be chosen, optionally, as a constant.
.LP
img1 and img2 are input images, img3 is the gating image, and mflg should be set to one if the gating image is to be used. img4 is the output image. Note that img1 or img2 can be used for both the input and the resulting output xvimage structure, in order to save space.
.LP
A non-zero mask pixel in img3 enables the output pixel to contain the difference. A zero mask pixel in img3 just transfers the value of
the corresponding pixel in img1 to the output pixel.
.LP
The mask image must be the same size as the input image.
The output image data type is the same as the input data type.
.LP
.I lvsubm
returns a value of one upon success and a zero on failure.
*         Input: .IP
1. img1 -- first xvimage strcuture
.br
2. img2 -- second xvimage structure
.br
3. img2_flg -- flag set if second image is available
.br
4. c -- constant value to subtract, if img2 not present
.br
5. img3 -- gating mask image
.br
6. mflg -- a flag set (equal 1) if gating image mask
           available
.sp
The inputs must be of the same size.
*        Output: .IP
1. img4 -- holds the result of the subtract
           (img4 = img1 - img2) operation.
           The output data
           type is the same as the input data
           type.
.sp
lvsubm returns zero upon failure and one upon
success.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvsubm
can be defined for all data types supported by Khoros, but at the moment it has been implemented just for the bit and unsigned char types.
The structuring elements are subsets of the 3x3 matrix and the origin is always at the center of this matrix.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Junior Barrera
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: Luiz Fernando Vieira, Adriano Nagelschmidt Rodrigues: bug fixes and constant image option.
****************************************************************/
/* -library_def */
int
lvsubm(xvimage *img1, xvimage *img2, int img2_flg, int c, xvimage *img3, int mflg, xvimage *img4)
/* -library_def_end */

/* -library_code */
{
    int   j, k, col32;
    unsigned char *c1, *c2, *c3, *c4;
    unsigned long *l1, *l2, *l3, *l4;

    if ((!img2_flg) && (c > 255 || c < 0)) {
      kfprintf(kstderr, "lvaddm: illegal constant.\n");
      return(0);
    }

    switch (img1->data_storage_type)
      {
        case VFF_TYP_BIT :
          if (!img2_flg) {
            kfprintf(kstderr, "lvsubm: cannot subtract constant from BIT images.\n");
	    return(0);
          }
          col32 = (img1->row_size + 31) >> 5;
          k = img1->col_size * col32;
          l1 = (unsigned long *)(img1->imagedata);
          l2 = (unsigned long *)(img2->imagedata);
          l4 = (unsigned long *)(img4->imagedata);
          if (!mflg)
            for(j=0; j<k; j++) {
              *l4 = *l1 & (*l1 ^ *l2);
              l1++;
              l2++;
              l4++;
            }
          else {
            l3 = (unsigned long *)(img3->imagedata);
            for (j=0; j<k; j++) {
              *l4 = *l1 & (*l3 & (*l1 ^ *l2));
              l1++;
              l2++;
              l3++;
              l4++;
            }
          }
          return(1);
        break;
        case VFF_TYP_1_BYTE : 
          k = img1->row_size * img1->col_size * img1->num_data_bands;
          c1 = (unsigned char *)(img1->imagedata);
	  if (img2_flg)
            c2 = (unsigned char *)(img2->imagedata);
          c4 = (unsigned char *)(img4->imagedata);
          if (!mflg) {
            if (img2_flg)
              for (j=0; j<k; j++) {
                if (*c1 > *c2)
                  *c4 = *c1 - *c2;
                else
                  *c4 = 0;
                c1++;
                c2++;
                c4++;
              }
            else 
              for (j=0; j<k; j++) {
                if (*c1 > c)
                  *c4 = *c1 - c;
                else
                  *c4 = 0;
                c1++;
                c4++;
              }
          }
          else {
            c3 = (unsigned char *)(img3->imagedata);
            if (img2_flg)
              for (j=0; j<k; j++) {
                if (*c3)
                  if (*c1 > *c2)
                    *c4 = *c1 - *c2;
                  else
                    *c4 = 0;
                c1++;
                c2++;
                c3++;
                c4++;
              }
            else
              for (j=0; j<k; j++) {
                if (*c3)
                  if (*c1 > c)
                    *c4 = *c1 - c;
                  else
                    *c4 = 0; 
                c1++;
                c3++;
                c4++;
              }
          } 
          return(1);
        break;
        case VFF_TYP_2_BYTE :
          kfprintf(kstderr,"lvsubm: BYTE2 not implemented\n");
          return(0);
        break;
        case VFF_TYP_4_BYTE :
          kfprintf(kstderr,"lvsubm: BYTE4 not implemented\n");
          return(0);
        break;
        case VFF_TYP_FLOAT :
          kfprintf(kstderr,"lvsubm: FLOAT not implemented\n");
          return(0);
        break;
        case VFF_TYP_COMPLEX :
          kfprintf(kstderr,"lvsubm: COMPLEX not implemented\n");
          return(0);
        break;
        case VFF_TYP_DOUBLE :
          kfprintf(kstderr,"lvsubm: DOUBLE not implemented\n");
          return(0);
        break;
        default:
          kfprintf(kstderr,"lvsubm: Unknown data storage type, %d\n",img1->data_storage_type);
          return(0);
          break;
      }
}
/* -library_code_end */
