 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vsymdif
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvsymdif
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvsymdif - Compares two images pixel by pixel and shows different pixels.
* 
*       Purpose: .I lvsimdif
compares two imput images pixel by pixel and shows the pixels that are different. img1 and img2 are the input images. img3 is the gating image, and mflg should be set to 1 if the gating image is to be used. img4 is the output image. Note that img1 or img2 can be used for both input and resulting output xvimage structure, in order to save space.
A non-zero mask pixel in img2 enables the output pixel to contain the closing. A zero mask pixel in img2 just transfers the value of the corresponding pixel in img1 to the output pixel.
.LP
The input and mask images must be of the same size, data type, and have 
the same number of data bands.
*         Input: .IP "img1" 15
xvimage structure
.IP "img2" 15
xvimage structure
.IP "img2" 15
gating xvimage structure
.IP "mflg" 15
a flag set if gating mask image is present
*        Output: .IP "img3" 15
output xvimage structure
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvsimdif can be defined for all data types supported by khoros, but at the moment it is implemented only for bit and unsigned char images.
.LP
Due to the simultaneous processing of 32 pixels in BIT images, input images of the BIT data type must be converted to a special data structure before the library is called. This can be accomplished by using the format routine. After the library call, the unformat routine can convert the image back to the BIT data type.
*    Written By: Adriano Nagelschmidt Rodrigues
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int
lvsimdif(xvimage *img1, xvimage *img2, xvimage *img3, int mflg, xvimage *img4)
/* -library_def_end */

/* -library_code */
{    
  xvimage *img5;

  img5 = createimage(img1->col_size, img1->row_size, img1->data_storage_type, img1->num_of_images,
img1->num_data_bands, img1->comment, img1->map_row_size, img1->map_col_size, img1->map_scheme,
img1->map_storage_type, img1->location_type, img1->location_dim);

  if (img5 == NULL) {
    (void) kfprintf(kstderr, "lvsimdif: createimage Failed\n");
    return(0);
  }

  if (!lvsubm(img1, img2, 1, 0, img3, mflg, img5))
    return(0);
  if (!lvsubm(img2, img1, 1, 0, img3, mflg, img4))
    return(0);
  if (!lvsup(img5, img4,  1, 0, img3, mflg, img4))
    return(0);

  freeimage(img5);
  return(1);
}
/* -library_code_end */
