 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vthreshm
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvthreshm
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvthreshm - Do the adaptative threshold of an image pixel by pixel.
* 
*       Purpose: lvthreshad reads 2 limit images (img2 and img3) and an input image (img1) to be compared with the limit images. The comparision will be made pixel by pixel: if a pixel is between the limit pixels then the respective pixel in the output image will be 255, 0 otherwise. Note that img1, img2 or img3 can be used for both the input and the resulting output xvimage structure, in order to save space.
*         Input: .IP "img1" 15
This is the image to be compared.
.IP "img2" 15  
This is the lower limit image of the range.
.IP "i2_flg" 15
A flag set (equal 1) if lower limit image is available
.IP "l" 15
This is the lower constant
.IP "img3" 15
This is the upper limit image of the range.
.IP "i3_flg" 15
A flag set (equal 1) if upper limit image is available
.IP "u" 15
This is the upper constant
.IP "img4" 15
This is an optional masking image.
.IP "mflg" 15
A flag set (equal 1) if gating image mask available
.IP
Note that all inputs must be of the same data type and size.
*        Output: .IP "img5" 15
This is the result of the comparision.  The output data
type is the same as the input data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: lvthreshad is presently only implemented for byte and short images
*    Written By: Roberto Hirata Jr. & Adriano Nagelschmidt Rodrigues
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: Adriano Nagelschmidt Rodrigues
   - added lower/upper numeric selection options
   - .pane modifications
****************************************************************/
/* -library_def */
int
lvthreshad(xvimage *img1, xvimage *img2, int i2_flg, int l, xvimage *img3, int i3_flg, int u, xvimage *img4, int mflg, xvimage *img5) 
/* -library_def_end */

/* -library_code */
{
  int    j, k;
  unsigned char *c1, *c2, *c3, *c4, *c5;
  short *s1, *s2, *s3, *s4;
 
  switch (img1->data_storage_type) {

    case VFF_TYP_BIT :
      kfprintf(kstderr,"lvthreshad: BIT not defined\n");
      return(0);
      break;

    case VFF_TYP_1_BYTE : 
      k =  img1->row_size * img1->col_size * img1->num_data_bands;
      c1 = (unsigned char *)(img1->imagedata);     
      c5 = (unsigned char *)(img5->imagedata);
      if ((!i2_flg && (l<0 || l>255)) || (!i3_flg && (u<0 || u>255))) {
        kfprintf(kstderr, "lvthreshad: integer parameter out of range for BYTE image\n");
        return(0);
      } 

      if (!mflg) {
        if (i2_flg && i3_flg) {
          c2 = (unsigned char *)(img2->imagedata);
          c3 = (unsigned char *)(img3->imagedata);
          for (j=0; j<k; j++) {             
            if ((*c1 >= *c2) && (*c1 <= *c3))
              *c5 = 255;
            else
              *c5 = 0;     
            c1++;
            c2++;
            c3++;
            c5++;
          }
        }
        if (i2_flg && !i3_flg) {
          c2 = (unsigned char *)(img2->imagedata);
          for (j=0; j<k; j++) {             
            if ((*c1 >= *c2) && (*c1 <= u))
              *c5 = 255;
            else
              *c5 = 0;     
            c1++;
            c2++;
            c5++;
          }
        }
        if (!i2_flg && i3_flg) {
          c3 = (unsigned char *)(img3->imagedata);
          for (j=0; j<k; j++) {             
            if ((*c1 >= l) && (*c1 <= *c3))
              *c5 = 255;
            else
              *c5 = 0;     
            c1++;
            c3++;
            c5++;
          }
        }
        if (!i2_flg && !i3_flg) {
          for (j=0; j<k; j++) {             
            if ((*c1 >= l) && (*c1 <= u))
              *c5 = 255;
            else
              *c5 = 0;     
            c1++;
            c5++;
          }
        }
      }        
      else {
        c4  = (unsigned char *)(img4->imagedata);
        if (i2_flg && i3_flg) {
          c2 = (unsigned char *)(img2->imagedata);
          c3 = (unsigned char *)(img3->imagedata);
          for (j=0; j<k; j++) {  
            if (*c4)           
              if ((*c1 >= *c2) && (*c1 <= *c3))
                *c5 = 255;
              else
                *c5 = 0; 
            else
              *c5 = *c1;    
            c1++;
            c2++;
            c3++;
            c4++;
            c5++;
          }
        }
        if (i2_flg && !i3_flg) {
          c2 = (unsigned char *)(img2->imagedata);
          for (j=0; j<k; j++) {
            if (*c4)             
              if ((*c1 >= *c2) && (*c1 <= u))
                *c5 = 255;
              else
                *c5 = 0;
            else
              *c5 = *c1;     
            c1++;
            c2++;
            c4++;
            c5++;
          }
        }
        if (!i2_flg && i3_flg) {
          c3 = (unsigned char *)(img3->imagedata);
          for (j=0; j<k; j++) { 
            if (*c4)          
              if ((*c1 >= l) && (*c1 <= *c3))
                *c5 = 255;
              else
                *c5 = 0; 
            else
              *c5 = *c1;    
            c1++;
            c3++;
            c4++; 
            c5++;
          }
        }
        if (!i2_flg && !i3_flg) {
          for (j=0; j<k; j++) { 
            if (*c4)            
              if ((*c1 >= l) && (*c1 <= u))
                *c5 = 255;
              else
                *c5 = 0;
            else
              *c5 = *c1;    
            c1++;
            c4++;
            c5++;
          }
        }
      }        
      return(1);
      break;
 
    case VFF_TYP_2_BYTE :
      k =  img1->row_size * img1->col_size * img1->num_data_bands;
      s1 = (short *)(img1->imagedata);     
      c5 = (unsigned char *)(img5->imagedata);
      if ((!i2_flg && (l< -32768 || l>32767)) || (!i3_flg && (u< -32768 || u>32767))) {
        kfprintf(kstderr, "lvthreshad: integer parameter out of range for BYTE image\n");
        return(0);
      } 

      if (!mflg) {
        if (i2_flg && i3_flg) {
          s2 = (short *)(img2->imagedata);
          s3 = (short *)(img3->imagedata);
          for (j=0; j<k; j++) {             
            if ((*s1 >= *s2) && (*s1 <= *s3))
              *c5 = 255;
            else
              *c5 = 0;     
            s1++;
            s2++;
            s3++;
            c5++;
          }
        }
        if (i2_flg && !i3_flg) {
          s2 = (short *)(img2->imagedata);
          for (j=0; j<k; j++) {             
            if ((*s1 >= *s2) && (*s1 <= u))
              *c5 = 255;
            else
              *c5 = 0;     
            s1++;
            s2++;
            c5++;
          }
        }
        if (!i2_flg && i3_flg) {
          s3 = (short *)(img3->imagedata);
          for (j=0; j<k; j++) {             
            if ((*s1 >= l) && (*s1 <= *s3))
              *c5 = 255;
            else
              *c5 = 0;     
            s1++;
            s3++;
            c5++;
          }
        }
        if (!i2_flg && !i3_flg) {
          for (j=0; j<k; j++) {             
            if ((*s1 >= l) && (*s1 <= u))
              *c5 = 255;
            else
              *c5 = 0;     
            s1++;
            c5++;
          }
        }
      }        
      else {
        s4  = (short *)(img4->imagedata);
        if (i2_flg && i3_flg) {
          s2 = (short *)(img2->imagedata);
          s3 = (short *)(img3->imagedata);
          for (j=0; j<k; j++) {  
            if (*s4)           
              if ((*s1 >= *s2) && (*s1 <= *s3))
                *c5 = 255;
              else
                *c5 = 0; 
            else
              *c5 = (char) *s1;    
            s1++;
            s2++;
            s3++;
            s4++;
            c5++;
          }
        }
        if (i2_flg && !i3_flg) {
          s2 = (short *)(img2->imagedata);
          for (j=0; j<k; j++) {
            if (*s4)             
              if ((*s1 >= *s2) && (*s1 <= u))
                *c5 = 255;
              else
                *c5 = 0;
            else
              *c5 = (char) *s1;     
            s1++;
            s2++;
            s4++;
            c5++;
          }
        }
        if (!i2_flg && i3_flg) {
          s3 = (short *)(img3->imagedata);
          for (j=0; j<k; j++) { 
            if (*s4)          
              if ((*s1 >= l) && (*s1 <= *s3))
                *c5 = 255;
              else
                *c5 = 0; 
            else
              *c5 = (char) *s1;    
            s1++;
            s3++;
            s4++; 
            c5++;
          }
        }
        if (!i2_flg && !i3_flg) {
          for (j=0; j<k; j++) { 
            if (*s4)            
              if ((*s1 >= l) && (*s1 <= u))
                *c5 = 255;
              else
                *c5 = 0;
            else
              *c5 = (char) *s1;    
            s1++;
            s4++;
            c5++;
          }
        }
      }        
      return(1); 
      break;

    case VFF_TYP_4_BYTE :
      kfprintf(kstderr,"lvthreshad: BYTE4 not implemented\n");
      return(0);
      break;

    case VFF_TYP_FLOAT :
      kfprintf(kstderr,"lvthreshad: FLOAT not implemented\n");
      return(0);
      break;

    case VFF_TYP_COMPLEX :
      kfprintf(kstderr,"lvthreshad: COMPLEX not implemented\n");
      return(0);
      break;

    case VFF_TYP_DOUBLE :
      kfprintf(kstderr,"lvthreshad: DOUBLE not implemented\n");
      return(0);
      break;

    case VFF_TYP_DCOMPLEX :
      kfprintf(kstderr,"lvthreshad: COMPLEXD not implemented\n");
      return(0);
      break;

    default:
      kfprintf(kstderr,"lvthreshad: Unknown data storage type, %d\n",img1->data_storage_type);
      return(0);
      break;
  }
}
/* -library_code_end */
