 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

/*
 * Copyright (C) 1993-1997 Junior Barrera, Roberto Lotufo.
 * All rights reserved.  See $MMACH/repos/license/License.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vtoggle
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvtoggle
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvtoggle - Do the toggle between two images pixel by pixel with an operation gating mask.
* 
*       Purpose: .I vtoggle
will do the toggle between two images pixel by pixel.
Let p1, p2 and p3 be the corresponding pixels, respectively, in images
1, 2 and 3. The pixel p2 is chosen if |p1 - p2| < |p1 - p3|, otherwise
the pixel p3 is chosen. 
img1 is the input image; img2 and img3 are the toggle options; img4 is the gating image, and mflg should be set to 1 if the gating image is to be used. img5 is the output image. Note that img1 or img2 can be used for both the input and the resulting output xvimage structure, in order to save space.
A non-zero mask pixel in img4 enables the output pixel to contain 
the sup. A zero mask pixel in img4 just transfers the value 
of the corresponding pixel in img1 to the output pixel.
.LP
The input images must be of the same size, data type, and have 
the same number of data bands, as must the optional mask image.
*         Input: .IP "img1" 15  
first xvimage structure
.IP "img2" 15
second xvimage structure
.IP "img3" 15
third xvimage structure
.IP "img4" 15
gating mask image
.IP "mflg" 15
a flag set (equal 1) if gating image mask available
.IP
Note that all inputs must be of the same data type and size.
*        Output: .IP "img5" 15
holds the result of the toggle operation.  The output data
type is the same as the input data type.
.IP "Return Value:" 15
1 on success, 0 on failure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: .I lvtoggle
can be defined for all data types (except bit) supported by Khoros, but at the moment it has been implemented just for unsigned char type.
*    Written By: Junior Barrera
*          Date: May 12, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: Port to new toolbox version: Luiz Fernando Vieira, Adriano Nagelschmidt Rodrigues
****************************************************************/
/* -library_def */
int
lvtoggle(xvimage *img1, xvimage *img2, xvimage *img3, xvimage *img4, int mflg, xvimage *img5)
/* -library_def_end */

/* -library_code */
{
  int    j, k;
  unsigned char *c1, *c2, *c3, *c4, *c5;

  switch (img1->data_storage_type) {
    case VFF_TYP_BIT :
      kfprintf(kstderr,"lvtoggle: BIT not defined\n");
      return(0);
      break;

    case VFF_TYP_1_BYTE : 
      k = img1->row_size * img1->col_size * img1->num_data_bands;
      c1 = (unsigned char *)(img1->imagedata);
      c2 = (unsigned char *)(img2->imagedata);
      c3 = (unsigned char *)(img3->imagedata);
      c5 = (unsigned char *)(img5->imagedata);

      if (!mflg)            
        for (j=0; j<k; j++) {
          if (abs(*c1 - *c2) <= abs(*c1 - *c3)) 
            *c5 = *c2;
          else 
            *c5 = *c3;
          c1++;
          c2++;
          c3++;
          c5++;
        }              
      else {
        c4 = (unsigned char *)(img4->imagedata);
        for (j=0; j<k; j++)
          if (*c4)
            if (abs(*c1 - *c2) <= abs(*c1 - *c3))
              *c5 = *c2;
            else 
              *c5 = *c3;
          else
            *c5 = *c1;
          c1++;
          c2++;
          c3++;
          c4++;
          c5++;
      }
      return(1);
      break;
 
    case VFF_TYP_2_BYTE :
      kfprintf(kstderr,"lvtoggle: BYTE2 not implemented\n");
      return(0);
      break;

    case VFF_TYP_4_BYTE :
      kfprintf(kstderr,"lvtoggle: BYTE4 not implemented\n");
      return(0);
      break;

    case VFF_TYP_FLOAT :
      kfprintf(kstderr,"lvtoggle: FLOAT not implemented\n");
      return(0);
      break;

    case VFF_TYP_COMPLEX :
      kfprintf(kstderr,"lvtoggle: COMPLEX not implemented\n");
      return(0);
      break;

    case VFF_TYP_DOUBLE :
      kfprintf(kstderr,"lvtoggle: DOUBLE not implemented\n");
      return(0);
      break;

    case VFF_TYP_DCOMPLEX :
      kfprintf(kstderr,"lvtoggle: COMPLEXD not implemented\n");
      return(0);
      break;

    default:
      kfprintf(kstderr,"lvtoggle: Unknown data storage type, %d\n",img1->data_storage_type);
      return(0);
      break;
    }
}
/* -library_code_end */
