/*
 *--------------------------------------------------------------------------
 * COPYRIGHT
 *
 * A Mathematical Morphology Toolbox for the Khoros System 
 *
 * Copyright 1993 - University of Sao Paulo - USP
 *                  University of Campinas - UNICAMP
 *                  National Institute for Space Research - INPE
 *
 * All rights reserved.
 *
 * Due to potential patent problems, this program may only be used
 * for research in morphological image processing and practical classes
 * in University courses. The authors do not take any resposibilities
 * for use beyond this scope, nor do we claim any suitability and  
 * operability of this software for any purpose. It is provided "as is"
 * without express or implied warranty.
 *
 * I DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL I BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including, for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *--------------------------------------------------------------------------
 */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: queue.c
 >>>>
 >>>> Date Last Updated: March 20 1994 
 >>>>
 >>>>          Routines: queue_init, queue_empty, queue_add & queue_first
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/

#include "internals.h"
/*
#define NULL ((cell_str *) 0)
*/


/****************************************************************
*
* Routine Name: queue_init
*
*         Date: March 11 1994
*
*      Purpose: Initialize queue (create first cell)  
*
*        Input: pointer to queue structure.
*          
*       Output: none. 
*
*   Written By: Adriano Nagelschmidt Rodrigues
*        
****************************************************************/

void queue_init(queue_str *qq)
{
  qq->first = (cell_str *) malloc(sizeof(cell_str));
  qq->last = qq->first;
  qq->first->next = NULL;
}


/****************************************************************
*
* Routine Name: queue_empty
*
*         Date: March 11 1994
*
*      Purpose: Verifies if queue is empty  
*
*        Input: pointer to queue structure.
*          
*       Output: none.
*
* Return value: 1 if queue is empty, 0 otherwise.    
*
*   Written By: Adriano Nagelschmidt Rodrigues
*        
****************************************************************/

int queue_empty(queue_str *qq)
{
  if (qq->first == qq->last)
    return(1);
  else
    return(0);
}


/****************************************************************
*
* Routine Name: queue_add
*
*         Date: March 11 1994
*
*      Purpose: Adds item to queue  
*
*        Input: pointer to queue structure, item structure.
*          
*       Output: none    
*
*   Written By: Adriano Nagelschmidt Rodrigues
*        
****************************************************************/

void queue_add(queue_str *qq, item_str *item_a)
{
  qq->last->next = (cell_str *) malloc(sizeof(cell_str));
  qq->last = qq->last->next;
  qq->last->item = *item_a;
  qq->last->next = NULL;
}


/****************************************************************
*
* Routine Name: queue_first
*
*         Date: March 11 1994
*
*      Purpose: Remove queue first item.  
*
*        Input: pointer to queue structure.
*          
*       Output: removed item structure.
*
* Return value: 1 upon sucess, 0 if queue is empty.    
*
*   Written By: Adriano Nagelschmidt Rodrigues
*        
****************************************************************/

int queue_first(queue_str *qq, item_str *item_r)
{
  cell_str *cell;

  if (qq->first == qq->last)
    return(0);
  cell = qq->first;
  qq->first = qq->first->next;
  *item_r = qq->first->item;
  free(cell);
}
