 /*
  * Khoros: $Id: vbandsp3.c,v 1.14.2.2 1997/05/31 20:04:25 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vbandsp3.c,v 1.14.2.2 1997/05/31 20:04:25 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vbandsp3
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vbandsp3.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Extract 3 Bands from a Multiband Image  (K1)
|
|       Purpose: main program for vbandsp3
|
|         Input:
|		char *clui_info->i_file; {input image filename}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		int clui_info->b_int; {Data band number to extract}
|		int clui_info->b_flag; {TRUE if -b specified}
|
|		char *clui_info->o1_file; {output image filename}
|		int   clui_info->o1_flag; {TRUE if -o1 specified}
|
|		char *clui_info->o2_file; {output image filename}
|		int   clui_info->o2_flag; {TRUE if -o2 specified}
|
|		char *clui_info->o3_file; {output image filename}
|		int   clui_info->o3_flag; {TRUE if -o3 specified}
|
|        Output:
|       Returns:
|
|    Written By: Tom Sauer
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = NULL;
xvimage *image, **images_list, *img1, *img2, *img3;

char    *img1_file,
        *img2_file,
        *img3_file;
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vbandsp3");
	kexit_handler(vbandsp3_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vbandsp3", KGEN_KROUTINE,  &clui_uis_spec,
		   vbandsp3_usage_additions, vbandsp3_get_args,
 		   vbandsp3_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
       /* read in input image */
image = readimage(clui_info->i_file);
if (image == NULL) kexit(KEXIT_FAILURE);                       /* Quit if bad image */

       /* can only have one image in the input file */
(void) proper_num_images(program,image,(unsigned long) 1,TRUE);

       /* Make sure that there are enough data bands available in the
          input image. if -b option + 3 must be greater than
          the total number of data bands */

if ( (clui_info->b_int + 3)  > image->num_data_bands) {
   (void) kfprintf(kstderr, "%s: Cannot extract out 3 data bands starting with data band %d,", program,clui_info->b_int);
   (void) kfprintf(kstderr,"\ninput image only contains %d data bands\n", image->num_data_bands );
    kexit(KEXIT_FAILURE);
}
      /* if the output files were not specified, then use
         the input filename, otherwise just copy the
         output file name to the new variable */

if (clui_info->o1_flag)
    img1_file = clui_info->o1_file;
else
    img1_file = kbasename(clui_info->i_file, NULL);
if (clui_info->o2_flag)
    img2_file = clui_info->o2_file;
else
    img2_file = kbasename(clui_info->i_file, NULL);
if (clui_info->o3_flag)
    img3_file = clui_info->o3_file;
else
    img3_file = kbasename(clui_info->i_file, NULL);

      /* if the output files are not specified, than take
         the input file and append an appropriate suffix
         to it, and use that as the output file name */

APPENDFILENAME(image, clui_info);

      /* create a list of pointers to xvimage structures, the
         number in the list will be the number of data bands */

images_list = create_images_list(image->num_data_bands);
if ( images_list == NULL)
{
   (void) kfprintf(kstderr, "create_images_list Failed\n");
   kexit(KEXIT_FAILURE);
}
/* -main_before_lib_call_end */

/* -main_library_call */
/* seprate out all bands into seprate images */

if(! lvbandsp1(image, images_list, clui_info->i_file, TRUE))
{
    (void) kfprintf(kstderr, "lvbandspt1 Failed\n");
    kexit(KEXIT_FAILURE);
}

      /* since we can only extract 3 bands, extract 3 bands starting
         with the band specified by the -b option */

img1 = images_list[(clui_info->b_int)];
img2 = images_list[clui_info->b_int+1];
img3 = images_list[clui_info->b_int+2];
/* -main_library_call_end */

/* -main_after_lib_call */
/* write out the 3 images */

writeimage(img1_file, img1);
writeimage(img2_file, img2);
writeimage(img3_file, img3);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vbandsp3_usage_additions
| 
|       Purpose: Prints usage additions in vbandsp3_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vbandsp3
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vbandsp3_usage_additions(void)
{
	kfprintf(kstderr, "\tExtract 3 Bands from a Multiband Image  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vbandsp3_free_args
| 
|       Purpose: Frees CLUI struct allocated in vbandsp3_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vbandsp3
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vbandsp3_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o1_file);
		kfree_and_NULL(clui_info->o2_file);
		kfree_and_NULL(clui_info->o3_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
