 /*
  * Khoros: $Id: vclose.c,v 1.14.2.4 1997/05/31 20:04:55 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vclose.c,v 1.14.2.4 1997/05/31 20:04:55 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vclose
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vclose.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Close Boundaries of an Edge Image  (K1)
|
|       Purpose: main program for vclose
|
|         Input:
|		char *clui_info->i1_file; {Input Boundary Image (result from vdrf, vgef, vsdef ...)}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {Input Gradient Image (result from vdrf, vsdef, sobel...)}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->o_file; {resulting output image or file}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->grad_int; {Gradient Threshold (used by the closing process)}
|		int clui_info->grad_flag; {TRUE if -grad specified}
|
|		int clui_info->len_int; {Minimum Gap Size In Pixels}
|		int clui_info->len_flag; {TRUE if -len specified}
|
|        Output:
|       Returns:
|
|    Written By: Pascal ADAM
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = "vclose";
xvimage *edge, *gradient;
int length, thresh;
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vclose");
	kexit_handler(vclose_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vclose", KGEN_KROUTINE,  &clui_uis_spec,
		   vclose_usage_additions, vclose_get_args,
 		   vclose_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    READINPUT (edge,gradient);
    CHECKINPUT (program,edge,gradient);

    if (edge -> row_size != gradient -> row_size ||
        edge -> col_size != gradient -> col_size )
    {
       (void) kfprintf(kstderr,"vclose: Input Images Are Not The Same Size\n\n");
       kexit(KEXIT_FAILURE);
    }

   
    length = clui_info->len_int;
    thresh = clui_info->grad_int;
/* -main_before_lib_call_end */

/* -main_library_call */
if(! lvclose((unsigned char *)edge->imagedata,
	     (unsigned char *)gradient->imagedata, 
	     (int)edge->row_size,
             (int)edge->col_size, length, thresh))
    {
        (void) kfprintf(kstderr, "vclose: lvclose Failed\n");
        kexit(KEXIT_FAILURE);
    }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,edge);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vclose_usage_additions
| 
|       Purpose: Prints usage additions in vclose_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vclose
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vclose_usage_additions(void)
{
	kfprintf(kstderr, "\tClose Boundaries of an Edge Image  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vclose_free_args
| 
|       Purpose: Frees CLUI struct allocated in vclose_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vclose
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vclose_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
