 /*
  * Khoros: $Id: vconvert.c,v 1.14.2.3 1997/05/31 20:05:54 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vconvert.c,v 1.14.2.3 1997/05/31 20:05:54 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vconvert
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vconvert.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Convert Data Storage Type  (K1)
|
|       Purpose: main program for vconvert
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->t_toggle; {lets user choose output data type}
|		int   clui_info->t_flag; {TRUE if -t specified}
|
|		int clui_info->b_logic; {if true, bands are processed independently}
|		int clui_info->b_flag; {TRUE if -b specified}
|
|		Optional M.E. group:
|		    float clui_info->n_float; {specifies normalization value}
|		    int   clui_info->n_flag; {TRUE if -n specified}
|
|		    float clui_info->s_float; {specifies scale factor}
|		    int   clui_info->s_flag; {TRUE if -s specified}
|
|        Output:
|       Returns:
|
|    Written By: Scott Wilson
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	char *program = "vshape";
	xvimage *image;
	int type;
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vconvert");
	kexit_handler(vconvert_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vconvert", KGEN_KROUTINE,  &clui_uis_spec,
		   vconvert_usage_additions, vconvert_get_args,
 		   vconvert_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        if (!vget_type(&type,clui_info->t_toggle))
          {
            (void)kfprintf(kstderr,"vconvert: unknown data type!\n");
           kexit(KEXIT_FAILURE);
          }
        image = readimage(clui_info->i_file);
        if (image == NULL) kexit(KEXIT_FAILURE);        /* Quit if error reading the image */

        CHECKINPUT(program,image);  /* Verify input image */
/* -main_before_lib_call_end */

/* -main_library_call */
if(! lvconvert(image,type,clui_info->n_flag,clui_info->s_flag,
                             clui_info->n_float,clui_info->s_float,
                             clui_info->b_logic) )
          {
            (void) kfprintf(kstderr, "lvconvert Failed\n");
            kexit(KEXIT_FAILURE);
          }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vconvert_usage_additions
| 
|       Purpose: Prints usage additions in vconvert_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vconvert
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vconvert_usage_additions(void)
{
	kfprintf(kstderr, "\tConvert Data Storage Type  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vconvert_free_args
| 
|       Purpose: Frees CLUI struct allocated in vconvert_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vconvert
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vconvert_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->t_toggle);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
