 /*
  * Khoros: $Id: vdetilt.c,v 1.14.2.3 1997/05/31 20:06:32 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vdetilt.c,v 1.14.2.3 1997/05/31 20:06:32 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vdetilt
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vdetilt.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Remove Illumination Gradient by Subtracting Best-Fit Plane  (K1)
|
|       Purpose: main program for vdetilt
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {resulting detilted image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->m_file; {find tilt  operation masking image}
|		int   clui_info->m_flag; {TRUE if -m specified}
|
|		int clui_info->p_logic; {if yes, remove mean from image}
|		int clui_info->p_flag; {TRUE if -p specified}
|
|        Output:
|       Returns:
|
|    Written By: Scott Wilson
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	char    *program = "vdetilt";
	xvimage *img1, *img2;
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vdetilt");
	kexit_handler(vdetilt_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vdetilt", KGEN_KROUTINE,  &clui_uis_spec,
		   vdetilt_usage_additions, vdetilt_get_args,
 		   vdetilt_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	READINPUT1(img1);
	CHECKINPUT(program,img1);

	if (clui_info->m_flag) 
	{
	  READINPUT2(img2);
	  MATCHINPUT(program,img1,img2);
	}
/* -main_before_lib_call_end */

/* -main_library_call */
	if(! lvdetilt(img1,img2,clui_info->m_flag,clui_info->p_logic) )
	{
	  kexit(KEXIT_FAILURE);
	}
/* -main_library_call_end */

/* -main_after_lib_call */
	writeimage(clui_info->o_file,img1);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vdetilt_usage_additions
| 
|       Purpose: Prints usage additions in vdetilt_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vdetilt
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vdetilt_usage_additions(void)
{
	kfprintf(kstderr, "\tRemove Illumination Gradient by Subtracting Best-Fit Plane\n");
	kfprintf(kstderr, "\t(K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vdetilt_free_args
| 
|       Purpose: Frees CLUI struct allocated in vdetilt_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vdetilt
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vdetilt_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->m_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
