 /*
  * Khoros: $Id: vfht.c,v 1.14.2.4 1997/05/31 20:07:29 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vfht.c,v 1.14.2.4 1997/05/31 20:07:29 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vfht
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vfht.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Compute 2-D Fast Hadamard Transform  (K1)
|
|       Purpose: main program for vfht
|
|         Input:
|		char *clui_info->i_file; {input image (viff file)}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output image (viff file)}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->s_logic; {scale output by 1/N (0 = no scale, 1 = scale)}
|		int clui_info->s_flag; {TRUE if -s specified}
|
|        Output:
|       Returns:
|
|    Written By: Per Lysne
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = "vfht";
xvimage *img;
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vfht");
	kexit_handler(vfht_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vfht", KGEN_KROUTINE,  &clui_uis_spec,
		   vfht_usage_additions, vfht_get_args,
 		   vfht_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    img = readimage (clui_info->i_file);
    if (img==NULL) {
        kfprintf (kstderr,"error in vfht: could not read %s: ",clui_info->i_file);
        kfprintf (kstderr,"exiting with an error\n");
        kexit(KEXIT_FAILURE);
    }

    CHECKINPUT (program, img);

    if (!kpowtwo((int)img->row_size)) {
        kfprintf (kstderr, "error in vfht: row length of the input image must");
        kfprintf (kstderr, " be a power of two: exiting with an error\n");
        kexit(KEXIT_FAILURE);
    }
    if (!kpowtwo((int)img->col_size)) {
        kfprintf (kstderr, "error in vfht: col length of the input image must");
        kfprintf (kstderr, " be a power of two: exiting with an error\n");
        kexit(KEXIT_FAILURE);
    }

    if (img->data_storage_type != VFF_TYP_FLOAT) {
        if (!lvconvert (img, VFF_TYP_FLOAT, 0, 1, 0.0, 1.0, 0)) {
            kfprintf (kstderr, "error in vfht: call to lvconvert failed\n");
            kexit (1);
        }
    }
/* -main_before_lib_call_end */

/* -main_library_call */
if (lvfht(img, clui_info->s_logic)!=1) {
        kfprintf (kstderr, "error in vfht: library call to lvfht failed: ");
        kfprintf (kstderr, "exiting with an error\n");
        kexit (1);
    }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage (clui_info->o_file, img);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vfht_usage_additions
| 
|       Purpose: Prints usage additions in vfht_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vfht
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vfht_usage_additions(void)
{
	kfprintf(kstderr, "\tCompute 2-D Fast Hadamard Transform  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vfht_free_args
| 
|       Purpose: Frees CLUI struct allocated in vfht_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vfht
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vfht_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
