 /*
  * Khoros: $Id: vgfractal.c,v 1.14.2.3 1997/05/31 20:08:57 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vgfractal.c,v 1.14.2.3 1997/05/31 20:08:57 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vgfractal
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vgfractal.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Create Fractal Image with Specified Fractal Dimension  (K1)
|
|       Purpose: main program for vgfractal
|
|         Input:
|		char *clui_info->o_file; {Output image filename}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->trigger_file; {trigger input}
|		int   clui_info->trigger_flag; {TRUE if -trigger specified}
|
|		int clui_info->r_int; {Number of recursions}
|		int clui_info->r_flag; {TRUE if -r specified}
|
|		float clui_info->h_float; {Hurst coefficient}
|		int   clui_info->h_flag; {TRUE if -h specified}
|
|		float clui_info->d_float; {Standard deviation}
|		int   clui_info->d_flag; {TRUE if -d specified}
|
|		int clui_info->s_int; {Seed}
|		int clui_info->s_flag; {TRUE if -s specified}
|
|		int clui_info->c_logic; {Generate coastline}
|		int clui_info->c_flag; {TRUE if -c specified}
|
|		int clui_info->m_logic; {Add random elements to each point}
|		int clui_info->m_flag; {TRUE if -m specified}
|
|        Output:
|       Returns:
|
|    Written By: Charlie Gage
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
xvimage *image;
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vgfractal");
	kexit_handler(vgfractal_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vgfractal", KGEN_KROUTINE,  &clui_uis_spec,
		   vgfractal_usage_additions, vgfractal_get_args,
 		   vgfractal_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
/* -main_before_lib_call_end */

/* -main_library_call */
/* Call the lib routine for generation of fractal image */

if(! lvgfractal(&image, clui_info->r_int, clui_info->h_float, clui_info->d_float, clui_info->s_int, clui_info->c_logic, clui_info->m_logic))
 {
   (void) kfprintf(kstderr, "lvgfractal Failed\\n");
   kexit(KEXIT_FAILURE);
 }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vgfractal_usage_additions
| 
|       Purpose: Prints usage additions in vgfractal_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgfractal
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vgfractal_usage_additions(void)
{
	kfprintf(kstderr, "\tCreate Fractal Image with Specified Fractal Dimension  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vgfractal_free_args
| 
|       Purpose: Frees CLUI struct allocated in vgfractal_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgfractal
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vgfractal_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->trigger_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
