 /*
  * Khoros: $Id: vhmed.c,v 1.14.2.3 1997/05/31 20:09:33 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vhmed.c,v 1.14.2.3 1997/05/31 20:09:33 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vhmed
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vhmed.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Median Filter the Image Using Histogram to Find Median  (K1)
|
|       Purpose: main program for vhmed
|
|         Input:
|		char *clui_info->i_file; {input image filename}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output image filename}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->w_int; {window width argument}
|		int clui_info->w_flag; {TRUE if -w specified}
|
|		int clui_info->h_int; {window height argument}
|		int clui_info->h_flag; {TRUE if -h specified}
|
|        Output:
|       Returns:
|
|    Written By: Marcelo Teran, Ramiro Jordan
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = "vhmed";
xvimage *image;
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vhmed");
	kexit_handler(vhmed_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vhmed", KGEN_KROUTINE,  &clui_uis_spec,
		   vhmed_usage_additions, vhmed_get_args,
 		   vhmed_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    image = readimage(clui_info->i_file);
    if (image == NULL){
        (void)kerror(NULL, NULL, "Image could not be read\n");
         kexit(1);               /* Quit if bad image */
    }

    CHECKINPUT(program,image);  /* Verify input image */
/* -main_before_lib_call_end */

/* -main_library_call */
if(! lvhmed(image,clui_info->w_int,clui_info->h_int)){
        (void) kerror(NULL, NULL, "lvhmed Failed\n");
        kexit(1);
    }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vhmed_usage_additions
| 
|       Purpose: Prints usage additions in vhmed_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vhmed
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vhmed_usage_additions(void)
{
	kfprintf(kstderr, "\tMedian Filter the Image Using Histogram to Find Median  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vhmed_free_args
| 
|       Purpose: Frees CLUI struct allocated in vhmed_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vhmed
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vhmed_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
