 /*
  * Khoros: $Id: vlabel.c,v 1.14.2.3 1997/05/31 20:10:00 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vlabel.c,v 1.14.2.3 1997/05/31 20:10:00 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vlabel
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vlabel.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Perform Labeling on Multiband or Cluster Image  (K1)
|
|       Purpose: main program for vlabel
|
|         Input:
|		char *clui_info->o_file; {output image after the labeling process}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		Required M.E. group:
|		    char *clui_info->i1_file; {input image}
|		    int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		    M.I. group:
|			char *clui_info->i2_file; {input image for cluster centers}
|			int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|			char *clui_info->i3_file; {input image for cluster numbers}
|			int   clui_info->i3_flag; {TRUE if -i3 specified}
|
|		Required M.E. group:
|		    int clui_info->n_int; {Number of region to get from labeling process}
|		    int clui_info->n_flag; {TRUE if -n specified}
|
|		    float clui_info->f_float; {Split & Merge Factor: higher values result in finer regions}
|		    int   clui_info->f_flag; {TRUE if -f specified}
|
|		int clui_info->w_int; {Specify the border width in pixels}
|		int clui_info->w_flag; {TRUE if -w specified}
|
|		int clui_info->m_toggle; {Distance Metric: 1 = Euclidean distance, 2 = city block}
|		int clui_info->m_flag; {TRUE if -m specified}
|
|		int clui_info->c_toggle; {Connectivity: 1 = 4-connectivity, 2 = 8-connectivity}
|		int clui_info->c_flag; {TRUE if -c specified}
|
|		float clui_info->s_float; {Minimum number of pixel to keep in label region}
|		int   clui_info->s_flag; {TRUE if -s specified}
|
|		int clui_info->merge_logic; {if flag= 1 the small regions will be merged together in connected regions, or will be included to existing labeled regions}
|		int clui_info->merge_flag; {TRUE if -merge specified}
|
|		char *clui_info->asc_file; {Output ASCII file for vlabel result display}
|		int   clui_info->asc_flag; {TRUE if -asc specified}
|
|        Output:
|       Returns:
|
|    Written By: Pascal ADAM
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	char *program = NULL;
	xvimage *image, 
		*clus_numb,
		*clus_cent,
		*label;

	int dist,
	    nc,
	    nr,
	    connect,
	    cas,
	    opt_auto,
	    merge,
	    region_number;

	kfile *printdev;
	char *dummy = '\0';
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vlabel");
	kexit_handler(vlabel_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vlabel", KGEN_KROUTINE,  &clui_uis_spec,
		   vlabel_usage_additions, vlabel_get_args,
 		   vlabel_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
if (clui_info->i1_flag == TRUE)  {
   image = readimage(clui_info->i1_file);
   if (image == NULL) {
      kerror("vlabel", "main", "image could not be read\n\n");
      kexit(KEXIT_FAILURE);
      }
   (void) proper_num_images(program,image,1,TRUE);
   (void) proper_map_enable(program,image,VFF_MAP_OPTIONAL,TRUE);

   if (lvconvert(image,VFF_TYP_FLOAT,0,1,0.0,1.0,FALSE) == 0 ) {
      kerror("vlabel", "main", "lvconvert failure\n\n");
      kexit(KEXIT_FAILURE);
      }
   nc = image->row_size;
   nr = image->col_size;

   if ( (clui_info->w_int > (nc/2)) ||
        (clui_info->w_int > (nr/2))   ) {
      kerror("vlabel", "main", "border too large for the image size\n\n");
      kexit(KEXIT_FAILURE);
      }
   }
if (clui_info->i2_flag == TRUE)  {
   clus_cent = readimage(clui_info->i2_file);
   if (clus_cent == NULL) {
      kerror("vlabel", "main", "cluster center image could not be read\n\n");
      kexit(KEXIT_FAILURE);
      }
   (void) proper_num_images(program,clus_cent,1,TRUE);
   (void) proper_map_enable(program,clus_cent,VFF_MAP_OPTIONAL,TRUE);

   if (clus_cent ->data_storage_type != VFF_TYP_4_BYTE) {
      kerror("vlabel", "main",
	     "cluster center image has to be a FLOAT image\n\n");
      kexit(KEXIT_FAILURE);
      }

   }

if (clui_info->i3_flag == TRUE)  {
   clus_numb = readimage(clui_info->i3_file);
   if (clus_numb == NULL) {
      kerror("vlabel", "main", "cluster number image could not be read\n\n");
      kexit(KEXIT_FAILURE);
      }
   (void) proper_num_images(program,clus_numb,1,TRUE);
   (void) proper_map_enable(program,clus_numb,VFF_MAP_OPTIONAL,TRUE);

   if (clus_numb ->data_storage_type != VFF_TYP_4_BYTE) {
      kerror("vlabel","main", "cluster number image has to be a INT image\n\n");
      kexit(KEXIT_FAILURE);
      }


   nc = clus_numb->row_size;
   nr = clus_numb->col_size;

   if ((clui_info->w_int > (nc/2))  ||
       (clui_info->w_int > (nr/2))   )  {
      kerror("vlabel", "main", "border too large for clus_numb size \n\n");
      kexit(KEXIT_FAILURE);
      }
   }


if ((clui_info->i1_flag == TRUE) && (clui_info->i3_flag == TRUE)) {
   if (   (clus_numb->row_size != image->row_size) ||
          (clus_numb->col_size != image->col_size) ) {
      kerror("vlabel", "main",
	     "cluster and normal image must have the same dimension\n\n");
      kexit(KEXIT_FAILURE);
      }
   }


if ((clui_info->i2_flag == TRUE) && (clui_info->i3_flag == FALSE)) {
   kerror("vlabel", "main", "cluster number image is missing\n\n");
   kexit(KEXIT_FAILURE);
   }
if ((clui_info->i1_flag==TRUE)&&(clui_info->i2_flag==TRUE)&&(clui_info->i3_flag==TRUE)) {
   kerror("vlabel", "main", "unable to process labelling with 3 inputs\n\n");
   kexit(KEXIT_FAILURE);
   }

/* set the distance metrics */
if (clui_info->m_flag == TRUE) dist = clui_info->m_toggle;
else dist = 1;

/* set the connectivity, 4 connectivity, or, 8 connectivity */
if (clui_info->c_flag == TRUE) connect = clui_info->c_toggle;
else connect = 1;


/* check from argument which case has to be treated */
if ((clui_info->i1_flag == TRUE)   &&
    (clui_info->i2_flag == FALSE)  &&
    (clui_info->i3_flag == FALSE)  )   {
    cas = 0;       /* labelling from a multiband image */
    }
 else if ((clui_info->i1_flag  == FALSE) &&
          (clui_info->i2_flag  == TRUE)  &&
          (clui_info->i3_flag  == TRUE)  )     {
         cas = 1;  /* labelling from a clustering result */
         }
      else if ((clui_info->i1_flag == TRUE)   &&
               (clui_info->i2_flag == FALSE)  &&
               (clui_info->i3_flag == TRUE))    {
              cas = 2;/*labelling from clustering helped by multiband image*/
              }

/* Allocate space for output image (labelled image) */
label = createimage((unsigned long)nr,(unsigned long)nc, VFF_TYP_4_BYTE, 1,
                     1,dummy, 0, 0, VFF_MS_NONE,
                     VFF_MAPTYP_NONE, VFF_LOC_IMPLICIT, 0);

if (label == NULL) {
   kerror("vlabel", "main", "Unable to allocate new image!\\n");
   kexit(KEXIT_FAILURE);
   }

/* check if automatic labelling required ( expected number of region) */
if (clui_info->n_flag == TRUE) {
   opt_auto = TRUE;
   region_number = clui_info->n_int;
   }
else opt_auto = FALSE;

/* check if merging option selected or not ( case where too small rejion */
/* are coinsidered UNDEFINED or are included in bigger ones).............*/
if (clui_info->merge_flag == FALSE) merge = TRUE;
else 
{
   if (clui_info->merge_logic == 1) merge = TRUE;
   else merge = FALSE;
}

/* ASCII file for labelling results display */
if (clui_info->asc_flag == TRUE) 
{
   if (clui_info->asc_file)
   {
      if (!(printdev = kfoutput (clui_info->asc_file))) 
      {
            kerror ("vlabel", "main", "Can't open ascii output file: %s\n",
		    clui_info->asc_file);
            kexit (KEXIT_FAILURE);
      }
   }
   else printdev = kstdout;
}
else printdev = kstdout;
/* -main_before_lib_call_end */

/* -main_library_call */
if (! lvlabel(image,
              clus_cent,
              clus_numb,
              cas,
              dist,
              connect,
              clui_info->w_int,
              label,
              clui_info->s_float,
              opt_auto,
              region_number,
              clui_info->f_float,
              merge,
              printdev))   {
       kexit(KEXIT_FAILURE);
   }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,label);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vlabel_usage_additions
| 
|       Purpose: Prints usage additions in vlabel_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vlabel
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vlabel_usage_additions(void)
{
	kfprintf(kstderr, "\tPerform Labeling on Multiband or Cluster Image  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vlabel_free_args
| 
|       Purpose: Frees CLUI struct allocated in vlabel_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vlabel
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vlabel_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->asc_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
