 /*
  * Khoros: $Id: vmindis.c,v 1.14.2.3 1997/05/31 20:10:39 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vmindis.c,v 1.14.2.3 1997/05/31 20:10:39 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vmindis
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vmindis.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Minimum Distance Classifier  (K1)
|
|       Purpose: main program for vmindis
|
|         Input:
|		char *clui_info->i1_file; {input image to be classified}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {input center/class image}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->o_file; {output specifying which vector belongs to which cluster}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->b_int; { specifies the border width in pixels (default = 0)}
|		int clui_info->b_flag; {TRUE if -b specified}
|
|        Output:
|       Returns:
|
|    Written By: Tom Sauer
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	char *program = "vmindis";
	xvimage *image, *center;
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vmindis");
	kexit_handler(vmindis_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vmindis", KGEN_KROUTINE,  &clui_uis_spec,
		   vmindis_usage_additions, vmindis_get_args,
 		   vmindis_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
/* -main_before_lib_call_end */

/* -main_library_call */
	READINPUT1 (image);
	READINPUT2 (image);
	CHECKINPUT (program, image);

	CHECKINPUT(program, center);
	if ( image -> num_data_bands != (center->num_data_bands - 1)) {
	   kerror("vmindis", "main",
		  "The center/class image does not contain the correct\n\
number of data_bands\n\
Should have %d bands, but contains %d bands\n",
		  (image->num_data_bands + 1), center->num_data_bands);
	   kexit(KEXIT_FAILURE);
	   }

  /* Call the lib routine for minimum distance algorithm */

	if(! lvmindis(image, center, clui_info->b_int))
  	{
	    kexit(KEXIT_FAILURE);
  	}
/* -main_library_call_end */

/* -main_after_lib_call */
	writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vmindis_usage_additions
| 
|       Purpose: Prints usage additions in vmindis_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vmindis
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vmindis_usage_additions(void)
{
	kfprintf(kstderr, "\tMinimum Distance Classifier  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vmindis_free_args
| 
|       Purpose: Frees CLUI struct allocated in vmindis_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vmindis
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vmindis_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
