 /*
  * Khoros: $Id: vpml.c,v 1.14.2.3 1997/05/31 20:11:07 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vpml.c,v 1.14.2.3 1997/05/31 20:11:07 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vpml
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vpml.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Estimate Fractal Dimension of Image Based on P(m,L)  (K1)
|
|       Purpose: main program for vpml
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o1_file; {resulting multiband fractal dimension output image}
|		int   clui_info->o1_flag; {TRUE if -o1 specified}
|
|		char *clui_info->o2_file; {output image specifying fractal dimension of each class}
|		int   clui_info->o2_flag; {TRUE if -o2 specified}
|
|		int clui_info->l_int; {select the initial size of the sliding window}
|		int clui_info->l_flag; {TRUE if -l specified}
|
|		int clui_info->u_int; {select the final size of the sliding window}
|		int clui_info->u_flag; {TRUE if -u specified}
|
|		int clui_info->s_int; {select the window step size or interval}
|		int clui_info->s_flag; {TRUE if -s specified}
|
|		int clui_info->q_int; {select the range of moments (-r/2 <= 0 >= r/2)}
|		int clui_info->q_flag; {TRUE if -q specified}
|
|		char *clui_info->f1_file; {output file for P(m,L) statistics}
|		int   clui_info->f1_flag; {TRUE if -f1 specified}
|
|        Output:
|       Returns:
|
|    Written By: Charlie Gage
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	char    *program = "vpml";
	char    *dummy = '\0';
	int     i, j, x, num_classes;
	int     *ptr;
	kfile   *printdev;
	xvimage *image, *image1, *fd_image;
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vpml");
	kexit_handler(vpml_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vpml", KGEN_KROUTINE,  &clui_uis_spec,
		   vpml_usage_additions, vpml_get_args,
 		   vpml_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	READINPUT(image) ;
	CHECKINPUT(program, image) ;

/* Check for proper input image type and convert if necessary */
/* vpml will only work with a BYTE or INTEGER type image. */
	switch (image->data_storage_type)
	 {
	  case VFF_TYP_1_BYTE:            /* convert char and short to integer */
	  case VFF_TYP_2_BYTE:
	     if (lvconvert(image, 4, 0, 1, 0.0, 1.0, FALSE) == 0)
	      {
	       kexit(KEXIT_FAILURE);
	      }
	     break;
	  case VFF_TYP_4_BYTE:           /* data type integer */
	     break;
	  default:
	     kerror("vpml", "main",
		   "ERROR, input image must be of data type BYTE or INTEGER\n");
	     kexit(KEXIT_FAILURE);
	 }

/* Determine row size of fractal dimension image,
                     depends on number of classes. */
	ptr = (int *)image->imagedata;
	num_classes = 0;
	for (i = 0; i < image->col_size; i++)
	 {
	   for (j = 0; j < image->row_size; j++)
	    {
	      x = ptr[image->col_size * j + i];
	      if (x > num_classes)
	         num_classes = x;
	    }
	 }
	 num_classes++;      /* this is the number of classes in input image */

/* Open file for P(m,L) statistics */

	if (clui_info->f1_flag) {
	   if ((printdev = kfoutput(clui_info->f1_file)) == NULL)
	   {
	      kerror("vpml", "main", "Can't open ascii output file: %s\n",
		     clui_info->f1_file);
	      kexit(KEXIT_FAILURE);
	   }
	}
	else
	  printdev = kstdout;

/* Allocate space for multiband output image (1 band for each moment) */
	image1 = createimage(image->col_size, image->row_size, 
			     (unsigned long) VFF_TYP_FLOAT,
			     (unsigned long) 1, 
			     (unsigned long) clui_info->q_int, dummy, 
			     (unsigned long) 0, 
			     (unsigned long) 0, 
			     (unsigned long) VFF_MS_NONE,
			     (unsigned long) VFF_MAPTYP_NONE, 
			     (unsigned long) VFF_LOC_IMPLICIT, 
			     (unsigned long) 0);

        if (image1 == NULL)
        {
           kerror("vpml", "main", "Unable to allocate new image!\n");
           kexit(KEXIT_FAILURE);
        }

/* Allocate space for image containing the fractal dimension values */
	fd_image = createimage((unsigned long) 1, 
			       (unsigned long) num_classes, 
			       (unsigned long) VFF_TYP_FLOAT, 
			       (unsigned long) 1,
			       (unsigned long) clui_info->q_int, dummy, 
			       (unsigned long) 0, 
			       (unsigned long) 0, 
			       (unsigned long) VFF_MS_NONE,
			       (unsigned long) VFF_MAPTYP_NONE, 
			       (unsigned long) VFF_LOC_IMPLICIT, 
			       (unsigned long) 0);

	if (fd_image == NULL)
	 {
	    kerror("vpml", "main", "Unable to allocate new image!\n");
	    kexit(KEXIT_FAILURE);
	 }
/* -main_before_lib_call_end */

/* -main_library_call */
/*---  Call the lib routine for P(m,L) algorithm ---*/

	if(! lvpml(image, image1, fd_image, printdev, clui_info->l_int,
		   clui_info->u_int, clui_info->s_int, clui_info->q_int))
	 {
	    kexit(KEXIT_FAILURE);
	 }
/* -main_library_call_end */

/* -main_after_lib_call */
/* write out the resulting output image and fractal dimension image */
	writeimage(clui_info->o1_file, image1);
	writeimage(clui_info->o2_file, fd_image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vpml_usage_additions
| 
|       Purpose: Prints usage additions in vpml_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vpml
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vpml_usage_additions(void)
{
	kfprintf(kstderr, "\tEstimate Fractal Dimension of Image Based on P(m,L)  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vpml_free_args
| 
|       Purpose: Frees CLUI struct allocated in vpml_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vpml
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vpml_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o1_file);
		kfree_and_NULL(clui_info->o2_file);
		kfree_and_NULL(clui_info->f1_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
