 /*
  * Khoros: $Id: vquant.h,v 1.14.2.2 1997/05/31 20:11:37 mirage Exp $
  */

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>>       Purpose: Include file for vquant
   >>>> 
   >>>>    Written By: Scott Wilson
   >>>> 
   >>>> Modifications: 
   >>>> 
   >>>>          Date: July 1, 1997
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#ifndef _vquant_h_
#define _vquant_h_

        /*-------------------------------------*
         |       #includes
        ---------------------------------------*/

#include <retro.h>

/* -include_includes */
struct vector
  {
    float *vec;                           /* Vector components */
    int count;                            /* Pixel count */
    struct vector *prev;                  /* Ptrs to previous & next vector */
    struct vector *next;
    struct pixel *pixlist;                /* List of pixels of this vector */
    struct pixel *pixtail;                /* End of pixel list */
  };

struct pixel
  {
    short x,y;                            /* Location of pixel */
    struct pixel *next;                   /* Pointer to next one */
  };

struct box
  {
    float *vmin,*vmax;                    /* Bounds on box */
    struct vector *vectors;               /* List of vectors in box */
    struct box *next;                     /* Next box */
    int count;                            /* Number of pixels in box */
  };
/* -include_includes_end */


        /*-------------------------------------*
         |       #defines
        ---------------------------------------*/

/* -include_defines */
/* -include_defines_end */

        /*-------------------------------------*
         |       typedefs
        ---------------------------------------*/

typedef struct _clui_info_struct {

/*
 *  Input image (required infile)
 */
char	*i_file;	/* Input image FILENAME */
int	 i_flag;	/* Input image FLAG */

/*
 *  Output cluster number image (required outfile)
 */
char	*o_file;	/* Output cluster number image FILENAME */
int	 o_flag;	/* Output cluster number image FLAG */

/*
 *  Number of vectors (optional integer)
 */
int	 n_int; 	/* Number of vectors INT */
int	 n_flag;	/* Number of vectors FLAG */

/*
 *  Output Image Map Enable (optional logical)
 */
int	 m_logic;	/* Output Image Map Enable LOGIC */
int	 m_flag;  	/* Output Image Map Enable FLAG */

/*
 *  Select split point:  false = mid-span, true = mean (optional logical)
 */
int	 s_logic;	/* Select split point:  false = mid-span, true = mean LOGIC */
int	 s_flag;  	/* Select split point:  false = mid-span, true = mean FLAG */

/*
 * Select splitting axis (Optional integer toggle)
 *	1 (Maximum span axis)
 *	2 (Maximum variance axis)
 *	3 (Principal eigenvector),
 */
int	 axis_toggle;		/* Select splitting axis INT TOGGLE */
int	 axis_flag;		/* Select splitting axis FLAG */

/*
 *  Output cluster center image (optional outfile)
 */
char	*o1_file;	/* Output cluster center image FILENAME */
int	 o1_flag;	/* Output cluster center image FLAG */

/*
 *  Output cluster variance image (optional outfile)
 */
char	*o2_file;	/* Output cluster variance image FILENAME */
int	 o2_flag;	/* Output cluster variance image FLAG */

} clui_info_struct;

/* -include_typedefs */
/* -include_typedefs_end */


        /*-------------------------------------*
         |       global variable declarations
        ---------------------------------------*/

extern clui_info_struct *clui_info;
extern uis_struct clui_uis_spec;
/* -include_variables */
/* -include_variables_end */


        /*-------------------------------------*
         |       macros
        ---------------------------------------*/

/* -include_macros */
#define READINPUT(image1)\
  image1 = readimage(clui_info->i_file);\
  if (image1 == NULL)\
    {\
      kerror(NULL, NULL, "unable to read input image");\
      kexit(1);\
    }

#define CHECKINPUT(program,image)\
  propertype(program,image,VFF_TYP_FLOAT,TRUE);\
  proper_num_images(program,image,1,TRUE);\
  proper_map_enable(program,image,VFF_MAP_OPTIONAL,TRUE);
/* -include_macros_end */

        /*-------------------------------------*
         |       routine definitions
        ---------------------------------------*/

int main PROTO((int, char **));
void vquant_get_args PROTO((kaddr));
void vquant_usage_additions PROTO((void));
void vquant_free_args PROTO((kexit_status, kaddr));

/* -include_routines */
/* -include_routines_end */

#endif
