 /*
  * Khoros: $Id: vsamp.c,v 1.14.2.3 1997/05/31 20:11:49 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vsamp.c,v 1.14.2.3 1997/05/31 20:11:49 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vsamp
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vsamp.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Sample an Image Along an Arbitrary Line  (K1)
|
|       Purpose: main program for vsamp
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->x1_int; {starting x coordinate for line}
|		int clui_info->x1_flag; {TRUE if -x1 specified}
|
|		int clui_info->x2_int; {ending x coordinate for line}
|		int clui_info->x2_flag; {TRUE if -x2 specified}
|
|		int clui_info->y1_int; {starting y coordinate for line}
|		int clui_info->y1_flag; {TRUE if -y1 specified}
|
|		int clui_info->y2_int; {ending y coordinate for line}
|		int clui_info->y2_flag; {TRUE if -y2 specified}
|
|		int clui_info->n_int; {number of samples}
|		int clui_info->n_flag; {TRUE if -n specified}
|
|        Output:
|       Returns:
|
|    Written By: Richard Krukar
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	xvimage *image,*out;
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vsamp");
	kexit_handler(vsamp_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vsamp", KGEN_KROUTINE,  &clui_uis_spec,
		   vsamp_usage_additions, vsamp_get_args,
 		   vsamp_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        kerror(NULL, "vsamp",
                "Program NOT executed!\n"
                "  vsamp functionality is replaced by the DATAMANIP::ksampline "
                "operator.\n"
                "  The following command line arguments have changed.\n"
            	"     -x1 (starting x coordinate for line), is now -wc1\n"
                "     -x2 (ending x coordinate for line), is now -wc2\n"
                "     -y1 (starting y coordinate for line), is now -hc1\n"
                "     -y2 (ending y coordinate for line), is now -hc1");
        kexit(KEXIT_FAILURE);

	if((image = readimage(clui_info->i_file)) == NULL) {
          kerror(NULL, NULL, "readimage failed on input file\n");
          kexit(1); }
        if (image->data_storage_type != VFF_TYP_FLOAT)
          {
            kerror(NULL, NULL, "Input image must be of type FLOAT\n");
            kexit(1);
          }
        if (image->num_data_bands != 1 || image->map_scheme != VFF_MS_NONE)
          {
            kerror(NULL, NULL,
		   "Input image must be single banded, and contain no map!\n");
            kexit(1);
          }
/* -main_before_lib_call_end */

/* -main_library_call */
if (!(lvsamp(image,&out,clui_info->x1_int,clui_info->y1_int,clui_info->x2_int,
        clui_info->y2_int,clui_info->n_int ))) {
   kerror(NULL, NULL, "lvsamp failed\n");
   kfprintf(kstderr, "vsamp: lvsamp failed\n");
   kexit(1); }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file, out);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vsamp_usage_additions
| 
|       Purpose: Prints usage additions in vsamp_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vsamp
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vsamp_usage_additions(void)
{
	kfprintf(kstderr, "\tSample an Image Along an Arbitrary Line  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vsamp_free_args
| 
|       Purpose: Frees CLUI struct allocated in vsamp_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vsamp
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vsamp_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
