 /*
  * Khoros: $Id: vsdef.c,v 1.14.2.3 1997/05/31 20:12:02 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vsdef.c,v 1.14.2.3 1997/05/31 20:12:02 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vsdef
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vsdef.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Second Derivative Operator for Symmetric Exponential Filter  (K1)
|
|       Purpose: main program for vsdef
|
|         Input:
|		char *clui_info->i_file; {Input Image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Output Image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		float clui_info->a0_float; {Exponential Filter Parameter for GEF}
|		int   clui_info->a0_flag; {TRUE if -a0 specified}
|
|		int clui_info->t1_int; {1st Hysteresis Threshold}
|		int clui_info->t1_flag; {TRUE if -t1 specified}
|
|		int clui_info->t2_int; {2nd Hysteresis Threshold}
|		int clui_info->t2_flag; {TRUE if -t2 specified}
|
|		int clui_info->l_int; {Minimum Number of Pixels in a Segment}
|		int clui_info->l_flag; {TRUE if -l specified}
|
|        Output:
|       Returns:
|
|    Written By: Pascal ADAM
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
    char *program = "vsdef";
    xvimage *image;
    int t1,t2,length;
    float a0;
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vsdef");
	kexit_handler(vsdef_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vsdef", KGEN_KROUTINE,  &clui_uis_spec,
		   vsdef_usage_additions, vsdef_get_args,
 		   vsdef_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    program = "vsdef";
 
    READINPUT(image);
    CHECKINPUT (program,image);

    t1     = clui_info->t1_int;
    t2     = clui_info->t2_int;
    length = clui_info->l_int;
    a0     = clui_info->a0_float;
/* -main_before_lib_call_end */

/* -main_library_call */
if(! lvsdef(image,a0,t1,t2,length))
    {
        (void) kerror(NULL, NULL, "lvsdef Failed\n");
        kexit(1);
    }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vsdef_usage_additions
| 
|       Purpose: Prints usage additions in vsdef_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vsdef
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vsdef_usage_additions(void)
{
	kfprintf(kstderr, "\tSecond Derivative Operator for Symmetric Exponential Filter\n");
	kfprintf(kstderr, "\t(K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vsdef_free_args
| 
|       Purpose: Frees CLUI struct allocated in vsdef_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vsdef
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vsdef_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
