 /*
  * Khoros: $Id: vslope.c,v 1.14.2.3 1997/05/31 20:12:28 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vslope.c,v 1.14.2.3 1997/05/31 20:12:28 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vslope
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vslope.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Compute Slope and Aspect Images from Elevation Data  (K1)
|
|       Purpose: main program for vslope
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		int clui_info->so_toggle; {slope options}
|		int clui_info->so_flag; {TRUE if -so specified}
|
|		int clui_info->to_toggle; {aspect options}
|		int clui_info->to_flag; {TRUE if -to specified}
|
|		float clui_info->z_float; {Value for Regions With no Aspect (flat)}
|		int   clui_info->z_flag; {TRUE if -z specified}
|
|		char *clui_info->s_file; {output slope image}
|		int   clui_info->s_flag; {TRUE if -s specified}
|
|		char *clui_info->t_file; {output aspect image}
|		int   clui_info->t_flag; {TRUE if -t specified}
|
|        Output:
|       Returns:
|
|    Written By: Per Lysne
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	char *program = "vslope";
	xvimage *img1, *img2, *img3;
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vslope");
	kexit_handler(vslope_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vslope", KGEN_KROUTINE,  &clui_uis_spec,
		   vslope_usage_additions, vslope_get_args,
 		   vslope_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    img1 = readimage(clui_info->i_file);
    if (img1 == NULL) {
        kerror ("vslope", "main",
		"error in vslope: could not read image from %s\n",
                clui_info->i_file);
        kexit (KEXIT_FAILURE);
    }
                    
    CHECKINPUT (program, img1);
        
    if (img1->data_storage_type != VFF_TYP_FLOAT) {
        if (!lvconvert (img1, VFF_TYP_FLOAT, 0, 1, 0.0, 1.0, 0)) {
            kerror ("vslope", "main",
		    "error in vslope: call to lvconvert failed\n");
            kexit (KEXIT_FAILURE);
        }
    }
/* -main_before_lib_call_end */

/* -main_library_call */
    if(!lvslope(img1,
                clui_info->s_flag, clui_info->so_toggle,
                clui_info->t_flag, clui_info->to_toggle,
                clui_info->z_float,
                &img2, &img3)) {
        kerror("vslope", "main", "error in vslope: call to lvslope failed\n");
        kexit(KEXIT_FAILURE);
    }
/* -main_library_call_end */

/* -main_after_lib_call */
    if (clui_info->s_flag) {
        if (!writeimage (clui_info->s_file, img2)) {
            kerror ("vslope","main", "Could not write slope image to %s\n",
		    clui_info->s_file);
            kexit (KEXIT_FAILURE);
        }
    }

    if (clui_info->t_flag) {
        if (!writeimage(clui_info->t_file, img3)) {
            kerror ("vslope", "vslope", "Could not write aspect image to %s\n",
		    clui_info->t_file);
            kexit (KEXIT_FAILURE);
        }
    }
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vslope_usage_additions
| 
|       Purpose: Prints usage additions in vslope_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vslope
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vslope_usage_additions(void)
{
	kfprintf(kstderr, "\tCompute Slope and Aspect Images from Elevation Data  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vslope_free_args
| 
|       Purpose: Frees CLUI struct allocated in vslope_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vslope
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vslope_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->s_file);
		kfree_and_NULL(clui_info->t_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
