 /*
  * Khoros: $Id: vspatial.c,v 1.14.2.3 1997/05/31 20:12:41 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vspatial.c,v 1.14.2.3 1997/05/31 20:12:41 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vspatial
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vspatial.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Compute Spatial Features Using NxM Window  (K1)
|
|       Purpose: main program for vspatial
|
|         Input:
|		char *clui_info->i_file; {input image filename}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output image filename}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->w_int; {window width argument}
|		int clui_info->w_flag; {TRUE if -w specified}
|
|		int clui_info->h_int; {window height argument}
|		int clui_info->h_flag; {TRUE if -h specified}
|
|		int clui_info->m_logic; {apply the Mean Spatial Operator }
|		int clui_info->m_flag; {TRUE if -m specified}
|
|		int clui_info->s_logic; {apply the Angular 2nd Moment Spatial Operator }
|		int clui_info->s_flag; {TRUE if -s specified}
|
|		int clui_info->c_logic; {apply the Contrast Spatial Operator }
|		int clui_info->c_flag; {TRUE if -c specified}
|
|		int clui_info->v_logic; {apply the Variance Spatial Operator }
|		int clui_info->v_flag; {TRUE if -v specified}
|
|		int clui_info->e_logic; {apply the Entropy Spatial Operator }
|		int clui_info->e_flag; {TRUE if -e specified}
|
|		int clui_info->d_logic; {apply the Dispersion Spatial Operator }
|		int clui_info->d_flag; {TRUE if -d specified}
|
|        Output:
|       Returns:
|
|    Written By: Charlie Gage
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
char *program = "vspatial";
xvimage *image;
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vspatial");
	kexit_handler(vspatial_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vspatial", KGEN_KROUTINE,  &clui_uis_spec,
		   vspatial_usage_additions, vspatial_get_args,
 		   vspatial_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
READINPUT(image);
CHECKINPUT(program, image);
/* -main_before_lib_call_end */

/* -main_library_call */
/* Call the lib routine for calculation of spatial operators */

if(! lvspatial(image, clui_info->w_int, clui_info->h_int, clui_info->m_logic, clui_info->v_logic, clui_info->c_logic, clui_info->s_logic, clui_info->e_logic, clui_info->d_logic))
 {
     (void) kfprintf(kstderr, "lvspatial Failed\\n");
     kexit(KEXIT_FAILURE);
 }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vspatial_usage_additions
| 
|       Purpose: Prints usage additions in vspatial_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vspatial
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vspatial_usage_additions(void)
{
	kfprintf(kstderr, "\tCompute Spatial Features Using NxM Window  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vspatial_free_args
| 
|       Purpose: Frees CLUI struct allocated in vspatial_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vspatial
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vspatial_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
