 /*
  * Khoros: $Id: vsurf.c,v 1.14.2.3 1997/05/31 20:13:07 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vsurf.c,v 1.14.2.3 1997/05/31 20:13:07 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vsurf
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vsurf.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Compute Surface Parameters (Normals and Angles)  (K1)
|
|       Purpose: main program for vsurf
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->norm_file; {output normals image}
|		int   clui_info->norm_flag; {TRUE if -norm specified}
|
|		char *clui_info->ang_file; {output angles image}
|		int   clui_info->ang_flag; {TRUE if -ang specified}
|
|        Output:
|       Returns:
|
|    Written By: Scott Wilson
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	xvimage *img1, *img2, *img3;
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vsurf");
	kexit_handler(vsurf_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vsurf", KGEN_KROUTINE,  &clui_uis_spec,
		   vsurf_usage_additions, vsurf_get_args,
 		   vsurf_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        img1 = readimage(clui_info->i_file);
        if (img1 == NULL)
	  {
            kerror("vsurf", "main", "Could not read %s\n", clui_info->i_file);
	    kexit(KEXIT_FAILURE);                    
	  }

        if (img1->data_storage_type != VFF_TYP_FLOAT)
          {
            kerror("vsurf", "main", "Surface image must be a FLOAT image\n");
            kexit(KEXIT_FAILURE);
          }

        if (img1->pixsizx != img1->pixsizy || img1->pixsizx == 0)
          {
            kerror("vsurf", "main", "Pixels must be square and non-zero\n");
            kexit(KEXIT_FAILURE);
          }
/* -main_before_lib_call_end */

/* -main_library_call */
	if(! lvsurf(img1, &img2, &img3) )
        {
            kexit(KEXIT_FAILURE);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
	(void) writeimage(clui_info->norm_file, img2);
        (void) writeimage(clui_info->ang_file, img3);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vsurf_usage_additions
| 
|       Purpose: Prints usage additions in vsurf_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vsurf
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vsurf_usage_additions(void)
{
	kfprintf(kstderr, "\tCompute Surface Parameters (Normals and Angles)  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vsurf_free_args
| 
|       Purpose: Frees CLUI struct allocated in vsurf_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vsurf
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vsurf_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->norm_file);
		kfree_and_NULL(clui_info->ang_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
