 /*
  * Khoros: $Id: vwmdd.c,v 1.14.2.3 1997/05/31 20:13:33 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vwmdd.c,v 1.14.2.3 1997/05/31 20:13:33 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vwmdd
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vwmdd.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Weighted Minimum Distance Detector  (K1)
|
|       Purpose: main program for vwmdd
|
|         Input:
|		char *clui_info->i1_file; {Input Image}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {Input Cluster Center/Class Image}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->i3_file; {Input Variance Image}
|		int   clui_info->i3_flag; {TRUE if -i3 specified}
|
|		char *clui_info->o_file; {Output specifying which vector belongs to which cluster}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		float clui_info->k_float; { Scaling factor}
|		int   clui_info->k_flag; {TRUE if -k specified}
|
|		int clui_info->b_int; { Border width in pixels}
|		int clui_info->b_flag; {TRUE if -b specified}
|
|		int clui_info->s_logic; {If true, Use Summed Method}
|		int clui_info->s_flag; {TRUE if -s specified}
|
|        Output:
|       Returns:
|
|    Written By: Tom Sauer
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
	char *program = "vwmdd";
	xvimage *image, *center, *varimg, *images[3];
/* -main_variable_list_end */

	khoros_init(argc, argv, "RETRO", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$RETRO/objects/kroutine/vwmdd");
	kexit_handler(vwmdd_free_args, NULL);

/* -main_get_args_call */
	kclui_init("RETRO", "vwmdd", KGEN_KROUTINE,  &clui_uis_spec,
		   vwmdd_usage_additions, vwmdd_get_args,
 		   vwmdd_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
/* -main_before_lib_call_end */

/* -main_library_call */
	READINPUT1 (image);
        READINPUT2 (center);
        READINPUT3 (varimg);

        CHECKINPUT (program, image);
        CHECKINPUT(program, center);
        CHECKINPUT(program, varimg);

          /* upconvert all images to float */
        images[0] = image;
        images[1] = center;
        images[2] = varimg;
        if (! lvcast(images, 3, VFF_TYP_FLOAT))
        {
            kexit(KEXIT_FAILURE);
        }

        if ( image->num_data_bands != (center->num_data_bands - 1)) 
        {
            kerror("vwmdd", "main", "The center/class image does not contain \
the correct number of data_bands.\n\
hould have %d bands, but contains %d bands\n", 
                   (image->num_data_bands + 1), center->num_data_bands);
            kexit(KEXIT_FAILURE);
        }

        if ( image->num_data_bands != varimg->num_data_bands) 
        {
             kerror("vwmdd", "main", "The variance image does not contain \
the correct\nnumber of data_bands\n\
Should have %d bands, but contains %d bands\n", 
                    image->num_data_bands , varimg->num_data_bands);
             kexit(KEXIT_FAILURE);
        }

    /* Call the lib routine for minimum distance algorithm */

        if(! lvwmdd(image, center, varimg, clui_info->b_int, clui_info->k_float, 
                    clui_info->s_logic))
        {
            kexit(KEXIT_FAILURE);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vwmdd_usage_additions
| 
|       Purpose: Prints usage additions in vwmdd_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vwmdd
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void vwmdd_usage_additions(void)
{
	kfprintf(kstderr, "\tWeighted Minimum Distance Detector  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vwmdd_free_args
| 
|       Purpose: Frees CLUI struct allocated in vwmdd_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vwmdd
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vwmdd_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->i3_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
