/*
  * Khoros: $Id: pro_map_sc.c,v 1.7 1997/01/20 07:41:06 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: pro_map_sc.c,v 1.7 1997/01/20 07:41:06 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		File Contents Title
   >>>>		
   >>>>   Public:
   >>>>                 proper_map_scheme()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/************************************************************
*
*  Routine Name: proper_map_scheme
*
*       Purpose: Check to see if a viff data structure has the correct
*               mapping scheme.
*
*         Input: prog 	- program name
*		*image	- viff data
*		type	- desired color map model
*		exit_flag
*
*        Output: 
*
*       Returns: If exit_flag is true and match fails, an error 
*          	message is printed.
*      		If exit_flag is false and match fails, return(0).
*
*  Restrictions: 
*    Written By:
*          Date: 
*      Verified:
*  Side Effects:
* Modifications:
*************************************************************/
int proper_map_scheme(char *prog, xvimage *image, long unsigned int type, int exit_flag)
{
   if(image->map_scheme != type) {
      if (exit_flag == TRUE) {
         (void) kfprintf(kstderr, "\nError in program %s: \n",prog);
         switch(type){
            case VFF_MS_NONE :
               (void) kfprintf(kstderr, 
"Mapping scheme should be VFF_MS_NONE\n");
               (void) kfprintf(kstderr, 
"This program does not operate with a data mapping, but one is present.\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_MS_ONEPERBAND :
               (void) kfprintf(kstderr, 
"Mapping scheme should be VFF_MS_ONEPERBAND\n");
               (void) kfprintf(kstderr, 
"Each data band should have its own unique mapping.\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_MS_CYCLE :
               (void) kfprintf(kstderr, 
"Mapping scheme should be VFF_MS_CYCLE\n");
               (void) kfprintf(kstderr, 
"There should be several maps available to cycle through for a single band of data.\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_MS_SHARED :
               (void) kfprintf(kstderr, 
"Mapping scheme should be VFF_MS_SHARED\n");
               (void) kfprintf(kstderr, 
"All of the data should share a single map.\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_MS_GROUP :
               (void) kfprintf(kstderr, "Mapping scheme should be VFF_MS_GROUP\n");
               kexit(KEXIT_FAILURE);
               break;
            default :
               (void) kfprintf(kstderr, "Mapping scheme is unknown.\n");
               kexit(KEXIT_FAILURE);
               break;
	 }
      }
      else return(0);
   }
return(1);
}
