 /*
  * Khoros: $Id: lvbandsp1.c,v 1.15.2.2 1997/05/31 20:16:05 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lvbandsp1.c,v 1.15.2.2 1997/05/31 20:16:05 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vbandsp1
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvbandsp1
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvbandsp1 - extract single band images from one multiband image
* 
*       Purpose: lvbandsprt extracts all bands from an image and creates
*		 separate images from each band.  The map is also carried along
*		 with the new image, and the map scheme will become ONEPERBAND
*		 regardless of the input image's map scheme.  The comment field
*		 of the new image will read the same as the comment field of
*		 the input image, except it is append with the band number and
*		 the filename from which the band was taken.
*
*		 The arguments to lvbandsprt include:
*		 NOTE: This library routine is common to both vbandspt1 and
*		 vbandspt3, and is called by each routine to perform the data
*		 band separation.
*         Input: img - the input xvimage structure to extract bands from.
*		 images - a list of pointers to xvimage structures.  The result
*			  is passed in from a call to
*			  creat_images_list(image->num_data_bands).
*		 img_file_name - a pointer to the name of the img file.
*		 mod_comment - this is a flag that if set to true modifies the
*			       new image's comment field by appending the data
*			       band number and the original file name from
*			       which the new image was created.
*        Output: images - returns a list of pointers to xvimage structures, the
*		 new images containing only one data band are returned in this
*		 structure.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: 
*    Written By: Tom Sauer
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 1/18/91 - Added an error message trapping for input images
*		 with only one band. C.Gage.
*
*		 8/18/91 - Added a line of code to fix bug reported by Bengt
*		 Martensson.  Location data not being copied.
****************************************************************/
/* -library_def */
int lvbandsp1(
    xvimage *img, 
    xvimage **images, 
    char *img_file_name, 
    int mod_comment)
/* -library_def_end */

/* -library_code */
{
        int     data_size,dcount,map_size,mcount,lsize,lcount;
        int     i;
        char    comment[KLENGTH], *file;

             /* check to see if only one images exists in the input file */

        if ( ! (proper_num_images("lvbandsprt",img,(unsigned long) 1,FALSE))) {
           kerror("vipl", "lvbandsprt",
		  "ERROR All inputs must contain only one image\n");
           return(FALSE);
       }


         /* go compute the image size and map size in bytes */

        if (! (imagesize(img,&data_size,&dcount,&map_size,
                             &mcount,&lsize,&lcount)))
        {
            kerror("vipl", "lvbandsprt", "ERROR imagesize failed\n");
            return(FALSE);
        }

         /* want the data_size for each band, not the total image data size */
         /* so divide by the number of image bands                          */

        data_size /= img->num_data_bands;

         /* if the map scheme is ONEPERBAND then only want the size of each */
         /* map, not the total map size                                     */

        if ( img->map_scheme == VFF_MS_ONEPERBAND) 
               map_size /= img->num_data_bands;

           /* loop through all data bands and separate them out into
              one band images, process the maps and copy the image
              header from the input image  */

        for (i = 0; i < img->num_data_bands; i++) {
           images[i]->imagedata = &img->imagedata[data_size*i];
           switch (img->map_scheme) {
                case VFF_MS_NONE:
                  break;
                case VFF_MS_ONEPERBAND:
                  images[i]->maps = &img->maps[map_size*i];
                  break;
                case VFF_MS_SHARED:
                  images[i]->maps = &img->maps[0];
                  break;
                default:
                  kerror("vipl", "lvbandsprt", "ERROR Invalid map scheme. \
Map scheme should be either\n\
               NONE, ONEPERBAND or SHARED\n");
                  return(FALSE);
              }
              copyheader(img, images[i]);
              images[i]->num_data_bands = 1;
              images[i]->color_space_model = VFF_CM_NONE;
	      images[i]->location = img->location;  /* bug fix for location
                                                       data */

                /* if the modify comment flag is set then get the
                   input image filename, append to the new image
                   the band number and the image name the band was
                   extracted from  */

              if ( mod_comment ) {
                 file = kbasename(img_file_name, NULL);
                 (void) ksprintf(comment, "Data Band %d from image:  %s.\n\n",
                    i, file);
                 create_comment(images[i]->comment, comment, img->comment);
              }
        }
        return(1);
}
/* -library_code_end */
