 /*
  * Khoros: $Id: lvcast.c,v 1.15.2.2 1997/05/31 20:16:07 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lvcast.c,v 1.15.2.2 1997/05/31 20:16:07 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vcast
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvcast
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvcast - cast input images to highest image data type
* 
*       Purpose: vcast will upconvert or upcast each input image to the highest
*		 input image data type.  Optionally, the user may elect to
*		 upcast all images to a specified data type, as long as it is
*		 a data type that is equal to or greater than the highest data
*		 type of the input images.  If the user tries to select a data
*		 type that is lower than the highest data type of the input
*		 images, an error message will be produced indicating that down
*		 casting is not allowed.
*
*		 There are no restrictions as to the input image data types.
*		 The data types can range from BIT to DOUBLE.  The input images
*		 may contain any combination of sizes or number of data bands.
*		 All map information from each input image is transferred to the
*		 corresponding output image. 
*
*		 Note that when converting from type BIT to BYTE, the output
*		 values will be 0 or 1.  This is different from the results one
*		 would obtain using the defaults with vconvert.  In trying to
*		 preserve the integrity of the data, vcast scales the data by 1.*		 This routine calls lvconvert to do the actual up convert/cast
*		 of the image data.
*
*         Input: images - array of pointers to xvimage structures
*		 image_num - integer indicating the number of input images
*		 up_type - optional target data type.  Pass in "-1" to select
*			   auto upconvert.
*        Output: images - array of pointers to resulting xvimage structures.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: 
*    Written By: Charlie Gage
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lvcast(
	xvimage **images,
	int image_num,
	int up_type)
/* -library_def_end */

/* -library_code */
{
 int chg_flag,
     prev_type,
     max_type,
     type,
     i;

 chg_flag = 0;
 max_type = 0;
 prev_type = (int) images[0]->data_storage_type;

   /*-------------------------------------------------------
    * Determine highest data type of input images 
    *------------------------------------------------------*/
 for (i=0; i < image_num; i++)
 {
   type = (int) images[i]->data_storage_type;
   if (type > max_type)
   {
     max_type = type;
   }
          /* Set flag if any data type differs from any other */
   if (prev_type != type)
   {
     chg_flag = 1;
   }
   prev_type = type;
 }

   /*------------------------------------------------------
    * Determine if requested data type is equal to
    *     or greater highest data type of input images.
    *-----------------------------------------------------*/
 if (up_type != -1)
 {
   if (up_type >= max_type)
   {
     max_type = up_type;
     chg_flag = 1;
   }
   else
   {
     kerror("vipl", "lvcast", "You have requested a data type that\n\
is lower than the highest data type of the\n\
input images.  Unable to down cast data.\n");
     return(FALSE);
   }
 }
   
   /*------------------------------------------------------
    * If images are of different data types, 
    *  then upconvert/cast them.
    *-----------------------------------------------------*/
 if (chg_flag == 1)
 {
   for (i=0; i < image_num; i++)
   {
                   /* Upconvert only those images that are not of max_type */
     if ((int) images[i]->data_storage_type != max_type)
     {
                       /* Call lvconvert to upconvert to max data type */
       if (lvconvert(images[i], max_type, 0, 1, (float) 0.0, (float) 1.0, FALSE) == 0)
       {
         kerror("vipl","lvcast", "ERROR, lvconvert failed\n");
         return(FALSE);
       }
     }
   }
 }

 return(1);
}  /*** End of lvcast ***/
/* -library_code_end */
