 /*
  * Khoros: $Id: lvfht.c,v 1.15.2.2 1997/05/31 20:16:26 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lvfht.c,v 1.15.2.2 1997/05/31 20:16:26 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vfht
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvfht
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
#define CHECKLIBINPUT(program, img) { \
        if (!propertype (program, img, (unsigned long) VFF_TYP_FLOAT, FALSE)) {               \
            kerror ("vipl", "lvfht",                                          \
                    "input image must be VFF_TYP_FLOAT\n");                   \
            return (FALSE);                                                   \
        }                                                                     \
        if (!proper_loc_type (program, img, (unsigned long) VFF_LOC_IMPLICIT, FALSE)) {       \
            kerror ("vipl", "lvfht",                                          \
	           "input image must be VFF_LOC_IMPLICIT\n");                 \
            return (FALSE);                                                   \
        }                                                                     \
        if (!proper_map_type (program, img, (unsigned long) VFF_MAPTYP_NONE, FALSE)) {        \
            kerror ("vipl", "lvfht",                                          \
	            "input image must be VFF_MAPTYP_NONE\n");                 \
            return (FALSE);                                                   \
        }                                                                     \
        if (!proper_map_enable (program, img, (unsigned long) VFF_MAP_OPTIONAL, FALSE)) {     \
            kerror ("vipl", "lvfht",                                          \
	            "input image must be VFF_MAP_OPTIONAL\n");                \
            return (FALSE);                                                   \
        }                                                                     \
        if (!proper_map_scheme (program, img, (unsigned long) VFF_MS_NONE, FALSE)) {          \
            kerror ("vipl", "lvfht",                                          \
	            "input image must be VFF_MS_NONE\n");                     \
            return (FALSE);                                                   \
        }                                                                     \
        if (!proper_num_images (program, img, (unsigned long) 1, FALSE)) {                    \
            kerror ("vipl", "lvfht",                                          \
	    "input image must contain only 1 image\n");                       \
            return (FALSE);                                                   \
        }                                                                     \
        if (!proper_num_bands (program, img, (unsigned long) 1, FALSE)) {                     \
            kerror ("vipl", "lvfht",                                          \
	            "input image must contain only 1 band\n");                \
            return (FALSE);                                                   \
        } }

static void fht1d PROTO((float *, int, int));
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvfht - Perorm fast hadamard transform on the input image
* 
*       Purpose: This command performs a fast hadamard transform on the image.
*
*		 If your input image had dimensions 4 row by 4 columns, and the
*		 image had a constant value as shown below
*
*		 !                 input image = { 1, 1, 1, 1,
*		 !                                 1, 1, 1, 1,
*		 !                                 1, 1, 1, 1,
*		 !                                 1, 1, 1, 1 }
*
*		 The output would be this
*
*		 !                 output image = { 1, 0, 0, 0,
*		 !                                  0, 0, 0, 0,
*		 !                                  0, 0, 0, 0,
*		 !                                  0, 0, 0, 0 }
*
*		 Because the input image was simply the first of the Hadamard
*		 basis functions with magnitude = 1.
*
*		 The Following references were used in creating this program :
*
*		 1.  "Digital Image Processing, Second Edition",
*		 Rafael C. Gonzalex, Paul Wintz, Addison-Wesley Publishing
*		 Company, Reading Massachusetts, 1987
*
*		 2.  "The C User's Journal", July 1989, "The Fast Walsh
*		 Transform", p101, Ian AshDown
*         Input: img - a pointer to an xvimage structure containing
*		       the image to be transformed
*		 inv_flag - if set this flag indicates that an inverse
*			    transform should be performed, otherwise a forward
*			    transform is executed.
*        Output: img - the input image is overwritten and the output is
*		       returned in its place.  Be careful not to overwrite
*		       important data.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: lvfht only works on images of type VFF_TYP_FLOAT.  The
*		 dimensions of the input image must both be non-zero powers of
*		 two.  Images with maps or explicit location data are not
*		 allowed.
*    Written By: Per Lysne
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lvfht (
	xvimage *img,
	int inv_flag)
/* -library_def_end */

/* -library_code */
{
    int N, M, i, j;
    float *data, *column;
    char *program="lvfht";

    /* Check for a bad input pointer. */
    if (img==NULL) {
        kerror ("vipl", "lvfht", "bad input xvimage pointer\n");
        return (FALSE);
    }

    /* Check for a correct type of input image. */
    CHECKLIBINPUT (program, img);

    /* Check for a legal size of input image. */
    if (!kpowtwo((int) img->row_size)) {
        kerror ("vipl","lvfht",
		"row length of the input image must be a power of two\n");
        return (FALSE);
    }
    if (!kpowtwo((int) img->col_size)) {
        kerror ("vipl","lvfht",
		"Column length of the input image must be a power of two\n");
        return (FALSE);
    }

    /* N is the number of samples. */
    M = img->row_size;
    N = img->col_size;

    column = (float *) kcalloc((size_t) N, sizeof(float));
    if (column==NULL) {
        kerror ("vipl", "lvfht", "Could not allocate column");
        return (FALSE);
    }

    /* Use data so I don't have to say img->imagedata all the time. */
    data=(float *) img->imagedata;

    /* Do the FHT across the rows. */
    for (i=0; i<N; i++)
        fht1d(&data[i*M], M, inv_flag);

    /* Do the FWT down the columns. */
    for (j=0; j<M; j++ ) {
        for (i=0; i<N; i++)
            column[i]=data[i*M+j];
        fht1d(column, N, inv_flag);
        for (i=0; i<N; i++)
            data[i*M+j]=column[i];
    }

    return(TRUE);
}



static void fht1d (
	float *data,
	 int N,
	 int inv_flag)
{
    int i, ip, j, k, l, le;
    float temp;

/*
     * This loop reorders the input data.
     */
    j=0;
    for (i=0; i<N-1; i++) {
        if (i<j) {
            temp=data[j];
            data[j]=data[i];
            data[i]=temp;
        }
        k=N/2;
        while (k<=j) {
            j=j-k;
            k=k/2;
        }
        j=j+k;
    }
    
/*
     * This loop does the Hadamard Transform.
     */
    l=1;
    while (l<N) {
        le=l;
        l=l*2;
        for (j=0; j<le; j++) {
            for (i=j; i<N; i=i+l) {
                ip=i+le;
                temp=data[ip];
                data[ip]=data[i]-temp;
                data[i]=data[i]+temp;
            }
        }
    }

    /* Scale the output of a forward transform. */
    if (inv_flag)
        for (i=0; i<N; i++)
            data[i]=data[i]/N;

    return;
}
/* -library_code_end */
