 /*
  * Khoros: $Id: lvgcirc.c,v 1.15.2.2 1997/05/31 20:16:35 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lvgcirc.c,v 1.15.2.2 1997/05/31 20:16:35 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vgcirc
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvgcirc
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvgcirc - creates an image of a filled circle
* 
*       Purpose: This routine creates an image of a filled circle. The
*		 background intensity is set by the -b argument. The
*		 foreground or circle intensity is set by the -f argument.
*		 The circle can be placed anywhere, and is NOT required to lie
*		 completely within the image boundaries.
*
*		 The position of the center of the circle and its diameter can
*		 all be specified as floating point numbers, allowing sub-pixel
*		 positioning of the circle.
*         Input: image - input image structure
*		 rows - the desired number of rows in output image
*		 cols - the desired number of columns in output image 
*		 type - the data storage type of the resulting image
*		 bgnd - the background intensity
*		 fgnd - the forground intensity
*		 x - the X coordinate of the center of the circle 
*		 y - the Y coordinate of the center of the circle 
*		 d - diameter of circle (in pixels)
*        Output: image - holds the resulting output image.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: lvgcirc will generate only BYTE or FLOAT images.
*    Written By: Scott Wilson
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lvgcirc (
	xvimage **image,
	int rows,
	int cols,
	int type,
	float bgnd,
	float fgnd,
	float x,
	float y,
	float d)
/* -library_def_end */

/* -library_code */
{
  int i,j;
  unsigned char *c;
  float *f,r;
  xvimage *img;

  img   = createimage((unsigned long) rows,  /* number of rows */
                     (unsigned long) cols,   /* number of columns */
                     (unsigned long) type,   /* data storage type */
                     (unsigned long) 1,      /* num_of_images */
                     (unsigned long) 1,      /* num_data_bands */
                     "created by vgcirc",    /* comment */
                     (unsigned long) 0,      /* map_row_size */
                     (unsigned long) 0,      /* map_col_size */
                     (unsigned long)
                     VFF_MS_NONE,            /* map_scheme */
                     (unsigned long)
                     VFF_MAPTYP_NONE,        /* map_storage_type */
                     (unsigned long)
                     VFF_LOC_IMPLICIT,       /* location_type */
                     (unsigned long) 0);     /* location_dim */


  if (img == NULL)
    {
      kerror("vipl","lvgcirc", "Unable to allocate new image!\n");
      *image = NULL;
      return(FALSE);
    }

  *image = img;
  r = d/2.0;
  r = r*r;

  switch(type)
    {
      case VFF_TYP_1_BYTE:
        c = (unsigned char *)(img->imagedata);
        for (i=0; i<rows; i++)
          for (j=0; j<cols; j++)
            if ((i-y)*(i-y)+(j-x)*(j-x) <= r) *c++ = fgnd;
            else *c++ = bgnd;
        break;
      case VFF_TYP_FLOAT:
        f = (float *)(img->imagedata);
        for (i=0; i<rows; i++)
          for (j=0; j<cols; j++)
            if ((i-y)*(i-y)+(j-x)*(j-x) <= r) *f++ = fgnd;
            else *f++ = bgnd;
        break;
      default:
        kerror("vipl","lvgcirc", "Unsupported data storage type requested\n\
         Supported types: BYTE and FLOAT\n");
        return(FALSE);
    }

  return(TRUE);
}
/* -library_code_end */
