 /*
  * Khoros: $Id: lvgettilt.c,v 1.15.2.2 1997/05/31 20:16:38 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lvgettilt.c,v 1.15.2.2 1997/05/31 20:16:38 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vgettilt
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvgettilt
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvgettilt - computes the best-fit plane parameters for the input VIFF file
* 
*       Purpose: lvgettilt & lvfindtilt compute the least-squares best
*		 fit plane for an image.  The plane will be given by
*		 f(x,y)=Ax+By+C with  the tilt coefficients in inverse
*		 pixels.
*
*		 lvmafdtilt computes the least-squares best fit plane
*		 for an image with a mask.  The plane will be given by
*		 f(x,y)=Ax+By+C with  the tilt coefficients in inverse
*		 pixels.  The plane coefficients will be computed only
*		 for those pixels enabled by the mask image.  If a mask
*		 image is supplied, it MUST be of the same type and
*		 size as the input image.
*
*         Input: lvgettilt: image  - input xvimage structure
*			    mimage - input mask xvimage structure
*			    mflg   - flag set to 1 if masking image
*				     is specified
*		 lvfindtilt: image - input xvimage structure
*
*		 lvmafdtilt: image  - input xvimage structure
*			     mimage - input mask xvimage structure
*
*        Output: lvgettilt, lvfindtilt, & lvmafdtilt:
*		 a - A coefficient of f(x,y)=Ax+By+C
*		 b - B coefficient of f(x,y)=Ax+By+C
*		 c - C coefficient of f(x,y)=Ax+By+C
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: If a mask image is supplied, it MUST be of the same
*		 type and size as the input image.
*		 The lvfindtilt and lvmafdtilt libraries will not work
*		 on BIT or COMPLEX data storage type files.  lvfindtilt
*		 and lvmafdtilt will only calculate the tilt of the
*		 first band in a multiband image.
*    Written By: Scott Wilson, Donna Koechner
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: Mon Jun 11 16:30:29 MDT 1990 Donna Koechner - updated to
*							       ghostwriter
*		 Wed Mar 20 12:36:43 MET 1991 Jean Pierre Guerin
*		    1- fixed loops to work right with non square images
*		    2- fixed bogus (int) caste
*		    3- fixed incorrect term in calculation of detc
*		    4- corrected pointer increments
*		 25-Sep-91 Scott Wilson - Changed (int) castes to (unsigned
*					  int) castes in byte processing
*					  loop - otherwise can get goofy sign
*					  extension problems.
*		 25-Sep-91 Scott Wilson - Found that the det{A,B,C} eqns were
*					  ALL wrong.  Fixed them up after
*					  rederiving the whole mess.
*		 26-Sep-91 Scott Wilson - Fixed incorrect center point
*					  calculation.
*   Declaration: int lvfindtilt(
*	         !      xvimage *image,
*	         !      double *a,
*	         !      double *b,
*	         !      double *c)
****************************************************************/
/* -library_def */
int lvgettilt(
	xvimage *image,
	xvimage *mimage,
	int mflg,
	double *a,
	double *b,
	double *c)
/* -library_def_end */

/* -library_code */
{
  if (mflg)
  {
    return(lvmafdtilt(image, mimage, a, b, c));
  }
  return(lvfindtilt(image, a, b, c));
}

/************************************************************
*
*  Routine Name: lvfindtilt - find the least-squares best fit plane for an image
*
*       Purpose: This routine computes the least-squares best fit plane for
*		 an image.  The plane will be given by f(x,y)=Ax+By+C
*                with  the tilt coefficients in inverse pixels.
*         Input: image - input xvimage structure
*        Output: a - the a coefficient of the plane
*		 b - the b coefficient of the plane
*		 c - the c coefficient of the plane.
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*    Written By: Scott Wilson, Donna Koechner
*          Date: Aug 15, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int lvfindtilt(
*	         !      xvimage *image,
*	         !      double *a,
*	         !      double *b,
*	         !      double *c)
*************************************************************/
int lvfindtilt(
	xvimage *image,
	double *a,
	double *b,
	double *c)
{
    double x2,xy,x,y2,y,xz,yz,z,n,det,deta,detb,detc, i_rxi_r, i_r,j_c;
    double rctr,cctr;
    int rows,cols;
    int i,j;
    unsigned char *ch;
    short *s;
    int *it;
    float *f;

    x2 = 0.0; xy = 0.0; x = 0.0; y2 = 0.0; y = 0.0; xz = 0.0;
    yz = 0.0; z  = 0.0;

    rows = image->col_size;
    cols = image->row_size;
    rctr = (rows-1)/2.0;
    cctr = (cols-1)/2.0;
    if (rctr == 0 || cctr == 0)
      {
        kerror("vipl", "lvfindtilt",
	       "Cannot find tilt on one pixel wide image!\n");
        return(FALSE);
      }

    n = rows*cols;

    switch(image->data_storage_type)
      {
        case VFF_TYP_BIT:
          kerror("vipl", "lvfindtilt", "Cannot find tilt for BIT image!\n");
          return(FALSE);
          /* break; */
        case VFF_TYP_1_BYTE:
          ch = (unsigned char *)image->imagedata;    
          for (i=0; i<rows; i++)
            { 
              i_r     = i - rctr;
              i_rxi_r = i_r * i_r;
              for (j=0; j<cols; j++, ch++)
                {
                  j_c= j - cctr;
                  x  += j_c;
                  x2 += j_c * j_c;
                  xy += j_c * i_r;
                  y  += i_r;
                  y2 += i_rxi_r;
                  xz += j_c * (unsigned int)*ch;
                  yz += i_r * (unsigned int)*ch;
                  z  += (unsigned int)*ch;
                }
            }
          break;
        case VFF_TYP_2_BYTE:
          s = (short *)image->imagedata;    
          for (i=0; i<rows; i++)
            { 
              i_r     = i - rctr;
              i_rxi_r = i_r * i_r;
              for (j=0; j<cols; j++, s++)
                {
                  j_c= j - cctr;
                  x  += j_c;
                  x2 += j_c * j_c;
                  xy += j_c * i_r;
                  y  += i_r;
                  y2 += i_rxi_r;
                  xz += j_c * (*s);
                  yz += i_r * (*s);
                  z  += *s;
                }
            }
          break;
        case VFF_TYP_4_BYTE:
          it = (int *)image->imagedata;    
          for (i=0; i<rows; i++)
            { 
              i_r     = i - rctr;
              i_rxi_r = i_r * i_r;
              for (j=0; j<cols; j++, it++)
                {
                  j_c= j - cctr;
                  x  += j_c;
                  x2 += j_c * j_c;
                  xy += j_c * i_r;
                  y  += i_r;
                  y2 += i_rxi_r;
                  xz += j_c * (*it);
                  yz += i_r * (*it);
                  z  += *it;
                }
            }
          break;
        case VFF_TYP_FLOAT:
          f = (float *)image->imagedata;    
          for (i=0; i<rows; i++)
            { 
              i_r     = i - rctr;
              i_rxi_r = i_r * i_r;
              for (j=0; j<cols; j++, f++)
                {
                  j_c= j - cctr;
                  x  += j_c;
                  x2 += j_c * j_c;
                  xy += j_c * i_r;
                  y  += i_r;
                  y2 += i_rxi_r;
                  xz += j_c * (*f);
                  yz += i_r * (*f);
                  z  += *f;
                }
            }
          break;
        case VFF_TYP_COMPLEX:
          kerror("vipl","lvfindtilt", "Cannot find tilt for COMPLEX image!\n");
          return(FALSE);
          /* break; */
        default:
          kerror("vipl","lvfindtilt", "Unkown data storage type: %d\n",
		 image->data_storage_type);
          return(FALSE);
          /* break; */
      }

    /* Find the matrix determinant */
    det = x2*(y2*n-y*y)-xy*(xy*n-y*x)+x*(xy*y-y2*x);

    /* Find the Cramer's determinants */
    deta = xz*(y2*n-y*y)-xy*(yz*n-y*z)+x*(yz*y-y2*z);
    detb = x2*(yz*n-y*z)-xz*(xy*n-x*y)+x*(xy*z-yz*x);
    detc = x2*(y2*z-yz*y)-xy*(xy*z-yz*x)+xz*(xy*y-y2*x);

    /* Compute the plane fit coefficients */
    *a = deta/det;
    *b = detb/det;
    *c = detc/det;

    return(TRUE);  /* success */

}


/**************************************************************
*
* MODULE NAME: lvmafdtilt
*
*     PURPOSE: Compute the least-squares best fit plane for an image.
*                The plane will be given by f(x,y)=Ax+By+Cz
*                with  the tilt coefficients in inverse pixels.
*                The plane coefficients will be computed only for those
*                pixels enabled by the mask image.
*
*       INPUT:
*             image - input xvimage structure
*             mimage - input mask xvimage structure
*
*             The mask image and the input image MUST be of the same
*             type and size!
*
*      OUTPUT: 
*              a,b,c - specify the coefficients of the plane.
*
*              lvmafdtilt will return a zero if failure and a
*              one upon success.
*
* CALLED FROM: main
*
* ROUTINES CALLED:
*
**************************************************************/

int lvmafdtilt(
	xvimage *image,
	xvimage *mimage,
	double  *a,
	double  *b,
	double  *c)
{
    double x2,xy,x,y2,y,xz,yz,z,n,det,deta,detb,detc, i_rxi_r, i_r,j_c;
    double rctr,cctr;
    int rows,cols;
    int i,j;
    unsigned char *ch,*mch;
    short *s,*ms;
    int *it,*mit;
    float *f,*mf;

    x2 = 0.0; xy = 0.0; x = 0.0; y2 = 0.0; y = 0.0; xz = 0.0;
    yz = 0.0; z  = 0.0;

    if (image->data_storage_type != mimage->data_storage_type)
      {
        kerror("vipl", "lvmafdtilt", "Mismatched input and mask!\n");
        return(FALSE);
      }
    rows = image->col_size;
    cols = image->row_size;
    rctr = (rows-1)/2.0;
    cctr = (cols-1)/2.0;
    if (rctr == 0 || cctr == 0)
      {
        kerror("vipl", "lvmafdtilt",
	       "Cannot find tilt for a one pixel wide image!\n");
        return(FALSE);
      }

    n = 0;

    switch(image->data_storage_type)
      {
        case VFF_TYP_BIT:
          kerror("vipl", "lvmafdtilt", "Cannot find tilt for BIT image!\n");
          return(FALSE);
          /* break; */
        case VFF_TYP_1_BYTE:
          ch = (unsigned char *)image->imagedata;    
          mch = (unsigned char *)mimage->imagedata;
          for (i=0; i<rows; i++)
            { 
              i_r     = i - rctr;
              i_rxi_r = i_r * i_r;
              for (j=0; j<cols; j++, ch++)
                {
                  if (*mch++)
                    {
                      j_c= j - cctr;
                      x  += j_c;
                      x2 += j_c * j_c;
                      xy += j_c * i_r;
                      y  += i_r;
                      y2 += i_rxi_r;
                      xz += j_c * (unsigned int)*ch;
                      yz += i_r * (unsigned int)*ch;
                      z  += (unsigned int)*ch;
                      n++;
                    }
                }
            }
          break;
        case VFF_TYP_2_BYTE:
          s = (short *)image->imagedata;    
          ms = (short *)mimage->imagedata;
          for (i=0; i<rows; i++)
            { 
              i_r     = i - rctr;
              i_rxi_r = i_r * i_r;
              for (j=0; j<cols; j++, s++)
                {
                  if (*ms++)
                    {
                      j_c= j - cctr;
                      x  += j_c;
                      x2 += j_c * j_c;
                      xy += j_c * i_r;
                      y  += i_r;
                      y2 += i_rxi_r;
                      xz += j_c * (*s);
                      yz += i_r * (*s);
                      z  += *s;
                      n++;
                    }
                }
            }
          break;
        case VFF_TYP_4_BYTE:
          it = (int *)image->imagedata;    
          mit = (int *)mimage->imagedata;
          for (i=0; i<rows; i++)
            { 
              i_r     = i - rctr;
              i_rxi_r = i_r * i_r;
              for (j=0; j<cols; j++, it++, mit++)
                {                                 /* i++ replaced by it++ */
                  if (*mit)
                    {
                      j_c = j - cctr;
                      x  += j_c;
                      x2 += j_c * j_c;
                      xy += j_c * i_r;
                      y  += i_r;
                      y2 += i_rxi_r;
                      xz += j_c * (*it);
                      yz += i_r * (*it);
                      z  += (*it);
                      n++;
                    }
                }
            }
          break;
        case VFF_TYP_FLOAT:
          f = (float *)image->imagedata;    
          mf = (float *)mimage->imagedata;
          for (i=0; i<rows; i++)
            { 
              i_r     = i - rctr;
              i_rxi_r = i_r * i_r;
              for (j=0; j<cols; j++, f++)
                {
                  if (*mf++)
                    {
                      j_c= j - cctr;
                      x  += j_c;
                      x2 += j_c * j_c;
                      xy += j_c * i_r;
                      y  += i_r;
                      y2 += i_rxi_r;
                      xz += j_c * (*f);
                      yz += i_r * (*f);
                      z  += *f;
                      n++;
                    }
                }
            }
          break;
        case VFF_TYP_COMPLEX:
          kerror("vipl","lvmafdtilt", "Cannot find tilt for COMPELX image!\n");
          return(FALSE);
          /* break; */
        default:
          kerror("vipl","lvmafdtilt", "Unknown data storage type: %d\n",
		 image->data_storage_type);
          return(FALSE);
          /* break; */
      }

    /* Find the matrix determinant */
    det = x2*(y2*n-y*y)-xy*(xy*n-y*x)+x*(xy*y-y2*x);

    /* Find the Cramer's determinants */
    deta = xz*(y2*n-y*y)-xy*(yz*n-y*z)+x*(yz*y-y2*z);
    detb = x2*(yz*n-y*z)-xz*(xy*n-x*y)+x*(xy*z-yz*x);
    detc = x2*(y2*z-yz*y)-xy*(xy*z-yz*x)+xz*(xy*y-y2*x);

    /* Compute the plane fit coefficients */
    *a = deta/det;
    *b = detb/det;
    *c = detc/det;

    return(TRUE);  /* success */
}
/* -library_code_end */
