 /*
  * Khoros: $Id: lvgrow.c,v 1.15.2.2 1997/05/31 20:16:46 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lvgrow.c,v 1.15.2.2 1997/05/31 20:16:46 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vgrow
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvgrow
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvgrow - Computes the median axis inverse transform.
* 
*       Purpose: lvgrow computes the median axis inverse transform.
*
*		 This transformation allows the recovery of the
*		 original image from a median axis image.
*
*		 This routine is useful for displaying the
*		 modifications introduced after any transformation
*		 of the median axis.
*
*		 The principle of this transform is to assign the
*		 value 255 to any pixel whose distance to at least
*		 one medial axis pixel, is less than or equal to
*		 the value of this pixel.
*
*		 The user is refered to the man pages of the median
*		 axis routine as an aid in understanding the inverse
*		 transform.
*
*		 For this routine, the image must be a byte image,
*		 with the gray level value of 0 for the background.
*		 The pixels that do not belong to the background are
*		 supposed to belong to a median axis, so it is advisable
*		 to use an image obtained from the median axis transform.
*         Input: image - an xvimage structure
*        Output: image - holds the result.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: vgrow works only on BYTE images that represent a
*		 coherent medial axis of an image.
*    Written By: Pascal Adam
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: Ghosted by Pascal ADAM Fri Jun  1 17:35:45 MDT 1990
*		 Bug fixed by Pascal ADAM Mon Jun  4 10:04:07 MDT 1990
****************************************************************/
/* -library_def */
int lvgrow(
	xvimage *image)
/* -library_def_end */

/* -library_code */
{

    unsigned char *cptr;

    int i, j, k, nc, nr, val;

    int fin;
 
    /* Check type,... of image */
    if (!(propertype("lvgrow", image, (unsigned long)VFF_TYP_1_BYTE, FALSE))) {
        kerror ("vipl", "lvgrow", "Image must be of type byte\n");
        return (FALSE);
    }
    if (!(proper_num_images ("lvgrow", image, (unsigned long) 1, FALSE))) {
        kerror ("vipl", "lvgrow", "Can only work on files with one image\n\n");
        return (FALSE);
    }
    if (!(proper_num_bands ("lvgrow", image, (unsigned long) 1, FALSE))) {
        kerror ("vipl", "lvgrow",
		"Can only work on images with 1 data band\n\n");
        return (FALSE);
    }

    nr = image->col_size;
    nc = image->row_size;


    cptr   = (unsigned char *) image->imagedata;


    /* Growing process ...... scanning of input image */
    /* until the growing process stops .............. */
    do 
    {
       fin = TRUE;
       for (i=1;i<nr;i++)
       {
           for (j=1;j<nc;j++)
           { 
               val = *(cptr+i*nc+j);
               switch(val) 
               {
                case 0: 
                case 1: 
                  break;

                default:
                  fin = FALSE;
                  val--;
                  *(cptr+i*nc+j) = val;

                  k= (i-1)*nc+j;
                  if ((k >= 0) || (k < nc*nr)) 
                  {
                     if (*(cptr+k) < (unsigned char) val ) *(cptr+k) = val;
                  }
                  else 
                  {
                    kinfo(KSTANDARD, "lvgrow failed\n \
The Input Image has to be a medial axis image\n");
                  }

                  k= (i+1)*nc+j;
                  if ((k >= 0) || (k < nc*nr)) 
                  {
                    if (*(cptr+k) < (unsigned char) val ) *(cptr+k) = val;
                  }
                  else 
                  {
                    kinfo(KSTANDARD, "lvgrow failed\n \
The Input Image has to be a medial axis image\n");
                  }

                  k= i*nc+j-1;
                  if ((k >= 0) || (k < nc*nr))
                  {
                    if (*(cptr+k) < (unsigned char) val ) *(cptr+k) = val;
                  }
                  else 
                  {
                    kinfo(KSTANDARD,"lvgrow failed\n \
The Input Image has to be a medial axis image\n");
                  }

                  k= i*nc+j+1;
                  if ((k >= 0) || (k < nc*nr)) 
                  {
                    if (*(cptr+k) < (unsigned char) val ) *(cptr+k) = val;
                  }
                  else 
                  {
                    kinfo(KSTANDARD,"lvgrow failed\n \
The Input Image has to be a medial axis image\n");
                  }
               }
           }
       }
    }
    while (fin == FALSE);
    
    cptr   = (unsigned char *) image->imagedata;

    for (i=0; i< nc*nr ;i++)  *(cptr+i) = *(cptr+i) * 255;

    return(TRUE);
        
}
/* -library_code_end */
