 /*
  * Khoros: $Id: lvshape.c,v 1.16.2.2 1997/05/31 20:17:18 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lvshape.c,v 1.16.2.2 1997/05/31 20:17:18 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vshape
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvshape
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
typedef struct xvshape  {

   double surf_image;   /* Number of pixels in the entire image ...... */
   double area;         /* Number of pixels in the object   = m00 .... */
   double percent_area; /* Ratio area / surf_image ................... */
   double real_area;    /* Area in square meters ..................... */
   double xcent;        /* x_cent of object .......................... */
   double ycent;        /* y_cent of object .......................... */
   double sigma_x;      /* variance = sqrt [ mu20 / m00 ] ............ */
   double sigma_y;      /*          = sqrt [ mu02 / m00 ] ............ */
                        /* Idem but on principal axis ................ */
   double sigma_MX;     /*          = sqrt [ MX / m00 ] .............. */
   double sigma_MY;     /*          = sqrt [ MY / m00 ] .............. */
   double theta;        /* angle between the major axis of the shape   */
                        /* and the horizontal ........................ */
                        /*                                             */
                        /*                                             */
                        /*                    |   2 mu11    |          */
                        /* theta = 0.5 arctan | ___________ |          */
                        /*                    |             |          */
                        /*                    | mu20 - mu02 |          */
                        /*                                             */
                        /*                                             */

   double eccentricity; /*  Measures the eccentricity of the figure    */
/*          mu02 cos(theta)^2 + mu20 sin(theta)^2 - mu11 sin(2 theta)  */
/*  = SQRT  _________________________________________________________  */
/*          mu02 sin(theta)^2 + mu20 cos(theta)^2 + mu11 cos(2 theta)  */
                        /*                                             */
                        /*                                             */
                        /* two dimensional (p+q)th order moments       */
                        /*                                             */
                        /*                  p  q                       */
                        /* m   = sum  sum  i  j   I(i,j) p,q = 0,1,2,..*/
                        /*  pq    i    i                               */ 
                        /*                                             */
                        /* I(i,j) is the grey level of the pixel, for  */
                        /* application where the grey level is a clus- */
                        /* ter number or a label number, the moments   */
                        /* will be computed for each "object" or class */
                        /* assuming that I(i,j) = 1 inside the class   */
                        /*                      = 0 outside the class  */
   double  m00;         /* represent the area if I(i,j) = 0 or 1       */
   double  m01;         /*    "      sum of y if ......                */
   double  m10;         /*    "      sum of x if ......                */
   double  m11;         /*    "      sum of xy if .....                */
   double  m20;         /*    "      sum of ...........                */
   double  m02;         /*    "      sum of ...........                */

                        /*                                             */
   double  m21;         /*    "      sum of ...........                */
   double  m12;         /*    "      sum of ...........                */
   double  m03;         /*    "      sum of ...........                */
   double  m30;         /*    "      sum of ...........                */

                        /* central moments ........................... */
                        /* If the coordinate system is shifted so that */
                        /* its origin coincides with the centroid x,y  */
                        /* the new set of moment are invariant under   */
                        /* the translation.............................*/
                        /*                                             */
                        /*                      _  p     _  q          */
                        /*  mu   = sum sum (i - x ) (j - y )   I(i,j)  */
                        /*    pq    i   j                              */
                        /*                                             */
                        /*                                             */
                        /*  Another way to compute mu   is:            */
                        /*                           pq                */
                        /*                                             */
                        /*           p   q  p  q    _ r   _ s          */
                        /*  mu  = sum sum  C  C  ( -x) ( -y) m         */
                        /*    pq   r=0 s=0  r  s              p-r,q-s  */
                        /*                                             */
                        /*   p        p!                               */
                        /*  C   = -----------                          */
                        /*   q    r! (p - r)!                          */
                        /*                                             */
                        
   double mu00;         
   double mu10;         
   double mu01;         
   double mu20;         
   double mu02;         
   double mu11;         
   double mu30;         
   double mu03;         
   double mu21;         
   double mu12;        
                        /* Normalized central moments ................ */
                        /*          mu                                 */
                        /*            pq                               */
                        /*  nu   =  _____     with k = (p+q)/2 + 1     */
                        /*    pq                                       */
                        /*              k                              */
                        /*          mu                                 */
                        /*            00                               */
   double nu00;         
   double nu10;         
   double nu01;         
   double nu20;         
   double nu02;         
   double nu11;         
   double nu30;         
   double nu03;         
   double nu21;         
   double nu12;        
                        /* moments invariants .......................  */ 
                        /* This set of momnents is invariant under     */
                        /* rotation, reflection, translation and size  */
    
   double M1;           /* M1 = mu20 + mu02     .....................  */
   double M2;           /* M2 = (mu20-mu02)^2 + 4*(mu11)^2  .........  */
   double M3;           /* M3 = (mu30 -3*mu12)^2 + (3*mu21 -mu03)^2 .  */
   double M4;           /* M4 = (mu30 + mu12)^2 + (mu03 + mu21)^2 ...  */
   double M5;           /* M5 =  ..... too long cf code .............  */
   double M6;           /* M6 =  ....................................  */ 
   double M7;           /* M7 =  ........ (attention: sensitive to     */
                        /*       reflection).........................  */
    int    right;	/* gives the rightmost point of the object     */
    int    left;	/* gives the leftmost point of the object      */
    int    top;		/* gives the uppermost point of the object     */
    int    bottom;	/* gives the lowest point of the object        */
  
    int    axis_x1deb;  /* gives moment on principal axis 1.           */
    int    axis_x1end;  /* gives moment on principal axis 1.           */
    int    axis_y1deb;  /* gives moment on principal axis 1.           */
    int    axis_y1end;  /* gives moment on principal axis 1.           */
    int    axis_x2deb;  /* gives moment on principal axis 2.           */
    int    axis_x2end;  /* gives moment on principal axis 2.           */
    int    axis_y2deb;  /* gives moment on principal axis 2.           */
    int    axis_y2end;  /* gives moment on principal axis 2.           */

} XVSHAPE;
#define         ALL_OPTS        61
static void draw_line  PROTO((int, int, int, int, int *, int, int, int));
static void comp_shape PROTO((double, XVSHAPE **, int, int, int, int, int));
static int printout    PROTO((int, XVSHAPE **, int, int, int, int, char *,
			      kfile *));
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvshape - Performs shape analysis on images.
* 
*       Purpose: lvshape performs a shape analysis on two different types of
*		 images, by extracting features like moments, area, centroids,
*		 etc.
*
*		 The input "image" is a pointer to an INT xvimage that can be:
*
*		 A grey level image -- In this particular case, the features
*		 will be computed for the entire image.  Some of these features
*		 may not appear to make sense, nevertheless they will still
*		 contribute significant information to the regions.
*
*		 A labeled or clustered image -- The grey level of a pixel in
*		 a labeled or clustered image is specifically used to assign a
*		 pixel to a particular labeled region.  For a labeled image
*		 (output of vlabel ipl routine), the classes or regions are
*		 connected, however this is not true for a cluster number image
*		 (output of vquant, vkmeans or visodata).  This difference can
*		 confuse a novice user, because vshape will not differentiate
*		 between connected or non-connected regions.
*
*		 General Description of lvshape.
*
*		 The features listed below are computed for each region or for 
*		 the entire image, depending on the input image.
*
*		 ! - \fBCentroid\fP on x and y axis.
*		 ! - \fBVariances\fP on x and y.
*		 ! - \fBArea\fP or \fBweighted area\fP.
*		 ! - \fBOrientation\fP of the principal axis.
*		 ! - \fBEccentricity\fP of the shape.
*		 ! - \fBBounded box\fP coordinates.
*
*		 Theses features are optional.
*
*		 ! - \fBStandard Moments\fP.
*		 ! - \fBCentral Moments\fP and \fBNormalized Central Moments\fP.
*		 ! - \fBInvariant Moments\fP.
*
*		 These features are provided in an optional ASCII file.
*
*		 As mentioned above, the centroid coordinates of the grey level
*		 input image provides more information than the general sense of
*		 the centroid of a region.
*
*		 In addition to the ASCII file, three other output files can be
*		 generated.
*
*		 The Axis Image -- This image contains a representation of the
*		 moments of the regions contained in the input image. It can
*		 only be generated when the input image is a not a grey level
*		 image.  There are two possibilities for representing the
*		 moment.
*
*		 The first one is an image containing a cross for each region.
*		 The cross center is located at the centroid of the region
*		 and the horizontal and vertical axes stop outside of
*		 the bounded box of the region.
*
*		 The second one is an image still containing a cross for each
*		 region but this time the cross is representative of the
*		 moment value in the principal axis system. The principal
*		 axis is the coordinate system in which the cross moment,
*		 m11 (cf reference for theory on moment below), is equal
*		 to zero. Therefore, this cross forms an angle (theta) with the
*		 x axis, and its length in both directions are proportional to
*		 the "length" of the shape is that direction.
*
*		 The Outline Image -- This image contains the outlines of all
*		 the regions contained in this image. Again, this can only be
*		 generated if this input image is not a grey level image.
*
*		 The VIFF Image -- The image contains all of the statistics.
*		 The VIFF output file has 61 (number of statistics) bands and
*		 the number of vectors in this file depends on the number of
*		 regions specified in the input image. If the statistics flags
*		 are not set, the corresponding statistics will default to zero.
*
*		 For these three output images there is a final possibility,
*		 which is to assign all outlines or crosses a grey level of 255,
*		 or their previous region number.  This enables the user to
*		 visualize the input image, the outline image, and the axis
*		 image using pseudo color to assign a different color to each
*		 region.  This aids in interpreting the regions, and yields an
*		 artistic view of the image.
*
*		 References: 
*
*		 Vision In Man And Machine  by
*		 Martin D. Levine. Chapter 10. "Shape". Page 480.  
*
*		 Digital Image Processing by
*		 R.C. Gonzales & P. Wintz. Chapter "Representation and
*		 description".  Page 418.
*
*		 IRE Transactions on information theory,vol IT-8,1962
*		 "Visual Pattern Recognition by Moment Invariants" by 
*		 Ming_Kuei Hu. Page 179.
*
*         Input: image - input image
*		 image_type - if 0, image containing clusters or
*			      labelled objs. if 1, grey level image.
*		 axis_flg - if 1, generates the image with
*			     axis representation.
*		 outline_flg - if 1, generates the image with
*			       outlines representation.
*		 file_flg -  if 1, generates the ASCII file
*			     with objs or image information.
*		 input_file - name of input file being analyzed
*		 printdev -  output device for printing
*		 print_flg - print to printdev if set.
*		 standard - if 1, computes and prints standard moments.
*		 central - if 1, computes and prints central moments.
*		 invariants - if 1, computes and prints invariant moments.
*        Output: image - holds the resulting image.
*		 axis - xvimage structure, axis image.
*		 outline - xvimage structure, outline image.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: For the x and y representation, x represents the vertical
*		 axis and y the horizontal axis. This routine implies a lot of
*	 	 mathematical computation in a direct coordinate system, I
*		 didn't use the conventional x and y.
*    Written By: Pascal ADAM, Shirley Lee
*          Date: July 1, 1997
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: Ghosted by Pascal ADAM  Fri Jun 15 13:38:24 MDT 1990
*		 Modified vshape to output VIFF file format by Shirley Lee
*		 Feb/26/1991.
****************************************************************/
/* -library_def */
int lvshape(
	int image_type,
	xvimage *image,
	int typ_axe,
	int typ_color,
	int axis_flg,
	xvimage **axis,
	int outline_flg,
	xvimage **outline,
	int viff_flg,
	xvimage **img3,
	int file_flg,
	char *input_file,
	kfile *printdev,
	int standard,
	int central,
	int invariant)
/* -library_def_end */

/* -library_code */
{
xvimage *o1_img,
        *o2_img,
        *o3_img;
int  nc, 
     nr, 
     color,
     row, 
     col, 
     row_deb, 
     row_end, 
     col_deb, 
     col_end, 
     ind, 
     numreg, 
     num;

char *dummy = '\0';

int *ptr, 
    *ptr_line, 
    *ptr_axe;

double pixel, 
       a1, 
       a2, 
       meters_surf;


int i;
float **tmp_obj;

XVSHAPE  **array_obj;



   nc = image->row_size;
   nr = image->col_size;

   /* Allocate space for output image (axis image) */

   o2_img=NULL;
   if (axis_flg == TRUE) 
   {
      o2_img  = createimage((unsigned long) nr,
                          (unsigned long) nc, 
                          (unsigned long) VFF_TYP_4_BYTE,
                          (unsigned long) 1,
                          (unsigned long) 1,
                          dummy, 
                          (unsigned long) 0, 
                          (unsigned long) 0, 
                          (unsigned long) VFF_MS_NONE,
                          (unsigned long) VFF_MAPTYP_NONE, 
                          (unsigned long) VFF_LOC_IMPLICIT, 
                          (unsigned long) 0);

      if (o2_img  == NULL) 
      {
         kerror("vipl", "lvshape", "Unable to allocate new image!\n\n");
         return(FALSE);
      }
   }

   /* Allocate space for output image (Outlines image) */

   o1_img = NULL;
   if (outline_flg == TRUE)
   {
      o1_img = createimage((unsigned long) nr,
                           (unsigned long)  nc, 
                           (unsigned long)  VFF_TYP_4_BYTE, 
                           (unsigned long)  1,
                           (unsigned long)  1,
                           dummy, 
                           (unsigned long)  0, 
                           (unsigned long)  0, 
                           (unsigned long)  VFF_MS_NONE,
                           (unsigned long)  VFF_MAPTYP_NONE, 
                           (unsigned long)  VFF_LOC_IMPLICIT, 
                           (unsigned long)  0);

      if (o1_img == NULL) 
      {
         kerror("vipl", "lvshape", "Unable to allocate new image!\n\n");
         return(FALSE);
      }
   }


   /* assign image data address to ptr.................................. */
   ptr = (int  *) image->imagedata;

   /* computes the square meters of a pixel with info: pixsizx, pixsizy  */
   meters_surf = (double) (image->pixsizx * image->pixsizy); 

   /* 2 differents cases can occur...................................... */
   /*  -  1.  the user had a grey level as input .................... */
   /*         In this case the moment mpq are computed with respect to   */
   /*         the grey level of the pixel at this point. cf vshape.h     */
   /*  -  2.  The user had an image in which the value of the pixel      */
   /*         represents a class (cluster image or labelled image).      */
   /*         This time the algorithm will compute the moments... for    */
   /*         each class.  ............................................. */
   /*                                                                    */
   /* ____ These different cases are processed beyond .................. */
   
   /* Computation of mpq for a grey level image ........................ */
   if (image_type == 0) { /* grey level image  ......................... */

      numreg = 1;  /* in this case there is only one region ............ */

      /* allocate memory for numreg XVSHAPE pointed by array_obj ... */
      array_obj= (XVSHAPE  **) kmalloc(sizeof(XVSHAPE *)*numreg);
      if (array_obj == NULL) {
         kerror("vipl","lvshape","No space for array_obj - malloc failed! \n");
         return(FALSE);
          }
      for (ind = 0; ind < numreg; ind++) {
         array_obj[ind] = (XVSHAPE *)kmalloc(sizeof(XVSHAPE));
         if (array_obj[ind] == NULL) {
            kerror("vipl", "lvshape",
		   "No space for array_obj[%d] - malloc failed!\n",
                ind);
            return(FALSE);
            }

      /* initialisation ................................................ */
      kbzero((char *)(array_obj[ind]),sizeof(XVSHAPE)); /* BUGFIX */
      array_obj[ind]->right = array_obj[ind]->bottom = 0;
      array_obj[ind]->left = nc;
      array_obj[ind]->top  = nr;
      }

      for(row = 0; row < nr; row++){
         for(col = 0; col < nc; col++){
            pixel = (double)ptr[row * nc + col];
            a1    = pixel * (double)col;
            a2    = pixel * (double)row;
            array_obj[0]->m00 += pixel; 
            array_obj[0]->m01 += a1; 
            array_obj[0]->m10 += a2;
            array_obj[0]->m11 += a1 * (double)row; 
            a1   *= (double)col;
            a2   *= (double)row; 
            array_obj[0]->m02 += a1; 
            array_obj[0]->m20 += a2; 
            array_obj[0]->m12 += a1 * (double)row; 
            array_obj[0]->m21 += a2 * (double)col; 
            array_obj[0]->m03 += a1 * (double)col; 
            array_obj[0]->m30 += a2 * (double)row; 
            }
         }  
      }

   /* Computation of mpq for a cluster or labelled image ............... */
   else {  
     
      /* scan image to find number of different regions or clusters .... */
      numreg = 0;
      for(ind = 0; ind < nc*nr; ind++) {
         if (ptr[ind] > numreg) 
           numreg = ptr[ind];
      }

      numreg++;  /* increment numreg to get the total number of reg ... */
 
      /* allocate memory for numreg XVSHAPE pointed by array_obj ... */
      array_obj=(XVSHAPE  **)kmalloc(sizeof(XVSHAPE *)*numreg); 
      if (array_obj == NULL) {
         kerror("vipl", "lvshape", "No space for array_obj - malloc failed!\n");
         return(FALSE);
          }

      for(ind = 0; ind < numreg; ind++) {
         array_obj[ind]=(XVSHAPE *)kmalloc(sizeof(XVSHAPE));

         if (array_obj[ind] == NULL) {
            kerror("vipl", "lvshape",
		   "No space for array_obj[%d] - malloc failed!\n", ind);
            return(FALSE);
            }
        
         /* initialisation ............................................. */
         kbzero((char *)(array_obj[ind]),sizeof(XVSHAPE)); /* BUGFIX */
         array_obj[ind]->right = array_obj[ind]->bottom = 0;
         array_obj[ind]->left = nc;
         array_obj[ind]->top  = nr;
         }

      /* scan the image and compute mpq for each regionodr cluster  */
      for(row = 0; row < nr; row++){
         for(col = 0; col < nc; col++){

            ind = ptr[row * nc + col]; /* ind contains the class number .. */

            /* gets the top, bottom, right and left border coord of obj. */
            if (array_obj[ind]->right<col) array_obj[ind]->right=col;
            if (array_obj[ind]->left>col) array_obj[ind]->left=col;
            if (array_obj[ind]->bottom<row) array_obj[ind]->bottom=row;
            if (array_obj[ind]->top>row) array_obj[ind]->top=row;

        
            array_obj[ind]->m00 += 1.0; 

            a1 = (double)col; a2 = (double)row;
            array_obj[ind]->m01 += a1;
            array_obj[ind]->m10 += a2; 
            array_obj[ind]->m11 += a1 * a2; 
            a1 *= a1; a2 *= a2;
            array_obj[ind]->m02 += a1; 
            array_obj[ind]->m20 += a2; 
            array_obj[ind]->m12 += (double)row * a1; 
            array_obj[ind]->m21 += (double)col * a2;
            array_obj[ind]->m03 += (double)col * a1;
            array_obj[ind]->m30 += (double)row * a2; 
            }
         }
      }
      
   /* In both case we know the information on mpq .................. */
   /* Let's compute all the other fields of the XVSHAPE ............ */

   (void)comp_shape( meters_surf,array_obj,numreg,central,invariant,nc, nr); 

   /* print out the results ........................................ */
   if (file_flg) printout(image_type, array_obj,numreg, standard, central, 
                          invariant, input_file, printdev);

   /* create output image files ONLY if image_type==1 (cannot represent */
   /* axis or outline when a grey level image is the input image....... */
   if ((image_type == 1) && (axis_flg == 1)) {
      ptr_axe = (int *)o2_img->imagedata;
      /* give to the background image its values ...................... */
      for (row = 0; row < nr ; row++) {
         for (col = 0; col < nc ; col++) {
            ptr_axe[row * nc + col] = 0;
            }
         }

      /* write  y&x axis of the found objs over the background image */ 
      for (ind = 1; ind < numreg; ind++) {
         if (typ_color == 0) color = 255; 
         else color = ind; 

         if (typ_axe == 1) {    
            draw_line(array_obj[ind]->axis_x1deb, 
                      array_obj[ind]->axis_y1deb,
                      array_obj[ind]->axis_x1end,
                      array_obj[ind]->axis_y1end,
                      ptr_axe,
                      nc,
                      nr,
                      color);

            draw_line(array_obj[ind]->axis_x2deb,
                      array_obj[ind]->axis_y2deb,
                      array_obj[ind]->axis_x2end,
                      array_obj[ind]->axis_y2end,
                      ptr_axe,
                      nc,
                      nr,
                      color);
            }
        
         else {
            row_deb = (int)(array_obj[ind]->top);
            row_end = (int)(array_obj[ind]->bottom);
            col_deb = (int)(array_obj[ind]->left);
            col_end = (int)(array_obj[ind]->right);
            for ( row = row_deb; row < row_end; row++) {
               for ( col = col_deb; col < col_end; col++) {
                  if ( (col == (int)(array_obj[ind]->ycent)) ||
                       (row == (int)(array_obj[ind]->xcent))  )
                     ptr_axe[row * nc + col] = color; 
                  }
               }
            }
         }
         *axis=o2_img;
      }
     
   if ((image_type == 1) && (outline_flg == 1)) {
      ptr_line = (int *)o1_img->imagedata;
      /* give to the background image its values .................. */
      for (row = 0; row < nr ; row++) {
         for (col = 0; col < nc ; col++) {
            ptr_line[row * nc + col] = 0;
            }
         }


      for ( row = 1; row < nr-1; row++){
         for ( col = 1; col < nc-1; col++){

            num = ptr[ row * nc + col];
            if (typ_color == 0) color = 255;
            else color = num;

            if ((ptr[ row * nc + col -1] != num) || 
                (ptr[ row * nc + col +1] != num) ||
                (ptr[(row -1)* nc + col] != num) ||
                (ptr[(row +1)* nc + col] != num)  )  
               ptr_line[row * nc + col] = color;
               
            }
         }
         *outline=o1_img;
      }

      if (viff_flg)  /* output in VIFF file format */ {

        o3_img = createimage((unsigned long) 1,      
                         (unsigned long) (numreg-1),        
                         (unsigned long) VFF_TYP_FLOAT,
                         (unsigned long) 1,      
                         (unsigned long) ALL_OPTS,      
                         dummy,                 
                         (unsigned long) 0,           
                         (unsigned long) 0,            
                         (unsigned long) VFF_MS_NONE,    
                         (unsigned long) VFF_MAPTYP_NONE, 
                         (unsigned long) VFF_LOC_IMPLICIT, 
                         (unsigned long) 0                
                        );

        if (o3_img == NULL)
        {
          kerror("vipl", "lvshape", "Unable to allocate new image!\n\n");
          return(FALSE);
        }

        tmp_obj = (float **)kcalloc((size_t) numreg,sizeof(float*));

        if (tmp_obj == NULL) {
           kerror("vipl", "lvshape", "No space for tmp_obj - malloc failed!\n");
            return(FALSE);
        }

        for (i=0; i<numreg; i++) {
          tmp_obj[i] = (float *)kcalloc((size_t) ALL_OPTS,sizeof(float));

          if (tmp_obj[i] == NULL) {
            kerror("vipl", "lvshape",
		   "No space for tmp_obj[%d] - malloc failed!\n", i);
             return(FALSE);
          }
        }

        /* change the array_obj[] struct to 2D array */

        for (i=0; i<(numreg-1); i++){
                tmp_obj[i][0] = (float)array_obj[i+1]->surf_image;
                tmp_obj[i][1] = (float)array_obj[i+1]->area;
                tmp_obj[i][2] = (float)array_obj[i+1]->percent_area;
                tmp_obj[i][3] = (float)array_obj[i+1]->real_area;
                tmp_obj[i][4] = (float)array_obj[i+1]->xcent;
                tmp_obj[i][5] = (float)array_obj[i+1]->ycent;
                tmp_obj[i][6] = (float)array_obj[i+1]->sigma_x;
                tmp_obj[i][7] = (float)array_obj[i+1]->sigma_y;
                tmp_obj[i][8] = (float)array_obj[i+1]->sigma_MX;
                tmp_obj[i][9] = (float)array_obj[i+1]->sigma_MY;
                tmp_obj[i][10] = (float)array_obj[i+1]->theta;
                tmp_obj[i][11] = (float)array_obj[i+1]->eccentricity;
                tmp_obj[i][12] = (float)array_obj[i+1]->m00;
                tmp_obj[i][13] = (float)array_obj[i+1]->m01;
                tmp_obj[i][14] = (float)array_obj[i+1]->m10;
                tmp_obj[i][15] = (float)array_obj[i+1]->m11;
                tmp_obj[i][16] = (float)array_obj[i+1]->m20;
                tmp_obj[i][17] = (float)array_obj[i+1]->m02;
                tmp_obj[i][18] = (float)array_obj[i+1]->m21;
                tmp_obj[i][19] = (float)array_obj[i+1]->m12;
                tmp_obj[i][20] = (float)array_obj[i+1]->m03;
                tmp_obj[i][21] = (float)array_obj[i+1]->m30;
                tmp_obj[i][22] = (float)array_obj[i+1]->mu00;
                tmp_obj[i][23] = (float)array_obj[i+1]->mu10;
                tmp_obj[i][24] = (float)array_obj[i+1]->mu01;
                tmp_obj[i][25] = (float)array_obj[i+1]->mu20;
                tmp_obj[i][26] = (float)array_obj[i+1]->mu02;
                tmp_obj[i][27] = (float)array_obj[i+1]->mu11;
                tmp_obj[i][28] = (float)array_obj[i+1]->mu30;
                tmp_obj[i][29] = (float)array_obj[i+1]->mu03;
                tmp_obj[i][30] = (float)array_obj[i+1]->mu21;
                tmp_obj[i][31] = (float)array_obj[i+1]->mu12;
                tmp_obj[i][32] = (float)array_obj[i+1]->nu00;
                tmp_obj[i][33] = (float)array_obj[i+1]->nu10;
                tmp_obj[i][34] = (float)array_obj[i+1]->nu01;
                tmp_obj[i][35] = (float)array_obj[i+1]->nu20;
                tmp_obj[i][36] = (float)array_obj[i+1]->nu02;
                tmp_obj[i][37] = (float)array_obj[i+1]->nu11;
                tmp_obj[i][38] = (float)array_obj[i+1]->nu30;
                tmp_obj[i][39] = (float)array_obj[i+1]->nu03;
                tmp_obj[i][40] = (float)array_obj[i+1]->nu21;
                tmp_obj[i][41] = (float)array_obj[i+1]->nu12;
                tmp_obj[i][42] = (float)array_obj[i+1]->M1;
                tmp_obj[i][43] = (float)array_obj[i+1]->M2;
                tmp_obj[i][44] = (float)array_obj[i+1]->M3;
                tmp_obj[i][45] = (float)array_obj[i+1]->M4;
                tmp_obj[i][46] = (float)array_obj[i+1]->M5;
                tmp_obj[i][47] = (float)array_obj[i+1]->M6;
                tmp_obj[i][48] = (float)array_obj[i+1]->M7;
                tmp_obj[i][49] = (float)array_obj[i+1]->right;
                tmp_obj[i][50] = (float)array_obj[i+1]->left;
                tmp_obj[i][51] = (float)array_obj[i+1]->top;
                tmp_obj[i][52] = (float)array_obj[i+1]->bottom;
                tmp_obj[i][53] = (float)array_obj[i+1]->axis_x1deb;
                tmp_obj[i][54] = (float)array_obj[i+1]->axis_x1end;
                tmp_obj[i][55] = (float)array_obj[i+1]->axis_y1deb;
                tmp_obj[i][56] = (float)array_obj[i+1]->axis_y1end;
                tmp_obj[i][57] = (float)array_obj[i+1]->axis_x2deb;
                tmp_obj[i][58] = (float)array_obj[i+1]->axis_x2end;
                tmp_obj[i][59] = (float)array_obj[i+1]->axis_y2deb;
                tmp_obj[i][60] = (float)array_obj[i+1]->axis_y2end;
          }

        kfree_and_NULL(o3_img->imagedata);

        o3_img->imagedata = (char *)unload_vector( (char **) tmp_obj,0,VFF_TYP_FLOAT,(numreg-1),ALL_OPTS,1,(numreg-1)); 

        *img3 = o3_img;
        } /* viff output */

        return(TRUE);
} 



/* draws a line kbetween ((double)a1,(double)b1) and (a2,b2) in the image data pointed */
/* by the color of this line is color ............................... */
static void draw_line(
	int a1,
	int b1,
	int a2,
	int b2,
	int *ptr,
	int nc,
	int nr,
	int color)
{
float slope;
float deb;

int row, col;

   if (a1 == a2) {
     for (col = (int)b1; col <= (int)b2; col++) {
         row = (int) a1;
         ptr[row * nc + col] = color;
         }
     }
   else {
      slope = (float)(b1 - b2) / (float)(a1 - a2);
      deb   = ((float) a1 * (float) b2 - (float) a2 * (float) b1) /
              (float)(a1 - a2);
  
      /* the equation of the line segment is : y = slope * x + deb . */ 

      /* to draw the line we must take care of the slope .... 2 cases . */
      if ((slope < 1.0) && (slope > -1.0)) {
         for (row = (int) a1; row <= (int) a2; row++) {
            col  = (int) ( (float) ( (float)(row) * slope  + deb ) );
            ptr[row * nc + col] = color;
            }
         }
      else {
         for (col = (int) b1; col <= (int)b2; col++ ) {
            row = (int) ( (float)( ( (float)(col) - deb) / slope) );
            ptr[row * nc + col] = color;
            }
         }
      }
   }

    
/* Computes area, x_cent, y_cent, variance, sigma_x, sigma_y, theta, eccentri-*/
/* city, and if required the central and invariant moments....................*/
static void comp_shape(
	double meters_surf,
	XVSHAPE **obj,
	int numreg,
	int central,
	int invariant,
	int nc,
	int nr)
{
#define POWER2(x) ((double)(x) * (double)(x))
int ind;  
double m00,m10,m01,m11,m02,m20,m21,m12,m03,m30,
       mu11,mu20,mu02,mu12,mu21,mu30,mu03,
       nu11,nu20,nu02,nu12,nu21,nu30,nu03,
       xm,ym,xm2,ym2,
       thet,cos2,sin2,cos_2,sin_2,
       a1,a2,a3,a4,a5,a6,a7,a8,a9,
       puissance;

     /* computes area, x_centroid, y_centroid ........................... */
     for(ind = 0; ind < numreg; ind++) {

        obj[ind]->surf_image = nc * nr;

        m00 = obj[ind]->area = obj[ind]->m00;


        obj[ind]->percent_area = obj[ind]->area*100.0 / obj[ind]->surf_image;

        obj[ind]->real_area = obj[ind]->area * meters_surf;

        if (m00 != 0.0) {
           m01 = obj[ind]->m01;
           m10 = obj[ind]->m10;
           m02 = obj[ind]->m02;
           m20 = obj[ind]->m20;
           m11 = obj[ind]->m11;
           m21 = obj[ind]->m21;
           m12 = obj[ind]->m12;
           m30 = obj[ind]->m30;
           m03 = obj[ind]->m03;
        

           xm = m10 / m00;
           obj[ind]->xcent = xm;

           ym = m01 / m00;
           obj[ind]->ycent = ym;
  
           mu20 = m20 - ( m00 * POWER2( xm ) ) ;

           mu02 = m02 - ( m00 * POWER2( ym ) ) ;

           mu11 = m11 - ( m00 * xm * ym );

           obj[ind]->sigma_x = sqrt( mu20 / m00 ); 
           obj[ind]->sigma_y = sqrt( mu02 / m00 ); 

           if (mu20 == mu02) {
              thet = KPI_4; 
              obj[ind]->theta = thet;
              }
           else { 
              thet = ((double) 2.0) * mu11 / (mu20 - mu02) ;
              thet = ((double)0.5)* atan( thet ); 
              obj[ind]->theta = thet;
              }

           cos2  = POWER2((cos( thet )) );
           sin2  = POWER2((sin( thet )) );
           cos_2 = cos( (((double)2.0)*thet) );
           sin_2 = sin( (((double)2.0)*thet) );  

           /* computes the eccentricity of the shape......*/
           a1    = mu02 * cos2 + mu20 * sin2 - mu11 * sin_2;
           a2    = mu02 * sin2 + mu20 * cos2 + mu11 * cos_2; 
           if ((a1 == (double)0.0) || (a2 == (double)0.0)) {
              obj[ind]->eccentricity = (double) -1.0;
              }
           else if ((a1/a2) > 0) obj[ind]->eccentricity = sqrt(a1/a2);
                else obj[ind]->eccentricity = sqrt(-a1/a2);


           /* computes the moments variance on X axis (1st principal axis */
           if (m00 == (double)0.0) {
              obj[ind]->sigma_MX = (double) -1.0;
              obj[ind]->sigma_MY = (double) -1.0;
              }
           else {
              obj[ind]->sigma_MX = sqrt((cos2*mu20+sin2*mu02+sin_2*mu11)/m00);
              obj[ind]->sigma_MY = sqrt((sin2*mu20+cos2*mu02-sin_2*mu11)/m00);
              }

           /* coeff 1.5 to make the axis bigger in the axis image */
           a1 = (double)(1.50) * cos(thet) * obj[ind]->sigma_MX; 
           a2 = (double)(1.50) * sin(thet) * obj[ind]->sigma_MX;
           obj[ind]->axis_x1deb = obj[ind]->xcent - (int)a1;
           if (obj[ind]->axis_x1deb < (double)0.0) 
              obj[ind]->axis_x1deb = (double)2.0;
           obj[ind]->axis_x1end = obj[ind]->xcent + (int)a1;
           if (obj[ind]->axis_x1end > (double)nr) 
              obj[ind]->axis_x1end = (double)(nr-2);
           obj[ind]->axis_y1deb = obj[ind]->ycent - (int)a2;
           if (obj[ind]->axis_y1deb < (double)0.0) 
              obj[ind]->axis_y1deb = (double)2.0;
           obj[ind]->axis_y1end = obj[ind]->ycent + (int)a2;
           if (obj[ind]->axis_y1end > (double)nc) 
              obj[ind]->axis_y1end = (double)(nc -2);

           a1 = -(double)(1.50) * sin(thet)*obj[ind]->sigma_MY;
           a2 =  (double)(1.50) * cos(thet)*obj[ind]->sigma_MY;
           obj[ind]->axis_x2deb = obj[ind]->xcent - (int)a1;
           if (obj[ind]->axis_x1deb < (double)0.0)
              obj[ind]->axis_x1deb = (double)2.0;
           obj[ind]->axis_x2end = obj[ind]->xcent + (int)a1;
           if (obj[ind]->axis_x1end > (double)nr)
              obj[ind]->axis_x1end = (double)(nr-2);
           obj[ind]->axis_y2deb = obj[ind]->ycent - (int)a2;
           if (obj[ind]->axis_y1deb < (double)0.0)
              obj[ind]->axis_y1deb = (double)2.0;
           obj[ind]->axis_y2end = obj[ind]->ycent + (int)a2;
           if (obj[ind]->axis_y1end > (double)nc)
              obj[ind]->axis_y1end = (double)(nc -2);


           if (central == 1 || invariant == 1) {

              /* pre_computation */
              xm2 = POWER2( xm );
              ym2 = POWER2( ym );

              /* The central moments of order 3 are as follows */
              obj[ind]->mu00 = m00;

              obj[ind]->mu10 = obj[ind]->mu01 = 0.0;

              obj[ind]->mu11 = mu11;
              obj[ind]->mu20 = mu20;
              obj[ind]->mu02 = mu02;

              mu30 = m30 - ((double)3.0)*xm*m20 +((double)2.0)*m10*xm2;
              obj[ind]->mu30 = mu30;

              mu03 = m03 - ((double)3.0)*ym*m02 +((double)2.0)*m01*ym2;
              obj[ind]->mu03 = mu03;

              mu21 = m21-m20*ym-((double)2.0)*m11*xm+((double)2.0)*xm2*m01; 
              obj[ind]->mu21 = mu21;
 
              mu12 = m12 -m02*xm - ((double)2.0)*m11*ym + ((double)2.0)*ym2*m10; 
              obj[ind]->mu12 = mu12;
              }

           if (invariant == 1) {
/*
              * The set of moments that are invariant to translation, 
              * rotation and scale change ( HU [1962] ) are given by 
              * the following relations
              */
              /* Pre-computations ..................................*/
              obj[ind]->nu00 = (double)1.0;
              obj[ind]->nu01 = (double)0.0;
              obj[ind]->nu10 = (double)0.0;

           
              if (m00 == (double)0.0) {

/*
                 nu02 = obj[ind]->nu02 = (double) -1.0; 
                 nu20 = obj[ind]->nu20 = (double) -1.0; 
                 nu11 = obj[ind]->nu11 = (double) -1.0; 

                 nu12 = obj[ind]->nu12 = (double) -1.0;
                 nu21 = obj[ind]->nu21 = (double) -1.0;
                 nu03 = obj[ind]->nu03 = (double) -1.0; 
                 nu30 = obj[ind]->nu30 = (double) -1.0;

                 a1  =(double) -1.0; 
                 a2  =(double) -1.0; 
                 a3  =(double) -1.0; 
                 a4  =(double) -1.0; 
                 a5  =(double) -1.0; 
                 a6  =(double) -1.0; 
                 a7  =(double) -1.0; 
                 a8  =(double) -1.0; 
                 a9  =(double) -1.0; 
              */

                 obj[ind]->M1  = (double) -1.0; 
                 obj[ind]->M2  = (double) -1.0; 
                 obj[ind]->M3  = (double) -1.0; 
                 obj[ind]->M4  = (double) -1.0; 
                 obj[ind]->M5  = (double) -1.0; 
                 obj[ind]->M6  = (double) -1.0; 
                 obj[ind]->M7  = (double) -1.0; 
                 }

              else {

                 puissance      = m00 * m00;
                 nu02 = obj[ind]->nu02 = mu02 / puissance;
                 nu20 = obj[ind]->nu20 = mu20 / puissance;
                 nu11 = obj[ind]->nu11 = mu11 / puissance;

                 puissance      = pow(m00,(double)2.5);
                 nu12 = obj[ind]->nu12 = mu12 / puissance;
                 nu21 = obj[ind]->nu21 = mu21 / puissance;
                 nu03 = obj[ind]->nu03 = mu03 / puissance;
                 nu30 = obj[ind]->nu30 = mu30 / puissance;

                 a1  = nu20 - nu02;
                 a2  = nu30 + nu12;
                 a3  = nu21 + nu03;
                 a4  = nu30 - 3.0 * nu12;
                 a5  = ((double)3.0) * nu21 - nu03;
                 a6  = POWER2(a2);
                 a7  = POWER2(a3);
                 a8  = a2 * (a6 - ((double)3.0) * a7);
                 a9  = a3 * (((double)3.0) * a6 - a7);

                 obj[ind]->M1  = nu20 + nu02;
                 obj[ind]->M2  = POWER2 (a1) + ((double)4.0) * POWER2(nu11);
                 obj[ind]->M3  = POWER2 (a4) + POWER2(a5);
                 obj[ind]->M4  = a6 +a7;
                 obj[ind]->M5  = a4 *a8  +a5 *a9;
                 obj[ind]->M6  = a1 *(a6 -a7) +((double)4.0) *nu11 *a2 *a3;
                 obj[ind]->M7 =  ((double)3.0 *nu21 - nu30) *a8 -a4 *a9;
                 }
              }
           }
        }
     }

/* Print information in INPUT kfile *********************************/
static int printout(
	int typ_im,
	XVSHAPE **obj,
	int numreg,
	int standard,
	int central,
	int invariant,
	char *input_file,
	kfile *printdev)
{
    int   i,new_i,i_deb;

    (void) kfprintf(printdev,"\nShape Information For File Name: %s\n",
                   input_file);
    new_i = 0;

    if (typ_im == 0 ) i_deb=0; /* if grey level input image print region 0 */
    else i_deb = 1;            /* if labelled image doesn't print region 0 */
    for (i = i_deb ; i < numreg ; i++) { 
     if (obj[i]->area != 0.0) {
      (void) kfprintf(printdev,"\"Object number  %d \"\n",new_i);        
      (void) kfprintf(printdev,
      "Entire Image    :  %.0f pixels \n",obj[i]->surf_image); 
      (void) kfprintf(printdev,
      "Object          :  %.0f pixels \n",obj[i]->area); 
      (void) kfprintf(printdev,
      "Ratio Obj/Image :  %.2f %%\n",obj[i]->percent_area); 
      (void) kfprintf(printdev,
      "Area            :  %e square meters \n",obj[i]->area); 
      (void) kfprintf(printdev,"Xcenter         :  %e\n",obj[i]->xcent); 
      (void) kfprintf(printdev,"Ycenter         :  %e\n",obj[i]->ycent); 
      (void) kfprintf(printdev,"Sigma_x         :  %e\n",obj[i]->sigma_x); 
      (void) kfprintf(printdev,"Sigma_y         :  %e\n",obj[i]->sigma_y); 
      (void) kfprintf(printdev,"Theta           :  %e\n",obj[i]->theta); 
      (void) kfprintf(printdev,"Eccentricity    :  %e\n",obj[i]->eccentricity); 
      (void) kfprintf(printdev,"Uppermost point :  %d\n",obj[i]->top); 
      (void) kfprintf(printdev,"Leftmost point  :  %d\n",obj[i]->left); 
      (void) kfprintf(printdev,"Lowest point    :  %d\n",obj[i]->bottom); 
      (void) kfprintf(printdev,"Rightmost point :  %d\n\n",obj[i]->right); 

      if (standard == 1) {
         (void) kfprintf(printdev,"Standard Moments\n");
         (void) kfprintf(printdev,"m00             :  %e\n",obj[i]->m00); 
         (void) kfprintf(printdev,"m10             :  %e\n",obj[i]->m10); 
         (void) kfprintf(printdev,"m01             :  %e\n",obj[i]->m01); 
         (void) kfprintf(printdev,"m11             :  %e\n",obj[i]->m11); 
         (void) kfprintf(printdev,"m20             :  %e\n",obj[i]->m20); 
         (void) kfprintf(printdev,"m02             :  %e\n",obj[i]->m02); 
         (void) kfprintf(printdev,"m30             :  %e\n",obj[i]->m30); 
         (void) kfprintf(printdev,"m03             :  %e\n",obj[i]->m03); 
         (void) kfprintf(printdev,"m12             :  %e\n",obj[i]->m12); 
         (void) kfprintf(printdev,"m21             :  %e\n\n",obj[i]->m21); 
         }
      if (central == 1) {
         (void) kfprintf(printdev,"Central  Moments\n");
         (void) kfprintf(printdev,"mu00            :  %e\n",obj[i]->mu00); 
         (void) kfprintf(printdev,"mu11            :  %e\n",obj[i]->mu11); 
         (void) kfprintf(printdev,"mu20            :  %e\n",obj[i]->mu20); 
         (void) kfprintf(printdev,"mu02            :  %e\n",obj[i]->mu02); 
         (void) kfprintf(printdev,"mu30            :  %e\n",obj[i]->mu30); 
         (void) kfprintf(printdev,"mu03            :  %e\n",obj[i]->mu03); 
         (void) kfprintf(printdev,"mu12            :  %e\n",obj[i]->mu12); 
         (void) kfprintf(printdev,"mu21            :  %e\n\n",obj[i]->mu21); 
         }
      if (invariant == 1) {
         (void) kfprintf(printdev,"Normalized Central  Moments\n");
         (void) kfprintf(printdev,"nu00            :  %e\n",obj[i]->nu00); 
         (void) kfprintf(printdev,"nu11            :  %e\n",obj[i]->nu11); 
         (void) kfprintf(printdev,"nu20            :  %e\n",obj[i]->nu20); 
         (void) kfprintf(printdev,"nu02            :  %e\n",obj[i]->nu02); 
         (void) kfprintf(printdev,"nu30            :  %e\n",obj[i]->nu30); 
         (void) kfprintf(printdev,"nu03            :  %e\n",obj[i]->nu03); 
         (void) kfprintf(printdev,"nu12            :  %e\n",obj[i]->nu12); 
         (void) kfprintf(printdev,"nu21            :  %e\n\n",obj[i]->nu21); 

         (void) kfprintf(printdev,"Invariant Moments\n");
         (void) kfprintf(printdev,"M1              :  %e\n",obj[i]->M1);
         (void) kfprintf(printdev,"M2              :  %e\n",obj[i]->M2);
         (void) kfprintf(printdev,"M3              :  %e\n",obj[i]->M3);
         (void) kfprintf(printdev,"M4              :  %e\n",obj[i]->M4);
         (void) kfprintf(printdev,"M5              :  %e\n",obj[i]->M5);
         (void) kfprintf(printdev,"M6              :  %e\n",obj[i]->M6);
         (void) kfprintf(printdev,"M7              :  %e\n\n",obj[i]->M7);;
         }

      (void) kfprintf(printdev,".....................................\n\n");
      }
     new_i++;
     }
     return(TRUE);
   }
/* -library_code_end */
