/*
  * Khoros: $Id: cp_mapdata.c,v 1.7.2.1 1997/04/24 22:22:34 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: cp_mapdata.c,v 1.7.2.1 1997/04/24 22:22:34 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		File Contents Title
   >>>>		
   >>>>   Public:
   >>>>                 copymapdata()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/************************************************************
*
*  Routine Name: copymapdata - Copy map data
*
*       Purpose: Copy map data and malloc space if requested.
*
*         Input: img1 - original image to copy map data from
*		 img2 - pointer to copied map data
*                malloc_flg - if set (=1) malloc room
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: Restrictions on data or input as applicable
*    Written By: Ron E. Neher
*          Date: 3/23/89
*      Verified:
*  Side Effects:
* Modifications:
*************************************************************/
int copymapdata(xvimage *img1, xvimage **img2, int malloc_flg)
{

long	lsize;

/* calculate room for the color map data */

    switch (img1->map_storage_type)
    {
        case VFF_MAPTYP_NONE:
	    return(1);
        case VFF_MAPTYP_1_BYTE:
            lsize = img1->map_col_size*img1->map_row_size;
            break;
        case VFF_MAPTYP_2_BYTE:
            lsize = img1->map_col_size*img1->map_row_size*sizeof(short);
            break;
        case VFF_MAPTYP_4_BYTE:
            lsize = img1->map_col_size*img1->map_row_size*sizeof(int);
            break;
        case VFF_MAPTYP_FLOAT:
            lsize = img1->map_col_size*img1->map_row_size*sizeof(float);
            break;
        case VFF_MAPTYP_DOUBLE:
            lsize = img1->map_col_size*img1->map_row_size*sizeof(double);
            break;
        case VFF_MAPTYP_COMPLEX:
            lsize = img1->map_col_size*img1->map_row_size*
		    sizeof(float)*2;
            break;
        default:
            (void) kfprintf(kstderr,"copymapdata: Not valid data storage \
type\n");
            return(0);
    }

     /* If map_scheme is ONEPERBAND, then need to copy all map 
      * data over to image2 
      */
    if (img1->map_scheme == VFF_MS_ONEPERBAND)
    {
      lsize *= img1->num_data_bands;
    }

    if (malloc_flg)
        if (((*img2)->maps = (char *)kmalloc((size_t) lsize)) == NULL)
        {
	    (void) kfprintf(kstderr,"copymapdata: Not enough memory for \
map data!\n");
	    return(0);
        }

    kmemcpy((kaddr) (*img2)->maps, (kaddr) img1->maps, (size_t) lsize);

    (*img2)->map_storage_type = img1->map_storage_type;
    (*img2)->map_row_size = img1->map_row_size;
    (*img2)->map_col_size = img1->map_col_size;

    return(1);
}
