/* Khoros: $Id: rowsize.c,v 1.6 1997/01/20 07:45:21 mirage Exp $ */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: rowsize.c,v 1.6 1997/01/20 07:45:21 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Library Routine for #object-name#
   >>>>		
   >>>>   Public:
   >>>>                 viff_rowsize()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/****************************************************************
* 
*  Routine Name: viff_rowsize - computes rowsize of image
* 
*       Purpose: computes the rowsize in bytes of an image according
*		 to the data type and width.
*
*         Input: data_storage_type - the data type of the scan line
*		 width - the width of the scan line
*        Output:
*       Returns: the row size in bytes on success, -1 otherwise
*
*  Restrictions:
*    Written By: Khoros 1 developer 
*          Date: Feb 27, 1994
*      Verified: 
*  Side Effects: 
* Modifications:
****************************************************************/
int viff_rowsize(
	int data_storage_type,
	int width)
{
	int	rowsize	= -1;		/* default to error value */


        switch (data_storage_type)
        {
           case VFF_TYP_BIT:
                rowsize = (width+7)/8;
                break;

           case VFF_TYP_1_BYTE:
                rowsize = width;
                break;

           case VFF_TYP_2_BYTE:
                rowsize = sizeof(short) * width;
                break;

           case VFF_TYP_4_BYTE:
                rowsize = sizeof(int) * width;
                break;

           case VFF_TYP_FLOAT:
                rowsize = sizeof(float) * width;
                break;

           case VFF_TYP_DOUBLE:
                rowsize = sizeof(double) * width;
                break;

           case VFF_TYP_COMPLEX:
                rowsize = sizeof(float) * width * 2;
                break;

           case VFF_TYP_DCOMPLEX:
                rowsize = sizeof(double) * width * 2;
                break;

	   default:
		(void) kfprintf(kstderr,"viff_rowsize: Unknown data storage "
				"type, %d\n", data_storage_type);
		break;
	}

	return rowsize;
}
