 /*
  * Khoros: $Id: kdbmupdate.c,v 1.1.2.1 1997/06/08 15:29:05 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kdbmupdate.c,v 1.1.2.1 1997/06/08 15:29:05 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for kdbmupdate
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "kdbmupdate.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - print, delete, or update (ascii) DBM key value
|
|       Purpose: main program for kdbmupdate
|
|         Input:
|		char *clui_info->dbm_file; {DBM file}
|		int   clui_info->dbm_flag; {TRUE if -dbm specified}
|
|		char *clui_info->key_string; {DBM key}
|		int   clui_info->key_flag; {TRUE if -key specified}
|
|		Required M.E. group:
|		    int clui_info->printval_flag; {TRUE if -printval specified}
|
|		    int clui_info->delete_flag; {TRUE if -delete specified}
|
|		    char *clui_info->update_string; {update value}
|		    int   clui_info->update_flag; {TRUE if -update specified}
|
|        Output:
|       Returns:
|
|    Written By: Kevin Sullivan
|          Date: July 1, 1997
| Modifications:
|
------------------------------------------------------------*/

int main(
   int  argc,
   char **argv)
{

/* -main_variable_list */
        kdbm    *dbm;
        kdatum  key, data;
/* -main_variable_list_end */

	khoros_init(argc, argv, "SUPPORT", PRODUCT_RELEASE_DATE,
	            PRODUCT_RELEASE_NAME, PRODUCT_RELEASE_VERSION,
	            PRODUCT_RELEASE_MAJOR, PRODUCT_RELEASE_MINOR,
	            "$SUPPORT/objects/kroutine/kdbmupdate");
	kexit_handler(kdbmupdate_free_args, NULL);

/* -main_get_args_call */
	kclui_init("SUPPORT", "kdbmupdate", KGEN_KROUTINE,  &clui_uis_spec,
		   kdbmupdate_usage_additions, kdbmupdate_get_args,
 		   kdbmupdate_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        if ((dbm = kdbm_open(clui_info->dbm_file, O_WRONLY, 0666)) == NULL)
        {
           kinfo(KSTANDARD, "kdbmupdate:  Failed to open database '%s'",
                 clui_info->dbm_file);
           kexit(KEXIT_FAILURE);
        }
/* -main_before_lib_call_end */

/* -main_library_call */
    key.dptr = clui_info->key_string;
    key.dsize = kstrlen(clui_info->key_string) + 1;

    if (clui_info->printval_flag)
    {
	data = kdbm_fetch(dbm, key);
	if (data.dptr != NULL)
	{
           kfprintf(kstdout,"'%s': %s\n", key.dptr, data.dptr);
	}
        else
        {
           kfprintf(kstderr,"key '%s' not found\n", key.dptr);
           kexit(KEXIT_FAILURE);
        }
    }
    else if (clui_info->delete_flag)
    {
	if (kdbm_delete(dbm, key) == -1)
	{
	   kfprintf(kstderr,"key '%s' not deleted\n", key.dptr);
           kexit(KEXIT_FAILURE);
        }
    }
    else /* clui_info->update_flag */
    {
	data.dptr = kstrdup(clui_info->update_string);
	data.dsize = kstrlen(data.dptr) + 1;
	if (!kdbm_store(dbm, key, data, KDBM_REPLACE) == -1)
        {
           kfprintf(kstderr,"key '%s' not stored\n", key.dptr);
           kexit(KEXIT_FAILURE);
        }
    }
/* -main_library_call_end */

/* -main_after_lib_call */
        kdbm_close(dbm);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: kdbmupdate_usage_additions
| 
|       Purpose: Prints usage additions in kdbmupdate_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kdbmupdate
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
void kdbmupdate_usage_additions(void)
{
	kfprintf(kstderr, "\tprint, delete, or update (ascii) DBM key value\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: kdbmupdate_free_args
| 
|       Purpose: Frees CLUI struct allocated in kdbmupdate_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kdbmupdate
|          Date: July 1, 1997
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
kdbmupdate_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->dbm_file);
		kfree_and_NULL(clui_info->key_string);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
