 /*
  * Khoros: $Id: prim_blast.c,v 1.1.2.1 1997/03/18 23:19:02 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: prim_blast.c,v 1.1.2.1 1997/03/18 23:19:02 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1997, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Geometry Services Primitive Code
   >>>>
   >>>>   Public:	
   >>>>			kgeom_blast_primitive()
   >>>>  Private:
   >>>>   Static:
   >>>>			_free_polyline()
   >>>>			_free_triangles()
   >>>>			_free_spheres()
   >>>>			_free_text()
   >>>>			_free_cylinders()
   >>>>			_free_directed_points()
   >>>>			_free_markers()
   >>>>			_free_polygon()
   >>>>			_free_rectangles()
   >>>>			_free_ellipses()
   >>>>			_free_mesh()
   >>>>			_free_texture()
   >>>>			_free_view_xform()
   >>>>			_free_object_xform()
   >>>>			_free_camera_orientation()
   >>>>			_free_vr_event()
   >>>>			_free_object()
   >>>>			_free_invalid()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static void _free_polyline  	     PROTO((kgeom_primitive *));
static void _free_triangles 	     PROTO((kgeom_primitive *));
static void _free_spheres   	     PROTO((kgeom_primitive *));
static void _free_text      	     PROTO((kgeom_primitive *));
static void _free_cylinders 	     PROTO((kgeom_primitive *));
static void _free_directed_points    PROTO((kgeom_primitive *));
static void _free_markers 	     PROTO((kgeom_primitive *));
static void _free_polygon 	     PROTO((kgeom_primitive *));
static void _free_rectangles 	     PROTO((kgeom_primitive *));
static void _free_ellipses 	     PROTO((kgeom_primitive *));
static void _free_mesh      	     PROTO((kgeom_primitive *));
static void _free_texture   	     PROTO((kgeom_primitive *));
static void _free_view_xform   	     PROTO((kgeom_primitive *));
static void _free_object_xform 	     PROTO((kgeom_primitive *));
static void _free_camera_orientation PROTO((kgeom_primitive *));
static void _free_vr_event	     PROTO((kgeom_primitive *));
static void _free_object    	     PROTO((kgeom_primitive *));
static void _free_invalid   	     PROTO((kgeom_primitive *));

static void (*free_function[]) PROTO((kgeom_primitive *)) =
{
   _free_invalid,
   _free_polyline,
   _free_triangles,
   _free_spheres,
   _free_text,
   _free_cylinders,
   _free_directed_points,
   _free_markers,
   _free_polygon,
   _free_rectangles,
   _free_ellipses,
   _free_mesh,
   _free_texture,
   _free_view_xform,
   _free_object_xform,
   _free_camera_orientation,
   _free_vr_event,
   _free_object
};

/************************************************************
*
*  Routine Name: kgeom_blast_primitive - destroy a geometry primitive
*
*       Purpose: This routine will free a given primitive and all
*		 associated data.
*
*		 Note that geometry services will free \fIall\fP the
*		 data pointers within the geometry primitive.  Because
*		 of this, be careful not to point to static arrays of
*		 data within the geometry primitive.  If geometry
*		 services tries to free a statically allocated array,
*		 it will result in a fatal error.
*       
*         Input: primitive - primitive to free
*        Output: none
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*
*    Written By: Steve Kubica
*          Date: December 1, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kgeom_blast_primitive(kgeom_primitive *primitive)
{
   if (primitive == NULL)
      return TRUE;

   /*-- free all primitive specific data --*/
   free_function[primitive_function_index(primitive->type)](primitive);
   kfree(primitive);
   return TRUE;
}

/*
 *    ================================================================== 
 *    primitive functions
 *    ==================================================================
 */

/*-----------------------------------------------------------
|  Routine Name: (static) _free_polyline
------------------------------------------------------------*/
static void _free_polyline(kgeom_primitive *primitive)
{
   kgeom_polyline *p = (kgeom_polyline *) primitive;

   kfree(p->locs);  kfree(p->cols);
   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_triangles
------------------------------------------------------------*/
static void _free_triangles(kgeom_primitive *primitive)
{
   kgeom_triangles *p = (kgeom_triangles *) primitive;

   kfree(p->texture);

   kfree(p->locs);  kfree(p->cols);  kfree(p->norms);  kfree(p->tcoords);
   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_spheres
------------------------------------------------------------*/
static void _free_spheres(kgeom_primitive *primitive)
{
   kgeom_spheres *p = (kgeom_spheres *) primitive;

   kfree(p->locs);  kfree(p->cols);  kfree(p->radii);
   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_text
------------------------------------------------------------*/
static void _free_text(kgeom_primitive *primitive)
{
   kgeom_text *p = (kgeom_text *) primitive;

   karray_free(p->strings, KSTRING, (size_t) p->nverts, NULL);
   karray_free(p->fonts,   KSTRING, (size_t) p->nverts, NULL);

   kfree(p->locs);  kfree(p->offsets);  kfree(p->ups);  kfree(p->flows);
   kfree(p->cols);  kfree(p->bgcols);
   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_cylinders
------------------------------------------------------------*/
static void _free_cylinders(kgeom_primitive *primitive)
{
   kgeom_cylinders *p = (kgeom_cylinders *) primitive;

   kfree(p->locs);  kfree(p->cols);  kfree(p->rad1);  kfree(p->rad2);

   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_directed_points
------------------------------------------------------------*/
static void _free_directed_points(kgeom_primitive *primitive)
{
   kgeom_directed_points *p = (kgeom_directed_points *) primitive;

   kfree(p->locs);  kfree(p->cols);  kfree(p->norms);

   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_markers
------------------------------------------------------------*/
static void _free_markers(kgeom_primitive *primitive)
{
   kgeom_markers *p = (kgeom_markers *) primitive;

   kfree(p->locs);  kfree(p->cols);  kfree(p->bgcols);

   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_polygon
------------------------------------------------------------*/
static void _free_polygon(kgeom_primitive *primitive)
{
   kgeom_polygon *p = (kgeom_polygon *) primitive;

   kfree(p->locs);  kfree(p->cols);  kfree(p->bgcol);

   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_rectangles
------------------------------------------------------------*/
static void _free_rectangles(kgeom_primitive *primitive)
{
   kgeom_rectangles *p = (kgeom_rectangles *) primitive;

   kfree(p->locs);  kfree(p->cols);  kfree(p->bgcols);

   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_ellipses
------------------------------------------------------------*/
static void _free_ellipses(kgeom_primitive *primitive)
{
   kgeom_ellipses *p = (kgeom_ellipses *) primitive;

   kfree(p->locs);  kfree(p->cols);  kfree(p->bgcols);  
   kfree(p->rad1);  kfree(p->rad2);

   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_mesh
------------------------------------------------------------*/
static void _free_mesh(kgeom_primitive *primitive)
{
   if (primitive->type == KGEOM_QUADMESH)
   {
      kgeom_quadmesh *p = (kgeom_quadmesh *) primitive;
      kfree(p->locs);  kfree(p->cols);  kfree(p->norms);  kfree(p->tcoords);
   }
   else /*-- KGEOM_OCTMESH --*/
   {
      kgeom_octmesh *p = (kgeom_octmesh *) primitive;
      kfree(p->locs);  kfree(p->cols);  kfree(p->norms);  kfree(p->tcoords);
   }
   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_texture
------------------------------------------------------------*/
static void _free_texture(kgeom_primitive *primitive)
{
   if (primitive->type == KGEOM_TEXTURE2D)
   {
      kgeom_texture2D *p = (kgeom_texture2D *) primitive;
      kfree(p->cols);
   }
   else /*-- KGEOM_TEXTURE3D --*/
   {
      kgeom_texture3D *p = (kgeom_texture3D *) primitive;
      kfree(p->cols);
   }
   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_view_xform
------------------------------------------------------------*/
static void _free_view_xform(kgeom_primitive *primitive)
{
   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_object_xform
------------------------------------------------------------*/
static void _free_object_xform(kgeom_primitive *primitive)
{
   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_camera_orientation
------------------------------------------------------------*/
static void _free_camera_orientation(kgeom_primitive *primitive)
{
   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_vr_event
------------------------------------------------------------*/
static void _free_vr_event(kgeom_primitive *primitive)
{
   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_object
------------------------------------------------------------*/
static void _free_object(kgeom_primitive *primitive)
{
   kgeom_subobject *p = (kgeom_subobject *) primitive;

   kgeom_blast_object((kgeom_object *)p->object);

   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _free_invalid
------------------------------------------------------------*/
static void _free_invalid(kgeom_primitive *primitive)
{
   return;
}
